package es.csa.commons.faces.component;

/*
 * #%L
 * Librerías comunes de Java Server Faces de CSA
 * %%
 * Copyright (C) 2015 CSA - Centro Regional de Servicios Avanzados S.A.
 * %%
 * This file is part of "Librerías comunes de CSA".
 * 
 * developed by: Centro Regional de Servicios Avanzados, S.A.
 * 
 * "Librerías comunes de CSA" is free software: you can redistribute it and/or modify it under the terms of:
 * - Apache License, Version 2.0 (the "License");  
 * - under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 * 
 * "Librerías comunes de CSA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or General Public License for more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE­2.0
 * 
 * You may obtain a copy of the General Public Licence at:
 * http://www.gnu.org/licenses/
 * #L%
 */

import java.io.IOException;
import java.util.Iterator;
import java.util.Set;

import javax.el.ELException;
import javax.el.ValueExpression;
import javax.faces.FacesException;
import javax.faces.application.FacesMessage;
import javax.faces.component.EditableValueHolder;
import javax.faces.component.UIComponent;
import javax.faces.component.UIComponentBase;
import javax.faces.context.FacesContext;
import javax.faces.validator.Validator;

import es.csa.commons.faces.util.BeanValidatorHelper;
import es.csa.commons.faces.util.MensajesUtils;
import es.csa.commons.faces.validator.HibernateValidator;

/**
 * JSF component class
 */
public class BeanValidator extends UIComponentBase {

  // -------------------------------------------------------------------------------------------------------- Constantes

  public static final String COMPONENT_TYPE = "es.csa.commons.faces.component.BeanValidator";

  public static final String COMPONENT_FAMILY = "es.csa.commons.faces.component.BeanValidator";

  public static final String INCORRECT_MESSAGE_ID = "es.csa.commons.faces.validator.BeanValidator.INCORRECT";

  // ------------------------------------------------------------------------------------------------------- Propiedades

  private boolean _transient = false;

  private String _summaryFormat = null;

  private String _detailFormat = null;

  private Set<String> _profile = null;

  private Object _value = null;

  // ----------------------------------------------------------------------------------------------------------- Métodos

  @Override
  public String getFamily() {
    return COMPONENT_FAMILY;
  }

  @Override
  public boolean getRendersChildren() {
    return true;
  }

  @Override
  public void processUpdates(FacesContext context) {
    super.processUpdates(context);
    Object value = getValue();
    if (null != value) {
      BeanValidatorHelper beanValidator = BeanValidatorHelper.getInstance(context);
      String[] messages = beanValidator.validateGraph(context, value, getProfile());
      if (null != messages && messages.length > 0) {
        context.renderResponse();
        for (String validatorMessage : messages) {
          FacesMessage message = MensajesUtils.createMessage(context, getSummaryFormat(), getDetailFormat(),
              FacesMessage.SEVERITY_ERROR, INCORRECT_MESSAGE_ID, validatorMessage);
          context.addMessage(null, message);
        }
      }
    }
  }

  @Override
  public void encodeBegin(FacesContext context) throws IOException {
    super.encodeBegin(context);
    HibernateValidator validator = (HibernateValidator) context.getApplication().createValidator(
        HibernateValidator.VALIDATOR_ID);
    validator.setSummaryFormat(getSummaryFormat());
    validator.setDetailFormat(getDetailFormat());
    setupValidators(this, validator);
  }

  @Override
  public void encodeChildren(FacesContext context) throws IOException {
    if (isRendered()) {
      for (UIComponent child : getChildren()) {
        if (child.isRendered()) {
          child.encodeAll(context);
        }
      }
    }
  }

  private void setupValidators(UIComponent component, Validator validator) {
    Iterator<UIComponent> facetsAndChildren = component.getFacetsAndChildren();
    while (facetsAndChildren.hasNext()) {
      UIComponent child = facetsAndChildren.next();
      if (child instanceof EditableValueHolder) {
        EditableValueHolder input = (EditableValueHolder) child;
        setupValidator(input, validator);
      }
      setupValidators(child, validator);
    }
  }

  /**
   * @param input
   */
  private void setupValidator(EditableValueHolder input, Validator validator) {
    Validator[] validators = input.getValidators();
    for (Validator validator2 : validators) {
      if (validator2 instanceof HibernateValidator) {
        return;
      }
    }
    input.addValidator(validator);
  }

  // ------------------------------------------------------------------------------------------------------- Propiedades

  public String getSummaryFormat() {
    if (this._summaryFormat != null) {
      return this._summaryFormat;
    }
    ValueExpression ve = getValueExpression("summaryFormat");
    if (ve != null) {
      String value = null;
      try {
        value = (String) ve.getValue(getFacesContext().getELContext());
      } catch (ELException e) {
        throw new FacesException(e);
      }
      return value;
    }
    return null;
  }

  public void setSummaryFormat(String format) {
    _summaryFormat = format;
  }

  public String getDetailFormat() {
    if (this._detailFormat != null) {
      return this._detailFormat;
    }
    ValueExpression ve = getValueExpression("detailFormat");
    if (ve != null) {
      String value = null;
      try {
        value = (String) ve.getValue(getFacesContext().getELContext());
      } catch (ELException e) {
        throw new FacesException(e);
      }
      return value;
    }
    return null;
  }

  public void setDetailFormat(String format) {
    _detailFormat = format;
  }

  public Object getValue() {
    if (this._value != null) {
      return this._value;
    }
    ValueExpression ve = getValueExpression("value");
    if (ve != null) {
      Object value = null;
      try {
        value = ve.getValue(getFacesContext().getELContext());
      } catch (ELException e) {
        throw new FacesException(e);
      }
      return value;
    }
    return null;
  }

  public void setValue(Object _value) {
    this._value = _value;
  }

  @SuppressWarnings("unchecked")
  public Set<String> getProfile() {
    if (this._profile != null) {
      return this._profile;
    }
    ValueExpression ve = getValueExpression("profile");
    if (ve != null) {
      Set<String> value = null;
      try {
        value = (Set<String>) ve.getValue(getFacesContext().getELContext());
      } catch (ELException e) {
        throw new FacesException(e);
      }
      return value;
    }
    return null;
  }

  public void setProfile(Set<String> _profile) {
    this._profile = _profile;
  }

  // ----------------------------------------------------------------------------------------- Salvar y restaurar estado

  @Override
  public boolean isTransient() {
    return _transient;
  }

  @Override
  public void setTransient(boolean transientValue) {
    _transient = transientValue;
  }

  @Override
  public Object saveState(FacesContext context) {
    Object[] states = new Object[5];
    states[0] = super.saveState(context);
    states[1] = _summaryFormat;
    states[2] = _detailFormat;
    states[3] = saveAttachedState(context, _profile);
    states[4] = saveAttachedState(context, _value);
    return states;
  }

  @Override
  @SuppressWarnings("unchecked")
  public void restoreState(FacesContext context, Object state) {
    Object[] states = (Object[]) state;
    super.restoreState(context, states[0]);
    _summaryFormat = (String) states[1];
    _detailFormat = (String) states[2];
    _profile = (Set<String>) restoreAttachedState(context, states[3]);
    _value = restoreAttachedState(context, states[4]);
  }

}
