package es.csa.commons.faces.component;

/*
 * #%L
 * Librerías comunes de Java Server Faces de CSA
 * %%
 * Copyright (C) 2015 CSA - Centro Regional de Servicios Avanzados S.A.
 * %%
 * This file is part of "Librerías comunes de CSA".
 * 
 * developed by: Centro Regional de Servicios Avanzados, S.A.
 * 
 * "Librerías comunes de CSA" is free software: you can redistribute it and/or modify it under the terms of:
 * - Apache License, Version 2.0 (the "License");  
 * - under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 * 
 * "Librerías comunes de CSA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or General Public License for more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE­2.0
 * 
 * You may obtain a copy of the General Public Licence at:
 * http://www.gnu.org/licenses/
 * #L%
 */

import java.beans.IntrospectionException;
import java.beans.PropertyDescriptor;
import java.io.IOException;
import java.lang.reflect.Method;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.Set;
import java.util.StringTokenizer;

import javax.el.ELException;
import javax.el.ValueExpression;
import javax.faces.FacesException;
import javax.faces.application.FacesMessage;
import javax.faces.application.FacesMessage.Severity;
import javax.faces.component.EditableValueHolder;
import javax.faces.component.UIComponent;
import javax.faces.component.UIComponentBase;
import javax.faces.context.FacesContext;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.apache.commons.beanutils.PropertyUtils;
import org.hibernate.validator.Digits;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotNull;
import org.richfaces.component.html.HtmlCalendar;

import es.csa.commons.faces.util.ValueExpressionAnalyzer;
import es.csa.commons.faces.util.ValueExpressionAnalyzer.ValueReference;

/**
 * JSF component class
 */
public class DecorateInput extends UIComponentBase {

  // -------------------------------------------------------------------------------------------------------- Constantes

  public static final String COMPONENT_TYPE = "es.csa.commons.faces.component.DecorateInput";

  public static final String COMPONENT_FAMILY = "es.csa.commons.faces.component.DecorateInput";

  // ------------------------------------------------------------------------------------------------------- Propiedades

  private boolean _transient = false;

  public static String _fatalClass = "with-fatal-messages";

  public static String _errorClass = "with-error-messages";

  public static String _warnClass = "with-warn-messages";

  public static String _infoClass = "with-info-messages";

  public static String _integerClass = "input-integer";

  public static String _decimalClass = "input-decimal";

  public static String _numberClass = "input-number";

  public static String _dateClass = "input-date";

  public static String _timeClass = "input-time";

  public static String _datetimeClass = "input-datetime";

  public static String _textClass = "input-text";

  public static String _legthClass = "input-with-length";

  public static String _requiredClass = "input-required";

  public static String _notDecorateMessages = "not-decorate-messages";

  public static String _notDecorateInput = "not-decorate-input";

  public static String _notDecorateRequired = "not-decorate-required";

  public static String _notDecorateMaxLength = "not-decorate-maxlength";

  public static String _notDecorate = "not-decorate";

  private Set<String> _profile = null;

  private Object _value = null;

  // ----------------------------------------------------------------------------------------------------------- Métodos

  @Override
  public String getFamily() {
    return COMPONENT_FAMILY;
  }

  @Override
  public boolean getRendersChildren() {
    return false;
  }

  @Override
  public void encodeBegin(FacesContext context) throws IOException {
    for (UIComponent child : getChildren()) {
      if (child instanceof EditableValueHolder) {
        ValueExpression valueExpression = child.getValueExpression("value");
        if (valueExpression == null) {
          continue;
        }
        ValueExpressionAnalyzer expressionAnalyzer = new ValueExpressionAnalyzer(valueExpression);

        ValueReference vr = expressionAnalyzer.getReference(context.getELContext());
        if (vr == null) {
          return;
        }

        // Obtener las clases actual

        Set<String> classes = new LinkedHashSet<String>();
        try {
          String childClass = null;
          if (child instanceof HtmlCalendar) {
            childClass = (String) PropertyUtils.getSimpleProperty(child, "inputClass");
          } else {
            childClass = (String) PropertyUtils.getSimpleProperty(child, "styleClass");
          }
          if (childClass != null) {
            StringTokenizer st = new StringTokenizer(childClass, " ");
            while (st.hasMoreTokens()) {
              classes.add(st.nextToken().trim());
            }
          }
        } catch (Exception e) {
          // No se puede acceder a la propiedad o no es una cadena, pues no hacemos nada.
          continue;
        }

        // Si hay un not-decorate, continuar, pues no hay que decorarlo
        if (classes.contains(_notDecorate)) {
          continue;
        }
        boolean decorateMessages = !classes.contains(_notDecorateMessages);
        boolean decorateInput = !classes.contains(_notDecorateInput);
        boolean decorateRequired = !classes.contains(_notDecorateRequired);
        boolean decorateMaxLength = !classes.contains(_notDecorateMaxLength);

        // Clases de mensajes
        if (decorateMessages) {
          boolean errorMessages = false, fatalMessages = false, infoMessages = false, warnMessages = false;
          Iterator<FacesMessage> componentMessages = context.getMessages(child.getClientId(context));
          while (componentMessages.hasNext()) {
            Severity severity = componentMessages.next().getSeverity();
            fatalMessages = fatalMessages || severity.equals(FacesMessage.SEVERITY_FATAL);
            errorMessages = errorMessages || severity.equals(FacesMessage.SEVERITY_ERROR);
            warnMessages = warnMessages || severity.equals(FacesMessage.SEVERITY_WARN);
            infoMessages = infoMessages || severity.equals(FacesMessage.SEVERITY_INFO);
          }
          if (fatalMessages)
            classes.add(_fatalClass);
          else
            classes.remove(_fatalClass);
          if (errorMessages)
            classes.add(_errorClass);
          else
            classes.remove(_errorClass);
          if (warnMessages)
            classes.add(_warnClass);
          else
            classes.remove(_warnClass);
          if (infoMessages)
            classes.add(_infoClass);
          else
            classes.remove(_infoClass);
        }

        // Analizar las anotaciones del campo
        if (decorateInput) {
          try {
            Method method = PropertyUtils.getReadMethod(new PropertyDescriptor(vr.getProperty(), vr.getBaseClass()));
            if (decorateRequired) {
              NotNull anNotNull = method.getAnnotation(NotNull.class);
              if (anNotNull != null) {
                classes.add(_requiredClass);
                ((EditableValueHolder) child).setRequired(true);
              }
            }

            if (decorateMaxLength) {
              Length anSize = method.getAnnotation(Length.class);
              if (anSize != null) {
                classes.add(_legthClass);
                try {
                  PropertyUtils.setSimpleProperty(child, "maxlength", anSize.max());
                } catch (Exception e) {
                  // No se puede acceder a la propiedad o no es una cadena, pues no hacemos nada.
                }
              }
            }

            Temporal anTemporal = method.getAnnotation(Temporal.class);
            if (anTemporal != null) {
              if (anTemporal.value() == TemporalType.DATE)
                classes.add(_dateClass);
              else if (anTemporal.value() == TemporalType.TIME)
                classes.add(_timeClass);
              else if (anTemporal.value() == TemporalType.TIMESTAMP) 
                classes.add(_datetimeClass);
            }

            Digits anDigits = method.getAnnotation(Digits.class);
            if (anDigits != null) {
              classes.add(_numberClass);
              if (anDigits.fractionalDigits() == 0)
                classes.add(_integerClass);
              else
                classes.add(_decimalClass);
              classes.add(_numberClass + "-integer-" + anDigits.integerDigits() + "-fractional-" + anDigits.fractionalDigits());
            }

            if (method.getReturnType().equals(String.class)) {
              classes.add(_textClass);
            }
          } catch (IntrospectionException e) {
            // No debería pasar, pero si pasa, continuamos sin más
          }
        }

        // Añadir las clases al campo y quitar las que ya no esten
        String childClass = "";
        for (String str : classes) {
          childClass = childClass + " " + str;
        }

        // Establece el inputClass y ya estamos
        try {
          if (child instanceof HtmlCalendar) {
            PropertyUtils.setSimpleProperty(child, "inputClass", (childClass).trim());
          } else {
            PropertyUtils.setSimpleProperty(child, "styleClass", (childClass).trim());
          }
        } catch (Exception e) {
          // No se puede acceder a la propiedad o no es una cadena, pues no hacemos nada.
        }

      }
    }
    super.encodeBegin(context);

  }

  // ------------------------------------------------------------------------------------------------------- Propiedades

  public Object getValue() {
    if (this._value != null) {
      return this._value;
    }
    ValueExpression ve = getValueExpression("value");
    if (ve != null) {
      Object value = null;
      try {
        value = ve.getValue(getFacesContext().getELContext());
      } catch (ELException e) {
        throw new FacesException(e);
      }
      return value;
    }
    return null;
  }

  public void setValue(Object _value) {
    this._value = _value;
  }

  @SuppressWarnings("unchecked")
  public Set<String> getProfile() {
    if (this._profile != null) {
      return this._profile;
    }
    ValueExpression ve = getValueExpression("profile");
    if (ve != null) {
      Set<String> value = null;
      try {
        value = (Set<String>) ve.getValue(getFacesContext().getELContext());
      } catch (ELException e) {
        throw new FacesException(e);
      }
      return value;
    }
    return null;
  }

  public void setProfile(Set<String> _profile) {
    this._profile = _profile;
  }

  // ----------------------------------------------------------------------------------------- Salvar y restaurar estado

  @Override
  public boolean isTransient() {
    return _transient;
  }

  @Override
  public void setTransient(boolean transientValue) {
    _transient = transientValue;
  }

  @Override
  public Object saveState(FacesContext context) {
    Object[] states = new Object[7];
    states[0] = super.saveState(context);
    states[1] = saveAttachedState(context, _profile);
    states[2] = saveAttachedState(context, _value);
    return states;
  }

  @Override
  @SuppressWarnings("unchecked")
  public void restoreState(FacesContext context, Object state) {
    Object[] states = (Object[]) state;
    super.restoreState(context, states[0]);
    _profile = (Set<String>) restoreAttachedState(context, states[1]);
    _value = restoreAttachedState(context, states[2]);
  }

}
