package es.csa.commons.faces.component;

/*
 * #%L
 * Librerías comunes de Java Server Faces de CSA
 * %%
 * Copyright (C) 2015 CSA - Centro Regional de Servicios Avanzados S.A.
 * %%
 * This file is part of "Librerías comunes de CSA".
 * 
 * developed by: Centro Regional de Servicios Avanzados, S.A.
 * 
 * "Librerías comunes de CSA" is free software: you can redistribute it and/or modify it under the terms of:
 * - Apache License, Version 2.0 (the "License");  
 * - under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 * 
 * "Librerías comunes de CSA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or General Public License for more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE­2.0
 * 
 * You may obtain a copy of the General Public Licence at:
 * http://www.gnu.org/licenses/
 * #L%
 */

import java.io.IOException;
import java.util.List;

import javax.el.ELException;
import javax.el.ValueExpression;
import javax.faces.FacesException;
import javax.faces.application.FacesMessage;
import javax.faces.component.EditableValueHolder;
import javax.faces.component.UIComponent;
import javax.faces.component.UIComponentBase;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.ConverterException;

import es.csa.commons.faces.util.MensajesUtils;

/**
 * JSF component class
 */
public class ListConverter extends UIComponentBase implements Converter {

  // -------------------------------------------------------------------------------------------------------- Constantes

  public static final String COMPONENT_TYPE = "es.csa.commons.faces.component.ListConverter";

  public static final String COMPONENT_FAMILY = "es.csa.commons.faces.component.ListConverter";

  public static final String CONVERTER_ID = "es.csa.common.faces.converter.ListConverter";

  public static final String LIST_NOT_FOUND_ID = "es.csa.faces.converter.ListConverter.LIST_NOT_FOUND_ID";

  // ------------------------------------------------------------------------------------------------------- Propiedades

  private boolean _transient = false;

  private String _summaryFormat = null;

  private String _detailFormat = null;

  private Object _value = null;

  // ----------------------------------------------------------------------------------------------------------- Métodos

  @Override
  public String getFamily() {
    return COMPONENT_FAMILY;
  }

  @Override
  public void encodeBegin(FacesContext context) throws IOException {
    super.encodeBegin(context);
    UIComponent parent = this.getParent();
    if (parent instanceof EditableValueHolder) ((EditableValueHolder) parent).setConverter(this);
  }

  // --------------------------------------------------------------------------------------------- Métodos de conversión

  public Object getAsObject(FacesContext context, UIComponent component, String value) {
    // "null" es la palabra clave para indicar un valor nulo
    if (value == null || value.equals("null")) {
      return null;
    }
    Integer index;
    try {
      index = Integer.parseInt(value);
      return getValueAsList().get(index);
    } catch (Exception e1) {
      String label = MensajesUtils.getLabel(context, component);
      FacesMessage message = MensajesUtils.createMessage(context, getSummaryFormat(), getDetailFormat(),
          FacesMessage.SEVERITY_ERROR, LIST_NOT_FOUND_ID, label);
      throw new ConverterException(message);
    }
  }

  public String getAsString(FacesContext context, UIComponent component, Object value) {
    if (value == null || value.equals("null")) {
      return "null";
    }
    try {
      return Integer.toString(getValueAsList().indexOf(value));
    } catch (Exception e) {
      String label = MensajesUtils.getLabel(context, component);
      FacesMessage message = MensajesUtils.createMessage(context, getSummaryFormat(), getDetailFormat(),
          FacesMessage.SEVERITY_ERROR, LIST_NOT_FOUND_ID, label);
      throw new ConverterException(message, e);
    }
  }

  // ------------------------------------------------------------------------------------------------------- Propiedades

  public String getSummaryFormat() {
    if (this._summaryFormat != null) {
      return this._summaryFormat;
    }
    ValueExpression ve = getValueExpression("summaryFormat");
    if (ve != null) {
      String value = null;
      try {
        value = (String) ve.getValue(getFacesContext().getELContext());
      } catch (ELException e) {
        throw new FacesException(e);
      }
      return value;
    }
    return null;
  }

  public void setSummaryFormat(String format) {
    _summaryFormat = format;
  }

  public String getDetailFormat() {
    if (this._detailFormat != null) {
      return this._detailFormat;
    }
    ValueExpression ve = getValueExpression("detailFormat");
    if (ve != null) {
      String value = null;
      try {
        value = (String) ve.getValue(getFacesContext().getELContext());
      } catch (ELException e) {
        throw new FacesException(e);
      }
      return value;
    }
    return null;
  }

  public void setDetailFormat(String format) {
    _detailFormat = format;
  }

  protected List<?> getValueAsList() {
    if (getValue() instanceof List<?>)
      return (List<?>) getValue();
    else
      return null;
  }

  public Object getValue() {
    if (this._value != null) {
      return this._value;
    }
    ValueExpression ve = getValueExpression("value");
    if (ve != null) {
      Object value = null;
      try {
        value = ve.getValue(getFacesContext().getELContext());
      } catch (ELException e) {
        throw new FacesException(e);
      }
      return value;
    }
    return null;
  }

  public void setValue(Object _value) {
    this._value = _value;
  }

  // ----------------------------------------------------------------------------------------- Salvar y restaurar estado

  @Override
  public boolean isTransient() {
    return _transient;
  }

  @Override
  public void setTransient(boolean transientValue) {
    _transient = transientValue;
  }

  @Override
  public Object saveState(FacesContext context) {
    Object[] states = new Object[5];
    states[0] = super.saveState(context);
    states[1] = _summaryFormat;
    states[2] = _detailFormat;
    states[3] = saveAttachedState(context, _value);
    return states;
  }

  @Override
  public void restoreState(FacesContext context, Object state) {
    Object[] states = (Object[]) state;
    super.restoreState(context, states[0]);
    _summaryFormat = (String) states[1];
    _detailFormat = (String) states[2];
    _value = restoreAttachedState(context, states[3]);
  }

}
