package es.csa.commons.faces.component;

/*
 * #%L
 * Librerías comunes de Java Server Faces de CSA
 * %%
 * Copyright (C) 2015 CSA - Centro Regional de Servicios Avanzados S.A.
 * %%
 * This file is part of "Librerías comunes de CSA".
 * 
 * developed by: Centro Regional de Servicios Avanzados, S.A.
 * 
 * "Librerías comunes de CSA" is free software: you can redistribute it and/or modify it under the terms of:
 * - Apache License, Version 2.0 (the "License");  
 * - under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 * 
 * "Librerías comunes de CSA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or General Public License for more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE­2.0
 * 
 * You may obtain a copy of the General Public Licence at:
 * http://www.gnu.org/licenses/
 * #L%
 */

import java.util.Iterator;
import java.util.Set;

import javax.el.ELException;
import javax.el.ValueExpression;
import javax.faces.FacesException;
import javax.faces.application.FacesMessage;
import javax.faces.application.FacesMessage.Severity;
import javax.faces.component.EditableValueHolder;
import javax.faces.component.UIComponent;
import javax.faces.component.UIComponentBase;
import javax.faces.context.FacesContext;

import org.apache.commons.beanutils.PropertyUtils;
import org.richfaces.component.html.HtmlCalendar;

/**
 * JSF component class
 */
public class WithMessagesStyleClass extends UIComponentBase {

  // -------------------------------------------------------------------------------------------------------- Constantes

  public static final String COMPONENT_TYPE = "es.csa.commons.faces.component.WithMessagesStyleClass";

  public static final String COMPONENT_FAMILY = "es.csa.commons.faces.component.WithMessagesStyleClass";

  // ------------------------------------------------------------------------------------------------------- Propiedades

  private boolean _transient = false;

  private String _fatalClass = "with-fatal-messages";

  private String _errorClass = "with-error-messages";

  private String _warnClass = "with-warn-messages";

  private String _infoClass = "with-info-messages";

  private Set<String> _profile = null;

  private Object _value = null;

  // ----------------------------------------------------------------------------------------------------------- Métodos

  @Override
  public String getFamily() {
    return COMPONENT_FAMILY;
  }

  @Override
  public boolean getRendersChildren() {
    return false;
  }

  @Override
  public void processValidators(FacesContext context) {
    super.processValidators(context);
    for (UIComponent child : getChildren()) {
      if (child instanceof EditableValueHolder) {
        boolean errorMessages = false, fatalMessages = false, infoMessages = false, warnMessages = false;
        Iterator<FacesMessage> componentMessages = context.getMessages(child.getClientId(context));
        while (componentMessages.hasNext()) {
          Severity severity = componentMessages.next().getSeverity();
          fatalMessages = fatalMessages || severity.equals(FacesMessage.SEVERITY_FATAL);
          errorMessages = errorMessages || severity.equals(FacesMessage.SEVERITY_ERROR);
          warnMessages = warnMessages || severity.equals(FacesMessage.SEVERITY_WARN);
          infoMessages = infoMessages || severity.equals(FacesMessage.SEVERITY_INFO);
        }
        String childClass = null;
        try {
          if (child instanceof HtmlCalendar) {
            childClass = (String) PropertyUtils.getSimpleProperty(child, "inputClass");
          } else {
            childClass = (String) PropertyUtils.getSimpleProperty(child, "styleClass");
          }
          if (childClass == null) childClass = "";
          if (fatalMessages && !childClass.contains(getFatalClass())) {
            childClass = (childClass + " " + getFatalClass()).trim();
          } else if (!fatalMessages && childClass.contains(getFatalClass())) {
            childClass = childClass.replaceAll("\\s*\\b" + getFatalClass() + "\\b\\s*", " ");
          }
          if (errorMessages && !childClass.contains(getErrorClass())) {
            childClass = (childClass + " " + getErrorClass()).trim();
          } else if (!errorMessages && childClass.contains(getErrorClass())) {
            childClass = childClass.replaceAll("\\s*\\b" + getErrorClass() + "\\b\\s*", " ");
          }
          if (warnMessages && !childClass.contains(getWarnClass())) {
            childClass = (childClass + " " + getWarnClass()).trim();
          } else if (!warnMessages && childClass.contains(getWarnClass())) {
            childClass = childClass.replaceAll("\\s*\\b" + getWarnClass() + "\\s*\\b", " ");
          }
          if (infoMessages && !childClass.contains(getInfoClass())) {
            childClass = (childClass + " " + getInfoClass()).trim();
          } else if (!infoMessages && childClass.contains(getInfoClass())) {
            childClass = childClass.replaceAll("\\s*\\b" + getInfoClass() + "\\s*\\b", " ");
          }
          if (child instanceof HtmlCalendar) {
            PropertyUtils.setSimpleProperty(child, "inputClass", (childClass).trim());
          } else {
            PropertyUtils.setSimpleProperty(child, "styleClass", (childClass).trim());
          }
        } catch (Exception e) {
          // No se puede acceder a la propiedad o no es una cadena, pues no hacemos nada.
        }
      }
    }
  }

  // ------------------------------------------------------------------------------------------------------- Propiedades

  public void setInfoClass(String infoClass) {
    _infoClass = infoClass;
  }

  public String getInfoClass() {
    if (this._infoClass != null) {
      return this._infoClass;
    }
    ValueExpression ve = getValueExpression("infoClass");
    if (ve != null) {
      String value = null;
      try {
        value = (String) ve.getValue(getFacesContext().getELContext());
      } catch (ELException e) {
        throw new FacesException(e);
      }
      return value;
    }
    return null;
  }

  public void setFatalClass(String fatalClass) {
    _fatalClass = fatalClass;
  }

  public String getFatalClass() {
    if (this._fatalClass != null) {
      return this._fatalClass;
    }
    ValueExpression ve = getValueExpression("fatalClass");
    if (ve != null) {
      String value = null;
      try {
        value = (String) ve.getValue(getFacesContext().getELContext());
      } catch (ELException e) {
        throw new FacesException(e);
      }
      return value;
    }
    return null;
  }

  public void setErrorClass(String errorClass) {
    _errorClass = errorClass;
  }

  public String getErrorClass() {
    if (this._errorClass != null) {
      return this._errorClass;
    }
    ValueExpression ve = getValueExpression("errorClass");
    if (ve != null) {
      String value = null;
      try {
        value = (String) ve.getValue(getFacesContext().getELContext());
      } catch (ELException e) {
        throw new FacesException(e);
      }
      return value;
    }
    return null;
  }

  public String getWarnClass() {
    if (this._warnClass != null) {
      return this._warnClass;
    }
    ValueExpression ve = getValueExpression("warnClass");
    if (ve != null) {
      String value = null;
      try {
        value = (String) ve.getValue(getFacesContext().getELContext());
      } catch (ELException e) {
        throw new FacesException(e);
      }
      return value;
    }
    return null;
  }

  public void setWarnClass(String warnClass) {
    _warnClass = warnClass;
  }

  public Object getValue() {
    if (this._value != null) {
      return this._value;
    }
    ValueExpression ve = getValueExpression("value");
    if (ve != null) {
      Object value = null;
      try {
        value = ve.getValue(getFacesContext().getELContext());
      } catch (ELException e) {
        throw new FacesException(e);
      }
      return value;
    }
    return null;
  }

  public void setValue(Object _value) {
    this._value = _value;
  }

  @SuppressWarnings("unchecked")
  public Set<String> getProfile() {
    if (this._profile != null) {
      return this._profile;
    }
    ValueExpression ve = getValueExpression("profile");
    if (ve != null) {
      Set<String> value = null;
      try {
        value = (Set<String>) ve.getValue(getFacesContext().getELContext());
      } catch (ELException e) {
        throw new FacesException(e);
      }
      return value;
    }
    return null;
  }

  public void setProfile(Set<String> _profile) {
    this._profile = _profile;
  }

  // ----------------------------------------------------------------------------------------- Salvar y restaurar estado

  @Override
  public boolean isTransient() {
    return _transient;
  }

  @Override
  public void setTransient(boolean transientValue) {
    _transient = transientValue;
  }

  @Override
  public Object saveState(FacesContext context) {
    Object[] states = new Object[7];
    states[0] = super.saveState(context);
    states[1] = _warnClass;
    states[2] = _infoClass;
    states[3] = _fatalClass;
    states[4] = _errorClass;
    states[5] = saveAttachedState(context, _profile);
    states[6] = saveAttachedState(context, _value);
    return states;
  }

  @Override
  @SuppressWarnings("unchecked")
  public void restoreState(FacesContext context, Object state) {
    Object[] states = (Object[]) state;
    super.restoreState(context, states[0]);
    _warnClass = (String) states[1];
    _infoClass = (String) states[2];
    _fatalClass = (String) states[3];
    _errorClass = (String) states[4];
    _profile = (Set<String>) restoreAttachedState(context, states[5]);
    _value = restoreAttachedState(context, states[6]);
  }

}
