package es.csa.commons.faces.converter;

/*
 * #%L
 * Librerías comunes de Java Server Faces de CSA
 * %%
 * Copyright (C) 2015 CSA - Centro Regional de Servicios Avanzados S.A.
 * %%
 * This file is part of "Librerías comunes de CSA".
 * 
 * developed by: Centro Regional de Servicios Avanzados, S.A.
 * 
 * "Librerías comunes de CSA" is free software: you can redistribute it and/or modify it under the terms of:
 * - Apache License, Version 2.0 (the "License");  
 * - under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 * 
 * "Librerías comunes de CSA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or General Public License for more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE­2.0
 * 
 * You may obtain a copy of the General Public Licence at:
 * http://www.gnu.org/licenses/
 * #L%
 */

import java.io.Serializable;
import java.util.Collection;

import javax.faces.application.FacesMessage;
import javax.faces.component.StateHolder;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.ConverterException;

import org.apache.commons.beanutils.PropertyUtils;

import es.csa.commons.faces.util.MensajesUtils;

/**
 * Converter for JPA entities to safely use in drop-down menus.
 * 
 * @author <a href="mailto:jalcalde@csa.es>Josué Alcalde González &lt;jalcalde@csa.es&gt;</a>
 * @version 15/11/2007-18:01:37
 */
public class EntityConverter implements Converter, Serializable, StateHolder {

  private static final long serialVersionUID = -999865872122589338L;

  // -------------------------------------------------------------------------------------------------------- Constantes

  public static final String CONVERTER_ID = "es.csa.commons.faces.converter.EntityConverter";

  public static final String UNIQUE_PROPERTY_NOT_FOUND_ID = "es.csa.faces.converter.EntityConverter.UNIQUE_PROPERTY_NOT_FOUND_ID";

  public static final String ENTITY_NOT_FOUND_ID = "es.csa.faces.converter.EntityConverter.ENTITY_NOT_FOUND_ID";

  // ------------------------------------------------------------------------------------------------------- Propiedades

  private boolean _transient = false;

  private Collection<?> _entities = null;

  private String _uniqueProperty = "id";

  private String _summaryFormat = null;

  private String _detailFormat = null;

  // --------------------------------------------------------------------------------------------- Métodos de conversión

  public Object getAsObject(FacesContext context, UIComponent component, String value) {
    // "null" es la palabra clave para indicar un valor nulo
    if (value == null || value.equals("null")) {
      return null;
    }
    for (Object obj : getEntities()) {
      try {
        if (PropertyUtils.getNestedProperty(obj, getUniqueProperty()).toString().equals(value)) return obj;
      } catch (Exception e) {
        String label = MensajesUtils.getLabel(context, component);
        FacesMessage message = MensajesUtils.createMessage(context, getSummaryFormat(), getDetailFormat(),
            FacesMessage.SEVERITY_ERROR, UNIQUE_PROPERTY_NOT_FOUND_ID, label);
        throw new ConverterException(message, e);
      }
    }
    String label = MensajesUtils.getLabel(context, component);
    FacesMessage message = MensajesUtils.createMessage(context, getSummaryFormat(), getDetailFormat(),
        FacesMessage.SEVERITY_ERROR, ENTITY_NOT_FOUND_ID, label);
    throw new ConverterException(message);
  }

  public String getAsString(FacesContext context, UIComponent component, Object value) {
    if (value == null || value.equals("null")) {
      return "null";
    }
    try {
      return (PropertyUtils.getNestedProperty(value, getUniqueProperty()).toString());
    } catch (Exception e) {
      String label = MensajesUtils.getLabel(context, component);
      FacesMessage message = MensajesUtils.createMessage(context, getSummaryFormat(), getDetailFormat(),
          FacesMessage.SEVERITY_ERROR, UNIQUE_PROPERTY_NOT_FOUND_ID, label);
      throw new ConverterException(message, e);
    }
  }

  // ---------------------------------------------------------------------------- Métodos de obtención y establecimiento

  public Collection<?> getEntities() {
    return _entities;
  }

  public void setEntities(Collection<?> entities) {
    this._entities = entities;
  }

  public String getUniqueProperty() {
    return _uniqueProperty;
  }

  public void setUniqueProperty(String uniqueProperty) {
    this._uniqueProperty = uniqueProperty;
  }

  public String getSummaryFormat() {
    return _summaryFormat;
  }

  public void setSummaryFormat(String format) {
    _summaryFormat = format;
  }

  public String getDetailFormat() {
    return _detailFormat;
  }

  public void setDetailFormat(String format) {
    _detailFormat = format;
  }

  // ----------------------------------------------------------------------------------------- Salvar y restaurar estado

  @Override
  public boolean isTransient() {
    return _transient;
  }

  @Override
  public void setTransient(boolean transientValue) {
    _transient = transientValue;
  }

  @Override
  public Object saveState(FacesContext context) {
    Object[] state = new Object[4];
    state[0] = _entities;
    state[1] = _uniqueProperty;
    state[2] = _detailFormat;
    state[3] = _summaryFormat;
    return state;
  }

  @Override
  public void restoreState(FacesContext context, Object state) {
    Object values[] = (Object[]) state;
    _entities = (Collection<?>) values[0];
    _uniqueProperty = (String) values[1];
    _detailFormat = (String) values[2];
    _summaryFormat = (String) values[3];
  }

}
