package es.csa.commons.faces.converter;

/*
 * #%L
 * Librerías comunes de Java Server Faces de CSA
 * %%
 * Copyright (C) 2015 CSA - Centro Regional de Servicios Avanzados S.A.
 * %%
 * This file is part of "Librerías comunes de CSA".
 * 
 * developed by: Centro Regional de Servicios Avanzados, S.A.
 * 
 * "Librerías comunes de CSA" is free software: you can redistribute it and/or modify it under the terms of:
 * - Apache License, Version 2.0 (the "License");  
 * - under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 * 
 * "Librerías comunes de CSA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or General Public License for more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE­2.0
 * 
 * You may obtain a copy of the General Public Licence at:
 * http://www.gnu.org/licenses/
 * #L%
 */

import static javax.faces.application.FacesMessage.SEVERITY_ERROR;

import java.io.Serializable;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.ConverterException;

import es.csa.commons.faces.util.MensajesUtils;

public class TimeToMinutesConverter implements Converter, Serializable {

  private static final long serialVersionUID = -3853649324211371687L;

  // -------------------------------------------------------------------------------------------------------- Constantes

  public static final String CONVERTER_ID = "es.csa.commons.faces.converter.TimeToMinutesConverter";

  public static final String INCORRECT_TIME_FORMAT_ID = "es.csa.commons.faces.converter.TimeToMinutesConverter.INCORRECT_TIME_FORMAT_ID";

  @Override
  public Object getAsObject(FacesContext context, UIComponent component, String value) {
    if (value == null) return null;
    if (value.equalsIgnoreCase("")) return null;
    Pattern pattern = Pattern.compile("^(\\d+)(\\:([0-5][0-9]))?$");
    Matcher matcher = pattern.matcher(value.trim());
    if (!matcher.find()) {
      String label = MensajesUtils.getLabel(context, component);
      FacesMessage message = MensajesUtils.createMessage(context, SEVERITY_ERROR, INCORRECT_TIME_FORMAT_ID, label);
      throw new ConverterException(message);
    } else {
      long longHours = Long.parseLong(matcher.group(1));
      long longMinutes = matcher.group(3) != null ? Long.parseLong(matcher.group(3)) : 0;
      return longHours * 60 + longMinutes;
    }
  }

  @Override
  public String getAsString(FacesContext context, UIComponent component, Object value) {
    if (value == null) return "";
    long minutos = ((Number) value).longValue();
    if (minutos < 0) return "";
    return minutos / 60 + ":" + (minutos % 60 < 10 ? "0" + minutos % 60 : minutos % 60);
  }

}
