package es.csa.commons.faces.util;

/*
 * #%L
 * Librerías comunes de Java Server Faces de CSA
 * %%
 * Copyright (C) 2015 CSA - Centro Regional de Servicios Avanzados S.A.
 * %%
 * This file is part of "Librerías comunes de CSA".
 * 
 * developed by: Centro Regional de Servicios Avanzados, S.A.
 * 
 * "Librerías comunes de CSA" is free software: you can redistribute it and/or modify it under the terms of:
 * - Apache License, Version 2.0 (the "License");  
 * - under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 * 
 * "Librerías comunes de CSA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or General Public License for more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE­2.0
 * 
 * You may obtain a copy of the General Public Licence at:
 * http://www.gnu.org/licenses/
 * #L%
 */
 
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import org.springframework.binding.message.MessageBuilder;
import org.springframework.binding.message.MessageContext;

public class MessageContextHelper implements Serializable {

  private static final long serialVersionUID = 7591107515083348334L;

  private class Message implements Serializable {
    
    private static final long serialVersionUID = -3260475142781167130L;
    
    private String level;
    private String code;
    private Object source;
    private Object[] args;

    private Message(String level, String code, Object... args) {
      this.level = level;
      this.code = code;
      this.args = args;
    }

    private Message(String level, Object source, String code, Object... args) {
      this(level, code, args);
      this.source = source;
    }

  }

  private List<Message> messages = new ArrayList<Message>();

  public void addMessage(String level, String code, Object... args) {
    messages.add(new Message(level, code, args));
  }

  public void addMessage(String level, Object source, String code, Object... args) {
    messages.add(new Message(level, source, code, args));
  }

  public void addMessage(String level, String code) {
    messages.add(new Message(level, code));
  }

  public void addMessage(String level, Object source, String code) {
    messages.add(new Message(level, source, code));
  }

  public void clearMessages() {
    messages.clear();
  }

  public void clearMessages(String code) {
    List<Message> removeMessages =  new ArrayList<Message>();
    for (Message m : messages)
      if (m.code.equals(code))
        removeMessages.add(m);
    messages.removeAll(messages);
  }

  public void renderMessages(MessageContext context) {
    for (Message m : messages) {
      MessageBuilder mb = new MessageBuilder();
      if (m.level.equalsIgnoreCase("info")) {
        mb.info();
      } else if (m.level.equalsIgnoreCase("warn")) {
        mb.warning();
      } else if (m.level.equalsIgnoreCase("error")) {
        mb.error();
      } else if (m.level.equalsIgnoreCase("fatal")) {
        mb.fatal();
      }
      if (m.source != null) {
        mb.source(m.source);
      }
      context.addMessage(mb.defaultText(m.code).code(m.code).args(m.args).build());
    }
    messages.clear();
  }

}
