package es.csa.commons.faces.util;

/*
 * #%L
 * Librerías comunes de Java Server Faces de CSA
 * %%
 * Copyright (C) 2015 CSA - Centro Regional de Servicios Avanzados S.A.
 * %%
 * This file is part of "Librerías comunes de CSA".
 * 
 * developed by: Centro Regional de Servicios Avanzados, S.A.
 * 
 * "Librerías comunes de CSA" is free software: you can redistribute it and/or modify it under the terms of:
 * - Apache License, Version 2.0 (the "License");  
 * - under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 * 
 * "Librerías comunes de CSA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or General Public License for more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE­2.0
 * 
 * You may obtain a copy of the General Public Licence at:
 * http://www.gnu.org/licenses/
 * #L%
 */
 
import java.beans.FeatureDescriptor;
import java.util.Iterator;
import java.util.Locale;

import javax.el.ELContext;
import javax.el.ELException;
import javax.el.ELResolver;
import javax.el.FunctionMapper;
import javax.el.ValueExpression;
import javax.el.VariableMapper;
import javax.faces.el.CompositeComponentExpressionHolder;

public class ValueExpressionAnalyzer {
  private ValueExpression expression;

  public ValueExpressionAnalyzer(ValueExpression expression) {
    this.expression = expression;
  }

  public ValueReference getReference(ELContext elContext) {
    InterceptingResolver resolver = new InterceptingResolver(elContext.getELResolver());
    try {
      expression.setValue(decorateELContext(elContext, resolver), null);
    } catch (ELException ele) {
      return null;
    }
    ValueReference reference = resolver.getValueReference();
    if (reference != null) {
      Object base = reference.getBase();
      if (base instanceof CompositeComponentExpressionHolder) {
        ValueExpression ve = ((CompositeComponentExpressionHolder) base).getExpression(reference.getProperty());
        if (ve != null) {
          this.expression = ve;
          reference = getReference(elContext);
        }
      }
    }
    return reference;
  }

  private ELContext decorateELContext(final ELContext context, final ELResolver resolver) {
    return new ELContext() {

      @Override
      public ELResolver getELResolver() {
        return resolver;
      }

      @SuppressWarnings("unchecked")
      @Override
      public Object getContext(Class key) {
        return context.getContext(key);
      }

      @Override
      public Locale getLocale() {
        return context.getLocale();
      }

      @Override
      public boolean isPropertyResolved() {
        return context.isPropertyResolved();
      }

      @SuppressWarnings("unchecked")
      @Override
      public void putContext(Class key, Object contextObject) {
        context.putContext(key, contextObject);
      }

      @Override
      public void setLocale(Locale locale) {
        context.setLocale(locale);
      }

      @Override
      public void setPropertyResolved(boolean resolved) {
        context.setPropertyResolved(resolved);
      }

      @Override
      public FunctionMapper getFunctionMapper() {
        return context.getFunctionMapper();
      }

      @Override
      public VariableMapper getVariableMapper() {
        return context.getVariableMapper();
      }
    };
  }

  private static class InterceptingResolver extends ELResolver {

    private ELResolver delegate;
    private ValueReference valueReference;

    public InterceptingResolver(ELResolver delegate) {
      this.delegate = delegate;
    }

    public ValueReference getValueReference() {
      return valueReference;
    }

    @Override
    public void setValue(ELContext context, Object base, Object property, Object value) {
      if (base != null && property != null) {
        context.setPropertyResolved(true);
        valueReference = new ValueReference(base, property.toString());
      }
    }

    @Override
    public Object getValue(ELContext context, Object base, Object property) {
      return delegate.getValue(context, base, property);
    }

    @Override
    public Class<?> getType(ELContext context, Object base, Object property) {
      return delegate.getType(context, base, property);
    }

    @Override
    public boolean isReadOnly(ELContext context, Object base, Object property) {
      return delegate.isReadOnly(context, base, property);
    }

    @Override
    public Iterator<FeatureDescriptor> getFeatureDescriptors(ELContext context, Object base) {
      return delegate.getFeatureDescriptors(context, base);
    }

    @Override
    public Class<?> getCommonPropertyType(ELContext context, Object base) {
      return delegate.getCommonPropertyType(context, base);
    }

  }

  public static class ValueReference {
    private Object base;
    private String property;

    public ValueReference(Object base, String property) {
      this.base = base;
      this.property = property;
    }

    @SuppressWarnings("unchecked")
    public Class getBaseClass() {
      return base.getClass();
    }

    public Object getBase() {
      return base;
    }

    public String getProperty() {
      return property;
    }
  }
}
