package es.csa.commons.faces.validator;

/*
 * #%L
 * Librerías comunes de Java Server Faces de CSA
 * %%
 * Copyright (C) 2015 CSA - Centro Regional de Servicios Avanzados S.A.
 * %%
 * This file is part of "Librerías comunes de CSA".
 * 
 * developed by: Centro Regional de Servicios Avanzados, S.A.
 * 
 * "Librerías comunes de CSA" is free software: you can redistribute it and/or modify it under the terms of:
 * - Apache License, Version 2.0 (the "License");  
 * - under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 * 
 * "Librerías comunes de CSA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or General Public License for more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE­2.0
 * 
 * You may obtain a copy of the General Public Licence at:
 * http://www.gnu.org/licenses/
 * #L%
 */
 
import java.io.Serializable;

import javax.faces.FacesException;
import javax.faces.application.FacesMessage;
import javax.faces.component.StateHolder;
import javax.faces.component.UIComponent;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.faces.validator.Validator;
import javax.faces.validator.ValidatorException;

import es.csa.commons.faces.util.MensajesUtils;

/**
 * Validador de igualdad (basado en el de myfaces tomahawk osea que tiene licencia Apache).
 * 
 * @author mwessendorf (latest modification by $Author: rsanchez $)
 */
public class EqualValidator implements Validator, StateHolder, Serializable {

  private static final long serialVersionUID = 893182173422637810L;

  /**
   * <p>
   * The standard converter id for this converter.
   * </p>
   */
  public static final String VALIDATOR_ID = "es.csa.commons.faces.validator.EqualValidator";

  /**
   * <p>
   * The message identifier of the {@link FacesMessage} to be created if the equal_for check fails.
   * </p>
   */
  public static final String EQUAL_MESSAGE_ID = "es.csa.commons.faces.validator.EqualValidator.Equal.INVALID";

  public static final String VALIDATOR_RESOURCE_BUNDLE = "es.csa.commons.faces.resources.FacesMessages";

  public EqualValidator() {}

  // the foreign component_id on which the validation is based.
  private String _for = null;

  // JSF-Field for StateHolder-IF
  private boolean _transient = false;

  private String _summaryFormat = null;

  private String _detailFormat = null;

  // ------------------------------------------------------------------------------------------------------- ValidatorIF

  public void validate(FacesContext facesContext, UIComponent uiComponent, Object value) throws ValidatorException {

    if (facesContext == null) throw new NullPointerException("facesContext");
    if (uiComponent == null) throw new NullPointerException("uiComponent");

    if (value == null) {
      return;
    }

    UIInput foreignComp = (UIInput) uiComponent.getParent().findComponent(_for);
    if (foreignComp == null)
      throw new FacesException("Unable to find component '" + _for + "' (calling findComponent on component '"
          + uiComponent.getId() + "')");

    if (foreignComp.getValue() == null || !foreignComp.getValue().toString().equals(value.toString())) {
      String label = MensajesUtils.getLabel(facesContext, uiComponent);
      FacesMessage message = MensajesUtils.createMessage(facesContext, getSummaryFormat(), getDetailFormat(),
          FacesMessage.SEVERITY_ERROR, EQUAL_MESSAGE_ID, label);
      throw new ValidatorException(message);
      // throw new ValidatorException(MessageUtils.getMessage(FacesMessage.SEVERITY_ERROR,
      // EQUAL_MESSAGE_ID, args));
    }

  }

  // ----------------------------------------------------------------------------------------------------- StateholderIF

  public Object saveState(FacesContext context) {
    Object state = _for;
    return state;
  }

  public void restoreState(FacesContext context, Object state) {
    _for = (String) state;
  }

  public boolean isTransient() {
    return _transient;
  }

  public void setTransient(boolean newTransientValue) {
    _transient = newTransientValue;
  }

  // ------------------------------------------------------------------------------------------------------- Propiedades

  public String getSummaryFormat() {
    return _summaryFormat;
  }

  public void setSummaryFormat(String format) {
    _summaryFormat = format;
  }

  public String getDetailFormat() {
    return _detailFormat;
  }

  public void setDetailFormat(String format) {
    _detailFormat = format;
  }

  /**
   * @return the foreign component_id, on which a value should be validated
   */
  public String getFor() {
    return _for;
  }

  /**
   * @param string
   *          the foreign component_id, on which a value should be validated
   */
  public void setFor(String string) {
    _for = string;
  }

}
