package es.csa.commons.faces.validator;

/*
 * #%L
 * Librerías comunes de Java Server Faces de CSA
 * %%
 * Copyright (C) 2015 CSA - Centro Regional de Servicios Avanzados S.A.
 * %%
 * This file is part of "Librerías comunes de CSA".
 * 
 * developed by: Centro Regional de Servicios Avanzados, S.A.
 * 
 * "Librerías comunes de CSA" is free software: you can redistribute it and/or modify it under the terms of:
 * - Apache License, Version 2.0 (the "License");  
 * - under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 * 
 * "Librerías comunes de CSA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or General Public License for more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE­2.0
 * 
 * You may obtain a copy of the General Public Licence at:
 * http://www.gnu.org/licenses/
 * #L%
 */
 
import java.io.Serializable;

import javax.el.ELException;
import javax.el.ValueExpression;
import javax.faces.FacesException;
import javax.faces.application.FacesMessage;
import javax.faces.component.EditableValueHolder;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.validator.Validator;
import javax.faces.validator.ValidatorException;

import es.csa.commons.faces.util.BeanValidatorHelper;
import es.csa.commons.faces.util.MensajesUtils;

/**
 * Validator using the hibernate validator libray.
 * 
 * @author <a href="mailto:jalcalde@csa.es>Josué Alcalde González &lt;jalcalde@csa.es&gt;</a>
 * @version 15/11/2007-18:01:37
 */
public class HibernateValidator implements Validator, Serializable {

  private static final long serialVersionUID = -4404982650309912679L;

  // -------------------------------------------------------------------------------------------------------- Constantes

  public static final String VALIDATOR_ID = "es.csa.commons.faces.validator.HibernateValidator";

  public static final String INCORRECT_MESSAGE_ID = "es.csa.commons.faces.validator.AttributeValidator.INCORRECT";

  // ------------------------------------------------------------------------------------------------------- Propiedades

  protected String _summaryFormat = null;

  protected String _detailFormat = null;

  // --------------------------------------------------------------------------------------------- Métodos de validación

  public void validate(FacesContext context, UIComponent component, Object convertedValue) throws ValidatorException {
    if (component instanceof EditableValueHolder) {
      // Validate input component
      EditableValueHolder input = (EditableValueHolder) component;
      try {
        ValueExpression valueExpression = component.getValueExpression("value");
        if (null != valueExpression) {
          BeanValidatorHelper beanValidator = BeanValidatorHelper.getInstance(context);
          String[] messages = beanValidator.validate(context, valueExpression, convertedValue);
          if (null != messages) {
            input.setValid(false);
            for (String validatorMessage : messages) {
              String label = MensajesUtils.getLabel(context, component);
              FacesMessage message = MensajesUtils.createMessage(context, getSummaryFormat(), getDetailFormat(),
                  FacesMessage.SEVERITY_ERROR, INCORRECT_MESSAGE_ID, validatorMessage, label);
              context.addMessage(component.getClientId(context), message);
            }
          }
        }
      } catch (ELException e) {
        throw new FacesException(e);
      }
    }
  }

  // ------------------------------------------------------------------------------------------------------- Propiedades

  public String getSummaryFormat() {
    return _summaryFormat;
  }

  public void setSummaryFormat(String format) {
    _summaryFormat = format;
  }

  public String getDetailFormat() {
    return _detailFormat;
  }

  public void setDetailFormat(String format) {
    _detailFormat = format;
  }

}
