package es.csa.commons.persistence.browser;

/*
 * #%L
 * Librerías comunes de Java Server Faces de CSA
 * %%
 * Copyright (C) 2015 CSA - Centro Regional de Servicios Avanzados S.A.
 * %%
 * This file is part of "Librerías comunes de CSA".
 * 
 * developed by: Centro Regional de Servicios Avanzados, S.A.
 * 
 * "Librerías comunes de CSA" is free software: you can redistribute it and/or modify it under the terms of:
 * - Apache License, Version 2.0 (the "License");  
 * - under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 * 
 * "Librerías comunes de CSA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or General Public License for more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE­2.0
 * 
 * You may obtain a copy of the General Public Licence at:
 * http://www.gnu.org/licenses/
 * #L%
 */
 
import java.util.ArrayList;
import java.util.List;

public class PagerDefaultImpl implements Pager {

  private static final long serialVersionUID = 5635967842274726702L;

  public static final int DEFAULT_PAGE_SIZE = 10;

  public static final int DEFAULT_PAGER_SIZE = 10;

  public static final int MIN_PAGE_SIZE = 1;

  public static final int MIN_PAGER_SIZE = 1;

  protected int pageSize;

  protected int pagerSize;

  protected int currentPage;

  protected int numRecords;

  public PagerDefaultImpl() {
    super();
    setPageSize(DEFAULT_PAGE_SIZE);
    setPagerSize(DEFAULT_PAGER_SIZE);
    setNumRecords(0);
  }

  public PagerDefaultImpl(int numRecords) {
    this();
    setNumRecords(numRecords);
  }

  public PagerDefaultImpl(int numRecords, int maxPageRecords) {
    this();
    setNumRecords(numRecords);
    setPageSize(maxPageRecords);
  }

  public PagerDefaultImpl(int numRecords, int maxPageRecords, int maxPages) {
    this();
    setNumRecords(numRecords);
    setPageSize(maxPageRecords);
    setPagerSize(maxPages);
  }

  @Override
  public int getCurrentPage() {
    return currentPage;
  }

  @Override
  public int getPageSize() {
    return pageSize;
  }

  @Override
  public int getPagerSize() {
    return pagerSize;
  }

  @Override
  public int getNumRecords() {
    return numRecords;
  }

  @Override
  public void setCurrentPage(int page) {
    // Establecer la página (mínimo: 1, máximo: número de páginas);
    this.currentPage = Math.min(Math.max(1, page), getNumPages());
  }

  @Override
  public void setPageSize(int pageSize) {
    if (this.pageSize == pageSize) return; 
    this.pageSize = Math.max(MIN_PAGE_SIZE, pageSize);
    // Resetear el número de página a 1
    setCurrentPage(1);
  }

  @Override
  public void setPagerSize(int pagerSize) {
    if (this.pagerSize == pagerSize) return; 
    this.pagerSize = Math.max(MIN_PAGER_SIZE, pagerSize);
  }

  @Override
  public void setNumRecords(int numRecords) {
    if (this.numRecords == numRecords) return; 
    this.numRecords = numRecords;
    // Resetear el número de página a 1
    setCurrentPage(1);
  }

  @Override
  public int getNumPages() {
    return getNumRecords() / getPageSize() + ((getNumRecords() % getPageSize() > 0) ? 1 : 0);
  }

  @Override
  public int getFirstDisplayedPage() {
    if (getPagerSize() > 0 && getCurrentPage() > 0) {
      int first = getCurrentPage() - (getPagerSize() / 2);
      //Limite superior
      first = getNumPages()-getPagerSize()+1<first ? getNumPages()-getPagerSize()+1 : first;
      //Limite inferior
      first = first < 1 ? 1 : first;
      return first;
    }
    // La primera página es la 1
    return 1;
  }

  @Override
  public int getLastDisplayedPage(){
    int last = getFirstDisplayedPage()+getPagerSize()-1;
    return last>getNumPages() ? getNumPages() : last;    
  }
  
  @Override
  public List<Integer> getPageNumbers(){
    List<Integer> numeros = new ArrayList<Integer>();
    for (int i = getFirstDisplayedPage(); i <= getLastDisplayedPage(); i++)
      numeros.add(i);
    return numeros;
  }
}
