package es.csa.commons.persistence.browser.hibernatecriteria;

/*
 * #%L
 * Librerías comunes de Java Server Faces de CSA
 * %%
 * Copyright (C) 2015 CSA - Centro Regional de Servicios Avanzados S.A.
 * %%
 * This file is part of "Librerías comunes de CSA".
 * 
 * developed by: Centro Regional de Servicios Avanzados, S.A.
 * 
 * "Librerías comunes de CSA" is free software: you can redistribute it and/or modify it under the terms of:
 * - Apache License, Version 2.0 (the "License");  
 * - under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 * 
 * "Librerías comunes de CSA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or General Public License for more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE­2.0
 * 
 * You may obtain a copy of the General Public Licence at:
 * http://www.gnu.org/licenses/
 * #L%
 */
 
import java.lang.reflect.ParameterizedType;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.criterion.CriteriaSpecification;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;

import es.csa.commons.persistence.browser.Browser;
import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.Pager;
import es.csa.commons.persistence.browser.PagerDefaultImpl;
import es.csa.commons.persistence.browser.Sort;

public class BrowserHibernateImpl<E> implements Browser<E, Criterion, Order> {

  private static final long serialVersionUID = -6888002879684142154L;

  protected transient EntityManager em;

  protected List<Alias> aliases;

  protected List<Filter<Criterion, E>> filters;

  protected Filter<Criterion, E> condition;

  protected List<Filter<Criterion, E>> conditions;

  protected Alias alias;

  protected Filter<Criterion, E> filter;

  protected Pager pager;

  protected Sort<Order> sort;

  protected transient List<E> records;

  protected Class<E> entityClass;

  @PersistenceContext
  protected void setEntityManager(EntityManager em) {
    this.em = em;
  }

  public EntityManager getEntityManager() {
    return this.em;
  }

  public BrowserHibernateImpl() {
    super();
    ParameterizedType genericSuperclass = (ParameterizedType) getClass().getGenericSuperclass();
    @SuppressWarnings("unchecked")
    Class<E> entityClass = (Class<E>) genericSuperclass.getActualTypeArguments()[0];
    this.entityClass = entityClass;
    this.aliases = new ArrayList<Alias>();
    this.filters = new ArrayList<Filter<Criterion, E>>();
    this.conditions = new ArrayList<Filter<Criterion, E>>();
    this.setCondition(new ConditionHibernateImpl<E>());
    this.setAlias(new AliasDefaultImpl());
    this.setFilter(new FilterHibernateImpl<E>(entityClass));
    this.setPager(new PagerDefaultImpl());
    this.setSort(new SortHibernateImpl());
    this.records = null;
  }

  public BrowserHibernateImpl(Class<E> entityClass) {
    super();
    this.entityClass = entityClass;
    this.aliases = new ArrayList<Alias>();
    this.filters = new ArrayList<Filter<Criterion, E>>();
    this.setAlias(new AliasDefaultImpl());
    this.setFilter(new FilterHibernateImpl<E>(entityClass));
    this.setPager(new PagerDefaultImpl());
    this.setSort(new SortHibernateImpl());
    this.records = null;
  }

  public void applyAlias() {
    aliases.add(getAlias());
    this.setAlias(new AliasDefaultImpl());
  }

  @Override
  public void applyFilter() {
    filters.add(getFilter());
    this.setFilter(new FilterHibernateImpl<E>(entityClass));
  }

  @Override
  public void applyCondition() {
    conditions.add(getCondition());
    this.setCondition(new ConditionHibernateImpl<E>());
  }

  public void clearAliases() {
    aliases.clear();
  }

  @Override
  public void clearFilters() {
    filters.clear();
  }

  @Override
  public void clearConditions() {
    conditions.clear();
  }

  @Override
  public List<Filter<Criterion, E>> getConditions() {
    return conditions;
  }

  public List<Alias> getAliases() {
    return aliases;
  }

  @Override
  public List<Filter<Criterion, E>> getFilters() {
    return filters;
  }

  @Override
  public Pager getPager() {
    return pager;
  }

  @Override
  public List<E> getRecords() {
    return records;
  }

  @Override
  public Sort<Order> getSort() {
    return sort;
  }

  public Alias getAlias() {
    return alias;
  }

  @Override
  public Filter<Criterion, E> getCondition() {
    return condition;
  }

  @Override
  public Filter<Criterion, E> getFilter() {
    return filter;
  }

  protected void setSort(Sort<Order> sort) {
    this.sort = sort;
  }

  protected void setPager(Pager pager) {
    this.pager = pager;
  }

  protected void setFilter(Filter<Criterion, E> filter) {
    this.filter = filter;
  }

  protected void setAlias(Alias alias) {
    this.alias = alias;
  }

  private void setCondition(Filter<Criterion, E> condition) {
    this.condition = condition;
  }

  @Override
  public void processRecords() {
    Criteria criteria = ((Session) em.getDelegate()).createCriteria(entityClass);
    criteria.setResultTransformer(CriteriaSpecification.DISTINCT_ROOT_ENTITY);
    for (Alias alias : getAliases())
      criteria.createAlias(alias.getAssociationPath(), alias.getAlias(), CriteriaSpecification.LEFT_JOIN);
    for (Filter<Criterion, E> c : getConditions())
      if (c != null && c.getCriterion() != null) criteria.add(c.getCriterion());
    for (Filter<Criterion, E> f : getFilters())
      if (f != null && f.getCriterion() != null) criteria.add(f.getCriterion());
    criteria.setProjection(Projections.rowCount());
    getPager().setNumRecords((Integer) criteria.uniqueResult());
    if (getPager().getNumRecords() == 0) {
      this.records = new ArrayList<E>();
      return;
    } else {
      criteria = ((Session) em.getDelegate()).createCriteria(entityClass);
      for (Alias alias : getAliases())
        criteria.createAlias(alias.getAssociationPath(), alias.getAlias(), CriteriaSpecification.LEFT_JOIN);
      for (Filter<Criterion, E> c : getConditions())
        if (c != null && c.getCriterion() != null) criteria.add(c.getCriterion());
      for (Filter<Criterion, E> f : getFilters())
        if (f != null && f.getCriterion() != null) criteria.add(f.getCriterion());
      if (getSort().getSortProperty() != null) criteria.addOrder(getSort().getOrder());
      criteria.setFirstResult((getPager().getCurrentPage() - 1) * getPager().getPageSize());
      criteria.setMaxResults(getPager().getPageSize());
      @SuppressWarnings("unchecked")
      List<E> records = criteria.list();
      this.records = records;
    }
  }

}
