package es.csa.commons.persistence.browser.hibernatecriteria;

/*
 * #%L
 * Librerías comunes de Java Server Faces de CSA
 * %%
 * Copyright (C) 2015 CSA - Centro Regional de Servicios Avanzados S.A.
 * %%
 * This file is part of "Librerías comunes de CSA".
 * 
 * developed by: Centro Regional de Servicios Avanzados, S.A.
 * 
 * "Librerías comunes de CSA" is free software: you can redistribute it and/or modify it under the terms of:
 * - Apache License, Version 2.0 (the "License");  
 * - under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 * 
 * "Librerías comunes de CSA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or General Public License for more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE­2.0
 * 
 * You may obtain a copy of the General Public Licence at:
 * http://www.gnu.org/licenses/
 * #L%
 */
 
import java.util.StringTokenizer;

import org.hibernate.criterion.Order;

import es.csa.commons.persistence.browser.Sort;

public class SortHibernateImpl implements Sort<Order> {

  private static final long serialVersionUID = 3699037319636009844L;

  // Provided data

  public boolean sortAsc;

  public String sortProperty;

  // Internal data

  /** La propiedad del filtro recortada solo a dos (para poder usar con hibernate). */
  protected String sortCutProperty;

  /** El criterion de hibernate. */
  protected transient Order sortOrder;

  public SortHibernateImpl() {
    super();
    sortAsc = true;
    sortOrder = null;
  }

  @Override
  public boolean getSortAsc() {
    return sortAsc;
  }

  @Override
  public void setSortAsc(boolean sortAsc) {
    sortOrder = null;
    this.sortAsc = sortAsc;
  }

  @Override
  public String getSortProperty() {
    return sortProperty;
  }

  @Override
  public void setSortProperty(String sortProperty) {
    sortOrder = null;
    this.sortProperty = sortProperty;
    this.sortCutProperty = cutProperty(sortProperty);
  }

  @Override
  public Order getOrder() {
    if (sortOrder == null)
      sortOrder = getSortAsc() ? Order.asc(sortCutProperty) : Order.desc(sortCutProperty);
    return sortOrder;
  }

  // -------------------------------------------------------------------------------------------------------- Utilidades

  /**
   * Devuelve las dos últimas propiedades de una cadena de varias separadas por puntos.
   * <p>
   * Ejemplo: acortarPropiedad("p1.p2.p3.p4.p5") => "p4.p5".
   * </p>
   * 
   * @param filterProperty
   *          La cadena con la propiedad con propiedades anidadas.
   * @return Las dos ultimas propiedades de la cadena.
   */
  public static String cutProperty(String property) {
    StringTokenizer st = new StringTokenizer(property, ".");
    int n = st.countTokens();
    if (n == 1) {
      return st.nextToken();
    } else {
      String[] aux = { st.nextToken(), st.nextToken() };
      while (st.hasMoreTokens()) {
        aux[0] = aux[1];
        aux[1] = st.nextToken();
      }
      return aux[0] + "." + aux[1];
    }
  }

}
