package es.csa.commons.persistence.browser.jpaql;

/*
 * #%L
 * Librerías comunes de Java Server Faces de CSA
 * %%
 * Copyright (C) 2015 CSA - Centro Regional de Servicios Avanzados S.A.
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * This file is part of "Librerías comunes de CSA".
 * 
 * developed by: Centro Regional de Servicios Avanzados, S.A.
 * developed by: Balidea <info@balidea.com>
 * 
 * "Librerías comunes de CSA" is free software: you can redistribute it and/or modify it under the terms of:
 * - Apache License, Version 2.0 (the "License");  
 * - under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 * 
 * "Librerías comunes de CSA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or General Public License for more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE­2.0
 * 
 * You may obtain a copy of the General Public Licence at:
 * http://www.gnu.org/licenses/
 * #L%
 */
 
import java.lang.reflect.ParameterizedType;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.hibernate.connection.UserSuppliedConnectionProvider;

import es.csa.commons.persistence.browser.Browser;
import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.Pager;
import es.csa.commons.persistence.browser.PagerDefaultImpl;
import es.csa.commons.persistence.browser.Sort;

public class BrowserJPAQL<E> implements Browser<E, CriterionJPAQL, String> {

  private static final long serialVersionUID = -6888002879684142154L;

  protected transient EntityManager em;

  protected List<Filter<CriterionJPAQL, E>> filters;

  protected Filter<CriterionJPAQL, E> condition;

  protected List<Filter<CriterionJPAQL, E>> conditions;

  protected Filter<CriterionJPAQL, E> filter;

  protected Pager pager;

  protected Sort<String> sort;

  protected transient List<E> records;

  protected Class<E> entityClass;

  @PersistenceContext
  protected void setEntityManager(EntityManager em) {
    this.em = em;
  }

  public EntityManager getEntityManager() {
    return this.em;
  }

  public BrowserJPAQL() {
    super();
    ParameterizedType genericSuperclass = (ParameterizedType) getClass().getGenericSuperclass();
    @SuppressWarnings("unchecked")
    Class<E> entityClass = (Class<E>) genericSuperclass.getActualTypeArguments()[0];
    this.entityClass = entityClass;
    this.filters = new ArrayList<Filter<CriterionJPAQL, E>>();
    this.conditions = new ArrayList<Filter<CriterionJPAQL, E>>();
    this.setCondition(new FilterJPAQLByQuery<E>());
    this.setFilter(new FilterJPAQLByProperty<E>(entityClass));
    this.setPager(new PagerDefaultImpl());
    this.setSort(new SortJPAQL());
    this.records = null;
  }

  public BrowserJPAQL(Class<E> entityClass) {
    super();
    this.entityClass = entityClass;
    this.filters = new ArrayList<Filter<CriterionJPAQL, E>>();
    this.conditions = new ArrayList<Filter<CriterionJPAQL, E>>();
    this.setCondition(new FilterJPAQLByQuery<E>());
    this.setFilter(new FilterJPAQLByProperty<E>(entityClass));
    this.setPager(new PagerDefaultImpl());
    this.setSort(new SortJPAQL());
    this.records = null;
  }

  @Override
  public void applyFilter() {
    filters.add(getFilter());
    //RF_XEA_12 --> para mantener el texto escrito en el filtro
    //this.setFilter(new FilterJPAQLByProperty<E>(entityClass));
  }

  @Override
  public void applyCondition() {
    conditions.add(getCondition());
    this.setCondition(new FilterJPAQLByQuery<E>());
  }

  @Override
  public void clearFilters() {
    filters.clear();
    //RF_XEA_12 --> para borrar el texto escrito en el filtro
    this.setFilter(new FilterJPAQLByProperty<E>(entityClass));
  }

  @Override
  public void clearConditions() {
    conditions.clear();
  }

  @Override
  public List<Filter<CriterionJPAQL, E>> getConditions() {
    return conditions;
  }

  @Override
  public List<Filter<CriterionJPAQL, E>> getFilters() {
    return filters;
  }

  @Override
  public Pager getPager() {
    return pager;
  }

  @Override
  public List<E> getRecords() {
    return records;
  }

  @Override
  public Sort<String> getSort() {
    return sort;
  }

  @Override
  public Filter<CriterionJPAQL, E> getCondition() {
    return condition;
  }

  @Override
  public Filter<CriterionJPAQL, E> getFilter() {
    return filter;
  }

  protected void setSort(Sort<String> sort) {
    this.sort = sort;
  }

  protected void setPager(Pager pager) {
    this.pager = pager;
  }

  protected void setFilter(Filter<CriterionJPAQL, E> filter) {
    this.filter = filter;
  }

  private void setCondition(Filter<CriterionJPAQL, E> condition) {
    this.condition = condition;
  }

  @Override
  public void processRecords() {
    String hqlQuery = "SELECT COUNT(e) FROM {0} e";
    hqlQuery = MessageFormat.format(hqlQuery, entityClass.getName());
    boolean where = true;
    for (Filter<CriterionJPAQL, E> c : getConditions()) {
      if (c.getCriterion() != null) {
        hqlQuery += (where ? " WHERE (" : " AND (") + c.getCriterion().getQuery() + ")";
        where = false;
      }
    }
    for (Filter<CriterionJPAQL, E> f : getFilters()) {
      if (f.getCriterion() != null) {
        hqlQuery += (where ? " WHERE (" : " AND (") + f.getCriterion().getQuery() + ")";
        where = false;
      }
    }
    Query query = em.createQuery(hqlQuery);
    int parameterPosition = 1;
    for (Filter<CriterionJPAQL, E> c : getConditions()) {
      if (c.getCriterion() != null) for (Object p : c.getCriterion().getParameters()) {
        query.setParameter(parameterPosition, p);
        parameterPosition++;
      }
    }
    for (Filter<CriterionJPAQL, E> f : getFilters()) {
      if (f.getCriterion() != null) for (Object p : f.getCriterion().getParameters()) {
        query.setParameter(parameterPosition, p);
        parameterPosition++;
      }
    }
    getPager().setNumRecords(((Number) query.getSingleResult()).intValue());
    if (getPager().getNumRecords() == 0) {
      this.records = new ArrayList<E>();
      return;
    } else {
      hqlQuery = "SELECT e FROM {0} e";
      hqlQuery = MessageFormat.format(hqlQuery, entityClass.getName());
      where = true;
      for (Filter<CriterionJPAQL, E> c : getConditions()) {
        if (c.getCriterion() != null) {
          hqlQuery += (where ? " WHERE (" : " AND (") + c.getCriterion().getQuery() + ")";
          where = false;
        }
      }
      for (Filter<CriterionJPAQL, E> f : getFilters()) {
        if (f.getCriterion() != null) {
          hqlQuery += (where ? " WHERE (" : " AND (") + f.getCriterion().getQuery() + ")";
          where = false;
        }
      }
      if (getSort().getOrder() != null) hqlQuery += " " + getSort().getOrder();
      query = em.createQuery(hqlQuery);
      parameterPosition = 1;
      for (Filter<CriterionJPAQL, E> c : getConditions()) {
        if (c.getCriterion() != null) for (Object p : c.getCriterion().getParameters()) {
          query.setParameter(parameterPosition, p);
          parameterPosition++;
        }
      }
      for (Filter<CriterionJPAQL, E> f : getFilters()) {
        if (f.getCriterion() != null)  for (Object p : f.getCriterion().getParameters()) {
          query.setParameter(parameterPosition, p);
          parameterPosition++;
        }
      }
      query.setFirstResult((getPager().getCurrentPage() - 1) * getPager().getPageSize());
      query.setMaxResults(getPager().getPageSize());
      @SuppressWarnings("unchecked")
      List<E> records = query.getResultList();
      this.records = records;      
    }
  }
}
