package es.csa.commons.persistence.entity;

/*
 * #%L
 * Librerías comunes de Java Server Faces de CSA
 * %%
 * Copyright (C) 2015 CSA - Centro Regional de Servicios Avanzados S.A.
 * %%
 * This file is part of "Librerías comunes de CSA".
 * 
 * developed by: Centro Regional de Servicios Avanzados, S.A.
 * 
 * "Librerías comunes de CSA" is free software: you can redistribute it and/or modify it under the terms of:
 * - Apache License, Version 2.0 (the "License");  
 * - under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 * 
 * "Librerías comunes de CSA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or General Public License for more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE­2.0
 * 
 * You may obtain a copy of the General Public Licence at:
 * http://www.gnu.org/licenses/
 * #L%
 */

import java.io.Serializable;
import java.sql.Blob;
import java.sql.SQLException;

import javax.persistence.Column;
import javax.persistence.Lob;
import javax.persistence.MappedSuperclass;
import javax.persistence.Transient;
import javax.sql.rowset.serial.SerialBlob;
import javax.sql.rowset.serial.SerialException;

import org.hibernate.validator.Length;

import es.csa.commons.util.FileUtils;

@MappedSuperclass
public abstract class AbstractFichero implements Serializable, IFichero {

  private static final long serialVersionUID = -3156935382182399591L;

  private String nombre;

  private String tipoMime;

  private Integer tamanio;

  private Blob contenido;

  // Encapsular el UploadItem
  private FicheroSubido ficheroSubido;

  public abstract long getId();

  @Length(max = 255)
  public String getNombre() {
    return isFicheroTemporal() ? getFicheroSubido().getNombre() : nombre;
  }

  public void setNombre(String nombreFichero) {
    this.nombre = nombreFichero;
  }

  @Length(max = 150)
  public String getTipoMime() {
    return isFicheroTemporal() ? getFicheroSubido().getTipoMime() : tipoMime;
  }

  public void setTipoMime(String tipoMime) {
    this.tipoMime = tipoMime;
  }

  public Integer getTamanio() {
    return isFicheroTemporal() ? getFicheroSubido().getTamanio() : tamanio;
  }

  public void setTamanio(Integer tamanio) {
    this.tamanio = tamanio;
  }

  @Transient
  public String getTamanioLegible() {
    return FileUtils.getHumanReadableSize(Long.valueOf(getTamanio()));
  }

  @Lob
  @Column(columnDefinition = "longblob")
  public Blob getContenido() {
    try {
      return isFicheroTemporal() ? new SerialBlob(getFicheroSubido().getContenido()) : contenido;
    } catch (SerialException e) {
      e.printStackTrace();
      return null;
    } catch (SQLException e) {
      e.printStackTrace();
      return null;
    }
  }

  public void setContenido(Blob contenido) {
    this.contenido = contenido;
  }

  @Transient
  public boolean isFicheroTemporal() {
    return this.getId() == 0 && getFicheroSubido() != null;
  }
  
  @Transient
  public FicheroSubido getFicheroSubido() {
    return ficheroSubido;
  }

  public void setFicheroSubido(FicheroSubido ficheroSubido) {
    this.ficheroSubido = ficheroSubido;
  }
}
