package es.csa.commons.persistence.entity;

/*
 * #%L
 * Librerías comunes de Java Server Faces de CSA
 * %%
 * Copyright (C) 2015 CSA - Centro Regional de Servicios Avanzados S.A.
 * %%
 * This file is part of "Librerías comunes de CSA".
 * 
 * developed by: Centro Regional de Servicios Avanzados, S.A.
 * 
 * "Librerías comunes de CSA" is free software: you can redistribute it and/or modify it under the terms of:
 * - Apache License, Version 2.0 (the "License");  
 * - under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 * 
 * "Librerías comunes de CSA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or General Public License for more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE­2.0
 * 
 * You may obtain a copy of the General Public Licence at:
 * http://www.gnu.org/licenses/
 * #L%
 */

import java.io.Serializable;
import java.util.Date;

import javax.persistence.PrePersist;
import javax.persistence.PreUpdate;

/**
 * Listener que crea y actualiza los datos de auditoria automaticamente para entidades JPA.
 * 
 * @author Josué Alcalde.
 */
public class AuditableListener implements Serializable {

  private static final long serialVersionUID = -1218767460467608089L;

  // -------------------------------------------------------------------------------------------------------- Utilidades

  @PrePersist
  public void create(Auditable entity) {
    if (entity != null) {
      entity.setAuditoria(new Auditoria());
      entity.getAuditoria().setFechaCreacion(new Date());
      entity.getAuditoria().setIdUsuarioCreacion(getUsuarioAuditoria());
      entity.getAuditoria().setProcesoCreacion(getProcesoAuditoria());
      entity.getAuditoria().setFechaActualizacion(new Date());
      entity.getAuditoria().setIdUsuarioActualizacion(getUsuarioAuditoria());
      entity.getAuditoria().setProcesoActualizacion(getProcesoAuditoria());
    }
  }

  @PreUpdate
  public void update(Auditable entity) {
    if (entity != null) {
      if (entity.getAuditoria() == null) {
        entity.setAuditoria(new Auditoria());
        entity.getAuditoria().setFechaCreacion(new Date());
        entity.getAuditoria().setIdUsuarioCreacion(getUsuarioAuditoria());
        entity.getAuditoria().setProcesoCreacion(getProcesoAuditoria());
      }
      entity.getAuditoria().setFechaActualizacion(new Date());
      entity.getAuditoria().setIdUsuarioActualizacion(getUsuarioAuditoria());
      entity.getAuditoria().setProcesoActualizacion(getProcesoAuditoria());
    }
  }

  // -------------------------------------------------------------------------------- Establecer datos para Thread Local

  private static ThreadLocal<Long> usuarioAuditoria = new ThreadLocal<Long>();

  private static ThreadLocal<String> procesoAuditoria = new ThreadLocal<String>();

  public static Long getUsuarioAuditoria() {
    return usuarioAuditoria.get();
  }

  public static void setUsuarioAuditoria(Long usuarioAuditoria) {
    AuditableListener.usuarioAuditoria.set(usuarioAuditoria);
  }

  public static String getProcesoAuditoria() {
    return procesoAuditoria.get();
  }

  public static void setProcesoAuditoria(String procesoAuditoria) {
    AuditableListener.procesoAuditoria.set(procesoAuditoria);
  }

  public static void setDatosAuditoria(Long usuarioAuditoria, String procesoAuditoria) {
    setUsuarioAuditoria(usuarioAuditoria);
    setProcesoAuditoria(procesoAuditoria);
  }

}
