package es.csa.commons.persistence.entity;

/*
 * #%L
 * Librerías comunes de Java Server Faces de CSA
 * %%
 * Copyright (C) 2015 CSA - Centro Regional de Servicios Avanzados S.A.
 * %%
 * This file is part of "Librerías comunes de CSA".
 * 
 * developed by: Centro Regional de Servicios Avanzados, S.A.
 * 
 * "Librerías comunes de CSA" is free software: you can redistribute it and/or modify it under the terms of:
 * - Apache License, Version 2.0 (the "License");  
 * - under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 * 
 * "Librerías comunes de CSA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or General Public License for more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE­2.0
 * 
 * You may obtain a copy of the General Public Licence at:
 * http://www.gnu.org/licenses/
 * #L%
 */

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Embeddable;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.validator.Digits;
import org.hibernate.validator.Size;

/**
 * Parte de datos auditables que se encuentran en todas las entidades.
 * 
 * @author Josué Alcalde <jalcalde@csa.es>
 */
@Embeddable
public class Auditoria implements Serializable {

  private static final long serialVersionUID = 3322409987124224L;

  // --------------------------------------------------------------------------------------------------------- Atributos

  private Long idUsuarioCreacion;

  private Date fechaCrecion;

  private String procesoCreacion;

  private Long idUsuarioActualizacion;

  private Date fechaActualizacion;

  private String procesoActualizacion;

  // ------------------------------------------------------------------------------------------------------- Propiedades

  @Temporal(TemporalType.TIMESTAMP)
  public Date getFechaActualizacion() {
    return fechaActualizacion;
  }

  public void setFechaActualizacion(Date fechaActualizacion) {
    this.fechaActualizacion = fechaActualizacion;
  }

  @Temporal(TemporalType.TIMESTAMP)
  public Date getFechaCreacion() {
    return fechaCrecion;
  }

  public void setFechaCreacion(Date fechaCrecion) {
    this.fechaCrecion = fechaCrecion;
  }

  @Digits(integerDigits = 10, fractionalDigits = 0)
  public Long getIdUsuarioActualizacion() {
    return idUsuarioActualizacion;
  }

  public void setIdUsuarioActualizacion(Long idUsuarioActualizacion) {
    this.idUsuarioActualizacion = idUsuarioActualizacion;
  }

  @Digits(integerDigits = 10, fractionalDigits = 0)
  public Long getIdUsuarioCreacion() {
    return idUsuarioCreacion;
  }

  public void setIdUsuarioCreacion(Long idUsuarioCreacion) {
    this.idUsuarioCreacion = idUsuarioCreacion;
  }

  @Size(max = 100)
  public String getProcesoActualizacion() {
    return procesoActualizacion;
  }

  public void setProcesoActualizacion(String proceso) {
    int length = proceso == null ? 0 : proceso.length();
    this.procesoActualizacion = length > 100 ? proceso.substring(length - 100, length) : proceso;
  }

  @Size(max = 100)
  public String getProcesoCreacion() {
    return procesoCreacion;
  }

  public void setProcesoCreacion(String proceso) {
    int length = proceso == null ? 0 : proceso.length();
    this.procesoCreacion = length > 100 ? proceso.substring(length - 100, length) : proceso;
  }

}
