package es.csa.commons.persistence.service;

/*
 * #%L
 * Librerías comunes de Java Server Faces de CSA
 * %%
 * Copyright (C) 2015 CSA - Centro Regional de Servicios Avanzados S.A.
 * %%
 * This file is part of "Librerías comunes de CSA".
 * 
 * developed by: Centro Regional de Servicios Avanzados, S.A.
 * 
 * "Librerías comunes de CSA" is free software: you can redistribute it and/or modify it under the terms of:
 * - Apache License, Version 2.0 (the "License");  
 * - under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 * 
 * "Librerías comunes de CSA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or General Public License for more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE­2.0
 * 
 * You may obtain a copy of the General Public Licence at:
 * http://www.gnu.org/licenses/
 * #L%
 */
 
import java.io.Serializable;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import javax.persistence.EntityManager;

public interface AbstractService<E, ID extends Serializable> {

  public abstract EntityManager getEntityManager();

  public abstract <T> void persist(T entidad);

  public abstract <T> void persistAll(Collection<T> entidades);

  public abstract <T> void persistAll(T... entidades);

  public abstract <T> void remove(T entitdad);

  public abstract <T> void removeAll(Collection<T> entidades);

  public abstract <T> void removeAll(Class<T> tipoEntidad, T... entidades);

  public abstract <T> T merge(T entitdad);

  public abstract <T> Collection<T> mergeAll(Collection<T> entidades);

  public abstract <T> Collection<T> mergeAll(T... entidades);

  public abstract <T> boolean contains(T entidad);

  public abstract <T> void refresh(T entidad);

  public abstract <T> void clear();

  public abstract int executeUpdate(String strQuery);

  public abstract int executeUpdate(String strQuery, Map<String, ?> params);

  public abstract int executeNamedUpdate(String queryName);

  public abstract int executeNamedUpdate(String queryName, Map<String, ?> params);

  public abstract E find(ID id);

  public abstract <T, K extends Serializable> T find(Class<T> entityClass, K id);

  public abstract List<E> findAll();

  public abstract <T> List<T> findAll(Class<T> entityClass);

  public abstract List<E> findAllOrder(String orden, boolean ascendente);

  public abstract <T> List<T> findAllOrder(Class<T> entityClass, String orden, boolean ascendente);

  public abstract E findOneWhere(String propiedad, Object valor);

  public abstract <T> T findOneWhere(Class<T> entityClass, String propiedad, Object valor);

  public abstract E findOneWhere(Map<String, Object> propiedades);

  public abstract <T> T findOneWhere(Class<T> entityClass, Map<String, Object> propiedades);

  public abstract List<E> findWhere(String propiedad, Object valor);

  public abstract <T> List<T> findWhere(Class<T> entityClass, String propiedad, Object valor);

  public abstract List<E> findWhere(Map<String, Object> propiedades);

  public abstract <T> List<T> findWhere(Class<T> entityClass, Map<String, Object> propiedades);

  public abstract List<E> findWhereOrder(String propiedad, Object valor, String orden, boolean ascendente);

  public abstract <T> List<T> findWhereOrder(Class<T> entityClass, String propiedad, Object valor, String orden,
      boolean ascendente);

  public abstract List<E> findWhereOrder(Map<String, Object> propiedades, String orden, boolean ascendente);

  public abstract <T> List<T> findWhereOrder(Class<T> entityClass, Map<String, Object> propiedades, String orden,
      boolean ascendente);

  public abstract List<E> findByQuery(String strQuery);

  public abstract <T> List<T> findByQuery(Class<T> entityClass, String strQuery);

  public abstract List<E> findByQuery(String strQuery, Map<String, ? extends Object> params);

  public abstract <T> List<T> findByQuery(Class<T> entityClass, String strQuery, Map<String, ? extends Object> params);

  public abstract List<E> findByNamedQuery(String queryName);

  public abstract <T> List<T> findByNamedQuery(Class<T> entityClass, String queryName);

  public abstract List<E> findByNamedQuery(String queryName, Map<String, ? extends Object> params);

  public abstract <T> List<T> findByNamedQuery(Class<T> entityClass, String queryName,
      Map<String, ? extends Object> params);

}
