package es.csa.commons.persistence.validator;

/*
 * #%L
 * Librerías comunes de Java Server Faces de CSA
 * %%
 * Copyright (C) 2015 CSA - Centro Regional de Servicios Avanzados S.A.
 * %%
 * This file is part of "Librerías comunes de CSA".
 * 
 * developed by: Centro Regional de Servicios Avanzados, S.A.
 * 
 * "Librerías comunes de CSA" is free software: you can redistribute it and/or modify it under the terms of:
 * - Apache License, Version 2.0 (the "License");  
 * - under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 * 
 * "Librerías comunes de CSA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or General Public License for more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE­2.0
 * 
 * You may obtain a copy of the General Public Licence at:
 * http://www.gnu.org/licenses/
 * #L%
 */
 
public class NIFUtils {

  public static final String DEFAULT_RESOURCE_BUNDLE = "es.csa.commons.persistence.resources.ValidatorMessages";

  /** Letras del NIF para Entidades Juridicas Nacionales. */
  public static final String letrasEntidadesNacionales = "ABCDEFGHJUV";

  /** Letras del NIF para Entidades Juridicas Extranjeras. */
  public static final String letrasEntidadesExtranjeras = "NW";

  /** Letras del NIF para Corporaciones Locales. */
  public static final String letrasCorporacionesLocales = "P";

  /** Letras del NIF para Corporaciones Locales. */
  public static final String letrasOrganismosPublicos = "Q";

  /** Letras del NIF para Congregaciones Religiosas. */
  public static final String letrasCongregacionesReligiosas = "R";

  /** Letras del NIF para Órganos del Estado. */
  public static final String letrasOrganosEstado = "S";

  /** Letras con letra de control. */
  public static final String letrasConLetraControl = letrasEntidadesExtranjeras + letrasCorporacionesLocales
      + letrasOrganismosPublicos + letrasCongregacionesReligiosas + letrasOrganosEstado;

  /** Letras con número de control. */
  public static final String letrasConNumeroControl = letrasEntidadesNacionales;

  /** Letras del NIF para entidades jurídicas. */
  public static final String letrasEntidadesJuridicas = letrasConLetraControl + letrasConNumeroControl;

  /** Letras del NIF para NIEs */
  public static final String letrasNIE = "XYZ";

  /** Letras del NIF para españoles menores de 14 sin DNI. */
  public static final String letrasSinDNIMenores = "K";

  /** Letras del NIF para españoles sin DNI no residentes. */
  public static final String letrasSinDNINoResidentes = "L";

  /** Letras del NIF para extranjeros sin NIE no residentes. */
  public static final String letrasSinNIENoResidentes = "M";

  /** Letras del NIF para personas físicas. */
  public static final String letrasPersonasFisicas = letrasNIE + letrasSinDNIMenores + letrasSinDNINoResidentes
      + letrasSinNIENoResidentes;

  /** Letras de control del NIF Personas Físicas. */
  public static final String controlEntidadesJuridicas = "ABCDEFGHIJ";

  /** Letras de control del NIF Entidades. */
  public static final String controlPersonasFisicas = "TRWAGMYFPDXBNJZSQVHLCKE";

  /** Letras de ambos tipos. */
  public static final String letras = letrasEntidadesJuridicas + letrasPersonasFisicas;

  /** Letras de control de ambos tipos. */
  public static final String control = controlEntidadesJuridicas + controlPersonasFisicas;

  public static boolean isValidNIF(String nif) {
    if (!isValid(nif))
    	return false;

    return true;
  }

  public static boolean isValidLengthAndFormat(String nif) {
    // Comprobar longitud
    if (nif == null || nif.length() != 9)
    	return false;

    // Comprobar si el primero es número o una letra de las validas
    if (!Character.isDigit(nif.charAt(0)) && letras.indexOf(nif.charAt(0)) < 0)
    	return false;

    // Comprobar que los caracteres intermedios son digitos
    for (int i = 1; i < 8; i++)
      if (!Character.isDigit(nif.charAt(i))) return false;

    // Comprobar que el ultimo es numero o letra de control válida
    if (!Character.isDigit(nif.charAt(8)) && control.indexOf(nif.charAt(8)) < 0)
    	return false;
    
    // Formato del NIF correcto (queda validar control)
    return true;
  }

  public static boolean isValid(String nif) {
    // Validar formato para evitar excepciones
    if (!isValidLengthAndFormat(nif))
    	return false;
    
    if (isPersonaFisica_(nif)) {
      // Personas Físicas
      return isValidPersonaFisica_(nif);
    }
    /* 3.4.3-R RF_XEA_02 
     * Se comenta esta funcion dado que no se contempla la posibilidad de entidades juridicas
    if (isEntidadJuridica_(nif)) {
      //Entidades jurídicas
      return isValidEntidadJuridica_(nif);
    }
    */

    return false;
  }

  public static boolean isPersonaFisica(String nif) {
    // Validar formato para evitar excepciones
    if (!isValidLengthAndFormat(nif))
    	return false;

    return (isPersonaFisica_(nif));
  }

  public static boolean isEntidadJuridica(String nif) {
    // Validar formato para evitar excepciones
    if (!isValidLengthAndFormat(nif))
    	return false;

    return (isEntidadJuridica_(nif));
  }

  public static boolean isValidEntidadJuridica(String nif) {
    if (!isValidLengthAndFormat(nif))
    	return false;

    return isValidEntidadJuridica_(nif);
  }

  public static boolean isValidPersonaFisica(String nif) {
    if (!isValidLengthAndFormat(nif))
    	return false;

    return isValidPersonaFisica_(nif);
  }

  // antes de usar, comprobar que tiene formato válido
  private static boolean isPersonaFisica_(String nif) {
    return (Character.isDigit(nif.charAt(0)) || letrasPersonasFisicas.indexOf(nif.charAt(0)) >= 0) &&
    	   (!Character.isDigit(nif.charAt(8)) && controlPersonasFisicas.indexOf(nif.charAt(8)) >= 0);
  }

  // antes de usar, comprobar que tiene formato válido
  private static boolean isEntidadJuridica_(String nif) {
    return (!Character.isDigit(nif.charAt(0)) && letrasEntidadesJuridicas.indexOf(nif.charAt(0)) >= 0) &&
    	   (Character.isDigit(nif.charAt(8)) || controlEntidadesJuridicas.indexOf(nif.charAt(8)) >= 0);
  }

  // antes de usar, comprobar que tiene formato válido
  private static boolean isValidEntidadJuridica_(String nif) {
    if (nif.length() != 9)
    	throw new IllegalArgumentException();

    for (int i = 1; i < 8; i++)
      if (!Character.isDigit(nif.charAt(i)))
    	  throw new IllegalArgumentException();

    // El algoritmo consiste en sumar todas las nifras pares por un lado.
    // Por otro lado, sumar las impares multiplicadas por 2. En este caso si salen nifras mayores
    // que 10 se suman sus nifras; esto equivale a restar 9.
    // Ambas sumas se vuelven a sumar y se restan las unidades resultantes de esa nifra de diez.
    // El número resultante es el código de control.

    // Nosotros haremos ambas sumas en una (puesto que ambas sumas se suman también y aprovechamos
    // la propiedad asociativa).

    // Considerar únicamente los 7 dígitos numéricos situados en las posiciones 2 a 8 del NIF.
    int r = 0;
    for (int i = 0; i < 7; i++) {
      int n = Integer.parseInt(Character.toString(nif.charAt(i + 1)));
      if (i % 2 == 0) {
        n *= 2;
        if (n > 9) {
          n -= 9;
        }
      }
      r += n;
    }
    int control = 10 - (r % 10);

    // Las hay con número de control y otras con letra de control
    if (letrasConNumeroControl.indexOf(nif.charAt(0)) >= 0) {
      return '0' + control == nif.charAt(8);
    }

    if (letrasConLetraControl.indexOf(nif.charAt(0)) >= 0) {
      return controlEntidadesJuridicas.charAt(control - 1) == nif.charAt(8);
    }

    return false;
  }

  // antes de usar, comprobar que tiene formato válido
  private static boolean isValidPersonaFisica_(String nif) {
    if (letrasPersonasFisicas.indexOf(nif.charAt(0)) >= 0) {
      if (letrasNIE.indexOf(nif.charAt(0)) >= 0)
        nif = letrasNIE.indexOf(nif.charAt(0)) + nif.substring(1, 9);
      else
        nif = "0" + nif.substring(1, 9);
    }

    return nif.charAt(8) == controlPersonasFisicas.charAt(Integer.parseInt(nif.substring(0, 8)) % 23);
  }
}