package es.csa.commons.util;

/*
 * #%L
 * Librerías comunes de Java Server Faces de CSA
 * %%
 * Copyright (C) 2015 CSA - Centro Regional de Servicios Avanzados S.A.
 * %%
 * This file is part of "Librerías comunes de CSA".
 * 
 * developed by: Centro Regional de Servicios Avanzados, S.A.
 * 
 * "Librerías comunes de CSA" is free software: you can redistribute it and/or modify it under the terms of:
 * - Apache License, Version 2.0 (the "License");  
 * - under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 * 
 * "Librerías comunes de CSA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or General Public License for more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE­2.0
 * 
 * You may obtain a copy of the General Public Licence at:
 * http://www.gnu.org/licenses/
 * #L%
 */
 
import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

public class ClassUtils {

  /**
   * Devuelve las dos últimas propiedades de una cadena de varias separadas por puntos.
   * <p>
   * Ejemplo: acortarPropiedad("p1.p2.p3.p4.p5") => "p4.p5".
   * </p>
   * 
   * @param property
   *          La cadena con la propiedad con propiedades anidadas.
   * @return Las dos ultimas propiedades de la cadena.
   */
  public static String cutProperty(String property) {
    StringTokenizer st = new StringTokenizer(property, ".");
    int numero = st.countTokens();
    if (numero == 1) {
      return st.nextToken();
    } else {
      String[] aux = { st.nextToken(), st.nextToken() };
      while (st.hasMoreTokens()) {
        aux[0] = aux[1];
        aux[1] = st.nextToken();
      }
      return aux[0] + "." + aux[1];
    }
  }

  /**
   * Obtiene la clase de una propiedad dada la clase a la que pertenece.
   * 
   * @param property
   *          Una propiedad dentro de la claseEntidad.
   * @param entityClass
   *          Una clase.
   * @return La clase de la propiedad. Si se trata de un tipo primitivo devuelve su equivalente para objetos.
   */
  public static Class<?> getPropertyClass(String property, Class<?> entityClass) {
    // Obtener la clase de la columna a filtrar
    // Las columnas puede estar anidadas del tipo (usuario.nombre)
    StringTokenizer st = new StringTokenizer(property, ".");
    List<String> propiedades = new ArrayList<String>();
    while (st.hasMoreTokens())
      propiedades.add(st.nextToken());
    for (String prop : propiedades) {
      BeanInfo beanInfo;
      try {
        beanInfo = Introspector.getBeanInfo(entityClass, Object.class);
      } catch (IntrospectionException e) {
        // TODO Gestionar o loguear la excepción
        return null;
      }
      PropertyDescriptor[] propDescs = beanInfo.getPropertyDescriptors();
      for (PropertyDescriptor propDesc : propDescs) {
        if (propDesc.getName().equals(prop)) {
          entityClass = propDesc.getPropertyType();
          break;
        }
      }
    }
    // Mapear tipos primitivos a su clase wrapper
    if (entityClass.equals(byte.class)) {
      return Byte.class;
    } else if (entityClass.equals(short.class)) {
      return Short.class;
    } else if (entityClass.equals(int.class)) {
      return Integer.class;
    } else if (entityClass.equals(long.class)) {
      return Long.class;
    } else if (entityClass.equals(float.class)) {
      return Float.class;
    } else if (entityClass.equals(double.class)) {
      return Double.class;
    } else if (entityClass.equals(char.class)) {
      return Character.class;
    } else if (entityClass.equals(boolean.class)) {
      return Boolean.class;
    }
    return entityClass;
  }

  /**
   * Comprueba si una clase es subclase de otras.
   * 
   * @param clazz
   *          La clase.
   * @param subclase
   *          La subclase candidata.
   * @return Si la clase es subclase de otra.
   */
  public static boolean isSubclass(Class<?> clazz, Class<?> subclazz) {
    try {
      clazz.asSubclass(subclazz);
    } catch (ClassCastException e) {
      return false;
    }
    return true;
  }

}
