package es.csa.commons.util;

/*
 * #%L
 * Librerías comunes de Java Server Faces de CSA
 * %%
 * Copyright (C) 2015 CSA - Centro Regional de Servicios Avanzados S.A.
 * %%
 * This file is part of "Librerías comunes de CSA".
 * 
 * developed by: Centro Regional de Servicios Avanzados, S.A.
 * 
 * "Librerías comunes de CSA" is free software: you can redistribute it and/or modify it under the terms of:
 * - Apache License, Version 2.0 (the "License");  
 * - under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 * 
 * "Librerías comunes de CSA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or General Public License for more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE­2.0
 * 
 * You may obtain a copy of the General Public Licence at:
 * http://www.gnu.org/licenses/
 * #L%
 */
 
import java.util.Enumeration;
import java.util.ResourceBundle;

/**
 * Un resource bundle compuesto de varios. Se puede crear con una lista de bundles y utilizará cada uno de ellos en
 * orden para buscar la propiedad correspondiente.
 * 
 * @author Josué Alcalde González
 */
public class CompoundResourceBundle extends ResourceBundle {

  private ResourceBundle[] bundles;

  private transient CompoundEnumeration<String> keys;

  @SuppressWarnings("unchecked")
  public CompoundResourceBundle(ResourceBundle... bundles) {
    this.bundles = bundles;
    Enumeration<String>[] keys = new Enumeration[bundles.length];
    for (int i = 0; i < bundles.length; i++)
      keys[i] = bundles[i].getKeys();
    this.keys = new CompoundEnumeration<String>(keys);
  }

  @Override
  public Enumeration<String> getKeys() {
    return keys;
  }

  @Override
  protected Object handleGetObject(String key) {
    for (ResourceBundle rb : bundles) {
      if (rb.containsKey(key)) return rb.getObject(key);
    }
    return null;
  }

}
