package es.csa.commons.util;

/*
 * #%L
 * Librerías comunes de Java Server Faces de CSA
 * %%
 * Copyright (C) 2015 CSA - Centro Regional de Servicios Avanzados S.A.
 * %%
 * This file is part of "Librerías comunes de CSA".
 * 
 * developed by: Centro Regional de Servicios Avanzados, S.A.
 * 
 * "Librerías comunes de CSA" is free software: you can redistribute it and/or modify it under the terms of:
 * - Apache License, Version 2.0 (the "License");  
 * - under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 * 
 * "Librerías comunes de CSA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or General Public License for more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE­2.0
 * 
 * You may obtain a copy of the General Public Licence at:
 * http://www.gnu.org/licenses/
 * #L%
 */
 
public class FileUtils {

  /**
   * Quita la ruta del nombre de un fichero y devuelve solo el nombre.
   * <p>
   * Su utilidad es eliminar la ruta en el nombre del fichero cuando Internet Explorer la envía.
   * </p>
   * 
   * @param filePath
   *          El nombre con ruta.
   * @return Solo el nombre de un fichero.
   */
  public static String removePath(String filePath) {
    // IE manda toda la ruta del ficehro :(
    return filePath.substring(filePath.lastIndexOf('\\') + 1, filePath.length());
  }

  /**
   * Traduce un tamaño bytes en algo legible por un humano.
   * 
   * @param size
   *          El tamaño en bytes.
   * @return Un número con un sufijo de bytes, B, kbytes, KB o megabytes, MB.
   */
  public static String getHumanReadableSize(Long size) {
    if (size == null) return "";
    if (size < 1024) return size.toString() + " B";
    Long kb = size / 1024;
    if (kb < 1024) return kb.toString() + " KB";
    Long mb = kb / 1024;
    return mb.toString() + " MB";
  }

}
