package es.csa.fdt.cyldigital.calendario.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.NotNull;
import org.springframework.context.i18n.LocaleContextHolder;

import es.csa.fdt.cyldigital.language.entity.ActividadLanguage;
import es.csa.fdt.cyldigital.servicios.entity.Actividad;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;

/**
 * Clase de Eventos de Calendario de Actividades.
 * 
 * @author csa
 *
 */
@Entity
@Table(name = "actividadcalendaritems")
@AttributeOverride(name = "id", column = @Column(name = "idCalendarItem"))
@Audited
@PrimaryKeyJoinColumn(name = "idCalendarItem")
public class ActividadCalendarItem extends CalendarItem implements Serializable {

  /** Constant serialVersionUID. */
  private static final long serialVersionUID = -1202724109856721337L;

  /** La actividad. */
  private Actividad actividad;

  /** El usuario. */
  private Usuario usuario;

  /**
   * Enumeración con los tipos de actividades.
   * @author csa
   *
   */
  public static enum ActividadType {
    
    /** ASESORAMIENTO. */
    ASESORAMIENTO,
    
    /** WIFI. */
    WIFI,
    
    /** PRACTICA taller. */
    PRACTICA_TALLER,
    
    /** CURSO. */
    CURSO,
    
    /** TALLER. */
    TALLER,
    
    /** CHARLA. */
    CHARLA,
    
    /** NAVEGACION. */
    NAVEGACION,
    
    /** EXTERNA */
    EXTERNA;
  }

  /** El tipo de actividad. */
  private ActividadType actividadType;

  /**
   * Enumeración de los tipos de excepciones.
   * @author csa
   *
   */
  public static enum Excepcion {
    
    /** SUSPENDIDA. */
    SUSPENDIDA,
    
    /** SISTEMA. */
    SISTEMA,
    
    /** OTRO. */
    OTRO;
  }

  /** La excepcion. */
  private Excepcion excepcion;

  /**
   * Construye un nuevo objeto de la clase
   */
  public ActividadCalendarItem() {
    super();
  }

  /**
   * Devuelve la actividad.
   * 
   * @return la actividad
   */
  @ManyToOne(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
  @JoinColumn(name = "idActividad", nullable = false)
  @NotNull
  public Actividad getActividad() {
    return actividad;
  }

  /**
   * Asigna la actividad.
   * 
   * @param actividad
   *          la actividad
   */
  public void setActividad(Actividad actividad) {
    this.actividad = actividad;
  }

  /**
   * Devuelve el usuario.
   * 
   * @return el usuario
   */
  @ManyToOne
  @JoinColumn(name = "idUsuario")
  public Usuario getUsuario() {
    return usuario;
  }

  /**
   * Asigna el usuario.
   * 
   * @param usuario
   *          el usuario
   */
  public void setUsuario(Usuario usuario) {
    this.usuario = usuario;
  }

  /**
   * Devuelve el tipo de actividad.
   * 
   * @return el tipo de actividad
   */
  @Enumerated(value = EnumType.STRING)
  public ActividadType getActividadType() {
    return actividadType;
  }

  /**
   * Asigna el tipo de actividad.
   * 
   * @param actividadType
   *          el tipo de actividad
   */
  public void setActividadType(ActividadType actividadType) {
    this.actividadType = actividadType;
    if (actividadType != null) {
      if (actividadType.equals(ActividadType.CURSO)) setTipo(CalendarItemType.HORARIO_CURSO);
      if (actividadType.equals(ActividadType.TALLER)) setTipo(CalendarItemType.HORARIO_TALLER);
      if (actividadType.equals(ActividadType.CHARLA)) setTipo(CalendarItemType.HORARIO_CHARLA);
      if (actividadType.equals(ActividadType.ASESORAMIENTO)) setTipo(CalendarItemType.HORARIO_ASESORAMIENTO);
      if (actividadType.equals(ActividadType.EXTERNA)) setTipo(CalendarItemType.HORARIO_EXTERNA);
    }
  }

  /**
   * Devuelve la excepcion.
   * 
   * @return la excepcion
   */
  @Enumerated(value = EnumType.STRING)
  public Excepcion getExcepcion() {
    return excepcion;
  }

  /**
   * Asigna la excepcion.
   * 
   * @param excepcion
   *          la excepcion
   */
  public void setExcepcion(Excepcion excepcion) {
    this.excepcion = excepcion;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.calendario.entity.CalendarItem#getDescription()
   */
  @Override
  @Transient
  public String getDescription() {
	  String nombreActividad = "";

	  if (getActividad() != null) {
		  String languageISO = LocaleContextHolder.getLocaleContext().getLocale().toString();
 
		  for (ActividadLanguage al : getActividad().getTextos()) {
			  if (al.getLanguage().getISO().compareTo(languageISO) == 0) {
				  nombreActividad = al.getNombre();
				  
				  break;
			  }
		  }
	  }			  

	  return (super.getDescription() != null && !super.getDescription().trim().isEmpty()) ?
			      super.getDescription()
			  :
				getUsuario() == null ?
				    (getActividad() == null ?
					    ""
					:
					    actividadType.toString().trim().replace("_", " ") + ": " + nombreActividad)
				:
				    "Asesoramiento al Usuario: " + getUsuario().getNombreCompleto();
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.calendario.entity.CalendarItem#getTitle()
   */
  @Override
  @Transient
  public String getTitle() {
    return (super.getTitle() != null && !super.getTitle().trim().isEmpty()) ? super.getTitle()
        : (actividadType != null ? actividadType.toString().trim().replace("_", " ") : null);
  }
}
