package es.csa.fdt.cyldigital.calendario.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.servicios.entity.Actividad;
import es.csa.fdt.cyldigital.servicios.entity.Charla;
import es.csa.fdt.cyldigital.servicios.entity.Curso;
import es.csa.fdt.cyldigital.servicios.entity.Taller;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;

/**
 * Clase de Asistencias de Calendario.
 * 
 * @author csa
 */
@Entity
@Table(name = "asistencias")
@AttributeOverride(name = "id", column = @Column(name = "idCalendarItem"))
@Audited
@PrimaryKeyJoinColumn(name = "idCalendarItem")
public class AsistenciaCalendarItem extends CalendarItem implements Serializable {

  private static final long serialVersionUID = -1202724109856721337L;

  /** actividad. */
  private Actividad actividad;

  /** usuario. */
  private Usuario usuario;

  /** automatica. */
  private boolean automatica;

  /**
   * Enumeración del tipo de excepciones.
   * @author csa
   *
   */
  public static enum Excepcion {
    
    /** SIN. */
    SIN,
    
    /** JUSTIFICADA. */
    JUSTIFICADA,
    
    /** SUSPENDIDA. */
    SUSPENDIDA,
    
    /** SISTEMA. */
    SISTEMA;
  }

  /** excepcion. */
  private Excepcion excepcion;

  /** detalles. */
  private String detalles;

  /**
   * Instancia un nueva asistencia en el calendario.
   */
  public AsistenciaCalendarItem() {
    super();
    setTipo(CalendarItemType.ASISTENCIA);
  }

  /**
   * Devuelve actividad.
   * 
   * @return actividad
   */
  @ManyToOne(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
  @JoinColumn(name = "idActividad", nullable = false)
  @NotNull
  public Actividad getActividad() {
    return actividad;
  }

  /**
   * Asigna actividad.
   * 
   * @param actividad
   *          actividad
   */
  public void setActividad(Actividad actividad) {
    this.actividad = actividad;
  }

  /**
   * Devuelve usuario.
   * 
   * @return usuario
   */
  @ManyToOne(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
  @JoinColumn(name = "idUsuario", nullable = false)
  @NotNull
  public Usuario getUsuario() {
    return usuario;
  }

  /**
   * Asigna usuario.
   * 
   * @param usuario
   *          usuario
   */
  public void setUsuario(Usuario usuario) {
    this.usuario = usuario;
  }

  /**
   * Comprueba si automatica.
   * 
   * @return automatica
   */
  public boolean isAutomatica() {
    return automatica;
  }

  /**
   * Asigna automatica.
   * 
   * @param automatica
   *          automatica
   */
  public void setAutomatica(boolean automatica) {
    this.automatica = automatica;
  }

  /**
   * Devuelve excepcion.
   * 
   * @return excepcion
   */
  @Enumerated(value = EnumType.STRING)
  public Excepcion getExcepcion() {
    return excepcion;
  }

  /**
   * Asigna excepcion.
   * 
   * @param excepcion
   *          excepcion
   */
  public void setExcepcion(Excepcion excepcion) {
    this.excepcion = excepcion;
  }

  /**
   * Devuelve detalles.
   * 
   * @return detalles
   */
  @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getDetalles() {
    return detalles;
  }

  /**
   * Asigna detalles.
   * 
   * @param detalles
   *          detalles
   */
  public void setDetalles(String detalles) {
    this.detalles = detalles;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.calendario.entity.CalendarItem#getDescription()
   */
  @Override
  @Transient
  public String getDescription() {
    return (super.getDescription() != null && !super.getDescription().trim().isEmpty()) ? super.getDescription()
        : (getActividadType() != null ? "Asistencia al " + getActividadType() : null);
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.calendario.entity.CalendarItem#getTitle()
   */
  @Override
  @Transient
  public String getTitle() {
    return (super.getTitle() != null && !super.getTitle().trim().isEmpty()) ? super.getTitle()
        : (getActividadType() != null ? "Asistencia al " + getActividadType() : null);
  }

  /**
   * Devuelve el tipo de actividad.
   * 
   * @return el tipo de actividad
   */
  @Transient
  public String getActividadType() {
    return actividad instanceof Curso ? "CURSO" : (actividad instanceof Taller ? "TALLER"
        : (actividad instanceof Charla ? "CHARLA" : null));
  }
}
