package es.csa.fdt.cyldigital.calendario.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;

import es.csa.fdt.cyldigital.usuarios.entity.Empleado;

/**
 * Clase de Ausencias de calendario.
 * 
 * @author csa
 */
@Entity
@Table(name = "ausenciacalendaritems")
@AttributeOverride(name = "id", column = @Column(name = "idCalendarItem"))
@Audited
@PrimaryKeyJoinColumn(name = "idCalendarItem")
public class AusenciaCalendarItem extends CalendarItem implements Serializable {

  private static final long serialVersionUID = -1202724109856721337L;

  /** empleado. */
  private Empleado empleado;

  /**
   * Instancia una nueva Ausencia en el calendario.
   */
  public AusenciaCalendarItem() {
    super();
    setAllDay(false);
    setTipo(CalendarItemType.AUSENCIA_EMPLEADO);
  }

  /**
   * Devuelve empleado.
   * 
   * @return empleado
   */
  @ManyToOne
  @JoinColumn(name = "idEmpleado")
  public Empleado getEmpleado() {
    return empleado;
  }

  /**
   * Asigna empleado.
   * 
   * @param empleado
   *          empleado
   */
  public void setEmpleado(Empleado empleado) {
    this.empleado = empleado;
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.calendario.entity.CalendarItem#getDescription()
   */
  @Override
  @Transient
  public String getDescription() {
    return (super.getDescription() != null && !super.getDescription().trim().isEmpty()) ? super.getDescription() : null;
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.calendario.entity.CalendarItem#getTitle()
   */
  @Override
  @Transient
  public String getTitle() {
    return (super.getTitle() != null && !super.getTitle().trim().isEmpty()) ? super.getTitle() : "Ausencia de "
        + empleado.getNombreCompleto();
  }

}
