package es.csa.fdt.cyldigital.calendario.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.Date;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.Lob;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;

/**
 * Clase de Eventos de Calendario
 * 
 * @author csa
 *
 */
@Entity
@Table(name = "calendaritems")
@Audited
@Inheritance(strategy = InheritanceType.JOINED)
@AttributeOverride(name = "id", column = @Column(name = "idCalendarItem"))
public class CalendarItem extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = -1202724109856721337L;

  /** CABECERA_CSS. */
  private static final String CABECERA_CSS = "fc-";

  /** Comienzo. */
  private Date start;

  /** Final. */
  private Date end;

  /** Nombre del estilo. */
  private String className;

  /** Es todo el día. */
  private boolean allDay;

  /** Título por defecto. */
  private String defaultTitle;

  /** Descripción por defecto. */
  private String defaultDescription;

  /**
   * Posibles valores del tipo de item de calendario. FESTIVO, CENTRO_HORARIO, ACTIVIDAD_HORARIO, ORDENADOR_USO
   * EMPLEADO_VACACIONES, EMPLEADO_FICHAJE, EMPLEADO_AUSENCIA,
   */
  public static enum CalendarItemType {
    
    /** FESTIVO. */
    FESTIVO,
    
    /** DOMINGO. */
    DOMINGO,
    
    /** HORARIO centro. */
    HORARIO_CENTRO,
    
    /** HORARIO curso. */
    HORARIO_CURSO,
    
    /** HORARIO taller. */
    HORARIO_TALLER,
    
    /** HORARIO charla. */
    HORARIO_CHARLA,
    
    /** HORARIO asesoramiento. */
    HORARIO_ASESORAMIENTO,
    
    /** HORARIO externa. */
    HORARIO_EXTERNA,
    
    /** ASISTENCIA. */
    ASISTENCIA,
    
    /** VACACIONES empleado. */
    VACACIONES_EMPLEADO,
    
    /** AUSENCIA empleado. */
    AUSENCIA_EMPLEADO,
    
    /** ACTIVIDAD usuario. */
    ACTIVIDAD_USUARIO,
    
    /** EVENT. */
    EVENT,
    
    /** WIFI. */
    WIFI,
    
    /** RESERVA. */
    RESERVA,
    
    /** EVENTO imagen. */
    EVENTO_IMAGEN,
    
    /** EVENTO video. */
    EVENTO_VIDEO,
    
    /** EVENTO video especial. */
    EVENTO_VIDEO_ESPECIAL,
    
    /** EVENTO anuncio. */
    EVENTO_ANUNCIO,
    
    /** EVENTO oferta. */
    EVENTO_OFERTA,
    
    /** EVENTO agenda. */
    EVENTO_AGENDA;
  }

  /** Tipo. */
  private CalendarItemType tipo;

  /**
   * Construye un nuevo objeto de la clase
   */
  public CalendarItem() {
    super();
  }

  /**
   * Devuelve el título.
   * 
   * @return título
   */
  @Transient
  public String getTitle() {
    return defaultTitle != null ? defaultTitle : null;
  }

  /**
   * Devuelve comienzo.
   * 
   * @return comienzo
   */
  @Temporal(TemporalType.TIMESTAMP)
  public Date getStart() {
    return start;
  }

  /**
   * Asigna comienzo.
   * 
   * @param start
   *          comienzo
   */
  public void setStart(Date start) {
    this.start = start;
  }

  /**
   * Devuelve final.
   * 
   * @return final
   */
  @Temporal(TemporalType.TIMESTAMP)
  public Date getEnd() {
    return end;
  }

  /**
   * Asigna final.
   * 
   * @param end
   *          final
   */
  public void setEnd(Date end) {
    this.end = end;
  }

  /**
   * Comprueba si es todo el día.
   * 
   * @return es todo el día
   */
  public boolean isAllDay() {
    return allDay;
  }

  /**
   * Asigna es todo el día.
   * 
   * @param allDay
   *          es todo el día
   */
  public void setAllDay(boolean allDay) {
    this.allDay = allDay;
  }

  /**
   * Devuelve tipo.
   * 
   * @return tipo
   */
  @Enumerated(value = EnumType.STRING)
  public CalendarItemType getTipo() {
    return tipo;
  }

  /**
   * Asigna tipo.
   * 
   * @param tipo
   *          tipo
   */
  public void setTipo(CalendarItemType tipo) {
    this.tipo = tipo;
  }

  /**
   * Parámetro 'className' utilizado por JSON para mapear el evento del calendario
   * 
   * @return nombre de la clase CSS a pintar
   */
  @Transient
  public String getClassName() {
    return (className != null && !className.isEmpty()) ? className : CABECERA_CSS
        + ((tipo != null ? tipo.toString() : getDefault()).toLowerCase().replace('_', '-'));
  }

  /**
   * Asigna nombre del estilo.
   * 
   * @param className
   *          nombre del estilo
   */
  public void setClassName(String className) {
    this.className = className;
  }

  /**
   * Parámetro 'default' utilizado por JSON para mapear el evento del calendario.
   * 
   * @return nombre de la clase CSS a pintar
   */
  @Transient
  public String getDefault() {
    return CalendarItemType.EVENT.toString();
  }

  /**
   * Devuelve la descripción.
   * 
   * @return Descripción
   */
  @Transient
  public String getDescription() {
    return defaultDescription != null ? defaultDescription : null;
  }

  /**
   * Devuelve título por defecto.
   * 
   * @return título por defecto
   */
  public String getDefaultTitle() {
    return defaultTitle;
  }

  /**
   * Asigna título por defecto.
   * 
   * @param defaultTitle
   *          título por defecto
   */
  public void setDefaultTitle(String defaultTitle) {
    this.defaultTitle = defaultTitle;
  }

  /**
   * Devuelve descripción por defecto.
   * 
   * @return descripción por defecto
   */
  @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getDefaultDescription() {
    return defaultDescription;
  }

  /**
   * Asigna descripción por defecto.
   * 
   * @param defaultDescription
   *          descripción por defecto
   */
  public void setDefaultDescription(String defaultDescription) {
    this.defaultDescription = defaultDescription;
  }

  /**
   * Devuelve los minutos duracion.
   * 
   * @return minutos duracion
   */
  @Transient
  public long getMinutosDuracion() {
    long minutos = 0;
    if (getStart() != null && getEnd() != null) {
      minutos = (getEnd().getTime() - getStart().getTime()) / 1000 / 60;
    }
    return minutos;
  }
}
