package es.csa.fdt.cyldigital.calendario.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.recursos.entity.EventoPanel;

/**
 * Clase Eventos de Calendario de Paneles.
 * 
 * @author csa
 */
@Entity
@Table(name = "eventospanelescalendaritems")
@AttributeOverride(name = "id", column = @Column(name = "idCalendarItem"))
@Audited
@PrimaryKeyJoinColumn(name = "idCalendarItem")
public class EventoPanelCalendarItem extends CalendarItem implements Serializable {

  private static final long serialVersionUID = -1202724109856721337L;

  /** evento. */
  private EventoPanel evento;

  /**
   * Construye un nuevo objeto de la clase
   */
  public EventoPanelCalendarItem() {
    super();
    setAllDay(false);
    setTipo(CalendarItemType.EVENTO_AGENDA);
  }

  /**
   * Devuelve evento.
   * 
   * @return evento
   */
  @ManyToOne(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
  @JoinColumn(name = "idEventoPanel", nullable = false)
  @NotNull
  public EventoPanel getEvento() {
    return evento;
  }

  /**
   * Asigna evento.
   * 
   * @param evento
   *          evento
   */
  public void setEvento(EventoPanel evento) {
    this.evento = evento;
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.calendario.entity.CalendarItem#getDescription()
   */
  @Override
  @Transient
  public String getDescription() {
    return evento.getNombre()
        + (evento.getDescripcion() != null || evento.getDescripcion().isEmpty() ? ". " + evento.getDescripcion() : "");
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.calendario.entity.CalendarItem#getTitle()
   */
  @Override
  @Transient
  public String getTitle() {
    return evento.getNombre();
  }

}
