package es.csa.fdt.cyldigital.calendario.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.Date;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;

import org.hibernate.envers.Audited;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.usuarios.entity.Localidad;
import es.csa.fdt.cyldigital.usuarios.entity.Provincia;

/**
 * Clase de Festivos de Calendario.
 * 
 * @author csa
 */
@Entity
@Table(name = "festivocalendaritems")
@AttributeOverride(name = "id", column = @Column(name = "idCalendarItem"))
@Audited
@PrimaryKeyJoinColumn(name = "idCalendarItem")
public class FestivoCalendarItem extends CalendarItem implements Serializable {

  private static final long serialVersionUID = -1202724109856721337L;

  /**
   * Enumeración de los tipos de festivos.
   * 
   * @author csa
   */
  public static enum TipoFestivo {

    /** NACIONAL. */
    NACIONAL,

    /** LOCAL. */
    LOCAL,
  }

  /** tipo festivo. */
  private TipoFestivo tipoFestivo;

  /** localidad. */
  private Localidad localidad;

  /** provincia. */
  private Provincia provincia;

  /**
   * Construye un nuevo objeto de la clase
   */
  public FestivoCalendarItem() {
    super();
    setStart(new Date());
    setAllDay(true);
    tipoFestivo = TipoFestivo.NACIONAL;
    setTipo(CalendarItemType.FESTIVO);
  }

  /**
   * Devuelve tipo festivo.
   * 
   * @return tipo festivo
   */
  @NotNull
  public TipoFestivo getTipoFestivo() {
    return tipoFestivo;
  }

  /**
   * Asigna tipo festivo.
   * 
   * @param tipoFestivo
   *          tipo festivo
   */
  public void setTipoFestivo(TipoFestivo tipoFestivo) {
    this.tipoFestivo = tipoFestivo;
  }

  /**
   * Devuelve localidad.
   * 
   * @return localidad
   */
  @ManyToOne
  @JoinColumn(name = "idLocalidad")
  public Localidad getLocalidad() {
    return localidad;
  }

  /**
   * Asigna localidad.
   * 
   * @param localidad
   *          localidad
   */
  public void setLocalidad(Localidad localidad) {
    this.localidad = localidad;
  }

  /**
   * Devuelve provincia.
   * 
   * @return provincia
   */
  @ManyToOne()
  @JoinColumn(name = "idProvincia")
  public Provincia getProvincia() {
    return provincia;
  }

  /**
   * Asigna provincia.
   * 
   * @param provincia
   *          provincia
   */
  public void setProvincia(Provincia provincia) {
    this.provincia = provincia;
  }
}
