package es.csa.fdt.cyldigital.calendario.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.recursos.entity.Recurso;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;

/**
 * Clase de Reservas de Recursos.
 * 
 * @author csa
 */
@Entity
@Table(name = "reservas")
@AttributeOverride(name = "id", column = @Column(name = "idCalendarItem"))
@Audited
@PrimaryKeyJoinColumn(name = "idCalendarItem")
public class ReservaCalendarItem extends CalendarItem implements Serializable {

  private static final long serialVersionUID = -1202724109856721337L;

  /** recurso. */
  private Recurso recurso;

  /** empleado. */
  private Empleado empleado;

  /** observaciones. */
  private String observaciones;

  /**
   * Construye un nuevo objeto de la clase
   */
  public ReservaCalendarItem() {
    super();
    setTipo(CalendarItemType.RESERVA);
  }

  /**
   * Devuelve recurso.
   * 
   * @return recurso
   */
  @ManyToOne(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
  @JoinColumn(name = "idRecurso", nullable = false)
  @NotNull
  public Recurso getRecurso() {
    return recurso;
  }

  /**
   * Asigna recurso.
   * 
   * @param recurso
   *          recurso
   */
  public void setRecurso(Recurso recurso) {
    this.recurso = recurso;
  }

  /**
   * Devuelve empleado.
   * 
   * @return empleado
   */
  @ManyToOne(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
  @JoinColumn(name = "idEmpleado", nullable = false)
  @NotNull
  public Empleado getEmpleado() {
    return empleado;
  }

  /**
   * Asigna empleado.
   * 
   * @param empleado
   *          empleado
   */
  public void setEmpleado(Empleado empleado) {
    this.empleado = empleado;
  }

  /**
   * Devuelve observaciones.
   * 
   * @return observaciones
   */
  @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getObservaciones() {
    return observaciones;
  }

  /**
   * Asigna observaciones.
   * 
   * @param observaciones
   *          observaciones
   */
  public void setObservaciones(String observaciones) {
    this.observaciones = observaciones;
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.calendario.entity.CalendarItem#getDescription()
   */
  @Override
  @Transient
  public String getDescription() {
    return (super.getDescription() != null && !super.getDescription().trim().isEmpty()) ? super.getDescription()
        : "Reserva del Recurso '" + getRecurso().getNombre() + "' por '" + getEmpleado().getNombreCompleto() + "'";
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.calendario.entity.CalendarItem#getTitle()
   */
  @Override
  @Transient
  public String getTitle() {
    return (super.getTitle() != null && !super.getTitle().trim().isEmpty()) ? super.getTitle() : "Reserva de Recurso";
  }

  /**
   * Comprueba si est finalizada.
   * 
   * @return true, si finalizada
   */
  @Transient
  public boolean isFinalizada() {
    return getId() > 0 && getEnd() != null && getEnd().after(new Date());
  }

  /**
   * Comprueba si está editada la fecha.
   * 
   * @return true, si editada fecha
   */
  @Transient
  public boolean isEditadaFecha() {
    return getId() > 0 && getStart() != null && getEnd() != null;
  }

  /**
   * Devuelve fechas legibles.
   * 
   * @return fechas legibles
   */
  @Transient
  public String getFechasLegibles() {
    String resultado = "";
    DateFormat df = new SimpleDateFormat("dd/MM/yyyy HH:mm");
    if (getStart() != null) {
      resultado += df.format(getStart());
    }
    if (getStart() != null && getEnd() != null) resultado += " - ";
    if (getEnd() != null) {
      resultado += df.format(getEnd());
    }
    return resultado;
  }

}
