package es.csa.fdt.cyldigital.calendario.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.centros.entity.Aula;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.recursos.entity.Ordenador;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;

/**
 * Clase de Eventos de la Actividad del Usuario en el Centro.
 * 
 * @author csa
 */
@Entity
@Table(name = "useractivitycalendaritems")
@AttributeOverride(name = "id", column = @Column(name = "idCalendarItem"))
@Audited
@PrimaryKeyJoinColumn(name = "idCalendarItem")
public class UserActivityCalendarItem extends CalendarItem implements Serializable {

  private static final long serialVersionUID = -3719378343433727805L;

  /** usuario. */
  private Usuario usuario;

  /** centro. */
  private Centro centro;

  /** aula. */
  private Aula aula;

  /** ordenador. */
  private Ordenador ordenador;

  /**
   * Construye un nuevo objeto de la clase
   */
  public UserActivityCalendarItem() {
    super();
    setTipo(CalendarItemType.ACTIVIDAD_USUARIO);
  }

  /**
   * Devuelve usuario.
   * 
   * @return usuario
   */
  @ManyToOne
  @JoinColumn(name = "idUsuario")
  @NotNull
  public Usuario getUsuario() {
    return usuario;
  }

  /**
   * Asigna usuario.
   * 
   * @param usuario
   *          usuario
   */
  public void setUsuario(Usuario usuario) {
    this.usuario = usuario;
  }

  /**
   * Devuelve centro.
   * 
   * @return centro
   */
  @ManyToOne
  @JoinColumn(name = "idCentro")
  @NotNull
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Devuelve aula.
   * 
   * @return aula
   */
  @ManyToOne
  @JoinColumn(name = "idAula")
  public Aula getAula() {
    return aula;
  }

  /**
   * Asigna aula.
   * 
   * @param aula
   *          aula
   */
  public void setAula(Aula aula) {
    this.aula = aula;
  }

  /**
   * Devuelve ordenador.
   * 
   * @return ordenador
   */
  @ManyToOne
  @JoinColumn(name = "idOrdenador")
  @NotNull
  public Ordenador getOrdenador() {
    return ordenador;
  }

  /**
   * Asigna ordenador.
   * 
   * @param ordenador
   *          ordenador
   */
  public void setOrdenador(Ordenador ordenador) {
    this.ordenador = ordenador;
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.calendario.entity.CalendarItem#getDescription()
   */
  @Override
  @Transient
  public String getDescription() {
    return (super.getDescription() != null && !super.getDescription().trim().isEmpty()) ? super.getDescription()
        : "Centro: "
            + (getCentro() == null ? "" : getCentro().getNombre() + " - Área: "
                + (getAula() == null ? "" : getAula().getNombre())) + " - Ordenador: "
            + (getOrdenador() == null ? "" : getOrdenador().getNombre());
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.calendario.entity.CalendarItem#getTitle()
   */
  @Override
  @Transient
  public String getTitle() {
    // TODO Auto-generated method stub
    return (super.getTitle() != null && !super.getTitle().trim().isEmpty()) ? super.getTitle() : "Actividad de "
        + (getUsuario() == null ? "" : getUsuario().getNombre() + " " + getUsuario().getApellido1());
  }

}
