package es.csa.fdt.cyldigital.calendario.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.usuarios.entity.Empleado;

/**
 * Clase de Vacaciones del Empleado.
 * 
 * @author csa
 */
@Entity
@Table(name = "vacacionescalendaritems")
@AttributeOverride(name = "id", column = @Column(name = "idCalendarItem"))
@Audited
@PrimaryKeyJoinColumn(name = "idCalendarItem")
public class VacacionesCalendarItem extends CalendarItem implements Serializable {

  private static final long serialVersionUID = -1202724109856721337L;

  /** empleado. */
  private Empleado empleado;

  /**
   * Enumeración de los estados de vacaciones.
   * 
   * @author csa
   */
  public static enum Estado {

    /** PENDIENTES. */
    PENDIENTES,

    /** APROBADAS. */
    APROBADAS,

    /** RECHAZADAS. */
    RECHAZADAS;
  }

  /** estado. */
  private Estado estado;

  /** dias. */
  private int dias;

  /**
   * Construye un nuevo objeto de la clase
   */
  public VacacionesCalendarItem() {
    super();
    setAllDay(true);
    setTipo(CalendarItemType.VACACIONES_EMPLEADO);
    setEstado(Estado.PENDIENTES);
  }

  /**
   * Devuelve empleado.
   * 
   * @return empleado
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idEmpleado")
  public Empleado getEmpleado() {
    return empleado;
  }

  /**
   * Asigna empleado.
   * 
   * @param empleado
   *          empleado
   */
  public void setEmpleado(Empleado empleado) {
    this.empleado = empleado;
  }

  /**
   * Devuelve estado.
   * 
   * @return estado
   */
  @Enumerated(value = EnumType.STRING)
  public Estado getEstado() {
    return estado;
  }

  /**
   * Asigna estado.
   * 
   * @param estado
   *          estado
   */
  public void setEstado(Estado estado) {
    this.estado = estado;
  }

  /**
   * Devuelve dias.
   * 
   * @return dias
   */
  @NotNull
  public int getDias() {
    return dias != 0 ? dias : getNumeroDias();
  }

  /**
   * Asigna dias.
   * 
   * @param dias
   *          dias
   */
  public void setDias(int dias) {
    this.dias = dias;
  }

  /**
   * Devuelve el numero de dias.
   * 
   * @return numero de dias
   */
  @Transient
  public int getNumeroDias() {
    int dias = 0;
    long dif;
    String stStart = "01/01/1970";
    String stEnd = "01/01/1970";
    DateFormat df = new SimpleDateFormat("dd/MM/yyyy");
    if (getStart() != null) {
      stStart = df.format(getStart());
    }
    if (getEnd() != null) {
      stEnd = df.format(getEnd());
    }
    try {
      dif = df.parse(stEnd).getTime() - df.parse(stStart).getTime();
    } catch (ParseException e) {
      return 0;
    }
    dias = ((int) (dif / (1000 * 3600 * 24))) + 1;
    return dias;
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.calendario.entity.CalendarItem#getDescription()
   */
  @Override
  @Transient
  public String getDescription() {
    return (super.getDescription() != null && !super.getDescription().trim().isEmpty()) ? super.getDescription() : null;
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.calendario.entity.CalendarItem#getTitle()
   */
  @Override
  @Transient
  public String getTitle() {
    return (super.getTitle() != null && !super.getTitle().trim().isEmpty()) ? super.getTitle() : "Vacaciones de "
        + empleado.getNombreCompleto();
  }

}
