package es.csa.fdt.cyldigital.calendario.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.recursos.entity.Router;

/**
 * Clase del histórico de claves de los routers.
 * 
 * @author csa
 */
@Entity
@Table(name = "wificalendaritems")
@AttributeOverride(name = "id", column = @Column(name = "idCalendarItem"))
@Audited
@PrimaryKeyJoinColumn(name = "idCalendarItem")
public class WifiCalendarItem extends CalendarItem implements Serializable {

  private static final long serialVersionUID = -3719378343433727805L;

  /** router. */
  private Router router;

  /** clave. */
  private String clave;

  /**
   * Construye un nuevo objeto de la clase
   */
  public WifiCalendarItem() {
    super();
    setTipo(CalendarItemType.WIFI);
  }

  /**
   * Devuelve router.
   * 
   * @return router
   */
  @ManyToOne
  @JoinColumn(name = "idRouter")
  @NotNull
  public Router getRouter() {
    return router;
  }

  /**
   * Asigna router.
   * 
   * @param router
   *          router
   */
  public void setRouter(Router router) {
    this.router = router;
  }

  /**
   * Devuelve clave.
   * 
   * @return clave
   */
  public String getClave() {
    return clave;
  }

  /**
   * Asigna clave.
   * 
   * @param clave
   *          clave
   */
  public void setClave(String clave) {
    this.clave = clave;
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.calendario.entity.CalendarItem#getDescription()
   */
  @Override
  @Transient
  public String getDescription() {
    return (super.getDescription() != null && !super.getDescription().trim().isEmpty()) ? super.getDescription()
        : "Router: "
            + (getRouter() == null ? "" : getRouter().getNombre() + " - SSID: " + getRouter().getSsid() == null ? ""
                : getRouter().getSsid() + " - Clave: " + getRouter().getClave() == null ? "" : getRouter().getClave());
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.calendario.entity.CalendarItem#getTitle()
   */
  @Override
  @Transient
  public String getTitle() {
    return (super.getTitle() != null && !super.getTitle().trim().isEmpty()) ? super.getTitle() : "Router "
        + (getRouter() == null ? "" : getRouter().getNombre());
  }

}
