package es.csa.fdt.cyldigital.calendario.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.richfaces.json.JSONArray;
import org.richfaces.json.JSONException;
import org.richfaces.json.JSONObject;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.calendario.entity.ActividadCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.ActividadCalendarItem.ActividadType;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.servicios.entity.Actividad;
import es.csa.fdt.cyldigital.servicios.entity.ActividadExterna;
import es.csa.fdt.cyldigital.servicios.entity.Asesoramiento;
import es.csa.fdt.cyldigital.servicios.entity.Charla;
import es.csa.fdt.cyldigital.servicios.entity.Curso;
import es.csa.fdt.cyldigital.servicios.entity.Taller;
import es.csa.fdt.cyldigital.util.ActivitySchedule;

/**
 * Servicio de eventos de calendario de actividades.
 * 
 * @author csa
 *
 */
@Service
@Repository
public class ActividadCalendarItemServiceImpl extends AbstractServiceImpl<ActividadCalendarItem, Long> implements
    ActividadCalendarItemService {

  private static final long serialVersionUID = -5575228180361917674L;

  private long idActividad;

  private JSONArray jsonA;

  private JSONArray jsonATemp;

  private String temp;

  private Date firstEvent;

  private Date lastEvent;

  private String idToRemove;

  private boolean toPersist;

  private boolean errorToPersist;

  /**
   * Devuelve el id de la actividad.
   * @return el id de la actividad
   */
  public long getIdActividad() {
    return idActividad;
  }

  /**
   * Asigna el id de la actividad.
   * @param idActividad el id de la actividad
   */
  public void setIdActividad(long idActividad) {
    this.idActividad = idActividad;
    setFirstEvent();
    setLastEvent();
  }

  /**
   * Devuelve el primer evento.
   * @return el primer evento
   */
  public Date getFirstEvent() {
    return firstEvent;
  }

  private void setFirstEvent() {
    this.firstEvent = idActividad >= 0 ? find(Actividad.class, idActividad).getFechaInicio() : null;
  }

  /**
   * Devuelve el último evento.
   * @return el último evento
   */
  public Date getLastEvent() {
    return lastEvent;
  }

  /**
   * Asigna el último evento.
   */
  public void setLastEvent() {
    this.lastEvent = idActividad >= 0 ? find(Actividad.class, idActividad).getFechaFin() : null;
  }

  /**
   * Devuelve el id a eliminar.
   * @return el id a eliminar
   */
  public String getIdToRemove() {
    return idToRemove;
  }

  /**
   * Asigna el id a eliminar.
   * @param idToRemove el id a eliminar
   */
  public void setIdToRemove(String idToRemove) {
    this.idToRemove = idToRemove;
  }

  /**
   * Devuelve el array de JSON.
   * @return el array de JSON
   */
  public JSONArray getJsonA() {
    return jsonA;
  }

  /**
   * Asigna el array de JSON.
   * @param jsonA el array de JSON
   */
  public void setJsonA(JSONArray jsonA) {
    this.jsonA = jsonA;
  }

  /**
   * Devuelve  el array de JSON temporal.
   * @return el array de JSON temporal
   */
  public JSONArray getJsonATemp() {
    return jsonATemp;
  }

  /**
   * Asigna  el array de JSON temporal.
   * @param jsonATemp el array de JSONtemporal
   */
  public void setJsonATemp(JSONArray jsonATemp) {
    this.jsonATemp = jsonATemp;
  }

  /**
   * Devuelve un evento temporal.
   * @return un evento temporal
   */
  public String getTemp() {
    return temp;
  }

  /**
   * Asigna un evento temporal.
   * @param temp un evento temporal
   */
  public void setTemp(String temp) {
    try {
      this.jsonATemp = new JSONArray(temp);
    } catch (Exception e) {}
  }

  /**
   * Comprueba si se debe persistir.
   * @return true, si se debe persistir
   */
  public boolean isToPersist() {
    return toPersist;
  }

  /**
   * Marca si se debe persistir.
   * @param toPersist si se debe persistir.
   */
  public void setToPersist(boolean toPersist) {
    this.toPersist = toPersist;
  }

  /**
   * Comprueba si hay error al persistir.
   * @return si hay error al persistir
   */
  public boolean isErrorToPersist() {
    return errorToPersist;
  }

  /**
   * Marca si hay error al persistir.
   * @param errorToPersist si hay error al persistir
   */
  public void setErrorToPersist(boolean errorToPersist) {
    this.errorToPersist = errorToPersist;
  }

  /**
   * Elimina un evento.
   */
  public void removeEvent() {
    if (idToRemove != null) {
      Long id = checkIdPattern(idToRemove);
      if (id != null) {
        ActividadCalendarItem aci = find(id);
        Actividad a = find(Actividad.class, idActividad);
        setMessagesEnabled(false);
        a.getCalendarItems().remove(aci);
        remove(aci);
        setMessagesEnabled(true);
      }
    }
  }

  /**
   * Acción de pre-guardado de eventos.
   */
  public void preSave() {
    if (jsonA == null) {
      jsonA = jsonATemp;
    } else {
      try {
        for (int i = 0; i < jsonATemp.length(); i++) {
          if (!compareAndChange((JSONObject) jsonATemp.get(i))) {
            // Añado el JSONObject temporal si no existía
            jsonA.put((JSONObject) jsonATemp.get(i));
          }
        }

        JSONArray jsonANew = new JSONArray();
        for (int x = 0; x < jsonA.length(); x++) {
        	String id = (String)((JSONObject)(jsonA.get(x))).get("id");
        	
        	for(int y = 0; y < jsonATemp.length(); y++)
        		if (id.equals((String)((JSONObject)jsonATemp.get(y)).get("id"))) {
        			jsonANew.put((JSONObject)(jsonA.get(x)));
        			break;
        		}
        }
        jsonA = jsonANew;

      } catch (JSONException e) {
        e.printStackTrace();
      }
    }
    if (toPersist) {
      persistCalendarItems();
    }
  }

  /**
   * Persiste los eventos de Calendario.
   */
  public void persistCalendarItems() {
    float numeroHoras = 0;
    if (jsonA != null) {
      Actividad actividad = find(Actividad.class, idActividad);
      
      firstEvent = null;
      lastEvent = null;
      
      Calendar horaInicioActividad = Calendar.getInstance();
      Calendar horaFinActividad = Calendar.getInstance();
      
      horaInicioActividad.set(1970, 1, 1, 23, 59, 59);
      horaFinActividad.set(1970, 1, 1, 0, 0, 0);
      
      Calendar fechaInicioItem = Calendar.getInstance();
      Calendar fechaFinItem = Calendar.getInstance();
      
      try {
        JSONObject o;
        for (int i = 0; i < jsonA.length(); i++) {
        	o = (JSONObject) jsonA.get(i);
        	numeroHoras += addCalendarItem(o, actividad);
          
        	if ((actividad instanceof Curso) || (actividad instanceof ActividadExterna)) {
	        	fechaInicioItem.setTimeInMillis((Long)o.get("start"));
	        	fechaFinItem.setTimeInMillis((Long)o.get("end"));
	          
	        	if ((fechaInicioItem.get(Calendar.HOUR_OF_DAY) < horaInicioActividad.get(Calendar.HOUR_OF_DAY)) ||
	        		((fechaInicioItem.get(Calendar.HOUR_OF_DAY) == horaInicioActividad.get(Calendar.HOUR_OF_DAY)) &&
	        		 (fechaInicioItem.get(Calendar.MINUTE) < horaInicioActividad.get(Calendar.MINUTE)))) {
	        		horaInicioActividad.set(Calendar.HOUR_OF_DAY, fechaInicioItem.get(Calendar.HOUR_OF_DAY));
	        		horaInicioActividad.set(Calendar.MINUTE, fechaInicioItem.get(Calendar.MINUTE));
	        		horaInicioActividad.set(Calendar.SECOND, fechaInicioItem.get(Calendar.SECOND));
	        	}
	          
	        	if ((fechaFinItem.get(Calendar.HOUR_OF_DAY) > horaFinActividad.get(Calendar.HOUR_OF_DAY)) ||
	        		((fechaFinItem.get(Calendar.HOUR_OF_DAY) == horaFinActividad.get(Calendar.HOUR_OF_DAY)) &&
	        		 (fechaFinItem.get(Calendar.MINUTE) > horaFinActividad.get(Calendar.MINUTE)))) {
	        		horaFinActividad.set(Calendar.HOUR_OF_DAY, fechaFinItem.get(Calendar.HOUR_OF_DAY));
	        		horaFinActividad.set(Calendar.MINUTE, fechaFinItem.get(Calendar.MINUTE));
	        		horaFinActividad.set(Calendar.SECOND, fechaFinItem.get(Calendar.SECOND));
	        	}
        	}
        }
      } catch (JSONException e) {
        e.printStackTrace();
      }

      if (actividad instanceof Curso) {
        persistCursoChanges(idActividad, numeroHoras, firstEvent, lastEvent, horaInicioActividad.getTime(), horaFinActividad.getTime());
      } else if (actividad instanceof Taller) {
        persistTallerChanges(idActividad, numeroHoras, firstEvent, lastEvent);
      } else if (actividad instanceof Charla) {
        persistCharlaChanges(idActividad, firstEvent, lastEvent);
      } else if (actividad instanceof ActividadExterna) {
    	persistExternaChanges(idActividad, /*numeroHoras,*/ firstEvent, lastEvent, horaInicioActividad.getTime(), horaFinActividad.getTime());
      }
      setMessagesEnabled(false);
      persist(actividad);
      setMessagesEnabled(true);
    }
  }

  private float addCalendarItem(JSONObject o, Actividad actividad) throws JSONException {
    ActividadCalendarItem aci;
    Long id = checkIdPattern((String) o.get("id"));
    
    if (id != null) {
      aci = find(id);
      aci.setStart(new Date((Long) o.get("start")));
      aci.setEnd(new Date((Long) o.get("end")));
    }
    else {
      aci = new ActividadCalendarItem();
      aci.setActividad(actividad);
      aci.setActividadType(actividad instanceof Curso ? ActividadType.CURSO :
    	  				  (actividad instanceof Taller ? ActividadType.TALLER :
    	  				  (actividad instanceof Charla ? ActividadType.CHARLA :
    	  				  (actividad instanceof Asesoramiento ? ActividadType.ASESORAMIENTO :
    	  				  (actividad instanceof ActividadExterna ? ActividadType.EXTERNA : null)))));
      aci.setStart(new Date((Long) o.get("start")));
      aci.setEnd(new Date((Long) o.get("end")));
    }
    
    setMessagesEnabled(false);
    persist(aci);
    setMessagesEnabled(true);

    if (firstEvent == null || aci.getStart().before(firstEvent)) {
      firstEvent = aci.getStart();
    }

    if (lastEvent == null || aci.getEnd().after(lastEvent)) {
      lastEvent = aci.getEnd();
    }

    return calculateDuracionEvent(aci);
  }

  /**
   * Compara si un JSONObject existe entre los existentes.
   * 
   * @param o
   *          JSONObject temporal.
   * @return Devuelve true si lo encuentra, sino false.
   * @throws JSONException
   */
  private boolean compareAndChange(JSONObject o) throws JSONException {
    String id = (String) o.get("id");
    for (int i = 0; i < jsonA.length(); i++) {
      if (id.equals((String) ((JSONObject) jsonA.get(i)).get("id"))) {
        substitute(((JSONObject) jsonA.get(i)), o);
        return true;
      }
    }
    return false;
  }

  /**
   * Sustituye las fechas de inicio y fin del JSONObject por las del objeto temporal.
   * 
   * @param o
   *          Objeto final
   * @param oTemp
   *          Objeto temporal
   * @throws JSONException
   */
  private void substitute(JSONObject o, JSONObject oTemp) throws JSONException {
    o.remove("start");
    o.remove("end");
    o.put("start", oTemp.get("start"));
    o.put("end", oTemp.get("end"));
  }

  /**
   * Comprueba que el 'id' del JSONObject sea el de un CalendarItem ya creado o de uno nuevo.
   * 
   * @param id
   *          'id' del JSONObject.
   * @return Devuelve el 'id' del CalendarItem en base de datos, si es nuevo devuelve 'null'.
   */
  private Long checkIdPattern(String id) {
    return id.toUpperCase().matches("ACT\\d+_ID\\d+") ? Long.valueOf(id.toUpperCase().split("ACT\\d+_ID")[1]) : null;
  }

  private float calculateDuracionEvent(ActividadCalendarItem aci) {
    return (float) ((aci.getEnd().getTime() - aci.getStart().getTime()) / (1000.0 * 60 * 60));
  }
  
  /**
   * Comprueba si existe alguna colisión entre las planificaciones de la lista schedules
   * 
   * @param schedules
   * 			Lista de planificaciones
   * 
   * @return <i>true</i> si existe alguna colisión. <i>false</i> si no existe ninguna colisión
   */
  public boolean hasSchedulesCollision(List<ActivitySchedule> schedules) {
	  boolean collision = false;
	  
	  for (int i = 0; i < schedules.size(); i++) {
		  ActivitySchedule as = schedules.get(i);
		  
		  for (int j = (i + 1); j < schedules.size(); j++) {
			  ActivitySchedule nas = schedules.get(j);
			  
			  if (as.collisionsWith(nas)) {
				  collision = true;
				  
				  break;
			  }
		  }
		  
		  if (collision)
			  break;
	  }
	  
	  return collision;
  }
  
  /**
   * Una vez calculados los items de calendario comprueba si son iguales
   * la fecha de inicio y fin efectivas.
   * 
   * @param schedules
   * 			Lista de planificaciones
   * 
   * @return <i>true</i> si las fechas de inicio y fin efectivas son iguales.
   * 		 <i>false</i> si las fechas de inicio y fin efectivas son diferente.
   */
  public long schedulesDurationInDays(List<ActivitySchedule> schedules) {
	  Long globalStartDateTimestamp = null;
	  Long globalEndDateTimestamp = null;

	  for (ActivitySchedule as : schedules) {
		  //If the schedule occurs at least one calendar item...
		  if (as.getActividadCalendarItems().size() > 0) {
			  if (((globalStartDateTimestamp != null) && (as.getEfectiveStartDateTimestamp() < globalStartDateTimestamp)) ||
				  (globalStartDateTimestamp == null))
				  globalStartDateTimestamp = as.getEfectiveStartDateTimestamp();
			  
			  if (((globalEndDateTimestamp != null) && (as.getEfectiveEndDateTimestamp() > globalEndDateTimestamp)) ||
				  (globalEndDateTimestamp == null))
				  globalEndDateTimestamp = as.getEfectiveEndDateTimestamp();
		  }
	  }

	  if ((globalStartDateTimestamp == null) || (globalEndDateTimestamp == null))
		  return 0L;
	  else
		  return (Math.round((globalEndDateTimestamp - globalStartDateTimestamp) / Long.valueOf(86400000L)) + 1L); //86400000 is one day in milliseconds
  }
  
  /**
   * Genera el calendario de la actividad en base a la lista de planificaciones schedules.
   * 
   * @param idActividad
   * 			Id de la actividad
   * 
   * @param schedules
   * 			Planificaciones de la actividad
   * 
   * @return <i>true</i> si el calendario se ha generado correctametne. <i>false</i> si el calendario no ha podido generarse.
   */
  public boolean generateCalendar(Long idActividad, List<ActivitySchedule> schedules) {
	  if (!hasSchedulesCollision(schedules)) {
		  Date firstItem = null;
		  Date lastItem = null;

		  Calendar itemTime = Calendar.getInstance();
		  Calendar minTime = Calendar.getInstance();
		  Calendar maxTime = Calendar.getInstance();
		  
		  minTime.set(1970, 1, 1, 23, 59, 59);
		  maxTime.set(1970, 1, 1, 0, 0, 0);
		  
		  float numeroHoras = 0;
		  
		  Actividad actividad = find(Actividad.class, idActividad);
		  		  
		  for (ActivitySchedule as : schedules) {
			  List<ActividadCalendarItem> laci = as.getActividadCalendarItems();
			  
			  for (ActividadCalendarItem aci : laci) {
				  ActividadCalendarItem newAci = new ActividadCalendarItem();
				  
				  //aci.setActividadType(actividad instanceof Curso ? ActividadType.CURSO :
				  //		(actividad instanceof ActividadExterna ? ActividadType.EXTERNA : null));
				  newAci.setActividadType(actividad instanceof Curso ? ActividadType.CURSO :
					  		(actividad instanceof ActividadExterna ? ActividadType.EXTERNA : null));
				  //aci.setActividad(actividad);
				  newAci.setActividad(actividad);
				  
				  newAci.setAllDay(aci.isAllDay());
				  newAci.setEnd(aci.getEnd());
				  newAci.setStart(aci.getStart());
				  
				  setMessagesEnabled(false);
				  //persist(aci);
				  persist(newAci);
				  setMessagesEnabled(true);
				
				  //Calculates first item
				  if (firstItem == null || newAci.getStart().before(firstItem))
					  firstItem = newAci.getStart();
				
				  //Calculates last item
				  if (lastItem == null || newAci.getEnd().after(lastItem))
					  lastItem = newAci.getEnd();
				  
				  //Calculates min hour
				  itemTime.setTime(newAci.getStart());
				  
				  if ((itemTime.get(Calendar.HOUR_OF_DAY) < minTime.get(Calendar.HOUR_OF_DAY)) ||
					  ((itemTime.get(Calendar.HOUR_OF_DAY) == minTime.get(Calendar.HOUR_OF_DAY)) &&
					   (itemTime.get(Calendar.MINUTE) < minTime.get(Calendar.MINUTE)))) {
					  minTime.set(Calendar.HOUR_OF_DAY, itemTime.get(Calendar.HOUR_OF_DAY));
					  minTime.set(Calendar.MINUTE, itemTime.get(Calendar.MINUTE));
					  minTime.set(Calendar.SECOND, itemTime.get(Calendar.SECOND));
				  }
				  
				  //Calculates max hour
				  itemTime.setTime(newAci.getEnd());
			          
				  if ((itemTime.get(Calendar.HOUR_OF_DAY) > maxTime.get(Calendar.HOUR_OF_DAY)) ||
					  ((itemTime.get(Calendar.HOUR_OF_DAY) == maxTime.get(Calendar.HOUR_OF_DAY)) &&
					   (itemTime.get(Calendar.MINUTE) > maxTime.get(Calendar.MINUTE)))) {
					  maxTime.set(Calendar.HOUR_OF_DAY, itemTime.get(Calendar.HOUR_OF_DAY));
					  maxTime.set(Calendar.MINUTE, itemTime.get(Calendar.MINUTE));
					  maxTime.set(Calendar.SECOND, itemTime.get(Calendar.SECOND));
				  }

				  //Calculate number of hour of activity
				  numeroHoras += calculateDuracionEvent(newAci);
			  }
		  }
		  
		  if (actividad instanceof Curso)
			  persistCursoChanges(idActividad, numeroHoras, firstItem, lastItem, minTime.getTime(), maxTime.getTime());
		  else if (actividad instanceof ActividadExterna)
			  persistExternaChanges(idActividad, /*numeroHoras,*/ firstItem, lastItem, minTime.getTime(), maxTime.getTime());
		  
		  return true;
	  }
	  else
		  return false;
  }

  /**
   * Genera el calendario de la actividad rellenando items de hora de inicio a hora de fin
   * para todos los días del intervalo desde la fechas de inicio hasta la fecha de fin.
   * 
   * @param idActividad
   * 			Id de la actividad
   */
  public void generateCalendar(Long idActividad) {
	  float numeroHoras = 0;
	  
	  firstEvent = null;
	  lastEvent = null;
	  
	  ActividadCalendarItem item;
	  
	  Calendar c = Calendar.getInstance();
	  Calendar tempStart = Calendar.getInstance();
	  Calendar tempEnd = Calendar.getInstance();
	  Calendar tempTime = Calendar.getInstance();

	  Actividad actividad = find(Actividad.class, idActividad);
	  
	  if (actividad != null) {
		  tempTime.setTime(actividad.getHoraInicio());
		  tempStart.setTime(actividad.getFechaInicio());
		  tempStart.set(Calendar.HOUR, tempTime.get(Calendar.HOUR_OF_DAY));
		  tempStart.set(Calendar.MINUTE, tempTime.get(Calendar.MINUTE));
		  tempStart.set(Calendar.SECOND, tempTime.get(Calendar.SECOND));

		  tempTime.setTime(actividad.getHoraFin());
		  tempEnd.setTime(actividad.getFechaFin());
		  tempEnd.set(Calendar.HOUR, tempTime.get(Calendar.HOUR_OF_DAY));
		  tempEnd.set(Calendar.MINUTE, tempTime.get(Calendar.MINUTE));
		  tempEnd.set(Calendar.SECOND, tempTime.get(Calendar.SECOND));

		  c.setTime(tempStart.getTime());

		  while (c.before(tempEnd) ||
				 ((c.get(Calendar.DAY_OF_YEAR) <= tempEnd.get(Calendar.DAY_OF_YEAR)) &&
				  (c.get(Calendar.YEAR) == tempEnd.get(Calendar.YEAR)))) {

			  item = new ActividadCalendarItem();

			  c.set(Calendar.HOUR_OF_DAY, tempStart.get(Calendar.HOUR_OF_DAY));
			  c.set(Calendar.MINUTE, tempStart.get(Calendar.MINUTE));
			  c.set(Calendar.SECOND, tempStart.get(Calendar.SECOND));
			  
			  item.setStart(c.getTime());
			  
			  c.set(Calendar.HOUR_OF_DAY, tempEnd.get(Calendar.HOUR_OF_DAY));
			  c.set(Calendar.MINUTE, tempEnd.get(Calendar.MINUTE));
			  c.set(Calendar.SECOND, tempEnd.get(Calendar.SECOND));
			  
			  item.setEnd(c.getTime());
			  
			  item.setAllDay(false);
			  item.setActividadType(actividad instanceof Curso ? ActividadType.CURSO :
				  					(actividad instanceof Taller ? ActividadType.TALLER :
				  					 (actividad instanceof Charla ? ActividadType.CHARLA :
				  					  (actividad instanceof Asesoramiento ? ActividadType.ASESORAMIENTO :
				  					   (actividad instanceof ActividadExterna ? ActividadType.EXTERNA : null)))));
			  item.setActividad(actividad);

			  setMessagesEnabled(false);
			  persist(item);
			  setMessagesEnabled(true);

			  if (firstEvent == null || item.getStart().before(firstEvent))
				  firstEvent = item.getStart();

			  if (lastEvent == null || item.getEnd().after(lastEvent))
				  lastEvent = item.getEnd();

			  numeroHoras += calculateDuracionEvent(item);

			  c.set(Calendar.DATE, c.get(Calendar.DATE) + 1);
		  }

		  if (actividad instanceof Curso)
			  persistCursoChanges(idActividad, numeroHoras, firstEvent, lastEvent, null, null);
		  else if (actividad instanceof Taller)
			  persistTallerChanges(idActividad, numeroHoras, firstEvent, lastEvent);
		  else if (actividad instanceof Charla)
			  persistCharlaChanges(idActividad, firstEvent, lastEvent);
		  else if (actividad instanceof ActividadExterna)
			  persistExternaChanges(idActividad, firstEvent, lastEvent, null, null);
	  }
  }
  
  /**
   * Genera el calendario de las actividades.
   * 
   * @param actividades
   * 			actividades para generar su calendario
   * 
   * @param schedules
   * 			planificaciones a generar para cada actividad
   */
  public void generateCalendarMultiple(List<Actividad> actividades, List<ActivitySchedule> schedules) {
    for (Actividad a : actividades)
    	generateCalendar(a.getId(), schedules);
  }

  /**
   * Genera el calendario de las actividades.
   * 
   * @param actividades
   * 			actividades para generar su calendario
   */
  public void generateCalendarMultiple(List<Actividad> actividades) {
    for (Actividad a : actividades)
    	generateCalendar(a.getId());
  }

  private void persistCursoChanges(Long idCurso, float horas, Date inicio, Date fin, Date horaMinima, Date horaMaxima) {
    Curso c = find(Curso.class, idCurso);
    
    horas = Math.round(horas*100F) / 100F;
    c.setNumeroHoras(horas);
    
    c.setFechaInicio(inicio);
    c.setFechaFin(fin);
    
    if (horaMinima != null)
    	c.setHoraInicio(horaMinima);
    
    if (horaMaxima != null)
    	c.setHoraFin(horaMaxima);
    
    setMessagesEnabled(false);
    persist(c);
    setMessagesEnabled(true);
  }

  private void persistTallerChanges(Long idTaller, float horas, Date inicio, Date fin) {
    Taller t = find(Taller.class, idTaller);
    
    horas = Math.round(horas*100F) / 100F;
    t.setNumeroHoras(horas);
    
    t.setFechaInicio(inicio);
    t.setFechaFin(fin);
    
    setMessagesEnabled(false);
    persist(t);
    setMessagesEnabled(true);
  }

  private void persistCharlaChanges(Long idCharla, Date inicio, Date fin) {
    Charla c = find(Charla.class, idCharla);
    
    c.setFechaInicio(inicio);
    c.setFechaFin(fin);
    
    c.setHoraInicio(inicio);
    c.setHoraFin(fin);
    
    setMessagesEnabled(false);
    persist(c);
    setMessagesEnabled(true);
  }
  
  private void persistExternaChanges(Long idExterna, Date inicio, Date fin, Date horaMinima, Date horaMaxima) {
	  ActividadExterna e = find(ActividadExterna.class, idExterna);
	  
	  e.setFechaInicio(inicio);
	  e.setFechaFin(fin);
	  
	  if (horaMinima != null)
		  e.setHoraInicio(horaMinima);

	  if (horaMaxima != null)
		  e.setHoraFin(horaMaxima);
	  
	  setMessagesEnabled(false);
	  persist(e);
	  setMessagesEnabled(true);
  }
}
