package es.csa.fdt.cyldigital.calendario.service;

import java.text.ParseException;
import java.text.SimpleDateFormat;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.Calendar;
import java.util.Date;
import java.util.List;

import javax.persistence.NoResultException;
import javax.persistence.Query;

import org.richfaces.json.JSONArray;
import org.richfaces.json.JSONException;
import org.richfaces.json.JSONObject;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.calendario.entity.ActividadCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.AsistenciaCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.AsistenciaCalendarItem.Excepcion;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.servicios.entity.Actividad;
import es.csa.fdt.cyldigital.servicios.entity.Charla;
import es.csa.fdt.cyldigital.servicios.entity.Curso;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCharla;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;

/**
 * Servicio de eventos de calendario de asistencias a actividades.
 * 
 * @author csa
 *
 */
@Service
@Repository
public class AsistenciaCalendarItemServiceImpl extends AbstractServiceImpl<AsistenciaCalendarItem, Long> implements
    AsistenciaCalendarItemService {

  private static final long serialVersionUID = -5575228180361917674L;

  private long idActividad;

  private long idUsuario;

  private JSONArray jsonA;

  private JSONArray jsonATemp;

  private String temp;

  private Date firstEvent;

  private Date lastEvent;

  private String idToRemove;

  private boolean toPersist;

  private boolean errorToPersist;

  /**
   * Devuelve id actividad.
   * 
   * @return id actividad
   */
  public long getIdActividad() {
    return idActividad;
  }

  /**
   * Devuelve id usuario.
   * 
   * @return id usuario
   */
  public long getIdUsuario() {
    return idUsuario;
  }

  /**
   * Asigna datos.
   * 
   * @param idActividad
   *          id actividad
   * @param idUsuario
   *          id usuario
   */
  public void setDatos(long idActividad, long idUsuario) {
    this.idActividad = idActividad;
    this.idUsuario = idUsuario;
    setFirstEvent();
    setLastEvent();
  }

  /**
   * Devuelve first event.
   * 
   * @return first event
   */
  public Date getFirstEvent() {
    return firstEvent;
  }

  private void setFirstEvent() {
    this.firstEvent = idActividad >= 0 ? find(Actividad.class, idActividad).getFechaInicio() : null;
  }

  /**
   * Devuelve last event.
   * 
   * @return last event
   */
  public Date getLastEvent() {
    return lastEvent;
  }

  /**
   * Asigna last event.
   */
  public void setLastEvent() {
    this.lastEvent = idActividad >= 0 ? find(Actividad.class, idActividad).getFechaFin() : null;
  }

  /**
   * Devuelve id to remove.
   * 
   * @return id to remove
   */
  public String getIdToRemove() {
    return idToRemove;
  }

  /**
   * Asigna id to remove.
   * 
   * @param idToRemove
   *          id to remove
   */
  public void setIdToRemove(String idToRemove) {
    this.idToRemove = idToRemove;
  }

  /**
   * Devuelve json a.
   * 
   * @return json a
   */
  public JSONArray getJsonA() {
    return jsonA;
  }

  /**
   * Asigna json a.
   * 
   * @param jsonA
   *          json a
   */
  public void setJsonA(JSONArray jsonA) {
    this.jsonA = jsonA;
  }

  /**
   * Devuelve json a temp.
   * 
   * @return json a temp
   */
  public JSONArray getJsonATemp() {
    return jsonATemp;
  }

  /**
   * Asigna json a temp.
   * 
   * @param jsonATemp
   *          json a temp
   */
  public void setJsonATemp(JSONArray jsonATemp) {
    this.jsonATemp = jsonATemp;
  }

  /**
   * Devuelve temp.
   * 
   * @return temp
   */
  public String getTemp() {
    return temp;
  }

  /**
   * Asigna temp.
   * 
   * @param temp
   *          temp
   */
  public void setTemp(String temp) {
    try {
      this.jsonATemp = new JSONArray(temp);
    } catch (Exception e) {}
  }

  /**
   * Checks if is to persist.
   * 
   * @return true, if is to persist
   */
  public boolean isToPersist() {
    return toPersist;
  }

  /**
   * Asigna to persist.
   * 
   * @param toPersist
   *          to persist
   */
  public void setToPersist(boolean toPersist) {
    this.toPersist = toPersist;
  }

  /**
   * Checks if is error to persist.
   * 
   * @return true, if is error to persist
   */
  public boolean isErrorToPersist() {
    return errorToPersist;
  }

  /**
   * Asigna error to persist.
   * 
   * @param errorToPersist
   *          error to persist
   */
  public void setErrorToPersist(boolean errorToPersist) {
    this.errorToPersist = errorToPersist;
  }

  /**
   * Elimina event.
   */
  public void removeEvent() {
    if (idToRemove != null) {
      Long id = checkIdPattern(idToRemove);
      if (id != null) {
        AsistenciaCalendarItem aci = find(id);
        Actividad a = find(Actividad.class, idActividad);
        setMessagesEnabled(false);
        a.getCalendarItems().remove(aci);
        remove(aci);
        setMessagesEnabled(true);
      }
    }
  }

  /**
   * Pre save.
   */
  public void preSave() {
	  if (jsonA == null) {
		  jsonA = jsonATemp;
	  } else {
		  try {
			  for (int i = 0; i < jsonATemp.length(); i++) {
				  if (!compareAndChange((JSONObject) jsonATemp.get(i))) {
					  // Añado el JSONObject temporal si no existía
					  jsonA.put((JSONObject) jsonATemp.get(i));
				  }
			  }

			  JSONArray jsonANew = new JSONArray();
			  for (int x = 0; x < jsonA.length(); x++) {
				  String id = (String)((JSONObject)(jsonA.get(x))).get("id");

				  for(int y = 0; y < jsonATemp.length(); y++) {
					  if (id.equals((String)((JSONObject)jsonATemp.get(y)).get("id"))) {
						  jsonANew.put((JSONObject)(jsonA.get(x)));
						  break;
					  }
				  }
			  }

			  jsonA = jsonANew;
		  }
		  catch (JSONException e) {
			  e.printStackTrace();
		  }
	  }
	  
	  if (toPersist)
		  persistCalendarItems();
  }

  /**
   * Persist calendar items.
   */
  public void persistCalendarItems() {
    errorToPersist = false;
    if (jsonA != null) {
      Actividad a = find(Actividad.class, idActividad);
      Usuario usuario = find(Usuario.class, idUsuario);
      try {
        JSONObject o;
        for (int i = 0; i < jsonA.length(); i++) {
          o = (JSONObject) jsonA.get(i);
          if (!validateItem(o, a)) {
            setMessagesEnabled(true);
            addErrorMessage("Los eventos de asistencia del usuario son incorrectos. Compruebe que corresponden con los horarios de la actividad");
            setMessagesEnabled(false);
            errorToPersist = true;
            return;
          }
          addCalendarItem(o, a, usuario);
        }
      } catch (JSONException e) {
        e.printStackTrace();
      }
    }
  }

  private boolean validateItem(JSONObject o, Actividad a) {
    return sameDay(o, a) && correctSchedule(o, a);
  }

  private boolean sameDay(JSONObject o, Actividad a) {
    // TODO Auto-generated method stub
    return true;
  }

  private boolean correctSchedule(JSONObject o, Actividad a) {
    // TODO Auto-generated method stub
    return true;
  }

  private boolean addCalendarItem(JSONObject o, Actividad a, Usuario usuario) throws JSONException {
    AsistenciaCalendarItem aci;
    Long id = checkIdPattern((String) o.get("id"));
    if (id != null) {
      aci = find(id);
      aci.setStart(new Date((Long) o.get("start")));
      aci.setEnd(new Date((Long) o.get("end")));
      aci.setDefaultDescription((String) o.get("descripcion"));
      aci.setExcepcion(Excepcion.valueOf((String) o.get("excepcion")));
    } else {
      aci = new AsistenciaCalendarItem();
      aci.setActividad(a);
      aci.setUsuario(usuario);
      aci.setStart(new Date((Long) o.get("start")));
      aci.setEnd(new Date((Long) o.get("end")));
      aci.setDefaultDescription((String) o.get("descripcion"));
      aci.setExcepcion(Excepcion.valueOf((String) o.get("excepcion")));
    }
    setMessagesEnabled(false);
    persist(aci);
    setMessagesEnabled(true);

    return true;
  }

  /**
   * Compara si un JSONObject existe entre los existentes.
   * 
   * @param o
   *          JSONObject temporal.
   * @return Devuelve true si lo encuentra, sino false.
   * @throws JSONException
   */
  private boolean compareAndChange(JSONObject o) throws JSONException {
    String id = (String) o.get("id");
    for (int i = 0; i < jsonA.length(); i++) {
      if (id.equals((String) ((JSONObject) jsonA.get(i)).get("id"))) {
        substitute(((JSONObject) jsonA.get(i)), o);
        return true;
      }
    }
    return false;
  }

  /**
   * Sustituye las fechas de inicio y fin del JSONObject por las del objeto temporal.
   * 
   * @param o
   *          Objeto final
   * @param oTemp
   *          Objeto temporal
   * @throws JSONException
   */
  private void substitute(JSONObject o, JSONObject oTemp) throws JSONException {
    o.remove("start");
    o.remove("end");
    o.put("start", oTemp.get("start"));
    o.put("end", oTemp.get("end"));
  }

  /**
   * Comprueba que el 'id' del JSONObject sea el de un CalendarItem ya creado o de uno nuevo.
   * 
   * @param id
   *          'id' del JSONObject.
   * @return Devuelve el 'id' del CalendarItem en base de datos, si es nuevo devuelve 'null'.
   */
  private Long checkIdPattern(String id) {
    return id.toUpperCase().matches("ASIS\\d+_ID\\d+") ? Long.valueOf(id.toUpperCase().split("ASIS\\d+_ID")[1]) : null;
  }
  
  /**
   * Rellena la asistencia por defecto de todos los usuarios matriculados en un curso
   * 
   * @param idC
   *          'id' del Curso.
   */
  public void fillDefaultAttendanceCurso(Long idCurso) {
	  Curso curso = find(Curso.class, idCurso);
	  
	  List<MatriculaCurso> lMatriculas = curso.getMatriculas();
	  List<ActividadCalendarItem> lActividadCI = curso.getCalendarItems();
	  List<AsistenciaCalendarItem> lAsistenciasCI = curso.getAsistencias();
	  
	  for (MatriculaCurso matricula : lMatriculas) {
		  if (matricula.isAdmitido() && !matricula.isEnListaEspera()) {
			  for (ActividadCalendarItem actividadCI : lActividadCI) {
				  Boolean existe = false;
				  
				  for (AsistenciaCalendarItem asistenciaCI : lAsistenciasCI) {
					  Calendar actividadStart = Calendar.getInstance();
					  Calendar asistenciaStart = Calendar.getInstance();

					  actividadStart.setTime(actividadCI.getStart());
					  asistenciaStart.setTime(asistenciaCI.getStart());

					  if ((matricula.getUsuario().getId() == asistenciaCI.getUsuario().getId()) &&
						  (actividadStart.get(Calendar.DAY_OF_MONTH) == asistenciaStart.get(Calendar.DAY_OF_MONTH)) &&
						  (actividadStart.get(Calendar.MONTH) == asistenciaStart.get(Calendar.MONTH)) &&
						  (actividadStart.get(Calendar.YEAR) == asistenciaStart.get(Calendar.YEAR))) {
						  existe = true;

						  break;
					  }
				  }

				  if (!existe) {
					  AsistenciaCalendarItem asistenciaCI = new AsistenciaCalendarItem();
					  
					  Usuario usuario = find(Usuario.class, matricula.getUsuario().getId());

					  asistenciaCI.setActividad(curso);
					  asistenciaCI.setUsuario(usuario);
					  asistenciaCI.setExcepcion(AsistenciaCalendarItem.Excepcion.SIN);
					  asistenciaCI.setAutomatica(false);
					  asistenciaCI.setDetalles(null);

					  asistenciaCI.setAllDay(actividadCI.isAllDay());
					  asistenciaCI.setDefaultDescription("Asistencia al CURSO");
					  asistenciaCI.setDefaultTitle(null);
					  asistenciaCI.setStart(actividadCI.getStart());
					  asistenciaCI.setEnd(actividadCI.getEnd());

					  setMessagesEnabled(false);
					  persist(asistenciaCI);
				      setMessagesEnabled(true);
				  }
			  }
		  }
	  }
  }

  /**
   * Rellena la asistencia por defecto de todos los usuarios matriculados en una charla
   * 
   * @param idC
   *          'id' del Curso.
   */
  public void fillDefaultAttendanceCharla(Long idCharla) {
	  
	  Charla charla = find(Charla.class, idCharla);
	  
	  List<MatriculaCharla> lMatriculas = charla.getMatriculas();
	  List<ActividadCalendarItem> lActividadCI = charla.getCalendarItems();
	  List<AsistenciaCalendarItem> lAsistenciasCI = charla.getAsistencias();

	  for (MatriculaCharla matricula : lMatriculas) {
		  if (matricula.isAdmitido() && !matricula.isEnListaEspera()) {
			  for (ActividadCalendarItem actividadCI : lActividadCI) {
				  Boolean existe = false;
				  
				  for (AsistenciaCalendarItem asistenciaCI : lAsistenciasCI) {
					  Calendar actividadStart = Calendar.getInstance();
					  Calendar asistenciaStart = Calendar.getInstance();

					  actividadStart.setTime(actividadCI.getStart());
					  asistenciaStart.setTime(asistenciaCI.getStart());

					  if ((matricula.getUsuario().getId() == asistenciaCI.getUsuario().getId()) &&
						  (actividadStart.get(Calendar.DAY_OF_MONTH) == asistenciaStart.get(Calendar.DAY_OF_MONTH)) &&
						  (actividadStart.get(Calendar.MONTH) == asistenciaStart.get(Calendar.MONTH)) &&
						  (actividadStart.get(Calendar.YEAR) == asistenciaStart.get(Calendar.YEAR))) {
						  existe = true;

						  break;
					  }
				  }

				  if (!existe) {
					  AsistenciaCalendarItem asistenciaCI = new AsistenciaCalendarItem();
					  
					  Usuario usuario = find(Usuario.class, matricula.getUsuario().getId());

					  asistenciaCI.setActividad(charla);
					  asistenciaCI.setUsuario(usuario);
					  asistenciaCI.setExcepcion(AsistenciaCalendarItem.Excepcion.SIN);
					  asistenciaCI.setAutomatica(false);
					  asistenciaCI.setDetalles(null);

					  asistenciaCI.setAllDay(actividadCI.isAllDay());
					  asistenciaCI.setDefaultDescription("Asistencia al CURSO");
					  asistenciaCI.setDefaultTitle(null);
					  asistenciaCI.setStart(actividadCI.getStart());
					  asistenciaCI.setEnd(actividadCI.getEnd());

					  setMessagesEnabled(false);
					  persist(asistenciaCI);
				      setMessagesEnabled(true);
				  }
			  }
		  }
	  }
  }
  
  //3.4.12 - RF_15_35 Cumprimentación automática de asistentes
  /**
   * Rellena la asistencia por defecto de todos los usuarios matriculados en un curso de el día actual
   * 
   * @param idC
   *          'id' del Curso.
 * @throws ParseException 
   */
  @SuppressWarnings("unchecked")
  public void fillDefaultAttendanceCursoToday(Long idCurso) throws ParseException {
	  Curso curso = find(Curso.class, idCurso);
	  
	  Query qry = null;
	  String strqry = "";
	  List<ActividadCalendarItem> lActividadCI = null;
	  SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
	  
	  try {
		  Calendar cal = Calendar.getInstance();
		  String strStart = cal.get(Calendar.YEAR) + "-" + (cal.get(Calendar.MONTH) + 1) + "-" + cal.get(Calendar.DAY_OF_MONTH) + " 00:00:00";
		  String strEnd = cal.get(Calendar.YEAR) + "-" + (cal.get(Calendar.MONTH) + 1) + "-" + cal.get(Calendar.DAY_OF_MONTH) + " 23:59:59";
		  
		  Date dateStart = sdf.parse(strStart);
		  Date dateEnd = sdf.parse(strEnd);
			 
		  strqry = "SELECT aci " + 
				   "FROM " + ActividadCalendarItem.class.getName() + " aci " +
				   "WHERE (aci.actividad.id = :idActividad) AND " + 
				   "(aci.start >= :startDate) AND (aci.end <= :endDate)";
		  
		  qry = getEntityManager().createQuery(strqry);
			
		  qry.setParameter("idActividad", idCurso);
		  qry.setParameter("startDate", dateStart);
		  qry.setParameter("endDate", dateEnd);
		  
		  lActividadCI = (List<ActividadCalendarItem>) qry.getResultList();
	  } catch (NoResultException nre) {
		  
	  }
	  
	  if(lActividadCI.size() > 0) {
		  List<MatriculaCurso> lMatriculas = curso.getMatriculas();
		  List<AsistenciaCalendarItem> lAsistenciasCI = curso.getAsistencias();
		  
		  for (MatriculaCurso matricula : lMatriculas) {
			  if (matricula.isAdmitido() && !matricula.isEnListaEspera()) {
				  for (ActividadCalendarItem actividadCI : lActividadCI) {
					  Boolean existe = false;
					  
					  for (int i=0; i < lAsistenciasCI.size() && !existe; i++) {
						  AsistenciaCalendarItem asistenciaCI = lAsistenciasCI.get(i);
						  
						  Calendar actividadStart = Calendar.getInstance();
						  Calendar asistenciaStart = Calendar.getInstance();

						  actividadStart.setTime(actividadCI.getStart());
						  asistenciaStart.setTime(asistenciaCI.getStart());

						  if ((matricula.getUsuario().getId() == asistenciaCI.getUsuario().getId()) &&
							  (actividadStart.get(Calendar.DAY_OF_MONTH) == asistenciaStart.get(Calendar.DAY_OF_MONTH)) &&
							  (actividadStart.get(Calendar.MONTH) == asistenciaStart.get(Calendar.MONTH)) &&
							  (actividadStart.get(Calendar.YEAR) == asistenciaStart.get(Calendar.YEAR))) {
							  
							  existe = true;
						  }
					  }
					  
					  if (!existe) {
						  AsistenciaCalendarItem asistenciaCI = new AsistenciaCalendarItem();
						  Usuario usuario = find(Usuario.class, matricula.getUsuario().getId());
						  
						  asistenciaCI.setActividad(curso);
						  asistenciaCI.setUsuario(usuario);
						  asistenciaCI.setExcepcion(AsistenciaCalendarItem.Excepcion.SIN);
						  asistenciaCI.setAutomatica(false);
						  asistenciaCI.setDetalles(null);

						  asistenciaCI.setAllDay(actividadCI.isAllDay());
						  asistenciaCI.setDefaultDescription("Asistencia al CURSO");
						  asistenciaCI.setDefaultTitle(null);
						  asistenciaCI.setStart(actividadCI.getStart());
						  asistenciaCI.setEnd(actividadCI.getEnd());

						  setMessagesEnabled(false);
						  persist(asistenciaCI);
					      setMessagesEnabled(true);
					  }
				  }
			  }
		  }
	  }
  }
  
//3.4.12 - RF_15_35 Cumprimentación automática de asistentes
  /**
   * Rellena la asistencia por defecto de todos los usuarios matriculados en un curso de el día actual
   * 
   * @param idC
   *          'id' del Curso.
 * @throws ParseException 
   */
  @SuppressWarnings("unchecked")
  public void fillDefaultAttendanceCharlaToday(Long idCharla) throws ParseException {
	  Charla charla = find(Charla.class, idCharla);
	  
	  Query qry = null;
	  String strqry = "";
	  List<ActividadCalendarItem> lActividadCI = null;
	  SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
	  
	  try {
		  Calendar cal = Calendar.getInstance();
		  String strStart = cal.get(Calendar.YEAR) + "-" + (cal.get(Calendar.MONTH) + 1) + "-" + cal.get(Calendar.DAY_OF_MONTH) + " 00:00:00";
		  String strEnd = cal.get(Calendar.YEAR) + "-" + (cal.get(Calendar.MONTH) + 1) + "-" + cal.get(Calendar.DAY_OF_MONTH) + " 23:59:59";
		  
		  Date dateStart = sdf.parse(strStart);
		  Date dateEnd = sdf.parse(strEnd);
			 
		  strqry = "SELECT aci " + 
				   "FROM " + ActividadCalendarItem.class.getName() + " aci " +
				   "WHERE (aci.actividad.id = :idActividad) AND " + 
				   "(aci.start >= :startDate) AND (aci.end <= :endDate)";
		  
		  qry = getEntityManager().createQuery(strqry);
			
		  qry.setParameter("idActividad", idCharla);
		  qry.setParameter("startDate", dateStart);
		  qry.setParameter("endDate", dateEnd);
		  
		  lActividadCI = (List<ActividadCalendarItem>) qry.getResultList();
	  } catch (NoResultException nre) {
		  
	  }
	  
	  if(lActividadCI.size() > 0) {
		  List<MatriculaCharla> lMatriculas = charla.getMatriculas();
		  List<AsistenciaCalendarItem> lAsistenciasCI = charla.getAsistencias();
		  
		  for (MatriculaCharla matricula : lMatriculas) {
			  if (matricula.isAdmitido() && !matricula.isEnListaEspera()) {
				  for (ActividadCalendarItem actividadCI : lActividadCI) {
					  Boolean existe = false;
					  
					  for (int i=0; i < lAsistenciasCI.size() && !existe; i++) {
						  AsistenciaCalendarItem asistenciaCI = lAsistenciasCI.get(i);
						  
						  Calendar actividadStart = Calendar.getInstance();
						  Calendar asistenciaStart = Calendar.getInstance();

						  actividadStart.setTime(actividadCI.getStart());
						  asistenciaStart.setTime(asistenciaCI.getStart());

						  if ((matricula.getUsuario().getId() == asistenciaCI.getUsuario().getId()) &&
							  (actividadStart.get(Calendar.DAY_OF_MONTH) == asistenciaStart.get(Calendar.DAY_OF_MONTH)) &&
							  (actividadStart.get(Calendar.MONTH) == asistenciaStart.get(Calendar.MONTH)) &&
							  (actividadStart.get(Calendar.YEAR) == asistenciaStart.get(Calendar.YEAR))) {
							  
							  existe = true;
						  }
					  }
					  
					  if (!existe) {
						  AsistenciaCalendarItem asistenciaCI = new AsistenciaCalendarItem();
						  Usuario usuario = find(Usuario.class, matricula.getUsuario().getId());
						  
						  asistenciaCI.setActividad(charla);
						  asistenciaCI.setUsuario(usuario);
						  asistenciaCI.setExcepcion(AsistenciaCalendarItem.Excepcion.SIN);
						  asistenciaCI.setAutomatica(false);
						  asistenciaCI.setDetalles(null);

						  asistenciaCI.setAllDay(actividadCI.isAllDay());
						  asistenciaCI.setDefaultDescription("Asistencia a la CHARLA");
						  asistenciaCI.setDefaultTitle(null);
						  asistenciaCI.setStart(actividadCI.getStart());
						  asistenciaCI.setEnd(actividadCI.getEnd());

						  setMessagesEnabled(false);
						  persist(asistenciaCI);
					      setMessagesEnabled(true);
					  }
				  }
			  }
		  }
	  }
  }
}
