package es.csa.fdt.cyldigital.calendario.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.richfaces.json.JSONArray;
import org.richfaces.json.JSONException;
import org.richfaces.json.JSONObject;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.calendario.entity.AusenciaCalendarItem;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;

/**
 * Servicio de ausencias de calendario.
 * 
 * @author csa
 *
 */
@Service
@Repository
public class AusenciaCalendarItemServiceImpl extends AbstractServiceImpl<AusenciaCalendarItem, Long> implements
    AusenciaCalendarItemService {

  private static final long serialVersionUID = -5575228180361917674L;

  private Empleado empleado;

  private List<AusenciaCalendarItem> ausencias;

  private Date fecha;

  private Date horaInicio;

  private Date horaFin;

  private long idEmpleado;

  private JSONArray jsonA;

  private JSONArray jsonATemp;

  private String temp;

  private String idToRemove;

  private boolean toPersist;

  /**
   * Construye un nuevo objeto de la clase
   */
  public AusenciaCalendarItemServiceImpl() {
    super();
    ausencias = new ArrayList<AusenciaCalendarItem>();
  }

  /**
   * Devuelve id empleado.
   * 
   * @return id empleado
   */
  public long getIdEmpleado() {
    return idEmpleado;
  }

  /**
   * Asigna id empleado.
   * 
   * @param idEmpleado
   *          id empleado
   */
  public void setIdEmpleado(long idEmpleado) {
    this.idEmpleado = idEmpleado;
  }

  /**
   * Devuelve id to remove.
   * 
   * @return id to remove
   */
  public String getIdToRemove() {
    return idToRemove;
  }

  /**
   * Asigna id to remove.
   * 
   * @param idToRemove
   *          id to remove
   */
  public void setIdToRemove(String idToRemove) {
    this.idToRemove = idToRemove;
  }

  /**
   * Devuelve json a.
   * 
   * @return json a
   */
  public JSONArray getJsonA() {
    return jsonA;
  }

  /**
   * Asigna json a.
   * 
   * @param jsonA
   *          json a
   */
  public void setJsonA(JSONArray jsonA) {
    this.jsonA = jsonA;
  }

  /**
   * Devuelve json a temp.
   * 
   * @return json a temp
   */
  public JSONArray getJsonATemp() {
    return jsonATemp;
  }

  /**
   * Asigna json a temp.
   * 
   * @param jsonATemp
   *          json a temp
   */
  public void setJsonATemp(JSONArray jsonATemp) {
    this.jsonATemp = jsonATemp;
  }

  /**
   * Devuelve temp.
   * 
   * @return temp
   */
  public String getTemp() {
    return temp;
  }

  /**
   * Asigna temp.
   * 
   * @param temp
   *          temp
   */
  public void setTemp(String temp) {
    try {
      this.jsonATemp = new JSONArray(temp);
    } catch (Exception e) {}
  }

  /**
   * Checks if is to persist.
   * 
   * @return true, if is to persist
   */
  public boolean isToPersist() {
    return toPersist;
  }

  /**
   * Asigna to persist.
   * 
   * @param toPersist
   *          to persist
   */
  public void setToPersist(boolean toPersist) {
    this.toPersist = toPersist;
  }

  /**
   * Devuelve empleado.
   * 
   * @return empleado
   */
  public Empleado getEmpleado() {
    return empleado;
  }

  /**
   * Asigna empleado.
   * 
   * @param empleado
   *          empleado
   */
  public void setEmpleado(Empleado empleado) {
    this.empleado = empleado;
  }

  /**
   * Devuelve ausencias.
   * 
   * @return ausencias
   */
  public List<AusenciaCalendarItem> getAusencias() {
    return ausencias;
  }

  /**
   * Asigna ausencias.
   * 
   * @param ausencias
   *          ausencias
   */
  public void setAusencias(List<AusenciaCalendarItem> ausencias) {
    this.ausencias = ausencias;
  }

  /**
   * Devuelve fecha.
   * 
   * @return fecha
   */
  public Date getFecha() {
    return fecha;
  }

  /**
   * Asigna fecha.
   * 
   * @param fecha
   *          fecha
   */
  public void setFecha(Date fecha) {
    this.fecha = fecha;
  }

  /**
   * Devuelve hora inicio.
   * 
   * @return hora inicio
   */
  public Date getHoraInicio() {
    return horaInicio;
  }

  /**
   * Asigna hora inicio.
   * 
   * @param horaInicio
   *          hora inicio
   */
  public void setHoraInicio(Date horaInicio) {
    this.horaInicio = horaInicio;
  }

  /**
   * Devuelve hora fin.
   * 
   * @return hora fin
   */
  public Date getHoraFin() {
    return horaFin;
  }

  /**
   * Asigna hora fin.
   * 
   * @param horaFin
   *          hora fin
   */
  public void setHoraFin(Date horaFin) {
    this.horaFin = horaFin;
  }

  /**
   * Initialize.
   * 
   * @param ausencia
   *          ausencia
   * @param idEmpleado
   *          id empleado
   */
  public void initialize(AusenciaCalendarItem ausencia, Long idEmpleado) {
    ausencia.setEmpleado(find(Empleado.class, idEmpleado));
    ausencia.setDefaultTitle("Vacaciones");

  }

  /**
   * Find ausencia.
   * 
   * @param id
   *          id
   * @return ausencia calendar item
   */
  public AusenciaCalendarItem findAusencia(Long id) {
    AusenciaCalendarItem a = find(id);
    fecha = a.getStart();
    horaInicio = a.getStart();
    horaFin = a.getEnd();
    return a;
  }

  /**
   * Persistir ausencia.
   * 
   * @param ausencia
   *          ausencia
   */
  public void persistirAusencia(AusenciaCalendarItem ausencia) {
    persist(ausencia);
  }

  /**
   * Elimina event.
   */
  public void removeEvent() {
    if (idToRemove != null) {
      Long id = checkIdPattern(idToRemove);
      if (id != null) {
        AusenciaCalendarItem aci = find(id);
        Empleado e = find(Empleado.class, idEmpleado);
        setMessagesEnabled(false);
        e.getAusencias().remove(aci);
        remove(aci);
        setMessagesEnabled(true);
      }
    }
  }

  /**
   * Pre save.
   */
  public void preSave() {
    if (jsonA == null) {
      jsonA = jsonATemp;
    } else {
      try {
        for (int i = 0; i < jsonATemp.length(); i++) {
          if (!compareAndChange((JSONObject) jsonATemp.get(i))) {
            // Añado el JSONObject temporal si no existía
            jsonA.put((JSONObject) jsonATemp.get(i));
          }
        }
      } catch (JSONException e) {
        e.printStackTrace();
      }
    }
    if (toPersist) {
      persistCalendarItems();
    }
  }

  /**
   * Persist calendar items.
   */
  public void persistCalendarItems() {
    if (jsonA != null) {
      Empleado empleado = find(Empleado.class, idEmpleado);
      try {
        JSONObject o;
        for (int i = 0; i < jsonA.length(); i++) {
          o = (JSONObject) jsonA.get(i);
          addCalendarItem(o, empleado);
        }
      } catch (JSONException e) {
        e.printStackTrace();
      }
      setMessagesEnabled(false);
      persist(empleado);
      setMessagesEnabled(true);
    }
  }

  private void addCalendarItem(JSONObject o, Empleado empleado) throws JSONException {
    AusenciaCalendarItem aci;
    Long id = checkIdPattern((String) o.get("id"));
    if (id != null) {
      aci = find(id);
      aci.setStart(new Date((Long) o.get("start")));
      aci.setEnd(new Date((Long) o.get("end")));
      aci.setDefaultDescription((String) o.get("descripcion"));
    } else {
      aci = new AusenciaCalendarItem();
      aci.setEmpleado(empleado);
      aci.setStart(new Date((Long) o.get("start")));
      aci.setEnd(new Date((Long) o.get("end")));
      aci.setDefaultDescription((String) o.get("descripcion"));
    }
    setMessagesEnabled(false);
    persist(aci);
    setMessagesEnabled(true);
  }

  /**
   * Compara si un JSONObject existe entre los existentes.
   * 
   * @param o
   *          JSONObject temporal.
   * @return Devuelve true si lo encuentra, sino false.
   * @throws JSONException
   */
  private boolean compareAndChange(JSONObject o) throws JSONException {
    String id = (String) o.get("id");
    for (int i = 0; i < jsonA.length(); i++) {
      if (id.equals((String) ((JSONObject) jsonA.get(i)).get("id"))) {
        substitute(((JSONObject) jsonA.get(i)), o);
        return true;
      }
    }
    return false;
  }

  /**
   * Sustituye las fechas de inicio y fin del JSONObject por las del objeto temporal.
   * 
   * @param o
   *          Objeto final
   * @param oTemp
   *          Objeto temporal
   * @throws JSONException
   */
  private void substitute(JSONObject o, JSONObject oTemp) throws JSONException {
    o.remove("start");
    o.remove("end");
    o.put("start", oTemp.get("start"));
    o.put("end", oTemp.get("end"));
  }

  /**
   * Comprueba que el 'id' del JSONObject sea el de un CalendarItem ya creado o de uno nuevo.
   * 
   * @param id
   *          'id' del JSONObject.
   * @return Devuelve el 'id' del CalendarItem en base de datos, si es nuevo devuelve 'null'.
   */
  private Long checkIdPattern(String id) {
    return id.toUpperCase().matches("AUS\\d+_ID\\d+") ? Long.valueOf(id.toUpperCase().split("AUS\\d+_ID")[1]) : null;
  }

}
