package es.csa.fdt.cyldigital.calendario.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.Calendar;
import java.util.List;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.calendario.entity.EventoPanelCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.CalendarItem.CalendarItemType;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.recursos.entity.EventoPanel;
import es.csa.fdt.cyldigital.recursos.entity.Panel;
import es.csa.fdt.cyldigital.recursos.entity.EventoPanel.Periodicidad;
import es.csa.fdt.cyldigital.recursos.entity.EventoPanel.TipoContenido;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;

/**
 * Servicio de eventos de calendario para los paneles informativos.
 * 
 * @author csa
 *
 */
@Service
@Repository
public class EventoPanelCalendarItemServiceImpl extends AbstractServiceImpl<EventoPanelCalendarItem, Long> implements
    EventoPanelCalendarItemService {

  private static final long serialVersionUID = -5575228180361917674L;

  private Panel panel;

  private List<Panel> paneles;

  /**
   * Construye un nuevo objeto de la clase
   */
  public EventoPanelCalendarItemServiceImpl() {
    super();
  }

  /**
   * Devuelve paneles.
   * 
   * @return paneles
   */
  public List<Panel> getPaneles() {
    if (paneles == null) paneles = findAllOrder(Panel.class, "centro.nombre", true);
    return paneles;
  }

  /**
   * Devuelve panel.
   * 
   * @return panel
   */
  public Panel getPanel() {
    return panel;
  }

  /**
   * Asigna panel.
   * 
   * @param panel
   *          panel
   */
  public void setPanel(Panel panel) {
    this.panel = panel;
  }

  /**
   * Asigna panel.
   * 
   * @param user
   *          panel
   */
  public void setPanel(UsuarioLdap user) {
    if (panel == null && user.getCentro() != null) {
      panel = findOneWhere(Panel.class, "centro.id", user.getCentro().getId());
    }
  }

  /**
   * Generate calendar.
   * 
   * @param idEvento
   *           id evento
   */
  public void generateCalendar(Long idEvento) {
    EventoPanelCalendarItem item;
    Calendar c = Calendar.getInstance();
    Calendar tempStart = Calendar.getInstance();
    Calendar tempEnd = Calendar.getInstance();
    Calendar tempTime = Calendar.getInstance();

    EventoPanel evento = find(EventoPanel.class, idEvento);
    if (evento != null) {

      tempTime.setTime(evento.getHoraInicio());
      tempStart.setTime(evento.getFechaInicio());
      tempStart.set(Calendar.HOUR, tempTime.get(Calendar.HOUR_OF_DAY));
      tempStart.set(Calendar.MINUTE, tempTime.get(Calendar.MINUTE));

      tempTime.setTime(evento.getHoraFin());
      tempEnd.setTime(evento.getFechaFin());
      tempEnd.set(Calendar.HOUR, tempTime.get(Calendar.HOUR_OF_DAY));
      tempEnd.set(Calendar.MINUTE, tempTime.get(Calendar.MINUTE));

      c.setTime(tempStart.getTime());

      while (c.before(tempEnd)
          || (c.get(Calendar.DAY_OF_YEAR) <= tempEnd.get(Calendar.DAY_OF_YEAR) && c.get(Calendar.YEAR) == tempEnd
              .get(Calendar.YEAR))) {
        boolean generar = false;

        if (evento.getPeriodicidad() == Periodicidad.DIARIA && c.get(Calendar.DAY_OF_WEEK) > 1) {
          generar = true;
        } else if (evento.getPeriodicidad() == Periodicidad.SEMANAL
            && c.get(Calendar.DAY_OF_WEEK) == tempStart.get(Calendar.DAY_OF_WEEK)) {
          generar = true;
        } else if (evento.getPeriodicidad() == Periodicidad.MENSUAL
            && c.get(Calendar.DAY_OF_MONTH) == tempStart.get(Calendar.DAY_OF_MONTH)
            && c.get(Calendar.MONTH) >= tempStart.get(Calendar.MONTH)) {
          generar = true;
        }

        // si para el día actual hay que generar un calendaritem
        if (generar) {
          item = new EventoPanelCalendarItem();

          c.set(Calendar.HOUR_OF_DAY, tempStart.get(Calendar.HOUR_OF_DAY));
          c.set(Calendar.MINUTE, tempStart.get(Calendar.MINUTE));
          item.setStart(c.getTime());
          c.set(Calendar.HOUR_OF_DAY, tempEnd.get(Calendar.HOUR_OF_DAY));
          c.set(Calendar.MINUTE, tempEnd.get(Calendar.MINUTE));
          item.setEnd(c.getTime());
          item.setAllDay(evento.isTodoElDia());
          item.setTipo(evento.getTipoContenido() == TipoContenido.AGENDA ? CalendarItemType.EVENTO_AGENDA : evento
              .getTipoContenido() == TipoContenido.ANUNCIO ? CalendarItemType.EVENTO_ANUNCIO : evento
              .getTipoContenido() == TipoContenido.IMAGEN ? CalendarItemType.EVENTO_IMAGEN
              : evento.getTipoContenido() == TipoContenido.OFERTA_FORMATIVA ? CalendarItemType.EVENTO_OFERTA : evento
                  .getTipoContenido() == TipoContenido.VIDEO ? CalendarItemType.EVENTO_VIDEO : evento
                  .getTipoContenido() == TipoContenido.VIDEO_ESPECIAL ? CalendarItemType.EVENTO_VIDEO_ESPECIAL
                  : CalendarItemType.EVENT);
          item.setEvento(evento);

          setMessagesEnabled(false);
          persist(item);
          setMessagesEnabled(true);
        }
        // aumento el contador de fecha actual
        c.set(Calendar.DATE, c.get(Calendar.DATE) + 1);
      }
    }
  }
}
