package es.csa.fdt.cyldigital.calendario.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.List;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.calendario.entity.FestivoCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.FestivoCalendarItem.TipoFestivo;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.usuarios.entity.Provincia;

/**
 * Servicio de Festivos de Calendario.
 * 
 * @author csa
 *
 */
@Service
@Repository
public class FestivoCalendarItemServiceImpl extends AbstractServiceImpl<FestivoCalendarItem, Long> implements
    FestivoCalendarItemService {

  private static final long serialVersionUID = -5575228180361917674L;

  private List<FestivoCalendarItem> festivos;

  private List<Provincia> provincias;

  /**
   * Construye un nuevo objeto de la clase
   */
  public FestivoCalendarItemServiceImpl() {
    super();
    festivos = new ArrayList<FestivoCalendarItem>();
  }

  /**
   * Devuelve provincias.
   * 
   * @return provincias
   */
  public List<Provincia> getProvincias() {
    if (provincias == null) provincias = findAllOrder(Provincia.class, "nombre", true);
    return provincias;
  }

  /**
   * Devuelve festivos.
   * 
   * @return festivos
   */
  public List<FestivoCalendarItem> getFestivos() {
    return festivos;
  }

  /**
   * Asigna festivos.
   * 
   * @param festivos
   *          festivos
   */
  public void setFestivos(List<FestivoCalendarItem> festivos) {
    this.festivos = festivos;
  }

  /**
   * Persist festivo.
   * 
   * @param festivo
   *           festivo
   */
  public void persistFestivo(FestivoCalendarItem festivo) {
    if (festivo.getTipoFestivo() == TipoFestivo.NACIONAL) {
      festivo.setLocalidad(null);
      festivo.setProvincia(null);
    }
    persist(festivo);
  }

  /**
   * Asigna provincia localidad.
   * 
   * @param idCentro
   *           id centro
   * @param festivo
   *           festivo
   * @return festivo calendar item
   */
  public FestivoCalendarItem setProvinciaLocalidad(Long idCentro, FestivoCalendarItem festivo) {
    if (festivo.getTipoFestivo() == null || festivo.getTipoFestivo() == TipoFestivo.NACIONAL) {
      Centro centro = find(Centro.class, idCentro);
      festivo.setProvincia(centro.getProvincia());
      festivo.setLocalidad(centro.getLocalidad());
    }
    return festivo;
  }
}
