package es.csa.fdt.cyldigital.calendario.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import javax.annotation.Resource;

import org.richfaces.json.JSONArray;
import org.richfaces.json.JSONException;
import org.richfaces.json.JSONObject;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.calendario.entity.ReservaCalendarItem;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.recursos.entity.Recurso;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;

/**
 * Servicio de reservas de calendario.
 * @author csa
 *
 */
@Service
@Repository
public class ReservaCalendarItemServiceImpl extends AbstractServiceImpl<ReservaCalendarItem, Long> implements
    ReservaCalendarItemService {

  private static final long serialVersionUID = -5575228180361917674L;

  private List<Centro> centros;

  private Centro centro;

  private boolean changeCentro = true;

  private List<Empleado> empleados;

  private long idRecurso;

  private long idEmpleado;

  private long idReserva;

  private JSONArray jsonA;

  private JSONArray jsonATemp;

  private String temp;

  private String firstEvent;

  private String lastEvent;

  private String idToRemove;

  private boolean toPersist;

  private boolean errorToPersist;

  @Resource
  private transient HashMap<String, Object> generalProperties;

  /**
   * Apply centro.
   * 
   * @param centro
   *           centro
   */
  public void applyCentro(Centro centro) {
    this.centro = centro;
    changeCentro = true;
  }

  /**
   * Construye un nuevo objeto de la clase
   */
  public ReservaCalendarItemServiceImpl() {
    super();
  }

  /**
   * Devuelve id recurso.
   * 
   * @return id recurso
   */
  public long getIdRecurso() {
    return idRecurso;
  }

  /**
   * Asigna id recurso.
   * 
   * @param idRecurso
   *          id recurso
   */
  public void setIdRecurso(long idRecurso) {
    this.idRecurso = idRecurso;
  }

  /**
   * Devuelve id empleado.
   * 
   * @return id empleado
   */
  public long getIdEmpleado() {
    return idEmpleado;
  }

  /**
   * Asigna id empleado.
   * 
   * @param idEmpleado
   *          id empleado
   */
  public void setIdEmpleado(long idEmpleado) {
    this.idEmpleado = idEmpleado;
  }

  /**
   * Devuelve id reserva.
   * 
   * @return id reserva
   */
  public long getIdReserva() {
    return idReserva;
  }

  /**
   * Asigna id reserva.
   * 
   * @param idReserva
   *          id reserva
   */
  public void setIdReserva(long idReserva) {
    ReservaCalendarItem r = find(idReserva);
    this.idReserva = idReserva;
    this.idEmpleado = r.getEmpleado().getId();
    this.idRecurso = r.getRecurso().getId();
    setFirstEvent();
    setLastEvent();
  }

  /**
   * Devuelve first event.
   * 
   * @return first event
   */
  public String getFirstEvent() {
    return firstEvent;
  }

  private void setFirstEvent() {
    DateFormat df = new SimpleDateFormat("yyyy-MM-dd");
    ReservaCalendarItem r = find(idReserva);
    this.firstEvent = (idReserva >= 0 && r.getStart() != null) ? df.format(find(idReserva).getStart()) : null;
  }

  /**
   * Devuelve last event.
   * 
   * @return last event
   */
  public String getLastEvent() {
    return lastEvent;
  }

  /**
   * Asigna last event.
   */
  public void setLastEvent() {
    DateFormat df = new SimpleDateFormat("yyyy-MM-dd");
    ReservaCalendarItem r = find(idReserva);
    this.lastEvent = (idReserva >= 0 && r.getStart() != null) ? df.format(find(idReserva).getEnd()) : null;
  }

  /**
   * Devuelve id to remove.
   * 
   * @return id to remove
   */
  public String getIdToRemove() {
    return idToRemove;
  }

  /**
   * Asigna id to remove.
   * 
   * @param idToRemove
   *          id to remove
   */
  public void setIdToRemove(String idToRemove) {
    this.idToRemove = idToRemove;
  }

  /**
   * Devuelve json a.
   * 
   * @return json a
   */
  public JSONArray getJsonA() {
    return jsonA;
  }

  /**
   * Asigna json a.
   * 
   * @param jsonA
   *          json a
   */
  public void setJsonA(JSONArray jsonA) {
    this.jsonA = jsonA;
  }

  /**
   * Devuelve json a temp.
   * 
   * @return json a temp
   */
  public JSONArray getJsonATemp() {
    return jsonATemp;
  }

  /**
   * Asigna json a temp.
   * 
   * @param jsonATemp
   *          json a temp
   */
  public void setJsonATemp(JSONArray jsonATemp) {
    this.jsonATemp = jsonATemp;
  }

  /**
   * Devuelve temp.
   * 
   * @return temp
   */
  public String getTemp() {
    return temp;
  }

  /**
   * Asigna temp.
   * 
   * @param temp
   *          temp
   */
  public void setTemp(String temp) {
    try {
      this.jsonATemp = new JSONArray(temp);
    } catch (Exception e) {}
  }

  /**
   * Checks if is to persist.
   * 
   * @return true, if is to persist
   */
  public boolean isToPersist() {
    return toPersist;
  }

  /**
   * Asigna to persist.
   * 
   * @param toPersist
   *          to persist
   */
  public void setToPersist(boolean toPersist) {
    this.toPersist = toPersist;
  }

  /**
   * Checks if is error to persist.
   * 
   * @return true, if is error to persist
   */
  public boolean isErrorToPersist() {
    return errorToPersist;
  }

  /**
   * Asigna error to persist.
   * 
   * @param errorToPersist
   *          error to persist
   */
  public void setErrorToPersist(boolean errorToPersist) {
    this.errorToPersist = errorToPersist;
  }

  /**
   * Devuelve centros.
   * 
   * @return centros
   */
  public List<Centro> getCentros() {
    if (centros == null) centros = findAllOrder(Centro.class, "nombre", true);
    return centros;
  }

  /**
   * Devuelve empleados.
   * 
   * @return empleados
   */
  public List<Empleado> getEmpleados() {
    if (centro != null && changeCentro) {
      empleados = findWhereOrder(Empleado.class, "centro.id", centro.getId(), "nombre", true);
      changeCentro = false;
    }
    return empleados;
  }

  /**
   * Actualiza centro.
   * 
   * @param reserva
   *           reserva
   * @param currentUser
   *           current user
   */
  public void actualizaCentro(ReservaCalendarItem reserva, UsuarioLdap currentUser) {
    centro = (reserva.getRecurso() != null && reserva.getRecurso().getCentro() != null) ? reserva.getRecurso()
        .getCentro() : null;

    if (currentUser.applyFilterCentro() && currentUser.getCentro() != null) {
      if (centros == null) {
        centros = new ArrayList<Centro>();
      }
      Centro c = find(Centro.class, currentUser.getCentro().getId());
      centro = c;
      changeCentro = true;
    } else if (generalProperties.containsKey("DEFAULT_CENTER_ID") && reserva.getRecurso() == null) {
      Centro c = find(Centro.class, Long.valueOf((String) generalProperties.get("DEFAULT_CENTER_ID")));
      if (c != null) {
        centro = c;
        changeCentro = true;
      }
    }
  }

  /**
   * Devuelve centro.
   * 
   * @return centro
   */
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Pre save.
   */
  public void preSave() {
    if (jsonA == null) {
      jsonA = jsonATemp;
    } else {
      try {
        for (int i = 0; i < jsonATemp.length(); i++) {
          if (!compareAndChange((JSONObject) jsonATemp.get(i))) {
            // Añado el JSONObject temporal si no existía
            jsonA.put((JSONObject) jsonATemp.get(i));
          }
        }
      } catch (JSONException e) {
        e.printStackTrace();
      }
    }
    if (toPersist) {
      persistCalendarItems();
    }
  }

  /**
   * Persist calendar items.
   */
  public void persistCalendarItems() {
    if (jsonA != null) {
      Empleado empleado = find(Empleado.class, idEmpleado);
      Recurso recurso = find(Recurso.class, idRecurso);
      firstEvent = null;
      lastEvent = null;
      try {
        JSONObject o;
        if (jsonA.length() == 0) {
          ReservaCalendarItem r = find(idReserva);
          r.setStart(null);
          r.setEnd(null);
          setMessagesEnabled(false);
          persist(r);
          setMessagesEnabled(true);
        }
        for (int i = 0; i < jsonA.length(); i++) {
          o = (JSONObject) jsonA.get(i);
          addCalendarItem(o, empleado, recurso);
        }
      } catch (JSONException e) {
        e.printStackTrace();
      }
    }
  }

  private void addCalendarItem(JSONObject o, Empleado empleado, Recurso recurso) throws JSONException {
    DateFormat df = new SimpleDateFormat("yyyy-MM-dd");
    ReservaCalendarItem rci;
    Long id = checkIdPattern((String) o.get("id"));
    if (id != null) {
      rci = find(id);
      rci.setStart(new Date((Long) o.get("start")));
      rci.setEnd(new Date((Long) o.get("end")));
    } else {
      rci = new ReservaCalendarItem();
      rci.setEmpleado(empleado);
      rci.setRecurso(recurso);
      rci.setStart(new Date((Long) o.get("start")));
      rci.setEnd(new Date((Long) o.get("end")));
    }
    setMessagesEnabled(false);
    persist(rci);
    setMessagesEnabled(true);

    try {
      if (firstEvent == null || rci.getStart().before(df.parse(firstEvent))) {
        firstEvent = df.format(rci.getStart());
      }

      if (lastEvent == null || rci.getEnd().after(df.parse(lastEvent))) {
        lastEvent = df.format(rci.getEnd());
      }
    } catch (ParseException e) {}
  }

  /**
   * Compara si un JSONObject existe entre los existentes.
   * 
   * @param o
   *          JSONObject temporal.
   * @return Devuelve true si lo encuentra, sino false.
   * @throws JSONException
   */
  private boolean compareAndChange(JSONObject o) throws JSONException {
    String id = (String) o.get("id");
    for (int i = 0; i < jsonA.length(); i++) {
      if (id.equals((String) ((JSONObject) jsonA.get(i)).get("id"))) {
        substitute(((JSONObject) jsonA.get(i)), o);
        return true;
      }
    }
    return false;
  }

  /**
   * Sustituye las fechas de inicio y fin del JSONObject por las del objeto temporal.
   * 
   * @param o
   *          Objeto final
   * @param oTemp
   *          Objeto temporal
   * @throws JSONException
   */
  private void substitute(JSONObject o, JSONObject oTemp) throws JSONException {
    o.remove("start");
    o.remove("end");
    o.put("start", oTemp.get("start"));
    o.put("end", oTemp.get("end"));
  }

  /**
   * Comprueba que el 'id' del JSONObject sea el de un CalendarItem ya creado o de uno nuevo.
   * 
   * @param id
   *          'id' del JSONObject.
   * @return Devuelve el 'id' del CalendarItem en base de datos, si es nuevo devuelve 'null'.
   */
  private Long checkIdPattern(String id) {
    if (id.toUpperCase().matches("RES\\d+")) {
      return Long.valueOf(id.toUpperCase().split("RES")[1]);
    } else if (id.toUpperCase().matches("RESNEW_\\d+_\\d+")) {
      return Long.valueOf(id.toUpperCase().split("RESNEW_\\d+_")[1]);
    } else {
      return null;
    }
  }

  /**
   * Elimina reserva from cancel.
   * 
   * @param reserva
   *           reserva
   */
  public void removeReservaFromCancel(ReservaCalendarItem reserva) {
    if (reserva.getId() != 0) {
      setMessagesEnabled(false);
      remove(reserva);
      setMessagesEnabled(true);
    }
  }

  /**
   * Persist reserva.
   * 
   * @param reserva
   *           reserva
   * @return true, if successful
   */
  public boolean persistReserva(ReservaCalendarItem reserva) {
    if (!reserva.getRecurso().isOcupado(reserva)) {
      persist(reserva);
      return true;
    } else {
      addErrorMessage("El recurso '" + reserva.getRecurso().getNombre()
          + "' está reservado o se encuentra ocupado en otra actividad");
      return false;
    }
  }

  /**
   * Persist reserva to calendario.
   * 
   * @param reserva
   *           reserva
   * @return true, if successful
   */
  public boolean persistReservaToCalendario(ReservaCalendarItem reserva) {
    setMessagesEnabled(false);
    persist(reserva);
    setMessagesEnabled(true);
    return true;
  }
}
