package es.csa.fdt.cyldigital.calendario.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.commons.persistence.browser.jpaql.FilterJPAQLByQuery;
import es.csa.fdt.cyldigital.calendario.entity.ReservaCalendarItem;
import es.csa.fdt.cyldigital.core.service.BrowserFilteredByActive;

/**
 * Servicio del browser de reservas.
 * 
 * @author csa
 *
 */
@Service
@Repository
public class ReservasCalendarItemBrowserImpl extends BrowserFilteredByActive<ReservaCalendarItem> implements
    ReservasCalendarItemBrowser<CriterionJPAQL, String> {

  private static final long serialVersionUID = -7227366052245717441L;

  /**
   * Construye un nuevo objeto de la clase
   */
  public ReservasCalendarItemBrowserImpl() {
    super();
    getSort().setSortProperty("start DESC");
  }

  /**
   * Filter by empleado.
   * 
   * @param idEmpleado
   *           id empleado
   */
  public void filterByEmpleado(Long idEmpleado) {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<ReservaCalendarItem> filter = new FilterJPAQLByQuery<ReservaCalendarItem>();

    query += " empleado.id = ? ";
    filter.getCriterion().addParameter(idEmpleado);

    for (Filter<CriterionJPAQL, ReservaCalendarItem> f : getFilters())
      if (f.getCriterion().getQuery().contains("empleado.id")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }

  /**
   * Filter by recurso.
   * 
   * @param idRecurso
   *           id recurso
   */
  public void filterByRecurso(Long idRecurso) {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<ReservaCalendarItem> filter = new FilterJPAQLByQuery<ReservaCalendarItem>();

    query += " recurso.id = ? ";
    filter.getCriterion().addParameter(idRecurso);

    for (Filter<CriterionJPAQL, ReservaCalendarItem> f : getFilters())
      if (f.getCriterion().getQuery().contains("recurso.id")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }

  /**
   * Refresh.
   */
  public void refresh() {
    getEntityManager().clear();
  }
  /*
   * @Override public void filterByCentro(Long idCentro) { setPropiedad("recurso.centro.id");
   * super.filterByCentro(idCentro); }
   */
}
