package es.csa.fdt.cyldigital.calendario.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.commons.persistence.browser.jpaql.FilterJPAQLByQuery;
import es.csa.fdt.cyldigital.calendario.entity.VacacionesCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.VacacionesCalendarItem.Estado;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.service.BrowserFilteredByActive;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario.Role;

/**
 * Servicio del browser de vacaciones de empleado.
 * @author csa
 *
 */
@Service
@Repository
public class VacacionesCalendarItemBrowserImpl extends BrowserFilteredByActive<VacacionesCalendarItem> implements
    VacacionesCalendarItemBrowser<CriterionJPAQL, String> {

  private static final long serialVersionUID = -7227366052245717441L;

  /**
   * Construye un nuevo objeto de la clase
   */
  public VacacionesCalendarItemBrowserImpl() {
    super();
    getSort().setSortAsc(true);
    getSort().setSortProperty("start");
  }

  /**
   * Filter by pendientes.
   */
  public void filterByPendientes() {
    String query = "";
    FilterJPAQLByQuery<VacacionesCalendarItem> filter = new FilterJPAQLByQuery<VacacionesCalendarItem>();

    query += " estado = ? ";
    filter.getCriterion().addParameter(Estado.PENDIENTES);
    filter.getCriterion().setQuery(query);
    getFilters().add(filter);

  }

  /**
   * Filter by empleado.
   * 
   * @param idEmpleado
   *           id empleado
   */
  public void filterByEmpleado(Long idEmpleado) {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<VacacionesCalendarItem> filter = new FilterJPAQLByQuery<VacacionesCalendarItem>();

    query += " empleado.id = ? ";
    filter.getCriterion().addParameter(idEmpleado);

    for (Filter<CriterionJPAQL, VacacionesCalendarItem> f : getFilters())
      if (f.getCriterion().getQuery().contains("empleado.id")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }

  /**
   * Refresh.
   */
  public void refresh() {
    getEntityManager().clear();
  } 
  
  /**
   * Asigna filter centro.
   * 
   * @param user
   *          filter centro
   */
  public void setFilterCentro(UsuarioLdap user) {
	   
    //if (user.applyFilterCentro()) {
    if (!user.getRolesList().contains(Role.ROLE_ADMIN) && (user.getCentro() != null)) { //Puesto así para Xunta de galicia    	
      removeFilterCentro();
      if (user.getCentro() != null) {
        filterByCentro(user.getCentro().getId());
      }
    }
  }

  private void removeFilterCentro() {
    int i = -1;
    for (Filter<CriterionJPAQL, VacacionesCalendarItem> f : getFilters())
      if (f.getCriterion().getQuery().contains(" id ")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
  }

  /**
   * Filter by centro.
   * 
   * @param idCentro
   *           id centro
   */
  public void filterByCentro(Long idCentro) {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<VacacionesCalendarItem> filter = new FilterJPAQLByQuery<VacacionesCalendarItem>();

    query += " empleado.centro.id = ? ";
    filter.getCriterion().addParameter(idCentro);

    for (Filter<CriterionJPAQL, VacacionesCalendarItem> f : getFilters())
      if (f.getCriterion().getQuery().contains(" empleado.centro.id ")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }
   

}
