package es.csa.fdt.cyldigital.calendario.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.List;

import javax.persistence.Query;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.calendario.entity.VacacionesCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.VacacionesCalendarItem.Estado;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.notificacion.Notification;
import es.csa.fdt.cyldigital.notificacion.NotificationService;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado.Puesto;
import es.csa.fdt.cyldigital.usuarios.service.UsuarioService;

/**
 * Servicio de vacaciones del empleado.
 * @author csa
 *
 */
@Service
@Repository
public class VacacionesCalendarItemServiceImpl extends AbstractServiceImpl<VacacionesCalendarItem, Long> implements
    VacacionesCalendarItemService {

  private static final long serialVersionUID = -5575228180361917674L;

  private Long oldId;

  /**
   * Construye un nuevo objeto de la clase
   */
  public VacacionesCalendarItemServiceImpl() {
    super();
  }

  /**
   * Devuelve old id.
   * 
   * @return old id
   */
  public Long getOldId() {
    return oldId;
  }

  /**
   * Asigna old id.
   * 
   * @param oldId
   *          old id
   */
  public void setOldId(Long oldId) {
    this.oldId = oldId;
  }

  /**
   * Find and save.
   * 
   * @param id
   *           id
   * @return vacaciones calendar item
   */
  public VacacionesCalendarItem findAndSave(Long id) {
    oldId = id;
    return find(id);
  }

  /**
   * Persistir vacaciones.
   * 
   * @param vacaciones
   *           vacaciones
   * @param idEmpleado
   *           id empleado
   */
  public void persistirVacaciones(VacacionesCalendarItem vacaciones, Long idEmpleado) {
    Empleado e = find(Empleado.class, idEmpleado);
    if (e != null) {
      // Si vuelvo a reeditar las vacaciones y cambio algún dato, las pongo pendientes de aceptación
      if (oldId != null && oldId > 0) {
        vacaciones.setEstado(Estado.PENDIENTES);
      }
      vacaciones.setDefaultTitle("Vacaciones");
      vacaciones.setEmpleado(e);
      persist(vacaciones);
    }
  }

  /**
   * Aprobar vacaciones.
   * 
   * @param vacaciones
   *           vacaciones
   */
  public void aprobarVacaciones(VacacionesCalendarItem vacaciones) {
    vacaciones.setEstado(Estado.APROBADAS);
    setMessagesEnabled(false);
    persist(vacaciones);
    setMessagesEnabled(true);
  }

  /**
   * Rechazar vacaciones.
   * 
   * @param vacaciones
   *           vacaciones
   */
  public void rechazarVacaciones(VacacionesCalendarItem vacaciones) {
    vacaciones.setEstado(Estado.RECHAZADAS);
    setMessagesEnabled(false);
    persist(vacaciones);
    setMessagesEnabled(true);
  }

  /**
   * Send notification.
   * 
   * @param idVacaciones
   *           id vacaciones
   */
  @SuppressWarnings("unchecked")
  public void sendNotification(Long idVacaciones) {
    VacacionesCalendarItem vacaciones = find(idVacaciones);
    String consulta = "SELECT e FROM " + Empleado.class.getName() + " e WHERE desactivado = false AND centro = :valor1 AND puesto = :valor2";
    Query query = getEntityManager().createQuery(consulta);
    query.setParameter("valor1", vacaciones.getEmpleado().getCentro());
    query.setParameter("valor2", Puesto.AREA_MANAGER);
    List<Usuario> areaManagers = query.getResultList();
    if (!areaManagers.isEmpty()) {
      Usuario u = find(Usuario.class, vacaciones.getEmpleado().getId());
      if (!containsUser(areaManagers, u)) {
        areaManagers.add(u);
      }
    }
    try {
      NotificationService<UsuarioService> ns = new NotificationService<UsuarioService>();
      Notification n = ns.notificarPeticionVacaciones(areaManagers, vacaciones);
      if ((n != null) && (n.getRecipients() != null) && (n.getRecipients().size() > 0)) {
        persist(n);
      }
    } catch (Exception e) {}
  }

  private boolean containsUser(List<Usuario> areaManagers, Usuario u) {
    for (Usuario usuario : areaManagers) {
      if (usuario.getId() == u.getId()) return true;
    }
    return false;
  }
}
