package es.csa.fdt.cyldigital.centros.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;

import es.csa.fdt.cyldigital.servicios.entity.ActividadExterna;
import es.csa.fdt.cyldigital.servicios.entity.Charla;
import es.csa.fdt.cyldigital.servicios.entity.Curso;
import es.csa.fdt.cyldigital.recursos.entity.Ordenador;
import es.csa.fdt.cyldigital.recursos.entity.Recurso;
import es.csa.fdt.cyldigital.servicios.entity.Taller;

/**
 * Clase Aula.
 * 
 * @author csa
 */
@Entity
@Table(name = "aulas")
@AttributeOverride(name = "id", column = @Column(name = "idAula"))
@Audited
public class Aula extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = -7648562176655166537L;

  private String nombre;

  private Centro centro;

  private List<Recurso> recursos;

  /** cursos. */
  protected List<Curso> cursos;

  /** talleres. */
  protected List<Taller> talleres;

  /** charlas. */
  protected List<Charla> charlas;
  
  /** externas */
  protected List<ActividadExterna> externas;

  private String ip;
  private String mask;

  private boolean navegacionControlada;
  private boolean localizacionPorDefecto;
  
  private int numeroPuestos; 

  /**
   * Construye un nuevo objeto de la clase
   */
  public Aula() {
    super();
    recursos = new ArrayList<Recurso>();
    cursos = new ArrayList<Curso>();
    talleres = new ArrayList<Taller>();
    charlas = new ArrayList<Charla>();
    externas = new ArrayList<ActividadExterna>();
    setNavegacionControlada(false);
  }

  /**
   * Devuelve nombre.
   * 
   * @return nombre
   */
  @NotEmpty
  @NotNull
  @Length(max = 255)
  public String getNombre() {
    return nombre;
  }

  /**
   * Asigna nombre.
   * 
   * @param nombre
   *          nombre
   */
  public void setNombre(String nombre) {
    this.nombre = nombre;
  }

  /**
   * Devuelve centro.
   * 
   * @return centro
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @NotNull
  @JoinColumn(name = "idCentro")
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Devuelve recursos.
   * 
   * @return recursos
   */
  @OneToMany(mappedBy = "aula")
  @OrderBy(value = "nombre ASC")
  public List<Recurso> getRecursos() {
    return recursos;
  }

  /**
   * Asigna recursos.
   * 
   * @param recursos
   *          recursos
   */
  public void setRecursos(List<Recurso> recursos) {
    this.recursos = recursos;
  }

  /**
   * Devuelve cursos.
   * 
   * @return cursos
   */
  @OneToMany(mappedBy = "aula")
  @OrderBy(value = "fechaInicio ASC")
  public List<Curso> getCursos() {
    return cursos;
  }

  /**
   * Asigna cursos.
   * 
   * @param cursos
   *          cursos
   */
  public void setCursos(List<Curso> cursos) {
    this.cursos = cursos;
  }
  
  /**
   * Devuelve externas.
   * 
   * @return externas
   */
  @OneToMany(mappedBy = "aula")
  @OrderBy(value = "fechaInicio ASC")
  public List<ActividadExterna> getExternas() {
    return externas;
  }

  /**
   * Asigna externas.
   * 
   * @param externas
   *          externas
   */
  public void setExternas(List<ActividadExterna> externas) {
    this.externas = externas;
  }

  /**
   * Devuelve talleres.
   * 
   * @return talleres
   */
  @OneToMany(mappedBy = "aula")
  @OrderBy(value = "fechaInicio ASC")
  public List<Taller> getTalleres() {
    return talleres;
  }

  /**
   * Asigna talleres.
   * 
   * @param talleres
   *          talleres
   */
  public void setTalleres(List<Taller> talleres) {
    this.talleres = talleres;
  }

  /**
   * Devuelve charlas.
   * 
   * @return charlas
   */
  @OneToMany(mappedBy = "aula")
  @OrderBy(value = "fechaInicio ASC")
  public List<Charla> getCharlas() {
    return charlas;
  }

  /**
   * Asigna charlas.
   * 
   * @param charlas
   *          charlas
   */
  public void setCharlas(List<Charla> charlas) {
    this.charlas = charlas;
  }

  /**
   * Devuelve ip.
   * 
   * @return ip
   */
  @Length(max = 15)
  public String getIp() {
    return ip;
  }

  /**
   * Asigna ip.
   * 
   * @param ip
   *          ip
   */
  public void setIp(String ip) {
    this.ip = ip;
  }

  /**
   * Devuelve mask.
   * 
   * @return mask
   */
  @Length(max = 15)
  public String getMask() {
    return mask;
  }

  /**
   * Asigna mask.
   * 
   * @param mask
   *          mask
   */
  public void setMask(String mask) {
    this.mask = mask;
  }

  /**
   * Checks if is navegacion controlada.
   * 
   * @return true, if is navegacion controlada
   */
  public boolean isNavegacionControlada() {
    return navegacionControlada;
  }

  /**
   * Asigna navegacion controlada.
   * 
   * @param navegacionControlada
   *          navegacion controlada
   */
  public void setNavegacionControlada(boolean navegacionControlada) {
    this.navegacionControlada = navegacionControlada;
  }

  /**
   * Checks if is localizacion por defecto.
   * 
   * @return true, if is localizacion por defecto
   */
  public boolean isLocalizacionPorDefecto() {
    return localizacionPorDefecto;
  }

  /**
   * Asigna localizacion por defecto.
   * 
   * @param localizacionPorDefecto
   *          localizacion por defecto
   */
  public void setLocalizacionPorDefecto(boolean localizacionPorDefecto) {
    this.localizacionPorDefecto = localizacionPorDefecto;
  }

  /**
   * Devuelve el número de puestos del aula
   * 
   * @return the numeroPuestos
   */
  @Column(nullable = false, columnDefinition = "INT NOT NULL DEFAULT 0")
  @NotNull
  public int getNumeroPuestos() {
	  return numeroPuestos;
  }

  /**
   * Asigna el número de puesto del aula
   * 
   * @param numeroPuestos the numeroPuestos to set
   */
  public void setNumeroPuestos(int numeroPuestos) {
	  this.numeroPuestos = numeroPuestos;
  }

  /**
   * Devuelve el número de equipos asignados al aula.
   * 
   * @return numero de equipos asignados al aula.
   */
  @Transient
  public Integer getNumeroOrdenadores() {
	  Integer ncomputers = 0;
	  
	  for (Recurso r : this.recursos)
		  if (r instanceof Ordenador)
			  ncomputers++;
	  
	  return ncomputers;
  }
}
