package es.csa.fdt.cyldigital.centros.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.Date;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import org.hibernate.envers.Audited;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;

/**
 * Clase BP.
 * 
 * @author csa
 */
@Entity
@Table(name = "BP")
@AttributeOverride(name = "id", column = @Column(name = "idBP"))
@Audited
public class BP extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = -7648562176655166537L;

  private Tipo tipo;
  private Centro centro;
  private Integer bpSemanal;
  private Integer bpMensual;
  private Integer bpAnual;
  private int minimo = -5;
  private int maximo = 5;
  private Date fechaModificacion;
  private boolean signo = true;

  /**
   * Enum Tipo.
   */
  public enum Tipo {
    ALTAS,
    ALTAS_MEDIAS,
    EDAD_MEDIA,
    BAJAS,
    NUMEROCURSOS,
    NUMEROALUMNOS_CURSOS,
    NUMEROALUMNOS_ASISTENTES_CURSOS,
    ALUMNOS_NUEVOS_CURSO,
    ALUMNOS_REPETIDORES_CURSO,
    EVALUACION_MEDIA_CURSOS,
    HORAS_FORMACION_CURSOS,
    PORC_OCUPACION_CURSOS,
    PORC_ASISTENTES_CURSOS,
    NUMEROALUMNOS_LISTA_ESPERA_CURSOS,
    PORCENTAJE_ALUMNOS_REPITEN_CURSO,
    CURSOS_CERRADOS,
    NUMEROTALLERES,
    NUMEROALUMNOS_TALLERES,
    NUMEROALUMNOS_ASISTENTES_TALLERES,
    ALUMNOS_NUEVOS_TALLER,
    ALUMNOS_REPETIDORES_TALLER,
    HORAS_FORMACION_TALLERES,
    PORC_OCUPACION_TALLERES,
    PORC_ASISTENTES_TALLERES,
    NUMEROALUMNOS_LISTA_ESPERA_TALLERES,
    PORCENTAJE_ALUMNOS_REPITEN_TALLER,
    TALLERES_CERRADOS,
    NUMEROCURSOS_TERCEROS,
    NUMEROALUMNOS_TERCEROS,
    NUMEROHORAS_TERCEROS,
    USUARIOS_AULA_VIRTUAL,
    PORC_NAVEGACION_LIBRE,
    HORAS_DISPONIBLES_EQUIPO,
    HORAS_USO_EQUIPO,
    N_CONSULTAS_ASESORAMIENTO,
    USUARIOS_WIFI;
  }

  /**
   * Construye un nuevo objeto de la clase
   */
  public BP() {
    super();
  }

  /**
   * Asigna tipo.
   * 
   * @param tipo
   *          tipo
   */
  public void setTipo(Tipo tipo) {
    this.tipo = tipo;
  }

  /**
   * Devuelve tipo.
   * 
   * @return tipo
   */
  @NotNull
  @Enumerated(value = EnumType.STRING)
  public Tipo getTipo() {
    return tipo;
  }

  /**
   * Asigna bp semanal.
   * 
   * @param bpSemanal
   *          bp semanal
   */
  public void setBpSemanal(Integer bpSemanal) {
    this.bpSemanal = bpSemanal;
  }

  /**
   * Devuelve bp semanal.
   * 
   * @return bp semanal
   */
  public Integer getBpSemanal() {
    return bpSemanal;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Devuelve centro.
   * 
   * @return centro
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idCentro")
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna bp mensual.
   * 
   * @param bpMensual
   *          bp mensual
   */
  public void setBpMensual(Integer bpMensual) {
    this.bpMensual = bpMensual;
  }

  /**
   * Devuelve bp mensual.
   * 
   * @return bp mensual
   */
  public Integer getBpMensual() {
    return bpMensual;
  }

  /**
   * Asigna bp anual.
   * 
   * @param bpAnual
   *          bp anual
   */
  public void setBpAnual(Integer bpAnual) {
    this.bpAnual = bpAnual;
  }

  /**
   * Devuelve bp anual.
   * 
   * @return bp anual
   */
  public Integer getBpAnual() {
    return bpAnual;
  }

  /**
   * Asigna fecha modificacion.
   * 
   * @param fechaModificacion
   *          fecha modificacion
   */
  public void setFechaModificacion(Date fechaModificacion) {
    this.fechaModificacion = fechaModificacion;
  }

  /**
   * Devuelve fecha modificacion.
   * 
   * @return fecha modificacion
   */
  public Date getFechaModificacion() {
    return fechaModificacion;
  }

  /**
   * Asigna minimo.
   * 
   * @param minimo
   *          minimo
   */
  public void setMinimo(int minimo) {
    this.minimo = minimo;
  }

  /**
   * Devuelve minimo.
   * 
   * @return minimo
   */
  public int getMinimo() {
    return minimo;
  }

  /**
   * Asigna maximo.
   * 
   * @param maximo
   *          maximo
   */
  public void setMaximo(int maximo) {
    this.maximo = maximo;
  }

  /**
   * Devuelve maximo.
   * 
   * @return maximo
   */
  public int getMaximo() {
    return maximo;
  }

  /**
   * Asigna signo.
   * 
   * @param signo
   *          signo
   */
  public void setSigno(boolean signo) {
    this.signo = signo;
  }

  /**
   * Checks if is signo.
   * 
   * @return true, if is signo
   */
  public boolean isSigno() {
    return signo;
  }

}
