package es.csa.fdt.cyldigital.centros.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.OrderBy;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Email;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.financiero.entity.Caja;
import es.csa.fdt.cyldigital.incidencias.entity.Incidencia;
import es.csa.fdt.cyldigital.notificacion.entity.Alarma;
import es.csa.fdt.cyldigital.notificacion.entity.InformeNotification;
import es.csa.fdt.cyldigital.recursos.entity.Recurso;
import es.csa.fdt.cyldigital.recursos.entity.Router;
import es.csa.fdt.cyldigital.servicios.entity.ActividadExterna;
import es.csa.fdt.cyldigital.servicios.entity.Charla;
import es.csa.fdt.cyldigital.servicios.entity.Curso;
import es.csa.fdt.cyldigital.servicios.entity.Taller;
import es.csa.fdt.cyldigital.servicios.entity.Wifi;
import es.csa.fdt.cyldigital.usuarios.entity.Localidad;
import es.csa.fdt.cyldigital.usuarios.entity.Provincia;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;

/**
 * Clase Centro.
 * 
 * @author csa
 */
@Entity
@Table(name = "centros")
@AttributeOverride(name = "id", column = @Column(name = "idCentro"))
@Audited
public class Centro extends AbstractEntity implements Serializable, Comparable<Centro> {

  private static final long serialVersionUID = -7648562176655166537L;

  private String nombre;
  private Localidad localidad;
  private Provincia provincia;
  private String direccion;
  private String telefono;
  private String fax;
  private String email;
  
  /*
  private Date horaIniManana;
  private Date horaFinManana;
  private Date horaIniTarde;
  private Date horaFinTarde;
  private Date horaIniSabado;
  private Date horaFinSabado;
  */
  
  private Date fechaApertura;

  // Monday schedule
  private Date mondayMorningStartHour;
  private Date mondayMorningEndHour;
  private Date mondayAfternoonStartHour;
  private Date mondayAfternoonEndHour;
  // Tuesday schedule
  private Date tuesdayMorningStartHour;
  private Date tuesdayMorningEndHour;
  private Date tuesdayAfternoonStartHour;
  private Date tuesdayAfternoonEndHour;
  // Wednesday schedule
  private Date wednesdayMorningStartHour;
  private Date wednesdayMorningEndHour;
  private Date wednesdayAfternoonStartHour;
  private Date wednesdayAfternoonEndHour;
  // Thursday schedule
  private Date thursdayMorningStartHour;
  private Date thursdayMorningEndHour;
  private Date thursdayAfternoonStartHour;
  private Date thursdayAfternoonEndHour;
  // Friday schedule
  private Date fridayMorningStartHour;
  private Date fridayMorningEndHour;
  private Date fridayAfternoonStartHour;
  private Date fridayAfternoonEndHour;
  // Saturday schedule
  private Date saturdayMorningStartHour;
  private Date saturdayMorningEndHour;
  private Date saturdayAfternoonStartHour;
  private Date saturdayAfternoonEndHour;
  // Sunday schedule
  private Date sundayMorningStartHour;
  private Date sundayMorningEndHour;
  private Date sundayAfternoonStartHour;
  private Date sundayAfternoonEndHour;
  
  private Caja caja;
  private String urlLdap;
  private Router routerPrincipal;

  /**
   * Nombre de la persona de contacto
   */
  private String contacto;

  private List<Usuario> usuarios;
  private List<Curso> cursos;
  private List<Taller> talleres;
  private List<Charla> charlas;
  private List<ActividadExterna> externas;
  private List<Wifi> wifis;
  private List<Aula> aulas;
  private List<Recurso> recursos;
  private List<Incidencia> incidencias;
  private List<InformeNotification> informes;
  private List<Alarma> alarmas;

  private String dc_ldap;

  /**
   * Construye un nuevo objeto de la clase
   */
  public Centro() {
    super();
    aulas = new ArrayList<Aula>();
    talleres = new ArrayList<Taller>();
    charlas = new ArrayList<Charla>();
    cursos = new ArrayList<Curso>();
    externas = new ArrayList<ActividadExterna>();
    usuarios = new ArrayList<Usuario>();
    recursos = new ArrayList<Recurso>();
    wifis = new ArrayList<Wifi>();
    informes = new ArrayList<InformeNotification>();
    alarmas = new ArrayList<Alarma>();
  }

  /**
   * Define el método de comparación para ordenar y buscar
   */
  @Transient
  public int compareTo(Centro o) {
	  	if ((this.getId() == o.getId()) && (this.getId() == 0))
	  		return (this.getNombre().compareTo(o.getNombre()));
	  	else
	  		return ((Long)this.getId()).compareTo((Long)o.getId());
  }

  /**
   * Devuelve el nombre del centro dentro del LDAP.
   * 
   * @return El nombre del centro en el DC del LDAP, si n o es nulo. Si es nulo, el nombre del centro en mayúsculas.
   */
  @Length(max = 255)
  public String getDc_ldap() {
    return dc_ldap == null ? (getDominio() != null ? getDominio().toUpperCase() : null) : dc_ldap;
  }

  /**
   * Asigna dc_ldap.
   * 
   * @param dcLdap
   *          dc_ldap
   */
  public void setDc_ldap(String dcLdap) {
    dc_ldap = dcLdap;
  }

  /**
   * Devuelve nombre.
   * 
   * @return nombre
   */
  @Length(max = 255)
  @NotNull
  @NotEmpty
  public String getNombre() {
    return nombre;
  }

  /**
   * Asigna nombre.
   * 
   * @param nombre
   *          nombre
   */
  public void setNombre(String nombre) {
    this.nombre = nombre;
  }

  /**
   * Devuelve localidad.
   * 
   * @return localidad
   */
  @ManyToOne(fetch = FetchType.EAGER)
  @NotNull
  @JoinColumn(name = "idLocalidad")
  public Localidad getLocalidad() {
    return localidad;
  }

  /**
   * Asigna localidad.
   * 
   * @param localidad
   *          localidad
   */
  public void setLocalidad(Localidad localidad) {
    this.localidad = localidad;
  }

  /**
   * Devuelve provincia.
   * 
   * @return provincia
   */
  @ManyToOne(fetch = FetchType.EAGER)
  @NotNull
  @JoinColumn(name = "idProvincia")
  public Provincia getProvincia() {
    return provincia;
  }

  /**
   * Asigna provincia.
   * 
   * @param provincia
   *          provincia
   */
  public void setProvincia(Provincia provincia) {
    this.provincia = provincia;
  }

  /**
   * Devuelve usuarios.
   * 
   * @return usuarios
   */
  @OneToMany(mappedBy = "centro")
  @OrderBy(value = "nombre ASC")
  public List<Usuario> getUsuarios() {
    return usuarios;
  }

  /**
   * Asigna usuarios.
   * 
   * @param usuarios
   *          usuarios
   */
  public void setUsuarios(List<Usuario> usuarios) {
    this.usuarios = usuarios;
  }

  /**
   * Devuelve cursos.
   * 
   * @return cursos
   */
  @OneToMany(mappedBy = "centro")
  public List<Curso> getCursos() {
    return cursos;
  }

  /**
   * Asigna cursos.
   * 
   * @param cursos
   *          cursos
   */
  public void setCursos(List<Curso> cursos) {
    this.cursos = cursos;
  }

  /**
   * Devuelve talleres.
   * 
   * @return talleres
   */
  @OneToMany(mappedBy = "centro")
  public List<Taller> getTalleres() {
    return talleres;
  }

  /**
   * Asigna talleres.
   * 
   * @param talleres
   *          talleres
   */
  public void setTalleres(List<Taller> talleres) {
    this.talleres = talleres;
  }

  /**
   * Devuelve charlas.
   * 
   * @return charlas
   */
  @OneToMany(mappedBy = "centro")
  public List<Charla> getCharlas() {
    return charlas;
  }

  /**
   * Asigna charlas.
   * 
   * @param charlas
   *          charlas
   */
  public void setCharlas(List<Charla> charlas) {
    this.charlas = charlas;
  }

  /**
   * @return the externas
   */
  @OneToMany(mappedBy = "centro")
  public List<ActividadExterna> getExternas() {
	  return externas;
  }

  /**
   * @param externas the externas to set
   */
  public void setExternas(List<ActividadExterna> externas) {
	  this.externas = externas;
  }

/**
   * Devuelve informes.
   * 
   * @return informes
   */
  @OneToMany(mappedBy = "centro")
  public List<InformeNotification> getInformes() {
    return informes;
  }

  /**
   * Asigna informes.
   * 
   * @param informes
   *          informes
   */
  public void setInformes(List<InformeNotification> informes) {
    this.informes = informes;
  }

  /**
   * Devuelve alarmas.
   * 
   * @return alarmas
   */
  @OneToMany(mappedBy = "centro")
  public List<Alarma> getAlarmas() {
    return alarmas;
  }

  /**
   * Asigna alarmas.
   * 
   * @param alarmas
   *          alarmas
   */
  public void setAlarmas(List<Alarma> alarmas) {
    this.alarmas = alarmas;
  }

  /**
   * Devuelve wifis.
   * 
   * @return wifis
   */
  @OneToMany(mappedBy = "centro")
  public List<Wifi> getWifis() {
    return wifis;
  }

  /**
   * Asigna wifis.
   * 
   * @param wifis
   *          wifis
   */
  public void setWifis(List<Wifi> wifis) {
    this.wifis = wifis;
  }

  /**
   * Devuelve aulas.
   * 
   * @return aulas
   */
  @OneToMany(mappedBy = "centro")
  @OrderBy(value = "nombre ASC")
  public List<Aula> getAulas() {
    return aulas;
  }

  /**
   * Asigna aulas.
   * 
   * @param aulas
   *          aulas
   */
  public void setAulas(List<Aula> aulas) {
    this.aulas = aulas;
  }

  /**
   * Devuelve recursos.
   * 
   * @return recursos
   */
  @OneToMany(mappedBy = "centro")
  @OrderBy(value = "nombre ASC")
  public List<Recurso> getRecursos() {
    return recursos;
  }

  /**
   * Asigna recursos.
   * 
   * @param recursos
   *          recursos
   */
  public void setRecursos(List<Recurso> recursos) {
    this.recursos = recursos;
  }

  /**
   * Devuelve hora ini manana.
   * 
   * @return hora ini manana
   */
  /*
  @Temporal(TemporalType.TIME)
  public Date getHoraIniManana() {
    return horaIniManana;
  }*/

  /**
   * Asigna hora ini manana.
   * 
   * @param horaIniManana
   *          hora ini manana
   */
  /*
  public void setHoraIniManana(Date horaIniManana) {
    this.horaIniManana = horaIniManana;
  }
  */

  /**
   * Devuelve hora fin manana.
   * 
   * @return hora fin manana
   */
  /*
  @Temporal(TemporalType.TIME)
  public Date getHoraFinManana() {
    return horaFinManana;
  }
  */

  /**
   * Asigna hora fin manana.
   * 
   * @param horaFinManana
   *          hora fin manana
   */
  /*
  public void setHoraFinManana(Date horaFinManana) {
    this.horaFinManana = horaFinManana;
  }
  */

  /**
   * Devuelve hora ini tarde.
   * 
   * @return hora ini tarde
   */
  /*
  @Temporal(TemporalType.TIME)
  public Date getHoraIniTarde() {
    return horaIniTarde;
  }
  */

  /**
   * Asigna hora ini tarde.
   * 
   * @param horaIniTarde
   *          hora ini tarde
   */
  /*
  public void setHoraIniTarde(Date horaIniTarde) {
    this.horaIniTarde = horaIniTarde;
  }
  */

  /**
   * Devuelve hora fin tarde.
   * 
   * @return hora fin tarde
   */
  /*
  @Temporal(TemporalType.TIME)
  public Date getHoraFinTarde() {
    return horaFinTarde;
  }
  */

  /**
   * Asigna hora fin tarde.
   * 
   * @param horaFinTarde
   *          hora fin tarde
   */
  /*
  public void setHoraFinTarde(Date horaFinTarde) {
    this.horaFinTarde = horaFinTarde;
  }
  */

  /**
   * Devuelve hora ini sabado.
   * 
   * @return hora ini sabado
   */
  /*
  @Temporal(TemporalType.TIME)
  public Date getHoraIniSabado() {
    return horaIniSabado;
  }
  */

  /**
   * Asigna hora ini sabado.
   * 
   * @param horaIniSabado
   *          hora ini sabado
   */
  /*
  public void setHoraIniSabado(Date horaIniSabado) {
    this.horaIniSabado = horaIniSabado;
  }
  */

  /**
   * Devuelve hora fin sabado.
   * 
   * @return hora fin sabado
   */
  /*
  @Temporal(TemporalType.TIME)
  public Date getHoraFinSabado() {
    return horaFinSabado;
  }
  */

  /**
   * Asigna hora fin sabado.
   * 
   * @param horaFinSabado
   *          hora fin sabado
   */
  /*
  public void setHoraFinSabado(Date horaFinSabado) {
    this.horaFinSabado = horaFinSabado;
  }
  */

  /**
   * Devuelve direccion.
   * 
   * @return direccion
   */
  @Length(max = 150)
  public String getDireccion() {
    return direccion;
  }

  /**
   * Asigna direccion.
   * 
   * @param direccion
   *          direccion
   */
  public void setDireccion(String direccion) {
    this.direccion = direccion;
  }

  /**
   * Devuelve nombre de la persona de contacto.
   * 
   * @return nombre de la persona de contacto
   */
  @Length(max = 50)
  public String getContacto() {
    return contacto;
  }

  /**
   * Asigna nombre de la persona de contacto.
   * 
   * @param contacto
   *          nombre de la persona de contacto
   */
  public void setContacto(String contacto) {
    this.contacto = contacto;
  }

  /**
   * Devuelve telefono.
   * 
   * @return telefono
   */
  @Length(max = 20)
  public String getTelefono() {
    return telefono;
  }

  /**
   * Asigna telefono.
   * 
   * @param telefono
   *          telefono
   */
  public void setTelefono(String telefono) {
    this.telefono = telefono;
  }

  /**
   * Devuelve fax.
   * 
   * @return fax
   */
  @Length(max = 20)
  public String getFax() {
    return fax;
  }

  /**
   * Asigna fax.
   * 
   * @param fax
   *          fax
   */
  public void setFax(String fax) {
    this.fax = fax;
  }

  /**
   * Devuelve email.
   * 
   * @return email
   */
  @Length(max = 255)
  @Email
  public String getEmail() {
    return email;
  }

  /**
   * Asigna email.
   * 
   * @param email
   *          email
   */
  public void setEmail(String email) {
    this.email = email == null ? null : email.trim();
  }

  /**
   * Devuelve incidencias.
   * 
   * @return incidencias
   */
  @OneToMany(mappedBy = "centro")
  public List<Incidencia> getIncidencias() {
    return incidencias;
  }

  /**
   * Asigna incidencias.
   * 
   * @param incidencias
   *          incidencias
   */
  public void setIncidencias(List<Incidencia> incidencias) {
    this.incidencias = incidencias;
  }

  /**
   * Devuelve url ldap.
   * 
   * @return url ldap
   */
  @Length(max = 255)
  public String getUrlLdap() {
    return urlLdap;
  }

  /**
   * Asigna url ldap.
   * 
   * @param urlLdap
   *          url ldap
   */
  public void setUrlLdap(String urlLdap) {
    this.urlLdap = urlLdap;
  }

  /**
   * Devuelve router principal.
   * 
   * @return router principal
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idRouter")
  public Router getRouterPrincipal() {
    return routerPrincipal;
  }

  /**
   * Asigna router principal.
   * 
   * @param routerPrincipal
   *          router principal
   */
  public void setRouterPrincipal(Router routerPrincipal) {
    this.routerPrincipal = routerPrincipal;
  }

  /**
   * Devuelve dominio.
   * 
   * @return dominio
   */
  @Transient
  public String getDominio() {
    if (getNombre() != null)
      return (getNombre().trim().toLowerCase().replace('ñ', 'n')
    		  								  .replace('á', 'a')
    		  								  .replace('é', 'e')
    		  								  .replace('í', 'i')
    		  								  .replace('ó', 'o')
    		  								  .replace('ú', 'u'));
    return "";
  }

  /**
   * Devuelve horario manana.
   * 
   * @return horario manana
   */
  /*
  @Transient
  public String getHorarioManana() {
    String resultado = "";
    DateFormat df = new SimpleDateFormat("HH:mm");
    if (getHoraIniManana() != null) {
      resultado += df.format(getHoraIniManana());
    }
    if (getHoraIniManana() != null && getHoraFinManana() != null) resultado += "-";
    if (getHoraFinManana() != null) {
      resultado += df.format(getHoraFinManana());
    }
    return resultado.isEmpty() ? "No disponible" : resultado;
  }
  */

  /**
   * Devuelve horario tarde.
   * 
   * @return horario tarde
   */
  /*
  @Transient
  public String getHorarioTarde() {
    String resultado = "";
    DateFormat df = new SimpleDateFormat("HH:mm");
    if (getHoraIniTarde() != null) {
      resultado += df.format(getHoraIniTarde());
    }
    if (getHoraIniTarde() != null && getHoraFinTarde() != null) resultado += "-";
    if (getHoraFinTarde() != null) {
      resultado += df.format(getHoraFinTarde());
    }
    return resultado.isEmpty() ? "No disponible" : resultado;
  }
  */

  /**
   * Devuelve horario sabado.
   * 
   * @return horario sabado
   */
  /*
  @Transient
  public String getHorarioSabado() {
    String resultado = "";
    DateFormat df = new SimpleDateFormat("HH:mm");
    if (getHoraIniSabado() != null) {
      resultado += df.format(getHoraIniSabado());
    }
    if (getHoraIniSabado() != null && getHoraFinSabado() != null) resultado += "-";
    if (getHoraFinSabado() != null) {
      resultado += df.format(getHoraFinSabado());
    }
    return resultado.isEmpty() ? "No disponible" : resultado;
  }
  */

  /**
   * Devuelve fecha apertura.
   * 
   * @return fecha apertura
   */
  @Temporal(TemporalType.DATE)
  public Date getFechaApertura() {
    return fechaApertura;
  }

  /**
   * Asigna fecha apertura.
   * 
   * @param fechaApertura
   *          fecha apertura
   */
  public void setFechaApertura(Date fechaApertura) {
    this.fechaApertura = fechaApertura;
  }

	/**
	 * @return the mondayMorningStartHour
	 */
	@Temporal(TemporalType.TIME)
	public Date getMondayMorningStartHour() {
		return mondayMorningStartHour;
	}
	
	/**
	 * @param mondayMorningStartHour the mondayMorningStartHour to set
	 */
	public void setMondayMorningStartHour(Date mondayMorningStartHour) {
		this.mondayMorningStartHour = mondayMorningStartHour;
	}
	
	/**
	 * @return the mondayMorningEndHour
	 */
	@Temporal(TemporalType.TIME)
	public Date getMondayMorningEndHour() {
		return mondayMorningEndHour;
	}
	
	/**
	 * @param mondayMorningEndHour the mondayMorningEndHour to set
	 */
	public void setMondayMorningEndHour(Date mondayMorningEndHour) {
		this.mondayMorningEndHour = mondayMorningEndHour;
	}
	
	/**
	 * @return the mondayAfternoonStartHour
	 */
	@Temporal(TemporalType.TIME)
	public Date getMondayAfternoonStartHour() {
		return mondayAfternoonStartHour;
	}
	
	/**
	 * @param mondayAfternoonStartHour the mondayAfternoonStartHour to set
	 */
	public void setMondayAfternoonStartHour(Date mondayAfternoonStartHour) {
		this.mondayAfternoonStartHour = mondayAfternoonStartHour;
	}
	
	/**
	 * @return the mondayAfternoonEndHour
	 */
	@Temporal(TemporalType.TIME)
	public Date getMondayAfternoonEndHour() {
		return mondayAfternoonEndHour;
	}
	
	/**
	 * @param mondayAfternoonEndHour the mondayAfternoonEndHour to set
	 */
	public void setMondayAfternoonEndHour(Date mondayAfternoonEndHour) {
		this.mondayAfternoonEndHour = mondayAfternoonEndHour;
	}
	
	/**
	 * @return the tuesdayMorningStartHour
	 */
	@Temporal(TemporalType.TIME)
	public Date getTuesdayMorningStartHour() {
		return tuesdayMorningStartHour;
	}
	
	/**
	 * @param tuesdayMorningStartHour the tuesdayMorningStartHour to set
	 */
	public void setTuesdayMorningStartHour(Date tuesdayMorningStartHour) {
		this.tuesdayMorningStartHour = tuesdayMorningStartHour;
	}
	
	/**
	 * @return the tuesdayMorningEndHour
	 */
	@Temporal(TemporalType.TIME)
	public Date getTuesdayMorningEndHour() {
		return tuesdayMorningEndHour;
	}
	
	/**
	 * @param tuesdayMorningEndHour the tuesdayMorningEndHour to set
	 */
	public void setTuesdayMorningEndHour(Date tuesdayMorningEndHour) {
		this.tuesdayMorningEndHour = tuesdayMorningEndHour;
	}
	
	/**
	 * @return the tuesdayAfternoonStartHour
	 */
	@Temporal(TemporalType.TIME)
	public Date getTuesdayAfternoonStartHour() {
		return tuesdayAfternoonStartHour;
	}
	
	/**
	 * @param tuesdayAfternoonStartHour the tuesdayAfternoonStartHour to set
	 */
	public void setTuesdayAfternoonStartHour(Date tuesdayAfternoonStartHour) {
		this.tuesdayAfternoonStartHour = tuesdayAfternoonStartHour;
	}
	
	/**
	 * @return the tuesdayAfternoonEndHour
	 */
	@Temporal(TemporalType.TIME)
	public Date getTuesdayAfternoonEndHour() {
		return tuesdayAfternoonEndHour;
	}
	
	/**
	 * @param tuesdayAfternoonEndHour the tuesdayAfternoonEndHour to set
	 */
	public void setTuesdayAfternoonEndHour(Date tuesdayAfternoonEndHour) {
		this.tuesdayAfternoonEndHour = tuesdayAfternoonEndHour;
	}
	
	/**
	 * @return the wednesdayMorningStartHour
	 */
	@Temporal(TemporalType.TIME)
	public Date getWednesdayMorningStartHour() {
		return wednesdayMorningStartHour;
	}
	
	/**
	 * @param wednesdayMorningStartHour the wednesdayMorningStartHour to set
	 */
	public void setWednesdayMorningStartHour(Date wednesdayMorningStartHour) {
		this.wednesdayMorningStartHour = wednesdayMorningStartHour;
	}
	
	/**
	 * @return the wednesdayMorningEndHour
	 */
	@Temporal(TemporalType.TIME)
	public Date getWednesdayMorningEndHour() {
		return wednesdayMorningEndHour;
	}
	
	/**
	 * @param wednesdayMorningEndHour the wednesdayMorningEndHour to set
	 */
	public void setWednesdayMorningEndHour(Date wednesdayMorningEndHour) {
		this.wednesdayMorningEndHour = wednesdayMorningEndHour;
	}
	
	/**
	 * @return the wednesdayAfternoonStartHour
	 */
	@Temporal(TemporalType.TIME)
	public Date getWednesdayAfternoonStartHour() {
		return wednesdayAfternoonStartHour;
	}
	
	/**
	 * @param wednesdayAfternoonStartHour the wednesdayAfternoonStartHour to set
	 */
	public void setWednesdayAfternoonStartHour(Date wednesdayAfternoonStartHour) {
		this.wednesdayAfternoonStartHour = wednesdayAfternoonStartHour;
	}
	
	/**
	 * @return the wednesdayAfternoonEndHour
	 */
	@Temporal(TemporalType.TIME)
	public Date getWednesdayAfternoonEndHour() {
		return wednesdayAfternoonEndHour;
	}
	
	/**
	 * @param wednesdayAfternoonEndHour the wednesdayAfternoonEndHour to set
	 */
	public void setWednesdayAfternoonEndHour(Date wednesdayAfternoonEndHour) {
		this.wednesdayAfternoonEndHour = wednesdayAfternoonEndHour;
	}
	
	/**
	 * @return the thursdayMorningStartHour
	 */
	@Temporal(TemporalType.TIME)
	public Date getThursdayMorningStartHour() {
		return thursdayMorningStartHour;
	}
	
	/**
	 * @param thursdayMorningStartHour the thursdayMorningStartHour to set
	 */
	public void setThursdayMorningStartHour(Date thursdayMorningStartHour) {
		this.thursdayMorningStartHour = thursdayMorningStartHour;
	}
	
	/**
	 * @return the thursdayMorningEndHour
	 */
	@Temporal(TemporalType.TIME)
	public Date getThursdayMorningEndHour() {
		return thursdayMorningEndHour;
	}
	
	/**
	 * @param thursdayMorningEndHour the thursdayMorningEndHour to set
	 */
	public void setThursdayMorningEndHour(Date thursdayMorningEndHour) {
		this.thursdayMorningEndHour = thursdayMorningEndHour;
	}
	
	/**
	 * @return the thursdayAfternoonStartHour
	 */
	@Temporal(TemporalType.TIME)
	public Date getThursdayAfternoonStartHour() {
		return thursdayAfternoonStartHour;
	}
	
	/**
	 * @param thursdayAfternoonStartHour the thursdayAfternoonStartHour to set
	 */
	public void setThursdayAfternoonStartHour(Date thursdayAfternoonStartHour) {
		this.thursdayAfternoonStartHour = thursdayAfternoonStartHour;
	}
	
	/**
	 * @return the thursdayAfternoonEndHour
	 */
	@Temporal(TemporalType.TIME)
	public Date getThursdayAfternoonEndHour() {
		return thursdayAfternoonEndHour;
	}
	
	/**
	 * @param thursdayAfternoonEndHour the thursdayAfternoonEndHour to set
	 */
	public void setThursdayAfternoonEndHour(Date thursdayAfternoonEndHour) {
		this.thursdayAfternoonEndHour = thursdayAfternoonEndHour;
	}
	
	/**
	 * @return the fridayMorningStartHour
	 */
	@Temporal(TemporalType.TIME)
	public Date getFridayMorningStartHour() {
		return fridayMorningStartHour;
	}
	
	/**
	 * @param fridayMorningStartHour the fridayMorningStartHour to set
	 */
	public void setFridayMorningStartHour(Date fridayMorningStartHour) {
		this.fridayMorningStartHour = fridayMorningStartHour;
	}
	
	/**
	 * @return the fridayMorningEndHour
	 */
	@Temporal(TemporalType.TIME)
	public Date getFridayMorningEndHour() {
		return fridayMorningEndHour;
	}
	
	/**
	 * @param fridayMorningEndHour the fridayMorningEndHour to set
	 */
	public void setFridayMorningEndHour(Date fridayMorningEndHour) {
		this.fridayMorningEndHour = fridayMorningEndHour;
	}
	
	/**
	 * @return the fridayAfternoonStartHour
	 */
	@Temporal(TemporalType.TIME)
	public Date getFridayAfternoonStartHour() {
		return fridayAfternoonStartHour;
	}
	
	/**
	 * @param fridayAfternoonStartHour the fridayAfternoonStartHour to set
	 */
	public void setFridayAfternoonStartHour(Date fridayAfternoonStartHour) {
		this.fridayAfternoonStartHour = fridayAfternoonStartHour;
	}
	
	/**
	 * @return the fridayAfternoonEndHour
	 */
	@Temporal(TemporalType.TIME)
	public Date getFridayAfternoonEndHour() {
		return fridayAfternoonEndHour;
	}
	
	/**
	 * @param fridayAfternoonEndHour the fridayAfternoonEndHour to set
	 */
	public void setFridayAfternoonEndHour(Date fridayAfternoonEndHour) {
		this.fridayAfternoonEndHour = fridayAfternoonEndHour;
	}
	
	/**
	 * @return the saturdayMorningStartHour
	 */
	@Temporal(TemporalType.TIME)
	public Date getSaturdayMorningStartHour() {
		return saturdayMorningStartHour;
	}
	
	/**
	 * @param saturdayMorningStartHour the saturdayMorningStartHour to set
	 */
	public void setSaturdayMorningStartHour(Date saturdayMorningStartHour) {
		this.saturdayMorningStartHour = saturdayMorningStartHour;
	}
	
	/**
	 * @return the saturdayMorningEndHour
	 */
	@Temporal(TemporalType.TIME)
	public Date getSaturdayMorningEndHour() {
		return saturdayMorningEndHour;
	}
	
	/**
	 * @param saturdayMorningEndHour the saturdayMorningEndHour to set
	 */
	public void setSaturdayMorningEndHour(Date saturdayMorningEndHour) {
		this.saturdayMorningEndHour = saturdayMorningEndHour;
	}
	
	/**
	 * @return the saturdayAfternoonStartHour
	 */
	@Temporal(TemporalType.TIME)
	public Date getSaturdayAfternoonStartHour() {
		return saturdayAfternoonStartHour;
	}
	
	/**
	 * @param saturdayAfternoonStartHour the saturdayAfternoonStartHour to set
	 */
	public void setSaturdayAfternoonStartHour(Date saturdayAfternoonStartHour) {
		this.saturdayAfternoonStartHour = saturdayAfternoonStartHour;
	}
	
	/**
	 * @return the saturdayAfternoonEndHour
	 */
	@Temporal(TemporalType.TIME)
	public Date getSaturdayAfternoonEndHour() {
		return saturdayAfternoonEndHour;
	}
	
	/**
	 * @param saturdayAfternoonEndHour the saturdayAfternoonEndHour to set
	 */
	public void setSaturdayAfternoonEndHour(Date saturdayAfternoonEndHour) {
		this.saturdayAfternoonEndHour = saturdayAfternoonEndHour;
	}
	
	/**
	 * @return the sundayMorningStartHour
	 */
	@Temporal(TemporalType.TIME)
	public Date getSundayMorningStartHour() {
		return sundayMorningStartHour;
	}
	
	/**
	 * @param sundayMorningStartHour the sundayMorningStartHour to set
	 */
	public void setSundayMorningStartHour(Date sundayMorningStartHour) {
		this.sundayMorningStartHour = sundayMorningStartHour;
	}
	
	/**
	 * @return the sundayMorningEndHour
	 */
	@Temporal(TemporalType.TIME)
	public Date getSundayMorningEndHour() {
		return sundayMorningEndHour;
	}
	
	/**
	 * @param sundayMorningEndHour the sundayMorningEndHour to set
	 */
	public void setSundayMorningEndHour(Date sundayMorningEndHour) {
		this.sundayMorningEndHour = sundayMorningEndHour;
	}
	
	/**
	 * @return the sundayAfternoonStartHour
	 */
	@Temporal(TemporalType.TIME)
	public Date getSundayAfternoonStartHour() {
		return sundayAfternoonStartHour;
	}
	
	/**
	 * @param sundayAfternoonStartHour the sundayAfternoonStartHour to set
	 */
	public void setSundayAfternoonStartHour(Date sundayAfternoonStartHour) {
		this.sundayAfternoonStartHour = sundayAfternoonStartHour;
	}
	
	/**
	 * @return the sundayAfternoonEndHour
	 */
	@Temporal(TemporalType.TIME)
	public Date getSundayAfternoonEndHour() {
		return sundayAfternoonEndHour;
	}
	
	/**
	 * @param sundayAfternoonEndHour the sundayAfternoonEndHour to set
	 */
	public void setSundayAfternoonEndHour(Date sundayAfternoonEndHour) {
		this.sundayAfternoonEndHour = sundayAfternoonEndHour;
	}

  /**
   * Devuelve caja.
   * 
   * @return caja
   */
  @OneToOne(cascade = CascadeType.ALL)
  @JoinColumn(name = "idCaja")
  public Caja getCaja() {
    return caja;
  }

  /**
   * Asigna caja.
   * 
   * @param caja
   *          caja
   */
  public void setCaja(Caja caja) {
    this.caja = caja;
  }
}
