package es.csa.fdt.cyldigital.centros.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.commons.persistence.browser.jpaql.FilterJPAQLByQuery;
import es.csa.fdt.cyldigital.centros.entity.Aula;
import es.csa.fdt.cyldigital.core.service.BrowserFilteredByActive;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;

/**
 * Servicio del Browser de Aulas.
 * 
 * @author csa
 */
@Service
@Repository
public class AulasBrowserImpl extends BrowserFilteredByActive<Aula> implements AulasBrowser<CriterionJPAQL, String> {

  private static final long serialVersionUID = -7227366052245717441L;

  /**
   * Construye un nuevo objeto de la clase
   */
  public AulasBrowserImpl() {
	  super();
	  
	  getSort().setSortAsc(false);
	  getSort().setSortProperty("id");
  }

  /**
   * Apply custom filter.
   * 
   * @param idCentro
   *           id centro
   */
  public void applyCustomFilter(long idCentro) {
	  String query = null;
	  Long id = null;
	  
	  clearFilters();
	  
	  if (Long.valueOf(idCentro) != null) {
		  try {
			  id = Long.valueOf(idCentro);
		  } catch (Exception e) {}

		  FilterJPAQLByQuery<Aula> filter = new FilterJPAQLByQuery<Aula>();
		  
		  if (id != null) {
			  query = " centro.id = ?";
			  
			  Object[] params = { id };
			  
			  filter.getCriterion().addParameters(params);
			  filter.getCriterion().setQuery(query);
			  
			  filters.add(filter);
		  }
	  }
  }

  /**
   * Asigna filter centro.
   * 
   * @param user
   *          filter centro
   */
  public void setFilterCentro(UsuarioLdap user) {
	  if (user.applyFilterCentro()) {
		  removeFilterCentro();
		  
		  if (user.getCentro() != null) {
			  filterByCentro(user.getCentro().getId());
		  }
	  }
  }

  private void removeFilterCentro() {
	  int i = -1;
	  
	  for (Filter<CriterionJPAQL, Aula> f : getFilters())
		  if (f.getCriterion().getQuery().contains("centro.id"))
			  i = getFilters().indexOf(f);
	  
	  if (i > -1)
		  getFilters().remove(i);
  }

  /**
   * Filter by centro.
   * 
   * @param idCentro
   *           id centro
   */
  public void filterByCentro(Long idCentro) {
	  String query = "";
	  boolean flag = false;
	  FilterJPAQLByQuery<Aula> filter = new FilterJPAQLByQuery<Aula>();

	  query += " centro.id = ? ";
	  
	  filter.getCriterion().addParameter(idCentro);

	  for (Filter<CriterionJPAQL, Aula> f : getFilters())
		  if (f.getCriterion().getQuery().contains("centro.id = ?"))
			  flag = true;

	  if (!flag) {
		  filter.getCriterion().setQuery(query);
		  getFilters().add(filter);
	  }
  }
}
