package es.csa.fdt.cyldigital.centros.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.Query;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.centros.entity.BP;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.centros.entity.BP.Tipo;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;


/**
 * Servicio para los BP.
 * 
 * @author csa
 *
 */
@Service
@Repository
public class BPServiceImpl extends AbstractServiceImpl<BP, Long> implements BPService {

  private static final long serialVersionUID = -5575228180361917674L;

  /**
   * Construye un nuevo objeto de la clase
   */
  public BPServiceImpl() {
    super();
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.centros.service.BPService#persistBP(es.csa.fdt.cyldigital.centros.entity.BP)
   */
  public boolean persistBP(BP bp) {
    persist(bp);
    return true;
  }

  // public void persist(BP bp) {
  //
  // }

  /**
   * Nuevo bp.
   * 
   * @param centro
   *           centro
   * @param tipo
   *           tipo
   * @return bP
   */
  public BP nuevoBP(Centro centro, Tipo tipo) {
    BP bp = new BP();
    Date fechaActual = new Date();
    bp.setCentro(centro);
    bp.setFechaModificacion(fechaActual);
    bp.setTipo(tipo);
    bp.setDesactivado(false);
    return bp;
  }

  /**
   * Devolver bp.
   * 
   * @param centro
   *           centro
   * @return list
   */
  @SuppressWarnings("unchecked")
  public List<BP> devolverBP(Centro centro) {
    List<BP> listaFinal = new ArrayList<BP>();

    for (Tipo tipo : Tipo.values()) {
      List<BP> lista = new ArrayList<BP>();
      if (centro != null) {
        Query q = getEntityManager().createQuery(
            "SELECT e FROM es.csa.fdt.cyldigital.centros.entity.BP e where centro = ? and fechaModificacion IN "
                + "(select max(fechaModificacion) from es.csa.fdt.cyldigital.centros.entity.BP "
                + "where centro=? and tipo=?) and tipo=?");
        q.setParameter(1, centro);
        q.setParameter(2, centro);
        q.setParameter(3, tipo);
        q.setParameter(4, tipo);
        lista = q.getResultList();
      } else {
        Query q = getEntityManager().createQuery(
            "SELECT e FROM es.csa.fdt.cyldigital.centros.entity.BP e where centro is null and fechaModificacion IN "
                + "(select max(fechaModificacion) from es.csa.fdt.cyldigital.centros.entity.BP "
                + "where centro is null and tipo=?) and tipo=?");
        q.setParameter(1, tipo);
        q.setParameter(2, tipo);
        lista = q.getResultList();
      }

      if (lista.size() != 0) {
        listaFinal.add(lista.get(0));
        disableMessages();
        persistBP(lista.get(0));
        enableMessages();
      } else {
        BP nuevo = nuevoBP(centro, tipo);
        listaFinal.add(nuevo);
        disableMessages();
        persistBP(nuevo);
        enableMessages();
      }
    }
    return listaFinal;

  }

}
