package es.csa.fdt.cyldigital.centros.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;

import javax.persistence.Query;

import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.faces.util.PredefinedLocaleResolver;
import es.csa.fdt.cyldigital.centros.entity.BP;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.centros.entity.BP.Tipo;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
//import es.csa.fdt.cyldigital.financiero.entity.Caja;
import es.csa.fdt.cyldigital.informes.CuadroDeMandos;
import es.csa.fdt.cyldigital.usuarios.entity.Localidad;
import es.csa.fdt.cyldigital.usuarios.entity.Provincia;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;

/**
 * Servicio para los Centros.
 * 
 * @author csa
 *
 */
@Service
@Repository
public class CentroServiceImpl extends AbstractServiceImpl<Centro, Long> implements CentroService {

  private List<Provincia> provincias;

  private static final long serialVersionUID = -5575228180361917674L;

  // ** Campos para estadisticas **/
  private Centro centro;
  private Date fechaInicio;
  private Date fechaFin;
  private String fechaInicioLegible;
  private String fechaFinLegible;
  private Long numeroUsuariosMatriculados;
  private ETiempoInformeUsuario tiempoInformeUsuario;
  
  // Monday
  private Boolean mondayMorning;
  private Boolean mondayAfternoon;
  // Tuesday
  private Boolean tuesdayMorning;
  private Boolean tuesdayAfternoon;
  // Wenesday
  private Boolean wednesdayMorning;
  private Boolean wednesdayAfternoon;
  // Thursday
  private Boolean thursdayMorning;
  private Boolean thursdayAfternoon;
  // Friday
  private Boolean fridayMorning;
  private Boolean fridayAfternoon;
  // Saturday
  private Boolean saturdayMorning;
  private Boolean saturdayAfternoon;
  // Sunday
  private Boolean sundayMorning;
  private Boolean sundayAfternoon;
 
  enum ETiempoInformeUsuario {
	  SEMANA,
	  MES,	 
	  ANYO,
	  MES_ANTERIOR,
	  ANYO_ANTERIOR,
	  INF_PERSONALIZADO;
  }
  
  private String nombreLocalidad = "";
  
  private List<Localidad> listLocalidades = null;
  
  private Long idLocalidadSeleccionada = -1L;

  /**
   * Construye un nuevo objeto de la clase
   */
  public CentroServiceImpl() {
    super();
    
    mondayMorning = true;
    mondayAfternoon = true;
    tuesdayMorning = true;
    tuesdayAfternoon = true;
    wednesdayMorning = true;
    wednesdayAfternoon = true;
    thursdayMorning = true;
    thursdayAfternoon = true;
    fridayMorning = true;
    fridayAfternoon = true;
    saturdayMorning = true;
    saturdayAfternoon = false;
    sundayMorning = false;
    sundayAfternoon = false;
  }

  /**
   * Find centro.
   * 
   * @param idCentro
   *           id centro
   * @return centro
   */
  public Centro findCentro(Long idCentro) {
	  if (idCentro != null) {
		  Centro centro; 

		  centro = find(idCentro);

		  mondayMorning = ((centro.getMondayMorningStartHour() != null) && (centro.getMondayMorningEndHour() != null));
		  mondayAfternoon = ((centro.getMondayAfternoonStartHour() != null) && (centro.getMondayAfternoonEndHour() != null));

		  tuesdayMorning = ((centro.getTuesdayMorningStartHour() != null) && (centro.getTuesdayMorningEndHour() != null));
		  tuesdayAfternoon = ((centro.getTuesdayAfternoonStartHour() != null) && (centro.getTuesdayAfternoonEndHour() != null));

		  wednesdayMorning = ((centro.getWednesdayMorningStartHour() != null) && (centro.getWednesdayMorningEndHour() != null));
		  wednesdayAfternoon = ((centro.getWednesdayAfternoonStartHour() != null) && (centro.getWednesdayAfternoonEndHour() != null));

		  thursdayMorning = ((centro.getThursdayMorningStartHour() != null) && (centro.getThursdayMorningEndHour() != null));
		  thursdayAfternoon = ((centro.getThursdayAfternoonStartHour() != null) && (centro.getThursdayAfternoonEndHour() != null));

		  fridayMorning = ((centro.getFridayMorningStartHour() != null) && (centro.getFridayMorningEndHour() != null));
		  fridayAfternoon = ((centro.getFridayAfternoonStartHour() != null) && (centro.getFridayAfternoonEndHour() != null));

		  saturdayMorning = ((centro.getSaturdayMorningStartHour() != null) && (centro.getSaturdayMorningEndHour() != null));
		  saturdayAfternoon = ((centro.getSaturdayAfternoonStartHour() != null) && (centro.getSaturdayAfternoonEndHour() != null));

		  sundayMorning = ((centro.getSundayMorningStartHour() != null) && (centro.getSundayMorningEndHour() != null));
		  sundayAfternoon = ((centro.getSundayAfternoonStartHour() != null) && (centro.getSundayAfternoonEndHour() != null));

		  return centro;
	  }
	  else
		  return null;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.centros.service.CentroService#persistCentro(es.csa.fdt.cyldigital.centros.entity.Centro)
   */
  public boolean persistCentro(Centro centro) {
	/*
    Caja c = new Caja();
    c.setValorCaja(new Long(0));
    centro.setCaja(c);
    */
	
	Boolean success = true;
	Locale language = LocaleContextHolder.getLocaleContext().getLocale();
	ResourceBundle rb = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", language);
	Object[] args = new Object[1];
	
	
	// Monday
	if (!mondayMorning) {
		centro.setMondayMorningStartHour(null);
		centro.setMondayMorningEndHour(null);
	}
	else if (centro.getMondayMorningEndHour().compareTo(centro.getMondayMorningStartHour()) <= 0) {
		args[0] = rb.getString("schedule.daysWeek.Monday");
		addErrorMessage("centros.schedule.error.StartHourEqualOrAfterEndHour", args);
		
		success = false;
	}
	
	if (!mondayAfternoon) {
		centro.setMondayAfternoonStartHour(null);
		centro.setMondayAfternoonEndHour(null);
	}
	else if (centro.getMondayAfternoonEndHour().compareTo(centro.getMondayAfternoonStartHour()) <= 0) {
		args[0] = rb.getString("schedule.daysWeek.Monday");
		addErrorMessage("centros.schedule.error.StartHourEqualOrAfterEndHour", args);
		
		success = false;
	}
	
	if ((mondayMorning && mondayAfternoon) &&
		(centro.getMondayAfternoonStartHour().compareTo(centro.getMondayMorningEndHour()) <= 0)) {
		args[0] = rb.getString("schedule.daysWeek.Monday");
		addErrorMessage("centros.schedule.error.MorningAfternoonOverlap", args);
		
		success = false;
	}

	// Tuesday
	if (!tuesdayMorning) {
		centro.setTuesdayMorningStartHour(null);
		centro.setTuesdayMorningEndHour(null);
	}
	else if (centro.getTuesdayMorningEndHour().compareTo(centro.getTuesdayMorningStartHour()) <= 0) {
		args[0] = rb.getString("schedule.daysWeek.Tuesday");
		addErrorMessage("centros.schedule.error.StartHourEqualOrAfterEndHour", args);
		
		success = false;
	}
	
	if (!tuesdayAfternoon) {
		centro.setTuesdayAfternoonStartHour(null);
		centro.setTuesdayAfternoonEndHour(null);
	}
	else if (centro.getTuesdayAfternoonEndHour().compareTo(centro.getTuesdayAfternoonStartHour()) <= 0) {
		args[0] = rb.getString("schedule.daysWeek.Tuesday");
		addErrorMessage("centros.schedule.error.StartHourEqualOrAfterEndHour", args);
		
		success = false;
	}
	
	if ((tuesdayMorning && tuesdayAfternoon) &&
		(centro.getTuesdayAfternoonStartHour().compareTo(centro.getTuesdayMorningEndHour()) <= 0)) {
		args[0] = rb.getString("schedule.daysWeek.Tuesday");
		addErrorMessage("centros.schedule.error.MorningAfternoonOverlap", args);
			
		success = false;
	}
	
	// Wednesday
	if (!wednesdayMorning) {
		centro.setWednesdayMorningStartHour(null);
		centro.setWednesdayMorningEndHour(null);
	}
	else if (centro.getWednesdayMorningEndHour().compareTo(centro.getWednesdayMorningStartHour()) <= 0) {
		args[0] = rb.getString("schedule.daysWeek.Wednesday");
		addErrorMessage("centros.schedule.error.StartHourEqualOrAfterEndHour", args);
		
		success = false;
	}
	
	if (!wednesdayAfternoon) {
		centro.setWednesdayAfternoonStartHour(null);
		centro.setWednesdayAfternoonEndHour(null);
	}
	else if (centro.getWednesdayAfternoonEndHour().compareTo(centro.getWednesdayAfternoonStartHour()) <= 0) {
		args[0] = rb.getString("schedule.daysWeek.Wednesday");
		addErrorMessage("centros.schedule.error.StartHourEqualOrAfterEndHour", args);
		
		success = false;
	}
	
	if ((wednesdayMorning && wednesdayAfternoon) &&
		(centro.getWednesdayAfternoonStartHour().compareTo(centro.getWednesdayMorningEndHour()) <= 0)) {
		args[0] = rb.getString("schedule.daysWeek.Wednesday");
		addErrorMessage("centros.schedule.error.MorningAfternoonOverlap", args);
			
		success = false;
	}
	
	// Thursday
	if (!thursdayMorning) {
		centro.setThursdayMorningStartHour(null);
		centro.setThursdayMorningEndHour(null);
	}
	else if (centro.getThursdayMorningEndHour().compareTo(centro.getThursdayMorningStartHour()) <= 0) {
		args[0] = rb.getString("schedule.daysWeek.Thursday");
		addErrorMessage("centros.schedule.error.StartHourEqualOrAfterEndHour", args);
		
		success = false;
	}
	
	if (!thursdayAfternoon) {
		centro.setThursdayAfternoonStartHour(null);
		centro.setThursdayAfternoonEndHour(null);
	}
	else if (centro.getThursdayAfternoonEndHour().compareTo(centro.getThursdayAfternoonStartHour()) <= 0) {
		args[0] = rb.getString("schedule.daysWeek.Thursday");
		addErrorMessage("centros.schedule.error.StartHourEqualOrAfterEndHour", args);
		
		success = false;
	}
	
	if ((thursdayMorning && thursdayAfternoon) &&
		(centro.getThursdayAfternoonStartHour().compareTo(centro.getThursdayMorningEndHour()) <= 0)) {
		args[0] = rb.getString("schedule.daysWeek.Thursday");
		addErrorMessage("centros.schedule.error.MorningAfternoonOverlap", args);
			
		success = false;
	}
	
	// Friday
	if (!fridayMorning) {
		centro.setFridayMorningStartHour(null);
		centro.setFridayMorningEndHour(null);
	}
	else if (centro.getFridayMorningEndHour().compareTo(centro.getFridayMorningStartHour()) <= 0) {
		args[0] = rb.getString("schedule.daysWeek.Friday");
		addErrorMessage("centros.schedule.error.StartHourEqualOrAfterEndHour", args);
		
		success = false;
	}
	
	if (!fridayAfternoon) {
		centro.setFridayAfternoonStartHour(null);
		centro.setFridayAfternoonEndHour(null);
	}
	else if (centro.getFridayAfternoonEndHour().compareTo(centro.getFridayAfternoonStartHour()) <= 0) {
		args[0] = rb.getString("schedule.daysWeek.Friday");
		addErrorMessage("centros.schedule.error.StartHourEqualOrAfterEndHour", args);
		
		success = false;
	}
	
	if ((fridayMorning && fridayAfternoon) &&
		(centro.getFridayAfternoonStartHour().compareTo(centro.getFridayMorningEndHour()) <= 0)) {
		args[0] = rb.getString("schedule.daysWeek.Friday");
		addErrorMessage("centros.schedule.error.MorningAfternoonOverlap", args);
			
		success = false;
	}
	
	// Saturday
	if (!saturdayMorning) {
		centro.setSaturdayMorningStartHour(null);
		centro.setSaturdayMorningEndHour(null);
	}
	else if (centro.getSaturdayMorningEndHour().compareTo(centro.getSaturdayMorningStartHour()) <= 0) {
		args[0] = rb.getString("schedule.daysWeek.Saturday");
		addErrorMessage("centros.schedule.error.StartHourEqualOrAfterEndHour", args);
		
		success = false;
	}
	
	if (!saturdayAfternoon) {
		centro.setSaturdayAfternoonStartHour(null);
		centro.setSaturdayAfternoonEndHour(null);
	}
	else if (centro.getSaturdayAfternoonEndHour().compareTo(centro.getSaturdayAfternoonStartHour()) <= 0) {
		args[0] = rb.getString("schedule.daysWeek.Saturday");
		addErrorMessage("centros.schedule.error.StartHourEqualOrAfterEndHour", args);
		
		success = false;
	}
	
	if ((saturdayMorning && saturdayAfternoon) &&
		(centro.getSaturdayAfternoonStartHour().compareTo(centro.getSaturdayMorningEndHour()) <= 0)) {
		args[0] = rb.getString("schedule.daysWeek.Saturday");
		addErrorMessage("centros.schedule.error.MorningAfternoonOverlap", args);
			
		success = false;
	}
	
	// Sunday
	if (!sundayMorning) {
		centro.setSundayMorningStartHour(null);
		centro.setSundayMorningEndHour(null);
	}
	else if (centro.getSundayMorningEndHour().compareTo(centro.getSundayMorningStartHour()) <= 0) {
		args[0] = rb.getString("schedule.daysWeek.Sunday");
		addErrorMessage("centros.schedule.error.StartHourEqualOrAfterEndHour", args);
		
		success = false;
	}
	
	if (!sundayAfternoon) {
		centro.setSundayAfternoonStartHour(null);
		centro.setSundayAfternoonEndHour(null);
	}
	else if (centro.getSundayAfternoonEndHour().compareTo(centro.getSundayAfternoonStartHour()) <= 0) {
		args[0] = rb.getString("schedule.daysWeek.Sunday");
		addErrorMessage("centros.schedule.error.StartHourEqualOrAfterEndHour", args);
		
		success = false;
	}
	
	if ((sundayMorning && sundayAfternoon) &&
		(centro.getSundayAfternoonStartHour().compareTo(centro.getSundayMorningEndHour()) <= 0)) {
		args[0] = rb.getString("schedule.daysWeek.Sunday");
		addErrorMessage("centros.schedule.error.MorningAfternoonOverlap", args);
			
		success = false;
	}

	if (success)
		persist(centro);

    return success;
  }

  /**
   * Elimina centro.
   * 
   * @param centro
   *           centro
   */
  public void removeCentro(Centro centro) {
    if (centro.getId() != 0) {
      centro.setDesactivado(true);
      persist(centro);
    }
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.centros.service.CentroService#getCentrosDisponibles()
   */
  public List<Centro> getCentrosDisponibles() {
	  return findAllOrder(Centro.class,"nombre",true);
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.centros.service.CentroService#getProvincias()
   */
  public List<Provincia> getProvincias() {
    if (provincias == null) provincias = findAllOrder(Provincia.class, "nombre", true);
    return provincias;
  }

  /**
   * Devuelve centro.
   * 
   * @return centro
   */
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Devuelve fecha inicio.
   * 
   * @return fecha inicio
   */
  public Date getFechaInicio() {
    return fechaInicio;
  }

  /**
   * Asigna fecha inicio.
   * 
   * @param fechaInicio
   *          fecha inicio
   */
  public void setFechaInicio(Date fechaInicio) {
    this.fechaInicio = fechaInicio;
  }

  /**
   * Devuelve fecha fin.
   * 
   * @return fecha fin
   */
  public Date getFechaFin() {
    return fechaFin;
  }

  /**
   * Asigna fecha fin.
   * 
   * @param fechaFin
   *          fecha fin
   */
  public void setFechaFin(Date fechaFin) {
    this.fechaFin = fechaFin;
  }

  /**
   * Devuelve numero usuarios matriculados.
   * 
   * @return numero usuarios matriculados
   */
  public Long getNumeroUsuariosMatriculados() {
    return numeroUsuariosMatriculados;
  }

  /**
   * Asigna numero usuarios matriculados.
   * 
   * @param numeroUsuariosMatriculados
   *          numero usuarios matriculados
   */
  public void setNumeroUsuariosMatriculados(Long numeroUsuariosMatriculados) {
    this.numeroUsuariosMatriculados = numeroUsuariosMatriculados;
  }

  /**
   * Tiene datos.
   * 
   * @param fechaInicio
   *           fecha inicio
   * @param fechaFinal
   *           fecha final
   * @param center
   *           center
   * @return boolean
   */
  public Boolean tieneDatos(Date fechaInicio, Date fechaFinal, Centro center) {
	  Query q = getEntityManager().createQuery("SELECT COUNT(ucyl) as numero " +
			  								   "FROM " + UsuarioCyL.class.getName() + " ucyl " +
			  								   "WHERE (ucyl.fechaCreacion BETWEEN ? AND ?) AND " +
			  								   		 "(ucyl.id IN (SELECT u.id " +
			  								   		 			  "FROM " + Usuario.class.getName() + " u " +
	  															  "WHERE (U.centro = ?)))");
	  q.setParameter(1, fechaInicio);
	  q.setParameter(2, fechaFinal);
	  q.setParameter(3, center);
  
	  if ((Long)q.getSingleResult() > 0)
		  return true;
	  else
		  return false;
  }

  /**
   * Calcular estadisticas.
   */
  @SuppressWarnings("unchecked")
  public void calcularEstadisticas() {
	  System.out.println("calculando estadisticas");
	  
	  Long nUsuariosMatriculados = 0L;
	  
	  if ((getFechaInicio() != null) && (getFechaFin() != null)) {
		  Calendar c = new GregorianCalendar();
		  c.setTime(getFechaFin());
		  c.add(Calendar.HOUR, 23);
		  c.add(Calendar.MINUTE, 59);
		  setFechaFin(c.getTime());
		  
		  Query q;
		  if (centro != null) {
			  q = getEntityManager().createQuery("SELECT COUNT(ucyl) " +
					  							 "FROM " + UsuarioCyL.class.getName() + " ucyl " +
			  									 "WHERE (ucyl.fechaCreacion BETWEEN ? AND ?) AND " +
			  									 	   "(ucyl.centro.id = ?)");
			  q.setParameter(1, getFechaInicio());
			  q.setParameter(2, getFechaFin());
			  q.setParameter(3, centro.getId());
		  }
		  else {
			  q = getEntityManager().createQuery("SELECT COUNT(ucyl) " +
					  							 "FROM " + UsuarioCyL.class.getName() + " ucyl " +
			  									 "WHERE (ucyl.fechaCreacion BETWEEN ? AND ?) AND " +
			  										   "(ucyl.centro.desactivado = FALSE)");
			  q.setParameter(1, getFechaInicio());
			  q.setParameter(2, getFechaFin());
		  }
		  nUsuariosMatriculados = (Long)q.getSingleResult();
		  setNumeroUsuariosMatriculados(nUsuariosMatriculados);
	  }
	  else {
		  setNumeroUsuariosMatriculados(0L);
	  }
  }
  
  
  /**
   * Calcular estadisticas por fechas
   */
  @SuppressWarnings("unchecked")
  public void calcularEstadisticasFechas() {
	  System.out.println("calculando estadisticas");
    
	  Long nUsuariosMatriculados = 0L;
	  
	  Query q;
	  if (centro != null) {
		  q = getEntityManager().createQuery("SELECT COUNT(ucyl) " +
				  							 "FROM " + UsuarioCyL.class.getName() + " ucyl " +
		  									 "WHERE (ucyl.fechaCreacion BETWEEN ? AND ?) AND " +
		  									 	   "(ucyl.centro.id = ?)");
		  q.setParameter(3, centro.getId());
	  }
	  else {
		  q = getEntityManager().createQuery("SELECT COUNT(ucyl) " +
				  							 "FROM " + UsuarioCyL.class.getName() + " ucyl " +
		  									 "WHERE (ucyl.fechaCreacion BETWEEN ? AND ?) AND " +
		  									 	   "(ucyl.centro.desactivado = FALSE)");
	  }

       SimpleDateFormat formatoDeFecha = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss");
       try {
    	   if(getTiempoInformeUsuario()== ETiempoInformeUsuario.SEMANA){
	        	 q.setParameter(1, formatoDeFecha.parse(getFechaInicioSemana()));
	             q.setParameter(2, formatoDeFecha.parse(getFechaHoyLegible()));
	        }
	        else if (getTiempoInformeUsuario()== ETiempoInformeUsuario.MES){
	        	q.setParameter(1, formatoDeFecha.parse(getFechaInicioMesLegible()));
	        	q.setParameter(2, formatoDeFecha.parse(getFechaHoyLegible()));
	        }
	        else if (getTiempoInformeUsuario() == ETiempoInformeUsuario.MES_ANTERIOR){
	        	q.setParameter(1, formatoDeFecha.parse(getFechaInicioMesAntLegible()));
	            q.setParameter(2, formatoDeFecha.parse(getFechaFinMesAntLegible()));
	        }
	        else if (getTiempoInformeUsuario() == ETiempoInformeUsuario.ANYO){
	        	q.setParameter(1, formatoDeFecha.parse(getFechaInicioAnyoLegible()));
	        	q.setParameter(2, formatoDeFecha.parse(getFechaHoyLegible()));
	        }
	        else if (getTiempoInformeUsuario() == ETiempoInformeUsuario.ANYO_ANTERIOR){
	        	q.setParameter(1, formatoDeFecha.parse(getFechaInicioAnyoPasadoLegible()));
	            q.setParameter(2, formatoDeFecha.parse(getFechaFinAnyoPasadoLegible()));
	        }
       }
       catch (ParseException ex) {
    	   ex.printStackTrace();
       }
       
       nUsuariosMatriculados = (Long)q.getSingleResult();
	   setNumeroUsuariosMatriculados(nUsuariosMatriculados);
  }

  /**
   * Devuelve fecha inicio legible.
   * 
   * @return fecha inicio legible
   */
  public String getFechaInicioLegible() {
    String aux;
    if (fechaInicio != null) {
      DateFormat df = new SimpleDateFormat("dd/MM/yyyy");
      aux = df.format(fechaInicio) + " 00:00:00";
      return aux;
    }
    return "";
  }

  /**
   * Asigna fecha inicio legible.
   * 
   * @param fechaInicioLegible
   *          fecha inicio legible
   */
  public void setFechaInicioLegible(String fechaInicioLegible) {
    this.fechaInicioLegible = fechaInicioLegible;
  }

  /**
   * Asigna fecha fin legible.
   * 
   * @param fechaFinLegible
   *          fecha fin legible
   */
  public void setFechaFinLegible(String fechaFinLegible) {
    this.fechaFinLegible = fechaFinLegible;
  }

  /**
   * Devuelve fecha fin legible.
   * 
   * @return fecha fin legible
   */
  public String getFechaFinLegible() {
    String aux;
    if (fechaFin != null) {
      DateFormat df = new SimpleDateFormat("dd/MM/yyyy");
      aux = df.format(fechaFin) + " 23:59:59";
      return aux;
    }
    return "";
  }

  /**
   * Devuelve fecha hoy legible.
   * 
   * @return fecha hoy legible
   */
  public String getFechaHoyLegible() {
    String aux;
    Date hoy = new Date();
    DateFormat df = new SimpleDateFormat("dd/MM/yyyy");
    aux = df.format(hoy) + " 23:59:59";
    return aux;
  }

  /**
   * Devuelve fecha inicio incidencia.
   * 
   * @return fecha inicio incidencia
   */
  public String getFechaInicioIncidencia() {
    Date fecha = new Date();
    String aux;
    String fechaConcatenada = "01/01/2009";
    SimpleDateFormat formateador = new SimpleDateFormat("dd/MM/yyyy");
    try {
      fecha = formateador.parse(fechaConcatenada);
    } catch (ParseException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    aux = formateador.format(fecha);
    return aux;
  }

  /**
   * Devuelve fecha inicio mes legible.
   * 
   * @return fecha inicio mes legible
   */
  public String getFechaInicioMesLegible() {
    String aux;
    Date fecha = new Date();
    Calendar c = new GregorianCalendar();
    c.setTime(fecha);
    int ano = c.get(Calendar.YEAR);
    int mes = c.get(Calendar.MONTH);
    mes = mes + 1;
    String fechaConcatenada = "01/" + mes + "/" + ano;
    SimpleDateFormat formateador = new SimpleDateFormat("dd/MM/yyyy");
    try {
      fecha = formateador.parse(fechaConcatenada);
    } catch (ParseException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    aux = formateador.format(fecha) + " 00:00:00";
    return aux;
  }

  /**
   * Devuelve fecha inicio semana.
   * 
   * @return fecha inicio semana
   */
  public String getFechaInicioSemana() {
    Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    cal.set(Calendar.HOUR_OF_DAY, 0);
    cal.set(Calendar.MINUTE, 0);
    cal.set(Calendar.SECOND, 1);
    cal.set(Calendar.DATE, (cal.get(Calendar.DATE) - (((cal.get(Calendar.DAY_OF_WEEK) - 1 == 0) ? 7 : (cal
        .get(Calendar.DAY_OF_WEEK) - 1)) - 1)));
    Date fecha = cal.getTime();
    SimpleDateFormat formateador = new SimpleDateFormat("dd/MM/yyyy");
    String aux = formateador.format(fecha) + " 00:00:00";
    return aux;
  }

  /**
   * Devuelve fecha fin mes legible.
   * 
   * @return fecha fin mes legible
   */
  public String getFechaFinMesLegible() {
    String aux;
    Date fecha = new Date();
    Calendar c = new GregorianCalendar();
    c.setTime(fecha);
    int ano = c.get(Calendar.YEAR);
    int mes = c.get(Calendar.MONTH);
    mes++;
    int dia;
    if (mes == 1 || mes == 3 || mes == 5 || mes == 7 || mes == 8 || mes == 10 || mes == 12) {
      dia = 31;
    } else if (mes == 2) {
      if ((ano % 4 == 0) && ((ano % 100 != 0) || (ano % 400 == 0))) {
        dia = 29;
      } else {
        dia = 28;
      }
    } else {
      dia = 30;
    }
    String fechaConcatenada = dia + "/" + mes + "/" + ano + " 23:59:59";
    SimpleDateFormat formateador = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss");
    try {
      fecha = formateador.parse(fechaConcatenada);
    } catch (ParseException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    aux = formateador.format(fecha);
    return aux;
  }

  /**
   * Devuelve fecha inicio año legible.
   * 
   * @return fecha inicio año legible
   */
  public String getFechaInicioAnyoLegible() {
    String aux;
    Date fecha = new Date();
    Calendar c = new GregorianCalendar();
    c.setTime(fecha);
    int ano = c.get(Calendar.YEAR);
    String fechaConcatenada = "01/01/" + ano;
    SimpleDateFormat formateador = new SimpleDateFormat("dd/MM/yyyy");
    try {
      fecha = formateador.parse(fechaConcatenada);
    } catch (ParseException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    aux = formateador.format(fecha) + " 00:00:00";
    return aux;
  }

  /**
   * Devuelve fecha fin año legible.
   * 
   * @return fecha fin año legible
   */
  public String getFechaFinAnyoLegible() {
    String aux;
    Date fecha = new Date();
    Calendar c = new GregorianCalendar();
    c.setTime(fecha);
    int ano = c.get(Calendar.YEAR);
    String fechaConcatenada = "31/12/" + ano + " 23:59:59";
    SimpleDateFormat formateador = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss");
    try {
      fecha = formateador.parse(fechaConcatenada);
    } catch (ParseException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    aux = formateador.format(fecha);
    return aux;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.centros.service.CentroService#inicializaCentro(es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap)
   */
  public void inicializaCentro(UsuarioLdap u) {
    this.centro = u.getUser().getCentro();
  }

  /**
   * Devuelve fecha inicio año pasado legible.
   * 
   * @return fecha inicio año pasado legible
   */
  public String getFechaInicioAnyoPasadoLegible() {
    String aux;
    Date fecha = new Date();
    Calendar c = new GregorianCalendar();
    c.setTime(fecha);
    int ano = c.get(Calendar.YEAR);
    ano--;
    String fechaConcatenada = "01/01/" + ano;
    SimpleDateFormat formateador = new SimpleDateFormat("dd/MM/yyyy");
    try {
      fecha = formateador.parse(fechaConcatenada);
    } catch (ParseException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    fechaInicio = fecha;
    aux = formateador.format(fecha) + " 00:00:00";
    return aux;
  }

  /**
   * Devuelve fecha fin año pasado legible.
   * 
   * @return fecha fin año pasado legible
   */
  public String getFechaFinAnyoPasadoLegible() {
    String aux;
    Date fecha = new Date();
    Calendar c = new GregorianCalendar();
    c.setTime(fecha);
    int ano = c.get(Calendar.YEAR);
    ano--;
    String fechaConcatenada = "31/12/" + ano + " 23:59:59";
    SimpleDateFormat formateador = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss");
    try {
      fecha = formateador.parse(fechaConcatenada);
    } catch (ParseException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    fechaFin = fecha;
    aux = formateador.format(fecha);
    return aux;
  }

  /**
   * CUADRO DE MANDOS *.
   * 
   * @return list
   */

  public List<HashMap<String, Object>> calcularCuadroAltas() {
    List<HashMap<String, Object>> lista = new ArrayList<HashMap<String, Object>>();
    HashMap<String, Object> altas = new HashMap<String, Object>();
    HashMap<String, Object> altasMedias = new HashMap<String, Object>();
    HashMap<String, Object> edadMedia = new HashMap<String, Object>();
    HashMap<String, Object> bajas = new HashMap<String, Object>();

    CuadroDeMandos cuadro = new CuadroDeMandos();
    BP bp = cuadro.devolverBP(Tipo.ALTAS, getEntityManager(), centro);
    altas.put("titulo", "Altas");
    altas.put("descripcion", "Número de altas que se han producido entre las fechas marcadas.");
    int aa = cuadro.altasSemana(centro, getEntityManager());
    altas.put("semana", aa);
    altas.put("BPSemana", bp.getBpSemanal());
    altas.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), aa, true));
    Integer color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    altas.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    int bb = cuadro.altasMes(centro, getEntityManager());
    altas.put("mes", bb);
    altas.put("BPMes", bp.getBpMensual());
    altas.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    Integer colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    altas.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    int cc = cuadro.altasAnyo(centro, getEntityManager());
    altas.put("acumulado", cc);
    altas.put("BPAcum", bp.getBpAnual());
    altas.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    Integer colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    altas.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(altas);

    bp = cuadro.devolverBP(Tipo.ALTAS_MEDIAS, getEntityManager(), centro);
    altasMedias.put("titulo", "Altas medias diarias");
    altasMedias.put("descripcion", "Número de altas medias diarias entre el periodo de fechas marcado.");
    String altasMediasDiariasSemana = (String.format("%.2f", cuadro
        .altasMediasDiariasSemana(getEntityManager(), centro)));
    aa = (int) cuadro.altasMediasDiariasSemana(getEntityManager(), centro);
    double altasDouble = cuadro.altasMediasDiariasSemana(getEntityManager(), centro);
    altasMedias.put("semana", altasMediasDiariasSemana);
    altasMedias.put("BPSemana", bp.getBpSemanal());
    altasMedias.put("porcSemana", cuadro.calcularPorcentajeDouble(bp.getBpSemanal(), altasDouble));
    color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    altasMedias.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    bb = (int) cuadro.altasMediasDiariasMes(centro, getEntityManager());
    String altasMediasDiariasMes = (String.format("%.2f", cuadro.altasMediasDiariasMes(centro, getEntityManager())));
    altasMedias.put("mes", altasMediasDiariasMes);
    altasMedias.put("BPMes", bp.getBpMensual());
    altasMedias.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    altasMedias.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    cc = (int) cuadro.altasMediasDiariasAnyo(centro, getEntityManager());
    String altasMediasDiariasAnyo = (String.format("%.2f", cuadro.altasMediasDiariasAnyo(centro, getEntityManager())));
    altasMedias.put("acumulado", altasMediasDiariasAnyo);
    altasMedias.put("BPAcum", bp.getBpAnual());
    altasMedias.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    altasMedias.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(altasMedias);

    bp = cuadro.devolverBP(Tipo.EDAD_MEDIA, getEntityManager(), centro);
    edadMedia.put("titulo", "Edad Media");
    edadMedia.put("descripcion", "Edad media de las personas que se han dado de alta entre las fechas marcadas.");
    String edadMediaSemana = (String.format("%.2f", cuadro.edadMediaSemana(centro, getEntityManager())));
    aa = (int) cuadro.edadMediaSemana(centro, getEntityManager());
    edadMedia.put("semana", edadMediaSemana);
    edadMedia.put("BPSemana", bp.getBpSemanal());
    edadMedia.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), aa, true));
    color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    edadMedia.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    String edadMediaMes = (String.format("%.2f", cuadro.edadMediaMes(centro, getEntityManager())));
    bb = (int) cuadro.edadMediaMes(centro, getEntityManager());
    edadMedia.put("mes", edadMediaMes);
    edadMedia.put("BPMes", bp.getBpMensual());
    edadMedia.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    edadMedia.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    String edadMediaAnyo = (String.format("%.2f", cuadro.edadMediaAnyo(centro, getEntityManager())));
    cc = (int) cuadro.edadMediaAnyo(centro, getEntityManager());
    edadMedia.put("acumulado", edadMediaAnyo);
    edadMedia.put("BPAcum", bp.getBpAnual());
    edadMedia.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    edadMedia.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(edadMedia);

    bp = cuadro.devolverBP(Tipo.BAJAS, getEntityManager(), centro);
    bajas.put("titulo", "Bajas");
    bajas.put("descripcion", "Número de personas que se han dado de baja entre las fechas marcadas.");
    bajas.put("semana", cuadro.getNumeroBajasSemana());
    bajas.put("BPSemana", bp.getBpSemanal());
    bajas.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), cuadro.getNumeroBajasSemana(), true));
    color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), cuadro.getNumeroBajasSemana(), true);
    bajas.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    bajas.put("mes", cuadro.getNumeroBajasMes());
    bajas.put("BPMes", bp.getBpMensual());
    bajas.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), cuadro.getNumeroBajasMes(), true));
    colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), cuadro.getNumeroBajasMes(), true);
    bajas.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    bajas.put("acumulado", cuadro.getNumeroBajasAnyo());
    bajas.put("BPAcum", bp.getBpAnual());
    bajas.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cuadro.getNumeroBajasAnyo(), true));
    colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cuadro.getNumeroBajasAnyo(), true);
    bajas.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(bajas);

    return lista;
  }

  /**
   * Calcular cuadro cursos.
   * 
   * @return list
   */
  public List<HashMap<String, Object>> calcularCuadroCursos() {
    List<HashMap<String, Object>> lista = new ArrayList<HashMap<String, Object>>();
    HashMap<String, Object> cursos = new HashMap<String, Object>();
    HashMap<String, Object> numAlumnos = new HashMap<String, Object>();
    HashMap<String, Object> numAsistentes = new HashMap<String, Object>();
    HashMap<String, Object> numNuevos = new HashMap<String, Object>();
    HashMap<String, Object> numRepetidores = new HashMap<String, Object>();
    HashMap<String, Object> numHoras = new HashMap<String, Object>();
    HashMap<String, Object> numEspera = new HashMap<String, Object>();
    HashMap<String, Object> evaluacionMedia = new HashMap<String, Object>();
    HashMap<String, Object> cursosAnulados = new HashMap<String, Object>();
    HashMap<String, Object> porcentajeAsistentes = new HashMap<String, Object>();
    HashMap<String, Object> porcRepiten = new HashMap<String, Object>();
    HashMap<String, Object> porcentajeOcupacion = new HashMap<String, Object>();

    CuadroDeMandos cuadro = new CuadroDeMandos();
    BP bp = cuadro.devolverBP(Tipo.NUMEROCURSOS, getEntityManager(), centro);
    cursos.put("titulo", "Nº de cursos");
    cursos.put("descripcion", "Número de cursos que han empezado entre las fechas marcadas.");
    int aa = cuadro.numeroCursos(cuadro.fechaIniSemanaPasada(), cuadro.fechaFinSemanaPasada(), centro,
        getEntityManager());
    float alumnosSemana = cuadro.getNumeroAlumnosMatriculados();
    int asistentesSemana = cuadro.getNumeroAlumnosAsistentes();
    float horasSemana = cuadro.getHorasFormacion();
    int esperaSemana = cuadro.getNumeroAlumnosEspera();
    int cerradosSemana = cuadro.getCursosCerrados();
    int repetidores = cuadro.getRepetidores();
    double notaMedia = cuadro.getEvaluacionMedia();
    double porcAsistentesSemana = cuadro.getPorcAsistentes();
    double porcOcupacionSemana = cuadro.getPorcOcupacion();
    cursos.put("semana", aa);
    cursos.put("BPSemana", bp.getBpSemanal());
    cursos.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), aa, true));
    Integer color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    cursos.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    SimpleDateFormat formatoDelTexto = new SimpleDateFormat("dd/MM/yyyy");
    SimpleDateFormat formateador = new SimpleDateFormat("dd/MM/yyyy HH:mm");
    Date fechaInicial = null;
    Date fechaFinal = null;
    try {
      fechaInicial = formatoDelTexto.parse(cuadro.getFechaInicioMesAntLegible());
      fechaFinal = formateador.parse(cuadro.getFechaFinMesAntLegible());
    } catch (ParseException ex) {
      ex.printStackTrace();
    }
    int bb = cuadro.numeroCursos(fechaInicial, fechaFinal, centro, getEntityManager());
    float alumnosMes = cuadro.getNumeroAlumnosMatriculados();
    int asistentesMes = cuadro.getNumeroAlumnosAsistentes();
    float horasMes = cuadro.getHorasFormacion();
    int esperaMes = cuadro.getNumeroAlumnosEspera();
    int cerradosMes = cuadro.getCursosCerrados();
    int repetidoresMes = cuadro.getRepetidores();
    double porcAsistentesMes = cuadro.getPorcAsistentes();
    double porcOcupacionMes = cuadro.getPorcOcupacion();
    double notaMediaMes = cuadro.getEvaluacionMedia();
    cursos.put("mes", bb);
    cursos.put("BPMes", bp.getBpMensual());
    cursos.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    Integer colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    cursos.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    try {
      fechaInicial = formatoDelTexto.parse(cuadro.getFechaInicioAnyoPasadoLegible());
      fechaFinal = formateador.parse(cuadro.getFechaFinAnyoPasadoLegible());
    } catch (ParseException ex) {
    	ex.printStackTrace();
    }
    int cc = cuadro.numeroCursos(fechaInicial, fechaFinal, centro, getEntityManager());
    float alumnosAnyo = cuadro.getNumeroAlumnosMatriculados();
    int asistentesAnyo = cuadro.getNumeroAlumnosAsistentes();
    float horasAnyo = cuadro.getHorasFormacion();
    int esperaAnyo = cuadro.getNumeroAlumnosEspera();
    int cerradosAnyo = cuadro.getCursosCerrados();
    int repetidoresAnyo = cuadro.getRepetidores();
    double porcAsistentesAnyo = cuadro.getPorcAsistentes();
    double porcOcupacionAnyo = cuadro.getPorcOcupacion();
    double notaMediaAnyo = cuadro.getEvaluacionMedia();
    cursos.put("acumulado", cc);
    cursos.put("BPAcum", bp.getBpAnual());
    cursos.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    Integer colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    cursos.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(cursos);

    bp = cuadro.devolverBP(Tipo.NUMEROALUMNOS_CURSOS, getEntityManager(), centro);
    numAlumnos.put("titulo", "Nº de alumnos matriculados");
    numAlumnos.put("descripcion", "Número de alumnos matriculados en cursos que empiezan entre las fechas marcadas.");
    aa = (int) alumnosSemana;
    numAlumnos.put("semana", aa);
    numAlumnos.put("BPSemana", bp.getBpSemanal());
    numAlumnos.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), aa, true));
    color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    numAlumnos.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    bb = (int) alumnosMes;
    numAlumnos.put("mes", bb);
    numAlumnos.put("BPMes", bp.getBpMensual());
    numAlumnos.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    numAlumnos.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    cc = (int) alumnosAnyo;
    numAlumnos.put("acumulado", cc);
    numAlumnos.put("BPAcum", bp.getBpAnual());
    numAlumnos.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    numAlumnos.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(numAlumnos);

    bp = cuadro.devolverBP(Tipo.NUMEROALUMNOS_ASISTENTES_CURSOS, getEntityManager(), centro);
    numAsistentes.put("titulo", "Nº de alumnos asistentes +80%");
    numAsistentes
        .put(
            "descripcion",
            "Número de alumnos matriculados en cursos que han empezado entre las fechas marcadas y que ya han terminado, con más del 80% de asistencia.");
    aa = asistentesSemana;
    numAsistentes.put("semana", aa);
    numAsistentes.put("BPSemana", bp.getBpSemanal());
    numAsistentes.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), aa, true));
    color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    numAsistentes.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    bb = asistentesMes;
    numAsistentes.put("mes", bb);
    numAsistentes.put("BPMes", bp.getBpMensual());
    numAsistentes.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    numAsistentes.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    cc = asistentesAnyo;
    numAsistentes.put("acumulado", cc);
    numAsistentes.put("BPAcum", bp.getBpAnual());
    numAsistentes.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    numAsistentes.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(numAsistentes);

    bp = cuadro.devolverBP(Tipo.ALUMNOS_NUEVOS_CURSO, getEntityManager(), centro);
    numNuevos.put("titulo", "Nº de alumnos nuevos");
    numNuevos.put("descripcion", "Número de alumnos matriculados en un curso por primera vez.");
    float nuevos = alumnosSemana - repetidores;
    aa = (int) nuevos;
    numNuevos.put("semana", aa);
    numNuevos.put("BPSemana", bp.getBpSemanal());
    numNuevos.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), aa, true));
    color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    numNuevos.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    nuevos = alumnosMes - repetidoresMes;
    bb = (int) nuevos;
    numNuevos.put("mes", bb);
    numNuevos.put("BPMes", bp.getBpMensual());
    numNuevos.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    numNuevos.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    nuevos = alumnosAnyo - repetidoresAnyo;
    cc = (int) nuevos;
    numNuevos.put("acumulado", cc);
    numNuevos.put("BPAcum", bp.getBpAnual());
    numNuevos.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    numNuevos.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(numNuevos);

    bp = cuadro.devolverBP(Tipo.ALUMNOS_REPETIDORES_CURSO, getEntityManager(), centro);
    numRepetidores.put("titulo", "Nº de alumnos repetidores");
    numRepetidores.put("descripcion", "Número de alumnos matriculados en un curso al que no van por primera vez.");
    aa = repetidores;
    numRepetidores.put("semana", aa);
    numRepetidores.put("BPSemana", bp.getBpSemanal());
    numRepetidores.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), aa, true));
    color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    numRepetidores.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    bb = repetidoresMes;
    numRepetidores.put("mes", bb);
    numRepetidores.put("BPMes", bp.getBpMensual());
    numRepetidores.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    numRepetidores.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    cc = repetidoresAnyo;
    numRepetidores.put("acumulado", cc);
    numRepetidores.put("BPAcum", bp.getBpAnual());
    numRepetidores.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    numRepetidores.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(numRepetidores);

    bp = cuadro.devolverBP(Tipo.HORAS_FORMACION_CURSOS, getEntityManager(), centro);
    numHoras.put("titulo", "Horas de formación");
    numHoras.put("descripcion", "Número de horas de cursos que han empezado entre las fechas marcadas.");
    aa = (int) horasSemana;
    numHoras.put("semana", aa);
    numHoras.put("BPSemana", bp.getBpSemanal());
    numHoras.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), aa, true));
    color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    numHoras.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    bb = (int) horasMes;
    numHoras.put("mes", bb);
    numHoras.put("BPMes", bp.getBpMensual());
    numHoras.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    numHoras.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    cc = (int) horasAnyo;
    numHoras.put("acumulado", cc);
    numHoras.put("BPAcum", bp.getBpAnual());
    numHoras.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    numHoras.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(numHoras);

    bp = cuadro.devolverBP(Tipo.PORC_ASISTENTES_CURSOS, getEntityManager(), centro);
    porcentajeAsistentes.put("titulo", "Porc. asistentes");
    porcentajeAsistentes
        .put(
            "descripcion",
            "Porcentaje calculado sacando el número de matrículas con más del 80% de asistencia entre el número total de matriculas de cursos abiertos entre las fechas marcadas.");
    aa = (int) porcAsistentesSemana;
    String porcAsistentesString = (String.format("%.2f", porcAsistentesSemana)) + "%";
    porcentajeAsistentes.put("semana", porcAsistentesString);
    porcentajeAsistentes.put("BPSemana", bp.getBpSemanal());
    porcentajeAsistentes.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), aa, true));
    color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    porcentajeAsistentes.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    bb = (int) porcAsistentesMes;
    String porcAsistMesString = (String.format("%.2f", porcAsistentesMes)) + "%";
    porcentajeAsistentes.put("mes", porcAsistMesString);
    porcentajeAsistentes.put("BPMes", bp.getBpMensual());
    porcentajeAsistentes.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    porcentajeAsistentes.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    cc = (int) porcAsistentesAnyo;
    String porcAsistAnyoString = (String.format("%.2f", porcAsistentesAnyo)) + "%";
    porcentajeAsistentes.put("acumulado", porcAsistAnyoString);
    porcentajeAsistentes.put("BPAcum", bp.getBpAnual());
    porcentajeAsistentes.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    porcentajeAsistentes.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(porcentajeAsistentes);

    bp = cuadro.devolverBP(Tipo.PORC_OCUPACION_CURSOS, getEntityManager(), centro);
    porcentajeOcupacion.put("titulo", "Porc. ocupación");
    porcentajeOcupacion
        .put(
            "descripcion",
            "Porcentaje calculado sacando el número de matrículas entre el número total de plazas de cursos que han empezado entre las fechas marcadas.");
    aa = (int) porcOcupacionSemana;
    String porcOcupacionString = (String.format("%.2f", porcOcupacionSemana)) + "%";
    porcentajeOcupacion.put("semana", porcOcupacionString);
    porcentajeOcupacion.put("BPSemana", bp.getBpSemanal());
    porcentajeOcupacion.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), aa, true));
    color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    porcentajeOcupacion.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    bb = (int) porcOcupacionMes;
    String porcOcupMesString = (String.format("%.2f", porcOcupacionMes)) + "%";
    porcentajeOcupacion.put("mes", porcOcupMesString);
    porcentajeOcupacion.put("BPMes", bp.getBpMensual());
    porcentajeOcupacion.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    porcentajeOcupacion.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    cc = (int) porcOcupacionAnyo;
    String porcOcupAnyoString = (String.format("%.2f", porcOcupacionAnyo)) + "%";
    porcentajeOcupacion.put("acumulado", porcOcupAnyoString);
    porcentajeOcupacion.put("BPAcum", bp.getBpAnual());
    porcentajeOcupacion.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    porcentajeOcupacion.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(porcentajeOcupacion);

    bp = cuadro.devolverBP(Tipo.EVALUACION_MEDIA_CURSOS, getEntityManager(), centro);
    evaluacionMedia.put("titulo", "Evaluación media");
    evaluacionMedia.put("descripcion",
        "Nota media de los alumnos que han cursado cursos en el periodo de tiempo indicado.");
    aa = (int) notaMedia;
    String notaMediaSemana = (String.format("%.2f", notaMedia)) + "%";
    evaluacionMedia.put("semana", notaMediaSemana);
    evaluacionMedia.put("BPSemana", bp.getBpSemanal());
    evaluacionMedia.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), aa, true));
    color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    evaluacionMedia.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    bb = (int) notaMediaMes;
    String notaMediaM = (String.format("%.2f", notaMediaMes)) + "%";
    evaluacionMedia.put("mes", notaMediaM);
    evaluacionMedia.put("BPMes", bp.getBpMensual());
    evaluacionMedia.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    evaluacionMedia.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    cc = (int) notaMediaAnyo;
    String notaMediaA = (String.format("%.2f", notaMediaAnyo)) + "%";
    evaluacionMedia.put("acumulado", notaMediaA);
    evaluacionMedia.put("BPAcum", bp.getBpAnual());
    evaluacionMedia.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    evaluacionMedia.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(evaluacionMedia);

    bp = cuadro.devolverBP(Tipo.NUMEROALUMNOS_LISTA_ESPERA_CURSOS, getEntityManager(), centro);
    numEspera.put("titulo", "Personas lista de espera");
    numEspera.put("descripcion",
        "Número de alumnos en listas de espera de cursos que han empezado entre las fechas marcadas.");
    aa = esperaSemana;
    numEspera.put("semana", aa);
    numEspera.put("BPSemana", bp.getBpSemanal());
    numEspera.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), aa, true));
    color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    numEspera.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    bb = esperaMes;
    numEspera.put("mes", bb);
    numEspera.put("BPMes", bp.getBpMensual());
    numEspera.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    numEspera.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    cc = esperaAnyo;
    numEspera.put("acumulado", cc);
    numEspera.put("BPAcum", bp.getBpAnual());
    numEspera.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    numEspera.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(numEspera);

    bp = cuadro.devolverBP(Tipo.PORCENTAJE_ALUMNOS_REPITEN_CURSO, getEntityManager(), centro);
    porcRepiten.put("titulo", "%Alum.repiten curso");
    porcRepiten.put("descripcion", "Porcentaje de alumnos que repiten curso.");
    aa = cuadro.calcularPorcentajeNumero((int) alumnosSemana, repetidores, false);
    porcRepiten.put("semana", cuadro.calcularPorcentaje((int) alumnosSemana, repetidores, false));
    porcRepiten.put("BPSemana", bp.getBpSemanal());
    porcRepiten.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), aa, true));
    color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    porcRepiten.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    bb = cuadro.calcularPorcentajeNumero((int) alumnosMes, repetidoresMes, false);
    porcRepiten.put("mes", cuadro.calcularPorcentaje((int) alumnosMes, repetidoresMes, false));
    porcRepiten.put("BPMes", bp.getBpMensual());
    porcRepiten.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    porcRepiten.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    cc = cuadro.calcularPorcentajeNumero((int) alumnosAnyo, repetidoresAnyo, false);
    porcRepiten.put("acumulado", cuadro.calcularPorcentaje((int) alumnosAnyo, repetidoresAnyo, false));
    porcRepiten.put("BPAcum", bp.getBpAnual());
    porcRepiten.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    porcRepiten.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(porcRepiten);

    bp = cuadro.devolverBP(Tipo.CURSOS_CERRADOS, getEntityManager(), centro);
    cursosAnulados.put("titulo", "Cursos anulados");
    cursosAnulados.put("descripcion",
        "Número de cursos que han sido eliminados y tenían fecha de comienzo entre las fechas marcadas.");
    aa = cerradosSemana;
    cursosAnulados.put("semana", aa);
    cursosAnulados.put("BPSemana", bp.getBpSemanal());
    cursosAnulados.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), aa, true));
    color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    cursosAnulados.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    bb = cerradosMes;
    cursosAnulados.put("mes", bb);
    cursosAnulados.put("BPMes", bp.getBpMensual());
    cursosAnulados.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    cursosAnulados.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    cc = cerradosAnyo;
    cursosAnulados.put("acumulado", cc);
    cursosAnulados.put("BPAcum", bp.getBpAnual());
    cursosAnulados.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    cursosAnulados.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(cursosAnulados);

    return lista;
  }

  /**
   * Calcular cuadro talleres.
   * 
   * @return list
   */
  public List<HashMap<String, Object>> calcularCuadroTalleres() {
    List<HashMap<String, Object>> lista = new ArrayList<HashMap<String, Object>>();
    HashMap<String, Object> talleres = new HashMap<String, Object>();
    HashMap<String, Object> numAlumnos = new HashMap<String, Object>();
    HashMap<String, Object> numAsistentes = new HashMap<String, Object>();
    HashMap<String, Object> numHoras = new HashMap<String, Object>();
    HashMap<String, Object> numNuevos = new HashMap<String, Object>();
    HashMap<String, Object> numRepetidores = new HashMap<String, Object>();
    HashMap<String, Object> numEspera = new HashMap<String, Object>();
    HashMap<String, Object> porcRepiten = new HashMap<String, Object>();
    HashMap<String, Object> talleresAnulados = new HashMap<String, Object>();
    HashMap<String, Object> porcentajeAsistentes = new HashMap<String, Object>();
    HashMap<String, Object> porcentajeOcupacion = new HashMap<String, Object>();

    CuadroDeMandos cuadro = new CuadroDeMandos();
    BP bp = cuadro.devolverBP(Tipo.NUMEROTALLERES, getEntityManager(), centro);
    talleres.put("titulo", "Nº de talleres");
    int aa = cuadro.numeroTalleres(cuadro.fechaIniSemanaPasada(), cuadro.fechaFinSemanaPasada(), centro,
        getEntityManager());
    float alumnosSemana = cuadro.getNumeroAlumnosMatriculados();
    int asistentesSemana = cuadro.getNumeroAlumnosAsistentes();
    float horasSemana = cuadro.getHorasFormacion();
    int esperaSemana = cuadro.getNumeroAlumnosEspera();
    int cerradosSemana = cuadro.getTalleresCerrados();
    int repetidores = cuadro.getRepetidores();
    double porcAsistentesSemana = cuadro.getPorcAsistentes();
    double porcOcupacionSemana = cuadro.getPorcOcupacion();
    talleres.put("semana", aa);
    talleres.put("descripcion", "Número de talleres que han empezado entre las fechas marcadas.");
    talleres.put("BPSemana", bp.getBpSemanal());
    talleres.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), aa, true));
    Integer color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    talleres.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    SimpleDateFormat formatoDelTexto = new SimpleDateFormat("dd/MM/yyyy");
    SimpleDateFormat formateador = new SimpleDateFormat("dd/MM/yyyy HH:mm");
    Date fechaInicial = null;
    Date fechaFinal = null;
    try {
      fechaInicial = formatoDelTexto.parse(cuadro.getFechaInicioMesAntLegible());
      fechaFinal = formateador.parse(cuadro.getFechaFinMesAntLegible());
    } catch (ParseException ex) {
      ex.printStackTrace();
    }
    int bb = cuadro.numeroTalleres(fechaInicial, fechaFinal, centro, getEntityManager());
    float alumnosMes = cuadro.getNumeroAlumnosMatriculados();
    int asistentesMes = cuadro.getNumeroAlumnosAsistentes();
    float horasMes = cuadro.getHorasFormacion();
    int esperaMes = cuadro.getNumeroAlumnosEspera();
    int cerradosMes = cuadro.getTalleresCerrados();
    double porcAsistentesMes = cuadro.getPorcAsistentes();
    double porcOcupacionMes = cuadro.getPorcOcupacion();
    int repetidoresMes = cuadro.getRepetidores();
    talleres.put("mes", bb);
    talleres.put("BPMes", bp.getBpMensual());
    talleres.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    Integer colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    talleres.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    try {
      fechaInicial = formatoDelTexto.parse(cuadro.getFechaInicioAnyoPasadoLegible());
      fechaFinal = formateador.parse(cuadro.getFechaFinAnyoPasadoLegible());
    } catch (ParseException ex) {
      ex.printStackTrace();
    }
    int cc = cuadro.numeroTalleres(fechaInicial, fechaFinal, centro, getEntityManager());
    float alumnosAnyo = cuadro.getNumeroAlumnosMatriculados();
    int asistentesAnyo = cuadro.getNumeroAlumnosAsistentes();
    float horasAnyo = cuadro.getHorasFormacion();
    int esperaAnyo = cuadro.getNumeroAlumnosEspera();
    int cerradosAnyo = cuadro.getTalleresCerrados();
    double porcAsistentesAnyo = cuadro.getPorcAsistentes();
    double porcOcupacionAnyo = cuadro.getPorcOcupacion();
    int repetidoresAnyo = cuadro.getRepetidores();
    talleres.put("acumulado", cc);
    talleres.put("BPAcum", bp.getBpAnual());
    talleres.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    Integer colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    talleres.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(talleres);

    bp = cuadro.devolverBP(Tipo.NUMEROALUMNOS_TALLERES, getEntityManager(), centro);
    numAlumnos.put("titulo", "Nº de alumnos matriculados");
    numAlumnos.put("descripcion",
        "Número de alumnos matriculados en talleres que han empezado entre las fechas marcadas.");
    aa = (int) alumnosSemana;
    numAlumnos.put("semana", aa);
    numAlumnos.put("BPSemana", bp.getBpSemanal());
    numAlumnos.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), aa, true));
    color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    numAlumnos.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    bb = (int) alumnosMes;
    numAlumnos.put("mes", bb);
    numAlumnos.put("BPMes", bp.getBpMensual());
    numAlumnos.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    numAlumnos.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    cc = (int) alumnosAnyo;
    numAlumnos.put("acumulado", cc);
    numAlumnos.put("BPAcum", bp.getBpAnual());
    numAlumnos.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    numAlumnos.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(numAlumnos);

    bp = cuadro.devolverBP(Tipo.NUMEROALUMNOS_ASISTENTES_TALLERES, getEntityManager(), centro);
    numAsistentes.put("titulo", "Nº de alumnos asistentes +80%");
    numAsistentes
        .put(
            "descripcion",
            "Número de alumnos matriculados en talleres que han empezado entre las fechas marcadas y que ya han terminado, con más del 80% de asistencia.");
    aa = asistentesSemana;
    numAsistentes.put("semana", aa);
    numAsistentes.put("BPSemana", bp.getBpSemanal());
    numAsistentes.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), aa, true));
    color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    numAsistentes.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    bb = asistentesMes;
    numAsistentes.put("mes", bb);
    numAsistentes.put("BPMes", bp.getBpMensual());
    numAsistentes.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    numAsistentes.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    cc = asistentesAnyo;
    numAsistentes.put("acumulado", cc);
    numAsistentes.put("BPAcum", bp.getBpAnual());
    numAsistentes.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    numAsistentes.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(numAsistentes);

    bp = cuadro.devolverBP(Tipo.ALUMNOS_NUEVOS_TALLER, getEntityManager(), centro);
    numNuevos.put("titulo", "Nº de alumnos nuevos");
    numNuevos.put("descripcion", "Número de alumnos matriculados en un taller por primera vez.");
    float nuevos = alumnosSemana - repetidores;
    aa = (int) nuevos;
    numNuevos.put("semana", aa);
    numNuevos.put("BPSemana", bp.getBpSemanal());
    numNuevos.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), aa, true));
    color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    numNuevos.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    nuevos = alumnosMes - repetidoresMes;
    bb = (int) nuevos;
    numNuevos.put("mes", bb);
    numNuevos.put("BPMes", bp.getBpMensual());
    numNuevos.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    numNuevos.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    nuevos = alumnosAnyo - repetidoresAnyo;
    cc = (int) nuevos;
    numNuevos.put("acumulado", cc);
    numNuevos.put("BPAcum", bp.getBpAnual());
    numNuevos.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    numNuevos.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(numNuevos);

    bp = cuadro.devolverBP(Tipo.ALUMNOS_REPETIDORES_TALLER, getEntityManager(), centro);
    numRepetidores.put("titulo", "Nº de alumnos repetidores");
    numRepetidores.put("descripcion", "Número de alumnos matriculados en un taller al que no van por primera vez.");
    aa = repetidores;
    numRepetidores.put("semana", aa);
    numRepetidores.put("BPSemana", bp.getBpSemanal());
    numRepetidores.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), aa, true));
    color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    numRepetidores.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    bb = repetidoresMes;
    numRepetidores.put("mes", bb);
    numRepetidores.put("BPMes", bp.getBpMensual());
    numRepetidores.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    numRepetidores.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    cc = repetidoresAnyo;
    numRepetidores.put("acumulado", cc);
    numRepetidores.put("BPAcum", bp.getBpAnual());
    numRepetidores.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    numRepetidores.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(numRepetidores);

    bp = cuadro.devolverBP(Tipo.HORAS_FORMACION_TALLERES, getEntityManager(), centro);
    numHoras.put("titulo", "Horas de formación");
    numHoras.put("descripcion", "Número de horas de talleres que han empezado entre las fechas marcadas.");
    aa = (int) horasSemana;
    numHoras.put("semana", aa);
    numHoras.put("BPSemana", bp.getBpSemanal());
    numHoras.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), aa, true));
    color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    numHoras.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    bb = (int) horasMes;
    numHoras.put("mes", bb);
    numHoras.put("BPMes", bp.getBpMensual());
    numHoras.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    numHoras.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    cc = (int) horasAnyo;
    numHoras.put("acumulado", cc);
    numHoras.put("BPAcum", bp.getBpAnual());
    numHoras.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    numHoras.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(numHoras);

    bp = cuadro.devolverBP(Tipo.PORC_ASISTENTES_TALLERES, getEntityManager(), centro);
    porcentajeAsistentes.put("titulo", "Porc. asistentes");
    porcentajeAsistentes
        .put(
            "descripcion",
            "Porcentaje calculado sacando el número de matrículas con más del 80% de asistencia entre el número total de matriculas de talleres abiertos entre las fechas marcadas.");
    aa = (int) porcAsistentesSemana;
    String porcAsistentesString = (String.format("%.2f", porcAsistentesSemana)) + "%";
    porcentajeAsistentes.put("semana", porcAsistentesString);
    porcentajeAsistentes.put("BPSemana", bp.getBpSemanal());
    porcentajeAsistentes.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), aa, true));
    color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    porcentajeAsistentes.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    bb = (int) porcAsistentesMes;
    String porcAsistMesString = (String.format("%.2f", porcAsistentesMes)) + "%";
    porcentajeAsistentes.put("mes", porcAsistMesString);
    porcentajeAsistentes.put("BPMes", bp.getBpMensual());
    porcentajeAsistentes.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    porcentajeAsistentes.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    cc = (int) porcAsistentesAnyo;
    String porcAsistAnyoString = (String.format("%.2f", porcAsistentesAnyo)) + "%";
    porcentajeAsistentes.put("acumulado", porcAsistAnyoString);
    porcentajeAsistentes.put("BPAcum", bp.getBpAnual());
    porcentajeAsistentes.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    porcentajeAsistentes.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(porcentajeAsistentes);

    bp = cuadro.devolverBP(Tipo.PORC_OCUPACION_TALLERES, getEntityManager(), centro);
    porcentajeOcupacion.put("titulo", "Porc. ocupación");
    porcentajeOcupacion
        .put(
            "descripcion",
            "Porcentaje calculado sacando el número de matrículas entre el número total de plazas de talleres que han empezado entre las fechas marcadas.");
    aa = (int) porcOcupacionSemana;
    String porcOcupacionString = (String.format("%.2f", porcOcupacionSemana)) + "%";
    porcentajeOcupacion.put("semana", porcOcupacionString);
    porcentajeOcupacion.put("BPSemana", bp.getBpSemanal());
    porcentajeOcupacion.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), aa, true));
    color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    porcentajeOcupacion.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    bb = (int) porcOcupacionMes;
    String porcOcupMesString = (String.format("%.2f", porcOcupacionMes)) + "%";
    porcentajeOcupacion.put("mes", porcOcupMesString);
    porcentajeOcupacion.put("BPMes", bp.getBpMensual());
    porcentajeOcupacion.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    porcentajeOcupacion.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    cc = (int) porcOcupacionAnyo;
    String porcOcupAnyoString = (String.format("%.2f", porcOcupacionAnyo)) + "%";
    porcentajeOcupacion.put("acumulado", porcOcupAnyoString);
    porcentajeOcupacion.put("BPAcum", bp.getBpAnual());
    porcentajeOcupacion.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    porcentajeOcupacion.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(porcentajeOcupacion);

    bp = cuadro.devolverBP(Tipo.NUMEROALUMNOS_LISTA_ESPERA_TALLERES, getEntityManager(), centro);
    numEspera.put("descripcion",
        "Número de alumnos en listas de espera de talleres que han empezado entre las fechas marcadas.");
    numEspera.put("titulo", "Personas lista de espera");
    aa = esperaSemana;
    numEspera.put("semana", aa);
    numEspera.put("BPSemana", bp.getBpSemanal());
    numEspera.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), aa, true));
    color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    numEspera.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    bb = esperaMes;
    numEspera.put("mes", bb);
    numEspera.put("BPMes", bp.getBpMensual());
    numEspera.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    numEspera.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    cc = esperaAnyo;
    numEspera.put("acumulado", cc);
    numEspera.put("BPAcum", bp.getBpAnual());
    numEspera.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    numEspera.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(numEspera);

    bp = cuadro.devolverBP(Tipo.PORCENTAJE_ALUMNOS_REPITEN_TALLER, getEntityManager(), centro);
    porcRepiten.put("titulo", "%Alum.repiten taller");
    porcRepiten.put("descripcion", "Porcentaje de alumnos que repiten taller.");
    aa = cuadro.calcularPorcentajeNumero((int) alumnosSemana, repetidores, false);
    porcRepiten.put("semana", cuadro.calcularPorcentaje((int) alumnosSemana, repetidores, false));
    porcRepiten.put("BPSemana", bp.getBpSemanal());
    porcRepiten.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), aa, true));
    color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    porcRepiten.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    bb = cuadro.calcularPorcentajeNumero((int) alumnosMes, repetidoresMes, false);
    porcRepiten.put("mes", cuadro.calcularPorcentaje((int) alumnosMes, repetidoresMes, false));
    porcRepiten.put("BPMes", bp.getBpMensual());
    porcRepiten.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    porcRepiten.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    cc = cuadro.calcularPorcentajeNumero((int) alumnosAnyo, repetidoresAnyo, false);
    porcRepiten.put("acumulado", cuadro.calcularPorcentaje((int) alumnosAnyo, repetidoresAnyo, false));
    porcRepiten.put("BPAcum", bp.getBpAnual());
    porcRepiten.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    porcRepiten.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(porcRepiten);

    bp = cuadro.devolverBP(Tipo.TALLERES_CERRADOS, getEntityManager(), centro);
    talleresAnulados.put("titulo", "Cursos anulados");
    talleresAnulados.put("descripcion",
        "Número de talleres que han sido eliminados y tenían fecha de comienzo entre las fechas marcadas.");
    aa = cerradosSemana;
    talleresAnulados.put("semana", aa);
    talleresAnulados.put("BPSemana", bp.getBpSemanal());
    talleresAnulados.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), aa, true));
    color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    talleresAnulados.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    bb = cerradosMes;
    talleresAnulados.put("mes", bb);
    talleresAnulados.put("BPMes", bp.getBpMensual());
    talleresAnulados.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    talleresAnulados.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    cc = cerradosAnyo;
    talleresAnulados.put("acumulado", cc);
    talleresAnulados.put("BPAcum", bp.getBpAnual());
    talleresAnulados.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    talleresAnulados.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(talleresAnulados);

    return lista;
  }

  /**
   * Calcular cuadro terceros.
   * 
   * @return list
   */
  public List<HashMap<String, Object>> calcularCuadroTerceros() {
    List<HashMap<String, Object>> lista = new ArrayList<HashMap<String, Object>>();
    HashMap<String, Object> cursosTerceros = new HashMap<String, Object>();
    HashMap<String, Object> alumnosTerceros = new HashMap<String, Object>();
    HashMap<String, Object> horasTerceros = new HashMap<String, Object>();
    CuadroDeMandos cuadro = new CuadroDeMandos();
    BP bp = new BP();
    bp = cuadro.devolverBP(Tipo.NUMEROCURSOS_TERCEROS, getEntityManager(), centro);
    cursosTerceros.put("titulo", "Nº cursos");
    cursosTerceros.put("descripcion", "Número de cursos impartidos por terceros entre las fechas marcadas");
    int aa = cuadro.cursosTerceros(getEntityManager(), cuadro.fechaIniSemanaPasada(), cuadro.fechaFinSemanaPasada(),
        centro);
    Integer horasSemana = cuadro.getHorasCursosTerceros();
    Integer alumnosSemana = cuadro.getAlumnosCursosTerceros();
    cursosTerceros.put("semana", aa);
    cursosTerceros.put("BPSemana", bp.getBpSemanal());
    cursosTerceros.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), aa, true));
    Integer color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    cursosTerceros.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    getFechaInicioMesAntLegible();
    getFechaFinMesAntLegible();
    int bb = cuadro.cursosTerceros(getEntityManager(), fechaInicio, fechaFin, centro);
    Integer horasMes = cuadro.getHorasCursosTerceros();
    Integer alumnosMes = cuadro.getAlumnosCursosTerceros();
    cursosTerceros.put("mes", bb);
    cursosTerceros.put("BPMes", bp.getBpMensual());
    cursosTerceros.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    Integer colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    cursosTerceros.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    getFechaInicioAnyoPasadoLegible();
    getFechaFinAnyoPasadoLegible();
    int cc = cuadro.cursosTerceros(getEntityManager(), fechaInicio, fechaFin, centro);
    Integer horasAnyo = cuadro.getHorasCursosTerceros();
    Integer alumnosAnyo = cuadro.getAlumnosCursosTerceros();
    cursosTerceros.put("acumulado", cc);
    cursosTerceros.put("BPAcum", bp.getBpAnual());
    cursosTerceros.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    Integer colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    cursosTerceros.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(cursosTerceros);

    bp = cuadro.devolverBP(Tipo.NUMEROALUMNOS_TERCEROS, getEntityManager(), centro);
    alumnosTerceros.put("titulo", "Nº alumnos");
    alumnosTerceros.put("descripcion",
        "Número de alumnos matriculados en cursos de terceros que han empezado entre las fechas marcadas");
    aa = alumnosSemana;
    alumnosTerceros.put("semana", aa);
    alumnosTerceros.put("BPSemana", bp.getBpSemanal());
    alumnosTerceros.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), aa, true));
    color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    alumnosTerceros.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    getFechaInicioMesAntLegible();
    getFechaFinMesAntLegible();
    bb = alumnosMes;
    alumnosTerceros.put("mes", bb);
    alumnosTerceros.put("BPMes", bp.getBpMensual());
    alumnosTerceros.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    alumnosTerceros.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    getFechaInicioAnyoPasadoLegible();
    getFechaFinAnyoPasadoLegible();
    cc = alumnosAnyo;
    alumnosTerceros.put("acumulado", cc);
    alumnosTerceros.put("BPAcum", bp.getBpAnual());
    alumnosTerceros.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    alumnosTerceros.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(alumnosTerceros);

    bp = cuadro.devolverBP(Tipo.NUMEROALUMNOS_TERCEROS, getEntityManager(), centro);
    horasTerceros.put("titulo", "Nº horas");
    horasTerceros.put("descripcion",
        "Horas de cursos impartidos por terceros que han empezado entre las fechas marcadas.");
    aa = horasSemana;
    horasTerceros.put("semana", aa);
    horasTerceros.put("BPSemana", bp.getBpSemanal());
    horasTerceros.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), aa, true));
    color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    horasTerceros.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    getFechaInicioMesAntLegible();
    getFechaFinMesAntLegible();
    bb = horasMes;
    horasTerceros.put("mes", bb);
    horasTerceros.put("BPMes", bp.getBpMensual());
    horasTerceros.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    horasTerceros.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    getFechaInicioAnyoPasadoLegible();
    getFechaFinAnyoPasadoLegible();
    cc = horasAnyo;
    horasTerceros.put("acumulado", cc);
    horasTerceros.put("BPAcum", bp.getBpAnual());
    horasTerceros.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    horasTerceros.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(horasTerceros);

    return lista;
  }

  /**
   * Calcular cuadro otros.
   * 
   * @return list
   */
  public List<HashMap<String, Object>> calcularCuadroOtros() {
    List<HashMap<String, Object>> lista = new ArrayList<HashMap<String, Object>>();
    HashMap<String, Object> AlumnosAulaVirtual = new HashMap<String, Object>();
    HashMap<String, Object> AlumnosWifi = new HashMap<String, Object>();
    HashMap<String, Object> ConsultasAsesoramiento = new HashMap<String, Object>();

    CuadroDeMandos cuadro = new CuadroDeMandos();
    BP bp = new BP();
    bp = cuadro.devolverBP(Tipo.USUARIOS_AULA_VIRTUAL, getEntityManager(), centro);
    AlumnosAulaVirtual.put("titulo", "Usuarios Aula Virtual");
    AlumnosAulaVirtual.put("descripcion", "Número de alumnos que han usado el aula virtual entre las fechas marcadas.");
    int aa = cuadro.getUsuariosAulaVirtual(getEntityManager(), cuadro.fechaIniSemanaPasada(), cuadro
        .fechaFinSemanaPasada(), centro);
    AlumnosAulaVirtual.put("semana", aa);
    AlumnosAulaVirtual.put("BPSemana", bp.getBpSemanal());
    AlumnosAulaVirtual.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), aa, true));
    Integer color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    AlumnosAulaVirtual.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    getFechaInicioMesAntLegible();
    getFechaFinMesAntLegible();
    int bb = cuadro.getUsuariosAulaVirtual(getEntityManager(), fechaInicio, fechaFin, centro);
    AlumnosAulaVirtual.put("mes", bb);
    AlumnosAulaVirtual.put("BPMes", bp.getBpMensual());
    AlumnosAulaVirtual.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    Integer colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    AlumnosAulaVirtual.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    getFechaInicioAnyoPasadoLegible();
    getFechaFinAnyoPasadoLegible();
    int cc = cuadro.getUsuariosAulaVirtual(getEntityManager(), fechaInicio, fechaFin, centro);
    AlumnosAulaVirtual.put("acumulado", cc);
    AlumnosAulaVirtual.put("BPAcum", bp.getBpAnual());
    AlumnosAulaVirtual.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    Integer colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    AlumnosAulaVirtual.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(AlumnosAulaVirtual);

    bp = cuadro.devolverBP(Tipo.N_CONSULTAS_ASESORAMIENTO, getEntityManager(), centro);
    ConsultasAsesoramiento.put("titulo", "Nº consultas asesoram.");
    ConsultasAsesoramiento.put("descripcion", "Número de consultas realizadas en el centro entre las fechas marcadas.");
    aa = cuadro.getUsuariosAsesoramiento(getEntityManager(), cuadro.fechaIniSemanaPasada(), cuadro
        .fechaFinSemanaPasada(), centro);
    ConsultasAsesoramiento.put("semana", aa);
    ConsultasAsesoramiento.put("BPSemana", bp.getBpSemanal());
    ConsultasAsesoramiento.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), aa, true));
    color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    ConsultasAsesoramiento
        .put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    getFechaInicioMesAntLegible();
    getFechaFinMesAntLegible();
    bb = cuadro.getUsuariosAsesoramiento(getEntityManager(), fechaInicio, fechaFin, centro);
    ConsultasAsesoramiento.put("mes", bb);
    ConsultasAsesoramiento.put("BPMes", bp.getBpMensual());
    ConsultasAsesoramiento.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    ConsultasAsesoramiento
        .put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    getFechaInicioAnyoPasadoLegible();
    getFechaFinAnyoPasadoLegible();
    cc = cuadro.getUsuariosAsesoramiento(getEntityManager(), fechaInicio, fechaFin, centro);
    ConsultasAsesoramiento.put("acumulado", cc);
    ConsultasAsesoramiento.put("BPAcum", bp.getBpAnual());
    ConsultasAsesoramiento.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    ConsultasAsesoramiento
        .put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(ConsultasAsesoramiento);

    bp = cuadro.devolverBP(Tipo.USUARIOS_WIFI, getEntityManager(), centro);
    AlumnosWifi.put("titulo", "Usuarios Wifi");
    AlumnosWifi.put("descripcion", "Número de alumnos que han usado el wifi del centro entre las fechas marcadas.");
    aa = cuadro.getUsuariosWifi(getEntityManager(), cuadro.fechaIniSemanaPasada(), cuadro.fechaFinSemanaPasada(),
        centro);
    AlumnosWifi.put("semana", aa);
    AlumnosWifi.put("BPSemana", bp.getBpSemanal());
    AlumnosWifi.put("porcSemana", cuadro.calcularPorcentaje(bp.getBpSemanal(), aa, true));
    color = cuadro.calcularPorcentajeNumero(bp.getBpSemanal(), aa, true);
    AlumnosWifi.put("colorSemana", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), color, bp.isSigno()));

    getFechaInicioMesAntLegible();
    getFechaFinMesAntLegible();
    bb = cuadro.getUsuariosWifi(getEntityManager(), fechaInicio, fechaFin, centro);
    AlumnosWifi.put("mes", bb);
    AlumnosWifi.put("BPMes", bp.getBpMensual());
    AlumnosWifi.put("porcMes", cuadro.calcularPorcentaje(bp.getBpMensual(), bb, true));
    colorMes = cuadro.calcularPorcentajeNumero(bp.getBpMensual(), bb, true);
    AlumnosWifi.put("colorMes", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorMes, bp.isSigno()));

    getFechaInicioAnyoPasadoLegible();
    getFechaFinAnyoPasadoLegible();
    cc = cuadro.getUsuariosWifi(getEntityManager(), fechaInicio, fechaFin, centro);
    AlumnosWifi.put("acumulado", cc);
    AlumnosWifi.put("BPAcum", bp.getBpAnual());
    AlumnosWifi.put("porcAcum", cuadro.calcularPorcentaje(bp.getBpAnual(), cc, true));
    colorAnyo = cuadro.calcularPorcentajeNumero(bp.getBpAnual(), cc, true);
    AlumnosWifi.put("colorAnyo", cuadro.devolverColor(bp.getMinimo(), bp.getMaximo(), colorAnyo, bp.isSigno()));
    lista.add(AlumnosWifi);

    return lista;
  }

  /**
   * Devuelve fecha inicio mes ant legible.
   * 
   * @return fecha inicio mes ant legible
   */
  public String getFechaInicioMesAntLegible() {
    String aux;
    Date fecha = new Date();
    Calendar c = new GregorianCalendar();
    c.setTime(fecha);
    int ano = c.get(Calendar.YEAR);
    int mes = c.get(Calendar.MONTH);
    mes = mes + 1;
    if (mes == 1) {
      mes = 12;
      ano--;
    } else {
      mes--;
    }
    String fechaConcatenada = "01/" + mes + "/" + ano;
    SimpleDateFormat formateador = new SimpleDateFormat("dd/MM/yyyy");
    try {
      fecha = formateador.parse(fechaConcatenada);
    } catch (ParseException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    fechaInicio = fecha;
    aux = formateador.format(fecha) + " 00:00:00";
    return aux;
  }

  /**
   * Devuelve fecha fin mes ant legible.
   * 
   * @return fecha fin mes ant legible
   */
  public String getFechaFinMesAntLegible() {
    String aux;
    Date fecha = new Date();
    Calendar c = new GregorianCalendar();
    c.setTime(fecha);
    int ano = c.get(Calendar.YEAR);
    int mes = c.get(Calendar.MONTH);
    mes++;
    int dia;
    if (mes == 1) {
      mes = 12;
      ano--;
    } else {
      mes--;
    }

    if (mes == 1 || mes == 3 || mes == 5 || mes == 7 || mes == 8 || mes == 10 || mes == 12) {
      dia = 31;
    } else if (mes == 2) {
      if ((ano % 4 == 0) && ((ano % 100 != 0) || (ano % 400 == 0))) {
        dia = 29;
      } else {
        dia = 28;
      }
    } else {
      dia = 30;
    }
    String fechaConcatenada = dia + "/" + mes + "/" + ano + " 23:59:59";
    SimpleDateFormat formateador = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss");
    try {
      fecha = formateador.parse(fechaConcatenada);
    } catch (ParseException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    fechaFin = fecha;
    aux = formateador.format(fecha);
    return aux;
  }  
 
	/**
	 * @return the tiempoInformeUsuario
	 */
	public ETiempoInformeUsuario getTiempoInformeUsuario() {
		return tiempoInformeUsuario;
	}
	
	/**
	 * @param tiempoInformeUsuario the tiempoInformeUsuario to set
	 */
	public void setTiempoInformeUsuario(ETiempoInformeUsuario tiempoInformeUsuario) {
		this.tiempoInformeUsuario = tiempoInformeUsuario;
	}

	/**
	 * @return the mondayMorning
	 */
	public Boolean getMondayMorning() {
		return mondayMorning;
	}

	/**
	 * @param mondayMorning the mondayMorning to set
	 */
	public void setMondayMorning(Boolean mondayMorning) {
		this.mondayMorning = mondayMorning;
	}

	/**
	 * @return the mondayAfternoon
	 */
	public Boolean getMondayAfternoon() {
		return mondayAfternoon;
	}

	/**
	 * @param mondayAfternoon the mondayAfternoon to set
	 */
	public void setMondayAfternoon(Boolean mondayAfternoon) {
		this.mondayAfternoon = mondayAfternoon;
	}

	/**
	 * @return the tuesdayMorning
	 */
	public Boolean getTuesdayMorning() {
		return tuesdayMorning;
	}

	/**
	 * @param tuesdayMorning the tuesdayMorning to set
	 */
	public void setTuesdayMorning(Boolean tuesdayMorning) {
		this.tuesdayMorning = tuesdayMorning;
	}

	/**
	 * @return the tuesdayAfternoon
	 */
	public Boolean getTuesdayAfternoon() {
		return tuesdayAfternoon;
	}

	/**
	 * @param tuesdayAfternoon the tuesdayAfternoon to set
	 */
	public void setTuesdayAfternoon(Boolean tuesdayAfternoon) {
		this.tuesdayAfternoon = tuesdayAfternoon;
	}

	/**
	 * @return the wenesdayMorning
	 */
	public Boolean getWednesdayMorning() {
		return wednesdayMorning;
	}

	/**
	 * @param wenesdayMorning the wenesdayMorning to set
	 */
	public void setWednesdayMorning(Boolean wednesdayMorning) {
		this.wednesdayMorning = wednesdayMorning;
	}

	/**
	 * @return the wenesdayAfternoon
	 */
	public Boolean getWednesdayAfternoon() {
		return wednesdayAfternoon;
	}

	/**
	 * @param wenesdayAfternoon the wenesdayAfternoon to set
	 */
	public void setWednesdayAfternoon(Boolean wednesdayAfternoon) {
		this.wednesdayAfternoon = wednesdayAfternoon;
	}

	/**
	 * @return the thursdayMorning
	 */
	public Boolean getThursdayMorning() {
		return thursdayMorning;
	}

	/**
	 * @param thursdayMorning the thursdayMorning to set
	 */
	public void setThursdayMorning(Boolean thursdayMorning) {
		this.thursdayMorning = thursdayMorning;
	}

	/**
	 * @return the thursdayAfternoon
	 */
	public Boolean getThursdayAfternoon() {
		return thursdayAfternoon;
	}

	/**
	 * @param thursdayAfternoon the thursdayAfternoon to set
	 */
	public void setThursdayAfternoon(Boolean thursdayAfternoon) {
		this.thursdayAfternoon = thursdayAfternoon;
	}

	/**
	 * @return the fridayMorning
	 */
	public Boolean getFridayMorning() {
		return fridayMorning;
	}

	/**
	 * @param fridayMorning the fridayMorning to set
	 */
	public void setFridayMorning(Boolean fridayMorning) {
		this.fridayMorning = fridayMorning;
	}

	/**
	 * @return the fridayAfternoon
	 */
	public Boolean getFridayAfternoon() {
		return fridayAfternoon;
	}

	/**
	 * @param fridayAfternoon the fridayAfternoon to set
	 */
	public void setFridayAfternoon(Boolean fridayAfternoon) {
		this.fridayAfternoon = fridayAfternoon;
	}

	/**
	 * @return the saturdayMorning
	 */
	public Boolean getSaturdayMorning() {
		return saturdayMorning;
	}

	/**
	 * @param saturdayMorning the saturdayMorning to set
	 */
	public void setSaturdayMorning(Boolean saturdayMorning) {
		this.saturdayMorning = saturdayMorning;
	}

	/**
	 * @return the saturdayAfternoon
	 */
	public Boolean getSaturdayAfternoon() {
		return saturdayAfternoon;
	}

	/**
	 * @param saturdayAfternoon the saturdayAfternoon to set
	 */
	public void setSaturdayAfternoon(Boolean saturdayAfternoon) {
		this.saturdayAfternoon = saturdayAfternoon;
	}

	/**
	 * @return the sundayMorning
	 */
	public Boolean getSundayMorning() {
		return sundayMorning;
	}

	/**
	 * @param sundayMorning the sundayMorning to set
	 */
	public void setSundayMorning(Boolean sundayMorning) {
		this.sundayMorning = sundayMorning;
	}

	/**
	 * @return the sundayAfternoon
	 */
	public Boolean getSundayAfternoon() {
		return sundayAfternoon;
	}

	/**
	 * @param sundayAfternoon the sundayAfternoon to set
	 */
	public void setSundayAfternoon(Boolean sundayAfternoon) {
		this.sundayAfternoon = sundayAfternoon;
	}

	
	


	  public String getNombreLocalidad() {
		  return nombreLocalidad;
	  }
	  
	  public void setNombreLocalidad(String nombreLocalidad) {
		  this.nombreLocalidad = nombreLocalidad;
	  }
	  
	  public List<Localidad> getListLocalidades() {
		  return listLocalidades;
	  }
	  
	  public void searchLocalidad(Provincia provincia) {
		  if (nombreLocalidad.length() > 1) {
			  String strqry = "SELECT localidad " +
			  				  "FROM " + Localidad.class.getName() + " localidad " +
			  				  "WHERE localidad.provincia.id = " + provincia.getId() + " AND" +
			  				  "      localidad.nombre LIKE '%%" + nombreLocalidad + "%%' " +
			  				  "ORDER BY localidad.nombre ASC";
			  
			  listLocalidades = findByQuery(Localidad.class, strqry);
		  }
		  else {
			  if (listLocalidades != null) {
				  listLocalidades.clear();
				  listLocalidades = null;
			  }
		  }
	  }
	  
	  /**
	   * @return the idLocalidadSeleccionada
	   */
	  public Long getIdLocalidadSeleccionada() {
		  return idLocalidadSeleccionada;
	  }

	  /**
	   * @param idLocalidadSeleccionada the idLocalidadSeleccionada to set
	   */
	  public void setIdLocalidadSeleccionada(Long idLocalidadSeleccionada) {
		  this.idLocalidadSeleccionada = idLocalidadSeleccionada;
	  }

	  public void applyLocalidadToCentro(Centro centroToSetLocalidad) {
		  if (idLocalidadSeleccionada == -1)
			  centroToSetLocalidad.setLocalidad(null);
		  else {
			  Map<String, Object> properties = new HashMap<String, Object>();
			  properties.put("provincia", centroToSetLocalidad.getProvincia());
			  properties.put("id", idLocalidadSeleccionada);
			  
			  centroToSetLocalidad.setLocalidad(findOneWhere(Localidad.class, properties));
		  }
	  }
}
