package es.csa.fdt.cyldigital.core.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;

import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.MappedSuperclass;
import javax.persistence.Version;

import org.hibernate.Hibernate;
import org.hibernate.validator.Digits;

/**
 * Clase Abstract Entity.
 * @author csa
 *
 */
@MappedSuperclass
public abstract class AbstractEntity implements Serializable {

  private static final long serialVersionUID = -3156935382182399591L;

  private long id;

  private Long version;

  private boolean desactivado;

  /**
   * Construye un nuevo objeto de la clase
   */
  public AbstractEntity() {
    super();
  }

  /**
   * Devuelve id.
   * 
   * @return id
   */
  @Id
  @GeneratedValue
  @Digits(integerDigits = 20)
  public long getId() {
    return id;
  }

  /**
   * Asigna id.
   * 
   * @param id
   *          id
   */
  public void setId(long id) {
    this.id = id;
  }

  /**
   * Devuelve version.
   * 
   * @return version
   */
  @Version
  public Long getVersion() {
    return version;
  }

  /**
   * Asigna version.
   * 
   * @param version
   *          version
   */
  public void setVersion(Long version) {
    this.version = version;
  }

  /* (non-Javadoc)
   * @see java.lang.Object#hashCode()
   */
  @Override
  public int hashCode() {
    final int prime = 31;
    int result = 1;
    result = prime * result + (int) (id ^ (id >>> 32));
    return result;
  }

  /* (non-Javadoc)
   * @see java.lang.Object#equals(java.lang.Object)
   */
  @Override
  public boolean equals(Object obj) {
    if (this == obj) return true;
    if (obj == null) return false;
    if (!Hibernate.getClass(this).equals(Hibernate.getClass(obj))) return false;
    AbstractEntity other = (AbstractEntity) obj;
    if (getId() == 0L || other.getId() == 0L || getId() != other.getId()) return false;
    return true;
  }

  /**
   * Devuelve desactivado.
   * 
   * @return desactivado
   */
  public boolean getDesactivado() {
    return desactivado;
  }

  /**
   * Asigna desactivado.
   * 
   * @param desactivado
   *          desactivado
   */
  public void setDesactivado(boolean desactivado) {
    this.desactivado = desactivado;
  }

}
