package es.csa.fdt.cyldigital.core.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.sql.Blob;
import java.sql.SQLException;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EntityListeners;
import javax.persistence.Lob;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.sql.rowset.serial.SerialBlob;
import javax.sql.rowset.serial.SerialException;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Length;

import es.csa.commons.persistence.entity.FicheroListener;
import es.csa.commons.persistence.entity.FicheroSubido;
import es.csa.commons.persistence.entity.IFichero;
import es.csa.commons.util.FileUtils;

/**
 * Clase File.
 * 
 * @author csa
 */
@Entity
@Table(name = "files")
@AttributeOverride(name = "id", column = @Column(name = "idFichero"))
@EntityListeners(FicheroListener.class)
public class File extends AbstractEntity implements Serializable, IFichero, Cloneable {
  private static final long serialVersionUID = -3156935382182399591L;

  private String nombre;

  private String tipoMime;

  private Integer tamanio;

  private Blob contenido;
  // Encapsular el UploadItem
  private FicheroSubido ficheroSubido;

  /* (non-Javadoc)
   * @see es.csa.commons.persistence.entity.IFichero#getNombre()
   */
  @Length(max = 255)
  @Audited
  public String getNombre() {
    return isFicheroTemporal() ? getFicheroSubido().getNombre() : nombre;
  }

  /* (non-Javadoc)
   * @see es.csa.commons.persistence.entity.IFichero#setNombre(java.lang.String)
   */
  public void setNombre(String nombreFichero) {
    this.nombre = nombreFichero;
  }

  /* (non-Javadoc)
   * @see es.csa.commons.persistence.entity.IFichero#getTipoMime()
   */
  @Length(max = 255)
  @Audited
  public String getTipoMime() {
    return isFicheroTemporal() ? getFicheroSubido().getTipoMime() : tipoMime;
  }

  /* (non-Javadoc)
   * @see es.csa.commons.persistence.entity.IFichero#setTipoMime(java.lang.String)
   */
  public void setTipoMime(String tipoMime) {
    this.tipoMime = tipoMime;
  }

  /* (non-Javadoc)
   * @see es.csa.commons.persistence.entity.IFichero#getTamanio()
   */
  @Audited
  public Integer getTamanio() {
    return isFicheroTemporal() ? getFicheroSubido().getTamanio() : tamanio;
  }

  /* (non-Javadoc)
   * @see es.csa.commons.persistence.entity.IFichero#setTamanio(java.lang.Integer)
   */
  public void setTamanio(Integer tamanio) {
    this.tamanio = tamanio;
  }

  /* (non-Javadoc)
   * @see es.csa.commons.persistence.entity.IFichero#getTamanioLegible()
   */
  @Transient
  @Audited
  public String getTamanioLegible() {
    return FileUtils.getHumanReadableSize(Long.valueOf(getTamanio()));
  }

  /* (non-Javadoc)
   * @see es.csa.commons.persistence.entity.IFichero#getContenido()
   */
  @Lob
  @Column(columnDefinition = "longblob")
  public Blob getContenido() {
    try {
      return isFicheroTemporal() ? new SerialBlob(getFicheroSubido().getContenido()) : contenido;
    } catch (SerialException e) {
      e.printStackTrace();
      return null;
    } catch (SQLException e) {
      e.printStackTrace();
      return null;
    }
  }

  /* (non-Javadoc)
   * @see es.csa.commons.persistence.entity.IFichero#setContenido(java.sql.Blob)
   */
  public void setContenido(Blob contenido) {
    this.contenido = contenido;
  }

  /* (non-Javadoc)
   * @see es.csa.commons.persistence.entity.IFichero#isFicheroTemporal()
   */
  @Transient
  public boolean isFicheroTemporal() {
    return this.getId() == 0 && getFicheroSubido() != null;
  }

  /* (non-Javadoc)
   * @see es.csa.commons.persistence.entity.IFichero#getFicheroSubido()
   */
  @Transient
  public FicheroSubido getFicheroSubido() {
    return ficheroSubido;
  }

  /* (non-Javadoc)
   * @see es.csa.commons.persistence.entity.IFichero#setFicheroSubido(es.csa.commons.persistence.entity.FicheroSubido)
   */
  public void setFicheroSubido(FicheroSubido ficheroSubido) {
    this.ficheroSubido = ficheroSubido;
  }
}
