package es.csa.fdt.cyldigital.core.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.Serializable;
import java.sql.Blob;
import java.sql.SQLException;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EntityListeners;
import javax.persistence.Lob;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.sql.rowset.serial.SerialBlob;
import javax.sql.rowset.serial.SerialException;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotNull;
import org.richfaces.event.UploadEvent;

import es.csa.commons.faces.util.FicheroSubidoRichFacesAdapter;
import es.csa.commons.persistence.entity.FicheroListener;
import es.csa.commons.persistence.entity.FicheroSubido;
import es.csa.commons.persistence.entity.IFichero;
import es.csa.commons.util.FileUtils;

/**
 * Clase FileOnDisk.
 * 
 * @author csa
 */
@Entity
@Table(name = "filesondisk")
@AttributeOverride(name = "id", column = @Column(name = "idFichero"))
@EntityListeners(FicheroListener.class)
public class FileOnDisk extends AbstractEntity implements Serializable, IFichero, Cloneable {
  private static final long serialVersionUID = -3156935382182399591L;

  /**
   * Enum Tipo.
   */
  public enum Tipo {
    
    /** ALBARAN. */
    ALBARAN,
    
    /** INCIDENCIA. */
    INCIDENCIA,
    
    /** DOCUMENTO FACTURA. */
    DOCUMENTOFACTURA,
    
    /** FILE PANEL. */
    FILEPANEL,
    
    /** GUION. */
    GUION,
    
    /** DOCUMENTO MOVIMIENTO. */
    DOCUMENTOMOVIMIENTO,
    
    /** Diploma template */
    DIPLOMATEMPLATE;
  }

  private Tipo tipo;
  private String nombre;
  private String nombreOriginal;
  private String path;
  private String tipoMime;
  private String extension;
  private Integer tamanio;

  private Blob contenido; // transient
  // Encapsular el UploadItem
  private FicheroSubido ficheroSubido;

  /* (non-Javadoc)
   * @see es.csa.commons.persistence.entity.IFichero#getNombre()
   */
  @Length(max = 255)
  @Audited
  public String getNombre() {
    return nombre;
  }

  /* (non-Javadoc)
   * @see es.csa.commons.persistence.entity.IFichero#setNombre(java.lang.String)
   */
  public void setNombre(String nombreFichero) {
    this.nombre = nombreFichero;
  }

  /* (non-Javadoc)
   * @see es.csa.commons.persistence.entity.IFichero#getTipoMime()
   */
  @Length(max = 255)
  @Audited
  public String getTipoMime() {
    return tipoMime;
  }

  /* (non-Javadoc)
   * @see es.csa.commons.persistence.entity.IFichero#setTipoMime(java.lang.String)
   */
  public void setTipoMime(String tipoMime) {
    this.tipoMime = tipoMime;
  }

  /* (non-Javadoc)
   * @see es.csa.commons.persistence.entity.IFichero#getTamanio()
   */
  @Audited
  public Integer getTamanio() {
    return tamanio;
  }

  /* (non-Javadoc)
   * @see es.csa.commons.persistence.entity.IFichero#setTamanio(java.lang.Integer)
   */
  public void setTamanio(Integer tamanio) {
    this.tamanio = tamanio;
  }

  /* (non-Javadoc)
   * @see es.csa.commons.persistence.entity.IFichero#getTamanioLegible()
   */
  @Transient
  @Audited
  public String getTamanioLegible() {
    return FileUtils.getHumanReadableSize(Long.valueOf(getTamanio()));
  }

  /* (non-Javadoc)
   * @see es.csa.commons.persistence.entity.IFichero#getContenido()
   */
  @Lob
  @Column(columnDefinition = "longblob")
  @Transient
  public Blob getContenido() {
    try {
      return isFicheroTemporal() ? new SerialBlob(getFicheroSubido().getContenido()) : contenido;
    } catch (SerialException e) {
      e.printStackTrace();
      return null;
    } catch (SQLException e) {
      e.printStackTrace();
      return null;
    }
  }

  /* (non-Javadoc)
   * @see es.csa.commons.persistence.entity.IFichero#setContenido(java.sql.Blob)
   */
  public void setContenido(Blob contenido) {
    this.contenido = contenido;
  }

  /* (non-Javadoc)
   * @see es.csa.commons.persistence.entity.IFichero#isFicheroTemporal()
   */
  @Transient
  public boolean isFicheroTemporal() {
    return this.getId() == 0 && getFicheroSubido() != null;
  }

  /* (non-Javadoc)
   * @see es.csa.commons.persistence.entity.IFichero#getFicheroSubido()
   */
  @Transient
  public FicheroSubido getFicheroSubido() {
    return ficheroSubido;
  }

  /* (non-Javadoc)
   * @see es.csa.commons.persistence.entity.IFichero#setFicheroSubido(es.csa.commons.persistence.entity.FicheroSubido)
   */
  public void setFicheroSubido(FicheroSubido ficheroSubido) {
    this.ficheroSubido = ficheroSubido;
  }

  /**
   * Devuelve tipo.
   * 
   * @return tipo
   */
  @NotNull
  public Tipo getTipo() {
    return tipo;
  }

  /**
   * Asigna tipo.
   * 
   * @param tipo
   *          tipo
   */
  public void setTipo(Tipo tipo) {
    this.tipo = tipo;
  }

  /**
   * Devuelve extension.
   * 
   * @return extension
   */
  public String getExtension() {
    return extension;
  }

  /**
   * Asigna extension.
   * 
   * @param extension
   *          extension
   */
  public void setExtension(String extension) {
    this.extension = extension;
  }

  /**
   * Aniadir fichero.
   * 
   * @param event
   *           event
   */
  public synchronized void aniadirFichero(UploadEvent event) {
    setFicheroSubido(new FicheroSubidoRichFacesAdapter(event.getUploadItem()));
  }

  /*
   * Si existe algo en ficheroSubido, graba su contenido en disco con un numero incremental. En bd se graban todos los
   * datos originales.
   */
  /**
   * Grabar en disco.
   * 
   * @param ruta
   *           ruta
   */
  public void grabarEnDisco(String ruta) {
    Integer i = getNextFileId(ruta);

    // grabamos los datos del fichero.
    String extension = getFicheroSubido().getNombre().substring(getFicheroSubido().getNombre().indexOf('.') + 1);
    setExtension(extension);
    setTipoMime(getFicheroSubido().getTipoMime());
    setNombre(i.toString());
    setNombreOriginal(getFicheroSubido().getNombre());
    setTamanio(getFicheroSubido().getTamanio());
    setPath(ruta);

    // grabamos el fichero en disco.
    java.io.File file = new File(ruta + i.toString());
    try {
      OutputStream out = new FileOutputStream(file);
      try {
        out.write(getFicheroSubido().getContenido());
        out.close();

      } catch (IOException e) {
        System.out.println(e.getMessage());
      }
    } catch (FileNotFoundException e) {
      System.out.println(e.getMessage());
    }

  }

  /*
   * Método temporal para grabar en disco un fichero desde otro que ya existe.
   */
  /**
   * Grabar en disco desde disco.
   * 
   * @param ruta
   *           ruta
   * @param foriginal
   *           foriginal
   */
  public void grabarEnDiscoDesdeDisco(String ruta, es.csa.fdt.cyldigital.core.entity.File foriginal) {
    Integer i = getNextFileId(ruta);
    java.io.File f = new File(ruta + i.toString());

    setNombre(i.toString());
    setNombreOriginal(foriginal.getNombre());
    setTamanio(foriginal.getTamanio());
    setTipoMime(foriginal.getTipoMime());
    setPath(ruta);

    // grabamos el fichero en disco.
    try {
      OutputStream out = new FileOutputStream(f);
      try {
        try {
          out.write(foriginal.getContenido().getBytes(1, foriginal.getTamanio()));
        } catch (SQLException e) {
          System.out.println(e.getMessage());
        }
        out.close();

      } catch (IOException e) {
        System.out.println(e.getMessage());
      }
    } catch (FileNotFoundException e) {
      System.out.println(e.getMessage());
    }
  }

  private Integer getNextFileId(String ruta) {
    Integer i = 1;

    // buscamos el id que dar al fichero
    java.io.File f = new File(ruta + i.toString());
    while (f.exists()) {
      i++;
      f = new File(ruta + i.toString());
    }
    return i;
  }

  /*
   * public void borrarDeDisco(RecursoServiceImpl service) { Parameter p = service.findOneWhere(Parameter.class,
   * "nombre", "pathfileondisk");
   * 
   * File f = new File(p.getValor() + nombre); if(!f.delete()) {
   * System.out.println("Error borrando el fichero"+p.getValor()+nombre); } }
   */

  /**
   * Devuelve nombre original.
   * 
   * @return nombre original
   */
  public String getNombreOriginal() {
    return nombreOriginal;
  }

  /**
   * Asigna nombre original.
   * 
   * @param nombreOriginal
   *          nombre original
   */
  public void setNombreOriginal(String nombreOriginal) {
    this.nombreOriginal = nombreOriginal;
  }

  /**
   * Devuelve path.
   * 
   * @return path
   */
  public String getPath() {
    return path;
  }

  /**
   * Asigna path.
   * 
   * @param path
   *          path
   */
  public void setPath(String path) {
    this.path = path;
  }

  /**
   * Generar fichero vacio.
   * 
   * @param ruta
   *           ruta
   */
  public void generarFicheroVacio(String ruta) {
    Integer i = getNextFileId(ruta);
    java.io.File f = new File(ruta + i.toString());
    nombre = i.toString();
    try {
      OutputStream out = new FileOutputStream(f);
      try {

        out.write(1);
        out.close();

      } catch (IOException e) {
        System.out.println(e.getMessage());
      }
    } catch (FileNotFoundException e) {
      System.out.println(e.getMessage());
    }
  }
}
