package es.csa.fdt.cyldigital.core.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.sql.Blob;
import java.sql.SQLException;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.sql.rowset.serial.SerialBlob;
import javax.sql.rowset.serial.SerialException;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotNull;

import es.csa.commons.persistence.entity.FicheroSubido;
import es.csa.commons.util.FileUtils;

/**
 * Clase FilePanel.
 * 
 * @author csa
 */
@Entity
@Table(name = "filespaneles")
@AttributeOverride(name = "id", column = @Column(name = "idFilePanel"))
@Audited
public class FilePanel extends AbstractEntity implements Serializable {
  private static final long serialVersionUID = -3156935382182399591L;

  /**
   * Enum Tipo.
   */
  public enum Tipo {

    /** IMAGEN. */
    IMAGEN,

    /** VIDEO. */
    VIDEO,

    /** VIDEO especial. */
    VIDEO_ESPECIAL;
  }

  /** file. */
  public FileOnDisk file;

  private Tipo tipo;
  private String nombre;

  private String tipoMime;
  private String extension;

  private Integer tamanio;

  private Blob contenido; // transient
  // Encapsular el UploadItem
  private FicheroSubido ficheroSubido;

  /**
   * Construye un nuevo objeto de la clase
   */
  public FilePanel() {
    super();
  }

  /**
   * Devuelve file.
   * 
   * @return file
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idFile")
  public FileOnDisk getFile() {
    return file;
  }

  /**
   * Asigna file.
   * 
   * @param file
   *          file
   */
  public void setFile(FileOnDisk file) {
    this.file = file;
  }

  /**
   * Devuelve tipo.
   * 
   * @return tipo
   */
  @NotNull
  public Tipo getTipo() {
    return tipo;
  }

  /**
   * Asigna tipo.
   * 
   * @param tipo
   *          tipo
   */
  public void setTipo(Tipo tipo) {
    this.tipo = tipo;
  }

  /**
   * Devuelve nombre.
   * 
   * @return nombre
   */
  @Length(max = 255)
  @Audited
  public String getNombre() {
    return isFicheroTemporal() ? getFicheroSubido().getNombre() : nombre;
  }

  /**
   * Asigna nombre.
   * 
   * @param nombreFichero
   *          nombre
   */
  public void setNombre(String nombreFichero) {
    this.nombre = nombreFichero;
  }

  /**
   * Devuelve tipo mime.
   * 
   * @return tipo mime
   */
  @Length(max = 255)
  @Audited
  public String getTipoMime() {
    return isFicheroTemporal() ? getFicheroSubido().getTipoMime() : tipoMime;
  }

  /**
   * Asigna tipo mime.
   * 
   * @param tipoMime
   *          tipo mime
   */
  public void setTipoMime(String tipoMime) {
    this.tipoMime = tipoMime;
  }

  /**
   * Devuelve tamanio.
   * 
   * @return tamanio
   */
  @Audited
  public Integer getTamanio() {
    return isFicheroTemporal() ? getFicheroSubido().getTamanio() : tamanio;
  }

  /**
   * Asigna tamanio.
   * 
   * @param tamanio
   *          tamanio
   */
  public void setTamanio(Integer tamanio) {
    this.tamanio = tamanio;
  }

  /**
   * Devuelve tamanio legible.
   * 
   * @return tamanio legible
   */
  @Transient
  @Audited
  public String getTamanioLegible() {
    return FileUtils.getHumanReadableSize(Long.valueOf(getTamanio()));
  }

  /**
   * Devuelve contenido.
   * 
   * @return contenido
   */
  @Lob
  @Column(columnDefinition = "longblob")
  @Transient
  public Blob getContenido() {
    try {
      return isFicheroTemporal() ? new SerialBlob(getFicheroSubido().getContenido()) : contenido;
    } catch (SerialException e) {
      e.printStackTrace();
      return null;
    } catch (SQLException e) {
      e.printStackTrace();
      return null;
    }
  }

  /**
   * Asigna contenido.
   * 
   * @param contenido
   *          contenido
   */
  public void setContenido(Blob contenido) {
    this.contenido = contenido;
  }

  /**
   * Checks if is fichero temporal.
   * 
   * @return true, if is fichero temporal
   */
  @Transient
  public boolean isFicheroTemporal() {
    return this.getId() == 0 && getFicheroSubido() != null;
  }

  /**
   * Devuelve fichero subido.
   * 
   * @return fichero subido
   */
  @Transient
  public FicheroSubido getFicheroSubido() {
    return ficheroSubido;
  }

  /**
   * Asigna fichero subido.
   * 
   * @param ficheroSubido
   *          fichero subido
   */
  public void setFicheroSubido(FicheroSubido ficheroSubido) {
    this.ficheroSubido = ficheroSubido;
  }

  /**
   * Devuelve extension.
   * 
   * @return extension
   */
  public String getExtension() {
    return extension;
  }

  /**
   * Asigna extension.
   * 
   * @param extension
   *          extension
   */
  public void setExtension(String extension) {
    this.extension = extension;
  }

}
