package es.csa.fdt.cyldigital.core.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.envers.RevisionNumber;
import org.hibernate.envers.RevisionTimestamp;
import org.hibernate.validator.Length;

/**
 * Clase RevisionEntity.
 * 
 * @author csa
 */
@Entity
@Table(name = "revisions")
@org.hibernate.envers.RevisionEntity(es.csa.fdt.cyldigital.core.entity.RevisionEntity.RevisionListener.class)
public class RevisionEntity implements Serializable {

  private static final long serialVersionUID = 1L;

  private long id;

  private String usuario;

  private Date fecha;

  private String proceso;

  /**
   * Construye un nuevo objeto de la clase
   */
  public RevisionEntity() {
    super();
    setFecha(new Date());
  }

  /**
   * Devuelve id.
   * 
   * @return id
   */
  @Id
  @GeneratedValue
  @RevisionNumber
  @Column(name = "idRevision")
  public long getId() {
    return id;
  }

  /**
   * Asigna id.
   * 
   * @param id
   *          id
   */
  public void setId(long id) {
    this.id = id;
  }

  /**
   * Devuelve timestamp.
   * 
   * @return timestamp
   */
  @RevisionTimestamp
  @Transient
  public long getTimestamp() {
    return getFecha().getTime();
  }

  /**
   * Asigna timestamp.
   * 
   * @param timestamp
   *          timestamp
   */
  public void setTimestamp(long timestamp) {
    getFecha().setTime(timestamp);
  }

  /**
   * Devuelve fecha.
   * 
   * @return fecha
   */
  @Temporal(TemporalType.TIMESTAMP)
  public Date getFecha() {
    return fecha;
  }

  /**
   * Asigna fecha.
   * 
   * @param fecha
   *          fecha
   */
  public void setFecha(Date fecha) {
    this.fecha = fecha;
  }

  /**
   * Devuelve usuario.
   * 
   * @return usuario
   */
  @Length(max = 100)
  public String getUsuario() {
    return usuario;
  }

  /**
   * Asigna usuario.
   * 
   * @param usuario
   *          usuario
   */
  public void setUsuario(String usuario) {
    this.usuario = stringEndOf(usuario, 100);
  }

  /**
   * Devuelve proceso.
   * 
   * @return proceso
   */
  @Length(max = 255)
  public String getProceso() {
    return proceso;
  }

  /**
   * Asigna proceso.
   * 
   * @param proceso
   *          proceso
   */
  public void setProceso(String proceso) {
    this.proceso = stringEndOf(proceso, 255);
  }

  private static String stringEndOf(String string, int maxlength) {
    String substring = "";
    if (string != null) {
      int startIndex = Math.max(0, string.length() - maxlength);
      int endIndex = Math.min(startIndex + maxlength, string.length());
      substring = string.substring(startIndex, endIndex);
    }
    return substring;
  }

  /**
   * listener interface for receiving revision events. The class that is interested in processing a revision event
   * implements this interface, and the object created with that class is registered with a component using the
   * component's <code>addRevisionListener<code> method. When
   * revision event occurs, that object's appropriate
   * method is invoked.
   * 
   * @see RevisionEvent
   */
  public static class RevisionListener implements org.hibernate.envers.RevisionListener {

    /* (non-Javadoc)
     * @see org.hibernate.envers.RevisionListener#newRevision(java.lang.Object)
     */
    public void newRevision(Object revEntity) {
      RevisionEntity revision = (RevisionEntity) revEntity;
      revision.setUsuario(getUser());
      revision.setProceso(getProcess());
    }

    private static ThreadLocal<String> user = new ThreadLocal<String>();

    private static ThreadLocal<String> process = new ThreadLocal<String>();

    /**
     * Devuelve user.
     * 
     * @return user
     */
    public static String getUser() {
      return user.get();
    }

    /**
     * Asigna user.
     * 
     * @param user
     *          user
     */
    public static void setUser(String user) {
      RevisionListener.user.set(user);
    }

    /**
     * Devuelve process.
     * 
     * @return process
     */
    public static String getProcess() {
      return process.get();
    }

    /**
     * Asigna process.
     * 
     * @param process
     *          process
     */
    public static void setProcess(String process) {
      RevisionListener.process.set(process);
    }

  }

}
