package es.csa.fdt.cyldigital.core.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import javax.persistence.EntityManager;

/**
 * Interfaz del servicio de objetos Abstract.
 * 
 * @author csa
 * 
 * @param <E>
 *           element type
 * @param <ID>
 *           generic type
 */
public interface AbstractService<E, ID extends Serializable> {

  /**
   * Devuelve entity manager.
   * 
   * @return entity manager
   */
  public abstract EntityManager getEntityManager();

  /**
   * Persist.
   * 
   * @param <T>
   *           generic type
   * @param entidad
   *           entidad
   */
  public abstract <T> void persist(T entidad);

  /**
   * Persist all.
   * 
   * @param <T>
   *           generic type
   * @param entidades
   *           entidades
   */
  public abstract <T> void persistAll(Collection<T> entidades);

  /**
   * Persist all.
   * 
   * @param <T>
   *           generic type
   * @param entidades
   *           entidades
   */
  public abstract <T> void persistAll(T... entidades);

  /**
   * Elimina.
   * 
   * @param <T>
   *           generic type
   * @param entitdad
   *           entitdad
   */
  public abstract <T> void remove(T entitdad);

  /**
   * Elimina all.
   * 
   * @param <T>
   *           generic type
   * @param entidades
   *           entidades
   */
  public abstract <T> void removeAll(Collection<T> entidades);

  /**
   * Elimina all.
   * 
   * @param <T>
   *           generic type
   * @param tipoEntidad
   *           tipo entidad
   * @param entidades
   *           entidades
   */
  public abstract <T> void removeAll(Class<T> tipoEntidad, T... entidades);

  /**
   * Merge.
   * 
   * @param <T>
   *           generic type
   * @param entitdad
   *           entitdad
   * @return t
   */
  public abstract <T> T merge(T entitdad);

  /**
   * Merge all.
   * 
   * @param <T>
   *           generic type
   * @param entidades
   *           entidades
   * @return collection
   */
  public abstract <T> Collection<T> mergeAll(Collection<T> entidades);

  /**
   * Merge all.
   * 
   * @param <T>
   *           generic type
   * @param entidades
   *           entidades
   * @return collection
   */
  public abstract <T> Collection<T> mergeAll(T... entidades);

  /**
   * Contains.
   * 
   * @param <T>
   *           generic type
   * @param entidad
   *           entidad
   * @return true, if successful
   */
  public abstract <T> boolean contains(T entidad);

  /**
   * Refresh.
   * 
   * @param <T>
   *           generic type
   * @param entidad
   *           entidad
   */
  public abstract <T> void refresh(T entidad);

  /**
   * Clear.
   * 
   * @param <T>
   *           generic type
   */
  public abstract <T> void clear();

  /**
   * Execute update.
   * 
   * @param strQuery
   *           str query
   * @return int
   */
  public abstract int executeUpdate(String strQuery);

  /**
   * Execute update.
   * 
   * @param strQuery
   *           str query
   * @param params
   *           params
   * @return int
   */
  public abstract int executeUpdate(String strQuery, Map<String, ?> params);

  /**
   * Execute named update.
   * 
   * @param queryName
   *           query name
   * @return int
   */
  public abstract int executeNamedUpdate(String queryName);

  /**
   * Execute named update.
   * 
   * @param queryName
   *           query name
   * @param params
   *           params
   * @return int
   */
  public abstract int executeNamedUpdate(String queryName, Map<String, ?> params);

  /**
   * Find.
   * 
   * @param id
   *           id
   * @return e
   */
  public abstract E find(ID id);

  /**
   * Find.
   * 
   * @param <T>
   *           generic type
   * @param <K>
   *           key type
   * @param entityClass
   *           entity class
   * @param id
   *           id
   * @return t
   */
  public abstract <T, K extends Serializable> T find(Class<T> entityClass, K id);

  /**
   * Find all.
   * 
   * @return list
   */
  public abstract List<E> findAll();

  /**
   * Find all.
   * 
   * @param <T>
   *           generic type
   * @param entityClass
   *           entity class
   * @return list
   */
  public abstract <T> List<T> findAll(Class<T> entityClass);

  /**
   * Find all order.
   * 
   * @param orden
   *           orden
   * @param ascendente
   *           ascendente
   * @return list
   */
  public abstract List<E> findAllOrder(String orden, boolean ascendente);

  /**
   * Find all order.
   * 
   * @param <T>
   *           generic type
   * @param entityClass
   *           entity class
   * @param orden
   *           orden
   * @param ascendente
   *           ascendente
   * @return list
   */
  public abstract <T> List<T> findAllOrder(Class<T> entityClass, String orden, boolean ascendente);

  /**
   * Find one where.
   * 
   * @param propiedad
   *           propiedad
   * @param valor
   *           valor
   * @return e
   */
  public abstract E findOneWhere(String propiedad, Object valor);

  /**
   * Find one where.
   * 
   * @param <T>
   *           generic type
   * @param entityClass
   *           entity class
   * @param propiedad
   *           propiedad
   * @param valor
   *           valor
   * @return t
   */
  public abstract <T> T findOneWhere(Class<T> entityClass, String propiedad, Object valor);

  /**
   * Find one where.
   * 
   * @param propiedades
   *           propiedades
   * @return e
   */
  public abstract E findOneWhere(Map<String, Object> propiedades);

  /**
   * Find one where.
   * 
   * @param <T>
   *           generic type
   * @param entityClass
   *           entity class
   * @param propiedades
   *           propiedades
   * @return t
   */
  public abstract <T> T findOneWhere(Class<T> entityClass, Map<String, Object> propiedades);

  /**
   * Find where.
   * 
   * @param propiedad
   *           propiedad
   * @param valor
   *           valor
   * @return list
   */
  public abstract List<E> findWhere(String propiedad, Object valor);

  /**
   * Find where.
   * 
   * @param <T>
   *           generic type
   * @param entityClass
   *           entity class
   * @param propiedad
   *           propiedad
   * @param valor
   *           valor
   * @return list
   */
  public abstract <T> List<T> findWhere(Class<T> entityClass, String propiedad, Object valor);

  /**
   * Find where.
   * 
   * @param propiedades
   *           propiedades
   * @return list
   */
  public abstract List<E> findWhere(Map<String, Object> propiedades);

  /**
   * Find where.
   * 
   * @param <T>
   *           generic type
   * @param entityClass
   *           entity class
   * @param propiedades
   *           propiedades
   * @return list
   */
  public abstract <T> List<T> findWhere(Class<T> entityClass, Map<String, Object> propiedades);

  /**
   * Find where order.
   * 
   * @param propiedad
   *           propiedad
   * @param valor
   *           valor
   * @param orden
   *           orden
   * @param ascendente
   *           ascendente
   * @return list
   */
  public abstract List<E> findWhereOrder(String propiedad, Object valor, String orden, boolean ascendente);

  /**
   * Find where order.
   * 
   * @param <T>
   *           generic type
   * @param entityClass
   *           entity class
   * @param propiedad
   *           propiedad
   * @param valor
   *           valor
   * @param orden
   *           orden
   * @param ascendente
   *           ascendente
   * @return list
   */
  public abstract <T> List<T> findWhereOrder(Class<T> entityClass, String propiedad, Object valor, String orden,
      boolean ascendente);

  /**
   * Find where order.
   * 
   * @param propiedades
   *           propiedades
   * @param orden
   *           orden
   * @param ascendente
   *           ascendente
   * @return list
   */
  public abstract List<E> findWhereOrder(Map<String, Object> propiedades, String orden, boolean ascendente);

  /**
   * Find where order.
   * 
   * @param <T>
   *           generic type
   * @param entityClass
   *           entity class
   * @param propiedades
   *           propiedades
   * @param orden
   *           orden
   * @param ascendente
   *           ascendente
   * @return list
   */
  public abstract <T> List<T> findWhereOrder(Class<T> entityClass, Map<String, Object> propiedades, String orden,
      boolean ascendente);

  /**
   * Find by query.
   * 
   * @param strQuery
   *           str query
   * @return list
   */
  public abstract List<E> findByQuery(String strQuery);

  /**
   * Find by query.
   * 
   * @param <T>
   *           generic type
   * @param entityClass
   *           entity class
   * @param strQuery
   *           str query
   * @return list
   */
  public abstract <T> List<T> findByQuery(Class<T> entityClass, String strQuery);

  /**
   * Find by query.
   * 
   * @param strQuery
   *           str query
   * @param params
   *           params
   * @return list
   */
  public abstract List<E> findByQuery(String strQuery, Map<String, ? extends Object> params);

  /**
   * Find by query.
   * 
   * @param <T>
   *           generic type
   * @param entityClass
   *           entity class
   * @param strQuery
   *           str query
   * @param params
   *           params
   * @return list
   */
  public abstract <T> List<T> findByQuery(Class<T> entityClass, String strQuery, Map<String, ? extends Object> params);

  /**
   * Find by named query.
   * 
   * @param queryName
   *           query name
   * @return list
   */
  public abstract List<E> findByNamedQuery(String queryName);

  /**
   * Find by named query.
   * 
   * @param <T>
   *           generic type
   * @param entityClass
   *           entity class
   * @param queryName
   *           query name
   * @return list
   */
  public abstract <T> List<T> findByNamedQuery(Class<T> entityClass, String queryName);

  /**
   * Find by named query.
   * 
   * @param queryName
   *           query name
   * @param params
   *           params
   * @return list
   */
  public abstract List<E> findByNamedQuery(String queryName, Map<String, ? extends Object> params);

  /**
   * Find by named query.
   * 
   * @param <T>
   *           generic type
   * @param entityClass
   *           entity class
   * @param queryName
   *           query name
   * @param params
   *           params
   * @return list
   */
  public abstract <T> List<T> findByNamedQuery(Class<T> entityClass, String queryName,
      Map<String, ? extends Object> params);

}
