package es.csa.fdt.cyldigital.core.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.jpaql.BrowserJPAQL;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.commons.persistence.browser.jpaql.FilterJPAQLByQuery;

/**
 * Browser that allows filter records by their active status
 * 
 * @author csa
 * @author Balidea Consulting & Programming
 * 
 * @param <T> Entities type to be browsed
 */
public class BrowserFilteredByActive<T> extends BrowserJPAQL<T> {

  private static final long serialVersionUID = -1354990880582430517L;

  /**
   * Determines whether active/inactive records should be filtered. It takes three possible values:
   * <ul>
   * 	<li><code>null</code>: no filter</li>
   * 	<li><code>true</code>: show only inactive records</li>
   * 	<li><code>false</code>: show only active records</li>
   * </ul> 
   */
  protected Boolean showInactives;

  /**
   * Constructs a new BrowserFilteredByActive
   */
  public BrowserFilteredByActive() {
    super();
    showInactives = false;
    filtrarDesactivados();
  }

  /**
   * Constructs a new BrowserFilteredByActive
   * 
   * @param entityClass entity class
   */
  public BrowserFilteredByActive(Class<T> entityClass) {
    super(entityClass);
    showInactives = false;
    filtrarDesactivados();
  }

  /**
   * Clears all the established filters in the browser and sets the active/inactive filter with its current criteria 
   * <p>
   * @see es.csa.commons.persistence.browser.jpaql.BrowserJPAQL#clearFilters()
   */
  @Override
  public void clearFilters() {
    super.clearFilters();
    filtrarDesactivados();
  }

  /**
   * Prepares the active/inactive filter query depending on the value of the current criteria
   */
  public void filtrarDesactivados() {
	  
	  if (showInactives != null) { 
		  
		  String query = "";
		  boolean flag = false;
		  FilterJPAQLByQuery<T> filter = new FilterJPAQLByQuery<T>();
		  
		  if (showInactives) {
			  query += " ( desactivado = ? ) ";
			  filter.getCriterion().addParameter(true);
		  } else {
			  query += " ( desactivado is null or desactivado = ? ) ";
			  filter.getCriterion().addParameter(false);
		  }

		  for (Filter<CriterionJPAQL, T> f : getFilters())
			  if (f.getClass() == FilterJPAQLByQuery.class)
				  if (f.getCriterion().getQuery().contains("desactivado")) flag = true;

		  if (!flag) {
			  filter.getCriterion().setQuery(query);
			  getFilters().add(filter);
		  }
	  }
    
  }

  /**
   * Checks the criteria status that determines whether active/inactive records should be filtered.
   * @return One of these three possible values:
   *    <ul>
   * 		<li><code>null</code>: if no filter is established</li>
   * 		<li><code>true</code>: if filter should show only inactive records</li>
   * 		<li><code>false</code>: if filter should show only active records</li>
   * 	</ul> 
   */
  public Boolean isVerDesactivados() {
    return showInactives;
  }

  /**
   * Sets the filter criteria that determines whether active/inactive records should be filtered.
   * @param showInactives Determines whether active/inactive records should be filtered. It takes three possible values:
   * 	<ul>
   * 		<li><code>null</code>: no filter</li>
   * 		<li><code>true</code>: show only inactive records</li>
   * 		<li><code>false</code>: show only active records</li>
   * 	</ul> 
   */
  public void setVerDesactivados(Boolean showInactives) {
    this.showInactives = showInactives;
  }

  /**
   * Unsets the filter criteria that determines whether active/inactive records should be filtered.
   */
  public void unsetVerDesactivados() {
	this.showInactives = null;
  }
  
}
