package es.csa.fdt.cyldigital.core.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.List;

import javax.persistence.NoResultException;
import javax.persistence.Query;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.commons.persistence.browser.jpaql.FilterJPAQLByQuery;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario.Role;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;

/**
 * Servicio del Browser BrowserFilteredByCenter.
 * 
 * @author csa
 * 
 * @param <T>
 *           generic type
 */
public class BrowserFilteredByCenter<T> extends BrowserFilteredByActive<T> {

  private static final long serialVersionUID = -1354990880582430517L;
  
  private static final Logger logger = LoggerFactory.getLogger(BrowserFilteredByCenter.class);

  /** centro. */
  protected Centro centro;
  
  /** extra attribute */
  private String extraAttribute;

  /** propiedad. */
  protected String propiedad;

  /** user. */
  protected UsuarioLdap user;

  /** centros. */
  public List<Centro> centros;
  
  /**
   * Construye un nuevo objeto de la clase
   * 
   * @param attribute
   *           extra attribute name
   */
  public BrowserFilteredByCenter(String extraAttribute) {
    super();
    
    this.extraAttribute = extraAttribute + ".";
    
    setPropiedad(this.extraAttribute + "centro.id");
    centros = new ArrayList<Centro>();
    //filtrarCentrosDesactivados();
  }

  /**
   * Construye un nuevo objeto de la clase
   */
  public BrowserFilteredByCenter() {
    super();
    
    this.extraAttribute = "";
    
    setPropiedad("centro.id");
    centros = new ArrayList<Centro>();
    //filtrarCentrosDesactivados();
  }

  /**
   * Construye un nuevo objeto de la clase
   * 
   * @param entityClass
   *           entity class
   */
  public BrowserFilteredByCenter(Class<T> entityClass) {
    super(entityClass);
    
    this.extraAttribute = "";
    
    //filtrarCentrosDesactivados();
  }

  /**
   * Filtrar centros desactivados.
   */
  public void filtrarCentrosDesactivados() {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<T> filter = new FilterJPAQLByQuery<T>();    
    query += " " + this.extraAttribute + "centro.desactivado = false ";

    for (Filter<CriterionJPAQL, T> f : getFilters())
      if (f.getCriterion().getQuery().contains(this.extraAttribute + "centro.desactivado")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }

  /**
   * Devuelve propiedad.
   * 
   * @return propiedad
   */
  public String getPropiedad() {
    return propiedad;
  }

  /**
   * Asigna propiedad.
   * 
   * @param propiedad
   *          propiedad
   */
  public void setPropiedad(String propiedad) {
    this.propiedad = propiedad;
  }

  /**
   * Devuelve user.
   * 
   * @return user
   */
  public UsuarioLdap getUser() {
    return user;
  }

  /**
   * Asigna user.
   * 
   * @param user
   *          user
   */
  public void setUser(UsuarioLdap user) {
    this.user = user;
  }

  /**
   * Devuelve centro.
   * 
   * @return centro
   */
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Devuelve centros disponibles.
   * 
   * @return centros disponibles
   */
  @SuppressWarnings("unchecked")
  public List<Centro> getCentrosDisponibles() {
    if (centros.isEmpty()) {
	    try{
	    	String stqry = "";
	    	
	        Query prequerycolaborador = em.createQuery("SELECT e FROM " + Parameter.class.getName()
	      		  + " e WHERE nombre LIKE 'extern_default_center_id'");    	
	        Parameter idcentrocolaborador = (Parameter) prequerycolaborador.getSingleResult();

	        Query prequeryadmin = em.createQuery("SELECT e FROM " + Parameter.class.getName()
		      		  + " e WHERE nombre LIKE 'admin_default_center_id'");    	
        	Parameter idcentroadmin = (Parameter) prequeryadmin.getSingleResult();
	        
        	if( (user != null && !user.getRolesList().contains(Role.ROLE_ADMIN)) || (user == null) ) {
		        stqry += "AND id != :idCentroColaboradores AND id != :idCentroAdmin ";
        	}
	        
	        Query query = em.createQuery("SELECT e FROM " + Centro.class.getName()
		            + " e WHERE desactivado = false " + stqry + "ORDER BY nombre ASC");
	        
	        if( (user != null && !user.getRolesList().contains(Role.ROLE_ADMIN)) || (user == null) ) {
		        query.setParameter("idCentroColaboradores", Long.parseLong(idcentrocolaborador.getValor()));
		        query.setParameter("idCentroAdmin", Long.parseLong(idcentroadmin.getValor()));
	        }
	        
	        centros = (List<Centro>) query.getResultList();
	    }
	    catch (NoResultException nre) {

	    }
	    catch (Exception e) {
			logger.warn("BrowserFilteredByCenter.getCentrosDisponibles: Error al consultar los centros disponibles." + e.getLocalizedMessage());
			e.printStackTrace();
		}
    }
    return centros;
  }

  /**
   * Filter by centro.
   * 
   * @param idCentro
   *           id centro
   */
  public void filterByCentro(Long idCentro) {
    String query = "";
    //String queryDes = "";
    boolean flag = false;
    //boolean flagDes = false;
    FilterJPAQLByQuery<T> filter = new FilterJPAQLByQuery<T>();
    //FilterJPAQLByQuery<T> filterDes = new FilterJPAQLByQuery<T>();

    query += " " + propiedad + " = ? ";
    filter.getCriterion().addParameter(idCentro);

    for (Filter<CriterionJPAQL, T> f : getFilters())
      if (f.getCriterion().getQuery().contains(propiedad)) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }

    // Aplicar el filtro de centros desactivados
    /*queryDes += " " + this.extraAttribute + "centro.desactivado = false ";

    for (Filter<CriterionJPAQL, T> f : getFilters())
      if (f.getCriterion().getQuery().contains(this.extraAttribute + "centro.desactivado")) flagDes = true;

    if (!flagDes) {
      filterDes.getCriterion().setQuery(queryDes);
      getFilters().add(filterDes);
    }*/
  }

  /**
   * Elimina filter centro.
   */
  protected void removeFilterCentro() {
    int i = -1;
    for (Filter<CriterionJPAQL, T> f : getFilters())
      if (f.getCriterion().getQuery().contains(propiedad)) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
  }

  /**
   * Filter.
   */
  public void filter() {
    removeFilterCentro();
    if (getCentro() != null) filterByCentro(getCentro().getId());
  }

  /**
   * Asigna filter centro.
   * 
   * @param usuario
   *          filter centro
   */
  public void setFilterCentro(UsuarioLdap usuario) {
	  user = usuario;
	  boolean sinFiltrarPorDefecto = user.getRolesList().contains(Role.ROLE_ADMIN)/* ||
	  								 user.getRolesList().contains(Role.ROLE_TRAINER_COORDINATOR) ||
	  								 user.getRolesList().contains(Role.ROLE_MANAGER)*/;

	  if (centro == null) {
		  if (!sinFiltrarPorDefecto && user.getCentro() != null) {
			  removeFilterCentro();
			  
			  if (user.getCentro() != null) {
				  filterByCentro(user.getCentro().getId());
				  centro = user.getCentro();// serviceCentros.find(Centro.class, user.getCentro().getId());
			  }
		  }
	  }
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.core.service.BrowserFilteredByActive#clearFilters()
   */
  @Override
  public void clearFilters() {
    super.clearFilters();
    // filterOld();
    //filtrarCentrosDesactivados();
    filter();
  }
}
