package es.csa.fdt.cyldigital.core.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import org.richfaces.event.UploadEvent;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.faces.util.FicheroSubidoRichFacesAdapter;
import es.csa.fdt.cyldigital.core.entity.FileOnDisk;
import es.csa.fdt.cyldigital.core.entity.FilePanel;
import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.core.entity.FileOnDisk.Tipo;

/**
 * Clase  FilePanelServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class FilePanelServiceImpl extends AbstractServiceImpl<FilePanel, Long> implements FilePanelService {

  private static final long serialVersionUID = -5575228180257917674L;

  private FileOnDisk lastUploaded;

  /**
   * Devuelve last uploaded.
   * 
   * @return last uploaded
   */
  public FileOnDisk getLastUploaded() {
    return lastUploaded;
  }

  /**
   * Asigna last uploaded.
   * 
   * @param lastUploaded
   *          last uploaded
   */
  public void setLastUploaded(FileOnDisk lastUploaded) {
    this.lastUploaded = lastUploaded;
  }

  /**
   * Aniadir fichero.
   * 
   * @param event
   *           event
   */
  public synchronized void aniadirFichero(UploadEvent event) {
    lastUploaded = new FileOnDisk();
    lastUploaded.setTipo(Tipo.FILEPANEL);
    lastUploaded.setFicheroSubido(new FicheroSubidoRichFacesAdapter(event.getUploadItem()));
    lastUploaded.setNombreOriginal(lastUploaded.getFicheroSubido().getNombre());
  }

  /**
   * Quitar fichero.
   * 
   * @param f
   *           f
   */
  public void quitarFichero(FilePanel f) {
    lastUploaded = null;
    f.setFile(null);
  }

  /*
   * public FilePanel quitarPdf(FilePanel f) { borrarFicheroTemporal(f); f.setFicheroSubido(null); f.setNombre(null);
   * setLastUploaded(null); setAnuncioRepetidoAvisado(true); return f; }
   * 
   * public void borrarFicheroTemporal(FilePanel file) { if (file != null && file.getFicheroSubido() != null) {
   * file.getFicheroSubido().deleteFichero(); } }
   */
  /**
   * Persist file panel.
   * 
   * @param f
   *           f
   * @return true, if successful
   */
  public boolean persistFilePanel(FilePanel f) {
    Parameter p = this.findOneWhere(Parameter.class, "nombre", "pathfileondisk");

    if (f.getTipo() != FilePanel.Tipo.VIDEO_ESPECIAL) {

      if (lastUploaded == null && f.getFile() == null) {
        enableMessages();
        addErrorMessage("Debe indicar un fichero. Recuerde que debe pulsar el botón 'Subir' tras pulsar en 'Subir fichero'");
        return false;
      }

      if (lastUploaded != null) {// es un nuevo fichero, en otro caso, estamos editando un fichero subido.
        f.setFile(new FileOnDisk());
        f.getFile().setTamanio(lastUploaded.getFicheroSubido().getContenido().length);

        if (f.getFile().getTamanio() > 10456589) {
          enableMessages();
          addErrorMessage("El máximo tamaño permitido son 10 Mb. Para vídeos superiores, utilice los vídeos especiales");
          return false;
        }
        // obtenemos del tipo mime la extension.
        lastUploaded.grabarEnDisco(p.getValor());
        disableMessages();
        persist(lastUploaded);
        enableMessages();
        f.setFile(lastUploaded);
      }
    } else {
      // tipo =VIDEO_ESPECIAL
      f.setTipo(FilePanel.Tipo.VIDEO_ESPECIAL);
      f.getFile().setTipo(Tipo.FILEPANEL);
      Boolean error = false;
      if (f.getFile().getNombreOriginal().isEmpty()) {
        enableMessages();
        addErrorMessage("Error de validación: Debe rellenar el nombre del fichero.");
        error = true;
      }
      if (f.getFile().getTipoMime().isEmpty()) {
        enableMessages();
        addErrorMessage("Error de validación: Debe rellenar el tipo MIME del fichero.");
        error = true;
      }
      if (f.getFile().getTamanio() == 0) {
        enableMessages();
        addErrorMessage("Error de validación: Debe rellenar el tamaño del fichero.");
        error = true;
      }
      if (f.getFile().getExtension().isEmpty()) {
        enableMessages();
        addErrorMessage("Error de validación: Debe rellenar la extensión.");
        error = true;
      }
      if (error) return false;

      disableMessages();
      f.getFile().generarFicheroVacio(p.getValor());
      persist(f.getFile());
      persist(f);
    }

    persist(f);
    return true;
  }

  /**
   * Inicializa.
   * 
   * @return file panel
   */
  public FilePanel inicializa() {
    FilePanel f = new FilePanel();
    f.setFile(new FileOnDisk());
    return f;
  }
}
