package es.csa.fdt.cyldigital.core.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.commons.persistence.browser.jpaql.FilterJPAQLByQuery;
import es.csa.fdt.cyldigital.core.entity.FilePanel;
import es.csa.fdt.cyldigital.recursos.entity.EventoPanel;

/**
 * Servicio del Browser de Ficheros de Paneles.
 * 
 * @author csa
 */
@Service
@Repository
public class FilePanelesBrowserImpl extends BrowserFilteredByActive<FilePanel> implements
    FilePanelesBrowser<CriterionJPAQL, String> {

  private static final long serialVersionUID = -7227366052245717441L; 

  /**
   * Construye un nuevo objeto de la clase
   */
  public FilePanelesBrowserImpl() {
    super();
  }

  /**
   * Asigna filter tipo.
   * 
   * @param e
   *          filter tipo
   */
  public void setFilterTipo(EventoPanel e) {
    removeFilterTipo();
    filterByTipo(e);
  }

  private void removeFilterTipo() {
    int i = -1;
    for (Filter<CriterionJPAQL, FilePanel> f : getFilters())
      if (f.getCriterion().getQuery().contains("tipo")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
  }

  /**
   * Filter by tipo.
   * 
   * @param e
   *           e
   */
  public void filterByTipo(EventoPanel e) {
    String query = "";

    boolean flag = false;
    FilterJPAQLByQuery<FilePanel> filter = new FilterJPAQLByQuery<FilePanel>();

    query += " tipo = ? ";
    filter.getCriterion().addParameter(e.getTipoContenido());

    for (Filter<CriterionJPAQL, FilePanel> f : getFilters())
      if (f.getCriterion().getQuery().contains("tipo")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }

}
