package es.csa.fdt.cyldigital.core.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import javax.persistence.NoResultException;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.evaluacion.entity.Formulario;
import es.csa.fdt.cyldigital.evaluacion.entity.FormularioConcreto;
import es.csa.fdt.cyldigital.evaluacion.entity.Grupo;
import es.csa.fdt.cyldigital.evaluacion.entity.Item;
import es.csa.fdt.cyldigital.evaluacion.entity.Pregunta;
import es.csa.fdt.cyldigital.evaluacion.entity.Respuesta;


/**
 * Clase ParameterServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class ParameterServiceImpl extends AbstractServiceImpl<Parameter, Long> implements ParameterService {

  private static final long serialVersionUID = -5575228180257917674L;

  /**
   * Construye un nuevo objeto de la clase
   */
  public ParameterServiceImpl() {
    super();
  }

  /**
   * Devuelve formulario inicial.
   * 
   * @return formulario inicial
   */
  public Long getFormularioInicial() {
    try {
      Parameter p = findOneWhere(Parameter.class, "nombre", "formularioInicial");
      if (p != null) if (p.getValor() != null && !p.getValor().isEmpty()) return Long.valueOf(p.getValor());
    } catch (NoResultException e) {}
    return null;
  }

  /**
   * Find formulario concreto.
   * 
   * @param id
   *           id
   * @return formulario concreto
   */
  public FormularioConcreto findFormularioConcreto(Long id) {
    return find(FormularioConcreto.class, id);
  }

  /**
   * Creacion formulario valores.
   * 
   * @param formulario
   *           formulario
   * @param concreto
   *           concreto
   * @return formulario concreto
   */
  public FormularioConcreto creacionFormularioValores(Formulario formulario, FormularioConcreto concreto) {
    try {
      for (Item item : formulario.getItems()) {
        if (item.isPregunta()) {
          Respuesta respuesta = new Respuesta();
          respuesta.setPregunta((Pregunta) item);
          if (!concreto.getRespuestas().containsKey(item)) concreto.getRespuestas().put((Pregunta) item, respuesta);
        } else {
          for (Pregunta pregunta : ((Grupo) item).getPreguntas()) {
            Respuesta respuesta = new Respuesta();
            respuesta.setPregunta(pregunta);
            if (!concreto.getRespuestas().containsKey(pregunta)) concreto.getRespuestas().put(pregunta, respuesta);
          }
        }
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
    return concreto;
  }

  /**
   * Find parameter.
   * 
   * @param id
   *           id
   * @return parameter
   */
  public Parameter findParameter(Long id) {
    return this.find(Parameter.class, id);
  }

  /**
   * Persist parameter.
   * 
   * @param p
   *           p
   */
  public void persistParameter(Parameter p) {
    persist(p);
  }
}
