package es.csa.fdt.cyldigital.diplomas.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;

import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.core.entity.FileOnDisk;

import es.csa.fdt.cyldigital.language.entity.DiplomaLanguage;

import es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso;

/**
 * Clase Diploma.
 * 
 * @author csa
 */
@Entity
@Table(name = "diplomas")
@AttributeOverride(name = "id", column = @Column(name = "idDiploma"))
@Audited
public class Diploma extends AbstractEntity implements Serializable {

	private static final long serialVersionUID = 8863019270785600417L;
	
	/** Indica si el diploma es el diploma por defecto */
	private boolean porDefecto;
	
	/** Textos en multiples idiomas asociados al diploma */
	private List<DiplomaLanguage> textos;

	/** Indicates whether the diploma takes the signature of the head of the center */
	private boolean signed;
	
	/** Fichero de la plantilla del diploma */
	private FileOnDisk ficheroPlantilla;
	
	/** */
	private List<MatriculaCurso> diplomasEmitidosCursos;
	
	/**
	 * Construye un nuevo objeto de la clase
	 */
	public Diploma() {
		super();
		
		setPorDefecto(false);
		ficheroPlantilla = new FileOnDisk();
	}
	
	/**
	 * @return the porDefecto
	 */
	@NotNull
	@Column(nullable = false, columnDefinition = "bit(1) default 0")
	public boolean isPorDefecto() {
		return porDefecto;
	}

	/**
	 * @param porDefecto the porDefecto to set
	 */
	public void setPorDefecto(boolean porDefecto) {
		this.porDefecto = porDefecto;
	}

	/**
	 * Devuelve textos en múltiples idiomas asociados al diploma
	 * 
	 * @return textos
	 */
  	@OneToMany(cascade = CascadeType.ALL)
  	@JoinColumn(name = "idDiploma")
	public List<DiplomaLanguage> getTextos() {
		return textos;
	}

	/**
	 * Asigna los textos en múltiples idiomas asociados al diploma
	 * 
	 * @param textos
	 *          textos
	 */
	public void setTextos(List<DiplomaLanguage> textos) {
		this.textos = textos;
	}

	/**
	 * Indicates whether the diploma takes the signature of the head of the center
	 * @return <code>true</code> if the diploma is signed, <code>false</code> otherwise.
	 */
	@NotNull
	@Column(nullable = false, columnDefinition = "bit(1) default 0")
	public boolean isSigned() {
		return this.signed;
	}

	/**
	 * Establishes whether the diploma takes the signature of the head of the center
	 * @param signed <code>true</code> to set the diploma as signed, <code>false</code> otherwise.
	 */
	public void setSigned(boolean signed) {
		this.signed = signed;
	}

	/**
	 * Devuelve el fichero de la ficheroPlantilla del diploma
	 * 
	 * @return ficheroPlantilla
	 */
	//@OneToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
	@OneToOne(cascade = CascadeType.ALL)
	@JoinColumn(name = "idFicheroPlantilla", nullable = false)
	@NotNull
	public FileOnDisk getFicheroPlantilla() {
	    return ficheroPlantilla;
	}
	
	/**
	 * Asigna el fichero de la ficheroPlantilla del diploma.
	 * 
	 * @param ficheroPlantilla
	 *          ficheroPlantilla
	 */
	public void setFicheroPlantilla(FileOnDisk ficheroPlantilla) {
	    this.ficheroPlantilla = ficheroPlantilla;
	}
	
	/**
	 * Devuelve las matrículas de cursos que han emitido diploma con esta plantilla
	 * 
	 * @return diplomasEmitidosCursos
	 */
	@OneToMany(fetch = FetchType.LAZY, cascade = CascadeType.ALL)
	@JoinColumn(name = "idPlantillaDiplomaEmitido")
	public List<MatriculaCurso> getDiplomasEmitidosCursos() {
		return diplomasEmitidosCursos;
	}

	/**
	 * Asigna las matrículas de cursos que han emitido diploma con esta plantilla
	 * 
	 * @param diplomasEmitidosCursos
	 * 			diplomasEmitidosCursos
	 */
	public void setDiplomasEmitidosCursos(List<MatriculaCurso> diplomasEmitidosCursos) {
		this.diplomasEmitidosCursos = diplomasEmitidosCursos;
	}

	@Transient
	public boolean isAnyIssued() {
		if ((diplomasEmitidosCursos != null) && (diplomasEmitidosCursos.size() > 0))
			return true;
		else
			return false;
	}
}
