package es.csa.fdt.cyldigital.diplomas.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.List;

import es.csa.fdt.cyldigital.core.service.AbstractService;

import es.csa.fdt.cyldigital.language.entity.Language;
import es.csa.fdt.cyldigital.diplomas.entity.Diploma;

import es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso;

/**
 * Interfaz del servicio de Diploma.
 * 
 * @author Fundetec
 */
public interface DiplomaService extends AbstractService<Diploma, Long> {
	/**
	 * Crea en textos los idiomas necesarios.
	 *
	 * @param diploma
	 * 			diploma
	 */
	public void createLanguages(Diploma diploma, List<Language> languages);
	
	/**
	 * Devuelve un buffer conteniendo una imagen con la primera página del PDF del diploma.
	 *
	 * @param idDiploma
	 * 			id del diploma
	 */
	public BufferedImage getImageFromPDF(Long idDiploma) throws IOException;
	
	/**
	 * Persist diploma.
	 * 
	 * @param diploma
	 *           diploma
	 */
	public boolean persistDiploma(Diploma diploma);
	
	/**
	 * Removes diploma.
	 * 
	 * @param diploma
	 *           diploma
	 */
	public boolean removeDiploma(Diploma diploma); 
	
	/**
	 * Issues a diploma for user
	 * 
	 * @param matricula
	 * 			MatriculaCurso over issues diploma
	 * 
	 * @param languageISO
	 * 			String with language for diploma.
	 * 
	 * @return ByteArrayOutputStream with diploma issued in PDF.
	 */
	public ByteArrayOutputStream issueDiploma(MatriculaCurso matricula, String languageISO);
}
