package es.csa.fdt.cyldigital.diplomas.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import com.lowagie.text.Document;
import com.lowagie.text.Element;
import com.lowagie.text.Font;
import com.lowagie.text.Image;
import com.lowagie.text.PageSize;
import com.lowagie.text.pdf.BaseFont;
import com.lowagie.text.pdf.ColumnText;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfImportedPage;
import com.lowagie.text.pdf.PdfReader;
import com.lowagie.text.pdf.PdfWriter;
import com.lowagie.text.Paragraph;
import com.lowagie.text.html.WebColors;
import com.lowagie.text.html.simpleparser.HTMLWorker;
import com.sun.pdfview.PDFFile;
import com.sun.pdfview.PDFPage;

import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.RandomAccessFile;
import java.io.Serializable;
import java.io.StringReader;
import java.nio.ByteBuffer;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;

import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.imageio.ImageIO;

import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringEscapeUtils;
import org.primefaces.model.StreamedContent;
import org.primefaces.model.DefaultStreamedContent;
import org.richfaces.event.UploadEvent;
import org.richfaces.json.JSONArray;
import org.richfaces.json.JSONObject;
import org.richfaces.model.UploadItem;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.faces.util.FicheroSubidoRichFacesAdapter;
import es.csa.fdt.cyldigital.core.entity.FileOnDisk;
import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.diplomas.entity.Diploma;
import es.csa.fdt.cyldigital.evaluacion.entity.Examen;
import es.csa.fdt.cyldigital.language.entity.ActividadLanguage;
import es.csa.fdt.cyldigital.language.entity.CursoLanguage;
import es.csa.fdt.cyldigital.language.entity.DiplomaLanguage;
import es.csa.fdt.cyldigital.language.entity.Language;
import es.csa.fdt.cyldigital.language.entity.ServicioCursoLanguage;
import es.csa.fdt.cyldigital.servicios.entity.Curso;
import es.csa.fdt.cyldigital.servicios.entity.ServicioCurso;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;

/**
 * Clase DiplomaServiceImpl.
 * 
 * @author Fundetec
 */
@Service
@Repository
public class DiplomaServiceImpl extends AbstractServiceImpl<Diploma, Long> implements DiplomaService {
	private static final long serialVersionUID = -5575228180257917674L;
	
	private static final Logger logger = LoggerFactory.getLogger(DiplomaServiceImpl.class);
	
	//Enumerate fields to add into text fields of diploma template
	private enum eDiplomaTextFields {
		NIF,
		NOMBRE_CURSO,
		HORAS_CURSO,
		NOTA_ALUM,
		//RF_XEA_10 Se añaden los campos FECHA_INI y FECHA_FIN
		FECHA_INI,
		FECHA_FIN
	};
	
	//Class to represent enumeration of fields in drop down list
	public class DiplomaTextField implements Serializable {
		private static final long serialVersionUID = -1861347584518837945L;
		
		private eDiplomaTextFields field;
		private String key;
		private String value;
		
		private DiplomaTextField(eDiplomaTextFields field) {
			this.field = field;
			
			key = "";
			for (String part : field.toString().split("_"))
				key += part.substring(0, 1) + part.substring(1).toLowerCase();
			
			value = "{" + field.toString() + "}";
		}
		
		public eDiplomaTextFields getField() {
			return field;
		}
		
		public String getKey() {
			return key;
		}
		
		public String getValue() {
			return value; 
		}
	};
	
	//Class to represent each elements in JSON string added to diploma template designer. 
	private class DiplomaImageElement implements Serializable {
		private static final long serialVersionUID = 4434772523559050751L;
		
		private JSONObject jsonO;
		
		public DiplomaImageElement(JSONObject jsonO) {
			this.jsonO = jsonO;
		}
		
		public String getId() {
			try {
				return jsonO.getString("id");
			}
			catch (Exception e) {
				e.printStackTrace();
			}

			return "";
		}
		
		public Float getWidth() {
			try {
				return Double.valueOf(jsonO.getDouble("width")).floatValue();
			}
			catch (Exception e) {
				e.printStackTrace();
			}

			return Float.valueOf(-1.0F);
		}
		
		public Float getHeight() {
			try {
				return Double.valueOf(jsonO.getDouble("height")).floatValue();
			}
			catch (Exception e) {
				e.printStackTrace();
			}

			return Float.valueOf(-1.0F);
		}
		
		public Integer getPosX() {
			try {
				return Integer.valueOf(jsonO.getInt("posX"));
			}
			catch (Exception e) {
				e.printStackTrace();
			}

			return Integer.valueOf(-1);
		}
		
		public Integer getPosY() {
			try {
				return Integer.valueOf(jsonO.getInt("posY"));
			}
			catch (Exception e) {
				e.printStackTrace();
			}

			return Integer.valueOf(-1);
		}
		
		public Integer getZIndex() {
			try {
				return Integer.valueOf(jsonO.getInt("zIndex"));
			}
			catch (Exception e) {
				e.printStackTrace();
			}

			return Integer.valueOf(-1);
		}
		
		public String getImagePath() {
			try {
				String imagePath = jsonO.getString("imagePath");
				
				return imagePath.substring(imagePath.indexOf("/") + 1);
			}
			catch (Exception e) {
				e.printStackTrace();
			}

			return "";
		}
	};
	
	//Class to sort elements added to diploma template designer bye z-index and insertion order 
	private class ComparatorDiplomaImageElementsASC implements Serializable, Comparator<DiplomaImageElement> {
		private static final long serialVersionUID = -1626254919634562131L;

		public int compare(DiplomaImageElement o1, DiplomaImageElement o2) {
			if (o1.getZIndex() < o2.getZIndex())
				return -1;
			else if (o1.getZIndex() > o2.getZIndex())
				return 1;
			else { // if (o1.getZIndex() == o2.getZIndex())
				Integer id1 = Integer.parseInt(o1.getId().substring(10));
				Integer id2 = Integer.parseInt(o2.getId().substring(10));

				return id1.compareTo(id2);
			}
		}
	};
	
	/////////////////////////Attributes/////////////////////////////////

	//Diploma layout text fields: Title, name, text and date.
	private List<DiplomaTextField> diplomaTextFields;
	
	//Boolean to indicate if is possible modify existing diploma template design (not new)
	private Boolean editExistingDiploma;
	
	//Method to define diploma template
	private String methodToDefineDiplomaTemplate;
	
	//In designer method, JSON string with all elements added
	private String stringDiplomaElements;

	//In uploader method
	//PDF file uploaded with diploma template 
	private FicheroSubidoRichFacesAdapter diplomaTemplate;
	
	//Boolean to indicate if PDF file with diploma template has been uploaded
	private Boolean diplomaTemplateUploaded;
	
	//Image preview of fist page of PDF file with diploma template uploaded
	private byte[] templateImage;
	
	/////////////////////////Default constructor/////////////////////////////////
  
	/**
	 * Construye un nuevo objeto de la clase
	 */
	public DiplomaServiceImpl() {
		super();
		
		diplomaTextFields = new ArrayList<DiplomaTextField>();
		for (eDiplomaTextFields field : eDiplomaTextFields.values())
			diplomaTextFields.add(new DiplomaTextField(field));
		
		editExistingDiploma = false;
		
		methodToDefineDiplomaTemplate = "designer";
		
		diplomaTemplate = null;
		diplomaTemplateUploaded = false;
		templateImage = null;
	}
	
	/////////////////////////Getters and Setters/////////////////////////////////
	
	/**
	 * @return the diplomaTextFields
	 */
	public List<DiplomaTextField> getDiplomaTextFields() {
		return diplomaTextFields;
	}
	
	/**
	 * @return the editExistingDiploma
	 */
	public boolean getEditExistingDiploma() {
		return editExistingDiploma;
	}
	
	/**
	 * @param editExistingDiploma Boolean indicates if is possible modify template of existing diploma template (not new)
	 */
	public void setEditExistingDiploma(boolean editExistingDiploma) {
		this.editExistingDiploma = editExistingDiploma;
	}
	
	/**
	 * @return the methodToDefineDiplomaTemplate
	 */
	public String getMethodToDefineDiplomaTemplate() {
		return methodToDefineDiplomaTemplate;
	}
	
	/**
	 * @param methodToDefineDiplomaTemplate the method to define diploma template
	 */
	public void setMethodToDefineDiplomaTemplate(String methodToDefineDiplomaTemplate) {
		this.methodToDefineDiplomaTemplate = methodToDefineDiplomaTemplate;
	}
	
	/**
	 * @return the stringDiplomaElements
	 */
	public String getStringDiplomaElements() {
		return stringDiplomaElements;
	}
  
	/**
	 * @param stringDiplomaElements the stringDiplomaElements to set
	 */
	public void setStringDiplomaElements(String stringDiplomaElements) {
		this.stringDiplomaElements = stringDiplomaElements;
	}
	
	/**
	 * @return the diplomaTemplateUploaded
	 */
	public boolean getDiplomaTemplateUploaded() {
		return diplomaTemplateUploaded;
	}
	
	/**
	 * @return the templateImage
	 */
	public StreamedContent getTemplateImage() {
	    if (FacesContext.getCurrentInstance().getRenderResponse()) {
	        // So, we're rendering the view. Return a stub StreamedContent so that it will generate right URL.
	        return new DefaultStreamedContent();
	    }
	    else {
	        // So, browser is requesting the image. Return a real StreamedContent with the image bytes.
	    	if (templateImage != null)
	    		return new DefaultStreamedContent(new ByteArrayInputStream(templateImage), "image/jpg", "diplomaTemplatePreview.jpg");
	    	else
	    		return new DefaultStreamedContent();
	    }
	}
	
	/////////////////////////Methods/////////////////////////////////

	/* (non-Javadoc)
	 * @see es.csa.fdt.cyldigital.evaluacion.service.DiplomaService#createLanguages(es.csa.fdt.cyldigital.evaluacion.entity.Diploma)
	 */
	public void createLanguages(Diploma diploma, List<Language> languages) {
		diploma.setTextos(new ArrayList<DiplomaLanguage>());

		for (Language language : languages) {
			DiplomaLanguage texto = new DiplomaLanguage();
			
			texto.setLanguage(language);
			texto.setDiploma(diploma);

			diploma.getTextos().add(texto);
		}
	}
	
	/* (non-Javadoc)
	 * @see java.util.Map<java.lang.String, java.lang.String> es.csa.fdt.cyldigital.evaluacion.service.DiplomaService#getDiplomaStaticLayouts(java.util.List<es.csa.fdt.cyldigital.language.entity.Language>)
	 */
	public Map<String, String> getDiplomaStaticLayouts(List<Language> languages) {
		Map<String, String> diplomaLayouts = new HashMap<String, String>();
		
		for (Language l : languages) {
			ResourceBundle diplomasBundle = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.diplomas", new Locale(l.getISO().split("_")[0], l.getISO().split("_")[1]));
			
			diplomaLayouts.put("diplomas.literalDiploma_"		+ l.getISO(), diplomasBundle.getString("diplomas.literalDiploma"));
			diplomaLayouts.put("diplomas.literalNombreAlumno_"	+ l.getISO(), diplomasBundle.getString("diplomas.literalNombreAlumno"));
			diplomaLayouts.put("diplomas.literalFechaDiploma_"	+ l.getISO(), diplomasBundle.getString("diplomas.literalFechaDiploma"));
			diplomaLayouts.put("diplomas.literalResponsable_" + l.getISO(), diplomasBundle.getString("diplomas.literalResponsable")); // new line RF_15_30
			diplomaLayouts.put("diplomas.literalSignature_"		+ l.getISO(), diplomasBundle.getString("diplomas.literalSignature"));
		}

		return diplomaLayouts;
	}
	
	/* (non-Javadoc)
	 * @see es.csa.fdt.cyldigital.evaluacion.service.DiplomaService#getDiplomaStaticLayouts(es.csa.fdt.cyldigital.diplomas.entity.Diploma)
	 */
	public void setDiplomaCustomLayouts(Diploma diploma) {
		for (DiplomaLanguage dl : diploma.getTextos()) {
			ResourceBundle diplomasBundle = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.diplomas",
																	 new Locale(dl.getLanguage().getISO().split("_")[0], dl.getLanguage().getISO().split("_")[1]));
			
			dl.setTextoDiplomaAsistencia(diplomasBundle.getString("diplomas.initialLiteralDiplomaAsistencia"));
			dl.setTextoDiplomaAprovechamiento(diplomasBundle.getString("diplomas.initialLiteralDiplomaAprovechamiento"));
		}
	}
	
	/*
	 * 
	 */
	private List<DiplomaImageElement> getDiplomaImageElements(String stringDiplomaElements) {
		List<DiplomaImageElement> ldie = new ArrayList<DiplomaImageElement>();
		
		try {
			JSONArray jsonDiplomaElements = new JSONArray(stringDiplomaElements);

			for (int i = 0; i < jsonDiplomaElements.length(); i++)
				ldie.add(new DiplomaImageElement(jsonDiplomaElements.getJSONObject(i)));
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		
		return ldie;
	}

	/*
	 * 
	 */
	private BufferedImage getImageFromPDF(ByteBuffer diplomaTemplatePDF) throws IOException {
		//Crete te image of PDF
		PDFFile pdfFile = new PDFFile(diplomaTemplatePDF);
		PDFPage pdfPage = pdfFile.getPage(0);
	      
		//Create rectangle to extract image.
		Rectangle2D pageRect;
		if (pdfFile.getMinorVersion() >= 5)
			pageRect = new Rectangle((int) pdfPage.getBBox().getY(),	 (int) pdfPage.getBBox().getX(),
									 (int) pdfPage.getBBox().getWidth(), (int) pdfPage.getBBox().getHeight());
		else
			//Inverting width and height because PDF page is in landscape and PDF version < 1.5
			pageRect = new Rectangle((int) pdfPage.getBBox().getY(),	  (int) pdfPage.getBBox().getX(),
									 (int) pdfPage.getBBox().getHeight(), (int) pdfPage.getBBox().getWidth());
	      
		java.awt.Image imagePage = pdfPage.getImage((int)pdfPage.getBBox().getWidth(),  // width
													(int)pdfPage.getBBox().getHeight(), // height
													pageRect,							// clip rect
													null,								// null for the ImageObserver
													true,								// fill background with white
													true);								// block until drawing is done
	      
		BufferedImage bufferedImage = new BufferedImage((int)pdfPage.getBBox().getWidth(),
														(int)pdfPage.getBBox().getHeight(),
														BufferedImage.TYPE_INT_RGB);
		Graphics2D bufImageGraphics = bufferedImage.createGraphics();
		bufImageGraphics.drawImage(imagePage, 0, 0, null);
		bufImageGraphics.dispose();
		
		return bufferedImage;
	}
	
	/*
	 * (non-Javadoc)
	 * @see es.csa.fdt.cyldigital.diplomas.service.DiplomaService#getImageFromPDF(java.lang.Long)
	 */
	public BufferedImage getImageFromPDF(Long idDiploma) throws IOException {
		Diploma d = find(idDiploma);

		if (d != null) {
			//Extract path to directory containing diploma templates generated.
			String pathDiplomas = findOneWhere(Parameter.class, "nombre", "path_generated_diplomas_ondisk").getValor();
		      
			//Gets PDF file stored on disk
			File fileOnDisk = new File(pathDiplomas + d.getFicheroPlantilla().getNombre());

			//Read PDF file in buffer
			RandomAccessFile raf = new RandomAccessFile(fileOnDisk, "r");
			byte[] b = new byte[(int)raf.length()];
			raf.readFully(b);
			ByteBuffer bb = ByteBuffer.wrap(b);

			BufferedImage bi = getImageFromPDF(bb);
			
			bb.clear();
			raf.close();
			
			return bi;
		}
		
		return null;
	}
	
	/*
	 * 
	 */
	public synchronized void uploadDiplomaTemplateListener(UploadEvent event) {
		ByteArrayOutputStream baos = null;
		
		try {
			InputStream is = new FileInputStream(event.getUploadItem().getFile());
        
			BufferedImage imageDiplomaTemplate = getImageFromPDF(ByteBuffer.wrap(IOUtils.toByteArray(is)));

			baos = new ByteArrayOutputStream();
			ImageIO.write(imageDiplomaTemplate, "jpg", baos);

			imageDiplomaTemplate.flush();
		}
		catch (Exception e) {
			logger.error("DiplomaServiceImpl.uploadDiplomaTemplateListener: " +
						 "Error al extraer el preview del PDF de plantilla de diploma." +
						 e.getLocalizedMessage());

			e.printStackTrace();
		}
		finally {
			diplomaTemplate = new FicheroSubidoRichFacesAdapter(event.getUploadItem());
			templateImage = baos.toByteArray();
			diplomaTemplateUploaded = true;
		}
	}
	
	/*
	 * 
	 */
	public synchronized void removeDiplomaTemplateListener(ActionEvent event) {
		diplomaTemplate = null;
		templateImage = null;
		diplomaTemplateUploaded = false;
	}
	
	/* (non-Javadoc)
	 * @see es.csa.fdt.cyldigital.evaluacion.service.DiplomaService#persistDiplomaDesigner(es.csa.fdt.cyldigital.evaluacion.entity.Diploma)
	 */
	private FicheroSubidoRichFacesAdapter generateDiplomaTemplateFromDesigner() {
		FicheroSubidoRichFacesAdapter fsrfa = null;
		
		try {
			List<DiplomaImageElement> ldie = getDiplomaImageElements(stringDiplomaElements);
			Collections.sort(ldie, new ComparatorDiplomaImageElementsASC());
			
			//Extract path to directory containing Diplomas Designer Image Files.
			String pathDDFiles = ((Parameter)findOneWhere(Parameter.class, "nombre", "path_diplomas_designer_ondisk")).getValor();

			//Create PDF document
			Document docDiploma = new Document(PageSize.A4.rotate());
			
			//Create stream to storage PDF Document
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			
			//Get PDF document writer
			PdfWriter pdfWriter = PdfWriter.getInstance(docDiploma, baos);

			//Open PDF document
			docDiploma.open();
			
			//Get canvas of PDF document's page
			PdfContentByte pdfCB = pdfWriter.getDirectContent();

			//Calculate aspect ratios to convert from designer to real
			Float arH = new Float(PageSize.A4.rotate().getWidth()  / 700.0F); /*horizontal aspect ratio = real width / represented width*/
			Float arV = new Float(PageSize.A4.rotate().getHeight() / 495.0F); /*vertical aspect ratio = real height / represented height*/
			
			//Add images elements from designer to PDF document
			for (DiplomaImageElement die : ldie) {
				Image img = Image.getInstance(pathDDFiles + die.getImagePath());
				
				img.scaleAbsolute(die.getWidth() * arH, die.getHeight() * arV);
				img.setAbsolutePosition(die.getPosX() * arH, (495.0F - die.getHeight() - die.getPosY()) * arV);
				
				pdfCB.addImage(img);
			}

			//Close document and writer
			docDiploma.close();
			pdfWriter.close();
			
			//Creates a virtual upload item with stream of PDF document
			fsrfa = new FicheroSubidoRichFacesAdapter(new UploadItem("DT_" + (new Date()).getTime() + ".pdf", baos.size(),
																	 "pdf/application", baos.toByteArray()));
		}
		catch (Exception e) {
			logger.info("DiplomaServiceImpl.generateDiplomaTemplateFromDesigner: " +
						"Error al generar fichero PDF de plantilla de diploma desde el diseñador. " +
						e.getLocalizedMessage());
		}
		
		return fsrfa;
	}
	
	/* (non-Javadoc)
	 * @see es.csa.fdt.cyldigital.evaluacion.service.DiplomaService#persistDiploma(es.csa.fdt.cyldigital.evaluacion.entity.Diploma)
	 */
	public boolean persistDiploma(Diploma diploma) {
		String oldFileDT = null;
		
		//Start first phase!
		Boolean result = false;
		Boolean saveToDisk = false;
		
		//If diploma template is new or is not new but has not issued diplomas with this template
		//and user wants edit diploma template design...
		//RF_XEA --> Se quita la condicion del campo isAnyIssued porque se quiere poder editar los diplomas aunque hayan sido emitidos
		//if ((diploma.getId() == 0) || ((diploma.getId() > 0) && !diploma.isAnyIssued() && editExistingDiploma)) {
		if ((diploma.getId() == 0) || ((diploma.getId() > 0) && editExistingDiploma)) {
			// If is designer method selected...
			if (methodToDefineDiplomaTemplate.compareTo("designer") == 0) {
				FicheroSubidoRichFacesAdapter fsrfa;
				
				// Generate diploma template from designer and if it is successful...
				if ((fsrfa = generateDiplomaTemplateFromDesigner()) != null) {
					//If diploma template is not new and result is true...
					if ((diploma.getId() > 0)) {
						//Save old diploma template for deleting PDF file after commit new diploma template.
						oldFileDT = diploma.getFicheroPlantilla().getPath() + diploma.getFicheroPlantilla().getNombre();
						disableMessages();
						remove(diploma.getFicheroPlantilla());
						enableMessages();
						
						diploma.setFicheroPlantilla(new FileOnDisk());
					}
					
					//Set generatedDiplomaTemplateFromDesigner to template file of diploma record to persist
					diploma.getFicheroPlantilla().setFicheroSubido(fsrfa);
					
					//First phase OK!
					result = true;
					saveToDisk = true;
				}
				else
					addErrorMessage("diplomas.errorDesignerPersistingDT");
			}
			// Uploader method selected (else from if is designer method selected) and if has uploaded template...
			else if (methodToDefineDiplomaTemplate.compareTo("uploader") == 0) {
				if (diplomaTemplate != null) {
					//If diploma template is not new and result is true...
					if ((diploma.getId() > 0)) {
						//Save old diploma template for deleting PDF file after commit new diploma template.
						oldFileDT = diploma.getFicheroPlantilla().getPath() + diploma.getFicheroPlantilla().getNombre();
						disableMessages();
						remove(diploma.getFicheroPlantilla());
						enableMessages();
						
						diploma.setFicheroPlantilla(new FileOnDisk());
					}
					
					diploma.getFicheroPlantilla().setFicheroSubido(diplomaTemplate);
		
					//First phase OK!
					result = true;
					saveToDisk = true;
				}
				else
					addErrorMessage("diplomas.errorUploaderPersistingDT");
			}
			else
				addErrorMessage("diplomas.errorGeneralPersistingDT");
		}
		else
			result = true;

		//If is first phase OK...
		if (result) {
			//Start last phase!
			result = false;
			
			//Extract path to directory containing diploma templates generated.
			Parameter pathGenDiplomas = findOneWhere(Parameter.class, "nombre", "path_generated_diplomas_ondisk");
		
			//If pathGenDiplomas exist in database...
			if (pathGenDiplomas != null) {
				//If Save to disk is true...
				if (saveToDisk)
					//Save in disk diploma template file
					diploma.getFicheroPlantilla().grabarEnDisco(pathGenDiplomas.getValor());
				
				//Set ISIS type of file to template file of diploma record to persist 
				diploma.getFicheroPlantilla().setTipo(FileOnDisk.Tipo.DIPLOMATEMPLATE);
				
				disableMessages();
				//Persist diploma template file in DB
				persist(diploma.getFicheroPlantilla());
				enableMessages();
				
				//Persist diploma template record in DB
				persist(diploma);
				
				//If new diploma template is marked like default...
				if (diploma.isPorDefecto()) {
					try {
						//Get old default diploma template
						Diploma oldDefaultDiploma = findOneWhere("porDefecto", true);

						//If has old default diploma template marked like default...
						if ((oldDefaultDiploma != null) && (oldDefaultDiploma.getId() != diploma.getId())) {
							//Set old default diploma template like NOT default
							oldDefaultDiploma.setPorDefecto(false);

							disableMessages();
							//Persist modified old default diploma template
							persist(oldDefaultDiploma);
							enableMessages();
						}
					}
					catch (Exception e){
						logger.info("DiplomaServiceImpl.persistDiploma: " +
									"No existe un diploma por defecto. No es necesario sustituirlo por el actual. " +
									e.getLocalizedMessage());
					}
				}
				
				//Last phase is OK!
				result = true;
			}
			else
				addErrorMessage("diplomas.errorPathPersistingDT");
		}
		
		//If result of persist operation is true and has a old file of diploma template to remove.
		if (result && (oldFileDT != null) && !oldFileDT.isEmpty())
			result = removeFileDT(oldFileDT);

		//Returns result of persist operation.
		return result;
	}
	
	/*
	 * 
	 */
	private boolean removeFileDT(String pathToFile) {
		if ((pathToFile != null) && !pathToFile.isEmpty()) {
			File file = new File(pathToFile);
			
			if (!file.delete()) {
				logger.error("DiplomaServiceImpl.removeFPD: " +
							 "Error al borrar, el fichero " + pathToFile + ".");

				return false;
			}
		}
		
		return true;
	}
	
	/*
	 * (non-Javadoc)
	 * @see es.csa.fdt.cyldigital.diplomas.service.DiplomaService#removeDiploma(es.csa.fdt.cyldigital.diplomas.entity.Diploma)
	 */
	public boolean removeDiploma(Diploma diploma) {
		Boolean result = true;
		
		//Re-check if diploma is not new and not is default diploma for delete or deactivate...
		if ((diploma.getId() > 0) && !diploma.isPorDefecto()) {
			//Get default diploma.
			Diploma defaultDiploma = findOneWhere("porDefecto", true);
			
			if (defaultDiploma != null) {
				//Get Servicios Curso with this diploma assigned.
				List<ServicioCurso> lsc = findWhere(ServicioCurso.class, "plantillaDiploma", diploma);
				
				disableMessages();
				//Assign default diploma to all servicio curso with
				//diploma to remove or deactivate assigned.
				for (ServicioCurso sc : lsc) {
					sc.setPlantillaDiploma(defaultDiploma);
					
					persist(sc);
				}
				enableMessages();
				
				//If diploma has been issued any time...
				if (diploma.isAnyIssued()) { //Deactivate diploma
					//Sets diploma like deactivated
					diploma.setDesactivado(true);
					
					//Persist diploma template like deactivated
					persist(diploma);
					
					disableMessages();
					//Also set and persist texts (diploma languages) like deactivated 
					for (DiplomaLanguage dl : diploma.getTextos()) {
						dl.setDesactivado(true);
						
						persist(dl);
					}
					enableMessages();
				}
				else { //Delete diploma
					//Gets path to file on disk with diploma template
					String fileDTToRemove = diploma.getFicheroPlantilla().getPath() + diploma.getFicheroPlantilla().getNombre();
					
					//If file path is null or empty...
					if ((fileDTToRemove == null) || fileDTToRemove.isEmpty()) {
						addErrorMessage("diplomas.errorGeneralRemovingDT");
						
						result = false;
					}
					//If is not possible delete file on disk...
					else if (!removeFileDT(fileDTToRemove)) {
						addErrorMessage("diplomas.errorFileRemovingDT");
							
						result = false;
					}
					//File deleted
					else
						//Remove diploma from database
						remove(diploma);
				}
			}
			else {
				addErrorMessage("diplomas.errorNotExistDefaultDTRemovingDT");
				
				result = false;
			}
		}
		else {
			addErrorMessage("diplomas.errorDefaultDTRemovingDT");
			
			result = false;
		}
		
		return result;
	}

	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	/**
	 * 
	 * @param userName
	 * @param languageISO
	 * @return
	 */
	private String composeDiplomaStudentName(String userName, ResourceBundle diplomasBundle) {
		String studentName = "";
		
		studentName = diplomasBundle.getString("diplomas.literalNombreAlumno");
		studentName = studentName.replace("{NOM_ALUM} {APE1_ALUM} {APE2_ALUM}", userName);
		
		return studentName;
	}

	//RF_XEA_10 Se añade a la funcion los parametros de las fechas
	/**
	 * 
	 * @param textTemplate
	 * @param userIN
	 * @param cursoName
	 * @param cursoHours
	 * @param userScore
	 * @param dateIni
	 * @param dateFin
	 * @return
	 */
	private String replaceValuesInDiplomaTextTemplate(String textTemplate, String userIN, String cursoName,
													  String cursoHours, String userScore, String dateIni, String dateFin) {
		for (DiplomaTextField dtf : diplomaTextFields) {
			switch (dtf.getField()) {
				case NIF:
					textTemplate = textTemplate.replace(dtf.getValue(), userIN);
					break;
					
				case NOMBRE_CURSO:
					textTemplate = textTemplate.replace(dtf.getValue(), cursoName);
					
					break;
					
				case HORAS_CURSO:
					textTemplate = textTemplate.replace(dtf.getValue(), cursoHours);
					
					break;
				
				case NOTA_ALUM:
					textTemplate = textTemplate.replace(dtf.getValue(), userScore);

					break;
				//RF_XEA_10 Se añaden los campos FECHA_INI y FECHA_FIN al diploma
				case FECHA_INI:
					textTemplate = textTemplate.replace(dtf.getValue(), dateIni);

					break;
				case FECHA_FIN:
					textTemplate = textTemplate.replace(dtf.getValue(), dateFin);

					break;
			}
		}
		
		return textTemplate;
	}
	
	/**
	 * 
	 * @param matricula
	 * @param languageISO
	 * @return
	 */
	private String composeDiplomaText(MatriculaCurso matricula, String languageISO) {
		String diplomaText = "";
		
		String userIN = "";
		String cursoName = "";
		String cursoHours = "";
		String userScore = "";
		//RF_XEA_10 Se añaden los campos fecha inicio y fecha fin
		String dateIni = "";
		String dateFin = "";
		
		//For get user's identification number
		UsuarioCyL user = find(UsuarioCyL.class, matricula.getUsuario().getId());
		
		if ((user.getNif() != null) && !user.getNif().isEmpty())
			userIN = user.getNif();
		else if ((user.getPasaporte() != null) && !user.getPasaporte().isEmpty())
			userIN = user.getPasaporte();
		
		//For get curso's name
		for (ActividadLanguage al : matricula.getCurso().getTextos()) {
			if (al.getLanguage().getISO().equals(languageISO)) {
				cursoName = al.getNombre();
				
				break;
			}
		}
		
		//For get curso duration in hours
		cursoHours = Float.toString(matricula.getCurso().getNumeroHoras());
		
		//RF_XEA_10 Se obtienen las fechas del curso para mostrar en el diploma
		SimpleDateFormat formatoFecha = new SimpleDateFormat("dd/MM/yyyy");
		dateIni = formatoFecha.format(matricula.getCurso().getFechaInicio());
		dateFin = formatoFecha.format(matricula.getCurso().getFechaFin());
		
		
		//For get user's score and diploma's text according to user's score
		Curso c = find(Curso.class, matricula.getCurso().getId());
		
		//RF_15_25
		Diploma diploma = null;
		ServicioCurso sc = find(ServicioCurso.class, matricula.getCurso().getServicio().getId());
		
		if(matricula.getCurso().getPlantillaDiploma() != null)
			diploma = matricula.getCurso().getPlantillaDiploma();
		else
			diploma = sc.getPlantillaDiploma();
		
		Examen examenTemplate = null;
		if(c.getExamen() != null)
			examenTemplate = c.getExamen();
		else
			examenTemplate = sc.getExamen();
		
		for (DiplomaLanguage dl : diploma.getTextos()) {
			if (dl.getLanguage().getISO().equals(languageISO)) {
				//If (has been done exam) AND
				//	 (
				//	  (
				//	   (activities of services WITH exam) AND
				//	   (score is greater or equal than number of questions to pass exam)
				//	  )
				//	  OR
				//	  (
				//	   (activities of services WITHOUT exam) AND
				//	   (score is greater or equal than 5.0)
				//	  )
				//	 ) -> exam is passed
				if ((matricula.getNota() != null) &&
					(
					 ((examenTemplate != null) && (matricula.getNota() >= examenTemplate.getNumPreguntasAprobado()))
					 ||
					 ((examenTemplate == null) && (matricula.getNota() >= 5.0F))
					)
				   ) {
						//Gets aprovechamiento text
						diplomaText = dl.getTextoDiplomaAprovechamiento();
						
						//Gets user's score
						userScore = Float.toString(matricula.getNota());
				}
				//If (has not been done exam) OR (exam is failed)
				else {
					//Gets asistencia text
					diplomaText = dl.getTextoDiplomaAsistencia();
					
					if (matricula.getNota() != null)
						//Gets user's score
						userScore = Float.toString(matricula.getNota());
					else
						//Sets user's score to 0
						userScore = "0";
				}
				
				//If activities of services WITH exam, add maximum possible score
				if (examenTemplate != null)
					userScore += "/" + examenTemplate.getNumPreguntasMostrar();
			}
		}
		
		//Replace values in diploma text template
		//RF_XEA_10 Se modifican los parametros de entrada de la funcion para permitir las fechas
		//diplomaText = replaceValuesInDiplomaTextTemplate(diplomaText, userIN, cursoName, cursoHours, userScore);
		diplomaText = replaceValuesInDiplomaTextTemplate(diplomaText, userIN, cursoName, cursoHours, userScore, dateIni, dateFin);
		
		return diplomaText;
	}
	
	/**
	 * 
	 * @param diplomaDateTemplate
	 * @param townName
	 * @param day
	 * @param month
	 * @param year
	 * @return
	 */
	private String replaceValuesInDiplomaDateTextTemplate(String diplomaDateTemplate, String townName,
														  String day, String month, String year) {
		//RF_15_30 {LUGAR}, a {DIA} de {MES} de {ANYO}
		diplomaDateTemplate = diplomaDateTemplate.replace("{LUGAR}", townName);
		diplomaDateTemplate = diplomaDateTemplate.replace("{DIA}",	 day);
		diplomaDateTemplate = diplomaDateTemplate.replace("{MES}",	 month);
		diplomaDateTemplate = diplomaDateTemplate.replace("{ANYO}",	 year);
		
		return diplomaDateTemplate;
	}
	
	/**
	 * 
	 * @param matricula
	 * @param languageISO
	 * @return
	 */
	private String composeDiplomaDate(MatriculaCurso matricula, ResourceBundle diplomasBundle, ResourceBundle messagesBundle) {
		String diplomaDate = "";
		
		String townName = "";
		String day = "";
		String month = "";
		String year = "";
		
		diplomaDate = diplomasBundle.getString("diplomas.literalFechaDiploma");
		String[] months = messagesBundle.getString("calendario.Meses").toLowerCase().split(", ");
		
		//For get name of town
		townName = matricula.getLocalidadDiplomaEmitido().getNombre();
		
		//For gets issue date
		Calendar date = Calendar.getInstance();

		//If diploma has been issued gets date stored in in matricula.
		date.setTimeInMillis(matricula.getFechaDiplomaEmitido().getTime());
		
		day = Integer.toString(date.get(Calendar.DAY_OF_MONTH));
		month = months[date.get(Calendar.MONTH)];
		year = Integer.toString(date.get(Calendar.YEAR));
		
		//Replace values in diploma date text template
		diplomaDate = replaceValuesInDiplomaDateTextTemplate(diplomaDate, townName, day, month, year);
		
		return diplomaDate;
	}
	
	/**
	 * 
	 * @param matricula
	 * @param diplomasBundle
	 * @return
	 */
	private String composeDiplomaSignature(MatriculaCurso matricula, ResourceBundle diplomasBundle) {
		
		Empleado teacher = matricula.getCurso().getProfesor();
		boolean signed = matricula.getCurso().getPlantillaDiploma().isSigned();
		  
		String signature = diplomasBundle.getString("diplomas.literalSignature");
		if (teacher != null && signed) {
			signature = signature.replace("{NOM_PROF}", teacher.getNombre());
			signature = signature.replace("{APE1_PROF}", teacher.getApellido1());
			signature = signature.replace("{APE2_PROF}", teacher.getApellido2());
		} else {
			signature = signature.replace("{NOM_PROF} {APE1_PROF} {APE2_PROF}", "");
		}
		
		return signature;
	}	
	
	/**
	 * 
	 * @param matricula
	 * @param languageISO
	 * @return
	 */
	private Map<String, String> buildDiplomaData(MatriculaCurso matricula, String languageISO) {
		Map<String, String> diplomaData = new HashMap<String, String>();
		
		//Gets bundles.
		Locale locale = new Locale(languageISO.split("_")[0], languageISO.split("_")[1]);
		
		ResourceBundle diplomasBundle = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.diplomas", locale);
		ResourceBundle messagesBundle = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", locale);
		
		//Results
		String pathDiplomaTemplate;
		String diplomaTitle;
		String diplomaStudentName;
		String diplomaText;
		String diplomaDate;
		String diplomaSignature;
		
		//For get diploma template's path
		FileOnDisk fod;		
		//RF_15_25
		if(matricula.getCurso().getPlantillaDiploma() != null)
			fod = matricula.getCurso().getPlantillaDiploma().getFicheroPlantilla();
		else
			fod = ((ServicioCurso)matricula.getCurso().getServicio()).getPlantillaDiploma().getFicheroPlantilla();
		
		//Compose results
		pathDiplomaTemplate = fod.getPath() + fod.getNombre();
		diplomaTitle = diplomasBundle.getString("diplomas.literalDiploma");
		diplomaStudentName = composeDiplomaStudentName(matricula.getUsuario().getNombreCompleto(), diplomasBundle);
		diplomaText = composeDiplomaText(matricula, languageISO);
		diplomaDate = composeDiplomaDate(matricula, diplomasBundle, messagesBundle);
		diplomaSignature = composeDiplomaSignature(matricula, diplomasBundle);
		
		diplomaData.put("pathDiplomaTemplate", pathDiplomaTemplate);
		diplomaData.put("diplomaTitle", diplomaTitle);
		diplomaData.put("diplomaStudentName", diplomaStudentName);
		diplomaData.put("diplomaText", diplomaText);
		diplomaData.put("diplomaDate", diplomaDate);
		diplomaData.put("diplomaSignature", diplomaSignature);
		
		return diplomaData;
	}

	
	/*
	 * (non-Javadoc)
	 * @see es.csa.fdt.cyldigital.diplomas.service.DiplomaService#issueDiploma(es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso, java.lang.String)
	 */
	public ByteArrayOutputStream issueDiploma(MatriculaCurso matricula, String languageISO) {
		/**********************************/
		Float TitleHeight = 36.0F;
		Float NameHeight = 33.0F;
		Float TextHeight = 96.0F;
		Float DateHeight = 20.0F;
		Float SignatureHeight = 20.0F;
		/**********************************/
		
		ByteArrayOutputStream diplomaIssued = new ByteArrayOutputStream();

		try {
			//Gets bundles.
			Locale locale = new Locale(languageISO.split("_")[0], languageISO.split("_")[1]);
			ResourceBundle diplomasBundle = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.diplomas", locale);
			
			//Prepare output document
			Document diplomaIssuedDocument = new Document(PageSize.A4.rotate());
			
			PdfWriter pdfWriter = PdfWriter.getInstance(diplomaIssuedDocument, diplomaIssued);
			
			diplomaIssuedDocument.open();
			PdfContentByte pdfCB = pdfWriter.getDirectContent();
			
			// Read service of course from database to obtain content of course 
			Map<String, Object> properties = new HashMap<String, Object>();
			properties.put("servicioCurso.id", matricula.getCurso().getServicio().getId());
			properties.put("language.ISO", languageISO);
			
			ServicioCursoLanguage scl = findOneWhere(ServicioCursoLanguage.class, properties);
			
			//Buid texts of diploma
			Map<String, String> diplomaData = buildDiplomaData(matricula, languageISO);
		
			//Load template of diploma
			// PR-XEA: Intentamos usar la plantilla del diploma asociada al curso, pero en algunos casos estos no son
			//cargados y dan error. En ese caso se usara la plantilla del diploma por defecto.
			PdfReader pdfReader = null;
			try {
				pdfReader = new PdfReader(diplomaData.get("pathDiplomaTemplate"));
			} catch (IOException ioe) {
				logger.error("DiplomaServiceImpl.issueDiploma: " +
						 "Error al emitir diploma. " + diplomaData.get("pathDiplomaTemplate") + " no se encuentra" +
						 " como archivo o recurso. Usando plantilla del diploma por defecto.");
				
				properties.remove("servicioCurso.id");
				properties.remove("language.ISO");
				properties.put("porDefecto", true);
				
				Diploma d = findOneWhere(Diploma.class, properties);
				FileOnDisk fod = d.getFicheroPlantilla();
				
				pdfReader = new PdfReader(fod.getPath() + fod.getNombre());
			}
			
			PdfImportedPage diplomaTemplate = pdfWriter.getImportedPage(pdfReader, 1); 

			// Copy first page of template into issued output diploma
			diplomaIssuedDocument.newPage();
			//RF_XEA_10 --> Esto soluciona el problema de ver los diplomas girados
			//pdfCB.addTemplate(diplomaTemplate, 0, 0);
			int rotation=pdfReader.getPageRotation(1);
			 if (rotation == 90 || rotation == 270) {
				 	pdfCB.addTemplate(diplomaTemplate,0,-1f,1f,0,0,pdfReader.getPageSizeWithRotation(1).getHeight());
			      }
			 else {
				 	pdfCB.addTemplate(diplomaTemplate,1f,0,0,1f,0,0);
			      }
			
			//Create font for write text
			BaseFont arialFont = BaseFont.createFont("es/csa/fdt/cyldigital/pdf/Arial.ttf", BaseFont.CP1252, BaseFont.EMBEDDED);
			
			//Column and paragraph variables
			ColumnText ct;
			Paragraph p;
	
			// Write the title of the diploma.
			ct = new ColumnText(pdfCB);
			ct.setSimpleColumn(60.0F, //Left
							   598.0F - 136.0F, //Bottom *********Mirar PageSize.A4.rotate().getHeight() para 598.0F****************
							   60.0F + 722.0F, //Right   *********Mirar PageSize.A4.rotate().getWidth()  para 842.0F****************
							   598.0F - TitleHeight - 136.0F); //Top
			p = new Paragraph(diplomaData.get("diplomaTitle"),
							  new Font(arialFont, 24.0F, Font.BOLDITALIC, WebColors.getRGBColor("#557294")));
			p.setAlignment(Element.ALIGN_CENTER);
			ct.addElement(p);
			
			ct.go();
			
			// Write the student's name
			ct = new ColumnText(pdfCB);
			ct.setSimpleColumn(60.0F, //Left
							   598.0F - TitleHeight - 182.0F, //Bottom
							   60.0F + 722.0F, //Right
							   598.0F - TitleHeight - NameHeight - 182.0F); //Top
			p = new Paragraph(diplomaData.get("diplomaStudentName"),
							  new Font(arialFont, 22.0F, Font.BOLD));
			p.setAlignment(Element.ALIGN_CENTER);
			ct.addElement(p);
			
			ct.go();
	
			// Write the text of the diploma
			ct = new ColumnText(pdfCB);
			ct.setSimpleColumn(60.0F, //Left
							   598.0F - TitleHeight - NameHeight - 194.0F, //Bottom
							   60.0F + 722.0F, //Right
							   598.0F - TitleHeight - NameHeight - TextHeight - 194.0F); //Top
			p = new Paragraph(diplomaData.get("diplomaText"),
							  new Font(arialFont, 16.0F, Font.NORMAL));
			p.setAlignment(Element.ALIGN_JUSTIFIED);
			ct.addElement(p);
			
			ct.go();
	
			// Write the date and location of issue of the diploma
			ct = new ColumnText(pdfCB);
			ct.setSimpleColumn(60.0F, //Left
							   598.0F - TitleHeight - NameHeight - TextHeight - 216.0F, //Bottom
							   60.0F + 722.0F, //Right
							   598.0F - TitleHeight - NameHeight - TextHeight - DateHeight - 216.0F); //Top
			p = new Paragraph(diplomaData.get("diplomaDate"),
							  new Font(arialFont, 13.0F, Font.ITALIC));
			p.setAlignment(Element.ALIGN_CENTER);
			ct.addElement(p);
	
			ct.go();
			

			if (matricula.getPlantillaDiplomaEmitido().isSigned()) {
				// Write the signature of issue of the diploma
				//Texto sinatura
				ct = new ColumnText(pdfCB);
				ct.setSimpleColumn(60.0F, //Left
						   598.0F - TitleHeight - NameHeight - TextHeight - DateHeight - 216.0F, //Bottom
						   60.0F + 722.0F, //Right
						   598.0F - TitleHeight - NameHeight - TextHeight - DateHeight - SignatureHeight - 260.0F); //Top
				p = new Paragraph(diplomasBundle.getString("diplomas.literalResponsable"),
						  			new Font(arialFont, 13.0F, Font.ITALIC));
				p.setAlignment(Element.ALIGN_CENTER);
				ct.addElement(p);
		

				ct.go();
				//Sinatura
				ct = new ColumnText(pdfCB);
				ct.setSimpleColumn(60.0F, //Left
								   598.0F - TitleHeight - NameHeight - TextHeight - DateHeight - 280.0F, //Bottom
								   60.0F + 722.0F, //Right
								   598.0F - TitleHeight - NameHeight - TextHeight - DateHeight - SignatureHeight - 280.0F); //Top
				p = new Paragraph(diplomaData.get("diplomaSignature"),
								  new Font(arialFont, 13.0F, Font.ITALIC));
				p.setAlignment(Element.ALIGN_CENTER);
				ct.addElement(p);
		

				ct.go();
			}
			
			//If exists content of course in course...
			Map<String, Object> propertiesCurso = new HashMap<String, Object>();
			propertiesCurso.put("curso.id", matricula.getCurso().getId());
			propertiesCurso.put("language.ISO", languageISO);
			
			CursoLanguage cl = (CursoLanguage)findOneWhere(CursoLanguage.class, propertiesCurso);

			if ((cl.getGuion() != null) && !cl.getGuion().isEmpty()) {
				//Add new page to issued output diploma
				diplomaIssuedDocument.newPage();
			
				//Write course content in the new page
				ct = new ColumnText(pdfCB);
				ct.setSimpleColumn(40.0F, //Left
								   598.0F - 40.0F, //Bottom
								   842.0F - 40.0F, //Right
								   40.0F); //Top
				
				p = new Paragraph("TEMARIO\n",
						  new Font(arialFont, 20.0F, Font.BOLD, WebColors.getRGBColor("#557294")));
				p.setAlignment(Element.ALIGN_CENTER);
				ct.addElement(p);
				
				//String sGuion = cl.getGuion().replaceAll("<br />", "\n");
				//sGuion = sGuion.replaceAll("<.*?>", "");

				//String sGuion = StringEscapeUtils.unescapeHtml(cl.getGuion());
				//p = new Paragraph("\n" + sGuion, new Font(arialFont, 10.0F, Font.NORMAL));
				
				StringReader strReader = new StringReader(cl.getGuion());
				ArrayList elementosParrafo = new ArrayList();				
				elementosParrafo = HTMLWorker.parseToList(strReader, null);
			    p = new Paragraph();
			    for (int k = 0; k < elementosParrafo.size(); ++k){
			    	p.add((com.lowagie.text.Element)elementosParrafo.get(k));
			    }
				
				p.setAlignment(Element.ALIGN_LEFT);
				ct.addElement(p);
				
				ct.go();
			//If exists content of course in service of course...
			} else if ((scl.getGuion() != null) && !scl.getGuion().isEmpty()) {
				//Add new page to issued output diploma
				diplomaIssuedDocument.newPage();
			
				//Write course content in the new page
				ct = new ColumnText(pdfCB);
				ct.setSimpleColumn(40.0F, //Left
								   598.0F - 40.0F, //Bottom
								   842.0F - 40.0F, //Right
								   40.0F); //Top
				
				p = new Paragraph("TEMARIO\n",
						  new Font(arialFont, 20.0F, Font.BOLD, WebColors.getRGBColor("#557294")));
				p.setAlignment(Element.ALIGN_CENTER);
				ct.addElement(p);
				
				//String sGuion = scl.getGuion().replaceAll("<br />", "\n");
				//sGuion = sGuion.replaceAll("<.*?>", "");
				
				//String sGuion = StringEscapeUtils.unescapeHtml(scl.getGuion());
				//p = new Paragraph("\n" + sGuion, new Font(arialFont, 10.0F, Font.NORMAL));
				
				StringReader strReader = new StringReader(scl.getGuion());
				ArrayList elementosParrafo = new ArrayList();				
				elementosParrafo = HTMLWorker.parseToList(strReader, null);
			    p = new Paragraph();
			    for (int k = 0; k < elementosParrafo.size(); ++k){
			    	p.add((com.lowagie.text.Element)elementosParrafo.get(k));
			    }
				
				p.setAlignment(Element.ALIGN_LEFT);
				ct.addElement(p);
				
				ct.go();
			}

			diplomaIssuedDocument.close();
			pdfWriter.close();
			pdfReader.close();
		}
		catch (Exception e) {
			logger.error("DiplomaServiceImpl.issueDiploma: " +
						 "Error al emitir diploma. " + e.getLocalizedMessage());

			e.printStackTrace();
		}

		return diplomaIssued;
	}
}
