package es.csa.fdt.cyldigital.evaluacion.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.annotations.Cascade;
import org.hibernate.annotations.IndexColumn;
import org.hibernate.envers.Audited;
import org.hibernate.validator.Digits;
import org.hibernate.validator.Min;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.language.entity.ExamenLanguage;

/**
 * Clase Formulario.
 * 
 * @author csa
 */
@Entity
@Table(name = "examenes")
@AttributeOverride(name = "id", column = @Column(name = "idExamen"))
@Audited
public class Examen extends AbstractEntity implements Serializable {

	private static final long serialVersionUID = 50516757113484534L;

	private List<ExamenLanguage> textos;

	private List<ItemExamen> items;
	
	private Integer numPreguntasMostrar;
	
	private Integer numPreguntasAprobado;
	
	/**
	 * Construye un nuevo objeto de la clase
	 */
	public Examen() {
		super();
		
		setNumPreguntasMostrar(1);
		setNumPreguntasAprobado(1);
	}

	/**
	 * Devuelve items.
	 * 
	 * @return items
	 */
	@OneToMany(cascade = CascadeType.ALL)
	@JoinColumn(name = "idExamen")
	@Cascade(value = org.hibernate.annotations.CascadeType.DELETE_ORPHAN)
	@IndexColumn(name = "secuencia", base = 0)
	public List<ItemExamen> getItems() {
		return items;
	}

	/**
	 * Asigna items.
	 * 
	 * @param items
	 *          items
	 */
	public void setItems(List<ItemExamen> items) {
		this.items = items; 
	}

	/**
	 * Devuelve textos del exámen
	 * 
	 * @return textos
	 */
	@OneToMany(cascade = CascadeType.ALL)
	@JoinColumn(name = "idExamen")
	public List<ExamenLanguage> getTextos() {
		return textos;
	}

	/**
	 * Asigna los textos al exámen
	 * 
	 * @param textos
	 *          textos
	 */
	public void setTextos(List<ExamenLanguage> textos) {
		this.textos = textos;
	}
  
	/**
	 * Devuelve el número de preguntas que se muestran del total.
	 * 
	 * numPreguntasMostrar <= numero de items (de tipo pregunta) 
	 * 
	 * @return the numPreguntasMostrar
	 */
	@NotNull
	@Digits(integerDigits = 10, fractionalDigits = 0, message = "{es.fdt.validator.Digits.message}")
	@Min(value = 1, message = "{es.fdt.validator.Min.message}")
	public Integer getNumPreguntasMostrar() {
		return numPreguntasMostrar;
	}

	/**
	 * @param numPreguntasMostrar the numPreguntasMostrar to set
	 */
	public void setNumPreguntasMostrar(Integer numPreguntasMostrar) {
		this.numPreguntasMostrar = numPreguntasMostrar;
	}

	/**
	 * Devuelve el número de preguntas respondidas correctamente que suponen un aprobado
	 * 
	 * numPreguntasAprobado <= numPreguntasMostrar
	 * 
	 * @return the numPreguntasAprobado
	 */
	@NotNull
	@Digits(integerDigits = 10, fractionalDigits = 0, message = "{es.fdt.validator.Digits.message}")
	@Min(value = 1, message = "{es.fdt.validator.Min.message}")
	public Integer getNumPreguntasAprobado() {
		return numPreguntasAprobado;
	}

	/**
	 * @param numPreguntasAprobado the numPreguntasAprobado to set
	 */
	public void setNumPreguntasAprobado(Integer numPreguntasAprobado) {
		this.numPreguntasAprobado = numPreguntasAprobado;
	}

	/**
	 * 
	 */
	@Transient
	public boolean isRespondido() {
		boolean respondido = false;

		for (ItemExamen i : items) {
			if (i.isPregunta()) {
				if ((((PreguntaExamen)i).getRespuestas() != null) &&
					!((PreguntaExamen)i).getRespuestas().isEmpty()) {
					respondido = true;

					break;
				}
			}
			else {
				for (PreguntaExamen pe : ((GrupoExamen)i).getPreguntas()) {
					if ((pe.getRespuestas() != null) && !pe.getRespuestas().isEmpty()) {
						respondido = true;

						break;
					}
				}
				
				if (respondido)
					break;
			}
		}

		return respondido;
	}
}
