package es.csa.fdt.cyldigital.evaluacion.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.annotations.MapKeyManyToMany;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;

/**
 * Clase ExamenConcreto.
 * 
 * @author Fundetec
 */
@Entity
@Table(name = "examenesconcretos")
@AttributeOverride(name = "id", column = @Column(name = "idExamenConcreto"))
@Audited
public class ExamenConcreto extends AbstractEntity implements Serializable {

	private static final long serialVersionUID = -7329760284004281362L;

	private Map<PreguntaExamen, RespuestaExamen> respuestas;

	/**
	 * Construye un nuevo objeto de la clase
	 */
	public ExamenConcreto() {
		super();

		respuestas = new HashMap<PreguntaExamen, RespuestaExamen>();
	}

	/**
	 * Devuelve respuestas.
	 * 
	 * @return respuestas
	 */
	@NotAudited
	@ManyToMany(targetEntity = RespuestaExamen.class, cascade = { javax.persistence.CascadeType.ALL })
	@JoinTable(name = "examenesconcretos_respuestas", joinColumns = @JoinColumn(name = "idExamenConcreto"), inverseJoinColumns = @JoinColumn(name = "idRespuestaExamen"))
	@MapKeyManyToMany(joinColumns = @JoinColumn(name = "idPreguntaExamen"))
	public Map<PreguntaExamen, RespuestaExamen> getRespuestas() {
		return respuestas;
	}

	/**
	 * Asigna respuestas.
	 * 
	 * @param respuestas
	 *           respuestas
	 */
	public void setRespuestas(Map<PreguntaExamen, RespuestaExamen> respuestas) {
		this.respuestas = respuestas;
	}
	
	/**
	 * 
	 */
	@Transient
	public List<PreguntaExamen> getPreguntas() {
		return new ArrayList<PreguntaExamen>(respuestas.keySet());
	}
	
	@Transient
	public Float getNumRespuestasCorrectas() {
		Float numRespuestasCorrectas = 0F;

		for(RespuestaExamen re : respuestas.values()) {
			if ((re.getOpcionRespondida() != null) && (re.getPreguntaExamen() != null))
				if (re.getOpcionRespondida().equals(re.getPreguntaExamen().getOpcionCorrecta()))
					numRespuestasCorrectas++;
		}

		return numRespuestasCorrectas;
	}
}
