package es.csa.fdt.cyldigital.evaluacion.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.annotations.Cascade;
import org.hibernate.annotations.IndexColumn;
import org.hibernate.envers.Audited;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.language.entity.FormularioLanguage;

/**
 * Clase Formulario.
 * 
 * @author csa
 */
@Entity
@Table(name = "formularios")
@AttributeOverride(name = "id", column = @Column(name = "idFormulario"))
@Audited
public class Formulario extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = 50516757113484534L;

  //private String nombre;

  //private String descripcion;

  //private String encabezado;

  //private String pie;
  
  private List<FormularioLanguage> textos;

  private List<Item> items;

  /**
   * Construye un nuevo objeto de la clase
   */
  public Formulario() {
    super();
  }

  /**
   * Devuelve nombre.
   * 
   * @return nombre
   */
/*  @NotNull
  @NotEmpty
  @Length(max = 100)
  public String getNombre() {
    return nombre;
  }*/

  /**
   * Asigna nombre.
   * 
   * @param nombre
   *          nombre
   */
  /*public void setNombre(String nombre) {
    this.nombre = nombre;
  }*/

  /**
   * Devuelve descripcion.
   * 
   * @return descripcion
   */
 /* @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getDescripcion() {
    return descripcion;
  }*/

  /**
   * Asigna descripcion.
   * 
   * @param descripcion
   *          descripcion
   */
  /*public void setDescripcion(String descripcion) {
    this.descripcion = descripcion;
  }*/

  /**
   * Devuelve items.
   * 
   * @return items
   */
  @OneToMany(cascade = CascadeType.ALL)
  @JoinColumn(name = "idFormulario")
  @Cascade(value = org.hibernate.annotations.CascadeType.DELETE_ORPHAN)
  @IndexColumn(name = "secuencia", base = 0)
  public List<Item> getItems() {
    return items;
  }

  /**
   * Asigna items.
   * 
   * @param items
   *          items
   */
  public void setItems(List<Item> items) {
    this.items = items;
  }

  /**
   * Devuelve encabezado.
   * 
   * @return encabezado
   */
  /*@Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getEncabezado() {
    return encabezado;
  }*/

  /**
   * Asigna encabezado.
   * 
   * @param encabezado
   *          encabezado
   */
  /*public void setEncabezado(String encabezado) {
    this.encabezado = encabezado;
  }*/

  /**
   * Devuelve pie.
   * 
   * @return pie
   */
  /*@Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getPie() {
    return pie;
  }*/

  /**
   * Asigna pie.
   * 
   * @param pie
   *          pie
   */
  /*public void setPie(String pie) {
    this.pie = pie;
  }*/
  
  /**
   * Devuelve textos del formulario
   * 
   * @return textos
   */
  @OneToMany(cascade = CascadeType.ALL)
  @JoinColumn(name = "idFormulario")
  public List<FormularioLanguage> getTextos() {
	  return textos;
  }

  /**
   * Asigna los textos al formulario
   * 
   * @param textos
   *          textos
   */
  public void setTextos(List<FormularioLanguage> textos) {
	this.textos = textos;
  }
  
  /**
   * 
   */
  @Transient
  public boolean isRespondido() {
	  boolean respondido = false;

	  for (Item i : items) {
		  if (i.isPregunta()) {
			  if ((((Pregunta)i).getRespuestas() != null) && !((Pregunta)i).getRespuestas().isEmpty()) {
				  respondido = true;

				  break;
			  }
		  }
		  else {
			  for(Pregunta p : ((Grupo)i).getPreguntas()) {
				  if ((p.getRespuestas() != null) && !p.getRespuestas().isEmpty()) {
					  respondido = true;

					  break;
				  }
			  }
			  
			  if (respondido)
				  break;
		  }
	  }

	  return respondido;
  }
}
