package es.csa.fdt.cyldigital.evaluacion.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.language.entity.ItemLanguage;

/**
 * Clase Item.
 * 
 * @author csa
 */
@Entity
@Table(name = "items")
@Audited
@Inheritance(strategy = InheritanceType.JOINED)
@AttributeOverride(name = "id", column = @Column(name = "idItem"))
public abstract class Item extends AbstractEntity {

  private static final long serialVersionUID = 1L;

  //private String nombre;
  //private String etiqueta;
  //private String descripcion;
  
  private List<ItemLanguage> textos;
  
  private Formulario formulario;

  /**
   * Construye un nuevo objeto de la clase
   */
  public Item() {
    super();
  }

  /**
   * Asigna nombre.
   * 
   * @param nombre
   *          nombre
   */
  /*public void setNombre(String nombre) {
    this.nombre = nombre;
  }*/

  /**
   * Devuelve nombre.
   * 
   * @return nombre
   */
  /*@Length(max = 150)
  @NotEmpty
  @NotNull
  public String getNombre() {
    return nombre;
  }*/

  /**
   * Devuelve formulario.
   * 
   * @return formulario
   */
  @ManyToOne(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
  @JoinColumn(name = "idFormulario")
  public Formulario getFormulario() {
    return formulario;
  }

  /**
   * Asigna formulario.
   * 
   * @param formulario
   *          formulario
   */
  public void setFormulario(Formulario formulario) {
    this.formulario = formulario;
  }

  /**
   * Asigna etiqueta.
   * 
   * @param etiqueta
   *          etiqueta
   */
 /* public void setEtiqueta(String etiqueta) {
    this.etiqueta = etiqueta;
  }*/

  /**
   * Devuelve etiqueta.
   * 
   * @return etiqueta
   */
 /* @Length(max = 150)
  @NotEmpty
  @NotNull
  public String getEtiqueta() {
    return etiqueta;
  }*/

  /**
   * Asigna descripcion.
   * 
   * @param descripcion
   *          descripcion
   */
  /*public void setDescripcion(String descripcion) {
    this.descripcion = descripcion;
  }*/

  /**
   * Devuelve descripcion.
   * 
   * @return descripcion
   */
  /*public String getDescripcion() {
    return descripcion;
  }*/
  
  /**
   * Devuelve textos del item
   * 
   * @return textos
   */
  @OneToMany(cascade = CascadeType.ALL)
  @JoinColumn(name = "idItem")
  public List<ItemLanguage> getTextos() {
	  return textos;
  }

  /**
   * Asigna los textos al item
   * 
   * @param textos
   *          textos
   */
  public void setTextos(List<ItemLanguage> textos) {
	this.textos = textos;
  }

  /**
   * Devuelve secuencia.
   * 
   * @return secuencia
   */
  @Transient
  public int getSecuencia() {
    if (getFormulario() != null && getFormulario().getItems() != null) { return getFormulario().getItems().indexOf(this); }
    return -1;
  }

  /**
   * Asigna secuencia.
   * 
   * @param secuencia
   *          secuencia
   */
  public void setSecuencia(int secuencia) {}

  /**
   * Checks if is pregunta.
   * 
   * @return true, if is pregunta
   */
  @Transient
  public abstract boolean isPregunta();

  /**
   * Checks if is grupo.
   * 
   * @return true, if is grupo
   */
  @Transient
  public abstract boolean isGrupo();
}
