package es.csa.fdt.cyldigital.evaluacion.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.language.entity.ItemExamenLanguage;

/**
 * Clase Item.
 * 
 * @author Fundetec
 */
@Entity
@Table(name = "itemsexamenes")
@Audited
@Inheritance(strategy = InheritanceType.JOINED)
@AttributeOverride(name = "id", column = @Column(name = "idItemExamen"))
public abstract class ItemExamen extends AbstractEntity {

	private static final long serialVersionUID = -341949460315246212L;

	private Examen examen;
	
	private List<ItemExamenLanguage> textos;

	/**
	 * Construye un nuevo objeto de la clase
	 */
	public ItemExamen() {
		super();
	}

	/**
	 * Devuelve examen.
	 * 
	 * @return examen
	 */
	@ManyToOne(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
	@JoinColumn(name = "idExamen")
	public Examen getExamen() {
		return examen;
	}

	/**
	 * Asigna examen.
	 * 
	 * @param examen
	 *            examen
	 */
	public void setExamen(Examen examen) {
		this.examen = examen;
	}

	/**
	 * Devuelve textos del item de examen
	 * 
	 * @return textos
	 */
	@OneToMany(cascade = CascadeType.ALL)
	@JoinColumn(name = "idItemExamen")
	public List<ItemExamenLanguage> getTextos() {
		return textos;
	}

	/**
	 * Asigna los textos al item de examen
	 * 
	 * @param textos
	 *            textos
	 */
	public void setTextos(List<ItemExamenLanguage> textos) {
		this.textos = textos;
	}

	/**
	 * Devuelve secuencia.
	 * 
	 * @return secuencia
	 */
	@Transient
	public int getSecuencia() {
		if (getExamen() != null && getExamen().getItems() != null)
			return getExamen().getItems().indexOf(this);

		return -1;
	}

	/**
	 * Asigna secuencia.
	 * 
	 * @param secuencia
	 *            secuencia
	 */
	public void setSecuencia(int secuencia) {
	}

	/**
	 * Checks if is pregunta of exam.
	 * 
	 * @return true, if is pregunta of exam
	 */
	@Transient
	public abstract boolean isPregunta();

	/**
	 * Checks if is grupo of exam.
	 * 
	 * @return true, if is grupo of exam
	 */
	@Transient
	public abstract boolean isGrupo();
}
