package es.csa.fdt.cyldigital.evaluacion.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.springframework.context.i18n.LocaleContextHolder;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.language.entity.OpcionPreguntaExamenLanguage;

/**
 * Clase PreguntaExamen.
 * 
 * @author Fundetec
 */
@Entity
@Table(name = "opcionespreguntasexamenes")
@AttributeOverride(name = "id", column = @Column(name = "idOpcionPreguntaExamen"))
@Audited
public class OpcionPreguntaExamen extends AbstractEntity implements Serializable {
	private static final long serialVersionUID = 4619021576078593454L;
	
	private PreguntaExamen preguntaExamen;
	
	private List<OpcionPreguntaExamenLanguage> textos;

	/**
	 * Construye un nuevo objeto de la clase
	 */
	public OpcionPreguntaExamen() {
		super();
	}
	
	/**
	 * @return the preguntaExamen
	 */
	@OneToOne(mappedBy = "opcionCorrecta")
	@ManyToOne(fetch = FetchType.EAGER, cascade = {CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH})
	@JoinColumn(name = "idPreguntaExamen", insertable = false, updatable = false)
	public PreguntaExamen getPreguntaExamen() {
		return preguntaExamen;
	}

	/**
	 * @param preguntaExamen the preguntaExamen to set
	 */
	public void setPreguntaExamen(PreguntaExamen preguntaExamen) {
		this.preguntaExamen = preguntaExamen;
	}

	/**
	 * @return the textos
	 */
	@OneToMany(cascade = CascadeType.ALL)
	@JoinColumn(name = "idOpcionPreguntaExamen")
	public List<OpcionPreguntaExamenLanguage> getTextos() {
		return textos;
	}

	/**
	 * @param textos the textos to set
	 */
	public void setTextos(List<OpcionPreguntaExamenLanguage> textos) {
		this.textos = textos;
	}
	
	@Transient
	public String getTextoOpcion() {
		String idioma = LocaleContextHolder.getLocaleContext().getLocale().toString();
		
		for (OpcionPreguntaExamenLanguage opel : textos)
			if (opel.getLanguage().getISO().contentEquals(idioma))
				return opel.getTextoOpcion();
		
		return "";
	}
	
	/**
	 * Devuelve secuencia dentro de pregunta de examen.
	 * 
	 * @return secuencia dentro de pregunta de examen
	 */
	@Transient
	public int getSecuenciaPregunta() {
		if (getPreguntaExamen() != null && getPreguntaExamen().getOpciones() != null)
			return getPreguntaExamen().getOpciones().indexOf(this);
		
		return -1;
	}

	/**
	 * Asigna secuencia dentro de pregunta de examen.
	 * 
	 * @param secuencia
	 *          secuencia dentro de pregunta de examen
	 */
	public void setSecuenciaPregunta(int secuencia) {}
}
