package es.csa.fdt.cyldigital.evaluacion.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Digits;
import org.hibernate.validator.NotNull;

/**
 * Clase Pregunta.
 * 
 * @author csa
 */
@Entity
@Table(name = "preguntas")
@AttributeOverride(name = "id", column = @Column(name = "idPregunta"))
@PrimaryKeyJoinColumn(name = "idItem")
@Audited
public class Pregunta extends Item implements Serializable {

  private static final long serialVersionUID = 6497519888332638645L;

  /**
   * Enum TipoPregunta.
   */
  public static enum TipoPregunta {
    ENTERO,
    DECIMAL,
    BOOLEANO,
    TEXTO,
    CADENA
  }

  private TipoPregunta tipo;
  private Integer minimo;
  private Integer maximo;
  private Float minimoDecimal;
  private Float maximoDecimal;
  private Boolean obligatorio;
  private List<Respuesta> respuestas;
  private Grupo grupo;

  /**
   * Construye un nuevo objeto de la clase
   */
  public Pregunta() {
    super();
    setObligatorio(false);
    setTipo(TipoPregunta.ENTERO);
    setMinimo(1);
    setMinimoDecimal(1F);
    setMaximo(5);
    setMaximoDecimal(5F);
  }

  /**
   * Devuelve tipo.
   * 
   * @return tipo
   */
  @Enumerated(value = EnumType.STRING)
  @NotNull
  public TipoPregunta getTipo() {
    return tipo;
  }

  /**
   * Asigna tipo.
   * 
   * @param tipo
   *          tipo
   */
  public void setTipo(TipoPregunta tipo) {
    this.tipo = tipo;
  }

  /**
   * Devuelve minimo.
   * 
   * @return minimo
   */
  @Digits(integerDigits = 10, fractionalDigits = 0)
  public Integer getMinimo() {
    return minimo;
  }

  /**
   * Asigna minimo.
   * 
   * @param minimo
   *          minimo
   */
  public void setMinimo(Integer minimo) {
    this.minimo = minimo;
  }

  /**
   * Devuelve maximo.
   * 
   * @return maximo
   */
  @Digits(integerDigits = 10, fractionalDigits = 0)
  public Integer getMaximo() {
    return maximo;
  }

  /**
   * Asigna maximo.
   * 
   * @param maximo
   *          maximo
   */
  public void setMaximo(Integer maximo) {
    this.maximo = maximo;
  }

  /**
   * Devuelve minimo decimal.
   * 
   * @return minimo decimal
   */
  @Digits(integerDigits = 10, fractionalDigits = 2)
  public Float getMinimoDecimal() {
    return minimoDecimal;
  }

  /**
   * Asigna minimo decimal.
   * 
   * @param minimoDecimal
   *          minimo decimal
   */
  public void setMinimoDecimal(Float minimoDecimal) {
    this.minimoDecimal = minimoDecimal;
  }

  /**
   * Devuelve maximo decimal.
   * 
   * @return maximo decimal
   */
  @Digits(integerDigits = 10, fractionalDigits = 2)
  public Float getMaximoDecimal() {
    return maximoDecimal;
  }

  /**
   * Asigna maximo decimal.
   * 
   * @param maximoDecimal
   *          maximo decimal
   */
  public void setMaximoDecimal(Float maximoDecimal) {
    this.maximoDecimal = maximoDecimal;
  }

  /**
   * Devuelve obligatorio.
   * 
   * @return obligatorio
   */
  public Boolean getObligatorio() {
    return obligatorio;
  }

  /**
   * Asigna obligatorio.
   * 
   * @param obligatorio
   *          obligatorio
   */
  public void setObligatorio(Boolean obligatorio) {
    this.obligatorio = obligatorio;
  }

  /**
   * Asigna respuestas.
   * 
   * @param respuestas
   *          respuestas
   */
  public void setRespuestas(List<Respuesta> respuestas) {
    this.respuestas = respuestas;
  }

  /**
   * Devuelve respuestas.
   * 
   * @return respuestas
   */
  @OneToMany(mappedBy = "pregunta")
  public List<Respuesta> getRespuestas() {
    return respuestas;
  }

  /**
   * Devuelve grupo.
   * 
   * @return grupo
   */
  @ManyToOne(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH }, fetch = FetchType.EAGER)
  @JoinColumn(name = "idGrupoPregunta")
  public Grupo getGrupo() {
    return grupo;
  }

  /**
   * Asigna grupo.
   * 
   * @param grupo
   *          grupo
   */
  public void setGrupo(Grupo grupo) {
    this.grupo = grupo;
  }

  /**
   * Devuelve secuencia grupo.
   * 
   * @return secuencia grupo
   */
  @Transient
  public int getSecuenciaGrupo() {
    if (getGrupo() != null && getGrupo().getPreguntas() != null) { return getGrupo().getPreguntas().indexOf(this); }
    return -1;
  }

  /**
   * Asigna secuencia grupo.
   * 
   * @param secuencia
   *          secuencia grupo
   */
  public void setSecuenciaGrupo(int secuencia) {}

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.evaluacion.entity.Item#isGrupo()
   */
  @Override
  @Transient
  public boolean isGrupo() {
    return false;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.evaluacion.entity.Item#isPregunta()
   */
  @Override
  @Transient
  public boolean isPregunta() {
    return true;
  }

}
