package es.csa.fdt.cyldigital.evaluacion.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.annotations.Cascade;
import org.hibernate.annotations.IndexColumn;
import org.hibernate.envers.Audited;

/**
 * Clase PreguntaExamen.
 * 
 * @author Fundetec
 */
@Entity
@Table(name = "preguntasexamenes")
@AttributeOverride(name = "id", column = @Column(name = "idPreguntaExamen"))
@PrimaryKeyJoinColumn(name = "idPreguntaExamen", referencedColumnName="idItemExamen")
@Audited
public class PreguntaExamen extends ItemExamen implements Serializable {
	private static final long serialVersionUID = -9213237378728308255L;

	private List<OpcionPreguntaExamen> opciones;

	private List<RespuestaExamen> respuestas;

	private GrupoExamen grupoExamen;

	private OpcionPreguntaExamen opcionCorrecta;

	/**
	 * Construye un nuevo objeto de la clase
	 */
	public PreguntaExamen() {
		super();
	}
	
	/**
	 * @return the opciones
	 */
	@OneToMany(cascade = CascadeType.ALL)
	@JoinColumn(name = "idPreguntaExamen", nullable = false)
	@Cascade(value = org.hibernate.annotations.CascadeType.DELETE_ORPHAN)
	@IndexColumn(name = "secuenciaPregunta", base = 0)
	public List<OpcionPreguntaExamen> getOpciones() {
		return opciones;
	}

	/**
	 * @param opciones the opciones to set
	 */
	public void setOpciones(List<OpcionPreguntaExamen> opciones) {
		this.opciones = opciones;
	}

	/**
	 * Devuelve respuestas de examen.
	 * 
	 * @return respuestas
	 */
	@OneToMany(mappedBy = "preguntaExamen")
	public List<RespuestaExamen> getRespuestas() {
		return respuestas;
	}
	
	/**
	 * Asigna respuestas de examen.
	 * 
	 * @param respuestas
	 *          respuestas de examen
	 */
	public void setRespuestas(List<RespuestaExamen> respuestas) {
		this.respuestas = respuestas;
	}

	/**
	 * Devuelve grupo de examen.
	 * 
	 * @return grupoExamen
	 */
	@ManyToOne(fetch = FetchType.EAGER, cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
	@JoinColumn(name = "idGrupoExamen")
	public GrupoExamen getGrupoExamen() {
		return grupoExamen;
	}

	/**
	 * Asigna grupo de examen.
	 * 
	 * @param grupoExamen
	 *          grupo de examen
	 */
	public void setGrupoExamen(GrupoExamen grupoExamen) {
		this.grupoExamen = grupoExamen;
	}
	
	/**
	 * @return the opcionCorrecta
	 */
	@OneToOne(cascade = CascadeType.ALL)
	@JoinColumn(name = "idOpcionCorrecta")
	public OpcionPreguntaExamen getOpcionCorrecta() {
		return opcionCorrecta;
	}

	/**
	 * @param opcionCorrecta the opcionCorrecta to set
	 */
	public void setOpcionCorrecta(OpcionPreguntaExamen opcionCorrecta) {
		this.opcionCorrecta = opcionCorrecta;
	}

	/**
	 * Devuelve secuencia grupo de examen.
	 * 
	 * @return secuencia grupo de examen
	 */
	@Transient
	public int getSecuenciaGrupo() {
		if (getGrupoExamen() != null && getGrupoExamen().getPreguntas() != null)
			return getGrupoExamen().getPreguntas().indexOf(this);
		
		return -1;
	}

	/**
	 * Asigna secuencia grupo de examen.
	 * 
	 * @param secuencia
	 *          secuencia grupo de examen
	 */
	public void setSecuenciaGrupo(int secuencia) {}
	
	/* (non-Javadoc)
	 * @see es.csa.fdt.cyldigital.evaluacion.entity.Item#isGrupo()
	 */
	@Override
	@Transient
	public boolean isGrupo() {
		return false;
	}

	/* (non-Javadoc)
	 * @see es.csa.fdt.cyldigital.evaluacion.entity.Item#isPregunta()
	 */
	@Override
	@Transient
	public boolean isPregunta() {
		return true;
	}
}
