package es.csa.fdt.cyldigital.evaluacion.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Digits;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;

/**
 * Clase Respuesta.
 * 
 * @author csa
 */
@Entity
@Audited
@Table(name = "respuestas")
@AttributeOverride(name = "id", column = @Column(name = "idRespuesta"))
public class Respuesta extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = 50516757113489734L;

  private Pregunta pregunta;

  private Boolean valorBoleano;

  private Float valorDecimal;

  private Integer valorEntero;

  private String valorCadena;

  private String valorTexto;

  /**
   * Construye un nuevo objeto de la clase
   */
  public Respuesta() {
    super();
  }

  /**
   * Devuelve pregunta.
   * 
   * @return pregunta
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.PERSIST, CascadeType.MERGE, CascadeType.REFRESH })
  @JoinColumn(name = "idPregunta")
  @NotNull
  public Pregunta getPregunta() {
    return pregunta;
  }

  /**
   * Asigna pregunta.
   * 
   * @param pregunta
   *          pregunta
   */
  public void setPregunta(Pregunta pregunta) {
    this.pregunta = pregunta;
  }

  /**
   * Devuelve valor boleano.
   * 
   * @return valor boleano
   */
  public Boolean getValorBoleano() {
    return valorBoleano;
  }

  /**
   * Asigna valor boleano.
   * 
   * @param valorBoleano
   *          valor boleano
   */
  public void setValorBoleano(Boolean valorBoleano) {
    this.valorBoleano = valorBoleano;
  }

  /**
   * Devuelve valor decimal.
   * 
   * @return valor decimal
   */
  @Digits(integerDigits = 10, fractionalDigits = 4)
  public Float getValorDecimal() {
    return valorDecimal;
  }

  /**
   * Asigna valor decimal.
   * 
   * @param valorDecimal
   *          valor decimal
   */
  public void setValorDecimal(Float valorDecimal) {
    this.valorDecimal = valorDecimal;
  }

  /**
   * Devuelve valor entero.
   * 
   * @return valor entero
   */
  @Digits(integerDigits = 10, fractionalDigits = 0)
  public Integer getValorEntero() {
    return valorEntero;
  }

  /**
   * Asigna valor entero.
   * 
   * @param valorEntero
   *          valor entero
   */
  public void setValorEntero(Integer valorEntero) {
    this.valorEntero = valorEntero;
  }

  /**
   * Devuelve valor cadena.
   * 
   * @return valor cadena
   */
  @Length(max = 255)
  public String getValorCadena() {
    return valorCadena;
  }

  /**
   * Asigna valor cadena.
   * 
   * @param valorCadena
   *          valor cadena
   */
  public void setValorCadena(String valorCadena) {
    this.valorCadena = valorCadena;
  }

  /**
   * Devuelve valor texto.
   * 
   * @return valor texto
   */
  @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getValorTexto() {
    return valorTexto;
  }

  /**
   * Asigna valor texto.
   * 
   * @param valorTexto
   *          valor texto
   */
  public void setValorTexto(String valorTexto) {
    this.valorTexto = valorTexto;
  }

  /**
   * Devuelve valor.
   * 
   * @return valor
   */
  @Transient
  public Object getValor() {
    if (getPregunta() != null && getPregunta().getTipo() != null) {
      switch (getPregunta().getTipo()) {
        case ENTERO:
          return getValorEntero();
        case DECIMAL:
          return getValorDecimal();
        case BOOLEANO:
          return getValorBoleano();
        case CADENA:
          return getValorCadena();
        case TEXTO:
          return getValorTexto();
      }
    }
    return null;
  }

  /**
   * Asigna valor.
   * 
   * @param valor
   *          valor
   */
  public void setValor(Object valor) {
    if (getPregunta() != null && getPregunta().getTipo() != null) {
      switch (getPregunta().getTipo()) {
        case ENTERO:
          setValorEntero((Integer) valor);
        case DECIMAL:
          setValorDecimal((Float) valor);
        case BOOLEANO:
          setValorBoleano((Boolean) valor);
        case CADENA:
          setValorCadena((String) valor);
        case TEXTO:
          setValorTexto((String) valor);
      }
    }
  }

}
