package es.csa.fdt.cyldigital.evaluacion.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;

import javax.faces.model.SelectItem;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;

import es.csa.fdt.cyldigital.evaluacion.entity.Examen;
import es.csa.fdt.cyldigital.evaluacion.entity.ExamenConcreto;
import es.csa.fdt.cyldigital.evaluacion.entity.GrupoExamen;
import es.csa.fdt.cyldigital.evaluacion.entity.ItemExamen;
import es.csa.fdt.cyldigital.evaluacion.entity.PreguntaExamen;
import es.csa.fdt.cyldigital.evaluacion.entity.RespuestaExamen;

import es.csa.fdt.cyldigital.language.entity.Language;
import es.csa.fdt.cyldigital.language.entity.ExamenLanguage;

import es.csa.fdt.cyldigital.servicios.entity.Curso;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso;
import es.csa.fdt.cyldigital.servicios.entity.ServicioCurso;

/**
 * Clase ExamenServiceImpl.
 * 
 * @author Fundetec
 */
@Service
@Repository
public class ExamenServiceImpl extends AbstractServiceImpl<Examen, Long> implements ExamenService {

	private static final long serialVersionUID = 5226104692243750130L;
	
	private static final Logger logger = LoggerFactory.getLogger(ExamenServiceImpl.class);
	
	//Alternative exams for select on remove
	private List<Examen> alternativeExams;
	
	//Alternative exams select items for select on remove
	private List<SelectItem> alternativeExamsSelectItems;
	
	//Services with editing exam assigned
	private List<ServicioCurso> servicesAssociated;
	
	/**
	 * Construye un nuevo objeto de la clase
	 */
	public ExamenServiceImpl() {
		super();
		
		alternativeExams = new ArrayList<Examen>();
		
		alternativeExamsSelectItems = new ArrayList<SelectItem>();
		
		servicesAssociated = new ArrayList<ServicioCurso>();
	}
  
	/**
	 * @return the alternativeExams
	 */
	public List<Examen> getAlternativeExams() {
		return alternativeExams;
	}

	/**
	 * @return the alternativeExamsSelectItems
	 */
	public List<SelectItem> getAlternativeExamsSelectItems() {
		return alternativeExamsSelectItems;
	}

	/**
	 * @return the servicesAssociated
	 */
	public List<ServicioCurso> getServicesAssociated() {
		return servicesAssociated;
	}

	/* (non-Javadoc)
	 * @see es.csa.fdt.cyldigital.evaluacion.service.ExamenService#createLanguages(es.csa.fdt.cyldigital.evaluacion.entity.Examen, java.util.List)
	 */
	public void createLanguages(Examen examen, List<Language> languages) {
		List<ExamenLanguage> textos = new ArrayList<ExamenLanguage>();

		for (Language language : languages) {
			ExamenLanguage texto = new ExamenLanguage();
			
			texto.setLanguage(language);
			texto.setExamen(examen);

			textos.add(texto);
		}

		examen.setTextos(textos);
	}

	/**
	 * @param examen
	 * @return
	 */
	public Boolean persist(Examen examen) {
		if (examen.getNumPreguntasAprobado() <= examen.getNumPreguntasMostrar()) {
			super.persist(examen);

			return true;
		}
		else {
			addErrorMessage("examenes.errorPassNumLTShowNum");

			return false;
		}
	}
	
	/**
	 * Prepara una serie de listas para realizar el borrado correctamente
	 * 
	 */
	public void prepareToRemoveExam(Long idExamToRemove) {
		if (alternativeExams == null)
			alternativeExams = new ArrayList<Examen>();
		
		if (!alternativeExams.isEmpty())
			alternativeExams.clear();
		
		if (alternativeExamsSelectItems == null)
			alternativeExamsSelectItems = new ArrayList<SelectItem>();
		
		if (!alternativeExamsSelectItems.isEmpty())
			alternativeExamsSelectItems.clear();
		
		if (servicesAssociated != null) {
			servicesAssociated.clear();
			servicesAssociated = null;
		}

		servicesAssociated = findWhere(ServicioCurso.class, "examen.id", idExamToRemove);
		
		if ((servicesAssociated != null) && (servicesAssociated.size() > 0)) {
			String idioma = LocaleContextHolder.getLocaleContext().getLocale().toString();
			
			Map<String, Object> properties = new HashMap<String, Object>();
			properties.put("examen.desactivado", false);
			properties.put("language.ISO", idioma);
			
			try {
				List<ExamenLanguage> lel = findWhereOrder(ExamenLanguage.class, properties, "nombre", true);
				
				for (ExamenLanguage el : lel) {
					if ((getNumQuestionsOfExamen(el.getExamen()) >= el.getExamen().getNumPreguntasMostrar()) &&
						(el.getExamen().getId() != idExamToRemove)) {
						alternativeExams.add(el.getExamen());
						alternativeExamsSelectItems.add(new SelectItem(el.getExamen(), el.getNombre()));
					}
				}
			}
			catch (Exception e) {
				alternativeExams.clear();
				alternativeExamsSelectItems.clear();
				
				logger.info("ExamenServiceImpl.prepareToRemoveExam: " +
							"Error al conformar las listas de examenes en formato SelectItems. " +
							e.getLocalizedMessage());
			}
		}
	}

	/* (non-Javadoc)
	 * @see es.csa.fdt.cyldigital.evaluacion.service.ExamenService#removeExamen(es.csa.fdt.cyldigital.evaluacion.entity.Examen,
	 * 																			es.csa.fdt.cyldigital.evaluacion.entity.Examen)
	 */
	public boolean removeExamen(Examen examen, Examen examenAlternativo) {
		boolean removed = false;

		if ((examen != null) && (examen.getId() != 0)) {
			//Get list of services with examen associated.
			//List<ServicioCurso> servicesAssociated = findWhere(ServicioCurso.class, "examen.id", examen.getId());

			if (servicesAssociated != null) {
				if ((examenAlternativo != null) && (examenAlternativo.getId() != 0)){
					//Set examenAlternativo to services with examen associated.
					disableMessages();
					for(ServicioCurso sc : servicesAssociated) {
						sc.setExamen(examenAlternativo);
		
						persist(sc);
					}
					disableMessages();
				}
				else {
					addErrorMessage("Se produjo un error eliminado el examen");
					
					return false;
				}
			}

			//If exam is answered...
			if (examen.isRespondido()) {
				//Set examen to not activated.
				disableMessages();
				for (ExamenLanguage el : examen.getTextos()) {
					el.setDesactivado(true);
					
					persist(el);
				}
				enableMessages();
				
				examen.setDesactivado(true);

				persist(examen);
			}
			else
				//Remove exam.
				remove(examen);

			removed = true;
		}

		return removed;
	}
	
	private int getNumQuestionsOfExamen(Examen examen) {
		int numPreguntas = 0;
		
		for (ItemExamen ie : examen.getItems()) {
			if (ie instanceof PreguntaExamen)
				numPreguntas++;
			else
				numPreguntas += ((GrupoExamen)ie).getPreguntas().size();
		}
		
		return numPreguntas;
	}
	
	/**
	 * Asocia el examen concreto a la matrícula y manda a hibernate guardar la matrícula
	 * 
	 * @param idExamen
	 * 		  Id de la plantilla de examen que se ha utilizado para hacer el examen concreto.
	 * 
	 * @param idExamenConcreto
	 * 		  Id del examen concreto realizado por un usuario alumno.
	 * 
	 * 
	 * @param idMatriculaCurso
	 * 		  Id de la matrícula del usuario alumno en el curso
	 * 
	 * 
	 * @param initialExam
	 * 		  Booleano que indica si el examen concreto que se ha realizado es inicial (true) o final (false).
	 */
	public boolean persistExamenConcretoInMatriculaCurso(Long idExamen, Long idExamenConcreto,
														 Long idMatriculaCurso, boolean initialExam) {
		boolean persitedOK = false;
		
		if ((idExamen != null) && (idExamen > 0) &&
			(idExamenConcreto != null) && (idExamenConcreto > 0) &&
			(idMatriculaCurso != null) && (idMatriculaCurso > 0)) {
			//Extrae la plantilla del examen realizado.
			Examen examen = find(Examen.class, idExamen);
			//Exatrae la matrícula en la actividad del usuario alumno que ha realizado el examen
			MatriculaCurso mc = find(MatriculaCurso.class, idMatriculaCurso);
			//Extrae el examen concreto realizado por el usuario alumno
			ExamenConcreto ec = find(ExamenConcreto.class, idExamenConcreto);
			
			Curso curso = null;
			if ((examen != null) && (mc != null) && (ec != null)) {
				//Extrae el curso del que se ha examinado el usuario alumno
				curso = find(Curso.class, mc.getCurso().getId());

				if (curso != null) {
					if (initialExam)
						//Asigna el examen concreto a la matrícula
						mc.setExamenInicial(ec);
					else {
						//Asigna el examen concreto a la matrícula
						mc.setExamenFinal(ec);
						
						//Establece en la matrícula la nota obtenida obtenida por el usurio en el curso  
						mc.setNota(ec.getNumRespuestasCorrectas());
					}
					
					//Si es el primer examen del curso...
					if (curso.getExamen() == null) {
						//Asignamos la plantilla al curso para que el resto
						//de exámenes se realicen en base a la misma plantilla.
						//
						//Sirve para prevenir que si se cambia, borra o desactiva la plantilla
						//del examen una vez realizado el primer examen del curso no se
						//realicen exámenes con diferentes plantillas a los alumnos. 
						curso.setExamen(examen);
						
						disableMessages();
						persist(curso);
						enableMessages();
					}

					persist(mc);

					persitedOK = true;
				}
			}
		}
		
		return persitedOK;
	}
	
	//RF_XEA_15 Se añade una funcion que guarda el examen de nivel realizado por un alumno para una matricula en concreto
	public boolean persistExamenConcretoNivelInMatriculaCurso(Long idExamen, Long idExamenConcreto, Long idMatriculaCurso) {
		
		boolean persitedOK = false;

		if ((idExamen != null) && (idExamen > 0) &&
			(idExamenConcreto != null) && (idExamenConcreto > 0) &&
			(idMatriculaCurso != null) && (idMatriculaCurso > 0)) {			
			//Extrae la plantilla del examen realizado.
			Examen examen = find(Examen.class, idExamen);
			//Extrae la matrícula en la actividad del usuario alumno que ha realizado el examen
			MatriculaCurso mc = find(MatriculaCurso.class, idMatriculaCurso);
			//Extrae el examen concreto realizado por el usuario alumno
			ExamenConcreto ec = find(ExamenConcreto.class, idExamenConcreto);

			Curso curso = null;
			if ((examen != null) && (mc != null) && (ec != null)) {
			//Extrae el curso del que se ha examinado el usuario alumno
				curso = find(Curso.class, mc.getCurso().getId());

				if (curso != null) {
					//Asigna el examen concreto a la matrícula
					mc.setExamenNivel(ec);
				}

				//Si es el primer examen del curso...
				if (curso.getExamenNivel() == null) {
					//Asignamos la plantilla al curso para que el resto
					//de exámenes se realicen en base a la misma plantilla.
					//
					//Sirve para prevenir que si se cambia, borra o desactiva la plantilla
					//del examen una vez realizado el primer examen del curso no se
					//realicen exámenes con diferentes plantillas a los alumnos. 
					curso.setExamenNivel(examen);
	
					disableMessages();
					persist(curso);
					enableMessages();
				}

				persist(mc);

				persitedOK = true;
			}
		}
		
		return persitedOK;
	}
	//RF_XEA_15 Se añade una funcion que guarda el examen de nivel realizado por un alumno para una matricula en concreto
}
