package es.csa.fdt.cyldigital.evaluacion.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.List;
import java.util.ArrayList;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.evaluacion.entity.Formulario;
import es.csa.fdt.cyldigital.evaluacion.entity.FormularioConcreto;
import es.csa.fdt.cyldigital.evaluacion.entity.Grupo;
import es.csa.fdt.cyldigital.evaluacion.entity.Item;
import es.csa.fdt.cyldigital.evaluacion.entity.Pregunta;
import es.csa.fdt.cyldigital.evaluacion.entity.Respuesta;

import es.csa.fdt.cyldigital.language.entity.Language;
import es.csa.fdt.cyldigital.language.entity.FormularioLanguage;


/**
 * Clase FormularioServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class FormularioServiceImpl extends AbstractServiceImpl<Formulario, Long> implements FormularioService {

  private static final long serialVersionUID = -5575228180257917674L;
  
  /**
   * Construye un nuevo objeto de la clase
   */
  public FormularioServiceImpl() {
    super();
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.evaluacion.service.FormularioService#persistFormulario(es.csa.fdt.cyldigital.evaluacion.entity.Formulario)
   */
  public void persistFormulario(Formulario formulario) {
    persist(formulario);
  }
  
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.evaluacion.service.FormularioService#removeFormulario(es.csa.fdt.cyldigital.evaluacion.entity.Formulario)
   */
  public void removeFormulario(Formulario formulario) {
    if (formulario.getId() != 0) remove(formulario);
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.evaluacion.service.FormularioService#createLanguages(es.csa.fdt.cyldigital.evaluacion.entity.Formulario)
   */
  public void createLanguages(Formulario formulario, List<Language> languages) {
	  List<FormularioLanguage> textos = new ArrayList<FormularioLanguage>();

	  for (Language language : languages) {
		  FormularioLanguage texto = new FormularioLanguage();
		  texto.setLanguage(language);
		  texto.setFormulario(formulario);

		  textos.add(texto);
	  }

	  formulario.setTextos(textos);
  }

  /**
   * Borrar item.
   * 
   * @param formulario
   *           formulario
   * @param idItem
   *           id item
   */
  /*public void borrarItem(Formulario formulario, Long idItem) {
    Item p = find(Item.class, idItem);
    boolean encontradoEnGrupo = false;
    int grupoPos = 0;

    if (p instanceof Pregunta && ((Pregunta) p).getGrupo() != null) {
      for (Item i : formulario.getItems())
        if (i instanceof Grupo) for (Pregunta pre : ((Grupo) i).getPreguntas())
          if (pre.equals(p)) {
            grupoPos = formulario.getItems().indexOf(i);
            encontradoEnGrupo = true;
          }
    } else if (p instanceof Grupo) {
      for (Pregunta pre : ((Grupo) p).getPreguntas()) {
        pre.setGrupo(null);
      }
      ((Grupo) p).getPreguntas().clear();
      formulario.getItems().remove(p);
    } else {
      formulario.getItems().remove(p);
    }

    if (encontradoEnGrupo) ((Grupo) formulario.getItems().get(grupoPos)).getPreguntas().remove(p);
  }*/

  /**
   * Subir item.
   * 
   * @param formulario
   *           formulario
   * @param idItem
   *           id item
   */
  /*public void subirItem(Formulario formulario, Long idItem) {
    Item p = find(Item.class, idItem);
    if (p instanceof Pregunta && ((Pregunta) p).getGrupo() != null) {
      int posicion = ((Pregunta) p).getSecuenciaGrupo();
      ((Pregunta) p).getGrupo().getPreguntas().remove(p);
      ((Pregunta) p).getGrupo().getPreguntas().add(posicion - 1, ((Pregunta) p));
    } else {
      int posicion = p.getSecuencia();
      formulario.getItems().remove(p);
      formulario.getItems().add(posicion - 1, p);
    }

  }*/

  /**
   * Bajar item.
   * 
   * @param formulario
   *           formulario
   * @param idItem
   *           id item
   */
  /*public void bajarItem(Formulario formulario, Long idItem) {
    Item p = find(Item.class, idItem);
    if (p instanceof Pregunta && ((Pregunta) p).getGrupo() != null) {
      int posicion = ((Pregunta) p).getSecuenciaGrupo();
      ((Pregunta) p).getGrupo().getPreguntas().remove(p);
      ((Pregunta) p).getGrupo().getPreguntas().add(posicion + 1, ((Pregunta) p));
    } else {
      int posicion = p.getSecuencia();
      formulario.getItems().remove(p);
      formulario.getItems().add(posicion + 1, p);
    }
  }*/

  /**
   * Find formulario concreto.
   * 
   * @param id
   *           id
   * @return formulario concreto
   */
  public FormularioConcreto findFormularioConcreto(Long id) {
    return find(FormularioConcreto.class, id);
  }

  /**
   * Creacion formulario valores.
   * 
   * @param formulario
   *           formulario
   * @param concreto
   *           concreto
   * @return formulario concreto
   */
  public FormularioConcreto creacionFormularioValores(Formulario formulario, FormularioConcreto concreto) {
    try {
      for (Item item : formulario.getItems()) {
        if (item.isPregunta()) {
          Respuesta respuesta = new Respuesta();
          respuesta.setPregunta((Pregunta) item);
          if (!concreto.getRespuestas().containsKey(item)) concreto.getRespuestas().put((Pregunta) item, respuesta);
        } else {
          for (Pregunta pregunta : ((Grupo) item).getPreguntas()) {
            Respuesta respuesta = new Respuesta();
            respuesta.setPregunta(pregunta);
            if (!concreto.getRespuestas().containsKey(pregunta)) concreto.getRespuestas().put(pregunta, respuesta);
          }
        }
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
    return concreto;
  }
  // public Formulario creacionFormularioValores(Formulario formulario,FormularioConcreto concreto) {
  // int numItems = 0;
  // try {
  // if (formulario.getValores().size() == 0) {
  // for (Item item : formulario.getItems()) {
  // if (item.isPregunta()) {
  // Respuesta respuesta = new Respuesta();
  // respuesta.setPregunta((Pregunta) item);
  // respuesta.setFormulario(formulario);
  // formulario.getValores().put((Pregunta) item, respuesta);
  // } else {
  // for (Pregunta pregunta : ((Grupo) item).getPreguntas()) {
  // Respuesta respuesta = new Respuesta();
  // respuesta.setPregunta(pregunta);
  // respuesta.setFormulario(formulario);
  // formulario.getValores().put(pregunta, respuesta);
  // }
  // }
  // }
  // } else {
  // for (Item item : formulario.getItems()) {
  // if (item.isPregunta()) {
  // numItems++;
  // } else {
  // for (@SuppressWarnings("unused") Pregunta pregunta : ((Grupo) item).getPreguntas()) {
  // numItems++;
  // }
  // }
  // }
  // if (formulario.getValores().size() != numItems) {
  // for (Item item : formulario.getItems()) {
  // if (item.isPregunta()) {
  // if (formulario.getValores().get(item) == null) {
  // Respuesta respuesta = new Respuesta();
  // respuesta.setPregunta((Pregunta) item);
  // respuesta.setFormulario(formulario);
  // formulario.getValores().put((Pregunta) item, respuesta);
  // }
  // } else {
  // for (Pregunta pregunta : ((Grupo) item).getPreguntas()) {
  // if (formulario.getValores().get(pregunta) == null) {
  // Respuesta respuesta = new Respuesta();
  // respuesta.setPregunta(pregunta);
  // respuesta.setFormulario(formulario);
  // formulario.getValores().put(pregunta, respuesta);
  // }
  // }
  // }
  // }
  // }
  // }
  // } catch (Exception e) {
  // e.printStackTrace();
  // }
  // return formulario;
  // }
}
