package es.csa.fdt.cyldigital.evaluacion.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.List;

import javax.faces.model.SelectItem;

import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;

import es.csa.fdt.cyldigital.evaluacion.entity.Examen;
import es.csa.fdt.cyldigital.evaluacion.entity.GrupoExamen;
import es.csa.fdt.cyldigital.evaluacion.entity.ItemExamen;
import es.csa.fdt.cyldigital.evaluacion.entity.PreguntaExamen;
import es.csa.fdt.cyldigital.evaluacion.entity.OpcionPreguntaExamen;

import es.csa.fdt.cyldigital.language.entity.ItemExamenLanguage;
import es.csa.fdt.cyldigital.language.entity.OpcionPreguntaExamenLanguage;
import es.csa.fdt.cyldigital.language.entity.Language;


/**
 * Clase PreguntaExamenServiceImpl.
 * 
 * @author Fundetec
 */
@Service
@Repository
public class PreguntaExamenServiceImpl extends AbstractServiceImpl<PreguntaExamen, Long> implements PreguntaExamenService {

	private static final long serialVersionUID = -509345017105102131L;

	private Long idExamen;
	private Long idGrupo;
	
	private List<SelectItem> optionsSelectItems;
	
	public PreguntaExamenServiceImpl() {
		optionsSelectItems = new ArrayList<SelectItem>();
	}
	
	/* (non-Javadoc)
	 * @see es.csa.fdt.cyldigital.evaluacion.service.PreguntaExamenService#createPreguntaExamen(java.lang.Long, java.lang.Long)
	 */
	public ItemExamen createPreguntaExamen(Long idExamen, Long idGrupo, List<Language> languages) {
		this.idExamen = idExamen;
		this.idGrupo = idGrupo;
    
		PreguntaExamen preguntaExamen = new PreguntaExamen();
    
		List<ItemExamenLanguage> textos = new ArrayList<ItemExamenLanguage>();

		for (Language language : languages) {
			ItemExamenLanguage texto = new ItemExamenLanguage();

			texto.setLanguage(language);
			texto.setItemExamen(preguntaExamen);

			textos.add(texto);
		}

		preguntaExamen.setTextos(textos);
		preguntaExamen.setOpciones(null);
		preguntaExamen.setOpcionCorrecta(null);

		return preguntaExamen;
	}
	
	/**
	 * Find a PreguntaExamen wit idPreguntaExamen
	 * 
	 * @return Option created to question of exam.
	 */
	@Override
	public PreguntaExamen find(Long idPreguntaExamen) {
		PreguntaExamen pe = super.find(idPreguntaExamen);
		
		for (OpcionPreguntaExamen ope : pe.getOpciones())
			optionsSelectItems.add(new SelectItem(ope));
		
		return pe;
	}
	
	/**
	 * create option to question of exam
	 * 
	 * @return Option created to question of exam.
	 */
	public OpcionPreguntaExamen createOptionToQuestion(List<Language> languages, OpcionPreguntaExamen opcion) {
		if ((languages != null) && !languages.isEmpty() && (opcion == null)) {
			opcion = new OpcionPreguntaExamen();
			
			List<OpcionPreguntaExamenLanguage> textos = new ArrayList<OpcionPreguntaExamenLanguage>();
			
			for (Language language : languages) {
				OpcionPreguntaExamenLanguage texto = new OpcionPreguntaExamenLanguage();

				texto.setLanguage(language);
				texto.setOpcion(opcion);

				textos.add(texto);
			}
			
			opcion.setTextos(textos);
		}

		return opcion;
	}
	
	/**
	 * Add option to question of exam
	 *
	 * @param preguntaExamen
	 * 		  The question that the option will be added 
	 * 
	 * @param opcion
	 * 		  Option to add to the question
	 * 
	 */
	public void saveOptionToQuestion(PreguntaExamen preguntaExamen, OpcionPreguntaExamen opcion, Boolean esNueva) {
		if ((preguntaExamen != null) && (opcion != null) && esNueva) {
			if (preguntaExamen.getOpciones() == null)
				preguntaExamen.setOpciones(new ArrayList<OpcionPreguntaExamen>());
			
			opcion.setPreguntaExamen(preguntaExamen);

			preguntaExamen.getOpciones().add(opcion);
			
			optionsSelectItems.add(new SelectItem(opcion));
		}
	}
	
	/**
	 * Remove option to question of exam
	 *
	 * @param preguntaExamen
	 * 		  The question from the option will be removed 
	 * 
	 * @param opcion
	 * 		  Option to remove from the question
	 * 
	 */
	public void removeOptionToQuestion(PreguntaExamen preguntaExamen, OpcionPreguntaExamen opcion) {
		if ((preguntaExamen != null) && (preguntaExamen.getOpciones() != null) && (opcion != null)) {
			int indexSiToRemove = -1;

			for (SelectItem si : optionsSelectItems) {
				if (((OpcionPreguntaExamen)si.getValue()).equals(opcion)) {
					indexSiToRemove = optionsSelectItems.indexOf(si);

					break;
				}
			}

			preguntaExamen.getOpciones().remove(opcion);

			if (indexSiToRemove >= 0)
				optionsSelectItems.remove(indexSiToRemove);
		}
	}

	/**
	 * @return the optionsSelectItems
	 */
	public List<SelectItem> getOptionsSelectItems() {
		String idioma = LocaleContextHolder.getLocaleContext().getLocale().toString();
		
		for (SelectItem si : optionsSelectItems) {
			for (OpcionPreguntaExamenLanguage opel : ((OpcionPreguntaExamen)si.getValue()).getTextos()) {
				if (opel.getLanguage().getISO().equals(idioma))
					si.setLabel(opel.getTextoOpcion());
			}
		}
			
		return optionsSelectItems;
	}

	/**
	 * Persist.
	 * 
	 * @param preguntaExamen
	 *           preguntaExamen
	 */
	public void persist(PreguntaExamen preguntaExamen) {
		if (preguntaExamen.getId() == 0) {
			Examen f = find(Examen.class, idExamen);
			
			if (idGrupo != null && idGrupo > 0) {
				GrupoExamen g = find(GrupoExamen.class, idGrupo);
				
				if (g != null) {
					preguntaExamen.setGrupoExamen(g);

					((GrupoExamen)f.getItems().get(f.getItems().indexOf(g))).getPreguntas().add(preguntaExamen);
				}
			}
			else {
				preguntaExamen.setExamen(f);
				f.getItems().add(preguntaExamen);
			}
		}

		disableMessages();
		super.persist(preguntaExamen);
		enableMessages();
		
		addInfoMessage("service.persist.ok");
	}
}
