package es.csa.fdt.cyldigital.evaluacion.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.List;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.evaluacion.entity.Formulario;
import es.csa.fdt.cyldigital.evaluacion.entity.Grupo;
import es.csa.fdt.cyldigital.evaluacion.entity.Item;
import es.csa.fdt.cyldigital.evaluacion.entity.Pregunta;
import es.csa.fdt.cyldigital.language.entity.ItemLanguage;
import es.csa.fdt.cyldigital.language.entity.Language;


/**
 * Clase PreguntaServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class PreguntaServiceImpl extends AbstractServiceImpl<Pregunta, Long> implements PreguntaService {

  private static final long serialVersionUID = -5575228180257917674L;

  private Long idFormulario;

  private Long idGrupo;

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.evaluacion.service.PreguntaService#createPregunta(java.lang.Long, java.lang.Long)
   */
  public Item createPregunta(Long idFormulario, Long idGrupo, List<Language> languages) {
    this.idFormulario = idFormulario;
    this.idGrupo = idGrupo;
    
    Item pregunta = new Pregunta();
    
    List<ItemLanguage> textos = new ArrayList<ItemLanguage>();

    for (Language language : languages) {
    	ItemLanguage texto = new ItemLanguage();
    	
    	texto.setLanguage(language);
    	texto.setItem(pregunta);

    	textos.add(texto);
    }

    pregunta.setTextos(textos);

    return pregunta;
  }

  /**
   * Persist.
   * 
   * @param pregunta
   *           pregunta
   */
  public void persist(Pregunta pregunta) {
    if (pregunta.getId() == 0) {
      Formulario f = find(Formulario.class, idFormulario);
      if (idGrupo != null && idGrupo > 0) {
        Grupo g = find(Grupo.class, idGrupo);
        if (g != null) {
          pregunta.setGrupo(g);
          ((Grupo) f.getItems().get(f.getItems().indexOf(g))).getPreguntas().add(pregunta);
        }
      } else {
        pregunta.setFormulario(f);
        f.getItems().add(pregunta);
      }
    }
    super.persist(pregunta);
  }
}
