package es.csa.fdt.cyldigital.financiero.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Table;

import org.hibernate.envers.Audited;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.financiero.entity.Movimiento.TipoOperacion;

/**
 * Clase Caja.
 * 
 * @author csa
 */
@Entity
@Table(name = "cajas")
@AttributeOverride(name = "id", column = @Column(name = "idCaja"))
@Audited
public class Caja extends AbstractEntity implements Serializable {
  private static final long serialVersionUID = -7648562176655166537L;

  private Centro centro;
  private List<Movimiento> movimientos;
  private float valorCaja;

  /**
   * Construye un nuevo objeto de la clase
   */
  public Caja() {
    super();
    movimientos=new ArrayList<Movimiento>();
  }
  
  /**
   * Devuelve centro.
   * 
   * @return centro
   */
  @OneToOne(mappedBy = "caja")
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Devuelve movimientos.
   * 
   * @return movimientos
   */
  @OneToMany(mappedBy = "caja", fetch = FetchType.EAGER)
  public List<Movimiento> getMovimientos() {
    return movimientos;
  }

  /**
   * Asigna movimientos.
   * 
   * @param movimientos
   *          movimientos
   */
  public void setMovimientos(List<Movimiento> movimientos) {
    this.movimientos = movimientos;
  }
  
  private Movimiento getMovimientoAnteriorAMes(int anyo, int mes){
    Calendar calendarInicio = new GregorianCalendar(anyo, mes, 1, 0, 0, 0);
    Movimiento movimientoAnterior = null;
    
    for(Movimiento movimiento : this.getMovimientos()){
      if(movimiento.getFechaOperacion().compareTo(calendarInicio.getTime()) >= 0){
        break;
      }else{
        movimientoAnterior = movimiento;
      }
    }
    
    return movimientoAnterior;
  }
  
  /**
   * Devuelve saldo anterior a mes.
   * 
   * @param anyo
   *           anyo
   * @param mes
   *           mes
   * @return saldo anterior a mes
   */
  public float getSaldoAnteriorAMes(Integer anyo, Integer mes){
    float saldoAnterior = 0;
    Movimiento movimientoAnterior = this.getMovimientoAnteriorAMes(anyo, mes);
    if(movimientoAnterior != null){
      saldoAnterior = movimientoAnterior.getImportePosterior();
    }
    
   return saldoAnterior;
  }
  
  /**
   * Devuelve saldo posterior a mes.
   * 
   * @param anyo
   *           anyo
   * @param mes
   *           mes
   * @return saldo posterior a mes
   */
  public float getSaldoPosteriorAMes(Integer anyo, Integer mes){
    Calendar calendarFin = new GregorianCalendar(anyo, mes + 1, 1, 0, 0, 0);
    float saldoPosterior = 0;
    Movimiento movimientoAux = null;
    
    for(Movimiento movimiento : this.getMovimientos()){
      if(movimiento.getFechaOperacion().compareTo(calendarFin.getTime()) > 0){
        break;
      }else{
        movimientoAux = movimiento;
      }
    }
    
    if(movimientoAux != null){
      saldoPosterior = movimientoAux.getImportePosterior();
    }
    
    return saldoPosterior;
  }
  
  private List<Movimiento> getMovimientosMes(int anyo, int mes){
    Calendar calendarInicio = new GregorianCalendar(anyo, mes, 1, 0, 0, 0);
    Calendar calendarFin = new GregorianCalendar(anyo, mes + 1, 1, 0, 0, 0);
    List<Movimiento> movimientosMes = new ArrayList<Movimiento>();
    
    for(Movimiento movimiento : this.getMovimientos()){
      if(movimiento.getFechaOperacion().compareTo(calendarInicio.getTime()) >= 0 && movimiento.getFechaOperacion().compareTo(calendarFin.getTime()) <= 0){
        movimientosMes.add(movimiento);
      }
    }
    
    return movimientosMes;
  }
  
  private List<Movimiento> getMovimientosAnyo(int anyo){
    Calendar calendarInicio = new GregorianCalendar(anyo, Calendar.JANUARY, 1, 0, 0, 0);
    Calendar calendarFin = new GregorianCalendar(anyo + 1, Calendar.JANUARY, 1, 0, 0, 0);
    List<Movimiento> movimientosMes = new ArrayList<Movimiento>();
    
    for(Movimiento movimiento : this.getMovimientos()){
      if(movimiento.getFechaOperacion().compareTo(calendarInicio.getTime()) >= 0 && movimiento.getFechaOperacion().compareTo(calendarFin.getTime()) <= 0){
        movimientosMes.add(movimiento);
      }
    }
    
    return movimientosMes;
  }
  
  private float sumarCuantiasTipoMovimientos(List<Movimiento> movimientos, TipoOperacion tipoOperacion){
    float cuantia = 0;
    
    for(Movimiento movimiento : movimientos){
      if(movimiento.getTipo() == tipoOperacion){
        cuantia += movimiento.getImporte();
      }
    }
    
    return cuantia;
  }
  
  /**
   * Devuelve cuantia abonos mes.
   * 
   * @param anyo
   *           anyo
   * @param mes
   *           mes
   * @return cuantia abonos mes
   */
  public float getCuantiaAbonosMes(int anyo, int mes){
    List<Movimiento> movimientos = this.getMovimientosMes(anyo, mes);
    
    return this.sumarCuantiasTipoMovimientos(movimientos, TipoOperacion.ABONOCAJA);
  }
  
  /**
   * Devuelve cuantia cargos mes.
   * 
   * @param anyo
   *           anyo
   * @param mes
   *           mes
   * @return cuantia cargos mes
   */
  public float getCuantiaCargosMes(int anyo, int mes){
    List<Movimiento> movimientos = this.getMovimientosMes(anyo, mes);
    
    return this.sumarCuantiasTipoMovimientos(movimientos, TipoOperacion.ANADIRFACTURA);
  }
  
  /**
   * Devuelve cuantia abonos anyo.
   * 
   * @param anyo
   *           anyo
   * @return cuantia abonos anyo
   */
  public float getCuantiaAbonosAnyo(int anyo){
    List<Movimiento> movimientos = this.getMovimientosAnyo(anyo);
    
    return this.sumarCuantiasTipoMovimientos(movimientos, TipoOperacion.ABONOCAJA);
  }
  
  /**
   * Devuelve cuantia cargos anyo.
   * 
   * @param anyo
   *           anyo
   * @return cuantia cargos anyo
   */
  public float getCuantiaCargosAnyo(int anyo){
    List<Movimiento> movimientos = this.getMovimientosAnyo(anyo);
    
    return this.sumarCuantiasTipoMovimientos(movimientos, TipoOperacion.ANADIRFACTURA);
  }

  /**
   * Devuelve valor caja.
   * 
   * @return valor caja
   */
  public float getValorCaja() {
    return valorCaja;
  }

  /**
   * Asigna valor caja.
   * 
   * @param valorCaja
   *          valor caja
   */
  public void setValorCaja(float valorCaja) {
    this.valorCaja = valorCaja;
  }
}
