package es.csa.fdt.cyldigital.financiero.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.annotations.Cascade;
import org.hibernate.annotations.IndexColumn;
import org.hibernate.envers.Audited;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.financiero.entity.EstadoFactura.TipoEstadoFactura;
import es.csa.fdt.cyldigital.financiero.entity.EstadoPago.TipoEstadoPago;
import es.csa.fdt.cyldigital.financiero.entity.Prevision.Mes;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;

/**
 * Clase Contenedor.
 * 
 * @author csa
 */
@Entity
@Table(name = "contenedores")
@Audited
@Inheritance(strategy = InheritanceType.JOINED)
@AttributeOverride(name = "id", column = @Column(name = "idContenedor"))
public abstract class Contenedor extends AbstractEntity {

  private static final long serialVersionUID = 5940574238905894230L;
  
  private final int gradoDesviacionAlta = 30;
  private final int gradoDesviacionMedia = 15;

  private String nombre;
  private ProyectoAnual proyecto;
  
  private Contenedor contenedorPadre;
  private List<Contenedor> elementos;
  
  private Contenedor contenedorProyectoGeneral;
  private List<Contenedor> contenedoresSubproyectos;
  
  private Prevision prevision;
  
  private List<Factura> facturas;

  private int secuencia;
  
  private boolean expandido;//Transient
  
  /**
   * Enum TipoContenedor.
   */
  public enum TipoContenedor{
    SECCION,
    ITEM
  }
  
  /**
   * Enum GradoDesviacion.
   */
  public enum GradoDesviacion{
    ALTO,
    MEDIO,
    BAJO
  }
  
  private TipoContenedor tipoContenedor;
  
  private ProyectoAnual proyectoRaiz;
  
  private Seccion seccionRaiz;
 
  /**
   * Constructor sin parametros 
   * 
   * @param null
   *           tipo contenedor
   */
  protected Contenedor() {
	    super();
  }
  
  /**
   * Construye un nuevo objeto de la clase
   * 
   * @param tipoContenedor
   *           tipo contenedor
   */
  public Contenedor(TipoContenedor tipoContenedor) {
    super();
    
    prevision = new Prevision();
    prevision.setContenedor(this);
    if(this.contenedorPadre != null){
      this.prevision.setPrevisionPadre(this.contenedorPadre.getPrevision());
    }
    
    secuencia = -1;
    
    this.setTipoContenedor(tipoContenedor);
  }
  
  /**
   * Construye un nuevo objeto de la clase
   * 
   * @param tipoContenedor
   *           tipo contenedor
   * @param nombre
   *           nombre
   */
  public Contenedor(TipoContenedor tipoContenedor, String nombre) {
    this(tipoContenedor);
    
    this.setNombre(nombre);
  }
  
  /**
   * Construye un nuevo objeto de la clase
   * 
   * @param tipoContenedor
   *           tipo contenedor
   * @param nombre
   *           nombre
   * @param proyectoRaiz
   *           proyecto raiz
   */
  public Contenedor(TipoContenedor tipoContenedor, String nombre, ProyectoAnual proyectoRaiz)
  {
    this(tipoContenedor, nombre);
    this.setProyectoRaiz(proyectoRaiz);
    this.nombre = nombre;
  }

  /**
   * Devuelve nombre.
   * 
   * @return nombre
   */
  @Length(max = 150)
  @NotEmpty
  @NotNull
  public String getNombre() {
    return nombre;
  }

  /**
   * Asigna nombre.
   * 
   * @param nombre
   *          nombre
   */
  public void setNombre(String nombre) {
    this.nombre = nombre;
  }

  /**
   * Devuelve contenedor padre.
   * 
   * @return contenedor padre
   */
  @ManyToOne(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
  @JoinColumn(name = "idPadre")
  public Contenedor getContenedorPadre() {
    return contenedorPadre;
  }

  /**
   * Asigna contenedor padre.
   * 
   * @param contenedorPadre
   *          contenedor padre
   */
  public void setContenedorPadre(Contenedor contenedorPadre) {
    this.contenedorPadre = contenedorPadre;
    if(this.contenedorPadre != null)
      this.prevision.setPrevisionPadre(this.contenedorPadre.getPrevision());
  }

  /*
   * Propiedad visible en ambas direcciones y que a diferencia de la propiedad ProyectoRaiz, nos
   * indica el proyecto al que pertenece dicho Contenedor, en cuanto al nivel de jerarquía se refiere.
   * Es decir, sólo tendrán esta propiedad asignada, aquellos Contenedores que sean descendientes
   * directos del proyecto (Seccion) y que no estén anidados a otros Contenedores (Item).
   */
  /**
   * Devuelve proyecto.
   * 
   * @return proyecto
   */
  @ManyToOne
  @JoinColumn(name = "idProyecto")
  public ProyectoAnual getProyecto() {
    return proyecto;
  }

  /**
   * Asigna proyecto.
   * 
   * @param proyecto
   *          proyecto
   */
  public void setProyecto(ProyectoAnual proyecto) {
    this.proyecto = proyecto;
  }

  /**
   * Devuelve elementos.
   * 
   * @return elementos
   */
  @OneToMany(cascade = { CascadeType.PERSIST, CascadeType.MERGE, CascadeType.REFRESH }, fetch = FetchType.EAGER)
  @JoinColumn(name = "idPadre")
  @Cascade(org.hibernate.annotations.CascadeType.DELETE_ORPHAN)
  @IndexColumn(name = "secuencia", base = 0)
  public List<Contenedor> getElementos() {
    return elementos;
  }

  /**
   * Asigna elementos.
   * 
   * @param elementos
   *          elementos
   */
  public void setElementos(List<Contenedor> elementos) {
    this.elementos = elementos;
  }

  /**
   * Devuelve contenedor proyecto general.
   * 
   * @return contenedor proyecto general
   */
  @ManyToOne
  @JoinColumn(name = "idContenedorProyectoGeneral")
  public Contenedor getContenedorProyectoGeneral() {
    return contenedorProyectoGeneral;
  }

  /**
   * Asigna contenedor proyecto general.
   * 
   * @param contenedorProyectoGeneral
   *          contenedor proyecto general
   */
  public void setContenedorProyectoGeneral(Contenedor contenedorProyectoGeneral) {
    this.contenedorProyectoGeneral = contenedorProyectoGeneral;
  }

  /**
   * Devuelve contenedores subproyectos.
   * 
   * @return contenedores subproyectos
   */
  @OneToMany(mappedBy = "contenedorProyectoGeneral", cascade = CascadeType.ALL, fetch = FetchType.EAGER)
  public List<Contenedor> getContenedoresSubproyectos() {
    return contenedoresSubproyectos;
  }

  /**
   * Asigna contenedores subproyectos.
   * 
   * @param contenedoresSubproyectos
   *          contenedores subproyectos
   */
  public void setContenedoresSubproyectos(List<Contenedor> contenedoresSubproyectos) {
    this.contenedoresSubproyectos = contenedoresSubproyectos;
  }
  
  /**
   * Devuelve prevision.
   * 
   * @return prevision
   */
  @OneToOne(mappedBy = "contenedor", cascade = {CascadeType.ALL})
  @Cascade(value = org.hibernate.annotations.CascadeType.DELETE_ORPHAN)
  public Prevision getPrevision() {
    return prevision;
  }

  /**
   * Asigna prevision.
   * 
   * @param prevision
   *          prevision
   */
  public void setPrevision(Prevision prevision) {
    this.prevision = prevision;
  }

  /**
   * Devuelve facturas.
   * 
   * @return facturas
   */
  @OneToMany(mappedBy = "contenedor", cascade = CascadeType.ALL,fetch=FetchType.EAGER)
  public List<Factura> getFacturas() {
    return facturas;
  }

  /**
   * Asigna facturas.
   * 
   * @param facturas
   *          facturas
   */
  public void setFacturas(List<Factura> facturas) {
    this.facturas = facturas;
  }

  /**
   * Devuelve secuencia.
   * 
   * @return secuencia
   */
  public int getSecuencia() {
    int paramSecuencia = -1;
    
    if(secuencia != -1) {
      paramSecuencia = secuencia;
    } else 
    {
      if (this.getClass() == Seccion.class) {
        if (getProyecto() != null && getProyecto().getElementos() != null) {
          paramSecuencia = getProyecto().getElementos().indexOf(this);
        } else {
          paramSecuencia = -1;
        }
      } else if(this.getClass() == ItemContenedor.class){
        if (getContenedorPadre() != null && getContenedorPadre().getElementos() != null) {
          paramSecuencia = getContenedorPadre().getElementos().indexOf(this);
        } else {
          paramSecuencia = -1;
        }
      }
    }

    return paramSecuencia;
  }

  /**
   * Asigna secuencia.
   * 
   * @param secuencia
   *          secuencia
   */
  public void setSecuencia(int secuencia) {}
  
  /**
   * Asigna secuencia inicial.
   * 
   * @param secuencia
   *          secuencia inicial
   */
  public void setSecuenciaInicial(int secuencia)
  {
    this.secuencia = secuencia;
  }
  
  /**
   * Devuelve tipo contenedor.
   * 
   * @return tipo contenedor
   */
  @Transient
  public TipoContenedor getTipoContenedor() {
    return tipoContenedor;
  }

  /**
   * Asigna tipo contenedor.
   * 
   * @param tipoContenedor
   *          tipo contenedor
   */
  public void setTipoContenedor(TipoContenedor tipoContenedor) {
    this.tipoContenedor = tipoContenedor;
  }
  
//  @Transient
//  public ProyectoAnual getProyectoRaiz()
//  {
//    ProyectoAnual proyectoRaiz = null;
//    
//    if(this.getProyecto() != null){
//      proyectoRaiz = this.getProyecto();
//    }else{
//      if(this.getContenedorPadre() != null)
//        proyectoRaiz = this.getContenedorPadre().getProyectoRaiz();
//    }
//    
//    return proyectoRaiz;
//  }
  
  /*
   * Propiedad visible sólo en la dirección de Contenedor, que nos facilita el proyecto del cual
   * forma parte dicho contenedor a pesar de que esté anidado dicho elemento en varios niveles.
   */
  /**
 * Devuelve proyecto raiz.
 * 
 * @return proyecto raiz
 */
@ManyToOne
  @JoinColumn(name = "idProyectoRaiz")
  public ProyectoAnual getProyectoRaiz() {
    return proyectoRaiz;
  }

  /**
   * Asigna proyecto raiz.
   * 
   * @param proyectoRaiz
   *          proyecto raiz
   */
  public void setProyectoRaiz(ProyectoAnual proyectoRaiz) {
    this.proyectoRaiz = proyectoRaiz;
  }
  
  /**
   * Devuelve seccion raiz.
   * 
   * @return seccion raiz
   */
  @ManyToOne
  @JoinColumn(name = "idSeccionRaiz")
  public Seccion getSeccionRaiz() {
    return seccionRaiz;
  }

  /**
   * Asigna seccion raiz.
   * 
   * @param seccionRaiz
   *          seccion raiz
   */
  public void setSeccionRaiz(Seccion seccionRaiz) {
    this.seccionRaiz = seccionRaiz;
  }
  
  /**
   * Devuelve esquema.
   * 
   * @return esquema
   */
  public String getEsquema() {
    return this.getEsquemaNDigitos(2);
  }

  /**
   * Asigna esquema.
   * 
   * @param esquema
   *          esquema
   */
  public void setEsquema(String esquema) {}
  
  /**
   * Devuelve esquema n digitos.
   * 
   * @param numeroDigitosEnteros
   *           numero digitos enteros
   * @return esquema n digitos
   */
  @Transient
  public String getEsquemaNDigitos(int numeroDigitosEnteros){
    StringBuffer esquema = new StringBuffer();
    if(this.getContenedorPadre() != null){
      esquema.append(this.getContenedorPadre().getEsquema());
      esquema.append('.');
    }
    
    NumberFormat numberInstance = NumberFormat.getNumberInstance();
    numberInstance.setMinimumIntegerDigits(numeroDigitosEnteros);
    
    esquema.append(numberInstance.format(this.getSecuencia() + 1));
    
    return esquema.toString();
  }
  
  /**
   * Devuelve responsable.
   * 
   * @return responsable
   */
  @Transient
  public abstract Empleado getResponsable();

  /**
   * Devuelve centro.
   * 
   * @return centro
   */
  @Transient
  public Centro getCentro()
  {
    Centro centro = null;
    
    if(this.getProyectoRaiz() != null){
      centro = this.getProyectoRaiz().getCentro();
    }
    
    return centro;
  }
  
  /**
   * Devuelve anyo.
   * 
   * @return anyo
   */
  @Transient
  public Integer getAnyo()
  {
    Integer anyo = null;
    
    if(this.getProyectoRaiz() != null){
      anyo = this.getProyectoRaiz().getAnyo();
    }
    
    return anyo;
  }
  
  /**
   * Devuelve valor facturas.
   * 
   * @return valor facturas
   */
  @Transient
  public Double getValorFacturas(){
    Double valorPagos =  this.getValorPagosPresupuestosAprobados(null, null, null);
    
    for(Contenedor elemento : this.getElementos()){
      valorPagos += elemento.getValorFacturas();
    }
    
    return valorPagos;
  }
  
  /**
   * Devuelve valor facturas.
   * 
   * @param mes
   *           mes
   * @return valor facturas
   */
  @Transient
  public Double getValorFacturas(Mes mes){
    Date fechaInicio = this.getFechaMes(mes, this.getAnyo(), true);
    Date fechaFin = this.getFechaMes(mes, this.getAnyo(), false);
    
    Double valorPagos =  this.getValorPagosPresupuestosAprobados(fechaInicio, fechaFin, null);
    
    for(Contenedor elemento : this.getElementos()){
      valorPagos += elemento.getValorFacturas(mes);
    }
    
    return valorPagos;
  }
  
  /**
   * Devuelve valor facturas.
   * 
   * @param mesString
   *           mes string
   * @return valor facturas
   */
  @Transient
  public Double getValorFacturas(String mesString){
    Mes mes = Mes.valueOf(mesString);
    return this.getValorFacturas(mes);
  }
  
  /**
   * Devuelve valor pagos efectivos facturas.
   * 
   * @return valor pagos efectivos facturas
   */
  @Transient
  public Double getValorPagosEfectivosFacturas(){
    Date ahora = new Date();
    
    Double valorPagos = this.getValorPagosPresupuestosAprobados(null, ahora, TipoEstadoPago.PAGADO);
    for(Contenedor elemento : this.getElementos()){
      valorPagos += elemento.getValorPagosEfectivosFacturas();
    }
    
    return valorPagos;
  }
  
  /**
   * Devuelve valor pagos efectivos facturas.
   * 
   * @param mes
   *           mes
   * @return valor pagos efectivos facturas
   */
  @Transient
  public Double getValorPagosEfectivosFacturas(Mes mes){
    Date fechaInicio = this.getFechaMes(mes, this.getAnyo(), true);
    Date fechaFin = this.getFechaMes(mes, this.getAnyo(), false);
    
    Double valorPagos = this.getValorPagosPresupuestosAprobados(fechaInicio, fechaFin, TipoEstadoPago.PAGADO);
    for(Contenedor elemento : this.getElementos()){
      valorPagos += elemento.getValorPagosEfectivosFacturas(mes);
    }
    
    return valorPagos;
  }
  
  /**
   * Devuelve valor pagos efectivos facturas.
   * 
   * @param mesString
   *           mes string
   * @return valor pagos efectivos facturas
   */
  @Transient
  public Double getValorPagosEfectivosFacturas(String mesString){
    Mes mes = Mes.valueOf(mesString);
    return this.getValorPagosEfectivosFacturas(mes);
  }
  
  @Transient
  private Double getValorPagosPresupuestosAprobados(Date fechaHoraInicio, Date fechaHoraFin, TipoEstadoPago tipoEstadoPagoMinimo){
    Double valorFacturas = this.getValorPagosPropiosPresupuestosAprobados(fechaHoraInicio, fechaHoraFin, tipoEstadoPagoMinimo);
    
    //Si el contenedor es de un subproyecto, añadimos la cuantía proporcional de las facturas
    //del contenedor del proyecto general.
    if(this.getContenedorProyectoGeneral() != null){
      for(Factura factura : this.getContenedorProyectoGeneral().getFacturas()){
        if(factura.getEstadoActual().getTipoEstadoFactura() != TipoEstadoFactura.PRESUPUESTO_PENDIENTE_APROBACION && factura.getEstadoActual().getTipoEstadoFactura() != TipoEstadoFactura.PRESUPUESTO_RECHAZADO){
          valorFacturas += factura.getValorCuantiaSubpagosCentro(this.getCentro(), fechaHoraInicio, fechaHoraFin, tipoEstadoPagoMinimo);
        }
      }
    }else{//if(this.getContenedorProyectoGeneral() == null) //En el caso de que tenga subproyectos, sumamos las facturas de los subcontenedores de los mismos.
      for(Contenedor contenedorSubproyecto : this.getContenedoresSubproyectos()){
        valorFacturas += contenedorSubproyecto.getValorPagosPropiosPresupuestosAprobados(fechaHoraInicio, fechaHoraFin, tipoEstadoPagoMinimo);
      }
    }
    
    return valorFacturas;
  }
  
  /**
   * Devuelve valor pagos propios presupuestos aprobados.
   * 
   * @param fechaHoraInicio
   *           fecha hora inicio
   * @param fechaHoraFin
   *           fecha hora fin
   * @param tipoEstadoPagoMinimo
   *           tipo estado pago minimo
   * @return valor pagos propios presupuestos aprobados
   */
  @Transient
  public Double getValorPagosPropiosPresupuestosAprobados(Date fechaHoraInicio, Date fechaHoraFin, TipoEstadoPago tipoEstadoPagoMinimo){
    Double valorFacturas = new Double(0);
    
    for(Factura factura : this.getFacturas()){
      if(factura.getEstadoActual().getTipoEstadoFactura() != TipoEstadoFactura.PRESUPUESTO_PENDIENTE_APROBACION && factura.getEstadoActual().getTipoEstadoFactura() != TipoEstadoFactura.PRESUPUESTO_RECHAZADO){
        valorFacturas += factura.getValorCuantiaPagos(fechaHoraInicio, fechaHoraFin, tipoEstadoPagoMinimo);
      }
    }
    
    return valorFacturas;
  }
  
  /**
   * Devuelve valor facturas justificadas.
   * 
   * @return valor facturas justificadas
   */
  @Transient
  public Double getValorFacturasJustificadas(){
    
    Double valorPagos =  this.getValorPagosFacturasJustificadas(null, null);
    
    for(Contenedor elemento : this.getElementos()){
      valorPagos += elemento.getValorFacturasJustificadas();
    }
    
    return valorPagos;
  }
  
  /**
   * Devuelve valor facturas justificadas.
   * 
   * @param mes
   *           mes
   * @return valor facturas justificadas
   */
  @Transient
  public Double getValorFacturasJustificadas(Mes mes){
    Date fechaInicio = this.getFechaMes(mes, this.getAnyo(), true);
    Date fechaFin = this.getFechaMes(mes, this.getAnyo(), false);
    
    Double valorPagos =  this.getValorPagosFacturasJustificadas(fechaInicio, fechaFin);
    
    for(Contenedor elemento : this.getElementos()){
      valorPagos += elemento.getValorFacturasJustificadas(mes);
    }
    
    return valorPagos;
  }
  
  /**
   * Devuelve valor facturas justificadas.
   * 
   * @param mesString
   *           mes string
   * @return valor facturas justificadas
   */
  @Transient
  public Double getValorFacturasJustificadas(String mesString){
    Mes mes = Mes.valueOf(mesString);
    return this.getValorFacturasJustificadas(mes);
  }
  
  @Transient
  private Double getValorPagosFacturasJustificadas(Date fechaHoraInicio, Date fechaHoraFin){
    Double valorFacturas = this.getValorPagosPropiosFacturasJustificadas(fechaHoraInicio, fechaHoraFin);
    
    //Si el contenedor es de un subproyecto, añadimos la cuantía proporcional de las facturas
    //del contenedor del proyecto general.
    if(this.getContenedorProyectoGeneral() != null){
      for(Factura factura : this.getContenedorProyectoGeneral().getFacturas()){
        if(factura.getJustificadaConsulta() == true && factura.getEstadoActual().getTipoEstadoFactura() != TipoEstadoFactura.PRESUPUESTO_PENDIENTE_APROBACION && factura.getEstadoActual().getTipoEstadoFactura() != TipoEstadoFactura.PRESUPUESTO_RECHAZADO){
          valorFacturas += factura.getValorCuantiaSubpagosCentro(this.getCentro(), fechaHoraInicio, fechaHoraFin, null);
        }
      }
    }else{//if(this.getContenedorProyectoGeneral() == null) //En el caso de que tenga subproyectos, sumamos las facturas de los subcontenedores de los mismos.
      for(Contenedor contenedorSubproyecto : this.getContenedoresSubproyectos()){
        valorFacturas += contenedorSubproyecto.getValorPagosPropiosFacturasJustificadas(fechaHoraInicio, fechaHoraFin);
      }
    }
    
    return valorFacturas;
  }
  
  /**
   * Devuelve valor pagos propios facturas justificadas.
   * 
   * @param fechaHoraInicio
   *           fecha hora inicio
   * @param fechaHoraFin
   *           fecha hora fin
   * @return valor pagos propios facturas justificadas
   */
  @Transient
  public Double getValorPagosPropiosFacturasJustificadas(Date fechaHoraInicio, Date fechaHoraFin){
    Double valorFacturas = new Double(0);
    
    for(Factura factura : this.getFacturas()){
      if(factura.getJustificadaConsulta() == true && factura.getEstadoActual().getTipoEstadoFactura() != TipoEstadoFactura.PRESUPUESTO_PENDIENTE_APROBACION && factura.getEstadoActual().getTipoEstadoFactura() != TipoEstadoFactura.PRESUPUESTO_RECHAZADO){
        valorFacturas += factura.getValorCuantiaPagos(fechaHoraInicio, fechaHoraFin, null);
      }
    }
    
    return valorFacturas;
  }
  
  /**
   * Devuelve valor facturas aprobadas junta.
   * 
   * @return valor facturas aprobadas junta
   */
  @Transient
  public Double getValorFacturasAprobadasJunta(){
    
    Double valorPagos =  this.getValorPagosFacturasAprobadasJunta(null, null);
    
    for(Contenedor elemento : this.getElementos()){
      valorPagos += elemento.getValorFacturasAprobadasJunta();
    }
    
    return valorPagos;
  }
  
  /**
   * Devuelve valor facturas aprobadas junta.
   * 
   * @param mes
   *           mes
   * @return valor facturas aprobadas junta
   */
  @Transient
  public Double getValorFacturasAprobadasJunta(Mes mes){
    Date fechaInicio = this.getFechaMes(mes, this.getAnyo(), true);
    Date fechaFin = this.getFechaMes(mes, this.getAnyo(), false);
    
    Double valorPagos =  this.getValorPagosFacturasAprobadasJunta(fechaInicio, fechaFin);
    
    for(Contenedor elemento : this.getElementos()){
      valorPagos += elemento.getValorFacturasAprobadasJunta(mes);
    }
    
    return valorPagos;
  }
  
  /**
   * Devuelve valor facturas aprobadas junta.
   * 
   * @param mesString
   *           mes string
   * @return valor facturas aprobadas junta
   */
  @Transient
  public Double getValorFacturasAprobadasJunta(String mesString){
    Mes mes = Mes.valueOf(mesString);
    return this.getValorFacturasAprobadasJunta(mes);
  }
  
  @Transient
  private Double getValorPagosFacturasAprobadasJunta(Date fechaHoraInicio, Date fechaHoraFin){
    Double valorFacturas = this.getValorPagosPropiosFacturasAprobadasJunta(fechaHoraInicio, fechaHoraFin);
    
    //Si el contenedor es de un subproyecto, añadimos la cuantía proporcional de las facturas
    //del contenedor del proyecto general.
    if(this.getContenedorProyectoGeneral() != null){
      for(Factura factura : this.getContenedorProyectoGeneral().getFacturas()){
        if(factura.getAprobadaJuntaConsulta() == true && factura.getEstadoActual().getTipoEstadoFactura() != TipoEstadoFactura.PRESUPUESTO_PENDIENTE_APROBACION && factura.getEstadoActual().getTipoEstadoFactura() != TipoEstadoFactura.PRESUPUESTO_RECHAZADO){
          valorFacturas += factura.getValorCuantiaSubpagosCentro(this.getCentro(), fechaHoraInicio, fechaHoraFin, null);
        }
      }
    }else{//if(this.getContenedorProyectoGeneral() == null) //En el caso de que tenga subproyectos, sumamos las facturas de los subcontenedores de los mismos.
      for(Contenedor contenedorSubproyecto : this.getContenedoresSubproyectos()){
        valorFacturas += contenedorSubproyecto.getValorPagosPropiosFacturasAprobadasJunta(fechaHoraInicio, fechaHoraFin);
      }
    }
    
    return valorFacturas;
  }
  
  /**
   * Devuelve valor pagos propios facturas aprobadas junta.
   * 
   * @param fechaHoraInicio
   *           fecha hora inicio
   * @param fechaHoraFin
   *           fecha hora fin
   * @return valor pagos propios facturas aprobadas junta
   */
  @Transient
  public Double getValorPagosPropiosFacturasAprobadasJunta(Date fechaHoraInicio, Date fechaHoraFin){
    Double valorFacturas = new Double(0);
    
    for(Factura factura : this.getFacturas()){
      if(factura.getAprobadaJuntaConsulta() == true && factura.getEstadoActual().getTipoEstadoFactura() != TipoEstadoFactura.PRESUPUESTO_PENDIENTE_APROBACION && factura.getEstadoActual().getTipoEstadoFactura() != TipoEstadoFactura.PRESUPUESTO_RECHAZADO){
        valorFacturas += factura.getValorCuantiaPagos(fechaHoraInicio, fechaHoraFin, null);
      }
    }
    
    return valorFacturas;
  }
  
  /**
   * Devuelve valor presupuestos.
   * 
   * @return valor presupuestos
   */
  @Transient
  public Double getValorPresupuestos(){
    Double valorFacturas = this.getValorPresupuestosPropios();
    
    //Si el contenedor es de un subproyecto, añadimos la cuantía proporcional de las facturas
    //del contenedor del proyecto general.
    if(this.getContenedorProyectoGeneral() != null){
      for(Factura factura : this.getContenedorProyectoGeneral().getFacturas()){
        if(factura.getEstadoActual().getTipoEstadoFactura() == TipoEstadoFactura.PRESUPUESTO_PENDIENTE_APROBACION){
          valorFacturas += factura.getValorCuantiaSubpagosCentro(this.getCentro(), null, null, null);
        }
      }
    }else{//if(this.getContenedorProyectoGeneral() == null) //En el caso de que tenga subproyectos, sumamos los presupuestos de los subcontenedores de los mismos.
      for(Contenedor contenedorSubproyecto : this.getContenedoresSubproyectos()){
        valorFacturas += contenedorSubproyecto.getValorPresupuestosPropios();
      }
    }
    
    for(Contenedor elemento : this.getElementos()){
      valorFacturas += elemento.getValorPresupuestos();
    }
    
    return valorFacturas;
  }
  
  /**
   * Devuelve valor presupuestos propios.
   * 
   * @return valor presupuestos propios
   */
  @Transient
  public Double getValorPresupuestosPropios(){
    Double valorFacturas = new Double(0);
    
    for(Factura factura : this.getFacturas()){
      if(factura.getEstadoActual().getTipoEstadoFactura() == TipoEstadoFactura.PRESUPUESTO_PENDIENTE_APROBACION){
        valorFacturas += factura.getValorCuantiaPagos(null, null, null);
      }
    }
    
    return valorFacturas;
  }
  
  /**
   * Devuelve desviacion prevision.
   * 
   * @return desviacion prevision
   */
  @Transient
  public Double getDesviacionPrevision(){
    Double desviacionPrevision = new Double(0);
    
    if(this.getPrevision().getCuantiaEfectiva() > 0){
      desviacionPrevision = (this.getValorFacturas() - this.getPrevision().getCuantiaEfectiva()) * new Double(100) / this.getPrevision().getCuantiaEfectiva();
    }else{
      desviacionPrevision = null;
    }
    
    if(desviacionPrevision != null && desviacionPrevision < 0){
      desviacionPrevision = new Double(0);
    }
    
    return desviacionPrevision;
  }
  
  /**
   * Devuelve desviacion prevision.
   * 
   * @param mes
   *           mes
   * @return desviacion prevision
   */
  @Transient
  public Double getDesviacionPrevision(Mes mes){
    Double desviacionPrevision = new Double(0);
    
    if(this.getPrevision().getCuantiaEfectivaMensual(mes) > 0){
      desviacionPrevision = (this.getValorFacturas(mes) - this.getPrevision().getCuantiaEfectivaMensual(mes)) * new Double(100) / this.getPrevision().getCuantiaEfectivaMensual(mes);
    }else{
      desviacionPrevision = null;
    }
    
    if(desviacionPrevision != null && desviacionPrevision < 0){
      desviacionPrevision = new Double(0);
    }
    
    return desviacionPrevision;
  }
  
  /**
   * Devuelve desviacion prevision.
   * 
   * @param mesString
   *           mes string
   * @return desviacion prevision
   */
  @Transient
  public Double getDesviacionPrevision(String mesString){
    Mes mes = Mes.valueOf(mesString);
    return this.getDesviacionPrevision(mes);
  }
  
  /**
   * Devuelve desviacion prevision evolutiva.
   * 
   * @return desviacion prevision evolutiva
   */
  @Transient
  public Double getDesviacionPrevisionEvolutiva(){
    Double desviacionPrevision = new Double(0);
    
    if(this.getPrevision().getCuantiaEvolutiva() > 0){
      desviacionPrevision = (this.getValorFacturas() - this.getPrevision().getCuantiaEvolutiva()) * new Double(100) / this.getPrevision().getCuantiaEvolutiva();
    }else{
      desviacionPrevision = null;
    }
    
    if(desviacionPrevision != null && desviacionPrevision < 0){
      desviacionPrevision = new Double(0);
    }
    
    return desviacionPrevision;
  }
  
  /**
   * Devuelve desviacion prevision evolutiva.
   * 
   * @param mes
   *           mes
   * @return desviacion prevision evolutiva
   */
  @Transient
  public Double getDesviacionPrevisionEvolutiva(Mes mes){
    Double desviacionPrevision = new Double(0);
    
    if(this.getPrevision().getCuantiaEvolutivaEfectivaMensual(mes) > 0){
      desviacionPrevision = (this.getValorFacturas(mes) - this.getPrevision().getCuantiaEvolutivaEfectivaMensual(mes)) * new Double(100) / this.getPrevision().getCuantiaEvolutivaEfectivaMensual(mes);
    }else{
      desviacionPrevision = null;
    }
    
    if(desviacionPrevision != null && desviacionPrevision < 0){
      desviacionPrevision = new Double(0);
    }
    
    return desviacionPrevision;
  }
  
  /**
   * Devuelve desviacion prevision evolutiva.
   * 
   * @param mesString
   *           mes string
   * @return desviacion prevision evolutiva
   */
  @Transient
  public Double getDesviacionPrevisionEvolutiva(String mesString){
    Mes mes = Mes.valueOf(mesString);
    return this.getDesviacionPrevisionEvolutiva(mes);
  }
  
  /**
   * Checks if is expandido.
   * 
   * @return true, if is expandido
   */
  @Transient
  public boolean isExpandido() {
    if(this.expandido && (this.getContenedorPadre() == null || this.getContenedorPadre().isExpandido() == true))
      return true;
    else
      return false;
  }

  /**
   * Asigna expandido.
   * 
   * @param expandido
   *          expandido
   */
  public void setExpandido(boolean expandido) {
    this.expandido = expandido;
  }

  /**
   * Devuelve grado desviacion prevision.
   * 
   * @return grado desviacion prevision
   */
  @Transient
  public GradoDesviacion getGradoDesviacionPrevision(){
    if(this.getDesviacionPrevision() != null){
      if(this.getDesviacionPrevision() >= this.gradoDesviacionAlta){
        return GradoDesviacion.ALTO;
      }else if(this.getDesviacionPrevision() >= this.gradoDesviacionMedia){
        return GradoDesviacion.MEDIO;
      }else
        return GradoDesviacion.BAJO;
    }
    return null;
  }
  
  /**
   * Devuelve grado desviacion prevision.
   * 
   * @param mes
   *           mes
   * @return grado desviacion prevision
   */
  @Transient
  public GradoDesviacion getGradoDesviacionPrevision(Mes mes){
    if(this.getDesviacionPrevision(mes) != null){
      if(this.getDesviacionPrevision(mes) >= this.gradoDesviacionAlta){
        return GradoDesviacion.ALTO;
      }else if(this.getDesviacionPrevision(mes) >= this.gradoDesviacionMedia){
        return GradoDesviacion.MEDIO;
      }else
        return GradoDesviacion.BAJO;
    }
    return null;
  }
  
  /**
   * Devuelve grado desviacion prevision.
   * 
   * @param mesString
   *           mes string
   * @return grado desviacion prevision
   */
  @Transient
  public GradoDesviacion getGradoDesviacionPrevision(String mesString){
    Mes mes = Mes.valueOf(mesString);
    return this.getGradoDesviacionPrevision(mes);
  }
  
  /**
   * Devuelve grado desviacion prevision evolutiva.
   * 
   * @return grado desviacion prevision evolutiva
   */
  @Transient
  public GradoDesviacion getGradoDesviacionPrevisionEvolutiva(){
    if(this.getDesviacionPrevisionEvolutiva() != null){
      if(this.getDesviacionPrevisionEvolutiva() >= this.gradoDesviacionAlta){
        return GradoDesviacion.ALTO;
      }else if(this.getDesviacionPrevisionEvolutiva() >= this.gradoDesviacionMedia){
        return GradoDesviacion.MEDIO;
      }else
        return GradoDesviacion.BAJO;
    }
    return null;
  }
  
  /**
   * Devuelve grado desviacion prevision evolutiva.
   * 
   * @param mes
   *           mes
   * @return grado desviacion prevision evolutiva
   */
  @Transient
  public GradoDesviacion getGradoDesviacionPrevisionEvolutiva(Mes mes){
    if(this.getDesviacionPrevisionEvolutiva() != null){
      if(this.getDesviacionPrevisionEvolutiva(mes) >= this.gradoDesviacionAlta){
        return GradoDesviacion.ALTO;
      }else if(this.getDesviacionPrevisionEvolutiva(mes) >= this.gradoDesviacionMedia){
        return GradoDesviacion.MEDIO;
      }else
        return GradoDesviacion.BAJO;
    }
    return null;
  }
  
  /**
   * Devuelve grado desviacion prevision evolutiva.
   * 
   * @param mesString
   *           mes string
   * @return grado desviacion prevision evolutiva
   */
  @Transient
  public GradoDesviacion getGradoDesviacionPrevisionEvolutiva(String mesString){
    Mes mes = Mes.valueOf(mesString);
    return this.getGradoDesviacionPrevisionEvolutiva(mes);
  }
  
  /**
   * Devuelve fecha mes.
   * 
   * @param mes
   *           mes
   * @param anyo
   *           anyo
   * @param inicio
   *           inicio
   * @return fecha mes
   */
  @Transient
  public Date getFechaMes(Mes mes, int anyo, boolean inicio){
    int mesInt = 0;
    int dia = 1;
    
    switch(mes){
      case Enero:
        mesInt = Calendar.JANUARY;
      break;
      case Febrero:
        mesInt = Calendar.FEBRUARY;
      break;
      case Marzo:
        mesInt = Calendar.MARCH;
      break;
      case Abril:
        mesInt = Calendar.APRIL;
      break;
      case Mayo:
        mesInt = Calendar.MAY;
      break;
      case Junio:
        mesInt = Calendar.JUNE;
      break;
      case Julio:
        mesInt = Calendar.JULY;
      break;
      case Agosto:
        mesInt = Calendar.AUGUST;
      break;
      case Septiembre:
        mesInt = Calendar.SEPTEMBER;
      break;
      case Octubre:
        mesInt = Calendar.OCTOBER;
      break;
      case Noviembre:
        mesInt = Calendar.NOVEMBER;
      break;
      case Diciembre:
        mesInt = Calendar.DECEMBER;
      break;
    }
    
    GregorianCalendar calendario = new GregorianCalendar(anyo, mesInt, dia, 0, 0, 0);
    
    if(!inicio){
      calendario.add(Calendar.MONTH, 1);
    }
    
    Date fecha = calendario.getTime();
    
    return fecha;
  }
  
  //Método utilizado en informes de financiero para obtener la propia instancia del objeto del data y acceder a sus
  //métodos.
  /**
   * Devuelve esta instancia.
   * 
   * @return esta instancia
   */
  @Transient
  public Contenedor getEstaInstancia(){
    return this;
  }
  
  /**
   * Formatear cuantias informes.
   * 
   * @param cuantia
   *           cuantia
   * @return string
   */
  @Transient
  public String formatearCuantiasInformes(Double cuantia){
    String cadenaFormateada = "";
    if(cuantia >= 1000000){
      cadenaFormateada = new DecimalFormat("#,##0.00").format(cuantia/new Double(1000000)).concat(" MM");
    }else{
      if(cuantia >= 10000){
        cadenaFormateada = new DecimalFormat("#,##0").format(cuantia);
      }else{
        cadenaFormateada = new DecimalFormat("#,##0.00").format(cuantia);
      }
    }
    
    return cadenaFormateada;
  }
}
