package es.csa.fdt.cyldigital.financiero.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.Date;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToOne;
import javax.persistence.OrderBy;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.core.entity.FileOnDisk;

/**
 * Clase DocumentoFactura.
 * 
 * @author csa
 */
@Entity
@Table(name = "documentosfactura")
@AttributeOverride(column = @Column(name = "idDocumentoFactura"), name = "id")
@Audited
public class DocumentoFactura extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = 4385689438560860895L;

  /**
   * Enum TipoDocumentoFactura.
   */
  public enum TipoDocumentoFactura {
    FACTURA,
    JUSTIFICACION_PAGO
  }

  /**
   * Enum EstadoDocumentoFactura.
   */
  public enum EstadoDocumentoFactura {
    PENDIENTE_JUSTIFICACION,
    PENDIENTE_APROBACION_JUNTA,
    APROBADA_JUNTA
  }

  private TipoDocumentoFactura tipoDocumentoFactura;

  private String nombre;

  private EstadoDocumentoFactura estadoDocumentoFactura;

  private Date fecha;

  private FileFactura fichero;
  private FileOnDisk ficheroEnDisco;

  private Pago pago;

  private List<Factura> facturas;

  /**
   * Devuelve tipo documento factura.
   * 
   * @return tipo documento factura
   */
  @NotNull
  @Enumerated(EnumType.STRING)
  public TipoDocumentoFactura getTipoDocumentoFactura() {
    return tipoDocumentoFactura;
  }

  /**
   * Asigna tipo documento factura.
   * 
   * @param tipoDocumentoFactura
   *          tipo documento factura
   */
  public void setTipoDocumentoFactura(TipoDocumentoFactura tipoDocumentoFactura) {
    this.tipoDocumentoFactura = tipoDocumentoFactura;
  }

  /**
   * Devuelve nombre.
   * 
   * @return nombre
   */
  @NotNull
  public String getNombre() {
    return nombre;
  }

  /**
   * Asigna nombre.
   * 
   * @param nombre
   *          nombre
   */
  public void setNombre(String nombre) {
    this.nombre = nombre;
  }

  /**
   * Devuelve estado documento factura.
   * 
   * @return estado documento factura
   */
  @Enumerated(EnumType.STRING)
  public EstadoDocumentoFactura getEstadoDocumentoFactura() {
    return estadoDocumentoFactura;
  }

  /**
   * Asigna estado documento factura.
   * 
   * @param estadoDocumentoFactura
   *          estado documento factura
   */
  public void setEstadoDocumentoFactura(EstadoDocumentoFactura estadoDocumentoFactura) {
    this.estadoDocumentoFactura = estadoDocumentoFactura;
  }

  /**
   * Devuelve fecha.
   * 
   * @return fecha
   */
  @NotNull
  @Temporal(TemporalType.DATE)
  public Date getFecha() {
    return fecha;
  }

  /**
   * Asigna fecha.
   * 
   * @param fecha
   *          fecha
   */
  public void setFecha(Date fecha) {
    this.fecha = fecha;
  }

  /**
   * Devuelve fichero.
   * 
   * @return fichero
   */
  @ManyToOne(cascade = { javax.persistence.CascadeType.ALL })
  @JoinColumn(name = "idFichero")
  public FileFactura getFichero() {
    return fichero;
  }

  /**
   * Asigna fichero.
   * 
   * @param fichero
   *          fichero
   */
  public void setFichero(FileFactura fichero) {
    this.fichero = fichero;
  }

  /**
   * Devuelve facturas.
   * 
   * @return facturas
   */
  @NotAudited
  @ManyToMany(targetEntity = Factura.class)
  @JoinTable(name = "documentos_facturas", joinColumns = @JoinColumn(name = "idDocumentoFactura"), inverseJoinColumns = @JoinColumn(name = "idFactura"))
  @OrderBy(value = "id")
  public List<Factura> getFacturas() {
    return facturas;
  }

  /**
   * Asigna facturas.
   * 
   * @param facturas
   *          facturas
   */
  public void setFacturas(List<Factura> facturas) {
    this.facturas = facturas;
  }

  /**
   * Devuelve pago.
   * 
   * @return pago
   */
  @OneToOne
  @JoinColumn(name = "idPago")
  public Pago getPago() {
    return pago;
  }

  /**
   * Asigna pago.
   * 
   * @param pago
   *          pago
   */
  public void setPago(Pago pago) {
    this.pago = pago;
  }

  /**
   * Devuelve fichero en disco.
   * 
   * @return fichero en disco
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idFicheroEnDisco")
  public FileOnDisk getFicheroEnDisco() {
    return ficheroEnDisco;
  }

  /**
   * Asigna fichero en disco.
   * 
   * @param ficheroEnDisco
   *          fichero en disco
   */
  public void setFicheroEnDisco(FileOnDisk ficheroEnDisco) {
    this.ficheroEnDisco = ficheroEnDisco;
  }

}
