package es.csa.fdt.cyldigital.financiero.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.Date;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.envers.Audited;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.core.entity.FileOnDisk;

/**
 * Clase DocumentoMovimiento.
 * 
 * @author csa
 */
@Entity
@Table(name = "documentosmovimiento")
@AttributeOverride(column = @Column(name = "idDocumentoMovimiento"), name = "id")
@Audited
public class DocumentoMovimiento extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = 5894230758907423890L;

  private String nombre;

  private Date fecha;

  private FileMovimiento fichero;
  private FileOnDisk file;

  private Movimiento movimiento;

  /**
   * Devuelve nombre.
   * 
   * @return nombre
   */
  @NotNull
  public String getNombre() {
    return nombre;
  }

  /**
   * Asigna nombre.
   * 
   * @param nombre
   *          nombre
   */
  public void setNombre(String nombre) {
    this.nombre = nombre;
  }

  /**
   * Devuelve fecha.
   * 
   * @return fecha
   */
  @NotNull
  @Temporal(TemporalType.DATE)
  public Date getFecha() {
    return fecha;
  }

  /**
   * Asigna fecha.
   * 
   * @param fecha
   *          fecha
   */
  public void setFecha(Date fecha) {
    this.fecha = fecha;
  }

  /**
   * Devuelve fichero.
   * 
   * @return fichero
   */
  @ManyToOne(cascade = { javax.persistence.CascadeType.ALL })
  @JoinColumn(name = "idFichero")
  public FileMovimiento getFichero() {
    return fichero;
  }

  /**
   * Asigna fichero.
   * 
   * @param fichero
   *          fichero
   */
  public void setFichero(FileMovimiento fichero) {
    this.fichero = fichero;
  }

  /**
   * Devuelve movimiento.
   * 
   * @return movimiento
   */
  @OneToOne
  @JoinColumn(name = "idMovimiento")
  public Movimiento getMovimiento() {
    return movimiento;
  }

  /**
   * Asigna movimiento.
   * 
   * @param movimiento
   *          movimiento
   */
  public void setMovimiento(Movimiento movimiento) {
    this.movimiento = movimiento;
  }

  /**
   * Devuelve file.
   * 
   * @return file
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idFile")
  public FileOnDisk getFile() {
    return file;
  }

  /**
   * Asigna file.
   * 
   * @param file
   *          file
   */
  public void setFile(FileOnDisk file) {
    this.file = file;
  }
}
