package es.csa.fdt.cyldigital.financiero.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.usuarios.entity.Usuario;

/**
 * Clase EstadoFactura.
 * 
 * @author csa
 */
@Entity
@Table(name="estadosfactura")
@AttributeOverride(column = @Column(name = "idEstadoFactura"), name = "id")
@Audited
public class EstadoFactura extends EstadoFinanciero implements Serializable{
  
  private static final long serialVersionUID = 2437597423895039437L;

  /**
   * Enum TipoEstadoFactura.
   */
  public enum TipoEstadoFactura{
    PRESUPUESTO_PENDIENTE_APROBACION,
    PENDIENTE_COMPRA,
    PRESUPUESTO_RECHAZADO,
    PENDIENTE_PAGO,
    PAGADA,
    REVISADA,
    ARCHIVADA
  }
  
  /** tipo estado factura. */
  public TipoEstadoFactura tipoEstadoFactura;
  
  /** factura. */
  public Factura factura;
  
  /**
   * Construye un nuevo objeto de la clase
   */
  public EstadoFactura(){
  }
  
  /**
   * Construye un nuevo objeto de la clase
   * 
   * @param tipoEstadoFactura
   *           tipo estado factura
   * @param factura
   *           factura
   * @param fecha
   *           fecha
   * @param usuario
   *           usuario
   */
  public EstadoFactura(TipoEstadoFactura tipoEstadoFactura, Factura factura, Date fecha, Usuario usuario){
    this();
    
    this.setTipoEstadoFactura(tipoEstadoFactura);
    this.setFactura(factura);
    this.setFecha(fecha);
    this.setUsuario(usuario);
  }
  
  /**
   * Construye un nuevo objeto de la clase
   * 
   * @param tipoEstadoFactura
   *           tipo estado factura
   * @param factura
   *           factura
   * @param fecha
   *           fecha
   * @param usuario
   *           usuario
   * @param observaciones
   *           observaciones
   */
  public EstadoFactura(TipoEstadoFactura tipoEstadoFactura, Factura factura, Date fecha, Usuario usuario, String observaciones){
    this(tipoEstadoFactura, factura, fecha, usuario);
    
    this.setObservaciones(observaciones);
  }

  /**
   * Devuelve tipo estado factura.
   * 
   * @return tipo estado factura
   */
  @NotNull
  @Enumerated(value = EnumType.STRING)
  public TipoEstadoFactura getTipoEstadoFactura() {
    return tipoEstadoFactura;
  }

  /**
   * Asigna tipo estado factura.
   * 
   * @param tipoEstadoFactura
   *          tipo estado factura
   */
  public void setTipoEstadoFactura(TipoEstadoFactura tipoEstadoFactura) {
    this.tipoEstadoFactura = tipoEstadoFactura;
  }

  /**
   * Devuelve factura.
   * 
   * @return factura
   */
  @NotNull
  @ManyToOne
  @JoinColumn(name = "idFactura")
  public Factura getFactura() {
    return factura;
  }

  /**
   * Asigna factura.
   * 
   * @param factura
   *          factura
   */
  public void setFactura(Factura factura) {
    this.factura = factura;
  }
  
  /**
   * Siguientes tipos estado factura.
   * 
   * @return list
   */
  @Transient
  public List<TipoEstadoFactura> siguientesTiposEstadoFactura(){
    List<TipoEstadoFactura> siguientesEstados = new ArrayList<TipoEstadoFactura>();
    
    if(this.tipoEstadoFactura == TipoEstadoFactura.PRESUPUESTO_PENDIENTE_APROBACION){
      siguientesEstados.add(TipoEstadoFactura.PENDIENTE_COMPRA);
      siguientesEstados.add(TipoEstadoFactura.PRESUPUESTO_RECHAZADO);
    }else if(this.tipoEstadoFactura == TipoEstadoFactura.PRESUPUESTO_RECHAZADO){
    }else if(this.tipoEstadoFactura == TipoEstadoFactura.PENDIENTE_COMPRA){
      siguientesEstados.add(TipoEstadoFactura.PENDIENTE_PAGO);
    }else if(this.tipoEstadoFactura == TipoEstadoFactura.PENDIENTE_PAGO){
      siguientesEstados.add(TipoEstadoFactura.PAGADA);
    }else if(this.tipoEstadoFactura == TipoEstadoFactura.PAGADA){
      siguientesEstados.add(TipoEstadoFactura.REVISADA);
    }else if(this.tipoEstadoFactura == TipoEstadoFactura.REVISADA){
      siguientesEstados.add(TipoEstadoFactura.ARCHIVADA);
    }else if(this.tipoEstadoFactura == TipoEstadoFactura.ARCHIVADA){
    }
    
    return siguientesEstados; 
  }
  
  /**
   * Anterior tipo estado factura.
   * 
   * @return tipo estado factura
   */
  @Transient
  public TipoEstadoFactura anteriorTipoEstadoFactura(){
    TipoEstadoFactura anteriorTipoEstado = null;
    
    if(this.tipoEstadoFactura == TipoEstadoFactura.PRESUPUESTO_PENDIENTE_APROBACION){
    }else if(this.tipoEstadoFactura == TipoEstadoFactura.PRESUPUESTO_RECHAZADO){
      anteriorTipoEstado = TipoEstadoFactura.PRESUPUESTO_PENDIENTE_APROBACION;
    }else if(this.tipoEstadoFactura == TipoEstadoFactura.PENDIENTE_COMPRA){
      anteriorTipoEstado = TipoEstadoFactura.PRESUPUESTO_PENDIENTE_APROBACION;
    }else if(this.tipoEstadoFactura == TipoEstadoFactura.PENDIENTE_PAGO){
      anteriorTipoEstado = TipoEstadoFactura.PENDIENTE_COMPRA;
    }else if(this.tipoEstadoFactura == TipoEstadoFactura.PAGADA){
      anteriorTipoEstado = TipoEstadoFactura.PENDIENTE_PAGO;
    }else if(this.tipoEstadoFactura == TipoEstadoFactura.REVISADA){
      anteriorTipoEstado = TipoEstadoFactura.PAGADA;
    }else if(this.tipoEstadoFactura == TipoEstadoFactura.ARCHIVADA){
      anteriorTipoEstado = TipoEstadoFactura.REVISADA;
    }
    
    return anteriorTipoEstado;
  }
  
  /**
   * Checks if is estado final.
   * 
   * @return true, if is estado final
   */
  @Transient
  public boolean isEstadoFinal(){
    boolean result = false; 
    if(this.getTipoEstadoFactura() == TipoEstadoFactura.ARCHIVADA)
      result = true;
    else if(this.getTipoEstadoFactura() == TipoEstadoFactura.PRESUPUESTO_RECHAZADO)
      result = true;
    
    return result;
  }
  
  /**
   * Checks if is estado modificable responsable area.
   * 
   * @return true, if is estado modificable responsable area
   */
  @Transient
  public boolean isEstadoModificableResponsableArea()
  {
    boolean result = false;
    if(this.getTipoEstadoFactura() == TipoEstadoFactura.PENDIENTE_COMPRA)
      result = true;
    
    return result;
  }
  
  /**
   * Checks if is estado aprobacion presupuesto.
   * 
   * @return true, if is estado aprobacion presupuesto
   */
  @Transient
  public boolean isEstadoAprobacionPresupuesto()
  {
    boolean result = false;
    if(this.getTipoEstadoFactura() == TipoEstadoFactura.PRESUPUESTO_PENDIENTE_APROBACION)
      result = true;
    
    return result;
  }
  
  /**
   * Checks if is estado modificacion pagos.
   * 
   * @return true, if is estado modificacion pagos
   */
  @Transient
  public boolean isEstadoModificacionPagos(){
    boolean result = false;
    if(this.getTipoEstadoFactura() == TipoEstadoFactura.PENDIENTE_PAGO)
      result = true;
    if(this.getTipoEstadoFactura() == TipoEstadoFactura.PAGADA)
      result = true;
    if(this.getTipoEstadoFactura() == TipoEstadoFactura.REVISADA)
      result = true;
  
  return result;
  }
  
  /**
   * Checks if is estado modificacion facturas.
   * 
   * @return true, if is estado modificacion facturas
   */
  @Transient
  public boolean isEstadoModificacionFacturas(){
    boolean result = false;
    if(this.getTipoEstadoFactura() == TipoEstadoFactura.PRESUPUESTO_PENDIENTE_APROBACION)
      result = true;
    if(this.getTipoEstadoFactura() == TipoEstadoFactura.PENDIENTE_COMPRA)
      result = true;
    if(this.getTipoEstadoFactura() == TipoEstadoFactura.PRESUPUESTO_RECHAZADO)
      result = true;
    if(this.getTipoEstadoFactura() == TipoEstadoFactura.ARCHIVADA)
      result = true;
  
  return result;
  }

}
