package es.csa.fdt.cyldigital.financiero.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.usuarios.entity.Usuario;

/**
 * Clase EstadoPago.
 * 
 * @author csa
 */
@Entity
@Table(name="estadospago")
@AttributeOverride(column = @Column(name = "idEstadoPago"), name = "id")
@Audited
public class EstadoPago extends EstadoFinanciero implements Serializable{
  
  private static final long serialVersionUID = 4758904237897758976L;

  /**
   * Enum TipoEstadoPago.
   */
  public enum TipoEstadoPago{
    PENDIENTE_PAGO,
    PAGADO,
    REVISADO
  }
  
  /** tipo estado pago. */
  public TipoEstadoPago tipoEstadoPago;
  
  /** pago. */
  public Pago pago;
  
  /**
   * Construye un nuevo objeto de la clase
   */
  public EstadoPago(){
  }
  
  /**
   * Construye un nuevo objeto de la clase
   * 
   * @param tipoEstadoPago
   *           tipo estado pago
   * @param pago
   *           pago
   * @param fecha
   *           fecha
   * @param usuario
   *           usuario
   */
  public EstadoPago(TipoEstadoPago tipoEstadoPago, Pago pago, Date fecha, Usuario usuario){
    this();
    
    this.setTipoEstadoPago(tipoEstadoPago);
    this.setPago(pago);
    this.setFecha(fecha);
    this.setUsuario(usuario);
  }
  
  /**
   * Construye un nuevo objeto de la clase
   * 
   * @param tipoEstadoPago
   *           tipo estado pago
   * @param pago
   *           pago
   * @param fecha
   *           fecha
   * @param usuario
   *           usuario
   * @param observaciones
   *           observaciones
   */
  public EstadoPago(TipoEstadoPago tipoEstadoPago, Pago pago, Date fecha, Usuario usuario, String observaciones){
    this(tipoEstadoPago, pago, fecha, usuario);
    
    this.setObservaciones(observaciones);
  }

  /**
   * Devuelve tipo estado pago.
   * 
   * @return tipo estado pago
   */
  @NotNull
  @Enumerated(value = EnumType.STRING)
  public TipoEstadoPago getTipoEstadoPago() {
    return tipoEstadoPago;
  }

  /**
   * Asigna tipo estado pago.
   * 
   * @param tipoEstadoPago
   *          tipo estado pago
   */
  public void setTipoEstadoPago(TipoEstadoPago tipoEstadoPago) {
    this.tipoEstadoPago = tipoEstadoPago;
  }

  /**
   * Devuelve pago.
   * 
   * @return pago
   */
  @NotNull
  @ManyToOne
  @JoinColumn(name = "idPago")
  public Pago getPago() {
    return pago;
  }

  /**
   * Asigna pago.
   * 
   * @param pago
   *          pago
   */
  public void setPago(Pago pago) {
    this.pago = pago;
  }
  
  /**
   * Devuelve siguientes tipos estado pago.
   * 
   * @return siguientes tipos estado pago
   */
  @Transient
  public List<TipoEstadoPago> getSiguientesTiposEstadoPago(){
    List<TipoEstadoPago> siguientesEstados = new ArrayList<TipoEstadoPago>();
    
    if(this.tipoEstadoPago == TipoEstadoPago.PENDIENTE_PAGO){
      siguientesEstados.add(TipoEstadoPago.PAGADO);
    }else if(this.tipoEstadoPago == TipoEstadoPago.PAGADO){
      siguientesEstados.add(TipoEstadoPago.REVISADO);
    }else if(this.tipoEstadoPago == TipoEstadoPago.REVISADO){
    }
    
    return siguientesEstados; 
  }
  
  /**
   * Devuelve anterior tipo estado pago.
   * 
   * @return anterior tipo estado pago
   */
  @Transient
  public TipoEstadoPago getAnteriorTipoEstadoPago(){
    TipoEstadoPago anteriorTipoEstado = null;
    
    if(this.tipoEstadoPago == TipoEstadoPago.PENDIENTE_PAGO){
    }else if(this.tipoEstadoPago == TipoEstadoPago.PAGADO){
      anteriorTipoEstado = TipoEstadoPago.PENDIENTE_PAGO;
    }else if(this.tipoEstadoPago == TipoEstadoPago.REVISADO){
      anteriorTipoEstado = TipoEstadoPago.PAGADO;
    }
    
    return anteriorTipoEstado;
  }
  
  /**
   * Checks if is estado final.
   * 
   * @return true, if is estado final
   */
  @Transient
  public boolean isEstadoFinal(){
    boolean result = false; 
    if(this.getTipoEstadoPago() == TipoEstadoPago.REVISADO)
      result = true;
    
    return result;
  }
  
  /**
   * Checks if is estado solo modificable responsable area.
   * 
   * @return true, if is estado solo modificable responsable area
   */
  @Transient
  public boolean isEstadoSoloModificableResponsableArea()
  {
    boolean result = false;
    if(this.getTipoEstadoPago() == TipoEstadoPago.PAGADO)
      result = true;
    
    return result;
  }
  
  /**
   * Checks if is estado modificacion facturas.
   * 
   * @return true, if is estado modificacion facturas
   */
  @Transient
  public boolean isEstadoModificacionFacturas(){
    EstadoFactura estadoFactura = this.getPago().getFactura().getEstadoActual();
    
    return estadoFactura.isEstadoModificacionFacturas();
  }

}
