package es.csa.fdt.cyldigital.financiero.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.OrderBy;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.financiero.entity.DocumentoFactura.EstadoDocumentoFactura;
import es.csa.fdt.cyldigital.financiero.entity.EstadoPago.TipoEstadoPago;
import es.csa.fdt.cyldigital.financiero.entity.Pago.TipoCuantiaSubproyectos;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.Proveedor;

/**
 * Clase Factura.
 * 
 * @author csa
 */
@Entity
@Table(name = "facturas")
@AttributeOverride(column = @Column(name = "idFactura"), name = "id")
@Audited
public class Factura extends AbstractEntity implements Serializable, Comparable<Factura>{

  private static final long serialVersionUID = 3467843027590842375L;
  
  /** Constante MAXPRESUPUESTOPEQUENO. */
  public static final long MAXPRESUPUESTOPEQUENO = 10000;
  
  /** Constante MAXPRESUPUESTOMEDIANO. */
  public static final long MAXPRESUPUESTOMEDIANO = 100000;
  
  /** Constante MAXPRESUPUESTOGRANDE. */
  public static final long MAXPRESUPUESTOGRANDE = 1000000;
  
  /**
   * Enum TipoPago.
   */
  public enum TipoPago{
    UNICO,
    MENSUAL,
    PLAZOS_FIJOS
  }
  
  private Contenedor contenedor;
  
  private Contrato contrato;
  
  private Proveedor proveedor;
  
  private String concepto;
  
  private Double cuantiaPresupuesto;
  
  private Date fechaIntroduccion;
  
  private List<EstadoFactura> estadosFactura;
  
  private EstadoFactura estadoActual;
  
  private TipoPago tipoPago;
  
  private Date fechaInicioPagos;
  
  private Date fechaFinPagos;
  
  private List<Pago> pagos;
  
  private List<DocumentoFactura> documentosFactura;
  
  private boolean justificadaConsulta;//Transient
  
  private boolean aprobadaJuntaConsulta;//Transient
  
  /**
   * Construye un nuevo objeto de la clase
   */
  public Factura(){
    this.setEstadosFactura(new ArrayList<EstadoFactura>());
    this.pagos = new ArrayList<Pago>();
    this.setDocumentosFactura(new ArrayList<DocumentoFactura>());
  }

  /**
   * Devuelve maxpresupuestopequeno.
   * 
   * @return maxpresupuestopequeno
   */
  public static long getMaxpresupuestopequeno() {
    return MAXPRESUPUESTOPEQUENO;
  }

  /**
   * Devuelve maxpresupuestomediano.
   * 
   * @return maxpresupuestomediano
   */
  public static long getMaxpresupuestomediano() {
    return MAXPRESUPUESTOMEDIANO;
  }

  /**
   * Devuelve maxpresupuestogrande.
   * 
   * @return maxpresupuestogrande
   */
  public static long getMaxpresupuestogrande() {
    return MAXPRESUPUESTOGRANDE;
  }

  /**
   * Devuelve contenedor.
   * 
   * @return contenedor
   */
  @ManyToOne
  @JoinColumn(name = "idContenedor")
  public Contenedor getContenedor() {
    return contenedor;
  }

  /**
   * Asigna contenedor.
   * 
   * @param contenedor
   *          contenedor
   */
  public void setContenedor(Contenedor contenedor) {
    this.contenedor = contenedor;
  }
  
  /**
   * Devuelve contrato.
   * 
   * @return contrato
   */
  @ManyToOne
  @JoinColumn(name = "idContrato")
  public Contrato getContrato() {
    return contrato;
  }

  /**
   * Asigna contrato.
   * 
   * @param contrato
   *          contrato
   */
  public void setContrato(Contrato contrato) {
    this.contrato = contrato;
  }

  /**
   * Devuelve proveedor.
   * 
   * @return proveedor
   */
  @ManyToOne
  @JoinColumn(name = "idProveedor")
  public Proveedor getProveedor() {
    return proveedor;
  }

  /**
   * Asigna proveedor.
   * 
   * @param proveedor
   *          proveedor
   */
  public void setProveedor(Proveedor proveedor) {
    this.proveedor = proveedor;
  }

  /**
   * Devuelve concepto.
   * 
   * @return concepto
   */
  @NotNull
  @NotEmpty
  public String getConcepto() {
    return concepto;
  }

  /**
   * Asigna concepto.
   * 
   * @param concepto
   *          concepto
   */
  public void setConcepto(String concepto) {
    this.concepto = concepto;
  }

  /**
   * Devuelve cuantia presupuesto.
   * 
   * @return cuantia presupuesto
   */
  @NotNull
  public Double getCuantiaPresupuesto() {
    return cuantiaPresupuesto;
  }

  /**
   * Asigna cuantia presupuesto.
   * 
   * @param cuantia
   *          cuantia presupuesto
   */
  public void setCuantiaPresupuesto(Double cuantia) {
    this.cuantiaPresupuesto = cuantia;
  }
  
  /**
   * Devuelve fecha introduccion.
   * 
   * @return fecha introduccion
   */
  @Temporal(value = TemporalType.TIMESTAMP)
  public Date getFechaIntroduccion() {
    return fechaIntroduccion;
  }

  /**
   * Asigna fecha introduccion.
   * 
   * @param fechaIntroduccion
   *          fecha introduccion
   */
  public void setFechaIntroduccion(Date fechaIntroduccion) {
    this.fechaIntroduccion = fechaIntroduccion;
  }
  
  /**
   * Devuelve estados factura.
   * 
   * @return estados factura
   */
  @OneToMany(mappedBy = "factura", cascade = CascadeType.ALL)
  @OrderBy(value = "fecha DESC")
  public List<EstadoFactura> getEstadosFactura() {
    return estadosFactura;
  }

  /**
   * Asigna estados factura.
   * 
   * @param estadosFactura
   *          estados factura
   */
  public void setEstadosFactura(List<EstadoFactura> estadosFactura) {
    this.estadosFactura = estadosFactura;
  }
  
  /**
   * Devuelve estado actual.
   * 
   * @return estado actual
   */
  @OneToOne
  @JoinColumn(name = "idEstadoActual")
  public EstadoFactura getEstadoActual() {
    return this.estadoActual;
  }

  /**
   * Asigna estado actual.
   * 
   * @param estadoActual
   *          estado actual
   */
  public void setEstadoActual(EstadoFactura estadoActual) {
    this.estadoActual = estadoActual;
  }
  
  /**
   * Devuelve tipo pago.
   * 
   * @return tipo pago
   */
  @Enumerated(value = EnumType.STRING)
  public TipoPago getTipoPago() {
    return tipoPago;
  }

  /**
   * Asigna tipo pago.
   * 
   * @param tipoPago
   *          tipo pago
   */
  public void setTipoPago(TipoPago tipoPago) {
    this.tipoPago = tipoPago;
  }
  
  /**
   * Devuelve fecha inicio pagos.
   * 
   * @return fecha inicio pagos
   */
  public Date getFechaInicioPagos() {
    return fechaInicioPagos;
  }

  /**
   * Asigna fecha inicio pagos.
   * 
   * @param fechaInicioPagos
   *          fecha inicio pagos
   */
  public void setFechaInicioPagos(Date fechaInicioPagos) {
    this.fechaInicioPagos = fechaInicioPagos;
  }

  /**
   * Devuelve fecha fin pagos.
   * 
   * @return fecha fin pagos
   */
  public Date getFechaFinPagos() {
    return fechaFinPagos;
  }

  /**
   * Asigna fecha fin pagos.
   * 
   * @param fechaFinPagos
   *          fecha fin pagos
   */
  public void setFechaFinPagos(Date fechaFinPagos) {
    this.fechaFinPagos = fechaFinPagos;
  }

  /**
   * Devuelve pagos.
   * 
   * @return pagos
   */
  @OneToMany(mappedBy = "factura", cascade = CascadeType.ALL,fetch=FetchType.EAGER)
  public List<Pago> getPagos() {
    return pagos;
  }

  /**
   * Asigna pagos.
   * 
   * @param pagos
   *          pagos
   */
  public void setPagos(List<Pago> pagos) {
    this.pagos = pagos;
  }

  /**
   * Devuelve documentos factura.
   * 
   * @return documentos factura
   */
  @NotAudited
  @ManyToMany(targetEntity = DocumentoFactura.class)
  @JoinTable(name = "documentos_facturas", joinColumns = @JoinColumn(name = "idFactura"), inverseJoinColumns = @JoinColumn(name = "idDocumentoFactura"))
  @OrderBy(value = "id")
  public List<DocumentoFactura> getDocumentosFactura() {
    return documentosFactura;
  }

  /**
   * Asigna documentos factura.
   * 
   * @param documentosFactura
   *          documentos factura
   */
  public void setDocumentosFactura(List<DocumentoFactura> documentosFactura) {
    this.documentosFactura = documentosFactura;
  }

  /**
   * Devuelve responsable aprobacion presupuesto.
   * 
   * @return responsable aprobacion presupuesto
   */
  @Transient
  public Empleado getResponsableAprobacionPresupuesto(){
    Empleado responsableAprobacion = null;
    Proyecto proyectoFactura = this.getContenedor().getProyectoRaiz().getProyecto();
    
    if(this.getCuantiaPresupuesto() <= Factura.MAXPRESUPUESTOPEQUENO)
      responsableAprobacion = proyectoFactura.getAprobadorCuantiaPequena();
    else{
      if(this.getCuantiaPresupuesto() <= Factura.MAXPRESUPUESTOMEDIANO)
        responsableAprobacion = proyectoFactura.getAprobadorCuantiaMediana();
      else{
        if(this.getCuantiaPresupuesto() <= Factura.MAXPRESUPUESTOGRANDE)
          responsableAprobacion = proyectoFactura.getAprobadorCuantiaGrande();
      }
    }
    
    return responsableAprobacion;
  }
  
  /**
   * Checks if is pago unico.
   * 
   * @return true, if is pago unico
   */
  @Transient
  public boolean isPagoUnico(){
    return this.getTipoPago() == TipoPago.UNICO;
  }
  
  /**
   * Checks if is pago mensual.
   * 
   * @return true, if is pago mensual
   */
  @Transient
  public boolean isPagoMensual(){
    return this.getTipoPago() == TipoPago.MENSUAL;
  }
  
  /**
   * Checks if is pago plazos.
   * 
   * @return true, if is pago plazos
   */
  @Transient
  public boolean isPagoPlazos(){
    return this.getTipoPago() == TipoPago.PLAZOS_FIJOS;
  }
  
  /**
   * Devuelve primer pago factura.
   * 
   * @return primer pago factura
   */
  @Transient
  public Pago getPrimerPagoFactura(){
    Pago primerPagoFactura = null;
    
    if(this.getPagos().size() > 0)
      primerPagoFactura = this.getPagos().get(0);
    
    return primerPagoFactura;
  }
  
  /**
   * Devuelve numero pagos.
   * 
   * @return numero pagos
   */
  @Transient
  public int getNumeroPagos(){
    int numeroPagos = 0;
    if(this.getPagos() != null){
      numeroPagos = this.getPagos().size();
    }
    
    return numeroPagos;
  }
  
  
  /**
   * Checks if is todos pagos completados.
   * 
   * @return true, if is todos pagos completados
   */
  @Transient
  public boolean isTodosPagosCompletados(){
    boolean result = true;
    for(Pago pago : this.getPagos()){
      if(pago.getEstadoActual().getTipoEstadoPago() != TipoEstadoPago.REVISADO){
        result = false;
        break;
      }
    }
    
    return result;
  }
  
  /**
   * Devuelve valor cuantia subpagos centro.
   * 
   * @param centro
   *           centro
   * @param fechaHoraInicio
   *           fecha hora inicio
   * @param fechaHoraFin
   *           fecha hora fin
   * @param tipoEstadoPagoMinimo
   *           tipo estado pago minimo
   * @return valor cuantia subpagos centro
   */
  @Transient
  public Double getValorCuantiaSubpagosCentro(Centro centro, Date fechaHoraInicio, Date fechaHoraFin, TipoEstadoPago tipoEstadoPagoMinimo){
    Double valorCuantiaPagosCentros = new Double(0);
    
    for(Pago pago: this.getPagos()){
      if((fechaHoraInicio == null || pago.getFechaEfectiva().compareTo(fechaHoraInicio) >= 0) && (fechaHoraFin == null || pago.getFechaEfectiva().compareTo(fechaHoraFin) < 0) && (tipoEstadoPagoMinimo == null || pago.getEstadoActual().getTipoEstadoPago().compareTo(tipoEstadoPagoMinimo) >= 0)){
        for(Pago pagoSubproyecto : pago.getPagoSubproyectos()){
          if(pagoSubproyecto.getCentro() == centro){
            if(pago.getTipoCuantiaSubproyectos() == TipoCuantiaSubproyectos.VALOR_FIJO){
              valorCuantiaPagosCentros += pagoSubproyecto.getCuantia();
            }else{//if(pago.getTipoCuantiaSubproyectos() == TipoCuantiaSubproyectos.PORCENTUAL)
              valorCuantiaPagosCentros += pagoSubproyecto.getCuantiaFijaSubproyecto();
            }
          }
        }
      }
    }
    
    return valorCuantiaPagosCentros;
  }
  
  /**
   * Devuelve valor cuantia pagos.
   * 
   * @param fechaHoraInicio
   *           fecha hora inicio
   * @param fechaHoraFin
   *           fecha hora fin
   * @param tipoEstadoPagoMinimo
   *           tipo estado pago minimo
   * @return valor cuantia pagos
   */
  @Transient
  public Double getValorCuantiaPagos(Date fechaHoraInicio, Date fechaHoraFin, TipoEstadoPago tipoEstadoPagoMinimo){
    Double valorCuantiaPagosCentros = new Double(0);
    
    for(Pago pago: this.getPagos()){
      if((fechaHoraInicio == null || pago.getFechaEfectiva().compareTo(fechaHoraInicio) >= 0) && (fechaHoraFin == null || pago.getFechaEfectiva().compareTo(fechaHoraFin) < 0) && (tipoEstadoPagoMinimo == null || pago.getEstadoActual().getTipoEstadoPago().compareTo(tipoEstadoPagoMinimo) >= 0)){
        valorCuantiaPagosCentros += pago.getCuantia();
      }
    }
    
    return valorCuantiaPagosCentros;
  }
  
  /**
   * Devuelve justificada.
   * 
   * @return justificada
   */
  public boolean getJustificada(){
    boolean result = false;
    for(DocumentoFactura documento : this.getDocumentosFactura()){
      if(documento.getEstadoDocumentoFactura().equals(EstadoDocumentoFactura.APROBADA_JUNTA) || documento.getEstadoDocumentoFactura().equals(EstadoDocumentoFactura.PENDIENTE_APROBACION_JUNTA)){
        result = true;
        break;
      }
    }
    
    this.justificadaConsulta = result;
    return result;
  }
  
  /**
   * Asigna justificada.
   * 
   * @param justificada
   *          justificada
   */
  public void setJustificada(boolean justificada){
    this.justificadaConsulta = justificada;
  }
  
  //Propiedad implementada para evitar el LazyLoadingExcepcion que se producía al acceder a los documentos de factura
  //cuando se consulta la propiedad justificada de factura desde Contenedor.getValorPagosFacturasJustificadas
  /**
   * Devuelve justificada consulta.
   * 
   * @return justificada consulta
   */
  @Transient
  public boolean getJustificadaConsulta(){
    return this.justificadaConsulta;
  }
  
  /**
   * Devuelve aprobada junta.
   * 
   * @return aprobada junta
   */
  public boolean getAprobadaJunta(){
    boolean result = false;
    for(DocumentoFactura documento : this.getDocumentosFactura()){
      if(documento.getEstadoDocumentoFactura().equals(EstadoDocumentoFactura.APROBADA_JUNTA)){
        result = true;
        break;
      }
    }
    
    this.aprobadaJuntaConsulta = result;
    return result;
  }
  
  /**
   * Asigna aprobada junta.
   * 
   * @param aprobadaJunta
   *          aprobada junta
   */
  public void setAprobadaJunta(boolean aprobadaJunta){
    this.aprobadaJuntaConsulta = aprobadaJunta;
  }
  
  //Propiedad implementada para evitar el LazyLoadingExcepcion que se producía al acceder a los documentos de factura
  //cuando se consulta la propiedad aprobada junta de factura desde Contenedor.getValorPagosFacturasAprobadasJunta
  /**
   * Devuelve aprobada junta consulta.
   * 
   * @return aprobada junta consulta
   */
  @Transient
  public boolean getAprobadaJuntaConsulta(){
    return this.aprobadaJuntaConsulta;
  }
  
  /* (non-Javadoc)
   * @see java.lang.Comparable#compareTo(java.lang.Object)
   */
  @Override
  public int compareTo(Factura o) {
    Long idFactura = new Long(this.getId());
    
    return idFactura.compareTo(o.getId());
  }

}
