package es.csa.fdt.cyldigital.financiero.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;

import es.csa.fdt.cyldigital.usuarios.entity.Empleado;

/**
 * Clase ItemContenedor.
 * 
 * @author csa
 */
@Entity
@Table(name = "itemscontenedor")
@AttributeOverride(name = "id", column = @Column(name = "idItem"))
@PrimaryKeyJoinColumn(name = "idContenedor")
@Audited
public class ItemContenedor extends Contenedor implements Serializable {

  private static final long serialVersionUID = 6497519888332638645L;

  /**
   * Construye un nuevo objeto de la clase
   */
  public ItemContenedor(){
    super(TipoContenedor.ITEM);
  }
  
  /**
   * Construye un nuevo objeto de la clase
   * 
   * @param nombre
   *           nombre
   */
  public ItemContenedor(String nombre) {
    super(TipoContenedor.ITEM, nombre);
  }
  
  /**
   * Construye un nuevo objeto de la clase
   * 
   * @param nombre
   *           nombre
   * @param proyectoRaiz
   *           proyecto raiz
   */
  public ItemContenedor(String nombre, ProyectoAnual proyectoRaiz)
  {
    super(TipoContenedor.ITEM, nombre, proyectoRaiz);
  }
  
  /**
   * Construye un nuevo objeto de la clase
   * 
   * @param nombre
   *           nombre
   * @param proyectoRaiz
   *           proyecto raiz
   * @param contenedorPadre
   *           contenedor padre
   */
  public ItemContenedor(String nombre, ProyectoAnual proyectoRaiz, Contenedor contenedorPadre)
  {
    this(nombre, proyectoRaiz);
    this.setContenedorPadre(contenedorPadre);
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.financiero.entity.Contenedor#getResponsable()
   */
  @Transient
  public Empleado getResponsable(){
    return this.getSeccionRaiz().getResponsable();
  }

  /**
   * Clone item contenedor.
   * 
   * @param proyectoRaiz
   *           proyecto raiz
   * @param seccionRaiz
   *           seccion raiz
   * @param contenedorPadre
   *           contenedor padre
   * @param contenedorProyectoGeneral
   *           contenedor proyecto general
   * @return item contenedor
   */
  @Transient
  public ItemContenedor cloneItemContenedor(ProyectoAnual proyectoRaiz, Seccion seccionRaiz, Contenedor contenedorPadre, Contenedor contenedorProyectoGeneral)
  {
    //Creamos un item con los mismos datos configurables (nombre y responsable).
    ItemContenedor item = new ItemContenedor(this.getNombre(), this.getProyectoRaiz());
    
    //Y le asignamos los parámetros variables como elemento independiente del actual y que nos vienen dados por parámetro.
    item.setProyectoRaiz(proyectoRaiz);
    item.setSeccionRaiz(seccionRaiz);
    item.setContenedorPadre(contenedorPadre);
    item.setContenedorProyectoGeneral(contenedorProyectoGeneral);
    
    return item;
  }

}
