package es.csa.fdt.cyldigital.financiero.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.Date;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.OneToOne;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Digits;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.usuarios.entity.Proveedor;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;

/**
 * Clase Movimiento.
 * 
 * @author csa
 */
@Entity
@Table(name = "movimientos")
@AttributeOverride(name = "id", column = @Column(name = "idMovimiento"))
@Audited
public class Movimiento extends AbstractEntity implements Serializable {
  private static final long serialVersionUID = -7648562176655166537L;

  /**
   * Enum TipoOperacion.
   */
  public enum TipoOperacion {
    ANADIRFACTURA,
    ABONOCAJA;
  }
  
  /**
   * Enum EstadoRecibi.
   */
  public enum EstadoRecibi {
    CONRECIBI,
    SINRECIBI;
  }

  private String concepto;
  private String observaciones;
  private float importe;
  private float importeAnterior;
  private float importePosterior;
  private Date fechaOperacion;
  private Date fechaFactura;
  private TipoOperacion tipo;
  private Proveedor proveedor;
  private Caja caja;
  private Usuario usuario;  
  private DocumentoMovimiento documentoMovimiento;

  /**
   * Devuelve fecha factura.
   * 
   * @return fecha factura
   */
  public Date getFechaFactura() {
    return fechaFactura;
  }

  /**
   * Asigna fecha factura.
   * 
   * @param fechaFactura
   *          fecha factura
   */
  public void setFechaFactura(Date fechaFactura) {
    this.fechaFactura = fechaFactura;
  }

  /**
   * Devuelve fecha operacion.
   * 
   * @return fecha operacion
   */
  @NotNull
  public Date getFechaOperacion() {
    return fechaOperacion;
  }

  /**
   * Asigna fecha operacion.
   * 
   * @param fechaOperacion
   *          fecha operacion
   */
  public void setFechaOperacion(Date fechaOperacion) {
    this.fechaOperacion = fechaOperacion;
  }

  /**
   * Devuelve concepto.
   * 
   * @return concepto
   */
  @Length(max = 255)
  @NotEmpty
  @NotNull
  public String getConcepto() {
    return concepto;
  }

  /**
   * Asigna concepto.
   * 
   * @param concepto
   *          concepto
   */
  public void setConcepto(String concepto) {
    this.concepto = concepto;
  }

  /**
   * Devuelve importe.
   * 
   * @return importe
   */
  @Digits(integerDigits = 20, fractionalDigits = 2)
  @NotNull
  public float getImporte() {
    return importe;
  }

  /**
   * Asigna importe.
   * 
   * @param importe
   *          importe
   */
  public void setImporte(float importe) {
    this.importe = importe;
  }

  /**
   * Devuelve tipo.
   * 
   * @return tipo
   */
  @Enumerated(EnumType.STRING)
  @NotNull
  public TipoOperacion getTipo() {
    return tipo;
  }

  /**
   * Asigna tipo.
   * 
   * @param tipoOperacion
   *          tipo
   */
  public void setTipo(TipoOperacion tipoOperacion) {
    this.tipo = tipoOperacion;
  }

  /**
   * Devuelve caja.
   * 
   * @return caja
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idCaja", nullable = false)
  @NotNull
  public Caja getCaja() {
    return caja;
  }

  /**
   * Asigna caja.
   * 
   * @param caja
   *          caja
   */
  public void setCaja(Caja caja) {
    this.caja = caja;
  }

  /**
   * Devuelve proveedor.
   * 
   * @return proveedor
   */
  @ManyToOne(fetch = FetchType.LAZY)
  @JoinColumn(name = "idProveedor")
  public Proveedor getProveedor() {
    return proveedor;
  }

  /**
   * Asigna proveedor.
   * 
   * @param proveedor
   *          proveedor
   */
  public void setProveedor(Proveedor proveedor) {
    this.proveedor = proveedor;
  }

  /**
   * Devuelve usuario.
   * 
   * @return usuario
   */
  @ManyToOne(fetch = FetchType.LAZY)
  @JoinColumn(name = "idUsuario", nullable = false)
  @NotNull
  public Usuario getUsuario() {
    return usuario;
  }
  
  /**
   * Asigna usuario.
   * 
   * @param usuario
   *          usuario
   */
  public void setUsuario(Usuario usuario) {
    this.usuario = usuario;
  }
  
  /**
   * Devuelve documento movimiento.
   * 
   * @return documento movimiento
   */
  @OneToOne(mappedBy = "movimiento", cascade = {CascadeType.ALL})
  public DocumentoMovimiento getDocumentoMovimiento() {
    return documentoMovimiento;
  }

  /**
   * Asigna documento movimiento.
   * 
   * @param documentoMovimiento
   *          documento movimiento
   */
  public void setDocumentoMovimiento(DocumentoMovimiento documentoMovimiento) {
    this.documentoMovimiento = documentoMovimiento;
  }

  /**
   * Devuelve observaciones.
   * 
   * @return observaciones
   */
  @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getObservaciones() {
    return observaciones;
  }
  
  /**
   * Asigna observaciones.
   * 
   * @param observaciones
   *          observaciones
   */
  public void setObservaciones(String observaciones) {
    this.observaciones = observaciones;
  }

  /**
   * Devuelve importe anterior.
   * 
   * @return importe anterior
   */
  public float getImporteAnterior() {
    return importeAnterior;
  }
  
  /**
   * Asigna importe anterior.
   * 
   * @param importeAnterior
   *          importe anterior
   */
  public void setImporteAnterior(float importeAnterior) {
    this.importeAnterior = importeAnterior;
  }

  /**
   * Devuelve importe posterior.
   * 
   * @return importe posterior
   */
  public float getImportePosterior() {
    return importePosterior;
  }
  
  /**
   * Asigna importe posterior.
   * 
   * @param importePosterior
   *          importe posterior
   */
  public void setImportePosterior(float importePosterior) {
    this.importePosterior = importePosterior;
  }
  
  /**
   * Devuelve estado recibi.
   * 
   * @return estado recibi
   */
  @Transient
  public EstadoRecibi getEstadoRecibi(){
    EstadoRecibi estadoRecibi = EstadoRecibi.SINRECIBI;
    
    if(this.getTipo() == TipoOperacion.ABONOCAJA){
      if(this.getDocumentoMovimiento() != null){
        estadoRecibi = EstadoRecibi.CONRECIBI;
      }
    }
    
    return estadoRecibi;
  }
}
