package es.csa.fdt.cyldigital.financiero.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.OrderBy;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.entity.AbstractEntity;

/**
 * Clase Pago.
 * 
 * @author csa
 */
@Entity
@Table(name = "pagos")
@AttributeOverride(name = "id", column = @Column(name = "idPago"))
@Audited
public class Pago extends AbstractEntity implements Serializable{
  
  private static final long serialVersionUID = 2309475094237590423L;
  
  /**
   * Enum TipoCuantiaSubproyectos.
   */
  public enum TipoCuantiaSubproyectos{
    VALOR_FIJO,
    PORCENTUAL
  }

  private Factura factura;
  
  private Pago pagoProyectoGeneral;
  
  private List<Pago> pagoSubproyectos;
  
  private Centro centro;
  
  private TipoCuantiaSubproyectos tipoCuantiaSubproyectos;
  
  private Double cuantia;
  
  private Double cuantiaPorcentual;
  
  private Date fechaEfectiva;
  
  private List<EstadoPago> estadosPago;
  
  private EstadoPago estadoActual;
  
  private DocumentoFactura documentoFactura;
  
  /**
   * Construye un nuevo objeto de la clase
   */
  public Pago(){
    this.setPagoSubproyectos(new ArrayList<Pago>());
    this.setEstadosPago(new ArrayList<EstadoPago>());
  }
  
  /**
   * Construye un nuevo objeto de la clase
   * 
   * @param pagoProyectoGeneral
   *           pago proyecto general
   * @param centro
   *           centro
   */
  public Pago(Pago pagoProyectoGeneral, Centro centro){
    this();
    this.setPagoProyectoGeneral(pagoProyectoGeneral);
    this.setCentro(centro);
  }

  /**
   * Devuelve factura.
   * 
   * @return factura
   */
  @ManyToOne(fetch = FetchType.LAZY)
  @JoinColumn(name = "idFactura")
  public Factura getFactura() {
    return factura;
  }

  /**
   * Asigna factura.
   * 
   * @param factura
   *          factura
   */
  public void setFactura(Factura factura) {
    this.factura = factura;
  }

  /**
   * Devuelve pago proyecto general.
   * 
   * @return pago proyecto general
   */
  @ManyToOne
  @JoinColumn(name = "idPagoProyectoGeneral")
  public Pago getPagoProyectoGeneral() {
    return pagoProyectoGeneral;
  }

  /**
   * Asigna pago proyecto general.
   * 
   * @param pagoProyectoGeneral
   *          pago proyecto general
   */
  public void setPagoProyectoGeneral(Pago pagoProyectoGeneral) {
    this.pagoProyectoGeneral = pagoProyectoGeneral;
  }

  /**
   * Devuelve pago subproyectos.
   * 
   * @return pago subproyectos
   */
  @OneToMany(mappedBy = "pagoProyectoGeneral", cascade = CascadeType.ALL,fetch=FetchType.EAGER)
  public List<Pago> getPagoSubproyectos() {
    return pagoSubproyectos;
  }

  /**
   * Asigna pago subproyectos.
   * 
   * @param pagoSubproyectos
   *          pago subproyectos
   */
  public void setPagoSubproyectos(List<Pago> pagoSubproyectos) {
    this.pagoSubproyectos = pagoSubproyectos;
  }

  /**
   * Devuelve centro.
   * 
   * @return centro
   */
  @ManyToOne
  @JoinColumn(name = "idCentro")
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Devuelve tipo cuantia subproyectos.
   * 
   * @return tipo cuantia subproyectos
   */
  @Enumerated(value = EnumType.STRING)
  public TipoCuantiaSubproyectos getTipoCuantiaSubproyectos() {
    return tipoCuantiaSubproyectos;
  }

  /**
   * Asigna tipo cuantia subproyectos.
   * 
   * @param tipoCuantiaSubproyectos
   *          tipo cuantia subproyectos
   */
  public void setTipoCuantiaSubproyectos(TipoCuantiaSubproyectos tipoCuantiaSubproyectos) {
    this.tipoCuantiaSubproyectos = tipoCuantiaSubproyectos;
  }

  /**
   * Devuelve cuantia.
   * 
   * @return cuantia
   */
  public Double getCuantia() {
    return cuantia;
  }

  /**
   * Asigna cuantia.
   * 
   * @param cuantia
   *          cuantia
   */
  public void setCuantia(Double cuantia) {
    this.cuantia = cuantia;
  }
  
  /**
   * Devuelve cuantia porcentual.
   * 
   * @return cuantia porcentual
   */
  public Double getCuantiaPorcentual() {
    return cuantiaPorcentual;
  }

  /**
   * Asigna cuantia porcentual.
   * 
   * @param cuantiaPorcentual
   *          cuantia porcentual
   */
  public void setCuantiaPorcentual(Double cuantiaPorcentual) {
    this.cuantiaPorcentual = cuantiaPorcentual;
  }

  /**
   * Devuelve fecha efectiva.
   * 
   * @return fecha efectiva
   */
  @NotNull
  public Date getFechaEfectiva() {
    return fechaEfectiva;
  }

  /**
   * Asigna fecha efectiva.
   * 
   * @param fechaEfectiva
   *          fecha efectiva
   */
  public void setFechaEfectiva(Date fechaEfectiva) {
    this.fechaEfectiva = fechaEfectiva;
  }
  
  /**
   * Devuelve estados pago.
   * 
   * @return estados pago
   */
  @OneToMany(mappedBy = "pago", cascade = CascadeType.ALL)
  @OrderBy(value = "fecha DESC")
  public List<EstadoPago> getEstadosPago() {
    return estadosPago;
  }

  /**
   * Asigna estados pago.
   * 
   * @param estadosPago
   *          estados pago
   */
  public void setEstadosPago(List<EstadoPago> estadosPago) {
    this.estadosPago = estadosPago;
  }

  /**
   * Devuelve estado actual.
   * 
   * @return estado actual
   */
  @OneToOne
  @JoinColumn(name = "idEstadoActual")
  public EstadoPago getEstadoActual() {
    return estadoActual;
  }

  /**
   * Asigna estado actual.
   * 
   * @param estadoPago
   *          estado actual
   */
  public void setEstadoActual(EstadoPago estadoPago) {
    this.estadoActual = estadoPago;
  }
  
  /**
   * Devuelve documento factura.
   * 
   * @return documento factura
   */
  @OneToOne(mappedBy = "pago", cascade = {CascadeType.ALL})
  public DocumentoFactura getDocumentoFactura() {
    return documentoFactura;
  }

  /**
   * Asigna documento factura.
   * 
   * @param documentoFactura
   *          documento factura
   */
  public void setDocumentoFactura(DocumentoFactura documentoFactura) {
    this.documentoFactura = documentoFactura;
  }

  /**
   * Devuelve cuantia fija subproyecto.
   * 
   * @return cuantia fija subproyecto
   */
  @Transient
  public Double getCuantiaFijaSubproyecto(){
    Double cuantiaFijaSubproyecto = null;
    
    if(this.getPagoProyectoGeneral() != null){
      if(this.getPagoProyectoGeneral().getTipoCuantiaSubproyectos() == TipoCuantiaSubproyectos.VALOR_FIJO){
        cuantiaFijaSubproyecto = this.getCuantia();
      }else{//if(this.getPagoProyectoGeneral().getTipoCuantiaSubproyectos() == TipoCuantiaSubproyectos.PORCENTUAL)
        cuantiaFijaSubproyecto = this.getCuantiaPorcentual() * this.getPagoProyectoGeneral().getCuantia() / new Double(100);
      } 
    }
    
    return cuantiaFijaSubproyecto;
  }

  /**
   * Checks if is tipo cuantia porcentual.
   * 
   * @return true, if is tipo cuantia porcentual
   */
  @Transient
  public boolean isTipoCuantiaPorcentual(){
   return this.getTipoCuantiaSubproyectos() == TipoCuantiaSubproyectos.PORCENTUAL; 
  }
  
  /**
   * Clone pago.
   * 
   * @param factura
   *           factura
   * @param pagoProyectoGeneral
   *           pago proyecto general
   * @return pago
   */
  @Transient
  public Pago clonePago(Factura factura, Pago pagoProyectoGeneral){
    Pago clonedPago = new Pago();
    clonedPago.setFactura(factura);
    clonedPago.setPagoProyectoGeneral(pagoProyectoGeneral);
    clonedPago.setCentro(this.getCentro());
    clonedPago.setTipoCuantiaSubproyectos(this.getTipoCuantiaSubproyectos());
    clonedPago.setCuantia(this.getCuantia());
    clonedPago.setCuantiaPorcentual(this.getCuantiaPorcentual());
    clonedPago.setFechaEfectiva(this.getFechaEfectiva());
    
    return clonedPago;
  }
  
  /**
   * Checks if is centro pago abierto.
   * 
   * @return true, if is centro pago abierto
   */
  @Transient
  public boolean isCentroPagoAbierto(){
    Centro centro = this.getCentro();
    Date fechaEfectivaPago = this.getFechaEfectiva();
    
    //Si el centro no tiene la fecha de apertura especificada, consideraremos que ha estado siempre abierto.
    if(centro.getFechaApertura() == null){
     return true;
    }else{
      //Si el centro ha abierto antes o a la vez que la fecha del pago. 
      if(centro.getFechaApertura().before(fechaEfectivaPago) || centro.getFechaApertura().equals(fechaEfectivaPago)){
        return true;
      }else{
        return false;
      }
    }
  }
  
}
