package es.csa.fdt.cyldigital.financiero.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;

/**
 * Clase Prevision.
 * 
 * @author csa
 */
@Entity
@Table(name = "previsiones")
@AttributeOverride(name = "id", column = @Column(name = "idPrevision"))
@Audited
public class Prevision extends AbstractEntity implements Serializable{

  private static final long serialVersionUID = 3409723094712943078L;
  
  private boolean asignado;
  
  private Double cuantia;

  private Double cuantiaInicial; //Transient
  
  private Double cuantiaPorcentual;
  
  private HashMap<Mes, Double> valoresFijos;
  
  //Selección Checkboxes
  //private HashMap<Mes, Boolean> seleccionValorFijo;
  
  private boolean asignadoEvolutiva;
  
  private Double cuantiaEvolutiva;
  
  private Double cuantiaEvolutivaInicial;//Transient
  
  private Double cuantiaPorcentualEvolutiva;
  
  private HashMap<Mes, Double> valoresFijosEvolutivos;
  
  //Selección Checkboxes
  //private HashMap<Mes, Boolean> seleccionValorFijoEvolutivo;
  
  //Selección Checkboxes
  //private boolean seleccionadoValores;//Transient
  
  //Selección Checkboxes
  //private HashMap<Mes, Boolean> seleccionMeses;//Transient
  
  private Prevision previsionPadre;
  
  private List<Prevision> previsionesDescendientes;
  
  private Contenedor contenedor;
  
  /**
   * Enum Mes.
   */
  public static enum Mes{
    Enero,
    Febrero,
    Marzo,
    Abril,
    Mayo,
    Junio,
    Julio,
    Agosto,
    Septiembre,
    Octubre,
    Noviembre,
    Diciembre
  }
  
  /**
   * Construye un nuevo objeto de la clase
   */
  public Prevision(){
    this.cuantia = new Double(0);
    this.cuantiaPorcentual = new Double(0);
    
    this.cuantiaEvolutiva = new Double(0);
    this.cuantiaPorcentualEvolutiva = new Double(0);
    
    this.setValoresFijos(this.inicializarValores());
    this.setValoresFijosEvolutivos(this.inicializarValores());
    
    //Selección Checkboxes
    /*this.setSeleccionValorFijo(this.inicializarValoresSeleccion());
    this.setSeleccionValorFijoEvolutivo(this.inicializarValoresSeleccion());
    
    this.setSeleccionMeses(this.inicializarValoresSeleccion());*/
  }
  
  private HashMap<Mes, Double> inicializarValores(){
    HashMap<Mes, Double> valores =  new HashMap<Mes, Double>();
    
    for(Mes mes: Mes.values()){
      valores.put(mes, new Double(0));
    }
    
    return valores;
  }
  
  //Selección Checkboxes
  /*private HashMap<Mes, Boolean> inicializarValoresSeleccion(){
    HashMap<Mes, Boolean> valores =  new HashMap<Mes, Boolean>();
    
    for(Mes mes: Mes.values()){
      valores.put(mes, new Boolean(true));
    }
    
    return valores;
  }*/
  
  /**
   * Checks if is asignado.
   * 
   * @return true, if is asignado
   */
  public boolean isAsignado() {
    return asignado;
  }

  /**
   * Asigna asignado.
   * 
   * @param asignado
   *          asignado
   */
  public void setAsignado(boolean asignado) {
    this.asignado = asignado;
  }
  
  /**
   * Devuelve cuantia.
   * 
   * @return cuantia
   */
  @NotNull
  public Double getCuantia() {
    return cuantia;
  }

  /**
   * Asigna cuantia.
   * 
   * @param cuantia
   *          cuantia
   */
  public void setCuantia(Double cuantia) {
    this.cuantia = cuantia;
  }
  
  // Utilizado para poder calcular porcentajes según se modifican las cuantias mensuales.
  /**
   * Devuelve cuantia inicial.
   * 
   * @return cuantia inicial
   */
  @Transient
  public Double getCuantiaInicial() {
    return cuantiaInicial;
  }

  /**
   * Asigna cuantia inicial.
   * 
   * @param cuantiaInicial
   *          cuantia inicial
   */
  public void setCuantiaInicial(Double cuantiaInicial) {
    this.cuantiaInicial = cuantiaInicial;
  }
  
  /**
   * Devuelve diferencia cuantia inicial.
   * 
   * @return diferencia cuantia inicial
   */
  @Transient
  public Double getDiferenciaCuantiaInicial(){
    Double diferencia;
    diferencia = this.getCuantia() - this.getCuantiaInicial();
    
    return diferencia;
  }

  /**
   * Devuelve cuantia porcentual.
   * 
   * @return cuantia porcentual
   */
  @NotNull
  public Double getCuantiaPorcentual() {
    return cuantiaPorcentual;
  }

  /**
   * Asigna cuantia porcentual.
   * 
   * @param cuantiaPorcentual
   *          cuantia porcentual
   */
  public void setCuantiaPorcentual(Double cuantiaPorcentual) {
    this.cuantiaPorcentual = cuantiaPorcentual;
  }

  /**
   * Devuelve valores fijos.
   * 
   * @return valores fijos
   */
  @Lob
  @NotAudited
  public HashMap<Mes, Double> getValoresFijos() {
    if(this.valoresFijos == null){
      this.setValoresFijos(this.inicializarValores());
    }
    return valoresFijos;
  }

  /**
   * Asigna valores fijos.
   * 
   * @param valoresFijos
   *           valores fijos
   */
  public void setValoresFijos(HashMap<Mes, Double> valoresFijos) {
    this.valoresFijos = valoresFijos;
  }
  
  // ------------------------------------------------------------------------------------------------Previsión Evolutiva
  
  /**
   * Checks if is asignado evolutiva.
   * 
   * @return true, if is asignado evolutiva
   */
  public boolean isAsignadoEvolutiva() {
    return asignadoEvolutiva;
  }

  /**
   * Asigna asignado evolutiva.
   * 
   * @param asignadoEvolutiva
   *          asignado evolutiva
   */
  public void setAsignadoEvolutiva(boolean asignadoEvolutiva) {
    this.asignadoEvolutiva = asignadoEvolutiva;
  }
  
  /**
   * Devuelve cuantia evolutiva.
   * 
   * @return cuantia evolutiva
   */
  @NotNull
  public Double getCuantiaEvolutiva() {
    return cuantiaEvolutiva;
  }

  /**
   * Asigna cuantia evolutiva.
   * 
   * @param cuantiaEvolutiva
   *          cuantia evolutiva
   */
  public void setCuantiaEvolutiva(Double cuantiaEvolutiva) {
    this.cuantiaEvolutiva = cuantiaEvolutiva;
  }
  
  // Utilizado para poder calcular porcentajes según se modifican las cuantias mensuales.
  /**
   * Devuelve cuantia evolutiva inicial.
   * 
   * @return cuantia evolutiva inicial
   */
  @Transient
  public Double getCuantiaEvolutivaInicial() {
    return cuantiaEvolutivaInicial;
  }

  /**
   * Asigna cuantia evolutiva inicial.
   * 
   * @param cuantiaEvolutivaInicial
   *          cuantia evolutiva inicial
   */
  public void setCuantiaEvolutivaInicial(Double cuantiaEvolutivaInicial) {
    this.cuantiaEvolutivaInicial = cuantiaEvolutivaInicial;
  }
  
  /**
   * Devuelve diferencia cuantia evolutiva inicial.
   * 
   * @return diferencia cuantia evolutiva inicial
   */
  @Transient
  public Double getDiferenciaCuantiaEvolutivaInicial(){
    Double diferencia;
    diferencia = this.getCuantiaEvolutiva() - this.getCuantiaEvolutivaInicial();
    
    return diferencia;
  }

  /**
   * Devuelve cuantia porcentual evolutiva.
   * 
   * @return cuantia porcentual evolutiva
   */
  @NotNull
  public Double getCuantiaPorcentualEvolutiva() {
    return cuantiaPorcentualEvolutiva;
  }

  /**
   * Asigna cuantia porcentual evolutiva.
   * 
   * @param cuantiaPorcentualEvolutiva
   *          cuantia porcentual evolutiva
   */
  public void setCuantiaPorcentualEvolutiva(Double cuantiaPorcentualEvolutiva) {
    this.cuantiaPorcentualEvolutiva = cuantiaPorcentualEvolutiva;
  }

  /**
   * Devuelve valores fijos evolutivos.
   * 
   * @return valores fijos evolutivos
   */
  @Lob
  @NotAudited
  public HashMap<Mes, Double> getValoresFijosEvolutivos() {
    if(this.valoresFijosEvolutivos == null){
      this.setValoresFijosEvolutivos(this.inicializarValores());
    }
    return valoresFijosEvolutivos;
  }

  /**
   * Asigna valores fijos evolutivos.
   * 
   * @param valoresFijosEvolutivos
   *           valores fijos evolutivos
   */
  public void setValoresFijosEvolutivos(HashMap<Mes, Double> valoresFijosEvolutivos) {
    this.valoresFijosEvolutivos = valoresFijosEvolutivos;
  }
  
  // -------------------------------------------------------------------------------------Relaciones con otros elementos
  
  /**
   * Devuelve prevision padre.
   * 
   * @return prevision padre
   */
  @ManyToOne
  @JoinColumn(name = "idPrevisionPadre")
  public Prevision getPrevisionPadre() {
    return previsionPadre;
  }

  /**
   * Asigna prevision padre.
   * 
   * @param previsionPadre
   *          prevision padre
   */
  public void setPrevisionPadre(Prevision previsionPadre) {
    this.previsionPadre = previsionPadre;
  }

  /**
   * Devuelve previsiones descendientes.
   * 
   * @return previsiones descendientes
   */
  @OneToMany(mappedBy = "previsionPadre", cascade = CascadeType.ALL, fetch = FetchType.EAGER)
  public List<Prevision> getPrevisionesDescendientes() {
    return previsionesDescendientes;
  }

  /**
   * Asigna previsiones descendientes.
   * 
   * @param previsionesDescendientes
   *          previsiones descendientes
   */
  public void setPrevisionesDescendientes(List<Prevision> previsionesDescendientes) {
    this.previsionesDescendientes = previsionesDescendientes;
  }

  /**
   * Devuelve contenedor.
   * 
   * @return contenedor
   */
  @OneToOne
  @JoinColumn(name = "idContenedor")
  public Contenedor getContenedor() {
    return contenedor;
  }

  /**
   * Asigna contenedor.
   * 
   * @param contenedor
   *          contenedor
   */
  public void setContenedor(Contenedor contenedor) {
    this.contenedor = contenedor;
  }
  
  //Selección Checkboxes
  /*@Lob
  @NotAudited
  public HashMap<Mes, Boolean> getSeleccionValorFijo() {
    if(this.seleccionValorFijo == null){
      this.setSeleccionValorFijo(this.inicializarValoresSeleccion());
    }
    return seleccionValorFijo;
  }

  public void setSeleccionValorFijo(HashMap<Mes, Boolean> seleccionValorFijo) {
    this.seleccionValorFijo = seleccionValorFijo;
  }*/

  //Selección Checkboxes
  /*@Lob
  @NotAudited
  public HashMap<Mes, Boolean> getSeleccionValorFijoEvolutivo() {
    if(this.seleccionValorFijoEvolutivo == null){
      this.setSeleccionValorFijoEvolutivo(this.inicializarValoresSeleccion());
    }
    return seleccionValorFijoEvolutivo;
  }

  public void setSeleccionValorFijoEvolutivo(HashMap<Mes, Boolean> seleccionValorFijoEvolutivo) {
    this.seleccionValorFijoEvolutivo = seleccionValorFijoEvolutivo;
  }*/
  
  // -------------------------------------------------------------------------------------------------Métodos de cálculo  
  
  //Selección Checkboxes
  /*@Transient
  public boolean isSeleccionadoValores() {
    return seleccionadoValores;
  }

  public void setSeleccionadoValores(boolean seleccionadosValores) {
    this.seleccionadoValores = seleccionadosValores;
  }*/

  //Selección Checkboxes
  /*@Transient
  public HashMap<Mes, Boolean> getSeleccionMeses() {
    return seleccionMeses;
  }

  public void setSeleccionMeses(HashMap<Mes, Boolean> seleccionMeses) {
    this.seleccionMeses = seleccionMeses;
  }*/

  /**
   * Devuelve tiene descendientes asignados.
   * 
   * @return tiene descendientes asignados
   */
  @Transient
  public boolean getTieneDescendientesAsignados()
  {
    boolean result = false;
    
    List<Prevision> descendientes = this.getPrevisionesDescendientes();
    if(descendientes != null){
      for(Prevision descendiente : descendientes)
      {
        if(descendiente.isAsignado() == true || descendiente.getTieneDescendientesAsignados())
        {
          result = true;
        }
      }
    }
    
    return result;
  }
  
  /**
   * Devuelve tiene descendientes asignados evolutiva.
   * 
   * @return tiene descendientes asignados evolutiva
   */
  @Transient
  public boolean getTieneDescendientesAsignadosEvolutiva()
  {
    boolean result = false;
    
    List<Prevision> descendientes = this.getPrevisionesDescendientes();
    if(descendientes != null){
      for(Prevision descendiente : descendientes)
      {
        if(descendiente.isAsignadoEvolutiva() == true || descendiente.getTieneDescendientesAsignadosEvolutiva())
        {
          result = true;
        }
      }
    }
    
    return result;
  }
  
  /**
   * Establecer previsiones evolutivas.
   */
  @SuppressWarnings("unchecked")
  public void establecerPrevisionesEvolutivas(){
    this.setAsignadoEvolutiva(this.isAsignado());
    this.setCuantiaEvolutiva(this.getCuantia());
    this.setCuantiaPorcentualEvolutiva(this.getCuantiaPorcentual());
    this.setValoresFijosEvolutivos((HashMap<Mes, Double>)this.getValoresFijos().clone());
  }
  
  /**
   * Devuelve cuantia efectiva.
   * 
   * @return cuantia efectiva
   */
  @Transient
  public Double getCuantiaEfectiva()
  {
    Double total = new Double(0);
    
    if(this.isAsignado() == true)
      total = this.getCuantia();
    else
    {
      List<Prevision> previsionesDescendientes = this.getPrevisionesDescendientes();
      for(Prevision previsionDescendiente : previsionesDescendientes)
      {
        total += previsionDescendiente.getCuantiaEfectiva();
      }
    }
    
    return total;
  }
  
  /**
   * Devuelve cuantia evolutiva efectiva.
   * 
   * @return cuantia evolutiva efectiva
   */
  @Transient
  public Double getCuantiaEvolutivaEfectiva()
  {
    Double total = new Double(0);
    
    if(this.isAsignadoEvolutiva() == true)
      total = this.getCuantiaEvolutiva();
    else
    {
      List<Prevision> previsionesDescendientes = this.getPrevisionesDescendientes();
      for(Prevision previsionDescendiente : previsionesDescendientes)
      {
        total += previsionDescendiente.getCuantiaEvolutivaEfectiva();
      }
    }
    
    return total;
  }
  
//  @Transient
//  public boolean isCentroAbierto(Mes mes){
//    boolean result = false;
//   
//   if(this.getContenedor().getCentro() != null){
//     Centro centro = this.getContenedor().getCentro();
//     if(centro.getFechaApertura() != null){
//       Calendar calendar = Calendar.getInstance();
//       calendar.setTimeInMillis(centro.getFechaApertura().getTime());
//       int anyoAperturaCentro = calendar.get(Calendar.YEAR);
//       if(this.getContenedor().getAnyo() != null){
//         Integer anyoPrevision = this.getContenedor().getAnyo();
//         if(anyoPrevision.compareTo(anyoAperturaCentro) < 0){
//           result = false;
//         }else if(anyoPrevision.compareTo(anyoAperturaCentro) > 0){
//           result = true;
//         }else{
//           Integer mesPrevision = new Integer(calendar.get(Calendar.MONTH));
//           if(mesPrevision.compareTo(mes.ordinal()) <= 0){
//             result = true;
//           }else{
//             result = false;
//           }
//         }
//       }
//     }else{
//       result = true;//Si no tiene fecha asignada, consideramos que estaba abierto.
//     }
//   }
//   
//   return result;
// }

  /**
 * Checks if is centro abierto anyo prevision.
 * 
 * @return true, if is centro abierto anyo prevision
 */
@Transient
  public boolean isCentroAbiertoAnyoPrevision(){
    boolean result = false;
   
   if(this.getContenedor().getCentro() != null){
     if(this.getContenedor().getCentro().getFechaApertura() != null){
       Calendar calendar = Calendar.getInstance();
       calendar.setTimeInMillis(this.getContenedor().getCentro().getFechaApertura().getTime());
       int anyoAperturaCentro = calendar.get(Calendar.YEAR);
       if(this.getContenedor().getAnyo() != null){
         Integer anyoPrevision = this.getContenedor().getAnyo();
         if(anyoPrevision.compareTo(anyoAperturaCentro) < 0){
           result = false;
         }else{ // if(anyoPrevision.compareTo(anyoAperturaCentro) >= 0)
           result = true;
         }
       }
     }else{
       result = true;//Si no tiene fecha asignada, consideramos que estaba abierto.
     }
   }
   
   return result;
 }
  
//  @Transient
//  public Double getNumeroMesesAbiertoCentro()
//  {
//    Double result = null;
//    
//    if(this.getContenedor().getCentro() != null){
//      Centro centro = this.getContenedor().getCentro();
//      if(centro.getFechaApertura() != null){
//        Calendar calendar = Calendar.getInstance();
//        calendar.setTimeInMillis(centro.getFechaApertura().getTime());
//        int anyoAperturaCentro = calendar.get(Calendar.YEAR);
//        if(this.getContenedor().getAnyo() != null){
//          Integer anyoPrevision = this.getContenedor().getAnyo();
//          if(anyoPrevision.compareTo(anyoAperturaCentro) < 0){
//            result = new Double(0);
//          }else if(anyoPrevision.compareTo(anyoAperturaCentro) > 0){
//            result = new Double(12);
//          }else{
//            result = new Double(12) - new Double(calendar.get(Calendar.MONTH));
//          }
//        }
//      }else{
//        result = new Double(12);//Si no tiene fecha asignada, consideramos que estaba abierto.
//      }
//    }
//    
//    return result;
//  }
  
  /**
 * Devuelve cuantia efectiva mensual.
 * 
 * @param mes
 *           mes
 * @return cuantia efectiva mensual
 */
@Transient
  public Double getCuantiaEfectivaMensual(Mes mes)
  {
    Double total = new Double(0);
    Contenedor contenedor = this.getContenedor();
    
    if(contenedor.getProyectoRaiz().getCentro() != null){
      if(this.isAsignado() == true)
        total = this.getValoresFijos().get(mes);
      else
      {
        List<Prevision> previsionesDescendientes = this.getPrevisionesDescendientes();
        for(Prevision previsionDescendiente : previsionesDescendientes)
        {
          if(previsionDescendiente.getValoresFijos().get(mes) != null){
            total += previsionDescendiente.getCuantiaEfectivaMensual(mes);
          }
        }
      }
    }else{
      List<Contenedor> contenedoresSubproyectos = contenedor.getContenedoresSubproyectos();
      for(Contenedor contenedorSubproyecto : contenedoresSubproyectos){
        Prevision previsionSubproyectoPrevision = contenedorSubproyecto.getPrevision();
        total += previsionSubproyectoPrevision.getCuantiaEfectivaMensual(mes);
      }
    }
    
    return total;
  }
  
  /**
   * Devuelve cuantia efectiva mensual.
   * 
   * @param mesString
   *           mes string
   * @return cuantia efectiva mensual
   */
  @Transient
  public Double getCuantiaEfectivaMensual(String mesString){
    Mes mes = Mes.valueOf(mesString);
    return this.getCuantiaEfectivaMensual(mes);
  }
  
  /**
   * Devuelve cuantia evolutiva efectiva mensual.
   * 
   * @param mes
   *           mes
   * @return cuantia evolutiva efectiva mensual
   */
  @Transient
  public Double getCuantiaEvolutivaEfectivaMensual(Mes mes)
  {
    Double total = new Double(0);
    
    if(contenedor.getProyectoRaiz().getCentro() != null){
      if(this.isAsignado() == true)
        total = this.getValoresFijosEvolutivos().get(mes);
      else
      {
        List<Prevision> previsionesDescendientes = this.getPrevisionesDescendientes();
        for(Prevision previsionDescendiente : previsionesDescendientes)
        {
          if(previsionDescendiente.getValoresFijosEvolutivos().get(mes) != null){
            total += previsionDescendiente.getCuantiaEvolutivaEfectivaMensual(mes);
          }
        }
      }
    }else{
      List<Contenedor> contenedoresSubproyectos = contenedor.getContenedoresSubproyectos();
      for(Contenedor contenedorSubproyecto : contenedoresSubproyectos){
        Prevision previsionSubproyectoPrevision = contenedorSubproyecto.getPrevision();
        total += previsionSubproyectoPrevision.getCuantiaEvolutivaEfectivaMensual(mes);
      }      
    }
    
    return total;
  }
  
  /**
   * Devuelve cuantia evolutiva efectiva mensual.
   * 
   * @param mesString
   *           mes string
   * @return cuantia evolutiva efectiva mensual
   */
  @Transient
  public Double getCuantiaEvolutivaEfectivaMensual(String mesString){
    Mes mes = Mes.valueOf(mesString);
    return this.getCuantiaEvolutivaEfectivaMensual(mes);
  }
  
  //Selección Checkboxes
  /*@Transient
  public void establecerSeleccionValoresSubproyectos(){
    List<Contenedor> contenedoresDescencientes = this.getContenedor().getContenedoresSubproyectos();
    
    for(Contenedor contenedoresDescenciente : contenedoresDescencientes){
      establecerSeleccionValoresSubproyecto(contenedoresDescenciente);
    }
  }
  
  @Transient
  public void establecerSeleccionValoresSubproyecto(Contenedor contenedorSubproyecto){
    boolean seleccionadoValores = true;
    
    Prevision previsionDescendiente = contenedorSubproyecto.getPrevision(); 
    HashMap<Mes, Boolean> seleccionValorFijo = previsionDescendiente.getSeleccionValorFijo();
    
    Set<Mes> meses = seleccionValorFijo.keySet();
    
    for(Mes mes : meses){
      if(seleccionValorFijo.get(mes) == false){
        seleccionadoValores = false;
        break;
      }
    }
    previsionDescendiente.setSeleccionadoValores(seleccionadoValores);
  }*/
  
  //Selección Checkboxes
  /*@Transient
  public void establecerSeleccionMeses(){
    for(Mes mes : Mes.values()){
      establecerSeleccionMes(mes);
    }    
  }
  
  @Transient
  public void establecerSeleccionMes(Mes mes){
    boolean seleccionadoMes = true;
    
    List<Contenedor> contenedoresDescencientes = this.getContenedor().getContenedoresSubproyectos();
  
    for(Contenedor contenedoresDescenciente : contenedoresDescencientes){  
      Prevision detallePrevisionDescendiente = contenedoresDescenciente.getPrevision(); 
      HashMap<Mes, Boolean> seleccionValorFijo = detallePrevisionDescendiente.getSeleccionValorFijo();
      
      if(seleccionValorFijo.get(mes) == false){
        seleccionadoMes = false;
        break;
      }
    }
    
    this.seleccionMeses.put(mes, seleccionadoMes);
  }*/
  
}
