package es.csa.fdt.cyldigital.financiero.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;

/**
 * Clase Proyecto.
 * 
 * @author csa
 */
@Entity
@Table(name = "proyectos")
@AttributeOverride( column = @Column(name="idProyecto"), name = "id" )
@Audited
public class Proyecto extends AbstractEntity implements Serializable{
  
  private static final long serialVersionUID = 2075894230759084237L;
  
  private String nombre;
  private Proyecto proyectoGeneral;
  private Centro centro;
  private List<Proyecto> subproyectos;
  private List<ProyectoAnual> proyectosAnuales;
  private Empleado aprobadorCuantiaPequena;
  private Empleado aprobadorCuantiaMediana;
  private Empleado aprobadorCuantiaGrande;

  /**
   * Devuelve nombre.
   * 
   * @return nombre
   */
  @NotNull
  @NotEmpty
  @Length(max = 255)
  public String getNombre() {
    return nombre;
  }

  /**
   * Asigna nombre.
   * 
   * @param nombre
   *          nombre
   */
  public void setNombre(String nombre) {
    this.nombre = nombre;
  }
  
  /**
   * Devuelve proyecto general.
   * 
   * @return proyecto general
   */
  @ManyToOne
  @JoinColumn(name = "idProyectoGeneral")
  public Proyecto getProyectoGeneral() {
    return proyectoGeneral;
  }

  /**
   * Asigna proyecto general.
   * 
   * @param proyectoGeneral
   *          proyecto general
   */
  public void setProyectoGeneral(Proyecto proyectoGeneral) {
    this.proyectoGeneral = proyectoGeneral;
  }
  
  /**
   * Es proyecto general.
   * 
   * @return true, if successful
   */
  @Transient
  public boolean esProyectoGeneral()
  {
    if(this.getProyectoGeneral() == null)
    {
      return true;
    }else{
      return false;
    }
  }
  
  /**
   * Devuelve centro.
   * 
   * @return centro
   */
  @ManyToOne
  @JoinColumn(name = "idCentro")
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Devuelve subproyectos.
   * 
   * @return subproyectos
   */
  @OneToMany(mappedBy = "proyectoGeneral", cascade = CascadeType.ALL, fetch = FetchType.EAGER)
  public List<Proyecto> getSubproyectos() {
    return subproyectos;
  }

  /**
   * Asigna subproyectos.
   * 
   * @param subproyectos
   *          subproyectos
   */
  public void setSubproyectos(List<Proyecto> subproyectos) {
    this.subproyectos = subproyectos;
  }
  
  /**
   * Devuelve subproyectos y general.
   * 
   * @return subproyectos y general
   */
  @Transient
  public List<Proyecto> getSubproyectosYGeneral(){
    ArrayList<Proyecto> subproyectosYGeneral = new ArrayList<Proyecto>(this.getSubproyectos());
    if(this.esProyectoGeneral()){
      subproyectosYGeneral.add(0, this);
    }
    
    return subproyectosYGeneral;
  }
  
  /**
   * Devuelve proyectos anuales.
   * 
   * @return proyectos anuales
   */
  @OneToMany(mappedBy = "proyecto", cascade = CascadeType.ALL)
  public List<ProyectoAnual> getProyectosAnuales() {
    return proyectosAnuales;
  }

  /**
   * Asigna proyectos anuales.
   * 
   * @param proyectosAnuales
   *          proyectos anuales
   */
  public void setProyectosAnuales(List<ProyectoAnual> proyectosAnuales) {
    this.proyectosAnuales = proyectosAnuales;
  }

  /**
   * Devuelve aprobador cuantia pequena.
   * 
   * @return aprobador cuantia pequena
   */
  @ManyToOne
  @JoinColumn(name="idAprobadorCuantiaPequena")
  @NotNull
  public Empleado getAprobadorCuantiaPequena() {
    return aprobadorCuantiaPequena;
  }

  /**
   * Asigna aprobador cuantia pequena.
   * 
   * @param aprobadorPresupuestosPequeños
   *          aprobador cuantia pequena
   */
  public void setAprobadorCuantiaPequena(Empleado aprobadorPresupuestosPequenyos) {
    this.aprobadorCuantiaPequena = aprobadorPresupuestosPequenyos;
  }

  /**
   * Devuelve aprobador cuantia mediana.
   * 
   * @return aprobador cuantia mediana
   */
  @ManyToOne
  @JoinColumn(name="idAprobadorCuantiaMediana")
  @NotNull
  public Empleado getAprobadorCuantiaMediana() {
    return aprobadorCuantiaMediana;
  }

  /**
   * Asigna aprobador cuantia mediana.
   * 
   * @param aprobadorPresupuestosMedianos
   *          aprobador cuantia mediana
   */
  public void setAprobadorCuantiaMediana(Empleado aprobadorPresupuestosMedianos) {
    this.aprobadorCuantiaMediana = aprobadorPresupuestosMedianos;
  }

  /**
   * Devuelve aprobador cuantia grande.
   * 
   * @return aprobador cuantia grande
   */
  @ManyToOne
  @JoinColumn(name="idAprobadorCuantiaGrande")
  @NotNull
  public Empleado getAprobadorCuantiaGrande() {
    return aprobadorCuantiaGrande;
  }

  /**
   * Asigna aprobador cuantia grande.
   * 
   * @param aprobadorPresupuestosGrandes
   *          aprobador cuantia grande
   */
  public void setAprobadorCuantiaGrande(Empleado aprobadorPresupuestosGrandes) {
    this.aprobadorCuantiaGrande = aprobadorPresupuestosGrandes;
  }
  
}
