package es.csa.fdt.cyldigital.financiero.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.PersistenceUnit;
import javax.persistence.Query;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.annotations.Cascade;
import org.hibernate.annotations.IndexColumn;
import org.hibernate.envers.Audited;
import org.hibernate.validator.Digits;
import org.hibernate.validator.NotNull;
import org.hibernate.validator.Validator;
import org.hibernate.validator.ValidatorClass;
import org.springframework.beans.factory.annotation.Configurable;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.entity.AbstractEntity;

/**
 * Clase ProyectoAnual.
 * 
 * @author csa
 */
@Entity
@Table(name = "proyectosanuales")
@AttributeOverride(column = @Column(name = "idProyectoAnual"), name = "id")
@Audited
@es.csa.fdt.cyldigital.financiero.entity.ProyectoAnual.AnyoUnico
public class ProyectoAnual extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = 508723905789423075L;

  private Proyecto proyecto;
  private ProyectoAnual proyectoGeneral;
  private Centro centro;
  private Integer anyo;
  private boolean previsionesInicialesCerradas;
  private List<ProyectoAnual> subproyectosAnuales;
  private List<Contenedor> elementos;
  
  /**
   * Construye un nuevo objeto de la clase
   */
  public ProyectoAnual()
  {
    super();
    this.setSubproyectosAnuales(new ArrayList<ProyectoAnual>());
  }
  
  /**
   * Devuelve proyecto.
   * 
   * @return proyecto
   */
  @ManyToOne
  @JoinColumn(name = "idProyecto")
  @NotNull
  public Proyecto getProyecto() {
    return proyecto;
  }

  /**
   * Asigna proyecto.
   * 
   * @param proyecto
   *          proyecto
   */
  public void setProyecto(Proyecto proyecto) {
    this.proyecto = proyecto;
  }
  
  /**
   * Devuelve proyecto general.
   * 
   * @return proyecto general
   */
  @ManyToOne
  @JoinColumn(name = "idProyectoGeneral")
  public ProyectoAnual getProyectoGeneral() {
    return proyectoGeneral;
  }

  /**
   * Asigna proyecto general.
   * 
   * @param proyectoPadre
   *          proyecto general
   */
  public void setProyectoGeneral(ProyectoAnual proyectoPadre) {
    this.proyectoGeneral = proyectoPadre;
  }
  
  /**
   * Es proyecto general.
   * 
   * @return true, if successful
   */
  @Transient
  public boolean esProyectoGeneral()
  {
    if(this.getProyectoGeneral() == null)
    {
      return true;
    }else{
      return false;
    }
  }

  /**
   * Devuelve centro.
   * 
   * @return centro
   */
  @OneToOne
  @JoinColumn(name = "idCentro")
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }
  
  /**
   * Devuelve anyo.
   * 
   * @return anyo
   */
  @NotNull
  @Digits(integerDigits = 4, fractionalDigits = 0)
  public Integer getAnyo() {
    return anyo;
  }

  /**
   * Asigna anyo.
   * 
   * @param anyo
   *          anyo
   */
  public void setAnyo(Integer anyo) {
    this.anyo = anyo;
  }

  /**
   * Checks if is previsiones iniciales cerradas.
   * 
   * @return true, if is previsiones iniciales cerradas
   */
  public boolean isPrevisionesInicialesCerradas() {
    return previsionesInicialesCerradas;
  }

  /**
   * Asigna previsiones iniciales cerradas.
   * 
   * @param previsionesInicialesCerradas
   *          previsiones iniciales cerradas
   */
  public void setPrevisionesInicialesCerradas(boolean previsionesInicialesCerradas) {
    this.previsionesInicialesCerradas = previsionesInicialesCerradas;
  }

  //EAGER, para evitar excepciones al generar informes
  /**
   * Devuelve subproyectos anuales.
   * 
   * @return subproyectos anuales
   */
  @OneToMany(mappedBy = "proyectoGeneral", cascade = CascadeType.ALL, fetch = FetchType.EAGER)
  public List<ProyectoAnual> getSubproyectosAnuales() {
    return subproyectosAnuales;
  }

  /**
   * Asigna subproyectos anuales.
   * 
   * @param subproyectos
   *          subproyectos anuales
   */
  public void setSubproyectosAnuales(List<ProyectoAnual> subproyectos) {
    this.subproyectosAnuales = subproyectos;
  }

  /**
   * Devuelve elementos.
   * 
   * @return elementos
   */
  @OneToMany(mappedBy = "proyecto", cascade = CascadeType.ALL)
  @Cascade(org.hibernate.annotations.CascadeType.DELETE_ORPHAN)
  @IndexColumn(name = "secuencia", base = 0)
  public List<Contenedor> getElementos() {
    return elementos;
  }

  /**
   * Asigna elementos.
   * 
   * @param elementos
   *          elementos
   */
  public void setElementos(List<Contenedor> elementos) {
    this.elementos = elementos;
  }
  
  // -------------------------------------------------------------------------------------------------------- Validators
  
  /**
   * Interfaz AnyoUnico.
   */
  @ValidatorClass(AnyoUnicoValidator.class)
  @Target( { ElementType.TYPE })
  @Retention(RetentionPolicy.RUNTIME)
  @Documented
  public static @interface AnyoUnico {

    /**
     * Message.
     * 
     * @return string
     */
    String message() default "{financiero.error.ejercicio.duplicado}";

  }

  /**
   * Clase AnyoUnicoValidator.
   */
  @Configurable
  public static class AnyoUnicoValidator implements Validator<ProyectoAnual.AnyoUnico> {

    private static final long serialVersionUID = 923478950742389573L;

    /** emf. */
    @PersistenceUnit
    EntityManagerFactory emf;

    /* (non-Javadoc)
     * @see org.hibernate.validator.Validator#initialize(java.lang.annotation.Annotation)
     */
    public void initialize(ProyectoAnual.AnyoUnico parameters) {}

    /* (non-Javadoc)
     * @see org.hibernate.validator.Validator#isValid(java.lang.Object)
     */
    public boolean isValid(Object value) {
      ProyectoAnual entity = (ProyectoAnual) value;
      EntityManager em=null;
      if(emf==null)
        return true;
      try{
        em = emf.createEntityManager();
        StringBuffer sql = new StringBuffer();
        sql.append("SELECT proyectoAnual FROM ").append(ProyectoAnual.class.getName()).append(" proyectoAnual WHERE proyectoGeneral IS NULL" +
        		" AND anyo = :anyo AND proyecto = :proyecto");
        Query query = em.createQuery(sql.toString());
        query.setParameter("anyo", entity.anyo);
        query.setParameter("proyecto", entity.proyecto);
      
        if (query.getResultList().size() > 0) return false;
        return true;
      } catch (RuntimeException e) {
        throw e;
      }
    }
  }

}
