package es.csa.fdt.cyldigital.financiero.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.usuarios.entity.Empleado;

/**
 * Clase Seccion.
 * 
 * @author csa
 */
@Entity
@Table(name = "secciones")
@AttributeOverride(name = "id", column = @Column(name = "idSeccion"))
@PrimaryKeyJoinColumn(name = "idContenedor")
@Audited
public class Seccion extends Contenedor implements Serializable {

  private static final long serialVersionUID = 4237895038905789235L;
  
  private Empleado responsable;
  
  /**
   * Construye un nuevo objeto de la clase
   */
  public Seccion(){
    super(TipoContenedor.SECCION);
    this.setSeccionRaiz(this);
  }

  /**
   * Construye un nuevo objeto de la clase
   * 
   * @param nombre
   *           nombre
   */
  public Seccion(String nombre){
    super(TipoContenedor.SECCION, nombre);
    this.setSeccionRaiz(this);
  }
  
  /**
   * Construye un nuevo objeto de la clase
   * 
   * @param nombre
   *           nombre
   * @param responsable
   *           responsable
   */
  public Seccion(String nombre, Empleado responsable)
  {
    super(TipoContenedor.SECCION, nombre);
    this.setSeccionRaiz(this);
    
    this.setResponsable(responsable);
  }
  
  /**
   * Construye un nuevo objeto de la clase
   * 
   * @param nombre
   *           nombre
   * @param responsable
   *           responsable
   * @param proyectoRaiz
   *           proyecto raiz
   */
  public Seccion(String nombre, Empleado responsable, ProyectoAnual proyectoRaiz)
  {
    super(TipoContenedor.SECCION, nombre, proyectoRaiz);
    this.setSeccionRaiz(this);
    
    this.setResponsable(responsable);
    this.setProyecto(proyectoRaiz);
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.financiero.entity.Contenedor#getResponsable()
   */
  @ManyToOne
  @JoinColumn(name = "idResponsable")
  @NotNull
  public Empleado getResponsable() {
    return responsable;
  }

  /**
   * Asigna responsable.
   * 
   * @param responsable
   *          responsable
   */
  public void setResponsable(Empleado responsable) {
    this.responsable = responsable;
  }
  
  /**
   * Devuelve responsable.
   * 
   * @param empleado
   *           empleado
   * @return responsable
   */
  @Transient
  public Empleado getResponsable(Empleado empleado){    
    return this.getResponsable();
  }

  /**
   * Clone seccion.
   * 
   * @param proyectoRaiz
   *           proyecto raiz
   * @param contenedorProyectoGeneral
   *           contenedor proyecto general
   * @return seccion
   */
  @Transient
  public Seccion cloneSeccion(ProyectoAnual proyectoRaiz, Contenedor contenedorProyectoGeneral)
  {
    //Creamos una sección con los mismos datos configurables (nombre y responsable).
    Seccion seccion = new Seccion(this.getNombre(), this.getResponsable());
    
    //Y le asignamos los parámetros variables como elemento independiente del actual y que nos vienen dados por parámetro.
    seccion.setProyecto(proyectoRaiz);
    seccion.setProyectoRaiz(proyectoRaiz);
    seccion.setContenedorProyectoGeneral(contenedorProyectoGeneral);
    
    return seccion;
  }
}
