package es.csa.fdt.cyldigital.financiero.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.List;

import javax.persistence.Query;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.persistence.browser.jpaql.BrowserJPAQL;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.financiero.entity.Contenedor;
import es.csa.fdt.cyldigital.financiero.entity.ProyectoAnual;

/**
 * Servicio del Browser de Contenedores.
 * 
 * @author csa
 */
@Service
@Repository
public class ContenedorBrowserImpl extends BrowserJPAQL<Contenedor> implements ContenedorBrowser<CriterionJPAQL, String> {

  private static final long serialVersionUID = 4230975894230758904L;
  
  private Centro centro;
  
  private Contenedor contenedor;
  
  /**
   * Construye un nuevo objeto de la clase
   */
  public ContenedorBrowserImpl(){
    super();
    getSort().setSortAsc(true);
    getSort().setSortProperty("esquema");
  }

  /**
   * Apply custom filter.
   * 
   * @param proyectoAnual
   *           proyecto anual
   */
  public void applyCustomFilter(ProyectoAnual proyectoAnual) {
    if (proyectoAnual != null){
      this.getCondition().getCriterion().setQuery("e.proyectoRaiz = ?");
      this.getCondition().getCriterion().addParameter(proyectoAnual);
    }
    
    this.applyCondition();
  }
  
  /**
   * Devuelve centro.
   * 
   * @return centro
   */
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }
  
  /**
   * Devuelve contenedor.
   * 
   * @return contenedor
   */
  public Contenedor getContenedor() {
    return contenedor;
  }

  /**
   * Asigna contenedor.
   * 
   * @param contenedor
   *          contenedor
   */
  public void setContenedor(Contenedor contenedor) {
    this.contenedor = contenedor;
  }

  /**
   * Clear.
   */
  public void clear() {
    this.getEntityManager().clear();
  }
  
  /**
   * Devuelve primer elemento.
   * 
   * @param elementos
   *           elementos
   * @return primer elemento
   */
  public List<Contenedor> getPrimerElemento(List<Contenedor> elementos){
    List<Contenedor> primerElemento = new ArrayList<Contenedor>();
    if(!elementos.isEmpty()){
      primerElemento.add(elementos.get(0));
    }
    
    return primerElemento;
  }
  
  /**
   * Devuelve resto elementos.
   * 
   * @param elementos
   *           elementos
   * @return resto elementos
   */
  public List<Contenedor> getRestoElementos(List<Contenedor> elementos){
   List<Contenedor> elementosAux = new ArrayList<Contenedor>();
   
   if(!elementos.isEmpty()){
     elementosAux.addAll(elementos);
     elementosAux.remove(0);
   }
   
   return elementosAux;
  }
  
  /**
   * Devuelve centros disponibles.
   * 
   * @return centros disponibles
   */
  @SuppressWarnings("unchecked")
  public List<Centro> getCentrosDisponibles() {
    Query query = em.createQuery("SELECT e FROM " + Centro.class.getName()
        + " e WHERE desactivado = false ORDER BY nombre ASC");
    List<Centro> centros = (List<Centro>) query.getResultList();
    return centros;
  }
  
}
