package es.csa.fdt.cyldigital.financiero.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.List;

import es.csa.commons.persistence.browser.jpaql.BrowserJPAQL;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.fdt.cyldigital.financiero.entity.DocumentoFactura;
import es.csa.fdt.cyldigital.financiero.entity.DocumentoFactura.TipoDocumentoFactura;

/**
 * Servicio del Browser de Documentos de facturas.
 * 
 * @author csa
 */
public class DocumentoFacturaBrowserImpl extends BrowserJPAQL<DocumentoFactura> implements DocumentoFacturaBrowser<CriterionJPAQL, String> {

  private static final long serialVersionUID = 3749507423950874239L;
  
  /**
   * Apply custom filter.
   * 
   * @param tipo
   *           tipo
   */
  public void applyCustomFilter(TipoDocumentoFactura tipo) {
    if(tipo != null){
      StringBuffer query = new StringBuffer();
      
      query.append("e.tipoDocumentoFactura = ?");
      this.getCondition().getCriterion().addParameter(tipo);
      
      if(tipo == TipoDocumentoFactura.JUSTIFICACION_PAGO){
        query.append(" AND e.pago IS NULL");
      }
      
      this.getCondition().getCriterion().setQuery(query.toString());
    }
    
    this.applyCondition();
  }

  /**
   * Devuelve records.
   * 
   * @param documentosYaSeleccionados
   *           documentos ya seleccionados
   * @return records
   */
  public List<DocumentoFactura> getRecords(List<DocumentoFactura> documentosYaSeleccionados) {
    List<DocumentoFactura> todosDocumentos = super.getRecords();
    for(DocumentoFactura documento : documentosYaSeleccionados){
      if(todosDocumentos.contains(documento)){
        todosDocumentos.remove(documento);
      }
    }
    
    return todosDocumentos;
  }
  
  
  
}
