package es.csa.fdt.cyldigital.financiero.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import org.richfaces.event.UploadEvent;

import es.csa.commons.faces.util.FicheroSubidoRichFacesAdapter;
import es.csa.fdt.cyldigital.core.entity.FileOnDisk;
import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.core.entity.FileOnDisk.Tipo;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.financiero.entity.DocumentoFactura;
import es.csa.fdt.cyldigital.financiero.entity.Factura;
import es.csa.fdt.cyldigital.financiero.entity.FileFactura;
import es.csa.fdt.cyldigital.financiero.entity.Pago;

/**
 * Clase DocumentoFacturaServiceImpl.
 * 
 * @author csa
 */
public class DocumentoFacturaServiceImpl extends AbstractServiceImpl<DocumentoFactura, Long> implements
    DocumentoFacturaService {

  private static final long serialVersionUID = 934758904237589074L;

  /**
   * Desasociar.
   * 
   * @param documento
   *           documento
   * @param factura
   *           factura
   */
  public void desasociar(DocumentoFactura documento, Factura factura) {
    if (documento.getFacturas().contains(factura)) {
      documento.getFacturas().remove(factura);
      factura.getDocumentosFactura().remove(documento);
    }
  }

  /**
   * Desasociar pago.
   * 
   * @param documento
   *           documento
   * @param pago
   *           pago
   */
  public void desasociarPago(DocumentoFactura documento, Pago pago) {
    if (documento.getPago() == pago) {
      documento.setPago(null);
      pago.setDocumentoFactura(null);
    }
  }

  /**************************************************** Ficheros ***************************************************************/
  private FileOnDisk lastUploaded2;
  private FileFactura lastUploaded;

  /*
   * public synchronized void aniadirFichero(UploadEvent event) { lastUploaded = new FileFactura();
   * lastUploaded.setFicheroSubido(new FicheroSubidoRichFacesAdapter(event.getUploadItem())); }
   * 
   * public DocumentoFactura quitarFichero(DocumentoFactura documentoFactura) {
   * borrarFicheroTemporal(documentoFactura.getFichero()); //Produce un error al volver a la pantalla de
   * DocumentoFacturaView y comprobar que el fichero ha sido borrado. // if(documentoFactura.getFichero() != null){ //
   * remove(documentoFactura.getFichero()); // } documentoFactura.setFichero(null); setLastUploaded(null); return
   * documentoFactura; }
   */

  /**
   * Aniadir fichero.
   * 
   * @param event
   *           event
   */
  public synchronized void aniadirFichero(UploadEvent event) {
    lastUploaded2 = new FileOnDisk();
    lastUploaded2.setFicheroSubido(new FicheroSubidoRichFacesAdapter(event.getUploadItem()));
  }

  /**
   * Quitar fichero.
   * 
   * @param d
   *           d
   */
  public void quitarFichero(DocumentoFactura d) {
    lastUploaded2 = null;
    d.setFicheroEnDisco(null);
  }

  /**
   * Borrar fichero temporal.
   * 
   * @param file
   *           file
   */
  public void borrarFicheroTemporal(FileFactura file) {
    if (file != null && file.getFicheroSubido() != null) {
      file.getFicheroSubido().deleteFichero();
    }
  }

  /**
   * Devuelve last uploaded.
   * 
   * @return last uploaded
   */
  public FileFactura getLastUploaded() {
    return lastUploaded;
  }

  /**
   * Asigna last uploaded.
   * 
   * @param lastUploaded
   *          last uploaded
   */
  public void setLastUploaded(FileFactura lastUploaded) {
    this.lastUploaded = lastUploaded;
  }

  /**
   * Do anniadir fichero a documento.
   * 
   * @param documento
   *           documento
   */
  public void doAnniadirFicheroADocumento(DocumentoFactura documento) {
    if (lastUploaded2 != null) {
      documento.setFicheroEnDisco(lastUploaded2);
      setLastUploaded2(null);
    }
  }

  /**
   * Devuelve ************************************************** Ficheros
   * **************************************************************.
   * 
   * @return ************************************************** Ficheros
   *         **************************************************************
   */
  public FileOnDisk getLastUploaded2() {
    return lastUploaded2;
  }

  /**
   * Asigna ************************************************** Ficheros
   * **************************************************************.
   * 
   * @param lastUploaded2
   *          ************************************************** Ficheros
   *          **************************************************************
   */
  public void setLastUploaded2(FileOnDisk lastUploaded2) {
    this.lastUploaded2 = lastUploaded2;
  }

  /**
   * Persist documento factura.
   * 
   * @param d
   *           d
   */
  public void persistDocumentoFactura(DocumentoFactura d) {
    if (lastUploaded2 != null) {
      Parameter p = findOneWhere(Parameter.class, "nombre", "pathfileondisk");
      lastUploaded2.setTipo(Tipo.DOCUMENTOFACTURA);
      lastUploaded2.grabarEnDisco(p.getValor());
      disableMessages();
      persist(lastUploaded2);
      d.setFicheroEnDisco(lastUploaded2);
      enableMessages();
      setLastUploaded2(null);
    }
    persist(d);
  }
}
