package es.csa.fdt.cyldigital.financiero.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import org.richfaces.event.UploadEvent;

import es.csa.commons.faces.util.FicheroSubidoRichFacesAdapter;
import es.csa.fdt.cyldigital.core.entity.FileOnDisk;
import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.core.entity.FileOnDisk.Tipo;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.financiero.entity.DocumentoMovimiento;
import es.csa.fdt.cyldigital.financiero.entity.FileMovimiento;
import es.csa.fdt.cyldigital.financiero.entity.Movimiento;

/**
 * Clase DocumentoMovimientoServiceImpl.
 * 
 * @author csa
 */
public class DocumentoMovimientoServiceImpl extends AbstractServiceImpl<DocumentoMovimiento, Long> implements
    DocumentoMovimientoService {

  private static final long serialVersionUID = 8937590437895074385L;

  /**
   * Desasociar movimiento.
   * 
   * @param documento
   *           documento
   * @param movimiento
   *           movimiento
   */
  public void desasociarMovimiento(DocumentoMovimiento documento, Movimiento movimiento) {
    if (documento.getMovimiento() == movimiento) {
      documento.setMovimiento(null);
      movimiento.setDocumentoMovimiento(null);
    }
  }

  /**************************************************** Ficheros ***************************************************************/

  private FileMovimiento lastUploaded;
  private FileOnDisk lastUploaded2;

  /*
   * public synchronized void aniadirFichero(UploadEvent event) { lastUploaded = new FileMovimiento();
   * lastUploaded.setFicheroSubido(new FicheroSubidoRichFacesAdapter(event.getUploadItem())); }
   */

  /*
   * public DocumentoMovimiento quitarFichero(DocumentoMovimiento documentoMovimiento) {
   * borrarFicheroTemporal(documentoMovimiento.getFichero());
   * 
   * documentoMovimiento.setFichero(null); setLastUploaded(null); return documentoMovimiento; }
   */

  /**
   * Borrar fichero temporal.
   * 
   * @param file
   *           file
   */
  public void borrarFicheroTemporal(FileMovimiento file) {
    if (file != null && file.getFicheroSubido() != null) {
      file.getFicheroSubido().deleteFichero();
    }
  }

  /**
   * Devuelve ************************************************** Ficheros
   * **************************************************************.
   * 
   * @return ************************************************** Ficheros
   *         **************************************************************
   */
  public FileMovimiento getLastUploaded() {
    return lastUploaded;
  }

  /**
   * Asigna ************************************************** Ficheros
   * **************************************************************.
   * 
   * @param lastUploaded
   *          ************************************************** Ficheros
   *          **************************************************************
   */
  public void setLastUploaded(FileMovimiento lastUploaded) {
    this.lastUploaded = lastUploaded;
  }

  /**
   * Do anniadir fichero a documento.
   * 
   * @param documento
   *           documento
   */
  public void doAnniadirFicheroADocumento(DocumentoMovimiento documento) {
    if (lastUploaded != null) {
      documento.setFichero(lastUploaded);
      setLastUploaded(null);
    }
  }

  /**
   * Devuelve last uploaded2.
   * 
   * @return last uploaded2
   */
  public FileOnDisk getLastUploaded2() {
    return lastUploaded2;
  }

  /**
   * Asigna last uploaded2.
   * 
   * @param lastUploaded2
   *          last uploaded2
   */
  public void setLastUploaded2(FileOnDisk lastUploaded2) {
    this.lastUploaded2 = lastUploaded2;
  }

  /**
   * Persist documento movimiento.
   * 
   * @param d
   *           d
   */
  public void persistDocumentoMovimiento(DocumentoMovimiento d) {
    Parameter p = findOneWhere(Parameter.class, "nombre", "pathfileondisk");

    if (lastUploaded2 != null) {
      disableMessages();
      lastUploaded2.setTipo(Tipo.DOCUMENTOFACTURA);
      lastUploaded2.grabarEnDisco(p.getValor());
      persist(lastUploaded2);
      d.setFile(lastUploaded2);
    }
    persist(d);
  }

  /**
   * Aniadir fichero.
   * 
   * @param event
   *           event
   */
  public synchronized void aniadirFichero(UploadEvent event) {
    lastUploaded2 = new FileOnDisk();
    lastUploaded2.setFicheroSubido(new FicheroSubidoRichFacesAdapter(event.getUploadItem()));
  }

  /**
   * Quitar fichero.
   * 
   * @param d
   *           d
   */
  public void quitarFichero(DocumentoMovimiento d) {
    lastUploaded2 = null;
    d.setFile(null);
  }
}
