package es.csa.fdt.cyldigital.financiero.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.Date;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.jpaql.BrowserJPAQL;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.commons.persistence.browser.jpaql.FilterJPAQLByQuery;
import es.csa.fdt.cyldigital.financiero.entity.Contenedor;
import es.csa.fdt.cyldigital.financiero.entity.Factura;
import es.csa.fdt.cyldigital.financiero.entity.Proyecto;
import es.csa.fdt.cyldigital.financiero.entity.DocumentoFactura.EstadoDocumentoFactura;
import es.csa.fdt.cyldigital.financiero.entity.EstadoFactura.TipoEstadoFactura;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.Proveedor;

/**
 * Servicio del Browser de Facturas.
 * 
 * @author csa
 */
@Service
@Repository
public class FacturaBrowserImpl extends BrowserJPAQL<Factura> implements FacturaBrowser<CriterionJPAQL, String> {

  private static final long serialVersionUID = 2984375904237950874L;
  
  /**
   * Construye un nuevo objeto de la clase
   */
  public FacturaBrowserImpl(){
    super();
  }

  /**
   * Apply custom filter.
   * 
   * @param contenedor
   *           contenedor
   */
  public void applyCustomFilter(Contenedor contenedor) {
    if (contenedor != null){
      this.getCondition().getCriterion().setQuery("e.contenedor = ?");
      this.getCondition().getCriterion().addParameter(contenedor);
    }
    
    if(contenedor.getContenedorProyectoGeneral() == null){//Es un contenedor de un proyecto general.
      String query = this.getCondition().getCriterion().getQuery();      
      query = query.concat(" OR e.contenedor.contenedorProyectoGeneral = ?");
      this.getCondition().getCriterion().setQuery(query);
      this.getCondition().getCriterion().addParameter(contenedor);
    }else{
      String query = this.getCondition().getCriterion().getQuery();
      query = query.concat(" OR ? MEMBER OF e.contenedor.contenedoresSubproyectos");
      this.getCondition().getCriterion().setQuery(query);
      this.getCondition().getCriterion().addParameter(contenedor);
    }
    
    this.applyCondition();
  }
  
  /**
   * Clear.
   */
  public void clear() {
    this.getEntityManager().clear();
  }
  
  /* Para el buscador de facturas */
  
  private Proyecto proyecto;
  private Proyecto subproyecto;
  private Proveedor proveedor;
  private Empleado responsableArea;
  private TipoEstadoFactura tipoEstadoFactura;
  private Date fechaInicio;
  private Date fechaFin;
  private EstadoDocumentoFactura estadoDocumentacion;
  
  
  /**
   * Devuelve proyecto.
   * 
   * @return proyecto
   */
  public Proyecto getProyecto() {
    return proyecto;
  }

  /**
   * Asigna proyecto.
   * 
   * @param proyecto
   *          proyecto
   */
  public void setProyecto(Proyecto proyecto) {
    this.proyecto = proyecto;
  }

  /**
   * Devuelve subproyecto.
   * 
   * @return subproyecto
   */
  public Proyecto getSubproyecto() {
    return subproyecto;
  }

  /**
   * Asigna subproyecto.
   * 
   * @param subproyecto
   *          subproyecto
   */
  public void setSubproyecto(Proyecto subproyecto) {
    this.subproyecto = subproyecto;
  }

  /**
   * Devuelve proveedor.
   * 
   * @return proveedor
   */
  public Proveedor getProveedor() {
    return proveedor;
  }

  /**
   * Asigna proveedor.
   * 
   * @param proveedor
   *          proveedor
   */
  public void setProveedor(Proveedor proveedor) {
    this.proveedor = proveedor;
  }

  /**
   * Devuelve responsable area.
   * 
   * @return responsable area
   */
  public Empleado getResponsableArea() {
    return responsableArea;
  }

  /**
   * Asigna responsable area.
   * 
   * @param responsableArea
   *          responsable area
   */
  public void setResponsableArea(Empleado responsableArea) {
    this.responsableArea = responsableArea;
  }

  /**
   * Devuelve tipo estado factura.
   * 
   * @return tipo estado factura
   */
  public TipoEstadoFactura getTipoEstadoFactura() {
    return tipoEstadoFactura;
  }

  /**
   * Asigna tipo estado factura.
   * 
   * @param tipoEstadoFactura
   *          tipo estado factura
   */
  public void setTipoEstadoFactura(TipoEstadoFactura tipoEstadoFactura) {
    this.tipoEstadoFactura = tipoEstadoFactura;
  }
  
  /**
   * Devuelve fecha inicio.
   * 
   * @return fecha inicio
   */
  public Date getFechaInicio() {
    return fechaInicio;
  }

  /**
   * Asigna fecha inicio.
   * 
   * @param fechaInicio
   *          fecha inicio
   */
  public void setFechaInicio(Date fechaInicio) {
    this.fechaInicio = fechaInicio;
  }
  
  /**
   * Devuelve fecha inicio in millis.
   * 
   * @return fecha inicio in millis
   */
  public long getFechaInicioInMillis(){
    return this.getFechaInicio() == null ? 0 : this.getFechaInicio().getTime();
  }

  /**
   * Devuelve fecha fin.
   * 
   * @return fecha fin
   */
  public Date getFechaFin() {
    return fechaFin;
  }

  /**
   * Asigna fecha fin.
   * 
   * @param fechaFin
   *          fecha fin
   */
  public void setFechaFin(Date fechaFin) {
    this.fechaFin = fechaFin;
  }
  
  /**
   * Devuelve estado documentacion.
   * 
   * @return estado documentacion
   */
  public EstadoDocumentoFactura getEstadoDocumentacion() {
    return estadoDocumentacion;
  }

  /**
   * Asigna estado documentacion.
   * 
   * @param estadoDocumentacion
   *          estado documentacion
   */
  public void setEstadoDocumentacion(EstadoDocumentoFactura estadoDocumentacion) {
    this.estadoDocumentacion = estadoDocumentacion;
  }

  /**
   * Devuelve fecha fin in millis.
   * 
   * @return fecha fin in millis
   */
  public long getFechaFinInMillis(){
    return this.getFechaFin() == null ? 0 : this.getFechaFin().getTime();
  }

  /**
   * Filter proyecto.
   */
  public void filterProyecto() {
    removeFilterProyecto();
    if (getProyecto() != null) filterByProyecto(getProyecto().getId());
  }
  
  private void removeFilterProyecto() {
    int i = -1;
    for (Filter<CriterionJPAQL, Factura> f : getFilters())
      if (f.getCriterion().getQuery().contains("contenedor.proyectoRaiz.proyecto.proyectoGeneral.id")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
  }
  
  /**
   * Filter by proyecto.
   * 
   * @param idProyecto
   *           id proyecto
   */
  public void filterByProyecto(Long idProyecto) {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<Factura> filter = new FilterJPAQLByQuery<Factura>();

    query += " contenedor.proyectoRaiz.proyecto.id = ? OR contenedor.proyectoRaiz.proyecto.proyectoGeneral.id = ?";
    filter.getCriterion().addParameter(idProyecto);
    filter.getCriterion().addParameter(idProyecto);

    for (Filter<CriterionJPAQL, Factura> f : getFilters())
      if (f.getCriterion().getQuery().contains("contenedor.proyectoRaiz.proyecto.proyectoGeneral.id")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }
  
  /**
   * Filter subproyecto.
   */
  public void filterSubproyecto() {
    removeFilterSubproyecto();
    if (getSubproyecto() != null) filterBySubproyecto(getSubproyecto().getId());
  }
  
  private void removeFilterSubproyecto() {
    int i = -1;
    for (Filter<CriterionJPAQL, Factura> f : getFilters())
      if (f.getCriterion().getQuery().contains("contenedor.proyectoRaiz.proyecto.id")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
  }
  
  /**
   * Filter by subproyecto.
   * 
   * @param idSubproyecto
   *           id subproyecto
   */
  public void filterBySubproyecto(Long idSubproyecto) {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<Factura> filter = new FilterJPAQLByQuery<Factura>();

    query += " contenedor.proyectoRaiz.proyecto.id = ?";
    filter.getCriterion().addParameter(idSubproyecto);

    for (Filter<CriterionJPAQL, Factura> f : getFilters())
      if (f.getCriterion().getQuery().contains("contenedor.proyectoRaiz.proyecto.id")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }
  
  /**
   * Filter proveedor.
   */
  public void filterProveedor() {
    removeFilterProveedor();
    if (getProveedor() != null) filterByProveedor(getProveedor().getId());
  }
  
  private void removeFilterProveedor() {
    int i = -1;
    for (Filter<CriterionJPAQL, Factura> f : getFilters())
      if (f.getCriterion().getQuery().contains("proveedor.id")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
  }
  
  /**
   * Filter by proveedor.
   * 
   * @param idProveedor
   *           id proveedor
   */
  public void filterByProveedor(Long idProveedor) {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<Factura> filter = new FilterJPAQLByQuery<Factura>();

    query += " proveedor.id = ? ";
    filter.getCriterion().addParameter(idProveedor);

    for (Filter<CriterionJPAQL, Factura> f : getFilters())
      if (f.getCriterion().getQuery().contains("proveedor.id")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }
  
  /**
   * Filter responsable area.
   */
  public void filterResponsableArea() {
    removeResponsableArea();
    if (getResponsableArea() != null) filterByResponsableArea(getResponsableArea().getId());
  }
  
  private void removeResponsableArea() {
    int i = -1;
    for (Filter<CriterionJPAQL, Factura> f : getFilters())
      if (f.getCriterion().getQuery().contains("contenedor.seccionRaiz.responsable.id")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
  }
  
  /**
   * Filter by responsable area.
   * 
   * @param idResponsableArea
   *           id responsable area
   */
  public void filterByResponsableArea(Long idResponsableArea) {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<Factura> filter = new FilterJPAQLByQuery<Factura>();

    query += " contenedor.seccionRaiz.responsable.id = ? ";
    filter.getCriterion().addParameter(idResponsableArea);

    for (Filter<CriterionJPAQL, Factura> f : getFilters())
      if (f.getCriterion().getQuery().contains("contenedor.seccionRaiz.responsable.id")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }
  
  /**
   * Filter estado documentacion.
   */
  public void filterEstadoDocumentacion() {
    removeFilterEstadoDocumentacion();
    if (getEstadoDocumentacion() != null) filterByEstadoDocumentacion(getEstadoDocumentacion());
  }
  
  private void removeFilterEstadoDocumentacion() {
    int i = -1;
    for (Filter<CriterionJPAQL, Factura> f : getFilters())
      if (f.getCriterion().getQuery().contains("justificada")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
  }
  
  /**
   * Filter by estado documentacion.
   * 
   * @param estadoDocumentoFactura
   *           estado documento factura
   */
  public void filterByEstadoDocumentacion(EstadoDocumentoFactura estadoDocumentoFactura) {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<Factura> filter = new FilterJPAQLByQuery<Factura>();
    
    if(estadoDocumentoFactura == EstadoDocumentoFactura.PENDIENTE_JUSTIFICACION){
      query += "justificada = false AND aprobadaJunta = false"; 
    }else if(estadoDocumentoFactura == EstadoDocumentoFactura.PENDIENTE_APROBACION_JUNTA){
      query += "justificada = true AND aprobadaJunta = false";
    }else if(estadoDocumentoFactura == EstadoDocumentoFactura.APROBADA_JUNTA){
      query += "justificada = true AND aprobadaJunta = true";
    }

    for (Filter<CriterionJPAQL, Factura> f : getFilters())
      if (f.getCriterion().getQuery().contains("justificada")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }
  
  /**
   * Filter tipo estado factura.
   */
  public void filterTipoEstadoFactura() {
    removeFilterTipoEstadoFactura();
    if (getTipoEstadoFactura() != null) filterByTipoEstadoFactura(getTipoEstadoFactura());
  }
  
  private void removeFilterTipoEstadoFactura() {
    int i = -1;
    for (Filter<CriterionJPAQL, Factura> f : getFilters())
      if (f.getCriterion().getQuery().contains("estadoActual.tipoEstadoFactura")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
  }
  
  /**
   * Filter by tipo estado factura.
   * 
   * @param tipoEstadoFactura
   *           tipo estado factura
   */
  public void filterByTipoEstadoFactura(TipoEstadoFactura tipoEstadoFactura) {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<Factura> filter = new FilterJPAQLByQuery<Factura>();

    query += " estadoActual.tipoEstadoFactura = ? ";
    filter.getCriterion().addParameter(tipoEstadoFactura);

    for (Filter<CriterionJPAQL, Factura> f : getFilters())
      if (f.getCriterion().getQuery().contains("estadoActual.tipoEstadoFactura")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }
  
  /**
   * Filter fechas.
   */
  public void filterFechas() {
    removeFilterFechas();
    if (getFechaInicio() != null || getFechaFin() != null) filterByFechas(getFechaInicio(), getFechaFin());
  }
  
  private void removeFilterFechas() {
    int i = -1;
    for (Filter<CriterionJPAQL, Factura> f : getFilters())
      if (f.getCriterion().getQuery().contains("fechaIntroduccion")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
  }
  
  /**
   * Filter by fechas.
   * 
   * @param fechaInicio
   *           fecha inicio
   * @param fechaFin
   *           fecha fin
   */
  public void filterByFechas(Date fechaInicio, Date fechaFin) {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<Factura> filter = new FilterJPAQLByQuery<Factura>();

    if(fechaInicio != null){
      query += " fechaIntroduccion >= ?";
      filter.getCriterion().addParameter(fechaInicio);
    }
    if(fechaFin != null){
      if(fechaInicio != null){
        query += "AND ";
      }
      query += " fechaIntroduccion <= ?";
      filter.getCriterion().addParameter(fechaFin);
    }

    for (Filter<CriterionJPAQL, Factura> f : getFilters())
      if (f.getCriterion().getQuery().contains("fechaIntroduccion")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }

}
