package es.csa.fdt.cyldigital.financiero.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.financiero.entity.Contenedor;
import es.csa.fdt.cyldigital.financiero.entity.DocumentoFactura;
import es.csa.fdt.cyldigital.financiero.entity.EstadoFactura;
import es.csa.fdt.cyldigital.financiero.entity.EstadoPago;
import es.csa.fdt.cyldigital.financiero.entity.Factura;
import es.csa.fdt.cyldigital.financiero.entity.Pago;
import es.csa.fdt.cyldigital.financiero.entity.Contenedor.TipoContenedor;
import es.csa.fdt.cyldigital.financiero.entity.DocumentoFactura.EstadoDocumentoFactura;
import es.csa.fdt.cyldigital.financiero.entity.EstadoFactura.TipoEstadoFactura;
import es.csa.fdt.cyldigital.financiero.entity.EstadoPago.TipoEstadoPago;
import es.csa.fdt.cyldigital.financiero.entity.Factura.TipoPago;
import es.csa.fdt.cyldigital.financiero.entity.Pago.TipoCuantiaSubproyectos;
import es.csa.fdt.cyldigital.notificacion.NotificationService;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;

/**
 * Clase FacturaServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class FacturaServiceImpl extends AbstractServiceImpl<Factura, Long> implements FacturaService {

  private static final long serialVersionUID = 4238594238950345438L;

  /**
   * Checks if is tipo seccion.
   * 
   * @param tipoContenedor
   *           tipo contenedor
   * @param idContenedor
   *           id contenedor
   * @return true, if is tipo seccion
   */
  public boolean isTipoSeccion(TipoContenedor tipoContenedor, Long idContenedor) {

    boolean result;

    if (tipoContenedor == TipoContenedor.SECCION) result = true;
    else result = false;

    return result;
  }

  /**
   * Asigna estado inicial factura.
   * 
   * @param factura
   *           factura
   * @param usuario
   *           usuario
   */
  public void setEstadoInicialFactura(Factura factura, Usuario usuario) {
    Date fechaIntroduccion = new Date();
    factura.setFechaIntroduccion(fechaIntroduccion);
    EstadoFactura estadoInicial = new EstadoFactura(TipoEstadoFactura.PRESUPUESTO_PENDIENTE_APROBACION, factura,
        fechaIntroduccion, usuario, "Alta de Presupuesto");
    factura.getEstadosFactura().add(estadoInicial);
    factura.setEstadoActual(estadoInicial);
  }

  /**
   * Asigna nuevo estado factura.
   * 
   * @param tipoEstadoFacturaAntiguo
   *           tipo estado factura antiguo
   * @param estadoFactura
   *           estado factura
   * @param factura
   *           factura
   * @param usuario
   *           usuario
   */
  @SuppressWarnings("unchecked")
  public void setNuevoEstadoFactura(TipoEstadoFactura tipoEstadoFacturaAntiguo, EstadoFactura estadoFactura, Factura factura, Usuario usuario) {
    NotificationService n = new NotificationService();
    
    if (tipoEstadoFacturaAntiguo != estadoFactura.getTipoEstadoFactura()) {
      estadoFactura.setFecha(new Date());
      estadoFactura.setFactura(factura);
      estadoFactura.setUsuario(usuario);

      factura.setEstadoActual(estadoFactura);
      super.persist(estadoFactura);
      
      if(estadoFactura.getTipoEstadoFactura() == TipoEstadoFactura.PRESUPUESTO_RECHAZADO){
        //Obtenemos el usuario que ha creado el presupuesto para notificarle el rechazo.
        Usuario creadorFactura = factura.getEstadosFactura().get(0).getUsuario();
        ArrayList<Usuario> destinatarios = new ArrayList<Usuario>();
        destinatarios.add(creadorFactura);
        n.notificarPresupuestoRechazado(this.getEntityManager(), destinatarios, factura);
      }
    }
  }

  /**
   * Siguientes tipos estado factura.
   * 
   * @param factura
   *           factura
   * @return list
   */
  public List<TipoEstadoFactura> siguientesTiposEstadoFactura(Factura factura) {
    EstadoFactura estadoFactura = factura.getEstadoActual();
    return estadoFactura.siguientesTiposEstadoFactura();
  }

  /**
   * Anterior tipo estado factura.
   * 
   * @param factura
   *           factura
   * @return tipo estado factura
   */
  public TipoEstadoFactura anteriorTipoEstadoFactura(Factura factura) {
    EstadoFactura estadoFactura = factura.getEstadoActual();
    return estadoFactura.anteriorTipoEstadoFactura();
  }

  /**
   * Find estado factura.
   * 
   * @param idEstadoFactura
   *           id estado factura
   * @return estado factura
   */
  public EstadoFactura findEstadoFactura(Long idEstadoFactura) {
    return (EstadoFactura) this.find(EstadoFactura.class, idEstadoFactura);
  }
  
  /**
   * Desasociar.
   * 
   * @param factura
   *           factura
   * @param documento
   *           documento
   */
  public void desasociar(Factura factura, DocumentoFactura documento){
    if(factura.getDocumentosFactura().contains(documento)){
      factura.getDocumentosFactura().remove(documento);
    }
  }
  
  /**
   * Aniadir documento factura.
   * 
   * @param factura
   *           factura
   * @param documento
   *           documento
   */
  public void aniadirDocumentoFactura(Factura factura, DocumentoFactura documento){
    if(!factura.getDocumentosFactura().contains(documento)){
      factura.getDocumentosFactura().add(documento);
    }
  }
  
  /**
   * Comprobar documentacion.
   * 
   * @param factura
   *           factura
   * @param estadoFactura
   *           estado factura
   * @return true, if successful
   */
  public boolean comprobarDocumentacion(Factura factura, EstadoFactura estadoFactura){
    boolean result = false;
    if(estadoFactura.getTipoEstadoFactura() != TipoEstadoFactura.PENDIENTE_PAGO && estadoFactura.getTipoEstadoFactura() != TipoEstadoFactura.ARCHIVADA){
      result = true;
    }else{
      List<DocumentoFactura> documentosFactura = factura.getDocumentosFactura();
      if(estadoFactura.getTipoEstadoFactura() == TipoEstadoFactura.PENDIENTE_PAGO){
        for(DocumentoFactura documentoFactura : documentosFactura){
          if(documentoFactura.getEstadoDocumentoFactura() == EstadoDocumentoFactura.PENDIENTE_APROBACION_JUNTA){
            result = true;
            break;
          }
        }
      }else if(estadoFactura.getTipoEstadoFactura() == TipoEstadoFactura.ARCHIVADA){
        for(DocumentoFactura documentoFactura : documentosFactura){
          if(documentoFactura.getEstadoDocumentoFactura() == EstadoDocumentoFactura.APROBADA_JUNTA){
            result = true;
            break;
          }
        }
      }
    }
    return result;
  }
  
  /**
   * Notificar alta presupuesto.
   * 
   * @param factura
   *           factura
   */
  @SuppressWarnings("unchecked")
  public void notificarAltaPresupuesto(Factura factura){
    NotificationService n = new NotificationService();
    
    //Obtenemos el usuario responsable de la aprobación del presupuestos atendiendo a su cuantía.
    Empleado responsableAprobacion = factura.getResponsableAprobacionPresupuesto();
    ArrayList<Usuario> destinatarios = new ArrayList<Usuario>();
    destinatarios.add(responsableAprobacion);
    n.notificarAprobacionPresupuesto(this.getEntityManager(), destinatarios, factura);
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.core.service.AbstractServiceImpl#remove(java.lang.Object)
   */
  @Override
  public <T> void remove(T entity) {
    Factura factura = (Factura)entity;
    factura.getContenedor().getFacturas().remove(factura);
    factura.setContenedor(null);
    super.remove(entity);
  }  
  
  //********************************************Pagos*****************************************************************//

  /**
   * Crear pagos.
   * 
   * @param factura
   *           factura
   * @param fechaEfectivaPrimerPago
   *           fecha efectiva primer pago
   * @param fechaFinMensual
   *           fecha fin mensual
   * @param numeroPlazos
   *           numero plazos
   * @param usuario
   *           usuario
   */
  public void crearPagos(Factura factura, Date fechaEfectivaPrimerPago, Date fechaFinMensual, int numeroPlazos, Usuario usuario){
    List<Contenedor> contenedoresSubproyectos = null;
    if(factura.getContenedor().getContenedorProyectoGeneral() == null){//Se crea desde un proyecto general.
      contenedoresSubproyectos = factura.getContenedor().getContenedoresSubproyectos();
    }
    else{//Se crea desde un subproyecto.
      contenedoresSubproyectos = new ArrayList<Contenedor>();
      contenedoresSubproyectos.add(factura.getContenedor());
    }
    
    int numeroPagos = 0;
    if(factura.getTipoPago() == TipoPago.UNICO){
      numeroPagos = 1;
    }else if(factura.getTipoPago() == TipoPago.MENSUAL){
      if(fechaEfectivaPrimerPago.after(fechaFinMensual)){
        this.addErrorMessage("La fecha de inicio es posterior a la fecha de fin.");
      }else{
        GregorianCalendar calendarFechaEfectivaPrimerPago = new GregorianCalendar();
        calendarFechaEfectivaPrimerPago.setTime(fechaEfectivaPrimerPago);
        GregorianCalendar calendarFechaFinMensual = new GregorianCalendar();
        calendarFechaFinMensual.setTime(fechaFinMensual);
        
        calendarFechaEfectivaPrimerPago.compareTo(calendarFechaFinMensual);
        
        int numeroPagosMensual = 0;
        while(calendarFechaEfectivaPrimerPago.compareTo(calendarFechaFinMensual) <=0){
          calendarFechaEfectivaPrimerPago.add(Calendar.MONTH, 1);
          numeroPagosMensual++;
        }
        
        numeroPagos = numeroPagosMensual;
      }
    }else if(factura.getTipoPago() == TipoPago.PLAZOS_FIJOS){
      numeroPagos = numeroPlazos;
    }
    
    //Eliminamos los pagos existentes en ese momento.
    for(Pago pago : factura.getPagos()){
      pago.setFactura(null);
      if(pago.getId() != 0){
        this.remove(pago);
      }
    }
    factura.getPagos().clear();
    
    Calendar calendarFechaEfectiva = null;
    for(int i = 1; i <= numeroPagos; i++){
      calendarFechaEfectiva = new GregorianCalendar();
      calendarFechaEfectiva.setTime(fechaEfectivaPrimerPago);
      calendarFechaEfectiva.add(Calendar.MONTH, i - 1);
      
      Pago pago = new Pago();
      pago.setFactura(factura);
      factura.getPagos().add(pago);
      
      pago.setFechaEfectiva(calendarFechaEfectiva.getTime());
      EstadoPago estadoPago = new EstadoPago(TipoEstadoPago.PENDIENTE_PAGO, pago, factura.getFechaIntroduccion(), usuario,
          "Alta de Pago");
      pago.setEstadoActual(estadoPago);
      pago.getEstadosPago().add(estadoPago);
      
      for(Contenedor contenedorSubproyecto : contenedoresSubproyectos)
      {
        Pago pagoSubproyecto = new Pago(pago, contenedorSubproyecto.getCentro());
        pago.getPagoSubproyectos().add(pagoSubproyecto);
        
        pagoSubproyecto.setFechaEfectiva(calendarFechaEfectiva.getTime());
      }
    }
    
    //Establecemos la fecha de Fin, a partir del la fecha del último plazo.
    if(calendarFechaEfectiva != null && factura.getTipoPago() != TipoPago.MENSUAL){
      factura.setFechaFinPagos(calendarFechaEfectiva.getTime());
    }
  }
  
  /**
   * Establecer presupuesto.
   * 
   * @param factura
   *           factura
   * @param fechaEfectivaPrimerPago
   *           fecha efectiva primer pago
   * @param fechaFinMensual
   *           fecha fin mensual
   * @param numeroPlazos
   *           numero plazos
   * @param tipoCuantiaSubproyectos
   *           tipo cuantia subproyectos
   * @return double
   */
  public Double establecerPresupuesto(Factura factura, Date fechaEfectivaPrimerPago, Date fechaFinMensual, int numeroPlazos, TipoCuantiaSubproyectos tipoCuantiaSubproyectos){
    int numeroPagos = 0;
    Double cuantiaPago = null;
    if(factura.getTipoPago() == TipoPago.UNICO){
      numeroPagos = 1;
    }else if(factura.getTipoPago() == TipoPago.MENSUAL){
      if(fechaEfectivaPrimerPago.after(fechaFinMensual)){
        //this.addErrorMessage("La fecha de inicio es posterior a la fecha de fin.");
      }else{
        Calendar calendarFechaEfectivaPrimerPago = new GregorianCalendar();
        calendarFechaEfectivaPrimerPago.setTime(fechaEfectivaPrimerPago);
        Calendar calendarFechaFinMensual = new GregorianCalendar();
        calendarFechaFinMensual.setTime(fechaFinMensual);
        
        int numeroPagosMensual = 0;
        for(; calendarFechaEfectivaPrimerPago.compareTo(calendarFechaFinMensual) <=0; numeroPagosMensual++){
          calendarFechaEfectivaPrimerPago.add(Calendar.MONTH, 1);
        }
        
        numeroPagos = numeroPagosMensual;
      }
    }else if(factura.getTipoPago() == TipoPago.PLAZOS_FIJOS){
      numeroPagos = numeroPlazos;
    }
    
    if(factura.getCuantiaPresupuesto() != null && numeroPagos != 0){
      if(factura.getTipoPago() == TipoPago.MENSUAL){
        cuantiaPago = factura.getCuantiaPresupuesto();
      }else{//if(factura.getTipoPago() == TipoPago.MENSUAL || factura.getTipoPago() == TipoPago.UNICO)
        cuantiaPago = factura.getCuantiaPresupuesto() / numeroPagos;
      }
    }
    
    List<Pago> pagos = factura.getPagos();
    for(Pago pago: pagos){
      pago.setCuantia(cuantiaPago);
      
      Double cuantiaSubproyecto = null, cuantiaPorcentualProyecto = null;
      if(pago.getCuantia() != null && this.getNumeroCentrosPagosAbiertos(pago) > 0){
        cuantiaSubproyecto = pago.getCuantia()/this.getNumeroCentrosPagosAbiertos(pago);
        cuantiaPorcentualProyecto = new Double(100)/this.getNumeroCentrosPagosAbiertos(pago);
      }
      
      List<Pago> pagoSubproyectos = pago.getPagoSubproyectos();
      for(Pago pagoSubproyecto : pagoSubproyectos){
        if(!pagoSubproyecto.isCentroPagoAbierto()){
          pagoSubproyecto.setCuantia(null);
          pagoSubproyecto.setCuantiaPorcentual(null);
        }else{
          if(tipoCuantiaSubproyectos == TipoCuantiaSubproyectos.VALOR_FIJO){
            pagoSubproyecto.setCuantia(cuantiaSubproyecto);
            pagoSubproyecto.setCuantiaPorcentual(null);
          }else{//if(tipoCuantiaSubproyectos == TipoCuantiaSubproyectos.PORCENTUAL)
            pagoSubproyecto.setCuantia(null);
            pagoSubproyecto.setCuantiaPorcentual(cuantiaPorcentualProyecto);
          }
        }
      }
    }
    
    return cuantiaPago;
    
  }
  
  /**
   * Save tipo cuantia subproyectos y fecha fin.
   * 
   * @param factura
   *           factura
   * @param tipoCuantiaSubproyectos
   *           tipo cuantia subproyectos
   */
  public void saveTipoCuantiaSubproyectosYFechaFin(Factura factura, TipoCuantiaSubproyectos tipoCuantiaSubproyectos){
    for(Pago pago: factura.getPagos()){
      pago.setTipoCuantiaSubproyectos(tipoCuantiaSubproyectos);
    }
    
    //Si el pago es único establecemos la fecha de fin a la misma que la de inicio.
    if(factura.getTipoPago() == TipoPago.UNICO){
      factura.setFechaFinPagos(factura.getFechaInicioPagos());
    }
  }
  
  /**
   * Comprobar total subcentros.
   * 
   * @param factura
   *           factura
   * @param tipoCuantiaSubproyectos
   *           tipo cuantia subproyectos
   * @return true, if successful
   */
  public boolean comprobarTotalSubcentros(Factura factura, TipoCuantiaSubproyectos tipoCuantiaSubproyectos){
    boolean resultado = true;
    int i = 0;
    
    List<Pago> pagos = factura.getPagos();
    
    for(Pago pago : pagos){
      Double cuantia  = new Double(0), cuantiaPorcentual = new Double(0);
      i++;
      boolean iteracionCorrecta = false;
      
      List<Pago> pagosSubproyectos = pago.getPagoSubproyectos();
      for(Pago pagoSubproyecto : pagosSubproyectos){
        if(tipoCuantiaSubproyectos == TipoCuantiaSubproyectos.VALOR_FIJO){
          if(pagoSubproyecto.getCuantia() != null){
            cuantia += pagoSubproyecto.getCuantia();
          }
        }else{//if(tipoCuantiaSubproyectos == TipoCuantiaSubproyectos.PORCENTUAL)
          if(pagoSubproyecto.getCuantiaPorcentual() != null){
            cuantiaPorcentual += pagoSubproyecto.getCuantiaPorcentual();
          }
        }
      }
      
      if(pago.getCuantia() != null){
        if(tipoCuantiaSubproyectos == TipoCuantiaSubproyectos.VALOR_FIJO){
          iteracionCorrecta = this.redondearDouble(pago.getCuantia(), 2) == this.redondearDouble(cuantia, 2);
        }else{//if(tipoCuantiaSubproyectos == TipoCuantiaSubproyectos.PORCENTUAL)
          iteracionCorrecta = this.redondearDouble(100, 2) == this.redondearDouble(cuantiaPorcentual, 2);
        }
      }
      
      if(iteracionCorrecta == false){
        NumberFormat numberInstance = NumberFormat.getNumberInstance();
        numberInstance.setMaximumFractionDigits(2);
        numberInstance.setMinimumFractionDigits(0);
        
        if(tipoCuantiaSubproyectos == TipoCuantiaSubproyectos.VALOR_FIJO){
          ArrayList<String> list = new ArrayList<String>(2);
          list.add(Integer.toString(i));
          list.add(numberInstance.format(this.redondearDouble(cuantia, 2)));
          list.add(numberInstance.format(this.redondearDouble(pago.getCuantia(), 2)));
          this.addErrorMessage("La suma de los valores asignados a los subproyectos del pago {0}: {1} €, no corresponde con el total del pago: {2} €.", list.toArray());
        }else{//if(tipoCuantiaSubproyectos == TipoCuantiaSubproyectos.PORCENTUAL)
          ArrayList<String> list = new ArrayList<String>(1);
          list.add(Integer.toString(i));
          list.add(numberInstance.format(this.redondearDouble(cuantiaPorcentual, 2)));
          this.addErrorMessage("La suma de los porcentajes a los subproyectos del pago {0}: {1} %, no suma un total del 100 %.", list.toArray());
        }
      }
      
      if(iteracionCorrecta == false){
        resultado = false;
      }
    }
    
    return resultado;
  }
  
  /**
   * Redondear double.
   * 
   * @param valor
   *           valor
   * @param numeroDecimales
   *           numero decimales
   * @return double
   */
  public double redondearDouble(double valor, int numeroDecimales){
    
    Double factor = Math.pow(10,numeroDecimales);

    valor = valor * factor;

    double tmp = Math.round(valor);

    return (double)tmp / factor;
  }
  
  /**
   * Devuelve max plazos.
   * 
   * @param fechaInicio
   *           fecha inicio
   * @return max plazos
   */
  public int getMaxPlazos(Date fechaInicio){
    Calendar calendarInicio = new GregorianCalendar();
    int maxPlazos = 12;
    
    if(fechaInicio != null){
      calendarInicio.setTime(fechaInicio);
      maxPlazos = Calendar.DECEMBER - calendarInicio.get(Calendar.MONTH) + 1; 
    }
    
    return maxPlazos;
  }
  
  /**
   * Devuelve fecha efectiva inicial.
   * 
   * @param anyoProyecto
   *           anyo proyecto
   * @return fecha efectiva inicial
   */
  public Date getFechaEfectivaInicial(int anyoProyecto){
    //Inicializamos la fecha efectiva de la factura a la fecha actual y el año del proyecto al que introducimos la factura.
    Calendar fechaEfectivaInicial = new GregorianCalendar();
    fechaEfectivaInicial.set(Calendar.MILLISECOND, 0);
    fechaEfectivaInicial.set(Calendar.SECOND, 0);
    fechaEfectivaInicial.set(Calendar.MINUTE, 0);
    fechaEfectivaInicial.set(Calendar.HOUR_OF_DAY, 0);
    fechaEfectivaInicial.set(Calendar.YEAR, anyoProyecto);
    Date fechaEfectiva = fechaEfectivaInicial.getTime();
    
    return fechaEfectiva;
  }
  
  /**
   * Devuelve max numero plazos.
   * 
   * @param maxPlazos
   *           max plazos
   * @param numeroPlazos
   *           numero plazos
   * @return max numero plazos
   */
  public int getMaxNumeroPlazos(int maxPlazos, int numeroPlazos){
    if(numeroPlazos > maxPlazos)
      return maxPlazos;
    else
      return numeroPlazos;
  }
  
  /**
   * Devuelve cuantia pago inicial.
   * 
   * @param factura
   *           factura
   * @return cuantia pago inicial
   */
  public Double getCuantiaPagoInicial(Factura factura){
    Double cuantiaPlazo = null;
    if(factura.getCuantiaPresupuesto() != null){
      if(factura.getTipoPago() != TipoPago.MENSUAL){
        cuantiaPlazo = factura.getCuantiaPresupuesto()/factura.getPagos().size();
      }else{
        cuantiaPlazo = factura.getCuantiaPresupuesto();
      }
    }
    
    return cuantiaPlazo;
  }
  
  /**
   * Devuelve numero centros pagos abiertos.
   * 
   * @param pagoGeneral
   *           pago general
   * @return numero centros pagos abiertos
   */
  public int getNumeroCentrosPagosAbiertos(Pago pagoGeneral){
    int numeroCentrosPagosAbiertos = 0;
    Date fechaEfectiva = pagoGeneral.getFechaEfectiva();
    
    for(Pago pagoSubproyecto : pagoGeneral.getPagoSubproyectos()){
      Centro centro = pagoSubproyecto.getCentro();
      
      //Si el centro no tiene la fecha de apertura especificada, consideraremos que ha estado siempre abierto.
      if(centro.getFechaApertura() == null){
       numeroCentrosPagosAbiertos++;
      }else{
        //Si el centro ha abierto antes o a la vez que la fecha del pago. 
        if(centro.getFechaApertura().before(fechaEfectiva) || centro.getFechaApertura().equals(fechaEfectiva)){
          numeroCentrosPagosAbiertos++;
        }
      }
    }
    
    return numeroCentrosPagosAbiertos;
  }
  
}
