package es.csa.fdt.cyldigital.financiero.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.List;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.financiero.entity.Contenedor;
import es.csa.fdt.cyldigital.financiero.entity.ItemContenedor;
import es.csa.fdt.cyldigital.financiero.entity.Prevision;
import es.csa.fdt.cyldigital.financiero.entity.Seccion;

/**
 * Clase ItemContenedorServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class ItemContenedorServiceImpl extends AbstractServiceImpl<ItemContenedor, Long> implements ItemContenedorService{

  private static final long serialVersionUID = 9023475842307950874L;
  
  /**
   * Crea item contenedor.
   * 
   * @param itemContenedor
   *           item contenedor
   * @param contenedorId
   *           contenedor id
   */
  public void createItemContenedor(ItemContenedor itemContenedor, Long contenedorId)
  {
    Contenedor contenedor = find(Contenedor.class, contenedorId);
    
    itemContenedor.setProyectoRaiz(contenedor.getProyectoRaiz());    
    itemContenedor.setContenedorPadre(contenedor);
    
//    Seccion seccionRaiz = obtenerSeccionRaiz(contenedor);
    Seccion seccionRaiz = contenedor.getSeccionRaiz();
    itemContenedor.setSeccionRaiz(seccionRaiz);
    
    contenedor.getElementos().add(itemContenedor);
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.core.service.AbstractServiceImpl#persist(java.lang.Object)
   */
  @Override
  public <T> void persist(T entity) {
    ItemContenedor itemContenedor = (ItemContenedor)entity;
    
    Contenedor contenedor = itemContenedor.getContenedorPadre();
    List<Contenedor> contenedoresSubproyectos = contenedor.getContenedoresSubproyectos();
    
    for(Contenedor contenedorSubproyecto : contenedoresSubproyectos)
    {
      ItemContenedor itemContenedorSubproyecto = new ItemContenedor(itemContenedor.getNombre(), contenedorSubproyecto.getProyectoRaiz(), contenedorSubproyecto);
      //itemContenedorSubproyecto.setSecuencia(itemContenedorSubproyecto.getSecuencia());
      
//      Seccion seccionRaiz = obtenerSeccionRaiz(contenedorSubproyecto);
      Seccion seccionRaiz = contenedorSubproyecto.getSeccionRaiz();
      itemContenedorSubproyecto.setSeccionRaiz(seccionRaiz);
      
      //super.persist(itemContenedorSubproyecto);
      
      itemContenedorSubproyecto.setContenedorProyectoGeneral(itemContenedor);
      
      contenedorSubproyecto.getElementos().add(itemContenedorSubproyecto);
    }
    
    super.persist(itemContenedor);
  }
  
  /**
   * Actualizar subproyectos.
   * 
   * @param itemContenedor
   *           item contenedor
   */
  public void actualizarSubproyectos(ItemContenedor itemContenedor){
    List<Contenedor> itemsSubproyectos = itemContenedor.getContenedoresSubproyectos();
    
    for(Contenedor itemSubproyecto : itemsSubproyectos)
    {
      itemSubproyecto.setNombre(itemContenedor.getNombre());
    }
  }
  
  /**
   * Elimina item contenedor.
   * 
   * @param idItemContenedor
   *           id item contenedor
   */
  public void removeItemContenedor(Long idItemContenedor)
  {
    ItemContenedor itemContenedor = find(idItemContenedor);
    
    List<Contenedor> contenedoresSubproyectos = itemContenedor.getContenedoresSubproyectos();
    
    for(Contenedor contenedorSubproyecto : contenedoresSubproyectos)
    {
      contenedorSubproyecto.getContenedorPadre().getElementos().remove(contenedorSubproyecto);
      contenedorSubproyecto.setContenedorPadre(null);
      
      Prevision previsionSubproyecto = contenedorSubproyecto.getPrevision();
      
//      previsionSubproyecto.setContenedor(null);
      previsionSubproyecto.getPrevisionPadre().getPrevisionesDescendientes().remove(previsionSubproyecto);
      previsionSubproyecto.setPrevisionPadre(null);
     
//      remove(contenedorSubproyecto);
    }
    
    itemContenedor.getContenedorPadre().getElementos().remove(itemContenedor);
    itemContenedor.setContenedorPadre(null);
    
    Prevision prevision = itemContenedor.getPrevision();
    
    prevision.setContenedor(null);
    prevision.getPrevisionPadre().getPrevisionesDescendientes().remove(prevision);
    prevision.setPrevisionPadre(null);
   
    remove(itemContenedor);
  } 
  
  /**
   * Subir item contenedor.
   * 
   * @param idItemContenedor
   *           id item contenedor
   */
  public void subirItemContenedor(Long idItemContenedor)
  {
    ItemContenedor itemContenedor = find(idItemContenedor);
    
    int posicion = itemContenedor.getSecuencia();
    
    itemContenedor.getContenedorPadre().getElementos().remove(itemContenedor);
    itemContenedor.getContenedorPadre().getElementos().add(posicion - 1, itemContenedor);
    
    List<Contenedor> contenedoresSubproyectos = itemContenedor.getContenedoresSubproyectos();
    
    for(Contenedor contenedorSubproyecto : contenedoresSubproyectos)
    {
      contenedorSubproyecto.getContenedorPadre().getElementos().remove(contenedorSubproyecto);
      contenedorSubproyecto.getContenedorPadre().getElementos().add(posicion - 1, contenedorSubproyecto);
    }
  }
  
  /**
   * Bajar item contenedor.
   * 
   * @param idItemContenedor
   *           id item contenedor
   */
  public void bajarItemContenedor(Long idItemContenedor)
  {
    ItemContenedor itemContenedor = find(idItemContenedor);
    
    int posicion = itemContenedor.getSecuencia();
    
    itemContenedor.getContenedorPadre().getElementos().remove(itemContenedor);
    itemContenedor.getContenedorPadre().getElementos().add(posicion + 1, itemContenedor);
    
    List<Contenedor> contenedoresSubproyectos = itemContenedor.getContenedoresSubproyectos();
    
    for(Contenedor contenedorSubproyecto : contenedoresSubproyectos)
    {
      contenedorSubproyecto.getContenedorPadre().getElementos().remove(contenedorSubproyecto);
      contenedorSubproyecto.getContenedorPadre().getElementos().add(posicion + 1, contenedorSubproyecto);
    }
  }
  
//  public static Seccion obtenerSeccionRaiz(Contenedor contenedorPadre){
//    Seccion seccionRaiz;
//    
//    if(contenedorPadre.getTipoContenedor() == Contenedor.TipoContenedor.ITEM){
//      ItemContenedor itemContenedorPadre = (ItemContenedor)contenedorPadre;
//      seccionRaiz = itemContenedorPadre.getSeccionRaiz();
//    }else{//if(contenedorPadre.getTipoContenedor() == Contenedor.TipoContenedor.SECCION)
//      seccionRaiz = (Seccion)contenedorPadre;
//    }
//    
//    return seccionRaiz;
//  }

}
