package es.csa.fdt.cyldigital.financiero.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.Date;
import java.util.List;

import javax.persistence.Query;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.financiero.entity.DocumentoMovimiento;
import es.csa.fdt.cyldigital.financiero.entity.Movimiento;
import es.csa.fdt.cyldigital.financiero.entity.Movimiento.TipoOperacion;
import es.csa.fdt.cyldigital.notificacion.NotificationService;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado.Puesto;

/**
 * Clase MovimientoServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class MovimientoServiceImpl extends AbstractServiceImpl<Movimiento, Long> implements MovimientoService {

  private static final long serialVersionUID = -5575228180361917674L;

  /**
   * Construye un nuevo objeto de la clase
   */
  public MovimientoServiceImpl() {
    super();
  }

  /**
   * Inicializa movimiento.
   * 
   * @param m
   *           m
   * @param tipomovimiento
   *           tipomovimiento
   * @param idCentro
   *           id centro
   * @param u
   *           u
   */
  public void inicializaMovimiento(Movimiento m, String tipomovimiento, Long idCentro, UsuarioLdap u) {
    Centro centro = find(Centro.class, idCentro);
    if (centro != null) {
      m.setFechaOperacion(new Date());
      m.setFechaFactura(new Date());
      m.setCaja(centro.getCaja());
      m.setUsuario(u.getUser());

      if (tipomovimiento.equals("abonocaja")) m.setTipo(TipoOperacion.ABONOCAJA);
      else m.setTipo(TipoOperacion.ANADIRFACTURA);
    }
  }

  /**
   * Persist movimiento.
   * 
   * @param m
   *           m
   */
  @SuppressWarnings("unchecked")
  public void persistMovimiento(Movimiento m) {
    if (m.getCaja() != null) {
      m.setImporteAnterior(m.getCaja().getValorCaja());

      if (m.getTipo() == TipoOperacion.ABONOCAJA)
        m.getCaja().setValorCaja(m.getCaja().getValorCaja() + m.getImporte());
      if (m.getTipo() == TipoOperacion.ANADIRFACTURA)
        m.getCaja().setValorCaja(m.getCaja().getValorCaja() - m.getImporte());
      m.setImportePosterior(m.getCaja().getValorCaja());
      persist(m);
      
      if(m.getCaja().getValorCaja() <= 0){
        //Lanzamos una notificación de caja vacía al/a los responsable/s del centro.
        String consulta = "SELECT DISTINCT e FROM " + Empleado.class.getName() + " e WHERE centro = :valor1 AND puesto = :valor2";
        Query query = getEntityManager().createQuery(consulta);
        query.setParameter("valor1", m.getCaja().getCentro());
        query.setParameter("valor2", Puesto.AREA_MANAGER);
        List<Usuario> destinatarios = query.getResultList();
        
        //En el caso de que no haya ningún responsable de centro para dicho centro.
        if(!destinatarios.isEmpty()){
          NotificationService n = new NotificationService();
          n.notificarCajaVacia(this.getEntityManager(), destinatarios, m.getCaja());
        }
      }
    }
  }
  
  /**
   * Aniadir documento movimiento.
   * 
   * @param movimiento
   *           movimiento
   * @param documento
   *           documento
   */
  public void aniadirDocumentoMovimiento(Movimiento movimiento, DocumentoMovimiento documento){
    movimiento.setDocumentoMovimiento(documento);
    documento.setMovimiento(movimiento);
  }
  
  /**
   * Desasociar.
   * 
   * @param movimiento
   *           movimiento
   * @param documento
   *           documento
   */
  public void desasociar(Movimiento movimiento, DocumentoMovimiento documento){
    if(movimiento.getDocumentoMovimiento() == documento){
      movimiento.setDocumentoMovimiento(null);
      documento.setMovimiento(null);
    }
  }

}
