package es.csa.fdt.cyldigital.financiero.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.jpaql.BrowserJPAQL;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.commons.persistence.browser.jpaql.FilterJPAQLByQuery;
import es.csa.fdt.cyldigital.financiero.entity.Movimiento;

/**
 * Servicio del Browser de Movimientos.
 * 
 * @author csa
 */
@Service
@Repository
public class MovimientosBrowserImpl extends BrowserJPAQL<Movimiento> implements
    MovimientosBrowser<CriterionJPAQL, String> {

  private static final long serialVersionUID = -7227366052245717441L;

  /**
   * Construye un nuevo objeto de la clase
   */
  public MovimientosBrowserImpl() {
    super();
    getSort().setSortProperty("fechaOperacion");
    getSort().setSortAsc(false);
  }

  /**
   * Asigna filter by caja.
   * 
   * @param idCaja
   *          filter by caja
   */
  public void setFilterByCaja(Long idCaja) {
    removeFilterCaja();
    filterByCaja(idCaja);
  }

private void removeFilterCaja() {
    int i = -1;
    for (Filter<CriterionJPAQL, Movimiento> f : getFilters())
      if (f.getCriterion().getQuery().contains("idCaja")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
  }

/**
 * Filter by caja.
 * 
 * @param idCaja
 *           id caja
 */
public void filterByCaja(Long idCaja) {
    String query = "";

    boolean flag = false;
    FilterJPAQLByQuery<Movimiento> filter = new FilterJPAQLByQuery<Movimiento>();

    query += " idCaja = ? ";
    filter.getCriterion().addParameter(idCaja);

    for (Filter<CriterionJPAQL, Movimiento> f : getFilters())
      if (f.getCriterion().getQuery().contains("idCaja")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }
}
