package es.csa.fdt.cyldigital.financiero.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.Date;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.financiero.entity.DocumentoFactura;
import es.csa.fdt.cyldigital.financiero.entity.EstadoFactura;
import es.csa.fdt.cyldigital.financiero.entity.EstadoPago;
import es.csa.fdt.cyldigital.financiero.entity.Factura;
import es.csa.fdt.cyldigital.financiero.entity.Pago;
import es.csa.fdt.cyldigital.financiero.entity.EstadoFactura.TipoEstadoFactura;
import es.csa.fdt.cyldigital.financiero.entity.EstadoPago.TipoEstadoPago;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;

/**
 * Clase PagoServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class PagoServiceImpl extends AbstractServiceImpl<Pago, Long> implements PagoService {

  private static final long serialVersionUID = 5794230759423075894L;
  
//  public void persist(Pago pagoProyectoGeneral, int numeroMensualidades) {
//    Factura factura = pagoProyectoGeneral.getFactura();
//    List<Pago> pagosSubproyectos = pagoProyectoGeneral.getPagoSubproyectos();
//    
//    for(Pago pagoSubproyecto : pagosSubproyectos){
//      pagoSubproyecto.setFechaEfectiva(pagoProyectoGeneral.getFechaEfectiva());
//    }
//    
//    //Persistimos el pago general y el de los subproyectos, en todos los caso, sea pago único o múltiple.
//    super.persist(pagoProyectoGeneral);
//    
//    if(factura.getTipoPago() == TipoPago.PLAZOS_FIJOS){
//      Pago pagoProyectoGeneralAux , pagoSubproyectoAux;
//      for(int i = 1; i < numeroMensualidades; i++){
//        //Aumentamos para cada iteración, un mes en la fecha efectiva de pago.
//        //int numeroMesesAdicionales = i;
//        Calendar calendarFechaEfectiva = new GregorianCalendar();
//        calendarFechaEfectiva.setTime(pagoProyectoGeneral.getFechaEfectiva());
//        calendarFechaEfectiva.add(Calendar.MONTH, i);
//        
//        pagoProyectoGeneralAux = pagoProyectoGeneral.clonePago(factura, null);
//        pagoProyectoGeneralAux.setFechaEfectiva(calendarFechaEfectiva.getTime());
//        
//        super.persist(pagoProyectoGeneralAux);
//        
//        for(Pago pagoSubproyecto : pagosSubproyectos){
//          pagoSubproyectoAux = pagoSubproyecto.clonePago(null, pagoProyectoGeneralAux);
//          pagoSubproyectoAux.setFechaEfectiva(calendarFechaEfectiva.getTime());
//          super.persist(pagoSubproyectoAux);
//        }
//      }
//    }
//  }  

  /**
 * Asigna nuevo estado pago.
 * 
 * @param tipoEstadoFacturaAntiguo
 *           tipo estado factura antiguo
 * @param estadoPago
 *           estado pago
 * @param pago
 *           pago
 * @param usuario
 *           usuario
 */
public void setNuevoEstadoPago(TipoEstadoPago tipoEstadoFacturaAntiguo, EstadoPago estadoPago, Pago pago, Usuario usuario) {    
    if (tipoEstadoFacturaAntiguo != estadoPago.getTipoEstadoPago()) {
      estadoPago.setFecha(new Date());
      estadoPago.setPago(pago);
      estadoPago.setUsuario(usuario);

      pago.getEstadosPago().add(estadoPago);
      pago.setEstadoActual(estadoPago);
      super.persist(estadoPago);
      
      if(isPagosFacturaSuperadoTipoEstado(pago.getFactura(), pago.getEstadoActual().getTipoEstadoPago())){
        setNuevoEstadoFactura(pago.getFactura(), pago.getEstadoActual().getTipoEstadoPago(), estadoPago.getFecha(), usuario);
      }
    }
  }
  
  /**
   * Checks if is pagos factura superado tipo estado.
   * 
   * @param factura
   *           factura
   * @param tipoEstadoPago
   *           tipo estado pago
   * @return true, if is pagos factura superado tipo estado
   */
  public boolean isPagosFacturaSuperadoTipoEstado(Factura factura, TipoEstadoPago tipoEstadoPago){
    boolean result = true;
    
    for(Pago pago : factura.getPagos()){
      if(pago.getEstadoActual().getTipoEstadoPago().compareTo(tipoEstadoPago) < 0){
        result = false;
        break;
      }
    }
    
    return result;
  }
  
  /**
   * Asigna nuevo estado factura.
   * 
   * @param factura
   *           factura
   * @param tipoEstadoPago
   *           tipo estado pago
   * @param fecha
   *           fecha
   * @param usuario
   *           usuario
   */
  public void setNuevoEstadoFactura(Factura factura, TipoEstadoPago tipoEstadoPago, Date fecha, Usuario usuario) {
    TipoEstadoFactura tipoEstadoFactura = null;
    String observaciones = null;

    if(tipoEstadoPago == TipoEstadoPago.PAGADO){
      tipoEstadoFactura = TipoEstadoFactura.PAGADA;
      observaciones = "Todos los pagos pagados";
    }
    else if(tipoEstadoPago == TipoEstadoPago.REVISADO){
      tipoEstadoFactura = TipoEstadoFactura.REVISADA;
      observaciones = "Todos los pagos revisados";
    }
    
    EstadoFactura estadoFactura = new EstadoFactura(tipoEstadoFactura, factura, fecha, usuario, observaciones);

    factura.getEstadosFactura().add(estadoFactura);
    factura.setEstadoActual(estadoFactura);
    super.persist(estadoFactura);
  }
  
  /**
   * Find estado pago.
   * 
   * @param idEstadoPago
   *           id estado pago
   * @return estado pago
   */
  public EstadoPago findEstadoPago(Long idEstadoPago) {
    return (EstadoPago) this.find(EstadoPago.class, idEstadoPago);
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.core.service.AbstractServiceImpl#clear()
   */
  public void clear(){
    this.getEntityManager().clear();
  }
  
  /**
   * Desasociar.
   * 
   * @param pago
   *           pago
   * @param documento
   *           documento
   */
  public void desasociar(Pago pago, DocumentoFactura documento){
    if(pago.getDocumentoFactura() == documento){
      pago.setDocumentoFactura(null);
    }
  }
  
  /**
   * Aniadir documento pago.
   * 
   * @param pago
   *           pago
   * @param documento
   *           documento
   */
  public void aniadirDocumentoPago(Pago pago, DocumentoFactura documento){
    pago.setDocumentoFactura(documento);
    documento.setPago(pago);
  }
  
  /**
   * Comprobar documentacion.
   * 
   * @param pago
   *           pago
   * @param estadoPago
   *           estado pago
   * @return true, if successful
   */
  public boolean comprobarDocumentacion(Pago pago, EstadoPago estadoPago){
    boolean result = false;
    if(estadoPago.getTipoEstadoPago() != TipoEstadoPago.PAGADO){
      result = true;
    }else{
      if(pago.getDocumentoFactura() != null){
        result = true;
      }
    }
    return result;
  }
}
