package es.csa.fdt.cyldigital.financiero.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.Collection;
import java.util.List;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.financiero.entity.Contenedor;
import es.csa.fdt.cyldigital.financiero.entity.Prevision;
import es.csa.fdt.cyldigital.financiero.entity.Prevision.Mes;

/**
 * Clase PrevisionEvolutivaServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class PrevisionEvolutivaServiceImpl extends AbstractServiceImpl<Prevision, Long> implements PrevisionEvolutivaService{

  private static final long serialVersionUID = 2745902375902347589L;
  
  private PrevisionServiceImpl previsionService;
  
  /**
   * Construye un nuevo objeto de la clase
   */
  public PrevisionEvolutivaServiceImpl(){
    super();
    this.previsionService = new PrevisionServiceImpl();
  }
  
  /**
   * Asigna nodo ascendientes no editados subproyectos y proyecto general.
   * 
   * @param prevision
   *          nodo ascendientes no editados subproyectos y proyecto general
   */
  public void setNodoAscendientesNoEditadosSubproyectosYProyectoGeneral(Prevision prevision)
  {
    prevision.setCuantiaEvolutiva(new Double(0));
    prevision.setCuantiaPorcentualEvolutiva(new Double(0));
    this.setAscendientesNoEditadosSubproyectosYProyectoGeneral(prevision);
  }
  
  /**
   * Asigna ascendientes no editados subproyectos y proyecto general.
   * 
   * @param prevision
   *          ascendientes no editados subproyectos y proyecto general
   */
  public void setAscendientesNoEditadosSubproyectosYProyectoGeneral(Prevision prevision)
  {
    if(prevision.getCuantiaEvolutiva() == 0)
      prevision.setCuantiaPorcentualEvolutiva(new Double(0));
    else
      prevision.setCuantiaPorcentualEvolutiva(new Double(100));
    
    if(prevision.getTieneDescendientesAsignadosEvolutiva() == false){ //Para el item o sección asignada y/o cuyo valor varía
    //Asignamos la parte proporcional al tiempo que lleva abierto dicho centro para ese año, para los subproyectos existentes.
      List<Contenedor> contenedoresSubproyectos = prevision.getContenedor().getContenedoresSubproyectos();
      if(contenedoresSubproyectos != null && contenedoresSubproyectos.size() > 0){ //Si editamos desde el proyecto general
        for(Contenedor contenedorSubproyecto : contenedoresSubproyectos){
          contenedorSubproyecto.getPrevision().setAsignadoEvolutiva(prevision.isAsignadoEvolutiva());
          contenedorSubproyecto.getPrevision().setCuantiaEvolutiva(this.getCuantiaProporcionalCentro(contenedorSubproyecto.getPrevision()));
          contenedorSubproyecto.getPrevision().setCuantiaPorcentualEvolutiva(this.getCuantiaPorcentualProporcionalCentro(contenedorSubproyecto.getPrevision()));
          for(Mes mes: Mes.values()){
            Double valorFijo = null;
            if(previsionService.isCentroAbierto(contenedorSubproyecto.getPrevision(), mes)){
              valorFijo = contenedorSubproyecto.getPrevision().getCuantiaEvolutiva()/previsionService.getNumeroMesesAbiertoCentro(contenedorSubproyecto.getPrevision());
            }
            contenedorSubproyecto.getPrevision().getValoresFijosEvolutivos().put(mes, valorFijo);
          }
        }
      }else{ //Si editamos desde un proyecto en particular
        for(Mes mes: Mes.values()){
          Double valorFijo = null;
          if(previsionService.isCentroAbierto(prevision, mes)){
            valorFijo = prevision.getCuantiaEvolutiva()/previsionService.getNumeroMesesAbiertoCentro(prevision);
          }
          prevision.getValoresFijosEvolutivos().put(mes, valorFijo);
        }
      }
    }else{ //Para items o secciones que tienen hijos asignados (padres de los items asignados y/o cuyo valor ha variado)
      this.sumatorioCuantiasPrevisionPadre(prevision);
    }
    
    if(prevision.getContenedor().getContenedorProyectoGeneral() != null){
      Prevision previsionProyectoGeneral = prevision.getContenedor().getContenedorProyectoGeneral().getPrevision();
      previsionProyectoGeneral.setCuantiaEvolutiva(this.getSumatorioCuantiaSubproyectos(previsionProyectoGeneral));
      if(previsionProyectoGeneral.getCuantiaEvolutiva() == 0)
        previsionProyectoGeneral.setCuantiaPorcentualEvolutiva(new Double(0));
      else
        previsionProyectoGeneral.setCuantiaPorcentualEvolutiva(new Double(100));
    }
    
    if(prevision.getPrevisionPadre() != null)
    {
      prevision.getPrevisionPadre().setAsignadoEvolutiva(false);
      prevision.getPrevisionPadre().setCuantiaEvolutiva(prevision.getPrevisionPadre().getCuantiaEvolutivaEfectiva());
      
      this.setAscendientesNoEditadosSubproyectosYProyectoGeneral(prevision.getPrevisionPadre());
    }
  }
  
  /**
   * Sumatorio cuantias prevision padre.
   * 
   * @param prevision
   *           prevision
   */
  public void sumatorioCuantiasPrevisionPadre(Prevision prevision){
    List<Contenedor> contenedoresSubproyectos = prevision.getContenedor().getContenedoresSubproyectos();
    
    for(Contenedor contenedorSubproyecto : contenedoresSubproyectos){
      Double sumatorioSubproyecto = new Double(0);
      Prevision previsionSubproyecto = contenedorSubproyecto.getPrevision();
      
      for(Mes mes : Mes.values()){
        Double sumatorioSubproyectoMes = null;
    
        List<Prevision> previsionesSubproyectoDescendientes = previsionSubproyecto.getPrevisionesDescendientes();
      
        for(Prevision previsionSubproyectoDescendiente : previsionesSubproyectoDescendientes){
          
          if(previsionSubproyectoDescendiente.getValoresFijosEvolutivos().get(mes) != null){
            if(sumatorioSubproyectoMes == null)
              sumatorioSubproyectoMes = new Double(0);
            sumatorioSubproyectoMes += previsionSubproyectoDescendiente.getValoresFijosEvolutivos().get(mes);
          }
        }
        
        previsionSubproyecto.getValoresFijosEvolutivos().put(mes, sumatorioSubproyectoMes);
        if(sumatorioSubproyectoMes != null)
        {
          sumatorioSubproyecto += sumatorioSubproyectoMes;
        }
      }
      
      previsionSubproyecto.setCuantiaEvolutiva(sumatorioSubproyecto);
      previsionSubproyecto.setCuantiaPorcentualEvolutiva(previsionSubproyecto.getCuantiaEvolutiva() * new Double(100) / prevision.getCuantiaEvolutiva());
    }
  }
  
  
  /**
   * Devuelve cuantia proporcional centro.
   * 
   * @param prevision
   *           prevision
   * @return cuantia proporcional centro
   */
  public Double getCuantiaProporcionalCentro(Prevision prevision)
  {
    Double result = new Double(0);
    
    Double coeficiente = previsionService.getNumeroMesesAbiertoCentro(prevision) / previsionService.getNumeroMesesAbiertosCentros(prevision);
      
    result = prevision.getContenedor().getContenedorProyectoGeneral().getPrevision().getCuantiaEvolutivaEfectiva() * coeficiente;
    
    return result;
  }
  
  /**
   * Devuelve cuantia porcentual proporcional centro.
   * 
   * @param prevision
   *           prevision
   * @return cuantia porcentual proporcional centro
   */
  public Double getCuantiaPorcentualProporcionalCentro(Prevision prevision)
  {
    Double result = new Double(0);
    
    if(prevision.getCuantiaEvolutiva() != 0)
      result = new Double(100);
    
    Double coeficiente = previsionService.getNumeroMesesAbiertoCentro(prevision) / previsionService.getNumeroMesesAbiertosCentros(prevision);
      
    result = result * coeficiente;
    
    return result;
  }
  
  /**
   * Devuelve sumatorio cuantia subproyectos.
   * 
   * @param prevision
   *           prevision
   * @return sumatorio cuantia subproyectos
   */
  public Double getSumatorioCuantiaSubproyectos(Prevision prevision){
    Double result = new Double(0);
    
    List<Contenedor> contenedoresSubproyectos = prevision.getContenedor().getContenedoresSubproyectos();
    if(contenedoresSubproyectos != null){
      for(Contenedor contenedorSubproyecto : contenedoresSubproyectos){
        Prevision previsionSubproyecto = contenedorSubproyecto.getPrevision(); 
        result += previsionSubproyecto.getCuantiaEvolutiva();
      }
    }
    
    return result;
  }
  
  /**
   * Asignar previsiones iniciales.
   * 
   * @param prevision
   *           prevision
   */
  public void asignarPrevisionesIniciales(Prevision prevision){
    prevision.setCuantiaEvolutivaInicial(prevision.getCuantiaEvolutiva());
    
    if(prevision.getPrevisionPadre() != null){
      this.asignarPrevisionesIniciales(prevision.getPrevisionPadre());      
    }
  }

  /**
   * Comprobar prevision.
   * 
   * @param prevision
   *           prevision
   * @return true, if successful
   */
  public boolean comprobarPrevision(Prevision prevision){
    boolean result = false;
    
//    if(prevision.getContenedor().getContenedorProyectoGeneral() != null){
//      result = true;        
//    }else{
//      if(!prevision.isAsignado()){
//        result = true;
//      } else {
        if(previsionService.redondearDouble(prevision.getCuantiaEvolutivaInicial(), 2) == previsionService.redondearDouble(prevision.getCuantiaEvolutiva(), 2)){
          result = true;
        }else{
          result = false;
          addErrorMessage("financiero.error.totalAsignado.distinto.totalActual");
        }
//      }
//    }
    
    return result;
  }
  
  /**
   * Recalcular total todos meses.
   * 
   * @param prevision
   *           prevision
   * @param mes
   *           mes
   */
  public void recalcularTotalTodosMeses(Prevision prevision, Mes mes){
    Double total = new Double(0);
    Collection<Double> valores = prevision.getValoresFijosEvolutivos().values();
    
    for(Double valor : valores){
      if(valor != null){
        total += valor;
      }
    }
    
    prevision.setCuantiaEvolutiva(total);
    
    if(prevision.getPrevisionPadre() != null){
      prevision.getPrevisionPadre().getValoresFijosEvolutivos().put(mes, prevision.getPrevisionPadre().getCuantiaEvolutivaEfectivaMensual(mes));
      this.recalcularTotalTodosMeses(prevision.getPrevisionPadre(), mes);
    }
  }
  
  /**
   * Recalcular total todos centros.
   * 
   * @param prevision
   *           prevision
   */
  public void recalcularTotalTodosCentros(Prevision prevision){
    Double total = new Double(0);
    
    List<Contenedor> contenedoresSubproyectos = prevision.getContenedor().getContenedoresSubproyectos();
    if(contenedoresSubproyectos != null && contenedoresSubproyectos.size() > 0){
      for(Contenedor contenedorSubproyecto : contenedoresSubproyectos){
        total += contenedorSubproyecto.getPrevision().getCuantiaEvolutiva();
      }
      
      prevision.setCuantiaEvolutiva(total);
    }
    
    if(prevision.getPrevisionPadre() != null){
      this.recalcularTotalTodosCentros(prevision.getPrevisionPadre());
    }
  }
  
  /**
   * Recalcular valores porcentuales.
   * 
   * @param prevision
   *           prevision
   */
  public void recalcularValoresPorcentuales(Prevision prevision){
    List<Contenedor> contenedoresSubproyectos = prevision.getContenedor().getContenedoresSubproyectos();
    if(contenedoresSubproyectos != null){
      for(Contenedor contenedorSubproyecto : contenedoresSubproyectos){
        Double result = contenedorSubproyecto.getPrevision().getCuantiaEvolutiva() * new Double(100)/prevision.getCuantiaEvolutivaInicial();
        contenedorSubproyecto.getPrevision().setCuantiaPorcentualEvolutiva(result);
      }
    }
    
    if(prevision.getPrevisionPadre() != null)
      this.recalcularValoresPorcentuales(prevision.getPrevisionPadre());
  }
  
  /**
   * Recalcular porcentaje todos centros.
   * 
   * @param prevision
   *           prevision
   */
  public void recalcularPorcentajeTodosCentros(Prevision prevision){
    Double porcentaje = new Double(0);
    
    List<Contenedor> contenedoresSubproyectos = prevision.getContenedor().getContenedoresSubproyectos();
    if(contenedoresSubproyectos != null && contenedoresSubproyectos.size() > 0){
      for(Contenedor contenedorSubproyecto : contenedoresSubproyectos){
        porcentaje += contenedorSubproyecto.getPrevision().getCuantiaPorcentualEvolutiva();
      }
      
      prevision.setCuantiaPorcentualEvolutiva(porcentaje);
    }
  }
  
  /**
   * Repartir total subproyecto.
   * 
   * @param prevision
   *           prevision
   */
  public void repartirTotalSubproyecto(Prevision prevision){
    for(Mes mes: Mes.values()){
      Double valorFijo = null;
      if(previsionService.isCentroAbierto(prevision, mes)){
        valorFijo = prevision.getCuantiaEvolutiva()/previsionService.getNumeroMesesAbiertoCentro(prevision);
      }
      prevision.getValoresFijosEvolutivos().put(mes, valorFijo);
    }
    
    if(prevision.getPrevisionPadre() != null){
      prevision.getPrevisionPadre().setCuantiaEvolutiva(prevision.getPrevisionPadre().getCuantiaEvolutivaEfectiva());
      this.repartirTotalSubproyecto(prevision.getPrevisionPadre());
    }
  }
  
  /**
   * Asignar porcentaje subproyecto a total.
   * 
   * @param previsionSubproyecto
   *           prevision subproyecto
   */
  public void asignarPorcentajeSubproyectoATotal(Prevision previsionSubproyecto){
    Double cuantiaEvolutivaInicial = previsionSubproyecto.getContenedor().getContenedorProyectoGeneral().getPrevision().getCuantiaEvolutivaInicial();
    
    previsionSubproyecto.setCuantiaEvolutiva(cuantiaEvolutivaInicial * previsionSubproyecto.getCuantiaPorcentualEvolutiva() / new Double(100));
    
    if(previsionSubproyecto.getPrevisionPadre() != null){
      this.asignarPorcentajeSubproyectoPadreATotal(previsionSubproyecto.getPrevisionPadre());
    }
  }
  
  /**
   * Asignar porcentaje subproyecto padre a total.
   * 
   * @param previsionSubproyectoPadre
   *           prevision subproyecto padre
   */
  public void asignarPorcentajeSubproyectoPadreATotal(Prevision previsionSubproyectoPadre){
    Double cuantiaTotal = previsionSubproyectoPadre.getContenedor().getContenedorProyectoGeneral().getPrevision().getCuantiaEvolutivaInicial();
    
    previsionSubproyectoPadre.setCuantiaPorcentualEvolutiva(previsionSubproyectoPadre.getCuantiaEvolutivaEfectiva() * new Double(100) / cuantiaTotal);
    
    if(previsionSubproyectoPadre.getPrevisionPadre() != null){
      this.asignarPorcentajeSubproyectoPadreATotal(previsionSubproyectoPadre.getPrevisionPadre());
    }
  }
  
}
