package es.csa.fdt.cyldigital.financiero.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.Calendar;
import java.util.Collection;
import java.util.List;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.financiero.entity.Contenedor;
import es.csa.fdt.cyldigital.financiero.entity.Prevision;
import es.csa.fdt.cyldigital.financiero.entity.Prevision.Mes;

/**
 * Clase PrevisionServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class PrevisionServiceImpl extends AbstractServiceImpl<Prevision, Long> implements PrevisionService{

  private static final long serialVersionUID = 4785092284950234785L;
  
  
  /**
   * Asigna nodo ascendientes no editados subproyectos y proyecto general.
   * 
   * @param prevision
   *          nodo ascendientes no editados subproyectos y proyecto general
   */
  public void setNodoAscendientesNoEditadosSubproyectosYProyectoGeneral(Prevision prevision)
  {
    prevision.setCuantia(new Double(0));
    prevision.setCuantiaPorcentual(new Double(0));
    this.setAscendientesNoEditadosSubproyectosYProyectoGeneral(prevision);
  }
  
  /**
   * Asigna ascendientes no editados subproyectos y proyecto general.
   * 
   * @param prevision
   *          ascendientes no editados subproyectos y proyecto general
   */
  public void setAscendientesNoEditadosSubproyectosYProyectoGeneral(Prevision prevision)
  {
    if(prevision.getCuantia() == 0)
      prevision.setCuantiaPorcentual(new Double(0));
    else
      prevision.setCuantiaPorcentual(new Double(100));
    
    if(prevision.getTieneDescendientesAsignados() == false){ //Para el item o sección asignada y/o cuyo valor varía
      //Asignamos la parte proporcional al tiempo que lleva abierto dicho centro para ese año, para los subproyectos existentes.
      List<Contenedor> contenedoresSubproyectos = prevision.getContenedor().getContenedoresSubproyectos();
      if(contenedoresSubproyectos != null && contenedoresSubproyectos.size() > 0){ //Si editamos desde el proyecto general
        for(Contenedor contenedorSubproyecto : contenedoresSubproyectos){
          contenedorSubproyecto.getPrevision().setAsignado(prevision.isAsignado());
          contenedorSubproyecto.getPrevision().setCuantia(this.getCuantiaProporcionalCentro(contenedorSubproyecto.getPrevision()));
          contenedorSubproyecto.getPrevision().setCuantiaPorcentual(this.getCuantiaPorcentualProporcionalCentro(contenedorSubproyecto.getPrevision()));
          for(Mes mes: Mes.values()){
            Double valorFijo = null;
            if(this.isCentroAbierto(contenedorSubproyecto.getPrevision(), mes)){
              valorFijo = contenedorSubproyecto.getPrevision().getCuantia()/this.getNumeroMesesAbiertoCentro(contenedorSubproyecto.getPrevision());
            }
            contenedorSubproyecto.getPrevision().getValoresFijos().put(mes, valorFijo);
          }
        }
      }else{ //Si editamos desde un proyecto en particular
        for(Mes mes: Mes.values()){
          Double valorFijo = null;
          if(this.isCentroAbierto(prevision, mes)){
            valorFijo = prevision.getCuantia()/this.getNumeroMesesAbiertoCentro(prevision);
          }
          prevision.getValoresFijos().put(mes, valorFijo);
        }
      }
    }else{ //Para items o secciones que tienen hijos asignados (padres de los items asignados y/o cuyo valor ha variado)
      this.sumatorioCuantiasPrevisionPadre(prevision);
    }
    
    if(prevision.getContenedor().getContenedorProyectoGeneral() != null){
      Prevision previsionProyectoGeneral = prevision.getContenedor().getContenedorProyectoGeneral().getPrevision();
      previsionProyectoGeneral.setCuantia(this.getSumatorioCuantiaSubproyectos(previsionProyectoGeneral));
      if(previsionProyectoGeneral.getCuantia() == 0)
        previsionProyectoGeneral.setCuantiaPorcentual(new Double(0));
      else
        previsionProyectoGeneral.setCuantiaPorcentual(new Double(100));
    }
    
    if(prevision.getPrevisionPadre() != null)
    {
      prevision.getPrevisionPadre().setAsignado(false);
      prevision.getPrevisionPadre().setCuantia(prevision.getPrevisionPadre().getCuantiaEfectiva());
      
      this.setAscendientesNoEditadosSubproyectosYProyectoGeneral(prevision.getPrevisionPadre());
    }
  }
  
  /**
   * Sumatorio cuantias prevision padre.
   * 
   * @param prevision
   *           prevision
   */
  public void sumatorioCuantiasPrevisionPadre(Prevision prevision){
    List<Contenedor> contenedoresSubproyectos = prevision.getContenedor().getContenedoresSubproyectos();
    
    for(Contenedor contenedorSubproyecto : contenedoresSubproyectos){
      Double sumatorioSubproyecto = new Double(0);
      Prevision previsionSubproyecto = contenedorSubproyecto.getPrevision();
      
      for(Mes mes : Mes.values()){
        Double sumatorioSubproyectoMes = null;
    
        List<Prevision> previsionesSubproyectoDescendientes = previsionSubproyecto.getPrevisionesDescendientes();
      
        for(Prevision previsionSubproyectoDescendiente : previsionesSubproyectoDescendientes){
          
          if(previsionSubproyectoDescendiente.getValoresFijos().get(mes) != null){
            if(sumatorioSubproyectoMes == null)
              sumatorioSubproyectoMes = new Double(0);
            sumatorioSubproyectoMes += previsionSubproyectoDescendiente.getValoresFijos().get(mes);
          }
        }
        
        previsionSubproyecto.getValoresFijos().put(mes, sumatorioSubproyectoMes);
        if(sumatorioSubproyectoMes != null)
        {
          sumatorioSubproyecto += sumatorioSubproyectoMes;
        }
      }
      
      previsionSubproyecto.setCuantia(sumatorioSubproyecto);
      previsionSubproyecto.setCuantiaPorcentual(previsionSubproyecto.getCuantia() * new Double(100) / prevision.getCuantia());
    }
  }
  
//  public Double getCuantiaEfectiva(Prevision prevision)
//  {
//    Double total = new Double(0);
//    
//    if(prevision.isAsignado() == true)
//      total = prevision.getCuantia();
//    else
//    {
//      List<Prevision> previsionesDescendientes = prevision.getPrevisionesDescendientes();
//      for(Prevision previsionDescendiente : previsionesDescendientes)
//      {
//        total += this.getCuantiaEfectiva(previsionDescendiente);
//      }
//    }
//    
//    return total;
//  }
  
  
  /**
 * Devuelve cuantia proporcional centro.
 * 
 * @param prevision
 *           prevision
 * @return cuantia proporcional centro
 */
public Double getCuantiaProporcionalCentro(Prevision prevision)
  {
    Double result = new Double(0);
    
    Double coeficiente = this.getNumeroMesesAbiertoCentro(prevision) / this.getNumeroMesesAbiertosCentros(prevision);
      
    result = prevision.getContenedor().getContenedorProyectoGeneral().getPrevision().getCuantiaEfectiva() * coeficiente;
    
    return result;
  }
  
  /**
   * Devuelve cuantia porcentual proporcional centro.
   * 
   * @param prevision
   *           prevision
   * @return cuantia porcentual proporcional centro
   */
  public Double getCuantiaPorcentualProporcionalCentro(Prevision prevision)
  {
    Double result = new Double(0);
    
    if(prevision.getCuantia() != 0)
      result = new Double(100);
    
    Double coeficiente = this.getNumeroMesesAbiertoCentro(prevision) / getNumeroMesesAbiertosCentros(prevision);
      
    result = result * coeficiente;
    
    return result;
  }
  
  /**
   * Devuelve numero meses abiertos centros.
   * 
   * @param prevision
   *           prevision
   * @return numero meses abiertos centros
   */
  public Double getNumeroMesesAbiertosCentros(Prevision prevision){
    Double total = new Double(0);
    Contenedor contenedorProyectoGeneral = null;
    
    if(prevision.getContenedor().getContenedorProyectoGeneral() == null){//Es del proyecto general.
      contenedorProyectoGeneral = prevision.getContenedor();
    }else{//Es de un subproyecto
      contenedorProyectoGeneral = prevision.getContenedor().getContenedorProyectoGeneral();
    }
    
    List<Contenedor> contenedoresSubproyectos = contenedorProyectoGeneral.getContenedoresSubproyectos();
    for(Contenedor contenedorSubproyecto : contenedoresSubproyectos){
      total += this.getNumeroMesesAbiertoCentro(contenedorSubproyecto.getPrevision());
    }
    
    return total;
  }
  
  /**
   * Devuelve numero meses abierto centro.
   * 
   * @param prevision
   *           prevision
   * @return numero meses abierto centro
   */
  public Double getNumeroMesesAbiertoCentro(Prevision prevision)
  {
    Double result = null;
    
    if(prevision.getContenedor().getCentro() != null){
      Centro centro = prevision.getContenedor().getCentro();
      if(centro.getFechaApertura() != null){
        Calendar calendar = Calendar.getInstance();
        calendar.setTimeInMillis(centro.getFechaApertura().getTime());
        int anyoAperturaCentro = calendar.get(Calendar.YEAR);
        if(prevision.getContenedor().getAnyo() != null){
          Integer anyoPrevision = prevision.getContenedor().getAnyo();
          if(anyoPrevision.compareTo(anyoAperturaCentro) < 0){
            result = new Double(0);
          }else if(anyoPrevision.compareTo(anyoAperturaCentro) > 0){
            result = new Double(12);
          }else{
            result = new Double(12) - new Double(calendar.get(Calendar.MONTH));
          }
        }
      }else{
        result = new Double(12);//Si no tiene fecha asignada, consideramos que estaba abierto.
      }
    }
    
    return result;
  }
  
  /**
   * Devuelve sumatorio cuantia subproyectos.
   * 
   * @param prevision
   *           prevision
   * @return sumatorio cuantia subproyectos
   */
  public Double getSumatorioCuantiaSubproyectos(Prevision prevision){
    Double result = new Double(0);
    
    List<Contenedor> contenedoresSubproyectos = prevision.getContenedor().getContenedoresSubproyectos();
    if(contenedoresSubproyectos != null){
      for(Contenedor contenedorSubproyecto : contenedoresSubproyectos){
        Prevision previsionSubproyecto = contenedorSubproyecto.getPrevision(); 
        result += previsionSubproyecto.getCuantia();
      }
    }
    
    return result;
  }
  
  /**
   * Checks if is centro abierto.
   * 
   * @param prevision
   *           prevision
   * @param mes
   *           mes
   * @return true, if is centro abierto
   */
  public boolean isCentroAbierto(Prevision prevision, Mes mes){
     boolean result = false;
    
    if(prevision.getContenedor().getCentro() != null){
      Centro centro = prevision.getContenedor().getCentro();
      if(centro.getFechaApertura() != null){
        Calendar calendar = Calendar.getInstance();
        calendar.setTimeInMillis(centro.getFechaApertura().getTime());
        int anyoAperturaCentro = calendar.get(Calendar.YEAR);
        if(prevision.getContenedor().getAnyo() != null){
          Integer anyoPrevision = prevision.getContenedor().getAnyo();
          if(anyoPrevision.compareTo(anyoAperturaCentro) < 0){
            result = false;
          }else if(anyoPrevision.compareTo(anyoAperturaCentro) > 0){
            result = true;
          }else{
            Integer mesPrevision = new Integer(calendar.get(Calendar.MONTH));
            if(mesPrevision.compareTo(mes.ordinal()) <= 0){
              result = true;
            }else{
              result = false;
            }
          }
        }
      }else{
        result = true;//Si no tiene fecha asignada, consideramos que estaba abierto.
      }
    }
    
    return result;
  }
  
  /**
   * Asignar previsiones iniciales.
   * 
   * @param prevision
   *           prevision
   */
  public void asignarPrevisionesIniciales(Prevision prevision){
    prevision.setCuantiaInicial(prevision.getCuantia());
    
    if(prevision.getPrevisionPadre() != null){
      this.asignarPrevisionesIniciales(prevision.getPrevisionPadre());      
    }
  }
  
  /**
   * Comprobar prevision.
   * 
   * @param prevision
   *           prevision
   * @return true, if successful
   */
  public boolean comprobarPrevision(Prevision prevision){
    boolean result = false;
    
//    if(prevision.getContenedor().getContenedorProyectoGeneral() != null){
//      result = true;        
//    }else{
//      if(!prevision.isAsignado()){
//        result = true;
//      } else {
        if(redondearDouble(prevision.getCuantiaInicial(), 2) == redondearDouble(prevision.getCuantia(), 2)){
          result = true;
        }else{
          result = false;
          addErrorMessage("financiero.error.totalAsignado.distinto.totalActual");
        }
//      }
//    }
    
    return result;
  }
  
//  public float redondearFloat(float valor, int numeroDecimales){
//    
//      float factor = (long)Math.pow(10,numeroDecimales);
//
//      valor = valor * factor;
//
//      float tmp = Math.round(valor);
//
//      return (float)tmp / factor;
//  }
  
  /**
 * Redondear double.
 * 
 * @param valor
 *           valor
 * @param numeroDecimales
 *           numero decimales
 * @return double
 */
public double redondearDouble(double valor, int numeroDecimales){
    
    Double factor = Math.pow(10,numeroDecimales);

    valor = valor * factor;

    double tmp = Math.round(valor);

    return (double)tmp / factor;
  }
  
  /**
   * Repartir total subproyecto.
   * 
   * @param prevision
   *           prevision
   */
  public void repartirTotalSubproyecto(Prevision prevision){
    for(Mes mes: Mes.values()){
      Double valorFijo = null;
      if(this.isCentroAbierto(prevision, mes)){
        valorFijo = prevision.getCuantia()/this.getNumeroMesesAbiertoCentro(prevision);
      }
      prevision.getValoresFijos().put(mes, valorFijo);
    }
    
    if(prevision.getPrevisionPadre() != null){
      prevision.getPrevisionPadre().setCuantia(prevision.getPrevisionPadre().getCuantiaEfectiva());
      this.repartirTotalSubproyecto(prevision.getPrevisionPadre());
    }
  }
  
  /**
   * Recalcular total todos centros.
   * 
   * @param prevision
   *           prevision
   */
  public void recalcularTotalTodosCentros(Prevision prevision){
    Double total = new Double(0);
    
    List<Contenedor> contenedoresSubproyectos = prevision.getContenedor().getContenedoresSubproyectos();
    if(contenedoresSubproyectos != null && contenedoresSubproyectos.size() > 0){
      for(Contenedor contenedorSubproyecto : contenedoresSubproyectos){
        total += contenedorSubproyecto.getPrevision().getCuantia();
      }
      
      prevision.setCuantia(total);
    }
    
    if(prevision.getPrevisionPadre() != null){
      this.recalcularTotalTodosCentros(prevision.getPrevisionPadre());
    }
  }
  
  /**
   * Recalcular valores porcentuales.
   * 
   * @param prevision
   *           prevision
   */
  public void recalcularValoresPorcentuales(Prevision prevision){
    List<Contenedor> contenedoresSubproyectos = prevision.getContenedor().getContenedoresSubproyectos();
    if(contenedoresSubproyectos != null){
      for(Contenedor contenedorSubproyecto : contenedoresSubproyectos){
        Double result = contenedorSubproyecto.getPrevision().getCuantia() * new Double(100)/prevision.getCuantiaInicial();
        contenedorSubproyecto.getPrevision().setCuantiaPorcentual(result);
      }
    }
    
    if(prevision.getPrevisionPadre() != null)
      this.recalcularValoresPorcentuales(prevision.getPrevisionPadre());
  }
  
  /**
   * Asignar porcentaje subproyecto a total.
   * 
   * @param previsionSubproyecto
   *           prevision subproyecto
   */
  public void asignarPorcentajeSubproyectoATotal(Prevision previsionSubproyecto){
    Double cuantiaInicial = previsionSubproyecto.getContenedor().getContenedorProyectoGeneral().getPrevision().getCuantiaInicial();
    
    previsionSubproyecto.setCuantia(cuantiaInicial * previsionSubproyecto.getCuantiaPorcentual() / new Double(100));
    
    if(previsionSubproyecto.getPrevisionPadre() != null){
      this.asignarPorcentajeSubproyectoPadreATotal(previsionSubproyecto.getPrevisionPadre());
    }
  }
  
  private void asignarPorcentajeSubproyectoPadreATotal(Prevision previsionSubproyectoPadre){
    Double cuantiaTotal = previsionSubproyectoPadre.getContenedor().getContenedorProyectoGeneral().getPrevision().getCuantiaInicial();
    
    previsionSubproyectoPadre.setCuantiaPorcentual(previsionSubproyectoPadre.getCuantiaEfectiva() * new Double(100) / cuantiaTotal);
    
    if(previsionSubproyectoPadre.getPrevisionPadre() != null){
      this.asignarPorcentajeSubproyectoPadreATotal(previsionSubproyectoPadre.getPrevisionPadre());
    }
  }
  
  /**
   * Recalcular porcentaje todos centros.
   * 
   * @param prevision
   *           prevision
   */
  public void recalcularPorcentajeTodosCentros(Prevision prevision){
    Double porcentaje = new Double(0);
    
    List<Contenedor> contenedoresSubproyectos = prevision.getContenedor().getContenedoresSubproyectos();
    if(contenedoresSubproyectos != null && contenedoresSubproyectos.size() > 0){
      for(Contenedor contenedorSubproyecto : contenedoresSubproyectos){
        porcentaje += contenedorSubproyecto.getPrevision().getCuantiaPorcentual();
      }
      
      prevision.setCuantiaPorcentual(porcentaje);
    }
  }
  
  /**
   * Recalcular total todos meses.
   * 
   * @param prevision
   *           prevision
   * @param mes
   *           mes
   */
  public void recalcularTotalTodosMeses(Prevision prevision, Mes mes){
    Double total = new Double(0);
    Collection<Double> valores = prevision.getValoresFijos().values();
    
    for(Double valor : valores){
      if(valor != null){
        total += valor;
      }
    }
    
    prevision.setCuantia(total);
    
    if(prevision.getPrevisionPadre() != null){
      prevision.getPrevisionPadre().getValoresFijos().put(mes, prevision.getPrevisionPadre().getCuantiaEfectivaMensual(mes));
      this.recalcularTotalTodosMeses(prevision.getPrevisionPadre(), mes);
    }
  }
  
//  public Double getCuantiaEfectivaMensual(Prevision prevision, Mes mes)
//  {
//    Double total = new Double(0);
//    
//    if(prevision.isAsignado() == true)
//      total = prevision.getValoresFijos().get(mes);
//    else
//    {
//      List<Prevision> previsionesDescendientes = prevision.getPrevisionesDescendientes();
//      for(Prevision previsionDescendiente : previsionesDescendientes)
//      {
//        if(previsionDescendiente.getValoresFijos().get(mes) != null){
//          total += this.getCuantiaEfectivaMensual(previsionDescendiente, mes);
//        }
//      }
//    }
//    
//    return total;
//  }
  
  //Selección Checkboxes
  /*public void marcarDesmarcarSubproyecto(Contenedor contenedorSubproyectoMarcado, Prevision detallePrevision){
    Prevision previsionSubproyectoMarcado = contenedorSubproyectoMarcado.getPrevision();
    
    Boolean nuevoValor = previsionSubproyectoMarcado.isSeleccionadoValores();
    HashMap<Mes, Boolean> seleccionValorFijo = previsionSubproyectoMarcado.getSeleccionValorFijo();
    Set<Mes> meses = seleccionValorFijo.keySet();
    
    for(Mes mes : meses){
      if(nuevoValor != seleccionValorFijo.get(mes)){
        seleccionValorFijo.put(mes, nuevoValor);
        detallePrevision.establecerSeleccionMes(mes);
      }
    }
  }*/
  
  //Selección Checkboxes
  /*public void marcarDesmarcarMes(Mes mesMarcado, Prevision detallePrevision){
    List<Contenedor> contenedoresSubproyectos = detallePrevision.getContenedor().getContenedoresSubproyectos();
    
    Boolean nuevoValor = detallePrevision.getSeleccionMeses().get(mesMarcado);
    
    for(Contenedor contenedorSubproyecto : contenedoresSubproyectos){
      Prevision previsionSubproyecto = contenedorSubproyecto.getPrevision();
      HashMap<Mes, Boolean> seleccionesValorFijoSubproyecto = previsionSubproyecto.getSeleccionValorFijo();
      
      if(nuevoValor != seleccionesValorFijoSubproyecto.get(mesMarcado)){
        seleccionesValorFijoSubproyecto.put(mesMarcado, nuevoValor);
        detallePrevision.establecerSeleccionValoresSubproyecto(contenedorSubproyecto);
      }
    }
  }*/
  
}
