package es.csa.fdt.cyldigital.financiero.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.HashMap;
import java.util.List;

import javax.persistence.Query;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.financiero.entity.Contenedor;
import es.csa.fdt.cyldigital.financiero.entity.ItemContenedor;
import es.csa.fdt.cyldigital.financiero.entity.Prevision;
import es.csa.fdt.cyldigital.financiero.entity.Proyecto;
import es.csa.fdt.cyldigital.financiero.entity.ProyectoAnual;
import es.csa.fdt.cyldigital.financiero.entity.Seccion;

/**
 * Clase ProyectoAnualServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class ProyectoAnualServiceImpl extends AbstractServiceImpl<ProyectoAnual, Long> implements ProyectoAnualService{
   
   private static final long serialVersionUID = 2374504192348922345L;

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.core.service.AbstractServiceImpl#persist(java.lang.Object)
   */
  @Override
  public <T> void persist(T entity) {
    ProyectoAnual proyectoAnual = (ProyectoAnual)entity;
    super.persist(proyectoAnual);
    
    Proyecto proyectoGeneral = proyectoAnual.getProyecto();
    List<Proyecto> subproyectos = proyectoGeneral.getSubproyectos();
    
    for(Proyecto subproyecto : subproyectos)
    {
      ProyectoAnual subproyectoAnual = new ProyectoAnual();
      subproyectoAnual.setProyectoGeneral(proyectoAnual);
      subproyectoAnual.setProyecto(subproyecto);
      subproyectoAnual.setCentro(subproyecto.getCentro());
      subproyectoAnual.setAnyo(proyectoAnual.getAnyo());
      
      super.persist(subproyectoAnual);
    }
  }
   
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.financiero.service.ProyectoAnualService#updateSubproyectosAnuales(es.csa.fdt.cyldigital.financiero.entity.ProyectoAnual)
   */
  public void updateSubproyectosAnuales(ProyectoAnual proyectoAnualGeneral)
  {
    List<ProyectoAnual> subproyectosAnuales = proyectoAnualGeneral.getSubproyectosAnuales();
    for(ProyectoAnual subproyectoAnual : subproyectosAnuales)
    {
      subproyectoAnual.setAnyo(proyectoAnualGeneral.getAnyo());
    }
  }
  
  /**
   * Devuelve ejercicios generados.
   * 
   * @param proyectoAnual
   *           proyecto anual
   * @return ejercicios generados
   */
  @SuppressWarnings("unchecked")
  public List<Integer> getEjerciciosGenerados(ProyectoAnual proyectoAnual)
  {
    List<Integer> ejercicios = null;
    
    if(proyectoAnual.getProyecto().getProyectoGeneral() != null) //Sólo para subproyectos anuales, ya que el combo no aparecerá para
    {                                              //proyectos generales.

      StringBuffer buffer = new StringBuffer();
      
      buffer.append("SELECT proyectoAnual.anyo FROM ").append(ProyectoAnual.class.getName()).append(" proyectoAnual, ");
      buffer.append(Proyecto.class.getName()).append(" proyecto WHERE proyectoAnual.proyecto = :subproyecto");
      
      Query query = this.getEntityManager().createQuery(buffer.toString());
      
      Proyecto subproyecto = proyectoAnual.getProyecto();
      
      query.setParameter("subproyecto", subproyecto);
      
      List<Integer> ejerciciosGenerados = query.getResultList();
      
      buffer = new StringBuffer();
      
      buffer.append("SELECT proyectoAnualGeneral.anyo FROM ").append(ProyectoAnual.class.getName()).append(" proyectoAnualGeneral");
      buffer.append(" WHERE proyectoAnualGeneral.proyecto = :proyectoGeneral");
      if(ejerciciosGenerados.size() > 0)
      {
        buffer.append(" AND proyectoAnualGeneral.anyo NOT IN (:ejerciciosGenerados)");
      }
      
      query = this.getEntityManager().createQuery(buffer.toString());
      
      Proyecto proyectoGeneral = subproyecto.getProyectoGeneral();
      
      query.setParameter("proyectoGeneral", proyectoGeneral);
      if(ejerciciosGenerados.size() > 0)
      {
        query.setParameter("ejerciciosGenerados", ejerciciosGenerados);
      }
      
      ejercicios = query.getResultList();
    }
    
    return ejercicios;
  }
  
//  @SuppressWarnings("unchecked")
//  public ProyectoAnual getProyectoAnualGeneral(Proyecto proyectoGeneral, Integer ejercicio)
//  {
//    StringBuffer buffer = new StringBuffer();
//    buffer.append("SELECT proyectoAnualGeneral FROM ").append(ProyectoAnual.class.getName()).append(" proyectoAnualGeneral");
//    buffer.append(" WHERE proyectoAnualGeneral.proyecto = :proyectoGeneral AND proyectoAnualGeneral.anyo = :anyo");
//    
//    Query query = this.getEntityManager().createQuery(buffer.toString());
//    query.setParameter("proyectoGeneral", proyectoGeneral);
//    query.setParameter("anyo", ejercicio);
//    
//    List<ProyectoAnual> proyectosAnualesGenerales = query.getResultList();
//    
//    return proyectosAnualesGenerales.get(0);
//  }
  
  /**
 * Asigna proyecto general.
 * 
 * @param proyecto
 *           proyecto
 * @param proyectoAnual
 *           proyecto anual
 */
public void setProyectoGeneral(Proyecto proyecto, ProyectoAnual proyectoAnual)
  {
    ProyectoAnual proyectoAnualGeneral = null;
    
    if(proyecto.esProyectoGeneral() == false){ //El proyecto es un proyectoGeneral, en lugar de un subproyecto y por tanto
                                            //no hace falta agregarle un proyecto general.
  //    ProyectoAnual proyectoAnualGeneral = this.getProyectoAnualGeneral(proyecto.getProyectoGeneral(), proyectoAnual.getAnyo());
      HashMap<String, Object> propiedades = new HashMap<String, Object>();
      propiedades.put("proyecto", proyecto.getProyectoGeneral());
      propiedades.put("anyo", proyectoAnual.getAnyo());
      
      proyectoAnualGeneral = this.findOneWhere(propiedades);
    }
    proyectoAnual.setProyectoGeneral(proyectoAnualGeneral);
  }
  
//  @SuppressWarnings("unchecked")
//  public void precargaItems(ProyectoAnual proyectoAnual, Object object)
//  {
//    SAXBuilder builder=new SAXBuilder(false); 
//    Document doc = null;
//    try {
//      StringBuilder path = new StringBuilder();
//      path.append("src/main/resources/xml/precarga_items.xml");
//      doc = builder.build(path.toString());
//      Element raiz = doc.getRootElement();
//      List<Element> secciones=raiz.getChildren("seccion");
//      for (Element elementSeccion: secciones){
//        Seccion seccion = new Seccion(elementSeccion.getAttributeValue("nombre"), proyectoAnual);
//        this.persist(seccion);
//        List<Element> items = elementSeccion.getChildren("item");
//        for(Element item : items)
//        {
//          parsearItem(item, seccion);
//        }
//      }
//    } catch (JDOMException e) {
//      // TODO Auto-generated catch block
//      e.printStackTrace();
//    } catch (IOException e) {
//      // TODO Auto-generated catch block
//      e.printStackTrace();
//    }
//  }
//  
//  @SuppressWarnings("unchecked")
//  public void parsearItem(Element item, Contenedor contenedorPadre)
//  {
//    ItemContenedor itemContenedor = new ItemContenedor(item.getAttributeValue("nombre"), contenedorPadre);
//    this.persist(itemContenedor);
//    
//    List<Element> items = item.getChildren("item");
//    for(Element subitem : items)
//    {
//      parsearItem(subitem, itemContenedor);
//    }
//  }
  
  /**
 * Replicar estructura.
 * 
 * @param proyectoAnual
 *           proyecto anual
 */
public void replicarEstructura(ProyectoAnual proyectoAnual)
  {
    if(proyectoAnual.esProyectoGeneral() == false)
    {
      ProyectoAnual proyectoGeneral = proyectoAnual.getProyectoGeneral();
      
      List<Contenedor> secciones = proyectoGeneral.getElementos();
      
      int i = 0;
      
      for(Contenedor contenedor : secciones)
      {
        Seccion seccion = (Seccion)contenedor;
        Seccion newSeccion = seccion.cloneSeccion(proyectoAnual, seccion);
        
//        newSeccion.setProyecto(proyectoAnual);
//        newSeccion.setProyectoRaiz(proyectoAnual);
//        newSeccion.setContenedorProyectoGeneral(seccion);
        
        newSeccion.setSecuenciaInicial(i);
        
        this.getEntityManager().persist(newSeccion);
        
        replicarEstructuraContenedorSubproyecto(seccion, newSeccion);
        
        i++;
      }
    }
  }
  
  private void replicarEstructuraContenedorSubproyecto(Contenedor contenedorProyectoGeneral, Contenedor contenedorSubProyecto)
  {
    List<Contenedor> contenedoresItems = contenedorProyectoGeneral.getElementos();
    
    int i = 0;
    
    for(Contenedor contenedorItem : contenedoresItems)
    {
      ItemContenedor item = (ItemContenedor)contenedorItem;
      
      //ItemContenedor newItem = item.cloneItemContenedor(contenedorSubProyecto.getProyectoRaiz(), ItemContenedorServiceImpl.obtenerSeccionRaiz(contenedorSubProyecto), contenedorSubProyecto, item);
      
      ItemContenedor newItem = item.cloneItemContenedor(contenedorSubProyecto.getProyectoRaiz(), contenedorSubProyecto.getSeccionRaiz(), contenedorSubProyecto, item);
      
//      newItem.setProyectoRaiz(contenedorSubProyecto.getProyectoRaiz());
//      newItem.setSeccionRaiz(ItemContenedorServiceImpl.obtenerSeccionRaiz(contenedorSubProyecto));
//      newItem.setContenedorPadre(contenedorSubProyecto);
//      newItem.setContenedorProyectoGeneral(item);
      
      newItem.setSecuenciaInicial(i);
      
      this.getEntityManager().persist(newItem);
      
      replicarEstructuraContenedorSubproyecto(item, newItem);
      
      i++;
    }
  }
  
  /**
   * Cerrar previsiones iniciales.
   * 
   * @param proyectoAnual
   *           proyecto anual
   */
  public void cerrarPrevisionesIniciales(ProyectoAnual proyectoAnual){
    proyectoAnual.setPrevisionesInicialesCerradas(true);
    
    //Ponemos como previsiones cerradas todos los subproyectos
    List<ProyectoAnual> subProyectosAnuales = proyectoAnual.getSubproyectosAnuales();
    for(ProyectoAnual subProyectoAnual : subProyectosAnuales){
      subProyectoAnual.setPrevisionesInicialesCerradas(true);
    }
    
    List<Prevision> previsiones = this.obtenerPrevisionesSecciones(proyectoAnual);
    
    for(Prevision prevision : previsiones){
      prevision.establecerPrevisionesEvolutivas();
      this.establecerPrevisionesEvolutivasSubItems(prevision);
    }
    
    List<ProyectoAnual> subproyectos = proyectoAnual.getSubproyectosAnuales();
    
    for(ProyectoAnual subproyecto : subproyectos){
      cerrarPrevisionesIniciales(subproyecto);
    }
  }
  
  //Método para obtener las previsiones de las secciones que componen el proyecto anual.
  @SuppressWarnings("unchecked")
  private List<Prevision> obtenerPrevisionesSecciones(ProyectoAnual proyectoAnual){
    StringBuffer buffer = new StringBuffer();
    buffer.append("SELECT prevision FROM ").append(Prevision.class.getName()).append(" prevision WHERE prevision.contenedor.proyecto = :proyectoAnual");

    Query query = this.getEntityManager().createQuery(buffer.toString());
    query.setParameter("proyectoAnual", proyectoAnual);
    
    return (List<Prevision>)query.getResultList();
  }
  
  private void establecerPrevisionesEvolutivasSubItems(Prevision previsionPadre){
    List<Prevision> previsionesDescendientes = previsionPadre.getPrevisionesDescendientes();
    
    for(Prevision previsionDescendiente : previsionesDescendientes){
      previsionDescendiente.establecerPrevisionesEvolutivas();
      this.establecerPrevisionesEvolutivasSubItems(previsionDescendiente);
    }
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.core.service.AbstractServiceImpl#clear()
   */
  public void clear(){
    this.getEntityManager().clear();
  }

 }
