package es.csa.fdt.cyldigital.financiero.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.List;
import java.util.ResourceBundle;

import javax.persistence.Query;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.financiero.entity.Proyecto;

/**
 * Clase ProyectoServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class ProyectoServiceImpl extends AbstractServiceImpl<Proyecto, Long> implements ProyectoService{

  private static final long serialVersionUID = 3598234759237512375L;
  
  @SuppressWarnings("unchecked")
  private List<Centro> getTodosCentros()
  {
    StringBuffer sql = new StringBuffer();
    sql.append("SELECT centro FROM ").append(Centro.class.getName()).append(" centro ORDER BY centro.localidad.nombre");
    
    Query query = this.getEntityManager().createQuery(sql.toString());
    List<Centro> centros = query.getResultList();
    
    return centros;
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.financiero.service.ProyectoService#getCentrosDisponibles(es.csa.fdt.cyldigital.financiero.entity.Proyecto)
   */
  @SuppressWarnings("unchecked")
  public List<Centro> getCentrosDisponibles(Proyecto proyecto)
  {
    List<Centro> centros = null;
    
    if(proyecto.getProyectoGeneral() != null) //Sólo para subproyectos, no proyectos generales. Los proyectos generales
    {                                         //no necesitan una lista de centros a los que asociarse al ser editados.
      StringBuffer buffer = new StringBuffer();
      
      Proyecto proyectoGeneral = proyecto.getProyectoGeneral();
      
      buffer.append("SELECT centro FROM ").append(Centro.class.getName()).append(" centro, ").append(Proyecto.class.getName()).append(" proyecto");
      buffer.append(" WHERE proyecto.proyectoGeneral = :proyectoGeneral AND proyecto.centro = centro");
      
      Query query = this.getEntityManager().createQuery(buffer.toString());
      query.setParameter("proyectoGeneral", proyectoGeneral);
      
      List<Centro> centrosAsociados = query.getResultList();
      
      buffer = new StringBuffer();
      buffer.append("SELECT centro FROM ").append(Centro.class.getName()).append(" centro");
      if(centrosAsociados.size() > 0)
      {
        buffer.append(" WHERE NOT centro IN (:centrosAsociados)");
      }
      if(proyecto.getCentro() != null)
      {
        buffer.append(" OR centro = :centroProyectoActual");
      }
      
      query = this.getEntityManager().createQuery(buffer.toString());
      if(centrosAsociados.size() > 0)
      {
        query.setParameter("centrosAsociados", centrosAsociados);
      }
      if(proyecto.getCentro() != null)
      {
        query.setParameter("centroProyectoActual", proyecto.getCentro());
      }
      
      centros = query.getResultList();
    }
    
    return centros;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.core.service.AbstractServiceImpl#persist(java.lang.Object)
   */
  @Override
  public <T> void persist(T entity) {
    Proyecto proyecto = (Proyecto)entity;
    super.persist(proyecto);
    
    if(proyecto.getProyectoGeneral() == null)
    {
      List<Centro> centros = this.getTodosCentros();
      for(Centro centro : centros)
      {
        Proyecto subproyecto = new Proyecto();
        subproyecto.setProyectoGeneral(proyecto);
        subproyecto.setCentro(centro);
        subproyecto.setAprobadorCuantiaGrande(proyecto.getAprobadorCuantiaGrande());
        subproyecto.setAprobadorCuantiaMediana(proyecto.getAprobadorCuantiaMediana());
        subproyecto.setAprobadorCuantiaPequena(proyecto.getAprobadorCuantiaPequena());
        
        StringBuffer buffer = new StringBuffer();
        ResourceBundle res = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages");
        buffer.append(res.getString("financiero.subproyecto.centro.nombre"));
        buffer.append(" ");
        buffer.append(centro.getNombre());
        subproyecto.setNombre(buffer.toString());
        
        super.persist(subproyecto);
      }
    }
  }
  
  /**
   * Asignar proyecto general y responsables.
   * 
   * @param nuevoProyecto
   *           nuevo proyecto
   * @param proyectoGeneral
   *           proyecto general
   */
  public void asignarProyectoGeneralYResponsables(Proyecto nuevoProyecto, Proyecto proyectoGeneral){
    nuevoProyecto.setProyectoGeneral(proyectoGeneral);
    
    if(proyectoGeneral != null){
      nuevoProyecto.setAprobadorCuantiaGrande(proyectoGeneral.getAprobadorCuantiaGrande());
      nuevoProyecto.setAprobadorCuantiaMediana(proyectoGeneral.getAprobadorCuantiaMediana());
      nuevoProyecto.setAprobadorCuantiaPequena(proyectoGeneral.getAprobadorCuantiaPequena());
    }
  }
  
  /**
   * Actualizar subproyectos.
   * 
   * @param proyectoGeneral
   *           proyecto general
   */
  public void actualizarSubproyectos(Proyecto proyectoGeneral){
    List<Proyecto> subproyectos = proyectoGeneral.getSubproyectos();
    
    for(Proyecto subproyecto : subproyectos){
      /* Actualizamos los datos comunes para proyecto general y subproyectos. */
      subproyecto.setAprobadorCuantiaGrande(proyectoGeneral.getAprobadorCuantiaGrande());
      subproyecto.setAprobadorCuantiaMediana(proyectoGeneral.getAprobadorCuantiaMediana());
      subproyecto.setAprobadorCuantiaPequena(proyectoGeneral.getAprobadorCuantiaPequena());
    }
  }
  
  /**
   * Para el menú seleccionable del buscador de facturas.
   * 
   * @return proyectos generales disponibles
   */
  @SuppressWarnings("unchecked")
  public List<Proyecto> getProyectosGeneralesDisponibles() {
    StringBuffer buffer = new StringBuffer();
    buffer.append("SELECT proyectoGeneral FROM ").append(Proyecto.class.getName()).append(" proyectoGeneral ");
    buffer.append("WHERE proyectoGeneral.proyectoGeneral is null");
    
    Query query = this.getEntityManager().createQuery(buffer.toString());
    return query.getResultList();
  }

}
