package es.csa.fdt.cyldigital.financiero.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.List;

import javax.persistence.Query;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.financiero.entity.Contenedor;
import es.csa.fdt.cyldigital.financiero.entity.ProyectoAnual;
import es.csa.fdt.cyldigital.financiero.entity.Seccion;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;

/**
 * Clase SeccionServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class SeccionServiceImpl extends AbstractServiceImpl<Seccion, Long> implements SeccionService{

  private static final long serialVersionUID = 2093592374895074239L;
  
  
  /**
   * Crea seccion.
   * 
   * @param seccion
   *           seccion
   * @param proyectoAnual
   *           proyecto anual
   */
  public void createSeccion(Seccion seccion, ProyectoAnual proyectoAnual)
  {
   seccion.setProyecto(proyectoAnual);
   seccion.setProyectoRaiz(proyectoAnual);
   proyectoAnual.getElementos().add(seccion);
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.core.service.AbstractServiceImpl#persist(java.lang.Object)
   */
  @Override
  public <T> void persist(T entity) {
    Seccion seccion = (Seccion)entity;
    super.persist(seccion);
    
    ProyectoAnual proyectoAnualGeneral = seccion.getProyecto();
    List<ProyectoAnual> subproyectos = proyectoAnualGeneral.getSubproyectosAnuales();
    
    for(ProyectoAnual subproyecto : subproyectos)
    {
      Seccion seccionSubproyecto = new Seccion(seccion.getNombre(), seccion.getResponsable(), subproyecto);
      seccionSubproyecto.setSecuencia(seccion.getSecuencia());
      
      seccionSubproyecto.setContenedorProyectoGeneral(seccion);
      
      subproyecto.getElementos().add(seccionSubproyecto);
      
      super.persist(seccionSubproyecto);
    }
  }
  
  /**
   * Actualizar subproyectos.
   * 
   * @param seccion
   *           seccion
   */
  public void actualizarSubproyectos(Seccion seccion){
    List<Contenedor> seccionesSubproyectos = seccion.getContenedoresSubproyectos();
    
    for(Contenedor seccionContenedorSubproyecto : seccionesSubproyectos)
    {
      Seccion seccionSubproyecto = (Seccion)seccionContenedorSubproyecto;
      seccionSubproyecto.setNombre(seccion.getNombre());
      seccionSubproyecto.setResponsable(seccion.getResponsable());
    }
  }
  
  /**
   * Elimina seccion.
   * 
   * @param idSeccion
   *           id seccion
   */
  public void removeSeccion(Long idSeccion)
  {
    Seccion seccion = find(Seccion.class, idSeccion);
    
    List<Contenedor> seccionesSubproyectos = seccion.getContenedoresSubproyectos();
    
    for(Contenedor seccionSubproyecto : seccionesSubproyectos)
    {
      seccionSubproyecto.getProyecto().getElementos().remove(seccionSubproyecto);
      seccionSubproyecto.setProyecto(null);
      
      remove(seccionSubproyecto);
    }
    
    seccion.getProyecto().getElementos().remove(seccion);
    seccion.setProyecto(null);
   
    remove(seccion);
  } 
  
  /**
   * Subir seccion.
   * 
   * @param idSeccion
   *           id seccion
   */
  public void subirSeccion(Long idSeccion)
  {
    Seccion seccion = find(Seccion.class, idSeccion);
    
    int posicion = seccion.getSecuencia();
    
    seccion.getProyecto().getElementos().remove(seccion);
    seccion.getProyecto().getElementos().add(posicion - 1, seccion);
    

    List<Contenedor> seccionesSubproyectos = seccion.getContenedoresSubproyectos();
    
    for(Contenedor seccionSubproyecto : seccionesSubproyectos)
    {
      seccionSubproyecto.getProyecto().getElementos().remove(seccionSubproyecto);
      seccionSubproyecto.getProyecto().getElementos().add(posicion - 1, seccionSubproyecto);
    }
  }
  
  /**
   * Bajar seccion.
   * 
   * @param idSeccion
   *           id seccion
   */
  public void bajarSeccion(Long idSeccion)
  {
    Seccion seccion = find(Seccion.class, idSeccion);
    
    int posicion = seccion.getSecuencia();
    
    seccion.getProyecto().getElementos().remove(seccion);
    seccion.getProyecto().getElementos().add(posicion + 1, seccion);
    
    List<Contenedor> seccionesSubproyectos = seccion.getContenedoresSubproyectos();
    
    for(Contenedor seccionSubproyecto : seccionesSubproyectos)
    {
      seccionSubproyecto.getProyecto().getElementos().remove(seccionSubproyecto);
      seccionSubproyecto.getProyecto().getElementos().add(posicion + 1, seccionSubproyecto);
    }
  }
  
  /**
   * Devuelve empleados responsables area.
   * 
   * @return empleados responsables area
   */
  @SuppressWarnings("unchecked")
  public List<Empleado> getEmpleadosResponsablesArea(){
    StringBuffer buffer = new StringBuffer();
    buffer.append("SELECT DISTINCT e FROM ").append(Empleado.class.getName()).append(" e, ").append(Seccion.class.getName()).append(" s ");
    buffer.append("WHERE s.responsable = e");
    
    Query query = this.getEntityManager().createQuery(buffer.toString());
    return query.getResultList();
  }
  
  /**
   * Devuelve secciones disponibles.
   * 
   * @return secciones disponibles
   */
  public List<Seccion> getSeccionesDisponibles() {
	    return findAll(Seccion.class);
	  }

}
