package es.csa.fdt.cyldigital.financiero.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.Collections;
import java.util.List;

import javax.persistence.Query;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.financiero.entity.Contenedor;
import es.csa.fdt.cyldigital.financiero.entity.Factura;

/**
 * Clase SeguimientoServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class SeguimientoServiceImpl extends AbstractServiceImpl<Contenedor, Long> implements SeguimientoService{

  private static final long serialVersionUID = 943075942307590423L;
  
  /**
   * Devuelve facturas prevision.
   * 
   * @param contenedor
   *           contenedor
   * @return facturas prevision
   */
  public List<Factura> getFacturasPrevision(Contenedor contenedor){
    List<Factura> facturas = this.getFacturasContenedorYSubcontenedores(contenedor);
    
    Collections.sort(facturas);
    
    return facturas;
  }
  
  @SuppressWarnings("unchecked")
  private List<Factura> getFacturasContenedorYSubcontenedores(Contenedor contenedor){
    StringBuffer queryString = new StringBuffer("SELECT e FROM ");
    queryString.append(Factura.class.getName());
    queryString.append(" e WHERE e.contenedor = :contenedor");
      
    if(contenedor.getContenedorProyectoGeneral() == null){//Es un contenedor de un proyecto general.
      queryString.append(" OR e.contenedor.contenedorProyectoGeneral = :contenedor");
    }else{
      queryString.append(" OR :contenedor MEMBER OF e.contenedor.contenedoresSubproyectos");
    }
    
    Query query = this.getEntityManager().createQuery(queryString.toString());
    query.setParameter("contenedor", contenedor);
   
    List<Factura> facturasContenedor =  query.getResultList();
    
    for(Contenedor subcontenedor : contenedor.getElementos()){
      facturasContenedor.addAll(this.getFacturasContenedorYSubcontenedores(subcontenedor));
    }
    
    return facturasContenedor;
  }
}

  
