package es.csa.fdt.cyldigital.incidencias.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;

/**
 * Clase CategoriaIncidencia.
 * 
 * @author csa
 */
@Entity
@Table(name = "categoriasincidencias")
@AttributeOverride(name = "id", column = @Column(name = "idCategoriaIncidencia"))
@Audited
public class CategoriaIncidencia extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = -7648562176655166537L;

  private boolean especial;
  private String nombre;
  private Empleado responsable;
  private List<Empleado> empleados;
  private List<Incidencia> incidencias;

  /**
   * Construye un nuevo objeto de la clase
   */
  public CategoriaIncidencia() {
    super();
    empleados = new ArrayList<Empleado>();
    incidencias = new ArrayList<Incidencia>();
  }

  /**
   * Devuelve nombre.
   * 
   * @return nombre
   */
  @Length(max = 255)
  @NotEmpty
  @NotNull
  public String getNombre() {
    return nombre;
  }

  /**
   * Asigna nombre.
   * 
   * @param nombre
   *          nombre
   */
  public void setNombre(String nombre) {
    this.nombre = nombre;
  }

  /**
   * Devuelve incidencias.
   * 
   * @return incidencias
   */
  @OneToMany(mappedBy = "categoriaIncidencia")
  public List<Incidencia> getIncidencias() {
    return incidencias;
  }

  /**
   * Asigna incidencias.
   * 
   * @param incidencias
   *          incidencias
   */
  public void setIncidencias(List<Incidencia> incidencias) {
    this.incidencias = incidencias;
  }

  /**
   * Devuelve empleados.
   * 
   * @return empleados
   */
  @NotAudited
  @ManyToMany(fetch = FetchType.LAZY, targetEntity = Empleado.class)
  @JoinTable(name = "empleados_categoriasincidencias", joinColumns = @JoinColumn(name = "idCategoriaIncidencia"), inverseJoinColumns = @JoinColumn(name = "idEmpleado"))
  public List<Empleado> getEmpleados() {
    return empleados;
  }

  /**
   * Asigna empleados.
   * 
   * @param empleados
   *          empleados
   */
  public void setEmpleados(List<Empleado> empleados) {
    this.empleados = empleados;
  }

  /**
   * Devuelve responsable.
   * 
   * @return responsable
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idUsuarioResponsable", nullable = false)
  @NotNull
  public Empleado getResponsable() {
    return responsable;
  }

  /**
   * Asigna responsable.
   * 
   * @param responsable
   *          responsable
   */
  public void setResponsable(Empleado responsable) {
    this.responsable = responsable;
  }

  /**
   * Añade empleado.
   * 
   * @param empleado
   *           empleado
   */
  @Transient
  public void addEmpleado(Empleado empleado) {
    if (empleado != null && !getEmpleados().contains(empleado)) {
      empleados.add(empleado);
    }
  }

  /**
   * Quitar empleado.
   * 
   * @param empleado
   *           empleado
   */
  @Transient
  public void quitarEmpleado(Empleado empleado) {
    if (empleado != null && getEmpleados().contains(empleado)) {
      empleados.remove(empleado);
    }
  }

  /**
   * Checks if is especial.
   * 
   * @return true, if is especial
   */
  public boolean isEspecial() {
    return especial;
  }

  /**
   * Asigna especial.
   * 
   * @param especial
   *          especial
   */
  public void setEspecial(boolean especial) {
    this.especial = especial;
  }
}
