package es.csa.fdt.cyldigital.incidencias.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.Date;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import org.hibernate.envers.Audited;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.incidencias.entity.Incidencia.Criticidad;
import es.csa.fdt.cyldigital.incidencias.entity.Incidencia.EstadoIncidencia;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;

/**
 * Clase EventoIncidencia.
 * 
 * @author csa
 */
@Entity
@Table(name = "eventosincidencia")
@AttributeOverride(name = "id", column = @Column(name = "idEstadoIncidencia"))
@Audited
public class EventoIncidencia extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = -7648562176655166537L;

  private String descripcion;
  private Date fechaEstado;
  private Incidencia incidencia;
  private EstadoIncidencia estadoIncidencia;
  private Usuario usuario;
  private Criticidad criticidad;
  private Lista lista;

  /**
   * Construye un nuevo objeto de la clase
   */
  public EventoIncidencia() {
    super();
  }

  /**
   * Devuelve descripcion.
   * 
   * @return descripcion
   */
  @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getDescripcion() {
    return descripcion;
  }

  /**
   * Asigna descripcion.
   * 
   * @param descripcion
   *          descripcion
   */
  public void setDescripcion(String descripcion) {
    this.descripcion = descripcion;
  }

  /**
   * Devuelve fecha estado.
   * 
   * @return fecha estado
   */
  public Date getFechaEstado() {
    return fechaEstado;
  }

  /**
   * Asigna fecha estado.
   * 
   * @param fechaEstado
   *          fecha estado
   */
  public void setFechaEstado(Date fechaEstado) {
    this.fechaEstado = fechaEstado;
  }

  /**
   * Devuelve incidencia.
   * 
   * @return incidencia
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idIncidencia", nullable = false)
  public Incidencia getIncidencia() {
    return incidencia;
  }

  /**
   * Asigna incidencia.
   * 
   * @param incidencia
   *          incidencia
   */
  public void setIncidencia(Incidencia incidencia) {
    this.incidencia = incidencia;
  }

  /**
   * Devuelve estado.
   * 
   * @return estado
   */
  @Enumerated(EnumType.STRING)
  public EstadoIncidencia getEstado() {
    return estadoIncidencia;
  }

  /**
   * Asigna estado.
   * 
   * @param estadoIncidencia
   *          estado
   */
  public void setEstado(EstadoIncidencia estadoIncidencia) {
    this.estadoIncidencia = estadoIncidencia;
  }

  /**
   * Devuelve usuario.
   * 
   * @return usuario
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idUsuario", nullable = false)
  public Usuario getUsuario() {
    return usuario;
  }

  /**
   * Asigna usuario.
   * 
   * @param usuario
   *          usuario
   */
  public void setUsuario(Usuario usuario) {
    this.usuario = usuario;
  }

  /**
   * Devuelve criticidad.
   * 
   * @return criticidad
   */
  @Enumerated(value = EnumType.STRING)
  public Criticidad getCriticidad() {
    if (criticidad == null && getIncidencia() != null) return getIncidencia().getCriticidad();
    return criticidad;
  }

  /**
   * Asigna criticidad.
   * 
   * @param criticidad
   *          criticidad
   */
  public void setCriticidad(Criticidad criticidad) {
    this.criticidad = criticidad;
  }

  /**
   * Asigna lista.
   * 
   * @param lista
   *          lista
   */
  public void setLista(Lista lista) {
    this.lista = lista;
  }

  /**
   * Devuelve lista.
   * 
   * @return lista
   */
  public Lista getLista() {
    return lista;
  }

}
