package es.csa.fdt.cyldigital.incidencias.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.Lob;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.core.entity.File;
import es.csa.fdt.cyldigital.core.entity.FileOnDisk;
import es.csa.fdt.cyldigital.recursos.entity.Recurso;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;

/**
 * Clase Incidencia.
 * 
 * @author csa
 */
@Entity
@Table(name = "incidencias")
@AttributeOverride(name = "id", column = @Column(name = "idIncidencia"))
@Audited
public class Incidencia extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = -7648562176655166537L;

  private String descripcion;
  private String nombre;
  private CategoriaIncidencia categoriaIncidencia;
  private Date fechaIncidencia;
  private Tipo tipo;
  private Criticidad criticidad;
  private EstadoIncidencia estado;
  private Date fechaCierre;
  private Centro centro;
  private Usuario usuario; /* usuario que la abre */
  private Empleado empleado; /* responsable de la incidencia */
  /*
   * private Alcance alcance; private int tiempoEstimado;
   */
  /** ficheros. */
  protected List<File> ficheros;
  
  /** ficheros adjuntos. */
  protected List<FileOnDisk> ficherosAdjuntos;

  /** eventos incidencia. */
  protected List<EventoIncidencia> eventosIncidencia;
  
  /** recursos. */
  protected List<Recurso> recursos;

  /**
   * Enum Tipo.
   */
  public enum Tipo {
    ENTORNO,
    FUNCIONAL,
    DISENO;
  }

  /**
   * Enum Criticidad.
   */
  public enum Criticidad {
    CRITICA,
    BLOQUEANTE,
    MAYOR,
    MENOR,
    NUEVA,
    DUDAS;
  }

  /*
   * public enum Alcance { PETICION_SERVICIO,FALLO,MEJORA_FUNCIONALIDAD }
   */
  /**
   * Enum EstadoIncidencia.
   */
  public enum EstadoIncidencia {
    PENDIENTE,
    RECHAZADA,
    ANULADA,
    CORREGIDA,
    CERRADA,
    COMENTARIO,
    CURSO,
    ACEPTADA,
    INFORMACION;
  }

  /**
   * Construye un nuevo objeto de la clase
   */
  public Incidencia() {
    super();
    recursos = new ArrayList<Recurso>();
    eventosIncidencia = new ArrayList<EventoIncidencia>();
    ficheros = new ArrayList<File>();
  }

  /**
   * Devuelve descripcion.
   * 
   * @return descripcion
   */
  @Lob
  @NotNull
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getDescripcion() {
    return descripcion;
  }

  /**
   * Asigna descripcion.
   * 
   * @param descripcion
   *          descripcion
   */
  public void setDescripcion(String descripcion) {
    this.descripcion = descripcion;
  }

  /**
   * Devuelve fecha incidencia.
   * 
   * @return fecha incidencia
   */
  @NotNull
  public Date getFechaIncidencia() {
    return fechaIncidencia;
  }

  /**
   * Asigna fecha incidencia.
   * 
   * @param fechaIncidencia
   *          fecha incidencia
   */
  public void setFechaIncidencia(Date fechaIncidencia) {
    this.fechaIncidencia = fechaIncidencia;
  }

  /**
   * Devuelve fecha cierre.
   * 
   * @return fecha cierre
   */
  public Date getFechaCierre() {
    return fechaCierre;
  }

  /**
   * Asigna fecha cierre.
   * 
   * @param fechaCierre
   *          fecha cierre
   */
  public void setFechaCierre(Date fechaCierre) {
    this.fechaCierre = fechaCierre;
  }

  /**
   * Devuelve centro.
   * 
   * @return centro
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idCentro")
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Devuelve tipo.
   * 
   * @return tipo
   */
  @NotNull
  @Enumerated(value = EnumType.STRING)
  public Tipo getTipo() {
    return tipo;
  }

  /**
   * Asigna tipo.
   * 
   * @param tipo
   *          tipo
   */
  public void setTipo(Tipo tipo) {
    this.tipo = tipo;
  }

  /**
   * Devuelve criticidad.
   * 
   * @return criticidad
   */
  @NotNull
  @Enumerated(value = EnumType.STRING)
  public Criticidad getCriticidad() {
    return criticidad;
  }

  /**
   * Asigna criticidad.
   * 
   * @param criticidad
   *          criticidad
   */
  public void setCriticidad(Criticidad criticidad) {
    this.criticidad = criticidad;
  }

  /**
   * Devuelve estado.
   * 
   * @return estado
   */
  @Enumerated(value = EnumType.STRING)
  @NotNull
  public EstadoIncidencia getEstado() {
    return estado;
  }

  /**
   * Asigna estado.
   * 
   * @param estadoIncidencia
   *          estado
   */
  public void setEstado(EstadoIncidencia estadoIncidencia) {
    this.estado = estadoIncidencia;
  }

  /**
   * Devuelve nombre.
   * 
   * @return nombre
   */
  @NotNull
  @NotEmpty
  @Length(max = 200)
  public String getNombre() {
    return nombre;
  }

  /**
   * Asigna nombre.
   * 
   * @param nombre
   *          nombre
   */
  public void setNombre(String nombre) {
    this.nombre = nombre;
  }

  /**
   * Devuelve eventos incidencia.
   * 
   * @return eventos incidencia
   */
  @OneToMany(mappedBy = "incidencia", cascade = { CascadeType.PERSIST, CascadeType.MERGE, CascadeType.REFRESH })
  @OrderBy(value = "fechaEstado ASC")
  public List<EventoIncidencia> getEventosIncidencia() {
    return eventosIncidencia;
  }

  /**
   * Asigna eventos incidencia.
   * 
   * @param eventoIncidencias
   *          eventos incidencia
   */
  public void setEventosIncidencia(List<EventoIncidencia> eventoIncidencias) {
    this.eventosIncidencia = eventoIncidencias;
  }

  /**
   * Devuelve usuario.
   * 
   * @return usuario
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idUsuario")
  public Usuario getUsuario() {
    return usuario;
  }

  /**
   * Asigna usuario.
   * 
   * @param usuario
   *          usuario
   */
  public void setUsuario(Usuario usuario) {
    this.usuario = usuario;
  }

  /**
   * Devuelve categoria incidencia.
   * 
   * @return categoria incidencia
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idCategoriaIncidencia", nullable = false)
  @NotNull
  public CategoriaIncidencia getCategoriaIncidencia() {
    return categoriaIncidencia;
  }

  /**
   * Asigna categoria incidencia.
   * 
   * @param categoriaIncidencia
   *          categoria incidencia
   */
  public void setCategoriaIncidencia(CategoriaIncidencia categoriaIncidencia) {
    this.categoriaIncidencia = categoriaIncidencia;
  }

  /**
   * Devuelve empleado.
   * 
   * @return empleado
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idEmpleado")
  public Empleado getEmpleado() {
    return empleado;
  }

  /**
   * Asigna empleado.
   * 
   * @param empleado
   *          empleado
   */
  public void setEmpleado(Empleado empleado) {
    this.empleado = empleado;
  }

  /**
   * Devuelve recursos.
   * 
   * @return recursos
   */
  @ManyToMany(targetEntity = Recurso.class, cascade = { javax.persistence.CascadeType.ALL })
  @JoinTable(name = "recursos_incidencias", joinColumns = @JoinColumn(name = "idIncidencia"), inverseJoinColumns = @JoinColumn(name = "idRecurso"))
  @NotAudited
  public List<Recurso> getRecursos() {
    return recursos;
  }

  /**
   * Asigna recursos.
   * 
   * @param recursos
   *          recursos
   */
  public void setRecursos(List<Recurso> recursos) {
    this.recursos = recursos;
  }

  /**
   * Devuelve ficheros.
   * 
   * @return ficheros
   */
  @NotAudited
  @ManyToMany(targetEntity = File.class, cascade = { javax.persistence.CascadeType.ALL })
  @JoinTable(name = "incidencias_files", joinColumns = @JoinColumn(name = "idIncidencia"), inverseJoinColumns = @JoinColumn(name = "idFichero"))
  public List<File> getFicheros() {
    return ficheros;
  }

  /**
   * Asigna ficheros.
   * 
   * @param ficheros
   *          ficheros
   */
  public void setFicheros(List<File> ficheros) {
    this.ficheros = ficheros;
  }

  /**
   * Devuelve numero ficheros.
   * 
   * @return numero ficheros
   */
  @Transient
  public int getNumeroFicheros() {
    return ficheros.size();
  }

  /**
   * Checks if is comentable.
   * 
   * @return true, if is comentable
   */
  @Transient
  public boolean isComentable() {
    if (estado == EstadoIncidencia.ANULADA || estado == EstadoIncidencia.CERRADA) return false;
    else return true;
  }

  /*
   * Descomentar los siguiente cuando se apruebe lo del alcance.
   * 
   * @Enumerated(value = EnumType.STRING)
   * 
   * @NotNull public Alcance getAlcance() { return alcance; }
   * 
   * public void setAlcance(Alcance alcance) { this.alcance = alcance; }
   * 
   * public int getTiempoEstimado() { return tiempoEstimado; }
   * 
   * public void setTiempoEstimado(int tiempoEstimado) { this.tiempoEstimado = tiempoEstimado; }
   */

  /**
   * Devuelve ficheros adjuntos.
   * 
   * @return ficheros adjuntos
   */
  @NotAudited
  @ManyToMany(targetEntity = FileOnDisk.class, cascade = { javax.persistence.CascadeType.ALL })
  @JoinTable(name = "incidencias_filesondisk", joinColumns = @JoinColumn(name = "idIncidencia"), inverseJoinColumns = @JoinColumn(name = "idFileOnDisk"))
  public List<FileOnDisk> getFicherosAdjuntos() {
    return ficherosAdjuntos;
  }

  /**
   * Asigna ficheros adjuntos.
   * 
   * @param ficherosAdjuntos
   *          ficheros adjuntos
   */
  public void setFicherosAdjuntos(List<FileOnDisk> ficherosAdjuntos) {
    this.ficherosAdjuntos = ficherosAdjuntos;
  }
}
