package es.csa.fdt.cyldigital.incidencias.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.List;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.incidencias.entity.CategoriaIncidencia;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;


/**
 * Clase CategoriaIncidenciaServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class CategoriaIncidenciaServiceImpl extends AbstractServiceImpl<CategoriaIncidencia, Long> implements
    CategoriaIncidenciaService {

  private static final long serialVersionUID = -5575228180361917674L;

  private List<Empleado> empleados;
  private List<Centro> centros;

  /**
   * Construye un nuevo objeto de la clase
   */
  public CategoriaIncidenciaServiceImpl() {
    super();
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.incidencias.service.CategoriaIncidenciaService#getEmpleados()
   */
  public List<Empleado> getEmpleados() {
    if (empleados == null) empleados = findAllOrder(Empleado.class, "nombre", true);
    return empleados;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.incidencias.service.CategoriaIncidenciaService#setEmpleados(java.util.List)
   */
  public void setEmpleados(List<Empleado> empleados) {
    this.empleados = empleados;
  }

  /**
   * Devuelve centros.
   * 
   * @return centros
   */
  public List<Centro> getCentros() {
    if (centros == null) centros = findAllOrder(Centro.class, "nombre", true);
    return centros;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.incidencias.service.CategoriaIncidenciaService#persistCategoriaIncidencia(es.csa.fdt.cyldigital.incidencias.entity.CategoriaIncidencia)
   */
  public void persistCategoriaIncidencia(CategoriaIncidencia c) {

    if (!c.getEmpleados().contains(c.getResponsable())) {
      c.getEmpleados().add(c.getResponsable());
    }

    persist(c);
  }

  /**
   * Devuelve categorias disponibles.
   * 
   * @return categorias disponibles
   */
  public List<CategoriaIncidencia> getCategoriasDisponibles() {
    return findAll(CategoriaIncidencia.class);
  }

  /*
   * public void quitarUsuario(Categoria categoria, Long idEmpleado) { }
   */

}
