package es.csa.fdt.cyldigital.incidencias.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import javax.annotation.Resource;

import org.richfaces.event.UploadEvent;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.faces.util.FicheroSubidoRichFacesAdapter;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.entity.File;
import es.csa.fdt.cyldigital.core.entity.FileOnDisk;
import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.core.entity.FileOnDisk.Tipo;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.incidencias.entity.CategoriaIncidencia;
import es.csa.fdt.cyldigital.incidencias.entity.EventoIncidencia;
import es.csa.fdt.cyldigital.incidencias.entity.Incidencia;
import es.csa.fdt.cyldigital.incidencias.entity.Incidencia.EstadoIncidencia;
import es.csa.fdt.cyldigital.notificacion.NotificationService;
import es.csa.fdt.cyldigital.recursos.entity.Recurso;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado.Puesto;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario.Role;


/**
 * Clase IncidenciaServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class IncidenciaServiceImpl extends AbstractServiceImpl<Incidencia, Long> implements IncidenciaService {

  private static final long serialVersionUID = -5575228180361917674L;
  private List<FileOnDisk> lastUploaded2;
  private List<File> lastUploaded;
  private String fechaInicioLegible;
  private String fechaFinLegible;

  private List<Centro> centros;
  private List<CategoriaIncidencia> categoriaincidencias;
  private List<Empleado> empleados;

  private Date fechaInicio;
  private Date fechaFin;
  private Integer numeroIncidencias;
  private Boolean calculado;
  private Centro centro;
  
  /** recurso. */
  protected List<Recurso> recurso;
  
  /** categorias. */
  protected List<CategoriaIncidencia> categorias;

  @Resource
  private transient HashMap<String, Object> generalProperties;

  /**
   * Construye un nuevo objeto de la clase
   */
  public IncidenciaServiceImpl() {
    super();
    lastUploaded = new ArrayList<File>();
    lastUploaded2 = new ArrayList<FileOnDisk>();
    recurso = new ArrayList<Recurso>();
    categorias = new ArrayList<CategoriaIncidencia>();
    calculado = false;
  }

  /**
   * Inicializa incidencia.
   * 
   * @param incidencia
   *           incidencia
   * @param currentUser
   *           current user
   */
  public void inicializaIncidencia(Incidencia incidencia, UsuarioLdap currentUser) {
    incidencia.setEstado(EstadoIncidencia.PENDIENTE);
    incidencia.setFechaIncidencia(new Date());
    if (currentUser.getUser().getCentro() != null) incidencia.setCentro(currentUser.getUser().getCentro());
    if (incidencia.getUsuario() == null) {
      incidencia.setUsuario(currentUser.getUser());
    }

    centro = null;

    if (currentUser.getCentro() != null) {
      Centro c = find(Centro.class, currentUser.getCentro().getId());
      incidencia.setCentro(c);
      centro = c;
    } else if (generalProperties.containsKey("DEFAULT_CENTER_ID") && incidencia.getCentro() == null) {
      Centro c = find(Centro.class, Long.valueOf((String) generalProperties.get("DEFAULT_CENTER_ID")));
      if (c != null) {
        incidencia.setCentro(c);
        centro = c;
      }
    }
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.incidencias.service.IncidenciaService#getCentros()
   */
  public List<Centro> getCentros() {
    if (centros == null) centros = findAllOrder(Centro.class, "nombre", true);
    return centros;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.incidencias.service.IncidenciaService#getCategoriaincidencias()
   */
  public List<CategoriaIncidencia> getCategoriaincidencias() {
    if (categoriaincidencias == null) categoriaincidencias = findAllOrder(CategoriaIncidencia.class, "nombre", true);
    return categoriaincidencias;
  }

  /**
   * Devuelve empleados.
   * 
   * @return empleados
   */
  public List<Empleado> getEmpleados() {
    if (empleados == null) empleados = findAllOrder(Empleado.class, "nombre", true);
    return empleados;
  }

  /**
   * Devuelve recursos.
   * 
   * @return recursos
   */
  public List<Recurso> getRecursos() {
    return recurso;
  }

  /**
   * Persist incidencia.
   * 
   * @param incidencia
   *           incidencia
   * @param evento
   *           evento
   * @param currentUser
   *           current user
   * @param accion
   *           accion
   * @param sendMail2
   *           send mail2
   */
  public void persistIncidencia(Incidencia incidencia, EventoIncidencia evento, UsuarioLdap currentUser, String accion,
      boolean sendMail2) {
    if (incidencia.getEstado() == null) {
      incidencia.setEstado(evento.getEstado());
    }
    persistEventoIncidencia(incidencia, evento, currentUser, incidencia.getEstado(), false, accion, sendMail2);
  }

  /**
   * Persist evento incidencia.
   * 
   * @param incidencia
   *           incidencia
   * @param evento
   *           evento
   * @param currentUser
   *           current user
   * @param estadofinal
   *           estadofinal
   * @param accion
   *           accion
   * @param sendMail2
   *           send mail2
   */
  public void persistEventoIncidencia(Incidencia incidencia, EventoIncidencia evento, UsuarioLdap currentUser,
      EstadoIncidencia estadofinal, String accion, boolean sendMail2) {
    persistEventoIncidencia(incidencia, evento, currentUser, estadofinal, true, accion, sendMail2);
  }

  /**
   * Persist evento incidencia.
   * 
   * @param incidencia
   *           incidencia
   * @param evento
   *           evento
   * @param currentUser
   *           current user
   * @param estadofinal
   *           estadofinal
   * @param modifyUsuario
   *           modify usuario
   * @param accion
   *           accion
   * @param sendMail2
   *           send mail2
   */
  @SuppressWarnings("unchecked")
  public void persistEventoIncidencia(Incidencia incidencia, EventoIncidencia evento, UsuarioLdap currentUser,
      EstadoIncidencia estadofinal, boolean modifyUsuario, String accion, boolean sendMail2) {
    String sendmail = "";
    Empleado e = this.find(Empleado.class, currentUser.getUser().getId());
    NotificationService n = new NotificationService();
    Parameter p = findOneWhere(Parameter.class, "nombre", "pathfileondisk");

    if (!lastUploaded2.isEmpty()) {
      disableMessages();
      for (FileOnDisk f : lastUploaded2) {
        f.grabarEnDisco(p.getValor());
        persist(f);
      }

      enableMessages();
      incidencia.getFicherosAdjuntos().addAll(lastUploaded2);
    }

    if (accion.equals("nuevaincidencia")) evento.setDescripcion("Incidencia creada");

    if (accion.equals("editarincidencia")) evento.setDescripcion("Incidencia editada");

    evento.setFechaEstado(new Date());
    evento.setUsuario(e);

    if (evento.getDescripcion() != null) evento.setDescripcion(evento.getDescripcion().replaceAll("\n", "<br/>"));

    evento.setEstado(estadofinal);
    if (estadofinal != EstadoIncidencia.COMENTARIO) {
      incidencia.setEstado(estadofinal);
    }
    if (estadofinal == EstadoIncidencia.CURSO || estadofinal == EstadoIncidencia.CORREGIDA
        || estadofinal == EstadoIncidencia.INFORMACION || estadofinal == EstadoIncidencia.RECHAZADA) {
      if (modifyUsuario || incidencia.getEmpleado() == null) {
        incidencia.setEmpleado(e);
      }
    }

    if (estadofinal == EstadoIncidencia.ANULADA || estadofinal == EstadoIncidencia.CERRADA)
      incidencia.setFechaCierre(new Date());

    evento.setIncidencia(incidencia);
    evento.setCriticidad(incidencia.getCriticidad());

    incidencia.getEventosIncidencia().add(evento);
    disableMessages();
    persist(incidencia);
    enableMessages();

    // por ultimo, mandar las notificaciones, si procede.
    if (sendMail2) {
      List<Parameter> parameters = findWhere(Parameter.class, "nombre", "mail_activate");
      if (parameters.size() == 1) {
        sendmail = parameters.get(0).getValor();
      }

      if (sendmail.equals("1")) {
        n.incidenciaEnviaMensaje(this, incidencia, accion);
        addInfoMessage("mail.email");
      } else {
        addWarnMessage("mail.nonotifications");
      }
    }
  }

  /**
   * To aceptar incidencia.
   * 
   * @return estado incidencia
   */
  public EstadoIncidencia toAceptarIncidencia() {
    return EstadoIncidencia.CURSO;
    // return EstadoIncidencia.ACEPTADA
  }

  /**
   * To poner en curso incidencia.
   * 
   * @return estado incidencia
   */
  public EstadoIncidencia toPonerEnCursoIncidencia() {
    return EstadoIncidencia.CURSO;
  }

  /**
   * To rechazar incidencia.
   * 
   * @return estado incidencia
   */
  public EstadoIncidencia toRechazarIncidencia() {
    return EstadoIncidencia.RECHAZADA;
  }

  /**
   * To aceptar rechazo.
   * 
   * @return estado incidencia
   */
  public EstadoIncidencia toAceptarRechazo() {
    return EstadoIncidencia.ANULADA;
  }

  /**
   * To rechazar rechazo.
   * 
   * @return estado incidencia
   */
  public EstadoIncidencia toRechazarRechazo() {
    return EstadoIncidencia.PENDIENTE;
  }

  /**
   * To corregir incidencia.
   * 
   * @return estado incidencia
   */
  public EstadoIncidencia toCorregirIncidencia() {
    return EstadoIncidencia.CORREGIDA;
  }

  /**
   * To aceptar correcion.
   * 
   * @return estado incidencia
   */
  public EstadoIncidencia toAceptarCorrecion() {
    return EstadoIncidencia.CERRADA;
  }

  /**
   * To rechazar correcion.
   * 
   * @return estado incidencia
   */
  public EstadoIncidencia toRechazarCorrecion() {
    return EstadoIncidencia.PENDIENTE;
  }

  /**
   * To solicitar info.
   * 
   * @return estado incidencia
   */
  public EstadoIncidencia toSolicitarInfo() {
    return EstadoIncidencia.INFORMACION;
  }

  /**
   * To informacion enviada.
   * 
   * @return estado incidencia
   */
  public EstadoIncidencia toInformacionEnviada() {
    return EstadoIncidencia.PENDIENTE;
  }

  /**
   * To anadir comentario.
   * 
   * @return estado incidencia
   */
  public EstadoIncidencia toAnadirComentario() {
    return EstadoIncidencia.COMENTARIO;
  }

  /*
   * public synchronized void aniadirFichero(UploadEvent event) { File aux = new File(); aux.setFicheroSubido(new
   * FicheroSubidoRichFacesAdapter(event.getUploadItem())); lastUploaded.add(aux); }
   * 
   * public Incidencia quitarFichero(Incidencia incidencia, File fichero) { // boolean mivariable =
   * lastUploaded.contains(fichero); borrarFicheroTemporal(fichero); if (incidencia.getFicheros().contains(fichero))
   * incidencia.getFicheros().remove(fichero); if (lastUploaded.contains(fichero)) { lastUploaded.remove(fichero); }
   * return incidencia; }
   */

  /**
   * Aniadir fichero.
   * 
   * @param event
   *           event
   */
  public synchronized void aniadirFichero(UploadEvent event) {
    FileOnDisk aux = new FileOnDisk();
    aux.setFicheroSubido(new FicheroSubidoRichFacesAdapter(event.getUploadItem()));
    aux.setTipo(Tipo.INCIDENCIA);
    aux.setNombreOriginal(aux.getFicheroSubido().getNombre());
    lastUploaded2.add(aux);
  }

  /**
   * Quitar fichero.
   * 
   * @param i
   *           i
   * @param file
   *           file
   */
  public void quitarFichero(Incidencia i, FileOnDisk file) {
    lastUploaded2.remove(file);
    i.getFicherosAdjuntos().remove(file);
  }

  /**
   * Borrar fichero temporal.
   * 
   * @param file
   *           file
   */
  public void borrarFicheroTemporal(File file) {
    if (file != null && file.getFicheroSubido() != null) {
      file.getFicheroSubido().deleteFichero();
    }
  }

  /**
   * Devuelve last uploaded.
   * 
   * @return last uploaded
   */
  public List<File> getLastUploaded() {
    return lastUploaded;
  }

  /**
   * Asigna fecha inicio.
   * 
   * @param fechaInicio
   *          fecha inicio
   */
  public void setFechaInicio(Date fechaInicio) {
    this.fechaInicio = fechaInicio;
  }

  /**
   * Devuelve fecha inicio.
   * 
   * @return fecha inicio
   */
  public Date getFechaInicio() {
    return fechaInicio;
  }

  /**
   * Asigna fecha fin.
   * 
   * @param fechaFin
   *          fecha fin
   */
  public void setFechaFin(Date fechaFin) {
    this.fechaFin = fechaFin;
  }

  /**
   * Devuelve fecha fin.
   * 
   * @return fecha fin
   */
  public Date getFechaFin() {
    return fechaFin;
  }

  /**
   * Calcular numero incidencias.
   */
  public void calcularNumeroIncidencias() {
    if (getFechaInicio() != null) {
      if (getFechaFin() != null) {
        List<Incidencia> incidencias = findAll();
        List<Incidencia> finales = new ArrayList<Incidencia>();
        Date fechaFin = new Date();
        Calendar c = new GregorianCalendar();
        c.setTime(getFechaFin());
        fechaFin = c.getTime();
        calculado = true;

        if (getCentro() != null) {
          Iterator<Incidencia> itr = incidencias.iterator();
          while (itr.hasNext()) {
            Incidencia incidencia = new Incidencia();
            incidencia = itr.next();
            if (incidencia.getCentro() != null) {
              Centro centroAux = incidencia.getCentro();
              if (incidencia.getFechaIncidencia().compareTo(getFechaInicio()) > 0
                  && incidencia.getFechaIncidencia().compareTo(fechaFin) < 0) {
                if (centroAux.equals(getCentro())) {
                  finales.add(incidencia);
                }
              }
            }
          }
        } else {
          Iterator<Incidencia> itr = incidencias.iterator();
          while (itr.hasNext()) {
            Incidencia incidencia = new Incidencia();
            incidencia = itr.next();
            if (incidencia.getFechaIncidencia().compareTo(getFechaInicio()) > 0
                && incidencia.getFechaIncidencia().compareTo(fechaFin) < 0) {
              finales.add(incidencia);
            }
          }
        }

        Integer numero = finales.size();
        setNumeroIncidencias(numero);
      } else {
        setNumeroIncidencias(0);
      }
    } else {
      setNumeroIncidencias(0);
    }
  }

  /**
   * Asigna numero incidencias.
   * 
   * @param numeroIncidencias
   *          numero incidencias
   */
  public void setNumeroIncidencias(Integer numeroIncidencias) {
    this.numeroIncidencias = numeroIncidencias;
  }

  /**
   * Devuelve numero incidencias.
   * 
   * @return numero incidencias
   */
  public Integer getNumeroIncidencias() {
    return numeroIncidencias;
  }

  /**
   * Devuelve fecha inicio legible.
   * 
   * @return fecha inicio legible
   */
  public String getFechaInicioLegible() {
    String aux;
    if (fechaInicio != null) {
      DateFormat df = new SimpleDateFormat("dd/MM/yyyy");
      aux = df.format(fechaInicio);
      return aux;
    }
    return "";
  }

  /**
   * Asigna fecha inicio legible.
   * 
   * @param fechaInicioLegible
   *          fecha inicio legible
   */
  public void setFechaInicioLegible(String fechaInicioLegible) {
    this.fechaInicioLegible = fechaInicioLegible;
  }

  /**
   * Asigna fecha fin legible.
   * 
   * @param fechaFinLegible
   *          fecha fin legible
   */
  public void setFechaFinLegible(String fechaFinLegible) {
    this.fechaFinLegible = fechaFinLegible;
  }

  /**
   * Devuelve fecha fin legible.
   * 
   * @return fecha fin legible
   */
  public String getFechaFinLegible() {
    String aux;
    if (fechaFin != null) {
      DateFormat df = new SimpleDateFormat("dd/MM/yyyy");
      aux = df.format(fechaFin);
      return aux;
    }
    return "";
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Devuelve centro.
   * 
   * @return centro
   */
  public Centro getCentro() {
    return centro;
  }

  /**
   * Añade recurso.
   * 
   * @param rec
   *           rec
   * @param incidencia
   *           incidencia
   */
  public void addRecurso(Recurso rec, Incidencia incidencia) {
    if (!incidencia.getRecursos().contains(rec)) incidencia.getRecursos().add(rec);

  }

  /**
   * Quitar recurso.
   * 
   * @param rec
   *           rec
   * @param incidencia
   *           incidencia
   */
  public void quitarRecurso(Recurso rec, Incidencia incidencia) {
    incidencia.getRecursos().remove(rec);
  }

  /**
   * Apply filter owner.
   * 
   * @param usuario
   *           usuario
   * @param id
   *           id
   * @return true, if successful
   */
  public boolean applyFilterOwner(UsuarioLdap usuario, long id) {
    // final Logger logger = Logger.getLogger(IncidenciaServiceImpl.class.getName());
    Empleado e = this.find(Empleado.class, usuario.getUser().getId());
    Incidencia i = this.find(Incidencia.class, id);

    if (!usuario.applyFilterCentro() || e.getCategoriasIncidencia().contains(i.getCategoriaIncidencia())
        || i.getUsuario().getId() == usuario.getUser().getId() || e.isResponsableIncidencias()) {
      return true;
    } else {
      enableMessages();
      addErrorMessage("incidencia.accesorestringido");
      return false;
    }
  }

  /**
   * Es empleado categoria.
   * 
   * @param incidencia
   *           incidencia
   * @param usuario
   *           usuario
   * @return true, if successful
   */
  public boolean esEmpleadoCategoria(Incidencia incidencia, UsuarioLdap usuario) {
    Empleado e = this.find(Empleado.class, usuario.getUser().getId());

    if (incidencia.getCategoriaIncidencia().getEmpleados().contains(e)) {
      return true;
    } else {
      return false;
    }
  }

  /**
   * Send mail new incidencia.
   * 
   * @param id
   *           id
   * @param nuevaincidencia
   *           nuevaincidencia
   */
  @SuppressWarnings("unchecked")
  public void sendMailNewIncidencia(Long id, boolean nuevaincidencia) {
    String sendmail = "";
    Incidencia i = this.find(id);

    if (nuevaincidencia) {
      NotificationService n = new NotificationService();

      List<Parameter> parameters = findWhere(Parameter.class, "nombre", "mail_activate");
      if (parameters.size() == 1) {
        sendmail = parameters.get(0).getValor();
      }

      if (sendmail.equals("1")) {
        n.incidenciaEnviaMensaje(this, i, "nuevaincidencia");
        addInfoMessage("mail.email");
      } else {
        addWarnMessage("mail.nonotifications");
      }
    }
  }

  /**
   * Asigna calculado.
   * 
   * @param calculado
   *          calculado
   */
  public void setCalculado(Boolean calculado) {
    this.calculado = calculado;
  }

  /**
   * Devuelve calculado.
   * 
   * @return calculado
   */
  public Boolean getCalculado() {
    return calculado;
  }

  /**
   * Check user allowed.
   * 
   * @param i
   *           i
   * @param u
   *           u
   * @return boolean
   */
  public Boolean checkUserAllowed(Incidencia i, Usuario u) {
    if (u.getRolesList().contains(Role.ROLE_ADMIN) /*|| u.getRolesList().contains(Role.ROLE_TRAINER_COORDINATOR)*/
        || u.getRolesList().contains(Role.ROLE_AREA_MANAGER) /*|| u.getRolesList().contains(Role.ROLE_INCIDENCE_FUNDETEC)
        || u.getRolesList().contains(Role.ROLE_INCIDENCE_VIEW_ALL)*/
        || u.getRolesList().contains(Role.ROLE_DIGITAL_ALLY) ) return true;
    else {
      Empleado e = this.find(Empleado.class, u.getId());
      if (u.getRolesList().contains(Role.ROLE_AREA_MANAGER) || u.getRolesList().contains(Role.ROLE_DIGITAL_ALLY)) {
        if (i.getCentro() == null || i.getCentro() == u.getCentro() || i.getUsuario() == u
            || e.getCategoriasIncidencia().contains(i.getCategoriaIncidencia())) return true;
      } else {
        if (i.getUsuario() == u || e.getCategoriasIncidencia().contains(i.getCategoriaIncidencia())) return true;
      }
    }
    return false;
  }

  /**
   * Devuelve next incidencia.
   * 
   * @param u
   *           u
   * @param i
   *           i
   * @param next
   *           next
   * @return next incidencia
   */
  public Incidencia getNextIncidencia(UsuarioLdap u, Incidencia i, Boolean next) {
    List<Incidencia> incidencias;

    if (next) incidencias = findAllOrder("id", true);
    else incidencias = findAllOrder("id", false);

    int actual = incidencias.indexOf(i);

    for (int j = actual + 1; j < incidencias.size(); j++) {
      if (checkUserAllowed(incidencias.get(j), u.getUser())) return incidencias.get(j);
    }

    return null;
  }

  /**
   * Devuelve categorias.
   * 
   * @param u
   *           u
   * @return categorias
   */
  public List<CategoriaIncidencia> getCategorias(UsuarioLdap u) {
    if (categorias.isEmpty()) {
      Empleado e = findOneWhere(Empleado.class, "id", u.getUser().getId());

      if (e.getPuesto().equals(Puesto.ADMIN) || e.isResponsableIncidencias()) {
        categorias = findAll(CategoriaIncidencia.class);
      } else {
        List<CategoriaIncidencia> aux = findAll(CategoriaIncidencia.class);
        for (CategoriaIncidencia c : aux) {
          if (c.isEspecial()) if (e.getCategoriasIncidencia().contains(c)) categorias.add(c);

          if (!c.isEspecial()) categorias.add(c);
        }
      }
    }
    return categorias;
  }

  /**
   * Devuelve last uploaded2.
   * 
   * @return last uploaded2
   */
  public List<FileOnDisk> getLastUploaded2() {
    return lastUploaded2;
  }

  /**
   * Asigna last uploaded2.
   * 
   * @param lastUploaded2
   *          last uploaded2
   */
  public void setLastUploaded2(List<FileOnDisk> lastUploaded2) {
    this.lastUploaded2 = lastUploaded2;
  }
}
