package es.csa.fdt.cyldigital.incidencias.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.Date;
import java.util.List;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.commons.persistence.browser.jpaql.FilterJPAQLByQuery;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.centros.service.CentroServiceImpl;
import es.csa.fdt.cyldigital.core.service.BrowserFilteredByActive;
import es.csa.fdt.cyldigital.incidencias.entity.CategoriaIncidencia;
import es.csa.fdt.cyldigital.incidencias.entity.Incidencia;
import es.csa.fdt.cyldigital.incidencias.entity.Incidencia.Criticidad;
import es.csa.fdt.cyldigital.incidencias.entity.Incidencia.EstadoIncidencia;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado.Puesto;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario.Role;

/**
 * Servicio del Browser de Incidencias.
 * 
 * @author csa
 */
@Service
@Repository
public class IncidenciasBrowserImpl extends BrowserFilteredByActive<Incidencia> implements
    IncidenciasBrowser<CriterionJPAQL, String> {
  private static final long serialVersionUID = -7227366052245717441L;

  private String customFilterProperty;

  @SuppressWarnings("unused")
  // private long idCentro;
  private Criticidad criticidad;
  private EstadoIncidencia estado;
  private CategoriaIncidencia categoria;
  /* private Alcance alcance; */
  private Date fechaIni;
  private Date fechaFin;
  private Centro centro;

  /*
   * 
   * public long getIdCentro() { return centro != null ? centro.getId() : 0; }
   * 
   * public void setIdCentro(long idCentro) { this.idCentro = idCentro; }
   */
  /**
   * Devuelve criticidad.
   * 
   * @return criticidad
   */
  public Criticidad getCriticidad() {
    return criticidad != null ? criticidad : null;
  }

  /**
   * Asigna criticidad.
   * 
   * @param criticidad
   *          criticidad
   */
  public void setCriticidad(Criticidad criticidad) {
    this.criticidad = criticidad;
  }

  /**
   * Devuelve estado.
   * 
   * @return estado
   */
  public EstadoIncidencia getEstado() {
    return estado != null ? estado : null;
  }

  /**
   * Asigna estado.
   * 
   * @param estado
   *          estado
   */
  public void setEstado(EstadoIncidencia estado) {
    this.estado = estado;
  }

  /**
   * Devuelve fecha ini.
   * 
   * @return fecha ini
   */
  public Date getFechaIni() {
    return fechaIni != null ? fechaIni : null;
  }

  /**
   * Asigna fecha ini.
   * 
   * @param fechaIni
   *          fecha ini
   */
  public void setFechaIni(Date fechaIni) {
    this.fechaIni = fechaIni;
  }

  /**
   * Devuelve fecha fin.
   * 
   * @return fecha fin
   */
  public Date getFechaFin() {

    return fechaFin != null ? fechaFin : null;
  }

  /**
   * Asigna fecha fin.
   * 
   * @param fechaFin
   *          fecha fin
   */
  public void setFechaFin(Date fechaFin) {
    this.fechaFin = fechaFin;
  }

  /**
   * Devuelve categoria.
   * 
   * @return categoria
   */
  public CategoriaIncidencia getCategoria() {
    return categoria;
  }

  /**
   * Asigna categoria.
   * 
   * @param categoria
   *          categoria
   */
  public void setCategoria(CategoriaIncidencia categoria) {
    this.categoria = categoria;
  }

  /**
   * Apply custom filter.
   * 
   * @param user
   *           user
   * @param servicio
   *           servicio
   */
  public void applyCustomFilter(UsuarioLdap user, CentroServiceImpl servicio) {
    String query = "nombre LIKE ? OR descripcion LIKE ? OR concat(trim(usuario.nombre),concat(' ',concat(trim(usuario.apellido1),concat(' ',trim(usuario.apellido2))))) LIKE ?";
    Long id = null;
    clearFilters(user, servicio);
    if (getCustomFilterProperty() != null && !getCustomFilterProperty().trim().isEmpty()) {
      try {
        id = Long.parseLong(customFilterProperty);
      } catch (Exception e) {}

      FilterJPAQLByQuery<Incidencia> filter = new FilterJPAQLByQuery<Incidencia>();
      if (id != null) {
        query += " OR id = ?";
        Object[] params = { "%" + customFilterProperty + "%", "%" + customFilterProperty + "%",
            "%" + customFilterProperty + "%", id };
        filter.getCriterion().addParameters(params);
      } else {
        Object[] params = { "%" + customFilterProperty + "%", "%" + customFilterProperty + "%",
            "%" + customFilterProperty + "%" };
        filter.getCriterion().addParameters(params);
      }
      filter.getCriterion().setQuery(query);
      filters.add(filter);
    }
    setCustomFilterProperty("");
  }

  /**
   * Construye un nuevo objeto de la clase
   */
  public IncidenciasBrowserImpl() {
    super();
    getSort().setSortAsc(false);
    getSort().setSortProperty("id");
  }

  /**
   * Filter criticidad.
   */
  public void filterCriticidad() {
    removeFilterCriticidad();
    if (getCriticidad() != null) filterByCriticidad(getCriticidad());
  }

  private void removeFilterCriticidad() {
    int i = -1;
    for (Filter<CriterionJPAQL, Incidencia> f : getFilters())
      if (f.getCriterion().getQuery().contains("criticidad")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
  }

  /**
   * Filter by criticidad.
   * 
   * @param criticidad
   *           criticidad
   */
  public void filterByCriticidad(Criticidad criticidad) {
    String query = "";

    boolean flag = false;
    FilterJPAQLByQuery<Incidencia> filter = new FilterJPAQLByQuery<Incidencia>();

    query += " criticidad = ? ";
    filter.getCriterion().addParameter(criticidad);

    for (Filter<CriterionJPAQL, Incidencia> f : getFilters())
      if (f.getCriterion().getQuery().contains("criticidad")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }

  /*
   * RECORDAR AÑADIR ESTO AL TRANSITION SEARCH. public void filterAlcance() { removeFilterAlcance(); if (getAlcance() !=
   * null) filterByAlcance(getAlcance()); }
   * 
   * private void removeFilterAlcance() { int i = -1; for (Filter<CriterionJPAQL, Incidencia> f : getFilters()) if
   * (f.getCriterion().getQuery().contains("alcance")) i = getFilters().indexOf(f); if (i > -1) getFilters().remove(i);
   * }
   * 
   * public void filterByAlcance(Alcance alcance) { String query = "";
   * 
   * boolean flag = false; FilterJPAQLByQuery<Incidencia> filter = new FilterJPAQLByQuery<Incidencia>();
   * 
   * query += " alcance = ? "; filter.getCriterion().addParameter(alcance);
   * 
   * for (Filter<CriterionJPAQL, Incidencia> f : getFilters()) if (f.getCriterion().getQuery().contains("alcance")) flag
   * = true;
   * 
   * if (!flag) { filter.getCriterion().setQuery(query); getFilters().add(filter); } }
   */
  /**
   * Filter categoria.
   */
  public void filterCategoria() {
    removeFilterCategoria();
    if (getCategoria() != null) filterByCategoria(getCategoria());
  }

  private void removeFilterCategoria() {
    int i = -1;
    for (Filter<CriterionJPAQL, Incidencia> f : getFilters())
      if (f.getCriterion().getQuery().contains("categoriaIncidencia.id")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
  }

  /**
   * Filter by categoria.
   * 
   * @param categoria
   *           categoria
   */
  public void filterByCategoria(CategoriaIncidencia categoria) {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<Incidencia> filter = new FilterJPAQLByQuery<Incidencia>();

    query += " categoriaIncidencia.id = ? ";
    filter.getCriterion().addParameter(categoria.getId());

    for (Filter<CriterionJPAQL, Incidencia> f : getFilters())
      if (f.getCriterion().getQuery().contains("categoriaIncidencia.id")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }

  /**
   * Filter estado.
   */
  public void filterEstado() {
    removeFilterEstado();
    if (getEstado() != null) filterByEstado(getEstado());
  }

  /**
   * Filter by estado.
   * 
   * @param estado
   *           estado
   */
  public void filterByEstado(EstadoIncidencia estado) {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<Incidencia> filter = new FilterJPAQLByQuery<Incidencia>();

    query += " estado = ? ";
    filter.getCriterion().addParameter(estado);

    for (Filter<CriterionJPAQL, Incidencia> f : getFilters())
      if (f.getCriterion().getQuery().contains("estado")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }

  /**
   * Filter fecha.
   */
  public void filterFecha() {
    removeFilterFecha();
    if (getFechaIni() != null && getFechaFin() != null) {
      filterByFecha(getFechaIni(), getFechaFin());
    }
  }

  private void removeFilterFecha() {
    int i = -1;
    for (Filter<CriterionJPAQL, Incidencia> f : getFilters())
      if (f.getCriterion().getQuery().contains("fechaIncidencia")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
  }

  /**
   * Filter by fecha.
   * 
   * @param fechaIni
   *           fecha ini
   * @param fechaFin
   *           fecha fin
   */
  public void filterByFecha(Date fechaIni, Date fechaFin) {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<Incidencia> filter = new FilterJPAQLByQuery<Incidencia>();

    /* query += " fechaIncidencia = ? "; */
    query += " DATE(fechaIncidencia) >= ? and DATE(fechaIncidencia) <= ?";

    filter.getCriterion().addParameter(fechaIni);
    filter.getCriterion().addParameter(fechaFin);

    for (Filter<CriterionJPAQL, Incidencia> f : getFilters())
      if (f.getCriterion().getQuery().contains("fechaIncidencia")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }

  /**
   * Devuelve custom filter property.
   * 
   * @return custom filter property
   */
  public String getCustomFilterProperty() {
    return customFilterProperty;
  }

  /**
   * Asigna custom filter property.
   * 
   * @param customFilter
   *          custom filter property
   */
  public void setCustomFilterProperty(String customFilter) {
    this.customFilterProperty = customFilter;
  }

  /**
   * Asigna filter estado.
   * 
   * @param user
   *          filter estado
   */
  public void setFilterEstado(UsuarioLdap user) {
    boolean isResponsableFinal = user.getUser() instanceof Empleado
        && ((Empleado) user.getUser()).isResponsableIncidencias();
    if (isResponsableFinal) {
      removeFilterEstado();
      filterByEstado(user);
    }
  }

  private void removeFilterEstado() {
    int i = -1;
    for (Filter<CriterionJPAQL, Incidencia> f : getFilters())
      if (f.getCriterion().getQuery().contains("estado")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
  }

  /**
   * Filter by estado.
   * 
   * @param user
   *           user
   */
  public void filterByEstado(UsuarioLdap user) {
    String query = "";
    boolean flag = false;
    Filter<CriterionJPAQL, Incidencia> filter = new FilterJPAQLByQuery<Incidencia>();

    query += " estado = ? ";
    filter.getCriterion().addParameter(EstadoIncidencia.RECHAZADA);

    for (Filter<CriterionJPAQL, Incidencia> f : getFilters())
      if (f.getCriterion().getQuery().contains("estado")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }

  /**
   * Asigna filter categoriayusuario.
   * 
   * @param user
   *           user
   * @param c
   *           c
   */
  public void setFilterCategoriayusuario(UsuarioLdap user, CentroServiceImpl c) {
    boolean isResponsableFinal = user.getUser() instanceof Empleado
        && ((Empleado) user.getUser()).isResponsableIncidencias();

    if (user.applyFilterCategoriayusuario() && !isResponsableFinal) {
      removeFilterCategoriayusuario();
      filterByCategoriayusuario(user, c);
    }
  }

  private void removeFilterCategoriayusuario() {
    int i = -1;
    for (Filter<CriterionJPAQL, Incidencia> f : getFilters())
      if (f.getCriterion().getQuery().contains("categoriaIncidencia")
          || f.getCriterion().getQuery().contains("usuario.id")) i = getFilters().indexOf(f);

    if (i > -1) getFilters().remove(i);

  }

  /**
   * Filter by categoriayusuario.
   * 
   * @param user
   *           user
   * @param servicio
   *           servicio
   */
  public void filterByCategoriayusuario(UsuarioLdap user, CentroServiceImpl servicio) {
    String query = "";
    Empleado e = new Empleado();

    if (user.getUser() instanceof Empleado) e = servicio.find(Empleado.class, user.getUser().getId());

    Filter<CriterionJPAQL, Incidencia> filter = new FilterJPAQLByQuery<Incidencia>();
    List<CategoriaIncidencia> categorias = servicio.findAll(CategoriaIncidencia.class);
    query += " ( usuario.id = ? or empleado.id = ?";
    filter.getCriterion().addParameter(user.getUser().getId());
    filter.getCriterion().addParameter(user.getUser().getId());

    for (CategoriaIncidencia c : e.getCategoriasIncidencia()) {
      query += " or categoriaIncidencia = ? ";
      filter.getCriterion().addParameter(c);
    }

    query += " ) ";
    for (CategoriaIncidencia c : categorias) {
      if (c.isEspecial()) {
        if (!e.getCategoriasIncidencia().contains(c)) {
          query += " and categoriaIncidencia <> ? ";
          filter.getCriterion().addParameter(c);
        }
      }
    }

    filter.getCriterion().setQuery(query);
    getFilters().add(filter);
  }

  /**
   * Asigna filter responsable centro.
   * 
   * @param user
   *           user
   * @param servicio
   *           servicio
   */
  public void setFilterResponsableCentro(UsuarioLdap user, CentroServiceImpl servicio) {
    boolean isResponsableFinal = user.getUser() instanceof Empleado
        && ((Empleado) user.getUser()).isResponsableIncidencias();
    Empleado e;
    e = (Empleado) user.getUser();

    if (e.getPuesto() == Puesto.AREA_MANAGER && !isResponsableFinal) {
      removeFilterResponsableCentro();
      filterByResponsableCentro(user, servicio);
    }
  }

  private void removeFilterResponsableCentro() {
    int i = -1;
    for (Filter<CriterionJPAQL, Incidencia> f : getFilters())
      if (f.getCriterion().getQuery().contains("centro") || f.getCriterion().getQuery().contains("usuario.id"))
        i = getFilters().indexOf(f);

    if (i > -1) getFilters().remove(i);

  }

  /**
   * Filter by responsable centro.
   * 
   * @param user
   *           user
   * @param servicio
   *           servicio
   */
  public void filterByResponsableCentro(UsuarioLdap user, CentroServiceImpl servicio) {
    String query = "";
    Filter<CriterionJPAQL, Incidencia> filter = new FilterJPAQLByQuery<Incidencia>();

    Empleado e = servicio.find(Empleado.class, user.getUser().getId());
    List<CategoriaIncidencia> categorias = servicio.findAll(CategoriaIncidencia.class);

    query += "( usuario.id = ? ";
    filter.getCriterion().addParameter(user.getUser().getId());

    query += "or centro = ? ";
    filter.getCriterion().addParameter(user.getUser().getCentro());

    query += "or centro is null ";

    for (CategoriaIncidencia c : e.getCategoriasIncidencia()) {
      query += "or categoriaIncidencia = ? ";
      filter.getCriterion().addParameter(c);
    }

    query += ")";

    for (CategoriaIncidencia c : categorias) {
      if (c.isEspecial()) if (!e.getCategoriasIncidencia().contains(c)) {
        query += " and categoriaIncidencia <> ? ";
        filter.getCriterion().addParameter(c);
      }
    }

    filter.getCriterion().setQuery(query);
    getFilters().add(filter);
  }

  /**
   * Select centro.
   * 
   * @param u
   *           u
   */
  public void selectCentro(UsuarioLdap u) {
    if (/*u.getRolesList().contains(Role.ROLE_INFORMER) ||*/ u.getRolesList().contains(Role.ROLE_TRAINER)) {
      /* this.setFilterCentro(u); */
    }
  }

  /**
   * Asigna filter categorias especiales.
   * 
   * @param user
   *           user
   * @param c
   *           c
   */
  public void setFilterCategoriasEspeciales(UsuarioLdap user, CentroServiceImpl c) {

    Empleado e = new Empleado();
    if (user.getUser() instanceof Empleado) e = c.find(Empleado.class, user.getUser().getId());

    if (e.getPuesto() == Puesto.MANAGER || e.getPuesto() == Puesto.TRAINER_COORDINATOR && !e.isResponsableIncidencias()) {
      removeFilterCategoriasEspeciales();
      filterByCategoriasEspeciales(user, c);
    }
  }

  private void removeFilterCategoriasEspeciales() {
    int i = -1;
    for (Filter<CriterionJPAQL, Incidencia> f : getFilters())
      if (f.getCriterion().getQuery().contains("categoriaIncidencia")) i = getFilters().indexOf(f);

    if (i > -1) getFilters().remove(i);

    i = -1;

    for (Filter<CriterionJPAQL, Incidencia> f : getFilters())
      if (f.getCriterion().getQuery().contains("usuario.id")) i = getFilters().indexOf(f);

    if (i > -1) getFilters().remove(i);
  }

  /**
   * Filter by categorias especiales.
   * 
   * @param user
   *           user
   * @param servicio
   *           servicio
   */
  public void filterByCategoriasEspeciales(UsuarioLdap user, CentroServiceImpl servicio) {
    String query = "";

    Empleado e = new Empleado();
    if (user.getUser() instanceof Empleado) e = servicio.find(Empleado.class, user.getUser().getId());

    List<CategoriaIncidencia> categorias = servicio.findAll(CategoriaIncidencia.class);
    Filter<CriterionJPAQL, Incidencia> filter = new FilterJPAQLByQuery<Incidencia>();

    for (CategoriaIncidencia c : categorias) {
      if (c.isEspecial()) {
        if (!c.getEmpleados().contains(e)) {
          if (!query.isEmpty()) query += " and ";
          query += " categoriaIncidencia <> ? ";
          filter.getCriterion().addParameter(c);
        }
      }
    }

    if (!query.isEmpty()) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }

  /**
   * Clear filters.
   * 
   * @param user
   *           user
   * @param servicio
   *           servicio
   */
  public void clearFilters(UsuarioLdap user, CentroServiceImpl servicio) {
    super.clearFilters();

    setFilterCategoriayusuario(user, servicio);
    setFilterResponsableCentro(user, servicio);
    setFilterCategoriasEspeciales(user, servicio);
  }

  /*
   * public Alcance getAlcance() { return alcance; }
   * 
   * public void setAlcance(Alcance alcance) { this.alcance = alcance; }
   */
  /**
   * Devuelve centro.
   * 
   * @return centro
   */
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Asigna filter centro.
   */
  public void setFilterCentro() {
    removeFilterCentro();
    if (getCentro() != null) filterByCentro(getCentro());
  }

  private void removeFilterCentro() {
    int i = -1;
    for (Filter<CriterionJPAQL, Incidencia> f : getFilters())
      if (f.getCriterion().getQuery().contains("centro")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
  }

  /**
   * Filter by centro.
   * 
   * @param centro
   *           centro
   */
  public void filterByCentro(Centro centro) {
    String query = "";

    boolean flag = false;
    FilterJPAQLByQuery<Incidencia> filter = new FilterJPAQLByQuery<Incidencia>();

    query += " centro = ? ";
    filter.getCriterion().addParameter(centro);

    for (Filter<CriterionJPAQL, Incidencia> f : getFilters())
      if (f.getCriterion().getQuery().contains("centro")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }

  /**
   * Filter.
   */
  public void filter() {
    removeFilterCentro();
    if (getCentro() != null) filterByCentro(getCentro());
  }

}
