package es.csa.fdt.cyldigital.informes;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import es.csa.commons.faces.util.PredefinedLocaleResolver;
import es.csa.fdt.cyldigital.calendario.entity.FestivoCalendarItem;
import es.csa.fdt.cyldigital.centros.entity.BP;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.centros.entity.BP.Tipo;
import es.csa.fdt.cyldigital.servicios.entity.Actividad;
import es.csa.fdt.cyldigital.servicios.entity.Curso;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaTaller;
import es.csa.fdt.cyldigital.servicios.entity.PracticaTaller;
import es.csa.fdt.cyldigital.servicios.entity.Servicio;
import es.csa.fdt.cyldigital.servicios.entity.Taller;
import es.csa.fdt.cyldigital.servicios.entity.Wifi;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado.Puesto;

/**
 * Clase CuadroDeMandos.
 * 
 * @author csa
 */
public class CuadroDeMandos {

  private Integer numeroUsuariosMatriculados;
  private Integer numeroBajasSemana = 0;
  private Integer numeroBajasMes = 0;
  private Integer numeroBajasAnyo = 0;
  private float numeroAlumnosMatriculados = 0;
  private Integer numeroAlumnosAsistentes = 0;
  private float horasFormacion = 0;
  private Integer numeroAlumnosEspera = 0;
  private Integer cursosCerrados = 0;
  private Integer talleresCerrados = 0;
  private double porcAsistentes = 0;
  private double porcOcupacion = 0;
  private Float mediaEdad;
  private Integer horasCursosTerceros = 0;
  private Integer alumnosCursosTerceros = 0;
  private Integer repetidores = 0;
  private double evaluacionMedia;

  /**
   * Checks if is cerrado.
   * 
   * @param act
   *           act
   * @param curso
   *           curso
   * @return true, if is cerrado
   */
  public boolean isCerrado(List<Actividad> act, Curso curso) {
    Iterator<Actividad> it = act.iterator();
    while (it.hasNext()) {
      Actividad actividad = it.next();
      if (actividad.getId() == curso.getId()) {
        return curso.isCerrada();
      }
    }
    return false;
  }

  /**
   * Calcular porcentaje double.
   * 
   * @param bp
   *           bp
   * @param valor
   *           valor
   * @return string
   */
  public String calcularPorcentajeDouble(Integer bp, double valor) {
    String sol = "";
    if (bp != null) {
      if (bp == 0) {
        if (valor == 0) {
          return "0%";
        }
        bp = 1;
      }
      float p = ((float) valor) * 100;
      float a = p / bp;
      a = a - 100;
      sol = (String.format("%.2f", a));
      sol = sol + "%";
      return sol;
    } else {
      sol = "--";
      return sol;
    }
  }

  /**
   * Devolver color.
   * 
   * @param minimo
   *           minimo
   * @param maximo
   *           maximo
   * @param porcentaje
   *           porcentaje
   * @param signo
   *           signo
   * @return string
   */
  public String devolverColor(int minimo, int maximo, Integer porcentaje, boolean signo) {
    if (porcentaje != null) {
      if (signo) {
        if (porcentaje < minimo) {
          return new String("ROJO");
        } else if (porcentaje > maximo) {
          return new String("VERDE");
        } else {
          return new String("AMARILLO");
        }
      } else {
        if (porcentaje < minimo) {
          return new String("VERDE");
        } else if (porcentaje > maximo) {
          return new String("ROJO");
        } else {
          return new String("AMARILLO");
        }
      }
    } else {
      return new String("NULO");
    }
  }

  /**
   * Calcular porcentaje numero.
   * 
   * @param bp
   *           bp
   * @param valor
   *           valor
   * @param isPorcentaje
   *           is porcentaje
   * @return integer
   */
  public Integer calcularPorcentajeNumero(Integer bp, int valor, boolean isPorcentaje) {
    if (bp != null) {
      if (bp == 0) {
        if (valor == 0) {
          return 0;
        }
        bp = 1;
      }
      Integer p = valor * 100;
      int a = p / bp;
      if (isPorcentaje) {
        a = a - 100;
      }
      return a;
    } else {
      return null;
    }
  }

  /**
   * Calcular porcentaje.
   * 
   * @param bp
   *           bp
   * @param valor
   *           valor
   * @param isPorcentaje
   *           is porcentaje
   * @return string
   */
  public String calcularPorcentaje(Integer bp, int valor, boolean isPorcentaje) {
    String sol = "";
    if (bp != null) {
      if (bp == 0) {
        if (valor == 0) {
          return "0%";
        }
        bp = 1;
      }
      Integer p = valor * 100;
      int a = p / bp;
      if (isPorcentaje) {
        a = a - 100;
      }
      if (a > 100) {
        sol = "> 100%";
      } else {
        sol = String.valueOf(a) + "%";
      }
      return sol;
    } else {
      sol = "--";
      return sol;
    }
  }

  /**
   * Devolver bp.
   * 
   * @param tipo
   *           tipo
   * @param entity
   *           entity
   * @param centro
   *           centro
   * @return bP
   */
  @SuppressWarnings("unchecked")
  public BP devolverBP(Tipo tipo, EntityManager entity, Centro centro) {
    BP bp = new BP();
    List<BP> lista = new ArrayList<BP>();
    if (centro != null) {
      Query q = entity.createQuery("Select e from es.csa.fdt.cyldigital.centros.entity.BP e where "
          + "tipo= ? and centro = ?");
      q.setParameter(1, tipo);
      q.setParameter(2, centro);
      lista = q.getResultList();
    } else {
      Query q = entity.createQuery("Select e from es.csa.fdt.cyldigital.centros.entity.BP e where "
          + "tipo= ? and centro is null");
      q.setParameter(1, tipo);
      lista = q.getResultList();
    }
    if (lista.size() != 0) {
      bp = lista.get(0);
      return bp;
    } else {
      bp.setBpSemanal(null);
      bp.setBpMensual(null);
      bp.setBpAnual(null);
      bp.setMaximo(5);
      bp.setMinimo(-5);
      bp.setCentro(centro);
      bp.setTipo(tipo);
      return bp;
    }
  }

  /**
   * Altas medias diarias semana.
   * 
   * @param entity
   *           entity
   * @param centro
   *           centro
   * @return double
   */
  @SuppressWarnings("unchecked")
  public double altasMediasDiariasSemana(EntityManager entity, Centro centro) {

    Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    cal.set(Calendar.HOUR_OF_DAY, 0);
    cal.set(Calendar.MINUTE, 0);
    cal.set(Calendar.SECOND, 1);
    cal.set(Calendar.DATE, (cal.get(Calendar.DATE) - (((cal.get(Calendar.DAY_OF_WEEK) - 1 == 0) ? 7 : (cal
        .get(Calendar.DAY_OF_WEEK) - 1)) - 1)));
    cal.set(Calendar.DATE, (cal.get(Calendar.DATE)) - 7);
    Date fechaIni = cal.getTime();
    cal.set(Calendar.DATE, (cal.get(Calendar.DATE)) + 6);
    Date fechaFin = cal.getTime();
    List<Long> numero = new ArrayList<Long>();
    if (centro == null) {

      Query q = entity
          .createQuery("select count(*) from es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL where fechaCreacion >= ? and fechaCreacion <= ? group by DATE_FORMAT(fechaCreacion,'%d/%m/%y') ");
      q.setParameter(1, fechaIni);
      q.setParameter(2, fechaFin);

      numero = q.getResultList();
    } else {
      Query q = entity
          .createQuery("select count(*) from es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL where fechaCreacion >= ? and fechaCreacion <= ? and centro=? group by DATE_FORMAT(fechaCreacion,'%d/%m/%y') ");
      q.setParameter(1, fechaIni);
      q.setParameter(2, fechaFin);
      q.setParameter(3, centro);

      numero = q.getResultList();
    }

    int i = 0;
    long media = 0;
    while (i < numero.size()) {
      media = media + numero.get(i);
      i++;
    }
    double media2 = media;

    media2 = media2 / numeroDiasLaborables(fechaIni, fechaFin, centro, entity);

    return media2;
  }

  /**
   * Altas medias diarias año.
   * 
   * @param centro
   *           centro
   * @param entity
   *           entity
   * @return double
   */
  @SuppressWarnings("unchecked")
  public double altasMediasDiariasAnyo(Centro centro, EntityManager entity) {
    Date fechaIni = new Date();
    Date fechaFin = new Date();
    SimpleDateFormat formateador = new SimpleDateFormat("dd/MM/yyyy");
    SimpleDateFormat formateador2 = new SimpleDateFormat("dd/MM/yyyy HH:mm");
    try {
      fechaIni = formateador.parse(getFechaInicioAnyoPasadoLegible());
      fechaFin = formateador2.parse(getFechaFinAnyoPasadoLegible());
    } catch (ParseException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    List<Long> numero = new ArrayList<Long>();
    if (centro == null) {
      Query q = entity
          .createQuery("select count(*) from es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL where fechaCreacion >= ? and fechaCreacion <= ? group by DATE_FORMAT(fechaCreacion,'%d/%m/%y') ");
      q.setParameter(1, fechaIni);
      q.setParameter(2, fechaFin);

      numero = q.getResultList();
    } else {
      Query q = entity
          .createQuery("select count(*) from es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL where fechaCreacion >= ? and fechaCreacion <= ? and centro=? group by DATE_FORMAT(fechaCreacion,'%d/%m/%y') ");
      q.setParameter(1, fechaIni);
      q.setParameter(2, fechaFin);
      q.setParameter(3, centro);

      numero = q.getResultList();
    }

    int i = 0;
    long media = 0;
    while (i < numero.size()) {
      media = media + numero.get(i);
      i++;
    }
    double media2 = media;

    media2 = media2 / numeroDiasLaborables(fechaIni, fechaFin, centro, entity);

    return media2;
  }

  /**
   * Altas medias diarias mes.
   * 
   * @param centro
   *           centro
   * @param entity
   *           entity
   * @return double
   */
  @SuppressWarnings("unchecked")
  public double altasMediasDiariasMes(Centro centro, EntityManager entity) {
    Date fechaIni = new Date();
    Date fechaFin = new Date();
    SimpleDateFormat formateador = new SimpleDateFormat("dd/MM/yyyy");
    try {
      fechaIni = formateador.parse(getFechaInicioMesAntLegible());
      fechaFin = formateador.parse(getFechaFinMesAntLegible());
    } catch (ParseException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    cal.add(Calendar.DAY_OF_MONTH, 1);
    cal.set(Calendar.DATE, cal.get(Calendar.DATE) - 1);
    List<Long> numero = new ArrayList<Long>();
    if (centro == null) {
      Query q = entity
          .createQuery("select count(*) from es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL where fechaCreacion >= ? and fechaCreacion <= ? group by DATE_FORMAT(fechaCreacion,'%d/%m/%y') ");
      q.setParameter(1, fechaIni);
      q.setParameter(2, fechaFin);

      numero = q.getResultList();
    } else {
      Query q = entity
          .createQuery("select count(*) from es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL where fechaCreacion >= ? and fechaCreacion <= ? and centro=? group by DATE_FORMAT(fechaCreacion,'%d/%m/%y') ");
      q.setParameter(1, fechaIni);
      q.setParameter(2, fechaFin);
      q.setParameter(3, centro);

      numero = q.getResultList();
    }
    int i = 0;
    long media = 0;
    while (i < numero.size()) {
      media = media + numero.get(i);
      i++;
    }
    double media2 = 0;
    media2 = media / numeroDiasLaborables(fechaIni, fechaFin, centro, entity);
    return media2;
  }

  /**
   * Edad media semana.
   * 
   * @param centro
   *           centro
   * @param entity
   *           entity
   * @return double
   */
  @SuppressWarnings("unchecked")
  public double edadMediaSemana(Centro centro, EntityManager entity) {

    Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    cal.set(Calendar.HOUR_OF_DAY, 0);
    cal.set(Calendar.MINUTE, 0);
    cal.set(Calendar.SECOND, 1);
    cal.set(Calendar.DATE, (cal.get(Calendar.DATE) - (((cal.get(Calendar.DAY_OF_WEEK) - 1 == 0) ? 7 : (cal
        .get(Calendar.DAY_OF_WEEK) - 1)) - 1)));
    cal.set(Calendar.DATE, (cal.get(Calendar.DATE)) - 7);
    Date fechaIni = cal.getTime();
    cal.set(Calendar.DATE, (cal.get(Calendar.DATE)) + 6);
    Date fechaFin = cal.getTime();
    List<UsuarioCyL> usuarios = new ArrayList<UsuarioCyL>();

    if (centro == null) {

      Query q = entity
          .createQuery("Select e from es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL e where  fechaCreacion >= ? and fechaCreacion <= ? ");
      q.setParameter(1, fechaIni);
      q.setParameter(2, fechaFin);

      usuarios = q.getResultList();
    } else {
      Query q = entity
          .createQuery("Select e from es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL e where  fechaCreacion >= ? and fechaCreacion <= ? and centro=?");
      q.setParameter(1, fechaIni);
      q.setParameter(2, fechaFin);
      q.setParameter(3, centro);

      usuarios = q.getResultList();
    }
    int i = 0;
    long media = 0;
    while (i < usuarios.size()) {
      media = media + usuarios.get(i).getEdad();
      i++;
    }
    double media2 = media;
    if (media == 0) {
      media2 = 0;
    } else {
      media2 = media2 / usuarios.size();
    }
    return media2;
  }

  /**
   * Edad media mes.
   * 
   * @param centro
   *           centro
   * @param entity
   *           entity
   * @return double
   */
  @SuppressWarnings("unchecked")
  public double edadMediaMes(Centro centro, EntityManager entity) {
    Date fechaIni = new Date();
    Date fechaFin = new Date();
    CuadroDeMandos cuadro = new CuadroDeMandos();
    SimpleDateFormat formateador = new SimpleDateFormat("dd/MM/yyyy");
    try {
      fechaIni = formateador.parse(cuadro.getFechaInicioMesAntLegible());
      fechaFin = formateador.parse(cuadro.getFechaFinMesAntLegible());
    } catch (ParseException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    List<UsuarioCyL> usuarios = new ArrayList<UsuarioCyL>();
    if (centro == null) {
      Query q = entity
          .createQuery("Select e from es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL e where  fechaCreacion >= ? and fechaCreacion <= ? ");
      q.setParameter(1, fechaIni);
      q.setParameter(2, fechaFin);

      usuarios = q.getResultList();
    } else {
      Query q = entity
          .createQuery("Select e from es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL e where  fechaCreacion >= ? and fechaCreacion <= ? and centro=?");
      q.setParameter(1, fechaIni);
      q.setParameter(2, fechaFin);
      q.setParameter(3, centro);

      usuarios = q.getResultList();

    }
    int i = 0;
    long media = 0;
    while (i < usuarios.size()) {
      media = media + usuarios.get(i).getEdad();
      i++;
    }
    if (media == 0) {
      return 0;
    }
    double media2 = media;
    media2 = media2 / usuarios.size();
    return media2;

  }

  /**
   * Edad media año.
   * 
   * @param centro
   *           centro
   * @param entity
   *           entity
   * @return double
   */
  @SuppressWarnings("unchecked")
  public double edadMediaAnyo(Centro centro, EntityManager entity) {
    Date fechaIni = new Date();
    Date fechaFin = new Date();
    CuadroDeMandos cuadro = new CuadroDeMandos();
    SimpleDateFormat formateador = new SimpleDateFormat("dd/MM/yyyy");
    SimpleDateFormat formateador2 = new SimpleDateFormat("dd/MM/yyyy HH:mm");
    try {
      fechaIni = formateador.parse(cuadro.getFechaInicioAnyoPasadoLegible());
      fechaFin = formateador2.parse(cuadro.getFechaFinAnyoPasadoLegible());
    } catch (ParseException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    List<UsuarioCyL> usuarios = new ArrayList<UsuarioCyL>();
    if (centro == null) {
      Query q = entity
          .createQuery("Select e from es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL e where  fechaCreacion >= ? and fechaCreacion <= ? ");
      q.setParameter(1, fechaIni);
      q.setParameter(2, fechaFin);

      usuarios = q.getResultList();
    } else {
      Query q = entity
          .createQuery("Select e from es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL e where  fechaCreacion >= ? and fechaCreacion <= ? and centro=? ");
      q.setParameter(1, fechaIni);
      q.setParameter(2, fechaFin);
      q.setParameter(3, centro);

      usuarios = q.getResultList();
    }
    int i = 0;
    long media = 0;
    while (i < usuarios.size()) {
      media = media + usuarios.get(i).getEdad();
      i++;
    }
    if (media == 0) {
      return 0;
    }
    double media2 = media;
    media2 = media2 / usuarios.size();
    return media2;

  }

  /**
   * Altas semana.
   * 
   * @param centro
   *           centro
   * @param entity
   *           entity
   * @return int
   */
  @SuppressWarnings("unchecked")
  public int altasSemana(Centro centro, EntityManager entity) {

    Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    cal.set(Calendar.HOUR_OF_DAY, 0);
    cal.set(Calendar.MINUTE, 0);
    cal.set(Calendar.SECOND, 1);
    cal.set(Calendar.DATE, (cal.get(Calendar.DATE) - (((cal.get(Calendar.DAY_OF_WEEK) - 1 == 0) ? 7 : (cal
        .get(Calendar.DAY_OF_WEEK) - 1)) - 1)));
    cal.set(Calendar.DATE, (cal.get(Calendar.DATE)) - 7);
    Date fechaIni = cal.getTime();
    cal.set(Calendar.DATE, (cal.get(Calendar.DATE)) + 6);
    cal.add(Calendar.HOUR, 23);
    cal.add(Calendar.MINUTE, 59);
    Date fechaFin = cal.getTime();
    List<UsuarioCyL> usuarios = new ArrayList<UsuarioCyL>();
    if (centro != null) {
      Query q = entity.createQuery("Select e from es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL e where "
          + " fechaCreacion >= ? and fechaCreacion <= ? and e.id in "
          + "(select id from es.csa.fdt.cyldigital.usuarios.entity.Usuario where idCentro = ?)");
      q.setParameter(1, fechaIni);
      q.setParameter(2, fechaFin);
      q.setParameter(3, centro.getId());
      usuarios = q.getResultList();
    } else {
      Query q = entity.createQuery("Select e from es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL e "
          + "where  fechaCreacion >= ? and fechaCreacion <= ? ");
      q.setParameter(1, fechaIni);
      q.setParameter(2, fechaFin);
      usuarios = q.getResultList();
    }
    int i = 0;
    while (i != usuarios.size()) {
      if (usuarios.get(i).getDesactivado()) {
        numeroBajasSemana++;
      }
      i++;
    }
    return usuarios.size();
  }

  /**
   * Altas mes.
   * 
   * @param centro
   *           centro
   * @param entity
   *           entity
   * @return int
   */
  @SuppressWarnings("unchecked")
  public int altasMes(Centro centro, EntityManager entity) {
    Date fechaIni = new Date();
    Date fechaFin = new Date();
    CuadroDeMandos cuadro = new CuadroDeMandos();
    SimpleDateFormat formateador = new SimpleDateFormat("dd/MM/yyyy");
    try {
      fechaIni = formateador.parse(cuadro.getFechaInicioMesAntLegible());
      fechaFin = formateador.parse(cuadro.getFechaFinMesAntLegible());
      Calendar c = new GregorianCalendar();
      c.setTime(fechaFin);
      c.add(Calendar.HOUR, 23);
      c.add(Calendar.MINUTE, 59);
      fechaFin = c.getTime();
    } catch (ParseException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    List<UsuarioCyL> usuarios = new ArrayList<UsuarioCyL>();

    if (centro != null) {
      Query q = entity
          .createQuery("Select e from es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL e where  fechaCreacion >= ? and fechaCreacion <= ? and e.id in (select id from es.csa.fdt.cyldigital.usuarios.entity.Usuario where idCentro = ?)");
      q.setParameter(1, fechaIni);
      q.setParameter(2, fechaFin);
      q.setParameter(3, centro.getId());
      usuarios = q.getResultList();
    } else {
      Query q = entity
          .createQuery("Select e from es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL e where  fechaCreacion >= ? and fechaCreacion <= ? ");
      q.setParameter(1, fechaIni);
      q.setParameter(2, fechaFin);
      usuarios = q.getResultList();
    }
    int i = 0;
    while (i != usuarios.size()) {
      if (usuarios.get(i).getDesactivado()) {
        numeroBajasMes++;
      }
      i++;
    } 

    return usuarios.size();
  }

  /**
   * Altas año.
   * 
   * @param centro
   *           centro
   * @param entity
   *           entity
   * @return int
   */
  @SuppressWarnings("unchecked")
  public int altasAnyo(Centro centro, EntityManager entity) {
    Date fechaIni = new Date();
    Date fechaFin = new Date();
    CuadroDeMandos cuadro = new CuadroDeMandos();
    SimpleDateFormat formateador = new SimpleDateFormat("dd/MM/yyyy");
    SimpleDateFormat formateador2 = new SimpleDateFormat("dd/MM/yyyy HH:mm");
    try {
      fechaIni = formateador.parse(cuadro.getFechaInicioAnyoPasadoLegible());
      fechaFin = formateador2.parse(cuadro.getFechaFinAnyoPasadoLegible());
      Calendar c = new GregorianCalendar();
      c.setTime(fechaFin);
      c.add(Calendar.HOUR, 23);
      c.add(Calendar.MINUTE, 59);
      fechaFin = c.getTime();
    } catch (ParseException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    List<UsuarioCyL> usuarios = new ArrayList<UsuarioCyL>();

    if (centro != null) {
      Query q = entity
          .createQuery("Select e from es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL e where  fechaCreacion >= ? and fechaCreacion <= ? and e.id in (select id from es.csa.fdt.cyldigital.usuarios.entity.Usuario where idCentro = ?)");
      q.setParameter(1, fechaIni);
      q.setParameter(2, fechaFin);
      q.setParameter(3, centro.getId());
      usuarios = q.getResultList();
    } else {
      Query q = entity
          .createQuery("Select e from es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL e where  fechaCreacion >= ? and fechaCreacion <= ? ");
      q.setParameter(1, fechaIni);
      q.setParameter(2, fechaFin);
      usuarios = q.getResultList();
    }
    int i = 0;
    while (i != usuarios.size()) {
      if (usuarios.get(i).getDesactivado()) {
        numeroBajasAnyo++;
      }
      i++;
    }
    return usuarios.size();

  }

  /**
   * Numero cursos.
   * 
   * @param fechaIni
   *           fecha ini
   * @param fechaFin
   *           fecha fin
   * @param centro
   *           centro
   * @param entity
   *           entity
   * @return int
   */
  @SuppressWarnings("unchecked")
  public int numeroCursos(Date fechaIni, Date fechaFin, Centro centro, EntityManager entity) {
    numeroAlumnosMatriculados = 0;
    numeroAlumnosAsistentes = 0;
    List<Curso> numero = new ArrayList<Curso>();
    if (centro != null) {
      Query q = entity.createQuery("Select e from es.csa.fdt.cyldigital.servicios.entity.Curso e where "
          + " e.fechaInicio >= ? and e.fechaInicio <= ? and e.centro = ?)");
      q.setParameter(1, fechaIni);
      q.setParameter(2, fechaFin);
      q.setParameter(3, centro);
      numero = q.getResultList();
    } else {
      Query q = entity.createQuery("Select e from es.csa.fdt.cyldigital.servicios.entity.Curso e where "
          + " e.fechaInicio >= ? and e.fechaInicio <= ?)");
      q.setParameter(1, fechaIni);
      q.setParameter(2, fechaFin);
      numero = q.getResultList();
    }
    Iterator<Curso> it = numero.iterator();
    float a = 0, horas = 0, notaMedia = 0, sumaNotas = 0;
    int b = 0, espera = 0, cerrado = 0, plazas = 0, repetidor = 0, evaluados = 0;
    while (it.hasNext()) {
      Curso curso = it.next();
      List<MatriculaCurso> matriculas = curso.getMatriculas();
      Iterator<MatriculaCurso> iterator = matriculas.iterator();
      while (iterator.hasNext()) {
        MatriculaCurso matricula = iterator.next();
        if (matricula.getNota() != null) {
          sumaNotas = sumaNotas + matricula.getNota();
          evaluados++;
        }
        if (getVecesServicio(matricula.getUsuario(), curso.getServicio()) > 1) {
          repetidor++;
        }
      }
      if (evaluados != 0) {
        if (evaluados > (int) sumaNotas) {
          notaMedia = 0;
        } else {
          notaMedia = sumaNotas / evaluados;
        }
      } else {
        notaMedia = 0;
      }
      if (!curso.getDesactivado()) {
        a = a + curso.getNumeroMatriculados();
        horas = horas + curso.getNumeroHoras();
        b = b + curso.getNumeroAsistentes();
        espera = espera + curso.getNumeroListaEspera();
        plazas = plazas + curso.getNumeroPlazas();
      } else {
        cerrado++;
      }
    }
    if (plazas == 0) {
      setPorcOcupacion((double) 0);
    } else {
      setPorcOcupacion((double) ((a * 100) / plazas));
    }
    if (a == 0) {
      setPorcAsistentes((double) 0);
    } else {
      setPorcAsistentes((double) ((b * 100) / a));
    }
    setEvaluacionMedia(notaMedia);
    setRepetidores(repetidor);
    setCursosCerrados(cerrado);
    setHorasFormacion(horas);
    setNumeroAlumnosMatriculados(a);
    setNumeroAlumnosAsistentes(b);
    setNumeroAlumnosEspera(espera);
    return numero.size();
  }

  private int getVecesServicio(Usuario u, Servicio s) {
    int veces = 0;
    for (MatriculaCurso m : u.getMatriculasCursos())
      if (m.getCurso().getServicio().getId() == s.getId()) veces += 1;
    for (MatriculaTaller m : u.getMatriculasTalleres())
      if (m.getTaller().getServicio().getId() == s.getId()) veces += 1;
    return veces;
  }

  /**
   * Numero talleres.
   * 
   * @param fechaIni
   *           fecha ini
   * @param fechaFin
   *           fecha fin
   * @param centro
   *           centro
   * @param entity
   *           entity
   * @return int
   */
  @SuppressWarnings("unchecked")
  public int numeroTalleres(Date fechaIni, Date fechaFin, Centro centro, EntityManager entity) {
    numeroAlumnosMatriculados = 0;
    numeroAlumnosAsistentes = 0;
    List<Taller> numero = new ArrayList<Taller>();
    if (centro != null) {
      Query q = entity.createQuery("Select e from es.csa.fdt.cyldigital.servicios.entity.Taller e where "
          + " e.fechaInicio >= ? and e.fechaInicio <= ? and e.centro = ?)");
      q.setParameter(1, fechaIni);
      q.setParameter(2, fechaFin);
      q.setParameter(3, centro);
      numero = q.getResultList();
    } else {
      Query q = entity.createQuery("Select e from es.csa.fdt.cyldigital.servicios.entity.Taller e where "
          + " e.fechaInicio >= ? and e.fechaInicio <= ?)");
      q.setParameter(1, fechaIni);
      q.setParameter(2, fechaFin);
      numero = q.getResultList();
    }
    Iterator<Taller> it = numero.iterator();
    float a = 0, horas = 0;
    int b = 0, espera = 0, cerrado = 0, plazas = 0, repetidor = 0;
    while (it.hasNext()) {
      Taller taller = it.next();
      List<MatriculaTaller> matriculas = taller.getMatriculas();
      Iterator<MatriculaTaller> iterator = matriculas.iterator();
      while (iterator.hasNext()) {
        MatriculaTaller matricula = iterator.next();
        if (getVecesServicio(matricula.getUsuario(), taller.getServicio()) > 1) {
          repetidor++;
        }
      }
      if (!taller.getDesactivado()) {
        a = a + taller.getNumeroMatriculados();
        horas = horas + taller.getNumeroHoras();
        b = b + taller.getNumeroAsistentes();
        espera = espera + taller.getNumeroListaEspera();
        plazas = plazas + taller.getNumeroPlazas();
      } else {
        cerrado++;
      }
    }
    if (plazas == 0) {
      setPorcOcupacion((double) 0);
    } else {
      setPorcOcupacion((double) ((a * 100) / plazas));
    }
    if (a == 0) {
      setPorcAsistentes((double) 0);
    } else {
      setPorcAsistentes((double) ((b * 100) / a));
    }
    setRepetidores(repetidor);
    setTalleresCerrados(cerrado);
    setHorasFormacion(horas);
    setNumeroAlumnosMatriculados(a);
    setNumeroAlumnosAsistentes(b);
    setNumeroAlumnosEspera(espera);
    return numero.size();
  }

  /**
   * Cursos terceros.
   * 
   * @param entity
   *           entity
   * @param fechaIni
   *           fecha ini
   * @param fechaFin
   *           fecha fin
   * @param centro
   *           centro
   * @return int
   */
  @SuppressWarnings("unchecked")
  public int cursosTerceros(EntityManager entity, Date fechaIni, Date fechaFin, Centro centro) {
    int horas = 0, alumnos = 0;
    List<Curso> numero = new ArrayList<Curso>();
    if (centro != null) {
      Query q = entity
          .createQuery("select e from es.csa.fdt.cyldigital.servicios.entity.Curso e where profesor in("
              + "select a from es.csa.fdt.cyldigital.usuarios.entity.Empleado a where puesto = ? and profesor=true) and fechainicio>=? and"
              + " fechainicio<=? and e.centro=?");
      q.setParameter(1, Puesto.EXTERNAL_PERSONAL);
      q.setParameter(2, fechaIni);
      q.setParameter(3, fechaFin);
      q.setParameter(4, centro);
      numero = q.getResultList();

    } else {
      Query q = entity
          .createQuery("select e from es.csa.fdt.cyldigital.servicios.entity.Curso e where profesor in("
              + "select a from es.csa.fdt.cyldigital.usuarios.entity.Empleado a where puesto = ? and profesor=true) and fechainicio>=? and"
              + " fechainicio<=?");
      q.setParameter(1, Puesto.EXTERNAL_PERSONAL);
      q.setParameter(2, fechaIni);
      q.setParameter(3, fechaFin);
      numero = q.getResultList();
    }
    Iterator it = numero.iterator();
    while (it.hasNext()) {
      Curso curso = (Curso) it.next();
      horas = (int) (horas + curso.getNumeroHoras());
      alumnos = (int) (alumnos + curso.getNumeroMatriculados());
    }
    horasCursosTerceros = horas;
    alumnosCursosTerceros = alumnos;
    return numero.size();
  }

  /**
   * Devuelve usuarios aula virtual.
   * 
   * @param entity
   *           entity
   * @param fechaIni
   *           fecha ini
   * @param fechaFin
   *           fecha fin
   * @param centro
   *           centro
   * @return usuarios aula virtual
   */
  @SuppressWarnings("unchecked")
  public Integer getUsuariosAulaVirtual(EntityManager entity, Date fechaIni, Date fechaFin, Centro centro) {
    List<PracticaTaller> numero = new ArrayList<PracticaTaller>();
    if (centro != null) {
      Query q = entity
          .createQuery("select e from es.csa.fdt.cyldigital.servicios.entity.PracticaTaller e where fecha>=? and fecha <=? and centro=?");
      q.setParameter(1, fechaIni);
      q.setParameter(2, fechaFin);
      q.setParameter(3, centro);
      numero = q.getResultList();

    } else {
      Query q = entity
          .createQuery("select e from es.csa.fdt.cyldigital.servicios.entity.PracticaTaller e where fecha>=? and fecha <=?");
      q.setParameter(1, fechaIni);
      q.setParameter(2, fechaFin);
      numero = q.getResultList();
    }

    return numero.size();
  }

  /**
   * Devuelve usuarios wifi.
   * 
   * @param entity
   *           entity
   * @param fechaIni
   *           fecha ini
   * @param fechaFin
   *           fecha fin
   * @param centro
   *           centro
   * @return usuarios wifi
   */
  @SuppressWarnings("unchecked")
  public Integer getUsuariosWifi(EntityManager entity, Date fechaIni, Date fechaFin, Centro centro) {
    List<Wifi> numero = new ArrayList<Wifi>();
    if (centro != null) {
      Query q = entity
          .createQuery("select e from es.csa.fdt.cyldigital.servicios.entity.Wifi e where fecha>=? and fecha <=? and centro=?");
      q.setParameter(1, fechaIni);
      q.setParameter(2, fechaFin);
      q.setParameter(3, centro);
      numero = q.getResultList();

    } else {
      Query q = entity
          .createQuery("select e from es.csa.fdt.cyldigital.servicios.entity.Wifi e where fecha>=? and fecha <=?");
      q.setParameter(1, fechaIni);
      q.setParameter(2, fechaFin);
      numero = q.getResultList();
    }

    return numero.size();
  }

  /**
   * Devuelve usuarios asesoramiento.
   * 
   * @param entity
   *           entity
   * @param fechaIni
   *           fecha ini
   * @param fechaFin
   *           fecha fin
   * @param centro
   *           centro
   * @return usuarios asesoramiento
   */
  @SuppressWarnings("unchecked")
  public Integer getUsuariosAsesoramiento(EntityManager entity, Date fechaIni, Date fechaFin, Centro centro) {
    List<Wifi> numero = new ArrayList<Wifi>();
    if (centro != null) {
      Query q = entity
          .createQuery("select e from es.csa.fdt.cyldigital.servicios.entity.Asesoramiento e where fecha>=? and fecha <=? and centro=?");
      q.setParameter(1, fechaIni);
      q.setParameter(2, fechaFin);
      q.setParameter(3, centro);
      numero = q.getResultList();

    } else {
      Query q = entity
          .createQuery("select e from es.csa.fdt.cyldigital.servicios.entity.Asesoramiento e where fecha>=? and fecha <=?");
      q.setParameter(1, fechaIni);
      q.setParameter(2, fechaFin);
      numero = q.getResultList();
    }

    return numero.size();
  }

  /**
   * **************************************** Cálculo de fechas ***************************************************.
   * 
   * @return date
   */

  public Date fechaIniSemanaPasada() {
    Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    cal.set(Calendar.HOUR_OF_DAY, 0);
    cal.set(Calendar.MINUTE, 0);
    cal.set(Calendar.SECOND, 1);
    cal.set(Calendar.DATE, (cal.get(Calendar.DATE) - (((cal.get(Calendar.DAY_OF_WEEK) - 1 == 0) ? 7 : (cal
        .get(Calendar.DAY_OF_WEEK) - 1)) - 1)));
    cal.set(Calendar.DATE, (cal.get(Calendar.DATE)) - 7);
    Date fechaIni = cal.getTime();
    cal.set(Calendar.DATE, (cal.get(Calendar.DATE)) + 6);
    return fechaIni;
  }

  /**
   * Fecha fin semana pasada.
   * 
   * @return date
   */
  public Date fechaFinSemanaPasada() {
    Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    cal.set(Calendar.HOUR_OF_DAY, 0);
    cal.set(Calendar.MINUTE, 0);
    cal.set(Calendar.SECOND, 1);
    cal.set(Calendar.DATE, (cal.get(Calendar.DATE) - (((cal.get(Calendar.DAY_OF_WEEK) - 1 == 0) ? 7 : (cal
        .get(Calendar.DAY_OF_WEEK) - 1)) - 1)));
    cal.set(Calendar.DATE, (cal.get(Calendar.DATE)) - 7);
    cal.set(Calendar.DATE, (cal.get(Calendar.DATE)) + 6);
    Date fechaFin = cal.getTime();
    return fechaFin;
  }

  /**
   * Numero dias laborables.
   * 
   * @param fechaIni
   *           fecha ini
   * @param fechaFin
   *           fecha fin
   * @param centro
   *           centro
   * @param entity
   *           entity
   * @return int
   */
  public int numeroDiasLaborables(Date fechaIni, Date fechaFin, Centro centro, EntityManager entity) {
    int i = 0;
    Calendar c = Calendar.getInstance();
    c.setTime(fechaIni);
    c.set(Calendar.HOUR_OF_DAY, 0);
    c.set(Calendar.MINUTE, 0);
    c.set(Calendar.SECOND, 0);
    c.set(Calendar.MILLISECOND, 0);
    fechaIni = c.getTime();
    Calendar c1 = Calendar.getInstance();
    c1.setTime(fechaFin);
    c1.set(Calendar.HOUR_OF_DAY, 0);
    c1.set(Calendar.MINUTE, 0);
    c1.set(Calendar.SECOND, 0);
    c1.set(Calendar.MILLISECOND, 0);
    fechaFin = c1.getTime();

    while (fechaIni.compareTo(fechaFin) != 0) {
      if (c.get(Calendar.DAY_OF_WEEK) != 1) {
        i++;

      }
      c.add(Calendar.DATE, 1); // le suma 1 día
      fechaIni = c.getTime();

    }
    i = i - findFestivoItems(centro, fechaIni, fechaFin, entity);
    return i;
  }

  /**
   * Find festivo items.
   * 
   * @param centro
   *           centro
   * @param fechaIni
   *           fecha ini
   * @param fechaFin
   *           fecha fin
   * @param entity
   *           entity
   * @return int
   */
  @SuppressWarnings("unchecked")
  public int findFestivoItems(Centro centro, Date fechaIni, Date fechaFin, EntityManager entity) {

    List<FestivoCalendarItem> festivos = new ArrayList<FestivoCalendarItem>();
    Query q = entity
        .createQuery("select e from es.csa.fdt.cyldigital.calendario.entity.FestivoCalendarItem e where e.id in "
            + "(select a.id from es.csa.fdt.cyldigital.calendario.entity.CalendarItem a where a.start >= ? and a.end <= ?)");
    q.setParameter(1, fechaIni);
    q.setParameter(2, fechaFin);

    festivos = q.getResultList();

    Iterator<FestivoCalendarItem> it = festivos.iterator();
    int i = 0;
    while (it.hasNext()) {
      FestivoCalendarItem festivo = it.next();
      if (centro == null) {
        if (festivo.getLocalidad() == null) {
          if (festivo.getProvincia() == null) {
            i++;
          }
        }
      } else if (festivo.getLocalidad() == null) {
        if (festivo.getProvincia() == null) {
          i++;
        }
      } else if (festivo.getLocalidad().equals(centro.getLocalidad())) {
        i++;
      }
    }
    return i;
  }

  /**
   * Devuelve fecha inicio mes ant legible.
   * 
   * @return fecha inicio mes ant legible
   */
  public String getFechaInicioMesAntLegible() {
    String aux;
    Date fecha = new Date();
    Calendar c = new GregorianCalendar();
    c.setTime(fecha);
    int ano = c.get(Calendar.YEAR);
    int mes = c.get(Calendar.MONTH);
    mes = mes + 1;
    if (mes == 1) {
      mes = 12;
      ano--;
    } else {
      mes--;
    }
    String fechaConcatenada = "01/" + mes + "/" + ano;
    SimpleDateFormat formateador = new SimpleDateFormat("dd/MM/yyyy");
    try {
      fecha = formateador.parse(fechaConcatenada);
    } catch (ParseException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    aux = formateador.format(fecha);
    return aux;
  }

  /**
   * Devuelve fecha fin mes ant legible.
   * 
   * @return fecha fin mes ant legible
   */
  public String getFechaFinMesAntLegible() {
    String aux;
    Date fecha = new Date();
    Calendar c = new GregorianCalendar();
    c.setTime(fecha);
    int ano = c.get(Calendar.YEAR);
    int mes = c.get(Calendar.MONTH);
    mes++;
    int dia;
    if (mes == 1) {
      mes = 12;
      ano--;
    } else {
      mes--;
    }

    if (mes == 1 || mes == 3 || mes == 5 || mes == 7 || mes == 8 || mes == 10 || mes == 12) {
      dia = 31;
    } else if (mes == 2) {
      if ((ano % 4 == 0) && ((ano % 100 != 0) || (ano % 400 == 0))) {
        dia = 29;
      } else {
        dia = 28;
      }
    } else {
      dia = 30;
    }
    String fechaConcatenada = dia + "/" + mes + "/" + ano + " 23:59";
    SimpleDateFormat formateador = new SimpleDateFormat("dd/MM/yyyy HH:mm");
    try {
      fecha = formateador.parse(fechaConcatenada);
    } catch (ParseException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    aux = formateador.format(fecha);
    return aux;
  }

  /**
   * Devuelve fecha inicio año pasado legible.
   * 
   * @return fecha inicio año pasado legible
   */
  public String getFechaInicioAnyoPasadoLegible() {
    String aux;
    Date fecha = new Date();
    Calendar c = new GregorianCalendar();
    c.setTime(fecha);
    int ano = c.get(Calendar.YEAR);
    ano--;
    String fechaConcatenada = "01/01/" + ano;
    SimpleDateFormat formateador = new SimpleDateFormat("dd/MM/yyyy");
    try {
      fecha = formateador.parse(fechaConcatenada);
    } catch (ParseException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    aux = formateador.format(fecha);
    return aux;
  }

  /**
   * Devuelve fecha fin año pasado legible.
   * 
   * @return fecha fin año pasado legible
   */
  public String getFechaFinAnyoPasadoLegible() {
    String aux;
    Date fecha = new Date();
    Calendar c = new GregorianCalendar();
    c.setTime(fecha);
    int ano = c.get(Calendar.YEAR);
    ano--;
    String fechaConcatenada = "31/12/" + ano + " 23:59";
    SimpleDateFormat formateador = new SimpleDateFormat("dd/MM/yyyy HH:mm");
    try {
      fecha = formateador.parse(fechaConcatenada);
    } catch (ParseException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    aux = formateador.format(fecha);
    return aux;
  }

  /**
   * ************************************** Getters and Setters *******************************************.
   * 
   * @param numeroUsuariosMatriculados
   *          numero usuarios matriculados
   */

  public void setNumeroUsuariosMatriculados(Integer numeroUsuariosMatriculados) {
    this.numeroUsuariosMatriculados = numeroUsuariosMatriculados;
  }

  /**
   * Devuelve numero usuarios matriculados.
   * 
   * @return numero usuarios matriculados
   */
  public Integer getNumeroUsuariosMatriculados() {
    return numeroUsuariosMatriculados;
  }

  /**
   * Asigna numero bajas semana.
   * 
   * @param numeroBajasSemana
   *          numero bajas semana
   */
  public void setNumeroBajasSemana(Integer numeroBajasSemana) {
    this.numeroBajasSemana = numeroBajasSemana;
  }

  /**
   * Devuelve numero bajas semana.
   * 
   * @return numero bajas semana
   */
  public Integer getNumeroBajasSemana() {
    return numeroBajasSemana;
  }

  /**
   * Asigna numero bajas mes.
   * 
   * @param numeroBajasMes
   *          numero bajas mes
   */
  public void setNumeroBajasMes(Integer numeroBajasMes) {
    this.numeroBajasMes = numeroBajasMes;
  }

  /**
   * Devuelve numero bajas mes.
   * 
   * @return numero bajas mes
   */
  public Integer getNumeroBajasMes() {
    return numeroBajasMes;
  }

  /**
   * Asigna numero alumnos matriculados.
   * 
   * @param numeroAlumnosMatriculados
   *          numero alumnos matriculados
   */
  public void setNumeroAlumnosMatriculados(float numeroAlumnosMatriculados) {
    this.numeroAlumnosMatriculados = numeroAlumnosMatriculados;
  }

  /**
   * Devuelve numero alumnos matriculados.
   * 
   * @return numero alumnos matriculados
   */
  public float getNumeroAlumnosMatriculados() {
    return numeroAlumnosMatriculados;
  }

  /**
   * Asigna numero bajas año.
   * 
   * @param numeroBajasAño
   *          numero bajas año
   */
  public void setNumeroBajasAnyo(Integer numeroBajasAnyo) {
    this.numeroBajasAnyo = numeroBajasAnyo;
  }

  /**
   * Devuelve numero bajas año.
   * 
   * @return numero bajas año
   */
  public Integer getNumeroBajasAnyo() {
    return numeroBajasAnyo;
  }

  /**
   * Asigna numero alumnos asistentes.
   * 
   * @param numeroAlumnosAsistentes
   *          numero alumnos asistentes
   */
  public void setNumeroAlumnosAsistentes(Integer numeroAlumnosAsistentes) {
    this.numeroAlumnosAsistentes = numeroAlumnosAsistentes;
  }

  /**
   * Devuelve numero alumnos asistentes.
   * 
   * @return numero alumnos asistentes
   */
  public Integer getNumeroAlumnosAsistentes() {
    return numeroAlumnosAsistentes;
  }

  /**
   * Asigna numero alumnos espera.
   * 
   * @param numeroAlumnosEspera
   *          numero alumnos espera
   */
  public void setNumeroAlumnosEspera(Integer numeroAlumnosEspera) {
    this.numeroAlumnosEspera = numeroAlumnosEspera;
  }

  /**
   * Devuelve numero alumnos espera.
   * 
   * @return numero alumnos espera
   */
  public Integer getNumeroAlumnosEspera() {
    return numeroAlumnosEspera;
  }

  /**
   * Asigna horas formacion.
   * 
   * @param horasFormacion
   *          horas formacion
   */
  public void setHorasFormacion(float horasFormacion) {
    this.horasFormacion = horasFormacion;
  }

  /**
   * Devuelve horas formacion.
   * 
   * @return horas formacion
   */
  public float getHorasFormacion() {
    return horasFormacion;
  }

  /**
   * Asigna cursos cerrados.
   * 
   * @param cursosCerrados
   *          cursos cerrados
   */
  public void setCursosCerrados(Integer cursosCerrados) {
    this.cursosCerrados = cursosCerrados;
  }

  /**
   * Devuelve cursos cerrados.
   * 
   * @return cursos cerrados
   */
  public Integer getCursosCerrados() {
    return cursosCerrados;
  }

  /**
   * Asigna porc asistentes.
   * 
   * @param porcAsistentes
   *          porc asistentes
   */
  public void setPorcAsistentes(double porcAsistentes) {
    this.porcAsistentes = porcAsistentes;
  }

  /**
   * Devuelve porc asistentes.
   * 
   * @return porc asistentes
   */
  public double getPorcAsistentes() {
    return porcAsistentes;
  }

  /**
   * Asigna porc ocupacion.
   * 
   * @param porcOcupacion
   *          porc ocupacion
   */
  public void setPorcOcupacion(double porcOcupacion) {
    this.porcOcupacion = porcOcupacion;
  }

  /**
   * Devuelve porc ocupacion.
   * 
   * @return porc ocupacion
   */
  public double getPorcOcupacion() {
    return porcOcupacion;
  }

  /**
   * Asigna media edad.
   * 
   * @param mediaEdad
   *          media edad
   */
  public void setMediaEdad(Float mediaEdad) {
    this.mediaEdad = mediaEdad;
  }

  /**
   * Devuelve media edad.
   * 
   * @return media edad
   */
  public Float getMediaEdad() {
    return mediaEdad;
  }

  /**
   * Asigna horas cursos terceros.
   * 
   * @param horasCursosTerceros
   *          horas cursos terceros
   */
  public void setHorasCursosTerceros(Integer horasCursosTerceros) {
    this.horasCursosTerceros = horasCursosTerceros;
  }

  /**
   * Devuelve horas cursos terceros.
   * 
   * @return horas cursos terceros
   */
  public Integer getHorasCursosTerceros() {
    return horasCursosTerceros;
  }

  /**
   * Asigna alumnos cursos terceros.
   * 
   * @param alumnosCursosTerceros
   *          alumnos cursos terceros
   */
  public void setAlumnosCursosTerceros(Integer alumnosCursosTerceros) {
    this.alumnosCursosTerceros = alumnosCursosTerceros;
  }

  /**
   * Devuelve alumnos cursos terceros.
   * 
   * @return alumnos cursos terceros
   */
  public Integer getAlumnosCursosTerceros() {
    return alumnosCursosTerceros;
  }

  /**
   * Asigna talleres cerrados.
   * 
   * @param talleresCerrados
   *          talleres cerrados
   */
  public void setTalleresCerrados(Integer talleresCerrados) {
    this.talleresCerrados = talleresCerrados;
  }

  /**
   * Devuelve talleres cerrados.
   * 
   * @return talleres cerrados
   */
  public Integer getTalleresCerrados() {
    return talleresCerrados;
  }

  /**
   * Asigna repetidores.
   * 
   * @param repetidores
   *          repetidores
   */
  public void setRepetidores(Integer repetidores) {
    this.repetidores = repetidores;
  }

  /**
   * Devuelve repetidores.
   * 
   * @return repetidores
   */
  public Integer getRepetidores() {
    return repetidores;
  }

  /**
   * Asigna evaluacion media.
   * 
   * @param evaluacionMedia
   *          evaluacion media
   */
  public void setEvaluacionMedia(double evaluacionMedia) {
    this.evaluacionMedia = evaluacionMedia;
  }

  /**
   * Devuelve evaluacion media.
   * 
   * @return evaluacion media
   */
  public double getEvaluacionMedia() {
    return evaluacionMedia;
  }

}
