package es.csa.fdt.cyldigital.language.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.lang.String;
import javax.persistence.*;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.core.entity.FileOnDisk;
import es.csa.fdt.cyldigital.servicios.entity.ActividadExterna;

/**
 * Entity implementation class for Entity: ActividadExternaLanguage
 *
 */
@Entity
@Table(name="actividadexternalanguages")
//@AttributeOverride(name = "id", column = @Column(name = "idActividadExternaLanguage"))
@Audited
@PrimaryKeyJoinColumn(name = "idActividadLanguage")
public class ActividadExternaLanguage extends ActividadLanguage implements Serializable {

	private static final long serialVersionUID = 1766604964890365678L;

	private ActividadExterna externa;
	
	/** Apodo de la actividad externa en el idioma language */ 
	private String apodo;
	
	/** Descripción de la actividad externa en el idioma language */ 
	private String descripcion;
	
	/** Fichero de la actividad externa en el idioma language */
	private FileOnDisk ficheroActExt;
	
	/**
	 * Devuelve externa
	 * 
	 * @return externa
	 */
    @ManyToOne(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
	@JoinColumn(name = "idActividadExterna")
	public ActividadExterna getExterna() {
		return externa;
	}

	/**
	 * Asigna  externa
	 * 
	 * @param externa
	 *          externa
	 */
	public void setExterna(ActividadExterna externa) {
		this.externa = externa;
	}

	/* (non-Javadoc)
	 * @see es.csa.fdt.cyldigital.language.entity.ActividadLanguage#getNombre()
	 */
	@Transient
	@Override
	public String getNombre() {
	  return getApodo();
	}

	/**
	 * Devuelve apodo
	 * 
	 * @return apodo
	 */
	@Length(max = 255)
	@NotEmpty
	@NotNull
	public String getApodo() {
		//if ((apodo == null || apodo.isEmpty()) && getActividad().getServicio() != null) return getActividad().getServicio().getNombre();
	    return apodo;
	}

	/**
	 * Asigna  apodo
	 * 
	 * @param apodo
	 *          apodo
	 */
	public void setApodo(String apodo) {
		this.apodo = apodo;
	}
	
	/**
	 * Devuelve descripcion
	 *
	 * @return descripcion
	 */
	@Lob
	@Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
	public String getDescripcion() {
		return this.descripcion;
	}

	/**
	 * Asigna  descripcion
	 * 
	 * @param descripcion
	 *          descripcion
	 */
	public void setDescripcion(String descripcion) {
		this.descripcion = descripcion;
	}
	
	
	/**
	 * Devuelve fichero de la actividad externa.
	 * 
	 * @return fichero actividad externa
	 */
	@ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
	@JoinColumn(name = "idFicheroActExt")
	public FileOnDisk getFicheroActExt() {
	    return ficheroActExt;
	}
	
	/**
	 * Asigna fichero actividad externa
	 * 
	 * @param ficheroActExt
	 *          fichero actividad externa
	 */
	public void setFicheroActExt(FileOnDisk ficheroActExt) {
	    this.ficheroActExt = ficheroActExt;
	}
}
