package es.csa.fdt.cyldigital.language.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.lang.String;
import javax.persistence.*;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.servicios.entity.Actividad;

/**
 * Entity implementation class for Entity: ActividadLanguage
 *
 */
@Entity
@Table(name="actividadlanguages", uniqueConstraints = @UniqueConstraint(columnNames = { "idActividad", "idLanguage" }))
@Audited
@Inheritance(strategy = InheritanceType.JOINED)
@AttributeOverride(name = "id", column = @Column(name = "idActividadLanguage"))
public abstract class ActividadLanguage extends AbstractEntity implements Serializable {

	private static final long serialVersionUID = 914088248420654730L;

	private Actividad actividad;
	
	private Language language;
	
	/**
	 * Devuelve actividad
	 * 
	 * @return actividad
	 */
	@ManyToOne(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
    @JoinColumn(name = "idActividad")    
	public Actividad getActividad() {
		return actividad;
	}

	/**
	 * Asigna  actividad
	 * 
	 * @param actividad
	 *          actividad
	 */
	public void setActividad(Actividad actividad) {
		this.actividad = actividad;
	}

	/**
	 * Devuelve language
	 * 
	 * @return language
	 */
	@ManyToOne(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
    @JoinColumn(name = "idLanguage", nullable = false)
    @NotNull
	public Language getLanguage() {
		return language;
	}

	/**
	 * Asigna  language
	 * 
	 * @param language
	 *          language
	 */
	public void setLanguage(Language language) {
		this.language = language;
	}
	
	@Transient
	public abstract String getNombre();
	
}
