package es.csa.fdt.cyldigital.language.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.lang.String;
import javax.persistence.*;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.servicios.entity.Charla;

/**
 * Entity implementation class for Entity: CharlaLanguage
 *
 */
@Entity
@Table(name="charlalanguages")
//@AttributeOverride(name = "id", column = @Column(name = "idCharlaLanguage"))
@Audited
@PrimaryKeyJoinColumn(name = "idActividadLanguage")
public class CharlaLanguage extends ActividadLanguage implements Serializable {

	private static final long serialVersionUID = -7353206279162669084L;

	private Charla charla;
	
	/** Nombre de la charla en el idioma language */ 
	private String nombre;
	
	// RF_15_29 Campo observación nas charlas
	/** Descripcion de la charla en el idioma language */
	private String descripcion;
	
	/**
	 * Devuelve charla
	 * 
	 * @return charla
	 */
    @ManyToOne(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
	@JoinColumn(name = "idCharla")
	public Charla getCharla() {
		return charla;
	}

	/**
	 * Asigna  curso
	 * 
	 * @param curso
	 *          curso
	 */
	public void setCharla(Charla charla) {
		this.charla = charla;
	}

	/**
	 * Devuelve nombre
	 * 
	 * @return nombre
	 */
	@Length(max = 200)
	@NotEmpty
	@NotNull
	public String getNombre() {
		return this.nombre;
	}

	/**
	 * Asigna  nombre
	 * 
	 * @param nombre
	 *          nombre
	 */
	public void setNombre(String nombre) {
		this.nombre = nombre;
	}
	
	// RF_15_29 Campo observación nas charlas
	/**
	 * Devuelve descripcion.
	 * 
	 * @return descripcion
	 */
	@Length(max = 200)
	public String getDescripcion() {
	  return descripcion;
	}

	// RF_15_29 Campo observación nas charlas
	/**
	 * Asigna descripcion.
	 * 
	 * @param descripcion
	 *          descripcion
	 */
	public void setDescripcion(String descripcion) {
	  this.descripcion = descripcion;
	}
}
