package es.csa.fdt.cyldigital.language.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.lang.String;
import javax.persistence.*;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.servicios.entity.Curso;

/**
 * Entity implementation class for Entity: CursoLanguage
 *
 */
@Entity
@Table(name="cursolanguages")
//@AttributeOverride(name = "id", column = @Column(name = "idCursoLanguage"))
@Audited
@PrimaryKeyJoinColumn(name = "idActividadLanguage")
public class CursoLanguage extends ActividadLanguage implements Serializable {

	private static final long serialVersionUID = 1766604964890365678L;

	private Curso curso;
	
	/** Apodo del curso en el idioma language */ 
	private String apodo;
	//3.4.11 - RF_15_52 Añadir guion curso
	/** Guion del curso en el idioma language */ 
	private String guion;
	
	/**
	 * Devuelve curso
	 * 
	 * @return curso
	 */
	@ManyToOne(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
    @JoinColumn(name = "idCurso")
	public Curso getCurso() {
		return curso;
	}

	/**
	 * Asigna  curso
	 * 
	 * @param curso
	 *          curso
	 */
	public void setCurso(Curso curso) {
		this.curso = curso;
	}

	/* (non-Javadoc)
	 * @see es.csa.fdt.cyldigital.language.entity.ActividadLanguage#getNombre()
	 */
	@Transient
	@Override
	public String getNombre() {
		return getApodo();
	}
	
	/**
	 * Devuelve apodo
	 * 
	 * @return apodo
	 */
	@Length(max = 255)
	@NotEmpty
	@NotNull
	public String getApodo() {
		//if ((apodo == null || apodo.isEmpty()) && getActividad().getServicio() != null) return getActividad().getServicio().getNombre();
	    return apodo;
	}

	/**
	 * Asigna  apodo
	 * 
	 * @param apodo
	 *          apodo
	 */
	public void setApodo(String apodo) {
		this.apodo = apodo;
	}

	//3.4.11 - RF_15_52 Añadir guion curso
	/**
	 * Devuelve guion
	 * 
	 * @return guion
	 */
	@Lob
	@Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
	public String getGuion() {
		return this.guion;
	}

	/**
	 * Asigna  guion
	 * 
	 * @param guion
	 *          guion
	 */
	public void setGuion(String guion) {
		this.guion = guion;
	}
}
