package es.csa.fdt.cyldigital.language.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.lang.String;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.UniqueConstraint;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.diplomas.entity.Diploma;

/**
 * Entity implementation class for Entity: DiplomaLanguage
 *
 */
@Entity
@Table(name="diplomalanguages", uniqueConstraints = @UniqueConstraint(columnNames = { "idDiploma", "idLanguage" }))
@Audited
@AttributeOverride(name = "id", column = @Column(name = "idDiplomaLanguage"))
public class DiplomaLanguage extends AbstractEntity implements Serializable {

	private static final long serialVersionUID = -1956121119612025174L;

	/** Diploma asociado */
	private Diploma diploma;
	
	/** Idioma */
	private Language language;
	
	/** Nombre del diploma en el idioma language */ 
	private String nombre;
	
	/** Descripción del del diploma en el idioma language */
	private String descripcion;
	
	/** Texto que aparecerá en el diploma de Asistencia */
	private String textoDiplomaAsistencia;
	
	/** Texto que aparecerá en el diploma de Asistencia y Aprovechamiento */
	private String textoDiplomaAprovechamiento;
	
	/**
	 * Devuelve diploma
	 * 
	 * @return diploma
	 */
	@ManyToOne(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
    @JoinColumn(name = "idDiploma")
	public Diploma getDiploma() {
		return diploma;
	}

	/**
	 * Asigna  diploma
	 * 
	 * @param diploma
	 *          diploma
	 */
	public void setDiploma(Diploma diploma) {
		this.diploma = diploma;
	}
	
	/**
	 * Devuelve language
	 * 
	 * @return language
	 */
	@ManyToOne(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
    @JoinColumn(name = "idLanguage", nullable = false)
    @NotNull
	public Language getLanguage() {
		return language;
	}

	/**
	 * Asigna  language
	 * 
	 * @param language
	 *          language
	 */
	public void setLanguage(Language language) {
		this.language = language;
	}

	/**
	 * Devuelve nombre.
	 * 
	 * @return nombre
	 */
	@Length(max = 100)
	@NotEmpty
	@NotNull
	public String getNombre() {
		return nombre;
	}

	/**
	 * Asigna nombre.
	 * 
	 * @param nombre
	 *          nombre
	 */
	public void setNombre(String nombre) {
		this.nombre = nombre;
	}
	
	/**
	 * Devuelve descripcion
	 * 
	 * @return descripcion
	 */
	@Lob
	@Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
	public String getDescripcion() {
		return this.descripcion;
	}

	/**
	 * Asigna  descripcion
	 * 
	 * @param descripcion
	 *          descripcion
	 */
	public void setDescripcion(String descripcion) {
		this.descripcion = descripcion;
	}

	/**
	 * @return the textoDiplomaAsistencia
	 */
	@Lob
	@Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
	public String getTextoDiplomaAsistencia() {
		return textoDiplomaAsistencia;
	}

	/**
	 * @param textoDiplomaAsistencia the textoDiplomaAsistencia to set
	 */
	public void setTextoDiplomaAsistencia(String textoDiplomaAsistencia) {
		this.textoDiplomaAsistencia = textoDiplomaAsistencia;
	}

	/**
	 * @return the textoDiplomaAprovechamiento
	 */
	@Lob
	@Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
	public String getTextoDiplomaAprovechamiento() {
		return textoDiplomaAprovechamiento;
	}

	/**
	 * @param textoDiplomaAprovechamiento the textoDiplomaAprovechamiento to set
	 */
	public void setTextoDiplomaAprovechamiento(String textoDiplomaAprovechamiento) {
		this.textoDiplomaAprovechamiento = textoDiplomaAprovechamiento;
	}
}
