package es.csa.fdt.cyldigital.language.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.lang.String;
import javax.persistence.*;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.evaluacion.entity.Formulario;

/**
 * Entity implementation class for Entity: FormularioLanguage
 *
 */
@Entity
@Table(name="formulariolanguages", uniqueConstraints = @UniqueConstraint(columnNames = { "idFormulario", "idLanguage" }))
@Audited
@AttributeOverride(name = "id", column = @Column(name = "idFormularioLanguage"))
public class FormularioLanguage extends AbstractEntity implements Serializable {

	private static final long serialVersionUID = -3098385434240665002L;

	private Formulario formulario;
	
	private Language language;
	
	/** Nombre del formulario en el idioma language */ 
	private String nombre;

	/** Descripción del formulario en el idioma language */
	private String descripcion;

	/** Encabezado del formulario en el idioma language */
	private String encabezado;

	/** Pie del formulario en el idioma language */
	private String pie;
	
	/**
	 * Devuelve formulario
	 * 
	 * @return formulario
	 */
	@ManyToOne(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
	@JoinColumn(name = "idFormulario")
	public Formulario getFormulario() {
		return formulario;
	}

	/**
	 * Asigna  formulario
	 * 
	 * @param formulario
	 *          formulario
	 */
	public void setFormulario(Formulario formulario) {
		this.formulario = formulario;
	}

	/**
	 * Devuelve language
	 * 
	 * @return language
	 */
	@ManyToOne(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
	@JoinColumn(name = "idLanguage", nullable = false)
	public Language getLanguage() {
		return language;
	}

	/**
	 * Asigna  language
	 * 
	 * @param language
	 *          language
	 */
	public void setLanguage(Language language) {
		this.language = language;
	}

	/**
	 * Devuelve nombre.
	 * 
	 * @return nombre
	 */
	@NotNull
	@NotEmpty
	@Length(max = 100)
	public String getNombre() {
	  return nombre;
	}

	/**
	 * Asigna nombre.
	 * 
	 * @param nombre
	 *          nombre
	 */
	public void setNombre(String nombre) {
	  this.nombre = nombre;
	}

	/**
	 * Devuelve descripcion.
	 * 
	 * @return descripcion
	 */
	@Lob
	@Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
	public String getDescripcion() {
	  return descripcion;
	}

	/**
	 * Asigna descripcion.
	 * 
	 * @param descripcion
	 *          descripcion
	 */
	public void setDescripcion(String descripcion) {
	  this.descripcion = descripcion;
	}

	/**
	 * Devuelve encabezado.
	 * 
	 * @return encabezado
	 */
	@Lob
	@Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
	public String getEncabezado() {
	  return encabezado;
	}

	/**
	 * Asigna encabezado.
	 * 
	 * @param encabezado
	 *          encabezado
	 */
	public void setEncabezado(String encabezado) {
	  this.encabezado = encabezado;
	}

	/**
	 * Devuelve pie.
	 * 
	 * @return pie
	 */
	@Lob
	@Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
	public String getPie() {
	  return pie;
	}

	/**
	 * Asigna pie.
	 * 
	 * @param pie
	 *          pie
	 */
	public void setPie(String pie) {
	  this.pie = pie;
	}
}
