package es.csa.fdt.cyldigital.language.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;

import java.lang.String;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.UniqueConstraint;

import org.hibernate.envers.Audited;

import org.hibernate.validator.Length;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.evaluacion.entity.ItemExamen;

/**
 * Entity implementation class for Entity: ItemLanguage
 *
 */
@Entity
@Table(name="itemsexameneslanguages", uniqueConstraints = @UniqueConstraint(columnNames = {"idItemExamen", "idLanguage"}))
@Audited
@AttributeOverride(name = "id", column = @Column(name = "idItemExamenLanguage"))
public class ItemExamenLanguage extends AbstractEntity implements Serializable {

	private static final long serialVersionUID = -4005761796794135051L;

	private ItemExamen itemExamen;

	private Language language;

	/** Nombre del item en el idioma language */ 
	private String nombre;

	/** Etiqueta del item en el idioma language */
	private String etiqueta;

	/** descripcion del item en el idioma language */
	private String descripcion;

	/**
	 * Devuelve item de examen
	 * 
	 * @return item
	 */
	@ManyToOne(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
    @JoinColumn(name = "idItemExamen")
	public ItemExamen getItemExamen() {
		return itemExamen;
	}

	/**
	 * Asigna  item de examen
	 * 
	 * @param item
	 *          item de examen
	 */
	public void setItemExamen(ItemExamen itemExamen) {
		this.itemExamen = itemExamen;
	}

	/**
	 * Devuelve language
	 * 
	 * @return language
	 */
	@ManyToOne(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
    @JoinColumn(name = "idLanguage", nullable = false)
	public Language getLanguage() {
		return language;
	}

	/**
	 * Asigna  language
	 * 
	 * @param language
	 *          language
	 */
	public void setLanguage(Language language) {
		this.language = language;
	}

	/**
	 * Asigna nombre.
	 * 
	 * @param nombre
	 *          nombre
	 */
	public void setNombre(String nombre) {
	  this.nombre = nombre;
	}

	/**
	 * Devuelve nombre.
	 * 
	 * @return nombre
	 */
	@Length(max = 150)
	@NotEmpty
	@NotNull
	public String getNombre() {
	  return nombre;
	}

	/**
	 * Asigna etiqueta.
	 * 
	 * @param etiqueta
	 *          etiqueta
	 */
	public void setEtiqueta(String etiqueta) {
	  this.etiqueta = etiqueta;
	}

	/**
	 * Devuelve etiqueta.
	 * 
	 * @return etiqueta
	 */
	@Length(max = 150)
	@NotEmpty
	@NotNull
	public String getEtiqueta() {
	  return etiqueta;
	}

	/**
	 * Asigna descripcion.
	 * 
	 * @param descripcion
	 *          descripcion
	 */
	public void setDescripcion(String descripcion) {
	  this.descripcion = descripcion;
	}

	/**
	 * Devuelve descripcion.
	 * 
	 * @return descripcion
	 */
	public String getDescripcion() {
	  return descripcion;
	}
}
