package es.csa.fdt.cyldigital.language.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.lang.String;
import javax.persistence.*;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.servicios.entity.PropuestaServicio;
import es.csa.fdt.cyldigital.servicios.entity.Servicio;

/**
 * Entity implementation class for Entity: TipoPracticaLanguage
 *
 */
@Entity
@Table(name="propuestaserviciolanguages", uniqueConstraints = @UniqueConstraint(columnNames = { "idPropuestaServicio", "idLanguage" }))
@Audited
@AttributeOverride(name = "id", column = @Column(name = "idPropuestaServicioLanguage"))
public class PropuestaServicioLanguage extends AbstractEntity implements Serializable {

	private static final long serialVersionUID = 6042443823210602474L;

	private PropuestaServicio propuesta;
	
	private Language language;
	
	/** Nombre de la propuesta de servicio en el idioma language */ 
	private String nombre;
	
	/** Comentarios de la propuesta de servicio en el idioma language */
	private String comentarios;
	
	/**
	 * Devuelve propuesta
	 * 
	 * @return propuesta
	 */
	@ManyToOne(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
    @JoinColumn(name = "idPropuestaServicio")
	public PropuestaServicio getPropuesta() {
		return propuesta;
	}

	/**
	 * Asigna  propuesta
	 * 
	 * @param propuesta
	 *          propuesta
	 */
	public void setPropuesta(PropuestaServicio propuesta) {
		this.propuesta = propuesta;
	}
	
	/**
	 * Devuelve language
	 * 
	 * @return language
	 */
	@ManyToOne(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
    @JoinColumn(name = "idLanguage", nullable = false)
	public Language getLanguage() {
		return language;
	}

	/**
	 * Asigna  language
	 * 
	 * @param language
	 *          language
	 */
	public void setLanguage(Language language) {
		this.language = language;
	}
	
	/**
	 * Devuelve nombre
	 * 
	 * @return nombre
	 */
	@Length(min = 1, max = 100)
	@NotNull
	public String getNombre() {
		return this.nombre;
	}

	/**
	 * Asigna  nombre
	 * 
	 * @param nombre
	 *          nombre
	 */
	public void setNombre(String nombre) {
		this.nombre = nombre;
	}
	
	/**
	   * Devuelve comentarios.
	   * 
	   * @return comentarios
	   */
	  @Lob
	  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
	  public String getComentarios() {
	    return comentarios;
	  }

	  /**
	   * Asigna comentarios.
	   * 
	   * @param comentarios
	   *          comentarios
	   */
	  public void setComentarios(String comentarios) {
	    this.comentarios = comentarios;
	  }
}
