package es.csa.fdt.cyldigital.language.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.lang.String;
import javax.persistence.*;

import org.hibernate.envers.Audited;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.servicios.entity.ServicioCurso;
import es.csa.fdt.cyldigital.core.entity.File;
import es.csa.fdt.cyldigital.core.entity.FileOnDisk;

/**
 * Entity implementation class for Entity: ServicioCursoLanguage
 *
 */
@Entity
@Table(name="serviciocursolanguages")
//@AttributeOverride(name = "idServicioLanguage", column = @Column(name = "idServicioCursoLanguage"))
@Audited
@PrimaryKeyJoinColumn(name = "idServicioLanguage")
public class ServicioCursoLanguage extends ServicioLanguage implements Serializable {

	private static final long serialVersionUID = 8041157930305482393L;
	
	private ServicioCurso servicioCurso;
	
	/** Guión del curso en el idioma language */ 
	private String guion;
	
	/** Descriptor del fichero del guión del curso en el idioma language */
	private File guionPdf;
	
	/** Fichero del guión del curso en el idioma language */
	private FileOnDisk ficheroGuion;

	/**
	 * Devuelve servicioCurso
	 * 
	 * @return servicioCurso
	 */
	@ManyToOne(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
    @JoinColumn( name = "idServicioCurso")
	public ServicioCurso getServicioCurso() {
		return servicioCurso;
	}

	/**
	 * Asigna  servicioCurso
	 * 
	 * @param servicioCurso
	 *          servicioCurso
	 */
	public void setServicioCurso(ServicioCurso servicioCurso) {
		this.servicioCurso = servicioCurso;
	}

	/**
	 * Devuelve guion
	 * 
	 * @return guion
	 */
	@Lob
	@Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
	public String getGuion() {
		return this.guion;
	}

	/**
	 * Asigna  guion
	 * 
	 * @param guion
	 *          guion
	 */
	public void setGuion(String guion) {
		this.guion = guion;
	}

	/**
	 * Devuelve guion pdf.
	 * 
	 * @return guion pdf
	 */
	 @ManyToOne(fetch = FetchType.LAZY, cascade = { javax.persistence.CascadeType.ALL })
	 @JoinColumn(name = "idGuionPdf")
	 public File getGuionPdf() {
		 return guionPdf;
	 }
	
	/**
	 * Asigna guion pdf.
	 * 
	 * @param guionPdf
	 *          guion pdf
	 */
	public void setGuionPdf(File guionPdf) {
	    this.guionPdf = guionPdf;
	}
	
	/**
	 * Devuelve fichero guion.
	 * 
	 * @return fichero guion
	 */
	@ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
	@JoinColumn(name = "idFicheroGuion")
	public FileOnDisk getFicheroGuion() {
	    return ficheroGuion;
	}
	
	/**
	 * Asigna fichero guion.
	 * 
	 * @param ficheroGuion
	 *          fichero guion
	 */
	public void setFicheroGuion(FileOnDisk ficheroGuion) {
	    this.ficheroGuion = ficheroGuion;
	}
}
