package es.csa.fdt.cyldigital.language.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.lang.String;
import javax.persistence.*;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.servicios.entity.Servicio;

/**
 * Entity implementation class for Entity: SevicioLanguage
 *
 */
@Entity
@Table(name="seviciolanguages", uniqueConstraints = @UniqueConstraint(columnNames = { "idServicio", "idLanguage" }))
@Audited
@Inheritance(strategy = InheritanceType.JOINED)
@AttributeOverride(name = "id", column = @Column(name = "idServicioLanguage"))
public class ServicioLanguage extends AbstractEntity implements Serializable {
	
	private static final long serialVersionUID = 323369072747731698L;

	private Servicio servicio;
	
	private Language language;
	
	/** Nombre del servicio servicio en el idioma language */ 
	private String nombre;
	
	/** Descripción del servicio servicio en el idioma language */
	private String descripcion;

	/**
	 * Devuelve servicio
	 * 
	 * @return servicio
	 */
	@ManyToOne(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
    @JoinColumn(name = "idServicio")
	public Servicio getServicio() {
		return servicio;
	}

	/**
	 * Asigna  servicio
	 * 
	 * @param servicio
	 *          servicio
	 */
	public void setServicio(Servicio servicio) {
		this.servicio = servicio;
	}

	/**
	 * Devuelve language
	 * 
	 * @return language
	 */
	@ManyToOne(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
    @JoinColumn(name = "idLanguage", nullable = false)
    @NotNull
	public Language getLanguage() {
		return language;
	}

	/**
	 * Asigna  language
	 * 
	 * @param language
	 *          language
	 */
	public void setLanguage(Language language) {
		this.language = language;
	}

	/**
	 * Devuelve nombre
	 * 
	 * @return nombre
	 */
	@Length(min = 1, max = 100)
	@NotNull
	public String getNombre() {
		return this.nombre;
	}

	/**
	 * Asigna  nombre
	 * 
	 * @param nombre
	 *          nombre
	 */
	public void setNombre(String nombre) {
		this.nombre = nombre;
	}

	/**
	 * Devuelve descripcion
	 * 
	 * @return descripcion
	 */
	@Lob
	@Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
	public String getDescripcion() {
		return this.descripcion;
	}

	/**
	 * Asigna  descripcion
	 * 
	 * @param descripcion
	 *          descripcion
	 */
	public void setDescripcion(String descripcion) {
		this.descripcion = descripcion;
	}
}
