package es.csa.fdt.cyldigital.language.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.lang.String;
import javax.persistence.*;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.servicios.entity.TipoPractica;

/**
 * Entity implementation class for Entity: TipoPracticaLanguage
 *
 */
@Entity
@Table(name="tipopracticalanguages", uniqueConstraints = @UniqueConstraint(columnNames = { "idTipoPractica", "idLanguage" }))
@Audited
@AttributeOverride(name = "id", column = @Column(name = "idTipoPracticaLanguage"))
public class TipoPracticaLanguage extends AbstractEntity implements Serializable {

	private static final long serialVersionUID = 5660693282058297965L;

	private TipoPractica tipoPractica;
	
	private Language language;
	
	/** Nombre del tipo de práctica en el idioma language */ 
	private String nombre;
	
	/**
	 * Devuelve tipoPractica
	 * 
	 * @return tipoPractica
	 */
	@ManyToOne(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
    @JoinColumn(name = "idTipoPractica")
	public TipoPractica getTipoPractica() {
		return tipoPractica;
	}

	/**
	 * Asigna  tipoPractica
	 * 
	 * @param tipoPractica
	 *          tipoPractica
	 */
	public void setTipoPractica(TipoPractica tipoPractica) {
		this.tipoPractica = tipoPractica;
	}

	/**
	 * Devuelve language
	 * 
	 * @return language
	 */
	@ManyToOne(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
    @JoinColumn(name = "idLanguage", nullable = false)
	public Language getLanguage() {
		return language;
	}

	/**
	 * Asigna  language
	 * 
	 * @param language
	 *          language
	 */
	public void setLanguage(Language language) {
		this.language = language;
	}

	/**
	 * Devuelve nombre
	 * 
	 * @return nombre
	 */
	@Length(max = 255)
	@NotEmpty
	@NotNull
	public String getNombre() {
		return this.nombre;
	}

	/**
	 * Asigna  nombre
	 * 
	 * @param nombre
	 *          nombre
	 */
	public void setNombre(String nombre) {
		this.nombre = nombre;
	}
}
