package es.csa.fdt.cyldigital.language.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.language.entity.ActividadExternaLanguage;
import es.csa.fdt.cyldigital.language.entity.Language;
import es.csa.fdt.cyldigital.language.entity.ServicioLanguage;
import es.csa.fdt.cyldigital.servicios.entity.ActividadExterna;
import es.csa.fdt.cyldigital.servicios.entity.ObservacionActividad;
import es.csa.fdt.cyldigital.servicios.entity.PreinscripcionCurso;
import es.csa.fdt.cyldigital.servicios.entity.PreinscripcionTaller;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario.Role;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;

/**
 * Clase ActividadExternaLanguageServiceImpl.
 * 
 * @author it-pronet
 */
@Service
@Repository
public class ActividadExternaLanguageServiceImpl extends AbstractServiceImpl<ActividadExternaLanguage, Long>
												 implements ActividadExternaLanguageService {

  private static final long serialVersionUID = 682451130095720418L;
  
  //private boolean areThereFicherosGuiones;
  
  private String nombreServicio;

  private boolean borrarPreinscripcion = false;

  /**
   * Construye un nuevo objeto de la clase
   */
  public ActividadExternaLanguageServiceImpl() {
    super();
    
    //areThereFicherosGuiones = false;
  }
  
  /*public boolean getAreThereFicherosGuiones() {
		return areThereFicherosGuiones;
	}*/

	/*private void setAreThereFicherosGuiones(ActividadExternaLanguage ael) {
		areThereFicherosGuiones = false;*/

		/*if (ael != null) {
			if (ael.getExterna().isCurso()) {
				for (ServicioCursoLanguage servicioCursoLanguage : ((ServicioCurso)ael.getExterna().getServicio()).getGuiones()) {
					if (servicioCursoLanguage.getFicheroGuion() != null) {
						areThereFicherosGuiones = true;

						break;
					}
				}
			}
			else if (ael.getExterna().isTaller()) {
				if (((ServicioTaller)ael.getExterna().getServicio()).getFicheroGuion() != null) {
					areThereFicherosGuiones = true;
				}
			}
		}*/
	/*}*/
  
	public ActividadExternaLanguage find(Long id, Language language) {
		HashMap<String, Object> propiedades = new HashMap<String, Object>();
		propiedades.put("externa.id", id);
		propiedades.put("language.id", language.getId());
	
		ActividadExternaLanguage ael = super.findOneWhere(propiedades);
		
		//setAreThereFicherosGuiones(ael);
		setNombreServicio(ael);
		
		return ael;
	}
	
	public void clear(ActividadExternaLanguage ael) {
		clear();

		//setAreThereFicherosGuiones(ael);
	}
	
	/**
	   * Devuelve next externa.
	   * 
	   * next=false: devuelve, si hay, la actividad anterior. A true, la posterior. Recordar que la actividad externa
	   * anterior es de fecha posterior al actual. Devuelve null si no hay una incidencia anterior/posterior, si hay,
	   * devuelve la actividad.
	   * 
	   * @param u
	   *           u
	   * @param externa
	   *           externa
	   * @param next
	   *           next
	   * @return next externa
	   */
	  public ActividadExterna getNextExterna(UsuarioLdap u, ActividadExterna externa, boolean next) {
	    List<ActividadExterna> externas = new ArrayList<ActividadExterna>();
	    
	    if (u.getRolesList().contains(Usuario.Role.ROLE_EXTERN)) { //Colaborador externo
			Empleado empleado = find(Empleado.class, u.getUser().getId());
			
			if (empleado.getProveedor() == null) {
				externas = findByQuery(ActividadExterna.class,
						   "SELECT e " +
						   "FROM es.csa.fdt.cyldigital.servicios.entity.ActividadExterna e " +
						   "WHERE e.colaborador.id = " + empleado.getId() + " " +
						   "ORDER BY e.fechaInicio DESC, e.horaInicio DESC");
			}
			else {
				externas = findByQuery(ActividadExterna.class,
						   "SELECT e " +
						   "FROM es.csa.fdt.cyldigital.servicios.entity.ActividadExterna e " +
						   "WHERE e.colaborador.proveedor.id = " + empleado.getProveedor().getId() + " " +
						   "ORDER BY e.fechaInicio DESC, e.horaInicio DESC");
			}
	    }
	    else if (u.getRolesList().contains(Usuario.Role.ROLE_AREA_MANAGER) || 
	    		 u.getRolesList().contains(Usuario.Role.ROLE_DIGITAL_ALLY)) { //Responsable de centro
			externas = findByQuery(ActividadExterna.class,
					   "SELECT e " +
					   "FROM es.csa.fdt.cyldigital.servicios.entity.ActividadExterna e " +
					   "WHERE e.centro.id = " + u.getUser().getCentro().getId() + " " +
					   "ORDER BY e.fechaInicio DESC, e.horaInicio DESC");
	    }
	    else { //u.getRolesList().contains(Usuario.Role.ROLE_ADMIN) //Administrador
	    	externas = findByQuery(ActividadExterna.class,
					   "SELECT e " +
					   "FROM es.csa.fdt.cyldigital.servicios.entity.ActividadExterna e " +
					   "ORDER BY e.fechaInicio DESC, e.horaInicio DESC");
	    }

	    if (next) {
	    	if (externas.indexOf(externa) + 1 < externas.size())
	    		return externas.get(externas.indexOf(externa) + 1);
	    }
	    else {
	    	if (externas.indexOf(externa) != 0)
	    		return externas.get(externas.indexOf(externa) - 1);
	    }
	    
	    return null;
	  }
	  //RF_XEA_12 Funcion que devuelve las actividades anterior y posterior
		/**
	   * Devuelve next externa.
	   * 
	   * next=false: devuelve, si hay, la actividad anterior. A true, la posterior. Recordar que la actividad externa
	   * anterior es de fecha posterior al actual. Devuelve null si no hay una incidencia anterior/posterior, si hay,
	   * devuelve la actividad.
	   * 
	   * @param u
	   *           u
	   * @param externa
	   *           externa
	   * @param next
	   *           next
	   * @return next externa
	   */
	  public ActividadExterna getNextExternaNew(UsuarioLdap u, ActividadExterna externa, boolean next, List<Filter<CriterionJPAQL, ActividadExternaLanguage>> filtros, Language idioma) {
	    
	    if (null != filtros){
		
			String query = "";
			SimpleDateFormat formatoFecha = new SimpleDateFormat("yyyy-MM-dd");
			List<ActividadExterna> externas = new ArrayList<ActividadExterna>(); 
			
			if ("es_ES".equals(idioma.getISO())) {
				query = "SELECT e from es.csa.fdt.cyldigital.language.entity.ActividadExternaLanguage e WHERE (e.language.ISO = 'es_ES')";
			} else {
				query = "SELECT e from es.csa.fdt.cyldigital.language.entity.ActividadExternaLanguage e WHERE (e.language.ISO = 'gl_ES')";
			}
			
			query += " AND ( ( e.desactivado is null or e.desactivado = false ) )";
			
			if (u.getRolesList().contains(Usuario.Role.ROLE_EXTERN)) { //Colaborador externo
				Empleado empleado = find(Empleado.class, u.getUser().getId());
				
				if (empleado.getProveedor() == null) {
					query += " AND (e.externa.colaborador.id = " + empleado.getId() + ")";
				}
				else {
					query += " AND (e.externa.colaborador.proveedor.id = " + empleado.getProveedor().getId() + ")";
				}
			}
			else if (u.getRolesList().contains(Usuario.Role.ROLE_AREA_MANAGER) ||
					 u.getRolesList().contains(Usuario.Role.ROLE_DIGITAL_ALLY)) { //Responsable de centro
				
				query += " AND (e.externa.centro.id = " + u.getUser().getCentro().getId() + ")";
		    }
		    // En caso de que no sea ni colaborador externo, ni responsable del centro, será el administrador
			
			//Filtros seleccionados desde la lista
			String condicion = "";
			String valor = "";
			for (Filter<CriterionJPAQL, ActividadExternaLanguage> f : filtros){
				
				//Centro
				if (f.getCriterion().getQuery().contains("externa.centro.id")){
					
					for (Object o : f.getCriterion().getParameters()) {
						valor = o.toString();
					}
					condicion = "e.externa.centro.id = " + valor;
					query += " AND (" + condicion + ")";
				}
				
				//Tematica
				if (f.getCriterion().getQuery().contains("tematica.id")){
					
					for (Object o : f.getCriterion().getParameters()) {
						valor = o.toString();
					}
					condicion = "(externa.servicio.tematica.id = " + valor + ") OR (externa.tematica.id = " + valor + ")"; 
					query += " AND (" + condicion + ")";
				}
				
				//Nombre
				if (f.getCriterion().getQuery().contains("apodo")){
					
					String cadena = "";
					for (Object o : f.getCriterion().getParameters()) {
						cadena += o.toString().substring(1, o.toString().length()-1) + "%";
					} 
					condicion = "LOWER(apodo) LIKE LOWER('%" + cadena + "')"; 
					query += " AND (" + condicion + ")";
				}
				
				//ID de la actividad
				if (f.getCriterion().getQuery().contains("externa.id")){
					
					for (Object o : f.getCriterion().getParameters()) {
						valor = o.toString();
					}
					condicion = "e.externa.id = " + valor;
					query += " AND (" + condicion + ")";
				}
				
				//Fecha inicio
				if (f.getCriterion().getQuery().contains("externa.fechaInicio")){
					
					for (Object o : f.getCriterion().getParameters()) {
						valor = formatoFecha.format((Date)o);
					}
					condicion = "e.externa.fechaInicio = '" + valor + "'";
					query += " AND (" + condicion + ")";
				}
				
				//Fecha fin
				if (f.getCriterion().getQuery().contains("externa.fechaFin")){
					
					for (Object o : f.getCriterion().getParameters()) {
						valor = formatoFecha.format((Date)o);
					}
					condicion = "e.externa.fechaFin = '" + valor + "'";;
					query += " AND (" + condicion + ")";
				}
			}
			
		    query += " ORDER BY e.externa.fechaInicio DESC, e.externa.horaInicio DESC";
		    
		    List<ActividadExternaLanguage> externasLang = new ArrayList<ActividadExternaLanguage>();
		    
		    externasLang = findByQuery(ActividadExternaLanguage.class, query);
		    
		    for (ActividadExternaLanguage ext : externasLang){
		    	externas.add(ext.getExterna());
		    }
	
		    if (next) {
		    	if (externas.indexOf(externa) + 1 < externas.size())
		    		return externas.get(externas.indexOf(externa) + 1);
		    }
		    else {
		    	if (externas.indexOf(externa) > 0)
		    		return externas.get(externas.indexOf(externa) - 1);
		    	else
		    		return null;
		    }
		    
		    return null;
		    
	    } else {
	    	
	    	return null;
	    }
}
	  
	  /**
	   * Anadir observacion guion.
	   * 
	   * @param externa
	   *           externa
	   * @param o
	   *           o
	   * @param usuario
	   *           usuario
	   */
	  public void anadirObservacionGuion(ActividadExterna externa, ObservacionActividad o, Usuario usuario) {
	    if (o.getObservacion().equals("")) addWarnMessage("No puede añadir un comentario vacío");
	    else {
	      o.setFecha(new Date());
	      o.setUsuario(usuario);
	      o.setActividad(externa);
	      externa.getObservacionesActividad().add(o);
	      persist(externa);
	    }
	  }
	  
	  /**
	   * Devuelve nueva observacion guion.
	   * 
	   * @return nueva observacion guion
	   */
	  public ObservacionActividad getNuevaObservacionGuion() {
	    return new ObservacionActividad();
	  }
	  
	  /**
	   * Matricula from preinscripcion.
	   * 
	   * @param externa
	   *           externa
	   * @param idPreinscripcion
	   *           id preinscripcion
	   * @return true, if successful
	   */
	  public boolean matriculaFromPreinscripcion(ActividadExterna externa, Long idPreinscripcion) {
	    PreinscripcionCurso pc = null;
	    PreinscripcionTaller pt = null;

	   /* if (externa.isCurso()) {
	      pc = find(PreinscripcionCurso.class, idPreinscripcion);
	    } else if (externa.isTaller()) {
	      pt = find(PreinscripcionTaller.class, idPreinscripcion);
	    }

	    boolean flag = false;
	    if (pc != null) {
	      Long idUsuario = pc.getUsuario().getId();
	      Usuario u = find(Usuario.class, idUsuario);

	      for (MatriculaExterna matr : externa.getMatriculas()) {
	        if (matr.getUsuario().getId() == idUsuario) {
	          flag = true;
	          addErrorMessage("El usuario ya está matriculado en esta actividad externa");
	        }
	      }

	      if (!flag) {
	        ((ServicioCurso) externa.getServicio()).getPreinscripciones().remove(pc);
	        u.getPreinscripcionesCursos().remove(pc);

	        MatriculaExterna m = new MatriculaExterna();
	        m.setAdmitido(true);
	        m.setFechaMatricula(new Date());

	        m.setExterna(externa);
	        m.setUsuario(u);
	        persist(m);

	        setMessagesEnabled(false);
	        remove(pc);
	        setMessagesEnabled(true);
	        return true;
	      }
	    }

	    else if (pt != null) {
	      Long idUsuario = pt.getUsuario().getId();
	      Usuario u = find(Usuario.class, idUsuario);

	     /* for (MatriculaExterna matr : externa.getMatriculas()) {
	        if (matr.getUsuario().getId() == idUsuario) {
	          flag = true;
	          addErrorMessage("El usuario ya está matriculado en esta actividad externa");
	        }
	      }*/

	  /*   if (!flag) {
	        ((ServicioTaller) externa.getServicio()).getPreinscripciones().remove(pt);
	        u.getPreinscripcionesTalleres().remove(pt);

	        MatriculaExterna m = new MatriculaExterna();
	        m.setAdmitido(true);
	        m.setFechaMatricula(new Date());

	        m.setExterna(externa);
	        m.setUsuario(u);
	        persist(m);

	        setMessagesEnabled(false);
	        remove(pt);
	        setMessagesEnabled(true);
	        return true;
	      }
	    }*/
	    return false;
	  }
  
  /**
   * Checks if is borrar preinscripcion.
   * 
   * @return true, if is borrar preinscripcion
   */
  public boolean isBorrarPreinscripcion() {
    return borrarPreinscripcion;
  }

  /**
   * Asigna borrar preinscripcion.
   * 
   * @param borrarPreinscripcion
   *          borrar preinscripcion
   */
  public void setBorrarPreinscripcion(boolean borrarPreinscripcion) {
    this.borrarPreinscripcion = borrarPreinscripcion;
  }

  /**
   * Rechazar actividad externa.
   * 
   * @param externa
   *           externa
   * @param idPreinscripcion
   *           id preinscripcion
   */
  public void rechazarActividadExterna(ActividadExterna externa, Long idPreinscripcion) {
    PreinscripcionCurso pc = null;
    PreinscripcionTaller pt = null;

   /* if (externa.isCurso()) {
      pc = find(PreinscripcionCurso.class, idPreinscripcion);
    } else if (externa.isTaller()) {
      pt = find(PreinscripcionTaller.class, idPreinscripcion);
    }

    if (borrarPreinscripcion) {
      if (pc != null) remove(pc);
      else if (pt != null) remove(pt);
    } else {
      if (pc != null) {
        pc.getExternasCursosRechazos().add(externa);
        pc.setPeso(pc.getPeso() + 1);
        persist(pc);
      } else if (pt != null) {
        pt.getExternasTalleresRechazos().add(externa);
        pt.setPeso(pt.getPeso() + 1);
        persist(pt);
      }
    }*/
  }
  
  private void setNombreServicio(ActividadExternaLanguage ael) {
	  if (ael.getExterna().getServicio() != null) {
		  for (ServicioLanguage s : ael.getExterna().getServicio().getTextos())
			  if (s.getLanguage().getISO().compareTo(ael.getLanguage().getISO()) == 0)
				  this.nombreServicio = s.getNombre();
	  }
  }
  
  public String getNombreServicio() {
	  return nombreServicio;
  }
  
  public void cerrarActividadExterna(ActividadExterna externa, Usuario usuario) {
	  if ((externa != null) && (externa.getId() != 0) && externa.isCerrable()) {
		  Empleado empleado = find(Empleado.class, usuario.getId());

		  	  //Con permisos de administrador
		  if (usuario.getRolesList().contains(Role.ROLE_ADMIN)
			  ||
			  //Con permisos de colaborador o solicitante de actividades externas
			  //y dueño de la actividad o de la misma entidad colaboradora que el dueño
		      ((usuario.getRolesList().contains(Role.ROLE_ACTIVITY_EXTERNAL_REQUEST) ||
		        usuario.getRolesList().contains(Role.ROLE_EXTERN)) &&
		       ((empleado.getId() == externa.getColaborador().getId()) ||
    		    ((empleado.getProveedor() != null) && (externa.getColaborador().getProveedor() != null) && 
		    	 (empleado.getProveedor().getId() == externa.getColaborador().getProveedor().getId()) &&
		         (empleado.getId() != externa.getColaborador().getId()))))
		      ||
		      //Con permisos de responsable de centro o edición de actividades y perteneciente al mismo centro
		      //al que pertenece la actividad de tipo cesión de aula.
		      ((usuario.getRolesList().contains(Role.ROLE_ACTIVITY_EDIT) ||
		        usuario.getRolesList().contains(Role.ROLE_AREA_MANAGER) ||
		        usuario.getRolesList().contains(Role.ROLE_DIGITAL_ALLY)) &&
		       ((externa.getCentro() != null) && (externa.getCentro().getId() == usuario.getCentro().getId())))
		     ) {
			  externa.setCerrada(true);
			  
			  Date horaFinEncuesta = new Date(System.currentTimeMillis() +
						  (Integer.parseInt(findOneWhere(Parameter.class, "nombre", "anonymous_survey_time").getValor()) * 60000L));

			  externa.setFinEncuesta(horaFinEncuesta);
			  
			  setMessagesEnabled(false);
			  persist(externa);
			  setMessagesEnabled(true);
			  
			  addInfoMessage("actividad.externa.exitoCerrar");
		  }
		  else {
			addErrorMessage("actividad.externa.errorCerrar");  
		  }
	  }
	  else {
		  addErrorMessage("actividad.externa.errorCerrar");
	  }
  }
}
