package es.csa.fdt.cyldigital.language.service;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.List;

import javax.persistence.Query;

import org.apache.commons.codec.binary.Base64;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.commons.persistence.browser.jpaql.FilterJPAQLByQuery;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.service.BrowserFilteredByActive;
import es.csa.fdt.cyldigital.language.entity.ActividadExternaLanguage;
import es.csa.fdt.cyldigital.language.entity.Language;
import es.csa.fdt.cyldigital.servicios.entity.ActividadExterna;
import es.csa.fdt.cyldigital.servicios.entity.Tematica;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.Proveedor;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;

/**
 * Servicio del Browser de Actividades Externas Language.
 * 
 * @author csa
 */
@Service
@Repository
public class ActividadesExternasLanguageBrowserImpl extends BrowserFilteredByActive<ActividadExternaLanguage>
													implements ActividadesExternasLanguageBrowser<CriterionJPAQL, String> {
	private static final long serialVersionUID = -7018853590578734831L;
	
	private Centro centro;
	private Proveedor proveedor;
	private Tematica tematica;
	private Long indIniciativaSingular;
	private Long indVideoconferencia;
	private Language language;
	private Date fechaInicio;
	private Date fechaFin;
	
  /**
   * Construye un nuevo objeto de la clase
   */
  public ActividadesExternasLanguageBrowserImpl() {
	getSort().setSortProperty("externa.fechaInicio DESC, externa.horaInicio DESC");
	
	tematica = null;
	centro = null;
  }

  public void applyLanguageCondition(Language language) {
	  // Si no hay condición de idioma o la condición de idioma debe cambiarse
	  if ((this.language == null) || (this.language != language)) {
		  // Busca condición de idioma antigua
		  int i = -1;
		  
		  for(Filter<CriterionJPAQL, ActividadExternaLanguage> condition: this.getConditions())
			  if (condition.getCriterion().getQuery().contains("e.language")) {
				  i = getConditions().indexOf(condition);
				  break;
			  }

		  // Borra condición de idioma antigua si existía
		  if (i >= 0)
			  this.getConditions().remove(i);
		  
		  // Guarda el idioma actual
		  this.language = language;
		  
		  //Aplica la condición de idioma
		  this.getCondition().getCriterion().setQuery("e.language = ?");
		  this.getCondition().getCriterion().addParameter(language);
			
		  this.applyCondition();
	  }
  }
  
  /**
   * Devuelve centro.
   * 
   * @return centro
   */
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Devuelve proveedor.
   * 
   * @return proveedor.
   */
  public Proveedor getProveedor() {
    return proveedor;
  }

  /**
   * Asigna provedor.
   * 
   * @param provedor
   *          provedor
   */
  public void setProveedor(Proveedor proveedor) {
    this.proveedor = proveedor;
  }  
  

  /**
   * Devuelve tematica.
   * 
   * @return tematica
   */
  public Tematica getTematica() {
    return tematica;
  }

  /**
   * Asigna tematica.
   * 
   * @param tematica
   *          tematica
   */
  public void setTematica(Tematica tematica) {
    this.tematica = tematica;
  }
  
    /**
	 * 
	 * @return
   	 */
	public Long getIndIniciativaSingular() {
		return indIniciativaSingular;
	}

    /**
    * 
    * @param IniciativaSingular
	*/
	public void setIndIniciativaSingular(Long indIniciativaSingular) {
		this.indIniciativaSingular = indIniciativaSingular;
	}
  
	/**
	 * 
	 * @return
	 */
	public Long getIndVideoconferencia() {
		return indVideoconferencia;
	}

	/**
	 * 
	 * @param Videoconferencia
	 */
	public void setIndVideoconferencia(Long indVideoconferencia) {
		this.indVideoconferencia = indVideoconferencia;
	}
	
  	/**
	 * @return the fechaInicio
	 */
	public Date getFechaInicio() {
		return fechaInicio;
	}

	/**
	 * @param fechaInicio 
	 * 		  the fechaInicio to set
	 */
	public void setFechaInicio(Date fechaInicio) {
		this.fechaInicio = fechaInicio;
	}
	
	/**
	 * @return the fechaFin
	 */
	public Date getFechaFin() {
		return fechaFin;
	}

	/**
	 * @param fechaInicio 
	 * 		  the fechaInicio to set
	 */
	public void setFechaFin(Date fechaFin) {
		this.fechaFin = fechaFin;
	}
  
  /**
   * Filter old.
   */
  public void filterOld() {
    removeFilterFecha();
    filterOldActividadesExternas();
  }

  private void removeFilterFecha() {
    int i = -1;
    for (Filter<CriterionJPAQL, ActividadExternaLanguage> f : getFilters())
      if (f.getCriterion().getQuery().contains("externa.fechaFin")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
  }

  private void filterOldActividadesExternas() {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<ActividadExternaLanguage> filter = new FilterJPAQLByQuery<ActividadExternaLanguage>();

    query += " externa.fechaFin >= now() ";

    for (Filter<CriterionJPAQL, ActividadExternaLanguage> f : getFilters())
      if (f.getCriterion().getQuery().contains("externa.fechaFin")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }
  
  //3.4.8 - RF_15_20 Consulta e exportación da información das actividades
  	public void filterFechas() {
		if (getFechaInicio() != null && getFechaFin() != null) {
			removeFilterFechaInicio();
			removeFilterFechaFin();
			addFilterFechas();
		} else if (getFechaInicio() != null && getFechaFin() == null) {
			removeFilterFechaInicio();
			addFilterFechaInicio();
		} else if (getFechaInicio() == null && getFechaFin() != null) {
			removeFilterFechaFin();
			addFilterFechaFin();
		}
	}

	private void removeFilterFechaInicio() {
		int i = -1;
		
		for (Filter<CriterionJPAQL, ActividadExternaLanguage> f : getFilters())
			if (f.getCriterion().getQuery().contains("externa.fechaFin"))
				i = getFilters().indexOf(f);
		
		if (i > -1)
			getFilters().remove(i);
	}

	private void addFilterFechaInicio() {
		String query = "";
		boolean flag = false;
		
		FilterJPAQLByQuery<ActividadExternaLanguage> filter = new FilterJPAQLByQuery<ActividadExternaLanguage>();

		query += " externa.fechaFin >= ? ";
		filter.getCriterion().addParameter(getFechaInicio());

		for (Filter<CriterionJPAQL, ActividadExternaLanguage> f : getFilters())
			if (f.getCriterion().getQuery().contains("externa.fechaFin"))
				flag = true;

		if (!flag) {
			filter.getCriterion().setQuery(query);
			getFilters().add(filter);
		}
	}
	
	private void removeFilterFechaFin() {
		int i = -1;
		
		for (Filter<CriterionJPAQL, ActividadExternaLanguage> f : getFilters())
			if (f.getCriterion().getQuery().contains("externa.fechaInicio"))
				i = getFilters().indexOf(f);
		
		if (i > -1)
			getFilters().remove(i);
	}

	private void addFilterFechaFin() {
		String query = "";
		boolean flag = false;
		
		FilterJPAQLByQuery<ActividadExternaLanguage> filter = new FilterJPAQLByQuery<ActividadExternaLanguage>();

		query += " externa.fechaInicio <= ? ";
		filter.getCriterion().addParameter(getFechaFin());

		for (Filter<CriterionJPAQL, ActividadExternaLanguage> f : getFilters())
			if (f.getCriterion().getQuery().contains("externa.fechaInicio"))
				flag = true;

		if (!flag) {
			filter.getCriterion().setQuery(query);
			getFilters().add(filter);
		}
	}
	
	private void addFilterFechas() {
		String query = "";
		boolean flag = false;
		
		FilterJPAQLByQuery<ActividadExternaLanguage> filter = new FilterJPAQLByQuery<ActividadExternaLanguage>();

		query += " externa.fechaFin >= ? ";
		filter.getCriterion().addParameter(getFechaInicio());
		query += " AND externa.fechaInicio <= ? ";
		filter.getCriterion().addParameter(getFechaFin());

		for (Filter<CriterionJPAQL, ActividadExternaLanguage> f : getFilters())
			if (f.getCriterion().getQuery().contains("externa.fechaFin") && 
					f.getCriterion().getQuery().contains("externa.fechaInicio"))
				flag = true;

		if (!flag) {
			filter.getCriterion().setQuery(query);
			getFilters().add(filter);
		}
	}

  /**
   * Filter by curso.
   * 
   * @param idServicio
   *           id servicio
   */
  public void filterByCurso(Long idServicio) {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<ActividadExternaLanguage> filter = new FilterJPAQLByQuery<ActividadExternaLanguage>();

    if (idServicio != null && idServicio > 0) {
      filter.getCriterion().addParameter(idServicio);
      query += " externa.servicio.id = ? ";
    }

    for (Filter<CriterionJPAQL, ActividadExternaLanguage> f : getFilters())
      if (f.getCriterion().getQuery().equals(query)) flag = true;

    if (!query.isEmpty() && !flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }
  
  /**
   * Filter by taller.
   * 
   * @param idServicio
   *           id servicio
   */
  /*
  public void filterByTaller(Long idServicio) {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<ActividadExternaLanguage> filter = new FilterJPAQLByQuery<ActividadExternaLanguage>();

    if (idServicio != null && idServicio > 0) {
      filter.getCriterion().addParameter(idServicio);
      query += " externa.servicio.id = ? ";
    }

    for (Filter<CriterionJPAQL, ActividadExternaLanguage> f : getFilters())
      if (f.getCriterion().getQuery().equals(query)) flag = true;

    if (!query.isEmpty() && !flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }
  */

  /**
   * Filter tematica.
   */
  public void filterTematica() {
	  removeFilterTematica();
	  
	  if (getTematica() != null)
		  filterByTematica(getTematica().getId());
  }

  private void removeFilterTematica() {
	  int i = -1;
    
	  for (Filter<CriterionJPAQL, ActividadExternaLanguage> f : getFilters()) {
		  /*if (f.getCriterion().getQuery().contains("externa.servicio.tematica.id") &&
			  f.getCriterion().getQuery().contains("externa.tematica.id"))
			  i = getFilters().indexOf(f);*/
		  if (f.getCriterion().getQuery().contains("externa.tematica.id"))
			  i = getFilters().indexOf(f);		  
	  }
    
	  if (i > -1)
		  getFilters().remove(i);
  }

  /**
   * Filter by tematica.
   * 
   * @param idTematica
   *           id tematica
   */
  public void filterByTematica(Long idTematica) {
	  String query = "";
	  boolean flag = false;
	  FilterJPAQLByQuery<ActividadExternaLanguage> filter = new FilterJPAQLByQuery<ActividadExternaLanguage>();

	  //query += " (externa.servicio.tematica.id = ?) OR (externa.tematica.id = ?) ";
	  query += " externa.tematica.id = ? ";
	  filter.getCriterion().addParameter(idTematica);
	  //filter.getCriterion().addParameter(idTematica);

	  for (Filter<CriterionJPAQL, ActividadExternaLanguage> f : getFilters()) {
		  /*if (f.getCriterion().getQuery().contains("externa.servicio.tematica.id") && 
			  f.getCriterion().getQuery().contains("externa.tematica.id"))
			  flag = true;*/
		  if (f.getCriterion().getQuery().contains("externa.tematica.id"))
			  flag = true;		  
	  }

	  if (!flag) {
		  filter.getCriterion().setQuery(query);
		  getFilters().add(filter);
	  }
  }
  
	/**
	 * Filter indIniciativaSingular.
	 */
	public void filterIndIniciativaSingular() {
		removeIndIniciativaSingular();

		if (getIndIniciativaSingular() != null){
			filterByIndIniciativaSingular(getIndIniciativaSingular());
		}
	}

	/**
	 * removeIndIniciativaSingular
	 */
	private void removeIndIniciativaSingular() {
		int i = -1;
		
		for (Filter<CriterionJPAQL, ActividadExternaLanguage> f : getFilters())
			if (f.getCriterion().getQuery().contains("externa.iniciativaSingular"))
				i = getFilters().indexOf(f);

		if (i > -1)
			getFilters().remove(i);
	}

	/**
	 * Filter by indIniciativaSingular.
	 * 
	 * @param iniciativaSingular (0-todos;1-Si;2-No)
	 *           
	 */
	public void filterByIndIniciativaSingular(Long indIniciativaSingular) {
		String query = "";
		boolean flag = false;

		if(indIniciativaSingular == null || indIniciativaSingular.longValue() <= 0 )
			return;
		
		FilterJPAQLByQuery<ActividadExternaLanguage> filter = new FilterJPAQLByQuery<ActividadExternaLanguage>();

		query += " externa.iniciativaSingular = ? ";
		filter.getCriterion().addParameter(indIniciativaSingular.longValue()==1);

		for (Filter<CriterionJPAQL, ActividadExternaLanguage> f : getFilters())
			if (f.getCriterion().getQuery().contains("externa.iniciativaSingular"))
				flag = true;

		if (!flag) {
			filter.getCriterion().setQuery(query);
			getFilters().add(filter);
		}
	}
  
	/**
	 * Filter indVideoconferencia.
	 */
	public void filterIndVideoconferencia() {
		removeIndVideoconferencia();

		if (getIndVideoconferencia() != null){
			filterByIndVideoconferencia(getIndVideoconferencia());
		}
	}

	/**
	 * removeIndVideoconferencia
	 */
	private void removeIndVideoconferencia() {
		int i = -1;
		
		for (Filter<CriterionJPAQL, ActividadExternaLanguage> f : getFilters())
			if (f.getCriterion().getQuery().contains("externa.videoconferencia"))
				i = getFilters().indexOf(f);

		if (i > -1)
			getFilters().remove(i);
	}

	/**
	 * Filter by indVideoconferencia.
	 * 
	 * @param indVideoconferencia (0-todos;1-Si;2-No)
	 *           
	 */
	public void filterByIndVideoconferencia(Long indVideoconferencia) {
		String query = "";
		boolean flag = false;

		if(indVideoconferencia == null || indVideoconferencia.longValue() <= 0 )
			return;
		
		FilterJPAQLByQuery<ActividadExternaLanguage> filter = new FilterJPAQLByQuery<ActividadExternaLanguage>();

		query += " externa.videoconferencia = ? ";
		filter.getCriterion().addParameter(indVideoconferencia.longValue()==1);

		for (Filter<CriterionJPAQL, ActividadExternaLanguage> f : getFilters())
			if (f.getCriterion().getQuery().contains("externa.videoconferencia"))
				flag = true;

		if (!flag) {
			filter.getCriterion().setQuery(query);
			getFilters().add(filter);
		}
	}

	
  /**
   * Filter centro.
   */
  public void filterCentro() {
	  removeFilterCentro();
	  
	  if (getCentro() != null)
		  filterByCentro(getCentro().getId());
  }

  private void removeFilterCentro() {
	  int i = -1;
    
	  for (Filter<CriterionJPAQL, ActividadExternaLanguage> f : getFilters()) {
		  if (f.getCriterion().getQuery().contains("externa.centro.id"))
			  i = getFilters().indexOf(f);
	  }
    
	  if (i > -1)
		  getFilters().remove(i);
  }
  
  /**
   * Filter by centro.
   * 
   * @param idCentro
   *           id centro
   */
  public void filterByCentro(Long idCentro) {
	  String query = "";
	  boolean flag = false;
	  FilterJPAQLByQuery<ActividadExternaLanguage> filter = new FilterJPAQLByQuery<ActividadExternaLanguage>();

	  query += " (externa.centro.id = ?) ";
	  filter.getCriterion().addParameter(idCentro);

	  for (Filter<CriterionJPAQL, ActividadExternaLanguage> f : getFilters()) {
		  if (f.getCriterion().getQuery().contains("externa.centro.id"))
			  flag = true;
	  }

	  if (!flag) {
		  filter.getCriterion().setQuery(query);
		  getFilters().add(filter);
	  }
  }
  
  /**
   * Filter proveedor.
   */
  public void filterProveedor() {
	  removeFilterProveedor();
	  
	  if (getProveedor() != null)
		  filterByProveedor(getProveedor().getId());
  }
  
  private void removeFilterProveedor() {
	  int i = -1;
    
	  for (Filter<CriterionJPAQL, ActividadExternaLanguage> f : getFilters()) {
		  if (f.getCriterion().getQuery().contains("externa.colaborador.proveedor.id"))
			  i = getFilters().indexOf(f);
	  }
    
	  if (i > -1)
		  getFilters().remove(i);
  }
  
  /**
   * Filter by proveedor.
   * 
   * @param idProveedor
   *           id proveedor
   */
  public void filterByProveedor(Long idProveedor) {
	  String query = "";
	  boolean flag = false;
	  FilterJPAQLByQuery<ActividadExternaLanguage> filter = new FilterJPAQLByQuery<ActividadExternaLanguage>();

	  query += " (externa.colaborador.proveedor.id = ?) ";
	  filter.getCriterion().addParameter(idProveedor);

	  for (Filter<CriterionJPAQL, ActividadExternaLanguage> f : getFilters()) {
		  if (f.getCriterion().getQuery().contains("externa.colaborador.proveedor.id"))
			  flag = true;
	  }

	  if (!flag) {
		  filter.getCriterion().setQuery(query);
		  getFilters().add(filter);
	  }
  }
  
  /**
   * Set condition para agentesTIC.
   */
  public void setConditionAgenteTIC(UsuarioLdap usuario) {
	  if (usuario.getRolesList().contains(Usuario.Role.ROLE_AREA_MANAGER) || 
		  usuario.getRolesList().contains(Usuario.Role.ROLE_DIGITAL_ALLY)) {
		  this.getCondition().getCriterion().setQuery("e.externa.centro.id = ?");
		  this.getCondition().getCriterion().addParameter(usuario.getCentro().getId());

		  this.applyCondition();
	  }
  }

  /**
   * Set condition para colaboradores.
   */
  public void setConditionColaborador(UsuarioLdap usuario) {
	  if (usuario.getRolesList().contains(Usuario.Role.ROLE_EXTERN)) {
		  String strqry = "SELECT e " +
		  				  "FROM " + Empleado.class.getName() + " e " +
		  				  "WHERE e.id = :idUsuario";
		  Query command = getEntityManager().createQuery(strqry);
		  command.setParameter("idUsuario", usuario.getUser().getId());

		  Empleado empleado = (Empleado)command.getSingleResult();

		  if (empleado.getProveedor() == null) {
			  this.getCondition().getCriterion().setQuery("e.externa.colaborador.id = ?");
			  this.getCondition().getCriterion().addParameter(empleado.getId());
		  }
		  else {
			  this.getCondition().getCriterion().setQuery("e.externa.colaborador.proveedor = ?");
			  this.getCondition().getCriterion().addParameter(empleado.getProveedor());
		  }

		  this.applyCondition();
	  }
  }

  /**
   * Set condition aceptadas.
   */
  public void setConditionAceptadas() {
	  //RF_XEA_22 se añade la condicion para mostrar las actividades cuando su fecha y hora de publicacion lo permitan
	  String query = "(e.externa.solicitada = ?) AND (e.externa.aceptada = ?) AND ((externa.fechaPublicacion is null AND externa.horaPublicacion is null) OR (externa.fechaPublicacion < curdate()) OR (externa.fechaPublicacion = curdate() AND (externa.horaPublicacion <= curtime() OR externa.horaPublicacion IS NULL))) ";
	  this.getCondition().getCriterion().setQuery(query);
	  this.getCondition().getCriterion().addParameter(false);
	  this.getCondition().getCriterion().addParameter(true);
	  this.applyCondition();
  }

  /**
   * Refresh.
   */
  public void refresh() {
	  getEntityManager().clear();
  }
  
  @Override
  public void clearFilters() {
	  super.clearFilters();

	  filterCentro();
	  filterProveedor();	  
	  filterTematica();
	  filterIndIniciativaSingular();
	  filterIndVideoconferencia();
  }
  
  @Override
  public void processRecords() {
	  super.processRecords();
	  
	  int i = 0;
	  while (i < records.size()) {
		  ActividadExterna ae = ((ActividadExternaLanguage)records.get(i)).getExterna();
		  
		  if ((ae.getCentro() != null) && ae.getCentro().getDesactivado())
			  records.remove(i);
		  else
			  i++;
	  }
  }
  
  //RF_XEA_12 Funcion para mantener el filtro al volver del detalle de un curso
  public List<Filter<CriterionJPAQL, ActividadExternaLanguage>> cargarFiltros(List<Filter<CriterionJPAQL, ActividadExternaLanguage>> filtros, 
		  Centro cen, Proveedor prv, Tematica tem, Long indIniciativaSingular, Long indVideoconferencia, Date fechaInicio, Date fechaFin, Filter<CriterionJPAQL, ActividadExternaLanguage> texto) {
		
	if (null != filtros){
		
		getFilters().removeAll(getFilters());
		
		for (Filter<CriterionJPAQL, ActividadExternaLanguage> f : filtros) {
			getFilters().add(f);
		}
		
		//Para establecer el valor del centro seleccionado en el filtro inicial
		if (null != cen) {
			setCentro(cen);
		}

		//Para establecer el valor del proveedor seleccionado en el filtro inicial
		if (null != prv) {
			setProveedor(prv);
		}
		
		//Para establecer el valor de la tematica en el filtro inicial
		if (null != tem) {
			setTematica(tem);
		}
		
		//Para establecer el valor de indIniciativaSingular en el filtro inicial
		setIndIniciativaSingular(indIniciativaSingular);
		
		//Para establecer el valor de indVideoconferencia en el filtro inicial
		setIndVideoconferencia(indVideoconferencia);
		
		//Para establecer el valor de la fecha inicio en el filtro inicial
		if (null != fechaInicio) {
			setFechaInicio(fechaInicio);
		}
				
		//Para establecer el valor de la fecha fin en el filtro inicial
		if (null != fechaFin) {
			setFechaFin(fechaFin);
		}
		
		//Para mantener el texto introducido en el campo del filtro
		if(null != texto) {
			setFilter(texto);
		}
	}
	
	return null;
  }

	/**
	 * Codifica los filtros empleados en el browser para trasladar al proceso de exportacion.
	 * @return
	 */
	public String getFiltersEncoded(){
		String result = null;
		String sFilterQuery = "";
		ArrayList<String> oFilterList = new ArrayList<String>();
		DateFormat dfo = new SimpleDateFormat("yyyy-MM-dd 00:00:00");
		
		for (Filter<CriterionJPAQL, ActividadExternaLanguage> f : getFilters()){
			if (f.getCriterion() != null) {
				sFilterQuery += " AND (" + f.getCriterion().getQuery() + ") ";
				
				for (Object oParameter : f.getCriterion().getParameters()) {
					String sParameter = "";
					if(oParameter instanceof Long){
						sParameter = oParameter.toString();
					} else if(oParameter instanceof String) {
						sParameter = "'" + oParameter.toString().trim() + "'";
					} else if(oParameter instanceof Date) {
						sParameter = "'" + dfo.format(oParameter) + "'";
					} else {
						sParameter = oParameter.toString();
					}
					
					oFilterList.add(sParameter);
			    }
			}
		}
		oFilterList.add(sFilterQuery);
		
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		try {
			new ObjectOutputStream(out).writeObject(oFilterList);
		} catch (IOException e) {
			e.printStackTrace();
		}
		result = new String(Base64.encodeBase64(out.toByteArray()));
	
		return result;
	}
  
}
