package es.csa.fdt.cyldigital.language.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.language.entity.CharlaLanguage;
import es.csa.fdt.cyldigital.language.entity.Language;
import es.csa.fdt.cyldigital.servicios.entity.Charla;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario.Role;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;

/**
 * Clase CharlaLanguageServiceImpl. 
 *
 * @author it-pronet*/

@Service
@Repository
public class CharlaLanguageServiceImpl extends AbstractServiceImpl<CharlaLanguage, Long>
										   implements CharlaLanguageService{

	private static final long serialVersionUID = -508581690737783870L;

	/**
	 * Construye un nuevo objeto de la clase
	 */
	public CharlaLanguageServiceImpl() {
		super();
	}
	
	public CharlaLanguage find(Long id, Language language) {
		HashMap<String, Object> propiedades = new HashMap<String, Object>();
		propiedades.put("charla.id", id);
		propiedades.put("language.id", language.getId());
	      
		return super.findOneWhere(propiedades);
	}	
	
	/**
	   * Devuelve next charla.
	   * 
	   * @param u
	   *           u
	   * @param c
	   *           c
	   * @param next
	   *           next
	   * @return next charla 
	   */
	public Charla getNextCharla(UsuarioLdap u, CharlaLanguage cl, boolean next) {
	    /*
	     * next=false: devuelve, si hay, la incidencia anterior. A true, la posterior. Recordar que el curso anterior es de
	     * fecha posterior al actual. Devuelve null si no hay una incidencia anterior/posterior, si la hay, devuelve su id
	     */
	    if (cl != null) {
	      Charla c = cl.getCharla();
	      List<Charla> charlas = new ArrayList<Charla>();

	      if (!u.getRolesList().contains(Role.ROLE_ACTIVITY_TRAINER_VIEW_ALL)) {
	        charlas = findByQuery(Charla.class,
	            " Select c from es.csa.fdt.cyldigital.servicios.entity.Charla c where centro="
	                + u.getUser().getCentro().getId() + " order by fechaInicio desc, horaInicio desc");
	      } else {
	        charlas = findByQuery(Charla.class,
	            " Select c from es.csa.fdt.cyldigital.servicios.entity.Charla c  order by fechaInicio desc, horaInicio desc");
	      }
	      if (charlas.indexOf(c) > -1) {
	        if (next) {
	          if (charlas.indexOf(c) + 1 < charlas.size()) return charlas.get(charlas.indexOf(c) + 1);
	        } else {
	          if (charlas.indexOf(c) != 0) return charlas.get(charlas.indexOf(c) - 1);
	        }
	      }
	    }
	    return null;
	  }
	//RF_XEA_12 Se redefine la funcion que devuelve la charla anterior y siguiente respetando los filtros de la lista de charlas
	/**
	   * Devuelve next charla.
	   * 
	   * @param u
	   *           u
	   * @param c
	   *           c
	   * @param next
	   *           next
	   * @param filtros
	   *           filtros
	   * @param idioma
	   *           idioma
	   * @return next charla 
	   */
	public Charla getNextCharlaNew(UsuarioLdap u, CharlaLanguage cl, boolean next, List<Filter<CriterionJPAQL,CharlaLanguage>> filtros, Language idioma) {
	    /*
	     * next=false: devuelve, si hay, la incidencia anterior. A true, la posterior. Recordar que el curso anterior es de
	     * fecha posterior al actual. Devuelve null si no hay una incidencia anterior/posterior, si la hay, devuelve su id
	     */
		
		if (cl != null) {
			if (null != filtros){
				
				String query = "";
				Charla c = cl.getCharla();
			    List<Charla> charlas = new ArrayList<Charla>();
			    SimpleDateFormat formatoFecha = new SimpleDateFormat("yyyy-MM-dd");
			    
			    if (filtros.size() == 2) {
			    	if (!u.getRolesList().contains(Role.ROLE_ACTIVITY_TRAINER_VIEW_ALL)) {
			    		query = " Select c from es.csa.fdt.cyldigital.servicios.entity.Charla c where centro="
				                + u.getUser().getCentro().getId() + " order by fechaInicio desc, horaInicio desc";
			    	} else {
			    		query = " Select c from es.csa.fdt.cyldigital.servicios.entity.Charla c  order by fechaInicio desc, horaInicio desc";
			    	}
			    	
			    	charlas = findByQuery(Charla.class,query);
			    } else {
			    	
			    	if ("es_ES".equals(idioma.getISO())) {
						query = "SELECT c from es.csa.fdt.cyldigital.language.entity.CharlaLanguage c WHERE (language.ISO = 'es_ES')";
					} else {
						query = "SELECT c from es.csa.fdt.cyldigital.language.entity.CharlaLanguage c WHERE (language.ISO = 'gl_ES')";
					}
			    	
			    	query += " AND ( ( desactivado is null or desactivado = false ) ) AND ( charla.centro.desactivado = false )";
			    	
			    	
			    	boolean noCentro = true;
			    	if (!u.getRolesList().contains(Role.ROLE_ACTIVITY_TRAINER_VIEW_ALL)) {
						query += " AND charla.centro.id = " + u.getUser().getCentro().getId();
						noCentro = false;
					}
			    	
			    	String condicion = "";
					String valor = "";
					for (Filter<CriterionJPAQL, CharlaLanguage> f : filtros){
						
						//Centro
						if (f.getCriterion().getQuery().contains("charla.centro.id") && noCentro){
							
							for (Object o : f.getCriterion().getParameters()) {
								valor = o.toString();
							}
							condicion = "charla.centro.id = " + valor;
							query += " AND (" + condicion + ")";
						}
						
						//Tematica
						if (f.getCriterion().getQuery().contains("charla.tematica.id")){
							
							for (Object o : f.getCriterion().getParameters()) {
								valor = o.toString();
							}
							condicion = "charla.tematica.id = " + valor;
							query += " AND (" + condicion + ")";
						}
						
						//Nombre
						if (f.getCriterion().getQuery().contains("nombre")){
		
							String cadena = "";
							for (Object o : f.getCriterion().getParameters()) {
								cadena += o.toString().substring(1, o.toString().length()-1) + "%";
							} 
							condicion = "LOWER(nombre) LIKE LOWER('%" + cadena + "')"; 
							query += " AND (" + condicion + ")";
						}
						
						//ID de la charla
						if (f.getCriterion().getQuery().contains("charla.id")){
							
							for (Object o : f.getCriterion().getParameters()) {
								valor = o.toString();
							}
							condicion = "charla.id = " + valor;
							query += " AND (" + condicion + ")";
						}
						
						//Fecha inicio
						if (f.getCriterion().getQuery().contains("charla.fechaInicio")){
							
							for (Object o : f.getCriterion().getParameters()) {
								valor = formatoFecha.format((Date)o);
							}
							condicion = "charla.fechaInicio = '" + valor + "'";
							query += " AND (" + condicion + ")";
						}
						
					}
					
					query += " order by charla.fechaInicio desc, charla.horaInicio desc";
					
					List<CharlaLanguage> charlasLang = new ArrayList<CharlaLanguage>();
					
					charlasLang = findByQuery(CharlaLanguage.class, query);
					
					for (CharlaLanguage cha : charlasLang){
						charlas.add(cha.getCharla());
					}	
			    }
			    
			    if (charlas.indexOf(c) > -1) {
			        if (next) {
			          if (charlas.indexOf(c) + 1 < charlas.size()) return charlas.get(charlas.indexOf(c) + 1);
			        } else {
			          if (charlas.indexOf(c) != 0) return charlas.get(charlas.indexOf(c) - 1);
			        }
			   } 
			} else {
				return null;
			}
		}
		
		return null;
		
	  }
	//RF_XEA_12	
}
