package es.csa.fdt.cyldigital.language.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.language.entity.CursoLanguage;
import es.csa.fdt.cyldigital.language.entity.Language;
import es.csa.fdt.cyldigital.servicios.entity.Curso;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso;
import es.csa.fdt.cyldigital.servicios.entity.ObservacionActividad;
import es.csa.fdt.cyldigital.servicios.entity.PreinscripcionCurso;
import es.csa.fdt.cyldigital.servicios.entity.ServicioCurso;
import es.csa.fdt.cyldigital.usuarios.entity.Localidad;
import es.csa.fdt.cyldigital.usuarios.entity.Provincia;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario.Role;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;


/**
 * Clase CursoLanguageServiceImpl.
 * 
 * @author it-pronet
 */
@Service
@Repository
public class CursoLanguageServiceImpl extends AbstractServiceImpl<CursoLanguage, Long> implements CursoLanguageService {

	private static final long serialVersionUID = -4705938957186549457L;

	//private boolean areThereFicherosGuiones;
	
	private boolean borrarPreinscripcion = false;
	
	private ServicioCurso servicioCurso;
	
	private boolean diplomaCenterTown;
	
	private List<Provincia> provincias;
	private Provincia provinciaDiploma;
	
	private String nombreLocalidad;
	private List<Localidad> listLocalidades;
	private Long idLocalidadDiplomaSelected;

	/**
	 * Construye un nuevo objeto de la clase
	 */
	public CursoLanguageServiceImpl() {
		super();

		//areThereFicherosGuiones = false;
		
		borrarPreinscripcion = false;
		
		servicioCurso = null;
		
		diplomaCenterTown = true;
		nombreLocalidad = "";
		listLocalidades = null;
		idLocalidadDiplomaSelected = -1L;
	}
  
	/*public boolean getAreThereFicherosGuiones() {
		return areThereFicherosGuiones;
	}

	private void setAreThereFicherosGuiones(CursoLanguage cl) {
		areThereFicherosGuiones = false;

		if (cl != null) {
			for (ServicioCursoLanguage servicioCursoLanguage : ((ServicioCurso)cl.getCurso().getServicio()).getGuiones()) {
				if (servicioCursoLanguage.getFicheroGuion() != null) {
					areThereFicherosGuiones = true;

					break;
				}
			}
		}
	}*/

	public CursoLanguage find(Long id, Language language) {
		HashMap<String, Object> propiedades = new HashMap<String, Object>();
		propiedades.put("curso.id", id);
		propiedades.put("language.id", language.getId());

		CursoLanguage cl = super.findOneWhere(propiedades);
		
		//setAreThereFicherosGuiones(cl);
		
		servicioCurso = find(ServicioCurso.class, cl.getCurso().getServicio().getId());
		
		return cl;
	}

	public void clear(CursoLanguage cl) {
		clear();

		//setAreThereFicherosGuiones(cl);
	}
  
	/**
	 * Devuelve next curso.
	 * 
	 * @param u
	 *           u
	 * @param curso
	 *           curso
	 * @param next
	 *           next
	 * @return next curso
	 */
	public Curso getNextCurso(UsuarioLdap u, CursoLanguage cursoLanguage, boolean next) {
		/*
		 * next=false: devuelve, si hay, la incidencia anterior. A true, la posterior. Recordar que el curso anterior es de
		 * fecha posterior al actual. Devuelve null si no hay una incidencia anterior/posterior, si la hay, devuelve su id
		 */
		Curso curso = cursoLanguage.getCurso();
		List<Curso> cursos = new ArrayList<Curso>();
	
		if (!u.getRolesList().contains(Role.ROLE_ACTIVITY_TRAINER_VIEW_ALL)) {
			cursos = findByQuery(Curso.class, " Select c from es.csa.fdt.cyldigital.servicios.entity.Curso c where centro="
											  + u.getUser().getCentro().getId() +
											  " order by fechaInicio desc, horaInicio desc");
		}
		else {
			cursos = findByQuery(Curso.class, " Select c from es.csa.fdt.cyldigital.servicios.entity.Curso c  order by fechaInicio desc, horaInicio desc");
	    }
	
	    if (next) {
	    	if (cursos.indexOf(curso) + 1 < cursos.size())
	    		return cursos.get(cursos.indexOf(curso) + 1);
	    }
	    else {
	    	if (cursos.indexOf(curso) != 0)
	    		return cursos.get(cursos.indexOf(curso) - 1);
	    }

	    return null;
	}
	//RF_XEA_12 Se redefine la funcion que devuelve los cursos anterior y siguiente respetando los filtros de la lista de cursos
	/**
	 * Devuelve next curso.
	 * 
	 * @param u
	 *           u
	 * @param curso
	 *           curso
	 * @param next
	 *           next
	 * @param filtros
	 *           filtros
	 * @param idioma
	 *           idioma
	 * @return next curso
	 */
	public Curso getNextCursoNew(UsuarioLdap u, CursoLanguage cursoLanguage, boolean next, List<Filter<CriterionJPAQL, CursoLanguage>> filtros, Language idioma, String nuevo) {
		/*
		 * next=false: devuelve, si hay, la incidencia anterior. A true, la posterior. Recordar que el curso anterior es de
		 * fecha posterior al actual. Devuelve null si no hay una incidencia anterior/posterior, si la hay, devuelve su id
		 */
		
		if (null != filtros && "".equals(nuevo)){
			String query = "";
			Curso curso = cursoLanguage.getCurso();
			List<Curso> cursos = new ArrayList<Curso>();
			SimpleDateFormat formatoFecha = new SimpleDateFormat("yyyy-MM-dd");
					
			if (filtros.size() == 2){
				if (!u.getRolesList().contains(Role.ROLE_ACTIVITY_TRAINER_VIEW_ALL)) {
					query = " Select c from es.csa.fdt.cyldigital.servicios.entity.Curso c where centro="
							  + u.getUser().getCentro().getId() +
							  " order by fechaInicio desc, horaInicio desc";
				} else {
					query = "Select c from es.csa.fdt.cyldigital.servicios.entity.Curso c  order by fechaInicio desc, horaInicio desc";
				}
					cursos = findByQuery(Curso.class, query);
			}else{
			
				if ("es_ES".equals(idioma.getISO())) {
					query = "SELECT c from es.csa.fdt.cyldigital.language.entity.CursoLanguage c WHERE (language.ISO = 'es_ES')";
				} else {
					query = "SELECT c from es.csa.fdt.cyldigital.language.entity.CursoLanguage c WHERE (language.ISO = 'gl_ES')";
				}
				
				query += " AND ( ( desactivado is null or desactivado = false ) ) AND ( curso.centro.desactivado = false )";
				
				boolean noCentro = true;
				if (!u.getRolesList().contains(Role.ROLE_ACTIVITY_TRAINER_VIEW_ALL)) {
					query += " AND curso.centro.id = " + u.getUser().getCentro().getId();
					noCentro = false;
				}
				
				String condicion = "";
				String valor = "";
				for (Filter<CriterionJPAQL, CursoLanguage> f : filtros){
					
					//Centro
					if (f.getCriterion().getQuery().contains("curso.centro.id") && noCentro){
						
						for (Object o : f.getCriterion().getParameters()) {
							valor = o.toString();
						}
						condicion = "curso.centro.id = " + valor;
						query += " AND (" + condicion + ")";
					}
					
					//Tematica
					if (f.getCriterion().getQuery().contains("curso.servicio.tematica.id")){
						
						for (Object o : f.getCriterion().getParameters()) {
							valor = o.toString();
						}
						condicion = "curso.servicio.tematica.id = " + valor;
						query += " AND (" + condicion + ")";
					}
					
					//Nombre
					if (f.getCriterion().getQuery().contains("apodo")){
						
						String cadena = "";
						for (Object o : f.getCriterion().getParameters()) {
							cadena += o.toString().substring(1, o.toString().length()-1) + "%";
						}
						condicion = "LOWER(apodo) LIKE LOWER('%" + cadena + "')"; 
						query += " AND (" + condicion + ")";
					}
					
					//ID del curso
					if (f.getCriterion().getQuery().contains("curso.id")){
						
						for (Object o : f.getCriterion().getParameters()) {
							valor = o.toString();
						}
						condicion = "curso.id = " + valor;
						query += " AND (" + condicion + ")";
					}
					
					//Fecha inicio
					if (f.getCriterion().getQuery().contains("curso.fechaInicio")){
						
						
						for (Object o : f.getCriterion().getParameters()) {
							valor = formatoFecha.format((Date)o);
						}
						condicion = "curso.fechaInicio = '" + valor + "'";
						query += " AND (" + condicion + ")";
					}
					
					//Fecha fin
					if (f.getCriterion().getQuery().contains("curso.fechaFin")){
						
						for (Object o : f.getCriterion().getParameters()) {
							valor = formatoFecha.format((Date)o);
						}
						condicion = "curso.fechaFin = '" + valor + "'";;
						query += " AND (" + condicion + ")";
					}
				}
				
				query += " order by curso.fechaInicio desc, curso.horaInicio desc";
				
				
				List<CursoLanguage> cursosLang = new ArrayList<CursoLanguage>();
				
				cursosLang = findByQuery(CursoLanguage.class, query);
				
				for (CursoLanguage cur : cursosLang){
					cursos.add(cur.getCurso());
				}
			}
			
			
			if (next) {
		    	if (cursos.indexOf(curso) + 1 < cursos.size())
		    		return cursos.get(cursos.indexOf(curso) + 1);
		    }
		    else {
		    	if (cursos.indexOf(curso) > 0)
		    		return cursos.get(cursos.indexOf(curso) - 1);
		    	else
		    		return null;		    		
		    }
			
		    return null;
		    
		} else {
			
			return null;
		}

	}
	//RF_XEA_12
	
	/**
	 * Devuelve nueva observacion guion.
	 * 
	 * @return nueva observacion guion
	 */
	public ObservacionActividad getNuevaObservacionGuion() {
		return new ObservacionActividad();
	}
	
	/**
	 * Anadir observacion guion.
	 *
	 * @param curso
	 *           curso
	 * @param o
	 *           o
	 * @param usuario
	 *           usuario
	 */
	public void anadirObservacionGuion(Curso curso, ObservacionActividad o, Usuario usuario) {
		if (o.getObservacion().equals(""))
			addWarnMessage("No puede añadir un comentario vacío");
		else {
	      o.setFecha(new Date());
	      o.setUsuario(usuario);
	      o.setActividad(curso);
	      curso.getObservacionesActividad().add(o);
	      persist(curso);
		}
	}
	
	/**
	 * Checks if is borrar preinscripcion.
	 * 
	 * @return true, if is borrar preinscripcion
	 */
	public boolean isBorrarPreinscripcion() {
		return borrarPreinscripcion;
	}

	/**
	 * Asigna borrar preinscripcion.
	 * 
	 * @param borrarPreinscripcion
	 *            borrar preinscripcion
	 */
	public void setBorrarPreinscripcion(boolean borrarPreinscripcion) {
		this.borrarPreinscripcion = borrarPreinscripcion;
	}

	/**
	 * Rechazar curso.
	 * 
	 * @param curso
	 *            curso
	 * @param idPreinscripcion
	 *            id preinscripcion
	 */
	public void rechazarCurso(Curso curso, Long idPreinscripcion) {
		if (borrarPreinscripcion) {
			PreinscripcionCurso p = find(PreinscripcionCurso.class,
					idPreinscripcion);
			if (p != null)
				remove(p);
		}
		else {
			PreinscripcionCurso p = find(PreinscripcionCurso.class,
					idPreinscripcion);
			if (p != null) {
				p.getCursosRechazos().add(curso);
				p.setPeso(p.getPeso() + 1);
				persist(p);
			}
		}
	}

	/**
	 * Matricula from preinscripcion.
	 * 
	 * @param curso
	 *            curso
	 * @param idPreinscripcion
	 *            id preinscripcion
	 * @return true, if successful
	 */
	public boolean matriculaFromPreinscripcion(Curso curso,	Long idPreinscripcion) {
		PreinscripcionCurso p = find(PreinscripcionCurso.class,	idPreinscripcion);
		boolean flag = false;
		if (p != null) {
			Long idUsuario = p.getUsuario().getId();
			UsuarioCyL u = find(UsuarioCyL.class, idUsuario);

			for (MatriculaCurso matr : curso.getMatriculas()) {
				if (matr.getUsuario().getId() == idUsuario) {
					flag = true;
					addErrorMessage("El usuario ya está matriculado en este curso");
				}
			}

			if (!flag) {
				((ServicioCurso) curso.getServicio()).getPreinscripciones().remove(p);
				u.getPreinscripcionesCursos().remove(p);

				MatriculaCurso m = new MatriculaCurso();
				m.setAdmitido(true);
				m.setDiplomaCreado(false);
				m.setFechaMatricula(new Date());

				m.setCurso(curso);
				m.setUsuario(u);
				persist(m);

				setMessagesEnabled(false);
				remove(p);
				setMessagesEnabled(true);
				return true;
			}
		}
		return false;
	}
	
	public boolean isDiplomaCenterTown() {
		return diplomaCenterTown;
	}
	
	public void setDiplomaCenterTown(boolean diplomaCenterTown) {
		this.diplomaCenterTown = diplomaCenterTown;
	}
	
	 public String getNombreLocalidad() {
		  return nombreLocalidad;
	  }
	  
	  public void setNombreLocalidad(String nombreLocalidad) {
		  this.nombreLocalidad = nombreLocalidad;
	  }
	  
	  public List<Localidad> getListLocalidades() {
		  return listLocalidades;
	  }
	  
	  public void searchLocalidad(Provincia provincia) {
		  if (nombreLocalidad.length() > 1) {
			  String strqry = "SELECT localidad " +
			  				  "FROM " + Localidad.class.getName() + " localidad " +
			  				  "WHERE localidad.provincia.id = " + provincia.getId() + " AND" +
			  				  "      localidad.nombre LIKE '%%" + nombreLocalidad + "%%' " +
			  				  "ORDER BY localidad.nombre ASC";
			  
			  listLocalidades = findByQuery(Localidad.class, strqry);
		  }
		  else {
			  if (listLocalidades != null) {
				  listLocalidades.clear();
				  listLocalidades = null;
			  }
		  }
	  }
	  
	  /**
	   * @return the IdLocalidadDiplomaSelected
	   */
	  public Long getIdLocalidadDiplomaSelected() {
		  return idLocalidadDiplomaSelected;
	  }

	  /**
	   * @param idLocalidadDiplomaSelected the idLocalidadDiplomaSelected to set
	   */
	  public void setIdLocalidadDiplomaSelected(Long idLocalidadDiplomaSelected) {
		  this.idLocalidadDiplomaSelected = idLocalidadDiplomaSelected;
	  }

	  
	  public void applyLocalidadToMatricula(MatriculaCurso matricula) {
		  if (idLocalidadDiplomaSelected == -1) {
			  matricula.setLocalidadDiplomaEmitido(null);
		  }
		  else {
			  Map<String, Object> properties = new HashMap<String, Object>();
			  properties.put("provincia", getProvinciaDiploma());
			  properties.put("id", idLocalidadDiplomaSelected);
			  
			  matricula.setLocalidadDiplomaEmitido(findOneWhere(Localidad.class, properties));
		  }
	  }
	  
	  public List<Provincia> getProvincias() {
		  if (provincias == null)
			  provincias = findAllOrder(Provincia.class, "nombre", true);
		  
		  return provincias;
	  }
	  
	  /**
	   * @return the provinciaDiploma
	   */
	  public Provincia getProvinciaDiploma() {
		  return provinciaDiploma;
	  }

	  /**
	   * @param provinciaDiploma the provinciaDiploma to set
	   */
	  public void setProvinciaDiploma(Provincia provinciaDiploma) {
		  this.provinciaDiploma = provinciaDiploma;
	  }
	  
	  /**
	   * @return the servicioCurso
	   */
	  public ServicioCurso getServicioCurso() {
		  return servicioCurso;
	  }

	  /**
	   * @param servicioCurso the servicioCurso to set
	   */
	  public void setServicioCurso(ServicioCurso servicioCurso) {
		  this.servicioCurso = servicioCurso;
	  }	  
}
