package es.csa.fdt.cyldigital.language.service;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


//import javax.faces.model.DataModel;

import java.util.List;

import javax.persistence.Query;

import org.apache.commons.codec.binary.Base64;

//import javax.persistence.Temporal;
//import javax.persistence.TemporalType;

//import org.hibernate.validator.NotNull;
//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.commons.persistence.browser.jpaql.FilterJPAQLByQuery;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.service.BrowserFilteredByCenter;
import es.csa.fdt.cyldigital.language.entity.CursoLanguage;
import es.csa.fdt.cyldigital.language.entity.Language;
import es.csa.fdt.cyldigital.servicios.entity.Tematica;

//import es.csa.fdt.cyldigital.servicios.service.MatriculaCursoServiceImpl;

/**
 * Servicio del Browser de CursosLanguage.
 * 
 * @author IT-PRONET
 */
@Service
@Repository
public class CursosLanguageBrowserImpl extends	  BrowserFilteredByCenter<CursoLanguage>
									   implements CursosLanguageBrowser<CriterionJPAQL, String> {

	private static final long serialVersionUID = -7042425707074770340L;
	
	//private static final Logger logger = LoggerFactory.getLogger(CursosLanguageBrowserImpl.class);
	
	private Tematica tematica;
	private Long indIniciativaSingular;
	private Long indVideoconferencia;
	private Long indPlazasLibres;
	private Date fechaInicio;
	private Date fechaFin;
	
	/**
	 * Construye un nuevo objeto de la clase
	 */
	public CursosLanguageBrowserImpl() {
		super("curso");

		getSort().setSortProperty("e.curso.fechaInicio DESC, e.curso.horaInicio DESC");
	}
	
	public void applyLanguageCondition(Language language) {
		this.clearConditions();
		
		this.getCondition().getCriterion().setQuery("e.language = ?");
		this.getCondition().getCriterion().addParameter(language);

		this.applyCondition();
	}

	/**
	 * Devuelve tematica.
	 * 
	 * @return tematica
	 */
	public Tematica getTematica() {
		return tematica;
	}

	/**
	 * Asigna tematica.
	 * 
	 * @param tematica
	 *          tematica
	 */
	public void setTematica(Tematica tematica) {
		this.tematica = tematica;
	}

	/**
	 * 
	 * @return
	 */
	public Long getIndIniciativaSingular() {
		return indIniciativaSingular;
	}

	/**
	 * 
	 * @param IniciativaSingular
	 */
	public void setIndIniciativaSingular(Long indIniciativaSingular) {
		this.indIniciativaSingular = indIniciativaSingular;
	}
	
	/**
	 * 
	 * @return
	 */
	public Long getIndVideoconferencia() {
		return indVideoconferencia;
	}

	/**
	 * 
	 * @param Videoconferencia
	 */
	public void setIndVideoconferencia(Long indVideoconferencia) {
		this.indVideoconferencia = indVideoconferencia;
	}
	
	/**
	 * 
	 * @return
	 */
	public Long getIndPlazasLibres() {
		return indPlazasLibres;
	}

	/**
	 * 
	 * @param indPlazasLibres
	 */
	public void setIndPlazasLibres(Long indPlazasLibres) {
		this.indPlazasLibres = indPlazasLibres;
	}
	
	/**
	 * @return the fechaInicio
	 */
	public Date getFechaInicio() {
		return fechaInicio;
	}

	/**
	 * @param fechaInicio 
	 * 		  the fechaInicio to set
	 */
	public void setFechaInicio(Date fechaInicio) {
		this.fechaInicio = fechaInicio;
	}
	
	/**
	 * @return the fechaFin
	 */
	public Date getFechaFin() {
		return fechaFin;
	}

	/**
	 * @param fechaInicio 
	 * 		  the fechaInicio to set
	 */
	public void setFechaFin(Date fechaFin) {
		this.fechaFin = fechaFin;
	}
	
	public void filterFechas() {
		if (getFechaInicio() != null && getFechaFin() != null) {
			removeFilterFechaInicio();
			removeFilterFechaFin();
			addFilterFechas();
		} else if (getFechaInicio() != null && getFechaFin() == null) {
			removeFilterFechaInicio();
			addFilterFechaInicio();
		} else if (getFechaInicio() == null && getFechaFin() != null) {
			removeFilterFechaFin();
			addFilterFechaFin();
		} else {
			removeFilterFechaInicio();
			removeFilterFechaFin();
		}
	}

	private void removeFilterFechaInicio() {
		int i = -1;
		
		for (Filter<CriterionJPAQL, CursoLanguage> f : getFilters())
			if (f.getCriterion().getQuery().contains("e.curso.fechaFin"))
				i = getFilters().indexOf(f);
		
		if (i > -1)
			getFilters().remove(i);
	}

	private void addFilterFechaInicio() {
		String query = "";
		boolean flag = false;
		
		FilterJPAQLByQuery<CursoLanguage> filter = new FilterJPAQLByQuery<CursoLanguage>();

		query += " e.curso.fechaFin >= ? ";
		filter.getCriterion().addParameter(getFechaInicio());

		for (Filter<CriterionJPAQL, CursoLanguage> f : getFilters())
			if (f.getCriterion().getQuery().contains("e.curso.fechaFin"))
				flag = true;

		if (!flag) {
			filter.getCriterion().setQuery(query);
			getFilters().add(filter);
		}
	}
	
	private void removeFilterFechaFin() {
		int i = -1;
		
		for (Filter<CriterionJPAQL, CursoLanguage> f : getFilters())
			if (f.getCriterion().getQuery().contains("e.curso.fechaInicio"))
				i = getFilters().indexOf(f);
		
		if (i > -1)
			getFilters().remove(i);
	}

	private void addFilterFechaFin() {
		String query = "";
		boolean flag = false;
		
		FilterJPAQLByQuery<CursoLanguage> filter = new FilterJPAQLByQuery<CursoLanguage>();

		query += " e.curso.fechaInicio <= ? ";
		filter.getCriterion().addParameter(getFechaFin());

		for (Filter<CriterionJPAQL, CursoLanguage> f : getFilters())
			if (f.getCriterion().getQuery().contains("e.curso.fechaInicio"))
				flag = true;

		if (!flag) {
			filter.getCriterion().setQuery(query);
			getFilters().add(filter);
		}
	}
	
	private void addFilterFechas() {
		String query = "";
		boolean flag = false;
		
		FilterJPAQLByQuery<CursoLanguage> filter = new FilterJPAQLByQuery<CursoLanguage>();

		query += " e.curso.fechaFin >= ? ";
		filter.getCriterion().addParameter(getFechaInicio());
		query += " AND e.curso.fechaInicio <= ? ";
		filter.getCriterion().addParameter(getFechaFin());

		for (Filter<CriterionJPAQL, CursoLanguage> f : getFilters())
			if (f.getCriterion().getQuery().contains("e.curso.fechaFin") && 
					f.getCriterion().getQuery().contains("e.curso.fechaInicio"))
				flag = true;

		if (!flag) {
			filter.getCriterion().setQuery(query);
			getFilters().add(filter);
		}
	}
	
	public void filterFechaHoraInicio() {
		removeFilterFechaHoraInicio();
		addFilterFechaHoraInicio();
	}
	
	private void removeFilterFechaHoraInicio() {
		int i = -1;
		
		for (Filter<CriterionJPAQL, CursoLanguage> f : getFilters())
			if (f.getCriterion().getQuery().contains("timestamp(e.curso.fechaInicio, e.curso.horaInicio)"))
				i = getFilters().indexOf(f);
		
		if (i > -1)
			getFilters().remove(i);
	}
	
	private void addFilterFechaHoraInicio() {
		String query = "";
		boolean flag = false;
		
		FilterJPAQLByQuery<CursoLanguage> filter = new FilterJPAQLByQuery<CursoLanguage>();

		//query += " timestamp(curso.fechaInicio, curso.horaInicio) >= now() ";
		query += " e.curso.fechaFin > curdate() ";
		
		//RF_XEA_22 añadir la condicion para que muestr solo los cursos que son publicos
		query += " AND ((e.curso.fechaPublicacion is null AND e.curso.horaPublicacion is null) OR (e.curso.fechaPublicacion < curdate()) OR (e.curso.fechaPublicacion = curdate() AND (e.curso.horaPublicacion <= curtime() OR e.curso.horaPublicacion is null))) ";
		//RF_XEA_22

		for (Filter<CriterionJPAQL, CursoLanguage> f : getFilters())
			//if (f.getCriterion().getQuery().contains("timestamp(curso.fechaInicio, curso.horaInicio)"))
			if (f.getCriterion().getQuery().contains("e.curso.fechaFin > now().curdate()"))
				flag = true;

		if (!flag) {
			filter.getCriterion().setQuery(query);
			getFilters().add(filter);
		}
	}

	/**
	 * Filter by curso.
	 * 
	 * @param idServicio
	 *           id servicio
	 */
	public void filterByCurso(Long idServicio) {
		String query = "";
		boolean flag = false;
		
		FilterJPAQLByQuery<CursoLanguage> filter = new FilterJPAQLByQuery<CursoLanguage>();

		if (idServicio != null && idServicio > 0) {
			filter.getCriterion().addParameter(idServicio);
			query += " curso.servicio.id = ? ";
		}

		for (Filter<CriterionJPAQL, CursoLanguage> f : getFilters())
			if (f.getCriterion().getQuery().equals(query)) flag = true;

		if (!query.isEmpty() && !flag) {
			filter.getCriterion().setQuery(query);
			getFilters().add(filter);
		}
	}

	/**
	 * Filter tematica.
	 */
	public void filterTematica() {
		removeFilterTematica();

		if (getTematica() != null)
			filterByTematica(getTematica().getId());
	}

	private void removeFilterTematica() {
		int i = -1;
		
		for (Filter<CriterionJPAQL, CursoLanguage> f : getFilters())
			if (f.getCriterion().getQuery().contains("e.curso.servicio.tematica.id"))
				i = getFilters().indexOf(f);

		if (i > -1)
			getFilters().remove(i);
	}

	/**
	 * Filter by tematica.
	 * 
	 * @param idTematica
	 *           id tematica
	 */
	public void filterByTematica(Long idTematica) {
		String query = "";
		boolean flag = false;

		FilterJPAQLByQuery<CursoLanguage> filter = new FilterJPAQLByQuery<CursoLanguage>();

		query += " e.curso.servicio.tematica.id = ? ";
		filter.getCriterion().addParameter(idTematica);

		for (Filter<CriterionJPAQL, CursoLanguage> f : getFilters())
			if (f.getCriterion().getQuery().contains("e.curso.servicio.tematica.id"))
				flag = true;

		if (!flag) {
			filter.getCriterion().setQuery(query);
			getFilters().add(filter);
		}
	}
	
	/**
	 * Filter indIniciativaSingular.
	 */
	public void filterIndIniciativaSingular() {
		removeIndIniciativaSingular();

		if (getIndIniciativaSingular() != null){
			filterByIndIniciativaSingular(getIndIniciativaSingular());
		}
	}

	/**
	 * removeIndIniciativaSingular
	 */
	private void removeIndIniciativaSingular() {
		int i = -1;
		
		for (Filter<CriterionJPAQL, CursoLanguage> f : getFilters())
			if (f.getCriterion().getQuery().contains("e.curso.iniciativaSingular"))
				i = getFilters().indexOf(f);

		if (i > -1)
			getFilters().remove(i);
	}

	/**
	 * Filter by indIniciativaSingular.
	 * 
	 * @param iniciativaSingular (0-todos;1-Si;2-No)
	 *           
	 */
	public void filterByIndIniciativaSingular(Long indIniciativaSingular) {
		String query = "";
		boolean flag = false;

		if(indIniciativaSingular == null || indIniciativaSingular.longValue() <= 0 )
			return;
		
		FilterJPAQLByQuery<CursoLanguage> filter = new FilterJPAQLByQuery<CursoLanguage>();

		query += " e.curso.iniciativaSingular = ? ";
		filter.getCriterion().addParameter(indIniciativaSingular.longValue()==1);

		for (Filter<CriterionJPAQL, CursoLanguage> f : getFilters())
			if (f.getCriterion().getQuery().contains("e.curso.iniciativaSingular"))
				flag = true;

		if (!flag) {
			filter.getCriterion().setQuery(query);
			getFilters().add(filter);
		}
	}
	
	/**
	 * Filter indVideoconferencia.
	 */
	public void filterIndVideoconferencia() {
		removeIndVideoconferencia();

		if (getIndVideoconferencia() != null){
			filterByIndVideoconferencia(getIndVideoconferencia());
		}
	}

	/**
	 * removeIndVideoconferencia
	 */
	private void removeIndVideoconferencia() {
		int i = -1;
		
		for (Filter<CriterionJPAQL, CursoLanguage> f : getFilters())
			if (f.getCriterion().getQuery().contains("e.curso.videoconferencia"))
				i = getFilters().indexOf(f);

		if (i > -1)
			getFilters().remove(i);
	}

	/**
	 * Filter by indVideoconferencia.
	 * 
	 * @param indVideoconferencia (0-todos;1-Si;2-No)
	 *           
	 */
	public void filterByIndVideoconferencia(Long indVideoconferencia) {
		String query = "";
		boolean flag = false;

		if(indVideoconferencia == null || indVideoconferencia.longValue() <= 0 )
			return;
		
		FilterJPAQLByQuery<CursoLanguage> filter = new FilterJPAQLByQuery<CursoLanguage>();

		query += " e.curso.videoconferencia = ? ";
		filter.getCriterion().addParameter(indVideoconferencia.longValue()==1);

		for (Filter<CriterionJPAQL, CursoLanguage> f : getFilters())
			if (f.getCriterion().getQuery().contains("e.curso.videoconferencia"))
				flag = true;

		if (!flag) {
			filter.getCriterion().setQuery(query);
			getFilters().add(filter);
		}
	}
	
	/**
	 * Filter indPlazasLibres.
	 */
	public void filterIndPlazasLibres() {
		removeIndPlazasLibres();

		if (getIndPlazasLibres() != null){
			filterByIndPlazasLibres(getIndPlazasLibres());
		}
	}

	/**
	 * removeIndPlazasLibres
	 */
	private void removeIndPlazasLibres() {
		//Se mantiene el metodo por compatibilidad.  
		//this.setIndPlazasLibres(null);
	}

	/**
	 * Filter by indPlazasLibres.
	 * 
	 * @param indPlazasLibres (0-todos;1-Si;2-No)
	 *           
	 */
	public void filterByIndPlazasLibres(Long indPlazasLibres) {
		//Se mantiene el metodo por compatibilidad.  
		this.setIndPlazasLibres(indPlazasLibres);
	}
	
	@Override
	public void processRecords() {
		
		if(getIndPlazasLibres() == null || getIndPlazasLibres().longValue() <= 0 ){
			super.processRecords();
			return;
		} else {
			
			//Filtro especial por plazas libres.
			String hqlQuery = "SELECT c.numeroPlazas " + 
	    		  " FROM es.csa.fdt.cyldigital.language.entity.CursoLanguage e " +
	    		  " INNER JOIN e.curso c " + 
	    		  " LEFT OUTER JOIN c.matriculas mc with mc.cancelada = false and mc.desactivado = false " + 
	    		  " WHERE 0=0 ";  

			for (Filter<CriterionJPAQL, CursoLanguage> c : getConditions()) {
				if (c.getCriterion() != null) {
					hqlQuery += " AND (" + c.getCriterion().getQuery() + ")";
				}
			}
			for (Filter<CriterionJPAQL, CursoLanguage> f : getFilters()) {
				if (f.getCriterion() != null) {
					hqlQuery += " AND (" + f.getCriterion().getQuery() + ")";
				}
			}
			hqlQuery += " GROUP BY e "; 
			if(getIndPlazasLibres().longValue()==1)
				hqlQuery += " HAVING count(mc) < c.numeroPlazas ";
			else 
				hqlQuery += " HAVING count(mc) >= c.numeroPlazas ";
			
			//Fix filters
			hqlQuery = hqlQuery.replaceAll(" desactivado", " e.desactivado");
			hqlQuery = hqlQuery.replaceAll(" curso.centro.id", " e.curso.centro.id");
			
			Query query = em.createQuery(hqlQuery);
		    int parameterPosition = 1;
		    for (Filter<CriterionJPAQL, CursoLanguage> c : getConditions()) {
		    	if (c.getCriterion() != null) for (Object p : c.getCriterion().getParameters()) {
		    		query.setParameter(parameterPosition, p);
		    		parameterPosition++;
		    	}
		    }
		    for (Filter<CriterionJPAQL, CursoLanguage> f : getFilters()) {
		    	if (f.getCriterion() != null) for (Object p : f.getCriterion().getParameters()) {
		    		query.setParameter(parameterPosition, p);
		    		parameterPosition++;
		    	}
		    }
		    
		    getPager().setNumRecords(query.getResultList().size());  
		    if (getPager().getNumRecords() == 0) {
		    	this.records = new ArrayList<CursoLanguage>();
		    	return;
			} else {
				hqlQuery = "SELECT e, c.numeroPlazas " + 
			    		  " FROM es.csa.fdt.cyldigital.language.entity.CursoLanguage e " +
			    		  " INNER JOIN e.curso c " + 
			    		  " LEFT OUTER JOIN c.matriculas mc with mc.cancelada = false and mc.desactivado = false " + 
			    		  " WHERE 0=0 ";
				
				for (Filter<CriterionJPAQL, CursoLanguage> c : getConditions()) {
					if (c.getCriterion() != null) {
						hqlQuery += " AND (" + c.getCriterion().getQuery() + ")";
					}
				}
				for (Filter<CriterionJPAQL, CursoLanguage> f : getFilters()) {
					if (f.getCriterion() != null) {
						hqlQuery += " AND (" + f.getCriterion().getQuery() + ")";
			        }
				}
				hqlQuery += " GROUP BY e "; 	    		   
				if(getIndPlazasLibres().longValue()==1)
					hqlQuery += " HAVING count(mc) < c.numeroPlazas ";
				else 
					hqlQuery += " HAVING count(mc) >= c.numeroPlazas ";

				if (getSort().getOrder() != null) hqlQuery += " " + getSort().getOrder();

				//Fix filters
				hqlQuery = hqlQuery.replaceAll(" desactivado", " e.desactivado");
				hqlQuery = hqlQuery.replaceAll(" curso.centro.id", " e.curso.centro.id");
				
				query = em.createQuery(hqlQuery);
				parameterPosition = 1;
				for (Filter<CriterionJPAQL, CursoLanguage> c : getConditions()) {
					if (c.getCriterion() != null) for (Object p : c.getCriterion().getParameters()) {
						query.setParameter(parameterPosition, p);
						parameterPosition++;
					}
				}
				for (Filter<CriterionJPAQL, CursoLanguage> f : getFilters()) {
					if (f.getCriterion() != null) for (Object p : f.getCriterion().getParameters()) {
						query.setParameter(parameterPosition, p);
						parameterPosition++;
					}
				}
				query.setFirstResult((getPager().getCurrentPage() - 1) * getPager().getPageSize());
				query.setMaxResults(getPager().getPageSize());
				
				this.records = new ArrayList<CursoLanguage>();
				Iterator iterRecord = query.getResultList().iterator();
				while ( iterRecord.hasNext() ) {
					Object[] oRecord = (Object[]) iterRecord.next();
					CursoLanguage oCursoLanguage = (CursoLanguage) oRecord[0];
					this.records.add(oCursoLanguage);
				}
			}
		}		
	}
	
	@Override
	public void clearFilters() {
		super.clearFilters();
		  
		filterTematica();
		filterIndIniciativaSingular();
		filterIndVideoconferencia();
		filterIndPlazasLibres();
		filterFechas();
	}
	
	//RF_XEA_12 Funcion para mantener el filtro al volver del detalle de un curso
	//3.4.8 - RF_15_20 Consulta e exportación da información das actividades
	public List<Filter<CriterionJPAQL, CursoLanguage>> cargarFiltros(List<Filter<CriterionJPAQL, CursoLanguage>> filtros, 
			Centro cen, Tematica tem, Long indIniciativaSingular, Long indVideoconferencia, Long indPlazasLibres, Date fechaInicio, Date fechaFin, Filter<CriterionJPAQL, CursoLanguage> texto) {
				
		if (null != filtros){
			
			getFilters().removeAll(getFilters());
			
			for (Filter<CriterionJPAQL, CursoLanguage> f : filtros) {
				getFilters().add(f);
			}
			
			//Para establecer el valor del centro seleccionado en el filtro inicial
			if (null != cen) {
				setCentro(cen);
			}
			
			//Para establecer el valor de la tmatica en el filtro inicial
			if (null != tem) {
				setTematica(tem);
			}

			//Para establecer el valor de indIniciativaSingular en el filtro inicial
			setIndIniciativaSingular(indIniciativaSingular);

			//Para establecer el valor de indVideoconferencia en el filtro inicial
			setIndVideoconferencia(indVideoconferencia);
			
			//Para establecer el valor de indPlazasLibres en el filtro inicial
			setIndPlazasLibres(indPlazasLibres);
			
			//Para estableces el valor de la fecha inicio en el filtro inicial
			if(null != fechaInicio){
				setFechaInicio(fechaInicio);
			}
			
			//Para estableces el valor de la fecha fin en el filtro inicial
			if(null != fechaFin){
				setFechaFin(fechaFin);
			}
			
			//Para mantener el texto introducido en el campo del filtro
			if(null != texto) {
				setFilter(texto);
			}
			
		}
		
		return null;
	}
	
	/**
	 * Codifica los filtros empleados en el browser para trasladar al proceso de exportacion.
	 * @return
	 */
	public String getFiltersEncoded(){
		String result = null;
		String sFilterQuery = "";
		ArrayList<String> oFilterList = new ArrayList<String>();
		DateFormat dfo = new SimpleDateFormat("yyyy-MM-dd 00:00:00");
		
		for (Filter<CriterionJPAQL, CursoLanguage> f : getFilters()){
			if (f.getCriterion() != null) {
				sFilterQuery += " AND (" + f.getCriterion().getQuery() + ") ";
				
				for (Object oParameter : f.getCriterion().getParameters()) {
					String sParameter = "";
					if(oParameter instanceof Long){
						sParameter = oParameter.toString();
					} else if(oParameter instanceof String) {
						sParameter = "'" + oParameter.toString().trim() + "'";
					} else if(oParameter instanceof Date) {
						sParameter = "'" + dfo.format(oParameter) + "'";
					} else {
						sParameter = oParameter.toString();
					}
					
					oFilterList.add(sParameter);
			    }
			}
		}
		oFilterList.add(sFilterQuery);
		
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		try {
			new ObjectOutputStream(out).writeObject(oFilterList);
		} catch (IOException e) {
			e.printStackTrace();
		}
		result = new String(Base64.encodeBase64(out.toByteArray()));
	
		return result;
	}
	
}
