package es.csa.fdt.cyldigital.language.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.faces.model.SelectItem;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;

import es.csa.fdt.cyldigital.diplomas.entity.Diploma;

import es.csa.fdt.cyldigital.language.entity.DiplomaLanguage;
import es.csa.fdt.cyldigital.language.entity.Language;

/**
 * Clase DiplomaLanguageServiceImpl.
 * 
 * @author Fundetec
 */
@Service
@Repository
public class DiplomaLanguageServiceImpl extends AbstractServiceImpl<DiplomaLanguage, Long> implements DiplomaLanguageService {
	private static final long serialVersionUID = -508581690737783870L;
	
	private static final Logger logger = LoggerFactory.getLogger(DiplomaLanguageServiceImpl.class);
	
	//Diplomas list for listConverter in drop downs controls
	private List<Diploma> diplomas;
	
	//Diplomas select items for drop down controls
	private List<SelectItem> diplomasSelectItems;

	/**
	 * Construye un nuevo objeto de la clase
	 */
	public DiplomaLanguageServiceImpl() {
		super();
		
		diplomas = null;
		diplomasSelectItems = null;
	}
	
	/**
	 * 
	 * @param idDiploma
	 * @param language
	 * @return
	 */
	public DiplomaLanguage find(Long idDiploma, Language language) {
		HashMap<String, Object> propiedades = new HashMap<String, Object>();
		
		propiedades.put("diploma.id", idDiploma);
		propiedades.put("language.id", language.getId());
	      
		return super.findOneWhere(propiedades);
	}
	
	public boolean reactivateDiploma(Long idDiploma) {
		try {
			Diploma diploma = find(Diploma.class, idDiploma);
			
			diploma.setDesactivado(false);
			
			disableMessages();
			
			persist(diploma);		
			
			for (DiplomaLanguage dl : diploma.getTextos()) {
				dl.setDesactivado(false);
				persist(dl);
			}
			
			enableMessages();
			
			addInfoMessage("diplomas.infDiplomaReactivado");
		}
		catch (Exception e) {
			logger.error("DiplomaLanguageServiceImpl.reactivateDiploma: " +
						 "Error reactivado el diploma con ID " + idDiploma + ". " +
						 e.getLocalizedMessage());

			enableMessages();
			
			addErrorMessage("diplomas.errorReactivatingDiploma");
			
			return false;
		}

		return true;
	}
	
	/**
	 * 
	 * @return
	 */
	public Diploma getDefaultDiploma() {
		return findOneWhere(Diploma.class, "porDefecto", true);
	}
	
	/**
	 * 
	 */
	public void createDiplomasForSelectControl() {
		if ((diplomas == null) || (diplomasSelectItems == null) ||
			!diplomas.isEmpty() || diplomasSelectItems.isEmpty()) {
			if (diplomas == null)
				diplomas = new ArrayList<Diploma>();
			
			if (diplomasSelectItems == null)
				diplomasSelectItems = new ArrayList<SelectItem>();

			diplomas.clear();
			diplomasSelectItems.clear();
		}
		
		if (diplomas.isEmpty() || diplomasSelectItems.isEmpty()) {
			String idioma = LocaleContextHolder.getLocaleContext().getLocale().toString();
			
			Map<String, Object> properties = new HashMap<String, Object>();
			properties.put("desactivado", false);
			properties.put("diploma.desactivado", false);
			properties.put("language.ISO", idioma);
			
			try {
				List<DiplomaLanguage> ldl = findWhereOrder(properties, "nombre", true);
				
				for (DiplomaLanguage dl : ldl) {
					diplomas.add(dl.getDiploma());
					diplomasSelectItems.add(new SelectItem(dl.getDiploma(), dl.getNombre()));
				}
			}
			catch (Exception e) {
				diplomas.clear();
				diplomasSelectItems.clear();
				
				logger.info("DiplomaLanguageServiceImpl.createDiplomas: " +
							"Error al conformar las listas de diplomas para el control Select. " +
							e.getLocalizedMessage());
			}
		}
	}
	
	/**
	 * 
	 * @return
	 */
	public List<Diploma> getDiplomas() {
		createDiplomasForSelectControl();
		
		return diplomas;
	}
	
	/**
	 * 
	 * @return
	 */
	public List<SelectItem> getDiplomasSelectItems() {
		createDiplomasForSelectControl();
		
		return diplomasSelectItems; 
	}
}
