package es.csa.fdt.cyldigital.language.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.faces.model.SelectItem;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.evaluacion.entity.Examen;
import es.csa.fdt.cyldigital.evaluacion.entity.ExamenConcreto;
import es.csa.fdt.cyldigital.evaluacion.entity.GrupoExamen;
import es.csa.fdt.cyldigital.evaluacion.entity.ItemExamen;
import es.csa.fdt.cyldigital.evaluacion.entity.PreguntaExamen;
import es.csa.fdt.cyldigital.evaluacion.entity.RespuestaExamen;
import es.csa.fdt.cyldigital.language.entity.ExamenLanguage;
import es.csa.fdt.cyldigital.language.entity.Language;
import es.csa.fdt.cyldigital.servicios.entity.ServicioCurso;

/**
 * Clase ExamenLanguageServiceImpl.
 * 
 * @author Fundetec
 */
@Service
@Repository
public class ExamenLanguageServiceImpl extends AbstractServiceImpl<ExamenLanguage, Long>
										   implements ExamenLanguageService {
	private static final long serialVersionUID = -1414444939877418907L;
	
	private static final Logger logger = LoggerFactory.getLogger(ExamenLanguageServiceImpl.class);

	private Boolean respondido;
	
	//Exams list for listConverter in drop downs controls
	private List<Examen> examenes;
	
	//Exams select items for drop down controls
	private List<SelectItem> examenesSelectItems;

	/**
	 * Construye un nuevo objeto de la clase
	 */
	public ExamenLanguageServiceImpl() {
		super();

		respondido = null;
		
		examenes = new ArrayList<Examen>();
		
		examenesSelectItems = new ArrayList<SelectItem>();
	}
	
	public ExamenLanguage find(Long id, Language language) {
		HashMap<String, Object> propiedades = new HashMap<String, Object>();
		
		propiedades.put("examen.id", id);
		propiedades.put("language.id", language.getId());
	      
		return findOneWhere(propiedades);
	}
	
	public boolean reactivateExamen(Long idExamen) {
		boolean reactivated = false;
		
		if ((idExamen != null) && (idExamen > 0)) {
			Examen e = findOneWhere(Examen.class, "id", idExamen);
			
			if (e != null) {
				disableMessages();
				for (ExamenLanguage el : e.getTextos()) {
					el.setDesactivado(false);
					
					persist(el);
				}
				enableMessages();
				
				e.setDesactivado(false);
				
				persist(e);
				
				reactivated = true;
			}
		}
		
		return reactivated;
	}
	
	/**
	 * Borrar item de examen.
	 * 
	 * @param examen
	 *           examen
	 * @param idItemExamen
	 *           id item de examen
	 */
	public void borrarItemExamen(Examen examen, Long idItemExamen) {
		ItemExamen p = find(ItemExamen.class, idItemExamen);

		boolean encontradoEnGrupo = false;
		int grupoPos = 0;

		//Search preguntaExamen in grupoExamen
		if (p instanceof PreguntaExamen && ((PreguntaExamen) p).getGrupoExamen() != null) {
			for (ItemExamen i : examen.getItems()) {
				if (i instanceof GrupoExamen) {
					for (PreguntaExamen pre : ((GrupoExamen) i).getPreguntas()) {
						if (pre.equals(p)) {
							grupoPos = examen.getItems().indexOf(i);

							encontradoEnGrupo = true;
						}
					}
				}
			}
		}
		//It's a grupoExamen
		else if (p instanceof GrupoExamen) {
			for (PreguntaExamen pre : ((GrupoExamen) p).getPreguntas())
				pre.setGrupoExamen(null);

			((GrupoExamen) p).getPreguntas().clear();
			
			examen.getItems().remove(p);
		}
		//It's a preguntaExamen out grupoExamen
		else {
			((PreguntaExamen)p).setExamen(null);
			
			examen.getItems().remove((ItemExamen)p);
		}
	
		//It's a preguntaExamen in grupoExamen
		if (encontradoEnGrupo) {
			((PreguntaExamen)p).setGrupoExamen(null);
			
			((GrupoExamen)examen.getItems().get(grupoPos)).getPreguntas().remove(p);
		}
	}

	/**
	 * Subir item de examen.
	 * 
	 * @param examen
	 *           examen
	 * @param idItemExamen
	 *           id item de examen
	 */
	public void subirItemExamen(Examen examen, Long idItemExamen) {
		ItemExamen p = find(ItemExamen.class, idItemExamen);
		
		if (p instanceof PreguntaExamen && ((PreguntaExamen) p).getGrupoExamen() != null) {
			int posicion = ((PreguntaExamen) p).getSecuenciaGrupo();
			
			((PreguntaExamen) p).getGrupoExamen().getPreguntas().remove(p);
			((PreguntaExamen) p).getGrupoExamen().getPreguntas().add(posicion - 1, ((PreguntaExamen) p));
		}
		else {
			int posicion = p.getSecuencia();
			
			examen.getItems().remove(p);
			examen.getItems().add(posicion - 1, p);
		}
	}
	
	/**
	 * Bajar item de examen.
	 * 
	 * @param examen
	 *           examen
	 * @param idItemExamen
	 *           id item de examen
	 */
	public void bajarItemExamen(Examen examen, Long idItemExamen) {
		ItemExamen p = find(ItemExamen.class, idItemExamen);
		
		if (p instanceof PreguntaExamen && ((PreguntaExamen) p).getGrupoExamen() != null) {
			int posicion = ((PreguntaExamen) p).getSecuenciaGrupo();
			
			((PreguntaExamen) p).getGrupoExamen().getPreguntas().remove(p);
			((PreguntaExamen) p).getGrupoExamen().getPreguntas().add(posicion + 1, ((PreguntaExamen)p));
		}
		else {
			int posicion = p.getSecuencia();

			examen.getItems().remove(p);
			examen.getItems().add(posicion + 1, p);
		}
	}
	
	public void setRespondido(Boolean respondido) {
		this.respondido = respondido;
	}
	  
	public boolean isRespondido() {
		return respondido;
	}

	public int getNumQuestionsOfExamen(Examen examen) {
		int numPreguntas = 0;
		
		for (ItemExamen ie : examen.getItems()) {
			if (ie instanceof PreguntaExamen)
				numPreguntas++;
			else
				numPreguntas += ((GrupoExamen)ie).getPreguntas().size();
		}
		
		return numPreguntas;
	}
	
	public boolean isExamAssociatedAnyService(Long idExamen) {
		Map<String, Object> properties = new HashMap<String, Object>();
		
		properties.put("desactivado", false);
		properties.put("examen.id", idExamen);
		
		return (findWhere(ServicioCurso.class, properties).size() > 0);
	}

	public List<PreguntaExamen> generatePreguntasExamen(Examen examen, Boolean all) {
		List<PreguntaExamen> preguntas = new ArrayList<PreguntaExamen>();
		
		for (ItemExamen ie : examen.getItems())
			if (ie instanceof PreguntaExamen)
				preguntas.add((PreguntaExamen)ie);
			else
				preguntas.addAll(((GrupoExamen)ie).getPreguntas());

		if (!all) {
			Collections.shuffle(preguntas);
			
			if (examen.getNumPreguntasMostrar() < preguntas.size()) {
				//List<PreguntaExamen> pToRemove = new ArrayList<PreguntaExamen>(preguntas.subList(examen.getNumPreguntasMostrar(), preguntas.size()));
				//preguntas.removeAll(pToRemove);
				
				preguntas.subList(examen.getNumPreguntasMostrar(), preguntas.size()).clear();
			}
		}

		return preguntas;
	}
	
	
	/**
	 * Load exams from database in exams and fill exams select item list
	 */
	private void createExamenesForSelectControl() {
		if (examenes == null)
			examenes = new ArrayList<Examen>();
			
		if (examenesSelectItems == null)
			examenesSelectItems = new ArrayList<SelectItem>();

		if (!examenes.isEmpty())
			examenes.clear();
		
		if (!examenesSelectItems.isEmpty())
			examenesSelectItems.clear();
		
		String idioma = LocaleContextHolder.getLocaleContext().getLocale().toString();
			
		Map<String, Object> properties = new HashMap<String, Object>();
		properties.put("examen.desactivado", false);
		properties.put("language.ISO", idioma);
		
		try {
			List<ExamenLanguage> ldl = findWhereOrder(properties, "nombre", true);
			
			for (ExamenLanguage el : ldl) {
				if (getNumQuestionsOfExamen(el.getExamen()) >= el.getExamen().getNumPreguntasMostrar()) {
					examenes.add(el.getExamen());
					examenesSelectItems.add(new SelectItem(el.getExamen(), el.getNombre()));
				}
			}
		}
		catch (Exception e) {
			examenes.clear();
			examenesSelectItems.clear();
			
			logger.info("ExamenLanguageServiceImpl.createExamenesForSelectControl: " +
						"Error al conformar las listas de examenes en formato SelectItems. " +
						e.getLocalizedMessage());
		}
	}
	
	/**
	 * 
	 * @return the examenes list
	 */
	public List<Examen> getExamenes() {
		createExamenesForSelectControl();

		return examenes;
	}
	
	/**
	 * 
	 * @return the examenes select item list
	 */
	public List<SelectItem> getExamenesSelectItems() {
		createExamenesForSelectControl();

		return examenesSelectItems; 
	}
	
	/**
	 * Get a existing or new concrete exam.
	 * 
	 * @param idExamenConcreto
	 *           id of concrete examen for possible future exam editing (Optional)
	 *           
	 * @param idExamenInicialConcreto
	 * 			 id of concrete initial examen (Optional)
	 * 
	 * @param examen
	 * 			 Exam template for create new concrete exam with empty answers and ramdom questions from examen
	 * 
	 * @return Concrete exam
	 */
	public ExamenConcreto getExamenConcreto(Long idExamenConcreto, Long idExamenInicialConcreto, Examen examen) {
		/* 
		 * Si idExamenConcreto != null y idExamenConcreto != 0 y (en el caso de que haya edición de examenes) Editable = true
		 * devuelve el examen o null (si no lo encuentra o lo encuentra pero, en el caso de que haya edición, Editable == false).
		 * 
		 * Si no si idExamenInicialConcreto != null devuelve un examen nuevo con las mismas preguntas del examen concreto
		 * que indica idExamenInicialConcreto y las respuestas vacías
		 * 
		 * Si no devuelve un nuevo ExamenConcreto vacío
		 */
		if ((idExamenConcreto != null) && (idExamenConcreto != 0)) {
			try {
				return find(ExamenConcreto.class, idExamenConcreto);
			}
			catch (Exception ex) {
				return null;
			}
		}
		else {
			if ((idExamenInicialConcreto != null)  && (idExamenInicialConcreto != 0)) {
				ExamenConcreto examenInicialConcreto;
				
				try {
					examenInicialConcreto = find(ExamenConcreto.class, idExamenInicialConcreto);
				}
				catch (Exception ex) {
					return null;
				}
				
				return createAnswersOfExam(examenInicialConcreto, new ExamenConcreto());
			}
			else
				return createAnswersOfExam(examen, new ExamenConcreto());
		}
	
	}
	
	/**
	 * Fill examenConcreto with questions of examenInicialConcreto and empty answers.
	 * 
	 * @param examenInicialConcreto
	 *           Concrete initial exam
	 *           
	 * @param examenConcreto
	 *           Concrete exam
	 *           
	 * @return Concrete exam filled
	 */
	private ExamenConcreto createAnswersOfExam(ExamenConcreto examenInicialConcreto, ExamenConcreto examenConcreto) {
		List<PreguntaExamen> preguntas = examenInicialConcreto.getPreguntas();

		try {
			for (PreguntaExamen pe : preguntas) {
				RespuestaExamen respuesta = new RespuestaExamen();
					
				respuesta.setPreguntaExamen(pe);
					
				if (!examenConcreto.getRespuestas().containsKey(pe))
					examenConcreto.getRespuestas().put(pe, respuesta);
			}
		}
		catch (Exception e) {
			e.printStackTrace();
		}

	    return examenConcreto;
	}
	  
	/**
	 * Fill examenConcreto with random questions of examen and empty answers.
	 * 
	 * @param examen
	 *           exam
	 *           
	 * @param examenConcreto
	 *           Concrete exam
	 *           
	 * @return Concrete exam filled
	 */
	private ExamenConcreto createAnswersOfExam(Examen examen, ExamenConcreto examenConcreto) {
		List<PreguntaExamen> preguntas = generatePreguntasExamen(examen, false);

		try {
			for (PreguntaExamen pe : preguntas) {
				RespuestaExamen respuesta = new RespuestaExamen();
					
				respuesta.setPreguntaExamen(pe);
					
				if (!examenConcreto.getRespuestas().containsKey(pe))
					examenConcreto.getRespuestas().put(pe, respuesta);
			}
		}
		catch (Exception e) {
			e.printStackTrace();
		}

	    return examenConcreto;
	}
	
	/**
	 * Get a existing or new concrete exam.
	 * 
	 * @param idExamenConcreto
	 *           id of concrete examen for possible future exam editing (Optional)
	 *           
	 * @param idExamenInicialConcreto
	 * 			 id of concrete initial examen (Optional)
	 * 
	 * @param examen
	 * 			 Exam template for create new concrete exam with empty answers and ramdom questions from examen
	 * 
	 * @return Concrete exam
	 */
	public ExamenConcreto getExamenConcretoNivel(Long idExamenConcreto, Long idExamenInicialConcreto, Examen examen) {
		try {
			addInfoMessage("examen.nivel.enviado");
			return find(ExamenConcreto.class, idExamenConcreto);
		}
		catch (Exception ex) {
			return null;
		}
	}
}
