package es.csa.fdt.cyldigital.language.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.HashMap;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.evaluacion.entity.Formulario;
import es.csa.fdt.cyldigital.evaluacion.entity.FormularioConcreto;
import es.csa.fdt.cyldigital.evaluacion.entity.Grupo;
import es.csa.fdt.cyldigital.evaluacion.entity.Item;
import es.csa.fdt.cyldigital.evaluacion.entity.Pregunta;
import es.csa.fdt.cyldigital.evaluacion.entity.Respuesta;
import es.csa.fdt.cyldigital.language.entity.FormularioLanguage;
import es.csa.fdt.cyldigital.language.entity.Language;
import es.csa.fdt.cyldigital.servicios.entity.ActividadExterna;

/**
 * Clase FormularioLanguageServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class FormularioLanguageServiceImpl extends AbstractServiceImpl<FormularioLanguage, Long>
										   implements FormularioLanguageService {

	private static final long serialVersionUID = -508581690737783870L;
	
	private Boolean respondido;

	/**
	 * Construye un nuevo objeto de la clase
	 */
	public FormularioLanguageServiceImpl() {
		super();
		respondido = null;
	}
	
	public FormularioLanguage find(Long id, Language language) {
		HashMap<String, Object> propiedades = new HashMap<String, Object>();
		propiedades.put("formulario.id", id);
		propiedades.put("language.id", language.getId());
	      
		return super.findOneWhere(propiedades);
	}
	
	public FormularioLanguage findFromExterna(Long idActividadExterna, Language language) {
		if (idActividadExterna != null) {
			ActividadExterna ae = find(ActividadExterna.class, idActividadExterna);
			
			if (ae.getCuestionarioSatisfaccion() != null)
				return find(ae.getCuestionarioSatisfaccion().getId(), language);
		}

		return null;
	}

	/**
	 * Borrar item.
	 * 
	 * @param formulario
	 *           formulario
	 * @param idItem
	 *           id item
	 */
	public void borrarItem(Formulario formulario, Long idItem) {
	  Item p = find(Item.class, idItem);
	  boolean encontradoEnGrupo = false;
	  int grupoPos = 0;
	
	  if (p instanceof Pregunta && ((Pregunta) p).getGrupo() != null) {
	    for (Item i : formulario.getItems())
	      if (i instanceof Grupo) for (Pregunta pre : ((Grupo) i).getPreguntas())
	        if (pre.equals(p)) {
	          grupoPos = formulario.getItems().indexOf(i);
	          encontradoEnGrupo = true;
	        }
	  } else if (p instanceof Grupo) {
	    for (Pregunta pre : ((Grupo) p).getPreguntas()) {
	      pre.setGrupo(null);
	    }
	    ((Grupo) p).getPreguntas().clear();
	    formulario.getItems().remove(p);
	  } else {
	    formulario.getItems().remove(p);
	  }
	
	  if (encontradoEnGrupo) ((Grupo) formulario.getItems().get(grupoPos)).getPreguntas().remove(p);
	}


	/**
	 * Subir item.
	 * 
	 * @param formulario
	 *           formulario
	 * @param idItem
	 *           id item
	 */
	public void subirItem(Formulario formulario, Long idItem) {
	  Item p = find(Item.class, idItem);
	  if (p instanceof Pregunta && ((Pregunta) p).getGrupo() != null) {
	    int posicion = ((Pregunta) p).getSecuenciaGrupo();
	    ((Pregunta) p).getGrupo().getPreguntas().remove(p);
	    ((Pregunta) p).getGrupo().getPreguntas().add(posicion - 1, ((Pregunta) p));
	  } else {
	    int posicion = p.getSecuencia();
	    formulario.getItems().remove(p);
	    formulario.getItems().add(posicion - 1, p);
	  }
	}
	
	/**
	 * Bajar item.
	 * 
	 * @param formulario
	 *           formulario
	 * @param idItem
	 *           id item
	 */
	public void bajarItem(Formulario formulario, Long idItem) {
	  Item p = find(Item.class, idItem);
	  if (p instanceof Pregunta && ((Pregunta) p).getGrupo() != null) {
	    int posicion = ((Pregunta) p).getSecuenciaGrupo();
	    ((Pregunta) p).getGrupo().getPreguntas().remove(p);
	    ((Pregunta) p).getGrupo().getPreguntas().add(posicion + 1, ((Pregunta) p));
	  } else {
	    int posicion = p.getSecuencia();
	    formulario.getItems().remove(p);
	    formulario.getItems().add(posicion + 1, p);
	  }
	}
	
	/**
	   * Find formulario concreto.
	   * 
	   * @param id
	   *           id
	   * @return formulario concreto
	   */
	  public FormularioConcreto findFormularioConcreto(Long id) {
	    return find(FormularioConcreto.class, id);
	  }
	  
	  /**
	   * Creacion formulario valores.
	   * 
	   * @param formulario
	   *           formulario
	   * @param concreto
	   *           concreto
	   * @return formulario concreto
	   */
	  public FormularioConcreto creacionFormularioValores(Formulario formulario, FormularioConcreto concreto) {
	    try {
	      for (Item item : formulario.getItems()) {
	        if (item.isPregunta()) {
	          Respuesta respuesta = new Respuesta();
	          respuesta.setPregunta((Pregunta) item);
	          if (!concreto.getRespuestas().containsKey(item)) concreto.getRespuestas().put((Pregunta) item, respuesta);
	        } else {
	          for (Pregunta pregunta : ((Grupo) item).getPreguntas()) {
	            Respuesta respuesta = new Respuesta();
	            respuesta.setPregunta(pregunta);
	            if (!concreto.getRespuestas().containsKey(pregunta)) concreto.getRespuestas().put(pregunta, respuesta);
	          }
	        }
	      }
	    } catch (Exception e) {
	      e.printStackTrace();
	    }
	    return concreto;
	  }	 
	  
	  /* (non-Javadoc)
	   * @see es.csa.fdt.cyldigital.evaluacion.service.FormularioService#persistFormulario(es.csa.fdt.cyldigital.evaluacion.entity.Formulario)
	   */
	  /*public void persistFormulario(Formulario formulario) {
	    persist(formulario);
	  }	*/  
	  
	  public boolean persistFormularioActExt(FormularioConcreto formulario, Long idActividadExterna) {
		  if (idActividadExterna != null) {
			  ActividadExterna ae = find(ActividadExterna.class, idActividadExterna);
			  
			  if (ae.isEncuestaDisponible()) {
				  persist(formulario);
				  return true;
			  }
		  }

		  addErrorMessage("actividad.externa.encuestaNoDisponible");

		  return false;
	  }
	  
	  public void setRespondido(Boolean respondido) {
		  this.respondido = respondido;
	  }
	  
	  public boolean isRespondido() {
		  return respondido;
	  }
}
