package es.csa.fdt.cyldigital.language.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.commons.persistence.browser.jpaql.FilterJPAQLByQuery;
import es.csa.fdt.cyldigital.core.service.BrowserFilteredByActive;
import es.csa.fdt.cyldigital.language.entity.Language;
import es.csa.fdt.cyldigital.language.entity.ServicioCursoLanguage;
import es.csa.fdt.cyldigital.servicios.entity.Tematica;

/**
 * Servicio del Browser de Servicios de Cursos en el idioma adecuado.
 * 
 * @author it-pronet
 */

@Service
@Repository
public class ServiciosCursosLanguageBrowserImpl extends	BrowserFilteredByActive<ServicioCursoLanguage>
												implements ServiciosCursosLanguageBrowser<CriterionJPAQL, String> {

	private static final long serialVersionUID = -8715967768835599075L;
	
	private Tematica tematica;
	
	/**
	 * Construye un nuevo objeto de la clase
	 */
	public ServiciosCursosLanguageBrowserImpl() {
		super();

		getSort().setSortAsc(false);
		getSort().setSortProperty("id");
	}

	/**
	 * Clear.
	 */
	public void clear() {
		this.getEntityManager().clear();
	}

	public void applyLanguageCondition(Language language) {
		this.clearConditions();
		
		this.getCondition().getCriterion().setQuery("e.language = ?");
		this.getCondition().getCriterion().addParameter(language);

		this.applyCondition();
	}
	
    public void clearFilters() {
	    super.clearFilters();
	    
	    if (tematica != null) filterByTematica(tematica.getId());
    }

	/**
	 * Devuelve tematica.
	 * 
	 * @return tematica
	 */
	public Tematica getTematica() {
		return tematica;
	}

	/**
	 * Asigna tematica.
	 * 
	 * @param tematica
	 *          tematica
	 */
	public void setTematica(Tematica tematica) {
		this.tematica = tematica;
	}
	
	/**
	 * Filter tematica.
	 */
	public void filterTematica() {
		removeFilterTematica();

		if (tematica != null) filterByTematica(tematica.getId());
	}

	private void removeFilterTematica() {
		int i = -1;
		
		for (Filter<CriterionJPAQL, ServicioCursoLanguage> f : getFilters())
			if (f.getCriterion().getQuery().contains("servicioCurso.tematica.id"))
				i = getFilters().indexOf(f);

		if (i > -1) getFilters().remove(i);
	}

	/**
	 * Filter by tematica.
	 * 
	 * @param idTematica
	 *           id tematica
	 */
	public void filterByTematica(Long idTematica) {
		String query = "";
		boolean flag = false;

		FilterJPAQLByQuery<ServicioCursoLanguage> filter = new FilterJPAQLByQuery<ServicioCursoLanguage>();

		query += " servicioCurso.tematica.id = ? ";
		filter.getCriterion().addParameter(idTematica);

		for (Filter<CriterionJPAQL, ServicioCursoLanguage> f : getFilters())
			if (f.getCriterion().getQuery().contains("servicioCurso.tematica.id")) flag = true;

		if (!flag) {
			filter.getCriterion().setQuery(query);
			getFilters().add(filter);
		}
	}
}
