package es.csa.fdt.cyldigital.language.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.HashMap;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.language.entity.Language;
import es.csa.fdt.cyldigital.language.entity.TematicaLanguage;
import es.csa.fdt.cyldigital.servicios.entity.Tematica;

/**
 * Clase TematicaLanguageServiceImpl.
 * 
 * @author it-pronet
 */
@Service
@Repository
public class TematicaLanguageServiceImpl extends AbstractServiceImpl<TematicaLanguage, Long>
										   implements TematicaLanguageService {

	private static final long serialVersionUID = -508581690737783870L;

	/**
	 * Construye un nuevo objeto de la clase
	 */
	public TematicaLanguageServiceImpl() {
		super();
	}
	
	public TematicaLanguage find(Long id, Language language) {
		HashMap<String, Object> propiedades = new HashMap<String, Object>();
		propiedades.put("tematica.id", id);
		propiedades.put("language.id", language.getId());
	      
		return super.findOneWhere(propiedades);
	}
	
	public void reactivateTematica(Long idTematica) {
		if ((idTematica != null) && (idTematica >= 0)) {
			Tematica t = find(Tematica.class, idTematica);
			
			disableMessages();
			for (TematicaLanguage tl : t.getTextos()) {
				tl.setDesactivado(false);
				
				persist(tl);
			}
			
			if ((t.getPadre() != null) && t.getPadre().getDesactivado())
				reactivateTematica(t.getPadre().getId());
			
			t.setDesactivado(false);
			
			persist(t);
			enableMessages();
			
			addInfoMessage("servicios.messageReactivatedTypology");
		}
	}
	
}
