package es.csa.fdt.cyldigital.language.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.List;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.fdt.cyldigital.core.service.BrowserFilteredByActive;
import es.csa.fdt.cyldigital.language.entity.Language;
import es.csa.fdt.cyldigital.language.entity.TematicaLanguage;


/**
 * Servicio del Browser de Tematica de Cursos en el idioma adecuado.
 * 
 * @author it-pronet
 */
@Service
@Repository
public class TematicasLanguageBrowserImpl extends	BrowserFilteredByActive<TematicaLanguage>
													implements TematicasLanguageBrowser<CriterionJPAQL, String> {
	
	private static final long serialVersionUID = -8715967768835599075L;
	
	/**
	 * Construye un nuevo objeto de la clase
	 */
	public TematicasLanguageBrowserImpl() {
		super();

		getSort().setSortAsc(false);
		getSort().setSortProperty("id");
		
		setVerDesactivados(true);
	}
	
	/**
	 * Clear.
	 */
	public void clear() {
		this.getEntityManager().clear();
	}
	
	/**
	 * Condiciona el browser para que devuelva solo Temáticas padre en
	 * un idioma determinado.
	 * 
	 * @param language
	 */
	public void applyLanguageAndOnlyFathersCondition(Language language) {
		clearConditions();
		
		getCondition().getCriterion().setQuery("e.language = ?");
		getCondition().getCriterion().addParameter(language);
		applyCondition();
		
		getCondition().getCriterion().setQuery("e.tematica.padre = null");
		applyCondition();

	}
	
	public List<TematicaLanguage> getRecordsDesactivados() {
		List<TematicaLanguage> oltl = super.getRecords();
		List<TematicaLanguage> ltl = new ArrayList<TematicaLanguage>();
		
		for (TematicaLanguage tl : oltl) {
			if (tl.getTematica().getDesactivado() || tl.getTematica().isWithChildsDeactivated()) {
				ltl.add(tl);
			}
		}
		
		oltl.clear();
			
		return ltl;
	}
}
