package es.csa.fdt.cyldigital.notificacion;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.net.URL;
import java.util.Date;
import java.util.List;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.activation.DataHandler;
import javax.activation.FileDataSource;
import javax.activation.URLDataSource;
import javax.mail.Message;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.URLName;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;
import javax.mail.PasswordAuthentication;

/**
 * Clase MailSender.
 * 
 * @author csa
 */
public class MailSender {

  private static final Logger logger = Logger.getLogger(MailSender.class.getName());
  
  private enum ESMTPProtocol {
	  // Tipos de protocolo disponibles. Pasan al constructor el puerto asociado.
	  SMTP(25), SSL(465), TLS(587);
	  
	  // Variable interna en donde se almacena el puerto.
	  private Integer port;
	  
	  // Constructor
	  private ESMTPProtocol (Integer port){
		  this.port = port;
	  }

	  // Establece el puerto asociado al protocolo
	  public void setPort(Integer port) {
		  this.port = port;
	  }
	  
	  // Devuelve el puerto asociado al protocolo
	  public Integer getPort (){
		  return port;
	  }
  };

  /**
   * Construye un nuevo objeto de la clase
   */
  public MailSender() {}

  /**
   * Envio.
   * 
   * @param senderAddress
   *           sender address
   * @param toAddress
   *           to address
   * @param ccAddress
   *           cc address
   * @param bccAddress
   *           bcc address
   * @param subject
   *           subject
   * @param isHTMLFormat
   *           is html format
   * @param body
   *           body
   * @param urlEnviar
   *           url enviar
   * @param extensionAdjunto
   *           extension adjunto
   * @param smtpServer
   *           servidor SMTP
   */
  public void envio(String senderAddress, String toAddress, String ccAddress, String bccAddress, String subject,
		  boolean isHTMLFormat, String body, String urlEnviar, String extensionAdjunto, List<String> mailSessionParams) {
	  
	  HiloMailSender hilo = new HiloMailSender(senderAddress, toAddress, ccAddress, bccAddress, subject,
			  isHTMLFormat, body, urlEnviar, extensionAdjunto, mailSessionParams);
	  hilo.start();

  }

  /**
   * Envio.
   * 
   * @param senderAddress
   *           sender address
   * @param toAddress
   *           to address
   * @param ccAddress
   *           cc address
   * @param bccAddress
   *           bcc address
   * @param subject
   *           subject
   * @param isHTMLFormat
   *           is html format
   * @param body
   *           body
   * @param urlEnviar
   *           url enviar
   * @param extensionAdjunto
   *           extension adjunto
   * @param smtpServer
   *           servidor SMTP           
   */
  public void envio(String senderAddress, String toAddress, String ccAddress, String bccAddress, String subject,
		  boolean isHTMLFormat, StringBuffer body, String urlEnviar, String extensionAdjunto, List<String> mailSessionParams) {
	  
	  HiloMailSender hilo = new HiloMailSender(senderAddress, toAddress, ccAddress, bccAddress, subject,
			  isHTMLFormat, body, urlEnviar, extensionAdjunto, mailSessionParams);
	  hilo.start();
  }
  
  /**
   * Envio.
   * 
   * @param senderAddress
   *           sender address
   * @param toAddress
   *           to address
   * @param ccAddress
   *           cc address
   * @param bccAddress
   *           bcc address
   * @param subject
   *           subject
   * @param isHTMLFormat
   *           is html format
   * @param body
   *           body
   * @param pathSignImage
   *           ruta de la imagen adjunta de la firma
   * @param smtpServer
   *           servidor SMTP           
   */
  public void envio(String senderAddress, String toAddress, String ccAddress, String bccAddress, String subject,
		  boolean isHTMLFormat, StringBuffer body, String pathSignImage, List<String> mailSessionParams) {
	  
	  HiloMailSender hilo = new HiloMailSender(senderAddress, toAddress, ccAddress, bccAddress, subject,
			  isHTMLFormat, body, pathSignImage, mailSessionParams);
	  hilo.start();
  }

  /**
   * Clase HiloMailSender.
   */
  public class HiloMailSender extends Thread {
    private String senderAddress, toAddress, ccAddress, bccAddress, subject;
    private boolean isHTMLFormat;
    private String body, urlEnviar, extensionAdjunto, pathImagenFirma, smtpServer, smtpUsername, smtpPassword, smtpNoStandarProtocol;
    private ESMTPProtocol smtpProtocol;
        

    /**
     * Construye un nuevo objeto de la clase
     * 
     * @param senderAddress
     *           sender address
     * @param toaddress
     *           toaddress
     * @param ccAddress
     *           cc address
     * @param bccAddress
     *           bcc address
     * @param subject
     *           subject
     * @param isHTMLFormat
     *           is html format
     * @param body
     *           body
     * @param urlEnviar
     *           url enviar
     * @param extensionAdjunto
     *           extension adjunto
     * @param smtpServer
     *           servidor SMTP                
     */
    public HiloMailSender(String senderAddress, String toaddress, String ccAddress, String bccAddress, String subject,
        boolean isHTMLFormat, StringBuffer body, String urlEnviar, String extensionAdjunto, List<String> mailSessionParams) {

    	this.senderAddress = senderAddress;
		this.toAddress = toaddress;
		this.ccAddress = ccAddress;
		this.bccAddress = bccAddress;
		this.subject = subject;
		this.isHTMLFormat = isHTMLFormat;
		this.body = body.toString();
		this.urlEnviar = urlEnviar;
		this.extensionAdjunto = extensionAdjunto;
		
		smtpServer = mailSessionParams.get(0);
		smtpUsername = mailSessionParams.get(3);
		smtpPassword = mailSessionParams.get(4);
		
		// if use SSL 
		if (mailSessionParams.get(1).equals("1"))
			smtpProtocol = ESMTPProtocol.SSL;
		else {
			// if use authentication
			if ((smtpUsername != null) && !smtpUsername.isEmpty() &&
				(smtpPassword != null) && !smtpPassword.isEmpty())
				smtpProtocol = ESMTPProtocol.TLS;
			else
				smtpProtocol = ESMTPProtocol.SMTP;
		}

		// if use a non standard port
		if ((mailSessionParams.get(2) != null) && !mailSessionParams.get(2).isEmpty())
			smtpProtocol.setPort(Integer.parseInt(mailSessionParams.get(2)));
    }
    
    /**
     * Construye un nuevo objeto de la clase
     * 
     * @param senderAddress
     *           sender address
     * @param toaddress
     *           toaddress
     * @param ccAddress
     *           cc address
     * @param bccAddress
     *           bcc address
     * @param subject
     *           subject
     * @param isHTMLFormat
     *           is html format
     * @param body
     *           body
     * @param pathImagenFirma
     *           ruta de la imagen de la firma
     * @param smtpServer
     *           servidor SMTP                
     */
    public HiloMailSender(String senderAddress, String toaddress, String ccAddress, String bccAddress, String subject,
        boolean isHTMLFormat, StringBuffer body, String pathImagenFirma, List<String> mailSessionParams) {

    	this.senderAddress = senderAddress;
		this.toAddress = toaddress;
		this.ccAddress = ccAddress;
		this.bccAddress = bccAddress;
		this.subject = subject;
		this.isHTMLFormat = isHTMLFormat;
		this.body = body.toString();
		this.pathImagenFirma = pathImagenFirma;
		
		smtpServer = mailSessionParams.get(0);
		smtpUsername = mailSessionParams.get(3);
		smtpPassword = mailSessionParams.get(4);
		
		// if use SSL 
		if (mailSessionParams.get(1).equals("1"))
			smtpProtocol = ESMTPProtocol.SSL;
		else {
			// if use authentication
			if ((smtpUsername != null) && !smtpUsername.isEmpty() &&
				(smtpPassword != null) && !smtpPassword.isEmpty())
				smtpProtocol = ESMTPProtocol.TLS;
			else
				smtpProtocol = ESMTPProtocol.SMTP;
		}

		// if use a non standard port
		if ((mailSessionParams.get(2) != null) && !mailSessionParams.get(2).isEmpty())
			smtpProtocol.setPort(Integer.parseInt(mailSessionParams.get(2)));
    }

    /**
     * Construye un nuevo objeto de la clase
     * 
     * @param senderAddress
     *           sender address
     * @param toaddress
     *           toaddress
     * @param ccAddress
     *           cc address
     * @param bccAddress
     *           bcc address
     * @param subject
     *           subject
     * @param isHTMLFormat
     *           is html format
     * @param body
     *           body
     * @param urlEnviar
     *           url enviar
     * @param extensionAdjunto
     *           extension adjunto
     */
    public HiloMailSender(String senderAddress, String toaddress, String ccAddress, String bccAddress, String subject,
        boolean isHTMLFormat, String body, String urlEnviar, String extensionAdjunto, List<String> mailSessionParams) {
    	
    	this.senderAddress = senderAddress;
		this.toAddress = toaddress;
		this.ccAddress = ccAddress;
		this.bccAddress = bccAddress;
		this.subject = subject;
		this.isHTMLFormat = isHTMLFormat;
		this.body = body;
		this.urlEnviar = urlEnviar;
		this.extensionAdjunto = extensionAdjunto;
		
		smtpServer = mailSessionParams.get(0);
		smtpUsername = mailSessionParams.get(3);
		smtpPassword = mailSessionParams.get(4);
		
		// if use SSL 
		if (mailSessionParams.get(1).equals("1"))
			smtpProtocol = ESMTPProtocol.SSL;
		else {
			// if use authentication
			if ((smtpUsername != null) && !smtpUsername.isEmpty() &&
				(smtpPassword != null) && !smtpPassword.isEmpty())
				smtpProtocol = ESMTPProtocol.TLS;
			else
				smtpProtocol = ESMTPProtocol.SMTP;
		}
		
		// if use a non standard port
		if ((mailSessionParams.get(2) != null) && !mailSessionParams.get(2).isEmpty())
			smtpProtocol.setPort(Integer.parseInt(mailSessionParams.get(2)));
    }

    /* (non-Javadoc)
     * @see java.lang.Thread#run()
     */
    @Override
    public void run() {
      send();
    }

    /**
     * Send.
     * 
     * @return true, if successful
     */
    public boolean send() {
    	MimeMultipart multipart = new MimeMultipart();
    	Properties properties = new Properties();
    	Session session = null;
    	Transport transport = null;

    	try {
    		switch (smtpProtocol) {
		      case SSL:
		    	  properties.put("mail.transport.protocol", "smtps");
		          properties.put("mail.smtp.socketFactory.port", smtpProtocol.getPort());
		          properties.put("mail.smtp.socketFactory.class","javax.net.ssl.SSLSocketFactory");
		          properties.put("mail.smtp.auth", "true");

		          session = Session.getInstance(properties);
		          
		          transport = session.getTransport();
		          
		          transport.connect(smtpServer, smtpProtocol.getPort(), smtpUsername, smtpPassword);

		          break;

		      case TLS:
		    	  properties.put("mail.transport.protocol", "smtp");
		    	  properties.put("mail.smtp.auth", "true");
		    	  properties.put("mail.smtp.starttls.enable", "true");

		    	  session = Session.getInstance(properties);
		    	  
		    	  transport = session.getTransport();

		    	  transport.connect(smtpServer, smtpProtocol.getPort(), smtpUsername, smtpPassword);
		    	  
		    	  break;

		      case SMTP:
		    	  properties.put("mail.transport.protocol", "smtp");
		    	  properties.put("mail.smtp.host", smtpServer);
		    	  properties.put("mail.smtp.port", smtpProtocol.getPort());
	    		  properties.put("mail.smtp.auth", "false");

	    		  session = Session.getInstance(properties, null);

		    	  break;
    		}

    		session.setDebug(true);

	        logger.log(Level.FINEST, "Comienzo envio de mensaje a " + Message.RecipientType.TO);
	        MimeMessage msg = new MimeMessage(session);

	        MimeBodyPart mbp = new MimeBodyPart();
	        mbp.setContent(body, "text/html;  charset=\"UTF-8\"");
	        multipart.addBodyPart(mbp);

	        // attachment
	        if ((urlEnviar != null) && (urlEnviar != "")) {
	        	MimeBodyPart mbp2 = new MimeBodyPart();
				URLDataSource fds = new URLDataSource(new URL(urlEnviar));
				mbp2.setDataHandler(new DataHandler(fds));
				mbp2.setFileName("informe." + extensionAdjunto);

				mbp2.setHeader("Content-Type", "application/" + extensionAdjunto);
				mbp2.setHeader("Content-Transfer-Encoding", "BASE64");
				multipart.addBodyPart(mbp2);
	        }
	        
	        // imagen de la firma
	        if ((pathImagenFirma != null) && !pathImagenFirma.isEmpty()) {
	        	MimeBodyPart imagenFirmaMBP = new MimeBodyPart();

	        	imagenFirmaMBP.setDataHandler(new DataHandler(new FileDataSource(pathImagenFirma)));
	        	imagenFirmaMBP.setDisposition(MimeBodyPart.INLINE);
	        	imagenFirmaMBP.setHeader("Content-ID", "<imagenFirma>");
	        	imagenFirmaMBP.setHeader("Content-Transfer-Encoding", "BASE64");

	        	multipart.addBodyPart(imagenFirmaMBP);
	        }
	        
	        msg.setContent(multipart);
	        msg.setFrom(new InternetAddress(senderAddress));
	        msg.setRecipients(Message.RecipientType.TO, toAddress);

	        if ((ccAddress != null) && !ccAddress.isEmpty())
	        	msg.setRecipients(Message.RecipientType.CC, ccAddress);

	        if ((bccAddress != null) && !bccAddress.isEmpty())
	        	msg.setRecipients(Message.RecipientType.BCC, bccAddress);

	        msg.setSubject(subject);
	        msg.setSentDate(new Date());

	        // Se envía el correo (en función del protocolo y de si hay o no credenciales)
	        if (smtpProtocol.equals(ESMTPProtocol.SSL) || smtpProtocol.equals(ESMTPProtocol.TLS)){
		        transport.sendMessage(msg, msg.getAllRecipients());

		        if (transport != null)
		        	transport.close();
	        } else // if (smtpProtocol.equals(ESMTPProtocol.SMTP))
	        	Transport.send(msg);

	        logger.log(Level.FINE, "Correo enviado correctamente a " + toAddress);
	      } catch (Exception e) {
	    	  logger.log(Level.SEVERE, e.toString());
	    	  e.printStackTrace();

	    	  return false;
	      }

	      return true;
    }
  }
}
