package es.csa.fdt.cyldigital.notificacion;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.Lob;
import javax.persistence.ManyToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;

import es.csa.commons.faces.util.PredefinedLocaleResolver;
import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;

/**
 * Clase Notification.
 * 
 * @author csa
 */
@Entity
@Table(name = "notificaciones")
@AttributeOverride(name = "id", column = @Column(name = "idNotificacion"))
@Audited
public class Notification extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = -6686849335429253839L;

  /**
   * Enum NotificationChanel.
   */
  public enum NotificationChanel {
    SMS,
    EMAIL,
    TLF;
  }

  private NotificationChanel chanel;
  private List<Usuario> recipients;
  private String notificationType;
  private String notificationIdentifier;
  private String subject;
  private String message;
  private Date notificationDate;
  private Date notificationStartTime;
  private Date notificationEndTime;
  private boolean htmlMessage;
  private boolean reported;
  private Date reportedDate;
  private int attempts;
  private String lastError;

  /**
   * Devuelve chanel.
   * 
   * @return chanel
   */
  @NotNull
  @Enumerated(value = EnumType.STRING)
  public NotificationChanel getChanel() {
    return chanel;
  }

  /**
   * Asigna chanel.
   * 
   * @param type
   *          chanel
   */
  public void setChanel(NotificationChanel type) {
    this.chanel = type;
  }

  /**
   * Devuelve recipients.
   * 
   * @return recipients
   */
  @NotAudited
  @NotEmpty
  @ManyToMany(targetEntity = Usuario.class)
  @JoinTable(name = "notificaciones_usuarios", joinColumns = @JoinColumn(name = "idNotificacion"), inverseJoinColumns = @JoinColumn(name = "idUsuario"))
  public List<Usuario> getRecipients() {
    return recipients;
  }

  /**
   * Asigna recipients.
   * 
   * @param recipients
   *          recipients
   */
  public void setRecipients(List<Usuario> recipients) {
    this.recipients = recipients;
  }

  /**
   * Devuelve subject.
   * 
   * @return subject
   */
  @Length(max = 150)
  @NotEmpty
  @NotNull
  @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getSubject() {
    return subject;
  }

  /**
   * Asigna subject.
   * 
   * @param subject
   *          subject
   */
  public void setSubject(String subject) {
    this.subject = subject;
  }

  /**
   * Devuelve message.
   * 
   * @return message
   */
  @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getMessage() {
    return message;
  }

  /**
   * Asigna message.
   * 
   * @param message
   *          message
   */
  public void setMessage(String message) {
    this.message = message;
  }

  /**
   * Checks if is reported.
   * 
   * @return true, if is reported
   */
  public boolean isReported() {
    return reported;
  }

  /**
   * Asigna reported.
   * 
   * @param reported
   *          reported
   */
  public void setReported(boolean reported) {
    this.reported = reported;
  }

  /**
   * Devuelve notification start time.
   * 
   * @return notification start time
   */
  @NotNull
  @Temporal(TemporalType.TIME)
  public Date getNotificationStartTime() {
    if (notificationStartTime == null && getChanel() != null) {
      Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
      switch (getChanel()) {
        case EMAIL:
          cal.set(Calendar.HOUR_OF_DAY, 0);
          cal.set(Calendar.MINUTE, 0);
          cal.set(Calendar.SECOND, 0);
          notificationStartTime = cal.getTime();
          break;
        case SMS:
        case TLF:
        default:
          cal.set(Calendar.HOUR_OF_DAY, 11);
          cal.set(Calendar.MINUTE, 0);
          cal.set(Calendar.SECOND, 0);
          notificationStartTime = cal.getTime();
          break;
      }
    }
    return notificationStartTime;
  }

  /**
   * Asigna notification start time.
   * 
   * @param notificationStartTime
   *          notification start time
   */
  public void setNotificationStartTime(Date notificationStartTime) {
    this.notificationStartTime = notificationStartTime;
  }

  /**
   * Devuelve notification end time.
   * 
   * @return notification end time
   */
  @NotNull
  @Temporal(TemporalType.TIME)
  public Date getNotificationEndTime() {
    if (notificationEndTime == null && getChanel() != null) {
      Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
      switch (getChanel()) {
        case EMAIL:
          cal.set(Calendar.HOUR_OF_DAY, 23);
          cal.set(Calendar.MINUTE, 59);
          cal.set(Calendar.SECOND, 59);
          notificationEndTime = cal.getTime();
          break;
        case SMS:
        case TLF:
        default:
          cal.set(Calendar.HOUR_OF_DAY, 22);
          cal.set(Calendar.MINUTE, 0);
          cal.set(Calendar.SECOND, 0);
          notificationEndTime = cal.getTime();
          break;
      }
    }
    return notificationEndTime;
  }

  /**
   * Asigna notification end time.
   * 
   * @param notificationEndTime
   *          notification end time
   */
  public void setNotificationEndTime(Date notificationEndTime) {
    this.notificationEndTime = notificationEndTime;
  }

  /**
   * Devuelve notification date.
   * 
   * @return notification date
   */
  @NotNull
  @Temporal(TemporalType.TIMESTAMP)
  public Date getNotificationDate() {
    if (notificationDate == null) {
      Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
      notificationDate = cal.getTime();
    }
    return notificationDate;
  }

  /**
   * Asigna notification date.
   * 
   * @param notificationDate
   *          notification date
   */
  public void setNotificationDate(Date notificationDate) {
    this.notificationDate = notificationDate;
  }

  /**
   * Devuelve notification type.
   * 
   * @return notification type
   */
  @Length(max = 255)
  public String getNotificationType() {
    return notificationType;
  }

  /**
   * Asigna notification type.
   * 
   * @param notificationType
   *          notification type
   */
  public void setNotificationType(String notificationType) {
    this.notificationType = notificationType;
  }

  /**
   * Devuelve notification identifier.
   * 
   * @return notification identifier
   */
  @Length(max = 255)
  public String getNotificationIdentifier() {
    return notificationIdentifier;
  }

  /**
   * Asigna notification identifier.
   * 
   * @param notificationIdentifier
   *          notification identifier
   */
  public void setNotificationIdentifier(String notificationIdentifier) {
    this.notificationIdentifier = notificationIdentifier;
  }

  /**
   * Devuelve reported date.
   * 
   * @return reported date
   */
  @Temporal(TemporalType.TIMESTAMP)
  public Date getReportedDate() {
    return reportedDate;
  }

  /**
   * Asigna reported date.
   * 
   * @param reportedDate
   *          reported date
   */
  public void setReportedDate(Date reportedDate) {
    this.reportedDate = reportedDate;
  }

  /**
   * Checks if is html message.
   * 
   * @return true, if is html message
   */
  public boolean isHtmlMessage() {
    return htmlMessage;
  }

  /**
   * Asigna html message.
   * 
   * @param htmlMessage
   *          html message
   */
  public void setHtmlMessage(boolean htmlMessage) {
    this.htmlMessage = htmlMessage;
  }

  /**
   * Devuelve attempts.
   * 
   * @return attempts
   */
  public int getAttempts() {
    return attempts;
  }

  /**
   * Asigna attempts.
   * 
   * @param attempts
   *          attempts
   */
  public void setAttempts(int attempts) {
    this.attempts = attempts;
  }

  /**
   * Devuelve last error.
   * 
   * @return last error
   */
  @Length(max = 255)
  public String getLastError() {
    return lastError;
  }

  /**
   * Asigna last error.
   * 
   * @param lastError
   *          last error
   */
  public void setLastError(String lastError) {
    this.lastError = lastError;
  }

}
