package es.csa.fdt.cyldigital.notificacion;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Repository;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import es.csa.commons.faces.util.PredefinedLocaleResolver;
import es.csa.fdt.cyldigital.calendario.entity.VacacionesCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.VacacionesCalendarItem.Estado;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.core.service.AbstractService;
import es.csa.fdt.cyldigital.financiero.entity.Caja;
import es.csa.fdt.cyldigital.financiero.entity.Factura;
import es.csa.fdt.cyldigital.incidencias.entity.EventoIncidencia;
import es.csa.fdt.cyldigital.incidencias.entity.Incidencia;
import es.csa.fdt.cyldigital.incidencias.entity.Incidencia.Criticidad;
import es.csa.fdt.cyldigital.incidencias.service.IncidenciaServiceImpl;
import es.csa.fdt.cyldigital.language.entity.ActividadExternaLanguage;
import es.csa.fdt.cyldigital.language.entity.CursoLanguage;
import es.csa.fdt.cyldigital.language.entity.CharlaLanguage;
import es.csa.fdt.cyldigital.language.entity.Language;
import es.csa.fdt.cyldigital.notificacion.Notification.NotificationChanel;
import es.csa.fdt.cyldigital.notificacion.entity.Alarma;
import es.csa.fdt.cyldigital.notificacion.entity.InformeNotification;
import es.csa.fdt.cyldigital.notificacion.entity.MailNotification;
import es.csa.fdt.cyldigital.notificacion.entity.Alarma.CondicionEvaluar;
import es.csa.fdt.cyldigital.notificacion.entity.InformeNotification.Periodo;
import es.csa.fdt.cyldigital.recursos.entity.Recurso;
import es.csa.fdt.cyldigital.servicios.entity.Actividad;
import es.csa.fdt.cyldigital.servicios.entity.ActividadExterna;
import es.csa.fdt.cyldigital.servicios.entity.Charla;
import es.csa.fdt.cyldigital.servicios.entity.Curso;
import es.csa.fdt.cyldigital.servicios.entity.CursoOnline;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCharla;
import es.csa.fdt.cyldigital.servicios.entity.Taller;
import es.csa.fdt.cyldigital.servicios.entity.ActividadExterna.EEstadoActividadExterna;
import es.csa.fdt.cyldigital.tareas.entity.InformeTarea;
import es.csa.fdt.cyldigital.tareas.entity.InformeTarea.EstadoTarea;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado.Puesto;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario.Role;

/**
 * Clase NotificationService.
 * 
 * @author csa
 * 
 * @param <T>
 *           generic type
 */
@SuppressWarnings({ "unchecked", "rawtypes" })
@Repository
public class NotificationService<T extends AbstractService> {

  private static final Logger logger = Logger.getLogger(NotificationService.class.getName());

  private MailSender mailSender;
  private static final String C_NOTIFICACION = "[Notificación CeMIT] ";
  private static final String C_N_INCIDENCIA = "[Incidencia XEA] ";
  private static final String RUTA_PLANTILLAS_EMAIL = "es/csa/fdt/cyldigital/plantillasEmail/";
  
  /**
   * Construye un nuevo objeto de la clase
   */
  public NotificationService() {
    super();
    mailSender = new MailSender();
  }

  /**
   * Notificar suscrito actividad.
   * 
   * @param servicio
   *           servicio
   * @param idUser
   *           id user
   * @param email
   *           email
   * @param actividad
   *           actividad
   * @return true, if successful
   */
  public boolean notificarSuscritoActividad(T servicio, Long idUser, String email, Actividad actividad) {

    boolean sendMessage = false;
    try {
      logger.log(Level.INFO, "Notificación al usuario suscrito, " + idUser);

      String ccAddress = "", bccAddress = "", subject = "";
      boolean isHTMLFormat = true;
      String senderAddress = "", urlEnviar = "";
      StringBuffer body = new StringBuffer();
      DateFormat df = new SimpleDateFormat("dd-MM-yyyy");
      DateFormat dfh = new SimpleDateFormat("HH:mm");
      
      if (actividad instanceof Curso) {
        subject = C_NOTIFICACION + "Ha sido matriculado en la actividad: "; //Comentado para implementar los servicios MULTILANGUAGE TODO: mirar y modificar + ((Curso) actividad).getServicio().getNombre();
        
        /*
         * Comentado para implementar los servicios MULTILANGUAGE
         * TODO: mirar y modificar
        body.append("Usted ha sido matriculado en la actividad " + ((Curso) actividad).getServicio().getNombre());
        */
        
        body.append(" que tendrá lugar en el centro cyldigital " + ((Curso) actividad).getCentro().getNombre());
        body.append(" del dia " + df.format(((Curso) actividad).getFechaInicio()) + " al "
            + df.format(((Curso) actividad).getFechaFin()));
        body.append(" en horario de " + dfh.format(((Curso) actividad).getHoraInicio()) + " a "
            + dfh.format(((Curso) actividad).getHoraFin()) + ".");
      } else if (actividad instanceof Taller) {
        subject = C_NOTIFICACION + "Ha sido matriculado en la actividad: "; //Comentado para implementar los servicios MULTILANGUAGE TODO: mirar y modificar + ((Taller) actividad).getServicio().getNombre();
        
        /*
         * Comentado para implementar los servicios MULTILANGUAGE
         * TODO: mirar y modificar
        body.append("Usted ha sido matriculado en la actividad " + ((Taller) actividad).getServicio().getNombre());
        */
        
        body.append(" que tendrá lugar en el centro cyldigital " + ((Taller) actividad).getCentro().getNombre());
        body.append(" del dia " + df.format(((Taller) actividad).getFechaInicio()) + " al "
            + df.format(((Taller) actividad).getFechaFin()));
        body.append(" en horario de " + dfh.format(((Taller) actividad).getHoraInicio()) + " a "
            + dfh.format(((Taller) actividad).getHoraFin()) + ".");
      } else if (actividad instanceof Charla) {
          /*
           * Comentado para implementar los servicios MULTILANGUAGE
           * TODO: mirar y modificar*/
        subject = C_NOTIFICACION + "Ha sido matriculado en la actividad: " /*+ ((Charla) actividad).getNombre()*/;
        /*
         * Comentado para implementar los servicios MULTILANGUAGE
         * TODO: mirar y modificar*/
        body.append("Usted ha sido matriculado en la actividad " /*+ ((Charla) actividad).getNombre()*/);
        body.append(" que tendrá lugar en el centro cyldigital " + ((Charla) actividad).getCentro().getNombre());
        body.append(" del dia " + df.format(((Charla) actividad).getFechaInicio()));
        body.append(" de " + dfh.format(((Charla) actividad).getHoraInicio()) + " a "
            + dfh.format(((Charla) actividad).getHoraFin()) + ".");
      } else if (actividad instanceof CursoOnline) {
        subject = C_NOTIFICACION + "Ha sido matriculado en la actividad: " + ((CursoOnline) actividad).getNombre();
        body.append("Usted ha sido matriculado en la actividad " + ((CursoOnline) actividad).getNombre());
        body.append(" mediante la plataforma de formación online (moodle)");
        body.append(" del dia " + df.format(((CursoOnline) actividad).getFechaInicio()) + " al "
            + df.format(((CursoOnline) actividad).getFechaFin()));
      }else {
        logger.log(Level.SEVERE, "La actividad pasada no es valida para la notificacion de mensajes: "
            + actividad.toString());
      }

      if (!email.isEmpty()) senderAddress = getSenderAddress(servicio);
      sendMessage = sendMessage(senderAddress, email, ccAddress, bccAddress, subject, isHTMLFormat, body, urlEnviar,
          null, getMailserverParamSession(servicio));

      if (sendMessage && !(actividad instanceof CursoOnline)) {
        // notificar el envío de correos
        Usuario u = (Usuario) servicio.find(Usuario.class, idUser);
        MailNotification m = new MailNotification();
        m.setFechaEnvio(new Date());
        m.setTipo(MailNotification.TipoElementoNotificado.ACTIVIDAD);
        m.setActividadAsociada(actividad);
        m.setDetallesEnvio("Usuario matriculado en la actividad");
        m.getUsuarios().add(u);
        servicio.persist(m);
      }
    } catch (Exception e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    return sendMessage;
  }

  /**
   * Notificar desuscrito actividad.
   * 
   * @param servicio
   *           servicio
   * @param u
   *           u
   * @param actividad
   *           actividad
   * @return true, if successful
   */
  public boolean notificarDesuscritoActividad(T servicio, Usuario u, Actividad actividad) {

    boolean sendMessage = false;
    try {
      String toAddress = "", ccAddress = "", bccAddress = "", subject = "";
      boolean isHTMLFormat = true;
      String senderAddress = "", urlEnviar = "";
      StringBuffer body = new StringBuffer();
      DateFormat df = new SimpleDateFormat("dd-MM-yyyy");
      DateFormat dfh = new SimpleDateFormat("HH:mm");

      if (actividad instanceof Curso) {
        subject = C_NOTIFICACION + "Ha sido desmatriculado de la actividad: "; //Comentado para implementar los servicios MULTILANGUAGE TODO: mirar y modificar + ((Curso) actividad).getServicio().getNombre();
        
        /*
         * Comentado para implementar los servicios MULTILANGUAGE
         * TODO: mirar y modificar
        body.append("Usted ha sido desmatriculado de la actividad " + ((Curso) actividad).getServicio().getNombre());
        */
        
        body.append(" que tendrá lugar en el centro cyldigital " + ((Curso) actividad).getCentro().getNombre());
        body.append(" del dia " + df.format(((Curso) actividad).getFechaInicio()) + " al "
            + df.format(((Curso) actividad).getFechaFin()));
        body.append(" en horario de " + dfh.format(((Curso) actividad).getHoraInicio()) + " a "
            + dfh.format(((Curso) actividad).getHoraFin()) + ".");
      } else if (actividad instanceof Taller) {
        subject = C_NOTIFICACION + "Ha sido desmatriculado de la actividad: "; //Comentado para implementar los servicios MULTILANGUAGE TODO: mirar y modificar + ((Taller) actividad).getServicio().getNombre();
        
        /*
         * Comentado para implementar los servicios MULTILANGUAGE
         * TODO: mirar y modificar
        body.append("Usted ha sido desmatriculado de la actividad " + ((Taller) actividad).getServicio().getNombre());
        */
        
        body.append(" que tendrá lugar en el centro cyldigital " + ((Taller) actividad).getCentro().getNombre());
        body.append(" del dia " + df.format(((Taller) actividad).getFechaInicio()) + " al "
            + df.format(((Taller) actividad).getFechaFin()));
        body.append(" en horario de " + dfh.format(((Taller) actividad).getHoraInicio()) + " a "
            + dfh.format(((Taller) actividad).getHoraFin()) + ".");
      } else if (actividad instanceof Charla) {
          /*
           * Comentado para implementar los servicios MULTILANGUAGE
           * TODO: mirar y modificar*/
        subject = C_NOTIFICACION + "Ha sido desmatriculado de la actividad: " /*+ ((Charla) actividad).getNombre()*/;
        /*
         * Comentado para implementar los servicios MULTILANGUAGE
         * TODO: mirar y modificar*/
        body.append("Usted ha sido desmatriculado de la  actividad " /*+ ((Charla) actividad).getNombre()*/);
        body.append(" que tendrá lugar en el centro cyldigital " + ((Charla) actividad).getCentro().getNombre());
        body.append(" del dia " + df.format(((Charla) actividad).getFechaInicio()));
        body.append(" de " + dfh.format(((Charla) actividad).getHoraInicio()) + " a "
            + dfh.format(((Charla) actividad).getHoraFin()) + ".");
      } else {
        logger.log(Level.SEVERE, "La actividad pasada no es valida para la notificacion de mensajes: "
            + actividad.toString());
      }

      UsuarioCyL ucyl = (UsuarioCyL) servicio.find(UsuarioCyL.class, u.getId());

      if (!subject.isEmpty()) toAddress = ucyl.getEmail();
      if (!toAddress.isEmpty()) senderAddress = getSenderAddress(servicio);
      sendMessage = sendMessage(senderAddress, toAddress, ccAddress, bccAddress, subject, isHTMLFormat, body,
          urlEnviar, null, getMailserverParamSession(servicio));

      if (sendMessage) {
        MailNotification m = new MailNotification();
        m.setFechaEnvio(new Date());
        m.setTipo(MailNotification.TipoElementoNotificado.ACTIVIDAD);
        m.setActividadAsociada(actividad);
        m.setDetallesEnvio("Usuario desmatriculado de la actividad");
        m.getUsuarios().add(u);
        servicio.persist(m);
      }
    } catch (Exception e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    return sendMessage;
  }

  /**
   * Notificar desuscrito curso online.
   * 
   * @param servicio
   *           servicio
   * @param email
   *           email
   * @param curso
   *           curso
   * @return true, if successful
   */
  public boolean notificarDesuscritoCursoOnline(T servicio, String email, CursoOnline curso) {

    boolean sendMessage = false;
    try {
      String toAddress = "", ccAddress = "", bccAddress = "", subject = "";
      boolean isHTMLFormat = true;
      String senderAddress = "", urlEnviar = "";
      StringBuffer body = new StringBuffer();
      DateFormat df = new SimpleDateFormat("dd-MM-yyyy");

      subject = C_NOTIFICACION + "Ha sido desmatriculado del curso online: "; //Comentado para implementar los servicios MULTILANGUAGE TODO: mirar y modificar + curso.getServicio().getNombre();
      
      /*
       * Comentado para implementar los servicios MULTILANGUAGE
       * TODO: mirar y modificar
      body.append("Usted ha sido desmatriculado del curso online " + curso.getServicio().getNombre());
      */
      
      body.append(" que tendrá lugar del dia " + df.format(curso.getFechaInicio()) + " al "
          + df.format(curso.getFechaFin()));

      if (!subject.isEmpty()) toAddress = email;
      if (!toAddress.isEmpty()) senderAddress = getSenderAddress(servicio);
      sendMessage = sendMessage(senderAddress, toAddress, ccAddress, bccAddress, subject, isHTMLFormat, body,
          urlEnviar, null, getMailserverParamSession(servicio));

    } catch (Exception e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    return sendMessage;
  }
  
  /**
   * Notificar suscrito curso online.
   * 
   * @param servicio
   *           servicio
   * @param idUser
   *           id user
   * @param email
   *           email
   * @param actividad
   *           actividad
   * @return true, if successful
   */
  public boolean notificarSuscritoCursoOnline(T servicio, Long idUser, String email, Actividad actividad) {

    boolean sendMessage = false;
    try {
      logger.log(Level.INFO, "Notificación al usuario suscrito, " + idUser);

      String ccAddress = "", bccAddress = "", subject = "";
      boolean isHTMLFormat = true;
      String senderAddress = "", urlEnviar = "";
      StringBuffer body = new StringBuffer();
      DateFormat df = new SimpleDateFormat("dd-MM-yyyy");

      if (actividad instanceof CursoOnline) {
        subject = C_NOTIFICACION + "Ha sido inscrito en la actividad: " + ((CursoOnline) actividad).getNombre();
        body.append("Usted ha sido inscrito en la actividad " + ((CursoOnline) actividad).getNombre());
        body.append(" mediante la plataforma de formación online (moodle)");
        body.append(" del dia " + df.format(((CursoOnline) actividad).getFechaInicio()) + " al "
            + df.format(((CursoOnline) actividad).getFechaFin()));
      }else {
        logger.log(Level.SEVERE, "La actividad pasada no es valida para la notificacion de mensajes: "
            + actividad.toString());
      }

      if (!email.isEmpty()) senderAddress = getSenderAddress(servicio);
      sendMessage = sendMessage(senderAddress, email, ccAddress, bccAddress, subject, isHTMLFormat, body, urlEnviar,
          null, getMailserverParamSession(servicio));
    } catch (Exception e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    return sendMessage;
  }
  
  /**
   * Notificar cuando un usuario aprueba un curso online.
   * 
   * @param servicio
   *           servicio
   * @param idUser
   *           id user
   * @param email
   *           email
   * @param actividad
   *           actividad
   * @return true, if successful
   */
  public boolean notificarAprobadoCursoOnline(T servicio, Long idUser, String email, Actividad actividad, Float nota) {

    boolean sendMessage = false;
    try {
      logger.log(Level.INFO, "Notificación al usuario suscrito, " + idUser);

      String ccAddress = "", bccAddress = "", subject = "";
      boolean isHTMLFormat = true;
      String senderAddress = "", urlEnviar = "";
      StringBuffer body = new StringBuffer();

      if (actividad instanceof CursoOnline) {
        subject = C_NOTIFICACION + "Ha aprobado el curso online: " + ((CursoOnline) actividad).getNombre();
        body.append("Usted ha aprobado el curso online " + ((CursoOnline) actividad).getNombre());
        body.append(" con una nota de " + nota.toString());
        body.append(". Ya puede imprimir su diploma desde la web cyldigital.es.");
      }else {
        logger.log(Level.SEVERE, "La actividad pasada no es valida para la notificacion de mensajes: "
            + actividad.toString());
      }

      if (!email.isEmpty()) senderAddress = getSenderAddress(servicio);
      sendMessage = sendMessage(senderAddress, email, ccAddress, bccAddress, subject, isHTMLFormat, body, urlEnviar,
          null, getMailserverParamSession(servicio));
    } catch (Exception e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    return sendMessage;
  }
  
  /**
   * Notificar cuando un usuario suspende un curso online.
   * 
   * @param servicio
   *           servicio
   * @param idUser
   *           id user
   * @param email
   *           email
   * @param actividad
   *           actividad
   * @return true, if successful
   */
  public boolean notificarSuspendidoCursoOnline(T servicio, Long idUser, String email, Actividad actividad, Float nota) {

    boolean sendMessage = false;
    try {
      logger.log(Level.INFO, "Notificación al usuario suscrito, " + idUser);

      String ccAddress = "", bccAddress = "", subject = "";
      boolean isHTMLFormat = true;
      String senderAddress = "", urlEnviar = "";
      StringBuffer body = new StringBuffer();

      if (actividad instanceof CursoOnline) {
        subject = C_NOTIFICACION + "Ha suspendido el curso online: " + ((CursoOnline) actividad).getNombre();
        body.append("Usted ha suspendido el curso online " + ((CursoOnline) actividad).getNombre());
        body.append(" con una nota de " + nota.toString());
        body.append(". Tiene la posibilidad de reinscribirse para el próximo curso desde la página web.");
      }else {
        logger.log(Level.SEVERE, "La actividad pasada no es valida para la notificacion de mensajes: "
            + actividad.toString());
      }

      if (!email.isEmpty()) senderAddress = getSenderAddress(servicio);
      sendMessage = sendMessage(senderAddress, email, ccAddress, bccAddress, subject, isHTMLFormat, body, urlEnviar,
          null, getMailserverParamSession(servicio));
    } catch (Exception e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    return sendMessage;
  }

  /**
   * Notificar resultado tarea programada.
   * 
   * @param em
   *           em
   * @param users
   *           users
   * @param informe
   *           informe
   * @return true, if successful
   */
  public boolean notificarResultadoTareaProgramada(EntityManager em, List<Usuario> users, InformeTarea informe) {
    logger.log(Level.INFO, "Notificacion a los administradores (Tarea ejecutada)");
    Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    DateFormat df = new SimpleDateFormat("dd/MM/yyyy HH:mm");

    try {
      Notification n = new Notification();
      n.setChanel(NotificationChanel.EMAIL);
      n.setHtmlMessage(true);
      n.setNotificationDate(cal.getTime());
      n.setNotificationIdentifier(df.format(cal.getTime()));
      n.setNotificationType("ResultadoTareaProgramada");
      n.setRecipients(users);

      n.setSubject(C_NOTIFICACION + " Tarea " + informe.getNombre() + " ejecutada "
          + (informe.getEstadoTarea().equals(EstadoTarea.ERROR) ? "con errores" : "corréctamente"));

      String plantilla = convertStreamToString("task_execution_result.html");
      plantilla = reemplazo(plantilla, "#TASKNAME#", informe.getNombre());
      plantilla = reemplazo(plantilla, "#TASKSTATUS#", informe.getEstadoTarea().toString());
      plantilla = reemplazo(plantilla, "#TASKSTARTDATE#", df.format(informe.getFechaInicio()));
      plantilla = reemplazo(plantilla, "#TASKENDDATE#", df.format(informe.getFechaFin()));
      plantilla = reemplazo(plantilla, "#DESCRIPTION#", informe.getMensaje());
      n.setMessage(plantilla);

      em.getTransaction().begin();
      em.persist(n);
      em.getTransaction().commit();

    } catch (RuntimeException e) {
      e.printStackTrace();
      return false;
    }
    return true;
  }

  /**
   * Notificar peticion vacaciones.
   * 
   * @param users
   *           users
   * @param item
   *           item
   * @return notification
   * @throws RuntimeException
   *            runtime exception
   */
  public Notification notificarPeticionVacaciones(List<Usuario> users, VacacionesCalendarItem item)
      throws RuntimeException {
    logger.log(Level.INFO, "Notificacion al responsable del centro (Petición de vacaciones)");
    Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    DateFormat df = new SimpleDateFormat("dd/MM/yyyy");

    Notification n = new Notification();
    n.setChanel(NotificationChanel.EMAIL);
    n.setHtmlMessage(true);
    n.setNotificationDate(cal.getTime());
    n.setNotificationIdentifier(df.format(cal.getTime()));
    n.setNotificationType("PeticionVacaciones");
    n.setRecipients(users);

    String objeto = null;
    if (item.getEstado() != null) {
      objeto = item.getEstado() == Estado.PENDIENTES ? " Petición de Vacaciones "
          : item.getEstado() == Estado.APROBADAS ? " Vacaciones Aprobadas"
              : item.getEstado() == Estado.RECHAZADAS ? " Vacaciones Rechazadas " : null;
    }

    n.setSubject(C_NOTIFICACION + objeto + " del Usuario " + item.getEmpleado().getNombreCompleto());

    String plantilla = convertStreamToString("peticion_vacaciones.html");
    plantilla = reemplazo(plantilla, "#VACACIONNAME#", objeto);
    plantilla = reemplazo(plantilla, "#VACACIONSTARTDATE#", df.format(item.getStart()));
    plantilla = reemplazo(plantilla, "#VACACIONENDDATE#", df.format(item.getEnd()));
    plantilla = reemplazo(plantilla, "#VACACIONDIAS#", "" + item.getDias());
    n.setMessage(plantilla);

    return n;
  }

  /**
   * Send notification.
   * 
   * @param notification
   *           notification
   * @param em
   *           em
   * @return true, if successful
   */
  public boolean sendNotification(Notification notification, EntityManager em) {
    switch (notification.getChanel()) {
      case EMAIL:
        sendEmailNotification(notification, em);
        return true;
      default:
        logger.log(Level.SEVERE, "El canal de notificación no esta disponible " + notification.getChanel());
        break;
    }
    return false;
  }

  private void sendEmailNotification(Notification notification, EntityManager em) {
	  
    String toAddress = "", senderAddress = "";
    boolean sendMessage = false;
    try {
      for (Usuario usuario : notification.getRecipients()) {
        if (usuario instanceof UsuarioCyL && !((UsuarioCyL) usuario).getEmail().trim().isEmpty())
          toAddress += (toAddress.trim().isEmpty() ? "" : ",") + ((UsuarioCyL) usuario).getEmail();
        if (usuario instanceof Empleado && !((Empleado) usuario).getEmail().trim().isEmpty())
          toAddress += (toAddress.trim().isEmpty() ? "" : ",") + ((Empleado) usuario).getEmail();
      }

      if (!toAddress.isEmpty()) senderAddress = getSenderAddress(em);

      sendMessage = sendMessage(senderAddress, toAddress, "", "", notification.getSubject(), notification.isHtmlMessage(), notification.getMessage(), null, null, getMailserverParamSession(em));

      if (sendMessage) {
        notification.setReported(true);
        notification.setReportedDate(new Date());
      } else {
        notification.setAttempts(notification.getAttempts() + 1);
      }
      em.getTransaction().begin();
      em.persist(notification);
      em.getTransaction().commit();
    } catch (Exception e) {
      e.printStackTrace();
      try {
        notification.setAttempts(notification.getAttempts() + 1);
        notification.setLastError(e.getLocalizedMessage());
        em.getTransaction().begin();
        em.persist(notification);
        em.getTransaction().commit();
      } catch (Exception ex) {
        ex.printStackTrace();
      }
    }
  }

  private String getSenderAddress(T servicio) {
    String sender = "";
    List<Parameter> parameters = servicio.findWhere(Parameter.class, "nombre", "mail_sender_address");

    if (parameters.size() == 1) {
      sender = parameters.get(0).getValor();
    }
    if (sender == null || sender == "") {
      sender = "nonresponder@xunta.gal";
    }

    return sender;
  }

  private String getSenderAddress(EntityManager em) {
    String sender = "";
    Query q = em.createQuery("SELECT e FROM " + Parameter.class.getName() + " e WHERE nombre = :valor1");
    q.setParameter("valor1", "mail_sender_address");
    Parameter p = (Parameter) q.getSingleResult();
    if (p != null) {
      sender = p.getValor();
    }

    if (sender == null || sender == "") {
      sender = "nonresponder@xunta.gal";
    }

    return sender;
  }

  private boolean sendMessage(String senderAddress, String toAddress, String ccAddress, String bccAddress, String subject,
		  boolean isHTMLFormat, StringBuffer body, String urlEnviar, String extensionAdjunto, List<String> mailSessionParams) {

	  if ((!toAddress.isEmpty() || (!ccAddress.isEmpty()) || (!bccAddress.isEmpty())) && !senderAddress.isEmpty()) {

		  mailSender.envio(senderAddress, toAddress, ccAddress, bccAddress,subject, 
				           isHTMLFormat, body, urlEnviar, extensionAdjunto, mailSessionParams);
		  return true;
	  }
	  return false;	  
  }
  
  private boolean sendMessage(String senderAddress, String toAddress, String ccAddress, String bccAddress, String subject,
		  boolean isHTMLFormat, StringBuffer body, String signImage, List<String> mailSessionParams) {

	  if ((!toAddress.isEmpty() || (!ccAddress.isEmpty()) || (!bccAddress.isEmpty())) && !senderAddress.isEmpty()) {

		  mailSender.envio(senderAddress, toAddress, ccAddress, bccAddress,subject, 
				           isHTMLFormat, body, signImage, mailSessionParams);
		  return true;
	  }
	  return false;	  
  }

  private boolean sendMessage(String senderAddress, String toAddress, String ccAddress, String bccAddress, String subject,
		  boolean isHTMLFormat, String body, String urlEnviar, String extensionAdjunto,  List<String> mailSessionParams) {
	  
	  if (!toAddress.isEmpty() && !senderAddress.isEmpty()) {

	      mailSender.envio(senderAddress, toAddress, ccAddress, bccAddress, subject, 
	    		           isHTMLFormat, body, urlEnviar, extensionAdjunto, mailSessionParams);
		  return true;
	  }
	  return false;
  }

  /**
   * Devuelve to address.
   * 
   * @param user
   *           user
   * @param servicio
   *           servicio
   * @return to address
   */
  public String getToAddress(Usuario user, T servicio) {
    String toAddress = "";

    // diferenciamos si es un empleado o un usuario el que crea la incidencia.
    Empleado e = ((IncidenciaServiceImpl) servicio).find(Empleado.class, user.getId());
    toAddress = e.getEmail();
    if (e.getEmail() == null || e.getEmail().isEmpty()) {
      UsuarioCyL u = ((IncidenciaServiceImpl) servicio).find(UsuarioCyL.class, user.getId());
      toAddress = u.getEmail();
    }

    return toAddress;
  }

  /**
   * Incidencia envia mensaje.
   * 
   * @param servicio
   *           servicio
   * @param incidencia
   *           incidencia
   * @param accion
   *           accion
   * @return true, if successful
   */
  public boolean incidenciaEnviaMensaje(T servicio, Incidencia incidencia, String accion) {
    boolean sendMessage = false;
    MailNotification m = new MailNotification();

    try {
      String toAddress = "", ccAddress = "", bccAddress = "", subject = "", senderAddress = "", urlEnviar = "";
      boolean isHTMLFormat = true;
      StringBuffer body = new StringBuffer();
      List<Usuario> usuariosParaEnviar = new ArrayList<Usuario>();
      List<Usuario> aux = new ArrayList<Usuario>();

      String nombre = incidencia.getNombre();
      String details = getDetails(incidencia, servicio);
      Criticidad criticidad = incidencia.getCriticidad();
      if (accion.equals("solicitarinformacion")) {
        subject = C_N_INCIDENCIA + "[ID " + incidencia.getId() + "] Se ha solicitado información sobre la incidencia '"
            + nombre + "' (" + criticidad + ")";
        body.append("Se ha solicitado m&aacute;s informaci&oacute;n sobre una incidencia de XEA." + details);
      } else if (accion.equals("aceptarincidencia")) {
        subject = C_N_INCIDENCIA + "[ID " + incidencia.getId() + "] Aceptada la incidencia '" + nombre + "' ("
            + criticidad + ")";
        body.append("La incidencia '" + nombre + "' ha sido aceptada y est&aacute; en curso." + details);
      } else if (accion.equals("corregirincidencia")) {
        subject = C_N_INCIDENCIA + "[ID " + incidencia.getId() + "] Corregida la incidencia '" + nombre + "' ("
            + criticidad + ").";
        body.append("La incidencia '" + nombre + "' ha sido corregida. " + details);
      } else if (accion.equals("aceptarrechazo")) {
        subject = C_N_INCIDENCIA + "[ID " + incidencia.getId() + "] Anulada la incidencia '" + nombre + "' ("
            + criticidad + ")";
        body.append("La incidencia '" + nombre + "' ha sido anulada por fundetec." + details);
      } else if (accion.equals("rechazarincidencia")) {
        subject = C_N_INCIDENCIA + "[ID " + incidencia.getId() + "] Rechazada la incidencia: '"
            + incidencia.getNombre() + "' (" + criticidad + ")";
        body.append("La incidencia '" + nombre + "' ha sido rechazada." + details);
      } else if (accion.equals("aceptarcorreccion")) {
        subject = C_N_INCIDENCIA + "[ID " + incidencia.getId() + "] Cerrada la incidencia '" + nombre + "' ("
            + criticidad + ").";
        body.append("La incidencia '" + nombre + "' ha sido cerrada." + details);
      } else if (accion.equals("rechazarrechazo")) {
        subject = C_N_INCIDENCIA + "[ID " + incidencia.getId() + "] Se ha reabierto la incidencia: '" + nombre + "' ("
            + criticidad + ") que había sido rechazada.";
        body.append("Se ha reabierto la incidencia '" + nombre + "'." + details);
      } else if (accion.equals("rechazarcorreccion")) {
        subject = C_N_INCIDENCIA + "[ID " + incidencia.getId() + "] Rechazada la corrección de la incidencia: '"
            + nombre + "' (" + criticidad + ").";
        body.append("Se ha rechazado la correcci&oacute;n de la incidencia '" + nombre + "'." + details);
      } else if (accion.equals("anadircomentario")) {
        subject = C_N_INCIDENCIA + "[ID " + incidencia.getId() + "] Nuevo comentario en la incidencia: '" + nombre
            + "' (" + criticidad + ").";
        body.append("Se ha publicado un comentario en la incidencia '" + nombre + "':" + "</br></br>"
            + incidencia.getEventosIncidencia().get(incidencia.getEventosIncidencia().size() - 1).getDescripcion()
            + "</br></br>" + details);
      } else if (accion.equals("editarincidencia")) {
        subject = C_N_INCIDENCIA + "[ID " + incidencia.getId() + "] Se ha editado la incidencia: '" + nombre + "' ("
            + criticidad + ").";
        body.append("Se ha editado la incidencia '" + nombre + "'." + details);
      } else if (accion.equals("informacionenviada")) {
        subject = C_N_INCIDENCIA + "[ID " + incidencia.getId() + "] Nueva información para la incidencia: '" + nombre
            + "' (" + criticidad + ").";
        body.append("El usuario ha facilitado nueva informaci&oacute;n por parte del  '" + nombre + "'." + details);
      } else {
        subject = C_N_INCIDENCIA + "[ID " + incidencia.getId() + "] Se ha abierto una incidencia: '" + nombre + "' ("
            + criticidad + ")";
        body.append("Se ha abierto una incidencia en XEA." + details);
      }

      // a partir de aquí, buscamos los usuarios a los que hay que notificar.
      if (accion.equals("rechazarincidencia")) {
        usuariosParaEnviar = getEmpleadosFundetec(servicio, incidencia);
      }

      // rellenar los destinatarios
      incidencia.getUsuario(); // esto es solo para poder acceder al usuario.
      toAddress = getToAddress(incidencia.getUsuario(), servicio);
      usuariosParaEnviar.add(incidencia.getUsuario());
      getRecipients(incidencia, servicio, usuariosParaEnviar);
      senderAddress = getSenderAddress(servicio);

      if (toAddress == null || toAddress.isEmpty()) {
        toAddress = bccAddress;
        logger.log(Level.SEVERE, "El usuario que abrió la incidencia no tiene dirección de correo electrónico, "
            + "no se le pudo enviar el correo.");
      }

      sendMessage = sendMessage(senderAddress, toAddress, ccAddress, bccAddress, subject, isHTMLFormat, body,
          urlEnviar, null, getMailserverParamSession(servicio));

      // registrar el envío del correo.
      if (sendMessage) {
        m.setFechaEnvio(new Date());
        m.setTipo(MailNotification.TipoElementoNotificado.INCIDENCIA);
        m.setIncidenciaAsociada(incidencia);
        m.setDetallesEnvio(accion);

        // quito los posibles repetidos para que no se graben en la notificación.
        aux.clear();
        for (Usuario u : usuariosParaEnviar) {
          if (!aux.contains(u)) aux.add(u);
        }

        if (!aux.contains(incidencia.getUsuario())) aux.add(incidencia.getUsuario());

        usuariosParaEnviar.clear();
        usuariosParaEnviar.addAll(aux);

        m.setUsuarios(usuariosParaEnviar);
      }

      servicio.persist(m);
    } catch (Exception e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    return sendMessage;

  }

  private List<Usuario> getEmpleadosFundetec(T servicio, Incidencia incidencia) {
    boolean sendallresponsibles = false;
    List<Usuario> resultado = new ArrayList<Usuario>();
    List<Empleado> empleadosfundetec;
    // Notificamos al empleado de fundetec del centro de la incidencia

    if (incidencia.getCentro() != null) {
      empleadosfundetec = servicio.findByQuery(Empleado.class,
          "select e from Usuario u, Empleado e where e.id=u.id and e.responsableIncidencias = true and u.centro.id="
              + incidencia.getCentro().getId());

      if (empleadosfundetec.size() > 0) {
        resultado.add(empleadosfundetec.get(0));
      } else sendallresponsibles = true;
    } else sendallresponsibles = true;

    if (sendallresponsibles) // si la incidencia no tiene centro o no hay empleado fundetec de ese centro,
    // notificamos a todos los responsables de incidencias.
    {
      empleadosfundetec = servicio.findByQuery(Empleado.class,
          "select e from Empleado e where e.responsableIncidencias = true");
      if (empleadosfundetec.size() > 0) {
        for (Empleado empleado : empleadosfundetec) {
          if (empleado.getEmail() != "") resultado.add(empleado);
        }
      } else {
        logger.log(Level.SEVERE,
            "No se encontró un responsable de incidencias. Es necesario indique a un empleado como responsable.");
      }
    }
    return resultado;
  }

  private String getDetails(Incidencia incidencia, T servicio) {
    String url = "";
    ResourceBundle res = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages");
    EventoIncidencia ultimoevento = incidencia.getEventosIncidencia().get(incidencia.getEventosIncidencia().size() - 1);

    List<Parameter> parameters = servicio.findWhere(Parameter.class, "nombre", "mail_application_url");
    if (parameters.size() == 1) {
      url = parameters.get(0).getValor();
    }

    String s = "" + "<br/ ><br/ ><b>T&iacute;tulo de la incidencia:</b>  " + incidencia.getNombre()
        + "<br/ ><b>Categor&iacute;a:</b> " + incidencia.getCategoriaIncidencia().getNombre()
        + "<br/ > <b>Usuario:</b> " + incidencia.getUsuario().getNombreCompleto()
        + "<br/ > <b>Fecha de la incidencia</b> " + incidencia.getFechaIncidencia() + "<br/ > <b>Tipo: </b> "
        + res.getString("tipo." + incidencia.getTipo()) + "<br/ > <b>Criticidad: </b> "
        + res.getString("criticidad." + incidencia.getCriticidad());

    if (incidencia.getCentro() != null) s += "<br/> <b>Centro:</b> " + incidencia.getCentro().getNombre();

    s += "<br/ > <b>Estado: </b> " + res.getString("estado.incidencia." + incidencia.getEstado())
        + "<br/ > <b>Descripci&oacute;n:</b><br/>" + incidencia.getDescripcion();

    if (ultimoevento.getDescripcion() != null && !ultimoevento.getDescripcion().trim().equals(""))
      s += "<br/><br /><b>Comentario sobre la actuaci&oacute;n:</b><br/>" + ultimoevento.getDescripcion();

    if (incidencia.getId() != 0)
      s += "<br/> <br/><a href='" + url + "/spring/incidencias/incidencia/view?id=" + incidencia.getId()
          + "'>Ver la incidencia</a><br/><br/><br/>";

    s += "<br/ > <br/><br/><b>Enviado por XEA</b><br/ >" + "<img src='" + url + "/images/logo.png' />" + "";
    return s;
  }

  private void getRecipients(Incidencia incidencia, T servicio, List<Usuario> usuariosParaEnviar) {
    List<Empleado> empleados = new ArrayList<Empleado>();

    if (!incidencia.getCategoriaIncidencia().isEspecial()) {
      empleados = ((IncidenciaServiceImpl) servicio).findWhere(Empleado.class, "puesto", Puesto.TRAINER_COORDINATOR);
      empleados.addAll(((IncidenciaServiceImpl) servicio).findWhere(Empleado.class, "puesto", Puesto.MANAGER));
      empleados.addAll(((IncidenciaServiceImpl) servicio).findWhere(Empleado.class, "puesto", Puesto.ADMIN));
      empleados.addAll(((IncidenciaServiceImpl) servicio).findWhere(Empleado.class, "puesto", Puesto.DIGITAL_ALLY_MANAGER));

      // o añadir un mapa de condiciones para el coordinador del centro
      List<Empleado> aux = ((IncidenciaServiceImpl) servicio).findWhere(Empleado.class, "puesto", Puesto.AREA_MANAGER);
      if (incidencia.getCentro() != null) for (Empleado empleado : aux) {
        if (empleado.getCentro().getId() == incidencia.getCentro().getId()) empleados.add(empleado);
      }
      else empleados.addAll(aux);

      // al no ser especial, quitamos a los que no quieren recibir correos.
      aux.clear();
      
      /* TODO: Comprobar más tarde, quitado para Xunta de Galicia
      for (Empleado e : empleados) {
        if (e.getRolesList().contains(Role.ROLE_INCIDENCE_RECEIVE_MAIL)) aux.add(e);
      }*/
      
      empleados.clear();
      empleados.addAll(aux);
    }

    // si es especial, mandar solo a los empleados de la categoria.
    empleados.addAll(incidencia.getCategoriaIncidencia().getEmpleados());

    usuariosParaEnviar.addAll(empleados);
  }

  private String reemplazo(String plantilla, String texto, String reemplazo) {
    plantilla = plantilla.replaceAll(texto, reemplazo);
    return plantilla;
  }

  // convierte el fichero en un String
  /**
   * Convert stream to string.
   * 
   * @param plantilla
   *           plantilla
   * @return string
   */
  public static String convertStreamToString(String plantilla) {
    try {
      InputStream is = NotificationService.class.getClassLoader()
          .getResourceAsStream(RUTA_PLANTILLAS_EMAIL + plantilla);
      BufferedReader reader = new BufferedReader(new InputStreamReader(is));
      StringBuilder sb = new StringBuilder();
      String line = null;
      while ((line = reader.readLine()) != null) {
        sb.append(line + "\n");
      }
      is.close();
      return sb.toString();
    } catch (IOException e) {
      e.printStackTrace();
    }
    return "";
  }

  /**
   * Send informe.
   * 
   * @param informe2
   *           informe2
   * @param em
   *           em
   */
  public void sendInforme(InformeNotification informe2, EntityManager em) {
    String toAddress = "", senderAddress = "", subject = "", urlEnviar = "", message = "";
    boolean sendMessage = false, htmlMessage = true;
    ResourceBundle res = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages");
    List<Usuario> usuariosAEnviar = new ArrayList<Usuario>();
    List<Usuario> aux = new ArrayList<Usuario>();

    String consulta = "SELECT e FROM " + Parameter.class.getName() + " e WHERE " + " nombre='mail_application_url'";

    Query query = em.createQuery(consulta);
    Parameter p = (Parameter) query.getSingleResult();

    try {
      Boolean primeraRonda = true;
      for (Empleado e : informe2.getEmpleados()) {
        if (!e.getEmail().isEmpty()) {
          if (!primeraRonda) {
            toAddress += ",";
            primeraRonda = false;
          }
          toAddress += e.getEmail();
          usuariosAEnviar.add(e);
        }
      }
      if (!toAddress.isEmpty()) {
        senderAddress = getSenderAddress(em);
      }

      subject = C_NOTIFICACION + "Envío del informe: " + informe2.getInforme().getNombre() + " (Periodicidad:"
          + res.getString("informeNotification.periodicidad." + informe2.getPeriodicidad()) + ")";
      urlEnviar = informe2.getInforme().getUrl();

      urlEnviar += "?idCentro=";
      if (informe2.getCentro() != null) urlEnviar += informe2.getCentro().getId();

      Calendar c = Calendar.getInstance();
      c.setTime(new Date());

      if (informe2.getPeriodo() == Periodo.ANIO_ACTUAL) {
        DateFormat df = new SimpleDateFormat("yyyy");
        urlEnviar += "&fechaInicio=1/1/" + df.format(c.getTime()) + "&fechaFin=31/12/" + df.format(c.getTime());

      } else if (informe2.getPeriodo() == Periodo.ANIO_PASADO) {
        DateFormat df = new SimpleDateFormat("yyyy");
        c.add(Calendar.YEAR, -1);
        urlEnviar += "&fechaInicio=1/1/" + df.format(c.getTime()) + "&fechaFin=31/12/" + df.format(c.getTime());
      } else if (informe2.getPeriodo() == Periodo.MES_ACTUAL) {
        DateFormat dl = new SimpleDateFormat("MM/yyyy");
        urlEnviar += "&fechaInicio=1/" + dl.format(c.getTime()) + "&fechaFin="
            + c.getActualMaximum(Calendar.DAY_OF_MONTH) + "/" + dl.format(c.getTime());
      } else if (informe2.getPeriodo() == Periodo.MES_PASADO) {
        DateFormat dl = new SimpleDateFormat("MM/yyyy");
        c.add(Calendar.MONTH, -1);
        urlEnviar += "&fechaInicio=1/" + dl.format(c.getTime()) + "&fechaFin="
            + c.getActualMaximum(Calendar.DAY_OF_MONTH) + "/" + dl.format(c.getTime());
      } else if (informe2.getPeriodo() == Periodo.SEMANA_ACTUAL) {
        DateFormat df = new SimpleDateFormat("dd/MM/yyyy");
        c.add(Calendar.DAY_OF_YEAR, 8 - c.get(Calendar.DAY_OF_WEEK));
        urlEnviar += "&fechaFin=" + df.format(c.getTime());
        c.add(Calendar.DAY_OF_YEAR, -6);

        urlEnviar += "&fechaInicio=" + df.format(c.getTime());
      }

      DateFormat dtime = new SimpleDateFormat("hh:MM");
      DateFormat df = new SimpleDateFormat("dd/MM/yyyy");
      urlEnviar += "%20" + dtime.format(c.getTime());

      if (informe2.getInforme().getTextoMensaje() == null) {
        message = "Se envía adjunto el informe solicitado:<br />" + "<br /><b>Informe:</b> "
            + informe2.getInforme().getNombre() + "<br />" + "<b>Descripción:</b> "
            + informe2.getInforme().getDescripcion() + "<br /><b>Fecha de envío:</b> " + df.format(informe2.getFecha())
            + " " + dtime.format(informe2.getHora()) + "<br /><b>Periodicidad de envío:</b> "
            + res.getString("informeNotification.periodicidad." + informe2.getPeriodicidad())
            + "<br /><b>Periodo:</b> " + res.getString("informeNotification.periodo." + informe2.getPeriodo());
      } else message = informe2.getInforme().getTextoMensaje();

      if (informe2.getCentro() != null) message += "<br /><b>Centro:</b> " + informe2.getCentro().getNombre();
      else message += "<b>Centro:</b> Todos los centros ";
      message += "<br /><br/ > <br/><br/><b>Enviado por XEA</b><br/ >" + "<img src='" + p.getValor()
          + "/images/logo.png' />";
      sendMessage = sendMessage(senderAddress, toAddress, "", "", subject, htmlMessage, message, urlEnviar, informe2
          .getInforme().getExtension(), getMailserverParamSession(em));

      if (sendMessage) {
        MailNotification m = new MailNotification();
        m.setFechaEnvio(new Date());
        m.setTipo(MailNotification.TipoElementoNotificado.INFORME);
        m.setInformeAsociado(informe2.getInforme());

        // quito los posibles repetidos
        for (Usuario u : usuariosAEnviar) {
          if (!aux.contains(u)) aux.add(u);
        }
        usuariosAEnviar = aux;
        m.setUsuarios(usuariosAEnviar);
        em.getTransaction().begin();
        em.persist(m);
        em.getTransaction().commit();
      }

    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  /**
   * Send alarma.
   * 
   * @param a
   *           a
   * @param em
   *           em
   */
  public void sendAlarma(Alarma a, EntityManager em) {
    String toAddress = "", senderAddress = "", subject = "", urlEnviar = "", message = "";
    boolean sendMessage = false, htmlMessage = true;
    ResourceBundle res = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages");
    DateFormat df = new SimpleDateFormat("dd/MM/yyyy");
    DateFormat dh = new SimpleDateFormat("HH:mm");
    List<Usuario> usuariosAEnviar = new ArrayList<Usuario>();

    String consulta = "SELECT e FROM " + Parameter.class.getName() + " e WHERE " + " nombre='mail_application_url'";

    Query query = em.createQuery(consulta);
    Parameter p = (Parameter) query.getSingleResult();

    try {
      for (Empleado e : a.getEmpleados()) {
        if (!e.getEmail().isEmpty()) {
          toAddress += e.getEmail();
          toAddress += ",";
          usuariosAEnviar.add(e);
        }
      }
      if (!toAddress.isEmpty()) {
        senderAddress = getSenderAddress(em);
      }

      subject = C_NOTIFICACION + "Alarma: " + a.getNombre() + ". Periodo: "
          + res.getString("alarma.periodoIndicador." + a.getPeriodoBP());

      message = "Se ha disparado esta alarma:<br />" + "<br /><b>Alarma:</b> " + a.getNombre()
          + "<br /><b>Descripción:</b> " + a.getDescripcion() + "<br /><b>Causa:</b> "
          + res.getString("BP." + a.getTipo()) + " " + res.getString("alarma.operando." + a.getOperando()) + " ";

      if (a.getCondicionEvaluar() == CondicionEvaluar.NIVELINFERIOR) message += " nivel inferior";
      else if (a.getCondicionEvaluar() == CondicionEvaluar.BP) message += " BP";
      else if (a.getCondicionEvaluar() == CondicionEvaluar.NIVELSUPERIOR) message += " nivel superior";
      else if (a.getCondicionEvaluar() == CondicionEvaluar.PORCENTAJEBP) message += a.getValorEvaluar() + " (del BP)";
      else if (a.getCondicionEvaluar() == CondicionEvaluar.PORCENTAJEBP) message += a.getValorEvaluar();

      message += " <br /><b>Periodo:</b> " + res.getString("alarma.periodoIndicador." + a.getPeriodoBP()) + "";
      if (a.getCentro() != null) message += "<br /><b>Centro: </b>" + a.getCentro().getNombre();
      else message += "<br /><b>Centro:</b> Todos los centros";
      message += "<br /><b>Notificar:</b> "
          + res.getString("alarma.frecuenciaNotificacion." + a.getFrecuenciaNotificacion()) + " a partir de: "
          + df.format(a.getFechaEnvio()) + " " + dh.format(a.getHoraEnvio());
      if (a.isNotificarSiCambia()) message += ", notificar solo si cambia.";

      message += "<br /><br/ > <br/><br/><b>Enviado por XEA</b><br/ >" + "<img src='" + p.getValor()
          + "/images/logo.png' />";

      sendMessage = sendMessage(senderAddress, toAddress, "", "", subject, htmlMessage, message, urlEnviar, null, getMailserverParamSession(em));

      if (sendMessage) {
        MailNotification m = new MailNotification();
        m.setFechaEnvio(new Date());
        m.setTipo(MailNotification.TipoElementoNotificado.ALARMA);
        m.setAlarmaAsociada(a);
        m.setUsuarios(usuariosAEnviar);
        em.getTransaction().begin();
        em.persist(m);
        em.getTransaction().commit();
      }

    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  /**
   * Notificar presupuesto rechazado.
   * 
   * @param em
   *           em
   * @param users
   *           users
   * @param factura
   *           factura
   * @return true, if successful
   */
  public boolean notificarPresupuestoRechazado(EntityManager em, List<Usuario> users, Factura factura) {
    logger.log(Level.INFO, "Notificación al creador del presupuesto.");
    Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    DateFormat df = new SimpleDateFormat("dd/MM/yyyy HH:mm");

    try {
      Notification n = new Notification();
      n.setChanel(NotificationChanel.EMAIL);
      n.setHtmlMessage(true);
      n.setNotificationDate(cal.getTime());
      n.setNotificationIdentifier(df.format(cal.getTime()));
      n.setNotificationType("PresupuestoRechazado");
      n.setRecipients(users);

      n.setSubject(C_NOTIFICACION + "El presupuesto " + Long.toString(factura.getId()) + " ha sido rechazado.");

      String plantilla = convertStreamToString("budget_rejection.html");
      plantilla = reemplazo(plantilla, "#BILLID#", Long.toString(factura.getId()));
      plantilla = reemplazo(plantilla, "#BILLCONCEPT#", factura.getConcepto());
      plantilla = reemplazo(plantilla, "#BILLAMOUNT#", factura.getCuantiaPresupuesto().toString());
      plantilla = reemplazo(plantilla, "#BILLCREATIONDATE#", df.format(factura.getFechaIntroduccion()));
      plantilla = reemplazo(plantilla, "#REJECTIONCAUSE#", factura.getEstadoActual().getObservaciones());
      n.setMessage(plantilla);

      em.persist(n);
    } catch (RuntimeException e) {
      e.printStackTrace();
      return false;
    }
    return true;
  }

  /**
   * Notificar aprobacion presupuesto.
   * 
   * @param em
   *           em
   * @param users
   *           users
   * @param factura
   *           factura
   * @return true, if successful
   */
  public boolean notificarAprobacionPresupuesto(EntityManager em, List<Usuario> users, Factura factura) {
    logger.log(Level.INFO, "Notificación al responsable de aprobación del presupuesto.");
    Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    DateFormat df = new SimpleDateFormat("dd/MM/yyyy HH:mm");

    try {
      Notification n = new Notification();
      n.setChanel(NotificationChanel.EMAIL);
      n.setHtmlMessage(true);
      n.setNotificationDate(cal.getTime());
      n.setNotificationIdentifier(df.format(cal.getTime()));
      n.setNotificationType("AprobacionPresupuesto");
      n.setRecipients(users);

      n.setSubject(C_NOTIFICACION + "Un nuevo presupuesto: '" + factura.getConcepto() + "' ha sido dado de alta.");

      String plantilla = convertStreamToString("budget_approval.html");
      plantilla = reemplazo(plantilla, "#ITEMNAME#", factura.getContenedor().getNombre());
      plantilla = reemplazo(plantilla, "#BILLCONCEPT#", factura.getConcepto());
      plantilla = reemplazo(plantilla, "#BILLAMOUNT#", factura.getCuantiaPresupuesto().toString());
      plantilla = reemplazo(plantilla, "#BILLCREATIONDATE#", df.format(factura.getFechaIntroduccion()));
      n.setMessage(plantilla);

      em.persist(n);

    } catch (RuntimeException e) {
      e.printStackTrace();
      return false;
    }
    return true;
  }

  /**
   * Notificar caja vacia.
   * 
   * @param em
   *           em
   * @param users
   *           users
   * @param caja
   *           caja
   * @return true, if successful
   */
  public boolean notificarCajaVacia(EntityManager em, List<Usuario> users, Caja caja) {
    logger.log(Level.INFO, "Notificación al responsable del centro.");
    Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    DateFormat df = new SimpleDateFormat("dd/MM/yyyy HH:mm");

    try {
      Notification n = new Notification();
      n.setChanel(NotificationChanel.EMAIL);
      n.setHtmlMessage(true);
      n.setNotificationDate(cal.getTime());
      n.setNotificationIdentifier(df.format(cal.getTime()));
      n.setNotificationType("CajaVacia");
      n.setRecipients(users);

      /*
       * Comentado para implementar los servicios MULTILANGUAGE
       * TODO: mirar y modificar 
      n.setSubject(C_NOTIFICACION + "La caja del centro de " + caja.getCentro().getLocalidad().getNombre()
          + " está vacía.");
       */

      String plantilla = convertStreamToString("empty_cash_alert.html");

      /*
       * Comentado para implementar los servicios MULTILANGUAGE
       * TODO: mirar y modificar
      plantilla = reemplazo(plantilla, "#OFFICECITY#", caja.getCentro().getLocalidad().getNombre());
      */
      
      n.setMessage(plantilla);

      em.persist(n);

    } catch (RuntimeException e) {
      e.printStackTrace();
      return false;
    }
    return true;
  }

  /**
   * Notificar cantidad insuficiente.
   * 
   * @param servicio
   *           servicio
   * @param r
   *           r
   * @return true, if successful
   */
  public boolean notificarCantidadInsuficiente(T servicio, Recurso r) {

    boolean sendMessage = false;

    try {
      String ccAddress = "", bccAddress = "", subject = "";
      boolean isHTMLFormat = true;
      String senderAddress = "", urlEnviar = "", url = "";
      StringBuffer body = new StringBuffer();
      ResourceBundle res = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages");

      // localizar los areamanger del centro
      List<Empleado> empleadosAEnviar = new ArrayList<Empleado>();
      List<Empleado> empleados = servicio.findAll(Empleado.class);

      for (Empleado e : empleados) {
        if (e.getCentro() == r.getCentro() && (e.getPuesto().equals(Puesto.AREA_MANAGER) || 
        		e.getPuesto().equals(Puesto.DIGITAL_ALLY_MANAGER)) ) {
          empleadosAEnviar.add(e);
          logger.log(Level.INFO, "Notificación al empleado: " + e.getId());
        }
      }

      // Obtenemos un parámetro de configuracion.
      List<Parameter> parameters = servicio.findWhere(Parameter.class, "nombre", "mail_application_url");
      if (parameters.size() == 1) {
        url = parameters.get(0).getValor();
      }

      // elaborar el mensaje
      subject = C_NOTIFICACION + "Recurso por debajo del stock de seguridad. ";
      body.append("Se dispone de " + r.getCantidad() + " " + res.getString("unidad." + r.getUnidad()).toLowerCase()
          + " del recurso '" + r.getNombre() + "', pero el stock de seguridad es " + r.getStockSeguridad()
          + ".<br /><br/ > <br/><br/><b>Enviado por XEA</b><br/ >" + "<img src='" + url + "/images/logo.png' />");

      if (empleadosAEnviar.size() > 0) senderAddress = getSenderAddress(servicio);
      sendMessage = sendMessage(senderAddress, empleadosAEnviar.get(0).getEmail(), ccAddress, bccAddress, subject,
          isHTMLFormat, body, urlEnviar, null, getMailserverParamSession(servicio));

      if (sendMessage) {
        // notificar el envío de correos
        MailNotification m = new MailNotification();
        m.setFechaEnvio(new Date());
        m.setTipo(MailNotification.TipoElementoNotificado.RECURSO);
        m.setRecursoAsociado(r);
        m.setDetallesEnvio("Recurso bajo minimo.");
        m.getUsuarios().addAll(empleadosAEnviar);
        servicio.persist(m);
      }
    } catch (Exception e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    return sendMessage;
  }

  
  
  /**
   * Notificar preregistro de usuario.
   * 
   * @param servicio
   *			servicio
   * @param usuario
   * 			usuario           
   * @return true, if successful
   */
  public boolean notificarPreregistroUsuario(T servicio, Usuario usuario) {
	  
	  boolean sendMessage = false;

	  boolean isHTMLFormat = true;
	  String url = "";
	  String senderAddress = "", urlEnviar = "";
	  String toAddress = "", ccAddress = "", bccAddress = "", subject = "";
      StringBuffer body = new StringBuffer();
      
      Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
      DateFormat df = new SimpleDateFormat("dd/MM/yyyy HH:mm");
	  
	  try {
	      // Se obtiene el remitente 
	      senderAddress = getSenderAddress(servicio);
	      
		  // Se localizan los areamanger del centro al que pretende suscribirse el usuario
	      List<Empleado> empleadosAEnviar = new ArrayList<Empleado>();
	      List<Empleado> empleados = servicio.findAll(Empleado.class);

	      for (Empleado e : empleados) {	    	  
	        if ((e.getCentro().getId() == usuario.getCentro().getId()) && (e.getPuesto().equals(Puesto.AREA_MANAGER) ||
	        		e.getPuesto().equals(Role.ROLE_DIGITAL_ALLY)) ) {
	        	if (e.getEmail() != null){
	        		empleadosAEnviar.add(e);
	        		toAddress += e.getEmail() + ",";
		        	logger.log(Level.INFO, "Notificación de nuevo preregistro de usuario al empleado: " + e.getId());
	        	}
	        }
	      }
	      
	      // Obtenemos la ruta de la aplicación para poder referenciar la imagen del logotipo de la firma del email.
	      List<Parameter> parameters = servicio.findWhere(Parameter.class, "nombre", "mail_application_url");
	      if (parameters.size() == 1) {	    	
	        url = parameters.get(0).getValor();
	      }
	      
	      // Se elabora el mensaje
	      
	      subject = C_NOTIFICACION + "Solicitud de registro de Usuario.";
	      
	      List<Language> languages = servicio.findAllOrder(Language.class, "id", false);
	      for (Language l : languages) {
	    	  
	    	  Locale languageLocale = new Locale(l.getISO().split("_")[0], l.getISO().split("_")[1]);
	    	  
	    	  body.append("<br/>" +
	    			"<strong>" + l.getName() + "</strong>" + "<br/><br/>" +
	    			      			 
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.ElCiudadano") + " " + usuario.getNombreCompleto()  + " " +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.txt1") + " " +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EnElCentro") + " " +  usuario.getCentro().getNombre() + " " +
	    			"<br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.IdentificadorUsuario") + " " + usuario.getId() + " " + "(<a href='" + url + "/spring/usuarios/usuariopre?id=" + usuario.getId() + "'/>" + "Ir a solicitud" + "</a>)" +"<br/>" +
	    			"<br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.InfContacto") + "<br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Nombre") + " " + usuario.getNombreCompleto() + "<br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Telefono") + " " + usuario.getTelefonos() + "<br/>" +
	       			"<br/><br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.RealizarCualquierGestion") +
	    			"<br/><br/>" +
	    			"<b>" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EnviadoPorISIS") + "</b>" +
	    			"<br/><br/>" +
	    	  		//"<img src='" + url + "/images/logoXunta.png' />" +
	    			"<img src='cid:imagenFirma' />" +
	    			"<br/><br/><br/><br/>"	    		
	    	  );
	      }
	      
	      String pathImagenFirma = ((ServletRequestAttributes) RequestContextHolder.currentRequestAttributes()).getRequest().getRealPath("/images/logoXunta.png");
	      
	      // Se envía el email
	      sendMessage = sendMessage(senderAddress, toAddress, ccAddress, bccAddress, subject,
	    		  isHTMLFormat, body, pathImagenFirma, getMailserverParamSession(servicio));
	      
	      // Si el email ha sisdo enviado correctamente, se registra el envío de la notificación
	      /*if (sendMessage) {
	          Notification n = new Notification();
	          n.setChanel(NotificationChanel.EMAIL);
	          n.setHtmlMessage(true);
	          n.setNotificationDate(cal.getTime());
	          n.setNotificationIdentifier(df.format(cal.getTime()));
	          n.setNotificationType("SolicitudPreregistroUsuario");
	          n.setSubject(subject);
	          
	          List<Usuario> usuariosAEnviar = new ArrayList<Usuario>();
	          for (Empleado e : empleadosAEnviar)
	        	  usuariosAEnviar.add(e);
	          
	          n.setRecipients(usuariosAEnviar);	          
	          servicio.persist(n);
	        }*/
	      
	  } catch (Exception e){
		  // TODO Auto-generated catch block
		  e.printStackTrace();
	  }

	  return sendMessage;
  }
   /**
   * Notificar registro de usuario.
   * 
   * @param servicio
   *			servicio
   * @param usuarioCyL
   * 			usuarioCyL
   * @return true, if successful
   */
  public boolean notificarRegistroUsuario(T servicio, UsuarioCyL usuarioCyl) {
	  boolean sendMessage = false;

	  boolean isHTMLFormat = true;
	  //String url = "";
	  String senderAddress = ""/*, urlEnviar = ""*/;
	  String toAddress = "", ccAddress = "", bccAddress = "", subject = "";
      StringBuffer body = new StringBuffer();
      
      //Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
      //DateFormat df = new SimpleDateFormat("dd/MM/yyyy HH:mm");
	  
	  try {
	      // Se obtiene el remitente 
	      senderAddress = getSenderAddress(servicio);
      
		  // Se localizan los areamanger del centro al que pretende suscribirse el usuario
	      //List<UsuarioCyL> usuariosCylAEnviar = new ArrayList<UsuarioCyL>();
	      //List<Empleado> empleadosAEnviar = new ArrayList<Empleado>();
	      
	      List<Empleado> empleados = servicio.findAll(Empleado.class);

	      for (Empleado e : empleados) {
	    	  if ((e.getCentro().getId() == usuarioCyl.getCentro().getId()) && (e.getPuesto().equals(Puesto.AREA_MANAGER) ||
	    			  e.getPuesto().equals(Puesto.DIGITAL_ALLY_MANAGER))) {
	    		  if (e.getEmail() != null){
	    			  //empleadosAEnviar.add(e);
	    			  bccAddress += e.getEmail() + ",";

	    			  logger.log(Level.INFO, "Notificación de registro de usuario al agente TIC: " + e.getId() + " - " + e.getNombreCompleto());
	    		  }
	    	  }
	      }
	      
	      //usuariosCylAEnviar.add(usuarioCyl);
	      toAddress = usuarioCyl.getEmail();

	      // Obtenemos la ruta de la aplicación para poder referenciar la imagen del logotipo de la firma del email.
	      /*
	      List<Parameter> parameters = servicio.findWhere(Parameter.class, "nombre", "mail_application_url");
	      if (parameters.size() == 1) {	    	
	        url = parameters.get(0).getValor();
	      }
	      */
	      
	      // Se elabora el mensaje
	      subject = C_NOTIFICACION + "Registro de usuario.";
	      
	      List<Language> languages = servicio.findAllOrder(Language.class, "id", false);
	      for (Language l : languages) {
	    	  
	    	  Locale languageLocale = new Locale(l.getISO().split("_")[0], l.getISO().split("_")[1]);
	    	  
	    	  body.append("<br/>" +
	    			"<strong>" + l.getName() + "</strong>" + "<br/><br/>" +
					ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.txt15") + " " +
	    			"<br/><br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.txt12") + "<br/><br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Nombre") + " " + usuarioCyl.getNombreCompleto() + "<br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.NIF") + " " + usuarioCyl.getNif() + "<br/>");
	    			
    			if (null != usuarioCyl.getNifAutorizado() && !usuarioCyl.getNifAutorizado().isEmpty())
    				body.append(
    						ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.NIFAutorizado") + " " + usuarioCyl.getNifAutorizado() + "<br/>");
	    			
    			body.append( 			
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Telefono") + " " + usuarioCyl.getTelefonos() + "<br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Email") + " " + usuarioCyl.getEmail() + "<br/><br/>" +
	    			"<br/><br/>" +
	    			"<br/><u>"+ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.txt16") + "</u><br/><br/>" +	    			
	    			"&emsp;" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.NombreUsuario") + ": <b>" + usuarioCyl.getUsername() + "</b><br/>" +
	    			"&emsp;" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Contrasena") + ": <b>" + usuarioCyl.getPassword() + "</b><br/>" +
	       			"<br/><br/>" +
	    			"<br/><br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.txt17") +
	    			"<br/><br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.txt18") +
	    			"<br/><br/>" +
	    			"<b>" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EnviadoPorISIS") + "</b>" +
	    			"<br/><br/>" +
	    			"<img src='cid:imagenFirma' />" +
	    			"<br/><br/><br/><br/>"	    		
	    	  );
	      }	      
	      
	      String pathImagenFirma = ((ServletRequestAttributes) RequestContextHolder.currentRequestAttributes()).getRequest().getRealPath("/images/logoXunta.png");
	            
	      // Se envía el email
	      sendMessage = sendMessage(senderAddress, toAddress, ccAddress, bccAddress, subject, isHTMLFormat,
	    		  body, pathImagenFirma, getMailserverParamSession(servicio));
	      
	      // Si el email ha sisdo enviado correctamente, se registra el envío de la notificación
	      /*if (sendMessage) {
	          Notification n = new Notification();
	          n.setChanel(NotificationChanel.EMAIL);
	          n.setHtmlMessage(true);
	          n.setNotificationDate(cal.getTime());
	          n.setNotificationIdentifier(df.format(cal.getTime()));
	          n.setNotificationType("RegistroUsuario");	   
	          n.setSubject(subject);
	         
	          List<Usuario> usuariosAEnviar = new ArrayList<Usuario>();
	          for (UsuarioCyL u : usuariosCylAEnviar)
	        	  usuariosAEnviar.add(u);
	          
	          for(Empleado e : empleadosAEnviar)
	        	  usuariosAEnviar.add(e);	          
	          
	          n.setRecipients(usuariosAEnviar);
	          servicio.persist(n);
	        }*/
	      
	  } catch (Exception e){
		  // TODO Auto-generated catch block
		  e.printStackTrace();
	  }

	  return sendMessage;
  }

  /**
   * Notificar restablecimiento de contraseña de usuario.
   * 
   * @param servicio
   *			servicio
   * @param usuarioCyL
   * 			usuarioCyL
   * @return true, if successful
   */
  public boolean notificarResetPassword(T servicio, UsuarioCyL usuarioCyl, String password) {
	  boolean sendMessage = false;

	  boolean isHTMLFormat = true;
	  String url = "";
	  String senderAddress = "";
	  String toAddress = "", subject = "";
      StringBuffer body = new StringBuffer();
      
      Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
      DateFormat df = new SimpleDateFormat("dd/MM/yyyy HH:mm");
	  
	  try {
	      // Se obtiene el remitente 
	      senderAddress = getSenderAddress(servicio);
      
	      //Se establece el destinatario
	      toAddress = usuarioCyl.getEmail();

	      // Obtenemos la ruta de la aplicación para poder referenciar la imagen del logotipo de la firma del email.
	      List<Parameter> parameters = servicio.findWhere(Parameter.class, "nombre", "mail_application_url");
	      if (parameters.size() == 1)	    	
	    	  url = parameters.get(0).getValor();
	      
	      // Se elabora el mensaje
	      subject = C_NOTIFICACION;
	      
	      List<Language> languages = servicio.findAllOrder(Language.class, "id", false);
	      for (Language l : languages) {
	    	  
	    	  Locale languageLocale = new Locale(l.getISO().split("_")[0], l.getISO().split("_")[1]);
	    	  ResourceBundle bundle = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale);
	    	  
	    	  subject += " | " + bundle.getString("password.reset.mail.subject");
	    	  
	    	  body.append("<br/>" +
	    			"<strong>" + l.getName() + "</strong>" + "<br/><br/>" +
	    			
	    			bundle.getString("password.mail.estimated") + " " + usuarioCyl.getNombre() + ".<br/><br/>" +
	    			bundle.getString("password.reset.mail.requestServed") + "<br/><br/><br/>" +
	    			bundle.getString("password.mail.newPassword") + ": <b>" + password + "</b><br/><br/>" +
	    			bundle.getString("password.mail.login") + ": <b>" + usuarioCyl.getUsername() + "</b><br/><br/>" +
	    			bundle.getString("password.reset.mail.modifyPassword") + "<br/><br/>" +
	    			
	    			//bundle.getString("mail.texto.RealizarCualquierGestion") + "<br/><br/>" +
	    			"<b>" + bundle.getString("mail.texto.EnviadoPorISIS") + "</b> <br/><br/>" +
	    	  		//"<img src='" + url + "/images/logoXunta.png' />" +
	    			"<img src='cid:imagenFirma' />" +
	    	  		"<br/><br/><br/><br/>"
	    	  );
	      }	      
	      
	      String pathImagenFirma = ((ServletRequestAttributes) RequestContextHolder.currentRequestAttributes()).getRequest().getRealPath("/images/logoXunta.png");
	      
	      // Se envía el email
	      sendMessage = sendMessage(senderAddress, toAddress, "", "", subject, isHTMLFormat, body, pathImagenFirma,
	    		  					getMailserverParamSession(servicio));
	  } catch (Exception e){
		  // TODO Auto-generated catch block
		  e.printStackTrace();
	  }

	  return sendMessage;
  }

  /**
   * Notificar cambio de contraseña de usuario.
   * 
   * @param servicio
   *			servicio
   * @param usuarioCyL
   * 			usuarioCyL
   * @return true, if successful
   */
  public boolean notificarChangePassword(T servicio, UsuarioCyL usuarioCyl, String password) {
	  boolean sendMessage = false;

	  boolean isHTMLFormat = true;
	  String url = "";
	  String senderAddress = "";
	  String toAddress = "", subject = "";
      StringBuffer body = new StringBuffer();
      
      Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
      DateFormat df = new SimpleDateFormat("dd/MM/yyyy HH:mm");
	  
	  try {
	      // Se obtiene el remitente 
	      senderAddress = getSenderAddress(servicio);
      
	      //Se establece el destinatario
	      toAddress = usuarioCyl.getEmail();

	      // Obtenemos la ruta de la aplicación para poder referenciar la imagen del logotipo de la firma del email.
	      List<Parameter> parameters = servicio.findWhere(Parameter.class, "nombre", "mail_application_url");
	      if (parameters.size() == 1)	    	
	    	  url = parameters.get(0).getValor();
	      
	      // Se elabora el mensaje
	      subject = C_NOTIFICACION;
	      
	      List<Language> languages = servicio.findAllOrder(Language.class, "id", false);
	      for (Language l : languages) {
	    	  
	    	  Locale languageLocale = new Locale(l.getISO().split("_")[0], l.getISO().split("_")[1]);
	    	  ResourceBundle bundle = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale);
	    	  
	    	  subject += " | " + bundle.getString("password.change.mail.subject");
	    	  
	    	  body.append("<br/>" +
	    			"<strong>" + l.getName() + "</strong><br/><br/>" +
	    			
	    			bundle.getString("password.mail.estimated") + " " + usuarioCyl.getNombre() + ".<br/><br/>" +
	    			bundle.getString("password.change.mail.requestServed") + "<br/><br/>" +
	    			bundle.getString("password.mail.newPassword") + ": <strong>" + password + "</strong><br/><br/>" +
	    			bundle.getString("password.mail.login") + ": <b>" + usuarioCyl.getUsername() + "</b><br/><br/>" +
	    			bundle.getString("password.change.mail.modifyPassword") + "<br/><br/>" +
	    			"<strong>" + bundle.getString("mail.texto.EnviadoPorISIS") + "</strong><br/><br/>" +
	    			"<img src='cid:imagenFirma' />" +
	    	  		"<br/><br/><br/><br/>"
	    	  );
	      }	      
	      
	      String pathImagenFirma = ((ServletRequestAttributes) RequestContextHolder.currentRequestAttributes()).getRequest().getRealPath("/images/logoXunta.png");
	     	      
	      // Se envía el email
	      sendMessage = sendMessage(senderAddress, toAddress, "", "", subject, isHTMLFormat, body, pathImagenFirma,
	    		  					getMailserverParamSession(servicio));
	  } catch (Exception e){
		  // TODO Auto-generated catch block
		  e.printStackTrace();
	  }

	  return sendMessage;
  }
  
  //3.4.8 - RF_15_38
  
  /**
   * Notificar cambio de contraseña de usuario personal.
   * 
   * @param servicio
   *			servicio
   * @param usuarioCyL
   * 			usuarioCyL
   * @return true, if successful
   */
  public boolean notificarChangePassword(T servicio, Empleado personal, String password) {
	  boolean sendMessage = false;

	  boolean isHTMLFormat = true;
	  String url = "";
	  String senderAddress = "";
	  String toAddress = "", subject = "";
      StringBuffer body = new StringBuffer();
      
      Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
      DateFormat df = new SimpleDateFormat("dd/MM/yyyy HH:mm");
	  
	  try {
	      // Se obtiene el remitente 
	      senderAddress = getSenderAddress(servicio);
      
	      //Se establece el destinatario
	      toAddress = personal.getEmail();

	      // Obtenemos la ruta de la aplicación para poder referenciar la imagen del logotipo de la firma del email.
	      List<Parameter> parameters = servicio.findWhere(Parameter.class, "nombre", "mail_application_url");
	      if (parameters.size() == 1)	    	
	    	  url = parameters.get(0).getValor();
	      
	      // Se elabora el mensaje
	      subject = C_NOTIFICACION;
	      
	      List<Language> languages = servicio.findAllOrder(Language.class, "id", false);
	      for (Language l : languages) {
	    	  
	    	  Locale languageLocale = new Locale(l.getISO().split("_")[0], l.getISO().split("_")[1]);
	    	  ResourceBundle bundle = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale);
	    	  
	    	  subject += " | " + bundle.getString("password.change.mail.subject");
	    	  
	    	  body.append("<br/>" +
		    			"<strong>" + l.getName() + "</strong><br/><br/>" +
		    			
		    			bundle.getString("password.mail.estimated") + " " + personal.getNombre() + ".<br/><br/>" +
		    			bundle.getString("password.change.mail.requestServed") + "<br/><br/>" +
		    			bundle.getString("password.mail.newPassword") + ": <b>" + password + "</b><br/><br/>" +
		    			bundle.getString("password.mail.login") + ": <b>" + personal.getUsername() + "</b><br/><br/>" +
		    			bundle.getString("password.change.mail.modifyPassword") + "<br/><br/>" +
		    			"<strong>" + bundle.getString("mail.texto.EnviadoPorISIS") + "</strong><br/><br/>" +
		    			"<img src='cid:imagenFirma' />" +
		    	  		"<br/><br/><br/><br/>"
		    );
	      }	      
	      
	      String pathImagenFirma = ((ServletRequestAttributes) RequestContextHolder.currentRequestAttributes()).getRequest().getRealPath("/images/logoXunta.png");
	      
	      // Se envía el email
	      sendMessage = sendMessage(senderAddress, toAddress, "", "", subject, isHTMLFormat, body, pathImagenFirma,
	    		  					getMailserverParamSession(servicio));
	  } catch (Exception e){
		  // TODO Auto-generated catch block
		  e.printStackTrace();
	  }

	  return sendMessage;
  }
  
   /**
   * Notificar prematricula del curso de un usuario.
   * 
   * @param usuario
   *           usuario
   * @param matriculacurso
   * 			matriculacurso
   * @param servicio
   * 			servicio          
   * @return true, if successful
   */
  public boolean notificarPreMatriculaCurso(T servicio, MatriculaCurso matriculaCurso, Usuario usuario) {
	  boolean sendMessage = false;

	  boolean isHTMLFormat = true;
	  String senderAddress = "";
	  String toAddress = "", ccAddress = "", bccAddress = "", subject = "";
      StringBuffer body = new StringBuffer();

	  try {
	      // Se obtiene el remitente 
	      senderAddress = getSenderAddress(servicio);
      
		  // Se localizan los areamanger del centro al que pretende suscribirse el usuario
	      List<UsuarioCyL> usuariosCylAEnviar = new ArrayList<UsuarioCyL>();
	      List<Empleado> empleadosAEnviar = new ArrayList<Empleado>();
	      List<Empleado> empleados = servicio.findAll(Empleado.class);

	      for (Empleado e : empleados) {	    	 
	    	  if ((e.getCentro().getId() == matriculaCurso.getCurso().getCentro().getId()) && (e.getPuesto().equals(Puesto.AREA_MANAGER) || 
	    			  e.getPuesto().equals(Puesto.DIGITAL_ALLY_MANAGER))) {
	    		  if (e.getEmail() != null){
	    			  empleadosAEnviar.add(e);
	    			  bccAddress += e.getEmail() + ",";
	    			  logger.log(Level.INFO, "Notificación de una solicitud de matrícula curso de usuario al empleado: " + e.getId());
	    		  }
	        }
	      }
	      
	      UsuarioCyL u = (UsuarioCyL) servicio.find(UsuarioCyL.class, matriculaCurso.getUsuario().getId());
	      
	      usuariosCylAEnviar.add(u);
	      toAddress = u.getEmail();

	      // Obtenemos la ruta de la aplicación para poder referenciar la imagen del logotipo de la firma del email.
	      //List<Parameter> parameters = servicio.findWhere(Parameter.class, "nombre", "mail_application_url");
	      //if (parameters.size() == 1) {	    	
	      //	url = parameters.get(0).getValor();
	      //}	      
	     
	      // Se elabora el mensaje

	      subject = C_NOTIFICACION + "Solicitud de matrícula en el curso con identificador " + matriculaCurso.getCurso().getId();
	      String separador = ": ";

	      List<Language> languages = servicio.findAllOrder(Language.class, "id", false);
	      for (Language l : languages) {

	    	  Locale languageLocale = new Locale(l.getISO().split("_")[0], l.getISO().split("_")[1]);
	    	  CursoLanguage cl = getNomenclatura(matriculaCurso.getCurso(), l.getISO());

	    	  subject += separador + cl.getNombre();
	    	  separador = " / ";

	    	  String estado = "<span style=\"color: ";
		      if (matriculaCurso.isAdmitido() && !matriculaCurso.isEnListaEspera())
		    	  estado += "#00CC00\">" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Matriculado");
		      else if (!matriculaCurso.isAdmitido() && !matriculaCurso.isEnListaEspera() && !matriculaCurso.isCancelada())
		    	  estado += "#CC0000\">" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Rechazado");
		      else if (!matriculaCurso.isAdmitido() && !matriculaCurso.isEnListaEspera() && matriculaCurso.isCancelada())
		    	  estado += "#CC0000\">" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Cancelada");
		      else if (!matriculaCurso.isAdmitido() && matriculaCurso.isEnListaEspera() && matriculaCurso.isPreMatriculadoWeb())
		    	  estado += "#FF9000\">" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Solicitado");
		      else
		    	  estado = "<span>";
		      estado += "</span>";
	    	  
	    	  body.append("<br/>" +
	    			"<strong>" + l.getName() + "</strong>" + "<br/><br/>" +

					ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.txt19") + " " + cl.getNombre() + " " +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.QueSeImpartira") + " " + cl.getCurso().getFechasLegibles() + " " +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.ConHorario") + " " + cl.getCurso().getHorarioLargo() + " " +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EnElCentro") + " " + matriculaCurso.getCurso().getCentro().getNombre() + " " +
	    			"- " + matriculaCurso.getCurso().getCentro().getDireccion() + "." +
	    			"<br/><br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EstadoMatricula") + " <b>" + estado + "</b>" +
	    			"<br/><br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.InfContacto") + "<br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Nombre") + " " + usuario.getNombreCompleto() + "<br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Telefono") + " " + usuario.getTelefonos() + "<br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Email") + " " + u.getEmail() + "<br/>" +
	       			"<br/><br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.txt20") +
	    			"<br/><br/>" +
	    			"<b>" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EnviadoPorISIS") + "</b>" +
	    			"<br/><br/>" +
	    			"<img src='cid:imagenFirma' />" +
	    			"<br/><br/><br/><br/>"	    		
	    	  );
	      }
	      
	      String pathImagenFirma = ((ServletRequestAttributes) RequestContextHolder.currentRequestAttributes()).getRequest().getRealPath("/images/logoXunta.png");
	      
	      // Se envía el email
	      sendMessage = sendMessage(senderAddress, toAddress, ccAddress, bccAddress, subject,
	    		  isHTMLFormat, body, pathImagenFirma, getMailserverParamSession(servicio));
	      
	      // Si el email ha sisdo enviado correctamente, se registra el envío de la notificación
	      /*if (sendMessage) {
	          Notification n = new Notification();
	          n.setChanel(NotificationChanel.EMAIL);
	          n.setHtmlMessage(true);
	          n.setNotificationDate(cal.getTime());
	          n.setNotificationIdentifier(df.format(cal.getTime()));
	          n.setNotificationType("SolicitudPreMatriculaCurso");
	          n.setSubject(subject);
	          
	          List<Usuario> usuariosAEnviar = new ArrayList<Usuario>();
	          
	          for (UsuarioCyL uCyL : usuariosCylAEnviar)
	        	  usuariosAEnviar.add(uCyL);
	          
	          for (Empleado e : empleadosAEnviar)
	        	  usuariosAEnviar.add(e);	        	  		         
	          
	          n.setRecipients(usuariosAEnviar);
	          servicio.persist(n);
	        }*/
	      
	  } catch (Exception e){
		  // TODO Auto-generated catch block
		  e.printStackTrace();
	  }

	  return sendMessage;
  }
  /**
   * Notificar cancelación prematricula del curso de un usuario. 
   * 
   * @param idUsuario
   *           idUsuario
   * @param servicio
   * 			servicio          
   * @return true, if successful
   */
  public boolean notificarCancelacionPreMatriculaCurso(T servicio, Long idUsuario, MatriculaCurso matriculaCurso) {
	  
	  boolean sendMessage = false;

	  boolean isHTMLFormat = true;
	  String url = "";
	  String senderAddress = "", urlEnviar = "";
	  String toAddress = "", ccAddress = "", bccAddress = "", subject = "";
      StringBuffer body = new StringBuffer();
      
      Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
      DateFormat df = new SimpleDateFormat("dd/MM/yyyy HH:mm");

      Usuario usuario = (Usuario) servicio.find(Usuario.class, idUsuario);
      
	  try {
	      // Se obtiene el remitente 
	      senderAddress = getSenderAddress(servicio);
      
		  // Se localizan los areamanger del centro al que pretende suscribirse el usuario
	      List<UsuarioCyL> usuariosCylAEnviar = new ArrayList<UsuarioCyL>();
	      List<Empleado> empleadosAEnviar = new ArrayList<Empleado>();
	      List<Empleado> empleados = servicio.findAll(Empleado.class);

	      for (Empleado e : empleados) {
	        if ((e.getCentro().getId() == matriculaCurso.getCurso().getCentro().getId()) && (e.getPuesto().equals(Puesto.AREA_MANAGER) ||
	        		e.getPuesto().equals(Puesto.DIGITAL_ALLY_MANAGER))) {
	        	if (e.getEmail() != null){
	        		empleadosAEnviar.add(e);
		        	bccAddress += e.getEmail() + ",";
		        	logger.log(Level.INFO, "Notificación de una cancelación de solicitud de matrícula curso de usuario al empleado: " + e.getId());
	        	}
	        }
	      }      
	      
	      UsuarioCyL u = (UsuarioCyL) servicio.find(UsuarioCyL.class, idUsuario);
	      
	      usuariosCylAEnviar.add(u);	      
	      toAddress =  u.getEmail();

	      // Obtenemos la ruta de la aplicación para poder referenciar la imagen del logotipo de la firma del email.
	      List<Parameter> parameters = servicio.findWhere(Parameter.class, "nombre", "mail_application_url");
	      if (parameters.size() == 1) {	    	
	        url = parameters.get(0).getValor();
	      }	      
	   
	      // Se elabora el mensaje
	      subject = C_NOTIFICACION + "Cancelación de la solicitud de matrícula en el curso con identificador " + matriculaCurso.getCurso().getId();
	      String separador = ": ";
	      
	      List<Language> languages = servicio.findAllOrder(Language.class, "id", false);
	      for (Language l : languages) {
	    	  
	    	  Locale languageLocale = new Locale(l.getISO().split("_")[0], l.getISO().split("_")[1]);
	    	  CursoLanguage cl = getNomenclatura(matriculaCurso.getCurso(), l.getISO());
	    	  
	    	  subject += separador + cl.getNombre();
	    	  separador = " / ";
	    	  
	    	  body.append("<br/>" +
	    			"<strong>" + l.getName() + "</strong>" + "<br/><br/>" +
	    			      			 
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Don") + " " + usuario.getNombreCompleto() + " " +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.txt3") + " " + cl.getNombre() + "&#160;" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EnElCentro") + " " + matriculaCurso.getCurso().getCentro().getNombre() + " " +
	    			"<br/><br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.InfContacto") + "<br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Nombre") + " " + usuario.getNombreCompleto() + "<br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Telefono") + " " + usuario.getTelefonos() + "<br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Email") + " " + u.getEmail() + "<br/>" +
	    			"<br/><br/>" +
	    			"<b>" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EnviadoPorISIS") + "</b>" +
	    			"<br/><br/>" +
	    			"<img src='cid:imagenFirma' />" +
	    			"<br/><br/><br/><br/>"	    	  		
	    	  );
	      }
	      
	      String pathImagenFirma = ((ServletRequestAttributes) RequestContextHolder.currentRequestAttributes()).getRequest().getRealPath("/images/logoXunta.png");
	      
	      // Se envía el email
	      sendMessage = sendMessage(senderAddress, toAddress, ccAddress, bccAddress, subject,
	    		  isHTMLFormat, body, pathImagenFirma, getMailserverParamSession(servicio));
	      
	      // Si el email ha sisdo enviado correctamente, se registra el envío de la notificación
	     /* if (sendMessage) {
	          Notification n = new Notification();
	          n.setChanel(NotificationChanel.EMAIL);
	          n.setHtmlMessage(true);
	          n.setNotificationDate(cal.getTime());
	          n.setNotificationIdentifier(df.format(cal.getTime()));
	          n.setNotificationType("CancelacionSolicitudPreMatriculaCurso");
	          n.setSubject(subject);
	          
	          List<Usuario> usuariosAEnviar = new ArrayList<Usuario>();
	          
	          for (UsuarioCyL uCyL : usuariosCylAEnviar)
	        	  usuariosAEnviar.add(uCyL);
	          
	          for (Empleado e : empleadosAEnviar)
	        	  usuariosAEnviar.add(e);
	          
	          n.setRecipients(usuariosAEnviar);
	          
	          servicio.persist(n);
	        }*/
	      
	  } catch (Exception e){
		  // TODO Auto-generated catch block
		  e.printStackTrace();
	  }

	  return sendMessage;
  }  
   /**
   * Notificar prematricula de la charla de un usuario.
   * 
   * @param usuario
   *           usuario
   * @param matriculacharla
   * 			matriculacharla
   * @param servicio
   * 			servicio          
   * @return true, if successful
   */
  public boolean notificarPreMatriculaCharla(T servicio, MatriculaCharla matriculaCharla, Usuario usuario) {
	  
	  boolean sendMessage = false;

	  boolean isHTMLFormat = true;
	  String senderAddress = "";
	  String toAddress = "", ccAddress = "", bccAddress = "", subject = "";
      StringBuffer body = new StringBuffer();
      
      Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
      DateFormat df = new SimpleDateFormat("dd/MM/yyyy HH:mm");

	  try {
	      // Se obtiene el remitente 
	      senderAddress = getSenderAddress(servicio);
      
		  // Se localizan los areamanger del centro al que pretende suscribirse el usuario
	      List<UsuarioCyL> usuariosCylAEnviar = new ArrayList<UsuarioCyL>();
	      List<Empleado> empleadosAEnviar = new ArrayList<Empleado>();
	      List<Empleado> empleados = servicio.findAll(Empleado.class);

	      for (Empleado e : empleados) {
	        if ((e.getCentro().getId() == matriculaCharla.getCharla().getCentro().getId()) && (e.getPuesto().equals(Puesto.AREA_MANAGER) ||
	        		e.getPuesto().equals(Puesto.DIGITAL_ALLY_MANAGER))) {
	        	if (e.getEmail() != null){
	        		empleadosAEnviar.add(e);
		        	bccAddress += e.getEmail() + ",";
		        	logger.log(Level.INFO, "Notificación de una solicitud matrícula charla de usuario al empleado: " + e.getId());
	        	}
	        }
	      }
	      
	      UsuarioCyL u = (UsuarioCyL) servicio.find(UsuarioCyL.class, matriculaCharla.getUsuario().getId());
	      
	      usuariosCylAEnviar.add(u);	
	      toAddress = u.getEmail();

	      // Obtenemos la ruta de la aplicación para poder referenciar la imagen del logotipo de la firma del email.
	      //List<Parameter> parameters = servicio.findWhere(Parameter.class, "nombre", "mail_application_url");
	      //if (parameters.size() == 1) {	    	
	      //  url = parameters.get(0).getValor();
	      //}
	      
	      // Se elabora el mensaje
	      subject = C_NOTIFICACION + "Solicitud de matrícula en la actividad con identificador " + matriculaCharla.getCharla().getId();
	      String separador = ": ";
	      
	      List<Language> languages = servicio.findAllOrder(Language.class, "id", false);
	      for (Language l : languages) {
	    	  
	    	  Locale languageLocale = new Locale(l.getISO().split("_")[0], l.getISO().split("_")[1]);
	    	  CharlaLanguage cl = getNomenclatura(matriculaCharla.getCharla(), l.getISO());
	    	  
	    	  subject += separador + cl.getNombre();
	    	  separador = " / ";
	    	  
	    	  String estado = "<span style=\"color: ";
		      if (matriculaCharla.isAdmitido() && !matriculaCharla.isEnListaEspera())
		    	  estado += "#00CC00\">" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Matriculado");
		      else if (!matriculaCharla.isAdmitido() && !matriculaCharla.isEnListaEspera() && !matriculaCharla.isCancelada())
		    	  estado += "#CC0000\">" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Rechazado");
		      else if (!matriculaCharla.isAdmitido() && !matriculaCharla.isEnListaEspera() && matriculaCharla.isCancelada())
		    	  estado += "#CC0000\">" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Cancelada");
		      else if (!matriculaCharla.isAdmitido() && matriculaCharla.isEnListaEspera() && matriculaCharla.isPreMatriculadoWeb())
		    	  estado += "#FF9000\">" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Solicitado");
		      else
		    	  estado = "<span>";
		      estado += "</span>";
	    	  
	    	  body.append("<br/>" +
	    			"<strong>" + l.getName() + "</strong>" + "<br/><br/>" +
	    			      			 
	    			
					ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.txt19") + " " + cl.getNombre() + " " +
//	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.ElUsuario") + " " + usuario.getNombreCompleto() + " " +
//	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.txt4") + " " + cl.getNombre() + "&#160;" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.QueSeImpartira") + " " + cl.getCharla().getFechaLegible() + " " +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.ConHorario") + " " + cl.getCharla().getHorario() + " " +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EnElCentro") + " " + matriculaCharla.getCharla().getCentro().getNombre() + " " +
	    			" (" + matriculaCharla.getCharla().getCentro().getDireccion() + ") " +
	    			"<br/><br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EstadoMatricula") + " <b>" + estado + "</b>" +
	    			"<br/><br/>" +
	    			/* Eliminado en 3.4.6-R
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.ProximamenteConfirmacion") +
	    			"<br/><br/>" +
	    			*/
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.InfContacto") + "<br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Nombre") + " " + usuario.getNombreCompleto() + "<br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Telefono") + " " + usuario.getTelefonos() + "<br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Email") + " " + u.getEmail() + "<br/>" +
	    			"<br/><br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.txt19") +
	    			"<br/><br/>" +
	    			"<b>" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EnviadoPorISIS") + "</b>" +
	    			"<br/><br/>" +
	    	  		//"<img src='" + url + "/images/logoXunta.png' />" +
	    			"<img src='cid:imagenFirma' />" +
	    			"<br/><br/><br/><br/>"	    	  		
	    	  );
	      }	      
	   
	      String pathImagenFirma = ((ServletRequestAttributes) RequestContextHolder.currentRequestAttributes()).getRequest().getRealPath("/images/logoXunta.png");
	      
	      // Se envía el email
	      sendMessage = sendMessage(senderAddress, toAddress, ccAddress, bccAddress, subject,
	    		  isHTMLFormat, body, pathImagenFirma, getMailserverParamSession(servicio));
	      
	      // Si el email ha sisdo enviado correctamente, se registra el envío de la notificación
	      /*if (sendMessage) {
	          Notification n = new Notification();
	          n.setChanel(NotificationChanel.EMAIL);
	          n.setHtmlMessage(true);
	          n.setNotificationDate(cal.getTime());
	          n.setNotificationIdentifier(df.format(cal.getTime()));
	          n.setNotificationType("SolicitudPreMatriculaCurso");
	          n.setSubject(subject);
	          
	          List<Usuario> usuariosAEnviar = new ArrayList<Usuario>();
	          
	          for (UsuarioCyL uCyL : usuariosCylAEnviar)
	        	  usuariosAEnviar.add(uCyL);
	          
	          for (Empleado e : empleadosAEnviar)
	        	  usuariosAEnviar.add(e);
	          
	          n.setRecipients(usuariosAEnviar);	          
	          servicio.persist(n);
	        }*/
	      
	  } catch (Exception e){
		  // TODO Auto-generated catch block
		  e.printStackTrace();
	  }

	  return sendMessage;
  }
  /**
   * Notificar matricula del curso de un usuario.
   * 
   * @param matriculacurso
   * 			matriculacurso
   * @param servicio
   * 			servicio          
   * @return true, if successful
   */
  public boolean notificarMatriculaCurso(T servicio, MatriculaCurso matriculaCurso) {
	  
	  boolean sendMessage = false;

	  boolean isHTMLFormat = true;
	  String url = "";
	  String senderAddress = "", urlEnviar = "";
	  String toAddress = "", ccAddress = "", bccAddress = "", subject = "";
      StringBuffer body = new StringBuffer();
      
      Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
      DateFormat df = new SimpleDateFormat("dd/MM/yyyy HH:mm");
      
      Usuario usuario = (Usuario) servicio.find(Usuario.class, matriculaCurso.getUsuario().getId());     

	  
	  try {
	      // Se obtiene el remitente 
	      senderAddress = getSenderAddress(servicio);
      
		  // Se localizan los areamanger del centro al que pretende suscribirse el usuario
	      List<UsuarioCyL> usuariosCylAEnviar = new ArrayList<UsuarioCyL>();
	      List<Empleado> empleadosAEnviar = new ArrayList<Empleado>();
	      List<Empleado> empleados = servicio.findAll(Empleado.class);

	      for (Empleado e : empleados) {
	        if ((e.getCentro().getId() == matriculaCurso.getCurso().getCentro().getId()) && (e.getPuesto().equals(Puesto.AREA_MANAGER) ||
	        		e.getPuesto().equals(Puesto.DIGITAL_ALLY_MANAGER))) {
	        	if (e.getEmail() != null){
	        		empleadosAEnviar.add(e);
		        	bccAddress += e.getEmail() + ",";
		        	logger.log(Level.INFO, "Notificación de una nueva matrícula curso de usuario al empleado: " + e.getId());
	        	}
	        }
	      }
	      
	      UsuarioCyL u = (UsuarioCyL) servicio.find(UsuarioCyL.class, matriculaCurso.getUsuario().getId());
	      
	      usuariosCylAEnviar.add(u);	
	      toAddress = u.getEmail();

	      // Obtenemos la ruta de la aplicación para poder referenciar la imagen del logotipo de la firma del email.
	      List<Parameter> parameters = servicio.findWhere(Parameter.class, "nombre", "mail_application_url");
	      if (parameters.size() == 1) {	    	
	        url = parameters.get(0).getValor();
	      }
	      
	      // Se elabora el mensaje      
	      
	      subject = C_NOTIFICACION + "Estado de la matrícula solicitada para el curso con identificador " + matriculaCurso.getCurso().getId();
	      String separador = ": ";
	      
	      List<Language> languages = servicio.findAllOrder(Language.class, "id", false);
	      for (Language l : languages) {
	    	  
	    	  Locale languageLocale = new Locale(l.getISO().split("_")[0], l.getISO().split("_")[1]);
	    	  CursoLanguage cl = getNomenclatura(matriculaCurso.getCurso(), l.getISO());
	    	  
	    	  subject += separador + cl.getNombre();
	    	  separador = " / ";
	    	  
	    	  String estado = "<span style=\"color: ";
		      if (matriculaCurso.isAdmitido() && !matriculaCurso.isEnListaEspera())
		    	  estado += "#00CC00\">" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Matriculado");
		      else if (!matriculaCurso.isAdmitido() && !matriculaCurso.isEnListaEspera() && !matriculaCurso.isCancelada())
		    	  estado += "#CC0000\">" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Rechazado");
		      else if (!matriculaCurso.isAdmitido() && !matriculaCurso.isEnListaEspera() && matriculaCurso.isCancelada())
		    	  estado += "#CC0000\">" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Cancelada");
		      else if (!matriculaCurso.isAdmitido() && matriculaCurso.isEnListaEspera() && matriculaCurso.isPreMatriculadoWeb())
		    	  estado += "#FF9000\">" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Solicitado");
		      else
		    	  estado = "<span>";
		      estado += "</span>";
	    	  
	    	  body.append("<br/>" +
	    			"<strong>" + l.getName() + "</strong>" + "<br/><br/>" +
	    			      			 
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.txt13") + " " + cl.getNombre() + " " +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.QueSeImpartira") + " " + matriculaCurso.getCurso().getFechasLegibles() + " " +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.ConHorario") + " " + matriculaCurso.getCurso().getHorarioLargo() + " " +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EnElCentro") + " " + matriculaCurso.getCurso().getCentro().getNombre() + " " +
	    			"- " + matriculaCurso.getCurso().getCentro().getDireccion() + "." +
	    			"<br/><br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EstadoMatricula") + " <b>" + estado + "</b><br/>" +
	    			"<br/><br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.InfContacto") + "<br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Nombre") + " " + usuario.getNombreCompleto() + "<br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Telefono") + " " + usuario.getTelefonos() + "<br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Email") + " " + u.getEmail() + "<br/>" +
	       			"<br/><br/>" +
	    			"<b>" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EnviadoPorISIS") + "</b>" +
	    			"<br/><br/>" +
	    	  		//"<img src='" + url + "/images/logoXunta.png' />" +
	    			"<img src='cid:imagenFirma' />" +
	    			"<br/><br/><br/><br/>"	    		
	    	  );
	      }	      
	      
	      String pathImagenFirma = ((ServletRequestAttributes) RequestContextHolder.currentRequestAttributes()).getRequest().getRealPath("/images/logoXunta.png");
	      
	      // Se envía el email
	      sendMessage = sendMessage(senderAddress, toAddress, ccAddress, bccAddress, subject,
	    		  isHTMLFormat, body, pathImagenFirma, getMailserverParamSession(servicio));
	      
	      // Si el email ha sisdo enviado correctamente, se registra el envío de la notificación
	     /* if (sendMessage) {
	          Notification n = new Notification();
	          n.setChanel(NotificationChanel.EMAIL);
	          n.setHtmlMessage(true);
	          n.setNotificationDate(cal.getTime());
	          n.setNotificationIdentifier(df.format(cal.getTime()));
	          n.setNotificationType("SolicitudPreMatriculaCurso");
	          n.setSubject(subject);
	          
	          List<Usuario> usuariosAEnviar = new ArrayList<Usuario>();

	          for (UsuarioCyL uCyL : usuariosCylAEnviar)
	        	  usuariosAEnviar.add(uCyL);
	        	  
	          for (Empleado e : empleadosAEnviar)
	        	  usuariosAEnviar.add(e);
	          
	          n.setRecipients(usuariosAEnviar);
	          servicio.persist(n);
	        }*/
	      
	  } catch (Exception e){
		  // TODO Auto-generated catch block
		  e.printStackTrace();
	  }

	  return sendMessage;
  }
   /**
   * Notificar matricula del curso de un usuario.
   * 
   * @param matriculaCharla
   * 			matriculaCharla
   * @param servicio
   * 			servicio          
   * @return true, if successful
   */
  public boolean notificarMatriculaCharla(T servicio, MatriculaCharla matriculaCharla) {
	  
	  boolean sendMessage = false;

	  boolean isHTMLFormat = true;
	  String url = "";
	  String senderAddress = "", urlEnviar = "";
	  String toAddress = "", ccAddress = "", bccAddress = "", subject = "";
      StringBuffer body = new StringBuffer();
      
      Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
      DateFormat df = new SimpleDateFormat("dd/MM/yyyy HH:mm");
      
      Usuario usuario = (Usuario) servicio.find(Usuario.class, matriculaCharla.getUsuario().getId());

	  
	  try {
	      // Se obtiene el remitente 
	      senderAddress = getSenderAddress(servicio);
      
		  // Se localizan los areamanger del centro al que pretende suscribirse el usuario
	      List<UsuarioCyL> usuariosCylAEnviar = new ArrayList<UsuarioCyL>();
	      List<Empleado> empleadosAEnviar = new ArrayList<Empleado>();
	      List<Empleado> empleados = servicio.findAll(Empleado.class);

	      for (Empleado e : empleados) {
	        if ((e.getCentro().getId() == matriculaCharla.getCharla().getCentro().getId()) && (e.getPuesto().equals(Puesto.AREA_MANAGER) ||
	        		e.getPuesto().equals(Puesto.DIGITAL_ALLY_MANAGER))) {
	        	if (e.getEmail() != null){
	        		empleadosAEnviar.add(e);
		        	bccAddress += e.getEmail() + ",";
		        	logger.log(Level.INFO, "Notificación de una nueva matrícula charla de usuario al empleado: " + e.getId());
	        	}
	        }
	      }
	      
	      UsuarioCyL u = (UsuarioCyL) servicio.find(UsuarioCyL.class, matriculaCharla.getUsuario().getId());
	      
	      usuariosCylAEnviar.add(u);	
	      toAddress = u.getEmail();

	      // Obtenemos la ruta de la aplicación para poder referenciar la imagen del logotipo de la firma del email.
	      List<Parameter> parameters = servicio.findWhere(Parameter.class, "nombre", "mail_application_url");
	      if (parameters.size() == 1) {	    	
	        url = parameters.get(0).getValor();
	      }
	      
	      // Se elabora el mensaje
	      
	      subject = C_NOTIFICACION + "Estado de la matrícula solicitada para la actividad con identificador " + matriculaCharla.getCharla().getId();
	      String separador = ": ";
	      
	      List<Language> languages = servicio.findAllOrder(Language.class, "id", false);
	      for (Language l : languages) {
	    	  
	    	  Locale languageLocale = new Locale(l.getISO().split("_")[0], l.getISO().split("_")[1]);
	    	  CharlaLanguage cl = getNomenclatura(matriculaCharla.getCharla(), l.getISO());
	    	  
	    	  subject += separador + cl.getNombre();
	    	  separador = " / ";
	    	  
	    	  String estado = "<span style=\"color: ";
		      if (matriculaCharla.isAdmitido() && !matriculaCharla.isEnListaEspera())
		    	  estado += "#00CC00\">" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Matriculado");
		      else if (!matriculaCharla.isAdmitido() && !matriculaCharla.isEnListaEspera() && !matriculaCharla.isCancelada())
		    	  estado += "#CC0000\">" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Rechazado");
		      else if (!matriculaCharla.isAdmitido() && !matriculaCharla.isEnListaEspera() && matriculaCharla.isCancelada())
		    	  estado += "#CC0000\">" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Cancelada");
		      else if (!matriculaCharla.isAdmitido() && matriculaCharla.isEnListaEspera() && matriculaCharla.isPreMatriculadoWeb())
		    	  estado += "#FF9000\">" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Solicitado");
		      else
		    	  estado = "<span>";
		      estado += "</span>";
	    	  
	    	  body.append("<br/>" +
	    			"<strong>" + l.getName() + "</strong>" + "<br/><br/>" +
	    			      			 
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.txt8") + " " + cl.getNombre() + " " +
	    			
	    			
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.QueSeImpartira") + " " + matriculaCharla.getCharla().getFechaLegible() + " " +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.ConHorario") + " " + matriculaCharla.getCharla().getHorario() + " " +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EnElCentro") + " " + matriculaCharla.getCharla().getCentro().getNombre() + " " +
	    			"( " + matriculaCharla.getCharla().getCentro().getDireccion() + ")." +
	    			"<br/><br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EstadoMatricula") + " <b>" + estado + "</b><br/>" +
	    			"<br/><br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.InfContacto") + "<br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Nombre") + " " + usuario.getNombreCompleto() + "<br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Telefono") + " " + usuario.getTelefonos() + "<br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Email") + " " + u.getEmail() + "<br/>" +
	       			"<br/><br/>" +
	    			"<b>" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EnviadoPorISIS") + "</b>" +
	    			"<br/><br/>" +
	    	  		//"<img src='" + url + "/images/logoXunta.png' />" +
	    			"<img src='cid:imagenFirma' />" +
	    			"<br/><br/><br/><br/>"	    		
	    	  );
	      }	      
	      
	      String pathImagenFirma = ((ServletRequestAttributes) RequestContextHolder.currentRequestAttributes()).getRequest().getRealPath("/images/logoXunta.png");
	      	      
	      // Se envía el email
	      sendMessage = sendMessage(senderAddress, toAddress, ccAddress, bccAddress, subject,
	    		  isHTMLFormat, body, pathImagenFirma, getMailserverParamSession(servicio));
	      
	      // Si el email ha sisdo enviado correctamente, se registra el envío de la notificación
	      /*if (sendMessage) {
	          Notification n = new Notification();
	          n.setChanel(NotificationChanel.EMAIL);
	          n.setHtmlMessage(true);
	          n.setNotificationDate(cal.getTime());
	          n.setNotificationIdentifier(df.format(cal.getTime()));
	          n.setNotificationType("SolicitudPreMatriculaCharla");
	          n.setSubject(subject);
	          
	          List<Usuario> usuariosAEnviar = new ArrayList<Usuario>();
	          
	          for (UsuarioCyL uCyL : usuariosCylAEnviar)
	        	  usuariosAEnviar.add(uCyL);
	          
	          for (Empleado e : empleadosAEnviar)
	        	  usuariosAEnviar.add(e);
	          
	          n.setRecipients(usuariosAEnviar);
	          servicio.persist(n);
	        }*/
	      
	  } catch (Exception e){
		  // TODO Auto-generated catch block
		  e.printStackTrace();
	  }

	  return sendMessage;
  }
   /**
   * Notificar cancelación prematricula de la charla de un usuario.
   * 
   * @param idUsuario
   *           idUsuario
   * @param servicio
   * 			servicio          
   * @return true, if successful
   */
  public boolean notificarCancelacionPreMatriculaCharla(T servicio, Long idUsuario, MatriculaCharla matriculaCharla) {
	  
	  boolean sendMessage = false;

	  boolean isHTMLFormat = true;
	  String url = "";
	  String senderAddress = "", urlEnviar = "";
	  String toAddress = "", ccAddress = "", bccAddress = "", subject = "";
      StringBuffer body = new StringBuffer();
      
      Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
      DateFormat df = new SimpleDateFormat("dd/MM/yyyy HH:mm");

      Usuario usuario = (Usuario) servicio.find(Usuario.class, idUsuario);
      
	  try {
	      // Se obtiene el remitente 
	      senderAddress = getSenderAddress(servicio);
      
		  // Se localizan los areamanger del centro al que pretende suscribirse el usuario
	      List<UsuarioCyL> usuariosCylAEnviar = new ArrayList<UsuarioCyL>();
	      List<Empleado> empleadosAEnviar = new ArrayList<Empleado>();
	      List<Empleado> empleados = servicio.findAll(Empleado.class);

	      for (Empleado e : empleados) {
	        if ((e.getCentro().getId() == matriculaCharla.getCharla().getCentro().getId()) && (e.getPuesto().equals(Puesto.AREA_MANAGER) || 
	        		e.getPuesto().equals(Puesto.DIGITAL_ALLY_MANAGER))) {
	        	if (e.getEmail() != null){
	        		empleadosAEnviar.add(e);
		        	bccAddress += e.getEmail() + ",";
		        	logger.log(Level.INFO, "Notificación de cancelación de solicitud de matrícula charla de usuario al empleado: " + e.getId());
	        	}
	        }
	      }      
	      
	      UsuarioCyL u = (UsuarioCyL) servicio.find(UsuarioCyL.class, idUsuario);
	      
	      usuariosCylAEnviar.add(u);	
	      toAddress =  u.getEmail();

	      // Obtenemos la ruta de la aplicación para poder referenciar la imagen del logotipo de la firma del email.
	      List<Parameter> parameters = servicio.findWhere(Parameter.class, "nombre", "mail_application_url");
	      if (parameters.size() == 1) {	    	
	        url = parameters.get(0).getValor();
	      }
	      
	      // Se elabora el mensaje
	      
	      subject = C_NOTIFICACION + "Cancelación de solicitud de matrícula, para la actividad con identificador " + matriculaCharla.getCharla().getId();
	      String separador = ": ";
	      
	      List<Language> languages = servicio.findAllOrder(Language.class, "id", false);
	      for (Language l : languages) {
	    	  
	    	  Locale languageLocale = new Locale(l.getISO().split("_")[0], l.getISO().split("_")[1]);
	    	  CharlaLanguage cl = getNomenclatura(matriculaCharla.getCharla(), l.getISO());
	    	  
	    	  subject += separador + cl.getNombre();
	    	  separador = " / ";
	    	  
	    	  body.append("<br/>" +
	    			"<strong>" + l.getName() + "</strong>" + "<br/><br/>" +
	    			      			 
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Don") + " " + usuario.getNombreCompleto() + " " +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.txt5") + " " + cl.getNombre() + " " +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EnElCentro") + " " + matriculaCharla.getCharla().getCentro().getNombre() + " " +
	    			"<br/><br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.InfContacto") + "<br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Nombre") + " " + usuario.getNombreCompleto() + "<br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Telefono") + " " + usuario.getTelefonos() + "<br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Email") + " " + u.getEmail() + "<br/>" +
	       			"<br/><br/>" +
	    			"<b>" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EnviadoPorISIS") + "</b>" +
	    			"<br/><br/>" +
	    	  		//"<img src='" + url + "/images/logoXunta.png' />" +
	    			"<img src='cid:imagenFirma' />" +
	    			"<br/><br/><br/><br/>"	    		
	    	  );
	      }	      
	     
	      String pathImagenFirma = ((ServletRequestAttributes) RequestContextHolder.currentRequestAttributes()).getRequest().getRealPath("/images/logoXunta.png");
	      
	      // Se envía el email
	      sendMessage = sendMessage(senderAddress, toAddress, ccAddress, bccAddress, subject,
	    		  isHTMLFormat, body, pathImagenFirma, getMailserverParamSession(servicio));
	      
	      // Si el email ha sisdo enviado correctamente, se registra el envío de la notificación
	     /* if (sendMessage) {
	          Notification n = new Notification();
	          n.setChanel(NotificationChanel.EMAIL);
	          n.setHtmlMessage(true);
	          n.setNotificationDate(cal.getTime());
	          n.setNotificationIdentifier(df.format(cal.getTime()));
	          n.setNotificationType("CancelacionSolicitudPreMatriculaCharla");
	          n.setSubject(subject);
	          
	          List<Usuario> usuariosAEnviar = new ArrayList<Usuario>();
	          
	          for (UsuarioCyL uCyL : usuariosCylAEnviar)
	        	  usuariosAEnviar.add(uCyL);
	          
	          for (Empleado e : empleadosAEnviar)
	        	  usuariosAEnviar.add(e);
	          
	          n.setRecipients(usuariosAEnviar);
	          servicio.persist(n);
	        }*/
	      
	  } catch (Exception e){
		  // TODO Auto-generated catch block
		  e.printStackTrace();
	  }

	  return sendMessage;
  }  
  /**
   * Notificar solicitud de actividad externa.
   * 
   * @param actividadExterna
   *			actividadExterna
   * @param servicio
   * 			servicio          
   * @return true, if successful
   */
  public boolean notificarSolActExterna(T servicio, ActividadExterna actividadExterna) {
	  
	  boolean sendMessage = false;

	  boolean isHTMLFormat = true;
	  String url = "";
	  String senderAddress = "", urlEnviar = "";
	  String toAddress = "", ccAddress = "", bccAddress = "", subject = "";
      StringBuffer body = new StringBuffer();
      
      Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
      DateFormat df = new SimpleDateFormat("dd/MM/yyyy HH:mm");
      
      Empleado empleado = (Empleado)servicio.find(Empleado.class, actividadExterna.getColaborador().getId());

	  
	  try {
	      // Se obtiene el remitente 
	      senderAddress = getSenderAddress(servicio);
      
		  // Se localizan los areamanger del centro al que pretende suscribirse el usuario	      
	      List<Empleado> empleadosAEnviar = new ArrayList<Empleado>();
	      List<Empleado> empleados = servicio.findAll(Empleado.class);
	      
	      // Para cualquier caso (Cesión de Aula, Cesión de Servicio o Ambos) se incluye siempre a los administradores del sistema
	      for (Empleado e : empleados) {
	    	  if (e.getPuesto().equals(Puesto.ADMIN)) {
	    		  empleadosAEnviar.add(e);
	    		  bccAddress += e.getEmail() + ",";
	    		  logger.log(Level.INFO, "Notificación de estado de la Actividad Externa: " + e.getId());
	    	  }
	      }

	      // En el caso de Cesión de Aula o Cesión de Aula y Servicio se incluye a los responsables del centro
	      if(actividadExterna.getCentro() != null){
	    	  Centro centro = (Centro) servicio.find(Centro.class, actividadExterna.getCentro().getId());
	    	  for (Empleado e : empleados) {
	  	        if ((e.getCentro().getId() == centro.getId()) && (e.getPuesto().equals(Puesto.AREA_MANAGER) ||
	  	        		e.getPuesto().equals(Puesto.DIGITAL_ALLY_MANAGER))) {
	  	        	if (e.getEmail() != null){
	  	        		empleadosAEnviar.add(e);
		  	        	bccAddress += e.getEmail() + ",";
		  	        	logger.log(Level.INFO, "Notificación de una nueva solicitud de Actividad Externa: " + e.getId());
	  	        	}
	  	        }
	  	      }
	      }
	      
	      empleadosAEnviar.add(empleado);	
	      toAddress= empleado.getEmail();

	      // Obtenemos la ruta de la aplicación para poder referenciar la imagen del logotipo de la firma del email.
	      List<Parameter> parameters = servicio.findWhere(Parameter.class, "nombre", "mail_application_url");
	      if (parameters.size() == 1) {	    	
	        url = parameters.get(0).getValor();
	      }
	      
	      //Se elabora el body	      
	      
	      subject = C_NOTIFICACION + "Solicitud para la actividad externa con identificador " + actividadExterna.getId();
	      String separador = ": ";
	      
	      List<Language> languages = servicio.findAllOrder(Language.class, "id", false);
	      for (Language l : languages) {
	    	  
	    	  Locale languageLocale = new Locale(l.getISO().split("_")[0], l.getISO().split("_")[1]);
	    	  ActividadExternaLanguage ael = getNomenclatura(actividadExterna, l.getISO(), servicio);
	    	  
	    	  subject += separador + ael.getNombre();
	    	  separador = " / ";
	    	  
	    	  // Estado de la solicitud de la Actividad Externa
	          String estado = "<span style=\"color: ";
	          if (actividadExterna.getEstado() == EEstadoActividadExterna.ACEPTADA)
	        	  estado += "#00CC00\">" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.AceptaSolicitudActExt");
	          else if (actividadExterna.getEstado() == EEstadoActividadExterna.SOLICITADA)
	        	  estado += "#FF9000\">" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.SolicitaSolicitudActExt");
	          else //if (actividadExterna.getEstado() == EEstadoActividadExterna.RECHAZADA)
	        	  estado += "#CC0000\">" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.RechazaSolicitudActExt");
	          estado += "</span>";
	    	  
	    	  body.append("<br/>" +
	    			"<strong>" + l.getName() + "</strong>" + "<br/><br/>" +
	    			      			 
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.ElColaborador") + " " + empleado.getNombreCompleto() + " " +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.DeLaEntidad") + " " + empleado.getProveedor().getNombre() + " " +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.txt6") + " " + ael.getNombre() + " " +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EntreLosDias") + " " + actividadExterna.getFechasLegibles() + " " +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.ConHorario") + " " + actividadExterna.getHorario() + " ");
	    	  
	    	  if (actividadExterna.getCentro() != null) {
	    		  Centro centro = (Centro) servicio.find(Centro.class, actividadExterna.getCentro().getId());
	    		  body.append(ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EnElCentro") + " " + centro.getNombre() + " " +
	    		  			  "( " + centro.getDireccion() + ") ");
	    	  }
	    		
	    	  body.append("<br/><br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EstadoSolicitud") + " <b>" + estado + "</b>" +

	    			"<br/><br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.InfContacto") + "<br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Nombre") + " " + empleado.getNombreCompleto() + "<br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Telefono") + " " + empleado.getTelefonos() + "<br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Email") + " " + empleado.getEmail() + "<br/>" +
	       			"<br/><br/>" +
	    			"<b>" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EnviadoPorISIS") + "</b>" +
	    			"<br/><br/>" +
	    	  		//"<img src='" + url + "/images/logoXunta.png' />" +
	    			"<img src='cid:imagenFirma' />" +
	    			"<br/><br/><br/><br/>"	    		
	    	  );
	      }
	      
	      String pathImagenFirma = ((ServletRequestAttributes) RequestContextHolder.currentRequestAttributes()).getRequest().getRealPath("/images/logoXunta.png");
	      
	      // Se envía el email
	      sendMessage = sendMessage(senderAddress, toAddress, ccAddress, bccAddress, subject,
	    		  isHTMLFormat, body, pathImagenFirma, getMailserverParamSession(servicio));
	      
	      // Si el email ha sisdo enviado correctamente, se registra el envío de la notificación
	      /*if (sendMessage) {
	          Notification n = new Notification();
	          n.setChanel(NotificationChanel.EMAIL);
	          n.setHtmlMessage(true);
	          n.setNotificationDate(cal.getTime());
	          n.setNotificationIdentifier(df.format(cal.getTime()));
	          n.setNotificationType("SolicitudActividadExterna");
	          n.setSubject(subject);
	          
	          List<Usuario> usuariosAEnviar = new ArrayList<Usuario>();

	          for (Empleado e : empleadosAEnviar)
	        	  usuariosAEnviar.add(e);
	          
	          n.setRecipients(usuariosAEnviar);
	          servicio.persist(n);
	        }*/
	      
	  } catch (Exception e){
		  // TODO Auto-generated catch block
		  e.printStackTrace();
	  }

	  return sendMessage;
  }
  /**
   * Notificar el cambio de estado de una solicitud de actividad externa.
   * 
   * @param actividadExterna
   *			actividadExterna
   * @param servicio
   * 			servicio          
   * @return true, if successful
   */
  public boolean notificarEstadoSolActExterna(T servicio, ActividadExterna actividadExterna) {
	  
	  boolean sendMessage = false;

	  boolean isHTMLFormat = true;
	  String url = "";
	  String senderAddress = "", urlEnviar = "";
	  String toAddress = "", ccAddress = "", bccAddress = "", subject = "";
      StringBuffer body = new StringBuffer();
      
      Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
      DateFormat df = new SimpleDateFormat("dd/MM/yyyy HH:mm");
      
      Empleado empleado = (Empleado)servicio.find(Empleado.class, actividadExterna.getColaborador().getId());
	  
	  try {
	      // Se obtiene el remitente 
	      senderAddress = getSenderAddress(servicio);
      
		  // Se localizan los areamanger del centro al que pretende suscribirse el usuario
	      List<UsuarioCyL> usuariosCylAEnviar = new ArrayList<UsuarioCyL>();
	      List<Empleado> empleadosAEnviar = new ArrayList<Empleado>();
	      List<Empleado> empleados = servicio.findAll(Empleado.class);	            
	      
	      // Para cualquier caso (Cesión de Aula, Cesión de Servicio o Ambos) se incluye siempre a los administradores del sistema
	      for (Empleado e : empleados) {
		        if (e.getPuesto().equals(Puesto.ADMIN)) {
		        	empleadosAEnviar.add(e);
		        	bccAddress += e.getEmail() + ",";
		        	logger.log(Level.INFO, "Notificación de estado de la Actividad Externa: " + e.getId());
		        }
		      }
	      
	      // En el caso de Cesión de Aula o Cesión de Aula y Servicio se incluye a los responsables del centro
	      if(actividadExterna.getCentro() != null){
	    	  Centro centro = (Centro) servicio.find(Centro.class, actividadExterna.getCentro().getId());
	    	  for (Empleado e : empleados) {
	  	        if ((e.getCentro().equals(centro)) && (e.getPuesto().equals(Puesto.AREA_MANAGER) ||
	  	        		e.getPuesto().equals(Puesto.DIGITAL_ALLY_MANAGER))) {
	  	        	if (e.getEmail() != null){
	  	        		empleadosAEnviar.add(e);
		  	        	bccAddress += e.getEmail() + ",";
		  	        	logger.log(Level.INFO, "Notificación de estado de la Actividad Externa: " + e.getId());
	  	        	}
	  	        }
	  	      }
	      }
	      
	      empleadosAEnviar.add(empleado);		
	      toAddress= empleado.getEmail();

	      // Obtenemos la ruta de la aplicación para poder referenciar la imagen del logotipo de la firma del email.
	      List<Parameter> parameters = servicio.findWhere(Parameter.class, "nombre", "mail_application_url");
	      if (parameters.size() == 1) {	    	
	        url = parameters.get(0).getValor();
	      }
	      
	      //Se elabora el body
	      ///
	      subject = C_NOTIFICACION + "Estado de la solicitud para la actividad externa con identificador " + actividadExterna.getId();
	      String separador = ": ";
	      
	      List<Language> languages = servicio.findAllOrder(Language.class, "id", false);
	      for (Language l : languages) {
	    	  
	    	  Locale languageLocale = new Locale(l.getISO().split("_")[0], l.getISO().split("_")[1]);
	    	  ActividadExternaLanguage ael = getNomenclatura(actividadExterna, l.getISO(), servicio);
	    	  
	    	  subject += separador + ael.getNombre();
	    	  separador = " / ";
	    	  
	    	  // Estado de la solicitud de la Actividad Externa
	          String estado = "<span style=\"color: ";
	          if (actividadExterna.getEstado() == EEstadoActividadExterna.ACEPTADA)
	        	  estado += "#00CC00\">" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.AceptaSolicitudActExt");
	          else if (actividadExterna.getEstado() == EEstadoActividadExterna.SOLICITADA)
	        	  estado += "#FF9000\">" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.SolicitaSolicitudActExt");
	          else //if (actividadExterna.getEstado() == EEstadoActividadExterna.RECHAZADA)
	        	  estado += "#CC0000\">" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.RechazaSolicitudActExt");
	          estado += "</span>";
	    	  
	    	  body.append("<br/>" +
	    			  	  "<strong>" + l.getName() + "</strong>" + "<br/><br/>" +

	    			  	  ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.txt7") + " " + ael.getNombre() + " " +
	    			  	  ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EntreLosDias") + " " + actividadExterna.getFechasLegibles() + " " +
	    			  	  ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.ConHorario") + " " + actividadExterna.getHorario() + " ");
	    			
	     	  if (actividadExterna.getCentro() != null) {
	     		  Centro centro = (Centro) servicio.find(Centro.class, actividadExterna.getCentro().getId());
	     		  body.append(" (" + centro.getDireccion() + ") ");
	     	  }
	    	  
	     	   body.append(ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.ParaElColaborador") + " " + empleado.getNombreCompleto() + " " +
	     			   	   ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.DeLaEntidad") + " " + empleado.getProveedor().getNombre() + " " +
	     			   	   "<br/><br/>" +
	     			   	   ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EstadoSolicitud") + " <b>" + estado + "</b>" +
	     			   	   "<br/><br/>" +
	     			   	   ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.InfContacto") + "<br/>" +
	     			   	   ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Nombre") + " " + empleado.getNombreCompleto() + "<br/>" +
	     			   	   ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Telefono") + " " + empleado.getTelefonos() + "<br/>" +
	     			   	   ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Email") + " " + empleado.getEmail() + "<br/>" +
	     			   	   "<br/><br/>" +
	     			   	   "<b>" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EnviadoPorISIS") + "</b>" +
	     			   	   "<br/><br/>" +
	     			   	   //"<img src='" + url + "/images/logoXunta.png' />" +
	     			   	   "<img src='cid:imagenFirma' />" +
	     			   	   "<br/><br/><br/><br/>"	    		
	    	  );
	      }
	     
	      String pathImagenFirma = ((ServletRequestAttributes) RequestContextHolder.currentRequestAttributes()).getRequest().getRealPath("/images/logoXunta.png");
	      
	      // Se envía el email
	      sendMessage = sendMessage(senderAddress, toAddress, ccAddress, bccAddress, subject,
	    		  isHTMLFormat, body, pathImagenFirma, getMailserverParamSession(servicio));
	      
	      // Si el email ha sisdo enviado correctamente, se registra el envío de la notificación
	      /*if (sendMessage) {
	          Notification n = new Notification();
	          n.setChanel(NotificationChanel.EMAIL);
	          n.setHtmlMessage(true);
	          n.setNotificationDate(cal.getTime());
	          n.setNotificationIdentifier(df.format(cal.getTime()));
	          n.setNotificationType("AceptacionRechazoSolicitudActExterna");
	          n.setSubject(subject);
	          
	          List<Usuario> usuariosAEnviar = new ArrayList<Usuario>();
	          for (Empleado e : empleadosAEnviar)
	        	  usuariosAEnviar.add(e);
	          
	          n.setRecipients(usuariosAEnviar);
	          servicio.persist(n);
	        }*/
	      
	  } catch (Exception e){
		  // TODO Auto-generated catch block
		  e.printStackTrace();
	  }

	  return sendMessage;
  }
    /**
   * Notificar solicitud de actividad externa.
   * 
   * @param actividadExterna
   *			actividadExterna
   * @param servicio
   * 			servicio          
   * @return true, if successful
   */
  public boolean notificarCancelarSolActExterna(T servicio, ActividadExterna actividadExterna) {
	  
	  boolean sendMessage = false;

	  boolean isHTMLFormat = true;
	  String url = "";
	  String senderAddress = "", urlEnviar = "";
	  String toAddress = "", ccAddress = "", bccAddress = "", subject = "";
      StringBuffer body = new StringBuffer();
      
      Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
      DateFormat df = new SimpleDateFormat("dd/MM/yyyy HH:mm");
      
      Empleado empleado = (Empleado)servicio.find(Empleado.class, actividadExterna.getColaborador().getId());

	  
	  try {
	      // Se obtiene el remitente 
	      senderAddress = getSenderAddress(servicio);
      
		  // Se localizan los areamanger del centro al que pretende suscribirse el usuario
	      List<Empleado> empleadosAEnviar = new ArrayList<Empleado>();
	      List<Empleado> empleados = servicio.findAll(Empleado.class);
	      
	      // Para cualquier caso (Cesión de Aula, Cesión de Servicio o Ambos) se incluye siempre a los administradores del sistema
	      for (Empleado e : empleados) {
	    	  if (e.getPuesto().equals(Puesto.ADMIN)) {
	    		  empleadosAEnviar.add(e);
	    		  bccAddress += e.getEmail() + ",";
	    		  logger.log(Level.INFO, "Notificación de estado de la Actividad Externa: " + e.getId());
	    	  }
	      }

	      // En el caso de Cesión de Aula o Cesión de Aula y Servicio se incluye a los responsables del centro
	      if(actividadExterna.getCentro() != null){
	    	  Centro centro = (Centro) servicio.find(Centro.class, actividadExterna.getCentro().getId());
	    	  for (Empleado e : empleados) {
	  	        if ((e.getCentro().getId() == centro.getId()) && (e.getPuesto().equals(Puesto.AREA_MANAGER) ||
	  	        		e.getPuesto().equals(Puesto.DIGITAL_ALLY_MANAGER))) {
	  	        	if (e.getEmail() != null){
	  	        		empleadosAEnviar.add(e);
		  	        	bccAddress += e.getEmail() + ",";
		  	        	logger.log(Level.INFO, "Notificación de cancelación de solicitud de Actividad Externa: " + e.getId());
	  	        	}
	  	        }
	  	      }
	      }
	      
	      empleadosAEnviar.add(empleado);
	      toAddress= empleado.getEmail();

	      // Obtenemos la ruta de la aplicación para poder referenciar la imagen del logotipo de la firma del email.
	      List<Parameter> parameters = servicio.findWhere(Parameter.class, "nombre", "mail_application_url");
	      if (parameters.size() == 1) {	    	
	        url = parameters.get(0).getValor();
	      }
	      
	      //Se elabora el body	      
	      
	      subject = C_NOTIFICACION + "Cancelación de solicitud para la actividad externa con identificador " + actividadExterna.getId();
	      String separador = ": ";
	      
	      List<Language> languages = servicio.findAllOrder(Language.class, "id", false);
	      for (Language l : languages) {
	    	  
	    	  Locale languageLocale = new Locale(l.getISO().split("_")[0], l.getISO().split("_")[1]);
	    	  ActividadExternaLanguage ael = getNomenclatura(actividadExterna, l.getISO(), servicio);
	    	  
	    	  subject += separador + ael.getNombre();
	    	  separador = " / ";
	    	  
	    	  body.append("<br/>" +
	    			"<strong>" + l.getName() + "</strong>" + "<br/><br/>" +
	    			      			 
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.ElColaborador") + " " + empleado.getNombreCompleto() + " " +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.txt14") + " " + ael.getNombre() + " " +
	    			"<br/><br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.InfContacto") + "<br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Nombre") + " " + empleado.getNombreCompleto() + "<br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Telefono") + " " + empleado.getTelefonos() + "<br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.Email") + " " + empleado.getEmail() + "<br/>" +
	       			"<br/><br/>" +
	    			"<b>" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EnviadoPorISIS") + "</b>" +
	    			"<br/><br/>" +
	    	  		//"<img src='" + url + "/images/logoXunta.png' />" +
	    			"<img src='cid:imagenFirma' />" +
	    			"<br/><br/><br/><br/>"	    		
	    	  );
	      }
	      
	      String pathImagenFirma = ((ServletRequestAttributes) RequestContextHolder.currentRequestAttributes()).getRequest().getRealPath("/images/logoXunta.png");
	      
	      // Se envía el email
	      sendMessage = sendMessage(senderAddress, toAddress, ccAddress, bccAddress, subject,
	    		  isHTMLFormat, body, pathImagenFirma, getMailserverParamSession(servicio));
	      
	      // Si el email ha sisdo enviado correctamente, se registra el envío de la notificación
	      /*if (sendMessage) {
	          Notification n = new Notification();
	          n.setChanel(NotificationChanel.EMAIL);
	          n.setHtmlMessage(true);
	          n.setNotificationDate(cal.getTime());
	          n.setNotificationIdentifier(df.format(cal.getTime()));
	          n.setNotificationType("CancelacionSolicitudActividadExterna");
	          n.setSubject(subject);
	          
	          List<Usuario> usuariosAEnviar = new ArrayList<Usuario>();
	          for (Empleado e : empleadosAEnviar)
	        	  usuariosAEnviar.add(e);
	          
	          n.setRecipients(usuariosAEnviar);
	          servicio.persist(n);
	        }*/
	      
	  } catch (Exception e){
		  // TODO Auto-generated catch block
		  e.printStackTrace();
	  }

	  return sendMessage;
  } 
   /**
   * Notificar cancelación de matricula del curso de un usuario. 
   * 
   * @param matriculacurso
   * 			matriculacurso
   * @param servicio
   * 			servicio          
   * @return true, if successful
   */
  public boolean notificarBorrarMatriculaCursoUsuario(T servicio, MatriculaCurso matriculaCurso) {
	  
	 boolean sendMessage = false;
	 boolean isHTMLFormat = true;
	 String url = "";
	 String senderAddress = "", urlEnviar = "";
	 String toAddress = "", ccAddress = "", bccAddress = "", subject = "";
	 StringBuffer body = new StringBuffer();
	 
	 Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
	 DateFormat df = new SimpleDateFormat("dd/MM/yyyy HH:mm");
	
	 Usuario usuario = (Usuario) servicio.find(Usuario.class, matriculaCurso.getUsuario().getId());
	 
	  try {
	      // Se obtiene el remitente 
	      senderAddress = getSenderAddress(servicio);
	 
		  // Se localizan los areamanger del centro al que pretende suscribirse el usuario
	      List<UsuarioCyL> usuariosCylAEnviar = new ArrayList<UsuarioCyL>();
	      List<Empleado> empleadosAEnviar = new ArrayList<Empleado>();
	      List<Empleado> empleados = servicio.findAll(Empleado.class);
	
	  for (Empleado e : empleados) {
	    if ((e.getCentro().getId() == matriculaCurso.getCurso().getCentro().getId()) && (e.getPuesto().equals(Puesto.AREA_MANAGER) ||
	    		e.getPuesto().equals(Puesto.DIGITAL_ALLY_MANAGER))) {
	    	if (e.getEmail() != null){
	    		empleadosAEnviar.add(e);
		    	bccAddress += e.getEmail() + ",";
		    	logger.log(Level.INFO, "Notificación del borrado de una matrícula curso de usuario al empleado: " + e.getId());
	    	}
	    }
	  }      
	  
	  UsuarioCyL u = (UsuarioCyL) servicio.find(UsuarioCyL.class, matriculaCurso.getUsuario().getId());
	  
	  usuariosCylAEnviar.add(u);
	  toAddress =  u.getEmail();
	
	  // Obtenemos la ruta de la aplicación para poder referenciar la imagen del logotipo de la firma del email.
	  List<Parameter> parameters = servicio.findWhere(Parameter.class, "nombre", "mail_application_url");
	  if (parameters.size() == 1) {	    	
	    url = parameters.get(0).getValor();
	  }
	  
	  // Se elabora el mensaje
	  
	  subject = C_NOTIFICACION + "Matrícula eliminada para el curso con identificador " + matriculaCurso.getCurso().getId();
	  String separador = ": ";
      
      List<Language> languages = servicio.findAllOrder(Language.class, "id", false);
      for (Language l : languages) {
    	  
    	  Locale languageLocale = new Locale(l.getISO().split("_")[0], l.getISO().split("_")[1]);
    	  CursoLanguage cl = getNomenclatura(matriculaCurso.getCurso(), l.getISO());
    	  
    	  subject += separador + cl.getNombre();
    	  separador = " / ";
    	  
    	  body.append("<br/>" +
    			"<strong>" + l.getName() + "</strong>" + "<br/><br/>" +
    			      			 
    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.txt9") + " " + cl.getNombre() + " " +
    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.ParaElUsuario") + " " + usuario.getNombreCompleto() + " " +
    			"<br/><br/>" +
    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.RealizarCualquierGestion") + 
       			"<br/><br/>" +
    			"<b>" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EnviadoPorISIS") + "</b>" +
    			"<br/><br/>" +
    	  		//"<img src='" + url + "/images/logoXunta.png' />" +
    			"<img src='cid:imagenFirma' />" +
    			"<br/><br/><br/><br/>"	    		
    	  );
      }	     
      
      String pathImagenFirma = ((ServletRequestAttributes) RequestContextHolder.currentRequestAttributes()).getRequest().getRealPath("/images/logoXunta.png");
      
	  // Se envía el email
	  sendMessage = sendMessage(senderAddress, toAddress, ccAddress, bccAddress, subject,
			  isHTMLFormat, body, pathImagenFirma, getMailserverParamSession(servicio));
	  
	  // Si el email ha sisdo enviado correctamente, se registra el envío de la notificación
	  /*if (sendMessage) {
	      Notification n = new Notification();
	      n.setChanel(NotificationChanel.EMAIL);
	      n.setHtmlMessage(true);
	      n.setNotificationDate(cal.getTime());
	      n.setNotificationIdentifier(df.format(cal.getTime()));
	      n.setNotificationType("BorrarMatriculaCurso");
	      n.setSubject(subject);
	          
	          List<Usuario> usuariosAEnviar = new ArrayList<Usuario>();
	          
	          for (UsuarioCyL uCyL : usuariosCylAEnviar)
	        	  usuariosAEnviar.add(uCyL);
	          
	          for (Empleado e : empleadosAEnviar)
	        	  usuariosAEnviar.add(e);
	          
	          n.setRecipients(usuariosAEnviar);
	          servicio.persist(n);
	        }*/
	      
	  } catch (Exception e){
		  // TODO Auto-generated catch block
		  e.printStackTrace();
	  }
	
	  return sendMessage;
 }
  
  /**
   * Notificar cancelación de matricula de charla de un usuario.
   * 
   * @param matriculaCharla
   * 			matriculaCharla
   * @param servicio
   * 			servicio          
   * @return true, if successful
   */
  public boolean notificarBorrarMatriculaCharlaUsuario(T servicio, MatriculaCharla matriculaCharla) {
	  
	 boolean sendMessage = false;

	 boolean isHTMLFormat = true;
	 String url = "";
	 String senderAddress = "", urlEnviar = "";
	 String toAddress = "", ccAddress = "", bccAddress = "", subject = "";
     StringBuffer body = new StringBuffer();
     
     Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
     DateFormat df = new SimpleDateFormat("dd/MM/yyyy HH:mm");

     Usuario usuario = (Usuario) servicio.find(Usuario.class, matriculaCharla.getUsuario().getId());
     
	  try {
	      // Se obtiene el remitente 
	      senderAddress = getSenderAddress(servicio);
     
		  // Se localizan los areamanger del centro al que pretende suscribirse el usuario
	      List<UsuarioCyL> usuariosCylAEnviar = new ArrayList<UsuarioCyL>();
	      List<Empleado> empleadosAEnviar = new ArrayList<Empleado>();
	      List<Empleado> empleados = servicio.findAll(Empleado.class);

	      for (Empleado e : empleados) {
	        if ((e.getCentro().getId() == matriculaCharla.getCharla().getCentro().getId()) && (e.getPuesto().equals(Puesto.AREA_MANAGER) ||
	        		e.getPuesto().equals(Puesto.DIGITAL_ALLY_MANAGER))) {
	        	if (e.getEmail() != null){
	        		empleadosAEnviar.add(e);
		        	bccAddress += e.getEmail() + ",";
		        	logger.log(Level.INFO, "Notificación borrado de la matricula charla de usuario al empleado: " + e.getId());
	        	}
	        }
	      }      
	      
	      UsuarioCyL u = (UsuarioCyL) servicio.find(UsuarioCyL.class, matriculaCharla.getUsuario().getId());
	      
	      usuariosCylAEnviar.add(u);
	      toAddress =  u.getEmail();

	      // Obtenemos la ruta de la aplicación para poder referenciar la imagen del logotipo de la firma del email.
	      List<Parameter> parameters = servicio.findWhere(Parameter.class, "nombre", "mail_application_url");
	      if (parameters.size() == 1) {	    	
	        url = parameters.get(0).getValor();
	      }
	      
	      // Se elabora el mensaje
	      
	      subject = C_NOTIFICACION + "Matrícula eliminada para la actividad con identificador " + matriculaCharla.getCharla().getId();
	      String separador = ": ";
	      
	      List<Language> languages = servicio.findAllOrder(Language.class, "id", false);
	      for (Language l : languages) {
	    	  
	    	  Locale languageLocale = new Locale(l.getISO().split("_")[0], l.getISO().split("_")[1]);
	    	  CharlaLanguage cl = getNomenclatura(matriculaCharla.getCharla(), l.getISO());
	    	  
	    	  subject += separador + cl.getNombre();
	    	  separador = " / ";
	    	  
	    	  body.append("<br/>" +
	    			"<strong>" + l.getName() + "</strong>" + "<br/><br/>" +
	    			      			 
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.txt10") + " " + cl.getNombre() + " " +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.ParaElUsuario") + " " + usuario.getNombreCompleto() + " " +
	    			"<br/><br/>" +
	    			ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.RealizarCualquierGestion") + 
	       			"<br/><br/>" +
	    			"<b>" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EnviadoPorISIS") + "</b>" +
	    			"<br/><br/>" +
	    	  		//"<img src='" + url + "/images/logoXunta.png' />" +
	    			"<img src='cid:imagenFirma' />" +
	    			"<br/><br/><br/><br/>"	    		
	    	  );
	      }
	  
	      String pathImagenFirma = ((ServletRequestAttributes) RequestContextHolder.currentRequestAttributes()).getRequest().getRealPath("/images/logoXunta.png");
	      
	      // Se envía el email
	      sendMessage = sendMessage(senderAddress, toAddress, ccAddress, bccAddress, subject,
	    		  isHTMLFormat, body, pathImagenFirma, getMailserverParamSession(servicio));
	      
	      // Si el email ha sisdo enviado correctamente, se registra el envío de la notificación
	      /*if (sendMessage) {
	          Notification n = new Notification();
	          n.setChanel(NotificationChanel.EMAIL);
	          n.setHtmlMessage(true);
	          n.setNotificationDate(cal.getTime());
	          n.setNotificationIdentifier(df.format(cal.getTime()));
	          n.setNotificationType("BorrarMatriculaCharla");
	          n.setSubject(subject);
	          
	          List<Usuario> usuariosAEnviar = new ArrayList<Usuario>();
	          
	          for (UsuarioCyL uCyL : usuariosCylAEnviar)
	        	  usuariosAEnviar.add(uCyL);
	          
	          for (Empleado e : empleadosAEnviar)
	        	  usuariosAEnviar.add(e);
	          
	          n.setRecipients(usuariosAEnviar);
	          servicio.persist(n);
	        }*/
	      
	  } catch (Exception e){
		  // TODO Auto-generated catch block
		  e.printStackTrace();
	  }

	  return sendMessage;
 }
  
  /**
   * Notificar cerrado automático de curso.
   * 
   * @param curso
   * 			curso
   * 
   * @param servicio
   * 			servicio
   *           
   * @return true, if successful
   */
  public void notificarCursoAutoCerrado(T servicio, Curso curso) {
	  boolean isHTMLFormat = true;
	  String senderAddress = "";
	  String bccAddress = "", subject = "";
	  StringBuffer body = new StringBuffer();
 
	  Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
	  DateFormat df = new SimpleDateFormat("dd/MM/yyyy HH:mm");

	  try {
		  // Se obtiene el remitente 
		  senderAddress = getSenderAddress(servicio);
		  
		  // Se localizan los areamanger del centro al que pretende suscribirse el usuario
		  Map<String, Object> properties = new HashMap<String, Object>();
		  properties.put("centro.id", curso.getCentro().getId());
		  properties.put("puesto", Puesto.AREA_MANAGER);
		  
		  List<Empleado> empleadosAEnviar = servicio.findWhere(Empleado.class, properties);
		  
		  properties.put("puesto", Puesto.DIGITAL_ALLY_MANAGER);
		  empleadosAEnviar.addAll(servicio.findWhere(Empleado.class, properties));

		  for (Empleado e : empleadosAEnviar) {
			  if (e.getEmail() != null){
				  bccAddress += e.getEmail() + ",";
				  logger.log(Level.INFO, "NotificationService:notificarCursoAutoCerrado -> " +
						  				 "Notificación cierre automático de curso al empleado: " + e.getId());
			  }
		  }      
		      
		  // Se elabora el mensaje
		  subject = C_NOTIFICACION + "Se ha cerrado automáticamente el curso con identificador " + curso.getId();
		  String separador = ": ";
		      
		  List<Language> languages = servicio.findAllOrder(Language.class, "id", false);
		  for (Language l : languages) {
			  Locale languageLocale = new Locale(l.getISO().split("_")[0], l.getISO().split("_")[1]);
			  CursoLanguage cl = getNomenclatura(curso, l.getISO());
		    	  
			  subject += separador + cl.getNombre();
			  separador = " / ";
		    	  
			  body.append("<br/>" +
					  	  "<strong>" + l.getName() + "</strong>" + "<br/><br/>" +
					  	  ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.cierreAutoCurso.txt1") + " \"" + cl.getNombre() + "\" " +
					  	  ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.cierreAutoCurso.txt2") + " " + curso.getId() + ", " +
					  	  ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.cierreAutoCurso.txt3") +
					  	  "<br/><br/>" +
					  	  df.format(cal.getTime()) +
					  	  "<br/><br/>" +
					  	  "<b>" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EnviadoPorISIS") + "</b>" +
					  	  "<br/><br/>" +
					  	  "<img src='cid:imagenFirma' />" +
					  	  "<br/><br/><br/><br/>"	    		
			  );
		  }
		  
		  String pathImagenFirma = ((ServletRequestAttributes) RequestContextHolder.currentRequestAttributes()).getRequest().getRealPath("/images/logoXunta.png");
		      
		  // Se envía el email
		  sendMessage(senderAddress, "", "", bccAddress, subject,
				  	  isHTMLFormat, body, pathImagenFirma, getMailserverParamSession(servicio));
	  }
	  catch (Exception e) {
		  e.printStackTrace();
	  }
  }
  
  /**
   * Notificación de aviso de próximo cierre automático de curso por la tarea automática de rastreo.
   * 
   * @param servicio
   * 			servicio
   * 
   * @param curso
   * 			curso
   * 
   * @param daysDeadlineAdvise
   * 			Plazo máximo consumido en días para el aviso del próximo cierre.
   * 
   * @param daysDeadlineClose
   * 			Plazo máximo en días para el cierre de la actividad antes del cierre automático 
   *           
   * @return true, if successful
   */
  public void notificarCursoAutoCerradoProximamenteByJob(ApplicationContext appContext, T servicio, Curso curso,
		  												 Integer daysDeadlineAdvise, Integer daysDeadlineClose) {
	  boolean isHTMLFormat = true;
	  String senderAddress = "";
	  String bccAddress = "", subject = "";
	  StringBuffer body = new StringBuffer();
 
	  Calendar cal = Calendar.getInstance(new Locale("es_ES"));
	  DateFormat df = new SimpleDateFormat("dd/MM/yyyy HH:mm");

	  try {
		  // Se obtiene el remitente 
		  senderAddress = getSenderAddress(servicio);
		  
		  // Se localizan los areamanger del centro al que pretende suscribirse el usuario
		  Map<String, Object> properties = new HashMap<String, Object>();
		  properties.put("centro.id", curso.getCentro().getId());
		  properties.put("puesto", Puesto.AREA_MANAGER);
		  
		  List<Empleado> empleadosAEnviar = servicio.findWhere(Empleado.class, properties);
		  
		  properties.put("puesto", Puesto.DIGITAL_ALLY_MANAGER);
		  empleadosAEnviar.addAll(servicio.findWhere(Empleado.class, properties));

		  for (Empleado e : empleadosAEnviar) {
			  if (e.getEmail() != null) {
				  bccAddress += e.getEmail() + ",";
				  logger.log(Level.INFO, "NotificationService:notificarCursoAutoCerradoProximamenteByJob -> " +
						  				 "Notificación aviso de cierre automático de curso al empleado: " + e.getId());
			  }
		  }

		  // Se elabora el mensaje
		  subject = C_NOTIFICACION + "Próximamente se cerrará automáticamente el curso con identificador " + curso.getId();
		  String separador = ": ";
		      
		  List<Language> languages = servicio.findAllOrder(Language.class, "id", false);
		  for (Language l : languages) {
			  Locale languageLocale = new Locale(l.getISO().split("_")[0], l.getISO().split("_")[1]);
			  CursoLanguage cl = getNomenclatura(curso, l.getISO());
		    	  
			  subject += separador + cl.getNombre();
			  separador = " / ";
		    	  
			  body.append("<br/>" +
					  	  "<strong>" + l.getName() + "</strong>" + "<br/><br/>" +
					  	  ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.cierreAutoCursoByJob.txt1") + " \"" + cl.getNombre() + "\" " +
					  	  ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.cierreAutoCursoByJob.txt2") + " " + curso.getId() + ", " +
					  	  ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.cierreAutoCursoAdviseByJob.txt3") + " " + daysDeadlineAdvise + " " +
					  	  ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.cierreAutoCursoAdviseByJob.txt4") + " " + daysDeadlineClose + " " +
					  	  ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.cierreAutoCursoAdviseByJob.txt5") +
					  	  "<br/><br/>" +
					  	  df.format(cal.getTime()) +
					  	  "<br/><br/>" +
					  	  "<b>" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EnviadoPorISIS") + "</b>" +
					  	  "<br/><br/>" +
					  	  "<img src='cid:imagenFirma' />" +
					  	  "<br/><br/><br/><br/>"	    		
			  );
		  }
		  
		  String pathImagenFirma = appContext.getResource("/images/logoXunta.png").getFile().getAbsolutePath();
		      
		  // Se envía el email
		  sendMessage(senderAddress, "", "", bccAddress, subject,
				  	  isHTMLFormat, body, pathImagenFirma, getMailserverParamSession(servicio));
	  }
	  catch (Exception e) {
		  e.printStackTrace();
	  }
  }
  
  /**
   * Notificar cerrado automático de curso por la tarea automática de rastreo.
   * 
   * @param servicio
   * 			servicio
   * 
   * @param curso
   * 			curso 
   * 
   * @param daysDeadlineClose
   * 			Plazo máximo consumido para el cierre automático del curso.
   *           
   * @return true, if successful
   */
  public void notificarCursoAutoCerradoByJob(ApplicationContext appContext, T servicio, Curso curso, Integer daysDeadlineClose) {
	  boolean isHTMLFormat = true;
	  String senderAddress = "";
	  String bccAddress = "", subject = "";
	  StringBuffer body = new StringBuffer();
 
	  Calendar cal = Calendar.getInstance(new Locale("es_ES"));
	  DateFormat df = new SimpleDateFormat("dd/MM/yyyy HH:mm");

	  try {
		  // Se obtiene el remitente 
		  senderAddress = getSenderAddress(servicio);
		  
		  // Se localizan los areamanger del centro al que pretende suscribirse el usuario
		  Map<String, Object> properties = new HashMap<String, Object>();
		  properties.put("centro.id", curso.getCentro().getId());
		  properties.put("puesto", Puesto.AREA_MANAGER);
		  
		  List<Empleado> empleadosAEnviar = servicio.findWhere(Empleado.class, properties);
		  
		  properties.put("puesto", Puesto.DIGITAL_ALLY_MANAGER);
		  empleadosAEnviar.addAll(servicio.findWhere(Empleado.class, properties));

		  for (Empleado e : empleadosAEnviar) {
			  if (e.getEmail() != null){
				  bccAddress += e.getEmail() + ",";
				  logger.log(Level.INFO, "NotificationService:notificarCursoAutoCerradoByJob -> " +
						  				 "Notificación cierre automático de curso al empleado: " + e.getId());
			  }
		  }      
		      
		  // Se elabora el mensaje
		  subject = C_NOTIFICACION + "Se ha cerrado automáticamente el curso con identificador " + curso.getId();
		  String separador = ": ";
		      
		  List<Language> languages = servicio.findAllOrder(Language.class, "id", false);
		  for (Language l : languages) {
			  Locale languageLocale = new Locale(l.getISO().split("_")[0], l.getISO().split("_")[1]);
			  CursoLanguage cl = getNomenclatura(curso, l.getISO());
		    	  
			  subject += separador + cl.getNombre();
			  separador = " / ";
		    	  
			  body.append("<br/>" +
					  	  "<strong>" + l.getName() + "</strong>" + "<br/><br/>" +
					  	  ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.cierreAutoCursoByJob.txt1") + " \"" + cl.getNombre() + "\" " +
					  	  ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.cierreAutoCursoByJob.txt2") + " " + curso.getId() + ", " +
					  	  ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.cierreAutoCursoByJob.txt3") + " " + daysDeadlineClose + " " +
					  	  ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.cierreAutoCursoByJob.txt4") +
					  	  "<br/><br/>" +
					  	  df.format(cal.getTime()) +
					  	  "<br/><br/>" +
					  	  "<b>" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EnviadoPorISIS") + "</b>" +
					  	  "<br/><br/>" +
					  	  "<img src='cid:imagenFirma' />" +
					  	  "<br/><br/><br/><br/>"	    		
			  );
		  }
		  
		  String pathImagenFirma = appContext.getResource("/images/logoXunta.png").getFile().getAbsolutePath();
		      
		  // Se envía el email
		  sendMessage(senderAddress, "", "", bccAddress, subject,
				  	  isHTMLFormat, body, pathImagenFirma, getMailserverParamSession(servicio));
	  }
	  catch (Exception e) {
		  e.printStackTrace();
	  }
  }
  
  /**
   * Notificación de aviso de próximo cierre automático de curso por la tarea automática de rastreo.
   * 
   * @param servicio
   * 			servicio
   * 
   * @param externa
   * 			Actividad externa
   * 
   * @param daysDeadlineAdvise
   * 			Plazo máximo consumido en días para el aviso del próximo cierre.
   * 
   * @param daysDeadlineClose
   * 			Plazo máximo en días para el cierre de la actividad antes del cierre automático 
   *           
   * @return true, if successful
   */
  public void notificarExternaAutoCerradaProximamenteByJob(ApplicationContext appContext, T servicio, ActividadExterna externa,
		  												   Integer daysDeadlineAdvise, Integer daysDeadlineClose) {
	  boolean isHTMLFormat = true;
	  String senderAddress = "";
	  String bccAddress = "", subject = "";
	  StringBuffer body = new StringBuffer();
 
	  Calendar cal = Calendar.getInstance(new Locale("es_ES"));
	  DateFormat df = new SimpleDateFormat("dd/MM/yyyy HH:mm");

	  try {
		  // Se obtiene el remitente 
		  senderAddress = getSenderAddress(servicio);
		  
		  //Si la actividad externa es cesión de aula o cesión de aula y servicio (tiene centro)
		  if (externa.getCentro() != null) {
			  // Se localizan los areamanger del centro al que pretende suscribirse el usuario
			  Map<String, Object> properties = new HashMap<String, Object>();
			  properties.put("centro.id", externa.getCentro().getId());
			  properties.put("puesto", Puesto.AREA_MANAGER);
			  
			  List<Empleado> empleadosAEnviar = servicio.findWhere(Empleado.class, properties);
			  
			  properties.put("puesto", Puesto.DIGITAL_ALLY_MANAGER);
			  empleadosAEnviar.addAll(servicio.findWhere(Empleado.class, properties));
	
			  for (Empleado e : empleadosAEnviar) {
				  if (e.getEmail() != null) {
					  bccAddress += e.getEmail() + ",";
					  
					  logger.log(Level.INFO, "NotificationService:notificarExternaAutoCerradaProximamenteByJob -> " +
							  				 "Notificación aviso de cierre automático de curso al empleado: " + e.getId());
				  }
			  }
		  }
		  
		  if (externa.getColaborador().getEmail() != null) {
			  bccAddress += externa.getColaborador().getEmail();
			  
			  logger.log(Level.INFO, "NotificationService:notificarExternaAutoCerradaProximamenteByJob -> " +
					  				 "Notificación aviso de cierre automático de curso al colaborador: " + externa.getColaborador().getId());
		  }

		  // Se elabora el mensaje
		  subject = C_NOTIFICACION + "Próximamente se cerrará automáticamente la actividad externa con identificador " + externa.getId();
		  String separador = ": ";
		      
		  List<Language> languages = servicio.findAllOrder(Language.class, "id", false);
		  for (Language l : languages) {
			  Locale languageLocale = new Locale(l.getISO().split("_")[0], l.getISO().split("_")[1]);
			  ActividadExternaLanguage ael = getNomenclatura(externa, l.getISO(), servicio);
		    	  
			  subject += separador + ael.getNombre();
			  separador = " / ";
		    	  
			  body.append("<br/>" +
					  	  "<strong>" + l.getName() + "</strong>" + "<br/><br/>" +
					  	  ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.cierreAutoExternaByJob.txt1") + " \"" + ael.getNombre() + "\" " +
					  	  ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.cierreAutoExternaByJob.txt2") + " " + externa.getId() + ", " +
					  	  ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.cierreAutoExternaAdviseByJob.txt3") + " " + daysDeadlineAdvise + " " +
					  	  ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.cierreAutoExternaAdviseByJob.txt4") + " " + daysDeadlineClose + " " +
					  	  ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.cierreAutoExternaAdviseByJob.txt5") +
					  	  "<br/><br/>" +
					  	  df.format(cal.getTime()) +
					  	  "<br/><br/>" +
					  	  "<b>" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EnviadoPorISIS") + "</b>" +
					  	  "<br/><br/>" +
					  	  "<img src='cid:imagenFirma' />" +
					  	  "<br/><br/><br/><br/>"	    		
			  );
		  }
		  
		  String pathImagenFirma = appContext.getResource("/images/logoXunta.png").getFile().getAbsolutePath();
		      
		  // Se envía el email
		  sendMessage(senderAddress, "", "", bccAddress, subject,
				  	  isHTMLFormat, body, pathImagenFirma, getMailserverParamSession(servicio));
	  }
	  catch (Exception e) {
		  e.printStackTrace();
	  }
  }
  
  /**
   * Notificar cerrado automático de curso por la tarea automática de rastreo.
   * 
   * @param servicio
   * 			servicio
   * 
   * @param externa
   * 			actividad externa 
   * 
   * @param daysDeadlineClose
   * 			Plazo máximo consumido para el cierre automático del curso.
   *           
   * @return true, if successful
   */
  public void notificarExternaAutoCerradaByJob(ApplicationContext appContext, T servicio, ActividadExterna externa, Integer daysDeadlineClose) {
	  boolean isHTMLFormat = true;
	  String senderAddress = "";
	  String bccAddress = "", subject = "";
	  StringBuffer body = new StringBuffer();
 
	  Calendar cal = Calendar.getInstance(new Locale("es_ES"));
	  DateFormat df = new SimpleDateFormat("dd/MM/yyyy HH:mm");

	  try {
		  // Se obtiene el remitente 
		  senderAddress = getSenderAddress(servicio);
		  
		  //Si la actividad externa es cesión de aula o cesión de aula y servicio (tiene centro)
		  if (externa.getCentro() != null) {
			  // Se localizan los areamanger del centro al que pretende suscribirse el usuario
			  Map<String, Object> properties = new HashMap<String, Object>();
			  properties.put("centro.id", externa.getCentro().getId());
			  properties.put("puesto", Puesto.AREA_MANAGER);
			  
			  List<Empleado> empleadosAEnviar = servicio.findWhere(Empleado.class, properties);
			  
			  properties.put("puesto", Puesto.DIGITAL_ALLY_MANAGER);
			  empleadosAEnviar.addAll(servicio.findWhere(Empleado.class, properties));
	
			  for (Empleado e : empleadosAEnviar) {
				  if (e.getEmail() != null){
					  bccAddress += e.getEmail() + ",";
					  
					  logger.log(Level.INFO, "NotificationService:notificarCursoAutoCerradoByJob -> " +
							  				 "Notificación cierre automático de curso al empleado: " + e.getId());
				  }
			  }
		  }
		  
		  if (externa.getColaborador().getEmail() != null) {
			  bccAddress += externa.getColaborador().getEmail();
			  
			  logger.log(Level.INFO, "NotificationService:notificarExternaAutoCerradaProximamenteByJob -> " +
					  				 "Notificación aviso de cierre automático de curso al colaborador: " + externa.getColaborador().getId());
		  }
		      
		  // Se elabora el mensaje
		  subject = C_NOTIFICACION + "Se ha cerrado automáticamente la actividad externa con identificador " + externa.getId();
		  String separador = ": ";
		      
		  List<Language> languages = servicio.findAllOrder(Language.class, "id", false);
		  for (Language l : languages) {
			  Locale languageLocale = new Locale(l.getISO().split("_")[0], l.getISO().split("_")[1]);
			  ActividadExternaLanguage ael = getNomenclatura(externa, l.getISO(), servicio);
		    	  
			  subject += separador + ael.getNombre();
			  separador = " / ";
		    	  
			  body.append("<br/>" +
					  	  "<strong>" + l.getName() + "</strong>" + "<br/><br/>" +
					  	  ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.cierreAutoExternaByJob.txt1") + " \"" + ael.getNombre() + "\" " +
					  	  ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.cierreAutoExternaByJob.txt2") + " " + externa.getId() + ", " +
					  	  ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.cierreAutoExternaByJob.txt3") + " " + daysDeadlineClose + " " +
					  	  ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.cierreAutoExternaByJob.txt4") +
					  	  "<br/><br/>" +
					  	  df.format(cal.getTime()) +
					  	  "<br/><br/>" +
					  	  "<b>" + ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", languageLocale).getString("mail.texto.EnviadoPorISIS") + "</b>" +
					  	  "<br/><br/>" +
					  	  "<img src='cid:imagenFirma' />" +
					  	  "<br/><br/><br/><br/>"	    		
			  );
		  }
		  
		  String pathImagenFirma = appContext.getResource("/images/logoXunta.png").getFile().getAbsolutePath();
		      
		  // Se envía el email
		  sendMessage(senderAddress, "", "", bccAddress, subject,
				  	  isHTMLFormat, body, pathImagenFirma, getMailserverParamSession(servicio));
	  }
	  catch (Exception e) {
		  e.printStackTrace();
	  }
  }
  
  private List<String> getMailserverParamSession(T servicio){
	  
	  List<String> mailSessionParams = new ArrayList<String>();
	  Parameter parameter = null;
	  String host="", useSSL="", port="", username="", password="";
	  
	  // Obtenemos el nombre del host SMTP.
      parameter = (Parameter)servicio.findOneWhere(Parameter.class, "nombre", "mail_session_smtp_host");
      if (parameter != null)
    	  host = parameter.getValor();
      
      // Obtenemos si se usa SSL con el servicio SMTP.
      parameter = (Parameter)servicio.findOneWhere(Parameter.class, "nombre", "mail_session_smtp_useSSL");
      if (parameter != null)
    	  useSSL = parameter.getValor();
      
	  // Obtenemos el puerto del servicio SMTP.
      parameter = (Parameter)servicio.findOneWhere(Parameter.class, "nombre", "mail_session_smtp_port");
      if (parameter != null)
    	  port = parameter.getValor();

      // Obtenemos el usuario del servicio SMTP.
      parameter = (Parameter)servicio.findOneWhere(Parameter.class, "nombre", "mail_session_smtp_username");
      if (parameter != null)
    	  username = parameter.getValor();

	  // Obtenemos la password del servicio SMTP.
      parameter = (Parameter)servicio.findOneWhere(Parameter.class, "nombre", "mail_session_smtp_password");
      if (parameter != null)
    	  password = parameter.getValor();
	  
      mailSessionParams.add(host);
      mailSessionParams.add(useSSL);
      mailSessionParams.add(port);
      mailSessionParams.add(username);
      mailSessionParams.add(password);
      
      return mailSessionParams;
  }

  private List<String> getMailserverParamSession(EntityManager em){
	  
	  List<String> mailSessionParams = new ArrayList<String>();
	  Parameter parameter = null;
	  String host="", useSSL="", port="", username="", password="";
	  
	  String sqlstr = "SELECT e FROM " + Parameter.class.getName() + " e WHERE " + " nombre = :paramName";
	  Query command = em.createQuery(sqlstr);
	  
	  // Obtenemos el nombre del host SMTP.
	  command.setParameter("paramName", "mail_session_smtp_host");
	  parameter = (Parameter)command.getSingleResult();
      if (parameter != null)
    	  host = parameter.getValor();

      // Obtenemos si se usa SSL con el servicio SMTP.
      command.setParameter("paramName", "mail_session_smtp_useSSL");
	  parameter = (Parameter)command.getSingleResult();
      if (parameter != null)
    	  useSSL = parameter.getValor();

	  // Obtenemos el puerto del servicio SMTP.
	  command.setParameter("paramName", "mail_session_smtp_port");
	  if (parameter != null)
    	  port = parameter.getValor();

	  // Obtenemos el usuario del servicio SMTP.
	  command.setParameter("paramName", "mail_session_smtp_username");
	  if (parameter != null)
    	  username = parameter.getValor();

	  // Obtenemos la password del servicio SMTP.
	  command.setParameter("paramName", "mail_session_smtp_password");
	  if (parameter != null)
    	  password = parameter.getValor();
      
      mailSessionParams.add(host);
      mailSessionParams.add(useSSL);
      mailSessionParams.add(port);
      mailSessionParams.add(username);
      mailSessionParams.add(password);      
      
      return mailSessionParams;
  }
  
  private CharlaLanguage getNomenclatura(Charla charla, String languageISO){
	  //Para sacar todos los campos de texto de la tabla CharlaLanguage
            
      for (CharlaLanguage cl :charla.getTextosCharla()) {
    	  if (cl.getLanguage().getISO().compareTo(languageISO) == 0)
    		  return cl;
      }
      
      return null;
  }
  
  private CursoLanguage getNomenclatura(Curso curso, String languageISO){
	  //Para sacar todos los campos de texto de la tabla CursoLanguage
	  
      for (CursoLanguage cl :curso.getTextosCurso()) {
    	  if (cl.getLanguage().getISO().compareTo(languageISO) == 0)
    		  return cl;
	  }
      
      return null;
  }
  
  /**
   * Para sacar todos los campos de texto de la tabla ActividadExternaLanguage
   * 
   * @param actividadExterna
   * @param languageISO
   * @param servicio
   * 
   * @return
   */
  private ActividadExternaLanguage getNomenclatura(ActividadExterna actividadExterna, String languageISO, T servicio){
	  if (actividadExterna.getId() != 0) {
		  Map<String, Object> properties = new HashMap<String, Object>();
		  properties.put("externa.id", actividadExterna.getId());
		  properties.put("language.ISO", languageISO);
		  
		  ActividadExternaLanguage ael = null;
		  
		  try {
			  ael = (ActividadExternaLanguage)servicio.findOneWhere(ActividadExternaLanguage.class, properties);
		  }
		  catch (Exception ex) {
			  return null;
		  }
		  
		  return ael;
	  }
	  else {
	      for (ActividadExternaLanguage cl :actividadExterna.getTextosExterna()) {
	    	  if (cl.getLanguage().getISO().compareTo(languageISO) == 0)
	    		  return cl;
	      }
	
	      return null;
	  }
  }
}
