package es.csa.fdt.cyldigital.notificacion.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.centros.entity.BP;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;

/**
 * Clase Alarma.
 * 
 * @author csa
 */
@Entity
@Table(name = "alarmas")
@Audited
@AttributeOverride(name = "id", column = @Column(name = "idAlarma"))
public class Alarma extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = 5721862807183200675L;

  /**
   * Enum FrecuenciaNotificacion.
   */
  public enum FrecuenciaNotificacion {
    CUANDO_CAMBIE,
    CINCO_MINUTOS,
    HORA,
    DIA,
    SEMANA,
    QUINCENA,
    MES,
    BIMENSUAL,
    TRIMESTRE,
    CUATRIMESTRE,
    SEMESTRE,
    ANIO
  }

  /**
   * Enum PeriodoBP.
   */
  public enum PeriodoBP {
    SEMANA,
    MES,
    ANIO
  }

  /**
   * Enum Operando.
   */
  public enum Operando {
    MAYOR,
    MENOR,
    MAYORIGUAL,
    MENORIGUAL,
    IGUAL,
    DIFERENTE
  }

  /**
   * Enum CondicionEvaluar.
   */
  public enum CondicionEvaluar {
    VALOR,
    BP,
    NIVELSUPERIOR,
    NIVELINFERIOR,
    PORCENTAJEBP
  }

  private String nombre;
  private String descripcion;
  private List<Empleado> empleados;
  private Centro centro;
  private FrecuenciaNotificacion frecuenciaNotificacion;
  private boolean notificarSiCambia;
  private Operando operando;
  private CondicionEvaluar condicionEvaluar;
  private int valorEvaluar;
  private BP.Tipo tipo;
  private PeriodoBP periodoBP;
  private Date fechaProximaEjecucion;
  private Date fechaEnvio;
  private Date horaEnvio;
  private double valorAnterior;

  /**
   * Construye un nuevo objeto de la clase
   */
  public Alarma() {
    super();
    empleados = new ArrayList<Empleado>();
  }

  /**
   * Devuelve nombre.
   * 
   * @return nombre
   */
  @NotNull
  public String getNombre() {
    return nombre;
  }

  /**
   * Asigna nombre.
   * 
   * @param nombre
   *          nombre
   */
  public void setNombre(String nombre) {
    this.nombre = nombre;
  }

  /**
   * Devuelve descripcion.
   * 
   * @return descripcion
   */
  @NotNull
  public String getDescripcion() {
    return descripcion;
  }

  /**
   * Asigna descripcion.
   * 
   * @param descripcion
   *          descripcion
   */
  public void setDescripcion(String descripcion) {
    this.descripcion = descripcion;
  }

  /**
   * Devuelve empleados.
   * 
   * @return empleados
   */
  @ManyToMany(targetEntity = Empleado.class, cascade = { javax.persistence.CascadeType.ALL })
  @JoinTable(name = "empleados_alarmas", joinColumns = @JoinColumn(name = "idAlarma"), inverseJoinColumns = @JoinColumn(name = "idUsuario"))
  @NotAudited
  @NotNull
  public List<Empleado> getEmpleados() {
    return empleados;
  }

  /**
   * Asigna empleados.
   * 
   * @param empleados
   *          empleados
   */
  public void setEmpleados(List<Empleado> empleados) {
    this.empleados = empleados;
  }

  /**
   * Devuelve centro.
   * 
   * @return centro
   */
  @ManyToOne(fetch = FetchType.LAZY)
  @JoinColumn(name = "idCentro")
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Devuelve frecuencia notificacion.
   * 
   * @return frecuencia notificacion
   */
  @NotNull
  public FrecuenciaNotificacion getFrecuenciaNotificacion() {
    return frecuenciaNotificacion;
  }

  /**
   * Asigna frecuencia notificacion.
   * 
   * @param frecuenciaNotificacion
   *          frecuencia notificacion
   */
  public void setFrecuenciaNotificacion(FrecuenciaNotificacion frecuenciaNotificacion) {
    this.frecuenciaNotificacion = frecuenciaNotificacion;
  }

  /**
   * Checks if is notificar si cambia.
   * 
   * @return true, if is notificar si cambia
   */
  public boolean isNotificarSiCambia() {
    return notificarSiCambia;
  }

  /**
   * Asigna notificar si cambia.
   * 
   * @param notificarSiCambia
   *          notificar si cambia
   */
  public void setNotificarSiCambia(boolean notificarSiCambia) {
    this.notificarSiCambia = notificarSiCambia;
  }

  /**
   * Devuelve condicion evaluar.
   * 
   * @return condicion evaluar
   */
  @NotNull
  public CondicionEvaluar getCondicionEvaluar() {
    return condicionEvaluar;
  }

  /**
   * Asigna condicion evaluar.
   * 
   * @param condicionEvaluar
   *          condicion evaluar
   */
  public void setCondicionEvaluar(CondicionEvaluar condicionEvaluar) {
    this.condicionEvaluar = condicionEvaluar;
  }

  /**
   * Devuelve valor evaluar.
   * 
   * @return valor evaluar
   */
  @NotNull
  public int getValorEvaluar() {
    return valorEvaluar;
  }

  /**
   * Asigna valor evaluar.
   * 
   * @param valorEvaluar
   *          valor evaluar
   */
  public void setValorEvaluar(int valorEvaluar) {
    this.valorEvaluar = valorEvaluar;
  }

  /**
   * Devuelve fecha proxima ejecucion.
   * 
   * @return fecha proxima ejecucion
   */
  public Date getFechaProximaEjecucion() {
    return fechaProximaEjecucion;
  }

  /**
   * Asigna fecha proxima ejecucion.
   * 
   * @param fechaProximaEjecucion
   *          fecha proxima ejecucion
   */
  public void setFechaProximaEjecucion(Date fechaProximaEjecucion) {
    this.fechaProximaEjecucion = fechaProximaEjecucion;
  }

  /**
   * Devuelve tipo.
   * 
   * @return tipo
   */
  @NotNull
  public BP.Tipo getTipo() {
    return tipo;
  }

  /**
   * Asigna tipo.
   * 
   * @param tipo
   *          tipo
   */
  public void setTipo(BP.Tipo tipo) {
    this.tipo = tipo;
  }

  /**
   * Devuelve periodo bp.
   * 
   * @return periodo bp
   */
  @NotNull
  public PeriodoBP getPeriodoBP() {
    return periodoBP;
  }

  /**
   * Asigna periodo bp.
   * 
   * @param periodoBP
   *          periodo bp
   */
  public void setPeriodoBP(PeriodoBP periodoBP) {
    this.periodoBP = periodoBP;
  }

  /**
   * Añade empleado.
   * 
   * @param empleado
   *           empleado
   */
  @Transient
  public void addEmpleado(Empleado empleado) {
    if (empleado != null && !getEmpleados().contains(empleado)) {
      empleados.add(empleado);
    }
  }

  /**
   * Quitar empleado.
   * 
   * @param empleado
   *           empleado
   */
  @Transient
  public void quitarEmpleado(Empleado empleado) {
    if (empleado != null && getEmpleados().contains(empleado)) {
      empleados.remove(empleado);
    }
  }

  /**
   * Devuelve operando.
   * 
   * @return operando
   */
  @NotNull
  public Operando getOperando() {
    return operando;
  }

  /**
   * Asigna operando.
   * 
   * @param operando
   *          operando
   */
  public void setOperando(Operando operando) {
    this.operando = operando;
  }

  /**
   * Devuelve valor anterior.
   * 
   * @return valor anterior
   */
  public double getValorAnterior() {
    return valorAnterior;
  }

  /**
   * Asigna valor anterior.
   * 
   * @param valorAnterior
   *          valor anterior
   */
  public void setValorAnterior(double valorAnterior) {
    this.valorAnterior = valorAnterior;
  }

  /**
   * Devuelve fecha envio.
   * 
   * @return fecha envio
   */
  @NotNull
  public Date getFechaEnvio() {
    return fechaEnvio;
  }

  /**
   * Asigna fecha envio.
   * 
   * @param fechaEnvio
   *          fecha envio
   */
  public void setFechaEnvio(Date fechaEnvio) {
    this.fechaEnvio = fechaEnvio;
  }

  /**
   * Devuelve hora envio.
   * 
   * @return hora envio
   */
  @NotNull
  public Date getHoraEnvio() {
    return horaEnvio;
  }

  /**
   * Asigna hora envio.
   * 
   * @param horaEnvio
   *          hora envio
   */
  public void setHoraEnvio(Date horaEnvio) {
    this.horaEnvio = horaEnvio;
  }
}
