package es.csa.fdt.cyldigital.notificacion.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import org.hibernate.envers.Audited;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;

/**
 * Clase Informe.
 * 
 * @author csa
 */
@Entity
@Table(name = "informes")
@Audited
@AttributeOverride(name = "id", column = @Column(name = "idInforme"))
public class Informe extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = 5721862807183200675L;

  private String nombre;
  private String descripcion;
  private String url;
  private String extension;
  private String textoMensaje;
  private boolean soloAdmin; // para que solo los admines los puedan modificar, son informes del sistema.
  private List<InformeNotification> informesnotifications;

  /**
   * Construye un nuevo objeto de la clase
   */
  public Informe() {
    super();
    setInformesnotifications(new ArrayList<InformeNotification>());
  }

  /**
   * Devuelve nombre.
   * 
   * @return nombre
   */
  public String getNombre() {
    return nombre;
  }

  /**
   * Asigna nombre.
   * 
   * @param nombre
   *          nombre
   */
  public void setNombre(String nombre) {
    this.nombre = nombre;
  }

  /**
   * Devuelve descripcion.
   * 
   * @return descripcion
   */
  public String getDescripcion() {
    return descripcion;
  }

  /**
   * Asigna descripcion.
   * 
   * @param descripcion
   *          descripcion
   */
  public void setDescripcion(String descripcion) {
    this.descripcion = descripcion;
  }

  /**
   * Devuelve url.
   * 
   * @return url
   */
  public String getUrl() {
    return url;
  }

  /**
   * Asigna url.
   * 
   * @param url
   *          url
   */
  public void setUrl(String url) {
    this.url = url;
  }

  /**
   * Devuelve informesnotifications.
   * 
   * @return informesnotifications
   */
  @OneToMany(mappedBy = "informe")
  public List<InformeNotification> getInformesnotifications() {
    return informesnotifications;
  }

  /**
   * Asigna informesnotifications.
   * 
   * @param informesnotifications
   *          informesnotifications
   */
  public void setInformesnotifications(List<InformeNotification> informesnotifications) {
    this.informesnotifications = informesnotifications;
  }

  /**
   * Devuelve extension.
   * 
   * @return extension
   */
  public String getExtension() {
    return extension;
  }

  /**
   * Asigna extension.
   * 
   * @param extension
   *          extension
   */
  public void setExtension(String extension) {
    this.extension = extension;
  }

  /**
   * Checks if is solo admin.
   * 
   * @return true, if is solo admin
   */
  public boolean isSoloAdmin() {
    return soloAdmin;
  }

  /**
   * Asigna solo admin.
   * 
   * @param soloAdmin
   *          solo admin
   */
  public void setSoloAdmin(boolean soloAdmin) {
    this.soloAdmin = soloAdmin;
  }

  /**
   * Devuelve texto mensaje.
   * 
   * @return texto mensaje
   */
  public String getTextoMensaje() {
    return textoMensaje;
  }

  /**
   * Asigna texto mensaje.
   * 
   * @param textoMensaje
   *          texto mensaje
   */
  public void setTextoMensaje(String textoMensaje) {
    this.textoMensaje = textoMensaje;
  }
}
