package es.csa.fdt.cyldigital.notificacion.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;

/**
 * Clase InformeNotification.
 * 
 * @author csa
 */
@Entity
@Table(name = "informesnotification")
@Audited
@AttributeOverride(name = "id", column = @Column(name = "idInformeNotification"))
public class InformeNotification extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = 5721862807183200675L;
  
  /** empleados. */
  protected List<Empleado> empleados;

  /**
   * Enum Periodicidad.
   */
  public enum Periodicidad {
    PUNTUAL,
    DIARIA,
    SEMANAL,
    QUINCENAL,
    MENSUAL,
    TRIMESTRAL,
    SEMESTRAL,
    ANUAL;
  }

  /**
   * Enum Periodo.
   */
  public enum Periodo {
    SEMANA_ACTUAL,
    MES_ACTUAL,
    MES_PASADO,
    ANIO_ACTUAL,
    ANIO_PASADO
  }

  private Date fecha;
  private Date hora;
  private Date fechaProximaEjecucion;
  private Informe informe;
  private Centro centro;
  private Periodicidad periodicidad;
  private Periodo periodo;

  /**
   * Construye un nuevo objeto de la clase
   */
  public InformeNotification() {
    super();
    empleados = new ArrayList<Empleado>();
  }

  /**
   * Devuelve fecha.
   * 
   * @return fecha
   */
  public Date getFecha() {
    return fecha;
  }

  /**
   * Asigna fecha.
   * 
   * @param fecha
   *          fecha
   */
  public void setFecha(Date fecha) {
    this.fecha = fecha;
  }

  /**
   * Devuelve informe.
   * 
   * @return informe
   */
  @ManyToOne(fetch = FetchType.LAZY)
  @JoinColumn(name = "idInforme")
  @NotNull
  public Informe getInforme() {
    return informe;
  }

  /**
   * Asigna informe.
   * 
   * @param informe
   *          informe
   */
  public void setInforme(Informe informe) {
    this.informe = informe;
  }

  /**
   * Devuelve centro.
   * 
   * @return centro
   */
  @ManyToOne(fetch = FetchType.LAZY)
  @JoinColumn(name = "idCentro")
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Devuelve empleados.
   * 
   * @return empleados
   */
  @ManyToMany(targetEntity = Empleado.class, cascade = { javax.persistence.CascadeType.REFRESH }, fetch = FetchType.LAZY)
  @JoinTable(name = "empleados_informenotification", joinColumns = @JoinColumn(name = "idInformeNotification"), inverseJoinColumns = @JoinColumn(name = "idUsuario"))
  @NotAudited
  @NotNull
  public List<Empleado> getEmpleados() {
    return empleados;
  }

  /**
   * Asigna empleados.
   * 
   * @param empleados
   *          empleados
   */
  public void setEmpleados(List<Empleado> empleados) {
    this.empleados = empleados;
  }

  /**
   * Devuelve periodicidad.
   * 
   * @return periodicidad
   */
  @NotNull
  public Periodicidad getPeriodicidad() {
    return periodicidad;
  }

  /**
   * Asigna periodicidad.
   * 
   * @param periodicidad
   *          periodicidad
   */
  public void setPeriodicidad(Periodicidad periodicidad) {
    this.periodicidad = periodicidad;
  }

  /**
   * Quitar empleado.
   * 
   * @param empleado
   *           empleado
   */
  @Transient
  public void quitarEmpleado(Empleado empleado) {
    if (empleado != null && getEmpleados().contains(empleado)) {
      empleados.remove(empleado);
    }
  }

  /**
   * Añade empleado.
   * 
   * @param empleado
   *           empleado
   */
  @Transient
  public void addEmpleado(Empleado empleado) {
    if (empleado != null && !getEmpleados().contains(empleado)) {
      empleados.add(empleado);
    }
  }

  /**
   * Devuelve hora.
   * 
   * @return hora
   */
  @Temporal(TemporalType.TIME)
  @NotNull
  public Date getHora() {
    return hora;
  }

  /**
   * Asigna hora.
   * 
   * @param hora
   *          hora
   */
  public void setHora(Date hora) {
    this.hora = hora;
  }

  /**
   * Devuelve fecha proxima ejecucion.
   * 
   * @return fecha proxima ejecucion
   */
  public Date getFechaProximaEjecucion() {
    return fechaProximaEjecucion;
  }

  /**
   * Asigna fecha proxima ejecucion.
   * 
   * @param fechaProximaEjecucion
   *          fecha proxima ejecucion
   */
  public void setFechaProximaEjecucion(Date fechaProximaEjecucion) {
    this.fechaProximaEjecucion = fechaProximaEjecucion;
  }

  /**
   * Devuelve periodo.
   * 
   * @return periodo
   */
  @NotNull
  public Periodo getPeriodo() {
    return periodo;
  }

  /**
   * Asigna periodo.
   * 
   * @param periodo
   *          periodo
   */
  public void setPeriodo(Periodo periodo) {
    this.periodo = periodo;
  }
}
