package es.csa.fdt.cyldigital.notificacion.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.calendario.entity.VacacionesCalendarItem;
import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.financiero.entity.Caja;
import es.csa.fdt.cyldigital.financiero.entity.Factura;
import es.csa.fdt.cyldigital.incidencias.entity.Incidencia;
import es.csa.fdt.cyldigital.recursos.entity.Recurso;
import es.csa.fdt.cyldigital.servicios.entity.Actividad;
import es.csa.fdt.cyldigital.tareas.entity.InformeTarea;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;

/**
 * Clase MailNotification.
 * 
 * @author csa
 */
@Entity
@Table(name = "mailnotifications")
@Audited
@AttributeOverride(name = "id", column = @Column(name = "idMailNotification"))
public class MailNotification extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = 5721862807183200675L;

  /**
   * Construye un nuevo objeto de la clase
   */
  public MailNotification() {
    super();
    usuarios = new ArrayList<Usuario>();
  }

  /**
   * Enum TipoElementoNotificado.
   */
  public enum TipoElementoNotificado {
    ACTIVIDAD,
    CAJA,
    PRESUPUESTO,
    ALARMA,
    INFORME,
    VACACIONES,
    TAREA,
    RECURSO,
    INCIDENCIA;
  }

  private Date fechaEnvio;
  private List<Usuario> usuarios; // usuario al que se ha enviado.
  private TipoElementoNotificado tipo;
  private String detallesEnvio; // se usa para diferenciar una suscripción de una desuscripción, etc.
  private Incidencia incidenciaAsociada;
  private Actividad actividadAsociada;
  private InformeTarea informeTareaAsociado;
  private VacacionesCalendarItem calendarItemAsociado;
  private Informe informeAsociado;
  private Alarma alarmaAsociada;
  private Factura facturaAsociada;
  private Caja cajaAsociada;
  private Recurso recursoAsociado;

  /**
   * Devuelve fecha envio.
   * 
   * @return fecha envio
   */
  @NotNull
  public Date getFechaEnvio() {
    return fechaEnvio;
  }

  /**
   * Asigna fecha envio.
   * 
   * @param fechaEnvio
   *          fecha envio
   */
  public void setFechaEnvio(Date fechaEnvio) {
    this.fechaEnvio = fechaEnvio;
  }

  /**
   * Devuelve tipo.
   * 
   * @return tipo
   */
  @NotNull
  public TipoElementoNotificado getTipo() {
    return tipo;
  }

  /**
   * Asigna tipo.
   * 
   * @param tipo
   *          tipo
   */
  public void setTipo(TipoElementoNotificado tipo) {
    this.tipo = tipo;
  }

  /**
   * Devuelve usuarios.
   * 
   * @return usuarios
   */
  @NotAudited
  @ManyToMany(fetch = FetchType.LAZY, targetEntity = Usuario.class)
  @JoinTable(name = "usuarios_mailnotifications", joinColumns = @JoinColumn(name = "idMailNotification"), inverseJoinColumns = @JoinColumn(name = "idUsuario"))
  public List<Usuario> getUsuarios() {
    return usuarios;
  }

  /**
   * Asigna usuarios.
   * 
   * @param usuarios
   *          usuarios
   */
  public void setUsuarios(List<Usuario> usuarios) {
    this.usuarios = usuarios;
  }

  /**
   * Devuelve detalles envio.
   * 
   * @return detalles envio
   */
  public String getDetallesEnvio() {
    return detallesEnvio;
  }

  /**
   * Asigna detalles envio.
   * 
   * @param detallesEnvio
   *          detalles envio
   */
  public void setDetallesEnvio(String detallesEnvio) {
    this.detallesEnvio = detallesEnvio;
  }

  /**
   * Devuelve incidencia asociada.
   * 
   * @return incidencia asociada
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idIncidencia")
  public Incidencia getIncidenciaAsociada() {
    return incidenciaAsociada;
  }

  /**
   * Asigna incidencia asociada.
   * 
   * @param incidenciaAsociada
   *          incidencia asociada
   */
  public void setIncidenciaAsociada(Incidencia incidenciaAsociada) {
    this.incidenciaAsociada = incidenciaAsociada;
  }

  /**
   * Devuelve actividad asociada.
   * 
   * @return actividad asociada
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idActividad")
  public Actividad getActividadAsociada() {
    return actividadAsociada;
  }

  /**
   * Asigna actividad asociada.
   * 
   * @param actividadAsociada
   *          actividad asociada
   */
  public void setActividadAsociada(Actividad actividadAsociada) {
    this.actividadAsociada = actividadAsociada;
  }

  /**
   * Devuelve informe tarea asociado.
   * 
   * @return informe tarea asociado
   */
  public InformeTarea getInformeTareaAsociado() {
    return informeTareaAsociado;
  }

  /**
   * Asigna informe tarea asociado.
   * 
   * @param informeTareaAsociado
   *          informe tarea asociado
   */
  public void setInformeTareaAsociado(InformeTarea informeTareaAsociado) {
    this.informeTareaAsociado = informeTareaAsociado;
  }

  /**
   * Devuelve calendar item asociado.
   * 
   * @return calendar item asociado
   */
  public VacacionesCalendarItem getCalendarItemAsociado() {
    return calendarItemAsociado;
  }

  /**
   * Asigna calendar item asociado.
   * 
   * @param calendarItemAsociado
   *          calendar item asociado
   */
  public void setCalendarItemAsociado(VacacionesCalendarItem calendarItemAsociado) {
    this.calendarItemAsociado = calendarItemAsociado;
  }

  /**
   * Devuelve informe asociado.
   * 
   * @return informe asociado
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idInforme")
  public Informe getInformeAsociado() {
    return informeAsociado;
  }

  /**
   * Asigna informe asociado.
   * 
   * @param informeAsociado
   *          informe asociado
   */
  public void setInformeAsociado(Informe informeAsociado) {
    this.informeAsociado = informeAsociado;
  }

  /**
   * Devuelve alarma asociada.
   * 
   * @return alarma asociada
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idAlarma")
  public Alarma getAlarmaAsociada() {
    return alarmaAsociada;
  }

  /**
   * Asigna alarma asociada.
   * 
   * @param alarmaAsociada
   *          alarma asociada
   */
  public void setAlarmaAsociada(Alarma alarmaAsociada) {
    this.alarmaAsociada = alarmaAsociada;
  }

  /**
   * Devuelve factura asociada.
   * 
   * @return factura asociada
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idFactura")
  public Factura getFacturaAsociada() {
    return facturaAsociada;
  }

  /**
   * Asigna factura asociada.
   * 
   * @param facturaAsociada
   *          factura asociada
   */
  public void setFacturaAsociada(Factura facturaAsociada) {
    this.facturaAsociada = facturaAsociada;
  }

  /**
   * Devuelve caja asociada.
   * 
   * @return caja asociada
   */
  public Caja getCajaAsociada() {
    return cajaAsociada;
  }

  /**
   * Asigna caja asociada.
   * 
   * @param cajaAsociada
   *          caja asociada
   */
  public void setCajaAsociada(Caja cajaAsociada) {
    this.cajaAsociada = cajaAsociada;
  }

  /**
   * Devuelve recurso asociado.
   * 
   * @return recurso asociado
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idRecurso")
  public Recurso getRecursoAsociado() {
    return recursoAsociado;
  }

  /**
   * Asigna recurso asociado.
   * 
   * @param recursoAsociado
   *          recurso asociado
   */
  public void setRecursoAsociado(Recurso recursoAsociado) {
    this.recursoAsociado = recursoAsociado;
  }
}
