package es.csa.fdt.cyldigital.notificacion.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.notificacion.entity.Alarma;

/**
 * Clase AlarmaServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class AlarmaServiceImpl extends AbstractServiceImpl<Alarma, Long> implements AlarmaService {

  private static final long serialVersionUID = -5575228180361917674L;

  /**
   * Construye un nuevo objeto de la clase
   */
  public AlarmaServiceImpl() {
    super();
  }

  private List<Centro> centros;

  /**
   * Devuelve centros.
   * 
   * @return centros
   */
  public List<Centro> getCentros() {
    if (centros == null) centros = findAllOrder(Centro.class, "nombre", true);
    return centros;
  }

  /**
   * Persist alarma.
   * 
   * @param a
   *           a
   * @return true, if successful
   */
  public boolean persistAlarma(Alarma a) {
    // fecha envío
    DateFormat dy = new SimpleDateFormat("yyyy");
    DateFormat dM = new SimpleDateFormat("MM");
    DateFormat dd = new SimpleDateFormat("dd");
    DateFormat dh = new SimpleDateFormat("HH");
    DateFormat dm = new SimpleDateFormat("mm");
    DateFormat def = new SimpleDateFormat("yyyy/MM/dd HH:mm");
    Date f = a.getFechaEnvio();
    Date h = a.getHoraEnvio();
    try {
      a.setFechaProximaEjecucion(def.parse(dy.format(f) + "/" + dM.format(f) + "/" + dd.format(f) + " " + dh.format(h)
          + ":" + dm.format(h)));
    } catch (ParseException e) {
      e.printStackTrace();
    }

    if (a.getEmpleados().isEmpty()) {
      addErrorMessage("alarma.error_noempleados");
      return false;
    } else if (a.getFechaProximaEjecucion().before(new Date())) {
      addErrorMessage("alarma.error_fechaanterior");
      return false;
    } else {
      persist(a);
      return true;
    }
  }

  /**
   * Elimina alarma.
   * 
   * @param a
   *           a
   */
  public void removeAlarma(Alarma a) {
    if (a.getId() != 0) {
      a.setDesactivado(true);
      disableMessages();
      persist(a);
      enableMessages();
      addInfoMessage("alarma.borrada");
    }
  }
}
