package es.csa.fdt.cyldigital.notificacion.service;

import java.util.ArrayList;
import java.util.HashMap;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.List;
import java.util.Map;

import javax.faces.model.SelectItem;

import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.language.entity.ServicioLanguage;
import es.csa.fdt.cyldigital.language.entity.TematicaLanguage;
import es.csa.fdt.cyldigital.notificacion.entity.Informe;
import es.csa.fdt.cyldigital.notificacion.entity.InformeNotification;
import es.csa.fdt.cyldigital.servicios.entity.Servicio;
import es.csa.fdt.cyldigital.servicios.entity.Tematica;

/**
 * Clase InformeNotificationServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class InformeNotificationServiceImpl extends AbstractServiceImpl<InformeNotification, Long> implements
    InformeNotificationService {

  private static final long serialVersionUID = -5575228180361917674L;
  
  private List<Servicio> servicios;
  
  private List<ServicioLanguage> serviciosLanguage;
  
  private List<SelectItem> serviciosSelectItems;
  
  private List<Tematica> tematicasMultiple;

  /**
   * Construye un nuevo objeto de la clase
   */
  public InformeNotificationServiceImpl() {
    super();
    
    tematicasMultiple = new ArrayList<Tematica>();
  }

  private List<Centro> centros;
  private List<Informe> informes;
  

  /**
   * Devuelve centros.
   * 
   * @return centros
   */
  public List<Centro> getCentros() {
    if (centros == null) centros = findAllOrder(Centro.class, "nombre", true);
    return centros;
  }

	/**
	* Devuelve informes.
	* 
	* @return informes
	*/
	public List<Informe> getInformes() {
	   if (informes == null) informes = findAll(Informe.class);
	   return informes;
	}
	
	public List<Servicio> getServicios() {
		if (servicios == null)
			servicios = new ArrayList<Servicio>();
		  
		if (!servicios.isEmpty())
			servicios.clear();
		
		List<ServicioLanguage> serviciosLanguage = getServiciosLanguage();

		for(ServicioLanguage servicioLanguage : serviciosLanguage)
			servicios.add(servicioLanguage.getServicio());
		
		return servicios;
	}

	public List<ServicioLanguage> getServiciosLanguage() {
		if (serviciosLanguage == null)
			serviciosLanguage = new ArrayList<ServicioLanguage>();
		
		if (!serviciosLanguage.isEmpty())
			serviciosLanguage.clear();
		
		String idioma = LocaleContextHolder.getLocaleContext().getLocale().toString();
		Map<String, Object> propiedades = new HashMap<String, Object>();
		  
		propiedades.put("language.ISO", idioma);
		propiedades.put("servicio.desactivado", false);
		  
		serviciosLanguage.addAll(findWhereOrder(ServicioLanguage.class, propiedades, "nombre", true));
		
		return serviciosLanguage;
	}

	public List<SelectItem> getServiciosSelectItems() {
		if (serviciosSelectItems == null)
			serviciosSelectItems = new ArrayList<SelectItem>();
		  
		if (!serviciosSelectItems.isEmpty())
			serviciosSelectItems.clear();
		  
		Map<String, Object> propiedades = new HashMap<String, Object>();
		String idioma = LocaleContextHolder.getLocaleContext().getLocale().toString();
		
		propiedades.put("language.ISO", idioma);
		propiedades.put("servicio.desactivado", false);
		
		if(tematicasMultiple.size() > 0) {
			for(Tematica t : tematicasMultiple) {
				propiedades.put("servicio.tematica", t);
				
				List<ServicioLanguage> serviciosLanguage = findWhereOrder(ServicioLanguage.class, propiedades, "nombre", true);
				
				for (ServicioLanguage servicioLanguage : serviciosLanguage) {
					serviciosSelectItems.add(new SelectItem(servicioLanguage, servicioLanguage.getNombre()));
				}
			}
		} else if(tematicasMultiple.size() == 0) { 
			List<ServicioLanguage> serviciosLanguage = findWhereOrder(ServicioLanguage.class, propiedades, "nombre", true);
		
			for (ServicioLanguage servicioLanguage : serviciosLanguage) {
				serviciosSelectItems.add(new SelectItem(servicioLanguage, servicioLanguage.getNombre()));
			}
		}
		
		return serviciosSelectItems;
	}

	public List<Tematica> getTematicasMultiple() {
		return tematicasMultiple;
	}

	public void setTematicasMultiple(List<Tematica> tematicasMultiple) {
		this.tematicasMultiple = tematicasMultiple;
	}

/**
   * Persist informe notification.
   * 
   * @param i
   *           i
   * @return true, if successful
   */
  public boolean persistInformeNotification(InformeNotification i) {
    if (i.getEmpleados().isEmpty()) {
      addErrorMessage("informeNotification.error_noempleados");
      return false;
    } else {
      i.setFechaProximaEjecucion(i.getFecha());
      persist(i);
      return true;
    }
  }

  /**
   * Elimina informe notification.
   * 
   * @param i
   *           i
   */
  public void removeInformeNotification(InformeNotification i) {
    if (i.getId() != 0) {
      i.setDesactivado(true);
      disableMessages();
      persist(i);
      enableMessages();
      addInfoMessage("informeNotification.borrado");
    }
  }

}
