package es.csa.fdt.cyldigital.notificacion.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import org.springframework.binding.message.MessageBuilder;
import org.springframework.binding.message.MessageContext;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;
import org.springframework.webflow.execution.RequestContextHolder;

import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.commons.persistence.validator.NIFUtils;
import es.csa.fdt.cyldigital.core.service.BrowserFilteredByCenter;
import es.csa.fdt.cyldigital.language.entity.ServicioLanguage;
import es.csa.fdt.cyldigital.notificacion.entity.InformeNotification;
import es.csa.fdt.cyldigital.servicios.entity.Tematica;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL.Sexo;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL.TipoDiscapacidad;

/**
 * Servicio del Browser de Informes de Notificación.
 * 
 * @author csa
 */
@Service
@Repository
public class InformesNotificationsBrowserImpl extends BrowserFilteredByCenter<InformeNotification> implements
    InformesNotificationsBrowser<CriterionJPAQL, String> {

  private static final long serialVersionUID = -7227366052245717441L;
  
  /*private String fechaInicioLegible;
  private String fechaFinLegible;*/
  
  //3.4.15-R - RF_15_21  
  private List<Tematica> tematicasMultiple;
  
  private ServicioLanguage servicio;
  
  private Date fechaInicioInforme;
  
  private Date fechaFinInforme;
  
  private String realizada;
  
  private String iniciativaSingular;
  
  private String videoconferencia;
  
  private String identificacion;
  
  private String edadX;
  
  private String edadY;
  
  private Sexo sexo;
  
  private TipoDiscapacidad adaptacion;
  
  private EReportType reportType;
  
  private EReportDatosTypes reportDatosType;
  
  private EActivityTypes activityTypes;
  
  private String[] fieldsXerais;
  
  private List<String> fieldsXeraisSelected;
  
  private String[] fieldsEstado;
  
  private List<String> fieldsEstadoSelected;
  
  private String[] fieldsParticipacion;
  
  private List<String> fieldsParticipacionSelected;
  
  private String[] fieldsIndicadores;
  
  private List<String> fieldsIndicadoresSelected;

  private String[] fieldsIndicadoresParticipacion;
  
  private List<String> fieldsIndicadoresParticipacionSelected;
  
  private List<String> fieldsSelected;
  
  private String orden;

  
  public enum EReportType {
	  DATOS,
	  INDICADORES;
  }
  
  public enum EReportDatosTypes {
	  ACTIVIDAD,
	  //USUARIO,
	  ACTIVIDADUSUARIO;
  }
  
  public enum EActivityTypes {
	  TODOS,
	  CURSO,
	  CHARLA,
	  AAEE;
  }  
  
  /**
   * Construye un nuevo objeto de la clase
   */
  public InformesNotificationsBrowserImpl() {
    super();

    setFechaInicioInforme(null);
    setFechaFinInforme(null);
    setReportType(EReportType.DATOS);
    setReportDatosType(EReportDatosTypes.ACTIVIDAD);
    setActivityTypes(EActivityTypes.TODOS);
    
    tematicasMultiple = new ArrayList<Tematica>();
    fieldsSelected = new ArrayList<String>();
    fieldsXeraisSelected = new ArrayList<String>(Arrays.asList(getFieldsXerais()));
    fieldsEstadoSelected = new ArrayList<String>(Arrays.asList(getFieldsEstado()));
    fieldsParticipacionSelected = new ArrayList<String>(Arrays.asList(getFieldsParticipacion()));
    fieldsIndicadoresSelected = new ArrayList<String>(Arrays.asList(getFieldsIndicadores()));
    fieldsIndicadoresParticipacionSelected = new ArrayList<String>(Arrays.asList(getFieldsIndicadoresParticipacion()));
  }
	
  	/**
	 * @return the reportType
	 */
	public EReportType getReportType() {
		return reportType;
	}
	
	/**
	 * @param reportType the reportType to set
	 */
	public void setReportType(EReportType reportType) {
		this.reportType = reportType;
	}
	
	/**
	 * @return the reportDatosType
	 */
	public EReportDatosTypes getReportDatosType() {
		return reportDatosType;
	}

	/**
	 * @param reportDatosType the reportDatosType to set
	 */
	public void setReportDatosType(EReportDatosTypes reportDatosType) {
		this.reportDatosType = reportDatosType;
	}
	
	/**
	 * @return the activityTypes
	 */
	public EActivityTypes getActivityTypes() {
		return activityTypes;
	}
	
	/**
	 * @param activityTypes the activityTypes to set
	 */
	public void setActivityTypes(EActivityTypes activityTypes) {
		this.activityTypes = activityTypes;
	}

	/**
	 * 
	 * @return the tematicasMultiple
	 */
	public List<Tematica> getTematicasMultiple() {
		return tematicasMultiple;
	}
	
	/**
	 * 
	 * @return the tematicasMultiple ids as string 
	 */
	public String getTematicasMultipleAsString() {
		String tematicas = "";
		if(getTematicasMultiple().size() > 0) {
			for(int i= 0; i < getTematicasMultiple().size(); i++) {
				Tematica t = getTematicasMultiple().get(i);
				tematicas += t.getId();
				if(i < getTematicasMultiple().size()-1) tematicas += ",";
			}
		}
		
		return tematicas;
	}

	/**
	 * 
	 * @param tematicasMultiple the tematicasMultiple to set
	 */
	public void setTematicasMultiple(List<Tematica> tematicasMultiple) {
		this.tematicasMultiple = tematicasMultiple;
	}

	/**
	 * 
	 * @return the servicio
	 */
	public ServicioLanguage getServicio() {
		return servicio;
	}

	/**
	 * 
	 * @param servicio the servicio to set
	 */
	public void setServicio(ServicioLanguage servicio) {
		this.servicio = servicio;
	}

	/**
	 * @return the fechaInicioInforme
	 */
	public Date getFechaInicioInforme() {
		return fechaInicioInforme;
	}

	/**
	 * @param fechaInicioInforme the fechaInicioInforme to set
	 */
	public void setFechaInicioInforme(Date fechaInicioInforme) {
		this.fechaInicioInforme = fechaInicioInforme;
	}
	
	/**
	 * @return the fechaFinInforme
	 */
	public Date getFechaFinInforme() {
		return fechaFinInforme;
	}
	
	/**
	 * @param fechaFinInforme the fechaFinInforme to set
	 */
	public void setFechaFinInforme(Date fechaFinInforme) {
		this.fechaFinInforme = fechaFinInforme;
	}
	
	/**
	 * Devuelve fecha inicio legible.
	 * 
	 * @return fecha inicio legible
	 */
	public String getFechaInicioLegible() {
	    String aux;
	    
	    if (getFechaInicioInforme() != null) {
	      DateFormat df = new SimpleDateFormat("dd/MM/yyyy");
	      
	      aux = df.format(getFechaInicioInforme()) + " 00:00:00";
	      
	      return aux;
	    }

	    return "";
	}
	
	/**
	 * Devuelve fecha fin legible.
	 * 
	 * @return fecha fin legible
	 */
	public String getFechaFinLegible() {
	    String aux;
	    
	    if (getFechaFinInforme() != null) {
	      DateFormat df = new SimpleDateFormat("dd/MM/yyyy");
	      
	      aux = df.format(getFechaFinInforme()) + " 23:59:59";
	      
	      return aux;
	    }
	    
	    return "";
	}

	public String getRealizada() {
		return realizada;
	}

	public void setRealizada(String realizada) {
		this.realizada = realizada;
	}

	public String getIniciativaSingular() {
		return iniciativaSingular;
	}

	public void setIniciativaSingular(String iniciativaSingular) {
		this.iniciativaSingular = iniciativaSingular;
	}
	
	public String getVideoconferencia() {
		return videoconferencia;
	}

	public void setVideoconferencia(String videoconferencia) {
		this.videoconferencia = videoconferencia;
	}
	
	public String getIdentificacion() {
		return identificacion;
	}

	public void setIdentificacion(String identificacion) {
		this.identificacion = identificacion;
	}

	public String getEdadX() {
		return edadX;
	}

	public void setEdadX(String edadX) {
		this.edadX = edadX;
	}

	public String getEdadY() {
		return edadY;
	}

	public void setEdadY(String edadY) {
		this.edadY = edadY;
	}
	
	public Sexo getSexo() {
		return sexo;
	}

	public void setSexo(Sexo sexo) {
		this.sexo = sexo;
	}

	public TipoDiscapacidad getAdaptacion() {
		return adaptacion;
	}

	public void setAdaptacion(TipoDiscapacidad adaptacion) {
		this.adaptacion = adaptacion;
	}
	
	/**
	 * Devuelve la lista los campos xerais
	 * @return lista los campos para el genrador de informes
	 */
	public String[] getFieldsXerais() {
		if(fieldsXerais == null)
			fieldsXerais = new String[16];
		
		fieldsXerais[0] = "ID"; 	fieldsXerais[1] = "TIPO"; 	fieldsXerais[2] = "TEMR"; 	fieldsXerais[3] = "TEMA"; 	
		fieldsXerais[4] = "SERV";	fieldsXerais[5] = "ALCU"; 	fieldsXerais[6] = "CENT"; 	fieldsXerais[7] = "PROV"; 
		fieldsXerais[8] = "PRZTOT";	fieldsXerais[9] = "FI"; 	fieldsXerais[10] = "FN"; 	fieldsXerais[11] = "HI"; 	
		fieldsXerais[12] = "HF";	fieldsXerais[13] = "DRTOT"; fieldsXerais[14] = "ISING";	fieldsXerais[15] = "VIDEO";
		
		return fieldsXerais;
	}

	/**
	 * Devuelve la lista los campos estado
	 * @return lista los campos para el generador de informes
	 */
	public String[] getFieldsEstado() {
		if(fieldsEstado == null)
			fieldsEstado = new String[6];
		
		fieldsEstado[0] = "DRPER";	fieldsEstado[1] = "NMAT"; 	fieldsEstado[2] = "NINS"; 	
		fieldsEstado[3] = "NAS"; 	fieldsEstado[4] = "EST"; 	fieldsEstado[5] = "RLZD"; 	
		
		return fieldsEstado;
	}

	/**
	 * Devuelve la lista los campos participacion
	 * @return lista los campos para el generador de informes
	 */
	public String[] getFieldsParticipacion() {
		if(fieldsParticipacion == null)
			fieldsParticipacion = new String[14];
		
		fieldsParticipacion[0] = "USER"; 	fieldsParticipacion[1] = "NOMB"; 	fieldsParticipacion[2] = "APE1"; 	
		fieldsParticipacion[3] = "APE2";	fieldsParticipacion[4] = "SEX"; 	fieldsParticipacion[5] = "EDAD";
		fieldsParticipacion[6] = "ADAPT";	fieldsParticipacion[7] = "FORM"; 	fieldsParticipacion[8] = "OCUP";
		fieldsParticipacion[9] = "ASIS"; 	fieldsParticipacion[10] = "HTPO";	fieldsParticipacion[11] = "HASI";
		fieldsParticipacion[12] = "HJUS";   fieldsParticipacion[13] = "HORP";
		
		return fieldsParticipacion;
	}
	
	/**
	 * Devuelve la lista los campos indicadores 
	 * @return lista los campos para el generador de informes
	 */
	public String[] getFieldsIndicadores() {
		if(fieldsIndicadores == null)
			fieldsIndicadores = new String[7];
 
		fieldsIndicadores[0] = "TIPO";
		fieldsIndicadores[1] = "PROV"; 
		fieldsIndicadores[2] = "CENT";
		fieldsIndicadores[3] = "TEMA";
		fieldsIndicadores[4] = "TPLX";
		fieldsIndicadores[5] = "ISING";
		fieldsIndicadores[6] = "VIDEO";
		
		return fieldsIndicadores;
	}
	
	/**
	 * Devuelve la lista los campos indicadores 
	 * @return lista los campos para el generador de informes
	 */
	public String[] getFieldsIndicadoresParticipacion() {
		if(fieldsIndicadoresParticipacion == null)
			fieldsIndicadoresParticipacion = new String[5];

		fieldsIndicadoresParticipacion[0] = "EDAD";
		fieldsIndicadoresParticipacion[1] = "SEXO";
		fieldsIndicadoresParticipacion[2] = "ADAP";
		fieldsIndicadoresParticipacion[3] = "FORM"; 	
		fieldsIndicadoresParticipacion[4] = "OCUP";
		
		return fieldsIndicadoresParticipacion;
	}
	
	/**
	 * Devuelve la lista de los campos xerais seleccionados 
	 * @return lista los campos para el generador de informes
	 */
	public List<String> getFieldsXeraisSelected() {
		return fieldsXeraisSelected;
	}

	/**
	 * 
	 * @param fieldsXeraisSelected
	 */
	public void setFieldsXeraisSelected(List<String> fieldsXeraisSelected) {
		this.fieldsXeraisSelected = fieldsXeraisSelected;
	}
	
	/**
	 * Devuelve la lista de los campos estado seleccionados 
	 * @return lista los campos para el generador de informes
	 */
	public List<String> getFieldsEstadoSelected() {
		return fieldsEstadoSelected;
	}
	
	/**
	 * 
	 * @param fieldsEstadoSelected
	 */
	public void setFieldsEstadoSelected(List<String> fieldsEstadoSelected) {
		this.fieldsEstadoSelected = fieldsEstadoSelected;
	}

	/**
	 * Devuelve la lista de los campos participacion seleccionados 
	 * @return lista los campos para el generador de informes
	 */ 
	public List<String> getFieldsParticipacionSelected() {
		return fieldsParticipacionSelected;
	}

	/**
	 * 
	 * @param fieldsParticipacionSelected
	 */
	public void setFieldsParticipacionSelected(List<String> fieldsParticipacionSelected) {
		this.fieldsParticipacionSelected = fieldsParticipacionSelected;
	}

	/**
	 * Devuelve la lista de los campos indicadores seleccionados 
	 * @return lista los campos para el generador de informes
	 */
	public List<String> getFieldsIndicadoresSelected() {
		return fieldsIndicadoresSelected;
	}
	
	/**
	 * 
	 * @param fieldsIndicadoresSelected
	 */
	public void setFieldsIndicadoresSelected(List<String> fieldsIndicadoresSelected) {
		this.fieldsIndicadoresSelected = fieldsIndicadoresSelected;
	}

	/**
	 * Devuelve la lista de los campos indicadores seleccionados 
	 * @return lista los campos para el generador de informes
	 */
	public List<String> getFieldsIndicadoresParticipacionSelected() {
		return fieldsIndicadoresParticipacionSelected;
	}
	
	/**
	 * 
	 * @param fieldsIndicadoresParticipacionSelected
	 */
	public void setFieldsIndicadoresParticipacionSelected(List<String> fieldsIndicadoresParticipacionSelected) {
		this.fieldsIndicadoresParticipacionSelected = fieldsIndicadoresParticipacionSelected;
	}
	
	public List<String> getFieldsSelected() {
		if(this.fieldsSelected.size() > 0)
			this.fieldsSelected.clear();
		
		if(this.getReportType().equals(EReportType.DATOS)){
			this.fieldsSelected.addAll(this.fieldsXeraisSelected);
			if(this.reportDatosType.equals(EReportDatosTypes.ACTIVIDAD)) 
				this.fieldsSelected.addAll(this.fieldsEstadoSelected);
			else if(this.reportDatosType.equals(EReportDatosTypes.ACTIVIDADUSUARIO)) 
				this.fieldsSelected.addAll(this.fieldsParticipacionSelected);	
		} else {
			this.fieldsSelected.addAll(this.fieldsIndicadoresSelected);
			if(this.reportDatosType.equals(EReportDatosTypes.ACTIVIDADUSUARIO))
				this.fieldsSelected.addAll(this.fieldsIndicadoresParticipacionSelected);
		}
		
		return fieldsSelected;
	}

	public String getOrden() {
		return orden;
	}

	public void setOrden(String orden) {
		this.orden = orden;
	}

	/**
	 * Comprueba si la fecha de inicio del informe es anterior a la fecha de fin.
	 * Además en el caso de no ser así añade mensaje de error.
	 * 
	 * @reurn True si la fecha de inicio es anterior o a la fecha de fin. False en otro caso.4
	 */
	public Boolean isFechasInformeCorrectas() {
		Boolean success = false;
		
		if (!(success = getFechaInicioInforme().before(getFechaFinInforme()))) {
			MessageContext context = RequestContextHolder.getRequestContext().getMessageContext();
			context.addMessage(new MessageBuilder().error().defaultText("reports.validator.fechaInicioAnteriorFechaFin").code("reports.validator.fechaInicioAnteriorFechaFin").args(null).build());
		}

		return success;
	}
	
	/**
	 * Comprueba todos los datos del formulario
	 * 
	 * @reurn True si la fecha de inicio es anterior o a la fecha de fin. False en otro caso.4
	 */
	public Boolean isInformeCorrecto() {
		Boolean success = false;
		
		MessageContext context = RequestContextHolder.getRequestContext().getMessageContext();

		if(getFechaInicioInforme() == null && getFechaFinInforme() == null)
			context.addMessage(new MessageBuilder().error().defaultText("reports.validator.fechasPeriodo").code("reports.validator.fechasPeriodo").args(null).build());
		else if(getFechaInicioInforme() != null && getFechaFinInforme() == null) 
			context.addMessage(new MessageBuilder().error().defaultText("reports.validator.fechaFinNull").code("reports.validator.fechaFinNull").args(null).build());
		else if(getFechaInicioInforme() == null && getFechaFinInforme() != null)
			context.addMessage(new MessageBuilder().error().defaultText("reports.validator.fechaInicioNull").code("reports.validator.fechaInicioNull").args(null).build());
		else if( (getFechaInicioInforme() != null && getFechaFinInforme() != null) && !(getFechaInicioInforme().before(getFechaFinInforme())) )
			context.addMessage(new MessageBuilder().error().defaultText("reports.validator.fechaInicioAnteriorFechaFin").code("reports.validator.fechaInicioAnteriorFechaFin").args(null).build());
		else if ( (getIdentificacion() != null) && (!getIdentificacion().isEmpty()) && !NIFUtils.isValid(getIdentificacion().toUpperCase()))
			context.addMessage(new MessageBuilder().error().defaultText("usuario.error.dni").code("usuario.error.dni").args(null).build());
		else if(getReportDatosType().equals(EReportDatosTypes.ACTIVIDADUSUARIO)) {
			if(!getEdadX().isEmpty() && getEdadY().isEmpty())
					context.addMessage(new MessageBuilder().error().defaultText("reports.validator.rangoEdadVacio").code("reports.validator.rangoEdadVacio").args(null).build());
			else if(getEdadX().isEmpty() && !getEdadY().isEmpty())
				context.addMessage(new MessageBuilder().error().defaultText("reports.validator.rangoEdadVacio").code("reports.validator.rangoEdadVacio").args(null).build());
			else if((!getEdadX().isEmpty() && !getEdadY().isEmpty()) && (Integer.valueOf(getEdadX()) > Integer.valueOf(getEdadY())))
				context.addMessage(new MessageBuilder().error().defaultText("reports.validator.rangoEdadXAnteriorEdadY").code("reports.validator.rangoEdadXAnteriorEdadY").args(null).build());
			else
				success = true;
		} else
			success = true;
		
		return success;
	}
}
