package es.csa.fdt.cyldigital.notificacion.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.commons.persistence.browser.jpaql.FilterJPAQLByQuery;
import es.csa.fdt.cyldigital.core.service.BrowserFilteredByActive;
import es.csa.fdt.cyldigital.notificacion.entity.MailNotification;
import es.csa.fdt.cyldigital.notificacion.entity.MailNotification.TipoElementoNotificado;

/**
 * Servicio del Browser de notificaciones por Mail.
 * 
 * @author csa
 */
@Service
@Repository
public class MailNotificationsBrowserImpl extends BrowserFilteredByActive<MailNotification> implements
    MailNotificationsBrowser<CriterionJPAQL, String> {

  private static final long serialVersionUID = -7227366052245717441L;

  /**
   * Construye un nuevo objeto de la clase
   */
  public MailNotificationsBrowserImpl() {
    super();
  }

  /** tipo. */
  public TipoElementoNotificado tipo;
  
  /** custom filter. */
  public String customFilter;

  /**
   * Devuelve tipo.
   * 
   * @return tipo
   */
  public TipoElementoNotificado getTipo()
  {
    return tipo;
  }
  
  /**
   * Asigna tipo.
   * 
   * @param tipo
   *          tipo
   */
  public void setTipo(TipoElementoNotificado tipo)
  {
    this.tipo=tipo;
  }
  
  /**
   * Devuelve custom filter.
   * 
   * @return custom filter
   */
  public String getCustomFilter()
  {
    return customFilter;
  }
  
  /**
   * Asigna custom filter.
   * 
   * @param customFilter
   *          custom filter
   */
  public void setCustomFilter(String customFilter)
  {
    this.customFilter=customFilter;
  }
  
  /**
   * Filter tipo.
   */
  public void filterTipo() {

    removeFilterTipo();
    if (getTipo() != null) {
      filterByTipo(getTipo());
    }
  }

  private void removeFilterTipo() {
    int i = -1;
    for (Filter<CriterionJPAQL, MailNotification> f : getFilters())
      if (f.getCriterion().getQuery().contains("tipo")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
  }

  /**
   * Filter by tipo.
   * 
   * @param tipo
   *           tipo
   */
  public void filterByTipo(TipoElementoNotificado tipo) {
    String query = "";

    boolean flag = false;
    FilterJPAQLByQuery<MailNotification> filter = new FilterJPAQLByQuery<MailNotification>();

    query += " tipo = ? ";
    filter.getCriterion().addParameter(tipo);

    for (Filter<CriterionJPAQL, MailNotification> f : getFilters())
      if (f.getCriterion().getQuery().contains("tipo")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }
  
  /**
   * Filter id.
   */
  public void filterId() {

    removeFilterId();
    if (getCustomFilter() != null) {
      filterById();
    }
  }

  private void removeFilterId() {
    int i = -1;
    for (Filter<CriterionJPAQL, MailNotification> f : getFilters())
      if (f.getCriterion().getQuery().contains("incidenciaAsociada")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
    
    i = -1;
    for (Filter<CriterionJPAQL, MailNotification> f : getFilters())
      if (f.getCriterion().getQuery().contains("actividadAsociada")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
    
    i = -1;
    for (Filter<CriterionJPAQL, MailNotification> f : getFilters())
      if (f.getCriterion().getQuery().contains("informeAsociada")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
    
    i = -1;
    for (Filter<CriterionJPAQL, MailNotification> f : getFilters())
      if (f.getCriterion().getQuery().contains("alarmaAsociada")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
  }

  /**
   * Filter by id.
   */
  public void filterById() {
    String query = "";
    boolean flag = false;
    Long id = Long.parseLong(customFilter);

    FilterJPAQLByQuery<MailNotification> filter = new FilterJPAQLByQuery<MailNotification>();

    query += " incidenciaAsociada.id= ? or actividadAsociada.id= ? or informeAsociado.id =? or alarmaAsociada.id= ? ";
    filter.getCriterion().addParameter(id);
    filter.getCriterion().addParameter(id);
    filter.getCriterion().addParameter(id);
    filter.getCriterion().addParameter(id);
  
    filter.getCriterion().setQuery(query);
    getFilters().add(filter);
  }
}
