package es.csa.fdt.cyldigital.recursos.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;

/**
 * Clase AnuncioPanel.
 * 
 * @author csa
 */
@Entity
@Table(name = "anunciospanel")
@AttributeOverride(name = "id", column = @Column(name = "idAnuncioPanel"))
@Audited
@Inheritance(strategy = InheritanceType.JOINED)
public class AnuncioPanel extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = -7648562176655166537L;

  /**
   * Enum TipoAnuncio.
   */
  public enum TipoAnuncio {
    NOTICIA,
    AVISO;
  }

  private String texto;
  private String nombre;
  private TipoAnuncio tipo;
  private List<EventoPanel> eventos;

  /**
   * Construye un nuevo objeto de la clase
   */
  public AnuncioPanel() {
    super();
  }

  /**
   * Devuelve texto.
   * 
   * @return texto
   */
  @NotNull
  @NotEmpty
  @Length(max = 100)
  public String getTexto() {
    return texto;
  }

  /**
   * Asigna texto.
   * 
   * @param texto
   *          texto
   */
  public void setTexto(String texto) {
    this.texto = texto;
  }

  /**
   * Devuelve nombre.
   * 
   * @return nombre
   */
  @NotNull
  @NotEmpty
  @Length(max = 100)
  public String getNombre() {
    return nombre;
  }

  /**
   * Asigna nombre.
   * 
   * @param nombre
   *          nombre
   */
  public void setNombre(String nombre) {
    this.nombre = nombre;
  }

  /**
   * Devuelve tipo.
   * 
   * @return tipo
   */
  @NotNull
  public TipoAnuncio getTipo() {
    return tipo;
  }

  /**
   * Asigna tipo.
   * 
   * @param tipo
   *          tipo
   */
  public void setTipo(TipoAnuncio tipo) {
    this.tipo = tipo;
  }

  /**
   * Devuelve eventos.
   * 
   * @return eventos
   */
  @OneToMany(mappedBy = "anuncio")
  public List<EventoPanel> getEventos() {
    return eventos;
  }

  /**
   * Asigna eventos.
   * 
   * @param eventos
   *          eventos
   */
  public void setEventos(List<EventoPanel> eventos) {
    this.eventos = eventos;
  }
}
