package es.csa.fdt.cyldigital.recursos.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.Date;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import org.hibernate.envers.Audited;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.core.entity.FileOnDisk;
import es.csa.fdt.cyldigital.usuarios.entity.Proveedor;

/**
 * Clase Averia.
 * 
 * @author csa
 */
@Entity
@Table(name = "averias")
@AttributeOverride(name = "id", column = @Column(name = "idAveria"))
@Audited
@Inheritance(strategy = InheritanceType.JOINED)
public class Averia extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = -7648562176655166537L;

  private Date fechaAveria;
  private Recurso recursoAveriado;
  private Recurso recursoSustituto;
  private Proveedor proveedor;
  private String observaciones;
  private Centro centro;
  private EstadoAveria estado;
  private FileOnDisk albaranEntrega;
  private FileOnDisk albaranRecepcion;

  /**
   * Enum EstadoAveria.
   */
  public enum EstadoAveria {
    SIN_REPARAR,
    EN_REPARACION,
    REPARADO;
  }

  /**
   * Construye un nuevo objeto de la clase
   */
  public Averia() {
    super();
  }

  /**
   * Devuelve fecha averia.
   * 
   * @return fecha averia
   */
  @NotNull
  public Date getFechaAveria() {
    return fechaAveria;
  }

  /**
   * Asigna fecha averia.
   * 
   * @param fechaAveria
   *          fecha averia
   */
  public void setFechaAveria(Date fechaAveria) {
    this.fechaAveria = fechaAveria;
  }

  /**
   * Devuelve recurso averiado.
   * 
   * @return recurso averiado
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idRecursoAveriado", nullable = false)
  @NotNull
  public Recurso getRecursoAveriado() {
    return recursoAveriado;
  }

  /**
   * Asigna recurso averiado.
   * 
   * @param recursoAveriado
   *          recurso averiado
   */
  public void setRecursoAveriado(Recurso recursoAveriado) {
    this.recursoAveriado = recursoAveriado;
  }

  /**
   * Devuelve recurso sustituto.
   * 
   * @return recurso sustituto
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idRecursoSustituto")
  public Recurso getRecursoSustituto() {
    return recursoSustituto;
  }

  /**
   * Asigna recurso sustituto.
   * 
   * @param recursoSustituto
   *          recurso sustituto
   */
  public void setRecursoSustituto(Recurso recursoSustituto) {
    this.recursoSustituto = recursoSustituto;
  }

  /**
   * Devuelve observaciones.
   * 
   * @return observaciones
   */
  @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getObservaciones() {
    return observaciones;
  }

  /**
   * Asigna observaciones.
   * 
   * @param observaciones
   *          observaciones
   */
  public void setObservaciones(String observaciones) {
    this.observaciones = observaciones;
  }

  /**
   * Devuelve centro.
   * 
   * @return centro
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idCentro", nullable = false)
  @NotNull
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Devuelve proveedor.
   * 
   * @return proveedor
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idProveedor")
  @NotNull
  public Proveedor getProveedor() {
    return proveedor;
  }

  /**
   * Asigna proveedor.
   * 
   * @param proveedor
   *          proveedor
   */
  public void setProveedor(Proveedor proveedor) {
    this.proveedor = proveedor;
  }

  /**
   * Devuelve estado.
   * 
   * @return estado
   */
  @NotNull
  public EstadoAveria getEstado() {
    return estado;
  }

  /**
   * Asigna estado.
   * 
   * @param estado
   *          estado
   */
  public void setEstado(EstadoAveria estado) {
    this.estado = estado;
  }

  /**
   * Devuelve albaran entrega.
   * 
   * @return albaran entrega
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idAlbaranEntrega")
  public FileOnDisk getAlbaranEntrega() {
    return albaranEntrega;
  }

  /**
   * Asigna albaran entrega.
   * 
   * @param albaranEntrega
   *          albaran entrega
   */
  public void setAlbaranEntrega(FileOnDisk albaranEntrega) {
    this.albaranEntrega = albaranEntrega;
  }

  /**
   * Devuelve albaran recepcion.
   * 
   * @return albaran recepcion
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idAlbaranRecepcion")
  public FileOnDisk getAlbaranRecepcion() {
    return albaranRecepcion;
  }

  /**
   * Asigna albaran recepcion.
   * 
   * @param albaranRecepcion
   *          albaran recepcion
   */
  public void setAlbaranRecepcion(FileOnDisk albaranRecepcion) {
    this.albaranRecepcion = albaranRecepcion;
  }

}
