package es.csa.fdt.cyldigital.recursos.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;

/**
 * Clase CategoriaRecurso.
 * 
 * @author csa
 */
@Entity
@Table(name = "categoriasrecursos")
@AttributeOverride(name = "id", column = @Column(name = "idCategoriaRecurso"))
@Audited
public class CategoriaRecurso extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = -7648562176655166537L;
  private String nombre;
  private String etiquetado;
  private List<Recurso> recursos;
  private boolean ciclico;
  private int diasRecuento;

  /**
   * Construye un nuevo objeto de la clase
   */
  public CategoriaRecurso() {
    super();
    recursos = new ArrayList<Recurso>();
    diasRecuento = 0;
  }

  /**
   * Devuelve nombre.
   * 
   * @return nombre
   */
  @Length(max = 255)
  @NotEmpty
  @NotNull
  public String getNombre() {
    return nombre;
  }

  /**
   * Asigna nombre.
   * 
   * @param nombre
   *          nombre
   */
  public void setNombre(String nombre) {
    this.nombre = nombre;
  }

  /**
   * Devuelve recursos.
   * 
   * @return recursos
   */
  @OneToMany(mappedBy = "categoria")
  public List<Recurso> getRecursos() {
    return recursos;
  }

  /**
   * Asigna recursos.
   * 
   * @param recursos
   *          recursos
   */
  public void setRecursos(List<Recurso> recursos) {
    this.recursos = recursos;
  }

  /**
   * Devuelve etiquetado.
   * 
   * @return etiquetado
   */
  @Length(max = 5)
  @NotEmpty
  @NotNull
  public String getEtiquetado() {
    return etiquetado;
  }

  /**
   * Asigna etiquetado.
   * 
   * @param etiquetado
   *          etiquetado
   */
  public void setEtiquetado(String etiquetado) {
    this.etiquetado = etiquetado;
  }

  /**
   * Asigna ciclico.
   * 
   * @param ciclico
   *          ciclico
   */
  public void setCiclico(boolean ciclico) {
    this.ciclico = ciclico;
  }

  /**
   * Checks if is ciclico.
   * 
   * @return true, if is ciclico
   */
  public boolean isCiclico() {
    return ciclico;
  }

  /**
   * Devuelve dias recuento.
   * 
   * @return dias recuento
   */
  public int getDiasRecuento() {
    return diasRecuento;
  }

  /**
   * Asigna dias recuento.
   * 
   * @param diasRecuento
   *          dias recuento
   */
  public void setDiasRecuento(int diasRecuento) {
    this.diasRecuento = diasRecuento;
  }
}
