package es.csa.fdt.cyldigital.recursos.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.Date;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import org.hibernate.envers.Audited;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.recursos.entity.Recurso.EstadoRecurso;

/**
 * Clase EstadosRecurso.
 * 
 * @author csa
 */
@Entity
@Table(name = "estadosrecurso")
@AttributeOverride(name = "id", column = @Column(name = "idEstadosRecurso"))
@Audited
public class EstadosRecurso extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = -7648562176655166537L;

  private Date fechaEstado;
  private Recurso recurso;
  private EstadoRecurso estado;

  /**
   * Construye un nuevo objeto de la clase
   */
  public EstadosRecurso() {
    super();
  }

  /**
   * Devuelve fecha estado.
   * 
   * @return fecha estado
   */
  public Date getFechaEstado() {
    return fechaEstado;
  }

  /**
   * Asigna fecha estado.
   * 
   * @param fechaEstado
   *          fecha estado
   */
  public void setFechaEstado(Date fechaEstado) {
    this.fechaEstado = fechaEstado;
  }

  /**
   * Devuelve recurso.
   * 
   * @return recurso
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idRecurso", nullable = false)
  public Recurso getRecurso() {
    return recurso;
  }

  /**
   * Asigna recurso.
   * 
   * @param recurso
   *          recurso
   */
  public void setRecurso(Recurso recurso) {
    this.recurso = recurso;
  }

  /**
   * Devuelve estado.
   * 
   * @return estado
   */
  @Enumerated(EnumType.STRING)
  public EstadoRecurso getEstado() {
    return estado;
  }

  /**
   * Asigna estado.
   * 
   * @param estado
   *          estado
   */
  public void setEstado(EstadoRecurso estado) {
    this.estado = estado;
  }

}
