package es.csa.fdt.cyldigital.recursos.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.calendario.entity.EventoPanelCalendarItem;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.core.entity.FilePanel;

/**
 * Clase EventoPanel.
 * 
 * @author csa
 */
@Entity
@Table(name = "eventospanel")
@AttributeOverride(name = "id", column = @Column(name = "idEventoPanel"))
@Audited
public class EventoPanel extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = -7648562176655166537L;

  /**
   * Enum TipoContenido.
   */
  public enum TipoContenido {
    IMAGEN,
    VIDEO,
    VIDEO_ESPECIAL,
    ANUNCIO,
    OFERTA_FORMATIVA,
    AGENDA;
  }

  /**
   * Enum Periodicidad.
   */
  public enum Periodicidad {
    DIARIA,
    SEMANAL,
    MENSUAL;
  }

  private String nombre;
  private String descripcion; /* descripcion general */
  private String pie;
  private String fechaDescriptiva; /* aparece sobre el anuncio */
  private TipoContenido tipoContenido;
  private Date fechaInicio;
  private Date fechaFin;
  private Date horaInicio;
  private Date horaFin;
  private Date fechaAnuncio; // fecha y hora en que se empieza a anunciar
  private Date horaAnuncio;
  private List<Panel> paneles;
  private boolean activado;
  private boolean todoElDia;
  private FilePanel filePanel;
  private Periodicidad periodicidad;
  private AnuncioPanel anuncio;

  private List<Centro> centrosTodos; /*
                                      * este array sirve para seleccionar paneles a través de los checks al crear nuevo
                                      * contenido
                                      */

  private List<EventoPanelCalendarItem> calendarItems;

  /**
   * Construye un nuevo objeto de la clase
   */
  public EventoPanel() {
    super();
    periodicidad = Periodicidad.DIARIA;
    paneles = new ArrayList<Panel>();
    calendarItems = new ArrayList<EventoPanelCalendarItem>();
  }

  /**
   * Devuelve nombre.
   * 
   * @return nombre
   */
  @NotNull
  @NotEmpty
  @Length(max = 200)
  public String getNombre() {
    return nombre;

  }

  /**
   * Asigna nombre.
   * 
   * @param nombre
   *          nombre
   */
  public void setNombre(String nombre) {
    this.nombre = nombre;
  }

  /**
   * Devuelve tipo contenido.
   * 
   * @return tipo contenido
   */
  public TipoContenido getTipoContenido() {
    return tipoContenido;
  }

  /**
   * Asigna tipo contenido.
   * 
   * @param tipoContenido
   *          tipo contenido
   */
  public void setTipoContenido(TipoContenido tipoContenido) {
    this.tipoContenido = tipoContenido;
  }

  /**
   * Devuelve fecha inicio.
   * 
   * @return fecha inicio
   */
  @NotNull
  public Date getFechaInicio() {
    return fechaInicio;
  }

  /**
   * Asigna fecha inicio.
   * 
   * @param fechaInicio
   *          fecha inicio
   */
  public void setFechaInicio(Date fechaInicio) {
    this.fechaInicio = fechaInicio;
  }

  /**
   * Devuelve fecha fin.
   * 
   * @return fecha fin
   */
  public Date getFechaFin() {
    return fechaFin;
  }

  /**
   * Asigna fecha fin.
   * 
   * @param fechaFin
   *          fecha fin
   */
  public void setFechaFin(Date fechaFin) {
    this.fechaFin = fechaFin;
  }

  /**
   * Devuelve descripcion.
   * 
   * @return descripcion
   */
  public String getDescripcion() {
    return descripcion;
  }

  /**
   * Asigna descripcion.
   * 
   * @param descripcion
   *          descripcion
   */
  public void setDescripcion(String descripcion) {
    this.descripcion = descripcion;
  }

  /**
   * Checks if is activado.
   * 
   * @return true, if is activado
   */
  public boolean isActivado() {
    return activado;
  }

  /**
   * Asigna activado.
   * 
   * @param activado
   *          activado
   */
  public void setActivado(boolean activado) {
    this.activado = activado;
  }

  /**
   * Devuelve file panel.
   * 
   * @return file panel
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { javax.persistence.CascadeType.ALL })
  @JoinColumn(name = "idFilePanel")
  public FilePanel getFilePanel() {
    return filePanel;
  }

  /**
   * Asigna file panel.
   * 
   * @param filePanel
   *          file panel
   */
  public void setFilePanel(FilePanel filePanel) {
    this.filePanel = filePanel;
  }

  /**
   * Devuelve hora inicio.
   * 
   * @return hora inicio
   */
  @Temporal(TemporalType.TIME)
  public Date getHoraInicio() {
    return horaInicio;
  }

  /**
   * Asigna hora inicio.
   * 
   * @param horaInicio
   *          hora inicio
   */
  public void setHoraInicio(Date horaInicio) {
    this.horaInicio = horaInicio;
  }

  /**
   * Devuelve hora fin.
   * 
   * @return hora fin
   */
  @Temporal(TemporalType.TIME)
  public Date getHoraFin() {
    return horaFin;
  }

  /**
   * Asigna hora fin.
   * 
   * @param horaFin
   *          hora fin
   */
  public void setHoraFin(Date horaFin) {
    this.horaFin = horaFin;
  }

  /**
   * Devuelve paneles.
   * 
   * @return paneles
   */
  @NotAudited
  @ManyToMany(fetch = FetchType.LAZY, targetEntity = Panel.class, cascade = { javax.persistence.CascadeType.ALL })
  @JoinTable(name = "paneles_eventospanel", joinColumns = @JoinColumn(name = "idEventoPanel"), inverseJoinColumns = @JoinColumn(name = "idPanel"))
  public List<Panel> getPaneles() {
    return paneles;
  }

  /**
   * Asigna paneles.
   * 
   * @param paneles
   *          paneles
   */
  public void setPaneles(List<Panel> paneles) {
    this.paneles = paneles;
  }

  /**
   * Devuelve calendar items.
   * 
   * @return calendar items
   */
  @OneToMany(fetch = FetchType.LAZY, mappedBy = "evento", cascade = { CascadeType.ALL })
  public List<EventoPanelCalendarItem> getCalendarItems() {
    return calendarItems;
  }

  /**
   * Asigna calendar items.
   * 
   * @param calendarItems
   *          calendar items
   */
  public void setCalendarItems(List<EventoPanelCalendarItem> calendarItems) {
    this.calendarItems = calendarItems;
  }

  /**
   * Devuelve centros todos.
   * 
   * @return centros todos
   */
  @Transient
  public List<Centro> getCentrosTodos() {
    if (centrosTodos == null) {
      centrosTodos = new ArrayList<Centro>();
      for (Panel p : getPaneles()) {
        if (!centrosTodos.contains(p.getCentro())) centrosTodos.add(p.getCentro());
      }
    }
    return centrosTodos;
  }

  /**
   * Asigna centros todos.
   * 
   * @param centrosTodos
   *          centros todos
   */
  public void setCentrosTodos(List<Centro> centrosTodos) {
    // TODO Mirar que paneles hay que añadir si hay mas centros que los que tengo
    this.centrosTodos = centrosTodos;
  }

  /**
   * Devuelve periodicidad.
   * 
   * @return periodicidad
   */
  public Periodicidad getPeriodicidad() {
    return periodicidad;
  }

  /**
   * Asigna periodicidad.
   * 
   * @param periodicidad
   *          periodicidad
   */
  public void setPeriodicidad(Periodicidad periodicidad) {
    this.periodicidad = periodicidad;
  }

  /**
   * Devuelve anuncio.
   * 
   * @return anuncio
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idAnuncioPanel")
  public AnuncioPanel getAnuncio() {
    return anuncio;
  }

  /**
   * Asigna anuncio.
   * 
   * @param anuncio
   *          anuncio
   */
  public void setAnuncio(AnuncioPanel anuncio) {
    this.anuncio = anuncio;
  }

  /**
   * Checks if is todo el dia.
   * 
   * @return true, if is todo el dia
   */
  public boolean isTodoElDia() {
    return todoElDia;
  }

  /**
   * Asigna todo el dia.
   * 
   * @param todoElDia
   *          todo el dia
   */
  public void setTodoElDia(boolean todoElDia) {
    this.todoElDia = todoElDia;
  }

  /**
   * Devuelve pie.
   * 
   * @return pie
   */
  public String getPie() {
    return pie;
  }

  /**
   * Asigna pie.
   * 
   * @param pie
   *          pie
   */
  public void setPie(String pie) {
    this.pie = pie;
  }

  /**
   * Devuelve fecha anuncio.
   * 
   * @return fecha anuncio
   */
  public Date getFechaAnuncio() {
    return fechaAnuncio;
  }

  /**
   * Asigna fecha anuncio.
   * 
   * @param fechaAnuncio
   *          fecha anuncio
   */
  public void setFechaAnuncio(Date fechaAnuncio) {
    this.fechaAnuncio = fechaAnuncio;
  }

  /**
   * Devuelve hora anuncio.
   * 
   * @return hora anuncio
   */
  @Temporal(TemporalType.TIME)
  public Date getHoraAnuncio() {
    return horaAnuncio;
  }

  /**
   * Asigna hora anuncio.
   * 
   * @param horaAnuncio
   *          hora anuncio
   */
  public void setHoraAnuncio(Date horaAnuncio) {
    this.horaAnuncio = horaAnuncio;
  }

  /**
   * Devuelve fecha descriptiva.
   * 
   * @return fecha descriptiva
   */
  public String getFechaDescriptiva() {
    return fechaDescriptiva;
  }

  /**
   * Asigna fecha descriptiva.
   * 
   * @param fechaDescriptiva
   *          fecha descriptiva
   */
  public void setFechaDescriptiva(String fechaDescriptiva) {
    this.fechaDescriptiva = fechaDescriptiva;
  }

}
