package es.csa.fdt.cyldigital.recursos.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Length;

import es.csa.fdt.cyldigital.usuarios.entity.Usuario;

/**
 * Clase Ordenador.
 * 
 * @author csa
 */
@Entity
@Table(name = "ordenadores")
@AttributeOverride(name = "id", column = @Column(name = "idOrdenador"))
@Audited
@PrimaryKeyJoinColumn(name = "idRecurso")
public class Ordenador extends Recurso implements Serializable {

  private static final long serialVersionUID = -7648562176655166537L;

  /**
   * Enum EstadoOrdenador.
   */
  public enum EstadoOrdenador {
    ENCENDIDO,
    EN_MODIFICACION,
    APAGADO,
  }

  private String ip;

  private String mac;

  private String so;

  private Usuario usuarioConectado;

  private Date fechaInicioSesion;

  private EstadoOrdenador estadoOrdenador;

  private String reparador;

  private String versionControlador;

  /** estadosordenadores. */
  protected List<EstadosRecurso> estadosordenadores;

  private boolean navegacionControlada;

  /**
   * Construye un nuevo objeto de la clase
   */
  public Ordenador() {
    super();
    navegacionControlada = false;
    estadoOrdenador = EstadoOrdenador.APAGADO;
  }

  /**
   * Devuelve ip.
   * 
   * @return ip
   */
  public String getIp() {
    return ip;
  }

  /**
   * Asigna ip.
   * 
   * @param ip
   *          ip
   */
  public void setIp(String ip) {
    this.ip = ip;
  }

  /**
   * Devuelve mac.
   * 
   * @return mac
   */
  public String getMac() {
    return mac;
  }

  /**
   * Asigna mac.
   * 
   * @param mac
   *          mac
   */
  public void setMac(String mac) {
    this.mac = mac;
  }

  /**
   * Devuelve so.
   * 
   * @return so
   */
  public String getSo() {
    return so;
  }

  /**
   * Asigna so.
   * 
   * @param so
   *          so
   */
  public void setSo(String so) {
    this.so = so;
  }

  /**
   * Devuelve reparador.
   * 
   * @return reparador
   */
  @Length(max = 255)
  public String getReparador() {
    return reparador;
  }

  /**
   * Asigna reparador.
   * 
   * @param reparador
   *          reparador
   */
  public void setReparador(String reparador) {
    this.reparador = reparador;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.recursos.entity.Recurso#isOrdenador()
   */
  @Override
  @Transient
  public boolean isOrdenador() {
    return true;
  }

  /**
   * Devuelve estado ordenador.
   * 
   * @return estado ordenador
   */
  @Enumerated(value = EnumType.STRING)
  public EstadoOrdenador getEstadoOrdenador() {
    return estadoOrdenador;
  }

  /**
   * Asigna estado ordenador.
   * 
   * @param estadoOrdenador
   *          estado ordenador
   */
  public void setEstadoOrdenador(EstadoOrdenador estadoOrdenador) {
    this.estadoOrdenador = estadoOrdenador;
  }

  /**
   * Devuelve usuario conectado.
   * 
   * @return usuario conectado
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idUsuario")
  public Usuario getUsuarioConectado() {
    return usuarioConectado;
  }

  /**
   * Asigna usuario conectado.
   * 
   * @param usuarioConectado
   *          usuario conectado
   */
  public void setUsuarioConectado(Usuario usuarioConectado) {
    this.usuarioConectado = usuarioConectado;
  }

  /**
   * Devuelve fecha inicio sesion.
   * 
   * @return fecha inicio sesion
   */
  @Temporal(TemporalType.TIMESTAMP)
  public Date getFechaInicioSesion() {
    return fechaInicioSesion;
  }

  /**
   * Asigna fecha inicio sesion.
   * 
   * @param fechanInicioSession
   *          fecha inicio sesion
   */
  public void setFechaInicioSesion(Date fechanInicioSession) {
    this.fechaInicioSesion = fechanInicioSession;
  }

  /**
   * Devuelve fecha inicio sesion js.
   * 
   * @return fecha inicio sesion js
   */
  @Transient
  public String getFechaInicioSesionJS() {
    // year, mth - 1, day, hr, min, sec

    String result = "";
    if (getFechaInicioSesion() != null) {
      Calendar c = new GregorianCalendar();
      c.setTime(getFechaInicioSesion());
      result += +c.get(Calendar.YEAR);
      result += "," + c.get(Calendar.MONTH);
      result += "," + c.get(Calendar.DAY_OF_MONTH);
      result += "," + c.get(Calendar.HOUR_OF_DAY);
      result += "," + c.get(Calendar.MINUTE);
      result += "," + c.get(Calendar.SECOND);
    }
    return result;
  }

  /**
   * Devuelve estadosordenadores.
   * 
   * @return estadosordenadores
   */
  @OneToMany(mappedBy = "recurso", cascade = { CascadeType.PERSIST, CascadeType.MERGE, CascadeType.REFRESH })
  @OrderBy(value = "fechaEstado ASC")
  public List<EstadosRecurso> getEstadosordenadores() {
    return estadosordenadores;
  }

  /**
   * Asigna estadosordenadores.
   * 
   * @param estadosordenadores
   *          estadosordenadores
   */
  public void setEstadosordenadores(List<EstadosRecurso> estadosordenadores) {
    this.estadosordenadores = estadosordenadores;
  }

  /**
   * Checks if is navegacion controlada.
   * 
   * @return true, if is navegacion controlada
   */
  public boolean isNavegacionControlada() {
    return navegacionControlada;
  }

  /**
   * Asigna navegacion controlada.
   * 
   * @param navegacionControlada
   *          navegacion controlada
   */
  public void setNavegacionControlada(boolean navegacionControlada) {
    this.navegacionControlada = navegacionControlada;
  }

  /**
   * Devuelve version controlador.
   * 
   * @return version controlador
   */
  @Length(max = 255)
  public String getVersionControlador() {
    return versionControlador;
  }

  /**
   * Asigna version controlador.
   * 
   * @param versionControlador
   *          version controlador
   */
  public void setVersionControlador(String versionControlador) {
    this.versionControlador = versionControlador;
  }

}
