package es.csa.fdt.cyldigital.recursos.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToMany;
import javax.persistence.OneToOne;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;

import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;
import org.hibernate.validator.NotNull;

/**
 * Clase Panel.
 * 
 * @author csa
 */
@Entity
@Table(name = "paneles")
@AttributeOverride(name = "id", column = @Column(name = "idPanel"))
@Audited
@PrimaryKeyJoinColumn(name = "idRecurso")
public class Panel extends Ordenador implements Serializable {

  private static final long serialVersionUID = -7648562176655166537L;

  private String claveIdentificativa;
  private String ipPublica;
  private Tema tema;
  private List<EventoPanel> eventosPanel;
  private OrientacionPanel orientacionPanel;

  /**
   * Enum OrientacionPanel.
   */
  public enum OrientacionPanel {
    HORIZONTAL,
    VERTICAL;
  }

  /**
   * Devuelve clave identificativa.
   * 
   * @return clave identificativa
   */
  @NotNull
  public String getClaveIdentificativa() {
    return claveIdentificativa;
  }

  /**
   * Asigna clave identificativa.
   * 
   * @param claveIdentificativa
   *          clave identificativa
   */
  public void setClaveIdentificativa(String claveIdentificativa) {
    this.claveIdentificativa = claveIdentificativa;
  }

  /**
   * Devuelve tema.
   * 
   * @return tema
   */
  @JoinColumn(name = "idTema")
  @OneToOne(cascade = CascadeType.ALL)
  public Tema getTema() {
    return tema;
  }

  /**
   * Asigna tema.
   * 
   * @param tema
   *          tema
   */
  public void setTema(Tema tema) {
    this.tema = tema;
  }

  /**
   * Devuelve eventos panel.
   * 
   * @return eventos panel
   */
  @NotAudited
  @ManyToMany(cascade = { CascadeType.ALL }, mappedBy = "paneles", targetEntity = EventoPanel.class, fetch = FetchType.LAZY)
  public List<EventoPanel> getEventosPanel() {
    return eventosPanel;
  }

  /**
   * Asigna eventos panel.
   * 
   * @param eventosPanel
   *          eventos panel
   */
  public void setEventosPanel(List<EventoPanel> eventosPanel) {
    this.eventosPanel = eventosPanel;
  }

  /**
   * Devuelve ip publica.
   * 
   * @return ip publica
   */
  @NotNull
  public String getIpPublica() {
    return ipPublica;
  }

  /**
   * Asigna ip publica.
   * 
   * @param ipPublica
   *          ip publica
   */
  public void setIpPublica(String ipPublica) {
    this.ipPublica = ipPublica;
  }

  /**
   * Devuelve orientacion panel.
   * 
   * @return orientacion panel
   */
  @NotNull
  public OrientacionPanel getOrientacionPanel() {
    return orientacionPanel;
  }

  /**
   * Asigna orientacion panel.
   * 
   * @param orientacionPanel
   *          orientacion panel
   */
  public void setOrientacionPanel(OrientacionPanel orientacionPanel) {
    this.orientacionPanel = orientacionPanel;
  }
}
