package es.csa.fdt.cyldigital.recursos.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.Date;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import org.hibernate.envers.Audited;
import org.hibernate.validator.NotNull;
import org.hibernate.validator.Range;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.recursos.entity.Recurso.Unidad;
import es.csa.fdt.cyldigital.usuarios.entity.Proveedor;

/**
 * Clase PedidoRecurso.
 * 
 * @author csa
 */
@Entity
@Table(name = "pedidosrecursos")
@AttributeOverride(name = "id", column = @Column(name = "idPedidoRecurso"))
@Audited
public class PedidoRecurso extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = -7648562176655166537L;

  private CategoriaRecurso categoria;
  private String producto;
  private int cantidad;
  private Date fechaPedido;
  private Date fechaEntrega;
  private Unidad unidad;
  private Proveedor proveedor;
  private Centro centro;

  /**
   * Construye un nuevo objeto de la clase
   */
  public PedidoRecurso() {
    super();
  }

  /**
   * Devuelve categoria.
   * 
   * @return categoria
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idCategoriaRecurso", nullable = false)
  @NotNull
  public CategoriaRecurso getCategoria() {
    return categoria;
  }

  /**
   * Asigna categoria.
   * 
   * @param categoria
   *          categoria
   */
  public void setCategoria(CategoriaRecurso categoria) {
    this.categoria = categoria;
  }

  /**
   * Devuelve producto.
   * 
   * @return producto
   */
  @NotNull
  public String getProducto() {
    return producto;
  }

  /**
   * Asigna producto.
   * 
   * @param producto
   *          producto
   */
  public void setProducto(String producto) {
    this.producto = producto;
  }

  /**
   * Devuelve cantidad.
   * 
   * @return cantidad
   */
  @Range(min = 0)
  @NotNull
  public int getCantidad() {
    return cantidad;
  }

  /**
   * Asigna cantidad.
   * 
   * @param cantidad
   *          cantidad
   */
  public void setCantidad(int cantidad) {
    this.cantidad = cantidad;
  }

  /**
   * Devuelve fecha pedido.
   * 
   * @return fecha pedido
   */
  @NotNull
  public Date getFechaPedido() {
    return fechaPedido;
  }

  /**
   * Asigna fecha pedido.
   * 
   * @param fechaPedido
   *          fecha pedido
   */
  public void setFechaPedido(Date fechaPedido) {
    this.fechaPedido = fechaPedido;
  }

  /**
   * Devuelve fecha entrega.
   * 
   * @return fecha entrega
   */
  public Date getFechaEntrega() {
    return fechaEntrega;
  }

  /**
   * Asigna fecha entrega.
   * 
   * @param fechaEntrega
   *          fecha entrega
   */
  public void setFechaEntrega(Date fechaEntrega) {
    this.fechaEntrega = fechaEntrega;
  }

  /**
   * Devuelve unidad.
   * 
   * @return unidad
   */
  @NotNull
  public Unidad getUnidad() {
    return unidad;
  }

  /**
   * Asigna unidad.
   * 
   * @param unidad
   *          unidad
   */
  public void setUnidad(Unidad unidad) {
    this.unidad = unidad;
  }

  /**
   * Devuelve proveedor.
   * 
   * @return proveedor
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idProveedor", nullable = false)
  @NotNull
  public Proveedor getProveedor() {
    return proveedor;
  }

  /**
   * Asigna proveedor.
   * 
   * @param proveedor
   *          proveedor
   */
  public void setProveedor(Proveedor proveedor) {
    this.proveedor = proveedor;
  }

  /**
   * Devuelve centro.
   * 
   * @return centro
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idCentro", nullable = false)
  @NotNull
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }
}
