package es.csa.fdt.cyldigital.recursos.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.calendario.entity.ActividadCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.ReservaCalendarItem;
import es.csa.fdt.cyldigital.centros.entity.Aula;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.incidencias.entity.Incidencia;
import es.csa.fdt.cyldigital.servicios.entity.Actividad;

/**
 * Clase Recurso.
 * 
 * @author csa
 */
@Entity
@Table(name = "recursos")
@AttributeOverride(name = "id", column = @Column(name = "idRecurso"))
@Audited
@Inheritance(strategy = InheritanceType.JOINED)
public class Recurso extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = -7648562176655166537L;

  private String nombre;
  private String observaciones;
  private String codEtiqueta;
  private Centro centro;
  private Aula aula;
  private CategoriaRecurso categoria;
  private int stockSeguridad;
  private int pedidoOptimo;
  private int cantidad;

  /**
   * Enum EstadoRecurso.
   */
  public enum EstadoRecurso {
    ACTIVO,
    AVERIADO,
    REPARACION,
    RETIRADO,
    STOCK,
    SIN_ASIGNAR;
  }

  /**
   * Enum Unidad.
   */
  public enum Unidad {
    UNIDAD,
    CAJA;
  }

  private Unidad unidad;
  private EstadoRecurso estado;
  private boolean asignado;
  
  /** incidencias. */
  protected List<Incidencia> incidencias;
  
  /** actividades. */
  protected List<Actividad> actividades;
  
  /** estadosrecursos. */
  protected List<EstadosRecurso> estadosrecursos;
  
  /** reservas. */
  protected List<ReservaCalendarItem> reservas;

  /**
   * Construye un nuevo objeto de la clase
   */
  public Recurso() {
    super();
    incidencias = new ArrayList<Incidencia>();
    actividades = new ArrayList<Actividad>();
    estadosrecursos = new ArrayList<EstadosRecurso>();
    reservas = new ArrayList<ReservaCalendarItem>();

    estado = EstadoRecurso.ACTIVO;
    asignado = false;
  }

  /**
   * Devuelve nombre.
   * 
   * @return nombre
   */
  @Length(max = 255)
  @NotEmpty
  @NotNull
  public String getNombre() {
    return nombre;
  }

  /**
   * Asigna nombre.
   * 
   * @param nombre
   *          nombre
   */
  public void setNombre(String nombre) {
    this.nombre = nombre;
  }

  /**
   * Devuelve centro.
   * 
   * @return centro
   */
  @ManyToOne(fetch = FetchType.LAZY)
  @JoinColumn(name = "idCentro")
  @NotNull
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Devuelve aula.
   * 
   * @return aula
   */
  @ManyToOne(fetch = FetchType.LAZY)
  @JoinColumn(name = "idAula")
  public Aula getAula() {
    return aula;
  }

  /**
   * Asigna aula.
   * 
   * @param aula
   *          aula
   */
  public void setAula(Aula aula) {
    this.aula = aula;
  }

  /**
   * Devuelve categoria.
   * 
   * @return categoria
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idCategoriaRecurso")
  @NotNull
  public CategoriaRecurso getCategoria() {
    return categoria;
  }

  /**
   * Asigna categoria.
   * 
   * @param categoria
   *          categoria
   */
  public void setCategoria(CategoriaRecurso categoria) {
    this.categoria = categoria;
  }

  /**
   * Devuelve estado.
   * 
   * @return estado
   */
  @Enumerated(value = EnumType.STRING)
  @NotNull
  public EstadoRecurso getEstado() {
    return estado;
  }

  /**
   * Asigna estado.
   * 
   * @param estado
   *          estado
   */
  public void setEstado(EstadoRecurso estado) {
    this.estado = estado;
  }

  /**
   * Checks if is asignado.
   * 
   * @return true, if is asignado
   */
  public boolean isAsignado() {
    return asignado;
  }

  /**
   * Asigna asignado.
   * 
   * @param asignado
   *          asignado
   */
  public void setAsignado(boolean asignado) {
    this.asignado = asignado;
  }

  /**
   * Checks if is en reserva.
   * 
   * @return true, if is en reserva
   */
  @Transient
  public boolean isEnReserva() {
    return (!asignado);
  }

  /**
   * Checks if is ordenador.
   * 
   * @return true, if is ordenador
   */
  @Transient
  public boolean isOrdenador() {
    return false;
  }

  /**
   * Checks if is router.
   * 
   * @return true, if is router
   */
  @Transient
  public boolean isRouter() {
    return false;
  }

  /**
   * Devuelve incidencias.
   * 
   * @return incidencias
   */
  @NotAudited
  @ManyToMany(fetch = FetchType.LAZY, cascade = { CascadeType.ALL }, mappedBy = "recursos", targetEntity = Incidencia.class)
  public List<Incidencia> getIncidencias() {
    return incidencias;
  }

  /**
   * Asigna incidencias.
   * 
   * @param incidencias
   *          incidencias
   */
  public void setIncidencias(List<Incidencia> incidencias) {
    this.incidencias = incidencias;
  }

  /**
   * Devuelve actividades.
   * 
   * @return actividades
   */
  @NotAudited
  @ManyToMany(fetch = FetchType.LAZY, cascade = { CascadeType.PERSIST, CascadeType.MERGE, CascadeType.REFRESH }, mappedBy = "recursos", targetEntity = Actividad.class)
  public List<Actividad> getActividades() {
    return actividades;
  }

  /**
   * Asigna actividades.
   * 
   * @param actividades
   *          actividades
   */
  public void setActividades(List<Actividad> actividades) {
    this.actividades = actividades;
  }

  /**
   * Devuelve observaciones.
   * 
   * @return observaciones
   */
  @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getObservaciones() {
    return observaciones;
  }

  /**
   * Asigna observaciones.
   * 
   * @param observaciones
   *          observaciones
   */
  public void setObservaciones(String observaciones) {
    this.observaciones = observaciones;
  }

  /**
   * Devuelve cod etiqueta.
   * 
   * @return cod etiqueta
   */
  public String getCodEtiqueta() {
    if ((codEtiqueta == null || codEtiqueta.isEmpty()) && getId() > 0 && getCategoria() != null)
      codEtiqueta = categoria.getEtiquetado() + "-" + getId();
    return codEtiqueta;
  }

  /**
   * Asigna cod etiqueta.
   * 
   * @param codEtiqueta
   *          cod etiqueta
   */
  public void setCodEtiqueta(String codEtiqueta) {
    this.codEtiqueta = codEtiqueta;
  }

  /**
   * Devuelve estadosrecursos.
   * 
   * @return estadosrecursos
   */
  @OneToMany(mappedBy = "recurso", cascade = { CascadeType.PERSIST, CascadeType.MERGE, CascadeType.REFRESH })
  @OrderBy(value = "fechaEstado ASC")
  public List<EstadosRecurso> getEstadosrecursos() {
    return estadosrecursos;
  }

  /**
   * Asigna estadosrecursos.
   * 
   * @param estadosrecursos
   *          estadosrecursos
   */
  public void setEstadosrecursos(List<EstadosRecurso> estadosrecursos) {
    this.estadosrecursos = estadosrecursos;
  }

  /**
   * Devuelve reservas.
   * 
   * @return reservas
   */
  @OneToMany(mappedBy = "recurso", cascade = { CascadeType.PERSIST, CascadeType.MERGE, CascadeType.REFRESH })
  public List<ReservaCalendarItem> getReservas() {
    return reservas;
  }

  /**
   * Asigna reservas.
   * 
   * @param reservas
   *          reservas
   */
  public void setReservas(List<ReservaCalendarItem> reservas) {
    this.reservas = reservas;
  }

  /**
   * Checks if is ocupado.
   * 
   * @param actividad
   *           actividad
   * @return true, if is ocupado
   */
  @Transient
  public boolean isOcupado(Actividad actividad) {
    for (Actividad a : getActividades()) {
      for (ActividadCalendarItem aci : a.getCalendarItems()) {
        for (ActividadCalendarItem aciNew : actividad.getCalendarItems()) {
          if (aciNew.getId() != aci.getId() && aciNew.getStart().before(aci.getEnd())
              && aciNew.getEnd().after(aci.getStart())) { return true; }
        }
      }
    }
    for (ReservaCalendarItem rci : getReservas()) {
      for (ActividadCalendarItem aciNew : actividad.getCalendarItems()) {
        if (aciNew.getStart().before(rci.getEnd()) && aciNew.getEnd().after(rci.getStart())) { return true; }
      }
    }
    return false;
  }

  /**
   * Checks if is ocupado.
   * 
   * @param reserva
   *           reserva
   * @return true, if is ocupado
   */
  @Transient
  public boolean isOcupado(ReservaCalendarItem reserva) {
    for (Actividad a : getActividades()) {
      for (ActividadCalendarItem aci : a.getCalendarItems()) {
        if (reserva.getStart().before(aci.getEnd()) && reserva.getEnd().after(aci.getStart())) { return true; }
      }
    }
    for (ReservaCalendarItem rci : getReservas()) {
      if (reserva.getId() != rci.getId() && reserva.getStart().before(rci.getEnd())
          && reserva.getEnd().after(rci.getStart())) { return true; }
    }
    return false;
  }
  
  public boolean isOcupado(List<ActividadCalendarItem> acis) {
	  for (Actividad a : getActividades()) {
	      for (ActividadCalendarItem aciOld : a.getCalendarItems()) {
	    	  for (ActividadCalendarItem aciNew : acis) {
	    		  if (aciNew.getId() != aciOld.getId() && aciNew.getStart().before(aciOld.getEnd()) && aciNew.getEnd().after(aciOld.getStart()))
	    			  return true;
	    	  }
	      }
	  }

	  for (ReservaCalendarItem rci : getReservas()) {
		  for (ActividadCalendarItem aciNew : acis) {
			  if (aciNew.getStart().before(rci.getEnd()) && aciNew.getEnd().after(rci.getStart()))
				  return true;
		  }
	  }

	  return false;
  }

  /**
   * Devuelve stock seguridad.
   * 
   * @return stock seguridad
   */
  public int getStockSeguridad() {
    return stockSeguridad;
  }

  /**
   * Asigna stock seguridad.
   * 
   * @param stockSeguridad
   *          stock seguridad
   */
  public void setStockSeguridad(int stockSeguridad) {
    this.stockSeguridad = stockSeguridad;
  }

  /**
   * Devuelve pedido optimo.
   * 
   * @return pedido optimo
   */
  public int getPedidoOptimo() {
    return pedidoOptimo;
  }

  /**
   * Asigna pedido optimo.
   * 
   * @param pedidoOptimo
   *          pedido optimo
   */
  public void setPedidoOptimo(int pedidoOptimo) {
    this.pedidoOptimo = pedidoOptimo;
  }

  /**
   * Devuelve cantidad.
   * 
   * @return cantidad
   */
  public int getCantidad() {
    return cantidad;
  }

  /**
   * Asigna cantidad.
   * 
   * @param cantidad
   *          cantidad
   */
  public void setCantidad(int cantidad) {
    this.cantidad = cantidad;
  }

  /**
   * Devuelve unidad.
   * 
   * @return unidad
   */
  public Unidad getUnidad() {
    return unidad;
  }

  /**
   * Asigna unidad.
   * 
   * @param unidad
   *          unidad
   */
  public void setUnidad(Unidad unidad) {
    this.unidad = unidad;
  }

}
