package es.csa.fdt.cyldigital.recursos.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.Table;

import org.hibernate.envers.Audited;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;

/**
 * Clase RemoteNotice.
 * 
 * @author csa
 */
@Entity
@Table(name = "remotenotification")
@AttributeOverride(name = "id", column = @Column(name = "idRemoteNotification"))
@Audited
public class RemoteNotice extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = -8268969092437378976L;

  /**
   * Enum NotificationType.
   */
  public enum NotificationType {
    DO_START,
    DO_SHOUTDOWN,
    DO_RESTART,
    DO_RUN_COMMAND,
    DO_RUN_USER_COMMAND,
    DO_LOGOUT,
    DO_SHOW_MSG,
    DO_CHECK_ALIVE,
    CLIENT_STARTED,
    CLIENT_STOPED,
    CLIENT_ALIVE,
    CLIENT_LOGIN,
    CLIENT_LOGOUT,
  }

  /**
   * Enum Status.
   */
  public enum Status {
    PENDING,
    IN_PROGRESS,
    DONE,
    ERROR,
    NOTIFY,
  }

  private NotificationType notificationType;
  private String userName;
  private String ipAddress;
  private long serverId;
  private String macAddress;
  private String message;
  private String resultMessage;
  private Status status;
  private int entero = 0;

  /**
   * Devuelve notification type.
   * 
   * @return notification type
   */
  @Enumerated(value = EnumType.STRING)
  public NotificationType getNotificationType() {
    return notificationType;
  }

  /**
   * Asigna notification type.
   * 
   * @param notificationType
   *          notification type
   */
  public void setNotificationType(NotificationType notificationType) {
    this.notificationType = notificationType;
  }

  /**
   * Devuelve message.
   * 
   * @return message
   */
  public String getMessage() {
    return message;
  }

  /**
   * Asigna message.
   * 
   * @param description
   *          message
   */
  public void setMessage(String description) {
    this.message = description;
  }

  /**
   * Devuelve user name.
   * 
   * @return user name
   */
  public String getUserName() {
    return userName;
  }

  /**
   * Asigna user name.
   * 
   * @param userName
   *          user name
   */
  public void setUserName(String userName) {
    this.userName = userName;
  }

  /**
   * Devuelve ip address.
   * 
   * @return ip address
   */
  public String getIpAddress() {
    return ipAddress;
  }

  /**
   * Asigna ip address.
   * 
   * @param ipAddress
   *          ip address
   */
  public void setIpAddress(String ipAddress) {
    this.ipAddress = ipAddress;
  }

  /**
   * Devuelve mac address.
   * 
   * @return mac address
   */
  public String getMacAddress() {
    return macAddress;
  }

  /**
   * Asigna mac address.
   * 
   * @param macAddress
   *          mac address
   */
  public void setMacAddress(String macAddress) {
    this.macAddress = macAddress;
  }

  /**
   * Devuelve status.
   * 
   * @return status
   */
  @Enumerated(value = EnumType.STRING)
  public Status getStatus() {
    return status;
  }

  /**
   * Asigna status.
   * 
   * @param status
   *          status
   */
  public void setStatus(Status status) {
    this.status = status;
  }

  /**
   * Devuelve server id.
   * 
   * @return server id
   */
  public long getServerId() {
    return serverId;
  }

  /**
   * Asigna server id.
   * 
   * @param serverId
   *          server id
   */
  public void setServerId(long serverId) {
    this.serverId = serverId;
  }

  /**
   * Devuelve result message.
   * 
   * @return result message
   */
  public String getResultMessage() {
    return resultMessage;
  }

  /**
   * Asigna result message.
   * 
   * @param resultMessage
   *          result message
   */
  public void setResultMessage(String resultMessage) {
    this.resultMessage = resultMessage;
  }

  /**
   * Campo de utilidad segun el tipo de mensaje LOGOUT - Segundos de espera.
   * 
   * @return entero
   */
  public int getEntero() {
    return entero;
  }

  /**
   * Asigna entero.
   * 
   * @param integer
   *          entero
   */
  public void setEntero(int integer) {
    this.entero = integer;
  }

}
