package es.csa.fdt.cyldigital.recursos.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.OneToMany;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.calendario.entity.WifiCalendarItem;

/**
 * Clase Router.
 * 
 * @author csa
 */
@Entity
@Table(name = "routers")
@AttributeOverride(name = "id", column = @Column(name = "idRouter"))
@Audited
@PrimaryKeyJoinColumn(name = "idRecurso")
public class Router extends Recurso implements Serializable {

  private static final long serialVersionUID = -7648562176655166537L;

  private String ssid;

  private List<WifiCalendarItem> clavesWifi;

  /**
   * Construye un nuevo objeto de la clase
   */
  public Router() {
    super();
    clavesWifi = new ArrayList<WifiCalendarItem>();
  }

  /**
   * Devuelve ssid.
   * 
   * @return ssid
   */
  @NotNull
  public String getSsid() {
    return ssid;
  }

  /**
   * Asigna ssid.
   * 
   * @param ssid
   *          ssid
   */
  public void setSsid(String ssid) {
    this.ssid = ssid;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.recursos.entity.Recurso#isRouter()
   */
  @Override
  @Transient
  public boolean isRouter() {
    return true;
  }

  /**
   * Devuelve claves wifi.
   * 
   * @return claves wifi
   */
  @OneToMany(mappedBy = "router", cascade = { CascadeType.PERSIST, CascadeType.MERGE, CascadeType.REFRESH })
  public List<WifiCalendarItem> getClavesWifi() {
    return clavesWifi;
  }

  /**
   * Asigna claves wifi.
   * 
   * @param clavesWifi
   *          claves wifi
   */
  public void setClavesWifi(List<WifiCalendarItem> clavesWifi) {
    this.clavesWifi = clavesWifi;
  }

  /**
   * Añade clave wifi.
   * 
   * @param claveWifi
   *           clave wifi
   */
  public void addClaveWifi(WifiCalendarItem claveWifi) {
    clavesWifi.add(claveWifi);
  }

  /**
   * Asigna end date clave wifi.
   * 
   * @param end
   *          end date clave wifi
   */
  public void setEndDateClaveWifi(Date end) {
    getUltimaClaveWifi().setEnd(end);
  }

  /**
   * Devuelve clave.
   * 
   * @return clave
   */
  public String getClave() {
    return getUltimaClaveWifi() == null ? null : getUltimaClaveWifi().getClave();
  }

  /**
   * Asigna clave.
   * 
   * @param clave
   *          clave
   */
  public void setClave(String clave) {}

  /**
   * Devuelve ultima clave wifi.
   * 
   * @return ultima clave wifi
   */
  @Transient
  public WifiCalendarItem getUltimaClaveWifi() {
    WifiCalendarItem last = null;
    for (WifiCalendarItem w : clavesWifi) {
      last = last == null ? w : (last.getStart().compareTo(w.getStart()) < 0 ? w : last);
    }
    return last == null ? null : last;
  }

}
