package es.csa.fdt.cyldigital.recursos.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.commons.persistence.browser.jpaql.FilterJPAQLByQuery;
import es.csa.fdt.cyldigital.core.service.BrowserFilteredByActive;
import es.csa.fdt.cyldigital.recursos.entity.AnuncioPanel;
import es.csa.fdt.cyldigital.recursos.entity.AnuncioPanel.TipoAnuncio;

/**
 * Servicio del Browser de Anuncios de Paneles.
 * 
 * @author csa
 */
@Service
@Repository
public class AnunciosPanelesBrowserImpl extends BrowserFilteredByActive<AnuncioPanel> implements
    AnunciosPanelesBrowser<CriterionJPAQL, String> {

  private static final long serialVersionUID = -7227366052245717441L;

  private TipoAnuncio tipoAnuncio;

  /**
   * Construye un nuevo objeto de la clase
   */
  public AnunciosPanelesBrowserImpl() {
    super();

  }

  /**
   * Devuelve tipo anuncio.
   * 
   * @return tipo anuncio
   */
  public TipoAnuncio getTipoAnuncio() {
    return tipoAnuncio;
  }

  /**
   * Asigna tipo anuncio.
   * 
   * @param tipoAnuncio
   *          tipo anuncio
   */
  public void setTipoAnuncio(TipoAnuncio tipoAnuncio) {
    this.tipoAnuncio = tipoAnuncio;
  }

  /**
   * Filter tipo anuncio.
   */
  public void filterTipoAnuncio() {

    removeFilterTipoAnuncio();
    if (getTipoAnuncio() != null) {
      filterByTipoAnuncio(getTipoAnuncio());
    }
  }

  private void removeFilterTipoAnuncio() {
    int i = -1;
    for (Filter<CriterionJPAQL, AnuncioPanel> f : getFilters())
      if (f.getCriterion().getQuery().contains("tipo")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
  }

  /**
   * Filter by tipo anuncio.
   * 
   * @param tipoAnuncio
   *           tipo anuncio
   */
  public void filterByTipoAnuncio(TipoAnuncio tipoAnuncio) {
    String query = "";

    boolean flag = false;
    FilterJPAQLByQuery<AnuncioPanel> filter = new FilterJPAQLByQuery<AnuncioPanel>();

    query += " tipo = ? ";
    filter.getCriterion().addParameter(tipoAnuncio);

    for (Filter<CriterionJPAQL, AnuncioPanel> f : getFilters())
      if (f.getCriterion().getQuery().contains("tipo")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }
}
