package es.csa.fdt.cyldigital.recursos.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.Date;

import org.richfaces.event.UploadEvent;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.faces.util.FicheroSubidoRichFacesAdapter;
import es.csa.fdt.cyldigital.core.entity.FileOnDisk;
import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.core.entity.FileOnDisk.Tipo;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.recursos.entity.Averia;
import es.csa.fdt.cyldigital.recursos.entity.Recurso;
import es.csa.fdt.cyldigital.recursos.entity.Averia.EstadoAveria;
import es.csa.fdt.cyldigital.recursos.entity.Recurso.EstadoRecurso;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;

/**
 * Clase AveriaServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class AveriaServiceImpl extends AbstractServiceImpl<Averia, Long> implements AveriaService {

  private static final long serialVersionUID = -5575228180361917674L;
  private FileOnDisk lastUploaded;

  /**
   * Construye un nuevo objeto de la clase
   */
  public AveriaServiceImpl() {
    super();
  }

  /**
   * Devuelve last uploaded.
   * 
   * @return last uploaded
   */
  public FileOnDisk getLastUploaded() {
    return lastUploaded;
  }

  /**
   * Asigna last uploaded.
   * 
   * @param lastUploaded
   *          last uploaded
   */
  public void setLastUploaded(FileOnDisk lastUploaded) {
    this.lastUploaded = lastUploaded;
  }

  /**
   * Persist averia.
   * 
   * @param averia
   *           averia
   * @param service
   *           service
   * @param accion
   *           accion
   * @return true, if successful
   */
  public boolean persistAveria(Averia averia, RecursoServiceImpl service, String accion) {

    Parameter p = findOneWhere(Parameter.class, "nombre", "pathfileondisk");

    if (accion.equals("reparar") || accion.equals("marcarcomoreparado")) {
      if (lastUploaded == null) {
        addErrorMessage("Debe adjuntar un albarán de entrega del recurso que se va a reparar.");
        return false;
      }

      disableMessages();
      lastUploaded.setTipo(Tipo.ALBARAN);
      lastUploaded.grabarEnDisco(p.getValor());
      persist(lastUploaded);

      if (accion.equals("reparar")) {
        averia.setEstado(EstadoAveria.EN_REPARACION);
        averia.setAlbaranEntrega(lastUploaded);
      } else if (accion.equals("marcarcomoreparado")) {
        averia.setEstado(EstadoAveria.REPARADO);
        averia.setAlbaranRecepcion(lastUploaded);
      }
    } else {
      // accion= nuevaAveria
      if (averia.getRecursoAveriado() == null) {
        addErrorMessage("Debe indicar un recurso averiado.");
        return false;
      }

      if (averia.getRecursoAveriado() == averia.getRecursoSustituto()) {
        addErrorMessage("El recurso averiado y el sustituto no pueden ser iguales.");
        return false;
      }

      if (averia.getRecursoSustituto() != null && averia.getCentro() != averia.getRecursoSustituto().getCentro()) {
        addErrorMessage("El recurso sustituto debería colocarse en el mismo centro donde se ubica la avería.");
        return false;
      }
    }
    disableMessages();
    // averia.getRecursoAveriado().setEstado(EstadoRecurso.AVERIADO);
    // averia.getRecursoAveriado().setAula(getLocalizacionDefecto(averia.getCentro()));
    service.persistRecurso(averia.getRecursoAveriado()); // para que se graben los estados

    if (averia.getRecursoSustituto() != null) {
      // averia.getRecursoSustituto().setEstado(EstadoRecurso.ACTIVO);
      // averia.getRecursoSustituto().setAula(averia.getRecursoAveriado().getAula());
      service.persistRecurso(averia.getRecursoSustituto());
    }

    // enableMessages();
    persist(averia);
    return true;
  }

  /**
   * Añade recurso.
   * 
   * @param r
   *           r
   * @param a
   *           a
   * @param recursoAveriado
   *           recurso averiado
   * @param recursoSustituto
   *           recurso sustituto
   */
  public void addRecurso(Recurso r, Averia a, boolean recursoAveriado, boolean recursoSustituto) {
    if (recursoAveriado) {
      a.setRecursoAveriado(r);
      r.setEstado(EstadoRecurso.REPARACION);
    }
    if (recursoSustituto) {
      a.setRecursoSustituto(r);
      r.setEstado(EstadoRecurso.ACTIVO);
    }
  }

  /**
   * Quitar recurso averiado.
   * 
   * @param a
   *           a
   */
  public void quitarRecursoAveriado(Averia a) {
    a.setRecursoAveriado(null);
  }

  /**
   * Quitar recurso sustituto.
   * 
   * @param a
   *           a
   */
  public void quitarRecursoSustituto(Averia a) {
    a.setRecursoSustituto(null);
  }

  /**
   * Inicializa.
   * 
   * @param a
   *           a
   * @param u
   *           u
   */
  public void inicializa(Averia a, UsuarioLdap u) {
    a.setEstado(EstadoAveria.SIN_REPARAR);
    a.setFechaAveria(new Date());
    a.setCentro(u.getCentro());
  }

  /**
   * Aniadir fichero.
   * 
   * @param event
   *           event
   */
  public synchronized void aniadirFichero(UploadEvent event) {
    lastUploaded = new FileOnDisk();
    lastUploaded.setFicheroSubido(new FicheroSubidoRichFacesAdapter(event.getUploadItem()));
  }

  /**
   * Quitar fichero.
   * 
   * @param a
   *           a
   */
  public void quitarFichero(Averia a) {
    lastUploaded = null;
  }
}
