package es.csa.fdt.cyldigital.recursos.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.Calendar;
import java.util.List;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.entity.FilePanel;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.recursos.entity.AnuncioPanel;
import es.csa.fdt.cyldigital.recursos.entity.EventoPanel;
import es.csa.fdt.cyldigital.recursos.entity.Panel;
import es.csa.fdt.cyldigital.recursos.entity.EventoPanel.TipoContenido;


/**
 * Clase EventoPanelServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class EventoPanelServiceImpl extends AbstractServiceImpl<EventoPanel, Long> implements EventoPanelService {

  private static final long serialVersionUID = -5575228180361917674L;
  private Centro centro; /* se usa? */
  private boolean todos;
  private List<Centro> centrosTodos;

  /**
   * Construye un nuevo objeto de la clase
   */
  public EventoPanelServiceImpl() {
    super();
  }

  /**
   * Inicializa evento panel.
   * 
   * @param evento
   *           evento
   * @return evento panel
   */
  public EventoPanel inicializaEventoPanel(EventoPanel evento) {
    evento.setActivado(true);
    return evento;
  }

  /**
   * Devuelve centro.
   * 
   * @return centro
   */
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Checks if is todos.
   * 
   * @return true, if is todos
   */
  public boolean isTodos() {
    return todos;
  }

  /**
   * Asigna todos.
   * 
   * @param todos
   *          todos
   */
  public void setTodos(boolean todos) {
    this.todos = todos;
  }

  /**
   * Completar centros.
   * 
   * @param e
   *           e
   */
  public void completarCentros(EventoPanel e) {
    if (isTodos()) {
      List<Centro> centros = findAll(Centro.class);
      e.setCentrosTodos(centros);

      List<Panel> paneles = findAll(Panel.class);
      e.setPaneles(paneles);
    } else {
      e.getCentrosTodos().clear();
      e.getPaneles().clear();
    }
  }

  /**
   * Quitar panel.
   * 
   * @param e
   *           e
   * @param p
   *           p
   */
  public void quitarPanel(EventoPanel e, Panel p) {
    setTodos(false);
    e.getCentrosTodos().remove(p.getCentro());
    e.getPaneles().remove(p);
  }

  /**
   * Añade file.
   * 
   * @param f
   *           f
   * @param e
   *           e
   */
  public void addFile(FilePanel f, EventoPanel e) {
    e.setFilePanel(f);
  }

  /**
   * Añade panel.
   * 
   * @param p
   *           p
   * @param e
   *           e
   */
  public void addPanel(Panel p, EventoPanel e) {
    e.getPaneles().add(p);
    // actualizamos eventopanel.centrosTodos, para que los checks sean congruentes con los paneles seleccionados.
    List<Panel> panelesDelCentro = findWhere(Panel.class, "centro.id", p.getCentro().getId());
    Boolean flag = true;

    for (Panel panel : panelesDelCentro) {
      if (!e.getPaneles().contains(panel)) flag = false;
    }

    if (flag) // el evento contiene todos los paneles del centro, luego hay que marcar ese centro en centrostodos
    {
      e.getCentrosTodos().add(p.getCentro());
    }
  }

  /**
   * Añade anuncio.
   * 
   * @param a
   *           a
   * @param e
   *           e
   */
  public void addAnuncio(AnuncioPanel a, EventoPanel e) {
    e.setAnuncio(a);
  }

  /**
   * Añade remove centro.
   * 
   * @param e
   *           e
   * @param anterior
   *           anterior
   */
  public void addRemoveCentro(EventoPanel e, List<Centro> anterior) {
    // donde e.paneles es la lista actual y anterior es la lista anterior de centros seleccionados
    List<Centro> actual = e.getCentrosTodos();

    // comprobamos qué se ha seleccionado.
    for (Centro centro : actual) {
      if (!anterior.contains(centro)) {
        anadirPanelesDelCentro(e, centro);
      }
      // si coinciden, no han cambiado
    }

    // comprobamos qué se ha deseleccionado
    for (Centro centro : anterior) {
      if (!actual.contains(centro)) {
        quitarPanelesDelCentro(e, centro);
      }
    }
    setTodos(false);
  }

  /**
   * Anadir paneles del centro.
   * 
   * @param e
   *           e
   * @param c
   *           c
   */
  public void anadirPanelesDelCentro(EventoPanel e, Centro c) {
    List<Panel> paneles = findWhere(Panel.class, "centro.id", c.getId());
    for (Panel p : paneles) {
      if (!e.getPaneles().contains(p)) e.getPaneles().add(p);
    }
  }

  /**
   * Quitar paneles del centro.
   * 
   * @param e
   *           e
   * @param c
   *           c
   */
  public void quitarPanelesDelCentro(EventoPanel e, Centro c) {
    List<Panel> paneles = findWhere(Panel.class, "centro.id", c.getId());
    for (Panel p : paneles) {
      e.getPaneles().remove(p);
    }
  }

  /**
   * Asigna centros todos.
   * 
   * @param centrosTodos
   *          centros todos
   */
  public void setCentrosTodos(List<Centro> centrosTodos) {
    // TODO Mirar que paneles hay que añadir si hay mas centros que los que tengo
    this.centrosTodos = centrosTodos;
  }

  /**
   * Devuelve centros todos.
   * 
   * @return centros todos
   */
  public List<Centro> getCentrosTodos() {
    return centrosTodos;
  }

  /**
   * Revisar contenido.
   * 
   * @param e
   *           e
   */
  public void revisarContenido(EventoPanel e) {
  /*
   * if (e.getFilePanel() != null) { if (e.getTipoContenido() == TipoContenido.IMAGEN && (e.getFilePanel().getTipoMime()
   * != "image/jpeg" || e.getFilePanel().getTipoMime() != "image/png")) { notificarErrorContenido(e); } if
   * (e.getTipoContenido() == TipoContenido.VIDEO && (e.getFilePanel().getTipoMime() != "")) {
   * notificarErrorContenido(e); } }
   */
  }

  /*
   * private void notificarErrorContenido(EventoPanel e) { e.setFilePanel(null); enableMessages();
   * addErrorMessage("El fichero no corresponde con el tipo indicado, se ha deseleccionado el fichero."); }
   */

  /**
   * Persist evento panel.
   * 
   * @param e
   *           e
   * @return true, if successful
   */
  public boolean persistEventoPanel(EventoPanel e) {

    if (e.isTodoElDia()) {
      Calendar c = Calendar.getInstance();
      c.set(Calendar.HOUR_OF_DAY, 0);
      c.set(Calendar.MINUTE, 0);
      c.set(Calendar.SECOND, 0);

      e.setHoraInicio(c.getTime());

      c.set(Calendar.HOUR_OF_DAY, 23);
      c.set(Calendar.MINUTE, 59);
      c.set(Calendar.SECOND, 59);
      e.setHoraFin(c.getTime());
    }
    if (e.getTipoContenido() == TipoContenido.AGENDA) {
      // poner todos los cursos, talleres, charlas y propuestas de hoy.
    }

    if (e.getTipoContenido() == TipoContenido.OFERTA_FORMATIVA) {
      // poner todos los cursos, talleres, charlas y propuestas de hoy.
    }

    if ((e.getTipoContenido() == TipoContenido.IMAGEN || e.getTipoContenido() == TipoContenido.VIDEO)
        && e.getFilePanel() == null) {
      enableMessages();
      addErrorMessage("Debe indicar un fichero");
      return false;
    }
    setMessagesEnabled(false);
    removeAll(e.getCalendarItems());
    setMessagesEnabled(true);
    e.getCalendarItems().clear();

    persist(e);
    return true;
  }
}
