package es.csa.fdt.cyldigital.recursos.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.Date;
import java.util.List;

import org.springframework.binding.message.MessageBuilder;
import org.springframework.binding.message.MessageContext;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;
import org.springframework.webflow.execution.RequestContextHolder;

import es.csa.fdt.cyldigital.centros.entity.Aula;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.recursos.entity.CategoriaRecurso;
import es.csa.fdt.cyldigital.recursos.entity.EstadosRecurso;
import es.csa.fdt.cyldigital.recursos.entity.Ordenador;
import es.csa.fdt.cyldigital.recursos.entity.RemoteNotice;
import es.csa.fdt.cyldigital.recursos.entity.Ordenador.EstadoOrdenador;
import es.csa.fdt.cyldigital.recursos.entity.Recurso.EstadoRecurso;
import es.csa.fdt.cyldigital.recursos.entity.RemoteNotice.NotificationType;
import es.csa.fdt.cyldigital.recursos.entity.RemoteNotice.Status;
import es.csa.fdt.cyldigital.servicios.entity.Actividad;
import es.csa.fdt.cyldigital.servicios.entity.PracticaTaller;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;


/**
 * Clase OrdenadorServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class OrdenadorServiceImpl extends AbstractServiceImpl<Ordenador, Long> implements OrdenadorService {

  private static final long serialVersionUID = -5575228180361917674L;

  private static final String DEFAULT_NOMBRE_CATEGORIA = "Ordenador";

  private static final String DEFAULT_ETIQUETADO_CATEGORIA = "ORDEN";

  private List<Aula> aulas;

  private List<Centro> centros;

  private List<CategoriaRecurso> categorias;

  private boolean cierreInmediato;

  private boolean comandoUsuario;

  private String mensaje;

  private int segundos;

  /**
   * Construye un nuevo objeto de la clase
   */
  public OrdenadorServiceImpl() {
    super();
    cierreInmediato = false;
    comandoUsuario = false;
    segundos = 60;
    mensaje = "";
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.recursos.service.OrdenadorService#getAulas()
   */
  public List<Aula> getAulas() {
    if (aulas == null) aulas = findAllOrder(Aula.class, "nombre", true);
    return aulas;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.recursos.service.OrdenadorService#getCentros()
   */
  public List<Centro> getCentros() {
    if (centros == null) centros = findAllOrder(Centro.class, "nombre", true);
    return centros;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.recursos.service.OrdenadorService#getCategoriasOrdenadores()
   */
  public List<CategoriaRecurso> getCategoriasOrdenadores() {
    return categorias != null ? categorias : findAllOrder(CategoriaRecurso.class, "nombre", true);
  }

  /**
   * Inicializa.
   * 
   * @param ordenador
   *           ordenador
   * @param u
   *           u
   */
  public void inicializa(Ordenador ordenador, UsuarioLdap u) {
    List<Aula> aulas = findWhere(Aula.class, "centro", u.getCentro());
    for (Aula a : aulas) {
      if (a.isLocalizacionPorDefecto()) ordenador.setAula(a);
    }

    CategoriaRecurso categoria = findOneWhere(CategoriaRecurso.class, "etiquetado", "ORD");
    ordenador.setCategoria(categoria);

    ordenador.setEstado(EstadoRecurso.STOCK);
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.recursos.service.OrdenadorService#persistOrdenador(es.csa.fdt.cyldigital.recursos.entity.Ordenador)
   */
  public boolean persistOrdenador(Ordenador ordenador) {
    if (!ordenador.getEstado().equals(EstadoRecurso.REPARACION) && ordenador.getReparador() != null
        && !ordenador.getReparador().isEmpty()) {
      ordenador.setReparador(null);
    }

    persistEstadosOrdenador(ordenador);
    return true;
  }

  /**
   * Persist estados ordenador.
   * 
   * @param ordenador
   *           ordenador
   */
  public void persistEstadosOrdenador(Ordenador ordenador) {
    if (ordenador.getId() != 0) {
      if (ordenador.getEstadosrecursos() != null
          && !ordenador.getEstadosrecursos().isEmpty()
          && !ordenador.getEstadosrecursos().get(ordenador.getEstadosrecursos().size() - 1).getEstado().equals(
              ordenador.getEstado())) {
        EstadosRecurso estado = new EstadosRecurso();
        estado.setFechaEstado(new Date());
        estado.setRecurso(ordenador);
        estado.setEstado(ordenador.getEstado());
        ordenador.getEstadosrecursos().add(estado);
      }
    } else {
      // si el recurso es nuevo, que grabe un primer estado
      EstadosRecurso estado = new EstadosRecurso();
      estado.setFechaEstado(new Date());
      estado.setRecurso(ordenador);
      estado.setEstado(ordenador.getEstado());
      ordenador.getEstadosrecursos().add(estado);
    }
    enableMessages();
    persist(ordenador);
  }

  /**
   * Elimina ordenador.
   * 
   * @param ordenador
   *           ordenador
   */
  public void removeOrdenador(Ordenador ordenador) {
    if (ordenador.getId() != 0) {
    	List<PracticaTaller> lpt = findWhere(PracticaTaller.class, "ordenador", ordenador);
    	
    	if (lpt.size() > 0)
    		addErrorMessage("recursos.error.practicasDeTaller");
    	else {
	    	setMessagesEnabled(false);

	    	List<EstadosRecurso> estadosRecursos = ordenador.getEstadosrecursos();
	    	for (EstadosRecurso er : estadosRecursos)
	    		remove(er);
	    	ordenador.getEstadosrecursos().clear();
	    	
	    	
	    	List<EstadosRecurso> estadosOrdenadores = ordenador.getEstadosordenadores();
	    	for (EstadosRecurso eo : estadosOrdenadores)
	    		remove(eo);
	    	ordenador.getEstadosordenadores().clear();

	    	List<Actividad> actividadesOrdenador = ordenador.getActividades();
	    	for (Actividad a : actividadesOrdenador) {
	    		a.getRecursos().remove(ordenador);
	    		persist(a);
	    	}
	    	ordenador.getActividades().clear();
	    	
	    	setMessagesEnabled(true);
	    	
	    	remove(ordenador);
    	}
    }
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.recursos.service.OrdenadorService#getOrdenadoresDisponibles()
   */
  public List<Ordenador> getOrdenadoresDisponibles() {
    return findAll(Ordenador.class);
  }

  /**
   * Encender ordenador.
   * 
   * @param idOrdenador
   *           id ordenador
   * @return true, if successful
   */
  public boolean encenderOrdenador(Long idOrdenador) {
    Ordenador ordenador = find(idOrdenador);
    if (ordenador != null && !ordenador.getEstadoOrdenador().equals(EstadoOrdenador.ENCENDIDO)) {
      if (ordenador.getIp() == null || ordenador.getIp().isEmpty()) {
        mostrarMensajeOrdenadorSinIp();
      } else {
        ordenador.setEstadoOrdenador(EstadoOrdenador.EN_MODIFICACION);
        persist(ordenador);

        RemoteNotice r = new RemoteNotice();
        r.setIpAddress(ordenador.getIp());
        r.setMacAddress(ordenador.getMac());
        r.setStatus(Status.PENDING);
        r.setServerId(ordenador.getCentro().getId());
        r.setNotificationType(NotificationType.DO_START);
        persist(r);
        return true;
      }
    }
    return false;
  }

  /**
   * Apagar ordenador.
   * 
   * @param idOrdenador
   *           id ordenador
   * @return true, if successful
   */
  public boolean apagarOrdenador(Long idOrdenador) {
    Ordenador ordenador = find(idOrdenador);
    if (ordenador != null && ordenador.getEstadoOrdenador().equals(EstadoOrdenador.ENCENDIDO)) {
      if (ordenador.getIp() == null || ordenador.getIp().isEmpty()) {
        mostrarMensajeOrdenadorSinIp();
      } else {

        ordenador.setEstadoOrdenador(EstadoOrdenador.EN_MODIFICACION);
        persist(ordenador);

        RemoteNotice r = new RemoteNotice();
        r.setIpAddress(ordenador.getIp());
        r.setMacAddress(ordenador.getMac());
        r.setStatus(Status.PENDING);
        r.setServerId(ordenador.getCentro().getId());
        r.setNotificationType(NotificationType.DO_SHOUTDOWN);
        disableMessages();
        persist(r);
        enableMessages();

        return true;
      }
    }
    return false;
  }

  /**
   * Bloquear.
   * 
   * @param idOrdenador
   *           id ordenador
   * @return true, if successful
   */
  public boolean bloquear(Long idOrdenador) {
    Ordenador ordenador = find(idOrdenador);
    if (ordenador != null) {
      ordenador.setNavegacionControlada(true);
      persist(ordenador);
      return true;
    }
    return false;
  }

  /**
   * Desbloquear.
   * 
   * @param idOrdenador
   *           id ordenador
   * @return true, if successful
   */
  public boolean desbloquear(Long idOrdenador) {
    Ordenador ordenador = find(idOrdenador);
    if (ordenador != null) {
      ordenador.setNavegacionControlada(false);
      persist(ordenador);
      return true;
    }
    return false;
  }

  /**
   * Reiniciar ordenador.
   * 
   * @param idOrdenador
   *           id ordenador
   * @return true, if successful
   */
  public boolean reiniciarOrdenador(Long idOrdenador) {
    Ordenador ordenador = find(idOrdenador);
    if (ordenador != null && ordenador.getEstadoOrdenador().equals(EstadoOrdenador.ENCENDIDO)) {
      if (ordenador.getIp() == null || ordenador.getIp().isEmpty()) {
        mostrarMensajeOrdenadorSinIp();
      } else {
        ordenador.setEstadoOrdenador(EstadoOrdenador.EN_MODIFICACION);
        persist(ordenador);

        RemoteNotice r = new RemoteNotice();
        r.setIpAddress(ordenador.getIp());
        r.setMacAddress(ordenador.getMac());
        r.setStatus(Status.PENDING);
        r.setServerId(ordenador.getCentro().getId());
        r.setNotificationType(NotificationType.DO_RESTART);
        super.disableMessages();
        persist(r);
        addInfoMessageAlways("Reinicio enviado.");
        super.enableMessages();
        return true;
      }
    }
    return false;
  }

  /**
   * Apagar todos.
   * 
   * @param ordenadores
   *           ordenadores
   * @return true, if successful
   */
  public boolean apagarTodos(List<Ordenador> ordenadores) {
    setMessagesEnabled(false);
    if (ordenadores != null) for (Ordenador o : ordenadores) {
      apagarOrdenador(o.getId());
    }
    setMessagesEnabled(true);
    addInfoMessageAlways("Apagado enviado a todos los ordenadores.");
    return true;
  }

  /**
   * Encender todos.
   * 
   * @param ordenadores
   *           ordenadores
   * @return true, if successful
   */
  public boolean encenderTodos(List<Ordenador> ordenadores) {
    setMessagesEnabled(false);
    if (ordenadores != null) for (Ordenador o : ordenadores) {
      encenderOrdenador(o.getId());
    }
    setMessagesEnabled(true);
    addInfoMessageAlways("Encendido enviado a todos los ordenadores");
    return true;
  }

  /**
   * Reiniciar todos.
   * 
   * @param ordenadores
   *           ordenadores
   * @return true, if successful
   */
  public boolean reiniciarTodos(List<Ordenador> ordenadores) {
    setMessagesEnabled(false);
    if (ordenadores != null) for (Ordenador o : ordenadores) {
      reiniciarOrdenador(o.getId());
    }
    setMessagesEnabled(true);
    addInfoMessageAlways("Reinicio enviado a todos los ordenadores");
    return true;
  }

  /**
   * Cierre session.
   * 
   * @param ordenadores
   *           ordenadores
   * @return true, if successful
   */
  public boolean cierreSession(List<Ordenador> ordenadores) {
    setMessagesEnabled(false);
    if (ordenadores != null) for (Ordenador o : ordenadores) {
      cierreSession(o.getId());
    }
    setMessagesEnabled(true);
    addInfoMessageAlways("Cierre de sesión enviado a todos los ordenadores");
    return true;
  }

  /**
   * Cierre session.
   * 
   * @param idOrdenador
   *           id ordenador
   * @return true, if successful
   */
  public boolean cierreSession(Long idOrdenador) {
    Ordenador ordenador = find(idOrdenador);
    if (ordenador != null && ordenador.getEstadoOrdenador().equals(EstadoOrdenador.ENCENDIDO)
        && ordenador.getFechaInicioSesion() != null) {
      if (ordenador.getIp() == null || ordenador.getIp().isEmpty()) {
        mostrarMensajeOrdenadorSinIp();
      } else {
        RemoteNotice r = new RemoteNotice();
        r.setIpAddress(ordenador.getIp());
        r.setMacAddress(ordenador.getMac());
        r.setStatus(Status.PENDING);
        r.setServerId(ordenador.getCentro().getId());
        r.setNotificationType(NotificationType.DO_LOGOUT);
        if (!isCierreInmediato()) {
          r.setMessage(getMensaje());
          if (getSegundos() > 0) r.setEntero(getSegundos());
          else r.setEntero(60);
        } else r.setEntero(0);
        super.disableMessages();
        persist(r);
        addInfoMessageAlways("Cierre de sesión enviado.");
        super.enableMessages();
        return true;
      }
    }
    return false;
  }

  /**
   * Enviar comando.
   * 
   * @param idOrdenador
   *           id ordenador
   * @return true, if successful
   */
  public boolean enviarComando(Long idOrdenador) {
    Ordenador ordenador = find(idOrdenador);
    if (ordenador != null && ordenador.getEstadoOrdenador().equals(EstadoOrdenador.ENCENDIDO)) {
      if (ordenador.getIp() == null || ordenador.getIp().isEmpty()) {
        mostrarMensajeOrdenadorSinIp();
      } else {
        RemoteNotice r = new RemoteNotice();
        r.setIpAddress(ordenador.getIp());
        r.setMacAddress(ordenador.getMac());
        r.setStatus(Status.PENDING);
        r.setServerId(ordenador.getCentro().getId());
        if (ordenador.getFechaInicioSesion() != null && isComandoUsuario()) r
            .setNotificationType(NotificationType.DO_RUN_USER_COMMAND);
        else r.setNotificationType(NotificationType.DO_RUN_COMMAND);
        r.setMessage(getMensaje());
        super.disableMessages();
        persist(r);
        super.enableMessages();
        addInfoMessageAlways("Comando enviado.");
        return true;
      }
    }
    return false;
  }

  /**
   * Envio mensaje.
   * 
   * @param idOrdenador
   *           id ordenador
   * @return true, if successful
   */
  public boolean envioMensaje(Long idOrdenador) {
    Ordenador ordenador = find(idOrdenador);
    if (ordenador != null && ordenador.getEstadoOrdenador().equals(EstadoOrdenador.ENCENDIDO)
        && ordenador.getFechaInicioSesion() != null) {
      if (ordenador.getIp() == null || ordenador.getIp().isEmpty()) {
        mostrarMensajeOrdenadorSinIp();
      } else {
        RemoteNotice r = new RemoteNotice();
        r.setIpAddress(ordenador.getIp());
        r.setMacAddress(ordenador.getMac());
        r.setStatus(Status.PENDING);
        r.setServerId(ordenador.getCentro().getId());
        r.setNotificationType(NotificationType.DO_SHOW_MSG);
        r.setMessage(getMensaje());
        super.disableMessages();
        persist(r);
        super.enableMessages();
        addInfoMessageAlways("Mensaje enviado.");
        return true;
      }
    }
    return false;
  }

  private void mostrarMensajeOrdenadorSinIp() {
    addErrorMessage("No se pueden realizar operaciones sobre ordenadores sin dirección IP");
  }

  /**
   * Generar codigo etiqueta.
   * 
   * @param idOrdenador
   *           id ordenador
   */
  public void generarCodigoEtiqueta(Long idOrdenador) {
    Ordenador ordenador = find(idOrdenador);
    String codigo = "";
    codigo = ordenador.getCategoria() != null ? ordenador.getCategoria().getEtiquetado() : "*ORD*";
    codigo += "-" + ordenador.getId();
    ordenador.setCodEtiqueta(codigo);
    persist(ordenador);
  }

  /**
   * Checks if is cierre inmediato.
   * 
   * @return true, if is cierre inmediato
   */
  public boolean isCierreInmediato() {
    return cierreInmediato;
  }

  /**
   * Asigna cierre inmediato.
   * 
   * @param cierreInmediato
   *          cierre inmediato
   */
  public void setCierreInmediato(boolean cierreInmediato) {
    this.cierreInmediato = cierreInmediato;
  }

  /**
   * Devuelve mensaje.
   * 
   * @return mensaje
   */
  public String getMensaje() {
    return mensaje;
  }

  /**
   * Asigna mensaje.
   * 
   * @param mensaje
   *          mensaje
   */
  public void setMensaje(String mensaje) {
    this.mensaje = mensaje;
  }

  /**
   * Devuelve segundos.
   * 
   * @return segundos
   */
  public int getSegundos() {
    return segundos;
  }

  /**
   * Asigna segundos.
   * 
   * @param segundos
   *          segundos
   */
  public void setSegundos(int segundos) {
    this.segundos = segundos;
  }

  /**
   * Checks if is comando usuario.
   * 
   * @return true, if is comando usuario
   */
  public boolean isComandoUsuario() {
    return comandoUsuario;
  }

  /**
   * Asigna comando usuario.
   * 
   * @param comandoUsuario
   *          comando usuario
   */
  public void setComandoUsuario(boolean comandoUsuario) {
    this.comandoUsuario = comandoUsuario;
  }

  /**
   * Añade info message always.
   * 
   * @param code
   *           code
   */
  public void addInfoMessageAlways(String code) {
    MessageContext context = RequestContextHolder.getRequestContext().getMessageContext();
    context.addMessage(new MessageBuilder().info().defaultText(code).code(code).args(null).build());
  }

}
