package es.csa.fdt.cyldigital.recursos.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.List;

import javax.persistence.Query;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.commons.persistence.browser.jpaql.FilterJPAQLByQuery;
import es.csa.fdt.cyldigital.centros.entity.Aula;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.centros.service.CentroServiceImpl;
import es.csa.fdt.cyldigital.core.service.BrowserFilteredByCenter;
import es.csa.fdt.cyldigital.recursos.entity.Ordenador;
import es.csa.fdt.cyldigital.recursos.entity.Recurso.EstadoRecurso;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario.Role;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;

/**
 * Servicio del Browser de Ordenadores.
 * 
 * @author csa
 */
@Service
@Repository
public class OrdenadoresBrowserImpl extends BrowserFilteredByCenter<Ordenador> implements
    OrdenadoresBrowser<CriterionJPAQL, String> {

  private static final long serialVersionUID = -7227366052245717441L;

  private List<Ordenador> ordenadores;

  private Aula aula;

  private EstadoRecurso estado;

  /**
   * Construye un nuevo objeto de la clase
   */
  public OrdenadoresBrowserImpl() {
    super();
    getSort().setSortAsc(true);
    getSort().setSortProperty("ip");
  }

  /**
   * Devuelve ordenadores.
   * 
   * @return ordenadores
   */
  public List<Ordenador> getOrdenadores() {
    if (ordenadores == null) {
      ordenadores = new ArrayList<Ordenador>();
    }
    return ordenadores;
  }

  /**
   * Asigna ordenadores.
   * 
   * @param ordenadores
   *          ordenadores
   */
  public void setOrdenadores(List<Ordenador> ordenadores) {
    this.ordenadores = ordenadores;
  }

  /**
   * Asigna records to ordenadores.
   */
  public void setRecordsToOrdenadores() {
    ordenadores = getRecords();
  }

  /**
   * Devuelve aula.
   * 
   * @return aula
   */
  public Aula getAula() {
    if (centro == null) return null;
    return aula;
  }

  /**
   * Asigna aula.
   * 
   * @param aula
   *          aula
   */
  public void setAula(Aula aula) {
    this.aula = aula;
  }

  /**
   * Devuelve estado.
   * 
   * @return estado
   */
  public EstadoRecurso getEstado() {
    return estado != null ? estado : null;
  }

  /**
   * Asigna estado.
   * 
   * @param estado
   *          estado
   */
  public void setEstado(EstadoRecurso estado) {
    this.estado = estado;
  }

  /**
   * Filter aula.
   */
  public void filterAula() {
    removeFilterAula();
    if (getAula() != null) filterByAula(getAula().getId());
  }

  private void removeFilterAula() {
    int i = -1;
    for (Filter<CriterionJPAQL, Ordenador> f : getFilters())
      if (f.getCriterion().getQuery().contains("aula.id")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
  }

  /**
   * Filter by aula.
   * 
   * @param idAula
   *           id aula
   */
  public void filterByAula(Long idAula) {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<Ordenador> filter = new FilterJPAQLByQuery<Ordenador>();

    query += " aula.id = ? ";
    filter.getCriterion().addParameter(idAula);

    for (Filter<CriterionJPAQL, Ordenador> f : getFilters())
      if (f.getCriterion().getQuery().contains("aula.id")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }

  /**
   * Find centro aula.
   * 
   * @param idCentro
   *           id centro
   * @param idAula
   *           id aula
   */
  public void findCentroAula(Long idCentro, Long idAula) {
    try {
      if (idCentro != null && idCentro > 0) {
        Query query = em.createQuery("SELECT e FROM " + Centro.class.getName() + " e WHERE id = :valor0");
        query.setParameter("valor0", idCentro);
        centro = (Centro) query.getSingleResult();
      }
      if (idAula != null && idAula > 0) {
        Query query2 = em.createQuery("SELECT e FROM " + Aula.class.getName() + " e WHERE id = :valor0");
        query2.setParameter("valor0", idAula);
        aula = (Aula) query2.getSingleResult();
      }
    } catch (RuntimeException e) {
      e.printStackTrace();
    }
  }

  /**
   * Filter estado.
   */
  public void filterEstado() {
    removeFilterEstado();
    if (getEstado() != null) filterByEstado(getEstado());
  }

  private void removeFilterEstado() {
    int i = -1;
    for (Filter<CriterionJPAQL, Ordenador> f : getFilters())
      if (f.getCriterion().getQuery().contains("estado")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
  }

  /**
   * Filter by estado.
   * 
   * @param estado
   *           estado
   */
  public void filterByEstado(EstadoRecurso estado) {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<Ordenador> filter = new FilterJPAQLByQuery<Ordenador>();

    query += " estado = ? ";
    filter.getCriterion().addParameter(estado);

    for (Filter<CriterionJPAQL, Ordenador> f : getFilters())
      if (f.getCriterion().getQuery().contains("estado")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }

  /**
   * Asigna filter centro.
   * 
   * @param usuario
   *           usuario
   * @param serviceCentros
   *           service centros
   */
  public void setFilterCentro(UsuarioLdap usuario, CentroServiceImpl serviceCentros) {
	  super.user = usuario;

	  if (centro == null) {
		  //if (user.applyFilterCentro() && (user.getCentro() != null)) {  //Así lo tenía castilla y león, pero en Xunta de galicia dependiendo de
		  																   //la pantalla puede flitrar o no por centro, no solo el rol
		  																   //define tal filtro.
		  if (!user.getRolesList().contains(Role.ROLE_ADMIN) && (user.getCentro() != null)) { //Puesto así para Xunta de galicia
			  super.removeFilterCentro();
			  
			  if (user.getCentro() != null) {
				  filterByCentro(user.getCentro().getId());
				  super.centro = serviceCentros.find(Centro.class, user.getCentro().getId());
			  }
		  }
	  }
  }

  /**
   * Refresh all records.
   */
  public void refreshAllRecords() {
    for (Ordenador o : getOrdenadores()) {
      getEntityManager().refresh(o);
    }
  }
}
