package es.csa.fdt.cyldigital.recursos.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.List;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.recursos.entity.CategoriaRecurso;
import es.csa.fdt.cyldigital.recursos.entity.Panel;
import es.csa.fdt.cyldigital.recursos.entity.Tema;

/**
 * Clase PanelServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class PanelServiceImpl extends AbstractServiceImpl<Panel, Long> implements PanelService {

  private static final long serialVersionUID = -5575228180361917674L;

  private Centro centro;
  private List<Panel> paneles;

  /**
   * Construye un nuevo objeto de la clase
   */
  public PanelServiceImpl() {
    super();
  }

  /**
   * Devuelve temas.
   * 
   * @return temas
   */
  public List<Tema> getTemas() {
    return findAll(Tema.class);
  }

  /**
   * Inicializa panel.
   * 
   * @param panel
   *           panel
   */
  public void inicializaPanel(Panel panel) {
    // TODO como no repetir el codigo de ordenador?
    List<CategoriaRecurso> categorias = findAllOrder(CategoriaRecurso.class, "nombre", true);
    if (categorias.size() > 1) {
      panel.setCategoria(categorias.get(1));
    }
  }

  /**
   * Persist panel.
   * 
   * @param panel
   *           panel
   */
  public void persistPanel(Panel panel) {
    persist(panel);
  }

  /**
   * Devuelve centro.
   * 
   * @return centro
   */
  public Centro getCentro() {
    return centro;
  }

  /**
   * Devuelve paneles.
   * 
   * @return paneles
   */
  public List<Panel> getPaneles() {
    return paneles;
  }

  /**
   * Asigna paneles.
   * 
   * @param paneles
   *          paneles
   */
  public void setPaneles(List<Panel> paneles) {
    this.paneles = paneles;
  }

}
