package es.csa.fdt.cyldigital.recursos.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.Date;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.recursos.entity.PedidoRecurso;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;

/**
 * Clase PedidoRecursoServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class PedidoRecursoServiceImpl extends AbstractServiceImpl<PedidoRecurso, Long> implements PedidoRecursoService {

  private static final long serialVersionUID = -5575228180361917674L;

  /**
   * Construye un nuevo objeto de la clase
   */
  public PedidoRecursoServiceImpl() {
    super();
  }

  /**
   * Persist pedido recurso.
   * 
   * @param pedido
   *           pedido
   * @return true, if successful
   */
  public boolean persistPedidoRecurso(PedidoRecurso pedido) {
    if (pedido.getFechaEntrega() != null) {
      if (pedido.getFechaEntrega().before(new Date())) return false;
    }

    if (pedido.getProveedor() == null) {
      addErrorMessage("El proveedor es obligatorio.");
      return false;
    }

    if (pedido.getCantidad() <= 0) {
      addErrorMessage("La cantidad debe ser mayor de cero.");
      return false;
    }

    persist(pedido);
    return true;

  }

  /**
   * Inicializa pedido recurso.
   * 
   * @param pedido
   *           pedido
   * @param u
   *           u
   */
  public void inicializaPedidoRecurso(PedidoRecurso pedido, UsuarioLdap u) {
    pedido.setFechaPedido(new Date());
    pedido.setCentro(u.getCentro());

  }
}
