package es.csa.fdt.cyldigital.recursos.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.Date;
import java.util.List;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.centros.entity.Aula;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.recursos.entity.CategoriaRecurso;
import es.csa.fdt.cyldigital.recursos.entity.EstadosRecurso;
import es.csa.fdt.cyldigital.recursos.entity.Ordenador;
import es.csa.fdt.cyldigital.recursos.entity.Recurso;
import es.csa.fdt.cyldigital.recursos.entity.Recurso.EstadoRecurso;
import es.csa.fdt.cyldigital.servicios.entity.Actividad;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;


/**
 * Clase RecursoServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class RecursoServiceImpl extends AbstractServiceImpl<Recurso, Long> implements RecursoService {

  private static final long serialVersionUID = -5575228180361917674L;

  private static final String DEFAULT_NOMBRE_CATEGORIA = "Recurso";

  private static final String DEFAULT_ETIQUETADO_CATEGORIA = "RECUR";

  private List<Aula> aulas;

  private List<Centro> centros;

  private List<CategoriaRecurso> categorias;

  /**
   * Construye un nuevo objeto de la clase
   */
  public RecursoServiceImpl() {
    super();
  }

  /**
   * Inicializa.
   * 
   * @param recurso
   *           recurso
   * @param u
   *           u
   */
  public void inicializa(Recurso recurso, UsuarioLdap u) {
    List<Aula> aulas = findWhere(Aula.class, "centro", u.getCentro());
    for (Aula a : aulas) {
      if (a.isLocalizacionPorDefecto()) recurso.setAula(a);
    }

    recurso.setEstado(EstadoRecurso.STOCK);
  }

  /**
   * Devuelve aulas.
   * 
   * @return aulas
   */
  public List<Aula> getAulas() {
    if (aulas == null) aulas = findAllOrder(Aula.class, "nombre", true);
    return aulas;
  }

  /**
   * Devuelve centros.
   * 
   * @return centros
   */
  public List<Centro> getCentros() {
    if (centros == null) centros = findAllOrder(Centro.class, "nombre", true);
    return centros;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.recursos.service.RecursoService#getCategoriasOtrosRecursos()
   */
  public List<CategoriaRecurso> getCategoriasOtrosRecursos() {
    return categorias != null ? categorias : findAllOrder(CategoriaRecurso.class, "nombre", true);
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.recursos.service.RecursoService#persistRecurso(es.csa.fdt.cyldigital.recursos.entity.Recurso)
   */
  public boolean persistRecurso(Recurso recurso) {
    if (recurso.getCategoria().isCiclico()) {
      if (recurso.getStockSeguridad() == 0 || recurso.getPedidoOptimo() == 0) {
        addErrorMessage("Debe indicar el stock de seguridad y el pedido óptimo");
        return false;
      }
    }
    
    /* TODO: Comprobar más tarde, quitado para Xunta de Galicia
    persistEstadosRecurso(recurso);*/
    
    /* TODO: Comprobar más tarde, puesto en lugar de la linea anterior para Xunta de Galicia */
    persist(recurso);

    return true;
  }

  /**
   * Persist estados recurso.
   * 
   * @param recurso
   *           recurso
   */
  public void persistEstadosRecurso(Recurso recurso) {

    if (recurso.getId() != 0) {
      if (recurso.getEstadosrecursos().size() == 0) {
        // si por algún motivo no se ha grabado el primer estado, se graba ahora.
        grabarNuevoEstadoRecurso(recurso);
      } else {
        EstadosRecurso estadoanterior;
        estadoanterior = recurso.getEstadosrecursos().get(recurso.getEstadosrecursos().size() - 1);

        if (recurso.getEstado().equals(estadoanterior.getEstado())) {} else {
          grabarNuevoEstadoRecurso(recurso);
        }
      }
    } else {
      // si el recurso es nuevo, que grabe un primer estado
      grabarNuevoEstadoRecurso(recurso);

    }
    persist(recurso);
  }

  private void grabarNuevoEstadoRecurso(Recurso recurso) {
    EstadosRecurso estado = new EstadosRecurso();
    estado.setFechaEstado(new Date());
    estado.setRecurso(recurso);
    estado.setEstado(recurso.getEstado());
    recurso.getEstadosrecursos().add(estado);
  }

  /**
   * Elimina recurso.
   * 
   * @param recurso
   *           recurso
   */
  public void removeRecurso(Recurso recurso) {
    if (recurso.getId() != 0) {
    	setMessagesEnabled(false);

    	List<EstadosRecurso> estadosRecursos = recurso.getEstadosrecursos();
    	for (EstadosRecurso er : estadosRecursos)
    		remove(er);
    	recurso.getEstadosrecursos().clear();

    	List<Actividad> actividadesRecurso = recurso.getActividades();
    	for (Actividad a : actividadesRecurso) {
    		a.getRecursos().remove(recurso);
    		persist(a);
    	}
    	recurso.getActividades().clear();

    	setMessagesEnabled(true);
    	
    	remove(recurso);
    }
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.recursos.service.RecursoService#getRecursosDisponibles()
   */
  public List<Recurso> getRecursosDisponibles() {
    return findAll(Recurso.class);
  }

  /**
   * Generar codigo etiqueta.
   * 
   * @param idRecurso
   *           id recurso
   */
  public void generarCodigoEtiqueta(Long idRecurso) {
    Recurso recurso = find(idRecurso);
    String codigo = "";
    codigo = recurso.getCategoria() != null ? recurso.getCategoria().getEtiquetado() : "*REC*";
    codigo += "-" + recurso.getId();
    recurso.setCodEtiqueta(codigo);
    persist(recurso);
  }

  /**
   * Checks if is ordenador.
   * 
   * @param recurso
   *           recurso
   * @return true, if is ordenador
   */
  public boolean isOrdenador(Recurso recurso) {
    return recurso instanceof Ordenador;
  }
}
