package es.csa.fdt.cyldigital.recursos.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.List;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.commons.persistence.browser.jpaql.FilterJPAQLByQuery;
import es.csa.fdt.cyldigital.centros.service.CentroServiceImpl;
import es.csa.fdt.cyldigital.core.service.BrowserFilteredByCenter;
import es.csa.fdt.cyldigital.language.entity.Language;
import es.csa.fdt.cyldigital.recursos.entity.CategoriaRecurso;
import es.csa.fdt.cyldigital.recursos.entity.Recurso;
import es.csa.fdt.cyldigital.recursos.entity.Recurso.EstadoRecurso;

/**
 * Servicio del Browser de Recursos.
 * 
 * @author csa
 */
@Service
@Repository
public class RecursosBrowserImpl extends BrowserFilteredByCenter<Recurso> implements
    RecursosBrowser<CriterionJPAQL, String> {

  private static final long serialVersionUID = -7227366052245717441L;

  @SuppressWarnings("unused")
  private long idCentro;

  private CategoriaRecurso categoria;

  @SuppressWarnings("unused")
  private long idCategoria;

  private EstadoRecurso estado;

  private String customFilterProperty;

  /**
   * Construye un nuevo objeto de la clase
   */
  public RecursosBrowserImpl() {
    super();
    
    getSort().setSortAsc(false);
    getSort().setSortProperty("id");
  }
  
  public void applyOtrosRecursosCondition() {
		this.clearConditions();
		
		this.getCondition().getCriterion().setQuery("(e.categoria.etiquetado NOT LIKE 'ORD') AND (e.categoria.etiquetado NOT LIKE 'ROU')");

		this.applyCondition();
  }

  /**
   * Devuelve id centro.
   * 
   * @return id centro
   */
  public long getIdCentro() {
    return centro != null ? centro.getId() : 0;
  }

  /**
   * Asigna id centro.
   * 
   * @param idCentro
   *          id centro
   */
  public void setIdCentro(long idCentro) {
    this.idCentro = idCentro;
  }

  /**
   * Devuelve categoria.
   * 
   * @return categoria
   */
  public CategoriaRecurso getCategoria() {
    return categoria;
  }

  /**
   * Asigna categoria.
   * 
   * @param categoria
   *          categoria
   */
  public void setCategoria(CategoriaRecurso categoria) {
    this.categoria = categoria;
  }

  /**
   * Devuelve id categoria.
   * 
   * @return id categoria
   */
  public long getIdCategoria() {
    return categoria != null ? categoria.getId() : 0;
  }

  /**
   * Asigna id categoria.
   * 
   * @param idCategoria
   *          id categoria
   */
  public void setIdCategoria(long idCategoria) {
    this.idCategoria = idCategoria;
  }

  /**
   * Devuelve estado.
   * 
   * @return estado
   */
  public EstadoRecurso getEstado() {
    return estado != null ? estado : null;
  }

  /**
   * Asigna estado.
   * 
   * @param estado
   *          estado
   */
  public void setEstado(EstadoRecurso estado) {
    this.estado = estado;
  }

  /**
   * Devuelve otros recursos.
   * 
   * @return otros recursos
   */
  public List<Recurso> getOtrosRecursos() {
    List<Recurso> recursos = getRecords();
    List<Recurso> otros = new ArrayList<Recurso>();

    for (Recurso r : recursos) {
      if (!r.isOrdenador() && !r.isRouter()) {
        otros.add(r);
      }
    }
    return otros;
  }

  /**
   * Apply custom filter.
   */
  public void applyCustomFilter() {
    String query = null;
    clearFilters();
    if (getCustomFilterProperty() != null && !getCustomFilterProperty().trim().isEmpty()) {
      FilterJPAQLByQuery<Recurso> filter = new FilterJPAQLByQuery<Recurso>();
      query = " nombre = ?";
      filter.getCriterion().addParameter(getCustomFilterProperty());
      filter.getCriterion().setQuery(query);
      filters.add(filter);

    }
  }

  /**
   * Filter categoria.
   */
  public void filterCategoria() {
    removeFilterCategoria();
    if (getCategoria() != null) filterByCategoria(getCategoria().getId());
  }

  private void removeFilterCategoria() {
    int i = -1;
    for (Filter<CriterionJPAQL, Recurso> f : getFilters())
      if (f.getCriterion().getQuery().contains("categoria.id")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
  }

  /**
   * Filter by categoria.
   * 
   * @param idCategoria
   *           id categoria
   */
  public void filterByCategoria(Long idCategoria) {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<Recurso> filter = new FilterJPAQLByQuery<Recurso>();

    query += " categoria.id = ? ";
    filter.getCriterion().addParameter(idCategoria);

    for (Filter<CriterionJPAQL, Recurso> f : getFilters())
      if (f.getCriterion().getQuery().contains("categoria.id")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }

  /**
   * Filter estado.
   */
  public void filterEstado() {
    removeFilterEstado();
    if (getEstado() != null) filterByEstado(getEstado());
  }

  private void removeFilterEstado() {
    int i = -1;
    for (Filter<CriterionJPAQL, Recurso> f : getFilters())
      if ((f.getCriterion().getQuery() != null) && (f.getCriterion().getQuery().contains("estado")))
    	  i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
  }

  /**
   * Filter by estado.
   * 
   * @param estado
   *           estado
   */
  public void filterByEstado(EstadoRecurso estado) {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<Recurso> filter = new FilterJPAQLByQuery<Recurso>();

    query += " estado = ? ";
    filter.getCriterion().addParameter(estado);

    for (Filter<CriterionJPAQL, Recurso> f : getFilters())
      if (f.getCriterion().getQuery().contains("estado")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }

  /**
   * Filter estado activo.
   */
  public void filterEstadoActivo() {
    removeFilterEstado();
    filterByEstadoActivo();
  }

  /**
   * Filter by estado activo.
   */
  public void filterByEstadoActivo() {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<Recurso> filter = new FilterJPAQLByQuery<Recurso>();

    query += " estado = ? ";
    filter.getCriterion().addParameter(EstadoRecurso.ACTIVO);

    for (Filter<CriterionJPAQL, Recurso> f : getFilters())
      if ((f.getCriterion().getQuery() != null) && (f.getCriterion().getQuery().contains("estado")))
    	  flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }

  /**
   * Asigna custom filter property.
   * 
   * @param customFilterProperty
   *          custom filter property
   */
  public void setCustomFilterProperty(String customFilterProperty) {
    this.customFilterProperty = customFilterProperty;
  }

  /**
   * Devuelve custom filter property.
   * 
   * @return custom filter property
   */
  public String getCustomFilterProperty() {
    return customFilterProperty;
  }

  /**
   * Find centro categoria.
   * 
   * @param serviceCentros
   *           service centros
   * @param idCentro
   *           id centro
   * @param serviceCateogrias
   *           service cateogrias
   * @param idCategoria
   *           id categoria
   */
  public void findCentroCategoria(CentroServiceImpl serviceCentros, Long idCentro,
      CategoriaRecursoServiceImpl serviceCateogrias, Long idCategoria) {
    centro = serviceCentros.find(idCentro);
    categoria = serviceCateogrias.find(idCategoria);
  }

  private void removeFilterQuitarRecursosEnReparacion() {
    int i = -1;
    int j = -1;

    for (Filter<CriterionJPAQL, Recurso> f : getFilters()) {
      if (f.getCriterion().getQuery().contains("estado")) {
        // el filtro de estado está dos veces.
        if (i == -1) i = getFilters().indexOf(f);
        else j = getFilters().indexOf(f);
      }
    }

    if (i > -1) getFilters().remove(i);
    if (j > -1) getFilters().remove(j);
  }

  /**
   * Asigna filter quitar recursos en reparacion.
   */
  public void setFilterQuitarRecursosEnReparacion() {
    removeFilterQuitarRecursosEnReparacion();
    String query = "";
    FilterJPAQLByQuery<Recurso> filter = new FilterJPAQLByQuery<Recurso>();

    query += " estado != ? and estado != ? ";
    filter.getCriterion().addParameter(EstadoRecurso.AVERIADO);
    filter.getCriterion().addParameter(EstadoRecurso.REPARACION);

    filter.getCriterion().setQuery(query);
    getFilters().add(filter);
  }

}
