package es.csa.fdt.cyldigital.recursos.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.calendario.entity.WifiCalendarItem;
import es.csa.fdt.cyldigital.centros.entity.Aula;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.recursos.entity.CategoriaRecurso;
import es.csa.fdt.cyldigital.recursos.entity.Router;
import es.csa.fdt.cyldigital.recursos.entity.Recurso.EstadoRecurso;


/**
 * Clase RouterServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class RouterServiceImpl extends AbstractServiceImpl<Router, Long> implements RouterService {

  private static final long serialVersionUID = -5575228180361917674L;

  private static final String DEFAULT_NOMBRE_CATEGORIA = "Router";

  private static final String DEFAULT_ETIQUETADO_CATEGORIA = "ROUTER";

  private List<Centro> centros;

  private List<CategoriaRecurso> categorias;

  private String clave;

  /**
   * Construye un nuevo objeto de la clase
   */
  public RouterServiceImpl() {
    super();
  }

  /**
   * Inicializa.
   * 
   * @param r
   *           r
   */
  public void inicializa(Router r) {
    CategoriaRecurso c = findOneWhere(CategoriaRecurso.class, "etiquetado", "ROU");
    r.setCategoria(c);
    r.setEstado(EstadoRecurso.STOCK);
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.recursos.service.RouterService#getCentros()
   */
  public List<Centro> getCentros() {
    if (centros == null) centros = findAllOrder(Centro.class, "nombre", true);
    return centros;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.recursos.service.RouterService#getCategoriasRouters()
   */
  public List<CategoriaRecurso> getCategoriasRouters() {
    return categorias != null ? categorias : findAllOrder(CategoriaRecurso.class, "nombre", true);
  }

  /**
   * Devuelve clave.
   * 
   * @return clave
   */
  public String getClave() {
    return clave;
  }

  /**
   * Asigna clave.
   * 
   * @param clave
   *          clave
   */
  public void setClave(String clave) {
    this.clave = clave;
  }

  /**
   * Actualiza datos.
   * 
   * @param router
   *           router
   */
  public void actualizaDatos(Router router) {
    List<CategoriaRecurso> categorias = findAllOrder(CategoriaRecurso.class, "nombre", true);
    if (categorias.size() > 0) {
      router.setCategoria(categorias.get(0));
    }
    // Si no existen servicios de char, creo uno por defecto, y le asocio a la actividad.
    else {
      CategoriaRecurso categoria = new CategoriaRecurso();
      categoria.setNombre(DEFAULT_NOMBRE_CATEGORIA);
      categoria.setEtiquetado(DEFAULT_ETIQUETADO_CATEGORIA);
      disableMessages();
      persist(categoria);
      enableMessages();
      this.categorias = new ArrayList<CategoriaRecurso>();
      this.categorias.add(categoria);
      router.setCategoria(categoria);
    }
    if (router.getId() != 0) {
      this.clave = router.getClave();
    }
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.recursos.service.RouterService#persistRouter(es.csa.fdt.cyldigital.recursos.entity.Router)
   */
  public boolean persistRouter(Router router) {
    if (router.getUltimaClaveWifi() == null || !clave.equals(router.getUltimaClaveWifi().getClave())) {
      if (router.getId() != 0) {
        router.setEndDateClaveWifi(new Date());
      }
      WifiCalendarItem item = new WifiCalendarItem();
      item.setStart(new Date());
      item.setClave(clave);
      item.setRouter(router);
      router.addClaveWifi(item);
    }
    Centro c = find(Centro.class, router.getCentro().getId());
    if (c.getRouterPrincipal() == null) {
      c.setRouterPrincipal(router);
    }

    if (router.getId() == 0)// actualizar solo cuando es nuevo
    {
      List<Aula> aulas = findWhere(Aula.class, "centro", router.getCentro());
      for (Aula a : aulas) {
        if (a.isLocalizacionPorDefecto()) router.setAula(a);
      }

      CategoriaRecurso categoria = findOneWhere(CategoriaRecurso.class, "etiquetado", "ROU");
      router.setCategoria(categoria);
    }

    persist(router);
    return true;
  }

  /**
   * Elimina router.
   * 
   * @param router
   *           router
   */
  public void removeRouter(Router router) {
    if (router.getId() != 0) remove(router);
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.recursos.service.RouterService#getRoutersDisponibles()
   */
  public List<Router> getRoutersDisponibles() {
    return findAll(Router.class);
  }

  /**
   * Generar codigo etiqueta.
   * 
   * @param idRouter
   *           id router
   */
  public void generarCodigoEtiqueta(Long idRouter) {
    Router router = find(idRouter);
    String codigo = "";
    codigo = router.getCategoria() != null ? router.getCategoria().getEtiquetado() : "*ROU*";
    codigo += "-" + router.getId();
    router.setCodEtiqueta(codigo);
    persist(router);
  }

}
