package es.csa.fdt.cyldigital.servicios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import es.csa.fdt.cyldigital.calendario.entity.ActividadCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.AsistenciaCalendarItem;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.language.entity.ActividadLanguage;
import es.csa.fdt.cyldigital.recursos.entity.Recurso;

/**
 * Clase Actividad.
 * 
 * @author csa
 */
@Entity
@Table(name = "actividades")
@Audited
@Inheritance(strategy = InheritanceType.JOINED)
@AttributeOverride(name = "id", column = @Column(name = "idActividad"))
public abstract class Actividad extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = -4709108123316711478L;

  /** servicio. */
  protected Servicio servicio;

  private boolean cerrada = false;
  private boolean tratada = false;
  private boolean videoconferencia = false;
  private boolean iniciativaSingular = false;
  
  //RF_XEA_22
  private Date fechaPublicacion;
  private Date horaPublicacion;
  //RF_XEA_22
  
  //RF_15_04
  private Date fechaLimiteInscripcion;
  private Date horaLimiteInscripcion;
  //RF_15_04
  
  /** observaciones actividad. */
  protected List<ObservacionActividad> observacionesActividad;
  
  /** calendar items. */
  protected List<ActividadCalendarItem> calendarItems;
  
  /** asistencias. */
  protected List<AsistenciaCalendarItem> asistencias;
  
  /** recursos. */
  protected List<Recurso> recursos;
  
  private List<ActividadLanguage> textos;

  // ------------------------------------------------------------------------------------------------------------GETTERS

  /**
   * Devuelve servicio.
   * 
   * @return servicio
   */
  @ManyToOne()
  @JoinColumn(name = "idServicio", nullable = true)
  //@NotNull
  public Servicio getServicio() {
    return servicio;
  }

  /**
   * Asigna servicio.
   * 
   * @param servicio
   *          servicio
   */
  public void setServicio(Servicio servicio) {
    this.servicio = servicio;
  }

  /**
   * Devuelve recursos.
   * 
   * @return recursos
   */
  @ManyToMany(targetEntity = Recurso.class, cascade = { CascadeType.PERSIST, CascadeType.MERGE, CascadeType.REFRESH })
  @JoinTable(name = "recursos_actividades", joinColumns = @JoinColumn(name = "idActividad"), inverseJoinColumns = @JoinColumn(name = "idRecurso"))
  @NotAudited
  public List<Recurso> getRecursos() {
    return recursos;
  }

  /**
   * Asigna recursos.
   * 
   * @param recursos
   *          recursos
   */
  public void setRecursos(List<Recurso> recursos) {
    this.recursos = recursos;
  }

  /**
   * Devuelve observaciones actividad.
   * 
   * @return observaciones actividad
   */
  @OneToMany(mappedBy = "actividad", cascade = { CascadeType.ALL })
  public List<ObservacionActividad> getObservacionesActividad() {
    return observacionesActividad;
  }

  /**
   * Asigna observaciones actividad.
   * 
   * @param observacionesActividad
   *          observaciones actividad
   */
  public void setObservacionesActividad(List<ObservacionActividad> observacionesActividad) {
    this.observacionesActividad = observacionesActividad;
  }

  /**
   * Devuelve calendar items.
   * 
   * @return calendar items
   */
  @OneToMany(fetch = FetchType.LAZY, mappedBy = "actividad", cascade = { CascadeType.ALL })
  public List<ActividadCalendarItem> getCalendarItems() {
    return calendarItems;
  }

  /**
   * Asigna calendar items.
   * 
   * @param calendarItems
   *          calendar items
   */
  public void setCalendarItems(List<ActividadCalendarItem> calendarItems) {
    this.calendarItems = calendarItems;
  }

  /**
   * Añade calendar item.
   * 
   * @param item
   *           item
   */
  public void addCalendarItem(ActividadCalendarItem item) {
    calendarItems.add(item);
  }

  /**
   * Devuelve asistencias.
   * 
   * @return asistencias
   */
  @OneToMany(fetch = FetchType.LAZY, mappedBy = "actividad", cascade = { CascadeType.ALL })
  public List<AsistenciaCalendarItem> getAsistencias() {
    return asistencias;
  }

  /**
   * Asigna asistencias.
   * 
   * @param asistencias
   *          asistencias
   */
  public void setAsistencias(List<AsistenciaCalendarItem> asistencias) {
    this.asistencias = asistencias;
  }

  /**
   * Añade asistencia.
   * 
   * @param item
   *           item
   */
  public void addAsistencia(AsistenciaCalendarItem item) {
    asistencias.add(item);
  }

  /**
   * Checks if is cerrada.
   * 
   * @return true, if is cerrada
   */
  public boolean isCerrada() {
    return cerrada;
  }

  /**
   * Asigna cerrada.
   * 
   * @param cerrado
   *          cerrada
   */
  public void setCerrada(boolean cerrado) {
    this.cerrada = cerrado;
  }
  
  //RF_XEA_22  
	/*
	 * (non-Javadoc)
	 * 
	 * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getFechaInicio()
	 */
	@Temporal(TemporalType.DATE)
	public Date getFechaPublicacion() {
		return fechaPublicacion;
	}

	/**
	 * Asigna fecha Publicacion.
	 * 
	 * @param fechaPublicacion
	 *            fecha Publicacion
	 */
	public void setFechaPublicacion(Date fechaPublicacion) {
		this.fechaPublicacion = fechaPublicacion;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getHoraFin()
	 */
	@Temporal(TemporalType.TIME)
	public Date getHoraPublicacion() {
		return horaPublicacion;
	}

	/**
	 * Asigna hora Publicacion
	 * 
	 * @param horaPublicacion
	 *            hora Publicacion
	 */
	public void setHoraPublicacion(Date horaPublicacion) {
		this.horaPublicacion = horaPublicacion;
	}  
  //RF_XEA_22
	
  //RF_15_04
	/*
	 * (non-Javadoc)
	 * 
	 * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getFechaLimiteInscripcion()
	 */
	@Temporal(TemporalType.DATE)
	public Date getFechaLimiteInscripcion() {
		return fechaLimiteInscripcion;
	}

	/**
	 * Asigna fecha limite inscripcion.
	 * 
	 * @param fechaLimiteInscripcion
	 *            Fecha Limite Inscripcion
	 */
	public void setFechaLimiteInscripcion(Date fechaLimiteInscripcion){
		this.fechaLimiteInscripcion = fechaLimiteInscripcion;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getHoraLimiteInscripcion()
	 */
	@Temporal(TemporalType.TIME)
	public Date getHoraLimiteInscripcion() {
		return horaLimiteInscripcion;
	}

	/**
	 * Asigna hora limite inscripcion
	 * 
	 * @param horaLimiteInscripcion
	 *            hora Limite Inscripcion
	 */
	public void setHoraLimiteInscripcion(Date horaLimiteInscripcion) {
		this.horaLimiteInscripcion = horaLimiteInscripcion;
	}  
  //RF_15_04
	
	
  /**
   * Checks if is tratada.
   * 
   * @return true, if is tratada
   */
  public boolean isTratada() {
    return tratada;
  }

  /**
   * Asigna tratada.
   * 
   * @param tratado
   *          tratada
   */
  public void setTratada(boolean tratado) {
    this.tratada = tratado;
  }
  
	/**
	 * Devuelve textos de la actividad
	 * 
	 * @return textos
	 */
	@OneToMany(cascade = CascadeType.ALL)
	@JoinColumn(name = "idActividad")
	public List<ActividadLanguage> getTextos() {
		return textos;
	}

	/**
	 * Asigna los textos de la actividad
	 * 
	 * @param textos
	 *          textos
	 */
	public void setTextos(List<ActividadLanguage> textos) {
		this.textos = textos;
	}

  /**
   * Devuelve nombre.
   * 
   * @return nombre
   */
  /*@Transient
  public abstract String getNombre();*/

  /**
   * Devuelve centro.
   * 
   * @return centro
   */
  @Transient
  public abstract Centro getCentro();

  /**
   * Devuelve fecha inicio.
   * 
   * @return fecha inicio
   */
  @Transient
  public abstract Date getFechaInicio();

  /**
   * Devuelve fecha fin.
   * 
   * @return fecha fin
   */
  @Transient
  public abstract Date getFechaFin();

  /**
   * Devuelve hora inicio.
   * 
   * @return hora inicio
   */
  @Transient
  public abstract Date getHoraInicio();

  /**
   * Devuelve hora fin.
   * 
   * @return hora fin
   */
  @Transient
  public abstract Date getHoraFin();

  /**
   * Devuelve Si la actividad es externa.
   * 
   * @return si es externa
   */
  @Transient
  public abstract boolean isExterna();
  
  /**
   * Devuelve Fecha y hora de publicacion
   * 
   * @return Fecha y hora de publicacion
   */
  @Transient
  public Date getFechaHoraPublicacion(){
	  
	  if (getFechaPublicacion() != null){
		  Calendar calendarioFecha = Calendar.getInstance();
		  Calendar calendarioHora = Calendar.getInstance();
		  calendarioFecha.setTime(getFechaPublicacion());
		  
		  if(getHoraPublicacion() != null){
			  calendarioHora.setTime(getHoraPublicacion()); 
			  calendarioFecha.set(Calendar.HOUR_OF_DAY, calendarioHora.get(Calendar.HOUR_OF_DAY));
			  calendarioFecha.set(Calendar.MINUTE, calendarioHora.get(Calendar.MINUTE));
			  calendarioFecha.set(Calendar.SECOND, calendarioHora.get(Calendar.SECOND));
		  } else {
			  calendarioFecha.set(Calendar.HOUR_OF_DAY, calendarioHora.get(Calendar.HOUR_OF_DAY));
			  calendarioFecha.set(Calendar.MINUTE, calendarioHora.get(Calendar.MINUTE));
			  calendarioFecha.set(Calendar.SECOND, calendarioHora.get(Calendar.SECOND));
		  }
		  
		  return calendarioFecha.getTime();
	  }
	  
	  return null;
  }
  
  /**
   * Devuelve Fecha y hora limite de inscripcion
   * 
   * @return Fecha y hora limite de inscripcion
   */
  @Transient
  public Date getFechaHoraLimiteInscripcion(){
	  
	  if (getFechaLimiteInscripcion() != null){
		  Calendar calendarioFecha = Calendar.getInstance();
		  Calendar calendarioHora = Calendar.getInstance();
		  calendarioFecha.setTime(getFechaLimiteInscripcion());
		  
		  if(getHoraLimiteInscripcion() != null){
			  calendarioHora.setTime(getHoraLimiteInscripcion()); 
			  calendarioFecha.set(Calendar.HOUR_OF_DAY, calendarioHora.get(Calendar.HOUR_OF_DAY));
			  calendarioFecha.set(Calendar.MINUTE, calendarioHora.get(Calendar.MINUTE));
			  calendarioFecha.set(Calendar.SECOND, calendarioHora.get(Calendar.SECOND));
		  } else {
			  calendarioFecha.set(Calendar.HOUR_OF_DAY, calendarioHora.get(Calendar.HOUR_OF_DAY));
			  calendarioFecha.set(Calendar.MINUTE, calendarioHora.get(Calendar.MINUTE));
			  calendarioFecha.set(Calendar.SECOND, calendarioHora.get(Calendar.SECOND));
		  }
		  
		  return calendarioFecha.getTime();
	  }
	  
	  return null;
  }
  
  /**
   * Devuelve Fecha y hora limite de inscripcion
   * 
   * @return Fecha y hora limite de inscripcion
   */
  @Transient
  public Date getFechaHoraInicio(){
	  
	  if (getFechaInicio() != null){
		  Calendar calendarioFecha = Calendar.getInstance();
		  Calendar calendarioHora = Calendar.getInstance();
		  calendarioFecha.setTime(getFechaInicio());
		  
		  if(getHoraInicio() != null){
			  calendarioHora.setTime(getHoraInicio()); 
			  calendarioFecha.set(Calendar.HOUR_OF_DAY, calendarioHora.get(Calendar.HOUR_OF_DAY));
			  calendarioFecha.set(Calendar.MINUTE, calendarioHora.get(Calendar.MINUTE));
			  calendarioFecha.set(Calendar.SECOND, calendarioHora.get(Calendar.SECOND));
		  } else {
			  calendarioFecha.set(Calendar.HOUR_OF_DAY, calendarioHora.get(Calendar.HOUR_OF_DAY));
			  calendarioFecha.set(Calendar.MINUTE, calendarioHora.get(Calendar.MINUTE));
			  calendarioFecha.set(Calendar.SECOND, calendarioHora.get(Calendar.SECOND));
		  }
		  
		  return calendarioFecha.getTime();
	  }  
	  return null;
  }
  
  /**
   * Devuelve Si la actividad admite inscripciones por fecha
   * 
   * @return si admite inscripciones
   */
  @Transient
  public boolean getIsEnPlazoInscripcion(){
	  return (getFechaHoraPublicacion()==null || getFechaHoraPublicacion().before(new Date())) &&
			 (getFechaHoraLimiteInscripcion()==null || getFechaHoraLimiteInscripcion().after(new Date())) && 
			 (getFechaHoraInicio()==null || getFechaHoraInicio().after(new Date())); //new line 3.4.8 - RF_15_44 
	  															   // getFechaInicio().after(new Date())
  }

  /**
   * Chequea si es una videoconferencia.
   * @return true, en caso de videoconferencia
   */
  public boolean isVideoconferencia() {
	return videoconferencia;
  }

  /**
   * Asigna si es una videoconferencia
   * @param videoconferencia
   */
  public void setVideoconferencia(boolean videoconferencia) {
	this.videoconferencia = videoconferencia;
  }

  /**
   * Chequea si es una iniciativa singular
   * @return
   */
  public boolean isIniciativaSingular() {
	return iniciativaSingular;
  }

  /**
   * Asigna si es una iniciativa singular
   * @param iniciativaSingular
   */
  public void setIniciativaSingular(boolean iniciativaSingular) {
	this.iniciativaSingular = iniciativaSingular;
  };
  
}
