package es.csa.fdt.cyldigital.servicios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Map;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.Lob;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.centros.entity.Aula;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.evaluacion.entity.Formulario;
import es.csa.fdt.cyldigital.evaluacion.entity.FormularioConcreto;
import es.csa.fdt.cyldigital.evaluacion.entity.Grupo;
import es.csa.fdt.cyldigital.evaluacion.entity.Item;
import es.csa.fdt.cyldigital.evaluacion.entity.Pregunta;
import es.csa.fdt.cyldigital.evaluacion.entity.Respuesta;
import es.csa.fdt.cyldigital.language.entity.ActividadExternaLanguage;
import es.csa.fdt.cyldigital.recursos.entity.Recurso;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;

/**
 * Clase ActividadExterna.
 * 
 * @author csa
 */
@Entity
@Table(name = "actividadesexternas")
//@AttributeOverride(name = "id", column = @Column(name = "idActividadExterna"))
@Audited
@PrimaryKeyJoinColumn(name = "idActividad")
//@es.csa.fdt.cyldigital.servicios.entity.ActividadExterna.FechasConcordantes
public class ActividadExterna extends Actividad implements Serializable {
  private static final long serialVersionUID = -2570942175315417764L;

  private static final int NUMERO_PLAZAS_DEFECTO = 0;
  
  private static final int NUMERO_ASISTENTES_DEFECTO = 0;

  //private static final int NUMERO_PLAZAS_LISTA_ESPERA = 5;
  
  private static final float MAX_VALORACION = 10;

  private Centro centro;

  private Aula aula;

  private Empleado colaborador;

  private Date fechaInicio;

  private Date fechaFin;

  private Date horaInicio;

  private Date horaFin;

  private int numeroPlazas;
  
  private int numeroAsistentes;

  //private float numeroHoras;

  //private List<MatriculaExterna> matriculas;

  private String observaciones;

  //private List<PreinscripcionCurso> preinscripcionesCursoRechazadas;

  //private List<PreinscripcionTaller> preinscripcionesTallerRechazadas;

  //private String apodo;

  private List<ActividadExternaLanguage> textosExterna;

  //private Integer numeroPlazasListaEspera;

  private Tematica tematica;
	 
  private String ubicacion;
  
  private boolean aceptada;
  
  private boolean solicitada;
  
  //El campo videoconferencia sSe traslada a nivel de clase Actividad
  //private boolean videoConferencia; 
  
  private Formulario cuestionarioSatisfaccion;
  
  private List<FormularioConcreto> formulariosConcretos;
  
  private Date finEncuesta;
  
  public enum EEstadoActividadExterna {
		SOLICITADA,
		RECHAZADA,
		ACEPTADA;
  }

  /**
   * Construye un nuevo objeto de la clase
   */
  public ActividadExterna() {
    super();
    setNumeroPlazas(NUMERO_PLAZAS_DEFECTO);
    setNumeroAsistentes(NUMERO_ASISTENTES_DEFECTO);
    
  //Today by default
	fechaInicio = new Date();
	
	//Tomorrow by default
	fechaInicio.setTime(fechaInicio.getTime() + 86400000);
	
	//Day after tomorrow by default
	fechaFin = new Date(fechaInicio.getTime() + 86400000);
	
	//00:00 by default
	horaInicio = new Date(0);
	
	//02:00 by default
	horaFin = new Date(horaInicio.getTime() + 7200000);
    
    recursos = new ArrayList<Recurso>();
    observacionesActividad = new ArrayList<ObservacionActividad>();
    formulariosConcretos = new ArrayList<FormularioConcreto>();
  }

  // ------------------------------------------------------------------------------------------------------------GETTERS

  /**
   * Devuelve id actividad externa.
   * 
   * @return id actividad externa
   */
  /*@Column(unique = true)
  @GeneratedValue(strategy = GenerationType.IDENTITY)
  @Digits(integerDigits = 20)
  public long getIdActividadExterna() {
    return idActividadExterna;
  }*/

  /**
   * Asigna id actividad externa.
   * 
   * @param idActividadExterna
   *          id actividad externa
   */
  /*public void setIdActividadExterna(long idActividadExterna) {
    this.idActividadExterna = idActividadExterna;
  }*/

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getNombre()
   */
  /*@Transient
  public String getNombre() {
    return getApodo();
  }*/

  /**
   * Devuelve apodo.
   * 
   * @return apodo
   */
  /*@Length(max = 255)
  @NotEmpty
  @NotNull
  public String getApodo() {
    if ((apodo == null || apodo.isEmpty()) && getServicio() != null) return getServicio().getNombre();

    return apodo;
  }*/

  /**
   * Asigna apodo.
   * 
   * @param apodo
   *          apodo
   */
  /*public void setApodo(String apodo) {
    this.apodo = apodo;
  }*/
  
  /**
   * Devuelve textos de la actividad externa
   * 
   * @return textos
   */
  @OneToMany(cascade = CascadeType.ALL)
  @JoinColumn(name = "idActividadExterna")
  public List<ActividadExternaLanguage> getTextosExterna() {
	  return textosExterna;
  }

  /**
   * Asigna los textos a la actividad externa
   * 
   * @param textos
   *          textos
   */
  public void setTextosExterna(List<ActividadExternaLanguage> textosExterna) {
	  this.textosExterna = textosExterna;
  }  

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getCentro()
   */
  @Override
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idCentro", nullable = true)
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Devuelve aula.
   * 
   * @return aula
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idAula", nullable = true)
  public Aula getAula() {
    return aula;
  }

  /**
   * Asigna aula.
   * 
   * @param aula
   *          aula
   */
  public void setAula(Aula aula) {
    this.aula = aula;
  }

  /**
   * Devuelve colaborador.
   * 
   * @return colaborador
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idEmpleado", nullable = false)
  @NotNull
  public Empleado getColaborador() {
    return colaborador;
  }

  /**
   * Asigna colaborador.
   * 
   * @param colaborador
   *          colaborador
   */
  public void setColaborador(Empleado colaborador) {
    this.colaborador = colaborador;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getFechaInicio()
   */
  @NotNull
  @Temporal(TemporalType.DATE)
  public Date getFechaInicio() {
    return fechaInicio;
  }

  /**
   * Asigna fecha inicio.
   * 
   * @param fechaInicio
   *          fecha inicio
   */
  public void setFechaInicio(Date fechaInicio) {
    this.fechaInicio = fechaInicio;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getFechaFin()
   */
  @NotNull
  @Temporal(TemporalType.DATE)
  public Date getFechaFin() {
    return fechaFin;
  }

  /**
   * Asigna fecha fin.
   * 
   * @param fechaFin
   *          fecha fin
   */
  public void setFechaFin(Date fechaFin) {
    this.fechaFin = fechaFin;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getHoraInicio()
   */
  @NotNull
  @Temporal(TemporalType.TIME)
  public Date getHoraInicio() {
    return horaInicio;
  }

  /**
   * Asigna hora inicio.
   * 
   * @param horaInicio
   *          hora inicio
   */
  public void setHoraInicio(Date horaInicio) {
    this.horaInicio = horaInicio;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getHoraFin()
   */
  @NotNull
  @Temporal(TemporalType.TIME)
  public Date getHoraFin() {
    return horaFin;
  }

  /**
   * Asigna hora fin.
   * 
   * @param horaFin
   *          hora fin
   */
  public void setHoraFin(Date horaFin) {
    this.horaFin = horaFin;
  }
  
  /**
   * Devuelve horario.
   * 
   * @return horario
   */
  @Transient
  public String getHorario() {
    String resultado = "";
    DateFormat df = new SimpleDateFormat("HH:mm");
    if (getHoraInicio() != null) resultado += df.format(getHoraInicio());
    if (getHoraInicio() != null && getHoraFin() != null) resultado += "-";
    if (getHoraFin() != null) {
      resultado += df.format(getHoraFin());
    }
    return resultado;
  }

  /**
   * Devuelve numero plazas.
   * 
   * @return numero plazas
   */
  @NotNull
  @org.hibernate.validator.Min(value = 1)
  public int getNumeroPlazas() {
    return numeroPlazas;
  }

  /**
   * Asigna numero plazas.
   * 
   * @param numeroPlazas
   *          numero plazas
   */
  public void setNumeroPlazas(int numeroPlazas) {
    this.numeroPlazas = numeroPlazas;
  }

  /**
   * Devuelve numero horas.
   * 
   * @return numero horas
   */
  /*@NotNull
  @Digits(integerDigits = 20, fractionalDigits = 2)
  @org.hibernate.validator.Min(value = 1)
  public float getNumeroHoras() {
    return numeroHoras;
  }*/

  /**
   * Asigna numero horas.
   * 
   * @param numeroHoras
   *          numero horas
   */
  /*public void setNumeroHoras(float numeroHoras) {
    this.numeroHoras = numeroHoras;
  }*/

  /**
   * Devuelve matriculas.
   * 
   * @return matriculas
   */
  /*@OneToMany(fetch = FetchType.LAZY, cascade = { javax.persistence.CascadeType.ALL }, mappedBy = "externa")
  // @Cascade(org.hibernate.annotations.CascadeType.DELETE_ORPHAN)
  @OrderBy(value = "fechaMatricula ASC")
  public List<MatriculaExterna> getMatriculas() {
    return matriculas;
  }*/

  /**
   * Asigna matriculas.
   * 
   * @param matriculas
   *          matriculas
   */
  /*public void setMatriculas(List<MatriculaExterna> matriculas) {
    this.matriculas = matriculas;
  }*/

  /**
   * Devuelve numero asistentes.
   * 
   * @return numero asistentes
   */
  @NotNull
  @org.hibernate.validator.Min(value = 1)
  public int getNumeroAsistentes() {
	return numeroAsistentes;
  }
  
  /**
   * Asigna numero asistentes.
   * 
   * @param numeroAsistentes
   *          numero asistentes
   */
  public void setNumeroAsistentes(int numeroAsistentes) {
	this.numeroAsistentes = numeroAsistentes;
  }

/**
   * Devuelve observaciones.
   * 
   * @return observaciones
   */
  @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getObservaciones() {
    return observaciones;
  }

  /**
   * Asigna observaciones.
   * 
   * @param observaciones
   *          observaciones
   */
  public void setObservaciones(String observaciones) {
    this.observaciones = observaciones;
  }

  /**
   * Devuelve preinscripciones curso rechazadas.
   * 
   * @return preinscripciones curso rechazadas
   */
  /*@ManyToMany(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH }, mappedBy = "externasCursosRechazos", targetEntity = PreinscripcionCurso.class)
  public List<PreinscripcionCurso> getPreinscripcionesCursoRechazadas() {
    return preinscripcionesCursoRechazadas;
  }*/

  /**
   * Asigna preinscripciones curso rechazadas.
   * 
   * @param preinscripcionesCursoRechazadas
   *          preinscripciones curso rechazadas
   */
  /*public void setPreinscripcionesCursoRechazadas(List<PreinscripcionCurso> preinscripcionesCursoRechazadas) {
    this.preinscripcionesCursoRechazadas = preinscripcionesCursoRechazadas;
  }*/

  /**
   * Devuelve preinscripciones taller rechazadas.
   * 
   * @return preinscripciones taller rechazadas
   */
  /*@ManyToMany(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH }, mappedBy = "externasTalleresRechazos", targetEntity = PreinscripcionTaller.class)
  public List<PreinscripcionTaller> getPreinscripcionesTallerRechazadas() {
    return preinscripcionesTallerRechazadas;
  }*/

  /**
   * Asigna preinscripciones taller rechazadas.
   * 
   * @param preinscripcionesTallerRechazadas
   *          preinscripciones taller rechazadas
   */
  /*public void setPreinscripcionesTallerRechazadas(List<PreinscripcionTaller> preinscripcionesTallerRechazadas) {
    this.preinscripcionesTallerRechazadas = preinscripcionesTallerRechazadas;
  }*/

  /**
   * Devuelve numero plazas lista espera.
   * 
   * @return numero plazas lista espera
   */
 /* @NotNull
  public Integer getNumeroPlazasListaEspera() {
    return numeroPlazasListaEspera == null ? NUMERO_PLAZAS_LISTA_ESPERA : numeroPlazasListaEspera;
  }*/

  /**
   * Asigna numero plazas lista espera.
   * 
   * @param numeroPlazasListaEspera
   *          numero plazas lista espera
   */
 /* public void setNumeroPlazasListaEspera(Integer numeroPlazasListaEspera) {
    this.numeroPlazasListaEspera = numeroPlazasListaEspera;
  }*/

  /**
   * Devuelve numero inscritos.
   * 
   * @return numero inscritos
   */
  /*@Transient
  public int getNumeroInscritos() {
    if (getMatriculas() != null) return getMatriculas().size();
    return 0;
  }*/

  /**
   * Devuelve numero matriculados.
   * 
   * @return numero matriculados
   */
/*  @Transient
  public int getNumeroMatriculados() {
    if (getMatriculas() != null) {
      int aux = 0;
      for (MatriculaExterna m : getMatriculas()) {
        if (!m.isEnListaEspera()) aux += 1;
      }
      return aux;
    }
    return 0;
  }*/

  /**
   * Devuelve numero lista espera.
   * 
   * @return numero lista espera
   */
/* @Transient
  public int getNumeroListaEspera() {
    if (getMatriculas() != null) {
      int aux = 0;
      for (MatriculaExterna m : getMatriculas()) {
        if (m.isEnListaEspera()) aux += 1;
      }
      return aux;
    }
    return 0;
  }*/

  /**
   * Devuelve numero confirmados.
   * 
   * @return numero confirmados
   */
 /* @Transient
  public int getNumeroConfirmados() {
    int i = 0;
    if (getMatriculas() != null) {
      for (MatriculaExterna m : getMatriculas()) {
        if (m.isAdmitido()) i++;
      }
      return i;
    }
    return 0;
  }*/

  /**
   * Devuelve numero sin confirmar.
   * 
   * @return numero sin confirmar
   */
/*  @Transient
  public int getNumeroSinConfirmar() {
    if (getMatriculas() != null) return getMatriculas().size() - getNumeroConfirmados();
    return 0;
  }*/

  /**
   * Devuelve numero admitidos.
   * 
   * @return numero admitidos
   */
/*  @Transient
  public int getNumeroAdmitidos() {
    int i = 0;
    if (getMatriculas() != null) for (MatriculaExterna m : getMatriculas()) {
      if (m.isAdmitido()) i++;
      return i;
    }
    return 0;
  }*/

  /**
   * Devuelve fechas legibles.
   * 
   * @return fechas legibles
   */
  @Transient
  public String getFechasLegibles() {
    String resultado = "";
    DateFormat df = new SimpleDateFormat("dd/MM/yyyy");

    if (getFechaInicio() != null)
    	resultado += df.format(getFechaInicio());
    
	if (getFechaInicio() != null && getFechaFin() != null)
	{
		resultado += " - " + df.format(getFechaFin());
	}

    return resultado;
  }

  /**
   * Devuelve horario.
   * 
   * @return horario
   */
/*  @Transient
  public String getHorario() {
    String resultado = "";
    DateFormat df = new SimpleDateFormat("HH:mm");
    if (getHoraInicio() != null) resultado += df.format(getHoraInicio());
    if (getHoraInicio() != null && getHoraFin() != null) resultado += "-";
    if (getHoraFin() != null) {
      resultado += df.format(getHoraFin());
    }
    return resultado;
  }*/

  /**
   * Devuelve horario largo.
   * 
   * @return horario largo
   */
/*  @Transient
  public String getHorarioLargo() {
    String resultado = "";
    DateFormat df = new SimpleDateFormat("HH:mm");
    if (getHoraInicio() != null) resultado += df.format(getHoraInicio());
    if (getHoraInicio() != null && getHoraFin() != null) resultado += " - ";
    if (getHoraFin() != null) {
      resultado += df.format(getHoraFin());
    }
    return resultado;
  }*/

  /**
   * Checks if is completo.
   * 
   * @return true, if is completo
   */
/*  @Transient
  public boolean isCompleto() {
    return getNumeroPlazas() - getNumeroInscritos() <= 0;
  }*/

  /**
   * Checks if is impartido.
   * 
   * @return true, if is impartido
   */
 /* @Transient
  public boolean isImpartido() {
    if (getFechaFin() != null && getFechaFin().before(new Date())) return true;
    return false;
  }*/

  /**
   * Checks if is curso.
   * 
   * @return true, if is curso
   */
  /*@Transient
  public boolean isCurso() {
    return servicio instanceof ServicioCurso;
  }*/

  /**
   * Checks if is taller.
   * 
   * @return true, if is taller
   */
  /*@Transient
  public boolean isTaller() {
	  return servicio instanceof ServicioTaller;
  }*/

  /**
   * Interfaz FechasConcordantes.
   */
/*  @ValidatorClass(value = FechasConcordantesValidator.class)
  @Target( { ElementType.TYPE })
  @Retention(RetentionPolicy.RUNTIME)
  @Documented
  public static @interface FechasConcordantes {

    /**
     * Message.
     * 
     * @return string
     */
   /* String message() default "{actividad.validator.fechaNoConcordante}";

  }*/

  /**
   * Clase FechasConcordantesValidator.
   */
/*  public static class FechasConcordantesValidator implements Validator<FechasConcordantes> {

    private static final long serialVersionUID = 985021755629904067L;*/

    /* (non-Javadoc)
     * @see org.hibernate.validator.Validator#initialize(java.lang.annotation.Annotation)
     */
/*    @Override
    public void initialize(FechasConcordantes parameters) {}*/

    /* (non-Javadoc)
     * @see org.hibernate.validator.Validator#isValid(java.lang.Object)
     */
/*    @Override
    public boolean isValid(Object value) {
      ActividadExterna externa = (ActividadExterna) value;
      if (externa.getFechaInicio().getTime() <= externa.getFechaFin().getTime()) return true;
      return false;
    }
  }*/
  
  /**
   * Devuelve tematica.
   * 
   * @return tematica
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idTematica")
  public Tematica getTematica() {
	  if (servicio != null)
		  return servicio.getTematica();
	  else
		  return tematica;
  }

  /**
   * Asigna tematica.
   * 
   * @param tematica
   *          tematica
   */
  public void setTematica(Tematica tematica) {
    this.tematica = tematica;
  }
  
  /**
    * Devuelve ubicacion
	*
	* @return ubicacion
	*/
  @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getUbicacion() {
	return this.ubicacion;
  }

  /**
	* Asigna  ubicacion
	* 
	* @param ubicacion
	*          ubicacion
	*/
  public void setUbicacion(String ubicacion) {
	this.ubicacion = ubicacion;
  }  
	
  /**
    * Checks if is aceptada 
	* 
	* @return true, if is aceptada 
	*/
  @NotNull
  @Column(name="aceptada", nullable = false, columnDefinition = "bit(1) default 0")
  public boolean isAceptada() {
	  return aceptada;
  }

  /**
   * Asigna aceptada 
   * 
   * @param aceptada
   *          aceptada 
   */
  public void setAceptada(boolean aceptada) {
	  this.aceptada = aceptada;
  }	

  /**
   * Checks if is solicitada 
   * 
   * @return true, if is solicitada 
   */
  @NotNull
  @Column(name="solicitada", nullable = false, columnDefinition = "bit(1) default 0")
  public boolean isSolicitada() {
	  return solicitada;
  }

  /**
	* Asigna solicitada 
	* 
	* @param solicitada
	*          solicitada 
	*/
  public void setSolicitada(boolean solicitada) {
	  this.solicitada = solicitada;
  }	
  
  /**
   * Devuelve cuestionario satisfaccion.
   * 
   * @return cuestionario satisfaccion
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idCuestionarioSatisfaccion")
  public Formulario getCuestionarioSatisfaccion() {
    return cuestionarioSatisfaccion;
  }

  /**
   * Asigna cuestionario satisfaccion.
   * 
   * @param cuestionarioSatisfaccion
   *          cuestionario satisfaccion
   */
  public void setCuestionarioSatisfaccion(Formulario cuestionarioSatisfaccion) {
    this.cuestionarioSatisfaccion = cuestionarioSatisfaccion;
  }
  
  /**
   * Devuelve formularios concretos.
   * 
   * @return formularios concretos
   */
  @NotAudited
  @ManyToMany(targetEntity = FormularioConcreto.class, cascade = { javax.persistence.CascadeType.ALL })
  @JoinTable(name = "actividadexterna_formulariosconcretos", joinColumns = @JoinColumn(name = "idActividadExterna"), inverseJoinColumns = @JoinColumn(name = "idFormularioConcreto"))
  public List<FormularioConcreto> getFormulariosConcretos() {
    return formulariosConcretos;
  }

  /**
   * Asigna formularios concretos.
   * 
   * @param formularios concretos
   *          formularios concretos
   */
  public void setFormulariosConcretos(List<FormularioConcreto> formulariosConcretos) {
    this.formulariosConcretos = formulariosConcretos;
  }
  
  /**
   * 
   */
  @Transient
  public EEstadoActividadExterna getEstado() {
	  EEstadoActividadExterna estado = EEstadoActividadExterna.RECHAZADA;
	  
	  if (solicitada && !aceptada)
		  estado = EEstadoActividadExterna.SOLICITADA;
	  else if (!solicitada && !aceptada)
		  estado = EEstadoActividadExterna.RECHAZADA;
	  else if (!solicitada && aceptada)
		  estado = EEstadoActividadExterna.ACEPTADA;
	  
	  return estado;
  }
  
  public void setEstado(EEstadoActividadExterna estado) {
	  switch (estado) {
	  	case SOLICITADA:
	  		this.solicitada = true;
	  		this.aceptada = false;
	  		
	  		break;
	  		
	  	case RECHAZADA:
	  		this.solicitada = false;
	  		this.aceptada = false;
	  		
	  		break;
	  		
	  	case ACEPTADA:
	  		this.solicitada = false;
	  		this.aceptada = true;
	  		
	  		break;
	  }
  }
  
  /**
   * @return the inicioEncuesta
   */
  @Temporal(TemporalType.TIMESTAMP)
  public Date getFinEncuesta() {
	  return finEncuesta;
  }
	
  /**
   * @param finEncuesta the finEncuesta to set
   */
  public void setFinEncuesta(Date finEncuesta) {
	  this.finEncuesta = finEncuesta;
  }

  @Transient
  public boolean isEncuestaDisponible() {
	  Date horaActual = new Date();
		
	  if ((cuestionarioSatisfaccion != null) && (finEncuesta != null) &&
		  (formulariosConcretos.size() < numeroPlazas) &&
		  (horaActual.getTime() < finEncuesta.getTime()))
		  return true;

	  return false;
  }

  @Transient
  public boolean getAreThereFiles() {
	  boolean result = false; 

	  if (servicio == null) {
		  for(ActividadExternaLanguage ael : textosExterna) {
			  if (ael.getFicheroActExt() != null) {
				  result = true;
				  break;
			  }
		  }
	  }
	  else {
		  result = servicio.getAreThereFiles();
	  }
	  
	  return result;
  }
  
  @Transient
  public boolean isCerrable() {
	  if ((System.currentTimeMillis() >= fechaFin.getTime()) && (getEstado() == EEstadoActividadExterna.ACEPTADA) && !isCerrada()) {
		  return true;
	  }
	  
	  return false;
  }
  
  @Transient
  @Override
  public boolean isExterna() {
	  return true;
  }
  
  /**
   * Checks if is iniciado.
   * 
   * @return true, if is iniciado
   */
  @Transient
  public boolean isIniciada() {
	  return (getId() > 0) && getFechaInicio().before(new Date());
  }
  
  /**
   * Checks if is finalizada.
   * 
   * @return true, if is finalizada
   */
  @Transient
  public boolean isFinalizada() {
	  //return (getId() > 0) && getFechaFin().before(new Date());
	  
	  if (getId() > 0 && (getFechaFin() != null) && (getHoraFin() != null)) {
		  Calendar hourCal = Calendar.getInstance();
		  Calendar finalCal = Calendar.getInstance();

		  hourCal.setTime(getHoraFin());
		  finalCal.setTime(getFechaFin());

		  finalCal.set(Calendar.HOUR_OF_DAY, hourCal.get(Calendar.HOUR_OF_DAY));
		  finalCal.set(Calendar.MINUTE, hourCal.get(Calendar.MINUTE));
		  finalCal.set(Calendar.SECOND, hourCal.get(Calendar.SECOND));

		  if (finalCal.getTime().before(new Date()))
			  return true;
	    }

	  return false;
  }

  /**
   * Checks if is editado calendario.
   * 
   * @return true, if is editado calendario
   */
  @Transient
  public boolean isEditadoCalendario() {
	  return (getCalendarItems() == null) ? false : !getCalendarItems().isEmpty();
  }
  
  	/**
	 * Devuelve valoracion curso.
	 * 
	 * @return valoracion curso
	 */
	@Transient
	public String getValoracion() {
		float valoracion = 0;
		int n = 0;
		
		if ((cuestionarioSatisfaccion != null) && (cuestionarioSatisfaccion.getItems() != null)) {
			for (FormularioConcreto fc : formulariosConcretos) {
				Map<Pregunta, Respuesta> respuestas = fc.getRespuestas();
				
				if ((respuestas != null) && !respuestas.isEmpty()) {
					List<Item> items = cuestionarioSatisfaccion.getItems();
					
					for (int p = 0; p < items.size(); p++) {
						if (items.get(p) instanceof Grupo) {
							List<Pregunta> preguntas = ((Grupo) items.get(p)).getPreguntas();
							
							if (preguntas != null) {
								for (int g = 0; g < preguntas.size(); g++) {
									Respuesta r = respuestas.get(preguntas.get(g));
									
									if (r.getValorEntero() != null) {
										valoracion += (MAX_VALORACION / preguntas.get(g).getMaximo().intValue()) * r.getValorEntero();
										
										n++;
									}
									else if (r.getValorDecimal() != null) {
										valoracion += (MAX_VALORACION / preguntas.get(g).getMaximoDecimal().floatValue()) * r.getValorDecimal();

										n++;
									}
								}
							}
						}
						else if (items.get(p) instanceof Pregunta) {
							Respuesta r = respuestas.get(items.get(p));

							if (r.getValorEntero() != null) {
								valoracion += (MAX_VALORACION / ((Pregunta)items.get(p)).getMaximo().intValue()) * r.getValorEntero();
								
								n++;
							}
							else if (r.getValorDecimal() != null) {
								valoracion += (MAX_VALORACION / ((Pregunta)items.get(p)).getMaximoDecimal().floatValue()) * r.getValorDecimal();
								n++;
							}
						}
					}
				}
			}
		}
		
		if (n == 0)
			return "Sin valoración";
		else {
			valoracion = (valoracion / n);
			
			return String.format("%.2f sobre %.2f", valoracion, MAX_VALORACION);
		}
	}
}
