package es.csa.fdt.cyldigital.servicios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Digits;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.calendario.entity.ActividadCalendarItem;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;

/**
 * Clase Asesoramiento.
 * 
 * @author csa
 */
@Entity
@Table(name = "asesoramientos")
@AttributeOverride(name = "id", column = @Column(name = "idAsesoramiento"))
@Audited
@PrimaryKeyJoinColumn(name = "idActividad")
public class Asesoramiento extends Actividad implements Serializable {

  private static final long serialVersionUID = -8361498848042755096L;

  /**
   * Enum Colectivo.
   */
  public static enum Colectivo {
    MAYORES,
    ADULTOS,
    JOVENES,
    NINOS,
    DISCAPACITADOS,
    EMPRESAS,
    INMIGRANTES,
    DESEMPLEADOS,
    ADMINISTRACION;
  }

  private Empleado dinamizador;

  private UsuarioCyL usuario;

  private TipoAsesoramiento tipoAsesoramiento;

  private Colectivo colectivo;

  private String tema;

  private Centro centro;

  private String observaciones;

  private int duracion;

  private Date fecha;

  private Date hora;

  private boolean resolucion;

  private long idAsesoramiento;

  /**
   * Construye un nuevo objeto de la clase
   */
  public Asesoramiento() {
    super();
    calendarItems = new ArrayList<ActividadCalendarItem>();
  }

  // ------------------------------------------------------------------------------------------------------------GETTERS

  /**
   * Devuelve id asesoramiento.
   * 
   * @return id asesoramiento
   */
  @Column(unique = true)
  @GeneratedValue(strategy = GenerationType.IDENTITY)
  @Digits(integerDigits = 20)
  public long getIdAsesoramiento() {
    return idAsesoramiento;
  }

  /**
   * Asigna id asesoramiento.
   * 
   * @param idAsesoramiento
   *          id asesoramiento
   */
  public void setIdAsesoramiento(long idAsesoramiento) {
    this.idAsesoramiento = idAsesoramiento;
  }

  /**
   * Devuelve tipo asesoramiento.
   * 
   * @return tipo asesoramiento
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idTipoAsesoramiento")
  @NotNull
  public TipoAsesoramiento getTipoAsesoramiento() {
    return tipoAsesoramiento;
  }

  /**
   * Asigna tipo asesoramiento.
   * 
   * @param tipoAsesoramiento
   *          tipo asesoramiento
   */
  public void setTipoAsesoramiento(TipoAsesoramiento tipoAsesoramiento) {
    this.tipoAsesoramiento = tipoAsesoramiento;
  }

  /**
   * Devuelve colectivo.
   * 
   * @return colectivo
   */
  @Enumerated(value = EnumType.STRING)
  @NotNull
  public Colectivo getColectivo() {
    return colectivo;
  }

  /**
   * Asigna colectivo.
   * 
   * @param colectivo
   *          colectivo
   */
  public void setColectivo(Colectivo colectivo) {
    this.colectivo = colectivo;
  }

  /**
   * Devuelve dinamizador.
   * 
   * @return dinamizador
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idEmpleado")
  @NotNull
  public Empleado getDinamizador() {
    return dinamizador;
  }

  /**
   * Asigna dinamizador.
   * 
   * @param dinamizador
   *          dinamizador
   */
  public void setDinamizador(Empleado dinamizador) {
    this.dinamizador = dinamizador;
  }

  /**
   * Devuelve usuario.
   * 
   * @return usuario
   */
  @NotNull
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idUsuario")
  public UsuarioCyL getUsuario() {
    return usuario;
  }

  /**
   * Asigna usuario.
   * 
   * @param usuario
   *          usuario
   */
  public void setUsuario(UsuarioCyL usuario) {
    this.usuario = usuario;
  }

  /**
   * Devuelve tema.
   * 
   * @return tema
   */
  @NotNull
  @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getTema() {
    return tema;
  }

  /**
   * Asigna tema.
   * 
   * @param tema
   *          tema
   */
  public void setTema(String tema) {
    this.tema = tema;
  }

  /**
   * Devuelve duracion.
   * 
   * @return duracion
   */
  public int getDuracion() {
    return duracion;
  }

  /**
   * Asigna duracion.
   * 
   * @param duracion
   *          duracion
   */
  public void setDuracion(int duracion) {
    this.duracion = duracion;
  }

  /**
   * Devuelve la duración, en minutos, con un formato del tipo "5h 40min".
   * 
   * @return Duracion formateada.
   */
  @Transient
  public String getDuracionString() {
    return "" + (duracion / 60 > 0 ? duracion / 60 + "h " : "0h ")
        + (duracion % 60 > 0 ? duracion % 60 + "min" : "0min");
  }

  /**
   * Devuelve fecha.
   * 
   * @return fecha
   */
  @NotNull
  @Temporal(TemporalType.DATE)
  public Date getFecha() {
    return fecha;
  }

  /**
   * Asigna fecha.
   * 
   * @param fecha
   *          fecha
   */
  public void setFecha(Date fecha) {
    this.fecha = fecha;
  }

  /**
   * Devuelve hora.
   * 
   * @return hora
   */
  @NotNull
  @Temporal(TemporalType.TIME)
  public Date getHora() {
    return hora;
  }

  /**
   * Asigna hora.
   * 
   * @param hora
   *          hora
   */
  public void setHora(Date hora) {
    this.hora = hora;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getCentro()
   */
  @Override
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idCentro")
  @NotNull
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Devuelve observaciones.
   * 
   * @return observaciones
   */
  @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getObservaciones() {
    return observaciones;
  }

  /**
   * Asigna observaciones.
   * 
   * @param observaciones
   *          observaciones
   */
  public void setObservaciones(String observaciones) {
    this.observaciones = observaciones;
  }

  /**
   * Devuelve fecha legible.
   * 
   * @return fecha legible
   */
  @Transient
  public String getFechaLegible() {
    String resultado = "";
    DateFormat dfFecha = new SimpleDateFormat("dd/MM/yyyy");
    DateFormat dfHora = new SimpleDateFormat("HH:mm");
    if (getFecha() != null) {
      resultado += dfFecha.format(getFecha());
    }
    if (getHora() != null) {
      resultado += " " + dfHora.format(getHora());
    }
    return resultado;
  }

  /**
   * Devuelve fecha duracion.
   * 
   * @return fecha duracion
   */
  @Transient
  public Date getFechaDuracion() {
    return duracion <= 0 ? null : new Date(fecha.getTime() + (duracion * 60000));
  }

  /**
   * Checks if is resolucion.
   * 
   * @return true, if is resolucion
   */
  @NotNull
  public boolean isResolucion() {
    return resolucion;
  }

  /**
   * Asigna resolucion.
   * 
   * @param resolucion
   *          resolucion
   */
  public void setResolucion(boolean resolucion) {
    this.resolucion = resolucion;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getNombre()
   */
  /*@Override
  @Transient
  public String getNombre() {
    return getTema();
  }*/

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getFechaFin()
   */
  @Override
  @Transient
  public Date getFechaFin() {
    return null;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getFechaInicio()
   */
  @Override
  @Transient
  public Date getFechaInicio() {
    return null;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getHoraFin()
   */
  @Override
  @Transient
  public Date getHoraFin() {
    return null;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getHoraInicio()
   */
  @Override
  @Transient
  public Date getHoraInicio() {
    return null;
  }
  
  @Transient
  @Override
  public boolean isExterna() {
	  return false;
  }
}
