package es.csa.fdt.cyldigital.servicios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.NotNull;
import org.springframework.security.context.SecurityContextHolder;

import es.csa.fdt.cyldigital.calendario.entity.ActividadCalendarItem;
import es.csa.fdt.cyldigital.centros.entity.Aula;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.recursos.entity.Recurso;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;
import es.csa.fdt.cyldigital.language.entity.CharlaLanguage;

/**
 * Clase Charla.
 * 
 * @author csa
 */
@Entity
@Table(name = "charlas")
//@AttributeOverride(name = "id", column = @Column(name = "idCharla"))
@Audited
@PrimaryKeyJoinColumn(name = "idActividad")
public class Charla extends Actividad implements Serializable {
  private static final long serialVersionUID = -8361498848042755096L;

  private static final int NUMERO_PLAZAS_DEFECTO = 50;

  private Empleado profesor;

  private Centro centro;

  private Tematica tematica;

  private Aula aula;

  private Date fechaInicio;
  
  private Date fechaFin;

  private Date horaInicio;

  private Date horaFin;

  private int numeroPlazas;

  private List<MatriculaCharla> matriculas;
  
  private List<CharlaLanguage> textosCharla;

  private String observaciones;

  private int edadMaxima;
  private int edadMinima;

  public enum EEstadoMatricula {
		NO_MATRICULADO,
		SOLICITUD_ENVIADA,
		MATRICULADO,
		RECHAZADO,
		CANCELADA;
  }
  
  /**
   * Construye un nuevo objeto de la clase
   */
  public Charla() {
    super();
    setNumeroPlazas(NUMERO_PLAZAS_DEFECTO);
    recursos = new ArrayList<Recurso>();
    calendarItems = new ArrayList<ActividadCalendarItem>();
  }

  // ------------------------------------------------------------------------------------------------------------GETTERS

  /**
   * Devuelve id charla.
   * 
   * @return id charla
   */
  /*@Column(unique = true)
  @GeneratedValue(strategy = GenerationType.IDENTITY)
  @Digits(integerDigits = 20)
  public long getIdCharla() {
    return idCharla;
  }*/

  /**
   * Asigna id charla.
   * 
   * @param idCharla
   *          id charla
   */
  /*public void setIdCharla(long idCharla) {
    this.idCharla = idCharla;
  }
*/
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getNombre()
   */
  /*@Length(max = 200)
  @NotEmpty
  @NotNull
  public String getNombre(Language language) {
    return nombre;
  }*/

  /**
   * Asigna nombre.
   * 
   * @param nombre
   *          nombre
   */
  /*public void setNombre(String nombre) {
    this.nombre = nombre;
  }*/
  
  /**
	* Devuelve textos de la charla.
	*
	* @return textosCharla
  */
	@OneToMany(cascade = CascadeType.ALL)
	@JoinColumn(name = "idCharla")
	public List<CharlaLanguage> getTextosCharla() {
		return textosCharla;
	}

	/**
	 * Asigna los textos a la charla.
	 * 
	 * @param textosCharla
	 *          textosCharla
	 */
	public void setTextosCharla(List<CharlaLanguage> textosCharla) {
		this.textosCharla = textosCharla;
	}

  /**
   * Devuelve profesor.
   * 
   * @return profesor
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idEmpleado")
  //@NotNull
  public Empleado getProfesor() {
    return profesor;
  }

  /**
   * Asigna profesor.
   * 
   * @param profesor
   *          profesor
   */
  public void setProfesor(Empleado profesor) {
    this.profesor = profesor;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getCentro()
   */
  @Override
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idCentro")
  @NotNull
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Devuelve tematica.
   * 
   * @return tematica
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idTematica")
  @NotNull
  public Tematica getTematica() {
    return tematica;
  }

  /**
   * Asigna tematica.
   * 
   * @param tematica
   *          tematica
   */
  public void setTematica(Tematica tematica) {
    this.tematica = tematica;
  }

  /**
   * Devuelve aula.
   * 
   * @return aula
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idAula")
  public Aula getAula() {
    return aula;
  }

  /**
   * Asigna aula.
   * 
   * @param aula
   *          aula
   */
  public void setAula(Aula aula) {
    this.aula = aula;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getFechaInicio()
   */
  @NotNull
  @Temporal(TemporalType.DATE)
  public Date getFechaInicio() {
    return fechaInicio;
  }

  /**
   * Asigna fecha inicio.
   * 
   * @param fechaInicio
   *          fecha inicio
   */
  public void setFechaInicio(Date fechaInicio) {
    this.fechaInicio = fechaInicio;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getFechaFin()
   */
  @NotNull
  @Temporal(TemporalType.DATE)
  public Date getFechaFin() {
    return fechaFin;
  }

  /**
   * Asigna fecha fin.
   * 
   * @param fechaFin
   *          fecha fin
   */
  public void setFechaFin(Date fechaFin) {
    this.fechaFin = fechaFin;
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getHoraInicio()
   */
  @NotNull
  @Temporal(TemporalType.TIME)
  public Date getHoraInicio() {
    return horaInicio;
  }
  
  @Transient
  public Long getTimeStampInicio() {
	  Calendar hourCal = Calendar.getInstance();
	  Calendar finalCal = Calendar.getInstance();
	  
	  hourCal.setTime(horaInicio);
	  finalCal.setTime(fechaInicio);
	  
	  finalCal.set(Calendar.HOUR_OF_DAY, hourCal.get(Calendar.HOUR_OF_DAY));
	  finalCal.set(Calendar.MINUTE, hourCal.get(Calendar.MINUTE));
	  finalCal.set(Calendar.SECOND, hourCal.get(Calendar.SECOND));
	  
	  return finalCal.getTimeInMillis();
  }
  
  @Transient
  public Long getTimeStampFin() {
	  Calendar hourCal = Calendar.getInstance();
	  Calendar finalCal = Calendar.getInstance();
	  
	  hourCal.setTime(horaFin);
	  finalCal.setTime(fechaFin);
	  
	  finalCal.set(Calendar.HOUR_OF_DAY, hourCal.get(Calendar.HOUR_OF_DAY));
	  finalCal.set(Calendar.MINUTE, hourCal.get(Calendar.MINUTE));
	  finalCal.set(Calendar.SECOND, hourCal.get(Calendar.SECOND));
	  
	  return finalCal.getTimeInMillis();
  }

  /**
   * Asigna hora inicio.
   * 
   * @param horaInicio
   *          hora inicio
   */
  public void setHoraInicio(Date horaInicio) {
    this.horaInicio = horaInicio;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getHoraFin()
   */
  @NotNull
  @Temporal(TemporalType.TIME)
  public Date getHoraFin() {
    return horaFin;
  }

  /**
   * Asigna hora fin.
   * 
   * @param horaFin
   *          hora fin
   */
  public void setHoraFin(Date horaFin) {
    this.horaFin = horaFin;
  }

  /**
   * Devuelve numero plazas.
   * 
   * @return numero plazas
   */
  @NotNull
  public int getNumeroPlazas() {
    return numeroPlazas;
  }

  /**
   * Asigna numero plazas.
   * 
   * @param numeroPlazas
   *          numero plazas
   */
  public void setNumeroPlazas(int numeroPlazas) {
    this.numeroPlazas = numeroPlazas;
  }

  /**
   * Devuelve matriculas.
   * 
   * @return matriculas
   */
  @OneToMany(fetch = FetchType.LAZY, cascade = { javax.persistence.CascadeType.ALL }, mappedBy = "charla")
  @OrderBy(value = "fechaConfirmacion ASC")
  public List<MatriculaCharla> getMatriculas() {
    return matriculas;
  }

  /**
   * Asigna matriculas.
   * 
   * @param matriculas
   *          matriculas
   */
  public void setMatriculas(List<MatriculaCharla> matriculas) {
    this.matriculas = matriculas;
  }

  /**
   * Devuelve observaciones.
   * 
   * @return observaciones
   */
  @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getObservaciones() {
    return observaciones;
  }

  /**
   * Asigna observaciones.
   * 
   * @param observaciones
   *          observaciones
   */
  public void setObservaciones(String observaciones) {
    this.observaciones = observaciones;
  }

  /**
   * Devuelve numero inscritos.
   * 
   * @return numero inscritos
   */
  @Transient 
  public int getNumeroInscritos() {
	    if (getMatriculas() != null) {
	    	List<MatriculaCharla> mch = new ArrayList<MatriculaCharla>();
	    	
	    	for(MatriculaCharla m : getMatriculas()) {
	    		if(!m.getDesactivado() && !m.isCancelada()) mch.add(m);
	    	}
	    	return mch.size();
	    }

	    return 0;
  }
  /**
   * Devuelve numero matriculados.
   * 
   * @return numero matriculados
   */
  @Transient
  public int getNumeroMatriculados() {
	int matriculados = 0;
	
    if (getMatriculas() != null) {
    	for (MatriculaCharla m : getMatriculas()) {
    		if (!m.isEnListaEspera() &&  m.isAdmitido())
    			matriculados++;
    	}
    }
    
    return matriculados;
  }

  /**
   * Devuelve numero asistentes.
   * 
   * @return numero asistentes
   */
  @Transient
  public int getNumeroAsistentes() {
    return isCerrada() ? getMatriculas().size() : 0;
  }

  /**
   * Devuelve numero lista espera.
   * 
   * @return numero lista espera
   */
  @Transient
  public int getNumeroListaEspera() {
	int enListaEspera = 0;

    if (getMatriculas() != null) {
		for (MatriculaCharla m : getMatriculas()) {
			if (m.isEnListaEspera() && !m.isAdmitido() && !m.getDesactivado())
				enListaEspera++;
		}
    }
    
    return enListaEspera;
  }

  /**
   * Devuelve numero confirmados.
   * 
   * @return numero confirmados
   */
  @Transient
  public int getNumeroConfirmados() {
    int i = 0;

    if (getMatriculas() != null) {
    	for (MatriculaCharla m : getMatriculas()) {
    		if (m.isAdmitido() && !m.isEnListaEspera())
    			i++;
    	}
    }

    return i;
  }

  /**
   * Devuelve numero sin confirmar.
   * 
   * @return numero sin confirmar
   */
  @Transient  
  public int getNumeroSinConfirmar() {
	  if (getMatriculas() != null)
		  return getMatriculas().size() - getNumeroConfirmados();
	  return 0;
  }

  /**
   * Devuelve numero de prematriculados por Web.
   * 
   * @return numero de prematriculados por Web
   */
  @Transient
  public int getNumeroPrematriculadosWeb() {
	  int prematriculadosWeb = 0;
	  
	  if (getMatriculas() != null) {
		  for (MatriculaCharla mc: getMatriculas()) {
			  if (mc.isPreMatriculadoWeb())
				  prematriculadosWeb++;
		  }
	  }
	  
	  return prematriculadosWeb;
  }

  /**
   * Devuelve numero rechazados.
   * 
   * @return numero de rechazados
   */
  @Transient
  public int getNumeroRechazados() {
	  int rechazados = 0;
	  
	  if (getMatriculas() != null) {
		  for (MatriculaCharla mc: getMatriculas()) {
			  if (!mc.isAdmitido() && !mc.isEnListaEspera())
				  rechazados++;
		  }
	  }
	  
	  return rechazados;
  }
  
  /**
   * Devuelve numero solicitudes web en curso.
   * 
   * @return numero solicitudes web en curso
   */
  @Transient
  public int getNumeroListaEsperaWeb() {
	  int enListaEsperaWeb = 0;

	  if (getMatriculas() != null) {
		  for (MatriculaCharla m : getMatriculas()) {
			  if (m.isEnListaEspera() && !m.isAdmitido() && m.isPreMatriculadoWeb() && !m.getDesactivado())
				  enListaEsperaWeb++;
		  }
	  }

	  return enListaEsperaWeb;
  }

  /**
   * Checks if is completo.
   * 
   * @return true, if is completo
   */
  @Transient
  public boolean isCompleto() {
    return getNumeroPlazas() - getNumeroInscritos() <= 0;
  }

  /**
   * Checks if is impartido.
   * 
   * @return true, if is impartido
   */
  @Transient
  public boolean isImpartido() {
    if (getId() > 0 && getFechaInicio() != null) {
    	Calendar hourCal = Calendar.getInstance();
  	  	Calendar finalCal = Calendar.getInstance();
  	  
  	  	hourCal.setTime(horaFin);
  	  	finalCal.setTime(fechaInicio);
  	  
  	  	finalCal.set(Calendar.HOUR_OF_DAY, hourCal.get(Calendar.HOUR_OF_DAY));
  	  	finalCal.set(Calendar.MINUTE, hourCal.get(Calendar.MINUTE));
  	  	finalCal.set(Calendar.SECOND, hourCal.get(Calendar.SECOND));
    	
    	if (finalCal.getTime().before(new Date()))
    		return true;
    }
    
	return false;
  }

  /**
   * Devuelve fecha legible.
   * 
   * @return fecha legible
   */
  @Transient
  public String getFechaLegible() {
    String resultado = "";
    DateFormat df = new SimpleDateFormat("dd/MM/yyyy");
    if (getFechaInicio() != null) {
      resultado += df.format(getFechaInicio());
    }
	if (getFechaInicio() != null && getFechaFin() != null)
	{
		resultado += " - " + df.format(getFechaFin());
	}
    return resultado;
  }

  /**
   * Devuelve horario.
   * 
   * @return horario
   */
  @Transient
  public String getHorario() {
    String resultado = "";
    DateFormat df = new SimpleDateFormat("HH:mm");
    if (getHoraInicio() != null) {
      resultado += df.format(getHoraInicio());
    }
    if (getHoraInicio() != null && getHoraFin() != null) resultado += "-";
    if (getHoraFin() != null) {
      resultado += df.format(getHoraFin());
    }
    return resultado;
  }

  /**
   * Devuelve edad maxima.
   * 
   * @return edad maxima
   */
  @NotNull
  @org.hibernate.validator.Min(value = 1)
  public int getEdadMaxima() {
    return edadMaxima;
  }

  /**
   * Asigna edad maxima.
   * 
   * @param edadMaxima
   *          edad maxima
   */
  public void setEdadMaxima(int edadMaxima) {
    this.edadMaxima = edadMaxima;
  }

  /**
   * Devuelve edad minima.
   * 
   * @return edad minima
   */
  @NotNull
  @org.hibernate.validator.Min(value = 1)
  public int getEdadMinima() {
    return edadMinima;
  }

  /**
   * Asigna edad minima.
   * 
   * @param edadMinima
   *          edad minima
   */
  public void setEdadMinima(int edadMinima) {
    this.edadMinima = edadMinima;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#isCerrada()
   */
  @Transient
  @Override
  public boolean isCerrada() {
    return getId() != 0 && getFechaInicio().before(new Date());
  }

  /**
   * Checks if is editado calendario.
   * 
   * @return true, if is editado calendario
   */
  @Transient
  public boolean isEditadoCalendario() {
    return !getCalendarItems().isEmpty();
  }

  /**
   * 
   */
  @Transient
  public EEstadoMatricula getEstadoMatricula() {
	  Usuario usuario = ((UsuarioLdap)SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUser();
	  
	  EEstadoMatricula estadoMatricula = EEstadoMatricula.NO_MATRICULADO;

	  for (MatriculaCharla mc : matriculas) {
		  if (usuario.equals(mc.getUsuario())) {
			  if (mc.isPreMatriculadoWeb() && mc.isEnListaEspera() && !mc.isAdmitido() && !mc.isCancelada())
				  estadoMatricula = EEstadoMatricula.SOLICITUD_ENVIADA;
			  else if (!mc.isEnListaEspera() && mc.isAdmitido() && !mc.isCancelada())
				  estadoMatricula = EEstadoMatricula.MATRICULADO;
			  else if (!mc.isEnListaEspera() && !mc.isAdmitido() && !mc.isCancelada())
				  estadoMatricula = EEstadoMatricula.RECHAZADO;
			  else if(mc.isCancelada())
				  estadoMatricula = EEstadoMatricula.CANCELADA;
		  }
	  }
	  
	  return estadoMatricula;
  }
  
  @Transient
  @Override
  public boolean isExterna() {
	  return false;
  }
}
