package es.csa.fdt.cyldigital.servicios.entity;

/*
 * #%L
 * Librerias del nucleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Map;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Digits;
import org.hibernate.validator.NotNull;
import org.hibernate.validator.Validator;
import org.hibernate.validator.ValidatorClass;
import org.springframework.security.context.SecurityContextHolder;

import es.csa.fdt.cyldigital.calendario.entity.ActividadCalendarItem;
import es.csa.fdt.cyldigital.centros.entity.Aula;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.diplomas.entity.Diploma;
import es.csa.fdt.cyldigital.evaluacion.entity.Examen;
import es.csa.fdt.cyldigital.evaluacion.entity.Formulario;
import es.csa.fdt.cyldigital.evaluacion.entity.FormularioConcreto;
import es.csa.fdt.cyldigital.evaluacion.entity.Grupo;
import es.csa.fdt.cyldigital.evaluacion.entity.Item;
import es.csa.fdt.cyldigital.evaluacion.entity.Pregunta;
import es.csa.fdt.cyldigital.evaluacion.entity.Respuesta;
import es.csa.fdt.cyldigital.language.entity.CursoLanguage;
import es.csa.fdt.cyldigital.recursos.entity.Recurso;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;

/**
 * Clase Curso.
 * 
 * @author Fundetec
 */
@Entity
@Table(name = "cursos")
@Audited
@PrimaryKeyJoinColumn(name = "idActividad")
@es.csa.fdt.cyldigital.servicios.entity.Curso.FechasConcordantes
public class Curso extends Actividad implements Serializable {
	private static final long serialVersionUID = -8361498848042755096L;

	private static final int NUMERO_PLAZAS_DEFECTO = 20;

	private static final int NUMERO_PLAZAS_LISTA_ESPERA = 5;

	private static final float MAX_VALORACION = 10;

	private Centro centro;

	private Aula aula;

	private Empleado profesor;

	private Date fechaInicio;

	private Date fechaFin;

	private Date horaInicio;

	private Date horaFin;

	private int numeroPlazas;

	private Examen examen;

	private boolean hayExamenInicial;
	
	//RF_XEA_15 añadir campos para la prueba de nivel
	
	private Examen examenNivel;
	
	private boolean esPruebaNivelObligatoria;
	
	//RF_XEA_15 añadir campos para la prueba de nivel

	private float numeroHoras;

	private List<MatriculaCurso> matriculas;

	private String observaciones;

	private Formulario cuestionarioSatisfaccion;

	private List<PreinscripcionCurso> preinscripcionesRechazadas;

	private List<CursoLanguage> textosCurso;

	private Integer numeroPlazasListaEspera;
	
	// RF_15_25 Configuración de cursos
	
	private Diploma plantillaDiploma;
	
	// RF_15_25 Configuración de cursos

	public enum EEstadoMatricula {
		NO_MATRICULADO,
		SOLICITUD_ENVIADA,
		MATRICULADO,
		RECHAZADO,
		CANCELADA;
	}

	// RF_??_?? Clonado de actividades
	private boolean esClonado;
	
	/**
	 * Construye un nuevo objeto de la clase
	 */
	public Curso() {
		super();

		setNumeroPlazas(NUMERO_PLAZAS_DEFECTO);
		//RF_XEA_01 --> Al quitar el examen como obligatorio en un servicio de un curso, cuando se crea un curso, por defecto el examen inicial debe de ir a false
		//              dado que el servicio no tiene asociado un examen y por lo tanto no tiene examen inicial.
		//RF_XEA_15 --> Como se añade al servicio de un curso tener o no una prueba de nivel, el campo esPruebaNivelObligatoria indica si la prueba es obligatoria
		//              o no en el caso de que el servicio tenga prueba de nivel. Por defecto aunque el servicio tenga prueba de nivel, este campo será false para que
		//              sea el usuario el que decida si la prueba de nivel es obligatoria o no en el momento de crear el curso.
		//setHayExamenInicial(true);
		setHayExamenInicial(false);
		setEsPruebaNivelObligatoria(false);
		
		//Today by default
		fechaInicio = new Date();
		
		//Tomorrow by default
		fechaInicio.setTime(fechaInicio.getTime() + 86400000);
		
		//Day after tomorrow by default
		fechaFin = new Date(fechaInicio.getTime() + 86400000);
		
		//00:00 by default
		horaInicio = new Date(0);
		
		//02:00 by default
		horaFin = new Date(horaInicio.getTime() + 7200000);

		recursos = new ArrayList<Recurso>();
		observacionesActividad = new ArrayList<ObservacionActividad>();
		calendarItems = new ArrayList<ActividadCalendarItem>();

		// RF_??_?? Clonado de actividades
		setEsClonado(false);

	}

	// RF_??_?? Clonado de actividades
	/**
	 * Establece si es un curso clonado
	 * 
	 * @param esClonado
	 *            the esClonado to set
	 */
	public void setEsClonado(boolean esClonado) {
		this.esClonado = esClonado;
	}

	
	/**
	 * Devuelve si es un curso clonado
	 * 
	 * @return the esClonado
	 */
	public boolean isEsClonado() {
		return esClonado;
	}
	/*
	 * (non-Javadoc)
	 * 
	 * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getCentro()
	 */
	@Override
	@ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
	@JoinColumn(name = "idCentro")
	@NotNull
	public Centro getCentro() {
		return centro;
	}

	/**
	 * Asigna centro.
	 * 
	 * @param centro
	 *            centro
	 */
	public void setCentro(Centro centro) {
		this.centro = centro;
	}

	/**
	 * Devuelve aula.
	 * 
	 * @return aula
	 */
	@ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
	@JoinColumn(name = "idAula")
	public Aula getAula() {
		return aula;
	}

	/**
	 * Asigna aula.
	 * 
	 * @param aula
	 *            aula
	 */
	public void setAula(Aula aula) {
		this.aula = aula;
	}

	/**
	 * Devuelve profesor.
	 * 
	 * @return profesor
	 */
	@ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
	@JoinColumn(name = "idPersonal")
	public Empleado getProfesor() {
		return profesor;
	}

	/**
	 * Asigna profesor.
	 * 
	 * @param profesor
	 *            profesor
	 */
	public void setProfesor(Empleado profesor) {
		this.profesor = profesor;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getFechaInicio()
	 */
	@NotNull
	@Temporal(TemporalType.DATE)
	public Date getFechaInicio() {
		return fechaInicio;
	}

	/**
	 * Asigna fecha inicio.
	 * 
	 * @param fechaInicio
	 *            fecha inicio
	 */
	public void setFechaInicio(Date fechaInicio) {
		this.fechaInicio = fechaInicio;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getFechaFin()
	 */
	@NotNull
	@Temporal(TemporalType.DATE)
	public Date getFechaFin() {
		return fechaFin;
	}

	/**
	 * Asigna fecha fin.
	 * 
	 * @param fechaFin
	 *            fecha fin
	 */
	public void setFechaFin(Date fechaFin) {
		this.fechaFin = fechaFin;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getHoraInicio()
	 */
	@NotNull
	@Temporal(TemporalType.TIME)
	public Date getHoraInicio() {
		return horaInicio;
	}

	@Transient
	public Long getTimeStampInicio() {
		Calendar hourCal = Calendar.getInstance();
		Calendar finalCal = Calendar.getInstance();

		hourCal.setTime(horaInicio);
		finalCal.setTime(fechaInicio);

		finalCal.set(Calendar.HOUR_OF_DAY, hourCal.get(Calendar.HOUR_OF_DAY));
		finalCal.set(Calendar.MINUTE, hourCal.get(Calendar.MINUTE));
		finalCal.set(Calendar.SECOND, hourCal.get(Calendar.SECOND));

		return finalCal.getTimeInMillis();
	}

	@Transient
	public Long getTimeStampFin() {
		Calendar hourCal = Calendar.getInstance();
		Calendar finalCal = Calendar.getInstance();

		hourCal.setTime(horaFin);
		finalCal.setTime(fechaFin);

		finalCal.set(Calendar.HOUR_OF_DAY, hourCal.get(Calendar.HOUR_OF_DAY));
		finalCal.set(Calendar.MINUTE, hourCal.get(Calendar.MINUTE));
		finalCal.set(Calendar.SECOND, hourCal.get(Calendar.SECOND));

		return finalCal.getTimeInMillis();
	}

	/**
	 * Asigna hora inicio.
	 * 
	 * @param horaInicio
	 *            hora inicio
	 */
	public void setHoraInicio(Date horaInicio) {
		this.horaInicio = horaInicio;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getHoraFin()
	 */
	@NotNull
	@Temporal(TemporalType.TIME)
	public Date getHoraFin() {
		return horaFin;
	}

	/**
	 * Asigna hora fin.
	 * 
	 * @param horaFin
	 *            hora fin
	 */
	public void setHoraFin(Date horaFin) {
		this.horaFin = horaFin;
	}

	/**
	 * Devuelve numero plazas.
	 * 
	 * @return numero plazas
	 */
	@NotNull
	public int getNumeroPlazas() {
		return numeroPlazas;
	}

	/**
	 * Asigna numero plazas.
	 * 
	 * @param numeroPlazas
	 *            numero plazas
	 */
	public void setNumeroPlazas(int numeroPlazas) {
		this.numeroPlazas = numeroPlazas;
	}

	/**
	 * Devuelve el examen que se utilizará a partir del primer examen realizado
	 * 
	 * @return the examen
	 */
	@ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
	@JoinColumn(name = "idExamen")
	public Examen getExamen() {
		return examen;
	}

	/**
	 * Establece el examen que se utilizará a partir del primer examen realizado
	 * 
	 * @param examen
	 *            the examen to set
	 */
	public void setExamen(Examen examen) {
		this.examen = examen;
	}
	
	//RF_XEA_15 añadir el examen de nivel
	/**
	 * Devuelve el examen que se utilizará como examen de nivel
	 * 
	 * @return the examenNivel
	 */
	@ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
	@JoinColumn(name = "idExamenNivel")
	public Examen getExamenNivel() {
		return examenNivel;
	}

	/**
	 * Establece el examen que se utilizará como examen de nivel
	 * 
	 * @param examen
	 *            the examen to set
	 */
	public void setExamenNivel(Examen examenNivel) {
		this.examenNivel = examenNivel;
	}
	//RF_XEA_15 añadir el examen de nivel

	/**
	 * Devuelve si se permite realizar el examen inicial
	 * 
	 * @return the hayExamenInicial
	 */
	@NotNull
	@Column(nullable = false, columnDefinition = "bit(1) default 1")
	public boolean isHayExamenInicial() {
		return hayExamenInicial;
	}

	/**
	 * Establece si se permite realizar el examen inicial
	 * 
	 * @param hayExamenInicial
	 *            the hayExamenInicial to set
	 */
	public void setHayExamenInicial(boolean hayExamenInicial) {
		this.hayExamenInicial = hayExamenInicial;
	}
	
	//RF_XEA_15 añadir si es obligatorio o no el examen de nivel
	
	/**
	 * Devuelve si la prueba de nivel es obligatoria o no
	 * 
	 * @return the esPruebaNivelObligatoria
	 */
	@NotNull
	@Column(nullable = false, columnDefinition = "bit(1) default 1")
	public boolean isEsPruebaNivelObligatoria() {
		return esPruebaNivelObligatoria;
	}

	/**
	 * Establece si la prueba de nivel es obligatoria o no
	 * 
	 * @param esPruebaNivelObligatoria
	 *            the esPruebaNivelObligatoria to set
	 */
	public void setEsPruebaNivelObligatoria(boolean esPruebaNivelObligatoria) {
		this.esPruebaNivelObligatoria = esPruebaNivelObligatoria;
	}
		
	//RF_XEA_15 añadir si es obligatorio o no el examen de nivel

	/**
	 * Devuelve numero horas.
	 * 
	 * @return numero horas
	 */
	@NotNull
	@Digits(integerDigits = 20, fractionalDigits = 2)
	public float getNumeroHoras() {
		return numeroHoras;
	}

	/**
	 * Asigna numero horas.
	 * 
	 * @param numeroHoras
	 *            numero horas
	 */
	public void setNumeroHoras(float numeroHoras) {
		this.numeroHoras = numeroHoras;
	}

	/**
	 * Devuelve matriculas.
	 * 
	 * @return matriculas
	 */
	@OneToMany(fetch = FetchType.LAZY, cascade = { javax.persistence.CascadeType.ALL }, mappedBy = "curso")
	// @Cascade(org.hibernate.annotations.CascadeType.DELETE_ORPHAN)
	@OrderBy(value = "fechaMatricula ASC")
	public List<MatriculaCurso> getMatriculas() {
		return matriculas;
	}

	/**
	 * Asigna matriculas.
	 * 
	 * @param matriculas
	 *            matriculas
	 */
	public void setMatriculas(List<MatriculaCurso> matriculas) {
		this.matriculas = matriculas;
	}

	/**
	 * Devuelve observaciones.
	 * 
	 * @return observaciones
	 */
	@Lob
	@Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
	public String getObservaciones() {
		return observaciones;
	}

	/**
	 * Asigna observaciones.
	 * 
	 * @param observaciones
	 *            observaciones
	 */
	public void setObservaciones(String observaciones) {
		this.observaciones = observaciones;
	}

	/**
	 * Devuelve preinscripciones rechazadas.
	 * 
	 * @return preinscripciones rechazadas
	 */
	@ManyToMany(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH },
				mappedBy = "cursosRechazos", targetEntity = PreinscripcionCurso.class)
	public List<PreinscripcionCurso> getPreinscripcionesRechazadas() {
		return preinscripcionesRechazadas;
	}

	/**
	 * Asigna preinscripciones rechazadas.
	 * 
	 * @param preinscripcionesRechazadas
	 *            preinscripciones rechazadas
	 */
	public void setPreinscripcionesRechazadas(
			List<PreinscripcionCurso> preinscripcionesRechazadas) {
		this.preinscripcionesRechazadas = preinscripcionesRechazadas;
	}

	/**
	 * Devuelve cuestionario satisfaccion.
	 * 
	 * @return cuestionario satisfaccion
	 */
	@NotNull
	@ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
	@JoinColumn(name = "idCuestionarioSatisfaccion")
	public Formulario getCuestionarioSatisfaccion() {
		return cuestionarioSatisfaccion;
	}

	/**
	 * Asigna cuestionario satisfaccion.
	 * 
	 * @param cuestionarioSatisfaccion
	 *            cuestionario satisfaccion
	 */
	public void setCuestionarioSatisfaccion(Formulario cuestionarioSatisfaccion) {
		this.cuestionarioSatisfaccion = cuestionarioSatisfaccion;
	}

	/**
	 * Devuelve numero plazas lista espera.
	 * 
	 * @return numero plazas lista espera
	 */
	@NotNull
	public Integer getNumeroPlazasListaEspera() {
		return numeroPlazasListaEspera == null ? NUMERO_PLAZAS_LISTA_ESPERA : numeroPlazasListaEspera;
	}

	/**
	 * Asigna numero plazas lista espera.
	 * 
	 * @param numeroPlazasListaEspera
	 *            numero plazas lista espera
	 */
	public void setNumeroPlazasListaEspera(Integer numeroPlazasListaEspera) {
		this.numeroPlazasListaEspera = numeroPlazasListaEspera;
	}

	/**
	 * Devuelve textos del curso.
	 * 
	 * @return textosCurso
	 */
	@OneToMany(cascade = CascadeType.ALL)
	@JoinColumn(name = "idCurso")
	public List<CursoLanguage> getTextosCurso() {
		return textosCurso;
	}

	/**
	 * Asigna los textos al curso.
	 * 
	 * @param textosCurso
	 *            textosCurso
	 */
	public void setTextosCurso(List<CursoLanguage> textosCurso) {
		this.textosCurso = textosCurso;
	}

	/**
	 * Devuelve numero inscritos.
	 * 
	 * @return numero inscritos
	 */
	@Transient
	public int getNumeroInscritos() {
		if (getMatriculas() != null) {
			List<MatriculaCurso> mc = new ArrayList<MatriculaCurso>();
    	
	    	for(MatriculaCurso m : getMatriculas()) {
	    		if(!m.getDesactivado() && !m.isCancelada()) mc.add(m);
	    	}
	    	return mc.size();
		}
		
		return 0;
	}

	/**
	 * Devuelve numero matriculados.
	 * 
	 * @return numero matriculados
	 */
	@Transient
	public int getNumeroMatriculados() {
		int matriculados = 0;

		if (getMatriculas() != null) {
			for (MatriculaCurso m : getMatriculas()) {
				if (!m.isEnListaEspera() && m.isAdmitido())
					matriculados++;
			}
		}

		return matriculados;
	}

	/**
	 * Devuelve numero asistentes.
	 * 
	 * @return numero asistentes
	 */
	@Transient
	public int getNumeroAsistentes() {
		int asistentes = 0;

		if (isCerrada()) {
			for (MatriculaCurso m : getMatriculas()) {
				if (m.getAsistencia() != null && m.getAsistencia() >= 80)
					asistentes++;
			}
		}

		return asistentes;
	}

	/**
	 * Devuelve numero lista espera.
	 * 
	 * @return numero lista espera
	 */
	@Transient
	public int getNumeroListaEspera() {
		int enListaEspera = 0;

		if (getMatriculas() != null) {
			for (MatriculaCurso m : getMatriculas()) {
				if (m.isEnListaEspera() && !m.isAdmitido())
					enListaEspera++;
			}
		}

		return enListaEspera;
	}

	/**
	 * Devuelve numero confirmados.
	 * 
	 * @return numero confirmados
	 */
	@Transient
	public int getNumeroConfirmados() {
		int i = 0;

		if (getMatriculas() != null) {
			for (MatriculaCurso m : getMatriculas()) {
				if (m.isAdmitido() && !m.isEnListaEspera())
					i++;
			}
		}

		return i;
	}

	/**
	 * Devuelve numero sin confirmar.
	 * 
	 * @return numero sin confirmar
	 */
	@Transient
	public int getNumeroSinConfirmar() {
		if (getMatriculas() != null)
			return getMatriculas().size() - getNumeroConfirmados();

		return 0;
	}

	/**
	 * Devuelve numero de prematriculados por Web.
	 * 
	 * @return numero de prematriculados por Web
	 */
	@Transient
	public int getNumeroPrematriculadosWeb() {
		int prematriculadosWeb = 0;

		if (getMatriculas() != null) {
			for (MatriculaCurso mc : getMatriculas()) {
				if (mc.isPreMatriculadoWeb())
					prematriculadosWeb++;
			}
		}

		return prematriculadosWeb;
	}

	/**
	 * Devuelve numero rechazados.
	 * 
	 * @return numero de rechazados
	 */
	@Transient
	public int getNumeroRechazados() {
		int rechazados = 0;

		if (getMatriculas() != null) {
			for (MatriculaCurso mc : getMatriculas()) {
				if (!mc.isAdmitido() && !mc.isEnListaEspera())
					rechazados++;
			}
		}

		return rechazados;
	}

	/**
	 * Devuelve numero solicitudes web en curso.
	 * 
	 * @return numero solicitudes web en curso
	 */
	@Transient
	public int getNumeroListaEsperaWeb() {
		int enListaEsperaWeb = 0;

		if (getMatriculas() != null) {
			for (MatriculaCurso m : getMatriculas()) {
				if (m.isEnListaEspera() && !m.isAdmitido()
						&& m.isPreMatriculadoWeb() && !m.getDesactivado())
					enListaEsperaWeb++;
			}
		}

		return enListaEsperaWeb;
	}

	/**
	 * Devuelve fechas legibles.
	 * 
	 * @return fechas legibles
	 */
	@Transient
	public String getFechasLegibles() {
		String resultado = "";
		DateFormat df = new SimpleDateFormat("dd/MM/yyyy");

		if (getFechaInicio() != null)
			resultado += df.format(getFechaInicio());

		if (getFechaInicio() != null && getFechaFin() != null)
		{
			resultado += " - " + df.format(getFechaFin());
		}

		return resultado;
	}

	/**
	 * Devuelve horario.
	 * 
	 * @return horario
	 */
	@Transient
	public String getHorario() {
		String resultado = "";
		DateFormat df = new SimpleDateFormat("HH:mm");

		if (getHoraInicio() != null)
			resultado += df.format(getHoraInicio());

		if (getHoraInicio() != null && getHoraFin() != null)
			resultado += "-";

		if (getHoraFin() != null) {
			resultado += df.format(getHoraFin());
		}

		return resultado;
	}

	/**
	 * Devuelve horario largo.
	 * 
	 * @return horario largo
	 */
	@Transient
	public String getHorarioLargo() {
		String resultado = "";
		DateFormat df = new SimpleDateFormat("HH:mm");

		if (getHoraInicio() != null)
			resultado += df.format(getHoraInicio());

		if (getHoraInicio() != null && getHoraFin() != null)
			resultado += " - ";

		if (getHoraFin() != null) {
			resultado += df.format(getHoraFin());
		}

		return resultado;
	}

	/**
	 * Checks if is completo.
	 * 
	 * @return true, if is completo
	 */
	@Transient
	public boolean isCompleto() {
		return getNumeroMatriculados() >= getNumeroPlazas();
	}

	/**
	 * Checks if is impartido.
	 * 
	 * @return true, if is impartido
	 */
	@Transient
	public boolean isImpartido() {
		if (getId() > 0 && (getFechaFin() != null) && (getHoraFin() != null)) {
			Calendar hourCal = Calendar.getInstance();
			Calendar finalCal = Calendar.getInstance();

			hourCal.setTime(getHoraFin());
			finalCal.setTime(getFechaFin());

			finalCal.set(Calendar.HOUR_OF_DAY, hourCal.get(Calendar.HOUR_OF_DAY));
			finalCal.set(Calendar.MINUTE, hourCal.get(Calendar.MINUTE));
			finalCal.set(Calendar.SECOND, hourCal.get(Calendar.SECOND));

			if (finalCal.getTime().before(new Date()))
				return true;
		}

		return false;
	}

	/**
	 * Interfaz FechasConcordantes.
	 */
	@ValidatorClass(value = FechasConcordantesValidator.class)
	@Target({ ElementType.TYPE })
	@Retention(RetentionPolicy.RUNTIME)
	@Documented
	public static @interface FechasConcordantes {

		/**
		 * Message.
		 * 
		 * @return string
		 */
		String message() default "{actividad.validator.fechaNoConcordante}";

	}

	/**
	 * Clase FechasConcordantesValidator.
	 */
	public static class FechasConcordantesValidator implements Validator<FechasConcordantes> {
		private static final long serialVersionUID = 985021755629904067L;

		/*
		 * (non-Javadoc)
		 * 
		 * @see
		 * org.hibernate.validator.Validator#initialize(java.lang.annotation
		 * .Annotation)
		 */
		@Override
		public void initialize(FechasConcordantes parameters) {
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.hibernate.validator.Validator#isValid(java.lang.Object)
		 */
		@Override
		public boolean isValid(Object value) {
			Curso curso = (Curso) value;
			
			if ((curso.getFechaInicio() != null) && (curso.getFechaFin() != null) &&
				(curso.getFechaInicio().getTime() <= curso.getFechaFin().getTime()))
				return true;
			
			return false;
		}
	}

	/**
	 * Devuelve valoracion curso.
	 * 
	 * @return valoracion curso
	 */
	@Transient
	public String getValoracion() {
		float valoracion = 0;
		
		int n = 0;
		for (int m = 0; m < matriculas.size(); m++) {
			MatriculaCurso mCurso = matriculas.get(m);
			
			if (mCurso != null) {
				Map<Pregunta, Respuesta> respuestas = mCurso.getFormularioConcreto().getRespuestas();
				
				if (cuestionarioSatisfaccion != null && cuestionarioSatisfaccion.getItems() != null &&
					respuestas != null && !respuestas.isEmpty()) {
					List<Item> items = cuestionarioSatisfaccion.getItems();
					
					for (int p = 0; p < items.size(); p++) {
						if (items.get(p) instanceof Grupo) {
							List<Pregunta> preguntas = ((Grupo) items.get(p)).getPreguntas();
							
							if (preguntas != null) {
								for (int g = 0; g < preguntas.size(); g++) {
									Respuesta r = respuestas.get(preguntas.get(g));
									
									if (r.getValorEntero() != null) {
										valoracion += (MAX_VALORACION / preguntas.get(g).getMaximo().intValue()) * r.getValorEntero();
										
										n++;
									}
									else if (r.getValorDecimal() != null) {
										valoracion += (MAX_VALORACION / preguntas.get(g).getMaximoDecimal().floatValue()) *
													  r.getValorDecimal();

										n++;
									}
								}
							}
						}
						else if (items.get(p) instanceof Pregunta) {
							Respuesta r = respuestas.get(items.get(p));

							if (r.getValorEntero() != null) {
								valoracion += (MAX_VALORACION / ((Pregunta) items.get(p)).getMaximo().intValue()) * r.getValorEntero();
								
								n++;
							}
							else if (r.getValorDecimal() != null) {
								valoracion += (MAX_VALORACION / ((Pregunta) items.get(p)).getMaximoDecimal().floatValue()) *
											  r.getValorDecimal();
								n++;
							}
						}
					}
				}
			}
		}
		
		if (n == 0)
			return "Sin valoración";
		else {
			valoracion = (valoracion / n);
			
			return String.format("%.2f sobre %.2f", valoracion, MAX_VALORACION);
		}
	}

	/**
	 * Checks if is iniciado.
	 * 
	 * @return true, if is iniciado
	 */
	@Transient
	public boolean isIniciado() {
		return getId() > 0 && getFechaInicio().before(new Date());
	}

	/**
	 * Checks if is editado calendario.
	 * 
	 * @return true, if is editado calendario
	 */
	@Transient
	public boolean isEditadoCalendario() {
		return !getCalendarItems().isEmpty();
	}

	/**
	 * Devuelve numero dias laborables.
	 * 
	 * @return numero dias laborables
	 */
	@Transient
	public int getNumeroDiasLaborables() {
		int dias = 0;

		Calendar tempStart = Calendar.getInstance();
		Calendar tempEnd = Calendar.getInstance();
		Calendar tempTime = Calendar.getInstance();

		tempTime.setTime(getHoraInicio());
		tempStart.setTime(getFechaInicio());
		tempStart.set(Calendar.HOUR, tempTime.get(Calendar.HOUR_OF_DAY));
		tempStart.set(Calendar.MINUTE, tempTime.get(Calendar.MINUTE));

		tempTime.setTime(getHoraFin());
		tempEnd.setTime(getFechaFin());
		tempEnd.set(Calendar.HOUR, tempTime.get(Calendar.HOUR_OF_DAY));
		tempEnd.set(Calendar.MINUTE, tempTime.get(Calendar.MINUTE));

		while (tempStart.before(tempEnd)) {
			if (tempStart.get(Calendar.DAY_OF_WEEK) > 1 && tempStart.get(Calendar.DAY_OF_WEEK) < 7)
				dias++;

			tempStart.set(Calendar.DATE, tempStart.get(Calendar.DATE) + 1);
		}

		return dias;
	}

	/**
	 * 
	 */
	@Transient
	public EEstadoMatricula getEstadoMatricula() {
		Usuario usuario = ((UsuarioLdap) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUser();

		EEstadoMatricula estadoMatricula = EEstadoMatricula.NO_MATRICULADO;

		for (MatriculaCurso mc : matriculas) {
			if (usuario.equals(mc.getUsuario())) {
				if (mc.isPreMatriculadoWeb() && mc.isEnListaEspera() && !mc.isAdmitido() && !mc.isCancelada())
					estadoMatricula = EEstadoMatricula.SOLICITUD_ENVIADA;
				else if (!mc.isEnListaEspera() && mc.isAdmitido() && !mc.isCancelada())
					estadoMatricula = EEstadoMatricula.MATRICULADO;
				else if (!mc.isEnListaEspera() && !mc.isAdmitido() && !mc.isCancelada())
					estadoMatricula = EEstadoMatricula.RECHAZADO;
				else if(mc.isCancelada())
					estadoMatricula = EEstadoMatricula.CANCELADA;
			}
		}

		return estadoMatricula;
	}

	@Transient
	@Override
	public boolean isExterna() {
		return false;
	}
	
	//RF_XEA_15 Funcion para obtener la matricula concreta del curso del usuario conectado
	@Transient
	public MatriculaCurso getMatriculaUsuario(){
		Usuario usuario = ((UsuarioLdap) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUser();
		
		for (MatriculaCurso mc : matriculas) {
			if(usuario.equals(mc.getUsuario())){
				return mc;
			}
		}
		
		return null;
	}

	  /**
	   * Counts the number of the evaluation questionnaires filled for this course
	   * <p>
	   * A evaluation questionnaire is considered filled if contains at least one question answered.
	   * 
	   * @return the number of the evaluation questionnaires that contains at least one question answered
	   */
	  @Transient
	  public int getEvaluationsNumber() {
		  
		  int counter = 0;
		  
		  for (MatriculaCurso matric : matriculas) {
			  FormularioConcreto evaluation = matric.getFormularioConcreto();
			  if (evaluation != null && evaluation.isFilled()) counter++;
		  }
		  
		  return counter;
		  
	  }	
	
	  /**
	   * Calculates the average score of the evaluation questionnaires
	   * <p>
	   * It takes into account only the filled evaluation questionnaires, and, for each one of those, it takes into account only the numeric type questions discarding the rest.
	   * <p>
	   * It doesn't consider the range of allowed values in the responses assuming coherent ranges for all the questions.
	   * If this is not the norm, response values normalization should be implemented: (value - min) / (max - min) 
	   * 
	   * @return the average score of the evaluation questionnaires
	   */
	  @Transient
	  public Float getAverageEvaluationScore() {
		  
		  int counter = 0;
		  float accum = 0f;
		  
		  for (MatriculaCurso matric : matriculas) {
			  FormularioConcreto evaluation = matric.getFormularioConcreto();
			  if (evaluation != null) {
				  Float averageScore = evaluation.getAverageScore();
				  if (evaluation.getAverageScore() != null) {
					  counter++;
					  accum += averageScore;
				  }
			  }
		  }

		  Float average = (counter == 0) ? null : accum / counter;
		  
		  return average;
		  
	  }
	  
	  	/**
		 * @return the plantillaDiploma
		 */
	    @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
		@JoinColumn(name = "idPlantillaDiploma")
		public Diploma getPlantillaDiploma() {
			return plantillaDiploma;
		}
		
		/**
		 * @param plantillaDiploma the plantillaDiploma to set
		 */
		public void setPlantillaDiploma(Diploma plantillaDiploma) {
			this.plantillaDiploma = plantillaDiploma;
		}
}
