package es.csa.fdt.cyldigital.servicios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.Lob;
import javax.persistence.ManyToMany;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;
import org.hibernate.validator.Digits;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;
import org.hibernate.validator.Validator;
import org.hibernate.validator.ValidatorClass;

import es.csa.commons.faces.util.PredefinedLocaleResolver;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;

/**
 * Clase CursoOnline.
 * 
 * @author csa
 */
@Entity
@Table(name = "cursosonline")
@AttributeOverride(name = "id", column = @Column(name = "idCursoOnline"))
@Audited
@PrimaryKeyJoinColumn(name = "idActividad")
@es.csa.fdt.cyldigital.servicios.entity.CursoOnline.FechasConcordantes
public class CursoOnline extends Actividad implements Serializable {

  private static final long serialVersionUID = -8361498848042755096L;

  private static final int NUMERO_PLAZAS_DEFECTO = 20;

  private static final int NUMERO_PLAZAS_LISTA_ESPERA = 5;

  private List<Empleado> profesores;

  private Date fechaInicio;

  private Date fechaFin;

  private int numeroPlazas;

  private List<MatriculaCursoOnline> matriculas;

  private String observaciones;

  private List<PreinscripcionCursoOnline> preinscripcionesRechazadas;

  private String apodo;

  private Integer numeroPlazasListaEspera;

  private long idCursoOnline;

  /**
   * Construye un nuevo objeto de la clase
   */
  public CursoOnline() {
    super();
    setNumeroPlazas(NUMERO_PLAZAS_DEFECTO);
    observacionesActividad = new ArrayList<ObservacionActividad>();
    profesores = new ArrayList<Empleado>();
  }

  // ------------------------------------------------------------------------------------------------------------GETTERS

  /**
   * Devuelve id curso online.
   * 
   * @return id curso online
   */
  @Column(unique = true)
  @GeneratedValue(strategy = GenerationType.IDENTITY)
  @Digits(integerDigits = 20)
  public long getIdCursoOnline() {
    return idCursoOnline;
  }

  /**
   * Asigna id curso online.
   * 
   * @param idCursoOnline
   *          id curso online
   */
  public void setIdCursoOnline(long idCursoOnline) {
    this.idCursoOnline = idCursoOnline;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getNombre()
   */
  @Transient
  public String getNombre() {
    return getApodo();
  }

  /**
   * Devuelve apodo.
   * 
   * @return apodo
   */
  @Length(max = 255)
  @NotEmpty
  @NotNull
  public String getApodo() {
    /*
     * Comentado para implementar los servicios MULTILANGUAGE
     * TODO: mirar y modificar
    if ((apodo == null || apodo.isEmpty()) && getServicio() != null) return getServicio().getNombre();
    */
    return apodo;
  }

  /**
   * Asigna apodo.
   * 
   * @param apodo
   *          apodo
   */
  public void setApodo(String apodo) {
    this.apodo = apodo;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getCentro()
   */
  @Override
  @Transient
  public Centro getCentro() {
    return null;
  }

  /**
   * Devuelve profesores.
   * 
   * @return profesores
   */
  @ManyToMany(targetEntity = Empleado.class, cascade = { CascadeType.ALL })
  @JoinTable(name = "profesores_actividades", joinColumns = @JoinColumn(name = "idActividad"), inverseJoinColumns = @JoinColumn(name = "idProfesor"))
  @NotAudited
  public List<Empleado> getProfesores() {
    return profesores;
  }

  /**
   * Asigna profesores.
   * 
   * @param profesores
   *          profesores
   */
  public void setProfesores(List<Empleado> profesores) {
    this.profesores = profesores;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getFechaInicio()
   */
  @NotNull
  @Temporal(TemporalType.DATE)
  public Date getFechaInicio() {
    return fechaInicio;
  }

  /**
   * Asigna fecha inicio.
   * 
   * @param fechaInicio
   *          fecha inicio
   */
  public void setFechaInicio(Date fechaInicio) {
    this.fechaInicio = fechaInicio;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getFechaFin()
   */
  @NotNull
  @Temporal(TemporalType.DATE)
  public Date getFechaFin() {
    return fechaFin;
  }

  /**
   * Asigna fecha fin.
   * 
   * @param fechaFin
   *          fecha fin
   */
  public void setFechaFin(Date fechaFin) {
    this.fechaFin = fechaFin;
  }

  /**
   * Devuelve numero plazas.
   * 
   * @return numero plazas
   */
  @NotNull
  public int getNumeroPlazas() {
    return numeroPlazas;
  }

  /**
   * Asigna numero plazas.
   * 
   * @param numeroPlazas
   *          numero plazas
   */
  public void setNumeroPlazas(int numeroPlazas) {
    this.numeroPlazas = numeroPlazas;
  }

  /**
   * Devuelve matriculas.
   * 
   * @return matriculas
   */
  @OneToMany(fetch = FetchType.LAZY, cascade = { javax.persistence.CascadeType.ALL }, mappedBy = "curso")
  // @Cascade(org.hibernate.annotations.CascadeType.DELETE_ORPHAN)
  @OrderBy(value = "fechaMatricula ASC")
  public List<MatriculaCursoOnline> getMatriculas() {
    return matriculas;
  }

  /**
   * Asigna matriculas.
   * 
   * @param matriculas
   *          matriculas
   */
  public void setMatriculas(List<MatriculaCursoOnline> matriculas) {
    this.matriculas = matriculas;
  }

  /**
   * Devuelve observaciones.
   * 
   * @return observaciones
   */
  @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getObservaciones() {
    return observaciones;
  }

  /**
   * Asigna observaciones.
   * 
   * @param observaciones
   *          observaciones
   */
  public void setObservaciones(String observaciones) {
    this.observaciones = observaciones;
  }

  /**
   * Devuelve preinscripciones rechazadas.
   * 
   * @return preinscripciones rechazadas
   */
  @ManyToMany(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH }, mappedBy = "cursosRechazos", targetEntity = PreinscripcionCursoOnline.class)
  public List<PreinscripcionCursoOnline> getPreinscripcionesRechazadas() {
    return preinscripcionesRechazadas;
  }

  /**
   * Asigna preinscripciones rechazadas.
   * 
   * @param preinscripcionesRechazadas
   *          preinscripciones rechazadas
   */
  public void setPreinscripcionesRechazadas(List<PreinscripcionCursoOnline> preinscripcionesRechazadas) {
    this.preinscripcionesRechazadas = preinscripcionesRechazadas;
  }

  /**
   * Devuelve numero plazas lista espera.
   * 
   * @return numero plazas lista espera
   */
  @NotNull
  public Integer getNumeroPlazasListaEspera() {
    return numeroPlazasListaEspera == null ? NUMERO_PLAZAS_LISTA_ESPERA : numeroPlazasListaEspera;
  }

  /**
   * Asigna numero plazas lista espera.
   * 
   * @param numeroPlazasListaEspera
   *          numero plazas lista espera
   */
  public void setNumeroPlazasListaEspera(Integer numeroPlazasListaEspera) {
    this.numeroPlazasListaEspera = numeroPlazasListaEspera;
  }

  /**
   * Devuelve numero inscritos.
   * 
   * @return numero inscritos
   */
  @Transient
  public int getNumeroInscritos() {
    if (getMatriculas() != null) return getMatriculas().size();
    return 0;
  }

  /**
   * Devuelve numero matriculados.
   * 
   * @return numero matriculados
   */
  @Transient
  public int getNumeroMatriculados() {
    if (getMatriculas() != null) {
      int aux = 0;
      for (MatriculaCursoOnline m : getMatriculas()) {
        if (!m.isEnListaEspera()) aux += 1;
      }
      return aux;
    }
    return 0;
  }

  /**
   * Devuelve numero lista espera.
   * 
   * @return numero lista espera
   */
  @Transient
  public int getNumeroListaEspera() {
    if (getMatriculas() != null) {
      int aux = 0;
      for (MatriculaCursoOnline m : getMatriculas()) {
        if (m.isEnListaEspera()) aux += 1;
      }
      return aux;
    }
    return 0;
  }

  /**
   * Devuelve numero confirmados.
   * 
   * @return numero confirmados
   */
  @Transient
  public int getNumeroConfirmados() {
    int i = 0;
    if (getMatriculas() != null) {
      for (MatriculaCursoOnline m : getMatriculas()) {
        if (m.isAdmitido()) i++;
      }
      return i;
    }
    return 0;
  }

  /**
   * Devuelve numero sin confirmar.
   * 
   * @return numero sin confirmar
   */
  @Transient
  public int getNumeroSinConfirmar() {
    if (getMatriculas() != null) return getMatriculas().size() - getNumeroConfirmados();
    return 0;
  }

  /**
   * Devuelve fecha inicio legible.
   * 
   * @return fecha inicio legible
   */
  @Transient
  public String getFechaInicioLegible() {
    String resultado = "";
    DateFormat df = new SimpleDateFormat("dd/MM/yyyy");
    if (getFechaInicio() != null) {
      resultado += df.format(getFechaInicio());
    }
    return resultado;
  }

  /**
   * Devuelve fecha fin legible.
   * 
   * @return fecha fin legible
   */
  @Transient
  public String getFechaFinLegible() {
    String resultado = "";
    DateFormat df = new SimpleDateFormat("dd/MM/yyyy");
    if (getFechaFin() != null) {
      resultado += df.format(getFechaFin());
    }
    return resultado;
  }

  /**
   * Devuelve fechas legibles.
   * 
   * @return fechas legibles
   */
  @Transient
  public String getFechasLegibles() {
    String resultado = "";
    DateFormat df = new SimpleDateFormat("dd/MM/yyyy");
    if (getFechaInicio() != null) {
      resultado += df.format(getFechaInicio());
    }
    if (getFechaInicio() != null && getFechaFin() != null) resultado += " - ";
    if (getFechaFin() != null) {
      resultado += df.format(getFechaFin());
    }
    return resultado;
  }

  /**
   * Devuelve horario.
   * 
   * @return horario
   */
  @Transient
  public String getHorario() {
    String resultado = "";
    DateFormat df = new SimpleDateFormat("HH:mm");
    if (getHoraInicio() != null) resultado += df.format(getHoraInicio());
    if (getHoraInicio() != null && getHoraFin() != null) resultado += "-";
    if (getHoraFin() != null) {
      resultado += df.format(getHoraFin());
    }
    return resultado;
  }

  /**
   * Devuelve horario largo.
   * 
   * @return horario largo
   */
  @Transient
  public String getHorarioLargo() {
    String resultado = "";
    DateFormat df = new SimpleDateFormat("HH:mm");
    if (getHoraInicio() != null) resultado += df.format(getHoraInicio());
    if (getHoraInicio() != null && getHoraFin() != null) resultado += " - ";
    if (getHoraFin() != null) {
      resultado += df.format(getHoraFin());
    }
    return resultado;
  }

  /**
   * Checks if is completo.
   * 
   * @return true, if is completo
   */
  @Transient
  public boolean isCompleto() {
    return getNumeroPlazas() - getNumeroInscritos() <= 0;
  }

  /**
   * Checks if is impartido.
   * 
   * @return true, if is impartido
   */
  @Transient
  public boolean isImpartido() {
    if (getId() > 0 && getFechaFin() != null && getFechaFin().before(new Date())) return true;
    return false;
  }

  /**
   * Interfaz FechasConcordantes.
   */
  @ValidatorClass(value = FechasConcordantesValidator.class)
  @Target( { ElementType.TYPE })
  @Retention(RetentionPolicy.RUNTIME)
  @Documented
  public static @interface FechasConcordantes {

    /**
     * Message.
     * 
     * @return string
     */
    String message() default "{actividad.validator.fechaNoConcordante}";

  }

  /**
   * Clase FechasConcordantesValidator.
   */
  public static class FechasConcordantesValidator implements Validator<FechasConcordantes> {

    private static final long serialVersionUID = 985021755629904067L;

    /* (non-Javadoc)
     * @see org.hibernate.validator.Validator#initialize(java.lang.annotation.Annotation)
     */
    @Override
    public void initialize(FechasConcordantes parameters) {}

    /* (non-Javadoc)
     * @see org.hibernate.validator.Validator#isValid(java.lang.Object)
     */
    @Override
    public boolean isValid(Object value) {
      CursoOnline curso = (CursoOnline) value;
      if (curso.getFechaInicio().getTime() <= curso.getFechaFin().getTime()) return true;
      return false;
    }
  }

  /**
   * Checks if is iniciado.
   * 
   * @return true, if is iniciado
   */
  @Transient
  public boolean isIniciado() {
    return getId() > 0 && getFechaInicio().before(new Date());
  }

  /**
   * Checks if is iniciado hoy.
   * 
   * @return true, if is iniciado hoy
   */
  @Transient
  public boolean isIniciadoHoy() {
    Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    Calendar cal2 = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    cal.setTime(getFechaInicio());
    cal2.setTime(new Date());

    return getId() > 0
        && (cal.get(Calendar.DAY_OF_YEAR) == cal2.get(Calendar.DAY_OF_YEAR) && cal.get(Calendar.YEAR) == cal2
            .get(Calendar.YEAR));
  }

  /**
   * Checks if is editado calendario.
   * 
   * @return true, if is editado calendario
   */
  @Transient
  public boolean isEditadoCalendario() {
    return !getCalendarItems().isEmpty();
  }

  /**
   * Devuelve numero dias laborables.
   * 
   * @return numero dias laborables
   */
  @Transient
  public int getNumeroDiasLaborables() {
    int dias = 0;

    Calendar tempStart = Calendar.getInstance();
    Calendar tempEnd = Calendar.getInstance();
    Calendar tempTime = Calendar.getInstance();

    tempTime.setTime(getHoraInicio());
    tempStart.setTime(getFechaInicio());
    tempStart.set(Calendar.HOUR, tempTime.get(Calendar.HOUR_OF_DAY));
    tempStart.set(Calendar.MINUTE, tempTime.get(Calendar.MINUTE));

    tempTime.setTime(getHoraFin());
    tempEnd.setTime(getFechaFin());
    tempEnd.set(Calendar.HOUR, tempTime.get(Calendar.HOUR_OF_DAY));
    tempEnd.set(Calendar.MINUTE, tempTime.get(Calendar.MINUTE));

    while (tempStart.before(tempEnd)) {
      if (tempStart.get(Calendar.DAY_OF_WEEK) > 1 && tempStart.get(Calendar.DAY_OF_WEEK) < 7) {
        dias++;
      }
      tempStart.set(Calendar.DATE, tempStart.get(Calendar.DATE) + 1);
    }

    return dias;
  }

  /**
   * Checks if is matriculas en reserva.
   * 
   * @return true, if is matriculas en reserva
   */
  @Transient
  public boolean isMatriculasEnReserva() {
    for (MatriculaCursoOnline m : getMatriculas()) {
      if (m.isEnListaEspera()) { return true; }
    }
    return false;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getHoraFin()
   */
  @Transient
  @Override
  public Date getHoraFin() {
    return null;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getHoraInicio()
   */
  @Transient
  @Override
  public Date getHoraInicio() {
    return null;
  }

  @Transient
  @Override
  public boolean isExterna() {
	  return false;
  }
}
