package es.csa.fdt.cyldigital.servicios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.PrePersist;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;
import javax.persistence.UniqueConstraint;

import org.hibernate.envers.Audited;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.calendario.entity.AsistenciaCalendarItem;
import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;

/**
 * Clase MatriculaCharla.
 * 
 * @author csa
 */
@Entity
@Table(name = "matriculascharlas", uniqueConstraints = @UniqueConstraint(columnNames = { "idUsuario", "idCharla" }))
@AttributeOverride(name = "id", column = @Column(name = "idMatriculaCharla"))
@Audited
public class MatriculaCharla extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = -5395138443458950L;

  private UsuarioCyL usuario;

  private Charla charla;

  private Date fechaConfirmacion;

  private String observaciones;
  
  private boolean enListaEspera;
  
  private boolean preMatriculadoWeb;
  
  private boolean admitido;
  
  private Empleado empleadoTramite;
  
  private Date fechaTramite;
  
  private boolean cancelada;

  /**
   * Construye un nuevo objeto de la clase
   */
  public MatriculaCharla() {
    super();
  }

  /**
   * Devuelve usuario.
   * 
   * @return usuario
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idUsuario", nullable = false)
  @NotNull
  public UsuarioCyL getUsuario() {
    return usuario;
  }

  /**
   * Asigna usuario.
   * 
   * @param usuario
   *          usuario
   */
  public void setUsuario(UsuarioCyL usuario) {
    this.usuario = usuario;
  }
  
  /**
   * Devuelve charla.
   * 
   * @return charla
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idCharla", nullable = false)
  @NotNull
  public Charla getCharla() {
    return charla;
  }

  /**
   * Asigna charla.
   * 
   * @param charla
   *          charla
   */
  public void setCharla(Charla charla) {
    this.charla = charla;
  }

  /**
   * Devuelve fecha confirmacion.
   * 
   * @return fecha confirmacion
   */
  @Temporal(TemporalType.TIMESTAMP)
  public Date getFechaConfirmacion() {
    return fechaConfirmacion;
  }

  /**
   * Asigna fecha confirmacion.
   * 
   * @param fechaConfirmacion
   *          fecha confirmacion
   */
  public void setFechaConfirmacion(Date fechaConfirmacion) {
    this.fechaConfirmacion = fechaConfirmacion;
  }

  /**
   * Pre persist.
   */
  @PrePersist
  public void prePersist() {
    if (getId() == 0) setFechaConfirmacion(new Date());
  }
  
  /**
   * Checks if is en lista espera.
   * 
   * @return true, if is en lista espera
   */
  @Column(name="enListaEspera", nullable = false, columnDefinition = "bit(1) default 0")
  @NotNull
  public boolean isEnListaEspera() {
    return enListaEspera;
  }

  /**
   * Asigna en lista espera.
   * 
   * @param enListaEspera
   *          en lista espera
   */
  public void setEnListaEspera(boolean enListaEspera) {
    this.enListaEspera = enListaEspera;
  }

  /**
   * Checks if is prematriculado web.
   * 
   * @return true, if is prematriculado web
   */
  @Column(name="preMatriculadoWeb", nullable = false, columnDefinition = "bit(1) default 0")
  @NotNull
  public boolean isPreMatriculadoWeb() {
    return preMatriculadoWeb;
  }

  /**
   * Asigna prematriculado web.
   * 
   * @param preMatriculadoWeb
   *          prematriculado web
   */
  public void setPreMatriculadoWeb(boolean preMatriculadoWeb) {
    this.preMatriculadoWeb = preMatriculadoWeb;
  }
  
  /**
   * Checks if is admitido 
   * 
   * @return true, if is admitido 
   */
  @Column(name="admitido", nullable = false, columnDefinition = "bit(1) default 0")
  @NotNull
  public boolean isAdmitido() {
    return admitido;
  }

  /**
   * Asigna admitido 
   * 
   * @param admitido
   *          admitido 
   */
  public void setAdmitido(boolean admitido) {
    this.admitido = admitido;
  }

  /**
   * Devuelve observaciones.
   * 
   * @return observaciones
   */
  @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getObservaciones() {
    return observaciones;
  }

  /**
   * Asigna observaciones.
   * 
   * @param observaciones
   *          observaciones
   */
  public void setObservaciones(String observaciones) {
    this.observaciones = observaciones;
  }

  /**
   * Devuelve asistencia.
   * 
   * @return asistencia
   */
  @Transient
  public Float getAsistencia() {
    return getAsistenciaCalculada();
  }

  /**
   * Devuelve asistencias usuario.
   * 
   * @return asistencias usuario
   */
  @Transient
  public List<AsistenciaCalendarItem> getAsistenciasUsuario() {
    List<AsistenciaCalendarItem> items = new ArrayList<AsistenciaCalendarItem>();
    for (AsistenciaCalendarItem a : charla.getAsistencias()) {
      if (a.getUsuario().getId() == usuario.getId()) {
        items.add(a);
      }
    }
    return items;
  }

  /**
   * Devuelve asistencia calculada.
   * 
   * @return asistencia calculada
   */
  @Transient
  public Float getAsistenciaCalculada() {
    float f = 100 * (float) ((float) getAsistenciasUsuario().size() / (float) (charla.getCalendarItems().size() == 0 ? 1
        : charla.getCalendarItems().size()));
    return new Float(f > 100 ? 100 : (f < 0 ? 0 : f));
  }

	/**
	 * @return the empleadoTramite
	 */
	@ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
	@JoinColumn(name = "idEmpleadoTramite")
	public Empleado getEmpleadoTramite() {
		return empleadoTramite;
	}
	
	/**
	 * @param empleadoTramite the empleadoTramite to set
	 */
	public void setEmpleadoTramite(Empleado empleadoTramite) {
		this.empleadoTramite = empleadoTramite;
	}
	
	/**
	 * @return the fechaTramite
	 */
	@Temporal(TemporalType.TIMESTAMP)
	public Date getFechaTramite() {
		return fechaTramite;
	}
	
	/**
	 * @param fechaTramite the fechaTramite to set
	 */
	public void setFechaTramite(Date fechaTramite) {
		this.fechaTramite = fechaTramite;
	}
	
	/**
   * Devuelve si la matrícula está cancelada por el usuario.
   * 
   * @return cancelada
   */
	@Column(name="cancelada", nullable = false, columnDefinition = "bit(1) default 0")
	@NotNull
  	public boolean isCancelada() {
		return cancelada;
	}

  /**
   * Asigna si una matrícula está cancelada por el usuario.
   * 
   * @param cancelada
   *          cancelada
   */
	public void setCancelada(boolean cancelada) {
		this.cancelada = cancelada;
	}

	/**
	 * 
	 */
	@Transient
    public Charla.EEstadoMatricula getEstadoMatricula() {
	  Charla.EEstadoMatricula estadoMatricula = Charla.EEstadoMatricula.NO_MATRICULADO;

	  if (isPreMatriculadoWeb() && isEnListaEspera() && !isAdmitido())
		  estadoMatricula = Charla.EEstadoMatricula.SOLICITUD_ENVIADA;
	  else if (!isEnListaEspera() && isAdmitido())
		  estadoMatricula = Charla.EEstadoMatricula.MATRICULADO;
	  else if (!isEnListaEspera() && !isAdmitido() && !isCancelada())
		  estadoMatricula = Charla.EEstadoMatricula.RECHAZADO;
	  else if(!isEnListaEspera() && !isAdmitido() && isCancelada())
			estadoMatricula = Charla.EEstadoMatricula.CANCELADA;

	  return estadoMatricula;
    }

	/**
	 * 
	 */
    public void setEstadoMatricula(Charla.EEstadoMatricula estadoMatricula) {
	  switch(estadoMatricula) {
	  	case SOLICITUD_ENVIADA:
	  		this.enListaEspera = true;
	  		this.admitido = false;
	  		this.cancelada = false;

	  		break;

	  	case MATRICULADO:
	  		this.enListaEspera = false;
	  		this.admitido = true;
	  		this.cancelada = false;

	  		break;

	  	case RECHAZADO:
	  		this.enListaEspera = false;
	  		this.admitido = false;
	  		this.cancelada = false;

	  		break;

	  	case CANCELADA:
	  		this.enListaEspera = false;
	  		this.admitido = false;
	  		this.cancelada = true;

	  		break;
	  }
    }
}
