package es.csa.fdt.cyldigital.servicios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.OneToOne;
import javax.persistence.PrePersist;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;
import javax.persistence.UniqueConstraint;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Digits;
import org.hibernate.validator.Max;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.calendario.entity.ActividadCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.AsistenciaCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.AsistenciaCalendarItem.Excepcion;
import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.diplomas.entity.Diploma;
import es.csa.fdt.cyldigital.evaluacion.entity.ExamenConcreto;
import es.csa.fdt.cyldigital.evaluacion.entity.FormularioConcreto;
import es.csa.fdt.cyldigital.usuarios.entity.Localidad;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;

/**
 * Clase MatriculaCurso.
 * 
 * @author csa
 */
@Entity
@Table(name = "matriculascursos", uniqueConstraints = @UniqueConstraint(columnNames = { "idUsuario", "idCurso" }))
@AttributeOverride(name = "id", column = @Column(name = "idMatriculaCurso"))
@Audited
public class MatriculaCurso extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = -5395138443458950L;

  private UsuarioCyL usuario;

  private Curso curso;

  private Date fechaMatricula;

  private boolean admitido;

  private boolean diplomaCreado;

  private Float nota;

  private Float asistencia;

  private boolean asistenciaManual;

  private String observaciones;

  private FormularioConcreto formularioConcreto;
  
  private boolean formularioConcretoEditable;

  private boolean enListaEspera;
  
  private boolean preMatriculadoWeb;
  
  private Empleado empleadoTramite;
  
  private Date fechaTramite;
  
  private ExamenConcreto examenInicial;
  
  private ExamenConcreto examenFinal;
  
  //RF_XEA_15 añadir campo examen de nivel
  
  private ExamenConcreto examenNivel;
  
  //RF_XEA_15 añadir campo examen de nivel
  
  private Diploma plantillaDiplomaEmitido;
  
  private Date fechaDiplomaEmitido;
  
  private Localidad localidadDiplomaEmitido;
  
  private boolean cancelada;

  /**
   * Construye un nuevo objeto de la clase
   */
  public MatriculaCurso() {
    super();
    enListaEspera = false;
    formularioConcretoEditable = true;
    formularioConcreto = new FormularioConcreto();
  }

  /**
   * Devuelve usuario.
   * 
   * @return usuario
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idUsuario", nullable = false)
  @NotNull
  public UsuarioCyL getUsuario() {
    return usuario;
  }

  /**
   * Asigna usuario.
   * 
   * @param usuario
   *          usuario
   */
  public void setUsuario(UsuarioCyL usuario) {
    this.usuario = usuario;
  }
 
  /**
   * Devuelve curso.
   * 
   * @return curso
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idCurso", nullable = false)
  @NotNull
  public Curso getCurso() {
    return curso;
  }

  /**
   * Asigna curso.
   * 
   * @param curso
   *          curso
   */
  public void setCurso(Curso curso) {
    this.curso = curso;
  }

  /**
   * Devuelve fecha matricula.
   * 
   * @return fecha matricula
   */
  @Temporal(TemporalType.TIMESTAMP)
  public Date getFechaMatricula() {
    return fechaMatricula;
  }

  /**
   * Asigna fecha matricula.
   * 
   * @param fechaMatricula
   *          fecha matricula
   */
  public void setFechaMatricula(Date fechaMatricula) {
    this.fechaMatricula = fechaMatricula;
  }

  /**
   * Checks if is admitido.
   * 
   * @return true, if is admitido
   */
  public boolean isAdmitido() {
    return admitido;
  }

  /**
   * Asigna admitido.
   * 
   * @param admitido
   *          admitido
   */
  public void setAdmitido(boolean admitido) {
    this.admitido = admitido;
  }

  /**
   * Checks if is diploma creado.
   * 
   * @return true, if is diploma creado
   */
  public boolean isDiplomaCreado() {
    return diplomaCreado;
  }

  /**
   * Asigna diploma creado.
   * 
   * @param diplomaCreado
   *          diploma creado
   */
  public void setDiplomaCreado(boolean diplomaCreado) {
    this.diplomaCreado = diplomaCreado;
  }

  /**
   * Checks if is en lista espera.
   * 
   * @return true, if is en lista espera
   */
  @Column(name="enListaEspera", nullable = false, columnDefinition = "bit(1) default 0")
  @NotNull
  public boolean isEnListaEspera() {
    return enListaEspera;
  }

  /**
   * Asigna en lista espera.
   * 
   * @param enListaEspera
   *          en lista espera
   */
  public void setEnListaEspera(boolean enListaEspera) {
    this.enListaEspera = enListaEspera;
  }

  /**
   * Checks if is prematriculado web.
   * 
   * @return true, if is prematriculado web
   */
  @Column(name="preMatriculadoWeb", nullable = false, columnDefinition = "bit(1) default 0")
  @NotNull
  public boolean isPreMatriculadoWeb() {
    return preMatriculadoWeb;
  }

  /**
   * Asigna prematriculado web.
   * 
   * @param preMatriculadoWeb
   *          prematriculado web
   */
  public void setPreMatriculadoWeb(boolean preMatriculadoWeb) {
    this.preMatriculadoWeb = preMatriculadoWeb;
  }

  /**
   * Devuelve nota.
   * 
   * @return nota
   */
  @Max(value = 10)
  @Digits(integerDigits = 2, fractionalDigits = 2)
  @org.hibernate.validator.Min(value = 0)
  public Float getNota() {
    return nota;
  }

  /**
   * Asigna nota.
   * 
   * @param nota
   *          nota
   */
  public void setNota(Float nota) {
    this.nota = nota;
  }

  /**
   * Devuelve asistencia.
   * 
   * @return asistencia
   */
  @Max(value = 100)
  @org.hibernate.validator.Min(value = 0)
  public Float getAsistencia() {
    return (asistenciaManual && asistencia != null) ? asistencia : getAsistenciaCalculada();
  }

  /**
   * Asigna asistencia.
   * 
   * @param asistencia
   *          asistencia
   */
  public void setAsistencia(Float asistencia) {
    this.asistencia = asistencia;
  }


  /**
   * Devuelve si la matrícula está cancelada por el usuario.
   * 
   * @return cancelada
   */
   @Column(name="cancelada", nullable = false, columnDefinition = "bit(1) default 0")
   @NotNull
  	public boolean isCancelada() {
		return cancelada;
	}

  /**
   * Asigna si una matrícula está cancelada por el usuario.
   * 
   * @param cancelada
   *          cancelada
   */
	public void setCancelada(boolean cancelada) {
		this.cancelada = cancelada;
	}

  /**
   * Devuelve asistencias usuario.
   * 
   * @return asistencias usuario
   */
  @Transient
  public List<AsistenciaCalendarItem> getAsistenciasUsuario() {
	  List<AsistenciaCalendarItem> items = new ArrayList<AsistenciaCalendarItem>();
    
	  if ((curso != null) && (usuario != null)) {
		  for (AsistenciaCalendarItem a : curso.getAsistencias()) {
			  if (a.getUsuario().getId() == usuario.getId())
				  items.add(a);
		  }
	  }
    
	  return items;
  }

  /**
   * Devuelve asistencia calculada.
   * 
   * @return asistencia calculada
   */
  @Transient
  public Float getAsistenciaCalculada() {
    float i = 0;
    
    /*
    for (ActividadCalendarItem ci : curso.getCalendarItems()) {
      boolean flag = false;
      for (AsistenciaCalendarItem a : getAsistenciasUsuario()) {
        if (a.getEnd() != null) {
          if (a.getStart().getTime() <= ci.getEnd().getTime() && a.getEnd().getTime() >= ci.getStart().getTime()
              && !flag) {
            i = a.getExcepcion() == Excepcion.JUSTIFICADA ? (float) (i + 0.5) : (i + 1);
            flag = true;
          }
        } else {
          if (a.getStart().getTime() <= ci.getEnd().getTime() && !flag) {
            i = a.getExcepcion() == Excepcion.JUSTIFICADA ? (float) (i + 0.5) : (i + 1);
            i++;
            flag = true;
          }
        }
      }
    }
    */
    
    for (AsistenciaCalendarItem a : getAsistenciasUsuario()) {
    	i = a.getExcepcion() == Excepcion.JUSTIFICADA ? (float) (i + 0.5) : (i + 1);
    }
    
    float f = 100 * (float) ((float) i / (float) (curso.getCalendarItems().size() == 0 ? curso
        .getNumeroDiasLaborables() : curso.getCalendarItems().size()));
    return new Float(f > 100 ? 100 : (f < 0 ? 0 : f));
  }

  /**
   * Checks if is asistencia manual.
   * 
   * @return true, if is asistencia manual
   */
  public boolean isAsistenciaManual() {
    return asistenciaManual;
  }

  /**
   * Asigna asistencia manual.
   * 
   * @param asistenciaManual
   *          asistencia manual
   */
  public void setAsistenciaManual(boolean asistenciaManual) {
    this.asistenciaManual = asistenciaManual;
  }

  /**
   * Pre persist.
   */
  @PrePersist
  public void prePersist() {
    if (getId() == 0) setFechaMatricula(new Date());
  }

  /**
   * Devuelve formulario concreto.
   * 
   * @return formulario concreto
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.ALL })
  @JoinColumn(name = "idFormularioConcreto")
  public FormularioConcreto getFormularioConcreto() {
    if (formularioConcreto == null) formularioConcreto = new FormularioConcreto();
    return formularioConcreto;
  }

  /**
   * Asigna formulario concreto.
   * 
   * @param formularioConcreto
   *          formulario concreto
   */
  public void setFormularioConcreto(FormularioConcreto formularioConcreto) {
    this.formularioConcreto = formularioConcreto;
  }
  
  /**
   * Devuelve formulario concreto editable.
   * 
   * @return formulario concreto editable
   */
  @Column(name="formularioConcretoEditable", nullable = false, columnDefinition = "bit(1) default 0")
  @NotNull
  public boolean isFormularioConcretoEditable() {
    return formularioConcretoEditable;
  }

  /**
   * Asigna formulario concreto editable.
   * 
   * @param formularioConcretoEditable
   *          formulario concreto editable
   */
  public void setFormularioConcretoEditable(boolean formularioConcretoEditable) {
    this.formularioConcretoEditable = formularioConcretoEditable;
  }

  /**
   * Devuelve observaciones.
   * 
   * @return observaciones
   */
  @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getObservaciones() {
    return observaciones;
  }

  /**
   * Asigna observaciones.
   * 
   * @param observaciones
   *          observaciones
   */
  public void setObservaciones(String observaciones) {
    this.observaciones = observaciones;
  }

	/**
	 * @return the empleadoTramite
	 */
  	@ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  	@JoinColumn(name = "idEmpleadoTramite")
	public Empleado getEmpleadoTramite() {
		return empleadoTramite;
	}
	
	/**
	 * @param empleadoTramite the empleadoTramite to set
	 */
	public void setEmpleadoTramite(Empleado empleadoTramite) {
		this.empleadoTramite = empleadoTramite;
	}
	
	/**
	 * @return the fechaTramite
	 */
	@Temporal(TemporalType.TIMESTAMP)
	public Date getFechaTramite() {
		return fechaTramite;
	}
	
	/**
	 * @param fechaTramite the fechaTramite to set
	 */
	public void setFechaTramite(Date fechaTramite) {
		this.fechaTramite = fechaTramite;
	}
	
 /**
   * 
   */
  @Transient
  public Curso.EEstadoMatricula getEstadoMatricula() {
	  Curso.EEstadoMatricula estadoMatricula = Curso.EEstadoMatricula.NO_MATRICULADO;

	  if (isPreMatriculadoWeb() && isEnListaEspera() && !isAdmitido())
		  estadoMatricula = Curso.EEstadoMatricula.SOLICITUD_ENVIADA;
	  else if (!isEnListaEspera() && isAdmitido())
		  estadoMatricula = Curso.EEstadoMatricula.MATRICULADO;
	  else if (!isEnListaEspera() && !isAdmitido() && !isCancelada())
		  estadoMatricula = Curso.EEstadoMatricula.RECHAZADO;
	  else if(!isEnListaEspera() && !isAdmitido() && isCancelada())
			estadoMatricula = Curso.EEstadoMatricula.CANCELADA;

	  return estadoMatricula;
  }
  
  
  /**
	* 
	*/
  public void setEstadoMatricula(Curso.EEstadoMatricula estadoMatricula) {
	  switch(estadoMatricula) {
	  	case SOLICITUD_ENVIADA:
	  		this.enListaEspera = true;
	  		this.admitido = false;
	  		this.cancelada = false;

	  		break;

	  	case MATRICULADO:
	  		this.enListaEspera = false;
	  		this.admitido = true;
	  		this.cancelada = false;

	  		break;

	  	case RECHAZADO:
	  		this.enListaEspera = false;
	  		this.admitido = false;
	  		this.cancelada = false;

	  		break;

	  	case CANCELADA:
	  		this.enListaEspera = false;
	  		this.admitido = false;
	  		this.cancelada = true;

	  		break;
	default:
		break;
	  }
  }

	/**
	 * @return the examenInicial
	 */
  	@OneToOne(cascade = {CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH})
  	@JoinColumn(name = "idExamenInicial")
  	public ExamenConcreto getExamenInicial() {
  		return examenInicial;
  	}

  	/**
  	 * @param examenInicial the examenInicial to set
  	 */
  	public void setExamenInicial(ExamenConcreto examenInicial) {
  		this.examenInicial = examenInicial;
  	}

  	/**
  	 * @return the examenFinal
  	 */
  	@OneToOne(cascade = {CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH})
  	@JoinColumn(name = "idExamenFinal")
  	public ExamenConcreto getExamenFinal() {
  		return examenFinal;
  	}

  	/**
  	 * @param examenFinal the examenFinal to set
  	 */
  	public void setExamenFinal(ExamenConcreto examenFinal) {
  		this.examenFinal = examenFinal;
  	}
  	
  	//RF_XEA_15 metodos set y get del examen de nivel
  	/**
  	 * @return the examenNivel
  	 */
  	@OneToOne(cascade = {CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH, CascadeType.REMOVE})
  	@JoinColumn(name = "idExamenNivel")
  	public ExamenConcreto getExamenNivel() {
  		return examenNivel;
  	}

  	/**
  	 * @param examenNivel the examenNivel to set
  	 */
  	public void setExamenNivel(ExamenConcreto examenNivel) {
  		this.examenNivel = examenNivel;
  	}
  	//RF_XEA_15 metodos set y get del examen de nivel

	/**
	 * @return the plantillaDiplomaEmitido
	 */
  	@ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
	@JoinColumn(name = "idPlantillaDiplomaEmitido")
	public Diploma getPlantillaDiplomaEmitido() {
		return plantillaDiplomaEmitido;
	}
	
	/**
	 * @param plantillaDiplomaEmitido the plantillaDiplomaEmitido to set
	 */
	public void setPlantillaDiplomaEmitido(Diploma plantillaDiplomaEmitido) {
		this.plantillaDiplomaEmitido = plantillaDiplomaEmitido;
	}
	
	/**
	 * @return the fechaDiplomaEmitido
	 */
	@Temporal(TemporalType.DATE)
	public Date getFechaDiplomaEmitido() {
		return fechaDiplomaEmitido;
	}
	
	/**
	 * @param fechaDiplomaEmitido the fechaDiplomaEmitido to set
	 */
	public void setFechaDiplomaEmitido(Date fechaDiplomaEmitido) {
		this.fechaDiplomaEmitido = fechaDiplomaEmitido;
	}
	
	/**
	 * @return the localidadDiplomaEmitido
	 */
	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "idLocalidadDiplomaEmitido")
	public Localidad getLocalidadDiplomaEmitido() {
		return localidadDiplomaEmitido;
	}
	
	/**
	 * @param localidadDiplomaEmitido the localidadDiplomaEmitido to set
	 */
	public void setLocalidadDiplomaEmitido(Localidad localidadDiplomaEmitido) {
		this.localidadDiplomaEmitido = localidadDiplomaEmitido;
	}  
}
