package es.csa.fdt.cyldigital.servicios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.Date;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.PrePersist;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;
import javax.persistence.UniqueConstraint;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Digits;
import org.hibernate.validator.Max;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioOnline;

/**
 * Clase MatriculaCursoOnline.
 * 
 * @author csa
 */
@Entity
@Table(name = "matriculascursosonline", uniqueConstraints = @UniqueConstraint(columnNames = { "idUsuarioOnline",
    "idCursoOnline" }))
@AttributeOverride(name = "id", column = @Column(name = "idMatriculaCursoOnline"))
@Audited
public class MatriculaCursoOnline extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = -5395138443458950L;

  private UsuarioOnline usuario;

  private CursoOnline curso;

  private Date fechaMatricula;

  private boolean admitido;

  private boolean diplomaCreado;

  private Float nota;

  private String observaciones;

  private boolean enListaEspera;

  /**
   * Enum EstadoMatricula.
   */
  public static enum EstadoMatricula {
    INSCRITO,
    INSCRITO_ESPERA,
    MATRICULADO,
    CURSANDO,
    APROBADO,
    NO_APROBADO,
    DIPLOMA;
  }

  /**
   * Construye un nuevo objeto de la clase
   */
  public MatriculaCursoOnline() {
    super();
    enListaEspera = false;
  }

  /**
   * Devuelve usuario.
   * 
   * @return usuario
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idUsuarioOnline", nullable = false)
  @NotNull
  public UsuarioOnline getUsuario() {
    return usuario;
  }

  /**
   * Asigna usuario.
   * 
   * @param usuario
   *          usuario
   */
  public void setUsuario(UsuarioOnline usuario) {
    this.usuario = usuario;
  }

  /**
   * Devuelve curso.
   * 
   * @return curso
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idCursoOnline", nullable = false)
  @NotNull
  public CursoOnline getCurso() {
    return curso;
  }

  /**
   * Asigna curso.
   * 
   * @param curso
   *          curso
   */
  public void setCurso(CursoOnline curso) {
    this.curso = curso;
  }

  /**
   * Devuelve fecha matricula.
   * 
   * @return fecha matricula
   */
  @Temporal(TemporalType.TIMESTAMP)
  public Date getFechaMatricula() {
    return fechaMatricula;
  }

  /**
   * Asigna fecha matricula.
   * 
   * @param fechaMatricula
   *          fecha matricula
   */
  public void setFechaMatricula(Date fechaMatricula) {
    this.fechaMatricula = fechaMatricula;
  }

  /**
   * Checks if is admitido.
   * 
   * @return true, if is admitido
   */
  public boolean isAdmitido() {
    return admitido;
  }

  /**
   * Asigna admitido.
   * 
   * @param admitido
   *          admitido
   */
  public void setAdmitido(boolean admitido) {
    this.admitido = admitido;
  }

  /**
   * Checks if is diploma creado.
   * 
   * @return true, if is diploma creado
   */
  public boolean isDiplomaCreado() {
    return diplomaCreado;
  }

  /**
   * Asigna diploma creado.
   * 
   * @param diplomaCreado
   *          diploma creado
   */
  public void setDiplomaCreado(boolean diplomaCreado) {
    this.diplomaCreado = diplomaCreado;
  }

  /**
   * Checks if is en lista espera.
   * 
   * @return true, if is en lista espera
   */
  @NotNull
  public boolean isEnListaEspera() {
    return enListaEspera;
  }

  /**
   * Asigna en lista espera.
   * 
   * @param enListaEspera
   *          en lista espera
   */
  public void setEnListaEspera(boolean enListaEspera) {
    this.enListaEspera = enListaEspera;
  }

  /**
   * Devuelve nota.
   * 
   * @return nota
   */
  @Max(value = 10)
  @Digits(integerDigits = 2, fractionalDigits = 2)
  @org.hibernate.validator.Min(value = 0)
  public Float getNota() {
    return nota;
  }

  /**
   * Asigna nota.
   * 
   * @param nota
   *          nota
   */
  public void setNota(Float nota) {
    this.nota = nota;
  }

  /**
   * Pre persist.
   */
  @PrePersist
  public void prePersist() {
    if (getId() == 0) setFechaMatricula(new Date());
  }

  /**
   * Devuelve observaciones.
   * 
   * @return observaciones
   */
  @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getObservaciones() {
    return observaciones;
  }

  /**
   * Asigna observaciones.
   * 
   * @param observaciones
   *          observaciones
   */
  public void setObservaciones(String observaciones) {
    this.observaciones = observaciones;
  }

  /**
   * Devuelve estado matriculacion.
   * 
   * @return estado matriculacion
   */
  @Transient
  public EstadoMatricula getEstadoMatriculacion() {
    if (!getCurso().isIniciado() && !isAdmitido()) {
      return !isEnListaEspera() ? EstadoMatricula.INSCRITO : EstadoMatricula.INSCRITO_ESPERA;
    } else if (!getCurso().isIniciado() && isAdmitido()) {
      return EstadoMatricula.MATRICULADO;
    } else if (!getCurso().isImpartido()) {
      return EstadoMatricula.CURSANDO;
    } else if (getCurso().isImpartido()) {
      return getNota() >= 5 ? EstadoMatricula.APROBADO : EstadoMatricula.NO_APROBADO;
    } else if (getCurso().isImpartido() && isDiplomaCreado()) {
      return EstadoMatricula.DIPLOMA;
    } else {
      return null;
    }
  }

}
