package es.csa.fdt.cyldigital.servicios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.Date;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.PrePersist;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.UniqueConstraint;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Digits;
import org.hibernate.validator.Max;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;

/**
 * Clase MatriculaExterna.
 * 
 * @author csa
 */
@Entity
@Table(name = "matriculasexternas", uniqueConstraints = @UniqueConstraint(columnNames = { "idUsuario",
    "idActividadExterna" }))
@AttributeOverride(name = "id", column = @Column(name = "idMatriculaExterna"))
@Audited
public class MatriculaExterna extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = -5395138443458950L;

  private Usuario usuario;

  private ActividadExterna externa;

  private Date fechaMatricula;

  private boolean admitido;

  private Float asistencia;

  private String observaciones;

  private boolean enListaEspera;

  /**
   * Construye un nuevo objeto de la clase
   */
  public MatriculaExterna() {
    super();
    enListaEspera = false;
  }

  /**
   * Devuelve usuario.
   * 
   * @return usuario
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idUsuario", nullable = false)
  @NotNull
  public Usuario getUsuario() {
    return usuario;
  }

  /**
   * Asigna usuario.
   * 
   * @param usuario
   *          usuario
   */
  public void setUsuario(Usuario usuario) {
    this.usuario = usuario;
  }

  /**
   * Devuelve externa.
   * 
   * @return externa
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idActividadExterna", nullable = false)
  @NotNull
  public ActividadExterna getExterna() {
    return externa;
  }

  /**
   * Asigna externa.
   * 
   * @param actividadExterna
   *          externa
   */
  public void setExterna(ActividadExterna actividadExterna) {
    this.externa = actividadExterna;
  }

  /**
   * Devuelve fecha matricula.
   * 
   * @return fecha matricula
   */
  @Temporal(TemporalType.TIMESTAMP)
  public Date getFechaMatricula() {
    return fechaMatricula;
  }

  /**
   * Asigna fecha matricula.
   * 
   * @param fechaMatricula
   *          fecha matricula
   */
  public void setFechaMatricula(Date fechaMatricula) {
    this.fechaMatricula = fechaMatricula;
  }

  /**
   * Checks if is admitido.
   * 
   * @return true, if is admitido
   */
  public boolean isAdmitido() {
    return admitido;
  }

  /**
   * Asigna admitido.
   * 
   * @param admitido
   *          admitido
   */
  public void setAdmitido(boolean admitido) {
    this.admitido = admitido;
  }

  /**
   * Checks if is en lista espera.
   * 
   * @return true, if is en lista espera
   */
  @NotNull
  public boolean isEnListaEspera() {
    return enListaEspera;
  }

  /**
   * Asigna en lista espera.
   * 
   * @param enListaEspera
   *          en lista espera
   */
  public void setEnListaEspera(boolean enListaEspera) {
    this.enListaEspera = enListaEspera;
  }

  /**
   * Devuelve asistencia.
   * 
   * @return asistencia
   */
  @Max(value = 100)
  @Digits(integerDigits = 3, fractionalDigits = 2)
  @org.hibernate.validator.Min(value = 0)
  public Float getAsistencia() {
    return asistencia;
  }

  /**
   * Asigna asistencia.
   * 
   * @param asistencia
   *          asistencia
   */
  public void setAsistencia(Float asistencia) {
    this.asistencia = asistencia;
  }

  /**
   * Pre persist.
   */
  @PrePersist
  public void prePersist() {
    if (getId() == 0) setFechaMatricula(new Date());
  }

  /**
   * Devuelve observaciones.
   * 
   * @return observaciones
   */
  @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getObservaciones() {
    return observaciones;
  }

  /**
   * Asigna observaciones.
   * 
   * @param observaciones
   *          observaciones
   */
  public void setObservaciones(String observaciones) {
    this.observaciones = observaciones;
  }

}
