package es.csa.fdt.cyldigital.servicios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.PrePersist;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;
import javax.persistence.UniqueConstraint;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Digits;
import org.hibernate.validator.Max;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.calendario.entity.ActividadCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.AsistenciaCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.AsistenciaCalendarItem.Excepcion;
import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.evaluacion.entity.FormularioConcreto;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;

/**
 * Clase MatriculaTaller.
 * 
 * @author csa
 */
@Entity
@Table(name = "matriculastalleres", uniqueConstraints = @UniqueConstraint(columnNames = { "idUsuario", "idTaller" }))
@AttributeOverride(name = "id", column = @Column(name = "idMatriculaTaller"))
@Audited
public class MatriculaTaller extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = -5395138443458950L;

  private Usuario usuario;

  private Taller taller;

  private Date fechaMatricula;

  private boolean admitido;

  private boolean diplomaCreado;

  private Float nota;

  private Float asistencia;

  private boolean asistenciaManual;

  private String observaciones;

  private FormularioConcreto formularioConcreto;

  private boolean enListaEspera;

  /**
   * Construye un nuevo objeto de la clase
   */
  public MatriculaTaller() {
    super();
    enListaEspera = false;
    formularioConcreto = new FormularioConcreto();
  }

  /**
   * Devuelve usuario.
   * 
   * @return usuario
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idUsuario", nullable = false)
  @NotNull
  public Usuario getUsuario() {
    return usuario;
  }

  /**
   * Asigna usuario.
   * 
   * @param usuario
   *          usuario
   */
  public void setUsuario(Usuario usuario) {
    this.usuario = usuario;
  }

  /**
   * Devuelve taller.
   * 
   * @return taller
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idTaller", nullable = false)
  @NotNull
  public Taller getTaller() {
    return taller;
  }

  /**
   * Asigna taller.
   * 
   * @param taller
   *          taller
   */
  public void setTaller(Taller taller) {
    this.taller = taller;
  }

  /**
   * Devuelve fecha matricula.
   * 
   * @return fecha matricula
   */
  @Temporal(TemporalType.TIMESTAMP)
  public Date getFechaMatricula() {
    return fechaMatricula;
  }

  /**
   * Asigna fecha matricula.
   * 
   * @param fechaMatricula
   *          fecha matricula
   */
  public void setFechaMatricula(Date fechaMatricula) {
    this.fechaMatricula = fechaMatricula;
  }

  /**
   * Checks if is admitido.
   * 
   * @return true, if is admitido
   */
  public boolean isAdmitido() {
    return admitido;
  }

  /**
   * Asigna admitido.
   * 
   * @param admitido
   *          admitido
   */
  public void setAdmitido(boolean admitido) {
    this.admitido = admitido;
  }

  /**
   * Checks if is diploma creado.
   * 
   * @return true, if is diploma creado
   */
  public boolean isDiplomaCreado() {
    return diplomaCreado;
  }

  /**
   * Asigna diploma creado.
   * 
   * @param diplomaCreado
   *          diploma creado
   */
  public void setDiplomaCreado(boolean diplomaCreado) {
    this.diplomaCreado = diplomaCreado;
  }

  /**
   * Checks if is en lista espera.
   * 
   * @return true, if is en lista espera
   */
  @NotNull
  public boolean isEnListaEspera() {
    return enListaEspera;
  }

  /**
   * Asigna en lista espera.
   * 
   * @param enListaEspera
   *          en lista espera
   */
  public void setEnListaEspera(boolean enListaEspera) {
    this.enListaEspera = enListaEspera;
  }

  /**
   * Devuelve nota.
   * 
   * @return nota
   */
  @Max(value = 10)
  @Digits(integerDigits = 2, fractionalDigits = 2)
  @org.hibernate.validator.Min(value = 0)
  public Float getNota() {
    return nota;
  }

  /**
   * Asigna nota.
   * 
   * @param nota
   *          nota
   */
  public void setNota(Float nota) {
    this.nota = nota;
  }

  /**
   * Devuelve asistencia.
   * 
   * @return asistencia
   */
  @Max(value = 100)
  @org.hibernate.validator.Min(value = 0)
  public Float getAsistencia() {
    return (asistenciaManual && asistencia != null) ? asistencia : getAsistenciaCalculada();
  }

  /**
   * Asigna asistencia.
   * 
   * @param asistencia
   *          asistencia
   */
  public void setAsistencia(Float asistencia) {
    this.asistencia = asistencia;
  }

  /**
   * Devuelve asistencias usuario.
   * 
   * @return asistencias usuario
   */
  @Transient
  public List<AsistenciaCalendarItem> getAsistenciasUsuario() {
    List<AsistenciaCalendarItem> items = new ArrayList<AsistenciaCalendarItem>();
    for (AsistenciaCalendarItem a : taller.getAsistencias()) {
      if (a.getUsuario().getId() == usuario.getId()) {
        items.add(a);
      }
    }
    return items;
  }

  /**
   * Devuelve asistencia calculada.
   * 
   * @return asistencia calculada
   */
  @Transient
  public Float getAsistenciaCalculada() {
    float i = 0;
    for (ActividadCalendarItem ci : taller.getCalendarItems()) {
      boolean flag = false;
      for (AsistenciaCalendarItem a : getAsistenciasUsuario()) {
        if (a.getEnd() != null) {
          if (a.getStart().getTime() <= ci.getEnd().getTime() && a.getEnd().getTime() >= ci.getStart().getTime()
              && !flag) {
            i = a.getExcepcion() == Excepcion.JUSTIFICADA ? (float) (i + 0.5) : (i + 1);
            flag = true;
          }
        } else {
          if (a.getStart().getTime() <= ci.getEnd().getTime() && !flag) {
            i = a.getExcepcion() == Excepcion.JUSTIFICADA ? (float) (i + 0.5) : (i + 1);
            i++;
            flag = true;
          }
        }
      }
    }
    float f = 100 * (float) ((float) i / (float) (taller.getCalendarItems().size() == 0 ? taller
        .getNumeroDiasLaborables() : taller.getCalendarItems().size()));
    return new Float(f > 100 ? 100 : (f < 0 ? 0 : f));
  }

  /**
   * Checks if is asistencia manual.
   * 
   * @return true, if is asistencia manual
   */
  public boolean isAsistenciaManual() {
    return asistenciaManual;
  }

  /**
   * Asigna asistencia manual.
   * 
   * @param asistenciaManual
   *          asistencia manual
   */
  public void setAsistenciaManual(boolean asistenciaManual) {
    this.asistenciaManual = asistenciaManual;
  }

  /**
   * Pre persist.
   */
  @PrePersist
  public void prePersist() {
    if (getId() == 0) setFechaMatricula(new Date());
  }

  /**
   * Devuelve formulario concreto.
   * 
   * @return formulario concreto
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.ALL })
  @JoinColumn(name = "idFormularioConcreto")
  public FormularioConcreto getFormularioConcreto() {
    if (formularioConcreto == null) formularioConcreto = new FormularioConcreto();
    return formularioConcreto;
  }

  /**
   * Asigna formulario concreto.
   * 
   * @param formularioConcreto
   *          formulario concreto
   */
  public void setFormularioConcreto(FormularioConcreto formularioConcreto) {
    this.formularioConcreto = formularioConcreto;
  }

  /**
   * Devuelve observaciones.
   * 
   * @return observaciones
   */
  @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getObservaciones() {
    return observaciones;
  }

  /**
   * Asigna observaciones.
   * 
   * @param observaciones
   *          observaciones
   */
  public void setObservaciones(String observaciones) {
    this.observaciones = observaciones;
  }
}
