package es.csa.fdt.cyldigital.servicios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Digits;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.calendario.entity.ActividadCalendarItem;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.recursos.entity.Ordenador;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;

/**
 * Clase Navegacion.
 * 
 * @author csa
 */
@Entity
@Table(name = "navegaciones")
@AttributeOverride(name = "id", column = @Column(name = "idNavegacion"))
@Audited
@PrimaryKeyJoinColumn(name = "idActividad")
public class Navegacion extends Actividad implements Serializable {

  private static final long serialVersionUID = -8361498848042755096L;

  private Usuario usuario;

  private Centro centro;

  private String observaciones;

  private Ordenador ordenador;

  private long idNavegacion;

  /**
   * Construye un nuevo objeto de la clase
   */
  public Navegacion() {
    super();
    calendarItems = new ArrayList<ActividadCalendarItem>();
  }

  // ------------------------------------------------------------------------------------------------------------GETTERS
  /**
   * Devuelve id navegacion.
   * 
   * @return id navegacion
   */
  @Column(unique = true)
  @GeneratedValue(strategy = GenerationType.IDENTITY)
  @Digits(integerDigits = 20)
  public long getIdNavegacion() {
    return idNavegacion;
  }

  /**
   * Asigna id navegacion.
   * 
   * @param idNavegacion
   *          id navegacion
   */
  public void setIdNavegacion(long idNavegacion) {
    this.idNavegacion = idNavegacion;
  }

  /**
   * Devuelve usuario.
   * 
   * @return usuario
   */
  @ManyToOne(fetch = FetchType.LAZY)
  @JoinColumn(name = "idUsuario")
  public Usuario getUsuario() {
    return usuario;
  }

  /**
   * Asigna usuario.
   * 
   * @param usuario
   *          usuario
   */
  public void setUsuario(Usuario usuario) {
    this.usuario = usuario;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getCentro()
   */
  @Override
  @ManyToOne(fetch = FetchType.LAZY)
  @JoinColumn(name = "idCentro")
  @NotNull
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Devuelve observaciones.
   * 
   * @return observaciones
   */
  @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getObservaciones() {
    return observaciones;
  }

  /**
   * Asigna observaciones.
   * 
   * @param observaciones
   *          observaciones
   */
  public void setObservaciones(String observaciones) {
    this.observaciones = observaciones;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getFechaInicio()
   */
  @Temporal(TemporalType.DATE)
  public Date getFechaInicio() {
    if (getCalendarItems() != null && !getCalendarItems().isEmpty()) { return getCalendarItems().get(0).getStart(); }
    return null;
  }

  /**
   * Asigna fecha inicio.
   * 
   * @param fechaInicio
   *          fecha inicio
   */
  public void setFechaInicio(Date fechaInicio) {}

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getFechaFin()
   */
  @Temporal(TemporalType.DATE)
  public Date getFechaFin() {
    if (getCalendarItems() != null && !getCalendarItems().isEmpty()) {
      Date d = null;
      for (ActividadCalendarItem a : getCalendarItems()) {
        if (d == null || a.getEnd().after(d)) d = a.getEnd();
      }
      return d;
    }
    return null;
  }

  /**
   * Asigna fecha fin.
   * 
   * @param fechaFin
   *          fecha fin
   */
  public void setFechaFin(Date fechaFin) {}

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getHoraFin()
   */
  @Temporal(TemporalType.TIME)
  public Date getHoraFin() {
    return getFechaFin();
  }

  /**
   * Asigna hora inicio.
   * 
   * @param horaInicio
   *          hora inicio
   */
  public void setHoraInicio(Date horaInicio) {}

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getHoraInicio()
   */
  @Temporal(TemporalType.TIME)
  public Date getHoraInicio() {
    return getFechaInicio();
  }

  /**
   * Asigna hora fin.
   * 
   * @param horaFin
   *          hora fin
   */
  public void setHoraFin(Date horaFin) {}

  /**
   * Devuelve ordenador.
   * 
   * @return ordenador
   */
  @ManyToOne(fetch = FetchType.LAZY, targetEntity = Ordenador.class)
  public Ordenador getOrdenador() {
    return ordenador;
  }

  /**
   * Asigna ordenador.
   * 
   * @param ordenador
   *          ordenador
   */
  public void setOrdenador(Ordenador ordenador) {
    this.ordenador = ordenador;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getNombre()
   */
  /*@Override
  @Transient
  public String getNombre() {
    return null;
  }*/
  
  @Transient
  @Override
  public boolean isExterna() {
	  return false;
  }
}
