package es.csa.fdt.cyldigital.servicios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.Date;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import org.hibernate.envers.Audited;
import org.hibernate.validator.NotEmpty;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;

/**
 * Clase ObservacionActividad.
 * 
 * @author csa
 */
@Entity
@Table(name = "observacionesactividad")
@AttributeOverride(name = "id", column = @Column(name = "idObservacionActividad"))
@Audited
public class ObservacionActividad extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = -7648562176655166537L;

  private String observacion;
  private Date fecha;
  private Usuario usuario;
  private Actividad actividad;

  /**
   * Construye un nuevo objeto de la clase
   */
  public ObservacionActividad() {
    super();

  }

  /**
   * Devuelve observacion.
   * 
   * @return observacion
   */
  @NotEmpty
  @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getObservacion() {
    return observacion;
  }

  /**
   * Asigna observacion.
   * 
   * @param observacion
   *          observacion
   */
  public void setObservacion(String observacion) {
    this.observacion = observacion;
  }

  /**
   * Devuelve fecha.
   * 
   * @return fecha
   */
  public Date getFecha() {
    return fecha;
  }

  /**
   * Asigna fecha.
   * 
   * @param fecha
   *          fecha
   */
  public void setFecha(Date fecha) {
    this.fecha = fecha;
  }

  /**
   * Devuelve usuario.
   * 
   * @return usuario
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idUsuario", nullable = false)
  public Usuario getUsuario() {
    return usuario;
  }

  /**
   * Asigna usuario.
   * 
   * @param usuario
   *          usuario
   */
  public void setUsuario(Usuario usuario) {
    this.usuario = usuario;
  }

  /**
   * Devuelve actividad.
   * 
   * @return actividad
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idActividad", nullable = false)
  public Actividad getActividad() {
    return actividad;
  }

  /**
   * Asigna actividad.
   * 
   * @param actividad
   *          actividad
   */
  public void setActividad(Actividad actividad) {
    this.actividad = actividad;
  }
}
