package es.csa.fdt.cyldigital.servicios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Digits;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.calendario.entity.ActividadCalendarItem;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.recursos.entity.Ordenador;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;

/**
 * Clase PracticaTaller.
 * 
 * @author csa
 */
@Entity
@Table(name = "practicastalleres")
//@AttributeOverride(name = "id", column = @Column(name = "idPracticaTaller"))
@Audited
@PrimaryKeyJoinColumn(name = "idActividad")
public class PracticaTaller extends Actividad implements Serializable {

  private static final long serialVersionUID = -8361498848042755096L;

  private UsuarioCyL usuario;

  private TipoPractica tipoPractica;

  private Centro centro;

  private String observaciones;

  private Date fecha;

  private Date hora;

  private Ordenador ordenador;

  //private long idPracticaTaller;

  /**
   * Construye un nuevo objeto de la clase
   */
  public PracticaTaller() {
    super();
    calendarItems = new ArrayList<ActividadCalendarItem>();
  }

  // ------------------------------------------------------------------------------------------------------------GETTERS
  /**
   * Devuelve id practica taller.
   * 
   * @return id practica taller
   */
  /*@Column(unique = true)
  @GeneratedValue(strategy = GenerationType.IDENTITY)
  @Digits(integerDigits = 20)
  public long getIdPracticaTaller() {
    return idPracticaTaller;
  }*/

  /**
   * Asigna id practica taller.
   * 
   * @param idPracticaTaller
   *          id practica taller
   */
 /* public void setIdPracticaTaller(long idPracticaTaller) {
    this.idPracticaTaller = idPracticaTaller;
  }*/

  /**
   * Devuelve tipo practica.
   * 
   * @return tipo practica
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idTipoPractica")
  @NotNull
  public TipoPractica getTipoPractica() {
    return tipoPractica;
  }

  /**
   * Asigna tipo practica.
   * 
   * @param tipoPractica
   *          tipo practica
   */
  public void setTipoPractica(TipoPractica tipoPractica) {
    this.tipoPractica = tipoPractica;
  }

  /**
   * Devuelve usuario.
   * 
   * @return usuario
   */
  @NotNull
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idUsuario")
  public UsuarioCyL getUsuario() {
    return usuario;
  }

  /**
   * Asigna usuario.
   * 
   * @param usuario
   *          usuario
   */
  public void setUsuario(UsuarioCyL usuario) {
    this.usuario = usuario;
  }

  /**
   * Devuelve fecha.
   * 
   * @return fecha
   */
  @NotNull
  @Temporal(TemporalType.DATE)
  public Date getFecha() {
    return fecha;
  }

  /**
   * Asigna fecha.
   * 
   * @param fecha
   *          fecha
   */
  public void setFecha(Date fecha) {
    this.fecha = fecha;
  }

  /**
   * Devuelve hora.
   * 
   * @return hora
   */
  @NotNull
  @Temporal(TemporalType.TIME)
  public Date getHora() {
    return hora;
  }

  /**
   * Asigna hora.
   * 
   * @param hora
   *          hora
   */
  public void setHora(Date hora) {
    this.hora = hora;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getCentro()
   */
  @Override
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idCentro")
  @NotNull
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Devuelve observaciones.
   * 
   * @return observaciones
   */
  @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getObservaciones() {
    return observaciones;
  }

  /**
   * Asigna observaciones.
   * 
   * @param observaciones
   *          observaciones
   */
  public void setObservaciones(String observaciones) {
    this.observaciones = observaciones;
  }

  /**
   * Devuelve ordenador.
   * 
   * @return ordenador
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idOrdenador")
  @NotNull
  public Ordenador getOrdenador() {
    return ordenador;
  }

  /**
   * Asigna ordenador.
   * 
   * @param ordenador
   *          ordenador
   */
  public void setOrdenador(Ordenador ordenador) {
    this.ordenador = ordenador;
  }

  /**
   * Devuelve fecha legible.
   * 
   * @return fecha legible
   */
  @Transient
  public String getFechaLegible() {
    String resultado = "";
    DateFormat dfFecha = new SimpleDateFormat("dd/MM/yyyy");
    DateFormat dfHora = new SimpleDateFormat("hh:mm");
    if (getFecha() != null) {
      resultado += dfFecha.format(getFecha());
    }
    if (getHora() != null) {
      resultado += " " + dfHora.format(getHora());
    }
    return resultado;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getNombre()
   */
  /*@Override
  @Transient
  public String getNombre() {
    return null;
  }*/

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getFechaFin()
   */
  @Override
  @Transient
  public Date getFechaFin() {
    return fecha;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getFechaInicio()
   */
  @Override
  @Transient
  public Date getFechaInicio() {
    return fecha;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getHoraFin()
   */
  @Override
  @Transient
  public Date getHoraFin() {
    return hora;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getHoraInicio()
   */
  @Override
  @Transient
  public Date getHoraInicio() {
    return hora;
  }

  @Transient
  @Override
  public boolean isExterna() {
	  return false;
  }
}
