package es.csa.fdt.cyldigital.servicios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.Date;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OrderBy;
import javax.persistence.PrePersist;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.UniqueConstraint;

import org.hibernate.envers.Audited;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioOnline;

/**
 * Clase PreinscripcionCursoOnline.
 * 
 * @author csa
 */
@Entity
@Table(name = "preinscripcionescursosonline", uniqueConstraints = @UniqueConstraint(columnNames = { "idUsuarioOnline",
    "idCurso" }))
@AttributeOverride(name = "id", column = @Column(name = "idPreinscripcionCursoOnline"))
@Audited
public class PreinscripcionCursoOnline extends AbstractEntity implements Serializable {

  /**
   * Enum PreferenciaHoraria.
   */
  public enum PreferenciaHoraria {
    /** Indiferente */
    I,
    /** Mañana */
    M,
    /** Tarde */
    T,
  }

  private static final long serialVersionUID = 1L;

  private UsuarioOnline usuario;

  private ServicioCursoOnline servicioCurso;

  private Date fecha;

  private List<CursoOnline> cursosRechazos;

  private int peso;

  /**
   * Construye un nuevo objeto de la clase
   */
  public PreinscripcionCursoOnline() {
    super();
    setPeso(0);
  }

  /**
   * Devuelve usuario.
   * 
   * @return usuario
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idUsuarioOnline", nullable = false)
  public UsuarioOnline getUsuario() {
    return usuario;
  }

  /**
   * Asigna usuario.
   * 
   * @param usuario
   *          usuario
   */
  public void setUsuario(UsuarioOnline usuario) {
    this.usuario = usuario;
  }

  /**
   * Devuelve servicio curso.
   * 
   * @return servicio curso
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idCurso", nullable = false)
  public ServicioCursoOnline getServicioCurso() {
    return servicioCurso;
  }

  /**
   * Asigna servicio curso.
   * 
   * @param curso
   *          servicio curso
   */
  public void setServicioCurso(ServicioCursoOnline curso) {
    this.servicioCurso = curso;
  }

  /**
   * Devuelve fecha.
   * 
   * @return fecha
   */
  @Temporal(TemporalType.TIMESTAMP)
  public Date getFecha() {
    return fecha;
  }

  /**
   * Asigna fecha.
   * 
   * @param fecha
   *          fecha
   */
  public void setFecha(Date fecha) {
    this.fecha = fecha;
  }

  /**
   * Numero de veces que ha rechazado asistir a un curso concreto.
   * 
   * @return cursos rechazos
   */
  @ManyToMany(targetEntity = CursoOnline.class, cascade = { javax.persistence.CascadeType.MERGE,
      javax.persistence.CascadeType.PERSIST, javax.persistence.CascadeType.REFRESH })
  @JoinTable(name = "preinscripcionescursosonline_rechazados", joinColumns = @JoinColumn(name = "idPreinscripcion"), inverseJoinColumns = @JoinColumn(name = "idCursoOnline"))
  @OrderBy(value = "fechaInicio ASC")
  public List<CursoOnline> getCursosRechazos() {
    return cursosRechazos;
  }

  /**
   * Asigna cursos rechazos.
   * 
   * @param cursosRechazos
   *          cursos rechazos
   */
  public void setCursosRechazos(List<CursoOnline> cursosRechazos) {
    this.cursosRechazos = cursosRechazos;
  }

  /**
   * Devuelve peso.
   * 
   * @return peso
   */
  public int getPeso() {
    return peso;
  }

  /**
   * Asigna peso.
   * 
   * @param peso
   *          peso
   */
  public void setPeso(int peso) {
    this.peso = peso;
  }

  /**
   * Pre persist.
   */
  @PrePersist
  public void prePersist() {
    if (getId() == 0) setFecha(new Date());
  }
}
