package es.csa.fdt.cyldigital.servicios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.Date;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.PrePersist;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.UniqueConstraint;

import org.hibernate.envers.Audited;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;

/**
 * Clase PreinscripcionPropuesta.
 * 
 * @author csa
 */
@Entity
@Table(name = "preinscripcionespropuestas", uniqueConstraints = @UniqueConstraint(columnNames = { "idUsuario",
    "idPropuestaServicio", "idCentro" }))
@AttributeOverride(name = "id", column = @Column(name = "idPreinscripcionPropuesta"))
@Audited
public class PreinscripcionPropuesta extends AbstractEntity implements Serializable {

  /**
   * Enum PreferenciaHoraria.
   */
  public enum PreferenciaHoraria {
    /** Indiferente */
    I,
    /** Mañana */
    M,
    /** Tarde */
    T,
  }

  private static final long serialVersionUID = 1L;

  private Usuario usuario;

  private PropuestaServicio propuestaServicio;

  private Centro centro;

  private Date fecha;

  private PreferenciaHoraria horarioPreferido;

  /**
   * Construye un nuevo objeto de la clase
   */
  public PreinscripcionPropuesta() {
    super();
  }

  /**
   * Devuelve usuario.
   * 
   * @return usuario
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idUsuario", nullable = false)
  @NotNull
  public Usuario getUsuario() {
    return usuario;
  }

  /**
   * Asigna usuario.
   * 
   * @param usuario
   *          usuario
   */
  public void setUsuario(Usuario usuario) {
    this.usuario = usuario;
  }

  /**
   * Devuelve propuesta servicio.
   * 
   * @return propuesta servicio
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idPropuestaServicio")
  public PropuestaServicio getPropuestaServicio() {
    return propuestaServicio;
  }

  /**
   * Asigna propuesta servicio.
   * 
   * @param propuestaServicio
   *          propuesta servicio
   */
  public void setPropuestaServicio(PropuestaServicio propuestaServicio) {
    this.propuestaServicio = propuestaServicio;
  }

  /**
   * Devuelve fecha.
   * 
   * @return fecha
   */
  @Temporal(TemporalType.TIMESTAMP)
  public Date getFecha() {
    return fecha;
  }

  /**
   * Asigna fecha.
   * 
   * @param fecha
   *          fecha
   */
  public void setFecha(Date fecha) {
    this.fecha = fecha;
  }

  /**
   * Devuelve centro.
   * 
   * @return centro
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idCentro", nullable = false)
  @NotNull
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Devuelve horario preferido.
   * 
   * @return horario preferido
   */
  @Enumerated(value = EnumType.STRING)
  @NotNull
  public PreferenciaHoraria getHorarioPreferido() {
    return this.horarioPreferido;
  }

  /**
   * Asigna horario preferido.
   * 
   * @param horarioPreferido
   *          horario preferido
   */
  public void setHorarioPreferido(PreferenciaHoraria horarioPreferido) {
    this.horarioPreferido = horarioPreferido;
  }

  /**
   * Pre persist.
   */
  @PrePersist
  public void prePersist() {
    if (getId() == 0) setFecha(new Date());
  }
}
