package es.csa.fdt.cyldigital.servicios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.Date;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OrderBy;
import javax.persistence.PrePersist;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.UniqueConstraint;

import org.hibernate.envers.Audited;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;

/**
 * Clase PreinscripcionTaller.
 * 
 * @author csa
 */
@Entity
@Table(name = "preinscripcionestalleres", uniqueConstraints = @UniqueConstraint(columnNames = { "idUsuario",
    "idTaller", "idCentro" }))
@AttributeOverride(name = "id", column = @Column(name = "idPreinscripcionTaller"))
@Audited
public class PreinscripcionTaller extends AbstractEntity implements Serializable {

  /**
   * Enum PreferenciaHoraria.
   */
  public enum PreferenciaHoraria {
    /** Indiferente */
    I,
    /** Mañana */
    M,
    /** Tarde */
    T,
  }

  private static final long serialVersionUID = 1L;

  private Usuario usuario;

  private ServicioTaller servicioTaller;

  private Centro centro;

  private Date fecha;

  private PreferenciaHoraria horarioPreferido;

  private List<Taller> talleresRechazos;

  private List<ActividadExterna> externasTalleresRechazos;

  private int peso;

  /**
   * Construye un nuevo objeto de la clase
   */
  public PreinscripcionTaller() {
    super();
    setPeso(0);
  }

  /**
   * Devuelve usuario.
   * 
   * @return usuario
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idUsuario", nullable = false)
  public Usuario getUsuario() {
    return usuario;
  }

  /**
   * Asigna usuario.
   * 
   * @param usuario
   *          usuario
   */
  public void setUsuario(Usuario usuario) {
    this.usuario = usuario;
  }

  /**
   * Devuelve servicio taller.
   * 
   * @return servicio taller
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idTaller", nullable = false)
  public ServicioTaller getServicioTaller() {
    return servicioTaller;
  }

  /**
   * Asigna servicio taller.
   * 
   * @param taller
   *          servicio taller
   */
  public void setServicioTaller(ServicioTaller taller) {
    this.servicioTaller = taller;
  }

  /**
   * Devuelve fecha.
   * 
   * @return fecha
   */
  @Temporal(TemporalType.TIMESTAMP)
  public Date getFecha() {
    return fecha;
  }

  /**
   * Asigna fecha.
   * 
   * @param fecha
   *          fecha
   */
  public void setFecha(Date fecha) {
    this.fecha = fecha;
  }

  /**
   * Devuelve centro.
   * 
   * @return centro
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idCentro", nullable = false)
  @NotNull
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Devuelve horario preferido.
   * 
   * @return horario preferido
   */
  @Enumerated(value = EnumType.STRING)
  @NotNull
  public PreferenciaHoraria getHorarioPreferido() {
    return this.horarioPreferido;
  }

  /**
   * Asigna horario preferido.
   * 
   * @param horarioPreferido
   *          horario preferido
   */
  public void setHorarioPreferido(PreferenciaHoraria horarioPreferido) {
    this.horarioPreferido = horarioPreferido;
  }

  /**
   * Numero de veces que ha rechazado asistir a un taller concreto.
   * 
   * @return talleres rechazos
   */
  @ManyToMany(targetEntity = Taller.class, cascade = { javax.persistence.CascadeType.MERGE,
      javax.persistence.CascadeType.PERSIST, javax.persistence.CascadeType.REFRESH })
  @JoinTable(name = "preinscripcionestalleres_rechazados", joinColumns = @JoinColumn(name = "idPreinscripcion"), inverseJoinColumns = @JoinColumn(name = "idTaller"))
  @OrderBy(value = "fechaInicio ASC")
  public List<Taller> getTalleresRechazos() {
    return talleresRechazos;
  }

  /**
   * Asigna talleres rechazos.
   * 
   * @param talleresRechazos
   *          talleres rechazos
   */
  public void setTalleresRechazos(List<Taller> talleresRechazos) {
    this.talleresRechazos = talleresRechazos;
  }

  /**
   * Devuelve externas talleres rechazos.
   * 
   * @return externas talleres rechazos
   */
  @ManyToMany(targetEntity = ActividadExterna.class, cascade = { javax.persistence.CascadeType.MERGE,
      javax.persistence.CascadeType.PERSIST, javax.persistence.CascadeType.REFRESH })
  @JoinTable(name = "preinscripcionesexternastalleres_rechazadas", joinColumns = @JoinColumn(name = "idPreinscripcion"), inverseJoinColumns = @JoinColumn(name = "idActividadExterna"))
  @OrderBy(value = "fechaInicio ASC")
  public List<ActividadExterna> getExternasTalleresRechazos() {
    return externasTalleresRechazos;
  }

  /**
   * Asigna externas talleres rechazos.
   * 
   * @param externasTalleresRechazos
   *          externas talleres rechazos
   */
  public void setExternasTalleresRechazos(List<ActividadExterna> externasTalleresRechazos) {
    this.externasTalleresRechazos = externasTalleresRechazos;
  }

  // @Transient
  // public int getPosicion() {
  // if (getServicioTaller() != null && getServicioTaller().getPreinscripciones() != null)
  // return getServicioTaller().getPreinscripciones().indexOf(this) + 1;
  // else
  // return 1;
  // }
  //
  // @Transient
  // public int getPosicionEnCentro() {
  // List<PreinscripcionTaller> preinscripciones = new ArrayList<PreinscripcionTaller>();
  // boolean flag=false;
  // if (getServicioTaller() != null && getServicioTaller().getPreinscripciones() != null) {
  // for (PreinscripcionTaller preinscripcionTaller : getServicioTaller().getPreinscripciones()) {
  // if(preinscripcionTaller.getCentro().getId()== getCentro().getId()){
  // preinscripciones.add(preinscripcionTaller);
  // }
  // }
  // return preinscripciones.indexOf(this) + 1;
  // } else
  // return 1;
  // }

  /**
   * Devuelve peso.
   * 
   * @return peso
   */
  public int getPeso() {
    return peso;
  }

  /**
   * Asigna peso.
   * 
   * @param peso
   *          peso
   */
  public void setPeso(int peso) {
    this.peso = peso;
  }

  /**
   * Pre persist.
   */
  @PrePersist
  public void prePersist() {
    if (getId() == 0) setFecha(new Date());
  }
}
