package es.csa.fdt.cyldigital.servicios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.OrderBy;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.annotations.Cascade;
import org.hibernate.envers.Audited;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.language.entity.PropuestaServicioLanguage;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
/**
 * Clase PropuestaServicio.
 * 
 * @author csa
 */
@Entity
@Table(name = "propuestasservicios")
@AttributeOverride(name = "id", column = @Column(name = "idPropuestaServicio"))
@Audited
public class PropuestaServicio extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = -8361498848042755096L;

  private List<PreinscripcionPropuesta> preinscripciones;
  
  private Empleado solicitante;

  //private String nombre;

  //private String comentarios;
  
  private List<PropuestaServicioLanguage> textos;

  /**
   * Construye un nuevo objeto de la clase
   */
  public PropuestaServicio() {
    super();
  }

  // ------------------------------------------------------------------------------------------------------------GETTERS

  /**
   * Devuelve nombre.
   * 
   * @return nombre
   */
  /*@Length(min = 1, max = 100)
  @NotNull
  public String getNombre() {
    return nombre;
  }*/

  /**
   * Asigna nombre.
   * 
   * @param nombre
   *          nombre
   */
 /* public void setNombre(String nombre) {
    this.nombre = nombre;
  }*/

  /**
   * Devuelve comentarios.
   * 
   * @return comentarios
   */
  /*@Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getComentarios() {
    return comentarios;
  }*/

  /**
   * Asigna comentarios.
   * 
   * @param comentarios
   *          comentarios
   */
  /*public void setComentarios(String comentarios) {
    this.comentarios = comentarios;
  }*/
  
  /**
	 * Devuelve textos de la propuesta de servicio
	 * 
	 * @return textos
	 */
	@OneToMany(cascade = CascadeType.ALL)
	@JoinColumn(name = "idPropuestaServicio")
	public List<PropuestaServicioLanguage> getTextos() {
		return textos;
	}

	/**
	 * Asigna los textos a la propuesta de servicio
	 * 
	 * @param textos
	 *          textos
	 */
	public void setTextos(List<PropuestaServicioLanguage> textos) {
		this.textos = textos;
	}

  /**
   * Devuelve preinscripciones.
   * 
   * @return preinscripciones
   */
  @OneToMany(fetch = FetchType.LAZY, cascade = { javax.persistence.CascadeType.ALL }, mappedBy = "propuestaServicio")
  @Cascade(org.hibernate.annotations.CascadeType.DELETE_ORPHAN)
  @OrderBy(value = "fecha ASC")
  public List<PreinscripcionPropuesta> getPreinscripciones() {
    return preinscripciones;
  }

  /**
   * Asigna preinscripciones.
   * 
   * @param preinscripciones
   *          preinscripciones
   */
  public void setPreinscripciones(List<PreinscripcionPropuesta> preinscripciones) {
    this.preinscripciones = preinscripciones;
  }

  /**
   * Devuelve numero preinscripciones.
   * 
   * @return numero preinscripciones
   */
  public int getNumeroPreinscripciones() {
    int resultado = 0;
    
    if (getPreinscripciones() != null && getPreinscripciones().size() > 0) {
    	for (PreinscripcionPropuesta p : getPreinscripciones()) {
    		if (!p.getDesactivado()) resultado++;
    	}
    }
    
    return resultado;
  }
  
  /**
   * Establece numero preinscripciones.
   * 
   * @param numero preinscripciones
   */
  public void setNumeroPreinscripciones(int numeroPreinscripciones) {
  }

  /**
   * @return the solicitante
   */
  @OneToOne(cascade = CascadeType.REFRESH)
  @JoinColumn(name = "idEmpleado")
  public Empleado getSolicitante() {
	  return solicitante;
  }

  /**
   * @param solicitante the solicitante to set
   */
  public void setSolicitante(Empleado solicitante) {
	  this.solicitante = solicitante;
  }
}
