package es.csa.fdt.cyldigital.servicios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotNull;


import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.language.entity.ServicioLanguage;

/**
 * Clase Servicio.
 * 
 * @author csa
 */
@Entity
@Table(name = "servicios")
@Audited
@Inheritance(strategy = InheritanceType.JOINED)
@AttributeOverride(name = "id", column = @Column(name = "idServicio"))
public abstract class Servicio extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = 9067364741355911439L;

  /**
   * Enum Nivel.
   */
  public enum Nivel {
    /** Avanzado **/
    A,
    /** Medio **/
    M,
    /** Bássico **/
    B
  }

  /** nombre. */
  //protected String nombre;

  /** descripcion. */
  //protected String descripcion;
  
  /** nombre y descripcion en el idioma adecuado */
  protected List<ServicioLanguage> textos;

  /** codigo junta. */
  protected String codigoJunta;

  /** edad minima. */
  protected int edadMinima;
  
  /** edad maxima. */
  protected int edadMaxima;

  private Tematica tematica;

  private List<Actividad> actividades;
  
  private boolean disponibleColaborador;
  
  private boolean ajenoPlanFormativo;

   /**
     * Devuelve nombre.
     * 
     * @return nombre
     */
    /*@Length(min = 1, max = 100)
    @NotNull*/
    /*public String getNombre(Language language) {
    	String RetNombre = "";
    	
    	for (ServicioLanguage OneText : textos) {
			if (language == OneText.getLanguage())
				RetNombre = OneText.getNombre();
		}
    	
    	return RetNombre;
    }*/

    /**
     * Asigna nombre.
     * 
     * @param nombre
     *          nombre
     */
    /*public void setNombre(String nombre, Language language) {
    	for (ServicioLanguage OneText : textos) {
			if (language == OneText.getLanguage())
				OneText.setNombre(nombre);
		}                                                                                                                                                                                                                                                                                                                                                                                                       
    }*/

    /**
     * Devuelve descripcion.
     * 
     * @return descripcion
     */
    /*@Lob
    @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")*/
    /*public String getDescripcion(Language language) {
    	String RetDescripcion = "";
    	
    	for (ServicioLanguage OneText : textos) {
			if (language == OneText.getLanguage())
				RetDescripcion = OneText.getDescripcion();
		}
    	
    	return RetDescripcion;
    }*/

    /**
	* Asigna descripcion.
	* 
	* @param descripcion
	*          descripcion
	*/
	/*public void setDescripcion(String descripcion, Language language) {
		for (ServicioLanguage OneText : textos) {
			if (language == OneText.getLanguage())
				OneText.setDescripcion(descripcion);
		}       
	}*/

	/**
	 * Devuelve textos del servicio
	 * 
	 * @return textos
	 */
	@OneToMany(cascade = CascadeType.ALL)
	@JoinColumn(name = "idServicio")
	public List<ServicioLanguage> getTextos() {
		return textos;
	}

	/**
	 * Asigna los textos al servicio
	 * 
	 * @param textos
	 *          textos
	 */
	public void setTextos(List<ServicioLanguage> textos) {
		this.textos = textos;
	}

// TODO Eliminar actividades
  /**
   * Devuelve actividades.
   * 
   * @return actividades
   */
  @OneToMany(mappedBy = "servicio", cascade = { javax.persistence.CascadeType.REFRESH })
  public List<Actividad> getActividades() {
    return actividades;
  }

  /**
   * Asigna actividades.
   * 
   * @param actividades
   *          actividades
   */
  public void setActividades(List<Actividad> actividades) {
    this.actividades = actividades;
  }

  /**
   * Devuelve codigo junta.
   * 
   * @return codigo junta
   */
  @Length(min = 0, max = 100)
  public String getCodigoJunta() {
    return codigoJunta;
  }

  /**
   * Asigna codigo junta.
   * 
   * @param codigoJunta
   *          codigo junta
   */
  public void setCodigoJunta(String codigoJunta) {
    this.codigoJunta = codigoJunta;
  }

  /**
   * Devuelve edad minima.
   * 
   * @return edad minima
   */
  @NotNull
  @org.hibernate.validator.Min(value = 1)
  public int getEdadMinima() {
    return edadMinima;
  }

  /**
   * Asigna edad minima.
   * 
   * @param edadMinima
   *          edad minima
   */
  public void setEdadMinima(int edadMinima) {
    this.edadMinima = edadMinima;
  }

  /**
   * Devuelve edad maxima.
   * 
   * @return edad maxima
   */
  @NotNull
  @org.hibernate.validator.Min(value = 1)
  public int getEdadMaxima() {
    return edadMaxima;
  }

  /**
   * Asigna edad maxima.
   * 
   * @param edadMaxima
   *          edad maxima
   */
  public void setEdadMaxima(int edadMaxima) {
    this.edadMaxima = edadMaxima;
  }

  /**
   * Devuelve tematica.
   * 
   * @return tematica
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idTematica")
  public Tematica getTematica() {
    return tematica;
  }

  /**
   * Asigna tematica.
   * 
   * @param tematica
   *          tematica
   */
  public void setTematica(Tematica tematica) {
    this.tematica = tematica;
  }
  
  /**
   * Checks if is disponibleColaborador 
   * 
   * @return true, if is disponibleColaborador 
   */
  @NotNull
  @Column(name="disponibleColaborador", nullable = false, columnDefinition = "bit(1) default 0")
  public boolean isDisponibleColaborador() {
    return disponibleColaborador ;
  }

  /**
   * Asigna disponibleColaborador 
   * 
   * @param disponibleColaborador
   *          disponibleColaborador 
   */
  public void setDisponibleColaborador(boolean disponibleColaborador) {
    this.disponibleColaborador = disponibleColaborador;
  }
  
  /**
   * Checks if is ajenoPlanFormativo 
   * 
   * @return true, if is ajenoPlanFormativo 
   */
  @NotNull
  @Column(name="ajenoPlanFormativo", nullable = false, columnDefinition = "bit(1) default 0")
  public boolean isAjenoPlanFormativo() {
    return ajenoPlanFormativo ;
  }

  /**
   * Asigna ajenoPlanFormativo 
   * 
   * @param ajenoPlanFormativo
   *          ajenoPlanFormativo 
   */
  public void setAjenoPlanFormativo(boolean ajenoPlanFormativo) {
    this.ajenoPlanFormativo = ajenoPlanFormativo;
  }

  @Transient
  public abstract boolean getAreThereFiles();
}
