package es.csa.fdt.cyldigital.servicios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.annotations.Cascade;
import org.hibernate.envers.Audited;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.diplomas.entity.Diploma;
import es.csa.fdt.cyldigital.evaluacion.entity.Examen;
import es.csa.fdt.cyldigital.language.entity.ServicioCursoLanguage;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario.Role;

/**
 * Clase ServicioCurso.
 * 
 * @author csa
 */
@Entity
@Table(name = "servicioscursos")
@Audited
@PrimaryKeyJoinColumn(name = "idServicio")
public class ServicioCurso extends Servicio implements Serializable {

  private static final long serialVersionUID = -8361498848042755096L;
  
  private List<PreinscripcionCurso> preinscripciones;

  private Nivel nivel;

  //private String guion;

  //private File guionPdf;
  //private FileOnDisk ficheroGuion;

  private List<ServicioCursoLanguage> guiones;

  private List<ServicioCursoEnlace> enlaces;
  
  private Examen examen;
  
  private Diploma plantillaDiploma;
  
  private List<Role> rolesEmisoresDiplomaList;
  
  //RF_XEA_15 Añadimos el campo para la prueba de nivel
  private Examen examenNivel;

  /**
   * Construye un nuevo objeto de la clase
   */
  public ServicioCurso() {
    super();

    rolesEmisoresDiplomaList = new ArrayList<Role>();
    rolesEmisoresDiplomaList.add(Role.ROLE_ADMIN);
  }

  /**
   * Devuelve preinscripciones.
   * 
   * @return preinscripciones
   */
  @OneToMany(fetch = FetchType.LAZY, cascade = { javax.persistence.CascadeType.ALL }, mappedBy = "servicioCurso")
  @Cascade(org.hibernate.annotations.CascadeType.DELETE_ORPHAN)
  @OrderBy(value = "peso ASC ,fecha ASC")
  public List<PreinscripcionCurso> getPreinscripciones() {
    return preinscripciones;
  }

  /**
   * Asigna preinscripciones.
   * 
   * @param preinscripciones
   *          preinscripciones
   */
  public void setPreinscripciones(List<PreinscripcionCurso> preinscripciones) {
    this.preinscripciones = preinscripciones;
  }

  /**
   * Devuelve nivel.
   * 
   * @return nivel
   */
  @Enumerated(value = EnumType.STRING)
  @NotNull
  public Nivel getNivel() {
    return nivel;
  }

  /**
   * Asigna nivel.
   * 
   * @param nivel
   *          nivel
   */
  public void setNivel(Nivel nivel) {
    this.nivel = nivel;
  }

  /**
   * Devuelve guion.
   * 
   * @return guion
   */
  /*@Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getGuion() {
    return guion;
  }*/

  /**
   * Asigna guion.
   * 
   * @param guion
   *          guion
   */
  /*public void setGuion(String guion) {
    this.guion = guion;
  }*/

  /**
   * Devuelve guion pdf.
   * 
   * @return guion pdf
   */
  /*@ManyToOne(fetch = FetchType.LAZY, cascade = { javax.persistence.CascadeType.ALL })
  @JoinColumn(name = "idGuionPdf")
  public File getGuionPdf() {
    return guionPdf;
  }*/

  /**
   * Asigna guion pdf.
   * 
   * @param guionPdf
   *          guion pdf
   */
  /*public void setGuionPdf(File guionPdf) {
    this.guionPdf = guionPdf;
  }*/

  /**
   * Devuelve numero preinscripciones.
   * 
   * @return numero preinscripciones
   */
  public int getNumeroPreinscripciones() {
    int resultado = 0;
    if (getPreinscripciones() != null && getPreinscripciones().size() > 0) {
      for (PreinscripcionCurso p : getPreinscripciones()) {
        if (!p.getDesactivado()) /* Filtra los desactivados */
        resultado++;
      }
    }
    return resultado;
  }

  /**
   * Asigna numero preinscripciones.
   * 
   * @param preinscripciones
   *          numero preinscripciones
   */
  public void setNumeroPreinscripciones(int preinscripciones) {}

  /**
   * Devuelve fichero guion.
   * 
   * @return fichero guion
   */
  /*@ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idFicheroGuion")
  public FileOnDisk getFicheroGuion() {
    return ficheroGuion;
  }*/

  /**
   * Asigna fichero guion.
   * 
   * @param ficheroGuion
   *          fichero guion
   */
  /*public void setFicheroGuion(FileOnDisk ficheroGuion) {
    this.ficheroGuion = ficheroGuion;
  }*/
  
  /**
	 * Devuelve guiones del servicio curso.
	 * 
	 * @return guiones
	 */
	@OneToMany(cascade = CascadeType.ALL)
	@JoinColumn(name = "idServicioCurso")
	public List<ServicioCursoLanguage> getGuiones() {
		return guiones;
	}

	/**
	 * Asigna los textos al servicio curso.
	 * 
	 * @param guiones
	 *          guiones
	 */
	public void setGuiones(List<ServicioCursoLanguage> guiones) {
		this.guiones = guiones;
	}

	/**
	 * Devuelve tipo de tipologías
	 * 
	 * @return tipo de tipologías
	 */
	/*@Enumerated(value = EnumType.STRING)	
	public Tipologias getTipologias() {
		return tipologias;
	}*/

	/**
	 * Asigna tipo de tipología
	 * 
	 * @param tipopologia
	 *          tipo de tipología
	 */
	/*public void setTipologias(Tipologias tipologias) {
		this.tipologias = tipologias;
	}*/
	
	@OneToMany(cascade = CascadeType.ALL)
	@JoinColumn(name = "idServicioCurso")
	public List<ServicioCursoEnlace> getEnlaces(){
		return enlaces;
	}
	
	public void setEnlaces(List<ServicioCursoEnlace> enlaces){
		this.enlaces = enlaces;
	}
	
	/**
	 * @return the examen
	 */
	@ManyToOne(cascade = {CascadeType.REFRESH})
	@JoinColumn(name = "idExamen")
	public Examen getExamen() {
		return examen;
	}

	/**
	 * @param examen the examen to set
	 */
	public void setExamen(Examen examen) {
		this.examen = examen;
	}
	
	//RF_XEA_15 set y get del examen de nivel
	/**
	 * @return the nivel exam
	 */
	@ManyToOne(cascade = {CascadeType.REFRESH})
	@JoinColumn(name = "idExamenNivel")
	public Examen getExamenNivel() {
		return examenNivel;
	}

	/**
	 * @param examen the nivel examen to set
	 */
	public void setExamenNivel(Examen examenNivel) {
		this.examenNivel = examenNivel;
	}
	//RF_XEA_15 set y get del examen de nivel

	/**
	 * @return the plantillaDiploma
	 */
	@ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
	@JoinColumn(name = "idPlantillaDiploma", nullable = false, columnDefinition="bigint(20)")
	@NotNull
	public Diploma getPlantillaDiploma() {
		return plantillaDiploma;
	}
	
	/**
	 * @param plantillaDiploma the plantillaDiploma to set
	 */
	public void setPlantillaDiploma(Diploma plantillaDiploma) {
		this.plantillaDiploma = plantillaDiploma;
	}
	
	/**
	 * @return the rolesEmisoresDiploma
	 */
	@Lob
	@Column(nullable = false, columnDefinition = "longtext character set utf8 collate utf8_general_ci")
	@NotNull
	public String getRolesEmisoresDiploma() {
		StringBuilder sb = new StringBuilder("");
		
	    for (Role rol : getRolesEmisoresDiplomaList()) {
	    	if (sb.length() != 0)
	    		sb.append(", ");
	      
	    	sb.append(rol.toString());
	    }
	    
	    return sb.toString();
	}	
	
	/**
	 * @param rolesEmisoresDiploma the rolesEmisoresDiploma to set
	 */
	public void setRolesEmisoresDiploma(String rolesEmisoresDiploma) {
		this.rolesEmisoresDiplomaList.clear();
		
	    if (rolesEmisoresDiploma != null) {
	    	StringTokenizer st = new StringTokenizer(rolesEmisoresDiploma, ",");
	      
	    	while (st.hasMoreTokens()) {
	    		try {
	    			Role role = Role.valueOf(st.nextToken().trim());
	          
	    			if (!this.rolesEmisoresDiplomaList.contains(role))
	    				this.rolesEmisoresDiplomaList.add(role);
	    		} catch (IllegalArgumentException e) {
	    			
	    		}
	    	}
	    }
	}
	
	/**
	 * @return the rolesEmisoresDiplomaList
	 */
	@Transient
	public List<Role> getRolesEmisoresDiplomaList() {
		return rolesEmisoresDiplomaList;
	}	
	
	/**
	 * @param rolesEmisoresDiplomaList the rolesEmisoresDiplomaList to set
	 */
	@Transient
	public void setRolesEmisoresDiplomaList(List<Role> rolesEmisoresDiplomaList) {
		this.rolesEmisoresDiplomaList = rolesEmisoresDiplomaList;
	}
	
	/**
	 * @return the rolesEmisoresDiplomaList in JSF format
	 */
	@Transient
	public String[] getRolesEmisoresDiplomaIface() {
		String[] rolesEmisoresDiplomaIface = new String[rolesEmisoresDiplomaList.size()];
		
		int i = 0;
		for (Role r : rolesEmisoresDiplomaList) {
			rolesEmisoresDiplomaIface[i] = r.toString();
			
			i++;
		}

		return rolesEmisoresDiplomaIface;
	}	
	
	/**
	 * @param rolesEmisoresDiplomaIface in JSF format to the rolesEmisoresDiplomaList to set
	 */
	@Transient
	public void setRolesEmisoresDiplomaIface(String[] rolesEmisoresDiplomaIface) {
		this.rolesEmisoresDiplomaList.clear();
		
		for (String sRole : rolesEmisoresDiplomaIface) {
			Role role = Role.valueOf(sRole);

			this.rolesEmisoresDiplomaList.add(role);
		}
	}

	/**
	 * @return si hay ficheros de guión asociados
	 */
	@Override
	@Transient
	public boolean getAreThereFiles() {
		boolean result = false;
		
		for(ServicioCursoLanguage scl : guiones) {
			if (scl.getFicheroGuion() != null) {
				result = true;
				
				break;
			}
		}

		return result;
	}
}
