package es.csa.fdt.cyldigital.servicios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.annotations.Cascade;
import org.hibernate.envers.Audited;
import org.hibernate.validator.Digits;

/**
 * Clase ServicioCursoOnline.
 * 
 * @author csa
 */
@Entity
@Table(name = "servicioscursosonline")
@AttributeOverride(name = "id", column = @Column(name = "idServicioCursoOnline"))
@Audited
@PrimaryKeyJoinColumn(name = "idServicio")
public class ServicioCursoOnline extends Servicio implements Serializable {

  private static final long serialVersionUID = -8361498848042755096L;

  private List<PreinscripcionCursoOnline> preinscripciones;

  private long idServicioCursoOnline;

  /**
   * Construye un nuevo objeto de la clase
   */
  public ServicioCursoOnline() {
    super();
    setEdadMaxima(100);
    setEdadMinima(1);
  }

  // ------------------------------------------------------------------------------------------------------------GETTERS
  /**
   * Devuelve id servicio curso online.
   * 
   * @return id servicio curso online
   */
  @Column(unique = true)
  @GeneratedValue(strategy = GenerationType.IDENTITY)
  @Digits(integerDigits = 20)
  public long getIdServicioCursoOnline() {
    return idServicioCursoOnline;
  }

  /**
   * Asigna id servicio curso online.
   * 
   * @param idServicioCursoOnline
   *          id servicio curso online
   */
  public void setIdServicioCursoOnline(long idServicioCursoOnline) {
    this.idServicioCursoOnline = idServicioCursoOnline;
  }

  /**
   * Devuelve preinscripciones.
   * 
   * @return preinscripciones
   */
  @OneToMany(fetch = FetchType.LAZY, cascade = { javax.persistence.CascadeType.ALL }, mappedBy = "servicioCurso")
  @Cascade(org.hibernate.annotations.CascadeType.DELETE_ORPHAN)
  @OrderBy(value = "peso ASC ,fecha ASC")
  public List<PreinscripcionCursoOnline> getPreinscripciones() {
    return preinscripciones;
  }

  /**
   * Asigna preinscripciones.
   * 
   * @param preinscripciones
   *          preinscripciones
   */
  public void setPreinscripciones(List<PreinscripcionCursoOnline> preinscripciones) {
    this.preinscripciones = preinscripciones;
  }

  /**
   * Devuelve numero preinscripciones.
   * 
   * @return numero preinscripciones
   */
  public int getNumeroPreinscripciones() {
    int resultado = 0;
    if (getPreinscripciones() != null && getPreinscripciones().size() > 0) {
      for (PreinscripcionCursoOnline p : getPreinscripciones()) {
        if (!p.getDesactivado()) /* Filtra los desactivados */
        resultado++;
      }
    }
    return resultado;
  }

  /**
   * Asigna numero preinscripciones.
   * 
   * @param preinscripciones
   *          numero preinscripciones
   */
  public void setNumeroPreinscripciones(int preinscripciones) {}

  /**
   * Devuelve actual curso online.
   * 
   * @return actual curso online
   */
  @Transient
  public CursoOnline getActualCursoOnline() {
    CursoOnline curso;

    for (Actividad a : getActividades()) {
      curso = (CursoOnline) a;
      if ((curso.isIniciado() || curso.isIniciadoHoy()) && !curso.isImpartido()) { return curso; }
    }
    return null;
  }
  
  @Override
  @Transient
  public boolean getAreThereFiles() {
	  return false;
  }
}
