package es.csa.fdt.cyldigital.servicios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.annotations.Cascade;
import org.hibernate.envers.Audited;
import org.hibernate.validator.Digits;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.core.entity.File;
import es.csa.fdt.cyldigital.core.entity.FileOnDisk;

/**
 * Clase ServicioTaller.
 * 
 * @author csa
 */
@Entity
@Table(name = "serviciostalleres")
@AttributeOverride(name = "id", column = @Column(name = "idServicioCurso"))
@Audited
@PrimaryKeyJoinColumn(name = "idServicio")
public class ServicioTaller extends Servicio implements Serializable {

  private static final long serialVersionUID = -8361498848042755096L;

  private List<PreinscripcionTaller> preinscripciones;

  private Nivel nivel;

  private String guion;

  private File guionPdf;
  private FileOnDisk ficheroGuion;

  private long idServicioTaller;

  /**
   * Construye un nuevo objeto de la clase
   */
  public ServicioTaller() {
    super();
  }

  /**
   * Devuelve id servicio taller.
   * 
   * @return id servicio taller
   */
  @Column(unique = true)
  @GeneratedValue(strategy = GenerationType.IDENTITY)
  @Digits(integerDigits = 20)
  public long getIdServicioTaller() {
    return idServicioTaller;
  }

  /**
   * Asigna id servicio taller.
   * 
   * @param idServicioTaller
   *          id servicio taller
   */
  public void setIdServicioTaller(long idServicioTaller) {
    this.idServicioTaller = idServicioTaller;
  }

  /**
   * Devuelve nivel.
   * 
   * @return nivel
   */
  @Enumerated(value = EnumType.STRING)
  @NotNull
  public Nivel getNivel() {
    return nivel;
  }

  /**
   * Asigna nivel.
   * 
   * @param nivel
   *          nivel
   */
  public void setNivel(Nivel nivel) {
    this.nivel = nivel;
  }

  /**
   * Devuelve guion.
   * 
   * @return guion
   */
  @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getGuion() {
    return guion;
  }

  /**
   * Asigna guion.
   * 
   * @param guion
   *          guion
   */
  public void setGuion(String guion) {
    this.guion = guion;
  }

  /**
   * Devuelve preinscripciones.
   * 
   * @return preinscripciones
   */
  @OneToMany(fetch = FetchType.LAZY, cascade = { javax.persistence.CascadeType.ALL }, mappedBy = "servicioTaller")
  @Cascade(org.hibernate.annotations.CascadeType.DELETE_ORPHAN)
  @OrderBy(value = "peso ASC ,fecha ASC")
  public List<PreinscripcionTaller> getPreinscripciones() {
    return preinscripciones;
  }

  /**
   * Asigna preinscripciones.
   * 
   * @param preinscripciones
   *          preinscripciones
   */
  public void setPreinscripciones(List<PreinscripcionTaller> preinscripciones) {
    this.preinscripciones = preinscripciones;
  }

  /**
   * Devuelve guion pdf.
   * 
   * @return guion pdf
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { javax.persistence.CascadeType.ALL })
  @JoinColumn(name = "idGuionPdf")
  public File getGuionPdf() {
    return guionPdf;
  }

  /**
   * Asigna guion pdf.
   * 
   * @param guionPdf
   *          guion pdf
   */
  public void setGuionPdf(File guionPdf) {
    this.guionPdf = guionPdf;
  }

  /**
   * Devuelve numero preinscripciones.
   * 
   * @return numero preinscripciones
   */
  public int getNumeroPreinscripciones() {
    int resultado = 0;

    if (getPreinscripciones() != null && getPreinscripciones().size() > 0) {
      for (PreinscripcionTaller p : getPreinscripciones())
        if (!p.getDesactivado()) resultado++;
    }
    return resultado;
  }

  /**
   * Asigna numero preinscripciones.
   * 
   * @param preinscripciones
   *          numero preinscripciones
   */
  public void setNumeroPreinscripciones(int preinscripciones) {}

  /**
   * Devuelve fichero guion.
   * 
   * @return fichero guion
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idFicheroGuion")
  public FileOnDisk getFicheroGuion() {
    return ficheroGuion;
  }

  /**
   * Asigna fichero guion.
   * 
   * @param ficheroGuion
   *          fichero guion
   */
  public void setFicheroGuion(FileOnDisk ficheroGuion) {
    this.ficheroGuion = ficheroGuion;
  }
  
  @Override
  @Transient
  public boolean getAreThereFiles() {
	  return false;
  }
}
