package es.csa.fdt.cyldigital.servicios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Map;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Digits;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;
import org.hibernate.validator.Validator;
import org.hibernate.validator.ValidatorClass;

import es.csa.fdt.cyldigital.calendario.entity.ActividadCalendarItem;
import es.csa.fdt.cyldigital.centros.entity.Aula;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.evaluacion.entity.Formulario;
import es.csa.fdt.cyldigital.evaluacion.entity.Grupo;
import es.csa.fdt.cyldigital.evaluacion.entity.Item;
import es.csa.fdt.cyldigital.evaluacion.entity.Pregunta;
import es.csa.fdt.cyldigital.evaluacion.entity.Respuesta;
import es.csa.fdt.cyldigital.recursos.entity.Recurso;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;

/**
 * Clase Taller.
 * 
 * @author csa
 */
@Entity
@Table(name = "talleres")
@AttributeOverride(name = "id", column = @Column(name = "idTaller"))
@Audited
@PrimaryKeyJoinColumn(name = "idActividad")
@es.csa.fdt.cyldigital.servicios.entity.Taller.FechasConcordantes
public class Taller extends Actividad implements Serializable {

  private static final long serialVersionUID = -8361498848042755096L;

  private static final int NUMERO_PLAZAS_LISTA_ESPERA = 0;

  private static final int NUMERO_PLAZAS_DEFECTO = 10;

  private static final int MAX_VALORACION = 10;

  private Centro centro;

  private Aula aula;

  private Empleado profesor;

  private Date fechaInicio;

  private Date fechaFin;

  private Date horaInicio;

  private Date horaFin;

  private int numeroPlazas;

  private float numeroHoras;

  private List<MatriculaTaller> matriculas;

  private String observaciones;

  private Formulario cuestionarioSatisfaccion;

  private List<PreinscripcionTaller> preinscripcionesRechazadas;

  private String apodo;

  private Integer numeroPlazasListaEspera;

  private long idTaller;

  /**
   * Construye un nuevo objeto de la clase
   */
  public Taller() {
    super();
    setNumeroPlazas(NUMERO_PLAZAS_DEFECTO);
    recursos = new ArrayList<Recurso>();
    observacionesActividad = new ArrayList<ObservacionActividad>();
    calendarItems = new ArrayList<ActividadCalendarItem>();
  }

  // ------------------------------------------------------------------------------------------------------------GETTERS

  /**
   * Devuelve id taller.
   * 
   * @return the id taller
   */
  @Column(unique = true)
  @GeneratedValue(strategy = GenerationType.IDENTITY)
  @Digits(integerDigits = 20)
  public long getIdTaller() {
    return idTaller;
  }

  /**
   * Asigna id taller.
   * 
   * @param idTaller
   *           new id taller
   */
  public void setIdTaller(long idTaller) {
    this.idTaller = idTaller;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getNombre()
   */
  @Transient
  public String getNombre() {
    return getApodo();
  }

  /**
   * Devuelve apodo.
   * 
   * @return the apodo
   */
  @Length(max = 255)
  @NotEmpty
  @NotNull
  public String getApodo() {
	/*
     * Comentado para implementar los servicios MULTILANGUAGE
     * TODO: mirar y modificar
    if ((apodo == null || apodo.isEmpty()) && getServicio() != null) return getServicio().getNombre();
    */
    return apodo;
  }

  /**
   * Asigna apodo.
   * 
   * @param apodo
   *           new apodo
   */
  public void setApodo(String apodo) {
    this.apodo = apodo;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getCentro()
   */
  @Override
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idCentro")
  @NotNull
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *           new centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Devuelve aula.
   * 
   * @return the aula
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idAula")
  public Aula getAula() {
    return aula;
  }

  /**
   * Asigna aula.
   * 
   * @param aula
   *           new aula
   */
  public void setAula(Aula aula) {
    this.aula = aula;
  }

  /**
   * Devuelve profesor.
   * 
   * @return the profesor
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idPersonal")
  public Empleado getProfesor() {
    return profesor;
  }

  /**
   * Asigna profesor.
   * 
   * @param profesor
   *           new profesor
   */
  public void setProfesor(Empleado profesor) {
    this.profesor = profesor;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getFechaInicio()
   */
  @NotNull
  @Temporal(TemporalType.DATE)
  public Date getFechaInicio() {
    return fechaInicio;
  }

  /**
   * Asigna fecha inicio.
   * 
   * @param fechaInicio
   *           new fecha inicio
   */
  public void setFechaInicio(Date fechaInicio) {
    this.fechaInicio = fechaInicio;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getFechaFin()
   */
  @NotNull
  @Temporal(TemporalType.DATE)
  public Date getFechaFin() {
    return fechaFin;
  }

  /**
   * Asigna fecha fin.
   * 
   * @param fechaFin
   *           new fecha fin
   */
  public void setFechaFin(Date fechaFin) {
    this.fechaFin = fechaFin;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getHoraInicio()
   */
  @NotNull
  @Temporal(TemporalType.TIME)
  public Date getHoraInicio() {
    return horaInicio;
  }

  /**
   * Asigna hora inicio.
   * 
   * @param horaInicio
   *           new hora inicio
   */
  public void setHoraInicio(Date horaInicio) {
    this.horaInicio = horaInicio;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getHoraFin()
   */
  @NotNull
  @Temporal(TemporalType.TIME)
  public Date getHoraFin() {
    return horaFin;
  }

  /**
   * Asigna hora fin.
   * 
   * @param horaFin
   *           new hora fin
   */
  public void setHoraFin(Date horaFin) {
    this.horaFin = horaFin;
  }

  /**
   * Devuelve numero plazas.
   * 
   * @return the numero plazas
   */
  @NotNull
  public int getNumeroPlazas() {
    return numeroPlazas;
  }

  /**
   * Asigna numero plazas.
   * 
   * @param numeroPlazas
   *           new numero plazas
   */
  public void setNumeroPlazas(int numeroPlazas) {
    this.numeroPlazas = numeroPlazas;
  }

  /**
   * Devuelve numero horas.
   * 
   * @return the numero horas
   */
  @NotNull
  @Digits(integerDigits = 20, fractionalDigits = 2)
  public float getNumeroHoras() {
    return numeroHoras;
  }

  /**
   * Asigna numero horas.
   * 
   * @param numeroHoras
   *           new numero horas
   */
  public void setNumeroHoras(float numeroHoras) {
    this.numeroHoras = numeroHoras;
  }

  /**
   * Devuelve matriculas.
   * 
   * @return the matriculas
   */
  @OneToMany(fetch = FetchType.LAZY, cascade = { javax.persistence.CascadeType.ALL }, mappedBy = "taller")
  // @Cascade(org.hibernate.annotations.CascadeType.DELETE_ORPHAN)
  @OrderBy(value = "fechaMatricula ASC")
  public List<MatriculaTaller> getMatriculas() {
    return matriculas;
  }

  /**
   * Asigna matriculas.
   * 
   * @param matriculas
   *           new matriculas
   */
  public void setMatriculas(List<MatriculaTaller> matriculas) {
    this.matriculas = matriculas;
  }

  /**
   * Devuelve observaciones.
   * 
   * @return the observaciones
   */
  @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getObservaciones() {
    return observaciones;
  }

  /**
   * Asigna observaciones.
   * 
   * @param observaciones
   *           new observaciones
   */
  public void setObservaciones(String observaciones) {
    this.observaciones = observaciones;
  }

  /**
   * Devuelve preinscripciones rechazadas.
   * 
   * @return the preinscripciones rechazadas
   */
  @ManyToMany(cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH }, mappedBy = "talleresRechazos", targetEntity = PreinscripcionTaller.class)
  public List<PreinscripcionTaller> getPreinscripcionesRechazadas() {
    return preinscripcionesRechazadas;
  }

  /**
   * Asigna preinscripciones rechazadas.
   * 
   * @param preinscripcionesRechazadas
   *           new preinscripciones rechazadas
   */
  public void setPreinscripcionesRechazadas(List<PreinscripcionTaller> preinscripcionesRechazadas) {
    this.preinscripcionesRechazadas = preinscripcionesRechazadas;
  }

  /**
   * Devuelve cuestionario satisfaccion.
   * 
   * @return the cuestionario satisfaccion
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idCuestionarioSatisfaccion")
  public Formulario getCuestionarioSatisfaccion() {
    return cuestionarioSatisfaccion;
  }

  /**
   * Asigna cuestionario satisfaccion.
   * 
   * @param cuestionarioSatisfaccion
   *           new cuestionario satisfaccion
   */
  public void setCuestionarioSatisfaccion(Formulario cuestionarioSatisfaccion) {
    this.cuestionarioSatisfaccion = cuestionarioSatisfaccion;
  }

  /**
   * Devuelve numero plazas lista espera.
   * 
   * @return the numero plazas lista espera
   */
  @NotNull
  public Integer getNumeroPlazasListaEspera() {
    return numeroPlazasListaEspera == null ? NUMERO_PLAZAS_LISTA_ESPERA : numeroPlazasListaEspera;
  }

  /**
   * Asigna numero plazas lista espera.
   * 
   * @param numeroPlazasListaEspera
   *           new numero plazas lista espera
   */
  public void setNumeroPlazasListaEspera(Integer numeroPlazasListaEspera) {
    this.numeroPlazasListaEspera = numeroPlazasListaEspera;
  }

  /**
   * Devuelve numero inscritos.
   * 
   * @return the numero inscritos
   */
  @Transient
  public int getNumeroInscritos() {
    if (getMatriculas() != null) return getMatriculas().size();
    return 0;
  }

  /**
   * Devuelve numero matriculados.
   * 
   * @return the numero matriculados
   */
  @Transient
  public int getNumeroMatriculados() {
    if (getMatriculas() != null) {
      int aux = 0;
      for (MatriculaTaller m : getMatriculas()) {
        if (!m.isEnListaEspera()) aux += 1;
      }
      return aux;
    }
    return 0;
  }

  /**
   * Devuelve numero asistentes.
   * 
   * @return the numero asistentes
   */
  @Transient
  public int getNumeroAsistentes() {
    int asistentes = 0;
    if (isCerrada()) {
      for (MatriculaTaller m : getMatriculas()) {
        if (m.getAsistencia() != null && m.getAsistencia() >= 80) asistentes++;
      }
    }
    return asistentes;
  }

  /**
   * Devuelve numero lista espera.
   * 
   * @return the numero lista espera
   */
  @Transient
  public int getNumeroListaEspera() {
    if (getMatriculas() != null) {
      int aux = 0;
      for (MatriculaTaller m : getMatriculas()) {
        if (m.isEnListaEspera()) aux += 1;
      }
      return aux;
    }
    return 0;
  }

  /**
   * Devuelve numero confirmados.
   * 
   * @return the numero confirmados
   */
  @Transient
  public int getNumeroConfirmados() {
    int i = 0;
    if (getMatriculas() != null) {
      for (MatriculaTaller m : getMatriculas()) {
        if (m.isAdmitido()) i++;
      }
      return i;
    }
    return 0;
  }

  /**
   * Devuelve numero sin confirmar.
   * 
   * @return the numero sin confirmar
   */
  @Transient
  public int getNumeroSinConfirmar() {
    if (getMatriculas() != null) return getMatriculas().size() - getNumeroConfirmados();
    return 0;
  }

  /**
   * Devuelve fechas legibles.
   * 
   * @return the fechas legibles
   */
  @Transient
  public String getFechasLegibles() {
    String resultado = "";
    DateFormat df = new SimpleDateFormat("dd/MM/yyyy");
    if (getFechaInicio() != null) {
      resultado += df.format(getFechaInicio());
    }
    if (getFechaInicio() != null && getFechaFin() != null) resultado += " - ";
    if (getFechaFin() != null) {
      resultado += df.format(getFechaFin());
    }
    return resultado;
  }

  /**
   * Devuelve horario.
   * 
   * @return the horario
   */
  @Transient
  public String getHorario() {
    String resultado = "";
    DateFormat df = new SimpleDateFormat("HH:mm");
    if (getHoraInicio() != null) resultado += df.format(getHoraInicio());
    if (getHoraInicio() != null && getHoraFin() != null) resultado += "-";
    if (getHoraFin() != null) {
      resultado += df.format(getHoraFin());
    }
    return resultado;
  }

  /**
   * Devuelve horario largo.
   * 
   * @return the horario largo
   */
  @Transient
  public String getHorarioLargo() {
    String resultado = "";
    DateFormat df = new SimpleDateFormat("HH:mm");
    if (getHoraInicio() != null) resultado += df.format(getHoraInicio());
    if (getHoraInicio() != null && getHoraFin() != null) resultado += " - ";
    if (getHoraFin() != null) {
      resultado += df.format(getHoraFin());
    }
    return resultado;
  }

  /**
   * Checks if is completo.
   * 
   * @return true, if is completo
   */
  @Transient
  public boolean isCompleto() {
    return getNumeroPlazas() - getNumeroInscritos() <= 0;
  }

  /**
   * Checks if is impartido.
   * 
   * @return true, if is impartido
   */
  @Transient
  public boolean isImpartido() {
    if (getId() > 0 && getFechaFin() != null && getFechaFin().before(new Date())) return true;
    return false;
  }

  /**
   * Interface FechasConcordantes.
   */
  @ValidatorClass(value = FechasConcordantesValidator.class)
  @Target( { ElementType.TYPE })
  @Retention(RetentionPolicy.RUNTIME)
  @Documented
  public static @interface FechasConcordantes {

    /**
     * Message.
     * 
     * @return the string
     */
    String message() default "{actividad.validator.fechaNoConcordante}";

  }

  /**
   * Clase FechasConcordantesValidator.
   */
  public static class FechasConcordantesValidator implements Validator<FechasConcordantes> {

    private static final long serialVersionUID = 985021755629904067L;

    /* (non-Javadoc)
     * @see org.hibernate.validator.Validator#initialize(java.lang.annotation.Annotation)
     */
    @Override
    public void initialize(FechasConcordantes parameters) {}

    /* (non-Javadoc)
     * @see org.hibernate.validator.Validator#isValid(java.lang.Object)
     */
    @Override
    public boolean isValid(Object value) {
      Taller taller = (Taller) value;
      if (taller.getFechaInicio().getTime() <= taller.getFechaFin().getTime()) return true;
      return false;
    }
  }

  /**
   * Devuelve valoracion taller.
   * 
   * @return the valoracion taller
   */
  @Transient
  public String getValoracionTaller() {
    float valoracion = 0;
    int n = 0;
    for (int m = 0; m < matriculas.size(); m++) {
      MatriculaTaller mTaller = matriculas.get(m);
      if (mTaller != null) {
        Map<Pregunta, Respuesta> respuestas = mTaller.getFormularioConcreto().getRespuestas();
        if (cuestionarioSatisfaccion != null && cuestionarioSatisfaccion.getItems() != null && respuestas != null
            && !respuestas.isEmpty()) {
          List<Item> items = cuestionarioSatisfaccion.getItems();
          for (int p = 0; p < items.size(); p++) {
            if (items.get(p) instanceof Grupo) {
              List<Pregunta> preguntas = ((Grupo) items.get(p)).getPreguntas();
              if (preguntas != null) {
                for (int g = 0; g < preguntas.size(); g++) {
                  Respuesta r = respuestas.get(preguntas.get(g));
                  if (r.getValorEntero() != null) {
                    valoracion += (MAX_VALORACION / preguntas.get(g).getMaximo().intValue()) * r.getValorEntero();
                    n++;
                  } else if (r.getValorDecimal() != null) {
                    valoracion += (MAX_VALORACION / preguntas.get(g).getMaximoDecimal().floatValue())
                        * r.getValorDecimal();
                    n++;
                  }
                }
              }
            } else if (items.get(p) instanceof Pregunta) {
              Respuesta r = respuestas.get(items.get(p));
              if (r.getValorEntero() != null) {
                valoracion += (MAX_VALORACION / ((Pregunta) items.get(p)).getMaximo().intValue()) * r.getValorEntero();
                n++;
              } else if (r.getValorDecimal() != null) {
                valoracion += (MAX_VALORACION / ((Pregunta) items.get(p)).getMaximoDecimal().floatValue())
                    * r.getValorDecimal();
                n++;
              }
            }
          }
        }
      }
    }
    if (n > 0) {
      valoracion = valoracion / n;
    }
    if (n == 0) {
      return "Sin valoración";
    } else {
      return valoracion + " sobre " + MAX_VALORACION;
    }
  }

  /**
   * Checks if is iniciado.
   * 
   * @return true, if is iniciado
   */
  @Transient
  public boolean isIniciado() {
    return getId() > 0 && getFechaInicio().before(new Date());
  }

  /**
   * Checks if is editado calendario.
   * 
   * @return true, if is editado calendario
   */
  @Transient
  public boolean isEditadoCalendario() {
    return !getCalendarItems().isEmpty();
  }

  /**
   * Devuelve numero dias laborables.
   * 
   * @return the numero dias laborables
   */
  @Transient
  public int getNumeroDiasLaborables() {
    int dias = 0;

    Calendar tempStart = Calendar.getInstance();
    Calendar tempEnd = Calendar.getInstance();
    Calendar tempTime = Calendar.getInstance();

    tempTime.setTime(getHoraInicio());
    tempStart.setTime(getFechaInicio());
    tempStart.set(Calendar.HOUR, tempTime.get(Calendar.HOUR_OF_DAY));
    tempStart.set(Calendar.MINUTE, tempTime.get(Calendar.MINUTE));

    tempTime.setTime(getHoraFin());
    tempEnd.setTime(getFechaFin());
    tempEnd.set(Calendar.HOUR, tempTime.get(Calendar.HOUR_OF_DAY));
    tempEnd.set(Calendar.MINUTE, tempTime.get(Calendar.MINUTE));

    while (tempStart.before(tempEnd)) {
      if (tempStart.get(Calendar.DAY_OF_WEEK) > 1 && tempStart.get(Calendar.DAY_OF_WEEK) < 7) {
        dias++;
      }
      tempStart.set(Calendar.DATE, tempStart.get(Calendar.DATE) + 1);
    }

    return dias;
  }

  @Transient
  @Override
  public boolean isExterna() {
	  return false;
  }
}
