package es.csa.fdt.cyldigital.servicios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.language.entity.TematicaLanguage;

/**
 * Clase Tematica.
 * 
 * @author it-pronet
 */
@Entity
@Table(name = "tematicas")
@AttributeOverride(name = "id", column = @Column(name = "idTematica"))
@Audited
public class Tematica extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = -7648562176655166537L;

  //private String nombre;
  
  private Tematica padre;
  
  private List<TematicaLanguage> textos;
  
  private List<Tematica> hijas;

  private List<Servicio> servicios;

  private List<Charla> charlas;
  
  private List<ActividadExterna> externas;

  /**
   * Construye un nuevo objeto de la clase
   */
  public Tematica() {
    super();
    padre = null;
    hijas = new ArrayList<Tematica>();
    servicios = new ArrayList<Servicio>();
    charlas = new ArrayList<Charla>();
    externas = new ArrayList<ActividadExterna>();
  }

  /**
   * Devuelve nombre.
   * 
   * @return nombre
   */
  /*@Length(max = 255)
  @NotEmpty
  @NotNull
  public String getNombre() {
    return nombre;
  }*/

  /**
   * Asigna nombre.
   * 
   * @param nombre
   *          nombre
   */
  /*public void setNombre(String nombre) {
    this.nombre = nombre;
  }*/
  
  /**
	 * Devuelve textos de la temática
	 * 
	 * @return textos
	 */
  	@OneToMany(cascade = CascadeType.ALL)
  	@JoinColumn(name = "idTematica")
	public List<TematicaLanguage> getTextos() {
		return textos;
	}

	/**
	 * Asigna los textos a la temática
	 * 
	 * @param textos
	 *          textos
	 */
	public void setTextos(List<TematicaLanguage> textos) {
		this.textos = textos;
	}
	
	/**
	 * Devuelve la temática padre.
	 * 
	 * @return the padre
	 */
    @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
    @JoinColumn(name = "idPadre")
	public Tematica getPadre() {
		return padre;
	}
	
	/**
	 * @param padre the padre to set
	 */
	public void setPadre(Tematica padre) {
		this.padre = padre;
	}

	/**
	 * Devuelve la las temáticas hija.
	 * 
	 * @return the hijas
	 */
	@OneToMany(mappedBy = "padre", cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
	public List<Tematica> getHijas() {
		return hijas;
	}

	/**
	 * @param hijas the hijas to set
	 */
	public void setHijas(List<Tematica> hijas) {
		this.hijas = hijas;
	}

  /**
   * Devuelve servicios.
   * 
   * @return servicios
   */
  @OneToMany(mappedBy = "tematica", cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
  public List<Servicio> getServicios() {
    return servicios;
  }

  /**
   * Asigna servicios.
   * 
   * @param servicios
   *          servicios
   */
  public void setServicios(List<Servicio> servicios) {
    this.servicios = servicios;
  }

  /**
   * Devuelve charlas.
   * 
   * @return charlas
   */
  @OneToMany(mappedBy = "tematica", cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
  public List<Charla> getCharlas() {
    return charlas;
  }

  /**
   * Asigna charlas.
   * 
   * @param charlas
   *          charlas
   */
  public void setCharlas(List<Charla> charlas) {
    this.charlas = charlas;
  }
  
  /**
   * Devuelve externas.
   * 
   * @return externas
   */
  @OneToMany(mappedBy = "tematica", cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
  public List<ActividadExterna> getExternas() {
    return externas;
  }

  /**
   * Asigna externas.
   * 
   * @param externas
   *          externas
   */
  public void setExternas(List<ActividadExterna> externas) {
    this.externas = externas;
  }
  
  @Transient
  public boolean isWithChildsActivated() {
	  if ((getId() != 0) && (padre == null) && (hijas.size() > 0)) {
		  for (Tematica hija : hijas) {
			  if (!hija.getDesactivado())
				  return true;
		  }
	  }
	  
	  return false;
  }
  
  @Transient
  public boolean isWithChildsDeactivated() {
	  if ((getId() != 0) && (padre == null) && (hijas.size() > 0)) {
		  for (Tematica hija : hijas) {
			  if (hija.getDesactivado())
				  return true;
		  }
	  }
	  
	  return false;
  }
  
  @Transient
  public boolean isBorrable() {
	  if ((getId() != 0) /* && !getDesactivado() */) {
		  if (padre != null) {
			  if (((servicios == null) || servicios.isEmpty()) &&
				  ((charlas == null) || charlas.isEmpty()) &&
				  ((externas == null) || externas.isEmpty()))
				  return true;
			  else
				  return false;
		  }  
		  else {
			  for (Tematica th : hijas){
				  if(!th.isBorrable())
					  return false;
			  }
			  
			  return true;
		  }
	  }
	  else
		  return false;
  }
  
  @Transient
  public boolean isDesactivable() {
	  if ((getId() != 0) && !getDesactivado()) {
		  if (padre != null) {
			  // if ((servicios == null) || servicios.isEmpty())
				  return true;
			  //else
			  //  return false;
		  }  
		  else {
			  /*
			  for (Tematica th : hijas){
				  if(!th.isDesactivable())
					  return false;
			  }
			  */
			  return true;
		  }
	  }
	  else
		  return false;
  }
}
