package es.csa.fdt.cyldigital.servicios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;

/**
 * Clase TipoAsesoramiento.
 * 
 * @author csa
 */
@Entity
@Table(name = "tiposasesoramientos")
@AttributeOverride(name = "id", column = @Column(name = "idTipoAsesoramiento"))
@Audited
public class TipoAsesoramiento extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = -7648562176655166537L;

  private String nombre;

  private List<Asesoramiento> asesoramientos;

  /**
   * Construye un nuevo objeto de la clase
   */
  public TipoAsesoramiento() {
    super();
    asesoramientos = new ArrayList<Asesoramiento>();
  }

  /**
   * Devuelve nombre.
   * 
   * @return nombre
   */
  @Length(max = 255)
  @NotEmpty
  @NotNull
  public String getNombre() {
    return nombre;
  }

  /**
   * Asigna nombre.
   * 
   * @param nombre
   *          nombre
   */
  public void setNombre(String nombre) {
    this.nombre = nombre;
  }

  /**
   * Devuelve asesoramientos.
   * 
   * @return asesoramientos
   */
  @OneToMany(mappedBy = "tipoAsesoramiento", cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH })
  public List<Asesoramiento> getAsesoramientos() {
    return asesoramientos;
  }

  /**
   * Asigna asesoramientos.
   * 
   * @param asesoramientos
   *          asesoramientos
   */
  public void setAsesoramientos(List<Asesoramiento> asesoramientos) {
    this.asesoramientos = asesoramientos;
  }

}
