package es.csa.fdt.cyldigital.servicios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Digits;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.calendario.entity.ActividadCalendarItem;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;

/**
 * Clase Wifi.
 * 
 * @author csa
 */
@Entity
@Table(name = "wifis")
//@AttributeOverride(name = "id", column = @Column(name = "idWifi"))
@Audited
@PrimaryKeyJoinColumn(name = "idActividad")
public class Wifi extends Actividad implements Serializable {

  private static final long serialVersionUID = -8361498848042755096L;

  private UsuarioCyL usuario;

  private Centro centro;

  private String observaciones;

  private Date fecha;

  //private long idWifi;

  /**
   * Construye un nuevo objeto de la clase
   */
  public Wifi() {
    super();
    calendarItems = new ArrayList<ActividadCalendarItem>();
  }

  // ------------------------------------------------------------------------------------------------------------GETTERS

  /**
   * Devuelve id wifi.
   * 
   * @return id wifi 
   */
  /*@Column(unique = true)
  @GeneratedValue(strategy = GenerationType.IDENTITY)
  @Digits(integerDigits = 20)
  public long getIdWifi() {
    return idWifi;
  }*/

  /**
   * Asigna id wifi.
   * 
   * @param idWifi
   *          id wifi
   */
  /*public void setIdWifi(long idWifi) {
    this.idWifi = idWifi;
  }*/

  /**
   * Devuelve usuario.
   * 
   * @return usuario
   */
  @NotNull
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idUsuario")
  public UsuarioCyL getUsuario() {
    return usuario;
  }

  /**
   * Asigna usuario.
   * 
   * @param usuario
   *          usuario
   */
  public void setUsuario(UsuarioCyL usuario) {
    this.usuario = usuario;
  }

  /**
   * Devuelve fecha.
   * 
   * @return fecha
   */
  @NotNull
  @Temporal(TemporalType.TIMESTAMP)
  public Date getFecha() {
    return fecha;
  }

  /**
   * Asigna fecha.
   * 
   * @param fecha
   *          fecha
   */
  public void setFecha(Date fecha) {
    this.fecha = fecha;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getCentro()
   */
  @Override
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idCentro")
  @NotNull
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Devuelve observaciones.
   * 
   * @return observaciones
   */
  @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getObservaciones() {
    return observaciones;
  }

  /**
   * Asigna observaciones.
   * 
   * @param observaciones
   *          observaciones
   */
  public void setObservaciones(String observaciones) {
    this.observaciones = observaciones;
  }

  /**
   * Devuelve fecha legible.
   * 
   * @return fecha legible
   */
  @Transient
  public String getFechaLegible() {
    String resultado = "";
    DateFormat df = new SimpleDateFormat("dd/MM/yyyy");
    if (getFecha() != null) {
      resultado += df.format(getFecha());
    }
    return resultado;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getNombre()
   */
  /*@Override
  @Transient
  public String getNombre() {
    return null;
  }*/

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getFechaFin()
   */
  @Override
  @Transient
  public Date getFechaFin() {
    return null;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getFechaInicio()
   */
  @Override
  @Transient
  public Date getFechaInicio() {
    return null;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getHoraFin()
   */
  @Override
  @Transient
  public Date getHoraFin() {
    return null;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.entity.Actividad#getHoraInicio()
   */
  @Override
  @Transient
  public Date getHoraInicio() {
    return null;
  }

  @Transient
  @Override
  public boolean isExterna() {
	  return false;
  }
}
