package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.List;

import es.csa.fdt.cyldigital.calendario.service.ActividadCalendarItemServiceImpl;
import es.csa.fdt.cyldigital.centros.entity.Aula;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.service.AbstractService;
import es.csa.fdt.cyldigital.servicios.entity.ActividadExterna;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaExterna;
import es.csa.fdt.cyldigital.servicios.entity.ServicioCurso;
import es.csa.fdt.cyldigital.servicios.entity.ServicioTaller;
import es.csa.fdt.cyldigital.servicios.entity.ActividadExterna.EEstadoActividadExterna;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;
import es.csa.fdt.cyldigital.util.ActivitySchedule;

/**
 * Interfaz del servicio de ActividadExterna.
 * 
 * @author csa
 */
public interface ActividadExternaService extends AbstractService<ActividadExterna, Long> {

  /**
   * Persist actividad externa.
   * 
   * @param u
   * 			Usuario que da de alta o solicita la actividad
   * 
   * @param oldEstado
   * 			Estado anterior de la actividad
   * 
   * @param actividadExterna
   *           actividad externa
   *           
   * @param schedules
   * 			planificaciones
   *           
   * @param serviceCal
   *           service cal
   *           
   * @return true, if successful
   */
  public boolean persistActividadExterna(UsuarioLdap u, EEstadoActividadExterna oldEstado, ActividadExterna actividadExterna,
		  								 List<ActivitySchedule> schedules, ActividadCalendarItemServiceImpl serviceCal);
  
  
  /**
   * Persist actividad externa múltiple
   * 
   * @param aeMultiple
   * 			Actividad externa a convertir en múltiple
   * 
   * @param schedules
   * 			planificaciones
   * 
   * @param serviceCal
   * 			Servicio de calendario
   * 
   * @return lista de actividades externas guardadas
   */
  public List<ActividadExterna> persistActividadExternaMultiple(ActividadExterna aeMultiple, List<ActivitySchedule> schedules,
		  														ActividadCalendarItemServiceImpl serviceCal);

  /**
   * Elimina actividad externa.
   * 
   * @param actividadExterna
   *           actividad externa
   */
  public void removeActividadExterna(ActividadExterna actividadExterna);

  /**
   * Devuelve servicios cursos disponibles.
   * 
   * @return servicios cursos disponibles
   */
  public List<ServicioCurso> getServiciosCursosDisponibles();

  /**
   * Devuelve servicios talleres disponibles.
   * 
   * @return servicios talleres disponibles
   */
  public List<ServicioTaller> getServiciosTalleresDisponibles();

  /**
   * Devuelve centros.
   * 
   * @return centros
   */
  public List<Centro> getCentros();

  /**
   * Devuelve colaboradores.
   * 
   * @return colaboradores
   */
  public List<Empleado> getColaboradores();

  /**
   * Devuelve aulas.
   * 
   * @return aulas
   */
  public List<Aula> getAulas();

  /**
   * Comprobar edad usuario.
   * 
   * @param m
   *           m
   */
  public void comprobarEdadUsuario(MatriculaExterna m);
}
