package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.faces.event.ActionEvent;
import javax.faces.model.SelectItem;

import org.richfaces.event.UploadEvent;
import org.richfaces.model.UploadItem;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.faces.util.FicheroSubidoRichFacesAdapter;
import es.csa.fdt.cyldigital.calendario.entity.ActividadCalendarItem;
import es.csa.fdt.cyldigital.calendario.service.ActividadCalendarItemServiceImpl;
import es.csa.fdt.cyldigital.centros.entity.Aula;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.entity.FileOnDisk;
import es.csa.fdt.cyldigital.core.entity.FileOnDisk.Tipo;
import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.evaluacion.entity.Formulario;
import es.csa.fdt.cyldigital.evaluacion.entity.FormularioConcreto;
import es.csa.fdt.cyldigital.language.entity.ActividadExternaLanguage;
import es.csa.fdt.cyldigital.language.entity.ActividadLanguage;
import es.csa.fdt.cyldigital.language.entity.FormularioLanguage;
import es.csa.fdt.cyldigital.language.entity.Language;
import es.csa.fdt.cyldigital.language.entity.ServicioCursoLanguage;
import es.csa.fdt.cyldigital.notificacion.NotificationService;
import es.csa.fdt.cyldigital.recursos.entity.Recurso;
import es.csa.fdt.cyldigital.servicios.entity.Actividad;
import es.csa.fdt.cyldigital.servicios.entity.ActividadExterna;
import es.csa.fdt.cyldigital.servicios.entity.ActividadExterna.EEstadoActividadExterna;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaExterna;
import es.csa.fdt.cyldigital.servicios.entity.ObservacionActividad;
import es.csa.fdt.cyldigital.servicios.entity.ServicioCurso;
import es.csa.fdt.cyldigital.servicios.entity.ServicioTaller;
import es.csa.fdt.cyldigital.servicios.entity.Taller;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.Proveedor;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario.Role;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;
import es.csa.fdt.cyldigital.util.ActivitySchedule;
import es.csa.fdt.cyldigital.util.SelectFilterByCenterAux;

/**
 * Clase ActividadExternaServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class ActividadExternaServiceImpl extends AbstractServiceImpl<ActividadExterna, Long> implements ActividadExternaService {

  //private String tipoActividad;
  // RF_15_31 Selección solicitante actividades externas
  private ActividadExterna externa;
	
  private List<Centro> centros;
  
  private Centro centro;

  private List<Empleado> colaboradores;
  
  //RF_15_31 Selección solicitante actividades externas
  private Proveedor proveedor;
  
  private List<Empleado> colaboradoresEntidad;
  //RF_15_31 Selección solicitante actividades externas

  private List<Aula> aulas;

  private static final long serialVersionUID = -5575228180361917674L;

  private boolean avisoFechaMostrado = false;

  private boolean avisoRecursos = false;

  //private boolean borrarPreinscripcion = false;
  
  private Map<String, FileOnDisk> filesUploaded;
  private String fileToRemove;

  private List<FormularioLanguage> formulariosLanguage;
  private List<Formulario> formularios;

  @Resource
  private transient HashMap<String, Object> generalProperties;
  
  public class AETeaching implements Serializable {
	  private static final long serialVersionUID = 7474925346900079661L;
	
	  private List<SelectItem> colaboradores;
	  private Empleado colaborador;
	  private List<SelectItem> centros;
	  private Centro centro;
	  private List<SelectItem> aulas;
	  private Aula aula;
	  // RF_15_31 Selección solicitante actividades externas
	  private Proveedor proveedor;
	  private List<SelectItem> colaboradoresEntidad;
	  // RF_15_31 Selección solicitante actividades externas
	  
	  public AETeaching() {
		  colaboradores = new ArrayList<SelectItem>();
		  centros = new ArrayList<SelectItem>();
		  aulas = new ArrayList<SelectItem>();
		  colaboradoresEntidad = new ArrayList<SelectItem>();
	  }
	  
	  //Colaboradores
	  public List<SelectItem> getColaboradores() {
		  colaboradores.clear();
		  
		  for (Empleado e : ActividadExternaServiceImpl.this.getColaboradores())
			  if (e.equals(getColaborador()) || !ActividadExternaServiceImpl.this.getSelectedColaboradores().contains(e))
				  colaboradores.add(new SelectItem(e, e.getNombreCompleto()));
		  
		  return colaboradores;
	  }
	  
	  //Colaborador
	  public Empleado getColaborador() {
		  return colaborador;
	  }
	  
	  public void setColaborador(Empleado colaborador) {
		  this.colaborador = colaborador;
	  }
	  
	  //Centros
	  public List<SelectItem> getCentros() {
		  centros.clear();

		  for (Centro c : ActividadExternaServiceImpl.this.getCentros())
			  if (c.equals(getCentro()) || !ActividadExternaServiceImpl.this.getSelectedCentros().contains(c))
				  centros.add(new SelectItem(c, c.getNombre()));
		  
		  return centros;
	  }
	  
	  //Centro
	  public Centro getCentro() {
		  return centro;
	  }
	  
	  public void setCentro(Centro centro) {
		  Long oldId = -1L;
		  if (this.centro != null) oldId = this.centro.getId();
		  
		  this.centro = centro;
		  
		  if (this.centro == null)
			  aulas.clear();
		  else if ((this.centro.getId() != oldId)) {
			  aulas.clear();
		  
			  for (Aula a : this.centro.getAulas())
				  aulas.add(new SelectItem(a, a.getNombre()));
		  }
	  }
	  
	  //Aulas
	  public List<SelectItem> getAulas() {
		  return aulas;
	  }

	  //Aula
	  public Aula getAula() {
		  return aula;
	  }
	  
	  public void setAula(Aula aula) {
		  this.aula = aula;
	  }
	  
	  // Proveedor
	  public Proveedor getProveedor() {
		  return proveedor;
	  }
	  
	  public void setProveedor(Proveedor proveedor) {
		  Long oldId = -1L;
		  if (this.proveedor != null) oldId = this.proveedor.getId();
		  
		  this.proveedor = proveedor;
		  ActividadExternaServiceImpl.this.proveedor = proveedor;
		  
		  if (this.proveedor == null)
			  colaboradoresEntidad.clear();
		  else if ((this.proveedor.getId() != oldId)) {
			  colaboradoresEntidad.clear();
			  
			  for (Empleado e : this.proveedor.getEmpleados())
				  colaboradoresEntidad.add(new SelectItem(e, e.getNombreCompleto()));
		  }
	  }
	  
	  //Proveedores
	  public List<SelectItem> getColaboradoresEntidad() {
		  return colaboradoresEntidad;
	  }
	  
  }
  
  private List<AETeaching> aeTeachings;
  
  private List<Empleado> selectedColaboradores;
  private List<Centro> selectedCentros;

  /**
   * Devuelve actividad.
   * 
   * @return actividad
   */
  public ActividadExterna getActividad() {
    return externa;
  }

  /**
   * Asigna actividad.
   * 
   * @param actividad
   *          actividad
   */
  public void setActividad(ActividadExterna externa) {
    this.externa = externa;
  }
  
  /**
   * Devuelve centro.
   * 
   * @return centro
   */
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Devuelve tipo actividad.
   * 
   * @return tipoActividad
   */
  /*public String getTipoActividad() {
		return tipoActividad;
	}*/

  /**
   * Asigna tipo actividad.
   * 
   * @param tipoActividad
   *          tipo actividad
   */
  /*public void setTipoActividad(String tipoActividad) {
		this.tipoActividad = tipoActividad;
	}*/

  /**
   * Apply centro.
   * 
   * @param externa
   *           externa
   */
  public void applyCentro(ActividadExterna externa) {
	  this.centro = externa.getCentro();
	  externa.setAula(null);
  }
  
  /**
   * Devuelve proveedor.
   * 
   * @return proveedor
   */
  public Proveedor getProveedor() {
    return proveedor;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setProveedor(Proveedor proveedor) {
    this.proveedor = proveedor;
  }
  
  /**
   * Construye un nuevo objeto de la clase
   */
  public ActividadExternaServiceImpl() {
	  super();
	  
	  this.filesUploaded = new HashMap<String, FileOnDisk>();
	  
	  this.selectedColaboradores = new ArrayList<Empleado>();
	  this.selectedCentros = new ArrayList<Centro>();
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.ActividadExterna#setEstadoInicial(es.csa.fdt.cyldigital.servicios.entity.ActividadExterna, java.lang.Long, java.lang.Boolean, java.lang.Boolean)
   */
  public void setEstadoInicial(ActividadExterna externa, Long idExterna, Boolean nueva, Boolean request) {
	  if (nueva && (idExterna == 0)) {
		  if (request)
			  externa.setEstado(EEstadoActividadExterna.SOLICITADA);
		  else
			  externa.setEstado(EEstadoActividadExterna.ACEPTADA);
	  }
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.ActividadExterna#createLanguages(es.csa.fdt.cyldigital.servicios.entity.ActividadExterna, List<es.csa.fdt.cyldigital.language.entity.Language> languages)
   */
  public void createLanguages(ActividadExterna externa, List<Language> languages) {
	  List<ActividadLanguage> textos = new ArrayList<ActividadLanguage>();
	  List<ActividadExternaLanguage> textosExterna = new ArrayList<ActividadExternaLanguage>();

	  for (Language language : languages) {
		  ActividadExternaLanguage textoExterna = new ActividadExternaLanguage();
		  textoExterna.setLanguage(language);
		  textoExterna.setExterna(externa);

		  textos.add((ActividadLanguage)textoExterna);
		  textosExterna.add(textoExterna);
	  }

	  externa.setTextosExterna(textosExterna);
	  externa.setTextos(textos);
  }
  
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.ActividadExternaService#persistActividadExterna(es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap, es.csa.fdt.cyldigital.servicios.entity.ActividadExterna.EEstadoActividadExterna, es.csa.fdt.cyldigital.servicios.entity.ActividadExterna, java.util.List, es.csa.fdt.cyldigital.calendario.service.ActividadCalendarItemServiceImpl)
   */
  /**public boolean persistActividadExterna(UsuarioLdap u, EEstadoActividadExterna oldEstado, ActividadExterna actividadExterna,
		  								 List<ActivitySchedule> schedules, ActividadCalendarItemServiceImpl serviceCal) {
	  boolean error = false;
	  
	  if (actividadExterna.getId() <= 0) {
		  for (ActivitySchedule as : schedules) {
			  if (as.getStartDate().getTime() > as.getEndDate().getTime()) {
				  addErrorMessage("actividad.validator.fechaInicioMayorFechaFin");
		
				  error = true;
					
				  break;
			  }
		
			  if (as.getStartTime().getTime() >= as.getEndTime().getTime()) {
				  addErrorMessage("actividad.validator.horaFinMenorOIgualHoraInicio");
		
				  error = true;
					
				  break;
			  }
		  }
		  
		  Long durationInDays = Long.valueOf(serviceCal.schedulesDurationInDays(schedules));
			
		  if (durationInDays <= 0L) {
			  addErrorMessage("actividad.validator.schedulesNotProducesDays");
				
			  error = true;
		  }
		  
		  if (serviceCal.hasSchedulesCollision(schedules)) {
			  addErrorMessage("actividad.validator.schedulesCollision");

			  error = true;
		  }
	  }
	  else { //if (actividadExterna.getId() > 0)
		  if (actividadExterna.getFechaInicio().getTime() > actividadExterna.getFechaFin().getTime()) {
			  addErrorMessage("actividad.validator.fechaInicioMayorFechaFin");
			  
			  error = true;
		  }
		  
		  if (actividadExterna.getHoraInicio().getTime() >= actividadExterna.getHoraFin().getTime()) {
			  addErrorMessage("actividad.validator.horaFinMenorOIgualHoraInicio");
			  
			  error = true;
		  }
	  }

	  if (!error) {
		  boolean warning = false;
			
		  if ((actividadExterna.getId() <= 0) && !avisoFechaMostrado && !actividadExterna.isCerrada()) {
			  for (ActivitySchedule as : schedules) {
				  if (as.getStartDate().getTime() < (new Date()).getTime()) {
					  warning = true;

					  break;
				  }
			  }
		  }
		  
		  if ((((actividadExterna.getId() > 0)  && (actividadExterna.getFechaInicio().getTime() < (new Date()).getTime())) ||
			   ((actividadExterna.getId() <= 0) && warning)) &&
			  !avisoFechaMostrado &&
			  !actividadExterna.isCerrada()) {
			  addWarnMessage("actividad.validator.fechaInicioAnterior");
			  
			  avisoFechaMostrado = true;
		  }
		  else if (((actividadExterna.getCentro() != null) && (actividadExterna.getAula() != null)) &&
				   (actividadExterna.getNumeroPlazas() > actividadExterna.getRecursos().size()) &&
				   !avisoRecursos) {
			  addWarnMessage("actividad.validator.plazasMayorRecursos");
			  
			  avisoRecursos = true;
		  } else {
			  Parameter p = findOneWhere(Parameter.class, "nombre", "pathfileondisk");
	
			  for (ActividadExternaLanguage ael : actividadExterna.getTextosExterna())
			  {
				  String fileKey = ael.getLanguage().getISO();
				  
				  if (filesUploaded.containsKey(fileKey))
				  {
					  ael.setFicheroActExt(filesUploaded.get(fileKey));
					  filesUploaded.get(fileKey).grabarEnDisco(p.getValor());
					  filesUploaded.get(fileKey).setTipo(Tipo.GUION);
					  
					  disableMessages();
					  
					  persist(filesUploaded.get(fileKey));
					  
					  enableMessages();
				  }
		
				  if (ael.getDescripcion() != null)
					  ael.setDescripcion(ael.getDescripcion().replaceAll("\n", "<br />"));
			  }

			  if (u.getRolesList().contains(Usuario.Role.ROLE_EXTERN) &&
				  (actividadExterna.getEstado() == EEstadoActividadExterna.RECHAZADA))
				  actividadExterna.setEstado(EEstadoActividadExterna.SOLICITADA);

			  if (actividadExterna.isCerrada()) {
				  Date horaFinEncuesta = new Date(System.currentTimeMillis() +
						  						  (Integer.parseInt(findOneWhere(Parameter.class, "nombre", "anonymous_survey_time").getValor()) * 60000L));
	
				  actividadExterna.setFinEncuesta(horaFinEncuesta);
			  }
			  
			  if ((actividadExterna.getId() > 0) && actividadExterna.getCalendarItems().isEmpty())
					serviceCal.generateCalendar(actividadExterna.getId());

			  persist(actividadExterna);
			  
			  System.out.println("actividadExterna: " + actividadExterna.getFechasLegibles());
			  System.out.println("actividadExterna: " + actividadExterna.getFechaInicio());
			  System.out.println("actividadExterna: " + actividadExterna.getFechaFin());
			  
			  if (oldEstado != actividadExterna.getEstado())
				  notificarEstadoSolicitudActExterna(actividadExterna);

			  return true;
		  }
	  }
	
	  return false;
  }**/
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.ActividadExternaService#persistActividadExterna(es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap, es.csa.fdt.cyldigital.servicios.entity.ActividadExterna.EEstadoActividadExterna, es.csa.fdt.cyldigital.servicios.entity.ActividadExterna, java.util.List, es.csa.fdt.cyldigital.calendario.service.ActividadCalendarItemServiceImpl)
   */
  public boolean persistActividadExterna(UsuarioLdap u, EEstadoActividadExterna oldEstado, ActividadExterna ae, List<ActivitySchedule> schedules, ActividadCalendarItemServiceImpl serviceCal) {
		Boolean error = false;
		
		if (ae.getId() <= 0) {
			for (ActivitySchedule as : schedules) {
				if (as.getStartDate().getTime() > as.getEndDate().getTime()) {
					addErrorMessage("actividad.validator.fechaInicioMayorFechaFin");
		
					error = true;
					
					break;
				}
		
				if (as.getStartTime().getTime() >= as.getEndTime().getTime()) {
					addErrorMessage("actividad.validator.horaFinMenorOIgualHoraInicio");
		
					error = true;
					
					break;
				}
			}
			
			Long durationInDays = Long.valueOf(serviceCal.schedulesDurationInDays(schedules));
			
			if (durationInDays <= 0L) {
				addErrorMessage("actividad.validator.schedulesNotProducesDays");
				
				error = true;
			}
			/* RF_15_06 Se elimina la limitación de mas de 1 día 
			else if (durationInDays == 1L) {
				addErrorMessage("actividad.validator.fechaInicioIgualFechaFin");
				
				error = true;
			}
			*/
			

			if (serviceCal.hasSchedulesCollision(schedules)) {
				addErrorMessage("actividad.validator.schedulesCollision");

				error = true;
			}
		} else { //if (curso.getId() > 0)
			/* RF_15_06 Se elimina la limitación de mas de 1 día
			if (curso.getFechaInicio().getTime() == curso.getFechaFin().getTime()) {
				addErrorMessage("actividad.validator.fechaInicioIgualFechaFin");
	
				error = true;
			}
			*/
	
			if (ae.getFechaInicio().getTime() > ae.getFechaFin().getTime()) {
				addErrorMessage("actividad.validator.fechaInicioMayorFechaFin");
	
				error = true;
			}
	
			if (ae.getHoraInicio().getTime() >= ae.getHoraFin().getTime()) {
				addErrorMessage("actividad.validator.horaFinMenorOIgualHoraInicio");
	
				error = true;
			}
		}
		
		if (!error) {
			boolean warning = false;
			
			if ((ae.getId() <= 0) && !avisoFechaMostrado && !ae.isCerrada()) {
				for (ActivitySchedule as : schedules) {
					if (as.getStartDate().getTime() < (new Date()).getTime()) {
						warning = true;

						break;
					}
				}
			}

			if ((((ae.getId() > 0)  && (ae.getFechaInicio().getTime() < (new Date()).getTime())) ||
				 ((ae.getId() <= 0) && warning)) &&
				!avisoFechaMostrado &&
				!ae.isCerrada()) {
				addWarnMessage("actividad.validator.fechaInicioAnterior");

				avisoFechaMostrado = true;
			} 
			/* se anula la verificacion del numero de recursos
			else if (!avisoRecursos && (ae.getNumeroPlazas() > ae.getRecursos().size())) {
				addWarnMessage("actividad.validator.plazasMayorRecursos");

				avisoRecursos = true;
			} 
			*/
			else {
				
				if (u.getRolesList().contains(Usuario.Role.ROLE_EXTERN) &&
						(ae.getEstado() == EEstadoActividadExterna.RECHAZADA))
					ae.setEstado(EEstadoActividadExterna.SOLICITADA);

				if (ae.isCerrada()) {
					Date horaFinEncuesta = new Date(System.currentTimeMillis() +
										  (Integer.parseInt(findOneWhere(Parameter.class, "nombre", "anonymous_survey_time").getValor()) * 60000L));
			
					ae.setFinEncuesta(horaFinEncuesta);
				}
				
				if ((ae.getId() > 0) && ae.getCalendarItems().isEmpty())
					serviceCal.generateCalendar(ae.getId());

				persist(ae);
				
				return true;
			}
		}

		return false;
	}
  
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.ActividadExternaService#persistActividadExternaMultiple(es.csa.fdt.cyldigital.servicios.entity.ActividadExterna, java.util.List, es.csa.fdt.cyldigital.calendario.service.ActividadCalendarItemServiceImpl)
   */
  public List<ActividadExterna> persistActividadExternaMultiple(ActividadExterna aeMultiple, List<ActivitySchedule> schedules,
		  														ActividadCalendarItemServiceImpl serviceCal) {
	  boolean error = false;
	  
	  for (ActivitySchedule as : schedules) {
		  if (as.getStartDate().getTime() > as.getEndDate().getTime()) {
			  addErrorMessage("actividad.validator.fechaInicioMayorFechaFin");
	
			  error = true;
				
			  break;
		  }
	
		  if (as.getStartTime().getTime() >= as.getEndTime().getTime()) {
			  addErrorMessage("actividad.validator.horaFinMenorOIgualHoraInicio");
	
			  error = true;
				
			  break;
		  }
	  }
	  
	  Long durationInDays = Long.valueOf(serviceCal.schedulesDurationInDays(schedules));
		
	  if (durationInDays <= 0L) {
		  addErrorMessage("actividad.validator.schedulesNotProducesDays");
		
		  error = true;
	  }
	  
	  if (serviceCal.hasSchedulesCollision(schedules)) {
		  addErrorMessage("actividad.validator.schedulesCollision");

		  error = true;
	  }

	  if (!error) {
		  boolean warning = false;
			
		  if ((aeMultiple.getId() <= 0) && !avisoFechaMostrado && !aeMultiple.isCerrada()) {
			  for (ActivitySchedule as : schedules) {
				  if (as.getStartDate().getTime() < (new Date()).getTime()) {
					  warning = true;

					  break;
				  }
			  }
		  }
			
		  if (((aeMultiple.getId() <= 0) && warning) &&
			  !avisoFechaMostrado &&
			  !aeMultiple.isCerrada()) {
			  addWarnMessage("actividad.validator.fechaInicioAnterior");

			  avisoFechaMostrado = true;
		  }
		  else {
			  Parameter p = findOneWhere(Parameter.class, "nombre", "pathfileondisk");
			  
			  List<ActividadExterna> listaAEs = new ArrayList<ActividadExterna>();
			  
			  for (AETeaching aet : aeTeachings) {
				  ActividadExterna ae = new ActividadExterna();

				  ae.setEstado(aeMultiple.getEstado());
				  ae.setColaborador(aet.getColaborador());
				  ae.setCentro(aet.getCentro());
				  ae.setAula(aet.getAula());
				  ae.setServicio(aeMultiple.getServicio());
				  ae.setTematica(aeMultiple.getTematica());
				  ae.setCuestionarioSatisfaccion(aeMultiple.getCuestionarioSatisfaccion());
				  ae.setVideoconferencia(aeMultiple.isVideoconferencia());
				  ae.setNumeroPlazas(aeMultiple.getNumeroPlazas());
				  ae.setNumeroAsistentes(aeMultiple.getNumeroAsistentes());
				  ae.setObservaciones(aeMultiple.getObservaciones());
				  
				  //RF_XEA_22
				  ae.setFechaPublicacion(aeMultiple.getFechaPublicacion());
				  ae.setHoraPublicacion(aeMultiple.getHoraPublicacion());
				  //RF_XEA_22

				  //RF_15_04
				  ae.setFechaLimiteInscripcion(aeMultiple.getFechaLimiteInscripcion());
				  ae.setHoraLimiteInscripcion(aeMultiple.getHoraLimiteInscripcion());
				  //RF_15_04
				  
				  ae.setTextos(new ArrayList<ActividadLanguage>());
				  ae.setTextosExterna(new ArrayList<ActividadExternaLanguage>());
	
				  for (ActividadExternaLanguage ael : aeMultiple.getTextosExterna()) {
					  ActividadExternaLanguage textoExterna = new ActividadExternaLanguage();
	
					  textoExterna.setExterna(ae);
					  textoExterna.setLanguage(ael.getLanguage());
					  textoExterna.setApodo(ael.getApodo());
					  if (ael.getDescripcion() != null)
						  textoExterna.setDescripcion(ael.getDescripcion().replaceAll("\n", "<br />"));
					  
					  String fileKey = ael.getLanguage().getISO();
					  if (filesUploaded.containsKey(fileKey)) {
						  FileOnDisk fod = new FileOnDisk();
						  UploadItem uploadItem = ((FicheroSubidoRichFacesAdapter)filesUploaded.get(fileKey).getFicheroSubido()).getUploadItem();
						  
						  fod.setFicheroSubido(new FicheroSubidoRichFacesAdapter(uploadItem));
						  fod.setTipo(Tipo.GUION);
						  fod.grabarEnDisco(p.getValor());
						  
						  textoExterna.setFicheroActExt(fod);
						  
						  disableMessages();
						  persist(fod);
						  enableMessages();
					  }

					  ae.getTextos().add((ActividadLanguage)textoExterna);
					  ae.getTextosExterna().add(textoExterna);
				  }

				  persist(ae);
				  
				  listaAEs.add(ae);
			  }
			  
			  return listaAEs;
		  }
	  }
	  
	  return null;
  }

  /**
   * Persist externa to calendario.
   * 
   * @param externa
   *           externa
   * @return true, if successful
   */
  public boolean persistExternaToCalendario(ActividadExterna externa) {
    /*if (externa.getNumeroHoras() == 0) {
      externa.setNumeroHoras(2);
    }*/

    if ((externa.getFechaInicio().getTime() < (new Date()).getTime()) && !avisoFechaMostrado && !externa.isCerrada()) {
    	addWarnMessage("actividad.validator.fechaInicioAnterior");
    	avisoFechaMostrado = true;
    } else {
    	setMessagesEnabled(false);
    	persist(externa);
    	setMessagesEnabled(true);

    	return true;
    }

    return false;
  }
  
  public void removeActividadExterna(Long idExterna, ActividadCalendarItemServiceImpl serviceCal) {
	  ActividadExterna externa = find(idExterna);
	  
	  if ((externa != null) && (!externa.getCalendarItems().isEmpty())) {
  		  serviceCal.disableMessages();
		  serviceCal.removeAll(externa.getCalendarItems());
		  serviceCal.enableMessages();
		  
		  externa.getCalendarItems().clear();
	  }
	  
	  disableMessages();
	  
	  for (ObservacionActividad oa : externa.getObservacionesActividad()) {
		  remove(oa);
	  }
	  
	  externa.getObservacionesActividad().clear();
	  
	  enableMessages();
	  
	  remove(externa);
	  
	  notificarCancelacionSolActExt(externa);
  }
  
  public void persistEstadoExterna(Long idExterna, boolean aceptada, ActividadCalendarItemServiceImpl serviceCal) {
	  ActividadExterna externa = find(idExterna);
	  
	  if (aceptada) {
		  externa.setEstado(EEstadoActividadExterna.ACEPTADA);

		  //serviceCal.generateCalendar(externa.getId());
	  }
	  else {
		  externa.setEstado(EEstadoActividadExterna.RECHAZADA);

		  /*
		  serviceCal.disableMessages();
		  serviceCal.removeAll(externa.getCalendarItems());
		  serviceCal.enableMessages();

		  externa.getCalendarItems().clear();
		  */
	  }
	  
	  disableMessages();
	  persist(externa);
	  enableMessages();

	  notificarEstadoSolicitudActExterna(externa);  
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.ActividadExternaService#removeActividadExterna(es.csa.fdt.cyldigital.servicios.entity.ActividadExterna)
   */
  public void removeActividadExterna(ActividadExterna actividadExterna) {
	  if (actividadExterna.getId() != 0)
		  remove(actividadExterna);
  }

  /**
   * Elimina externa from cancel.
   * 
   * @param externa
   *           externa
   */
  public void removeExternaFromCancel(ActividadExterna externa) {
	  if (externa.getId() != 0) {
		  setMessagesEnabled(false);
		  remove(externa);
		  setMessagesEnabled(true);
	  }
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.CursoService#getServiciosLanguageDisponibles()
   */
  public List<ServicioCursoLanguage> getServiciosCursosLanguageDisponibles() {
	  String idioma = LocaleContextHolder.getLocaleContext().getLocale().toString();
	  
	  Map<String, Object> condiciones = new HashMap<String, Object>();
	  condiciones.put("language.ISO", idioma);
	  condiciones.put("desactivado", false);
	  condiciones.put("servicioCurso.desactivado", false);
	  condiciones.put("servicio.disponibleColaborador", true);
	  
	  return findWhereOrder(ServicioCursoLanguage.class, condiciones, "nombre", true);
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.CursoService#getServiciosDisponibles()
   */
  public List<ServicioCurso> getServiciosCursosDisponibles() {
	  //return findAllOrder(ServicioCurso.class, "nombre", true);
	  
	  List<ServicioCurso> serviciosCurso = new ArrayList<ServicioCurso>();
	  List<ServicioCursoLanguage> serviciosCursoLanguage;
	  
	  serviciosCursoLanguage = getServiciosCursosLanguageDisponibles();
	  
	  for(ServicioCursoLanguage servicioCursoLanguage : serviciosCursoLanguage)
		  serviciosCurso.add(servicioCursoLanguage.getServicioCurso());
	  
	  return serviciosCurso;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.ActividadExternaService#getServiciosTalleresDisponibles()
   */
  public List<ServicioTaller> getServiciosTalleresDisponibles() {
    return findAllOrder(ServicioTaller.class, "nombre", true);
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.ActividadExternaService#getColaboradores()
   */
  public List<Empleado> getColaboradores() {
	  if (colaboradores == null){
		  Map<String, Object> properties = new HashMap<String, Object>();
		  
		  properties.put("puesto", Empleado.Puesto.EXTERNAL_PERSONAL);
		  properties.put("profesor", true);

		  colaboradores = findWhereOrder(Empleado.class, properties, "nombre", true);
	  }
	  
	  return colaboradores;
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.ActividadExternaService#getCentros()
   */
  public List<Centro> getCentros() {
    if (centros == null) {
    	String conditions = "";
    	Map<String, Object> params = new HashMap<String, Object>();
    	
    	List<Parameter> paramCentroAdmin = findWhere(Parameter.class, "nombre", "admin_default_center_id");
		if ((paramCentroAdmin != null) && (paramCentroAdmin.size() > 0)) {
			if (conditions.isEmpty())
				conditions = "WHERE ";
			
			conditions += "(c.id <> :idCentroAdmin) ";
			
			params.put("idCentroAdmin", Long.parseLong(paramCentroAdmin.get(0).getValor()));
		}
    	
		List<Parameter> paramCentroColaboradores = findWhere(Parameter.class, "nombre", "extern_default_center_id");
		if ((paramCentroColaboradores != null) && (paramCentroColaboradores.size() > 0)) {
			if (conditions.isEmpty())
				conditions = "WHERE ";
			else
				conditions += "AND ";
			
			conditions += "(c.id <> :idCentroColaboradores) ";
			
			params.put("idCentroColaboradores", Long.parseLong(paramCentroColaboradores.get(0).getValor()));
		}
		
		String strqry = "SELECT c " +
						"FROM " + Centro.class.getName() + " c " +
						conditions +
						"ORDER BY c.nombre ASC";
    	
    	centros = findByQuery(Centro.class, strqry, params);
    }
    
    return centros;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.ActividadExternaService#getAulas()
   */
  public List<Aula> getAulas() {
    if (aulas == null)
    	aulas = findAllOrder(Aula.class, "nombre", true);

    return aulas;
  }

  /**
   * Checks if is borrar preinscripcion.
   * 
   * @return true, if is borrar preinscripcion
   */
  /*public boolean isBorrarPreinscripcion() {
    return borrarPreinscripcion;
  }*/

  /**
   * Asigna borrar preinscripcion.
   * 
   * @param borrarPreinscripcion
   *          borrar preinscripcion
   */
  /*public void setBorrarPreinscripcion(boolean borrarPreinscripcion) {
    this.borrarPreinscripcion = borrarPreinscripcion;
  }*/

  /**
   * Rechazar actividad externa.
   * 
   * @param externa
   *           externa
   * @param idPreinscripcion
   *           id preinscripcion
   */
  /*public void rechazarActividadExterna(ActividadExterna externa, Long idPreinscripcion) {
    PreinscripcionCurso pc = null;
    PreinscripcionTaller pt = null;

    if (externa.isCurso()) {
      pc = find(PreinscripcionCurso.class, idPreinscripcion);
    } else if (externa.isTaller()) {
      pt = find(PreinscripcionTaller.class, idPreinscripcion);
    }

    if (borrarPreinscripcion) {
      if (pc != null) remove(pc);
      else if (pt != null) remove(pt);
    } else {
      if (pc != null) {
        pc.getExternasCursosRechazos().add(externa);
        pc.setPeso(pc.getPeso() + 1);
        persist(pc);
      } else if (pt != null) {
        pt.getExternasTalleresRechazos().add(externa);
        pt.setPeso(pt.getPeso() + 1);
        persist(pt);
      }
    }
  }*/

  /**
   * Matricula from preinscripcion.
   * 
   * @param externa
   *           externa
   * @param idPreinscripcion
   *           id preinscripcion
   * @return true, if successful
   */
  /*public boolean matriculaFromPreinscripcion(ActividadExterna externa, Long idPreinscripcion) {
    PreinscripcionCurso pc = null;
    PreinscripcionTaller pt = null;

    if (externa.isCurso()) {
      pc = find(PreinscripcionCurso.class, idPreinscripcion);
    } else if (externa.isTaller()) {
      pt = find(PreinscripcionTaller.class, idPreinscripcion);
    }

    boolean flag = false;
    if (pc != null) {
      Long idUsuario = pc.getUsuario().getId();
      Usuario u = find(Usuario.class, idUsuario);

      for (MatriculaExterna matr : externa.getMatriculas()) {
        if (matr.getUsuario().getId() == idUsuario) {
          flag = true;
          addErrorMessage("El usuario ya está matriculado en esta actividad externa");
        }
      }

      if (!flag) {
        ((ServicioCurso) externa.getServicio()).getPreinscripciones().remove(pc);
        u.getPreinscripcionesCursos().remove(pc);

        MatriculaExterna m = new MatriculaExterna();
        m.setAdmitido(true);
        m.setFechaMatricula(new Date());

        m.setExterna(externa);
        m.setUsuario(u);
        persist(m);

        setMessagesEnabled(false);
        remove(pc);
        setMessagesEnabled(true);
        return true;
      }
    }

    else if (pt != null) {
      Long idUsuario = pt.getUsuario().getId();
      Usuario u = find(Usuario.class, idUsuario);

      for (MatriculaExterna matr : externa.getMatriculas()) {
        if (matr.getUsuario().getId() == idUsuario) {
          flag = true;
          addErrorMessage("El usuario ya está matriculado en esta actividad externa");
        }
      }

      if (!flag) {
        ((ServicioTaller) externa.getServicio()).getPreinscripciones().remove(pt);
        u.getPreinscripcionesTalleres().remove(pt);

        MatriculaExterna m = new MatriculaExterna();
        m.setAdmitido(true);
        m.setFechaMatricula(new Date());

        m.setExterna(externa);
        m.setUsuario(u);
        persist(m);

        setMessagesEnabled(false);
        remove(pt);
        setMessagesEnabled(true);
        return true;
      }
    }
    return false;
  }*/
  
  /**
   * Comprueba la lista de planificaciones y crea una nueva con un elemento si no no existe o está vacía.
   * 
   * @return the schedules
   */
  public List<ActivitySchedule> getSchedulesList(List<ActivitySchedule> las) {
	  if (las == null)
		  las = new ArrayList<ActivitySchedule>();
		
	  if (las.isEmpty())
		  addNewSchedule(las);

	  return las;
  }

  /**
   * Actualiza centro.
   * 
   * @param externa
   *           externa
   * @param currentUser
   *           current user
   */
  public void actualizaCentro(ActividadExterna externa, UsuarioLdap currentUser, SelectFilterByCenterAux serviceCenterAux) {
	  centro = externa.getCentro() != null ? externa.getCentro() : null;

	  if ((currentUser.getCentro() != null) && 
		  (currentUser.getCentro().getId() != serviceCenterAux.getCentroColaboradores().getId()) && 
		  (currentUser.getCentro().getId() != serviceCenterAux.getCentroTest().getId())) {
		  Centro c = find(Centro.class, currentUser.getCentro().getId());
		  externa.setCentro(c);
		  centro = c;
	  }
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.ActividadExternaService#comprobarEdadUsuario(es.csa.fdt.cyldigital.servicios.entity.MatriculaExterna)
   */
  public void comprobarEdadUsuario(MatriculaExterna m) {

    if (m.getExterna() != null && m.getUsuario() != null) {
      UsuarioCyL u = find(UsuarioCyL.class, m.getUsuario().getId());
      if (u.getEdad() < m.getExterna().getServicio().getEdadMinima()
          || u.getEdad() > m.getExterna().getServicio().getEdadMaxima()) {
        addWarnMessage("matricula.validator.edad_incorrecta");
      }
    }
  }

  /**
   * Anadir observacion guion.
   * 
   * @param externa
   *           externa
   * @param o
   *           o
   * @param usuario
   *           usuario
   */
  /*public void anadirObservacionGuion(ActividadExterna externa, ObservacionActividad o, Usuario usuario) {
    if (o.getObservacion().equals("")) addWarnMessage("No puede añadir un comentario vacío");
    else {
      o.setFecha(new Date());
      o.setUsuario(usuario);
      o.setActividad(externa);
      externa.getObservacionesActividad().add(o);
      persist(externa);
    }
  }*/

  /**
   * Devuelve nueva observacion guion.
   * 
   * @return nueva observacion guion
   */
  /*public ObservacionActividad getNuevaObservacionGuion() {
    return new ObservacionActividad();
  }*/

  /**
   * Devuelve next externa.
   * 
   * @param u
   *           u
   * @param externa
   *           externa
   * @param next
   *           next
   * @return next externa
   */
  /*public ActividadExterna getNextExterna(UsuarioLdap u, ActividadExterna externa, boolean next) {
    
     * next=false: devuelve, si hay, la incidencia anterior. A true, la posterior. Recordar que la actividad externa
     * anterior es de fecha posterior al actual. Devuelve null si no hay una incidencia anterior/posterior, si la hay,
     * devuelve su id
     
    List<ActividadExterna> externas = new ArrayList<ActividadExterna>();

    if (!u.getRolesList().contains(Role.ROLE_ACTIVITY_TRAINER_VIEW_ALL)) {
      externas = findByQuery(ActividadExterna.class,
          " Select e from es.csa.fdt.cyldigital.servicios.entity.ActividadExterna e where centro="
              + u.getUser().getCentro().getId() + " order by fechaInicio desc, horaInicio desc");
    } else {
      externas = findByQuery(ActividadExterna.class,
          " Select e from es.csa.fdt.cyldigital.servicios.entity.ActividadExterna e  order by fechaInicio desc, horaInicio desc");
    }

    if (next) {
      if (externas.indexOf(externa) + 1 < externas.size()) return externas.get(externas.indexOf(externa) + 1);
    } else {
      if (externas.indexOf(externa) != 0) return externas.get(externas.indexOf(externa) - 1);
    }
    return null;
  }*/
  
  /**
   * Añade una nueva línea de planificación del curso
   * 
   * Crea la lista si no existe previamente. 
   *
   */
  public void addNewSchedule(List<ActivitySchedule> schedules) {
	  if (schedules == null)
		  schedules = new ArrayList<ActivitySchedule>();

	  schedules.add(new ActivitySchedule());
  }
	
  /**
   * Añade una nueva línea de planificación del curso
   * 
   * Crea la lista si no existe previamente. 
   *
   */
  public void removeSchedule(List<ActivitySchedule> schedules, ActivitySchedule schedule) {
	  if ((schedule != null) && (schedules != null) && !schedules.isEmpty())
		  schedules.remove(schedule);
  }

  /**
   * Añade recurso.
   * 
   * @param rec
   *           rec
   * @param externa
   *           externa
   */
  public void addRecurso(Recurso rec, ActividadExterna externa) {
    if (rec.isOcupado(externa)) {
      addWarnMessage("El recurso '" + rec.getNombre() + "' está reservado o se encuentra ocupado en otra actividad");
    } else if (!externa.getRecursos().contains(rec)) externa.getRecursos().add(rec);
  }

  /**
   * Quitar recurso.
   * 
   * @param rec
   *           rec
   * @param externa
   *           externa
   */
  public void quitarRecurso(Recurso rec, ActividadExterna externa) {
    externa.getRecursos().remove(rec);
  }

  /**
   * Quitar todos recursos.
   * 
   * @param externa
   *           externa
   */
  public void quitarTodosRecursos(ActividadExterna externa) {
    externa.getRecursos().removeAll(externa.getRecursos());
  }

  /**
   * Añade recursos aula.
   * 
   * @param externa
   *           externa
   */
  public void addRecursosAula(ActividadExterna externa) {
    String message = "";
    
    int i = 0;
    
    if ((externa != null) && (externa.getAula() != null) && (externa.getFechaInicio() != null) && (externa.getFechaFin() != null) &&
    	(externa.getHoraInicio() != null) && (externa.getHoraFin()) != null) {
    	
    	List<ActividadCalendarItem> acis = generateCalendar(externa);
    	
    	for (Recurso r : externa.getAula().getRecursos()) {
    		if (!r.isOcupado(acis)) {
    			addRecurso(r, externa);
    			i++;
    		}
    		else
    			message += "(" + r.getNombre() + ") ";
    	}
    	
    	if (i < externa.getAula().getRecursos().size()) {
    		addWarnMessage("actividad.anyadirRecursos.recursosOcupados");
		  	addWarnMessage(message);
    	}
    }
    else
    	addErrorMessage("actividad.anyadirRecursosAula.noCalendario");
  }
  
  public List<ActividadCalendarItem> generateCalendar(Actividad actividad) {
	  List<ActividadCalendarItem> acis = new ArrayList<ActividadCalendarItem>();

	  ActividadCalendarItem item;

	  Calendar c = Calendar.getInstance();
	  Calendar tempStart = Calendar.getInstance();
	  Calendar tempEnd = Calendar.getInstance();
	  Calendar tempTime = Calendar.getInstance();

	  if (actividad != null) {
		  tempTime.setTime(actividad.getHoraInicio());
		  tempStart.setTime(actividad.getFechaInicio());
		  tempStart.set(Calendar.HOUR, tempTime.get(Calendar.HOUR_OF_DAY));
		  tempStart.set(Calendar.MINUTE, tempTime.get(Calendar.MINUTE));
	
		  tempTime.setTime(actividad.getHoraFin());
		  tempEnd.setTime(actividad.getFechaFin());
		  tempEnd.set(Calendar.HOUR, tempTime.get(Calendar.HOUR_OF_DAY));
		  tempEnd.set(Calendar.MINUTE, tempTime.get(Calendar.MINUTE));
	
		  c.setTime(tempStart.getTime());
	
		  while (c.before(tempEnd) || (c.get(Calendar.DAY_OF_YEAR) <= tempEnd.get(Calendar.DAY_OF_YEAR) &&
				 c.get(Calendar.YEAR) == tempEnd.get(Calendar.YEAR))) {
			  if ((c.get(Calendar.DAY_OF_WEEK) > 1 && c.get(Calendar.DAY_OF_WEEK) < 7) || actividad instanceof Taller) {
				  item = new ActividadCalendarItem();
		
				  c.set(Calendar.HOUR_OF_DAY, tempStart.get(Calendar.HOUR_OF_DAY));
				  c.set(Calendar.MINUTE, tempStart.get(Calendar.MINUTE));
				  item.setStart(c.getTime());
				  c.set(Calendar.HOUR_OF_DAY, tempEnd.get(Calendar.HOUR_OF_DAY));
				  c.set(Calendar.MINUTE, tempEnd.get(Calendar.MINUTE));
				  item.setEnd(c.getTime());
				  item.setAllDay(false);
		  
				  /*item.setActividadType(actividad instanceof Curso ? ActividadType.CURSO
																	 : (actividad instanceof Taller ? ActividadType.TALLER
																	 : (actividad instanceof Charla ? ActividadType.CHARLA
																	 : (actividad instanceof Asesoramiento ? ActividadType.ASESORAMIENTO :
																	 : (actividad instanceof ActividadExterna ? ActividadType.EXTERNA : null)))));
				  item.setActividad(actividad);*/
		  
				  acis.add(item);
			  }

			  c.set(Calendar.DATE, c.get(Calendar.DATE) + 1);
		  }
	  }
	
	  return acis;
  }
  
  /**
   * Aniadir fichero.
   * 
   * @param event
   *           event
   */
  public synchronized void aniadirFichero(UploadEvent event) {
	  FileOnDisk fileUploaded = new FileOnDisk();
	  String fileKey = (String)event.getComponent().getAttributes().get("lang");
	  
	  fileUploaded.setFicheroSubido(new FicheroSubidoRichFacesAdapter(event.getUploadItem()));
	  
	  filesUploaded.put(fileKey, fileUploaded);
  }
  
  /**
   * Marcar fichero para quitar.
   * 
   * @param event
   *           event
   */
  public synchronized void marcarFicheroParaQuitar(ActionEvent event) {
	  fileToRemove = (String)event.getComponent().getAttributes().get("lang");
  }

  /**
   * Quitar fichero.
   * 
   * @param s
   *           s
   * @param service
   *           service
   */
  public void quitarFichero(ActividadExterna ae) {
	filesUploaded.remove(fileToRemove);
	
	for(ActividadExternaLanguage ael: ae.getTextosExterna())
		if (ael.getLanguage().getISO().compareTo(fileToRemove) == 0)
			ael.setFicheroActExt(null);
  }
  
  /**
   * Devuelve filesUploaded
   * 
   * @return the filesUploaded
   */
  public Map<String, FileOnDisk> getFilesUploaded() {
  	return filesUploaded;
  }

  /**
   * Asigna filesUploaded
   * 
   * @param filesUploaded the filesUploaded to set
   */
  public void setFilesUploaded(Map<String, FileOnDisk> filesUploaded) {
  	this.filesUploaded = filesUploaded;
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.CursoService#getFormulariosLanguage()
   */
  public List<FormularioLanguage> getFormulariosLanguage() {
	if (formulariosLanguage == null) {
		String idioma = LocaleContextHolder.getLocaleContext().getLocale().toString();
	  
		formulariosLanguage = findWhereOrder(FormularioLanguage.class, "language.ISO", idioma, "nombre", true);
	}
	
	return formulariosLanguage;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.CursoService#getFormularios()
   */
  public List<Formulario> getFormularios() {
	if (formularios == null) {
		formularios = new ArrayList<Formulario>();
		List<FormularioLanguage> formulariosLanguage = getFormulariosLanguage();

		for(FormularioLanguage formularioLanguage : formulariosLanguage)
			formularios.add(formularioLanguage.getFormulario());
	}
	
	return formularios;
  }

  public void setFormularios(List<Formulario> formularios) {
	this.formularios = formularios;
  }

  public void asociarFormularioConcreto(Long idActividadExterna, Long idFormularioConcreto) {
	  if ((idActividadExterna != null) && (idFormularioConcreto != null)) {
		  ActividadExterna ae = find(idActividadExterna);
		  FormularioConcreto fc = find(FormularioConcreto.class, idFormularioConcreto);
	  	 
		  ae.getFormulariosConcretos().add(fc);
		  
		  disableMessages();
		  persist(ae);
		  enableMessages();
	  }
  }
  
  public Empleado getEmpleado(Usuario u) {
	  Empleado empleado = find(Empleado.class, u.getId());
	  
	  return empleado;
  }
  
  public boolean reOpenTiempoEncuesta(ActividadExterna externa, Usuario usuario) {
	  if ((externa != null) && (externa.getId() != 0) &&
		  (externa.getCuestionarioSatisfaccion() != null) &&
		  externa.isCerrada()) {
		  Empleado empleado = find(Empleado.class, usuario.getId());

		  //Con permisos de administrador
		  if (usuario.getRolesList().contains(Role.ROLE_ADMIN)
			  ||
			  //Con permisos de colaborador o solicitante de actividades externas
			  //y dueño de la actividad o de la misma entidad colaboradora que el dueño
		      ((usuario.getRolesList().contains(Role.ROLE_ACTIVITY_EXTERNAL_REQUEST) ||
		        usuario.getRolesList().contains(Role.ROLE_EXTERN)) &&
		       ((empleado.getId() == externa.getColaborador().getId()) ||
    		    ((empleado.getProveedor() != null) && (externa.getColaborador().getProveedor() != null) && 
		    	 (empleado.getProveedor().getId() == externa.getColaborador().getProveedor().getId()) &&
		         (empleado.getId() != externa.getColaborador().getId()))))
		      ||
		      //Con permisos de responsable de centro o edición de actividades y perteneciente al mismo centro
		      //al que pertenece la actividad de tipo cesión de aula.
		      ((usuario.getRolesList().contains(Role.ROLE_ACTIVITY_EDIT) ||
		        usuario.getRolesList().contains(Role.ROLE_AREA_MANAGER) ||
		        usuario.getRolesList().contains(Role.ROLE_DIGITAL_ALLY)) &&
		       ((externa.getCentro() != null) && (externa.getCentro().getId() == usuario.getCentro().getId())))
		     ) {

			  Date horaFinEncuesta = new Date(System.currentTimeMillis() +
					  						  (Integer.parseInt(findOneWhere(Parameter.class, "nombre", "anonymous_survey_time").getValor()) * 60000L));

			  externa.setFinEncuesta(horaFinEncuesta);
			  
			  disableMessages();
			  persist(externa);
			  enableMessages();
			  
			  String[] params = {(new SimpleDateFormat("dd/MM/yyyy HH:mm")).format(horaFinEncuesta)};
			  addInfoMessage("actividad.externa.exitoReOpenTiempoEncuesta", params);
			  
			  return true;
		  }
		  else {
			addErrorMessage("actividad.externa.errorProfileReOpenTiempoEncuesta");  
			
			return false;
		  }
	  }
	  else {
		  addErrorMessage("actividad.externa.errorAENoCerradaReOpenTiempoEncuesta");
		  
		  return false;
	  }
  }
  
  public boolean deleteFormularioConcreto(ActividadExterna externa, Long idFormularioConcreto, Usuario usuario) {
	  if ((externa != null) &&
		  (externa.getCuestionarioSatisfaccion() != null) &&
		  externa.isCerrada()) {
		  Empleado empleado = find(Empleado.class, usuario.getId());

		  	  //Con permisos de administrador
		  if (usuario.getRolesList().contains(Role.ROLE_ADMIN)
			  ||
			  //Con permisos de colaborador o solicitante de actividades externas
			  //y dueño de la actividad o de la misma entidad colaboradora que el dueño
		      ((usuario.getRolesList().contains(Role.ROLE_ACTIVITY_EXTERNAL_REQUEST) ||
		        usuario.getRolesList().contains(Role.ROLE_EXTERN)) &&
		       ((empleado.getId() == externa.getColaborador().getId()) ||
    		    ((empleado.getProveedor() != null) && (externa.getColaborador().getProveedor() != null) && 
		    	 (empleado.getProveedor().getId() == externa.getColaborador().getProveedor().getId()) &&
		         (empleado.getId() != externa.getColaborador().getId()))))
		      ||
		      //Con permisos de responsable de centro o edición de actividades y perteneciente al mismo centro
		      //al que pertenece la actividad de tipo cesión de aula.
		      ((usuario.getRolesList().contains(Role.ROLE_ACTIVITY_EDIT) ||
		        usuario.getRolesList().contains(Role.ROLE_AREA_MANAGER) || 
		        usuario.getRolesList().contains(Role.ROLE_DIGITAL_ALLY)) &&
		       ((externa.getCentro() != null) && (externa.getCentro().getId() == usuario.getCentro().getId())))
		     ) {
			  FormularioConcreto fc = find(FormularioConcreto.class, idFormularioConcreto);
			  
			  externa.getFormulariosConcretos().remove(fc);
			  
			  remove(fc);
			  
			  return true;
		  }
		  else {
			addErrorMessage("actividad.externa.errorProfileDeletingFormularioConcreto");
			
			return false;
		  }
	  }
	  else {
		  addErrorMessage("actividad.externa.errorExternaDeletingFormularioConcreto");
		  
		  return false;
	  }
  }
  
  /**
   * Notificación de la solicitud de una Actividad Externa
   * 
   * @param actividadExterna
   *           actividadExterna  
   * @return true, if successful
   */

  private void notificarSolicitudActExterna(ActividadExterna actividadExterna) {
	  
	  System.out.println("actividadExterna: " + actividadExterna.getId());
	  System.out.println("actividadExterna: " + actividadExterna.getFechasLegibles());
	  System.out.println("actividadExterna: " + actividadExterna.getFechaInicio());
	  System.out.println("actividadExterna: " + actividadExterna.getFechaFin());
	  
	  NotificationService<ActividadExternaServiceImpl> n = new NotificationService<ActividadExternaServiceImpl>();
	  String mailActivate = "";
	  String mailRequestexternactivityActivate = "";
	  
	  
	  try {
		  List<Parameter> parameters1 = findWhere(Parameter.class, "nombre", "mail_activate");
		  if (parameters1.size() > 0)
			  mailActivate = parameters1.get(0).getValor();

		  List<Parameter> parameters2 = findWhere(Parameter.class, "nombre", "mail_requestexternactivity_activate");
		  if (parameters2.size() > 0)
			  mailRequestexternactivityActivate = parameters2.get(0).getValor();

		  if (mailActivate.equals("1")){
			  if (mailRequestexternactivityActivate.equals("1")){
				  if (n.notificarSolActExterna(this, actividadExterna))
					  addInfoMessage("mail.email");
				  else
					  addErrorMessage("mail.errormail");
			  }
			  else{
				  // están desactivadas las notificaciones por email asociadas al preregistro de usuario en el sistema.
				  addWarnMessage("mail.requestexternactivity.nonotifications");
			  }
		  }
		  else { 
			  // están desactivadas las notificaciones por email en el sistema.
			  addWarnMessage("mail.nonotifications");
		  }
	  } catch (Exception e){
		  e.printStackTrace();
	  }
  }
  
  /**
   * Cancelación de solicitud por parte del colaborador de una Actividad Externa
   * 
   * @param actividadExterna
   *           actividadExterna  
   * @return true, if successful
   */
  @SuppressWarnings("unchecked")
  private void notificarCancelacionSolActExt(ActividadExterna actividadExterna) {
	  
	  NotificationService n = new NotificationService();
	  String mailActivate = "";
	  String mailRequestexternactivityActivate = "";
	  
	  
	  try {
		  List<Parameter> parameters1 = findWhere(Parameter.class, "nombre", "mail_activate");
		  if (parameters1.size() > 0)
			  mailActivate = parameters1.get(0).getValor();

		  List<Parameter> parameters2 = findWhere(Parameter.class, "nombre", "mail_requestexternactivity_activate");
		  if (parameters2.size() > 0)
			  mailRequestexternactivityActivate = parameters2.get(0).getValor();

		  if (mailActivate.equals("1")){
			  if (mailRequestexternactivityActivate.equals("1")){
				  if (n.notificarCancelarSolActExterna(this, actividadExterna))
					  addInfoMessage("mail.email");
				  else
					  addErrorMessage("mail.errormail");
			  }
			  else{
				  // están desactivadas las notificaciones por email asociadas al preregistro de usuario en el sistema.
				  addWarnMessage("mail.requestexternactivity.nonotifications");
			  }
		  }
		  else { 
			  // están desactivadas las notificaciones por email en el sistema.
			  addWarnMessage("mail.nonotifications");
		  }
	  } catch (Exception e){
		  e.printStackTrace();
	  }
  }
  
  /**
   * Notificar el estado de la solicitud de la Actividad Externa.
   * 
   * @param actividadExterna
   *           actividadExterna
   * @return true, if successful
   */
  @SuppressWarnings("unchecked")
  private void notificarEstadoSolicitudActExterna(ActividadExterna actividadExterna) {
	  
	  NotificationService n = new NotificationService();
	  String mailActivate = "";
	  String mailRequestexternactivityActivate = "";
	  
	  
	  try {
		  List<Parameter> parameters1 = findWhere(Parameter.class, "nombre", "mail_activate");
		  if (parameters1.size() > 0)
			  mailActivate = parameters1.get(0).getValor();

		  List<Parameter> parameters2 = findWhere(Parameter.class, "nombre", "mail_requestexternactivity_activate");
		  if (parameters2.size() > 0)
			  mailRequestexternactivityActivate = parameters2.get(0).getValor();

		  if (mailActivate.equals("1")){
			  if (mailRequestexternactivityActivate.equals("1")){
				  if (n.notificarEstadoSolActExterna(this, actividadExterna))
					  addInfoMessage("mail.email");
				  else
					  addErrorMessage("mail.errormail");
			  }
			  else{
				  // están desactivadas las notificaciones por email asociadas al preregistro de usuario en el sistema.
				  addWarnMessage("mail.requestexternactivity.nonotifications");
			  }
		  }
		  else { 
			  // están desactivadas las notificaciones por email en el sistema.
			  addWarnMessage("mail.nonotifications");
		  }
	  } catch (Exception e){
		  e.printStackTrace();
	  }
  }

  /**
   * Notifica la solicitud o el cambio de estado según el estado de la actividad externa 
   * @param actividadExterna
   * 			actividadExterna
   */
  public void notificarActExterna(ActividadExterna actividadExterna){
	  if(actividadExterna.getEstado().equals(EEstadoActividadExterna.ACEPTADA) || 
	     actividadExterna.getEstado().equals(EEstadoActividadExterna.RECHAZADA)) {	 
		  notificarEstadoSolicitudActExterna(actividadExterna);
	  } else if (actividadExterna.getEstado().equals(EEstadoActividadExterna.SOLICITADA)) {
		  notificarEstadoSolicitudActExterna(actividadExterna);
		  notificarSolicitudActExterna(actividadExterna);
	  }
  }
  
  /**
   * Notifica la creación de una actividad múltiple. 
   * @param lae
   * 		Lista de actividades externas
   */
  public void notificarActExternaMultiple(List<ActividadExterna> lae){
	  for (ActividadExterna ae : lae)
		  notificarActExterna(ae);
  }
  
  public List<AETeaching> getAeTeachings() {
	  if (aeTeachings == null) {
		  aeTeachings = new ArrayList<AETeaching>();
		  addNewTeaching();
	  }

	  return aeTeachings;
  }

  public void addNewTeaching() {
	  aeTeachings.add(new AETeaching());
  }
  
  public void removeTeaching(AETeaching aeTeachingToRemove) {
	  aeTeachings.remove(aeTeachingToRemove);
  }
  
  public List<Empleado> getSelectedColaboradores() {
	  return selectedColaboradores;
  }
  
  public List<Centro> getSelectedCentros() {
	  return selectedCentros;
  }
  
  public void updateSelectedColaboradores() {
	  selectedColaboradores.clear();
	  
	  for (AETeaching ae : aeTeachings){
		  selectedColaboradores.add(ae.getColaborador());
	  }
  }
  
  public void updateSelectedCentros() {
	  selectedCentros.clear();
	  
	  for (AETeaching ae : aeTeachings){
		  selectedCentros.add(ae.getCentro());
	  }
  }
  
  //RF_15_31 Selección solicitante actividades externas
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.ActividadExternaService#getColaboradores()
   */
  public List<Empleado> getColaboradoresEntidad() {
	  
	  if(proveedor == null) {
		  if(externa.getColaborador() != null) {
			  proveedor = externa.getColaborador().getProveedor();
		  }
	  }
	  
	  if(proveedor != null) {
		  Map<String, Object> properties = new HashMap<String, Object>(); 
		  
		  properties.put("puesto", Empleado.Puesto.EXTERNAL_PERSONAL);
		  properties.put("proveedor.id", proveedor.getId());
		  
		  colaboradoresEntidad = findWhereOrder(Empleado.class, properties, "nombre", true);
	  }

	  return colaboradoresEntidad;
  }
  
  /**
   * 
   */
  public void isColaborador(Usuario usuario) {
	  boolean isColaborador = false;
	  
	  //ROLE_ACTIVITY_EXTERNAL_REQUEST, ROLE_EXTERN
	  if ( (usuario.getRolesList().contains(Usuario.Role.ROLE_EXTERN)) || 
		   (usuario.getRolesList().contains(Usuario.Role.ROLE_ACTIVITY_EXTERNAL_REQUEST)))
		  isColaborador = true;
	  
	  if(isColaborador) {
		  Empleado empleado = find(Empleado.class, usuario.getId());
		  setProveedor(empleado.getProveedor());
		  getColaboradoresEntidad();
	  }
  }
  
}
