package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.commons.persistence.browser.jpaql.FilterJPAQLByQuery;
import es.csa.fdt.cyldigital.core.service.BrowserFilteredByCenter;
import es.csa.fdt.cyldigital.servicios.entity.ActividadExterna;
import es.csa.fdt.cyldigital.servicios.entity.Tematica;

/**
 * Servicio del Browser de Actividades Externas.
 * 
 * @author csa
 */
@Service
@Repository
public class ActividadesExternasBrowserImpl extends BrowserFilteredByCenter<ActividadExterna> implements
    ActividadesExternasBrowser<CriterionJPAQL, String> {

  private static final long serialVersionUID = -7227366052245717441L;

  private Tematica tematica;

  /**
   * Construye un nuevo objeto de la clase
   */
  public ActividadesExternasBrowserImpl() {
    super();
    getSort().setSortProperty("fechaInicio DESC,horaInicio DESC");
  }

  /**
   * Devuelve tematica.
   * 
   * @return tematica
   */
  public Tematica getTematica() {
    return tematica;
  }

  /**
   * Asigna tematica.
   * 
   * @param tematica
   *          tematica
   */
  public void setTematica(Tematica tematica) {
    this.tematica = tematica;
  }

  /**
   * Filter old.
   */
  public void filterOld() {
    removeFilterFecha();
    filterOldActividadesExternas();
  }

  private void removeFilterFecha() {
    int i = -1;
    for (Filter<CriterionJPAQL, ActividadExterna> f : getFilters())
      if (f.getCriterion().getQuery().contains("fechaFin")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
  }

  private void filterOldActividadesExternas() {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<ActividadExterna> filter = new FilterJPAQLByQuery<ActividadExterna>();

    query += " fechaFin >= now() ";

    for (Filter<CriterionJPAQL, ActividadExterna> f : getFilters())
      if (f.getCriterion().getQuery().contains("fechaFin")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }

  /**
   * Filter by curso.
   * 
   * @param idServicio
   *           id servicio
   */
  public void filterByCurso(Long idServicio) {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<ActividadExterna> filter = new FilterJPAQLByQuery<ActividadExterna>();

    if (idServicio != null && idServicio > 0) {
      filter.getCriterion().addParameter(idServicio);
      query += " idServicio = ? ";
    }

    for (Filter<CriterionJPAQL, ActividadExterna> f : getFilters())
      if (f.getCriterion().getQuery().equals(query)) flag = true;

    if (!query.isEmpty() && !flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }

  /**
   * Filter by taller.
   * 
   * @param idServicio
   *           id servicio
   */
  public void filterByTaller(Long idServicio) {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<ActividadExterna> filter = new FilterJPAQLByQuery<ActividadExterna>();

    if (idServicio != null && idServicio > 0) {
      filter.getCriterion().addParameter(idServicio);
      query += " idServicio = ? ";
    }

    for (Filter<CriterionJPAQL, ActividadExterna> f : getFilters())
      if (f.getCriterion().getQuery().equals(query)) flag = true;

    if (!query.isEmpty() && !flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }

}
