package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import javax.annotation.Resource;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.faces.util.PredefinedLocaleResolver;
import es.csa.fdt.cyldigital.calendario.entity.ActividadCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.ActividadCalendarItem.ActividadType;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.servicios.entity.Asesoramiento;
import es.csa.fdt.cyldigital.servicios.entity.TipoAsesoramiento;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario.Role;


/**
 * Clase AsesoramientoServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class AsesoramientoServiceImpl extends AbstractServiceImpl<Asesoramiento, Long> implements AsesoramientoService {

  private List<Centro> centros;

  private Centro centro;

  private boolean changeCentro = true;

  private List<TipoAsesoramiento> tiposAsesoramientos;

  private List<Empleado> dinamizadores;

  private static final long serialVersionUID = -5575228180361917674L;

  @Resource
  private transient HashMap<String, Object> generalProperties;

  /**
   * Apply centro.
   * 
   * @param centro
   *           centro
   */
  public void applyCentro(Centro centro) {
    this.centro = centro;
    changeCentro = true;
  }

  /**
   * Construye un nuevo objeto de la clase
   */
  public AsesoramientoServiceImpl() {
    super();
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.AsesoramientoService#persistAsesoramiento(es.csa.fdt.cyldigital.servicios.entity.Asesoramiento)
   */
  public boolean persistAsesoramiento(Asesoramiento asesoramiento) {
    Calendar c = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    Calendar cTime = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    ActividadCalendarItem item = new ActividadCalendarItem();
    item.setActividadType(ActividadType.ASESORAMIENTO);
    item.setActividad(asesoramiento);
    item.setUsuario(asesoramiento.getUsuario());

    c.setTime(asesoramiento.getFecha());
    cTime.setTime(asesoramiento.getHora());
    c.set(Calendar.HOUR, cTime.get(Calendar.HOUR));
    c.set(Calendar.MINUTE, cTime.get(Calendar.MINUTE));

    item.setStart(c.getTime());

    if (asesoramiento.isResolucion()) {
      item.setEnd(asesoramiento.getFechaDuracion() == null ? c.getTime() : new Date(c.getTimeInMillis()
          + (asesoramiento.getDuracion() * 60000)));
    }
    asesoramiento.addCalendarItem(item);
    persist(asesoramiento);
    return true;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.AsesoramientoService#removeAsesoramiento(es.csa.fdt.cyldigital.servicios.entity.Asesoramiento)
   */
  public void removeAsesoramiento(Asesoramiento asesoramiento) {
    if (asesoramiento.getId() != 0) {
      for (ActividadCalendarItem aci : asesoramiento.getCalendarItems()) {
        remove(aci);
      }
      asesoramiento.getCalendarItems().clear();
      remove(asesoramiento);
    }
  }

  /**
   * Devuelve tipos asesoramientos.
   * 
   * @return tipos asesoramientos
   */
  public List<TipoAsesoramiento> getTiposAsesoramientos() {
    if (tiposAsesoramientos == null) tiposAsesoramientos = findAllOrder(TipoAsesoramiento.class, "nombre", true);
    return tiposAsesoramientos;
  }

  /**
   * Devuelve dinamizadores.
   * 
   * @return dinamizadores
   */
  public List<Empleado> getDinamizadores() {
    if (centro != null && changeCentro) {
      List<Empleado> empleados = findWhereOrder(Empleado.class, "centro.id", centro.getId(), "nombre", true);
      dinamizadores = new ArrayList<Empleado>();
      for (Empleado e : empleados) {
        if (e.isProfesor()) {
          dinamizadores.add(e);
        }
      }
      changeCentro = false;
    }
    return dinamizadores;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.AsesoramientoService#getCentros()
   */
  public List<Centro> getCentros() {
    if (centros == null) centros = findAllOrder(Centro.class, "nombre", true);
    return centros;
  }

  /**
   * Actualiza centro.
   * 
   * @param asesoramiento
   *           asesoramiento
   * @param currentUser
   *           current user
   */
  public void actualizaCentro(Asesoramiento asesoramiento, UsuarioLdap currentUser) {
    centro = asesoramiento.getCentro() != null ? asesoramiento.getCentro() : null;

    if (currentUser.getCentro() != null) {
      Centro c = find(Centro.class, currentUser.getCentro().getId());
      asesoramiento.setCentro(c);
      centro = c;
      changeCentro = true;
    } else if (generalProperties.containsKey("DEFAULT_CENTER_ID") && asesoramiento.getCentro() == null) {
      Centro c = find(Centro.class, Long.valueOf((String) generalProperties.get("DEFAULT_CENTER_ID")));
      if (c != null) {
        asesoramiento.setCentro(c);
        centro = c;
        changeCentro = true;
      }
    }
  }

  /**
   * Devuelve next asesoramiento.
   * 
   * @param u
   *           u
   * @param a
   *           a
   * @param next
   *           next
   * @return next asesoramiento
   */
  public Asesoramiento getNextAsesoramiento(UsuarioLdap u, Asesoramiento a, boolean next) {
    if (a != null) {
      List<Asesoramiento> asesoramientos = new ArrayList<Asesoramiento>();

      if (!u.getRolesList().contains(Role.ROLE_ACTIVITY_TRAINER_VIEW_ALL)) {
        asesoramientos = findByQuery(Asesoramiento.class,
            " Select a from es.csa.fdt.cyldigital.servicios.entity.Asesoramiento a where centro="
                + u.getUser().getCentro().getId() + " order by fecha desc");
      } else {
        asesoramientos = findByQuery(Asesoramiento.class,
            " Select a from es.csa.fdt.cyldigital.servicios.entity.Asesoramiento a  order by fecha desc");
      }

      if (asesoramientos.indexOf(a) > -1) {
        if (next) {
          if (asesoramientos.indexOf(a) + 1 < asesoramientos.size())
            return asesoramientos.get(asesoramientos.indexOf(a) + 1);
        } else {
          if (asesoramientos.indexOf(a) != 0) return asesoramientos.get(asesoramientos.indexOf(a) - 1);
        }
      }
    }
    return null;
  }

}
