package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.List;

import javax.persistence.Query;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.commons.persistence.browser.jpaql.FilterJPAQLByQuery;
import es.csa.fdt.cyldigital.core.service.BrowserFilteredByCenter;
import es.csa.fdt.cyldigital.servicios.entity.Asesoramiento;
import es.csa.fdt.cyldigital.servicios.entity.TipoAsesoramiento;
import es.csa.fdt.cyldigital.servicios.entity.Asesoramiento.Colectivo;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;


/**
 * Clase AsesoramientosBrowserImpl.
 * @author csa
 */
@Service
@Repository
public class AsesoramientosBrowserImpl extends BrowserFilteredByCenter<Asesoramiento> implements
    AsesoramientosBrowser<CriterionJPAQL, String> {

  private static final long serialVersionUID = -7227366052245717441L;

  private TipoAsesoramiento tipoAsesoramiento;
  private Empleado dinamizador;
  private Colectivo colectivo;

  private List<Empleado> dinamizadores;

  /**
   * Construye un nuevo objeto de la clase
   */
  public AsesoramientosBrowserImpl() {
    super();
    getSort().setSortAsc(false);
    getSort().setSortProperty("fecha");
    dinamizadores = new ArrayList<Empleado>();
  }

  /**
   * Devuelve dinamizadores.
   * 
   * @return dinamizadores
   */
  @SuppressWarnings("unchecked")
  public List<Empleado> getDinamizadores() {
    if (dinamizadores.isEmpty()) {
      Query query = getEntityManager().createQuery(
          "SELECT e FROM " + Empleado.class.getName() + " e " + " WHERE profesor = ? ");
      query.setParameter(1, true);
      dinamizadores = (List<Empleado>) query.getResultList();
    }
    return dinamizadores;
  }

  /**
   * Devuelve tipo asesoramiento.
   * 
   * @return tipo asesoramiento
   */
  public TipoAsesoramiento getTipoAsesoramiento() {
    return tipoAsesoramiento;
  }

  /**
   * Asigna tipo asesoramiento.
   * 
   * @param tipoAsesoramiento
   *          tipo asesoramiento
   */
  public void setTipoAsesoramiento(TipoAsesoramiento tipoAsesoramiento) {
    this.tipoAsesoramiento = tipoAsesoramiento;
  }

  /**
   * Devuelve colectivo.
   * 
   * @return colectivo
   */
  public Colectivo getColectivo() {
    return colectivo;
  }

  /**
   * Asigna colectivo.
   * 
   * @param colectivo
   *          colectivo
   */
  public void setColectivo(Colectivo colectivo) {
    this.colectivo = colectivo;
  }

  /**
   * Devuelve dinamizador.
   * 
   * @return dinamizador
   */
  public Empleado getDinamizador() {
    return dinamizador;
  }

  /**
   * Asigna dinamizador.
   * 
   * @param dinamizador
   *          dinamizador
   */
  public void setDinamizador(Empleado dinamizador) {
    this.dinamizador = dinamizador;
  }

  /**
   * Filter tipo asesoramiento.
   */
  public void filterTipoAsesoramiento() {
    removeFilterTipoAsesoramiento();
    if (getTipoAsesoramiento() != null) filterByTipoAsesoramiento(getTipoAsesoramiento().getId());
  }

  private void removeFilterTipoAsesoramiento() {
    int i = -1;
    for (Filter<CriterionJPAQL, Asesoramiento> f : getFilters())
      if (f.getCriterion().getQuery().contains("tipoAsesoramiento.id")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
  }

  /**
   * Filter by tipo asesoramiento.
   * 
   * @param idTipoAsesoramiento
   *           id tipo asesoramiento
   */
  public void filterByTipoAsesoramiento(Long idTipoAsesoramiento) {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<Asesoramiento> filter = new FilterJPAQLByQuery<Asesoramiento>();

    query += " tipoAsesoramiento.id = ? ";
    filter.getCriterion().addParameter(idTipoAsesoramiento);

    for (Filter<CriterionJPAQL, Asesoramiento> f : getFilters())
      if (f.getCriterion().getQuery().contains("tipoAsesoramiento.id")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }

  /**
   * Filter colectivo.
   */
  public void filterColectivo() {
    removeFilterColectivo();
    if (getColectivo() != null) filterByColectivo(getColectivo());
  }

  private void removeFilterColectivo() {
    int i = -1;
    for (Filter<CriterionJPAQL, Asesoramiento> f : getFilters())
      if (f.getCriterion().getQuery().contains("colectivo")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
  }

  /**
   * Filter by colectivo.
   * 
   * @param colectivo
   *           colectivo
   */
  public void filterByColectivo(Colectivo colectivo) {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<Asesoramiento> filter = new FilterJPAQLByQuery<Asesoramiento>();

    query += " colectivo = ? ";
    filter.getCriterion().addParameter(colectivo);

    for (Filter<CriterionJPAQL, Asesoramiento> f : getFilters())
      if (f.getCriterion().getQuery().contains("colectivo")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }

  /**
   * Filter dinamizador.
   */
  public void filterDinamizador() {
    removeFilterDinamizador();
    if (getDinamizador() != null) filterByDinamizador(getDinamizador().getId());
  }

  private void removeFilterDinamizador() {
    int i = -1;
    for (Filter<CriterionJPAQL, Asesoramiento> f : getFilters())
      if (f.getCriterion().getQuery().contains("dinamizador.id")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
  }

  /**
   * Filter by dinamizador.
   * 
   * @param idDinamizador
   *           id dinamizador
   */
  public void filterByDinamizador(Long idDinamizador) {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<Asesoramiento> filter = new FilterJPAQLByQuery<Asesoramiento>();

    query += " dinamizador.id = ? ";
    filter.getCriterion().addParameter(idDinamizador);

    for (Filter<CriterionJPAQL, Asesoramiento> f : getFilters())
      if (f.getCriterion().getQuery().contains("dinamizador.id")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }

  /**
   * Filter asesoramiento.
   */
  public void filterAsesoramiento() {
    removeFilterDinamizador();
    removeFilterTipoAsesoramiento();
    if (getDinamizador() != null) filterByDinamizador(getDinamizador().getId());
    if (getTipoAsesoramiento() != null) filterByTipoAsesoramiento(getTipoAsesoramiento().getId());
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.core.service.BrowserFilteredByCenter#clearFilters()
   */
  @Override
  public void clearFilters() {
    super.clearFilters();
    filterAsesoramiento();
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.core.service.BrowserFilteredByCenter#filter()
   */
  @Override
  public void filter() {
    removeFilterDinamizador();
    super.filter();
  }
}
