package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.calendario.entity.ActividadCalendarItem;
import es.csa.fdt.cyldigital.calendario.service.ActividadCalendarItemServiceImpl;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.language.entity.ActividadLanguage;
import es.csa.fdt.cyldigital.language.entity.CharlaLanguage;
import es.csa.fdt.cyldigital.language.entity.CursoLanguage;
import es.csa.fdt.cyldigital.language.entity.Language;
import es.csa.fdt.cyldigital.language.entity.TematicaLanguage;
import es.csa.fdt.cyldigital.recursos.entity.Recurso;
import es.csa.fdt.cyldigital.servicios.entity.Actividad;
import es.csa.fdt.cyldigital.servicios.entity.Charla;
import es.csa.fdt.cyldigital.servicios.entity.Curso;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCharla;
import es.csa.fdt.cyldigital.servicios.entity.Taller;
import es.csa.fdt.cyldigital.servicios.entity.Tematica;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario.Role;




/**
 * Clase CharlaServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class CharlaServiceImpl extends AbstractServiceImpl<Charla, Long> implements CharlaService {

  private List<Centro> centros;

  private Centro centro;
  
  private List<Centro> centrosCharlaMultiple;

  private boolean changeCentro = true;

  private List<Tematica> tematicas;  

  private List<TematicaLanguage> tematicasLanguage;

  private List<Empleado> profesores;

  private boolean isWarningMostrado = false;

  private static final long serialVersionUID = -5575228180361917674L;

  private boolean avisoRecursos = false;

  @Resource
  private transient HashMap<String, Object> generalProperties;

  /**
   * Devuelve centro.
   * 
   * @return centro
   */
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }
  
  /**
   * Devuelve centrosCharlaMultiple.
   * 
   * @return centrosCharlaMultiple
   */
  public List<Centro> getCentrosCharlaMultiple() {
	  return centrosCharlaMultiple;
  }
  
  /**
   * Asigna centrosCharlaMultiple.
   * 
   * @param centrosCharlaMultiple
   *          centrosCharlaMultiple
   */
  public void setCentrosCharlaMultiple(List<Centro> centrosCharlaMultiple) {
    this.centrosCharlaMultiple = centrosCharlaMultiple;
  }

  /**
   * Apply centro.
   * 
   * @param charla
   *           charla
   */
  public void applyCentro(Charla charla) {
    this.centro = charla.getCentro();
    charla.setAula(null);
    changeCentro = true;
  }

  /**
   * Construye un nuevo objeto de la clase
   */
  public CharlaServiceImpl() {
    super();
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.TematicaServiceImpl#createLanguages(es.csa.fdt.cyldigital.servicios.entity.Tematica)
   */
  public void createLanguages(Charla charla, List<Language> languages) {
	  List<ActividadLanguage> textos = new ArrayList<ActividadLanguage>();
	  List<CharlaLanguage> textosCharla = new ArrayList<CharlaLanguage>();

	  for (Language language : languages) {
		  CharlaLanguage textoCharla = new CharlaLanguage();
		  
		  textoCharla.setLanguage(language);
		  textoCharla.setCharla(charla);
		  
		  textos.add((ActividadLanguage)textoCharla);
		  
		  textosCharla.add(textoCharla);
	  }
	  
	  charla.setTextosCharla(textosCharla);
	  charla.setTextos(textos);
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.CharlaService#persistCharla(es.csa.fdt.cyldigital.servicios.entity.Charla, es.csa.fdt.cyldigital.calendario.service.ActividadCalendarItemServiceImpl)
   */
  public boolean persistCharla(Charla charla, ActividadCalendarItemServiceImpl serviceCal) {
    if (charla.getFechaInicio().getTime() < (new Date()).getTime() && !isWarningMostrado) {
      addWarnMessage("actividad.validator.fechaInicioAnterior");
      
      isWarningMostrado = true;
    }
    /*  se anula la verificacion del numero de recursos
    else if (!avisoRecursos && charla.getNumeroPlazas() > charla.getRecursos().size()) {
      addWarnMessage("actividad.validator.plazasMayorRecursos");
      
      avisoRecursos = true;
    }*/
    else {
      if (charla.getId() > 0 && charla.getCalendarItems().isEmpty())
        serviceCal.generateCalendar(charla.getId());

      persist(charla);
      
      return true;
    }
    
    return false;
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.CharlaService#persistCharlaMultiple(es.csa.fdt.cyldigital.servicios.entity.Charla)
   */
  public List<Charla> persistCharlaMultiple(Charla charlaMultiple) {
	  if ((charlaMultiple.getFechaInicio().getTime() < (new Date()).getTime()) && !isWarningMostrado) {
		  addWarnMessage("actividad.validator.fechaInicioAnterior");

		  isWarningMostrado = true;
	  }
	  else {
		  List<Charla> listaCharlas = new ArrayList<Charla>();
		  
		  for (Centro c : centrosCharlaMultiple) {
			  Charla charla = new Charla();

			  charla.setServicio(charlaMultiple.getServicio());

			  charla.setEdadMinima(charlaMultiple.getEdadMinima());
			  charla.setEdadMaxima(charlaMultiple.getEdadMaxima());
			  charla.setFechaInicio(charlaMultiple.getFechaInicio());
			  charla.setHoraInicio(charlaMultiple.getHoraInicio());
			  charla.setFechaFin(charlaMultiple.getFechaFin()); //new line RF_15_41
			  charla.setHoraFin(charlaMultiple.getHoraFin());
			  charla.setNumeroPlazas(charlaMultiple.getNumeroPlazas());
			  charla.setObservaciones(charlaMultiple.getObservaciones());
			  charla.setTematica(charlaMultiple.getTematica());
			  charla.setCentro(c);
			  
			  //RF_XEA_22 Añadir fecha y hora de publicacion
			  charla.setFechaPublicacion(charlaMultiple.getFechaPublicacion());
			  charla.setHoraPublicacion(charlaMultiple.getHoraPublicacion());
			  //RF_XEA_22
			  
			  //RF_15_04
			  charla.setFechaLimiteInscripcion(charlaMultiple.getFechaLimiteInscripcion());
			  charla.setHoraLimiteInscripcion(charlaMultiple.getHoraLimiteInscripcion());
			  //RF_15_04
			  
			  charla.setTextos(new ArrayList<ActividadLanguage>());
			  charla.setTextosCharla(new ArrayList<CharlaLanguage>());

			  for (CharlaLanguage textoCharlaMultiple : charlaMultiple.getTextosCharla()) {
				  CharlaLanguage textoCharla = new CharlaLanguage();

				  textoCharla.setCharla(charla);
				  textoCharla.setLanguage(textoCharlaMultiple.getLanguage());
				  textoCharla.setNombre(textoCharlaMultiple.getNombre());
				  textoCharla.setDescripcion(textoCharlaMultiple.getDescripcion()); //3.4.11 - RF_15_25
				  
				  charla.getTextos().add((ActividadLanguage)textoCharla);
				  charla.getTextosCharla().add(textoCharla);
			  }

			  persist(charla);
			  
			  listaCharlas.add(charla);
		  }

		  return listaCharlas;
	  }

	  return null;
  }

  /**
   * Persist charla to calendario.
   * 
   * @param charla
   *           charla
   * @return true, if successful
   */
  public boolean persistCharlaToCalendario(Charla charla) {
    if (charla.getFechaInicio().getTime() < (new Date()).getTime() && !isWarningMostrado) {
      addWarnMessage("actividad.validator.fechaInicioAnterior");
      isWarningMostrado = true;
    } else {
      setMessagesEnabled(false);
      persist(charla);
      setMessagesEnabled(true);
      return true;
    }
    return false;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.CharlaService#removeCharla(es.csa.fdt.cyldigital.servicios.entity.Charla)
   */
  public void removeCharla(Charla charla) {
    if (charla.getId() != 0) remove(charla);
  }

  /**
   * Elimina charla from cancel.
   * 
   * @param charla
   *           charla
   */
  public void removeCharlaFromCancel(Charla charla) {
    if (charla.getId() != 0) {
      setMessagesEnabled(false);
      remove(charla);
      setMessagesEnabled(true);
    }
  }

  /**
   * Devuelve tematicas.
   * 
   * @return tematicas
   */
 /* public List<Tematica> getTematicas() {
    if (tematicas == null) tematicas = findAllOrder(Tematica.class, "nombre", true);
    return tematicas;
  }
  */
  
  /**
   * Devuelve tematicasLanguage.
   * 
   * @return tematicasLanguage
   */
  public List<TematicaLanguage> getTematicasLanguage() {
	  String idioma = LocaleContextHolder.getLocaleContext().getLocale().toString();
	  
	  return tematicasLanguage != null ? tematicasLanguage : findWhereOrder(TematicaLanguage.class, "language.ISO", idioma, "nombre", true);
  }
  
  
  
  /**
   * Devuelve tematicas.
   * 
   * @return tematicas
   */
  public List<Tematica> getTematicas() {
	  if (this.tematicas != null)
		  return this.tematicas;
	  else {
		  List<Tematica> tematicas = new ArrayList<Tematica>();
		  List<TematicaLanguage> tematicasLanguage;
		  
		  tematicasLanguage = getTematicasLanguage();
		  
		  for(TematicaLanguage tematicaLanguage : tematicasLanguage)
			  tematicas.add(tematicaLanguage.getTematica());
		  
		  return tematicas;
	  }
  }
  

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.CharlaService#getCentros()
   */
  public List<Centro> getCentros() {
    if (centros == null) centros = findAllOrder(Centro.class, "nombre", true);
    return centros;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.CharlaService#getProfesores()
   */
  public List<Empleado> getProfesores() {
	  if (centro != null && changeCentro) {
		  Map<String, Object> Propiedades = new HashMap<String, Object>();
		  Propiedades.put("centro.id", centro.getId());
		  Propiedades.put("desactivado", false);
    	
		  //List<Empleado> empleados = findWhereOrder(Empleado.class, "centro.id", centro.getId(), "nombre", true);
		  List<Empleado> empleados = findWhereOrder(Empleado.class, Propiedades, "nombre", true);
      
		  profesores = new ArrayList<Empleado>();
		  for (Empleado e : empleados) {
			  if (e.isProfesor()) {
				  profesores.add(e);
			  }
		  }
		  changeCentro = false;
	  }
	  
	  return profesores;
  }

  /**
   * Actualiza centro.
   * 
   * @param charla
   *           charla
   * @param currentUser
   *           current user
   */
  public void actualizaCentro(Charla charla, UsuarioLdap currentUser) {
    centro = charla.getCentro() != null ? charla.getCentro() : null;

    if (currentUser.getCentro() != null) {
      Centro c = find(Centro.class, currentUser.getCentro().getId());
      charla.setCentro(c);
      centro = c;
      changeCentro = true;
    } else if (generalProperties.containsKey("DEFAULT_CENTER_ID") && charla.getCentro() == null) {
      Centro c = find(Centro.class, Long.valueOf((String) generalProperties.get("DEFAULT_CENTER_ID")));
      if (c != null) {
        charla.setCentro(c);
        centro = c;
        changeCentro = true;
      }
    }
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.CharlaService#comprobarEdadUsuario(es.csa.fdt.cyldigital.servicios.entity.MatriculaCharla)
   */
  public void comprobarEdadUsuario(MatriculaCharla m) {

    if (m.getCharla() != null && m.getUsuario() != null) {
      UsuarioCyL u = find(UsuarioCyL.class, m.getUsuario().getId());
      if (u.getEdad() < m.getCharla().getEdadMinima() || u.getEdad() > m.getCharla().getEdadMaxima()) {
        addWarnMessage("matricula.validator.edad_incorrecta");
      }
    }
  }

  /**
   * Devuelve next charla.
   * 
   * @param u
   *           u
   * @param c
   *           c
   * @param next
   *           next
   * @return next charla
   */
  public Charla getNextCharla(UsuarioLdap u, Charla c, boolean next) {
    /*
     * next=false: devuelve, si hay, la incidencia anterior. A true, la posterior. Recordar que el curso anterior es de
     * fecha posterior al actual. Devuelve null si no hay una incidencia anterior/posterior, si la hay, devuelve su id
     */
    if (c != null) {

      List<Charla> charlas = new ArrayList<Charla>();

      if (!u.getRolesList().contains(Role.ROLE_ACTIVITY_TRAINER_VIEW_ALL)) {
        charlas = findByQuery(Charla.class,
            " Select c from es.csa.fdt.cyldigital.servicios.entity.Charla c where centro="
                + u.getUser().getCentro().getId() + " order by fechaInicio desc, horaInicio desc");
      } else {
        charlas = findByQuery(Charla.class,
            " Select c from es.csa.fdt.cyldigital.servicios.entity.Charla c  order by fechaInicio desc, horaInicio desc");
      }
      if (charlas.indexOf(c) > -1) {
        if (next) {
          if (charlas.indexOf(c) + 1 < charlas.size()) return charlas.get(charlas.indexOf(c) + 1);
        } else {
          if (charlas.indexOf(c) != 0) return charlas.get(charlas.indexOf(c) - 1);
        }
      }
    }
    return null;
  }

  /**
   * Añade recurso.
   * 
   * @param rec
   *           rec
   * @param charla
   *           charla
   */
  public void addRecurso(Recurso rec, Charla charla) {
    if (rec.isOcupado(charla)) {
      addWarnMessage("El recurso '" + rec.getNombre() + "' está reservado o se encuentra ocupado en otra actividad");
    } else if (!charla.getRecursos().contains(rec)) charla.getRecursos().add(rec);
  }

  /**
   * Quitar recurso.
   * 
   * @param rec
   *           rec
   * @param charla
   *           charla
   */
  public void quitarRecurso(Recurso rec, Charla charla) {
    charla.getRecursos().remove(rec);
  }

  /**
   * Quitar todos recursos.
   * 
   * @param charla
   *           charla
   */
  public void quitarTodosRecursos(Charla charla) {
    charla.getRecursos().removeAll(charla.getRecursos());
  }

  /**
   * Añade recursos aula.
   * 
   * @param charla
   *           charla
   */
  public void addRecursosAula(Charla charla) {
	  String message = "";
	  
	  int i = 0;

	  if ((charla != null) && (charla.getAula() != null) && (charla.getFechaInicio() != null) &&
		  (charla.getFechaFin() != null) && (charla.getHoraInicio() != null) && (charla.getHoraFin() != null)) {
		  
		  List<ActividadCalendarItem> acis = generateCalendar(charla);
		  
		  for (Recurso r : charla.getAula().getRecursos()) {
			  if (!r.isOcupado(acis)) {
				  addRecurso(r, charla);
				  i++;
			  }
			  else
				  message += "(" + r.getNombre() + ") ";
		  }
		  
		  if (i < charla.getAula().getRecursos().size()) {
			  addWarnMessage("actividad.anyadirRecursos.recursosOcupados");
			  addWarnMessage(message);
		  }
	  } else
		  addErrorMessage("actividad.anyadirRecursosAula.noCalendario");
  }
  
  public List<ActividadCalendarItem> generateCalendar(Actividad actividad) {
	  List<ActividadCalendarItem> acis = new ArrayList<ActividadCalendarItem>();

	  ActividadCalendarItem item;

	  Calendar c = Calendar.getInstance();
	  Calendar tempStart = Calendar.getInstance();
	  Calendar tempEnd = Calendar.getInstance();
	  Calendar tempTime = Calendar.getInstance();

	  if (actividad != null) {
		  tempTime.setTime(actividad.getHoraInicio());
		  tempStart.setTime(actividad.getFechaInicio());
		  tempStart.set(Calendar.HOUR, tempTime.get(Calendar.HOUR_OF_DAY));
		  tempStart.set(Calendar.MINUTE, tempTime.get(Calendar.MINUTE));
	
		  tempTime.setTime(actividad.getHoraFin());
		  tempEnd.setTime(actividad.getFechaFin());
		  tempEnd.set(Calendar.HOUR, tempTime.get(Calendar.HOUR_OF_DAY));
		  tempEnd.set(Calendar.MINUTE, tempTime.get(Calendar.MINUTE));
	
		  c.setTime(tempStart.getTime());
	
		  while (c.before(tempEnd) || (c.get(Calendar.DAY_OF_YEAR) <= tempEnd.get(Calendar.DAY_OF_YEAR) &&
				 c.get(Calendar.YEAR) == tempEnd.get(Calendar.YEAR))) {
			  if ((c.get(Calendar.DAY_OF_WEEK) > 1 && c.get(Calendar.DAY_OF_WEEK) < 7) || actividad instanceof Taller) {
				  item = new ActividadCalendarItem();
		
				  c.set(Calendar.HOUR_OF_DAY, tempStart.get(Calendar.HOUR_OF_DAY));
				  c.set(Calendar.MINUTE, tempStart.get(Calendar.MINUTE));
				  item.setStart(c.getTime());
				  c.set(Calendar.HOUR_OF_DAY, tempEnd.get(Calendar.HOUR_OF_DAY));
				  c.set(Calendar.MINUTE, tempEnd.get(Calendar.MINUTE));
				  item.setEnd(c.getTime());
				  item.setAllDay(false);
		  
				  /*item.setActividadType(actividad instanceof Curso ? ActividadType.CURSO
																	 : (actividad instanceof Taller ? ActividadType.TALLER
																	 : (actividad instanceof Charla ? ActividadType.CHARLA
																	 : (actividad instanceof Asesoramiento ? ActividadType.ASESORAMIENTO :
																	 : (actividad instanceof ActividadExterna ? ActividadType.EXTERNA : null)))));
				  item.setActividad(actividad);*/
		  
				  acis.add(item);
			  }

			  c.set(Calendar.DATE, c.get(Calendar.DATE) + 1);
		  }
	  }
	
	  return acis;
  }
}
