package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.Date;
import java.util.List;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.evaluacion.entity.Formulario;
import es.csa.fdt.cyldigital.servicios.entity.Actividad;
import es.csa.fdt.cyldigital.servicios.entity.CursoOnline;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCursoOnline;
import es.csa.fdt.cyldigital.servicios.entity.ObservacionActividad;
import es.csa.fdt.cyldigital.servicios.entity.PreinscripcionCursoOnline;
import es.csa.fdt.cyldigital.servicios.entity.ServicioCursoOnline;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioOnline;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado.Puesto;


/**
 * Clase CursoOnlineServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class CursoOnlineServiceImpl extends AbstractServiceImpl<CursoOnline, Long> implements CursoOnlineService {

  private List<Formulario> formularios;

  private static final long serialVersionUID = -5575228180361917674L;

  private boolean avisoFechaMostrado = false;

  private boolean borrarPreinscripcion = false;

  private boolean notasCorrectas = false;

  /**
   * Construye un nuevo objeto de la clase
   */
  public CursoOnlineServiceImpl() {
    super();
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.CursoOnlineService#persistCurso(es.csa.fdt.cyldigital.servicios.entity.CursoOnline)
   */
  public boolean persistCurso(CursoOnline curso) {
    for (Actividad temp : curso.getServicio().getActividades()){
      CursoOnline cTemp = (CursoOnline) temp;
      if (cTemp.getId() != curso.getId() && (cTemp.getFechaInicio().after(curso.getFechaFin()) || cTemp.getFechaFin().before(curso.getFechaInicio()))) {
        addWarnMessage("actividad.validator.cursosSimultaneos");
        return false;
      }
    }
    
    if (curso.getFechaInicio().getTime() < (new Date()).getTime() && !avisoFechaMostrado && !curso.isCerrada()) {
      addWarnMessage("actividad.validator.fechaInicioAnterior");
      avisoFechaMostrado = true;
    } else {
      if (curso.isCerrada() && curso.isMatriculasEnReserva()) {
        for (MatriculaCursoOnline m : curso.getMatriculas()) {
          if (m.isEnListaEspera()) {
            preinscripcionFromMatriculaReserva(curso, m.getId());
          }
        }
      }
      persist(curso);
      return true;
    }
    return false;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.CursoOnlineService#removeCurso(es.csa.fdt.cyldigital.servicios.entity.CursoOnline)
   */
  public boolean removeCurso(CursoOnline curso) {
    if (curso.getId() != 0) {
      if (!curso.getProfesores().isEmpty()) {
        addWarnMessage("Antes de borrar el curso online, quite todos los profesores asociados.");
        return false;
      }
      remove(curso);
      return true;
    }
    return false;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.CursoOnlineService#getServiciosDisponibles()
   */
  public List<ServicioCursoOnline> getServiciosDisponibles() {
    return findAllOrder(ServicioCursoOnline.class, "nombre", true);
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.CursoOnlineService#getProfesores()
   */
  public List<Empleado> getProfesores() {
    return findWhereOrder(Empleado.class, "puesto", Puesto.ONLINE_TRAINER, "nombre", true);
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.CursoOnlineService#getFormularios()
   */
  public List<Formulario> getFormularios() {
    if (formularios == null) formularios = findAllOrder(Formulario.class, "nombre", true);
    return formularios;
  }

  /**
   * Checks if is borrar preinscripcion.
   * 
   * @return true, if is borrar preinscripcion
   */
  public boolean isBorrarPreinscripcion() {
    return borrarPreinscripcion;
  }

  /**
   * Asigna borrar preinscripcion.
   * 
   * @param borrarPreinscripcion
   *          borrar preinscripcion
   */
  public void setBorrarPreinscripcion(boolean borrarPreinscripcion) {
    this.borrarPreinscripcion = borrarPreinscripcion;
  }

  /**
   * Rechazar curso.
   * 
   * @param curso
   *           curso
   * @param idPreinscripcion
   *           id preinscripcion
   */
  public void rechazarCurso(CursoOnline curso, Long idPreinscripcion) {
    if (borrarPreinscripcion) {
      PreinscripcionCursoOnline p = find(PreinscripcionCursoOnline.class, idPreinscripcion);
      if (p != null) remove(p);
    } else {
      PreinscripcionCursoOnline p = find(PreinscripcionCursoOnline.class, idPreinscripcion);
      if (p != null) {
        p.getCursosRechazos().add(curso);
        p.setPeso(p.getPeso() + 1);
        persist(p);
      }
    }
  }

  /**
   * Matricula from preinscripcion.
   * 
   * @param curso
   *           curso
   * @param idPreinscripcion
   *           id preinscripcion
   * @return true, if successful
   */
  public boolean matriculaFromPreinscripcion(CursoOnline curso, Long idPreinscripcion) {
    PreinscripcionCursoOnline p = find(PreinscripcionCursoOnline.class, idPreinscripcion);
    boolean flag = false;
    if (p != null) {
      Long idUsuario = p.getUsuario().getId();
      UsuarioOnline u = find(UsuarioOnline.class, idUsuario);

      for (MatriculaCursoOnline matr : curso.getMatriculas()) {
        if (matr.getUsuario().getId() == idUsuario) {
          flag = true;
          addErrorMessage("El usuario ya está matriculado en este curso");
        }
      }

      if (!flag) {
        ((ServicioCursoOnline) curso.getServicio()).getPreinscripciones().remove(p);
        u.getPreinscripcionesCursos().remove(p);

        MatriculaCursoOnline m = new MatriculaCursoOnline();
        m.setAdmitido(true);
        m.setDiplomaCreado(false);
        m.setFechaMatricula(new Date());

        m.setCurso(curso);
        m.setUsuario(u);
        persist(m);

        setMessagesEnabled(false);
        remove(p);
        setMessagesEnabled(true);
        return true;
      }
    }
    return false;
  }

  /**
   * Preinscripcion from matricula reserva.
   * 
   * @param curso
   *           curso
   * @param idMatricula
   *           id matricula
   * @return true, if successful
   */
  public boolean preinscripcionFromMatriculaReserva(CursoOnline curso, Long idMatricula) {
    MatriculaCursoOnline m = find(MatriculaCursoOnline.class, idMatricula);
    if (m != null) {
      Long idUsuario = m.getUsuario().getId();
      UsuarioOnline u = find(UsuarioOnline.class, idUsuario);

      curso.getMatriculas().remove(m);
      u.getMatriculasCursos().remove(m);

      PreinscripcionCursoOnline p = new PreinscripcionCursoOnline();
      p.setPeso(0);
      p.setFecha(new Date());

      p.setServicioCurso((ServicioCursoOnline) curso.getServicio());
      p.setUsuario(m.getUsuario());
      persist(p);

      setMessagesEnabled(false);
      remove(m);
      setMessagesEnabled(true);
      return true;
    }
    return false;
  }

  /**
   * Anadir observacion guion.
   * 
   * @param curso
   *           curso
   * @param o
   *           o
   * @param usuario
   *           usuario
   */
  public void anadirObservacionGuion(CursoOnline curso, ObservacionActividad o, Usuario usuario) {
    if (o.getObservacion().equals("")) addWarnMessage("No puede añadir un comentario vacío");
    else {
      o.setFecha(new Date());
      o.setUsuario(usuario);
      o.setActividad(curso);
      curso.getObservacionesActividad().add(o);
      persist(curso);
    }
  }

  /**
   * Devuelve nueva observacion guion.
   * 
   * @return nueva observacion guion
   */
  public ObservacionActividad getNuevaObservacionGuion() {
    return new ObservacionActividad();
  }

  /**
   * Devuelve next curso.
   * 
   * @param u
   *           u
   * @param curso
   *           curso
   * @param next
   *           next
   * @return next curso
   */
  public CursoOnline getNextCurso(UsuarioLdap u, CursoOnline curso, boolean next) {
    /*
     * next=false: devuelve, si hay, la incidencia anterior. A true, la posterior. Recordar que el curso anterior es de
     * fecha posterior al actual. Devuelve null si no hay una incidencia anterior/posterior, si la hay, devuelve su id
     */
    List<CursoOnline> cursos = findByQuery(CursoOnline.class,
        " Select c from es.csa.fdt.cyldigital.servicios.entity.CursoOnline c  order by fechaInicio desc");

    if (next) {
      if (cursos.indexOf(curso) + 1 < cursos.size()) return cursos.get(cursos.indexOf(curso) + 1);
    } else {
      if (cursos.indexOf(curso) != 0) return cursos.get(cursos.indexOf(curso) - 1);
    }
    return null;
  }

  /**
   * Notas puestas.
   * 
   * @param curso
   *           curso
   */
  public void notasPuestas(CursoOnline curso) {
    int i = 0;
    if (curso.getMatriculas() != null) for (MatriculaCursoOnline m : curso.getMatriculas()) {
      if (m.isAdmitido() && m.getNota() == null) i++;
    }
    if (i > 0) setNotasCorrectas(false);
    else setNotasCorrectas(true);
  }

  /**
   * Asigna notas correctas.
   * 
   * @param notasCorrectas
   *          notas correctas
   */
  public void setNotasCorrectas(boolean notasCorrectas) {
    this.notasCorrectas = notasCorrectas;
  }

  /**
   * Checks if is notas correctas.
   * 
   * @return true, if is notas correctas
   */
  public boolean isNotasCorrectas() {
    return notasCorrectas;
  }

  /**
   * Añade profesor.
   * 
   * @param empl
   *           empl
   * @param curso
   *           curso
   */
  public void addProfesor(Empleado empl, CursoOnline curso) {
    if (!curso.getProfesores().contains(empl)) curso.getProfesores().add(empl);
  }

  /**
   * Quitar profesor.
   * 
   * @param empl
   *           empl
   * @param curso
   *           curso
   */
  public void quitarProfesor(Empleado empl, CursoOnline curso) {
    curso.getProfesores().remove(empl);
  }

  /**
   * Quitar todos profesores.
   * 
   * @param curso
   *           curso
   */
  public void quitarTodosProfesores(CursoOnline curso) {
    curso.getProfesores().removeAll(curso.getProfesores());
  }
}
