package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.List;

import es.csa.fdt.cyldigital.calendario.service.ActividadCalendarItemServiceImpl;
import es.csa.fdt.cyldigital.centros.entity.Aula;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.service.AbstractService;
import es.csa.fdt.cyldigital.evaluacion.entity.Formulario;
import es.csa.fdt.cyldigital.servicios.entity.Curso;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso;
import es.csa.fdt.cyldigital.servicios.entity.ServicioCurso;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.util.ActivitySchedule;

/**
 * Interfaz del servicio de Curso.
 * 
 * @author csa
 */
public interface CursoService extends AbstractService<Curso, Long> {

	/**
	 * Persist curso.
	 * 
	 * @param curso
	 *            curso
	 * @param serviceCal
	 *            service cal
	 * @return true, if successful
	 */
	public boolean persistCurso(Curso curso, List<ActivitySchedule> schedules, ActividadCalendarItemServiceImpl serviceCal);
	
	/**
	 * Persist multiple curso
	 * 
	 * @param cursoMultiple
	 * 			curso to multiply
	 * 
	 * @param schedules
	 * 			schedules for each curso
	 * 
	 * @param serviceCal
	 * 			Calendar Service
	 * 
	 * @return List of completed cursos
	 */
	public List<Curso> persistCursoMultiple(Curso cursoMultiple, List<ActivitySchedule> schedules, ActividadCalendarItemServiceImpl serviceCal);

	/**
	 * Elimina curso.
	 * 
	 * @param curso
	 *            curso
	 */
	public void removeCurso(Curso curso);

	/**
	 * Devuelve servicios disponibles.
	 * 
	 * @return servicios disponibles
	 */
	public List<ServicioCurso> getServiciosDisponibles();

	/**
	 * Devuelve centros.
	 * 
	 * @return centros
	 */
	public List<Centro> getCentros();

	/**
	 * Devuelve profesores.
	 * 
	 * @return profesores
	 */
	public List<Empleado> getProfesores();

	/**
	 * Devuelve aulas.
	 * 
	 * @return aulas
	 */
	public List<Aula> getAulas();

	/**
	 * Devuelve formularios.
	 * 
	 * @return formularios
	 */
	public List<Formulario> getFormularios();

	/**
	 * Comprobar edad usuario.
	 * 
	 * @param m
	 *            m
	 */
	public void comprobarEdadUsuario(MatriculaCurso m);
}
