package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerias del nucleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.faces.model.SelectItem;
import javax.persistence.NoResultException;
import javax.persistence.Query;

import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.calendario.entity.ActividadCalendarItem;
import es.csa.fdt.cyldigital.calendario.service.ActividadCalendarItemServiceImpl;

import es.csa.fdt.cyldigital.centros.entity.Aula;
import es.csa.fdt.cyldigital.centros.entity.Centro;

import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.diplomas.entity.Diploma;
import es.csa.fdt.cyldigital.evaluacion.entity.Examen;
import es.csa.fdt.cyldigital.evaluacion.entity.Formulario;

import es.csa.fdt.cyldigital.language.entity.ActividadLanguage;
import es.csa.fdt.cyldigital.language.entity.CursoLanguage;
import es.csa.fdt.cyldigital.language.entity.FormularioLanguage;
import es.csa.fdt.cyldigital.language.entity.Language;
import es.csa.fdt.cyldigital.language.entity.ServicioCursoLanguage;
import es.csa.fdt.cyldigital.notificacion.NotificationService;

import es.csa.fdt.cyldigital.recursos.entity.Recurso;

import es.csa.fdt.cyldigital.servicios.entity.Actividad;
import es.csa.fdt.cyldigital.servicios.entity.Curso;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso;
import es.csa.fdt.cyldigital.servicios.entity.ServicioCurso;
import es.csa.fdt.cyldigital.servicios.entity.Taller;
import es.csa.fdt.cyldigital.servicios.entity.Tematica;

import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;

import es.csa.fdt.cyldigital.util.ActivitySchedule;

/**
 * Clase CursoServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class CursoServiceImpl extends AbstractServiceImpl<Curso, Long> implements CursoService {
	private static final long serialVersionUID = -5575228180361917674L;

	private List<Centro> centros;

	private Centro centro;

	private List<Centro> centrosCursoMultiple;
	
	private Tematica tematica;

	private boolean changeCentro = true;

	private List<Empleado> profesores;

	private List<Aula> aulas;

	private List<Formulario> formularios;
   
    private List<SelectItem> serviciosSelectItems;
	
	private boolean avisoFechaMostrado = false;

	private boolean avisoRecursos = false;

	private boolean closable = false;

	@Resource
	private transient HashMap<String, Object> generalProperties;
	
	/**
	 * Construye un nuevo objeto de la clase
	 */
	public CursoServiceImpl() {
		super();
	}

	/**
	 * Devuelve centro.
	 * 
	 * @return centro
	 */
	public Centro getCentro() {
		return centro;
	}

	/**
	 * Asigna centro.
	 * 
	 * @param centro
	 *            centro
	 */
	public void setCentro(Centro centro) {
		this.centro = centro;
	}

	/**
	 * Apply centro.
	 * 
	 * @param curso
	 *            curso
	 */
	public void applyCentro(Curso curso) {
		this.centro = curso.getCentro();
		curso.setAula(null);
		changeCentro = true;
	}

	public Tematica getTematica() {
		return tematica;
	}
	
	public void setTematica(Tematica tematica) {
		this.tematica = tematica;
	}
	
	public void applyTematica(Tematica tematica){
		this.tematica = tematica;
	}	
	
	/**
	 * Devuelve centrosCursoMultiple.
	 * 
	 * @return centrosCursoMultiple
	 */
	public List<Centro> getCentrosCursoMultiple() {
		return centrosCursoMultiple;
	}

	/**
	 * Asigna centrosCursoMultiple.
	 * 
	 * @param centrosCursoMultiple
	 *            centrosCursoMultiple
	 */
	public void setCentrosCursoMultiple(List<Centro> centrosCursoMultiple) {
		this.centrosCursoMultiple = centrosCursoMultiple;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * es.csa.fdt.cyldigital.servicios.service.CursoService#createLanguages(es.csa.fdt.cyldigital.servicios.entity.Curso,
	 *																		List<es.csa.fdt.cyldigital.language.entity.Language> languages)
	 */
	public void createLanguages(Curso curso, List<Language> languages) {
		List<ActividadLanguage> textos = new ArrayList<ActividadLanguage>();
		List<CursoLanguage> textosCurso = new ArrayList<CursoLanguage>();

		for (Language language : languages) {
			CursoLanguage textoCurso = new CursoLanguage();
	
			textoCurso.setLanguage(language);
			textoCurso.setCurso(curso);

			textos.add((ActividadLanguage) textoCurso);
			textosCurso.add(textoCurso);
		}

		curso.setTextosCurso(textosCurso);
		curso.setTextos(textos);
	}
	
	/**
	 * Clonado de un curso
	 */
	public Curso cloneCurso(Long idCursoOriginal, Curso cd, UsuarioLdap currentUser) {
		if (idCursoOriginal != null) {
			try {
				Curso co = find(idCursoOriginal);
				
				if (co != null) {
					for (CursoLanguage col : co.getTextosCurso()) {
						for (CursoLanguage cdl : cd.getTextosCurso()) {
							if (cdl.getLanguage().getISO().compareTo(col.getLanguage().getISO()) == 0) {
								cdl.setApodo(col.getApodo());
								//3.4.11 - RF_15_52 Añadir guion curso
								cdl.setGuion(col.getGuion());
							}
						}
					}
					
					if (!co.getServicio().getDesactivado())
						cd.setServicio(co.getServicio());
					
					cd.setCuestionarioSatisfaccion(co.getCuestionarioSatisfaccion());
					cd.setHayExamenInicial(co.isHayExamenInicial());
					//RF_XEA_15 Añadir el campo esPruebaNivelObligatoria en el clonado de un curso
					cd.setEsPruebaNivelObligatoria(co.isEsPruebaNivelObligatoria());
					cd.setNumeroPlazas(co.getNumeroPlazas());
					
					//RF_??_?? Clonado de actividades
					cd.setObservaciones(co.getObservaciones());	
					if(co.getCentro().getId() == currentUser.getCentro().getId()) {
						cd.setProfesor(co.getProfesor());
						cd.setAula(co.getAula());						
					}
					cd.setVideoconferencia(co.isVideoconferencia());
					cd.setIniciativaSingular(co.isIniciativaSingular());
					cd.setFechaPublicacion(co.getFechaPublicacion());			
					cd.setHoraPublicacion(co.getHoraPublicacion());
					cd.setFechaLimiteInscripcion(co.getFechaLimiteInscripcion());
					cd.setHoraLimiteInscripcion(co.getHoraLimiteInscripcion());
					cd.setFechaInicio(co.getFechaInicio());
					cd.setHoraInicio(co.getHoraInicio());										
					cd.setFechaFin(co.getFechaFin());
					cd.setHoraFin(co.getHoraFin());
									
					List<Recurso> recursos = new ArrayList<Recurso>(); 
					//recorrer recursos y asignar sólo si pertenecen al centro del usuario autenticado
					for (Recurso rec: co.getRecursos()) {
						if(rec.getCentro().getId() == currentUser.getCentro().getId()){
							Recurso re = find(Recurso.class, rec.getId());
							recursos.add(re);
						}
					}
					cd.setRecursos(recursos);
					
					cd.setExamen(co.getExamen());
					cd.setExamenNivel(co.getExamenNivel());
					cd.setPlantillaDiploma(co.getPlantillaDiploma());
					cd.setNumeroHoras(co.getNumeroHoras());
					//cd.setEsClonado(true);
					
					// PR-XEA --> NO SE CLONA EL CALENDARIO
					/*List<ActividadCalendarItem> acis = new ArrayList<ActividadCalendarItem>();
					for (ActividadCalendarItem aci: co.getCalendarItems()) {
						ActividadCalendarItem ac = new ActividadCalendarItem();
						ac.setActividadType(aci.getActividadType());
						ac.setAllDay(aci.isAllDay());
						ac.setClassName(aci.getClassName());
						ac.setDefaultDescription(aci.getDefaultDescription());
						ac.setDefaultTitle(aci.getDefaultTitle());
						ac.setDesactivado(aci.getDesactivado());
						ac.setEnd(aci.getEnd());
						ac.setExcepcion(aci.getExcepcion());
						ac.setStart(aci.getStart());
						ac.setTipo(aci.getTipo());
						ac.setActividad(cd);					
						acis.add(ac);
					} 
					cd.setCalendarItems(acis);
					//RF_Clonar cursos	*/

				}
			}
			catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		return cd;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see es.csa.fdt.cyldigital.servicios.service.CursoService#persistCurso(es.csa.fdt.cyldigital.servicios.entity.Curso,
	 *																		  es.csa.fdt.cyldigital.calendario.service.ActividadCalendarItemServiceImpl)
	 */
	public boolean persistCurso(Curso curso, List<ActivitySchedule> schedules, ActividadCalendarItemServiceImpl serviceCal) {
		Boolean error = false;
		
		if (curso.getId() <= 0) {
			for (ActivitySchedule as : schedules) {
				if (as.getStartDate().getTime() > as.getEndDate().getTime()) {
					addErrorMessage("actividad.validator.fechaInicioMayorFechaFin");
		
					error = true;
					
					break;
				}
		
				if (as.getStartTime().getTime() >= as.getEndTime().getTime()) {
					addErrorMessage("actividad.validator.horaFinMenorOIgualHoraInicio");
		
					error = true;
					
					break;
				}
			}
			
			Long durationInDays = Long.valueOf(serviceCal.schedulesDurationInDays(schedules));
			
			if (durationInDays <= 0L) {
				addErrorMessage("actividad.validator.schedulesNotProducesDays");
				
				error = true;
			}
			/* RF_15_06 Se elimina la limitación de mas de 1 día 
			else if (durationInDays == 1L) {
				addErrorMessage("actividad.validator.fechaInicioIgualFechaFin");
				
				error = true;
			}
			*/
			

			if (serviceCal.hasSchedulesCollision(schedules)) {
				addErrorMessage("actividad.validator.schedulesCollision");

				error = true;
			}
		}
		else { //if (curso.getId() > 0)
			/* RF_15_06 Se elimina la limitación de mas de 1 día
			if (curso.getFechaInicio().getTime() == curso.getFechaFin().getTime()) {
				addErrorMessage("actividad.validator.fechaInicioIgualFechaFin");
	
				error = true;
			}
			*/
	
			if (curso.getFechaInicio().getTime() > curso.getFechaFin().getTime()) {
				addErrorMessage("actividad.validator.fechaInicioMayorFechaFin");
	
				error = true;
			}
	
			if (curso.getHoraInicio().getTime() >= curso.getHoraFin().getTime()) {
				addErrorMessage("actividad.validator.horaFinMenorOIgualHoraInicio");
	
				error = true;
			}
		}
		
		if (!error) {
			boolean warning = false;
			
			if ((curso.getId() <= 0) && !avisoFechaMostrado && !curso.isCerrada()) {
				for (ActivitySchedule as : schedules) {
					if (as.getStartDate().getTime() < (new Date()).getTime()) {
						warning = true;

						break;
					}
				}
			}

			if ((((curso.getId() > 0)  && (curso.getFechaInicio().getTime() < (new Date()).getTime())) ||
				 ((curso.getId() <= 0) && warning)) &&
				!avisoFechaMostrado &&
				!curso.isCerrada()) {
				addWarnMessage("actividad.validator.fechaInicioAnterior");

				avisoFechaMostrado = true;
			}
			/*  se anula la verificacion del numero de recursos
			else if (!avisoRecursos && (curso.getNumeroPlazas() > curso.getRecursos().size())) {
				addWarnMessage("actividad.validator.plazasMayorRecursos");

				avisoRecursos = true;
			}*/
			else {
				if ((curso.getId() > 0) && curso.getCalendarItems().isEmpty())
					serviceCal.generateCalendar(curso.getId());

				persist(curso);

				return true;
			}
		}

		return false;
	}

	
	/* (non-Javadoc)
	 * @see es.csa.fdt.cyldigital.servicios.service.CursoService#persistCursoMultiple(es.csa.fdt.cyldigital.servicios.entity.Curso,
	 * 																				  java.util.List,
	 *																				  es.csa.fdt.cyldigital.calendario.service.ActividadCalendarItemServiceImpl)
	 */
	public List<Curso> persistCursoMultiple(Curso cursoMultiple, List<ActivitySchedule> schedules,
											ActividadCalendarItemServiceImpl serviceCal) {
		Boolean error = false;
		
		for (ActivitySchedule as : schedules) {
			if (as.getStartDate().getTime() > as.getEndDate().getTime()) {
				addErrorMessage("actividad.validator.fechaInicioMayorFechaFin");
				error = true;
				break;
			}
			
			if (as.getStartTime().getTime() >= as.getEndTime().getTime()) {
				addErrorMessage("actividad.validator.horaFinMenorOIgualHoraInicio");
				error = true;
				break;
			}
		}
		
		Long durationInDays = Long.valueOf(serviceCal.schedulesDurationInDays(schedules));
		
		if (durationInDays <= 0L) {
			addErrorMessage("actividad.validator.schedulesNotProducesDays");	
			error = true;
		}
		/* RF_15_06 Se elimina la limitación de mas de 1 día
		else if (durationInDays == 1L) {
			addErrorMessage("actividad.validator.fechaInicioIgualFechaFin");
			
			error = true;
		}
		*/
		
		if (serviceCal.hasSchedulesCollision(schedules)) {
			addErrorMessage("actividad.validator.schedulesCollision");
			error = true;
		}

		if (!error) {
			boolean warning = false;
			if ((cursoMultiple.getId() <= 0) && !avisoFechaMostrado && !cursoMultiple.isCerrada()) {
				for (ActivitySchedule as : schedules) {
					if (as.getStartDate().getTime() < (new Date()).getTime()) {
						warning = true;
						break;
					}
				}
			}
			
			if (((cursoMultiple.getId() <= 0) && warning) &&
				!avisoFechaMostrado &&
				!cursoMultiple.isCerrada()) {
				addWarnMessage("actividad.validator.fechaInicioAnterior");
				avisoFechaMostrado = true;
			} else {
				List<Curso> listaCursos = new ArrayList<Curso>();

				for (Centro c : centrosCursoMultiple) {
					Curso curso = new Curso();

					curso.setServicio(cursoMultiple.getServicio());
					
					//3.4.11 - RF_15_25 - Configuración de cursos
					curso.setExamen(cursoMultiple.getExamen());
					curso.setExamenNivel(cursoMultiple.getExamenNivel());
					curso.setPlantillaDiploma(cursoMultiple.getPlantillaDiploma());
					
					curso.setCentro(c);
					curso.setCuestionarioSatisfaccion(cursoMultiple.getCuestionarioSatisfaccion());
					curso.setNumeroPlazas(cursoMultiple.getNumeroPlazas());
					curso.setHayExamenInicial(cursoMultiple.isHayExamenInicial());
					//RF_XEA_15 Se añade en el alta de un curso multiple el campo de si la prueba de nivel es obligatoria o no
					curso.setEsPruebaNivelObligatoria(cursoMultiple.isEsPruebaNivelObligatoria());
					curso.setObservaciones(cursoMultiple.getObservaciones());
					
					//RF_XEA_22
					curso.setFechaPublicacion(cursoMultiple.getFechaPublicacion());
					curso.setHoraPublicacion(cursoMultiple.getHoraPublicacion());
					//RF_XEA_22

  				    //RF_15_04
					curso.setFechaLimiteInscripcion(cursoMultiple.getFechaLimiteInscripcion());
					curso.setHoraLimiteInscripcion(cursoMultiple.getHoraLimiteInscripcion());
  				    //RF_15_04

					curso.setTextos(new ArrayList<ActividadLanguage>());
					curso.setTextosCurso(new ArrayList<CursoLanguage>());

					for (CursoLanguage textoCursoMultiple : cursoMultiple.getTextosCurso()) {
						CursoLanguage textoCurso = new CursoLanguage();

						textoCurso.setCurso(curso);
						textoCurso.setLanguage(textoCursoMultiple.getLanguage());
						textoCurso.setApodo(textoCursoMultiple.getApodo());
	  				    //RF_15_52
						textoCurso.setGuion(textoCursoMultiple.getGuion());

						curso.getTextos().add((ActividadLanguage) textoCurso);
						curso.getTextosCurso().add(textoCurso);
					}

					persist(curso);

					listaCursos.add(curso);
				}

				return listaCursos;
			}
		}

		return null;
	}

	/**
	 * Persist curso to calendario.
	 * 
	 * @param curso
	 *            curso
	 * @return true, if successful
	 */
	public boolean persistCursoToCalendario(Curso curso) {
		if (curso.getNumeroHoras() == 0) {
			curso.setNumeroHoras(2);
		}

		if (curso.getFechaInicio().getTime() < (new Date()).getTime() && !avisoFechaMostrado && !curso.isCerrada()) {
			addWarnMessage("actividad.validator.fechaInicioAnterior");
			
			avisoFechaMostrado = true;
		}
		else {
			setMessagesEnabled(false);
			persist(curso);
			setMessagesEnabled(true);

			return true;
		}

		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see es.csa.fdt.cyldigital.servicios.service.CursoService#removeCurso(es.csa.fdt.cyldigital.servicios.entity.Curso)
	 */
	public void removeCurso(Curso curso) {
		if (curso.getId() != 0)
			remove(curso);
	}

	/**
	 * Elimina curso from cancel.
	 * 
	 * @param curso
	 *            curso
	 */
	public void removeCursoFromCancel(Curso curso) {
		if (curso.getId() != 0) {
			setMessagesEnabled(false);
			remove(curso);
			setMessagesEnabled(true);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see es.csa.fdt.cyldigital.servicios.service.CursoService#getServiciosLanguageDisponibles()
	 */
	public List<ServicioCursoLanguage> getServiciosLanguageDisponibles() {
		String idioma = LocaleContextHolder.getLocaleContext().getLocale().toString();

		Map<String, Object> properties = new HashMap<String, Object>();
		properties.put("language.ISO", idioma);
		properties.put("desactivado", false);
		properties.put("servicioCurso.desactivado", false);

		return findWhereOrder(ServicioCursoLanguage.class, properties, "nombre", true);
	}

	/**
	 * 
	 * @return the servicios select item list
	 */
	public List<SelectItem> getServiciosSelectItems() {
		if (serviciosSelectItems == null)
			serviciosSelectItems = new ArrayList<SelectItem>();
		  
		if (!serviciosSelectItems.isEmpty())
			serviciosSelectItems.clear();
		  
		Map<String, Object> propiedades = new HashMap<String, Object>();
		String idioma = LocaleContextHolder.getLocaleContext().getLocale().toString();
		
		propiedades.put("language.ISO", idioma);
		propiedades.put("desactivado", false);
		propiedades.put("servicioCurso.desactivado", false);
		if(this.tematica!=null){
			propiedades.put("servicio.tematica", tematica);
		}

		List<ServicioCursoLanguage> serviciosCursoLanguage = findWhereOrder(ServicioCursoLanguage.class, propiedades, "nombre", true);
		
		for (ServicioCursoLanguage servicioCursoLanguage : serviciosCursoLanguage) {
			serviciosSelectItems.add(new SelectItem(servicioCursoLanguage.getServicioCurso(), servicioCursoLanguage.getNombre()));
		}

		return serviciosSelectItems;
	}
	
		
	/*
	 * (non-Javadoc)
	 * 
	 * @see es.csa.fdt.cyldigital.servicios.service.CursoService#getServiciosDisponibles()
	 */
	public List<ServicioCurso> getServiciosDisponibles() {
		List<ServicioCurso> serviciosCurso = new ArrayList<ServicioCurso>();
		List<ServicioCursoLanguage> serviciosCursoLanguage;

		serviciosCursoLanguage = getServiciosLanguageDisponibles();

		for (ServicioCursoLanguage servicioCursoLanguage : serviciosCursoLanguage)
			serviciosCurso.add(servicioCursoLanguage.getServicioCurso());

		return serviciosCurso;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see es.csa.fdt.cyldigital.servicios.service.CursoService#getCentros()
	 */
	public List<Centro> getCentros() {
		if (centros == null)
			centros = findAllOrder(Centro.class, "nombre", true);

		return centros;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see es.csa.fdt.cyldigital.servicios.service.CursoService#getProfesores()
	 */
	public List<Empleado> getProfesores() {
		if (centro != null && changeCentro) {
			Map<String, Object> Propiedades = new HashMap<String, Object>();
			Propiedades.put("centro.id", centro.getId());
			Propiedades.put("desactivado", false);

			List<Empleado> empleados = findWhereOrder(Empleado.class, Propiedades, "nombre", true);

			profesores = new ArrayList<Empleado>();
			
			for (Empleado e : empleados) {
				if (e.isProfesor())
					profesores.add(e);
			}
			
			changeCentro = false;
		}

		return profesores;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see es.csa.fdt.cyldigital.servicios.service.CursoService#getAulas()
	 */
	public List<Aula> getAulas() {
		if (aulas == null)
			aulas = findAllOrder(Aula.class, "nombre", true);

		return aulas;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see es.csa.fdt.cyldigital.servicios.service.CursoService#getNombreCurso()
	 */
	public String getNombreCurso(Long idCurso) {
		return getApodoCurso(idCurso);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see es.csa.fdt.cyldigital.servicios.service.CursoService#getApodoCurso()
	 */
	public String getApodoCurso(Long idCurso) {
		String idioma = LocaleContextHolder.getLocaleContext().getLocale().toString();

		HashMap<String, Object> propiedades = new HashMap<String, Object>();
		propiedades.put("language.ISO", idioma);
		propiedades.put("curso.id", idCurso);

		CursoLanguage cursoLanguage = findOneWhere(CursoLanguage.class, propiedades);

		if (cursoLanguage != null)
			return cursoLanguage.getApodo();
		else
			return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see es.csa.fdt.cyldigital.servicios.service.CursoService#getFormulariosLanguage()
	 */
	public List<FormularioLanguage> getFormulariosLanguage() {
		String idioma = LocaleContextHolder.getLocaleContext().getLocale().toString();

		return findWhereOrder(FormularioLanguage.class, "language.ISO", idioma, "nombre", true);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see es.csa.fdt.cyldigital.servicios.service.CursoService#getFormularios()
	 */
	public List<Formulario> getFormularios() {
		List<FormularioLanguage> formulariosLanguage;
		
		if (formularios == null)
			formularios = new ArrayList<Formulario>();
		formularios.clear();

		formulariosLanguage = getFormulariosLanguage();

		for (FormularioLanguage formularioLanguage : formulariosLanguage)
			formularios.add(formularioLanguage.getFormulario());

		return formularios;
	}

	/**
	 * Comprueba la lista de planificaciones y crea una nueva con un elemento si no no existe o está vacía.
	 * 
	 * @return the schedules
	 */
	public List<ActivitySchedule> getSchedulesList(List<ActivitySchedule> las) {
		if (las == null)
			las = new ArrayList<ActivitySchedule>();
		
		if (las.isEmpty())
			addNewSchedule(las);
		
		return las;
	}

	/**
	 * Actualiza centro.
	 * 
	 * @param curso
	 *            curso
	 * @param currentUser
	 *            current user
	 */
	public void actualizaCentro(Curso curso, UsuarioLdap currentUser) {
		centro = curso.getCentro() != null ? curso.getCentro() : null;

		if (currentUser.getCentro() != null) {
			Centro c = find(Centro.class, currentUser.getCentro().getId());

			curso.setCentro(c);

			centro = c;
			changeCentro = true;
		}
		else if (generalProperties.containsKey("DEFAULT_CENTER_ID") && curso.getCentro() == null) {
			Centro c = find(Centro.class, Long.valueOf((String) generalProperties.get("DEFAULT_CENTER_ID")));

			if (c != null) {
				curso.setCentro(c);

				centro = c;
				changeCentro = true;
			}
		}
	}

	/**
	 * Actualiza aula.
	 * 
	 * @param curso
	 *            curso
	 */
	public void actualizaAula(Curso curso) {
		centro = curso.getCentro() != null ? curso.getCentro() : null;

		if (centro != null) {
			for (Aula a : centro.getAulas()) {
				if (a.getNombre().toLowerCase().contains("formaci")) {
					curso.setAula(a);
					
					return;
				}
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see es.csa.fdt.cyldigital.servicios.service.CursoService#comprobarEdadUsuario(es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso)
	 */
	public void comprobarEdadUsuario(MatriculaCurso m) {
		if (m.getCurso() != null && m.getUsuario() != null) {
			UsuarioCyL u = find(UsuarioCyL.class, m.getUsuario().getId());
			
			if ((u.getEdad() < m.getCurso().getServicio().getEdadMinima()) || (u.getEdad() > m.getCurso().getServicio().getEdadMaxima()))
				addWarnMessage("matricula.validator.edad_incorrecta");
		}
	}
	
	/**
	 * Añade una nueva línea de planificación del curso
	 * 
	 * Crea la lista si no existe previamente. 
	 *
	 */
	public void addNewSchedule(List<ActivitySchedule> schedules) {
		if (schedules == null)
			schedules = new ArrayList<ActivitySchedule>();

		schedules.add(new ActivitySchedule());
	}
	
	/**
	 * Añade una nueva línea de planificación del curso
	 * 
	 * Crea la lista si no existe previamente. 
	 *
	 */
	public void removeSchedule(List<ActivitySchedule> schedules, ActivitySchedule schedule) {
		if ((schedule != null) && (schedules != null) && !schedules.isEmpty())
			schedules.remove(schedule);
	}
	
	/**
	 * Check if course is closable by id.
	 * 
	 * @param idCurso
	 *            id of course
	 */
	public boolean checkAndCloseCurso(Long idCurso) {
		if (idCurso != null) {
			clear();
			
			Curso curso = find(idCurso);
			
			checkClosable(curso);
			
			if (isClosable()) {
				curso.setCerrada(true);
				
				disableMessages();
				persist(curso);
				enableMessages();
				
				notificaCursoAutoCerrado(curso);
				
				return true;
			}
		}
		
		return false;
	}
	
	/**
	 * notificación que se ha borrado una matricula de un usuario
	 * 
	 * @param matriculaCurso
	 *           matriculaCurso  
	 * @return true, if successful
	 */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	private void notificaCursoAutoCerrado(Curso curso) {
		NotificationService n = new NotificationService();
		String mailActivate = "";
		String mailRequestenrollmentActivate = "";  

		try {
			mailActivate = findOneWhere(Parameter.class, "nombre", "mail_activate").getValor();
			mailRequestenrollmentActivate = findOneWhere(Parameter.class, "nombre", "mail_autoclosetask_activate").getValor();

			if (mailActivate.equals("1") && mailRequestenrollmentActivate.equals("1"))
				n.notificarCursoAutoCerrado(this, curso);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Check if course is closable by course.
	 * 
	 * @param curso
	 *            curso
	 */
	public void checkClosable(Curso curso) {
		int admittedUsers = 0;
		int withoutScoreOrSurvey = 0;

		if (curso.getMatriculas() != null) {
			for (MatriculaCurso m : curso.getMatriculas()) {
				if (m.isAdmitido()) {
					admittedUsers++;
					
					if ((m.getNota() == null) ||
						((m.getFormularioConcreto() == null) ||
						 (m.getFormularioConcreto().getRespuestas() == null) ||
						 (m.getFormularioConcreto().getRespuestas().size() == 0)))
						withoutScoreOrSurvey++;
				}
			}
		}

		//Si no hay matrículas o
		//	 hay matrículas de usuarios admitidos sin nota o sin encuesta o
		//	 no hay matrículas de usuarios admitidos 
		if ((curso.getMatriculas() == null) || (curso.getMatriculas().isEmpty()) ||
			(withoutScoreOrSurvey > 0) || (admittedUsers == 0))
			setClosable(false);
		else
			setClosable(true);
	}

	/**
	 * Asign closable.
	 * 
	 * @param closable
	 *            closable
	 */
	public void setClosable(boolean closable) {
		this.closable = closable;
	}

	/**
	 * Checks if is closable.
	 * 
	 * @return true, if is closable (regardless dates) 
	 */
	public boolean isClosable() {
		return closable;
	}

	/**
	 * Añade recurso.
	 * 
	 * @param rec
	 *            rec
	 * @param curso
	 *            curso
	 */
	public void addRecurso(Recurso rec, Curso curso) {
		if (rec.isOcupado(curso))
			addWarnMessage("El recurso '" + rec.getNombre() + "' está reservado o se encuentra ocupado en otra actividad");
		else if (!curso.getRecursos().contains(rec))
			curso.getRecursos().add(rec);
	}

	/**
	 * Quitar recurso.
	 * 
	 * @param rec
	 *            rec
	 * @param curso
	 *            curso
	 */
	public void quitarRecurso(Recurso rec, Curso curso) {
		curso.getRecursos().remove(rec);
	}

	/**
	 * Quitar todos recursos.
	 * 
	 * @param curso
	 *            curso
	 */
	public void quitarTodosRecursos(Curso curso) {
		curso.getRecursos().removeAll(curso.getRecursos());
	}

	/**
	 * Añade recursos aula.
	 * 
	 * @param curso
	 *            curso
	 */
	public void addRecursosAula(Curso curso) {
		String message = "";
		int i = 0;

		if ((curso != null) && (curso.getAula() != null) && (curso.getFechaInicio() != null) &&
			(curso.getFechaFin() != null) && (curso.getHoraInicio() != null) && (curso.getHoraFin() != null)) {

			List<ActividadCalendarItem> acis = generateCalendar(curso);

			for (Recurso r : curso.getAula().getRecursos()) {
				if (!r.isOcupado(acis)) {
					addRecurso(r, curso);
					
					i++;
				} else
					message += "(" + r.getNombre() + ") ";
			}

			if (i < curso.getAula().getRecursos().size()) {
				addWarnMessage("actividad.anyadirRecursos.recursosOcupados");
				
				addWarnMessage(message);
			}
		} else
			addErrorMessage("actividad.anyadirRecursosAula.noCalendario");
	}

	public List<ActividadCalendarItem> generateCalendar(Actividad actividad) {
		List<ActividadCalendarItem> acis = new ArrayList<ActividadCalendarItem>();

		ActividadCalendarItem item;

		Calendar c = Calendar.getInstance();
		Calendar tempStart = Calendar.getInstance();
		Calendar tempEnd = Calendar.getInstance();
		Calendar tempTime = Calendar.getInstance();

		if (actividad != null) {
			tempTime.setTime(actividad.getHoraInicio());
			tempStart.setTime(actividad.getFechaInicio());
			
			tempStart.set(Calendar.HOUR, tempTime.get(Calendar.HOUR_OF_DAY));
			tempStart.set(Calendar.MINUTE, tempTime.get(Calendar.MINUTE));

			tempTime.setTime(actividad.getHoraFin());
			tempEnd.setTime(actividad.getFechaFin());
			
			tempEnd.set(Calendar.HOUR, tempTime.get(Calendar.HOUR_OF_DAY));
			tempEnd.set(Calendar.MINUTE, tempTime.get(Calendar.MINUTE));

			c.setTime(tempStart.getTime());

			while (c.before(tempEnd) ||
				   ((c.get(Calendar.DAY_OF_YEAR) <= tempEnd.get(Calendar.DAY_OF_YEAR)) &&
				    (c.get(Calendar.YEAR) == tempEnd.get(Calendar.YEAR)))) {
				
				if (((c.get(Calendar.DAY_OF_WEEK) > 1) && (c.get(Calendar.DAY_OF_WEEK) < 7)) ||
					(actividad instanceof Taller)) {
					item = new ActividadCalendarItem();

					c.set(Calendar.HOUR_OF_DAY, tempStart.get(Calendar.HOUR_OF_DAY));
					c.set(Calendar.MINUTE, tempStart.get(Calendar.MINUTE));
					
					item.setStart(c.getTime());
					
					c.set(Calendar.HOUR_OF_DAY, tempEnd.get(Calendar.HOUR_OF_DAY));
					c.set(Calendar.MINUTE, tempEnd.get(Calendar.MINUTE));
					
					item.setEnd(c.getTime());
					
					item.setAllDay(false);

					acis.add(item);
				}

				c.set(Calendar.DATE, c.get(Calendar.DATE) + 1);
			}
		}

		return acis;
	}
	
	public void cleanCamposServicio(Curso curso){
		curso.setExamenNivel(null);
		curso.setExamen(null);
		curso.setPlantillaDiploma(null);
		//Guion Servicio
		for (CursoLanguage cursoLanguage : curso.getTextosCurso()){
			cursoLanguage.setGuion("");
		}
	}
	
	// RF_15_25 - Configuración de cursos
	public void setConfServicio(Curso curso) {
		
		String strqry = "";
		Query qry = null;
		
		try {
			//ExamenNivel Servicio
			strqry = "SELECT sc.examenNivel AS examenNivel " +
							"FROM " + ServicioCurso.class.getName() + " sc " + 
							"WHERE (sc.id = :idServicio)";
	
			qry = getEntityManager().createQuery(strqry);
			
			qry.setParameter("idServicio", curso.getServicio().getId());
			
			curso.setExamenNivel((Examen) qry.getSingleResult());
			
		} catch (NoResultException nre) {
			curso.setExamenNivel(null);
		}
		
		
		try {
			// Examen Servicio
			strqry = "SELECT sc.examen AS examen " +
					"FROM " + ServicioCurso.class.getName() + " sc " + 
					"WHERE (sc.id = :idServicio)";
			
			qry = getEntityManager().createQuery(strqry);
			
			qry.setParameter("idServicio", curso.getServicio().getId());
			
			curso.setExamen((Examen) qry.getSingleResult());
			
		} catch (NoResultException nre) {
			curso.setExamen(null);
		}
		
		
		try {
			//Plantilla Diploma Servicio
			strqry = "SELECT sc.plantillaDiploma AS plantillaDiploma " +
					"FROM " + ServicioCurso.class.getName() + " sc " + 
					"WHERE (sc.id = :idServicio)";
			
			qry = getEntityManager().createQuery(strqry);
			
			qry.setParameter("idServicio", curso.getServicio().getId());
			
			curso.setPlantillaDiploma((Diploma) qry.getSingleResult());
			
		} catch (NoResultException nre) {
			curso.setPlantillaDiploma(null);
		}			
			
		// 3.4.11 - RF_15_52 Añadir guion curso
		try {
			//Guion Servicio
			List<ServicioCursoLanguage> servicioCursoLanguages = findWhere(ServicioCursoLanguage.class, "servicioCurso.id", curso.getServicio().getId());
				
			List<CursoLanguage> textosCurso = curso.getTextosCurso();
				
			for (ServicioCursoLanguage servicioCursoLanguage : servicioCursoLanguages) {
				for (CursoLanguage cursoLanguage : textosCurso){
					if (servicioCursoLanguage.getLanguage().getISO().equals(cursoLanguage.getLanguage().getISO())){
						cursoLanguage.setGuion(servicioCursoLanguage.getGuion());
					}
				}
			}
							
				
		} catch (NoResultException nre) {
			curso.setTextosCurso(null);
		}
	
	}
}
