package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.Map;

import javax.persistence.NoResultException;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;
import org.springframework.context.i18n.LocaleContextHolder;

import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.notificacion.NotificationService;
import es.csa.fdt.cyldigital.servicios.entity.Charla;
import es.csa.fdt.cyldigital.servicios.entity.Charla.EEstadoMatricula;
import es.csa.fdt.cyldigital.servicios.entity.Curso;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCharla;
import es.csa.fdt.cyldigital.servicios.service.MatriculaCharlaServiceImpl;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario.Role;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;
import es.csa.fdt.cyldigital.language.entity.CharlaLanguage;


/**
 * Clase MatriculaCharlaServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class MatriculaCharlaServiceImpl extends AbstractServiceImpl<MatriculaCharla, Long>
										implements MatriculaCharlaService {

	  private static final long serialVersionUID = -5575228180361917674L;
	  
	  private static final Logger logger = Logger.getLogger(MatriculaCharlaServiceImpl.class.getName());
	  
	  private List<Charla> charlas;

	  private List<Usuario> usuarios;
  
	  private List<MatriculaCharla> matriculaMultipleCharla;

  /**
   * Construye un nuevo objeto de la clase
   */
  public MatriculaCharlaServiceImpl() {
    super();
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaCursoService#persistMatriculaMultipleCurso(java.util.List<es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso>, Long, String)
   */
  public boolean persistMatriculaMultipleCharla(List<MatriculaCharla> matriculas, Long idEmpleado, String observaciones) {
	  Boolean result = true;
	  
	  if (matriculas.size() > 0) {
		  disableMessages();
		  
		  for (MatriculaCharla mc : matriculas) {
			  mc.setObservaciones(observaciones);
			  
			  if (!persistMatriculaCharla(mc, idEmpleado)) {
				  result = false;
				  break;
			  }
		  }
		  
		  enableMessages();
	  }
	  else {
		  addErrorMessage("matricula.validator.usuario_necesario");
		  
		  return false;
	  }

	  return result;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaCharlaService#persistMatriculaCharla(es.csa.fdt.cyldigital.servicios.entity.MatriculaCharla)
   */
  @SuppressWarnings("unchecked")
  public boolean persistMatriculaCharla(MatriculaCharla matriculaCharla, Long idEmpleado) {
    NotificationService n = new NotificationService();
    String sendmail = "", email = "";

    List<Parameter> parameters = findWhere(Parameter.class, "nombre", "mail_activate");
    if (parameters.size() > 0) {
      sendmail = parameters.get(0).getValor();
    }   
       
    if ((isYaMatriculado(matriculaCharla.getUsuario(), matriculaCharla.getId(), matriculaCharla.getCharla())) && (matriculaCharla.getId()== 0))
    {    
    	addErrorMessage("matricula.validator.yainscrito");
    	return false;
    	
    } else if (matriculaCharla.getUsuario() == null) {
      addErrorMessage("matricula.validator.usuario_necesario");
      return false;
    } else {
     /* if (matriculaCharla.getId() == 0) {
        // por último, notificamos si procede.
        if (sendmail.equals("1")) {
          UsuarioCyL u = this.find(UsuarioCyL.class, matriculaCharla.getUsuario().getId());
          if (u.iscomunicacionMail()) // el usuario desea notificaciones.
          {
            email = ((UsuarioCyL) matriculaCharla.getUsuario()).getEmail();
            if (email.isEmpty() || email == null) {
              addWarnMessage("mail.nomailavailable");
            } else // no tenemos el email.
            {
              if (n.notificarSuscritoActividad(this, matriculaCharla.getUsuario().getId(), email, matriculaCharla
                  .getCharla())) addInfoMessage("mail.email");
              else addErrorMessage("mail.errormail");// ha ocurrido algún error.
            }
          } else {// el usuario no desea notificaciones.
            addInfoMessage("mail.noemail");
          }
        } else addInfoMessage("mail.nonotifications");
      }*/
      disableMessages();  
      
      if (matriculaCharla.getId() == 0){
    	  matriculaCharla.setAdmitido(true);
          matriculaCharla.setEnListaEspera(false);
          matriculaCharla.setPreMatriculadoWeb(false);
      }
     
      Empleado empleado = find(Empleado.class, idEmpleado);
      Date fecha = new Date();

      matriculaCharla.setFechaTramite(fecha);
      matriculaCharla.setEmpleadoTramite(empleado);
      
      persist(matriculaCharla);
      
      enableMessages();
      
      //Envio la notificación sea el caso que sea (matriculado por WebAdmin o por WebInscripcion, rechazado o solicitud)
      notificarMatriculadoCharla(matriculaCharla);     
      
      return true;   
    }
  }
  
  /*public void persistPreMatriculaCharla(MatriculaCharla matriculaCharla) {
	  matriculaCharla.setEnListaEspera(true);
	  matriculaCharla.setPrematriculadoWeb(true);
	  matriculaCharla.setAdmitido(false);
	     
	  persist(matriculaCharla);	  
  } 
  
 
  public void persistPreCharlaChanges(Long idCharla, Long idUsuario) {
	  Map<String, Object> propiedades = new HashMap<String, Object>();
	  
	  propiedades.put("charla.id", idCharla);
	  propiedades.put("usuario.id", idUsuario);
	
	  try {
		  MatriculaCharla m = findOneWhere(propiedades);
		  
		  m.setEnListaEspera(false);
		  m.setAdmitido(false);  
		  persist(m);
	  }
	  catch (NoResultException ex) {

	  }
	  catch (Exception e) {
		  logger.log(Level.WARNING, "MatriculaCharlaServiceImpl.persistPreCharlaChanges: Error leyendo matricula de la base de datos. " + e.getLocalizedMessage());
		  
		  e.printStackTrace();
	}
	
  }*/
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaCharlaService#persistPrematriculacion(es.csa.fdt.cyldigital.servicios.entity.MatriculaCharla, es.csa.fdt.cyldigital.servicios.entity.Charla, es.csa.fdt.cyldigital.usuarios.entity.Usuario)
   * 
   * Solo se invoca desde la web de inscripción.
   */
  public boolean persistPrematriculacion(MatriculaCharla matricula, Charla charla, Usuario usuario) {	  
	  if (matricula.getId() == 0) {
		if (usuario == null) {
			addErrorMessage("matricula.validator.usuarioNecesario");
			
			return false;
		}
		else if (charla == null) {
			addErrorMessage("matricula.validator.charlaNecesaria");
			
			return false;
		}		
		else if (isYaMatriculado(usuario, matricula.getId(), charla)) {
			addErrorMessage("matricula.validator.yaMatriculado");
			
			return false;
		}
		else {
			//Todo OK. Se procedede a la prematriculacion WEB


			Map<String, Object> propiedades = new HashMap<String, Object>();
			
			propiedades.put("charla.id", charla.getId());
			propiedades.put("usuario.id", usuario.getId());
			
			try{				
				MatriculaCharla matriculaSearch = findOneWhere(propiedades);
								
				matriculaSearch.setPreMatriculadoWeb(true);
				matriculaSearch.setEnListaEspera(true);
				matriculaSearch.setAdmitido(false);
				matriculaSearch.setCancelada(false);
				matriculaSearch.setDesactivado(false);

				notificarPrematriculaCharla(matriculaSearch, usuario);
				
				return true;
				
			}catch (NoResultException nre) {
				//No solicitada la prematrícula
				//Creamos una nueva
			}		
			
			//Todo OK. Y no existe matrícula cancelada, Se procedede a la prematriculacion WEB
			
			matricula.setCharla(charla);
		    matricula.setUsuario((UsuarioCyL)usuario);
			matricula.setPreMatriculadoWeb(true);
			matricula.setEnListaEspera(true);
			matricula.setAdmitido(false);
			matricula.setCancelada(false);
			
			// Mandar el correo notificando la matriculación
	        //enviarMailMatriculado(matricula);			
			notificarPrematriculaCharla(matricula, usuario);
			
		
			disableMessages();
			
			persist(matricula);
			
			enableMessages();
			
			return true;
		}
	}
	else {
		//Ya existe, no se puede volver a crear el registro, para modificar se utilizará getMatriculaACancelar().
		
		addErrorMessage("matricula.validator.yaMatriculadoOSolicitado");
		
		return false;
	}
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaCursoService#persistCancelarPrematriculacion(es.csa.fdt.cyldigital.servicios.entity.Charla, es.csa.fdt.cyldigital.servicios.entity.Curso)
   * 
   * Solo se invoca desde la web de inscripción.
   */
  public boolean persistCancelarPrematriculacion(Long idCharla, Long idUsuario) {
	
	if (idUsuario == null) {
		addErrorMessage("matricula.validator.usuarioNecesario");
		return false;
	} else if (idCharla == null) {
		addErrorMessage("matricula.validator.cursoNecesario");
		return false;
	} else {
		Map<String, Object> propiedades = new HashMap<String, Object>();
		
		propiedades.put("charla.id", idCharla);
		propiedades.put("usuario.id", idUsuario);

		try {
			MatriculaCharla matricula = findOneWhere(propiedades);

			if (!matricula.isAdmitido() && matricula.isPreMatriculadoWeb() && matricula.isEnListaEspera()) {
				//Todo OK. Se procedede a la cancelación de la prematriculacion WEB
				
				if ((matricula.getCharla() != null) && (matricula.getCharla().getMatriculas() != null)){
					matricula.setEnListaEspera(false);
					matricula.setCancelada(true);
					//matricula.setDesactivado(true);
				}
			    if ((matricula.getUsuario() != null) && (matricula.getUsuario().getMatriculasCharlas() != null)){
			    	matricula.setEnListaEspera(false);
			    	matricula.setCancelada(true);
					//matricula.setDesactivado(true);
			    }
			    
			    //Mandar el correo notificando la cancelación de la solicitud de matrícula.
			    notificarCancelacionPreMatriculaCharla(idUsuario, matricula);
			    disableMessages();				
				enableMessages();		
				return true;
			} else {
				//No solicitada la prematrícula
				addErrorMessage("matricula.validator.noMatriculado");
				return false;
			}
		} catch (NoResultException nre) {
			//No solicitada la prematrícula
			addErrorMessage("matricula.validator.noMatriculado");
			return false;
		} catch (Exception e) {
			logger.log(Level.WARNING,
					   "MatriculaCharlaServiceImpl.persitCancelarPrematriculacion: Error al extraer matrícula de la base de datos." +
					   e.getLocalizedMessage());
			e.printStackTrace();
			return false;
		}
	}
  }

  @SuppressWarnings("unchecked")
  private void enviarMailMatriculado(MatriculaCharla matriculaCharla) {
	  String sendmail = "";
	  String email = "";
	  NotificationService n = new NotificationService();

	  // Se mira el parámetro general sobre mails activados o desactivados.
	  List<Parameter> parameters = findWhere(Parameter.class, "nombre", "mail_activate");
	  if (parameters.size() > 0) {
		  sendmail = parameters.get(0).getValor();
	  }

	  if (sendmail.equals("1")) {
		  UsuarioCyL u = this.find(UsuarioCyL.class, matriculaCharla.getUsuario().getId());
		  
		  if (u.iscomunicacionMail()) { // el usuario desea notificaciones
			  email = ((UsuarioCyL) matriculaCharla.getUsuario()).getEmail();
			  
			  if (email.isEmpty() || email == null) {
				  addWarnMessage("mail.nomailavailable");
			  } else {
				  if (n.notificarSuscritoActividad(this, matriculaCharla.getUsuario().getId(), email, matriculaCharla.getCharla()))
					  addInfoMessage("mail.email");
				  else
					  addWarnMessage("mail.errormail");// ha ocurrido algún tipo de error.
			  }
		  }
		  else {
			  addInfoMessage("mail.noemail");// el usuario no desea notificaciones por correo electrónico.
		  }
	  }
	  else { // está desactivado el envío de correos electrónicos en el sistema.
		  addWarnMessage("mail.nonotifications");
	  }
  }
  
  /**
   * Persist usuario.
   * 
   * @param usuario
   *           usuario
   * @param matriculacurso
   *           matriculacurso       
   * @return true, if successful
   */
  @SuppressWarnings("unchecked")
  private void notificarPrematriculaCharla(MatriculaCharla matriculaCharla, Usuario usuario) {
	  
	  NotificationService n = new NotificationService();
	  String mailActivate = "";
	  String mailRequestenrollmentActivate = "";
	  
	  
	  try {
		  List<Parameter> parameters1 = findWhere(Parameter.class, "nombre", "mail_activate");
		  if (parameters1.size() > 0)
			  mailActivate = parameters1.get(0).getValor();

		  List<Parameter> parameters2 = findWhere(Parameter.class, "nombre", "mail_requestenrollment_activate");
		  if (parameters2.size() > 0)
			  mailRequestenrollmentActivate = parameters2.get(0).getValor();

		  if (mailActivate.equals("1")){
			  if (mailRequestenrollmentActivate.equals("1")){
				  if (n.notificarPreMatriculaCharla(this, matriculaCharla, usuario))
					  addInfoMessage("mail.email");
				  else
					  addErrorMessage("mail.errormail");
			  }
			  else{
				  // están desactivadas las notificaciones por email asociadas a las solicitudes de matícula en una actividad.
				  // se obvia la advertencia puesto que el destinatario siempre será un ciudadano a través de la web de inscripción.
				  //addWarnMessage("mail.requestenrollment.nonotifications");
			  }
		  }
		  else { 
			  // están desactivadas las notificaciones por email en el sistema.
			  // se obvia la advertencia puesto que el destinatario siempre será un ciudadano a través de la web de inscripción.
			  //addWarnMessage("mail.nonotifications");
		  }
	  } catch (Exception e){
		  e.printStackTrace();
	  }
  }
  
  /**
   * notificación cancelación de una prematricula charla de un usuario.
   * 
   * @param idUsuario
   *           idUsuario    
   * @return true, if successful
   */
  @SuppressWarnings("unchecked")
  private void notificarCancelacionPreMatriculaCharla(Long idUsuario, MatriculaCharla matriculaCharla){
	  
	  NotificationService n = new NotificationService();
	  String mailActivate = "";
	  String mailRequestenrollmentActivate = "";
	  
	  
	  try {
		  List<Parameter> parameters1 = findWhere(Parameter.class, "nombre", "mail_activate");
		  if (parameters1.size() > 0)
			  mailActivate = parameters1.get(0).getValor();

		  List<Parameter> parameters2 = findWhere(Parameter.class, "nombre", "mail_requestenrollment_activate");
		  if (parameters2.size() > 0)
			  mailRequestenrollmentActivate = parameters2.get(0).getValor();

		  if (mailActivate.equals("1")){
			  if (mailRequestenrollmentActivate.equals("1")){
				  if (n.notificarCancelacionPreMatriculaCharla(this, idUsuario, matriculaCharla))
					  addInfoMessage("mail.email");
				  else
					  addErrorMessage("mail.errormail");
			  }
			  else{
				  // están desactivadas las notificaciones por email asociadas a las solicitudes de matícula en una actividad.
				  // se obvia la advertencia puesto que el destinatario siempre será un ciudadano a través de la web de inscripción.
				  //addWarnMessage("mail.requestenrollment.nonotifications");
			  }
		  }
		  else {
			  // están desactivadas las notificaciones por email en el sistema.
			  // se obvia la advertencia puesto que el destinatario siempre será un ciudadano a través de la web de inscripción.
			  //addWarnMessage("mail.nonotifications");
		  }
	  } catch (Exception e){
		  e.printStackTrace();
	  }
  }
  
  /**
   * Persist usuario.
   * 
   * @param usuario
   *           usuario
   * @param matriculacharla
   *           matriculacharla       
   * @return true, if successful
   */
  @SuppressWarnings("unchecked")
  private void notificarMatriculadoCharla(MatriculaCharla matriculaCharla) {
	  
	  NotificationService n = new NotificationService();
	  String mailActivate = "";
	  String mailRequestenrollmentActivate = "";
	  
	  
	  try {
		  List<Parameter> parameters1 = findWhere(Parameter.class, "nombre", "mail_activate");
		  if (parameters1.size() > 0)
			  mailActivate = parameters1.get(0).getValor();

		  List<Parameter> parameters2 = findWhere(Parameter.class, "nombre", "mail_enrollment_activate");
		  if (parameters2.size() > 0)
			  mailRequestenrollmentActivate = parameters2.get(0).getValor();

		  if (mailActivate.equals("1")){
			  if (mailRequestenrollmentActivate.equals("1")){
				  if (n.notificarMatriculaCharla(this, matriculaCharla))
					  addInfoMessage("mail.email");
				  else
					  addErrorMessage("mail.errormail");
			  }
			  else{
				  // están desactivadas las notificaciones por email asociadas a las matriculaciones en una actividad.
				  addWarnMessage("mail.enrollment.nonotifications");
			  }
		  }
		  else { 
			  // están desactivadas las notificaciones por email en el sistema.
			  addWarnMessage("mail.nonotifications");
		  }
	  } catch (Exception e){
		  e.printStackTrace();
	  }
  }

  private boolean isYaMatriculado(Usuario u, Long idMatriculaCharla, Charla c) {
		  
	  if (u != null && u.getMatriculasCharlas() != null) {
		  if (idMatriculaCharla != 0)
		  {
			  for (MatriculaCharla a : u.getMatriculasCharlas()) {
				  if (a.getCharla().equals(c) && a.getCharla().getCentro().equals(c.getCentro()) && a.getId()== idMatriculaCharla && !a.isCancelada()) {
					  return true;
				  }				  
			  }
		  } else {
			  for (MatriculaCharla a : u.getMatriculasCharlas()) {
				  if (a.getCharla().equals(c) && a.getCharla().getCentro().equals(c.getCentro()) && !a.isCancelada()) {
					  return true;
				  }				  
			  }
		  }
	  }
	  return false;
  }


  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaCharlaService#removeMatriculaCharla(es.csa.fdt.cyldigital.servicios.entity.MatriculaCharla)
   */
  public void removeMatriculaCharla(MatriculaCharla matriculaCharla) {
    if (matriculaCharla.getId() != 0) {
      if (matriculaCharla.getCharla() != null) matriculaCharla.getCharla().getMatriculas().remove(matriculaCharla);
      if (matriculaCharla.getUsuario().getMatriculasCharlas() != null)
        matriculaCharla.getUsuario().getMatriculasCharlas().remove(matriculaCharla);
      notificaMatriculaBorrada(matriculaCharla);
      remove(matriculaCharla);
    }
  }  
  
  /**
   * notificación que se ha borrado una matricula de un usuario
   * 
   * @param matriculaCurso
   *           matriculaCurso  
   * @return true, if successful
   */
  @SuppressWarnings("unchecked")
  private void notificaMatriculaBorrada(MatriculaCharla matriculaCharla) {
	  
	  NotificationService n = new NotificationService();
	  String mailActivate = "";
	  String mailRequestenrollmentActivate = "";  
	 
	  
	  try {
		  List<Parameter> parameters1 = findWhere(Parameter.class, "nombre", "mail_activate");
		  if (parameters1.size() > 0)
			  mailActivate = parameters1.get(0).getValor();

		  List<Parameter> parameters2 = findWhere(Parameter.class, "nombre", "mail_enrollment_activate");
		  if (parameters2.size() > 0)
			  mailRequestenrollmentActivate = parameters2.get(0).getValor();

		  if (mailActivate.equals("1")){
			  if (mailRequestenrollmentActivate.equals("1")){
				  if (n.notificarBorrarMatriculaCharlaUsuario(this, matriculaCharla))
					  addInfoMessage("mail.email");
				  else
					  addErrorMessage("mail.errormail");
			  }
			  else{
				  // están desactivadas las notificaciones por email asociadas a las matriculaciones en una actividad.
				  addWarnMessage("mail.enrollment.nonotifications");
			  }
		  }
		  else { 
			  // están desactivadas las notificaciones por email en el sistema.
			  addWarnMessage("mail.nonotifications");
		  }
	  } catch (Exception e){
		  e.printStackTrace();
	  }
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaCharlaService#getCharlas()
   */
  public List<Charla> getCharlas() {
    if (charlas == null) charlas = findAllOrder(Charla.class, "fecha", true);
    return charlas;
  };

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaCharlaService#getUsuarios()
   */
  public List<Usuario> getUsuarios() {
    if (usuarios == null) usuarios = findAllOrder(Usuario.class, "nombre", true);
    return usuarios;

  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaCharlaService#actualizaValores(es.csa.fdt.cyldigital.servicios.entity.MatriculaCharla, java.lang.Long, java.lang.Long)
   */
  public void actualizaValores(MatriculaCharla a, Long idCharla, Long idUsuario) {
    if (a != null && a.getId() == 0) {
      if (idCharla != null) a.setCharla(find(Charla.class, idCharla));
      if (idUsuario != null) a.setUsuario(find(UsuarioCyL.class, idUsuario));
    }
  }

  /* (non-Javadoc)
  * @see es.csa.fdt.cyldigital.servicios.service.MatriculaCursoService#getCursosLanguageDelCentro(es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap)
  */
  public List<CharlaLanguage> getCharlasLanguageDelCentro(UsuarioLdap u) {
  List<CharlaLanguage> cl = new ArrayList<CharlaLanguage>();

  String idioma = LocaleContextHolder.getLocaleContext().getLocale().toString();

  HashMap<String, Object> propiedades = new HashMap<String, Object>();
  
  if (!u.getRolesList().contains(Role.ROLE_ADMIN))
  	propiedades.put("charla.centro", u.getCentro());
  	
  propiedades.put("language.ISO", idioma);

  if (u.getCentro() != null) return findWhereOrder(CharlaLanguage.class, propiedades, "charla.fechaInicio", true);
  return cl;
  }
  
  
  /**
   * Devuelve charlas del centro.
   * 
   * @param u
   *           u
   * @return charlas del centro
   */
  public List<Charla> getCharlasDelCentro(UsuarioLdap u) {
    List<Charla> c = new ArrayList<Charla>();
    
    if (u.getCentro() != null) {
    	if (!u.getRolesList().contains(Role.ROLE_ADMIN))
    		return findWhereOrder(Charla.class, "centro", u.getCentro(), "fechaInicio", true);
    	else
    		return findAllOrder(Charla.class, "fechaInicio", true);
    }
    
    return c;
  }
  
  public List<MatriculaCharla> findSolicitudesWeb(Long idCharla) {
	  List<MatriculaCharla> lmc = new ArrayList<MatriculaCharla>();
	  
	  if (idCharla != null) {
		  Map<String, Object> propiedades = new HashMap<String, Object>();
		  
		  propiedades.put("charla.id", idCharla);
		  propiedades.put("preMatriculadoWeb", true);
		  propiedades.put("enListaEspera", true);
		  propiedades.put("admitido", false);
		  
		  try {
			  lmc = findWhere(propiedades);
		  }
		  catch (NoResultException nre) {

		  }
		  catch (Exception e) {
				logger.log(Level.WARNING,
						   "MatriculaCharlaServiceImpl.findSolicitudesWeb: Error al extraer solicitud de matrícula de la base de datos." +
						   e.getLocalizedMessage());

				e.printStackTrace();

		  }
	  }
	  
	  return lmc;
  }
  
  public List<MatriculaCharla> getMatriculaMultipleCharla() {
	  if (matriculaMultipleCharla == null)
		  matriculaMultipleCharla = new ArrayList<MatriculaCharla>();
	  
	  return matriculaMultipleCharla;
  }
  
  public void setMatriculaMultipleCharla(List<MatriculaCharla> lmc) {
	  matriculaMultipleCharla.clear();
	  
	  matriculaMultipleCharla.addAll(lmc);
  }
  
  public List<MatriculaCharla> createTemporalMatriculaMultipleCharla() {
	  List<MatriculaCharla> tmmc = new ArrayList<MatriculaCharla>();
	  
	  tmmc.addAll(matriculaMultipleCharla);
	  
	  return tmmc;
  }
  
  public MatriculaCharla anyadeUsuarioAMatricular(List<MatriculaCharla> tmmc, Long idCharla, Usuario alumno) {
	  MatriculaCharla mc = new MatriculaCharla();
	  
	  actualizaValores(mc, idCharla, alumno.getId());

	  tmmc.add(mc);
	  
	  return mc;
  }
  
  public void removeUsuarioAMatricular(List<MatriculaCharla> tmmc, Usuario alumno) {
	  for (MatriculaCharla mc : tmmc) {
		  if (mc.getUsuario().getId() == alumno.getId()) {
			  tmmc.remove(mc);
			  
			  break;
		  }
	  }
  }
  
  public void purgeTemporalMatriculaMultiple(List<MatriculaCharla> tmmc) {
	  tmmc.clear();
  }
  
}
