package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.List;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.notificacion.NotificationService;
import es.csa.fdt.cyldigital.servicios.entity.CursoOnline;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCursoOnline;
import es.csa.fdt.cyldigital.servicios.entity.PreinscripcionCursoOnline;
import es.csa.fdt.cyldigital.servicios.entity.ServicioCursoOnline;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioOnline;

/**
 * Clase MatriculaCursoOnlineServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class MatriculaCursoOnlineServiceImpl extends AbstractServiceImpl<MatriculaCursoOnline, Long> implements
    MatriculaCursoOnlineService {

  private List<CursoOnline> cursos;

  private List<UsuarioOnline> usuarios;

  private boolean rellenarResultado = false;

  private boolean admitido = false;

  private PreinscripcionCursoOnline preinscripcion;

  private static final long serialVersionUID = -5575228180361917674L;

  /**
   * Construye un nuevo objeto de la clase
   */
  public MatriculaCursoOnlineServiceImpl() {
    super();
    setPreinscripcion(new PreinscripcionCursoOnline());
  }

  /**
   * Comprueba si está admitido al entrar en la matrícula.
   * 
   * @return true, si está admitido
   */
  public boolean isAdmitido() {
    return admitido;
  }

  /**
   * Asigna si está admitido al entrar en la matrícula.
   * 
   * @param admitido
   *          admitido
   */
  public void setAdmitido(boolean admitido) {
    this.admitido = admitido;
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * es.csa.fdt.cyldigital.servicios.service.MatriculaCursoOnlineService#persistMatriculaCurso(es.csa.fdt.cyldigital
   * .servicios.entity.MatriculaCursoOnline, boolean)
   */
  public boolean persistMatriculaCurso(MatriculaCursoOnline matriculaCurso, boolean enlistaesperaanterior) {

    if (matriculaCurso.getId() == 0 && isYaMatriculado(matriculaCurso.getUsuario(), matriculaCurso.getCurso())) {
      return false;
    } else if (matriculaCurso.getId() == 0 && matriculaCurso.getUsuario() == null) {
      addErrorMessage("matricula.validator.usuario_necesario");
      return false;
    } else if (matriculaCurso.getId() == 0
        && matriculaCurso.getCurso().getMatriculas().size() >= matriculaCurso.getCurso().getNumeroPlazas()) {
      // es una reserva nueva.

      // getPreinscripcion().setCentro(matriculaCurso.getCurso().getCentro());
      // getPreinscripcion().setUsuario(matriculaCurso.getUsuario());
      // getPreinscripcion().setServicioCurso((ServicioCurso) matriculaCurso.getCurso().getServicio());
      // getPreinscripcion().setFecha(new Date());
      // disableMessages();
      // persist(getPreinscripcion());
      // enableMessages();
      // addInfoMessage("preinscripcion.usuario.preinscrito", new Object[] {
      // matriculaCurso.getUsuario().getNombreCompleto(),
      // ((ServicioCurso) matriculaCurso.getCurso().getServicio()).getNombre() });
      matriculaCurso.setEnListaEspera(true);
      disableMessages();
      persist(matriculaCurso);
      enableMessages();

      /*
       * Comentado para implementar los servicios MULTILANGUAGE
       * TODO: mirar y modificar
      addInfoMessage("preinscripcion.usuario.preinscrito", new Object[] {
          matriculaCurso.getUsuario().getNombreCompleto(),
          ((ServicioCursoOnline) matriculaCurso.getCurso().getServicio()).getNombre() });
      */
      return true;
    } else {
      if (matriculaCurso.getId() == 0 || (matriculaCurso.isAdmitido() && !this.isAdmitido())) {
        // la matricula es nueva, mandar el correo. Sino, es una modificacion, no enviarlo.
        // por último notificamos la matriculación
        enviarMailMatriculado(matriculaCurso, matriculaCurso.isAdmitido());
      }

      if (matriculaCurso.getId() != 0 && matriculaCurso.isEnListaEspera() == false && enlistaesperaanterior == true) {
        if (matriculaCurso.getCurso().getNumeroMatriculados() > matriculaCurso.getCurso().getNumeroPlazas()) {
          addErrorMessage("Plazas de matriculación completas, no se puede pasar al usuario de la lista de espera a "
              + "la de matriculación.");
          return false;
        } else {
          // se ha pasado una reserva a matrícula, hay que notificarlo
          enviarMailMatriculado(matriculaCurso, false);
        }
      }
      persist(matriculaCurso);
      return true;
    }
  }

  @SuppressWarnings("unchecked")
  private void enviarMailMatriculado(MatriculaCursoOnline matriculaCurso, boolean admitido) {
    String sendmail = "";
    NotificationService n = new NotificationService();

    List<Parameter> parameters = findWhere(Parameter.class, "nombre", "mail_activate");
    if (parameters.size() > 0) {
      sendmail = parameters.get(0).getValor();
    }
    if (sendmail.equals("1")) {
      if (admitido) {
        if (n.notificarSuscritoCursoOnline(this, matriculaCurso.getUsuario().getId(), matriculaCurso.getUsuario()
            .getEmail(), matriculaCurso.getCurso())) addInfoMessage("mail.email");
        else addWarnMessage("mail.errormail");
      } else {
        if (n.notificarSuscritoCursoOnline(this, matriculaCurso.getUsuario().getId(), matriculaCurso.getUsuario()
            .getEmail(), matriculaCurso.getCurso())) addInfoMessage("mail.email");
        else addWarnMessage("mail.errormail");
      }

    } else {
      addWarnMessage("mail.nonotifications");
    }
  }

  private boolean isYaMatriculado(UsuarioOnline u, CursoOnline c) {
    if (u != null && u.getMatriculasCursos() != null) for (MatriculaCursoOnline p : u.getMatriculasCursos()) {
      if (p.getCurso().equals(c)) {
        addErrorMessage("matricula.validator.yainscrito");
        return true;
      }
    }
    // else {
    // if (u != null && u.getPreinscripcionesCursos() != null)
    // for (PreinscripcionCurso p : u.getPreinscripcionesCursos()) {
    // if (p.getServicioCurso().equals(((ServicioCurso) c.getServicio())) && p.getCentro().equals(c.getCentro())) {
    // addErrorMessage("preinscripcion.validator.yainscrito");
    // return true;
    // }
    // }
    // }
    return false;
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * es.csa.fdt.cyldigital.servicios.service.MatriculaCursoOnlineService#removeMatriculaCurso(es.csa.fdt.cyldigital.
   * servicios.entity.MatriculaCursoOnline)
   */
  public void removeMatriculaCurso(MatriculaCursoOnline matriculaCurso) {
    if (matriculaCurso.getId() != 0) {
      if (matriculaCurso.getCurso() != null) matriculaCurso.getCurso().getMatriculas().remove(matriculaCurso);
      if (matriculaCurso.getUsuario().getMatriculasCursos() != null)
        matriculaCurso.getUsuario().getMatriculasCursos().remove(matriculaCurso);
      if (!matriculaCurso.isEnListaEspera()) notificaMatriculaBorrada(matriculaCurso);
      remove(matriculaCurso);
    }
  }

  @SuppressWarnings("unchecked")
  private void notificaMatriculaBorrada(MatriculaCursoOnline matriculaCurso) {
    NotificationService n = new NotificationService();
    String sendmail = "";
    List<Parameter> parameters = findWhere(Parameter.class, "nombre", "mail_activate");
    if (parameters.size() > 0) {
      sendmail = parameters.get(0).getValor();
    }

    if (sendmail.equals("1")) {
      if (n.notificarDesuscritoCursoOnline(this, matriculaCurso.getUsuario().getEmail(), matriculaCurso.getCurso())) addInfoMessage("mail.email");
      else addWarnMessage("mail.errormail");
    } else addWarnMessage("mail.nonotifications");
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaCursoOnlineService#getCursos()
   */
  public List<CursoOnline> getCursos() {
    if (cursos == null) cursos = findAllOrder(CursoOnline.class, "fechaInicio", true);
    return cursos;
  }

  /*
   * (non-Javadoc)
   * 
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaCursoOnlineService#getUsuarios()
   */
  public List<UsuarioOnline> getUsuarios() {
    if (usuarios == null) usuarios = findAllOrder(UsuarioOnline.class, "nombre", true);
    return usuarios;

  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * es.csa.fdt.cyldigital.servicios.service.MatriculaCursoOnlineService#actualizaValores(es.csa.fdt.cyldigital.servicios
   * .entity.MatriculaCursoOnline, java.lang.Long, java.lang.Long)
   */
  public void actualizaValores(MatriculaCursoOnline m, Long idCurso, Long idUsuario) {
    if (m != null && m.getId() == 0) {
      if (idCurso != null) m.setCurso(find(CursoOnline.class, idCurso));
      if (idUsuario != null) m.setUsuario(find(UsuarioOnline.class, idUsuario));
    }
  }

  /**
   * Checks if is rellenar resultado.
   * 
   * @return true, if is rellenar resultado
   */
  public boolean isRellenarResultado() {
    return rellenarResultado;
  }

  /**
   * Asigna rellenar resultado.
   * 
   * @param rellenarResultado
   *          rellenar resultado
   */
  public void setRellenarResultado(boolean rellenarResultado) {
    this.rellenarResultado = rellenarResultado;
  }

  /**
   * Devuelve preinscripcion.
   * 
   * @return preinscripcion
   */
  public PreinscripcionCursoOnline getPreinscripcion() {
    return preinscripcion;
  }

  /**
   * Asigna preinscripcion.
   * 
   * @param presinscripcion
   *          preinscripcion
   */
  public void setPreinscripcion(PreinscripcionCursoOnline presinscripcion) {
    this.preinscripcion = presinscripcion;
  }

  /**
   * Devuelve cursos online.
   * 
   * @return cursos online
   */
  public List<CursoOnline> getCursosOnline() {
    return findAllOrder(CursoOnline.class, "fechaInicio", true);
  }
}
