package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.List;

import es.csa.fdt.cyldigital.core.service.AbstractService;
import es.csa.fdt.cyldigital.servicios.entity.Curso;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso;
import es.csa.fdt.cyldigital.servicios.entity.Curso.EEstadoMatricula;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;

/**
 * Interfaz del servicio de MatriculaCurso.
 * 
 * @author csa
 */
public interface MatriculaCursoService extends AbstractService<MatriculaCurso, Long> {

  /**
   * Persist matricula curso.
   * 
   * @param matriculaCurso
   *           matricula curso
   * @param enlistadeespera
   *           enlistadeespera
   * @return true, if successful
   */
	public boolean persistMatriculaCurso(MatriculaCurso matriculaCurso, EEstadoMatricula estadoMatriculaAnterior, Long idEmpleado);

  /**
   * Elimina matricula curso.
   * 
   * @param matriculaCurso
   *           matricula curso
   */
  public void removeMatriculaCurso(MatriculaCurso matriculaCurso);

  /**
   * Devuelve cursos.
   * 
   * @return cursos
   */
  public List<Curso> getCursos();

  /**
   * Devuelve usuarios.
   * 
   * @return usuarios
   */
  public List<Usuario> getUsuarios();

  /**
   * Actualiza valores.
   * 
   * @param m
   *           m
   * @param idCurso
   *           id curso
   * @param idUsuario
   *           id usuario
   */
  public void actualizaValores(MatriculaCurso m, Long idCurso, Long idUsuario);

  /**
   * Devuelve cursos del centro.
   * 
   * @param u
   *           u
   * @return cursos del centro
   */
  public List<Curso> getCursosDelCentro(UsuarioLdap u);
}
