package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.NoResultException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.binding.message.MessageBuilder;
import org.springframework.binding.message.MessageContext;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;
import org.springframework.webflow.execution.RequestContextHolder;

import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.evaluacion.entity.FormularioConcreto;
import es.csa.fdt.cyldigital.evaluacion.entity.Pregunta;
import es.csa.fdt.cyldigital.evaluacion.entity.Respuesta;

import es.csa.fdt.cyldigital.language.entity.CursoLanguage;

import es.csa.fdt.cyldigital.notificacion.NotificationService;

import es.csa.fdt.cyldigital.servicios.entity.Curso;
import es.csa.fdt.cyldigital.servicios.entity.Curso.EEstadoMatricula;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso;
import es.csa.fdt.cyldigital.servicios.entity.PreinscripcionCurso;
import es.csa.fdt.cyldigital.servicios.entity.ServicioCurso;

import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.Localidad;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario.Role;


/**
 * Clase MatriculaCursoServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class MatriculaCursoServiceImpl extends AbstractServiceImpl<MatriculaCurso, Long>
									   implements MatriculaCursoService {

	private static final long serialVersionUID = -5575228180361917674L;
	  
	private static final Logger logger = LoggerFactory.getLogger(MatriculaCursoServiceImpl.class);
	  
	private List<Curso> cursos;

	private List<Usuario> usuarios;

	private boolean rellenarResultado = false;

	private PreinscripcionCurso preinscripcion;
	
	private List<MatriculaCurso> matriculaMultipleCurso;
	
  /**
   * Construye un nuevo objeto de la clase
   */
  public MatriculaCursoServiceImpl() {
    super();
    setPreinscripcion(new PreinscripcionCurso());
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaCursoService#persistMatriculaMultipleCurso(java.util.List<es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso>, Long, String)
   */
  public boolean persistMatriculaMultipleCurso(List<MatriculaCurso> matriculas, Long idEmpleado, String observaciones) {
	  Boolean result = true;
	  
	  if (matriculas.size() > 0) {
		  disableMessages();
		  
		  for (MatriculaCurso mc : matriculas) {
			  mc.setObservaciones(observaciones);
			  
			  if (!persistMatriculaCurso(mc, null, idEmpleado)) {
				  result = false;
				  break;
			  }
		  }
		  
		  enableMessages();
	  }
	  else {
		  addErrorMessage("matricula.validator.usuario_necesario");
		  
		  return false;
	  }

	  return result;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaCursoService#persistMatriculaCurso(es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso, es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso.EEstadoMatricula, Long)
   */
  public boolean persistMatriculaCurso(MatriculaCurso matriculaCurso, EEstadoMatricula estadoMatriculaAnterior, Long idEmpleado) {
	//Si es una nueva matrícula y el usuario ya está matriculado
	if (isYaMatriculado(matriculaCurso.getUsuario(), matriculaCurso.getId(), matriculaCurso.getCurso()) &&
		(matriculaCurso.getId()== 0)){
		addErrorMessage("matricula.validator.yainscrito");
      
		return false;
		   //Si es una nueva matrícula y no se ha seleccionado usuario
    } else if ((matriculaCurso.getId() == 0) &&
    		   (matriculaCurso.getUsuario() == null)) {
    	addErrorMessage("matricula.validator.usuario_necesario");
      
    	return false;
    	   //Si es una nueva matrícula y ya está el curso completo
    } else if ((matriculaCurso.getCurso().getNumeroMatriculados() >= matriculaCurso.getCurso().getNumeroPlazas()) && 
    		   (matriculaCurso.getId() == 0)) {
    	addErrorMessage("matricula.validator.plazas_completas");

    	return false;
    	   //Este caso es improbable que se dé, pero se comprueba por si llegase a ocurrir.
    	   //Si ya está el curso completo (si el estado nuevo es matriculado está incluida en la cuenta de número de matriculado, por eso se pregunta por > y no por >=),
    	   //anteriormente no estaba admitido, el nuevo estado es admitido y la matrícula ya existía,
    } else if ((matriculaCurso.getCurso().getNumeroMatriculados() > matriculaCurso.getCurso().getNumeroPlazas()) &&
    		   (estadoMatriculaAnterior != EEstadoMatricula.MATRICULADO) &&
    		   (matriculaCurso.getEstadoMatricula() == EEstadoMatricula.MATRICULADO) &&
    		   (matriculaCurso.getId() != 0)) {
    	addErrorMessage("matricula.validator.plazas_completas");
    	
    	return false;    	
    } else {
      if (matriculaCurso.getId() == 0){
    	  matriculaCurso.setAdmitido(true);
    	  matriculaCurso.setEnListaEspera(false);
    	  matriculaCurso.setPreMatriculadoWeb(false);
    	  matriculaCurso.setCancelada(false);
      }
     
      Empleado empleado = find(Empleado.class, idEmpleado);
      Date fecha = new Date();

      matriculaCurso.setFechaTramite(fecha);
      matriculaCurso.setEmpleadoTramite(empleado);

      disableMessages();
      persist(matriculaCurso);
      enableMessages();

      //Envio la notificación sea el caso que sea (matriculado por WebAdmin o por WebInscripcion, rechazado o solicitud)
      if (((matriculaCurso.getId() != 0) &&(matriculaCurso.getEstadoMatricula() != estadoMatriculaAnterior)) ||
    	  (matriculaCurso.getId() == 0))
    	  notificarMatriculadoCurso(matriculaCurso);
      
      return true;
    }
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaCursoService#persistPrematriculacion(es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso, es.csa.fdt.cyldigital.servicios.entity.Curso, es.csa.fdt.cyldigital.usuarios.entity.Usuario)
   * 
   * Solo se invoca desde la web de inscripción.
   */
  public boolean persistPrematriculacion(MatriculaCurso matricula, Curso curso, Usuario usuario) {
	
	if (matricula.getId() == 0) {
		if (usuario == null) {
			addErrorMessage("matricula.validator.usuarioNecesario");
			
			return false;
		}
		else if (curso == null) {
			addErrorMessage("matricula.validator.cursoNecesario");
			
			return false;
		}
		else if (isYaMatriculado(usuario, matricula.getId(), curso)) {
			addErrorMessage("matricula.validator.yaMatriculado");
			
			return false;
		}
		else {

			Map<String, Object> propiedades = new HashMap<String, Object>();
			
			propiedades.put("curso.id", curso.getId());
			propiedades.put("usuario.id", usuario.getId());
			
			try{
				
				MatriculaCurso matriculaSearch = findOneWhere(propiedades);
				
				//Todo OK. Y existe matrícula cancelada, Se vuelve a activar la matrícula					
				matriculaSearch.setPreMatriculadoWeb(true);
				matriculaSearch.setEnListaEspera(true);
				matriculaSearch.setAdmitido(false);
				matriculaSearch.setCancelada(false);
				matriculaSearch.setDesactivado(false);

				notificarPrematriculaCurso(matriculaSearch, usuario);
				
				return true;
				
			}catch (NoResultException nre) {
				//No solicitada la prematrícula
				//Creamos una nueva
			}
			
			//Todo OK. Y no existe matrícula cancelada, Se procedede a la prematriculacion WEB
			matricula.setCurso(curso);
		    matricula.setUsuario((UsuarioCyL)usuario);
			matricula.setPreMatriculadoWeb(true);
			matricula.setEnListaEspera(true);
			matricula.setAdmitido(false);
			matricula.setCancelada(false);
			
			// Mandar el correo notificando la matriculación
	        //enviarMailMatriculado(matricula);
			notificarPrematriculaCurso(matricula, usuario);
	        
			disableMessages();
							
			persist(matricula);
			
			enableMessages();
			
			return true;
		
		}
	}
	else {
		//Ya existe, no se puede volver a crear el registro, para modificar se utilizará getMatriculaACancelar().
		
		addErrorMessage("matricula.validator.yaMatriculadoOSolicitado");
		
		return false;
	}
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaCursoService#persistCancelarPrematriculacion(es.csa.fdt.cyldigital.servicios.entity.Curso, es.csa.fdt.cyldigital.servicios.entity.Curso)
   * 
   * Solo se invoca desde la web de inscripción.
   */
  public boolean persistCancelarPrematriculacion(Long idCurso, Long idUsuario) {
	
	if (idUsuario == null) {
		addErrorMessage("matricula.validator.usuarioNecesario");
		return false;
	} else if (idCurso == null) {
		addErrorMessage("matricula.validator.cursoNecesario");
		
		return false;
	} else {
		Map<String, Object> propiedades = new HashMap<String, Object>();
		
		propiedades.put("curso.id", idCurso);
		propiedades.put("usuario.id", idUsuario);

		try {
			MatriculaCurso matricula = findOneWhere(propiedades);

			if (!matricula.isAdmitido() && matricula.isPreMatriculadoWeb() && matricula.isEnListaEspera() && !matricula.isCancelada()) {
				//Todo OK. Se procedede a la cancelación de la prematriculacion WEB
				
				if ((matricula.getCurso() != null) && (matricula.getCurso().getMatriculas() != null)){
					matricula.setEnListaEspera(false);
					matricula.setCancelada(true);
					//matricula.setDesactivado(true);
				}
			      
			    if ((matricula.getUsuario() != null) && (matricula.getUsuario().getMatriculasCursos() != null)){
			    	matricula.setEnListaEspera(false);
			    	matricula.setCancelada(true);
					//matricula.setDesactivado(true);
			    }
			    
			    //Se manda la notificación de cancelación
			    notificarCancelacionPreMatriculaCurso(idUsuario, matricula);
			    disableMessages();
			    enableMessages();
				return true;
			} else {
				//No solicitada la prematrícula
				addErrorMessage("matricula.validator.noMatriculado");
				return false;
			}
		} catch (NoResultException nre) {
			//No solicitada la prematrícula
			addErrorMessage("matricula.validator.noMatriculado");
			return false;
		} catch (Exception e) {
			logger.warn("MatriculaCursoServiceImpl.persitCancelarPrematriculacion: Error al extraer matrícula de la base de datos." +
					e.getLocalizedMessage());
			e.printStackTrace();
			return false;
		}
	}
  }
  
  public void persistDiplomaIssued(MatriculaCurso matricula, Long idCurso) {
	  Curso c = find(Curso.class, idCurso);
	  ServicioCurso sc = find(ServicioCurso.class, c.getServicio().getId());
	  
	  if (matricula.getLocalidadDiplomaEmitido() == null)
		 matricula.setLocalidadDiplomaEmitido(c.getCentro().getLocalidad());
	  
	  matricula.setDiplomaCreado(true);
	  matricula.setFechaDiplomaEmitido(new Date());
	  
	  //RF_15_30 - Redeseño da firma nos diplomas
	  if(c.getPlantillaDiploma() != null)
		  matricula.setPlantillaDiplomaEmitido(c.getPlantillaDiploma());
	  else
		  matricula.setPlantillaDiplomaEmitido(sc.getPlantillaDiploma());
	  
	  disableMessages();
	  persist(matricula);
	  enableMessages();
  }

  @SuppressWarnings("unchecked")
  private void enviarMailMatriculado(MatriculaCurso matriculaCurso) {
    String sendmail = "";
    String email = "";
    NotificationService n = new NotificationService();

    // se mira el parámetro general sobre mails activados o desactivados.
    List<Parameter> parameters = findWhere(Parameter.class, "nombre", "mail_activate");
    if (parameters.size() > 0) {
      sendmail = parameters.get(0).getValor();
    }

    if (sendmail.equals("1")) {
      UsuarioCyL u = this.find(UsuarioCyL.class, matriculaCurso.getUsuario().getId());
      if (u.iscomunicacionMail()) // el usuario desea notificaciones
      {
        email = ((UsuarioCyL) matriculaCurso.getUsuario()).getEmail();
        if (email.isEmpty() || email == null) {
          addWarnMessage("mail.nomailavailable");
        } else {
          if (n.notificarSuscritoActividad(this, matriculaCurso.getUsuario().getId(), email, matriculaCurso.getCurso())) addInfoMessage("mail.email");
          else addWarnMessage("mail.errormail");// ha ocurrido algún tipo de error.
        }
      } else {
        addInfoMessage("mail.noemail");// el usuario no desea notificaciones por correo electrónico.
      }
    } else {// está desactivado el envío de correos electrónicos en el sistema.
      addWarnMessage("mail.nonotifications");
    }
  }
  
  
  /**
   * Persist usuario.
   * 
   * @param usuario
   *           usuario
   * @param matriculacurso
   *           matriculacurso       
   * @return true, if successful
   */
  @SuppressWarnings("unchecked")
  private void notificarPrematriculaCurso(MatriculaCurso matriculaCurso, Usuario usuario) {
	  
	  NotificationService n = new NotificationService();
	  String mailActivate = "";
	  String mailRequestenrollmentActivate = "";
	  
	  
	  try {
		  List<Parameter> parameters1 = findWhere(Parameter.class, "nombre", "mail_activate");
		  if (parameters1.size() > 0)
			  mailActivate = parameters1.get(0).getValor();

		  List<Parameter> parameters2 = findWhere(Parameter.class, "nombre", "mail_requestenrollment_activate");
		  if (parameters2.size() > 0)
			  mailRequestenrollmentActivate = parameters2.get(0).getValor();

		  if (mailActivate.equals("1")){
			  if (mailRequestenrollmentActivate.equals("1")){
				  if (n.notificarPreMatriculaCurso(this, matriculaCurso, usuario))
					  addInfoMessage("mail.email");
				  else
					  addErrorMessage("mail.errormail");
			  }
			  else{
				  // están desactivadas las notificaciones por email asociadas a las solicitudes de matícula en una actividad.
				  // se obvia la advertencia puesto que el destinatario siempre será un ciudadano a través de la web de inscripción.
				  //addWarnMessage("mail.requestenrollment.nonotifications");
			  }
		  }
		  else {
			  // están desactivadas las notificaciones por email en el sistema.
			  // se obvia la advertencia puesto que el destinatario siempre será un ciudadano a través de la web de inscripción.
			  //addWarnMessage("mail.nonotifications");
		  }
	  } catch (Exception e){
		  e.printStackTrace();
	  }
  }
  
  /**
   * notificación cancelación de una prematricula curso de un usuario.
   * 
   * @param idUsuario
   *           idUsuario    
   * @return true, if successful
   */
  @SuppressWarnings("unchecked")
  private void notificarCancelacionPreMatriculaCurso(Long idUsuario, MatriculaCurso matriculaCurso) {
	  
	  NotificationService n = new NotificationService();
	  String mailActivate = "";
	  String mailRequestenrollmentActivate = "";
	  
	  
	  try {
		  List<Parameter> parameters1 = findWhere(Parameter.class, "nombre", "mail_activate");
		  if (parameters1.size() > 0)
			  mailActivate = parameters1.get(0).getValor();

		  List<Parameter> parameters2 = findWhere(Parameter.class, "nombre", "mail_requestenrollment_activate");
		  if (parameters2.size() > 0)
			  mailRequestenrollmentActivate = parameters2.get(0).getValor();

		  if (mailActivate.equals("1")){
			  if (mailRequestenrollmentActivate.equals("1")){
				  if (n.notificarCancelacionPreMatriculaCurso(this, idUsuario, matriculaCurso))
					  addInfoMessage("mail.email");
				  else
					  addErrorMessage("mail.errormail");
			  }
			  else{
				  // están desactivadas las notificaciones por email asociadas a las solicitudes de matícula en una actividad.
				  // se obvia la advertencia puesto que el destinatario siempre será un ciudadano a través de la web de inscripción.
				  //addWarnMessage("mail.requestenrollment.nonotifications");
			  }
		  }
		  else {
			  // están desactivadas las notificaciones por email en el sistema.
			  // se obvia la advertencia puesto que el destinatario siempre será un ciudadano a través de la web de inscripción.
			  //addWarnMessage("mail.nonotifications");
		  }
	  } catch (Exception e){
		  e.printStackTrace();
	  }
  }
  
  /**
   * Persist usuario.
   * 
   * @param usuario
   *           usuario
   * @param matriculacurso
   *           matriculacurso       
   * @return true, if successful
   */
  @SuppressWarnings("unchecked")
  private void notificarMatriculadoCurso(MatriculaCurso matriculaCurso) {
	  
	  NotificationService n = new NotificationService();
	  String mailActivate = "";
	  String mailRequestenrollmentActivate = "";
	  
	  
	  try {
		  List<Parameter> parameters1 = findWhere(Parameter.class, "nombre", "mail_activate");
		  if (parameters1.size() > 0)
			  mailActivate = parameters1.get(0).getValor();

		  List<Parameter> parameters2 = findWhere(Parameter.class, "nombre", "mail_enrollment_activate");
		  if (parameters2.size() > 0)
			  mailRequestenrollmentActivate = parameters2.get(0).getValor();

		  if (mailActivate.equals("1")){
			  if (mailRequestenrollmentActivate.equals("1")){
				  if (n.notificarMatriculaCurso(this, matriculaCurso))
					  addInfoMessage("mail.email");
				  else
					  addErrorMessage("mail.errormail");
			  }
			  else{
				  // están desactivadas las notificaciones por email asociadas a las matriculaciones en una actividad.
				  addWarnMessage("mail.enrollment.nonotifications");
			  }
		  }
		  else {
			  // están desactivadas las notificaciones por email en el sistema.
			  addWarnMessage("mail.nonotifications");
		  }
	  } catch (Exception e){
		  e.printStackTrace();
	  }
  }

  /*private boolean isYaMatriculado(Usuario u, Curso c) {
    if (u != null && u.getMatriculasCursos() != null) {
    	for (MatriculaCurso p : u.getMatriculasCursos()) {
    		if (p.getCurso().equals(c) && p.getCurso().getCentro().equals(c.getCentro())) {
    			return true;
    		}
    	}
    }

    return false;
  }*/
  
  private boolean isYaMatriculado(Usuario u, Long idMatriculaCurso, Curso c) {
	  if ((u != null) && (u.getMatriculasCursos()!= null)) {
		  if (idMatriculaCurso != 0) {
			  for (MatriculaCurso p : u.getMatriculasCursos()) {
				  if (p.getCurso().equals(c) && p.getCurso().getCentro().equals(c.getCentro()) &&
					  (p.getId() == idMatriculaCurso) && !p.isCancelada()) {
					  return true;
				  }				  
			  }
		  } else {
			  for (MatriculaCurso p : u.getMatriculasCursos()) {
				  if (p.getCurso().equals(c) && p.getCurso().getCentro().equals(c.getCentro())  && !p.isCancelada()) {
					  return true;
				  }				  
			  }
		  }
	  }
	  return false;
  }
  

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaCursoService#removeMatriculaCurso(es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso)
   */
  public void removeMatriculaCurso(MatriculaCurso matriculaCurso) {
    if (matriculaCurso.getId() != 0) {
      if (matriculaCurso.getCurso() != null)
    	  matriculaCurso.getCurso().getMatriculas().remove(matriculaCurso);
      
      if (matriculaCurso.getUsuario().getMatriculasCursos() != null)
    	  matriculaCurso.getUsuario().getMatriculasCursos().remove(matriculaCurso);
      
      if (!matriculaCurso.isEnListaEspera())
    	  notificaMatriculaBorrada(matriculaCurso);
      
      disableMessages();
      remove(matriculaCurso);
      enableMessages();
    }
  }
 
  
  /**
   * notificación que se ha borrado una matricula de un usuario
   * 
   * @param matriculaCurso
   *           matriculaCurso  
   * @return true, if successful
   */
  @SuppressWarnings("unchecked")
  private void notificaMatriculaBorrada(MatriculaCurso matriculaCurso) {
	  
	  NotificationService n = new NotificationService();
	  String mailActivate = "";
	  String mailRequestenrollmentActivate = "";  
	 
	  
	  try {
		  List<Parameter> parameters1 = findWhere(Parameter.class, "nombre", "mail_activate");
		  if (parameters1.size() > 0)
			  mailActivate = parameters1.get(0).getValor();

		  List<Parameter> parameters2 = findWhere(Parameter.class, "nombre", "mail_enrollment_activate");
		  if (parameters2.size() > 0)
			  mailRequestenrollmentActivate = parameters2.get(0).getValor();

		  if (mailActivate.equals("1")){
			  if (mailRequestenrollmentActivate.equals("1")){
				  if (n.notificarBorrarMatriculaCursoUsuario(this, matriculaCurso))
					  addInfoMessage("mail.email");
				  else
					  addErrorMessage("mail.errormail");
			  }
			  else{
				  // están desactivadas las notificaciones por email asociadas a las matriculaciones en una actividad.
				  addWarnMessage("mail.enrollment.nonotifications");
			  }
		  }
		  else { 
			  // están desactivadas las notificaciones por email en el sistema.
			  addWarnMessage("mail.nonotifications");
		  }
	  } catch (Exception e){
		  e.printStackTrace();
	  }
  }
  

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaCursoService#getCursos()
   */
  public List<Curso> getCursos() {
    if (cursos == null) cursos = findAllOrder(Curso.class, "fechaInicio", true);
    return cursos;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaCursoService#getUsuarios()
   */
  public List<Usuario> getUsuarios() {
    if (usuarios == null) usuarios = findAllOrder(Usuario.class, "nombre", true);
    return usuarios;

  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaCursoService#actualizaValores(es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso, java.lang.Long, java.lang.Long)
   */
  public void actualizaValores(MatriculaCurso m, Long idCurso, Long idUsuario) {
	  if ((m != null) && (m.getId() == 0)) {
		  if (idCurso != null)
			  m.setCurso(find(Curso.class, idCurso));
		  
		  if (idUsuario != null)
			  m.setUsuario(find(UsuarioCyL.class, idUsuario));
	  }
  }

  /**
   * Checks if is rellenar resultado.
   * 
   * @return true, if is rellenar resultado
   */
  public boolean isRellenarResultado() {
    return rellenarResultado;
  }

  /**
   * Asigna rellenar resultado.
   * 
   * @param rellenarResultado
   *          rellenar resultado
   */
  public void setRellenarResultado(boolean rellenarResultado) {
    this.rellenarResultado = rellenarResultado;
  }

  /**
   * Devuelve preinscripcion.
   * 
   * @return preinscripcion
   */
  public PreinscripcionCurso getPreinscripcion() {
    return preinscripcion;
  }

  /**
   * Asigna preinscripcion.
   * 
   * @param presinscripcion
   *          preinscripcion
   */
  public void setPreinscripcion(PreinscripcionCurso presinscripcion) {
    this.preinscripcion = presinscripcion;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaCursoService#getCursosLanguageDelCentro(es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap)
   */
  public List<CursoLanguage> getCursosLanguageDelCentro(UsuarioLdap u) {
    List<CursoLanguage> cl = new ArrayList<CursoLanguage>();
    
    String idioma = LocaleContextHolder.getLocaleContext().getLocale().toString();
    
    HashMap<String, Object> propiedades = new HashMap<String, Object>();
    
    if (!u.getRolesList().contains(Role.ROLE_ADMIN))
    	propiedades.put("curso.centro", u.getCentro());

    propiedades.put("language.ISO", idioma);
    
    if (u.getCentro() != null) return findWhereOrder(CursoLanguage.class, propiedades, "curso.fechaInicio", true);
    return cl;
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaCursoService#getCursosDelCentro(es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap)
   */
  public List<Curso> getCursosDelCentro(UsuarioLdap u) {
    List<Curso> c = new ArrayList<Curso>();
    
    if (u.getCentro() != null) {
    	if (!u.getRolesList().contains(Role.ROLE_ADMIN))
    		return findWhereOrder(Curso.class, "centro", u.getCentro(), "fechaInicio", true);
    	else
    		return findAllOrder(Curso.class, "fechaInicio", true);
    }
    
    return c;
  }
  
  public List<MatriculaCurso> findSolicitudesWeb(Long idCurso) {
	  List<MatriculaCurso> lmc = new ArrayList<MatriculaCurso>();
	  
	  if (idCurso != null) {
		  Map<String, Object> propiedades = new HashMap<String, Object>();
		  
		  propiedades.put("curso.id", idCurso);
		  propiedades.put("preMatriculadoWeb", true);
		  propiedades.put("enListaEspera", true);
		  propiedades.put("admitido", false);
		  
		  try {
			  lmc = findWhere(propiedades);
		  }
		  catch (NoResultException nre) {

		  }
		  catch (Exception e) {
			  logger.error("MatriculaCursoServiceImpl.findSolicitudesWeb: Error al extraer solicitudes de matrícula de la base de datos." +
					  	   e.getLocalizedMessage());

			  e.printStackTrace();
		  }
	  }
	  
	  return lmc;
  }
  
  public void setEncuestaToEditable(Long idMatriculaCurso) {
	  MatriculaCurso m = find(idMatriculaCurso);
	  
	  m.setFormularioConcretoEditable(true);
	  
	  disableMessages();
	  persist(m);
	  enableMessages();
  }
  
  public void deleteEncuestaFC(Long idMatriculaCurso) {
	  MatriculaCurso m = find(idMatriculaCurso);

	  FormularioConcreto fc = m.getFormularioConcreto();
	  
	  Map<Pregunta, Respuesta> mpr = fc.getRespuestas();
	  
	  disableMessages();
	  
	  for (Map.Entry<Pregunta, Respuesta> pr : mpr.entrySet()) {
		  Respuesta r = pr.getValue();

		  pr.setValue(null);
		  remove(r);
	  }

	  fc.getRespuestas().clear();
	  persist(fc);

	  enableMessages();
  }
  
  public void setEncuestaToNotEditable(Long idMatriculaCurso) {
	  MatriculaCurso m = find(idMatriculaCurso);
	  
	  m.setFormularioConcretoEditable(false);
	  
	  disableMessages();
	  persist(m);
	  enableMessages();
  }
  
  public List<MatriculaCurso> getMatriculaMultipleCurso() {
	  if (matriculaMultipleCurso == null)
		  matriculaMultipleCurso = new ArrayList<MatriculaCurso>();
	  
	  return matriculaMultipleCurso;
  }
  
  public void setMatriculaMultipleCurso(List<MatriculaCurso> lmc) {
	  matriculaMultipleCurso.clear();
	  
	  matriculaMultipleCurso.addAll(lmc);
  }
  
  public List<MatriculaCurso> createTemporalMatriculaMultipleCurso() {
	  List<MatriculaCurso> tmmc = new ArrayList<MatriculaCurso>();
	  
	  tmmc.addAll(matriculaMultipleCurso);
	  
	  return tmmc;
  }
  
  public MatriculaCurso anyadeUsuarioAMatricular(List<MatriculaCurso> tmmc, Long idCurso, Usuario alumno) {
	  MatriculaCurso mc = new MatriculaCurso();
	  
	  actualizaValores(mc, idCurso, alumno.getId());

	  tmmc.add(mc);
	  
	  return mc;
  }
  
  public void removeUsuarioAMatricular(List<MatriculaCurso> tmmc, Usuario alumno) {
	  for (MatriculaCurso mc : tmmc) {
		  if (mc.getUsuario().getId() == alumno.getId()) {
			  tmmc.remove(mc);
			  
			  break;
		  }
	  }
  }
  
  public void purgeTemporalMatriculaMultiple(List<MatriculaCurso> tmmc) {
	  tmmc.clear();
  }
  
}
