package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.language.entity.ActividadExternaLanguage;
import es.csa.fdt.cyldigital.notificacion.NotificationService;
import es.csa.fdt.cyldigital.servicios.entity.ActividadExterna;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaExterna;
import es.csa.fdt.cyldigital.servicios.entity.PreinscripcionCurso;
import es.csa.fdt.cyldigital.servicios.entity.PreinscripcionTaller;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;


/**
 * Clase MatriculaExternaServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class MatriculaExternaServiceImpl extends AbstractServiceImpl<MatriculaExterna, Long> implements
    MatriculaExternaService {

  private List<ActividadExterna> externas;

  private List<Usuario> usuarios;

  private boolean rellenarResultado = false;

  private PreinscripcionCurso preinscripcionCurso;

  private PreinscripcionTaller preinscripcionTaller;

  private static final long serialVersionUID = -5575228180361917674L;

  /**
   * Construye un nuevo objeto de la clase
   */
  public MatriculaExternaServiceImpl() {
    super();
    setPreinscripcionCurso(new PreinscripcionCurso());
    setPreinscripcionTaller(new PreinscripcionTaller());
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaExternaService#persistMatriculaExterna(es.csa.fdt.cyldigital.servicios.entity.MatriculaExterna)
   */
  @SuppressWarnings("unchecked")
  public boolean persistMatriculaExterna(MatriculaExterna matriculaExterna) {
    NotificationService n = new NotificationService();
    String sendmail = "", email;

    if (matriculaExterna.getId() == 0 && isYaMatriculado(matriculaExterna.getUsuario(), matriculaExterna.getExterna())) {
      return false;
    } else if (matriculaExterna.getId() == 0 && matriculaExterna.getUsuario() == null) {
      addErrorMessage("matricula.validator.usuario_necesario");
      return false;
    } else if (matriculaExterna.getId() == 0
        /*&& matriculaExterna.getExterna().getMatriculas().size() >= matriculaExterna.getExterna().getNumeroPlazas()*/) {
        //es una reserva nueva.
      
      matriculaExterna.setEnListaEspera(true);
      disableMessages();
      persist(matriculaExterna);
      enableMessages();
      /*if (matriculaExterna.getExterna().isCurso()) {
          /*
           * Comentado para implementar los servicios MULTILANGUAGE
           * TODO: mirar y modificar
        addInfoMessage("preinscripcion.usuario.preinscrito", new Object[] {
            matriculaExterna.getUsuario().getNombreCompleto(),
            ((ServicioCurso) matriculaExterna.getExterna().getServicio()).getNombre() });
            */
     /* } else if (matriculaExterna.getExterna().isTaller()) {
          /*
           * Comentado para implementar los servicios MULTILANGUAGE
           * TODO: mirar y modificar
        addInfoMessage("preinscripcion.usuario.preinscrito", new Object[] {
            matriculaExterna.getUsuario().getNombreCompleto(),
            ((ServicioTaller) matriculaExterna.getExterna().getServicio()).getNombre() });
            */
     /* }*/

      // por último notificamos la matriculación
      List<Parameter> parameters = findWhere(Parameter.class, "nombre", "mail_activate");
      if (parameters.size() == 1) {
        sendmail = parameters.get(0).getValor();
      }
      if (sendmail.equals("1")) {
        UsuarioCyL u = this.find(UsuarioCyL.class, matriculaExterna.getUsuario().getId());
        if (u.iscomunicacionMail())// el usuario quiere recibir notificaciones por correo.
        { // el usuario desea notificaciones por email
          email = ((UsuarioCyL) matriculaExterna.getUsuario()).getEmail();
          if (email.isEmpty() || email == null) {
            addWarnMessage("mail.nomailavailable");
          } else // no tenemos el mail del usuario
          {
            if (n.notificarSuscritoActividad(this, matriculaExterna.getUsuario().getId(), email, matriculaExterna
                .getExterna())) addInfoMessage("mail.email");
            else addWarnMessage("mail.errormail"); // ha ocurrido algún error
          }
        } else {// el usuario no desea recibir notificaciones.
          addInfoMessage("mail.noemail");
        }
      } else {// las notificaciones del sistema están desactivadas.
        addWarnMessage("mail.nonotifications");
      }

      return true;
    } else {
      disableMessages();
      persist(matriculaExterna);
      return true;
    }
  }

  private boolean isYaMatriculado(Usuario u, ActividadExterna c) {
    if (u != null && u.getMatriculasExternas() != null) for (MatriculaExterna p : u.getMatriculasExternas()) {
      if (p.getExterna().equals(c) && p.getExterna().getCentro().equals(c.getCentro())) {
        addErrorMessage("matricula.validator.yainscrito");
        return true;
      }
    }
    return false;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaExternaService#removeMatriculaExterna(es.csa.fdt.cyldigital.servicios.entity.MatriculaExterna)
   */
  public void removeMatriculaExterna(MatriculaExterna matriculaExterna) {
    if (matriculaExterna.getId() != 0) {
      /*if (matriculaExterna.getExterna() != null)
        matriculaExterna.getExterna().getMatriculas().remove(matriculaExterna);*/
      if (matriculaExterna.getUsuario().getMatriculasExternas() != null)
        matriculaExterna.getUsuario().getMatriculasExternas().remove(matriculaExterna);
      notificaMatriculaBorrada(matriculaExterna);
      remove(matriculaExterna);
    }
  }

  @SuppressWarnings("unchecked")
  private void notificaMatriculaBorrada(MatriculaExterna matriculaExterna) {
    NotificationService n = new NotificationService();
    String sendmail = "";
    List<Parameter> parameters = findWhere(Parameter.class, "nombre", "mail_activate");
    if (parameters.size() == 1) {
      sendmail = parameters.get(0).getValor();
    }

    if (sendmail.equals("1")) {
      UsuarioCyL u = this.find(UsuarioCyL.class, matriculaExterna.getUsuario().getId());
      if (u.iscomunicacionMail()) {
        if (n.notificarDesuscritoActividad(this, matriculaExterna.getUsuario(), matriculaExterna.getExterna())) addInfoMessage("mail.email");
        else addWarnMessage("mail.errormail");
      } else {
        addInfoMessage("mail.noemail");
      }
    } else addWarnMessage("mail.nonotifications");
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaExternaService#getExternas()
   */
  public List<ActividadExterna> getExternas() {
    if (externas == null) externas = findAllOrder(ActividadExterna.class, "fechaInicio", true);
    return externas;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaExternaService#getUsuarios()
   */
  public List<Usuario> getUsuarios() {
    if (usuarios == null) usuarios = findAllOrder(Usuario.class, "nombre", true);
    return usuarios;

  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaExternaService#actualizaValores(es.csa.fdt.cyldigital.servicios.entity.MatriculaExterna, java.lang.Long, java.lang.Long)
   */
  public void actualizaValores(MatriculaExterna m, Long idExterna, Long idUsuario) {
    if (m != null && m.getId() == 0) {
      if (idExterna != null) m.setExterna(find(ActividadExterna.class, idExterna));
      if (idUsuario != null) m.setUsuario(find(Usuario.class, idUsuario));
    }
  }

  /**
   * Checks if is rellenar resultado.
   * 
   * @return true, if is rellenar resultado
   */
  public boolean isRellenarResultado() {
    return rellenarResultado;
  }

  /**
   * Asigna rellenar resultado.
   * 
   * @param rellenarResultado
   *          rellenar resultado
   */
  public void setRellenarResultado(boolean rellenarResultado) {
    this.rellenarResultado = rellenarResultado;
  }

  /**
   * Devuelve preinscripcion curso.
   * 
   * @return preinscripcion curso
   */
  public PreinscripcionCurso getPreinscripcionCurso() {
    return preinscripcionCurso;
  }

  /**
   * Asigna preinscripcion curso.
   * 
   * @param presinscripcion
   *          preinscripcion curso
   */
  public void setPreinscripcionCurso(PreinscripcionCurso presinscripcion) {
    this.preinscripcionCurso = presinscripcion;
  }

  /**
   * Devuelve preinscripcion taller.
   * 
   * @return preinscripcion taller
   */
  public PreinscripcionTaller getPreinscripcionTaller() {
    return preinscripcionTaller;
  }

  /**
   * Asigna preinscripcion taller.
   * 
   * @param preinscripcionTaller
   *          preinscripcion taller
   */
  public void setPreinscripcionTaller(PreinscripcionTaller preinscripcionTaller) {
    this.preinscripcionTaller = preinscripcionTaller;
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaCursoService#getCursosLanguageDelCentro(es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap)
   */
  public List<ActividadExternaLanguage> getExternasLanguageDelCentro(UsuarioLdap u) {
    List<ActividadExternaLanguage> cl = new ArrayList<ActividadExternaLanguage>();
    
    String idioma = LocaleContextHolder.getLocaleContext().getLocale().toString();
    
    HashMap<String, Object> propiedades = new HashMap<String, Object>();
    propiedades.put("externa.centro", u.getCentro());
    propiedades.put("language.ISO", idioma);
    
    if (u.getCentro() != null) return findWhereOrder(ActividadExternaLanguage.class, propiedades, "externa.fechaInicio", true);
    return cl;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaExternaService#getExternasDelCentro(es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap)
   */
  public List<ActividadExterna> getExternasDelCentro(UsuarioLdap u) {
    List<ActividadExterna> c = new ArrayList<ActividadExterna>();
    
    if (u.getCentro() != null) return findWhereOrder(ActividadExterna.class, "centro", u.getCentro(), "fechaInicio", true);
    return c;
  }
}
