package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.List;

import es.csa.fdt.cyldigital.core.service.AbstractService;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaTaller;
import es.csa.fdt.cyldigital.servicios.entity.Taller;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;

/**
 * Interfaz del servicio de MatriculaTaller.
 * 
 * @author csa
 */
public interface MatriculaTallerService extends AbstractService<MatriculaTaller, Long> {

  /**
   * Persist matricula taller.
   * 
   * @param matriculaTaller
   *           matricula taller
   * @param b
   *           b
   * @return true, if successful
   */
  public boolean persistMatriculaTaller(MatriculaTaller matriculaTaller, boolean b);

  /**
   * Elimina matricula taller.
   * 
   * @param matriculaTaller
   *           matricula taller
   */
  public void removeMatriculaTaller(MatriculaTaller matriculaTaller);

  /**
   * Devuelve talleres.
   * 
   * @return talleres
   */
  public List<Taller> getTalleres();

  /**
   * Devuelve usuarios.
   * 
   * @return usuarios
   */
  public List<Usuario> getUsuarios();

  /**
   * Actualiza valores.
   * 
   * @param m
   *           m
   * @param idTaller
   *           id taller
   * @param idUsuario
   *           id usuario
   */
  public void actualizaValores(MatriculaTaller m, Long idTaller, Long idUsuario);

  /**
   * Devuelve talleres del centro.
   * 
   * @param u
   *           u
   * @return talleres del centro
   */
  public List<Taller> getTalleresDelCentro(UsuarioLdap u);

}
