package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.List;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.notificacion.NotificationService;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaTaller;
import es.csa.fdt.cyldigital.servicios.entity.PreinscripcionTaller;
import es.csa.fdt.cyldigital.servicios.entity.ServicioTaller;
import es.csa.fdt.cyldigital.servicios.entity.Taller;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;


/**
 * Clase MatriculaTallerServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class MatriculaTallerServiceImpl extends AbstractServiceImpl<MatriculaTaller, Long> implements
    MatriculaTallerService {

  private List<Taller> talleres;

  private List<Usuario> usuarios;

  private boolean rellenarResultado = false;

  private PreinscripcionTaller preinscripcion;

  private static final long serialVersionUID = -5575228180361917674L;

  /**
   * Construye un nuevo objeto de la clase
   */
  public MatriculaTallerServiceImpl() {
    super();
    setPreinscripcion(new PreinscripcionTaller());
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaTallerService#persistMatriculaTaller(es.csa.fdt.cyldigital.servicios.entity.MatriculaTaller, boolean)
   */
  public boolean persistMatriculaTaller(MatriculaTaller matriculaTaller, boolean enlistaesperaanterior) {

    if (matriculaTaller.getId() == 0 && isYaMatriculado(matriculaTaller.getUsuario(), matriculaTaller.getTaller())) {
      return false;
    } else if (matriculaTaller.getId() == 0 && matriculaTaller.getUsuario() == null) {
      addErrorMessage("matricula.validator.usuario_necesario");
      return false;
    } else if (matriculaTaller.getId() == 0
        && matriculaTaller.getTaller().getMatriculas().size() >= matriculaTaller.getTaller().getNumeroPlazas()) {

      matriculaTaller.setEnListaEspera(true);
      disableMessages();
      persist(matriculaTaller);
      enableMessages();
      
      /*
       * Comentado para implementar los servicios MULTILANGUAGE
       * TODO: mirar y modificar
      addInfoMessage("preinscripcion.usuario.preinscrito", new Object[] {
          matriculaTaller.getUsuario().getNombreCompleto(),
          ((ServicioTaller) matriculaTaller.getTaller().getServicio()).getNombre() });
       */
      
      return true;
    } else { // es una matricula, no una reserva

      if (matriculaTaller.getId() == 0) {
        // la matricula es nueva, mandar el correo. Sino, es una modificacion, no enviarlo.
        // por último notificamos la matriculación
        enviarMailMatriculado(matriculaTaller);
      }

      if (matriculaTaller.getId() != 0 && matriculaTaller.isEnListaEspera() == false && enlistaesperaanterior == true) {
        if (matriculaTaller.getTaller().getNumeroMatriculados() > matriculaTaller.getTaller().getNumeroPlazas()) {
          addErrorMessage("Plazas de matriculación completas, no se puede pasar al usuario de la lista de espera a "
              + "la de matriculación.");
          return false;
        } else {
          // se ha pasado una reserva a matrícula, hay que notificarlo
          enviarMailMatriculado(matriculaTaller);
        }
      }
      disableMessages();
      persist(matriculaTaller);
      return true;
    }
  }

  @SuppressWarnings("unchecked")
  private void enviarMailMatriculado(MatriculaTaller matricula) {
    String sendmail = "", email = "";
    NotificationService n = new NotificationService();

    List<Parameter> parameters = findWhere(Parameter.class, "nombre", "mail_activate");
    if (parameters.size() > 0) {
      sendmail = parameters.get(0).getValor();
    }
    if (sendmail.equals("1")) {
      UsuarioCyL u = this.find(UsuarioCyL.class, matricula.getUsuario().getId());
      if (u.iscomunicacionMail())// el usuario desea notificaciones por email.
      {
        email = ((UsuarioCyL) matricula.getUsuario()).getEmail();
        if (email.isEmpty() || email == null) {
          addInfoMessage("mail.nomailavailable");
        } else {
          if (n.notificarSuscritoActividad(this, matricula.getUsuario().getId(), email, matricula.getTaller())) addInfoMessage("mail.email");
          else addWarnMessage("mail.errormail");// ha ocurrido algún error.
        }
      } else {
        addInfoMessage("mail.noemail");// el usuario no desea notificaciones.
      }
    } else {
      addWarnMessage("mail.nonotifications");
    }
  }

  private boolean isYaMatriculado(Usuario u, Taller c) {
    if (u != null && u.getMatriculasTalleres() != null) for (MatriculaTaller p : u.getMatriculasTalleres()) {
      if (p.getTaller().equals(c) && p.getTaller().getCentro().equals(c.getCentro())) {
        addErrorMessage("matricula.validator.yainscrito");
        return true;
      }
    }
    return false;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaTallerService#removeMatriculaTaller(es.csa.fdt.cyldigital.servicios.entity.MatriculaTaller)
   */
  public void removeMatriculaTaller(MatriculaTaller matriculaTaller) {
    if (matriculaTaller.getId() != 0) {
      if (matriculaTaller.getTaller() != null) matriculaTaller.getTaller().getMatriculas().remove(matriculaTaller);
      if (matriculaTaller.getUsuario().getMatriculasTalleres() != null)
        matriculaTaller.getUsuario().getMatriculasTalleres().remove(matriculaTaller);
      notificaMatriculaBorrada(matriculaTaller);
      remove(matriculaTaller);
    }
  }

  @SuppressWarnings("unchecked")
  private void notificaMatriculaBorrada(MatriculaTaller matriculaTaller) {
    NotificationService n = new NotificationService();
    String sendmail = "";
    List<Parameter> parameters = findWhere(Parameter.class, "nombre", "mail_activate");
    if (parameters.size() > 0) {
      sendmail = parameters.get(0).getValor();
    }

    if (sendmail.equals("1")) {
      UsuarioCyL u = this.find(UsuarioCyL.class, matriculaTaller.getUsuario().getId());
      if (u.iscomunicacionMail()) {
        if (n.notificarDesuscritoActividad(this, matriculaTaller.getUsuario(), matriculaTaller.getTaller())) addInfoMessage("mail.email");
        else addWarnMessage("mail.errormail");
      } else {
        addInfoMessage("mail.noemail");
      }
    } else addWarnMessage("mail.nonotifications");
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaTallerService#getTalleres()
   */
  public List<Taller> getTalleres() {
    if (talleres == null) talleres = findAllOrder(Taller.class, "fechaInicio", true);
    return talleres;
  };

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaTallerService#getUsuarios()
   */
  public List<Usuario> getUsuarios() {
    if (usuarios == null) usuarios = findAllOrder(Usuario.class, "nombre", true);
    return usuarios;

  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaTallerService#actualizaValores(es.csa.fdt.cyldigital.servicios.entity.MatriculaTaller, java.lang.Long, java.lang.Long)
   */
  public void actualizaValores(MatriculaTaller m, Long idTaller, Long idUsuario) {
    if (m != null && m.getId() == 0) {
      if (idTaller != null) m.setTaller(find(Taller.class, idTaller));
      if (idUsuario != null) m.setUsuario(find(Usuario.class, idUsuario));
    }
  }

  /**
   * Checks if is rellenar resultado.
   * 
   * @return true, if is rellenar resultado
   */
  public boolean isRellenarResultado() {
    return rellenarResultado;
  }

  /**
   * Asigna rellenar resultado.
   * 
   * @param rellenarResultado
   *          rellenar resultado
   */
  public void setRellenarResultado(boolean rellenarResultado) {
    this.rellenarResultado = rellenarResultado;
  }

  /**
   * Devuelve preinscripcion.
   * 
   * @return preinscripcion
   */
  public PreinscripcionTaller getPreinscripcion() {
    return preinscripcion;
  }

  /**
   * Asigna preinscripcion.
   * 
   * @param presinscripcion
   *          preinscripcion
   */
  public void setPreinscripcion(PreinscripcionTaller presinscripcion) {
    this.preinscripcion = presinscripcion;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.MatriculaTallerService#getTalleresDelCentro(es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap)
   */
  public List<Taller> getTalleresDelCentro(UsuarioLdap u) {
    List<Taller> t = new ArrayList<Taller>();
    if (u.getCentro() != null) return findWhereOrder(Taller.class, "centro", u.getCentro(), "fechaInicio", true);
    return t;
  }
}
