package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */
 
import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.commons.persistence.browser.jpaql.FilterJPAQLByQuery;
import es.csa.fdt.cyldigital.core.service.BrowserFilteredByActive;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso;

public class MatriculasCursoBrowserImpl extends BrowserFilteredByActive<MatriculaCurso> implements MatriculasCursoBrowser<CriterionJPAQL, String>{
	
	private static final long serialVersionUID = 6541032809852684406L;
	
	private String customFilterProperty;

	public MatriculasCursoBrowserImpl(){
		super();
		getSort().setSortProperty("idMatriculaCurso");
	}
	
	public void setFilterByCurso(Long idCurso) {
		removeFilterCurso();
		filterByCurso(idCurso);
	}
	
	private void removeFilterCurso() {
	    int i = -1;
	    for (Filter<CriterionJPAQL, MatriculaCurso> f : getFilters())
	      if (f.getCriterion().getQuery().contains("idCurso")) i = getFilters().indexOf(f);
	    if (i > -1) getFilters().remove(i);
	  }

	/**
	 * Filter by curso.
	 * 
	 * @param idCurso
	 *           id curso
	 */
	public void filterByCurso(Long idCurso) {
	    String query = "";

	    boolean flag = false;
	    FilterJPAQLByQuery<MatriculaCurso> filter = new FilterJPAQLByQuery<MatriculaCurso>();

	    query += " idCurso = ? ";
	    filter.getCriterion().addParameter(idCurso);
	    
	    query += " AND (enListaEspera = 0 AND cancelada = 0)";

	    for (Filter<CriterionJPAQL, MatriculaCurso> f : getFilters())
	      if (f.getCriterion().getQuery().contains("idCurso")) flag = true;

	    if (!flag) {
	      filter.getCriterion().setQuery(query);
	      getFilters().add(filter);
	    }
	  }
	
	public void applyCustomFilter() {
		  
		  String query = "COALESCE(trim(usuario.nif),'') LIKE ? OR " +
				         "COALESCE(trim(usuario.telefono),'') LIKE ? OR " +
				         "concat(COALESCE(trim(usuario.nombre),''),concat(' ',concat(COALESCE(trim(usuario.apellido1),''),concat(' ',COALESCE(trim(usuario.apellido2),''))))) LIKE ?";

		  Long id = null;
		  clearFilters();
		  if (getCustomFilterProperty() != null && !getCustomFilterProperty().trim().isEmpty()) {
			  try {
				  id = Long.parseLong(customFilterProperty);
			  }
			  catch (Exception e) {
			  }

			  FilterJPAQLByQuery<MatriculaCurso> filter = new FilterJPAQLByQuery<MatriculaCurso>();
			  if (id != null) {
				  query += " OR id = ?";

				  Object[] params = { "%" + customFilterProperty + "%",
						              "%" + customFilterProperty + "%",
						              "%" + customFilterProperty + "%",
						               id };

				  filter.getCriterion().addParameters(params);
			  }
			  else {
				  Object[] params = { "%" + customFilterProperty + "%",
						              "%" + customFilterProperty + "%",
						              "%" + customFilterProperty + "%" };

				  filter.getCriterion().addParameters(params);
			  }

			  filter.getCriterion().setQuery(query);
			  filters.add(filter);
		  }

		  //setCustomFilterProperty("");

	  }
	  /* (non-Javadoc)
	   * @see es.csa.fdt.cyldigital.usuarios.service.UsuariosBrowser#setCustomFilterProperty(java.lang.String)
	   */
	  public void setCustomFilterProperty(String customFilter) {
	    this.customFilterProperty = customFilter;
	  }

	  /* (non-Javadoc)
	   * @see es.csa.fdt.cyldigital.usuarios.service.UsuariosBrowser#getCustomFilterProperty()
	   */
	  public String getCustomFilterProperty() {
	    return customFilterProperty;
	  }
	  
	  public void borrarTextoFiltro(){
		  setCustomFilterProperty("");
	  }
}
