package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.centros.service.CentroServiceImpl;
import es.csa.fdt.cyldigital.core.service.BrowserFilteredByCenter;
import es.csa.fdt.cyldigital.servicios.entity.Navegacion;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;

/**
 * Servicio del Browser de Navegaciones.
 * 
 * @author csa
 */
@Service
@Repository
public class NavegacionBrowserImpl extends BrowserFilteredByCenter<Navegacion> implements
    NavegacionBrowser<CriterionJPAQL, String> {

  private static final long serialVersionUID = -7227366052245717441L;

  /**
   * Construye un nuevo objeto de la clase
   */
  public NavegacionBrowserImpl() {
    super();
    getSort().setSortAsc(false);
    getSort().setSortProperty("id");
  }

  /**
   * Asigna filter centro.
   * 
   * @param usuario
   *           usuario
   * @param serviceCentros
   *           service centros
   */
  public void setFilterCentro(UsuarioLdap usuario, CentroServiceImpl serviceCentros) {
    super.user = usuario;
    if (centro == null) {
      if (user.applyFilterCentro() || user.getCentro() != null) {
        super.removeFilterCentro();
        if (user.getCentro() != null) {
          filterByCentro(user.getCentro().getId());
          super.centro = serviceCentros.find(Centro.class, user.getCentro().getId());
        }
      }
    }
  }

  /**
   * Refresh all records.
   */
  public void refreshAllRecords() {
    for (Navegacion nav : getRecords()) {
      getEntityManager().refresh(nav);
    }
  }

}
