package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.Query;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.fdt.cyldigital.calendario.entity.ActividadCalendarItem;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.centros.service.CentroServiceImpl;
import es.csa.fdt.cyldigital.core.service.BrowserFilteredByCenter;
import es.csa.fdt.cyldigital.recursos.entity.Ordenador;
import es.csa.fdt.cyldigital.servicios.entity.Navegacion;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;

/**
 * Servicio del Browser de Navegacion de Ordenadores.
 * 
 * @author csa
 */
@Service
@Repository
public class NavegacionOrdenadoresBrowserImpl extends BrowserFilteredByCenter<Ordenador> implements
    NavegacionOrdenadorBrowser<CriterionJPAQL, String> {

  private static final long serialVersionUID = -7227366052245717441L;

  private List<Ordenador> ordenadores;

  private List<Navegacion> navegaciones;

  /**
   * Construye un nuevo objeto de la clase
   */
  public NavegacionOrdenadoresBrowserImpl() {
    super();
    getSort().setSortAsc(true);
    getSort().setSortProperty("ip");
    ordenadores = new ArrayList<Ordenador>();
    navegaciones = new ArrayList<Navegacion>();
  }

  /**
   * Devuelve ordenadores.
   * 
   * @return ordenadores
   */
  public List<Ordenador> getOrdenadores() {
    if (ordenadores == null) {
      ordenadores = new ArrayList<Ordenador>();
    }
    return ordenadores;
  }

  /**
   * Asigna ordenadores.
   * 
   * @param ordenadores
   *          ordenadores
   */
  public void setOrdenadores(List<Ordenador> ordenadores) {
    this.ordenadores = ordenadores;
  }

  /**
   * Devuelve navegaciones.
   * 
   * @return navegaciones
   */
  @SuppressWarnings("unchecked")
  public List<Navegacion> getNavegaciones() {
    if (navegaciones == null || navegaciones.isEmpty()) {
      String consulta = "SELECT e FROM " + ActividadCalendarItem.class.getName()
          + " e WHERE ActividadType = ? and start <= ? and end >= ? ";
      Query query = getEntityManager().createQuery(consulta);
      query.setParameter(1, ActividadCalendarItem.ActividadType.NAVEGACION);
      query.setParameter(2, new Date());
      query.setParameter(3, new Date());
      List<ActividadCalendarItem> items = (List<ActividadCalendarItem>) query.getResultList();
      for (ActividadCalendarItem item : items) {
        if (navegaciones.contains(item.getActividad())) navegaciones.add((Navegacion) item.getActividad());
      }
    }
    return navegaciones;
  }

  /**
   * Asigna navegaciones.
   * 
   * @param navegaciones
   *          navegaciones
   */
  public void setNavegaciones(List<Navegacion> navegaciones) {
    this.navegaciones = navegaciones;
  }

  /**
   * Asigna records to ordenadores.
   */
  public void setRecordsToOrdenadores() {
    ordenadores.clear();
    for (Ordenador o : getRecords()) {
      if (o.getAula() != null && o.getAula().isNavegacionControlada()) ordenadores.add(o);
    }
  }

  /**
   * Asigna filter centro.
   * 
   * @param usuario
   *           usuario
   * @param serviceCentros
   *           service centros
   */
  public void setFilterCentro(UsuarioLdap usuario, CentroServiceImpl serviceCentros) {
    super.user = usuario;
    if (centro == null) {
      if (user.applyFilterCentro() || user.getCentro() != null) {
        super.removeFilterCentro();
        if (user.getCentro() != null) {
          filterByCentro(user.getCentro().getId());
          super.centro = serviceCentros.find(Centro.class, user.getCentro().getId());
        }
      }
    }
  }

  /**
   * Refresh all records.
   */
  public void refreshAllRecords() {
    for (Ordenador o : getOrdenadores()) {
      getEntityManager().refresh(o);
    }
  }

  /**
   * Clear entity manager.
   */
  @SuppressWarnings("unchecked")
  public void clearEntityManager() {
    getEntityManager().clear();
    navegaciones.clear();
    String consulta = "SELECT e FROM " + ActividadCalendarItem.class.getName()
        + " e WHERE ActividadType = ? and start <= ? ";
    Query query = getEntityManager().createQuery(consulta);
    query.setParameter(1, ActividadCalendarItem.ActividadType.NAVEGACION);
    query.setParameter(2, new Date());
    List<ActividadCalendarItem> items = (List<ActividadCalendarItem>) query.getResultList();
    for (ActividadCalendarItem item : items) {
      if (navegaciones.contains(item.getActividad())) navegaciones.add((Navegacion) item.getActividad());
    }
  }
}
