package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import javax.persistence.Query;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.faces.util.PredefinedLocaleResolver;
import es.csa.fdt.cyldigital.calendario.entity.ActividadCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.UserActivityCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.ActividadCalendarItem.ActividadType;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.recursos.entity.Ordenador;
import es.csa.fdt.cyldigital.servicios.entity.Navegacion;
import es.csa.fdt.cyldigital.servicios.entity.ObservacionActividad;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario.Role;

/**
 * Clase NavegacionServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class NavegacionServiceImpl extends AbstractServiceImpl<Navegacion, Long> implements NavegacionService {

  private static final int DURACION = 60;

  private int duracion;

  private List<Centro> centros;

  private static final long serialVersionUID = -5575228180361917674L;

  private List<Navegacion> navegaciones;

  private String observacion;

  private Long idUsuario;

  private boolean warningMostrado = false;

  /**
   * Construye un nuevo objeto de la clase
   */
  public NavegacionServiceImpl() {
    super();
    duracion = DURACION;
  }

  /**
   * Devuelve duracion.
   * 
   * @return duracion
   */
  public int getDuracion() {
    return duracion;
  }

  /**
   * Asigna duracion.
   * 
   * @param duracion
   *          duracion
   */
  public void setDuracion(int duracion) {
    this.duracion = duracion;
  }

  /**
   * Devuelve id usuario.
   * 
   * @return id usuario
   */
  public Long getIdUsuario() {
    return idUsuario;
  }

  /**
   * Asigna id usuario.
   * 
   * @param idUsuario
   *          id usuario
   */
  public void setIdUsuario(Long idUsuario) {
    this.idUsuario = idUsuario;
  }

  /**
   * Actualiza ordenador.
   * 
   * @param navegacion
   *           navegacion
   * @param idOrdenador
   *           id ordenador
   */
  public void actualizaOrdenador(Navegacion navegacion, Long idOrdenador) {
    if (idOrdenador != null && idOrdenador > 0L && navegacion != null) {
      Ordenador o = find(Ordenador.class, idOrdenador);
      if (o != null) {
        navegacion.setOrdenador(o);
        navegacion.setCentro(o.getCentro());
      }
    }
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.NavegacionService#persistNavegacion(es.csa.fdt.cyldigital.servicios.entity.Navegacion, es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap)
   */
  public boolean persistNavegacion(Navegacion navegacion, UsuarioLdap u) {
    Usuario us = find(Usuario.class, idUsuario);
    
    if (us == null) {
      addErrorMessage("El usuario no se encuentra en nuestra la base de datos");
      return false;
    } else {

      if (us.getDesactivado()) {
        addErrorMessage("El usuario está desactivado, debe darlo de alta de nuevo para poder operar con el");
        return false;
      }

      if (!compruebaUsuario(us)) { return false; }
      navegacion.setUsuario(us);
      if (navegacion.getId() == 0) {
        ActividadCalendarItem ac = new ActividadCalendarItem();
        Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
        cal.setTime(new Date());
        ac.setActividadType(ActividadType.NAVEGACION);
        ac.setActividad(navegacion);
        ac.setUsuario(navegacion.getUsuario());
        ac.setStart(cal.getTime());
        Calendar cal2 = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
        cal2.setTimeInMillis(cal.getTimeInMillis());
        cal2.set(Calendar.MINUTE, cal.get(Calendar.MINUTE) + getDuracion());
        ac.setEnd(cal2.getTime());
        navegacion.addCalendarItem(ac);

        Navegacion anterior = getNavegacion(navegacion.getOrdenador(), getNavegacionesActivas());
        Date now = new Date();
        if (anterior != null) {
          if (anterior.getFechaFin().after(now) && anterior.getHoraFin().after(now)) {
            anterior.setCerrada(true);
            ObservacionActividad o = new ObservacionActividad();
            o.setActividad(anterior);
            o.setFecha(new Date());
            o.setObservacion("Reasignación de ordenador a : " + navegacion.getUsuario().getNombreCompleto());
            o.setUsuario(u.getUser());
            anterior.getObservacionesActividad().add(o);
            persist(anterior);
          }
        }
      }
      if (navegacion.getOrdenador().isOcupado(navegacion)){
        addErrorMessage("El recurso ya está reservado u ocupado para otra Actividad");
        return false;
      }
      persist(navegacion);
      return true;
    }
  }

  private boolean compruebaUsuario(Usuario u) {
    if (warningMostrado) {
      warningMostrado = false;
      return true;
    }
    if (u != null) {
      int minutos = tiempoSesionUsuario(u, "hoy");
      if (minutos >= 60) {
        addWarnMessage("El usuario lleva navegando hoy " + minutos + " minutos. Si continua se le dará más tiempo");
        warningMostrado = true;
      }
      minutos = tiempoSesionUsuario(u, "semana");
      if (minutos >= 300) {
        addWarnMessage("El usuario lleva navegando esta semana " + minutos
            + " minutos. Si continua se le dará más tiempo");
        warningMostrado = true;
      }
      if (warningMostrado) return false;
    }
    return true;
  }

  @SuppressWarnings("unchecked")
  private int tiempoSesionUsuario(Usuario u, String periodo) {
    int minutos = 0;
    Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    Calendar now = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    Calendar cal2 = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    cal2.set(Calendar.HOUR_OF_DAY, 23);
    cal2.set(Calendar.MINUTE, 59);
    cal2.set(Calendar.SECOND, 59);

    if (periodo.equalsIgnoreCase("hoy")) {
      cal.setFirstDayOfWeek(Calendar.MONDAY);
      cal.set(Calendar.HOUR_OF_DAY, 0);
      cal.set(Calendar.MINUTE, 0);
      cal.set(Calendar.SECOND, 1);
    } else {
      cal.set(Calendar.HOUR_OF_DAY, 0);
      cal.set(Calendar.MINUTE, 0);
      cal.set(Calendar.SECOND, 1);
      cal.set(Calendar.DATE, (cal.get(Calendar.DATE) - (((cal.get(Calendar.DAY_OF_WEEK) - 1 == 0) ? 7 : (cal
          .get(Calendar.DAY_OF_WEEK) - 1)) - 1)));
    }

    String consulta = "SELECT e FROM " + UserActivityCalendarItem.class.getName()
        + " e WHERE usuario = ? and start >= ? " + " and start  <= ?";

    Query query = getEntityManager().createQuery(consulta);
    query.setParameter(1, u);
    query.setParameter(2, cal.getTime());
    query.setParameter(3, cal2.getTime());

    List<UserActivityCalendarItem> uas = (List<UserActivityCalendarItem>) query.getResultList();
    for (UserActivityCalendarItem ua : uas) {
      if (ua.getOrdenador() != null && ua.getOrdenador().getAula() != null
          && ua.getOrdenador().getAula().isNavegacionControlada()) {
        long end = (ua.getEnd() == null) ? now.getTimeInMillis() : ua.getEnd().getTime();
        if (!periodo.equalsIgnoreCase("hoy") && ua.getEnd() == null) {
          Calendar cal3 = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
          cal3.set(Calendar.HOUR_OF_DAY, 0);
          cal3.set(Calendar.MINUTE, 0);
          cal3.set(Calendar.SECOND, 1);
          if (cal3.before(ua.getStart()))
            minutos += (end - ua.getStart().getTime() > 0) ? (end - ua.getStart().getTime()) / 60000 : 0;
        } else minutos += (end - ua.getStart().getTime() > 0) ? (end - ua.getStart().getTime()) / 60000 : 0;
      }
    }
    return minutos;
  }

  /**
   * Añade ticket.
   * 
   * @param navegacion
   *           navegacion
   * @param u
   *           u
   * @return true, if successful
   */
  public boolean addTicket(Navegacion navegacion, UsuarioLdap u) {
    ActividadCalendarItem ac = new ActividadCalendarItem();
    Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    ac.setActividadType(ActividadType.NAVEGACION);
    ac.setActividad(navegacion);
    ac.setUsuario(navegacion.getUsuario());
    ac.setStart(cal.getTime());
    cal.set(Calendar.MINUTE, cal.get(Calendar.MINUTE) + getDuracion());
    ac.setEnd(cal.getTime());

    ObservacionActividad o = new ObservacionActividad();
    o.setActividad(navegacion);
    o.setFecha(new Date());
    if (getObservacion() != null && !getObservacion().trim().isEmpty()) {
      o.setObservacion(getObservacion());
    } else {
      o.setObservacion("Ampliación de tiempo");
    }
    o.setUsuario(u.getUser());
    navegacion.getObservacionesActividad().add(o);

    navegacion.addCalendarItem(ac);

    if (navegacion.getOrdenador().isOcupado(navegacion)){
      addErrorMessage("El recurso ya está reservado u ocupado para otra Actividad");
      return false;
    }
    persist(navegacion);
    return true;
  }

  /**
   * Borrar ticket.
   * 
   * @param navegacion
   *           navegacion
   * @param ticket
   *           ticket
   * @return true, if successful
   */
  public boolean borrarTicket(Navegacion navegacion, ActividadCalendarItem ticket) {
    if (ticket != null && navegacion.getCalendarItems().indexOf(ticket) > -1) {
      disableMessages();
      ticket.setActividad(null);
      navegacion.getCalendarItems().remove(navegacion.getCalendarItems().indexOf(ticket));
      remove(ticket);
      persist(navegacion);
      // navegacion.getCalendarItems().remove(navegacion.getCalendarItems().indexOf(ticket));
      // persist(navegacion);
      enableMessages();
      return true;
    }
    return false;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.NavegacionService#removeNavegacion(es.csa.fdt.cyldigital.servicios.entity.Navegacion)
   */
  public void removeNavegacion(Navegacion navegacion) {
    if (navegacion.getId() != 0) remove(navegacion);
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.NavegacionService#getCentros()
   */
  public List<Centro> getCentros() {
    if (centros == null) centros = findAllOrder(Centro.class, "nombre", true);
    return centros;
  }

  /**
   * Devuelve next navegacion.
   * 
   * @param u
   *           u
   * @param a
   *           a
   * @param next
   *           next
   * @return next navegacion
   */
  public Navegacion getNextNavegacion(UsuarioLdap u, Navegacion a, boolean next) {
    List<Navegacion> navegacions = new ArrayList<Navegacion>();
    if (a != null) {
      if (!u.getRolesList().contains(Role.ROLE_ACTIVITY_TRAINER_VIEW_ALL)) {
        navegacions = findByQuery(Navegacion.class,
            " Select a from es.csa.fdt.cyldigital.servicios.entity.Navegacion a where centro="
                + u.getUser().getCentro().getId() + " order by id desc");
      } else {
        navegacions = findByQuery(Navegacion.class,
            " Select a from es.csa.fdt.cyldigital.servicios.entity.Navegacion a  order by id desc");
      }
      if (navegacions.indexOf(a) > -1) {
        if (next) {
          if (navegacions.indexOf(a) + 1 < navegacions.size()) return navegacions.get(navegacions.indexOf(a) + 1);
        } else {
          if (navegacions.indexOf(a) != 0) return navegacions.get(navegacions.indexOf(a) - 1);
        }
      }
    }
    return null;
  }

  /**
   * Refresh navegaciones.
   */
  public void refreshNavegaciones() {
    navegaciones = null;
  }

  /**
   * Devuelve navegacion.
   * 
   * @param o
   *           o
   * @param navegaciones
   *           navegaciones
   * @return navegacion
   */
  public static Navegacion getNavegacion(Ordenador o, List<Navegacion> navegaciones) {
    Navegacion result = null;
    if (o != null) {
      for (Navegacion navegacion : navegaciones) {
        if (navegacion.getOrdenador() != null && o.getId() == navegacion.getOrdenador().getId()) {
          Date now = new Date();
          if (navegacion.getFechaInicio() != null && navegacion.getFechaInicio().before(now)
              && navegacion.getFechaFin() != null && navegacion.getFechaFin().after(now)
              && navegacion.getHoraInicio() != null && navegacion.getHoraInicio().before(now)
              && navegacion.getHoraFin() != null && navegacion.getHoraFin().after(now)) { return navegacion; }
        }
      }
    }
    return result;
  }

  /**
   * Reload navegaciones.
   */
  public void reloadNavegaciones() {
    navegaciones.clear();
  }

  /**
   * Devuelve navegaciones activas.
   * 
   * @return navegaciones activas
   */
  @SuppressWarnings("unchecked")
  public List<Navegacion> getNavegacionesActivas() {
    if (navegaciones == null || navegaciones.isEmpty()) {
      String consulta = "SELECT e FROM " + Navegacion.class.getName() + " e WHERE fechaInicio  <= ? and fechaFin >= ? "
          + " and horaInicio  <= ? and horaFin >= ? and cerrada = ? order by fechaFin DESC, horaFin DESC";

      Query query = getEntityManager().createQuery(consulta);
      Date now = new Date();
      query.setParameter(1, now);
      query.setParameter(2, now);
      query.setParameter(3, now);
      query.setParameter(4, now);
      query.setParameter(5, false);

      navegaciones = (List<Navegacion>) query.getResultList();
    }
    return navegaciones;
  }

  /**
   * Cancelar navegacion.
   * 
   * @param navegacion
   *           navegacion
   * @param u
   *           u
   * @return true, if successful
   */
  public boolean cancelarNavegacion(Navegacion navegacion, UsuarioLdap u) {
    if (navegacion != null && navegacion.getId() != 0) {
      refresh(navegacion);
      ObservacionActividad o = new ObservacionActividad();
      if (getObservacion() != null && !getObservacion().trim().isEmpty()) {
        o.setObservacion(getObservacion());
      } else {
        o.setObservacion("Cierre de navegacion");
      }
      o.setFecha(new Date());
      o.setUsuario(u.getUser());
      o.setActividad(navegacion);
      navegacion.getObservacionesActividad().add(o);
      navegacion.setCerrada(true);
      persist(navegacion);
      return true;
    }
    return false;
  }

  /**
   * Devuelve observacion.
   * 
   * @return observacion
   */
  public String getObservacion() {
    return observacion;
  }

  /**
   * Asigna observacion.
   * 
   * @param observacion
   *          observacion
   */
  public void setObservacion(String observacion) {
    this.observacion = observacion;
  }

  /**
   * Anadir observacion.
   * 
   * @param navegacion
   *           navegacion
   * @param o
   *           o
   * @param usuario
   *           usuario
   */
  public void anadirObservacion(Navegacion navegacion, ObservacionActividad o, Usuario usuario) {
    if (o.getObservacion().equals("")) addWarnMessage("No puede añadir un comentario vacío");
    else {
      o.setFecha(new Date());
      o.setUsuario(usuario);
      o.setActividad(navegacion);
      navegacion.getObservacionesActividad().add(o);
      persist(navegacion);
    }
  }

  /**
   * Devuelve nueva observacion.
   * 
   * @return nueva observacion
   */
  public ObservacionActividad getNuevaObservacion() {
    return new ObservacionActividad();
  }
}
