package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import javax.annotation.Resource;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.fdt.cyldigital.calendario.entity.ActividadCalendarItem;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.language.entity.CharlaLanguage;
import es.csa.fdt.cyldigital.recursos.entity.Ordenador;
import es.csa.fdt.cyldigital.servicios.entity.Actividad;
import es.csa.fdt.cyldigital.servicios.entity.PracticaTaller;
import es.csa.fdt.cyldigital.servicios.entity.Taller;
import es.csa.fdt.cyldigital.servicios.entity.TipoPractica;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario.Role;


/**
 * Clase PracticaTallerServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class PracticaTallerServiceImpl extends AbstractServiceImpl<PracticaTaller, Long> implements
    PracticaTallerService {

  private List<Centro> centros;

  private Centro centro;

  private boolean changeCentro = true;

  private List<TipoPractica> tiposPracticas;

  private List<Ordenador> ordenadores;

  private static final long serialVersionUID = -5575228180361917674L;

  @Resource
  private transient HashMap<String, Object> generalProperties;

  /**
   * Apply centro.
   * 
   * @param centro
   *           centro
   */
  public void applyCentro(Centro centro) {
    this.centro = centro;
    changeCentro = true;
  }

  /**
   * Construye un nuevo objeto de la clase
   */
  public PracticaTallerServiceImpl() {
    super();
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.PracticaTallerService#persistPracticaTaller(es.csa.fdt.cyldigital.servicios.entity.PracticaTaller)
   */
  public boolean persistPracticaTaller(PracticaTaller practicaTaller) {
	  if (practicaTaller.getOrdenador().isOcupado(generateCalendar(practicaTaller))) {
		  addErrorMessage("actividad.anyadirRecursos.recursosOcupados");
		  addErrorMessage(practicaTaller.getOrdenador().getNombre());
		  
		  return false;
	  }

	  persist(practicaTaller);
	  return true;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.PracticaTallerService#removePracticaTaller(es.csa.fdt.cyldigital.servicios.entity.PracticaTaller)
   */
  public void removePracticaTaller(PracticaTaller practicaTaller) {
    if (practicaTaller.getId() != 0) remove(practicaTaller);
  }

  /**
   * Devuelve tipos practicas.
   * 
   * @return tipos practicas
   */
  public List<TipoPractica> getTiposPracticas() {
    if (tiposPracticas == null) tiposPracticas = findAllOrder(TipoPractica.class, "nombre", true);
    return tiposPracticas;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.PracticaTallerService#getCentros()
   */
  public List<Centro> getCentros() {
    if (centros == null) centros = findAllOrder(Centro.class, "nombre", true);
    return centros;
  }

  /**
   * Devuelve ordenadores.
   * 
   * @return ordenadores
   */
  public List<Ordenador> getOrdenadores() {
    if (centro != null && changeCentro) {
      ordenadores = findWhereOrder(Ordenador.class, "centro.id", centro.getId(), "nombre", true);
      changeCentro = false;
    }
    return ordenadores;
  }

  /**
   * Actualiza centro.
   * 
   * @param practicaTaller
   *           practica taller
   * @param currentUser
   *           current user
   */
  public void actualizaCentro(PracticaTaller practicaTaller, UsuarioLdap currentUser) {
    if (practicaTaller.getCentro() != null)
    	centro = practicaTaller.getCentro();
    else {
    	if (currentUser.getCentro() != null) {
    	      Centro c = find(Centro.class, currentUser.getCentro().getId());
    	      
    	      practicaTaller.setCentro(c);
    	      
    	      centro = c;
    	      changeCentro = true;
    	      
    	} else if (generalProperties.containsKey("DEFAULT_CENTER_ID") && practicaTaller.getCentro() == null) {
    	      Centro c = find(Centro.class, Long.valueOf((String) generalProperties.get("DEFAULT_CENTER_ID")));
    	      
    	      if (c != null) {
    	        practicaTaller.setCentro(c);
    	        
    	        centro = c;
    	        changeCentro = true;
    	      }
    	}
    }
  }

  /**
   * Devuelve next practica taller.
   * 
   * @param u
   *           u
   * @param a
   *           a
   * @param next
   *           next
   * @return next practica taller
   */
  public PracticaTaller getNextPracticaTaller(UsuarioLdap u, PracticaTaller a, boolean next) {
    if (a != null) {
      List<PracticaTaller> practicasTalleres = new ArrayList<PracticaTaller>();

      if (!u.getRolesList().contains(Role.ROLE_ACTIVITY_TRAINER_VIEW_ALL)) {
        practicasTalleres = findByQuery(PracticaTaller.class,
            " Select a from es.csa.fdt.cyldigital.servicios.entity.PracticaTaller a where centro="
                + u.getUser().getCentro().getId() + " order by fecha desc");
      } else {
        practicasTalleres = findByQuery(PracticaTaller.class,
            " Select a from es.csa.fdt.cyldigital.servicios.entity.PracticaTaller a  order by fecha desc");
      }
      if (practicasTalleres.indexOf(a) > -1) {
        if (next) {
          if (practicasTalleres.indexOf(a) + 1 < practicasTalleres.size())
            return practicasTalleres.get(practicasTalleres.indexOf(a) + 1);
        } else {
          if (practicasTalleres.indexOf(a) != 0) return practicasTalleres.get(practicasTalleres.indexOf(a) - 1);
        }
      }
    }
    return null;
  }
  
  /**
   * Devuelve next practica taller.
   * 
   * @param u
   *           u
   * @param a
   *           a
   * @param next
   *           next
   * @return next practica taller
   */
  public PracticaTaller getNextPracticaTallerNew(UsuarioLdap u, PracticaTaller a, boolean next, List<Filter<CriterionJPAQL,PracticaTaller>> filtros) {
	  
    if (a != null) {
      List<PracticaTaller> practicasTalleres = new ArrayList<PracticaTaller>();
      
      if (null != filtros){
    	  
    	  String query = "";
    	  SimpleDateFormat formatoFecha = new SimpleDateFormat("yyyy-MM-dd");
    	  
    	  if (filtros.size() == 2) {
    		  if (!u.getRolesList().contains(Role.ROLE_ACTIVITY_TRAINER_VIEW_ALL)) {
    			  query = " Select a from es.csa.fdt.cyldigital.servicios.entity.PracticaTaller a where centro="
    		                + u.getUser().getCentro().getId() + " order by fecha desc";
    		  } else {
    			  query = " Select a from es.csa.fdt.cyldigital.servicios.entity.PracticaTaller a  order by fecha desc";
    		  }
    		  
    		  practicasTalleres = findByQuery(PracticaTaller.class,query);
    		  
    	  } else {
    		  
    		  query = " Select a from es.csa.fdt.cyldigital.servicios.entity.PracticaTaller a";
    		  query += " WHERE ( ( desactivado is null or desactivado = false ) ) AND ( centro.desactivado = false )";
    		  boolean noCentro = true;
    		  
    		  if (!u.getRolesList().contains(Role.ROLE_ACTIVITY_TRAINER_VIEW_ALL)) {
    			query += " AND centro=" + u.getUser().getCentro().getId();
    			noCentro = false;
    		  }
    		  
    		  
    		  String condicion = "";
			  String valor = "";
			  
			  for (Filter<CriterionJPAQL, PracticaTaller> f : filtros){
				
				//Centro
				if (f.getCriterion().getQuery().contains("centro.id") && noCentro){
					
					for (Object o : f.getCriterion().getParameters()) {
						valor = o.toString();
					}
					condicion = "centro.id = " + valor;
					query += " AND (" + condicion + ")";
				}
				
				//Tipo practica
				if (f.getCriterion().getQuery().contains("tipoPractica.id")){
					
					for (Object o : f.getCriterion().getParameters()) {
						valor = o.toString();
					}
					condicion = "tipoPractica.id = " + valor;
					query += " AND (" + condicion + ")";
				}
				
				//ID
				if (("id = ?").equals(f.getCriterion().getQuery())){
					
					for (Object o : f.getCriterion().getParameters()) {
						valor = o.toString();
					}
					condicion = "id = " + valor;
					query += " AND (" + condicion + ")";
				}
				
				//Fecha
				if (f.getCriterion().getQuery().contains("fecha")){
					
					for (Object o : f.getCriterion().getParameters()) {
						valor = formatoFecha.format((Date)o);
					}
					condicion = "fecha = '" + valor + "'";
					query += " AND (" + condicion + ")";
				}
			  }
			  
        	  query += " order by fecha desc";

        	  practicasTalleres = findByQuery(PracticaTaller.class,query);
    		  
        	  if (practicasTalleres.indexOf(a) > -1) {
		        if (next) {
		          if (practicasTalleres.indexOf(a) + 1 < practicasTalleres.size())
		            return practicasTalleres.get(practicasTalleres.indexOf(a) + 1);
		        } else {
		          if (practicasTalleres.indexOf(a) != 0) return practicasTalleres.get(practicasTalleres.indexOf(a) - 1);
		        }
        	  }
    	  }
    	  
      } else {
    	  return null;
      }
    }
    
    return null;
  }
  
  public List<ActividadCalendarItem> generateCalendar(Actividad actividad) {
	  List<ActividadCalendarItem> acis = new ArrayList<ActividadCalendarItem>();

	  ActividadCalendarItem item;

	  Calendar c = Calendar.getInstance();
	  Calendar tempStart = Calendar.getInstance();
	  Calendar tempEnd = Calendar.getInstance();
	  Calendar tempTime = Calendar.getInstance();

	  if (actividad != null) {
		  tempTime.setTime(actividad.getHoraInicio());
		  tempStart.setTime(actividad.getFechaInicio());
		  tempStart.set(Calendar.HOUR, tempTime.get(Calendar.HOUR_OF_DAY));
		  tempStart.set(Calendar.MINUTE, tempTime.get(Calendar.MINUTE));
	
		  tempTime.setTime(actividad.getHoraFin());
		  tempEnd.setTime(actividad.getFechaFin());
		  tempEnd.set(Calendar.HOUR, tempTime.get(Calendar.HOUR_OF_DAY));
		  tempEnd.set(Calendar.MINUTE, tempTime.get(Calendar.MINUTE));
	
		  c.setTime(tempStart.getTime());
	
		  while (c.before(tempEnd) || (c.get(Calendar.DAY_OF_YEAR) <= tempEnd.get(Calendar.DAY_OF_YEAR) &&
				 c.get(Calendar.YEAR) == tempEnd.get(Calendar.YEAR))) {
			  if ((c.get(Calendar.DAY_OF_WEEK) > 1 && c.get(Calendar.DAY_OF_WEEK) < 7) || actividad instanceof Taller) {
				  item = new ActividadCalendarItem();
		
				  c.set(Calendar.HOUR_OF_DAY, tempStart.get(Calendar.HOUR_OF_DAY));
				  c.set(Calendar.MINUTE, tempStart.get(Calendar.MINUTE));
				  item.setStart(c.getTime());
				  c.set(Calendar.HOUR_OF_DAY, tempEnd.get(Calendar.HOUR_OF_DAY));
				  c.set(Calendar.MINUTE, tempEnd.get(Calendar.MINUTE));
				  item.setEnd(c.getTime());
				  item.setAllDay(false);
		  
				  /*item.setActividadType(actividad instanceof Curso ? ActividadType.CURSO
																	 : (actividad instanceof Taller ? ActividadType.TALLER
																	 : (actividad instanceof Charla ? ActividadType.CHARLA
																	 : (actividad instanceof Asesoramiento ? ActividadType.ASESORAMIENTO :
																	 : (actividad instanceof ActividadExterna ? ActividadType.EXTERNA : null)))));
				  item.setActividad(actividad);*/
		  
				  acis.add(item);
			  }

			  c.set(Calendar.DATE, c.get(Calendar.DATE) + 1);
		  }
	  }
	
	  return acis;
  }

}
