package es.csa.fdt.cyldigital.servicios.service;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.List;

import org.apache.commons.codec.binary.Base64;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.commons.persistence.browser.jpaql.FilterJPAQLByQuery;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.service.BrowserFilteredByCenter;
import es.csa.fdt.cyldigital.servicios.entity.PracticaTaller;
import es.csa.fdt.cyldigital.servicios.entity.TipoPractica;

/**
 * Servicio del Browser de Practicas Talleres.
 * 
 * @author csa
 */
@Service
@Repository
public class PracticasTalleresBrowserImpl extends BrowserFilteredByCenter<PracticaTaller> implements
    PracticasTalleresBrowser<CriterionJPAQL, String> {

  private static final long serialVersionUID = -7227366052245717441L;

  private TipoPractica tipoPractica;
  
  private Date fechaInicio;
  
  private Date fechaFin;

  /**
   * Construye un nuevo objeto de la clase
   */
  public PracticasTalleresBrowserImpl() {
    super();
    getSort().setSortAsc(false);
    getSort().setSortProperty("fecha");
  }

  /**
   * Devuelve tipo practica.
   * 
   * @return tipo practica
   */
  public TipoPractica getTipoPractica() {
    return tipoPractica;
  }

  /**
   * Asigna tipo practica.
   * 
   * @param tipoPractica
   *          tipo practica
   */
  public void setTipoPractica(TipoPractica tipoPractica) {
    this.tipoPractica = tipoPractica;
  }
  
  /**
	 * @return the fechaInicio
	 */
	public Date getFechaInicio() {
		return fechaInicio;
	}

	/**
	 * @param fechaInicio 
	 * 		  the fechaInicio to set
	 */
	public void setFechaInicio(Date fechaInicio) {
		this.fechaInicio = fechaInicio;
	}
	
	/**
	 * @return the fechaFin
	 */
	public Date getFechaFin() {
		return fechaFin;
	}

	/**
	 * @param fechaInicio 
	 * 		  the fechaInicio to set
	 */
	public void setFechaFin(Date fechaFin) {
		this.fechaFin = fechaFin;
	}

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.core.service.BrowserFilteredByCenter#setCentro(es.csa.fdt.cyldigital.centros.entity.Centro)
   */
  @Override
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Filter tipo practica.
   */
  public void filterTipoPractica() {
    removeFilterTipoPractica();
    if (getTipoPractica() != null) filterByTipoPractica(getTipoPractica().getId());
  }

  private void removeFilterTipoPractica() {
    int i = -1;
    for (Filter<CriterionJPAQL, PracticaTaller> f : getFilters())
      if (f.getCriterion().getQuery().contains("tipoPractica.id")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
  }

  /**
   * Filter by tipo practica.
   * 
   * @param idTipoPractica
   *           id tipo practica
   */
  public void filterByTipoPractica(Long idTipoPractica) {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<PracticaTaller> filter = new FilterJPAQLByQuery<PracticaTaller>();

    query += " tipoPractica.id = ? ";
    filter.getCriterion().addParameter(idTipoPractica);

    for (Filter<CriterionJPAQL, PracticaTaller> f : getFilters())
      if (f.getCriterion().getQuery().contains("tipoPractica.id")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }

  /**
   * Filter practicas.
   */
  public void filterPracticas() {
    removeFilterTipoPractica();
    if (getTipoPractica() != null) filterByTipoPractica(getTipoPractica().getId());
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.core.service.BrowserFilteredByCenter#clearFilters()
   */
  @Override
  public void clearFilters() {
    super.clearFilters();
    filterPracticas();
  }
  
  	//3.4.8 - RF_15_20 Consulta e exportación da información das actividades
  	public void filterFechas() {
  		removeFilterFecha();
		if (getFechaInicio() != null && getFechaFin() != null) {
			addFilterFechas();
		} else if (getFechaInicio() != null && getFechaFin() == null) {
			addFilterFechaInicio();
		} else if (getFechaInicio() == null && getFechaFin() != null) {
			addFilterFechaFin();
		}
	}
  
  public void removeFilterFecha() {
	  int i = -1;
	  
	  for (Filter<CriterionJPAQL, PracticaTaller> f : getFilters())
		  if (f.getCriterion().getQuery().contains("fecha"))
			i = getFilters().indexOf(f);
		
		if (i > -1)
			getFilters().remove(i);
  }
  
  public void addFilterFechaInicio() {
	  String query = "";
	  boolean flag = false;
	  
	  FilterJPAQLByQuery<PracticaTaller> filter = new FilterJPAQLByQuery<PracticaTaller>();
	  
	  query += " fecha >= ? ";
	  filter.getCriterion().addParameter(getFechaInicio());
	  
	  for (Filter<CriterionJPAQL, PracticaTaller> f : getFilters())
		  if (f.getCriterion().getQuery().contains("fecha"))
			flag = true;

		if (!flag) {
			filter.getCriterion().setQuery(query);
			getFilters().add(filter);
		}
  }
  
  public void addFilterFechaFin() {
	  String query = "";
	  boolean flag = false;
	  
	  FilterJPAQLByQuery<PracticaTaller> filter = new FilterJPAQLByQuery<PracticaTaller>();
	  
	  query += " fecha <= ? ";
	  filter.getCriterion().addParameter(getFechaFin());
	  
	  for (Filter<CriterionJPAQL, PracticaTaller> f : getFilters())
		  if (f.getCriterion().getQuery().contains("fecha"))
			flag = true;

		if (!flag) {
			filter.getCriterion().setQuery(query);
			getFilters().add(filter);
		}
  }
  
  public void addFilterFechas() {
	  String query = "";
	  boolean flag = false;
	  
	  FilterJPAQLByQuery<PracticaTaller> filter = new FilterJPAQLByQuery<PracticaTaller>();
	  
	  query += " fecha >= ? ";
	  filter.getCriterion().addParameter(getFechaInicio());
	  query += " AND fecha <= ? ";
	  filter.getCriterion().addParameter(getFechaFin());
	  
	  for (Filter<CriterionJPAQL, PracticaTaller> f : getFilters())
		  if (f.getCriterion().getQuery().contains("fecha"))
			flag = true;

		if (!flag) {
			filter.getCriterion().setQuery(query);
			getFilters().add(filter);
		}
  }
  
  //RF_XEA_12 Funcion para mantener el filtro al volver del detalle de un curso
  public List<Filter<CriterionJPAQL, PracticaTaller>> cargarFiltros(List<Filter<CriterionJPAQL, PracticaTaller>> filtros, 
		  Centro cen, TipoPractica tipo, Date fechaInicio, Date fechaFin, Filter<CriterionJPAQL, PracticaTaller> texto) {
		
	if (null != filtros){
		
		getFilters().removeAll(getFilters());
		
		for (Filter<CriterionJPAQL, PracticaTaller> f : filtros) {
			getFilters().add(f);
			
		}
		
		//Para establecer el valor del centro seleccionado en el filtro inicial
		if (null != cen) {
			setCentro(cen);
		}
		
		//Para establecer el valor de la tipo de practica en el filtro inicial
		if (null != tipo) {
			setTipoPractica(tipo);
		}
		
		//Para establecer el valor de la fecha inicio en el filtro inicial
		if (null != fechaInicio) {
			setFechaInicio(fechaInicio);
		}
		
		//Para establecer el valor de la fecha fin en el filtro inicial
		if (null != fechaFin) {
			setFechaFin(fechaFin);
		}
		
		//Para mantener el texto introducido en el campo del filtro
		if(null != texto) {
			setFilter(texto);
		}
	}
	
	return null;
			
  }
  
	/**
	 * Codifica los filtros empleados en el browser para trasladar al proceso de exportacion.
	 * @return
	 */
	public String getFiltersEncoded(){
		String result = null;
		String sFilterQuery = "";
		ArrayList<String> oFilterList = new ArrayList<String>();
		DateFormat dfo = new SimpleDateFormat("yyyy-MM-dd 00:00:00");
		
		for (Filter<CriterionJPAQL, PracticaTaller> f : getFilters()){
			if (f.getCriterion() != null) {
				sFilterQuery += " AND (" + f.getCriterion().getQuery() + ") ";
				
				for (Object oParameter : f.getCriterion().getParameters()) {
					String sParameter = "";
					if(oParameter instanceof Long){
						sParameter = oParameter.toString();
					} else if(oParameter instanceof String) {
						sParameter = "'" + oParameter.toString().trim() + "'";
					} else if(oParameter instanceof Date) {
						sParameter = "'" + dfo.format(oParameter) + "'";
					} else {
						sParameter = oParameter.toString();
					}
					
					oFilterList.add(sParameter);
			    }
			}
		}
		oFilterList.add(sFilterQuery);
		
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		try {
			new ObjectOutputStream(out).writeObject(oFilterList);
		} catch (IOException e) {
			e.printStackTrace();
		}
		result = new String(Base64.encodeBase64(out.toByteArray()));
	
		return result;
	}
  
}
