package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.Date;
import java.util.Iterator;
import java.util.List;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.servicios.entity.CursoOnline;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCursoOnline;
import es.csa.fdt.cyldigital.servicios.entity.PreinscripcionCursoOnline;
import es.csa.fdt.cyldigital.servicios.entity.ServicioCursoOnline;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioOnline;


/**
 * Clase PreinscripcionCursoOnlineServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class PreinscripcionCursoOnlineServiceImpl extends AbstractServiceImpl<PreinscripcionCursoOnline, Long>
    implements PreinscripcionCursoOnlineService {

  private List<ServicioCursoOnline> cursos;

  private List<UsuarioOnline> usuarios;

  private static final long serialVersionUID = -5575228180361917674L;

  /**
   * Construye un nuevo objeto de la clase
   */
  public PreinscripcionCursoOnlineServiceImpl() {
    super();
  }

  /**
   * Persist preinscripcion curso.
   * 
   * @param u
   *           u
   * @param c
   *           c
   * @return int
   */
  public int persistPreinscripcionCurso(UsuarioOnline u, CursoOnline c) {

    // if (isYaPreinscritoCurso(u, c)) {
    // addErrorMessage("preinscripcion.validator.yainscrito");
    // return 0;
    // } else {

    /* Si no esta completo y no esta repetido */
    if (!c.isCompleto() && !yaMatriculado(c, u)) {
      MatriculaCursoOnline m = new MatriculaCursoOnline();
      m.setCurso(c);
      m.setUsuario(u);
      m.setFechaMatricula(new Date());
      m.setAdmitido(false);
      disableMessages();
      persist(m);
      enableMessages();
      
      /*
       * Comentado para implementar los servicios MULTILANGUAGE
       * TODO: mirar y modificar
      addInfoMessage("matricula.usuario.matriculado", new Object[] { u.getNombreCompleto(),
          ((ServicioCursoOnline) c.getServicio()).getNombre() });
      
      */
      return 1;
      /* Si esta completo, hay sitio en la lista de espera, no esta en la lista de interesados y no esta repetido */
    } else if (c.isCompleto() && isListaEspera(c, u) && !yaMatriculado(c, u) && !yaMatriculadoInteresado(c, u)) {
      MatriculaCursoOnline m = new MatriculaCursoOnline();
      m.setCurso(c);
      m.setUsuario(u);
      m.setFechaMatricula(new Date());
      m.setAdmitido(false);
      m.setEnListaEspera(true);
      disableMessages();
      persist(m);
      enableMessages();
      
      /*
       * Comentado para implementar los servicios MULTILANGUAGE
       * TODO: mirar y modificar
      addInfoMessage("matricula.usuario.matriculado", new Object[] { u.getNombreCompleto(),
          ((ServicioCursoOnline) c.getServicio()).getNombre() });
      */
      
      return 2;
      /* si esta repetido */
    } else if (yaMatriculado(c, u)) {
      return 6;
      /* si ya esta en lista de interesados */
    } else if (yaMatriculadoInteresado(c, u)) {
      return 0;
      /* si esta completo y no hay sitio en lista de espera, entonces irá a lista de interesados */
    } else {
      PreinscripcionCursoOnline p = new PreinscripcionCursoOnline();
      p.setUsuario(u);
      p.setServicioCurso((ServicioCursoOnline) c.getServicio());
      p.setFecha(new Date());
      disableMessages();
      persist(p);
      enableMessages();
      
      /*
       * Comentado para implementar los servicios MULTILANGUAGE
       * TODO: mirar y modificar
      addInfoMessage("preinscripcion.usuario.preinscrito", new Object[] { u.getNombreCompleto(),
          ((ServicioCursoOnline) c.getServicio()).getNombre() });
      */
      
      return 2;
    }
    // }
  }

  // private boolean isYaPreinscritoCurso(Usuario u, Curso c) {
  // if (!c.isCompleto()) {
  // if (u != null && u.getMatriculasCursos() != null) for (MatriculaCurso p : u.getMatriculasCursos()) {
  // if (p.getCurso().equals(c) && p.getCurso().getCentro().equals(c.getCentro())) return true;
  // }
  // } else {
  // if (u != null && u.getPreinscripcionesCursos() != null)
  // for (PreinscripcionCurso p : u.getPreinscripcionesCursos()) {
  // if (p.getServicioCurso().equals((c.getServicio())) && p.getCentro().equals(c.getCentro())) return true;
  // }
  // }
  // return false;
  // }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.PreinscripcionCursoOnlineService#persistPreinscripcionCurso(es.csa.fdt.cyldigital.servicios.entity.PreinscripcionCursoOnline)
   */
  public boolean persistPreinscripcionCurso(PreinscripcionCursoOnline preinscripcionCurso) {
    if (isYaPreinscrito(preinscripcionCurso)) {
      addErrorMessage("preinscripcion.validator.yainscrito");
      return false;
    } else if (preinscripcionCurso.getUsuario() == null) {
      addErrorMessage("matricula.validator.usuario_necesario");
      return false;
    } else {
      preinscripcionCurso.setFecha(new Date());
      disableMessages();
      persist(preinscripcionCurso);
      enableMessages();

      /*
       * Comentado para implementar los servicios MULTILANGUAGE
       * TODO: mirar y modificar
      addInfoMessage("preinscripcion.usuario.preinscrito", new Object[] {
          preinscripcionCurso.getUsuario().getNombreCompleto(), preinscripcionCurso.getServicioCurso().getNombre() });
      */

      return true;
    }
  }

  private boolean isYaPreinscrito(PreinscripcionCursoOnline preinscripcionCurso) {
    if (preinscripcionCurso.getUsuario() != null
        && preinscripcionCurso.getUsuario().getPreinscripcionesCursos() != null)
      for (PreinscripcionCursoOnline p : preinscripcionCurso.getUsuario().getPreinscripcionesCursos()) {
        if (p.getServicioCurso().equals(preinscripcionCurso.getServicioCurso()) && !(p.equals(preinscripcionCurso)))
          return true;
      }
    return false;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.PreinscripcionCursoOnlineService#removePreinscripcionCurso(es.csa.fdt.cyldigital.servicios.entity.PreinscripcionCursoOnline)
   */
  public void removePreinscripcionCurso(PreinscripcionCursoOnline preinscripcionCurso) {
    if (preinscripcionCurso.getId() != 0) {
      if (preinscripcionCurso.getServicioCurso() != null) {
        preinscripcionCurso.getServicioCurso().getPreinscripciones().remove(preinscripcionCurso);
      }
      if (preinscripcionCurso.getUsuario().getPreinscripcionesCursos() != null) {
        preinscripcionCurso.getUsuario().getPreinscripcionesCursos().remove(preinscripcionCurso);
      }
    }
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.PreinscripcionCursoOnlineService#getCursos()
   */
  public List<ServicioCursoOnline> getCursos() {
    if (cursos == null) cursos = findAllOrder(ServicioCursoOnline.class, "nombre", true);
    return cursos;
  };

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.PreinscripcionCursoOnlineService#getUsuarios()
   */
  public List<UsuarioOnline> getUsuarios() {
    if (usuarios == null) usuarios = findAllOrder(UsuarioOnline.class, "nombre", true);
    return usuarios;

  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.PreinscripcionCursoOnlineService#actualizaValores(es.csa.fdt.cyldigital.servicios.entity.PreinscripcionCursoOnline, java.lang.Long, java.lang.Long)
   */
  public void actualizaValores(PreinscripcionCursoOnline m, Long idCurso, Long idUsuario) {
    if (m != null && m.getId() == 0) {
      if (idCurso != null) m.setServicioCurso(find(ServicioCursoOnline.class, idCurso));
      if (idUsuario != null) m.setUsuario(find(UsuarioOnline.class, idUsuario));
    }
  }

  /**
   * Actualizar usuario.
   * 
   * @param preCurso
   *           pre curso
   * @param usuario
   *           usuario
   * @return preinscripcion curso online
   */
  public PreinscripcionCursoOnline actualizarUsuario(PreinscripcionCursoOnline preCurso, UsuarioOnline usuario) {
    preCurso.setUsuario(usuario);
    return preCurso;
  }

  /**
   * Checks if is lista espera.
   * 
   * @param c
   *           c
   * @param u
   *           u
   * @return true, if is lista espera
   */
  public boolean isListaEspera(CursoOnline c, UsuarioOnline u) {
    Integer numero = 0;
    Iterator<MatriculaCursoOnline> it = c.getMatriculas().iterator();
    while (it.hasNext()) {
      MatriculaCursoOnline matricula = it.next();
      if (matricula.isEnListaEspera()) {
        numero++;
      }
    }
    if (numero < c.getNumeroPlazasListaEspera()) {
      return true;
    } else {
      return false;
    }
  }

  /**
   * Ya matriculado.
   * 
   * @param c
   *           c
   * @param u
   *           u
   * @return true, if successful
   */
  public boolean yaMatriculado(CursoOnline c, UsuarioOnline u) {
    Iterator<MatriculaCursoOnline> it = c.getMatriculas().iterator();
    while (it.hasNext()) {
      MatriculaCursoOnline matricula = it.next();
      if (matricula.getUsuario().equals(u)) { return true; }
    }
    return false;
  }

  /**
   * Ya matriculado interesado.
   * 
   * @param c
   *           c
   * @param u
   *           u
   * @return true, if successful
   */
  public boolean yaMatriculadoInteresado(CursoOnline c, UsuarioOnline u) {

    Iterator<PreinscripcionCursoOnline> it = u.getPreinscripcionesCursos().iterator();
    while (it.hasNext()) {
      PreinscripcionCursoOnline preins = it.next();
      if (preins.getServicioCurso().equals(c.getServicio())) { return true; }
    }
    return false;
  }

}
