package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.servicios.entity.Curso;
import es.csa.fdt.cyldigital.language.entity.ServicioCursoLanguage;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso;
import es.csa.fdt.cyldigital.servicios.entity.PreinscripcionCurso;
import es.csa.fdt.cyldigital.servicios.entity.ServicioCurso;
import es.csa.fdt.cyldigital.servicios.entity.PreinscripcionCurso.PreferenciaHoraria;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;


/**
 * Clase PreinscripcionCursoServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class PreinscripcionCursoServiceImpl extends AbstractServiceImpl<PreinscripcionCurso, Long> implements
    PreinscripcionCursoService {
	
	private String idioma;

  private List<ServicioCurso> cursos;
  
  private List<ServicioCursoLanguage> cursosLanguage;

  private List<Usuario> usuarios;

  private List<Centro> centros;

  private static final long serialVersionUID = -5575228180361917674L;

  /**
   * Construye un nuevo objeto de la clase
   */
  public PreinscripcionCursoServiceImpl() {
    super();
    
    idioma = "";
  }

  /**
   * Persist preinscripcion curso.
   * 
   * @param u
   *           u
   * @param c
   *           c
   * @return int
   */
  public int persistPreinscripcionCurso(UsuarioCyL u, Curso c) {

    // if (isYaPreinscritoCurso(u, c)) {
    // addErrorMessage("preinscripcion.validator.yainscrito");
    // return 0;
    // } else {

    /* Si no esta completo y no esta repetido */
    if (!c.isCompleto() && !yaMatriculado(c, u)) {
      MatriculaCurso m = new MatriculaCurso();
      m.setCurso(c);
      m.setUsuario(u);
      m.setFechaMatricula(new Date());
      m.setAdmitido(false);
      disableMessages();
      persist(m);
      enableMessages();
      /*
       * Comentado para implementar los servicios MULTILANGUAGE
       * TODO: mirar y modificar
      addInfoMessage("matricula.usuario.matriculado", new Object[] { u.getNombreCompleto(),
          ((ServicioCurso) c.getServicio()).getNombre() });
      */
      return 1;
      /* Si esta completo, hay sitio en la lista de espera, no esta en la lista de interesados y no esta repetido */
    } else if (c.isCompleto() && isListaEspera(c, u) && !yaMatriculado(c, u) && !yaMatriculadoInteresado(c, u)) {
      MatriculaCurso m = new MatriculaCurso();
      m.setCurso(c);
      m.setUsuario(u);
      m.setFechaMatricula(new Date());
      m.setAdmitido(false);
      m.setEnListaEspera(true);
      disableMessages();
      persist(m);
      enableMessages();
      /*
       * Comentado para implementar los servicios MULTILANGUAGE
       * TODO: mirar y modificar
      addInfoMessage("matricula.usuario.matriculado", new Object[] { u.getNombreCompleto(),
          ((ServicioCurso) c.getServicio()).getNombre() });
      */
      return 2;
      /* si esta repetido */
    } else if (yaMatriculado(c, u)) {
      return 6;
      /* si ya esta en lista de interesados */
    } else if (yaMatriculadoInteresado(c, u)) {
      return 0;
      /* si esta completo y no hay sitio en lista de espera, entonces irá a lista de interesados */
    } else {
      PreinscripcionCurso p = new PreinscripcionCurso();
      p.setCentro(c.getCentro());
      p.setUsuario(u);
      p.setServicioCurso((ServicioCurso) c.getServicio());
      p.setFecha(new Date());
      p.setHorarioPreferido(PreferenciaHoraria.I);
      disableMessages();
      persist(p);
      enableMessages();
      /*
       * Comentado para implementar los servicios MULTILANGUAGE
       * TODO: mirar y modificar
      addInfoMessage("preinscripcion.usuario.preinscrito", new Object[] { u.getNombreCompleto(),
          ((ServicioCurso) c.getServicio()).getNombre() });
      */
      return 2;
    }
    // }
  }

  // private boolean isYaPreinscritoCurso(Usuario u, Curso c) {
  // if (!c.isCompleto()) {
  // if (u != null && u.getMatriculasCursos() != null) for (MatriculaCurso p : u.getMatriculasCursos()) {
  // if (p.getCurso().equals(c) && p.getCurso().getCentro().equals(c.getCentro())) return true;
  // }
  // } else {
  // if (u != null && u.getPreinscripcionesCursos() != null)
  // for (PreinscripcionCurso p : u.getPreinscripcionesCursos()) {
  // if (p.getServicioCurso().equals((c.getServicio())) && p.getCentro().equals(c.getCentro())) return true;
  // }
  // }
  // return false;
  // }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.PreinscripcionCursoService#persistPreinscripcionCurso(es.csa.fdt.cyldigital.servicios.entity.PreinscripcionCurso)
   */
  public boolean persistPreinscripcionCurso(PreinscripcionCurso preinscripcionCurso) {
    if (isYaPreinscrito(preinscripcionCurso)) {
      addErrorMessage("preinscripcion.validator.yainscrito");
      return false;
    } else if (preinscripcionCurso.getUsuario() == null) {
      addErrorMessage("matricula.validator.usuario_necesario");
      return false;
    } else {
      preinscripcionCurso.setFecha(new Date());
      disableMessages();
      persist(preinscripcionCurso);
      enableMessages();
      /*
       * Comentado para implementar los servicios MULTILANGUAGE
       * TODO: mirar y modificar
      addInfoMessage("preinscripcion.usuario.preinscrito", new Object[] {
          preinscripcionCurso.getUsuario().getNombreCompleto(), preinscripcionCurso.getServicioCurso().getNombre() });
      */
      return true;
    }
  }

  private boolean isYaPreinscrito(PreinscripcionCurso preinscripcionCurso) {
    if (preinscripcionCurso.getUsuario() != null
        && preinscripcionCurso.getUsuario().getPreinscripcionesCursos() != null)
      for (PreinscripcionCurso p : preinscripcionCurso.getUsuario().getPreinscripcionesCursos()) {
        if (p.getServicioCurso().equals(preinscripcionCurso.getServicioCurso())
            && p.getCentro().equals(preinscripcionCurso.getCentro()) && !(p.equals(preinscripcionCurso))) return true;
      }
    return false;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.PreinscripcionCursoService#removePreinscripcionCurso(es.csa.fdt.cyldigital.servicios.entity.PreinscripcionCurso)
   */
  public void removePreinscripcionCurso(PreinscripcionCurso preinscripcionCurso) {
    if (preinscripcionCurso.getId() != 0) {
      if (preinscripcionCurso.getServicioCurso() != null) {
        preinscripcionCurso.getServicioCurso().getPreinscripciones().remove(preinscripcionCurso);
      }
      if (preinscripcionCurso.getUsuario().getPreinscripcionesCursos() != null) {
        preinscripcionCurso.getUsuario().getPreinscripcionesCursos().remove(preinscripcionCurso);
      }
    }
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.PreinscripcionCursoService#getCursosLanguage()
   */
  public List<ServicioCursoLanguage> getCursosLanguage() {
	  String idioma = LocaleContextHolder.getLocaleContext().getLocale().toString();
	  
	  if ((cursosLanguage == null) || (idioma.compareTo(this.idioma) != 0))
		  cursosLanguage = findWhereOrder(ServicioCursoLanguage.class, "language.ISO", idioma, "nombre", true);
	  
	  this.idioma = idioma;
	  
	  return cursosLanguage;
  };

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.PreinscripcionCursoService#getCursos()
   */
  public List<ServicioCurso> getCursos() {
	  /*if (cursos == null) cursos = findAllOrder(ServicioCurso.class, "nombre", true);
	  	return cursos;*/

	  if (cursos != null)
		  return cursos;
	  else {
		  cursos = new ArrayList<ServicioCurso>();
		  List<ServicioCursoLanguage> serviciosCursosLanguage;

		  serviciosCursosLanguage = getCursosLanguage();

		  for(ServicioCursoLanguage servicioCursoLanguage : serviciosCursosLanguage)
			  cursos.add(servicioCursoLanguage.getServicioCurso());

		  return cursos;
	  }
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.PreinscripcionCursoService#getUsuarios()
   */
  public List<Usuario> getUsuarios() {
    if (usuarios == null) usuarios = findAllOrder(Usuario.class, "nombre", true);
    return usuarios;

  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.PreinscripcionCursoService#actualizaValores(es.csa.fdt.cyldigital.servicios.entity.PreinscripcionCurso, java.lang.Long, java.lang.Long)
   */
  public void actualizaValores(PreinscripcionCurso m, Long idCurso, Long idUsuario) {
    if (m != null && m.getId() == 0) {
      if (idCurso != null) m.setServicioCurso(find(ServicioCurso.class, idCurso));
      if (idUsuario != null) m.setUsuario(find(Usuario.class, idUsuario));
    }
  }

  /**
   * Devuelve centros.
   * 
   * @return centros
   */
  public List<Centro> getCentros() {
    if (centros == null) centros = findAllOrder(Centro.class, "nombre", true);
    return centros;
  }

  /**
   * Actualizar usuario.
   * 
   * @param preCurso
   *           pre curso
   * @param usuario
   *           usuario
   * @return preinscripcion curso
   */
  public PreinscripcionCurso actualizarUsuario(PreinscripcionCurso preCurso, Usuario usuario) {
    preCurso.setUsuario(usuario);
    return preCurso;
  }

  /**
   * Checks if is lista espera.
   * 
   * @param c
   *           c
   * @param u
   *           u
   * @return true, if is lista espera
   */
  public boolean isListaEspera(Curso c, Usuario u) {
    Integer numero = 0;
    Iterator<MatriculaCurso> it = c.getMatriculas().iterator();
    while (it.hasNext()) {
      MatriculaCurso matricula = it.next();
      if (matricula.isEnListaEspera()) {
        numero++;
      }
    }
    if (numero < c.getNumeroPlazasListaEspera()) {
      return true;
    } else {
      return false;
    }
  }

  /**
   * Ya matriculado.
   * 
   * @param c
   *           c
   * @param u
   *           u
   * @return true, if successful
   */
  public boolean yaMatriculado(Curso c, Usuario u) {
    Iterator<MatriculaCurso> it = c.getMatriculas().iterator();
    while (it.hasNext()) {
      MatriculaCurso matricula = it.next();
      if (matricula.getUsuario().equals(u)) { return true; }
    }
    return false;
  }

  /**
   * Ya matriculado interesado.
   * 
   * @param c
   *           c
   * @param u
   *           u
   * @return true, if successful
   */
  public boolean yaMatriculadoInteresado(Curso c, Usuario u) {

    Iterator<PreinscripcionCurso> it = u.getPreinscripcionesCursos().iterator();
    while (it.hasNext()) {
      PreinscripcionCurso preins = it.next();
      if (preins.getServicioCurso().equals(c.getServicio())) { return true; }
    }
    return false;
  }

}
