package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.commons.persistence.browser.jpaql.FilterJPAQLByQuery;
import es.csa.fdt.cyldigital.core.service.BrowserFilteredByCenter;
import es.csa.fdt.cyldigital.servicios.entity.PreinscripcionCurso;
import es.csa.fdt.cyldigital.util.SortJPAQLExtended;

/**
 * Servicio del Browser de Preinscripcion de Cursos.
 * 
 * @author csa
 */
public class PreinscripcionCursosBrowserImpl extends BrowserFilteredByCenter<PreinscripcionCurso> implements
    PreinscripcionCursosBrowser<CriterionJPAQL, String>
{
  private static final long serialVersionUID = -7227366052245717441L;

  /**
   * Construye un nuevo objeto de la clase
   */
  public PreinscripcionCursosBrowserImpl() {
    super();
    SortJPAQLExtended sort = new SortJPAQLExtended();
    sort.getDefaultProperties().add("peso");
    sort.getDefaultSorts().add(true);
    sort.getDefaultProperties().add("fecha");
    sort.getDefaultSorts().add(true);
    setSort(sort);
  }

  /**
   * Filter by servicio curso.
   * 
   * @param idCurso
   *           id curso
   */
  public void filterByServicioCurso(Long idCurso) {
    filterByServicioCursoCentro(idCurso, null);
  }

  /**
   * Filter by servicio curso centro.
   * 
   * @param idCurso
   *           id curso
   * @param idCentro
   *           id centro
   */
  public void filterByServicioCursoCentro(Long idCurso, Long idCentro) {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<PreinscripcionCurso> filter = new FilterJPAQLByQuery<PreinscripcionCurso>();

    if (idCurso != null && idCurso > 0) {
      filter.getCriterion().addParameter(idCurso);
      query += "idCurso = ? ";
    }

    if (idCentro != null && idCentro > 0) {
      filter.getCriterion().addParameter(idCentro);

      if (query.isEmpty()) query += "centro.id = ? ";
      else query += "AND centro.id = ? ";

      filter.getCriterion().addParameter(idCurso);
      query += "and not exists (from e.cursosRechazos WHERE idCurso = ?) ";
    }

    for (Filter<CriterionJPAQL, PreinscripcionCurso> f : getFilters())
    	if (f.getClass() == FilterJPAQLByQuery.class)
    		if (f.getCriterion().getQuery().equals(query))
    			flag = true;

    if (!query.isEmpty() && !flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }
}
