package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.Date;
import java.util.List;

import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.language.entity.PropuestaServicioLanguage;
import es.csa.fdt.cyldigital.language.entity.ServicioLanguage;
import es.csa.fdt.cyldigital.servicios.entity.PreinscripcionPropuesta;
import es.csa.fdt.cyldigital.servicios.entity.PropuestaServicio;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;


/**
 * Clase PreinscripcionPropuestaServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class PreinscripcionPropuestaServiceImpl extends AbstractServiceImpl<PreinscripcionPropuesta, Long> implements
    PreinscripcionPropuestaService {

  private List<Usuario> usuarios;

  private List<Centro> centros;

  private static final long serialVersionUID = -5575228180361917674L;

  /**
   * Construye un nuevo objeto de la clase
   */
  public PreinscripcionPropuestaServiceImpl() {
    super();
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.PreinscripcionPropuestaService#persistPreinscripcionPropuesta(es.csa.fdt.cyldigital.servicios.entity.PreinscripcionPropuesta)
   */
  public boolean persistPreinscripcionPropuesta(PreinscripcionPropuesta preinscripcionPropuesta) {
    preinscripcionPropuesta.setFecha(new Date());
    disableMessages();
    persist(preinscripcionPropuesta);
    enableMessages();
    
    String idioma = LocaleContextHolder.getLocaleContext().getLocale().toString();
    String nombrePropuesta = "";
	  List<PropuestaServicioLanguage> psl = preinscripcionPropuesta.getPropuestaServicio().getTextos();
	  for (PropuestaServicioLanguage pl : psl)
		  if (pl.getLanguage().getISO().compareTo(idioma) == 0)
			  nombrePropuesta = pl.getNombre();
	  
	  addInfoMessage("preinscripcion.usuario.preinscrito", new Object[] {
		        preinscripcionPropuesta.getUsuario().getNombreCompleto(),
		        nombrePropuesta });
    
    
    return true;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.PreinscripcionPropuestaService#removePreinscripcionPropuesta(es.csa.fdt.cyldigital.servicios.entity.PreinscripcionPropuesta)
   */
  public void removePreinscripcionPropuesta(PreinscripcionPropuesta preinscripcionPropuesta) {
    if (preinscripcionPropuesta.getId() != 0) {
      if (preinscripcionPropuesta.getPropuestaServicio() != null) {
        preinscripcionPropuesta.getPropuestaServicio().getPreinscripciones().remove(preinscripcionPropuesta);
      }
    }
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.PreinscripcionPropuestaService#getUsuarios()
   */
  public List<Usuario> getUsuarios() {
    if (usuarios == null) usuarios = findAllOrder(Usuario.class, "nombre", true);
    return usuarios;

  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.PreinscripcionPropuestaService#actualizaValores(es.csa.fdt.cyldigital.servicios.entity.PreinscripcionPropuesta, java.lang.Long, java.lang.Long)
   */
  public void actualizaValores(PreinscripcionPropuesta p, Long idPropuesta, Long idUsuario) {
    if (p != null && p.getId() == 0) {
      if (idPropuesta != null) p.setPropuestaServicio(find(PropuestaServicio.class, idPropuesta));
      if (idUsuario != null) p.setUsuario(find(Usuario.class, idUsuario));
    }
  }

  /**
   * Devuelve centros.
   * 
   * @return centros
   */
  public List<Centro> getCentros() {
    if (centros == null) centros = findAllOrder(Centro.class, "nombre", true);
    return centros;
  }

}
