package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.Date;
import java.util.Iterator;
import java.util.List;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaTaller;
import es.csa.fdt.cyldigital.servicios.entity.PreinscripcionTaller;
import es.csa.fdt.cyldigital.servicios.entity.ServicioTaller;
import es.csa.fdt.cyldigital.servicios.entity.Taller;
import es.csa.fdt.cyldigital.servicios.entity.PreinscripcionTaller.PreferenciaHoraria;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;


/**
 * Clase PreinscripcionTallerServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class PreinscripcionTallerServiceImpl extends AbstractServiceImpl<PreinscripcionTaller, Long> implements
    PreinscripcionTallerService {

  private List<ServicioTaller> talleres;

  private List<Usuario> usuarios;

  private List<Centro> centros;

  private static final long serialVersionUID = -5575228180361917674L;

  /**
   * Construye un nuevo objeto de la clase
   */
  public PreinscripcionTallerServiceImpl() {
    super();
  }

  /**
   * Persist preinscripcion taller.
   * 
   * @param u
   *           u
   * @param c
   *           c
   * @return int
   */
  public int persistPreinscripcionTaller(Usuario u, Taller c) {

    // if (isYaPreinscritoTaller(u, c)) {
    // return 6;
    // } else {
    /* Si no esta completo y no esta repetido */
    if (!c.isCompleto() && !yaMatriculado(c, u)) {
      MatriculaTaller m = new MatriculaTaller();
      m.setTaller(c);
      m.setUsuario(u);
      m.setFechaMatricula(new Date());
      m.setAdmitido(false);
      disableMessages();
      persist(m);
      enableMessages();
      /*
       * Comentado para implementar los servicios MULTILANGUAGE
       * TODO: mirar y modificar
      addInfoMessage("matricula.usuario.matriculado", new Object[] { u.getNombreCompleto(),
          ((ServicioTaller) c.getServicio()).getNombre() });
      */
      return 4;
      /* Si esta completo, hay sitio en la lista de espera y no esta repetido */
    } else if (c.isCompleto() && isListaEspera(c, u) && !yaMatriculado(c, u) && !yaMatriculadoInteresado(c, u)) {
      MatriculaTaller m = new MatriculaTaller();
      m.setTaller(c);
      m.setUsuario(u);
      m.setFechaMatricula(new Date());
      m.setAdmitido(false);
      m.setEnListaEspera(true);
      disableMessages();
      persist(m);
      enableMessages();
      /*
       * Comentado para implementar los servicios MULTILANGUAGE
       * TODO: mirar y modificar
      addInfoMessage("matricula.usuario.matriculado", new Object[] { u.getNombreCompleto(),
          ((ServicioTaller) c.getServicio()).getNombre() });ç
      */
      return 5;
      /* si esta repetido */
    } else if (yaMatriculado(c, u)) {
      return 6;
      /* si esta completo y no hay sitio en lista de espera, entonces irá a lista de interesados */
    } else if (yaMatriculadoInteresado(c, u)) {
      return 3;
      /* si esta completo y no hay sitio en lista de espera, entonces irá a lista de interesados */
    } else {
      PreinscripcionTaller p = new PreinscripcionTaller();
      p.setCentro(c.getCentro());
      p.setUsuario(u);
      p.setServicioTaller((ServicioTaller) c.getServicio());
      p.setFecha(new Date());
      p.setHorarioPreferido(PreferenciaHoraria.I);
      disableMessages();
      persist(p);
      enableMessages();
      /*
       * Comentado para implementar los servicios MULTILANGUAGE
       * TODO: mirar y modificar
      addInfoMessage("preinscripcion.usuario.preinscrito", new Object[] { u.getNombreCompleto(),
          ((ServicioTaller) c.getServicio()).getNombre() });
      */
      return 5;
    }
    // }
  }

  // private boolean isYaPreinscritoTaller(Usuario u, Taller c) {
  // if (!c.isCompleto()) {
  // if (u != null && u.getMatriculasTalleres() != null) for (MatriculaTaller p : u.getMatriculasTalleres()) {
  // if (p.getTaller().equals(c) && p.getTaller().getCentro().equals(c.getCentro())) return true;
  // }
  // } else {
  // if (u != null && u.getPreinscripcionesTalleres() != null)
  // for (PreinscripcionTaller p : u.getPreinscripcionesTalleres()) {
  // if (p.getServicioTaller().equals((c.getServicio())) && p.getCentro().equals(c.getCentro())) return true;
  // }
  // }
  // return false;
  // }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.PreinscripcionTallerService#persistPreinscripcionTaller(es.csa.fdt.cyldigital.servicios.entity.PreinscripcionTaller)
   */
  public boolean persistPreinscripcionTaller(PreinscripcionTaller preinscripcionTaller) {
    if (isYaPreinscrito(preinscripcionTaller)) {
      addErrorMessage("preinscripcion.validator.yainscrito");
      return false;
    } else if (preinscripcionTaller.getUsuario() == null) {
      addErrorMessage("matricula.validator.usuario_necesario");
      return false;
    } else {
      preinscripcionTaller.setFecha(new Date());
      disableMessages();
      persist(preinscripcionTaller);
      enableMessages();
      /*
       * Comentado para implementar los servicios MULTILANGUAGE
       * TODO: mirar y modificar
      addInfoMessage("preinscripcion.usuario.preinscrito", new Object[] {
          preinscripcionTaller.getUsuario().getNombreCompleto(), preinscripcionTaller.getServicioTaller().getNombre() });
      */
      return true;
    }
  }

  private boolean isYaPreinscrito(PreinscripcionTaller preinscripcionTaller) {
    if (preinscripcionTaller.getUsuario() != null
        && preinscripcionTaller.getUsuario().getPreinscripcionesTalleres() != null)
      for (PreinscripcionTaller p : preinscripcionTaller.getUsuario().getPreinscripcionesTalleres()) {
        if (p.getServicioTaller().equals(preinscripcionTaller.getServicioTaller())
            && p.getCentro().equals(preinscripcionTaller.getCentro()) && !(p.equals(preinscripcionTaller)))
          return true;
      }
    return false;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.PreinscripcionTallerService#removePreinscripcionTaller(es.csa.fdt.cyldigital.servicios.entity.PreinscripcionTaller)
   */
  public void removePreinscripcionTaller(PreinscripcionTaller preinscripcionTaller) {
    if (preinscripcionTaller.getId() != 0) {
      if (preinscripcionTaller.getServicioTaller() != null) {
        preinscripcionTaller.getServicioTaller().getPreinscripciones().remove(preinscripcionTaller);
      }
    }
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.PreinscripcionTallerService#getTalleres()
   */
  public List<ServicioTaller> getTalleres() {
    if (talleres == null) talleres = findAllOrder(ServicioTaller.class, "nombre", true);
    return talleres;
  };

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.PreinscripcionTallerService#getUsuarios()
   */
  public List<Usuario> getUsuarios() {
    if (usuarios == null) usuarios = findAllOrder(Usuario.class, "nombre", true);
    return usuarios;

  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.PreinscripcionTallerService#actualizaValores(es.csa.fdt.cyldigital.servicios.entity.PreinscripcionTaller, java.lang.Long, java.lang.Long)
   */
  public void actualizaValores(PreinscripcionTaller m, Long idTaller, Long idUsuario) {
    if (m != null && m.getId() == 0) {
      if (idTaller != null) m.setServicioTaller(find(ServicioTaller.class, idTaller));
      if (idUsuario != null) m.setUsuario(find(Usuario.class, idUsuario));
    }
  }

  /**
   * Devuelve centros.
   * 
   * @return centros
   */
  public List<Centro> getCentros() {
    if (centros == null) centros = findAllOrder(Centro.class, "nombre", true);
    return centros;
  }

  /**
   * Actualizar usuario.
   * 
   * @param preTaller
   *           pre taller
   * @param usuario
   *           usuario
   * @return preinscripcion taller
   */
  public PreinscripcionTaller actualizarUsuario(PreinscripcionTaller preTaller, Usuario usuario) {
    preTaller.setUsuario(usuario);
    return preTaller;
  }

  /**
   * Checks if is lista espera.
   * 
   * @param c
   *           c
   * @param u
   *           u
   * @return true, if is lista espera
   */
  public boolean isListaEspera(Taller c, Usuario u) {
    Integer numero = 0;
    Iterator<MatriculaTaller> it = c.getMatriculas().iterator();
    while (it.hasNext()) {
      MatriculaTaller matricula = it.next();
      if (matricula.isEnListaEspera()) {
        numero++;
      }
    }
    if (numero < c.getNumeroPlazasListaEspera()) {
      return true;
    } else {
      return false;
    }
  }

  /**
   * Ya matriculado.
   * 
   * @param c
   *           c
   * @param u
   *           u
   * @return true, if successful
   */
  public boolean yaMatriculado(Taller c, Usuario u) {
    Iterator<MatriculaTaller> it = c.getMatriculas().iterator();
    while (it.hasNext()) {
      MatriculaTaller matricula = it.next();
      if (matricula.getUsuario().equals(u)) { return true; }
    }
    return false;
  }

  /**
   * Ya matriculado interesado.
   * 
   * @param c
   *           c
   * @param u
   *           u
   * @return true, if successful
   */
  public boolean yaMatriculadoInteresado(Taller c, Usuario u) {
    Iterator<PreinscripcionTaller> it = u.getPreinscripcionesTalleres().iterator();
    while (it.hasNext()) {
      PreinscripcionTaller preins = it.next();
      if (preins.getServicioTaller().equals(c.getServicio())) { return true; }
    }
    return false;
  }

}
