package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.commons.persistence.browser.jpaql.FilterJPAQLByQuery;
import es.csa.fdt.cyldigital.core.service.BrowserFilteredByCenter;
import es.csa.fdt.cyldigital.servicios.entity.PreinscripcionTaller;
import es.csa.fdt.cyldigital.util.SortJPAQLExtended;

/**
 * Servicio del Browser de Preinscripcion a Talleres.
 * 
 * @author csa
 */
public class PreinscripcionTalleresBrowserImpl extends BrowserFilteredByCenter<PreinscripcionTaller> implements
    PreinscripcionTalleresBrowser<CriterionJPAQL, String> {
  private static final long serialVersionUID = -7227366052245717441L;

  /**
   * Construye un nuevo objeto de la clase
   */
  public PreinscripcionTalleresBrowserImpl() {
    super();
    SortJPAQLExtended sort = new SortJPAQLExtended();
    sort.getDefaultProperties().add("peso");
    sort.getDefaultSorts().add(true);
    sort.getDefaultProperties().add("fecha");
    sort.getDefaultSorts().add(true);
    setSort(sort);
  }

  /**
   * Filter by servicio taller.
   * 
   * @param idTaller
   *           id taller
   */
  public void filterByServicioTaller(Long idTaller) {
    filterByServicioTallerCentro(idTaller, null);
  }

  /**
   * Filter by servicio taller centro.
   * 
   * @param idTaller
   *           id taller
   * @param idCentro
   *           id centro
   */
  public void filterByServicioTallerCentro(Long idTaller, Long idCentro) {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<PreinscripcionTaller> filter = new FilterJPAQLByQuery<PreinscripcionTaller>();

    if (idTaller != null && idTaller > 0) {
      filter.getCriterion().addParameter(idTaller);

      query += "idTaller = ? ";

    }
    if (idCentro != null && idCentro > 0) {
      filter.getCriterion().addParameter(idCentro);
      if (query.isEmpty()) query += "centro.id = ?";
      else query += " AND centro.id = ?";
      filter.getCriterion().addParameter(idTaller);
      query += "and not exists (from e.talleresRechazos WHERE idTaller=?) ";
    }
    for (Filter<CriterionJPAQL, PreinscripcionTaller> f : getFilters())
      if (f.getCriterion().getQuery().equals(query)) flag = true;

    if (!query.isEmpty() && !flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }
}
