package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.List;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.language.entity.Language;
import es.csa.fdt.cyldigital.language.entity.PropuestaServicioLanguage;
import es.csa.fdt.cyldigital.servicios.entity.PreinscripcionPropuesta;
import es.csa.fdt.cyldigital.servicios.entity.PreinscripcionTaller;
import es.csa.fdt.cyldigital.servicios.entity.PropuestaServicio;
import es.csa.fdt.cyldigital.servicios.entity.ServicioTaller;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;


/**
 * Clase PropuestaServicioServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class PropuestaServicioServiceImpl extends AbstractServiceImpl<PropuestaServicio, Long> implements
    PropuestaServicioService {

  private static final long serialVersionUID = -5575228180361917674L;

  /**
   * Construye un nuevo objeto de la clase
   */
  public PropuestaServicioServiceImpl() {
    super();
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.PropuestaServicioService#persistPropuesta(es.csa.fdt.cyldigital.servicios.entity.PropuestaServicio)
   */
  public boolean persistPropuesta(PropuestaServicio propuesta, Long idEmpleado) {
	Empleado e = find(Empleado.class, idEmpleado);
	
	propuesta.setSolicitante(e);

    persist(propuesta);
    
    return true;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.PropuestaServicioService#getPropuestasDisponibles()
   */
  public List<PropuestaServicio> getPropuestasDisponibles() {
    return findAll(PropuestaServicio.class);
  }

  /**
   * Elimina propuesta.
   * 
   * @param propuesta
   *           propuesta
   */
  public void removePropuesta(PropuestaServicio propuesta) {
    if (propuesta.getId() != 0) {
    	//refresh(propuesta);
    	
    	propuesta.getPreinscripciones().clear();

    	remove(propuesta);
    }
  }

  /**
   * Put interesados in curso.
   * 
   * @param idCurso
   *           id curso
   * @param idPropuesta
   *           id propuesta
   */
  /*public void putInteresadosInCurso(Long idCurso, Long idPropuesta) {
    ServicioCurso c = find(ServicioCurso.class, idCurso);
    refresh(c);
    PropuestaServicio ps = find(idPropuesta);
    refresh(ps);
    try {
      if (ps != null) {
        for (PreinscripcionPropuesta pp : ps.getPreinscripciones()) {
          if (!pp.getDesactivado()) {
            PreinscripcionCurso pc = new PreinscripcionCurso();
            Centro centro = find(Centro.class, pp.getCentro().getId());
            pc.setCentro(centro);
            pc.setFecha(pp.getFecha());
            pc.setHorarioPreferido(PreferenciaHoraria.valueOf(pp.getHorarioPreferido().toString()));
            UsuarioCyL u = find(UsuarioCyL.class, pp.getUsuario().getId());
            pc.setUsuario(u);
            pc.setServicioCurso(c);
            if (c.getPreinscripciones() == null) c.setPreinscripciones(new ArrayList<PreinscripcionCurso>());
            c.getPreinscripciones().add(pc);
          }
        }
        persist(c);

        setMessagesEnabled(false);
        remove(ps);
        setMessagesEnabled(true);
      } else {
        remove(c);
      }
    } catch (Exception e) {
      remove(c);
    }
  }*/

  /**
   * Put interesados in taller.
   * 
   * @param idTaller
   *           id taller
   * @param idPropuesta
   *           id propuesta
   */
  public void putInteresadosInTaller(Long idTaller, Long idPropuesta) {
    ServicioTaller t = find(ServicioTaller.class, idTaller);
    refresh(t);
    PropuestaServicio ps = find(idPropuesta);
    refresh(ps);
    try {
      if (ps != null) {
        for (PreinscripcionPropuesta pp : ps.getPreinscripciones()) {
          if (!pp.getDesactivado()) {
            PreinscripcionTaller pt = new PreinscripcionTaller();
            Centro centro = find(Centro.class, pp.getCentro().getId());
            pt.setCentro(centro);
            pt.setFecha(pp.getFecha());
            pt.setHorarioPreferido(es.csa.fdt.cyldigital.servicios.entity.PreinscripcionTaller.PreferenciaHoraria
                .valueOf(pp.getHorarioPreferido().toString()));
            UsuarioCyL u = find(UsuarioCyL.class, pp.getUsuario().getId());
            pt.setUsuario(u);
            pt.setServicioTaller(t);
            if (t.getPreinscripciones() == null) t.setPreinscripciones(new ArrayList<PreinscripcionTaller>());
            t.getPreinscripciones().add(pt);

          }
        }
        persist(t);

        setMessagesEnabled(false);
        remove(ps);
        setMessagesEnabled(true);
      } else {
        remove(t);
      }
    } catch (Exception e) {
      remove(t);
    }
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.TematicaServiceImpl#createLanguages(es.csa.fdt.cyldigital.servicios.entity.Tematica)
   */
  public void createLanguages(PropuestaServicio propuesta, List<Language> languages) {	  
	  List<PropuestaServicioLanguage> textos = new ArrayList<PropuestaServicioLanguage>();

	  for (Language language : languages) {
		  PropuestaServicioLanguage texto = new PropuestaServicioLanguage();
		  texto.setLanguage(language);
		  texto.setPropuesta(propuesta);

		  textos.add(texto);
	  }
	  propuesta.setTextos(textos);
  }
}
