package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */
 
import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.commons.persistence.browser.jpaql.FilterJPAQLByQuery;
import es.csa.fdt.cyldigital.core.service.BrowserFilteredByCenter;
import es.csa.fdt.cyldigital.language.entity.CursoLanguage;
import es.csa.fdt.cyldigital.language.entity.Language;

public class ProxImparticionesBrowserImpl extends BrowserFilteredByCenter<CursoLanguage> implements ProxImparticionesBrowser<CriterionJPAQL, String>{

	private static final long serialVersionUID = -2774336220281467314L;
	
	/**
	 * Construye un nuevo objeto de la clase
	 */
	public ProxImparticionesBrowserImpl() {
		super("curso");

		getSort().setSortProperty("curso.id ASC");
	}
	
	public void applyLanguageCondition(Language language) {
		this.clearConditions();
		
		this.getCondition().getCriterion().setQuery("e.language = ?");
		this.getCondition().getCriterion().addParameter(language);

		this.applyCondition();
	}
	
	/**
	 * Filter by curso.
	 * 
	 * @param idServicio
	 *           id servicio
	 */
	public void filterByCurso(Long idServicio, Long idCurso) {
		String query = "";
		boolean flag = false;
		
		FilterJPAQLByQuery<CursoLanguage> filter = new FilterJPAQLByQuery<CursoLanguage>();

		if (idServicio != null && idServicio > 0) {
			filter.getCriterion().addParameter(idServicio);
			query += " curso.servicio.id = ? ";
		}
		
		query += " AND curso.fechaInicio >= curdate()";
		String filtroCurso = " AND curso.id != " + idCurso;
		query += filtroCurso;

		for (Filter<CriterionJPAQL, CursoLanguage> f : getFilters())
			if (f.getCriterion().getQuery().equals(query)) flag = true;

		if (!query.isEmpty() && !flag) {
			filter.getCriterion().setQuery(query);
			getFilters().add(filter);
		}
	}

}
