package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.List;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.language.entity.Language;
import es.csa.fdt.cyldigital.language.entity.ServicioLanguage;
import es.csa.fdt.cyldigital.servicios.entity.Charla;
import es.csa.fdt.cyldigital.servicios.entity.ServicioCharla;
import es.csa.fdt.cyldigital.servicios.entity.ServicioPracticaTaller;


/**
 * Clase ServicioCharlaServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class ServicioCharlaServiceImpl extends AbstractServiceImpl<ServicioCharla, Long> implements
    ServicioCharlaService {

  private static final long serialVersionUID = -5575228180361917674L;

  private static final String DEFAULT_NOMBRE_SERVICIO = "Charla";

  /**
   * Construye un nuevo objeto de la clase
   */
  public ServicioCharlaServiceImpl() {
    super();
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.ServicioCharlaService#persistServicioCharla(es.csa.fdt.cyldigital.servicios.entity.ServicioCharla)
   */
  public void persistServicioCharla(ServicioCharla charla) {
    persist(charla);
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.ServicioCharlaService#removeServicioCharla(es.csa.fdt.cyldigital.servicios.entity.ServicioCharla)
   */
  public void removeServicioCharla(ServicioCharla charla) {
    if (charla.getId() != 0) remove(charla);
  }

  /**
   * Actualiza servicio.
   * 
   * @param charla
   *           charla
   */
  public void actualizaServicio(Charla charla) {
    List<ServicioCharla> serviciosCharlas = findAll(ServicioCharla.class);
    // Si existe algún servicio de charla, asocio el primero a la actividad.
    if (serviciosCharlas.size() > 0) {
      charla.setServicio(serviciosCharlas.get(0));
    }
    // Si no existen servicios de char, creo uno por defecto, y le asocio a la actividad.
    else {
      ServicioCharla servCharla = new ServicioCharla();
         
      List<Language> ll = findAll(Language.class);
      List<ServicioLanguage> lsl = new ArrayList<ServicioLanguage>();
	  for (Language l : ll) {
		  ServicioLanguage sl = new ServicioLanguage();
		  
		  sl.setLanguage(l);
		  sl.setServicio(servCharla);
		  sl.setNombre(DEFAULT_NOMBRE_SERVICIO);
		  
		  lsl.add(sl);
	  }
	  servCharla.setTextos(lsl);	  
      servCharla.setEdadMaxima(1);
      servCharla.setEdadMinima(1);
      disableMessages();
      persist(servCharla);
      enableMessages();
      charla.setServicio(servCharla);
    }
  }

}
