package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
import java.util.List;
import java.util.ArrayList;

import org.richfaces.event.UploadEvent;
import javax.faces.event.ActionEvent;
import javax.faces.model.SelectItem;
import javax.faces.model.SelectItemGroup;

import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.faces.util.FicheroSubidoRichFacesAdapter;
import es.csa.fdt.cyldigital.core.entity.FileOnDisk;
import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.core.entity.FileOnDisk.Tipo;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.language.entity.Language;
import es.csa.fdt.cyldigital.language.entity.PropuestaServicioLanguage;
import es.csa.fdt.cyldigital.language.entity.ServicioLanguage;
import es.csa.fdt.cyldigital.language.entity.ServicioCursoLanguage;
import es.csa.fdt.cyldigital.language.entity.TematicaLanguage;
import es.csa.fdt.cyldigital.servicios.entity.PropuestaServicio;
import es.csa.fdt.cyldigital.servicios.entity.Servicio;
import es.csa.fdt.cyldigital.servicios.entity.ServicioCurso;
import es.csa.fdt.cyldigital.servicios.entity.ServicioCursoEnlace;
import es.csa.fdt.cyldigital.servicios.entity.Tematica;
import es.csa.fdt.cyldigital.enlaces.Enlace;


/**
 * Clase ServicioCursoServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class ServicioCursoServiceImpl extends AbstractServiceImpl<ServicioCurso, Long> implements ServicioCursoService {
	private static final long serialVersionUID = -5575228180361917674L;

	//private File lastUploaded;
	//private FileOnDisk lastUploaded2;

	private Map<String, FileOnDisk> filesUploaded;
  
	private String fileToRemove;
	//private Set<Integer> languagesToUpdate;
  
	private List<ServicioCursoEnlace> linksToAdd;
	private List<ServicioCursoEnlace> linksToRemove;
  
	/*private List<Tematica> tematicas;
	private List<TematicaLanguage> tematicasLanguage;
	private List<SelectItem> tematicasSelectItems;*/
  
	/**
	 * Construye un nuevo objeto de la clase
	 */
	public ServicioCursoServiceImpl() {
		super();
    
		this.filesUploaded = new HashMap<String, FileOnDisk>();
		//this.languagesToUpdate = new HashSet<Integer>();
		this.linksToAdd = new ArrayList<ServicioCursoEnlace>();
		this.linksToRemove = new ArrayList<ServicioCursoEnlace>();
	}
  
	/* (non-Javadoc)
	 * @see es.csa.fdt.cyldigital.evaluacion.service.FormularioService#createLanguages(es.csa.fdt.cyldigital.evaluacion.entity.Formulario)
	 */
	public void createLanguages(ServicioCurso servicioCurso, List<Language> languages) {
		List<ServicioLanguage> textos = new ArrayList<ServicioLanguage>();
		List<ServicioCursoLanguage> guiones = new ArrayList<ServicioCursoLanguage>();

		int i = 0;
		for (Language language : languages) {
			ServicioCursoLanguage guion = new ServicioCursoLanguage();
			guion.setLanguage(language);
			guion.setServicio((Servicio)servicioCurso);
			guion.setServicioCurso(servicioCurso);

			textos.add((ServicioLanguage)guion);
			guiones.add(guion);
		  
			//languagesToUpdate.add(i);
			i++;
		}

		servicioCurso.setTextos(textos);
		servicioCurso.setGuiones(guiones);
	}
  
	public Enlace getNuevoEnlace() {
		return new Enlace();
	}
  
	public void eliminarEnlace(ServicioCurso servicioCurso, ServicioCursoEnlace servicioCursoEnlace) {
		if (!linksToAdd.contains(servicioCursoEnlace))
			this.linksToRemove.add(servicioCursoEnlace);
		else
			linksToAdd.remove(servicioCursoEnlace);
	  
		servicioCurso.getEnlaces().remove(servicioCursoEnlace);
	}
  
	public void anyadirEnlace(ServicioCurso servicioCurso, Enlace enlace) {
		ServicioCursoEnlace sce = new ServicioCursoEnlace();
	  
		sce.setEnlace(enlace);
		sce.setServicioCurso(servicioCurso);
	  
		if (servicioCurso.getEnlaces() == null)
			servicioCurso.setEnlaces(new ArrayList<ServicioCursoEnlace>());
	  
		servicioCurso.getEnlaces().add(sce);
		
		this.linksToAdd.add(sce);
	}
  
	/* (non-Javadoc)
	 * @see es.csa.fdt.cyldigital.servicios.service.ServicioCursoService#persistServicioCurso(es.csa.fdt.cyldigital.servicios.entity.ServicioCurso)
	 */
	public boolean persistServicioCurso(ServicioCurso curso) {
		if ((curso.getTematica() != null) && (curso.getTematica().getId() != 0)) {
			Parameter p = findOneWhere(Parameter.class, "nombre", "pathfileondisk");
		  
			for (ServicioCursoLanguage guion : curso.getGuiones()) {
				String fileKey = guion.getLanguage().getISO();
			  
				if (filesUploaded.containsKey(fileKey)) {
					guion.setFicheroGuion(filesUploaded.get(fileKey));
					filesUploaded.get(fileKey).grabarEnDisco(p.getValor());
					filesUploaded.get(fileKey).setTipo(Tipo.GUION);
				  
					disableMessages();
					persist(filesUploaded.get(fileKey));
					enableMessages();
				}
	
				/* 3.4.15-R #108439 
				if (guion.getGuion() != null)
					guion.setGuion(guion.getGuion().replaceAll("\n", "<br />"));
				*/
	
				if (guion.getDescripcion() != null)
					guion.setDescripcion(guion.getDescripcion().replaceAll("\n", "<br />"));
			}
		  
			for (ServicioCursoEnlace sce : this.linksToRemove) {
				Enlace enlace = sce.getEnlace();
			  
				remove(sce);
				remove(enlace);
			}
		  
			persist(curso);
		  
			return true;
		}
		else {
			addErrorMessage("servicios.errorTipologiaNull");
		  
			return false;
		}
	}

	/* (non-Javadoc)
	 * @see es.csa.fdt.cyldigital.servicios.service.ServicioCursoService#removeServicioCurso(es.csa.fdt.cyldigital.servicios.entity.ServicioCurso)
	 */
	public void removeServicioCurso(ServicioCurso curso) {
		if (curso.getId() != 0) {
			if (curso.getEnlaces() != null) {
				disableMessages();
				
				for (ServicioCursoEnlace sce : curso.getEnlaces()) {
					Enlace enlace = sce.getEnlace();

					remove(sce);
					remove(enlace);
				}
				
				enableMessages();
			}

			remove(curso);
		}
	}
  
	public void deactivateServicioCurso(ServicioCurso curso) {
		if (curso.getId() != 0) {
			disableMessages();

			for (ServicioCursoLanguage scl : curso.getGuiones()) {
				scl.setDesactivado(true);
				
				persist(scl);
			}
			
			curso.setDesactivado(true);

			persist(curso);
			
			enableMessages();
			
			addInfoMessage("servicios.messageDeactivatedService");
		}
	}

	/**
	 * Update from propuesta.
	 * 
	 * @param idPropuesta
	 *           id propuesta
	 * @param sc
	 *           course service
	 *           
	 * @return course service updated
	 */
	public ServicioCurso updateFromPropuesta(Long idPropuesta, ServicioCurso sc) {
		PropuestaServicio ps = find(PropuestaServicio.class, idPropuesta);

		if (ps != null) {
			List<PropuestaServicioLanguage> lpsl = ps.getTextos();
    	
			List<ServicioLanguage> lsl = new ArrayList<ServicioLanguage>();
			List<ServicioCursoLanguage> lscl = new ArrayList<ServicioCursoLanguage>();
    	
			for (PropuestaServicioLanguage psl : lpsl) {
				ServicioCursoLanguage scl = new ServicioCursoLanguage();
    		
				scl.setLanguage(psl.getLanguage());
				scl.setServicio((Servicio)sc);
				scl.setServicioCurso(sc);
    		
				scl.setNombre(psl.getNombre());
				scl.setDescripcion(psl.getComentarios());
    		
				lsl.add((ServicioLanguage)scl);
				lscl.add(scl);
			}
    	
			sc.setTextos(lsl);
			sc.setGuiones(lscl);
		}

		return sc;
	}

	/**
	 * Aniadir fichero.
	 * 
	 * @param event
	 *           event
	 */
	public synchronized void aniadirFichero(UploadEvent event) {
		FileOnDisk fileUploaded = new FileOnDisk();
		String fileKey = (String)event.getComponent().getAttributes().get("lang");

		fileUploaded.setFicheroSubido(new FicheroSubidoRichFacesAdapter(event.getUploadItem()));

		filesUploaded.put(fileKey, fileUploaded);
	}
  
	/**
	 * Marcar fichero para quitar.
	 * 
	 * @param event
	 *           event
	 */
	public synchronized void marcarFicheroParaQuitar(ActionEvent event) {
		fileToRemove = (String)event.getComponent().getAttributes().get("lang");
	}

	/**
	 * Quitar fichero.
	 * 
	 * @param s
	 *           s
	 * @param service
	 *           service
	 */
	public void quitarFichero(ServicioCurso s) {
		filesUploaded.remove(fileToRemove);
	
		for(ServicioCursoLanguage guion: s.getGuiones())
			if (guion.getLanguage().getISO().compareTo(fileToRemove) == 0)
				guion.setFicheroGuion(null);
	}

	/**
	 * Devuelve filesUploaded
	 * 
	 * @return the filesUploaded
	 */
	public Map<String, FileOnDisk> getFilesUploaded() {
		return filesUploaded;
	}

	/**
	 * Asigna filesUploaded
	 * 
	 * @param filesUploaded the filesUploaded to set
	 */
	public void setFilesUploaded(Map<String, FileOnDisk> filesUploaded) {
		this.filesUploaded = filesUploaded;
	}
}
