package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.List;

import org.richfaces.event.UploadEvent;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.faces.util.FicheroSubidoRichFacesAdapter;
import es.csa.fdt.cyldigital.core.entity.File;
import es.csa.fdt.cyldigital.core.entity.FileOnDisk;
import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.core.entity.FileOnDisk.Tipo;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.servicios.entity.PropuestaServicio;
import es.csa.fdt.cyldigital.servicios.entity.ServicioTaller;
import es.csa.fdt.cyldigital.servicios.entity.Tematica;


/**
 * Clase ServicioTallerServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class ServicioTallerServiceImpl extends AbstractServiceImpl<ServicioTaller, Long> implements
    ServicioTallerService {

  private static final long serialVersionUID = -5575228180361917674L;

  private File lastUploaded;
  private FileOnDisk lastUploaded2;

  private List<Tematica> tematicas;

  /**
   * Construye un nuevo objeto de la clase
   */
  public ServicioTallerServiceImpl() {
    super();
  }

  /**
   * Devuelve tematicas.
   * 
   * @return tematicas
   */
  public List<Tematica> getTematicas() {
    return tematicas != null ? tematicas : findAllOrder(Tematica.class, "nombre", true);
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.ServicioTallerService#persistServicioTaller(es.csa.fdt.cyldigital.servicios.entity.ServicioTaller)
   */
  public void persistServicioTaller(ServicioTaller taller) {
    if (lastUploaded2 != null) {
      Parameter p = findOneWhere(Parameter.class, "nombre", "pathfileondisk");

      taller.setFicheroGuion(lastUploaded2);
      lastUploaded2.grabarEnDisco(p.getValor());
      lastUploaded2.setTipo(Tipo.GUION);

      disableMessages();
      persist(lastUploaded2);
      enableMessages();
    }
    
    /* 3.4.15-R #108439 
    if (taller.getGuion() != null) taller.setGuion(taller.getGuion().replaceAll("\n", "<br>"));
    */
    
    /*
     * Comentado para implementar los servicios MULTILANGUAGE
     * TODO: mirar y modificar
    if (taller.getDescripcion() != null) taller.setDescripcion(taller.getDescripcion().replaceAll("\n", "<br>"));
    */

    persist(taller);
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.ServicioTallerService#removeServicioTaller(es.csa.fdt.cyldigital.servicios.entity.ServicioTaller)
   */
  public void removeServicioTaller(ServicioTaller taller) {
    if (taller.getId() != 0) remove(taller);
  }

  /**
   * Aniadir pdf.
   * 
   * @param event
   *           event
   */
  public synchronized void aniadirPdf(UploadEvent event) {
    lastUploaded = new File();
    lastUploaded.setFicheroSubido(new FicheroSubidoRichFacesAdapter(event.getUploadItem()));
  }

  /**
   * Quitar pdf.
   * 
   * @param taller
   *           taller
   * @return servicio taller
   */
  public ServicioTaller quitarPdf(ServicioTaller taller) {
    borrarFicheroTemporal(taller.getGuionPdf());
    taller.setGuionPdf(null);
    setLastUploaded(null);
    return taller;
  }

  /**
   * Borrar fichero temporal.
   * 
   * @param file
   *           file
   */
  public void borrarFicheroTemporal(File file) {
    if (file != null && file.getFicheroSubido() != null) {
      file.getFicheroSubido().deleteFichero();
    }
  }

  /**
   * Devuelve last uploaded.
   * 
   * @return last uploaded
   */
  public File getLastUploaded() {
    return lastUploaded;
  }

  /**
   * Asigna last uploaded.
   * 
   * @param lastUploaded
   *          last uploaded
   */
  public void setLastUploaded(File lastUploaded) {
    this.lastUploaded = lastUploaded;
  }

  /**
   * Do anniadir fichero a documento.
   * 
   * @param taller
   *           taller
   */
  public void doAnniadirFicheroADocumento(ServicioTaller taller) {
    if (lastUploaded != null) {
      taller.setGuionPdf(lastUploaded);
      setLastUploaded(null);
    }
  }

  /**
   * Update from propuesta.
   * 
   * @param idPropuesta
   *           id propuesta
   * @param t
   *           t
   * @return servicio taller
   */
  public ServicioTaller updateFromPropuesta(Long idPropuesta, ServicioTaller t) {
    PropuestaServicio ps = find(PropuestaServicio.class, idPropuesta);
    if (ps != null) {
        /*
         * Comentado para implementar los servicios MULTILANGUAGE
         * TODO: mirar y modificar
      t.setNombre(ps.getNombre());
      t.setDescripcion(ps.getComentarios());
      */
    }
    return t;
  }

  /**
   * Devuelve last uploaded2.
   * 
   * @return last uploaded2
   */
  public FileOnDisk getLastUploaded2() {
    return lastUploaded2;
  }

  /**
   * Asigna last uploaded2.
   * 
   * @param lastUploaded2
   *          last uploaded2
   */
  public void setLastUploaded2(FileOnDisk lastUploaded2) {
    this.lastUploaded2 = lastUploaded2;
  }

  /**
   * Aniadir fichero.
   * 
   * @param event
   *           event
   */
  public synchronized void aniadirFichero(UploadEvent event) {
    lastUploaded2 = new FileOnDisk();
    lastUploaded2.setFicheroSubido(new FicheroSubidoRichFacesAdapter(event.getUploadItem()));
  }

  /**
   * Quitar fichero.
   * 
   * @param s
   *           s
   */
  public void quitarFichero(ServicioTaller s) {
    lastUploaded2 = null;
    s.setFicheroGuion(null);
  }
}
