package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.List;

import es.csa.fdt.cyldigital.calendario.service.ActividadCalendarItemServiceImpl;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.service.AbstractService;
import es.csa.fdt.cyldigital.evaluacion.entity.Formulario;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaTaller;
import es.csa.fdt.cyldigital.servicios.entity.ServicioTaller;
import es.csa.fdt.cyldigital.servicios.entity.Taller;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;

/**
 * Interfaz del servicio de Taller.
 * 
 * @author csa
 */
public interface TallerService extends AbstractService<Taller, Long> {

  /**
   * Persist taller.
   * 
   * @param taller
   *           taller
   * @param serviceCal
   *           service cal
   * @return true, if successful
   */
  public boolean persistTaller(Taller taller, ActividadCalendarItemServiceImpl serviceCal);

  /**
   * Elimina taller.
   * 
   * @param taller
   *           taller
   */
  public void removeTaller(Taller taller);

  /**
   * Devuelve servicios disponibles.
   * 
   * @return servicios disponibles
   */
  public List<ServicioTaller> getServiciosDisponibles();

  /**
   * Devuelve centros.
   * 
   * @return centros
   */
  public List<Centro> getCentros();

  /**
   * Devuelve profesores.
   * 
   * @return profesores
   */
  public List<Empleado> getProfesores();

  /**
   * Devuelve formularios.
   * 
   * @return formularios
   */
  public List<Formulario> getFormularios();

  /**
   * Comprobar edad usuario.
   * 
   * @param m
   *           m
   */
  public void comprobarEdadUsuario(MatriculaTaller m);
}
