package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import javax.annotation.Resource;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.calendario.service.ActividadCalendarItemServiceImpl;
import es.csa.fdt.cyldigital.centros.entity.Aula;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.evaluacion.entity.Formulario;
import es.csa.fdt.cyldigital.recursos.entity.Recurso;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaTaller;
import es.csa.fdt.cyldigital.servicios.entity.ObservacionActividad;
import es.csa.fdt.cyldigital.servicios.entity.PreinscripcionTaller;
import es.csa.fdt.cyldigital.servicios.entity.ServicioTaller;
import es.csa.fdt.cyldigital.servicios.entity.Taller;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario.Role;


/**
 * Clase TallerServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class TallerServiceImpl extends AbstractServiceImpl<Taller, Long> implements TallerService {

  private List<Centro> centros;

  private Centro centro;

  private boolean changeCentro = true;

  private List<Empleado> profesores;

  private List<Formulario> formularios;

  private static final long serialVersionUID = -5575228180361917674L;

  private boolean avisoFechaMostrado = false;

  private boolean avisoRecursos = false;

  private boolean borrarPreinscripcion = false;

  @Resource
  private transient HashMap<String, Object> generalProperties;

  /**
   * Devuelve centro.
   * 
   * @return centro
   */
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Apply centro.
   * 
   * @param taller
   *           taller
   */
  public void applyCentro(Taller taller) {
    this.centro = taller.getCentro();
    taller.setAula(null);
    changeCentro = true;
  }

  /**
   * Construye un nuevo objeto de la clase
   */
  public TallerServiceImpl() {
    super();
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.TallerService#persistTaller(es.csa.fdt.cyldigital.servicios.entity.Taller, es.csa.fdt.cyldigital.calendario.service.ActividadCalendarItemServiceImpl)
   */
  public boolean persistTaller(Taller taller, ActividadCalendarItemServiceImpl serviceCal) {
    if (taller.getFechaInicio().getTime() < (new Date()).getTime() && !avisoFechaMostrado && !taller.isCerrada()) {
      addWarnMessage("actividad.validator.fechaInicioAnterior");
      avisoFechaMostrado = true;
    }
    /*  se anula la verificacion del numero de recursos
    else if (!avisoRecursos && taller.getNumeroPlazas() > taller.getRecursos().size()) {
      addWarnMessage("actividad.validator.plazasMayorRecursos");
      avisoRecursos = true;      
    } 
    */
    else {
      if (taller.getId() > 0 && taller.getCalendarItems().isEmpty()) {
        serviceCal.generateCalendar(taller.getId());
      }
      persist(taller);
      return true;
    }
    return false;
  }

  /**
   * Persist taller to calendario.
   * 
   * @param taller
   *           taller
   * @return true, if successful
   */
  public boolean persistTallerToCalendario(Taller taller) {
    if (taller.getNumeroHoras() == 0) {
      taller.setNumeroHoras(2);
    }
    if (taller.getFechaInicio().getTime() < (new Date()).getTime() && !avisoFechaMostrado && !taller.isCerrada()) {
      addWarnMessage("actividad.validator.fechaInicioAnterior");
      avisoFechaMostrado = true;
    } else {
      setMessagesEnabled(false);
      persist(taller);
      setMessagesEnabled(true);
      return true;
    }
    return false;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.TallerService#removeTaller(es.csa.fdt.cyldigital.servicios.entity.Taller)
   */
  public void removeTaller(Taller taller) {
    if (taller.getId() != 0) remove(taller);
  }

  /**
   * Elimina taller from cancel.
   * 
   * @param taller
   *           taller
   */
  public void removeTallerFromCancel(Taller taller) {
    if (taller.getId() != 0) {
      setMessagesEnabled(false);
      remove(taller);
      setMessagesEnabled(true);
    }
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.TallerService#getServiciosDisponibles()
   */
  public List<ServicioTaller> getServiciosDisponibles() {
    return findAllOrder(ServicioTaller.class, "nombre", true);
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.TallerService#getCentros()
   */
  public List<Centro> getCentros() {
    if (centros == null) centros = findAllOrder(Centro.class, "nombre", true);
    return centros;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.TallerService#getProfesores()
   */
  public List<Empleado> getProfesores() {
    if (centro != null && changeCentro) {
      List<Empleado> empleados = findWhereOrder(Empleado.class, "centro.id", centro.getId(), "nombre", true);
      profesores = new ArrayList<Empleado>();
      for (Empleado e : empleados) {
        if (e.isProfesor()) {
          profesores.add(e);
        }
      }
      changeCentro = false;
    }
    return profesores;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.TallerService#getFormularios()
   */
  public List<Formulario> getFormularios() {
    if (formularios == null) formularios = findAllOrder(Formulario.class, "nombre", true);
    return formularios;
  }

  /**
   * Checks if is borrar preinscripcion.
   * 
   * @return true, if is borrar preinscripcion
   */
  public boolean isBorrarPreinscripcion() {
    return borrarPreinscripcion;
  }

  /**
   * Asigna borrar preinscripcion.
   * 
   * @param borrarPreinscripcion
   *          borrar preinscripcion
   */
  public void setBorrarPreinscripcion(boolean borrarPreinscripcion) {
    this.borrarPreinscripcion = borrarPreinscripcion;
  }

  /**
   * Rechazar taller.
   * 
   * @param taller
   *           taller
   * @param idPreinscripcion
   *           id preinscripcion
   */
  public void rechazarTaller(Taller taller, Long idPreinscripcion) {
    if (borrarPreinscripcion) {
      PreinscripcionTaller p = find(PreinscripcionTaller.class, idPreinscripcion);
      if (p != null) remove(p);
    } else {
      PreinscripcionTaller p = find(PreinscripcionTaller.class, idPreinscripcion);
      if (p != null) {
        p.getTalleresRechazos().add(taller);
        p.setPeso(p.getPeso() + 1);
        persist(p);
      }
    }
  }

  /**
   * Matricula from preinscripcion.
   * 
   * @param taller
   *           taller
   * @param idPreinscripcion
   *           id preinscripcion
   * @return true, if successful
   */
  public boolean matriculaFromPreinscripcion(Taller taller, Long idPreinscripcion) {
    PreinscripcionTaller p = find(PreinscripcionTaller.class, idPreinscripcion);
    boolean flag = false;
    if (p != null) {
      Long idUsuario = p.getUsuario().getId();
      Usuario u = find(Usuario.class, idUsuario);

      for (MatriculaTaller matr : taller.getMatriculas()) {
        if (matr.getUsuario().getId() == idUsuario) {
          flag = true;
          addErrorMessage("El usuario ya está matriculado en este taller");
        }
      }

      if (!flag) {
        ((ServicioTaller) taller.getServicio()).getPreinscripciones().remove(p);
        u.getPreinscripcionesTalleres().remove(p);

        MatriculaTaller m = new MatriculaTaller();
        m.setAdmitido(true);
        m.setDiplomaCreado(false);
        m.setFechaMatricula(new Date());

        m.setTaller(taller);
        m.setUsuario(u);
        persist(m);
        setMessagesEnabled(false);
        remove(p);
        setMessagesEnabled(true);

        return true;
      }
    }
    return false;
  }

  /**
   * Actualiza centro.
   * 
   * @param taller
   *           taller
   * @param currentUser
   *           current user
   */
  public void actualizaCentro(Taller taller, UsuarioLdap currentUser) {
    centro = taller.getCentro() != null ? taller.getCentro() : null;

    if (currentUser.getCentro() != null) {
      Centro c = find(Centro.class, currentUser.getCentro().getId());
      taller.setCentro(c);
      centro = c;
      changeCentro = true;
    } else if (generalProperties.containsKey("DEFAULT_CENTER_ID") && taller.getCentro() == null) {
      Centro c = find(Centro.class, Long.valueOf((String) generalProperties.get("DEFAULT_CENTER_ID")));
      if (c != null) {
        taller.setCentro(c);
        centro = c;
        changeCentro = true;
      }
    }
  }

  /**
   * Actualiza aula.
   * 
   * @param taller
   *           taller
   */
  public void actualizaAula(Taller taller) {
    centro = taller.getCentro() != null ? taller.getCentro() : null;

    if (centro != null) {
      for (Aula a : centro.getAulas()) {
        if (a.getNombre().toLowerCase().contains("taller")) {
          taller.setAula(a);
          return;
        }
      }
    }
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.TallerService#comprobarEdadUsuario(es.csa.fdt.cyldigital.servicios.entity.MatriculaTaller)
   */
  public void comprobarEdadUsuario(MatriculaTaller m) {

    if (m.getTaller() != null && m.getUsuario() != null) {
      UsuarioCyL u = find(UsuarioCyL.class, m.getUsuario().getId());
      if (u.getEdad() < m.getTaller().getServicio().getEdadMinima()
          || u.getEdad() > m.getTaller().getServicio().getEdadMaxima()) {
        addWarnMessage("matricula.validator.edad_incorrecta");
      }
    }
  }

  /**
   * Anadir observacion guion.
   * 
   * @param taller
   *           taller
   * @param o
   *           o
   * @param usuario
   *           usuario
   */
  public void anadirObservacionGuion(Taller taller, ObservacionActividad o, Usuario usuario) {
    if (o.getObservacion().equals("")) addWarnMessage("No puede añadir un comentario vacío");
    else {
      o.setFecha(new Date());
      o.setUsuario(usuario);
      o.setActividad(taller);
      taller.getObservacionesActividad().add(o);
      persist(taller);
    }
  }

  /**
   * Devuelve nueva observacion guion.
   * 
   * @return nueva observacion guion
   */
  public ObservacionActividad getNuevaObservacionGuion() {
    return new ObservacionActividad();
  }

  /**
   * Devuelve next taller.
   * 
   * @param u
   *           u
   * @param t
   *           t
   * @param next
   *           next
   * @return next taller
   */
  public Taller getNextTaller(UsuarioLdap u, Taller t, boolean next) {
    /*
     * next=false: devuelve, si hay, la incidencia anterior. A true, la posterior. Recordar que el curso anterior es de
     * fecha posterior al actual. Devuelve null si no hay una incidencia anterior/posterior, si la hay, devuelve su id
     */
    if (t != null) { // en ocasiones el taller se queda a nulo por ahi.
      List<Taller> talleres = new ArrayList<Taller>();

      if (!u.getRolesList().contains(Role.ROLE_ACTIVITY_TRAINER_VIEW_ALL)) {
        talleres = findByQuery(Taller.class,
            " Select c from es.csa.fdt.cyldigital.servicios.entity.Taller c where centro="
                + u.getUser().getCentro().getId() + " order by fechaInicio desc, horaInicio desc");
      } else {
        talleres = findByQuery(Taller.class,
            " Select c from es.csa.fdt.cyldigital.servicios.entity.Taller c  order by fechaInicio desc, horaInicio desc");
      }

      if (talleres.indexOf(t) > -1) {
        if (next) {
          if (talleres.indexOf(t) + 1 < talleres.size()) return talleres.get(talleres.indexOf(t) + 1);
        } else {
          if (talleres.indexOf(t) != 0) return talleres.get(talleres.indexOf(t) - 1);
        }
      }
    }
    return null;
  }

  /**
   * Añade recurso.
   * 
   * @param rec
   *           rec
   * @param taller
   *           taller
   */
  public void addRecurso(Recurso rec, Taller taller) {
    if (rec.isOcupado(taller)) {
      addWarnMessage("El recurso '" + rec.getNombre() + "' está reservado o se encuentra ocupado en otra actividad");
    } else if (!taller.getRecursos().contains(rec)) taller.getRecursos().add(rec);
  }

  /**
   * Quitar recurso.
   * 
   * @param rec
   *           rec
   * @param taller
   *           taller
   */
  public void quitarRecurso(Recurso rec, Taller taller) {
    taller.getRecursos().remove(rec);
  }

  /**
   * Quitar todos recursos.
   * 
   * @param taller
   *           taller
   */
  public void quitarTodosRecursos(Taller taller) {
    taller.getRecursos().removeAll(taller.getRecursos());
  }

  /**
   * Añade recursos aula.
   * 
   * @param taller
   *           taller
   */
  public void addRecursosAula(Taller taller) {
    String message = "";
    int i = 0;
    if (taller != null && taller.getAula() != null && taller.getFechaInicio() != null && taller.getFechaFin() != null
        && taller.getHoraInicio() != null && taller.getHoraFin() != null) {
      for (Recurso r : taller.getAula().getRecursos()) {
        if (!r.isOcupado(taller)) {
          addRecurso(r, taller);
          i++;
        } else {
          message += "(" + r.getNombre() + ") ";
        }
      }
      if (i < taller.getAula().getRecursos().size()) {
        addWarnMessage("Existen recursos reservados u ocupados en otra actividad, que no pueden agregarse al taller "
            + message);
      }
    } else {
      addErrorMessage("Antes de añadir recursos agregue el calendario del taller");
    }
  }
}
