package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.faces.model.SelectItem;
import javax.faces.model.SelectItemGroup;

import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.language.entity.Language;
import es.csa.fdt.cyldigital.language.entity.TematicaLanguage;
import es.csa.fdt.cyldigital.servicios.entity.Tematica;



/**
 * Clase TematicaServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class TematicaServiceImpl extends AbstractServiceImpl<Tematica, Long> implements TematicaService {

  private static final long serialVersionUID = -5575228180361917674L;
  
  private List<Tematica> tematicas;
  private List<TematicaLanguage> tematicasLanguage;
  private List<SelectItem> tematicasSelectItems;

  /**
   * Construye un nuevo objeto de la clase
   */
  public TematicaServiceImpl() {
    super();
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.TematicaService#persistTematica(es.csa.fdt.cyldigital.servicios.entity.Tematica)
   */
  public boolean persistTematica(Tematica tematica) {

    persist(tematica);
    return true;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.TematicaService#getTematicasDisponibles()
   */
  /*public List<Tematica> getTematicasDisponibles() {
    return findAll(Tematica.class);
  }
  
  public List<TematicaLanguage> getTematicasLanguageDisponibles() {
	  String idioma = LocaleContextHolder.getLocaleContext().getLocale().toString();

	  return findWhere(TematicaLanguage.class, "language.ISO", idioma);
  }*/
  
  /**
   * Devuelve tematicasItems.
   * 
   * @return tematicasItems
   */
  public List<SelectItem> getTematicasSelectItems() {
	  if (tematicasSelectItems == null)
		  tematicasSelectItems = new ArrayList<SelectItem>();
	  
	  if (!tematicasSelectItems.isEmpty())
		  tematicasSelectItems.clear();
		  
	  Map<String, Object> propiedades = new HashMap<String, Object>();
	  
	  String idioma = LocaleContextHolder.getLocaleContext().getLocale().toString();
	  
	  propiedades.put("language.ISO", idioma);
	  propiedades.put("tematica.padre", null);
	  propiedades.put("tematica.desactivado", false);

	  List<TematicaLanguage> tematicasPadre = findWhereOrder(TematicaLanguage.class, propiedades, "nombre", true);
	  
	  for (TematicaLanguage tematicaPadre : tematicasPadre) {
		  propiedades.put("tematica.padre", tematicaPadre.getTematica());
		  propiedades.put("tematica.desactivado", false);
		  
		  SelectItemGroup tematicasSelectItemGroup;
		  List<TematicaLanguage> tematicasHija = findWhereOrder(TematicaLanguage.class, propiedades, "nombre", true);

		  List<SelectItem> tematicasHijaSelectItem = new ArrayList<SelectItem>();

		  for (TematicaLanguage tematicaHija : tematicasHija)
			  tematicasHijaSelectItem.add(new SelectItem(tematicaHija.getTematica(), tematicaHija.getNombre()));
		  
		  SelectItem[] tematicasHijaArray = tematicasHijaSelectItem.toArray(new SelectItem[tematicasHija.size()]);
		  
		  tematicasSelectItemGroup = new SelectItemGroup(tematicaPadre.getNombre(),
				  										 "",
				  										 false,
				  										 tematicasHijaArray);
		  
		  tematicasSelectItems.add(tematicasSelectItemGroup);
	  }
	  
	  return tematicasSelectItems;
  }

  /**
   * Devuelve tematicasLanguage.
   * 
   * @return tematicasLanguage
   */
  public List<TematicaLanguage> getTematicasLanguage() {
	  if (tematicasLanguage == null)
		  tematicasLanguage = new ArrayList<TematicaLanguage>();
	  
	  if (!tematicasLanguage.isEmpty())
		  tematicasLanguage.clear();
  
	  String idioma = LocaleContextHolder.getLocaleContext().getLocale().toString();
	  Map<String, Object> propiedades = new HashMap<String, Object>();
	  
	  propiedades.put("language.ISO", idioma);
	  propiedades.put("tematica.padre", null);
	  propiedades.put("tematica.desactivado", false);
	  
	  List<TematicaLanguage> tematicasPadre = findWhereOrder(TematicaLanguage.class, propiedades, "nombre", true);
	  
	  for (TematicaLanguage tematicaPadre : tematicasPadre) {
		  propiedades.put("tematica.padre", tematicaPadre.getTematica());
		  propiedades.put("tematica.desactivado", false);

		  List<TematicaLanguage> tematicasHija = findWhereOrder(TematicaLanguage.class, propiedades, "nombre", true);
		  
		  tematicasLanguage.add(tematicaPadre);
		  tematicasLanguage.addAll(tematicasHija);
	  }
	  
	  return tematicasLanguage;
  }

  /**
   * Devuelve tematicas.
   * 
   * @return tematicas
   */
  public List<Tematica> getTematicas() {
	  if (tematicas == null)
		  tematicas = new ArrayList<Tematica>();
	  
	  if (!tematicas.isEmpty())
		  tematicas.clear();
	  
	  List<TematicaLanguage> tematicasLanguage = getTematicasLanguage();

	  for(TematicaLanguage tematicaLanguage : tematicasLanguage)
		  tematicas.add(tematicaLanguage.getTematica());
	  
	  return tematicas;
  }

  /**
   * Elimina tematica.
   * 
   * @param tematica
   *           tematica
   */
  public void removeTematica(Tematica tematica) {
	  if (tematica.isBorrable()) {
		  disableMessages();
		  
		  if (tematica.getHijas() != null) {
			  for (Tematica th : tematica.getHijas())
				  removeTematica(th);
		  }
		  
		  enableMessages();

	      remove(tematica);
	  }
  }
  
  /**
   * Desactiva tematica.
   * 
   * @param tematica
   *           tematica
   */
  public void deactivateTematica(Tematica tematica) {
	  if (tematica.isDesactivable()) {
		  disableMessages();
		  
		  if (tematica.getHijas() != null) {
			  for (Tematica th : tematica.getHijas())
				  deactivateTematica(th);
		  }

		  for (TematicaLanguage tl : tematica.getTextos()) {
			  tl.setDesactivado(true);

			  persist(tl);
		  }

	      tematica.setDesactivado(true);
	      
	      persist(tematica);
	      
	      enableMessages();
	      
	      addInfoMessage("servicios.messageDeactivatedTypology");
	  }
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.TematicaServiceImpl#createLanguages(es.csa.fdt.cyldigital.servicios.entity.Tematica)
   */
  public void createLanguages(Tematica tematica, List<Language> languages) {	  
	  List<TematicaLanguage> textos = new ArrayList<TematicaLanguage>();

	  for (Language language : languages) {
		  TematicaLanguage texto = new TematicaLanguage();
		  texto.setLanguage(language);
		  texto.setTematica(tematica);

		  textos.add(texto);
	  }

	  tematica.setTextos(textos);
  }
  
  public Tematica find(Long id) {
	  if (id != null)
		  return super.find(id);
	  
	  return null;
  }
}
