package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.List;

import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.language.entity.Language;
import es.csa.fdt.cyldigital.language.entity.TematicaLanguage;
import es.csa.fdt.cyldigital.language.entity.TipoPracticaLanguage;
import es.csa.fdt.cyldigital.servicios.entity.PracticaTaller;
import es.csa.fdt.cyldigital.servicios.entity.Tematica;
import es.csa.fdt.cyldigital.servicios.entity.TipoPractica;


/**
 * Clase TipoPracticaServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class TipoPracticaServiceImpl extends AbstractServiceImpl<TipoPractica, Long> implements TipoPracticaService {

  private static final long serialVersionUID = -5575228180361917674L;

  /**
   * Construye un nuevo objeto de la clase
   */
  public TipoPracticaServiceImpl() {
    super();
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.TipoPracticaService#persistTipoPractica(es.csa.fdt.cyldigital.servicios.entity.TipoPractica)
   */
  public boolean persistTipoPractica(TipoPractica tipoPractica) {

    persist(tipoPractica);
    return true;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.TipoPracticaService#getTiposPracticasDisponibles()
   */
  public List<TipoPractica> getTiposPracticasDisponibles() {
    return findAll(TipoPractica.class);
  }

  /**
   * Elimina tipo practica.
   * 
   * @param tipoPractica
   *           tipo practica
   */
  public void removeTipoPractica(TipoPractica tipoPractica) {
    if (tipoPractica.getId() != 0) {
      for (PracticaTaller a : tipoPractica.getPracticasTalleres()) {
        a.setTipoPractica(null);
      }
      tipoPractica.getPracticasTalleres().clear();
      remove(tipoPractica);
    }
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.TematicaServiceImpl#createLanguages(es.csa.fdt.cyldigital.servicios.entity.Tematica)
   */
  public void createLanguages(TipoPractica tipoPractica, List<Language> languages){	  
	  List<TipoPracticaLanguage> textos = new ArrayList<TipoPracticaLanguage>();

	  for (Language language : languages) {
		  TipoPracticaLanguage texto = new TipoPracticaLanguage();
		  texto.setLanguage(language);
		  texto.setTipoPractica(tipoPractica);

		  textos.add(texto);
	  }

	  tipoPractica.setTextos(textos);
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.TipoPractica#getTipoPracticasDisponibles()
   */
  public List<TipoPractica> getTipoPracticasDisponibles() {
    return findAll(TipoPractica.class);
  }
  
  public List<TipoPracticaLanguage> getTipoPracticasLanguageDisponibles() {
	  String idioma = LocaleContextHolder.getLocaleContext().getLocale().toString();

	  return findWhere(TipoPracticaLanguage.class, "language.ISO", idioma);
  } 
  
}
