package es.csa.fdt.cyldigital.servicios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import javax.annotation.Resource;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.calendario.entity.ActividadCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.ActividadCalendarItem.ActividadType;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.servicios.entity.Wifi;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario.Role;


/**
 * Clase WifiServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class WifiServiceImpl extends AbstractServiceImpl<Wifi, Long> implements WifiService {

  private List<Centro> centros;

  private static final long serialVersionUID = -5575228180361917674L;

  @Resource
  private transient HashMap<String, Object> generalProperties;

  /**
   * Construye un nuevo objeto de la clase
   */
  public WifiServiceImpl() {
    super();
  }

  /**
   * Actualiza fecha.
   * 
   * @param wifi
   *           wifi
   */
  public void actualizaFecha(Wifi wifi) {
    if (wifi.getId() == 0) {
      wifi.setFecha(new Date());
    }
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.WifiService#persistWifi(es.csa.fdt.cyldigital.servicios.entity.Wifi)
   */
  public boolean persistWifi(Wifi wifi) {
    ActividadCalendarItem item = new ActividadCalendarItem();
    item.setActividadType(ActividadType.WIFI);
    item.setActividad(wifi);
    item.setUsuario(wifi.getUsuario());
    item.setStart(wifi.getFecha());
    wifi.addCalendarItem(item);

    persist(wifi);
    return true;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.WifiService#removeWifi(es.csa.fdt.cyldigital.servicios.entity.Wifi)
   */
  public void removeWifi(Wifi wifi) {
    if (wifi.getId() != 0) remove(wifi);
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.servicios.service.WifiService#getCentros()
   */
  public List<Centro> getCentros() {
    if (centros == null) centros = findAllOrder(Centro.class, "nombre", true);
    return centros;
  }

  /**
   * Actualiza centro.
   * 
   * @param wifi
   *           wifi
   * @param currentUser
   *           current user
   */
  public void actualizaCentro(Wifi wifi, UsuarioLdap currentUser) {
    if (currentUser.getCentro() != null) {
      Centro c = find(Centro.class, currentUser.getCentro().getId());
      wifi.setCentro(c);
    } else if (generalProperties.containsKey("DEFAULT_CENTER_ID") && wifi.getCentro() == null) {
      Centro c = find(Centro.class, Long.valueOf((String) generalProperties.get("DEFAULT_CENTER_ID")));
      if (c != null) wifi.setCentro(c);
    }
  }

  /**
   * Devuelve next wifi.
   * 
   * @param u
   *           u
   * @param a
   *           a
   * @param next
   *           next
   * @return next wifi
   */
  public Wifi getNextWifi(UsuarioLdap u, Wifi a, boolean next) {
    if (a != null) {
      List<Wifi> wifis = new ArrayList<Wifi>();

      if (!u.getRolesList().contains(Role.ROLE_ACTIVITY_TRAINER_VIEW_ALL)) {
        wifis = findByQuery(Wifi.class, " Select a from es.csa.fdt.cyldigital.servicios.entity.Wifi a where centro="
            + u.getUser().getCentro().getId() + " order by fecha desc");
      } else {
        wifis = findByQuery(Wifi.class,
            " Select a from es.csa.fdt.cyldigital.servicios.entity.Wifi a  order by fecha desc");
      }

      if (wifis.indexOf(a) > -1) {
        if (next) {
          if (wifis.indexOf(a) + 1 < wifis.size()) return wifis.get(wifis.indexOf(a) + 1);
        } else {
          if (wifis.indexOf(a) != 0) return wifis.get(wifis.indexOf(a) - 1);
        }
      }
    }
    return null;
  }

}
