package es.csa.fdt.cyldigital.tareas.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.Date;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.Lob;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;

/**
 * Clase InformeTarea.
 * 
 * @author csa
 */
@Entity
@Table(name = "informestareas")
@Audited
@AttributeOverride(name = "id", column = @Column(name = "idInformeTarrea"))
public class InformeTarea extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = -8257900510191741304L;

  /**
   * Enum EstadoTarea.
   */
  public static enum EstadoTarea {
    // ROLES GENERALES (NO UTILIZAR SI ES POSIBLE)
    NOT_DONE,
    ERROR,
    OK,
  }

  private Date fechaInicio;
  private Date fechaFin;
  private String nombre;
  private EstadoTarea estadoTarea;
  private String mensaje;

  /**
   * Devuelve nombre.
   * 
   * @return nombre
   */
  @Length(max = 255)
  @NotEmpty
  @NotNull
  public String getNombre() {
    return nombre;
  }

  /**
   * Asigna nombre.
   * 
   * @param nombre
   *          nombre
   */
  public void setNombre(String nombre) {
    this.nombre = nombre;
  }

  /**
   * Devuelve fecha inicio.
   * 
   * @return fecha inicio
   */
  @Temporal(TemporalType.TIMESTAMP)
  public Date getFechaInicio() {
    return fechaInicio;
  }

  /**
   * Asigna fecha inicio.
   * 
   * @param fecha
   *          fecha inicio
   */
  public void setFechaInicio(Date fecha) {
    this.fechaInicio = fecha;
  }

  /**
   * Devuelve fecha fin.
   * 
   * @return fecha fin
   */
  @Temporal(TemporalType.TIMESTAMP)
  public Date getFechaFin() {
    return fechaFin;
  }

  /**
   * Asigna fecha fin.
   * 
   * @param fechaFin
   *          fecha fin
   */
  public void setFechaFin(Date fechaFin) {
    this.fechaFin = fechaFin;
  }

  /**
   * Devuelve estado tarea.
   * 
   * @return estado tarea
   */
  @Enumerated(EnumType.STRING)
  public EstadoTarea getEstadoTarea() {
    return estadoTarea;
  }

  /**
   * Asigna estado tarea.
   * 
   * @param estadoTarea
   *          estado tarea
   */
  public void setEstadoTarea(EstadoTarea estadoTarea) {
    this.estadoTarea = estadoTarea;
  }

  /**
   * Devuelve mensaje.
   * 
   * @return mensaje
   */
  @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getMensaje() {
    return mensaje;
  }

  /**
   * Asigna mensaje.
   * 
   * @param mensaje
   *          mensaje
   */
  public void setMensaje(String mensaje) {
    this.mensaje = mensaje;
  }

}
