package es.csa.fdt.cyldigital.tareas.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.Date;

import javax.annotation.Resource;

import org.quartz.JobDetail;
import org.quartz.Scheduler;
import org.quartz.SchedulerException;
import org.quartz.SimpleTrigger;
import org.springframework.scheduling.quartz.SchedulerFactoryBean;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;


/**
 * Clase EjecucionTareaServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class EjecucionTareaServiceImpl extends AbstractServiceImpl<Parameter, Long> implements EjecucionTareaService {

  private static final long serialVersionUID = -5575228180257917674L;

  private String nombreTarea = "";
  private String nombreClase = "";
  private String resultado = "No se ha ejecutado ninguna tarea";

  /**
   * Construye un nuevo objeto de la clase
   */
  public EjecucionTareaServiceImpl() {
    super();
  }

  /**
   * Devuelve nombre tarea.
   * 
   * @return nombre tarea
   */
  public String getNombreTarea() {
    return nombreTarea;
  }

  /**
   * Asigna nombre tarea.
   * 
   * @param nombreTarea
   *          nombre tarea
   */
  public void setNombreTarea(String nombreTarea) {
    this.nombreTarea = nombreTarea;
  }

  /**
   * Devuelve resultado.
   * 
   * @return resultado
   */
  public String getResultado() {
    return resultado;
  }

  /**
   * Asigna resultado.
   * 
   * @param resultado
   *          resultado
   */
  public void setResultado(String resultado) {
    this.resultado = resultado;
  }

  /**
   * Devuelve nombre clase.
   * 
   * @return nombre clase
   */
  public String getNombreClase() {
    return nombreClase;
  }

  /**
   * Asigna nombre clase.
   * 
   * @param nombreClase
   *          nombre clase
   */
  public void setNombreClase(String nombreClase) {
    this.nombreClase = nombreClase;
  }

  /**
   * Ejecutar.
   * 
   * @param scheduler
   *           scheduler
   */
  public void ejecutar(org.quartz.impl.StdScheduler scheduler) {
    resultado = "Comienzo de ejecución <br />";
    JobDetail jobDetail;
    try {
      jobDetail = new JobDetail("Tarea manual:" + nombreTarea, "Grupo de tareas: " + nombreTarea, Class
          .forName(nombreClase));
      SimpleTrigger trigger = new SimpleTrigger("mySimpleTrigger", "Grupo de tareas: " + nombreTarea, new Date(), null,
          0, 0L);
      try {
        scheduler.scheduleJob(jobDetail, trigger);
        try {
          scheduler.start();
          resultado += "Trabajo planificado <br />";
        } catch (SchedulerException e) {
          e.printStackTrace();
          resultado += "Error al iniciar el scheduler <br />";
        }
      } catch (SchedulerException e) {
        e.printStackTrace();
        resultado += "Error al planificar el trabajo<br />";
      }
    } catch (ClassNotFoundException e) {
      e.printStackTrace();
      resultado += "Error al intentar crear la clase, class not found exception <br />";
    }

  }

}
