package es.csa.fdt.cyldigital.tareas.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.commons.persistence.browser.jpaql.FilterJPAQLByQuery;
import es.csa.fdt.cyldigital.core.service.BrowserFilteredByActive;
import es.csa.fdt.cyldigital.tareas.entity.InformeTarea;
import es.csa.fdt.cyldigital.tareas.entity.InformeTarea.EstadoTarea;

/**
 * Servicio del Browser de Informes de Tareas.
 * 
 * @author csa
 */
@Service
@Repository
public class InformeTareasBrowserImpl extends BrowserFilteredByActive<InformeTarea> implements
    InformeTareasBrowser<CriterionJPAQL, String> {

  private static final long serialVersionUID = -7227366052245717441L;

  private EstadoTarea estadoTarea;

  /**
   * Construye un nuevo objeto de la clase
   */
  public InformeTareasBrowserImpl() {
    super();
    getSort().setSortAsc(false);
    getSort().setSortProperty("id");
  }

  /**
   * Devuelve estado tarea.
   * 
   * @return estado tarea
   */
  public EstadoTarea getEstadoTarea() {
    return estadoTarea;
  }

  /**
   * Asigna estado tarea.
   * 
   * @param estadoTarea
   *          estado tarea
   */
  public void setEstadoTarea(EstadoTarea estadoTarea) {
    this.estadoTarea = estadoTarea;
  }

  /**
   * Filter estado.
   */
  public void filterEstado() {
    removeFilterEstado();
    if (getEstadoTarea() != null) filterByEstado(getEstadoTarea());
  }

  private void removeFilterEstado() {
    int i = -1;
    for (Filter<CriterionJPAQL, InformeTarea> f : getFilters())
      if (f.getCriterion().getQuery().contains("estadoTarea")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
  }

  /**
   * Filter by estado.
   * 
   * @param estado
   *           estado
   */
  public void filterByEstado(EstadoTarea estado) {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<InformeTarea> filter = new FilterJPAQLByQuery<InformeTarea>();

    query += " estadoTarea = ? ";
    filter.getCriterion().addParameter(estado);

    for (Filter<CriterionJPAQL, InformeTarea> f : getFilters())
      if (f.getCriterion().getQuery().contains("estadoTarea")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }
}
