package es.csa.fdt.cyldigital.usuarios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.ManyToMany;
import javax.persistence.Table;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotEmpty;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;

/**
 * Clase CodigoPostal.
 * 
 * @author csa
 */
@Entity
@Table(name = "codigospostales")
@AttributeOverride(name = "id", column = @Column(name = "idCodigoPostal"))
@Audited
public class CodigoPostal extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = -7648562176655166537L;

  private String codigo;

  private List<Localidad> localidades;

  /**
   * Construye un nuevo objeto de la clase
   */
  public CodigoPostal() {
    super();
  }

  /**
   * Devuelve codigo.
   * 
   * @return codigo
   */
  @Length(max = 5)
  @NotEmpty
  public String getCodigo() {
    return codigo;
  }

  /**
   * Asigna codigo.
   * 
   * @param nombre
   *          codigo
   */
  public void setCodigo(String nombre) {
    this.codigo = nombre;
  }

  /**
   * Devuelve localidad.
   * 
   * @return localidad
   */
  @ManyToMany(cascade = { CascadeType.ALL }, mappedBy = "codigosPostales", targetEntity = Localidad.class)
  public List<Localidad> getLocalidad() {
    return localidades;
  }

  /**
   * Asigna localidad.
   * 
   * @param localidades
   *          localidad
   */
  public void setLocalidad(List<Localidad> localidades) {
    this.localidades = localidades;
  }

}
