package es.csa.fdt.cyldigital.usuarios.entity;

/*
 * #%L
 * Librerias del nucleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.apache.commons.lang.StringUtils;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotNull;
import org.hibernate.validator.Pattern;

import es.csa.fdt.cyldigital.calendario.entity.AusenciaCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.ReservaCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.VacacionesCalendarItem;
import es.csa.fdt.cyldigital.incidencias.entity.CategoriaIncidencia;
import es.csa.fdt.cyldigital.notificacion.entity.Alarma;
import es.csa.fdt.cyldigital.notificacion.entity.InformeNotification;
import es.csa.fdt.cyldigital.servicios.entity.ActividadExterna;
import es.csa.fdt.cyldigital.servicios.entity.Asesoramiento;
import es.csa.fdt.cyldigital.servicios.entity.Charla;
import es.csa.fdt.cyldigital.servicios.entity.Curso;
import es.csa.fdt.cyldigital.servicios.entity.CursoOnline;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCharla;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso;
import es.csa.fdt.cyldigital.servicios.entity.Taller;

/**
 * Clase Empleado.
 * 
 * @author csa
 */
@Entity
@Table(name = "empleados")
@Audited
@AttributeOverride(name = "id", column = @Column(name = "idEmpleado"))
public class Empleado extends Usuario implements Serializable {

  private static final long serialVersionUID = 5721862807183200675L;
  private static final String EMAIL_PATTERN = 
			"^$|[_A-Za-z0-9-\\+]+(\\.[_A-Za-z0-9-]+)*@"
			+ "[A-Za-z0-9-]+(\\.[A-Za-z0-9]+)*(\\.[A-Za-z]{2,})$";

  /**
   * Enum Puesto.
   */
  public static enum Puesto {
    INFORMER,
    TRAINER,
    MANAGER,
    AREA_MANAGER,
    TRAINER_COORDINATOR,
    GESTOR_012,
    ADMIN,
    EXTERNAL_PERSONAL,
    FINANCIAL_MANAGER,
    ONLINE_TRAINER,
    DIGITAL_ALLY_MANAGER,
  }

  /** cursos. */
  protected List<Curso> cursos;
  
  /** matriculascursos */
  protected List<MatriculaCurso> tramitesMatriculaCurso;
  
  /** talleres. */
  protected List<Taller> talleres;
  
  /** charlas. */
  protected List<Charla> charlas;
  
  /** externas */
  protected List<ActividadExterna> externas;
  
  /** matriculascharlas */
  protected List<MatriculaCharla> tramitesMatriculaCharla;
  
  /** asesoramientos. */
  protected List<Asesoramiento> asesoramientos;
  
  /** categoriasincidencia. */
  protected List<CategoriaIncidencia> categoriasincidencia;
  
  /** categoriasresponsable. */
  protected List<CategoriaIncidencia> categoriasresponsable;
  
  /** informes. */
  protected List<InformeNotification> informes;
  
  /** alarmas. */
  protected List<Alarma> alarmas;

  /** cursos online. */
  protected List<CursoOnline> cursosOnline;

  private boolean responsableIncidencias;
  private boolean profesor;
  private boolean accesibleFinanciera;
  private Puesto puesto;
  private String username;
  private String email;

  private Proveedor proveedor;

  private List<AusenciaCalendarItem> ausencias;

  private List<VacacionesCalendarItem> vacaciones;

  private List<ReservaCalendarItem> reservas;

  private String observaciones;

  private String idWeb;
  private String password;
  private String usernameWeb;
  private boolean cambioObligatorioPswdLDAP = false;
  private String historicoPswdLDAP;

  /**
   * Construye un nuevo objeto de la clase
   */
  public Empleado() {
    super();
    cursos = new ArrayList<Curso>();
    talleres = new ArrayList<Taller>();
    charlas = new ArrayList<Charla>();
    externas = new ArrayList<ActividadExterna>();
    asesoramientos = new ArrayList<Asesoramiento>();
    ausencias = new ArrayList<AusenciaCalendarItem>();
    vacaciones = new ArrayList<VacacionesCalendarItem>();
    reservas = new ArrayList<ReservaCalendarItem>();
    cursosOnline = new ArrayList<CursoOnline>();    
    
    setPassword("");
  }

  /**
   * Devuelve cursos.
   * 
   * @return cursos
   */
  @OneToMany(fetch = FetchType.LAZY, cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH }, mappedBy = "profesor")
  @OrderBy(value = "fechaInicio ASC")
  public List<Curso> getCursos() {
    return cursos;
  }

  /**
   * Asigna cursos.
   * 
   * @param cursos
   *          cursos
   */
  public void setCursos(List<Curso> cursos) {
    this.cursos = cursos;
  }

  /**
   * Devuelve tramitesMatriculaCurso.
   * 
   * @return tramitesMatriculaCurso
   */
  @OneToMany(fetch = FetchType.LAZY, cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH }, mappedBy = "empleadoTramite")
  @OrderBy(value = "fechaTramite ASC")
  public List<MatriculaCurso> getTramitesMatriculaCurso() {
    return tramitesMatriculaCurso;
  }

  /**
   * Asigna tramitesMatriculaCurso.
   * 
   * @param tramitesMatriculaCurso
   *          tramitesMatriculaCurso
   */
  public void setTramitesMatriculaCurso(List<MatriculaCurso> tramitesMatriculaCurso) {
    this.tramitesMatriculaCurso = tramitesMatriculaCurso;
  }

  /**
   * Devuelve talleres.
   * 
   * @return talleres
   */
  @OneToMany(fetch = FetchType.LAZY, cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH }, mappedBy = "profesor")
  @OrderBy(value = "fechaInicio ASC")
  public List<Taller> getTalleres() {
    return talleres;
  }

  /**
   * Asigna talleres.
   * 
   * @param talleres
   *          talleres
   */
  public void setTalleres(List<Taller> talleres) {
    this.talleres = talleres;
  }

  /**
   * Devuelve charlas.
   * 
   * @return charlas
   */
  @OneToMany(fetch = FetchType.LAZY, cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH }, mappedBy = "profesor")
  @OrderBy(value = "fechaInicio ASC")
  public List<Charla> getCharlas() {
    return charlas;
  }

  /**
   * Asigna charlas.
   * 
   * @param charlas
   *          charlas
   */
  public void setCharlas(List<Charla> charlas) {
    this.charlas = charlas;
  }
  
  /**
   * Devuelve externas.
   * 
   * @return externas
   */
  @OneToMany(fetch = FetchType.LAZY, cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH }, mappedBy = "colaborador")
  @OrderBy(value = "fechaInicio ASC")
  public List<ActividadExterna> getExternas() {
    return externas;
  }

  /**
   * Asigna charlas.
   * 
   * @param charlas
   *          charlas
   */
  public void setExternas(List<ActividadExterna> externas) {
    this.externas = externas;
  }
  
  /**
   * Devuelve tramitesMatriculaCurso.
   * 
   * @return tramitesMatriculaCurso
   */
  @OneToMany(fetch = FetchType.LAZY, cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH }, mappedBy = "empleadoTramite")
  @OrderBy(value = "fechaTramite ASC")
  public List<MatriculaCharla> getTramitesMatriculaCharla() {
    return tramitesMatriculaCharla;
  }

  /**
   * Asigna tramitesMatriculaCharla.
   * 
   * @param tramitesMatriculaCharla
   *          tramitesMatriculaCharla
   */
  public void setTramitesMatriculaCharla(List<MatriculaCharla> tramitesMatriculaCharla) {
    this.tramitesMatriculaCharla = tramitesMatriculaCharla;
  }

  /**
   * Devuelve asesoramientos.
   * 
   * @return asesoramientos
   */
  @OneToMany(fetch = FetchType.LAZY, cascade = { CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH }, mappedBy = "dinamizador")
  @OrderBy(value = "fecha ASC")
  public List<Asesoramiento> getAsesoramientos() {
    return asesoramientos;
  }

  /**
   * Asigna asesoramientos.
   * 
   * @param asesoramientos
   *          asesoramientos
   */
  public void setAsesoramientos(List<Asesoramiento> asesoramientos) {
    this.asesoramientos = asesoramientos;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.entity.Usuario#isUsuarioPre()
   */
  @Override
  @Transient
  public boolean isUsuarioPre() {
    return false;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.entity.Usuario#isUsuarioCyL()
   */
  @Override
  @Transient
  public boolean isUsuarioCyL() {
    return false;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.entity.Usuario#isEmpleado()
   */
  @Override
  @Transient
  public boolean isEmpleado() {
    return true;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.entity.Usuario#getDescripcion()
   */
  @Override
  @Transient
  public String getDescripcion() {
    String descripcion = "";
    if (!getNif().isEmpty()) descripcion += " (NIF: " + getNif() + ") ";
    if (!getTelefonos().trim().isEmpty()) descripcion += " Tlf: " + getTelefonos();
    if (getCentro() != null) descripcion += " (Centro: " + getCentro().getNombre() + ") ";
    return descripcion.trim();
  }

  /**
   * Devuelve puesto.
   * 
   * @return puesto
   */
  @Enumerated(value = EnumType.STRING)
  @NotNull
  public Puesto getPuesto() {
    return puesto;
  }

  /**
   * Asigna puesto.
   * 
   * @param puesto
   *          puesto
   */
  public void setPuesto(Puesto puesto) {
    this.puesto = puesto;
  }

  /**
   * Devuelve username.
   * 
   * @return username
   */
  @Length(max = 20)
  @Column(unique = true)
  public String getUsername() {
    return username;
  }

  /**
   * Asigna username.
   * 
   * @param username
   *          username
   */
  public void setUsername(String username) {
    this.username = username;
  }

  /**
   * Devuelve email.
   * 
   * @return email
   */
  @Length(max = 255)
  //@Email
  @Pattern(regex=EMAIL_PATTERN, message = "El email no tiene el formato correcto")
  public String getEmail() {
    return email;
  }

  /**
   * Asigna email.
   * 
   * @param email
   *          email
   */
  public void setEmail(String email) {
    this.email = email == null ? null : email.trim();
  }

  /**
   * Checks if is profesor.
   * 
   * @return true, if is profesor
   */
  public boolean isProfesor() {
    return profesor;
  }

  /**
   * Asigna profesor.
   * 
   * @param profesor
   *          profesor
   */
  public void setProfesor(boolean profesor) {
    this.profesor = profesor;
  }

  /**
   * Checks if is profesor online.
   * 
   * @return true, if is profesor online
   */
  @Transient
  public boolean isProfesorOnline() {
    return puesto == Puesto.ONLINE_TRAINER;
  }

  /**
   * Checks if is accesible financiera.
   * 
   * @return true, if is accesible financiera
   */
  public boolean isAccesibleFinanciera() {
    return accesibleFinanciera;
  }

  /**
   * Asigna accesible financiera.
   * 
   * @param accesibleFinanciera
   *          accesible financiera
   */
  public void setAccesibleFinanciera(boolean accesibleFinanciera) {
    this.accesibleFinanciera = accesibleFinanciera;
  }

  /**
   * Checks if is externo.
   * 
   * @return true, if is externo
   */
  @Transient
  public boolean isExterno() {
    return this.getPuesto() == Puesto.EXTERNAL_PERSONAL;
  }

  /**
   * Devuelve categorias incidencia.
   * 
   * @return categorias incidencia
   */
  @NotAudited
  @ManyToMany(fetch = FetchType.LAZY, mappedBy = "empleados", targetEntity = CategoriaIncidencia.class)
  public List<CategoriaIncidencia> getCategoriasIncidencia() {
    return categoriasincidencia;
  }

  /**
   * Asigna categorias incidencia.
   * 
   * @param categoriasincidencia
   *          categorias incidencia
   */
  public void setCategoriasIncidencia(List<CategoriaIncidencia> categoriasincidencia) {
    this.categoriasincidencia = categoriasincidencia;
  }

  /**
   * Checks if is responsable incidencias.
   * 
   * @return true, if is responsable incidencias
   */
  public boolean isResponsableIncidencias() {
    return responsableIncidencias;
  }

  /**
   * Asigna responsable incidencias.
   * 
   * @param responsableIncidencias
   *          responsable incidencias
   */
  public void setResponsableIncidencias(boolean responsableIncidencias) {
    this.responsableIncidencias = responsableIncidencias;
  }

  /**
   * Devuelve proveedor.
   * 
   * @return proveedor
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idProveedor")
  public Proveedor getProveedor() {
    return proveedor;
  }

  /**
   * Asigna proveedor.
   * 
   * @param proveedor
   *          proveedor
   */
  public void setProveedor(Proveedor proveedor) {
    this.proveedor = proveedor;
  }

  /**
   * Devuelve ausencias.
   * 
   * @return ausencias
   */
  @OneToMany(mappedBy = "empleado")
  public List<AusenciaCalendarItem> getAusencias() {
    return ausencias;
  }

  /**
   * Asigna ausencias.
   * 
   * @param ausencias
   *          ausencias
   */
  public void setAusencias(List<AusenciaCalendarItem> ausencias) {
    this.ausencias = ausencias;
  }

  /**
   * Devuelve vacaciones.
   * 
   * @return vacaciones
   */
  @OneToMany(mappedBy = "empleado")
  public List<VacacionesCalendarItem> getVacaciones() {
    return vacaciones;
  }

  /**
   * Asigna vacaciones.
   * 
   * @param vacaciones
   *          vacaciones
   */
  public void setVacaciones(List<VacacionesCalendarItem> vacaciones) {
    this.vacaciones = vacaciones;
  }

  /**
   * Devuelve reservas.
   * 
   * @return reservas
   */
  @OneToMany(mappedBy = "empleado")
  public List<ReservaCalendarItem> getReservas() {
    return reservas;
  }

  /**
   * Asigna reservas.
   * 
   * @param reservas
   *          reservas
   */
  public void setReservas(List<ReservaCalendarItem> reservas) {
    this.reservas = reservas;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.entity.Usuario#getRoles()
   */
  @Override
  @Transient
  protected String getRoles() {
    StringBuilder sb = new StringBuilder("");
    for (Role rol : this.getRolesList()) {
      if (sb.length() != 0) sb.append(", ");
      sb.append(rol.toString());
    }
    return sb.toString();
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.entity.Usuario#getRolesArray()
   */
  @Override
  @Transient
  public Role[] getRolesArray() {
    return this.getRolesList().toArray(new Role[0]);
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.entity.Usuario#getRolesList()
   */
  @Override
  @Transient
  public List<Role> getRolesList() {
    // Pido los roles del padre
    List<Role> roles = super.getRolesList();
    if (getPuesto() != null && !isOverrideDefaultRoles()) {
      roles = new ArrayList<Role>();
      switch (getPuesto()) {
        case ADMIN:
          if (!roles.contains(Role.ROLE_ADMIN)) roles.add(Role.ROLE_ADMIN);
          roles.addAll(getDefaultRolesFiltered(roles, Role.ROLE_ADMIN));
          break;
        case AREA_MANAGER:
          if (!roles.contains(Role.ROLE_AREA_MANAGER)) roles.add(Role.ROLE_AREA_MANAGER);
          roles.addAll(getDefaultRolesFiltered(roles, Role.ROLE_AREA_MANAGER));
          break;
        case EXTERNAL_PERSONAL:
          if (!roles.contains(Role.ROLE_EXTERN)) roles.add(Role.ROLE_EXTERN);
          roles.addAll(getDefaultRolesFiltered(roles, Role.ROLE_EXTERN));
          break;
        /*
        case GESTOR_012:
          if (!roles.contains(Role.ROLE_012)) roles.add(Role.ROLE_012);
          roles.addAll(getDefaultRolesFiltered(roles, Role.ROLE_012));
          break;
        case INFORMER:
          if (!roles.contains(Role.ROLE_INFORMER)) roles.add(Role.ROLE_INFORMER);
          roles.addAll(getDefaultRolesFiltered(roles, Role.ROLE_INFORMER));
          break;
        case MANAGER:
          if (!roles.contains(Role.ROLE_MANAGER)) roles.add(Role.ROLE_MANAGER);
          roles.addAll(getDefaultRolesFiltered(roles, Role.ROLE_MANAGER));
          break;
        */
        case TRAINER:
          if (!roles.contains(Role.ROLE_TRAINER)) roles.add(Role.ROLE_TRAINER);
          roles.addAll(getDefaultRolesFiltered(roles, Role.ROLE_TRAINER));
          break;
        /*
        case TRAINER_COORDINATOR:
          if (!roles.contains(Role.ROLE_TRAINER_COORDINATOR)) roles.add(Role.ROLE_TRAINER_COORDINATOR);
          roles.addAll(getDefaultRolesFiltered(roles, Role.ROLE_TRAINER_COORDINATOR));
          break;
        case FINANCIAL_MANAGER:
          if (!roles.contains(Role.ROLE_FINANCIAL_MANAGER)) roles.add(Role.ROLE_FINANCIAL_MANAGER);
          roles.addAll(getDefaultRolesFiltered(roles, Role.ROLE_FINANCIAL_MANAGER));
          break;
        case ONLINE_TRAINER:
          if (!roles.contains(Role.ROLE_ONLINE_TRAINER)) roles.add(Role.ROLE_ONLINE_TRAINER);
          roles.addAll(getDefaultRolesFiltered(roles, Role.ROLE_ONLINE_TRAINER));
          break;
        */
          
        case DIGITAL_ALLY_MANAGER:
        	if (!roles.contains(Role.ROLE_DIGITAL_ALLY)) roles.add(Role.ROLE_DIGITAL_ALLY);
            roles.addAll(getDefaultRolesFiltered(roles, Role.ROLE_DIGITAL_ALLY));
            break;
        default:
          break;
      }
    }
    
    /*
     * TODO: Comprobar más tarde, quitado para Xunta de Galicia
     * 
     * if (isResponsableIncidencias() && !roles.contains(Role.ROLE_TRAINER_COORDINATOR))
     * roles.add(Role.ROLE_INCIDENCE_FUNDETEC);
     *
     * if (isAccesibleFinanciera()) roles.add(Role.ROLE_FINANCIAL_VIEW);
     * */
    
    return roles;
  }

  private List<Role> getDefaultRolesFiltered(List<Role> currentRoles, Role rolePrincipal) {
    List<Role> rolesAdd = new ArrayList<Role>();
    for (Role role : getDefaultRoles(rolePrincipal)) {
      if (!currentRoles.contains(role)) rolesAdd.add(role);
    }
    return rolesAdd;
  }

  private List<Role> getDefaultRoles(Role role) {
    List<Role> perms = new ArrayList<Role>();
    switch (role) {
      case ROLE_ADMIN:
        /** Centros **/
        perms.add(Role.ROLE_CENTER_VIEW_OWN);
        perms.add(Role.ROLE_CENTER_VIEW_ALL);
        perms.add(Role.ROLE_CENTER_EDIT_OWN);
        perms.add(Role.ROLE_CENTER_EDIT_ALL);
        perms.add(Role.ROLE_CENTER_DELETE);
        perms.add(Role.ROLE_CENTER_CREATE);
        perms.add(Role.ROLE_CENTER_AULA_VIEW);
        perms.add(Role.ROLE_CENTER_AULA_EDIT);
        perms.add(Role.ROLE_CENTER_EDIT_URLDAP);
        perms.add(Role.ROLE_CENTER_FESTIVOS_VIEW);
        perms.add(Role.ROLE_CENTER_FESTIVOS_EDIT);
        perms.add(Role.ROLE_CENTER_FESTIVOS_DELETE);
        perms.add(Role.ROLE_CENTER_FESTIVOS_CREATE);
        perms.add(Role.ROLE_CENTER_CALENDARIO_VIEW);
        
        /** Usuarios **/
        perms.add(Role.ROLE_USER_VIEW);
        perms.add(Role.ROLE_USER_EDIT);
        perms.add(Role.ROLE_USER_VIEW_ALL_CENTER);
        perms.add(Role.ROLE_USER_PRE_DELETE);
        perms.add(Role.ROLE_USER_DELETE);
        perms.add(Role.ROLE_USER_VIEW_EVALUATIONS);
        perms.add(Role.ROLE_USER_ONLINE);
        
        /** Servicios **/
        perms.add(Role.ROLE_SERVICE_VIEW);
        perms.add(Role.ROLE_SERVICE_EDIT);
        perms.add(Role.ROLE_SERVICE_DELETE);
        perms.add(Role.ROLE_SERVICE_EDIT_REGISTER_USER_OWN);
        perms.add(Role.ROLE_SERVICE_EDIT_REGISTER_USER_ALL);
        perms.add(Role.ROLE_SERVICE_PROPUESTAS_EDIT);
        perms.add(Role.ROLE_SERVICE_ONLINE);
        
        /** Actividades **/
        perms.add(Role.ROLE_ACTIVITY_VIEW);
        perms.add(Role.ROLE_ACTIVITY_EDIT);
        perms.add(Role.ROLE_ACTIVITY_INTERNAL_APPROVE);
        perms.add(Role.ROLE_ACTIVITY_EXTERNAL_APPROVE);
        perms.add(Role.ROLE_ACTIVITY_DELETE);
        perms.add(Role.ROLE_ACTIVITY_REGISTER_USER);
        perms.add(Role.ROLE_ACTIVITY_TRAINER_VIEW_ALL);
        perms.add(Role.ROLE_ACTIVITY_TRAINER_VIEW_OWN);
        perms.add(Role.ROLE_ACTIVITY_ANNOTATION_ABOUT_SERVICE);
        perms.add(Role.ROLE_ACTIVITY_ONLINE);
        
        /** Diplomas **/
        perms.add(Role.ROLE_DIPLOMA_VIEW);
        perms.add(Role.ROLE_DIPLOMA_EDIT);
        perms.add(Role.ROLE_DIPLOMA_DELETE);
        
        /** Exámenes **/
        perms.add(Role.ROLE_EXAM_VIEW);
        perms.add(Role.ROLE_EXAM_EDIT);
        perms.add(Role.ROLE_EXAM_DELETE);
        
        /** Personal **/
        perms.add(Role.ROLE_PERSONAL_VIEW_OWN);
        perms.add(Role.ROLE_PERSONAL_VIEW_ALL);
        perms.add(Role.ROLE_PERSONAL_EDIT);
        perms.add(Role.ROLE_PERSONAL_DELETE);
        perms.add(Role.ROLE_PERSONAL_TYPE);
        perms.add(Role.ROLE_PERSONAL_PERMISSION);
        perms.add(Role.ROLE_PERSONAL_CHANGE_PASSWORD);
        perms.add(Role.ROLE_PERSONAL_EDIT_VACACIONES);
        perms.add(Role.ROLE_PERSONAL_EDIT_RESPONSABLEINCIDENCIAS);
        
        /** Proveedores **/
        perms.add(Role.ROLE_SUPPLIER_CREATE);
        perms.add(Role.ROLE_SUPPLIER_VIEW);
        perms.add(Role.ROLE_SUPPLIER_EDIT);
        perms.add(Role.ROLE_SUPPLIER_DELETE);
        
        /** Evaluaciones **/
        perms.add(Role.ROLE_EVALUATION_NEW);
        perms.add(Role.ROLE_EVALUATION_VIEW);
        perms.add(Role.ROLE_EVALUATION_EDIT);
        
        /** Recursos **/
        perms.add(Role.ROLE_RECURSO_NEW);
        perms.add(Role.ROLE_RECURSO_ORDENADOR_ON_OFF);
        perms.add(Role.ROLE_RECURSO_VIEW);
        perms.add(Role.ROLE_RECURSO_EDIT_ORDENADOR);
        perms.add(Role.ROLE_RECURSO_EDIT_ROUTER);
        perms.add(Role.ROLE_RECURSO_EDIT_RECURSO);
        perms.add(Role.ROLE_RECURSO_VIEW_CATEGORIA);
        perms.add(Role.ROLE_RECURSO_EDIT_CATEGORIA);
        perms.add(Role.ROLE_RECURSO_SEND_USER_COMMAND);
        perms.add(Role.ROLE_RECURSO_SEND_ROOT_COMMAND);
        perms.add(Role.ROLE_RECURSO_VIEW_RESERVA);
        perms.add(Role.ROLE_RECURSO_EDIT_RESERVA);
        perms.add(Role.ROLE_RECURSO_VIEW_AVERIAS);
        perms.add(Role.ROLE_RECURSO_EDIT_AVERIAS);
        perms.add(Role.ROLE_RECURSO_VIEW_PEDIDOS);
        perms.add(Role.ROLE_RECURSO_EDIT_PEDIDOS);
        
        /** Incidencias **/
        // perms.add(Role.ROLE_INCIDENCE_VIEW_PAGE);
        // perms.add(Role.ROLE_INCIDENCE_CREATE);
        // perms.add(Role.ROLE_INCIDENCE_CATEGORIES_TAB);
        // perms.add(Role.ROLE_INCIDENCE_CATEGORIES_CREATE);
        // perms.add(Role.ROLE_INCIDENCE_CATEGORIES_EDIT);
        // perms.add(Role.ROLE_INCIDENCE_REPORTS_TAB);
        // perms.add(Role.ROLE_INCIDENCE_SELECT_ALL_CATEGORIES);
        // perms.add(Role.ROLE_INCIDENCE_SELECT_OWN_CATEGORIES);
        // perms.add(Role.ROLE_INCIDENCE_VIEW);
        // perms.add(Role.ROLE_INCIDENCE_VIEW_ALL);
        // perms.add(Role.ROLE_INCIDENCE_RECEIVE_MAIL);
        // perms.add(Role.ROLE_INCIDENCE_FUNDETEC);
        
        /** Parámetros de configuración **/
        perms.add(Role.ROLE_PARAMETERS_VIEW);
        
        /** Financiero **/
        //perms.add(Role.ROLE_FINANCIAL_VIEW);
        //perms.add(Role.ROLE_FINANCIAL_ADMIN);
        
        /** Informes automaticos **/
        perms.add(Role.ROLE_SEND_AUTOMATIC_FORMS_VIEW);
        perms.add(Role.ROLE_SEND_AUTOMATIC_FORMS_EDIT);
        
        /** Cuadro de mandos **/
        perms.add(Role.ROLE_CUADRO_VIEW);
        // perms.add(Role.ROLE_CUADRO_CENTRO_OWN);
        perms.add(Role.ROLE_CUADRO_EDIT);
        perms.add(Role.ROLE_ALARM_EDIT);
        
        /** WIKI **/
        // perms.add(Role.ROLE_WIKI_BURGOS);
        // perms.add(Role.ROLE_WIKI_PALENCIA);
        // perms.add(Role.ROLE_WIKI_LEON);
        // perms.add(Role.ROLE_WIKI_ZAMORA);
        // perms.add(Role.ROLE_WIKI_VALLADOLID);
        // perms.add(Role.ROLE_WIKI_SORIA);
        // perms.add(Role.ROLE_WIKI_SEGOVIA);
        // perms.add(Role.ROLE_WIKI_SALAMANCA);
        // perms.add(Role.ROLE_WIKI_AVILA);
        // perms.add(Role.ROLE_WIKI_GENERAL);
        // perms.add(Role.QUALITE);
        // perms.add(Role.ADMIN);*/
        
        /** Contratos **/
        // perms.add(Role.ROLE_CONTRACT_CREATE);
        // perms.add(Role.ROLE_CONTRACT_VIEW);
        // perms.add(Role.ROLE_CONTRACT_EDIT);
        // perms.add(Role.ROLE_CONTRACT_DELETE);
        
        /** Paneles **/
        // perms.add(Role.ROLE_PANEL_VIEW);
        // perms.add(Role.ROLE_PANEL_CREATE_PANEL);
        // perms.add(Role.ROLE_PANEL_CREATE_NEWS);
        // perms.add(Role.ROLE_PANEL_CREATE_EVENTS);
        // perms.add(Role.ROLE_PANEL_VIEW_ALLPANEL);
        // perms.add(Role.ROLE_PANEL_EDIT_ALLPANEL);
        // perms.add(Role.ROLE_PANEL_UPLOAD_FILES);
        
        /** Notificaciones **/
        // perms.add(Role.ROLE_NOTIFICATIONS_VIEW);
        
        break;

      case ROLE_AREA_MANAGER:
        /** Centros **/    	  
        perms.add(Role.ROLE_CENTER_VIEW_OWN);
        // perms.add(Role.ROLE_CENTER_VIEW_ALL);
        // perms.add(Role.ROLE_CENTER_EDIT_ALL);
        perms.add(Role.ROLE_CENTER_EDIT_OWN);
        // perms.add(Role.ROLE_CENTER_DELETE);
        // perms.add(Role.ROLE_CENTER_CREATE);
        perms.add(Role.ROLE_CENTER_AULA_VIEW);
        perms.add(Role.ROLE_CENTER_AULA_EDIT);
        // perms.add(Role.ROLE_CENTER_EDIT_URLDAP);
        perms.add(Role.ROLE_CENTER_FESTIVOS_VIEW);
        perms.add(Role.ROLE_CENTER_FESTIVOS_EDIT);
        perms.add(Role.ROLE_CENTER_FESTIVOS_DELETE);
        perms.add(Role.ROLE_CENTER_FESTIVOS_CREATE);
        perms.add(Role.ROLE_CENTER_CALENDARIO_VIEW);
        
        /** Usuarios **/
        perms.add(Role.ROLE_USER_VIEW);
        // perms.add(Role.ROLE_USER_VIEW_ALL_CENTER);
        perms.add(Role.ROLE_USER_PRE_DELETE);
        perms.add(Role.ROLE_USER_EDIT);
        perms.add(Role.ROLE_USER_DELETE);
        perms.add(Role.ROLE_USER_VIEW_EVALUATIONS);
        // perms.add(Role.ROLE_USER_ONLINE);
        
        /** Servicios **/
        perms.add(Role.ROLE_SERVICE_VIEW);
        // perms.add(Role.ROLE_SERVICE_EDIT);
        // perms.add(Role.ROLE_SERVICE_DELETE);
        perms.add(Role.ROLE_SERVICE_EDIT_REGISTER_USER_OWN);
        // perms.add(Role.ROLE_SERVICE_EDIT_REGISTER_USER_ALL);
        perms.add(Role.ROLE_SERVICE_PROPUESTAS_EDIT);
        // perms.add(Role.ROLE_SERVICE_ONLINE);
        
        /** Actividades **/
        perms.add(Role.ROLE_ACTIVITY_VIEW);
        perms.add(Role.ROLE_ACTIVITY_EDIT);
        perms.add(Role.ROLE_ACTIVITY_INTERNAL_REQUEST);
        perms.add(Role.ROLE_ACTIVITY_EXTERNAL_APPROVE);
        perms.add(Role.ROLE_ACTIVITY_DELETE);
        perms.add(Role.ROLE_ACTIVITY_REGISTER_USER);
        perms.add(Role.ROLE_ACTIVITY_TRAINER_VIEW_ALL);
        perms.add(Role.ROLE_ACTIVITY_TRAINER_VIEW_OWN);
        perms.add(Role.ROLE_ACTIVITY_ANNOTATION_ABOUT_SERVICE);
        // perms.add(Role.ROLE_ACTIVITY_ONLINE);
        
        /** Diplomas **/
        perms.add(Role.ROLE_DIPLOMA_VIEW);
        // perms.add(Role.ROLE_DIPLOMA_EDIT);
        // perms.add(Role.ROLE_DIPLOMA_DELETE);
        
        /** Exámenes **/
        perms.add(Role.ROLE_EXAM_VIEW);
        // perms.add(Role.ROLE_EXAM_EDIT);
        // perms.add(Role.ROLE_EXAM_DELETE);
        
        /** Personal **/
        perms.add(Role.ROLE_PERSONAL_VIEW_OWN);
        //perms.add(Role.ROLE_PERSONAL_VIEW_ALL);
        //perms.add(Role.ROLE_PERSONAL_EDIT);
        perms.add(Role.ROLE_PERSONAL_DELETE);
        perms.add(Role.ROLE_PERSONAL_TYPE);
        // perms.add(Role.ROLE_PERSONAL_PERMISSION);
        perms.add(Role.ROLE_PERSONAL_CHANGE_PASSWORD);
        perms.add(Role.ROLE_PERSONAL_EDIT_VACACIONES);
        // perms.add(Role.ROLE_PERSONAL_EDIT_RESPONSABLEINCIDENCIAS);
        
        /** Proveedores **/
        // perms.add(Role.ROLE_SUPPLIER_CREATE);
        perms.add(Role.ROLE_SUPPLIER_VIEW);
        // perms.add(Role.ROLE_SUPPLIER_EDIT);
        // perms.add(Role.ROLE_SUPPLIER_DELETE);
        
        /** Evaluaciones **/
        perms.add(Role.ROLE_EVALUATION_NEW);
        perms.add(Role.ROLE_EVALUATION_VIEW);
        perms.add(Role.ROLE_EVALUATION_EDIT);
        
        /** Recursos **/
        perms.add(Role.ROLE_RECURSO_NEW);
        perms.add(Role.ROLE_RECURSO_ORDENADOR_ON_OFF);
        perms.add(Role.ROLE_RECURSO_VIEW);
        perms.add(Role.ROLE_RECURSO_EDIT_ORDENADOR);
        perms.add(Role.ROLE_RECURSO_EDIT_ROUTER);
        perms.add(Role.ROLE_RECURSO_EDIT_RECURSO);
        perms.add(Role.ROLE_RECURSO_VIEW_CATEGORIA);
        perms.add(Role.ROLE_RECURSO_EDIT_CATEGORIA);
        // perms.add(Role.ROLE_RECURSO_SEND_USER_COMMAND);
        // perms.add(Role.ROLE_RECURSO_SEND_ROOT_COMMAND);
        perms.add(Role.ROLE_RECURSO_VIEW_RESERVA);
        perms.add(Role.ROLE_RECURSO_EDIT_RESERVA);
        perms.add(Role.ROLE_RECURSO_VIEW_AVERIAS);
        perms.add(Role.ROLE_RECURSO_EDIT_AVERIAS);
        perms.add(Role.ROLE_RECURSO_VIEW_PEDIDOS);
        perms.add(Role.ROLE_RECURSO_EDIT_PEDIDOS);
        
        /** Incidencias **/
        // perms.add(Role.ROLE_INCIDENCE_VIEW_PAGE);
        // perms.add(Role.ROLE_INCIDENCE_CREATE);
        // perms.add(Role.ROLE_INCIDENCE_CATEGORIES_TAB);
        // perms.add(Role.ROLE_INCIDENCE_CATEGORIES_CREATE);
        // perms.add(Role.ROLE_INCIDENCE_CATEGORIES_EDIT);
        // perms.add(Role.ROLE_INCIDENCE_REPORTS_TAB);
        // perms.add(Role.ROLE_INCIDENCE_SELECT_ALL_CATEGORIES);
        // perms.add(Role.ROLE_INCIDENCE_SELECT_OWN_CATEGORIES);
        // perms.add(Role.ROLE_INCIDENCE_VIEW);
        // perms.add(Role.ROLE_INCIDENCE_VIEW_ALL);
        // perms.add(Role.ROLE_INCIDENCE_FUNDETEC);
        // perms.add(Role.ROLE_INCIDENCE_RECEIVE_MAIL);
        
        /** Parámetros de configuración **/
        // perms.add(Role.ROLE_PARAMETERS_VIEW);
        
        /** Financiero **/
        // perms.add(Role.ROLE_FINANCIAL_VIEW);
        // perms.add(Role.ROLE_FINANCIAL_ADMIN);
        
        /** Informes automaticos **/
        perms.add(Role.ROLE_SEND_AUTOMATIC_FORMS_VIEW);
        perms.add(Role.ROLE_SEND_AUTOMATIC_FORMS_EDIT);
        
        /** Cuadro de mandos **/
        perms.add(Role.ROLE_CUADRO_CENTRO_OWN);
        perms.add(Role.ROLE_CUADRO_VIEW);
        // perms.add(Role.ROLE_CUADRO_EDIT);
        perms.add(Role.ROLE_ALARM_EDIT);
        
        /** WIKI **/
        // perms.add(Role.ROLE_WIKI_BURGOS);
        // perms.add(Role.ROLE_WIKI_PALENCIA);
        // perms.add(Role.ROLE_WIKI_LEON);
        // perms.add(Role.ROLE_WIKI_ZAMORA);
        // perms.add(Role.ROLE_WIKI_VALLADOLID);
        // perms.add(Role.ROLE_WIKI_SORIA);
        // perms.add(Role.ROLE_WIKI_SEGOVIA);
        // perms.add(Role.ROLE_WIKI_SALAMANCA);
        // perms.add(Role.ROLE_WIKI_AVILA);
        // perms.add(Role.ROLE_WIKI_GENERAL);
        
        // perms.add(Role.QUALITE);
        // perms.add(Role.ADMIN);
        
        /** Contratos **/
        // perms.add(Role.ROLE_CONTRACT_CREATE);
        // perms.add(Role.ROLE_CONTRACT_VIEW);
        // perms.add(Role.ROLE_CONTRACT_EDIT);
        // perms.add(Role.ROLE_CONTRACT_DELETE);
        
        /** Paneles **/
        // perms.add(Role.ROLE_PANEL_VIEW);
        // perms.add(Role.ROLE_PANEL_CREATE_PANEL);
        // perms.add(Role.ROLE_PANEL_CREATE_NEWS);
        // perms.add(Role.ROLE_PANEL_CREATE_EVENTS);
        
        // perms.add(Role.ROLE_PANEL_VIEW_ALLPANEL);
        // perms.add(Role.ROLE_PANEL_EDIT_ALLPANEL);
        // perms.add(Role.ROLE_PANEL_UPLOAD_FILES);

        /** Notificaciones **/
        // perms.add(Role.ROLE_NOTIFICATIONS_VIEW);
        break;

      case ROLE_EXTERN:
        /** Centros **/
        perms.add(Role.ROLE_CENTER_VIEW_OWN);
        perms.add(Role.ROLE_CENTER_VIEW_ALL);
        // perms.add(Role.ROLE_CENTER_EDIT_ALL);
        // perms.add(Role.ROLE_CENTER_EDIT_OWN);
        // perms.add(Role.ROLE_CENTER_DELETE);
        // perms.add(Role.ROLE_CENTER_CREATE);
        // perms.add(Role.ROLE_CENTER_AULA_VIEW);
        // perms.add(Role.ROLE_CENTER_AULA_EDIT);
        // perms.add(Role.ROLE_CENTER_EDIT_URLDAP);
        // perms.add(Role.ROLE_CENTER_FESTIVOS_VIEW);
        // perms.add(Role.ROLE_CENTER_FESTIVOS_EDIT);
        // perms.add(Role.ROLE_CENTER_FESTIVOS_DELETE);
        // perms.add(Role.ROLE_CENTER_FESTIVOS_CREATE);
        perms.add(Role.ROLE_CENTER_CALENDARIO_VIEW);
        
        /** Usuarios **/
        // perms.add(Role.ROLE_USER_VIEW);
        // perms.add(Role.ROLE_USER_VIEW_ALL_CENTER);
        // perms.add(Role.ROLE_USER_EDIT);
        // perms.add(Role.ROLE_USER_PRE_DELETE);
        // perms.add(Role.ROLE_USER_DELETE);
        // perms.add(Role.ROLE_USER_VIEW_EVALUATIONS);
        // perms.add(Role.ROLE_USER_ONLINE);
        
        /** Servicios **/
        perms.add(Role.ROLE_SERVICE_VIEW);
        // perms.add(Role.ROLE_SERVICE_EDIT);
        // perms.add(Role.ROLE_SERVICE_DELETE);
        // perms.add(Role.ROLE_SERVICE_EDIT_REGISTER_USER_OWN);
        // perms.add(Role.ROLE_SERVICE_EDIT_REGISTER_USER_ALL);
        // perms.add(Role.ROLE_SERVICE_PROPUESTAS_EDIT);
        // perms.add(Role.ROLE_SERVICE_ONLINE);
        
        /** Actividades **/
        perms.add(Role.ROLE_ACTIVITY_VIEW);
        //perms.add(Role.ROLE_ACTIVITY_EDIT);
        perms.add(Role.ROLE_ACTIVITY_EXTERNAL_REQUEST);
        perms.add(Role.ROLE_ACTIVITY_DELETE);
        // perms.add(Role.ROLE_ACTIVITY_REGISTER_USER);
        perms.add(Role.ROLE_ACTIVITY_TRAINER_VIEW_ALL);
        perms.add(Role.ROLE_ACTIVITY_TRAINER_VIEW_OWN);
        perms.add(Role.ROLE_ACTIVITY_ANNOTATION_ABOUT_SERVICE);
        // perms.add(Role.ROLE_ACTIVITY_ONLINE);
        
        /** Diplomas **/
        // perms.add(Role.ROLE_DIPLOMA_VIEW);
        // perms.add(Role.ROLE_DIPLOMA_EDIT);
        // perms.add(Role.ROLE_DIPLOMA_DELETE);
        
        /** Exámenes **/
        // perms.add(Role.ROLE_EXAM_VIEW);
        // perms.add(Role.ROLE_EXAM_EDIT);
        // perms.add(Role.ROLE_EXAM_DELETE);
        
        /** Personal **/
        perms.add(Role.ROLE_PERSONAL_VIEW_OWN);
        // perms.add(Role.ROLE_PERSONAL_VIEW_ALL);
        // perms.add(Role.ROLE_PERSONAL_EDIT);
        // perms.add(Role.ROLE_PERSONAL_DELETE);
        // perms.add(Role.ROLE_PERSONAL_TYPE);
        // perms.add(Role.ROLE_PERSONAL_PERMISSION);
        perms.add(Role.ROLE_PERSONAL_CHANGE_PASSWORD);
        // perms.add(Role.ROLE_PERSONAL_EDIT_VACACIONES);
        // perms.add(Role.ROLE_PERSONAL_EDIT_RESPONSABLEINCIDENCIAS);
        
        /** Proveedores **/
        // perms.add(Role.ROLE_SUPPLIER_CREATE);
        // perms.add(Role.ROLE_SUPPLIER_VIEW);
        // perms.add(Role.ROLE_SUPPLIER_EDIT);
        // perms.add(Role.ROLE_SUPPLIER_DELETE);
        
        /** Evaluaciones **/
        // perms.add(Role.ROLE_EVALUATION_NEW);
        // perms.add(Role.ROLE_EVALUATION_VIEW);
        // perms.add(Role.ROLE_EVALUATION_EDIT);
        
        /** Recursos **/
        // perms.add(Role.ROLE_RECURSO_NEW);
        // perms.add(Role.ROLE_RECURSO_ORDENADOR_ON_OFF);
        // perms.add(Role.ROLE_RECURSO_VIEW);
        // perms.add(Role.ROLE_RECURSO_EDIT_ORDENADOR);
        // perms.add(Role.ROLE_RECURSO_EDIT_ROUTER);
        // perms.add(Role.ROLE_RECURSO_EDIT_RECURSO);
        // perms.add(Role.ROLE_RECURSO_VIEW_CATEGORIA);
        // perms.add(Role.ROLE_RECURSO_EDIT_CATEGORIA);
        // perms.add(Role.ROLE_RECURSO_SEND_USER_COMMAND);
        // perms.add(Role.ROLE_RECURSO_SEND_ROOT_COMMAND);
        // perms.add(Role.ROLE_RECURSO_VIEW_RESERVA);
        // perms.add(Role.ROLE_RECURSO_EDIT_RESERVA);
        // perms.add(Role.ROLE_RECURSO_VIEW_AVERIAS);
        // perms.add(Role.ROLE_RECURSO_EDIT_AVERIAS);
        // perms.add(Role.ROLE_RECURSO_VIEW_PEDIDOS);
        // perms.add(Role.ROLE_RECURSO_EDIT_PEDIDOS);
        
        /** Incidencias **/
        // perms.add(Role.ROLE_INCIDENCE_VIEW_PAGE);
        // perms.add(Role.ROLE_INCIDENCE_CREATE);
        // perms.add(Role.ROLE_INCIDENCE_CATEGORIES_TAB);
        // perms.add(Role.ROLE_INCIDENCE_CATEGORIES_CREATE);
        // perms.add(Role.ROLE_INCIDENCE_CATEGORIES_EDIT);
        // perms.add(Role.ROLE_INCIDENCE_REPORTS_TAB);
        // perms.add(Role.ROLE_INCIDENCE_SELECT_ALL_CATEGORIES);
        // perms.add(Role.ROLE_INCIDENCE_SELECT_OWN_CATEGORIES);
        // perms.add(Role.ROLE_INCIDENCE_VIEW);
        // perms.add(Role.ROLE_INCIDENCE_VIEW_ALL);
        // perms.add(Role.ROLE_INCIDENCE_FUNDETEC);
        // perms.add(Role.ROLE_INCIDENCE_RECEIVE_MAIL);

        /** Parámetros de configuración **/
        // perms.add(Role.ROLE_PARAMETERS_VIEW);
        
        /** Financiero **/
        // perms.add(Role.ROLE_FINANCIAL_VIEW);
        // perms.add(Role.ROLE_FINANCIAL_ADMIN);
        
        /** Informes automaticos **/
        // perms.add(Role.ROLE_SEND_AUTOMATIC_FORMS_VIEW);
        // perms.add(Role.ROLE_SEND_AUTOMATIC_FORMS_EDIT);
        
        /** Cuadro de mandos **/
        // perms.add(Role.ROLE_CUADRO_VIEW);
        // perms.add(Role.ROLE_CUADRO_EDIT);
        // perms.add(Role.ROLE_CUADRO_CENTRO_OWN);
        // perms.add(Role.ROLE_ALARM_EDIT);
        
        /** WIKI **/
        // perms.add(Role.ROLE_WIKI_BURGOS);
        // perms.add(Role.ROLE_WIKI_PALENCIA);
        // perms.add(Role.ROLE_WIKI_LEON);
        // perms.add(Role.ROLE_WIKI_ZAMORA);
        // perms.add(Role.ROLE_WIKI_VALLADOLID);
        // perms.add(Role.ROLE_WIKI_SORIA);
        // perms.add(Role.ROLE_WIKI_SEGOVIA);
        // perms.add(Role.ROLE_WIKI_SALAMANCA);
        // perms.add(Role.ROLE_WIKI_AVILA);
        // perms.add(Role.ROLE_WIKI_GENERAL);
        // perms.add(Role.QUALITE);
        // perms.add(Role.ADMIN);
        
        /** Contratos **/
        // perms.add(Role.ROLE_CONTRACT_CREATE);
        // perms.add(Role.ROLE_CONTRACT_VIEW);
        // perms.add(Role.ROLE_CONTRACT_EDIT);
        // perms.add(Role.ROLE_CONTRACT_DELETE);
        
        /** Paneles **/
        // perms.add(Role.ROLE_PANEL_VIEW);
        // perms.add(Role.ROLE_PANEL_CREATE_PANEL);
        // perms.add(Role.ROLE_PANEL_CREATE_NEWS);
        // perms.add(Role.ROLE_PANEL_CREATE_EVENTS);
        // perms.add(Role.ROLE_PANEL_VIEW_ALLPANEL);
        // perms.add(Role.ROLE_PANEL_EDIT_ALLPANEL);
        // perms.add(Role.ROLE_PANEL_UPLOAD_FILES);
        
        /** Notificaciones **/
        // perms.add(Role.ROLE_NOTIFICATIONS_VIEW);
        break;

      //case ROLE_012:
        /** Centros **/
        // perms.add(Role.ROLE_CENTER_VIEW_OWN);
        // perms.add(Role.ROLE_CENTER_VIEW_ALL);
        // perms.add(Role.ROLE_CENTER_EDIT_ALL);
        // perms.add(Role.ROLE_CENTER_EDIT_OWN);
        // perms.add(Role.ROLE_CENTER_DELETE);
        // perms.add(Role.ROLE_CENTER_CREATE);
        // perms.add(Role.ROLE_CENTER_AULA_VIEW);
        // perms.add(Role.ROLE_CENTER_AULA_EDIT);
        // perms.add(Role.ROLE_CENTER_EDIT_URLDAP);
        // perms.add(Role.ROLE_CENTER_FESTIVOS_VIEW);
        // perms.add(Role.ROLE_CENTER_FESTIVOS_EDIT);
        // perms.add(Role.ROLE_CENTER_FESTIVOS_DELETE);
        // perms.add(Role.ROLE_CENTER_FESTIVOS_CREATE);
        // perms.add(Role.ROLE_CENTER_CALENDARIO_VIEW);
    	  
        /** Usuarios **/
    	// perms.add(Role.ROLE_USER_VIEW);
    	// perms.add(Role.ROLE_USER_VIEW_ALL_CENTER);
    	// perms.add(Role.ROLE_USER_EDIT);
        // perms.add(Role.ROLE_USER_PRE_DELETE);
        // perms.add(Role.ROLE_USER_DELETE);
        // perms.add(Role.ROLE_USER_VIEW_EVALUATIONS);
        // perms.add(Role.ROLE_USER_ONLINE);
    	  
        /** Servicios **/
        // perms.add(Role.ROLE_SERVICE_VIEW);
        // perms.add(Role.ROLE_SERVICE_EDIT);
        // perms.add(Role.ROLE_SERVICE_DELETE);
        // perms.add(Role.ROLE_SERVICE_EDIT_REGISTER_USER_OWN);
        // perms.add(Role.ROLE_SERVICE_EDIT_REGISTER_USER_ALL);
        // perms.add(Role.ROLE_SERVICE_PROPUESTAS_EDIT);
        // perms.add(Role.ROLE_SERVICE_ONLINE);
    	  
        /** Actividades **/
        // perms.add(Role.ROLE_ACTIVITY_VIEW);
        // perms.add(Role.ROLE_ACTIVITY_EDIT);
        // perms.add(Role.ROLE_ACTIVITY_DELETE);
        // perms.add(Role.ROLE_ACTIVITY_REGISTER_USER);
        // perms.add(Role.ROLE_ACTIVITY_TRAINER_VIEW_ALL);
        // perms.add(Role.ROLE_ACTIVITY_TRAINER_VIEW_OWN);
        // perms.add(Role.ROLE_ACTIVITY_ANNOTATION_ABOUT_SERVICE);
        // perms.add(Role.ROLE_ACTIVITY_ONLINE);
    	  
        /** Personal **/
        // perms.add(Role.ROLE_PERSONAL_VIEW_OWN);
        // perms.add(Role.ROLE_PERSONAL_VIEW_ALL);
        // perms.add(Role.ROLE_PERSONAL_EDIT);
        // perms.add(Role.ROLE_PERSONAL_DELETE);
        // perms.add(Role.ROLE_PERSONAL_TYPE);
        // perms.add(Role.ROLE_PERSONAL_PERMISSION);
        // perms.add(Role.ROLE_PERSONAL_CHANGE_PASSWORD);
        // perms.add(Role.ROLE_PERSONAL_EDIT_VACACIONES);
        // perms.add(Role.ROLE_PERSONAL_EDIT_RESPONSABLEINCIDENCIAS);
    	  
        /** Proveedores **/
        // perms.add(Role.ROLE_SUPPLIER_CREATE);
        // perms.add(Role.ROLE_SUPPLIER_VIEW);
        // perms.add(Role.ROLE_SUPPLIER_EDIT);
        // perms.add(Role.ROLE_SUPPLIER_DELETE);
    	  
        /** Evaluaciones **/
        // perms.add(Role.ROLE_EVALUATION_NEW);
        // perms.add(Role.ROLE_EVALUATION_VIEW);
        // perms.add(Role.ROLE_EVALUATION_EDIT);
    	  
        /** Recursos **/
        // perms.add(Role.ROLE_RECURSO_NEW);
        // perms.add(Role.ROLE_RECURSO_ORDENADOR_ON_OFF);
        // perms.add(Role.ROLE_RECURSO_VIEW);
        // perms.add(Role.ROLE_RECURSO_EDIT_ORDENADOR);
        // perms.add(Role.ROLE_RECURSO_EDIT_ROUTER);
        // perms.add(Role.ROLE_RECURSO_EDIT_RECURSO);
        // perms.add(Role.ROLE_RECURSO_VIEW_CATEGORIA);
        // perms.add(Role.ROLE_RECURSO_EDIT_CATEGORIA);
        // perms.add(Role.ROLE_RECURSO_SEND_USER_COMMAND);
        // perms.add(Role.ROLE_RECURSO_SEND_ROOT_COMMAND);
        // perms.add(Role.ROLE_RECURSO_VIEW_RESERVA);
        // perms.add(Role.ROLE_RECURSO_EDIT_RESERVA);
        // perms.add(Role.ROLE_RECURSO_VIEW_AVERIAS);
        // perms.add(Role.ROLE_RECURSO_EDIT_AVERIAS);
        // perms.add(Role.ROLE_RECURSO_VIEW_PEDIDOS);
        // perms.add(Role.ROLE_RECURSO_EDIT_PEDIDOS);
    	  
        /** Incidencias **/
        // perms.add(Role.ROLE_INCIDENCE_VIEW_PAGE);
        // perms.add(Role.ROLE_INCIDENCE_CREATE);
        // perms.add(Role.ROLE_INCIDENCE_CATEGORIES_TAB);
        // perms.add(Role.ROLE_INCIDENCE_CATEGORIES_EDIT);
        // perms.add(Role.ROLE_INCIDENCE_CATEGORIES_CREATE);
        // perms.add(Role.ROLE_INCIDENCE_REPORTS_TAB);
        // perms.add(Role.ROLE_INCIDENCE_SELECT_ALL_CATEGORIES);
        // perms.add(Role.ROLE_INCIDENCE_SELECT_OWN_CATEGORIES);
        // perms.add(Role.ROLE_INCIDENCE_VIEW);
        // perms.add(Role.ROLE_INCIDENCE_VIEW_ALL);
        // perms.add(Role.ROLE_INCIDENCE_FUNDETEC);
    	// perms.add(Role.ROLE_INCIDENCE_RECEIVE_MAIL);

        /** Parámetros de configuración **/
        // perms.add(Role.ROLE_PARAMETERS_VIEW);
    	  
        /** Financiero **/
        // perms.add(Role.ROLE_FINANCIAL_VIEW);
        // perms.add(Role.ROLE_FINANCIAL_ADMIN);
    	  
        /** Informes automaticos **/
        // perms.add(Role.ROLE_SEND_AUTOMATIC_FORMS_VIEW);
        // perms.add(Role.ROLE_SEND_AUTOMATIC_FORMS_EDIT);
    	  
        /** Cuadro de mandos **/
        // perms.add(Role.ROLE_CUADRO_VIEW);
        // perms.add(Role.ROLE_CUADRO_EDIT);
        // perms.add(Role.ROLE_CUADRO_CENTRO_OWN);
        // perms.add(Role.ROLE_ALARM_EDIT);
    	  
        /** WIKI **/
        // perms.add(Role.ROLE_WIKI_BURGOS);
        // perms.add(Role.ROLE_WIKI_PALENCIA);
        // perms.add(Role.ROLE_WIKI_LEON);
        // perms.add(Role.ROLE_WIKI_ZAMORA);
        // perms.add(Role.ROLE_WIKI_VALLADOLID);
        // perms.add(Role.ROLE_WIKI_SORIA);
        // perms.add(Role.ROLE_WIKI_SEGOVIA);
        // perms.add(Role.ROLE_WIKI_SALAMANCA);
        // perms.add(Role.ROLE_WIKI_AVILA);
        // perms.add(Role.ROLE_WIKI_GENERAL);
    	  
        // perms.add(Role.QUALITE);
        // perms.add(Role.ADMIN);
    	  
        /** Contratos **/
        // perms.add(Role.ROLE_CONTRACT_CREATE);
        // perms.add(Role.ROLE_CONTRACT_VIEW);
        // perms.add(Role.ROLE_CONTRACT_EDIT);
        // perms.add(Role.ROLE_CONTRACT_DELETE);
    	  
        /** Paneles **/
        // perms.add(Role.ROLE_PANEL_VIEW);
        // perms.add(Role.ROLE_PANEL_CREATE_PANEL);
        // perms.add(Role.ROLE_PANEL_CREATE_NEWS);
        // perms.add(Role.ROLE_PANEL_CREATE_EVENTS);
        // perms.add(Role.ROLE_PANEL_VIEW_ALLPANEL);
        // perms.add(Role.ROLE_PANEL_EDIT_ALLPANEL);
        // perms.add(Role.ROLE_PANEL_UPLOAD_FILES);
    	  
        /** Notificaciones **/
        // perms.add(Role.ROLE_NOTIFICATIONS_VIEW);
        //break;

      //case ROLE_INFORMER:
        /** Centros **/
        // perms.add(Role.ROLE_CENTER_VIEW_OWN);
        // perms.add(Role.ROLE_CENTER_VIEW_ALL);
        // perms.add(Role.ROLE_CENTER_EDIT_ALL);
        // perms.add(Role.ROLE_CENTER_EDIT_OWN);
        // perms.add(Role.ROLE_CENTER_DELETE);
        // perms.add(Role.ROLE_CENTER_CREATE);
        // perms.add(Role.ROLE_CENTER_AULA_VIEW);
        // perms.add(Role.ROLE_CENTER_AULA_EDIT);
        // perms.add(Role.ROLE_CENTER_EDIT_URLDAP);
        // perms.add(Role.ROLE_CENTER_FESTIVOS_VIEW);
        // perms.add(Role.ROLE_CENTER_FESTIVOS_EDIT);
        // perms.add(Role.ROLE_CENTER_FESTIVOS_DELETE);
        // perms.add(Role.ROLE_CENTER_FESTIVOS_CREATE);
        // perms.add(Role.ROLE_CENTER_CALENDARIO_VIEW);
    	  
        /** Usuarios **/
    	// perms.add(Role.ROLE_USER_VIEW);
        // perms.add(Role.ROLE_USER_VIEW_ALL_CENTER);
    	// perms.add(Role.ROLE_USER_PRE_DELETE);
    	// perms.add(Role.ROLE_USER_EDIT);
        // perms.add(Role.ROLE_USER_DELETE);
        // perms.add(Role.ROLE_USER_VIEW_EVALUATIONS);
        // perms.add(Role.ROLE_USER_ONLINE);
    	  
        /** Servicios **/
    	// perms.add(Role.ROLE_SERVICE_VIEW);
        // perms.add(Role.ROLE_SERVICE_EDIT);
        // perms.add(Role.ROLE_SERVICE_DELETE);
    	// perms.add(Role.ROLE_SERVICE_EDIT_REGISTER_USER_OWN);
        // perms.add(Role.ROLE_SERVICE_EDIT_REGISTER_USER_ALL);
        // perms.add(Role.ROLE_SERVICE_PROPUESTAS_EDIT);
        // perms.add(Role.ROLE_SERVICE_ONLINE);
    	  
        /** Actividades **/
    	// perms.add(Role.ROLE_ACTIVITY_VIEW);
        // perms.add(Role.ROLE_ACTIVITY_EDIT);
        // perms.add(Role.ROLE_ACTIVITY_DELETE);
    	// perms.add(Role.ROLE_ACTIVITY_REGISTER_USER);
        // perms.add(Role.ROLE_ACTIVITY_TRAINER_VIEW_ALL);
    	// perms.add(Role.ROLE_ACTIVITY_TRAINER_VIEW_OWN);
        // perms.add(Role.ROLE_ACTIVITY_ANNOTATION_ABOUT_SERVICE);
        // perms.add(Role.ROLE_ACTIVITY_ONLINE);
    	  
        /** Personal **/
    	// perms.add(Role.ROLE_PERSONAL_VIEW_OWN);
        // perms.add(Role.ROLE_PERSONAL_VIEW_ALL);
        // perms.add(Role.ROLE_PERSONAL_EDIT);
        // perms.add(Role.ROLE_PERSONAL_DELETE);
        // perms.add(Role.ROLE_PERSONAL_TYPE);
        // perms.add(Role.ROLE_PERSONAL_PERMISSION);
        // perms.add(Role.ROLE_PERSONAL_CHANGE_PASSWORD);
        // perms.add(Role.ROLE_PERSONAL_EDIT_VACACIONES);
        // perms.add(Role.ROLE_PERSONAL_EDIT_RESPONSABLEINCIDENCIAS);
    	  
        /** Proveedores **/
        // perms.add(Role.ROLE_SUPPLIER_CREATE);
        // perms.add(Role.ROLE_SUPPLIER_VIEW);
        // perms.add(Role.ROLE_SUPPLIER_EDIT);
        // perms.add(Role.ROLE_SUPPLIER_DELETE);
    	  
        /** Evaluaciones **/
        // perms.add(Role.ROLE_EVALUATION_NEW);
        // perms.add(Role.ROLE_EVALUATION_VIEW);
        // perms.add(Role.ROLE_EVALUATION_EDIT);
    	  
        /** Recursos **/
        // perms.add(Role.ROLE_RECURSO_NEW);
    	// perms.add(Role.ROLE_RECURSO_ORDENADOR_ON_OFF);
    	// perms.add(Role.ROLE_RECURSO_VIEW);
        // perms.add(Role.ROLE_RECURSO_EDIT_ORDENADOR);
        // perms.add(Role.ROLE_RECURSO_EDIT_ROUTER);
        // perms.add(Role.ROLE_RECURSO_EDIT_RECURSO);
    	// perms.add(Role.ROLE_RECURSO_VIEW_CATEGORIA);
        // perms.add(Role.ROLE_RECURSO_EDIT_CATEGORIA);
        // perms.add(Role.ROLE_RECURSO_SEND_USER_COMMAND);
        // perms.add(Role.ROLE_RECURSO_SEND_ROOT_COMMAND);
    	// perms.add(Role.ROLE_RECURSO_VIEW_RESERVA);
    	// perms.add(Role.ROLE_RECURSO_EDIT_RESERVA);
    	// perms.add(Role.ROLE_RECURSO_VIEW_AVERIAS);
    	// perms.add(Role.ROLE_RECURSO_EDIT_AVERIAS);
        // perms.add(Role.ROLE_RECURSO_VIEW_PEDIDOS);
        // perms.add(Role.ROLE_RECURSO_EDIT_PEDIDOS);

        /** Incidencias **/
    	// perms.add(Role.ROLE_INCIDENCE_VIEW_PAGE);
    	// perms.add(Role.ROLE_INCIDENCE_CREATE);
        // perms.add(Role.ROLE_INCIDENCE_CATEGORIES_TAB);
        // perms.add(Role.ROLE_INCIDENCE_REPORTS_TAB);
        // perms.add(Role.ROLE_INCIDENCE_SELECT_ALL_CATEGORIES);
        // perms.add(Role.ROLE_INCIDENCE_CATEGORIES_CREATE);
        // perms.add(Role.ROLE_INCIDENCE_CATEGORIES_EDIT);
    	// perms.add(Role.ROLE_INCIDENCE_SELECT_OWN_CATEGORIES);
    	// perms.add(Role.ROLE_INCIDENCE_VIEW);
        // perms.add(Role.ROLE_INCIDENCE_VIEW_ALL);
        // perms.add(Role.ROLE_INCIDENCE_FUNDETEC);
    	// perms.add(Role.ROLE_INCIDENCE_RECEIVE_MAIL);

        /** Parámetros de configuración **/
        // perms.add(Role.ROLE_PARAMETERS_VIEW);
    	  
        /** Financiero **/
        // perms.add(Role.ROLE_FINANCIAL_VIEW);
        // perms.add(Role.ROLE_FINANCIAL_ADMIN);
    	  
        /** Informes automaticos **/
        // perms.add(Role.ROLE_SEND_AUTOMATIC_FORMS_VIEW);
        // perms.add(Role.ROLE_SEND_AUTOMATIC_FORMS_EDIT);
    	  
        /** Cuadro de mandos **/
        // perms.add(Role.ROLE_CUADRO_VIEW);
        // perms.add(Role.ROLE_CUADRO_EDIT);
        // perms.add(Role.ROLE_CUADRO_CENTRO_OWN);
        // perms.add(Role.ROLE_ALARM_EDIT);
    	  
        /** WIKI **/
        // perms.add(Role.ROLE_WIKI_BURGOS);
        // perms.add(Role.ROLE_WIKI_PALENCIA);
        // perms.add(Role.ROLE_WIKI_LEON);
        // perms.add(Role.ROLE_WIKI_ZAMORA);
        // perms.add(Role.ROLE_WIKI_VALLADOLID);
        // perms.add(Role.ROLE_WIKI_SORIA);
        // perms.add(Role.ROLE_WIKI_SEGOVIA);
        // perms.add(Role.ROLE_WIKI_SALAMANCA);
        // perms.add(Role.ROLE_WIKI_AVILA);
    	// perms.add(Role.ROLE_WIKI_GENERAL);
        
        // perms.add(Role.QUALITE);
        // perms.add(Role.ADMIN);
        /** Contratos **/
        // perms.add(Role.ROLE_CONTRACT_CREATE);
        // perms.add(Role.ROLE_CONTRACT_VIEW);
        // perms.add(Role.ROLE_CONTRACT_EDIT);
        // perms.add(Role.ROLE_CONTRACT_DELETE);

    	  /** Paneles **/
    	// perms.add(Role.ROLE_PANEL_VIEW);
        // perms.add(Role.ROLE_PANEL_CREATE_PANEL);
    	// perms.add(Role.ROLE_PANEL_CREATE_NEWS);
    	// perms.add(Role.ROLE_PANEL_CREATE_EVENTS);
        // perms.add(Role.ROLE_PANEL_VIEW_ALLPANEL);
        // perms.add(Role.ROLE_PANEL_EDIT_ALLPANEL);
    	// perms.add(Role.ROLE_PANEL_UPLOAD_FILES);
        
        /** Notificaciones **/
        // perms.add(Role.ROLE_NOTIFICATIONS_VIEW);
        //break;

      //case ROLE_MANAGER:
        /** Centros **/
    	// perms.add(Role.ROLE_CENTER_VIEW_OWN);
    	// perms.add(Role.ROLE_CENTER_VIEW_ALL);
    	// perms.add(Role.ROLE_CENTER_EDIT_ALL);
    	// perms.add(Role.ROLE_CENTER_EDIT_OWN);
        // perms.add(Role.ROLE_CENTER_DELETE);
        // perms.add(Role.ROLE_CENTER_CREATE);
    	// perms.add(Role.ROLE_CENTER_AULA_VIEW);
    	// perms.add(Role.ROLE_CENTER_AULA_EDIT);
        // perms.add(Role.ROLE_CENTER_EDIT_URLDAP);
    	// perms.add(Role.ROLE_CENTER_FESTIVOS_VIEW);
    	// perms.add(Role.ROLE_CENTER_FESTIVOS_EDIT);
    	// perms.add(Role.ROLE_CENTER_FESTIVOS_DELETE);
    	// perms.add(Role.ROLE_CENTER_FESTIVOS_CREATE);
    	// perms.add(Role.ROLE_CENTER_CALENDARIO_VIEW);
    	  
        /** Usuarios **/
    	// perms.add(Role.ROLE_USER_VIEW);
    	// perms.add(Role.ROLE_USER_VIEW_ALL_CENTER);
    	// perms.add(Role.ROLE_USER_PRE_DELETE);
    	// perms.add(Role.ROLE_USER_EDIT);
    	// perms.add(Role.ROLE_USER_DELETE);
    	// perms.add(Role.ROLE_USER_VIEW_EVALUATIONS);
    	// perms.add(Role.ROLE_USER_ONLINE);
    	  
        /** Servicios **/
    	// perms.add(Role.ROLE_SERVICE_VIEW);
        // perms.add(Role.ROLE_SERVICE_EDIT);
        // perms.add(Role.ROLE_SERVICE_DELETE);
    	// perms.add(Role.ROLE_SERVICE_EDIT_REGISTER_USER_OWN);
    	// perms.add(Role.ROLE_SERVICE_EDIT_REGISTER_USER_ALL);
    	// perms.add(Role.ROLE_SERVICE_PROPUESTAS_EDIT);
    	// perms.add(Role.ROLE_SERVICE_ONLINE);
    	  
        /** Actividades **/
    	// perms.add(Role.ROLE_ACTIVITY_VIEW);
    	// perms.add(Role.ROLE_ACTIVITY_EDIT);
    	// perms.add(Role.ROLE_ACTIVITY_DELETE);
    	// perms.add(Role.ROLE_ACTIVITY_REGISTER_USER);
    	// perms.add(Role.ROLE_ACTIVITY_TRAINER_VIEW_ALL);
    	// perms.add(Role.ROLE_ACTIVITY_TRAINER_VIEW_OWN);
        // perms.add(Role.ROLE_ACTIVITY_ANNOTATION_ABOUT_SERVICE);
    	// perms.add(Role.ROLE_ACTIVITY_ONLINE);
    	  
        /** Personal **/
    	// perms.add(Role.ROLE_PERSONAL_VIEW_OWN);
    	// perms.add(Role.ROLE_PERSONAL_VIEW_ALL);
    	// perms.add(Role.ROLE_PERSONAL_EDIT);
    	// perms.add(Role.ROLE_PERSONAL_DELETE);
    	// perms.add(Role.ROLE_PERSONAL_TYPE);
        // perms.add(Role.ROLE_PERSONAL_PERMISSION);
        // perms.add(Role.ROLE_PERSONAL_CHANGE_PASSWORD);
    	// perms.add(Role.ROLE_PERSONAL_EDIT_VACACIONES);
        // perms.add(Role.ROLE_PERSONAL_EDIT_RESPONSABLEINCIDENCIAS);
    	  
        /** Proveedores **/
    	// perms.add(Role.ROLE_SUPPLIER_CREATE);
    	// perms.add(Role.ROLE_SUPPLIER_VIEW);
    	// perms.add(Role.ROLE_SUPPLIER_EDIT);
    	// perms.add(Role.ROLE_SUPPLIER_DELETE);
    	  
        /** Evaluaciones **/
    	// perms.add(Role.ROLE_EVALUATION_NEW);
    	// perms.add(Role.ROLE_EVALUATION_VIEW);
    	// perms.add(Role.ROLE_EVALUATION_EDIT);
    	  
        /** Recursos **/
    	// perms.add(Role.ROLE_RECURSO_NEW);
    	// perms.add(Role.ROLE_RECURSO_ORDENADOR_ON_OFF);
    	// perms.add(Role.ROLE_RECURSO_VIEW);
    	// perms.add(Role.ROLE_RECURSO_EDIT_ORDENADOR);
    	// perms.add(Role.ROLE_RECURSO_EDIT_ROUTER);
    	// perms.add(Role.ROLE_RECURSO_EDIT_RECURSO);
    	// perms.add(Role.ROLE_RECURSO_VIEW_CATEGORIA);
    	// perms.add(Role.ROLE_RECURSO_EDIT_CATEGORIA);
        // perms.add(Role.ROLE_RECURSO_SEND_USER_COMMAND);
        // perms.add(Role.ROLE_RECURSO_SEND_ROOT_COMMAND);
    	// perms.add(Role.ROLE_RECURSO_VIEW_RESERVA);
    	// perms.add(Role.ROLE_RECURSO_EDIT_RESERVA);
    	// perms.add(Role.ROLE_RECURSO_VIEW_AVERIAS);
    	// perms.add(Role.ROLE_RECURSO_EDIT_AVERIAS);
    	// perms.add(Role.ROLE_RECURSO_VIEW_PEDIDOS);
    	// perms.add(Role.ROLE_RECURSO_EDIT_PEDIDOS);
    	  
        /** Incidencias **/
    	// perms.add(Role.ROLE_INCIDENCE_VIEW_PAGE);
    	// perms.add(Role.ROLE_INCIDENCE_CREATE);
        // perms.add(Role.ROLE_INCIDENCE_CATEGORIES_TAB);
        // perms.add(Role.ROLE_INCIDENCE_CATEGORIES_CREATE);
        // perms.add(Role.ROLE_INCIDENCE_CATEGORIES_EDIT);
    	// perms.add(Role.ROLE_INCIDENCE_REPORTS_TAB);
    	// perms.add(Role.ROLE_INCIDENCE_SELECT_ALL_CATEGORIES);
    	// perms.add(Role.ROLE_INCIDENCE_SELECT_OWN_CATEGORIES);
    	// perms.add(Role.ROLE_INCIDENCE_VIEW);
        // perms.add(Role.ROLE_INCIDENCE_VIEW_ALL);
        // perms.add(Role.ROLE_INCIDENCE_FUNDETEC);
    	// perms.add(Role.ROLE_INCIDENCE_RECEIVE_MAIL);

        /** Parámetros de configuración **/
        // perms.add(Role.ROLE_PARAMETERS_VIEW);
    	  
        /** Financiero **/
        // perms.add(Role.ROLE_FINANCIAL_VIEW);
        // perms.add(Role.ROLE_FINANCIAL_ADMIN);
    	  
        /** Informes automaticos **/
    	// perms.add(Role.ROLE_SEND_AUTOMATIC_FORMS_VIEW);
    	// perms.add(Role.ROLE_SEND_AUTOMATIC_FORMS_EDIT);
    	  
        /** Cuadro de mandos **/
    	// perms.add(Role.ROLE_CUADRO_VIEW);
    	// perms.add(Role.ROLE_CUADRO_EDIT);
        // perms.add(Role.ROLE_CUADRO_CENTRO_OWN);
    	// perms.add(Role.ROLE_ALARM_EDIT);
    	  
        /** WIKI **/
    	// perms.add(Role.ROLE_WIKI_BURGOS);
    	// perms.add(Role.ROLE_WIKI_PALENCIA);
    	// perms.add(Role.ROLE_WIKI_LEON);
    	// perms.add(Role.ROLE_WIKI_ZAMORA);
    	// perms.add(Role.ROLE_WIKI_VALLADOLID);
    	// perms.add(Role.ROLE_WIKI_SORIA);
    	// perms.add(Role.ROLE_WIKI_SEGOVIA);
    	// perms.add(Role.ROLE_WIKI_SALAMANCA);
    	// perms.add(Role.ROLE_WIKI_AVILA);
    	// perms.add(Role.ROLE_WIKI_GENERAL);
    	// perms.add(Role.QUALITE);
        // perms.add(Role.ADMIN);
    	  
        /** Contratos **/
        // perms.add(Role.ROLE_CONTRACT_CREATE);
        // perms.add(Role.ROLE_CONTRACT_VIEW);
        // perms.add(Role.ROLE_CONTRACT_EDIT);
        // perms.add(Role.ROLE_CONTRACT_DELETE);

    	/** Paneles **/
    	// perms.add(Role.ROLE_PANEL_VIEW);
    	// perms.add(Role.ROLE_PANEL_CREATE_PANEL);
    	// perms.add(Role.ROLE_PANEL_CREATE_NEWS);
    	// perms.add(Role.ROLE_PANEL_CREATE_EVENTS);
    	// perms.add(Role.ROLE_PANEL_VIEW_ALLPANEL);
    	// perms.add(Role.ROLE_PANEL_EDIT_ALLPANEL);
    	// perms.add(Role.ROLE_PANEL_UPLOAD_FILES);

        /** Notificaciones **/
    	// perms.add(Role.ROLE_NOTIFICATIONS_VIEW);

        //break;
      case ROLE_TRAINER:
        /** Centros **/
        // perms.add(Role.ROLE_CENTER_VIEW_OWN);
        // perms.add(Role.ROLE_CENTER_VIEW_ALL);
        // perms.add(Role.ROLE_CENTER_EDIT_ALL);
        // perms.add(Role.ROLE_CENTER_EDIT_OWN);
        // perms.add(Role.ROLE_CENTER_DELETE);
        // perms.add(Role.ROLE_CENTER_CREATE);
        // perms.add(Role.ROLE_CENTER_AULA_VIEW);
        // perms.add(Role.ROLE_CENTER_AULA_EDIT);
        // perms.add(Role.ROLE_CENTER_EDIT_URLDAP);
        // perms.add(Role.ROLE_CENTER_FESTIVOS_VIEW);
        // perms.add(Role.ROLE_CENTER_FESTIVOS_EDIT);
        // perms.add(Role.ROLE_CENTER_FESTIVOS_DELETE);
        // perms.add(Role.ROLE_CENTER_FESTIVOS_CREATE);
        perms.add(Role.ROLE_CENTER_CALENDARIO_VIEW);
        
        /** Usuarios **/
        perms.add(Role.ROLE_USER_VIEW);
        // perms.add(Role.ROLE_USER_VIEW_ALL_CENTER);
        // perms.add(Role.ROLE_USER_PRE_DELETE);
        perms.add(Role.ROLE_USER_EDIT);
        // perms.add(Role.ROLE_USER_DELETE);
        perms.add(Role.ROLE_USER_VIEW_EVALUATIONS);
        // perms.add(Role.ROLE_USER_ONLINE);
        
        /** Servicios **/
        perms.add(Role.ROLE_SERVICE_VIEW);
        // perms.add(Role.ROLE_SERVICE_EDIT);
        // perms.add(Role.ROLE_SERVICE_DELETE);
        perms.add(Role.ROLE_SERVICE_EDIT_REGISTER_USER_OWN);
        // perms.add(Role.ROLE_SERVICE_EDIT_REGISTER_USER_ALL);
        // perms.add(Role.ROLE_SERVICE_PROPUESTAS_EDIT);
        // perms.add(Role.ROLE_SERVICE_ONLINE);
        
        /** Actividades **/
        perms.add(Role.ROLE_ACTIVITY_VIEW);
        perms.add(Role.ROLE_ACTIVITY_EDIT);
        // perms.add(Role.ROLE_ACTIVITY_DELETE);
        perms.add(Role.ROLE_ACTIVITY_REGISTER_USER);
        // perms.add(Role.ROLE_ACTIVITY_TRAINER_VIEW_ALL);
        perms.add(Role.ROLE_ACTIVITY_TRAINER_VIEW_OWN);
        perms.add(Role.ROLE_ACTIVITY_ANNOTATION_ABOUT_SERVICE);
        // perms.add(Role.ROLE_ACTIVITY_ONLINE);
        
        /** Diplomas **/
        perms.add(Role.ROLE_DIPLOMA_VIEW);
        // perms.add(Role.ROLE_DIPLOMA_EDIT);
        // perms.add(Role.ROLE_DIPLOMA_DELETE);
        
        /** Exámenes **/
        perms.add(Role.ROLE_EXAM_VIEW);
        // perms.add(Role.ROLE_EXAM_EDIT);
        // perms.add(Role.ROLE_EXAM_DELETE);
        
        /** Personal **/
        perms.add(Role.ROLE_PERSONAL_VIEW_OWN);
        // perms.add(Role.ROLE_PERSONAL_VIEW_ALL);
        // perms.add(Role.ROLE_PERSONAL_EDIT);
        // perms.add(Role.ROLE_PERSONAL_DELETE);
        // perms.add(Role.ROLE_PERSONAL_TYPE);
        // perms.add(Role.ROLE_PERSONAL_PERMISSION);
        perms.add(Role.ROLE_PERSONAL_CHANGE_PASSWORD);
        // perms.add(Role.ROLE_PERSONAL_EDIT_VACACIONES);
        // perms.add(Role.ROLE_PERSONAL_EDIT_RESPONSABLEINCIDENCIAS);
        
        /** Proveedores **/
        // perms.add(Role.ROLE_SUPPLIER_CREATE);
        // perms.add(Role.ROLE_SUPPLIER_VIEW);
        // perms.add(Role.ROLE_SUPPLIER_EDIT);
        // perms.add(Role.ROLE_SUPPLIER_DELETE);
        
        /** Evaluaciones **/
        perms.add(Role.ROLE_EVALUATION_NEW);
        perms.add(Role.ROLE_EVALUATION_VIEW);
        perms.add(Role.ROLE_EVALUATION_EDIT);
        
        /** Recursos **/
        perms.add(Role.ROLE_RECURSO_NEW);
        perms.add(Role.ROLE_RECURSO_ORDENADOR_ON_OFF);
        perms.add(Role.ROLE_RECURSO_VIEW);
        perms.add(Role.ROLE_RECURSO_EDIT_ORDENADOR);
        perms.add(Role.ROLE_RECURSO_EDIT_ROUTER);
        perms.add(Role.ROLE_RECURSO_EDIT_RECURSO);
        perms.add(Role.ROLE_RECURSO_VIEW_CATEGORIA);
        perms.add(Role.ROLE_RECURSO_EDIT_CATEGORIA);
        // perms.add(Role.ROLE_RECURSO_SEND_USER_COMMAND);
        // perms.add(Role.ROLE_RECURSO_SEND_ROOT_COMMAND);
        perms.add(Role.ROLE_RECURSO_VIEW_RESERVA);
        perms.add(Role.ROLE_RECURSO_EDIT_RESERVA);
        perms.add(Role.ROLE_RECURSO_VIEW_AVERIAS);
        perms.add(Role.ROLE_RECURSO_EDIT_AVERIAS);
        // perms.add(Role.ROLE_RECURSO_VIEW_PEDIDOS);
        // perms.add(Role.ROLE_RECURSO_EDIT_PEDIDOS);
        
        /** Incidencias **/
        // perms.add(Role.ROLE_INCIDENCE_VIEW_PAGE);
        // perms.add(Role.ROLE_INCIDENCE_CREATE);
        // perms.add(Role.ROLE_INCIDENCE_CATEGORIES_TAB);
        // perms.add(Role.ROLE_INCIDENCE_CATEGORIES_CREATE);
        // perms.add(Role.ROLE_INCIDENCE_CATEGORIES_EDIT);
        // perms.add(Role.ROLE_INCIDENCE_REPORTS_TAB);
        // perms.add(Role.ROLE_INCIDENCE_SELECT_ALL_CATEGORIES);
        // perms.add(Role.ROLE_INCIDENCE_SELECT_OWN_CATEGORIES);
        // perms.add(Role.ROLE_INCIDENCE_VIEW);
        // perms.add(Role.ROLE_INCIDENCE_VIEW_ALL);
        // perms.add(Role.ROLE_INCIDENCE_FUNDETEC);
        // perms.add(Role.ROLE_INCIDENCE_RECEIVE_MAIL);*/

        /** Parámetros de configuración **/
        // perms.add(Role.ROLE_PARAMETERS_VIEW);
        
        /** Financiero **/
        // perms.add(Role.ROLE_FINANCIAL_VIEW);
        // perms.add(Role.ROLE_FINANCIAL_ADMIN);
        
        /** Informes automaticos **/
        // perms.add(Role.ROLE_SEND_AUTOMATIC_FORMS_VIEW);
        // perms.add(Role.ROLE_SEND_AUTOMATIC_FORMS_EDIT);
        
        /** Cuadro de mandos **/
        // perms.add(Role.ROLE_CUADRO_VIEW);
        // perms.add(Role.ROLE_CUADRO_EDIT);
        // perms.add(Role.ROLE_CUADRO_CENTRO_OWN);
        // perms.add(Role.ROLE_ALARM_EDIT);
        
        /** WIKI **/
        // perms.add(Role.ROLE_WIKI_BURGOS);
        // perms.add(Role.ROLE_WIKI_PALENCIA);
        // perms.add(Role.ROLE_WIKI_LEON);
        // perms.add(Role.ROLE_WIKI_ZAMORA);
        // perms.add(Role.ROLE_WIKI_VALLADOLID);
        // perms.add(Role.ROLE_WIKI_SORIA);
        // perms.add(Role.ROLE_WIKI_SEGOVIA);
        // perms.add(Role.ROLE_WIKI_SALAMANCA);
        // perms.add(Role.ROLE_WIKI_AVILA);
        // perms.add(Role.ROLE_WIKI_GENERAL);
        
        // perms.add(Role.QUALITE);
        // perms.add(Role.ADMIN);
        
        /** Contratos **/
        // perms.add(Role.ROLE_CONTRACT_CREATE);
        // perms.add(Role.ROLE_CONTRACT_VIEW);
        // perms.add(Role.ROLE_CONTRACT_EDIT);
        // perms.add(Role.ROLE_CONTRACT_DELETE);
        
        /** Paneles **/
        // perms.add(Role.ROLE_PANEL_VIEW);
        // perms.add(Role.ROLE_PANEL_CREATE_PANEL);
        // perms.add(Role.ROLE_PANEL_CREATE_NEWS);
        // perms.add(Role.ROLE_PANEL_CREATE_EVENTS);
        // perms.add(Role.ROLE_PANEL_VIEW_ALLPANEL);
        // perms.add(Role.ROLE_PANEL_EDIT_ALLPANEL);
        // perms.add(Role.ROLE_PANEL_UPLOAD_FILES);*/
        
        /** Notificaciones **/
        // perms.add(Role.ROLE_NOTIFICATIONS_VIEW);
        break;

      //case ROLE_TRAINER_COORDINATOR:
        /** Centros **/
    	// perms.add(Role.ROLE_CENTER_VIEW_OWN);
    	// perms.add(Role.ROLE_CENTER_VIEW_ALL);
    	// perms.add(Role.ROLE_CENTER_EDIT_ALL);
    	// perms.add(Role.ROLE_CENTER_EDIT_OWN);
        // perms.add(Role.ROLE_CENTER_DELETE);
        // perms.add(Role.ROLE_CENTER_CREATE);
    	// perms.add(Role.ROLE_CENTER_AULA_VIEW);
    	// perms.add(Role.ROLE_CENTER_AULA_EDIT);
        // perms.add(Role.ROLE_CENTER_EDIT_URLDAP);
    	// perms.add(Role.ROLE_CENTER_FESTIVOS_VIEW);
    	// perms.add(Role.ROLE_CENTER_FESTIVOS_EDIT);
    	// perms.add(Role.ROLE_CENTER_FESTIVOS_DELETE);
    	// perms.add(Role.ROLE_CENTER_FESTIVOS_CREATE);
    	// perms.add(Role.ROLE_CENTER_CALENDARIO_VIEW);
    	  
        /** Usuarios **/
    	// perms.add(Role.ROLE_USER_VIEW);
    	// perms.add(Role.ROLE_USER_VIEW_ALL_CENTER);
    	// perms.add(Role.ROLE_USER_PRE_DELETE);
    	// perms.add(Role.ROLE_USER_EDIT);
    	// perms.add(Role.ROLE_USER_DELETE);
    	// perms.add(Role.ROLE_USER_VIEW_EVALUATIONS);
    	// perms.add(Role.ROLE_USER_ONLINE);
    	  
        /** Servicios **/
    	// perms.add(Role.ROLE_SERVICE_VIEW);
    	// perms.add(Role.ROLE_SERVICE_EDIT);
    	// perms.add(Role.ROLE_SERVICE_DELETE);
    	// perms.add(Role.ROLE_SERVICE_EDIT_REGISTER_USER_OWN);
    	// perms.add(Role.ROLE_SERVICE_EDIT_REGISTER_USER_ALL);
    	// perms.add(Role.ROLE_SERVICE_PROPUESTAS_EDIT);
    	// perms.add(Role.ROLE_SERVICE_ONLINE);
    	  
        /** Actividades **/
    	// perms.add(Role.ROLE_ACTIVITY_VIEW);
    	// perms.add(Role.ROLE_ACTIVITY_EDIT);
    	// perms.add(Role.ROLE_ACTIVITY_DELETE);
    	// perms.add(Role.ROLE_ACTIVITY_REGISTER_USER);
    	// perms.add(Role.ROLE_ACTIVITY_TRAINER_VIEW_ALL);
    	// perms.add(Role.ROLE_ACTIVITY_TRAINER_VIEW_OWN);
    	// perms.add(Role.ROLE_ACTIVITY_ANNOTATION_ABOUT_SERVICE);
    	// perms.add(Role.ROLE_ACTIVITY_ONLINE);
    	  
        /** Personal **/
    	// perms.add(Role.ROLE_PERSONAL_VIEW_OWN);
    	// perms.add(Role.ROLE_PERSONAL_VIEW_ALL);
    	// perms.add(Role.ROLE_PERSONAL_EDIT);
    	// perms.add(Role.ROLE_PERSONAL_DELETE);
    	// perms.add(Role.ROLE_PERSONAL_TYPE);
        // perms.add(Role.ROLE_PERSONAL_PERMISSION);
        // perms.add(Role.ROLE_PERSONAL_CHANGE_PASSWORD);
    	// perms.add(Role.ROLE_PERSONAL_EDIT_VACACIONES);
        // perms.add(Role.ROLE_PERSONAL_EDIT_RESPONSABLEINCIDENCIAS);
    	  
        /** Proveedores **/
    	// perms.add(Role.ROLE_SUPPLIER_CREATE);
    	// perms.add(Role.ROLE_SUPPLIER_VIEW);
    	// perms.add(Role.ROLE_SUPPLIER_EDIT);
    	// perms.add(Role.ROLE_SUPPLIER_DELETE);
    	  
        /** Evaluaciones **/
    	// perms.add(Role.ROLE_EVALUATION_NEW);
    	// perms.add(Role.ROLE_EVALUATION_VIEW);
    	// perms.add(Role.ROLE_EVALUATION_EDIT);
    	  
        /** Recursos **/
    	// perms.add(Role.ROLE_RECURSO_NEW);
    	// perms.add(Role.ROLE_RECURSO_ORDENADOR_ON_OFF);
    	// perms.add(Role.ROLE_RECURSO_VIEW);
    	// perms.add(Role.ROLE_RECURSO_EDIT_ORDENADOR);
    	// perms.add(Role.ROLE_RECURSO_EDIT_ROUTER);
    	// perms.add(Role.ROLE_RECURSO_EDIT_RECURSO);
    	// perms.add(Role.ROLE_RECURSO_VIEW_CATEGORIA);
    	// perms.add(Role.ROLE_RECURSO_EDIT_CATEGORIA);
        // perms.add(Role.ROLE_RECURSO_SEND_USER_COMMAND);
        // perms.add(Role.ROLE_RECURSO_SEND_ROOT_COMMAND);
    	// perms.add(Role.ROLE_RECURSO_VIEW_RESERVA);
    	// perms.add(Role.ROLE_RECURSO_EDIT_RESERVA);
    	// perms.add(Role.ROLE_RECURSO_VIEW_AVERIAS);
    	// perms.add(Role.ROLE_RECURSO_EDIT_AVERIAS);
    	// perms.add(Role.ROLE_RECURSO_VIEW_PEDIDOS);
    	// perms.add(Role.ROLE_RECURSO_EDIT_PEDIDOS);
        /** Incidencias **/
    	// perms.add(Role.ROLE_INCIDENCE_VIEW_PAGE);
    	// perms.add(Role.ROLE_INCIDENCE_CREATE);
        // perms.add(Role.ROLE_INCIDENCE_CATEGORIES_TAB);
        // perms.add(Role.ROLE_INCIDENCE_CATEGORIES_CREATE);
        // perms.add(Role.ROLE_INCIDENCE_CATEGORIES_EDIT);
    	// perms.add(Role.ROLE_INCIDENCE_REPORTS_TAB);
    	// perms.add(Role.ROLE_INCIDENCE_SELECT_ALL_CATEGORIES);
    	// perms.add(Role.ROLE_INCIDENCE_SELECT_OWN_CATEGORIES);
    	// perms.add(Role.ROLE_INCIDENCE_VIEW);
    	// perms.add(Role.ROLE_INCIDENCE_VIEW_ALL);
        // perms.add(Role.ROLE_INCIDENCE_FUNDETEC);
    	// perms.add(Role.ROLE_INCIDENCE_RECEIVE_MAIL);

        /** Parámetros de configuración **/
        // perms.add(Role.ROLE_PARAMETERS_VIEW);
    	  
        /** Financiero **/
        // perms.add(Role.ROLE_FINANCIAL_VIEW);
        // perms.add(Role.ROLE_FINANCIAL_ADMIN);
    	  
        /** Contratos **/
        // perms.add(Role.ROLE_CONTRACT_CREATE);
        // perms.add(Role.ROLE_CONTRACT_VIEW);
        // perms.add(Role.ROLE_CONTRACT_EDIT);
        // perms.add(Role.ROLE_CONTRACT_DELETE);
    	  
        /** Informes automaticos **/
    	// perms.add(Role.ROLE_SEND_AUTOMATIC_FORMS_VIEW);
    	// perms.add(Role.ROLE_SEND_AUTOMATIC_FORMS_EDIT);
    	  
        /** Cuadro de mandos **/
    	// perms.add(Role.ROLE_CUADRO_VIEW);
    	// perms.add(Role.ROLE_CUADRO_EDIT);
        // perms.add(Role.ROLE_CUADRO_CENTRO_OWN);
    	// perms.add(Role.ROLE_ALARM_EDIT);
    	  
        /** WIKI **/
    	// perms.add(Role.ROLE_WIKI_BURGOS);
    	// perms.add(Role.ROLE_WIKI_PALENCIA);
    	// perms.add(Role.ROLE_WIKI_LEON);
    	// perms.add(Role.ROLE_WIKI_ZAMORA);
    	// perms.add(Role.ROLE_WIKI_VALLADOLID);
    	// perms.add(Role.ROLE_WIKI_SORIA);
    	// perms.add(Role.ROLE_WIKI_SEGOVIA);
    	// perms.add(Role.ROLE_WIKI_SALAMANCA);
    	// perms.add(Role.ROLE_WIKI_AVILA);
    	// perms.add(Role.ROLE_WIKI_GENERAL);
    	// perms.add(Role.QUALITE);
        // perms.add(Role.ADMIN);

        /** Paneles **/
    	// perms.add(Role.ROLE_PANEL_VIEW);
    	// perms.add(Role.ROLE_PANEL_CREATE_PANEL);
    	// perms.add(Role.ROLE_PANEL_CREATE_NEWS);
    	// perms.add(Role.ROLE_PANEL_CREATE_EVENTS);
    	// perms.add(Role.ROLE_PANEL_VIEW_ALLPANEL);
    	// perms.add(Role.ROLE_PANEL_EDIT_ALLPANEL);
    	// perms.add(Role.ROLE_PANEL_UPLOAD_FILES);
        
        /** Notificaciones **/
    	// perms.add(Role.ROLE_NOTIFICATIONS_VIEW);

        //break;
      //case ROLE_FINANCIAL_MANAGER:
        /** Centros **/
        // perms.add(Role.ROLE_CENTER_VIEW_OWN);
        // perms.add(Role.ROLE_CENTER_VIEW_ALL);
        // perms.add(Role.ROLE_CENTER_EDIT_ALL);
        // perms.add(Role.ROLE_CENTER_EDIT_OWN);
        // perms.add(Role.ROLE_CENTER_DELETE);
        // perms.add(Role.ROLE_CENTER_CREATE);
        // perms.add(Role.ROLE_CENTER_AULA_VIEW);
        // perms.add(Role.ROLE_CENTER_AULA_EDIT);
        // perms.add(Role.ROLE_CENTER_EDIT_URLDAP);
        // perms.add(Role.ROLE_CENTER_FESTIVOS_VIEW);
        // perms.add(Role.ROLE_CENTER_FESTIVOS_EDIT);
        // perms.add(Role.ROLE_CENTER_FESTIVOS_DELETE);
        // perms.add(Role.ROLE_CENTER_FESTIVOS_CREATE);
        // perms.add(Role.ROLE_CENTER_CALENDARIO_VIEW);
    	  
        /** Usuarios **/
        // perms.add(Role.ROLE_USER_VIEW);
        // perms.add(Role.ROLE_USER_VIEW_ALL_CENTER);
        // perms.add(Role.ROLE_USER_PRE_DELETE);
        // perms.add(Role.ROLE_USER_EDIT);
        // perms.add(Role.ROLE_USER_DELETE);
        // perms.add(Role.ROLE_USER_VIEW_EVALUATIONS);
        // perms.add(Role.ROLE_USER_ONLINE);
    	  
        /** Servicios **/
        // perms.add(Role.ROLE_SERVICE_VIEW);
        // perms.add(Role.ROLE_SERVICE_EDIT);
        // perms.add(Role.ROLE_SERVICE_DELETE);
        // perms.add(Role.ROLE_SERVICE_EDIT_REGISTER_USER_OWN);
        // perms.add(Role.ROLE_SERVICE_EDIT_REGISTER_USER_ALL);
        // perms.add(Role.ROLE_SERVICE_PROPUESTAS_EDIT);
        // perms.add(Role.ROLE_SERVICE_ONLINE);
    	  
        /** Actividades **/
        // perms.add(Role.ROLE_ACTIVITY_VIEW);
        // perms.add(Role.ROLE_ACTIVITY_EDIT);
        // perms.add(Role.ROLE_ACTIVITY_DELETE);
        // perms.add(Role.ROLE_ACTIVITY_REGISTER_USER);
        // perms.add(Role.ROLE_ACTIVITY_TRAINER_VIEW_ALL);
        // perms.add(Role.ROLE_ACTIVITY_TRAINER_VIEW_OWN);
        // perms.add(Role.ROLE_ACTIVITY_ANNOTATION_ABOUT_SERVICE);
        // perms.add(Role.ROLE_ACTIVITY_ONLINE);
    	  
        /** Personal **/
        // perms.add(Role.ROLE_PERSONAL_VIEW_OWN);
        // perms.add(Role.ROLE_PERSONAL_VIEW_ALL);
        // perms.add(Role.ROLE_PERSONAL_EDIT);
        // perms.add(Role.ROLE_PERSONAL_DELETE);
        // perms.add(Role.ROLE_PERSONAL_TYPE);
        // perms.add(Role.ROLE_PERSONAL_PERMISSION);
        // perms.add(Role.ROLE_PERSONAL_CHANGE_PASSWORD);
        // perms.add(Role.ROLE_PERSONAL_EDIT_VACACIONES);
        // perms.add(Role.ROLE_PERSONAL_EDIT_RESPONSABLEINCIDENCIAS);
    	  
        /** Proveedores **/
    	// perms.add(Role.ROLE_SUPPLIER_CREATE);
    	// perms.add(Role.ROLE_SUPPLIER_VIEW);
    	// perms.add(Role.ROLE_SUPPLIER_EDIT);
    	// perms.add(Role.ROLE_SUPPLIER_DELETE);
    	  
        /** Evaluaciones **/
        // perms.add(Role.ROLE_EVALUATION_NEW);
        // perms.add(Role.ROLE_EVALUATION_VIEW);
        // perms.add(Role.ROLE_EVALUATION_EDIT);
    	  
        /** Recursos **/
        // perms.add(Role.ROLE_RECURSO_NEW);
        // perms.add(Role.ROLE_RECURSO_ORDENADOR_ON_OFF);
        // perms.add(Role.ROLE_RECURSO_VIEW);
        // perms.add(Role.ROLE_RECURSO_EDIT_ORDENADOR);
        // perms.add(Role.ROLE_RECURSO_EDIT_ROUTER);
        // perms.add(Role.ROLE_RECURSO_EDIT_RECURSO);
        // perms.add(Role.ROLE_RECURSO_VIEW_CATEGORIA);
        // perms.add(Role.ROLE_RECURSO_EDIT_CATEGORIA);
        // perms.add(Role.ROLE_RECURSO_SEND_USER_COMMAND);
        // perms.add(Role.ROLE_RECURSO_SEND_ROOT_COMMAND);
        // perms.add(Role.ROLE_RECURSO_VIEW_RESERVA);
        // perms.add(Role.ROLE_RECURSO_EDIT_RESERVA);
        // perms.add(Role.ROLE_RECURSO_VIEW_AVERIAS);
        // perms.add(Role.ROLE_RECURSO_EDIT_AVERIAS);
        // perms.add(Role.ROLE_RECURSO_VIEW_PEDIDOS);
        // perms.add(Role.ROLE_RECURSO_EDIT_PEDIDOS);
    	  
        /** Incidencias **/
        // perms.add(Role.ROLE_INCIDENCE_VIEW_PAGE);
        // perms.add(Role.ROLE_INCIDENCE_CREATE);
        // perms.add(Role.ROLE_INCIDENCE_CATEGORIES_TAB);
        // perms.add(Role.ROLE_INCIDENCE_REPORTS_TAB);
        // perms.add(Role.ROLE_INCIDENCE_SELECT_ALL_CATEGORIES);
        // perms.add(Role.ROLE_INCIDENCE_CATEGORIES_CREATE);
        // perms.add(Role.ROLE_INCIDENCE_CATEGORIES_EDIT);
        // perms.add(Role.ROLE_INCIDENCE_SELECT_OWN_CATEGORIES);
        // perms.add(Role.ROLE_INCIDENCE_VIEW);
        // perms.add(Role.ROLE_INCIDENCE_VIEW_ALL);
        // perms.add(Role.ROLE_INCIDENCE_FUNDETEC);
    	// perms.add(Role.ROLE_INCIDENCE_RECEIVE_MAIL);

        /** Parámetros de configuración **/
        // perms.add(Role.ROLE_PARAMETERS_VIEW);
    	  
        /** Financiero **/
    	// perms.add(Role.ROLE_FINANCIAL_VIEW);
    	// perms.add(Role.ROLE_FINANCIAL_ADMIN); */

        /** Informes automaticos **/
        // perms.add(Role.ROLE_SEND_AUTOMATIC_FORMS_VIEW);
        // perms.add(Role.ROLE_SEND_AUTOMATIC_FORMS_EDIT);
    	  
        /** Cuadro de mandos **/
        // perms.add(Role.ROLE_CUADRO_VIEW);
        // perms.add(Role.ROLE_CUADRO_EDIT);
        // perms.add(Role.ROLE_CUADRO_CENTRO_OWN);
        // perms.add(Role.ROLE_ALARM_EDIT);
    	  
        /** WIKI **/
        // perms.add(Role.ROLE_WIKI_BURGOS);
        // perms.add(Role.ROLE_WIKI_PALENCIA);
        // perms.add(Role.ROLE_WIKI_LEON);
        // perms.add(Role.ROLE_WIKI_ZAMORA);
        // perms.add(Role.ROLE_WIKI_VALLADOLID);
        // perms.add(Role.ROLE_WIKI_SORIA);
        // perms.add(Role.ROLE_WIKI_SEGOVIA);
        // perms.add(Role.ROLE_WIKI_SALAMANCA);
        // perms.add(Role.ROLE_WIKI_AVILA);
        // perms.add(Role.ROLE_WIKI_GENERAL);
        // perms.add(Role.QUALITE);
        // perms.add(Role.ADMIN);
    	  
        /** Contratos **/
    	// perms.add(Role.ROLE_CONTRACT_CREATE);
    	// perms.add(Role.ROLE_CONTRACT_VIEW);
    	// perms.add(Role.ROLE_CONTRACT_EDIT);
    	// perms.add(Role.ROLE_CONTRACT_DELETE);
        
        /** Paneles **/
        // perms.add(Role.ROLE_PANEL_VIEW);
        // perms.add(Role.ROLE_PANEL_CREATE_PANEL);
        // perms.add(Role.ROLE_PANEL_CREATE_NEWS);
        // perms.add(Role.ROLE_PANEL_CREATE_EVENTS);
        // perms.add(Role.ROLE_PANEL_VIEW_ALLPANEL);
        // perms.add(Role.ROLE_PANEL_EDIT_ALLPANEL);
        // perms.add(Role.ROLE_PANEL_UPLOAD_FILES);
    	  
        /** Notificaciones **/
    	// perms.add(Role.ROLE_NOTIFICATIONS_VIEW);

        //break;
      //case ROLE_ONLINE_TRAINER:
        /** Centros **/
        // perms.add(Role.ROLE_CENTER_VIEW_OWN);
        // perms.add(Role.ROLE_CENTER_VIEW_ALL);
        // perms.add(Role.ROLE_CENTER_EDIT_ALL);
        // perms.add(Role.ROLE_CENTER_EDIT_OWN);
        // perms.add(Role.ROLE_CENTER_DELETE);
        // perms.add(Role.ROLE_CENTER_CREATE);
        // perms.add(Role.ROLE_CENTER_AULA_VIEW);
        // perms.add(Role.ROLE_CENTER_AULA_EDIT);
        // perms.add(Role.ROLE_CENTER_EDIT_URLDAP);
        // perms.add(Role.ROLE_CENTER_FESTIVOS_VIEW);
        // perms.add(Role.ROLE_CENTER_FESTIVOS_EDIT);
        // perms.add(Role.ROLE_CENTER_FESTIVOS_DELETE);
        // perms.add(Role.ROLE_CENTER_FESTIVOS_CREATE);
    	// perms.add(Role.ROLE_CENTER_CALENDARIO_VIEW);
    	  
        /** Usuarios **/
    	// perms.add(Role.ROLE_USER_VIEW);
        // perms.add(Role.ROLE_USER_VIEW_ALL_CENTER);
    	// perms.add(Role.ROLE_USER_PRE_DELETE);
    	// perms.add(Role.ROLE_USER_EDIT);
        // perms.add(Role.ROLE_USER_DELETE);
    	// perms.add(Role.ROLE_USER_VIEW_EVALUATIONS);
    	// perms.add(Role.ROLE_USER_ONLINE);
    	  
        /** Servicios **/
    	// perms.add(Role.ROLE_SERVICE_VIEW);
        // perms.add(Role.ROLE_SERVICE_EDIT);
        // perms.add(Role.ROLE_SERVICE_DELETE);
    	// perms.add(Role.ROLE_SERVICE_EDIT_REGISTER_USER_OWN);
        // perms.add(Role.ROLE_SERVICE_EDIT_REGISTER_USER_ALL);
        // perms.add(Role.ROLE_SERVICE_PROPUESTAS_EDIT);
    	// perms.add(Role.ROLE_SERVICE_ONLINE);
    	  
        /** Actividades **/
    	// perms.add(Role.ROLE_ACTIVITY_VIEW);
    	// perms.add(Role.ROLE_ACTIVITY_EDIT);
        // perms.add(Role.ROLE_ACTIVITY_DELETE);
    	// perms.add(Role.ROLE_ACTIVITY_REGISTER_USER);
        // perms.add(Role.ROLE_ACTIVITY_TRAINER_VIEW_ALL);
    	// perms.add(Role.ROLE_ACTIVITY_TRAINER_VIEW_OWN);
    	// perms.add(Role.ROLE_ACTIVITY_ANNOTATION_ABOUT_SERVICE);
    	// perms.add(Role.ROLE_ACTIVITY_ONLINE);
    	  
        /** Personal **/
    	// perms.add(Role.ROLE_PERSONAL_VIEW_OWN);
        // perms.add(Role.ROLE_PERSONAL_VIEW_ALL);
        // perms.add(Role.ROLE_PERSONAL_EDIT);
        // perms.add(Role.ROLE_PERSONAL_DELETE);
        // perms.add(Role.ROLE_PERSONAL_TYPE);
        // perms.add(Role.ROLE_PERSONAL_PERMISSION);
        // perms.add(Role.ROLE_PERSONAL_CHANGE_PASSWORD);
        // perms.add(Role.ROLE_PERSONAL_EDIT_VACACIONES);
        // perms.add(Role.ROLE_PERSONAL_EDIT_RESPONSABLEINCIDENCIAS);
    	  
        /** Proveedores **/
        // perms.add(Role.ROLE_SUPPLIER_CREATE);
        // perms.add(Role.ROLE_SUPPLIER_VIEW);
        // perms.add(Role.ROLE_SUPPLIER_EDIT);
        // perms.add(Role.ROLE_SUPPLIER_DELETE);
    	  
        /** Evaluaciones **/
    	// perms.add(Role.ROLE_EVALUATION_NEW);
    	// perms.add(Role.ROLE_EVALUATION_VIEW);
    	// perms.add(Role.ROLE_EVALUATION_EDIT);
    	  
        /** Recursos **/
    	// perms.add(Role.ROLE_RECURSO_NEW);
    	// perms.add(Role.ROLE_RECURSO_ORDENADOR_ON_OFF);
    	// perms.add(Role.ROLE_RECURSO_VIEW);
    	// perms.add(Role.ROLE_RECURSO_EDIT_ORDENADOR);
    	// perms.add(Role.ROLE_RECURSO_EDIT_ROUTER);
    	// perms.add(Role.ROLE_RECURSO_EDIT_RECURSO);
    	// perms.add(Role.ROLE_RECURSO_VIEW_CATEGORIA);
    	// perms.add(Role.ROLE_RECURSO_EDIT_CATEGORIA);
        // perms.add(Role.ROLE_RECURSO_SEND_USER_COMMAND);
        // perms.add(Role.ROLE_RECURSO_SEND_ROOT_COMMAND);
    	// perms.add(Role.ROLE_RECURSO_VIEW_RESERVA);
    	// perms.add(Role.ROLE_RECURSO_EDIT_RESERVA);
        // perms.add(Role.ROLE_RECURSO_VIEW_AVERIAS);
        // perms.add(Role.ROLE_RECURSO_EDIT_AVERIAS);
        // perms.add(Role.ROLE_RECURSO_VIEW_PEDIDOS);
        // perms.add(Role.ROLE_RECURSO_EDIT_PEDIDOS);
    	  
        /** Incidencias **/
    	// perms.add(Role.ROLE_INCIDENCE_VIEW_PAGE);
    	// perms.add(Role.ROLE_INCIDENCE_CREATE);
        // perms.add(Role.ROLE_INCIDENCE_CATEGORIES_TAB);
        // perms.add(Role.ROLE_INCIDENCE_CATEGORIES_CREATE);
        // perms.add(Role.ROLE_INCIDENCE_CATEGORIES_EDIT);
        // perms.add(Role.ROLE_INCIDENCE_REPORTS_TAB);
        // perms.add(Role.ROLE_INCIDENCE_SELECT_ALL_CATEGORIES);
    	// perms.add(Role.ROLE_INCIDENCE_SELECT_OWN_CATEGORIES);
    	// perms.add(Role.ROLE_INCIDENCE_VIEW);
        // perms.add(Role.ROLE_INCIDENCE_VIEW_ALL);
        // perms.add(Role.ROLE_INCIDENCE_FUNDETEC);
    	  
        /** Parámetros de configuración **/
        // perms.add(Role.ROLE_PARAMETERS_VIEW);
    	  
        /** Financiero **/
        // perms.add(Role.ROLE_FINANCIAL_VIEW);
        // perms.add(Role.ROLE_FINANCIAL_ADMIN);
    	  
        /** Informes automaticos **/
        // perms.add(Role.ROLE_SEND_AUTOMATIC_FORMS_VIEW);
        // perms.add(Role.ROLE_SEND_AUTOMATIC_FORMS_EDIT);
    	  
        /** Cuadro de mandos **/
        // perms.add(Role.ROLE_CUADRO_VIEW);
        // perms.add(Role.ROLE_CUADRO_EDIT);
        // perms.add(Role.ROLE_CUADRO_CENTRO_OWN);
        // perms.add(Role.ROLE_ALARM_EDIT);
    	  
        /** WIKI **/
        // perms.add(Role.ROLE_WIKI_BURGOS);
        // perms.add(Role.ROLE_WIKI_PALENCIA);
        // perms.add(Role.ROLE_WIKI_LEON);
        // perms.add(Role.ROLE_WIKI_ZAMORA);
        // perms.add(Role.ROLE_WIKI_VALLADOLID);
        // perms.add(Role.ROLE_WIKI_SORIA);
        // perms.add(Role.ROLE_WIKI_SEGOVIA);
        // perms.add(Role.ROLE_WIKI_SALAMANCA);
        // perms.add(Role.ROLE_WIKI_AVILA);
    	// perms.add(Role.ROLE_WIKI_GENERAL);
    	  
        // perms.add(Role.QUALITE);
        // perms.add(Role.ADMIN);
    	  
        /** Contratos **/
        // perms.add(Role.ROLE_CONTRACT_CREATE);
        // perms.add(Role.ROLE_CONTRACT_VIEW);
        // perms.add(Role.ROLE_CONTRACT_EDIT);
        // perms.add(Role.ROLE_CONTRACT_DELETE);
    	  
        /** Paneles **/
        // perms.add(Role.ROLE_PANEL_VIEW);
        // perms.add(Role.ROLE_PANEL_CREATE_PANEL);
    	// perms.add(Role.ROLE_PANEL_CREATE_NEWS);
    	// perms.add(Role.ROLE_PANEL_CREATE_EVENTS);
        // perms.add(Role.ROLE_PANEL_VIEW_ALLPANEL);
        // perms.add(Role.ROLE_PANEL_EDIT_ALLPANEL);
    	// perms.add(Role.ROLE_PANEL_UPLOAD_FILES);
    	  
        /** Notificaciones **/
        // perms.add(Role.ROLE_NOTIFICATIONS_VIEW);
        //break;
      
      case ROLE_DIGITAL_ALLY:
          /** Centros **/    	  
          perms.add(Role.ROLE_CENTER_VIEW_OWN);
          //perms.add(Role.ROLE_CENTER_EDIT_OWN);
          perms.add(Role.ROLE_CENTER_AULA_VIEW);
          //perms.add(Role.ROLE_CENTER_AULA_EDIT);
          perms.add(Role.ROLE_CENTER_FESTIVOS_VIEW);
          //perms.add(Role.ROLE_CENTER_FESTIVOS_EDIT);
          //perms.add(Role.ROLE_CENTER_FESTIVOS_DELETE);
          //perms.add(Role.ROLE_CENTER_FESTIVOS_CREATE);
          perms.add(Role.ROLE_CENTER_CALENDARIO_VIEW);
          
          /** Usuarios **/
          //perms.add(Role.ROLE_USER_VIEW);
          perms.add(Role.ROLE_USER_VIEW_ALL_CENTER);
          
          /** Servicios **/
          perms.add(Role.ROLE_SERVICE_VIEW);
          perms.add(Role.ROLE_SERVICE_EDIT_REGISTER_USER_OWN);
          perms.add(Role.ROLE_SERVICE_PROPUESTAS_EDIT);
          
          /** Actividades **/
          perms.add(Role.ROLE_ACTIVITY_VIEW);
          perms.add(Role.ROLE_ACTIVITY_EDIT);
          perms.add(Role.ROLE_ACTIVITY_INTERNAL_REQUEST);
          perms.add(Role.ROLE_ACTIVITY_EXTERNAL_APPROVE);
          perms.add(Role.ROLE_ACTIVITY_DELETE);
          perms.add(Role.ROLE_ACTIVITY_REGISTER_USER);
          perms.add(Role.ROLE_ACTIVITY_TRAINER_VIEW_ALL);
          perms.add(Role.ROLE_ACTIVITY_TRAINER_VIEW_OWN);
          perms.add(Role.ROLE_ACTIVITY_ANNOTATION_ABOUT_SERVICE);
          
          /** Diplomas **/
          perms.add(Role.ROLE_DIPLOMA_VIEW);
          // perms.add(Role.ROLE_DIPLOMA_EDIT);
          // perms.add(Role.ROLE_DIPLOMA_DELETE);
          
          /** Exámenes **/
          perms.add(Role.ROLE_EXAM_VIEW);
          
          /** Personal **/
          perms.add(Role.ROLE_PERSONAL_VIEW_OWN);
          perms.add(Role.ROLE_PERSONAL_DELETE);
          perms.add(Role.ROLE_PERSONAL_TYPE);          
          perms.add(Role.ROLE_PERSONAL_EDIT_VACACIONES);
          perms.add(Role.ROLE_PERSONAL_CHANGE_PASSWORD);
          
          /** Proveedores **/
          //perms.add(Role.ROLE_SUPPLIER_VIEW);
          
          /** Evaluaciones **/
          perms.add(Role.ROLE_EVALUATION_NEW);
          perms.add(Role.ROLE_EVALUATION_VIEW);
          perms.add(Role.ROLE_EVALUATION_EDIT);
          
          /** Recursos **/
          perms.add(Role.ROLE_RECURSO_NEW);
          perms.add(Role.ROLE_RECURSO_ORDENADOR_ON_OFF);
          perms.add(Role.ROLE_RECURSO_VIEW);
          perms.add(Role.ROLE_RECURSO_EDIT_ORDENADOR);
          perms.add(Role.ROLE_RECURSO_EDIT_ROUTER);
          perms.add(Role.ROLE_RECURSO_EDIT_RECURSO);
          perms.add(Role.ROLE_RECURSO_VIEW_CATEGORIA);
          perms.add(Role.ROLE_RECURSO_EDIT_CATEGORIA);
          perms.add(Role.ROLE_RECURSO_VIEW_RESERVA);
          perms.add(Role.ROLE_RECURSO_EDIT_RESERVA);
          perms.add(Role.ROLE_RECURSO_VIEW_AVERIAS);
          perms.add(Role.ROLE_RECURSO_EDIT_AVERIAS);
          perms.add(Role.ROLE_RECURSO_VIEW_PEDIDOS);
          perms.add(Role.ROLE_RECURSO_EDIT_PEDIDOS);
          
          /** Informes automaticos **/
          //perms.add(Role.ROLE_SEND_AUTOMATIC_FORMS_VIEW);
          //perms.add(Role.ROLE_SEND_AUTOMATIC_FORMS_EDIT);
          
          /** Cuadro de mandos **/
          //perms.add(Role.ROLE_CUADRO_CENTRO_OWN);
          //perms.add(Role.ROLE_CUADRO_VIEW);
          //perms.add(Role.ROLE_ALARM_EDIT);
          break;
 
      default:
        break;
    }
    return perms;
  }

  /**
   * Devuelve informes.
   * 
   * @return informes
   */
  @NotAudited
  @ManyToMany(fetch = FetchType.LAZY, cascade = { CascadeType.ALL }, mappedBy = "empleados", targetEntity = InformeNotification.class)
  public List<InformeNotification> getInformes() {
    return informes;
  }

  /**
   * Asigna informes.
   * 
   * @param informes
   *          informes
   */
  public void setInformes(List<InformeNotification> informes) {
    this.informes = informes;
  }

  /**
   * Devuelve alarmas.
   * 
   * @return alarmas
   */
  @NotAudited
  @ManyToMany(fetch = FetchType.LAZY, cascade = { CascadeType.ALL }, mappedBy = "empleados", targetEntity = Alarma.class)
  public List<Alarma> getAlarmas() {
    return alarmas;
  }

  /**
   * Asigna alarmas.
   * 
   * @param alarmas
   *          alarmas
   */
  public void setAlarmas(List<Alarma> alarmas) {
    this.alarmas = alarmas;
  }

  /**
   * Devuelve observaciones.
   * 
   * @return observaciones
   */
  @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getObservaciones() {
    return observaciones;
  }

  /**
   * Asigna observaciones.
   * 
   * @param observaciones
   *          observaciones
   */
  public void setObservaciones(String observaciones) {
    this.observaciones = observaciones;
  }

  /**
   * Devuelve cursos online.
   * 
   * @return cursos online
   */
  @NotAudited
  @ManyToMany(fetch = FetchType.LAZY, cascade = { CascadeType.ALL }, mappedBy = "profesores", targetEntity = CursoOnline.class)
  public List<CursoOnline> getCursosOnline() {
    return cursosOnline;
  }

  /**
   * Asigna cursos online.
   * 
   * @param cursosOnline
   *          cursos online
   */
  public void setCursosOnline(List<CursoOnline> cursosOnline) {
    this.cursosOnline = cursosOnline;
  }

  /**
   * Devuelve id web.
   * 
   * @return id web
   */
  public String getIdWeb() {
    return idWeb;
  }

  /**
   * Asigna id web.
   * 
   * @param idWeb
   *          id web
   */
  public void setIdWeb(String idWeb) {
    this.idWeb = idWeb;
  }

  /**
   * Devuelve username web.
   * 
   * @return username web
   */
  public String getUsernameWeb() {
    return usernameWeb;
  }

  /**
   * Asigna username web.
   * 
   * @param usernameWeb
   *          username web
   */
  public void setUsernameWeb(String usernameWeb) {
    this.usernameWeb = usernameWeb;
  }
  
  /**
   * Devuelve el password
   * @return password
   */
  public String getPassword() {
    return password;
  }

  /**
   * Asigna el password
   * @param password password
   */
  public void setPassword(String password) {
    this.password = password;
  }

  /**
   * Asigna el indicador de cambio obligatorio de contraseña
   * @return
   */
  public boolean isCambioObligatorioPswdLDAP() {
	  return cambioObligatorioPswdLDAP;
  }

  /**
   * Devuelve el indicador de cambio obligatorio de contraseña
   * @param cambioObligatorioPswdLDAP
   */
  public void setCambioObligatorioPswdLDAP(boolean cambioObligatorioPswdLDAP) {
	  this.cambioObligatorioPswdLDAP = cambioObligatorioPswdLDAP;
  }

  public String getHistoricoPswdLDAP() {
	  return historicoPswdLDAP;
  }

  public void setHistoricoPswdLDAP(String historicoPswdLDAP) {
	  this.historicoPswdLDAP = historicoPswdLDAP;
  }
 
  /**
   * Devuelve una lista con el historico del hash de las contraseñas
   * @return ArrayList<String>
   */
  @Transient
  public ArrayList<String> getListHistoricoPswdLDAP(){
	  ArrayList<String> result = new ArrayList<String>();
	  
	  String sHistoricoPswdLDAP = getHistoricoPswdLDAP();
	  if(sHistoricoPswdLDAP!=null){
		  result = new ArrayList<String>(Arrays.asList(sHistoricoPswdLDAP.split("(?<=\\G.{40})"))); 
	  }

	  return result;
  }
  
  public void addHistoricoPswdLDAP(String sHash){
	  ArrayList<String> listPswd = this.getListHistoricoPswdLDAP();
	  listPswd.add(sHash);
	  String sHistoricoCompleto = StringUtils.join(listPswd.iterator(), "");
	  if(sHistoricoCompleto!=null){
		  if (sHistoricoCompleto.length() > 200)
			  sHistoricoCompleto = sHistoricoCompleto.substring(sHistoricoCompleto.length()-200);
		  this.setHistoricoPswdLDAP(sHistoricoCompleto);
	  }
  }
 
}
