package es.csa.fdt.cyldigital.usuarios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Length;

import es.csa.commons.util.FileUtils;
import es.csa.fdt.cyldigital.core.entity.AbstractEntity;

/**
 * Clase Fichero.
 * 
 * @author csa
 */
@Entity
@Table(name = "ficheros")
@AttributeOverride(name = "id", column = @Column(name = "idFichero"))
@Audited
public class Fichero extends AbstractEntity implements Serializable {
  private static final long serialVersionUID = -3156935382182399591L;

  private String nombre;

  private String tipoMime;

  private Integer tamanio;

  private String path;

  /**
   * Devuelve nombre.
   * 
   * @return nombre
   */
  @Length(max = 255)
  public String getNombre() {
    return nombre;
  }

  /**
   * Asigna nombre.
   * 
   * @param nombreFichero
   *          nombre
   */
  public void setNombre(String nombreFichero) {
    this.nombre = nombreFichero;
  }

  /**
   * Devuelve tipo mime.
   * 
   * @return tipo mime
   */
  @Length(max = 150)
  public String getTipoMime() {
    return tipoMime;
  }

  /**
   * Asigna tipo mime.
   * 
   * @param tipoMime
   *          tipo mime
   */
  public void setTipoMime(String tipoMime) {
    this.tipoMime = tipoMime;
  }

  /**
   * Devuelve tamanio.
   * 
   * @return tamanio
   */
  public Integer getTamanio() {
    return tamanio;
  }

  /**
   * Asigna tamanio.
   * 
   * @param tamanio
   *          tamanio
   */
  public void setTamanio(Integer tamanio) {
    this.tamanio = tamanio;
  }

  /**
   * Devuelve path.
   * 
   * @return path
   */
  public String getPath() {
    return path;
  }

  /**
   * Asigna path.
   * 
   * @param path
   *          path
   */
  public void setPath(String path) {
    this.path = path;
  }

  /**
   * Devuelve tamanio legible.
   * 
   * @return tamanio legible
   */
  @Transient
  public String getTamanioLegible() {
    return FileUtils.getHumanReadableSize(Long.valueOf(getTamanio()));
  }
}
