package es.csa.fdt.cyldigital.usuarios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OrderBy;
import javax.persistence.Table;

import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.hibernate.envers.Audited;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;

/**
 * Clase Localidad.
 * 
 * @author csa
 */
@Entity
@Cache(usage = CacheConcurrencyStrategy.READ_ONLY)
@Table(name = "localidades")
@AttributeOverride(name = "id", column = @Column(name = "idLocalidad"))
@Audited
public class Localidad extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = -7648562176655166537L;

  private String nombre;

  private Provincia provincia;

  private List<CodigoPostal> codigosPostales;

  /**
   * Construye un nuevo objeto de la clase
   */
  public Localidad() {
    super();
  }

  /**
   * Devuelve nombre.
   * 
   * @return nombre
   */
  @Length(max = 255)
  @NotEmpty
  public String getNombre() {
    return nombre;
  }

  /**
   * Asigna nombre.
   * 
   * @param nombre
   *          nombre
   */
  public void setNombre(String nombre) {
    this.nombre = nombre;
  }

  /**
   * Devuelve provincia.
   * 
   * @return provincia
   */
  @ManyToOne(fetch = FetchType.LAZY)
  @JoinColumn(name = "idProvincia")
  @NotNull
  public Provincia getProvincia() {
    return provincia;
  }

  /**
   * Asigna provincia.
   * 
   * @param provincia
   *          provincia
   */
  public void setProvincia(Provincia provincia) {
    this.provincia = provincia;
  }

  /**
   * Devuelve codigos postales.
   * 
   * @return codigos postales
   */
  @ManyToMany(targetEntity = CodigoPostal.class, cascade = { javax.persistence.CascadeType.ALL })
  @JoinTable(name = "localidades_codigospostales", joinColumns = @JoinColumn(name = "idLocalidad"), inverseJoinColumns = @JoinColumn(name = "idCodigoPostal"))
  @OrderBy(value = "codigo ASC")
  public List<CodigoPostal> getCodigosPostales() {
    return codigosPostales;
  }

  /**
   * Asigna codigos postales.
   * 
   * @param codigosPostales
   *          codigos postales
   */
  public void setCodigosPostales(List<CodigoPostal> codigosPostales) {
    this.codigosPostales = codigosPostales;
  }
}
