package es.csa.fdt.cyldigital.usuarios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Email;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.financiero.entity.Contrato;
import es.csa.fdt.cyldigital.financiero.entity.Movimiento;

/**
 * Clase Proveedor.
 * 
 * @author csa
 */
@Entity
@Table(name = "proveedores")
@AttributeOverride(name = "id", column = @Column(name = "idProveedor"))
@Audited
public class Proveedor extends AbstractEntity implements Serializable, Comparable<Proveedor> {

  private static final long serialVersionUID = -7648562176655166537L;

   
  /**
   * Enum TipoEmpresa.
   */
  public static enum TipoEmpresa {	
	AUTONOMO,
	ASOCIACION,
	COLEGIO,
	EMPRESA,
	UNIVESIDAD,
	CENTRO,
	OTROS,
  }
  
  /**
   * Enum Sector.
   */
  public static enum Sector {
	  TODOS1,
	  AGRIC3,
	  FORES5,
	  ACUIC7,
	  PIEDR9,
	  ALIME11,
	  ALIME12,
	  ALIME13,
	  ALIME14,
	  ALIME15,
	  ALIME16,
	  TEXTI18,
	  MADER20,
	  MADER21,
	  MADER22,
	  MADER23,
	  ARTES25,
	  INDUS27,
	  INDUS29,
	  METAL31,
	  FABRI33,
	  MAQUI35,
	  AUTOM37,
	  AUTOM38,
	  AUTOM39,
	  AUTOM40,
	  NAVAL42,
	  NAVAL43,
	  NAVAL44,
	  ENERG46,
	  AGUAS48,
	  MEDIO50,
	  CONST52,
	  VENTA54,
	  PEQUE56,
	  TRANS58,
	  TURIS60,
	  ACTIV62,
	  ACTIV64,
	  ACTIV65,
	  ACTIV66,
	  ACTIV67,
	  ACTIV68,
	  ACTIV69,
	  ADMIN71,
	  EDUCA73,
	  ACTIV75,
	  ACTIV76,
	  ACTIV77,
	  INDUS79,
	  INDUS80,
	  OTROS82,
	  OTROS83,
	  OTROS84,
	  OTROS85,
	  MANUF87,
	  MANUF88,
	  MANUF89,
	  MANUF90,
	  MANUF91,
	  COMER93,
	  TELEC95,
	  TELEC96,
	  TELEC97,
	  TELEC98,
	  ACTIV100,
	  ACTIV101,
	  OTROS103,
	  OTROS104,
	  AUDIO106,
	  AUDIO107,
	  AUDIO108,
	  AUDIO109,
	  AUDIO110,
	  AUDIO111,
	  OTROS113,
  }

  private String nombre;
  private String cif;
  private String direccion;
  private String cp;
  private Provincia provincia;
  private Localidad localidad;
  private String telefono;
  private String fax;
  private String email;

  private String comentarios;

 
  private TipoEmpresa tipoEmpresa;
  private Sector sector;

  private List<Empleado> empleados;
  private List<Contrato> contratos;
  private List<Movimiento> movimientos;

  /**
   * Construye un nuevo objeto de la clase
   */
  public Proveedor() {
    super();
    empleados = new ArrayList<Empleado>();
    movimientos = new ArrayList<Movimiento>();
  }
  
  /**
   * Define el método de comparación para ordenar y buscar por Id
   */
  @Transient
  public int compareTo(Proveedor o) {
	  if ((this.getId() == o.getId()) && (this.getId() == 0))
	  		return (this.getNombre().compareTo(o.getNombre()));
	  else
	  	return ((Long)this.getId()).compareTo((Long)o.getId());
  }

  /**
   * Devuelve nombre.
   * 
   * @return nombre
   */
  @Length(max = 50)
  @NotEmpty
  @NotNull
  public String getNombre() {
    return nombre;
  }

  /**
   * Asigna nombre.
   * 
   * @param nombre
   *          nombre
   */
  public void setNombre(String nombre) {
    this.nombre = nombre;
  }

  /**
   * Devuelve cif.
   * 
   * @return cif
   */
  @Length(max = 20)
  public String getCif() {
    return cif;
  }

  /**
   * Asigna cif.
   * 
   * @param cif
   *          cif
   */
  public void setCif(String cif) {
    this.cif = cif;
  }

  /**
   * Devuelve direccion.
   * 
   * @return direccion
   */
  @Length(max = 150)
  public String getDireccion() {
    return direccion;
  }

  /**
   * Asigna direccion.
   * 
   * @param direccion
   *          direccion
   */
  public void setDireccion(String direccion) {
    this.direccion = direccion;
  }

  /**
   * Devuelve cp.
   * 
   * @return cp
   */
  public String getCp() {
    return cp;
  }

  /**
   * Asigna cp.
   * 
   * @param cp
   *          cp
   */
  public void setCp(String cp) {
    this.cp = cp;
  }

  /**
   * Devuelve provincia.
   * 
   * @return provincia
   */
  @ManyToOne(fetch = FetchType.LAZY)
  @JoinColumn(name = "idProvincia")
  @NotNull
  public Provincia getProvincia() {
    return provincia;
  }

  /**
   * Asigna provincia.
   * 
   * @param provincia
   *          provincia
   */
  public void setProvincia(Provincia provincia) {
    this.provincia = provincia;
  }

  /**
   * Devuelve localidad.
   * 
   * @return localidad
   */
  @ManyToOne(fetch = FetchType.LAZY)
  @JoinColumn(name = "idLocalidad")
  @NotNull
  public Localidad getLocalidad() {
    return localidad;
  }

  /**
   * Asigna localidad.
   * 
   * @param localidad
   *          localidad
   */
  public void setLocalidad(Localidad localidad) {
    this.localidad = localidad;
  }

  /**
   * Devuelve telefono.
   * 
   * @return telefono
   */
  @Length(max = 20)
  @NotNull
  public String getTelefono() {
    return telefono;
  }

  /**
   * Asigna telefono.
   * 
   * @param telefono
   *          telefono
   */
  public void setTelefono(String telefono) {
    this.telefono = telefono;
  }

  /**
   * Devuelve fax.
   * 
   * @return fax
   */
  @Length(max = 20)
  public String getFax() {
    return fax;
  }

  /**
   * Asigna fax.
   * 
   * @param fax
   *          fax
   */
  public void setFax(String fax) {
    this.fax = fax;
  }

  /**
   * Devuelve email.
   * 
   * @return email
   */
  @Length(max = 255)
  @Email
  public String getEmail() {
    return email;
  }

  /**
   * Asigna email.
   * 
   * @param email
   *          email
   */
  public void setEmail(String email) {
    this.email = email;
  }
  
  /**
   * Devuelve tipoempresa.
   * 
   * @return tipoempresa
   */
  @Enumerated(value = EnumType.STRING)
  @Column(nullable = false)
  @NotNull
  public TipoEmpresa getTipoEmpresa() {
    return tipoEmpresa;
  }

  /**
   * Asigna tipoempresa.
   * 
   * @param tipoempresa
   *          tipoempresa
   */
  public void setTipoEmpresa(TipoEmpresa tipoEmpresa) {
    this.tipoEmpresa = tipoEmpresa;
  }
  
  
  /**
   * Devuelve sector.
   * 
   * @return sector
   */
  @Enumerated(value = EnumType.STRING)
  @Column(nullable = false)
  @NotNull
  public Sector getSector() {
    return sector;
  }

  /**
   * Asigna sector.
   * 
   * @param sector
   *          sector
   */
  public void setSector(Sector sector) {
    this.sector = sector;
  }

  /**
   * Devuelve empleados.
   * 
   * @return empleados
   */
  @OneToMany(mappedBy = "proveedor")
  public List<Empleado> getEmpleados() {
    return empleados;
  }

  /**
   * Asigna empleados.
   * 
   * @param empleados
   *          empleados
   */
  public void setEmpleados(List<Empleado> empleados) {
    this.empleados = empleados;
  }

  /**
   * Devuelve comentarios.
   * 
   * @return comentarios
   */
  @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  public String getComentarios() {
    return comentarios;
  }

  /**
   * Asigna comentarios.
   * 
   * @param comentarios
   *          comentarios
   */
  public void setComentarios(String comentarios) {
    this.comentarios = comentarios;
  }

  /**
   * Devuelve contratos.
   * 
   * @return contratos
   */
  @OneToMany(mappedBy = "proveedor", cascade = CascadeType.ALL)
  public List<Contrato> getContratos() {
    return contratos;
  }

  /**
   * Asigna contratos.
   * 
   * @param contratos
   *          contratos
   */
  public void setContratos(List<Contrato> contratos) {
    this.contratos = contratos;
  }

  /**
   * Devuelve movimientos.
   * 
   * @return movimientos
   */
  @OneToMany(mappedBy = "proveedor")
  public List<Movimiento> getMovimientos() {
    return movimientos;
  }

  /**
   * Asigna movimientos.
   * 
   * @param movimientos
   *          movimientos
   */
  public void setMovimientos(List<Movimiento> movimientos) {
    this.movimientos = movimientos;
  }
}
