package es.csa.fdt.cyldigital.usuarios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.hibernate.annotations.Fetch;
import org.hibernate.annotations.FetchMode;
import org.hibernate.envers.Audited;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotEmpty;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;

/**
 * Clase Provincia.
 * 
 * @author csa
 */
@Entity
@Table(name = "provincias")
@AttributeOverride(name = "id", column = @Column(name = "idProvincia"))
@Audited
public class Provincia extends AbstractEntity implements Serializable, Comparable<Provincia> {

  private static final long serialVersionUID = -7648562176655166537L;

  private String nombre;

  private int codigo_provincia;

  private List<Localidad> localidades;

  /**
   * Construye un nuevo objeto de la clase
   */
  public Provincia() {
    super();
  }

  /**
   * Define el método de comparación para ordenar y buscar
   */
  @Transient
  public int compareTo(Provincia o) {
	  if ((this.getId() == o.getId()) && (this.getId() == 0))
	  		return (this.getNombre().compareTo(o.getNombre()));
	  else
	  	return ((Long)this.getId()).compareTo((Long)o.getId());
  }

  /**
   * Devuelve nombre.
   * 
   * @return nombre
   */
  @Length(max = 255)
  @NotEmpty
  public String getNombre() {
    return nombre;
  }

  /**
   * Asigna nombre.
   * 
   * @param nombre
   *          nombre
   */
  public void setNombre(String nombre) {
    this.nombre = nombre;
  }

  /**
   * Devuelve localidades.
   * 
   * @return localidades
   */
  @OneToMany(mappedBy = "provincia", cascade = { javax.persistence.CascadeType.ALL })
  @OrderBy(value = "nombre ASC")
  @Cache(usage = CacheConcurrencyStrategy.READ_ONLY)
  @Fetch(value = FetchMode.SELECT)
  public List<Localidad> getLocalidades() {
    return localidades;
  }

  /**
   * Asigna localidades.
   * 
   * @param localidades
   *          localidades
   */
  public void setLocalidades(List<Localidad> localidades) {
    this.localidades = localidades;
  }

  /**
   * Devuelve codigo_provincia.
   * 
   * @return codigo_provincia
   */
  public int getCodigo_provincia() {
    return codigo_provincia;
  }

  /**
   * Asigna codigo_provincia.
   * 
   * @param codigoProvincia
   *          codigo_provincia
   */
  public void setCodigo_provincia(int codigoProvincia) {
    codigo_provincia = codigoProvincia;
  }
}
