package es.csa.fdt.cyldigital.usuarios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.Date;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.envers.Audited;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;

/**
 * Clase RegistroLdap.
 * 
 * @author csa
 */
@Entity
@Table(name = "registrosldap")
@AttributeOverride(name = "id", column = @Column(name = "idRegistro"))
@Audited
public class RegistroLdap extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = -7648562176655166537L;

  private boolean correcto;

  private String mensaje;

  private Date fecha;

  private UsuarioCyL usuarioCyL;

  /**
   * Construye un nuevo objeto de la clase
   */
  public RegistroLdap() {
    super();
    setFecha(new Date());
  }

  /**
   * Construye un nuevo objeto de la clase
   * 
   * @param user
   *           user
   */
  public RegistroLdap(UsuarioCyL user) {
    this();
    setUsuario(user);
  }

  /**
   * Construye un nuevo objeto de la clase
   * 
   * @param user
   *           user
   * @param mensaje
   *           mensaje
   */
  public RegistroLdap(UsuarioCyL user, String mensaje) {
    this(user);
    setMensaje(mensaje);
  }

  /**
   * Construye un nuevo objeto de la clase
   * 
   * @param user
   *           user
   * @param mensaje
   *           mensaje
   * @param correcto
   *           correcto
   */
  public RegistroLdap(UsuarioCyL user, String mensaje, boolean correcto) {
    this(user, mensaje);
    setCorrecto(correcto);
  }

  /**
   * Checks if is correcto.
   * 
   * @return true, if is correcto
   */
  public boolean isCorrecto() {
    return correcto;
  }

  /**
   * Asigna correcto.
   * 
   * @param correcto
   *          correcto
   */
  public void setCorrecto(boolean correcto) {
    this.correcto = correcto;
  }

  /**
   * Devuelve mensaje.
   * 
   * @return mensaje
   */
  public String getMensaje() {
    return mensaje;
  }

  /**
   * Asigna mensaje.
   * 
   * @param mensaje
   *          mensaje
   */
  public void setMensaje(String mensaje) {
    this.mensaje = mensaje;
  }

  /**
   * Devuelve fecha.
   * 
   * @return fecha
   */
  @Temporal(TemporalType.TIMESTAMP)
  public Date getFecha() {
    return fecha;
  }

  /**
   * Asigna fecha.
   * 
   * @param fechaAlta
   *          fecha
   */
  public void setFecha(Date fechaAlta) {
    this.fecha = fechaAlta;
  }

  /**
   * Devuelve usuario.
   * 
   * @return usuario
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idUsuario")
  @NotNull
  public UsuarioCyL getUsuario() {
    return usuarioCyL;
  }

  /**
   * Asigna usuario.
   * 
   * @param usuarioCyL
   *          usuario
   */
  public void setUsuario(UsuarioCyL usuarioCyL) {
    this.usuarioCyL = usuarioCyL;
  }
}
