package es.csa.fdt.cyldigital.usuarios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.Date;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.envers.Audited;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;

/**
 * Clase RegistroSmartCard.
 * 
 * @author csa
 */
@Entity
@Table(name = "registrossmartcard")
@AttributeOverride(name = "id", column = @Column(name = "idSmartcard"))
@Audited
public class RegistroSmartCard extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = -7648562176655166537L;

  /**
   * Enum TipoRegistro.
   */
  public enum TipoRegistro {
    /** Grabación */
    G,
    /** Impresion */
    I;
  }

  private TipoRegistro tipoRegistro;

  private boolean correcto;

  private Date fecha;

  private UsuarioCyL usuarioCyL;

  /**
   * Construye un nuevo objeto de la clase
   */
  public RegistroSmartCard() {
    super();
  }

  /**
   * Checks if is correcto.
   * 
   * @return true, if is correcto
   */
  public boolean isCorrecto() {
    return correcto;
  }

  /**
   * Asigna correcto.
   * 
   * @param correcto
   *          correcto
   */
  public void setCorrecto(boolean correcto) {
    this.correcto = correcto;
  }

  /**
   * Devuelve fecha.
   * 
   * @return fecha
   */
  @Temporal(TemporalType.TIMESTAMP)
  public Date getFecha() {
    return fecha;
  }

  /**
   * Asigna fecha.
   * 
   * @param ultimaImpresion
   *          fecha
   */
  public void setFecha(Date ultimaImpresion) {
    this.fecha = ultimaImpresion;
  }

  /**
   * Devuelve usuario.
   * 
   * @return usuario
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idUsuario")
  @NotNull
  public UsuarioCyL getUsuario() {
    return usuarioCyL;
  }

  /**
   * Asigna usuario.
   * 
   * @param usuarioCyL
   *          usuario
   */
  public void setUsuario(UsuarioCyL usuarioCyL) {
    this.usuarioCyL = usuarioCyL;
  }

  /**
   * Devuelve tipo registro.
   * 
   * @return tipo registro
   */
  @Enumerated(EnumType.STRING)
  public TipoRegistro getTipoRegistro() {
    return tipoRegistro;
  }

  /**
   * Asigna tipo registro.
   * 
   * @param tipoRegistro
   *          tipo registro
   */
  public void setTipoRegistro(TipoRegistro tipoRegistro) {
    this.tipoRegistro = tipoRegistro;
  }
}
