package es.csa.fdt.cyldigital.usuarios.entity;

/*
 * #%L
 * Librerias del nucleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.persistence.UniqueConstraint;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;

import es.csa.fdt.cyldigital.calendario.entity.ActividadCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.AsistenciaCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.UserActivityCalendarItem;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.financiero.entity.Movimiento;
import es.csa.fdt.cyldigital.incidencias.entity.EventoIncidencia;
import es.csa.fdt.cyldigital.incidencias.entity.Incidencia;
import es.csa.fdt.cyldigital.notificacion.Notification;
import es.csa.fdt.cyldigital.notificacion.entity.MailNotification;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCharla;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaExterna;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaTaller;
import es.csa.fdt.cyldigital.servicios.entity.PreinscripcionCurso;
import es.csa.fdt.cyldigital.servicios.entity.PreinscripcionTaller;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL.TipoUsuario;

/**
 * Clase Usuario.
 * 
 * @author csa
 */
@Entity
@Table(name = "usuarios", uniqueConstraints = @UniqueConstraint(columnNames = { "idUsuario", "idCentro" }))
@Audited
@Inheritance(strategy = InheritanceType.JOINED)
@AttributeOverride(name = "id", column = @Column(name = "idUsuario"))
public abstract class Usuario extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = 5721862807183200675L;

  private String nombre;
  private String apellido1;
  private String apellido2;
  private String telefono;
  private int peso;
  
  /** movimientos. */
  protected List<Movimiento> movimientos;
  
  /** matriculas cursos. */
  protected List<MatriculaCurso> matriculasCursos;
  
  /** matriculas talleres. */
  protected List<MatriculaTaller> matriculasTalleres;
  
  /** matriculas externas. */
  protected List<MatriculaExterna> matriculasExternas;
  
  /** preinscripciones cursos. */
  protected List<PreinscripcionCurso> preinscripcionesCursos;
  
  /** preinscripciones talleres. */
  protected List<PreinscripcionTaller> preinscripcionesTalleres;
  
  /** matriculas charlas. */
  protected List<MatriculaCharla> matriculasCharlas;
  
  /** incidencias. */
  protected List<Incidencia> incidencias;
  
  /** estadoincidencias. */
  protected List<EventoIncidencia> estadoincidencias;
  
  /** comentario observaciones. */
  protected List<ComentarioObservacion> comentarioObservaciones;
  
  /** notificaciones. */
  protected List<Notification> notificaciones;
  
  /** notificaciones enviadas. */
  protected List<MailNotification> notificacionesEnviadas;

  private String nif;
  private String direccion;
  private String cp;
  private Localidad localidad;
  private Provincia provincia;

  private Centro centro;

  private List<Role> rolesList;
  private boolean overrideDefaultRoles = false;

  private List<UserActivityCalendarItem> userActivityCalendarItems;

  private List<ActividadCalendarItem> actividadCalendarItems;

  private List<AsistenciaCalendarItem> asistencias;

  /**
   * Enum Role.
   */
  public static enum Role {
    // ROLES GENERALES (NO UTILIZAR SI ES POSIBLE)
    ROLE_ADMIN,
    //ROLE_TRAINER_COORDINATOR,
    ROLE_AREA_MANAGER,
    //ROLE_MANAGER,
    //ROLE_INFORMER,
    ROLE_TRAINER,
    //ROLE_ONLINE_TRAINER,
    //ROLE_012,
    ROLE_USER,
    ROLE_WS,
    ROLE_EXTERN,
    //ROLE_FINANCIAL_MANAGER,
    ROLE_DIGITAL_ALLY,
    
    // Centros
    ROLE_CENTER_VIEW_OWN,
    ROLE_CENTER_VIEW_ALL,
    ROLE_CENTER_EDIT_OWN,
    ROLE_CENTER_EDIT_ALL,
    ROLE_CENTER_DELETE,
    ROLE_CENTER_CREATE,
    ROLE_CENTER_AULA_VIEW,
    ROLE_CENTER_AULA_EDIT,
    ROLE_CENTER_EDIT_URLDAP,
    ROLE_CENTER_FESTIVOS_VIEW,
    ROLE_CENTER_FESTIVOS_EDIT,
    ROLE_CENTER_FESTIVOS_DELETE,
    ROLE_CENTER_FESTIVOS_CREATE,
    ROLE_CENTER_CALENDARIO_VIEW,
    
    // Usuarios
    ROLE_USER_VIEW,
    ROLE_USER_VIEW_ALL_CENTER,
    ROLE_USER_VIEW_EVALUATIONS,
    ROLE_USER_EDIT,
    ROLE_USER_DELETE,
    ROLE_USER_PRE_DELETE,
    ROLE_USER_ONLINE,
    
    // Servicios
    ROLE_SERVICE_VIEW,
    ROLE_SERVICE_EDIT,
    ROLE_SERVICE_DELETE,
    ROLE_SERVICE_EDIT_REGISTER_USER_OWN,
    ROLE_SERVICE_EDIT_REGISTER_USER_ALL,
    ROLE_SERVICE_PROPUESTAS_EDIT,
    ROLE_SERVICE_ONLINE,
    
    // Actividades
    ROLE_ACTIVITY_VIEW,
    ROLE_ACTIVITY_EDIT,
    ROLE_ACTIVITY_INTERNAL_REQUEST,
    ROLE_ACTIVITY_EXTERNAL_REQUEST,
    ROLE_ACTIVITY_INTERNAL_APPROVE,
    ROLE_ACTIVITY_EXTERNAL_APPROVE,
    ROLE_ACTIVITY_DELETE,
    ROLE_ACTIVITY_REGISTER_USER,
    ROLE_ACTIVITY_TRAINER_VIEW_ALL,
    ROLE_ACTIVITY_TRAINER_VIEW_OWN,
    ROLE_ACTIVITY_ANNOTATION_ABOUT_SERVICE,
    ROLE_ACTIVITY_ONLINE,
    
    // Diplomas
    ROLE_DIPLOMA_VIEW,
    ROLE_DIPLOMA_EDIT,
    ROLE_DIPLOMA_DELETE,
    
    // Exámenes
    ROLE_EXAM_VIEW,
    ROLE_EXAM_EDIT,
    ROLE_EXAM_DELETE,
    
    // Personal
    ROLE_PERSONAL_VIEW_OWN,
    ROLE_PERSONAL_VIEW_ALL,
    ROLE_PERSONAL_EDIT,
    ROLE_PERSONAL_DELETE,
    ROLE_PERSONAL_TYPE,
    ROLE_PERSONAL_PERMISSION,
    ROLE_PERSONAL_CHANGE_PASSWORD,
    ROLE_PERSONAL_EDIT_VACACIONES,
    ROLE_PERSONAL_EDIT_RESPONSABLEINCIDENCIAS,
    
    // Proveedores
    ROLE_SUPPLIER_CREATE,
    ROLE_SUPPLIER_VIEW,
    ROLE_SUPPLIER_EDIT,
    ROLE_SUPPLIER_DELETE,
    
    // Evaluaciones
    ROLE_EVALUATION_NEW,
    ROLE_EVALUATION_VIEW,
    ROLE_EVALUATION_EDIT,
    
    // Recursos
    ROLE_RECURSO_NEW,
    ROLE_RECURSO_ORDENADOR_ON_OFF,
    ROLE_RECURSO_VIEW,
    ROLE_RECURSO_EDIT_ORDENADOR,
    ROLE_RECURSO_EDIT_ROUTER,
    ROLE_RECURSO_EDIT_RECURSO, // editar inventario
    ROLE_RECURSO_VIEW_CATEGORIA,
    ROLE_RECURSO_EDIT_CATEGORIA,
    ROLE_RECURSO_SEND_USER_COMMAND,
    ROLE_RECURSO_SEND_ROOT_COMMAND,
    ROLE_RECURSO_VIEW_RESERVA,
    ROLE_RECURSO_EDIT_RESERVA,
    ROLE_RECURSO_VIEW_AVERIAS,
    ROLE_RECURSO_EDIT_AVERIAS,
    ROLE_RECURSO_VIEW_PEDIDOS,
    ROLE_RECURSO_EDIT_PEDIDOS,
    
    // Incicencias
    /* TODO: Comprobar más tarde, quitado para Xunta de Galicia
    ROLE_INCIDENCE_VIEW_PAGE, // si puede acceder a la pagina de incidencias
    ROLE_INCIDENCE_CREATE,
    ROLE_INCIDENCE_CATEGORIES_TAB,
    ROLE_INCIDENCE_REPORTS_TAB,
    ROLE_INCIDENCE_SELECT_ALL_CATEGORIES, // ver todas las categorias en el select
    ROLE_INCIDENCE_SELECT_OWN_CATEGORIES,
    ROLE_INCIDENCE_CATEGORIES_CREATE,
    ROLE_INCIDENCE_CATEGORIES_EDIT,
    ROLE_INCIDENCE_VIEW, // ver una incidencia, user no puede
    ROLE_INCIDENCE_VIEW_ALL,
    ROLE_INCIDENCE_FUNDETEC, // rol especial que ve todas las incidencias y puede hacer operaciones especiales
    ROLE_INCIDENCE_RECEIVE_MAIL,*/ // desactivar para no recibir correos.

    // Parámetros de configuración
    ROLE_PARAMETERS_VIEW,
    
    // Financiero
    /* TODO: Comprobar más tarde, quitado para Xunta de Galicia
    ROLE_FINANCIAL_VIEW,
    ROLE_FINANCIAL_ADMIN,*/
    
    // Informes automáticos
    ROLE_SEND_AUTOMATIC_FORMS_VIEW,
    ROLE_SEND_AUTOMATIC_FORMS_EDIT,
    
    // Cuadro de mandos
    ROLE_CUADRO_CENTRO_OWN,
    ROLE_CUADRO_VIEW,
    ROLE_CUADRO_EDIT,
    ROLE_ALARM_EDIT;
    
    // wiki
    /* TODO: Comprobar más tarde, quitado para Xunta de Galicia
    ROLE_WIKI_BURGOS,
    ROLE_WIKI_PALENCIA,
    ROLE_WIKI_LEON,
    ROLE_WIKI_ZAMORA,
    ROLE_WIKI_VALLADOLID,
    ROLE_WIKI_SORIA,
    ROLE_WIKI_SEGOVIA,
    ROLE_WIKI_SALAMANCA,
    ROLE_WIKI_AVILA,
    ROLE_WIKI_GENERAL,
    QUALITE,
    ADMIN,*/
    
    // Contratos
    /* TODO: Comprobar más tarde, quitado para Xunta de Galicia
    ROLE_CONTRACT_CREATE,
    ROLE_CONTRACT_VIEW,
    ROLE_CONTRACT_EDIT,
    ROLE_CONTRACT_DELET,*/
    
    // Paneles
    /* TODO: Comprobar más tarde, quitado para Xunta de Galicia
    ROLE_PANEL_VIEW,
    ROLE_PANEL_CREATE_PANEL,
    ROLE_PANEL_CREATE_NEWS,
    ROLE_PANEL_CREATE_EVENTS,
    ROLE_PANEL_VIEW_ALLPANEL,
    ROLE_PANEL_EDIT_ALLPANEL,
    ROLE_PANEL_UPLOAD_FILES,*/
    
    // Notificaciones
    /* TODO: Comprobar más tarde, quitado para Xunta de Galicia
     * ROLE_NOTIFICATIONS_VIEW;*/
  }

  /**
   * Construye un nuevo objeto de la clase
   */
  public Usuario() {
    super();
    setPeso(0);
    movimientos = new ArrayList<Movimiento>();
    matriculasCursos = new ArrayList<MatriculaCurso>();
    matriculasTalleres = new ArrayList<MatriculaTaller>();
    matriculasExternas = new ArrayList<MatriculaExterna>();
    preinscripcionesCursos = new ArrayList<PreinscripcionCurso>();
    preinscripcionesTalleres = new ArrayList<PreinscripcionTaller>();
    matriculasCharlas = new ArrayList<MatriculaCharla>();
    rolesList = new ArrayList<Role>();
    userActivityCalendarItems = new ArrayList<UserActivityCalendarItem>();
    notificaciones = new ArrayList<Notification>();
    notificacionesEnviadas = new ArrayList<MailNotification>();
    actividadCalendarItems = new ArrayList<ActividadCalendarItem>();
    asistencias = new ArrayList<AsistenciaCalendarItem>();
  }

  /**
   * Devuelve roles.
   * 
   * @return roles
   */
  @Lob
  @Column(columnDefinition = "longtext character set utf8 collate utf8_general_ci")
  protected String getRoles() {
    StringBuilder sb = new StringBuilder("");
    for (Role rol : getRolesList()) {
      if (sb.length() != 0) sb.append(", ");
      sb.append(rol.toString());
    }
    return sb.toString();
  }

  /**
   * Asigna roles.
   * 
   * @param roles
   *          roles
   */
  protected void setRoles(String roles) {
    this.rolesList.clear();
    if (roles != null) {
      StringTokenizer st = new StringTokenizer(roles, ",");
      while (st.hasMoreTokens()) {
        try {
          Role role = Role.valueOf(st.nextToken().trim());
          if (!rolesList.contains(role)) rolesList.add(role);
        } catch (IllegalArgumentException e) {}
      }
    }
  }

  /**
   * Devuelve roles array.
   * 
   * @return roles array
   */
  @Transient
  public Role[] getRolesArray() {
    return this.rolesList.toArray(new Role[0]);
  }

  /**
   * Asigna roles array.
   * 
   * @param rolesArray
   *          roles array
   */
  @Transient
  public void setRolesArray(Role[] rolesArray) {
    this.rolesList.clear();
    for (Role role : rolesArray)
      if (!rolesList.contains(role)) rolesList.add(role);
  }

  /**
   * Devuelve roles list.
   * 
   * @return roles list
   */
  @Transient
  public List<Role> getRolesList() {
    return rolesList;
  }

  /**
   * Asigna roles list.
   * 
   * @param rolesList
   *          roles list
   */
  public void setRolesList(List<Role> rolesList) {
    this.rolesList.clear();
    for (Role role : rolesList) {
      if (!this.rolesList.contains(role)) this.rolesList.add(role);
    }
  }

  /**
   * Devuelve nombre.
   * 
   * @return nombre
   */
  @Length(max = 50)
  @NotEmpty
  @NotNull
  public String getNombre() {
    return nombre;
  }

  /**
   * Asigna nombre.
   * 
   * @param nombre
   *          nombre
   */
  public void setNombre(String nombre) {
    this.nombre = nombre;
  }

  /**
   * Devuelve apellido1.
   * 
   * @return apellido1
   */
  @Length(max = 50)
  @NotNull
  public String getApellido1() {
    return apellido1;
  }

  /**
   * Asigna apellido1.
   * 
   * @param apellido1
   *          apellido1
   */
  public void setApellido1(String apellido1) {
    this.apellido1 = apellido1;
  }

  /**
   * Devuelve centro.
   * 
   * @return centro
   */
  @ManyToOne(fetch = FetchType.EAGER, cascade = { CascadeType.REFRESH })
  @JoinColumn(name = "idCentro", nullable = false)
  @NotNull
  public Centro getCentro() {
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Devuelve apellido2.
   * 
   * @return apellido2
   */
  @Length(max = 50)
  public String getApellido2() {
    return apellido2;
  }

  /**
   * Asigna apellido2.
   * 
   * @param apellido2
   *          apellido2
   */
  public void setApellido2(String apellido2) {
    this.apellido2 = apellido2;
  }

  /**
   * Devuelve telefono.
   * 
   * @return telefono
   */
  @Length(max = 20)
  @NotNull
  public String getTelefono() {
    return telefono;
  }

  /**
   * Asigna telefono.
   * 
   * @param telefono
   *          telefono
   */
  public void setTelefono(String telefono) {
    this.telefono = telefono;
  }

  /**
   * Devuelve movimientos.
   * 
   * @return movimientos
   */
  @OneToMany(fetch = FetchType.LAZY, cascade = { javax.persistence.CascadeType.MERGE,
      javax.persistence.CascadeType.PERSIST, javax.persistence.CascadeType.REFRESH }, mappedBy = "usuario")
  public List<Movimiento> getMovimientos() {
    return movimientos;
  }

  /**
   * Asigna movimientos.
   * 
   * @param movimientos
   *          movimientos
   */
  public void setMovimientos(List<Movimiento> movimientos) {
    this.movimientos = movimientos;
  }

  /**
   * Devuelve matriculas cursos.
   * 
   * @return matriculas cursos
   */
  @OneToMany(fetch = FetchType.LAZY, cascade = { javax.persistence.CascadeType.MERGE,
      javax.persistence.CascadeType.PERSIST, javax.persistence.CascadeType.REFRESH }, mappedBy = "usuario")
  @OrderBy(value = "fechaMatricula ASC")
  public List<MatriculaCurso> getMatriculasCursos() {
    return matriculasCursos;
  }

  /**
   * Asigna matriculas cursos.
   * 
   * @param matriculasCursos
   *          matriculas cursos
   */
  public void setMatriculasCursos(List<MatriculaCurso> matriculasCursos) {
    this.matriculasCursos = matriculasCursos;
  }

  /**
   * Devuelve matriculas talleres.
   * 
   * @return matriculas talleres
   */
  @OneToMany(fetch = FetchType.LAZY, cascade = { javax.persistence.CascadeType.MERGE,
      javax.persistence.CascadeType.PERSIST, javax.persistence.CascadeType.REFRESH }, mappedBy = "usuario")
  @OrderBy(value = "fechaMatricula ASC")
  public List<MatriculaTaller> getMatriculasTalleres() {
    return matriculasTalleres;
  }

  /**
   * Asigna matriculas talleres.
   * 
   * @param matriculasTalleres
   *          matriculas talleres
   */
  public void setMatriculasTalleres(List<MatriculaTaller> matriculasTalleres) {
    this.matriculasTalleres = matriculasTalleres;
  }

  /**
   * Devuelve matriculas externas.
   * 
   * @return matriculas externas
   */
  @OneToMany(fetch = FetchType.LAZY, cascade = { javax.persistence.CascadeType.MERGE,
      javax.persistence.CascadeType.PERSIST, javax.persistence.CascadeType.REFRESH }, mappedBy = "usuario")
  @OrderBy(value = "fechaMatricula ASC")
  public List<MatriculaExterna> getMatriculasExternas() {
    return matriculasExternas;
  }

  /**
   * Asigna matriculas externas.
   * 
   * @param matriculasExternas
   *          matriculas externas
   */
  public void setMatriculasExternas(List<MatriculaExterna> matriculasExternas) {
    this.matriculasExternas = matriculasExternas;
  }

  /**
   * Devuelve preinscripciones cursos.
   * 
   * @return preinscripciones cursos
   */
  @OneToMany(fetch = FetchType.LAZY, cascade = { javax.persistence.CascadeType.MERGE,
      javax.persistence.CascadeType.PERSIST, javax.persistence.CascadeType.REFRESH }, mappedBy = "usuario")
  @OrderBy(value = "fecha ASC")
  public List<PreinscripcionCurso> getPreinscripcionesCursos() {
    return preinscripcionesCursos;
  }

  /**
   * Asigna preinscripciones cursos.
   * 
   * @param preinscripcionesCursos
   *          preinscripciones cursos
   */
  public void setPreinscripcionesCursos(List<PreinscripcionCurso> preinscripcionesCursos) {
    this.preinscripcionesCursos = preinscripcionesCursos;
  }

  /**
   * Devuelve preinscripciones talleres.
   * 
   * @return preinscripciones talleres
   */
  @OneToMany(fetch = FetchType.LAZY, cascade = { javax.persistence.CascadeType.MERGE,
      javax.persistence.CascadeType.PERSIST, javax.persistence.CascadeType.REFRESH }, mappedBy = "usuario")
  @OrderBy(value = "fecha ASC")
  public List<PreinscripcionTaller> getPreinscripcionesTalleres() {
    return preinscripcionesTalleres;
  }

  /**
   * Asigna preinscripciones talleres.
   * 
   * @param preinscripcionesTalleres
   *          preinscripciones talleres
   */
  public void setPreinscripcionesTalleres(List<PreinscripcionTaller> preinscripcionesTalleres) {
    this.preinscripcionesTalleres = preinscripcionesTalleres;
  }

  /**
   * Devuelve matriculas charlas.
   * 
   * @return matriculas charlas
   */
  @OneToMany(fetch = FetchType.LAZY, cascade = { javax.persistence.CascadeType.MERGE,
      javax.persistence.CascadeType.PERSIST, javax.persistence.CascadeType.REFRESH }, mappedBy = "usuario")
  @OrderBy(value = "fechaConfirmacion ASC")
  public List<MatriculaCharla> getMatriculasCharlas() {
    return matriculasCharlas;
  }

  /**
   * Asigna matriculas charlas.
   * 
   * @param matriculasCharlas
   *          matriculas charlas
   */
  public void setMatriculasCharlas(List<MatriculaCharla> matriculasCharlas) {
    this.matriculasCharlas = matriculasCharlas;
  }

  /**
   * Devuelve peso.
   * 
   * @return peso
   */
  public int getPeso() {
    return peso;
  }

  /**
   * Asigna peso.
   * 
   * @param peso
   *          peso
   */
  public void setPeso(int peso) {
    this.peso = peso;
  }

  /**
   * Devuelve nif.
   * 
   * @return nif
   */
//@NIFPersonaFisica
  @Length(max = 20)  
  public String getNif() {
    // if (nif == null && getId() > 0) return CABECERANIF + getId();
    return nif;
  }

  /**
   * Asigna nif.
   * 
   * @param nif
   *          nif
   */
  public void setNif(String nif) {
    this.nif = nif;
  }

  /**
   * Devuelve direccion.
   * 
   * @return direccion
   */
  @Length(max = 150)
  public String getDireccion() {
    return direccion;
  }

  /**
   * Asigna direccion.
   * 
   * @param direccion
   *          direccion
   */
  public void setDireccion(String direccion) {
    this.direccion = direccion;
  }

  /**
   * Devuelve cp.
   * 
   * @return cp
   */
  public String getCp() {
    return cp;
  }

  /**
   * Asigna cp.
   * 
   * @param cp
   *          cp
   */
  public void setCp(String cp) {
    this.cp = cp;
  }

  /**
   * Devuelve localidad.
   * 
   * @return localidad
   */
  @ManyToOne(fetch = FetchType.LAZY)
  @JoinColumn(name = "idLocalidad") 
  @NotNull
  public Localidad getLocalidad() {
    return localidad;
  }

  /**
   * Asigna localidad.
   * 
   * @param localidad
   *          localidad
   */
  public void setLocalidad(Localidad localidad) {
    this.localidad = localidad;
  }

  /**
   * Devuelve provincia.
   * 
   * @return provincia
   */
  @ManyToOne(fetch = FetchType.LAZY)
  @JoinColumn(name = "idProvincia")
  @NotNull
  public Provincia getProvincia() {
    return provincia;
  }

  /**
   * Asigna provincia.
   * 
   * @param provincia
   *          provincia
   */
  public void setProvincia(Provincia provincia) {
    this.provincia = provincia;
  }

  /**
   * Checks if is usuario cy l.
   * 
   * @return true, if is usuario cy l
   */
  @Transient
  public abstract boolean isUsuarioCyL();

  /**
   * Checks if is usuario pre.
   * 
   * @return true, if is usuario pre
   */
  @Transient
  public abstract boolean isUsuarioPre();

  /**
   * Checks if is empleado.
   * 
   * @return true, if is empleado
   */
  @Transient
  public abstract boolean isEmpleado();

  /**
   * Descripción del usuario a implementar por el usuario concreto con la información descriptiva suficiente.
   * 
   * @return Cadena con la descripción
   */
  @Transient
  public abstract String getDescripcion();

  /**
   * Devuelve apellidos.
   * 
   * @return apellidos
   */
  @Transient
  public String getApellidos() {
    String apellidos = "";
    apellidos += getCompoundString(getApellido1());
    apellidos += getCompoundString(getApellido2());
    return apellidos;
  }

  /**
   * Devuelve nombre completo.
   * 
   * @return nombre completo
   */
  @Transient
  public String getNombreCompleto() {
    String nombreCompleto = "";
    nombreCompleto += getNombre().trim();
    nombreCompleto += getCompoundString(getApellidos());
    return nombreCompleto;
  }

  /**
   * Devuelve telefonos.
   * 
   * @return telefonos
   */
  @Transient  
  public String getTelefonos() {
    String telefonos = "";
    telefonos += getCompoundString(getTelefono());
    return telefonos;
  }

  @Transient
  private String getCompoundString(String addition) {
    if (addition != null)
    	if (!(addition.trim()).isEmpty())
    		return " " + addition.trim();

    return "";
  }

  /**
   * Devuelve tipo usuario.
   * 
   * @return tipo usuario
   */
  @Transient
  public TipoUsuario getTipoUsuario() {
    return null;
  }

  /**
   * Devuelve incidencias.
   * 
   * @return incidencias
   */
  @OneToMany(mappedBy = "usuario")
  public List<Incidencia> getIncidencias() {
    return incidencias;
  }

  /**
   * Asigna incidencias.
   * 
   * @param incidencias
   *          incidencias
   */
  public void setIncidencias(List<Incidencia> incidencias) {
    this.incidencias = incidencias;
  }

  /**
   * Devuelve estado incidencias.
   * 
   * @return estado incidencias
   */
  @OneToMany(mappedBy = "usuario")
  public List<EventoIncidencia> getEstadoIncidencias() {
    return estadoincidencias;
  }

  /**
   * Asigna estado incidencias.
   * 
   * @param estadoincidencias
   *          estado incidencias
   */
  public void setEstadoIncidencias(List<EventoIncidencia> estadoincidencias) {
    this.estadoincidencias = estadoincidencias;
  }

  /**
   * Devuelve comentario observaciones.
   * 
   * @return comentario observaciones
   */
  @OneToMany(mappedBy = "usuario")
  public List<ComentarioObservacion> getComentarioObservaciones() {
    return comentarioObservaciones;
  }

  /**
   * Asigna comentario observaciones.
   * 
   * @param comentarioObservaciones
   *          comentario observaciones
   */
  public void setComentarioObservaciones(List<ComentarioObservacion> comentarioObservaciones) {
    this.comentarioObservaciones = comentarioObservaciones;
  }

  /**
   * Checks if is override default roles.
   * 
   * @return true, if is override default roles
   */
  public boolean isOverrideDefaultRoles() {
    return overrideDefaultRoles;
  }

  /**
   * Asigna override default roles.
   * 
   * @param overrideDefaulRoles
   *          override default roles
   */
  public void setOverrideDefaultRoles(boolean overrideDefaulRoles) {
    this.overrideDefaultRoles = overrideDefaulRoles;
  }

  /**
   * Devuelve user activity calendar items.
   * 
   * @return user activity calendar items
   */
  @OneToMany(mappedBy = "usuario")
  public List<UserActivityCalendarItem> getUserActivityCalendarItems() {
    return userActivityCalendarItems;
  }

  /**
   * Asigna user activity calendar items.
   * 
   * @param userActivityCalendarItems
   *          user activity calendar items
   */
  public void setUserActivityCalendarItems(List<UserActivityCalendarItem> userActivityCalendarItems) {
    this.userActivityCalendarItems = userActivityCalendarItems;
  }

  /**
   * Devuelve notificaciones.
   * 
   * @return notificaciones
   */
  @ManyToMany(cascade = { CascadeType.ALL }, mappedBy = "recipients", targetEntity = Notification.class)
  public List<Notification> getNotificaciones() {
    return notificaciones;
  }

  /**
   * Asigna notificaciones.
   * 
   * @param notificaciones
   *          notificaciones
   */
  public void setNotificaciones(List<Notification> notificaciones) {
    this.notificaciones = notificaciones;
  }

  /**
   * Devuelve notificaciones enviadas.
   * 
   * @return notificaciones enviadas
   */
  @ManyToMany(cascade = { CascadeType.ALL }, mappedBy = "usuarios", targetEntity = MailNotification.class)
  public List<MailNotification> getNotificacionesEnviadas() {
    return notificacionesEnviadas;
  }

  /**
   * Asigna notificaciones enviadas.
   * 
   * @param notificacionesEnviadas
   *          notificaciones enviadas
   */
  public void setNotificacionesEnviadas(List<MailNotification> notificacionesEnviadas) {
    this.notificacionesEnviadas = notificacionesEnviadas;
  }

  /**
   * Devuelve actividad calendar items.
   * 
   * @return actividad calendar items
   */
  @OneToMany(mappedBy = "usuario")
  public List<ActividadCalendarItem> getActividadCalendarItems() {
    return actividadCalendarItems;
  }

  /**
   * Asigna actividad calendar items.
   * 
   * @param actividadCalendarItems
   *          actividad calendar items
   */
  public void setActividadCalendarItems(List<ActividadCalendarItem> actividadCalendarItems) {
    this.actividadCalendarItems = actividadCalendarItems;
  }

  /**
   * Devuelve asistencias.
   * 
   * @return asistencias
   */
  @OneToMany(mappedBy = "usuario")
  public List<AsistenciaCalendarItem> getAsistencias() {
    return asistencias;
  }

  /**
   * Asigna asistencias.
   * 
   * @param asistencias
   *          asistencias
   */
  public void setAsistencias(List<AsistenciaCalendarItem> asistencias) {
    this.asistencias = asistencias;
  }

}
