package es.csa.fdt.cyldigital.usuarios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.OrderBy;
import javax.persistence.PrePersist;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.apache.commons.codec.digest.DigestUtils;
import org.hibernate.envers.Audited;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotNull;
import org.hibernate.validator.Pattern;

import es.csa.fdt.cyldigital.evaluacion.entity.FormularioConcreto;
import es.csa.fdt.cyldigital.usuarios.entity.RegistroSmartCard.TipoRegistro;

/**
 * Clase UsuarioCyL.
 * 
 * @author csa
 */
@Entity
@Table(name = "usuarioscyl")
@AttributeOverride(name = "id", column = @Column(name = "idUsuario"))
@Audited
@PrimaryKeyJoinColumn(name = "idUsuario")
// @UsuarioUnico
public class UsuarioCyL extends Usuario implements Serializable {

  private static final long serialVersionUID = -7648562176655166537L;
  
  private static final String EMAIL_PATTERN = 
			"^$|[_A-Za-z0-9-\\+]+(\\.[_A-Za-z0-9-]+)*@"
			+ "[A-Za-z0-9-]+(\\.[A-Za-z0-9]+)*(\\.[A-Za-z]{2,})$";

  /**
   * Enum Sexo.
   */
  public enum Sexo {
    /** Hombre */
    H,
    /** Mujer */
    M;
  }

  /**
   * Enum MedioPublicidad.
   */
  public enum MedioPublicidad {
    
    /** Agente TIC*/
    AGENTETIC,
    /** Asociación*/
    ASOCIACION,
    /**Ayuntamiento*/
    AYUNTAMIENTO,
    /**Xunta de Galicia*/
    XUNTAGALICIA,
    /**Web cemit.xunta.es*/
    WEBCEMIT,
    /**Medios de comunicación*/
    MEDIOSCOMUN,
    /**Publicidad*/
    PUBLICIDAD,
    /**Otras personas*/
    OTRASPERSONAS,
    /**Otras instituciones*/
    OTRASINSTITUCIONES,
    /**Otras*/ 
    OTRAS,
    /**No contesta*/
    NOCONTESTA;       

  }

  /**
   * Enum Publicidad.
   */
  /*public enum Publicidad {
    PUBLICIDAD_PRENSA,
    PUBLICIDAD_RADIO,
    PUBLICIDAD_MUPIS,
    PUBLICIDAD_LONA,
    PUBLICIDAD_POSTAL_FREE,
    PUBLICIDAD_PUBLI_MAPA,
    PUBLICIDAD_OTRA,
    TELEVISION,
    PUBLICIDAD_EN_BUS,
    BANDEROLAS,
    BUZONEO;

  }*/

  /**
   * Enum TipoUsuario.
   */
  public enum TipoUsuario {
    
	/** Sin Conocimientos*/
	SC,
	/** Básicos */
    B,
    /** Nivel usuario/a */
    NU,
    /** Técnico/Avanzado */
    TA;
  }

  /**
   * Enum TipoIdentificacion.
   */
	public enum TipoIdentificacion {
    /** Tarjeta CyL Digital */
    //C,
	/** Identificación generada para menor de 14 años sin ningún tipo de identificación */
	M,
	/** Pasaporte, N.I.E. u otro tipo de identificación para extranjeros sin DNI */
	E,
    /** N.I.F. */
    N;
  }

  /**
   * Enum TipoAdaptacion.
   */
  public enum TipoDiscapacidad {

	  /**Sin Adaptación*/
	  SINADAPTACION,
	  /** Adaptación física*/
	  ADAPTACIONFISICA,
	  /** Adaptación intelectual y sensorial*/
	  ADAPTACIONINTELECTUAL,
	  /**Adaptación múltiple*/
	  ADAPTACIONMULTIPLE,
	  /** Otras Adaptaciones*/
	  OTRASADAPTACIONES; 
  }
  
  /**
   * Enum TipoFrecunciaInternet
   */
  public enum TipoFrecuenciaInternet {
	  /**Nunca*/
	  NUNCA,
	  /**Una vez al mes*/
	  UNAVEZMES,
	  /**2/3 veces a la semana*/
	  DOSTERCIOS,
	  /**Todos los días*/
	  TODOSDIAS;
  }
  
  /**
   * Enum UtilizasInternet
   */
  public enum TipoUtilidadInternet {
	  NA,
	  /**Ocio*/
	  OCIO,
	  /**Trabajo*/
	  TRABAJO,
	  /**Ambos*/
	  AMBOS;
  }
  
  /**
   * Enum UtilizarSoftware
   */
  public enum TipoUtilizarSwLibre{
	  /**Si*/
	  SI,
	  /**No*/
	  NO;
  }
  
  /** 
   * Enum TipoOcupacion
   */
  public enum TipoOcupacion{
	  /**Empleado por cuenta propia*/	 
	  EMPLEADOSCUENTAPROPIA,
	  /**Empleado asalariado*/
	  EMPLEADOASALARIADO,
	  /**Desempleado registrado/a en el INEM*/
	  DESEMPLEADOSREGISTRADOSINEM,
	  /**Desempleado no registrado en el INEM*/
	  DESEMPLEADOSNOREGISTRADOSINEM,
	  /**Jubilido/a*/
	  JUBILADO,
	  /**PENSIONISTA*/
	  PENSIONISTA,
	  /**Estudiante*/
	  ESTUDIANTE,
	  /**Otros*/
	  OTROS; 
  }
  
  /** 
   * Enum TipoColectivo
   */
  public enum TipoColectivo{
	  /**Ciudadano*/
	  CIUDADANO,
	  /**Voluntariado**/
	  VOLUNTARIADO,
	  /**Asociación*/
	  ASOCIACION,
	  /**Fundación*/
	  FUNDACION,
	  /**Administración pública*/
	  ADMPUBLICA,
	  /**Profesional TIC */
	  PROFESIONALTIC,
	  /**Empres privada*/
	  EMPRESAPRIVADA,
	  /**Agrupación de empresas*/
	  AGRUPEMPRESAS,
	  /**Otros*/
	  OTROS;

  }

  /**
   * Enum TipoEstudiosRealizados
   */    
  public enum TipoEstudiosRealizados{
	  /**Sin Estudios*/
	  SINESTUDIOS,
	  /**Estudios primarios*/
	  ESTUDIOSPRIMARIOS,
	  /**Titulado de graduado ESO/Graduado escolar*/
	  GRADUADOESOESCOLAR,
	  /**FP1/Curso Formación Grado Medio*/
	  FP1,
	  /**FP2/Curso Formación Grado Superior*/
	  FP2,
	  /**Bachillerato/COU*/
	  BACHILLERATO,
	  /**Diplomatura-Grado*/
	  DIPLOMATURA,
	  /**Licenciatura-Master*/
	  LICENCIATURA,
	  /**Doctor*/
	  DOCTOR,
	  /**Otra titulación*/
	  OTRATITULACION;
	  
  }

  /**
   * Enum TipoSectorProfesional
   */
  public enum TipoSectorProfesional{
	  /**Profesiones, artes y oficios*/
	  PROFESIONESARTESOFICIOS,
	  /**Administración pública*/
	  ADMPUBLICA,
	  /**Banca*/
	  BANCA,
	  /**Comercio*/
	  COMERCIO,
	  /**Construcción*/
	  CONSTRUCCION,
	  /**Informática y Telecomunicaciones*/
	  INFTELECOMUNICACIONES,
	  /**Ingeniería*/
	  INGENIERIA,
	  /**Educación y formación*/
	  EDUCACIONFORMACION,
	  /**Legal*/
	  LEGAL,
	  /**Sanidad y salud*/
	  SANIDADSALUD;
  }
  
  private String movil;
  private String email;

  private String nifAutorizado;
  private String pasaporte;
  private Boolean extranjero;
  private boolean menor;
  private String nombreCompletoAutorizado;
  private String username;

  private boolean poseeOrdenador;
  private boolean accesoInternet;

  private boolean comunicacionTlf;
  private boolean comunicacionMail;
  private boolean comunicacionSms;

  private Date fechaNacimiento;
  private TipoUsuario tipoUsuario;  
  private Sexo sexo;
  private MedioPublicidad medioPublicidad;
  private String medioPublicidadOtro;
  //private Publicidad publicidad;
  private TipoDiscapacidad tipoDiscapacidad;  
  private TipoFrecuenciaInternet tipoFrecuenciaInternet;
  private TipoUtilidadInternet tipoUtilidadInternet;
  private TipoUtilizarSwLibre tipoUtilizarSwLibre;
  private TipoOcupacion tipoOcupacion;
  private TipoColectivo tipoColectivo;
  private TipoEstudiosRealizados tipoEstudiosRealizados;
  private TipoSectorProfesional tipoSectorProfesional;
  
  private boolean aceptaPolitica;
  private String password;
  private TipoIdentificacion tipoIdentificacion;
  private Date fechaCreacion;
  private Date ultimoAcceso;
  private boolean desactivadoLDAP;

  private Fichero fichero;

  private List<RegistroLdap> registrosLdap;

  private List<RegistroSmartCard> registrosSmartCard;

  private FormularioConcreto formularioConcreto;

  private boolean cesionPermitida;

  /*
   * foto_fid int(10) unsigned default NULL,
   */
  /**
   * Construye un nuevo objeto de la clase
   */
  public UsuarioCyL() {
    super();
    formularioConcreto = new FormularioConcreto();
    setPassword("");
    /*setTipoIdentificacion(TipoIdentificacion.C);*/
    setRegistrosLdap(new ArrayList<RegistroLdap>());
    setRegistrosSmartCard(new ArrayList<RegistroSmartCard>());
    setExtranjero(false);
    setAceptaPolitica(false);
    //3.4.3-R RF_XEA_05 - Cambio criterio campo cesion permitida
    //setCesionPermitida(true); 
    setCesionPermitida(false);
    setcomunicacionTlf(true);
    setcomunicacionMail(true);
    setcomunicacionSms(false);
  }

  /**
   * Devuelve nif autorizado.
   * 
   * @return nif autorizado
   */
  @Length(max = 100)
  public String getNifAutorizado() {
    return nifAutorizado;
  }

  /**
   * Asigna nif autorizado.
   * 
   * @param nifAutorizado
   *          nif autorizado
   */
  public void setNifAutorizado(String nifAutorizado) {
    this.nifAutorizado = nifAutorizado;
  }

  /**
   * Devuelve username.
   * 
   * @return username
   */
  @Length(max = 20)
  @Column(unique = true)
  public String getUsername() {
    return username;
  }

  /**
   * Asigna username.
   * 
   * @param username
   *          username
   */
  public void setUsername(String username) {
    this.username = username;
  }

  /**
   * Devuelve movil.
   * 
   * @return movil
   */
  @Length(max = 20)
  public String getMovil() {
    return movil;
  }

  /**
   * Asigna movil.
   * 
   * @param telefono2
   *          movil
   */
  public void setMovil(String telefono2) {
    this.movil = telefono2;
  }

  /**
   * Devuelve fecha nacimiento.
   * 
   * @return fecha nacimiento
   */
  @Temporal(TemporalType.DATE)
  @NotNull
  public Date getFechaNacimiento() {
    return fechaNacimiento;
  }

  /**
   * Asigna fecha nacimiento.
   * 
   * @param fechaNacimiento
   *          fecha nacimiento
   */
  public void setFechaNacimiento(Date fechaNacimiento) {
    this.fechaNacimiento = fechaNacimiento;
  }

  /**
   * Devuelve fecha creacion.
   * 
   * @return fecha creacion
   */
  @Temporal(TemporalType.TIMESTAMP)
  public Date getFechaCreacion() {
    return fechaCreacion;
  }

  /**
   * Asigna fecha creacion.
   * 
   * @param fechaCreacion
   *          fecha creacion
   */
  public void setFechaCreacion(Date fechaCreacion) {
    this.fechaCreacion = fechaCreacion;
  }
  
  /**
   * Devuelve fecha de último acceso.
   * 
   * @return fecha ultimo acceso
   */
  @Temporal(TemporalType.TIMESTAMP)
  public Date getUltimoAcceso() {
	return ultimoAcceso;
  }

  /**
   * Asigna fecha de acceso.
   * 
   * @param ultimoAcceso
   *          fecha ultimo acceso
   */
  public void setUltimoAcceso(Date ultimoAcceso) {
	  this.ultimoAcceso = ultimoAcceso;
  }

  /**
   * Devuelve nombre completo autorizado.
   * 
   * @return nombre completo autorizado
   */
  @Length(max = 255)
  public String getNombreCompletoAutorizado() {
    return nombreCompletoAutorizado;
  }

  /**
   * Asigna nombre completo autorizado.
   * 
   * @param comentarioNif
   *          nombre completo autorizado
   */
  public void setNombreCompletoAutorizado(String comentarioNif) {
    this.nombreCompletoAutorizado = comentarioNif;
  }

  /**
   * Devuelve email.
   * 
   * @return email
   */
  @Length(max = 255)
  //@Email
  @Pattern(regex=EMAIL_PATTERN, message = "El email no tiene el formato correcto")
  public String getEmail() {
    return email;
  }

  /**
   * Asigna email.
   * 
   * @param email
   *          email
   */
  public void setEmail(String email) {
    this.email = email == null ? null : email.trim();
  }

  /**
   * Devuelve sexo.
   * 
   * @return sexo
   */
  @Enumerated(value = EnumType.STRING)
  @NotNull
  public Sexo getSexo() {
    return this.sexo;
  }

  /**
   * Asigna sexo.
   * 
   * @param sexo
   *          sexo
   */
  public void setSexo(Sexo sexo) {
    this.sexo = sexo;
  }

  /**
   * Devuelve medio publicidad.
   * 
   * @return medio publicidad
   */
  @Enumerated(value = EnumType.STRING)
  @NotNull
  public MedioPublicidad getMedioPublicidad() {
    return this.medioPublicidad;
  }

  /**
   * Asigna medio publicidad.
   * 
   * @param medioPublicidad
   *          medio publicidad
   */
  public void setMedioPublicidad(MedioPublicidad medioPublicidad) {
    this.medioPublicidad = medioPublicidad;
  }

  /**
   * Devuelve medio publicidad otro.
   * 
   * @return medio publicidad otro
   */
  @Length(max = 255) 
  public String getMedioPublicidadOtro() {
    return medioPublicidadOtro;
  }

  /**
   * Asigna medio publicidad otro.
   * 
   * @param medioPublicidadOtro
   *          medio publicidad otro
   */
  public void setMedioPublicidadOtro(String medioPublicidadOtro) {
    this.medioPublicidadOtro = medioPublicidadOtro;
  }

  /**
   * Asigna publicidad.
   * 
   * @param publicidad
   *          publicidad
   */
  /*public void setPublicidad(Publicidad publicidad) {
    this.publicidad = publicidad;
  }*/

  /**
   * Devuelve publicidad.
   * 
   * @return publicidad
   */
  /*@Enumerated(value = EnumType.STRING)
  public Publicidad getPublicidad() {
    return publicidad;
  }*/

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.entity.Usuario#getTipoUsuario()
   */
  @Enumerated(value = EnumType.STRING)
  @Override
  @NotNull
  public TipoUsuario getTipoUsuario() {
    return tipoUsuario;
  }

  /**
   * Asigna tipo usuario.
   * 
   * @param tipoUsuario
   *          tipo usuario
   */
  public void setTipoUsuario(TipoUsuario tipoUsuario) {
    this.tipoUsuario = tipoUsuario;
  }

  /**
   * Checks if is acepta politica.
   * 
   * @return true, if is acepta politica
   */
  @NotNull
  public boolean isAceptaPolitica() {
    return aceptaPolitica;
  }

  /**
   * Asigna acepta politica.
   * 
   * @param aceptaPolitica
   *          acepta politica
   */
  public void setAceptaPolitica(boolean aceptaPolitica) {
    this.aceptaPolitica = aceptaPolitica;
  }

  /**
   * Checks si esta desactivaddo en el LDAP.
   * 
   * @return true, si esta desactivado en el LDAP
   */
  @NotNull
  public boolean isDesactivadoLDAP() {
    return desactivadoLDAP;
  }

  /**
   * Asigna desactivadoLDAP
   * 
   * @param desactivadoLDAP
   *          desactivadoLDAP
   */
  public void setDesactivadoLDAP(boolean desactivadoLDAP) {
    this.desactivadoLDAP = desactivadoLDAP;
  }
  
  /**
   * Devuelve password.
   * 
   * @return password
   */
  public String getPassword() {
    return password;
  }

  /**
   * Asigna password.
   * 
   * @param password
   *          password
   */
  public void setPassword(String password) {
    this.password = password;
  }

  /**
   * Checks if is menor.
   * 
   * @return true, if is menor
   */
  public boolean isMenor() {
    return menor;
  }

  /**
   * Asigna menor.
   * 
   * @param menor
   *          menor
   */
  public void setMenor(boolean menor) {
    this.menor = menor;
  }

  /**
   * Devuelve pasaporte.
   * 
   * @return pasaporte
   */
  @Length(max = 100)
  public String getPasaporte() {
    return pasaporte;
  }

  /**
   * Asigna pasaporte.
   * 
   * @param pasaporte
   *          pasaporte
   */
  public void setPasaporte(String pasaporte) {
    this.pasaporte = pasaporte;
  }

  /**
   * Checks if is extranjero.
   * 
   * @return true, if is extranjero
   */
  public boolean isExtranjero() {
    if (extranjero == null) return false;
    return extranjero;
  }

  /**
   * Asigna extranjero.
   * 
   * @param extranjero
   *          extranjero
   */
  public void setExtranjero(boolean extranjero) {
    this.extranjero = extranjero;
  }

  /**
   * Devuelve foto.
   * 
   * @return foto
   */
  @OneToOne(cascade = CascadeType.ALL)
  @JoinColumn(name = "idFoto")
  public Fichero getFoto() {
    return fichero;
  }

  /**
   * Asigna foto.
   * 
   * @param fichero
   *          foto
   */
  public void setFoto(Fichero fichero) {
    this.fichero = fichero;
  }

  /**
   * Devuelve tipo identificacion.
   * 
   * @return tipo identificacion
   */
  @Enumerated(value = EnumType.STRING)
  @NotNull
  public TipoIdentificacion getTipoIdentificacion() {
    return tipoIdentificacion;
  }

  /**
   * Asigna tipo identificacion.
   * 
   * @param tipoIdentificacion
   *          tipo identificacion
   */
  public void setTipoIdentificacion(TipoIdentificacion tipoIdentificacion) {
    this.tipoIdentificacion = tipoIdentificacion;
  }

  /**
   * Devuelve registros ldap.
   * 
   * @return registros ldap
   */
  @OneToMany(mappedBy = "usuario", cascade = { javax.persistence.CascadeType.MERGE,
      javax.persistence.CascadeType.PERSIST, javax.persistence.CascadeType.REMOVE })
  @OrderBy(value = "fecha DESC")
  public List<RegistroLdap> getRegistrosLdap() {
    return registrosLdap;
  }

  /**
   * Asigna registros ldap.
   * 
   * @param ldap
   *          registros ldap
   */
  public void setRegistrosLdap(List<RegistroLdap> ldap) {
    this.registrosLdap = ldap;
  }

  /**
   * Devuelve registros smart card.
   * 
   * @return registros smart card
   */
  @OneToMany(mappedBy = "usuario", cascade = { javax.persistence.CascadeType.MERGE,
      javax.persistence.CascadeType.PERSIST, javax.persistence.CascadeType.REMOVE })
  @OrderBy(value = "fecha DESC")
  public List<RegistroSmartCard> getRegistrosSmartCard() {
    return registrosSmartCard;
  }

  /**
   * Asigna registros smart card.
   * 
   * @param smartcard
   *          registros smart card
   */
  public void setRegistrosSmartCard(List<RegistroSmartCard> smartcard) {
    this.registrosSmartCard = smartcard;
  }

  /**
   * Checks if is cesion permitida.
   * 
   * @return true, if is cesion permitida
   */
  public boolean isCesionPermitida() {
    return cesionPermitida;
  }

  /**
   * Asigna cesion permitida.
   * 
   * @param cesionPermitida
   *          cesion permitida
   */
  public void setCesionPermitida(boolean cesionPermitida) {
    this.cesionPermitida = cesionPermitida;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.entity.Usuario#isUsuarioCyL()
   */
  @Override
  @Transient
  public boolean isUsuarioCyL() {
    return true;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.entity.Usuario#isUsuarioPre()
   */
  @Override
  @Transient
  public boolean isUsuarioPre() {
    return false;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.entity.Usuario#isEmpleado()
   */
  @Override
  @Transient
  public boolean isEmpleado() {
    return false;
  }

  /**
   * Devuelve edad.
   * 
   * @return edad
   */
  @Transient
  public Integer getEdad() {
    if (fechaNacimiento == null) return null;

    Calendar calendarNacimiento = Calendar.getInstance();
    calendarNacimiento.setTime(fechaNacimiento);
    Calendar calendarHoy = Calendar.getInstance();
    int factor = 0;
    if (calendarHoy.get(Calendar.DAY_OF_YEAR) < calendarNacimiento.get(Calendar.DAY_OF_YEAR)) {
      factor = -1;
    }
    int edad = calendarHoy.get(Calendar.YEAR) - calendarNacimiento.get(Calendar.YEAR) + factor;

    return edad;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.entity.Usuario#getTelefonos()
   */
  @Transient
  @Override
  public String getTelefonos() {
    String telefonos = "";
    telefonos += getCompoundString(getTelefono());
    if (!getCompoundString(getMovil()).isEmpty() && !telefonos.isEmpty()) telefonos += " - ";
    if (!getCompoundString(getMovil()).isEmpty()) telefonos += getCompoundString(getMovil()).trim();
    return telefonos;
  }

  /**
   * Devuelve direccion completa.
   * 
   * @return direccion completa
   */
  @Transient
  public String getDireccionCompleta() {
    String direccionCompleta = "";
    if (getLocalidad() != null) {
      direccionCompleta += getCompoundString(getDireccion());
      direccionCompleta += getCompoundString(getCp());
      direccionCompleta += getCompoundString(getLocalidad().getNombre());
      if (!getCompoundString(getLocalidad().getProvincia().getNombre()).isEmpty())
        direccionCompleta += " (" + getCompoundString(getLocalidad().getProvincia().getNombre()).trim() + ")";
    } else {
      if (getDireccion() != null && !getDireccion().trim().isEmpty()) {
        direccionCompleta += getCompoundString(getDireccion());
        direccionCompleta += getCompoundString(getCp());
        direccionCompleta += getCompoundString(" -Sin localidad- ");
        if (getProvincia() != null && !getCompoundString(getProvincia().getNombre()).isEmpty()) {
          direccionCompleta += " (" + getCompoundString(getProvincia().getNombre()).trim() + ")";
        } else {
          direccionCompleta += " ( Sin provincia )";
        }

      }

    }
    return direccionCompleta;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.entity.Usuario#getDescripcion()
   */
  @Override
  @Transient
  public String getDescripcion() {
    String descripcion = "";
    if (!isExtranjero()) {
      if (!getNif().isEmpty()) descripcion += " (NIF: " + getNif() + ") ";
    } else if (isExtranjero()) descripcion += " (Identificación: " + getPasaporte() + ")";
    else if (isMenor()) descripcion += " (NIF Autorizado: " + getNif() + ")";

    if (!getTelefonos().trim().isEmpty()) descripcion += " Tlf: " + getTelefonos();
    if (getCentro() != null) descripcion += " (Centro: " + getCentro().getNombre() + ") ";
    if (descripcion.trim() == null) return " ";
    else return descripcion.trim();
  }

  /**
   * Checks if is registrado ldap.
   * 
   * @return true, if is registrado ldap
   */
  @Transient
  public boolean isRegistradoLdap() {
    if (getRegistrosLdap() != null && !getRegistrosLdap().isEmpty()) return true;
    return false;
  }

  /**
   * Checks if is impresa tarjeta.
   * 
   * @return true, if is impresa tarjeta
   */
  @Transient
  public boolean isImpresaTarjeta() {
    if (getRegistrosSmartCard() != null && !getRegistrosSmartCard().isEmpty())
      if (getUltimoRegistroTarjeta(TipoRegistro.I) != null) return true;
    return false;
  }

  /**
   * Checks if is grabada tarjeta.
   * 
   * @return true, if is grabada tarjeta
   */
  @Transient
  public boolean isGrabadaTarjeta() {
    if (getRegistrosSmartCard() != null && !getRegistrosSmartCard().isEmpty())
      if (getUltimoRegistroTarjeta(TipoRegistro.G) != null) return true;
    return false;
  }

  /**
   * Devuelve ultimo registro tarjeta.
   * 
   * @param tipo
   *           tipo
   * @return ultimo registro tarjeta
   */
  @Transient
  public RegistroSmartCard getUltimoRegistroTarjeta(TipoRegistro tipo) {
    RegistroSmartCard reg = null;
    for (RegistroSmartCard r : getRegistrosSmartCard()) {
      if (r.getTipoRegistro().equals(tipo)) {
        reg = r;
        return reg;
      }
    }
    return reg;
  }

  /**
   * Devuelve ultimo registro grabacion tarjeta.
   * 
   * @return ultimo registro grabacion tarjeta
   */
  @Transient
  public RegistroSmartCard getUltimoRegistroGrabacionTarjeta() {
    return getUltimoRegistroTarjeta(TipoRegistro.G);
  }

  /**
   * Devuelve ultimo registro impresion tarjeta.
   * 
   * @return ultimo registro impresion tarjeta
   */
  @Transient
  public RegistroSmartCard getUltimoRegistroImpresionTarjeta() {
    return getUltimoRegistroTarjeta(TipoRegistro.I);
  }

  /**
   * Devuelve ultimo registro lap.
   * 
   * @return ultimo registro lap
   */
  @Transient
  public RegistroLdap getUltimoRegistroLap() {
    if (getRegistrosLdap() != null && !getRegistrosLdap().isEmpty()) return getRegistrosLdap().get(0);
    return null;
  }

  @Transient
  private String getCompoundString(String addition) {
    if (addition != null) if (!(addition.trim()).isEmpty()) return " " + addition.trim();
    return "";
  }

  /**
   * Devuelve password m d5.
   * 
   * @return password m d5
   */
  @Transient
  public String getPasswordMD5() {
    return DigestUtils.md5Hex(getPassword() != null ? getPassword() : "");
  }

  /**
   * Devuelve id to print.
   * 
   * @return id to print
   */
  @Transient
  public String getIdToPrint() {
    String identificador = "" + getId();
    for (int i = 0; i < 7 - identificador.length(); i++) {
      identificador = "0" + identificador;
    }
    return identificador;
  }

  /**
   * Devuelve fecha creacion formateada.
   * 
   * @return fecha creacion formateada
   */
  @Transient
  public String getFechaCreacionFormateada() {
    DateFormat df = new SimpleDateFormat("dd/MM/yyyy");
    return getFechaCreacion() != null ? df.format(getFechaCreacion()).toString() : "";
  }
  
  /**
   * Devuelve fecha último acceso formateada.
   * 
   * @return fecha creacion formateada
   */
  @Transient
  public String getFechaUltimoAccesoFormateada() {
    DateFormat df = new SimpleDateFormat("dd/MM/yyyy");
    return getUltimoAcceso() != null ? df.format(getUltimoAcceso()).toString() : "";
  }

/**
   * Pre persist.
   */
  @PrePersist
  public void prePersist() {
    if (getId() == 0) setFechaCreacion(new Date());
  }

  /**
   * Devuelve formulario concreto.
   * 
   * @return formulario concreto
   */
  @ManyToOne(fetch = FetchType.LAZY, cascade = { CascadeType.PERSIST, CascadeType.MERGE, CascadeType.REFRESH })
  @JoinColumn(name = "idFormularioConcreto", nullable = false)
  @NotNull
  public FormularioConcreto getFormularioConcreto() {
    return formularioConcreto;
  }

  /**
   * Asigna formulario concreto.
   * 
   * @param formularioConcreto
   *          formulario concreto
   */
  public void setFormularioConcreto(FormularioConcreto formularioConcreto) {
    this.formularioConcreto = formularioConcreto;
  }

  /**
   * Asigna posee ordenador.
   * 
   * @param poseeOrdenador
   *          posee ordenador
   */
  public void setPoseeOrdenador(boolean poseeOrdenador) {
    this.poseeOrdenador = poseeOrdenador;
  }

  /**
   * Checks if is posee ordenador.
   * 
   * @return true, if is posee ordenador
   */

  
  public boolean isPoseeOrdenador() {
    return poseeOrdenador;
  }

  /**
   * Asigna acceso internet.
   * 
   * @param accesoInternet
   *          acceso internet
   */
  public void setAccesoInternet(boolean accesoInternet) {
    this.accesoInternet = accesoInternet;
  }

  /**
   * Checks if is acceso internet.
   * 
   * @return true, if is acceso internet
   */


  public boolean isAccesoInternet() {
    return accesoInternet;
  }

  /**
   * Asigna comunicacion tlf.
   * 
   * @param comunicacionTlf
   *          comunicacion tlf
   */
  public void setcomunicacionTlf(boolean comunicacionTlf) {
    this.comunicacionTlf = comunicacionTlf;
  }

  /**
   * Checks if is comunicacion tlf.
   * 
   * @return true, if is comunicacion tlf
   */
  public boolean iscomunicacionTlf() {
    return comunicacionTlf;
  }

  /**
   * Asigna comunicacion mail.
   * 
   * @param comunicacionMail
   *          comunicacion mail
   */
  public void setcomunicacionMail(boolean comunicacionMail) {
    this.comunicacionMail = comunicacionMail;
  }

  /**
   * Checks if is comunicacion mail.
   * 
   * @return true, if is comunicacion mail
   */
  public boolean iscomunicacionMail() {
    return comunicacionMail;
  }

  /**
   * Asigna comunicacion sms.
   * 
   * @param comunicacionSms
   *          comunicacion sms
   */
  public void setcomunicacionSms(boolean comunicacionSms) {
    this.comunicacionSms = comunicacionSms;
  }

  /**
   * Checks if is comunicacion sms.
   * 
   * @return true, if is comunicacion sms
   */
  public boolean iscomunicacionSms() {
    return comunicacionSms;
  }

  /**
   * Devuelve tipo discapacidad.
   * 
   * @return tipo discapacidad
   */
  @Enumerated(value = EnumType.STRING)
  @NotNull
  public TipoDiscapacidad getTipoDiscapacidad() {
    return tipoDiscapacidad;
  }

  /**
   * Asigna tipo discapacidad.
   * 
   * @param tipoDiscapacidad
   *          tipo discapacidad
   */
  public void setTipoDiscapacidad(TipoDiscapacidad tipoDiscapacidad) {
    this.tipoDiscapacidad = tipoDiscapacidad;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.entity.Usuario#getRolesList()
   */
  @Override
  @Transient
  public List<Role> getRolesList() {
    // Pido los roles del padre
    List<Role> roles = super.getRolesList();
    if (!roles.contains(Role.ROLE_USER)) roles.add(Role.ROLE_USER);
    return roles;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.entity.Usuario#getRoles()
   */
  @Override
  @Transient
  protected String getRoles() {
    StringBuilder sb = new StringBuilder("");
    for (Role rol : this.getRolesList()) {
      if (sb.length() != 0) sb.append(", ");
      sb.append(rol.toString());
    }
    return sb.toString();
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.entity.Usuario#getRolesArray()
   */
  @Override
  @Transient
  public Role[] getRolesArray() {
    return this.getRolesList().toArray(new Role[0]);
  }

/**
 * Devuelve frecuencia de conexión a internet
 * 
 * @return frecuencia
 */
@Enumerated(value = EnumType.STRING)
@NotNull
public TipoFrecuenciaInternet getTipoFrecuenciaInternet() {
	return tipoFrecuenciaInternet;
}

/**
 * Asigna frecuencia de conexión a internet
 * 
 * @param tipoFrecuenciaInternet
 *          tipo frecuencia internet
 */
public void setTipoFrecuenciaInternet(TipoFrecuenciaInternet tipoFrecuenciaInternet) {
	this.tipoFrecuenciaInternet = tipoFrecuenciaInternet;
	
	if (this.tipoFrecuenciaInternet == TipoFrecuenciaInternet.NUNCA)
		setTipoUtilidadInternet(TipoUtilidadInternet.NA);
}

/**
 * Devuelve para que utilizas internet
 * 
 * @return utilidad internet
 */
@Enumerated(value = EnumType.STRING)
@NotNull
public TipoUtilidadInternet getTipoUtilidadInternet() {
	return tipoUtilidadInternet;
}

/**
 * Asigna utilidad de internet
 * 
 * @param tipoUtilidadInternet
 *          tipo utilidad internet
 */
public void setTipoUtilidadInternet(TipoUtilidadInternet tipoUtilidadInternet) {
	if (this.tipoFrecuenciaInternet == TipoFrecuenciaInternet.NUNCA)
		this.tipoUtilidadInternet = TipoUtilidadInternet.NA;
	else
		this.tipoUtilidadInternet = tipoUtilidadInternet;
}

/**
 * Devuelve para que utilizas internet
 * 
 * @return utilidad internet
 */
@Enumerated(value = EnumType.STRING)
@NotNull
public TipoUtilizarSwLibre getTipoUtilizarSwLibre() {
	return tipoUtilizarSwLibre;
}

/**
 * Asigna Utilizar Sofware libre
 * 
 * @param tipoUtilizarSwLibre
 *          tipo de Sofware libre
 */
public void setTipoUtilizarSwLibre(TipoUtilizarSwLibre tipoUtilizarSwLibre) {
	this.tipoUtilizarSwLibre = tipoUtilizarSwLibre;
}

/**
 * Devuelve tipo de ocupación
 * 
 * @return tipo de ocupación
 */
@Enumerated(value = EnumType.STRING)
@NotNull
public TipoOcupacion getTipoOcupacion() {
	return tipoOcupacion;
}

/**
 * Asigna tipo de ocupación
 * 
 * @param TipoOcupacion
 *          tipo de ocupación
 */
public void setTipoOcupacion(TipoOcupacion tipoOcupacion) {
	this.tipoOcupacion = tipoOcupacion;
}

/**
 * Devuelve tipo de colectivo
 * 
 * @return tipo de colectivo
 */
@Enumerated(value = EnumType.STRING)
@NotNull
public TipoColectivo getTipoColectivo() {
	return tipoColectivo;
}

/**
 * Asigna tipo colectivo
 * 
 * @param tipoColectivo
 *          tipo colectivo
 */
public void setTipoColectivo(TipoColectivo tipoColectivo) {
	this.tipoColectivo = tipoColectivo;
}

/**
 * Devuelve tipo de estudios realizados
 * 
 * @return tipo de estudios
 */
@Enumerated(value = EnumType.STRING)
@NotNull
public TipoEstudiosRealizados getTipoEstudiosRealizados() {
	return tipoEstudiosRealizados;
}

/**
 * Asigna tipo de estudios realizados
 * 
 * @param tipoEstudiosRealizados
 *          tipo estudios realizados
 */
public void setTipoEstudiosRealizados(
		TipoEstudiosRealizados tipoEstudiosRealizados) {
	this.tipoEstudiosRealizados = tipoEstudiosRealizados;
}

/**
 * Devuelve tipo de sector profesional
 * 
 * @return tipo de sector profesional
 */
@Enumerated(value = EnumType.STRING)
public TipoSectorProfesional getTipoSectorProfesional() {
	return tipoSectorProfesional;
}

/**
 * Asigna tipo de sector profesional
 * 
 * @param tipoSectorProfesional
 *          tipo de sector profesional
 */
public void setTipoSectorProfesional(TipoSectorProfesional tipoSectorProfesional) {
	this.tipoSectorProfesional = tipoSectorProfesional;
}

}
