package es.csa.fdt.cyldigital.usuarios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

import javax.persistence.Transient;

import org.springframework.security.GrantedAuthority;
import org.springframework.security.GrantedAuthorityImpl;
import org.springframework.security.userdetails.UserDetails;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario.Role;

/**
 * Clase UsuarioLdap.
 * 
 * @author csa
 */
public class UsuarioLdap implements UserDetails {

  private static final long serialVersionUID = 1300506300127286763L;

  private String username;

  private List<Role> rolesList;

  private Centro centro;

  private Usuario user;

  private static final String ROLES_SIN_FILTRO_CENTRO = "ROLE_ADMIN,ROLE_AREA_MANAGER,ROLE_TRAINER,ROLE_EXTERN,ROLE_DIGITAL_ALLY"; //,ROLE_TRAINER_COORDINATOR,ROLE_MANAGER,ROLE_INFORMER
  private static final String ROLES_SIN_FILTRO_USUARIO = "ROLE_ADMIN,ROLE_AREA_MANAGER,ROLE_DIGITAL_ALLY"; //,ROLE_TRAINER_COORDINATOR,ROLE_MANAGER
  private static final String ROLES_SIN_FILTRO_CATEGORIA = "ROLE_ADMIN,ROLE_AREA_MANAGER,ROLE_DIGITAL_ALLY"; //,ROLE_TRAINER_COORDINATOR,ROLE_MANAGER
  private static final String ROLES_SIN_FILTRO_CATEGORIA_Y_USUARIO = "ROLE_ADMIN,ROLE_AREA_MANAGER,ROLE_DIGITAL_ALLY"; //,ROLE_TRAINER_COORDINATOR,ROLE_MANAGER

  /**
   * Construye un nuevo objeto de la clase
   */
  public UsuarioLdap() {
    rolesList = new ArrayList<Role>();
  }

  /**
   * Devuelve user.
   * 
   * @return user
   */
  public Usuario getUser() {
    return user;
  }

  /**
   * Asigna user.
   * 
   * @param user
   *          user
   */
  public void setUser(Usuario user) {
    this.user = user;
  }

  /**
   * Devuelve centro.
   * 
   * @return centro
   */
  public Centro getCentro() {
    if (centro == null && user != null) return user.getCentro();
    return centro;
  }

  /**
   * Asigna centro.
   * 
   * @param centro
   *          centro
   */
  public void setCentro(Centro centro) {
    this.centro = centro;
  }

  /**
   * Devuelve nombre completo.
   * 
   * @return nombre completo
   */
  public String getNombreCompleto() {
    if (user != null) return user.getNombreCompleto();
    else return getUsername();
  }
  

  // --------------------------------------------------------------------------------------------METODOS DE USER DETAILS

  /* (non-Javadoc)
   * @see org.springframework.security.userdetails.UserDetails#getUsername()
   */
  public String getUsername() {
    return username;
  }

  /**
   * Asigna username.
   * 
   * @param username
   *          username
   */
  public void setUsername(String username) {
    this.username = username;
  }

  /**
   * Devuelve roles list.
   * 
   * @return roles list
   */
  public List<Role> getRolesList() {
    List<Role> roles = new ArrayList<Role>();
    for (Role role : rolesList) {
      if (!roles.contains(role)) roles.add(role);
    }
    // Obtengo los del usuario
    if (getUser() != null) for (Role role : getUser().getRolesList()) {
      if (!roles.contains(role)) roles.add(role);
    }
    return roles;
  }

  /**
   * Asigna roles list.
   * 
   * @param rolesList
   *          roles list
   */
  public void setRolesList(List<Role> rolesList) {
    this.rolesList.clear();
    for (Role role : rolesList) {
      if (!rolesList.contains(role)) rolesList.add(role);
    }
  }

  /**
   * Devuelve roles.
   * 
   * @return roles
   */
  protected String getRoles() {
    StringBuilder sb = new StringBuilder("");
    for (Role rol : getRolesList()) {
      if (sb.length() != 0) sb.append(", ");
      sb.append(rol.toString());
    }
    return sb.toString();
  }

  /**
   * Asigna roles.
   * 
   * @param roles
   *          roles
   */
  protected void setRoles(String roles) {
    rolesList.clear();
    StringTokenizer st = new StringTokenizer(roles, ",");
    while (st.hasMoreTokens()) {
      try {
        Role role = Role.valueOf(st.nextToken().trim());
        if (!rolesList.contains(role)) rolesList.add(role);
      } catch (IllegalArgumentException e) {}
    }
  }

  /**
   * Devuelve roles array.
   * 
   * @return roles array
   */
  public Role[] getRolesArray() {
    return getRolesList().toArray(new Role[0]);
  }

  /**
   * Asigna roles array.
   * 
   * @param rolesArray
   *          roles array
   */
  public void setRolesArray(Role[] rolesArray) {
    this.rolesList.clear();
    for (Role role : rolesArray)
      this.rolesList.add(role);
  }

  /* (non-Javadoc)
   * @see org.springframework.security.userdetails.UserDetails#getAuthorities()
   */
  public GrantedAuthority[] getAuthorities() {
    List<Role> roles = this.getRolesList();
    GrantedAuthority[] gas = new GrantedAuthority[roles.size()];
    Iterator<Role> it = roles.iterator();
    for (int i = 0; it.hasNext(); i++)
      gas[i] = new GrantedAuthorityImpl(it.next().name());
    return gas;
  }

  /* (non-Javadoc)
   * @see org.springframework.security.userdetails.UserDetails#getPassword()
   */
  public String getPassword() {
    return "PASSWORD"; // Dummy password (estamos utilizando NTLM)
  }

  /**
   * Asigna password.
   * 
   * @param password
   *          password
   */
  public void setPassword(String password) {}

  /* (non-Javadoc)
   * @see org.springframework.security.userdetails.UserDetails#isAccountNonExpired()
   */
  public boolean isAccountNonExpired() {
    return true;
  }

  /* (non-Javadoc)
   * @see org.springframework.security.userdetails.UserDetails#isAccountNonLocked()
   */
  public boolean isAccountNonLocked() {
    return true;
  }

  /* (non-Javadoc)
   * @see org.springframework.security.userdetails.UserDetails#isCredentialsNonExpired()
   */
  public boolean isCredentialsNonExpired() {
    return true;
  }

  /* (non-Javadoc)
   * @see org.springframework.security.userdetails.UserDetails#isEnabled()
   */
  public boolean isEnabled() {
    return true;
  }

  /**
   * Checks if is apply filter centro.
   * 
   * @return true, if is apply filter centro
   */
  @Transient
  public boolean isApplyFilterCentro() {
    return applyFilterCentro();
  }

  /**
   * Checks if is apply filter empleado.
   * 
   * @return true, if is apply filter empleado
   */
  @Transient
  public boolean isApplyFilterEmpleado() {
    return applyFilterEmpleado();
  }

  /**
   * Checks if is apply filter usuario.
   * 
   * @return true, if is apply filter usuario
   */
  @Transient
  public boolean isApplyFilterUsuario() {
    return applyFilterUsuario();
  }

  /**
   * Apply filter centro.
   * 
   * @return true, if successful
   */
  public boolean applyFilterCentro() {
    boolean result = true;
    String[] aRoles = ROLES_SIN_FILTRO_CENTRO.split(",");
    for (String aRole : aRoles) {
      if (getRolesList().contains(Role.valueOf(aRole))) {
        result = false;
      }
    }
    return result;
  }

  /**
   * Apply filter categoria.
   * 
   * @return true, if successful
   */
  public boolean applyFilterCategoria() {
    boolean result = true;
    String[] aRoles = ROLES_SIN_FILTRO_CATEGORIA.split(",");
    for (String aRole : aRoles) {
      if (getRolesList().contains(Role.valueOf(aRole))) {
        result = false;
      }
    }
    return result;
  }

  /**
   * Apply filter categoriayusuario.
   * 
   * @return true, if successful
   */
  public boolean applyFilterCategoriayusuario() {
    boolean result = true;
    String[] aRoles = ROLES_SIN_FILTRO_CATEGORIA_Y_USUARIO.split(",");
    for (String aRole : aRoles) {
      if (getRolesList().contains(Role.valueOf(aRole))) {
        result = false;
      }
    }
    return result;
  }

  /**
   * Apply filter usuario.
   * 
   * @return true, if successful
   */
  public boolean applyFilterUsuario() {
    boolean result = true;
    String[] aRoles = ROLES_SIN_FILTRO_USUARIO.split(",");
    for (String aRole : aRoles) {
      if (getRolesList().contains(Role.valueOf(aRole))) {
        result = false;
      }
    }
    return result;
  }

  /**
   * Apply filter empleado.
   * 
   * @return true, if successful
   */
  public boolean applyFilterEmpleado() {
    boolean result = true;
    String[] aRoles = ROLES_SIN_FILTRO_CENTRO.split(",");
    for (String aRole : aRoles) {
      if (getRolesList().contains(Role.valueOf(aRole))) {
        result = false;
      }
    }
    return result;
  }

  /**
   * Checks if is empleado.
   * 
   * @return true, if is empleado
   */
  public boolean isEmpleado() {
    return user instanceof Empleado;
  }

  /**
   * Checks if is usuario.
   * 
   * @return true, if is usuario
   */
  public boolean isUsuario() {
    return user instanceof UsuarioCyL;
  }

}
