package es.csa.fdt.cyldigital.usuarios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.OrderBy;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Length;
import org.hibernate.validator.Pattern;

import es.csa.fdt.cyldigital.core.entity.AbstractEntity;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCursoOnline;
import es.csa.fdt.cyldigital.servicios.entity.PreinscripcionCursoOnline;

/**
 * Clase UsuarioOnline.
 * 
 * @author csa
 */
@Entity
@Table(name = "usuariosonline")
@AttributeOverride(name = "id", column = @Column(name = "idUsuarioOnline"))
@Audited
//@PrimaryKeyJoinColumn(name = "idUsuarioOnline")
public class UsuarioOnline extends AbstractEntity implements Serializable {

  private static final long serialVersionUID = -7648562176655166637L;
  private static final String EMAIL_PATTERN = 
			"^$|[_A-Za-z0-9-\\+]+(\\.[_A-Za-z0-9-]+)*@"
			+ "[A-Za-z0-9-]+(\\.[A-Za-z0-9]+)*(\\.[A-Za-z]{2,})$";

  private String idWeb;
  private String login;

  private String email;

  private Usuario usuario;

  /** matriculas cursos. */
  protected List<MatriculaCursoOnline> matriculasCursos;
  
  /** preinscripciones cursos. */
  protected List<PreinscripcionCursoOnline> preinscripcionesCursos;

  /**
   * Construye un nuevo objeto de la clase
   */
  public UsuarioOnline() {
    matriculasCursos = new ArrayList<MatriculaCursoOnline>();
    preinscripcionesCursos = new ArrayList<PreinscripcionCursoOnline>();
  }

  /**
   * Devuelve id web.
   * 
   * @return id web
   */
  public String getIdWeb() {
    return idWeb;
  }

  /**
   * Asigna id web.
   * 
   * @param idWeb
   *          id web
   */
  public void setIdWeb(String idWeb) {
    this.idWeb = idWeb;
  }

  /**
   * Devuelve login.
   * 
   * @return login
   */
  public String getLogin() {
    return login;
  }

  /**
   * Asigna login.
   * 
   * @param login
   *          login
   */
  public void setLogin(String login) {
    this.login = login;
  }

  /**
   * Devuelve email.
   * 
   * @return email
   */
  @Length(max = 255)
  //@Email
  @Pattern(regex=EMAIL_PATTERN, message = "El email no tiene el formato correcto")
  public String getEmail() {
    return email;
  }

  /**
   * Asigna email.
   * 
   * @param email
   *          email
   */
  public void setEmail(String email) {
    this.email = email == null ? null : email.trim();
  }

  /**
   * Devuelve usuario.
   * 
   * @return usuario
   */
  @OneToOne
  @JoinColumn(name = "idUsuario")
  public Usuario getUsuario() {
    return usuario;
  }

  /**
   * Asigna usuario.
   * 
   * @param usuario
   *          usuario
   */
  public void setUsuario(Usuario usuario) {
    this.usuario = usuario;
  }

  /**
   * Devuelve matriculas cursos.
   * 
   * @return matriculas cursos
   */
  @OneToMany(fetch = FetchType.LAZY, cascade = { javax.persistence.CascadeType.MERGE,
      javax.persistence.CascadeType.PERSIST, javax.persistence.CascadeType.REFRESH }, mappedBy = "usuario")
  @OrderBy(value = "fechaMatricula ASC")
  public List<MatriculaCursoOnline> getMatriculasCursos() {
    return matriculasCursos;
  }

  /**
   * Asigna matriculas cursos.
   * 
   * @param matriculasCursos
   *          matriculas cursos
   */
  public void setMatriculasCursos(List<MatriculaCursoOnline> matriculasCursos) {
    this.matriculasCursos = matriculasCursos;
  }

  /**
   * Devuelve preinscripciones cursos.
   * 
   * @return preinscripciones cursos
   */
  @OneToMany(fetch = FetchType.LAZY, cascade = { javax.persistence.CascadeType.MERGE,
      javax.persistence.CascadeType.PERSIST, javax.persistence.CascadeType.REFRESH }, mappedBy = "usuario")
  @OrderBy(value = "fecha ASC")
  public List<PreinscripcionCursoOnline> getPreinscripcionesCursos() {
    return preinscripcionesCursos;
  }

  /**
   * Asigna preinscripciones cursos.
   * 
   * @param preinscripcionesCursos
   *          preinscripciones cursos
   */
  public void setPreinscripcionesCursos(List<PreinscripcionCursoOnline> preinscripcionesCursos) {
    this.preinscripcionesCursos = preinscripcionesCursos;
  }

  /**
   * Devuelve nombre completo.
   * 
   * @return nombre completo
   */
  @Transient
  public String getNombreCompleto() {
    return login + "(" + idWeb + ")";
  }

  /**
   * Checks if is usuario cy l.
   * 
   * @return true, if is usuario cy l
   */
  @Transient
  public boolean isUsuarioCyL() {
    return usuario != null && usuario instanceof UsuarioCyL;
  }

  /**
   * Checks if is usuario empleado.
   * 
   * @return true, if is usuario empleado
   */
  @Transient
  public boolean isUsuarioEmpleado() {
    return usuario != null && usuario instanceof Empleado;
  }

}
