package es.csa.fdt.cyldigital.usuarios.entity;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.Calendar;
import java.util.Date;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotNull;
import org.hibernate.validator.Pattern;

/**
 * Clase UsuarioPre.
 * 
 * @author csa
 */
@Entity
@Table(name = "usuariospre")
@AttributeOverride(name = "id", column = @Column(name = "idUsuario"))
@Audited
@PrimaryKeyJoinColumn(name = "idUsuario")
public class UsuarioPre extends Usuario implements Serializable {
	private static final long serialVersionUID = -7648562176655166637L;
	
	private static final String EMAIL_PATTERN = 
			"^$|[_A-Za-z0-9-\\+]+(\\.[_A-Za-z0-9-]+)*@"
			+ "[A-Za-z0-9-]+(\\.[A-Za-z0-9]+)*(\\.[A-Za-z]{2,})$";
	
	private Date fechaNacimiento;
	private UsuarioCyL.Sexo sexo;
	private UsuarioCyL.TipoUsuario tipoUsuario;
	private UsuarioCyL.MedioPublicidad medioPublicidad;
	private String medioPublicidadOtro;
	private UsuarioCyL.TipoFrecuenciaInternet tipoFrecuenciaInternet;
	private UsuarioCyL.TipoUtilidadInternet tipoUtilidadInternet;
	private UsuarioCyL.TipoUtilizarSwLibre tipoUtilizarSwLibre;
	private UsuarioCyL.TipoOcupacion tipoOcupacion;	
	private UsuarioCyL.TipoColectivo tipoColectivo;	
	private UsuarioCyL.TipoSectorProfesional tipoSectorProfesional;
	private UsuarioCyL.TipoEstudiosRealizados tipoEstudiosRealizados;	
	private UsuarioCyL.TipoDiscapacidad tipoDiscapacidad;
	private boolean poseeOrdenador;	
	private boolean accesoInternet;	
	private boolean menor;
	private boolean extranjero;	
	private String pasaporte;
	private UsuarioCyL.TipoIdentificacion tipoIdentificacion;
	private String nifAutorizado;
	private String nombreCompletoAutorizado;	
	private String movil;
	private String email;
	private boolean comunicacionTlf;
	private boolean comunicacionMail;
	private boolean comunicacionSms;
	private boolean aceptaPolitica;
	private boolean cesionPermitida;
	
	//PR-XEA -> Alta de menor con NIF está a fallar. Posibilidad de dar de alta usuarios menores de 14 años y con NIF.
	//private boolean menorConNIF;
	
	/**
	 * Construye un nuevo objeto de la clase
	 */
	public UsuarioPre() {
		super();

		setPoseeOrdenador(false);
		setAccesoInternet(false);
		setMenor(false);
		setExtranjero(false);

		setComunicacionTlf(true);
		setComunicacionMail(true);
	    setComunicacionSms(false);

		setAceptaPolitica(false);
		//3.4.3-R RF_XEA_05 - Cambio criterio campo cesion permitida
		//setCesionPermitida(true);
		setCesionPermitida(false);
	}

	/* (non-Javadoc)
	 * @see es.csa.fdt.cyldigital.usuarios.entity.Usuario#getDescripcion()
	 */
	@Override
	@Transient
	public String getDescripcion() {
		String descripcion = "";
		
		if (!getNif().isEmpty())
			descripcion += " (NIF: " + getNif() + ") ";
		
		if (!getTelefono().trim().isEmpty())
			descripcion += " (Tlf: " + getTelefono() + ")";
		
		if (getCentro() != null)
			descripcion += " (Centro: " + getCentro().getNombre() + ") ";

		return descripcion.trim();
	}

	/* (non-Javadoc)
	 * @see es.csa.fdt.cyldigital.usuarios.entity.Usuario#isUsuarioCyL()
	 */
	@Override
	@Transient
	public boolean isUsuarioCyL() {
		return false;
	}

	/* (non-Javadoc)
	 * @see es.csa.fdt.cyldigital.usuarios.entity.Usuario#isUsuarioPre()
	 */
	@Override
	@Transient
	public boolean isUsuarioPre() {
		return true;
	}

	/* (non-Javadoc)
	 * @see es.csa.fdt.cyldigital.usuarios.entity.Usuario#isEmpleado()
	 */
	@Override
	@Transient
	public boolean isEmpleado() {
		return false;
	}
	
	/**
	 * Devuelve edad.
	 * 
	 * @return edad
	 */
	@Transient
	public Integer getEdad() {
		if (fechaNacimiento == null)
			return null;

		Calendar calendarNacimiento = Calendar.getInstance();
		calendarNacimiento.setTime(fechaNacimiento);
	    Calendar calendarHoy = Calendar.getInstance();
	    
	    int factor = 0;
	    if (calendarHoy.get(Calendar.DAY_OF_YEAR) < calendarNacimiento.get(Calendar.DAY_OF_YEAR))
	    	factor = -1;

	    int edad = calendarHoy.get(Calendar.YEAR) - calendarNacimiento.get(Calendar.YEAR) + factor;

	    return edad;
	}
	
	/**
	 * Devuelve direccion completa.
	 * 
	 * @return direccion completa
	 */
	@Transient
	public String getDireccionCompleta() {
		String direccionCompleta = "";
		
		if (getLocalidad() != null) {
			direccionCompleta += getCompoundString(getDireccion());
			direccionCompleta += getCompoundString(getCp());
			direccionCompleta += getCompoundString(getLocalidad().getNombre());
			
			if (!getCompoundString(getLocalidad().getProvincia().getNombre()).isEmpty())
				direccionCompleta += " (" + getCompoundString(getLocalidad().getProvincia().getNombre()).trim() + ")";
		}
		else {
			if (getDireccion() != null && !getDireccion().trim().isEmpty()) {
				direccionCompleta += getCompoundString(getDireccion());
				direccionCompleta += getCompoundString(getCp());
				direccionCompleta += getCompoundString(" -Sin localidad- ");
				
				if (getProvincia() != null && !getCompoundString(getProvincia().getNombre()).isEmpty()) {
					direccionCompleta += " (" + getCompoundString(getProvincia().getNombre()).trim() + ")";
				}
				else {
					direccionCompleta += " ( Sin provincia )";
				}
			}
		}

	    return direccionCompleta;
	}
	
	@Transient
	private String getCompoundString(String addition) {
		if (addition != null) if (!(addition.trim()).isEmpty())
			return " " + addition.trim();
		
		return "";
	}

	/**
	 * @return the fechaNacimiento
	 */
	@Temporal(TemporalType.DATE)
	@NotNull
	public Date getFechaNacimiento() {
		return fechaNacimiento;
	}

	/**
	 * @param fechaNacimiento the fechaNacimiento to set
	 */
	public void setFechaNacimiento(Date fechaNacimiento) {
		this.fechaNacimiento = fechaNacimiento;
	}

	/**
	 * @return the sexo
	 */
	@Enumerated(value = EnumType.STRING)
	@NotNull
	public UsuarioCyL.Sexo getSexo() {
		return sexo;
	}

	/**
	 * @param sexo the sexo to set
	 */
	public void setSexo(UsuarioCyL.Sexo sexo) {
		this.sexo = sexo;
	}

	/**
	 * @return the tipoUsuario
	 */
	@Enumerated(value = EnumType.STRING)
	@Override
	@NotNull
	public UsuarioCyL.TipoUsuario getTipoUsuario() {
		return tipoUsuario;
	}

	/**
	 * @param tipoUsuario the tipoUsuario to set
	 */
	public void setTipoUsuario(UsuarioCyL.TipoUsuario tipoUsuario) {
		this.tipoUsuario = tipoUsuario;
	}

	/**
	 * @return the medioPublicidad
	 */
	@Enumerated(value = EnumType.STRING)
	@NotNull
	public UsuarioCyL.MedioPublicidad getMedioPublicidad() {
		return medioPublicidad;
	}

	/**
	 * @param medioPublicidad the medioPublicidad to set
	 */
	public void setMedioPublicidad(UsuarioCyL.MedioPublicidad medioPublicidad) {
		this.medioPublicidad = medioPublicidad;
	}

	/**
	 * @return the medioPublicidadOtro
	 */
	@Length(max = 255)
	public String getMedioPublicidadOtro() {
		return medioPublicidadOtro;
	}

	/**
	 * @param medioPublicidadOtro the medioPublicidadOtro to set
	 */
	public void setMedioPublicidadOtro(String medioPublicidadOtro) {
		this.medioPublicidadOtro = medioPublicidadOtro;
	}

	/**
	 * @return the tipoFrecuenciaInternet
	 */
	@Enumerated(value = EnumType.STRING)
	@NotNull
	public UsuarioCyL.TipoFrecuenciaInternet getTipoFrecuenciaInternet() {
		return tipoFrecuenciaInternet;
	}

	/**
	 * @param tipoFrecuenciaInternet the tipoFrecuenciaInternet to set
	 */
	public void setTipoFrecuenciaInternet(UsuarioCyL.TipoFrecuenciaInternet tipoFrecuenciaInternet) {
		this.tipoFrecuenciaInternet = tipoFrecuenciaInternet;
		
		if (this.tipoFrecuenciaInternet == UsuarioCyL.TipoFrecuenciaInternet.NUNCA)
			setTipoUtilidadInternet(UsuarioCyL.TipoUtilidadInternet.NA);
	}

	/**
	 * @return the tipoUtilidadInternet
	 */
	@Enumerated(value = EnumType.STRING)
	@NotNull
	public UsuarioCyL.TipoUtilidadInternet getTipoUtilidadInternet() {
		return tipoUtilidadInternet;
	}

	/**
	 * @param tipoUtilidadInternet the tipoUtilidadInternet to set
	 */
	public void setTipoUtilidadInternet(UsuarioCyL.TipoUtilidadInternet tipoUtilidadInternet) {
		if (this.tipoFrecuenciaInternet == UsuarioCyL.TipoFrecuenciaInternet.NUNCA)
			this.tipoUtilidadInternet = UsuarioCyL.TipoUtilidadInternet.NA;
		else
			this.tipoUtilidadInternet = tipoUtilidadInternet;
	}

	/**
	 * @return the tipoUtilizarSwLibre
	 */
	@Enumerated(value = EnumType.STRING)
	@NotNull
	public UsuarioCyL.TipoUtilizarSwLibre getTipoUtilizarSwLibre() {
		return tipoUtilizarSwLibre;
	}

	/**
	 * @param tipoUtilizarSwLibre the tipoUtilizarSwLibre to set
	 */
	public void setTipoUtilizarSwLibre(UsuarioCyL.TipoUtilizarSwLibre tipoUtilizarSwLibre) {
		this.tipoUtilizarSwLibre = tipoUtilizarSwLibre;
	}

	/**
	 * @return the tipoOcupacion
	 */
	@Enumerated(value = EnumType.STRING)
	@NotNull
	public UsuarioCyL.TipoOcupacion getTipoOcupacion() {
		return tipoOcupacion;
	}

	/**
	 * @param tipoOcupacion the tipoOcupacion to set
	 */
	public void setTipoOcupacion(UsuarioCyL.TipoOcupacion tipoOcupacion) {
		this.tipoOcupacion = tipoOcupacion;
	}

	/**
	 * @return the tipoColectivo
	 */
	@Enumerated(value = EnumType.STRING)
	@NotNull
	public UsuarioCyL.TipoColectivo getTipoColectivo() {
		return tipoColectivo;
	}

	/**
	 * @param tipoColectivo the tipoColectivo to set
	 */
	public void setTipoColectivo(UsuarioCyL.TipoColectivo tipoColectivo) {
		this.tipoColectivo = tipoColectivo;
	}

	/**
	 * @return the tipoSectorProfesional
	 */
	@Enumerated(value = EnumType.STRING)
	public UsuarioCyL.TipoSectorProfesional getTipoSectorProfesional() {
		return tipoSectorProfesional;
	}

	/**
	 * @param tipoSectorProfesional the tipoSectorProfesional to set
	 */
	public void setTipoSectorProfesional(
			UsuarioCyL.TipoSectorProfesional tipoSectorProfesional) {
		this.tipoSectorProfesional = tipoSectorProfesional;
	}

	/**
	 * @return the tipoEstudiosRealizados
	 */
	@Enumerated(value = EnumType.STRING)
	@NotNull
	public UsuarioCyL.TipoEstudiosRealizados getTipoEstudiosRealizados() {
		return tipoEstudiosRealizados;
	}

	/**
	 * @param tipoEstudiosRealizados the tipoEstudiosRealizados to set
	 */
	public void setTipoEstudiosRealizados(
			UsuarioCyL.TipoEstudiosRealizados tipoEstudiosRealizados) {
		this.tipoEstudiosRealizados = tipoEstudiosRealizados;
	}

	/**
	 * @return the tipoDiscapacidad
	 */
	@Enumerated(value = EnumType.STRING)
	@NotNull
	public UsuarioCyL.TipoDiscapacidad getTipoDiscapacidad() {
		return tipoDiscapacidad;
	}

	/**
	 * @param tipoDiscapacidad the tipoDiscapacidad to set
	 */
	public void setTipoDiscapacidad(UsuarioCyL.TipoDiscapacidad tipoDiscapacidad) {
		this.tipoDiscapacidad = tipoDiscapacidad;
	}

	/**
	 * @return the poseeOrdenador
	 */
	
	public boolean isPoseeOrdenador() {
		return poseeOrdenador;
	}

	/**
	 * @param poseeOrdenador the poseeOrdenador to set
	 */
	public void setPoseeOrdenador(boolean poseeOrdenador) {
		this.poseeOrdenador = poseeOrdenador;
	}

	/**
	 * @return the accesoInternet
	 */
	public boolean isAccesoInternet() {
		return accesoInternet;
	}

	/**
	 * @param accesoInternet the accesoInternet to set
	 */
	public void setAccesoInternet(boolean accesoInternet) {
		this.accesoInternet = accesoInternet;
	}

	/**
	 * @return the menor
	 */
	public boolean isMenor() {
		return menor;
	}

	/**
	 * @param menor the menor to set
	 */
	public void setMenor(boolean menor) {
		this.menor = menor;
	}

	/**
	 * @return the extranjero
	 */
	public boolean isExtranjero() {
		return extranjero;
	}

	/**
	 * @param extranjero the extranjero to set
	 */
	public void setExtranjero(boolean extranjero) {
		this.extranjero = extranjero;
	}

	/**
	 * @return the pasaporte
	 */
	@Length(max = 100)
	public String getPasaporte() {
		return pasaporte;
	}

	/**
	 * @param pasaporte the pasaporte to set
	 */
	public void setPasaporte(String pasaporte) {
		this.pasaporte = pasaporte;
	}

	/**
	 * @return the tipoIdentificacion
	 */
	@Enumerated(value = EnumType.STRING)
	@NotNull
	public UsuarioCyL.TipoIdentificacion getTipoIdentificacion() {
		return tipoIdentificacion;
	}

	/**
	 * @param tipoIdentificacion the tipoIdentificacion to set
	 */
	public void setTipoIdentificacion(
			UsuarioCyL.TipoIdentificacion tipoIdentificacion) {
		this.tipoIdentificacion = tipoIdentificacion;
	}

	/**
	 * @return the nifAutorizado
	 */
	@Length(max = 100)
	public String getNifAutorizado() {
		return nifAutorizado;
	}

	/**
	 * @param nifAutorizado the nifAutorizado to set
	 */
	public void setNifAutorizado(String nifAutorizado) {
		this.nifAutorizado = nifAutorizado;
	}

	/**
	 * @return the nombreCompletoAutorizado
	 */
	@Length(max = 255)
	public String getNombreCompletoAutorizado() {
		return nombreCompletoAutorizado;
	}

	/**
	 * @param nombreCompletoAutorizado the nombreCompletoAutorizado to set
	 */
	public void setNombreCompletoAutorizado(String nombreCompletoAutorizado) {
		this.nombreCompletoAutorizado = nombreCompletoAutorizado;
	}

	/**
	 * @return the movil
	 */
	@Length(max = 20)
	public String getMovil() {
		return movil;
	}

	/**
	 * @param movil the movil to set
	 */
	public void setMovil(String movil) {
		this.movil = movil;
	}

	/**
	 * @return the email
	 */
	@Length(max = 255)
	//@Email
	@Pattern(regex=EMAIL_PATTERN, message = "El email no tiene el formato correcto")
	public String getEmail() {
		return email;
	}

	/**
	 * @param email the email to set
	 */
	public void setEmail(String email) {
		this.email = email;
	}

	/**
	 * @return the comunicacionTlf
	 */
	public boolean isComunicacionTlf() {
		return comunicacionTlf;
	}

	/**
	 * @param comunicacionTlf the comunicacionTlf to set
	 */
	public void setComunicacionTlf(boolean comunicacionTlf) {
		this.comunicacionTlf = comunicacionTlf;
	}

	/**
	 * @return the comunicacionMail
	 */
	public boolean isComunicacionMail() {
		return comunicacionMail;
	}

	/**
	 * @param comunicacionMail the comunicacionMail to set
	 */
	public void setComunicacionMail(boolean comunicacionMail) {
		this.comunicacionMail = comunicacionMail;
	}

	/**
	 * @return the comunicacionSms
	 */
	public boolean isComunicacionSms() {
		return comunicacionSms;
	}

	/**
	 * @param comunicacionSms the comunicacionSms to set
	 */
	public void setComunicacionSms(boolean comunicacionSms) {
		this.comunicacionSms = comunicacionSms;
	}

	/**
	 * @return the aceptaPolitica
	 */
	@NotNull
	public boolean isAceptaPolitica() {
		return aceptaPolitica;
	}

	/**
	 * @param aceptaPolitica the aceptaPolitica to set
	 */
	public void setAceptaPolitica(boolean aceptaPolitica) {
		this.aceptaPolitica = aceptaPolitica;
	}

	/**
	 * @return the cesionPermitida
	 */
	public boolean isCesionPermitida() {
		return cesionPermitida;
	}

	/**
	 * @param cesionPermitida the cesionPermitida to set
	 */
	public void setCesionPermitida(boolean cesionPermitida) {
		this.cesionPermitida = cesionPermitida;
	}
}
