package es.csa.fdt.cyldigital.usuarios.service;

import java.util.Date;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.List;

import es.csa.fdt.cyldigital.core.service.AbstractService;
import es.csa.fdt.cyldigital.servicios.entity.Curso;
import es.csa.fdt.cyldigital.servicios.entity.Taller;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;

/**
 * Interfaz del servicio de Empleado.
 * 
 * @author csa
 */
public interface EmpleadoService extends AbstractService<Empleado, Long> {

  /**
   * Persist empleado.
   * 
   * @param personal
   *           personal
   * @param user
   *           user
   * @param oldUserName
   *           oldUserName
   * @return true, if successful
   */
  public boolean persistEmpleado(Empleado personal, UsuarioLdap user,String oldUserName);

  /**
   * Devuelve personal disponible.
   * 
   * @return personal disponible
   */
  public List<Empleado> getPersonalDisponible();

  /**
   * Devuelve cursos.
   * 
   * @return cursos
   */
  public List<Curso> getCursos();

  /**
   * Devuelve talleres.
   * 
   * @return talleres
   */
  public List<Taller> getTalleres();


  /**
   * Devuelve la fecha del ultimo cambio de contraseña. En dias epoch.
   * 
   * @param empleado
   * 			empleado
   * 
   * @return Long 
   */
  public Long getPasswordLastChange(Empleado personal);

    /**
   * Devuelve la fecha del ultimo cambio de contraseña. 
   * 
   * @param empleado
   * 			empleado
   * 
   * @return Date 
   */
  public Date getPasswordLastChangeDate(Empleado personal);
  
  /**
   * Devuelve los que quedan de valided de la contraseña. 
   */
  public Long getDaysPasswordValid(Empleado personal);
  
  /**
   * Devuelve true si la contraseña ha expirado.
   * 
   * @param empleado
   * 			empleado
   * 
   * @return boolean
   */
  public boolean isPasswordExpired(Empleado personal);
    
}
