package es.csa.fdt.cyldigital.usuarios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2017  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.regex.Pattern;

import javax.annotation.Resource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.util.AditionalStringUtils;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.notificacion.NotificationService;
import es.csa.fdt.cyldigital.notificacion.entity.Informe;
import es.csa.fdt.cyldigital.notificacion.entity.InformeNotification;
import es.csa.fdt.cyldigital.notificacion.entity.InformeNotification.Periodicidad;
import es.csa.fdt.cyldigital.notificacion.entity.InformeNotification.Periodo;
import es.csa.fdt.cyldigital.servicios.entity.Curso;
import es.csa.fdt.cyldigital.servicios.entity.Taller;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado.Puesto;
import es.csa.fdt.cyldigital.usuarios.entity.Localidad;
import es.csa.fdt.cyldigital.usuarios.entity.Proveedor;
import es.csa.fdt.cyldigital.usuarios.entity.Provincia;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario.Role;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;
import es.csa.fdt.cyldigital.util.SelectFilterByCenterAux;


/**
 * Clase EmpleadoServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class EmpleadoServiceImpl extends AbstractServiceImpl<Empleado, Long> implements EmpleadoService {

  private static final long serialVersionUID = 918589720985035611L;
  private List<Curso> cursos;
  private List<Taller> talleres;
  private List<Centro> centros;
  private List<Proveedor> proveedores;
  
  // private static final Logger logger = Logger.getLogger(EmpleadoServiceImpl.class.getName());

  @Resource
  private transient HashMap<String, Object> generalProperties;

  @Autowired
  private LdapComponent ldap;
  
  /**
   * Construye un nuevo objeto de la clase
   */
  public EmpleadoServiceImpl() {
    super();
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.EmpleadoService#persistEmpleado(es.csa.fdt.cyldigital.usuarios.entity.Empleado, es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap)
   */
  public boolean persistEmpleado(Empleado personal, UsuarioLdap user, String oldUserName) {
    if (personal.getUsername() == null || personal.getUsername().isEmpty()) {
      enableMessages();
      addErrorMessage("username.empleado.vacio");
      return false;
    }
    
    //3.4.3-R RF_XEA_19 
    //Transformar el nombre y apellidos del personal
    if ((personal.getNombre() != null) && !personal.getNombre().isEmpty())
    	personal.setNombre(AditionalStringUtils.formatCapitalized(personal.getNombre()));
    if ((personal.getApellido1() != null) && !personal.getApellido1().isEmpty())
    	personal.setApellido1(AditionalStringUtils.formatCapitalized(personal.getApellido1()));
    if ((personal.getApellido2() != null) && !personal.getApellido2().isEmpty())
    	personal.setApellido2(AditionalStringUtils.formatCapitalized(personal.getApellido2()));
    
    List<Empleado> empleados = findWhere("username", personal.getUsername());
    if (empleados != null && !empleados.isEmpty()) {
    	for(Empleado e : empleados)
    		if (e.getId() != personal.getId()) {
    			enableMessages();
    		    addErrorMessage("username.empleado.existe.empleado");
    		    
    		    return false;
    		}

    	/*&& (empleados.get(0).getId() != personal.getId())) {
        enableMessages();
        addErrorMessage("username.empleado.existe.empleado");
        return false;*/
    }
    
    List<UsuarioCyL> usuarios = findWhere(UsuarioCyL.class, "username", personal.getUsername());
    if (usuarios != null && !usuarios.isEmpty()) {
      enableMessages();
      addErrorMessage("username.empleado.existe.usuario");
      return false;
    }
    if (personal.getUsername() == null || personal.getUsername().isEmpty()) {
      enableMessages();
      addErrorMessage("username.empleado.vacio");
      return false;
    }
    if (personal.getPuesto() == Puesto.ONLINE_TRAINER
        && (personal.getUsernameWeb() == null || personal.getUsernameWeb().isEmpty() || personal.getIdWeb() == null || personal
            .getIdWeb().isEmpty())) {
      if (user.getRolesList().contains(Role.ROLE_ADMIN)) {
        enableMessages();
        addErrorMessage("profesor.online.vacio.admin");
        return false;
      } else {
        enableMessages();
        addInfoMessage("profesor.online.vacio");
      }
    }  

    persist(personal);
    
    // generamos los informes que debe recibir el empleado. Puede pasar a ser AREA MANAGER O dejar de serlo. Los
    // borramos todos y los generamos de nuevo.
    disableMessages();
    Informe informe = findOneWhere(Informe.class, "nombre", "Informe de inventario");
    List<InformeNotification> informes = findWhere(InformeNotification.class, "informe", informe);
    for (InformeNotification i : informes) {
      remove(i);
    }

    Calendar c = Calendar.getInstance();
    if (Calendar.getInstance().get(Calendar.DAY_OF_MONTH) < 15) c.set(Calendar.DAY_OF_MONTH, 15);
    else c.set(Calendar.DAY_OF_MONTH, 1);

    c.set(Calendar.HOUR_OF_DAY, 9);
    c.set(Calendar.MINUTE, 0);

    empleados = findWhere(Empleado.class, "puesto", Puesto.AREA_MANAGER);
    empleados.addAll(findWhere(Empleado.class, "puesto", Puesto.DIGITAL_ALLY_MANAGER));
    
    for (Empleado e : empleados) {
      InformeNotification i = new InformeNotification();

      i.setFecha(new Date());
      i.setHora(c.getTime());
      i.setFechaProximaEjecucion(c.getTime());

      i.setInforme(informe);
      i.setCentro(e.getCentro());
      i.setPeriodicidad(Periodicidad.QUINCENAL);
      i.setPeriodo(Periodo.MES_PASADO);
      i.getEmpleados().add(e);

      persist(i);
    }

    /*boolean creacion = personal.getId() == 0;

    if (creacion) {
      ldap.createUser(personal, this);
      ldap.updateGroup(personal.getCentro().getDominio().replaceAll(" ", ""), personal, this);
      ldap.updateGroup(findOneWhere(Parameter.class, "nombre", "openldap_nombreGrupoAgentesTIC").getValor(), personal, this);
    } else {
      // ldap.updateUser(usuarioCyL, oldUsername);
      ldap.actualizarInformacionLDAP(personal, oldUserName, this);
    }*/

    return true;
  }

  /**
   * Elimina empleado.
   * 
   * @param personal
   *           personal
   */
  public void removeEmpleado(Empleado personal) {
    if (personal.getId() != 0) {
      personal.setDesactivado(true);
      personal.setProveedor(null);
      persist(personal);
    }
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.EmpleadoService#getPersonalDisponible()
   */
  public List<Empleado> getPersonalDisponible() {
    return findAll(Empleado.class);
  }

  /**
   * Devuelve centros.
   * 
   * @return centros
   */
  public List<Centro> getCentros() {
    if (centros == null) centros = findAllOrder(Centro.class, "nombre", true);
    return centros;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.EmpleadoService#getCursos()
   */
  public List<Curso> getCursos() {
    if (cursos == null) cursos = findAllOrder(Curso.class, "nombre", true);
    return cursos;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.EmpleadoService#getTalleres()
   */
  public List<Taller> getTalleres() {
    if (talleres == null) talleres = findAllOrder(Taller.class, "nombre", true);
    return talleres;
  }

  /**
   * Devuelve proveedores.
   * 
   * @return proveedores
   */
  public List<Proveedor> getProveedores() {
    if (proveedores == null) proveedores = findAllOrder(Proveedor.class, "nombre", true);
    return proveedores;
  }
  
  public String getUserEmailAddress(Long idPersonal) {
	  return find(idPersonal).getEmail();
  }

  /**
   * Actualiza centro.
   * 
   * @param empleado
   *           empleado
   * @param currentUser
   *           current user
   */
  public void actualizaCentro(Empleado empleado, UsuarioLdap currentUser, SelectFilterByCenterAux serviceCenterAux) {
    if (currentUser.getCentro() != null) {
    	Centro c;
    	
    	if (empleado.getPuesto() != null) {
	    	switch (empleado.getPuesto()) {
	    		case EXTERNAL_PERSONAL:
	    			c = serviceCenterAux.getCentroColaboradores();
	    			break;
	    	
	    		case ADMIN:
	    			c = serviceCenterAux.getCentroTest();
	    			break;
	    		
	    		default:
	    			c = find(Centro.class, currentUser.getCentro().getId());
	    	}
    	}
    	else
    		c = find(Centro.class, currentUser.getCentro().getId());
    	
    	empleado.setCentro(c);
    } else if (generalProperties.containsKey("DEFAULT_CENTER_ID") && empleado.getCentro() == null) {
      Centro c = find(Centro.class, Long.valueOf((String) generalProperties.get("DEFAULT_CENTER_ID")));
      if (c != null) {
        empleado.setCentro(c);
      }
    }
  }
  
  public Localidad getSinLocalidad() {
	  return find(Localidad.class, 0L);
  }  
  
  public Provincia getSinProvincia() {
	  return find(Provincia.class, 0L);
  }
  
  public void synchronizeDirectory(Empleado empleado, String oldUserName, Centro oldCentro, Boolean isNew) {
	  if (isNew) {
	      ldap.createUser(empleado, this);
	      ldap.updateGroup(empleado.getCentro().getDominio().replaceAll(" ", ""), empleado, this);
	      
	      if (empleado.getPuesto().equals(Puesto.AREA_MANAGER))
	    	  ldap.updateGroup(findOneWhere(Parameter.class, "nombre", "openldap_nombreGrupoAgentesTIC").getValor(), empleado, this);
	      
	  } else {
	      // ldap.updateUser(usuarioCyL, oldUsername);
	      ldap.actualizarInformacionLDAP(empleado, oldUserName, oldCentro, this);
	  }
  }
  
  //3.4.8 - RF_15_38
  public Boolean changePersonalPassword(Long idEmpleado, String newPassword1, String newPassword2) {
	  try {		  

		  Empleado personal = find(idEmpleado);

		  if (personal != null) {
			  newPassword1 = newPassword1.trim();
			  newPassword2 = newPassword2.trim();

			  if (newPassword1.compareTo(newPassword2) != 0) 
				  addErrorMessage("password.change.passwordNoMatch");
			  else if ((newPassword1.length() < 8) || (newPassword1.length() > 14) || newPassword1.contains(" "))
				  addErrorMessage("password.change.passwordLength");				  
			  else if (!passwordPattern(newPassword1)){
				  addErrorMessage("password.change.caracteresEspeciales"); //RF_15_46 - Relaxar as restriccións dos contrasinais  
			  } else {
				  Boolean result = false;

				  //Almacenamos el hash de la contraseña del LDAP en caso que no estubiera almacenada. 
				  String sHashContrasinalLDAP = ldap.getObtenerHashContrasinal(personal, this);	
				  if(!personal.getListHistoricoPswdLDAP().contains(sHashContrasinalLDAP)){
					  personal.addHistoricoPswdLDAP(sHashContrasinalLDAP);
					  disableMessages();
					  persist(personal);
					  enableMessages();
				  }
				  
				  //Si el establecimiento de la contraseña es CORRECTO
				  if (ldap.setPassword(personal, newPassword1, this) == 0) {					  
					  result = true;
					  
					  //Si está activo verificamos la nueva contraseña en el historico de contraseñas
					  String  indicadorCheckHistorico = findOneWhere(Parameter.class, "nombre", "employees_pswd_hist_check").getValor();
					  String sHashNuevaContrasinalLDAP = ldap.getObtenerHashContrasinal(personal, this);
					  if (indicadorCheckHistorico.equals("1") && personal.getListHistoricoPswdLDAP().contains(sHashNuevaContrasinalLDAP) && personal.getPuesto().equals(Puesto.AREA_MANAGER)){						  
						  addErrorMessage("password.change.repeated");
						  personal.setCambioObligatorioPswdLDAP(true);
						  result = false;
					  } else {						  
						  if(!personal.getListHistoricoPswdLDAP().contains(sHashContrasinalLDAP))
							  personal.addHistoricoPswdLDAP(sHashNuevaContrasinalLDAP);
						  personal.setCambioObligatorioPswdLDAP(false);
						  
						  if(personal.getEmail() != null && !personal.getEmail().isEmpty())
							  notificarChangePassword(personal, newPassword1);
					  }

				  } else { //Si el establecimiento de la contraseña es INCORRECTO
					  addErrorMessage("password.change.fail");
				  }

				  //Guardamos el usuario
				  disableMessages();
				  persist(personal);
				  enableMessages();

				  return result;
			  }
		  } else
			  addErrorMessage("password.change.noUser");
	  }
	  catch (Exception ex) {
		  addErrorMessage("password.change.fail");
		  ex.printStackTrace();
	  }
	  return false;
  }
  
  /**
   * Notificar reestablecimiento de contraseña.
   * 
   * @param emailAddress
   *           dirección de Email
   *           
   * @return true, if successful
   */
  @SuppressWarnings({ "rawtypes", "unchecked" })
  private void notificarChangePassword(Empleado personal, String password) {
	  NotificationService n = new NotificationService();

	  try {
		  if (!n.notificarChangePassword(this, personal, password))
			  addErrorMessage("mail.errormail");
	  } catch (Exception e){
		  e.printStackTrace();
	  }
  }
  
  //RF_15_46 - Relaxar as restriccións dos contrasinais  
  private boolean passwordPattern(String password) {
	  final String PASSWORD_PATTERN = "^([a-zA-Z0-9@.]{8,14})$";
	  Pattern p = Pattern.compile(PASSWORD_PATTERN);
	  return p.matcher(password).matches();
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.EmpleadoService#getPasswordLastChange(Empleado personal);
   */
  public Long getPasswordLastChange(Empleado personal){
	  Long value = null;
	  
	  try {
		  if(personal!=null){		  
			  value = ldap.getPasswordLastChange(personal, this);
		  }
	  } catch (NullPointerException ex){
		  //no existe el atributo de expiración para este usuario, así que no se hace nada.
	  }

	  return value;
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.EmpleadoService#getPasswordLastChangeDate(Empleado personal);
   */
  public Date getPasswordLastChangeDate(Empleado personal){
	  Date value = null;
	  
	  Long passwordLastChange = getPasswordLastChange(personal);
	  
	  if(passwordLastChange!=null){
		  value = new Date(passwordLastChange * 1000 * 60 * 60 * 24);
	  }

	  return value;
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.EmpleadoService#getDaysPasswordValid(Empleado personal);
   */
  public Long getDaysPasswordValid(Empleado personal){
	  
	  // Obtenemos el parametro de expiracion
	  long lPasswordExpiration = 0;
	  List<Parameter> oParameters = findWhere(Parameter.class, "nombre", "employees_pswd_expiration");
	  if (oParameters.size() == 1) {
		  String sPasswordExpiration = oParameters.get(0).getValor();
		  try{
			  lPasswordExpiration = Long.parseLong(sPasswordExpiration);
		  } catch (NumberFormatException e){
			  lPasswordExpiration = -1;
		  }
	  }
	  if (lPasswordExpiration <= 0){
		  return null; 
	  }

	  //La valided aplica unicamente al rol Axente TIC (AEA_MANAGER) 
	  if(!personal.getPuesto().equals(Puesto.AREA_MANAGER)){
		  return null; 
	  }
	  
	  Long ldapShadowLastChange = getPasswordLastChange(personal);
	  long shadowLastChange = (ldapShadowLastChange!=null)?ldapShadowLastChange.longValue():-1;

	  if(shadowLastChange < 0){
		  return null;
	  }
	  
	  long hoyEpoch =  new Date().getTime() / 1000 / 60 / 60 / 24;
	  long limiteAntesCambio = hoyEpoch - lPasswordExpiration;
	  
	  return  shadowLastChange - limiteAntesCambio;
	  
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.EmpleadoService#isPasswordExpired(Empleado personal);
   */
  public boolean isPasswordExpired(Empleado personal){
	  
	  Long daysPasswordValid = getDaysPasswordValid(personal);
	  
	  return (daysPasswordValid!=null && daysPasswordValid.longValue() < 0);
	  
  }
  
}
