package es.csa.fdt.cyldigital.usuarios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.commons.persistence.browser.jpaql.FilterJPAQLByQuery;
import es.csa.fdt.cyldigital.core.service.BrowserFilteredByCenter;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;

/**
 * Servicio del Browser de Empleados.
 * 
 * @author csa
 */
@Service
@Repository
public class EmpleadosBrowserImpl extends BrowserFilteredByCenter<Empleado> implements
    EmpleadosBrowser<CriterionJPAQL, String> {

  private static final long serialVersionUID = -7227366052245717441L;

  private String customFilterProperty;

  /**
   * Construye un nuevo objeto de la clase
   */
  public EmpleadosBrowserImpl() {
    super();
    getSort().setSortAsc(false);
    getSort().setSortProperty("id");
  }

  /**
   * Apply custom filter.
   */
  public void applyCustomFilter() {
    String query = "nif LIKE ? OR concat(nombre,concat(' ',concat(apellido1,concat(' ',apellido2)))) LIKE ?";
    Long id = null;
    clearFilters();
    if (getCustomFilterProperty() != null && !getCustomFilterProperty().trim().isEmpty()) {
      try {
        id = Long.parseLong(customFilterProperty);
      } catch (Exception e) {}

      FilterJPAQLByQuery<Empleado> filter = new FilterJPAQLByQuery<Empleado>();
      if (id != null) {
        query += " OR id = ?";
        Object[] params = { "%" + customFilterProperty + "%", "%" + customFilterProperty + "%", id };
        filter.getCriterion().addParameters(params);
      } else {
        Object[] params = { "%" + customFilterProperty + "%", "%" + customFilterProperty + "%" };
        filter.getCriterion().addParameters(params);
      }
      filter.getCriterion().setQuery(query);
      filters.add(filter);
    }
    setCustomFilterProperty("");

  }

  /**
   * Asigna custom filter property.
   * 
   * @param customFilterProperty
   *          custom filter property
   */
  public void setCustomFilterProperty(String customFilterProperty) {
    this.customFilterProperty = customFilterProperty;
  }

  /**
   * Devuelve custom filter property.
   * 
   * @return custom filter property
   */
  public String getCustomFilterProperty() {
    return customFilterProperty;
  }

  /**
   * Filter by profesor online.
   */
  public void filterByProfesorOnline() {
    String query = "";
    FilterJPAQLByQuery<Empleado> filter = new FilterJPAQLByQuery<Empleado>();

    query += " puesto = 'ONLINE_TRAINER'";
    filter.getCriterion().setQuery(query);
    getFilters().add(filter);
  }
  
  /**
   * Set condition para colaboradores.
   */
  public void setConditionColaborador(UsuarioLdap usuario) {
	  if (usuario.getRolesList().contains(Usuario.Role.ROLE_EXTERN)) {
		  this.clearConditions();

		  this.getCondition().getCriterion().setQuery("e.id = ?");
		  this.getCondition().getCriterion().addParameter(usuario.getUser().getId());
		  
		  this.applyCondition();
	  }
  }
}
