package es.csa.fdt.cyldigital.usuarios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.File;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import org.hibernate.validator.InvalidStateException;
import org.hibernate.validator.InvalidValue;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.DefaultTransactionDefinition;

import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.usuarios.entity.Fichero;
import es.csa.fdt.cyldigital.usuarios.entity.Localidad;
import es.csa.fdt.cyldigital.usuarios.entity.Provincia;
import es.csa.fdt.cyldigital.usuarios.entity.RegistroLdap;
import es.csa.fdt.cyldigital.usuarios.entity.RegistroSmartCard;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;
import es.csa.fdt.cyldigital.usuarios.entity.RegistroSmartCard.TipoRegistro;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL.Sexo;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL.TipoIdentificacion;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL.TipoUsuario;
import es.csa.fdt.cyldigital.util.ImageUtils;

/**
 * Clase ImportacionServiceImpl.
 * 
 * @author csa
 */
public class ImportacionServiceImpl extends AbstractServiceImpl<UsuarioCyL, Long> implements ImportacionService {
  /**
   * Limpieza previa... Delete from digitalusers WHERE nid in(211,707,300,1258,458,1496,1335,783,536,512,286)
   */

  private static final long serialVersionUID = -1120064689884937913L;

  private String dbUrl = "jdbc:mysql://localhost:3306/cyldigital-centro?useUnicode=true&amp;characterEncoding=UTF-8";
  private String usuario = "cyldigitalweb";
  private String password = "cyldigitalclient";

  private StringBuilder errores = new StringBuilder();

  private List<UsuarioCyL> usuarioCyLs;

  @Autowired
  private transient HashMap<String, Object> generalProperties;

  @Autowired
  private transient org.springframework.orm.jpa.JpaTransactionManager txManager;

  /**
   * Devuelve usuarios.
   * 
   * @return usuarios
   */
  public List<UsuarioCyL> getUsuarios() {
    return usuarioCyLs;
  }

  /**
   * Asigna usuarios.
   * 
   * @param usuarioCyLs
   *          usuarios
   */
  public void setUsuarios(List<UsuarioCyL> usuarioCyLs) {
    this.usuarioCyLs = usuarioCyLs;
  }

  /**
   * Leer usuarios.
   */
  public void leerUsuarios() {
    try {
      usuarioCyLs = new ArrayList<UsuarioCyL>();
      Class.forName("com.mysql.jdbc.Driver");
      Connection con = DriverManager.getConnection(dbUrl, usuario, password);
      Statement sentencia = con.createStatement();
      ResultSet r = sentencia
          .executeQuery("SELECT a.*,b.*,f.filename, f.filesize, f.filemime,FROM_UNIXTIME(a.fecha_alta) as fAlta, if(fecha_nacimiento < 0,DATE_SUB(FROM_UNIXTIME(0),Interval (fecha_nacimiento * -1) Second),FROM_UNIXTIME(fecha_nacimiento)) as fNacimiento  FROM (digitalusers a inner join (SELECT nid,max(fid) as fid FROM digitalusers_foto group by nid) b on a.nid=b.nid) left join files f on b.fid=f.fid");
      while (r.next()) {
        UsuarioCyL u = new UsuarioCyL();
        u.setId(r.getLong("id"));
        u.setNombre(r.getString("nombre"));
        String apellidos = r.getString("apellidos");
        if (apellidos.trim().lastIndexOf(" ") >= 0) {
          u.setApellido1(apellidos.substring(0, apellidos.trim().lastIndexOf(" ")).trim());
          u.setApellido2(apellidos.substring(apellidos.trim().lastIndexOf(" "), apellidos.length()).trim());
        } else {
          u.setApellido1(apellidos);
          u.setApellido2("");
        }
        u.setDireccion(r.getString("direccion"));
        u.setCp(r.getString("codigo_postal"));

        String p = r.getString("provincia").trim();
        String loc = r.getString("localidad").trim();

        if (p.equals("Burgis") || p.equals("Burogs") || p.equals("Buergos") || p.equals("Burgoas")) p = "Burgos";
        if (p.equals("San Sebastian")) p = "Guipuzcoa";
        if (loc.equals("09003") || loc.equalsIgnoreCase("burgos") || loc.equals("Burgosq")) loc = "Burgos";
        if (loc.equals("Castalla")) p = "Alicante";
        if (loc.equalsIgnoreCase("Villanueva rio ubierna")) loc = "Villanueva de Rio Ubierna";
        if (loc.equals("Humienta revillarruz")) loc = "Humienta";
        if (loc.equals("Boadilla del monte")) loc = "Boadilla del Monte";
        if (loc.equals("L Nuez de Arriba Urbel del Castillo")) loc = "Nuez de Arriba, La";
        if (loc.equals("Cardenajimeno")) loc = "Cardeñajimeno";
        if (loc.equals("San Mamés") || loc.equals("San Mames")) loc = "San Mames de Burgos";
        if (loc.equals("Arcos")) loc = "Arcos de la Llana";
        if (loc.equals("Villafria")) loc = "Villafria de Burgos";
        if (loc.equals("Valle Valdelucio")) loc = "Quintanas de Valdelucio";
        if (loc.equals("Cuellar")) p = "Segovia";
        if (loc.equals("Boniel")) loc = "Buniel";
        if (loc.equals("Galapagar")) loc = "";

        List<Provincia> provincias = findWhere(Provincia.class, "nombre", p);

        if (provincias != null && provincias.size() == 1) {
          u.setProvincia(provincias.get(0));
          for (Localidad l : u.getProvincia().getLocalidades()) {
            if (!loc.isEmpty()) if (l.getNombre().equals(loc)) u.setLocalidad(l);
          }
          if (u.getLocalidad() == null && !loc.isEmpty())
            errores.append("Localidad no encontrada: " + loc + "-En-" + u.getProvincia().getNombre() + "-En-"
                + u.getCp() + "\n");
        } else {
          errores.append("Provincia no encontrada: " + p + "\n");
        }

        u.setTelefono(r.getString("telefono_fijo"));
        u.setMovil(r.getString("telefono_movil"));
        u.setEmail(r.getString("email").replaceAll(" ", "").replaceAll("@\\.", "@"));

        // OJO
        String dni = r.getString("dni").replaceAll(" ", "").replaceAll("-", "").replaceAll("\\.", "").toUpperCase();
        u.setNif(dni);

        /*
         * private String nifAutorizado; private boolean menor; private String nombreCompletoAutorizado;
         */
        // OJO
        u.setUsername(r.getString("dni"));
        u.setFechaCreacion(r.getDate("fAlta"));
        u.setFechaNacimiento(r.getDate("fNacimiento"));

        int tipo_usuario = r.getInt("tipo_usuario");
        if (tipo_usuario == 0) u.setTipoUsuario(TipoUsuario.SC);
        else if (tipo_usuario == 1) u.setTipoUsuario(TipoUsuario.B);
        else if (tipo_usuario == 2) u.setTipoUsuario(TipoUsuario.NU);
        else u.setTipoUsuario(TipoUsuario.TA);

        if (r.getBoolean("es_hombre")) u.setSexo(Sexo.H);
        else u.setSexo(Sexo.M);

        // Quedará a Nulo
        // private EstadoCivil estadoCivil;
        u.setAceptaPolitica(r.getBoolean("acepta_politica"));

        // Quedara a vacio
        // private String password;
        if (r.getBoolean("con_smartcard")) {
          /*u.setTipoIdentificacion(TipoIdentificacion.C);*/
          RegistroSmartCard reg = new RegistroSmartCard();
          reg.setTipoRegistro(TipoRegistro.G);
          reg.setFecha(new Date());
          reg.setCorrecto(true);
          reg.setUsuario(u);
          u.getRegistrosSmartCard().add(reg);

          RegistroSmartCard rei = new RegistroSmartCard();
          rei.setTipoRegistro(TipoRegistro.I);
          rei.setFecha(new Date());
          rei.setCorrecto(true);
          rei.setUsuario(u);
          u.getRegistrosSmartCard().add(rei);
        } else {
          u.setTipoIdentificacion(TipoIdentificacion.N);
        }

        u.getRegistrosLdap().add(new RegistroLdap(u, "UsuarioCyL registrado (importación)", true));

        if (r.getString("filename") != null && !r.getString("filename").isEmpty()) {
          Fichero f = new Fichero();
          f.setNombre(r.getString("filename"));
          f.setTamanio(r.getInt("filesize"));
          f.setTipoMime(r.getString("filemime"));

          File file = new File(generalProperties.get("FILE_PATH") + "" + generalProperties.get("USER_PHOTO_PATH")
              + u.getId() + ".jpg");
          File dest = new File(generalProperties.get("FILE_PATH") + "" + generalProperties.get("USER_PHOTO_PATH")
              + f.getNombre());
          File destThumb = new File(generalProperties.get("FILE_PATH") + ""
              + generalProperties.get("USER_PHOTO_PATH_THUMB") + u.getId() + ".jpg");

          if (dest.exists()) {
            // if (destThumb.exists()) destThumb.delete();
            try {
              ImageUtils.resize(dest, destThumb, 72, 1f);
            } catch (IOException e) {
              e.printStackTrace();
            }
            if (!(u.getId() + ".jpg").equals(f.getNombre())) dest.renameTo(file);
          }
          f.setNombre(u.getId() + ".jpg");
          u.setFoto(f);
        }

        // System.out.println("UsuarioCyL: "+u.getNombreCompleto());
        usuarioCyLs.add(u);

      }

    } catch (ClassNotFoundException e) {
      e.printStackTrace();
    } catch (SQLException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    System.out.println(errores);
    System.out.println("Numero de usuarioCyLs a importar: " + usuarioCyLs.size());
  }

  /**
   * Guardar usuarios.
   */
  public void guardarUsuarios() {
    // persistAll(usuarioCyLs);
    // List<UsuarioCyL> guardados = new ArrayList<UsuarioCyL>();
    errores = new StringBuilder();
    // int i = 0;
    DefaultTransactionDefinition def;
    TransactionStatus status;
    def = new DefaultTransactionDefinition();;
    def.setPropagationBehavior(TransactionDefinition.PROPAGATION_REQUIRED);
    status = txManager.getTransaction(def);

    for (UsuarioCyL u : usuarioCyLs) {
      try {
        getEntityManager().persist(u);
      } catch (Exception e) {
        System.out.println("Error en usuario" + u.getNombreCompleto() + " " + u.getNif() + "\n");
      }
    }

    try {
      txManager.commit(status);
      def.setPropagationBehavior(TransactionDefinition.PROPAGATION_REQUIRED);
      status = txManager.getTransaction(def);
    } catch (Exception e) {
      Throwable ex = e;
      while (ex != null) {
        if (ex instanceof InvalidStateException) {
          InvalidStateException ise = (InvalidStateException) ex;
          for (InvalidValue iv : ise.getInvalidValues()) {
            System.out.println("ERROR DE VALIDACION" + iv.getMessage());
          }
        }
        ex = ex.getCause();
      }
      e.printStackTrace();
      System.out.println("error");
    }

    System.out.println(errores);
  }
}
