package es.csa.fdt.cyldigital.usuarios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2017  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */

import java.io.InputStream;
import java.io.Serializable;
import java.io.UnsupportedEncodingException;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Random;

import javax.annotation.Resource;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.BasicAttributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.ModificationItem;

import org.jasypt.util.password.ConfigurablePasswordEncryptor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.ldap.AuthenticationException;
import org.springframework.ldap.InvalidAttributeValueException;
import org.springframework.ldap.NameAlreadyBoundException;
import org.springframework.ldap.NameNotFoundException;
import org.springframework.ldap.OperationNotSupportedException;
import org.springframework.ldap.core.AttributesMapper;
import org.springframework.ldap.core.DirContextOperations;
import org.springframework.ldap.core.DistinguishedName;
import org.springframework.ldap.core.LdapTemplate;
import org.springframework.stereotype.Component;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.RegistroLdap;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario.Role;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;

/**
 * Clase LdapComponent.
 * 
 * @author csa
 */
@Component
public class LdapComponent implements Serializable {

  private static final long serialVersionUID = 5444757842578031398L;

  private static final Logger logger = LoggerFactory.getLogger(LdapComponent.class);

  @Autowired
  private transient LdapTemplate ldapTemplate;
  
  @Resource
  private transient HashMap<String, Object> generalProperties;

  /**
   * Construye un nuevo objeto de la clase
   */
  public LdapComponent() {}

  /**
   * Crea user.
   * 
   * @param usuarioCyL
   *          usuario cy l
   */
  public void createUser(UsuarioCyL usuarioCyL, UsuarioServiceImpl service) {
	  if (generalProperties.containsKey("MAKE_LDAP_CHANGES")
	      && ((String) generalProperties.get("MAKE_LDAP_CHANGES")).equalsIgnoreCase("true")) {
	    RegistroLdap reg = new RegistroLdap(usuarioCyL);
	    try {
	      //Guardamos la password que se le generó al usuario.
	      String password = usuarioCyL.getPassword();
	      //Eliminamos la password del usuario para que se genere con la password por defecto.
	      usuarioCyL.setPassword("");
	      
	      ldapTemplate.bind(userToDistinguishedName(usuarioCyL, service), null, buildAttributes(usuarioCyL, service));
	      
	      //Si el usuario es nuevo y tiene email
	      if (usuarioCyL.getId() == 0) {
		      //Como el usuario se ha registrado bien en LDAP intentamos ponerle la password que se le generó.
		      //Si la password no se establece correctamente...
		      if (setPassword(usuarioCyL, password, service) == 0) {
		    	  reg.setMensaje("UsuarioCyL registrado");
		    	  logger.info("ldapRegister() - UsuarioCyL con contraseña establecida - ", usuarioCyL.getNombreCompleto());
		      } else {
		    	  //Damos al usuario por registrado pero añadimos la coletilla que informa de que la contraseña no se ha podido establecer.
		    	  reg.setMensaje("UsuarioCyL registrado (Se produjo un error al establecer la contraseña, el usuario tiene la contraseña por defecto)");
		    	  logger.info("ldapRegister() - UsuarioCyL con contraseña establecida - ", usuarioCyL.getNombreCompleto());
		      }
	      }
	      
	      reg.setCorrecto(true);
	      
	    } catch (InvalidAttributeValueException exc) {
	      reg.setMensaje("Error en registro de usuario - Atributo no valido");
	      reg.setCorrecto(false);
	      logger.error("ldapRegister() - Error al registrar el usuario (atributo no valido)", exc);
	    } catch (NameAlreadyBoundException exc) {
	      reg.setMensaje("Error en registro de usuario - El usuario ya existe");
	      reg.setCorrecto(false);
	      logger.error("ldapRegister() - Error al registrar el usuario (el usuario existe)", exc);
	    } catch (NameNotFoundException exc) {
	      reg.setMensaje("Error en registro de usuario - UsuarioCyL no encontrado");
	      reg.setCorrecto(false);
	      logger.error("ldapRegister() - Error al registrar el usuario (usuario no encontrado)", exc);
	    } catch (OperationNotSupportedException exc) {
	      reg.setMensaje("Error en registro de usuario - Operacion no soportada");
	      reg.setCorrecto(false);
	      logger
	          .error("ldapRegister() - Error al registrar el usuario (no se puede añadir)(maybe PasswordStreng)", exc);
	    } catch (Exception exc) {
	      reg.setMensaje("Error en registro de usuario");
	      reg.setCorrecto(false);
	      logger.error("ldapRegister() - Error al registrar el usuario", exc);
	    }
	    usuarioCyL.getRegistrosLdap().add(reg);
	  }
  }

  public void createUser(Empleado empleado, EmpleadoServiceImpl service) {
	  if (generalProperties.containsKey("MAKE_LDAP_CHANGES")
	      && ((String) generalProperties.get("MAKE_LDAP_CHANGES")).equalsIgnoreCase("true")) {
	    try {
	      ldapTemplate.bind(userToDistinguishedName(empleado, service), null, buildAttributes(empleado, service));
	      logger.info("ldapRegister() - Personal creado - ", empleado.getNombreCompleto());
	    } catch (InvalidAttributeValueException exc) {
	      logger.error("ldapRegister() - Error al registrar el empleado (atributo no valido)", exc);
	    } catch (NameAlreadyBoundException exc) {
	      logger.error("ldapRegister() - Error al registrar el empleado (el usuario existe)", exc);
	    } catch (NameNotFoundException exc) {
	      logger.error("ldapRegister() - Error al registrar el empleado (usuario no encontrado)", exc);
	    } catch (OperationNotSupportedException exc) {
	      logger
	          .error("ldapRegister() - Error al registrar el empleado (no se puede añadir)(maybe PasswordStreng)", exc);
	    } catch (Exception exc) {
	      logger.error("ldapRegister() - Error al registrar el empleado", exc);
	    }
	  }
  }

  
  /**
   * Update group.
   * 
   * @param group
   *          Grupo
   * @param usuarioCyL
   *          usuario CyL
   */
  public void updateGroup(String group, Empleado empleado, EmpleadoServiceImpl servicio) {
	  
	  if (generalProperties.containsKey("MAKE_LDAP_CHANGES") 
			  && ((String) generalProperties.get("MAKE_LDAP_CHANGES")).equalsIgnoreCase("true")) {
		  
		  try {
			  DirContextOperations ctx = ldapTemplate.lookupContext(groupToDistinguishedName(group, servicio));
			  DirContextOperations user = ldapTemplate.lookupContext(userToDistinguishedName(empleado, servicio));
			  
			  String userDN = user.getStringAttribute("uid");
			  
			  ctx.addAttributeValue("memberUid", userDN);
			  
			  ldapTemplate.modifyAttributes(ctx);
			  
			  logger.info("updateGroup() - empleado aÃ±adido al grupo del centro ", empleado.getNombreCompleto() + " " + group);
		  } catch (Exception exc) {
			  logger.error("updateGroup() - Error al añadir el empleado al grupo del centro", exc);
		  }
	  }
  }
  
  
  /**
   * Update user.
   * 
   * @param usuarioCyL
   *          usuario cy l
   * @param oldUserName
   *          old user name
   */
  public void updateUser(UsuarioCyL usuarioCyL, String oldUserName, UsuarioServiceImpl service) {
	  
	  if (generalProperties.containsKey("MAKE_LDAP_CHANGES")
			  && ((String) generalProperties.get("MAKE_LDAP_CHANGES")).equalsIgnoreCase("true")) {
		  
		  try {
			  if (!usuarioCyL.getUsername().equals(oldUserName)) {
				  ldapTemplate.rename(userToDistinguishedName(oldUserName, usuarioCyL.getCentro(), service),
						  			  userToDistinguishedName(usuarioCyL, service));
				  
				  usuarioCyL.getRegistrosLdap().add(new RegistroLdap(usuarioCyL, "UsuarioCyL renombrado", true));
			  }
			  
			  DirContextOperations ctx = ldapTemplate.lookupContext(userToDistinguishedName(usuarioCyL, service));
			  ctx.setAttributeValue("homeDirectory", "/home/" + getCname(usuarioCyL));
			  ctx.setAttributeValue("displayName", usuarioCyL.getNombre()+ " " + usuarioCyL.getApellido1());
			  
			  if (!usuarioCyL.getNombre().isEmpty())
				  ctx.setAttributeValue("cn", usuarioCyL.getNombre());
			  else
				  ctx.setAttributeValue("cn", "n/a");
			  
			  if (!((usuarioCyL.getApellido1() + usuarioCyL.getApellido2()).isEmpty()))
				  ctx.setAttributeValue("sn", usuarioCyL.getApellido1() + " " + usuarioCyL.getApellido2());
			  else
				  ctx.setAttributeValue("sn", "n/a");
			  
			  if (!getCname(usuarioCyL).isEmpty())
				  ctx.setAttributeValue("description", getCname(usuarioCyL));
			  else
				  ctx.setAttributeValue("description", "n/a");
			  
			  if (!usuarioCyL.getNombre().isEmpty())
				  ctx.setAttributeValue("givenName", usuarioCyL.getNombre());
			  else
				  ctx.setAttributeValue("givenName", "n/a");
			  
			  if (!usuarioCyL.getEmail().isEmpty())
				  ctx.setAttributeValue("mail", usuarioCyL.getEmail());
			  else
				  ctx.setAttributeValue("mail", "n/a");
			  
			  if (!usuarioCyL.getDireccion().isEmpty())
				  ctx.setAttributeValue("postalAddress", usuarioCyL.getDireccion());
			  else
				  ctx.setAttributeValue("postalAddress", "n/a");
			  
			  if (!usuarioCyL.getCp().isEmpty())
				  ctx.setAttributeValue("postalCode", usuarioCyL.getCp());
			  else
				  ctx.setAttributeValue("postalCode", "n/a");
			  
			  if (!usuarioCyL.getLocalidad().getNombre().isEmpty())
				  ctx.setAttributeValue("postOfficeBox", usuarioCyL.getLocalidad().getNombre());
			  else
				  ctx.setAttributeValue("postOfficeBox", "n/a");
			  
			  if (!usuarioCyL.getProvincia().getNombre().isEmpty())
				  ctx.setAttributeValue("st", usuarioCyL.getProvincia().getNombre());
			  else
				  ctx.setAttributeValue("st", "n/a");
			  
			  if (!usuarioCyL.getTelefono().isEmpty())
				  ctx.setAttributeValue("telephoneNumber", usuarioCyL.getTelefono());
			  else
				  ctx.setAttributeValue("telephoneNumber", "n/a");
			  
			  ldapTemplate.modifyAttributes(ctx);
			  
			  usuarioCyL.getRegistrosLdap().add(new RegistroLdap(usuarioCyL, "UsuarioCyL modificado", true));
			  logger.info("ldapRegister() - UsuarioCyL modificado - ", usuarioCyL.getNombreCompleto());
		  } catch (Exception exc) {
			  logger.error("updateUser() - Error al modificar el usuario", exc);
			  usuarioCyL.getRegistrosLdap().add(new RegistroLdap(usuarioCyL, "Error al actualizar el usuario", false));
		  }
	  }
  }

  public void updateUser(Empleado empleado, String oldUserName, Centro oldCentro, EmpleadoServiceImpl service) {
	  
	  if (generalProperties.containsKey("MAKE_LDAP_CHANGES")
			  && ((String) generalProperties.get("MAKE_LDAP_CHANGES")).equalsIgnoreCase("true")) {
		  
		  try {  		
			  if (!empleado.getUsername().equals(oldUserName) || (empleado.getCentro() != oldCentro)) {
				  ldapTemplate.rename(empleadoToDistinguishedName(oldUserName, oldCentro, service),
						  		  	userToDistinguishedName(empleado, service));
			  }
        
			  DirContextOperations ctx = ldapTemplate.lookupContext(userToDistinguishedName(empleado, service));
			  
			  if (!empleado.getCentro().getLocalidad().getNombre().isEmpty()) 
				  ctx.setAttributeValue("gecos", empleado.getCentro().getLocalidad().getNombre()
        			.replace('ñ', 'n').replace('á', 'a').replace('é', 'e').replace('í', 'i').replace('ó', 'o').replace('ú', 'u')
        			.replace('Ñ', 'N').replace('Á', 'A').replace('É', 'E').replace('Í', 'I').replace('Ó', 'O').replace('Ú', 'U'));
        
			  ctx.setAttributeValue("homeDirectory", "/home/" + getCname(empleado));
			  ctx.setAttributeValue("displayName", empleado.getNombre()+ " " + empleado.getApellido1());
        
			  if (!((empleado.getApellido1() + empleado.getApellido2()).isEmpty()))
				  ctx.setAttributeValue("sn", empleado.getApellido1() + " " + empleado.getApellido2());
			  else
				  ctx.setAttributeValue("sn", "n/a");
			  
			  if (!empleado.getNombre().isEmpty())
				  ctx.setAttributeValue("givenName", empleado.getNombre());
			  else
				  ctx.setAttributeValue("givenName", "n/a");
			  
			  if (!empleado.getEmail().isEmpty())
				  ctx.setAttributeValue("mail", empleado.getEmail());
			  else
				  ctx.setAttributeValue("mail", "n/a");
        
			  if (!empleado.getCentro().getDireccion().isEmpty())
				  ctx.setAttributeValue("postalAddress", empleado.getCentro().getDireccion());
			  else
				  ctx.setAttributeValue("postalAddress", "n/a");
			  
			  if (!empleado.getCentro().getLocalidad().getNombre().isEmpty())
				  ctx.setAttributeValue("postOfficeBox", empleado.getCentro().getLocalidad().getNombre());
			  else
				  ctx.setAttributeValue("postOfficeBox", "n/a");
			  
			  if (!empleado.getCentro().getProvincia().getNombre().isEmpty())
				  ctx.setAttributeValue("st", empleado.getCentro().getProvincia().getNombre());
			  else
				  ctx.setAttributeValue("st", "n/a");
        
			  if (!empleado.getTelefono().isEmpty())
				  ctx.setAttributeValue("telephoneNumber", empleado.getTelefono());
			  else
				  ctx.setAttributeValue("telephoneNumber", "n/a");
        
			  ldapTemplate.modifyAttributes(ctx);

			  logger.info("ldapRegister() - UsuarioCyL modificado - ", empleado.getNombreCompleto());
		  } catch (Exception exc) {
			  logger.error("updateUser() - Error al modificar el usuario", exc);
		  }
	  }
  }

  @SuppressWarnings("unused")
  private String getEmailSuffix(Centro centro) {
    return "@" + centro.getDominio().replaceAll(" ", "").toLowerCase() + "."
        + (String) generalProperties.get("LDAP_EMAIL_SUFFIX");
  }

  /**
   * Delete user.
   * 
   * @param usuarioCyL
   *          usuario cy l
   */
  public void deleteUser(UsuarioCyL usuarioCyL, UsuarioServiceImpl service) {
	  
	  if (generalProperties.containsKey("MAKE_LDAP_CHANGES")
			  && ((String) generalProperties.get("MAKE_LDAP_CHANGES")).equalsIgnoreCase("true")) {
      
		  try {
			  DistinguishedName userDN = userToDistinguishedName(usuarioCyL, service);
			  ldapTemplate.unbind(userDN);
			  logger.info("ldapRegister() - UsuarioCyL borrado - ", usuarioCyL.getNombreCompleto());
		  } catch (Exception exc) {
			  logger.error("deleteUser() - Error al borrar el usuario " + usuarioCyL.getNombreCompleto(), exc);
		  }
	  }
  }
  public void deactivateUser(UsuarioCyL usuarioCyL, UsuarioServiceImpl service){
	  
	  if (generalProperties.containsKey("MAKE_LDAP_CHANGES")
			  && ((String) generalProperties.get("MAKE_LDAP_CHANGES")).equalsIgnoreCase("true")) {

		  try {
			  DistinguishedName userDN = userToDistinguishedName(usuarioCyL, service);
			  ldapTemplate.unbind(userDN);
			  usuarioCyL.setDesactivadoLDAP(true);
			  logger.info("ldapRegister() - UsuarioCyL desactivado - ", usuarioCyL.getNombreCompleto());
		  } catch (Exception exc) {
			  logger.error("deactivateUser() - Error al desactivar el usuario " + usuarioCyL.getNombreCompleto(), exc);
		  }
	  }
  }
  
  public void reactivateUser(UsuarioCyL usuarioCyL, UsuarioServiceImpl service){
	  
	  if (generalProperties.containsKey("MAKE_LDAP_CHANGES")
			  && ((String) generalProperties.get("MAKE_LDAP_CHANGES")).equalsIgnoreCase("true")) {

		  try {
			  DistinguishedName userDN = userToDistinguishedName(usuarioCyL, service);
			  ldapTemplate.bind(userDN, null, buildAttributes(usuarioCyL, service));
			  usuarioCyL.setDesactivadoLDAP(false);
			  logger.info("ldapRegister() - UsuarioCyL reactivado - ", usuarioCyL.getNombreCompleto());
		  } catch (Exception exc) {
			  logger.error("reactivateUser() - Error al reactivar el usuario " + usuarioCyL.getNombreCompleto(), exc);
		  }
		  	
	  }
  }
  /**
   * Comprueba si el usuario está en el LDAP
   * 
   * @param ucyl
   * @param service
   * 
   * @return true si está en el LDAP, false si no lo está.
   * 
   * */
  public boolean checkUserActive(UsuarioCyL ucyl, UsuarioServiceImpl service){
	  boolean value = true;
	  try{
		  ldapTemplate.lookup(userToDistinguishedName(ucyl, service));
	  }catch(NameNotFoundException ex){
		  value = false;
	  }
	  return value;
  }
  
  
  // ----------------------------------------------------------------------------------------------Funciones de Utilidad
  
  /**
   * Modifica la contraseña del usuario
   * 
   * @param ucyl
   * @param password
   * @param service
   * @return true si no hubo ningún problema al actualizar la contraseña
   */
  public int setPassword(UsuarioCyL ucyl, String password, UsuarioServiceImpl service) {
	  //SAMBA
	  
	  String shell = service.findOneWhere(Parameter.class, "nombre", "change_password_shell").getValor();
	  String shellParams = service.findOneWhere(Parameter.class, "nombre", "change_password_shellParams").getValor();
	  String withQuotes = service.findOneWhere(Parameter.class, "nombre", "change_password_withQuotes").getValor().toLowerCase().trim();
	  String echo = service.findOneWhere(Parameter.class, "nombre", "change_password_echo").getValor();
	  String sudo = service.findOneWhere(Parameter.class, "nombre", "change_password_sudo").getValor();
	  String changePassCmd = service.findOneWhere(Parameter.class, "nombre", "change_password_command").getValor();
	  
      try {
    	  //Lanza el comando de cambio de la contraseña en LDAP

    	  String finalCommand;
    	  if (withQuotes.compareTo("true") == 0)
    		  finalCommand = "'(" + echo + " " + password + "; " + echo + " " + password + ";) | " + sudo + " " + changePassCmd + " " + ucyl.getUsername() + "'";
    	  else
    		  finalCommand = "(" + echo + " " + password + "; " + echo + " " + password + ";) | " + sudo + " " + changePassCmd + " " + ucyl.getUsername();
    	  
    	  String[] cmd = {shell, shellParams, finalCommand};
    	  
    	  logger.info("INFO: setPassword() - Se va a ejecutará el siguiente comando construido: " + cmd[0] + " " + cmd[1] + " " + cmd[2]);
    	  System.out.println("INFO: setPassword() UsuarioCyL - Se va a ejecutará el siguiente comando construido: " + cmd[0] + " " + cmd[1] + " " + cmd[2]);
    	  
    	  Process process = Runtime.getRuntime().exec(cmd);
    	  
    	  //Recogemos el valor del stado de salida del comando
    	  int exitValue = process.waitFor();
    	  
    	  //Si no terminó correctamente
    	  if (exitValue != 0) {
    		  InputStream es = process.getErrorStream();
        	  String errorMessage = "";
        	  for (int i = 0; i < es.available(); i++)
        		  errorMessage += (char)(es.read());
        	  
	          logger.error("ERROR: setPassword() - Se produjo un error al ejecutar el comando. Exit status: [" + exitValue + "]. Mensaje de error: [" + errorMessage + "]");
    	  }
    	  else {
	    	  InputStream os = process.getInputStream();
	    	  String outputMessage = "";
	    	  for (int i = 0; i < os.available(); i++)
	    		  outputMessage += (char)(os.read());
	    	  
	    	  logger.info("INFO: setPassword() - Se ha establecido la contraseña con éxito. Mensaje de salida: [" + outputMessage + "]");
    	  }

    	  //Devolvemos el estado de salida del comando
		  return exitValue;
      }
      catch (Exception exc) {
    	  logger.error("ERROR: setPassword() - Ocurrió una excepción al intentar establecer la password de usuario.", exc);
      }
	  
	  //LDAP
      /*
	  try {
		Attribute passwordAttribute = new BasicAttribute("userPassword", password);
		ModificationItem[] modificationItems = new ModificationItem[1];
		modificationItems[0] = new ModificationItem(DirContext.REPLACE_ATTRIBUTE, passwordAttribute);
		ldapTemplate.modifyAttributes(userToDistinguishedName(ucyl, service), modificationItems);
		
		logger.info("ldapRegister() - UsuarioCyL modificado - ", ucyl.getNombreCompleto());
		 
		return 0;
	  } catch (AuthenticationException ae) {
			  logger.error("setPassword - Error al modificar la contraseña del usuario de tipo Usuario. " 
					  + userToDistinguishedName(ucyl, service));
			  ae.printStackTrace();
	  } catch (Exception e) {
			  logger.error("setPassword - Error al modificar la contraseña del usuario de tipo Usuario. " 
					  + userToDistinguishedName(ucyl, service));
			  e.printStackTrace();
	  }
	  */
      
	  return -1;
  }

  /**
   * Modifica la contraseña del empleado (Axente TIC)
   * @param personal
   * @param password
   * @param service
   * @return true si no hubo ningún problema al actualizar la contraseña
   */  
  public int setPassword(Empleado personal, String password, EmpleadoServiceImpl service) {
	  //SAMBA
	  
	  String shell = service.findOneWhere(Parameter.class, "nombre", "change_password_shell").getValor();
	  String shellParams = service.findOneWhere(Parameter.class, "nombre", "change_password_shellParams").getValor();
	  String withQuotes = service.findOneWhere(Parameter.class, "nombre", "change_password_withQuotes").getValor().toLowerCase().trim();
	  String echo = service.findOneWhere(Parameter.class, "nombre", "change_password_echo").getValor();
	  String sudo = service.findOneWhere(Parameter.class, "nombre", "change_password_sudo").getValor();
	  String changePassCmd = service.findOneWhere(Parameter.class, "nombre", "change_password_command").getValor();
	  
      try {
    	  //Lanza el comando de cambio de la contraseña en LDAP

    	  String finalCommand;
    	  if (withQuotes.compareTo("true") == 0)
    		  finalCommand = "'(" + echo + " " + password + "; " + echo + " " + password + ";) | " + sudo + " " + changePassCmd + " " + personal.getUsername() + "'";
    	  else
    		  finalCommand = "(" + echo + " " + password + "; " + echo + " " + password + ";) | " + sudo + " " + changePassCmd + " " + personal.getUsername();
    	  
    	  String[] cmd = {shell, shellParams, finalCommand};
    	  
    	  logger.info("INFO: setPassword() - Se va a ejecutará el siguiente comando construido: " + cmd[0] + " " + cmd[1] + " " + cmd[2]);
    	  System.out.println("INFO: setPassword() Empleado - Se va a ejecutará el siguiente comando construido: " + cmd[0] + " " + cmd[1] + " " + cmd[2]);
    	  
    	  Process process = Runtime.getRuntime().exec(cmd);
    	  
    	  //Recogemos el valor del stado de salida del comando
    	  int exitValue = process.waitFor();
    	  
    	  //Si no terminó correctamente
    	  if (exitValue != 0) {
    		  InputStream es = process.getErrorStream();
        	  String errorMessage = "";
        	  for (int i = 0; i < es.available(); i++)
        		  errorMessage += (char)(es.read());
        	  
	          logger.error("ERROR: setPassword() - Se produjo un error al ejecutar el comando. Exit status: [" + exitValue + "]. Mensaje de error: [" + errorMessage + "]");
    	  }
    	  else {
	    	  InputStream os = process.getInputStream();
	    	  String outputMessage = "";
	    	  for (int i = 0; i < os.available(); i++)
	    		  outputMessage += (char)(os.read());
	    	  
	    	  logger.info("INFO: setPassword() - Se ha establecido la contraseña con éxito. Mensaje de salida: [" + outputMessage + "]");
    	  }

    	  //Devolvemos el estado de salida del comando
		  return exitValue;
      }
      catch (Exception exc) {
    	  logger.error("ERROR: setPassword() - Ocurrió una excepción al intentar establecer la password de usuario personal.", exc);
      }
	  
	  //LDAP
      /*
	  try {
		Attribute passwordAttribute = new BasicAttribute("userPassword", password);
		ModificationItem[] modificationItems = new ModificationItem[1];
		modificationItems[0] = new ModificationItem(DirContext.REPLACE_ATTRIBUTE, passwordAttribute);
		ldapTemplate.modifyAttributes(empleadoToDistinguishedName(personal.getUsername(), personal.getCentro(), service), modificationItems);

		logger.info("ldapRegister() - Empleado modificado - ", personal.getNombreCompleto());
		return 0;
	  } catch (AuthenticationException ae) {
			  logger.error("setPassword - Error al modificar la contraseña del usuario de tipo Empleado. " 
					  + empleadoToDistinguishedName(personal.getUsername(), personal.getCentro(), service));
			  ae.printStackTrace();
	  } catch (Exception e) {
			  logger.error("setPassword - Error al modificar la contraseña del usuario de tipo Empleado. " 
					  + empleadoToDistinguishedName(personal.getUsername(), personal.getCentro(), service));
			  e.printStackTrace();
	  }
      */
	  return -1;
	  
  }

  /**
   *Obtiene la fecha del ultimo acceso registrado en el LDAP de un usuario
   * @param personal
   * @param service
   * @return
   */
  public Date getUltimoAccesoLDAP(Empleado usuario, EmpleadoServiceImpl service){
	  Date ultimoAcceso = null; 
	  try{
		  Attribute oAttribute = getAtributoLDAP(usuario, service, "sambaLogonTime");
		  Long lUltimoAcceso = Long.parseLong((String) oAttribute.get()) * 1000;
		  if(lUltimoAcceso > 0)
			  ultimoAcceso = new Date(lUltimoAcceso);
	  } catch (Exception exc) {
		  logger.error("getPasswordLastChange() - Error obtendo o ultimo acceso", exc.getLocalizedMessage());
	  }

	  return ultimoAcceso;
  }
  
  /**
   * Obtiene la fecha del ultimo acceso registrado en el LDAP de un usuario
   * @param personal
   * @param service
   * @return
   */
  public Date getUltimoAccesoLDAP(UsuarioCyL usuario, UsuarioServiceImpl service){
	  Date ultimoAcceso = null; 
	  try{
		  Attribute oAttribute = getAtributoLDAP(usuario, service, "sambaLogonTime");
		  Long lUltimoAcceso = Long.parseLong((String) oAttribute.get()) * 1000;
		  if(lUltimoAcceso > 0)
			  ultimoAcceso = new Date(lUltimoAcceso);
	  } catch (Exception exc) {
		  logger.error("getPasswordLastChange() - Error obtendo o ultimo acceso", exc.getLocalizedMessage());
	  }

	  return ultimoAcceso;
  }
  
  
  public void setUltimoAccesoLDAP(UsuarioCyL usuario, UsuarioServiceImpl service, Date fecha){
	   
		  DistinguishedName dn = userToDistinguishedName(usuario, service);
		  DirContextOperations ctx = ldapTemplate.lookupContext(dn);
		  
		  Long epoch = fecha.getTime()/1000;
		  
		  ctx.setAttributeValue("sambaLogonTime", epoch.toString());
		  ldapTemplate.modifyAttributes(ctx);
	  
  }
  
  /**
   * Obtiene la fecha del ultimo cambio de contraseña en formato dias epoch de un empleado
   * @param personal
   * @param service
   * @return
   */
  public Long getPasswordLastChange(Empleado usuario, EmpleadoServiceImpl service){
	  Long lastChange = null; 
	  try{
		  Attribute oAttribute = getAtributoLDAP(usuario, service, "shadowLastChange");
		  lastChange = Long.parseLong((String) oAttribute.get());
	  } catch (Exception exc) {
		  logger.error("getPasswordLastChange() - Error obtendo o ultimo cambio de contrasinal", exc);
	  }

	  return lastChange;
  }
  
  /**
   * Obtiene la fecha del ultimo cambio de contraseña en formato dias epoch de un usuario
   * @param personal
   * @param service
   * @return
   */
  public Long getPasswordLastChange(UsuarioCyL usuario, UsuarioServiceImpl service){
	  Long lastChange = null; 
	  try{
		  Attribute oAttribute = getAtributoLDAP(usuario, service, "shadowLastChange");
		  lastChange = Long.parseLong((String) oAttribute.get());
	  } catch (Exception exc) {
		  logger.error("getPasswordLastChange() - Error obtendo o ultimo cambio de contrasinal", exc);
	  }

	  return lastChange;
  }
  
  /**
   * Obtiene el hash del hash de la contrasinal del dempleado
   * @param personal
   * @param service
   * @return
   */
  public String getObtenerHashContrasinal(Empleado usuario, EmpleadoServiceImpl service){
	  String hashContrasinal = null; 
	  try{
		  Attribute oAttribute = getAtributoLDAP(usuario, service, "userPassword");
		  String contrasinalLDAP = new String ((byte[]) oAttribute.get()); 
		  
		  ConfigurablePasswordEncryptor passwordEncryptor = new ConfigurablePasswordEncryptor();
		  passwordEncryptor.setAlgorithm("SHA-1");
		  passwordEncryptor.setPlainDigest(true);
		  passwordEncryptor.setStringOutputType("hexadecimal");		  
		  hashContrasinal = passwordEncryptor.encryptPassword(contrasinalLDAP);
		  
	  } catch (Exception exc) {
		  logger.error("getObtenerHashContrasinal() - Error obtendo o hash da contrasinal", exc);
	  }

	  return hashContrasinal;
  }
  
  /**
   * Obtiene el hash del hash de la contrasinal del usuario
   * @param personal
   * @param service
   * @return
   */
  public String getObtenerHashContrasinal(UsuarioCyL usuario, UsuarioServiceImpl service){
	  String hashContrasinal = null; 
	  try{
		  Attribute oAttribute = getAtributoLDAP(usuario, service, "userPassword");
		  String contrasinalLDAP = new String ((byte[]) oAttribute.get()); 
		  
		  ConfigurablePasswordEncryptor passwordEncryptor = new ConfigurablePasswordEncryptor();
		  passwordEncryptor.setAlgorithm("SHA-1");
		  passwordEncryptor.setPlainDigest(true);
		  passwordEncryptor.setStringOutputType("hexadecimal");		  
		  hashContrasinal = passwordEncryptor.encryptPassword(contrasinalLDAP);
		  
	  } catch (Exception exc) {
		  logger.error("getObtenerHashContrasinal() - Error obtendo o hash da contrasinal", exc);
	  }

	  return hashContrasinal;
  }
  
  /**
   * Obtiene un atributo del ldap
   * @param personal
   * @param service
   * @param nombreAtributo
   * @return Attribute
   */
	private Attribute getAtributoLDAP(Empleado usuario, EmpleadoServiceImpl service, String nombreAtributo) {
		Attribute result = null;

		try {
			DistinguishedName dn = userToDistinguishedName(usuario, service);
			DirContextOperations context = ldapTemplate.lookupContext(dn);
			Attributes attr = context.getAttributes();
			result = attr.get(nombreAtributo);
		} catch (Exception exc) {
			logger.error("getAtributoLDAP() - Error obtendo o atributo " + nombreAtributo, exc.getLocalizedMessage());
		}

		return result;
	}
  
	/**
	 * Obtiene un atributo del ldap
	 * @param personal
	 * @param service
	 * @param nombreAtributo
	 * @return Attribute
	 */
	private Attribute getAtributoLDAP(UsuarioCyL usuario, UsuarioServiceImpl service, String nombreAtributo) {
		Attribute result = null;

		try {
			DistinguishedName dn = userToDistinguishedName(usuario, service);
			DirContextOperations context = ldapTemplate.lookupContext(dn);
			Attributes attr = context.getAttributes();
			result = attr.get(nombreAtributo);
		} catch (Exception exc) {
			logger.error("getAtributoLDAP() - Error obtendo o atributo " + nombreAtributo, exc.getLocalizedMessage());
		}

		return result;
	}
	
  private String getCname(UsuarioCyL usuarioCyL) {
    return usuarioCyL.getUsername();
  }

  private String getCname(Empleado empleado) {
    return empleado.getUsername();
  }

  private DistinguishedName userToDistinguishedName(UsuarioCyL usuarioCyL, UsuarioServiceImpl service) {
    return userToDistinguishedName(usuarioCyL.getUsername(), usuarioCyL.getCentro(), service);
  }

  private DistinguishedName userToDistinguishedName(Empleado empleado, EmpleadoServiceImpl service) {
    return empleadoToDistinguishedName(empleado.getUsername(), empleado.getCentro(), service);
  }

  /**
   * Devuelve el DN del usuario en el LDAP
   * @param userName
   * @param c
   * @param service
   * @return 
   */
  
  private DistinguishedName userToDistinguishedName(String userName, Centro c, UsuarioServiceImpl service) {
	  DistinguishedName dn = new DistinguishedName();
	  String userSuffix = service.findOneWhere(Parameter.class, "nombre", "openldap_user_suffix").getValor();
	  
	  if (generalProperties.containsKey("LDAP_USER_GROUPS")) {
		  String[] user_groups = ((String) generalProperties.get("LDAP_USER_GROUPS")).split(",");
		  
		  // Concatena la Unidad Organizativa destinada a los usuarios
		  if (userSuffix !="") {
			  if (userSuffix.toLowerCase().startsWith("ou=")){
				  dn.add("ou",userSuffix.substring(3));
			  } else {
				  dn.add("ou", userSuffix);
			  }
		  }
		  
		  // Concatena la Unidad Organizativa correspondiente al centro
		  if (c != null && c.getDominio() != null && !c.getDominio().isEmpty())
			  dn.add("ou", c.getDominio().replaceAll(" ", "").toUpperCase().replace('Ñ', 'N').replace('Á', 'A')
					  .replace('É', 'E').replace('Í', 'I').replace('Ó','O').replace('Ú', 'U'));
		  
		  // Concatena la Unidad Organizativa definida en el POM para el almacenamiento de usuarios
		  for (String grupo : user_groups)
			  dn.add("ou", grupo);
	  }
	  
	  // Concatena el uid del usuario
	  dn.add("uid", userName);
	  
	  return dn;
  }
  
  
  /**
   * Devuelve el DN del Empleado en el LDAP
   * @param userName
   * @param c
   * @param service
   * @return
   */
  private DistinguishedName empleadoToDistinguishedName(String userName, Centro c, EmpleadoServiceImpl service) {
    DistinguishedName dn = new DistinguishedName();
    String userSuffix = service.findOneWhere(Parameter.class, "nombre", "openldap_user_suffix").getValor();
    
    if (generalProperties.containsKey("LDAP_PERSONAL_GROUPS")) {
      String[] user_groups = ((String) generalProperties.get("LDAP_PERSONAL_GROUPS")).split(",");

      // Concatena la Unidad Organizativa destinada a los usuarios
      if (userSuffix !=""){
    	  if (userSuffix.toLowerCase().startsWith("ou=")){
    		  dn.add("ou",userSuffix.substring(3));
    	  }else{
    		  dn.add("ou", userSuffix);
    	  }
      }

      // Concatena la Unidad Organizativa correspondiente al centro
      if (c != null && c.getDominio() != null && !c.getDominio().isEmpty())
        dn.add("ou", c.getDominio().replaceAll(" ", "").toUpperCase());

      // Concatena la Unidad Organizativa definida en el POM para el almacenamiento de empleador
      for (String grupo : user_groups) {
        dn.add("ou", grupo);
      }
    }
    // Concatena el uid del empleados
    dn.add("uid", userName);
    return dn;
  }
  
  private DistinguishedName groupToDistinguishedName(String group, EmpleadoServiceImpl service) {
	  DistinguishedName dn = new DistinguishedName();
	  String groupSuffix = service.findOneWhere(Parameter.class, "nombre", "openldap_group_suffix").getValor();

      // Concatena la Unidad Organizativa destinada a los grupos
      if (groupSuffix !=""){
    	  if (groupSuffix.startsWith("ou=") || (groupSuffix.startsWith("OU="))){
    		  dn.add("ou",groupSuffix.substring(3));
    	  }else{
    		  dn.add("ou", groupSuffix);
    	  }
      }

	  dn.add("cn", group);
	  return dn;
  }
  
  private Attributes buildAttributes(UsuarioCyL usuarioCyL, UsuarioServiceImpl service) throws UnsupportedEncodingException {
    Attributes attrs = new BasicAttributes();
    String uidNumber = "";
    
    uidNumber = searchFreeUidNumber(service.findOneWhere(Parameter.class, "nombre", "openldap_user_suffix").getValor());
    
    BasicAttribute ocattr = new BasicAttribute("objectclass");
    ocattr.add("top");
    ocattr.add("person");
    ocattr.add("organizationalPerson");
    ocattr.add("inetOrgPerson");
    ocattr.add("sambaSamAccount");
    ocattr.add("posixAccount");
    ocattr.add("shadowAccount");
    attrs.put(ocattr);

    if (!usuarioCyL.getCentro().getDominio().isEmpty()) attrs.put("ou", usuarioCyL.getCentro().getDominio().replaceAll(" ", "").toUpperCase());
    if (!usuarioCyL.getNombre().isEmpty()) attrs.put("cn", usuarioCyL.getNombre());
    
    attrs.put("homeDirectory", "/home/" + getCname(usuarioCyL));
    attrs.put("sambaAcctFlags", "[UX]");
    attrs.put("sambaSID", service.findOneWhere(Parameter.class, "nombre", "openldap_sambaSID").getValor() + "-" + uidNumber);
    attrs.put("sambaKickoffTime", service.findOneWhere(Parameter.class, "nombre", "openldap_sambaKickoffTime").getValor());
    attrs.put("sambaLMPassword", service.findOneWhere(Parameter.class, "nombre", "openldap_sambaLMPassword").getValor());
    attrs.put("sambaLogoffTime", service.findOneWhere(Parameter.class, "nombre", "openldap_sambaLogoffTime").getValor());
    attrs.put("sambaLogonTime", service.findOneWhere(Parameter.class, "nombre", "openldap_sambaLogonTime").getValor());
    attrs.put("sambaNTPassword", service.findOneWhere(Parameter.class, "nombre", "openldap_sambaNTPassword").getValor());
    attrs.put("sambaPwdCanChange", service.findOneWhere(Parameter.class, "nombre", "openldap_sambaPwdCanChange").getValor());
    attrs.put("sambaPwdLastSet", service.findOneWhere(Parameter.class, "nombre", "openldap_sambaPwdLastSet").getValor());
    attrs.put("sambaPwdMustChange", service.findOneWhere(Parameter.class, "nombre", "openldap_sambaPwdMustChange").getValor());
    attrs.put("shadowLastChange", service.findOneWhere(Parameter.class, "nombre", "openldap_shadowLastChange").getValor());
    attrs.put("shadowMax", service.findOneWhere(Parameter.class, "nombre", "openldap_shadowMax").getValor());
    attrs.put("loginShell", service.findOneWhere(Parameter.class, "nombre", "openldap_loginShell").getValor());
    attrs.put("gecos", "");

    if (!usuarioCyL.getNombre().isEmpty() || !usuarioCyL.getApellido1().isEmpty()) 
    	attrs.put("displayName", usuarioCyL.getNombre()+ " " + usuarioCyL.getApellido1());
    if (!usuarioCyL.getApellido1().isEmpty() || !usuarioCyL.getApellido2().isEmpty()) 
    	attrs.put("sn", usuarioCyL.getApellido1()+ " " + usuarioCyL.getApellido2());
    if (!getCname(usuarioCyL).isEmpty()) attrs.put("uid", getCname(usuarioCyL));
    attrs.put("uidNumber", uidNumber);
    if (!getCname(usuarioCyL).isEmpty()) attrs.put("description", getCname(usuarioCyL));
    attrs.put("employeeType", service.findOneWhere(Parameter.class, "nombre", "openldap_employeeType_usuarios").getValor());
    if (!usuarioCyL.getNombre().isEmpty()) attrs.put("givenName", usuarioCyL.getNombre());
    if (!usuarioCyL.getEmail().isEmpty()) attrs.put("mail", usuarioCyL.getEmail());
    if (!usuarioCyL.getDireccion().isEmpty()) attrs.put("postalAddress", usuarioCyL.getDireccion());
    if (!usuarioCyL.getCp().isEmpty()) attrs.put("postalCode", usuarioCyL.getCp());
    if (!usuarioCyL.getLocalidad().getNombre().isEmpty()) attrs.put("postOfficeBox", usuarioCyL.getLocalidad().getNombre());
    if (!usuarioCyL.getProvincia().getNombre().isEmpty()) attrs.put("st", usuarioCyL.getProvincia().getNombre());
    if (!usuarioCyL.getTelefono().isEmpty()) attrs.put("telephoneNumber", usuarioCyL.getTelefono());
    
    // Establece el identificador de grupo al que pertenece
    /*if (generalProperties.containsKey("LDAP_USER_DEFAULT_PERMS")) {
      String[] perms = ((String) generalProperties.get("LDAP_USER_DEFAULT_PERMS")).split(",");
      int permisos = 0;
      for (String permiso : perms)
        permisos += Integer.valueOf(permiso);
      attrs.put("gidNumber", String.valueOf(permisos));
      attrs.put("sambaPrimaryGroupSID", service.findOneWhere(Parameter.class, "nombre", "openldap_sambaSID").getValor() + "-" + String.valueOf(permisos));
    }*/
    String groupPermsions = service.findOneWhere(Parameter.class, "nombre", "openldap_gidNumber_usuarios").getValor();
    attrs.put("gidNumber", groupPermsions);
    attrs.put("sambaPrimaryGroupSID", service.findOneWhere(Parameter.class, "nombre", "openldap_sambaSID").getValor() + "-" + groupPermsions);
    
    //Establece la contraseña de usuario
    //attrs.put("userPassword", encodePassword(service.findOneWhere(Parameter.class, "nombre", "openldap_userPassword").getValor()));
    String userPassword = service.findOneWhere(Parameter.class, "nombre", "openldap_userPassword").getValor();
    if ((userPassword != null) && !userPassword.isEmpty())
    	attrs.put("userPassword", userPassword);

    return attrs;
  }

  private Attributes buildAttributes(Empleado empleado, EmpleadoServiceImpl service) throws UnsupportedEncodingException {
	    Attributes attrs = new BasicAttributes();
	    String uidNumber = "";

	    /*if (empleado.getNif().startsWith("baja_"))
	    	uidNumber =  empleado.getNif().substring(5, empleado.getNif().length()).replace("_", "");
	    else
	    	uidNumber =  empleado.getNif().substring(0, empleado.getNif().length()-1);*/
	    	    
	    uidNumber = generalProperties.get("LDAP_UIDNUMBER_EMPLEADO_PREFIX").toString() + stringFromInteger(7, empleado.getId());
	    
	    BasicAttribute ocattr = new BasicAttribute("objectclass");
	    ocattr.add("top");
	    ocattr.add("person");
	    ocattr.add("organizationalPerson");
	    ocattr.add("inetOrgPerson");
	    ocattr.add("sambaSamAccount");
	    ocattr.add("posixAccount");
	    ocattr.add("shadowAccount");
	    attrs.put(ocattr);

	    if (!empleado.getCentro().getDominio().isEmpty()) attrs.put("ou", empleado.getCentro().getDominio().replaceAll(" ", "").toUpperCase());
	    
	    if (empleado.getPuesto().equals(Empleado.Puesto.ADMIN))
	    	attrs.put("cn", service.findOneWhere(Parameter.class, "nombre", "openldap_cn_administradores").getValor());
	    
	    if (empleado.getPuesto().equals(Empleado.Puesto.AREA_MANAGER))
	    	attrs.put("cn", service.findOneWhere(Parameter.class, "nombre", "openldap_cn_empleados").getValor());
	    
	    if (empleado.getPuesto().equals(Empleado.Puesto.TRAINER))
	    	attrs.put("cn", service.findOneWhere(Parameter.class, "nombre", "openldap_cn_trainers").getValor());
	    
	    if (empleado.getPuesto().equals(Empleado.Puesto.EXTERNAL_PERSONAL))
	    	attrs.put("cn", service.findOneWhere(Parameter.class, "nombre", "openldap_cn_colaboradores").getValor());
	    
	    if (empleado.getPuesto().equals(Empleado.Puesto.DIGITAL_ALLY_MANAGER))
	    	attrs.put("cn", service.findOneWhere(Parameter.class, "nombre", "openldap_cn_aliados").getValor());
	    
	    
	    if (!empleado.getCentro().getLocalidad().getNombre().isEmpty()) attrs.put("gecos", empleado.getCentro().getLocalidad().getNombre()
	    																		  .replace('ñ', 'n').replace('á', 'a').replace('é', 'e').replace('í', 'i').replace('ó', 'o').replace('ú', 'u')
	    																		  .replace('Ñ', 'N').replace('Á', 'A').replace('É', 'E').replace('Í', 'I').replace('Ó', 'O').replace('Ú', 'U'));
	    
	    attrs.put("homeDirectory", "/home/" + getCname(empleado));
	    attrs.put("sambaAcctFlags", "[UX]");
	    attrs.put("sambaSID", service.findOneWhere(Parameter.class, "nombre", "openldap_sambaSID").getValor() + "-" + uidNumber);
	    attrs.put("sambaKickoffTime", service.findOneWhere(Parameter.class, "nombre", "openldap_sambaKickoffTime").getValor());
	    attrs.put("sambaLMPassword", service.findOneWhere(Parameter.class, "nombre", "openldap_sambaLMPassword").getValor());
	    attrs.put("sambaLogoffTime", service.findOneWhere(Parameter.class, "nombre", "openldap_sambaLogoffTime").getValor());
	    attrs.put("sambaLogonTime", service.findOneWhere(Parameter.class, "nombre", "openldap_sambaLogonTime").getValor());
	    attrs.put("sambaNTPassword", service.findOneWhere(Parameter.class, "nombre", "openldap_sambaNTPassword").getValor());
	    attrs.put("sambaPwdCanChange", service.findOneWhere(Parameter.class, "nombre", "openldap_sambaPwdCanChange").getValor());
	    attrs.put("sambaPwdLastSet", service.findOneWhere(Parameter.class, "nombre", "openldap_sambaPwdLastSet").getValor());
	    attrs.put("sambaPwdMustChange", service.findOneWhere(Parameter.class, "nombre", "openldap_sambaPwdMustChange").getValor());
	    attrs.put("shadowLastChange", service.findOneWhere(Parameter.class, "nombre", "openldap_shadowLastChange").getValor());
	    attrs.put("shadowMax", service.findOneWhere(Parameter.class, "nombre", "openldap_shadowMax").getValor());
	    attrs.put("loginShell", service.findOneWhere(Parameter.class, "nombre", "openldap_loginShell").getValor());
	    
	    if (!empleado.getApellido1().isEmpty() || !empleado.getApellido2().isEmpty()) 
	    	attrs.put("sn", empleado.getApellido1()+ " " + empleado.getApellido2());
	    if (!getCname(empleado).isEmpty()) attrs.put("uid", getCname(empleado));
	    attrs.put("uidNumber", uidNumber);
	    //attrs.put("description", getCname(empleado));
	    if (!empleado.getNombre().isEmpty() || !empleado.getApellido1().isEmpty()) 
	    	attrs.put("displayName", empleado.getNombre()+ " " + empleado.getApellido1());
	    attrs.put("departmentNumber", service.findOneWhere(Parameter.class, "nombre", "openldap_departmentNumber_empleados").getValor());
	    
	    if (empleado.getPuesto().equals(Empleado.Puesto.AREA_MANAGER))
	    	attrs.put("employeeType", service.findOneWhere(Parameter.class, "nombre", "openldap_employeeType_empleados").getValor());
	    if (empleado.getPuesto().equals(Empleado.Puesto.EXTERNAL_PERSONAL))
	    	attrs.put("employeeType", service.findOneWhere(Parameter.class, "nombre", "openldap_employeeType_colaboradores").getValor());
	    if (empleado.getPuesto().equals(Empleado.Puesto.DIGITAL_ALLY_MANAGER))
	    	attrs.put("employeeType", service.findOneWhere(Parameter.class, "nombre", "openldap_employeeType_aliado").getValor());
	    
	    if (!empleado.getNombre().isEmpty()) attrs.put("givenName", empleado.getNombre());
	    if (!empleado.getEmail().isEmpty()) attrs.put("mail", empleado.getEmail());
	    attrs.put("physicalDeliveryOfficeName", service.findOneWhere(Parameter.class, "nombre", "openldap_physicalDeliveryOfficeName_empleados").getValor());
	    if (!empleado.getTelefono().isEmpty()) attrs.put("telephoneNumber", empleado.getTelefono());
	    
	    if (!empleado.getCentro().getDireccion().isEmpty()) attrs.put("postalAddress", empleado.getCentro().getDireccion());
	    //if (!empleado.getCp().isEmpty()) attrs.put("postalCode", empleado.getCp());
	    if (!empleado.getCentro().getLocalidad().getNombre().isEmpty()) attrs.put("postOfficeBox", empleado.getCentro().getLocalidad().getNombre());
	    if (!empleado.getCentro().getProvincia().getNombre().isEmpty()) attrs.put("st", empleado.getCentro().getProvincia().getNombre());

	    
	    // Establece el identificador de grupo al que pertenece
	    /*if (generalProperties.containsKey("LDAP_USER_DEFAULT_PERMS")) {
	      String[] perms = ((String) generalProperties.get("LDAP_PERSONAL_DEFAULT_PERMS")).split(",");
	      int permisos = 0;
	      for (String permiso : perms)
	        permisos += Integer.valueOf(permiso);
	      attrs.put("gidNumber", String.valueOf(permisos));
	      attrs.put("sambaPrimaryGroupSID", service.findOneWhere(Parameter.class, "nombre", "openldap_sambaSID").getValor() + "-" + String.valueOf(permisos));
	    }*/
	    String groupPermsions = service.findOneWhere(Parameter.class, "nombre", "openldap_gidNumber_empleados").getValor();
	    attrs.put("gidNumber", groupPermsions);
	    attrs.put("sambaPrimaryGroupSID", service.findOneWhere(Parameter.class, "nombre", "openldap_sambaSID").getValor() + "-" + groupPermsions);


	    //Establece la contraseña de empleado
	    //attrs.put("userPassword", encodePassword(service.findOneWhere(Parameter.class, "nombre", "openldap_userPassword").getValor()));
	    String userPassword = service.findOneWhere(Parameter.class, "nombre", "openldap_userPassword").getValor();
	    if ((userPassword != null) && !userPassword.isEmpty())
	    	attrs.put("userPassword", userPassword);

	    return attrs;  
  }

  @SuppressWarnings("unused")
  private byte[] encodePassword(String password) throws UnsupportedEncodingException {
	  String passwordPreffix = "{SHA}";

	  ConfigurablePasswordEncryptor passwordEncryptor = new ConfigurablePasswordEncryptor();
	  passwordEncryptor.setAlgorithm("SHA-1");
	  passwordEncryptor.setPlainDigest(true);
	  
	  String encryptedPassword = passwordEncryptor.encryptPassword(password);
	  
	  if (passwordEncryptor.checkPassword(password, encryptedPassword)) {
		  // correct!
		  logger.info("encodePassword() - Password encriptada correctamente - ", password);
	  } else {
		  // bad login!
		  logger.info("encodePassword() - Password mal correctamente - ", password);
	  }
	  	  
	  return (passwordPreffix + encryptedPassword).getBytes("UTF-8");
  }

  /**
   * Actualizar informacion ldap.
   * 
   * @param user
   *          user
   * @param oldName
   *          old name
   */
  public void actualizarInformacionLDAP(UsuarioCyL user, String oldName, UsuarioServiceImpl service) {
    if (generalProperties.containsKey("MAKE_LDAP_CHANGES") &&
    	((String) generalProperties.get("MAKE_LDAP_CHANGES")).equalsIgnoreCase("true")) {

      try {

        if (user.getUsername() != null) {
          Object o = ldapTemplate.lookup(userToDistinguishedName(oldName, user.getCentro(), service));
          if (o != null) updateUser(user, oldName, service);
        }
      } catch (NameNotFoundException e) {
        try {
          if (user.getNombreCompleto() != null) {
            Object o = ldapTemplate.lookup(userToDistinguishedName(oldName, user.getCentro(), service));
            if (o != null) updateUser(user, oldName, service);
          }
        } catch (NameNotFoundException n) {
          try {
            if (user.getNif() != null && !user.getNif().trim().isEmpty()) {
              Object o = ldapTemplate.lookup(userToDistinguishedName(user.getNif(), user.getCentro(), service));
              if (o != null) updateUser(user, oldName, service);
            }
          } catch (NameNotFoundException nn) {
            try {
              createUser(user, service);
            } catch (Exception ex) {
              ex.printStackTrace();
              user.getRegistrosLdap().add(new RegistroLdap(user, "Error al actualizar LdapComponent", false));
            }
          }
        } catch (Exception ex) {
          ex.printStackTrace();
          user.getRegistrosLdap().add(new RegistroLdap(user, "Error al actualizar LdapComponent", false));
        }
      } catch (Exception e) {
        e.printStackTrace();
        user.getRegistrosLdap().add(new RegistroLdap(user, "Error al actualizar LdapComponent", false));
      }
    }

  }

  public void actualizarInformacionLDAP(Empleado empleado, String oldName, Centro oldCentro, EmpleadoServiceImpl service) {
    if (generalProperties.containsKey("MAKE_LDAP_CHANGES")
        && ((String) generalProperties.get("MAKE_LDAP_CHANGES")).equalsIgnoreCase("true")) {
    	
    	try {
			Object o = ldapTemplate.lookup(empleadoToDistinguishedName(oldName, oldCentro, service));
			if (o != null)
				updateUser(empleado, oldName, oldCentro, service);
    	}
    	catch (NameNotFoundException nn) {
            try {
            	createUser(empleado, service);
            	updateGroup(empleado.getCentro().getDominio().replaceAll(" ", ""), empleado, service);
            	if (empleado.getPuesto().equals(Role.ROLE_AREA_MANAGER))
            		updateGroup(service.findOneWhere(Parameter.class, "nombre", "openldap_nombreGrupoAgentesTIC").getValor(), empleado, service);
            }
            catch (Exception ex) {
            	ex.printStackTrace();
            }
        }
    	catch (Exception ex) {
    		ex.printStackTrace();
    	}
    }

  }

  /**
   * Actualizar informacion ldap.
   * 
   * @param user
   *          user
   */
  public void actualizarInformacionLDAP(UsuarioCyL user, UsuarioServiceImpl service) {
    actualizarInformacionLDAP(user, user.getUsername(), service);
  }
  
  /**
   * Convierte un número en una cadena con una cantidad de cifras
   * 
   * @param size
   * 			Número de cifras en la cadena del resultado
   * @param number
   * 			Número que se desea convertir.
   */
  private String stringFromInteger(int size, Long number) {
	  String result = number.toString();
	  
	  for (int i = 0; result.length() < size; i++)
		  result = "0" + result;
	  
	  return result;
  }
  
  private String searchFreeUidNumber(String userSuffix) {
	  DistinguishedName dn = new DistinguishedName();
	  
	  if (userSuffix !=""){
    	  if (userSuffix.toLowerCase().startsWith("ou=")) {
    		  dn.add("ou",userSuffix.substring(3));
    	  }
    	  else {
    		  dn.add("ou", userSuffix);
    	  }
      }
	  
	  Random generador = new Random();
	  Integer generatedUidNumber;
	  
	  javax.naming.directory.SearchControls controls = new javax.naming.directory.SearchControls();
      controls.setSearchScope(javax.naming.directory.SearchControls.SUBTREE_SCOPE);
      
      List encontrados;
      
      do {
    	  generatedUidNumber = generador.nextInt(909999999);

    	  encontrados = ldapTemplate.search(dn, "(& (objectClass=person) (uidNumber=" + generatedUidNumber + "))", controls,
    			  							new AttributesMapper() {
    		  									public Object mapFromAttributes(Attributes attrs)
    		  										throws NamingException {
    		  										return attrs.get("uid").get();
    		  									}
    	  									});
      } while (encontrados.size() > 0);
      
      return String.valueOf(generatedUidNumber);
  }
}
