package es.csa.fdt.cyldigital.usuarios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;
import java.util.List;

import javax.naming.NamingException;
import javax.naming.directory.Attributes;

import org.springframework.ldap.core.AttributesMapper;
import org.springframework.ldap.core.LdapTemplate;
import org.springframework.ldap.filter.AndFilter;
import org.springframework.ldap.filter.EqualsFilter;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;

/**
 * Clase LdapService.
 * 
 * @author csa
 */
@Service
public class LdapService implements Serializable {

  private static final long serialVersionUID = 6770140088243104703L;

  private transient LdapTemplate ldapTemplate;

  /**
   * Devuelve ldap template.
   * 
   * @return ldap template
   */
  public LdapTemplate getLdapTemplate() {
    return ldapTemplate;
  }

  /**
   * Asigna ldap template.
   * 
   * @param ldapTemplate
   *          ldap template
   */
  public void setLdapTemplate(LdapTemplate ldapTemplate) {
    this.ldapTemplate = ldapTemplate;
  }

  /**
   * Devuelve ldap attributes.
   * 
   * @param usuario
   *           usuario
   * @return ldap attributes
   */
  @SuppressWarnings("unchecked")
  public Attributes getLdapAttributes(UsuarioLdap usuario) {

    Attributes result = null;
    System.out.println("Buscando atributos del usuario" + usuario.getUsername());
    // Crear filtro LDAP
    AndFilter filter = new AndFilter();
    filter.and(new EqualsFilter("objectclass", "user"));
    filter.and(new EqualsFilter("sAMAccountName", usuario.getUsername()));

    System.out.println("Buscando en el general : filtro:" + filter.encode());

    // Repetir la busqueda
    try {
      List<Attributes> results = ldapTemplate.search("", filter.encode(), new AttributesMapper() {
        public Object mapFromAttributes(Attributes attrs) throws NamingException {
          return attrs;
        }
      });

      if (results != null && !results.isEmpty()) {
        return results.get(0);
      } else {
        System.out.println("En el general no se ha encontrado");
      }

    } catch (RuntimeException e) {
      System.out.println("no se pudieron cargar atributos LDAP (loguear que el LDAP no está disponible");
      e.printStackTrace();
    }

    if (result == null) {
      System.out.println("-----EL USUARIO NO SE HA ENCONTRADO------");
    }

    return result;
  }

}
