package es.csa.fdt.cyldigital.usuarios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.Localidad;
import es.csa.fdt.cyldigital.usuarios.entity.Proveedor;
import es.csa.fdt.cyldigital.usuarios.entity.Provincia;


/**
 * Clase ProveedorServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class ProveedorServiceImpl extends AbstractServiceImpl<Proveedor, Long> implements ProveedorService {

  private static final long serialVersionUID = 918589720985035611L;

  private List<Provincia> provincias;
  private List<Proveedor> proveedores;
  private String nombreLocalidad = "";
  private List<Localidad> listLocalidades = null;
  private Long idLocalidadSeleccionada = -1L;

  /**
   * Construye un nuevo objeto de la clase
   */
  public ProveedorServiceImpl() {
    super();
  }

  /**
   * Devuelve provincias.
   * 
   * @return provincias
   */
  public List<Provincia> getProvincias() {
    if (provincias == null) provincias = findAllOrder(Provincia.class, "nombre", true);
    return provincias;
  }

  /**
   * Elimina proveedor.
   * 
   * @param proveedor
   *           proveedor
   */
  public void removeProveedor(Proveedor proveedor) {
    if (proveedor.getId() != 0) {
    	List<Empleado> empleados = proveedor.getEmpleados();
    	
    	for (Empleado e : empleados) {
    		e.setProveedor(null);
    		
    		setMessagesEnabled(false);
    		persist(e);
    		setMessagesEnabled(true);
    	}

    	proveedor.getEmpleados().clear();
    	
    	remove(proveedor);
    }
  }
  
  /**
   * Devuelve proveedores disponibles.
   * 
   * @return proveedores disponibles
   */
  public List<Proveedor> getProveedoresDisponibles(){
    return findAll(Proveedor.class);
  }

  /**
   * Devuelve proveedores.
   * 
   * @return proveedores
   */
  public List<Proveedor> getProveedores() {
    if (proveedores == null) proveedores = findAllOrder(Proveedor.class, "nombre", true);
    return proveedores;
  }
  
  public String getNombreLocalidad() {
	  return nombreLocalidad;
  }

  public void setNombreLocalidad(String nombreLocalidad) {
	  this.nombreLocalidad = nombreLocalidad;
  }

  public List<Localidad> getListLocalidades() {
	  return listLocalidades;
  }

  public void searchLocalidad(Provincia provincia) {
	  if (nombreLocalidad.length() > 1) {
		  String strqry = "SELECT localidad " +
		  				  "FROM " + Localidad.class.getName() + " localidad " +
		  				  "WHERE localidad.provincia.id = " + provincia.getId() + " AND" +
		  				  "      localidad.nombre LIKE '%%" + nombreLocalidad + "%%' " +
		  				  "ORDER BY localidad.nombre ASC";
		  
		  listLocalidades = findByQuery(Localidad.class, strqry);
	  }
	  else {
		  if (listLocalidades != null) {
			  listLocalidades.clear();
			  listLocalidades = null;
		  }
	  }
  }

  /**
   * @return the idLocalidadSeleccionada
   */
  public Long getIdLocalidadSeleccionada() {
	  return idLocalidadSeleccionada;
  }

  /**
   * @param idLocalidadSeleccionada the idLocalidadSeleccionada to set
   */
  public void setIdLocalidadSeleccionada(Long idLocalidadSeleccionada) {
	  this.idLocalidadSeleccionada = idLocalidadSeleccionada;
  }
 
  public void applyLocalidadToUsuario(Proveedor usuarioToSetLocalidad) {
	  if (idLocalidadSeleccionada == -1)
		  usuarioToSetLocalidad.setLocalidad(null);
	  else {
		  Map<String, Object> properties = new HashMap<String, Object>();
		  properties.put("provincia", usuarioToSetLocalidad.getProvincia());
		  properties.put("id", idLocalidadSeleccionada);
		  
		  usuarioToSetLocalidad.setLocalidad(findOneWhere(Localidad.class, properties));
	  }
  }
}
