package es.csa.fdt.cyldigital.usuarios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.commons.persistence.browser.jpaql.FilterJPAQLByQuery;
import es.csa.fdt.cyldigital.core.service.BrowserFilteredByActive;
import es.csa.fdt.cyldigital.usuarios.entity.Proveedor;

/**
 * Servicio del Browser de Proveedores.
 * 
 * @author csa
 */
@Service
@Repository
public class ProveedoresBrowserImpl extends BrowserFilteredByActive<Proveedor> implements
    ProveedoresBrowser<CriterionJPAQL, String> {

  private static final long serialVersionUID = -7227366052245717441L;

  private String customFilterProperty;
  
  protected Proveedor.TipoEmpresa type;
  
  protected Proveedor.Sector sector;

  /**
   * Construye un nuevo objeto de la clase
   */
  public ProveedoresBrowserImpl() {
    super();
    getSort().setSortAsc(false);
    getSort().setSortProperty("id");
  }

  public Proveedor.TipoEmpresa getType() {
	    return type;
  }

  public void setType(Proveedor.TipoEmpresa type) {
	  this.type = type;
  }


  public Proveedor.Sector getSector() {
	    return sector;
  }

  public void setSector(Proveedor.Sector sector) {
	  this.sector = sector;
  }
  
  public void filterByType(Proveedor.TipoEmpresa type) {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<Proveedor> filter = new FilterJPAQLByQuery<Proveedor>();

    query += " tipoEmpresa = ? ";
    filter.getCriterion().addParameter(type);

    for (Filter<CriterionJPAQL, Proveedor> f : getFilters())
      if (f.getCriterion().getQuery().contains("tipoEmpresa")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }

  public void filterBySector(Proveedor.Sector sector) {
	    String query = "";
	    boolean flag = false;
	    FilterJPAQLByQuery<Proveedor> filter = new FilterJPAQLByQuery<Proveedor>();

	    query += " sector = ? ";
	    filter.getCriterion().addParameter(sector);

	    for (Filter<CriterionJPAQL, Proveedor> f : getFilters())
	      if (f.getCriterion().getQuery().contains("sector")) flag = true;

	    if (!flag) {
	      filter.getCriterion().setQuery(query);
	      getFilters().add(filter);
	    }
  }

  protected void removeTypeFilter() {
	    int i = -1;
	    for (Filter<CriterionJPAQL, Proveedor> f : getFilters())
	      if (f.getCriterion().getQuery().contains("tipoEmpresa")) i = getFilters().indexOf(f);
	    if (i > -1) getFilters().remove(i);
	  }
  
  protected void removeSectorFilter() {
	    int i = -1;
	    for (Filter<CriterionJPAQL, Proveedor> f : getFilters())
	      if (f.getCriterion().getQuery().contains("sector")) i = getFilters().indexOf(f);
	    if (i > -1) getFilters().remove(i);
	  }

  public void filter() {
	removeTypeFilter();
    if (getType() != null) filterByType(getType());
	removeSectorFilter();
    if (getSector() != null) filterBySector(getSector());
  }

  @Override
  public void clearFilters() {
    super.clearFilters();
    filter();
  }
  
  /**
   * Apply custom filter.
   */
  public void applyCustomFilter() {
    String query = "cif LIKE ? OR nombre LIKE ?";
    Long id = null;
    clearFilters();
    if (getCustomFilterProperty() != null && !getCustomFilterProperty().trim().isEmpty()) {
      try {
        id = Long.parseLong(customFilterProperty);
      } catch (Exception e) {}

      FilterJPAQLByQuery<Proveedor> filter = new FilterJPAQLByQuery<Proveedor>();
      if (id != null) {
        query += " OR id = ?";
        Object[] params = { "%" + customFilterProperty + "%", "%" + customFilterProperty + "%", id };
        filter.getCriterion().addParameters(params);
      } else {
        Object[] params = { "%" + customFilterProperty + "%", "%" + customFilterProperty + "%" };
        filter.getCriterion().addParameters(params);
      }
      filter.getCriterion().setQuery(query);
      filters.add(filter);
    }
    setCustomFilterProperty("");

  }

  /**
   * Asigna custom filter property.
   * 
   * @param customFilterProperty
   *          custom filter property
   */
  public void setCustomFilterProperty(String customFilterProperty) {
    this.customFilterProperty = customFilterProperty;
  }

  /**
   * Devuelve custom filter property.
   * 
   * @return custom filter property
   */
  public String getCustomFilterProperty() {
    return customFilterProperty;
  }
}
