package es.csa.fdt.cyldigital.usuarios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.List;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import com.google.common.cache.Cache;

import es.csa.commons.persistence.validator.NIFUtils;
import es.csa.commons.util.AditionalStringUtils;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.notificacion.NotificationService;
import es.csa.fdt.cyldigital.usuarios.entity.CodigoPostal;
import es.csa.fdt.cyldigital.usuarios.entity.Provincia;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioPre;

/**
 * Clase UsuarioGenServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class UsuarioGenServiceImpl extends AbstractServiceImpl<Usuario, Long> implements UsuarioGenService {

  private static final long serialVersionUID = -5575228180361917674L;

  private List<Centro> centros;

  private List<Provincia> provincias;
  
  private String lastCP = "";
  
  private boolean avisoMenorNoIdentificado;

  /**
   * 
   */
  public UsuarioGenServiceImpl() {
	  super();
	  
	  avisoMenorNoIdentificado = false;
  }

  /**
   * Devuelve centros.
   * 
   * @return centros
   */
  public List<Centro> getCentros() {
    if (centros == null) centros = findAllOrder(Centro.class, "nombre", true);
    return centros;
  }

  /**
   * Devuelve provincias.
   * 
   * @return provincias
   */
  public List<Provincia> getProvincias() {
    if (provincias == null) provincias = findAllOrder(Provincia.class, "nombre", true);
    return provincias;
  }

  /**
   * Info message.
   * 
   * @param code
   *           code
   */
  public void infoMessage(String code) {
    addInfoMessage(code, null);
  }

  /**
   * Info message.
   * 
   * @param code
   *           code
   * @param args
   *           args
   */
  public void infoMessage(String code, Object[] args) {
    addInfoMessage(code, args);
  }

  /**
   * Warn message.
   * 
   * @param code
   *           code
   */
  public void warnMessage(String code) {
    addWarnMessage(code, null);
  }

  /**
   * Warn message.
   * 
   * @param code
   *           code
   * @param args
   *           args
   */
  public void warnMessage(String code, Object[] args) {
    addWarnMessage(code, args);
  }

  /**
   * Error message.
   * 
   * @param code
   *           code
   */
  public void errorMessage(String code) {
    addErrorMessage(code, null);
  }

  /**
   * Error message.
   * 
   * @param code
   *           code
   * @param args
   *           args
   */
  public void errorMessage(String code, Object[] args) {
    addErrorMessage(code, args);
  }

  /**
   * Persist usuario.
   * 
   * @param usuario
   *           usuario
   * @return true, if successful
   */
  public boolean persistUsuario(UsuarioPre usuarioPre, UsuarioCyL usuarioCyL, UsuarioServiceImpl usuarioService) {
	  if (usuarioPre.getLocalidad().getId() == -1L) {
		  addErrorMessage("usuario.error.localidad");
		  
		  return false;
	  }
	  
	  if(!usuarioService.checkEmailConfirmacion(usuarioPre.getEmail())) {
		  addErrorMessage("usuario.error.emailConfirmacion");
		  
		  return false;
	  }
	   
	  ServletRequestAttributes attr = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
	  HttpServletRequest httpReq = attr.getRequest();
	  // Get the token sent with the request
      String token = (String) httpReq.getParameter("_csrfToken");

      // Validate that the token is in the cache
      @SuppressWarnings("unchecked")
	  Cache<String, Boolean> csrfPreventionSaltCache = (Cache<String, Boolean>)
          httpReq.getSession().getAttribute("_csrfSecurityCache");
      
      if (csrfPreventionSaltCache != null &&
    		  token != null &&
              csrfPreventionSaltCache.getIfPresent(token) != null &&
              csrfPreventionSaltCache.getIfPresent(token)) {
    	  			System.out.println("Token de validación CSRF encontrado.");
      } else {
    	  // Otherwise we throw an exception aborting the request flow
    	  addErrorMessage("Potential CSRF detected!! Inform a scary sysadmin ASAP.");
    	  return false;
      }
	  
	  //3.4.3-R RF_XEA_19 
	  //Transformar el nombre y apellidos
	  usuarioPre.setNombre(AditionalStringUtils.formatCapitalized(usuarioPre.getNombre())); 	  
	  usuarioPre.setApellido1(AditionalStringUtils.formatCapitalized(usuarioPre.getApellido1()));	  
	  if ((usuarioPre.getApellido2() != null) && !usuarioPre.getApellido2().isEmpty())
		  usuarioPre.setApellido2(AditionalStringUtils.formatCapitalized(usuarioPre.getApellido2()));	  
	  
	  if ((usuarioPre.getNif() != null) && !usuarioPre.getNif().isEmpty())
		  usuarioPre.setNif(usuarioPre.getNif().toUpperCase());
	  
	  if ((usuarioPre.getPasaporte() != null) && !usuarioPre.getPasaporte().isEmpty())
		  usuarioPre.setPasaporte(usuarioPre.getPasaporte().toUpperCase());
	  
	  if ((usuarioPre.getNifAutorizado() != null) && !usuarioPre.getNifAutorizado().isEmpty())
		  usuarioPre.setNifAutorizado(usuarioPre.getNifAutorizado().toUpperCase());
	  
	  if ((usuarioPre.getNombreCompletoAutorizado() != null) && !usuarioPre.getNombreCompletoAutorizado().isEmpty())
		  usuarioPre.setNombreCompletoAutorizado(AditionalStringUtils.formatCapitalized(usuarioPre.getNombreCompletoAutorizado()));
    
	  if (!(compruebaDNI(usuarioPre) && compruebaCP(usuarioPre) && compruebaFNacimiento(usuarioPre)))
		  return false;
	  else {
		  if (!usuarioPre.isAceptaPolitica()) {
			  addErrorMessage("usuarios.alta.solicitud.ErrorAceptaPolitica");

			  return false;
		  }
		  
		  if (!compruebaEmail(usuarioPre)) {
			  if (usuarioPre.getEdad() >= 14) {
				  addErrorMessage("usuarios.alta.errorEmailDucplicado");
				  
				  avisoMenorNoIdentificado = false;
				  
				  return false;
			  }
			  else { //if (usuarioPre.getEdad() < 14)
				  if (!avisoMenorNoIdentificado) {
					  addWarnMessage("usuarios.alta.avisoEmailDuplicadoMenor");
					  
					  avisoMenorNoIdentificado = true;
					  
					  return false;
				  }
			  }
		  }

		  if ( usuarioPre.getEdad() >= 14 ) {
			  convertPreToCyL(usuarioPre, usuarioCyL);
			  
			  return usuarioService.persistUsuario(usuarioCyL, "", "", 0L, false);
		  }
		  else {
			  setMessagesEnabled(false);
			  persist(usuarioPre);
			  setMessagesEnabled(true);

			  return true;
		  }
	  }

	  /*
	  usuario.setNif(usuario.getNif().toUpperCase());

	  if ((usuario.getNif() != null) && !usuario.getNif().isEmpty()) { 
		  List<Usuario> usuarios = findWhere(Usuario.class, "nif", usuario.getNif());

		  if (usuarios.size() > 0) {
			  addErrorMessage("usuario.error.dniYaUsado"); //+ usuarios.get(0).getNombreCompleto()
			  return false;
		  }

		  if (!NIFUtils.isValid(usuario.getNif())) {
			  addErrorMessage("usuario.error.dni");
			  return false;
		  }
	  }

	  disableMessages();
	  persist(usuario);
	  enableMessages();
    
	  //notificarSolicitudPreinscripcionUsuario(usuario);
    
	  return true;
	  */
  }

  /**
   * Comprueba que el Email no este duplicado
   * 
   */
  private Boolean compruebaEmail(UsuarioPre upreToPersist) {
	  Boolean result = true;
	  
	  List<UsuarioCyL> lucyl = findWhere(UsuarioCyL.class, "email", upreToPersist.getEmail());
	  List<UsuarioPre> lupre = findWhere(UsuarioPre.class, "email", upreToPersist.getEmail());
	  
	  if (lucyl != null) {
		  for (UsuarioCyL ucyl : lucyl) {
			  if (ucyl.getId() != upreToPersist.getId())
				  result = false;
		  }
	  }

	  if (lupre != null) {
		  for (UsuarioPre upre : lupre) {
			  if (upre.getId() != upreToPersist.getId())
				  result = false;
		  }
	  }

	  return result;
  }
  
  private boolean compruebaDNI(UsuarioPre usuarioPre) {
	  //Usuario extranjero y mayor de 14 años con identificación.
	  if (usuarioPre.isExtranjero() && !usuarioPre.isMenor()) {
		  if ((usuarioPre.getPasaporte() == null) || usuarioPre.getPasaporte().isEmpty()) {
			  addErrorMessage("Los usuarios extranjeros sin N.I.F. deben introducir el pasaporte");
			  return false;
		  }
		  else {
			  //Se comprueba que no exista un usuario dado de alta con el mismo pasaporte.
			  List<UsuarioCyL> extranjerosCyL = findWhere(UsuarioCyL.class, "pasaporte", usuarioPre.getPasaporte().trim());
		      if (extranjerosCyL.size() > 0) {
		    	  addErrorMessage("El pasaporte está siendo usado por otro usuario dado de alta en la red CeMIT. Verifíquelo y en caso de ser correcto acuda al centro para solucionarlo y formalizar su alta de usuario.");
		    		  
	    		  return false;
		      }
		      
		      //Se comprueba que no exista otra preinscripción con el mismo pasaporte.
		      List<UsuarioPre> extranjerosPre = findWhere(UsuarioPre.class, "pasaporte", usuarioPre.getPasaporte().trim());
		      if (extranjerosPre.size() > 0) {
    			  addErrorMessage("Ya existe una usuario preinscrito con este pasaporte. Verifíquelo y en caso de ser correcto acuda al centro para solucionarlo y formalizar su alta de usuario.");

    			  return false;
		      }

		      return true;
		  }
	  }
	  else {
		  if (usuarioPre.isMenor()) {
			  usuarioPre.setNif("");

			  if ((usuarioPre.getNifAutorizado() != null) && !usuarioPre.getNifAutorizado().trim().isEmpty()) {
				  // comprobación básica de q no esté repetido el usuario. El nif autorizador puede estarlo.
				  String nombrePre = usuarioPre.getNombreCompleto().trim().toLowerCase().replace(" ", "");
				  
				  List<UsuarioCyL> usuariosCyLMismoNifAutorizado = findWhere(UsuarioCyL.class, "nifAutorizado", usuarioPre.getNifAutorizado());

				  for (UsuarioCyL u : usuariosCyLMismoNifAutorizado) {
					  String nombreCyL = u.getNombreCompleto().trim().toLowerCase().replace(" ", "");

					  if ((nombreCyL.compareTo(nombrePre) == 0) &&
						  (u.getFechaNacimiento().compareTo(usuarioPre.getFechaNacimiento()) == 0)) {
						  addErrorMessage("Actualmente existe un usuario de la red CeMIT con el mismo nombre, fecha de nacimiento y autorizado por una persona con el mismo número de identificación. Verifíquelo y en caso de ser correcto acuda al centro para solucionarlo y formalizar su alta de usuario.");
						  
						  return false;
					  }
				  }
				  
				  List<UsuarioPre> usuariosPreMismoNifAutorizado = findWhere(UsuarioPre.class, "nifAutorizado", usuarioPre.getNifAutorizado());

				  for (UsuarioPre up : usuariosPreMismoNifAutorizado) {
					  String nombrePreDeLista = up.getNombreCompleto().trim().toLowerCase().replace(" ", "");

					  if ((nombrePreDeLista.compareTo(nombrePre) == 0) &&
						  (up.getFechaNacimiento().compareTo(usuarioPre.getFechaNacimiento()) == 0)) {
						  addErrorMessage("Actualmente existe otro usuario preinscrito con el mismo nombre, fecha de nacimiento y autorizado por una persona con el mismo número de identificación. Verifíquelo y en caso de ser correcto acuda al centro para solucionarlo y formalizar su alta de usuario.");
						  
						  return false;
					  }
				  }
	          
				  return true;
			  }
		  }
	      
	      // Llegado este punto no es menor de 14 años sin N.I.F. ni extranjero
		  if (usuarioPre.getNif() != null)
			  usuarioPre.setNif(usuarioPre.getNif().toUpperCase());
		  
		  if (NIFUtils.isValid(usuarioPre.getNif())) {			  
			  List<Usuario> usersMismoNIF = buscarUsuariosNIF(usuarioPre.getNif());
	    	  
			  if (usersMismoNIF.isEmpty())
				  return true;
			  else {
				  String mensaje = "";
	            
				  if(usersMismoNIF.get(0) instanceof UsuarioPre) {
					  mensaje = "Ya existe un usuario preinscrito con el mismo N.I.F.";
				  }
				  else if (usersMismoNIF.get(0) instanceof UsuarioCyL) {
					  mensaje = "Ya existe un usaruio dado de alta en la red CeMIT con el mismo N.I.F.";
				  }
				  
				  addErrorMessage(mensaje + " Verifíquelo y en caso de ser correcto acuda al centro para solucionarlo y formalizar su alta de usuario.");
	            
				  return false;
			  }
		  } else {
			  addErrorMessage("usuario.error.dni");
	    	  
			  return false;
		  }
	  }
  }
  
  private List<Usuario> buscarUsuariosNIF(String nif) {
	  List<Usuario> usuarios = findWhere(Usuario.class, "nif", nif);

	  return usuarios;
  }
  
  private boolean compruebaCP(UsuarioPre usuarioPre) {
	  if ((usuarioPre.getLocalidad() != null) && (usuarioPre.getCp() != null) &&
		  !usuarioPre.getCp().trim().isEmpty() && !lastCP.equals(usuarioPre.getCp()))
		  for (CodigoPostal cod : usuarioPre.getLocalidad().getCodigosPostales()) {
			  if (cod.getCodigo().equals(usuarioPre.getCp().trim()))
				  return true;
		  }
	  else
		  return true;

	  lastCP = usuarioPre.getCp();
	  
	  addWarnMessage("usuario.error.codigo_postal");
	  
	  return false;
  }
  
  private boolean compruebaFNacimiento(UsuarioPre usuarioPre) {
	  if (usuarioPre.getFechaNacimiento() != null) {
		  if (usuarioPre.getEdad() < 1) {
			  addErrorMessage("usuario.error.fecha_nacimiento");

			  return false;
		  }
	      
		  if (usuarioPre.isMenor() && (usuarioPre.getEdad() >= 14)) {
			  addErrorMessage("Se ha marcado el usuario como menor de 14 sin N.I.F. pero tiene 14 años o más.");

			  return false;
		  }

		  /*if (usuarioPre.getEdad() < 18) {*/
	      if (usuarioPre.getEdad() < 14) {
			  if ((usuarioPre.getNifAutorizado() == null) || usuarioPre.getNifAutorizado().trim().isEmpty()) {
				  addErrorMessage("El usuario es menor y debe introudcir el N.I.F. de su padre o tutor que lo atorice.");

				  return false;
			  }

			  if ((usuarioPre.getNombreCompletoAutorizado() == null) || usuarioPre.getNombreCompletoAutorizado().trim().isEmpty()) {
				  addErrorMessage("El usuario es menor y debe introducir el nombre completo de su padre o tutor que lo autorice.");

				  return false;
			  }
		  }

		  if (!usuarioPre.isExtranjero() && usuarioPre.getEdad() >= 14 /*&& usuarioCyL.getEdad() < 18*/) {
			  //usuarioCyL.setMenor(false);
	    	
			  if ((usuarioPre.getNif() == null) || usuarioPre.getNif().trim().isEmpty()) {
				  //addErrorMessage("El usuario tiene entre 14 y 18 años por lo que debe proporcionar su nº de N.I.F.");
				  addErrorMessage("El usuario tiene 14 años o más por lo que debe proporcionar su N.I.F.");
	          
				  return false;
			  }
		  }

		  /*if (usuarioCyL.getEdad() > 18 && usuarioCyL.isMenor()) {
			  addWarnMessage("El usuario está marcado como menor pero su edad es mayor que 18");
	          return false;
		  }*/
	  }

	  return true;
  }

  /**
   * Find pre to cy l.
   * 
   * @param id
   *           id
   * @return usuario
   */
  public Usuario findPreToCyL(Long id) {
    UsuarioPre up = find(UsuarioPre.class, id);

    UsuarioCyL uCyL = new UsuarioCyL();

    uCyL.setNombre(up.getNombre());
    uCyL.setApellido1(up.getApellido1());
    uCyL.setApellido2(up.getApellido2());
    uCyL.setTelefono(up.getTelefono());
    uCyL.setMovil(up.getTelefono());
    uCyL.setCentro(up.getCentro());
    uCyL.setNif(up.getNif());
    uCyL.setProvincia(up.getProvincia());
    uCyL.setLocalidad(up.getLocalidad());
    uCyL.setDireccion(up.getDireccion());
    uCyL.setCp(up.getCp());
    
    uCyL.setFechaNacimiento(up.getFechaNacimiento());
    uCyL.setSexo(up.getSexo());
    uCyL.setTipoUsuario(up.getTipoUsuario());
    uCyL.setMedioPublicidad(up.getMedioPublicidad());
    uCyL.setMedioPublicidadOtro(up.getMedioPublicidadOtro());
    uCyL.setTipoFrecuenciaInternet(up.getTipoFrecuenciaInternet());
    uCyL.setTipoUtilidadInternet(up.getTipoUtilidadInternet());
    uCyL.setTipoUtilizarSwLibre(up.getTipoUtilizarSwLibre());
    uCyL.setTipoOcupacion(up.getTipoOcupacion());
    uCyL.setTipoColectivo(up.getTipoColectivo());
    uCyL.setTipoSectorProfesional(up.getTipoSectorProfesional());
    uCyL.setTipoEstudiosRealizados(up.getTipoEstudiosRealizados());
    uCyL.setTipoDiscapacidad(up.getTipoDiscapacidad());
    uCyL.setPoseeOrdenador(up.isPoseeOrdenador());
    uCyL.setAccesoInternet(up.isAccesoInternet());
    uCyL.setMenor(up.isMenor());
    uCyL.setExtranjero(up.isExtranjero());
    uCyL.setPasaporte(up.getPasaporte());
    uCyL.setNifAutorizado(up.getNifAutorizado());
    uCyL.setNombreCompletoAutorizado(up.getNombreCompletoAutorizado());
    uCyL.setMovil(up.getMovil());
    uCyL.setEmail(up.getEmail());
    uCyL.setcomunicacionTlf(up.isComunicacionTlf());
    uCyL.setcomunicacionMail(up.isComunicacionMail());
    uCyL.setcomunicacionSms(up.isComunicacionSms());
    uCyL.setAceptaPolitica(up.isAceptaPolitica());
    uCyL.setCesionPermitida(up.isCesionPermitida());

    /*for (MatriculaCurso m : up.getMatriculasCursos()) {
      m.setUsuario(uCyL);
    }
    uCyL.setMatriculasCursos(up.getMatriculasCursos());

    for (MatriculaTaller mt : up.getMatriculasTalleres()) {
      mt.setUsuario(uCyL);
    }
    uCyL.setMatriculasTalleres(up.getMatriculasTalleres());

    for (PreinscripcionCurso pc : up.getPreinscripcionesCursos()) {
      pc.setUsuario(uCyL);
    }
    uCyL.setPreinscripcionesCursos(up.getPreinscripcionesCursos());

    for (PreinscripcionTaller pt : up.getPreinscripcionesTalleres()) {
      pt.setUsuario(uCyL);
    }
    uCyL.setPreinscripcionesTalleres(up.getPreinscripcionesTalleres());*/

    return uCyL;
  }
  
  /**
   * Convert pre to cyl.
   * 
   * @param id
   *           id
   * @return usuario
   */
  private void convertPreToCyL(UsuarioPre up, UsuarioCyL ucyl) {
    ucyl.setNombre(up.getNombre());
    ucyl.setApellido1(up.getApellido1());
    ucyl.setApellido2(up.getApellido2());
    ucyl.setFechaNacimiento(up.getFechaNacimiento());
    ucyl.setSexo(up.getSexo());
    
    ucyl.setTipoUsuario(up.getTipoUsuario());
    ucyl.setMedioPublicidad(up.getMedioPublicidad());
    ucyl.setMedioPublicidadOtro(up.getMedioPublicidadOtro());
    ucyl.setTipoFrecuenciaInternet(up.getTipoFrecuenciaInternet());
    ucyl.setTipoUtilidadInternet(up.getTipoUtilidadInternet());
    ucyl.setTipoUtilizarSwLibre(up.getTipoUtilizarSwLibre());
    ucyl.setTipoOcupacion(up.getTipoOcupacion());
    ucyl.setTipoColectivo(up.getTipoColectivo());
    ucyl.setTipoSectorProfesional(up.getTipoSectorProfesional());
    ucyl.setTipoEstudiosRealizados(up.getTipoEstudiosRealizados());
    ucyl.setCentro(up.getCentro());
    ucyl.setTipoDiscapacidad(up.getTipoDiscapacidad());
    ucyl.setPoseeOrdenador(up.isPoseeOrdenador());
    ucyl.setAccesoInternet(up.isAccesoInternet());
   
    ucyl.setMenor(up.isMenor());
    ucyl.setExtranjero(up.isExtranjero());
    ucyl.setTipoIdentificacion(up.getTipoIdentificacion());
    ucyl.setNif(up.getNif());
    ucyl.setPasaporte(up.getPasaporte());
    ucyl.setNifAutorizado(up.getNifAutorizado());
    ucyl.setNombreCompletoAutorizado(up.getNombreCompletoAutorizado());
    
    ucyl.setDireccion(up.getDireccion());
    ucyl.setProvincia(up.getProvincia());
    ucyl.setLocalidad(up.getLocalidad());
    ucyl.setCp(up.getCp());
    ucyl.setTelefono(up.getTelefono());
    ucyl.setMovil(up.getMovil());
    ucyl.setEmail(up.getEmail());
    
    ucyl.setcomunicacionTlf(up.isComunicacionTlf());
    ucyl.setcomunicacionMail(up.isComunicacionMail());
    ucyl.setcomunicacionSms(up.isComunicacionSms());
    
    ucyl.setAceptaPolitica(up.isAceptaPolitica());
    ucyl.setCesionPermitida(up.isCesionPermitida());

    /*for (MatriculaCurso m : up.getMatriculasCursos()) {
      m.setUsuario(ucyl);
    }
    ucyl.setMatriculasCursos(up.getMatriculasCursos());

    for (MatriculaTaller mt : up.getMatriculasTalleres()) {
      mt.setUsuario(ucyl);
    }
    ucyl.setMatriculasTalleres(up.getMatriculasTalleres());

    for (PreinscripcionCurso pc : up.getPreinscripcionesCursos()) {
      pc.setUsuario(ucyl);
    }
    ucyl.setPreinscripcionesCursos(up.getPreinscripcionesCursos());

    for (PreinscripcionTaller pt : up.getPreinscripcionesTalleres()) {
      pt.setUsuario(ucyl);
    }
    ucyl.setPreinscripcionesTalleres(up.getPreinscripcionesTalleres());*/
  }

  /**
   * Persist usuario.
   * 
   * @param usuario
   *           usuario
   * @return true, if successful
   */
  @SuppressWarnings("unchecked")
  public void notificarSolicitudPreinscripcionUsuario(Usuario usuario) {
	  
	  NotificationService n = new NotificationService();
	  String mailActivate = "";
	  String mailPreuserregistrationActivate = "";
	  
	  
	  try {
		  List<Parameter> parameters1 = findWhere(Parameter.class, "nombre", "mail_activate");
		  if (parameters1.size() > 0)
			  mailActivate = parameters1.get(0).getValor();

		  List<Parameter> parameters2 = findWhere(Parameter.class, "nombre", "mail_preuserregistration_activate");
		  if (parameters2.size() > 0)
			  mailPreuserregistrationActivate = parameters2.get(0).getValor();

		  if (mailActivate.equals("1")){
			  if (mailPreuserregistrationActivate.equals("1"))
				  	n.notificarPreregistroUsuario(this, usuario);
					// No lo pongo ya que al no tener el usuario email no se tiene que enseñar el mensaje ni tampoco si da 
			  		// error al enviarlo a su agente correspondiente
			  		//addInfoMessage("mail.email");
				  	//else
					//addErrorMessage("mail.errormail");
			  
			  else{
				  // están desactivadas las notificaciones por email asociadas al preregistro de usuario en el sistema.
				  // se obvia la advertencia puesto que el destinatario siempre será un ciudadano a través de la web de inscripción.
				  //addWarnMessage("mail.preuserregistration.nonotifications");
			  }
		  }
		  else { 
			  // están desactivadas las notificaciones por email en el sistema.
			  // se obvia la advertencia puesto que el destinatario siempre será un ciudadano a través de la web de inscripción.
			  //addWarnMessage("mail.nonotifications");
		  }
	  } catch (Exception e){
		  e.printStackTrace();
	  }
  }
}
