package es.csa.fdt.cyldigital.usuarios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioPre;


/**
 * Clase UsuarioPreServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class UsuarioPreServiceImpl extends AbstractServiceImpl<UsuarioPre, Long> implements UsuarioPreService {

  private static final long serialVersionUID = -5575228180361917674L;

  private String resultadoOp;

  private UsuarioPre usuariopre;

  /**
   * Construye un nuevo objeto de la clase
   */
  public UsuarioPreServiceImpl() {
    super();
  }

  /**
   * Devuelve usuario.
   * 
   * @return usuario
   */
  public UsuarioPre getUsuario() {
    return usuariopre;
  }

  /**
   * Asigna usuario.
   * 
   * @param usuariopre
   *          usuario
   */
  public void setUsuario(UsuarioPre usuariopre) {
    this.usuariopre = usuariopre;
  }

  /**
   * Devuelve resultado op.
   * 
   * @return resultado op
   */
  public String getResultadoOp() {
    return resultadoOp;
  }

  /**
   * Asigna resultado op.
   * 
   * @param resultadoOp
   *          resultado op
   */
  public void setResultadoOp(String resultadoOp) {
    this.resultadoOp = resultadoOp;
  }

  /**
   * Elimina usuario pre.
   * 
   * @param usuariopre
   *           usuariopre
   */
  public void removeUsuarioPre(UsuarioPre usuariopre) {
    if (usuariopre.getId() != 0) {
      if (usuariopre.getMatriculasCursos().isEmpty() && usuariopre.getMatriculasTalleres().isEmpty()
          && usuariopre.getMatriculasCharlas().isEmpty() && usuariopre.getPreinscripcionesCursos().isEmpty()
          && usuariopre.getPreinscripcionesTalleres().isEmpty()) {
        remove(usuariopre);
      } else {
        enableMessages();
        addErrorMessage("remove.usuario.matriculado");
        disableMessages();
      }
    }
  }

  /**
   * Fill search data.
   * 
   * @param nombre
   *           nombre
   * @param apellido1
   *           apellido1
   * @param apellido2
   *           apellido2
   * @param telefono
   *           telefono
   * @param nif
   *           nif
   * @return usuario pre
   */
  public UsuarioPre fillSearchData(String nombre, String apellido1, String apellido2, String telefono, String nif) {
    UsuarioPre usuario = new UsuarioPre();
    usuario.setNombre(nombre);
    usuario.setApellido1(apellido1);
    usuario.setApellido2(apellido2);
    usuario.setTelefono(telefono);
    usuario.setNif(nif);
    return usuario;
  }
}
