package es.csa.fdt.cyldigital.usuarios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.File;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.regex.Pattern;

import javax.annotation.Resource;
import javax.persistence.NoResultException;
import javax.persistence.NonUniqueResultException;
import javax.persistence.Query;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import es.csa.commons.persistence.validator.NIFUtils;
import es.csa.commons.util.AditionalStringUtils;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.notificacion.NotificationService;
import es.csa.fdt.cyldigital.servicios.entity.Asesoramiento;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCharla;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaExterna;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaTaller;
import es.csa.fdt.cyldigital.servicios.entity.PracticaTaller;
import es.csa.fdt.cyldigital.servicios.entity.PreinscripcionCurso;
import es.csa.fdt.cyldigital.servicios.entity.PreinscripcionPropuesta;
import es.csa.fdt.cyldigital.servicios.entity.PreinscripcionTaller;
import es.csa.fdt.cyldigital.usuarios.entity.CodigoPostal;
import es.csa.fdt.cyldigital.usuarios.entity.ComentarioObservacion;
import es.csa.fdt.cyldigital.usuarios.entity.Fichero;
import es.csa.fdt.cyldigital.usuarios.entity.Localidad;
import es.csa.fdt.cyldigital.usuarios.entity.Provincia;
import es.csa.fdt.cyldigital.usuarios.entity.RegistroLdap;
import es.csa.fdt.cyldigital.usuarios.entity.RegistroSmartCard;
import es.csa.fdt.cyldigital.usuarios.entity.RegistroSmartCard.TipoRegistro;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioPre;
import es.csa.fdt.cyldigital.util.ImageUtils;


/**
 * Clase UsuarioServiceImpl.
 * 
 * @author csa
 */
@Service
@Repository
public class UsuarioServiceImpl extends AbstractServiceImpl<UsuarioCyL, Long> implements UsuarioService {

  private static final long serialVersionUID = -5575228180361917674L;

  // private static final Logger logger = LoggerFactory.getLogger(UsuarioServiceImpl.class);

  private List<Provincia> provincias;
  
  private String lastCP = "";

  private boolean usuarioDuplicado;

  private boolean usuarioNoDuplicado;

  private boolean usuarioPrereg;

  private boolean usuarioNoPrereg;

  private List<UsuarioPre> usuariosYaPrereg;

  private boolean refrescado = false;

  private boolean fnacimientoOk = false;

  private String resultadoOp;

  private String image;

  private UsuarioCyL usuarioCyL;
  
  private Usuario usuario2;

  private Fichero imagenBorrada;

  private boolean accionEjecutada;

  private RegistroSmartCard registroSmartcard;
  
  private String nombreLocalidad = "";
  
  private List<Localidad> listLocalidades = null;
  
  private Long idLocalidadSeleccionada = -1L;
  
  private String passwordNuevoUsuarioCyL;
  
  private Boolean userWithoutEmailAndDefaultPass;
  
  private Boolean avisoMenorNoIdentificado = false;
  
  private Boolean avisoUsuarioSinEmailMostrado = false;
  
  private Localidad dummyLocalidad;
  
  //3.4.8 - RF_15_33 Formulario alta usuario- Confirmación de email
  private String emailConfirmacion = "";
  
  //3.4.14 - Usuarios sen mail
  private String password1 = "";
  private String password2 = "";
  
  @Resource
  private transient HashMap<String, Object> generalProperties;

  // @Autowired
  // private transient org.springframework.orm.jpa.JpaTransactionManager txManager;

  @Autowired
  private LdapComponent ldap;

  private List<Centro> centros;
  
  /**
   * Construye un nuevo objeto de la clase
   */
  public UsuarioServiceImpl() {
    super();
    
    userWithoutEmailAndDefaultPass = false;
    
    dummyLocalidad = new Localidad();
    dummyLocalidad.setId(-1);
  }

  /**
   * Devuelve centros.
   * 
   * @return centros
   */
  public List<Centro> getCentros() {
    if (centros == null) centros = findAllOrder(Centro.class, "nombre", true);
    return centros;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.UsuarioService#getProvincias()
   */
  public List<Provincia> getProvincias() {
    if (provincias == null)
    	provincias = findAllOrder(Provincia.class, "nombre", true);

    return provincias;
  }

  /**
   * Checks if is usuario no duplicado.
   * 
   * @return true, if is usuario no duplicado
   */
  public boolean isUsuarioNoDuplicado() {
    return usuarioNoDuplicado;
  }

  /**
   * Asigna usuario no duplicado.
   * 
   * @param usuarioNoDuplicado
   *          usuario no duplicado
   */
  public void setUsuarioNoDuplicado(boolean usuarioNoDuplicado) {
    this.usuarioNoDuplicado = usuarioNoDuplicado;
  }

  /**
   * Checks if is usuario duplicado.
   * 
   * @return true, if is usuario duplicado
   */
  public boolean isUsuarioDuplicado() {
    return usuarioDuplicado;
  }

  /**
   * Devuelve image.
   * 
   * @return image
   */
  public String getImage() {
    return image;
  }

  /**
   * Asigna image.
   * 
   * @param image
   *          image
   */
  public void setImage(String image) {
    this.image = image;
  }

  /**
   * Devuelve usuario.
   * 
   * @return usuario
   */
  public UsuarioCyL getUsuario() {
    return usuarioCyL;
  }

  /**
   * Asigna usuario.
   * 
   * @param usuarioCyL
   *          usuario
   */
  public void setUsuario(UsuarioCyL usuarioCyL) {
    this.usuarioCyL = usuarioCyL;
  }
 
  
  /**
   * Devuelve usuario2.
   * 
   * @return usuario2
   */
  public Usuario getUsuario2() {
    return usuario2;
  }

  /**
   * Asigna usuario2.
   * 
   * @param usuario
   *          usuario2
   */
  public void setUsuario2(Usuario usuario2) {
    this.usuario2 = usuario2;
  }

  /**
   * Devuelve resultado op.
   * 
   * @return resultado op
   */
  public String getResultadoOp() {
    return resultadoOp;
  }

  /**
   * Asigna resultado op.
   * 
   * @param resultadoOp
   *          resultado op
   */
  public void setResultadoOp(String resultadoOp) {
    this.resultadoOp = resultadoOp;
  }

  /**
   * Checks if is refrescado.
   * 
   * @return true, if is refrescado
   */
  public boolean isRefrescado() {
    return refrescado;
  }

  /**
   * Asigna refrescado.
   * 
   * @param refrescado
   *          refrescado
   */
  public void setRefrescado(boolean refrescado) {
    this.refrescado = refrescado;
  }

  /**
   * Checks if is fnacimiento ok.
   * 
   * @return true, if is fnacimiento ok
   */
  public boolean isFnacimientoOk() {
    return fnacimientoOk;
  }

  /**
   * Asigna fnacimiento ok.
   * 
   * @param fnacimientoOk
   *          fnacimiento ok
   */
  public void setFnacimientoOk(boolean fnacimientoOk) {
    this.fnacimientoOk = fnacimientoOk;
  }

  /**
   * Checks if is accion ejecutada.
   * 
   * @return true, if is accion ejecutada
   */
  public boolean isAccionEjecutada() {
    return accionEjecutada;
  }

  /**
   * Asigna accion ejecutada.
   * 
   * @param accionEjecutada
   *          accion ejecutada
   */
  public void setAccionEjecutada(boolean accionEjecutada) {
    this.accionEjecutada = accionEjecutada;
  }

  /**
   * Devuelve usuarios ya prereg.
   * 
   * @return usuarios ya prereg
   */
  public List<UsuarioPre> getUsuariosYaPrereg() {
    return usuariosYaPrereg;
  }

  /**
   * Asigna usuarios ya prereg.
   * 
   * @param usuariosYaPrereg
   *          usuarios ya prereg
   */
  public void setUsuariosYaPrereg(List<UsuarioPre> usuariosYaPrereg) {
    this.usuariosYaPrereg = usuariosYaPrereg;
  }

  /**
   * Checks if is usuario prereg.
   * 
   * @return true, if is usuario prereg
   */
  public boolean isUsuarioPrereg() {
    return usuarioPrereg;
  }

  /**
   * Asigna usuario prereg.
   * 
   * @param usuarioPrereg
   *          usuario prereg
   */
  public void setUsuarioPrereg(boolean usuarioPrereg) {
    this.usuarioPrereg = usuarioPrereg;
  }

  /**
   * Checks if is usuario no prereg.
   * 
   * @return true, if is usuario no prereg
   */
  public boolean isUsuarioNoPrereg() {
    return usuarioNoPrereg;
  }

  /**
   * Asigna usuario no prereg.
   * 
   * @param usuarioNoPrereg
   *          usuario no prereg
   */
  public void setUsuarioNoPrereg(boolean usuarioNoPrereg) {
    this.usuarioNoPrereg = usuarioNoPrereg;
  }
  
  public String getPassword1() {
	return password1;
  }
	
  public void setPassword1(String password1) {
	this.password1 = password1;
  }
	
  public String getPassword2() {
	return password2;
  }
	
  public void setPassword2(String password2) {
	this.password2 = password2;
  }

/* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.UsuarioService#persistUsuario(es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL, java.lang.String, java.lang.String, java.lang.Long, boolean)
   */
  @Transactional(readOnly = false)
  public boolean persistUsuario(UsuarioCyL usuarioCyL, String oldUsername, String oldNif, Long id, boolean prereg) {
	  boolean creacion = usuarioCyL.getId() == 0;
	  boolean result = false;
	  
	  //3.4.3-R RF_XEA_04 --> Comprobar que cuando se edita un usuario el campo localidad no es vacio 
	  if (usuarioCyL.getLocalidad().getId() == -1L) {
		  addErrorMessage("usuario.error.localidad");
		  return false;
	  }
	  
	  if(!userWithoutEmailAndDefaultPass) {
		  if(!checkEmailConfirmacion(usuarioCyL.getEmail())) {
			  addErrorMessage("usuario.error.emailConfirmacion");
			  return false;
		  }
  	  }
	  
	  if(userWithoutEmailAndDefaultPass) {
		  password1 = password1.trim();
		  password2 = password2.trim();
		  
		  if (password1.compareTo(password2) != 0) { 
			  addErrorMessage("password.change.passwordNoMatch");
			  return false;
		  } else if ((password1.length() < 8) || (password1.length() > 14) || password1.contains(" ")) {
			  addErrorMessage("password.change.passwordLength");				  
			  return false;
		  } else if (!passwordPattern(password1)) {
			  addErrorMessage("password.change.caracteresEspeciales");
			  return false;
		  }
	  }
	  
	  
	  // 3.4.3-R RF_XEA_19 
	  //Transformar el nombre y apellidos
	  usuarioCyL.setNombre(AditionalStringUtils.formatCapitalized(usuarioCyL.getNombre()));	  
	  usuarioCyL.setApellido1(AditionalStringUtils.formatCapitalized(usuarioCyL.getApellido1()));	  
	  if ((usuarioCyL.getApellido2() != null) && !usuarioCyL.getApellido2().isEmpty())
		  usuarioCyL.setApellido2(AditionalStringUtils.formatCapitalized(usuarioCyL.getApellido2()));
	  
	  if ((usuarioCyL.getNombreCompletoAutorizado() != null) && !usuarioCyL.getNombreCompletoAutorizado().isEmpty())
		  usuarioCyL.setNombreCompletoAutorizado(AditionalStringUtils.formatCapitalized(usuarioCyL.getNombreCompletoAutorizado().toLowerCase()));
	  
	  UsuarioPre uPre = find(UsuarioPre.class, id);
    
	  if (compruebaDNI(usuarioCyL, prereg, uPre) && compruebaCP(usuarioCyL) && compruebaFNacimiento(usuarioCyL)) {
		  //Al poner disabled el campo de email en el caso de no quererlo y querer password por defecto, el atributo
		  //llega con NULL y es preferible que lleve cadena vacía para evitar errores.
		  //Por tanto se aplica la siguiente condición:
		  //Si el email es null lo establecemos a ""
		  if (usuarioCyL.getEmail() == null) usuarioCyL.setEmail("");
		  
		  //Si es nuevo y se ha decidido que no lleve email o por el contrario no es nuevo, entonces hay que comprar el email
		  if (((creacion && !userWithoutEmailAndDefaultPass) || !creacion)) {
			  //El email solo se comprueba si no es blanco, porque puede haber más emails en blanco.
			  //Si el email va vacio compruebaEmail siempre se da por bueno y se devuelve true
			  if (!compruebaEmail(usuarioCyL)) {
				  if (usuarioCyL.getEdad() >= 14) {
					  addErrorMessage("usuarios.alta.errorEmailDucplicado");
					  
					  avisoMenorNoIdentificado = false;
					  
					  return false;
				  }
				  else { //if (usuarioCyL.getEdad() < 14)
					  if (!avisoMenorNoIdentificado) {
						  addWarnMessage("usuarios.alta.avisoEmailDuplicadoMenor");
						  
						  avisoMenorNoIdentificado = true;
						  
						  return false;
					  }
				  }
			  }
		  }
		  
		  usuarioCyL.setUsername(getUserName(usuarioCyL, oldNif));

		  if (usuarioCyL.getId() == 0)
			  usuarioCyL.setFechaCreacion(new Date());
		  	  
		  if (creacion && !userWithoutEmailAndDefaultPass)
			  usuarioCyL.setPassword(generatePassword()); 
		  else if(creacion && userWithoutEmailAndDefaultPass)
			  usuarioCyL.setPassword(password1);
		  		  

		  if (imagenBorrada != null && imagenBorrada.getId() != 0) getEntityManager().remove(imagenBorrada);
		  
		  setMessagesEnabled(false);

		  //Si venía de un usuario ya preregistrado se borra
		  if (prereg == true) {
			  Usuario usu = find(Usuario.class, id);

			  if ((uPre != null) && (usu != null) && (uPre.getId() == usu.getId())) {    	
				  usuarioCyL.getCentro().getUsuarios().remove(uPre);

				  /* 
				   * Se comenta porque actualmente no se puede preinscribir ni matricular en actividades
				   * a un usuario preinscrito en la red
				   * 
				   * uPre.setMatriculasCursos(new ArrayList<MatriculaCurso>());
				   * uPre.setMatriculasTalleres(new ArrayList<MatriculaTaller>());
				   * uPre.setPreinscripcionesCursos(new ArrayList<PreinscripcionCurso>());
				   * uPre.setPreinscripcionesTalleres(new ArrayList<PreinscripcionTaller>());
				   */

				  remove(uPre);
			  }
		  }
		  
		  //Nome e apelidos no formulario de alta sen espazos
		  if(usuarioCyL.getNombre() != null) usuarioCyL.setNombre(usuarioCyL.getNombre().trim());
		  if(usuarioCyL.getApellido1() != null) usuarioCyL.setApellido1(usuarioCyL.getApellido1().trim());
		  if(usuarioCyL.getApellido2() != null)usuarioCyL.setApellido2(usuarioCyL.getApellido2().trim());
		  if(usuarioCyL.getDireccion() != null)usuarioCyL.setDireccion(usuarioCyL.getDireccion().trim());
		  if(usuarioCyL.getNif() != null) usuarioCyL.setNif(usuarioCyL.getNif().trim());
		  if(usuarioCyL.getNifAutorizado() != null) usuarioCyL.setNifAutorizado(usuarioCyL.getNifAutorizado().trim());
		  if(usuarioCyL.getPasaporte() != null) usuarioCyL.setPasaporte(usuarioCyL.getPasaporte().trim());
		  if(usuarioCyL.getTelefono() != null) usuarioCyL.setTelefono(usuarioCyL.getTelefono().trim());
		  if(usuarioCyL.getUsername() != null) usuarioCyL.setUsername(usuarioCyL.getUsername().trim());
		  if(usuarioCyL.getCp() != null) usuarioCyL.setCp(usuarioCyL.getCp().trim());
		  
		  persist(usuarioCyL);

		  if (creacion) {
			  passwordNuevoUsuarioCyL = usuarioCyL.getPassword();
			  
			  ldap.createUser(usuarioCyL, this);
			  
			  persist(usuarioCyL);
		  } else {
			  ldap.actualizarInformacionLDAP(usuarioCyL, oldUsername, this);
			  
			  persist(usuarioCyL);
		  }

		  setMessagesEnabled(true);
		  result = true;   
	  }
	  
    return result;     
  }
  
  /**
   * Comprueba que el Email no este duplicado
   * 
   */
  private Boolean compruebaEmail(UsuarioCyL ucylToPersist) {
	  Boolean result = true;
	  
	  if ((ucylToPersist.getEmail() != null) && (!ucylToPersist.getEmail().isEmpty())) {
		  List<UsuarioCyL> lucyl = findWhere("email", ucylToPersist.getEmail());
		  List<UsuarioPre> lupre = findWhere(UsuarioPre.class, "email", ucylToPersist.getEmail());
		  
		  if (lucyl != null) {
			  for (UsuarioCyL ucyl : lucyl) {
				  if (ucyl.getId() != ucylToPersist.getId())
					  result = false;
			  }
		  }
	
		  if (lupre != null) {
			  for (UsuarioPre upre : lupre) {
				  if (upre.getId() != ucylToPersist.getId())
					  result = false;
			  }
		  }
	  }

	  return result;
  }

  /**
   * Elimina usuario.
   * 
   * @param usuarioCyL
   *           usuario cy l
   * @return true, if successful
   */
  public boolean removeUsuario(UsuarioCyL usuarioCyL) {
	  if (usuarioCyL.getId() != 0) {
	      String error = "";
	      Usuario u = find(Usuario.class, usuarioCyL.getId());
	      List<Asesoramiento> asesoramientos = findWhere(Asesoramiento.class, "usuario.id", u.getId());
	      List<PracticaTaller> practicas = findWhere(PracticaTaller.class, "usuario.id", u.getId());
	      List<PreinscripcionPropuesta> propuestas = findWhere(PreinscripcionPropuesta.class, "usuario.id", u.getId());

	      for (MatriculaCurso m : u.getMatriculasCursos()) {
	        if (m.getCurso().getFechaInicio().compareTo(new Date()) > 0) error = "curso";
	      }

	      for (MatriculaCharla m : u.getMatriculasCharlas()) {
	        if (m.getCharla().getFechaInicio().compareTo(new Date()) > 0) error = "charla";
	      }

	      for (MatriculaTaller m : u.getMatriculasTalleres()) {
	        if (m.getTaller().getFechaInicio().compareTo(new Date()) > 0) error = "taller";
	      }

	      for (Asesoramiento m : asesoramientos) {
	        if (m.getFecha().compareTo(new Date()) > 0) error = "asesoramiento";
	      }

	      for (PracticaTaller m : practicas) {
	        if (m.getFecha().compareTo(new Date()) > 0) error = "practica";
	      }

	      for (MatriculaExterna m : u.getMatriculasExternas()) {
	        if (m.getExterna().getFechaInicio().compareTo(new Date()) > 0) error = "externa";
	      }

	      if (error != "") {
	        enableMessages();
	        addErrorMessage("remove.usuario." + error);
	        disableMessages();
	        return false;
	      } else {
	        for (PreinscripcionCurso m : u.getPreinscripcionesCursos()) {
	          m.setDesactivado(true);
	          // actualizamos serviciocurso.numeropreinscripciones.
	          m.getServicioCurso().setNumeroPreinscripciones(m.getServicioCurso().getNumeroPreinscripciones());
	        }
	        for (PreinscripcionTaller m : u.getPreinscripcionesTalleres()) {
	          m.setDesactivado(true);
	          m.getServicioTaller().setNumeroPreinscripciones(m.getServicioTaller().getNumeroPreinscripciones());
	        }

	        for (PreinscripcionPropuesta m : propuestas) {
	          m.setDesactivado(true);
	          m.getPropuestaServicio().setNumeroPreinscripciones(m.getPropuestaServicio().getNumeroPreinscripciones());
	        }

	        u.setDesactivado(true);
	        ldap.deleteUser(usuarioCyL, this);

	        DateFormat df = new SimpleDateFormat("yyyyMMdd_HHmmss");
	        
	        removePhoto(usuarioCyL);

	        // hacer esto después de ldap.delteUSer porque ahi se basa en los datos del us.
	        usuarioCyL.setcomunicacionMail(false);
	        usuarioCyL.setcomunicacionSms(false);
	        usuarioCyL.setcomunicacionTlf(false);
	        usuarioCyL.setNombre("Usuario");
	        usuarioCyL.setApellido1("de baja");
	        usuarioCyL.setApellido2("");
	        usuarioCyL.setDireccion("");
	        usuarioCyL.setAccesoInternet(false);
	        usuarioCyL.setAceptaPolitica(false);
	        usuarioCyL.setCesionPermitida(false);
	        usuarioCyL.setComentarioObservaciones(null);
	        usuarioCyL.setCp("");
	        usuarioCyL.setDireccion("");
	        usuarioCyL.setTelefono("");
	        usuarioCyL.setEmail("");
	        usuarioCyL.setMovil("");
	        usuarioCyL.setNif("");
	        usuarioCyL.setNifAutorizado("");
	        usuarioCyL.setPasaporte("");
	        usuarioCyL.setNombreCompletoAutorizado("");
	        usuarioCyL.getRolesList().clear();
	        usuarioCyL.setUsername("baja_" + df.format(new Date()).toString());	        
	        persist(u);

	        enableMessages();
	        addInfoMessage("remove.usuario.ok");
	        disableMessages();

	        return true;
	      }
	    }
	    addErrorMessage("remove.usuario.nousuarioseleccionado");

	    return false;
  }
  
  
  public boolean desactivarUsuarioLDAP(UsuarioCyL usuarioCyL){
      if (usuarioCyL.getId() != 0) {
	  ldap.deactivateUser(usuarioCyL, this);
	  persist(usuarioCyL);
	  if (usuarioCyL.isDesactivadoLDAP()){
	      addInfoMessage("deactivate.usuario.ok");
	  	return true;
	  }
	  else{
	      addErrorMessage("deactivate.usuario.error");
	      return false;
	  }
      }
      addErrorMessage("deactivate.usuario.nousuarioseleccionado");
      return false;
  }
  
  public boolean desactivarUsuarioLDAPTarea(UsuarioCyL usuarioCyL){
      if (usuarioCyL.getId() != 0) {
	  ldap.deactivateUser(usuarioCyL, this);
	  if (usuarioCyL.isDesactivadoLDAP()){
	      addInfoMessage("deactivate.usuario.ok");
	  	return true;
	  }
	  else{
	      addErrorMessage("deactivate.usuario.error");
	      return false;
	  }
      }
      addErrorMessage("deactivate.usuario.nousuarioseleccionado");
      return false;
  }
  
  
  public boolean reactivarUsuarioLDAP(UsuarioCyL usuarioCyL){
	  if (usuarioCyL != null){
		  if (usuarioCyL.getId() != 0) {
			  if(!ldap.checkUserActive(usuarioCyL, this)){
				  if(!usuarioCyL.isDesactivadoLDAP()){
					  //Ya está activado, todo correcto.
					  return true;
				  }
				  ldap.reactivateUser(usuarioCyL, this);
			  }
			  else{
				  usuarioCyL.setDesactivadoLDAP(false);
			  }
			  persist(usuarioCyL);
			  if (!usuarioCyL.isDesactivadoLDAP()){
			      addInfoMessage("reactivate.usuario.ok");
			      return true;
			  }
			  else{
				  addErrorMessage("reactivate.usuario.error");
				  return false;
			  }
		  }
		  else{
			  addErrorMessage("reactivate.usuario.nousuarioseleccionado");
			  return false;
		  }
	  }
	  else{
		  return false;
	  }
  }
  
  
  /**
   * Persist usuario image.
   * 
   * @param usuarioCyL
   *           usuario cy l
   */
  public void persistUsuarioImage(UsuarioCyL usuarioCyL) {
    if (usuarioCyL.getId() != 0) {
      String nombre = getfileName(usuarioCyL);
      File f = new File(getFilePath("USER_PHOTO_PATH_TMP") + nombre);
      File dest = new File(getFilePath("USER_PHOTO_PATH") + nombre);
      File destThumb = new File(getFilePath("USER_PHOTO_PATH_THUMB") + nombre);

      if (f.exists()) {
        try {
          ImageUtils.resize(f, destThumb, 72, 1f);// Height
          f.renameTo(dest);
          if (usuarioCyL.getFoto() == null) {
            Fichero fichero = new Fichero();
            fichero.setNombre(nombre);
            fichero.setTamanio((int) f.length());
            fichero.setTipoMime("image/jpeg");
            usuarioCyL.setFoto(fichero);

            getEntityManager().persist(usuarioCyL);
          }
        } catch (IOException e) {
          e.printStackTrace();
        }
      }
    }
  }

  /**
   * Elimina photo.
   * 
   * @param usuarioCyL
   *           usuario cy l
   */
  public void removePhoto(UsuarioCyL usuarioCyL) {
    if (usuarioCyL.getFoto() != null) {
      String nombreFoto = usuarioCyL.getFoto().getNombre();

      setMessagesEnabled(false);
      removeAll(Fichero.class, usuarioCyL.getFoto());
      setMessagesEnabled(true);
      usuarioCyL.setFoto(null);
      persist(usuarioCyL);

      ImageUtils.removeImageFilesystem(nombreFoto, getFilePath("USER_PHOTO_PATH"));
      ImageUtils.removeImageFilesystem(nombreFoto, getFilePath("USER_PHOTO_PATH_THUMB"));
    }
  }

  /**
   * Actualiza centro.
   * 
   * @param usuario
   *           usuario
   * @param currentUser
   *           current user
   */
  public void actualizaCentro(UsuarioCyL usuario, UsuarioLdap currentUser) {
    if (currentUser.getCentro() != null) {
      Centro c = find(Centro.class, currentUser.getCentro().getId());
      usuario.setCentro(c);// no añadir el de currentuser porque no estará completo.
    } else if (generalProperties.containsKey("DEFAULT_CENTER_ID") && usuario.getCentro() == null) {
      Centro c = find(Centro.class, Long.valueOf((String) generalProperties.get("DEFAULT_CENTER_ID")));
      if (c != null) usuario.setCentro(c);
    }
  }

  /**
   * Registra impresion politica.
   * 
   * @param user
   *           user
   */
  public void registraImpresionPolitica(UsuarioCyL user) {
  // TODO Crear la tabla que permita guardar la impresion de la politica de privacidad
  }

  /**
   * Registra grabacion tarjeta.
   */
  public void registraGrabacionTarjeta() {
    registraModificacionTarjeta(TipoRegistro.G);
  }

  /**
   * Registra impresion tarjeta.
   */
  public void registraImpresionTarjeta() {
    setResultadoOp("okOperacionCorrecta");
    registraModificacionTarjeta(TipoRegistro.I);
    addInfoMessage("Trabajo enviado a la impresora");
  }

  private void registraModificacionTarjeta(TipoRegistro tipoReg) {
    registroSmartcard = new RegistroSmartCard();
    registroSmartcard.setFecha(new Date());
    registroSmartcard.setTipoRegistro(tipoReg);
    if (getResultadoOp() != null && !getResultadoOp().isEmpty() && getResultadoOp().length() > 2) registroSmartcard
        .setCorrecto(getResultadoOp().substring(0, 2).equals("ok"));
    else registroSmartcard.setCorrecto(false);
  }

  /**
   * Persist smart card change.
   * 
   * @param user
   *           user
   */
  public void persistSmartCardChange(UsuarioCyL user) {
    if (registroSmartcard != null) {
      registroSmartcard.setUsuario(user);
      user.getRegistrosSmartCard().add(registroSmartcard);
    }
    persist(user);
  }

  private String getFilePath(String key) {
    return generalProperties.get("FILE_PATH") + "" + generalProperties.get(key);
  }

  /**
   * Crea temp image.
   * 
   * @return string
   */
  public String createTempImage() {
    String nombre = getfileName(usuarioCyL);
    ImageUtils.createImageFilesystem(nombre, getFilePath("USER_PHOTO_PATH_TMP"), ImageUtils.getImage(180, 240,
        getImage()));
    return null;
  }

  private String getUserName(UsuarioCyL usuarioCyL, String oldNif) {
	  String noNifPrefix = ((String) generalProperties.get("LDAP_USERNAME_NO_NIF_PREFIX"));
	  
	  //Si no tiene username
	  if ((usuarioCyL.getUsername() == null) || usuarioCyL.getUsername().isEmpty()) {
		  if (usuarioCyL.getNif() != null && !usuarioCyL.getNif().isEmpty()) {
			  //Generar username a partir del NIF
			  String letras = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
		      String username = "";

		      // Si es un usuario nuevo
		      if (usuarioCyL.getId() == 0) {
		    	  int i = 0;
		    	  username = usuarioCyL.getNif();
		    	  while (!buscarUsuariosUserName(username, usuarioCyL.getId(), false).isEmpty()) {
		    		  username = usuarioCyL.getNif() + "_" + letras.substring(i, i + 1);
		    		  i++;
		    	  }
		      }
		      //Si no es un usuario nuevo
		      else {
		    	  //Si el nif ha cambiado
		    	  if (!oldNif.equals(usuarioCyL.getNif())) {
		    		  int i = 0;
		    		  username = usuarioCyL.getNif();
		    		  while (!buscarUsuariosUserName(username, usuarioCyL.getId(), false).isEmpty()) {
		    			  username = usuarioCyL.getNif() + "_" + letras.substring(i, i + 1);
		    			  i++;
		    		  }
		    	  }
		    	  //Si el nif es el mismo
		    	  else
		    		  username = usuarioCyL.getUsername();
		      }
		      return username;
		  }
		  else {
			  //Generar el username del tipo XG....
			  
			  String singleResult = null;

		      Query query = getEntityManager().createQuery("SELECT MAX(u.username)" +
		    		  									   "FROM " + UsuarioCyL.class.getName() + " u " +
		    		  									   "WHERE u.username LIKE " +
		    		  									   "'" + ((String) generalProperties.get("LDAP_USERNAME_NO_NIF_PREFIX")) + "%%'");

		      singleResult = (String) query.getSingleResult();

		      if (singleResult == null || singleResult.length() != (noNifPrefix.length() + 7)) {
		    	  return noNifPrefix + stringFromInteger(7, 1);
		      }
		      else {
		    	  System.out.println("noNifPrefix " + noNifPrefix);
		    	  System.out.println("singleResult " + singleResult);
				
		    	  int max = Integer.parseInt(singleResult.substring(noNifPrefix.length()));
		    	  max += 1;

		    	  return noNifPrefix + stringFromInteger(7, max);
		      }
		  }
	  }
	  //Si tiene username y el username no comienza ni por "LDAP_USERNAME_NO_NIF_PREFIX" ni por UE (UE es caso espcial para
	  //usuarios migrados de Xunta de Galicia).
	  else if (((usuarioCyL.getUsername() != null) && !usuarioCyL.getUsername().isEmpty()) &&
			   !usuarioCyL.getUsername().toUpperCase().startsWith(noNifPrefix) &&
			   !usuarioCyL.getUsername().toUpperCase().startsWith("UE")) {
		  
		  //Si tiene NIF (susceptible de cambiar el NIF, porque se puso por error o parecido).
		  if (usuarioCyL.getNif() != null && !usuarioCyL.getNif().isEmpty()) {
			//Generar username a partir del NIF
			  String letras = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
		      String username = "";

		      //Si es un usuario nuevo
		      if (usuarioCyL.getId() == 0) {
		    	  int i = 0;
		    	  username = usuarioCyL.getNif();
		    	  while (!buscarUsuariosUserName(username, usuarioCyL.getId(), false).isEmpty()) {
		    		  username = usuarioCyL.getNif() + "_" + letras.substring(i, i + 1);
		    		  i++;
		    	  }
		      }
		      //Si no es un usuario nuevo
		      else {
		    	  //Si el nif ha cambiado
		    	  if (!oldNif.equals(usuarioCyL.getNif())) {
		    		  int i = 0;
		    		  username = usuarioCyL.getNif();
		    		  while (!buscarUsuariosUserName(username, usuarioCyL.getId(), false).isEmpty()) {
		    			  username = usuarioCyL.getNif() + "_" + letras.substring(i, i + 1);
		    			  i++;
		    		  }
		    	  }
		    	  else
		    		  username = usuarioCyL.getUsername();
		      }
		      
		      return username;
		  }
		  //Si no tiene NIF (Este caso no debería de darse nunca, a excepción de convertir un usuario dado de alta con NIF a menor
		  //o extranjero ambos sin NIF).
		  else {
			  //Generar el username del tipo XG....
			  
			  String singleResult = null;

		      Query query = getEntityManager().createQuery("SELECT MAX(u.username)" +
		    		  									   "FROM " + UsuarioCyL.class.getName() + " u " +
		    		  									   "WHERE u.username LIKE " +
		    		  									   "'" + ((String) generalProperties.get("LDAP_USERNAME_NO_NIF_PREFIX")) + "%%'");

		      singleResult = (String) query.getSingleResult();

		      if (singleResult == null || singleResult.length() != (noNifPrefix.length() + 7)) {
		    	  return noNifPrefix + stringFromInteger(7, 1);
		      }
		      else {
		    	  System.out.println("noNifPrefix " + noNifPrefix);
		    	  System.out.println("singleResult " + singleResult);
				
		    	  int max = Integer.parseInt(singleResult.substring(noNifPrefix.length()));
		    	  max += 1;

		    	  return noNifPrefix + stringFromInteger(7, max);
		      }
		  }
	  }
	  //Si tiene username y el username contiene XG o UE.
	  else
		  return usuarioCyL.getUsername();
  }

  private String stringFromInteger(int size, Integer number) {
    String result = number.toString();
    for (int i = 0; result.length() < size; i++) {
      result = "0" + result;
    }
    return result;
  }

  /**
   * Actualizar informacion ldap.
   * 
   * @param user
   *           user
   */
  public void actualizarInformacionLDAP(UsuarioCyL user) {
    ldap.actualizarInformacionLDAP(user, this);
    persist(user);
  }

  /**
   * Import pre to cy l.
   * 
   * @param uCyL
   *           u cy l
   * @param id
   *           id
   * @return long
   */
  public Long importPreToCyL(UsuarioCyL uCyL, Long id) {
    UsuarioPre u = find(UsuarioPre.class, id);

    uCyL.setNombre(u.getNombre().trim());
    uCyL.setApellido1(u.getApellido1().trim());
    uCyL.setApellido2(u.getApellido2().trim());
    uCyL.setTelefono(u.getTelefono().trim());
    uCyL.setMovil(u.getTelefono().trim());
    uCyL.setCentro(u.getCentro());
    uCyL.setProvincia(u.getProvincia());
    uCyL.setCp(u.getCp().trim());
    uCyL.setLocalidad(u.getLocalidad());

    for (MatriculaCurso m : u.getMatriculasCursos()) {
      m.setUsuario(uCyL);
    }
    uCyL.setMatriculasCursos(u.getMatriculasCursos());
    for (MatriculaTaller mt : u.getMatriculasTalleres()) {
      mt.setUsuario(uCyL);
    }
    uCyL.setMatriculasTalleres(u.getMatriculasTalleres());
    for (PreinscripcionCurso pc : u.getPreinscripcionesCursos()) {
      pc.setUsuario(uCyL);
    }
    uCyL.setPreinscripcionesCursos(u.getPreinscripcionesCursos());

    usuarioPrereg = false;
    usuariosYaPrereg = new ArrayList<UsuarioPre>();

    return id;
  }

  // ------------------------------------------------------------------------------------Comprobaciones Pre Persistencia
  private boolean compruebaDNI(UsuarioCyL usuarioCyL, boolean prereg, UsuarioPre uPre) {
    if (usuarioCyL.isExtranjero() && !usuarioCyL.isMenor()) {
    	if ((usuarioCyL.getPasaporte() == null) || usuarioCyL.getPasaporte().isEmpty()) {
    		addErrorMessage("Los usuarios extranjeros sin N.I.F. deben introducir el pasaporte");
    		return false;
    	}
    	else {
	      String mensaje = "";
	      List<UsuarioCyL> extranjeros = findWhere(UsuarioCyL.class, "pasaporte", usuarioCyL.getPasaporte().trim());
	      if (extranjeros.size() > 0) {
	        for (UsuarioCyL u : extranjeros) {
	          if (usuarioCyL == u) {
	            if (u.getDesactivado())
	              mensaje += ". Este usuario está dado de baja, puede darlo de alta en Usuarios > Usuarios de baja > Ver detalles > Re alta";
	          } else {
	            mensaje = "El pasaporte está siendo usado por:" + u.getNombreCompleto();
	          }
	        }
	        if (!mensaje.equals("")) {
	          addErrorMessage(mensaje);
	          return false;
	        }
	      }

	      return true;
    	}
    }
    else {
      if (usuarioCyL.isMenor()) {

        usuarioCyL.setNif("");
        
        if ((usuarioCyL.getNifAutorizado() != null) && !usuarioCyL.getNifAutorizado().trim().isEmpty()) {
          // comprobación básica de q no esté repetido el usuario. El nif autorizador puede estarlo.
          List<UsuarioCyL> usuariosigualnifautorizado = findWhere(UsuarioCyL.class, "nifAutorizado", usuarioCyL.getNifAutorizado());
          String nombre2 = usuarioCyL.getNombreCompleto().trim().toLowerCase().replace(" ", "");

          for (UsuarioCyL u : usuariosigualnifautorizado) {
            String nombre1 = u.getNombreCompleto().trim().toLowerCase().replace(" ", "");

            if (nombre1.compareTo(nombre2) == 0 &&
            	u.getFechaNacimiento().compareTo(usuarioCyL.getFechaNacimiento()) == 0 &&
            	u.getId() != usuarioCyL.getId()) {
              String mensaje = "Actualmente existe el usuario:" + u.getNombreCompleto() + " nacido el "
                  			   + u.getFechaNacimiento() + " y autorizado por:" + u.getNombreCompletoAutorizado();

              if (u.getDesactivado())
                mensaje += ". \n\nEste usuario está dado de baja, puede darlo de alta en Usuarios > Usuarios de baja > Ver detalles > Re alta";

              addErrorMessage(mensaje);
              return false;
            }
          }
          
          return true;
        }
        /*else {
          addErrorMessage("El usuario es menor y debe introudcir el N.I.F. de su padre o tutor que lo atorice.");
          
          return false;
        }*/
      }
      
      // Llegado este punto no es menor ni extranjero
      
      if (usuarioCyL.getNif() != null)
    	  usuarioCyL.setNif(usuarioCyL.getNif().toUpperCase());
      
      if (NIFUtils.isValid(usuarioCyL.getNif())) {
    	  List<Usuario> users;
    	  
    	  if (prereg)
    		  users = buscarUsuariosNIF(usuarioCyL.getNif(), uPre.getId(), false);
    	  else
    		  users = buscarUsuariosNIF(usuarioCyL.getNif(), usuarioCyL.getId(), false);
    	  
    	  if (users.isEmpty()) {
    		  return true;
    	  } else {
    		  if (isUsuarioNoDuplicado()) {
    			  return true;
    		  }
    		  else {
	            usuarioDuplicado = true;
	            String mensaje = "El N.I.F. esta siendo utilizado por:";
	            
	            for (Usuario u : users) {
	            	mensaje += " \"" + u.getNombreCompleto() + "\"";
	            	if (u.getDesactivado())
	            		mensaje += ".\n\n Este usuario está dado de baja, puede darlo de alta en 'Usuarios > Usuarios de baja'.";
	            }
	            
	            addErrorMessage(mensaje);
	            
	            return false;
    		  }
    	  }
      }
      else {
    	  addErrorMessage("usuario.error.dni");
    	  
    	  return false;
      }
    }
  }

  private List<Usuario> buscarUsuariosNIF(String nif, long id, boolean includeOwn) {
    List<Usuario> usuarioCyLs = findWhere(Usuario.class, "nif", nif);
    List<Usuario> u1 = new ArrayList<Usuario>();

    for (Usuario u : usuarioCyLs)
      if (includeOwn || id == 0 || id != u.getId()) u1.add(u);

    return u1;
  }

  private List<UsuarioCyL> buscarUsuariosUserName(String usuario, long id, boolean includeOwn) {
    List<UsuarioCyL> usuarioCyLs = findWhere("username", usuario);
    List<UsuarioCyL> u1 = new ArrayList<UsuarioCyL>();
    for (UsuarioCyL u : usuarioCyLs)
      if (includeOwn || id == 0 || id != u.getId()) u1.add(u);
    return u1;
  }

  private boolean compruebaCP(UsuarioCyL usuarioCyL) {
    if (usuarioCyL.getLocalidad() != null && usuarioCyL.getCp() != null && (!usuarioCyL.getCp().trim().isEmpty())
        && !lastCP.equals(usuarioCyL.getCp())) for (CodigoPostal cod : usuarioCyL.getLocalidad().getCodigosPostales()) {
      if (cod.getCodigo().equals(usuarioCyL.getCp().trim())) return true;
    }
    else return true;
    lastCP = usuarioCyL.getCp();
    addWarnMessage("usuario.error.codigo_postal");
    return false;
  }

  private boolean compruebaFNacimiento(UsuarioCyL usuarioCyL) {
    if (usuarioCyL.getFechaNacimiento() != null) {
      if (usuarioCyL.getEdad() < 1 || usuarioCyL.getEdad() > 150) {
        addErrorMessage("usuario.error.fecha_nacimiento");
        
        setFnacimientoOk(true);
        
        return false;
      }
      
      if (usuarioCyL.getEdad() > 100) {
        if (!isFnacimientoOk()) {
          addWarnMessage("usuario.error.fecha_nacimiento");
          
          setFnacimientoOk(true);
          
          return false;
        }
      }
      
      if (usuarioCyL.isMenor() && (usuarioCyL.getEdad() >= 14)) {
    	  addErrorMessage("Se ha marcado el usuario como menor de 14 sin N.I.F. pero tiene 14 años o más");

    	  return false;
      }

      /*if (usuarioCyL.getEdad() < 18) {*/
      if (usuarioCyL.getEdad() < 14) {
        if ((usuarioCyL.getNifAutorizado() == null) || usuarioCyL.getNifAutorizado().trim().isEmpty()) {
          addErrorMessage("El usuario es menor y debe introudcir el N.I.F. de su padre o tutor que lo atorice.");
          
          return false;
        }

        if ((usuarioCyL.getNombreCompletoAutorizado() == null) || usuarioCyL.getNombreCompletoAutorizado().trim().isEmpty()) {
          addErrorMessage("El usuario es menor y debe introducir el nombre completo de su padre o tutor que lo autorice.");
          
          return false;
        }
      }

      if (!usuarioCyL.isExtranjero() && usuarioCyL.getEdad() >= 14 /*&& usuarioCyL.getEdad() < 18*/) {
    	//usuarioCyL.setMenor(false);
    	
        if ((usuarioCyL.getNif() == null) || usuarioCyL.getNif().trim().isEmpty()) {
          //addErrorMessage("El usuario tiene entre 14 y 18 años por lo que debe proporcionar su nº de N.I.F.");
          addErrorMessage("El usuario tiene 14 años o más por lo que debe proporcionar su N.I.F.");
          
          return false;
        }
      }

      /*if (usuarioCyL.getEdad() > 18 && usuarioCyL.isMenor()) {
        addWarnMessage("El usuario está marcado como menor pero su edad es mayor que 18");
        return false;
      }*/
    }
    return true;
  }

  /**
   * Check usuario ya prereg.
   * 
   * @param usuarioCyL
   *           usuario cy l
   * @param id
   *           id
   * @return true, if successful
   */
  public boolean checkUsuarioYaPrereg(UsuarioCyL usuarioCyL, Long id) {
    if (usuarioNoPrereg) {
      usuarioPrereg = false;
      return true;
    }
    List<UsuarioPre> usuariosNombre = buscarUsuariosPreNombreCompleto(usuarioCyL.getNombre(),
        usuarioCyL.getApellido1(), usuarioCyL.getApellido2(), id, false);
    List<UsuarioPre> usuariosTelefono = buscarUsuariosPreTelefono(usuarioCyL.getTelefono(), usuarioCyL.getMovil(), id,
        false);
    if (usuariosNombre.isEmpty() && usuariosTelefono.isEmpty()) {
      return true;
    } else {
      for (UsuarioPre u : usuariosTelefono) {
        if (!usuariosNombre.contains(u)) {
          usuariosNombre.add(u);
        }
      }
      usuariosYaPrereg = usuariosNombre;
      usuarioPrereg = true;

      String mensaje = "Posible usuario ya pre-registrado.";
      addErrorMessage(mensaje);
      return false;
    }
  }

  private List<UsuarioPre> buscarUsuariosPreNombreCompleto(String nombre, String apellido1, String apellido2, long id,
      boolean includeOwn) {
    Map<String, Object> params = new HashMap<String, Object>();
    params.put("nombre", nombre);
    params.put("apellido1", apellido1);
    params.put("apellido2", apellido2);
    List<UsuarioPre> usuarios = findWhere(UsuarioPre.class, params);

    List<UsuarioPre> u1 = new ArrayList<UsuarioPre>();
    for (UsuarioPre u : usuarios)
      if (includeOwn || id == 0 || id != u.getId()) u1.add(u);
    return u1;
  }

  private List<UsuarioPre> buscarUsuariosPreTelefono(String telefono, String movil, long id, boolean includeOwn) {
    List<UsuarioPre> usuariosTfn = findWhere(UsuarioPre.class, "telefono", telefono);
    List<UsuarioPre> usuariosMvl = findWhere(UsuarioPre.class, "telefono", movil);

    List<UsuarioPre> uSearch = new ArrayList<UsuarioPre>();
    for (UsuarioPre u1 : usuariosTfn) {
      if (includeOwn || id == 0 || id != u1.getId()) uSearch.add(u1);
    }
    for (UsuarioPre u2 : usuariosMvl) {
      if (includeOwn || id == 0 || id != u2.getId()) {
        if (!uSearch.contains(u2)) {
          uSearch.add(u2);
        }
      }
    }
    return uSearch;
  }

  // ---------------------------------------------------------------------------------------------------------Fotografía

  private String getfileName(UsuarioCyL user) {
    String nombre = user.getId() + ".jpg";
    // int i = 0; while ((new File(USER_PHOTO_PATH + nombre)).exists()) { nombre = idUsuario + "_" + i + ".jpg"; i +=
    // 1;}
    return nombre;
  }

  // ------------------------------------------------------------------------------------------Generación de Contraseñas
  /**
   * Generate password.
   * 
   * @return string
   */
  public String generatePassword() {
    String resultado = "";
    String numeros = "0123456789";
    String letras = "bcdfghjkmnpqrstvwxyz";
    String mayusculas = "BCDFGHJKMNPQRSTVWXYZ";
    String caracter = "@.";

    // RF_15_46 - Relaxar as restriccións dos contrasinais
    resultado += getRandomString(numeros, 1);
    resultado += getRandomString(letras, 6);
    resultado += getRandomString(mayusculas, 1);
    resultado += getRandomString(caracter, 1);

    //List<String> list = Arrays.asList(resultado.split(""));
    String[] resultadoSplited = resultado.split("");
    List<String> list = Arrays.asList(Arrays.copyOfRange(resultadoSplited,1,resultadoSplited.length));
    
    Collections.shuffle(list);
    
    //RF_XEA --> Si la clave generada empieza por el caracter '#', se vuelve a generar dicha clave que el primer caracter no sea '#'.
    //           Se realiza este cambio porque en LDAP estaban dando problemas las claves que empiezan por '#'
    /*while (list.get(0).equals("€")){
    	Collections.shuffle(list);
    }^*/
    
    return join(list);
  }

  private String getRandomString(String base, int longitud) {
    String resultado = "";
    int te = 0;
    Random r = new Random();
    if (base.length() > 0) for (int i = 0; i < longitud; i++) {
      te = r.nextInt(base.length() - 1);
      resultado += base.charAt(te);
    }
    return resultado;
  }

  private String join(Collection<?> collection) {
    StringBuilder builder = new StringBuilder();
    for (Object name : collection)
      builder.append(name);
    return builder.toString();
  }

  /**
   * Find usuario cy l.
   * 
   * @param id
   *           id
   * @return usuario cy l
   */
  public UsuarioCyL findUsuarioCyL(Long id) {
    // TODO Auto-generated method stub
    if (id != null) return find(UsuarioCyL.class, id);
    else return null;
  }

  /**
   * Anadir observacion.
   * 
   * @param usuario
   *           usuario
   * @param c
   *           c
   */
  public void anadirObservacion(UsuarioCyL usuario, ComentarioObservacion c) {
    if (!c.getObservacion().trim().isEmpty()) {
      c.setFechaObservacion(new Date());
      c.setUsuario(usuario);
      usuario.getComentarioObservaciones().add(c);
      persist(c);
    }
  }

  /**
   * Devuelve comentario observacion nuevo.
   * 
   * @return comentario observacion nuevo
   */
  public ComentarioObservacion getComentarioObservacionNuevo() {
    return new ComentarioObservacion();
  }

  /**
   * Mostrarmensaje.
   * 
   * @param msg
   *           msg
   */
  public void mostrarmensaje(String msg) {
    setMessagesEnabled(true);
    if (msg.compareTo("borrarusuario") == 0) addInfoMessage("remove.usuario.ok");
    if (msg.compareTo("altausuario") == 0) addInfoMessage("remove.usuario.alta");
  }

  /**
   * Realta usuario.
   * 
   * @param id
   *           id
   */
  public void realtaUsuario(Long id) {
	  
	  Usuario u = find(id);
	  u.setDesactivado(false);
	  
	  List<Asesoramiento> asesoramientos = findWhere(Asesoramiento.class, "usuario.id", u.getId());
	  List<PreinscripcionPropuesta> propuestas = findWhere(PreinscripcionPropuesta.class, "usuario.id", u.getId());
	  List<PracticaTaller> practicas = findWhere(PracticaTaller.class, "usuario.id", u.getId());
	  
	  for (Asesoramiento m : asesoramientos)
		  m.setDesactivado(false);
	  
	  for (PracticaTaller m : practicas)
		  m.setDesactivado(false);

	  for (MatriculaCurso m : u.getMatriculasCursos())
		  m.setDesactivado(false);

	  for (MatriculaCharla m : u.getMatriculasCharlas())
		  m.setDesactivado(false);

	  for (MatriculaTaller m : u.getMatriculasTalleres())
		  m.setDesactivado(false);

	  for (MatriculaExterna m : u.getMatriculasExternas())
		  m.setDesactivado(false);

	  for (PreinscripcionCurso m : u.getPreinscripcionesCursos())
		  m.setDesactivado(false);

	  for (PreinscripcionTaller m : u.getPreinscripcionesTalleres()) 
		  m.setDesactivado(false);

	  for (PreinscripcionPropuesta m : propuestas)
		  m.setDesactivado(false);
	  
	  UsuarioCyL usuariocyl = find(UsuarioCyL.class, u.getId());
	  ldap.createUser(usuariocyl,this);
	  
	  enableMessages();
	  addInfoMessage("remove.usuario.alta");
	  disableMessages();
  }
  
  /**
   * 
   */
  public boolean isColaborador(Usuario usuario) {
	  boolean isColaborador = false;
	  
	  if (usuario.getRolesList().contains(Usuario.Role.ROLE_EXTERN))
		  isColaborador = true;
	  
	  return isColaborador; 
  } 
  
  public String getPasswordNuevoUsuarioCyL() {
	  return passwordNuevoUsuarioCyL;
  }
  
  /**
   * Notificar
   * 
   * @param usuario
   *           usuario
   */
  @SuppressWarnings("unchecked")
  public void notificarInscripcionUsuario(UsuarioCyL usuarioCyl) {
	  
	  NotificationService n = new NotificationService();
	  String mailActivate = "";
	  String mailUserregistrationActivate ="";
	  
	  try {
		  List<Parameter> parameters1 = findWhere(Parameter.class, "nombre", "mail_activate");
		  if (parameters1.size() > 0)
			  mailActivate = parameters1.get(0).getValor();

		  List<Parameter> parameters2 = findWhere(Parameter.class, "nombre", "mail_userregistration_activate");
		  if (parameters2.size() > 0)
			  mailUserregistrationActivate = parameters2.get(0).getValor();

		  if (mailActivate.equals("1")){
			  if (mailUserregistrationActivate.equals("1")){
				  if (n.notificarRegistroUsuario(this, usuarioCyl))
					  addInfoMessage("mail.email");
				  else
					  addErrorMessage("mail.errormail");
			  }
			  else{
				  // están desactivadas las notificaciones por email asociadas al preregistro de usuario en el sistema.
				  addWarnMessage("mail.userregistration.nonotifications");
			  }
		  }
		  else { 
			  // están desactivadas las notificaciones por email en el sistema.
			  addWarnMessage("mail.nonotifications");
		  }
	  } catch (Exception e){
		  e.printStackTrace();
	  }
  }
  
  public Boolean resetUserPassword(UsuarioCyL ucyl) {
	  try {
		  ucyl = findOneWhere("email", ucyl.getEmail());

		  if (ucyl != null) {
			  this.reactivarUsuarioLDAP(ucyl);
			  String password = generatePassword();
			  RegistroLdap reg = new RegistroLdap(ucyl);
			  Boolean result = false;

			  //Ponemos el estado anterior que tuviera, la intención es solo poner el mensaje del cambio de contraseña
			  //Sin que cambie el estado del registro en LDAP.
			  //reg.setCorrecto(ucyl.getRegistrosLdap().get(ucyl.getRegistrosLdap().size() - 1).isCorrecto()); 

			  //Si el establecimiento de la contraseña es CORRECTO
			  if (ldap.setPassword(ucyl, password, this) == 0) {
				  reg.setMensaje("Se reestableció la contraseña del usuario");
				  reg.setCorrecto(true);
				  result = true;
				  notificarResetPassword(ucyl, password);
			  } else { //Si el establecimiento de la contraseña es INCORRECTO
				  reg.setMensaje("Error al reestablecer la contraseña");
				  reg.setCorrecto(false);
				  
				  addErrorMessage("password.reset.fail");
			  }

			  //Añadimos el registro de acciones sobre LDAP
			  ucyl.getRegistrosLdap().add(reg);

			  //Guardamos el usuario
			  disableMessages();
			  persist(ucyl);
			  enableMessages();

			  return result;
		  }
		  else
			  addErrorMessage("password.reset.noResult");
	  }
	  catch (NoResultException nre) {
		  addErrorMessage("password.reset.noResult");
	  }
	  catch (NonUniqueResultException nure) {
		  addErrorMessage("password.reset.nonUniqueResult");
	  }
	  catch (Exception ex) {
		  addErrorMessage("password.reset.fail");
	  }

	  return false;
  }
  
  public Boolean changeUserPassword(Long idUsuario, String newPassword1, String newPassword2, Boolean esModificadoDesdeAdministracion) {
	  try {
		  UsuarioCyL ucyl = find(idUsuario);

		  if (ucyl != null) {
			  if (!avisoUsuarioSinEmailMostrado && ((ucyl.getEmail() == null) || ucyl.getEmail().isEmpty())){
				  if(esModificadoDesdeAdministracion){
					  addWarnMessage("password.change.emailEmpty");
					  avisoUsuarioSinEmailMostrado = true;
				  } else {
					  addErrorMessage("password.change.emailEmpty");
				  }
			  } else {
				  newPassword1 = newPassword1.trim();
				  newPassword2 = newPassword2.trim();
				  
				  if (newPassword1.compareTo(newPassword2) != 0) 
					  addErrorMessage("password.change.passwordNoMatch");
				  else if ((newPassword1.length() < 8) || (newPassword1.length() > 14) || newPassword1.contains(" "))
					  addErrorMessage("password.change.passwordLength");				  
				  else if (!passwordPattern(newPassword1)){
					  addErrorMessage("password.change.caracteresEspeciales"); //RF_15_46 - Relaxar as restriccións dos contrasinais 
				  } else {
					  RegistroLdap reg = new RegistroLdap(ucyl);
					  Boolean result = false;
					  
					  //Ponemos el estado anterior que tuviera, la intención es solo poner el mensaje del cambio de contraseña
					  //Sin que cambie el estado del registro en LDAP.
					  //reg.setCorrecto(ucyl.getRegistrosLdap().get(ucyl.getRegistrosLdap().size() - 1).isCorrecto()); 
		
					  //Si el establecimiento de la contraseña es CORRECTO
					  if (ldap.setPassword(ucyl, newPassword1, this) == 0) {						 
						  reg.setMensaje("Se cambió la contraseña del usuario");
						  reg.setCorrecto(true);
						  
						  result = true;
						  
						  if(ucyl.getEmail() != null && !ucyl.getEmail().isEmpty()){
							  notificarChangePassword(ucyl, newPassword1);
						  }
						  
					  } else { //Si el establecimiento de la contraseña es INCORRECTO
						  reg.setMensaje("Error al intentar cambiar la contraseña del usuario");
						  reg.setCorrecto(false);
						  addErrorMessage("password.change.fail");
					  }
		
					  //Añadimos el registro de acciones sobre LDAP
					  ucyl.getRegistrosLdap().add(reg);
					  
					  //Guardamos el usuario
					  disableMessages();
					  persist(ucyl);
					  enableMessages();
		
					  return result;
				  }
			  }
		  }
		  else
			  addErrorMessage("password.change.noUser");
	  }
	  catch (Exception ex) {
		  addErrorMessage("password.change.fail");
	  }

	  return false;
  }
  
  public String getUserEmailAddress(Long idUsuarioCyL) {
	  return find(idUsuarioCyL).getEmail();
  }
  
  /**
   * Notificar reestablecimiento de contraseña.
   * 
   * @param emailAddress
   *           dirección de Email
   *           
   * @return true, if successful
   */
  private void notificarResetPassword(UsuarioCyL ucyl, String password) {
	  NotificationService n = new NotificationService();

	  try {
		  if (!n.notificarResetPassword(this, ucyl, password))
			  addErrorMessage("mail.errormail");
	  } catch (Exception e){
		  e.printStackTrace();
	  }
  }

  /**
   * Notificar reestablecimiento de contraseña.
   * 
   * @param emailAddress
   *           dirección de Email
   *           
   * @return true, if successful
   */
  private void notificarChangePassword(UsuarioCyL ucyl, String password) {
	  NotificationService n = new NotificationService();

	  try {
		  if (!n.notificarChangePassword(this, ucyl, password))
			  addErrorMessage("mail.errormail");
	  } catch (Exception e){
		  e.printStackTrace();
	  }
  }


  
  
  public String getNombreLocalidad() {
	  return nombreLocalidad;
  }

  public void setNombreLocalidad(String nombreLocalidad) {
	  this.nombreLocalidad = nombreLocalidad;
  }

  public List<Localidad> getListLocalidades() {
	  return listLocalidades;
  }

  public void searchLocalidad(Provincia provincia) {
	  if (nombreLocalidad.length() > 1 ) {
		  String strqry = "SELECT localidad " +
		  				  "FROM " + Localidad.class.getName() + " localidad " +
		  				  "WHERE localidad.provincia.id = " + provincia.getId() + " AND" +
		  				  "      localidad.nombre LIKE '%%" + nombreLocalidad + "%%' " +
		  				  "ORDER BY localidad.nombre ASC";
		  
		  listLocalidades = findByQuery(Localidad.class, strqry);
		  
	  } else {
		  if (listLocalidades != null) {
			  listLocalidades.clear();
			  listLocalidades = null;
		  }
	  }
	  
	  //RF_15_33 
	  /**if (null != listLocalidades && !listLocalidades.isEmpty()){
		  listLocalidades.clear();
		  listLocalidades = null;
	  }
	  
	  String strqry = "SELECT localidad " + 
			  "FROM " + Localidad.class.getName() + " localidad " +
			  "WHERE localidad.provincia.id = " + provincia.getId() +
			  "ORDER BY localidad.nombre ASC";
	  
	  listLocalidades = findByQuery(Localidad.class, strqry);**/
  }

  /**
   * @return the idLocalidadSeleccionada
   */
  public Long getIdLocalidadSeleccionada() {
	  return idLocalidadSeleccionada;
  }

  /**
   * @param idLocalidadSeleccionada the idLocalidadSeleccionada to set
   */
  public void setIdLocalidadSeleccionada(Long idLocalidadSeleccionada) {
	  this.idLocalidadSeleccionada = idLocalidadSeleccionada;
  }

  /**
   * @return the dummyLocalidad
   */
  public Localidad getDummyLocalidad() {
	  return dummyLocalidad;
  }

  public void applyLocalidadToUsuario(UsuarioCyL usuarioToSetLocalidad) {
	  applyLocalidadToUsuario((Usuario)usuarioToSetLocalidad);
  }

  public void applyLocalidadToUsuario(UsuarioPre usuarioToSetLocalidad) {
	  applyLocalidadToUsuario((Usuario)usuarioToSetLocalidad);
  }
  
  private void applyLocalidadToUsuario(Usuario usuarioToSetLocalidad) {
	  if (idLocalidadSeleccionada == -1){
		  usuarioToSetLocalidad.setLocalidad(dummyLocalidad);
		//3.4.3-R RF_XEA_04 --> Borra la lista de localidades buscadas cuando se pulsa el botón "X" y asi al recargar la pagina no se muestre
		// la lista de localidades anteriormente buscada 
		  if (null != listLocalidades && !listLocalidades.isEmpty()){
			  listLocalidades.clear();
			  listLocalidades = null;
		  }
	  } else {
		  Map<String, Object> properties = new HashMap<String, Object>();
		  properties.put("provincia", usuarioToSetLocalidad.getProvincia());
		  properties.put("id", idLocalidadSeleccionada);
		  
		  usuarioToSetLocalidad.setLocalidad(findOneWhere(Localidad.class, properties));
	  }
  }
  
  public boolean getUserWithoutEmailAndDefaultPass() {
	  return userWithoutEmailAndDefaultPass;
  }
  
  public void setUserWithoutEmailAndDefaultPass(boolean userWithoutEmailAndDefaultPass) {
	  this.userWithoutEmailAndDefaultPass = userWithoutEmailAndDefaultPass;
  }
  
  
  //RF_XEA_08 - Muestra el numero total de alumnos
  public Long obtenerNumeroAlumnos(){
	  
	  StringBuilder sb = new StringBuilder();
	  sb.append("SELECT COUNT(ucyl.id) AS numero ").
	  append("FROM ").
	  append(UsuarioCyL.class.getName()).
	  append(" ucyl ").
	  append("JOIN ucyl.centro cen ").
	  append("WHERE ucyl.desactivado = 0 AND cen.desactivado = 0");
	  
	  Query q = getEntityManager().createQuery(sb.toString());
	  
	  return (Long)q.getSingleResult();
  }
  
  //RF_15_37 Usuarios activos
  public void persistUltimoAcceso(UsuarioCyL ucyl) {
	  try {		  
		  if(ucyl != null) {
			  ucyl.setUltimoAcceso(new Date());
		  }  
	  } catch(Exception e) {
		  e.printStackTrace();
	  }
  }
  
  public void updateLdapLogonTime(UsuarioCyL ucyl){
	  try {		  
		  if(ucyl != null) {
			  ldap.setUltimoAccesoLDAP(ucyl, this, ucyl.getUltimoAcceso());
		  }  
	  } catch(Exception e) {
		  e.printStackTrace();
	  }
	  
  }
  
  //RF_15_33 Formulario alta usuario- Confirmación de email
  public String getEmailConfirmacion() {
	  return emailConfirmacion;
  }

  public void setEmailConfirmacion(String emailConfirmacion) {
	  this.emailConfirmacion = emailConfirmacion;
  }
  
  public boolean checkEmailConfirmacion(String email) {
	  if(!email.isEmpty() || email != null) {
		  if(emailConfirmacion.equals(email)) {
			  return true;
		  }
  		}
	  return false;
  }
  
  //RF_15_46 - Relaxar as restriccións dos contrasinais 
  public boolean passwordPattern(String password) {
	  final String PASSWORD_PATTERN = "^([a-zA-Z0-9@.]{8,14})$";
	  Pattern p = Pattern.compile(PASSWORD_PATTERN);
	  return p.matcher(password).matches();
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.EmpleadoService#getUltimoAccesoLDAP(UsuarioCyL usuario);
   */
  public Date getUltimoAccesoLDAP(UsuarioCyL usuario){
	  Date value = null;
	  
	  try {
		  if(usuario!=null){		  
			  value = ldap.getUltimoAccesoLDAP(usuario, this);
		  }
	  } catch (NullPointerException ex){
	  }
	  return value;
  }
  
}
