package es.csa.fdt.cyldigital.usuarios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.List;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.commons.persistence.browser.jpaql.FilterJPAQLByQuery;
import es.csa.fdt.cyldigital.core.service.BrowserFilteredByCenter;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCharla;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;

/**
 * Servicio del Browser de Usuarios registrados.
 * 
 * @author csa
 */
@Service
@Repository
public class UsuariosBrowserImpl extends BrowserFilteredByCenter<UsuarioCyL> implements
    UsuariosBrowser<CriterionJPAQL, String> {

  private static final long serialVersionUID = -7227366052245717441L;

  private String customFilterProperty;

  private String nameFilterProperty;

  private String surnameFilterProperty;

  private String surname2FilterProperty;

  private String phoneFilterProperty;
  
  private Long indActivoLDAP;
  

  /**
   * Construye un nuevo objeto de la clase
   */
  public UsuariosBrowserImpl() {
    super();
    getSort().setSortAsc(false);
    getSort().setSortProperty("id");
  }

  /**
   * Filter indActivoLDAP.
   */
  public void filterIndActivoLDAP() {
	  removeIndActivoLDAP();

	  if (getIndActivoLDAP() != null){
		  filterByIndActivoLDAP(getIndActivoLDAP());
	  }
  }
	
  /**
   * removeIndActivoLDAP
   */
  private void removeIndActivoLDAP() {
	  int i = -1;

	  for (Filter<CriterionJPAQL, UsuarioCyL> f : getFilters())
		  if (f.getCriterion().getQuery().contains("desactivadoLDAP"))
			  i = getFilters().indexOf(f);

	  if (i > -1)
		  getFilters().remove(i);
  }
  
  /**
   * Filter by indActivoLDAP.
   * 
   * @param indActivoLDAP (0-todos;1-Si;2-No)
   *           
   */
  public void filterByIndActivoLDAP(Long indActivoLDAP) {
	  String query = "";
	  boolean flag = false;

	  if(indActivoLDAP == null || indActivoLDAP.longValue() <= 0 )
		  return;

	  FilterJPAQLByQuery<UsuarioCyL> filter = new FilterJPAQLByQuery<UsuarioCyL>();

	  query += " desactivadoLDAP = ? ";
	  filter.getCriterion().addParameter(indActivoLDAP.longValue()==2);

	  for (Filter<CriterionJPAQL, UsuarioCyL> f : getFilters())
		  if (f.getCriterion().getQuery().contains("desactivadoLDAP"))
			  flag = true;

	  if (!flag) {
		  filter.getCriterion().setQuery(query);
		  getFilters().add(filter);
	  }
  }
	
  @Override
  public void clearFilters() {
	  super.clearFilters();

	  filter(); //FilterCentro
	  filterIndActivoLDAP();
  }
  
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.UsuariosBrowser#applyCustomFilter()
   */
  public void applyCustomFilter() {
	  /*String query = "nif LIKE ? OR concat(trim(nombre),concat(' ',concat(trim(apellido1),concat(' ',trim(apellido2))))) LIKE ?";*/
	  //RF_XEA_07 se añade a la query para que filtre también por email
	  String query = "COALESCE(trim(nif),'') LIKE ? OR " +
	  				 "COALESCE(trim(pasaporte),'') LIKE ? OR " +
	  				 "COALESCE(trim(username),'') LIKE ? OR " +
	  				 "COALESCE(trim(telefono),'') LIKE ? OR " +
	  				 "COALESCE(trim(movil),'') LIKE ? OR " +
	  				 "COALESCE(trim(email),'') LIKE ? OR " +
	  				 "concat(COALESCE(trim(nombre),''),concat(' ',concat(COALESCE(trim(apellido1),''),concat(' ',COALESCE(trim(apellido2),''))))) LIKE ?";
	
	  Long id = null;
	  clearFilters();
	  if (getCustomFilterProperty() != null && !getCustomFilterProperty().trim().isEmpty()) {
		  try {
			  id = Long.parseLong(customFilterProperty);
		  }
		  catch (Exception e) {
		  }

		  FilterJPAQLByQuery<UsuarioCyL> filter = new FilterJPAQLByQuery<UsuarioCyL>();
		  if (id != null) {
			  query += " OR id = ?";
    	  
			  Object[] params = { "%" + customFilterProperty + "%",
					  			  "%" + customFilterProperty + "%",
					  			  "%" + customFilterProperty + "%",
					  			  "%" + customFilterProperty + "%",
					  			  "%" + customFilterProperty + "%",
					  			  "%" + customFilterProperty + "%",
					  			  "%" + customFilterProperty + "%",
					  			  id };
    	  
			  filter.getCriterion().addParameters(params);
		  }
		  else {
			  Object[] params = { "%" + customFilterProperty + "%",
					  			  "%" + customFilterProperty + "%",
					  			  "%" + customFilterProperty + "%",
					  			  "%" + customFilterProperty + "%",
					  			  "%" + customFilterProperty + "%",
					  			  "%" + customFilterProperty + "%",
					  			  "%" + customFilterProperty + "%" };
    	  
			  filter.getCriterion().addParameters(params);
		  }
      
		  filter.getCriterion().setQuery(query);
		  filters.add(filter);
	  }
    
	  setCustomFilterProperty("");
  }

  /**
   * Apply inscription filter.
   */
  public void applyInscriptionFilter() {
    String query = "";
    String separator = "";
    clearFilters();
    FilterJPAQLByQuery<UsuarioCyL> filter = new FilterJPAQLByQuery<UsuarioCyL>();
    if (getNameFilterProperty() != null && !getNameFilterProperty().trim().isEmpty()) {
      filter.getCriterion().addParameter("%" + nameFilterProperty + "%");
      query += "nombre LIKE ?";
      separator = " OR ";
    }
    if (getSurnameFilterProperty() != null && !getSurnameFilterProperty().trim().isEmpty()) {
      filter.getCriterion().addParameter("%" + surnameFilterProperty + "%");
      query += separator + "apellido1 LIKE ?";
      separator = " OR ";
    }
    if (getSurname2FilterProperty() != null && !getSurname2FilterProperty().trim().isEmpty()) {
      filter.getCriterion().addParameter("%" + surname2FilterProperty + "%");
      query += separator + "apellido2 LIKE ?";
      separator = " OR ";
    }
    if (getPhoneFilterProperty() != null && !getPhoneFilterProperty().trim().isEmpty()) {
      filter.getCriterion().addParameter("%" + phoneFilterProperty + "%");
      query += separator + "telefono LIKE ?";
    }

    if (query != null && !query.trim().isEmpty()) {
      filter.getCriterion().setQuery(query);
      filters.add(filter);

      setNameFilterProperty("");
      setSurnameFilterProperty("");
      setSurname2FilterProperty("");
      setPhoneFilterProperty("");
    }
  }
  
  /**
   * Añade como condición el filtrado de usuarios que no aparecen en la lista de matrículas recibidas en el parámetro.
   * 
   * @param matriculas
   */
  public void addNoMatriculadoCursoCondition(List<MatriculaCurso> matriculas) {
	  if (matriculas.size() > 0) {
		  String query = null; 
		
		  FilterJPAQLByQuery<UsuarioCyL> filter = new FilterJPAQLByQuery<UsuarioCyL>();
	
		  Object[] idsMatriculados = new Object[matriculas.size()];

		  int i = 0;
		  for (MatriculaCurso mc : matriculas) {
			  if (i == 0)
				  query = "(id <> ?)";
			  else
				  query += " AND (id <> ?)";
			  
			  idsMatriculados[i] = mc.getUsuario().getId();
			  
			  i++;
		  }

		  filter.getCriterion().setQuery(query);
		  filter.getCriterion().addParameters(idsMatriculados);

		  conditions.add(filter);
	  }
  }
  
  /**
   * Añade como condición el filtrado de usuarios que no aparecen en la lista de matrículas recibidas en el parámetro.
   * 
   * @param matriculas
   */
  public void addNoMatriculadoCharlaCondition(List<MatriculaCharla> matriculas) {
	  if (matriculas.size() > 0) {
		  String query = null; 
		
		  FilterJPAQLByQuery<UsuarioCyL> filter = new FilterJPAQLByQuery<UsuarioCyL>();
	
		  Object[] idsMatriculados = new Object[matriculas.size()];

		  int i = 0;
		  for (MatriculaCharla mc : matriculas) {
			  if (i == 0)
				  query = "(id <> ?)";
			  else
				  query += " AND (id <> ?)";
			  
			  idsMatriculados[i] = mc.getUsuario().getId();
			  
			  i++;
		  }

		  filter.getCriterion().setQuery(query);
		  filter.getCriterion().addParameters(idsMatriculados);

		  conditions.add(filter);
	  }
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.UsuariosBrowser#setCustomFilterProperty(java.lang.String)
   */
  public void setCustomFilterProperty(String customFilter) {
    this.customFilterProperty = customFilter.trim();
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.UsuariosBrowser#getCustomFilterProperty()
   */
  public String getCustomFilterProperty() {
    return customFilterProperty;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.UsuariosBrowser#getNameFilterProperty()
   */
  public String getNameFilterProperty() {
    return nameFilterProperty;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.UsuariosBrowser#setNameFilterProperty(java.lang.String)
   */
  public void setNameFilterProperty(String nameFilterProperty) {
    this.nameFilterProperty = nameFilterProperty;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.UsuariosBrowser#getSurnameFilterProperty()
   */
  public String getSurnameFilterProperty() {
    return surnameFilterProperty;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.UsuariosBrowser#setSurnameFilterProperty(java.lang.String)
   */
  public void setSurnameFilterProperty(String surnameFilterProperty) {
    this.surnameFilterProperty = surnameFilterProperty;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.UsuariosBrowser#getSurname2FilterProperty()
   */
  public String getSurname2FilterProperty() {
    return surname2FilterProperty;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.UsuariosBrowser#setSurname2FilterProperty(java.lang.String)
   */
  public void setSurname2FilterProperty(String surname2FilterProperty) {
    this.surname2FilterProperty = surname2FilterProperty;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.UsuariosBrowser#getPhoneFilterProperty()
   */
  public String getPhoneFilterProperty() {
    return phoneFilterProperty;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.UsuariosBrowser#setPhoneFilterProperty(java.lang.String)
   */
  public void setPhoneFilterProperty(String phoneFilterProperty) {
    this.phoneFilterProperty = phoneFilterProperty;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.UsuariosBrowser#getIndActivoLDAP()
   */
  public Long getIndActivoLDAP() {
	return indActivoLDAP;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.UsuariosBrowser#setIndActivoLDAP(java.lang.Long)
   */
  public void setIndActivoLDAP(Long indActivoLDAP) {
	this.indActivoLDAP = indActivoLDAP;
  }
  
}
