package es.csa.fdt.cyldigital.usuarios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.commons.persistence.browser.jpaql.FilterJPAQLByQuery;
import es.csa.fdt.cyldigital.core.service.BrowserFilteredByActive;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;

/**
 * Servicio del Browser de Usuarios.
 * 
 * @author csa
 */
@Service
@Repository
public class UsuariosGenBrowserImpl extends BrowserFilteredByActive<Usuario> implements
    UsuariosGenBrowser<CriterionJPAQL, String> {

  private static final long serialVersionUID = -7227366052245717441L;

  private boolean conNif;

  private String nif;

  private String nameFilterProperty;

  private String surnameFilterProperty;

  private String surname2FilterProperty;

  private String phoneFilterProperty;

  /**
   * Construye un nuevo objeto de la clase
   */
  public UsuariosGenBrowserImpl() {
    super();
    getSort().setSortAsc(false);
    getSort().setSortProperty("id");
    setConNif(true);
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.UsuariosGenBrowser#applyInscriptionFilter()
   */
  public boolean applyInscriptionFilter() {
    boolean validate = true;
    String query = "";
    String separator = "";
    clearFilters();
    FilterJPAQLByQuery<Usuario> filter = new FilterJPAQLByQuery<Usuario>();
    if (conNif) {
      if (getNif() != null && !getNif().trim().isEmpty()) {
        filter.getCriterion().addParameter("%" + nif + "%");
        query += "nif LIKE ?";
      }
      if (validate && query != null && !query.trim().isEmpty()) {
        filter.getCriterion().setQuery(query);
        filters.add(filter);
        setNif("");
      }
    } else {
      if (getNameFilterProperty() != null && !getNameFilterProperty().trim().isEmpty()) {
        filter.getCriterion().addParameter("%" + nameFilterProperty + "%");
        query += "nombre LIKE ?";
        separator = " AND ";
      }
      if (getSurnameFilterProperty() != null && !getSurnameFilterProperty().trim().isEmpty()) {
        filter.getCriterion().addParameter("%" + surnameFilterProperty + "%");
        query += separator + "apellido1 LIKE ?";
        separator = " AND ";
      }
      if (getSurname2FilterProperty() != null && !getSurname2FilterProperty().trim().isEmpty()) {
        filter.getCriterion().addParameter("%" + surname2FilterProperty + "%");
        query += separator + "apellido2 LIKE ?";
        separator = " AND ";
      }
      if (getPhoneFilterProperty() != null && !getPhoneFilterProperty().trim().isEmpty()) {
        filter.getCriterion().addParameter("%" + phoneFilterProperty + "%");
        query += separator + "telefono LIKE ?";
      }
      if (validate && query != null && !query.trim().isEmpty()) {
        filter.getCriterion().setQuery(query);
        filters.add(filter);
        setNameFilterProperty("");
        setSurnameFilterProperty("");
        setSurname2FilterProperty("");
        setPhoneFilterProperty("");
      }
    }
    return validate;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.UsuariosGenBrowser#getNif()
   */
  public String getNif() {
    return nif;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.UsuariosGenBrowser#setNif(java.lang.String)
   */
  public void setNif(String nif) {
    this.nif = nif;
  }

  /**
   * Checks if is con nif.
   * 
   * @return true, if is con nif
   */
  public boolean isConNif() {
    return conNif;
  }

  /**
   * Asigna con nif.
   * 
   * @param conNif
   *          con nif
   */
  public void setConNif(boolean conNif) {
    this.conNif = conNif;
  }

  /**
   * Devuelve name filter property.
   * 
   * @return name filter property
   */
  public String getNameFilterProperty() {
    return nameFilterProperty;
  }

  /**
   * Asigna name filter property.
   * 
   * @param nameFilterProperty
   *          name filter property
   */
  public void setNameFilterProperty(String nameFilterProperty) {
    this.nameFilterProperty = nameFilterProperty;
  }

  /**
   * Devuelve surname filter property.
   * 
   * @return surname filter property
   */
  public String getSurnameFilterProperty() {
    return surnameFilterProperty;
  }

  /**
   * Asigna surname filter property.
   * 
   * @param surnameFilterProperty
   *          surname filter property
   */
  public void setSurnameFilterProperty(String surnameFilterProperty) {
    this.surnameFilterProperty = surnameFilterProperty;
  }

  /**
   * Devuelve surname2 filter property.
   * 
   * @return surname2 filter property
   */
  public String getSurname2FilterProperty() {
    return surname2FilterProperty;
  }

  /**
   * Asigna surname2 filter property.
   * 
   * @param surname2FilterProperty
   *          surname2 filter property
   */
  public void setSurname2FilterProperty(String surname2FilterProperty) {
    this.surname2FilterProperty = surname2FilterProperty;
  }

  /**
   * Devuelve phone filter property.
   * 
   * @return phone filter property
   */
  public String getPhoneFilterProperty() {
    return phoneFilterProperty;
  }

  /**
   * Asigna phone filter property.
   * 
   * @param phoneFilterProperty
   *          phone filter property
   */
  public void setPhoneFilterProperty(String phoneFilterProperty) {
    this.phoneFilterProperty = phoneFilterProperty;
  }

}
