package es.csa.fdt.cyldigital.usuarios.service;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import es.csa.commons.persistence.browser.Filter;
import es.csa.commons.persistence.browser.jpaql.CriterionJPAQL;
import es.csa.commons.persistence.browser.jpaql.FilterJPAQLByQuery;
import es.csa.fdt.cyldigital.core.service.BrowserFilteredByActive;
import es.csa.fdt.cyldigital.core.service.BrowserFilteredByCenter;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioPre;

/**
 * Servicio del Browser de Usuarios Preregistrados.
 * 
 * @author csa
 */
@Service
@Repository
/*public class UsuariosPreBrowserImpl extends BrowserFilteredByActive<UsuarioPre> implements
    UsuariosPreBrowser<CriterionJPAQL, String> {*/
public class UsuariosPreBrowserImpl extends BrowserFilteredByCenter<UsuarioPre> implements
    UsuariosPreBrowser<CriterionJPAQL, String> {
	
  private static final long serialVersionUID = -7227366052245717441L;

  private String customFilterProperty;

  private String nameFilterProperty;

  private String surnameFilterProperty;

  private String surname2FilterProperty;

  private String phoneFilterProperty;

  /**
   * Construye un nuevo objeto de la clase
   */
  public UsuariosPreBrowserImpl() {
    super();
    getSort().setSortAsc(false);
    getSort().setSortProperty("id");
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.UsuariosPreBrowser#applyCustomFilter()
   */
  public void applyCustomFilter() {
    String query = "nif LIKE ? OR concat(nombre,concat(' ',concat(apellido1,concat(' ',apellido2)))) LIKE ?";
    Long id = null;
    clearFilters();
    if (getCustomFilterProperty() != null && !getCustomFilterProperty().trim().isEmpty()) {
      try {
        id = Long.parseLong(customFilterProperty);
      } catch (Exception e) {}

      FilterJPAQLByQuery<UsuarioPre> filter = new FilterJPAQLByQuery<UsuarioPre>();
      if (id != null) {
        query += " OR id = ?";
        Object[] params = { "%" + customFilterProperty + "%", "%" + customFilterProperty + "%", id };
        filter.getCriterion().addParameters(params);
      } else {
        Object[] params = { "%" + customFilterProperty + "%", "%" + customFilterProperty + "%" };
        filter.getCriterion().addParameters(params);
      }
      filter.getCriterion().setQuery(query);
      filters.add(filter);
    }
    setCustomFilterProperty("");
  }

  /**
   * Apply inscription filter.
   */
  public void applyInscriptionFilter() {
    String query = "";
    String separator = "";
    clearFilters();
    FilterJPAQLByQuery<UsuarioPre> filter = new FilterJPAQLByQuery<UsuarioPre>();
    if (getNameFilterProperty() != null && !getNameFilterProperty().trim().isEmpty()) {
      filter.getCriterion().addParameter("%" + nameFilterProperty + "%");
      query += "nombre LIKE ?";
      separator = " OR ";
    }
    if (getSurnameFilterProperty() != null && !getSurnameFilterProperty().trim().isEmpty()) {
      filter.getCriterion().addParameter("%" + surnameFilterProperty + "%");
      query += separator + "apellido1 LIKE ?";
      separator = " OR ";
    }
    if (getSurname2FilterProperty() != null && !getSurname2FilterProperty().trim().isEmpty()) {
      filter.getCriterion().addParameter("%" + surname2FilterProperty + "%");
      query += separator + "apellido2 LIKE ?";
      separator = " OR ";
    }
    if (getPhoneFilterProperty() != null && !getPhoneFilterProperty().trim().isEmpty()) {
      filter.getCriterion().addParameter("%" + phoneFilterProperty + "%");
      query += separator + "telefono LIKE ?";
    }

    if (query != null && !query.trim().isEmpty()) {
      filter.getCriterion().setQuery(query);
      filters.add(filter);

      setNameFilterProperty("");
      setSurnameFilterProperty("");
      setSurname2FilterProperty("");
      setPhoneFilterProperty("");
    }
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.UsuariosPreBrowser#setCustomFilterProperty(java.lang.String)
   */
  public void setCustomFilterProperty(String customFilter) {
    this.customFilterProperty = customFilter;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.UsuariosPreBrowser#getCustomFilterProperty()
   */
  public String getCustomFilterProperty() {
    return customFilterProperty;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.UsuariosPreBrowser#getNameFilterProperty()
   */
  public String getNameFilterProperty() {
    return nameFilterProperty;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.UsuariosPreBrowser#setNameFilterProperty(java.lang.String)
   */
  public void setNameFilterProperty(String nameFilterProperty) {
    this.nameFilterProperty = nameFilterProperty;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.UsuariosPreBrowser#getSurnameFilterProperty()
   */
  public String getSurnameFilterProperty() {
    return surnameFilterProperty;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.UsuariosPreBrowser#setSurnameFilterProperty(java.lang.String)
   */
  public void setSurnameFilterProperty(String surnameFilterProperty) {
    this.surnameFilterProperty = surnameFilterProperty;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.UsuariosPreBrowser#getSurname2FilterProperty()
   */
  public String getSurname2FilterProperty() {
    return surname2FilterProperty;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.UsuariosPreBrowser#setSurname2FilterProperty(java.lang.String)
   */
  public void setSurname2FilterProperty(String surname2FilterProperty) {
    this.surname2FilterProperty = surname2FilterProperty;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.UsuariosPreBrowser#getPhoneFilterProperty()
   */
  public String getPhoneFilterProperty() {
    return phoneFilterProperty;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.usuarios.service.UsuariosPreBrowser#setPhoneFilterProperty(java.lang.String)
   */
  public void setPhoneFilterProperty(String phoneFilterProperty) {
    this.phoneFilterProperty = phoneFilterProperty;
  }

  /**
   * Asigna filter centro.
   * 
   * @param user
   *          filter centro
   */
  /*public void setFilterCentro(UsuarioLdap user) {
    if (user.applyFilterCentro()) {
      removeFilterCentro();
      if (user.getCentro() != null) {
        filterByCentro(user.getCentro().getId());
      }
    }
  }*/

  /*private void removeFilterCentro() {
    int i = -1;
    for (Filter<CriterionJPAQL, UsuarioPre> f : getFilters())
      if (f.getCriterion().getQuery().contains("centro.id")) i = getFilters().indexOf(f);
    if (i > -1) getFilters().remove(i);
  }*/

  /**
   * Filter by centro.
   * 
   * @param idCentro
   *           id centro
   */
  public void filterByCentro(Long idCentro) {
    String query = "";
    boolean flag = false;
    FilterJPAQLByQuery<UsuarioPre> filter = new FilterJPAQLByQuery<UsuarioPre>();

    query += " centro.id = ? ";
    filter.getCriterion().addParameter(idCentro);

    for (Filter<CriterionJPAQL, UsuarioPre> f : getFilters())
      if (f.getCriterion().getQuery().contains("centro.id")) flag = true;

    if (!flag) {
      filter.getCriterion().setQuery(query);
      getFilters().add(filter);
    }
  }

}
