package es.csa.fdt.cyldigital.util;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.Serializable;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import es.csa.fdt.cyldigital.calendario.entity.ActividadCalendarItem;

/**
 * Clase ActivitySchedule.
 * 
 * @author Fundetec
 */
public class ActivitySchedule implements Serializable {
	private static final long serialVersionUID = 9099068029496750617L;
	
	private Date startDate;
	private Date startTime;
	private Date endDate;
	private Date endTime;
	
	private Integer[] weekDays;
	
	private List<ActividadCalendarItem> actividadCalendarItems;
	
	private Long efectiveStartDateTimestamp;
	private Long efectiveEndDateTimestamp;
	
	/**
	 * @return the startDate
	 */
	public Date getStartDate() {
		return startDate;
	}

	/**
	 * @param startDate the startDate to set
	 */
	public void setStartDate(Date startDate) {
		this.startDate = startDate;
		
		calculateCalendaritems();
	}

	/**
	 * @return the startTime
	 */
	public Date getStartTime() {
		return startTime;
	}

	/**
	 * @param startTime the startTime to set
	 */
	public void setStartTime(Date startTime) {
		this.startTime = startTime;
		
		calculateCalendaritems();
	}

	/**
	 * @return the endDate
	 */
	public Date getEndDate() {
		return endDate;
	}

	/**
	 * @param endDate the endDate to set
	 */
	public void setEndDate(Date endDate) {
		this.endDate = endDate;
		
		calculateCalendaritems();
	}

	/**
	 * @return the endTime
	 */
	public Date getEndTime() {
		return endTime;
	}

	/**
	 * @param endTime the endTime to set
	 */
	public void setEndTime(Date endTime) {
		this.endTime = endTime;
		
		calculateCalendaritems();
	}

	/**
	 * @return the weekDays
	 */
	public Integer[] getWeekDays() {
		return weekDays;
	}

	/**
	 * @param weekDays the weekDays to set
	 */
	public void setWeekDays(Integer[] weekDays) {
		this.weekDays = weekDays;
		
		calculateCalendaritems();
	}
	
	public List<ActividadCalendarItem> getActividadCalendarItems() {
		if (actividadCalendarItems == null)
			calculateCalendaritems();

		return actividadCalendarItems;
	}
	
	/**
	 * @return the efectiveStartDateTimestamp
	 */
	public Long getEfectiveStartDateTimestamp() {
		if (actividadCalendarItems == null)
			calculateCalendaritems();

		return efectiveStartDateTimestamp;
	}

	/**
	 * @return the efectiveEndDateTimestamp
	 */
	public Long getEfectiveEndDateTimestamp() {
		if (actividadCalendarItems == null)
			calculateCalendaritems();
		
		return efectiveEndDateTimestamp;
	}
	
	public boolean collisionsWith(ActivitySchedule schedule) {
		boolean collision = false;
		
		List<ActividadCalendarItem> ownAcis = getActividadCalendarItems();
		List<ActividadCalendarItem> otherAcis = schedule.getActividadCalendarItems();

		for (ActividadCalendarItem ownAci : ownAcis) {
			for (ActividadCalendarItem otherAci : otherAcis) {
				/*if ((((ownAci.getStart().compareTo(otherAci.getStart()) >= 0) &&
					  (ownAci.getStart().compareTo(otherAci.getEnd()) < 0)) ||
					 ((ownAci.getEnd().compareTo(otherAci.getStart()) > 0) &&
					  (ownAci.getEnd().compareTo(otherAci.getEnd()) <= 0)))
					||
					(((otherAci.getStart().compareTo(ownAci.getStart()) >= 0) &&
					  (otherAci.getStart().compareTo(ownAci.getEnd()) < 0)) ||
					 ((otherAci.getEnd().compareTo(ownAci.getStart()) > 0) &&
					  (otherAci.getEnd().compareTo(ownAci.getEnd()) <= 0)))   ) {*/
				if ((ownAci.getStart().compareTo(otherAci.getEnd()) < 0) &&
					(ownAci.getEnd().compareTo(otherAci.getStart()) > 0)) {
					collision = true;
					
					break;
				}
			}

			if (collision)
				break;
		}
		
		return collision;
	}

	private void calculateCalendaritems() {
		if ((startDate != null) && (startTime != null) &&
			(endDate   != null) && (endTime   != null) &&
			(startDate.compareTo(endDate) <= 0) &&
			(startTime.compareTo(endTime) < 0) &&
			(weekDays  != null)) {

			if (actividadCalendarItems == null)
				actividadCalendarItems = new ArrayList<ActividadCalendarItem>();

			if (!actividadCalendarItems.isEmpty())
				actividadCalendarItems.clear();

			efectiveStartDateTimestamp = null;
			efectiveEndDateTimestamp = null;

			List<Integer> lwd = Arrays.asList(this.getWeekDays());
			
			Calendar cItem = Calendar.getInstance();
			Calendar cDate = Calendar.getInstance();
			
			Calendar cStartDate = Calendar.getInstance();
			Calendar cEndDate = Calendar.getInstance();
			
			Calendar cStartTime = Calendar.getInstance();
			Calendar cEndTime = Calendar.getInstance();
			
			cStartDate.setTime(this.getStartDate());
			cStartTime.setTime(this.getStartTime());
			
			cEndDate.setTime(this.getEndDate());
			cEndTime.setTime(this.getEndTime());
	
			for (cDate.setTime(cStartDate.getTime()); (cDate.compareTo(cEndDate) <= 0); cDate.add(Calendar.DATE, 1)) {
				if (lwd.contains(Integer.valueOf(cDate.get(Calendar.DAY_OF_WEEK)))) {
					ActividadCalendarItem item = new ActividadCalendarItem();
	
					cItem.setTime(cDate.getTime());
				  
					cItem.set(Calendar.HOUR_OF_DAY, cStartTime.get(Calendar.HOUR_OF_DAY));
					cItem.set(Calendar.MINUTE, cStartTime.get(Calendar.MINUTE));
				  
					item.setStart(cItem.getTime());
				  
					cItem.set(Calendar.HOUR_OF_DAY, cEndTime.get(Calendar.HOUR_OF_DAY));
					cItem.set(Calendar.MINUTE, cEndTime.get(Calendar.MINUTE));
				  
					item.setEnd(cItem.getTime());
					
					if (((efectiveStartDateTimestamp != null) && (cDate.getTime().getTime() < efectiveStartDateTimestamp)) ||
						(efectiveStartDateTimestamp == null))
						efectiveStartDateTimestamp = cDate.getTime().getTime();
						
					if (((efectiveEndDateTimestamp != null) && (cDate.getTime().getTime() > efectiveEndDateTimestamp)) ||
						(efectiveEndDateTimestamp == null))
						efectiveEndDateTimestamp = cDate.getTime().getTime();
				  
					item.setAllDay(false);
	
					actividadCalendarItems.add(item);
				}
			}
		}
	}
}
