package es.csa.fdt.cyldigital.util;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.awt.image.ConvolveOp;
import java.awt.image.Kernel;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;

/**
 * Clase ImageUtils.
 * 
 * @author csa
 */
public class ImageUtils {

  /**
   * Devuelve image.
   * 
   * @param width
   *           width
   * @param height
   *           height
   * @param datos
   *           datos
   * @return image
   */
  public static BufferedImage getImage(int width, int height, String datos) {
    String[] datosArray = datos.split(",");
    String rgbText = "";

    BufferedImage imageBuffer = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);

    for (int i = 0; i < width; i++)
      for (int j = 0; j < height; j++)
        imageBuffer.setRGB(i, j, 0xffffff);

    try {
      for (int row = 0; row < height; row++) {
        for (int col = 0; col < width; col++) {
          int field = (row * (width + 1)) + col;
          rgbText = ((datosArray[field]).trim().isEmpty()) ? "ffffff" : datosArray[field];
          rgbText = getCorrectedColor(rgbText);
          try {
            imageBuffer.setRGB(col, row, getRgbFromText(rgbText));
          } catch (Exception e) {
            System.out.println(e);
          }
        }
      }
      return imageBuffer;
    } catch (Exception e) {
      e.printStackTrace();
    }
    return null;
  }

  /**
   * Resize.
   * 
   * @param originalFile
   *           original file
   * @param resizedFile
   *           resized file
   * @param newWidth
   *          width
   * @param quality
   *           quality
   * @throws IOException
   *           Signals that an I/O exception has occurred.
   */
  public static void resize(File originalFile, File resizedFile, int newWidth, float quality) throws IOException {
    ImageIcon ii = new ImageIcon(originalFile.getCanonicalPath());
    Image i = ii.getImage();
    Image resizedImage = null;

    // int iWidth = 180;// i.getWidth(null);
    // int iHeight = 240;

    // if (iWidth > iHeight) {
    // resizedImage = i.getScaledInstance(newWidth, (newWidth * iHeight) / iWidth, Image.SCALE_SMOOTH);
    // } else {
    // resizedImage = i.getScaledInstance((newWidth * iWidth) / iHeight, newWidth, Image.SCALE_SMOOTH);
    // }
    resizedImage = i.getScaledInstance(58, 72, Image.SCALE_SMOOTH);

    Image temp = new ImageIcon(resizedImage).getImage();
    BufferedImage bufferedImage = new BufferedImage(54, 72, BufferedImage.TYPE_INT_RGB);
    Graphics g = bufferedImage.createGraphics();
    g.setColor(Color.white);
    g.fillRect(0, 0, 54, 72);
    g.drawImage(temp, 0, 0, null);
    g.dispose();

    // Soften.
    float softenFactor = 0.05f;
    float[] softenArray = { 0, softenFactor, 0, softenFactor, 1 - (softenFactor * 4), softenFactor, 0, softenFactor, 0 };
    Kernel kernel = new Kernel(3, 3, softenArray);
    ConvolveOp cOp = new ConvolveOp(kernel, ConvolveOp.EDGE_NO_OP, null);
    bufferedImage = cOp.filter(bufferedImage, null);

    FileOutputStream out = new FileOutputStream(resizedFile);
    ImageIO.write(bufferedImage, "jpeg", out);
  }

  /**
   * Crea image filesystem.
   * 
   * @param nombre
   *           nombre
   * @param path
   *           path
   * @param imagen
   *           imagen
   */
  public static void createImageFilesystem(String nombre, String path, BufferedImage imagen) {
    try {
      FileOutputStream fileOutputStream = new FileOutputStream(path + nombre);
      ImageIO.write(imagen, "jpg", fileOutputStream);
      fileOutputStream.flush();
      fileOutputStream.close();
    } catch (FileNotFoundException e) {
      e.printStackTrace();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  /**
   * Elimina image filesystem.
   * 
   * @param nombre
   *           nombre
   * @param path
   *           path
   */
  public static void removeImageFilesystem(String nombre, String path) {
    File f = new File(path + nombre);
    if (f.exists()) f.delete();
  }

  private static int getRgbFromText(String rgbText) {
    return (getColorInInt(rgbText, 0) << 16 | getColorInInt(rgbText, 1) << 8 | getColorInInt(rgbText, 2));
  }

  /**
   * Devuelve color in int.
   * 
   * @param colorString
   *           color string
   * @param col
   *           col
   * @return color in int
   */
  public static int getColorInInt(String colorString, int col) {
    StringBuilder color = new StringBuilder(colorString);
    StringBuilder sb = new StringBuilder(new Character(color.charAt(col == 0 ? 2 : col == 1 ? 4 : 6)).toString());
    sb.append(new Character(color.charAt(col == 0 ? 3 : col == 1 ? 5 : 7)).toString());
    return Integer.decode("0x" + sb);
  }

  /**
   * Devuelve corrected color.
   * 
   * @param color
   *           color
   * @return corrected color
   */
  public static String getCorrectedColor(String color) {
    String newColor = "0x";
    int shortcome = 6 - color.length();// pad with zeros
    for (int i = 0; i < shortcome; i++)
      newColor += "0";
    // the color value always starts at the index 2
    for (int i = 0; i < color.length(); i++)
      newColor += new Character(color.charAt(i)).toString();
    return newColor;
  }
}
