package es.csa.fdt.cyldigital.util;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.Query;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.core.entity.AbstractEntity;

import java.io.Serializable;

/**
 * Clase SelectFilterByCenterAux.
 * 
 * @author 
 */
public class SelectFilterByCenterAux extends AbstractServiceImpl<AbstractEntity, Serializable>{
	
	  /**
	 * 
	 */
	private static final long serialVersionUID = 620949812302193939L;
	
	private List<Centro> centrosDisponibles;
	
	private static final Logger logger = Logger.getLogger(SelectFilterByCenterAux.class.getName());
	
	  public SelectFilterByCenterAux() {
		
	  }

	  /**
	   * Devuelve el centro de Colaboradores que está parametrizado.
	   * 
	   * @return Centro
	   */
	  public Centro getCentroColaboradores() {
		  Centro centroColaboradores = null;

		  try {
			  String idCentroColaboradores = findOneWhere(Parameter.class, "nombre", "extern_default_center_id").getValor();
			 
			  centroColaboradores = findOneWhere(Centro.class, "id", Long.parseLong(idCentroColaboradores));
		  }
		  catch (NoResultException nre) {
			  logger.log(Level.WARNING, "SelectFilterByCenterAux.getCentroColaboradores: Centro de colaboradores no encontrado. " + nre.getLocalizedMessage());
			  
			  nre.printStackTrace();
		  }
		  catch (Exception e) {
			  logger.log(Level.WARNING, "SelectFilterByCenterAux.getCentroColaboradores: Error leyendo centro de colaboradores de la base de datos. " + e.getLocalizedMessage());
			  
			  e.printStackTrace();
		  }
		  
		  return centroColaboradores;
			  
	  }
	  

	  /**
	   * Devuelve el centro de Test que está parametrizado.
	   * 
	   * @return Centro
	   */
	  public Centro getCentroTest() {
		  Centro centroTest = null;

		  try {
			  String idCentroTest = findOneWhere(Parameter.class, "nombre", "admin_default_center_id").getValor();
			 
			  centroTest = findOneWhere(Centro.class, "id", Long.parseLong(idCentroTest));
		  }
		  catch (NoResultException nre) {
			  logger.log(Level.WARNING, "SelectFilterByCenterAux.getCentroTest: Centro Test no encontrado. " + nre.getLocalizedMessage());
			  
			  nre.printStackTrace();
		  }
		  catch (Exception e) {
			  logger.log(Level.WARNING, "SelectFilterByCenterAux.getCentroTest: Error leyendo Centro Test de la base de datos. " + e.getLocalizedMessage());
			  
			  e.printStackTrace();
		  }
		  
		  return centroTest;
	  }
	  
	  /* (non-Javadoc)
	   * @see es.csa.fdt.cyldigital.servicios.service.ActividadExternaService#getCentrosDisponibles()
	   */
	  public List<Centro> getCentrosDisponibles() {
	    if ((this.centrosDisponibles == null) || this.centrosDisponibles.isEmpty()) {
	    	if (this.centrosDisponibles == null)
	    		this.centrosDisponibles = new ArrayList<Centro>();
	    	
		    try{
		    	EntityManager em = getEntityManager();
		    	
		        Query prequerycolaborador = em.createQuery("SELECT e " +
		        										   "FROM " + Parameter.class.getName() + " e " + 
		        										   "WHERE (e.nombre LIKE 'extern_default_center_id')");
		        Parameter idcentrocolaborador = (Parameter) prequerycolaborador.getSingleResult();

		        Query prequeryadmin = em.createQuery("SELECT e " +
		        									 "FROM " + Parameter.class.getName() + " e " +
		        									 "WHERE (e.nombre LIKE 'admin_default_center_id')");
	        	Parameter idcentroadmin = (Parameter) prequeryadmin.getSingleResult();
		        
		        Query query = em.createQuery("SELECT e " +
		        							 "FROM " + Centro.class.getName() + " e " +
		        							 "WHERE (e.desactivado = false) AND " +
		        							 	   "(e.id <> :idCentroColaboradores) AND " +
		        							 	   "(e.id <> :idCentroAdmin) " + 
		        							 "ORDER BY e.nombre ASC");
		        
		        query.setParameter("idCentroColaboradores", Long.parseLong(idcentrocolaborador.getValor()));
		        query.setParameter("idCentroAdmin", Long.parseLong(idcentroadmin.getValor()));
		        
		        this.centrosDisponibles = (List<Centro>) query.getResultList();
		    }
		    catch (NoResultException nre) {

		    }
		    catch (Exception e) {
				logger.log(Level.WARNING, "BrowserFilteredByCenter.getCentrosDisponibles: Error al consultar los centros disponibles." + e.getLocalizedMessage());

				e.printStackTrace();
			}
	    }
	    
	    return this.centrosDisponibles;
	  }
}
