package es.csa.fdt.cyldigital.util;

/*
 * #%L
 * Librerías del núcleo - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.List;

import es.csa.commons.persistence.browser.Sort;

/**
 * Clase SortJPAQLExtended.
 * 
 * @author csa
 */
public class SortJPAQLExtended implements Sort<String> {

  private static final long serialVersionUID = 2646531606944284784L;

  // Provided data

  /** Si está ordenado ascendente */
  public boolean sortAsc;

  private List<Boolean> defaultSorts;

  private List<String> defaultProperties;

  /** Propiedad de ordenación */
  public String sortProperty;

  /** El criterion de hibernate. */
  protected transient String sortOrder;

  /**
   * Construye un nuevo objeto de la clase
   */
  public SortJPAQLExtended() {
    super();
    sortAsc = true;
    sortOrder = null;
    defaultSorts = new ArrayList<Boolean>();
    defaultProperties = new ArrayList<String>();
  }

  /**
   * Devuelve default sorts.
   * 
   * @return default sorts
   */
  public List<Boolean> getDefaultSorts() {
    return defaultSorts;
  }

  /**
   * Asigna default sorts.
   * 
   * @param sorts
   *          default sorts
   */
  public void setDefaultSorts(List<Boolean> sorts) {
    this.defaultSorts = sorts;
  }

  /**
   * Devuelve default properties.
   * 
   * @return default properties
   */
  public List<String> getDefaultProperties() {
    return defaultProperties;
  }

  /**
   * Asigna default properties.
   * 
   * @param properties
   *          default properties
   */
  public void setDefaultProperties(List<String> properties) {
    this.defaultProperties = properties;
  }

  /* (non-Javadoc)
   * @see es.csa.commons.persistence.browser.Sort#getSortAsc()
   */
  @Override
  public boolean getSortAsc() {
    return sortAsc;
  }

  /* (non-Javadoc)
   * @see es.csa.commons.persistence.browser.Sort#setSortAsc(boolean)
   */
  @Override
  public void setSortAsc(boolean sortAsc) {
    sortOrder = null;
    this.sortAsc = sortAsc;
  }

  /* (non-Javadoc)
   * @see es.csa.commons.persistence.browser.Sort#getSortProperty()
   */
  @Override
  public String getSortProperty() {
    return sortProperty;
  }

  /* (non-Javadoc)
   * @see es.csa.commons.persistence.browser.Sort#setSortProperty(java.lang.String)
   */
  @Override
  public void setSortProperty(String sortProperty) {
    sortOrder = null;
    this.sortProperty = sortProperty;
  }

  /* (non-Javadoc)
   * @see es.csa.commons.persistence.browser.Sort#getOrder()
   */
  @Override
  public String getOrder() {
    if (sortProperty == null && getDefaultProperties().isEmpty()) return null;
    else if (sortProperty == null) sortOrder = "ORDER BY " + getDefaultOrder(false);
    else sortOrder = "ORDER BY " + sortProperty + (getSortAsc() ? " ASC" : " DESC") + getDefaultOrder(true);
    return sortOrder;
  }

  private String getDefaultOrder(boolean previous) {
    String order = previous ? " , " : " ";
    for (int i = 0; i < getDefaultProperties().size(); i++) {
      if (i != 0) order += " , ";
      order += getDefaultProperties().get(i) + (getDefaultSorts().get(i) ? " ASC " : " DESC ");
    }
    return order;
  }
}
