package es.csa.fdt.cyldigital.integracion;

/*
 * #%L
 * Servicios Integrados para aplicaciones remotas - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import static com.lowagie.text.Font.BOLD;
import static com.lowagie.text.Font.DEFAULTSIZE;
import static com.lowagie.text.Font.ITALIC;
import static com.lowagie.text.Font.NORMAL;
import static com.lowagie.text.pdf.PdfContentByte.ALIGN_CENTER;
import static com.lowagie.text.pdf.PdfContentByte.ALIGN_LEFT;

import java.awt.Color;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.text.DateFormat;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.servlet.http.HttpServletResponse;

import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

import com.lowagie.text.Font;
import com.lowagie.text.FontFactory;
import com.lowagie.text.pdf.BaseFont;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfReader;
import com.lowagie.text.pdf.PdfStamper;

import es.csa.fdt.cyldigital.servicios.entity.CursoOnline;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCursoOnline;
import es.csa.fdt.cyldigital.servicios.service.MatriculaCursoOnlineService;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioOnline;

/**
 * Clase DiplomaOnlineControllerImpl.
 */
@Controller
public class DiplomaOnlineControllerImpl {

  private static final Logger logger = LoggerFactory.getLogger(DiplomaOnlineControllerImpl.class);

  private static final long serialVersionUID = -4581725741365803463L;

  private static Map<String, Object[]> CARACTERISTICAS_DIPLOMA;

  private static int MAX_WIDTH = 650;

  public static final String DIPLOMA_PDF = "es/csa/fdt/cyldigital/pdf/diploma_fondo.pdf";

  public static final String DIPLOMA_PDF_VACIO = "es/csa/fdt/cyldigital/pdf/diploma_vacio.pdf";

  public static final Font ARIAL_NORMAL = FontFactory.getFont("es/csa/fdt/cyldigital/pdf/Arial.ttf", Font.BOLD);

  public static final Font HELVETICA_NORMAL = FontFactory.getFont(FontFactory.HELVETICA, DEFAULTSIZE, NORMAL);

  public static final Font HELVETICA_BOLD = FontFactory.getFont(FontFactory.HELVETICA, DEFAULTSIZE, BOLD);

  public static final Font HELVETICA_I = FontFactory.getFont(FontFactory.HELVETICA, DEFAULTSIZE, ITALIC);

  public static final Font COURIER_NORMAL = FontFactory.getFont(FontFactory.COURIER, DEFAULTSIZE, NORMAL);

  @Autowired
  private MatriculaCursoOnlineService serviceCursoOnline;

  @Autowired
  public es.csa.commons.faces.util.PredefinedLocaleResolver prl;

  static {

    Object[][] posicionesDiploma = { { HELVETICA_BOLD, 22.0f, ALIGN_CENTER, 425f, 335f, null }, // nombreUsuario
        { HELVETICA_NORMAL, 16f, ALIGN_LEFT, 110f, 303f, null }, // descripcion
        { HELVETICA_I, 12.5f, ALIGN_CENTER, 417f, 215.5f, null }, // fecha
    };
    CARACTERISTICAS_DIPLOMA = new HashMap<String, Object[]>();
    CARACTERISTICAS_DIPLOMA.put("nombreUsuario", posicionesDiploma[0]);
    CARACTERISTICAS_DIPLOMA.put("descripcion", posicionesDiploma[1]);
    CARACTERISTICAS_DIPLOMA.put("fecha", posicionesDiploma[2]);

  }

  /**
   * Construye un nuevo objeto de la clase
   */
  public DiplomaOnlineControllerImpl() {}

  /**
   * Stream diploma curso online.
   * 
   * @param idWeb
   *           id web
   * @param idCurso
   *           id curso
   * @param identificacion
   *           identificacion
   * @param response
   *           response
   * @throws IOException
   *           Signals that an I/O exception has occurred.
   */
  @RequestMapping("/servirDiplomaCursoOnline")
  @Transactional(readOnly = true)
  public void streamDiplomaCursoOnline(@RequestParam("idWeb") String idWeb, @RequestParam("idCurso") String idCurso,
      @RequestParam("nombre") String identificacion, HttpServletResponse response) throws IOException {

    // busco el usuario online
    Map<String, Object> propiedades = new HashMap<String, Object>();
    propiedades.put("idWeb", idWeb);
    List<UsuarioOnline> lu = serviceCursoOnline.findWhere(UsuarioOnline.class, propiedades);
    UsuarioOnline u;
    if (lu != null && !lu.isEmpty()) {
      u = lu.get(0);
      CursoOnline c = serviceCursoOnline.find(CursoOnline.class, Long.parseLong(idCurso));
      if (c != null) {
        MatriculaCursoOnline m = null;
        for (MatriculaCursoOnline matriculaCurso : c.getMatriculas()) {
          if (matriculaCurso.getUsuario().getId() == u.getId()) {
            m = matriculaCurso;
          }
        }
        if (identificacion == null || identificacion.isEmpty()) {
          identificacion = " Sin identificacion ";
        }

        if (m != null) {
          try {
            makeHeader(response, "Diploma - " /*+ m.getCurso().getServicio().getNombre().replace("\"", "'")*/ + " -ID: "
                + m.getUsuario().getId() + " " + identificacion);

            IOUtils.copy(new ByteArrayInputStream(getPdfDiplomaCursoOnline(m, u, DIPLOMA_PDF, identificacion)
                .toByteArray()), response.getOutputStream());

          } catch (RuntimeException e) {
            e.printStackTrace();
          }
        }
      }
    }
  }

  /**
   * Devuelve pdf diploma curso online.
   * 
   * @param matricula
   *           matricula
   * @param u
   *           u
   * @param pdf
   *           pdf
   * @param nombre
   *           nombre
   * @return the pdf diploma curso online
   */
  public ByteArrayOutputStream getPdfDiplomaCursoOnline(MatriculaCursoOnline matricula, UsuarioOnline u, String pdf,
      String nombre) {
    ByteArrayOutputStream pdf_diploma = new ByteArrayOutputStream();
    try {
      PdfReader reader;
      reader = new PdfReader(pdf);
      pdf_diploma = generarPDF(CARACTERISTICAS_DIPLOMA, getDatosDiplomaCursoOnline(matricula, u, nombre), reader);
    } catch (IOException e) {
      logger.error("Error al generar diploma de curso online, no se encuentra la plantilla de pdf");
    }
    return pdf_diploma;
  }

  private void makeHeader(HttpServletResponse response, String nombre) {
    response.setContentType("application/pdf");
    response.setHeader("Cache-Control", "no-store, no-cache, must-revalidate");
    response.addHeader("Cache-Control", "post-check=0, pre-check=0");
    response.setHeader("Pragma", "no-cache");
    response.setCharacterEncoding("ISO-8859-15");
    response.setHeader("Content-Disposition", "attachment;filename=\"" + nombre + ".pdf\"");
  }

  /**
   * Devuelve datos diploma curso online.
   * 
   * @param matricula
   *           matricula
   * @param u
   *           u
   * @param nombre
   *           nombre
   * @return the datos diploma curso online
   */
  public Map<String, Object> getDatosDiplomaCursoOnline(MatriculaCursoOnline matricula, UsuarioOnline u, String nombre) {
    Map<String, Object> datos = new HashMap<String, Object>();
    Locale locale = prl.getLocale();
    DateFormat df1 = new SimpleDateFormat("dd", locale);
    DateFormat df2 = new SimpleDateFormat("MMMM", locale);
    DateFormat df3 = new SimpleDateFormat("yyyy", locale);

    String nombreUsuario = "D/DÑA  " + nombre;
    String descripcion = "";

    descripcion += "Ha realizado el curso \"";
    descripcion += /*matricula.getCurso().getServicio().getNombre().replace("\"", "'") +*/ "\", desde el ";
    descripcion += matricula.getCurso().getFechaInicioLegible();
    descripcion += " hasta el ";
    descripcion += matricula.getCurso().getFechaFinLegible()
        + ", impartido por la Fundación para el Desarrollo Infotecnológico de Empresas y Sociedad (FUNDETEC), a través de la Plataforma de Formación Online.";
    String fecha = df1.format(matricula.getCurso().getFechaFin()) + " de "
        + df2.format(matricula.getCurso().getFechaFin()) + " de " + df3.format(matricula.getCurso().getFechaFin());

    datos.put("nombreUsuario", nombreUsuario);
    datos.put("descripcion", descripcion.replaceAll("\\.0", ""));
    datos.put("fecha", fecha);
    return datos;
  }

  /**
   * Generar pdf.
   * 
   * @param titulos
   *           titulos
   * @param datos
   *           datos
   * @param reader
   *           reader
   * @return the byte array output stream
   */
  public static ByteArrayOutputStream generarPDF(Map<String, Object[]> titulos, Map<String, Object> datos,
      PdfReader reader) {
    ByteArrayOutputStream result = new ByteArrayOutputStream();
    try {

      PdfStamper stamper = new PdfStamper(reader, result);

      int i = 1;
      PdfContentByte over = stamper.getOverContent(i);

      for (String e : titulos.keySet()) {
        Object[] obj = titulos.get(e);
        String dato = null;

        if (datos.get(e) == null || datos.get(e).toString().isEmpty()) dato = null;
        else {
          if (e.equals("duracion")) {
            NumberFormat numberInstance = NumberFormat.getNumberInstance();
            numberInstance.setMaximumFractionDigits(2);
            numberInstance.setMinimumFractionDigits(0);
            dato = numberInstance.format(datos.get(e));
          }
        }

        if (dato == null) dato = datos.get(e) == null ? "" : datos.get(e).toString();

        over.beginText();
        over.setFontAndSize(((Font) obj[0]).getBaseFont(), ((Float) obj[1]));
        if (obj[5] != null) over.setColorFill(((Color) obj[5]));

        String cortado = "";
        int j = 0;
        if (((Font) obj[0]).getBaseFont().getWidthPoint(dato, ((Float) obj[1])) > MAX_WIDTH) {
          cortado = getSPlitedString(((Font) obj[0]).getBaseFont(), dato, ((Float) obj[1]), cortado, j);
          dato = dato.substring(cortado.length(), dato.length());
          over.showTextAligned(((Integer) obj[2]), cortado, ((Float) obj[3]), ((Float) obj[4]), 0);
          over.endText();
          j++;
          while (!dato.equals("")) {
            over.beginText();
            over.setFontAndSize(((Font) obj[0]).getBaseFont(), ((Float) obj[1]));
            if (obj[5] != null) over.setColorFill(((Color) obj[5]));
            cortado = getSPlitedString(((Font) obj[0]).getBaseFont(), dato, ((Float) obj[1]), cortado, j);
            dato = dato.substring(cortado.length(), dato.length());
            over.showTextAligned(((Integer) obj[2]), cortado, ((Float) obj[3]), ((Float) obj[4]) - (j * 22), 0);
            over.endText();
            j++;
          }
        } else {
          over.showTextAligned(((Integer) obj[2]), dato, ((Float) obj[3]), ((Float) obj[4]), 0);
          over.endText();
        }
      }
      stamper.close();
    } catch (Exception e) {
      e.printStackTrace();
    }
    return result;
  }

  private static String getSPlitedString(BaseFont baseFont, String dato, Float float1, String cortado, int j) {
    String aux = "" + dato;
    if (baseFont.getWidthPoint(dato, float1) > MAX_WIDTH) {
      while (baseFont.getWidthPoint(aux, float1) > MAX_WIDTH) {
        aux = aux.substring(0, aux.length() - 1);
      }
      while (!aux.equals("") && !aux.substring(aux.length() - 1, aux.length()).equals(" ")) {
        aux = aux.substring(0, aux.length() - 1);
      }
      cortado = aux + "";
      return cortado;
    } else if (dato.length() > 0) {
      cortado = dato;
      return cortado;
    } else {
      return "";
    }
  }
}
