package es.csa.fdt.cyldigital.integracion;

/*
 * #%L
 * Servicios Integrados para aplicaciones remotas - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


//import com.sun.xml.messaging.saaj.soap.impl.ElementImpl;
//import com.sun.xml.messaging.saaj.soap.impl.TextImpl;

import java.lang.Integer;
import java.util.Map;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.servlet.http.HttpServletRequest;

import org.apache.cxf.binding.soap.SoapMessage;
import org.apache.cxf.interceptor.Fault;
import org.apache.cxf.ws.security.wss4j.WSS4JInInterceptor;

import org.apache.ws.security.WSConstants;
import org.apache.ws.security.WSSecurityEngineResult;
import org.apache.ws.security.WSUsernameTokenPrincipal;
import org.apache.ws.security.handler.WSHandlerConstants;
import org.apache.ws.security.handler.WSHandlerResult;
//import org.apache.ws.security.message.token.Timestamp;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.security.Authentication;
import org.springframework.security.AuthenticationManager;
import org.springframework.security.context.SecurityContextHolder;
import org.springframework.security.providers.UsernamePasswordAuthenticationToken;
import org.springframework.util.Assert;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

/**
 * Clase WSAuthenticationInInterceptor.
 */
public class WSAuthenticationInInterceptor extends WSS4JInInterceptor implements InitializingBean {

  private static final Logger logger = Logger.getLogger(WSAuthenticationInInterceptor.class.getName());

  private AuthenticationManager authenticationManager;

  /**
   * Construye un nuevo objeto de la clase
   */
  public WSAuthenticationInInterceptor() {
    super();
  }

  /**
   * Construye un nuevo objeto de la clase
   * 
   * @param properties
   *           properties
   */
  public WSAuthenticationInInterceptor(Map<String, Object> properties) {
    super(properties);
  }

  /* (non-Javadoc)
   * @see org.springframework.beans.factory.InitializingBean#afterPropertiesSet()
   */
  @Override
  public void afterPropertiesSet() throws Exception {
    Assert.notNull(getAuthenticationManager(), "Authentication manager must be set");
    Assert.notNull(getProperties(), "Interceptor properties must be set, even if empty");
  }

  /**
   * Devuelve authentication manager.
   * 
   * @return the authenticationManager
   */
  public AuthenticationManager getAuthenticationManager() {
    return authenticationManager;
  }

  /**
   * Asigna authentication manager.
   * 
   * @param authenticationManager
   *           authenticationManager to set
   */
  public void setAuthenticationManager(AuthenticationManager authenticationManager) {
    this.authenticationManager = authenticationManager;
  }

  /* (non-Javadoc)
   * @see org.apache.cxf.ws.security.wss4j.WSS4JInInterceptor#handleMessage(org.apache.cxf.binding.soap.SoapMessage)
   */
  @Override
  public void handleMessage(SoapMessage message) throws Fault {
	  try {
		  super.handleMessage(message);
      
		  Vector<WSHandlerResult> result = (Vector<WSHandlerResult>)message.getContextualProperty(WSHandlerConstants.RECV_RESULTS);
      
		  if (result != null && !result.isEmpty()) {
			  Integer timeLimitInSeconds = -1;
			  
			  for (WSHandlerResult res : result) {
				  // loop through security engine results
				  for (WSSecurityEngineResult securityResult : (Vector<WSSecurityEngineResult>) res.getResults()) {
					  int action = (Integer) securityResult.get(WSSecurityEngineResult.TAG_ACTION);
					  
					  // determine if the action was a username token
					  if ((action & WSConstants.UT) > 0) {
						  // get the principal object
						  WSUsernameTokenPrincipal principal = (WSUsernameTokenPrincipal)securityResult.get(WSSecurityEngineResult.TAG_PRINCIPAL);
						  
						  if (principal.getPassword() == null)
							  principal.setPassword("");

						  Authentication authentication = new UsernamePasswordAuthenticationToken(principal.getName(), principal.getPassword());
						  authentication = authenticationManager.authenticate(authentication);
						  
						  if (!authentication.isAuthenticated()) {
							  logger.log(Level.INFO, "El usuario no ha sido autenticado.");

							  //throw new Fault("El usuario no ha sido autenticado.");
						  }
						  
						  SecurityContextHolder.getContext().setAuthentication(authentication);
					  }
					  /* 
					   * Para action Timestamp
					   * 
					   * else if (action == WSConstants.TS) {
						  Timestamp ts = (Timestamp)securityResult.get(WSSecurityEngineResult.TAG_TIMESTAMP);
						  
						  if ((ts != null) &&
							  (ts.getCustomElements() != null) &&
						      (ts.getCustomElements().firstElement() != null) &&
						      (ts.getCustomElements().firstElement() instanceof ElementImpl)) {
							  ElementImpl customElement = (ElementImpl)ts.getCustomElements().firstElement();

							  if ((customElement.getFirstChild() != null) &&
							      (customElement.getFirstChild() instanceof TextImpl))
								  timeLimitInSeconds = Integer.valueOf(((TextImpl)(customElement.getFirstChild())).getValue());
							  
							  logger.log(Level.INFO, "timeLimitInSeconds = " + timeLimitInSeconds);
						  }
					  }*/
				  }
			  }
		  }
		  else {
			  logger.log(Level.INFO, "No se han recibido credenciales");
		  }
	  } catch (RuntimeException ex) {
		  logger.log(Level.SEVERE, ex.getMessage(), ex);
		  
		  throw ex;
	  }
  }
}
