package es.csa.fdt.cyldigital.integracion;

import java.text.SimpleDateFormat;

/*
 * #%L
 * Servicios Integrados para aplicaciones remotas - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jws.WebMethod;
import javax.jws.WebService;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Query;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.security.annotation.Secured;
import org.springframework.security.context.SecurityContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import es.csa.fdt.cyldigital.core.entity.File;
import es.csa.fdt.cyldigital.core.service.ParameterServiceImpl;
import es.csa.fdt.cyldigital.evaluacion.entity.Examen;
import es.csa.fdt.cyldigital.evaluacion.entity.Formulario;
import es.csa.fdt.cyldigital.integracion.integracionCeMIT.TipoActividad;
import es.csa.fdt.cyldigital.language.entity.CharlaLanguage;
import es.csa.fdt.cyldigital.language.entity.CursoLanguage;
import es.csa.fdt.cyldigital.language.entity.ExamenLanguage;
import es.csa.fdt.cyldigital.language.entity.FormularioLanguage;
import es.csa.fdt.cyldigital.language.entity.ServicioCursoLanguage;
import es.csa.fdt.cyldigital.language.entity.TematicaLanguage;
import es.csa.fdt.cyldigital.recursos.service.RemoteNoticeService;
import es.csa.fdt.cyldigital.servicios.entity.Charla;
import es.csa.fdt.cyldigital.servicios.entity.Curso;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCharla;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso;
import es.csa.fdt.cyldigital.servicios.entity.ServicioCurso;
import es.csa.fdt.cyldigital.servicios.entity.Tematica;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;
import es.csa.fdt.cyldigital.usuarios.service.UsuarioService;
import es.csa.fdt.cyldigital.usuarios.service.UsuarioServiceImpl;

/**
 * Clase integracionAuthCeMIT.
 */
@Component("integracionAuthCeMIT")
@WebService
@Secured(value = {"ROLE_USER"})
public class integracionAuthCeMIT {
	private static final Logger logger = Logger.getLogger(integracionAuthCeMIT.class.getName());

	@Autowired
	private RemoteNoticeService service;
	
	@Autowired
	ApplicationContext appContext;
	
	private UsuarioServiceImpl usuarioServiceImpl = new UsuarioServiceImpl();
	
	@WebMethod
	@Transactional
	public WSUsuarioCeMIT AutenticaUsuarioCeMIT() {

		long startTime = System.currentTimeMillis();

		UsuarioLdap uldap = (UsuarioLdap)SecurityContextHolder.getContext().getAuthentication().getPrincipal();

		long elapsedTime = System.currentTimeMillis() - startTime;
		logger.log(Level.INFO, "Webservice AutenticaUsuarioCeMIT request - " + elapsedTime + "ms");
		
		// 3.4.8 - RF_15_37 Usuarios activos
      	if ( uldap.getUser() instanceof UsuarioCyL) {
			UsuarioCyL ucyl = (UsuarioCyL) uldap.getUser();
			usuarioServiceImpl.persistUltimoAcceso(ucyl);
			EntityManagerFactory emf = (EntityManagerFactory)appContext.getBean("entityManagerFactory", EntityManagerFactory.class);
    	  	EntityManager em = emf.createEntityManager();
    	  	
    	  	ParameterServiceImpl parameterService = new ParameterServiceImpl();
			parameterService.setEntityManager(em);
			
			if (parameterService.findOneWhere("nombre", "users_change_ldap_logontime").getValor().equals("1")){
				UsuarioService usuarioService = (UsuarioService) appContext.getBean("usuarioServiceImpl");
				usuarioService.updateLdapLogonTime(ucyl);
			}
		}
		
		return crearWSUsuarioCeMIT(uldap);
	}
	
	/**
	 * Método web que busca todas las actividades asociadas al usuario especificado en la cabecera de atenticación.
	 *
	 * @param languageISO
	 * 			Identificador ISO del idioma en el que se desean los resultados ('gl_ES' o 'es_ES').
	 * 
	 * @param fieldOrder
	 * 			(opcional)
	 * 			Nombre del campo del objeto Provincia por el que se desea ordenar.
	 * 			"" o null indica que no se desea ordenar.
	 * 
	 * @param orderASC
	 * 			(opcional)
	 * 			Indica en que sentido se desea ordenar el resultado. Si fieldOrder es "" o null este parámetro es ignorado.
	 *			Si no se especifica, se toma por defecto orden ascendente.
	 * 			true indica ascendente y false descendente. 
	 *
	 * @param recordsByPage
	 * 			(opcional)
	 *          Número de registros a devolver.  
	 *          <= 0 indica devolver todos los registros.
	 *           
	 * @param page
	 * 			(opcional)
	 *          Número de página a devolver. Sólo si el parámetro recordsByPage es > 0.
	 *          En el caso de especificar un número <= 0 se ignora el parámetro recordsByPage y se devuelven todos los registros.
	 *          1 es la primera página.
	 * 
	 * @return listado con todas las actividades asociadas al usuario especificado en la cabecera de atenticación.
	 */
	@WebMethod
	@Transactional
	public List<WSActividadUsuarioCeMIT> BuscaActividadesUsuarioCeMIT(String languageISO, String fieldOrder, Boolean orderASC,
																	  Integer recordsByPage, Integer page) {

		long startTime = System.currentTimeMillis();

		List<WSActividadUsuarioCeMIT> lwsaucemit = new ArrayList<WSActividadUsuarioCeMIT>();
		
		UsuarioLdap uldap = (UsuarioLdap)SecurityContextHolder.getContext().getAuthentication().getPrincipal();
		Long idUsuario = uldap.getUser().getId();
		
		Locale language = new Locale(languageISO.split("_")[0], languageISO.split("_")[1]);

		String strqry = "SELECT c, cl, sc, scl, tsc, tscl, mc " +
						"FROM " + Curso.class.getName() + " c INNER JOIN " + 
							 "c.textosCurso cl INNER JOIN " +
							 "cl.language lc WITH (lc.ISO = :languageISO) INNER JOIN " +
							 "c.servicio sc INNER JOIN " +
							 "sc.textos scl INNER JOIN " +
							 "scl.language lsc WITH (lsc.ISO = :languageISO) INNER JOIN " +
							 "sc.tematica tsc INNER JOIN " +
							 "tsc.textos tscl INNER JOIN " +
							 "tscl.language ltsc WITH (ltsc.ISO = :languageISO) INNER JOIN " +
							 "c.matriculas mc FETCH ALL PROPERTIES WITH (((mc.preMatriculadoWeb = true) AND " +
							 											 "(mc.enListaEspera     = true) AND " +
							 											 "(mc.admitido          = false)) OR " +
							 											"((mc.enListaEspera     = false) AND " +
							 											 "(mc.admitido          = true))) INNER JOIN " +
							 "mc.usuario u WITH (u.id = :idUsuario)";
		
		try {
			Query command = service.getEntityManager().createQuery(strqry);
			
			command.setParameter("languageISO", languageISO);
			command.setParameter("idUsuario", idUsuario);
			
			@SuppressWarnings("unchecked")
			List<Object[]> lcuu = (List<Object[]>)command.getResultList();
			
			for (Object[] lo : lcuu)
				lwsaucemit.add(crearWSActividadUsuarioCeMIT((Curso)lo[0], (CursoLanguage)lo[1], (ServicioCurso)lo[2],
															(ServicioCursoLanguage)lo[3], (Tematica)lo[4], (TematicaLanguage)lo[5],
															(MatriculaCurso)lo[6], language));
			
			lcuu.clear();
		}
		catch (Exception e) {
			logger.log(Level.SEVERE,
					   "BuscaCentrosCeMIT: Error al devolver la búsqueda de centros CeMIT: " +
					   e.getLocalizedMessage());
		
			e.printStackTrace();
		}
		
		strqry = "SELECT c, cl, tc, tcl, mc " +
				 "FROM " + Charla.class.getName() + " c INNER JOIN " + 
				 	  "c.textosCharla cl INNER JOIN " +
				 	  "cl.language lc WITH (lc.ISO = :languageISO) INNER JOIN " +
				 	  "c.tematica tc INNER JOIN " +
				 	  "tc.textos tcl INNER JOIN " +
				 	  "tcl.language ltc WITH (ltc.ISO = :languageISO) INNER JOIN " +
				 	  "c.matriculas mc FETCH ALL PROPERTIES WITH (((mc.preMatriculadoWeb = true) AND " +
				 	  											  "(mc.enListaEspera     = true) AND " +
				 	  											  "(mc.admitido          = false)) OR " +
				 	  											 "((mc.enListaEspera     = false) AND " +
				 	  											  "(mc.admitido          = true))) INNER JOIN " +
				 	  "mc.usuario u WITH (u.id = :idUsuario)";

		try {
			Query command = service.getEntityManager().createQuery(strqry);
			
			command.setParameter("languageISO", languageISO);
			command.setParameter("idUsuario", idUsuario);
			
			@SuppressWarnings("unchecked")
			List<Object[]> lchu = (List<Object[]>)command.getResultList();
			
			for (Object[] lo : lchu)
				lwsaucemit.add(crearWSActividadUsuarioCeMIT((Charla)lo[0], (CharlaLanguage)lo[1],
															(Tematica)lo[2], (TematicaLanguage)lo[3],
															(MatriculaCharla)lo[4], language));
			
			lchu.clear();
		}
		catch (Exception e) {
			logger.log(Level.SEVERE,
					   "BuscaCentrosCeMIT: Error al devolver la búsqueda de centros CeMIT: " +
					   e.getLocalizedMessage());
		
			e.printStackTrace();
		}
		
		if (fieldOrder != null) {
			if (!fieldOrder.isEmpty()) {
				if (orderASC == null)
					orderASC = true;
				
				if (fieldOrder.compareTo("idActividad") == 0) {
					if (orderASC)
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorIdActividadASC);
					else
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorIdActividadDESC);
				}
				else if (fieldOrder.compareTo("idCentro") == 0) {
					if (orderASC)
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorIdCentroASC);
					else
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorIdCentroDESC);
				}
				else if (fieldOrder.compareTo("nombreCentro") == 0) {
					if (orderASC)
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorNombreCentroASC);
					else
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorNombreCentroDESC);
				}
				else if (fieldOrder.compareTo("tipo") == 0) {
					if (orderASC)
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorTipoASC);
					else
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorTipoDESC);
				}
				else if (fieldOrder.compareTo("idTematica") == 0) {
					if (orderASC)
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorIdTematicaASC);
					else
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorIdTematicaDESC);
				}
				else if (fieldOrder.compareTo("tematica") == 0) {
					if (orderASC)
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorTematicaASC);
					else
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorTematicaDESC);
				}
				else if (fieldOrder.compareTo("nivel") == 0) {
					if (orderASC)
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorNivelASC);
					else
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorNivelDESC);
				}
				else if (fieldOrder.compareTo("nombre") == 0) {
					if (orderASC)
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorNombreASC);
					else
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorNombreDESC);
				}
				else if (fieldOrder.compareTo("descripcion") == 0) {
					if (orderASC)
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorDescripcionASC);
					else
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorDescripcionDESC);
				}
				else if (fieldOrder.compareTo("estado") == 0) {
					if (orderASC)
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorEstadoASC);
					else
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorEstadoDESC);
				}
				else if (fieldOrder.compareTo("fechaEstado") == 0) {
					if (orderASC)
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorFechaEstadoASC);
					else
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorFechaEstadoDESC);
				}
				else if (fieldOrder.compareTo("horaEstado") == 0) {
					if (orderASC)
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorHoraEstadoASC);
					else
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorHoraEstadoDESC);
				}
				else if (fieldOrder.compareTo("fechaInicio") == 0) {
					if (orderASC)
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorFechaInicioASC);
					else
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorFechaInicioDESC);
				}
				else if (fieldOrder.compareTo("fechaFin") == 0) {
					if (orderASC)
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorFechaFinASC);
					else
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorFechaFinDESC);
				}
				else if (fieldOrder.compareTo("numeroHoras") == 0) {
					if (orderASC)
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorNumeroHorasASC);
					else
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorNumeroHorasDESC);
				}
				else if (fieldOrder.compareTo("asistencia") == 0) {
					if (orderASC)
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorAsistenciaASC);
					else
						Collections.sort(lwsaucemit, WSActividadUsuarioCeMIT.comparatorAsistenciaDESC);
				}
			}
		}
		
		if ((recordsByPage != null) && (page != null)) {
			if ((recordsByPage > 0) && (page > 0)) {
				int firstIndex = (page - 1) * recordsByPage;
				int lastIndex = (page * recordsByPage);
				
				if (firstIndex < lwsaucemit.size()) {
					lastIndex = (lastIndex <= lwsaucemit.size()) ? lastIndex : lwsaucemit.size();
					
					return lwsaucemit.subList(firstIndex, lastIndex);
				}
				else
					lwsaucemit.clear();
			}
		}

		long elapsedTime = System.currentTimeMillis() - startTime;
		logger.log(Level.INFO, "Webservice BuscaActividadesUsuarioCeMIT request - " + elapsedTime + "ms");

		return lwsaucemit;
	}

	/**
	 * Método web que busca las actividades asociadas a los parámetros de búsqueda especificados exceptuando las de los mismos
	 * servicios ya realizadas (en el caso de cursos), las ya superadas (>= 80% de asistencia), en las que ya está matriculado
	 * y las que ya ha solicitado.
	 *
	 * @param idCentro
	 * 			Identificador del centro.
	 * 
	 * @param idTipologia1
	 * 			Primer identificador de tipología
	 * 
	 * @param idTipologia2
	 * 			Segundo identificador de tipología
	 *  
	 * @param idTipologia3
	 * 			Tercer identificador de tipología
	 * 
	 * @param languageISO
	 * 			Identificador ISO del idioma en el que se desean los resultados ('gl_ES' o 'es_ES').
	 * 
	 * @param fieldOrder
	 * 			(opcional)
	 * 			Nombre del campo del objeto Provincia por el que se desea ordenar.
	 * 			"" o null indica que no se desea ordenar.
	 * 
	 * @param orderASC
	 * 			(opcional)
	 * 			Indica en que sentido se desea ordenar el resultado. Si fieldOrder es "" o null este parámetro es ignorado.
	 *			Si no se especifica, se toma por defecto orden ascendente.
	 * 			true indica ascendente y false descendente. 
	 *
	 * @param recordsByPage
	 * 			(opcional)
	 *          Número de registros a devolver.  
	 *          <= 0 indica devolver todos los registros.
	 *           
	 * @param page
	 * 			(opcional)
	 *          Número de página a devolver. Sólo si el parámetro recordsByPage es > 0.
	 *          En el caso de especificar un número <= 0 se ignora el parámetro recordsByPage y se devuelven todos los registros.
	 *          1 es la primera página.
	 * 
	 * @return listado con todas las actividades asociadas a los parámetros de búsqueda especificados exceptuando las de los mismos
	 * 		   servicios ya realizadas (en el caso de cursos), las ya superadas (>= 80% de asistencia), en las que ya está matriculado
	 * 		   y las que ya ha solicitado.
	 * 
	 * 		   Los filtros funcionan de la siguiente manera: 
	 * 		   · idCentro <= 0 Indica que se devolverán las actividades de los centros de la provincia del centro del usuario.
	 * 
	 * 		   · idCentro > 0 Indica el indicador de un centro y se devolverán las actividades planificadas en dicho centro.
	 * 
	 * 		   · idCentro = null Indica que se devolverán las actividades planificadas en el centro del usuario.
	 * 
	 * 		   · (idTipologia1 <= 0) y (idTipologia2 <= 0) y (idTipologia3 <= 0) Indica que no se atiende a la tipología de las
	 * 			 actividades, se devolverán las actividades seleccionadas por el parámetro identificador del centro de cualquier
	 * 			 tipología.
	 * 
	 *		   · (idTipologia1 > 0) y (idTipologia2 <= 0) y (idTipologia3 <= 0) Indica que se devolverán las actividades seleccionadas
	 * 			 por el parámetro identificador del centro y que además pertenezcan a la tipología especificada por idTipologia1
	 * 
	 * 		   · (idTipologia1 > 0) y (idTipologia2 > 0) y (idTipologia3 <= 0) Indica que se devolverán las actividades seleccionadas
	 * 			 por el parámetro identificador del centro y que además pertenezcan a la tipología especificada por idTipologia1 e
	 * 			 idTipolgia2.
	 * 
	 * 		   · (idTipologia1 > 0) y (idTipologia2 > 0) y (idTipologia3 > 0) Indica que se devolverán las actividades seleccionadas
	 * 			 por el parámetro identificador del centro y que además pertenezcan a la tipología especificada por idTipologia1,
	 * 			 idTipologia2 e idTipologia3.
	 */
	@WebMethod
	@Transactional
	public List<WSActividadTipologiaCeMIT> BuscaActividadesTipologiaCeMIT(Long idCentro, Long idTipologia1, Long idTipologia2,
																		  Long idTipologia3, String languageISO, String fieldOrder,
																		  Boolean orderASC, Integer recordsByPage, Integer page) {

		long startTime = System.currentTimeMillis();

		List<WSActividadTipologiaCeMIT> lwsatcemit = new ArrayList<WSActividadTipologiaCeMIT>();
		
		//Parámetros mal conformados, devolvemos la lista vacía
		if (((idTipologia1 == null) || (idTipologia2 == null) || (idTipologia3 == null)))
			return lwsatcemit;
		
		UsuarioLdap uldap = (UsuarioLdap)SecurityContextHolder.getContext().getAuthentication().getPrincipal();
		Long idUsuario = uldap.getUser().getId();
		
		Locale language = new Locale(languageISO.split("_")[0], languageISO.split("_")[1]);
		Long idProvincia = null;

		String strqry = "SELECT c, cl, sc, scl, tsc, tscl " +
						"FROM " + Curso.class.getName() + " c INNER JOIN " +
							 "c.textosCurso cl INNER JOIN " +
							 "cl.language lc WITH (lc.ISO = :languageISO) INNER JOIN " +
							 "c.servicio sc INNER JOIN " +
							 "sc.textos scl INNER JOIN " +
							 "scl.language lsc WITH (lsc.ISO = :languageISO) INNER JOIN " +
							 "sc.tematica tsc INNER JOIN " +
							 "tsc.textos tscl INNER JOIN " +
							 "tscl.language ltsc WITH (ltsc.ISO = :languageISO) " +
						"WHERE (c.fechaFin > now()) AND " +
							  "(c.id NOT IN (SELECT DISTINCT mc.curso.id " +
											"FROM " + MatriculaCurso.class.getName() + " mc INNER JOIN " +
												 "mc.usuario u WITH (u.id = :idUsuario) " +
											"WHERE " + //Solicitado o matriculado y no iniciado
												  "((((mc.preMatriculadoWeb = true) AND " +
												     "(mc.enListaEspera     = true) AND " +
												     "(mc.admitido          = false)) OR " +
												    "((mc.enListaEspera     = false) AND " +
												     "(mc.admitido          = true))) AND " +
												   "(mc.curso.fechaFin >= now())) OR " +
												  //Matriculado, acabado y con asistencia mayor al 80%
												  "((mc.enListaEspera     = false) AND " +
												   "(mc.admitido          = true) AND " +
												   "(mc.curso.fechaFin < now()) AND " +
												   "(mc.asistencia >= 80)))) AND " +
							  //Servicios con actividades acabadas o no y con asistencia o no, a los que pertenecen cualquier actividad solicitada o matriculada
							  "(sc.id NOT IN (SELECT DISTINCT mc.curso.servicio.id " +
											 "FROM " + MatriculaCurso.class.getName() + " mc INNER JOIN " +
												  "mc.usuario u WITH (u.id = :idUsuario) " +
											 "WHERE " + //Solicitado
												   "(((mc.preMatriculadoWeb = true) AND " +
												     "(mc.enListaEspera     = true) AND " +
												     "(mc.admitido          = false)) OR " +
												   //Matriculado
												    "((mc.enListaEspera     = false) AND " +
												     "(mc.admitido          = true)))))";
		
		//Condiciones atendiendo al parámetro idCentro
		
		if ((idCentro == null) || (idCentro > 0)) {
			strqry += " AND (c.centro.id = :idCentro)";
			
			if (idCentro == null)
				idCentro = uldap.getUser().getCentro().getId();
		}
		else { //if (idCentro <= 0)
			idProvincia = uldap.getUser().getCentro().getProvincia().getId();
			strqry += " AND (c.centro.provincia.id = :idProvincia)";
		}
		
		//Condiciones atendiendo a los parámetros idTipologia1, idTipologia2, idTipologia3
		if ((idTipologia1 > 0) && (idTipologia2 <= 0) && (idTipologia3 <= 0)) {
			strqry += " AND (tsc.id = :idTipologia1)";
		}			
		else if ((idTipologia1 > 0) && (idTipologia2 > 0) && (idTipologia3 <= 0)) {
			strqry += " AND ((tsc.id = :idTipologia1) OR " +
							"(tsc.id = :idTipologia2))";
		}			
		else if ((idTipologia1 > 0) && (idTipologia2 > 0) && (idTipologia3 > 0)) {
			strqry += " AND ((tsc.id = :idTipologia1) OR " +
							"(tsc.id = :idTipologia2) OR " +
							"(tsc.id = :idTipologia3))";
		}
		//Parametros mal conformados, devolvemos el resultado vacío.
		else if (!((idTipologia1 <= 0) && (idTipologia2 <= 0) && (idTipologia3 <= 0)))
			return lwsatcemit;
		

		try {
			Query command = service.getEntityManager().createQuery(strqry);
			
			//Parámetros fijos
			command.setParameter("languageISO", languageISO);
			command.setParameter("idUsuario", idUsuario);
			
			//Parámetros que dependen del argumento idCentro
			if ((idCentro == null) || (idCentro > 0))
				command.setParameter("idCentro", idCentro);
			else //if (idCentro <= 0)
				command.setParameter("idProvincia", idProvincia);
			
			//Parámetros que dependen de los argumentos idTipologia1, idTipologia2 e idTipologia3
			if ((idTipologia1 > 0) && (idTipologia2 <= 0) && (idTipologia3 <= 0)) {
				command.setParameter("idTipologia1", idTipologia1);
			}			
			else if ((idTipologia1 > 0) && (idTipologia2 > 0) && (idTipologia3 <= 0)) {
				command.setParameter("idTipologia1", idTipologia1);
				command.setParameter("idTipologia2", idTipologia2);
			}			
			else if ((idTipologia1 > 0) && (idTipologia2 > 0) && (idTipologia3 > 0)) {
				command.setParameter("idTipologia1", idTipologia1);
				command.setParameter("idTipologia2", idTipologia2);
				command.setParameter("idTipologia3", idTipologia3);
			}
			
			@SuppressWarnings("unchecked")
			List<Object[]> lcut = (List<Object[]>)command.getResultList();
			
			for (Object[] lo : lcut)
				lwsatcemit.add(crearWSActividadTipologiaCeMIT((Curso)lo[0], (CursoLanguage)lo[1], (ServicioCurso)lo[2],
															  (ServicioCursoLanguage)lo[3], (Tematica)lo[4],
															  (TematicaLanguage)lo[5], language));
			
			lcut.clear();
		}
		catch (Exception e) {
			logger.log(Level.SEVERE,
					   "BuscaActividadesTipologiaCeMIT: Error al devolver la búsqueda de actividades por tipologia CeMIT: " +
					   e.getLocalizedMessage());
		
			e.printStackTrace();
		}
		
		strqry = "SELECT c, cl, tc, tcl " +
				 "FROM Charla c INNER JOIN " +
					  "c.textosCharla cl INNER JOIN " +
					  "cl.language lc WITH (lc.ISO = :languageISO) INNER JOIN " +
					  "c.tematica tc INNER JOIN " +
					  "tc.textos tcl INNER JOIN " +
					  "tcl.language ltc WITH (ltc.ISO = :languageISO) " +
				 "WHERE (c.fechaInicio > now()) AND " +
					   "(c.id NOT IN (SELECT DISTINCT mc.charla.id " +
									 "FROM MatriculaCharla mc INNER JOIN " +
										  "mc.usuario u WITH (u.id = :idUsuario) " +
									 "WHERE " + //Solicitado o matriculado y no iniciado 
										   "((((mc.preMatriculadoWeb = true) AND " +
										      "(mc.enListaEspera     = true) AND " +
										      "(mc.admitido          = false)) OR " +
										     "((mc.enListaEspera     = false) AND " +
										      "(mc.admitido          = true))) AND " +
										    "(mc.charla.fechaInicio >= now())) OR " +
										    //Matriculado, acabado y con asistencia mayor al 80%
										   "((mc.enListaEspera     = false) AND " +
										    "(mc.admitido          = true) AND " +
										    "(mc.charla.fechaInicio < now()) AND " +
										    "(mc.charla.asistencias.size > 0))))";

		//Condiciones atendiendo al parámetro idCentro
		if ((idCentro == null) || (idCentro > 0)) {
			strqry += " AND (c.centro.id = :idCentro)";
		
			if (idCentro == null)
				idCentro = uldap.getUser().getCentro().getId();
		}
		else { //if (idCentro <= 0)
			idProvincia = uldap.getUser().getCentro().getProvincia().getId();
			strqry += " AND (c.centro.provincia.id = :idProvincia)";
		}
		
		//Condiciones atendiendo a los parámetros idTipologia1, idTipologia2, idTipologia3
		if ((idTipologia1 > 0) && (idTipologia2 <= 0) && (idTipologia3 <= 0)) {
			strqry += " AND (tc.id = :idTipologia1)";
		}			
		else if ((idTipologia1 > 0) && (idTipologia2 > 0) && (idTipologia3 <= 0)) {
			strqry += " AND ((tc.id = :idTipologia1) OR " +
							"(tc.id = :idTipologia2))";
		}			
		else if ((idTipologia1 > 0) && (idTipologia2 > 0) && (idTipologia3 > 0)) {
			strqry += " AND ((tc.id = :idTipologia1) OR " +
							"(tc.id = :idTipologia2) OR " +
							"(tc.id = :idTipologia3))";
		}
		//Parametros mal conformados, devolvemos el resultado vacío.
		else if (!((idTipologia1 <= 0) && (idTipologia2 <= 0) && (idTipologia3 <= 0)))
			return lwsatcemit;
		
		try {
			Query command = service.getEntityManager().createQuery(strqry);
		
			//Parámetros fijos
			command.setParameter("languageISO", languageISO);
			command.setParameter("idUsuario", idUsuario);
		
			//Parámetros que dependen del argumento idCentro
			if ((idCentro == null) || (idCentro > 0))
				command.setParameter("idCentro", idCentro);
			else //if (idCentro <= 0)
				command.setParameter("idProvincia", idProvincia);
		
			//Parámetros que dependen de los argumentos idTipologia1, idTipologia2 e idTipologia3
			if ((idTipologia1 > 0) && (idTipologia2 <= 0) && (idTipologia3 <= 0))
				command.setParameter("idTipologia1", idTipologia1);
			else if ((idTipologia1 > 0) && (idTipologia2 > 0) && (idTipologia3 <= 0)) {
				command.setParameter("idTipologia1", idTipologia1);
				command.setParameter("idTipologia2", idTipologia2);
			}			
			else if ((idTipologia1 > 0) && (idTipologia2 > 0) && (idTipologia3 > 0)) {
				command.setParameter("idTipologia1", idTipologia1);
				command.setParameter("idTipologia2", idTipologia2);
				command.setParameter("idTipologia3", idTipologia3);
			}
		
			@SuppressWarnings("unchecked")
			List<Object[]> lcht = (List<Object[]>)command.getResultList();
		
			for (Object[] lo : lcht)
				lwsatcemit.add(crearWSActividadTipologiaCeMIT((Charla)lo[0], (CharlaLanguage)lo[1], (Tematica)lo[2],
															  (TematicaLanguage)lo[3], language));
		
			lcht.clear();
		}
		catch (Exception e) {
			logger.log(Level.SEVERE,
					   "BuscaActividadesTipologiaCeMIT: Error al devolver la búsqueda de actividades por tipologia CeMIT: " +
					   e.getLocalizedMessage());
		
			e.printStackTrace();
		}

		if (fieldOrder != null) {
			if (!fieldOrder.isEmpty()) {
				if (orderASC == null)
					orderASC = true;
				
				if (fieldOrder.compareTo("idActividad") == 0) {
					if (orderASC)
						Collections.sort(lwsatcemit, WSActividadTipologiaCeMIT.comparatorIdActividadASC);
					else
						Collections.sort(lwsatcemit, WSActividadTipologiaCeMIT.comparatorIdActividadDESC);
				}
				else if (fieldOrder.compareTo("idCentro") == 0) {
					if (orderASC)
						Collections.sort(lwsatcemit, WSActividadTipologiaCeMIT.comparatorIdCentroASC);
					else
						Collections.sort(lwsatcemit, WSActividadTipologiaCeMIT.comparatorIdCentroDESC);
				}
				else if (fieldOrder.compareTo("nombreCentro") == 0) {
					if (orderASC)
						Collections.sort(lwsatcemit, WSActividadTipologiaCeMIT.comparatorNombreCentroASC);
					else
						Collections.sort(lwsatcemit, WSActividadTipologiaCeMIT.comparatorNombreCentroDESC);
				}
				else if (fieldOrder.compareTo("tipo") == 0) {
					if (orderASC)
						Collections.sort(lwsatcemit, WSActividadTipologiaCeMIT.comparatorTipoASC);
					else
						Collections.sort(lwsatcemit, WSActividadTipologiaCeMIT.comparatorTipoDESC);
				}
				else if (fieldOrder.compareTo("idTematica") == 0) {
					if (orderASC)
						Collections.sort(lwsatcemit, WSActividadTipologiaCeMIT.comparatorIdTematicaASC);
					else
						Collections.sort(lwsatcemit, WSActividadTipologiaCeMIT.comparatorIdTematicaDESC);
				}
				else if (fieldOrder.compareTo("tematica") == 0) {
					if (orderASC)
						Collections.sort(lwsatcemit, WSActividadTipologiaCeMIT.comparatorTematicaASC);
					else
						Collections.sort(lwsatcemit, WSActividadTipologiaCeMIT.comparatorTematicaDESC);
				}
				else if (fieldOrder.compareTo("nivel") == 0) {
					if (orderASC)
						Collections.sort(lwsatcemit, WSActividadTipologiaCeMIT.comparatorNivelASC);
					else
						Collections.sort(lwsatcemit, WSActividadTipologiaCeMIT.comparatorNivelDESC);
				}
				else if (fieldOrder.compareTo("nombre") == 0) {
					if (orderASC)
						Collections.sort(lwsatcemit, WSActividadTipologiaCeMIT.comparatorNombreASC);
					else
						Collections.sort(lwsatcemit, WSActividadTipologiaCeMIT.comparatorNombreDESC);
				}
				else if (fieldOrder.compareTo("descripcion") == 0) {
					if (orderASC)
						Collections.sort(lwsatcemit, WSActividadTipologiaCeMIT.comparatorDescripcionASC);
					else
						Collections.sort(lwsatcemit, WSActividadTipologiaCeMIT.comparatorDescripcionDESC);
				}
				else if (fieldOrder.compareTo("fechaInicio") == 0) {
					if (orderASC)
						Collections.sort(lwsatcemit, WSActividadTipologiaCeMIT.comparatorFechaInicioASC);
					else
						Collections.sort(lwsatcemit, WSActividadTipologiaCeMIT.comparatorFechaInicioDESC);
				}
				else if (fieldOrder.compareTo("fechaFin") == 0) {
					if (orderASC)
						Collections.sort(lwsatcemit, WSActividadTipologiaCeMIT.comparatorFechaFinASC);
					else
						Collections.sort(lwsatcemit, WSActividadTipologiaCeMIT.comparatorFechaFinDESC);
				}
				else if (fieldOrder.compareTo("numeroHoras") == 0) {
					if (orderASC)
						Collections.sort(lwsatcemit, WSActividadTipologiaCeMIT.comparatorNumeroHorasASC);
					else
						Collections.sort(lwsatcemit, WSActividadTipologiaCeMIT.comparatorNumeroHorasDESC);
				}
			}
		}

		if ((recordsByPage != null) && (page != null)) {
			if ((recordsByPage > 0) && (page > 0)) {
				int firstIndex = (page - 1) * recordsByPage;
				int lastIndex = (page * recordsByPage);
				
				if (firstIndex < lwsatcemit.size()) {
					lastIndex = (lastIndex <= lwsatcemit.size()) ? lastIndex : lwsatcemit.size();
					
					return lwsatcemit.subList(firstIndex, lastIndex);
				}
				else
					lwsatcemit.clear();
			}
		}

		long elapsedTime = System.currentTimeMillis() - startTime;
		logger.log(Level.INFO, "Webservice BuscaActividadesTipologiaCeMIT request - " + elapsedTime + "ms");

		return lwsatcemit;
	}
	
	/**
	 * Método web que busca todas las encuestas, examenes y pruebas pendientes asociadas al usuario 
	 * especificado en la cabecera de atenticación.
	 *
	 * @param tipo
	 * 			(obligatorio)
	 * 			tipo de formulario (encuesta, examen, prueba de nivel,...) de la actividad
	 * 			0 = Encuesta
	 * 			1 = Examen (inicial, final, prueba de nivel)
	 * 
	 * @param languageISO
	 * 			(obligatorio)
	 * 			Identificador ISO del idioma en el que se desean los resultados ('gl_ES' o 'es_ES').
	 * 
	 * @param recordsByPage
	 * 			(opcional)
	 *          Número de registros a devolver.  
	 *          <= 0 indica devolver todos los registros.
	 *           
	 * @param page
	 * 			(opcional)
	 *          Número de página a devolver. Sólo si el parámetro recordsByPage es > 0.
	 *          En el caso de especificar un número <= 0 se ignora el parámetro recordsByPage y 
	 *          se devuelven todos los registros.
	 *          1 es la primera página.
	 * 
	 * @return listado con todas las actividades asociadas al usuario especificado en la cabecera de atenticación.
	 */
	
	@WebMethod
	@Transactional
	public List<WSFormularioCeMIT> FormulariosPendientesUsuarioCeMIT(Long tipo, 
																String languageISO, String recordsByPage, String page) {

		long startTime = System.currentTimeMillis();

		UsuarioLdap uldap = (UsuarioLdap)SecurityContextHolder.getContext().getAuthentication().getPrincipal();
		Long idUsuario = uldap.getUser().getId();
		
		List<WSFormularioCeMIT> formulariosCeMIT = new ArrayList<WSFormularioCeMIT>();
		
		if (!languageISO.isEmpty() && tipo.SIZE > 0) {
			
			Locale language = new Locale(languageISO.split("_")[0], languageISO.split("_")[1]);
			
			String strqry = ""; 
			
			if(tipo == 0L) //Encuesta
				strqry += "SELECT c, cl, ft, mc " + 
						"FROM " + Curso.class.getName() + " c INNER JOIN " +
						"c.textosCurso cl INNER JOIN " + 
						"cl.language lc WITH (lc.ISO = :languageISO) INNER JOIN " +
						"c.cuestionarioSatisfaccion f INNER JOIN " +
						"f.textos ft INNER JOIN " +
						"ft.language fl WITH (fl.ISO = :languageISO) INNER JOIN " +
						"c.matriculas mc INNER JOIN mc.usuario u WITH (u.id = :idUsuario) " +
						"WHERE (c.desactivado = false) AND (c.cerrada = false) AND (now() >= c.fechaFin)";
			else if(tipo == 1L) //Examenes (nivel, inicial, final)
				strqry += "SELECT c, cl, mc " + 
						  "FROM " + Curso.class.getName() + " c INNER JOIN " + 
						  "c.textosCurso cl INNER JOIN cl.language lc WITH (lc.ISO = :languageISO) INNER JOIN " + 
						  "c.matriculas mc INNER JOIN mc.usuario u WITH (u.id = :idUsuario) " +
						  "WHERE (c.desactivado = false) AND (c.cerrada = false)";
			
			try {
				Query command = service.getEntityManager().createQuery(strqry);
				
				command.setParameter("languageISO", languageISO);
				command.setParameter("idUsuario", idUsuario);
				
				@SuppressWarnings("unchecked")
				List<Object[]> lcec = (List<Object[]>)command.getResultList();
				
				for (Object[] lo : lcec) {
					if(tipo == 0L) //Encuesta
						formulariosCeMIT.add(crearFormularioCeMIT((Curso)lo[0],(CursoLanguage)lo[1],
											(FormularioLanguage)lo[2], (MatriculaCurso)lo[3], language));		
					else if(tipo == 1L) //Examenes
						formulariosCeMIT.addAll(crearFormularioCeMIT((Curso)lo[0],(CursoLanguage)lo[1], 
								(MatriculaCurso)lo[2], languageISO));
				}
				lcec.clear();			
			} catch (Exception e) {
				logger.log(Level.SEVERE,
						   "FormulariosPendientesUsuarioCeMIT: Error al devolver la búsqueda de formularios pendientes"
						   + " del usuario CeMIT: " +
						   e.getLocalizedMessage());
			
				e.printStackTrace();
			}
		}
		
		long elapsedTime = System.currentTimeMillis() - startTime;
		logger.log(Level.INFO, "Webservice FormulariosPendientesUsuarioCeMIT request - " + elapsedTime + "ms");
		
		return formulariosCeMIT;
	}
	
	////////////////////////////////////////////////////////CLASES INTERNAS//////////////////////////////////////////////////

	/**
	 * WSUsuarioCeMIT class.
	 */
	public static class WSUsuarioCeMIT {
		public Long idUsuario;
		public String nombre;
		public String apellidos;
		public String username;
		public Long idCentro;
		public String nif;
		public String fechaNacimiento;
		public String direccion;
		public String phone;
		public String movil;
		public String email;
	}

	private WSUsuarioCeMIT crearWSUsuarioCeMIT(UsuarioLdap uldap) {
		WSUsuarioCeMIT usuarioCeMIT = new WSUsuarioCeMIT();
		SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
		UsuarioCyL ucyl = null;
		
		if ( uldap.getUser() instanceof UsuarioCyL)
			ucyl = (UsuarioCyL) uldap.getUser();

		usuarioCeMIT.idUsuario = uldap.getUser().getId();
		usuarioCeMIT.nombre = uldap.getUser().getNombre();
		usuarioCeMIT.apellidos = uldap.getUser().getApellidos();
		usuarioCeMIT.username = uldap.getUsername();
		usuarioCeMIT.idCentro = uldap.getCentro().getId();
		usuarioCeMIT.nif = (ucyl != null) ? ucyl.getNif() : uldap.getUser().getNif();
		usuarioCeMIT.fechaNacimiento = (ucyl != null) ? sdf.format(ucyl.getFechaNacimiento()) : "";
		usuarioCeMIT.direccion = (ucyl != null) ? ucyl.getDireccion() : uldap.getUser().getDireccion();
		usuarioCeMIT.phone = uldap.getUser().getTelefono();
		usuarioCeMIT.movil = (ucyl != null) ? ucyl.getMovil() : "";
		usuarioCeMIT.email = (ucyl != null) ? ucyl.getEmail() : "";
		
		return usuarioCeMIT;
	}

	/**
	 * TipoActividad enum.
	 */
	protected enum TipoEstadoActividad {
		FINALIZADO (1L, "integracion.estadoActividad.finalizado"),
		EN_EJECUCION (2L, "integracion.estadoActividad.enEjecucion"),
		SOLICITADO (3L, "integracion.estadoActividad.solicitado"),
		SOLICITADOFINALIZADO (4L, "integracion.estadoActividad.solicitadoFinalizado"),
		MATRICULADO (5L, "integracion.estadoActividad.matriculado");
		
		private final Long idTipoEstadoActividad;
		private final String nombre;
		
		TipoEstadoActividad(Long idTipoEstadoActividad, String nombre) {
	        this.idTipoEstadoActividad = idTipoEstadoActividad;
	        this.nombre = nombre;
	    }
		
		public Long getIdTipoEstadoActividad() { return idTipoEstadoActividad; }
	    public String getNombre() { return nombre; }
	}

	/**
	 * WSActividadUsuarioCeMIT class.
	 */
	public static class WSActividadUsuarioCeMIT {
		public Long idActividad;
		public Long idCentro;
		public String nombreCentro;
		public String tipo;
		public Long idTematica;
		public String tematica;
		public String nivel;
		public String nombre;
		public String descripcion;
		public String estado;
		public Date fechaEstado;
		public Date horaEstado;
		public Date fechaInicio;
		public Date fechaFin;
		public Float numeroHoras;
		public Float asistencia;
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorIdActividadASC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.idActividad > o2.idActividad)
					return 1;
				else if (o1.idActividad < o2.idActividad)
					return -1;
				else // if (o1.idActividad == o2.idActividad)
					return 0;
			}
		};
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorIdActividadDESC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.idActividad > o2.idActividad)
					return -1;
				else if (o1.idActividad < o2.idActividad)
					return 1;
				else // if (o1.getIdActividad == o2.getIdActividad)
					return 0;
			}
		};
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorIdCentroASC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.idCentro > o2.idCentro)
					return 1;
				else if (o1.idCentro < o2.idCentro)
					return -1;
				else // if (o1.idCentro == o2.idCentro)
					return 0;
			}
		};
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorIdCentroDESC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.idCentro > o2.idCentro)
					return -1;
				else if (o1.idCentro < o2.idCentro)
					return 1;
				else // if (o1.idCentro == o2.idCentro)
					return 0;
			}
		};
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorNombreCentroASC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.nombreCentro.compareToIgnoreCase(o2.nombreCentro) < 0)
					return -1;
				else if (o1.nombreCentro.compareToIgnoreCase(o2.nombreCentro) > 0)
					return 1;
				else // if (o1.nombreCentro.compareToIgnoreCase(o2.nombreCentro) == 0)
					return 0;
			}
		};
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorNombreCentroDESC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.nombreCentro.compareToIgnoreCase(o2.nombreCentro) < 0)
					return 1;
				else if (o1.nombreCentro.compareToIgnoreCase(o2.nombreCentro) > 0)
					return -1;
				else // if (o1.nombreCentro.compareToIgnoreCase(o2.nombreCentro) == 0)
					return 0;
			}
		};
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorTipoASC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.tipo.compareToIgnoreCase(o2.tipo) < 0)
					return -1;
				else if (o1.tipo.compareToIgnoreCase(o2.tipo) > 0)
					return 1;
				else // if (o1.tipo.compareToIgnoreCase(o2.tipo) == 0)
					return 0;
			}
		};
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorTipoDESC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.tipo.compareToIgnoreCase(o2.tipo) < 0)
					return 1;
				else if (o1.tipo.compareToIgnoreCase(o2.tipo) > 0)
					return -1;
				else // if (o1.tipo.compareToIgnoreCase(o2.tipo) == 0)
					return 0;
			}
		};
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorIdTematicaASC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.idTematica > o2.idTematica)
					return 1;
				else if (o1.idTematica < o2.idTematica)
					return -1;
				else // if (o1.idTematica == o2.idTematica)
					return 0;
			}
		};
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorIdTematicaDESC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.idTematica > o2.idTematica)
					return -1;
				else if (o1.idTematica < o2.idTematica)
					return 1;
				else // if (o1.idTematica == o2.idTematica)
					return 0;
			}
		};
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorTematicaASC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.tematica.compareToIgnoreCase(o2.tematica) < 0)
					return -1;
				else if (o1.tematica.compareToIgnoreCase(o2.tematica) > 0)
					return 1;
				else // if (o1.tematica.compareToIgnoreCase(o2.tematica) == 0)
					return 0;
			}
		};
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorTematicaDESC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.tematica.compareToIgnoreCase(o2.tematica) < 0)
					return 1;
				else if (o1.tematica.compareToIgnoreCase(o2.tematica) > 0)
					return -1;
				else // if (o1.tematica.compareToIgnoreCase(o2.tematica) == 0)
					return 0;
			}
		};
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorNivelASC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.nivel.compareToIgnoreCase(o2.nivel) < 0)
					return -1;
				else if (o1.nivel.compareToIgnoreCase(o2.nivel) > 0)
					return 1;
				else // if (o1.nivel.compareToIgnoreCase(o2.nivel) == 0)
					return 0;
			}
		};
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorNivelDESC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.nivel.compareToIgnoreCase(o2.nivel) < 0)
					return 1;
				else if (o1.nivel.compareToIgnoreCase(o2.nivel) > 0)
					return -1;
				else // if (o1.nivel.compareToIgnoreCase(o2.nivel) == 0)
					return 0;
			}
		};
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorNombreASC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.nombre.compareToIgnoreCase(o2.nombre) < 0)
					return -1;
				else if (o1.nombre.compareToIgnoreCase(o2.nombre) > 0)
					return 1;
				else // if (o1.nombre.compareToIgnoreCase(o2.nombre) == 0)
					return 0;
			}
		};
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorNombreDESC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.nombre.compareToIgnoreCase(o2.nombre) < 0)
					return 1;
				else if (o1.nombre.compareToIgnoreCase(o2.nombre) > 0)
					return -1;
				else // if (o1.nombre.compareToIgnoreCase(o2.nombre) == 0)
					return 0;
			}
		};
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorDescripcionASC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.descripcion.compareToIgnoreCase(o2.descripcion) < 0)
					return -1;
				else if (o1.descripcion.compareToIgnoreCase(o2.descripcion) > 0)
					return 1;
				else // if (o1.descripcion.compareToIgnoreCase(o2.descripcion) == 0)
					return 0;
			}
		};
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorDescripcionDESC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.descripcion.compareToIgnoreCase(o2.descripcion) < 0)
					return 1;
				else if (o1.descripcion.compareToIgnoreCase(o2.descripcion) > 0)
					return -1;
				else // if (o1.descripcion.compareToIgnoreCase(o2.descripcion) == 0)
					return 0;
			}
		};
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorEstadoASC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.estado.compareToIgnoreCase(o2.estado) < 0)
					return -1;
				else if (o1.estado.compareToIgnoreCase(o2.estado) > 0)
					return 1;
				else // if (o1.estado.compareToIgnoreCase(o2.estado) == 0)
					return 0;
			}
		};
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorEstadoDESC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.estado.compareToIgnoreCase(o2.estado) < 0)
					return 1;
				else if (o1.estado.compareToIgnoreCase(o2.estado) > 0)
					return -1;
				else // if (o1.estado.compareToIgnoreCase(o2.estado) == 0)
					return 0;
			}
		};
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorFechaEstadoASC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.fechaEstado.before(o2.fechaEstado))
					return -1;
				else if (o1.fechaEstado.after(o2.fechaEstado))
					return 1;
				else // if (o1.fechaEstado.equals(o2.fechaEstado))
					return 0;
			}
		};
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorFechaEstadoDESC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.fechaEstado.before(o2.fechaEstado))
					return 1;
				else if (o1.fechaEstado.after(o2.fechaEstado))
					return -1;
				else // if (o1.fechaEstado.equals(o2.fechaEstado))
					return 0;
			}
		};
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorHoraEstadoASC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.horaEstado.before(o2.horaEstado))
					return -1;
				else if (o1.horaEstado.after(o2.horaEstado))
					return 1;
				else // if (o1.horaEstado.equals(o2.horaEstado))
					return 0;
			}
		};
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorHoraEstadoDESC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.horaEstado.before(o2.horaEstado))
					return 1;
				else if (o1.horaEstado.after(o2.horaEstado))
					return -1;
				else // if (o1.horaEstado.equals(o2.horaEstado))
					return 0;
			}
		};
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorFechaInicioASC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.fechaInicio.before(o2.fechaInicio))
					return -1;
				else if (o1.fechaInicio.after(o2.fechaInicio))
					return 1;
				else // if (o1.fechaInicio.equals(o2.fechaInicio))
					return 0;
			}
		};
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorFechaInicioDESC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.fechaInicio.before(o2.fechaInicio))
					return 1;
				else if (o1.fechaInicio.after(o2.fechaInicio))
					return -1;
				else // if (o1.fechaInicio.equals(o2.fechaInicio))
					return 0;
			}
		};
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorFechaFinASC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.fechaFin.before(o2.fechaFin))
					return -1;
				else if (o1.fechaFin.after(o2.fechaFin))
					return 1;
				else // if (o1.fechaFin.equals(o2.fechaFin))
					return 0;
			}
		};
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorFechaFinDESC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.fechaFin.before(o2.fechaFin))
					return 1;
				else if (o1.fechaFin.after(o2.fechaFin))
					return -1;
				else // if (o1.fechaFin.equals(o2.fechaFin))
					return 0;
			}
		};
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorNumeroHorasASC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.numeroHoras > o2.numeroHoras)
					return 1;
				else if (o1.numeroHoras < o2.numeroHoras)
					return -1;
				else // if (o1.numeroHoras == o2.numeroHoras)
					return 0;
			}
		};
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorNumeroHorasDESC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.numeroHoras > o2.numeroHoras)
					return -1;
				else if (o1.numeroHoras < o2.numeroHoras)
					return 1;
				else // if (o1.numeroHoras == o2.numeroHoras)
					return 0;
			}
		};
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorAsistenciaASC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.asistencia > o2.asistencia)
					return 1;
				else if (o1.asistencia < o2.asistencia)
					return -1;
				else // if (o1.asistencia == o2.asistencia)
					return 0;
			}
		};
		
		public static Comparator<WSActividadUsuarioCeMIT> comparatorAsistenciaDESC = new Comparator<WSActividadUsuarioCeMIT>() {
			public int compare(WSActividadUsuarioCeMIT o1, WSActividadUsuarioCeMIT o2) {
				if (o1.asistencia > o2.asistencia)
					return -1;
				else if (o1.asistencia < o2.asistencia)
					return 1;
				else // if (o1.asistencia == o2.asistencia)
					return 0;
			}
		};
	}

	private WSActividadUsuarioCeMIT crearWSActividadUsuarioCeMIT(Curso c, CursoLanguage cl, ServicioCurso sc, ServicioCursoLanguage scl, 
																 Tematica tsc, TematicaLanguage tscl, MatriculaCurso mc, Locale language) {
		WSActividadUsuarioCeMIT actividadUsuarioCeMIT = new WSActividadUsuarioCeMIT();
		
		actividadUsuarioCeMIT.idActividad = c.getId();
		actividadUsuarioCeMIT.idCentro = c.getCentro().getId();
		actividadUsuarioCeMIT.nombreCentro = c.getCentro().getNombre();
		actividadUsuarioCeMIT.tipo = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", language).getString(TipoActividad.CURSO.getNombre());
		actividadUsuarioCeMIT.idTematica = tsc.getId();
		actividadUsuarioCeMIT.tematica = tscl.getNombre();
		actividadUsuarioCeMIT.nivel = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", language).getString("nivel." + sc.getNivel());
		actividadUsuarioCeMIT.nombre = cl.getApodo();
		actividadUsuarioCeMIT.descripcion = (scl.getDescripcion() != null) ? scl.getDescripcion() : "";

		//Calcula el estado de la actividad y su fecha y hora
		if (mc.getEstadoMatricula() == Curso.EEstadoMatricula.MATRICULADO) {
			if (c.isImpartido()) {
				actividadUsuarioCeMIT.estado = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", language).getString(TipoEstadoActividad.FINALIZADO.getNombre());
				actividadUsuarioCeMIT.fechaEstado = c.getFechaFin();
				actividadUsuarioCeMIT.horaEstado = c.getHoraFin();
			}
			else if (c.isIniciado()) {
				actividadUsuarioCeMIT.estado = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", language).getString(TipoEstadoActividad.EN_EJECUCION.getNombre());
				actividadUsuarioCeMIT.fechaEstado = c.getFechaInicio();
				actividadUsuarioCeMIT.horaEstado = c.getHoraInicio();
			}
			else {
				actividadUsuarioCeMIT.estado = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", language).getString(TipoEstadoActividad.MATRICULADO.getNombre());
				actividadUsuarioCeMIT.fechaEstado = mc.getFechaTramite();
				actividadUsuarioCeMIT.horaEstado = mc.getFechaTramite();
			}
		}
		else if ((mc.getEstadoMatricula() == Curso.EEstadoMatricula.SOLICITUD_ENVIADA)) {
			if (c.isImpartido()) {
				actividadUsuarioCeMIT.estado = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", language).getString(TipoEstadoActividad.SOLICITADOFINALIZADO.getNombre());
				actividadUsuarioCeMIT.fechaEstado = c.getFechaFin();
				actividadUsuarioCeMIT.horaEstado = c.getHoraFin();
			}
			else { // if (!c.isImpartido())
				actividadUsuarioCeMIT.estado = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", language).getString(TipoEstadoActividad.SOLICITADO.getNombre());
				actividadUsuarioCeMIT.fechaEstado = mc.getFechaTramite();
				actividadUsuarioCeMIT.horaEstado = mc.getFechaTramite();
			}
		}
		
		actividadUsuarioCeMIT.fechaInicio = c.getFechaInicio();
		actividadUsuarioCeMIT.fechaFin = c.getFechaFin();
		actividadUsuarioCeMIT.numeroHoras = c.getNumeroHoras();
		actividadUsuarioCeMIT.asistencia = mc.getAsistencia();

		return actividadUsuarioCeMIT;
	}
	
	private WSActividadUsuarioCeMIT crearWSActividadUsuarioCeMIT(Charla c, CharlaLanguage cl, Tematica tc,
																 TematicaLanguage tcl, MatriculaCharla mc, Locale language) {
		WSActividadUsuarioCeMIT actividadUsuarioCeMIT = new WSActividadUsuarioCeMIT();
		
		actividadUsuarioCeMIT.idActividad = c.getId();
		actividadUsuarioCeMIT.idCentro = c.getCentro().getId();
		actividadUsuarioCeMIT.nombreCentro = c.getCentro().getNombre();
		actividadUsuarioCeMIT.tipo = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", language).getString(TipoActividad.CHARLA.getNombre());
		actividadUsuarioCeMIT.idTematica = tc.getId();
		actividadUsuarioCeMIT.tematica = tcl.getNombre();
		actividadUsuarioCeMIT.nivel = "";
		actividadUsuarioCeMIT.nombre = cl.getNombre();
		actividadUsuarioCeMIT.descripcion = (cl.getDescripcion() != null) ? cl.getDescripcion() : ""; // RF_15_29 Campo observación nas charlas
		
		//Calcula el estado de la actividad y su fecha y hora
		if (mc.getEstadoMatricula() == Charla.EEstadoMatricula.MATRICULADO) {
			if (c.isImpartido()) {
				actividadUsuarioCeMIT.estado = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", language).getString(TipoEstadoActividad.FINALIZADO.getNombre());
				actividadUsuarioCeMIT.fechaEstado = c.getFechaFin();
				actividadUsuarioCeMIT.horaEstado = c.getHoraFin();
			}
			else {
				actividadUsuarioCeMIT.estado = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", language).getString(TipoEstadoActividad.MATRICULADO.getNombre());
				actividadUsuarioCeMIT.fechaEstado = mc.getFechaTramite();
				actividadUsuarioCeMIT.horaEstado = mc.getFechaTramite();
			}
		}
		else if ((mc.getEstadoMatricula() == Charla.EEstadoMatricula.SOLICITUD_ENVIADA)) {
			if (c.isImpartido()) {
				actividadUsuarioCeMIT.estado = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", language).getString(TipoEstadoActividad.SOLICITADOFINALIZADO.getNombre());
				actividadUsuarioCeMIT.fechaEstado = c.getFechaFin();
				actividadUsuarioCeMIT.horaEstado = c.getHoraFin();
			}
			else { // if (!c.isImpartido())
				actividadUsuarioCeMIT.estado = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", language).getString(TipoEstadoActividad.SOLICITADO.getNombre());
				actividadUsuarioCeMIT.fechaEstado = mc.getFechaTramite();
				actividadUsuarioCeMIT.horaEstado = mc.getFechaTramite();
			}				
		}

		actividadUsuarioCeMIT.fechaInicio = c.getFechaInicio();
		actividadUsuarioCeMIT.fechaFin = c.getFechaInicio();
		actividadUsuarioCeMIT.numeroHoras = (float)((c.getHoraFin().getTime() - c.getHoraInicio().getTime()) / 1000.0 / 60.0 / 60.0);
		actividadUsuarioCeMIT.asistencia = mc.getAsistencia();

		return actividadUsuarioCeMIT;
	}
	
	/**
	 * WSActividadTipologiaCeMIT class.
	 */
	public static class WSActividadTipologiaCeMIT {
		public Long idActividad;
		public Long idCentro;
		public String nombreCentro;
		public String tipo;
		public Long idTematica;
		public String tematica;
		public String nivel;
		public String nombre;
		public String descripcion;
		public Date fechaInicio;
		public Date fechaFin;
		public Float numeroHoras;
		
		public static Comparator<WSActividadTipologiaCeMIT> comparatorIdActividadASC = new Comparator<WSActividadTipologiaCeMIT>() {
			public int compare(WSActividadTipologiaCeMIT o1, WSActividadTipologiaCeMIT o2) {
				if (o1.idActividad > o2.idActividad)
					return 1;
				else if (o1.idActividad < o2.idActividad)
					return -1;
				else // if (o1.idActividad == o2.idActividad)
					return 0;
			}
		};
		
		public static Comparator<WSActividadTipologiaCeMIT> comparatorIdActividadDESC = new Comparator<WSActividadTipologiaCeMIT>() {
			public int compare(WSActividadTipologiaCeMIT o1, WSActividadTipologiaCeMIT o2) {
				if (o1.idActividad > o2.idActividad)
					return -1;
				else if (o1.idActividad < o2.idActividad)
					return 1;
				else // if (o1.getIdActividad == o2.getIdActividad)
					return 0;
			}
		};
		
		public static Comparator<WSActividadTipologiaCeMIT> comparatorIdCentroASC = new Comparator<WSActividadTipologiaCeMIT>() {
			public int compare(WSActividadTipologiaCeMIT o1, WSActividadTipologiaCeMIT o2) {
				if (o1.idCentro > o2.idCentro)
					return 1;
				else if (o1.idCentro < o2.idCentro)
					return -1;
				else // if (o1.idCentro == o2.idCentro)
					return 0;
			}
		};
		
		public static Comparator<WSActividadTipologiaCeMIT> comparatorIdCentroDESC = new Comparator<WSActividadTipologiaCeMIT>() {
			public int compare(WSActividadTipologiaCeMIT o1, WSActividadTipologiaCeMIT o2) {
				if (o1.idCentro > o2.idCentro)
					return -1;
				else if (o1.idCentro < o2.idCentro)
					return 1;
				else // if (o1.idCentro == o2.idCentro)
					return 0;
			}
		};
		
		public static Comparator<WSActividadTipologiaCeMIT> comparatorNombreCentroASC = new Comparator<WSActividadTipologiaCeMIT>() {
			public int compare(WSActividadTipologiaCeMIT o1, WSActividadTipologiaCeMIT o2) {
				if (o1.nombreCentro.compareToIgnoreCase(o2.nombreCentro) < 0)
					return -1;
				else if (o1.nombreCentro.compareToIgnoreCase(o2.nombreCentro) > 0)
					return 1;
				else // if (o1.nombreCentro.compareToIgnoreCase(o2.nombreCentro) == 0)
					return 0;
			}
		};
		
		public static Comparator<WSActividadTipologiaCeMIT> comparatorNombreCentroDESC = new Comparator<WSActividadTipologiaCeMIT>() {
			public int compare(WSActividadTipologiaCeMIT o1, WSActividadTipologiaCeMIT o2) {
				if (o1.nombreCentro.compareToIgnoreCase(o2.nombreCentro) < 0)
					return 1;
				else if (o1.nombreCentro.compareToIgnoreCase(o2.nombreCentro) > 0)
					return -1;
				else // if (o1.nombreCentro.compareToIgnoreCase(o2.nombreCentro) == 0)
					return 0;
			}
		};
		
		public static Comparator<WSActividadTipologiaCeMIT> comparatorTipoASC = new Comparator<WSActividadTipologiaCeMIT>() {
			public int compare(WSActividadTipologiaCeMIT o1, WSActividadTipologiaCeMIT o2) {
				if (o1.tipo.compareToIgnoreCase(o2.tipo) < 0)
					return -1;
				else if (o1.tipo.compareToIgnoreCase(o2.tipo) > 0)
					return 1;
				else // if (o1.tipo.compareToIgnoreCase(o2.tipo) == 0)
					return 0;
			}
		};
		
		public static Comparator<WSActividadTipologiaCeMIT> comparatorTipoDESC = new Comparator<WSActividadTipologiaCeMIT>() {
			public int compare(WSActividadTipologiaCeMIT o1, WSActividadTipologiaCeMIT o2) {
				if (o1.tipo.compareToIgnoreCase(o2.tipo) < 0)
					return 1;
				else if (o1.tipo.compareToIgnoreCase(o2.tipo) > 0)
					return -1;
				else // if (o1.tipo.compareToIgnoreCase(o2.tipo) == 0)
					return 0;
			}
		};
		
		public static Comparator<WSActividadTipologiaCeMIT> comparatorIdTematicaASC = new Comparator<WSActividadTipologiaCeMIT>() {
			public int compare(WSActividadTipologiaCeMIT o1, WSActividadTipologiaCeMIT o2) {
				if (o1.idTematica > o2.idTematica)
					return 1;
				else if (o1.idTematica < o2.idTematica)
					return -1;
				else // if (o1.idTematica == o2.idTematica)
					return 0;
			}
		};
		
		public static Comparator<WSActividadTipologiaCeMIT> comparatorIdTematicaDESC = new Comparator<WSActividadTipologiaCeMIT>() {
			public int compare(WSActividadTipologiaCeMIT o1, WSActividadTipologiaCeMIT o2) {
				if (o1.idTematica > o2.idTematica)
					return -1;
				else if (o1.idTematica < o2.idTematica)
					return 1;
				else // if (o1.idTematica == o2.idTematica)
					return 0;
			}
		};
		
		public static Comparator<WSActividadTipologiaCeMIT> comparatorTematicaASC = new Comparator<WSActividadTipologiaCeMIT>() {
			public int compare(WSActividadTipologiaCeMIT o1, WSActividadTipologiaCeMIT o2) {
				if (o1.tematica.compareToIgnoreCase(o2.tematica) < 0)
					return -1;
				else if (o1.tematica.compareToIgnoreCase(o2.tematica) > 0)
					return 1;
				else // if (o1.tematica.compareToIgnoreCase(o2.tematica) == 0)
					return 0;
			}
		};
		
		public static Comparator<WSActividadTipologiaCeMIT> comparatorTematicaDESC = new Comparator<WSActividadTipologiaCeMIT>() {
			public int compare(WSActividadTipologiaCeMIT o1, WSActividadTipologiaCeMIT o2) {
				if (o1.tematica.compareToIgnoreCase(o2.tematica) < 0)
					return 1;
				else if (o1.tematica.compareToIgnoreCase(o2.tematica) > 0)
					return -1;
				else // if (o1.tematica.compareToIgnoreCase(o2.tematica) == 0)
					return 0;
			}
		};
		

		public static Comparator<WSActividadTipologiaCeMIT> comparatorNivelASC = new Comparator<WSActividadTipologiaCeMIT>() {
			public int compare(WSActividadTipologiaCeMIT o1, WSActividadTipologiaCeMIT o2) {
				if (o1.nivel.compareToIgnoreCase(o2.nivel) < 0)
					return -1;
				else if (o1.nivel.compareToIgnoreCase(o2.nivel) > 0)
					return 1;
				else // if (o1.nivel.compareToIgnoreCase(o2.nivel) == 0)
					return 0;
			}
		};
		
		public static Comparator<WSActividadTipologiaCeMIT> comparatorNivelDESC = new Comparator<WSActividadTipologiaCeMIT>() {
			public int compare(WSActividadTipologiaCeMIT o1, WSActividadTipologiaCeMIT o2) {
				if (o1.nivel.compareToIgnoreCase(o2.nivel) < 0)
					return 1;
				else if (o1.nivel.compareToIgnoreCase(o2.nivel) > 0)
					return -1;
				else // if (o1.nivel.compareToIgnoreCase(o2.nivel) == 0)
					return 0;
			}
		};
		
		public static Comparator<WSActividadTipologiaCeMIT> comparatorNombreASC = new Comparator<WSActividadTipologiaCeMIT>() {
			public int compare(WSActividadTipologiaCeMIT o1, WSActividadTipologiaCeMIT o2) {
				if (o1.nombre.compareToIgnoreCase(o2.nombre) < 0)
					return -1;
				else if (o1.nombre.compareToIgnoreCase(o2.nombre) > 0)
					return 1;
				else // if (o1.nombre.compareToIgnoreCase(o2.nombre) == 0)
					return 0;
			}
		};
		
		public static Comparator<WSActividadTipologiaCeMIT> comparatorNombreDESC = new Comparator<WSActividadTipologiaCeMIT>() {
			public int compare(WSActividadTipologiaCeMIT o1, WSActividadTipologiaCeMIT o2) {
				if (o1.nombre.compareToIgnoreCase(o2.nombre) < 0)
					return 1;
				else if (o1.nombre.compareToIgnoreCase(o2.nombre) > 0)
					return -1;
				else // if (o1.nombre.compareToIgnoreCase(o2.nombre) == 0)
					return 0;
			}
		};
		
		public static Comparator<WSActividadTipologiaCeMIT> comparatorDescripcionASC = new Comparator<WSActividadTipologiaCeMIT>() {
			public int compare(WSActividadTipologiaCeMIT o1, WSActividadTipologiaCeMIT o2) {
				if (o1.descripcion.compareToIgnoreCase(o2.descripcion) < 0)
					return -1;
				else if (o1.descripcion.compareToIgnoreCase(o2.descripcion) > 0)
					return 1;
				else // if (o1.descripcion.compareToIgnoreCase(o2.descripcion) == 0)
					return 0;
			}
		};
		
		public static Comparator<WSActividadTipologiaCeMIT> comparatorDescripcionDESC = new Comparator<WSActividadTipologiaCeMIT>() {
			public int compare(WSActividadTipologiaCeMIT o1, WSActividadTipologiaCeMIT o2) {
				if (o1.descripcion.compareToIgnoreCase(o2.descripcion) < 0)
					return 1;
				else if (o1.descripcion.compareToIgnoreCase(o2.descripcion) > 0)
					return -1;
				else // if (o1.descripcion.compareToIgnoreCase(o2.descripcion) == 0)
					return 0;
			}
		};
		
		public static Comparator<WSActividadTipologiaCeMIT> comparatorFechaInicioASC = new Comparator<WSActividadTipologiaCeMIT>() {
			public int compare(WSActividadTipologiaCeMIT o1, WSActividadTipologiaCeMIT o2) {
				if (o1.fechaInicio.before(o2.fechaInicio))
					return -1;
				else if (o1.fechaInicio.after(o2.fechaInicio))
					return 1;
				else // if (o1.fechaInicio.equals(o2.fechaInicio))
					return 0;
			}
		};
		
		public static Comparator<WSActividadTipologiaCeMIT> comparatorFechaInicioDESC = new Comparator<WSActividadTipologiaCeMIT>() {
			public int compare(WSActividadTipologiaCeMIT o1, WSActividadTipologiaCeMIT o2) {
				if (o1.fechaInicio.before(o2.fechaInicio))
					return 1;
				else if (o1.fechaInicio.after(o2.fechaInicio))
					return -1;
				else // if (o1.fechaInicio.equals(o2.fechaInicio))
					return 0;
			}
		};
		
		public static Comparator<WSActividadTipologiaCeMIT> comparatorFechaFinASC = new Comparator<WSActividadTipologiaCeMIT>() {
			public int compare(WSActividadTipologiaCeMIT o1, WSActividadTipologiaCeMIT o2) {
				if (o1.fechaFin.before(o2.fechaFin))
					return -1;
				else if (o1.fechaFin.after(o2.fechaFin))
					return 1;
				else // if (o1.fechaFin.equals(o2.fechaFin))
					return 0;
			}
		};
		
		public static Comparator<WSActividadTipologiaCeMIT> comparatorFechaFinDESC = new Comparator<WSActividadTipologiaCeMIT>() {
			public int compare(WSActividadTipologiaCeMIT o1, WSActividadTipologiaCeMIT o2) {
				if (o1.fechaFin.before(o2.fechaFin))
					return 1;
				else if (o1.fechaFin.after(o2.fechaFin))
					return -1;
				else // if (o1.fechaFin.equals(o2.fechaFin))
					return 0;
			}
		};
		
		public static Comparator<WSActividadTipologiaCeMIT> comparatorNumeroHorasASC = new Comparator<WSActividadTipologiaCeMIT>() {
			public int compare(WSActividadTipologiaCeMIT o1, WSActividadTipologiaCeMIT o2) {
				if (o1.numeroHoras > o2.numeroHoras)
					return 1;
				else if (o1.numeroHoras < o2.numeroHoras)
					return -1;
				else // if (o1.numeroHoras == o2.numeroHoras)
					return 0;
			}
		};
		
		public static Comparator<WSActividadTipologiaCeMIT> comparatorNumeroHorasDESC = new Comparator<WSActividadTipologiaCeMIT>() {
			public int compare(WSActividadTipologiaCeMIT o1, WSActividadTipologiaCeMIT o2) {
				if (o1.numeroHoras > o2.numeroHoras)
					return -1;
				else if (o1.numeroHoras < o2.numeroHoras)
					return 1;
				else // if (o1.numeroHoras == o2.numeroHoras)
					return 0;
			}
		};
	}

	private WSActividadTipologiaCeMIT crearWSActividadTipologiaCeMIT(Curso c, CursoLanguage cl, ServicioCurso sc,
																	 ServicioCursoLanguage scl, Tematica tsc,
																	 TematicaLanguage tscl, Locale language) {
		WSActividadTipologiaCeMIT actividadTipologiaCeMIT = new WSActividadTipologiaCeMIT();
		
		actividadTipologiaCeMIT.idActividad = c.getId();
		actividadTipologiaCeMIT.idCentro = c.getCentro().getId();
		actividadTipologiaCeMIT.nombreCentro = c.getCentro().getNombre();
		actividadTipologiaCeMIT.tipo = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", language).getString(TipoActividad.CURSO.getNombre());
		actividadTipologiaCeMIT.idTematica = tsc.getId();
		actividadTipologiaCeMIT.tematica = tscl.getNombre();
		actividadTipologiaCeMIT.nivel = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", language).getString("nivel." + sc.getNivel());
		actividadTipologiaCeMIT.nombre = cl.getApodo();
		actividadTipologiaCeMIT.descripcion = (scl.getDescripcion() != null) ? scl.getDescripcion() : "";
		actividadTipologiaCeMIT.fechaInicio = c.getFechaInicio();
		actividadTipologiaCeMIT.fechaFin = c.getFechaFin();
		actividadTipologiaCeMIT.numeroHoras = c.getNumeroHoras();

		return actividadTipologiaCeMIT;
	}
	
	private WSActividadTipologiaCeMIT crearWSActividadTipologiaCeMIT(Charla c, CharlaLanguage cl, Tematica tc,
																	 TematicaLanguage tcl, Locale language) {
		WSActividadTipologiaCeMIT actividadTipologiaCeMIT = new WSActividadTipologiaCeMIT();
		
		actividadTipologiaCeMIT.idActividad = c.getId();
		actividadTipologiaCeMIT.idCentro = c.getCentro().getId();
		actividadTipologiaCeMIT.nombreCentro = c.getCentro().getNombre();
		actividadTipologiaCeMIT.tipo = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", language).getString(TipoActividad.CHARLA.getNombre());
		actividadTipologiaCeMIT.idTematica = tc.getId();
		actividadTipologiaCeMIT.tematica = tcl.getNombre();
		actividadTipologiaCeMIT.nivel = "";
		actividadTipologiaCeMIT.nombre = cl.getNombre();
		actividadTipologiaCeMIT.descripcion = (cl.getDescripcion() != null) ? cl.getDescripcion() : ""; // RF_15_29 Campo observación nas charlas
		actividadTipologiaCeMIT.fechaInicio = c.getFechaInicio();
		actividadTipologiaCeMIT.fechaFin = c.getFechaInicio();
		actividadTipologiaCeMIT.numeroHoras = (float)((c.getHoraFin().getTime() - c.getHoraInicio().getTime()) / 1000.0 / 60.0 / 60.0);

		return actividadTipologiaCeMIT;
	}
	
	/**
	 * WSUsuarioCeMIT class.
	 */
	public static class WSFormularioCeMIT {
		private HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.currentRequestAttributes()).getRequest();
		public Long idActividad;
		public String nombreActividad;
		public String fechasActividad;
		public String tipoFormulario;
		public String nombreFormulario;
		public String enlaceFormulario = request.getScheme() + "://" + request.getServerName() + "/usuarios/spring/expediente";
	}
	
	private WSFormularioCeMIT crearFormularioCeMIT(Curso c, CursoLanguage cl, FormularioLanguage fl, MatriculaCurso mc,
												   Locale language) {
		WSFormularioCeMIT formularioCeMIT = null;
	
		if(mc.isAdmitido() && c.getCuestionarioSatisfaccion() != null && (mc.getAsistencia() >= 80) &&
			mc.isFormularioConcretoEditable() == true) {
			formularioCeMIT = new WSFormularioCeMIT();
			
			formularioCeMIT.idActividad = c.getId();
			formularioCeMIT.nombreActividad = cl.getApodo();
			formularioCeMIT.fechasActividad = c.getFechasLegibles();
			formularioCeMIT.nombreFormulario = fl.getNombre();
			formularioCeMIT.tipoFormulario = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", language).getString("examenes.encuesta");
		}

		return formularioCeMIT;
	}
	
	private List<WSFormularioCeMIT> crearFormularioCeMIT(Curso c, CursoLanguage cl, MatriculaCurso mc, String languageISO) {
		List<WSFormularioCeMIT> wsff = new ArrayList<WSFormularioCeMIT>();
		Locale language = new Locale(languageISO.split("_")[0], languageISO.split("_")[1]);
		
		int l = (languageISO.split("_")[0].equals("gl")) ? 1 : 0;
		
		if(!mc.isAdmitido() && mc.isEnListaEspera() && mc.isPreMatriculadoWeb() && c.getExamenNivel() != null &&
			mc.getExamenNivel() == null) {
			
			WSFormularioCeMIT wsf = new WSFormularioCeMIT();
			wsf.idActividad = c.getId();
			wsf.nombreActividad = cl.getApodo();
			wsf.fechasActividad = c.getFechasLegibles();
			wsf.tipoFormulario = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", language).getString("examenes.pruebaNivel");
			wsf.nombreFormulario = c.getExamenNivel().getTextos().get(l).getNombre();
			wsff.add(wsf);
		}
		
		if(c.getExamen() != null && c.isHayExamenInicial() && mc.isAdmitido() && mc.getExamenInicial() == null && 
			new Date().getTime() >= c.getFechaInicio().getTime() && 
			new Date().getTime() < (c.getFechaInicio().getTime() + 86400000) ) {
			
			WSFormularioCeMIT wsf = new WSFormularioCeMIT();
			wsf.idActividad = c.getId();
			wsf.nombreActividad = cl.getApodo();
			wsf.fechasActividad = c.getFechasLegibles();
			wsf.tipoFormulario = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", language).getString("examenes.inicial");
			wsf.nombreFormulario = c.getExamen().getTextos().get(l).getNombre();
			wsff.add(wsf);
		}
		
		if(c.getExamen() != null && mc.isAdmitido() && mc.getExamenFinal() == null && (mc.getAsistencia() >= 80) &&
			new Date().getTime() >= c.getFechaFin().getTime() &&
			new Date().getTime() < (c.getFechaFin().getTime() + 86400000)) {
			
			WSFormularioCeMIT wsf = new WSFormularioCeMIT();
			wsf.idActividad = c.getId();
			wsf.nombreActividad = cl.getApodo();
			wsf.fechasActividad = c.getFechasLegibles();
			wsf.tipoFormulario = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", language).getString("examenes.final");
			wsf.nombreFormulario = c.getExamen().getTextos().get(l).getNombre();
			wsff.add(wsf);
		}
		
		return wsff;
	}
}
