package es.csa.fdt.cyldigital.integracion;

/*
 * #%L
 * Servicios Integrados para aplicaciones remotas - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.text.DateFormat;
import java.text.SimpleDateFormat;

import java.util.List;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Map;
import java.util.HashMap;
import java.util.Date;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jws.WebMethod;
import javax.jws.WebService;

import javax.persistence.NoResultException;
import javax.persistence.Query;
import javax.servlet.http.HttpServletRequest;
import javax.xml.ws.WebServiceContext;
import javax.xml.ws.handler.MessageContext;

import org.apache.cxf.jaxws.context.WebServiceContextImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import es.csa.fdt.cyldigital.recursos.service.RemoteNoticeService;
import es.csa.fdt.cyldigital.usuarios.entity.Localidad;
import es.csa.fdt.cyldigital.usuarios.entity.Provincia;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.servicios.entity.Servicio.Nivel;
import es.csa.fdt.cyldigital.servicios.entity.Actividad;
import es.csa.fdt.cyldigital.servicios.entity.Charla;
import es.csa.fdt.cyldigital.servicios.entity.Curso;
import es.csa.fdt.cyldigital.servicios.entity.Tematica;
import es.csa.fdt.cyldigital.language.entity.ServicioCursoLanguage;
import es.csa.fdt.cyldigital.language.entity.TematicaLanguage;
import es.csa.fdt.cyldigital.language.entity.CursoLanguage;
import es.csa.fdt.cyldigital.language.entity.CharlaLanguage;

/**
 * Clase integracionCeMIT.
 */
@Component("integracionCeMIT")
@WebService
public class integracionCeMIT {
	
	private static final Logger logger = Logger.getLogger(integracionCeMIT.class.getName());
	
	@Autowired
	private RemoteNoticeService service;
	
	private WebServiceContext wsContext = new WebServiceContextImpl();
	
	/**
	 * Método web que calcula las provincias que tengan algún centro de la red CeMIT.
	 * 
	 * @param fieldOrder
	 * 			(opcional)
	 * 			Nombre del campo del objeto Provincia por el que se desea ordenar.
	 * 			"" o null indica que no se desea ordenar.
	 * 
	 * @param orderASC
	 * 			(opcional)
	 * 			Indica en que sentido se desea ordenar el resultado. Si fieldOrder es "" o null este parámetro es ignorado.
	 *			Si no se especifica, se toma por defecto orden ascendente.
	 * 			true indica ascendente y false descendente. 
	 *
	 * @param recordsByPage
	 * 			(opcional)
	 *          Número de registros a devolver.  
	 *          <= 0 indica devolver todos los registros.
	 *           
	 * @param page
	 * 			(opcional)
	 *          Número de página a devolver. Sólo si el parámetro recordsByPage es > 0.
	 *          En el caso de especificar un número <= 0 se ignora el parámetro recordsByPage y se devuelven todos los registros.
	 *          1 es la primera página.
	 * 
	 * @return listado con Identificador y Nombre de las provincias que tengan
	 *         algún centro de la red CeMIT.
	 */
	@WebMethod
	@Transactional
	public List<WSProvincia> ProvinciasCeMIT(String fieldOrder, Boolean orderASC, Integer recordsByPage, Integer page) {
		
		long startTime = System.currentTimeMillis();
		
		List<WSProvincia> wsProvincias = new ArrayList<WSProvincia>();
		
		String strqry = "SELECT DISTINCT e.provincia " +
						"FROM " + Centro.class.getName() + " e " + 
						"WHERE e.desactivado = 0";
					
		if (fieldOrder != null)
			if (!fieldOrder.isEmpty()) {
				if (orderASC == null)
					orderASC = true;
				
				strqry += " ORDER BY e.provincia." + fieldOrder + (orderASC ? " ASC" : " DESC");
			}

		try {
			Query command = service.getEntityManager().createQuery(strqry);
	
			if ((recordsByPage != null) && (page != null))
				if ((recordsByPage > 0) && (page > 0)) {
					command.setFirstResult((page - 1) * recordsByPage);
					command.setMaxResults(recordsByPage);
				}

			@SuppressWarnings("unchecked")
			List<Provincia> provincias = (List<Provincia>)command.getResultList();

			for (Provincia p : provincias)
				wsProvincias.add(crearWSProvincia(p));
		}
		catch (Exception e) {
			logger.log(Level.SEVERE,
					   "ProvinciasCeMIT: Error al devoler las Provincias con centros CeMIT: " +
					   e.getLocalizedMessage());

		    e.printStackTrace();
		}
		
		long elapsedTime = System.currentTimeMillis() - startTime;
		logger.log(Level.INFO, "Webservice ProvinciasCeMIT request - " + elapsedTime + "ms");

		return wsProvincias;
	}

	/**
	 * Método web que calcula las localidades con tengan algún centro de la red CeMit en una provincia.
	 *
	 * @param idProvincia
	 * 		    Identificador de la provincia de la cual se quieren conocer sus localidades con centros CeMIT.
	 * 
	 * @param fieldOrder
	 * 			(opcional)
	 * 			Nombre del campo del objeto Provincia por el que se desea ordenar.
	 * 			"" o null indica que no se desea ordenar.
	 * 
	 * @param orderASC
	 * 			(opcional)
	 * 			Indica en que sentido se desea ordenar el resultado. Si fieldOrder es "" o null este parámetro es ignorado.
	 *			Si no se especifica, se toma por defecto orden ascendente.
	 * 			true indica ascendente y false descendente. 
	 *
	 * @param recordsByPage
	 * 			(opcional)
	 *          Número de registros a devolver.  
	 *          <= 0 indica devolver todos los registros.
	 *           
	 * @param page
	 * 			(opcional)
	 *          Número de página a devolver. Sólo si el parámetro recordsByPage es > 0.
	 *          En el caso de especificar un número <= 0 se ignora el parámetro recordsByPage y se devuelven todos los registros.
	 *          1 es la primera página.
	 * 
	 * @return listado con Identificador y Nombre de las localidades existentes en la provinciaprovincias que tengan
	 *         algún centro de la red CeMIT.
	 */
	@WebMethod
	@Transactional
	public List<WSLocalidad> LocalidadesCeMIT(Long idProvincia, String fieldOrder, Boolean orderASC, Integer recordsByPage, Integer page) {
		
		long startTime = System.currentTimeMillis();

		List<WSLocalidad> wsLocalidades = new ArrayList<WSLocalidad>();

		String strqry = "SELECT DISTINCT e.localidad " +
						"FROM " + Centro.class.getName() + " e " + 
						"WHERE (e.desactivado = 0) AND" +
						"      (e.provincia.id = " + idProvincia + ")";
	
		if (fieldOrder != null)
			if (!fieldOrder.isEmpty()) {
				if (orderASC == null)
					orderASC = true;
				
				strqry += " ORDER BY e.localidad." + fieldOrder + (orderASC ? " ASC" : " DESC");
			}
		
		try {
			Query command = service.getEntityManager().createQuery(strqry);
			
			if ((recordsByPage != null) && (page != null))
				if ((recordsByPage > 0) && (page > 0)) {
					command.setFirstResult((page - 1) * recordsByPage);
					command.setMaxResults(recordsByPage);
				}
			
			@SuppressWarnings("unchecked")
			List<Localidad> localidades = (List<Localidad>)command.getResultList();
			
			for (Localidad l : localidades)
				wsLocalidades.add(crearWSLocalidad(l));
		}
		catch (Exception e) {
			logger.log(Level.SEVERE,
					   "LocalidadesCeMIT: Error al devoler las Localidades con centros CeMIT: " +
					   e.getLocalizedMessage());
		
			e.printStackTrace();
		}
		
		long elapsedTime = System.currentTimeMillis() - startTime;
		logger.log(Level.INFO, "Webservice LocalidadesCeMIT request - " + elapsedTime + "ms");

		return wsLocalidades;
	}
  
	/**
	 * Método web que busca los centros CeMIT existentes en una localidad de una provincia y que su nombre cumple
	 * el patrón indicado.
	 *
	 * @param idProvincia
	 * 			(opcional)
	 * 			Identificador de la provincia de la que se desean buscar los centros.
	 * 			null ó <=0 indica 'cualquier provincia'
	 * 
	 * @param idLocalidad
	 * 			(opcional)
	 * 			Identificador de la localidad de la que se desean buscar los centros.
	 * 			null ó <=0 indica 'cualquier localidad'
	 * 
	 * @param patronCentro
	 * 			(opcional)
	 * 			Cadena de texto patrón para la identificación de centros. No aplica si es "" o null.
	 * 
	 * @param languageISO
	 * 			Identificador ISO del idioma en el que se desean los resultados ('gl_ES' o 'es_ES').
	 * 
	 * @param fieldOrder
	 * 			(opcional)
	 * 			Nombre del campo del objeto Provincia por el que se desea ordenar.
	 * 			"" o null indica que no se desea ordenar.
	 * 
	 * @param orderASC
	 * 			(opcional)
	 * 			Indica en que sentido se desea ordenar el resultado. Si fieldOrder es "" o null este parámetro es ignorado.
	 *			Si no se especifica, se toma por defecto orden ascendente.
	 * 			true indica ascendente y false descendente. 
	 *
	 * @param recordsByPage
	 * 			(opcional)
	 *          Número de registros a devolver.  
	 *          <= 0 indica devolver todos los registros.
	 *           
	 * @param page
	 * 			(opcional)
	 *          Número de página a devolver. Sólo si el parámetro recordsByPage es > 0.
	 *          En el caso de especificar un número <= 0 se ignora el parámetro recordsByPage y se devuelven todos los registros.
	 *          1 es la primera página.
	 * 
	 * @return listado con todos los datos de cada centro CeMIT existente en la localidad y provincia y que su nombre cumpla
	 * 		   el patrón indicado.
	 */
	@WebMethod
	@Transactional
	public List<WSCentro> BuscaCentrosCeMIT(Long idProvincia, Long idLocalidad, String patronCentro, String languageISO,
											String fieldOrder, Boolean orderASC, Integer recordsByPage, Integer page) {
		
		long startTime = System.currentTimeMillis();

		List<WSCentro> wsCentros = new ArrayList<WSCentro>();
		
		Locale language = new Locale(languageISO.split("_")[0], languageISO.split("_")[1]);

		String strqry = "SELECT e " +
						"FROM " + Centro.class.getName() + " e " + 
						"WHERE (e.desactivado = 0)";
		
		if ((idProvincia != null) && (idProvincia > 0))
			strqry += " AND (e.provincia.id = " + idProvincia + ")";
		
		if ((idLocalidad != null) && (idLocalidad > 0))
			strqry += " AND (e.localidad.id = " + idLocalidad + ")";
		
		if ((patronCentro != null) && (!patronCentro.isEmpty()))
			strqry += " AND (e.nombre LIKE '%" + patronCentro + "%')";

		if (fieldOrder != null)
			if (!fieldOrder.isEmpty()) {
				if (orderASC == null)
					orderASC = true;
				
				if (fieldOrder.compareTo("idCentro") == 0) {
					strqry += " ORDER BY e.id " + (orderASC ? "ASC" : "DESC");
				}
				else if (fieldOrder.compareTo("localidad") == 0) {
					strqry += " ORDER BY e.localidad.nombre " + (orderASC ? "ASC" : "DESC");
				}
				else if (fieldOrder.compareTo("provincia") == 0) {
					strqry += " ORDER BY e.provincia.nombre " + (orderASC ? "ASC" : "DESC");
				}
				else if (fieldOrder.compareTo("horario") == 0) {
					strqry += " ORDER BY e.horaIniManana " + (orderASC ? "ASC, " : "DESC, ") +
							  "          e.horaIniTarde " + (orderASC ? "ASC, " : "DESC, ") +
							  "          e.horaIniSabado " + (orderASC ? "ASC" : "DESC");
				}
				else {
					strqry += " ORDER BY e." + fieldOrder + (orderASC ? " ASC" : " DESC");
				}
			}
		
		try {
			Query command = service.getEntityManager().createQuery(strqry);
			
			if ((recordsByPage != null) && (page != null))
				if ((recordsByPage > 0) && (page > 0)) {
					command.setFirstResult((page - 1) * recordsByPage);
					command.setMaxResults(recordsByPage);
				}
			
			@SuppressWarnings("unchecked")
			List<Centro> centros = (List<Centro>)command.getResultList();
			
			for (Centro c : centros)
				wsCentros.add(crearWSCentro(c, language));
		}
		catch (Exception e) {
			logger.log(Level.SEVERE,
					   "BuscaCentrosCeMIT: Error al devolver la búsqueda de centros CeMIT: " +
					   e.getLocalizedMessage());
		
			e.printStackTrace();
		}

		long elapsedTime = System.currentTimeMillis() - startTime;
		logger.log(Level.INFO, "Webservice BuscaCentrosCeMIT request - " + elapsedTime + "ms");

		return wsCentros;
	}

	/**
	 * Método web que consulta los datos de un centro CeMIT.
	 *
	 * @param idCentro
	 * 			Identificador del centro CeMIT.
	 * 
	 * @param languageISO
	 * 			Identificador ISO del idioma en el que se desean los resultados ('gl_ES' o 'es_ES').
	 *  
	 * @return Centro de la red CeMIT asociado al identificador indicado.
	 */
	@WebMethod
	@Transactional
	public WSCentro CentroCeMIT(Long idCentro, String languageISO) {
		
		long startTime = System.currentTimeMillis();

		WSCentro wsCentro = null;
		
		Locale language = new Locale(languageISO.split("_")[0], languageISO.split("_")[1]);
		
		try {
			Centro centro = service.find(Centro.class, idCentro);
			
			wsCentro = crearWSCentro(centro, language);
			}
		catch (Exception e) {
			logger.log(Level.SEVERE,
					   "CentroCeMIT: Error al consultar el centro CeMIT: " +
					   e.getLocalizedMessage());
		
			e.printStackTrace();
		}

		long elapsedTime = System.currentTimeMillis() - startTime;
		logger.log(Level.INFO, "Webservice CentroCeMIT request - " + elapsedTime + "ms");
		
		return wsCentro;
	}

	/**
	 * Método web que consulta todos los centros de la red CeMIT.
	 * @param languageISO
	 * 			Identificador ISO del idioma en el que se desean los resultados ('gl_ES' o 'es_ES').
	 * 
	 * @param fieldOrder
	 * 			(opcional)
	 * 			Nombre del campo del objeto Provincia por el que se desea ordenar.
	 * 			"" o null indica que no se desea ordenar.
	 * 
	 * @param orderASC
	 * 			(opcional)
	 * 			Indica en que sentido se desea ordenar el resultado. Si fieldOrder es "" o null este parámetro es ignorado.
	 *			Si no se especifica, se toma por defecto orden ascendente.
	 * 			true indica ascendente y false descendente. 
	 * 
	 * @param recordsByPage
	 * 			(opcional)
	 *          Número de registros a devolver.  
	 *          <= 0 indica devolver todos los registros.
	 *           
	 * @param page
	 * 			(opcional)
	 *          Número de página a devolver. Sólo si el parámetro recordsByPage es > 0.
	 *          En el caso de especificar un número <= 0 se ignora el parámetro recordsByPage y se devuelven todos los registros.
	 *          1 es la primera página.
	 * 
	 * @return listado con todos los datos de cada centro CeMIT.
	 */
	@WebMethod
	@Transactional
	public List<WSCentro> CentrosCeMIT(String languageISO, String fieldOrder, Boolean orderASC, Integer recordsByPage, Integer page) {
		
		long startTime = System.currentTimeMillis();

		List<WSCentro> wsCentros = new ArrayList<WSCentro>();
		
		Locale language = new Locale(languageISO.split("_")[0], languageISO.split("_")[1]);

		String strqry = "SELECT e " +
						"FROM " + Centro.class.getName() + " e " + 
						"WHERE (e.desactivado = 0)";

		if (fieldOrder != null)
			if (!fieldOrder.isEmpty()) {
				if (orderASC == null)
					orderASC = true;
				
				if (fieldOrder.compareTo("idCentro") == 0) {
					strqry += " ORDER BY e.id " + (orderASC ? "ASC" : "DESC");
				}
				else if (fieldOrder.compareTo("localidad") == 0) {
					strqry += " ORDER BY e.localidad.nombre " + (orderASC ? "ASC" : "DESC");
				}
				else if (fieldOrder.compareTo("provincia") == 0) {
					strqry += " ORDER BY e.provincia.nombre " + (orderASC ? "ASC" : "DESC");
				}
				else if (fieldOrder.compareTo("horario") == 0) {
					strqry += " ORDER BY e.horaIniManana " + (orderASC ? "ASC, " : "DESC, ") +
							  "          e.horaIniTarde " + (orderASC ? "ASC, " : "DESC, ") +
							  "          e.horaIniSabado " + (orderASC ? "ASC" : "DESC");
				}
				else {
					strqry += " ORDER BY e." + fieldOrder + (orderASC ? " ASC" : " DESC");
				}
			}
		
		try {
			Query command = service.getEntityManager().createQuery(strqry);
			
			if ((recordsByPage != null) && (page != null))
				if ((recordsByPage > 0) && (page > 0)) {
					command.setFirstResult((page - 1) * recordsByPage);
					command.setMaxResults(recordsByPage);
				}
			
			@SuppressWarnings("unchecked")
			List<Centro> centros = (List<Centro>)command.getResultList();
			
			for (Centro c : centros)
				wsCentros.add(crearWSCentro(c, language));
		}
		catch (Exception e) {
			logger.log(Level.SEVERE,
					   "CentrosCeMIT: Error al devoler todos los centros CeMIT: " +
					   e.getLocalizedMessage());
		
			e.printStackTrace();
		}

		long elapsedTime = System.currentTimeMillis() - startTime;
		logger.log(Level.INFO, "Webservice CentrosCeMIT request - " + elapsedTime + "ms");

		return wsCentros;
	}

	/**
	 * Método web que consulta los tipos de actividades impartidas en los centros de la red CeMIT.
	 *
	 * @param languageISO
	 * 			Identificador ISO del idioma en el que se desean los resultados ('gl_ES' o 'es_ES').
	 * 
	 * @return Listado con los nombres de los tipos de actividades CeMIT.
	 */
	@WebMethod
	@Transactional
	public List<WSTipoActividad> TiposActividadCeMIT(String languageISO) {
		
		long startTime = System.currentTimeMillis();

		List<WSTipoActividad> wsTiposActividad = new ArrayList<WSTipoActividad>();
		
		try {
			Locale language = new Locale(languageISO.split("_")[0], languageISO.split("_")[1]);

			for (TipoActividad tipoActividad : TipoActividad.values())
				wsTiposActividad.add(crearWSTipoActividad(tipoActividad, language));
		}
		catch (Exception e) {
			logger.log(Level.SEVERE,
					   "TiposActividadCeMIT: Error al devolver los tipos de actividades CeMIT en idioma " + languageISO + ": " +
					   e.getLocalizedMessage());
		
			e.printStackTrace();
		}

		long elapsedTime = System.currentTimeMillis() - startTime;
		logger.log(Level.INFO, "Webservice TiposActividadCeMIT request - " + elapsedTime + "ms");
		
		return wsTiposActividad;
	}
  
	/**
	 * Método web que consulta las temática asignadas a todas las actividades del tipo indicado.
	 *
	 * @param idTipoActividad
	 * 			(obligatorio)
	 * 			Identificador del tipo de actividad a consultar.
	 *
	 * @param languageISO
	 * 			(obligatorio)
	 * 			Identificador ISO del idioma en el que se desean los resultados ('gl_ES' o 'es_ES').
	 * 
	 * @param fieldOrder
	 * 			(opcional)
	 * 			Nombre del campo del objeto Provincia por el que se desea ordenar.
	 * 			"" o null indica que no se desea ordenar.
	 * 
	 * @param orderASC
	 * 			(opcional)
	 * 			Indica en que sentido se desea ordenar el resultado. Si fieldOrder es "" o null este parámetro es ignorado.
	 *			Si no se especifica, se toma por defecto orden ascendente.
	 * 			true indica ascendente y false descendente. 
	 * 
	 * @param recordsByPage
	 * 			(opcional)
	 *          Número de registros a devolver.  
	 *          <= 0 indica devolver todos los registros.
	 *           
	 * @param page
	 * 			(opcional)
	 *          Número de página a devolver. Sólo si el parámetro recordsByPage es > 0.
	 *          En el caso de especificar un número <= 0 se ignora el parámetro recordsByPage y se devuelven todos los registros.
	 *          1 es la primera página.
	 * 
	 * @return listado con Identificador y Nombre de las temáticas asignadas a las actividades del centro.
	 */
	@WebMethod
	@Transactional
	public List<WSTematica> TematicasActividadesCeMIT(Long idTipoActividad, String languageISO,
													  String fieldOrder, Boolean orderASC, Integer recordsByPage, Integer page) {
		
		long startTime = System.currentTimeMillis();

		List<WSTematica> wsTematicas = new ArrayList<WSTematica>();
		
		String strqryActividades = null;
		
		if (idTipoActividad == TipoActividad.CURSO.getIdTipoActividad()) {
			strqryActividades = "SELECT DISTINCT e.servicio.tematica.id " +
								"FROM " + Curso.class.getName() + " e " +
								"WHERE (e.desactivado = 0) AND" +
								"      (e.servicio.tematica IS NOT NULL)";
		}
		else if (idTipoActividad == TipoActividad.CHARLA.getIdTipoActividad()) {
			strqryActividades = "SELECT DISTINCT e.tematica.id " +
								"FROM " + Charla.class.getName() + " e " +
								"WHERE (e.desactivado = 0) AND" +
								"      (e.tematica IS NOT NULL)";
		}

		if (strqryActividades != null)
		{
			String strqry = "SELECT e " +
							"FROM " + TematicaLanguage.class.getName() + " e " +
							"WHERE (e.desactivado = 0) AND " +
							"      (e.language.ISO LIKE '" + languageISO + "') AND" +
							"      (e.tematica.id IN (" + strqryActividades + "))";
	
			if (fieldOrder != null)
				if (!fieldOrder.isEmpty()) {
					if (orderASC == null)
						orderASC = true;
	
					if (fieldOrder.compareTo("idTematica") == 0) {
						strqry += " ORDER BY e.tematica.id " + (orderASC ? "ASC" : "DESC");
					}
					else if (fieldOrder.compareTo("padre") == 0) {
						strqry += " ORDER BY e.tematica.padre.id " + (orderASC ? "ASC" : "DESC");
					}
					else { //nombre o error
						strqry += " ORDER BY e." + fieldOrder + (orderASC ? " ASC" : " DESC");
					}
				}
	
			try {
				Query command = service.getEntityManager().createQuery(strqry);
				
				if ((recordsByPage != null) && (page != null))
					if ((recordsByPage > 0) && (page > 0)) {
						command.setFirstResult((page - 1) * recordsByPage);
						command.setMaxResults(recordsByPage);
					}
				
				@SuppressWarnings("unchecked")
				List<TematicaLanguage> tematicasLanguage = (List<TematicaLanguage>)command.getResultList();
				
				for (TematicaLanguage tl : tematicasLanguage)
					wsTematicas.add(crearWSTematica(tl));
			}
			catch (Exception e) {
				logger.log(Level.SEVERE,
						   "TematicasActividadesCeMIT: Error al devolver las temáticas de las actividades CeMIT: " +
						   e.getLocalizedMessage());
			
				e.printStackTrace();
			}
			
		} else{
			logger.log(Level.SEVERE,
					   "TematicasActividadesCeMIT: Error, el tipo de actividad no existe.");
		}

		long elapsedTime = System.currentTimeMillis() - startTime;
		logger.log(Level.INFO, "Webservice TematicasActividadesCeMIT request - " + elapsedTime + "ms");
		
		return wsTematicas;
	}

	/**
	 * Método web que busca las actividades CeMIT existentes por localización, tipo, temática y patrón.
	 *
	 * @param idProvincia
	 * 			(opcional)
	 * 			Identificador de la provincia a la que deben pertenecer la actividades consultadas.
	 * 			Tomado en cuenta únicamente si el parámetro idCentro es null o <=0
	 * 			null ó <=0 indica 'cualquier provincia'
	 * 
	 * @param idLocalidad
	 * 			(opcional)
	 * 			Identificador de la localidad a la que deben pertenecer la actividades consultadas.
	 * 			Tomado en cuenta únicamente si el parámetro idCentro es null o <=0
	 * 			null ó <=0 indica 'cualquier localidad'
	 * 
	 * @param idTipoActividad
	 * 			(obligatorio)
	 * 			Identificador del tipo de actividad al que deben pertenecer la actividades consultadas.
	 * 			CURSO o CHARLA
	 * 			
	 * 
	 * @param idTematica
	 * 			(opcional)
	 * 			Identificador de la temática a la que deben pertenecer las actividades consultadas.
	 * 			null ó <0 indica 'todas las temáticas'
	 * 
	 * @param idCentro
	 * 			(opcional)
	 * 			Identificador del Centro del que se quieren obtener las actividades.
	 * 			null ó <0 indica 'cualquier centro' 
	 * 
	 * @param patronActividad
	 * 			(opcional)
	 * 			Cadena de texto patrón para la identificación de actividades por el nombre. Modo like.
	 * 			null, "" o "%" indica cualquier nombre
	 * 
	 * @param languageISO
	 * 			(obligatorio)
	 * 			Identificador ISO del idioma en el que se desean los resultados ('gl_ES' o 'es_ES').
	 * 
	 * @param fieldOrder
	 * 			(opcional)
	 * 			Nombre del campo del objeto Provincia por el que se desea ordenar.
	 * 			"" o null indica que no se desea ordenar.
	 * 
	 * @param orderASC
	 * 			(opcional)
	 * 			Indica en que sentido se desea ordenar el resultado. Si fieldOrder es "" o null este parámetro es ignorado.
	 *			Si no se especifica, se toma por defecto orden ascendente.
	 * 			true indica ascendente y false descendente. 
	 * 
	 * @param recordsByPage
	 * 			(opcional)
	 *          Número de registros a devolver.  
	 *          <= 0 indica devolver todos los registros.
	 *           
	 * @param page
	 * 			(opcional)
	 *          Número de página a devolver. Sólo si el parámetro recordsByPage es > 0.
	 *          En el caso de especificar un número <= 0 se ignora el parámetro recordsByPage y se devuelven todos los registros.
	 *          1 es la primera página.
	 * 
	 * @return listado con todos los datos de cada una de las actividades existentes en la localidad y provincia indicadas,
	 * 		   filtradas por el tipo y la temática especificdas y que su nombre cumpla el patrón recibido.
	 */
	@WebMethod
	@Transactional
	public List<WSActividad> BuscaActividadesCeMIT(Long idProvincia, Long idLocalidad, Long idTipoActividad, Long idTematica,
												   Long idCentro, String patronActividad, String languageISO,
												   String fieldOrder, Boolean orderASC, Integer recordsByPage, Integer page) {
		
		long startTime = System.currentTimeMillis();

		List<WSActividad> wsActividades = new ArrayList<WSActividad>();
		
		Locale language = new Locale(languageISO.split("_")[0], languageISO.split("_")[1]);

		String patterFieldName = "";
		
		if (idTipoActividad == TipoActividad.CURSO.getIdTipoActividad())
			patterFieldName = "apodo";
		else if (idTipoActividad == TipoActividad.CHARLA.getIdTipoActividad())
			patterFieldName = "nombre";

		if (((idTipoActividad == TipoActividad.CURSO.getIdTipoActividad()) ||
			 (idTipoActividad == TipoActividad.CHARLA.getIdTipoActividad())) &&
			!languageISO.isEmpty()) {
			
			String strqry = "";
			
			//SELECCION, TABLAS Y FILTRADOS FIJOS
			if (idTipoActividad == TipoActividad.CURSO.getIdTipoActividad())
				strqry = "SELECT c.id AS idActividad, " +
								"c.centro.id AS idCentro, " +
								"c.centro.nombre AS nombreCentro, " +
								"tl.nombre AS tematica, " +
								"scl.servicioCurso.nivel AS nivel, " +
								"cl.apodo AS nombre, " +
								"scl.descripcion AS descripcion, " +
								"c.numeroPlazas AS plazasTotales, " +
								"(SELECT CASE WHEN (c.numeroPlazas - CAST(count(m) AS integer)) < 0 THEN 0 ELSE (c.numeroPlazas - CAST(count(m) AS integer)) END FROM c.matriculas m WHERE (m.admitido = TRUE AND m.enListaEspera = FALSE) AND (m.desactivado = 0)) as plazasLibres, " +
								"scl.ficheroGuion.id AS idFicheroGuion, " +
								"c.observaciones AS observaciones, " +
								"p.nombre || ' ' || p.apellido1 || ' ' || p.apellido2 AS profesor, " +
								"c.horaInicio AS horaInicio, " +
								"c.horaFin AS horaFin, " +
								"c.fechaInicio AS fechaInicio, " +
								"c.fechaFin AS fechaFin, " +
								"scl.servicio.edadMinima AS edadMinima, " +
								"scl.servicio.edadMaxima AS edadMaxima, " +
								"c.numeroHoras AS numeroHoras, " +
								"(SELECT CAST(count(m) AS integer) FROM c.matriculas m WHERE (m.desactivado = 0 AND m.cancelada = 0)) as numeroInscritos, " +
								"c.fechaLimiteInscripcion AS fechaLimiteInscripcion, " + // new line RF_15_47
								"c.horaLimiteInscripcion AS horaLimiteInscripcion, " + //new line RF_15_47
								"c, " +	//new line RF_15_47
								"c.servicio.tematica.id AS idTematica, " + //R 3.4.15
								"c.servicio.tematica.padre.id AS idTematicaPadre " + //R 3.4.15
						 "FROM es.csa.fdt.cyldigital.servicios.entity.Curso c INNER JOIN " +
							  "c.textosCurso cl INNER JOIN " +
							  "cl.language lc WITH (lc.ISO LIKE :languageISO) INNER JOIN " +
							  "c.servicio.tematica.textos tl INNER JOIN " +
							  "tl.language lt WITH (lt.ISO LIKE :languageISO) LEFT JOIN " +
							  "c.profesor p, " +
							  "es.csa.fdt.cyldigital.language.entity.ServicioCursoLanguage scl INNER JOIN " +
							  "scl.language lsc WITH (lsc.ISO = :languageISO) " +
						 "WHERE (c.desactivado = 0) AND ((c.fechaPublicacion IS NULL AND c.horaPublicacion IS NULL) OR (c.fechaPublicacion < curdate()) OR (c.fechaPublicacion = curdate() AND (c.horaPublicacion <= curtime() OR c.horaPublicacion IS NULL))) AND " +
						 
							   "(c.servicio.id = scl.servicio.id)";
			else
				strqry = "SELECT c.id AS idActividad, " +
								"c.centro.id AS idCentro, " +
								"c.centro.nombre AS nombreCentro, " +
								"tl.nombre AS tematica, " +
								"cl.nombre AS nombre, " +
								"cl.descripcion AS descripcion, " + // RF_15_29 Campo observación nas charlas
								"c.numeroPlazas AS plazasTotales, " +								
								"(SELECT CASE WHEN (c.numeroPlazas - CAST(count(m) AS integer)) < 0 THEN 0 ELSE (c.numeroPlazas - CAST(count(m) AS integer)) END FROM c.matriculas m WHERE (m.admitido = TRUE AND m.enListaEspera = FALSE) AND (m.desactivado = 0)) as plazasLibres, " +
								"c.observaciones AS observaciones, " +
								"p.nombre || ' ' || p.apellido1 || ' ' || p.apellido2 AS profesor, " +
								"c.horaInicio AS horaInicio, " +
								"c.horaFin AS horaFin, " +
								"c.fechaInicio AS fechaInicio, " +
								"c.fechaInicio AS fechaFin, " +
								"c.edadMinima AS edadMinima, " +
								"c.edadMaxima AS edadMaxima, " +
								"time_to_sec(c.horaFin - c.horaInicio) / 3600.0F AS numeroHoras, " +
								"(SELECT CAST(count(m) AS integer) FROM c.matriculas m WHERE (m.desactivado = 0 AND m.cancelada = 0)) as numeroInscritos, " +
								"c.fechaLimiteInscripcion AS fechaLimiteInscripcion, " + // new line RF_15_47
								"c.horaLimiteInscripcion AS horaLimiteInscripcion, " + //new line RF_15_47
								"c, " +	//new line RF_15_47
								"c.servicio.tematica.id AS idTematica, " + //R 3.4.15
								"c.servicio.tematica.padre.id AS idTematicaPadre " + //R 3.4.15
						 "FROM es.csa.fdt.cyldigital.servicios.entity.Charla c INNER JOIN " +
							  "c.textosCharla cl INNER JOIN " +
							  "cl.language lc WITH (lc.ISO LIKE :languageISO) INNER JOIN " +
							  "c.tematica.textos tl INNER JOIN " +
							  "tl.language lt WITH (lt.ISO LIKE :languageISO) LEFT JOIN " +
							  "c.profesor p " +
						 "WHERE (c.desactivado = 0) AND ((c.fechaPublicacion IS NULL AND c.horaPublicacion IS NULL) OR (c.fechaPublicacion < curdate()) OR (c.fechaPublicacion = curdate() AND (c.horaPublicacion <= curtime() OR c.horaPublicacion IS NULL))) " ;

			//FILTRADO
			if ((idCentro != null) && (idCentro > 0))
				strqry += " AND (c.centro.id = :idCentro)";
			else {
				if ((idProvincia != null) && (idProvincia > 0))
					strqry += " AND (c.centro.provincia.id = :idProvincia)";
				
				if ((idLocalidad != null) && (idLocalidad > 0))
					strqry += " AND (c.centro.localidad.id = :idLocalidad)";
			}
	
			if ((idTematica != null) && (idTematica > 0))
				strqry += " AND (tl.tematica.id = :idTematica)";

			if ((patronActividad != null) && (!patronActividad.isEmpty()))
					strqry += " AND ((c.centro.nombre LIKE :patronActividad) OR" + //new line RF_15_24
							" (c.centro.provincia.nombre LIKE :patronActividad) OR" + //new line RF_15_24
							" (c.centro.localidad.nombre LIKE :patronActividad) OR" + //new line RF_15_24
							" (cl." + patterFieldName + " LIKE :patronActividad))";			

			//AGRUPACION (por el JOIN con las matriculaciones que de no agruparlo por actividad provocaría que se recuperase cada
			//            actividad tantas veces como matriculas haya en la misma)
			strqry += " GROUP BY c";
			
			//ORDENACION
			if ((fieldOrder != null) && (!fieldOrder.isEmpty())) {
				if (orderASC == null)
					orderASC = true;
	
				if (fieldOrder.compareTo("idActividad") == 0) {
					strqry += " ORDER BY c.id " + (orderASC ? "ASC" : "DESC");
				}
				else if (fieldOrder.compareTo("idCentro") == 0) {
					strqry += " ORDER BY c.centro.id " + (orderASC ? "ASC" : "DESC");
				}
				else if (fieldOrder.compareTo("nombreCentro") == 0) {
					strqry += " ORDER BY c.centro.nombre " + (orderASC ? "ASC" : "DESC");
				}
				else if (fieldOrder.compareTo("tematica") == 0) {
						strqry += " ORDER BY tl.nombre " + (orderASC ? "ASC" : "DESC");
				}
				else if ((fieldOrder.compareTo("nivel") == 0) && (idTipoActividad == TipoActividad.CURSO.getIdTipoActividad())) {
					strqry += " ORDER BY scl.servicioCurso.nivel " + (orderASC ? "ASC" : "DESC");
				}
				else if (fieldOrder.compareTo("nombre") == 0) {
					strqry += " ORDER BY cl." + patterFieldName + " " + (orderASC ? "ASC" : "DESC");
				}
				else if ((fieldOrder.compareTo("descripcion") == 0) && (idTipoActividad == TipoActividad.CURSO.getIdTipoActividad())) {
					strqry += " ORDER BY scl.descripcion " + (orderASC ? "ASC" : "DESC");
				}
				else if (fieldOrder.compareTo("plazasTotales") == 0) {
					strqry += " ORDER BY c.numeroPlazas " + (orderASC ? "ASC" : "DESC");
				}
				else if (fieldOrder.compareTo("plazasLibres") == 0) {
					/* performance issue strqry += " ORDER BY (c.numeroPlazas - count(distinct m)) " + (orderASC ? "ASC" : "DESC");*/
					strqry += " ORDER BY (c.numeroPlazas - count(m)) " + (orderASC ? "ASC" : "DESC");
				}
				else if (fieldOrder.compareTo("numeroInscritos") == 0) {
					strqry += " ORDER BY count(mi) " + (orderASC ? "ASC" : "DESC");
				}				
				else if ((fieldOrder.compareTo("programa") == 0) && (idTipoActividad == TipoActividad.CURSO.getIdTipoActividad())) {
					strqry += " ORDER BY scl.ficheroGuion.nombre " + (orderASC ? "ASC" : "DESC");
				}
				else if (fieldOrder.compareTo("observaciones") == 0) {
					strqry += " ORDER BY c.observaciones " + (orderASC ? "ASC" : "DESC");
				}
				else if (fieldOrder.compareTo("profesor") == 0) {
					strqry += " ORDER BY p.nombre " + (orderASC ? "ASC" : "DESC") + "," +
							  		   " p.apellido1 " + (orderASC ? "ASC" : "DESC") + "," +
							  		   " p.apellido2 "    + (orderASC ? "ASC" : "DESC");
				}
				else if (fieldOrder.compareTo("horario") == 0) {
					strqry += " ORDER BY c.horaInicio " + (orderASC ? "ASC" : "DESC") + ", " + 
										"c.horaFin " + (orderASC ? "ASC" : "DESC");
				}
				else if (fieldOrder.compareTo("fechaInicio") == 0) {
					strqry += " ORDER BY c.fechaInicio " + (orderASC ? "ASC" : "DESC");
				}
				else if ((fieldOrder.compareTo("fechaFin") == 0)) {
					if (idTipoActividad == TipoActividad.CURSO.getIdTipoActividad())
						strqry += " ORDER BY c.fechaFin " + (orderASC ? "ASC" : "DESC");
					else if (idTipoActividad == TipoActividad.CHARLA.getIdTipoActividad())
						strqry += " ORDER BY c.fechaInicio " + (orderASC ? "ASC" : "DESC");
				}
				else if (fieldOrder.compareTo("edadMinima") == 0) {
					if (idTipoActividad == TipoActividad.CURSO.getIdTipoActividad())
						strqry += " ORDER BY scl.servicio.edadMinima " + (orderASC ? "ASC" : "DESC");
					else if (idTipoActividad == TipoActividad.CHARLA.getIdTipoActividad())
						strqry += " ORDER BY c.edadMinima " + (orderASC ? "ASC" : "DESC");
				}
				else if (fieldOrder.compareTo("edadMaxima") == 0) {
					if (idTipoActividad == TipoActividad.CURSO.getIdTipoActividad())
						strqry += " ORDER BY scl.servicio.edadMaxima " + (orderASC ? "ASC" : "DESC");
					else if (idTipoActividad == TipoActividad.CHARLA.getIdTipoActividad())
						strqry += " ORDER BY c.edadMaxima " + (orderASC ? "ASC" : "DESC");
				}
				else if (fieldOrder.compareTo("numeroHoras") == 0) {
					if (idTipoActividad == TipoActividad.CURSO.getIdTipoActividad())
						strqry += " ORDER BY c.numeroHoras " + (orderASC ? "ASC" : "DESC");
					else if (idTipoActividad == TipoActividad.CHARLA.getIdTipoActividad())
						strqry += " ORDER BY time_to_sec(c.horaFin - c.horaInicio) / 3600.0F " + (orderASC ? "ASC" : "DESC");
				}
			}
			
			//FINAL
			try {
				Query command = service.getEntityManager().createQuery(strqry);
				
				//PAGINADO
				if ((recordsByPage != null) && (recordsByPage > 0) && (page != null) && (page > 0)) {
					command.setFirstResult((page - 1) * recordsByPage);
					command.setMaxResults(recordsByPage);
				}
				
				//IDIOMA
				command.setParameter("languageISO",	languageISO);
				
				//PARAMETROS
				if ((idCentro != null) && (idCentro > 0))
					command.setParameter("idCentro", idCentro);
				else {
					if ((idProvincia != null) && (idProvincia > 0))
						command.setParameter("idProvincia", idProvincia);
					
					if ((idLocalidad != null) && (idLocalidad > 0))
						command.setParameter("idLocalidad", idLocalidad);
				}
				
				if ((idTematica != null) && (idTematica > 0))
					command.setParameter("idTematica",	idTematica);
	
				if ((patronActividad != null) && (!patronActividad.isEmpty()))
					command.setParameter("patronActividad", "%" + patronActividad + "%");

				//EJECUCION
				@SuppressWarnings("unchecked")
				List<Object[]> actividades = command.getResultList();
				
				if (idTipoActividad == TipoActividad.CURSO.getIdTipoActividad()) {
					//COMPOSICION CURSOS
					for (Object[] cursodb : actividades) 
						wsActividades.add(crearWSActividadCurso(cursodb, language));
				}
				else if (idTipoActividad == TipoActividad.CHARLA.getIdTipoActividad()) {
					//COMPOSICION CHARLAS
					for (Object[] charladb : actividades)
						wsActividades.add(crearWSActividadCharla(charladb, language));
				}
			}
			catch (Exception e) {
				logger.log(Level.SEVERE,
						   "BuscaActividadesCeMIT: Error al devolver la búsqueda de actividades CeMIT: " +
						   e.getLocalizedMessage());
			
				e.printStackTrace();
			}
		}

		long elapsedTime = System.currentTimeMillis() - startTime;
		logger.log(Level.INFO, "Webservice BuscaActividadesCeMIT request - " + elapsedTime + "ms");
		
		//RESULTADO
		return wsActividades;
	}
	
	/**
	 * Método web que busca las próximas actividades CeMIT existentes.
	 *
	 * @param idProvincia
	 * 			(opcional)
	 * 			Identificador de la provincia a la que deben pertenecer la actividades consultadas.
	 * 			Tomado en cuenta únicamente si el parámetro idCentro es null o <=0
	 * 			null ó <=0 indica 'cualquier provincia'
	 * 
	 * @param idLocalidad
	 * 			(opcional)
	 * 			Identificador de la localidad a la que deben pertenecer la actividades consultadas.
	 * 			Tomado en cuenta únicamente si el parámetro idCentro es null o <=0
	 * 			null ó <=0 indica 'cualquier localidad'
	 * 
	 * @param idTipoActividad
	 * 			(obligatorio)
	 * 			Identificador del tipo de actividad al que deben pertenecer la actividades consultadas.
	 * 			(0)TODAS, (1)CURSO o (2)CHARLA
	 * 			
	 * 
	 * @param idTematica
	 * 			(opcional)
	 * 			Identificador de la temática a la que deben pertenecer las actividades consultadas.
	 * 			null ó <0 indica 'todas las temáticas'
	 * 
	 * @param idCentro
	 * 			(opcional)
	 * 			Identificador del Centro del que se quieren obtener las actividades.
	 * 			null ó <0 indica 'cualquier centro' 
	 * 
	 * @param patronActividad
	 * 			(opcional)
	 * 			Cadena de texto patrón para la identificación de actividades por el nombre. Modo like.
	 * 			null, "" o "%" indica cualquier nombre
	 * 
	 * @param plazasLibres
	 * 		  (opcional)
	 * 		  Comprueba si se tienen que devolver actividades que no tengan plazas libres.
	 * 		  Se toma por defecto false (devuelve todas las actividades), si se indica true, devuelve solo
	 * 		  las actividades que tengan plazas libres. 
	 * 
	 * @param enPlazo
	 * 		  (opcional)
	 * 		  Comprueba si se tienen que devolver actividades que no estén dentro del plazo de inscripción.
	 * 		  Se toma por defecto false (devuelve todas las actividades), si se indica true, devuelve solo
	 * 		  las actividades que estén en plazo de inscripción	
	 * 
	 * @param fechaInicio
	 * 		  (opcional)  
	 * 		  Fecha de inicio del rango de fechas. Si fechaInicio es "" o null, el campo es ignorado. 
	 * 		  Si se especifica una fecha de inicio, se mostrará el historico de actividades.
	 * 
	 * 
	 * @param fechaFin
	 * 		  (opcional)
	 * 		  Fecha fin del rando de fechas. Si fechaFin es "" o null, el campo es ignorado.
	 * 	 	  Si se especifica una fecha de fin, se mostrará el histórico de actividades.
	 * 
	 * 
	 * @param languageISO
	 * 			(obligatorio)
	 * 			Identificador ISO del idioma en el que se desean los resultados ('gl_ES' o 'es_ES').
	 * 
	 * @param fieldOrder
	 * 			(opcional)
	 * 			Nombre del campo del objeto Provincia por el que se desea ordenar.
	 * 			"" o null indica que no se desea ordenar.
	 * 
	 * @param orderASC
	 * 			(opcional)
	 * 			Indica en que sentido se desea ordenar el resultado. Si fieldOrder es "" o null este parámetro es ignorado.
	 *			Si no se especifica, se toma por defecto orden ascendente.
	 * 			true indica ascendente y false descendente. 
	 * 
	 * @param recordsByPage
	 * 			(opcional)
	 *          Número de registros a devolver.  
	 *          <= 0 indica devolver todos los registros.
	 *           
	 * @param page
	 * 			(opcional)
	 *          Número de página a devolver. Sólo si el parámetro recordsByPage es > 0.
	 *          En el caso de especificar un número <= 0 se ignora el parámetro recordsByPage y se devuelven todos los registros.
	 *          1 es la primera página.
	 * 
	 * @return listado con todos los datos de cada una de las actividades existentes en la localidad y provincia indicadas,
	 * 		   filtradas por el tipo y la temática especificdas y que su nombre cumpla el patrón recibido.
	 * 
	 */
	@WebMethod
	@Transactional
	public WSProximaActividad ProximasActividadesCeMIT
							  (final Long idProvincia, final Long idLocalidad, final Long idTipoActividad, final Long idTematica,
							   final Long idCentro, final String patronActividad, final Boolean plazasLibres, final Boolean enPlazo, 
							   final String fechaInicio, final String fechaFin, final String languageISO, final String fieldOrder, 
							   final Boolean orderASC, final Integer recordsByPage, final Integer page)  {
		
		long startTime = System.currentTimeMillis();
		
		WSProximaActividad wsProxActividad = new WSProximaActividad();
		List<WSActividad> wsActividades = new ArrayList<WSActividad>();
		
		//final Locale language = new Locale(languageISO.split("_")[0], languageISO.split("_")[1]);
		
		if (((idTipoActividad == TipoActividad.TODOS.getIdTipoActividad()) ||
			 (idTipoActividad == TipoActividad.CURSO.getIdTipoActividad()) ||
			 (idTipoActividad == TipoActividad.CHARLA.getIdTipoActividad())) &&
			  !languageISO.isEmpty()) {
				
			//SELECCION, TABLAS Y FILTRADOS FIJOS
			if (idTipoActividad == TipoActividad.TODOS.getIdTipoActividad() || 
				idTipoActividad == TipoActividad.CURSO.getIdTipoActividad()) {
				
				wsActividades.addAll(crearWSProximasActividades(idProvincia, idLocalidad, 1L, idTematica, 
					idCentro, patronActividad, plazasLibres, enPlazo, fechaInicio, 
					fechaFin, languageISO, fieldOrder, orderASC));
			}
			
			if (idTipoActividad == TipoActividad.TODOS.getIdTipoActividad() || 
				idTipoActividad == TipoActividad.CHARLA.getIdTipoActividad()) {
				
				wsActividades.addAll(crearWSProximasActividades(idProvincia, idLocalidad, 2L, idTematica, 
						idCentro, patronActividad, plazasLibres, enPlazo, fechaInicio, 
						fechaFin, languageISO, fieldOrder, orderASC));
				
			}
			
			//ORDENACIÓN
			//Si los campos fechas están vacios, se ordenará ascendentemente en todos los casos.
			if( (fechaInicio == null || fechaInicio.isEmpty()) && (fechaFin == null || fechaFin.isEmpty())) {
				Collections.sort(wsActividades, new Comparator<WSActividad>() {
					@Override
					public int compare(WSActividad o1, WSActividad o2) {
						return compararASC(o1, o2, fieldOrder);
					}
				});
			} else { //Si alguno de los campos fechas tienen valor, se ordenará descendentemente
				Collections.sort(wsActividades, new Comparator<WSActividad>() {
					@Override
					public int compare(WSActividad o1, WSActividad o2) {
						return compararDESC(o1, o2, fieldOrder);
					}
				});
			}
			
			int resultNum = wsActividades.size();
			
			if ((recordsByPage != null) && (recordsByPage > 0) && (page != null) && (page > 0)) {
				int start = (page - 1) * recordsByPage;
				int end = Math.min(((page - 1) * recordsByPage) + recordsByPage, wsActividades.size());
				
				if(end > wsActividades.size())
					end = wsActividades.size();
				
				wsActividades = wsActividades.subList(start, end);
			}
			
			wsProxActividad.numResultados = resultNum;
			wsProxActividad.actividad = wsActividades;
			
		}
		
		long elapsedTime = System.currentTimeMillis() - startTime;
		logger.log(Level.INFO, "Webservice ProximasActividadesCeMIT request - " + elapsedTime + "ms");
		
		return wsProxActividad;
	}
	
	
	
	

	/**
	 * Método web que consulta los datos de una actividad CeMIT.
	 *
	 * @param languageISO
	 * 			(obligatorio)
	 * 			Identificador ISO del idioma en el que se desean los resultados ('gl_ES' o 'es_ES').
	 *
	 * @param idActividad
	 * 			(obligatorio)
	 * 			Identificador de la Actividad CeMIT.
	 * 
	 * @return Actividad CeMIT asociada al identificador indicado.
	 */
	@WebMethod
	@Transactional
	public WSActividad ActividadCeMIT(String languageISO, Long idActividad) {
		
		long startTime = System.currentTimeMillis();

		WSActividad wsActividad = null;
		
		if ((languageISO != null) && !languageISO.isEmpty() && (idActividad != null)) {
			Locale language = new Locale(languageISO.split("_")[0], languageISO.split("_")[1]);
			
			String strqry = null;
			Query command = null;

			Boolean found = false;
			
			try {
				strqry = "SELECT e " +
						 "FROM " + CursoLanguage.class.getName() + " e " +
						 "WHERE (e.desactivado = 0) AND" +
						 "      (e.curso.id = :idActividad) AND" +
						 "	    (e.language.ISO = :languageISO)";
				
				command = service.getEntityManager().createQuery(strqry);
							
				command.setParameter("idActividad", idActividad);
				command.setParameter("languageISO", languageISO);
					
				CursoLanguage cursoLanguage = (CursoLanguage)command.getSingleResult();
				
				wsActividad = crearWSActividad(cursoLanguage, language);
				
				found = true;
			}
			catch (NoResultException e) {
				logger.log(Level.WARNING, "ActividadCeMIT: La actividad CeMIT "+ idActividad + " buscada no es un curso se buscará en charlas: " +
						 e.getLocalizedMessage());
			}
			catch (Exception e) {
				logger.log(Level.SEVERE, "ActividadCeMIT: Error al buscar la actividad CeMIT en cursos: " +
										 e.getLocalizedMessage());
			
				e.printStackTrace();
			}
			
			try {
				if (!found) {
					strqry = "SELECT e " +
							 "FROM " + CharlaLanguage.class.getName() + " e " +
							 "WHERE (e.desactivado = 0) AND" + 
							 "      (e.charla.id = :idActividad) AND" +
							 "	    (e.language.ISO = :languageISO)";
	
					command = service.getEntityManager().createQuery(strqry);
					
					command.setParameter("idActividad", idActividad);
					command.setParameter("languageISO", languageISO);
	
					CharlaLanguage charlaLanguage = (CharlaLanguage)command.getSingleResult();
	
					wsActividad = crearWSActividad(charlaLanguage, language);
					
					found = true;
				}
			}
			catch (NoResultException e) {
				logger.log(Level.WARNING, "ActividadCeMIT: La actividad CeMIT "+ idActividad + " buscada tampoco es una charla. Actividad no encontrada: " +
						   e.getLocalizedMessage());
			}
			catch (Exception e) {
				logger.log(Level.SEVERE, "ActividadCeMIT: Error al buscar la actividad CeMIT en charlas: " +
						 e.getLocalizedMessage());

				e.printStackTrace();
			}
		}

		long elapsedTime = System.currentTimeMillis() - startTime;
		logger.log(Level.INFO, "Webservice ActividadCeMIT request - " + elapsedTime + "ms");

		return wsActividad;
	}

	/**
	 * Método web que consulta los datos de varias actividades CeMIT (las destacadas).
	 *
	 * @param languageISO
	 * 			(obligatorio)
	 * 			Identificador ISO del idioma en el que se desean los resultados ('gl_ES' o 'es_ES').
	 * 
	 * @param recordsByPage
	 * 			(opcional)
	 *          Número de registros a devolver.  
	 *          <= 0 indica devolver todos los registros.
	 *           
	 * @param page
	 * 			(opcional)
	 *          Número de página a devolver. Sólo si el parámetro recordsByPage es > 0.
	 *          En el caso de especificar un número <= 0 se ignora el parámetro recordsByPage y se devuelven todos los registros.
	 *          1 es la primera página. 
	 *
	 * @param idActividades
	 * 			(obligatorio)
	 * 			Identificadores de las actividades destacadas CeMIT.
	 * 
	 * @return Actividades destacadas CeMIT asociada a los identificadores indicados.
	 */
	@WebMethod
	@Transactional
	public List<WSActividad> ActividadesDestacadasCeMIT(String languageISO, Integer recordsByPage, Integer page, Long... idActividades) {
		
		long startTime = System.currentTimeMillis();

		List<WSActividad> wsActividades = new ArrayList<WSActividad>();
		
		if ((languageISO   != null) && !languageISO.isEmpty() &&
			(idActividades != null) && (idActividades.length > 0)) {
			
			Locale language = new Locale(languageISO.split("_")[0], languageISO.split("_")[1]);
			
			String strqry = null;
			String paramsqry = "";
			Query command = null;
			
			Integer countId = 1;
			for (@SuppressWarnings("unused") Long idActividad : idActividades) {
				if (countId > 1)
					paramsqry += ", ";
				
				paramsqry += ":idActividad" + countId.toString();
				
				countId++;
			}

			try {
				strqry = "SELECT e " +
						 "FROM " + Actividad.class.getName() + " e " + 
						 "WHERE (e.desactivado = 0) AND" +
						 "		(e.id IN (" + paramsqry + ")) " +
						 "ORDER BY e.id";
				
				command = service.getEntityManager().createQuery(strqry);
				
				if ((recordsByPage != null) && (recordsByPage > 0) && (page != null) && (page > 0)) {
					command.setFirstResult((page - 1) * recordsByPage);
					command.setMaxResults(recordsByPage);
				}

				countId = 1;
				for (Long idActividad : idActividades) {
					command.setParameter("idActividad" + countId.toString(), idActividad);

					countId++;
				}

				@SuppressWarnings("unchecked")
				List<Actividad> actividades = (List<Actividad>)command.getResultList();
				
				Map<String, Object> Properties = new HashMap<String, Object>();
				Properties.put("language.ISO", language.toString());
				
				for (Actividad actividad : actividades) {
					if (actividad instanceof Curso) {
						Properties.remove("charla.id");
						Properties.put("curso.id", actividad.getId());		
						
						CursoLanguage csl = service.findOneWhere(CursoLanguage.class, Properties);
						wsActividades.add(crearWSActividad(csl, language));
					} else if (actividad instanceof Charla) {
						Properties.remove("curso.id");
						Properties.put("charla.id", actividad.getId());

						CharlaLanguage cll = service.findOneWhere(CharlaLanguage.class, Properties);
						wsActividades.add(crearWSActividad(cll, language));
					}
				}
			}
			catch (Exception e) {
				logger.log(Level.SEVERE, "ActividadesDestacadasCeMIT: Error al devolver las actividades destacadas CeMIT: " +
										 e.getLocalizedMessage());
			
				e.printStackTrace();
			}
		}

		long elapsedTime = System.currentTimeMillis() - startTime;
		logger.log(Level.INFO, "Webservice ActividadesDestacadasCeMIT request - " + elapsedTime + "ms");
		
		return wsActividades;
	}

	////////////////////////////////////////////////////////CLASES INTERNAS//////////////////////////////////////////////////

	/**
	 * WSProvincia class.
	 */
	public static class WSProvincia {
		public Long idProvincia;
		public String nombre;
	}

	private WSProvincia crearWSProvincia(Provincia p) {
		WSProvincia provincia = new WSProvincia();

		provincia.idProvincia = p.getId();
		provincia.nombre = p.getNombre();

		return provincia;
	}

	/**
	 * WSLocalidad class.
	 */
	public static class WSLocalidad {
		public Long idLocalidad;
		public String provincia;
		public String nombre;
	}

	private WSLocalidad crearWSLocalidad(Localidad l) {
		WSLocalidad localidad = new WSLocalidad();

		localidad.idLocalidad = l.getId();
		localidad.provincia = l.getProvincia().getNombre();
		localidad.nombre = l.getNombre();

		return localidad;
	}

	/**
	 * WSCentro class.
	 */
	public static class WSCentro {
		public Long idCentro;
		public String nombre;
		public String direccion;
		public String email;
		public String telefono;
		public String fax;
		public String contacto;
		public String localidad;
		public String provincia;
		public String horario;
	}
	
	private WSCentro crearWSCentro(Centro c, Locale language) {
		WSCentro centro = new WSCentro();
		
		ResourceBundle rb = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", language);

		centro.idCentro = c.getId();
		centro.nombre = c.getNombre();
		centro.direccion = c.getDireccion();
		centro.email = c.getEmail();
		centro.telefono = c.getTelefono();
		centro.fax = c.getFax();
		centro.contacto = c.getContacto();
		centro.localidad = c.getLocalidad().getNombre();
		centro.provincia = c.getProvincia().getNombre();
		
		SimpleDateFormat sdf = new SimpleDateFormat("HH:mm");
		String horario = "";
		
		//Lunes
		//Mañana
		horario += rb.getString("integracion.horario.daysWeek.Monday") + " -> " +
				   rb.getString("integracion.horario.Manyana") + ": ";
		if ((c.getMondayMorningStartHour() == null) || (c.getMondayMorningEndHour() == null))
			horario += rb.getString("integracion.horario.Cerrado") + " ";
		else
			horario += sdf.format(c.getMondayMorningStartHour()) + " - " + sdf.format(c.getMondayMorningEndHour()) + " ";
		//Tarde
		horario += rb.getString("integracion.horario.Tarde") + ": ";
		if ((c.getMondayAfternoonStartHour() == null) || (c.getMondayAfternoonEndHour() == null))
			horario += rb.getString("integracion.horario.Cerrado") + "\n";
		else
			horario += sdf.format(c.getMondayAfternoonStartHour()) + " - " + sdf.format(c.getMondayAfternoonEndHour()) + "\n";
			
		//Martes
		//Mañana
		horario += rb.getString("integracion.horario.daysWeek.Tuesday") + " -> " +
				   rb.getString("integracion.horario.Manyana") + ": ";
		if ((c.getTuesdayMorningStartHour() == null) || (c.getTuesdayMorningEndHour() == null))
			horario += rb.getString("integracion.horario.Cerrado") + " ";
		else
			horario += sdf.format(c.getTuesdayMorningStartHour()) + " - " + sdf.format(c.getTuesdayMorningEndHour()) + " ";
		//Tarde
		horario += rb.getString("integracion.horario.Tarde") + ": ";
		if ((c.getTuesdayAfternoonStartHour() == null) || (c.getTuesdayAfternoonEndHour() == null))
			horario += rb.getString("integracion.horario.Cerrado") + "\n";
		else
			horario += sdf.format(c.getTuesdayAfternoonStartHour()) + " - " + sdf.format(c.getTuesdayAfternoonEndHour()) + "\n";
						 
		//Miércoles
		//Mañana
		horario += rb.getString("integracion.horario.daysWeek.Wednesday") + " -> " +
				   rb.getString("integracion.horario.Manyana") + ": ";
		if ((c.getWednesdayMorningStartHour() == null) || (c.getWednesdayMorningEndHour() == null))
			horario += rb.getString("integracion.horario.Cerrado") + " ";
		else
			horario += sdf.format(c.getWednesdayMorningStartHour()) + " - " + sdf.format(c.getWednesdayMorningEndHour()) + " ";
		//Tarde
		horario += rb.getString("integracion.horario.Tarde") + ": ";
		if ((c.getWednesdayAfternoonStartHour() == null) || (c.getWednesdayAfternoonEndHour() == null))
			horario += rb.getString("integracion.horario.Cerrado") + "\n";
		else
			horario += sdf.format(c.getWednesdayAfternoonStartHour()) + " - " + sdf.format(c.getWednesdayAfternoonEndHour()) + "\n";
		
		//Jueves
		//Mañana
		horario += rb.getString("integracion.horario.daysWeek.Thursday") + " -> " +
				   rb.getString("integracion.horario.Manyana") + ": ";
		if ((c.getThursdayMorningStartHour() == null) || (c.getThursdayMorningEndHour() == null))
			horario += rb.getString("integracion.horario.Cerrado") + " ";
		else
			horario += sdf.format(c.getThursdayMorningStartHour()) + " - " + sdf.format(c.getThursdayMorningEndHour()) + " ";
		//Tarde
		horario += rb.getString("integracion.horario.Tarde") + ": ";
		if ((c.getThursdayAfternoonStartHour() == null) || (c.getThursdayAfternoonEndHour() == null))
			horario += rb.getString("integracion.horario.Cerrado") + "\n";
		else
			horario += sdf.format(c.getThursdayAfternoonStartHour()) + " - " + sdf.format(c.getThursdayAfternoonEndHour()) + "\n";
		
		//Viernes
		//Mañana
		horario += rb.getString("integracion.horario.daysWeek.Friday") + " -> " +
				   rb.getString("integracion.horario.Manyana") + ": ";
		if ((c.getFridayMorningStartHour() == null) || (c.getFridayMorningEndHour() == null))
			horario += rb.getString("integracion.horario.Cerrado") + " ";
		else
			horario += sdf.format(c.getFridayMorningStartHour()) + " - " + sdf.format(c.getFridayMorningEndHour()) + " ";
		//Tarde
		horario += rb.getString("integracion.horario.Tarde") + ": ";
		if ((c.getFridayAfternoonStartHour() == null) || (c.getFridayAfternoonEndHour() == null))
			horario += rb.getString("integracion.horario.Cerrado") + "\n";
		else
			horario += sdf.format(c.getFridayAfternoonStartHour()) + " - " + sdf.format(c.getFridayAfternoonEndHour()) + "\n";
		
		//Sábado
		//Mañana
		horario += rb.getString("integracion.horario.daysWeek.Saturday") + " -> " +
				   rb.getString("integracion.horario.Manyana") + ": ";
		if ((c.getSaturdayMorningStartHour() == null) || (c.getSaturdayMorningEndHour() == null))
			horario += rb.getString("integracion.horario.Cerrado") + " ";
		else
			horario += sdf.format(c.getSaturdayMorningStartHour()) + " - " + sdf.format(c.getSaturdayMorningEndHour()) + " ";
		//Tarde
		horario += rb.getString("integracion.horario.Tarde") + ": ";
		if ((c.getSaturdayAfternoonStartHour() == null) || (c.getSaturdayAfternoonEndHour() == null))
			horario += rb.getString("integracion.horario.Cerrado") + "\n";
		else
			horario += sdf.format(c.getSaturdayAfternoonStartHour()) + " - " + sdf.format(c.getSaturdayAfternoonEndHour()) + "\n";
		
		//Domingo
		//Mañana
		horario += rb.getString("integracion.horario.daysWeek.Sunday") + " -> " +
				   rb.getString("integracion.horario.Manyana") + ": ";
		if ((c.getSundayMorningStartHour() == null) || (c.getSundayMorningEndHour() == null))
			horario += rb.getString("integracion.horario.Cerrado") + " ";
		else
			horario += sdf.format(c.getSundayMorningStartHour()) + " - " + sdf.format(c.getSundayMorningEndHour()) + " ";
		//Tarde
		horario += rb.getString("integracion.horario.Tarde") + ": ";
		if ((c.getSundayAfternoonStartHour() == null) || (c.getSundayAfternoonEndHour() == null))
			horario += rb.getString("integracion.horario.Cerrado") + "\n";
		else
			horario += sdf.format(c.getSundayAfternoonStartHour()) + " - " + sdf.format(c.getSundayAfternoonEndHour()) + "\n";
			
		centro.horario = horario;	

		return centro;
	}
	
	
	/**
	 * TipoActividad enum.
	 */
	protected enum TipoActividad {
		TODOS (0L, "integracion.tipoActividad.Todos"),
		CURSO (1L, "integracion.tipoActividad.Curso"),
		CHARLA (2L, "integracion.tipoActividad.Charla");
		
		private final Long idTipoActividad;
		private final String nombre;
		
		TipoActividad(Long idTipoActividad, String nombre) {
	        this.idTipoActividad = idTipoActividad;
	        this.nombre = nombre;
	    }
		
		public Long getIdTipoActividad() { return idTipoActividad; }
	    public String getNombre() { return nombre; }
	}

	/**
	 * WSTipoActividad class.
	 */
	public static class WSTipoActividad {
		public Long idTipoActividad;
		public String nombre;
	}
	
	private WSTipoActividad crearWSTipoActividad(TipoActividad ta, Locale language) {
		WSTipoActividad tipoActividad = new WSTipoActividad();

		tipoActividad.idTipoActividad = ta.getIdTipoActividad();
		tipoActividad.nombre = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", language).getString(ta.getNombre());

		return tipoActividad;
	}

	/**
	 * WSTematica class.
	 */
	public static class WSTematica {
		public Long idTematica;
		public String nombre;		
		public Long idPadre;
		public String nombrePadre; //R 3.4.15
	}
	
	private WSTematica crearWSTematica(TematicaLanguage tl) {
		WSTematica tematica = new WSTematica();

		tematica.idTematica = tl.getTematica().getId();
		tematica.nombre = tl.getNombre();
		
		if (tl.getTematica().getPadre() != null){
			tematica.idPadre = tl.getTematica().getPadre().getId();
			
			//R 3.4.15
			Map<String, Object> Properties = new HashMap<String, Object>();
			Properties.put("tematica.id", tematica.idPadre);
			Properties.put("language", tl.getLanguage());
			TematicaLanguage tlPadre = service.findOneWhere(TematicaLanguage.class, Properties);
			tematica.nombrePadre = tlPadre.getNombre();
			
		}
		
		return tematica;
	}

	/**
	 * WSActividad class.
	 */
	public static class WSActividad {
		public Long idActividad;
		public Long idCentro;
		public String nombreCentro;
		public String tipo;
		public String tematica;
		public Long idTematica;			//R 3.4.15
		public Long idTematicaPadre;	//R 3.4.15
		public String nivel;
		public String nombre;
		public String descripcion;
		public Integer plazasTotales;
		public Integer plazasLibres;
		public Integer numeroInscritos;
		public String programa;
		public String observaciones;
		public String profesor;
		public String horario;
		public Date fechaInicio;
		public Date fechaFin;
		public Integer edadMinima;
		public Integer edadMaxima;
		public Float numeroHoras;
		//RF_15_47 - Modificar servicios web inscricións para Cemit
		public String fechaLimiteInscripcion;
		public String horaLimiteInscripcion;
		public boolean isEnPlazoInscripcion;
	}
	
	private WSActividad crearWSActividadCurso(Object[] cursodb, Locale language) {
		WSActividad actividad = new WSActividad();

		HttpServletRequest req = (HttpServletRequest)wsContext.getMessageContext().get(MessageContext.SERVLET_REQUEST);
		DateFormat df = new SimpleDateFormat("HH:mm");
		
		actividad.idActividad = (Long)cursodb[0];
		actividad.idCentro = (Long)cursodb[1];
		actividad.nombreCentro = (String)cursodb[2];
		actividad.tipo = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", language).getString(TipoActividad.CURSO.getNombre());
		actividad.tematica = (String)cursodb[3];
		actividad.idTematica = (Long)cursodb[23];	//R 3.4.15
		actividad.idTematicaPadre = (Long)cursodb[24];  //R 3.4.15		
		actividad.nivel = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", language).getString("nivel." + (Nivel)cursodb[4]);
		actividad.nombre = (String)cursodb[5];
		actividad.descripcion = (cursodb[6] != null) ? (String)cursodb[6] : "";
		actividad.plazasTotales = (Integer)cursodb[7];
		actividad.plazasLibres = (Integer)cursodb[8];
		
		if (cursodb[9] != null)
			actividad.programa =  "http://" + req.getServerName() + ":" + req.getServerPort() + req.getContextPath() +
								  "/spring/obtenerFileOnDisk?id=" + (Long)cursodb[9];
		else
			actividad.programa = "";
		
		actividad.observaciones = (cursodb[10] != null) ? (String)cursodb[10] : "";
		actividad.profesor = (cursodb[11] != null) ? (String)cursodb[11] : "";
		actividad.horario = df.format((Date)cursodb[12]) + " - " + df.format((Date)cursodb[13]);
		actividad.fechaInicio = (Date)cursodb[14];
		actividad.fechaFin = (Date)cursodb[15];
		actividad.edadMinima = (Integer)cursodb[16];
		actividad.edadMaxima = (Integer)cursodb[17];
		actividad.numeroHoras = (Float)cursodb[18];
		actividad.numeroInscritos = (Integer)cursodb[19];
		//RF_15_47 - Modificar servicios web inscricións para Cemit
		actividad.fechaLimiteInscripcion = ((Date)cursodb[20] != null) ? ((Date)cursodb[20]).toString() : "" ;
		actividad.horaLimiteInscripcion = ((Date)cursodb[21] != null) ? ((Date)cursodb[21]).toString() : "" ;
		Curso c = (Curso) cursodb[22];
		actividad.isEnPlazoInscripcion = c.getIsEnPlazoInscripcion();

		return actividad;
	}
	
	private WSActividad crearWSActividad(CursoLanguage cursol, Locale language) {
		WSActividad actividad = new WSActividad();

		HttpServletRequest req = (HttpServletRequest)wsContext.getMessageContext().get(MessageContext.SERVLET_REQUEST);
		
		Map<String, Object> Properties = new HashMap<String, Object>();
		
		Tematica oTematica = cursol.getCurso().getServicio().getTematica();
		Properties.put("tematica.id", oTematica.getId());
		Properties.put("language.ISO", language.toString());
		TematicaLanguage tl = service.findOneWhere(TematicaLanguage.class, Properties);
		
		Properties.remove("tematica.id");
		Properties.put("servicio.id", cursol.getCurso().getServicio().getId());		
		ServicioCursoLanguage scl = service.findOneWhere(ServicioCursoLanguage.class, Properties);
		
		actividad.idActividad = cursol.getCurso().getId();
		actividad.idCentro = cursol.getCurso().getCentro().getId();
		actividad.nombreCentro = cursol.getCurso().getCentro().getNombre();
		actividad.tipo = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", language).getString(TipoActividad.CURSO.getNombre());
		actividad.tematica = tl.getNombre();
		actividad.idTematica = oTematica.getId();
		actividad.idTematicaPadre = oTematica.getPadre().getId();
		actividad.nivel = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", language).getString("nivel." + scl.getServicioCurso().getNivel());
		actividad.nombre = cursol.getApodo();
		actividad.descripcion = (scl.getDescripcion() != null) ? scl.getDescripcion() : "";
		actividad.plazasTotales = cursol.getCurso().getNumeroPlazas();
		actividad.plazasLibres = cursol.getCurso().getNumeroPlazas() - cursol.getCurso().getNumeroConfirmados();
		if (scl.getFicheroGuion() != null)
			actividad.programa =  "http://" + req.getServerName() + ":" + req.getServerPort() + req.getContextPath() + "/spring/obtenerFileOnDisk?id=" + scl.getFicheroGuion().getId();
		else
			actividad.programa = "";
		actividad.observaciones = (cursol.getCurso().getObservaciones() != null) ? cursol.getCurso().getObservaciones() : "";
		actividad.profesor = (cursol.getCurso().getProfesor() != null) ? cursol.getCurso().getProfesor().getNombreCompleto() : "";
		actividad.horario = cursol.getCurso().getHorario();
		actividad.fechaInicio = cursol.getCurso().getFechaInicio();
		actividad.fechaFin = cursol.getCurso().getFechaFin();
		actividad.edadMinima = cursol.getCurso().getServicio().getEdadMinima();
		actividad.edadMaxima = cursol.getCurso().getServicio().getEdadMaxima();
		actividad.numeroHoras = cursol.getCurso().getNumeroHoras();
		actividad.numeroInscritos = cursol.getCurso().getNumeroInscritos();
		//RF_15_47 - Modificar servicios web inscricións para Cemit
		actividad.fechaLimiteInscripcion = (cursol.getCurso().getFechaLimiteInscripcion() != null) ? cursol.getCurso().getFechaLimiteInscripcion().toString() : "";
		actividad.horaLimiteInscripcion = (cursol.getCurso().getHoraLimiteInscripcion() != null) ? cursol.getCurso().getHoraLimiteInscripcion().toString() : "";
		actividad.isEnPlazoInscripcion = cursol.getActividad().getIsEnPlazoInscripcion();
		
		return actividad;
	}
	
	private WSActividad crearWSActividadCharla(Object[] charladb, Locale language) {
		WSActividad actividad = new WSActividad();
		
		DateFormat df = new SimpleDateFormat("HH:mm");

		actividad.idActividad = (Long)charladb[0];
		actividad.idCentro = (Long)charladb[1];
		actividad.nombreCentro = (String)charladb[2];
		actividad.tipo = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", language).getString(TipoActividad.CHARLA.getNombre());
		actividad.tematica = (String)charladb[3];
		actividad.idTematica = (Long)charladb[21];	//R 3.4.15
		actividad.idTematicaPadre = (Long)charladb[22];  //R 3.4.15		
		actividad.nivel = "";
		actividad.nombre = (String)charladb[4];
		actividad.descripcion = (charladb[5] != null) ? (String)charladb[5] : ""; // RF_15_29 Campo observación nas charlas
		actividad.plazasTotales = (Integer)charladb[6];
		actividad.plazasLibres = (Integer)charladb[7];
		actividad.programa =  "";
		actividad.observaciones = (charladb[8] != null) ? (String)charladb[8] : "";
		actividad.profesor = (charladb[9] != null) ? (String)charladb[9] : "";
		actividad.horario = df.format((Date)charladb[10]) + " - " + df.format((Date)charladb[11]);
		actividad.fechaInicio = (Date)charladb[12];
		actividad.fechaFin = (Date)charladb[13];
		actividad.edadMinima = (Integer)charladb[14];
		actividad.edadMaxima = (Integer)charladb[15];
		actividad.numeroHoras = (Float)charladb[16];
		actividad.numeroInscritos = (Integer)charladb[17];
		//RF_15_47 - Modificar servicios web inscricións para Cemit
		actividad.fechaLimiteInscripcion = ((Date)charladb[18] != null) ? ((Date)charladb[18]).toString() : "";
		actividad.horaLimiteInscripcion = ((Date)charladb[19] != null) ? ((Date)charladb[19]).toString() : "";
		Charla c = (Charla) charladb[20];
		actividad.isEnPlazoInscripcion = c.getIsEnPlazoInscripcion();
		
		return actividad;
	}
	
	private WSActividad crearWSActividad(CharlaLanguage charlal, Locale language) {
		WSActividad actividad = new WSActividad();

		Map<String, Object> Properties = new HashMap<String, Object>();

		Tematica oTematica = charlal.getCharla().getTematica();
		Properties.put("tematica.id", oTematica.getId());
		Properties.put("language.ISO", language.toString());
		TematicaLanguage tl = service.findOneWhere(TematicaLanguage.class, Properties);

		actividad.idActividad = charlal.getCharla().getId();
		actividad.idCentro = charlal.getCharla().getCentro().getId();
		actividad.nombreCentro = charlal.getCharla().getCentro().getNombre();
		actividad.tipo = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", language).getString(TipoActividad.CHARLA.getNombre());
		actividad.tematica = tl.getNombre();
		actividad.idTematica = oTematica.getId();
		actividad.idTematicaPadre = oTematica.getPadre().getId();
		actividad.nivel = "";
		actividad.nombre = charlal.getNombre();
		actividad.descripcion = (charlal.getDescripcion() != null) ? charlal.getDescripcion() : ""; // RF_15_29 Campo observación nas charlas
		actividad.plazasTotales = charlal.getCharla().getNumeroPlazas();
		actividad.plazasLibres = charlal.getCharla().getNumeroPlazas() - charlal.getCharla().getNumeroConfirmados();
		actividad.programa =  "";
		actividad.observaciones = (charlal.getCharla().getObservaciones() != null) ? charlal.getCharla().getObservaciones() : "";
		actividad.profesor = (charlal.getCharla().getProfesor() != null) ? charlal.getCharla().getProfesor().getNombreCompleto(): "";
		actividad.horario = charlal.getCharla().getHorario();
		actividad.fechaInicio = charlal.getCharla().getFechaInicio();
		actividad.fechaFin = charlal.getCharla().getFechaInicio();
		actividad.edadMinima = charlal.getCharla().getEdadMinima();
		actividad.edadMaxima = charlal.getCharla().getEdadMaxima();
		actividad.numeroHoras = (float)((charlal.getCharla().getHoraFin().getTime() - charlal.getCharla().getHoraInicio().getTime()) / 1000.0 / 60.0 / 60.0);
		actividad.numeroInscritos = charlal.getCharla().getNumeroInscritos();
		//RF_15_47 - Modificar servicios web inscricións para Cemit
		actividad.fechaLimiteInscripcion = (charlal.getCharla().getFechaLimiteInscripcion() != null) ? charlal.getCharla().getFechaLimiteInscripcion().toString() : "";
		actividad.horaLimiteInscripcion = (charlal.getCharla().getHoraLimiteInscripcion() != null) ? charlal.getCharla().getHoraLimiteInscripcion().toString() : "";
		actividad.isEnPlazoInscripcion = charlal.getActividad().getIsEnPlazoInscripcion();
				
		return actividad;
	}
	
	/**
	 * WSActividad class.
	 */
	public static class WSProximaActividad {
		public Integer numResultados;
		public List<WSActividad> actividad;
	}
	
	private List<WSActividad> crearWSProximasActividades(final Long idProvincia, final Long idLocalidad, final Long idTipoActividad, final Long idTematica,
			   final Long idCentro, final String patronActividad, final Boolean plazasLibres, final Boolean enPlazo, 
			   final String fechaInicio, final String fechaFin, final String languageISO, final String fieldOrder, 
			   Boolean orderASC) {	
		
		Locale language = new Locale(languageISO.split("_")[0], languageISO.split("_")[1]);
		List<WSActividad> wsActividades = new ArrayList<WSActividad>();
		
		/*String patterFieldName = "";
		
		if (idTipoActividad == TipoActividad.CURSO.getIdTipoActividad())
			patterFieldName = "apodo";
		else if (idTipoActividad == TipoActividad.CHARLA.getIdTipoActividad())
			patterFieldName = "nombre";*/

		if (((idTipoActividad == TipoActividad.CURSO.getIdTipoActividad()) ||
			 (idTipoActividad == TipoActividad.CHARLA.getIdTipoActividad())) &&
			!languageISO.isEmpty()) {
			
			String strqry = "";
			
			//SELECCION, TABLAS Y FILTRADOS FIJOS
			if (idTipoActividad == TipoActividad.CURSO.getIdTipoActividad())
				strqry = "SELECT c.id AS idActividad, " +
						 	"c.centro.id AS idCentro, " +
							"c.centro.nombre AS nombreCentro, " +
							"tl.nombre AS tematica, " +
							"scl.servicioCurso.nivel AS nivel, " +
							"cl.apodo AS nombre, " +
							"scl.descripcion AS descripcion, " +
							"c.numeroPlazas AS plazasTotales, " +
							"(SELECT CASE WHEN (c.numeroPlazas - CAST(count(m) AS integer)) < 0 THEN 0 ELSE (c.numeroPlazas - CAST(count(m) AS integer)) END FROM c.matriculas m WHERE (m.admitido = TRUE AND m.enListaEspera = FALSE) AND (m.desactivado = 0)) as plazasLibres, " +
							"scl.ficheroGuion.id AS idFicheroGuion, " +
							"c.observaciones AS observaciones, " +
							"p.nombre || ' ' || p.apellido1 || ' ' || p.apellido2 AS profesor, " +
							"c.horaInicio AS horaInicio, " +
							"c.horaFin AS horaFin, " +
							"c.fechaInicio AS fechaInicio, " +
							"c.fechaFin AS fechaFin, " +
							"scl.servicio.edadMinima AS edadMinima, " +
							"scl.servicio.edadMaxima AS edadMaxima, " +
							"c.numeroHoras AS numeroHoras, " +
							"(SELECT CAST(count(m) AS integer) FROM c.matriculas m WHERE (m.desactivado = 0 AND m.cancelada = 0)) as numeroInscritos, " +
							"c.fechaLimiteInscripcion AS fechaLimiteInscripcion, " + // new line RF_15_47
							"c.horaLimiteInscripcion AS horaLimiteInscripcion, " + //new line RF_15_47
							"c, " +	//new line RF_15_47	
							"c.servicio.tematica.id AS idTematica, " + //R 3.4.15
							"c.servicio.tematica.padre.id AS idTematicaPadre " + //R 3.4.15
						 "FROM es.csa.fdt.cyldigital.servicios.entity.Curso c INNER JOIN " +
							"c.textosCurso cl INNER JOIN " +
							"cl.language lc WITH (lc.ISO LIKE :languageISO) INNER JOIN " +
							"c.servicio.tematica.textos tl INNER JOIN " +
							"tl.language lt WITH (lt.ISO LIKE :languageISO) LEFT JOIN " +
							"c.profesor p, " +
							"es.csa.fdt.cyldigital.language.entity.ServicioCursoLanguage scl INNER JOIN " +
							"scl.language lsc WITH (lsc.ISO = :languageISO) " +
						 "WHERE (c.desactivado = 0) AND (c.servicio.id = scl.servicio.id) ";
			else
				strqry = "SELECT c.id AS idActividad, " +
						 	"c.centro.id AS idCentro, " +
							"c.centro.nombre AS nombreCentro, " +
							"tl.nombre AS tematica, " +
							"cl.nombre AS nombre, " +
							"cl.descripcion AS descripcion, " + // RF_15_29 Campo observación nas charlas
							"c.numeroPlazas AS plazasTotales, " +								
							"(SELECT CASE WHEN (c.numeroPlazas - CAST(count(m) AS integer)) < 0 THEN 0 ELSE (c.numeroPlazas - CAST(count(m) AS integer)) END FROM c.matriculas m WHERE (m.admitido = TRUE AND m.enListaEspera = FALSE) AND (m.desactivado = 0)) as plazasLibres, " +
							"c.observaciones AS observaciones, " +
							"p.nombre || ' ' || p.apellido1 || ' ' || p.apellido2 AS profesor, " +
							"c.horaInicio AS horaInicio, " +
							"c.horaFin AS horaFin, " +
							"c.fechaInicio AS fechaInicio, " +
							"c.fechaInicio AS fechaFin, " +
							"c.edadMinima AS edadMinima, " +
							"c.edadMaxima AS edadMaxima, " +
							"time_to_sec(c.horaFin - c.horaInicio) / 3600.0F AS numeroHoras, " +
							"(SELECT CAST(count(m) AS integer) FROM c.matriculas m WHERE (m.desactivado = 0 AND m.cancelada = 0)) as numeroInscritos, " +
							"c.fechaLimiteInscripcion AS fechaLimiteInscripcion, " + // new line RF_15_47
							"c.horaLimiteInscripcion AS horaLimiteInscripcion, " + //new line RF_15_47
							"c, " +	//new line RF_15_47	
							"c.tematica.id AS idTematica, " + // R 3.4.15
							"c.tematica.padre.id AS idTematicaPadre " + // R 3.4.15
						 "FROM es.csa.fdt.cyldigital.servicios.entity.Charla c INNER JOIN " +
							"c.textosCharla cl INNER JOIN " +
							"cl.language lc WITH (lc.ISO LIKE :languageISO) INNER JOIN " +
							"c.tematica.textos tl INNER JOIN " +
							"tl.language lt WITH (lt.ISO LIKE :languageISO) LEFT JOIN " +
							"c.profesor p " +
						 "WHERE (c.desactivado = 0)" ;
			
			//FILTRADO
			if( (fechaInicio == null || fechaInicio.isEmpty()) && (fechaFin == null || fechaFin.isEmpty())) {
				strqry += " AND (c.cerrada = false) AND (c.fechaInicio >= NOW())";
			} else if( (fechaInicio != null && !fechaInicio.isEmpty()) && (fechaFin == null || fechaFin.isEmpty())) {
				strqry += " AND (c.fechaInicio >= :fechaInicio)";
			} else if( (fechaInicio == null || fechaInicio.isEmpty()) && (fechaFin != null && !fechaFin.isEmpty())) {
				strqry += " AND (c.fechaInicio <= :fechaFin)";
			} else {
				strqry += " AND (c.fechaInicio <= :fechaFin) AND (c.fechaFin >= :fechaInicio)";
			}
			
			if ((idCentro != null) && (idCentro > 0))
				strqry += " AND (c.centro.id = :idCentro)";
			else {
				if ((idProvincia != null) && (idProvincia > 0))
					strqry += " AND (c.centro.provincia.id = :idProvincia)";
				
				if ((idLocalidad != null) && (idLocalidad > 0))
					strqry += " AND (c.centro.localidad.id = :idLocalidad)";
			}
	
			if ((idTematica != null) && (idTematica > 0))
				strqry += " AND (tl.tematica.id = :idTematica)";

			//PR-XEA: Se comentan estas lineas ya que aumentaba el tiempo de la consulta. 
			//La comprobación de este campo se realiza más abajo.
			/*if ((patronActividad != null) && (!patronActividad.isEmpty()))*/		
				/*strqry += " AND (cl." + patterFieldName + " LIKE :patronActividad)";*/
				/*strqry += " AND (CONCAT(c.centro.nombre,'|',c.centro.provincia.nombre,'|'," + 
						  "cl." + patterFieldName + ") LIKE :patronActividad)";*/
				
				/*strqry += " AND ((c.centro.nombre LIKE :patronActividad) OR" + //new line RF_15_24
							" (c.centro.provincia.nombre LIKE :patronActividad) OR" + //new line RF_15_24
							" (c.centro.localidad.nombre LIKE :patronActividad) OR" + //new line RF_15_24
							" (cl." + patterFieldName + " LIKE :patronActividad))";		*/
			
			if(plazasLibres != null && plazasLibres)
				strqry += " AND (((SELECT CASE WHEN (c.numeroPlazas - CAST(count(m) AS integer)) < 0 THEN 0 ELSE (c.numeroPlazas - CAST(count(m) AS integer)) END FROM c.matriculas m WHERE (m.admitido = TRUE AND m.enListaEspera = FALSE))) > 0)";

			/* La validacion de fecha de publicacion es obligatoria */
			strqry += " AND ((c.fechaPublicacion IS NULL AND c.horaPublicacion IS NULL) OR (c.fechaPublicacion < curdate()) OR (c.fechaPublicacion = curdate() AND (c.horaPublicacion <= curtime() OR c.horaPublicacion IS NULL))) ";
			
			if ( (enPlazo != null && enPlazo) )
				strqry += "  AND ((c.fechaLimiteInscripcion IS NULL AND c.horaLimiteInscripcion IS NULL) OR " + 
							   "(c.fechaLimiteInscripcion > curdate()) OR " +
							   "(c.fechaLimiteInscripcion = curdate() AND c.horaLimiteInscripcion >= curtime())) AND " +
							   "((c.fechaInicio > curdate()) OR " + 
							   "(c.fechaInicio = curdate() AND c.horaInicio >= curtime()))";

			//AGRUPACION (por el JOIN con las matriculaciones que de no agruparlo por actividad provocaría que se recuperase cada
			//            actividad tantas veces como matriculas haya en la misma)
			strqry += " GROUP BY c";
			
			//FINAL
			try {
				Query command = service.getEntityManager().createQuery(strqry);
				
				//IDIOMA
				command.setParameter("languageISO",	languageISO);
				
				//PARAMETROS
				SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
				if( fechaInicio != null && !fechaInicio.isEmpty())
					command.setParameter("fechaInicio", sdf.parse(fechaInicio));
				
				if( fechaFin != null && !fechaFin.isEmpty())
					command.setParameter("fechaFin", sdf.parse(fechaFin));
				
				if ((idCentro != null) && (idCentro > 0))
					command.setParameter("idCentro", idCentro);
				else {
					if ((idProvincia != null) && (idProvincia > 0))
						command.setParameter("idProvincia", idProvincia);
					
					if ((idLocalidad != null) && (idLocalidad > 0))
						command.setParameter("idLocalidad", idLocalidad);
				}
				
				if ((idTematica != null) && (idTematica > 0))
					command.setParameter("idTematica",	idTematica);
	
				/*if ((patronActividad != null) && (!patronActividad.isEmpty()))
					command.setParameter("patronActividad", "%" + patronActividad + "%");*/

				//EJECUCION
				@SuppressWarnings("unchecked")
				List<Object[]> actividades = command.getResultList();
				
				if (idTipoActividad == TipoActividad.CURSO.getIdTipoActividad()) {
					//COMPOSICION CURSOS
					for (Object[] cursodb : actividades) {	
						boolean generate = false;
						WSActividad WSca = crearWSActividadCurso(cursodb, language);
						
						if ((patronActividad != null) && (!patronActividad.isEmpty())){	
							Centro centro = service.find(Centro.class, WSca.idCentro);
							
							if(centro.getNombre().toLowerCase().contains(patronActividad.toLowerCase())) generate = true;
							if(centro.getProvincia().getNombre().toLowerCase().contains(patronActividad.toLowerCase())) generate = true;
							if(centro.getLocalidad().getNombre().toLowerCase().contains(patronActividad.toLowerCase())) generate = true;	
							if(WSca.nombre.toLowerCase().contains(patronActividad.toLowerCase())) generate = true;
							
						} else generate = true; 
						
						if(generate)
							wsActividades.add(WSca);
					}
				} else if (idTipoActividad == TipoActividad.CHARLA.getIdTipoActividad()) {
					//COMPOSICION CHARLAS
					for (Object[] charladb : actividades) {
						boolean generate = false;
						WSActividad WSca = crearWSActividadCharla(charladb, language);
						
						if ((patronActividad != null) && (!patronActividad.isEmpty())){	
							Centro centro = service.find(Centro.class, WSca.idCentro);
							
							if(centro.getNombre().toLowerCase().contains(patronActividad.toLowerCase())) generate = true;
							if(centro.getProvincia().getNombre().toLowerCase().contains(patronActividad.toLowerCase())) generate = true;
							if(centro.getLocalidad().getNombre().toLowerCase().contains(patronActividad.toLowerCase())) generate = true;	
							if(WSca.nombre.toLowerCase().contains(patronActividad.toLowerCase())) generate = true;
							
						} else generate = true; 
						
						if(generate)
							wsActividades.add(WSca);
						
					}
				}
			} catch (Exception e) {
				logger.log(Level.SEVERE,
						   "ProximasActividadesCeMIT: Error al devolver la búsqueda de actividades CeMIT: " +
						   e.getLocalizedMessage());
			
				e.printStackTrace();
			}
		}
		
		return wsActividades;
	}
	
	private int compararASC(WSActividad o1, WSActividad o2, String fieldOrder) {
		int valor = 0;
		
		if(o1.fechaInicio.compareTo(o2.fechaInicio) == 0)
			valor =  o1.nombre.compareTo(o2.nombre);
		else
			valor =  o1.fechaInicio.compareTo(o2.fechaInicio);
		
		
		/*if ((fieldOrder == null) || (fieldOrder.isEmpty()))
			fieldOrder = "fechaInicio";			
			
		if (fieldOrder.compareTo("idActividad") == 0)
			valor = o1.idActividad.compareTo(o2.idActividad);
		else if (fieldOrder.compareTo("idCentro") == 0)
			valor =  o1.idCentro.compareTo(o2.idCentro);
		else if (fieldOrder.compareTo("nombreCentro") == 0)
			valor =  o1.nombreCentro.compareTo(o2.nombreCentro);
		else if (fieldOrder.compareTo("tematica") == 0)
			valor =  o1.tematica.compareTo(o2.tematica);
		else if ((fieldOrder.compareTo("nivel") == 0))
			valor =  o1.nivel.compareTo(o2.nivel);
		else if (fieldOrder.compareTo("nombre") == 0)
			valor =  o1.nombre.compareTo(o2.nombre);
		else if ((fieldOrder.compareTo("descripcion") == 0))
			valor =  o1.descripcion.compareTo(o2.descripcion);
		else if (fieldOrder.compareTo("plazasTotales") == 0)
			valor =  o1.plazasTotales.compareTo(o2.plazasTotales);
		else if (fieldOrder.compareTo("plazasLibres") == 0)
			valor =  o1.plazasLibres.compareTo(o2.plazasLibres);
		else if (fieldOrder.compareTo("numeroInscritos") == 0)
			valor =  o1.numeroInscritos.compareTo(o2.numeroInscritos);
		else if ((fieldOrder.compareTo("programa") == 0))
			valor =  o1.programa.compareTo(o2.programa);
		else if (fieldOrder.compareTo("observaciones") == 0)
			valor =  o1.observaciones.compareTo(o2.observaciones); 
		else if (fieldOrder.compareTo("profesor") == 0)
			valor =  o1.profesor.compareTo(o2.profesor); 
		else if (fieldOrder.compareTo("horario") == 0)
			valor =  o1.horario.compareTo(o2.horario); 
		else if (fieldOrder.compareTo("fechaInicio") == 0)
			if(o1.fechaInicio.compareTo(o2.fechaInicio) == 0)
				valor =  o1.nombre.compareTo(o2.nombre);
			else
				valor =  o1.fechaInicio.compareTo(o2.fechaInicio); 
		else if ((fieldOrder.compareTo("fechaFin") == 0))
			if(o1.fechaFin.compareTo(o2.fechaFin) == 0)
				valor =  o1.nombre.compareTo(o2.nombre);
			else
				valor =  o1.fechaFin.compareTo(o2.fechaFin); 
		else if (fieldOrder.compareTo("edadMinima") == 0)
			valor =  o1.edadMinima.compareTo(o2.edadMinima); 
		else if (fieldOrder.compareTo("edadMaxima") == 0)
			valor =  o1.edadMaxima.compareTo(o2.edadMaxima); 
		else if (fieldOrder.compareTo("numeroHoras") == 0)
			valor =  o1.numeroHoras.compareTo(o2.numeroHoras); 
		*/
		return valor;
	}
	
	private int compararDESC(WSActividad o1, WSActividad o2, String fieldOrder) {
		int valor = 0;
		
		if(o2.fechaInicio.compareTo(o1.fechaInicio) == 0)
			valor =  o1.nombre.compareTo(o2.nombre);
		else
			valor =  o2.fechaInicio.compareTo(o1.fechaInicio);
		
		/*if ((fieldOrder == null) || (fieldOrder.isEmpty()))
			fieldOrder = "fechaInicio";	
		
		if (fieldOrder.compareTo("idActividad") == 0)
			valor = o2.idActividad.compareTo(o1.idActividad);
		else if (fieldOrder.compareTo("idCentro") == 0)
			valor =  o2.idCentro.compareTo(o1.idCentro);
		else if (fieldOrder.compareTo("nombreCentro") == 0)
			valor =  o2.nombreCentro.compareTo(o1.nombreCentro);
		else if (fieldOrder.compareTo("tematica") == 0)
			valor =  o2.tematica.compareTo(o1.tematica);
		else if ((fieldOrder.compareTo("nivel") == 0))
			valor =  o2.nivel.compareTo(o1.nivel);
		else if (fieldOrder.compareTo("nombre") == 0)
			valor =  o2.nombre.compareTo(o1.nombre);
		else if ((fieldOrder.compareTo("descripcion") == 0))
			valor =  o2.descripcion.compareTo(o1.descripcion);
		else if (fieldOrder.compareTo("plazasTotales") == 0)
			valor =  o2.plazasTotales.compareTo(o1.plazasTotales);
		else if (fieldOrder.compareTo("plazasLibres") == 0)
			valor =  o2.plazasLibres.compareTo(o1.plazasLibres);
		else if (fieldOrder.compareTo("numeroInscritos") == 0)
			valor =  o2.numeroInscritos.compareTo(o1.numeroInscritos);
		else if ((fieldOrder.compareTo("programa") == 0))
			valor =  o2.programa.compareTo(o1.programa);
		else if (fieldOrder.compareTo("observaciones") == 0)
			valor =  o2.observaciones.compareTo(o1.observaciones); 
		else if (fieldOrder.compareTo("profesor") == 0)
			valor =  o2.profesor.compareTo(o1.profesor); 
		else if (fieldOrder.compareTo("horario") == 0)
			valor =  o2.horario.compareTo(o1.horario); 
		else if (fieldOrder.compareTo("fechaInicio") == 0)
			if(o2.fechaInicio.compareTo(o1.fechaInicio) == 0)
				valor =  o2.nombre.compareTo(o1.nombre);
			else
				valor =  o2.fechaInicio.compareTo(o1.fechaInicio); 
		else if ((fieldOrder.compareTo("fechaFin") == 0))
			if(o2.fechaFin.compareTo(o1.fechaFin) == 0)
				valor =  o2.nombre.compareTo(o1.nombre);
			else
				valor =  o2.fechaFin.compareTo(o1.fechaFin); 
		else if (fieldOrder.compareTo("edadMinima") == 0)
			valor =  o2.edadMinima.compareTo(o1.edadMinima); 
		else if (fieldOrder.compareTo("edadMaxima") == 0)
			valor =  o2.edadMaxima.compareTo(o1.edadMaxima); 
		else if (fieldOrder.compareTo("numeroHoras") == 0)
			valor =  o2.numeroHoras.compareTo(o1.numeroHoras); 
		*/
		
		return valor;
	}
}
