package es.csa.fdt.cyldigital.integracion;

/*
 * #%L
 * Servicios Integrados para aplicaciones remotas - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.crypto.Cipher;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.DESedeKeySpec;
import javax.crypto.spec.IvParameterSpec;
import javax.jws.WebMethod;
import javax.jws.WebService;
import javax.persistence.Query;

import org.apache.commons.codec.binary.Base64;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import es.csa.fdt.cyldigital.calendario.entity.ActividadCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.CalendarItem;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.recursos.service.RemoteNoticeService;
import es.csa.fdt.cyldigital.servicios.entity.Actividad;
import es.csa.fdt.cyldigital.servicios.entity.Charla;
import es.csa.fdt.cyldigital.servicios.entity.Curso;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCharla;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaTaller;
import es.csa.fdt.cyldigital.servicios.entity.Servicio;
import es.csa.fdt.cyldigital.servicios.entity.Servicio.Nivel;
import es.csa.fdt.cyldigital.servicios.entity.ServicioCharla;
import es.csa.fdt.cyldigital.servicios.entity.ServicioCurso;
import es.csa.fdt.cyldigital.servicios.entity.ServicioTaller;
import es.csa.fdt.cyldigital.servicios.entity.Taller;
import es.csa.fdt.cyldigital.usuarios.entity.Localidad;
import es.csa.fdt.cyldigital.usuarios.entity.Provincia;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioPre;

/**
 * Clase integracionCyLDigital.
 */
@Component("integracionCyLDigital")
@WebService
public class integracionCyLDigital {

  private static final Logger logger = Logger.getLogger(integracionCyLDigital.class.getName());

  private static final int MAX_NOTICE_BURST = 10;

  @Autowired
  private RemoteNoticeService service;

  /**
   * Devuelve servicios cursos.
   * 
   * @return the servicios cursos
   */
  @WebMethod
  @Transactional
  public List<WSServicioCurso> getServiciosCursos() {
    List<WSServicioCurso> cursos = new ArrayList<WSServicioCurso>();
    try {
      service.setMessagesEnabled(false);
      if (service != null) {
        List<ServicioCurso> cursosBd = service.findAll(ServicioCurso.class);
        for (ServicioCurso servicioCurso : cursosBd) {
          WSServicioCurso curso = crearWSServicioCurso(servicioCurso);
          cursos.add(curso);
        }
      }
    } catch (RuntimeException e) {
      logger.log(Level.SEVERE, "Error al devoler las los servicios del tipo curso: " + e.getLocalizedMessage());
      e.printStackTrace();
    }
    return cursos;
  }

  /**
   * Devuelve servicios.
   * 
   * @return the servicios
   */
  @WebMethod
  @Transactional
  public List<WSServicio> getServicios() {
    List<WSServicio> servicios = new ArrayList<WSServicio>();
    try {
      service.setMessagesEnabled(false);
      if (service != null) {
        List<ServicioCurso> cursosBd = service.findAll(ServicioCurso.class);
        for (ServicioCurso servicioCurso : cursosBd) {
          WSServicio curso = crearWSServicio(servicioCurso);
          servicios.add(curso);
        }
        List<ServicioTaller> talleres = service.findAll(ServicioTaller.class);
        for (ServicioTaller ServicioTaller : talleres) {
          WSServicio taller = crearWSServicio(ServicioTaller);
          servicios.add(taller);
        }
      }
    } catch (RuntimeException e) {
      logger.log(Level.SEVERE, "Error al devoler las los servicios del tipo curso: " + e.getLocalizedMessage());
      e.printStackTrace();
    }
    return servicios;
  }

  /**
   * Lista de todos los cursos.
   * 
   * @return the cursos
   */
  @WebMethod
  @Transactional
  public List<WSCurso> getCursos() {
    List<WSCurso> cursos = new ArrayList<WSCurso>();
    try {
      service.setMessagesEnabled(false);
      if (service != null) {
        List<Curso> cursosBd = service.findAll(Curso.class);
        for (Curso c : cursosBd) {
          if (!c.getDesactivado()) {
            WSCurso curso = crearWSCurso(c);
            cursos.add(curso);
          }
        }
      }
    } catch (RuntimeException e) {
      logger.log(Level.SEVERE, "Error al devoler las los servicios del tipo curso: " + e.getLocalizedMessage());
      e.printStackTrace();
    }
    return cursos;
  }

  /**
   * Lista de todos los cursos activos.
   * 
   * @return the cursos activos
   */
  @WebMethod
  @Transactional
  public List<WSCurso> getCursosActivos() {
    List<WSCurso> cursos = new ArrayList<WSCurso>();
    for (WSCurso wsCurso : getCursos())
      if (wsCurso.fechaFin != null && wsCurso.fechaFin.after(new Date())) cursos.add(wsCurso);
    return cursos;
  }

  /**
   * Lista de todos los cursos.
   * 
   * @return the actividades formacion
   */
  @WebMethod
  @Transactional
  public List<WSActividad> getActividadesFormacion() {
    List<WSActividad> actividades = new ArrayList<WSActividad>();
    try {
      service.setMessagesEnabled(false);
      if (service != null) {
        List<Curso> cursosBd = service.findAll(Curso.class);
        for (Curso c : cursosBd) {
          if (!c.getDesactivado()) {
            WSActividad curso = crearWSActividad(c);
            actividades.add(curso);
          }
        }
        List<Taller> talleresBd = service.findAll(Taller.class);
        for (Taller t : talleresBd) {
          if (!t.getDesactivado()) {
            WSActividad taller = crearWSActividad(t);
            actividades.add(taller);
          }
        }
        List<Charla> charlasBd = service.findAll(Charla.class);
        for (Charla c : charlasBd) {
          if (!c.getDesactivado()) {
            WSActividad charla = crearWSActividad(c);
            actividades.add(charla);
          }
        }
      }
    } catch (RuntimeException e) {
      logger.log(Level.SEVERE, "Error al devoler las los servicios del tipo curso: " + e.getLocalizedMessage());
      e.printStackTrace();
    }
    return actividades;
  }

  /**
   * Lista de todos los cursos activos.
   * 
   * @return the actividades formacion activas
   */
  @WebMethod
  @Transactional
  public List<WSActividad> getActividadesFormacionActivas() {
    List<WSActividad> actividades = new ArrayList<WSActividad>();
    for (WSActividad wsActividad : getActividadesFormacion())
      if (wsActividad.fechaFin != null && wsActividad.fechaFin.after(new Date())) actividades.add(wsActividad);
    return actividades;
  }

  /**
   * Devuelve los datos de un curso.
   * 
   * @param idCurso
   *           id curso
   * @return the curso
   */
  @WebMethod
  @Transactional
  public WSRespuestaGetCurso getCurso(String idCurso) {
    WSRespuestaGetCurso respuesta = new WSRespuestaGetCurso();
    try {
      if (idCurso != null && !idCurso.isEmpty()) {
        Long id = Long.parseLong(idCurso);
        Curso c = service.find(Curso.class, id);
        if (c != null) {
          respuesta.correcto = true;
          respuesta.curso = crearWSCurso(c);
        } else {
          logger.log(Level.SEVERE, "El curso pedido no existe.");
          respuesta.mensaje = "El curso solicitado no existe en nuestra base de datos.";
        }
      } else {
        logger.log(Level.SEVERE, "El curso pedido no existe.");
        respuesta.mensaje = "El curso solicitado no existe en nuestra base de datos.";
      }
    } catch (RuntimeException e) {
      logger.log(Level.SEVERE, "Error al devoler los datos de un curso: " + e.getLocalizedMessage());
      e.printStackTrace();
    }
    return respuesta;
  }

  /**
   * Lista de todos usuarios activos.
   * 
   * @return the usuarios
   */
  @WebMethod
  @Transactional
  public List<WSUsuario> getUsuarios() {
    List<WSUsuario> usuarios = new ArrayList<WSUsuario>();
    try {
      service.setMessagesEnabled(false);
      if (service != null) {
        List<UsuarioCyL> usuariosBd = service.findAll(UsuarioCyL.class);
        for (UsuarioCyL u : usuariosBd) {
          if (!u.getDesactivado()) {
            WSUsuario usuario = crearWSUsuario((Usuario)u);
            usuarios.add(usuario);
          }
        }
        List<UsuarioPre> usuariosBd2 = service.findAll(UsuarioPre.class);
        for (UsuarioPre u : usuariosBd2) {
          if (!u.getDesactivado()) {
            WSUsuario usuario = crearWSUsuario((Usuario)u);
            usuarios.add(usuario);
          }
        }
      }
    } catch (RuntimeException e) {
      logger.log(Level.SEVERE, "Error al devoler las los usuario: " + e.getLocalizedMessage());
      e.printStackTrace();
    }
    return usuarios;
  }

  /**
   * Retorna el usuario si encuentra un usuario con el mismo id y email.
   * 
   * @param id
   *           id
   * @param mail
   *           mail
   * @return the usuario from id and mail
   */
  @WebMethod
  @Transactional
  public WSRespuestaGetUsuario getUsuarioFromIdAndMail(String id, String mail) {
    WSRespuestaGetUsuario respuesta = new WSRespuestaGetUsuario();
    try {
      if (id != null && !id.isEmpty()) {
        Long idUsuario = Long.parseLong(id);
        Usuario u = service.find(Usuario.class, idUsuario);
        if (u != null) {
          if (u.isEmpleado()) {
            logger.log(Level.SEVERE,
                "Error el usuario buscado se corresponde con personal no con un usuario del centro.");
            respuesta.mensaje = "Error el usuario buscado se corresponde con personal no con un usuario del centro.";
          } else if (u.isUsuarioPre()) {
            logger.log(Level.SEVERE,
                "Error el usuario buscado se corresponde con un usuario preinscrito, y estos no tienen email.");
            respuesta.mensaje = "Error el usuario buscado se corresponde con un usuario preinscrito, y estos no tienen email.";
          } else {
            if (mail != null
                && (((UsuarioCyL) u).getEmail() != null && ((UsuarioCyL) u).getEmail().trim()
                    .equalsIgnoreCase(mail.trim()))) {
              respuesta.usuario = crearWSUsuario(u);
              respuesta.correcto = true;
            } else {
              logger.log(Level.SEVERE, "El usuario no se corresponde con su email.");
              respuesta.mensaje = "El usuario no se corresponde con su email.";
            }
          }
        } else {
          logger.log(Level.SEVERE, "El usuario pedido no existe.");
          respuesta.mensaje = "El código de la tarjeta no existe.";
        }
      }
    } catch (NumberFormatException e) {
      logger.log(Level.SEVERE, "Error al buscar usuario con id y mail (id incorrecto): " + e.getLocalizedMessage());
      respuesta.mensaje = "Error al buscar usuario con id y mail (id no parseable)";
      e.printStackTrace();
    }
    return respuesta;
  }

  /**
   * Retorna el usuario si encuentra un usuario con el mismo id y email.
   * 
   * @param id
   *           id
   * @param nif
   *           nif
   * @return the usuario from id and nif
   */
  @WebMethod
  @Transactional
  public WSRespuestaGetUsuario getUsuarioFromIdAndNif(String id, String nif) {
    WSRespuestaGetUsuario respuesta = new WSRespuestaGetUsuario();
    try {
      if (id != null && !id.isEmpty()) {
        Long idUsuario = Long.parseLong(id);
        Usuario u = service.find(Usuario.class, idUsuario);
        if (u != null) {
          if (u.isEmpleado()) {
            logger.log(Level.SEVERE,
                "Error el usuario buscado se corresponde con personal no con un usuario del centro.");
            respuesta.mensaje = "Error el usuario buscado se corresponde con personal no con un usuario del centro.";
          } else if (u.isUsuarioPre()) {
            logger.log(Level.SEVERE,
                "Error el usuario buscado se corresponde con un usuario preinscrito, y estos no tienen email.");
            respuesta.mensaje = "Error el usuario buscado se corresponde con un usuario preinscrito, y estos no tienen email.";
          } else {
            if (nif != null
                && ((((UsuarioCyL) u).getNif() != null && ((UsuarioCyL) u).getNif().trim().equalsIgnoreCase(nif.trim()))
                    || (((UsuarioCyL) u).getNifAutorizado() != null && ((UsuarioCyL) u).getNifAutorizado().trim()
                        .equalsIgnoreCase(nif.trim())) || (((UsuarioCyL) u).getPasaporte() != null && ((UsuarioCyL) u)
                    .getPasaporte().trim().equalsIgnoreCase(nif.trim())))) {
              respuesta.usuario = crearWSUsuario(u);
              respuesta.correcto = true;
            } else {
              logger.log(Level.SEVERE,
                  "El usuario no se corresponde con su nif ni con el nif de la persona autorizada.");
              respuesta.mensaje = "El usuario no se corresponde con su nif ni con el nif de la persona autorizada.";
            }
          }
        } else {
          logger.log(Level.SEVERE, "El usuario pedido no existe.");
          respuesta.mensaje = "El código de la tarjeta no existe.";
        }
      }
    } catch (NumberFormatException e) {
      logger.log(Level.SEVERE, "Error al buscar usuario con id y nif (id incorrecto): " + e.getLocalizedMessage());
      respuesta.mensaje = "Error al buscar usuario con id y nif (id no parseable)";
      e.printStackTrace();
    }
    return respuesta;
  }

  /**
   * Checks if is usuario matriculado.
   * 
   * @param idUsuario
   *           id usuario
   * @param idServicio
   *           id servicio
   * @return true, if is usuario matriculado
   */
  @WebMethod
  @Transactional
  public boolean isUsuarioMatriculado(String idUsuario, String idServicio) {
    if (idServicio != null) {
      Servicio servicio = service.find(Servicio.class, Long.parseLong(idServicio));
      if (servicio != null) {
        for (Actividad actividad : servicio.getActividades()) {
          if (servicio instanceof ServicioCurso) {
            for (MatriculaCurso mc : ((Curso) actividad).getMatriculas()) {
              if (mc.getUsuario().getId() == Long.parseLong(idUsuario) && !mc.isEnListaEspera() && mc.isAdmitido()) { return true; }
            }
          } else if (servicio instanceof ServicioTaller) {
            for (MatriculaTaller mt : ((Taller) actividad).getMatriculas()) {
              if (mt.getUsuario().getId() == Long.parseLong(idUsuario) && !mt.isEnListaEspera() && mt.isAdmitido()) { return true; }
            }
          } else if (servicio instanceof ServicioCharla) {
            for (MatriculaCharla mc : ((Charla) actividad).getMatriculas()) {
              if (mc.getUsuario().getId() == Long.parseLong(idUsuario)) { return true; }
            }
          }
        }
      }
    }
    return false;
  }

  /**
   * Retorna el usuario si encuentra un usuario con el mismo id y email.
   * 
   * @param id
   *           id
   * @return the historial usuario
   */
  @WebMethod
  @Transactional
  public WSRespuestaHistorial getHistorialUsuario(String id) {
    WSRespuestaHistorial respuesta = new WSRespuestaHistorial();
    try {
      if (id != null && !id.isEmpty()) {
        Long idUsuario = Long.parseLong(id);
        Usuario u = service.find(Usuario.class, idUsuario);
        if (u != null) {
          if (u.isEmpleado()) {
            logger.log(Level.SEVERE,
                "Error el usuario buscado se corresponde con personal no con un usuario del centro.");
            respuesta.mensaje = "Error el usuario buscado se corresponde con personal no con un usuario del centro.";
          } else {
            for (MatriculaCurso m : u.getMatriculasCursos()) {
              respuesta.matriculasCurso.add(crearWSMatriculaCurso(m));
            }
            for (MatriculaTaller m : u.getMatriculasTalleres()) {
              respuesta.matriculasTaller.add(crearWSMatriculaTaller(m));
            }
            for (MatriculaCharla m : u.getMatriculasCharlas()) {
              respuesta.matriculasCharla.add(crearWSMatriculaCharla(m));
            }
            respuesta.correcto = true;
          }
        }
      }
    } catch (NumberFormatException e) {
      logger.log(Level.SEVERE, "Error al buscar usuario con id: " + e.getLocalizedMessage());
      respuesta.mensaje = "Error al buscar el historial del usuario (id no parseable)";
      e.printStackTrace();
    }
    return respuesta;
  }

  /**
   * Retorna el usuario si encuentra un usuario con el mismo id y email.
   * 
   * @param id
   *           id
   * @return the usuario from id
   */
  @WebMethod
  @Transactional
  public WSRespuestaGetUsuario getUsuarioFromId(String id) {
    WSRespuestaGetUsuario respuesta = new WSRespuestaGetUsuario();
    try {
      if (id != null && !id.isEmpty()) {
        Long idUsuario = Long.parseLong(id);
        Usuario u = service.find(Usuario.class, idUsuario);
        if (u != null) {
          if (u.isEmpleado()) {
            logger.log(Level.SEVERE,
                "Error el usuario buscado se corresponde con personal no con un usuario del centro.");
            respuesta.mensaje = "Error el usuario buscado se corresponde con personal no con un usuario del centro.";
          } else {
            respuesta.usuario = crearWSUsuario(u);
            respuesta.correcto = true;
          }
        } else {
          respuesta.mensaje = "Error usuario con id=" + id + " no existe";
        }
      }
    } catch (NumberFormatException e) {
      logger.log(Level.SEVERE, "Error al buscar usuario con id (id incorrecto): " + e.getLocalizedMessage());
      respuesta.mensaje = "Error al buscar usuario con id (id incorrecto)";
      e.printStackTrace();
    }
    return respuesta;
  }

  /**
   * Retorna el password de un usuario dado su username, encriptado con 3DES.
   * 
   * @param username
   *           username
   * @return the user password
   */
  @WebMethod
  @Transactional
  public WSRespuestaGetUserPassword getUserPassword(String username) {
    WSRespuestaGetUserPassword respuesta = new WSRespuestaGetUserPassword();
    try {
      if (username != null && !username.isEmpty()) {
        List<UsuarioCyL> users = service.findWhere(UsuarioCyL.class, "username", username);
        if (users != null && !users.isEmpty()) {
          if (users.size() == 1) {
            respuesta.correcto = true;
            String password = users.get(0).getPassword() == null ? "" : users.get(0).getPassword();
            respuesta.password = encryptData(password);
            respuesta.domain = users.get(0).getCentro().getDominio();
          } else {
            logger.log(Level.SEVERE, "Hay más de un usuario con ese nombre de usuario.");
            respuesta.mensaje = "Hay más de un usuario con ese nombre de usuario.";
          }
        } else {
          logger.log(Level.SEVERE, "No se encontro al usuario.");
          respuesta.mensaje = "No existe el usuario en la bd";
        }
      }
    } catch (NumberFormatException e) {
      logger.log(Level.SEVERE, "Error al buscar usuario con id (id incorrecto): " + e.getLocalizedMessage());
      respuesta.mensaje = "Error al buscar usuario con id (id incorrecto)";
      e.printStackTrace();
    }
    return respuesta;
  }

  private static String encryptData(String data) {
    try {
      Cipher cipher = Cipher.getInstance("DESede/CBC/PKCS5Padding");
      String key = "2010@ISISS@DNIE@PASSWORD"; // 24 de longitud
      DESedeKeySpec keyspec = new DESedeKeySpec(key.getBytes());
      SecretKeyFactory keyfactory = SecretKeyFactory.getInstance("DESede");
      SecretKey secretKey = keyfactory.generateSecret(keyspec);

      cipher.init(Cipher.ENCRYPT_MODE, secretKey, new IvParameterSpec(new byte[8]));

      byte[] encodingResult = cipher.doFinal(data.getBytes());

      // Base 64
      String result = new String(Base64.encodeBase64(encodingResult));
      /*
       * Hexadecimal: String result = new String(Hex.encodeHex(encodingResult));
       */
      return result;
    } catch (Exception e) {
      System.out.println(e.toString());
      return null;
    }
  }

  /**
   * Pre registrar usuario.
   * 
   * @param nombre
   *           nombre
   * @param apellido1
   *           apellido1
   * @param apellido2
   *           apellido2
   * @param telefono
   *           telefono
   * @param nif
   *           nif
   * @param centro
   *           centro
   * @param direccion
   *           direccion
   * @param cp
   *           cp
   * @param provincia
   *           provincia
   * @param localidad
   *           localidad
   * @return the wS respuesta pre registro
   */
  @WebMethod
  @Transactional
  public WSRespuestaPreRegistro preRegistrarUsuario(String nombre, String apellido1, String apellido2, String telefono,
      String nif, Long centro, String direccion, String cp, String provincia, String localidad) {
    WSRespuestaPreRegistro respuesta = new WSRespuestaPreRegistro();
    UsuarioPre u = new UsuarioPre();
    try {
      if (nombre == null || nombre.trim().isEmpty()) {
        respuesta.mensaje = "Error: El nombre es obligatorio";
        return respuesta;
      }
      u.setNombre(nombre);

      if (apellido1 == null || apellido1.trim().isEmpty()) {
        respuesta.mensaje = "Error: El primer apellido es obligatorio";
        return respuesta;
      }
      u.setApellido1(apellido1);

      if (apellido2 == null || apellido2.trim().isEmpty()) {
        respuesta.mensaje = "Error: El segundo apellido es obligatorio";
        return respuesta;
      }
      u.setApellido2(apellido2);

      if (telefono == null || telefono.trim().isEmpty()) {
        respuesta.mensaje = "Error: El teléfono es obligatorio";
        return respuesta;
      } else {
        telefono = telefono.replaceAll("[^0-9]", "");
        if (!telefono.trim().isEmpty()) u.setTelefono(telefono.trim());
      }

      if (centro == null || centro > 0) {
        respuesta.mensaje = "Error: El centro es obligatorio";
        return respuesta;
      } else {
        Centro c = service.find(Centro.class, centro);
        if (c == null) {
          respuesta.mensaje = "Error: El centro no es un centro valido";
          return respuesta;
        } else u.setCentro(c);
      }

      if (direccion != null && !direccion.trim().isEmpty()) {
        u.setDireccion(direccion);
      }

      if (provincia != null && !provincia.trim().isEmpty()) {
        String consulta = "SELECT e FROM " + Provincia.class.getName() + " e " + "WHERE nombre like ?";
        Query query = service.getEntityManager().createQuery(consulta);
        query.setParameter(1, provincia.trim());
        List<Provincia> provincias = (List<Provincia>) query.getResultList();
        if (provincias.size() != 1) {
          respuesta.mensaje = "Error: La provincia no es una provincia valida";
          return respuesta;
        } else u.setProvincia(provincias.get(0));
      }

      if (provincia != null && !provincia.trim().isEmpty() && localidad != null && !localidad.trim().isEmpty()) {
        String consulta = "SELECT e FROM " + Localidad.class.getName() + " e "
            + "WHERE nombre like '%:valor1%' and provincia = :valor2";
        Query query = service.getEntityManager().createQuery(consulta);
        query.setParameter("valor1", localidad.trim());
        query.setParameter("valor1", u.getProvincia());
        List<Localidad> localidades = (List<Localidad>) query.getResultList();
        if (localidades.size() != 1) {
          respuesta.mensaje = "Advertencia: La localidad no es valida";
          return respuesta;
        } else u.setLocalidad(localidades.get(0));
      }
      if (nif != null && !nif.trim().isEmpty()) u.setNif(nif.trim().toUpperCase());

      if (cp != null) {
        cp = cp.replaceAll("[^0-9]", "");
        if (!cp.trim().isEmpty()) u.setCp(cp.trim());
      }

      service.persist(u);
      respuesta.correcto = true;
      respuesta.usuario = crearWSUsuario(u);

    } catch (RuntimeException e) {
      logger.log(Level.SEVERE, "Error en el preregistro del usuario: " + e.getLocalizedMessage());
      respuesta.mensaje = "Error en el preregistro del usuario: " + e.getLocalizedMessage();
      e.printStackTrace();
    }

    return respuesta;
  }

  /**
   * Matricular usuario.
   * 
   * @param idUsuario
   *           id usuario
   * @param idCurso
   *           id curso
   * @return the wS respuesta matriculacion
   */
  @WebMethod
  @Transactional
  public WSRespuestaMatriculacion matricularUsuario(String idUsuario, String idCurso) {
    WSRespuestaMatriculacion respuesta = new WSRespuestaMatriculacion();
    MatriculaCurso m = new MatriculaCurso();
    try {
      if (idUsuario == null || idUsuario.trim().isEmpty()) {
        respuesta.mensaje = "Error: El usuario es obligatorio";
        return respuesta;
      } else {
        Long id = Long.parseLong(idUsuario);
        UsuarioCyL u = service.find(UsuarioCyL.class, id);
        if (u != null) {
          m.setUsuario(u);
        } else {
          respuesta.mensaje = "Error: El usuario no se encuentra en nuestra base de datos";
          return respuesta;
        }
      }

      if (idCurso == null || idCurso.trim().isEmpty()) {
        respuesta.mensaje = "Error: El usuario es obligatorio";
        return respuesta;
      } else {
        Long id = Long.parseLong(idCurso);
        Curso c = service.find(Curso.class, id);
        if (c != null) {
          m.setCurso(c);
        } else {
          respuesta.mensaje = "Error: El curso no se encuentra en nuestra base de datos";
          return respuesta;
        }
      }

      if (m.getCurso().isCompleto()) {
        respuesta.mensaje = "Error: El curso está completo";
        return respuesta;
      }

      if (m.getCurso().getFechaInicio() != null && m.getCurso().getFechaInicio().before(new Date())) {
        respuesta.mensaje = "Error: El curso ya ha comenzado";
        return respuesta;
      }

      m.setAdmitido(false);
      m.setFechaMatricula(new Date());
      m.setObservaciones("Inscrito desde www.cyldigital.es");

      service.persist(m);

      respuesta.correcto = true;
      respuesta.matricula = crearWSMatriculaCurso(m);

    } catch (NumberFormatException e) {
      logger.log(Level.SEVERE, "Error al convertir el idUsuario o el idCurso a número: " + e.getLocalizedMessage());
      respuesta.mensaje = "Error al convertir el idUsuario o el idCurso a un número.";
      e.printStackTrace();
    } catch (RuntimeException e) {
      logger.log(Level.SEVERE, "Error en la matriculación del usuario: " + e.getLocalizedMessage());
      respuesta.mensaje = "Error en la matriculación usuario: " + e.getLocalizedMessage();
      e.printStackTrace();
    }

    return respuesta;
  }
  
  // ----------------------------------------------------------------------------------------------------CLASES INTERNAS

  /**
   * Debe ser estatica para que jaxb la cargue.
   */
  public static class WSServicio {
    public long id;
    public String nombre;
    public String descripcion;
    public String codigoJunta;
    public int edadMinima;
    public int edadMaxima;
    public String tematica;
    public String nivel;
    public String tipo;
  }

  private WSServicio crearWSServicio(Servicio ser) {
    WSServicio servicio = new WSServicio();
    servicio.id = ser.getId();
    servicio.codigoJunta = ser.getCodigoJunta();
    //servicio.nombre = ser.getNombre();
    //servicio.descripcion = ser.getDescripcion();
    servicio.edadMinima = ser.getEdadMinima();
    servicio.edadMaxima = ser.getEdadMaxima();
    //servicio.tematica = ser.getTematica() != null ? ser.getTematica().getNombre() : "";
    if (ser instanceof ServicioCurso) {
      servicio.nivel = ((ServicioCurso) ser).getNivel().equals(Nivel.A) ? "Avanzado" : ((ServicioCurso) ser).getNivel()
          .equals(Nivel.M) ? "Medio" : "Bajo";
      servicio.tipo = "curso";
    } else if (ser instanceof ServicioTaller) {
      servicio.nivel = ((ServicioTaller) ser).getNivel().equals(Nivel.A) ? "Avanzado" : ((ServicioTaller) ser)
          .getNivel().equals(Nivel.M) ? "Medio" : "Bajo";
      servicio.tipo = "taller";
    } else {
      servicio.nivel = "";
      servicio.tipo = "desconocido";
    }

    return servicio;
  }

  /**
   * Clase WSServicioCurso.
   */
  public static class WSServicioCurso {
    public long id;
    public String nombre;
    public String descripcion;
    public String codigoJunta;
    public int edadMinima;
    public int edadMaxima;
    public String tematica;
    public String nivel;
  }

  private WSServicioCurso crearWSServicioCurso(ServicioCurso servicioCurso) {
    WSServicioCurso curso = new WSServicioCurso();
    curso.id = servicioCurso.getId();
    curso.codigoJunta = servicioCurso.getCodigoJunta();
    //curso.nombre = servicioCurso.getNombre();
    //curso.descripcion = servicioCurso.getDescripcion();
    curso.edadMinima = servicioCurso.getEdadMinima();
    curso.edadMaxima = servicioCurso.getEdadMaxima();
    //curso.tematica = servicioCurso.getTematica() != null ? servicioCurso.getTematica().getNombre() : "";
    curso.nivel = servicioCurso.getNivel().equals(Nivel.A) ? "Avanzado"
        : servicioCurso.getNivel().equals(Nivel.M) ? "Medio" : "Bajo";
    return curso;
  }

  /**
   * Clase WSActividad.
   */
  public static class WSActividad {
    public long id;
    public int numeroInscritos;
    public int numeroPlazas;
    public float numeroHoras;
    public String nombre;
    public String descripcion;
    public String codigoJunta;
    public int edadMinima;
    public int edadMaxima;
    public String tematica;
    public Long idTematica;
    public String nivel;
    public String horario;
    public Date fechaFin;
    public Date fechaInicio;
    public String centro;
    public int plazasEnListaEspera;
    public String tipo;
    public List<WSCalendarItem> calendarItems;
  }

  private WSActividad crearWSActividad(Actividad act) {
    WSActividad actividad = new WSActividad();
    actividad.id = act.getId();
    actividad.codigoJunta = act.getServicio().getCodigoJunta();
    //actividad.nombre = act.getServicio().getNombre();
    //actividad.descripcion = act.getServicio().getDescripcion();
    actividad.edadMinima = act.getServicio().getEdadMinima();
    actividad.edadMaxima = act.getServicio().getEdadMaxima();
    //actividad.tematica = act.getServicio().getTematica() != null ? act.getServicio().getTematica().getNombre() : "";
    actividad.idTematica = act.getServicio().getTematica() != null ? act.getServicio().getTematica().getId() : null;
    actividad.centro = act.getCentro().getNombre();
    actividad.fechaInicio = act.getFechaInicio();
    actividad.fechaFin = act.getFechaFin();
    if (act instanceof Curso) {
      actividad.horario = ((Curso) act).getHorarioLargo();
      actividad.numeroHoras = ((Curso) act).getNumeroHoras();
      actividad.numeroPlazas = ((Curso) act).getNumeroPlazas();
      actividad.numeroInscritos = ((Curso) act).getNumeroInscritos();
      actividad.plazasEnListaEspera = ((Curso) act).getNumeroPlazasListaEspera();
      actividad.tipo = "curso";
      actividad.nivel = ((ServicioCurso) act.getServicio()).getNivel().equals(Nivel.A) ? "Avanzado"
          : ((ServicioCurso) act.getServicio()).getNivel().equals(Nivel.M) ? "Medio" : "Bajo";
    } else if (act instanceof Taller) {
      actividad.horario = ((Taller) act).getHorarioLargo();
      actividad.numeroHoras = ((Taller) act).getNumeroHoras();
      actividad.numeroPlazas = ((Taller) act).getNumeroPlazas();
      actividad.numeroInscritos = ((Taller) act).getNumeroInscritos();
      actividad.plazasEnListaEspera = ((Taller) act).getNumeroPlazasListaEspera();
      actividad.tipo = "taller";
      actividad.nivel = ((ServicioTaller) act.getServicio()).getNivel().equals(Nivel.A) ? "Avanzado"
          : ((ServicioTaller) act.getServicio()).getNivel().equals(Nivel.M) ? "Medio" : "Bajo";
    } else if (act instanceof Charla) {
      actividad.horario = ((Charla) act).getHorario();
      actividad.numeroHoras = (((Charla) act).getHoraFin().getTime() - ((Charla) act).getHoraInicio().getTime()) / 1000 / 60 / 60;
      actividad.numeroPlazas = ((Charla) act).getNumeroPlazas();
      actividad.numeroInscritos = ((Charla) act).getNumeroInscritos();
      actividad.plazasEnListaEspera = 0;
      actividad.tipo = "charla";
      actividad.nivel = "";
    }
    actividad.calendarItems = new ArrayList<WSCalendarItem>();
    for (ActividadCalendarItem item : act.getCalendarItems()) {
      actividad.calendarItems.add(crearWSCalendarItem(item));
    }
    return actividad;
  }

  /**
   * Clase WSCurso.
   */
  public static class WSCurso {
    public long id;
    public int numeroInscritos;
    public int numeroPlazas;
    public float numeroHoras;
    public String nombre;
    public String descripcion;
    public String codigoJunta;
    public int edadMinima;
    public int edadMaxima;
    public String tematica;
    public Long idTematica;
    public String nivel;
    public String horario;
    public Date fechaFin;
    public Date fechaInicio;
    public String centro;
    public int plazasEnListaEspera;
    public List<WSCalendarItem> calendarItems;
  }

  /**
   * Clase WSCalendarItem.
   */
  public static class WSCalendarItem {
    public long id;
    public Date fechaInicio;
    public Date fechaFin;
    public String tipo;
  }

  /**
   * Clase WSTaller.
   */
  public static class WSTaller {
    public long id;
    public int numeroInscritos;
    public int numeroPlazas;
    public float numeroHoras;
    public String nombre;
    public String descripcion;
    public String codigoJunta;
    public int edadMinima;
    public int edadMaxima;
    public String tematica;
    public Long idTematica;
    public String nivel;
    public String horario;
    public Date fechaFin;
    public Date fechaInicio;
    public String centro;
    public int plazasEnListaEspera;
    List<WSCalendarItem> calendarItems;
  }

  /**
   * Clase WSCharla.
   */
  public static class WSCharla {
    public long id;
    public int numeroInscritos;
    public int numeroPlazas;
    public float numeroHoras;
    public String nombre;
    public String descripcion;
    public String codigoJunta;
    public int edadMinima;
    public int edadMaxima;
    public String tematica;
    public Long idTematica;
    public String nivel;
    public String horario;
    public Date fechaFin;
    public Date fechaInicio;
    public String centro;
    public int plazasEnListaEspera;
  }

  private WSCurso crearWSCurso(Curso c) {
    WSCurso curso = new WSCurso();
    curso.id = c.getId();
    curso.codigoJunta = c.getServicio().getCodigoJunta();
    //curso.nombre = c.getServicio().getNombre();
    //curso.descripcion = c.getServicio().getDescripcion();
    curso.edadMinima = c.getServicio().getEdadMinima();
    curso.edadMaxima = c.getServicio().getEdadMaxima();
    //curso.tematica = c.getServicio().getTematica() != null ? c.getServicio().getTematica().getNombre() : "";
    curso.idTematica = c.getServicio().getTematica() != null ? c.getServicio().getTematica().getId() : null;
    curso.nivel = ((ServicioCurso) c.getServicio()).getNivel().equals(Nivel.A) ? "Avanzado" : ((ServicioCurso) c
        .getServicio()).getNivel().equals(Nivel.M) ? "Medio" : "Bajo";
    curso.centro = c.getCentro().getNombre();
    curso.fechaInicio = c.getFechaInicio();
    curso.fechaFin = c.getFechaFin();
    curso.horario = c.getHorarioLargo();
    curso.numeroHoras = c.getNumeroHoras();
    curso.numeroPlazas = c.getNumeroPlazas();
    curso.numeroInscritos = c.getNumeroInscritos();
    curso.plazasEnListaEspera = c.getNumeroPlazasListaEspera();
    curso.calendarItems = new ArrayList<WSCalendarItem>();
    for (ActividadCalendarItem item : c.getCalendarItems()) {
      curso.calendarItems.add(crearWSCalendarItem(item));
    }
    return curso;
  }

  private <T extends CalendarItem> WSCalendarItem crearWSCalendarItem(T item) {
    WSCalendarItem ci = new WSCalendarItem();
    ci.id = item.getId();
    ci.fechaInicio = item.getStart();
    ci.fechaFin = item.getEnd();
    ci.tipo = item.getTipo().toString();
    return ci;
  }

  private WSTaller crearWSTaller(Taller t) {
    WSTaller taller = new WSTaller();
    taller.id = t.getId();
    taller.codigoJunta = t.getServicio().getCodigoJunta();
    //taller.nombre = t.getServicio().getNombre();
    //taller.descripcion = t.getServicio().getDescripcion();
    taller.edadMinima = t.getServicio().getEdadMinima();
    taller.edadMaxima = t.getServicio().getEdadMaxima();
    //taller.tematica = t.getServicio().getTematica() != null ? t.getServicio().getTematica().getNombre() : "";
    taller.idTematica = t.getServicio().getTematica() != null ? t.getServicio().getTematica().getId() : null;
    taller.nivel = ((ServicioTaller) t.getServicio()).getNivel().equals(Nivel.A) ? "Avanzado" : ((ServicioTaller) t
        .getServicio()).getNivel().equals(Nivel.M) ? "Medio" : "Bajo";
    taller.centro = t.getCentro().getNombre();
    taller.fechaInicio = t.getFechaInicio();
    taller.fechaFin = t.getFechaFin();
    taller.horario = t.getHorarioLargo();
    taller.numeroHoras = t.getNumeroHoras();
    taller.numeroPlazas = t.getNumeroPlazas();
    taller.numeroInscritos = t.getNumeroInscritos();
    taller.plazasEnListaEspera = t.getNumeroPlazasListaEspera();
    taller.calendarItems = new ArrayList<WSCalendarItem>();
    for (ActividadCalendarItem item : t.getCalendarItems()) {
      taller.calendarItems.add(crearWSCalendarItem(item));
    }
    return taller;
  }

  private WSCharla crearWSCharla(Charla c) {
    WSCharla charla = new WSCharla();
    charla.id = c.getId();
    charla.codigoJunta = c.getServicio().getCodigoJunta();
    //charla.nombre = c.getServicio().getNombre();
    //charla.descripcion = c.getServicio().getDescripcion();
    charla.edadMinima = c.getServicio().getEdadMinima();
    charla.edadMaxima = c.getServicio().getEdadMaxima();
    //charla.tematica = c.getServicio().getTematica() != null ? c.getServicio().getTematica().getNombre() : "";
    charla.idTematica = c.getServicio().getTematica() != null ? c.getServicio().getTematica().getId() : null;
    charla.centro = c.getCentro().getNombre();
    charla.fechaInicio = c.getFechaInicio();
    charla.fechaFin = c.getFechaFin();
    charla.horario = c.getHorario();
    charla.numeroPlazas = c.getNumeroPlazas();
    charla.numeroInscritos = c.getNumeroInscritos();
    charla.numeroHoras = (((Charla) c).getHoraFin().getTime() - ((Charla) c).getHoraInicio().getTime()) / 1000 / 60 / 60;
    charla.plazasEnListaEspera = 0;
    charla.nivel = "";

    return charla;
  }

  /**
   * Clase WSUsuario.
   */
  public static class WSUsuario {
    public long id;
    public String nif;

    public String nombre;
    public String apellido1;
    public String apellido2;
    public String telefono;
    public String movil;
    public String email;
    public String direccion;
    public String cp;
    public String localidad;
    public String provincia;
    public String centro;

    public String nifPersonaAutorizada;
    public String nombreCompletoAutorizado;
    public String pasaporte;
    public boolean extranjero;
    public boolean menor;

    public Date fechaNacimiento;
    public String tipoUsuario;
    public String tipoDiscapacidad;
    public String sexo;
    public String medioPublicidad;
    public String medioPublicidadOtro;
    public String publicidad;

    public boolean poseeOrdenador;
    public boolean accesoInternet;
    public boolean deseaComunicacionTlf;
    public boolean deseaComunicacionMail;
    public boolean deseaComunicacionSms;

    public Date fechaCreacion;
    public boolean cesionPermitida;
    public boolean preInscrito;
  }

  private WSUsuario crearWSUsuario(Usuario u) {
    WSUsuario usuario = new WSUsuario();
    usuario.id = u.getId();

    usuario.nif = u.getNif();
    usuario.nombre = u.getNombre();
    usuario.apellido1 = u.getApellido1();
    usuario.apellido2 = u.getApellido2();
    usuario.telefono = u.getTelefono();

    usuario.direccion = u.getDireccion();
    usuario.cp = u.getCp();
    usuario.localidad = u.getLocalidad() != null ? u.getLocalidad().getNombre() : "";
    usuario.provincia = u.getProvincia() != null ? u.getProvincia().getNombre() : "";
    usuario.centro = u.getCentro() != null ? u.getCentro().getNombre() : "";

    if (u instanceof UsuarioCyL) {
      UsuarioCyL ucyl = (UsuarioCyL) u;
      usuario.movil = ucyl.getMovil();
      usuario.email = ucyl.getEmail();
      usuario.nifPersonaAutorizada = ucyl.getNifAutorizado();
      usuario.nombreCompletoAutorizado = ucyl.getNombreCompletoAutorizado();
      usuario.pasaporte = ucyl.getPasaporte();
      usuario.extranjero = ucyl.isExtranjero();
      usuario.menor = ucyl.isMenor();
      usuario.fechaNacimiento = ucyl.getFechaNacimiento();
      usuario.tipoUsuario = ucyl.getTipoUsuario() != null ? ucyl.getTipoUsuario().toString().toLowerCase() : "";
      usuario.tipoDiscapacidad = ucyl.getTipoDiscapacidad() != null ? ucyl.getTipoDiscapacidad().toString()
          .toLowerCase() : "";
      usuario.sexo = ucyl.getSexo().toString().toLowerCase();
      usuario.medioPublicidad = ucyl.getMedioPublicidad() != null ? ucyl.getMedioPublicidad().toString().toLowerCase()
          : "";
      usuario.medioPublicidadOtro = ucyl.getMedioPublicidadOtro();
      //usuario.publicidad = ucyl.getPublicidad() != null ? ucyl.getPublicidad().toString().toLowerCase() : "";
      usuario.poseeOrdenador = ucyl.isPoseeOrdenador();
      usuario.accesoInternet = ucyl.isAccesoInternet();
      usuario.deseaComunicacionTlf = ucyl.iscomunicacionTlf();
      usuario.deseaComunicacionMail = ucyl.iscomunicacionMail();
      usuario.deseaComunicacionSms = ucyl.iscomunicacionSms();
      usuario.fechaCreacion = ucyl.getFechaCreacion();
      usuario.cesionPermitida = ucyl.isCesionPermitida();
      usuario.preInscrito = false;
    } else if (u instanceof UsuarioPre) {
      usuario.preInscrito = true;
    }
    return usuario;
  }

  /**
   * Clase WSRespuestaGetUsuario.
   */
  public static class WSRespuestaGetUsuario {
    public boolean correcto;
    public String mensaje;
    public WSUsuario usuario;

    /**
     * Construye un nuevo objeto de la clase
     */
    public WSRespuestaGetUsuario() {
      super();
      correcto = false;
      mensaje = "";
      usuario = null;
    }
  }

  /**
   * Clase WSRespuestaGetUserPassword.
   */
  public static class WSRespuestaGetUserPassword {
    public boolean correcto;
    public String mensaje;
    public String password;
    public String domain;

    /**
     * Construye un nuevo objeto de la clase
     */
    public WSRespuestaGetUserPassword() {
      super();
      correcto = false;
      mensaje = "";
      password = "";
      domain = "";
    }
  }

  /**
   * Clase WSMatriculaCurso.
   */
  public static class WSMatriculaCurso {
    public WSActividad actividad;
    public WSUsuario usuario;
    public Date fechaMatricula;
    public Float nota;
    public Float asistencia;
    public boolean enListaEspera;
    public boolean admitido;

    /**
     * Construye un nuevo objeto de la clase
     */
    public WSMatriculaCurso() {
      super();
    }
  }

  /**
   * Clase WSMatriculaTaller.
   */
  public static class WSMatriculaTaller {
    public WSActividad actividad;
    public WSUsuario usuario;
    public Date fechaMatricula;
    public Float nota;
    public Float asistencia;
    public boolean enListaEspera;
    public boolean admitido;

    /**
     * Construye un nuevo objeto de la clase
     */
    public WSMatriculaTaller() {
      super();
    }
  }

  /**
   * Clase WSMatriculaCharla.
   */
  public static class WSMatriculaCharla {
    public WSActividad actividad;
    public WSUsuario usuario;
    public Date fechaMatricula;
    public boolean enListaEspera;
    public boolean admitido;

    /**
     * Construye un nuevo objeto de la clase
     */
    public WSMatriculaCharla() {
      super();
    }
  }

  private WSMatriculaCurso crearWSMatriculaCurso(MatriculaCurso m) {
    WSMatriculaCurso matricula = new WSMatriculaCurso();
    matricula.admitido = m.isAdmitido();
    matricula.asistencia = m.getAsistencia();
    matricula.nota = m.getNota();
    matricula.fechaMatricula = m.getFechaMatricula();
    matricula.enListaEspera = m.isEnListaEspera();
    matricula.actividad = crearWSActividad(m.getCurso());
    matricula.usuario = crearWSUsuario((Usuario)m.getUsuario());
    return matricula;
  }

  private WSMatriculaTaller crearWSMatriculaTaller(MatriculaTaller m) {
    WSMatriculaTaller matricula = new WSMatriculaTaller();
    matricula.admitido = m.isAdmitido();
    matricula.asistencia = m.getAsistencia();
    matricula.nota = m.getNota();
    matricula.fechaMatricula = m.getFechaMatricula();
    matricula.enListaEspera = m.isEnListaEspera();
    matricula.actividad = crearWSActividad(m.getTaller());
    matricula.usuario = crearWSUsuario((Usuario)m.getUsuario());
    return matricula;
  }

  private WSMatriculaCharla crearWSMatriculaCharla(MatriculaCharla m) {
    WSMatriculaCharla matricula = new WSMatriculaCharla();
    matricula.fechaMatricula = m.getFechaConfirmacion();
    matricula.actividad = crearWSActividad(m.getCharla());
    matricula.usuario = crearWSUsuario((Usuario)m.getUsuario());
    return matricula;
  }

  /**
   * Clase WSRespuestaHistorial.
   */
  public static class WSRespuestaHistorial {
    public boolean correcto;
    public String mensaje;
    public List<WSMatriculaCurso> matriculasCurso;
    public List<WSMatriculaTaller> matriculasTaller;
    public List<WSMatriculaCharla> matriculasCharla;

    /**
     * Construye un nuevo objeto de la clase
     */
    public WSRespuestaHistorial() {
      super();
      correcto = false;
      mensaje = "";
      matriculasCurso = new ArrayList<WSMatriculaCurso>();
      matriculasTaller = new ArrayList<WSMatriculaTaller>();
      matriculasCharla = new ArrayList<WSMatriculaCharla>();
    }
  }

  /**
   * Clase WSRespuestaPreRegistro.
   */
  public static class WSRespuestaPreRegistro {
    public boolean correcto;
    public String mensaje;
    public WSUsuario usuario;

    /**
     * Construye un nuevo objeto de la clase
     */
    public WSRespuestaPreRegistro() {
      super();
      correcto = false;
      mensaje = "";
      usuario = null;
    }
  }

  /**
   * Clase WSRespuestaGetCurso.
   */
  public static class WSRespuestaGetCurso {
    public boolean correcto;
    public String mensaje;
    public WSCurso curso;

    /**
     * Construye un nuevo objeto de la clase
     */
    public WSRespuestaGetCurso() {
      super();
      correcto = false;
      mensaje = "";
      curso = null;
    }
  }

  /**
   * Clase WSRespuestaGetTaller.
   */
  public static class WSRespuestaGetTaller {
    public boolean correcto;
    public String mensaje;
    public WSTaller taller;

    /**
     * Construye un nuevo objeto de la clase
     */
    public WSRespuestaGetTaller() {
      super();
      correcto = false;
      mensaje = "";
      taller = null;
    }
  }

  /**
   * Clase WSRespuestaGetCharla.
   */
  public static class WSRespuestaGetCharla {
    public boolean correcto;
    public String mensaje;
    public WSCharla charla;

    /**
     * Construye un nuevo objeto de la clase
     */
    public WSRespuestaGetCharla() {
      super();
      correcto = false;
      mensaje = "";
      charla = null;
    }
  }

  /**
   * Clase WSRespuestaMatriculacion.
   */
  public static class WSRespuestaMatriculacion {
    public boolean correcto;
    public String mensaje;
    public WSMatriculaCurso matricula;

    /**
     * Construye un nuevo objeto de la clase
     */
    public WSRespuestaMatriculacion() {
      super();
      correcto = false;
      mensaje = "";
      matricula = null;
    }
  }

}
