package es.csa.fdt.cyldigital.integracion;

/*
 * #%L
 * Servicios Integrados para aplicaciones remotas - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jws.WebMethod;
import javax.jws.WebService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.notificacion.NotificationService;
import es.csa.fdt.cyldigital.recursos.service.RemoteNoticeService;
import es.csa.fdt.cyldigital.servicios.entity.CursoOnline;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCursoOnline;
import es.csa.fdt.cyldigital.servicios.entity.PreinscripcionCursoOnline;
import es.csa.fdt.cyldigital.servicios.entity.ServicioCursoOnline;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioOnline;

/**
 * Clase integracionMoodle.
 */
@Component("integracionMoodle")
@WebService
public class integracionMoodle {

  private static final Logger logger = Logger.getLogger(integracionMoodle.class.getName());

  private static final int MAX_NOTICE_BURST = 10;

  @Autowired
  private RemoteNoticeService service;

  @Autowired
  private NotificationService serviceNotification;

  /**
   * Lista de todos los servicios de cursos online.
   * 
   * @return the servicios online
   */
  @WebMethod
  @Transactional
  public List<WSServicioOnline> getServiciosOnline() {
    List<WSServicioOnline> servicios = new ArrayList<WSServicioOnline>();
    try {
      service.setMessagesEnabled(false);
      if (service != null) {
        List<ServicioCursoOnline> cursosBd = service.findAll(ServicioCursoOnline.class);
        for (ServicioCursoOnline servicioCurso : cursosBd) {
          WSServicioOnline curso = crearWSServicio(servicioCurso);
          servicios.add(curso);
        }
      }
    } catch (RuntimeException e) {
      logger.log(Level.SEVERE, "Error al devolver los servicios del tipo curso online: " + e.getLocalizedMessage());
      e.printStackTrace();
    }
    return servicios;
  }

  /**
   * Devuelve cursos online for cron.
   * 
   * @return the cursos online for cron
   */
  @WebMethod
  @Transactional
  public List<WSCursoMoodle> getCursosOnlineForCron() {
    List<WSCursoMoodle> servicios = new ArrayList<WSCursoMoodle>();
    try {
      service.setMessagesEnabled(false);
      if (service != null) {
        List<ServicioCursoOnline> cursosBd = service.findAll(ServicioCursoOnline.class);
        for (ServicioCursoOnline servicioCurso : cursosBd) {
          CursoOnline curso = servicioCurso.getActualCursoOnline();
          if (curso != null) {
            WSCursoMoodle wsCurso = crearWSCursoMoodle(curso);
            servicios.add(wsCurso);
          }
        }
      }
    } catch (RuntimeException e) {
      logger.log(Level.SEVERE, "Error al devolver los servicios del tipo curso online: " + e.getLocalizedMessage());
      e.printStackTrace();
    }
    return servicios;
  }

  /**
   * Asigna notas curso online.
   * 
   * @param idServicio
   *          id servicio
   * @param idUsuario
   *          id usuario
   * @param nota
   *          nota
   * @return the wS respuesta set notas
   */
  @WebMethod
  @Transactional
  public WSRespuestaSetNotas setNotasCursoOnline(Long idServicio, Long idUsuario, Float nota) {
    WSRespuestaSetNotas respuesta = new WSRespuestaSetNotas();
    try {
      service.setMessagesEnabled(false);
      respuesta.correcto = false;
      ServicioCursoOnline sco = service.find(ServicioCursoOnline.class, idServicio);
      if (sco != null) {
        CursoOnline co = sco.getActualCursoOnline();
        for (MatriculaCursoOnline mco : co.getMatriculas()) {
          if (Long.parseLong(mco.getUsuario().getIdWeb()) == idUsuario) {
            MatriculaCursoOnline m = service.find(MatriculaCursoOnline.class, mco.getId());
            if (nota != null) {
              // Método para redondear la parte decimal a 2 decimales
              nota = Float.valueOf(((float) (Math.round(nota * 100))) / 100);
              m.setNota(nota);
              service.persist(m);
              respuesta.correcto = true;
              notificarNotaMatriculaUsuario(m);
            }
          }
        }
        if (!respuesta.correcto) {
          logger.log(Level.SEVERE, "El usuario online " + idUsuario + " no está matriculado del curso.");
          respuesta.mensaje = "El usuario online no está matriculado del curso en nuestra base de datos.";
        }
      } else {
        logger.log(Level.SEVERE, "El servicio online " + idServicio + " no existe.");
        respuesta.mensaje = "El servicio online no existe en nuestra base de datos.";
      }

    } catch (Exception e) {
      logger.log(Level.SEVERE, "Error al introducir las notas del curso online: " + e.getLocalizedMessage());
      respuesta.mensaje = "Error al introducir las notas del curso online. ";
      e.printStackTrace();
    }
    return respuesta;
  }

  /**
   * Lista de todos los cursos online.
   * 
   * @return the cursos online
   */
  @WebMethod
  @Transactional
  public List<WSCursoOnline> getCursosOnline() {
    List<WSCursoOnline> cursos = new ArrayList<WSCursoOnline>();
    try {
      service.setMessagesEnabled(false);
      if (service != null) {
        List<CursoOnline> cursosBd = service.findAll(CursoOnline.class);
        for (CursoOnline c : cursosBd) {
          if (!c.getDesactivado()) {
            WSCursoOnline curso = crearWSCurso(c);
            cursos.add(curso);
          }
        }
      }
    } catch (RuntimeException e) {
      logger.log(Level.SEVERE, "Error al devolver los servicios del tipo curso: " + e.getLocalizedMessage());
      e.printStackTrace();
    }
    return cursos;
  }

  /**
   * Lista de todos los cursos activos.
   * 
   * @return the cursos online activos
   */
  @WebMethod
  @Transactional
  public List<WSCursoOnline> getCursosOnlineActivos() {
    List<WSCursoOnline> cursos = new ArrayList<WSCursoOnline>();
    for (WSCursoOnline wsCurso : getCursosOnline())
      if (wsCurso.fechaFin != null && wsCurso.fechaFin.after(new Date())) cursos.add(wsCurso);
    return cursos;
  }

  /**
   * Checks if is usuario online matriculado.
   * 
   * @param idWeb
   *          id web
   * @param idCurso
   *          id curso
   * @return true, if is usuario online matriculado
   */
  @WebMethod
  @Transactional
  public boolean isUsuarioOnlineMatriculado(String idWeb, String idCurso) {
    if (idCurso != null) {
      CursoOnline curso = service.find(CursoOnline.class, Long.parseLong(idCurso));

      if (curso != null) {
        Map<String, Object> propiedades = new HashMap<String, Object>();
        propiedades.put("idWeb", idWeb);
        List<UsuarioOnline> lu = service.findWhere(UsuarioOnline.class, propiedades);
        UsuarioOnline u;
        if (lu != null && !lu.isEmpty()) {
          u = lu.get(0);
          for (MatriculaCursoOnline mc : curso.getMatriculas()) {
            if (mc.getUsuario().getId() == u.getId()) { return true; }
          }
          for (PreinscripcionCursoOnline pc : ((ServicioCursoOnline) curso.getServicio()).getPreinscripciones()) {
            if (pc.getUsuario().getId() == u.getId()) { return true; }
          }
        }
      }
    }
    return false;
  }

  /**
   * Checks if is usuario online repite curso.
   * 
   * @param idWeb
   *          id web
   * @param idCurso
   *          id curso
   * @return true, if is usuario online repite curso
   */
  @WebMethod
  @Transactional
  public boolean isUsuarioOnlineRepiteCurso(String idWeb, String idCurso) {
    if (idCurso != null) {
      CursoOnline curso = service.find(CursoOnline.class, Long.parseLong(idCurso));

      if (curso != null) {
        Map<String, Object> propiedades = new HashMap<String, Object>();
        propiedades.put("idWeb", idWeb);
        List<UsuarioOnline> lu = service.findWhere(UsuarioOnline.class, propiedades);
        UsuarioOnline u;
        if (lu != null && !lu.isEmpty()) {
          u = lu.get(0);
          for (PreinscripcionCursoOnline pc : ((ServicioCursoOnline) curso.getServicio()).getPreinscripciones()) {
            if (pc.getUsuario().getId() == u.getId()) { return true; }
          }
        }
      }
    }
    return false;
  }

  /**
   * Checks if is usuario online aprobado.
   * 
   * @param idWeb
   *          id web
   * @param idCurso
   *          id curso
   * @return true, if is usuario online aprobado
   */
  @WebMethod
  @Transactional
  public boolean isUsuarioOnlineAprobado(String idWeb, String idCurso) {
    if (idCurso != null) {
      CursoOnline curso = service.find(CursoOnline.class, Long.parseLong(idCurso));

      if (curso != null) {
        Map<String, Object> propiedades = new HashMap<String, Object>();
        propiedades.put("idWeb", idWeb);
        List<UsuarioOnline> lu = service.findWhere(UsuarioOnline.class, propiedades);
        UsuarioOnline u;
        if (lu != null && !lu.isEmpty()) {
          u = lu.get(0);
          for (MatriculaCursoOnline mc : curso.getMatriculas()) {
            if (mc.getUsuario().getId() == u.getId() && mc.getNota() >= 5) { return true; }
          }
        }
      }
    }
    return false;
  }

  /**
   * Retorna el usuario online si encuentra un usuario con el mismo id y login.
   * 
   * @param id
   *          id
   * @return the historial usuario online
   */
  @WebMethod
  @Transactional
  public WSRespuestaHistorial getHistorialUsuarioOnline(String id) {
    WSRespuestaHistorial respuesta = new WSRespuestaHistorial();
    if (id != null && !id.isEmpty()) {
      Map<String, Object> propiedades = new HashMap<String, Object>();
      propiedades.put("idWeb", id);
      List<UsuarioOnline> lu = service.findWhere(UsuarioOnline.class, propiedades);
      if (lu != null && !lu.isEmpty()) {
        UsuarioOnline u = lu.get(0);
        for (MatriculaCursoOnline m : u.getMatriculasCursos()) {
          respuesta.matriculasCurso.add(crearWSMatriculaCurso(m));
        }
        for (PreinscripcionCursoOnline p : u.getPreinscripcionesCursos()) {
          respuesta.preinscripcionesCurso.add(crearWSPreinscripcionCurso(p));
        }
        respuesta.correcto = true;
      }
    }
    return respuesta;
  }

  /**
   * Matricular usuario online.
   * 
   * @param idWeb
   *          id web
   * @param login
   *          login
   * @param email
   *          email
   * @param idCursoOnline
   *          id curso online
   * @param idIsis
   *          id isis
   * @return the wS respuesta matriculacion
   */
  @WebMethod
  @Transactional
  public WSRespuestaMatriculacion matricularUsuarioOnline(String idWeb, String login, String email,
      String idCursoOnline, String idIsis) {
    WSRespuestaMatriculacion respuesta = new WSRespuestaMatriculacion();

    if (isUsuarioOnlineMatriculado(idWeb, idCursoOnline)) {
      respuesta.mensaje = "Error: El usuario ya está inscrito del curso online.";
      return respuesta;
    }
    MatriculaCursoOnline m = new MatriculaCursoOnline();
    PreinscripcionCursoOnline p = new PreinscripcionCursoOnline();
    UsuarioOnline u;
    try {
      service.setMessagesEnabled(false);

      // Crear o modificar el Usuario Online
      if (idCursoOnline == null || idCursoOnline.trim().isEmpty()) {
        respuesta.mensaje = "Error: El curso es obligatorio";
        return respuesta;
      } else {
        CursoOnline c = service.find(CursoOnline.class, Long.parseLong(idCursoOnline));
        if (c != null) {
          if ((c.getNumeroPlazas() + c.getNumeroPlazasListaEspera()) <= c.getNumeroInscritos()) {
            p.setServicioCurso((ServicioCursoOnline) c.getServicio());
            respuesta.enListaInteresados = true;
          } else {
            m.setCurso(c);
          }
        } else {
          respuesta.mensaje = "Error: El curso online no se encuentra en nuestra base de datos";
          return respuesta;
        }
        if (idWeb == null || idWeb.trim().isEmpty()) {
          respuesta.mensaje = "Error: El usuario online es obligatorio";
          return respuesta;
        } else {
          Map<String, Object> propiedades = new HashMap<String, Object>();
          propiedades.put("idWeb", idWeb);
          List<UsuarioOnline> lu = service.findWhere(UsuarioOnline.class, propiedades);
          if (lu != null && !lu.isEmpty()) {
            u = lu.get(0);

            // si no existe el usuario online en bbdd lo creo
          } else {
            u = new UsuarioOnline();
            u.setIdWeb(idWeb);
          }
          u.setLogin(login);
          u.setEmail(email);

          // se asocia el usuario de Isis si se puede
          if (idIsis != null && !idIsis.trim().isEmpty()) {
            Usuario isis = service.find(Usuario.class, Long.parseLong(idIsis));
            u.setUsuario(isis);
          }
          service.persist(u);

        }
      }
      if (respuesta.enListaInteresados) {
        // persistir la preinscripcion
        p.setUsuario(u);
        p.setFecha(new Date());
        service.persist(p);
      } else {
        // persistir la matrícula
        m.setUsuario(u);
        m.setEnListaEspera(m.getCurso().getNumeroMatriculados() >= m.getCurso().getNumeroPlazas());
        m.setAdmitido(false);
        m.setFechaMatricula(new Date());
        m.setObservaciones("Inscrito desde www.cyldigital.es");
        service.persist(m);

        respuesta.enListaEspera = m.isEnListaEspera();
        respuesta.inscrito = !m.isEnListaEspera();
      }

      respuesta.correcto = true;

    } catch (NumberFormatException e) {
      logger.log(Level.SEVERE, "Error al convertir el idCursoOnline a número: " + e.getLocalizedMessage());
      respuesta.mensaje = "Error al convertir el idCursoOnline a un número.";
      e.printStackTrace();
    } catch (RuntimeException e) {
      logger.log(Level.SEVERE, "Error en la matriculación del usuario online: " + e.getLocalizedMessage());
      respuesta.mensaje = "Error en la matriculación usuario online: " + e.getLocalizedMessage();
      e.printStackTrace();
    }

    return respuesta;
  }

  /**
   * Repetir matricula usuario online.
   * 
   * @param idWeb
   *          id web
   * @param idCursoOnline
   *          id curso online
   * @return the wS respuesta matriculacion
   */
  @WebMethod
  @Transactional
  public WSRespuestaMatriculacion repetirMatriculaUsuarioOnline(String idWeb, String idCursoOnline) {
    WSRespuestaMatriculacion respuesta = new WSRespuestaMatriculacion();

    PreinscripcionCursoOnline p = new PreinscripcionCursoOnline();
    UsuarioOnline u;
    try {
      service.setMessagesEnabled(false);

      // Crear o modificar el Usuario Online
      if (idCursoOnline == null || idCursoOnline.trim().isEmpty()) {
        respuesta.mensaje = "Error: El curso es obligatorio";
        return respuesta;
      } else {
        CursoOnline c = service.find(CursoOnline.class, Long.parseLong(idCursoOnline));
        if (c != null) {
          p.setServicioCurso((ServicioCursoOnline) c.getServicio());
          respuesta.enListaInteresados = true;
        } else {
          respuesta.mensaje = "Error: El servicio curso online no se encuentra en nuestra base de datos";
          return respuesta;
        }
        if (idWeb == null || idWeb.trim().isEmpty()) {
          respuesta.mensaje = "Error: El usuario online es obligatorio";
          return respuesta;
        } else {
          Map<String, Object> propiedades = new HashMap<String, Object>();
          propiedades.put("idWeb", idWeb);
          List<UsuarioOnline> lu = service.findWhere(UsuarioOnline.class, propiedades);
          if (lu != null && !lu.isEmpty()) {
            u = lu.get(0);
            // persistir la preinscripcion
            p.setUsuario(u);
            p.setFecha(new Date());
            service.persist(p);
            respuesta.correcto = true;
          }
        }
      }

    } catch (NumberFormatException e) {
      logger.log(Level.SEVERE, "Error al convertir el idCursoOnline a número: " + e.getLocalizedMessage());
      respuesta.mensaje = "Error al convertir el idCursoOnline a un número.";
      e.printStackTrace();
    } catch (RuntimeException e) {
      logger.log(Level.SEVERE, "Error en la matriculación del usuario online: " + e.getLocalizedMessage());
      respuesta.mensaje = "Error en la matriculación usuario online: " + e.getLocalizedMessage();
      e.printStackTrace();
    }

    return respuesta;
  }

  /**
   * Elimina matricula curso.
   * 
   * @param idWeb
   *          id web
   * @param idCursoOnline
   *          id curso online
   * @return the wS respuesta des matriculacion
   */
  @WebMethod
  @Transactional
  public WSRespuestaDesMatriculacion removeMatriculaCurso(String idWeb, String idCursoOnline) {
    service.setMessagesEnabled(false);
    WSRespuestaDesMatriculacion respuesta = new WSRespuestaDesMatriculacion();
    try {
      // busco el usuario online
      Map<String, Object> propiedades = new HashMap<String, Object>();
      propiedades.put("idWeb", idWeb);
      List<UsuarioOnline> lu = service.findWhere(UsuarioOnline.class, propiedades);
      UsuarioOnline u;
      if (lu != null && !lu.isEmpty()) {
        u = lu.get(0);
      } else {
        respuesta.mensaje = "Error: el usuario no existe en la base de datos";
        return respuesta;
      }

      CursoOnline c = service.find(CursoOnline.class, Long.parseLong(idCursoOnline));

      if (c == null) {
        respuesta.mensaje = "Error: el curso online no existe en la base de datos";
      } else {
        MatriculaCursoOnline m = null;
        for (MatriculaCursoOnline matriculaCurso : c.getMatriculas()) {
          if (matriculaCurso.getUsuario().getId() == u.getId()) {
            m = matriculaCurso;
          }
        }
        if (m != null) {
          if (m.getCurso() != null) {
            m.getCurso().getMatriculas().remove(m);
          }
          if (m.getUsuario().getMatriculasCursos() != null) {
            m.getUsuario().getMatriculasCursos().remove(m);
          }
          notificaMatriculaBorrada(m);
          service.remove(m);
          respuesta.correcto = true;
        }

        if (!respuesta.correcto) {
          PreinscripcionCursoOnline p = null;
          for (PreinscripcionCursoOnline preinscripcionCurso : ((ServicioCursoOnline) c.getServicio())
              .getPreinscripciones()) {
            if (preinscripcionCurso.getUsuario().getId() == u.getId()) {
              p = preinscripcionCurso;
            }
          }
          if (p != null) {
            if (p.getServicioCurso() != null) {
              p.getServicioCurso().getPreinscripciones().remove(p);
            }
            if (p.getUsuario() != null) {
              p.getUsuario().getPreinscripcionesCursos().remove(p);
            }
            notificaPreinscripcionBorrada(u, c);
            service.remove(p);
            respuesta.correcto = true;
          }
        }

        if (!respuesta.correcto) {
          respuesta.mensaje = "Error: el usuario no está matriculado de este curso.";
        }
      }

    } catch (NumberFormatException e) {
      logger.log(Level.SEVERE, "Error al convertir el idCursoOnline a número: " + e.getLocalizedMessage());
      respuesta.mensaje = "Error al convertir el idCursoOnline a un número.";
      e.printStackTrace();
    } catch (RuntimeException e) {
      logger.log(Level.SEVERE, "Error en la cancelación de la matriculación del usuario online: "
          + e.getLocalizedMessage());
      respuesta.mensaje = "Error en la cancelación de la matriculación del usuario online: " + e.getLocalizedMessage();
      e.printStackTrace();
    }
    return respuesta;
  }

  @SuppressWarnings("unchecked")
  private void notificaMatriculaBorrada(MatriculaCursoOnline matriculaCurso) {
    String sendmail = "";
    List<Parameter> parameters = service.findWhere(Parameter.class, "nombre", "mail_activate");
    if (parameters.size() > 0) {
      sendmail = parameters.get(0).getValor();
    }

    if (sendmail.equals("1")) {
      serviceNotification.notificarDesuscritoCursoOnline(service, matriculaCurso.getUsuario().getEmail(),
          matriculaCurso.getCurso());
    }
  }

  @SuppressWarnings("unchecked")
  private void notificaPreinscripcionBorrada(UsuarioOnline u, CursoOnline c) {
    String sendmail = "";
    List<Parameter> parameters = service.findWhere(Parameter.class, "nombre", "mail_activate");
    if (parameters.size() > 0) {
      sendmail = parameters.get(0).getValor();
    }

    if (sendmail.equals("1")) {
      serviceNotification.notificarDesuscritoCursoOnline(service, u.getEmail(), c);
    }
  }

  @SuppressWarnings("unchecked")
  private void notificarNotaMatriculaUsuario(MatriculaCursoOnline matriculaCurso) {
    String sendmail = "";
    List<Parameter> parameters = service.findWhere(Parameter.class, "nombre", "mail_activate");
    if (parameters.size() > 0) {
      sendmail = parameters.get(0).getValor();
    }

    if (sendmail.equals("1")) {
      if (matriculaCurso.getNota() >= 5) serviceNotification.notificarAprobadoCursoOnline(service, matriculaCurso
          .getUsuario().getId(), matriculaCurso.getUsuario().getEmail(), matriculaCurso.getCurso(), matriculaCurso
          .getNota());
      else if (matriculaCurso.getNota() < 5) serviceNotification.notificarSuspendidoCursoOnline(service, matriculaCurso
          .getUsuario().getId(), matriculaCurso.getUsuario().getEmail(), matriculaCurso.getCurso(), matriculaCurso
          .getNota());
      else {

      }
    }
  }

  // ----------------------------------------------------------------------------------------------------CLASES INTERNAS

  /**
   * Debe ser estatica para que jaxb la cargue.
   */
  public static class WSServicioOnline {
    public long id;
    public String nombre;
    public String descripcion;
    public String tematica;
  }

  private WSServicioOnline crearWSServicio(ServicioCursoOnline ser) {
    WSServicioOnline servicio = new WSServicioOnline();
    servicio.id = ser.getId();
    //servicio.nombre = ser.getNombre();
    //servicio.descripcion = ser.getDescripcion();
    //servicio.tematica = ser.getTematica() != null ? ser.getTematica().getNombre() : "";
    return servicio;
  }

  /**
   * Clase WSCursoOnline.
   */
  public static class WSCursoOnline {
    public long id;
    public int numeroInscritos;
    public int numeroPlazas;
    public String nombre;
    public String descripcion;
    public String tematica;
    public Long idTematica;
    public Date fechaFin;
    public Date fechaInicio;
    public int plazasEnListaEspera;
  }

  private WSCursoOnline crearWSCurso(CursoOnline c) {
    WSCursoOnline curso = new WSCursoOnline();
    curso.id = c.getId();
    //curso.nombre = c.getServicio().getNombre();
    //curso.descripcion = c.getServicio().getDescripcion();
    //curso.tematica = c.getServicio().getTematica() != null ? c.getServicio().getTematica().getNombre() : "";
    curso.idTematica = c.getServicio().getTematica() != null ? c.getServicio().getTematica().getId() : null;
    curso.fechaInicio = c.getFechaInicio();
    curso.fechaFin = c.getFechaFin();
    curso.numeroPlazas = c.getNumeroPlazas();
    curso.numeroInscritos = c.getNumeroInscritos();
    curso.plazasEnListaEspera = c.getNumeroPlazasListaEspera();
    return curso;
  }

  /**
   * Clase WSCursoMoodle.
   */
  public static class WSCursoMoodle {
    public long idServicio;
    public String nombre;
    public String descripcion;
    public Date fechaInicio;
    public List<WSMatriculaMoodle> matriculas;
    public List<WSProfesorOnline> profesores;

    /**
     * Construye un nuevo objeto de la clase
     */
    public WSCursoMoodle() {
      super();
      matriculas = new ArrayList<WSMatriculaMoodle>();
      profesores = new ArrayList<WSProfesorOnline>();
    }
  }

  /**
   * Clase WSMatriculaMoodle.
   */
  public static class WSMatriculaMoodle {
    public long idServicio;
    public WSUsuarioOnline usuario;
    public Date fechaMatricula;

    /**
     * Construye un nuevo objeto de la clase
     */
    public WSMatriculaMoodle() {
      super();
      usuario = new WSUsuarioOnline();
    }
  }

  private WSCursoMoodle crearWSCursoMoodle(CursoOnline c) {
    WSCursoMoodle curso = new WSCursoMoodle();
    curso.idServicio = c.getServicio().getId();
    //curso.nombre = c.getServicio().getNombre();
    curso.descripcion = c.getObservaciones();
    curso.fechaInicio = c.getFechaInicio();

    for (MatriculaCursoOnline m : c.getMatriculas()) {
      if (m.isAdmitido()) {
        curso.matriculas.add(crearWSMatriculaMoodle(m));
      }
    }

    for (Empleado p : c.getProfesores()) {
      curso.profesores.add(crearWSProfesor(p));
    }
    return curso;
  }

  private WSMatriculaMoodle crearWSMatriculaMoodle(MatriculaCursoOnline m) {
    WSMatriculaMoodle matricula = new WSMatriculaMoodle();
    matricula.idServicio = m.getCurso().getServicio().getId();
    matricula.usuario = crearWSUsuario(m.getUsuario());
    matricula.fechaMatricula = m.getFechaMatricula();
    return matricula;
  }

  /**
   * Clase WSProfesorOnline.
   */
  public static class WSProfesorOnline {
    public String username;
    public String idWeb;
  }

  private WSProfesorOnline crearWSProfesor(Empleado e) {
    WSProfesorOnline profesor = new WSProfesorOnline();
    profesor.username = e.getUsernameWeb();
    profesor.idWeb = e.getIdWeb();

    return profesor;
  }

  /**
   * Clase WSUsuarioOnline.
   */
  public static class WSUsuarioOnline {
    public String idWeb;
    public String login;
    public String email;
    public long idUsuario;
    public String nombre;
    public String apellidos;
    public String ciudad;
  }

  private WSUsuarioOnline crearWSUsuario(UsuarioOnline u) {
    WSUsuarioOnline usuario = new WSUsuarioOnline();
    usuario.idWeb = u.getIdWeb();
    usuario.login = u.getLogin();
    usuario.email = u.getEmail();
    usuario.idUsuario = u.getId();
    if (u.getUsuario() != null) {
      usuario.nombre = u.getUsuario().getNombre();
      usuario.apellidos = u.getUsuario().getApellidos();
      usuario.ciudad = u.getUsuario().getLocalidad().getNombre();
    }

    return usuario;
  }

  /**
   * Clase WSRespuestaGetUsuario.
   */
  public static class WSRespuestaGetUsuario {
    public boolean correcto;
    public String mensaje;
    public WSUsuarioOnline usuario;

    /**
     * Construye un nuevo objeto de la clase
     */
    public WSRespuestaGetUsuario() {
      super();
      correcto = false;
      mensaje = "";
      usuario = null;
    }
  }

  /**
   * Clase WSRespuestaGetUserPassword.
   */
  public static class WSRespuestaGetUserPassword {
    public boolean correcto;
    public String mensaje;
    public String password;
    public String domain;

    /**
     * Construye un nuevo objeto de la clase
     */
    public WSRespuestaGetUserPassword() {
      super();
      correcto = false;
      mensaje = "";
      password = "";
      domain = "";
    }
  }

  /**
   * Clase WSRespuestaSetNotas.
   */
  public static class WSRespuestaSetNotas {
    public boolean correcto;
    public String mensaje;

    /**
     * Construye un nuevo objeto de la clase
     */
    public WSRespuestaSetNotas() {
      super();
      correcto = false;
      mensaje = "";
    }
  }

  /**
   * Clase WSNotaCurso.
   */
  public static class WSNotaCurso {
    public long idServicioCurso;
    public long idUsuario;
    public Float nota;
  }

  /**
   * Clase WSMatriculaCurso.
   */
  public static class WSMatriculaCurso {
    public WSCursoOnline actividad;
    public WSUsuarioOnline usuario;
    public Date fechaMatricula;
    public Float nota;
    public boolean enListaEspera;
    public boolean admitido;

    /**
     * Construye un nuevo objeto de la clase
     */
    public WSMatriculaCurso() {
      super();
    }
  }

  /**
   * Clase WSPreinscripcionCurso.
   */
  public static class WSPreinscripcionCurso {
    public WSServicioOnline servicioOnline;
    public WSUsuarioOnline usuario;
    public Date fechaInscripcion;

    /**
     * Construye un nuevo objeto de la clase
     */
    public WSPreinscripcionCurso() {
      super();
    }
  }

  private WSMatriculaCurso crearWSMatriculaCurso(MatriculaCursoOnline m) {
    WSMatriculaCurso matricula = new WSMatriculaCurso();
    matricula.actividad = crearWSCurso(m.getCurso());
    matricula.admitido = m.isAdmitido();
    matricula.nota = m.getNota();
    matricula.fechaMatricula = m.getFechaMatricula();
    matricula.enListaEspera = m.isEnListaEspera();
    matricula.usuario = crearWSUsuario(m.getUsuario());
    return matricula;
  }

  private WSPreinscripcionCurso crearWSPreinscripcionCurso(PreinscripcionCursoOnline p) {
    WSPreinscripcionCurso preinscripcion = new WSPreinscripcionCurso();
    preinscripcion.servicioOnline = crearWSServicio(p.getServicioCurso());
    preinscripcion.fechaInscripcion = p.getFecha();
    preinscripcion.usuario = crearWSUsuario(p.getUsuario());
    return preinscripcion;
  }

  /**
   * Clase WSRespuestaHistorial.
   */
  public static class WSRespuestaHistorial {
    public boolean correcto;
    public String mensaje;
    public List<WSMatriculaCurso> matriculasCurso;
    public List<WSPreinscripcionCurso> preinscripcionesCurso;

    /**
     * Construye un nuevo objeto de la clase
     */
    public WSRespuestaHistorial() {
      super();
      correcto = false;
      mensaje = "";
      matriculasCurso = new ArrayList<WSMatriculaCurso>();
      preinscripcionesCurso = new ArrayList<WSPreinscripcionCurso>();
    }
  }

  /**
   * Clase WSRespuestaPreRegistro.
   */
  public static class WSRespuestaPreRegistro {
    public boolean correcto;
    public String mensaje;
    public WSUsuarioOnline usuario;

    /**
     * Construye un nuevo objeto de la clase
     */
    public WSRespuestaPreRegistro() {
      super();
      correcto = false;
      mensaje = "";
      usuario = null;
    }
  }

  /**
   * Clase WSRespuestaGetCurso.
   */
  public static class WSRespuestaGetCurso {
    public boolean correcto;
    public String mensaje;
    public WSCursoOnline curso;

    /**
     * Construye un nuevo objeto de la clase
     */
    public WSRespuestaGetCurso() {
      super();
      correcto = false;
      mensaje = "";
      curso = null;
    }
  }

  /**
   * Clase WSRespuestaMatriculacion.
   */
  public static class WSRespuestaMatriculacion {
    public boolean correcto;
    public String mensaje;

    public boolean inscrito;
    public boolean enListaEspera;
    public boolean enListaInteresados;

    /**
     * Construye un nuevo objeto de la clase
     */
    public WSRespuestaMatriculacion() {
      super();
      correcto = false;
      mensaje = "";
      inscrito = false;
      enListaEspera = false;
      enListaInteresados = false;
    }
  }

  /**
   * Clase WSRespuestaDesMatriculacion.
   */
  public static class WSRespuestaDesMatriculacion {
    public boolean correcto;
    public String mensaje;

    /**
     * Construye un nuevo objeto de la clase
     */
    public WSRespuestaDesMatriculacion() {
      super();
      correcto = false;
      mensaje = "";
    }
  }

}
