package es.csa.fdt.cyldigital.integracion;

/*
 * #%L
 * Servicios Integrados para aplicaciones remotas - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.RandomAccessFile;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jws.WebMethod;
import javax.jws.WebService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import es.csa.fdt.cyldigital.core.entity.FilePanel;
import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.recursos.entity.EventoPanel;
import es.csa.fdt.cyldigital.recursos.entity.Panel;
import es.csa.fdt.cyldigital.recursos.service.RemoteNoticeService;
import es.csa.fdt.cyldigital.servicios.entity.Charla;
import es.csa.fdt.cyldigital.servicios.entity.Curso;
import es.csa.fdt.cyldigital.servicios.entity.Taller;

/**
 * Clase integracionPaneles.
 */
@Component("integracionPaneles")
@WebService
public class integracionPaneles {

  private static final Logger logger = Logger.getLogger(integracionPaneles.class.getName());

  private static final int MAX_NOTICE_BURST = 10;

  @Autowired
  private RemoteNoticeService service;

  /**
   * Clase WSImagenWrapper.
   */
  public static class WSImagenWrapper {
    public String titulo;
    public String fecha;
    public String hora;
    public List<WSImagen> wsImagenes;

    /**
     * Construye un nuevo objeto de la clase
     */
    public WSImagenWrapper() {
      super();
      titulo = "";
      fecha = "";
      hora = "";

    }
  }

  /**
   * Clase WSImagen.
   */
  public static class WSImagen {
    public String cuando;
    public String descripcion;
    public String pie;
    public String fichero;
    public Long idFichero;
    public Long idCentro;
    public Long idPanel;

    /**
     * Construye un nuevo objeto de la clase
     */
    public WSImagen() {
      super();
      cuando = "";
      descripcion = "";
      pie = "";
      fichero = "";
    }
  }

  /**
   * Clase WSFile.
   */
  public static class WSFile {
    public String resultado;
 /*   public List<Byte> resultadobytes;*/
    public byte[] lista;

    /**
     * Construye un nuevo objeto de la clase
     */
    public WSFile() {
      super();
      resultado = "";
      /*resultadobytes = new ArrayList<Byte>();*/

    }
  }

  /**
   * Clase WSVideoWrapper.
   */
  public static class WSVideoWrapper {
    public String titulo;
    public String fecha;
    public String hora;
    public List<WSVideo> wsVideos;

    /**
     * Construye un nuevo objeto de la clase
     */
    public WSVideoWrapper() {
      super();
      titulo = "";
      fecha = "";
      hora = "";

    }
  }

  /**
   * Clase WSVideo.
   */
  public static class WSVideo {
    public String cuando;
    public String descripcion;
    public String pie;
    public String fichero;
    public Long idFichero;
    public Long idCentro;
    public Long idPanel;

    /**
     * Construye un nuevo objeto de la clase
     */
    public WSVideo() {
      super();
      cuando = "";
      descripcion = "";
      pie = "";
      fichero = "";
    }
  }

  /**
   * Clase WSEventoWrapper.
   */
  public static class WSEventoWrapper {
    public String titulo;
    public String fecha;
    public String hora;

    public List<WSEvento> wsEventos;

    /**
     * Construye un nuevo objeto de la clase
     */
    public WSEventoWrapper() {
      super();
      titulo = "";
      fecha = "";
      hora = "";

    }
  }

  /**
   * Clase WSEvento.
   */
  public static class WSEvento {
    public String cuando;
    public String descripcion;
    public Long idCentro;
    public Long idPanel;
    private Date fechaParaComparar;

    /**
     * Construye un nuevo objeto de la clase
     */
    public WSEvento() {
      super();
      cuando = "";
      descripcion = "";
    }
  }

  private WSImagenWrapper createWSImagenWrapper() {
    WSImagenWrapper i = new WSImagenWrapper();
    DateFormat df = new SimpleDateFormat("dd/MM/yyyy");
    DateFormat dh = new SimpleDateFormat("HH:mm");

    i.fecha = df.format(new Date());
    i.hora = dh.format(new Date());
    i.titulo = "Próximos eventos";
    i.wsImagenes = new ArrayList<WSImagen>();
    return i;
  }

  private WSImagen createWSImagen(EventoPanel e, Panel p) {
    WSImagen i = new WSImagen();
    DateFormat df = new SimpleDateFormat("dd/MM/yyyy");

    i.cuando = df.format(new Date());
    i.descripcion = e.getDescripcion();
    i.pie = e.getPie();
    i.fichero = e.getFilePanel().getFile().getNombre();
    i.idFichero = e.getFilePanel().getId();
    i.idCentro = p.getCentro().getId();
    i.idPanel = p.getId();
    return i;
  }

  private WSVideoWrapper createWSVideoWrapper() {
    WSVideoWrapper i = new WSVideoWrapper();
    DateFormat df = new SimpleDateFormat("dd/MM/yyyy");
    DateFormat dh = new SimpleDateFormat("HH:mm");

    i.fecha = df.format(new Date());
    i.hora = dh.format(new Date());
    i.titulo = "Vídeos";
    i.wsVideos = new ArrayList<WSVideo>();
    return i;
  }

  private WSVideo createWSVideo(EventoPanel e, Panel p) {
    WSVideo i = new WSVideo();
    DateFormat df = new SimpleDateFormat("dd/MM/yyyy");

    i.cuando = df.format(new Date());
    i.descripcion = e.getDescripcion();
    i.pie = e.getPie();
    i.fichero = e.getFilePanel().getFile().getNombre();
    i.idFichero = e.getFilePanel().getId();
    i.idCentro = p.getCentro().getId();
    i.idPanel = p.getId();
    return i;
  }

  private WSEventoWrapper createWSEventoWrapper() {
    WSEventoWrapper e = new WSEventoWrapper();
    DateFormat df = new SimpleDateFormat("dd/MM/yyyy");
    DateFormat dh = new SimpleDateFormat("HH:mm");

    e.fecha = df.format(new Date());
    e.hora = dh.format(new Date());
    e.titulo = "Programación";
    e.wsEventos = new ArrayList<WSEvento>();
    return e;
  }

  private WSEvento createWSEvento(String descripcion, Date fecha, Panel p) {
    DateFormat df = new SimpleDateFormat("dd/MM/yyyy");
    WSEvento ew = new WSEvento();

    ew.descripcion = descripcion;
    ew.cuando = df.format(fecha);
    ew.idCentro = p.getCentro().getId();
    ew.idPanel = p.getId();
    ew.fechaParaComparar = fecha;
    return ew;
  }

  /**
   * Lista de los próximos eventos.
   * 
   * @return the wS imagen wrapper
   */
  @WebMethod
  @Transactional
  public WSImagenWrapper XMLObtenerImagenes() {
    List<EventoPanel> eventos = new ArrayList<EventoPanel>();
    WSImagenWrapper iw = new WSImagenWrapper();
    WSImagen i = new WSImagen();

    DateFormat df = new SimpleDateFormat("yyyy-MM-dd 00:00:00");
    DateFormat dh = new SimpleDateFormat("HH:mm");

    try {
      eventos = service.findByQuery(EventoPanel.class, "select e from " + EventoPanel.class.getName()
          + " e where fechaInicio<='" + df.format(new Date()) + "' and fechaFin>='" + df.format(new Date())
          + "' and horaInicio<='" + dh.format(new Date()) + "' and horaFin>='" + dh.format(new Date())
          + "' and tipoContenido=0");

      iw = createWSImagenWrapper(); // inicializa los parametros comunes, de Titulo, Fecha y Hora.
      for (EventoPanel e : eventos) {
        // cada evento puede estar programado para varios paneles
        for (Panel p : e.getPaneles()) {
          i = createWSImagen(e, p);
          iw.wsImagenes.add(i);
        }

      }

    } catch (RuntimeException e) {
      logger.log(Level.SEVERE, "Error al devolver la imagen a mostrar: " + e.getLocalizedMessage());
      e.printStackTrace();
    }

    return iw;
  }

  /**
   * Lista de los próximos vídeos.
   * 
   * @return the wS video wrapper
   */
  @WebMethod
  @Transactional
  public WSVideoWrapper XMLObtenerVideos() {
    List<EventoPanel> eventos = new ArrayList<EventoPanel>();
    WSVideoWrapper iw = new WSVideoWrapper();
    WSVideo i = new WSVideo();

    DateFormat df = new SimpleDateFormat("yyyy-MM-dd 00:00:00");
    DateFormat dh = new SimpleDateFormat("HH:mm");

    try {
      eventos = service.findByQuery(EventoPanel.class, "select e from " + EventoPanel.class.getName()
          + " e where fechaInicio<='" + df.format(new Date()) + "' and fechaFin>='" + df.format(new Date())
          + "' and horaInicio<='" + dh.format(new Date()) + "' and horaFin>='" + dh.format(new Date())
          + "' and tipoContenido=1");

      iw = createWSVideoWrapper(); // inicializa los parametros comunes, de Titulo, Fecha y Hora.
      for (EventoPanel e : eventos) {
        for (Panel p : e.getPaneles()) {
          i = createWSVideo(e, p);
          iw.wsVideos.add(i);
        }
      }

    } catch (RuntimeException e) {
      logger.log(Level.SEVERE, "Error al devolver el vídeo a mostrar: " + e.getLocalizedMessage());
      e.printStackTrace();
    }

    return iw;
  }

  /**
   * Devuelve una imagen en binario.
   * 
   * @param id
   *           id
   * @return the wS file
   */
  @WebMethod
  @Transactional
  public WSFile PeticionImagen(String id) {
    WSFile wsf = new WSFile();
    File archivo = null;
    RandomAccessFile raf =null;
    
    Parameter p = getFilesPath();
    FilePanel f = getIdFilePanel(id);

    try {
      archivo = new File(p.getValor() + "/" + f.getFile().getNombre());
  
      raf = new RandomAccessFile(archivo, "r");
      byte[] loader = new byte[(int) raf.length()];
      while ((raf.read(loader)) > 0) {
        wsf.lista=loader;
      }
    } catch (Exception e) {
      e.printStackTrace();
    } finally {
      try {
        if (raf != null) {
          raf.close();
        }
      } catch (Exception e2) {
        e2.printStackTrace();
      }
    }
    return wsf;
  }

  /**
   * Devuelve un video en binario.
   * 
   * @param id
   *           id
   * @return the wS file
   */
  @WebMethod
  @Transactional
  public WSFile PeticionVideo(String id) {
      return PeticionImagen(id);
      //los id de imagenes y videos son los mismos, o es una cosa o es otra. No existe idImagen=1,idVideo=1.
  }
  

  private FilePanel getIdFilePanel(String id) {
    Long idFilePanel = new Long(id);
    FilePanel f = new FilePanel();
    f = service.find(FilePanel.class, idFilePanel);
    if (f == null) {
      logger.log(Level.SEVERE, "Error,fichero no encontrado en la base de datos.");
    }
    return f;
  }

  private Parameter getFilesPath() {
    Parameter p = new Parameter();
    p = service.findOneWhere(Parameter.class, "nombre", "panel_application_path");
    if (p == null) {
      logger.log(Level.SEVERE, "Error, parametro no inicializado, falta panel_application_path");
    }
    return p;
  }

  /**
   * Lista de los próximos eventos.
   * 
   * @return the wS evento wrapper
   */
  @WebMethod
  @Transactional
  public WSEventoWrapper XMLObtenerEventos() {
    List<EventoPanel> eventos = new ArrayList<EventoPanel>();

    WSEventoWrapper ew = new WSEventoWrapper();
    WSEvento e = new WSEvento();

    List<Curso> cursos = new ArrayList<Curso>();
    List<Taller> talleres = new ArrayList<Taller>();
    List<Charla> charlas = new ArrayList<Charla>();

    DateFormat df = new SimpleDateFormat("yyyy-MM-dd 00:00:00");
    DateFormat dh = new SimpleDateFormat("HH:mm");

    try {
      eventos = service.findByQuery(EventoPanel.class, "select e from " + EventoPanel.class.getName()
          + " e where fechaInicio<='" + df.format(new Date()) + "' and fechaFin>='" + df.format(new Date())
          + "' and horaInicio<='" + dh.format(new Date()) + "' and horaFin>='" + dh.format(new Date())
          + "' and tipoContenido=4");

      ew = createWSEventoWrapper(); // inicializa los parametros comunes, de Titulo, Fecha y Hora.
      for (EventoPanel evento : eventos) {
        cursos = service.findByQuery(Curso.class, "select e from " + Curso.class.getName() + " e where fechaInicio >'"
            + df.format(new Date()) + "'");
        talleres = service.findByQuery(Taller.class, "select e from " + Taller.class.getName()
            + " e where fechaInicio >'" + df.format(new Date()) + "'");
        charlas = service.findByQuery(Charla.class, "select e from " + Charla.class.getName()
            + " e where fechaInicio >'" + df.format(new Date()) + "'");

        for (Panel p : evento.getPaneles()) {

          for (Curso c : cursos) {
            e = new WSEvento();
            e = createWSEvento("CURSO: " /*+ c.getApodo()*/, c.getFechaInicio(), p);
            ew.wsEventos.add(e);
          }

          for (Taller c : talleres) {
            e = new WSEvento();
            e = createWSEvento("TALLER: " + c.getApodo(), c.getFechaInicio(), p);
            ew.wsEventos.add(e);
          }

          for (Charla c : charlas) {
            e = new WSEvento();
            e = createWSEvento("CHARLA: " /*+ c.getNombre()*/, c.getFechaInicio(), p);
            ew.wsEventos.add(e);
          }
        }
      }

      Collections.sort(ew.wsEventos, new DateComparator());

    } catch (RuntimeException ex) {
      logger.log(Level.SEVERE, "Error al devolver los eventos a mostrar: " + ex.getLocalizedMessage());
      ex.printStackTrace();
    }
    return ew;
  }

  /**
   * Lista de los próximos eventos.
   * 
   * @return the wS evento wrapper
   */
  @WebMethod
  @Transactional
  public WSEventoWrapper XMLObtenerEventosDelDia() {
    List<EventoPanel> eventos = new ArrayList<EventoPanel>();

    WSEventoWrapper ew = new WSEventoWrapper();
    WSEvento e = new WSEvento();

    List<Curso> cursos = new ArrayList<Curso>();
    List<Taller> talleres = new ArrayList<Taller>();
    List<Charla> charlas = new ArrayList<Charla>();

    DateFormat df = new SimpleDateFormat("yyyy-MM-dd");
    DateFormat dh = new SimpleDateFormat("HH:mm");

    try {

      eventos = service.findByQuery(EventoPanel.class, "select e from " + EventoPanel.class.getName()
          + " e where fechaInicio<='" + df.format(new Date()) + "' and fechaFin>='" + df.format(new Date())
          + "' and horaInicio<='" + dh.format(new Date()) + "' and horaFin>='" + dh.format(new Date())
          + "' and tipoContenido=5");

      ew = createWSEventoWrapper(); // inicializa los parametros comunes, de Titulo, Fecha y Hora.
      for (EventoPanel evento : eventos) {
        cursos = service.findByQuery(Curso.class, "select e from " + Curso.class.getName() + " e where fechaInicio >'"
            + df.format(new Date()) + "'  and fechaFin<='" + df.format(new Date()) + "'");
        talleres = service.findByQuery(Taller.class, "select e from " + Taller.class.getName()
            + " e where fechaInicio >'" + df.format(new Date()) + "' and fechaFin<='" + df.format(new Date()) + "'");
        charlas = service.findByQuery(Charla.class, "select e from " + Charla.class.getName()
            + " e where fechaInicio ='" + df.format(new Date()) + "'");

        for (Panel p : evento.getPaneles()) {

          for (Curso c : cursos) {
            e = new WSEvento();
            e = createWSEvento("CURSO: " /*+ c.getApodo()*/, c.getFechaInicio(), p);
            ew.wsEventos.add(e);
          }

          for (Taller c : talleres) {
            e = new WSEvento();
            e = createWSEvento("TALLER: " + c.getApodo(), c.getFechaInicio(), p);
            ew.wsEventos.add(e);
          }

          for (Charla c : charlas) {
            e = new WSEvento();
            e = createWSEvento("CHARLA: " /*+ c.getNombre()*/, c.getFechaInicio(), p);
            ew.wsEventos.add(e);
          }
        }
      }

      Collections.sort(ew.wsEventos, new DateComparator());

    } catch (RuntimeException ex) {
      logger.log(Level.SEVERE, "Error al devolver los eventos a mostrar: " + ex.getLocalizedMessage());
      ex.printStackTrace();
    }
    return ew;
  }

  /**
   * Clase DateComparator.
   */
  public class DateComparator implements Comparator {

    /* (non-Javadoc)
     * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
     */
    public int compare(Object o1, Object o2) {
      WSEvento u1 = (WSEvento) o1;
      WSEvento u2 = (WSEvento) o2;
      return u1.fechaParaComparar.compareTo(u2.fechaParaComparar);
    }

    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object o) {
      return this == o;
    }
  }
}
