package es.csa.fdt.cyldigital.integracion;

/*
 * #%L
 * Servicios Integrados para aplicaciones remotas - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.StringTokenizer;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jws.WebMethod;
import javax.jws.WebService;
import javax.persistence.Query;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import es.csa.fdt.cyldigital.calendario.entity.UserActivityCalendarItem;
import es.csa.fdt.cyldigital.centros.entity.Aula;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.recursos.entity.CategoriaRecurso;
import es.csa.fdt.cyldigital.recursos.entity.Ordenador;
import es.csa.fdt.cyldigital.recursos.entity.RemoteNotice;
import es.csa.fdt.cyldigital.recursos.entity.Ordenador.EstadoOrdenador;
import es.csa.fdt.cyldigital.recursos.entity.Recurso.EstadoRecurso;
import es.csa.fdt.cyldigital.recursos.entity.RemoteNotice.NotificationType;
import es.csa.fdt.cyldigital.recursos.entity.RemoteNotice.Status;
import es.csa.fdt.cyldigital.recursos.service.RemoteNoticeService;
import es.csa.fdt.cyldigital.servicios.entity.Navegacion;
import es.csa.fdt.cyldigital.servicios.entity.ObservacionActividad;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;

/**
 * Clase remoteControl.
 */
@Component("remoteControl")
@WebService
public class remoteControl {

  private static final Logger logger = Logger.getLogger(WSAuthenticationInInterceptor.class.getName());

  private static final int MAX_NOTICE_BURST = 10;

  @Autowired
  private RemoteNoticeService service;

  /**
   * Devuelve pending notifications.
   * 
   * @param centerId
   *           center id
   * @return the pending notifications
   */
  @WebMethod
  @Transactional
  public List<RemoteNotice> getPendingNotifications(long centerId) {
    List<RemoteNotice> notices = new ArrayList<RemoteNotice>();
    try {
      service.setMessagesEnabled(false);
      if (service != null) {
        HashMap<String, Object> filtros = new HashMap<String, Object>();
        filtros.put("serverId", centerId);
        filtros.put("status", RemoteNotice.Status.PENDING);
        List<RemoteNotice> aux = new ArrayList<RemoteNotice>();
        aux = service.findWhere(RemoteNotice.class, filtros);
        for (int j = 0; j < aux.size() && j < MAX_NOTICE_BURST; j++) {
          aux.get(j).setStatus(Status.IN_PROGRESS);
          notices.add(aux.get(j));
        }
        service.persistAll(notices);
      }
    } catch (RuntimeException e) {
      logger.log(Level.SEVERE, "Error al devoler las notificaciones pendientes: " + e.getLocalizedMessage());
      e.printStackTrace();
    }
    return notices;
  }

  /**
   * Notify current alive.
   * 
   * @param notifications
   *           notifications
   */
  @WebMethod
  @Transactional
  public void notifyCurrentAlive(List<RemoteNotice> notifications) {
    try {
      List<String> macsAlive = new ArrayList<String>();
      List<Ordenador> ordenadores = service.findAll(Ordenador.class);
      Long idCentro = -1L;
      if (notifications != null) {
        for (RemoteNotice r : notifications) {
          macsAlive.add(r.getMacAddress());
          idCentro = r.getServerId();
          for (Ordenador ordenador : ordenadores) {
            if (ordenador.getCentro().getId() == r.getServerId()) {
              if (r.getMacAddress().equalsIgnoreCase(ordenador.getMac())
                  && ordenador.getEstadoOrdenador().equals(EstadoOrdenador.APAGADO)) {
                ordenador.setEstadoOrdenador(EstadoOrdenador.ENCENDIDO);
                if ((ordenador.getFechaInicioSesion() != null || ordenador.getUsuarioConectado() != null)
                    && getUsuarioFromRemoteNotification(r).isEmpty()) {
                  if (ordenador.getUsuarioConectado() != null) {
                    registrarFinSesion(ordenador.getUsuarioConectado(), ordenador);
                    ordenador.setUsuarioConectado(null);
                  }
                  ordenador.setFechaInicioSesion(null);
                }
                cambioIpAula(ordenador, r);
                service.persist(ordenador);
              }
            }
          }
        }
        for (Ordenador ordenador : ordenadores) {
          if (ordenador.getCentro().getId() == idCentro) {
            if (ordenador.getEstadoOrdenador().equals(EstadoOrdenador.ENCENDIDO)
                && !macsAlive.contains(ordenador.getMac())) {
              ordenador.setEstadoOrdenador(EstadoOrdenador.APAGADO);
              if (ordenador.getUsuarioConectado() != null) {
                registrarFinSesion(ordenador.getUsuarioConectado(), ordenador);
                ordenador.setUsuarioConectado(null);
              }
              ordenador.setFechaInicioSesion(null);
              service.persist(ordenador);
            }
          }
        }
      }
    } catch (RuntimeException e) {
      logger.log(Level.SEVERE, "Error al notificar los ordenadores vivos: " + e.getLocalizedMessage());
      e.printStackTrace();
    }
  }

  private String getUsuarioFromRemoteNotification(RemoteNotice r) {
    String username = "";
    if (r.getMessage() != null && r.getMessage() != "") {
      if (r.getMessage().contains("user=") && r.getMessage().contains(";")) {
        username = r.getMessage().replace("user=", "").replace(";", "");
      }
    }
    return username;
  }

  private void usuario(RemoteNotice remoteNotice, Ordenador ordenador) {
    Usuario u = null;
    switch (remoteNotice.getNotificationType()) {
      case CLIENT_LOGIN:
      case CLIENT_STARTED:
        u = getUsuarioFromName(remoteNotice.getUserName());
        if (ordenador.getUsuarioConectado() != null) {
          registrarFinSesion(ordenador.getUsuarioConectado(), ordenador);
        }
        if (u != null) {
          registrarInicioSesion(u, ordenador);
        }
        ordenador.setUsuarioConectado(u);
        service.persist(ordenador);
        break;
      case CLIENT_LOGOUT:
      case CLIENT_STOPED:
        u = getUsuarioFromName(remoteNotice.getUserName());
        if (ordenador.getUsuarioConectado() != null) {
          registrarFinSesion(ordenador.getUsuarioConectado(), ordenador);
        }
        ordenador.setUsuarioConectado(u);
        service.persist(ordenador);
        break;
      default:
        break;
    }
  }

  /**
   * Notify changes.
   * 
   * @param notifications
   *           notifications
   */
  @WebMethod
  @Transactional
  public void notifyChanges(List<RemoteNotice> notifications) {
    try {
      service.setMessagesEnabled(false);
      if (notifications != null) {
        for (RemoteNotice remoteNotice : notifications) {
          logger.log(Level.INFO, "Notificacion recibida: " + remoteNotice.getNotificationType() + " en "
              + remoteNotice.getIpAddress() + " (" + remoteNotice.getMacAddress() + ")");
          if (remoteNotice.getId() > 0) {
            RemoteNotice r = service.find(remoteNotice.getId());
            if (r != null) {
              r.setStatus(remoteNotice.getStatus());
              remoteNotice = r;
            }
          }
          service.persist(remoteNotice);
          switch (remoteNotice.getNotificationType()) {
            case DO_START:
              if (!remoteNotice.getStatus().equals(Status.ERROR)) logger
                  .log(Level.INFO, "Orden de encendido ejecutada");
              else logger.log(Level.SEVERE, "Error al ejecutar orden de encendido");
              break;
            case DO_LOGOUT:
              if (!remoteNotice.getStatus().equals(Status.ERROR)) logger.log(Level.INFO,
                  "Orden de cierre de sesión ejecutada");
              else logger.log(Level.SEVERE, "Error al ejecutar orden de cierre de session");
              break;
            case DO_SHOUTDOWN:
              if (!remoteNotice.getStatus().equals(Status.ERROR))
                logger.log(Level.INFO, "Orden de apagado del ordenador ejecutada");
              break;
            case DO_SHOW_MSG:
              logger.log(Level.INFO, "Mensaje entregado");
              break;
            case CLIENT_STARTED:
              logger.log(Level.INFO, "Ordenador encendido");

              registrarOrdenadorEncendido(remoteNotice);
              break;
            case CLIENT_STOPED:
              logger.log(Level.INFO, "Ordenador apagado");
              registrarOrdenadorApagado(remoteNotice);
              break;
            case CLIENT_LOGIN:
              registrarClientLogin(remoteNotice);
              logger.log(Level.INFO, "El usuario a iniciado la sesion");
              break;
            case CLIENT_LOGOUT:
              registrarClientLogout(remoteNotice);
              logger.log(Level.INFO, "El usuario a cerrado la sesion");
              break;
            default:
              break;
          }
        }
      }
    } catch (RuntimeException e) {
      logger.log(Level.SEVERE, "Error al notificar mensajes a la web: " + e.getLocalizedMessage());
      e.printStackTrace();
    }
  }

  private void registrarClientLogin(RemoteNotice remoteNotice) {
    Ordenador ordenador = registrarActualizarOrdenador(remoteNotice);
    ordenador.setEstadoOrdenador(EstadoOrdenador.ENCENDIDO);
    ordenador.setFechaInicioSesion(new Date());
    registrarCambioUsuario(remoteNotice, ordenador);
  }

  private void registrarClientLogout(RemoteNotice remoteNotice) {
    Ordenador ordenador = registrarActualizarOrdenador(remoteNotice);
    ordenador.setEstadoOrdenador(EstadoOrdenador.ENCENDIDO);
    remoteNotice.setUserName(null);
    ordenador.setFechaInicioSesion(null);
    registrarCambioUsuario(remoteNotice, ordenador);
  }

  @Transactional
  private void registrarOrdenadorApagado(RemoteNotice remoteNotice) {
    Ordenador ordenador = registrarActualizarOrdenador(remoteNotice);
    ordenador.setEstadoOrdenador(EstadoOrdenador.APAGADO);
    remoteNotice.setUserName(null);
    ordenador.setFechaInicioSesion(null);
    registrarCambioUsuario(remoteNotice, ordenador);
  }

  @Transactional
  private void registrarOrdenadorEncendido(RemoteNotice remoteNotice) {
    Ordenador ordenador = registrarActualizarOrdenador(remoteNotice);
    ordenador.setEstadoOrdenador(EstadoOrdenador.ENCENDIDO);
    ordenador.setFechaInicioSesion(null);
    registrarCambioUsuario(remoteNotice, ordenador);
  }

  private Ordenador registrarActualizarOrdenador(RemoteNotice remoteNotice) {
    List<Ordenador> o = null;
    Ordenador ordenador;
    try {
      o = service.findWhere(Ordenador.class, "mac", remoteNotice.getMacAddress());
    } catch (RuntimeException e) {}
    if (o != null && !o.isEmpty()) {
      ordenador = o.get(0);
      if (ordenador.getIp() == null || ordenador.getIp().isEmpty()) ordenador.setIp(remoteNotice.getIpAddress());
      if (ordenador.getMac() == null || ordenador.getMac().isEmpty()) ordenador.setMac(remoteNotice.getMacAddress());
      if (!getDetail(remoteNotice, "hostname").isEmpty() && ordenador.getNombre().toLowerCase().contains("ord-"))
        ordenador.setNombre(getDetail(remoteNotice, "hostname"));
      if (!getDetail(remoteNotice, "os").isEmpty() && (ordenador.getSo() == null || ordenador.getSo().isEmpty()))
        ordenador.setSo(getDetail(remoteNotice, "os"));
    } else {
      ordenador = new Ordenador();
      ordenador.setCentro(service.find(Centro.class, remoteNotice.getServerId()));
      ordenador.setCategoria(service.find(CategoriaRecurso.class, 1L));
      ordenador.setNombre((!getDetail(remoteNotice, "hostname").isEmpty()) ? getDetail(remoteNotice, "hostname")
          : "Ord-" + (new Date()).getTime());
      ordenador.setSo((!getDetail(remoteNotice, "os").isEmpty()) ? getDetail(remoteNotice, "os") : "Desconocido");
      ordenador.setEstado(EstadoRecurso.ACTIVO);
      ordenador.setObservaciones("Ordenador detectado");

      if (remoteNotice.getIpAddress() != null && !remoteNotice.getIpAddress().isEmpty())
        ordenador.setIp(remoteNotice.getIpAddress());

      if (remoteNotice.getMacAddress() != null && !remoteNotice.getMacAddress().isEmpty())
        ordenador.setMac(remoteNotice.getMacAddress());
    }
    // Comprobación de la ip
    cambioIpAula(ordenador, remoteNotice);
    service.persist(ordenador);
    return ordenador;
  }

  private void cambioIpAula(Ordenador ordenador, RemoteNotice remoteNotice) {
    if (!ordenador.getIp().equalsIgnoreCase(remoteNotice.getIpAddress())
        || (ordenador.getAula() == null)
        || (ordenador.getAula() != null && ordenador.getAula().getIp() != null
            && !ordenador.getAula().getIp().isEmpty() && !isIpInAula(ordenador.getIp(), ordenador.getAula()))) {

      ordenador.setIp(remoteNotice.getIpAddress());
      List<Aula> aulas = (List<Aula>) service.findAll(Aula.class);
      List<Aula> aulasAux = new ArrayList<Aula>();
      for (Aula aula : aulas) {
        if (aula.getCentro().getId() == remoteNotice.getServerId()) aulasAux.add(aula);
      }
      Aula aula = getAulaForIp(aulasAux, remoteNotice.getIpAddress());
      if (aula != null) ordenador.setAula(aula);
    }
  }

  private String getDetail(RemoteNotice r, String key) {
    String result = "";
    String cadena = key + "=";
    if (r.getNotificationType().equals(NotificationType.CLIENT_STARTED)) {
      if (r.getMessage() != null && !r.getMessage().trim().isEmpty()) {
        String[] datos = r.getMessage().split(";");
        for (String dato : datos) {
          if (dato.contains(cadena)) { return dato.replace(cadena, ""); }
        }
      }
    }
    return result;
  }

  private void registrarCambioUsuario(RemoteNotice remoteNotice, Ordenador ordenador) {
    Usuario u = null;
    switch (remoteNotice.getNotificationType()) {
      case CLIENT_LOGIN:
      case CLIENT_STARTED:
        u = getUsuarioFromName(remoteNotice.getUserName());
        if (ordenador.getUsuarioConectado() != null) {
          registrarFinSesion(ordenador.getUsuarioConectado(), ordenador);
        }
        if (u != null) {
          registrarInicioSesion(u, ordenador);
        }
        ordenador.setUsuarioConectado(u);
        service.persist(ordenador);
        break;
      case CLIENT_LOGOUT:
      case CLIENT_STOPED:
        u = getUsuarioFromName(remoteNotice.getUserName());
        if (ordenador.getUsuarioConectado() != null) {
          registrarFinSesion(ordenador.getUsuarioConectado(), ordenador);
        }
        ordenador.setUsuarioConectado(u);
        service.persist(ordenador);
        break;
      default:
        break;
    }
  }

  private void registrarInicioSesion(Usuario u, Ordenador o) {
    UserActivityCalendarItem calendarItem = new UserActivityCalendarItem();
    calendarItem.setUsuario(u);
    calendarItem.setStart(new Date());
    calendarItem.setOrdenador(o);
    calendarItem.setCentro(o.getCentro());
    calendarItem.setAula(o.getAula());
    service.persist(calendarItem);

    try {
      if (o.isNavegacionControlada()) {
        String consulta = "SELECT e FROM " + Navegacion.class.getName()
            + " e WHERE fechaInicio  <= ? and fechaFin >= ? and horaInicio  <= ? and horaFin >= ? and cerrada = ? "
            + " and ordenador = ? and usuario = ? ";
        Query query = service.getEntityManager().createQuery(consulta);
        Date now = new Date();
        query.setParameter(1, now);
        query.setParameter(2, now);
        query.setParameter(3, now);
        query.setParameter(4, now);
        query.setParameter(5, false);
        query.setParameter(6, o);
        query.setParameter(7, u);

        List<Navegacion> navegaciones = query.getResultList();
        if (navegaciones == null || navegaciones.size() == 0) {
          RemoteNotice r = new RemoteNotice();
          r.setDesactivado(false);
          r.setEntero(15);
          r.setIpAddress(o.getIp());
          r.setMacAddress(o.getMac());
          r.setNotificationType(NotificationType.DO_LOGOUT);
          r.setMessage("Es necesario pedir permiso en información para usar este equipo. Su sesión va a ser cerrada.");
          r.setServerId(o.getCentro().getId());
          r.setStatus(Status.PENDING);
          service.persist(r);
        }
      }
    } catch (RuntimeException e) {
      logger.log(Level.SEVERE, "Error al controlar la sesión de un usuario: " + e.getLocalizedMessage());
    }

  }

  @SuppressWarnings("unchecked")
  private void registrarFinSesion(Usuario u, Ordenador o) {
    logger.info("Registrando salida de usuario" + (u != null ? u.getNombreCompleto() : "no ususario"));

    String consulta = "SELECT e FROM " + UserActivityCalendarItem.class.getName() + " e " + "WHERE start = ("
        + "SELECT MAX(start) FROM " + UserActivityCalendarItem.class.getName()
        + " ee WHERE ordenador = :valor1) and ordenador = :valor2 ";

    Query query = service.getEntityManager().createQuery(consulta);
    query.setParameter("valor1", o);
    query.setParameter("valor2", o);
    List<UserActivityCalendarItem> calendarItems = (List<UserActivityCalendarItem>) query.getResultList();
    if (!calendarItems.isEmpty() && calendarItems.size() == 1) {
      UserActivityCalendarItem calendarItem = calendarItems.get(0);
      calendarItem.setEnd(new Date());
      service.persist(calendarItem);
      logger.info("Salida de usuario registrada");

      String consulta2 = "SELECT e FROM " + Navegacion.class.getName() + " e " + "WHERE  fechaInicio = ("
          + "SELECT MAX(fechaInicio) FROM " + Navegacion.class.getName()
          + " ee WHERE usuario = :valor1 and ordenador = :valor2 ) and usuario = :valor3 and ordenador = :valor4 ";

      Query query2 = service.getEntityManager().createQuery(consulta2);
      query2.setParameter("valor1", u);
      query2.setParameter("valor2", o);
      query2.setParameter("valor3", u);
      query2.setParameter("valor4", o);
      List<Navegacion> navegaciones = (List<Navegacion>) query2.getResultList();
      if (!navegaciones.isEmpty() && navegaciones.size() == 1) {
        Navegacion navegacion = navegaciones.get(0);
        ObservacionActividad obs = new ObservacionActividad();
        obs.setObservacion("Cierre de navegacion automático. Tarjeta desconectada.");
        obs.setFecha(new Date());
        obs.setUsuario(u);
        obs.setActividad(navegacion);
        navegacion.getObservacionesActividad().add(obs);
        navegacion.setCerrada(true);
        service.persist(navegacion);
      }
    }
  }

  private Usuario getUsuarioFromName(String nombre) {
    Usuario u = null;
    if (nombre != null && !nombre.trim().isEmpty()) {
      List<Empleado> empleados = service.findWhere(Empleado.class, "username", nombre);
      if (!empleados.isEmpty() && empleados.size() == 1) {
        u = empleados.get(0);
      } else {
        List<UsuarioCyL> usuarios = service.findWhere(UsuarioCyL.class, "username", nombre);
        if (!usuarios.isEmpty() && usuarios.size() == 1) u = usuarios.get(0);
      }
    }
    return u;
  }

  private Aula getAulaForIp(List<Aula> aulas, String ip) {
    for (Aula aula : aulas) {
      if (aula.getIp() != null && !aula.getIp().isEmpty() && isIpInAula(ip, aula)) return aula;
    }
    return null;
  }

  private boolean isIpInAula(String ip, Aula aula) {
    int[] ipAux = splitIp(ip);
    int[] ipAulaAux = splitIp(aula.getIp());
    int[] maskAux = splitIp(aula.getMask());
    for (int i = 0; i < 4; i++) {
      if ((ipAux[i] & maskAux[i]) != ipAulaAux[i]) { return false; }
    }
    return true;
  }

  private static int[] splitIp(String ip) {
    int[] result = new int[4];
    StringTokenizer st = new StringTokenizer(ip, ".");
    for (int i = 0; i < 4; i++) {
      result[i] = Integer.parseInt(st.nextToken());
    }
    return result;
  }

}
