package cn.bluejoe.elfinder.controller.executors;

/*
 * #%L
 * Librerías del núcleo para Web - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */

import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.fileupload.FileItemStream;
import org.apache.commons.io.IOUtils;
import org.json.JSONObject;

import cn.bluejoe.elfinder.controller.executor.AbstractJsonCommandExecutor;
import cn.bluejoe.elfinder.controller.executor.CommandExecutor;
import cn.bluejoe.elfinder.controller.executor.FsItemEx;
import cn.bluejoe.elfinder.service.FsService;

public class UploadCommandExecutor extends AbstractJsonCommandExecutor implements CommandExecutor
{
	@Override
	public void execute(FsService fsService, HttpServletRequest request, ServletContext servletContext, JSONObject json)
			throws Exception
	{
		String target = request.getParameter("target");
		List<FileItemStream> listFiles = (List<FileItemStream>) request.getAttribute(FileItemStream.class.getName());
		
		List<FsItemEx> added = new ArrayList<FsItemEx>();
		Map<String, String> errors = new HashMap<String, String>();
		
		//Extract mime-types
		List<String> mimes = new ArrayList<String>();
		for (int i = 0; request.getParameter("mimes[" + i + "]") != null; i++)
			mimes.add(request.getParameter("mimes[" + i + "]"));
		
		FsItemEx dir = super.findItem(fsService, target);
		for (FileItemStream fis : listFiles)
		{
			Boolean isUploadable = false;
			
			for (String mime : mimes) {
				if ((mime.contains("/") && fis.getContentType().equals(mime)) ||
					(!mime.contains("/") && fis.getContentType().startsWith(mime))) {
					isUploadable = true;
					break;
				}
			}
			
			if (isUploadable) {
				String fileName = fis.getName();
				
				FsItemEx newFile = new FsItemEx(dir, fileName);
				
				newFile.createFile();
				
				InputStream is = fis.openStream();
				OutputStream os = newFile.openOutputStream();
	
				IOUtils.copy(is, os);
				
				os.close();
				is.close();
	
				added.add(newFile);
			}
			else
				errors.put(fis.getName(), "El fichero no es de tipo imágen");
		}

		if (added.size() == listFiles.size())
			json.put("added", files2JsonArray(request, added));
		else if (added.size() > 0) {
			Map<String, FsItemEx> files = new LinkedHashMap<String, FsItemEx>();
			files.put(dir.getHash(), dir);
			addChildren(files, dir, mimes.toArray(new String[0]));
			
			String[] select = new String[added.size()];
			for (FsItemEx fi : added)
				select[added.indexOf(fi)] = fi.getHash();

			json.put("files", files2JsonArray(request, files.values()));
			json.put("cwd", getFsItemInfo(request, dir));
			json.put("options", getOptions(request, dir));
			json.put("select", select);
			json.put("error", "Algunos ficheros no fueron subidos.");
			json.put("errorData", errors);
		}
		else
			json.put("error", "Error. Los ficheros que intenta subir no son imágenes.");
	}
}
