package cn.bluejoe.elfinder.localfs;

/*
 * #%L
 * Librerías del núcleo para Web - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */

import hidden.org.codehaus.plexus.util.FileUtils;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;

import javax.imageio.ImageIO;

import org.springframework.beans.factory.annotation.Autowired;

import cn.bluejoe.elfinder.service.FsItem;
import cn.bluejoe.elfinder.service.FsVolume;
import cn.bluejoe.elfinder.util.MimeTypesUtils;

import es.csa.fdt.cyldigital.core.service.ParameterService;

public class LocalFsVolume implements FsVolume
{
	private String _name;

	private String _paramWithRootDir;
	
	private File _rootDir;
	
	@Autowired
	private ParameterService parameterService;

	private File asFile(FsItem fsi)
	{
		return ((LocalFsItem) fsi).getFile();
	}

	@Override
	public void createFile(FsItem fsi) throws IOException
	{
		asFile(fsi).createNewFile();
	}

	@Override
	public void createFolder(FsItem fsi) throws IOException
	{
		asFile(fsi).mkdirs();
	}

	@Override
	public void deleteFile(FsItem fsi) throws IOException
	{
		FileUtils.forceDelete(asFile(fsi));
	}

	@Override
	public void deleteFolder(FsItem fsi) throws IOException
	{
		FileUtils.forceDelete(asFile(fsi));
	}

	@Override
	public boolean exists(FsItem newFile)
	{
		return asFile(newFile).exists();
	}

	private LocalFsItem fromFile(File file)
	{
		return new LocalFsItem(this, file);
	}

	@Override
	public FsItem fromPath(String relativePath)
	{
		return fromFile(new File(getRootDir(), relativePath));
	}

	@Override
	public String getDimensions(FsItem fsi) throws IOException
	{
		if (getMimeType(fsi).startsWith("image")) {
			BufferedImage image = ImageIO.read(openInputStream(fsi));
			
			return String.valueOf(image.getWidth()) + 'x' + String.valueOf(image.getHeight());
		}
		else
			return null;
	}

	@Override
	public long getLastModified(FsItem fsi)
	{
		return asFile(fsi).lastModified();
	}

	@Override
	public String getMimeType(FsItem fsi)
	{
		File file = asFile(fsi);
		if (file.isDirectory())
			return "directory";

		String ext = FileUtils.getExtension(file.getName());
		if (ext != null && !ext.isEmpty())
		{
			String mimeType = MimeTypesUtils.getMimeType(ext);
			return mimeType == null ? MimeTypesUtils.UNKNOWN_MIME_TYPE : mimeType;
		}

		return MimeTypesUtils.UNKNOWN_MIME_TYPE;
	}

	public String getName()
	{
		return _name;
	}

	@Override
	public String getName(FsItem fsi)
	{
		return asFile(fsi).getName();
	}

	@Override
	public FsItem getParent(FsItem fsi)
	{
		return fromFile(asFile(fsi).getParentFile());
	}

	@Override
	public String getPath(FsItem fsi) throws IOException
	{
		String fullPath = asFile(fsi).getCanonicalPath();
		String rootPath = getRootDir().getCanonicalPath();
		String relativePath = fullPath.substring(rootPath.length());
		return relativePath.replace('\\', '/');
	}

	@Override
	public FsItem getRoot()
	{
		return fromFile(getRootDir());
	}

	public File getRootDir()
	{
		if (_rootDir == null)
			setRootDir(new File(parameterService.findOneWhere("nombre", _paramWithRootDir).getValor()));

		return _rootDir;
	}

	@Override
	public long getSize(FsItem fsi)
	{
		return asFile(fsi).length();
	}

	@Override
	public String getThumbnailFileName(FsItem fsi)
	{
		return null;
	}

	@Override
	public boolean hasChildFolder(FsItem fsi)
	{
		return asFile(fsi).isDirectory() && asFile(fsi).listFiles(new FileFilter()
		{

			@Override
			public boolean accept(File arg0)
			{
				return arg0.isDirectory();
			}
		}).length > 0;
	}

	@Override
	public boolean isFolder(FsItem fsi)
	{
		return asFile(fsi).isDirectory();
	}

	@Override
	public boolean isRoot(FsItem fsi)
	{
		return (getRootDir().getPath().compareTo(asFile(fsi).getPath()) == 0);
	}

	@Override
	public FsItem[] listChildren(FsItem fsi)
	{
		List<FsItem> list = new ArrayList<FsItem>();
		File[] cs = asFile(fsi).listFiles();
		if (cs == null)
		{
			return new FsItem[0];
		}

		for (File c : cs)
		{
			list.add(fromFile(c));
		}

		return list.toArray(new FsItem[0]);
	}

	@Override
	public InputStream openInputStream(FsItem fsi) throws IOException
	{
		return new FileInputStream(asFile(fsi));
	}

	@Override
	public OutputStream openOutputStream(FsItem fsi) throws IOException
	{
		return new FileOutputStream(asFile(fsi));
	}

	@Override
	public void rename(FsItem src, FsItem dst) throws IOException
	{
		File srcFile = asFile(src);
		
		if (srcFile.isDirectory())
			FileUtils.copyDirectoryStructure(srcFile, asFile(dst));
		else
			FileUtils.copyFile(srcFile, asFile(dst));
		
		FileUtils.forceDelete(srcFile);
	}

	public void setName(String name)
	{
		_name = name;
	}
	
	public void setParamWithRootDir(String paramWithRootDir) {
		_paramWithRootDir = paramWithRootDir;
	}

	private void setRootDir(File rootDir)
	{
		if (!rootDir.exists())
			rootDir.mkdirs();

		_rootDir = rootDir;
	}
}
