package es.csa.fdt.cyldigital.controller;

/*
 * #%L
 * Librerías del núcleo para Web - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.persistence.Query;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.io.IOUtils;
import org.richfaces.json.JSONArray;
import org.richfaces.json.JSONException;
import org.richfaces.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

import es.csa.commons.faces.util.PredefinedLocaleResolver;
import es.csa.fdt.cyldigital.calendario.entity.ActividadCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.AsistenciaCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.AusenciaCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.CalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.CentroCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.EventoPanelCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.FestivoCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.ReservaCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.UserActivityCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.VacacionesCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.ActividadCalendarItem.ActividadType;
import es.csa.fdt.cyldigital.calendario.entity.AsistenciaCalendarItem.Excepcion;
import es.csa.fdt.cyldigital.calendario.entity.FestivoCalendarItem.TipoFestivo;
import es.csa.fdt.cyldigital.calendario.entity.VacacionesCalendarItem.Estado;
import es.csa.fdt.cyldigital.calendario.service.CalendarItemService;
import es.csa.fdt.cyldigital.centros.entity.Aula;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.recursos.entity.EventoPanel;
import es.csa.fdt.cyldigital.recursos.entity.Panel;
import es.csa.fdt.cyldigital.recursos.entity.Recurso;
import es.csa.fdt.cyldigital.servicios.entity.Actividad;
import es.csa.fdt.cyldigital.servicios.entity.ActividadExterna;
import es.csa.fdt.cyldigital.servicios.entity.Asesoramiento;
import es.csa.fdt.cyldigital.servicios.entity.Charla;
import es.csa.fdt.cyldigital.servicios.entity.Curso;
import es.csa.fdt.cyldigital.servicios.entity.Taller;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;

/**
 * Clase CalendarControllerImpl.
 */
@Controller
public class CalendarControllerImpl {

  private static final long serialVersionUID = -4581725741365803463L;

  /**
   * Instancia del servicio genérico de calendario.
   */
  @Autowired
  private CalendarItemService calendarService;

  /**
   * Rango de meses que se desea ver en cada renderización del calendario.
   */
  private static final int MAX_MONTHS_WITH_ITEMS = 3;

  /**
   * Array de objetos JSON con los items a mostrar en el calendario
   */
  private JSONArray jsonA;

  /**
   * Array de objetos JSON con los festivos
   */
  private String festivoA;

  /**
   * Fecha actual de renderización del calendario
   */
  private Calendar fecha;

  /**
   * Constructor de la clase-controlador.
   */
  public CalendarControllerImpl() {}

  /**
   * Retorna la hora del servidor en un formato entendible por javascript.
   * 
   * @param response
   *           response
   * @return the server time
   */
  @RequestMapping("/getServerTime")
  public void getServerTime(HttpServletResponse response) {
    String result = "";
    Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    result += +cal.get(Calendar.YEAR);
    result += "," + cal.get(Calendar.MONTH);
    result += "," + cal.get(Calendar.DAY_OF_MONTH);
    result += "," + cal.get(Calendar.HOUR_OF_DAY);
    result += "," + cal.get(Calendar.MINUTE);
    result += "," + cal.get(Calendar.SECOND);

    try {
      IOUtils.copy(new ByteArrayInputStream(result.getBytes()), response.getOutputStream());
      response.getOutputStream().flush();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  /**
   * Url que publica los items del calendario del centro.
   * 
   * @param idCentro
   *          Centro del que se publica el calendario
   * @param vista
   *          Vista actual publicada
   * @param fechaString
   *          Fecha actual publicada
   * @param tipo
   *          Acción para ver eventos: previos, posteriores, actuales
   * @param response
   *           response
   * @return the centro items
   */
  @Transactional
  @RequestMapping("/getCentroCalendarItemsOld")
  public void getCentroItemsOld(@RequestParam("idcentro") String idCentro, @RequestParam("vista") String vista,
		  					 @RequestParam("fecha") String fechaString, @RequestParam("tipo") String tipo,
		  					 HttpServletResponse response) {
	  jsonA = new JSONArray();
	  festivoA = "";

	  calculateFecha(vista, fechaString.equals("null") ? null : fechaString, tipo);
	  findFestivoItems(Long.parseLong(idCentro));
	  findCentroItems(Long.parseLong(idCentro));

	  getActividadesFromCentro(idCentro, null);

	  try {
		  IOUtils.copy(new ByteArrayInputStream(jsonA.toString().getBytes()), response.getOutputStream());
		  response.getOutputStream().flush();
	  }
	  catch (IOException e) {
		  e.printStackTrace();
	  }
  }
  
  /**
   * Url que publica los items del calendario del centro.
   * 
   * @param idCentro
   *          Centro del que se publica el calendario
   * @param vista
   *          Vista actual publicada
   * @param fechaString
   *          Fecha actual publicada
   * @param tipo
   *          Acción para ver eventos: previos, posteriores, actuales
   * @param response
   *           response
   * @return the centro items
   */
  @Transactional
  @RequestMapping("/getCentroCalendarItems")
  public void getCentroItems(@RequestParam("idcentro") String idCentro, @RequestParam("vista") String vista,
		  					 @RequestParam("fecha") String fechaString, @RequestParam("tipo") String tipo,
		  					 HttpServletResponse response) {
	  jsonA = new JSONArray();
	  festivoA = "";

	  calculateFecha(vista, fechaString.equals("null") ? null : fechaString, tipo);
	  findFestivoItems(Long.parseLong(idCentro));
	  
	  List<CalendarItem> lcci = findCentroItems(Long.parseLong(idCentro));
	  List<CalendarItem> laci = getActividadesFromCentro(idCentro, null);
	  
	  setOpenClassroom(lcci, laci);

	  try {
		  IOUtils.copy(new ByteArrayInputStream(jsonA.toString().getBytes()), response.getOutputStream());
		  response.getOutputStream().flush();
	  }
	  catch (IOException e) {
		  e.printStackTrace();
	  }
  }

  /**
   * Devuelve aula items.
   * 
   * @param idAula
   *           id aula
   * @param idCentro
   *           id centro
   * @param vista
   *           vista
   * @param fechaString
   *           fecha string
   * @param tipo
   *           tipo
   * @param response
   *           response
   * @return the aula items
   */
  @Transactional
  @RequestMapping("/getAulaCalendarItems")
  public void getAulaItems(@RequestParam("idaula") String idAula, @RequestParam("idcentro") String idCentro,
		  				   @RequestParam("vista") String vista, @RequestParam("fecha") String fechaString,
		  				   @RequestParam("tipo") String tipo, HttpServletResponse response) {
    jsonA = new JSONArray();
    festivoA = "";

    calculateFecha(vista, fechaString.equals("null") ? null : fechaString, tipo);
    findFestivoItems(Long.parseLong(idCentro));

    getActividadesFromCentro(idCentro, idAula);

    try {
      IOUtils.copy(new ByteArrayInputStream(jsonA.toString().getBytes()), response.getOutputStream());
      response.getOutputStream().flush();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  @SuppressWarnings("unchecked")
  private void getActividadesFromCentroOld(String idCentro, String idAula) {
	  String consulta = "";
		
	  Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
	  Calendar cal2 = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
	  
	  cal.setTime(fecha.getTime());
	  cal.set(Calendar.DATE, cal.getActualMinimum(Calendar.DATE));
	  cal.add(Calendar.MONTH, -(MAX_MONTHS_WITH_ITEMS / 2));
	  
	  cal2.setTime(fecha.getTime());
	  cal2.set(Calendar.DATE, cal2.getActualMinimum(Calendar.DATE));
	  cal2.add(Calendar.MONTH, (MAX_MONTHS_WITH_ITEMS / 2));
	  cal2.set(Calendar.DATE, cal2.getActualMaximum(Calendar.DATE));

	  consulta = "SELECT e " + 
	  			 "FROM " + Curso.class.getName() + " e " +
	  			 "WHERE ((e.fechaInicio BETWEEN ? AND ?) OR " +
	  			 		"(e.fechaFin BETWEEN ? AND ?)) AND " +
	  			 	   "(e.centro.id = ?)";
		
	  if (idAula != null)
		  consulta += " and (e.aula.id = ?)";
		
	  Query query = calendarService.getEntityManager().createQuery(consulta);
		
	  query.setParameter(1, cal.getTime());
	  query.setParameter(2, cal2.getTime());
	  query.setParameter(3, cal.getTime());
	  query.setParameter(4, cal2.getTime());
	  query.setParameter(5, Long.parseLong(idCentro));
		
	  if (idAula != null)
		  query.setParameter(6, Long.parseLong(idAula));
		
	  List<Curso> actividades = (List<Curso>) query.getResultList();
		
	  for (Curso actividad : actividades)
		  findActividadItems(actividad.getId(), false);
		
	  consulta = "SELECT e " +
	  			 "FROM " + Taller.class.getName() + " e " + 
	  			 "WHERE ((e.fechaInicio BETWEEN ? AND ?) OR " +
	  			 		"(e.fechaFin BETWEEN ? AND ?)) AND " +
	  			 	   "(e.centro.id = ?)";
		
	  if (idAula != null)
		  consulta += " and (e.aula.id = ?)";
		
	  query = calendarService.getEntityManager().createQuery(consulta);
		
	  query.setParameter(1, cal.getTime());
	  query.setParameter(2, cal2.getTime());
	  query.setParameter(3, cal.getTime());
	  query.setParameter(4, cal2.getTime());
	  query.setParameter(5, Long.parseLong(idCentro));
		
	  if (idAula != null)
		  query.setParameter(6, Long.parseLong(idAula));
		
	  List<Taller> talleres = (List<Taller>) query.getResultList();
		
	  for (Taller actividad : talleres)
		  findActividadItems(actividad.getId(), false);
		
	  consulta = "SELECT e " +
	  			 "FROM " + Charla.class.getName() + " e " +
	  			 "WHERE (e.fechaInicio BETWEEN ? AND ?) AND " +
	  			 	   "(e.centro.id = ?)";
		
	  if (idAula != null)
		  consulta += " and (e.aula.id = ?)";
		
	  query = calendarService.getEntityManager().createQuery(consulta);
		
	  query.setParameter(1, cal.getTime());
	  query.setParameter(2, cal2.getTime());
	  query.setParameter(3, Long.parseLong(idCentro));
		
	  if (idAula != null)
		  query.setParameter(4, Long.parseLong(idAula));
		
	  List<Charla> charlas = (List<Charla>) query.getResultList();
		
	  for (Charla actividad : charlas)
		  findActividadItems(actividad.getId(), false);
		
	  consulta = "SELECT e " +
	  			 "FROM " + ActividadExterna.class.getName() + " e " +
	  			 "WHERE ((e.solicitada = 0) AND (e.aceptada = 1)) AND " +
	  			 	   "((e.fechaInicio BETWEEN ? AND ?) OR " +
	  			 		"(e.fechaFin BETWEEN ? AND ?)) AND " +
	  			 		"(e.centro.id = ?)";
		
	  if (idAula != null)
		  consulta += " and (e.aula.id = ?)";
		
	  query = calendarService.getEntityManager().createQuery(consulta);
		
	  query.setParameter(1, cal.getTime());
	  query.setParameter(2, cal2.getTime());
	  query.setParameter(3, cal.getTime());
	  query.setParameter(4, cal2.getTime());
	  query.setParameter(5, Long.parseLong(idCentro));
		
	  if (idAula != null)
		  query.setParameter(6, Long.parseLong(idAula));
		
	  List<ActividadExterna> externas = (List<ActividadExterna>) query.getResultList();
		
	  for (ActividadExterna externa : externas)
		  findActividadItems(externa.getId(), false);
  }
  
  @SuppressWarnings("unchecked")
  private List<CalendarItem> getActividadesFromCentro(String idCentro, String idAula) {
	  List<CalendarItem> returnItems = new ArrayList<CalendarItem>();
	  
	  String consulta = "";
		
	  Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
	  Calendar cal2 = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
	  
	  cal.setTime(fecha.getTime());
	  cal.set(Calendar.DATE, cal.getActualMinimum(Calendar.DATE));
	  cal.add(Calendar.MONTH, -(MAX_MONTHS_WITH_ITEMS / 2));
	  
	  cal2.setTime(fecha.getTime());
	  cal2.set(Calendar.DATE, cal2.getActualMinimum(Calendar.DATE));
	  cal2.add(Calendar.MONTH, (MAX_MONTHS_WITH_ITEMS / 2));
	  cal2.set(Calendar.DATE, cal2.getActualMaximum(Calendar.DATE));

	  //Cursos
	  consulta = "SELECT e " + 
	  			 "FROM " + Curso.class.getName() + " e " +
	  			 "WHERE ((e.fechaInicio BETWEEN ? AND ?) OR " +
	  			 		"(e.fechaFin BETWEEN ? AND ?)) AND " +
	  			 	   "(e.centro.id = ?)";
		
	  if (idAula != null)
		  consulta += " and (e.aula.id = ?)";
		
	  Query query = calendarService.getEntityManager().createQuery(consulta);
		
	  query.setParameter(1, cal.getTime());
	  query.setParameter(2, cal2.getTime());
	  query.setParameter(3, cal.getTime());
	  query.setParameter(4, cal2.getTime());
	  query.setParameter(5, Long.parseLong(idCentro));
		
	  if (idAula != null)
		  query.setParameter(6, Long.parseLong(idAula));
		
	  List<Curso> cursos = (List<Curso>) query.getResultList();
		
	  for (Curso curso : cursos)
		  returnItems.addAll(findActividadItems(curso.getId(), false));

	  //Talleres
	  consulta = "SELECT e " +
	  			 "FROM " + Taller.class.getName() + " e " + 
	  			 "WHERE ((e.fechaInicio BETWEEN ? AND ?) OR " +
	  			 		"(e.fechaFin BETWEEN ? AND ?)) AND " +
	  			 	   "(e.centro.id = ?)";
		
	  if (idAula != null)
		  consulta += " and (e.aula.id = ?)";
		
	  query = calendarService.getEntityManager().createQuery(consulta);
		
	  query.setParameter(1, cal.getTime());
	  query.setParameter(2, cal2.getTime());
	  query.setParameter(3, cal.getTime());
	  query.setParameter(4, cal2.getTime());
	  query.setParameter(5, Long.parseLong(idCentro));
		
	  if (idAula != null)
		  query.setParameter(6, Long.parseLong(idAula));
		
	  List<Taller> talleres = (List<Taller>) query.getResultList();
		
	  for (Taller taller : talleres)
		  returnItems.addAll(findActividadItems(taller.getId(), false));

	  //Charlas
	  consulta = "SELECT e " +
	  			 "FROM " + Charla.class.getName() + " e " +
	  			 "WHERE (e.fechaInicio BETWEEN ? AND ?) AND " +
	  			 	   "(e.centro.id = ?)";
		
	  if (idAula != null)
		  consulta += " and (e.aula.id = ?)";
		
	  query = calendarService.getEntityManager().createQuery(consulta);
		
	  query.setParameter(1, cal.getTime());
	  query.setParameter(2, cal2.getTime());
	  query.setParameter(3, Long.parseLong(idCentro));
		
	  if (idAula != null)
		  query.setParameter(4, Long.parseLong(idAula));
		
	  List<Charla> charlas = (List<Charla>) query.getResultList();
		
	  for (Charla charla : charlas)
		  returnItems.addAll(findActividadItems(charla.getId(), false));

	  //Actividades externas
	  consulta = "SELECT e " +
	  			 "FROM " + ActividadExterna.class.getName() + " e " +
	  			 "WHERE ((e.solicitada = 0) AND (e.aceptada = 1)) AND " +
	  			 	   "((e.fechaInicio BETWEEN ? AND ?) OR " +
	  			 		"(e.fechaFin BETWEEN ? AND ?)) AND " +
	  			 		"(e.centro.id = ?)";
		
	  if (idAula != null)
		  consulta += " and (e.aula.id = ?)";
		
	  query = calendarService.getEntityManager().createQuery(consulta);
		
	  query.setParameter(1, cal.getTime());
	  query.setParameter(2, cal2.getTime());
	  query.setParameter(3, cal.getTime());
	  query.setParameter(4, cal2.getTime());
	  query.setParameter(5, Long.parseLong(idCentro));
		
	  if (idAula != null)
		  query.setParameter(6, Long.parseLong(idAula));
		
	  List<ActividadExterna> externas = (List<ActividadExterna>) query.getResultList();
		
	  for (ActividadExterna externa : externas)
		  returnItems.addAll(findActividadItems(externa.getId(), false));
	  
	  return returnItems;
  }

  /**
   * Url que publica los items festivos del calendario.
   * 
   * @param idCentro
   *          Centro del que se aplican los festivos del calendario
   * @param vista
   *          Vista actual publicada
   * @param fechaString
   *          Fecha actual publicada
   * @param tipo
   *          Acción para ver eventos: previos, posteriores, actuales
   * @param response
   *           response
   * @return the festivo items
   */
  @RequestMapping("/getFestivoCalendarItems")
  public void getFestivoItems(@RequestParam("idcentro") String idCentro, @RequestParam("vista") String vista,
      @RequestParam("fecha") String fechaString, @RequestParam("tipo") String tipo, HttpServletResponse response) {
    jsonA = new JSONArray();
    festivoA = "";

    calculateFecha(vista, fechaString.equals("null") ? null : fechaString, tipo);
    findFestivoItems(Long.parseLong(idCentro));
    try {
      IOUtils.copy(new ByteArrayInputStream(jsonA.toString().getBytes()), response.getOutputStream());
      response.getOutputStream().flush();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  /**
   * Url que publica los items de ausencias del calendario de un empleado.
   * 
   * @param idEmpleado
   *           id empleado
   * @param idCentro
   *          Centro del que se publica el calendario
   * @param vista
   *          Vista actual publicada
   * @param fechaString
   *          Fecha actual publicada
   * @param tipo
   *          Acción para ver eventos: previos, posteriores, actuales
   * @param isEdition
   *           is edition
   * @param response
   *           response
   * @return the empleado items
   */

  @RequestMapping("/getEmpleadoCalendarItems")
  @Transactional(readOnly = true)
  public void getEmpleadoItems(@RequestParam("idempleado") String idEmpleado,
      @RequestParam("idcentro") String idCentro, @RequestParam("vista") String vista,
      @RequestParam("fecha") String fechaString, @RequestParam("tipo") String tipo,
      @RequestParam("isedition") boolean isEdition, HttpServletResponse response) {
    jsonA = new JSONArray();
    festivoA = "";

    calculateFecha(vista, fechaString, tipo);
    if (Long.valueOf(idEmpleado) > 0) {
      findAusenciaItems(Long.parseLong(idEmpleado), isEdition);
    }
    try {
      IOUtils.copy(new ByteArrayInputStream(jsonA.toString().getBytes()), response.getOutputStream());
      response.getOutputStream().flush();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  /**
   * Devuelve empleado items no edit.
   * 
   * @param idEmpleado
   *           id empleado
   * @param idCentro
   *           id centro
   * @param vista
   *           vista
   * @param fechaString
   *           fecha string
   * @param tipo
   *           tipo
   * @param isEdition
   *           is edition
   * @param response
   *           response
   * @return the empleado items no edit
   */
  @RequestMapping("/getEmpleadoCalendarItemsNoEdit")
  @Transactional(readOnly = true)
  public void getEmpleadoItemsNoEdit(@RequestParam("idempleado") String idEmpleado,
      @RequestParam("idcentro") String idCentro, @RequestParam("vista") String vista,
      @RequestParam("fecha") String fechaString, @RequestParam("tipo") String tipo,
      @RequestParam("isedition") boolean isEdition, HttpServletResponse response) {
    jsonA = new JSONArray();
    festivoA = "";

    calculateFecha(vista, fechaString, tipo);
    findFestivoItems(Long.valueOf(idCentro));
    if (Long.valueOf(idEmpleado) > 0) {
      findVacacionesItems(Long.parseLong(idEmpleado), isEdition);
      getActividadesFromEmpleado(idEmpleado, "0");
    }
    try {
      IOUtils.copy(new ByteArrayInputStream(jsonA.toString().getBytes()), response.getOutputStream());
      response.getOutputStream().flush();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  /**
   * Devuelve empleado items view.
   * 
   * @param idEmpleado
   *           id empleado
   * @param idCentro
   *           id centro
   * @param vista
   *           vista
   * @param fechaString
   *           fecha string
   * @param tipo
   *           tipo
   * @param response
   *           response
   * @return the empleado items view
   */
  @RequestMapping("/getEmpleadoCalendarItemsView")
  @Transactional(readOnly = true)
  public void getEmpleadoItemsView(@RequestParam("idempleado") String idEmpleado,
      @RequestParam("idcentro") String idCentro, @RequestParam("vista") String vista,
      @RequestParam("fecha") String fechaString, @RequestParam("tipo") String tipo, HttpServletResponse response) {
    jsonA = new JSONArray();
    festivoA = "";

    calculateFecha(vista, fechaString, tipo);
    findFestivoItems(Long.valueOf(idCentro));
    if (Long.valueOf(idEmpleado) > 0) {
      findAusenciaItems(Long.parseLong(idEmpleado), false);
      findVacacionesItems(Long.parseLong(idEmpleado), false);
      getActividadesFromEmpleado(idEmpleado, "0");
    }
    try {
      IOUtils.copy(new ByteArrayInputStream(jsonA.toString().getBytes()), response.getOutputStream());
      response.getOutputStream().flush();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  /**
   * Devuelve actividad items.
   * 
   * @param idActividad
   *           id actividad
   * @param idCentro
   *           id centro
   * @param vista
   *           vista
   * @param fechaString
   *           fecha string
   * @param tipo
   *           tipo
   * @param isEdition
   *           is edition
   * @param response
   *           response
   * @return the actividad items
   */
  @RequestMapping("/getActividadCalendarItems")
  @Transactional(readOnly = true)
  public void getActividadItems(@RequestParam("idactividad") String idActividad,
      @RequestParam("idcentro") String idCentro, @RequestParam("vista") String vista,
      @RequestParam("fecha") String fechaString, @RequestParam("tipo") String tipo,
      @RequestParam("isedition") boolean isEdition, HttpServletResponse response) {
    jsonA = new JSONArray();
    festivoA = "";

    calculateFecha(vista, fechaString.equals("null") ? null : fechaString, tipo);

    if (Long.valueOf(idActividad) > 0) {
      findActividadItems(Long.parseLong(idActividad), isEdition);
    }

    try {
      IOUtils.copy(new ByteArrayInputStream(jsonA.toString().getBytes()), response.getOutputStream());
      response.getOutputStream().flush();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  /**
   * Devuelve actividad items view.
   * 
   * @param idActividad
   *           id actividad
   * @param idCentro
   *           id centro
   * @param vista
   *           vista
   * @param fechaString
   *           fecha string
   * @param tipo
   *           tipo
   * @param response
   *           response
   * @return the actividad items view
   */
  @RequestMapping("/getActividadCalendarItemsView")
  @Transactional(readOnly = true)
  public void getActividadItemsView(@RequestParam("idactividad") String idActividad,
      @RequestParam("idcentro") String idCentro, @RequestParam("vista") String vista,
      @RequestParam("fecha") String fechaString, @RequestParam("tipo") String tipo, HttpServletResponse response) {
    jsonA = new JSONArray();
    festivoA = "";

    calculateFecha(vista, fechaString.equals("null") ? null : fechaString, tipo);
    
    if ((idCentro != null) && !idCentro.isEmpty())
    	findFestivoItems(Long.valueOf(idCentro));

    if (Long.valueOf(idActividad) > 0)
    	findActividadItems(Long.parseLong(idActividad), false);

    try {
      IOUtils.copy(new ByteArrayInputStream(jsonA.toString().getBytes()), response.getOutputStream());
      response.getOutputStream().flush();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  /**
   * Devuelve actividad items no edit.
   * 
   * @param idActividad
   *           id actividad
   * @param idCentro
   *           id centro
   * @param idEmpleado
   *           id empleado
   * @param vista
   *           vista
   * @param fechaString
   *           fecha string
   * @param tipo
   *           tipo
   * @param response
   *           response
   * @return the actividad items no edit
   */
  @RequestMapping("/getActividadCalendarItemsNoEdit")
  @Transactional(readOnly = true)
  public void getActividadItemsNoEdit(@RequestParam("idactividad") String idActividad,
      @RequestParam("idcentro") String idCentro, @RequestParam("idempleado") String idEmpleado,
      @RequestParam("vista") String vista, @RequestParam("fecha") String fechaString,
      @RequestParam("tipo") String tipo, HttpServletResponse response) {
    jsonA = new JSONArray();
    festivoA = "";

    calculateFecha(vista, fechaString.equals("null") ? null : fechaString, tipo);
    
    if ((idCentro != null) && (!idCentro.isEmpty()))
    	findFestivoItems(Long.valueOf(idCentro));

    if (Long.valueOf(idEmpleado) > 0) {
      findAusenciaItems(Long.parseLong(idEmpleado), false);
      findVacacionesItems(Long.parseLong(idEmpleado), false);
      getActividadesFromEmpleado(idEmpleado, idActividad);
    }

    try {
      IOUtils.copy(new ByteArrayInputStream(jsonA.toString().getBytes()), response.getOutputStream());
      response.getOutputStream().flush();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  /**
   * Devuelve asistencia items.
   * 
   * @param idActividad
   *           id actividad
   * @param idUsuario
   *           id usuario
   * @param idCentro
   *           id centro
   * @param vista
   *           vista
   * @param fechaString
   *           fecha string
   * @param tipo
   *           tipo
   * @param response
   *           response
   * @return the asistencia items
   */
  @RequestMapping("/getAsistenciaCalendarItems")
  @Transactional(readOnly = true)
  public void getAsistenciaItems(@RequestParam("idactividad") String idActividad,
      @RequestParam("idusuario") String idUsuario, @RequestParam("idcentro") String idCentro,
      @RequestParam("vista") String vista, @RequestParam("fecha") String fechaString,
      @RequestParam("tipo") String tipo, HttpServletResponse response) {
    jsonA = new JSONArray();
    festivoA = "";

    calculateFecha(vista, fechaString.equals("null") ? null : fechaString, tipo);
    if (Long.valueOf(idActividad) > 0) {
      findAsistenciaItems(Long.parseLong(idActividad), Long.parseLong(idUsuario));
    }
    try {
      IOUtils.copy(new ByteArrayInputStream(jsonA.toString().getBytes()), response.getOutputStream());
      response.getOutputStream().flush();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  /**
   * Devuelve asistencia items no edit.
   * 
   * @param idActividad
   *           id actividad
   * @param idUsuario
   *           id usuario
   * @param idCentro
   *           id centro
   * @param vista
   *           vista
   * @param fechaString
   *           fecha string
   * @param tipo
   *           tipo
   * @param response
   *           response
   * @return the asistencia items no edit
   */
  @RequestMapping("/getAsistenciaCalendarItemsNoEdit")
  @Transactional(readOnly = true)
  public void getAsistenciaItemsNoEdit(@RequestParam("idactividad") String idActividad,
      @RequestParam("idusuario") String idUsuario, @RequestParam("idcentro") String idCentro,
      @RequestParam("vista") String vista, @RequestParam("fecha") String fechaString,
      @RequestParam("tipo") String tipo, HttpServletResponse response) {
    jsonA = new JSONArray();
    festivoA = "";

    calculateFecha(vista, fechaString.equals("null") ? null : fechaString, tipo);
    findFestivoItems(Long.valueOf(idCentro));
    if (Long.valueOf(idActividad) > 0) {
      findActividadItems(Long.parseLong(idActividad), false);
    }
    try {
      IOUtils.copy(new ByteArrayInputStream(jsonA.toString().getBytes()), response.getOutputStream());
      response.getOutputStream().flush();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  /**
   * Devuelve reserva items.
   * 
   * @param idReserva
   *           id reserva
   * @param idCentro
   *           id centro
   * @param vista
   *           vista
   * @param fechaString
   *           fecha string
   * @param tipo
   *           tipo
   * @param isEdition
   *           is edition
   * @param response
   *           response
   * @return the reserva items
   */
  @RequestMapping("/getReservaCalendarItems")
  @Transactional(readOnly = true)
  public void getReservaItems(@RequestParam("idreserva") String idReserva, @RequestParam("idcentro") String idCentro,
      @RequestParam("vista") String vista, @RequestParam("fecha") String fechaString,
      @RequestParam("tipo") String tipo, @RequestParam("isedition") boolean isEdition, HttpServletResponse response) {
    jsonA = new JSONArray();
    festivoA = "";

    calculateFecha(vista, fechaString.equals("null") ? null : fechaString, tipo);

    if (Long.valueOf(idReserva) > 0) {
      findReservaItems(Long.parseLong(idReserva), isEdition);
    }

    try {
      IOUtils.copy(new ByteArrayInputStream(jsonA.toString().getBytes()), response.getOutputStream());
      response.getOutputStream().flush();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  /**
   * Devuelve recurso items view.
   * 
   * @param idRecurso
   *           id recurso
   * @param idCentro
   *           id centro
   * @param vista
   *           vista
   * @param fechaString
   *           fecha string
   * @param tipo
   *           tipo
   * @param response
   *           response
   * @return the recurso items view
   */
  @RequestMapping("/getRecursoCalendarItemsView")
  @Transactional(readOnly = true)
  public void getRecursoItemsView(@RequestParam("idrecurso") String idRecurso,
      @RequestParam("idcentro") String idCentro, @RequestParam("vista") String vista,
      @RequestParam("fecha") String fechaString, @RequestParam("tipo") String tipo, HttpServletResponse response) {
    jsonA = new JSONArray();
    festivoA = "";

    calculateFecha(vista, fechaString.equals("null") ? null : fechaString, tipo);
  	findFestivoItems(Long.valueOf(idCentro));

    if (Long.valueOf(idRecurso) > 0) {
      getReservasFromRecurso(idRecurso);
      getActividadesFromRecurso(idRecurso);
    }

    try {
      IOUtils.copy(new ByteArrayInputStream(jsonA.toString().getBytes()), response.getOutputStream());
      response.getOutputStream().flush();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  /**
   * Devuelve reserva items no edit.
   * 
   * @param idReserva
   *           id reserva
   * @param idEmpleado
   *           id empleado
   * @param idCentro
   *           id centro
   * @param vista
   *           vista
   * @param fechaString
   *           fecha string
   * @param tipo
   *           tipo
   * @param response
   *           response
   * @return the reserva items no edit
   */
  @RequestMapping("/getReservaCalendarItemsNoEdit")
  @Transactional(readOnly = true)
  public void getReservaItemsNoEdit(@RequestParam("idreserva") String idReserva,
      @RequestParam("idempleado") String idEmpleado, @RequestParam("idcentro") String idCentro,
      @RequestParam("vista") String vista, @RequestParam("fecha") String fechaString,
      @RequestParam("tipo") String tipo, HttpServletResponse response) {
    jsonA = new JSONArray();
    festivoA = "";

    calculateFecha(vista, fechaString.equals("null") ? null : fechaString, tipo);
    findFestivoItems(Long.valueOf(idCentro));

    if (Long.valueOf(idEmpleado) > 0) {
      findAusenciaItems(Long.parseLong(idEmpleado), false);
      findVacacionesItems(Long.parseLong(idEmpleado), false);
      getActividadesFromEmpleado(idEmpleado, "0");
    }

    try {
      IOUtils.copy(new ByteArrayInputStream(jsonA.toString().getBytes()), response.getOutputStream());
      response.getOutputStream().flush();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  /**
   * Url que publica los items del calendario de la actividad de un usuario.
   * 
   * @param idUsuario
   *          Usuario del que se mostrará su actividad
   * @param vista
   *          Vista actual publicada
   * @param fechaString
   *          Fecha actual publicada
   * @param tipo
   *          Acción para ver eventos: previos, posteriores, actuales
   * @param response
   *           response
   * @return the user activity items
   */
  @RequestMapping("/getUserActivityCalendarItems")
  public void getUserActivityItems(@RequestParam("idUsuario") String idUsuario, @RequestParam("vista") String vista,
      @RequestParam("fecha") String fechaString, @RequestParam("tipo") String tipo, HttpServletResponse response) {
    jsonA = new JSONArray();
    Usuario u = calendarService.find(Usuario.class, Long.parseLong(idUsuario));

    calculateFecha(vista, fechaString.equals("null") ? null : fechaString, tipo);
    findFestivoItems(u.getCentro().getId());
    findUserActivityItems(Long.parseLong(idUsuario));

    try {
      IOUtils.copy(new ByteArrayInputStream(jsonA.toString().getBytes()), response.getOutputStream());
      response.getOutputStream().flush();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  /**
   * Devuelve eventos panel items view.
   * 
   * @param idPanel
   *           id panel
   * @param idCentro
   *           id centro
   * @param vista
   *           vista
   * @param fechaString
   *           fecha string
   * @param tipo
   *           tipo
   * @param response
   *           response
   * @return the eventos panel items view
   */
  @RequestMapping("/getEventosPanelCalendarItems")
  @Transactional(readOnly = true)
  public void getEventosPanelItemsView(@RequestParam("idpanel") String idPanel,
      @RequestParam("idcentro") String idCentro, @RequestParam("vista") String vista,
      @RequestParam("fecha") String fechaString, @RequestParam("tipo") String tipo, HttpServletResponse response) {
    jsonA = new JSONArray();
    festivoA = "";

    calculateFecha(vista, fechaString.equals("null") ? null : fechaString, tipo);
    findFestivoItems(Long.valueOf(idCentro));

    if (Long.valueOf(idPanel) > 0) {
      findEventosPanelItems(Long.parseLong(idPanel), false);
    }

    try {
      IOUtils.copy(new ByteArrayInputStream(jsonA.toString().getBytes()), response.getOutputStream());
      response.getOutputStream().flush();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  @Transactional(readOnly = true)
  private void getReservasFromRecurso(String idRecurso) {
    Recurso r = calendarService.find(Recurso.class, Long.valueOf(idRecurso));
    for (ReservaCalendarItem rci : r.getReservas()) {
      findReservaItems(rci.getId(), false);
    }
  }

  @Transactional(readOnly = true)
  private void getActividadesFromRecurso(String idRecurso) {
    Recurso r = calendarService.find(Recurso.class, Long.valueOf(idRecurso));
    for (Actividad actividad : r.getActividades()) {
      findActividadItems(actividad.getId(), false);
    }
  }

  @SuppressWarnings("unchecked")
  @Transactional(readOnly = true)
  private void getActividadesFromEmpleado(String idEmpleado, String idActividad) {
    String consulta = "";
    
    Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    Calendar cal2 = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    
    cal.setTime(fecha.getTime());
    cal.set(Calendar.DATE, cal.getActualMinimum(Calendar.DATE));
    cal.add(Calendar.MONTH, -(MAX_MONTHS_WITH_ITEMS / 2));
    
    cal2.setTime(fecha.getTime());
    cal2.set(Calendar.DATE, cal2.getActualMinimum(Calendar.DATE));
    cal2.add(Calendar.MONTH, (MAX_MONTHS_WITH_ITEMS / 2));
    cal2.set(Calendar.DATE, cal2.getActualMaximum(Calendar.DATE));
    
    consulta = "SELECT e " +
    		   "FROM " + Curso.class.getName() + " e " +
    		   "WHERE ((e.fechaInicio BETWEEN ? AND ?) OR " +
    		   		  "(e.fechaFin BETWEEN ? AND ?)) AND " +
    		   		 "(e.profesor = ?)";
    Query query = calendarService.getEntityManager().createQuery(consulta);

    query.setParameter(1, cal.getTime());
    query.setParameter(2, cal2.getTime());
    query.setParameter(3, cal.getTime());
    query.setParameter(4, cal2.getTime());
    query.setParameter(5, calendarService.find(Empleado.class, Long.parseLong(idEmpleado)));

    List<Curso> actividades = (List<Curso>) query.getResultList();

    for (Curso actividad : actividades) {
    	if (idActividad != null && Long.parseLong(idActividad) != actividad.getId()) {
    		findActividadItems(actividad.getId(), false);
    	}
    }

    consulta = "SELECT e " +
    		   "FROM " + Taller.class.getName() + " e " +
    		   "WHERE ((e.fechaInicio BETWEEN ? AND ?) OR " +
    		   		  "(e.fechaFin BETWEEN ? AND ?)) AND " +
    		   		 "(e.profesor = ?)";
    query = calendarService.getEntityManager().createQuery(consulta);

    query.setParameter(1, cal.getTime());
    query.setParameter(2, cal2.getTime());
    query.setParameter(3, cal.getTime());
    query.setParameter(4, cal2.getTime());
    query.setParameter(5, calendarService.find(Empleado.class, Long.parseLong(idEmpleado)));

    List<Taller> talleres = (List<Taller>) query.getResultList();

    for (Taller actividad : talleres) {
    	if (idActividad != null && Long.parseLong(idActividad) != actividad.getId()) {
    		findActividadItems(actividad.getId(), false);
    	}
    }

    consulta = "SELECT e " +
    		   "FROM " + Charla.class.getName() + " e " +
    		   "WHERE (e.fechaInicio BETWEEN ? AND ?) AND " +
    		   		 "(e.profesor = ?)";
    query = calendarService.getEntityManager().createQuery(consulta);

    query.setParameter(1, cal.getTime());
    query.setParameter(2, cal2.getTime());
    query.setParameter(3, calendarService.find(Empleado.class, Long.parseLong(idEmpleado)));

    List<Charla> charlas = (List<Charla>) query.getResultList();

    for (Charla actividad : charlas) {
    	if (idActividad != null && Long.parseLong(idActividad) != actividad.getId()) {
    		findActividadItems(actividad.getId(), false);
    	}
    }

    consulta = "SELECT e " +
    		   "FROM " + Asesoramiento.class.getName() + " e " +
    		   "WHERE (e.fecha BETWEEN ? AND ? ) AND " +
    		   		 "(e.dinamizador = ?)";
    query = calendarService.getEntityManager().createQuery(consulta);
    
    query.setParameter(1, cal.getTime());
    query.setParameter(2, cal2.getTime());
    query.setParameter(3, calendarService.find(Empleado.class, Long.parseLong(idEmpleado)));

    List<Asesoramiento> asesoramientos = (List<Asesoramiento>) query.getResultList();

    for (Asesoramiento asesoramiento : asesoramientos) {
    	if (idActividad != null && Long.parseLong(idActividad) != asesoramiento.getId()) {
    		findActividadItems(asesoramiento.getId(), false);
    	}
    }
    
    consulta = "SELECT e " +
    		   "FROM " + ActividadExterna.class.getName() + " e " +
    		   "WHERE ((e.fechaInicio BETWEEN ? AND ?) OR " +
    		   		  "(e.fechaFin BETWEEN ? AND ?)) AND " +
    		   		 "(e.colaborador = ?)";
    query = calendarService.getEntityManager().createQuery(consulta);

    query.setParameter(1, cal.getTime());
    query.setParameter(2, cal2.getTime());
    query.setParameter(3, cal.getTime());
    query.setParameter(4, cal2.getTime());
    query.setParameter(5, calendarService.find(Empleado.class, Long.parseLong(idEmpleado)));

    List<ActividadExterna> externas = (List<ActividadExterna>) query.getResultList();

    for (ActividadExterna externa : externas) {
    	if (idActividad != null && Long.parseLong(idActividad) != externa.getId()) {
    		findActividadItems(externa.getId(), false);
    	}
    }
  }

  @SuppressWarnings("unchecked")
  private void findUserActivityItems(long idUsuario) {
    JSONObject jsonO;

    String consulta = "SELECT e FROM " + UserActivityCalendarItem.class.getName() + " e "
        + "WHERE (start >= ? and start <= ? ) and usuario= ? ";
    Query query = calendarService.getEntityManager().createQuery(consulta);

    Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    cal.set(Calendar.MONTH, (fecha.get(Calendar.MONTH) - (MAX_MONTHS_WITH_ITEMS / 2)));
    Calendar cal2 = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    cal2.set(Calendar.MONTH, (fecha.get(Calendar.MONTH) + (MAX_MONTHS_WITH_ITEMS / 2)));
    Usuario u = calendarService.find(Usuario.class, idUsuario);
    query.setParameter(1, cal.getTime());
    query.setParameter(2, cal2.getTime());
    query.setParameter(3, u);
    List<UserActivityCalendarItem> calendarItems = (List<UserActivityCalendarItem>) query.getResultList();
    try {
      DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm");
      for (UserActivityCalendarItem uaci : calendarItems) {
        // Objeto de JSON
        jsonO = new JSONObject();
        jsonO.put("title", uaci.getTitle());
        jsonO.put("start", df.format(uaci.getStart()));
        jsonO.put("end", df.format(uaci.getEnd() == null ? uaci.getStart() : uaci.getEnd()));
        jsonO.put("allDay", uaci.isAllDay());
        jsonO.put("className", uaci.getClassName());
        jsonO.put("descripcion", uaci.getDescription());
        jsonO.put("editable", false);
        jsonA.put(jsonO);
      }
    } catch (JSONException e) {
      e.printStackTrace();
    }
  }

  /**
   * Busca los eventos del calendario del centro.
   * 
   * @param idCentro
   *          Centro
   */
  public void findCentroItemsOld(Long idCentro) {
	  JSONObject jsonO;
	  
	  if (idCentro != null) {
		  List<CentroCalendarItem> items = calendarService.findWhere(CentroCalendarItem.class, "centro.id", idCentro);
		  
		  items.addAll(getCentroItems(idCentro));	  

		  try {
			  DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm");
			  DateFormat festivoDf = new SimpleDateFormat("dd/MM/yyyy");
			  
			  for (CentroCalendarItem ci : items) {
				  if (!festivoA.contains("" + festivoDf.format(ci.getStart()))) {
					  jsonO = new JSONObject();
					  
					  jsonO.put("title", ci.getTitle());
					  jsonO.put("start", df.format(ci.getStart()));
					  jsonO.put("end", df.format(ci.getEnd()));
					  jsonO.put("allDay", ci.isAllDay());
					  jsonO.put("className", ci.getClassName());
					  jsonO.put("descripcion", ci.getDescription());
					  jsonO.put("editable", false);
					  
					  jsonA.put(jsonO);
				  }
			  }
		  }
		  catch (JSONException e) {
			  e.printStackTrace();
		  }
	  }
  }
  
  /**
   * Busca los eventos del calendario del centro.
   * 
   * @param idCentro
   *          Centro
   */
  public List<CalendarItem> findCentroItems(Long idCentro) {
	  JSONObject jsonO;
	  
	  List<CalendarItem> returnItems = new ArrayList<CalendarItem>();
	  List<CentroCalendarItem> items = calendarService.findWhere(CentroCalendarItem.class, "centro.id", idCentro);
	  
	  if (idCentro != null) {
		  items.addAll(getCentroItems(idCentro));	  

		  try {
			  DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm");
			  DateFormat festivoDf = new SimpleDateFormat("dd/MM/yyyy");
			  
			  for (CentroCalendarItem ci : items) {
				  if (!festivoA.contains("" + festivoDf.format(ci.getStart()))) {
					  jsonO = new JSONObject();
					  
					  jsonO.put("title", ci.getTitle());
					  jsonO.put("start", df.format(ci.getStart()));
					  jsonO.put("end", df.format(ci.getEnd()));
					  jsonO.put("allDay", ci.isAllDay());
					  jsonO.put("className", ci.getClassName());
					  jsonO.put("descripcion", ci.getDescription());
					  jsonO.put("editable", false);
					  
					  jsonA.put(jsonO);
					  
					  returnItems.add(ci);
				  }
			  }
		  }
		  catch (JSONException e) {
			  e.printStackTrace();
		  }
	  }
	  
	  return returnItems;
  }

  /**
   * Busca los eventos festivos del calendario para ese centro.
   * 
   * @param idCentro
   *           id centro
   */
  public void findFestivoItems(Long idCentro) {
	  JSONObject jsonO;
	  boolean ok = true;
	  int month;
	  int year;
	  
	  Centro centro = calendarService.find(Centro.class, idCentro);
	  List<FestivoCalendarItem> festivos = calendarService.findAll(FestivoCalendarItem.class);
	  long timeInMillis = fecha == null ? 0 : fecha.getTimeInMillis();
	  
	  try {
		  DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm");
		  DateFormat festivoDf = new SimpleDateFormat("dd/MM/yyyy");
		  
		  Calendar c = Calendar.getInstance();
		  
		  for (int m = 0; m < MAX_MONTHS_WITH_ITEMS; m++) {
			  if (timeInMillis == 0)
				  timeInMillis = c.getTimeInMillis();
			  else
				  c.setTimeInMillis(timeInMillis);
			  
			  month = ((c.get(Calendar.MONTH) + 12) - (MAX_MONTHS_WITH_ITEMS / 2) + m) % 12;
			  c.set(Calendar.MONTH, month);
			  year = c.get(Calendar.YEAR);

			  for (FestivoCalendarItem fci : festivos) {
				  Calendar fest = Calendar.getInstance();
				  fest.setTime(fci.getStart());
				  ok = (fest.get(Calendar.YEAR) == year && fest.get(Calendar.MONTH) == month);

				  // Agrega sólo los festivos que aplican a este centro, por
				  // provincia y localidad,
				  // y a los meses que se quieren mostrar.
				  if (ok && (fci.getTipoFestivo() == TipoFestivo.NACIONAL ||
						  	 (fci.getProvincia().getId() == centro.getProvincia().getId() &&
						  	  (fci.getLocalidad() == null || fci.getLocalidad().getId() == centro.getLocalidad().getId())))) {
					  // Cadena con las fechas de los festivos del calendario
					  festivoA += "" + festivoDf.format(fci.getStart()) + ",";
					  
					  // Objeto de JSON
					  jsonO = new JSONObject();
					  jsonO.put("title", fci.getDefaultTitle().toUpperCase());
					  jsonO.put("start", df.format(fci.getStart()));
					  jsonO.put("allDay", fci.isAllDay());
					  jsonO.put("className", fci.getClassName());
					  jsonO.put("descripcion", fci.getDescription());
					  jsonO.put("editable", false);
					  jsonO.put("festivo", true);
					  jsonA.put(jsonO);
				  }
			  }
		  }
	  }
	  catch (JSONException e) {
		  e.printStackTrace();
	  }
  }

  /**
   * Find empleado items.
   * 
   * @param idEmpleado
   *           id empleado
   */
  public void findEmpleadoItems(Long idEmpleado) {
    JSONObject jsonO;
    List<ActividadCalendarItem> items = new ArrayList<ActividadCalendarItem>();
    if (idEmpleado != null) {
      items.addAll(getEmpleadoItems(idEmpleado));
    }
    try {
      DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm");
      DateFormat festivoDf = new SimpleDateFormat("dd/MM/yyyy");
      for (ActividadCalendarItem ci : items) {
        if (!festivoA.contains("" + festivoDf.format(ci.getStart()))) {
          jsonO = new JSONObject();
          jsonO.put("title", ci.getTitle());
          jsonO.put("start", df.format(ci.getStart()));
          jsonO.put("end", df.format(ci.getEnd()));
          jsonO.put("allDay", ci.isAllDay());
          jsonO.put("className", ci.getClassName());
          jsonO.put("descripcion", ci.getDescription());
          jsonO.put("editable", false);
          jsonO.put("actividad", true);
          jsonA.put(jsonO);
        }
      }
    } catch (JSONException e) {
      e.printStackTrace();
    }
  }

  /**
   * Busca los eventos del calendario del empleado.
   * 
   * @param idEmpleado
   *          Empleado
   * @param isEdition
   *           is edition
   */

  public void findAusenciaItems(Long idEmpleado, boolean isEdition) {
    JSONObject jsonO;
    int count = 0;
    Empleado empleado = calendarService.find(Empleado.class, idEmpleado);
    List<AusenciaCalendarItem> items = empleado.getAusencias();
    try {
      DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm");
      DateFormat festivoDf = new SimpleDateFormat("dd/MM/yyyy");
      for (AusenciaCalendarItem ai : items) {
        if (!festivoA.contains("" + festivoDf.format(ai.getStart()))) {
          ai.setEmpleado(empleado);

          jsonO = new JSONObject();
          jsonO.put("id", ai.getId() > 0 ? ("AUS" + idEmpleado + "_ID" + ai.getId())
              : ("AUS" + idEmpleado + "_ITEM" + count++));

          jsonO.put("title", ai.getTitle() == null ? "AUSENCIA" : ai.getTitle().toUpperCase());
          jsonO.put("start", df.format(ai.getStart().getTime()));
          jsonO.put("end", df.format(ai.getEnd().getTime()));
          jsonO.put("allDay", ai.isAllDay());
          jsonO.put("className", ai.getClassName());
          jsonO.put("descripcion", ai.getDescription());
          jsonO.put("editable", isEdition);
          jsonO.put("ausencia", true);
          jsonA.put(jsonO);
        }
      }
    } catch (JSONException e) {
      e.printStackTrace();
    }
  }

  /**
   * Find vacaciones items.
   * 
   * @param idEmpleado
   *           id empleado
   * @param isEdition
   *           is edition
   */
  public void findVacacionesItems(Long idEmpleado, boolean isEdition) {
    JSONObject jsonO;
    int count = 0;
    Empleado empleado = calendarService.find(Empleado.class, idEmpleado);
    List<VacacionesCalendarItem> items = empleado.getVacaciones();
    try {
      DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm");
      DateFormat festivoDf = new SimpleDateFormat("dd/MM/yyyy");
      for (VacacionesCalendarItem vi : items) {
        if (!festivoA.contains("" + festivoDf.format(vi.getStart())) && vi.getEstado().equals(Estado.APROBADAS)) {
          vi.setEmpleado(empleado);

          jsonO = new JSONObject();
          jsonO.put("id", vi.getId() > 0 ? ("VAC" + vi.getEmpleado().getId() + "_ID" + vi.getId()) : ("VAC"
              + idEmpleado + "_ITEM" + count++));
          jsonO.put("title", vi.getTitle() == null ? "VACACIONES" : vi.getTitle().toUpperCase());
          jsonO.put("start", df.format(vi.getStart().getTime()));
          jsonO.put("end", df.format(vi.getEnd().getTime()));
          jsonO.put("allDay", vi.isAllDay());
          jsonO.put("className", vi.getClassName());
          jsonO.put("descripcion", vi.getDescription());
          jsonO.put("editable", isEdition);
          jsonO.put("vacaciones", true);
          jsonA.put(jsonO);
        }
      }
    } catch (JSONException e) {
      e.printStackTrace();
    }
  }

  /**
   * Find actividad items.
   * 
   * @param idActividad
   *           id actividad
   * @param isEdition
   *           is edition
   */
  public void findActividadItemsOld(Long idActividad, boolean isEdition) {
	  JSONObject jsonO;
	  int count = 0;
    
	  Actividad actividad = calendarService.find(Actividad.class, idActividad);
	  List<ActividadCalendarItem> items = actividad.getCalendarItems().isEmpty() ? getActividadItems(idActividad) : actividad.getCalendarItems();
    
	  try {
		  DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm");
		  DateFormat festivoDf = new SimpleDateFormat("dd/MM/yyyy");
      
		  for (ActividadCalendarItem ai : items) {
			  if (!festivoA.contains("" + festivoDf.format(ai.getStart()))) {
				  ai.setActividad(actividad);
				  ai.setActividadType(actividad instanceof Curso ? ActividadType.CURSO :
					  				  (actividad instanceof Taller ? ActividadType.TALLER :
					  				   (actividad instanceof Charla ? ActividadType.CHARLA :
					  					(actividad instanceof Asesoramiento ? ActividadType.ASESORAMIENTO :
					  					 (actividad instanceof ActividadExterna ? ActividadType.EXTERNA : null)))));

				  jsonO = new JSONObject();
          
				  jsonO.put("id", ai.getId() > 0 ? ("ACT" + ai.getActividad().getId() + "_ID" + ai.getId()) :
					  							   ("ACT" + idActividad + "_ITEM" + count++));
				  jsonO.put("title", ai.getTitle());
				  jsonO.put("start", df.format(ai.getStart()));
				  if (ai.getEnd() != null)
					  jsonO.put("end", df.format(ai.getEnd()));
				  jsonO.put("allDay", ai.isAllDay());
				  jsonO.put("className", ai.getClassName() + (isEdition ? "" : " fc-no-editable"));
				  jsonO.put("descripcion", ai.getDescription());
				  jsonO.put("editable", isEdition);
				  jsonO.put("actividad", true);
				  jsonO.put("excepcion", true);
				  jsonO.put("tipoexcepcion", "");
          
				  jsonA.put(jsonO);
			  }
		  }
	  } catch (JSONException e) {
		  e.printStackTrace();
	  }
  }
  
  /**
   * Find actividad items.
   * 
   * @param idActividad
   *           id actividad
   * @param isEdition
   *           is edition
   */
  public List<CalendarItem> findActividadItems(Long idActividad, boolean isEdition) {
	  JSONObject jsonO;
	  int count = 0;
    
	  Actividad actividad = calendarService.find(Actividad.class, idActividad);
	  
	  List<CalendarItem> returnItems = new ArrayList<CalendarItem>();
	  List<ActividadCalendarItem> items = actividad.getCalendarItems().isEmpty() ? getActividadItems(idActividad) : actividad.getCalendarItems();
	  
    
	  try {
		  DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm");
		  DateFormat festivoDf = new SimpleDateFormat("dd/MM/yyyy");
      
		  for (ActividadCalendarItem ai : items) {
			  if (!festivoA.contains("" + festivoDf.format(ai.getStart()))) {
				  ai.setActividad(actividad);
				  ai.setActividadType(actividad instanceof Curso ? ActividadType.CURSO :
					  				  (actividad instanceof Taller ? ActividadType.TALLER :
					  				   (actividad instanceof Charla ? ActividadType.CHARLA :
					  					(actividad instanceof Asesoramiento ? ActividadType.ASESORAMIENTO :
					  					 (actividad instanceof ActividadExterna ? ActividadType.EXTERNA : null)))));

				  jsonO = new JSONObject();
          
				  jsonO.put("id", ai.getId() > 0 ? ("ACT" + ai.getActividad().getId() + "_ID" + ai.getId()) :
					  							   ("ACT" + idActividad + "_ITEM" + count++));
				  jsonO.put("title", ai.getTitle());
				  jsonO.put("start", df.format(ai.getStart()));
				  if (ai.getEnd() != null)
					  jsonO.put("end", df.format(ai.getEnd()));
				  jsonO.put("allDay", ai.isAllDay());
				  jsonO.put("className", ai.getClassName() + (isEdition ? "" : " fc-no-editable"));
				  jsonO.put("descripcion", ai.getDescription());
				  jsonO.put("editable", isEdition);
				  jsonO.put("actividad", true);
				  jsonO.put("excepcion", true);
				  jsonO.put("tipoexcepcion", "");
          
				  jsonA.put(jsonO);
				  
				  returnItems.add(ai);
			  }
		  }
	  } catch (JSONException e) {
		  e.printStackTrace();
	  }
	  
	  return returnItems;
  }

  /**
   * Find reserva items.
   * 
   * @param idReserva
   *           id reserva
   * @param isEdition
   *           is edition
   */
  public void findReservaItems(Long idReserva, boolean isEdition) {
    JSONObject jsonO;
    ReservaCalendarItem reserva = calendarService.find(ReservaCalendarItem.class, idReserva);
    try {
      DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm");
      DateFormat festivoDf = new SimpleDateFormat("dd/MM/yyyy");
      if (reserva.getStart() != null && !festivoA.contains("" + festivoDf.format(reserva.getStart()))) {

        jsonO = new JSONObject();
        jsonO.put("id", "RES" + reserva.getId());
        jsonO.put("title", reserva.getTitle());
        jsonO.put("start", df.format(reserva.getStart()));
        jsonO.put("end", df.format(reserva.getEnd()));
        jsonO.put("allDay", reserva.isAllDay());
        jsonO.put("className", reserva.getClassName() + (isEdition ? "" : " fc-no-editable"));
        jsonO.put("descripcion", reserva.getDescription());
        jsonO.put("editable", isEdition);
        jsonO.put("reserva", true);
        jsonA.put(jsonO);
      }

    } catch (JSONException e) {
      e.printStackTrace();
    }
  }

  /**
   * Find asistencia items.
   * 
   * @param idActividad
   *           id actividad
   * @param idUsuario
   *           id usuario
   */
  public void findAsistenciaItems(Long idActividad, Long idUsuario) {
    JSONObject jsonO;
    int count = 0;
    Actividad actividad = calendarService.find(Actividad.class, idActividad);
    List<AsistenciaCalendarItem> items = actividad.getAsistencias().isEmpty() ? new ArrayList<AsistenciaCalendarItem>()
        : actividad.getAsistencias();
    try {
      DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm");
      DateFormat festivoDf = new SimpleDateFormat("dd/MM/yyyy");
      for (AsistenciaCalendarItem ai : items) {
        if (ai.getUsuario().getId() == idUsuario && !festivoA.contains("" + festivoDf.format(ai.getStart()))) {
          ai.setActividad(actividad);

          jsonO = new JSONObject();
          jsonO.put("id", ai.getId() > 0 ? ("ASIS" + ai.getActividad().getId() + "_ID" + ai.getId()) : ("ASIS"
              + idActividad + "_ITEM" + count++));
          jsonO.put("title", ai.getTitle());
          jsonO.put("start", df.format(ai.getStart()));
          jsonO.put("end", df.format(ai.getEnd()));
          jsonO.put("allDay", ai.isAllDay());
          jsonO.put("className", ai.getExcepcion() == Excepcion.JUSTIFICADA ? (ai.getClassName() + "-just") : ai
              .getClassName());
          jsonO.put("descripcion", ai.getDescription());
          jsonO.put("editable", true);
          jsonO.put("asistencia", true);
          jsonO.put("automatica", ai.isAutomatica());
          jsonO.put("detalles", ai.getDetalles());
          jsonO.put("excepcion", ai.getExcepcion());
          jsonA.put(jsonO);
        }
      }
    } catch (JSONException e) {
      e.printStackTrace();
    }
  }

  /**
   * Find eventos panel items.
   * 
   * @param idPanel
   *           id panel
   * @param isEdition
   *           is edition
   */
  public void findEventosPanelItems(Long idPanel, boolean isEdition) {
    JSONObject jsonO;
    int count = 0;
    Panel panel = calendarService.find(Panel.class, idPanel);
    List<EventoPanel> eventos = panel.getEventosPanel();
    for (EventoPanel e : eventos) {
      List<EventoPanelCalendarItem> items = e.getCalendarItems();
      try {
        DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm");
        DateFormat festivoDf = new SimpleDateFormat("dd/MM/yyyy");
        for (EventoPanelCalendarItem eci : items) {
          if (!festivoA.contains("" + festivoDf.format(eci.getStart()))) {
            eci.setEvento(e);
            jsonO = new JSONObject();
            jsonO.put("id", eci.getId() > 0 ? ("EVE" + eci.getEvento().getId() + "_ID" + eci.getId()) : ("EVE"
                + idPanel + "_ITEM" + count++));
            jsonO.put("title", eci.getTitle());
            jsonO.put("start", df.format(eci.getStart()));
            if (eci.getEnd() != null) {
              jsonO.put("end", df.format(eci.getEnd()));
            }
            jsonO.put("allDay", eci.isAllDay());
            jsonO.put("className", eci.getClassName());
            jsonO.put("descripcion", eci.getDescription());
            jsonO.put("editable", isEdition);
            jsonO.put("actividad", true);
            jsonO.put("excepcion", true);
            jsonO.put("tipoexcepcion", "");
            jsonA.put(jsonO);
          }
        }
      } catch (JSONException ex) {
        ex.printStackTrace();
      }
    }
  }
  
  /**
   * Genera un item de calendario de centro.
   * 
   * @param cal
   *			Calendario maestro
   * 
   * @param startHour
   * 			Hora de inicio del item
   * 
   * @param endHour
   *			Hora de fin del item
   * 
   * @param text
   *			Texto del item
   * 
   * @return Lista con los eventos del centro.
   */
  private CentroCalendarItem createCentroCalendarItem(Calendar cal, Date startHour, Date endHour, String text) {
	  CentroCalendarItem item = new CentroCalendarItem();

	  Calendar tempStart = Calendar.getInstance();
      Calendar tempEnd = Calendar.getInstance();

	  tempStart.setTime(startHour);
	  tempStart.set(Calendar.SECOND, 0);
	  tempStart.set(cal.get(Calendar.YEAR), cal.get(Calendar.MONTH), cal.get(Calendar.DATE));

	  tempEnd.setTime(endHour);
	  tempEnd.set(Calendar.SECOND, 0);
	  tempEnd.set(cal.get(Calendar.YEAR), cal.get(Calendar.MONTH), cal.get(Calendar.DATE));

	  item.setDefaultTitle(text);
	  item.setDefaultDescription(text);
	  item.setStart(tempStart.getTime());
	  item.setEnd(tempEnd.getTime());
	  item.setAllDay(false);

	  return item;
  }

  /**
   * Genera los eventos de centro del calendario.
   * 
   * @param idCentro
   *          Centro.
   * @return Lista con los eventos del centro.
   */
  public List<CentroCalendarItem> getCentroItems(Long idCentro) {
	  List<CentroCalendarItem> centroItems = new ArrayList<CentroCalendarItem>();
	  Centro centro = calendarService.find(Centro.class, idCentro);

	  if (centro != null) {
		  Calendar c = Calendar.getInstance();
		  Calendar lastDate;

		  if (fecha == null)
			  c = Calendar.getInstance();
		  else
			  c.setTimeInMillis(fecha.getTimeInMillis());

		  c.add(Calendar.MONTH, -(MAX_MONTHS_WITH_ITEMS / 2));
		  c.set(Calendar.DATE, 1);

		  lastDate = (Calendar)c.clone();
		  lastDate.add(Calendar.MONTH, MAX_MONTHS_WITH_ITEMS);

		  Locale language = LocaleContextHolder.getLocaleContext().getLocale();
		  ResourceBundle rb = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", language);

		  String morningsSchedule = rb.getString("schedule.Mornings");
		  String afternoonsSchedule = rb.getString("schedule.Afternoons");

		  while (c.compareTo(lastDate) < 0) {
			  switch (c.get(Calendar.DAY_OF_WEEK)) {
			  	  case Calendar.MONDAY: {
			  		  if ((centro.getMondayMorningStartHour() != null) && (centro.getMondayMorningEndHour() != null))
			  			   centroItems.add(createCentroCalendarItem(c, centro.getMondayMorningStartHour(),
			  					   									   centro.getMondayMorningEndHour(),
			  					   									   morningsSchedule));

			  		  if ((centro.getMondayAfternoonStartHour() != null) && (centro.getMondayAfternoonEndHour() != null))
			  			  centroItems.add(createCentroCalendarItem(c, centro.getMondayAfternoonStartHour(),
			  					  									  centro.getMondayAfternoonEndHour(),
			  					  									  afternoonsSchedule));

			  		  break;
			  	  }

			  	  case Calendar.TUESDAY: {
			  		  if ((centro.getTuesdayMorningStartHour() != null) && (centro.getTuesdayMorningEndHour() != null))
			  			   centroItems.add(createCentroCalendarItem(c, centro.getTuesdayMorningStartHour(),
			  					   									   centro.getTuesdayMorningEndHour(),
			  					   									   morningsSchedule));

			  		  if ((centro.getTuesdayAfternoonStartHour() != null) && (centro.getTuesdayAfternoonEndHour() != null))
			  			   centroItems.add(createCentroCalendarItem(c, centro.getTuesdayAfternoonStartHour(),
			  					   									   centro.getTuesdayAfternoonEndHour(),
			  					   									   afternoonsSchedule));

			  		  break;
			  	  }

			  	  case Calendar.WEDNESDAY: {
			  		  if ((centro.getWednesdayMorningStartHour() != null) && (centro.getWednesdayMorningEndHour() != null))
			  			   centroItems.add(createCentroCalendarItem(c, centro.getWednesdayMorningStartHour(),
			  					   									   centro.getWednesdayMorningEndHour(),
			  					   									   morningsSchedule));

			  		  if ((centro.getWednesdayAfternoonStartHour() != null) && (centro.getWednesdayAfternoonEndHour() != null))
			  			   centroItems.add(createCentroCalendarItem(c, centro.getWednesdayAfternoonStartHour(),
			  					   									   centro.getWednesdayAfternoonEndHour(),
			  					   									   afternoonsSchedule));

			  		  break;
			  	  }

			  	  case Calendar.THURSDAY: {
			  		  if ((centro.getThursdayMorningStartHour() != null) && (centro.getThursdayMorningEndHour() != null))
			  			   centroItems.add(createCentroCalendarItem(c, centro.getThursdayMorningStartHour(),
			  					   									   centro.getThursdayMorningEndHour(),
			  					   									   morningsSchedule));

			  		  if ((centro.getThursdayAfternoonStartHour() != null) && (centro.getThursdayAfternoonEndHour() != null))
			  			   centroItems.add(createCentroCalendarItem(c, centro.getThursdayAfternoonStartHour(),
			  					   									   centro.getThursdayAfternoonEndHour(),
			  					   									   afternoonsSchedule));

			  		  break;
			  	  }

			  	  case Calendar.FRIDAY: {
			  		  if ((centro.getFridayMorningStartHour() != null) && (centro.getFridayMorningEndHour() != null))
			  			   centroItems.add(createCentroCalendarItem(c, centro.getFridayMorningStartHour(),
			  					   									   centro.getFridayMorningEndHour(),
			  					   									   morningsSchedule));

			  		  if ((centro.getFridayAfternoonStartHour() != null) && (centro.getFridayAfternoonEndHour() != null))
			  			  centroItems.add(createCentroCalendarItem(c, centro.getFridayAfternoonStartHour(),
			  					  									  centro.getFridayAfternoonEndHour(),
			  					  									  afternoonsSchedule));

			  		  break;
			  	  }

			  	  case Calendar.SATURDAY: {
			  		  if ((centro.getSaturdayMorningStartHour() != null) && (centro.getSaturdayMorningEndHour() != null))
			  			   centroItems.add(createCentroCalendarItem(c, centro.getSaturdayMorningStartHour(),
			  					   									   centro.getSaturdayMorningEndHour(),
			  					   									   morningsSchedule));

			  		  if ((centro.getSaturdayAfternoonStartHour() != null) && (centro.getSaturdayAfternoonEndHour() != null))
			  			   centroItems.add(createCentroCalendarItem(c, centro.getSaturdayAfternoonStartHour(),
			  					   									   centro.getSaturdayAfternoonEndHour(),
			  					   									   afternoonsSchedule));

			  		  break;
			  	  }

			  	  case Calendar.SUNDAY: {
			  		  if ((centro.getSundayMorningStartHour() != null) && (centro.getSundayMorningEndHour() != null))
			  			   centroItems.add(createCentroCalendarItem(c, centro.getSundayMorningStartHour(),
			  					   									   centro.getSundayMorningEndHour(),
			  					   									   morningsSchedule));

			  		  if ((centro.getSundayAfternoonStartHour() != null) && (centro.getSundayAfternoonEndHour() != null))
			  			   centroItems.add(createCentroCalendarItem(c, centro.getSundayAfternoonStartHour(),
			  					   									   centro.getSundayAfternoonEndHour(),
			  					   									   afternoonsSchedule));

			  		  break;
			  	  }
			  }

			  c.add(Calendar.DATE, 1);
		  }
	  }

	  return centroItems;
  }

  /**
   * Devuelve empleado items.
   * 
   * @param idEmpleado
   *           id empleado
   * @return the empleado items
   */
  public List<ActividadCalendarItem> getEmpleadoItems(Long idEmpleado) {
    List<ActividadCalendarItem> actividadItems = new ArrayList<ActividadCalendarItem>();

    List<Curso> cursos = calendarService.findWhere(Curso.class, "profesor.id", idEmpleado);
    List<Taller> talleres = calendarService.findWhere(Taller.class, "profesor.id", idEmpleado);
    List<Charla> charlas = calendarService.findWhere(Charla.class, "profesor.id", idEmpleado);
    List<Asesoramiento> asesoramientos = calendarService.findWhere(Asesoramiento.class, "dinamizador.id", idEmpleado);

    for (Curso c : cursos) {
      if (c != null && c.getId() > 0) {
        actividadItems.addAll(getActividadItems(c.getId()));
      }
    }
    for (Taller t : talleres) {
      if (t != null && t.getId() > 0) {
        actividadItems.addAll(getActividadItems(t.getId()));
      }
    }
    for (Charla c : charlas) {
      if (c != null && c.getId() > 0) {
        actividadItems.addAll(getActividadItems(c.getId()));
      }
    }
    for (Asesoramiento a : asesoramientos) {
      if (a != null && a.getId() > 0) {
        actividadItems.addAll(getActividadItems(a.getId()));
      }
    }

    return actividadItems;
  }

  /**
   * Genera los eventos del horario de la actividad en el calendario.
   * 
   * @param idActividad
   *          Actividad.
   * @return Lista con los eventos del horario de la actividad.
   */

  public List<ActividadCalendarItem> getActividadItems(Long idActividad) {
    List<ActividadCalendarItem> actividadItems = new ArrayList<ActividadCalendarItem>();
    ActividadCalendarItem item;
    Calendar c = Calendar.getInstance();
    Calendar tempStart = Calendar.getInstance();
    Calendar tempEnd = Calendar.getInstance();
    Calendar tempTime = Calendar.getInstance();

    Actividad actividad = calendarService.find(Actividad.class, idActividad);
    if (actividad != null) {
      if (actividad instanceof Curso || actividad instanceof Taller) {

        tempTime.setTime(actividad.getHoraInicio());
        tempStart.setTime(actividad.getFechaInicio());
        tempStart.set(Calendar.HOUR, tempTime.get(Calendar.HOUR_OF_DAY));
        tempStart.set(Calendar.MINUTE, tempTime.get(Calendar.MINUTE));

        tempTime.setTime(actividad.getHoraFin());
        tempEnd.setTime(actividad.getFechaFin());
        tempEnd.set(Calendar.HOUR, tempTime.get(Calendar.HOUR_OF_DAY));
        tempEnd.set(Calendar.MINUTE, tempTime.get(Calendar.MINUTE));

        c.setTime(tempStart.getTime());

        while (c.before(tempEnd)
            || (c.get(Calendar.DAY_OF_YEAR) <= tempEnd.get(Calendar.DAY_OF_YEAR) && c.get(Calendar.YEAR) == tempEnd
                .get(Calendar.YEAR))) {

          if (c.get(Calendar.DAY_OF_WEEK) > 1 && c.get(Calendar.DAY_OF_WEEK) < 7) {
            item = new ActividadCalendarItem();

            c.set(Calendar.HOUR_OF_DAY, tempStart.get(Calendar.HOUR_OF_DAY));
            c.set(Calendar.MINUTE, tempStart.get(Calendar.MINUTE));
            item.setStart(c.getTime());
            c.set(Calendar.HOUR_OF_DAY, tempEnd.get(Calendar.HOUR_OF_DAY));
            c.set(Calendar.MINUTE, tempEnd.get(Calendar.MINUTE));
            item.setEnd(c.getTime());

            item.setAllDay(false);
            actividadItems.add(item);
          }
          c.set(Calendar.DATE, c.get(Calendar.DATE) + 1);
        }

      }

      else if (actividad instanceof Charla) {
        Charla charla = (Charla) actividad;
        c.setTime(charla.getFechaInicio());
        if (charla.getHoraInicio() != null && charla.getHoraFin() != null) {

          tempStart.setTime(charla.getHoraInicio());
          tempEnd.setTime(charla.getHoraFin());

          item = new ActividadCalendarItem();
          c.set(Calendar.HOUR_OF_DAY, tempStart.get(Calendar.HOUR_OF_DAY));
          c.set(Calendar.MINUTE, tempStart.get(Calendar.MINUTE));
          item.setStart(c.getTime());
          c.set(Calendar.HOUR_OF_DAY, tempEnd.get(Calendar.HOUR_OF_DAY));
          c.set(Calendar.MINUTE, tempEnd.get(Calendar.MINUTE));
          item.setEnd(c.getTime());

          item.setAllDay(false);
          actividadItems.add(item);

        }

      }

      else if (actividad instanceof Asesoramiento) {
        Asesoramiento asesoramiento = (Asesoramiento) actividad;
        c.setTime(asesoramiento.getFecha());
        if (asesoramiento.getHora() != null) {

          tempStart.setTime(asesoramiento.getHora());
          tempEnd.setTime(asesoramiento.getHora());
          tempEnd.set(Calendar.MINUTE, asesoramiento.getDuracion());

          item = new ActividadCalendarItem();
          c.set(Calendar.HOUR_OF_DAY, tempStart.get(Calendar.HOUR_OF_DAY));
          c.set(Calendar.MINUTE, tempStart.get(Calendar.MINUTE));
          item.setStart(c.getTime());
          c.set(Calendar.HOUR_OF_DAY, tempEnd.get(Calendar.HOUR_OF_DAY));
          c.set(Calendar.MINUTE, tempEnd.get(Calendar.MINUTE));
          item.setEnd(c.getTime());

          item.setAllDay(false);
          actividadItems.add(item);

        }

      }

    }
    return actividadItems;
  }

  /**
   * A partir de unos parámetros, calcula la fecha actual en la que se va a posicionar el calendario
   * 
   * @param vista
   *          Tipo de vista (mes, semana o diaria)
   * @param fechaString
   *          Fecha actual en el calendario
   * @param tipo
   *          Tipo de movimiento que se va a producir (atrás, adelante, día de hoy)
   */
  private void calculateFecha(String vista, String fechaString, String tipo) {
	  if (tipo.equals("prev"))
		  prevFecha(vista, fechaString);
	  else if (tipo.equals("next"))
		  nextFecha(vista, fechaString);
	  else {
		  fecha = Calendar.getInstance();
		  
		  if (fechaString != null)
			  fecha.setTime(new Date(Long.parseLong(fechaString)));
	  }
  }

  /**
   * Calcula la fecha previa, en base a la fecha actual, y la vista a la que se dirige.
   * 
   * @param vista
   *          Vista a publicar.
   * @param fechaString
   *          Fecha actual.
   */
  public void prevFecha(String vista, String fechaString) {
	  fecha = Calendar.getInstance();
	  fecha.setTime(new Date(Long.parseLong(fechaString)));

	  if (vista.equals("month")) {
		  fecha.set(Calendar.DATE, 1);
		  fecha.set(Calendar.MONTH, fecha.get(Calendar.MONTH) - 1);
	  }
	  else if (vista.equals("agendaWeek"))
		  fecha.set(Calendar.DATE, fecha.get(Calendar.DATE) - 7);
	  else if (vista.equals("agendaDay"))
		  fecha.set(Calendar.DATE, fecha.get(Calendar.DATE) - 1);
  }

  /**
   * Calcula la fecha posterior, en base a la fecha actual, y la vista a la que se dirige.
   * 
   * @param vista
   *          Vista a publicar.
   * @param fechaString
   *          Fecha actual.
   */
  public void nextFecha(String vista, String fechaString) {
	  fecha = Calendar.getInstance();
	  fecha.setTime(new Date(Long.parseLong(fechaString)));

	  if (vista.equals("month")) {
		  fecha.set(Calendar.DATE, 1);
		  fecha.set(Calendar.MONTH, fecha.get(Calendar.MONTH) + 1);
	  }
	  else if (vista.equals("agendaWeek"))
		  fecha.set(Calendar.DATE, fecha.get(Calendar.DATE) + 7);
	  else if (vista.equals("agendaDay"))
		  fecha.set(Calendar.DATE, fecha.get(Calendar.DATE) + 1);
  }
  
  private void setOpenClassroom(List<CalendarItem> lci, List<CalendarItem> lai) {
	  List<CalendarItem> locci = getOpenClassromCalendarItems(lci, lai);
	  
	  if (locci.size() > 0) {
		  //Crea los items de calendario para pintar y los añade al array
		  //de objetos JSON para ser interpretados por el navegador
		  try {
			  Locale language = LocaleContextHolder.getLocaleContext().getLocale();
			  ResourceBundle rb = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.messages", language);

			  String openClassroom = rb.getString("schedule.OpenClassroom");
			  
			  DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm");
			  
			  for (CalendarItem occi : locci) {
				  JSONObject jsonO = new JSONObject();

				  jsonO.put("id", "AA_ITEM" + locci.indexOf(occi));
				  jsonO.put("title", openClassroom);
				  jsonO.put("start", df.format(occi.getStart()));
				  
				  if (occi.getEnd() != null)
					  jsonO.put("end", df.format(occi.getEnd()));
				  
				  jsonO.put("allDay", false);
				  jsonO.put("className", "fc-horario-aulaabierta fc-no-editable");
				  jsonO.put("descripcion", openClassroom);
				  jsonO.put("editable", false);
				  jsonO.put("actividad", true);
				  jsonO.put("excepcion", true);
				  jsonO.put("tipoexcepcion", "");
          
				  jsonA.put(jsonO);
			  }
		  }
		  catch (JSONException ex) {
			  ex.printStackTrace();
		  }
	  }
  }
	  
  private static Comparator<CalendarItem> comparatorStartDateTimeCalendarItem = new Comparator<CalendarItem>() {
	  public int compare(CalendarItem o1, CalendarItem o2) {
		  return (o1.getStart().compareTo(o2.getStart()));
	  }
  };

  private static int isActivityInGap(CalendarItem ai, CalendarItem gi) {
	  if (ai.getEnd().before(gi.getStart()))
		  return -1;
	  else if (ai.getStart().after(gi.getEnd()))
		  return 1;
	  else
		  return 0;
  }
  
  public static List<CalendarItem> getOpenClassromCalendarItems(List<CalendarItem> lci, List<CalendarItem> lai) {
	  //Ordena ambos array atendiendo a la fecha y la hora
	  Collections.sort(lci, comparatorStartDateTimeCalendarItem);
	  Collections.sort(lai, comparatorStartDateTimeCalendarItem);

	  List<CalendarItem> locci = new ArrayList<CalendarItem>();

	  for (CalendarItem ci : lci) {
		  int inGap = -1; //Por defecto suponemos que el item de actividad es posterior al gap del item de centro
		  
		  //Siempre se coloca el indice sobre el primer item de actividad por si algún item ocupa más de un
		  //gap de item de centro y necesita ser procesado más de una vez.
		  int laiIndex = 0;
		  
		  //Los items de actividad anteriores al gap del items de centro actual se saltan
		  while ((laiIndex < lai.size()) && (inGap < 0)) {
			  //Comprobamos si el item de actividad se solapa en algún punto con
			  //el gap del item del centro (0), es anterior (-1) o posterior (1)
			  inGap = isActivityInGap(lai.get(laiIndex), ci);

			  if (inGap < 0)
				  laiIndex++;
		  }

		  //Encontrado item de actividad con hora de inicio dentro del item del centro actual
		  if (inGap == 0) {
			  //Buscar los huecos que dejan los lai dentro del ci
			  
			  Date occiInicio = new Date(ci.getStart().getTime());
			  Date occiFin = new Date(ci.getEnd().getTime());

			  //Para comprobar después si el siguiente item de actividad pertenece al gap
			  boolean nextInCiGap;
			  
			  //***Inicio del procesado de items de actividad dentro del gap del item de centro actual***//
			  do {
				  //Inicializamos hora de inicio y fin del item de actividad o grupo de items de actividades solapadas
				  Date aiInicio = new Date(lai.get(laiIndex).getStart().getTime());
				  Date aiFin = new Date(lai.get(laiIndex).getEnd().getTime());
				  
				  //***Inicio de la detección de items de actividad solapados dentro del item del centro actual.***//
				  //Puede pasar en Centros con más de un aula o por errores de planificación.

				  //Mientra haya un item de actividad siguiente y
				  //ese item de actividad siguiente se solapa en algún punto con del gap del item del centro actual y
				  //ese item de actividad siguiente comience antes del fin maximo o comienza justo en el fin máximo.
				  while ((laiIndex < (lai.size() - 1)) &&
						 (isActivityInGap(lai.get(laiIndex + 1), ci) == 0) &&
						 (lai.get(laiIndex + 1).getStart().before(aiFin) ||
						  aiFin.getTime() == lai.get(laiIndex + 1).getStart().getTime())) {

					  //Se posiciona el siguiente item de actividad
					  laiIndex++;

					  //Si el fin maximo tiene lugar antes que el fin del item de actividad
					  if (aiFin.before(lai.get(laiIndex).getEnd()))
						  //Asigna como nuevo máximo el final del item de actividad
						  aiFin.setTime(lai.get(laiIndex).getEnd().getTime());
				  }
				  //***Fin de la detección de items de actividad solapados dentro del item del centro actual.***
				  
				  //Si la hora de inicio de la actividad es posterior a la hora de inicio del gap y
				  //la hora de inicio de la actividad es anterior a la hora de fin del gap
				  if (aiInicio.after(occiInicio) && aiInicio.before(occiFin)) {
				      //Se crea un item de aula abierta entre el inicio del gap y el inicio de la actividad
					  CalendarItem occi = new CalendarItem();
					  
					  occi.setStart(new Date(occiInicio.getTime()));
					  occi.setEnd(new Date(aiInicio.getTime()));
					  
					  locci.add(occi);
					  
					  //Se asigna el inicio del gap al final de la actividad.
					  occiInicio.setTime(aiFin.getTime());
				  }
				  //Si el inicio del item de la actividad es anterior o igual al inicio del gap
				  else if (aiInicio.getTime() <= occiInicio.getTime())
					  //Se asigna el inicio del gap al final de la actividad.
					  occiInicio.setTime(aiFin.getTime());
				  
				  //Avanza al siguiente item de actividad
				  laiIndex++;
				  
				  //Calcula si el siguiente item de actividad se solapa en algún punto con el gap del item del centro
				  nextInCiGap = (laiIndex < lai.size()) && (isActivityInGap(lai.get(laiIndex), ci) == 0);
				  
				  //Si el siguiente item de actividad no están en el gap del item del centro y
				  //queda tiempo entre el inicio y el fin del gap parcial
				  if (!nextInCiGap && (occiInicio.compareTo(occiFin) < 0)) {
					  //Se crea un item de aula abierta
					  CalendarItem occi = new CalendarItem();
					  
					  occi.setStart(new Date(occiInicio.getTime()));
					  occi.setEnd(new Date(occiFin.getTime()));
					  
					  locci.add(occi);
				  }
			  } while(nextInCiGap);
			  //***Fin del procesado de items de actividad dentro del gap del item de centro actual.***/
		  }
		  else {
			  //Crear item de aula abierta para todo el ci
			  CalendarItem occi = new CalendarItem();
			  
			  occi.setStart(new Date(ci.getStart().getTime()));
			  occi.setEnd(new Date(ci.getEnd().getTime()));
				  
			  locci.add(occi);
		  }			  
	  }
	  
	  return locci;
  }
}
