package es.csa.fdt.cyldigital.controller;

/*
 * #%L
 * Librerías del núcleo para Web - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.context.WebApplicationContext;
import org.springframework.web.servlet.support.RequestContextUtils;

import es.csa.fdt.cyldigital.incidencias.entity.Incidencia.Criticidad;
import es.csa.fdt.cyldigital.incidencias.entity.Incidencia.EstadoIncidencia;
import es.csa.fdt.cyldigital.listings.ListingHelperXLSGenerator;
import es.csa.fdt.cyldigital.notificacion.service.InformesNotificationsBrowserImpl.EActivityTypes;
import es.csa.fdt.cyldigital.notificacion.service.InformesNotificationsBrowserImpl.EReportDatosTypes;
import es.csa.fdt.cyldigital.notificacion.service.InformesNotificationsBrowserImpl.EReportType;
import es.csa.fdt.cyldigital.recursos.entity.Recurso.EstadoRecurso;

/**
 * Clase ExcelControllerImpl.
 */
@Controller
public class ExcelControllerImpl {
	private class ExcelComputation {
		private Long idExcel;
		private String nombreFichero;
		private String method;
		private Map<Integer, Object> parameters;
		private Locale locale;
		private ByteArrayOutputStream outputStream;
		private EDocumentFormats formato;
		
		private Thread thread;
		private Boolean consumed;
		private Long finishTimestamp;

		public ExcelComputation(Long idExcel, String method, Map<Integer, Object> parameters, EDocumentFormats formato) {
			this.idExcel = idExcel;
			this.nombreFichero = null;
			this.method = method;
			this.parameters = parameters;
			this.locale = LocaleContextHolder.getLocaleContext().getLocale();
			this.outputStream = new ByteArrayOutputStream();
			this.formato = formato;

			this.consumed = false;
			this.finishTimestamp = -1L;
		}
		
		public Long getIdExcel() {
			return idExcel;
		}
		
		public String getNombreFichero() {
			return nombreFichero;
		}
		
		public void setNombreFichero(String nombreFichero) {
			this.nombreFichero = nombreFichero;
		}

		public String getMethod() {
			return method;
		}

		public Map<Integer, Object> getParameters() {
			return parameters;
		}
		
		public Locale getLocale() {
			return locale;
		}

		public ByteArrayOutputStream getOutputStream() {
			return outputStream;
		}

		public Boolean isConsumed() {
			return consumed;
		}

		public void setToConsumed() {
			this.consumed = true;
		}

		public Long getFinishTimestamp() {
			return finishTimestamp;
		}

		public void setFinishTimestamp(Long finishTimestamp) {
			this.finishTimestamp = finishTimestamp;
		}

		public Thread getThread() {
			return thread;
		}

		public void setThread(Thread thread) {
			this.thread = thread;
		}
		
		public EDocumentFormats getFormato() {
		    return formato;
		}
		
	}
	
	@SuppressWarnings("unused")
	private static final long serialVersionUID = -4581725741365803463L;
	
	private static HashMap<Long, ExcelComputation> results = new HashMap<Long, ExcelComputation>();

	public enum EDocumentFormats {
		XLS,
		CSV;
	} 
	
	public String idioma;
	
	private Long launchExcelComputation(HttpServletRequest request, String method, Map<Integer, Object> parameters) {
	    return launchExcelComputation(request, method, parameters, EDocumentFormats.XLS);
	}
	
	private Long launchExcelComputation(HttpServletRequest request, String method, Map<Integer, Object> parameters, EDocumentFormats formato) {
		//Asigna un id al listado
	    Long idExcel = new Date().getTime();
	    
	    //Crea el objeto con los datos necesarios para la generación del informe
	    ExcelComputation excelComputation = new ExcelComputation(idExcel, method, parameters, formato);
	    
	    //Busca el componente generador de listados Excel.
	    ServletContext servletContext = request.getSession().getServletContext();
	    WebApplicationContext webApplicationContext = RequestContextUtils.getWebApplicationContext(request, servletContext);
	    ListingHelperXLSGenerator helper = (ListingHelperXLSGenerator) webApplicationContext.getBean("listingHelperXLSGenerator");
	    helper.setFormato(excelComputation.getFormato());
	    
	    //Crea y lanza el thread para la computación del informe
	    ExcelComputationThread excelComputationThread = new ExcelComputationThread(excelComputation, helper);
	    excelComputation.setThread(excelComputationThread);
	    
	    synchronized (results) {
	    	results.put(excelComputation.getIdExcel(), excelComputation);
	    }
	    
	    excelComputationThread.start();
	    
	    return idExcel;
	}

	/**
	 * Stream diploma curso.
	 * 
	 * @param idCurso
	 *            id curso
	 * @param response
	 *            response
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirMatriculadosCurso")
	@Transactional(readOnly = true)
	public void streamMatriculadosCurso(HttpServletRequest request, @RequestParam("idCurso") String idCurso, HttpServletResponse response) throws IOException {
		//Parametros del método
	    Map<Integer, Object> parameters = new HashMap<Integer, Object>();
	    parameters.put(0, idCurso);
	    
	    Long idExcel = launchExcelComputation(request, "streamMatriculadosCurso", parameters);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}

	/**
	 * Stream diploma taller.
	 * 
	 * @param idTaller
	 *            id taller
	 * @param response
	 *            response
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirMatriculadosTaller")
	@Transactional(readOnly = true)
	public void streamMatriculadosTaller(HttpServletRequest request, @RequestParam("idTaller") String idTaller, HttpServletResponse response) throws IOException {
		//Parametros del método
	    Map<Integer, Object> parameters = new HashMap<Integer, Object>();
	    parameters.put(0, idTaller);
	    
	    Long idExcel = launchExcelComputation(request, "streamMatriculadosTaller", parameters);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}

	/**
	 * Stream diploma charla.
	 * 
	 * @param idCharla
	 *            id charla
	 * @param response
	 *            response
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirMatriculadosCharla")
	@Transactional(readOnly = true)
	public void streamMatriculadosCharla(HttpServletRequest request, @RequestParam("idCharla") String idCharla, HttpServletResponse response) throws IOException {
		//Parametros del método
	    Map<Integer, Object> parameters = new HashMap<Integer, Object>();
	    parameters.put(0, idCharla);

	    Long idExcel = launchExcelComputation(request, "streamMatriculadosCharla", parameters);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}


	/**
	 * Stream empleados.
	 * 
	 * @param response
	 *            response
	 * @param idCentro
	 *            id centro
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirEmpleados")
	@Transactional(readOnly = true)
	public void streamPersonal(HttpServletRequest request, @RequestParam("centroId") Long idCentro, HttpServletResponse response) throws IOException {
		//Parametros del método
	    Map<Integer, Object> parameters = new HashMap<Integer, Object>();
	    parameters.put(0, idCentro);
	    
	    Long idExcel = launchExcelComputation(request, "streamEmpleados", parameters);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}		
	
	/**
	 * Stream proveedores.
	 * 
	 * @param response
	 *            response
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirProveedores")
	@Transactional(readOnly = true)
	public void streamProveedores(HttpServletRequest request, HttpServletResponse response) throws IOException {
		//Parametros del método
	    Map<Integer, Object> parameters = new HashMap<Integer, Object>();
	    
	    Long idExcel = launchExcelComputation(request, "streamProveedores", parameters);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}	
	
	/**
	 * Stream usuarios.
	 * 
	 * @param response
	 *            response
	 * @param idCentro
	 *            id centro
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirUsuarios")
	@Transactional(readOnly = true)
	public void streamUsuarios(HttpServletRequest request, 
			@RequestParam("centroId") Long idCentro,
			@RequestParam("indActivoLDAP") Long indActivoLDAP, 
			@RequestParam(value="formato", required=false) EDocumentFormats formato,
			HttpServletResponse response) throws IOException {
		//Parametros del método
	    Map<Integer, Object> parameters = new HashMap<Integer, Object>();
	    parameters.put(0, idCentro);
	    parameters.put(1, indActivoLDAP);
	    
	    //Establecemos el formato por defecto.
	    if(formato==null)formato=EDocumentFormats.XLS;
	    
	    Long idExcel = launchExcelComputation(request, "streamUsuarios", parameters, formato);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}

	/**
	 * Stream usuarios pre.
	 * 
	 * @param response
	 *            response
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirUsuariosPre")
	@Transactional(readOnly = true)
	public void streamUsuariosPre(
			HttpServletRequest request, 
			@RequestParam("centroId") Long idCentro, 
			HttpServletResponse response) throws IOException {
		
		Map<Integer, Object> parameters = new HashMap<Integer, Object>();
	    parameters.put(0, idCentro);
		
		Long idExcel = launchExcelComputation(request, "streamUsuariosPre", parameters);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}

	/**
	 * Stream cursos con plaza.
	 * 
	 * @param response
	 *            response
	 * @param idCentro
	 *            id centro
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirCursosConPlaza")
	@Transactional(readOnly = true)
	public void streamCursosConPlaza(HttpServletRequest request, @RequestParam("centroId") Long idCentro, HttpServletResponse response) throws IOException {
		//Parametros del método
	    Map<Integer, Object> parameters = new HashMap<Integer, Object>();
	    parameters.put(0, idCentro);

	    Long idExcel = launchExcelComputation(request, "streamCursosConPlaza", parameters);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}

	/**
	 * Stream cursos con los aplicados en el browser.
	 * @param response
	 *            response
	 * @param filtros
	 *            filtros del browser
	 * @param indPlazasLibres
	 *            filtro especial para las plazas libres            
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirCursosBrowser")
	@Transactional(readOnly = true)
	public void streamCursosBrowser(HttpServletRequest request, 
			@RequestParam("centroId") Long idCentro,
			@RequestParam("filtros") String filtros, 
			@RequestParam("indPlazasLibres") Long indPlazasLibres, 
			@RequestParam(value="formato", required=false) EDocumentFormats formato,
			HttpServletResponse response) throws IOException {
		
		//Parametros del método
	    Map<Integer, Object> parameters = new HashMap<Integer, Object>();
	    parameters.put(0, idCentro);
	    parameters.put(1, filtros);
	    parameters.put(2, indPlazasLibres);

	    //Establecemos el formato por defecto.
	    if(formato==null)formato=EDocumentFormats.XLS;
	    
	    Long idExcel = launchExcelComputation(request, "streamCursosBrowser", parameters, formato);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}
	
	/**
	 * Stream cursos.
	 * 
	 * @param response
	 *            response
	 * @param idCentro
	 *            id centro
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirCursos")
	@Transactional(readOnly = true)
	public void streamCursos(HttpServletRequest request, 
			@RequestParam("centroId") Long idCentro,
			@RequestParam("tematicaId") Long idTematica,
			@RequestParam("indIniciativaSingular") Long indIniciativaSingular,
			@RequestParam("indVideoconferencia") Long indVideoconferencia, 
			@RequestParam("fechaInicio") Long fechaInicio, 
			@RequestParam("fechaFin") Long fechaFin, HttpServletResponse response) throws IOException {
		//Parametros del método
	    Map<Integer, Object> parameters = new HashMap<Integer, Object>();
	    parameters.put(0, idCentro);
	    parameters.put(1, idTematica);
	    parameters.put(2, indIniciativaSingular);
	    parameters.put(3, indVideoconferencia);
    	parameters.put(4, fechaInicio);
    	parameters.put(5, fechaFin);

	    Long idExcel = launchExcelComputation(request, "streamCursos", parameters);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}

	/**
	 * Stream servicios cursos.
	 * 
	 * @param response
	 *            response
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirServiciosCursos")
	@Transactional(readOnly = true)
	public void streamServiciosCursos(HttpServletRequest request, HttpServletResponse response) throws IOException {
	    Long idExcel = launchExcelComputation(request, "streamServiciosCursos", null);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}

	/**
	 * Stream servicios talleres.
	 * 
	 * @param response
	 *            response
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirServiciosTalleres")
	@Transactional(readOnly = true)
	public void streamServiciosTalleres(HttpServletRequest request, HttpServletResponse response) throws IOException {
	    Long idExcel = launchExcelComputation(request, "streamServiciosTalleres", null);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}

	/**
	 * Stream talleres con plaza.
	 * 
	 * @param response
	 *            response
	 * @param idCentro
	 *            id centro
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirTalleresConPlaza")
	@Transactional(readOnly = true)
	public void streamTalleresConPlaza(HttpServletRequest request, @RequestParam("centroId") Long idCentro, HttpServletResponse response) throws IOException {
		//Parametros del método
	    Map<Integer, Object> parameters = new HashMap<Integer, Object>();
	    parameters.put(0, idCentro);

	    Long idExcel = launchExcelComputation(request, "streamTalleresConPlaza", parameters);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}

	/**
	 * Stream talleres.
	 * 
	 * @param response
	 *            response
	 * @param idCentro
	 *            id centro
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirTalleres")
	@Transactional(readOnly = true)
	public void streamTalleres(HttpServletRequest request, @RequestParam("centroId") Long idCentro, HttpServletResponse response) throws IOException {
		//Parametros del método
	    Map<Integer, Object> parameters = new HashMap<Integer, Object>();
	    parameters.put(0, idCentro);

	    Long idExcel = launchExcelComputation(request, "streamTalleres", parameters);

	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}

	/**
	 * Stream cursos personal.
	 * 
	 * @param idPersonal
	 *            id personal
	 * @param nombrePersonal
	 *            nombre personal
	 * @param response
	 *            response
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirCursosPersonal")
	@Transactional(readOnly = true)
	public void streamCursosPersonal(HttpServletRequest request, @RequestParam("idPersonal") String idPersonal,
			@RequestParam("nombre") String nombrePersonal, HttpServletResponse response) throws IOException {
		//Parametros del método
	    Map<Integer, Object> parameters = new HashMap<Integer, Object>();
	    parameters.put(0, idPersonal);
	    parameters.put(1, nombrePersonal);
	    
	    Long idExcel = launchExcelComputation(request, "streamCursosPersonal", parameters);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}

	/**
	 * Stream cursos proximos personal.
	 * 
	 * @param idPersonal
	 *            id personal
	 * @param nombrePersonal
	 *            nombre personal
	 * @param response
	 *            response
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirCursosProximosPersonal")
	@Transactional(readOnly = true)
	public void streamCursosProximosPersonal(HttpServletRequest request, @RequestParam("idPersonal") String idPersonal,
			@RequestParam("nombre") String nombrePersonal, HttpServletResponse response) throws IOException {
		//Parametros del método
	    Map<Integer, Object> parameters = new HashMap<Integer, Object>();
	    parameters.put(0, idPersonal);
	    parameters.put(1, nombrePersonal);
	    
	    Long idExcel = launchExcelComputation(request, "streamCursosProximosPersonal", parameters);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}

	/**
	 * Stream talleres personal.
	 * 
	 * @param idPersonal
	 *            id personal
	 * @param nombrePersonal
	 *            nombre personal
	 * @param response
	 *            response
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirTalleresPersonal")
	@Transactional(readOnly = true)
	public void streamTalleresPersonal(HttpServletRequest request, @RequestParam("idPersonal") String idPersonal,
			@RequestParam("nombre") String nombrePersonal, HttpServletResponse response) throws IOException {
		//Parametros del método
	    Map<Integer, Object> parameters = new HashMap<Integer, Object>();
	    parameters.put(0, idPersonal);
	    parameters.put(1, nombrePersonal);
	    
	    Long idExcel = launchExcelComputation(request, "streamTalleresPersonal", parameters);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}

	/**
	 * Stream talleres proximos personal.
	 * 
	 * @param idPersonal
	 *            id personal
	 * @param nombrePersonal
	 *            nombre personal
	 * @param response
	 *            response
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirTalleresProximosPersonal")
	@Transactional(readOnly = true)
	public void streamTalleresProximosPersonal(HttpServletRequest request, @RequestParam("idPersonal") String idPersonal,
			@RequestParam("nombre") String nombrePersonal, HttpServletResponse response) throws IOException {
		//Parametros del método
	    Map<Integer, Object> parameters = new HashMap<Integer, Object>();
	    parameters.put(0, idPersonal);
	    parameters.put(1, nombrePersonal);
	    
	    Long idExcel = launchExcelComputation(request, "streamTalleresProximosPersonal", parameters);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}

	/**
	 * Stream charlas con plaza.
	 * 
	 * @param response
	 *            response
	 * @param idCentro
	 *            id centro
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirCharlasConPlaza")
	@Transactional(readOnly = true)
	public void streamCharlasConPlaza(HttpServletRequest request, @RequestParam("centroId") Long idCentro, HttpServletResponse response) throws IOException {
		//Parametros del método
	    Map<Integer, Object> parameters = new HashMap<Integer, Object>();
	    parameters.put(0, idCentro);
	    
	    Long idExcel = launchExcelComputation(request, "streamCharlasConPlaza", parameters);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}

	/**
	 * Stream charlas con los aplicados en el browser.
	 * @param response
	 *            response
	 * @param filtros
	 *            filtros del browser
	 * @param indPlazasLibres
	 *            filtro especial para las plazas libres            
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirCharlasBrowser")
	@Transactional(readOnly = true)
	public void streamCharlasBrowser(HttpServletRequest request, 
			@RequestParam("centroId") Long idCentro,
			@RequestParam("filtros") String filtros, 
			@RequestParam("indPlazasLibres") Long indPlazasLibres, 
			@RequestParam(value="formato", required=false) EDocumentFormats formato,
			HttpServletResponse response) throws IOException {
		
		//Parametros del método
	    Map<Integer, Object> parameters = new HashMap<Integer, Object>();
	    parameters.put(0, idCentro);
	    parameters.put(1, filtros);
	    parameters.put(2, indPlazasLibres);

	    //Establecemos el formato por defecto.
	    if(formato==null)formato=EDocumentFormats.XLS;
	    
	    Long idExcel = launchExcelComputation(request, "streamCharlasBrowser", parameters, formato);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}
	
	/**
	 * Stream charlas.
	 * 
	 * @param response
	 *            response
	 * @param idCentro
	 *            id centro
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirCharlas")
	@Transactional(readOnly = true)
	public void streamCharlas(HttpServletRequest request, 
			@RequestParam("centroId") Long idCentro, 
			@RequestParam("tematicaId") Long idTematica,
			@RequestParam("indIniciativaSingular") Long indIniciativaSingular,
			@RequestParam("indVideoconferencia") Long indVideoconferencia, 
			@RequestParam("fechaInicio") Long fechaInicio,
			@RequestParam("fechaFin") Long fechaFin, HttpServletResponse response) throws IOException {
		//Parametros del método
	    Map<Integer, Object> parameters = new HashMap<Integer, Object>();
	    parameters.put(0, idCentro);
	    parameters.put(1, idTematica);
	    parameters.put(2, indIniciativaSingular);
	    parameters.put(3, indVideoconferencia);
	    parameters.put(4, fechaInicio);
	    parameters.put(5, fechaFin);

	    Long idExcel = launchExcelComputation(request, "streamCharlas", parameters);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}

	/**
	 * Stream todo inventario.
	 * 
	 * @param response
	 *            response
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirTodoInventario")
	@Transactional(readOnly = true)
	public void streamTodoInventario(HttpServletRequest request, HttpServletResponse response) throws IOException {
	    Long idExcel = launchExcelComputation(request, "streamTodoInventario", null);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}

	/**
	 * Stream inventario.
	 * 
	 * @param idCentro
	 *            id centro
	 * @param idCategoria
	 *            id categoria
	 * @param estado
	 *            estado
	 * @param response
	 *            response
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirInventario")
	@Transactional(readOnly = true)
	public void streamInventario(HttpServletRequest request, @RequestParam("idCentro") long idCentro,
			@RequestParam("idCategoria") long idCategoria, @RequestParam("estado") EstadoRecurso estado,
			HttpServletResponse response) throws IOException {
		//Parametros del método
	    Map<Integer, Object> parameters = new HashMap<Integer, Object>();
	    parameters.put(0, idCentro);
	    parameters.put(1, idCategoria);
	    parameters.put(2, estado);
	    
	    Long idExcel = launchExcelComputation(request, "streamInventario", parameters);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}

	/**
	 * Stream todo incidencias.
	 * 
	 * @param response
	 *            response
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirTodoIncidencias")
	@Transactional(readOnly = true)
	public void streamTodoIncidencias(HttpServletRequest request, HttpServletResponse response) throws IOException {
	    Long idExcel = launchExcelComputation(request, "streamTodoIncidencias", null);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}

	/**
	 * Stream facturas.
	 * 
	 * @param idProyecto
	 *            id proyecto
	 * @param idSubproyecto
	 *            id subproyecto
	 * @param idProveedor
	 *            id proveedor
	 * @param idResponsableArea
	 *            id responsable area
	 * @param fechaInicioInMillis
	 *            fecha inicio in millis
	 * @param fechaFinInMillis
	 *            fecha fin in millis
	 * @param stringTipoEstadoFactura
	 *            string tipo estado factura
	 * @param stringEstadoDocumentacion
	 *            string estado documentacion
	 * @param response
	 *            response
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirFacturas")
	@Transactional(readOnly = true)
	public void streamFacturas(HttpServletRequest request, @RequestParam("idProyecto") long idProyecto,
			@RequestParam("idSubproyecto") long idSubproyecto, @RequestParam("idProveedor") long idProveedor,
			@RequestParam("idResponsableArea") long idResponsableArea, @RequestParam("fechaInicio") long fechaInicioInMillis,
			@RequestParam("fechaFin") long fechaFinInMillis, @RequestParam("tipoEstadoFactura") String stringTipoEstadoFactura,
			@RequestParam("estadoDocumentacion") String stringEstadoDocumentacion, HttpServletResponse response) throws IOException {
		//Parametros del método
	    Map<Integer, Object> parameters = new HashMap<Integer, Object>();
	    parameters.put(0, idProyecto);
	    parameters.put(1, idSubproyecto);
	    parameters.put(2, idProveedor);
	    parameters.put(3, idResponsableArea);
	    parameters.put(4, fechaInicioInMillis);
	    parameters.put(5, fechaFinInMillis);
	    parameters.put(6, stringTipoEstadoFactura);
	    parameters.put(7, stringEstadoDocumentacion);
	    
	    Long idExcel = launchExcelComputation(request, "streamFacturas", parameters);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}

	/**
	 * Stream incidencias.
	 * 
	 * @param idCentro
	 *            id centro
	 * @param criticidad
	 *            criticidad
	 * @param estado
	 *            estado
	 * @param idCategoriaIncidencia
	 *            id categoria incidencia
	 * @param response
	 *            response
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirIncidencias")
	@Transactional(readOnly = true)
	public void streamIncidencias(HttpServletRequest request, @RequestParam("idCentro") long idCentro,
			@RequestParam("criticidad") Criticidad criticidad, @RequestParam("estado") EstadoIncidencia estado,
			@RequestParam("idCategoria") long idCategoriaIncidencia, HttpServletResponse response) throws IOException {
		//Parametros del método
	    Map<Integer, Object> parameters = new HashMap<Integer, Object>();
	    parameters.put(0, idCentro);
	    parameters.put(1, criticidad);
	    parameters.put(2, estado);
	    parameters.put(3, idCategoriaIncidencia);
	    
	    Long idExcel = launchExcelComputation(request, "streamIncidencias", parameters);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}

	/**
	 * Stream asesoramientos.
	 * 
	 * @param response
	 *            response
	 * @param idCentro
	 *            id centro
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirAsesoramientos")
	@Transactional(readOnly = true)
	public void streamAsesoramientos(HttpServletRequest request, @RequestParam("centroId") Long idCentro,
			HttpServletResponse response) throws IOException {
		//Parametros del método
	    Map<Integer, Object> parameters = new HashMap<Integer, Object>();
	    parameters.put(0, idCentro);
	    
	    Long idExcel = launchExcelComputation(request, "streamAsesoramientos", parameters);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}

	/**
	 * Stream wifis.
	 * 
	 * @param response
	 *            response
	 * @param idCentro
	 *            id centro
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirWifis")
	@Transactional(readOnly = true)
	public void streamWifis(HttpServletRequest request, @RequestParam("centroId") Long idCentro, HttpServletResponse response) throws IOException {
		//Parametros del método
	    Map<Integer, Object> parameters = new HashMap<Integer, Object>();
	    parameters.put(0, idCentro);
	    
	    Long idExcel = launchExcelComputation(request, "streamWifis", parameters);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}

	/**
	 * Stream practicas talleres.
	 * 
	 * @param response
	 *            response
	 * @param idCentro
	 *            id centro
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirPracticasTalleres")
	@Transactional(readOnly = true)
	public void streamPracticasTalleres(HttpServletRequest request, 
			@RequestParam("centroId") Long idCentro, 
			@RequestParam("fechaInicio") Long fechaInicio,
			@RequestParam("fechaFin") Long fechaFin,
			HttpServletResponse response) throws IOException {
		//Parametros del método
	    Map<Integer, Object> parameters = new HashMap<Integer, Object>();
	    parameters.put(0, idCentro);
	    parameters.put(1, fechaInicio);
	    parameters.put(2, fechaFin);
	    
	    Long idExcel = launchExcelComputation(request, "streamPracticasTalleres", parameters);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}

	/**
	 * Stream PracticasTalleres con los aplicados en el browser.
	 * @param response
	 *            response
	 * @param filtros
	 *            filtros del browser
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirPracticasTalleresBrowser")
	@Transactional(readOnly = true)
	public void streamPracticasTalleresBrowser(HttpServletRequest request, 
			@RequestParam("centroId") Long idCentro,
			@RequestParam("filtros") String filtros, 
			@RequestParam(value="formato", required=false) EDocumentFormats formato,
			HttpServletResponse response) throws IOException {
		
		//Parametros del método
	    Map<Integer, Object> parameters = new HashMap<Integer, Object>();
	    parameters.put(0, idCentro);
	    parameters.put(1, filtros);

	    //Establecemos el formato por defecto.
	    if(formato==null)formato=EDocumentFormats.XLS;
	    
	    Long idExcel = launchExcelComputation(request, "streamPracticasTalleresBrowser", parameters, formato);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}
	
	/**
	 * Stream externas con plaza.
	 * 
	 * @param response
	 *            response
	 * @param idCentro
	 *            id centro
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirExternasConPlaza")
	@Transactional(readOnly = true)
	public void streamExternasConPlaza(HttpServletRequest request, @RequestParam("centroId") Long idCentro,
			@RequestParam("proveedorId") Long idProveedor,
			HttpServletResponse response) throws IOException {
		//Parametros del método
	    Map<Integer, Object> parameters = new HashMap<Integer, Object>();
	    parameters.put(0, idCentro);
	    parameters.put(1, idProveedor);
	    
	    Long idExcel = launchExcelComputation(request, "streamExternasConPlaza", parameters);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}

	/**
	 * Stream Externas con los aplicados en el browser.
	 * @param response
	 *            response
	 * @param filtros
	 *            filtros del browser
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirExternasBrowser")
	@Transactional(readOnly = true)
	public void streamExternasBrowser(HttpServletRequest request, 
			@RequestParam("centroId") Long idCentro,
			@RequestParam("filtros") String filtros, 
			@RequestParam(value="formato", required=false) EDocumentFormats formato,
			HttpServletResponse response) throws IOException {
		
		//Parametros del método
	    Map<Integer, Object> parameters = new HashMap<Integer, Object>();
	    parameters.put(0, idCentro);
	    parameters.put(1, filtros);

	    //Establecemos el formato por defecto.
	    if(formato==null)formato=EDocumentFormats.XLS;
	    
	    Long idExcel = launchExcelComputation(request, "streamExternasBrowser", parameters, formato);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}
	
	/**
	 * Stream externas.
	 * 
	 * @param response
	 *            response
	 * @param idCentro
	 *            id centro
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/servirExternas")
	@Transactional(readOnly = true)
	public void streamExternas(HttpServletRequest request, 
			@RequestParam("centroId") Long idCentro
			, @RequestParam("proveedorId") Long idProveedor
			, @RequestParam("tematicaId") Long idTematica
			, @RequestParam("indIniciativaSingular") Long indIniciativaSingular
			, @RequestParam("indVideoconferencia") Long indVideoconferencia 
			, @RequestParam("fechaInicio") Long fechaInicio
			, @RequestParam("fechaFin") Long fechaFin
			, HttpServletResponse response) throws IOException {
		//Parametros del método
	    Map<Integer, Object> parameters = new HashMap<Integer, Object>();
	    parameters.put(0, idCentro);
	    parameters.put(1, idProveedor);
	    parameters.put(2, idTematica);
	    parameters.put(3, indIniciativaSingular);
	    parameters.put(4, indVideoconferencia);
	    parameters.put(5, fechaInicio);
	    parameters.put(6, fechaFin);

	    Long idExcel = launchExcelComputation(request, "streamExternas", parameters);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}
	
	/**
	 * Stream externas.
	 * 
	 * @param response
	 *            response
	 * @param idCentro
	 *            id centro
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@RequestMapping("/exportReportDatos")
	@Transactional(readOnly = true)
	public void exportReportDatos(HttpServletRequest request, 
			@RequestParam("reportType") EReportType reportType,
			@RequestParam("fields") String fields,
			@RequestParam("centroId") Long centroId, 
			@RequestParam("tematicas") String tematicas, 
			@RequestParam("servicioId") Long servicioId,
			@RequestParam("activityType") EActivityTypes activityType,
			@RequestParam("fechaInicio") Long fechaInicio,
			@RequestParam("fechaFin") Long fechaFin,
			@RequestParam("realizada") Boolean realizada,
			@RequestParam("iniciativaSingular") Boolean iniciativaSingular,
			@RequestParam("videoconferencia") Boolean videoconferencia,
			@RequestParam("user") String user,
			@RequestParam("edadX") Integer edadX,
			@RequestParam("edadY") Integer edadY,
			@RequestParam("sexo") String sexo,
			@RequestParam("adaptacion") String adaptacion,
			@RequestParam("reportDatosType") EReportDatosTypes reportDatosType,
			@RequestParam("orden") String orden,
			HttpServletResponse response) throws IOException {
		
		//Parametros del método
	    Map<Integer, Object> parameters = new HashMap<Integer, Object>();
	    parameters.put(0, reportType);
	    parameters.put(1, fields);
	    parameters.put(2, centroId);
	    parameters.put(3, tematicas);
	    parameters.put(4, servicioId);
	    parameters.put(5, activityType);
	    parameters.put(6, fechaInicio);
	    parameters.put(7, fechaFin);
	    parameters.put(8, realizada);
	    parameters.put(9, iniciativaSingular);
	    parameters.put(10, videoconferencia);
	    parameters.put(11, user);
	    parameters.put(12, edadX);
	    parameters.put(13, edadY);
	    parameters.put(14, sexo);
	    parameters.put(15, adaptacion);
	    parameters.put(16, reportDatosType);
	    parameters.put(17, orden);
	    
	    Long idExcel = 0L;
	    
    	idExcel = launchExcelComputation(request, "streamGeneradorInformes", parameters);
	    
	    //Redirige a la página de espera y comprobación que informa del procesamiento
	    response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
	}

	private void makeHeaderXLS(HttpServletResponse response, String nombre) {
		response.setContentType("application/vnd.ms-excel");
		response.setHeader("Cache-Control", "no-store, no-cache, must-revalidate");
		response.addHeader("Cache-Control", "post-check=0, pre-check=0");
		response.setHeader("Pragma", "no-cache");
		response.setCharacterEncoding("ISO-8859-15");
		response.setHeader("Content-Disposition", "attachment;filename=\"" + nombre + ".xls\"");
	}
	
	private void makeHeaderCSV(HttpServletResponse response, String nombre) {
		response.setContentType("text/csv");
		response.setHeader("Cache-Control", "no-store, no-cache, must-revalidate");
		response.addHeader("Cache-Control", "post-check=0, pre-check=0");
		response.setHeader("Pragma", "no-cache");
		response.setCharacterEncoding("ISO-8859-15");
		response.setHeader("Content-Disposition", "attachment;filename=\"" + nombre + ".csv\"");
	}
	
	private static class ExcelComputationThread extends Thread {
		private ExcelComputation excelComputation;
		private ListingHelperXLSGenerator helper;

		public ExcelComputationThread(ExcelComputation excelComputation, ListingHelperXLSGenerator helper) {
			this.excelComputation = excelComputation;
			this.helper = helper;
		}

		@Override
		public void run() {
			try {
				excelComputation.setNombreFichero(helper.runGenerateXlsMethod(
						excelComputation.getMethod(),
						excelComputation.getParameters(),
						excelComputation.getLocale(),
						excelComputation.getOutputStream()));
			}
			catch (Exception e) {
				e.printStackTrace();
			}
			
			excelComputation.setFinishTimestamp(new Date().getTime());

			// Mientras no se consuma y no hallan pasado 10 minutos desde la
			// generación sin consumirse...
			while (!excelComputation.isConsumed()
					&& ((new Date().getTime() - excelComputation.getFinishTimestamp()) < 600000L)) {
				synchronized (this) {
					try {
						this.wait(30000L);
					} catch (Exception e) {
						e.printStackTrace();
					}
				}
			}

			// Una vez consumido o habiendo pasado 10 minutos sin consumir lo
			// eliminamos de los resultados
			synchronized (results) {
				results.remove(excelComputation.getIdExcel());
			}
		}
	}

	@RequestMapping("/isListingGenerated")
	@Transactional(readOnly = true)
	public void isListadoGenerated(@RequestParam("idExcel") String idExcel, HttpServletRequest request, HttpServletResponse response) throws Exception {
		ExcelComputation ec = null;

		if ((idExcel == null) || idExcel.isEmpty() || ((ec = results.get(Long.parseLong(idExcel))) == null))
			response.sendRedirect(request.getContextPath());
		else {
			if (ec.getFinishTimestamp() > -1L)
				response.sendRedirect(request.getContextPath() + "/spring/listings/download?idExcel=" + idExcel);
			else
				response.sendRedirect(request.getContextPath() + "/spring/listings/processing?idExcel=" + idExcel);
		}
	}
	
	@RequestMapping("/getListing")
	@Transactional(readOnly = true)
	public void getListado(@RequestParam("idExcel") String idExcel, HttpServletRequest request, HttpServletResponse response) throws Exception {
		if ((idExcel == null) || idExcel.isEmpty())
			response.sendRedirect(request.getContextPath());
		else {
			ExcelComputation excelComputation = results.get(Long.parseLong(idExcel));

			if ((excelComputation != null)
					&& (excelComputation.getNombreFichero() != null)
					&& !excelComputation.getNombreFichero().isEmpty()
					&& (excelComputation.getOutputStream().size() > 0)) {
				
		    	if (excelComputation.getFormato() == EDocumentFormats.XLS){
		    	    makeHeaderXLS(response, excelComputation.getNombreFichero());
		    	}
		    	else if (excelComputation.getFormato() == EDocumentFormats.CSV){
		    	    makeHeaderCSV(response, excelComputation.getNombreFichero());
		    	}
				
				excelComputation.getOutputStream().writeTo(response.getOutputStream());
				
				excelComputation.getOutputStream().flush();
				excelComputation.getOutputStream().close();
				
				response.getOutputStream().flush();
				response.getOutputStream().close();
				
				response.flushBuffer();
				
				excelComputation.setToConsumed();

				synchronized (excelComputation.getThread()) {
					excelComputation.getThread().notify();
				}
			}
			// Error en la generación del listado
			else {
				if (excelComputation != null)
					response.sendRedirect(request.getContextPath() + "/spring/listings/listingError");
				else
					response.sendRedirect(request.getContextPath());
			}
		}
	}
}
