package es.csa.fdt.cyldigital.controller;

/*
 * #%L
 * Librerías del núcleo para Web - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.sql.SQLException;

import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.FileCopyUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

import es.csa.fdt.cyldigital.core.entity.File;
import es.csa.fdt.cyldigital.core.entity.FileOnDisk;
import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.core.service.FileService;
import es.csa.fdt.cyldigital.financiero.entity.FileFactura;
import es.csa.fdt.cyldigital.financiero.entity.FileMovimiento;

/**
 * Clase FileController.
 */
@Controller
public class FileController {

  @Autowired
  private FileService service;

  /**
   * Servir file.
   * 
   * @param idFile
   *           id file
   * @param response
   *           response
   * @throws IOException
   *           Signals that an I/O exception has occurred.
   * @throws SQLException
   *           the sQL exception
   */
  @RequestMapping("/obtenerFichero")
  public void servirFile(@RequestParam("id") String idFile, HttpServletResponse response) throws IOException,
      SQLException {
    System.out.println("obteniendo");
    File f = service.find(Long.parseLong(idFile));
    if (f != null) {
      response.setHeader("Content-Disposition", "attachment; filename=" + f.getNombre().replace(' ', '_'));
      response.setContentType(f.getTipoMime());
      response.setContentLength(f.getTamanio().intValue());
      FileCopyUtils.copy(f.getContenido().getBinaryStream(), response.getOutputStream());
      response.getOutputStream().flush();
    }
  }

  /**
   * Servir file factura.
   * 
   * @param idFile
   *           id file
   * @param response
   *           response
   * @throws IOException
   *           Signals that an I/O exception has occurred.
   * @throws SQLException
   *           the sQL exception
   */
  @RequestMapping("/obtenerFicheroFactura")
  public void servirFileFactura(@RequestParam("id") String idFile, HttpServletResponse response) throws IOException,
      SQLException {
    System.out.println("obteniendo");
    FileFactura f = service.find(FileFactura.class, Long.parseLong(idFile));
    if (f != null) {
      response.setHeader("Content-Disposition", "attachment; filename=" + f.getNombre().replace(' ', '_'));
      response.setContentType(f.getTipoMime());
      response.setContentLength(f.getTamanio().intValue());
      FileCopyUtils.copy(f.getContenido().getBinaryStream(), response.getOutputStream());
      response.getOutputStream().flush();
    }
  }

  /**
   * Servir file movimiento.
   * 
   * @param idFile
   *           id file
   * @param response
   *           response
   * @throws IOException
   *           Signals that an I/O exception has occurred.
   * @throws SQLException
   *           the sQL exception
   */
  @RequestMapping("/obtenerFicheroMovimiento")
  public void servirFileMovimiento(@RequestParam("id") String idFile, HttpServletResponse response) throws IOException,
      SQLException {
    System.out.println("obteniendo");
    FileMovimiento f = service.find(FileMovimiento.class, Long.parseLong(idFile));
    if (f != null) {
      response.setHeader("Content-Disposition", "attachment; filename=" + f.getNombre().replace(' ', '_'));
      response.setContentType(f.getTipoMime());
      response.setContentLength(f.getTamanio().intValue());
      FileCopyUtils.copy(f.getContenido().getBinaryStream(), response.getOutputStream());
      response.getOutputStream().flush();
    }
  }

  /**
   * Servir file on disk.
   * 
   * @param idFile
   *           id file
   * @param response
   *           response
   * @throws IOException
   *           Signals that an I/O exception has occurred.
   * @throws SQLException
   *           the sQL exception
   */
  @RequestMapping("/obtenerFileOnDisk")
  public void servirFileOnDisk(@RequestParam("id") String idFile, HttpServletResponse response) throws IOException,
      SQLException {
    System.out.println("obteniendo file on disk");

    FileOnDisk f = service.find(FileOnDisk.class, Long.parseLong(idFile));
    if (f != null) {

      Parameter p = service.findOneWhere(Parameter.class, "nombre", "pathfileondisk");
      if (p == null) {
        System.out.println("Falta configurar el parámetro fileondiskpath en la tabla Parameters");
      } else {
        java.io.File file = new java.io.File(p.getValor() + f.getNombre());
        InputStream in = new FileInputStream(file);

        response.setHeader("Content-Disposition", "attachment; filename=" + f.getNombreOriginal().replace(' ', '_'));
        response.setContentType(f.getTipoMime());
        response.setContentLength(f.getTamanio().intValue());
        FileCopyUtils.copy(in, response.getOutputStream());
        response.getOutputStream().flush();
        in.close();
      }
    }
  }
}
