package es.csa.fdt.cyldigital.controller;

/*
 * #%L
 * Librerías del núcleo para Web - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.HashMap;

import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.FileCopyUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;

import com.lowagie.text.pdf.Barcode;
import com.lowagie.text.pdf.Barcode39;

import es.csa.commons.util.IOUtils;
import es.csa.fdt.cyldigital.core.entity.FileOnDisk;
import es.csa.fdt.cyldigital.core.entity.FilePanel;
import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.usuarios.entity.Fichero;
import es.csa.fdt.cyldigital.usuarios.service.ImagenService;
import es.csa.fdt.cyldigital.util.ImageUtils;

/**
 * Clase ImageControllerImpl.
 */
@Controller
public class ImageControllerImpl {

  private static final long serialVersionUID = -4581725741365803463L;

  @Autowired
  private ImagenService service;

  @Autowired
  private transient HashMap<String, Object> generalProperties;

  /**
   * Construye un nuevo objeto de la clase
   */
  public ImageControllerImpl() {}

  /**
   * Stream image content.
   * 
   * @param idUsuario
   *           id usuario
   * @param response
   *           response
   * @throws IOException
   *           Signals that an I/O exception has occurred.
   */
  @RequestMapping("/servirImagen")
  public void streamImageContent(@RequestParam("idUsuario") String idUsuario, HttpServletResponse response)
      throws IOException {
    writeImageOutputStream(response, getFilePath("USER_PHOTO_PATH") + getfileName(idUsuario), "image/jpeg");
  }

  /**
   * Stream image content anuncio.
   * 
   * @param idFilePanel
   *           id file panel
   * @param response
   *           response
   * @throws IOException
   *           Signals that an I/O exception has occurred.
   */
  @RequestMapping("/servirImagenAnuncioPanel")
  public void streamImageContentAnuncio(@RequestParam("idFilePanel") String idFilePanel, HttpServletResponse response)
      throws IOException {

    Parameter p = service.findOneWhere(Parameter.class, "nombre", "pathfileOnDisk");
    FilePanel f = service.find(FilePanel.class, new Long(idFilePanel));
    FileOnDisk file = service.find(FileOnDisk.class, f.getFile().getId());// hay que accederlo así por el java lazy...

    writeImageOutputStream(response, p.getValor() + file.getNombre(), file.getTipoMime());
  }

  /**
   * Stream image thumb content.
   * 
   * @param idUsuario
   *           id usuario
   * @param response
   *           response
   * @throws IOException
   *           Signals that an I/O exception has occurred.
   */
  @RequestMapping("/servirImagenMini")
  public void streamImageThumbContent(@RequestParam("idUsuario") String idUsuario, HttpServletResponse response)
      throws IOException {
    writeImageOutputStream(response, getFilePath("USER_PHOTO_PATH_THUMB") + getfileName(idUsuario), "image/jpeg");
  }

  /**
   * Stream image tmp content.
   * 
   * @param idUsuario
   *           id usuario
   * @param response
   *           response
   * @throws IOException
   *           Signals that an I/O exception has occurred.
   */
  @RequestMapping("/servirImagenTmp")
  public void streamImageTmpContent(@RequestParam("idUsuario") String idUsuario, HttpServletResponse response)
      throws IOException {
    writeImageOutputStream(response, getFilePath("USER_PHOTO_PATH_TMP") + getfileName(idUsuario), "image/jpeg");
  }

  private void writeImageOutputStream(HttpServletResponse response, String path, String tipoMime) throws IOException,
      FileNotFoundException {
    File f = new File(path);

    if (f.exists()) {
      response.setContentLength(new Long(f.length()).intValue());
      response.setContentType(tipoMime);
      response.setHeader("Cache-Control", "no-store, no-cache, must-revalidate");
      response.addHeader("Cache-Control", "post-check=0, pre-check=0");
      response.setHeader("Pragma", "no-cache");
      FileCopyUtils.copy(new FileInputStream(f), response.getOutputStream());
    }
  }

  private String getfileName(String idUsuario) {
    String nombre = idUsuario + ".jpg";
    return nombre;
  }

  /**
   * Process cam image upload.
   * 
   * @param idUsuario
   *           id usuario
   * @param img
   *           img
   * @param response
   *           response
   * @throws IOException
   *           Signals that an I/O exception has occurred.
   */
  @RequestMapping("/imageCamUpload")
  public void processCamImageUpload(@RequestParam("idusuario") String idUsuario, @RequestParam("img") String img,
      HttpServletResponse response) throws IOException {

    if (img != null && !img.isEmpty()) {
      String nombre = getfileName(idUsuario);
      FileOutputStream fileOutputStream = new FileOutputStream(getFilePath("USER_PHOTO_PATH_TMP") + nombre);
      ImageIO.write(ImageUtils.getImage(180, 240, img), "jpg", fileOutputStream);
    } else {
      System.out.println("No llega imagen");
    }
    response.sendError(404);
  }

  private String getFilePath(String key) {
    return generalProperties.get("FILE_PATH") + "" + generalProperties.get(key);
  }

  /**
   * Process image upload.
   * 
   * @param idUsuario
   *           id usuario
   * @param img
   *           img
   * @return the string
   * @throws IOException
   *           Signals that an I/O exception has occurred.
   */
  @RequestMapping("/imageUpload")
  public String processImageUpload(@RequestParam("idusuario") String idUsuario, @RequestParam("img") MultipartFile img)
      throws IOException {
    String nombre = getfileName(idUsuario);
    Fichero m = new Fichero();
    m.setNombre(nombre);
    m.setTamanio((int) img.getSize());
    m.setTipoMime(img.getContentType());
    FileCopyUtils.copy(img.getInputStream(), new FileOutputStream(getFilePath("USER_PHOTO_PATH_TMP") + nombre));
    service.setMessagesEnabled(false);
    service.persist(m);
    service.setMessagesEnabled(true);
    return "redirect:usuarios";
  }

  /**
   * Devuelve imprimir code bar.
   * 
   * @param code
   *           code
   * @param response
   *           response
   * @return the imprimir code bar
   * @throws IOException
   *           Signals that an I/O exception has occurred.
   */
  @RequestMapping("/imageCodeBar")
  public void getImprimirCodeBar(@RequestParam("codEtiqueta") String code, HttpServletResponse response)
      throws IOException {

    response.setContentType("image/png");
    response.setHeader("Cache-Control", "no-store, no-cache, must-revalidate");
    response.addHeader("Cache-Control", "post-check=0, pre-check=0");
    response.setHeader("Pragma", "no-cache");
    IOUtils.copy(new ByteArrayInputStream(getBarcode(code)), response.getOutputStream());
  }

  private byte[] getBarcode(String code) {
    ByteArrayOutputStream result = new ByteArrayOutputStream();
    Barcode39 barcode = new Barcode39();
    barcode.setCodeType(Barcode.CODE128);
    barcode.setGenerateChecksum(true);
    barcode.setChecksumText(true);
    barcode.setCode(code);
    barcode.setBarHeight(40);
    Image bc = barcode.createAwtImage(new Color(0x00, 0x0, 0x00), new Color(0xFF, 0xFF, 0xFF));
    try {
      BufferedImage outImage = new BufferedImage(bc.getWidth(null), 40 + 15, BufferedImage.TYPE_INT_ARGB);

      Graphics2D g2d = outImage.createGraphics();
      Font font = new Font("Arial", Font.BOLD, 12);

      FontMetrics fontMetrics = g2d.getFontMetrics(font);
      int stringWidth = fontMetrics.stringWidth(code);
      int stringHeight = fontMetrics.getAscent();

      g2d.setFont(font);
      g2d.setPaint(Color.black);

      g2d.drawImage(bc, 0, 0, bc.getWidth(null), 40, null);
      g2d.drawString(code, (bc.getWidth(null) - stringWidth) / 2, 40 + stringHeight);
      g2d.dispose();

      ImageIO.write(outImage, "PNG", result);

    } catch (IOException e) {
      System.out.println("Erro: " + e.getMessage());
      e.printStackTrace();
    }
    return result.toByteArray();
  }
}
