package es.csa.fdt.cyldigital.controller;

/*
 * #%L
 * Librerías del núcleo para Web - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import com.lowagie.text.Document;
import com.lowagie.text.DocumentException;
import com.lowagie.text.Font;

import static com.lowagie.text.Font.BOLD;
import static com.lowagie.text.Font.DEFAULTSIZE;
import static com.lowagie.text.Font.ITALIC;
import static com.lowagie.text.Font.NORMAL;
import com.lowagie.text.FontFactory;
import com.lowagie.text.pdf.BaseFont;
import com.lowagie.text.pdf.PdfContentByte;
import static com.lowagie.text.pdf.PdfContentByte.ALIGN_CENTER;
import static com.lowagie.text.pdf.PdfContentByte.ALIGN_LEFT;
import com.lowagie.text.pdf.PdfCopy;
import com.lowagie.text.pdf.PdfReader;
import com.lowagie.text.pdf.PdfStamper;

import java.awt.Color;
import java.awt.image.BufferedImage;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import java.text.DateFormat;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;

import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;

import javax.imageio.ImageIO;

import javax.servlet.http.HttpServletResponse;

import org.apache.commons.io.IOUtils;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.security.Authentication;
import org.springframework.security.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

import es.csa.fdt.cyldigital.diplomas.service.DiplomaService;

import es.csa.fdt.cyldigital.language.entity.ServicioCursoLanguage;
import es.csa.fdt.cyldigital.language.entity.ServicioLanguage;

import es.csa.fdt.cyldigital.servicios.entity.Curso;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaTaller;
import es.csa.fdt.cyldigital.servicios.entity.Servicio;
import es.csa.fdt.cyldigital.servicios.entity.ServicioCurso;
import es.csa.fdt.cyldigital.servicios.entity.ServicioTaller;
import es.csa.fdt.cyldigital.servicios.entity.Taller;
import es.csa.fdt.cyldigital.servicios.service.MatriculaCursoService;
import es.csa.fdt.cyldigital.servicios.service.MatriculaTallerService;

import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario.Role;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;

/**
 * Clase PdfControllerImpl.
 */
@Controller
public class PdfControllerImpl {

  private static final Logger logger = LoggerFactory.getLogger(PdfControllerImpl.class);

  private static final long serialVersionUID = -4581725741365803463L;

  private static Map<String, Object[]> CARACTERISTICAS_DIPLOMA;

  private static Map<String, Object[]> CARACTERISTICAS_GUION;

  private static int MAX_WIDTH = 650;

  public static final String DIPLOMA_PDF = "es/csa/fdt/cyldigital/pdf/diploma_completo.pdf";

  public static final String GUION_PDF = "es/csa/fdt/cyldigital/pdf/diploma_guion.pdf";

  public static final String DIPLOMA_PDF_VACIO = "es/csa/fdt/cyldigital/pdf/diploma_vacio.pdf";

  public static final String GUION_PDF_VACIO = "es/csa/fdt/cyldigital/pdf/diploma_vacio.pdf";

  public static final boolean DIPLOMA_CON_PLANTILLA = false;

  public static final boolean GUION_CON_PLANTILLA = false;

  public static final Font ARIAL_NORMAL = FontFactory.getFont("es/csa/fdt/cyldigital/pdf/Arial.ttf", Font.BOLD);

  public static final Font HELVETICA_NORMAL = FontFactory.getFont(FontFactory.HELVETICA, DEFAULTSIZE, NORMAL);

  public static final Font HELVETICA_BOLD = FontFactory.getFont(FontFactory.HELVETICA, DEFAULTSIZE, BOLD);

  public static final Font HELVETICA_I = FontFactory.getFont(FontFactory.HELVETICA, DEFAULTSIZE, ITALIC);

  public static final Font COURIER_NORMAL = FontFactory.getFont(FontFactory.COURIER, DEFAULTSIZE, NORMAL);

  @Autowired
  private MatriculaCursoService serviceCurso;

  @Autowired
  private MatriculaTallerService serviceTaller;
  
  @Autowired
  private DiplomaService serviceDiploma;

  @Autowired
  public es.csa.commons.faces.util.PredefinedLocaleResolver prl;
  
  public String idioma;

  static {

    Object[][] posicionesDiploma = { { HELVETICA_BOLD, 22.0f, ALIGN_CENTER, 425f, 335f, null }, // nombreUsuario
        { HELVETICA_NORMAL, 16f, ALIGN_LEFT, 110f, 303f, null }, // descripcion
        { HELVETICA_I, 12.5f, ALIGN_CENTER, 417f, 215.5f, null }, // fecha
    };
    Object[][] posicionesGuion = { { HELVETICA_NORMAL, 22.0f, ALIGN_LEFT, 425f, 335f, null }, // guion
    };
    CARACTERISTICAS_DIPLOMA = new HashMap<String, Object[]>();
    CARACTERISTICAS_DIPLOMA.put("nombreUsuario", posicionesDiploma[0]);
    CARACTERISTICAS_DIPLOMA.put("descripcion", posicionesDiploma[1]);
    CARACTERISTICAS_DIPLOMA.put("fecha", posicionesDiploma[2]);

    CARACTERISTICAS_GUION = new HashMap<String, Object[]>();
    CARACTERISTICAS_GUION.put("guion", posicionesGuion[0]);

  }

  /**
   * Construye un nuevo objeto de la clase
   */
  public PdfControllerImpl() {}

  /**
   * Stream diploma curso.
   * 
   * @param idUsuario
   *           id usuario
   * @param response
   *           response
   * @throws IOException
   *           Signals that an I/O exception has occurred.
   */
  @RequestMapping("/servirDiplomaCurso")
  @Transactional(readOnly = true)
  public void streamDiplomaCurso(@RequestParam("idMatricula") String idUsuario, HttpServletResponse response)
      throws IOException {
	String nombreDiplomaCurso = "";
    String identificacion = "";

    if (idUsuario != null && !idUsuario.trim().isEmpty()) {
      long id = Long.parseLong(idUsuario);
      MatriculaCurso m = serviceCurso.find(id);
      UsuarioCyL u = serviceCurso.find(UsuarioCyL.class, m.getUsuario().getId());

      if (u.getPasaporte() != null && !u.getPasaporte().isEmpty()) identificacion = " Pasaporte: " + u.getPasaporte();
      else if (m.getUsuario().getNif() != null && !m.getUsuario().getNif().isEmpty()) identificacion = " NIF:"
          + m.getUsuario().getNif();
      else {
        if (u.getNombreCompletoAutorizado() != null && !u.getNombreCompletoAutorizado().isEmpty()
            && u.getNifAutorizado() != null && !u.getNifAutorizado().isEmpty()) identificacion = " NIF autorizador: "
            + u.getNifAutorizado();
        else identificacion += " Sin identificacion ";
      }

      if (m != null) {
        try {
        	/*Cambiado
             * TODO: verificar su funcionamiento /*+ m.getCurso().getServicio().getNombre().replace("\"", "'")*/
             
          nombreDiplomaCurso = GetServicioNombre(m.getCurso().getServicio());    
          makeHeader(response, "Diploma - " + nombreDiplomaCurso.replace("\"", "'") + " -ID: "
              + m.getUsuario().getId() + identificacion + m.getUsuario().getNombreCompleto());

          if (DIPLOMA_CON_PLANTILLA) IOUtils.copy(new ByteArrayInputStream(getPdfDiplomaCurso(m, u, DIPLOMA_PDF)
              .toByteArray()), response.getOutputStream());

          else IOUtils.copy(new ByteArrayInputStream(getPdfDiplomaCurso(m, u, DIPLOMA_PDF_VACIO).toByteArray()),
              response.getOutputStream());

        } catch (RuntimeException e) {
          e.printStackTrace();
        }
      }
    }
  }

  /**
   * Stream diplomas curso.
   * 
   * @param idCurso
   *           id curso
   * @param response
   *           response
   * @throws IOException
   *           Signals that an I/O exception has occurred.
   */
  @RequestMapping("/servirDiplomasCurso")
  @Transactional(readOnly = true)
  public void streamDiplomasCurso(@RequestParam("idCurso") String idCurso, HttpServletResponse response)
      throws IOException {
	String nombreDiplomasCurso = "";
    PdfReader diploma;

    if (idCurso != null && !idCurso.trim().isEmpty()) {
      long id = Long.parseLong(idCurso);
      Curso curso = serviceCurso.find(Curso.class, id);
      UsuarioCyL u;

      if (curso != null) {
        try {
        	/*Cambiado
             * TODO: verificar su funcionamiento /*+ curso.getServicio().getNombre().replace("\"", "'")*/
        	
          nombreDiplomasCurso = GetServicioNombre(curso.getServicio());    
          makeHeader(response, "Diplomas - " + nombreDiplomasCurso.replace("\"", "'"));
          int i = 0;

          ByteArrayOutputStream resultado = new ByteArrayOutputStream();
          for (MatriculaCurso m : curso.getMatriculas()) {
            u = serviceCurso.find(UsuarioCyL.class, m.getUsuario().getId());
            if (m.getAsistencia() != null && m.getAsistencia() >= 80) {
              if (DIPLOMA_CON_PLANTILLA) diploma = new PdfReader(new ByteArrayInputStream(getPdfDiplomaCurso(m, u,
                  DIPLOMA_PDF).toByteArray()));
              else diploma = new PdfReader(new ByteArrayInputStream(getPdfDiplomaCurso(m, u, DIPLOMA_PDF_VACIO)
                  .toByteArray()));

              if (i > 0) {
                resultado = mergePdfsCompletos(resultado, diploma);
              }

              if (i > 0) {
                // resultado = mergePdfsCompletos(resultado,
                // guion);
              } else {
                if (DIPLOMA_CON_PLANTILLA) IOUtils.copy(new ByteArrayInputStream(getPdfDiplomaCurso(m, u, DIPLOMA_PDF)
                    .toByteArray()), resultado);
                else IOUtils.copy(new ByteArrayInputStream(getPdfDiplomaCurso(m, u, DIPLOMA_PDF_VACIO).toByteArray()),
                    resultado);

              }

              i++;
            }
          }
          IOUtils.copy(new ByteArrayInputStream(resultado.toByteArray()), response.getOutputStream());

        } catch (RuntimeException e) {
          e.printStackTrace();
        }
      }
    }
  }

  /**
   * Stream diplomas curso.
   * 
   * @param response
   *           response
   * @throws IOException
   *           Signals that an I/O exception has occurred.
   */
  @RequestMapping("/servirDiplomas")
  @Transactional(readOnly = true)
  public void streamDiplomasCurso(HttpServletResponse response) throws IOException {
    PdfReader diploma;
    UsuarioCyL u;

    List<MatriculaCurso> matriculas = serviceCurso
        .findByQuery(MatriculaCurso.class,
            "Select m from MatriculaCurso m order by m.usuario.nombre ASC, m.usuario.apellido1 ASC, m.usuario.apellido2 ASC");

    if (matriculas != null) {
      try {
        makeHeader(response, "Diplomas hasta la fecha ");
        int i = 0;

        ByteArrayOutputStream resultado = new ByteArrayOutputStream();
        for (MatriculaCurso m : matriculas) {
          u = serviceCurso.find(UsuarioCyL.class, m.getUsuario().getId());
          if (m.getAsistencia() != null && m.getAsistencia() >= 80) {
            if (DIPLOMA_CON_PLANTILLA) diploma = new PdfReader(new ByteArrayInputStream(getPdfDiplomaCurso(m, u,
                DIPLOMA_PDF).toByteArray()));
            else diploma = new PdfReader(new ByteArrayInputStream(getPdfDiplomaCurso(m, u, DIPLOMA_PDF_VACIO)
                .toByteArray()));

            if (i > 0) {
              resultado = mergePdfsCompletos(resultado, diploma);
            }
            if (i > 0) {} else {
              if (DIPLOMA_CON_PLANTILLA) IOUtils.copy(new ByteArrayInputStream(getPdfDiplomaCurso(m, u, DIPLOMA_PDF)
                  .toByteArray()), resultado);
              else IOUtils.copy(new ByteArrayInputStream(getPdfDiplomaCurso(m, u, DIPLOMA_PDF_VACIO).toByteArray()),
                  resultado);
            }
            i++;
          }
        }
        IOUtils.copy(new ByteArrayInputStream(resultado.toByteArray()), response.getOutputStream());
      } catch (RuntimeException e) {
        e.printStackTrace();
      }
    }
  }

  /**
   * Stream diplomas taller.
   * 
   * @param idTaller
   *           id taller
   * @param response
   *           response
   * @throws IOException
   *           Signals that an I/O exception has occurred.
   */
  @RequestMapping("/servirDiplomasTaller")
  @Transactional(readOnly = true)
  public void streamDiplomasTaller(@RequestParam("idTaller") String idTaller, HttpServletResponse response)
      throws IOException {
	String nombreDiplomasTaller = "";
    PdfReader diploma;
    UsuarioCyL u;

    if (idTaller != null && !idTaller.trim().isEmpty()) {
      long id = Long.parseLong(idTaller);
      Taller taller = serviceTaller.find(Taller.class, id);

      if (taller != null) {
        try {
        	/*Cambiado
             * TODO: verificar su funcionamiento /*+ taller.getServicio().getNombre().replace("\"", "'")*/
          
          nombreDiplomasTaller = GetServicioNombre(taller.getServicio());
          makeHeader(response, "Diplomas - " + nombreDiplomasTaller.replace("\"", "'"));
          int i = 0;

          ByteArrayOutputStream resultado = new ByteArrayOutputStream();
          for (MatriculaTaller m : taller.getMatriculas()) {
            u = serviceTaller.find(UsuarioCyL.class, m.getUsuario().getId());

            if (m.getAsistencia() != null && m.getAsistencia() >= 80) {
              if (DIPLOMA_CON_PLANTILLA) diploma = new PdfReader(new ByteArrayInputStream(getPdfDiplomaTaller(m, u,
                  DIPLOMA_PDF).toByteArray()));
              else diploma = new PdfReader(new ByteArrayInputStream(getPdfDiplomaTaller(m, u, DIPLOMA_PDF_VACIO)
                  .toByteArray()));

              if (i > 0) {
                resultado = mergePdfsCompletos(resultado, diploma);
              }

              if (i > 0) {
                // resultado = mergePdfsCompletos(resultado,
                // guion);
              } else {
                if (DIPLOMA_CON_PLANTILLA) IOUtils.copy(new ByteArrayInputStream(getPdfDiplomaTaller(m, u, DIPLOMA_PDF)
                    .toByteArray()), resultado);
                else IOUtils.copy(new ByteArrayInputStream(getPdfDiplomaTaller(m, u, DIPLOMA_PDF_VACIO).toByteArray()),
                    resultado);
              }
              i++;
            }
          }
          IOUtils.copy(new ByteArrayInputStream(resultado.toByteArray()), response.getOutputStream());
        } catch (RuntimeException e) {
          // TODO Auto-generated catch block
          e.printStackTrace();
        }
      }
    }
  }

  /**
   * Stream diploma taller.
   * 
   * @param idUsuario
   *           id usuario
   * @param response
   *           response
   * @throws IOException
   *           Signals that an I/O exception has occurred.
   */
  @RequestMapping("/servirDiplomaTaller")
  @Transactional(readOnly = true)
  public void streamDiplomaTaller(@RequestParam("idMatricula") String idUsuario, HttpServletResponse response)
      throws IOException {
	String nombreDiplomaTaller = "";
    UsuarioCyL u;
    String identificacion = "";

    if (idUsuario != null && !idUsuario.trim().isEmpty()) {
      long id = Long.parseLong(idUsuario);
      MatriculaTaller m = serviceTaller.find(id);
      u = serviceTaller.find(UsuarioCyL.class, m.getUsuario().getId());

      if (m != null) {

        if (u.getPasaporte() != null && !u.getPasaporte().isEmpty()) identificacion = " Pasaporte: " + u.getPasaporte();
        else if (m.getUsuario().getNif() != null && !m.getUsuario().getNif().isEmpty()) identificacion = " NIF:"
            + m.getUsuario().getNif();
        else {
          if (u.getNombreCompletoAutorizado() != null && !u.getNombreCompletoAutorizado().isEmpty()
              && u.getNifAutorizado() != null && !u.getNifAutorizado().isEmpty()) identificacion = " NIF autorizador: "
              + u.getNifAutorizado();
          else identificacion += " Sin identificacion ";
        }

        try {
        	/*Cambiado
             * TODO: verificar su funcionamiento /*+ m.getTaller().getServicio().getNombre().replace("\"", "'")*/
        
          nombreDiplomaTaller = GetServicioNombre(m.getTaller().getServicio());
          makeHeader(response, "Diploma - " + nombreDiplomaTaller.replace("\"", "'") + " -ID: "
              + m.getUsuario().getId() + identificacion + " " + m.getUsuario().getNombreCompleto());

          if (DIPLOMA_CON_PLANTILLA) IOUtils.copy(new ByteArrayInputStream(getPdfDiplomaTaller(m, u, DIPLOMA_PDF)
              .toByteArray()), response.getOutputStream());
          else IOUtils.copy(new ByteArrayInputStream(getPdfDiplomaTaller(m, u, DIPLOMA_PDF_VACIO).toByteArray()),
              response.getOutputStream());

        } catch (RuntimeException e) {
          // TODO Auto-generated catch block
          e.printStackTrace();
        }
      }
    }
  }
  
  /**
   * Stream Diploma Template Preview
   * 
   * @param idDiploma
   *           id of diploma template
   * @param response
   *           response
   * @throws IOException
   *           Signals that an I/O exception has occurred.
   */
  @RequestMapping("/getDiplomaTemplatePreview")
  @Transactional(readOnly = true)
  public void streamDiplomaTemplatePreview(@RequestParam("idDiploma") String idDiploma, HttpServletResponse response)
  										   throws IOException {
	  if ((idDiploma != null) && !idDiploma.trim().isEmpty()) {
		  long id = Long.parseLong(idDiploma);

	      //Get image from PDF
	      BufferedImage bufferedImage = serviceDiploma.getImageFromPDF(id);

	      //Response
	      response.setContentType("image/jpeg");
	      ImageIO.write(bufferedImage, "jpg", response.getOutputStream());
	      response.getOutputStream().flush();
	      response.getOutputStream().close();
	  }
  }
  
  @RequestMapping("/emitirDiplomaCurso")
  @Transactional(readOnly = true)
  public void streamEmitirDiplomaCurso(@RequestParam("idMatricula") String idMatriculaString,
		  							   HttpServletResponse response) {
	  List<Role> rolesUser = ((UsuarioLdap)SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUser().getRolesList();
	  
	  if ((idMatriculaString != null) && !idMatriculaString.trim().isEmpty()) {
		  String idioma = LocaleContextHolder.getLocaleContext().getLocale().toString();
		  
		  Long idMatricula = Long.parseLong(idMatriculaString);
		  
		  MatriculaCurso mc = serviceCurso.find(idMatricula);
		  
		  Boolean canIssue = false;
		  List<Role> rolesDiplomaServicio = getRolesDiplomaServicio(mc.getCurso().getServicio());
		  for (Role r : rolesDiplomaServicio) {
			  if (rolesUser.contains(r)) {
				  canIssue = true;
				  break;
			  }
		  }
		  
		  //Response
		  if (canIssue) {
			  String nombreServicioCurso = GetServicioNombre(mc.getCurso().getServicio());    
	          makeHeader(response,
	        		  	 "Diploma - " + nombreServicioCurso.replace("\"", "'") + " - ID: " +
	        		  	 mc.getUsuario().getId() + " - " + mc.getUsuario().getNombreCompleto());
		  }

          try {
    		  if (canIssue) {
    			  IOUtils.copy(new ByteArrayInputStream(serviceDiploma.issueDiploma(mc, idioma).toByteArray()),
    					  	   response.getOutputStream());
    		  }
	
			  response.getOutputStream().flush();
		      response.getOutputStream().close();
          }
          catch (Exception e) {
        	  e.printStackTrace();
          }
	  }
  }

  /**
   * Devuelve pdf diploma curso.
   * 
   * @param matricula
   *           matricula
   * @param u
   *           u
   * @param pdf
   *           pdf
   * @return the pdf diploma curso
   */
  public ByteArrayOutputStream getPdfDiplomaCurso(MatriculaCurso matricula, UsuarioCyL u, String pdf) {
    ByteArrayOutputStream pdf_diploma = new ByteArrayOutputStream();
    try {
      PdfReader reader;
      reader = new PdfReader(pdf);
      pdf_diploma = generarPDF(CARACTERISTICAS_DIPLOMA, getDatosDiplomaCurso(matricula, u), reader);
    } catch (IOException e) {
      logger.error("Error al generar diploma de curso, no se encuentra la plantilla de pdf");
    }
    return pdf_diploma;
  }

  /**
   * Devuelve pdf diploma taller.
   * 
   * @param matricula
   *           matricula
   * @param u
   *           u
   * @param pdf
   *           pdf
   * @return the pdf diploma taller
   */
  public ByteArrayOutputStream getPdfDiplomaTaller(MatriculaTaller matricula, UsuarioCyL u, String pdf) {
    ByteArrayOutputStream pdf_diploma = new ByteArrayOutputStream();
    try {
      PdfReader reader;
      reader = new PdfReader(pdf);
      pdf_diploma = generarPDF(CARACTERISTICAS_DIPLOMA, getDatosDiplomaTaller(matricula, u), reader);
    } catch (IOException e) {
      logger.error("Error al generar diploma de taller, no se encuentra la plantilla de pdf");
    }
    return pdf_diploma;
  }

  /**
   * Devuelve pdf guion curso.
   * 
   * @param matricula
   *           matricula
   * @param pdf
   *           pdf
   * @return the pdf guion curso
   */
  public ByteArrayOutputStream getPdfGuionCurso(MatriculaCurso matricula, String pdf) {
    ByteArrayOutputStream pdf_diploma = new ByteArrayOutputStream();
    try {
      PdfReader reader;
      reader = new PdfReader(pdf);
      pdf_diploma = generarPDF(CARACTERISTICAS_GUION, getDatosGuionCurso(matricula), reader);

    } catch (IOException e) {
      logger.error("Error al generar guion de curso, no se encuentra la plantilla de pdf");
    }
    return pdf_diploma;
  }

  /**
   * Devuelve pdf guion taller.
   * 
   * @param matricula
   *           matricula
   * @param pdf
   *           pdf
   * @return the pdf guion taller
   */
  public ByteArrayOutputStream getPdfGuionTaller(MatriculaTaller matricula, String pdf) {
    ByteArrayOutputStream pdf_diploma = new ByteArrayOutputStream();
    try {
      PdfReader reader;
      reader = new PdfReader(pdf);
      pdf_diploma = generarPDF(CARACTERISTICAS_GUION, getDatosGuionTaller(matricula), reader);

    } catch (IOException e) {
      logger.error("Error al generar guion de taller, no se encuentra la plantilla de pdf");
    }
    return pdf_diploma;
  }

  private void makeHeader(HttpServletResponse response, String nombre) {
    response.setContentType("application/pdf");
    response.setHeader("Cache-Control", "no-store, no-cache, must-revalidate");
    response.addHeader("Cache-Control", "post-check=0, pre-check=0");
    response.setHeader("Pragma", "no-cache");
    response.setCharacterEncoding("ISO-8859-15");
    response.setHeader("Content-Disposition", "attachment;filename=\"" + nombre + ".pdf\"");
  }

  /**
   * Devuelve datos diploma curso.
   * 
   * @param matricula
   *           matricula
   * @param u
   *           u
   * @return the datos diploma curso
   */
  public Map<String, Object> getDatosDiplomaCurso(MatriculaCurso matricula, UsuarioCyL u) {
    Map<String, Object> datos = new HashMap<String, Object>();
    Locale locale = prl.getLocale();
    DateFormat df1 = new SimpleDateFormat("dd", locale);
    DateFormat df2 = new SimpleDateFormat("MMMM", locale);
    DateFormat df3 = new SimpleDateFormat("yyyy", locale);

    String nombreUsuario = "";
    String descripcion = "";
    String nombreDatosDiplomaCurso = "";
    
    ResourceBundle res = ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.excel", LocaleContextHolder.getLocaleContext().getLocale());
    
    nombreUsuario = res.getString("pdf.D_DNA") + "  " + matricula.getUsuario().getNombreCompleto();

    // si pasaporte vacío, imprimir dni, sino, el autorizado, y sino una descripción genérica.
    if (u.getPasaporte() != null && !u.getPasaporte().isEmpty()) descripcion += res.getString("pdf.ConPasaporte") + " " + u.getPasaporte();
    else if (matricula.getUsuario().getNif() != null && !matricula.getUsuario().getNif().isEmpty()) descripcion += res.getString("pdf.ConDni") 
    + " " + matricula.getUsuario().getNif() + " ";
    else {
      if (u.getNombreCompletoAutorizado() != null && !u.getNombreCompletoAutorizado().isEmpty()
          && u.getNifAutorizado() != null && !u.getNifAutorizado().isEmpty()) descripcion += res.getString("pdf.Autorizado")+ " "
          + u.getNombreCompletoAutorizado() + " (" + u.getNifAutorizado() + ")";
      else descripcion += res.getString("pdf.MenorEdadSinDni") + " ";
    }

    descripcion += res.getString("pdf.Realizado") + " ";
    /*
     * Comentado para implementar los servicios MULTILANGUAGE
     * TODO: verificar su funcionamiento /*matricula.getCurso().getServicio().getNombre().replace("\"", "'") +*/
    
    nombreDatosDiplomaCurso = GetServicioNombre(matricula.getCurso().getServicio());
    descripcion +="'" + nombreDatosDiplomaCurso.replace("\"", "'")+"'" + " " + res.getString("pdf.Duracion") + " ";
    descripcion += matricula.getCurso().getNumeroHoras()
        + " " + res.getString("pdf.Impartido") + " ";
    String fecha = matricula.getCurso().getCentro().getLocalidad().getNombre() + ", "
        + df1.format(matricula.getCurso().getFechaFin()) + " de " + df2.format(matricula.getCurso().getFechaFin())
        + " de " + df3.format(matricula.getCurso().getFechaFin());

    datos.put("nombreUsuario", nombreUsuario);
    datos.put("descripcion", descripcion.replaceAll("\\.0", ""));
    datos.put("fecha", fecha);
    return datos;
  }

  /**
   * Devuelve datos diploma taller.
   * 
   * @param matricula
   *           matricula
   * @param u
   *           u
   * @return the datos diploma taller
   */
  public Map<String, Object> getDatosDiplomaTaller(MatriculaTaller matricula, UsuarioCyL u) {
    Map<String, Object> datos = new HashMap<String, Object>();
    Locale locale = prl.getLocale();
    DateFormat df1 = new SimpleDateFormat("dd", locale);
    DateFormat df2 = new SimpleDateFormat("MMMM", locale);
    DateFormat df3 = new SimpleDateFormat("yyyy", locale);

    String nombreUsuario = "D/DÑA  " + matricula.getUsuario().getNombreCompleto();

    String descripcion = "";
    String nombreDatosDiplomaTaller = "";

    // si pasaporte vacío, imprimir dni, sino, el autorizado, y sino una descripción genérica.
    if (u.getPasaporte() != null && !u.getPasaporte().isEmpty()) descripcion += "Con pasaporte " + u.getPasaporte();
    else if (matricula.getUsuario().getNif() != null && !matricula.getUsuario().getNif().isEmpty()) descripcion += "Con DNI "
        + matricula.getUsuario().getNif();
    else {
      if (u.getNombreCompletoAutorizado() != null && !u.getNombreCompletoAutorizado().isEmpty()
          && u.getNifAutorizado() != null && !u.getNifAutorizado().isEmpty()) descripcion += "Autorizado por "
          + u.getNombreCompletoAutorizado() + " (" + u.getNifAutorizado() + ")";
      else descripcion += "Menor de edad y sin DNI o pasaporte";
    }

    descripcion += ", ha realizado el curso \"";
    /*Cambiado
     * TODO: verificar su funcionamiento /*matricula.getTaller().getServicio().getNombre().replace("\"", "'") +*/
    
    nombreDatosDiplomaTaller = GetServicioNombre(matricula.getTaller().getServicio());
    descripcion += nombreDatosDiplomaTaller.replace("\"", "'") + "\", con una duración de ";
    descripcion += matricula.getTaller().getNumeroHoras()
        + " horas, impartido por la Xunta de Galicia.";
    String fecha = matricula.getTaller().getCentro().getLocalidad().getNombre() + ", "
        + df1.format(matricula.getTaller().getFechaFin()) + " de " + df2.format(matricula.getTaller().getFechaFin())
        + " de " + df3.format(matricula.getTaller().getFechaFin());

    datos.put("nombreUsuario", nombreUsuario);
    datos.put("descripcion", descripcion.replaceAll("\\.0", ""));
    datos.put("fecha", fecha);
    return datos;
  }

  /**
   * Devuelve datos guion curso.
   * 
   * @param matricula
   *           matricula
   * @return the datos guion curso
   */
  public static Map<String, Object> getDatosGuionCurso(MatriculaCurso matricula) {
    Map<String, Object> datos = new HashMap<String, Object>();
    /*Cambiado
     * TODO: verificar su funcionamiento
    datos.put("guion", ((ServicioCurso) matricula.getCurso().getServicio()).getGuion());*/
    
    String guion = GetServicioGuion((ServicioCurso) matricula.getCurso().getServicio());
    datos.put("guion", (guion));
       
    return datos;
  }
 
  /**
   * Devuelve datos guion taller.
   * 
   * @param matricula
   *           matricula
   * @return the datos guion taller
   */
  public static Map<String, Object> getDatosGuionTaller(MatriculaTaller matricula) {
    Map<String, Object> datos = new HashMap<String, Object>();
    /*
     * Comentado para implementar los servicios MULTILANGUAGE
     * TODO: mirar y modificar "Este no se hace por ahora, ya que no hay talleres"
    datos.put("guion", ((ServicioTaller) matricula.getTaller().getServicio()).getGuion());
    */
    return datos;
  }

  /**
   * Generar pdf.
   * 
   * @param titulos
   *           titulos
   * @param datos
   *           datos
   * @param reader
   *           reader
   * @return the byte array output stream
   */
  public static ByteArrayOutputStream generarPDF(Map<String, Object[]> titulos, Map<String, Object> datos,
      PdfReader reader) {
    ByteArrayOutputStream result = new ByteArrayOutputStream();
    try {

      PdfStamper stamper = new PdfStamper(reader, result);

      int i = 1;
      PdfContentByte over = stamper.getOverContent(i);

      for (String e : titulos.keySet()) {
        Object[] obj = titulos.get(e);
        String dato = null;

        if (datos.get(e) == null || datos.get(e).toString().isEmpty()) dato = null;
        else {
          if (e.equals("duracion")) {
            NumberFormat numberInstance = NumberFormat.getNumberInstance();
            numberInstance.setMaximumFractionDigits(2);
            numberInstance.setMinimumFractionDigits(0);
            dato = numberInstance.format(datos.get(e));
          }
        }

        if (dato == null) dato = datos.get(e) == null ? "" : datos.get(e).toString();

        over.beginText();
        over.setFontAndSize(((Font) obj[0]).getBaseFont(), ((Float) obj[1]));
        if (obj[5] != null) over.setColorFill(((Color) obj[5]));

        String cortado = "";
        int j = 0;
        if (((Font) obj[0]).getBaseFont().getWidthPoint(dato, ((Float) obj[1])) > MAX_WIDTH) {
          cortado = getSPlitedString(((Font) obj[0]).getBaseFont(), dato, ((Float) obj[1]), cortado, j);
          dato = dato.substring(cortado.length(), dato.length());
          over.showTextAligned(((Integer) obj[2]), cortado, ((Float) obj[3]), ((Float) obj[4]), 0);
          over.endText();
          j++;
          while (!dato.equals("")) {
            over.beginText();
            over.setFontAndSize(((Font) obj[0]).getBaseFont(), ((Float) obj[1]));
            if (obj[5] != null) over.setColorFill(((Color) obj[5]));
            cortado = getSPlitedString(((Font) obj[0]).getBaseFont(), dato, ((Float) obj[1]), cortado, j);
            dato = dato.substring(cortado.length(), dato.length());
            over.showTextAligned(((Integer) obj[2]), cortado, ((Float) obj[3]), ((Float) obj[4]) - (j * 22), 0);
            over.endText();
            j++;
          }
        } else {
          over.showTextAligned(((Integer) obj[2]), dato, ((Float) obj[3]), ((Float) obj[4]), 0);
          over.endText();
        }
      }
      stamper.close();
    } catch (Exception e) {
      e.printStackTrace();
    }
    return result;
  }

  private static String getSPlitedString(BaseFont baseFont, String dato, Float float1, String cortado, int j) {
    String aux = "" + dato;
    if (baseFont.getWidthPoint(dato, float1) > MAX_WIDTH) {
      while (baseFont.getWidthPoint(aux, float1) > MAX_WIDTH) {
        aux = aux.substring(0, aux.length() - 1);
      }
      while (!aux.equals("") && !aux.substring(aux.length() - 1, aux.length()).equals(" ")) {
        aux = aux.substring(0, aux.length() - 1);
      }
      cortado = aux + "";
      return cortado;
    } else if (dato.length() > 0) {
      cortado = dato;
      return cortado;
    } else {
      return "";
    }
  }

  /**
   * Merge pdfs.
   * 
   * @param pdf1
   *           pdf1
   * @param pdf2
   *           pdf2
   * @param salida
   *           salida
   */
  public void mergePdfs(PdfReader pdf1, PdfReader pdf2, OutputStream salida) {
    try {
      Document document = new Document();
      PdfCopy writer;
      writer = new PdfCopy(document, salida);
      document.open();
      writer.addPage(writer.getImportedPage(pdf1, 1));
      writer.addPage(writer.getImportedPage(pdf2, 1));
      document.close();
    } catch (DocumentException e) {
      e.printStackTrace();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  private ByteArrayOutputStream mergePdfsCompletos(ByteArrayOutputStream pdf1, PdfReader pdf2) {
    ByteArrayOutputStream resultado = new ByteArrayOutputStream();
    try {
      PdfReader pdfReader = new PdfReader(new ByteArrayInputStream(pdf1.toByteArray()));
      resultado = mergePdfsCompletos(pdfReader, pdf2);
    } catch (IOException e) {
      e.printStackTrace();
    }
    return resultado;
  }

  private ByteArrayOutputStream mergePdfsCompletos(PdfReader pdf1, PdfReader pdf2) {
    ByteArrayOutputStream resultado = new ByteArrayOutputStream();
    try {
      Document document = new Document();
      PdfCopy writer;
      writer = new PdfCopy(document, resultado);
      document.open();
      for (int i = 1; i <= pdf1.getNumberOfPages(); i++) {
        writer.addPage(writer.getImportedPage(pdf1, i));
      }
      for (int i = 1; i <= pdf2.getNumberOfPages(); i++) {
        writer.addPage(writer.getImportedPage(pdf2, i));
      }
      document.close();
    } catch (DocumentException e) {
      e.printStackTrace();
    } catch (IOException e) {
      e.printStackTrace();
    }
    return resultado;
  }
  
  public List<Role> getRolesDiplomaServicio(Servicio s) {
	  ServicioCurso sc = serviceCurso.find(ServicioCurso.class, s.getId());
	  
	  return sc.getRolesEmisoresDiplomaList();
  }
  
  public String GetServicioNombre(Servicio s)
  {
	  idioma = LocaleContextHolder.getLocaleContext().getLocale().toString();
	  String nombreServicio = "";
	  if (s != null)
	  {		 		  
	      List<ServicioLanguage> lsl = s.getTextos();
	      for (ServicioLanguage sl : lsl)
	      	if (sl.getLanguage().getISO().compareTo(idioma) == 0)
	      		nombreServicio = sl.getNombre();      
		  return (nombreServicio);
	  }
	  else
		  return ("");
  } 
  
  public static String GetServicioGuion(ServicioCurso sc)
  {
	  String idioma = LocaleContextHolder.getLocaleContext().getLocale().toString();
	  String guion = "";
	  
	  if (sc != null)
	  {		 		  
	      List<ServicioCursoLanguage> lscl = sc.getGuiones();
	      for (ServicioCursoLanguage scl : lscl)
	      	if (scl.getLanguage().getISO().compareTo(idioma) == 0)	      		
	      		guion = (scl.getGuion() != null) ? scl.getGuion(): "NO";      
		  return (guion);
	  }
	  else
		  return ("");
  }
  
}
