package es.csa.fdt.cyldigital.indicadores.financiero;

/*
 * #%L
 * Librerías del núcleo para Web - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import es.csa.fdt.cyldigital.financiero.entity.ProyectoAnual;
import es.csa.fdt.cyldigital.financiero.service.ProyectoAnualService;
import es.csa.fdt.cyldigital.indicadores.ReportHelper;

/**
 * Clase ReportHelperCajas.
 */
@Component("reportHelperFinancieroCajas")
public class ReportHelperCajas implements ReportHelper {

  /** service. */
  @Autowired
  public ProyectoAnualService service;

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.indicadores.ReportHelper#getReportModel(java.lang.String, java.util.Map)
   */
  @Override
  public Map<String, Object> getReportModel(String string, Map<String, Object> params) {
    Map<String, Object> map = new HashMap<String, Object>();
    List<ProyectoAnual> proyectosAnuales = new ArrayList<ProyectoAnual>();
    
    String pathCyl = (String) (params.get("rutaCyl"));
    String pathIsis = (String) (params.get("rutaIsis"));
    String idProyecto = (String) (params.get("idProyecto"));
    Long idProyectoAnual = Long.parseLong(idProyecto);

    ProyectoAnual proyectoAnual = service.find(idProyectoAnual);
    String subtitle = "";
    if(!proyectoAnual.esProyectoGeneral()){
      proyectosAnuales.add(proyectoAnual);
      subtitle = "Centro CyL Digital de ".concat(proyectoAnual.getCentro().getNombre());
    }else{
      for(ProyectoAnual subproyectoAnual : proyectoAnual.getSubproyectosAnuales()){
        proyectosAnuales.add(subproyectoAnual);
      }
      subtitle = "Centros CyL Digital";
    }
    
    map.put("data", proyectosAnuales);
    map.put("anyo", proyectoAnual.getAnyo());
    map.put("title", "Informe detallado de cajas");
    map.put("subtitle", subtitle);
    map.put("pathCyl", pathCyl);
    map.put("pathIsis", pathIsis);

    return map;
  }
  
  /**
   * Formatear cuantia.
   * 
   * @param cuantia
   *           cuantia
   * @return the string
   */
  public static String formatearCuantia(float cuantia){
    String cadenaFormateada = "";
    if(cuantia >= 1000000){
      cadenaFormateada = new DecimalFormat("#,##0.00").format(cuantia/new Float(1000000)).concat(" MM");
    }else{
      if(cuantia >= 10000){
        cadenaFormateada = new DecimalFormat("#,##0").format(cuantia);
      }else{
        cadenaFormateada = new DecimalFormat("#,##0.00").format(cuantia);
      }
    }
    
    return cadenaFormateada;
  }

}
