package es.csa.fdt.cyldigital.indicadores.incidencias;

/*
 * #%L
 * Librerías del núcleo para Web - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.persistence.Query;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.centros.service.CentroService;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.incidencias.entity.EventoIncidencia;
import es.csa.fdt.cyldigital.incidencias.entity.Lista;
import es.csa.fdt.cyldigital.incidencias.entity.Incidencia.EstadoIncidencia;
import es.csa.fdt.cyldigital.incidencias.service.EventoIncidenciaService;
import es.csa.fdt.cyldigital.indicadores.ReportHelper;

/**
 * Clase ReportHelperIncidencias.
 */
@Component("reportHelperIncidenciasIncidencias")
public class ReportHelperIncidencias extends AbstractServiceImpl<Centro, Long> implements ReportHelper {

  private static final long serialVersionUID = 1L;

  /** servicio. */
  @Autowired
  public EventoIncidenciaService servicio;

  /** servicio2. */
  @Autowired
  public CentroService servicio2;

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.indicadores.ReportHelper#getReportModel(java.lang.String, java.util.Map)
   */
  @SuppressWarnings("unchecked")
  @Override
  @Transactional(readOnly = true)
  public Map<String, Object> getReportModel(String string, Map<String, Object> params) {
    Map<String, Object> map = new HashMap<String, Object>();
    // List<EventoIncidencia> eventos = servicio.findAll();
    Integer i = 0;
    List<Integer> a = new ArrayList<Integer>();
    while (i < 85) {
      a.add(0);
      i++;
    }
    List<EventoIncidencia> finales = new ArrayList<EventoIncidencia>();
    Centro centro = new Centro();
    if ((params.get("idCentro").toString()) != "") {
      long id = Long.parseLong((params.get("idCentro")).toString());
      centro = servicio2.findOneWhere("id", id);
    } else {
      centro.setNombre("Todos los centros");
    }
    // SimpleDateFormat formateador = new SimpleDateFormat("dd/MM/yyyy");
    Date fechaInicio = new Date();
    Date fechaFin = new Date();
    String pathCyl = (String) (params.get("rutaCyl"));
    String pathIsis = (String) (params.get("rutaIsis"));
    fechaInicio = (Date) params.get("fechaInicio");
    fechaFin = (Date) (params.get("fechaFin"));
    Calendar c = new GregorianCalendar();
    c.setTime(fechaFin);
    c.add(Calendar.HOUR, 23);
    c.add(Calendar.MINUTE, 59);
    fechaFin = c.getTime();
    Boolean flag = true;

    List<EventoIncidencia> incidencias = listaInicio(fechaInicio);
    finales = recorrerLista(incidencias, a, params, flag);
    flag = false;
    Query query = getEntityManager().createQuery(
        "select e from EventoIncidencia e " + "where e.id" + " in (SELECT max(a.id) "
            + "FROM  EventoIncidencia a where fechaEstado < ? and estado!= ? group by a.incidencia.id)");
    query.setParameter(1, fechaFin);
    query.setParameter(2, EstadoIncidencia.COMENTARIO);

    List<EventoIncidencia> incidenciasFin = query.getResultList();
    finales = recorrerLista(incidenciasFin, a, params, flag);

    map.put("data", finales);
    map.put("title", "Estadística de incidencias");
    map.put("total", finales.size());
    map.put("centro", centro.getNombre());
    map.put("pathCyl", pathCyl);
    map.put("pathIsis", pathIsis);

    return map;
  }

  /**
   * Lista inicio.
   * 
   * @param fechaInicio
   *           fecha inicio
   * @return the list
   */
  @SuppressWarnings("unchecked")
  public List<EventoIncidencia> listaInicio(Date fechaInicio) {
    Query q = getEntityManager().createQuery(
        "select e from EventoIncidencia e " + "where e.id" + " in (SELECT max(a.id) "
            + "FROM  EventoIncidencia a where fechaEstado < ? and estado!= ? group by a.incidencia.id)");
    q.setParameter(1, fechaInicio);
    q.setParameter(2, EstadoIncidencia.COMENTARIO);

    List<EventoIncidencia> incidencias = q.getResultList();
    return incidencias;
  }

  /**
   * Recorrer lista.
   * 
   * @param incidencias
   *           incidencias
   * @param a
   *           a
   * @param params
   *           params
   * @param flag
   *           flag
   * @return the list
   */
  public List<EventoIncidencia> recorrerLista(List<EventoIncidencia> incidencias, List<Integer> a,
      Map<String, Object> params, Boolean flag) {
    Lista lista = new Lista();
    Centro centro = new Centro();
    List<EventoIncidencia> finales = new ArrayList<EventoIncidencia>();
    if ((params.get("idCentro").toString()) != "") {
      long id = Long.parseLong((params.get("idCentro")).toString());
      centro = servicio2.findOneWhere("id", id);
      Iterator<EventoIncidencia> itr = incidencias.iterator();
      while (itr.hasNext()) {
        EventoIncidencia incidencia = new EventoIncidencia();
        incidencia = itr.next();
        if (incidencia.getIncidencia().getCentro() != null) {
          Centro centroAux = incidencia.getIncidencia().getCentro();
          if (centroAux.equals(centro)) {
            lista.setLista(calcularTipo(incidencia, a, flag));
            incidencia.setLista(lista);
            finales.add(incidencia);

          }
        } else {
          lista.setLista(calcularTipo(incidencia, a, flag));
          incidencia.setLista(lista);
          finales.add(incidencia);
        }
      }
    } else {
      centro.setNombre("Todos los centros");
      Iterator<EventoIncidencia> itr = incidencias.iterator();
      while (itr.hasNext()) {
        EventoIncidencia incidencia = new EventoIncidencia();
        incidencia = itr.next();
        lista.setLista(calcularTipo(incidencia, a, flag));
        incidencia.setLista(lista);
        finales.add(incidencia);

      }
    }
    return finales;
  }

  /**
   * Calcular tipo.
   * 
   * @param incidencia
   *           incidencia
   * @param a
   *           a
   * @param flag
   *           flag
   * @return the list
   */
  public List<Integer> calcularTipo(EventoIncidencia incidencia, List<Integer> a, Boolean flag) {
    Integer b = 0;
    int i;

    if (incidencia.getCriticidad().toString() == "CRITICA") {
      if (incidencia.getEstado().toString() == "PENDIENTE") {
        if (flag) {
          i = 0;
        } else {
          i = 42;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "INFORMACION") {
        if (flag) {
          i = 6;
        } else {
          i = 48;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "CURSO") {
        if (flag) {
          i = 12;
        } else {
          i = 54;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "CORREGIDA") {
        if (flag) {
          i = 18;
        } else {
          i = 60;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "RECHAZADA") {
        if (flag) {
          i = 24;
        } else {
          i = 66;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "ANULADA") {
        if (flag) {
          i = 30;
        } else {
          i = 72;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "CERRADA") {
        if (flag) {
          i = 36;
        } else {
          i = 78;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      }

    } else if (incidencia.getCriticidad().toString() == "BLOQUEANTE") {
      if (incidencia.getEstado().toString() == "PENDIENTE") {
        if (flag) {
          i = 1;
        } else {
          i = 43;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "INFORMACION") {
        if (flag) {
          i = 7;
        } else {
          i = 49;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "CURSO") {
        if (flag) {
          i = 13;
        } else {
          i = 55;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "CORREGIDA") {
        if (flag) {
          i = 19;
        } else {
          i = 61;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "RECHAZADA") {
        if (flag) {
          i = 25;
        } else {
          i = 67;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "ANULADA") {
        if (flag) {
          i = 31;
        } else {
          i = 73;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "CERRADA") {
        if (flag) {
          i = 37;
        } else {
          i = 79;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      }

    } else if (incidencia.getCriticidad().toString() == "MAYOR") {
      if (incidencia.getEstado().toString() == "PENDIENTE") {
        if (flag) {
          i = 2;
        } else {
          i = 44;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "INFORMACION") {
        if (flag) {
          i = 8;
        } else {
          i = 50;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "CURSO") {
        if (flag) {
          i = 14;
        } else {
          i = 56;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "CORREGIDA") {
        if (flag) {
          i = 20;
        } else {
          i = 62;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "RECHAZADA") {
        if (flag) {
          i = 26;
        } else {
          i = 68;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "ANULADA") {
        if (flag) {
          i = 32;
        } else {
          i = 74;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "CERRADA") {
        if (flag) {
          i = 38;
        } else {
          i = 80;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      }

    } else if (incidencia.getCriticidad().toString() == "MENOR") {
      if (incidencia.getEstado().toString() == "PENDIENTE") {
        if (flag) {
          i = 3;
        } else {
          i = 45;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "INFORMACION") {
        if (flag) {
          i = 9;
        } else {
          i = 51;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "CURSO") {
        if (flag) {
          i = 15;
        } else {
          i = 57;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "CORREGIDA") {
        if (flag) {
          i = 21;
        } else {
          i = 63;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "RECHAZADA") {
        if (flag) {
          i = 27;
        } else {
          i = 69;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "ANULADA") {
        if (flag) {
          i = 33;
        } else {
          i = 75;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "CERRADA") {
        if (flag) {
          i = 39;
        } else {
          i = 81;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      }

    } else if (incidencia.getCriticidad().toString() == "DUDAS") {
      if (incidencia.getEstado().toString() == "PENDIENTE") {
        if (flag) {
          i = 4;
        } else {
          i = 46;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "INFORMACION") {
        if (flag) {
          i = 10;
        } else {
          i = 52;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "CURSO") {
        if (flag) {
          i = 16;
        } else {
          i = 58;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "CORREGIDA") {
        if (flag) {
          i = 22;
        } else {
          i = 64;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "RECHAZADA") {
        if (flag) {
          i = 28;
        } else {
          i = 70;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "ANULADA") {
        if (flag) {
          i = 34;
        } else {
          i = 76;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "CERRADA") {
        if (flag) {
          i = 40;
        } else {
          i = 82;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      }

    } else if (incidencia.getCriticidad().toString() == "NUEVA") {
      if (incidencia.getEstado().toString() == "PENDIENTE") {
        if (flag) {
          i = 5;
        } else {
          i = 47;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "INFORMACION") {
        if (flag) {
          i = 11;
        } else {
          i = 53;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "CURSO") {
        if (flag) {
          i = 17;
        } else {
          i = 59;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "CORREGIDA") {
        if (flag) {
          i = 23;
        } else {
          i = 65;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "RECHAZADA") {
        if (flag) {
          i = 29;
        } else {
          i = 71;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "ANULADA") {
        if (flag) {
          i = 35;
        } else {
          i = 77;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      } else if (incidencia.getEstado().toString() == "CERRADA") {
        if (flag) {
          i = 41;
        } else {
          i = 83;
        }
        b = a.get(i);
        b++;
        a.set(i, b);
      }

    }
    return a;

  }

}
