package es.csa.fdt.cyldigital.indicadores.incidencias;

/*
 * #%L
 * Librerías del núcleo para Web - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.incidencias.entity.CategoriaIncidencia;
import es.csa.fdt.cyldigital.incidencias.entity.Incidencia;
import es.csa.fdt.cyldigital.incidencias.service.IncidenciaService;
import es.csa.fdt.cyldigital.indicadores.ReportHelper;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado.Puesto;

/**
 * Clase ReportHelperIncidenciasDetallado.
 */
@Transactional
@Component("reportHelperIncidenciasIncidenciasDetallado")
public class ReportHelperIncidenciasDetallado implements ReportHelper {

  /** servicio. */
  @Autowired
  public IncidenciaService servicio;

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.indicadores.ReportHelper#getReportModel(java.lang.String, java.util.Map)
   */
  @Override
  @Transactional
  public Map<String, Object> getReportModel(String string, Map<String, Object> params) {

    Map<String, Object> map = new HashMap<String, Object>();
    String pathCyl = (String) (params.get("rutaCyl"));
    String pathIsis = (String) (params.get("rutaIsis"));
    Date fechaInicio = (Date) params.get("fechaInicio");
    Date fechaFin = (Date) params.get("fechaFin");
    String idUsuarioPeticion = (String) params.get("idUsuarioPeticion"); // usuario que realiza la petición.
    String idCentro = (String) params.get("idCentro");
    List<Incidencia> incidencias;
    String nombreCentro = "";
    DateFormat df = new SimpleDateFormat("yyyy-MM-dd");
    String query = "";
    long idUsuario = Long.parseLong(idUsuarioPeticion);
    Empleado e = servicio.find(Empleado.class, idUsuario);
    List<CategoriaIncidencia> categoriasEspeciales = servicio.findWhere(CategoriaIncidencia.class, "especial", true);

    //
    // criterios de búsqueda genéricos, fecha y centro, categorias del usuario y filtrar las especiales.
    if (!idCentro.isEmpty()) {
      long id = Long.parseLong((params.get("idCentro")).toString());
      nombreCentro = (servicio.find(Centro.class, id)).getNombre();

      query = "select e from " + Incidencia.class.getName() + " e  where centro.id=" + id + " and fechaIncidencia >= '"
          + df.format(fechaInicio) + "' and fechaIncidencia <= '" + df.format(fechaFin) + "'";
    } else {
      // TODO cambiar cómo se procesan las fechas.
      nombreCentro = "Todos";
      query = "select e from " + Incidencia.class.getName() + " e  where " + " fechaIncidencia >= '"
          + df.format(fechaInicio) + "' and fechaIncidencia <= '" + df.format(fechaFin) + "'";
    }

    if (!e.isResponsableIncidencias()) {
      for (CategoriaIncidencia c : categoriasEspeciales) {
        if (!e.getCategoriasIncidencia().contains(c)) query += " and categoriaIncidencia.id!=" + c.getId();
      }
    }

    //
    // aplicamos criterios propios del rol
    if (!e.getPuesto().equals(Puesto.ADMIN) && !e.getPuesto().equals(Puesto.TRAINER_COORDINATOR)
        && !e.getPuesto().equals(Puesto.MANAGER)) {
      if (e.getPuesto().equals(Puesto.AREA_MANAGER) || e.getPuesto().equals(Puesto.DIGITAL_ALLY_MANAGER)) {
        query += " and (usuario.id=" + e.getId() + " or centro is null or centro.id=" + e.getCentro().getId();

      } else {
        query += " and (usuario.id=" + e.getId() + " or centro.id=" + e.getCentro().getId();

      }
      if (!e.getPuesto().equals(Puesto.MANAGER) && !e.getPuesto().equals(Puesto.TRAINER_COORDINATOR)
          && !e.getPuesto().equals(Puesto.ADMIN) && !e.isResponsableIncidencias()) {
        for (CategoriaIncidencia c : e.getCategoriasIncidencia()) {
          query += " or categoriaIncidencia.id=" + c.getId();
        }
      }
      query += ")";
    }

    incidencias = servicio.findByQuery(query);

    // lo siguiente se hace únicamente para cargar los datos y que estén disponibles.
    for (Incidencia i : incidencias) {
      i.getCategoriaIncidencia().getNombre();
      if (i.getCentro() != null) i.getCentro().getNombre();

    }
    map.put("data", incidencias);
    map.put("title", "Incidencias detalladas");
    map.put("total", incidencias.size());
    map.put("centro", nombreCentro);
    map.put("pathCyl", pathCyl);
    map.put("pathIsis", pathIsis);

    return map;
  }

}
