package es.csa.fdt.cyldigital.reports;

/*
 * #%L
 * Librerías del núcleo para Web - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import javax.persistence.Query;
import javax.persistence.TemporalType;

import org.springframework.beans.factory.annotation.Autowired;

import es.csa.fdt.cyldigital.calendario.entity.CalendarItem;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.centros.service.CentroService;
import es.csa.fdt.cyldigital.servicios.entity.ActividadExterna;
import es.csa.fdt.cyldigital.servicios.entity.Charla;
import es.csa.fdt.cyldigital.servicios.entity.Curso;

/**
 * Clase ReportHelperBase.
 */
public class ReportHelperBase {
	@Autowired
	public CentroService service;
	
	private CalendarItem createCentroCalendarItem(Calendar cal, Date startHour, Date endHour) {
		CalendarItem item = new CalendarItem();

		Calendar tempStart = Calendar.getInstance();
		Calendar tempEnd = Calendar.getInstance();

		tempStart.setTime(startHour);
		tempStart.set(Calendar.SECOND, 0);
		tempStart.set(cal.get(Calendar.YEAR), cal.get(Calendar.MONTH), cal.get(Calendar.DATE));

		tempEnd.setTime(endHour);
		tempEnd.set(Calendar.SECOND, 0);
		tempEnd.set(cal.get(Calendar.YEAR), cal.get(Calendar.MONTH), cal.get(Calendar.DATE));

		item.setStart(tempStart.getTime());
		item.setEnd(tempEnd.getTime());
		item.setAllDay(false);

		return item;
	}
	
	protected List<CalendarItem> getCentroItems(Long idCentro, Date fechaInicio, Date fechaFin) {
		List<CalendarItem> centroItems = new ArrayList<CalendarItem>();
		Centro centro = service.find(idCentro);

		if (centro != null) {
			Calendar c = Calendar.getInstance();
			Calendar lastDate = Calendar.getInstance();

			c.setTimeInMillis(fechaInicio.getTime());
			lastDate.setTimeInMillis(fechaFin.getTime());

			while (c.compareTo(lastDate) <= 0) {
				switch (c.get(Calendar.DAY_OF_WEEK)) {
					case Calendar.MONDAY: {
						if ((centro.getMondayMorningStartHour() != null) && (centro.getMondayMorningEndHour() != null))
							centroItems.add(createCentroCalendarItem(c, centro.getMondayMorningStartHour(),
																	 centro.getMondayMorningEndHour()));

						if ((centro.getMondayAfternoonStartHour() != null) && (centro.getMondayAfternoonEndHour() != null))
							centroItems.add(createCentroCalendarItem(c, centro.getMondayAfternoonStartHour(),
																	 centro.getMondayAfternoonEndHour()));

						break;
					}

					case Calendar.TUESDAY: {
						if ((centro.getTuesdayMorningStartHour() != null) && (centro.getTuesdayMorningEndHour() != null))
							centroItems.add(createCentroCalendarItem(c, centro.getTuesdayMorningStartHour(),
																	 centro.getTuesdayMorningEndHour()));

						if ((centro.getTuesdayAfternoonStartHour() != null) && (centro.getTuesdayAfternoonEndHour() != null))
							centroItems.add(createCentroCalendarItem(c, centro.getTuesdayAfternoonStartHour(),
																	 centro.getTuesdayAfternoonEndHour()));

						break;
					}

					case Calendar.WEDNESDAY: {
						if ((centro.getWednesdayMorningStartHour() != null) && (centro.getWednesdayMorningEndHour() != null))
							centroItems.add(createCentroCalendarItem(c, centro.getWednesdayMorningStartHour(),
																	 centro.getWednesdayMorningEndHour()));

						if ((centro.getWednesdayAfternoonStartHour() != null) && (centro.getWednesdayAfternoonEndHour() != null))
							centroItems.add(createCentroCalendarItem(c, centro.getWednesdayAfternoonStartHour(),
																	 centro.getWednesdayAfternoonEndHour()));

						break;
					}

					case Calendar.THURSDAY: {
						if ((centro.getThursdayMorningStartHour() != null) && (centro.getThursdayMorningEndHour() != null))
							centroItems.add(createCentroCalendarItem(c, centro.getThursdayMorningStartHour(),
																	 centro.getThursdayMorningEndHour()));

						if ((centro.getThursdayAfternoonStartHour() != null) && (centro.getThursdayAfternoonEndHour() != null))
							centroItems.add(createCentroCalendarItem(c, centro.getThursdayAfternoonStartHour(),
																	 centro.getThursdayAfternoonEndHour()));

						break;
					}

					case Calendar.FRIDAY: {
						if ((centro.getFridayMorningStartHour() != null) && (centro.getFridayMorningEndHour() != null))
							centroItems.add(createCentroCalendarItem(c, centro.getFridayMorningStartHour(),
																	 centro.getFridayMorningEndHour()));

						if ((centro.getFridayAfternoonStartHour() != null) && (centro.getFridayAfternoonEndHour() != null))
							centroItems.add(createCentroCalendarItem(c, centro.getFridayAfternoonStartHour(),
																	 centro.getFridayAfternoonEndHour()));

						break;
					}

					case Calendar.SATURDAY: {
						if ((centro.getSaturdayMorningStartHour() != null) && (centro.getSaturdayMorningEndHour() != null))
							centroItems.add(createCentroCalendarItem(c, centro.getSaturdayMorningStartHour(),
																	 centro.getSaturdayMorningEndHour()));

						if ((centro.getSaturdayAfternoonStartHour() != null) && (centro.getSaturdayAfternoonEndHour() != null))
							centroItems.add(createCentroCalendarItem(c, centro.getSaturdayAfternoonStartHour(),
																	 centro.getSaturdayAfternoonEndHour()));

						break;
					}

					case Calendar.SUNDAY: {
						if ((centro.getSundayMorningStartHour() != null) && (centro.getSundayMorningEndHour() != null))
							centroItems.add(createCentroCalendarItem(c, centro.getSundayMorningStartHour(),
																	 centro.getSundayMorningEndHour()));

						if ((centro.getSundayAfternoonStartHour() != null) && (centro.getSundayAfternoonEndHour() != null))
							centroItems.add(createCentroCalendarItem(c, centro.getSundayAfternoonStartHour(),
																	 centro.getSundayAfternoonEndHour()));

						break;
					}
				}

				c.add(Calendar.DATE, 1);
			}
		}
		
		return centroItems;
	}
	
	protected List<CalendarItem> findActividadesItems(Long idCentro, Date FechaInicio, Date FechaFin) {
		String strqry;
		Query qry;
		List<CalendarItem> returnItems = new ArrayList<CalendarItem>();
		
		strqry = " a INNER JOIN " +
				 "a.centro c INNER JOIN " +
				 "a.calendarItems aci " +
				 "WHERE (a.desactivado = false) AND " +
				       "((aci.start BETWEEN :fechaInicio AND :fechaFin) OR " +
				        "(aci.end BETWEEN :fechaInicio AND :fechaFin)) AND " +
				       "(c.id = :idCentro)";

		//Cursos
		qry = service.getEntityManager().createQuery("SELECT aci FROM " + Curso.class.getName() + strqry);
		qry.setParameter("idCentro", idCentro);
		qry.setParameter("fechaInicio", FechaInicio, TemporalType.TIMESTAMP);
		qry.setParameter("fechaFin", FechaFin, TemporalType.TIMESTAMP);
		
		returnItems.addAll((List<CalendarItem>)qry.getResultList());
		
		//Charlas
		qry = service.getEntityManager().createQuery("SELECT aci FROM " + Charla.class.getName() + strqry);
		qry.setParameter("idCentro", idCentro);
		qry.setParameter("fechaInicio", FechaInicio, TemporalType.TIMESTAMP);
		qry.setParameter("fechaFin", FechaFin, TemporalType.TIMESTAMP);
		
		returnItems.addAll((List<CalendarItem>)qry.getResultList());
		
		//Actividades externas
		qry = service.getEntityManager().createQuery("SELECT aci FROM " + ActividadExterna.class.getName() + strqry);
		qry.setParameter("idCentro", idCentro);
		qry.setParameter("fechaInicio", FechaInicio, TemporalType.TIMESTAMP);
		qry.setParameter("fechaFin", FechaFin, TemporalType.TIMESTAMP);
		
		returnItems.addAll((List<CalendarItem>)qry.getResultList());

		//Devolver resultados
		return returnItems;
	}
}
