package es.csa.fdt.cyldigital.reports;

/*
 * #%L
 * Librerías del núcleo para Web - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;

import javax.persistence.Query;
import javax.persistence.TemporalType;

import net.sf.jasperreports.engine.JRParameter;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import es.csa.fdt.cyldigital.calendario.entity.AsistenciaCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.CalendarItem;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.controller.CalendarControllerImpl;
import es.csa.fdt.cyldigital.evaluacion.entity.Pregunta;
import es.csa.fdt.cyldigital.evaluacion.entity.Pregunta.TipoPregunta;
import es.csa.fdt.cyldigital.indicadores.ReportHelper;

import es.csa.fdt.cyldigital.servicios.entity.ActividadExterna;
import es.csa.fdt.cyldigital.servicios.entity.Charla;
import es.csa.fdt.cyldigital.servicios.entity.Curso;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso;
import es.csa.fdt.cyldigital.servicios.entity.Servicio.Nivel;
import es.csa.fdt.cyldigital.servicios.entity.ServicioCurso;

import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;
import es.fdt.isis.reports.entity.Activities;
import es.fdt.isis.reports.entity.ActivitiesAndRankingOfCenter;
import es.fdt.isis.reports.entity.ActivitiesByLevel;
import es.fdt.isis.reports.entity.ActivitiesByService;
import es.fdt.isis.reports.entity.AverageRatingOfQuestion;
import es.fdt.isis.reports.entity.OcupacionOfActivitiesByType;
import es.fdt.isis.reports.entity.RegisteredUsersByCenter;
import es.fdt.isis.reports.entity.UsersExamPassedByCentersAndTrainers;
import es.fdt.isis.reports.entity.UsersExamPassedByServices;

/**
 * Clase ReportHelperByCentro.
 */
@Component("reportHelperCentrosCentro")
public class ReportHelperByCentro extends ReportHelperBase implements ReportHelper {
	private Logger logger = LoggerFactory.getLogger(ReportHelperByCentro.class);
	private Long startTime;
	
	private Locale locale;

	/* (non-Javadoc)	
	 * @see es.fdt.isis.reports.ReportHelper#getReportModel(java.lang.String, java.util.Map)
	 */
	@Override
	@Transactional(readOnly = true)
	public Map<String, Object> getReportModel(String string, Map<String, Object> params) {
	    Map<String, Object> map = new HashMap<String, Object>(20);
	    
	    Long idCentro = Long.parseLong((String)params.get("idCentro"));

	    Date fechaInicio = (Date)params.get("fechaInicio");
	    Date fechaFin = (Date)params.get("fechaFin");
	    locale = (Locale) params.get("locale");

	    //Cálculo de datos
	    List<Object> CenterReportData = new ArrayList<Object>(14);
	    
	    Activities a;

	    startTime = System.nanoTime();
	    //CenterReportData.add(0, null);
	    CenterReportData.add(0, (a = getActivities(idCentro, fechaInicio, fechaFin)));
	    logger.info("ReportHelperByCentro:getActivities: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	    
	    startTime = System.nanoTime();
	    //CenterReportData.add(1, null);
	    CenterReportData.add(1, getOcupacionOfActivitiesByTypeList(a));
	    logger.info("ReportHelperByCentro:getOcupacionOfActivitiesByTypeList: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	    
	    startTime = System.nanoTime();
	    //CenterReportData.add(2, null);
	    CenterReportData.add(2, getMaxValueInOcupacionOfActivitiesByTypeList(a));
	    logger.info("ReportHelperByCentro:getMaxValueInOcupacionOfActivitiesByTypeList: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	    
	    startTime = System.nanoTime();
	    //CenterReportData.add(3, null);
	    CenterReportData.add(3, getCursosByServiceList(idCentro, fechaInicio, fechaFin));
	    logger.info("ReportHelperByCentro:getCursosByServiceList: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	    
	    startTime = System.nanoTime();
	    //CenterReportData.add(4, null);
	    CenterReportData.add(4, getCharlasByServiceList(idCentro, fechaInicio, fechaFin));
	    logger.info("ReportHelperByCentro:getCharlasByServiceList: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	    
	    startTime = System.nanoTime();
	    //CenterReportData.add(5, null);
	    CenterReportData.add(5, getExternasCAByServiceList(idCentro, fechaInicio, fechaFin));
	    logger.info("ReportHelperByCentro:getExternasCAByServiceList: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	    
	    startTime = System.nanoTime();
	    //CenterReportData.add(6, null);
	    CenterReportData.add(6, getExternasCAySByServiceList(idCentro, fechaInicio, fechaFin));
	    logger.info("ReportHelperByCentro:getExternasCAySByServiceList: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	    
	    startTime = System.nanoTime();
	    //CenterReportData.add(7, null);
	    CenterReportData.add(7, getActivitiesByLevelList(idCentro, fechaInicio, fechaFin));
	    logger.info("ReportHelperByCentro:getActivitiesByLevelList: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	    
	    startTime = System.nanoTime();
	    //CenterReportData.add(8, null);
	    CenterReportData.add(8, getPresenceOfCenter(idCentro, fechaInicio, fechaFin));
	    logger.info("ReportHelperByCentro:getPresenceOfCenter: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	    
	    startTime = System.nanoTime();
	    //CenterReportData.add(9, null);
	    CenterReportData.add(9, getPresenceOfferedInCenter(idCentro, fechaInicio, fechaFin));
	    logger.info("ReportHelperByCentro:getPresenceOfferedInCenter: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	    
	    startTime = System.nanoTime();
	    //CenterReportData.add(10, null); 
	    CenterReportData.add(10, getRegisteredUsersOfCenter(idCentro, fechaInicio, fechaFin));
	    logger.info("ReportHelperByCentro:getRegisteredUsersOfCenter: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	    
	    startTime = System.nanoTime();
	    //CenterReportData.add(11, null);
	    CenterReportData.add(11, getUsersExamPassedByServicesList(idCentro, fechaInicio, fechaFin));
	    logger.info("ReportHelperByCentro:getUsersExamPassedByServicesList: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	    
	    startTime = System.nanoTime();
	    //CenterReportData.add(12, null);
	    CenterReportData.add(12, getUsersExamPassedByTrainersList(idCentro, fechaInicio, fechaFin));
	    logger.info("ReportHelperByCentro:getUsersExamPassedByTrainersList: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	    
	    startTime = System.nanoTime();
	    //CenterReportData.add(13, null);
	    CenterReportData.add(13, getAverageRatingOfQuestionList(idCentro, fechaInicio, fechaFin));
	    logger.info("ReportHelperByCentro:getAverageRatingOfQuestionList: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	    
	    //Pone los parametros
	    map.put(JRParameter.REPORT_RESOURCE_BUNDLE, ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.informes", locale));
	    map.put(JRParameter.REPORT_LOCALE, locale);
	    map.put("nombreCentro", service.find(idCentro).getNombre());
	    map.put("CenterReportData", CenterReportData);

	    return map;
	}

	private Activities getActivities(Long idCentro,Date fechaInicio, Date fechaFin) {
	    String strqry;
		Activities a = new Activities();
		//3.4.12 - RF_15_22
		Long nCursos;
		Long nCharlas;
		Long nExternasCA;
		Long nExternasCAyS;
		
		//3.4.8 - RF_15_22
		// Número de horas de Aula Abierta
		startTime = System.nanoTime();
		//Calcula los CalendarItem de aula abierta en el centro entre las fechas dadas
		List<CalendarItem> OpenClassromCalendarItems = null;
		OpenClassromCalendarItems = CalendarControllerImpl.getOpenClassromCalendarItems(getCentroItems(idCentro, fechaInicio, fechaFin),
																								findActividadesItems(idCentro, fechaInicio, fechaFin));

		//Cálculo de horas de aula abierta en el centro con los Calendar Item
		Float numeroHorasAA = 0.0F;
		for (CalendarItem occi : OpenClassromCalendarItems)
			numeroHorasAA += ((occi.getEnd().getTime() - occi.getStart().getTime()) / 3600000.0F); //Extrae la duración en horas.
		
		//Establece el resultado en la estructura
		a.setNumeroHorasAA(numeroHorasAA);
		logger.info("ReportHelperByCentro:getActivities:Número de horas de aula abierta: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		
		
		/////// CURSOS ///////
		// Número de cursos
		strqry = "SELECT COUNT(cu) " +
				"FROM " + Curso.class.getName() + " cu " +
				"WHERE (cu.desactivado = false) AND " +
				"(cu.centro.id = :idCentro) AND " +
				"((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio))";
		startTime = System.nanoTime();
		
		nCursos = (Long)getQuerySingleResult(strqry, idCentro, fechaInicio, fechaFin);
		a.setNumeroCursos(nCursos);
		logger.info("ReportHelperByCentro:getActivities:Numero de cursos: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		
		if(nCursos > 0) {
			//Número de horas de cursos
			strqry = 
			"SELECT COALESCE(SUM(ROUND((HOUR(cuci.end) - HOUR(cuci.start)) + ((MINUTE(cuci.end) - MINUTE(cuci.start)) / 60.00F), 2)), 0.00F) + 0.00F " + //new line
			"FROM " + Curso.class.getName() + " cu INNER JOIN " + 
			"cu.calendarItems cuci " +
			"WHERE (cu.desactivado = false) AND " +
			"(cu.centro.id = :idCentro) AND " +
			"((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio)) AND " + 
			"((cuci.start <= :fechaFin) AND (cuci.end >= :fechaInicio))";
			
			startTime = System.nanoTime();
			a.setNumeroHorasCursos((Float)getQuerySingleResult(strqry, idCentro, fechaInicio, fechaFin));
			logger.info("ReportHelperByCentro:getActivities:Número de horas de cursos: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		
			// Número horas terminadas de cursos (Horas asistidas)
			strqry = 
					"SELECT COALESCE(SUM(ROUND((HOUR(acu.end) - HOUR(acu.start)) + ((MINUTE(acu.end) - MINUTE(acu.start)) / 60.00F), 2)), 0.00F) + 0.00F " +
					"FROM " + Curso.class.getName() + " cu INNER JOIN " +
					"cu.asistencias acu " +
					"WHERE (cu.desactivado = false) AND " +
					"(cu.centro.id = :idCentro) AND " +
					"((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio)) AND " + 
					"((acu.start <= :fechaFin) AND (acu.end >= :fechaInicio)) AND " +
					"(acu.excepcion <> :excepcionAsistencia)";
			
			startTime = System.nanoTime();
			a.setNumeroHorasTerminadasCursos((Float)getQuerySingleResult(strqry, idCentro, fechaInicio, fechaFin, AsistenciaCalendarItem.Excepcion.JUSTIFICADA));
			logger.info("ReportHelperByCentro:getActivities:Número horas terminadas d cursos: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			
			// Número de plazas ofertadas de cursos
			strqry = "SELECT COALESCE(SUM(cu.numeroPlazas), 0) " +
					"FROM " + Curso.class.getName() + " cu " +
					"WHERE (cu.desactivado = false) AND " +
					"(cu.centro.id = :idCentro) AND " +
					"((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio))";
			
			startTime = System.nanoTime();
			a.setNumeroPlazasOfertadasCursos((Long)getQuerySingleResult(strqry, idCentro, fechaInicio, fechaFin));
			logger.info("ReportHelperByCentro:getActivities:Número de plazas ofertadas de cursos: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		
			// Número de plazas cubiertas en cursos (usuarios matriculados)
			strqry = "SELECT COUNT(mcu) " +
					 "FROM " + Curso.class.getName() + " cu INNER JOIN " +
					 "cu.matriculas mcu " +
					 "WHERE (cu.desactivado = false) AND " +
					 "(cu.centro.id = :idCentro) AND " +
					 "((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio)) AND " +
					 "(mcu.admitido = TRUE) AND " +
					 "(mcu.enListaEspera = FALSE)";
			
			startTime = System.nanoTime();
			a.setNumeroPlazasCubiertasCursos((Long)getQuerySingleResult(strqry, idCentro, fechaInicio, fechaFin));
			logger.info("ReportHelperByCentro:getActivities:Número de plazas cubiertas en cursos: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			
			// Número de plazas terminadas en cursos (usuarios matriculados con asistencia >= 80%)
			strqry = "SELECT DISTINCT cu, acu.usuario.id " +
					 "FROM " + Curso.class.getName() + " cu INNER JOIN " +
					 "cu.asistencias acu " +
					 "WHERE (cu.desactivado = false) AND " +
					 "(cu.centro.id = :idCentro) AND " +
					 "((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio)) AND " +
					 "(acu.excepcion <> :excepcionAsistencia) " +
					 "GROUP BY cu, acu.usuario " +
			 		 "HAVING ((COUNT(acu) * 100.00) / cu.calendarItems.size) >= 80.00";
			
			startTime = System.nanoTime();
			a.setNumeroPlazasTerminadasCursos(getSizeOfQueryResultList(strqry, idCentro, fechaInicio, fechaFin, AsistenciaCalendarItem.Excepcion.JUSTIFICADA));
			logger.info("ReportHelperByCentro:getActivities:Número de plazas terminadas en cursos: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			
		} else {
			a.setNumeroHorasCursos(0F);
			a.setNumeroHorasTerminadasCursos(0F);
			a.setNumeroPlazasOfertadasCursos(0L);
			a.setNumeroPlazasCubiertasCursos(0L);
			a.setNumeroPlazasTerminadasCursos(0L);
		}
		
		/////// CHARLAS ///////
		// Número de charlas
		strqry = "SELECT COUNT(ch) " +
				"FROM " + Charla.class.getName() + " ch " +
				"WHERE (ch.desactivado = false) AND " +
				"(ch.centro.id = :idCentro) AND " +
				"((ch.fechaInicio <= :fechaFin) AND (ch.fechaFin >= :fechaInicio))";
		
		startTime = System.nanoTime();
		nCharlas = (Long)getQuerySingleResult(strqry, idCentro, fechaInicio, fechaFin);
		
		a.setNumeroCharlas(nCharlas);
		logger.info("ReportHelperByCentro:getActivities:Número de charlas: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		
		if(nCharlas > 0) {
			// Número de horas de charlas
			strqry = "SELECT COALESCE(SUM(ROUND((HOUR(chci.end) - HOUR(chci.start)) + ((MINUTE(chci.end) - MINUTE(chci.start)) / 60.00F), 2)), 0.00F) + 0.00F " +
					 "FROM " + Charla.class.getName() + " ch INNER JOIN " +
					 "ch.calendarItems chci " +
					 "WHERE (ch.desactivado = false) AND " +
					 "(ch.centro.id = :idCentro) AND " +
					 "((ch.fechaInicio <= :fechaFin) AND (ch.fechaFin >= :fechaInicio)) AND " + 
					 "((chci.start <= :fechaFin) AND (chci.end >= :fechaInicio))";
			
			startTime = System.nanoTime();
			a.setNumeroHorasCharlas((Float)getQuerySingleResult(strqry, idCentro, fechaInicio, fechaFin));
			logger.info("ReportHelperByCentro:getActivities:Número de horas de charlas: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		
			// Número horas terminadas de charlas (Horas asistidas)
			strqry = "SELECT COALESCE(SUM(ROUND((HOUR(ach.end) - HOUR(ach.start)) + ((MINUTE(ach.end) - MINUTE(ach.start)) / 60.00F), 2)), 0.00F) + 0.00F " +
					 "FROM " + Charla.class.getName() + " ch INNER JOIN " +
					 "ch.asistencias ach " +
					 "WHERE (ch.desactivado = false) AND " +
					 "(ch.centro.id = :idCentro) AND " +
					 "((ch.fechaInicio <= :fechaFin) AND (ch.fechaFin >= :fechaInicio)) AND " + 
					 "((ach.start <= :fechaFin) AND (ach.end >= :fechaInicio)) AND " + 
					 "(ach.excepcion <> :excepcionAsistencia)";
			
			startTime = System.nanoTime();
			a.setNumeroHorasTerminadasCharlas((Float)getQuerySingleResult(strqry, idCentro, fechaInicio, fechaFin, AsistenciaCalendarItem.Excepcion.JUSTIFICADA));
			logger.info("ReportHelperByCentro:getActivities:Número horas terminadas de charlas: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		
			// Número de plazas ofertadas de charlas
			strqry = "SELECT COALESCE(SUM(ch.numeroPlazas), 0) " +
					 "FROM " + Charla.class.getName() + " ch " +
					 "WHERE (ch.desactivado = false) AND " +
					 "(ch.centro.id = :idCentro) AND " +
					 "((ch.fechaInicio <= :fechaFin) AND (ch.fechaFin >= :fechaInicio))";
			
			startTime = System.nanoTime();
			a.setNumeroPlazasOfertadasCharlas((Long)getQuerySingleResult(strqry, idCentro, fechaInicio, fechaFin));
			logger.info("ReportHelperByCentro:getActivities:Número de plazas ofertadas de charlas: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			
			// Número de plazas cubiertas en charlas (usuarios matriculados)
			strqry = "SELECT COUNT(mch) " +
				   	 "FROM " + Charla.class.getName() + " ch INNER JOIN " +
				   	 "ch.matriculas mch " +
				   	 "WHERE (ch.desactivado = false) AND " +
					 "(ch.centro.id = :idCentro) AND " +
					 "((ch.fechaInicio <= :fechaFin) AND (ch.fechaFin >= :fechaInicio)) AND " +
				   	 "(mch.admitido = TRUE) AND " +
				   	 "(mch.enListaEspera = FALSE)";
			
			startTime = System.nanoTime();
			a.setNumeroPlazasCubiertasCharlas((Long)getQuerySingleResult(strqry, idCentro, fechaInicio, fechaFin));
			logger.info("ReportHelperByCentro:getActivities:Número de plazas terminadas en charlas: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		
			// Número de plazas terminadas en charlas (usuarios matriculados con asistencia >= 80%)
			strqry = "SELECT DISTINCT ch, ach.usuario.id " +
		   			 "FROM " + Charla.class.getName() + " ch INNER JOIN " +
		   			 "ch.asistencias ach " +
		   			 "WHERE (ch.desactivado = false) AND " +
					 "(ch.centro.id = :idCentro) AND " +
					 "((ch.fechaInicio <= :fechaFin) AND (ch.fechaFin >= :fechaInicio)) AND " +
					 "(ach.excepcion <> :excepcionAsistencia) " +
		   			 "GROUP BY ch, ach.usuario " +
		   			 "HAVING ((COUNT(ach) * 100.00) / ch.calendarItems.size) >= 80.00";
			
			startTime = System.nanoTime();
			a.setNumeroPlazasTerminadasCharlas(getSizeOfQueryResultList(strqry, idCentro, fechaInicio, fechaFin, AsistenciaCalendarItem.Excepcion.JUSTIFICADA));
			logger.info("ReportHelperByCentro:getActivities:Número de plazas terminadas en charlas: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		} else {
			a.setNumeroHorasCharlas(0F);
			a.setNumeroHorasTerminadasCharlas(0F);
			a.setNumeroPlazasOfertadasCharlas(0L);
			a.setNumeroPlazasCubiertasCharlas(0L);
			a.setNumeroPlazasTerminadasCharlas(0L);
		}
		
		// Número de actividades externas en cesión de aula
		strqry = "SELECT COUNT(ae) " +
				"FROM " + ActividadExterna.class.getName() + " ae " +
				"WHERE (ae.desactivado = false) AND " +
				"(ae.centro.id = :idCentro) AND " +
				"(ae.servicio IS EMPTY) AND " +
				"((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio))";
		
		startTime = System.nanoTime();
		nExternasCA = (Long)getQuerySingleResult(strqry, idCentro, fechaInicio, fechaFin);
		
		a.setNumeroExternasCA(nExternasCA);
		logger.info("ReportHelperByCentro:getActivities:Número de actividades externas en cesión de aula: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		
		if(nExternasCA > 0) {
			// Número de horas de actividades externas en cesión de aula
			strqry = "SELECT COALESCE(SUM(ROUND((HOUR(aeci.end) - HOUR(aeci.start)) + ((MINUTE(aeci.end) - MINUTE(aeci.start)) / 60.00F), 2)), 0.00F) + 0.00F " +
					 "FROM " + ActividadExterna.class.getName() + " ae INNER JOIN " +
					 "ae.calendarItems aeci " +
					 "WHERE (ae.desactivado = false) AND " +
					 "(ae.centro.id = :idCentro) AND " +
					 "(ae.servicio IS EMPTY) AND " +
					 "((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio)) AND " + 
					 "((aeci.start <= :fechaFin) AND (aeci.end >= :fechaInicio))";
			
			startTime = System.nanoTime();
			a.setNumeroHorasExternasCA((Float)getQuerySingleResult(strqry, idCentro, fechaInicio, fechaFin));
			logger.info("ReportHelperByCentro:getActivities:Número de horas de actividades externas en cesión de aula: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		
			// Número de plazas ofertadas de actividades externas en cesión de aula
			strqry = "SELECT COALESCE(SUM(ae.numeroPlazas), 0) " +
				   	 "FROM " + ActividadExterna.class.getName() + " ae " +
				   	 "WHERE (ae.desactivado = false) AND " +
					 "(ae.centro.id = :idCentro) AND " +
					 "(ae.servicio IS EMPTY) AND " +
					 "((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio))";
			
			startTime = System.nanoTime();
			a.setNumeroPlazasOfertadasExternasCA((Long)getQuerySingleResult(strqry, idCentro, fechaInicio, fechaFin));
			logger.info("ReportHelperByCentro:getActivities:Número de plazas ofertadas de actividades externas en cesión de aula: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		
			// Número de plazas terminadas en actividades externas en cesión de aula (usuarios con las encuestas de satisfacción cumplemientadas)
			strqry = "SELECT COUNT(fcae) " +
				     "FROM " + ActividadExterna.class.getName() + " ae INNER JOIN " +
				     "ae.formulariosConcretos fcae " +
				     "WHERE (ae.desactivado = false) AND " +
					 "(ae.centro.id = :idCentro) AND " +
					 "(ae.servicio IS EMPTY) AND " +
					 "((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio))";
			
			startTime = System.nanoTime();
			a.setNumeroPlazasTerminadasExternasCA((Long)getQuerySingleResult(strqry, idCentro, fechaInicio, fechaFin));
			logger.info("ReportHelperByCentro:getActivities:Número de plazas terminadas en actividades externas en cesión de aula: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		} else {
			a.setNumeroHorasExternasCA(0F);
			a.setNumeroPlazasOfertadasExternasCA(0L);
			a.setNumeroPlazasTerminadasExternasCA(0L);
		}
		
		// Número de actividades externas en cesión de servicio
		a.setNumeroExternasCS(0L);
		// Número de horas de actividades externas en cesión de servicio
		a.setNumeroHorasExternasCS(0F);
		// Número de plazas ofertadas de actividades externas en cesión de servicio
		a.setNumeroPlazasOfertadasExternasCS(0L);
		// Número de plazas terminadas en actividades externas en cesión de servicio (usuarios con las encuestas de satisfacción cumplemientadas)
		a.setNumeroPlazasTerminadasExternasCS(0L);
		
		// Número de actividades externas en cesión de aula y servicio
		strqry = "SELECT COUNT(ae) " +
				 "FROM " + ActividadExterna.class.getName() + " ae " +
				 "WHERE (ae.desactivado = false) AND " +
				 "(ae.centro.id = :idCentro) AND " +
				 "(ae.servicio IS NOT EMPTY) AND " +
				 "((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio))";
		
		startTime = System.nanoTime();
		nExternasCAyS = (Long)getQuerySingleResult(strqry, idCentro, fechaInicio, fechaFin);
		
		a.setNumeroExternasCAyS(nExternasCAyS);
		logger.info("ReportHelperByCentro:getActivities:Número de actividades externas en cesión de aula y servicio: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		
		if(nExternasCAyS > 0) {
			// Número de horas de actividades externas en cesión de aula y servicio
			strqry = "SELECT COALESCE(SUM(ROUND((HOUR(aeci.end) - HOUR(aeci.start)) + ((MINUTE(aeci.end) - MINUTE(aeci.start)) / 60.00F), 2)), 0.00F) + 0.00F " +
					 "FROM " + ActividadExterna.class.getName() + " ae INNER JOIN " +
					 "ae.calendarItems aeci " +
					 "WHERE (ae.desactivado = false) AND " +
					 "(ae.centro.id = :idCentro) AND " +
					 "(ae.servicio IS NOT EMPTY) AND " +
					 "((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio)) AND " + 
					 "((aeci.start <= :fechaFin) AND (aeci.end >= :fechaInicio))";
			
			startTime = System.nanoTime();
			a.setNumeroHorasExternasCAyS((Float)getQuerySingleResult(strqry, idCentro, fechaInicio, fechaFin));
			logger.info("ReportHelperByCentro:getActivities:Número de horas de actividades externas en cesión de aula y servicio: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		
			// Número de plazas ofertadas de actividades externas en cesión de aula y servicio
			strqry = "SELECT COALESCE(SUM(ae.numeroPlazas), 0) " +
				   	 "FROM " + ActividadExterna.class.getName() + " ae " +
				   	 "WHERE (ae.desactivado = false) AND " +
					 "(ae.centro.id = :idCentro) AND " +
					 "(ae.servicio IS NOT EMPTY) AND " +
					 "((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio))";
			
			startTime = System.nanoTime();
			a.setNumeroPlazasOfertadasExternasCAyS((Long)getQuerySingleResult(strqry, idCentro, fechaInicio, fechaFin));
			logger.info("ReportHelperByCentro:getActivities:Número de plazas ofertadas de actividades externas en cesión de aula y servicio: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			
			// Número de plazas terminadas en actividades externas en cesión de aula y servicio (usuarios con las encuestas de satisfacción cumplemientadas)
			strqry = "SELECT COUNT(fcae) " +
				     "FROM " + ActividadExterna.class.getName() + " ae INNER JOIN " +
				     "ae.formulariosConcretos fcae " +
				     "WHERE (ae.desactivado = false) AND " +
					 "(ae.centro.id = :idCentro) AND " +
					 "(ae.servicio IS NOT EMPTY) AND " +
					 "((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio))";
			
			startTime = System.nanoTime();
			a.setNumeroPlazasTerminadasExternasCAyS((Long)getQuerySingleResult(strqry, idCentro, fechaInicio, fechaFin));
			logger.info("ReportHelperByCentro:getActivities:Numero de plazas terminadas en actividades externas en cesión de aula y servicio: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			
		} else {
			a.setNumeroHorasExternasCAyS(0F);
			a.setNumeroPlazasOfertadasExternasCAyS(0L);
			a.setNumeroPlazasTerminadasExternasCAyS(0L);
		}
		
		return a;
	}
	
	private List<OcupacionOfActivitiesByType> getOcupacionOfActivitiesByTypeList(Activities gar) {
	    List<OcupacionOfActivitiesByType> loat = new ArrayList<OcupacionOfActivitiesByType>(3);

	    loat.add(0, new OcupacionOfActivitiesByType("Cursos", gar.getNumeroPlazasCubiertasCursos(), gar.getNumeroPlazasOfertadasCursos()));

	    loat.add(1, new OcupacionOfActivitiesByType("Charlas", gar.getNumeroPlazasCubiertasCharlas(), gar.getNumeroPlazasOfertadasCharlas()));

	    loat.add(2, new OcupacionOfActivitiesByType("Cursos entidades colaboradoras",	
	    											gar.getNumeroPlazasCubiertasExternasCA() + gar.getNumeroPlazasCubiertasExternasCS() + gar.getNumeroPlazasCubiertasExternasCAyS(),
	    											gar.getNumeroPlazasOfertadasExternasCA() + gar.getNumeroPlazasOfertadasExternasCS() + gar.getNumeroPlazasOfertadasExternasCAyS()));

		return loat;
	}
	
	private Long getMaxValueInOcupacionOfActivitiesByTypeList(Activities gar) {
	    Long maximo = gar.getNumeroPlazasCubiertasCursos();
	    
	    if (gar.getNumeroPlazasOfertadasCursos() > maximo)
	    	maximo = gar.getNumeroPlazasOfertadasCursos();

	    
	    if (gar.getNumeroPlazasCubiertasCharlas() > maximo)
	    	maximo = gar.getNumeroPlazasCubiertasCharlas();

	    
	    if (gar.getNumeroPlazasOfertadasCharlas() > maximo)
	    	maximo = gar.getNumeroPlazasOfertadasCharlas();
	    
	    
	    Long numeroPlazasCubiertasExternas = gar.getNumeroPlazasCubiertasExternasCA() +
	    									 gar.getNumeroPlazasCubiertasExternasCS() +
	    									 gar.getNumeroPlazasCubiertasExternasCAyS();
	    if (numeroPlazasCubiertasExternas > maximo)
	    	maximo = numeroPlazasCubiertasExternas;

	    
	    Long numeroPlazasOfertadasExternas = gar.getNumeroPlazasOfertadasExternasCA() +
   	 										 gar.getNumeroPlazasOfertadasExternasCS() +
   	 										 gar.getNumeroPlazasOfertadasExternasCAyS();
	    if (numeroPlazasOfertadasExternas > maximo)
	    	maximo = numeroPlazasOfertadasExternas;

	    
		return maximo;
	}
	
	private List<ActivitiesByService> getCursosByServiceList(Long idCentroForStatistic, Date fechaInicio, Date fechaFin) {
			 //Map<Long, ActivitiesAndRankingOfCenter> maarc) {
		// Crea una lista de centros
		startTime = System.nanoTime();
		//3.4.8 - RF_15_22
		Centro c = service.find(idCentroForStatistic);
		//List<Centro> lc = service.findAllOrder(Centro.class, "nombre", true);
		logger.info("ReportHelperByCentro:getCursosByServiceList:lista de centros: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		
		// Crea una lista de servicios que tiene cursos asociados entre las fechas especificadas indicando sus temáticas
		String strqry = "SELECT t1.id AS idTematica1, " +
							"tl1.nombre AS nombreTematica1, " +
							"t2.id AS idTematica2, " +
							"tl2.nombre AS nombreTematica2, " +
							"s.id AS idServicio, " +
							"sl.nombre AS nombreServicio " +
						"FROM " + Curso.class.getName() + " cu INNER JOIN " +
							"cu.servicio s INNER JOIN " +
							"s.textos sl INNER JOIN " +
							"s.tematica t2 INNER JOIN " +
							"t2.textos tl2 INNER JOIN " +
							"t2.padre t1 INNER JOIN " +
							"t1.textos tl1 " +
						"WHERE (cu.desactivado = false) AND " +
							"(sl.language.ISO = :languageISO) AND " +
							"(tl1.language.ISO = :languageISO) AND " +
							"(tl2.language.ISO = :languageISO) AND " +
							"((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio)) " +
						"GROUP BY t1, " +
							"t2, " +
							"s " +
							"ORDER BY tl1.nombre ASC, " +
							"tl2.nombre ASC, " +
							"sl.nombre ASC, " +
							"t1.id ASC, " +
							"t2.id ASC, " +
							"s.id ASC";
		
		Query qry = service.getEntityManager().createQuery(strqry);
		
		qry.setParameter("languageISO", locale.toString());
		qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
		qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
		
		startTime = System.nanoTime();
		@SuppressWarnings("unchecked")
		List<Object[]> ls = (List<Object[]>)qry.getResultList();
		logger.info("ReportHelperByCentro:getCursosByServiceList:lista de servicios que tiene cursos asociados: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		
		// Crea la lista vacía para almacenar el resultados a devolver
		List<ActivitiesByService> las = new ArrayList<ActivitiesByService>();
		
		// Recorre la lista de servicios
		for (Object[] lobj : ls) {
			// Extrae los datos del servicio
			Long idT1             = (Long)lobj[0];	 // Identificador de tematica 1er nivel
			String nombreT1       = (String)lobj[1]; // Nombre de tematica 1er nivel
			Long idT2             = (Long)lobj[2];	 // Identificador de tematica 2º nivel
			String nombreT2       = (String)lobj[3]; // Nombre de tematica 2º nivel
			Long idServicio       = (Long)lobj[4];	 // Identificador de servicio
			String nombreServicio = (String)lobj[5]; // Nombre de servicio
		
			//3.4.8 - RF_15_22
			// Consulta el número de cursos y el número de plazas ofertadas de cursos para el centro y el servicio
			strqry = "SELECT COALESCE(COUNT(cu) + 0L, 0L) AS numeroCursos, " +
						"COALESCE(SUM(cu.numeroPlazas) + 0L, 0L) AS numeroPlazasOfertadasCursos " +
					 "FROM " + Curso.class.getName() + " cu " +
					 "WHERE (cu.desactivado = false) AND " +
					 	"(cu.centro.id = :idCentro) AND " +
					 	"(cu.servicio.id = :idServicio) AND " +							   
					 	"((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio))";
			qry = service.getEntityManager().createQuery(strqry);
		
			qry.setParameter("idCentro", c.getId());
			qry.setParameter("idServicio", idServicio);
			qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
			qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
			
			startTime = System.nanoTime();
			Object[] l_nCu_nPCu = (Object[])qry.getSingleResult();
			logger.info("ReportHelperByCentro:getCursosByServiceList:número de cursos y el número de plazas ofertadas de cursos para el centro y el servicio: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		
			// Extrae los resultados
			Long numeroActividades = (Long)l_nCu_nPCu[0];
			Long plazasOfertadas = (Long)l_nCu_nPCu[1];
			Float horas = 0F;
			Long usuarios = 0L;
			
			if(numeroActividades > 0) {
				// Consulta el número de horas de cursos para el centro y el servicio entre las fechas del informe
				strqry = "SELECT COALESCE(SUM(ROUND((HOUR(cuci.end) - HOUR(cuci.start)) + ((MINUTE(cuci.end) - MINUTE(cuci.start)) / 60.00F), 2)), 0.00F) + 0.00F " +
						"FROM " + Curso.class.getName() + " cu INNER JOIN " +
							"cu.calendarItems cuci " +
						"WHERE (cu.desactivado = false) AND " +
							"(cu.centro.id = :idCentro) AND " +
							"(cu.servicio.id = :idServicio) AND " +
							"((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio)) AND " +
							"((cuci.start <= :fechaFin) AND (cuci.end >= :fechaInicio))";
		
				qry = service.getEntityManager().createQuery(strqry);
		
				qry.setParameter("idCentro", c.getId());
				qry.setParameter("idServicio", idServicio);
				qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
				qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
		
				startTime = System.nanoTime();
				horas = (Float)qry.getSingleResult();
				logger.info("ReportHelperByCentro:getCursosByServiceList: número de horas de cursos para el centro y el servicio entre las fechas del informe: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		
				// Consulta el número de usuarios de cursos para el centro y el servicio
				strqry = "SELECT DISTINCT cu, acu.usuario.id " +
						 "FROM " + Curso.class.getName() + " cu INNER JOIN " +
						 	"cu.asistencias acu " +
						 "WHERE (cu.desactivado = false) AND " +
						 	"(cu.centro.id = :idCentro) AND " +
						 	"(cu.servicio.id = :idServicio) AND " +
						 	"((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio)) AND " +
						 	"(acu.excepcion <> :excepcionAsistencia) " +
						 "GROUP BY cu, acu.usuario " +
						 "HAVING ((COUNT(acu) * 100.00F) / cu.calendarItems.size) >= 80.00F";
		
				qry = service.getEntityManager().createQuery(strqry);
		
				qry.setParameter("idCentro", c.getId());
				qry.setParameter("idServicio", idServicio);
				qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
				qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
				qry.setParameter("excepcionAsistencia", AsistenciaCalendarItem.Excepcion.JUSTIFICADA);
		
				// Extrae el resultado
				startTime = System.nanoTime();
				usuarios = (long)qry.getResultList().size();
				logger.info("ReportHelperByCentro:getCursosByServiceList:número de usuarios de cursos para el centro y el servicio: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			}
		
			////////////////Almacenamiento de datos para calculo de Ranking////////////////
			//Si no existe el registro del centro en maarc, lo crea
			/*if (!maarc.containsKey(c.getId())) {
				ActivitiesAndRankingOfCenter aarc = new ActivitiesAndRankingOfCenter();
				aarc.setIdCentro(c.getId());
				maarc.put(c.getId(), aarc);
			}*/
		
			//Añade los valores de la consulta al registro del centro en maarc
			/*maarc.get(c.getId()).setNumeroActividades(maarc.get(c.getId()).getNumeroActividades() + numeroActividades);
			maarc.get(c.getId()).setHoras(maarc.get(c.getId()).getHoras() + horas);
			maarc.get(c.getId()).setUsuarios(maarc.get(c.getId()).getUsuarios() + usuarios);
			maarc.get(c.getId()).setPlazasOfertadas(maarc.get(c.getId()).getPlazasOfertadas() + plazasOfertadas);*/
			////////////////Almacenamiento de datos para calculo de Ranking////////////////
		
			// Si es un registro del centro del que se desea extraer los datos, se almacena para devolverlo al final
			ActivitiesByService as = new ActivitiesByService(
			idT1,			   // Identificador de tematica 1er nivel
			nombreT1,		   // Nombre de tematica 1er nivel
			idT2,			   // Identificador de tematica 2º nivel
			nombreT2,		   // Nombre de tematica 2º nivel
			idServicio,		   // Identificador de servicio
			nombreServicio,	   // Nombre de servicio
			numeroActividades, // Número de actividades
			horas,			   // Número de horas
			usuarios,		   // Número de ususarios
			plazasOfertadas	   // Número de plazas ofertadas
			);
		
			las.add(as);
		}
		
		return las;
	}
	
	private List<ActivitiesByService> getCharlasByServiceList(Long idCentroForStatistic, Date fechaInicio, Date fechaFin) {
			   //Map<Long, ActivitiesAndRankingOfCenter> maarc) {
		// Crea una lista de centros
		startTime = System.nanoTime();
		Centro c = service.find(idCentroForStatistic);
		//List<Centro> lc = service.findAllOrder(Centro.class, "nombre", true);
		logger.info("ReportHelperByCentro:getCharlasByServiceList:lista de centros: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);

		// Crea una lista de temáticas de nivel 2 que tiene charlas asociados entre las fechas especificadas
		//indicando sus temáticas de nivel 1
		String strqry = "SELECT t1.id AS idTematica1, " +
							"tl1.nombre AS nombreTematica1, " +
							"t2.id AS idTematica2, " +
							"tl2.nombre AS nombreTematica2 " +
						"FROM " + Charla.class.getName() + " ch INNER JOIN " +
							"ch.tematica t2 INNER JOIN " +
							"t2.textos tl2 INNER JOIN " +
							"t2.padre t1 INNER JOIN " +
							"t1.textos tl1 " +
						"WHERE (ch.desactivado = false) AND " +
							"(tl1.language.ISO = :languageISO) AND " +
							"(tl2.language.ISO = :languageISO) AND " +
							"((ch.fechaInicio <= :fechaFin) AND (ch.fechaFin >= :fechaInicio)) " +
						"GROUP BY t1, " +
							"t2 " +
							"ORDER BY tl1.nombre ASC, " +
							"tl2.nombre ASC, " +
							"t1.id ASC, " +
							"t2.id ASC ";

		Query qry = service.getEntityManager().createQuery(strqry);

		qry.setParameter("languageISO", locale.toString());
		qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
		qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);

		startTime = System.nanoTime();
		@SuppressWarnings("unchecked")
		List<Object[]> lt2 = (List<Object[]>)qry.getResultList();
		logger.info("ReportHelperByCentro:getCharlasByServiceList:lista de temáticas de nivel 2 que tiene charlas asociados entre las fechas especificadas indicando sus temáticas de nivel 1: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);

		//Crea la lista vacía para almacenar el resultados a devolver
		List<ActivitiesByService> las = new ArrayList<ActivitiesByService>();

		// Recorre la lista de temáticas de nivel 2
		for (Object[] lobj : lt2) {
			// Extrae los datos de la temática
			Long idT1             = (Long)lobj[0];	  // Identificador de tematica 1er nivel
			String nombreT1       = (String)lobj[1];  // Nombre de tematica 1er nivel
			Long idT2             = (Long)lobj[2];	  // Identificador de tematica 2º nivel
			String nombreT2       = (String)lobj[3];  // Nombre de tematica 2º nivel
			Long idServicio       = Long.valueOf(0L); // Identificador de servicio
			String nombreServicio = "";				  // Nombre de servicio

			// Consulta el número de charlas y el número de plazas ofertadas de charlas para
			// el centro y el servicio
			strqry = "SELECT COALESCE(COUNT(ch) + 0L, 0L) AS numeroCharlas, " +
						"COALESCE(SUM(ch.numeroPlazas) + 0L, 0L) AS numeroPlazasOfertadasCharlas " +
					 "FROM " + Charla.class.getName() + " ch " +
					 "WHERE (ch.desactivado = false) AND " +
					 "(ch.centro.id = :idCentro) AND " +
					 "(ch.tematica.id = :idTematica) AND " +
					 "((ch.fechaInicio <= :fechaFin) AND (ch.fechaFin >= :fechaInicio))";

			qry = service.getEntityManager().createQuery(strqry);

			qry.setParameter("idCentro", c.getId());
			qry.setParameter("idTematica", idT2);
			qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
			qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);

			startTime = System.nanoTime();
			Object[] l_nCh_nPCh = (Object[])qry.getSingleResult();
			logger.info("ReportHelperByCentro:getCharlasByServiceList:número de charlas y el número de plazas ofertadas de charlas para el centro y el servicio: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);

			// Extrae los resultados
			Long numeroActividades = (Long)l_nCh_nPCh[0];
			Long plazasOfertadas = (Long)l_nCh_nPCh[1];
			Float horas = 0F;
			Long usuarios = 0L;
			
			if(numeroActividades > 0) {
				// Consulta el número de horas de charlas para el centro y el servicio
				strqry = "SELECT COALESCE(SUM(ROUND((HOUR(chci.end) - HOUR(chci.start)) + ((MINUTE(chci.end) - MINUTE(chci.start)) / 60.00F), 2)), 0.00F) + 0.00F AS numeroHorasCharlas " +
						 "FROM " + Charla.class.getName() + " ch INNER JOIN " +
						 	"ch.calendarItems chci " +
						 "WHERE (ch.desactivado = false) AND " +
						 	"(ch.centro.id = :idCentro) AND " +
						 	"(ch.tematica.id = :idTematica) AND " +
						 	"((ch.fechaInicio <= :fechaFin) AND (ch.fechaFin >= :fechaInicio)) AND " + //AND is new
						 	"((chci.start <= :fechaFin) AND (chci.end >= :fechaInicio))"; //new line
	
				qry = service.getEntityManager().createQuery(strqry);
	
				qry.setParameter("idCentro", c.getId());
				qry.setParameter("idTematica", idT2);
				qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
				qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
	
				// Extrae el resultado
				startTime = System.nanoTime();
				horas = (Float)qry.getSingleResult();
				logger.info("ReportHelperByCentro:getCharlasByServiceList:número de horas de charlas para el centro y el servicio: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	
				// Consulta el número de usuarios en charlas para el centro y el servicio
				strqry = "SELECT DISTINCT ch, ach.usuario.id " +
						 "FROM " + Charla.class.getName() + " ch INNER JOIN " +
						 	"ch.asistencias ach " +
						 "WHERE (ch.desactivado = false) AND " +
						 	"(ch.centro.id = :idCentro) AND " +
						 	"(ch.tematica.id = :idTematica) AND " +
						 	"((ch.fechaInicio <= :fechaFin) AND (ch.fechaFin >= :fechaInicio)) AND " +
						 	"(ach.excepcion <> :excepcionAsistencia) " +
						 "GROUP BY ch, ach.usuario " +
						 "HAVING ((COUNT(ach) * 100.00F) / ch.calendarItems.size) >= 80.00F";
	
				qry = service.getEntityManager().createQuery(strqry);
	
				qry.setParameter("idCentro", c.getId());
				qry.setParameter("idTematica", idT2);
				qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
				qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
				qry.setParameter("excepcionAsistencia", AsistenciaCalendarItem.Excepcion.JUSTIFICADA);
	
				// Extrae el resultado
				startTime = System.nanoTime();
				usuarios = (long)qry.getResultList().size();
				logger.info("ReportHelperByCentro:getCharlasByServiceList:número de usuarios en charlas para el centro y el servicio: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			}
			
			////////////////Almacenamiento de datos para calculo de Ranking////////////////
			//Si no existe el registro del centro en maarc, lo crea
			/*if (!maarc.containsKey(c.getId())) {
				ActivitiesAndRankingOfCenter aarc = new ActivitiesAndRankingOfCenter();
				aarc.setIdCentro(c.getId());

				maarc.put(c.getId(), aarc);
			}

			//Añade los valores de la consulta al registro del centro en maarc
			maarc.get(c.getId()).setNumeroActividades(maarc.get(c.getId()).getNumeroActividades() + numeroActividades);
			maarc.get(c.getId()).setHoras(maarc.get(c.getId()).getHoras() + horas);
			maarc.get(c.getId()).setUsuarios(maarc.get(c.getId()).getUsuarios() + usuarios);
			maarc.get(c.getId()).setPlazasOfertadas(maarc.get(c.getId()).getPlazasOfertadas() + plazasOfertadas);*/
			////////////////Fin de almacenamiento de datos para calculo de Ranking////////////////

			// Si es un registro del centro del que se desea extraer los datos, se almacena para devolverlo al final
			ActivitiesByService as = new ActivitiesByService(
			idT1,			   // Identificador de tematica 1er nivel
			nombreT1,		   // Nombre de tematica 1er nivel
			idT2,			   // Identificador de tematica 2º nivel
			nombreT2,		   // Nombre de tematica 2º nivel
			idServicio,		   // Identificador de servicio
			nombreServicio,	   // Nombre de servicio
			numeroActividades, // Número de actividades
			horas,			   // Número de horas
			usuarios,		   // Número de ususarios
			plazasOfertadas	   // Número de plazas ofertadas
			);

			las.add(as);
		}

		return las;
	}
	
	private List<ActivitiesByService> getExternasCAByServiceList(Long idCentroForStatistic, Date fechaInicio, Date fechaFin) {
			   //Map<Long, ActivitiesAndRankingOfCenter> maarc) {
		// Crea una lista de centros
		startTime = System.nanoTime();
		//3.4.8 - RF_15_22
		Centro c = service.find(idCentroForStatistic);
		//List<Centro> lc = service.findAllOrder(Centro.class, "nombre", true);
		logger.info("ReportHelperByCentro:getExternasCAByServiceList:lista de centros: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);

		// Crea una lista de temáticas de nivel 2 que tiene actividades externas en cesión de aula asociados
		// entre las fechas especificadas indicando sus temáticas de nivel 1
		String strqry = "SELECT t1.id AS idTematica1, " +
							"tl1.nombre AS nombreTematica1, " +
							"t2.id AS idTematica2, " +
							"tl2.nombre AS nombreTematica2 " +
						"FROM " + ActividadExterna.class.getName() + " ae INNER JOIN " +
							"ae.tematica t2 INNER JOIN " +
							"t2.textos tl2 INNER JOIN " +
							"t2.padre t1 INNER JOIN " +
							"t1.textos tl1 " +
						"WHERE (ae.desactivado = false) AND " +
							"(tl1.language.ISO = :languageISO) AND " +
							"(tl2.language.ISO = :languageISO) AND " +
							"(ae.centro IS NOT EMPTY) AND " +
							"(ae.servicio IS EMPTY) AND " +
							"((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio)) " +
						"GROUP BY t1, " +
							"t2 " +
						"ORDER BY tl1.nombre ASC, " +
							"tl2.nombre ASC, " +
							"t1.id ASC, " +
							"t2.id ASC ";

		Query qry = service.getEntityManager().createQuery(strqry);

		qry.setParameter("languageISO", locale.toString());
		qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
		qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);

		startTime = System.nanoTime();
		@SuppressWarnings("unchecked")
		List<Object[]> lt2 = (List<Object[]>)qry.getResultList();
		logger.info("ReportHelperByCentro:getExternasCAByServiceList:lista de temáticas de nivel 2 que tiene actividades externas en cesión de aula asociados entre las fechas especificadas indicando sus temáticas de nivel 1: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);

		// Crea la lista vacía para almacenar el resultados a devolver
		List<ActivitiesByService> las = new ArrayList<ActivitiesByService>();

		// Recorre la lista de temáticas de nivel 2
		for (Object[] lobj : lt2) {
			// Extrae los datos de la temática
			Long idT1             = (Long)lobj[0];	  // Identificador de tematica 1er nivel
			String nombreT1       = (String)lobj[1];  // Nombre de tematica 1er nivel
			Long idT2             = (Long)lobj[2];	  // Identificador de tematica 2º nivel
			String nombreT2       = (String)lobj[3];  // Nombre de tematica 2º nivel
			Long idServicio       = Long.valueOf(0L); // Identificador de servicio
			String nombreServicio = "";				  // Nombre de servicio

			// Consulta el número de actividades externas en cesión de aula y el número de plazas ofertadas
			// de las mismas para el centro y el servicio
			strqry = "SELECT COALESCE(COUNT(ae) + 0L, 0L) AS numeroExternasCA, " +
					 	"COALESCE(SUM(ae.numeroPlazas) + 0L, 0L) AS numeroPlazasOfertadasExternasCA " +
					 "FROM " + ActividadExterna.class.getName() + " ae " +
					 "WHERE (ae.desactivado = false) AND " +
					 	"(ae.centro IS NOT EMPTY) AND " +
					 	"(ae.centro.id = :idCentro) AND " +
					 	"(ae.servicio IS EMPTY) AND " +
					 	"(ae.tematica.id = :idTematica) AND " +
					 	"((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio))";

			qry = service.getEntityManager().createQuery(strqry);

			qry.setParameter("idCentro", c.getId());
			qry.setParameter("idTematica", idT2);
			qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
			qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);

			startTime = System.nanoTime();
			Object[] l_nCh_nPCh = (Object[])qry.getSingleResult();
			logger.info("ReportHelperByCentro:getExternasCAByServiceList:número de actividades externas en cesión de aula y el número de plazas ofertadas de las mismas para el centro y el servicio: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);

			// Extrae los resultados
			Long numeroActividades = (Long)l_nCh_nPCh[0];
			Long plazasOfertadas = (Long)l_nCh_nPCh[1];
			Float horas = 0F;
			Long usuarios = 0L;
			
			if(numeroActividades > 0) {
				// Consulta el número de horas de actividades externas en cesión de aula para el centro y el servicio
				strqry = "SELECT COALESCE(SUM(ROUND((HOUR(aeci.end) - HOUR(aeci.start)) + ((MINUTE(aeci.end) - MINUTE(aeci.start)) / 60.00F), 2)), 0.00F) + 0.00F AS numeroHorasExternasCA " +
						 "FROM " + ActividadExterna.class.getName() + " ae INNER JOIN " +
						 	"ae.calendarItems aeci " +
						 "WHERE (ae.desactivado = false) AND " +
						 	"(ae.centro IS NOT EMPTY) AND " +
						 	"(ae.centro.id = :idCentro) AND " +
						 	"(ae.servicio IS EMPTY) AND " +
						 	"(ae.tematica.id = :idTematica) AND " +
						 	"((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio)) AND " + //AND is new
						 	"((aeci.start <= :fechaFin) AND (aeci.end >= :fechaInicio))"; //new line
	
				qry = service.getEntityManager().createQuery(strqry);
	
				qry.setParameter("idCentro", c.getId());
				qry.setParameter("idTematica", idT2);
				qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
				qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
	
				// Extrae el resultado
				startTime = System.nanoTime();
				horas = (Float)qry.getSingleResult();
				logger.info("ReportHelperByCentro:getExternasCAByServiceList:número de horas de actividades externas en cesión de aula para el centro y el servicio: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	
				// Consulta el número de usuarios en actividades externas en cesión de aula para el centro y el servicio
				strqry = "SELECT COALESCE(COUNT(fcae) + 0L, 0L) AS numeroUsuariosExternasCA " +
						 "FROM " + ActividadExterna.class.getName() + " ae INNER JOIN " +
						 	"ae.formulariosConcretos fcae " +
						 "WHERE (ae.desactivado = false) AND " +
						 	"(ae.centro IS NOT EMPTY) AND " +
						 	"(ae.centro.id = :idCentro) AND " +
						 	"(ae.servicio IS EMPTY) AND " +
						 	"(ae.tematica.id = :idTematica) AND " +
						 	"((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio))";
	
				qry = service.getEntityManager().createQuery(strqry);
	
				qry.setParameter("idCentro", c.getId());
				qry.setParameter("idTematica", idT2);
				qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
				qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
	
				// Extrae el resultado
				startTime = System.nanoTime();
				usuarios = (Long)qry.getSingleResult();
				logger.info("ReportHelperByCentro:getExternasCAByServiceList:número de usuarios en actividades externas en cesión de aula para el centro y el servicio: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			}
			
			////////////////Almacenamiento de datos para calculo de Ranking////////////////
			//Si no existe el registro del centro en maarc, lo crea
			/*if (!maarc.containsKey(c.getId())) {
				ActivitiesAndRankingOfCenter aarc = new ActivitiesAndRankingOfCenter();
				aarc.setIdCentro(c.getId());

				maarc.put(c.getId(), aarc);
			}

			//Añade los valores de la consulta al registro del centro en maarc
			maarc.get(c.getId()).setNumeroActividades(maarc.get(c.getId()).getNumeroActividades() + numeroActividades);
			maarc.get(c.getId()).setHoras(maarc.get(c.getId()).getHoras() + horas);
			maarc.get(c.getId()).setUsuarios(maarc.get(c.getId()).getUsuarios() + usuarios);
			maarc.get(c.getId()).setPlazasOfertadas(maarc.get(c.getId()).getPlazasOfertadas() + plazasOfertadas);*/
			////////////////Fin de almacenamiento de datos para calculo de Ranking////////////////

			// Si es un registro del centro del que se desea extraer los datos, se almacena para devolverlo al final
			ActivitiesByService as = new ActivitiesByService(
				idT1,			   // Identificador de tematica 1er nivel
				nombreT1,		   // Nombre de tematica 1er nivel
				idT2,			   // Identificador de tematica 2º nivel
				nombreT2,		   // Nombre de tematica 2º nivel
				idServicio,		   // Identificador de servicio
				nombreServicio,	   // Nombre de servicio
				numeroActividades, // Número de actividades
				horas,			   // Número de horas
				usuarios,		   // Número de ususarios
				plazasOfertadas	   // Número de plazas ofertadas
			);

			las.add(as);
		}

		return las;
	}

	private List<ActivitiesByService> getExternasCAySByServiceList(Long idCentroForStatistic, Date fechaInicio, Date fechaFin) {
		    //Map<Long, ActivitiesAndRankingOfCenter> maarc) {
		// Crea una lista de centros
		startTime = System.nanoTime();
		//3.4.8 - RF_15_22
		Centro c = service.find(idCentroForStatistic);
		//List<Centro> lc = service.findAllOrder(Centro.class, "nombre", true);
		logger.info("ReportHelperByCentro:getExternasCAySByServiceList:lista de centros: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);

		// Crea una lista de servicios que tienen actividades externas en cesión de aula y servicio asociadas
		// entre las fechas especificadas indicando sus temáticas
		String strqry = "SELECT t1.id AS idTematica1, " +
							"tl1.nombre AS nombreTematica1, " +
							"t2.id AS idTematica2, " +
							"tl2.nombre AS nombreTematica2, " +
							"s.id AS idServicio, " +
							"sl.nombre AS nombreServicio " +
						"FROM " + ActividadExterna.class.getName() + " ae INNER JOIN " +
							"ae.servicio s INNER JOIN " +
							"s.textos sl INNER JOIN " +
							"s.tematica t2 INNER JOIN " +
							"t2.textos tl2 INNER JOIN " +
							"t2.padre t1 INNER JOIN " +
							"t1.textos tl1 " +
						"WHERE (ae.desactivado = false) AND " +
							"(sl.language.ISO = :languageISO) AND " +
							"(tl1.language.ISO = :languageISO) AND " +
							"(tl2.language.ISO = :languageISO) AND " +
							"(ae.centro IS NOT EMPTY) AND " +
							"(ae.servicio IS NOT EMPTY) AND " +
							"((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio)) " +
						"GROUP BY t1, " +
							"t2, " +
							"s " +
						"ORDER BY tl1.nombre ASC, " +
							"tl2.nombre ASC, " +
							"sl.nombre ASC " +
							"t1.id ASC, " +
							"t2.id ASC, " +
							"s.id ASC";

		Query qry = service.getEntityManager().createQuery(strqry);

		qry.setParameter("languageISO", locale.toString());
		qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
		qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);

		startTime = System.nanoTime();
		@SuppressWarnings("unchecked")
		List<Object[]> ls = (List<Object[]>)qry.getResultList();
		logger.info("ReportHelperByCentro:getExternasCAySByServiceList:lista de servicios que tienen actividades externas en cesión de aula y servicio asociadas entre las fechas especificadas indicando sus temáticas: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);

		// Crea la lista vacía para almacenar el resultados a devolver
		List<ActivitiesByService> las = new ArrayList<ActivitiesByService>();

		// Recorre la lista de servicios
		for (Object[] lobj : ls) {
			// Extrae los datos del servicio
			Long idT1             = (Long)lobj[0];	 // Identificador de tematica 1er nivel
			String nombreT1       = (String)lobj[1]; // Nombre de tematica 1er nivel
			Long idT2             = (Long)lobj[2];	 // Identificador de tematica 2º nivel
			String nombreT2       = (String)lobj[3]; // Nombre de tematica 2º nivel
			Long idServicio       = (Long)lobj[4];	 // Identificador de servicio
			String nombreServicio = (String)lobj[5]; // Nombre de servicio

			// Consulta el número de actividades externas en cesión de aula y servicio y el número de plazas ofertadas
			// de las mismas para el centro y el servicio
			strqry = "SELECT COALESCE(COUNT(ae) + 0L, 0L) AS numeroExternasCAyS, " +
					"COALESCE(SUM(ae.numeroPlazas) + 0L, 0L) AS numeroPlazasOfertadasExternasCAyS " +
					"FROM " + ActividadExterna.class.getName() + " ae " +
					"WHERE (ae.desactivado = false) AND " +
						"(ae.centro IS NOT EMPTY) AND " +
						"(ae.centro.id = :idCentro) AND " +
						"(ae.servicio IS NOT EMPTY) AND " +
						"(ae.servicio.id = :idServicio) AND " +
						"((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio))";

			qry = service.getEntityManager().createQuery(strqry);

			qry.setParameter("idCentro", c.getId());
			qry.setParameter("idServicio", idServicio);
			qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
			qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);

			startTime = System.nanoTime();
			Object[] l_nCh_nPCh = (Object[])qry.getSingleResult();
			logger.info("ReportHelperByCentro:getExternasCAySByServiceList:número de actividades externas en cesión de aula y servicio y el número de plazas ofertadas de las mismas para el centro y el servicio: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);

			// Extrae los resultados
			Long numeroActividades = (Long)l_nCh_nPCh[0];
			Long plazasOfertadas = (Long)l_nCh_nPCh[1];
			Float horas = 0F;
			Long usuarios = 0L;
			
			if(numeroActividades > 0) {
				// Consulta el número de horas de actividades externas en cesión de aula y servicio para el centro
				// y el servicio
				strqry = "SELECT COALESCE(SUM(ROUND((HOUR(aeci.end) - HOUR(aeci.start)) + ((MINUTE(aeci.end) - MINUTE(aeci.start)) / 60.00F), 2)), 0.00F) + 0.00F AS numeroHorasExternasCAyS " +
						 "FROM " + ActividadExterna.class.getName() + " ae INNER JOIN " +
						 	"ae.calendarItems aeci " +
						 "WHERE (ae.desactivado = false) AND " +
						 	"(ae.centro IS NOT EMPTY) AND " +
						 	"(ae.centro.id = :idCentro) AND " +
						 	"(ae.servicio IS NOT EMPTY) AND " +
						 	"(ae.servicio.id = :idServicio) AND " +						 	   
						 	"((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio))";

				qry = service.getEntityManager().createQuery(strqry);

				qry.setParameter("idCentro", c.getId());
				qry.setParameter("idServicio", idServicio);
				qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
				qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);

				// Extrae el resultado
				startTime = System.nanoTime();
				horas = (Float)qry.getSingleResult();
				logger.info("ReportHelperByCentro:getExternasCAySByServiceList:número de horas de actividades externas en cesión de aula y servicio para el centro y el servicio: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);

				// Consulta el número de usuarios en actividades externas en cesión de aula y servicio para el centro
				// y el servicio
				strqry = "SELECT COALESCE(COUNT(fcae) + 0L, 0L) AS numeroUsuariosExternasCAyS " +
						 "FROM " + ActividadExterna.class.getName() + " ae INNER JOIN " +
						 	"ae.formulariosConcretos fcae " +
						 "WHERE (ae.desactivado = false) AND " +
						 	"(ae.centro IS NOT EMPTY) AND " +
						 	"(ae.centro.id = :idCentro) AND " +
						 	"(ae.servicio IS NOT EMPTY) AND " +
						 	"(ae.servicio.id = :idServicio) AND " +
						 	"((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio))";

				qry = service.getEntityManager().createQuery(strqry);

				qry.setParameter("idCentro", c.getId());
				qry.setParameter("idServicio", idServicio);
				qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
				qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);

				// Extrae el resultado
				startTime = System.nanoTime();
				usuarios = (Long)qry.getSingleResult();
				logger.info("ReportHelperByCentro:getExternasCAySByServiceList:número de usuarios en actividades externas en cesión de aula y servicio para el centro y el servicio: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			}
			
			////////////////Almacenamiento de datos para calculo de Ranking////////////////
			//Si no existe el registro del centro en maarc, lo crea
			/*if (!maarc.containsKey(c.getId())) {
				ActivitiesAndRankingOfCenter aarc = new ActivitiesAndRankingOfCenter();
				aarc.setIdCentro(c.getId());
				maarc.put(c.getId(), aarc);
			}

			//Añade los valores de la consulta al registro del centro en maarc
			maarc.get(c.getId()).setNumeroActividades(maarc.get(c.getId()).getNumeroActividades() + numeroActividades);
			maarc.get(c.getId()).setHoras(maarc.get(c.getId()).getHoras() + horas);
			maarc.get(c.getId()).setUsuarios(maarc.get(c.getId()).getUsuarios() + usuarios);
			maarc.get(c.getId()).setPlazasOfertadas(maarc.get(c.getId()).getPlazasOfertadas() + plazasOfertadas);*/
			////////////////Fin de almacenamiento de datos para calculo de Ranking////////////////

			// Si es un registro del centro del que se desea extraer los datos, se almacena para devolverlo al final
			ActivitiesByService as = new ActivitiesByService(
				idT1,			   // Identificador de tematica 1er nivel
				nombreT1,		   // Nombre de tematica 1er nivel
				idT2,			   // Identificador de tematica 2º nivel
				nombreT2,		   // Nombre de tematica 2º nivel
				idServicio,		   // Identificador de servicio
				nombreServicio,	   // Nombre de servicio
				numeroActividades, // Número de actividades
				horas,			   // Número de horas
				usuarios,		   // Número de ususarios
				plazasOfertadas	   // Número de plazas ofertadas
			);

			las.add(as);
		}
		
		return las;
	}
	
	public Map<Long, ActivitiesAndRankingOfCenter> CalculateRankings(Map<Long, ActivitiesAndRankingOfCenter> maarc) {
		/*List<ActivitiesAndRankingOfCenter> laarc = new ArrayList<ActivitiesAndRankingOfCenter>(maarc.values());
		
		Long longValue;
		Float floatValue;
		Long ranking;
		
		// Ordena por número de actividades de mayor a menor
		Collections.sort(laarc, ActivitiesAndRankingOfCenter.comparatorNumeroActividadesDESC);

		//Asigna el ranking por número de actividades
		ranking = Long.valueOf(0L);
		longValue = Long.MAX_VALUE;
		for (ActivitiesAndRankingOfCenter aarc : laarc) {
			if (longValue > aarc.getNumeroActividades()) {
				longValue = aarc.getNumeroActividades();
				
				ranking++;
			}

			aarc.setRankingNumeroActividades(ranking);
		}
		
		// Ordena por número de horas de mayor a menor
		Collections.sort(laarc, ActivitiesAndRankingOfCenter.comparatorHorasDESC);
		for (ActivitiesAndRankingOfCenter aarc : laarc)
			maarc.put(aarc.getIdCentro(), aarc);

		//Asigna el ranking por número de horas
		ranking = Long.valueOf(0L);
		floatValue = Float.MAX_VALUE;
		for (ActivitiesAndRankingOfCenter aarc : laarc) {
			if (floatValue > aarc.getHoras()) {
				floatValue = aarc.getHoras();
				
				ranking++;
			}

			aarc.setRankingHoras(ranking);
		}
		
		//Ordena por número de usuarios de mayor a menor
		Collections.sort(laarc, ActivitiesAndRankingOfCenter.comparatorUsuariosDESC);
		for (ActivitiesAndRankingOfCenter aarc : laarc)
			maarc.put(aarc.getIdCentro(), aarc);

		//Asigna el ranking por número de usuarios
		ranking = Long.valueOf(0L);
		longValue = Long.MAX_VALUE;
		for (ActivitiesAndRankingOfCenter aarc : laarc) {
			if (longValue > aarc.getUsuarios()) {
				longValue = aarc.getUsuarios();
				
				ranking++;
			}

			aarc.setRankingUsuarios(ranking);
		}
		
		//Ordena por número de plazas ofertadas de mayor a menor
		Collections.sort(laarc, ActivitiesAndRankingOfCenter.comparatorPlazasOfertadasDESC);
		for (ActivitiesAndRankingOfCenter aarc : laarc)
			maarc.put(aarc.getIdCentro(), aarc);

		//Asigna el ranking por número de plazas ofertadas
		ranking = Long.valueOf(0L);
		longValue = Long.MAX_VALUE;
		for (ActivitiesAndRankingOfCenter aarc : laarc) {
			if (longValue > aarc.getPlazasOfertadas()) {
				longValue = aarc.getPlazasOfertadas();
				
				ranking++;
			}

			aarc.setRankingPlazasOfertadas(ranking);
		}
		
		//Recompone el HashMap
		maarc = new HashMap<Long, ActivitiesAndRankingOfCenter>();
		for (ActivitiesAndRankingOfCenter aarc : laarc)
			maarc.put(aarc.getIdCentro(), aarc);*/
		
		return maarc;
	}
	
	/*private ActivitiesAndRankingOfCenter getMaxActivitiesRankings(Map<Long, ActivitiesAndRankingOfCenter> maarc) {
		ActivitiesAndRankingOfCenter aarcMax = new ActivitiesAndRankingOfCenter();
		
		List<ActivitiesAndRankingOfCenter> laarc = new ArrayList<ActivitiesAndRankingOfCenter>(maarc.values());
		
		aarcMax.setNumeroActividades(Long.MAX_VALUE);
		aarcMax.setHoras(Float.MAX_VALUE);
		aarcMax.setUsuarios(Long.MAX_VALUE);
		aarcMax.setPlazasOfertadas(Long.MAX_VALUE);
		
		for (ActivitiesAndRankingOfCenter aarc : laarc) {
			if (aarc.getNumeroActividades() < aarcMax.getNumeroActividades()) {
				aarcMax.setNumeroActividades(aarc.getNumeroActividades());
				aarcMax.setRankingNumeroActividades(aarc.getRankingNumeroActividades());
			}
			
			if (aarc.getHoras() < aarcMax.getHoras()) {
				aarcMax.setHoras(aarc.getHoras());
				aarcMax.setRankingHoras(aarc.getRankingHoras());
			}
			
			if (aarc.getUsuarios() < aarcMax.getUsuarios()) {
				aarcMax.setUsuarios(aarc.getUsuarios());
				aarcMax.setRankingUsuarios(aarc.getRankingUsuarios());
			}
			
			if (aarc.getPlazasOfertadas() < aarcMax.getPlazasOfertadas()) {
				aarcMax.setPlazasOfertadas(aarc.getPlazasOfertadas());
				aarcMax.setRankingPlazasOfertadas(aarc.getRankingPlazasOfertadas());
			}
		}
		
		return aarcMax;
	}*/
	
	private List<ActivitiesByLevel> getActivitiesByLevelList(Long idCentro, Date fechaInicio, Date fechaFin) {
		List<ActivitiesByLevel> lal = new ArrayList<ActivitiesByLevel>();

		for(Nivel n : Nivel.values()) {
			Long numeroCursos_DP;
			Long numeroCursos_FP;
			Long numeroExternasCAyS_DP;
			Long numeroExternasCAyS_FP;
			
			String strqry;

			Query qry;
			
			strqry = "SELECT COUNT(cu) AS numeroCursosDP " + 
					 "FROM " + Curso.class.getName() + " cu, " + ServicioCurso.class.getName() + " sc " +
					 "WHERE (cu.desactivado = false) AND " +
					 	   "(cu.centro.id = :idCentro) AND " +
					 	   "(cu.servicio.id = sc.id) AND " +
					 	   "(sc.nivel = :level) AND " +
					 	   "(cu.servicio.ajenoPlanFormativo = FALSE) AND " +
					 	   "((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio))";
			
			qry = service.getEntityManager().createQuery(strqry);
			
			qry.setParameter("idCentro", idCentro);
			qry.setParameter("level", n);
			qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
			qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
			
			startTime = System.nanoTime();
			numeroCursos_DP = (Long)qry.getSingleResult();
			logger.info("ReportHelperByCentro:getActivitiesByLevelList:número de cursos DP: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			
			strqry = "SELECT COUNT(cu) AS numeroCursosFP " +
					 "FROM " + Curso.class.getName() + " cu, " + ServicioCurso.class.getName() + " sc " +
					 "WHERE (cu.desactivado = false) AND " +
					 	   "(cu.centro.id = :idCentro) AND " +
					 	   "(cu.servicio.id = sc.id) AND " +
					 	   "(sc.nivel = :level) AND " +
					 	   "(cu.servicio.ajenoPlanFormativo = TRUE) AND " +
					 	   "((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio))";

			qry = service.getEntityManager().createQuery(strqry);
			
			qry.setParameter("idCentro", idCentro);
			qry.setParameter("level", n);
			qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
			qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
			
			startTime = System.nanoTime();
			numeroCursos_FP = (Long)qry.getSingleResult();
			logger.info("ReportHelperByCentro:getActivitiesByLevelList:número de cursos FP: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	
			strqry = "SELECT COUNT(ae) AS numerExternasDP " +
					 "FROM " + ActividadExterna.class.getName() + " ae, " + ServicioCurso.class.getName() + " sc " +
					 "WHERE (ae.desactivado = false) AND " +
					 	   "(ae.centro.id = :idCentro) AND " +
					 	   "(ae.servicio.id = sc.id) AND " +
					 	   "(sc.nivel = :level) AND " +
					 	   "(ae.servicio.ajenoPlanFormativo = FALSE) AND " +
					 	   "((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio))";
			
			qry = service.getEntityManager().createQuery(strqry);
			
			qry.setParameter("idCentro", idCentro);
			qry.setParameter("level", n);
			qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
			qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
			
			startTime = System.nanoTime();
			numeroExternasCAyS_DP = (Long)qry.getSingleResult();
			logger.info("ReportHelperByCentro:getActivitiesByLevelList:número de actividades externas CAyS DP: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	
			strqry = "SELECT COUNT(ae) AS numerExternasDP " +
					 "FROM " + ActividadExterna.class.getName() + " ae, " + ServicioCurso.class.getName() + " sc " +
					 "WHERE (ae.desactivado = false) AND " +
					 	   "(ae.centro.id = :idCentro) AND " +
					 	   "(ae.servicio.id = sc.id) AND " +
					 	   "(sc.nivel = :level) AND " +
					 	   "(ae.servicio.ajenoPlanFormativo = TRUE) AND " +
					 	   "((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio))";
			
			qry = service.getEntityManager().createQuery(strqry);
			
			qry.setParameter("idCentro", idCentro);
			qry.setParameter("level", n);
			qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
			qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
			
			startTime = System.nanoTime();
			numeroExternasCAyS_FP = (Long)qry.getSingleResult();
			logger.info("ReportHelperByCentro:getActivitiesByLevelList:número de actividades externas CAyS FP: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);

			ActivitiesByLevel al = new ActivitiesByLevel(
					n, // Nivel
					numeroCursos_DP,		// Número de cursos dentro del plan formativo asociados al nivel
					numeroCursos_FP,		// Número de cursosfuera del plan formativo asociados al nivel
					numeroExternasCAyS_DP,  // Número de actividades externas (sólo cesión de aula y servicio) dentro del plan formativo asociados al nivel
					numeroExternasCAyS_FP,  // Número de actividades externas (sólo cesión de aula y servicio) fuera del plan formativo asociados al nivel
					locale);

			lal.add(al);
		}
		
		return lal;
	}
	
	private RegisteredUsersByCenter getPresenceOfCenter(Long idCentro, Date fechaInicio, Date fechaFin) {
		// Crea estructura de respuesta
		RegisteredUsersByCenter ruc = new RegisteredUsersByCenter();
		Long numeroSC = 0L, numeroB = 0L, numeroNU = 0L, numeroTA = 0L;
		
		Query qry = null;
		
		//3.4.8 - RF_15_22
		// Número de asistencias de usuarios sin conocimientos registrados - CURSOS
		String strqry = "SELECT COUNT(aci) " +
						"FROM " + UsuarioCyL.class.getName() + " ucyl " +
						"INNER JOIN ucyl.asistencias aci, " +
						Curso.class.getName() + " cu " + 
						"WHERE ((aci.start <= :fechaFin) AND (aci.end >= :fechaInicio)) AND " + //NEW LINE
						"(aci.excepcion <> :excepcionAsistencia) AND " +
						"(ucyl.tipoUsuario = :tipoUsuario) AND " +
						"(aci.actividad.desactivado = false) AND " +
						"(cu.centro.id = :idCentro) AND " +
						"((aci.actividad.id = cu.id))";
		
		qry = service.getEntityManager().createQuery(strqry);

		qry.setParameter("idCentro", idCentro);
		qry.setParameter("tipoUsuario", UsuarioCyL.TipoUsuario.SC);
		qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
		qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
		qry.setParameter("excepcionAsistencia", AsistenciaCalendarItem.Excepcion.JUSTIFICADA);

		numeroSC = (Long)qry.getSingleResult();
		
		// Número de asistencias de usuarios sin conocimientos registrados - CHARLAS
		strqry = "SELECT COUNT(aci) " +
				"FROM " + UsuarioCyL.class.getName() + " ucyl " +
				"INNER JOIN ucyl.asistencias aci, " +
				Charla.class.getName() + " ch " + 
				"WHERE ((aci.start <= :fechaFin) AND (aci.end >= :fechaInicio)) AND " + //NEW LINE
				"(aci.excepcion <> :excepcionAsistencia) AND " +
				"(ucyl.tipoUsuario = :tipoUsuario) AND " +
				"(aci.actividad.desactivado = false) AND " +
				"(ch.centro.id = :idCentro) AND " +
				"((aci.actividad.id = ch.id))";
		
		qry = service.getEntityManager().createQuery(strqry);

		qry.setParameter("idCentro", idCentro);
		qry.setParameter("tipoUsuario", UsuarioCyL.TipoUsuario.SC);
		qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
		qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
		qry.setParameter("excepcionAsistencia", AsistenciaCalendarItem.Excepcion.JUSTIFICADA);

		numeroSC = numeroSC + (Long)qry.getSingleResult();
		
		ruc.setRegisteredUsersSC(numeroSC);
		
		// Número de asistencias de usuarios básicos registrados - CURSOS
		strqry = "SELECT COUNT(aci) " +
				"FROM " + UsuarioCyL.class.getName() + " ucyl " +
				"INNER JOIN ucyl.asistencias aci, " +
				Curso.class.getName() + " cu " + 
				"WHERE ((aci.start <= :fechaFin) AND (aci.end >= :fechaInicio)) AND " + //NEW LINE
				"(aci.excepcion <> :excepcionAsistencia) AND " +
				"(ucyl.tipoUsuario = :tipoUsuario) AND " +
				"(aci.actividad.desactivado = false) AND " +
				"(cu.centro.id = :idCentro) AND " +
				"((aci.actividad.id = cu.id))";
		
		qry = service.getEntityManager().createQuery(strqry);

		qry.setParameter("idCentro", idCentro);
		qry.setParameter("tipoUsuario", UsuarioCyL.TipoUsuario.B);
		qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
		qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
		qry.setParameter("excepcionAsistencia", AsistenciaCalendarItem.Excepcion.JUSTIFICADA);

		numeroB = (Long)qry.getSingleResult();
				
		// Número de asistencias de usuarios básicos registrados - CHARLAS
		strqry = "SELECT COUNT(aci) " +
				"FROM " + UsuarioCyL.class.getName() + " ucyl " +
				"INNER JOIN ucyl.asistencias aci, " +
				Charla.class.getName() + " ch " + 
				"WHERE ((aci.start <= :fechaFin) AND (aci.end >= :fechaInicio)) AND " + //NEW LINE
				"(aci.excepcion <> :excepcionAsistencia) AND " +
				"(ucyl.tipoUsuario = :tipoUsuario) AND " +
				"(aci.actividad.desactivado = false) AND " +
				"(ch.centro.id = :idCentro) AND " +
				"((aci.actividad.id = ch.id))";
				
		qry = service.getEntityManager().createQuery(strqry);

		qry.setParameter("idCentro", idCentro);
		qry.setParameter("tipoUsuario", UsuarioCyL.TipoUsuario.B);
		qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
		qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
		qry.setParameter("excepcionAsistencia", AsistenciaCalendarItem.Excepcion.JUSTIFICADA);

		numeroB = numeroB + (Long)qry.getSingleResult();
				
		ruc.setRegisteredUsersB(numeroB);
		
		// Número de asistencias de usuarios nivel usuario registrados - CURSOS
		strqry = "SELECT COUNT(aci) " +
				"FROM " + UsuarioCyL.class.getName() + " ucyl " +
				"INNER JOIN ucyl.asistencias aci, " +
				Curso.class.getName() + " cu " + 
				"WHERE ((aci.start <= :fechaFin) AND (aci.end >= :fechaInicio)) AND " + //NEW LINE
				"(aci.excepcion <> :excepcionAsistencia) AND " +
				"(ucyl.tipoUsuario = :tipoUsuario) AND " +
				"(aci.actividad.desactivado = false) AND " +
				"(cu.centro.id = :idCentro) AND " +
				"((aci.actividad.id = cu.id))";
				
		qry = service.getEntityManager().createQuery(strqry);

		qry.setParameter("idCentro", idCentro);
		qry.setParameter("tipoUsuario", UsuarioCyL.TipoUsuario.NU);
		qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
		qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
		qry.setParameter("excepcionAsistencia", AsistenciaCalendarItem.Excepcion.JUSTIFICADA);

		numeroNU = (Long)qry.getSingleResult();
						
		// Número de asistencias de usuarios nivel usuario registrados - CHARLAS
		strqry = "SELECT COUNT(aci) " +
				"FROM " + UsuarioCyL.class.getName() + " ucyl " +
				"INNER JOIN ucyl.asistencias aci, " +
				Charla.class.getName() + " ch " + 
				"WHERE ((aci.start <= :fechaFin) AND (aci.end >= :fechaInicio)) AND " + //NEW LINE
				"(aci.excepcion <> :excepcionAsistencia) AND " +
				"(ucyl.tipoUsuario = :tipoUsuario) AND " +
				"(aci.actividad.desactivado = false) AND " +
				"(ch.centro.id = :idCentro) AND " +
				"((aci.actividad.id = ch.id))";
						
		qry = service.getEntityManager().createQuery(strqry);

		qry.setParameter("idCentro", idCentro);
		qry.setParameter("tipoUsuario", UsuarioCyL.TipoUsuario.NU);
		qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
		qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
		qry.setParameter("excepcionAsistencia", AsistenciaCalendarItem.Excepcion.JUSTIFICADA);

		numeroNU = numeroNU + (Long)qry.getSingleResult();
						
		ruc.setRegisteredUsersNU(numeroNU);
		
		// Número de asistencias de usuarios técnicos avanzados registrados - CURSOS
		strqry = "SELECT COUNT(aci) " +
				"FROM " + UsuarioCyL.class.getName() + " ucyl " +
				"INNER JOIN ucyl.asistencias aci, " +
				Curso.class.getName() + " cu " + 
				"WHERE ((aci.start <= :fechaFin) AND (aci.end >= :fechaInicio)) AND " + //NEW LINE
				"(aci.excepcion <> :excepcionAsistencia) AND " +
				"(ucyl.tipoUsuario = :tipoUsuario) AND " +
				"(aci.actividad.desactivado = false) AND " +
				"(cu.centro.id = :idCentro) AND " +
				"((aci.actividad.id = cu.id))";
						
		qry = service.getEntityManager().createQuery(strqry);

		qry.setParameter("idCentro", idCentro);
		qry.setParameter("tipoUsuario", UsuarioCyL.TipoUsuario.TA);
		qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
		qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
		qry.setParameter("excepcionAsistencia", AsistenciaCalendarItem.Excepcion.JUSTIFICADA);

		numeroTA = (Long)qry.getSingleResult();
								
		// Número de asistencias de usuarios técnicos avanzados registrados - CHARLAS
		strqry = "SELECT COUNT(aci) " +
				"FROM " + UsuarioCyL.class.getName() + " ucyl " +
				"INNER JOIN ucyl.asistencias aci, " +
				Charla.class.getName() + " ch " + 
				"WHERE ((aci.start <= :fechaFin) AND (aci.end >= :fechaInicio)) AND " + //NEW LINE
				"(aci.excepcion <> :excepcionAsistencia) AND " +
				"(ucyl.tipoUsuario = :tipoUsuario) AND " +
				"(aci.actividad.desactivado = false) AND " +
				"(ch.centro.id = :idCentro) AND " +
				"((aci.actividad.id = ch.id))";
								
		qry = service.getEntityManager().createQuery(strqry);

		qry.setParameter("idCentro", idCentro);
		qry.setParameter("tipoUsuario", UsuarioCyL.TipoUsuario.TA);
		qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
		qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
		qry.setParameter("excepcionAsistencia", AsistenciaCalendarItem.Excepcion.JUSTIFICADA);

		numeroTA = numeroTA + (Long)qry.getSingleResult();
								
		ruc.setRegisteredUsersTA(numeroTA);
		
		return ruc;
	}
	
	private Long getPresenceOfferedInCenter(Long idCentro, Date fechaInicio, Date fechaFin) {
		Long presenceOfferedInCenter = 0L;
		
		String strqry = "SELECT COALESCE(SUM(cu.numeroPlazas * cu.calendarItems.size), 0L) " +
						"FROM " + Curso.class.getName() + " cu " +
						"WHERE (cu.desactivado = false) AND " +
							"(cu.centro.id = :idCentro) AND " +
							"((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio))";
		
		startTime = System.nanoTime();
		Long partialCursos = (Long)getQuerySingleResult(strqry, idCentro, fechaInicio, fechaFin);
		logger.info("ReportHelperByCentro:getPresenceOfferedInCenter:partialCursos: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		
		strqry = "SELECT COALESCE(SUM(ch.numeroPlazas * ch.calendarItems.size), 0L) " +
				 "FROM " + Charla.class.getName() + " ch " +
		 		 "WHERE (ch.desactivado = false) AND " +
		 		 	   "(ch.centro.id = :idCentro) AND " +
		 		 	   "((ch.fechaInicio <= :fechaFin) AND (ch.fechaFin >= :fechaInicio))";

		startTime = System.nanoTime();
		Long partialCharlas = (Long)getQuerySingleResult(strqry, idCentro, fechaInicio, fechaFin);
		logger.info("ReportHelperByCentro:getPresenceOfferedInCenter:partialCharlas: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		
		if (partialCursos != null)
			presenceOfferedInCenter += partialCursos;
		
		if (partialCharlas != null)
			presenceOfferedInCenter += partialCharlas;
				
		return presenceOfferedInCenter;
	}
	
	private RegisteredUsersByCenter getRegisteredUsersOfCenter(Long idCentro, Date fechaInicio, Date fechaFin) {
		String strqry = "SELECT (SELECT COUNT(ucyl) " +
								"FROM " + UsuarioCyL.class.getName() + " ucyl " +
								"WHERE (ucyl.tipoUsuario = 'SC') AND " +
									  "(ucyl.centro.id = c.id) AND " +
									  "(ucyl.desactivado = FALSE) AND " +
									  "((ucyl.fechaCreacion <= :fechaFin) AND (ucyl.fechaCreacion >= :fechaInicio))) AS numeroUsuariosSC, " + 
									  //"(ucyl.fechaCreacion BETWEEN :fechaInicio AND :fechaFin)) AS numeroUsuariosSC, " +
							   "(SELECT COUNT(ucyl) " +
								"FROM " + UsuarioCyL.class.getName() + " ucyl " +
								"WHERE (ucyl.tipoUsuario = 'B') AND " +
									  "(ucyl.centro.id = c.id) AND " +
									  "(ucyl.desactivado = FALSE) AND " +
									  "((ucyl.fechaCreacion <= :fechaFin) AND (ucyl.fechaCreacion >= :fechaInicio))) AS numeroUsuariosB, " +
									  //"(ucyl.fechaCreacion BETWEEN :fechaInicio AND :fechaFin)) AS numeroUsuariosB, " +
							   "(SELECT COUNT(ucyl) " +
								"FROM " + UsuarioCyL.class.getName() + " ucyl " +
								"WHERE (ucyl.tipoUsuario = 'NU') AND " +
									  "(ucyl.centro.id = c.id) AND " +
									  "(ucyl.desactivado = FALSE) AND " +
									  "((ucyl.fechaCreacion <= :fechaFin) AND (ucyl.fechaCreacion >= :fechaInicio))) AS numeroUsuariosNU, " +
									  //"(ucyl.fechaCreacion BETWEEN :fechaInicio AND :fechaFin)) AS numeroUsuariosNU, " +
							   "(SELECT COUNT(ucyl) " +
								"FROM " + UsuarioCyL.class.getName() + " ucyl " +
								"WHERE (ucyl.tipoUsuario = 'TA') AND " +
									  "(ucyl.centro.id = c.id) AND " +
									  "(ucyl.desactivado = FALSE) AND " +
									  "((ucyl.fechaCreacion <= :fechaFin) AND (ucyl.fechaCreacion >= :fechaInicio))) AS numeroUsuariosTA " +
									  //"(ucyl.fechaCreacion BETWEEN :fechaInicio AND :fechaFin)) AS numeroUsuariosTA " +
						"FROM " + Centro.class.getName() + " c " +
						"WHERE (c.id = :idCentro)";

		Query qry = service.getEntityManager().createQuery(strqry);

		qry.setParameter("idCentro", idCentro);
		qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
		qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
		
		@SuppressWarnings("unchecked")
		List<Object[]> datatmp = (List<Object[]>)qry.getResultList();

		RegisteredUsersByCenter ruc = new RegisteredUsersByCenter();
			
		if ((datatmp != null) && (datatmp.size() == 1)) {		
			ruc.setRegisteredUsersSC((Long)((datatmp.get(0))[0]));	// Número de usuarios sin conocimientos registrados
			ruc.setRegisteredUsersB((Long)((datatmp.get(0))[1]));	// Número de usuarios básicos registrados 
			ruc.setRegisteredUsersNU((Long)((datatmp.get(0))[2]));	// Número de usuarios nivel usuario registrados
			ruc.setRegisteredUsersTA((Long)((datatmp.get(0))[3]));	// Número de usuarios técnicos avanzados registrados
		}

		return ruc;
	}
	
	private List<UsersExamPassedByServices> getUsersExamPassedByServicesList(Long idCentro, Date fechaInicio, Date fechaFin) {
		String strqry = "SELECT sc.id AS idServicio, " +
							   "scl.nombre AS nombreServicio " +
						"FROM " + Curso.class.getName() + " cu INNER JOIN " +
							 "cu.servicio sc INNER JOIN " +
							 "sc.textos scl " +
						"WHERE (sc.examen IS NOT NULL) AND " +
							  "(scl.language.ISO = :languageISO) AND " +
							  "(cu.centro.id = :idCentro) AND " +
							  "((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio)) " +
						"GROUP BY sc " +
						"ORDER BY scl.nombre ASC, " +
								 "sc.id ASC";
		
		Query qry = service.getEntityManager().createQuery(strqry);

		qry.setParameter("idCentro", idCentro);
		qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
		qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
		qry.setParameter("languageISO", locale.toString());
		
		@SuppressWarnings("unchecked")
		List<Object[]> datatmp = (List<Object[]>)qry.getResultList();

		List<UsersExamPassedByServices> lueps = new ArrayList<UsersExamPassedByServices>();
		for (Object[] lobj : datatmp) {
			Long idServicio = (Long)lobj[0];
			String nombreServicio = (String)lobj[1];

			strqry = "SELECT DISTINCT cu.id, acu.usuario.id " +
					 "FROM " + Curso.class.getName() + " cu INNER JOIN " +
					 	  "cu.servicio sc INNER JOIN " +
						  "cu.asistencias acu " +
					 "WHERE (sc.examen IS NOT NULL) AND " +
					 	   "(cu.centro.id = :idCentro) AND " +
						   "(sc.id = :idServicio) AND " +
						   "((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio)) " +
					 "GROUP BY cu.id, acu.usuario " +
					 "HAVING ((COUNT(acu) * 100.00) / cu.calendarItems.size) >= 80.00";
			
			qry = service.getEntityManager().createQuery(strqry);

			qry.setParameter("idCentro", idCentro);
			qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
			qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
			qry.setParameter("idServicio", idServicio);
			
			Long numUsuariosCanMakeExam = Long.valueOf((long)qry.getResultList().size());

			strqry = "SELECT COUNT(DISTINCT mcu) " +
					 "FROM " + MatriculaCurso.class.getName() + " mcu INNER JOIN " +
						  "mcu.curso cu INNER JOIN " +
						  "cu.servicio sc " +
					 "WHERE (sc.examen IS NOT NULL) AND " +
						   "(mcu.examenFinal IS NOT NULL) AND " +
						   "(mcu.nota >= cu.examen.numPreguntasAprobado) AND " +
						   "(cu.centro.id = :idCentro) AND " +
						   "(sc.id = :idServicio) AND " +
						   "((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio))";
			
			qry = service.getEntityManager().createQuery(strqry);

			qry.setParameter("idCentro", idCentro);
			qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
			qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
			qry.setParameter("idServicio", idServicio);
			
			Long numAprobados = (Long)qry.getSingleResult();

			lueps.add(new UsersExamPassedByServices(idServicio, nombreServicio, numUsuariosCanMakeExam, numAprobados));
		}
		
		return lueps;
	}
	
	private List<UsersExamPassedByCentersAndTrainers> getUsersExamPassedByTrainersList(Long idCentro, Date fechaInicio, Date fechaFin) {
		String strqry = "SELECT p.id AS idProfesor, " +
							  "CONCAT(p.apellido1, ' ', p.apellido2, ', ', p.nombre) AS nombreProfesor " +	   
						"FROM " + Curso.class.getName() + " cu INNER JOIN " +
							 "cu.servicio sc INNER JOIN " +
							 "cu.profesor p " +
						"WHERE (sc.examen IS NOT NULL) AND " +
							  "(cu.centro.id = :idCentro) AND " +
							  "((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio)) " +
						"GROUP BY p " +
						"ORDER BY p.apellido1 ASC, " +
								 "p.apellido2 ASC, " +
								 "p.nombre ASC, " +
								 "p.id ASC";
		
		Query qry = service.getEntityManager().createQuery(strqry);

		qry.setParameter("idCentro", idCentro);
		qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
		qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
		
		@SuppressWarnings("unchecked")
		List<Object[]> datatmp = (List<Object[]>)qry.getResultList();

		List<UsersExamPassedByCentersAndTrainers> luept = new ArrayList<UsersExamPassedByCentersAndTrainers>();
		for (Object[] lobj : datatmp) {
			Long idProfesor = (Long)lobj[0];
			String nombreProfesor = (String)lobj[1];
			
			strqry = "SELECT DISTINCT cu.id, acu.usuario.id " +
					 "FROM " + Curso.class.getName() + " cu INNER JOIN " +
						  "cu.asistencias acu INNER JOIN " +
						  "cu.servicio sc " +
					 "WHERE (sc.examen IS NOT NULL) AND " +
					 	   "(cu.centro.id = :idCentro) AND " +
						   "(cu.profesor.id = :idProfesor) AND " +
						   "((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio)) " +
					 "GROUP BY cu.id, acu.usuario " +
					 "HAVING ((COUNT(acu) * 100.00) / cu.calendarItems.size) >= 80.00";
			
			qry = service.getEntityManager().createQuery(strqry);

			qry.setParameter("idCentro", idCentro);
			qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
			qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
			qry.setParameter("idCentro", idCentro);
			qry.setParameter("idProfesor", idProfesor);
			
			Long numUsuariosCanMakeExam = Long.valueOf((long)qry.getResultList().size());
			
			strqry = "SELECT COUNT(DISTINCT mcu) " +
					 "FROM " + MatriculaCurso.class.getName() + " mcu INNER JOIN " +
						  "mcu.curso cu INNER JOIN " +
						  "cu.servicio sc " +
					 "WHERE (sc.examen  IS NOT NULL) AND " +
						   "(mcu.examenFinal IS NOT NULL) AND " +
						   "(mcu.nota >= cu.examen.numPreguntasAprobado) AND " +
						   "(cu.centro.id = :idCentro) AND " +
						   "(cu.profesor.id = :idProfesor) AND " +
						   "((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio))";
			
			qry = service.getEntityManager().createQuery(strqry);

			qry.setParameter("idCentro", idCentro);
			qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
			qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
			qry.setParameter("idCentro", idCentro);
			qry.setParameter("idProfesor", idProfesor);

			Long numAprobados = (Long)qry.getSingleResult();

			luept.add(new UsersExamPassedByCentersAndTrainers(Long.valueOf(-1L), "",
															  idProfesor, nombreProfesor,
															  numUsuariosCanMakeExam, numAprobados));
		}

		return luept;
	}
	
	private List<AverageRatingOfQuestion> getAverageRatingOfQuestionList(Long idCentro, Date fechaInicio, Date fechaFin) {
		String strqry = null;
		Query qry = null;
		
		// Extrae lista de preguntas de encuestas de satisfacción respondidas en cursos del centro históricamente
		strqry = "SELECT (tpcu.etiqueta || ' ' || tpcu.nombre) AS textoPregunta, " +
				        "pcu AS pregunta " +
				 "FROM " + MatriculaCurso.class.getName() + " mcu INNER JOIN " +
				      "mcu.curso cu INNER JOIN " +
				      "mcu.formularioConcreto.respuestas rcu INNER JOIN " +
				      "rcu.pregunta pcu WITH ((pcu.obligatorio = TRUE) AND " +
				                             "((pcu.tipo = 'ENTERO') OR " +
				                              "(pcu.tipo = 'DECIMAL'))) INNER JOIN " +
				      "pcu.textos tpcu INNER JOIN " +
				      "tpcu.language ltpcu WITH (ltpcu.ISO = :languageISO) " +
				 "WHERE (cu.desactivado = false) AND " +
				 	   "(cu.centro.id = :idCentro) " +
				 "GROUP BY pcu " +
				 "ORDER BY tpcu.etiqueta ASC, " +
				          "tpcu.nombre ASC";
		
		qry = service.getEntityManager().createQuery(strqry);
		
		qry.setParameter("languageISO", locale.toString());
		qry.setParameter("idCentro", idCentro);
		
		startTime = System.nanoTime();
		@SuppressWarnings("unchecked")
		List<Object[]> preguntasCentro = (List<Object[]>)qry.getResultList();
		logger.info("ReportHelperByCentro:getAverageRatingOfQuestionList:lista de preguntas de encuestas de satisfacción respondidas en cursos del centro históricamente: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		
		// Crea la estructura de respuesta
		List<AverageRatingOfQuestion> larq = new ArrayList<AverageRatingOfQuestion>();
		
		// Se realizan las consultas por cada pregunta y se rellena la estructura de respuesta.
		for (Object[] pregunta : preguntasCentro) {
			Pregunta p = (Pregunta)pregunta[1];
			
			// Extrae el valor medio en el periodo seleccionado de las respuestas de la pregunta selecionada en el centro
			strqry = "SELECT COALESCE((CASE pcu_sp.tipo " +
					                       "WHEN 'ENTERO' THEN (ROUND(AVG(rcu_sp.valorEntero), 2) * 1.0F) " +
					                       "WHEN 'DECIMAL' THEN (ROUND(AVG(rcu_sp.valorDecimal), 2) * 1.0F) " +
					                  "END), -1.0F) AS valorMedioPeriodo " +
					 "FROM " + MatriculaCurso.class.getName() + " mcu_sp INNER JOIN " +
						  "mcu_sp.curso cu_sp WITH (((cu_sp.fechaInicio <= :fechaFin) AND (cu_sp.fechaFin >= :fechaInicio))) INNER JOIN " +
					      "mcu_sp.formularioConcreto.respuestas rcu_sp INNER JOIN " +
					      "rcu_sp.pregunta pcu_sp WITH (pcu_sp.id = :idPregunta) " +
					 "WHERE (cu_sp.desactivado = false) AND " +
					 	   "(cu_sp.centro.id = :idCentro)";
			
			qry = service.getEntityManager().createQuery(strqry);
			
			qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
			qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
			qry.setParameter("idPregunta", p.getId());
			qry.setParameter("idCentro", idCentro);
			
			startTime = System.nanoTime();
			Float valorMedioPeriodo = (Float)qry.getSingleResult();
			logger.info("ReportHelperByCentro:getAverageRatingOfQuestionList:valor medio en el periodo seleccionado de las respuestas de la pregunta selecionada en el centro: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			
			// Extrae el valor medio histórico de las respuestas de la pregunta selecionada en el centro
			strqry = "SELECT COALESCE((CASE pcu_sh.tipo " +
					                       "WHEN 'ENTERO' THEN (ROUND(AVG(rcu_sh.valorEntero), 2) * 1.0F) " +
					                       "WHEN 'DECIMAL' THEN (ROUND(AVG(rcu_sh.valorDecimal), 2) * 1.0F) " +
					                  "END), -1.0F) AS valorMedioHistorico " +
					 "FROM " + MatriculaCurso.class.getName() + " mcu_sh INNER JOIN " +
					      "mcu_sh.curso cu_sh INNER JOIN " +
					      "cu_sh.matriculas mcu_sh INNER JOIN " +
					      "mcu_sh.formularioConcreto.respuestas rcu_sh INNER JOIN " +
					      "rcu_sh.pregunta pcu_sh WITH (pcu_sh.id = :idPregunta) " +
					 "WHERE (cu_sh.desactivado = false) AND " +
					 	 //"(cu_sh.servicio.desactivado = false) AND " +
					 	   "(cu_sh.centro.id = :idCentro)";
			
			qry = service.getEntityManager().createQuery(strqry);
			
			qry.setParameter("idPregunta", p.getId());
			qry.setParameter("idCentro", idCentro);
			
			startTime = System.nanoTime();
			Float valorMedioHistorico = (Float)qry.getSingleResult();
			logger.info("ReportHelperByCentro:getAverageRatingOfQuestionList:valor medio histórico de las respuestas de la pregunta selecionada en el centro: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			
			// Se almacena en la estructura de respuesta
			AverageRatingOfQuestion arq = new AverageRatingOfQuestion(
					(String)(pregunta[0]),																	  // Pregunta
					((p.getTipo() == TipoPregunta.ENTERO) ? new Float(p.getMinimo()) : p.getMinimoDecimal()), // Valor mínimo posible
					((p.getTipo() == TipoPregunta.ENTERO) ? new Float(p.getMaximo()) : p.getMaximoDecimal()), // Valor máximo posible
					valorMedioPeriodo,																		  // Valor medio del periodo
					valorMedioHistorico);																	  // Valor medio global

			larq.add(arq);
		}
		
		return larq;
	}

	private Object getQuerySingleResult(String strqry, Long idCentro, Date fechaInicio, Date fechaFin) {
		Query qry = service.getEntityManager().createQuery(strqry);

		qry.setParameter("idCentro", idCentro);
		qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
		qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);

		return qry.getSingleResult();
	}
	
	private Object getQuerySingleResult(String strqry, Long idCentro, Date fechaInicio, Date fechaFin, AsistenciaCalendarItem.Excepcion excepcion) {
		Query qry = service.getEntityManager().createQuery(strqry);

		qry.setParameter("idCentro", idCentro);
		qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
		qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
		qry.setParameter("excepcionAsistencia", excepcion);

		return qry.getSingleResult();
	}
	
	private Long getSizeOfQueryResultList(String strqry, Long idCentro, Date fechaInicio, Date fechaFin, AsistenciaCalendarItem.Excepcion excepcion) {
		Query qry = service.getEntityManager().createQuery(strqry);

		qry.setParameter("idCentro", idCentro);
		qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
		qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
		qry.setParameter("excepcionAsistencia", excepcion);

		return new Long((long)qry.getResultList().size());
	}
}
