package es.csa.fdt.cyldigital.reports;

/*
 * #%L
 * Librerías del núcleo para Web - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;

import javax.persistence.Query;
import javax.persistence.TemporalType;

import net.sf.jasperreports.engine.JRParameter;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import es.csa.fdt.cyldigital.calendario.entity.CalendarItem;
import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.controller.CalendarControllerImpl;
import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.evaluacion.entity.Pregunta;
import es.csa.fdt.cyldigital.evaluacion.entity.Pregunta.TipoPregunta;
import es.csa.fdt.cyldigital.indicadores.ReportHelper;
import es.csa.fdt.cyldigital.language.entity.TematicaLanguage;
import es.csa.fdt.cyldigital.servicios.entity.ActividadExterna;
import es.csa.fdt.cyldigital.servicios.entity.Charla;
import es.csa.fdt.cyldigital.servicios.entity.Curso;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso;
import es.csa.fdt.cyldigital.servicios.entity.Servicio.Nivel;
import es.csa.fdt.cyldigital.servicios.entity.ServicioCurso;
import es.csa.fdt.cyldigital.usuarios.entity.Proveedor;
import es.csa.fdt.cyldigital.usuarios.entity.Provincia;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;

import es.fdt.isis.reports.entity.Activities;
import es.fdt.isis.reports.entity.ActivitiesByCenter;
import es.fdt.isis.reports.entity.ActivitiesByLevel;
import es.fdt.isis.reports.entity.ActivitiesByThematic;
import es.fdt.isis.reports.entity.AverageRatingOfQuestion;
import es.fdt.isis.reports.entity.AverageRatingOfQuestionByProvince;
import es.fdt.isis.reports.entity.AverageRatingOfQuestionsByCenter;
import es.fdt.isis.reports.entity.AverageRatingOfQuestionsByCollaborativeEntity;
import es.fdt.isis.reports.entity.OcupacionOfActivitiesByType;
import es.fdt.isis.reports.entity.RegisteredUsersByCenter;
import es.fdt.isis.reports.entity.UsersExamPassedByCentersAndTrainers;
import es.fdt.isis.reports.entity.UsersExamPassedByServices;

/**
 * Clase ReportHelperGlobal.
 */
@Component("reportHelperGlobalGlobal")
public class ReportHelperGlobal extends ReportHelperBase implements ReportHelper  {
	private Logger logger = LoggerFactory.getLogger(ReportHelperGlobal.class);
	private Long startTime;
	
	private Long idCentroAdmin;
	private Long idCentroColaboradores;
	
	private Locale locale;
	
	/* (non-Javadoc)	
	 * @see es.fdt.isis.reports.ReportHelper#getReportModel(java.lang.String, java.util.Map)
	 */
	@Override
	@Transactional(readOnly = true)
	public Map<String, Object> getReportModel(String string, Map<String, Object> params) {
		Map<String, Object> map = new HashMap<String, Object>(20);
	    
		Date fechaInicio = (Date)params.get("fechaInicio");
		Date fechaFin = (Date)params.get("fechaFin");
		
	    locale = (Locale) params.get("locale");
	    
	    //Extracción de los índices del centro de Administradores y del centro de Colaboradores
	    idCentroAdmin = Long.parseLong(((Parameter)service.findOneWhere(Parameter.class, "nombre", "admin_default_center_id")).getValor());
		idCentroColaboradores = Long.parseLong(((Parameter)service.findOneWhere(Parameter.class, "nombre", "extern_default_center_id")).getValor());
		
	    //Cálculo de datos
	    List<Object> GlobalReportData = new ArrayList<Object>();
	    
	    Activities a;
	    List<RegisteredUsersByCenter> lruc; 
	    
	    startTime = System.nanoTime();
	    //GlobalReportData.add(0, null);
	    GlobalReportData.add(0, (a = getActivities(fechaInicio, fechaFin)));
	    logger.info("ReportHelperGlobal:getActivities: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	    
	    startTime = System.nanoTime();
	    //GlobalReportData.add(1, null);
	    GlobalReportData.add(1, getOcupacionOfActivitiesByTypeList(a));
	    logger.info("ReportHelperGlobal:getOcupacionOfActivitiesByTypeList: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	    
	    startTime = System.nanoTime();
	    //GlobalReportData.add(2, null);
	    GlobalReportData.add(2, getMaxValueInOcupacionOfActivitiesByTypeList(a));
	    logger.info("ReportHelperGlobal:getMaxValueInOcupacionOfActivitiesByTypeList: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	    
	    startTime = System.nanoTime();
	    //GlobalReportData.add(3, null);
	    GlobalReportData.add(3, getActivitiesByCenterList(a, fechaInicio, fechaFin));
	    logger.info("ReportHelperGlobal:getActivitiesByCenterList: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	    
	    startTime = System.nanoTime();
	    //GlobalReportData.add(4, null);
	    GlobalReportData.add(4, getActivitiesByThematicList(fechaInicio, fechaFin));
	    logger.info("ReportHelperGlobal:getActivitiesByThematicList: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	    
	    startTime = System.nanoTime();
	    //GlobalReportData.add(5, null);
	    GlobalReportData.add(5, getActivitiesByLevelList(fechaInicio, fechaFin));
	    logger.info("ReportHelperGlobal:getActivitiesByLevelList: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	    
	    startTime = System.nanoTime();
	    //GlobalReportData.add(6, null);
	    GlobalReportData.add(6, (lruc = getRegisteredUsersByCenterList(fechaInicio, fechaFin)));
	    logger.info("ReportHelperGlobal:getRegisteredUsersByCenterList: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	    
	    startTime = System.nanoTime();
	    //GlobalReportData.add(7, null);
	    GlobalReportData.add(7, getTotalRegisteredUsers(lruc));
	    logger.info("ReportHelperGlobal:getTotalRegisteredUsers: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	    
	    startTime = System.nanoTime();
	    //GlobalReportData.add(8, null);
	    GlobalReportData.add(8, getUsersExamPassedByServicesList(fechaInicio, fechaFin));
	    logger.info("ReportHelperGlobal:getUsersExamPassedByServicesList: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	    
	    startTime = System.nanoTime();
	    //GlobalReportData.add(9, null);
	    GlobalReportData.add(9, getUsersExamPassedByCentersAndTrainersList(fechaInicio, fechaFin));
	    logger.info("ReportHelperGlobal:getUsersExamPassedByCentersAndTrainersList: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	    
	    startTime = System.nanoTime();
	    //GlobalReportData.add(10, null);
	    GlobalReportData.add(10, getAverageRatingOfQuestionByProvinceList(fechaInicio, fechaFin));
	    logger.info("ReportHelperGlobal:getAverageRatingOfQuestionByProvinceList: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	    
	    startTime = System.nanoTime();
	    //GlobalReportData.add(11, null);
	    GlobalReportData.add(11, getAverageRatingOfQuestionList(fechaInicio, fechaFin));
	    logger.info("ReportHelperGlobal:getAverageRatingOfQuestionList: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	    
	    startTime = System.nanoTime();
	    //GlobalReportData.add(12, null);
	    GlobalReportData.add(12, getAverageRatingOfQuestionsByCenterList(fechaInicio, fechaFin));
	    logger.info("ReportHelperGlobal:getAverageRatingOfQuestionsByCenterList: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	    
	    startTime = System.nanoTime();
	    //GlobalReportData.add(13, null);
	    GlobalReportData.add(13, getAverageRatingOfQuestionsByCollaborativeEntityList(fechaInicio, fechaFin));
	    logger.info("ReportHelperGlobal:getAverageRatingOfQuestionsByCollaborativeEntityList: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);

	    //Pone los parametros
	    map.put(JRParameter.REPORT_RESOURCE_BUNDLE, ResourceBundle.getBundle("es.csa.fdt.cyldigital.resources.informes", locale));
	    map.put(JRParameter.REPORT_LOCALE, locale);
	    map.put("GlobalReportData", GlobalReportData);

	    return map;
	}

	private Activities getActivities(Date fechaInicio, Date fechaFin) {
	    String strqry;
	    //3.4.12 - RF_15_22
	  	Long nCursos;
	  	Long nCharlas;
	  	Long nExternasCA;
	  	Long nExternasS;
	  	Long nExternasCAyS;

		Activities a = new Activities();
		
		/////////////////////////////// Actividades ///////////////////////////////
		// CURSOS //
		// Número de cursos
		strqry = "SELECT COUNT(cu) " +
				 "FROM " + Curso.class.getName() + " cu " +
				 "WHERE (cu.desactivado = false) AND " +
				 "((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio))";
		
		startTime = System.nanoTime();
		nCursos = (Long)getQuerySingleResult(strqry, fechaInicio, fechaFin);
		a.setNumeroCursos(nCursos);
		logger.info("ReportHelperGlobal:getActivities:Número de cursos: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		
		if(nCursos > 0) {
			/////////////////////// Número de horas de cursos /////////////////////
			strqry = "SELECT COALESCE(SUM(ROUND((HOUR(cuci.end) - HOUR(cuci.start)) + ((MINUTE(cuci.end) - MINUTE(cuci.start)) / 60.00), 2)), 0.00) + 0.00 " + //new line
					 "FROM " + Curso.class.getName() + " cu INNER JOIN " + //INNER JOIN is new
					 	"cu.calendarItems cuci " + //new line
					 "WHERE (cu.desactivado = false) AND " +
					 	"((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio)) AND " + 
						"((cuci.start <= :fechaFin) AND (cuci.end >= :fechaInicio))"; //new line
			
			startTime = System.nanoTime();
			a.setNumeroHorasCursos(((Double)getQuerySingleResult(strqry, fechaInicio, fechaFin)).floatValue());
			logger.info("ReportHelperGlobal:getActivities:Número de horas de cursos: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			
			////////////////////// Número horas terminadas de cursos (Horas asistidas) /////////////////
			strqry = "SELECT COALESCE(SUM(ROUND((HOUR(acu.end) - HOUR(acu.start)) + ((MINUTE(acu.end) - MINUTE(acu.start)) / 60.00), 2)), 0.00) + 0.00 " +
					 "FROM " + Curso.class.getName() + " cu INNER JOIN " +
					 	"cu.asistencias acu " +
					 "WHERE (cu.desactivado = false) AND " +
					 	"((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio)) AND " + 
						"((acu.start <= :fechaFin) AND (acu.end >= :fechaInicio))"; //new line";
			
			startTime = System.nanoTime();
			a.setNumeroHorasTerminadasCursos(((Double)getQuerySingleResult(strqry, fechaInicio, fechaFin)).floatValue());
			logger.info("ReportHelperGlobal:getActivities:Número horas terminadas de cursos: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			
			///////////////////// Número de plazas ofertadas de cursos ///////////////////////
			strqry = "SELECT COALESCE(SUM(cu.numeroPlazas), 0) " +
					 "FROM " + Curso.class.getName() + " cu " +
					 "WHERE (cu.desactivado = false) AND " +
					 		"((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio))";
			
			startTime = System.nanoTime();
			a.setNumeroPlazasOfertadasCursos((Long)getQuerySingleResult(strqry, fechaInicio, fechaFin));
			logger.info("ReportHelperGlobal:getActivities:Número de plazas ofertadas de cursos: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			
			/////////////////////////// Número de plazas cubiertas en cursos (usuarios matriculados) ////////////////////////
			strqry = "SELECT COUNT(mcu) " +
					 "FROM " + Curso.class.getName() + " cu INNER JOIN " +
					 	"cu.matriculas mcu " +
					 "WHERE (cu.desactivado = false) AND " +
					 	"((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio)) AND " +
					 	"(mcu.admitido = TRUE) AND " +
					 	"(mcu.enListaEspera = FALSE)";
			
			startTime = System.nanoTime();
			a.setNumeroPlazasCubiertasCursos((Long)getQuerySingleResult(strqry, fechaInicio, fechaFin));
			logger.info("ReportHelperGlobal:getActivities:Número de plazas cubiertas en cursos: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			
			/////////////////////////////// Número de plazas terminadas en cursos (usuarios matriculados con asistencia >= 80%) //////////////////////////////
			strqry = "SELECT DISTINCT cu, acu.usuario.id " +
					 "FROM " + Curso.class.getName() + " cu INNER JOIN " +
					 	"cu.asistencias acu " +
					 "WHERE (cu.desactivado = false) AND " +
					 	"((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio)) " +
					 "GROUP BY cu, acu.usuario " +
					 "HAVING ((COUNT(acu) * 100.00) / cu.calendarItems.size) >= 80.00";
			
			startTime = System.nanoTime();
			a.setNumeroPlazasTerminadasCursos(getSizeOfQueryResultList(strqry, fechaInicio, fechaFin));
			logger.info("ReportHelperGlobal:getActivities:Número de plazas terminadas en cursos: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		} else {
			a.setNumeroHorasCursos(0F);
			a.setNumeroHorasTerminadasCursos(0F);
			a.setNumeroPlazasOfertadasCursos(0L);
			a.setNumeroPlazasCubiertasCursos(0L);
			a.setNumeroPlazasTerminadasCursos(0L);
		}
		
		// CHARLAS //
		// Número de charlas
		strqry = "SELECT COUNT(ch) " +
				 "FROM " + Charla.class.getName() + " ch " +
				 "WHERE (ch.desactivado = false) AND " +
				 		"((ch.fechaInicio <= :fechaFin) AND (ch.fechaFin >= :fechaInicio))";
				
		startTime = System.nanoTime();
		nCharlas = (Long)getQuerySingleResult(strqry, fechaInicio, fechaFin);
		a.setNumeroCharlas(nCharlas);
		logger.info("ReportHelperGlobal:getActivities:Número de charlas: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		
		if(nCharlas > 0) {
			/////////////////////////////////////// Número de horas de charlas ////////////////////////////////////
			strqry = "SELECT COALESCE(SUM(ROUND((HOUR(chci.end) - HOUR(chci.start)) + ((MINUTE(chci.end) - MINUTE(chci.start)) / 60.00), 2)), 0.00) + 0.00 " +
					 "FROM " + Charla.class.getName() + " ch INNER JOIN " +
					 	"ch.calendarItems chci " +
					 "WHERE (ch.desactivado = false) AND " +
					 	"((ch.fechaInicio <= :fechaFin) AND (ch.fechaFin >= :fechaInicio)) AND " + 
					 	"((chci.start <= :fechaFin) AND (chci.end >= :fechaInicio))"; //new line
			
			startTime = System.nanoTime();
			a.setNumeroHorasCharlas(((Double)getQuerySingleResult(strqry, fechaInicio, fechaFin)).floatValue());
			logger.info("ReportHelperGlobal:getActivities:Número de horas de charlas: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			
			///////////////////////////////////// Número horas terminadas de charlas (Horas asistidas) /////////////////////////////
			strqry = "SELECT COALESCE(SUM(ROUND((HOUR(ach.end) - HOUR(ach.start)) + ((MINUTE(ach.end) - MINUTE(ach.start)) / 60.00), 2)), 0.00) + 0.00 " +
					 "FROM " + Charla.class.getName() + " ch INNER JOIN " +
						"ch.asistencias ach " +
					 "WHERE (ch.desactivado = false) AND " +
					 	"((ch.fechaInicio <= :fechaFin) AND (ch.fechaFin >= :fechaInicio)) AND " + 
					 	"((ach.start <= :fechaFin) AND (ach.end >= :fechaInicio))"; //new line";
			
			startTime = System.nanoTime();
			a.setNumeroHorasTerminadasCharlas(((Double)getQuerySingleResult(strqry, fechaInicio, fechaFin)).floatValue());
			logger.info("ReportHelperGlobal:getActivities:Número horas terminadas de charlas: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			
			///////////////////////////////////// Número de plazas ofertadas de charlas /////////////////////////////////////
			strqry = "SELECT COALESCE(SUM(ch.numeroPlazas), 0) " +
					 "FROM " + Charla.class.getName() + " ch " +
					 "WHERE (ch.desactivado = false) AND " +
					 	  "((ch.fechaInicio <= :fechaFin) AND (ch.fechaFin >= :fechaInicio))";
			
			startTime = System.nanoTime();
			a.setNumeroPlazasOfertadasCharlas((Long)getQuerySingleResult(strqry, fechaInicio, fechaFin));
			logger.info("ReportHelperGlobal:getActivities:Número de plazas ofertadas de charlas: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			
			//////////////////////////////////// Número de plazas cubiertas en charlas ////////////////////////////////////
			strqry = "SELECT COUNT(mch) " +
				   	 "FROM " + Charla.class.getName() + " ch INNER JOIN " +
				   	 	"ch.matriculas mch " +
				   	 "WHERE (ch.desactivado = false) AND " +
				   	 	"((ch.fechaInicio <= :fechaFin) AND (ch.fechaFin >= :fechaInicio)) AND " +
				   	 	"(mch.admitido = TRUE) AND " +
				   	 	"(mch.enListaEspera = FALSE)";
			
			startTime = System.nanoTime();
			a.setNumeroPlazasCubiertasCharlas((Long)getQuerySingleResult(strqry, fechaInicio, fechaFin));
			logger.info("ReportHelperGlobal:getActivities:Número de plazas cubiertas en charlas: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			
			//////////////// Número de plazas terminadas en charlas (usuarios matriculados con asistencia >= 80%) /////////////
			strqry = "SELECT DISTINCT ch, ach.usuario.id " +
					 "FROM " + Charla.class.getName() + " ch INNER JOIN " +
					 	"ch.asistencias ach " +
					 "WHERE (ch.desactivado = false) AND " +
					 	"((ch.fechaInicio <= :fechaFin) AND (ch.fechaFin >= :fechaInicio)) " +
					 "GROUP BY ch, ach.usuario " +
					 "HAVING ((COUNT(ach) * 100.00) / ch.calendarItems.size) >= 80.00";
			
			startTime = System.nanoTime();
			a.setNumeroPlazasTerminadasCharlas(getSizeOfQueryResultList(strqry, fechaInicio, fechaFin));
			logger.info("ReportHelperGlobal:getActivities:Número de plazas terminadas en charlas: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			
		} else {
			a.setNumeroHorasCharlas(0F);
			a.setNumeroHorasTerminadasCharlas(0F);
			a.setNumeroPlazasOfertadasCharlas(0L);
			a.setNumeroPlazasCubiertasCharlas(0L);
			a.setNumeroPlazasTerminadasCharlas(0L);
		}
		
		//ACTIVIDADES EXTERNAS EN CESIÓN DE AULA
		// Número de actividades externas en cesión de aula
		strqry = "SELECT COUNT(ae) " +
				 "FROM " + ActividadExterna.class.getName() + " ae " +
				 "WHERE (ae.desactivado = false) AND " +
				 	"(ae.centro IS NOT EMPTY) AND " +
					"(ae.servicio IS EMPTY) AND " +
					"((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio))";
				
		startTime = System.nanoTime();
		nExternasCA = (Long)getQuerySingleResult(strqry, fechaInicio, fechaFin);
		a.setNumeroExternasCA(nExternasCA);
		logger.info("ReportHelperGlobal:getActivities:Número de actividades externas en cesión de aula: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);

		if(nExternasCA > 0) {
			////////////////////////////////////// Número de horas de actividades externas en cesión de aula //////////////////////////////////
			strqry = "SELECT COALESCE(SUM(ROUND((HOUR(aeci.end) - HOUR(aeci.start)) + ((MINUTE(aeci.end) - MINUTE(aeci.start)) / 60.00), 2)), 0.00) + 0.00 " +
					 "FROM " + ActividadExterna.class.getName() + " ae INNER JOIN " +
					 	  "ae.calendarItems aeci " +
					 "WHERE (ae.desactivado = false) AND " +
					 	"(ae.centro IS NOT EMPTY) AND " +
					 	"(ae.servicio IS EMPTY) AND " +
					 	"((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio)) AND " + 
						"((aeci.start <= :fechaFin) AND (aeci.end >= :fechaInicio))"; //new line
			startTime = System.nanoTime();
			a.setNumeroHorasExternasCA(((Double)getQuerySingleResult(strqry, fechaInicio, fechaFin)).floatValue());
			logger.info("ReportHelperGlobal:getActivities:Número de horas de actividades externas en cesión de aula: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			
			//////////////////////////////////// Número de plazas ofertadas de actividades externas en cesión de aula ///////////////////////////////////
			strqry = "SELECT COALESCE(SUM(ae.numeroPlazas), 0) " +
				   	 "FROM " + ActividadExterna.class.getName() + " ae " +
				   	 "WHERE (ae.desactivado = false) AND " +
					 	   "(ae.centro IS NOT EMPTY) AND " +
				    	   "(ae.servicio IS EMPTY) AND " +
				    	   "((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio))";
			
			startTime = System.nanoTime();
			a.setNumeroPlazasOfertadasExternasCA((Long)getQuerySingleResult(strqry, fechaInicio, fechaFin));
			logger.info("ReportHelperGlobal:getActivities:Número de plazas ofertadas de actividades externas en cesión de aula: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			
			//////////////////////////////// Número de plazas terminadas en actividades externas en cesión de aula (usuarios con las encuestas de satisfacción cumplemientadas) ///////////////////
			strqry = "SELECT COUNT(fcae) " +
				     "FROM " + ActividadExterna.class.getName() + " ae INNER JOIN " +
				    	  "ae.formulariosConcretos fcae " +
				     "WHERE (ae.desactivado = false) AND " +
					 	   "(ae.centro IS NOT EMPTY) AND " +
				    	   "(ae.servicio IS EMPTY) AND " +
				    	   "((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio))";
			
			startTime = System.nanoTime();
			a.setNumeroPlazasTerminadasExternasCA((Long)getQuerySingleResult(strqry, fechaInicio, fechaFin));
			logger.info("ReportHelperGlobal:getActivities:Número de plazas terminadas en actividades externas en cesión de aula: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		} else {
			a.setNumeroHorasExternasCA(0F);
			a.setNumeroPlazasOfertadasExternasCA(0L);
			a.setNumeroPlazasTerminadasExternasCA(0L);
		}
		
		//ACTIVIDADES EXTERNAS EN CESIÓN DE SERVICIO
		// Número de actividades externas en cesión de servicio
		strqry = "SELECT COUNT(ae) " +
				 "FROM " + ActividadExterna.class.getName() + " ae " +
				 "WHERE (ae.desactivado = false) AND " +
				 	   "(ae.centro IS EMPTY) AND " +
				 	   "(ae.servicio IS NOT EMPTY) AND " +
				 	   "((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio))";
		
		startTime = System.nanoTime();
		nExternasS = (Long)getQuerySingleResult(strqry, fechaInicio, fechaFin);
		a.setNumeroExternasCS(nExternasS);
		logger.info("ReportHelperGlobal:getActivities:Número de actividades externas en cesión de servicio: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		
		if(nExternasS > 0) {
			////////////////////////////// Número de horas de actividades externas en cesión de servicio //////////////////////////
			strqry = "SELECT COALESCE(SUM(ROUND((HOUR(aeci.end) - HOUR(aeci.start)) + ((MINUTE(aeci.end) - MINUTE(aeci.start)) / 60.00), 2)), 0.00) + 0.00 " +
					 "FROM " + ActividadExterna.class.getName() + " ae INNER JOIN " +
					 	  "ae.calendarItems aeci " +
					 "WHERE (ae.desactivado = false) AND " +
					 	   "(ae.centro IS EMPTY) AND " +
					 	   "(ae.servicio IS NOT EMPTY) AND " +
					 	   "((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio)) AND " + 
					 	   "((aeci.start <= :fechaFin) AND (aeci.end >= :fechaInicio))"; //new line";
			
			startTime = System.nanoTime();
			a.setNumeroHorasExternasCS(((Double)getQuerySingleResult(strqry, fechaInicio, fechaFin)).floatValue());
			logger.info("ReportHelperGlobal:getActivities:Número de horas de actividades externas en cesión de servicio: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			
			//////////////////////////////////// Número de plazas ofertadas de actividades externas en cesión de servicio ////////////////////////////
			strqry = "SELECT COALESCE(SUM(ae.numeroPlazas), 0) " +
				   	 "FROM " + ActividadExterna.class.getName() + " ae " +
				   	 "WHERE (ae.desactivado = false) AND " +
					 	   "(ae.centro IS EMPTY) AND " +
				    	   "(ae.servicio IS NOT EMPTY) AND " +
				    	   "((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio))";
			
			startTime = System.nanoTime();
			a.setNumeroPlazasOfertadasExternasCS((Long)getQuerySingleResult(strqry, fechaInicio, fechaFin));
			logger.info("ReportHelperGlobal:getActivities:Número de plazas ofertadas de actividades externas en cesión de servicio: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			
			////////////////////////////////// Número de plazas terminadas en actividades externas en cesión de servicio (usuarios con las encuestas de satisfacción cumplemientadas) ////////////////
			strqry = "SELECT COUNT(fcae) " +
				     "FROM " + ActividadExterna.class.getName() + " ae INNER JOIN " +
				    	  "ae.formulariosConcretos fcae " +
				     "WHERE (ae.desactivado = false) AND " +
					 	   "(ae.centro IS EMPTY) AND " +
				    	   "(ae.servicio IS NOT EMPTY) AND " +
				    	   "((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio))";
			
			startTime = System.nanoTime();
			a.setNumeroPlazasTerminadasExternasCS((Long)getQuerySingleResult(strqry, fechaInicio, fechaFin));
			logger.info("ReportHelperGlobal:getActivities:Número de plazas terminadas en actividades externas en cesión de servicio: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		} else {
			a.setNumeroHorasExternasCS(0F);
			a.setNumeroPlazasOfertadasExternasCS(0L);
			a.setNumeroPlazasTerminadasExternasCS(0L);
		}
		
		//ACTIVIDADES EXTERNAS EN CESIÓN DE AULA Y SERVICIO
		// Número de actividades externas en cesión de aula y servicio
		strqry = "SELECT COUNT(ae) " +
				 "FROM " + ActividadExterna.class.getName() + " ae " +
				 "WHERE (ae.desactivado = false) AND " +
				 	   "(ae.centro IS NOT EMPTY) AND " +
				 	   "(ae.servicio IS NOT EMPTY) AND " +
				 	   "((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio))";
		
		startTime = System.nanoTime();
		nExternasCAyS = (Long)getQuerySingleResult(strqry, fechaInicio, fechaFin);
		a.setNumeroExternasCAyS(nExternasCAyS);
		logger.info("ReportHelperGlobal:getActivities:Número de actividades externas en cesión de aula y servicio: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		
		if(nExternasCAyS > 0) {
			////////////////////////////////////////// Número de horas de actividades externas en cesión de aula y servicio //////////////////////////////
			strqry = "SELECT COALESCE(SUM(ROUND((HOUR(aeci.end) - HOUR(aeci.start)) + ((MINUTE(aeci.end) - MINUTE(aeci.start)) / 60.00), 2)), 0.00) + 0.00 " +
					 "FROM " + ActividadExterna.class.getName() + " ae INNER JOIN " +
					 	  "ae.calendarItems aeci " +
					 "WHERE (ae.desactivado = false) AND " +
					 	   "(ae.centro IS NOT EMPTY) AND " +
					 	   "(ae.servicio IS NOT EMPTY) AND " +
					 	   "((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio)) AND " + 
					 	   "((aeci.start <= :fechaFin) AND (aeci.end >= :fechaInicio))"; //new line";
			
			startTime = System.nanoTime();
			a.setNumeroHorasExternasCAyS(((Double)getQuerySingleResult(strqry, fechaInicio, fechaFin)).floatValue());
			logger.info("ReportHelperGlobal:getActivities:Número de horas de actividades externas en cesión de aula y servicio: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			
			////////////////////////////////////////// Número de plazas ofertadas de actividades externas en cesión de aula y servicio ////////////////////////////
			strqry = "SELECT COALESCE(SUM(ae.numeroPlazas), 0) " +
				   	 "FROM " + ActividadExterna.class.getName() + " ae " +
				   	 "WHERE (ae.desactivado = false) AND " +
					 	   "(ae.centro IS NOT EMPTY) AND " +
				    	   "(ae.servicio IS NOT EMPTY) AND " +
				    	   "((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio))";
			
			startTime = System.nanoTime();
			a.setNumeroPlazasOfertadasExternasCAyS((Long)getQuerySingleResult(strqry, fechaInicio, fechaFin));
			logger.info("ReportHelperGlobal:getActivities:Número de plazas ofertadas de actividades externas en cesión de aula y servicio: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			
			//////////////////////////////// Número de plazas terminadas en actividades externas en cesión de aula y servicio (usuarios con las encuestas de satisfacción cumplemientadas) /////////////////
			strqry = "SELECT COUNT(fcae) " +
				     "FROM " + ActividadExterna.class.getName() + " ae INNER JOIN " +
				    	  "ae.formulariosConcretos fcae " +
				     "WHERE (ae.desactivado = false) AND " +
					 	   "(ae.centro IS NOT EMPTY) AND " +
				    	   "(ae.servicio IS NOT EMPTY) AND " +
				    	   "((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio))";
			
			startTime = System.nanoTime();
			a.setNumeroPlazasTerminadasExternasCAyS((Long)getQuerySingleResult(strqry, fechaInicio, fechaFin));
			logger.info("ReportHelperGlobal:getActivities:Número de plazas terminadas en actividades externas en cesión de aula y servicio: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			
		} else {
			a.setNumeroHorasExternasCAyS(0F);
			a.setNumeroPlazasOfertadasExternasCAyS(0L);
			a.setNumeroPlazasTerminadasExternasCAyS(0L);
		}

		return a;
	}
	
	private List<OcupacionOfActivitiesByType> getOcupacionOfActivitiesByTypeList(Activities gar) {
	    List<OcupacionOfActivitiesByType> loat = new ArrayList<OcupacionOfActivitiesByType>(3);

	    loat.add(0, new OcupacionOfActivitiesByType("Cursos", gar.getNumeroPlazasCubiertasCursos(), gar.getNumeroPlazasOfertadasCursos()));

	    loat.add(1, new OcupacionOfActivitiesByType("Charlas", gar.getNumeroPlazasCubiertasCharlas(), gar.getNumeroPlazasOfertadasCharlas()));

	    loat.add(2, new OcupacionOfActivitiesByType("Cursos entidades colaboradoras",	
	    											gar.getNumeroPlazasCubiertasExternasCA() + gar.getNumeroPlazasCubiertasExternasCS() + gar.getNumeroPlazasCubiertasExternasCAyS(),
	    											gar.getNumeroPlazasOfertadasExternasCA() + gar.getNumeroPlazasOfertadasExternasCS() + gar.getNumeroPlazasOfertadasExternasCAyS()));

		return loat;
	}
	
	private Long getMaxValueInOcupacionOfActivitiesByTypeList(Activities gar) {
	    Long maximo = gar.getNumeroPlazasCubiertasCursos();
	    
	    if (gar.getNumeroPlazasOfertadasCursos() > maximo)
	    	maximo = gar.getNumeroPlazasOfertadasCursos();

	    
	    if (gar.getNumeroPlazasCubiertasCharlas() > maximo)
	    	maximo = gar.getNumeroPlazasCubiertasCharlas();

	    
	    if (gar.getNumeroPlazasOfertadasCharlas() > maximo)
	    	maximo = gar.getNumeroPlazasOfertadasCharlas();
	    
	    
	    Long numeroPlazasCubiertasExternas = gar.getNumeroPlazasCubiertasExternasCA() +
	    									 gar.getNumeroPlazasCubiertasExternasCS() +
	    									 gar.getNumeroPlazasCubiertasExternasCAyS();
	    if (numeroPlazasCubiertasExternas > maximo)
	    	maximo = numeroPlazasCubiertasExternas;

	    
	    Long numeroPlazasOfertadasExternas = gar.getNumeroPlazasOfertadasExternasCA() +
   	 										 gar.getNumeroPlazasOfertadasExternasCS() +
   	 										 gar.getNumeroPlazasOfertadasExternasCAyS();
	    if (numeroPlazasOfertadasExternas > maximo)
	    	maximo = numeroPlazasOfertadasExternas;

	    
		return maximo;
	}
	
	private List<ActivitiesByCenter> getActivitiesByCenterList(Activities a, Date fechaInicio, Date fechaFin) {
		
		String strqry = "SELECT p AS provincia, " +
						"c AS centro, " +
						"(SELECT COUNT(cu) " +
								"FROM " + Curso.class.getName() + " cu " +
								"WHERE (cu.desactivado = false) AND " +
									  "(cu.centro.id = c.id) AND " +
									  "((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio))) AS numeroCursos, " +
							   "(SELECT COUNT(ch) " +
								"FROM " + Charla.class.getName() + " ch " +
								"WHERE (ch.desactivado = false) AND " +
									  "(ch.centro.id = c.id) AND " +
									  "((ch.fechaInicio <= :fechaFin) AND (ch.fechaFin >= :fechaInicio))) AS numeroCharlas, " +
							   "(SELECT COUNT(ae) " +
							   	"FROM " + ActividadExterna.class.getName() + " ae " +
							   	"WHERE (ae.desactivado = false) AND " +
									  "(ae.centro IS NOT EMPTY) AND " +
							   		  "(ae.centro.id = c.id) AND " +
							   		  "(ae.servicio IS EMPTY) AND " +
							   		  "((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio))) AS numeroExternasCA, " +
							   "(SELECT COUNT(ae) " +
								"FROM " + ActividadExterna.class.getName() + " ae " +
								"WHERE (ae.desactivado = false) AND " +
									  "(ae.centro IS NOT EMPTY) AND " +
									  "(ae.centro.id = c.id) AND " +
									  "(ae.servicio IS NOT EMPTY) AND " +
									  "((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio))) AS numeroExternasCAyS, " +
							   "(SELECT COALESCE(SUM(ROUND((HOUR(cuci.end) - HOUR(cuci.start)) + ((MINUTE(cuci.end) - MINUTE(cuci.start)) / 60.00), 2)), 0.00) + 0.00 " + //new line
							   	"FROM " + Curso.class.getName() + " cu INNER JOIN " + //INNER JOIN is new
							   		  "cu.calendarItems cuci " + //new line
							   	"WHERE (cu.desactivado = false) AND " +
									  "(cu.centro.id = c.id) AND " +
									  "((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio)) AND " + 
									  "((cuci.start <= :fechaFin) AND (cuci.end >= :fechaInicio))) AS numeroHorasCursos, " + // is new
							   "(SELECT COALESCE(SUM(ROUND((HOUR(chci.end) - HOUR(chci.start)) + ((MINUTE(chci.end) - MINUTE(chci.start)) / 60.00), 2)), 0.00) + 0.00 " +
							   	"FROM " + Charla.class.getName() + " ch INNER JOIN " +
							   		  "ch.calendarItems chci " +
							   	"WHERE (ch.desactivado = false) AND " +
									  "(ch.centro.id = c.id) AND " +
									  "((ch.fechaInicio <= :fechaFin) AND (ch.fechaFin >= :fechaInicio))) AS numeroHorasCharlas, " + //(chci.end BETWEEN :fechaInicio AND :fechaFin)) is new
							   "(SELECT COALESCE(SUM(ROUND((HOUR(aeci.end) - HOUR(aeci.start)) + ((MINUTE(aeci.end) - MINUTE(aeci.start)) / 60.00), 2)), 0.00) + 0.00 " +
							   	"FROM " + ActividadExterna.class.getName() + " ae INNER JOIN " +
							   		  "ae.calendarItems aeci " +
							   	"WHERE (ae.desactivado = false) AND " +
									  "(ae.centro IS NOT EMPTY) AND " +
							   		  "(ae.centro.id = c.id) AND " +
							   		  "(ae.servicio IS EMPTY) AND " +
							   		  "((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio)) AND " + 
									  "((aeci.start <= :fechaFin) AND (aeci.end >= :fechaInicio))) AS numeroHorasExternasCA, " + //(aeci.end BETWEEN :fechaInicio AND :fechaFin)) is new
							   "(SELECT COALESCE(SUM(ROUND((HOUR(aeci.end) - HOUR(aeci.start)) + ((MINUTE(aeci.end) - MINUTE(aeci.start)) / 60.00), 2)), 0.00) + 0.00 " +
							   	"FROM " + ActividadExterna.class.getName() + " ae INNER JOIN " +
							   		  "ae.calendarItems aeci " +
							   	"WHERE (ae.desactivado = false) AND " +
									  "(ae.centro IS NOT EMPTY) AND " +
							   		  "(ae.centro.id = c.id) AND " +
							   		  "(ae.servicio IS NOT EMPTY) AND " +
							   		  "((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio)) AND " + 
									  "((aeci.start <= :fechaFin) AND (aeci.end >= :fechaInicio))) AS numeroHorasExternasCAyS, " + //(aeci.end BETWEEN :fechaInicio AND :fechaFin)) is new
							   "(SELECT COUNT(fcae) " +
							    "FROM " + ActividadExterna.class.getName() + " ae INNER JOIN " +
							    	  "ae.formulariosConcretos fcae " +
							    "WHERE (ae.desactivado = false) AND " +
									  "(ae.centro IS NOT EMPTY) AND " +
							   		  "(ae.centro.id = c.id) AND " +
							   		  "(ae.servicio IS EMPTY) AND " +
							   		  "((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio))) AS numeroUsuariosExternasCA, " +
					    	   "(SELECT COUNT(fcae) " +
							    "FROM " + ActividadExterna.class.getName() + " ae INNER JOIN " +
							    	  "ae.formulariosConcretos fcae " +
							    "WHERE (ae.desactivado = false) AND " +
									  "(ae.centro IS NOT EMPTY) AND " +
							   		  "(ae.centro.id = c.id) AND " +
							   		  "(ae.servicio IS NOT EMPTY) AND " +
							   		  "((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio))) AS numeroUsuariosExternasCAyS " +
						"FROM " + Centro.class.getName() + " c INNER JOIN " +
							 "c.provincia p " +
						"WHERE (c.desactivado = FALSE) AND " +
							  "(c.id <> :idCentroAdmin) AND " +
							  "(c.id <> :idCentroColaboradores)";

		Query qry = service.getEntityManager().createQuery(strqry);

		qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
		qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
		qry.setParameter("idCentroAdmin", idCentroAdmin);
		qry.setParameter("idCentroColaboradores", idCentroColaboradores);

		@SuppressWarnings("unchecked")
		List<Object[]> datatmp = (List<Object[]>)qry.getResultList();

		//Variable para el cálculo de horas de aula abierta total en la red
		Float numeroHorasAA = 0.0F;

		List<ActivitiesByCenter> lac = new ArrayList<ActivitiesByCenter>();
		for (Object[] lobj : datatmp) {
			Centro centro = (Centro)lobj[1];

			//Extrae la lista de usuarios con una asistencia >= 80% a los cursos del centro 
			strqry = "SELECT DISTINCT cu, acu.usuario.id " +
					 "FROM " + Curso.class.getName() + " cu INNER JOIN " +
					 	  "cu.asistencias acu " +
					 "WHERE (cu.desactivado = false) AND " +
					 	  "(cu.centro.id = :idCentro) AND " +
					 	  "((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio)) " +
					 "GROUP BY cu, acu.usuario " +
					 "HAVING ((COUNT(acu) * 100.00) / cu.calendarItems.size) >= 80.00";

			qry = service.getEntityManager().createQuery(strqry);

			qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
			qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
			qry.setParameter("idCentro", centro.getId());

			//Almacena el número de usuarios con una asistencia >= 80% a los cursos del centro 
			Long numUsuariosCursos = new Long((long)qry.getResultList().size());

			//Extrae la lista de usuarios con una asistencia >= 80% a las charlas del centro
			strqry = "SELECT DISTINCT ch, ach.usuario.id " +
					 "FROM " + Charla.class.getName() + " ch INNER JOIN " +
					 	  "ch.asistencias ach " +
					 "WHERE (ch.desactivado = false) AND " +
					 	   "(ch.centro.id = :idCentro) AND " +
					 	   "((ch.fechaInicio <= :fechaFin) AND (ch.fechaFin >= :fechaInicio)) " +
					 "GROUP BY ch, ach.usuario " +
					 "HAVING ((COUNT(ach) * 100.00) / ch.calendarItems.size) >= 80.00";

			qry = service.getEntityManager().createQuery(strqry);

			qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
			qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
			qry.setParameter("idCentro", centro.getId());

			//Almacena el número de usuarios con una asistencia >= 80% a las charlas del centro 
			Long numUsuariosCharlas = new Long((long)qry.getResultList().size());

			//Calcula los CalendarItem de aula abierta en el centro entre las fechas dadas
			List<CalendarItem> OpenClassromCalendarItems = null;
			OpenClassromCalendarItems = CalendarControllerImpl.getOpenClassromCalendarItems(getCentroItems(centro.getId(), fechaInicio, fechaFin),
																							findActividadesItems(centro.getId(), fechaInicio, fechaFin));

			//Cálculo de horas de aula abierta en el centro con los Calendar Item
			Float numeroHorasAACentro = 0.0F;
			for (CalendarItem occi : OpenClassromCalendarItems)
				numeroHorasAACentro += ((occi.getEnd().getTime() - occi.getStart().getTime()) / 3600000.0F); //Extrae la duración en horas.

			//Agrega el cálculo de horas de aula abierta en el centro al cómputo global de horas de aula abierta total en la red
			numeroHorasAA += numeroHorasAACentro;

			//Almacena todos los resultados en la estrutura a devolver
			ActivitiesByCenter ac = new ActivitiesByCenter((Provincia)lobj[0],
														   centro,
														   (Long)lobj[2], // Número de cursos
														   (Long)lobj[3], // Número de charlas
														   (Long)lobj[4], // Número de actividades externas CA
														   (Long)lobj[5], // Número de actividades externas CAyS
														   numeroHorasAACentro,			   // Numero de horas de aula abierta en el centro
														   ((Double)lobj[6]).floatValue(), // Número de horas de cursos
														   ((Double)lobj[7]).floatValue(), // Número de horas de charlas
														   ((Double)lobj[8]).floatValue(), // Número de horas de actividades externas CA
														   ((Double)lobj[9]).floatValue(), // Número de horas de actividades externas CAyS
														   numUsuariosCursos,
														   numUsuariosCharlas,
														   (Long)lobj[10],  // Número de ususarios en actividades externas CA   (número de encuestas de satisfacción realizadas)
														   (Long)lobj[11]); // Número de ususarios en actividades externas CAyS (número de encuestas de satisfacción realizadas)

			lac.add(ac);
		}

		//Almacena el número de horas calculado de aula abierta en la red en la estructura definitiva
		a.setNumeroHorasAA(numeroHorasAA);

		return lac;
	}
	
	private List<ActivitiesByThematic> getActivitiesByThematicList(Date fechaInicio, Date fechaFin) {
		
		String strqry = "SELECT tl.tematica.id AS idTematica, " +
						"tl.nombre AS nombreTematica, " +
						"(SELECT COUNT(cu) " +
						"FROM " + Curso.class.getName() + " cu " +
						"WHERE (cu.desactivado = false) AND " +
							 "((cu.servicio.tematica.id = tl.tematica.id) OR " +
							 "(cu.servicio.tematica.padre.id = tl.tematica.id)) AND " +
							 "(cu.servicio.ajenoPlanFormativo = FALSE) AND " +
							 "((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio))) AS numeroCursosDP, " +
						"(SELECT COUNT(cu) " +
						"FROM " + Curso.class.getName() + " cu " +
						"WHERE (cu.desactivado = false) AND " +
							"((cu.servicio.tematica.id = tl.tematica.id) OR " +
							"(cu.servicio.tematica.padre.id = tl.tematica.id)) AND " +
							"(cu.servicio.ajenoPlanFormativo = TRUE) AND " +
							"((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio))) AS numeroCursosFP, " +
						"(SELECT COUNT(ch) " +
						"FROM " + Charla.class.getName() + " ch " +
						"WHERE (ch.desactivado = false) AND " +
							"((ch.tematica.id = tl.tematica.id) OR " +
							"(ch.tematica.padre.id = tl.tematica.id)) AND " +
							"((ch.fechaInicio <= :fechaFin) AND (ch.fechaFin >= :fechaInicio))) AS numeroCharlas, " +
						"(SELECT COUNT(ae) " +
						"FROM " + ActividadExterna.class.getName() + " ae INNER JOIN " +
							"ae.servicio s " +
						"WHERE (ae.desactivado = false) AND " +
							"((s.tematica.id = tl.tematica.id) OR " +
							"(s.tematica.padre.id = tl.tematica.id)) AND " +
							"(s.ajenoPlanFormativo = FALSE) AND " +
							"((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio))) AS numerExternasDP, " +
						"(SELECT COUNT(ae) " +
						"FROM " + ActividadExterna.class.getName() + " ae INNER JOIN " +
							"ae.servicio s " +
						"WHERE (ae.desactivado = false) AND " +
							"((s.tematica.id = tl.tematica.id) OR " +
							"(s.tematica.padre.id = tl.tematica.id)) AND" +
							"(s.ajenoPlanFormativo = TRUE) AND " +
							"((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio))) AS numeroExternasCSyFP, " +
						"(SELECT COUNT(ae) " +
						"FROM " + ActividadExterna.class.getName() + " ae LEFT JOIN " +
							"ae.servicio s " +
						"WHERE (ae.desactivado = false) AND " +
							"(s IS NULL) AND " +
							"((ae.tematica.id = tl.tematica.id) OR " +
							"(ae.tematica.padre.id = tl.tematica.id)) AND " +
							"((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio))) AS numeroExternasCAyFP " +
						"FROM " + TematicaLanguage.class.getName() + " tl " +
						"WHERE tl.language.ISO = :languageISO AND " +
							  "tl.tematica.padre IS EMPTY";
		
		Query qry = service.getEntityManager().createQuery(strqry);
		
		qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
		qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
		qry.setParameter("languageISO", locale.toString());
		
		@SuppressWarnings("unchecked")
		List<Object[]> datatmp = (List<Object[]>)qry.getResultList();

		List<ActivitiesByThematic> lat = new ArrayList<ActivitiesByThematic>();
		for (Object[] lobj : datatmp) {
			ActivitiesByThematic at = new ActivitiesByThematic((Long)lobj[0],   //id de la temática
															   (String)lobj[1], //nombre de la temática
															   (Long)lobj[2],   // Número de cursos dentro del plan formativo asociados a la temática
															   (Long)lobj[3],   // Número de cursosfuera del plan formativo asociados a la temática
															   (Long)lobj[4],   // Número de charlas asociados a la temática
															   (Long)lobj[5],   // Número de actividades externas dentro del plan formativo asociados a la temática
											   (Long)lobj[6] + (Long)lobj[7]);  // Número de actividades externas fuera del plan formativo asociados a la temática

			lat.add(at);
		}
		
		return lat;
	}
	
	private List<ActivitiesByLevel> getActivitiesByLevelList(Date fechaInicio, Date fechaFin) {
		List<ActivitiesByLevel> lal = new ArrayList<ActivitiesByLevel>();

		for(Nivel n : Nivel.values()) {
			Long numeroCursosDP;
			Long numeroCursosFP;
			Long numeroExternasDP;
			Long numeroExternasFP;
			
			String strqry;

			Query qry;
			
			strqry = "SELECT COUNT(cu) AS numeroCursosDP " + 
					 "FROM " + Curso.class.getName() + " cu, " + ServicioCurso.class.getName() + " sc " +
					 "WHERE (cu.desactivado = false) AND " +
					 	   "(cu.servicio.id = sc.id) AND " +
					 	   "(sc.nivel = :level) AND " +
					 	   "(cu.servicio.ajenoPlanFormativo = FALSE) AND " +
					 	   "((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio))";
			
			qry = service.getEntityManager().createQuery(strqry);
			
			qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
			qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
			qry.setParameter("level", n);
			
			startTime = System.nanoTime();
			numeroCursosDP = (Long)qry.getSingleResult();
			logger.info("ReportHelperGlobal:getActivitiesByLevelList:Número cursos DP: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			
			strqry = "SELECT COUNT(cu) AS numeroCursosFP " +
					 "FROM " + Curso.class.getName() + " cu, " + ServicioCurso.class.getName() + " sc " +
					 "WHERE (cu.desactivado = false) AND " +
					 	   "(cu.servicio.id = sc.id) AND " +
					 	   "(sc.nivel = :level) AND " +
					 	   "(cu.servicio.ajenoPlanFormativo = TRUE) AND " +
					 	   "((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio))";

			qry = service.getEntityManager().createQuery(strqry);
			
			qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
			qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
			qry.setParameter("level", n);
			
			startTime = System.nanoTime();
			numeroCursosFP = (Long)qry.getSingleResult();
			logger.info("ReportHelperGlobal:getActivitiesByLevelList:Número cursos FP: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	
			strqry = "SELECT COUNT(ae) AS numerExternasDP " +
					 "FROM " + ActividadExterna.class.getName() + " ae, " + ServicioCurso.class.getName() + " sc " +
					 "WHERE (ae.desactivado = false) AND " +
					 	   "(ae.servicio IS NOT EMPTY) AND " +
					 	   "(ae.servicio.id = sc.id) AND " +
					 	   "(sc.nivel = :level) AND " +
					 	   "(ae.servicio.ajenoPlanFormativo = FALSE) AND " +
					 	   "((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio)))";
			
			qry = service.getEntityManager().createQuery(strqry);
			
			qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
			qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
			qry.setParameter("level", n);
			
			startTime = System.nanoTime();
			numeroExternasDP = (Long)qry.getSingleResult();
			logger.info("ReportHelperGlobal:getActivitiesByLevelList:Número actividades externas DP: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
	
			strqry = "SELECT COUNT(ae) AS numerExternasDP " +
					 "FROM " + ActividadExterna.class.getName() + " ae, " + ServicioCurso.class.getName() + " sc " +
					 "WHERE (ae.desactivado = false) AND " +
					 	   "(ae.servicio IS NOT EMPTY) AND " +
					 	   "(ae.servicio.id = sc.id) AND " +
					 	   "(sc.nivel = :level) AND " +
					 	   "(ae.servicio.ajenoPlanFormativo = TRUE) AND " +
					 	   "((ae.fechaInicio <= :fechaFin) AND (ae.fechaFin >= :fechaInicio))";
			
			qry = service.getEntityManager().createQuery(strqry);
			
			qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
			qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
			qry.setParameter("level", n);
			
			startTime = System.nanoTime();
			numeroExternasFP = (Long)qry.getSingleResult();
			logger.info("ReportHelperGlobal:getActivitiesByLevelList:Número actividades externas FP: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);

			ActivitiesByLevel al = new ActivitiesByLevel(n, // Nivel
														 numeroCursosDP,    // Número de cursos dentro del plan formativo asociados al nivel
														 numeroCursosFP,    // Número de cursosfuera del plan formativo asociados al nivel
														 numeroExternasDP,  // Número de actividades externas dentro del plan formativo asociados al nivel
														 numeroExternasFP,  // Número de actividades externas fuera del plan formativo asociados al nivel
														 locale);

			lal.add(al);
		}
		
		return lal;
	}
	
	private List<RegisteredUsersByCenter> getRegisteredUsersByCenterList(Date fechaInicio, Date fechaFin) {
		
		String strqry = "SELECT p AS provincia, " +
						"c AS centro, " +
						"(SELECT COUNT(ucyl) " +
							"FROM " + UsuarioCyL.class.getName() + " ucyl " +
							"WHERE (ucyl.tipoUsuario = 'SC') AND " +
							"(ucyl.centro.id = c.id) AND " +
							"(ucyl.desactivado = FALSE) AND " +
							"(ucyl.fechaCreacion BETWEEN :fechaInicio AND :fechaFin)) AS numeroUsuariosSC, " +
						"(SELECT COUNT(ucyl) " +
							"FROM " + UsuarioCyL.class.getName() + " ucyl " +
							"WHERE (ucyl.tipoUsuario = 'B') AND " +
							"(ucyl.centro.id = c.id) AND " +
							"(ucyl.desactivado = FALSE) AND " +
							"(ucyl.fechaCreacion BETWEEN :fechaInicio AND :fechaFin)) AS numeroUsuariosB, " +
						"(SELECT COUNT(ucyl) " +
							"FROM " + UsuarioCyL.class.getName() + " ucyl " +
							"WHERE (ucyl.tipoUsuario = 'NU') AND " +
							"(ucyl.centro.id = c.id) AND " +
							"(ucyl.desactivado = FALSE) AND " +
							"(ucyl.fechaCreacion BETWEEN :fechaInicio AND :fechaFin)) AS numeroUsuariosNU, " +
						"(SELECT COUNT(ucyl) " +
							"FROM " + UsuarioCyL.class.getName() + " ucyl " +
							"WHERE (ucyl.tipoUsuario = 'TA') AND " +
								"(ucyl.centro.id = c.id) AND " +
								"(ucyl.desactivado = FALSE) AND " +
								"(ucyl.fechaCreacion BETWEEN :fechaInicio AND :fechaFin)) AS numeroUsuariosTA " +
						"FROM " + Centro.class.getName() + " c INNER JOIN " +
						     "c.provincia p " +
						"WHERE (c.desactivado = FALSE) AND " +
							  "(c.id <> :idCentroAdmin) AND " +
							  "(c.id <> :idCentroColaboradores)";

		Query qry = service.getEntityManager().createQuery(strqry);

		qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
		qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
		qry.setParameter("idCentroAdmin", idCentroAdmin);
		qry.setParameter("idCentroColaboradores", idCentroColaboradores);		
		
		@SuppressWarnings("unchecked")
		List<Object[]> datatmp = (List<Object[]>)qry.getResultList();

		List<RegisteredUsersByCenter> lruc = new ArrayList<RegisteredUsersByCenter>();
		for (Object[] lobj : datatmp) {
			RegisteredUsersByCenter ruc = new RegisteredUsersByCenter((Provincia)lobj[0], // Provincia
																	  (Centro)lobj[1],	  // Centro
																	  (Long)lobj[2],	  // Número de usuarios sin conocimientos registrados
																	  (Long)lobj[3],	  // Número de usuarios básicos registrados 
																	  (Long)lobj[4],	  // Número de usuarios nivel usuario registrados
																	  (Long)lobj[5],	  // Número de usuarios técnicos avanzados registrados
																	  Long.valueOf(-1L),  // Ranking provincial
																	  Long.valueOf(-1L)); // Ranking global

			lruc.add(ruc);
		}
		
		Long ranking;
		Long centerRegisteredUsers;
		
		// Ordena por cantidad de usuarios registrados de menor a mayor
		Collections.sort(lruc, RegisteredUsersByCenter.comparatorRegisteredUsersDESC);
		
		// Asigna el ranking global;
		ranking = 0L;
		centerRegisteredUsers = Long.MAX_VALUE;
		for (RegisteredUsersByCenter ruc : lruc) {
			if (centerRegisteredUsers > ruc.getCenterRegisteredUsers()) {
				centerRegisteredUsers = ruc.getCenterRegisteredUsers();
				ranking++;
			}
			
			ruc.setRankingGlobal(ranking);
		}

		// Ordena por cantidad de usuarios registrados dentro de cada provincia
		Collections.sort(lruc, RegisteredUsersByCenter.comparatorProvinciaASCRegisteredUsersDESC);
		
		// Asigna el ranking provincial
		Provincia p = null;
		ranking = 0L;
		centerRegisteredUsers = -1L;
		for (RegisteredUsersByCenter ruc : lruc) {
			if (p != ruc.getProvincia()) {
				p = ruc.getProvincia();
				ranking = 0L;
				centerRegisteredUsers = Long.MAX_VALUE;
			}
			
			if (centerRegisteredUsers > ruc.getCenterRegisteredUsers()) {
				centerRegisteredUsers = ruc.getCenterRegisteredUsers();
				ranking++;
			}

			ruc.setRankingProvincial(ranking);
		}

		// Ordena por el nombre de la provincia y el nombre del centro
		Collections.sort(lruc, RegisteredUsersByCenter.comparatorProvinciaASCCentroASC);
		
		return lruc;
	}
	
	private Long getTotalRegisteredUsers(List<RegisteredUsersByCenter> lruc) {
		Long totalRegisteredUsers = 0L;

		for (RegisteredUsersByCenter ruc : lruc)
			totalRegisteredUsers += ruc.getCenterRegisteredUsers();

		return totalRegisteredUsers;
	}

	private List<UsersExamPassedByServices> getUsersExamPassedByServicesList(Date fechaInicio, Date fechaFin) {
		
		String strqry = "SELECT sc.id AS idServicio, scl.nombre AS nombreServicio " +
						"FROM " + Curso.class.getName() + " cu INNER JOIN " +
							 "cu.servicio sc INNER JOIN " +
							 "sc.textos scl " +
						"WHERE (sc.examen IS NOT NULL) AND " +
							  "(scl.language.ISO = :languageISO) AND " +
							  "((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio)) " +
						"GROUP BY sc " +
						"ORDER BY scl.nombre ASC, " +
								 "sc.id ASC";
		
		Query qry = service.getEntityManager().createQuery(strqry);

		qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
		qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
		qry.setParameter("languageISO", locale.toString());
		
		@SuppressWarnings("unchecked")
		List<Object[]> datatmp = (List<Object[]>)qry.getResultList();

		List<UsersExamPassedByServices> lueps = new ArrayList<UsersExamPassedByServices>();
		for (Object[] lobj : datatmp) {
			Long idServicio = (Long)lobj[0];
			String nombreServicio = (String)lobj[1];

			strqry = "SELECT DISTINCT cu.id, acu.usuario.id " +
					 "FROM " + Curso.class.getName() + " cu INNER JOIN " +
					 	  "cu.servicio sc INNER JOIN " +
						  "cu.asistencias acu " +
					 "WHERE (sc.examen IS NOT NULL) AND " +
						   "(sc.id = :idServicio) AND " +
						   "((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio)) " +
					 "GROUP BY cu.id, acu.usuario " +
					 "HAVING ((COUNT(acu) * 100.00) / cu.calendarItems.size) >= 80.00";
			
			qry = service.getEntityManager().createQuery(strqry);

			qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
			qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
			qry.setParameter("idServicio", idServicio);
			
			Long numUsuariosCanMakeExam = Long.valueOf((long)qry.getResultList().size());

			strqry = "SELECT COUNT(DISTINCT mcu) " +
					 "FROM " + MatriculaCurso.class.getName() + " mcu INNER JOIN " +
						  "mcu.curso cu INNER JOIN " +
						  "cu.servicio sc " +
					 "WHERE (sc.examen IS NOT NULL) AND " +
						   "(mcu.examenFinal IS NOT NULL) AND " +
						   "(mcu.nota >= cu.examen.numPreguntasAprobado) AND " +
						   "(sc.id = :idServicio) AND " +
						   "((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio))";
			
			qry = service.getEntityManager().createQuery(strqry);

			qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
			qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
			qry.setParameter("idServicio", idServicio);
			
			Long numAprobados = (Long)qry.getSingleResult();

			lueps.add(new UsersExamPassedByServices(idServicio, nombreServicio, numUsuariosCanMakeExam, numAprobados));
		}
		
		return lueps;
	}
	
	private List<UsersExamPassedByCentersAndTrainers> getUsersExamPassedByCentersAndTrainersList(Date fechaInicio, Date fechaFin) {
		String strqry = "SELECT c.id AS idCentro, " +
							"c.nombre AS nombreCentro, " +
							"p.id AS idProfesor, " +
							"CONCAT(p.apellido1, ' ', p.apellido2, ', ', p.nombre) AS nombreProfesor " +	   
						"FROM " + Curso.class.getName() + " cu INNER JOIN " +
							"cu.servicio sc INNER JOIN " +
							"cu.centro c INNER JOIN " +
							"cu.profesor p " +
						"WHERE (sc.examen IS NOT NULL) AND " +
							"((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio)) " +
						"GROUP BY c, p " +
						"ORDER BY c.nombre ASC, " +
								 "c.id ASC, " +
								 "p.apellido1 ASC, " +
								 "p.apellido2 ASC, " +
								 "p.nombre ASC, " +
								 "p.id ASC";
		
		Query qry = service.getEntityManager().createQuery(strqry);

		qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
		qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
		
		@SuppressWarnings("unchecked")
		List<Object[]> datatmp = (List<Object[]>)qry.getResultList();

		List<UsersExamPassedByCentersAndTrainers> luepct = new ArrayList<UsersExamPassedByCentersAndTrainers>();
		for (Object[] lobj : datatmp) {
			Long idCentro = (Long)lobj[0];
			String nombreCentro = (String)lobj[1];
			Long idProfesor = (Long)lobj[2];
			String nombreProfesor = (String)lobj[3];
			
			strqry = "SELECT DISTINCT cu.id, acu.usuario.id " +
					 "FROM " + Curso.class.getName() + " cu INNER JOIN " +
						  "cu.asistencias acu INNER JOIN " +
						  "cu.servicio sc " +
					 "WHERE (sc.examen IS NOT NULL) AND " +
					 	   "(cu.centro.id = :idCentro) AND " +
						   "(cu.profesor.id = :idProfesor) AND " +
						   "((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio)) " +
					 "GROUP BY cu.id, acu.usuario " +
					 "HAVING ((COUNT(acu) * 100.00) / cu.calendarItems.size) >= 80.00";
			
			qry = service.getEntityManager().createQuery(strqry);

			qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
			qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
			qry.setParameter("idCentro", idCentro);
			qry.setParameter("idProfesor", idProfesor);
			
			Long numUsuariosCanMakeExam = Long.valueOf((long)qry.getResultList().size());
			
			strqry = "SELECT COUNT(DISTINCT mcu) " +
					 "FROM " + MatriculaCurso.class.getName() + " mcu INNER JOIN " +
						  "mcu.curso cu INNER JOIN " +
						  "cu.servicio sc " +
					 "WHERE (sc.examen  IS NOT NULL) AND " +
						   "(mcu.examenFinal IS NOT NULL) AND " +
						   "(mcu.nota >= cu.examen.numPreguntasAprobado) AND " +
						   "(cu.centro.id = :idCentro) AND " +
						   "(cu.profesor.id = :idProfesor) AND " +
						   "((cu.fechaInicio <= :fechaFin) AND (cu.fechaFin >= :fechaInicio))";
			
			qry = service.getEntityManager().createQuery(strqry);

			qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
			qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
			qry.setParameter("idCentro", idCentro);
			qry.setParameter("idProfesor", idProfesor);

			Long numAprobados = (Long)qry.getSingleResult();

			luepct.add(new UsersExamPassedByCentersAndTrainers(idCentro, nombreCentro,
															   idProfesor, nombreProfesor,
															   numUsuariosCanMakeExam, numAprobados));
		}

		return luepct;
	}
	
	private List<AverageRatingOfQuestionByProvince> getAverageRatingOfQuestionByProvinceList(Date fechaInicio, Date fechaFin) {
		String strqry = null;
		Query qry = null;
		
		// Extrae lista de provincias que tienen centro
		strqry = "SELECT p AS provincia " +
				 "FROM " + Centro.class.getName() + " c INNER JOIN " +
				 	   "c.provincia p " +
				 "WHERE (c.desactivado = FALSE) AND " +
				 	   "(c.id <> :idCentroAdmin) AND " +
				 	   "(c.id <> :idCentroColaboradores) " +
				 "GROUP BY p " +
				 "ORDER BY p.nombre ASC";
		
		qry = service.getEntityManager().createQuery(strqry);
		
		qry.setParameter("idCentroAdmin", idCentroAdmin);
		qry.setParameter("idCentroColaboradores", idCentroColaboradores);

		startTime = System.nanoTime();
		@SuppressWarnings("unchecked")
		List<Provincia> provincias = (List<Provincia>)qry.getResultList();
		logger.info("ReportHelperGlobal:getAverageRatingOfQuestionByProvinceList:lista de provincias que tienen centro: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		
		// Extrae lista de preguntas de encuestas de satisfacción respondidas en cursos históricamente
		strqry = "SELECT (tpcu.etiqueta || ' ' || tpcu.nombre) AS textoPregunta, " +
						"pcu AS pregunta " +
				 "FROM " + MatriculaCurso.class.getName() + " mcu INNER JOIN " +
					  "mcu.formularioConcreto.respuestas rcu INNER JOIN " +
					  "rcu.pregunta pcu WITH ((pcu.obligatorio = TRUE) AND " +
											 "((pcu.tipo = 'ENTERO') OR " +
											  "(pcu.tipo = 'DECIMAL'))) INNER JOIN " +
					  "pcu.textos tpcu INNER JOIN " +
					  "tpcu.language ltpcu WITH (ltpcu.ISO = :languageISO) " +
				 "WHERE (mcu.curso.desactivado = FALSE) " +
				 "GROUP BY pcu " +
				 "ORDER BY tpcu.etiqueta ASC, " +
						  "tpcu.nombre ASC";
		
		qry = service.getEntityManager().createQuery(strqry);
		
		qry.setParameter("languageISO", locale.toString());
		
		startTime = System.nanoTime();
		@SuppressWarnings("unchecked")
		List<Object[]> preguntas = (List<Object[]>)qry.getResultList();
		logger.info("ReportHelperGlobal:getAverageRatingOfQuestionByProvinceList:lista de preguntas de encuestas de satisfacción respondidas en cursos históricamente: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		
		// Crea la estructura de respuesta
		List<AverageRatingOfQuestionByProvince> larqp = new ArrayList<AverageRatingOfQuestionByProvince>();
		
		// Se realizan las consultas por cada provincia y pregunta y se rellena la estructura de respuesta.
		for (Provincia provincia : provincias) {
			for (Object[] pregunta : preguntas) {
				Pregunta p = (Pregunta)pregunta[1];
				
				// Extrae el valor medio en el periodo seleccionado de las respuestas de la pregunta y provincia selecionados
				strqry = "SELECT COALESCE((CASE pcu_sp.tipo " +
											"WHEN 'ENTERO' THEN (ROUND(AVG(rcu_sp.valorEntero), 2) * 1.0F) " +
											"WHEN 'DECIMAL' THEN (ROUND(AVG(rcu_sp.valorDecimal), 2) * 1.0F) " +
										  "END), -1.0F) AS valorMedioPeriodo " +
						 "FROM " + MatriculaCurso.class.getName() + " mcu_sp INNER JOIN " +
						 	   "mcu_sp.curso cu_sp WITH (((cu_sp.fechaInicio BETWEEN :fechaInicio AND :fechaFin) OR " +
						 								 "(cu_sp.fechaFin BETWEEN :fechaInicio AND :fechaFin)) OR " +
						 								"((:fechaInicio BETWEEN cu_sp.fechaInicio AND cu_sp.fechaFin) AND " +
						 								 "(:fechaFin BETWEEN cu_sp.fechaInicio AND cu_sp.fechaFin))) INNER JOIN " +
						 	   "cu_sp.centro c_sp WITH (c_sp.provincia.id = :idProvincia) INNER JOIN " +
						 	   "mcu_sp.formularioConcreto.respuestas rcu_sp INNER JOIN " +
						 	   "rcu_sp.pregunta pcu_sp WITH (pcu_sp.id = :idPregunta) " +
						 "WHERE (cu_sp.desactivado = FALSE)";
				
				qry = service.getEntityManager().createQuery(strqry);
				
				qry.setParameter("idProvincia", provincia.getId());
				qry.setParameter("idPregunta", p.getId());
				qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
				qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
				
				startTime = System.nanoTime();
				Float valorMedioPeriodo = (Float)qry.getSingleResult();
				logger.info("ReportHelperGlobal:getAverageRatingOfQuestionByProvinceList:valor medio en el periodo seleccionado de las respuestas de la pregunta y provincia selecionados: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
				
				// Extrae el valor medio histórico de las respuestas de la pregunta y provincia selecionados
				strqry = "SELECT COALESCE((CASE pcu_sh.tipo " +
											"WHEN 'ENTERO' THEN (ROUND(AVG(rcu_sh.valorEntero), 2) * 1.0F) " +
											"WHEN 'DECIMAL' THEN (ROUND(AVG(rcu_sh.valorDecimal), 2) * 1.0F) " +
										  "END), -1.0F) AS valorMedioHistorico " +
						 "FROM " + MatriculaCurso.class.getName() + " mcu_sh INNER JOIN " +
						 	  "mcu_sh.curso.centro c_sh INNER JOIN " +
						 	  "mcu_sh.formularioConcreto.respuestas rcu_sh INNER JOIN " +
						 	  "rcu_sh.pregunta pcu_sh WITH (pcu_sh.id = :idPregunta) " +
						 "WHERE (mcu_sh.curso.desactivado = FALSE) AND " +
						 	   "(c_sh.provincia.id = :idProvincia)";
				
				qry = service.getEntityManager().createQuery(strqry);
				
				qry.setParameter("idProvincia", provincia.getId());
				qry.setParameter("idPregunta", p.getId());
				
				startTime = System.nanoTime();
				Float valorMedioHistorico = (Float)qry.getSingleResult();
				logger.info("ReportHelperGlobal:getAverageRatingOfQuestionByProvinceList:valor medio histórico de las respuestas de la pregunta y provincia selecionados: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
				
				// Se almacena en la estructura de respuesta
				AverageRatingOfQuestionByProvince arqp = new AverageRatingOfQuestionByProvince(
						(Provincia)provincia,																	  // Provincia
						(String)(pregunta[0]),																	  // Pregunta
						((p.getTipo() == TipoPregunta.ENTERO) ? new Float(p.getMinimo()) : p.getMinimoDecimal()), // Valor mínimo posible
						((p.getTipo() == TipoPregunta.ENTERO) ? new Float(p.getMaximo()) : p.getMaximoDecimal()), // Valor máximo posible
						valorMedioPeriodo,																		  // Valor medio del periodo
						valorMedioHistorico);																	  // Valor medio global

				larqp.add(arqp);
			}
		}
		
		return larqp; 
	}
	
	private List<AverageRatingOfQuestion> getAverageRatingOfQuestionList(Date fechaInicio, Date fechaFin) {
		String strqry = null;
		Query qry = null;
		
		// Extrae lista de preguntas de encuestas de satisfacción respondidas en cursos históricamente
		strqry = "SELECT (tpcu.etiqueta || ' ' || tpcu.nombre) AS textoPregunta, " +
						"pcu AS pregunta " +
				 "FROM " + MatriculaCurso.class.getName() + " mcu INNER JOIN " +
					  "mcu.formularioConcreto.respuestas rcu INNER JOIN " +
					  "rcu.pregunta pcu WITH ((pcu.obligatorio = TRUE) AND " +
											 "((pcu.tipo = 'ENTERO') OR " +
											  "(pcu.tipo = 'DECIMAL'))) INNER JOIN " +
					  "pcu.textos tpcu INNER JOIN " +
					  "tpcu.language ltpcu WITH (ltpcu.ISO = :languageISO) " +
				 "WHERE (mcu.curso.desactivado = FALSE) " +
				 "GROUP BY pcu " +
				 "ORDER BY tpcu.etiqueta ASC, " +
						  "tpcu.nombre ASC";
		
		qry = service.getEntityManager().createQuery(strqry);
		
		qry.setParameter("languageISO", locale.toString());
		
		startTime = System.nanoTime();
		@SuppressWarnings("unchecked")
		List<Object[]> preguntas = (List<Object[]>)qry.getResultList();
		logger.info("ReportHelperGlobal:getAverageRatingOfQuestionList:lista de preguntas de encuestas de satisfacción respondidas en cursos históricamente: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		
		// Crea la estructura de respuesta
		List<AverageRatingOfQuestion> larq = new ArrayList<AverageRatingOfQuestion>();
		
		// Se realizan las consultas por cada pregunta y se rellena la estructura de respuesta.
		for (Object[] pregunta : preguntas) {
			Pregunta p = (Pregunta)pregunta[1];
			
			// Extrae el valor medio en el periodo seleccionado de las respuestas de la pregunta selecionada
			strqry = "SELECT COALESCE((CASE pcu_sp.tipo " +
										"WHEN 'ENTERO' THEN (ROUND(AVG(rcu_sp.valorEntero), 2) * 1.0F) " +
										"WHEN 'DECIMAL' THEN (ROUND(AVG(rcu_sp.valorDecimal), 2) * 1.0F) " +
									  "END), -1.0F) AS valorMedioPeriodo " +
					 "FROM " + MatriculaCurso.class.getName() + " mcu_sp INNER JOIN " +
					 	   "mcu_sp.curso cu_sp WITH (((cu_sp.fechaInicio <= :fechaFin) AND (cu_sp.fechaFin >= :fechaInicio))) INNER JOIN " +
					 	   "mcu_sp.formularioConcreto.respuestas rcu_sp INNER JOIN " +
					 	   "rcu_sp.pregunta pcu_sp WITH (pcu_sp.id = :idPregunta) " + 
					 "WHERE (cu_sp.desactivado = FALSE)"; // AND " +
					 	  //"(cu_sp.servicio.desactivado = FALSE)";
			
			qry = service.getEntityManager().createQuery(strqry);
			
			qry.setParameter("idPregunta", p.getId());
			qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
			qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);
			
			startTime = System.nanoTime();
			Float valorMedioPeriodo = (Float)qry.getSingleResult();
			logger.info("ReportHelperGlobal:getAverageRatingOfQuestionList:valor medio en el periodo seleccionado de las respuestas de la pregunta selecionada: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			
			// Extrae el valor medio histórico de las respuestas de la pregunta selecionada
			strqry = "SELECT COALESCE((CASE pcu_sh.tipo " +
										"WHEN 'ENTERO' THEN (ROUND(AVG(rcu_sh.valorEntero), 2) * 1.0F) " +
										"WHEN 'DECIMAL' THEN (ROUND(AVG(rcu_sh.valorDecimal), 2) * 1.0F) " +
									  "END), -1.0F) AS valorMedioHistorico " +
					 "FROM " + MatriculaCurso.class.getName() + " mcu_sh INNER JOIN " +
					 	  "mcu_sh.formularioConcreto.respuestas rcu_sh INNER JOIN " +
					 	  "rcu_sh.pregunta pcu_sh WITH (pcu_sh.id = :idPregunta) " +
					 "WHERE (mcu_sh.curso.desactivado = FALSE)"; // AND " +
					 	 //"(mcu_sh.curso.servicio.desactivado = FALSE)";
			
			qry = service.getEntityManager().createQuery(strqry);
			
			qry.setParameter("idPregunta", p.getId());
			
			startTime = System.nanoTime();
			Float valorMedioHistorico = (Float)qry.getSingleResult();
			logger.info("ReportHelperGlobal:getAverageRatingOfQuestionList:valor medio histórico de las respuestas de la pregunta selecionada: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			
			// Se almacena en la estructura de respuesta
			AverageRatingOfQuestion arq = new AverageRatingOfQuestion(
					(String)(pregunta[0]),																	  // Pregunta
					((p.getTipo() == TipoPregunta.ENTERO) ? new Float(p.getMinimo()) : p.getMinimoDecimal()), // Valor mínimo posible
					((p.getTipo() == TipoPregunta.ENTERO) ? new Float(p.getMaximo()) : p.getMaximoDecimal()), // Valor máximo posible
					valorMedioPeriodo,																		  // Valor medio del periodo
					valorMedioHistorico);																	  // Valor medio global

			larq.add(arq);
		}
		
		return larq; 
	}

	private List<AverageRatingOfQuestionsByCenter> getAverageRatingOfQuestionsByCenterList(Date fechaInicio, Date fechaFin) {
		String strqry = null;
		Query qry = null;
		
		// Extrae la lista de centros activados de la red que no sean el de Administradores ni el de Colaboradores.
		startTime = System.nanoTime();

		strqry = "SELECT c " +
				 "FROM " + Centro.class.getName() + " c " +
				 "WHERE (c.desactivado = FALSE) AND " +
				 	   "(c.id <> :idCentroAdmin) AND " +
				 	   "(c.id <> :idCentroColaboradores)";
		
		qry = service.getEntityManager().createQuery(strqry);

		qry.setParameter("idCentroAdmin", idCentroAdmin);
		qry.setParameter("idCentroColaboradores", idCentroColaboradores);
		
		@SuppressWarnings("unchecked")
		List<Centro> centros = (List<Centro>)qry.getResultList();
		logger.info("ReportHelperGlobal:getAverageRatingOfQuestionsByCenterList:lista de centros de la red: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		
		// Crea la estructura de respuesta.
		List<AverageRatingOfQuestionsByCenter> larqc = new ArrayList<AverageRatingOfQuestionsByCenter>();
		
		// Se realizan las consultas por cada centro y se rellena la estructura de respuesta.
		for (Centro c : centros) {
			// Extrae la valoración media del período del centro
			strqry = "SELECT COALESCE((CASE pcu.tipo " +
											"WHEN 'ENTERO' THEN (ROUND(AVG(rcu.valorEntero), 2) * 1.0F) " +
											"WHEN 'DECIMAL' THEN (ROUND(AVG(rcu.valorDecimal), 2) * 1.0F) " +
									  "END), -1.0F) AS valorMedioPeriodo " +
					 "FROM " + MatriculaCurso.class.getName() + " mcu INNER JOIN " +
					 	  "mcu.curso cu WITH (((cu.fechaInicio BETWEEN :fechaInicio AND :fechaFin) OR " +
					 			  			  "(cu.fechaFin BETWEEN :fechaInicio AND :fechaFin)) OR " +
					 			  			 "((:fechaInicio BETWEEN cu.fechaInicio AND cu.fechaFin) AND " +
					 			  			  "(:fechaFin BETWEEN cu.fechaInicio AND cu.fechaFin))) INNER JOIN " +
						  "mcu.formularioConcreto.respuestas rcu INNER JOIN " +
						  "rcu.pregunta pcu WITH ((pcu.tipo = 'ENTERO') OR " +
												 "(pcu.tipo = 'DECIMAL')) " +
					 "WHERE (cu.desactivado = FALSE) AND " +
					 	 //"(cu.servicio.desactivado = FALSE) AND " +
					 	   "(cu.centro.id = :idCentro)";
			
			qry = service.getEntityManager().createQuery(strqry);

			qry.setParameter("idCentro", c.getId());
			qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
			qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);

			startTime = System.nanoTime();
			Float valorMedioPeriodo = (Float)qry.getSingleResult();
			logger.info("ReportHelperGlobal:getAverageRatingOfQuestionsByCenterList:valoración media del período del centro: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);

			// Extrae la valoración media histórica del centro
			strqry = "SELECT COALESCE((CASE pcu.tipo " +
											"WHEN 'ENTERO' THEN (ROUND(AVG(rcu.valorEntero), 2) * 1.0F) " +
											"WHEN 'DECIMAL' THEN (ROUND(AVG(rcu.valorDecimal), 2) * 1.0F) " +
									  "END), -1.0F) AS valorMedioHistorico " +
					 "FROM " + MatriculaCurso.class.getName() + " mcu INNER JOIN " +
						  "mcu.formularioConcreto.respuestas rcu INNER JOIN " +
						  "rcu.pregunta pcu WITH ((pcu.tipo = 'ENTERO') OR " +
												 "(pcu.tipo = 'DECIMAL')) " +
					 "WHERE (mcu.curso.desactivado = FALSE) AND " +
					 	 //"(mcu.curso.servicio.desactivado = false) AND " +
					 	   "(mcu.curso.centro.id = :idCentro)";
			
			qry = service.getEntityManager().createQuery(strqry);

			qry.setParameter("idCentro", c.getId());
			
			startTime = System.nanoTime();
			Float valorMedioHistorico = (Float)qry.getSingleResult();
			logger.info("ReportHelperGlobal:getAverageRatingOfQuestionsByCenterList:valoración media histórica del centro: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			
			// Se almacenan los resultados en la estructura de respuesta
			AverageRatingOfQuestionsByCenter arqc = new AverageRatingOfQuestionsByCenter(
					c.getProvincia(),	 // Provincia
					c,					 // Centro
					valorMedioPeriodo,	 // Valor medio del periodo
					valorMedioHistorico, // Valor medio histórico
					Long.valueOf(-1L),	 // Ranking provincial del período
					Long.valueOf(-1L));	 // Ranking global histórico

			larqc.add(arqc);
		}
		
		// Cálculo de rankings
		Long ranking;
		Float historicalAverageRating;

		// Ordena por valor medio historica de menor a mayor
		Collections.sort(larqc, AverageRatingOfQuestionsByCenter.comparatorHistoricalAverageRatingDESC);

		// Asigna el ranking global histórico;
		ranking = 0L;
		historicalAverageRating = Float.MAX_VALUE;
		for (AverageRatingOfQuestionsByCenter arqc : larqc) {
			if (arqc.getHistoricalAverageRating() > -1.0F) {
				if (historicalAverageRating > arqc.getHistoricalAverageRating()) {
					historicalAverageRating = arqc.getHistoricalAverageRating();
					ranking++;
				}

				arqc.setRankingHistoricalAverageRatingGlobal(ranking);
			}
			else
				arqc.setRankingHistoricalAverageRatingGlobal(-1L);
		}

		// Ordena por valor medio del período dentro de cada provincia
		Collections.sort(larqc, AverageRatingOfQuestionsByCenter.comparatorProvinciaASCPeriodAverageRatingDESC);
		
		// Asigna el ranking provincial del periodo
		Provincia p = null;
		ranking = 0L;
		historicalAverageRating = Float.MAX_VALUE;
		for (AverageRatingOfQuestionsByCenter arqc : larqc) {
			if (p != arqc.getProvincia()) {
				p = arqc.getProvincia();
				ranking = 0L;
				historicalAverageRating = Float.MAX_VALUE;
			}

			if (arqc.getPeriodAverageRating() > -1.0F) {
				if (historicalAverageRating > arqc.getPeriodAverageRating()) {
					historicalAverageRating = arqc.getPeriodAverageRating();
					ranking++;
				}

				arqc.setRankingPeriodAverageRatingProvincial(ranking);
			}
			else
				arqc.setRankingPeriodAverageRatingProvincial(-1L);
		}

		// Ordena por el nombre de la provincia y el nombre del centro
		Collections.sort(larqc, AverageRatingOfQuestionsByCenter.comparatorProvinciaASCCentroASC);

		return larqc;
	}

	private List<AverageRatingOfQuestionsByCollaborativeEntity> getAverageRatingOfQuestionsByCollaborativeEntityList(Date fechaInicio, Date fechaFin) {
		String strqry = null;
		Query qry = null;
		
		// Extrae la lista de entidades colaboradoras de la red.
		startTime = System.nanoTime();
		List<Proveedor> proveedores = (List<Proveedor>)service.findAll(Proveedor.class);
		logger.info("ReportHelperGlobal:getAverageRatingOfQuestionsByCollaborativeEntityList:lista de entidades colaboradoras de la red: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
		
		// Crea la estructura de respuesta.
		List<AverageRatingOfQuestionsByCollaborativeEntity> larqce = new ArrayList<AverageRatingOfQuestionsByCollaborativeEntity>();
		
		// Se realizan las consultas por cada centro y se rellena la estructura de respuesta.
		for (Proveedor p : proveedores) {
			
			// Extrae la valoración media histórica de la entidad colaboradora
			strqry = "SELECT COALESCE((CASE pae.tipo " +
					                       "WHEN 'ENTERO' THEN (ROUND(AVG(rae.valorEntero), 2) * 1.0F) " +
					                       "WHEN 'DECIMAL' THEN (ROUND(AVG(rae.valorDecimal), 2) * 1.0F) " +
					                  "END), -1.0F) AS valorMedioHistorico " +
					 "FROM " + ActividadExterna.class.getName() + " ae INNER JOIN " +
					      "ae.colaborador co INNER JOIN " +
					      "ae.formulariosConcretos fae INNER JOIN " +
					      "fae.respuestas rae INNER JOIN " +
					      "rae.pregunta pae WITH ((pae.tipo = 'ENTERO') OR " +
					                             "(pae.tipo = 'DECIMAL')) " +
					 "WHERE (ae.desactivado = false) AND " +
					 	   "(co.proveedor.id = :idEntidadColaboradora)";
			
			qry = service.getEntityManager().createQuery(strqry);

			qry.setParameter("idEntidadColaboradora", p.getId());
			
			startTime = System.nanoTime();
			Float valorMedioHistorico = (Float)qry.getSingleResult();
			logger.info("ReportHelperGlobal:getAverageRatingOfQuestionsByCollaborativeEntityList:valoración media histórica de la entidad colaboradora: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
			
			//Si existe algún dato histórico del proveedor
			if (valorMedioHistorico > -1.0F) {
				// Extrae la valoración media del período de la entidad colaboradora
				strqry = "SELECT COALESCE((CASE pae.tipo " +
						                       "WHEN 'ENTERO' THEN (ROUND(AVG(rae.valorEntero), 2) * 1.0F) " +
						                       "WHEN 'DECIMAL' THEN (ROUND(AVG(rae.valorDecimal), 2) * 1.0F) " +
						                  "END), -1.0F) AS valorMedioPeriodo " +
						 "FROM " + ActividadExterna.class.getName() + " ae INNER JOIN " +
						      "ae.colaborador co INNER JOIN " +
						      "ae.formulariosConcretos fae INNER JOIN " +
						      "fae.respuestas rae INNER JOIN " +
						      "rae.pregunta pae WITH ((pae.tipo = 'ENTERO') OR " +
						                             "(pae.tipo = 'DECIMAL')) " +
						 "WHERE (co.proveedor.id = :idEntidadColaboradora) AND " +
						 	   "(ae.desactivado = false) AND " +
						       "(((ae.fechaInicio BETWEEN :fechaInicio AND :fechaFin) OR " +
						         "(ae.fechaFin BETWEEN :fechaInicio AND :fechaFin)) OR " +
						 		"((:fechaInicio BETWEEN ae.fechaInicio AND ae.fechaFin) AND " +
						 		 "(:fechaFin BETWEEN ae.fechaInicio AND ae.fechaFin)))";
				
				qry = service.getEntityManager().createQuery(strqry);

				qry.setParameter("idEntidadColaboradora", p.getId());
				qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
				qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);

				startTime = System.nanoTime();
				Float valorMedioPeriodo = (Float)qry.getSingleResult();
				logger.info("ReportHelperGlobal:getAverageRatingOfQuestionsByCollaborativeEntityList:valoración media del período de la entidad colaboradora: {} segundos\n", (System.nanoTime() - startTime) / 1000000000.0);
				
				// Se almacenan los resultados en la estructura de respuesta
				AverageRatingOfQuestionsByCollaborativeEntity arqce = new AverageRatingOfQuestionsByCollaborativeEntity(
						p,					 // Entidad colaboradora
						valorMedioPeriodo,	 // Valor medio del periodo
						valorMedioHistorico, // Valor medio histórico
						Long.valueOf(-1L));	 // Ranking global histórico
	
				larqce.add(arqce);
			}
		}

		// Cálculo de rankings
		Long ranking;
		Float historicalAverageRating;

		// Ordena por valor medio historica de menor a mayor
		Collections.sort(larqce, AverageRatingOfQuestionsByCollaborativeEntity.comparatorHistoricalAverageRatingDESC);

		// Asigna el ranking global histórico;
		ranking = 0L;
		historicalAverageRating = Float.MAX_VALUE;
		for (AverageRatingOfQuestionsByCollaborativeEntity arqce : larqce) {
			if (arqce.getHistoricalAverageRating() > -1.0F) {
				if (historicalAverageRating > arqce.getHistoricalAverageRating()) {
					historicalAverageRating = arqce.getHistoricalAverageRating();
					ranking++;
				}

				arqce.setRankingHistoricalAverageRatingGlobal(ranking);
			}
			else
				arqce.setRankingHistoricalAverageRatingGlobal(-1L);
		}

		// Ordena por el nombre de la entidad colaboradora
		Collections.sort(larqce, AverageRatingOfQuestionsByCollaborativeEntity.comparatorCollaborativeEntityASC);

		return larqce;
	}

	private Object getQuerySingleResult(String strqry, Date fechaInicio, Date fechaFin) {
		Query qry = service.getEntityManager().createQuery(strqry);

		qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
		qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);

		return qry.getSingleResult();
	}

	private Long getSizeOfQueryResultList(String strqry, Date fechaInicio, Date fechaFin) {
		Query qry = service.getEntityManager().createQuery(strqry);

		qry.setParameter("fechaInicio", fechaInicio, TemporalType.TIMESTAMP);
		qry.setParameter("fechaFin", fechaFin, TemporalType.TIMESTAMP);

		return new Long((long)qry.getResultList().size());
	}
}
