package es.csa.fdt.cyldigital.security;

import java.io.Serializable;
import java.net.UnknownHostException;
import java.util.EnumSet;

/*
 * #%L
 * Librerías del núcleo para Web - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.HashMap;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;

import jcifs.UniAddress;
import jcifs.smb.NtlmPasswordAuthentication;
import jcifs.smb.SmbException;
import jcifs.smb.SmbSession;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.support.MessageSourceAccessor;
import org.springframework.dao.DataAccessException;
import org.springframework.ldap.AuthenticationException;
import org.springframework.ldap.NamingException;
import org.springframework.ldap.core.DistinguishedName;
import org.springframework.ldap.core.LdapTemplate;
import org.springframework.ldap.filter.EqualsFilter;
import org.springframework.ldap.filter.OrFilter;
import org.springframework.security.Authentication;
import org.springframework.security.AuthenticationServiceException;
import org.springframework.security.BadCredentialsException;
import org.springframework.security.SpringSecurityMessageSource;
import org.springframework.security.providers.UsernamePasswordAuthenticationToken;
import org.springframework.security.providers.dao.AbstractUserDetailsAuthenticationProvider;
import org.springframework.security.userdetails.UserDetails;
import org.springframework.security.userdetails.UserDetailsService;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import org.springframework.web.servlet.theme.SessionThemeResolver;
import org.springframework.context.ApplicationContext;

import com.google.common.cache.Cache;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.core.service.AbstractServiceImpl;
import es.csa.fdt.cyldigital.core.service.ParameterServiceImpl;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado.Puesto;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario.Role;
import es.csa.fdt.cyldigital.usuarios.service.UsuarioLdapService;
import es.csa.fdt.cyldigital.usuarios.service.UsuarioServiceImpl;
import es.csa.fdt.cyldigital.usuarios.service.UsuarioService;

/**
 * Clase CyLDigitalLdapAuthenticationProvider.
 */
public class CyLDigitalLdapAuthenticationProvider extends AbstractUserDetailsAuthenticationProvider {

private static final Logger logger = Logger.getLogger(CyLDigitalLdapAuthenticationProvider.class.getName());

  @Autowired
  private UsuarioLdapService service;

  @Autowired
  private transient HashMap<String, Object> generalProperties;

  private LdapTemplate ldapTemplate;

  private String hostLdap;
  
  private Integer portLdapSmb;

  private UserDetailsService userDetailsService;
  
  @Autowired
  private ApplicationContext appContext;
  
  private UsuarioServiceImpl usuarioServiceImpl = new UsuarioServiceImpl();

  protected MessageSourceAccessor messages = SpringSecurityMessageSource.getAccessor();

  /**
   * Construye un nuevo objeto de la clase
   */
  public CyLDigitalLdapAuthenticationProvider() {
    super();
  }

  /**
   * Devuelve user details service.
   * 
   * @return the user details service
   */
  public UserDetailsService getUserDetailsService() {
    return userDetailsService;
  }

  /**
   * Asigna user details service.
   * 
   * @param userDetailsService
   *           new user details service
   */
  public void setUserDetailsService(UserDetailsService userDetailsService) {
    this.userDetailsService = userDetailsService;
  }

  /**
   * Devuelve ldap template.
   * 
   * @return the ldap template
   */
  public LdapTemplate getLdapTemplate() {
    return ldapTemplate;
  }

  /**
   * Asigna ldap template.
   * 
   * @param ldapTemplate
   *           new ldap template
   */
  public void setLdapTemplate(LdapTemplate ldapTemplate) {
    this.ldapTemplate = ldapTemplate;
  }

  /**
   * Devuelve host ldap.
   * 
   * @return the host ldap
   */
  public String getHostLdap() {
    return hostLdap;
  }

  /**
   * Asigna host ldap.
   * 
   * @param hostLdap
   *           new host ldap
   */
  public void setHostLdap(String hostLdap) {
    this.hostLdap = hostLdap;
  }

  /**
   * @return the portLdapSmb
   */
  public Integer getPortLdapSmb() {
	  return portLdapSmb;
  }

  /**
   * @param portLdapSmb the portLdapSmb to set
   */
  public void setPortLdapSmb(Integer portLdapSmb) {
	  this.portLdapSmb = portLdapSmb;
  }

/* (non-Javadoc)
   * @see org.springframework.security.providers.dao.AbstractUserDetailsAuthenticationProvider#authenticate(org.springframework.security.Authentication)
   */
  @Override
  @Transactional
  public Authentication authenticate(Authentication authentication) throws AuthenticationException {
	  boolean csrfToken = false;
	  
	  ServletRequestAttributes attr = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
	  HttpServletRequest httpReq = attr.getRequest();
	  
	  // Get the token sent with the request
      String token = (String) httpReq.getParameter("_csrfToken");

      // Validate that the token is in the cache
      @SuppressWarnings("unchecked")
	  Cache<String, Boolean> csrfPreventionSaltCache = (Cache<String, Boolean>)
          httpReq.getSession().getAttribute("_csrfSecurityCache");
      
      if (csrfPreventionSaltCache != null &&
    		  token != null &&
              csrfPreventionSaltCache.getIfPresent(token) != null &&
              csrfPreventionSaltCache.getIfPresent(token)) {
    	  			System.out.println("Token de validación CSRF encontrado.");
    	  			csrfToken = true;
      } else {
    	  csrfToken = false;
    	  String path = httpReq.getServletPath();
    	  if(path.contains("integracion"))
    		  csrfToken = true;
      }
      
      if(csrfToken) {
	    Assert.isInstanceOf(UsernamePasswordAuthenticationToken.class, authentication, messages.getMessage(
	        "AbstractUserDetailsAuthenticationProvider.onlySupports",
	        "Only UsernamePasswordAuthenticationToken is supported"));
	
	    UsernamePasswordAuthenticationToken userToken = (UsernamePasswordAuthenticationToken) authentication;
	
	    String username = userToken.getName();
	
	    if (!StringUtils.hasLength(username)) { throw new BadCredentialsException(messages.getMessage(
	        "LdapAuthenticationProvider.emptyUsername", "Empty Username")); }
	
	    String password = (String) authentication.getCredentials();
	
	    String nombreLdap = getNombreLdap(username);
	
	    String nombreApplicacion = getNombreApplicacion(username);
	
	    try {
	      if (nombreLdap.contains("@") && !nombreLdap.contains((String) generalProperties.get("LDAP_EMAIL_SUFFIX"))) {
	        nombreLdap += "." + (String) generalProperties.get("LDAP_EMAIL_SUFFIX");
	        logger.log(Level.FINE, "El usuario a validar no contenia el sufijo completo. Ahora: " + nombreLdap);
	      }
	
	      UserDetails user = userDetailsService.loadUserByUsername(nombreApplicacion);
	      
	      if ((user instanceof UsuarioLdap) && (((UsuarioLdap) user).getUser() == null)) {
	    	  throw new BadCredentialsException(messages.getMessage("LdapAuthenticationProvider.notFound", "Usuario no encontrado")); }
	      
	      if ((user instanceof UsuarioLdap) && ((UsuarioLdap) user).getUser() != null
	          && ((UsuarioLdap) user).getUser().getDesactivado()) { throw new BadCredentialsException(messages.getMessage(
	          "LdapAuthenticationProvider.notFound", 
            "El usuario se encuentra desactivado. "
            + "Póngase en contacto con el administrador del sitio para que sea reactivado." )); }
	
	      boolean authenticated = false;
	      EqualsFilter userPrincipalNameFilter = new EqualsFilter("userPrincipalName", nombreLdap);
	      EqualsFilter usernameFilter = new EqualsFilter("uid", nombreLdap);
	      EqualsFilter userSamFilter = new EqualsFilter("sAMAccountName", nombreLdap);
		      
	      // JCIFS TEST	      
	      //SAMBA
	      
	      try {
	        UniAddress uniaddress = UniAddress.getByName(getHostLdap());
	        Integer porLdapSmbAuthentication = getPortLdapSmb();
	        
	        String domain = (user != null && (((UsuarioLdap) user).getCentro()) != null) ? ((UsuarioLdap) user).getCentro().getDominio() : "";
	        logger.log(Level.INFO, "Validando: " + username + " en " + getHostLdap() + " por el puerto LdapSmb " + porLdapSmbAuthentication + " con dominio " + domain);
	        
	        NtlmPasswordAuthentication ntlmpasswordauthentication = new NtlmPasswordAuthentication(domain, username, password);
	        if (porLdapSmbAuthentication != null)
	        	SmbSession.logon(uniaddress, porLdapSmbAuthentication, ntlmpasswordauthentication);
	        else
	        	SmbSession.logon(uniaddress, ntlmpasswordauthentication);
	        
	        // Si llega hasta aqui es correcto
	        logger.log(Level.INFO, "Validado correcto: " + username + " en " + getHostLdap() + " por el puerto LdapSmb " + porLdapSmbAuthentication + " con dominio " + domain);
	        authenticated = true;
	        
	        // 3.4.8 - RF_15_37 Usuarios activos
	        if ( ((UsuarioLdap)user).getUser() instanceof UsuarioCyL) {
	          UsuarioCyL ucyl = (UsuarioCyL)  ((UsuarioLdap)user).getUser();
	          usuarioServiceImpl.persistUltimoAcceso(ucyl);
	        }
	      } catch (SmbException e) {
	          logger.log(Level.INFO, "La validación del usuario ha fallado");
	          e.printStackTrace();
	      } catch (UnknownHostException e) {
	        logger.log(Level.SEVERE, "El host ldap no se ecuentra :" + getHostLdap());
	        e.printStackTrace();
	      }
	      
	      //LDAP
	      /*
	      if (!authenticated) {
	    	  if (((UsuarioLdap) user).getCentro() != null) { 
	    		DistinguishedName userDN = null;
	    		
	    		try {
		    		if ( ((UsuarioLdap) user).getUser() instanceof UsuarioCyL) 
		    			userDN = userToDistinguishedName(username, ((UsuarioLdap) user).getCentro(), service);
		    		else if(((UsuarioLdap) user).getUser() instanceof Empleado)
		    			userDN = empleadoToDistinguishedName(username, ((UsuarioLdap) user).getCentro(), service);
		    		
		    		if(userDN != null) {
		    			logger.log(Level.INFO, "Validando: " + username + " en " + getHostLdap() + " con dominio " + ((UsuarioLdap) user).getCentro().getDominio());
		    			authenticated = ldapTemplate.authenticate(userDN, usernameFilter.encode(), password);
		    			//Si llega hasta aquí es que se ha logueado correctamente
		    			logger.log(Level.INFO, "Validado correcto: " + username + " en " + getHostLdap() + " con dominio " + ((UsuarioLdap) user).getCentro().getDominio());
		    			
		    			// 3.4.8 - RF_15_37 Usuarios activos
		    			if ( ((UsuarioLdap)user).getUser() instanceof UsuarioCyL && authenticated) {
		    				UsuarioCyL ucyl = (UsuarioCyL)  ((UsuarioLdap)user).getUser();
				    		usuarioServiceImpl.persistUltimoAcceso(ucyl);
				    	}
		    		}
	    		} catch (AuthenticationException ae) {
	    			logger.log(Level.SEVERE, "authenticate - Usuario o contraseña incorrecta.");
	  			  	ae.printStackTrace();
	  		  	} catch (Exception e) {
	  		  		logger.log(Level.SEVERE, "authenticate - Usuario o contraseña incorrecta.");
	  		  		e.printStackTrace();
	  		  	}
	    	  }
	      }
	      */
	      if (!authenticated) {
	    	  if ( ((UsuarioLdap)user).getUser() instanceof UsuarioCyL){
	    		  UsuarioCyL ucyl = (UsuarioCyL)  ((UsuarioLdap)user).getUser();
	    		  if(ucyl.isDesactivadoLDAP()){
	    			  throw new BadCredentialsException(messages.getMessage("LdapAuthenticationProvider.notFound", 
	    					  	"Usuario desactivado por inactividad. "
	    					  + "\nPuede reactivar el usuario a traves en el enlace superior de recuperación de contraseña. "
	    					  + "\nEmpleando la dirección de correo asociada a su usuario. "
	    					  + "\nEn caso que no tuviera registrada una dirección de correo deberá ponerse en contacto con la Oficina CeMIT")); 
	    		  }
	    	  }
	    	  throw new BadCredentialsException(messages.getMessage("LdapAuthenticationProvider.notFound", 
	    			  "Usuario o contraseña incorrecta")); 
	      }
	      
	      //Update Ldap logonTime
	      if( ((UsuarioLdap)user).getUser() instanceof UsuarioCyL && authenticated) {
	    	  	EntityManagerFactory emf = (EntityManagerFactory)appContext.getBean("entityManagerFactory", EntityManagerFactory.class);
	    	  	EntityManager em = emf.createEntityManager();
	    	  	
	    	  	ParameterServiceImpl parameterService = new ParameterServiceImpl();
				parameterService.setEntityManager(em);
				if (parameterService.findOneWhere("nombre", "users_change_ldap_logontime").getValor().equals("1")){
					UsuarioService usuarioService = (UsuarioService) appContext.getBean("usuarioServiceImpl");
					UsuarioCyL ucyl = (UsuarioCyL)  ((UsuarioLdap)user).getUser();
					usuarioService.updateLdapLogonTime(ucyl);
				}
	      }
	      
	
	      return new UsernamePasswordAuthenticationToken(user, userToken.getCredentials(), user.getAuthorities());
	
	    } catch (NamingException ldapAccessFailure) {
	      throw new AuthenticationServiceException(ldapAccessFailure.getMessage(), ldapAccessFailure);
	    }
      } else {
      	// Otherwise we throw an exception aborting the request flow
      	  throw new BadCredentialsException(messages.getMessage("LdapAuthenticationProvider.notFound", "Potential CSRF detected!! Inform a scary sysadmin ASAP.")); 
        }
  }

  private String getNombreApplicacion(String username) {
    String result = username;
    if (username.contains("\\")) {
      result = username.substring(username.indexOf("\\") + 1, username.length());
    }
    if (username.contains("/")) {
      result = username.substring(username.indexOf("/") + 1, username.length());
    }
    if (username.contains("@")) {
      result = username.substring(0, username.indexOf("@"));
    }
    return result;
  }

  private String getNombreLdap(String username) {
    String result = username;
    if (username.contains("\\")) {
      result = username.substring(username.indexOf("\\") + 1, username.length()) + "@"
          + username.substring(0, username.indexOf("\\"));
    }
    if (username.contains("/")) {
      result = username.substring(username.indexOf("/") + 1, username.length()) + "@"
          + username.substring(0, username.indexOf("/"));
    }
    return result;
  }

  /* (non-Javadoc)
   * @see org.springframework.security.providers.dao.AbstractUserDetailsAuthenticationProvider#supports(java.lang.Class)
   */
  @Override
  public boolean supports(Class authentication) {
    return (UsernamePasswordAuthenticationToken.class.isAssignableFrom(authentication));
  }

  /* (non-Javadoc)
   * @see org.springframework.security.providers.dao.AbstractUserDetailsAuthenticationProvider#additionalAuthenticationChecks(org.springframework.security.userdetails.UserDetails, org.springframework.security.providers.UsernamePasswordAuthenticationToken)
   */
  @Override
  protected void additionalAuthenticationChecks(UserDetails userDetails,
      UsernamePasswordAuthenticationToken authentication) throws org.springframework.security.AuthenticationException {
  // TODO Auto-generated method stub

  }

  /* (non-Javadoc)
   * @see org.springframework.security.providers.dao.AbstractUserDetailsAuthenticationProvider#retrieveUser(java.lang.String, org.springframework.security.providers.UsernamePasswordAuthenticationToken)
   */
  @Override
  protected UserDetails retrieveUser(String username, UsernamePasswordAuthenticationToken authentication)
      throws org.springframework.security.AuthenticationException {

    System.out.println("Devolviendo usuario");
    UsuarioLdap user = new UsuarioLdap();
    user.setUsername(username);
    try {
      attachUserInformation(user);
      if (user.getUser() != null) {
        user.setCentro(user.getUser().getCentro());
        if (user.getUser().isEmpleado()) {
          Puesto puesto = ((Empleado) user.getUser()).getPuesto();
          if (puesto.equals(Puesto.ADMIN)) addRole(user, Role.ROLE_ADMIN);
          //if (puesto.equals(Puesto.TRAINER_COORDINATOR)) addRole(user, Role.ROLE_TRAINER_COORDINATOR);
          //if (puesto.equals(Puesto.MANAGER)) addRole(user, Role.ROLE_MANAGER);
          if (puesto.equals(Puesto.AREA_MANAGER)) addRole(user, Role.ROLE_AREA_MANAGER);
          if (puesto.equals(Puesto.TRAINER)) addRole(user, Role.ROLE_TRAINER);
          //if (puesto.equals(Puesto.INFORMER)) addRole(user, Role.ROLE_INFORMER);
          //if (puesto.equals(Puesto.GESTOR_012)) addRole(user, Role.ROLE_012);
          if (puesto.equals(Puesto.EXTERNAL_PERSONAL)) addRole(user, Role.ROLE_EXTERN);
          //if (puesto.equals(Puesto.FINANCIAL_MANAGER)) addRole(user, Role.ROLE_FINANCIAL_MANAGER);
          if (puesto.equals(Puesto.DIGITAL_ALLY_MANAGER)) addRole(user, Role.ROLE_DIGITAL_ALLY);
        } else {
          addRole(user, Role.ROLE_USER);
        }
      }
    } catch (DataAccessException repositoryProblem) {
      throw new AuthenticationServiceException(repositoryProblem.getMessage(), repositoryProblem);
    }
    if (user == null) throw new AuthenticationServiceException("User cannot be null");
    return user;
  }

  private void addRole(UsuarioLdap user, Role role) {
    if (!user.getRolesList().contains(role)) user.getRolesList().add(role);
  }

  private void attachUserInformation(UsuarioLdap user) {
    try {
      List<Empleado> e = service.findWhere(Empleado.class, "username", user.getUsername());
      if (e != null && !e.isEmpty() && e.size() == 1) {
        user.setUser(e.get(0));
      } else {
        List<UsuarioCyL> u = service.findWhere(UsuarioCyL.class, "username", user.getUsername());
        if (u != null && !u.isEmpty() && u.size() == 1) {
          user.setUser(u.get(0));
        }
      }
    } catch (RuntimeException e) {
      logger.log(Level.SEVERE, "Error al añadir la información del usuario: " + e.getLocalizedMessage());
      e.printStackTrace();
    }
  }
  
  private DistinguishedName userToDistinguishedName(String userName, Centro c, UsuarioLdapService service) {
	  DistinguishedName dn = new DistinguishedName();
	  String userSuffix = service.findOneWhere(Parameter.class, "nombre", "openldap_user_suffix").getValor();
	    
	  if (generalProperties.containsKey("LDAP_USER_GROUPS")) {
		  String[] user_groups = ((String) generalProperties.get("LDAP_USER_GROUPS")).split(",");
		  // Concatena la Unidad Organizativa destinada a los usuarios
	      if (userSuffix !=""){
	    	  if (userSuffix.toLowerCase().startsWith("ou=")){
	    		  dn.add("ou",userSuffix.substring(3));
	    	  }else{
	    		  dn.add("ou", userSuffix);
	    	  }
	      }
	      
	      // Concatena la Unidad Organizativa correspondiente al centro
	      if (c != null && c.getDominio() != null && !c.getDominio().isEmpty())
	          dn.add("ou", c.getDominio().replaceAll(" ", "").toUpperCase().replace('Ñ', 'N').replace('Á', 'A').replace('É', 'E').replace('Í', 'I').replace('Ó','O').replace('Ú', 'U'));

	      // Concatena la Unidad Organizativa definida en el POM para el almacenamiento de usuarios
	      for (String grupo : user_groups) {
	        dn.add("ou", grupo);
	      }
	    }
	  // Concatena el uid del usuario
	  dn.add("uid", userName);
	  return dn;
  }
  
  private DistinguishedName empleadoToDistinguishedName(String userName, Centro c, UsuarioLdapService service) {
	  DistinguishedName dn = new DistinguishedName();
	    String userSuffix = service.findOneWhere(Parameter.class, "nombre", "openldap_user_suffix").getValor();
	    
	    if (generalProperties.containsKey("LDAP_PERSONAL_GROUPS")) {
	      String[] user_groups = ((String) generalProperties.get("LDAP_PERSONAL_GROUPS")).split(",");

	      // Concatena la Unidad Organizativa destinada a los usuarios
	      if (userSuffix !=""){
	    	  if (userSuffix.toLowerCase().startsWith("ou=")){
	    		  dn.add("ou",userSuffix.substring(3));
	    	  }else{
	    		  dn.add("ou", userSuffix);
	    	  }
	      }

	      // Concatena la Unidad Organizativa correspondiente al centro
	      if (c != null && c.getDominio() != null && !c.getDominio().isEmpty())
	        dn.add("ou", c.getDominio().replaceAll(" ", "").toUpperCase());

	      // Concatena la Unidad Organizativa definida en el POM para el almacenamiento de empleador
	      for (String grupo : user_groups) {
	        dn.add("ou", grupo);
	      }
	    }
	    // Concatena el uid del empleados
	    dn.add("uid", userName);
	    return dn;
  }
}
