package es.csa.fdt.cyldigital.security;

/*
 * #%L
 * Librerías del núcleo para Web - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2015  Xunta de Galicia <oficinacemit@xunta.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Balidea <info@balidea.com>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */

import java.io.IOException;
import java.security.SecureRandom;
import java.util.concurrent.TimeUnit;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.RandomStringUtils;

import com.google.common.cache.Cache;
import com.google.common.cache.CacheBuilder;


public class CyLDigitalSecurityFilter implements Filter {

	@Override
	public void destroy() {
	}

	@Override
	public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain)
			throws IOException, ServletException {
		
		//RF_15_48 - Habilitar marcas HttpOnly e Secure en cookies
		HttpServletRequest req = (HttpServletRequest) request; // For GET 
		HttpServletResponse res = (HttpServletResponse) response; // For GET 
		
		Cookie[] cookies = req.getCookies();
		
		if (cookies != null){
			for (Cookie ck : cookies) {
				res.addHeader("Set-Cookie", ck.getName() + "=" +  ck.getValue() + "; Secure; HttpOnly");
			}
		}		
		
		//RF_15_48 - Formularios necesitan protección CSRF
		@SuppressWarnings("unchecked")
		Cache<String, Boolean> csrfSecurityCache = (Cache<String, Boolean>) req.getSession().getAttribute("_csrfSecurityCache");
		
		if (csrfSecurityCache == null){
			
			csrfSecurityCache = CacheBuilder.newBuilder().maximumSize(5000).expireAfterWrite(20, TimeUnit.MINUTES).build();
            req.getSession().setAttribute("_csrfSecurityCache", csrfSecurityCache);
        }
		
		// Generate the token and store it in the users cache
        String salt = RandomStringUtils.random(20, 0, 0, true, true, null, new SecureRandom());
        csrfSecurityCache.put(salt, Boolean.TRUE);

        // Add the token to the current request so it can be used
        // by the page rendered in this request
        req.setAttribute("_csrfToken", salt);
        
		chain.doFilter(request, response);
	}

	@Override
	public void init(FilterConfig arg0) throws ServletException {
		
	}

}
