package es.csa.fdt.cyldigital.security;

/*
 * #%L
 * Librerías del núcleo para Web - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.security.AuthenticationServiceException;
import org.springframework.security.userdetails.UserDetails;
import org.springframework.security.userdetails.UserDetailsService;
import org.springframework.security.userdetails.UsernameNotFoundException;
import org.springframework.transaction.annotation.Transactional;

import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario.Role;
import es.csa.fdt.cyldigital.usuarios.service.UsuarioLdapService;

/**
 * Clase CyLDigitalUserDetailsServiceImpl.
 */
public class CyLDigitalUserDetailsServiceImpl implements UserDetailsService {

  private static final Logger logger = Logger.getLogger(CyLDigitalUserDetailsServiceImpl.class.getName());

  @Autowired
  private UsuarioLdapService service;

  /* (non-Javadoc)
   * @see org.springframework.security.userdetails.UserDetailsService#loadUserByUsername(java.lang.String)
   */
  @Transactional(readOnly = true)
  public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException, DataAccessException {
    System.out.println("Devolviendo usuario");
    UsuarioLdap user = new UsuarioLdap();
    user.setUsername(username);
    try {
      attachUserInformation(user);
      if (user.getUser() != null) {
        user.setCentro(user.getUser().getCentro());
      }
    } catch (DataAccessException repositoryProblem) {
      throw new AuthenticationServiceException(repositoryProblem.getMessage(), repositoryProblem);
    }
    if (user == null) throw new AuthenticationServiceException("User cannot be null");
    return user;
  }

  private void attachUserInformation(UsuarioLdap user) {
    try {
      List<Empleado> e = service.findWhere(Empleado.class, "username", user.getUsername());
      if (e != null && !e.isEmpty() && e.size() == 1) {
        user.setUser(e.get(0));
      } else {
        List<UsuarioCyL> u = service.findWhere(UsuarioCyL.class, "username", user.getUsername());
        if (u != null && !u.isEmpty() && u.size() == 1) {
          user.setUser(u.get(0));
        }
      }
    } catch (RuntimeException e) {
      logger.log(Level.SEVERE, "Error al añadir la información del usuario: " + e.getLocalizedMessage());
      e.printStackTrace();
    }
  }

  private void addRole(UsuarioLdap user, Role role) {
    if (!user.getRolesList().contains(role)) user.getRolesList().add(role);
  }

}
