package es.csa.fdt.cyldigital.security;

/*
 * #%L
 * Librerías del núcleo para Web - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.io.BufferedReader;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.security.Principal;
import java.util.Enumeration;
import java.util.Locale;
import java.util.Map;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletInputStream;
import javax.servlet.ServletRequest;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.springframework.security.util.PortResolver;
import org.springframework.security.wrapper.SecurityContextHolderAwareRequestWrapper;

/**
 * Modificaciones para que funcion NTLM con SecurityContextHolderAwareFilter.
 */
public class SavedRequestAwareWrapper extends SecurityContextHolderAwareRequestWrapper {

  protected org.springframework.security.wrapper.SavedRequestAwareWrapper delegate;

  /**
   * Construye un nuevo objeto de la clase
   * 
   * @param request
   *           request
   * @param portResolver
   *           port resolver
   * @param rolePrefix
   *           role prefix
   */
  public SavedRequestAwareWrapper(HttpServletRequest request, PortResolver portResolver, String rolePrefix) {
    super(request, portResolver, rolePrefix);
    this.delegate = new org.springframework.security.wrapper.SavedRequestAwareWrapper(request, portResolver, rolePrefix);
  }

  /* (non-Javadoc)
   * @see java.lang.Object#equals(java.lang.Object)
   */
  @Override
  public boolean equals(Object obj) {
    return delegate.equals(obj);
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequestWrapper#getAttribute(java.lang.String)
   */
  @Override
  public Object getAttribute(String name) {
    return delegate.getAttribute(name);
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequestWrapper#getAttributeNames()
   */
  @Override
  public Enumeration<?> getAttributeNames() {
    return delegate.getAttributeNames();
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequestWrapper#getAuthType()
   */
  @Override
  public String getAuthType() {
    return delegate.getAuthType();
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequestWrapper#getCharacterEncoding()
   */
  @Override
  public String getCharacterEncoding() {
    return delegate.getCharacterEncoding();
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequestWrapper#getContentLength()
   */
  @Override
  public int getContentLength() {
    return delegate.getContentLength();
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequestWrapper#getContentType()
   */
  @Override
  public String getContentType() {
    return delegate.getContentType();
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequestWrapper#getContextPath()
   */
  @Override
  public String getContextPath() {
    return delegate.getContextPath();
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequestWrapper#getCookies()
   */
  @Override
  public Cookie[] getCookies() {
    return delegate.getCookies();
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequestWrapper#getDateHeader(java.lang.String)
   */
  @Override
  public long getDateHeader(String name) {
    return delegate.getDateHeader(name);
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequestWrapper#getHeader(java.lang.String)
   */
  @Override
  public String getHeader(String name) {
    if (name.equalsIgnoreCase("Authorization")) {
      return super.getHeader(name);
    } else {
      return delegate.getHeader(name);
    }
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequestWrapper#getHeaderNames()
   */
  @Override
  public Enumeration<?> getHeaderNames() {
    return delegate.getHeaderNames();
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequestWrapper#getHeaders(java.lang.String)
   */
  @Override
  public Enumeration<?> getHeaders(String name) {
    return delegate.getHeaders(name);
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequestWrapper#getInputStream()
   */
  @Override
  public ServletInputStream getInputStream() throws IOException {
    return delegate.getInputStream();
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequestWrapper#getIntHeader(java.lang.String)
   */
  @Override
  public int getIntHeader(String name) {
    return delegate.getIntHeader(name);
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequestWrapper#getLocalAddr()
   */
  @Override
  public String getLocalAddr() {
    return delegate.getLocalAddr();
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequestWrapper#getLocale()
   */
  @Override
  public Locale getLocale() {
    return delegate.getLocale();
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequestWrapper#getLocales()
   */
  @Override
  public Enumeration<?> getLocales() {
    return delegate.getLocales();
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequestWrapper#getLocalName()
   */
  @Override
  public String getLocalName() {
    return delegate.getLocalName();
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequestWrapper#getLocalPort()
   */
  @Override
  public int getLocalPort() {
    return delegate.getLocalPort();
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequestWrapper#getMethod()
   */
  @Override
  public String getMethod() {
    return delegate.getMethod();
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequestWrapper#getParameter(java.lang.String)
   */
  @Override
  public String getParameter(String name) {
    return delegate.getParameter(name);
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequestWrapper#getParameterMap()
   */
  @Override
  public Map<?, ?> getParameterMap() {
    return delegate.getParameterMap();
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequestWrapper#getParameterNames()
   */
  @Override
  public Enumeration<?> getParameterNames() {
    return delegate.getParameterNames();
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequestWrapper#getParameterValues(java.lang.String)
   */
  @Override
  public String[] getParameterValues(String name) {
    return delegate.getParameterValues(name);
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequestWrapper#getPathInfo()
   */
  @Override
  public String getPathInfo() {
    return delegate.getPathInfo();
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequestWrapper#getPathTranslated()
   */
  @Override
  public String getPathTranslated() {
    return delegate.getPathTranslated();
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequestWrapper#getProtocol()
   */
  @Override
  public String getProtocol() {
    return delegate.getProtocol();
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequestWrapper#getQueryString()
   */
  @Override
  public String getQueryString() {
    return delegate.getQueryString();
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequestWrapper#getReader()
   */
  @Override
  public BufferedReader getReader() throws IOException {
    return delegate.getReader();
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequestWrapper#getRealPath(java.lang.String)
   */
  @Override
  public String getRealPath(String path) {
    return delegate.getRealPath(path);
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequestWrapper#getRemoteAddr()
   */
  @Override
  public String getRemoteAddr() {
    return delegate.getRemoteAddr();
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequestWrapper#getRemoteHost()
   */
  @Override
  public String getRemoteHost() {
    return delegate.getRemoteHost();
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequestWrapper#getRemotePort()
   */
  @Override
  public int getRemotePort() {
    return delegate.getRemotePort();
  }

  /* (non-Javadoc)
   * @see org.springframework.security.wrapper.SecurityContextHolderAwareRequestWrapper#getRemoteUser()
   */
  @Override
  public String getRemoteUser() {
    return delegate.getRemoteUser();
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequestWrapper#getRequest()
   */
  @Override
  public ServletRequest getRequest() {
    return delegate.getRequest();
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequestWrapper#getRequestDispatcher(java.lang.String)
   */
  @Override
  public RequestDispatcher getRequestDispatcher(String path) {
    return delegate.getRequestDispatcher(path);
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequestWrapper#getRequestedSessionId()
   */
  @Override
  public String getRequestedSessionId() {
    return delegate.getRequestedSessionId();
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequestWrapper#getRequestURI()
   */
  @Override
  public String getRequestURI() {
    return delegate.getRequestURI();
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequestWrapper#getRequestURL()
   */
  @Override
  public StringBuffer getRequestURL() {
    return delegate.getRequestURL();
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequestWrapper#getScheme()
   */
  @Override
  public String getScheme() {
    return delegate.getScheme();
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequestWrapper#getServerName()
   */
  @Override
  public String getServerName() {
    return delegate.getServerName();
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequestWrapper#getServerPort()
   */
  @Override
  public int getServerPort() {
    return delegate.getServerPort();
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequestWrapper#getServletPath()
   */
  @Override
  public String getServletPath() {
    return delegate.getServletPath();
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequestWrapper#getSession()
   */
  @Override
  public HttpSession getSession() {
    return delegate.getSession();
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequestWrapper#getSession(boolean)
   */
  @Override
  public HttpSession getSession(boolean create) {
    return delegate.getSession(create);
  }

  /* (non-Javadoc)
   * @see org.springframework.security.wrapper.SecurityContextHolderAwareRequestWrapper#getUserPrincipal()
   */
  @Override
  public Principal getUserPrincipal() {
    return delegate.getUserPrincipal();
  }

  /* (non-Javadoc)
   * @see java.lang.Object#hashCode()
   */
  @Override
  public int hashCode() {
    return delegate.hashCode();
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequestWrapper#isRequestedSessionIdFromCookie()
   */
  @Override
  public boolean isRequestedSessionIdFromCookie() {
    return delegate.isRequestedSessionIdFromCookie();
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequestWrapper#isRequestedSessionIdFromUrl()
   */
  @Override
  public boolean isRequestedSessionIdFromUrl() {
    return delegate.isRequestedSessionIdFromUrl();
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequestWrapper#isRequestedSessionIdFromURL()
   */
  @Override
  public boolean isRequestedSessionIdFromURL() {
    return delegate.isRequestedSessionIdFromURL();
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequestWrapper#isRequestedSessionIdValid()
   */
  @Override
  public boolean isRequestedSessionIdValid() {
    return delegate.isRequestedSessionIdValid();
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequestWrapper#isSecure()
   */
  @Override
  public boolean isSecure() {
    return delegate.isSecure();
  }

  /* (non-Javadoc)
   * @see org.springframework.security.wrapper.SecurityContextHolderAwareRequestWrapper#isUserInRole(java.lang.String)
   */
  @Override
  public boolean isUserInRole(String role) {
    return delegate.isUserInRole(role);
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequestWrapper#removeAttribute(java.lang.String)
   */
  @Override
  public void removeAttribute(String name) {
    delegate.removeAttribute(name);
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequestWrapper#setAttribute(java.lang.String, java.lang.Object)
   */
  @Override
  public void setAttribute(String name, Object o) {
    delegate.setAttribute(name, o);
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequestWrapper#setCharacterEncoding(java.lang.String)
   */
  @Override
  public void setCharacterEncoding(String enc) throws UnsupportedEncodingException {
    delegate.setCharacterEncoding(enc);
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequestWrapper#setRequest(javax.servlet.ServletRequest)
   */
  @Override
  public void setRequest(ServletRequest request) {
    delegate.setRequest(request);
  }

  /* (non-Javadoc)
   * @see java.lang.Object#toString()
   */
  @Override
  public String toString() {
    return delegate.toString();
  }

}
