package es.csa.fdt.cyldigital.security;

/*
 * #%L
 * Librerías del núcleo para Web - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.HashMap;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.persistence.NoResultException;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.ldap.AuthenticationException;
import org.springframework.security.AuthenticationServiceException;
import org.springframework.security.providers.UsernamePasswordAuthenticationToken;
import org.springframework.security.providers.dao.AbstractUserDetailsAuthenticationProvider;
import org.springframework.security.userdetails.UserDetails;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioLdap;
import es.csa.fdt.cyldigital.usuarios.entity.Empleado.Puesto;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario.Role;
import es.csa.fdt.cyldigital.usuarios.service.LdapService;
import es.csa.fdt.cyldigital.usuarios.service.UsuarioLdapService;

/**
 * Clase UserDetailsNTLMAuthenticationProvider.
 */
public class UserDetailsNTLMAuthenticationProvider extends AbstractUserDetailsAuthenticationProvider {

  private static final Logger logger = Logger.getLogger(UserDetailsNTLMAuthenticationProvider.class.getName());

  private LdapService ldapService;

  @Autowired
  private UsuarioLdapService service;

  @Autowired
  private transient HashMap<String, Object> generalProperties;

  /**
   * Devuelve ldap service.
   * 
   * @return the ldap service
   */
  public LdapService getLdapService() {
    return ldapService;
  }

  /**
   * Asigna ldap service.
   * 
   * @param ldapService
   *           new ldap service
   */
  public void setLdapService(LdapService ldapService) {
    this.ldapService = ldapService;
  }

  /* (non-Javadoc)
   * @see org.springframework.security.providers.dao.AbstractUserDetailsAuthenticationProvider#retrieveUser(java.lang.String, org.springframework.security.providers.UsernamePasswordAuthenticationToken)
   */
  @Override
  protected UserDetails retrieveUser(String username, UsernamePasswordAuthenticationToken authentication)
      throws AuthenticationException {
    System.out.println("Devolviendo usuario");
    UsuarioLdap user = new UsuarioLdap();
    user.setUsername(username);
    try {
      if (generalProperties.containsKey("USE_LDAP_GROUPS_ROLES")
          && ((String) generalProperties.get("USE_LDAP_GROUPS_ROLES")).equals("true")) {
        Attributes attrs = ldapService.getLdapAttributes(user);
        if (attrs != null) {
          locateUserCenter(user, attrs);
          locateUserRoles(user, attrs);
        } else {
          logger.log(Level.WARNING, "NTLM_Security: Usuario no encontrado en el directorio activo");
        }
        attachUserInformation(user);
      } else {
        attachUserInformation(user);
        if (user.getUser() != null) {
          user.setCentro(user.getUser().getCentro());
          if (user.getUser().isEmpleado()) {
            Puesto puesto = ((Empleado) user.getUser()).getPuesto();
            if (puesto.equals(Puesto.ADMIN)) addRole(user, Role.ROLE_ADMIN);
            //if (puesto.equals(Puesto.TRAINER_COORDINATOR)) addRole(user, Role.ROLE_TRAINER_COORDINATOR);
            //if (puesto.equals(Puesto.MANAGER)) addRole(user, Role.ROLE_MANAGER);
            if (puesto.equals(Puesto.AREA_MANAGER)) addRole(user, Role.ROLE_AREA_MANAGER);
            if (puesto.equals(Puesto.TRAINER)) addRole(user, Role.ROLE_TRAINER);
            //if (puesto.equals(Puesto.INFORMER)) addRole(user, Role.ROLE_INFORMER);
            //if (puesto.equals(Puesto.GESTOR_012)) addRole(user, Role.ROLE_012);
            if (puesto.equals(Puesto.EXTERNAL_PERSONAL)) addRole(user, Role.ROLE_EXTERN);
            //if (puesto.equals(Puesto.FINANCIAL_MANAGER)) addRole(user, Role.ROLE_FINANCIAL_MANAGER);
            if (puesto.equals(Puesto.DIGITAL_ALLY_MANAGER)) addRole(user, Role.ROLE_DIGITAL_ALLY);
          } else {
            addRole(user, Role.ROLE_USER);
          }
        }
      }
    } catch (DataAccessException repositoryProblem) {
      throw new AuthenticationServiceException(repositoryProblem.getMessage(), repositoryProblem);
    }
    if (user == null) throw new AuthenticationServiceException("User cannot be null");
    return user;
  }

  /* (non-Javadoc)
   * @see org.springframework.security.providers.dao.AbstractUserDetailsAuthenticationProvider#additionalAuthenticationChecks(org.springframework.security.userdetails.UserDetails, org.springframework.security.providers.UsernamePasswordAuthenticationToken)
   */
  @Override
  protected void additionalAuthenticationChecks(UserDetails userDetails,
      UsernamePasswordAuthenticationToken authentication) throws org.springframework.security.AuthenticationException {
  // No hacer nada para evitar problemas con NTLM
  }

  private void attachUserInformation(UsuarioLdap user) {
    try {
      List<Empleado> e = service.findWhere(Empleado.class, "username", user.getUsername());
      if (e != null && !e.isEmpty() && e.size() == 1) {
        user.setUser(e.get(0));
      } else {
        List<UsuarioCyL> u = service.findWhere(UsuarioCyL.class, "username", user.getUsername());
        if (u != null && !u.isEmpty() && u.size() == 1) {
          user.setUser(u.get(0));
        }
      }
    } catch (RuntimeException e) {
      logger.log(Level.SEVERE, "Error al añadir la información del usuario: " + e.getLocalizedMessage());
      e.printStackTrace();
    }
  }

  private void locateUserCenter(UsuarioLdap user, Attributes attrs) {
    List<Centro> centros = service.findAll(Centro.class);
    System.out.println("Buscando el centro del usuario: MemberOf: " + attrs.get("memberOf") + " DistinguishedName: "
        + attrs.get("distinguishedName"));
    for (Centro centro : centros) {
      if (attrContains("DC=" + centro.getDominio(), attrs.get("memberOf"))) user.setCentro(centro);
    }
    if (user.getCentro() == null) {
      for (Centro centro : centros) {
        if (attrContains("DC=" + centro.getDominio(), attrs.get("distinguishedName"))) user.setCentro(centro);
      }
    }
  }

  private void locateUserRoles(UsuarioLdap user, Attributes attrs) {
    locateUserRole(user, attrs, "ldap_groups_superadmin", Role.ROLE_ADMIN);
    //locateUserRole(user, attrs, "ldap_groups_trainercoordinator", Role.ROLE_TRAINER_COORDINATOR);
    //locateUserRole(user, attrs, "ldap_groups_manager", Role.ROLE_MANAGER);
    locateUserRole(user, attrs, "ldap_groups_areamanager", Role.ROLE_AREA_MANAGER);
    //locateUserRole(user, attrs, "ldap_groups_informer", Role.ROLE_INFORMER);
    locateUserRole(user, attrs, "ldap_groups_trainer", Role.ROLE_TRAINER);
    //locateUserRole(user, attrs, "ldap_groups_012", Role.ROLE_012);
    locateUserRole(user, attrs, "ldap_groups_user", Role.ROLE_USER);
    locateUserRole(user, attrs, "ldap_groups_ws", Role.ROLE_WS);
    locateUserRole(user, attrs, "ldap_groups_areamanager", Role.ROLE_DIGITAL_ALLY);
  }

  private void locateUserRole(UsuarioLdap user, Attributes attrs, String nombreParametro, Role role) {
    try {
      Parameter p = service.findOneWhere(Parameter.class, "nombre", nombreParametro);
      if (p != null && p.getValor() != null && !user.getRolesList().contains(role)) {
        String[] grupos = p.getValor().split(",");
        if (attrs.get("memberOf") != null) for (String grupo : grupos) {
          if (!grupo.trim().isEmpty() && attrContains("OU=" + grupo, attrs.get("memberOf"))) addRole(user, role);
        }
        if (attrs.get("distinguishedName") != null)
          for (String grupo : grupos) {
            if (!grupo.trim().isEmpty() && attrContains("OU=" + grupo, attrs.get("distinguishedName")))
              addRole(user, role);
          }
      }
    } catch (NoResultException e) {
      logger.log(Level.SEVERE, "NTLM_Security: No se encuentra el parametro " + nombreParametro
          + " en la tabla parámetros");
    } catch (RuntimeException e) {
      logger.log(Level.SEVERE, "NTLM_Security: Error al obtener la información de la tabla parametros");
    }

  }

  private void addRole(UsuarioLdap user, Role role) {
    if (!user.getRolesList().contains(role)) user.getRolesList().add(role);
  }

  private boolean attrContains(String cadena, Attribute attr) {
    logger.log(Level.FINER, "Comprobando grupo:" + cadena);
    if (attr != null)
      for (int i = 0; i < attr.size(); i++) {
        try {
          if (((String) attr.get(i)).toLowerCase().contains((cadena).toLowerCase())) {
            logger.log(Level.FINER, "Comprobado grupo:" + cadena + " coincidencite con :"
                + ((String) attr.get(i)).toLowerCase());
            return true;
          }
        } catch (NamingException e) {
          e.printStackTrace();
        }
      }
    return false;
  }
}
