package es.csa.fdt.cyldigital.tasks;

/*
 * #%L
 * Librerías del núcleo para Web - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.Date;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;

import es.csa.fdt.cyldigital.servicios.entity.Curso;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaTaller;
import es.csa.fdt.cyldigital.servicios.entity.PreinscripcionCurso;
import es.csa.fdt.cyldigital.servicios.entity.PreinscripcionTaller;
import es.csa.fdt.cyldigital.servicios.entity.ServicioCurso;
import es.csa.fdt.cyldigital.servicios.entity.ServicioTaller;
import es.csa.fdt.cyldigital.servicios.entity.Taller;
import es.csa.fdt.cyldigital.servicios.entity.PreinscripcionCurso.PreferenciaHoraria;
import es.csa.fdt.cyldigital.tareas.entity.InformeTarea;

/**
 * Clase ActividadCompletadaJob.
 */
public class ActividadCompletadaJob extends TransactionalQuartzTask {
  private static final Logger logger = Logger.getLogger(ActividadCompletadaJob.class.getName());

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.tasks.TransactionalQuartzTask#executeTransactional(org.quartz.JobExecutionContext, javax.persistence.EntityManager, es.csa.fdt.cyldigital.tareas.entity.InformeTarea)
   */
  @Override
  @SuppressWarnings("unchecked")
  protected void executeTransactional(JobExecutionContext arg0, EntityManager em, InformeTarea informe)
      throws JobExecutionException {

    /*
     * Solo se procesan aquellos cursos que han sido marcados como cerrados
     */
    String consulta = "SELECT e FROM " + Curso.class.getName()
        + " e WHERE fechaFin <= :valor1 and cerrada = :valor2 and tratada= :valor3";
    Query query = em.createQuery(consulta);
    query.setParameter("valor1", new Date());
    query.setParameter("valor2", true);
    query.setParameter("valor3", false);

    List<Curso> cursos = query.getResultList();

    boolean flag = false;
    for (Curso curso : cursos) {
      /*
       * Tras la finalización de un curso, aquellos usuarios que se quedaron en lista de espera son añadidos
       * automáticamente a la lista de interesados del servicio
       */
      for (MatriculaCurso matricula : curso.getMatriculas()) {
        if (matricula.isEnListaEspera()) {
          flag = false;
          for (PreinscripcionCurso preinscripcion : ((ServicioCurso) curso.getServicio()).getPreinscripciones()) {
            /*
             * A los que ya estaban en la lista de espera se les beneficia
             */
            if (preinscripcion.getUsuario().getId() == matricula.getUsuario().getId()) {
              preinscripcion.setPeso(preinscripcion.getPeso() - 1);
              em.getTransaction().begin();
              em.persist(preinscripcion);
              em.getTransaction().commit();
              flag = true;
            }
          }
          if (!flag) {
            PreinscripcionCurso p = new PreinscripcionCurso();
            p.setServicioCurso((ServicioCurso) curso.getServicio());
            p.setFecha(new Date());
            p.setUsuario(matricula.getUsuario());
            p.setCentro(matricula.getUsuario().getCentro());
            p.setHorarioPreferido(PreferenciaHoraria.I);
            p.setPeso(-1);

            em.getTransaction().begin();
            em.persist(p);
            em.getTransaction().commit();
            /*
             * Comentado para implementar los servicios MULTILANGUAGE
             * TODO: mirar y modificar
            logger.log(Level.FINE, "Usuario añadido a la lista de interesados: " + p.getUsuario().getId() + " - "
                + p.getServicioCurso().getNombre() + " (" + p.getServicioCurso().getId() + ")");
             */
          }
        } else {
          /*
           * A los usuarios cuya asistencia es menor del 50% se les penaliza
           */
          if (matricula.getAsistencia() == null || matricula.getAsistencia() < 50) {
            matricula.getUsuario().setPeso(matricula.getUsuario().getPeso() + 1);
            em.getTransaction().begin();
            em.persist(matricula.getUsuario());
            em.getTransaction().commit();
            /*
             * Comentado para implementar los servicios MULTILANGUAGE
             * TODO: mirar y modificar*/
            logger.log(Level.FINE, "Usuario penalizado por no asistencia: " + matricula.getUsuario().getId() + " - "
                /*+ matricula.getCurso().getApodo()*/ + " (" + matricula.getCurso().getId() + ")");
          }
        }
      }
      /*
       * Resistrar que el curso ya ha sido tratado
       */
      curso.setTratada(true);
      em.getTransaction().begin();
      em.persist(curso);
      em.getTransaction().commit();
      /*
       * Comentado para implementar los servicios MULTILANGUAGE
       * TODO: mirar y modificar*/
      logger.log(Level.FINE, "Curso tratado: " /*+ curso.getApodo()*/ + " (" + curso.getId() + ")");
    }

    /*
     * Solo se procesan aquellos talleres que han sido marcados como cerrados
     */
    consulta = "SELECT e FROM " + Taller.class.getName()
        + " e WHERE fechaFin <= :valor1 and cerrada = :valor2 and tratada= :valor3";
    query = em.createQuery(consulta);
    query.setParameter("valor1", new Date());
    query.setParameter("valor2", true);
    query.setParameter("valor3", false);

    List<Taller> talleres = query.getResultList();

    flag = false;
    for (Taller taller : talleres) {
      /*
       * Tras la finalización de un taller, aquellos usuarios que se quedaron en lista de espera son añadidos
       * automáticamente a la lista de interesados del servicio
       */
      for (MatriculaTaller matricula : taller.getMatriculas()) {
        if (matricula.isEnListaEspera()) {
          flag = false;
          for (PreinscripcionTaller preinscripcion : ((ServicioTaller) taller.getServicio()).getPreinscripciones()) {
            /*
             * A los que ya estaban en la lista de espera se les beneficia
             */
            if (preinscripcion.getUsuario().getId() == matricula.getUsuario().getId()) {
              preinscripcion.setPeso(preinscripcion.getPeso() - 1);
              em.getTransaction().begin();
              em.persist(preinscripcion);
              em.getTransaction().commit();
              flag = true;
            }
          }
          if (!flag) {
            PreinscripcionTaller p = new PreinscripcionTaller();
            p.setServicioTaller((ServicioTaller) taller.getServicio());
            p.setFecha(new Date());
            p.setUsuario(matricula.getUsuario());
            p.setCentro(matricula.getUsuario().getCentro());
            p.setHorarioPreferido(es.csa.fdt.cyldigital.servicios.entity.PreinscripcionTaller.PreferenciaHoraria.I);
            p.setPeso(-1);

            em.getTransaction().begin();
            em.persist(p);
            em.getTransaction().commit();
            /*
             * Comentado para implementar los servicios MULTILANGUAGE
             * TODO: mirar y modificar
            logger.log(Level.FINE, "Usuario añadido a la lista de interesados: " + p.getUsuario().getId() + " - "
                + p.getServicioTaller().getNombre() + " (" + p.getServicioTaller().getId() + ")");
             */
          }
        } else {
          /*
           * A los usuarios cuya asistencia es menor del 50% se les penaliza
           */
          if (matricula.getAsistencia() == null || matricula.getAsistencia() < 50) {
            matricula.getUsuario().setPeso(matricula.getUsuario().getPeso() + 1);
            em.getTransaction().begin();
            em.persist(matricula.getUsuario());
            em.getTransaction().commit();
            /*
             * Comentado para implementar los servicios MULTILANGUAGE
             * TODO: mirar y modificar
            logger.log(Level.FINE, "Usuario penalizado por no asistencia: " + matricula.getUsuario().getId() + " - "
                + matricula.getTaller().getApodo() + " (" + matricula.getTaller().getId() + ")");
             */
          }
        }
      }
      /*
       * Resistrar que el taller ya ha sido tratado
       */
      taller.setTratada(true);
      em.getTransaction().begin();
      em.persist(taller);
      em.getTransaction().commit();
      logger.log(Level.FINE, "Taller tratado: " + taller.getApodo() + " (" + taller.getId() + ")");
    }
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.tasks.TransactionalQuartzTask#getTaskName()
   */
  @Override
  protected String getTaskName() {
    return "ActividadCompletadaJob";
  }

}
