package es.csa.fdt.cyldigital.tasks;

/*
 * #%L
 * Librerías del núcleo para Web - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import javax.servlet.http.HttpServletResponse;

import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

import es.csa.commons.faces.util.PredefinedLocaleResolver;
import es.csa.fdt.cyldigital.calendario.entity.ActividadCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.AsistenciaCalendarItem;
import es.csa.fdt.cyldigital.calendario.entity.UserActivityCalendarItem;
import es.csa.fdt.cyldigital.calendario.service.AsistenciaCalendarItemService;
import es.csa.fdt.cyldigital.servicios.entity.Actividad;
import es.csa.fdt.cyldigital.servicios.entity.Charla;
import es.csa.fdt.cyldigital.servicios.entity.Curso;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCharla;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaTaller;
import es.csa.fdt.cyldigital.servicios.entity.Taller;
import es.csa.fdt.cyldigital.tareas.entity.InformeTarea;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;

/**
 * Clase ActualizacionAsistenciaJob.
 */
@Controller
public class ActualizacionAsistenciaJob extends TransactionalQuartzTask {

  private static final Logger logger = Logger.getLogger(ActualizacionAsistenciaJob.class.getName());

  private static final int UNIR_ASISTENCIAS_MINUTOS = 20;
  private static final int MIN_DURACION_ASISTENCIA = 5;

  @Autowired
  private AsistenciaCalendarItemService serviceAsistencia;

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.tasks.TransactionalQuartzTask#executeTransactional(org.quartz.JobExecutionContext, javax.persistence.EntityManager, es.csa.fdt.cyldigital.tareas.entity.InformeTarea)
   */
  @Transactional
  @Override
  protected void executeTransactional(JobExecutionContext arg0, EntityManager em, InformeTarea informe)
      throws JobExecutionException {

    List<AsistenciaCalendarItem> asistencias = new ArrayList<AsistenciaCalendarItem>();

    List<ActividadCalendarItem> activities = getYesterdayActivityCalendarItems(em);

    for (ActividadCalendarItem activityCalendarItem : activities) {
      logger.log(Level.SEVERE, "Asistencia::Actividad -> " + activityCalendarItem.getId());
      List<Usuario> usuarios = getMatriculadosActividad(activityCalendarItem.getActividad());
      AsistenciaCalendarItem anterior = null;
      for (Usuario u : usuarios) {

        List<UserActivityCalendarItem> useractivities = getYesterdayUserActivity(em, u);

        for (UserActivityCalendarItem activity : useractivities) {
          // Si usuario, duracion, horario
          // logger.log(Level.SEVERE, "Asistencia::Actividad de usuario -> " + activity.getUsuario().getId()
          // + " -- idActividad --" + activity.getId());
          if (isDuracionCorrecta(activity) && isEnHorarioCurso(activity, activityCalendarItem)) {
            // Si está en el aula valida
            if (!isComprobarAula() || isAulaValida(activity, activityCalendarItem.getActividad())) {
              // Merge si necesario
              // logger.log(Level.SEVERE, "Asistencia::Asistencia a crear");
              AsistenciaCalendarItem actual = createAsistencia(activityCalendarItem.getActividad(), activity
                  .getUsuario(), activity);
              if (anterior == null) {
                anterior = actual;
                // logger.log(Level.SEVERE, "Asistencia::Anterior null");
              } else if (isAsistenciasJuntas(actual, anterior)) {
                // logger.log(Level.SEVERE, "Asistencia::Juntar asistencias");
                anterior = juntarAsistencias(actual, anterior);
              } else {
                if (checkNoOtherAsistencia(anterior, em)) {
                  // logger.log(Level.SEVERE, "Asistencia::Añadir asistencia");
                  asistencias.add(anterior);
                  anterior = actual;
                }
              }
            }
          }
        }
        if (anterior != null) {
          if (checkNoOtherAsistencia(anterior, em)) {
            // logger.log(Level.SEVERE, "Asistencia::Añadir asistencia");
            asistencias.add(anterior);
          }
        }
      }

    }

    for (AsistenciaCalendarItem asistencia : asistencias) {
      em.getTransaction().begin();
      em.persist(asistencia);
      em.getTransaction().commit();
    }

  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.tasks.TransactionalQuartzTask#getTaskName()
   */
  @Override
  protected String getTaskName() {
    return "ActualizacionAsistenciaJob";
  }

  // -------------------------------------------------------------------------------------------------METODOS AUXILIARES

  private AsistenciaCalendarItem createAsistencia(Actividad actividad, Usuario usuario,
      UserActivityCalendarItem useractivity) {
    AsistenciaCalendarItem asistencia = new AsistenciaCalendarItem();
    asistencia.setAutomatica(true);
    asistencia.setActividad(actividad);
    asistencia.setUsuario(usuario);
    asistencia.setAllDay(false);
    Calendar cal2 = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    cal2.setTime(useractivity.getStart());
    cal2.set(Calendar.MINUTE, (cal2.get(Calendar.MINUTE)) + 10);
    asistencia.setStart(useractivity.getStart());
    asistencia.setEnd(useractivity.getEnd() == null ? cal2.getTime() : useractivity.getEnd());
    asistencia.setExcepcion(es.csa.fdt.cyldigital.calendario.entity.AsistenciaCalendarItem.Excepcion.SIN);
    return asistencia;
  }

  private List<UserActivityCalendarItem> getYesterdayUserActivity(EntityManager em, Usuario usuario) {
    // Importante que se devuelvan ordenadas para que las podamos juntar
    String consulta = "SELECT k FROM " + UserActivityCalendarItem.class.getName()
        + " k WHERE start >= ? and  usuario = ? ORDER BY usuario ASC, start ASC";

    Query query = em.createQuery(consulta);
    Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    Calendar cal2 = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());

    // OJO Sólo se procesan las asistencias del dia anterior
    cal.set(Calendar.HOUR_OF_DAY, 0);
    cal.set(Calendar.MINUTE, 0);
    cal.set(Calendar.SECOND, 1);
    cal.set(Calendar.DATE, cal.get(Calendar.DATE) - 1);

    // cal2.set(Calendar.HOUR_OF_DAY, 23);
    // cal2.set(Calendar.MINUTE, 59);
    // cal2.set(Calendar.SECOND, 59);
    // cal2.set(Calendar.DATE, (cal2.get(Calendar.DATE)) - 1);

    query.setParameter(1, cal.getTime());
    // query.setParameter(2, cal2.getTime());
    query.setParameter(2, usuario);

    List<UserActivityCalendarItem> uas = (List<UserActivityCalendarItem>) query.getResultList();
    return uas;
  }

  private List<ActividadCalendarItem> getYesterdayActivityCalendarItems(EntityManager em) {

    List<Actividad> activities = new ArrayList<Actividad>();

    String consulta = "SELECT e FROM " + ActividadCalendarItem.class.getName() + " e WHERE start >= ?";

    Query query = em.createQuery(consulta);
    Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    // Calendar cal2 = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());

    // OJO Sólo se procesan las asistencias del dia anterior

    cal.set(Calendar.DATE, (cal.get(Calendar.DATE)) - 1);
    // cal2.set(Calendar.DATE, (cal2.get(Calendar.DATE)) - 1);

    cal.set(Calendar.HOUR_OF_DAY, 0);
    cal.set(Calendar.MINUTE, 0);
    cal.set(Calendar.SECOND, 1);

    query.setParameter(1, cal.getTime());
    DateFormat f = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss");
    logger.log(Level.SEVERE, consulta + " - " + f.format(cal.getTime()));
    // query.setParameter(2, cal2.getTime());

    return (List<ActividadCalendarItem>) query.getResultList();

  }

  private List<Usuario> getMatriculadosActividad(Actividad actividad) {
    List<Usuario> usuarios = new ArrayList<Usuario>();

    if (actividad instanceof Curso) {
      for (MatriculaCurso matricula : ((Curso) actividad).getMatriculas())
        if (matricula.isAdmitido() && !matricula.isEnListaEspera()) usuarios.add(matricula.getUsuario());
    } else if (actividad instanceof Taller) {
      for (MatriculaTaller matricula : ((Taller) actividad).getMatriculas())
        if (matricula.isAdmitido() && !matricula.isEnListaEspera()) usuarios.add(matricula.getUsuario());
    } else if (actividad instanceof Charla) {
      for (MatriculaCharla matricula : ((Charla) actividad).getMatriculas())
        usuarios.add(matricula.getUsuario());
    }

    return usuarios;
  }

  private boolean isDuracionCorrecta(UserActivityCalendarItem u) {
    // logger.log(Level.SEVERE, "Asistencia::Duracion actividad " + u.getId() + "-> " + u.getMinutosDuracion());
    // logger.log(Level.SEVERE, "Asistencia::Duracion actividad start " + u.getStart().getTime() + "- end -"
    // + (u.getEnd() != null ? u.getEnd().getTime() : "null"));
    return u.getEnd() == null || u.getMinutosDuracion() > MIN_DURACION_ASISTENCIA;
  }

  private boolean isEnHorarioCurso(UserActivityCalendarItem activity, ActividadCalendarItem activityCalendarItem) {
    // IF inicio en el intervalo OR fin en el intervalo OR inicio anterior y fin exterior
    boolean inHorario = false;

    if (activity.getEnd() != null) {
      inHorario = activity.getStart().getTime() <= activityCalendarItem.getEnd().getTime()
          && activity.getEnd().getTime() >= activityCalendarItem.getStart().getTime();
    } else {
      inHorario = activity.getStart().getTime() <= activityCalendarItem.getEnd().getTime()
          && activity.getStart().getTime() >= activityCalendarItem.getStart().getTime();
    }

    logger.log(Level.SEVERE, "Asistencia::In Horario-> " + (inHorario ? "Si" : "No"));
    return inHorario;

  }

  private boolean isComprobarAula() {
    return false;
  }

  private boolean isAulaValida(UserActivityCalendarItem activity, Actividad actividad) {
    if (actividad instanceof Curso) {
      if (((Curso) actividad).getAula().equals(activity.getAula())) return true;
    } else if (actividad instanceof Taller) {
      if (((Taller) actividad).getAula().equals(activity.getAula())) return true;
    } else if (actividad instanceof Charla) {
      if (((Charla) actividad).getAula().equals(activity.getAula())) return true;
    }
    return false;
  }

  private boolean isAsistenciasJuntas(AsistenciaCalendarItem a, AsistenciaCalendarItem b) {
    if (a.getUsuario().equals(b.getUsuario())) {
      if (a.getActividad().equals(b.getActividad())) {
        if (a.getEnd() != null) {
          if (a.getEnd().before(b.getEnd()) && a.getEnd().getTime() <= b.getStart().getTime()
              && a.getEnd().getTime() + (UNIR_ASISTENCIAS_MINUTOS * 1000) >= b.getStart().getTime()) return true;
        } else {
          Calendar cal2 = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
          cal2.setTime(a.getStart());
          cal2.set(Calendar.MINUTE, (cal2.get(Calendar.MINUTE)) + 10);
          if (cal2.getTime().before(b.getEnd()) && cal2.getTime().getTime() <= b.getStart().getTime()
              && cal2.getTime().getTime() + (UNIR_ASISTENCIAS_MINUTOS * 1000) >= b.getStart().getTime()) return true;
        }
      }
    }
    return false;
  }

  private AsistenciaCalendarItem juntarAsistencias(AsistenciaCalendarItem actual, AsistenciaCalendarItem anterior) {
    SimpleDateFormat formateador = new SimpleDateFormat("dd/MM/yyyy HH:mm");

    if (actual.getEnd() != null) {
      anterior.setEnd(actual.getEnd());
      if (anterior.getDetalles() == null || anterior.getDetalles().isEmpty()) {
        anterior.setDetalles("Agrupación de las asistencias: " + " (" + formateador.format(anterior.getStart()) + " - "
            + formateador.format(anterior.getEnd()) + ")");
      }
      anterior.setDetalles(anterior.getDetalles() + " (" + formateador.format(actual.getStart()) + " - "
          + formateador.format(actual.getEnd()) + ")");
    } else {
      Calendar cal2 = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
      cal2.setTime(actual.getStart());
      cal2.set(Calendar.MINUTE, (cal2.get(Calendar.MINUTE)) + 10);
      anterior.setEnd(cal2.getTime());
      if (anterior.getDetalles() == null || anterior.getDetalles().isEmpty()) {
        anterior.setDetalles("Agrupación de las asistencias: "
            + " ("
            + formateador.format(anterior.getStart())
            + " - "
            + (anterior.getEnd() != null ? formateador.format(anterior.getEnd()) : formateador.format(anterior
                .getStart())) + ")");
      }
      anterior.setDetalles(anterior.getDetalles() + " (" + formateador.format(actual.getStart()) + " - "
          + (actual.getEnd() != null ? formateador.format(actual.getEnd()) : formateador.format(actual.getStart()))
          + ")");
    }
    return anterior;
  }

  /**
   * Se comprueba que la asistencia automática generada no haya sido creada ya automáticamente.
   * 
   * @param asistencia
   *          Asistencia nueva
   * @param em
   *          Entity Manager
   * @return true si se puede generar, false y ya existe una asistencia creada
   */
  private boolean checkNoOtherAsistencia(AsistenciaCalendarItem asistencia, EntityManager em) {
    String consulta = "SELECT e FROM " + AsistenciaCalendarItem.class.getName()
        + " e WHERE actividad = ? and usuario = ? and e.start = ? and e.end = ?";

    Query query = em.createQuery(consulta);
    Calendar calStart = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    Calendar calEnd = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());

    calStart.setTime(asistencia.getStart());
    calEnd.setTime(asistencia.getEnd());

    query.setParameter(1, asistencia.getActividad());
    query.setParameter(2, asistencia.getUsuario());
    query.setParameter(3, calStart.getTime());
    query.setParameter(4, calEnd.getTime());

    return query.getResultList().isEmpty();
  }

  /**
   * Stream asistencia curso.
   * 
   * @param idCurso
   *           id curso
   * @param response
   *           response
   */
  @RequestMapping("/generarAsistenciaCurso")
  @Transactional(readOnly = true)
  public void streamAsistenciaCurso(@RequestParam("idCurso") String idCurso, HttpServletResponse response) {
    Curso curso = serviceAsistencia.find(Curso.class, idCurso);
    ActividadCalendarItem activityCalendarItem = getTodayCalendarItem(curso);

    List<AsistenciaCalendarItem> asistencias = new ArrayList<AsistenciaCalendarItem>();

    if (activityCalendarItem != null) {

      List<Usuario> usuarios = getMatriculadosActividad(activityCalendarItem.getActividad());
      AsistenciaCalendarItem anterior = null;
      for (Usuario u : usuarios) {

        List<UserActivityCalendarItem> useractivities = getYesterdayUserActivity(serviceAsistencia.getEntityManager(),
            u);

        for (UserActivityCalendarItem activity : useractivities) {
          // Si usuario, duracion, horario
          // logger.log(Level.SEVERE, "Asistencia::Actividad de usuario -> " + activity.getUsuario().getId()
          // + " -- idActividad --" + activity.getId());
          if (isDuracionCorrecta(activity) && isEnHorarioCurso(activity, activityCalendarItem)) {
            // Si está en el aula valida
            if (!isComprobarAula() || isAulaValida(activity, activityCalendarItem.getActividad())) {
              // Merge si necesario
              // logger.log(Level.SEVERE, "Asistencia::Asistencia a crear");
              AsistenciaCalendarItem actual = createAsistencia(activityCalendarItem.getActividad(), activity
                  .getUsuario(), activity);
              if (anterior == null) {
                anterior = actual;
                // logger.log(Level.SEVERE, "Asistencia::Anterior null");
              } else if (isAsistenciasJuntas(actual, anterior)) {
                // logger.log(Level.SEVERE, "Asistencia::Juntar asistencias");
                anterior = juntarAsistencias(actual, anterior);
              } else {
                if (checkNoOtherAsistencia(anterior, serviceAsistencia.getEntityManager())) {
                  // logger.log(Level.SEVERE, "Asistencia::Añadir asistencia");
                  asistencias.add(anterior);
                  anterior = actual;
                }
              }
            }
          }
        }
        if (anterior != null) {
          if (checkNoOtherAsistencia(anterior, serviceAsistencia.getEntityManager())) {
            // logger.log(Level.SEVERE, "Asistencia::Añadir asistencia");
            asistencias.add(anterior);
          }
        }
      }

    }
  }

  private ActividadCalendarItem getTodayCalendarItem(Actividad a) {
    Calendar calNow = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    Calendar calItem = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());

    for (ActividadCalendarItem activityCalendarItem : a.getCalendarItems()) {
      calItem.setTime(activityCalendarItem.getStart());
      if (calNow.get(Calendar.DAY_OF_YEAR) == calItem.get(Calendar.DAY_OF_YEAR)
          && calNow.get(Calendar.YEAR) == calItem.get(Calendar.YEAR)) { return activityCalendarItem; }
    }
    return null;
  }
}
