package es.csa.fdt.cyldigital.tasks;

/*
 * #%L
 * Librerías del núcleo para Web - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.logging.Logger;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;

import es.csa.commons.faces.util.PredefinedLocaleResolver;
import es.csa.fdt.cyldigital.recursos.entity.Ordenador;
import es.csa.fdt.cyldigital.recursos.entity.RemoteNotice;
import es.csa.fdt.cyldigital.recursos.entity.RemoteNotice.NotificationType;
import es.csa.fdt.cyldigital.recursos.entity.RemoteNotice.Status;
import es.csa.fdt.cyldigital.servicios.entity.Navegacion;
import es.csa.fdt.cyldigital.tareas.entity.InformeTarea;

/**
 * Clase ControlNavegacionJob.
 */
public class ControlNavegacionJob extends TransactionalQuartzTask {

  private static final Logger logger = Logger.getLogger(ControlNavegacionJob.class.getName());

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.tasks.TransactionalQuartzTask#executeTransactional(org.quartz.JobExecutionContext, javax.persistence.EntityManager, es.csa.fdt.cyldigital.tareas.entity.InformeTarea)
   */
  @Override
  @SuppressWarnings("unchecked")
  protected void executeTransactional(JobExecutionContext arg0, EntityManager em, InformeTarea informe)
      throws JobExecutionException {

    String consulta = "SELECT e FROM " + Ordenador.class.getName() + " e WHERE " + " usuarioConectado is not null "
        + " and navegacionControlada = ? ";

    Query query = em.createQuery(consulta);

    query.setParameter(1, true);

    List<Ordenador> ordenadores = (List<Ordenador>) query.getResultList();

    for (Ordenador ordenador : ordenadores) {
      String consulta2 = "SELECT e FROM "
          + Navegacion.class.getName()
          + " e WHERE  ordenador = ? "
          + " and fechaInicio <= ? and fechaFin >= ? and horaInicio < ? and horaFin > ? and cerrada = ? order by fechaFin DESC, horaFin DESC";

      Query query2 = em.createQuery(consulta2);

      Date d = new Date();
      query2.setParameter(1, ordenador);
      query2.setParameter(2, d);
      query2.setParameter(3, d);
      query2.setParameter(4, d);
      query2.setParameter(5, d);
      query2.setParameter(6, false);

      List<Navegacion> navegaciones = (List<Navegacion>) query2.getResultList();

      if (navegaciones == null || navegaciones.isEmpty()) {
        cerrarSesion(ordenador, em);
      } else {
        Navegacion n = navegaciones.get(0);
        mostrarPrimerMensaje(n, em);
        mostrarSegundoMensaje(n, em);
      }
    }
  }

  @SuppressWarnings("unchecked")
  private void mostrarPrimerMensaje(Navegacion n, EntityManager em) {
    Calendar calNow = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    Calendar cal2 = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    cal.setTime(n.getFechaFin());
    cal.set(Calendar.MINUTE, cal.get(Calendar.MINUTE) - 5);
    cal.set(Calendar.SECOND, cal.get(Calendar.SECOND) - 35);
    cal2.setTime(n.getFechaFin());
    cal2.set(Calendar.MINUTE, cal2.get(Calendar.MINUTE) - 5);
    cal2.set(Calendar.SECOND, cal2.get(Calendar.SECOND) + 35);
    if (calNow.getTime().before(cal2.getTime()) && calNow.getTime().after(cal.getTime())) {
      String consulta = "SELECT e FROM " + RemoteNotice.class.getName() + " e WHERE "
          + " resultMessage = ? and entero = ?";
      Query query = em.createQuery(consulta);
      query.setParameter(1, "control-navegacion:" + n.getId());
      query.setParameter(2, 5);
      List<RemoteNotice> rrnn = (List<RemoteNotice>) query.getResultList();

      if (rrnn == null || rrnn.isEmpty()) {
        RemoteNotice r = new RemoteNotice();
        r.setEntero(5);
        r.setIpAddress(n.getOrdenador().getIp());
        r.setMacAddress(n.getOrdenador().getMac());
        r.setMessage("Le quedan 5 minutos de sesión, si desea más tiempo pídalo en recepción."
            + " Tras este tiempo se cerrará su sesión automáticamente.");
        r.setNotificationType(NotificationType.DO_SHOW_MSG);
        r.setResultMessage("control-navegacion:" + n.getId());
        r.setServerId(n.getOrdenador().getCentro().getId());
        r.setStatus(Status.PENDING);
        r.setUserName(n.getUsuario() != null ? n.getUsuario().getNombreCompleto() : "");
        em.getTransaction().begin();
        em.persist(r);
        em.getTransaction().commit();
      }
    }
  }

  @SuppressWarnings("unchecked")
  private void mostrarSegundoMensaje(Navegacion n, EntityManager em) {
    Calendar calNow = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    Calendar cal2 = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
    cal.setTime(n.getFechaFin());
    cal.set(Calendar.MINUTE, cal.get(Calendar.MINUTE) - 1);
    cal.set(Calendar.SECOND, cal.get(Calendar.SECOND) - 35);
    cal2.setTime(n.getFechaFin());
    cal2.set(Calendar.MINUTE, cal2.get(Calendar.MINUTE) - 1);
    cal2.set(Calendar.SECOND, cal2.get(Calendar.SECOND) + 35);
    if (calNow.getTime().before(cal2.getTime()) && calNow.getTime().after(cal.getTime())) {
      String consulta = "SELECT e FROM " + RemoteNotice.class.getName() + " e WHERE "
          + " resultMessage = ? and entero = ?";
      Query query = em.createQuery(consulta);
      query.setParameter(1, "control-navegacion:" + n.getId());
      query.setParameter(2, 1);
      List<RemoteNotice> rrnn = (List<RemoteNotice>) query.getResultList();

      if (rrnn == null || rrnn.isEmpty()) {
        RemoteNotice r = new RemoteNotice();
        r.setEntero(1);
        r.setIpAddress(n.getOrdenador().getIp());
        r.setMacAddress(n.getOrdenador().getMac());
        r.setMessage("Le queda 1 minuto de sesión, si desea más tiempo pídalo en recepción."
            + " Tras este tiempo se cerrará su sesión automáticamente.");
        r.setNotificationType(NotificationType.DO_SHOW_MSG);
        r.setResultMessage("control-navegacion:" + n.getId());
        r.setServerId(n.getOrdenador().getCentro().getId());
        r.setStatus(Status.PENDING);
        r.setUserName(n.getUsuario() != null ? n.getUsuario().getNombreCompleto() : "");
        em.getTransaction().begin();
        em.persist(r);
        em.getTransaction().commit();
      }
    }
  }

  private void cerrarSesion(Ordenador ordenador, EntityManager em) {
    // Miro a ver que el mensaje no este ya pendiente de entrega
    String consulta = "SELECT e FROM " + RemoteNotice.class.getName() + " e WHERE "
        + " ipAddress = ? and notificationType = ? and status = ?";

    Query query = em.createQuery(consulta);
    query.setParameter(1, ordenador.getIp());
    query.setParameter(2, NotificationType.CLIENT_LOGOUT);
    query.setParameter(3, Status.PENDING);
    List<RemoteNotice> rrnn = (List<RemoteNotice>) query.getResultList();

    if (rrnn == null || rrnn.isEmpty()) {
      RemoteNotice r = new RemoteNotice();
      r.setEntero(10);
      r.setIpAddress(ordenador.getIp());
      r.setMacAddress(ordenador.getMac());
      r
          .setMessage("No tiene permiso para usar este ordenador por tanto su sesión se va a cerrar. Pída permiso en recepción.");
      r.setNotificationType(NotificationType.DO_LOGOUT);
      r.setServerId(ordenador.getCentro().getId());
      r.setStatus(Status.PENDING);
      em.getTransaction().begin();
      em.persist(r);
      em.getTransaction().commit();
    }
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.tasks.TransactionalQuartzTask#getTaskName()
   */
  @Override
  protected String getTaskName() {
    return "ControlNavegacionJob";
  }
}
