package es.csa.fdt.cyldigital.tasks;

/*
 * #%L
 * Librerías del núcleo para Web - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;

import es.csa.fdt.cyldigital.centros.entity.BP;
import es.csa.fdt.cyldigital.centros.entity.BP.Tipo;
import es.csa.fdt.cyldigital.informes.CuadroDeMandos;
import es.csa.fdt.cyldigital.notificacion.NotificationService;
import es.csa.fdt.cyldigital.notificacion.entity.Alarma;
import es.csa.fdt.cyldigital.notificacion.entity.Alarma.CondicionEvaluar;
import es.csa.fdt.cyldigital.notificacion.entity.Alarma.FrecuenciaNotificacion;
import es.csa.fdt.cyldigital.notificacion.entity.Alarma.Operando;
import es.csa.fdt.cyldigital.notificacion.entity.Alarma.PeriodoBP;
import es.csa.fdt.cyldigital.notificacion.service.InformeNotificationService;
import es.csa.fdt.cyldigital.tareas.entity.InformeTarea;

/**
 * Clase EnvioAlarmasJob.
 */
public class EnvioAlarmasJob extends TransactionalQuartzTask {

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.tasks.TransactionalQuartzTask#executeTransactional(org.quartz.JobExecutionContext, javax.persistence.EntityManager, es.csa.fdt.cyldigital.tareas.entity.InformeTarea)
   */
  @Override
  @SuppressWarnings("unchecked")
  protected void executeTransactional(JobExecutionContext arg0, EntityManager em, InformeTarea informe)
      throws JobExecutionException {

    String consulta = "SELECT e FROM " + Alarma.class.getName() + " e WHERE "
        + " fechaProximaEjecucion < ?  and desactivado=false";

    Query query = em.createQuery(consulta);
    query.setParameter(1, new Date());

    List<Alarma> alarmas = (List<Alarma>) query.getResultList();

    NotificationService service = new NotificationService<InformeNotificationService>();
    DecimalFormat d = new DecimalFormat("#.####");

    for (Alarma a : alarmas) {
      double valorActual = 0;
      double valorParaComparar = 0;
      boolean cumpleAlarma = false;
      boolean enviar = true;

      try {
        valorActual = calcularValorActual(a, em);
        String aux = d.format(valorActual);
        valorActual = new Double(aux.replace(",", "."));

        valorParaComparar = valorParaComparar(a, em);
        aux = d.format(valorParaComparar);
        valorParaComparar = new Double(aux.replace(",", "."));
      } catch (NumberFormatException e) {
        e.printStackTrace();
      }
      cumpleAlarma = realizarComparacion(a, valorActual, valorParaComparar);

      if (cumpleAlarma) {
        // Si es cuando cambie y no ha cambiado, no envio
        if ((a.getFrecuenciaNotificacion() == FrecuenciaNotificacion.CUANDO_CAMBIE)
            && valorActual == a.getValorAnterior()) {
          enviar = false;
        } else if (!comprobarSiCambio(a, em, valorActual)) {
          enviar = false;
        }
        if (enviar) service.sendAlarma(a, em);
      }
      a.setFechaProximaEjecucion(calcularFechaProximaNotificacion(a));

      a.setValorAnterior(valorActual);
      em.getTransaction().begin();
      em.persist(a);
      em.getTransaction().commit();
    }
  }

  /**
   * @param a
   * @param em
   * @param valorActual
   * @return true si hay que enviar
   */
  private boolean comprobarSiCambio(Alarma a, EntityManager em, double valorActual) {
    if (!a.isNotificarSiCambia()) return true; // Da igual si cambia o no, se envía siempre.
    else {// notificar solo cuando cambie. Suponemos que el anterior ya va a estar truncado.
      if (a.getValorAnterior() == valorActual) return false;
      else return true;
    }
  }

  private boolean realizarComparacion(Alarma a, double valorActual, double valorParaComparar) {
    if (a.getOperando() == Operando.DIFERENTE) {
      if (valorActual != valorParaComparar) return true;
    } else if (a.getOperando() == Operando.IGUAL) {
      if (valorActual == valorParaComparar) return true;
    } else if (a.getOperando() == Operando.MAYOR) {
      if (valorActual > valorParaComparar) return true;
    } else if (a.getOperando() == Operando.MAYORIGUAL) {
      if (valorActual >= valorParaComparar) return true;
    } else if (a.getOperando() == Operando.MENOR) {
      if (valorActual < valorParaComparar) return true;
    } else if (a.getOperando() == Operando.MENORIGUAL) {
      if (valorActual <= valorParaComparar) return true;
    }
    return false;
  }

  private Date getFechaInicioSemanaActual() {
    Calendar c = Calendar.getInstance();
    c.add(Calendar.DAY_OF_YEAR, 8 - c.get(Calendar.DAY_OF_WEEK));
    c.add(Calendar.DAY_OF_YEAR, -6);
    return c.getTime();

  }

  private Date getFechaFinSemanaActual() {
    Calendar c = Calendar.getInstance();
    c.add(Calendar.DAY_OF_YEAR, 8 - c.get(Calendar.DAY_OF_WEEK));
    return c.getTime();
  }

  private Date getFechaInicioMesActual() {
    DateFormat dy = new SimpleDateFormat("yyyy");
    DateFormat dm = new SimpleDateFormat("MM");
    DateFormat def = new SimpleDateFormat("yyyy/MM/dd HH:mm");
    Date f = new Date();

    try {
      f = def.parse(dy.format(f) + "/" + dm.format(f) + "/01 00:00");
    } catch (ParseException e) {
      e.printStackTrace();
    }
    return f;
  }

  private Date getFechaFinMesActual() {
    Calendar c = Calendar.getInstance();
    c.setTime(new Date());
    c.set(Calendar.YEAR, Calendar.MONTH, c.getActualMaximum(Calendar.DAY_OF_MONTH));
    return c.getTime();
  }

  private Date getFechaInicioAnioActual() {
    DateFormat dy = new SimpleDateFormat("yyyy");
    DateFormat def = new SimpleDateFormat("yyyy/MM/dd HH:mm");
    Date f = new Date();

    try {
      f = def.parse(dy.format(f) + "/01/01 00:00");
    } catch (ParseException e) {
      e.printStackTrace();
    }
    return f;
  }

  private Date getFechaFinAnioActual() {
    DateFormat dy = new SimpleDateFormat("yyyy");
    DateFormat def = new SimpleDateFormat("yyyy/MM/dd HH:mm");
    Date f = new Date();

    try {
      f = def.parse(dy.format(f) + "/12/31 23:59");
    } catch (ParseException e) {
      e.printStackTrace();
    }
    return f;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.tasks.TransactionalQuartzTask#getTaskName()
   */
  @Override
  protected String getTaskName() {
    return "EnvioAlarmasJob";
  }

  private double calcularValorActual(Alarma a, EntityManager em) {
    double resultado = 0;
    CuadroDeMandos c = new CuadroDeMandos();

    if (a.getTipo() == Tipo.ALTAS) {
      if (a.getPeriodoBP() == PeriodoBP.SEMANA) resultado = (double) c.altasSemana(a.getCentro(), em);

      if (a.getPeriodoBP() == PeriodoBP.MES) resultado = (double) c.altasMes(a.getCentro(), em);

      if (a.getPeriodoBP() == PeriodoBP.ANIO) resultado = (double) c.altasAnyo(a.getCentro(), em);
    } else if (a.getTipo() == Tipo.ALTAS_MEDIAS) {
      if (a.getPeriodoBP() == PeriodoBP.SEMANA) resultado = c.altasMediasDiariasSemana(em, a.getCentro());

      if (a.getPeriodoBP() == PeriodoBP.MES) resultado = c.altasMediasDiariasMes(a.getCentro(), em);

      if (a.getPeriodoBP() == PeriodoBP.ANIO) resultado = c.altasMediasDiariasAnyo(a.getCentro(), em);
    } else if (a.getTipo() == Tipo.BAJAS) {
      if (a.getPeriodoBP() == PeriodoBP.SEMANA) resultado = c.getNumeroBajasSemana();

      if (a.getPeriodoBP() == PeriodoBP.MES) resultado = c.getNumeroBajasMes();

      if (a.getPeriodoBP() == PeriodoBP.ANIO) resultado = c.getNumeroBajasAnyo();
    } else if (a.getTipo() == Tipo.CURSOS_CERRADOS) {
      if (a.getPeriodoBP() == PeriodoBP.SEMANA) {
        c.numeroCursos(getFechaInicioSemanaActual(), getFechaFinSemanaActual(), a.getCentro(), em);
        resultado = c.getCursosCerrados();
      }

      if (a.getPeriodoBP() == PeriodoBP.MES) {
        c.numeroCursos(getFechaInicioMesActual(), getFechaFinMesActual(), a.getCentro(), em);
        resultado = c.getCursosCerrados();
      }

      if (a.getPeriodoBP() == PeriodoBP.ANIO) {
        c.numeroCursos(getFechaInicioAnioActual(), getFechaFinAnioActual(), a.getCentro(), em);
        c.getCursosCerrados();
      }
    } else if (a.getTipo() == Tipo.TALLERES_CERRADOS) {
      if (a.getPeriodoBP() == PeriodoBP.SEMANA) {
        c.numeroTalleres(getFechaInicioSemanaActual(), getFechaFinSemanaActual(), a.getCentro(), em);
        resultado = c.getCursosCerrados();
      }

      if (a.getPeriodoBP() == PeriodoBP.MES) {
        c.numeroTalleres(getFechaInicioMesActual(), getFechaFinMesActual(), a.getCentro(), em);
        resultado = c.getCursosCerrados();
      }

      if (a.getPeriodoBP() == PeriodoBP.ANIO) {
        c.numeroTalleres(getFechaInicioAnioActual(), getFechaFinAnioActual(), a.getCentro(), em);
        resultado = c.getCursosCerrados();
      }
    } else if (a.getTipo() == Tipo.EDAD_MEDIA) {
      if (a.getPeriodoBP() == PeriodoBP.SEMANA) resultado = c.edadMediaSemana(a.getCentro(), em);

      if (a.getPeriodoBP() == PeriodoBP.MES) resultado = c.edadMediaMes(a.getCentro(), em);

      if (a.getPeriodoBP() == PeriodoBP.ANIO) resultado = c.edadMediaAnyo(a.getCentro(), em);
    } else if (a.getTipo() == Tipo.HORAS_FORMACION_CURSOS) {
      if (a.getPeriodoBP() == PeriodoBP.SEMANA) {
        c.numeroCursos(getFechaInicioSemanaActual(), getFechaFinSemanaActual(), a.getCentro(), em);
        resultado = c.getHorasFormacion();
      }

      if (a.getPeriodoBP() == PeriodoBP.MES) {
        c.numeroCursos(getFechaInicioMesActual(), getFechaFinMesActual(), a.getCentro(), em);
        resultado = c.getHorasFormacion();
      }

      if (a.getPeriodoBP() == PeriodoBP.ANIO) {
        c.numeroCursos(getFechaInicioAnioActual(), getFechaFinAnioActual(), a.getCentro(), em);
        resultado = c.getHorasFormacion();
      }
    } else if (a.getTipo() == Tipo.HORAS_FORMACION_TALLERES) {
      if (a.getPeriodoBP() == PeriodoBP.SEMANA) {
        c.numeroTalleres(getFechaInicioSemanaActual(), getFechaFinSemanaActual(), a.getCentro(), em);
        resultado = c.getHorasFormacion();
      }

      if (a.getPeriodoBP() == PeriodoBP.MES) {
        c.numeroTalleres(getFechaInicioMesActual(), getFechaFinMesActual(), a.getCentro(), em);
        resultado = c.getHorasFormacion();
      }

      if (a.getPeriodoBP() == PeriodoBP.ANIO) {
        c.numeroTalleres(getFechaInicioAnioActual(), getFechaFinAnioActual(), a.getCentro(), em);
        resultado = c.getHorasFormacion();
      }
    } else if (a.getTipo() == Tipo.NUMEROALUMNOS_ASISTENTES_CURSOS) {
      if (a.getPeriodoBP() == PeriodoBP.SEMANA) {
        c.numeroCursos(getFechaInicioSemanaActual(), getFechaFinSemanaActual(), a.getCentro(), em);
        resultado = c.getNumeroAlumnosAsistentes();
      }

      if (a.getPeriodoBP() == PeriodoBP.MES) {
        c.numeroCursos(getFechaInicioMesActual(), getFechaFinMesActual(), a.getCentro(), em);
        resultado = c.getNumeroAlumnosAsistentes();
      }

      if (a.getPeriodoBP() == PeriodoBP.ANIO) {
        c.numeroCursos(getFechaInicioAnioActual(), getFechaFinAnioActual(), a.getCentro(), em);
        resultado = c.getNumeroAlumnosAsistentes();
      }
    } else if (a.getTipo() == Tipo.NUMEROALUMNOS_ASISTENTES_TALLERES) {
      if (a.getPeriodoBP() == PeriodoBP.SEMANA) {
        c.numeroTalleres(getFechaInicioSemanaActual(), getFechaFinSemanaActual(), a.getCentro(), em);
        resultado = c.getNumeroAlumnosAsistentes();
      }

      if (a.getPeriodoBP() == PeriodoBP.MES) {
        c.numeroTalleres(getFechaInicioMesActual(), getFechaFinMesActual(), a.getCentro(), em);
        resultado = c.getNumeroAlumnosAsistentes();
      }

      if (a.getPeriodoBP() == PeriodoBP.ANIO) {
        c.numeroTalleres(getFechaInicioAnioActual(), getFechaFinAnioActual(), a.getCentro(), em);
        resultado = c.getNumeroAlumnosAsistentes();
      }
    } else if (a.getTipo() == Tipo.NUMEROALUMNOS_CURSOS) {
      if (a.getPeriodoBP() == PeriodoBP.SEMANA) {
        c.numeroCursos(getFechaInicioSemanaActual(), getFechaFinSemanaActual(), a.getCentro(), em);
        resultado = c.getNumeroAlumnosMatriculados();
      }

      if (a.getPeriodoBP() == PeriodoBP.MES) {
        c.numeroCursos(getFechaInicioMesActual(), getFechaFinMesActual(), a.getCentro(), em);
        resultado = c.getNumeroAlumnosMatriculados();
      }

      if (a.getPeriodoBP() == PeriodoBP.ANIO) {
        c.numeroCursos(getFechaInicioAnioActual(), getFechaFinAnioActual(), a.getCentro(), em);
        resultado = c.getNumeroAlumnosMatriculados();
      }
    } else if (a.getTipo() == Tipo.NUMEROALUMNOS_TALLERES) {
      if (a.getPeriodoBP() == PeriodoBP.SEMANA) {
        c.numeroTalleres(getFechaInicioSemanaActual(), getFechaFinSemanaActual(), a.getCentro(), em);
        resultado = c.getNumeroAlumnosMatriculados();
      }

      if (a.getPeriodoBP() == PeriodoBP.MES) {
        c.numeroTalleres(getFechaInicioMesActual(), getFechaFinMesActual(), a.getCentro(), em);
        resultado = c.getNumeroAlumnosMatriculados();
      }

      if (a.getPeriodoBP() == PeriodoBP.ANIO) {
        c.numeroTalleres(getFechaInicioAnioActual(), getFechaFinAnioActual(), a.getCentro(), em);
        resultado = c.getNumeroAlumnosMatriculados();
      }
    } else if (a.getTipo() == Tipo.NUMEROALUMNOS_LISTA_ESPERA_CURSOS) {
      if (a.getPeriodoBP() == PeriodoBP.SEMANA) {
        c.numeroCursos(getFechaInicioSemanaActual(), getFechaFinSemanaActual(), a.getCentro(), em);
        resultado = c.getNumeroAlumnosEspera();
      }

      if (a.getPeriodoBP() == PeriodoBP.MES) {
        c.numeroCursos(getFechaInicioMesActual(), getFechaFinMesActual(), a.getCentro(), em);
        resultado = c.getNumeroAlumnosEspera();
      }

      if (a.getPeriodoBP() == PeriodoBP.ANIO) {
        c.numeroCursos(getFechaInicioAnioActual(), getFechaFinAnioActual(), a.getCentro(), em);
        resultado = c.getNumeroAlumnosEspera();
      }
    } else if (a.getTipo() == Tipo.NUMEROALUMNOS_LISTA_ESPERA_TALLERES) {
      if (a.getPeriodoBP() == PeriodoBP.SEMANA) {
        c.numeroTalleres(getFechaInicioSemanaActual(), getFechaFinSemanaActual(), a.getCentro(), em);
        resultado = c.getNumeroAlumnosEspera();
      }

      if (a.getPeriodoBP() == PeriodoBP.MES) {
        c.numeroTalleres(getFechaInicioMesActual(), getFechaFinMesActual(), a.getCentro(), em);
        resultado = c.getNumeroAlumnosEspera();
      }

      if (a.getPeriodoBP() == PeriodoBP.ANIO) {
        c.numeroTalleres(getFechaInicioAnioActual(), getFechaFinAnioActual(), a.getCentro(), em);
        resultado = c.getNumeroAlumnosEspera();
      }
    } else if (a.getTipo() == Tipo.NUMEROCURSOS) {
      if (a.getPeriodoBP() == PeriodoBP.SEMANA)
        resultado = c.numeroCursos(getFechaInicioSemanaActual(), getFechaFinSemanaActual(), a.getCentro(), em);

      if (a.getPeriodoBP() == PeriodoBP.MES)
        resultado = c.numeroCursos(getFechaInicioMesActual(), getFechaFinMesActual(), a.getCentro(), em);

      if (a.getPeriodoBP() == PeriodoBP.ANIO)
        resultado = c.numeroCursos(getFechaInicioAnioActual(), getFechaFinAnioActual(), a.getCentro(), em);
    } else if (a.getTipo() == Tipo.NUMEROTALLERES) {
      if (a.getPeriodoBP() == PeriodoBP.SEMANA)
        resultado = c.numeroTalleres(getFechaInicioSemanaActual(), getFechaFinSemanaActual(), a.getCentro(), em);

      if (a.getPeriodoBP() == PeriodoBP.MES)
        resultado = c.numeroTalleres(getFechaInicioMesActual(), getFechaFinMesActual(), a.getCentro(), em);

      if (a.getPeriodoBP() == PeriodoBP.ANIO)
        resultado = c.numeroTalleres(getFechaInicioAnioActual(), getFechaFinAnioActual(), a.getCentro(), em);
    } else if (a.getTipo() == Tipo.PORC_ASISTENTES_CURSOS) {
      if (a.getPeriodoBP() == PeriodoBP.SEMANA) {
        c.numeroCursos(getFechaInicioSemanaActual(), getFechaFinSemanaActual(), a.getCentro(), em);
        resultado = c.getPorcAsistentes();
      }

      if (a.getPeriodoBP() == PeriodoBP.MES) {
        c.numeroCursos(getFechaInicioMesActual(), getFechaFinMesActual(), a.getCentro(), em);
        resultado = c.getPorcAsistentes();
      }

      if (a.getPeriodoBP() == PeriodoBP.ANIO) {
        c.numeroCursos(getFechaInicioAnioActual(), getFechaFinAnioActual(), a.getCentro(), em);
        resultado = c.getPorcAsistentes();
      }
    } else if (a.getTipo() == Tipo.PORC_ASISTENTES_TALLERES) {
      if (a.getPeriodoBP() == PeriodoBP.SEMANA) {
        c.numeroTalleres(getFechaInicioSemanaActual(), getFechaFinSemanaActual(), a.getCentro(), em);
        resultado = c.getPorcAsistentes();
      }

      if (a.getPeriodoBP() == PeriodoBP.MES) {
        c.numeroTalleres(getFechaInicioMesActual(), getFechaFinMesActual(), a.getCentro(), em);
        resultado = c.getPorcAsistentes();
      }

      if (a.getPeriodoBP() == PeriodoBP.ANIO) {
        c.numeroTalleres(getFechaInicioAnioActual(), getFechaFinAnioActual(), a.getCentro(), em);
        resultado = c.getPorcAsistentes();
      }
    } else if (a.getTipo() == Tipo.PORC_OCUPACION_CURSOS) {
      if (a.getPeriodoBP() == PeriodoBP.SEMANA) {
        c.numeroCursos(getFechaInicioSemanaActual(), getFechaFinSemanaActual(), a.getCentro(), em);
        resultado = c.getPorcOcupacion();
      }

      if (a.getPeriodoBP() == PeriodoBP.MES) {
        c.numeroCursos(getFechaInicioMesActual(), getFechaFinMesActual(), a.getCentro(), em);
        resultado = c.getPorcOcupacion();
      }

      if (a.getPeriodoBP() == PeriodoBP.ANIO) {
        c.numeroCursos(getFechaInicioAnioActual(), getFechaFinAnioActual(), a.getCentro(), em);
        resultado = c.getPorcOcupacion();
      }
    } else if (a.getTipo() == Tipo.PORC_OCUPACION_TALLERES) {
      if (a.getPeriodoBP() == PeriodoBP.SEMANA) {
        c.numeroTalleres(getFechaInicioSemanaActual(), getFechaFinSemanaActual(), a.getCentro(), em);
        resultado = c.getPorcOcupacion();
      }

      if (a.getPeriodoBP() == PeriodoBP.MES) {
        c.numeroTalleres(getFechaInicioMesActual(), getFechaFinMesActual(), a.getCentro(), em);
        resultado = c.getPorcOcupacion();
      }

      if (a.getPeriodoBP() == PeriodoBP.ANIO) {
        c.numeroTalleres(getFechaInicioAnioActual(), getFechaFinAnioActual(), a.getCentro(), em);
        resultado = c.getPorcOcupacion();
      }
    }
    return resultado;
  }

  private double valorParaComparar(Alarma a, EntityManager em) {

    Query query;
    if (a.getCentro() != null) {
      String consulta = "SELECT e FROM " + BP.class.getName() + " e WHERE " + " tipo = ? and centro = ? ";
      query = em.createQuery(consulta);
      query.setParameter(1, a.getTipo());
      query.setParameter(2, a.getCentro());
    } else {
      String consulta = "SELECT e FROM " + BP.class.getName() + " e WHERE " + " tipo = ? and centro = null ";
      query = em.createQuery(consulta);
      query.setParameter(1, a.getTipo());
    }

    BP bp = (BP) query.getSingleResult();

    if (a.getCondicionEvaluar() == CondicionEvaluar.VALOR) {
      return (double) a.getValorEvaluar();
    } else if (a.getCondicionEvaluar() == CondicionEvaluar.NIVELINFERIOR) {
      return (double) bp.getMinimo();
    } else if (a.getCondicionEvaluar() == CondicionEvaluar.NIVELSUPERIOR) {
      return (double) bp.getMaximo();
    } else if (a.getCondicionEvaluar() == CondicionEvaluar.PORCENTAJEBP) {

      if (a.getPeriodoBP() == PeriodoBP.ANIO) return (double) (bp.getBpAnual() * a.getValorEvaluar() / 100);
      else if (a.getPeriodoBP() == PeriodoBP.MES) return (double) (bp.getBpMensual() * a.getValorEvaluar() / 100);
      else
      // semana
      return (double) (bp.getBpSemanal() * a.getValorEvaluar() / 100);

    } else if (a.getCondicionEvaluar() == CondicionEvaluar.BP) {
      if (a.getPeriodoBP() == PeriodoBP.ANIO) return (double) bp.getBpAnual();
      else if (a.getPeriodoBP() == PeriodoBP.MES) return (double) bp.getBpMensual();
      else
      // semana
      return (double) bp.getBpSemanal();
    } else {
      return 0;
    }
  }

  private Date calcularFechaProximaNotificacion(Alarma a) {
    Calendar c = Calendar.getInstance();
    Date aux = a.getHoraEnvio();
    DateFormat HH = new SimpleDateFormat("HH");
    DateFormat mm = new SimpleDateFormat("mm");
    DateFormat df = new SimpleDateFormat("yyyy/MM/dd");
    DateFormat def = new SimpleDateFormat("yyyy/MM/dd HH:mm");

    if (a.getFrecuenciaNotificacion() == FrecuenciaNotificacion.CINCO_MINUTOS
        || a.getFrecuenciaNotificacion() == FrecuenciaNotificacion.HORA) {
      c.setTime(a.getFechaProximaEjecucion());
    } else {
      try {
        // nos aseguramos de respetar la fecha de envío. No debería hacer falta esto, pero no está de más.
        c.setTime(def.parse(df.format(new Date()) + " " + HH.format(aux) + ":" + mm.format(aux)));
      } catch (ParseException e) {
        e.printStackTrace();
      }
    }

    if (a.getFrecuenciaNotificacion() == FrecuenciaNotificacion.ANIO) {
      c.add(Calendar.YEAR, 1);
    } else if (a.getFrecuenciaNotificacion() == FrecuenciaNotificacion.BIMENSUAL) {
      c.add(Calendar.MONTH, 2);
    } else if (a.getFrecuenciaNotificacion() == FrecuenciaNotificacion.CINCO_MINUTOS) {
      c.add(Calendar.MINUTE, 5);
    } else if (a.getFrecuenciaNotificacion() == FrecuenciaNotificacion.CUATRIMESTRE) {
      c.add(Calendar.MONTH, 4);
    } else if (a.getFrecuenciaNotificacion() == FrecuenciaNotificacion.DIA) {
      c.add(Calendar.DAY_OF_YEAR, 1);
    } else if (a.getFrecuenciaNotificacion() == FrecuenciaNotificacion.HORA) {
      c.add(Calendar.HOUR, 1);
    } else if (a.getFrecuenciaNotificacion() == FrecuenciaNotificacion.MES) {
      c.add(Calendar.MONTH, 1);
    } else if (a.getFrecuenciaNotificacion() == FrecuenciaNotificacion.QUINCENA) {
      c.add(Calendar.DAY_OF_YEAR, 14);
    } else if (a.getFrecuenciaNotificacion() == FrecuenciaNotificacion.SEMANA) {
      c.add(Calendar.WEEK_OF_YEAR, 1);
    } else if (a.getFrecuenciaNotificacion() == FrecuenciaNotificacion.SEMESTRE) {
      c.add(Calendar.MONTH, 6);
    } else if (a.getFrecuenciaNotificacion() == FrecuenciaNotificacion.TRIMESTRE) {
      c.add(Calendar.MONTH, 3);
    } // si hay que notificar cuando cambie, dejamos una fecha pasada, para que se ejecute cada minuto.
    return c.getTime();
  }

}
