package es.csa.fdt.cyldigital.tasks;

/*
 * #%L
 * Librerías del núcleo para Web - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;

import es.csa.fdt.cyldigital.notificacion.NotificationService;
import es.csa.fdt.cyldigital.notificacion.entity.InformeNotification;
import es.csa.fdt.cyldigital.notificacion.entity.InformeNotification.Periodicidad;
import es.csa.fdt.cyldigital.notificacion.service.InformeNotificationService;
import es.csa.fdt.cyldigital.tareas.entity.InformeTarea;

/**
 * Clase EnvioInformesJob.
 */
public class EnvioInformesJob extends TransactionalQuartzTask {

  private static final Logger logger = Logger.getLogger(EnvioInformesJob.class.getName());

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.tasks.TransactionalQuartzTask#executeTransactional(org.quartz.JobExecutionContext, javax.persistence.EntityManager, es.csa.fdt.cyldigital.tareas.entity.InformeTarea)
   */
  @Override
  @SuppressWarnings("unchecked")
  protected void executeTransactional(JobExecutionContext arg0, EntityManager em, InformeTarea informe)
      throws JobExecutionException {

    String consulta = "SELECT e FROM " + InformeNotification.class.getName() + " e WHERE "
        + " fechaProximaEjecucion < ? and hora < ? and desactivado=false";

    Query query = em.createQuery(consulta);
    query.setParameter(1, new Date());
    query.setParameter(2, new Date());

    List<InformeNotification> informes = (List<InformeNotification>) query.getResultList();

    NotificationService service = new NotificationService<InformeNotificationService>();

    for (InformeNotification i : informes) {
      service.sendInforme(i, em);
      Calendar c = getFechaProximaEjecucion(i);
      i.setFechaProximaEjecucion(c.getTime());
    }
  }

  private Calendar getFechaProximaEjecucion(InformeNotification i) {
    Calendar c = Calendar.getInstance();
    DateFormat d = new SimpleDateFormat("dd/MM/yyyy");
    if (i.getPeriodicidad() == Periodicidad.ANUAL) c.add(Calendar.YEAR, 1);
    else if (i.getPeriodicidad() == Periodicidad.DIARIA) c.add(Calendar.DAY_OF_YEAR, 1);
    else if (i.getPeriodicidad() == Periodicidad.MENSUAL) c.add(Calendar.MONTH, 1);
    else if (i.getPeriodicidad() == Periodicidad.PUNTUAL) c.add(Calendar.YEAR, 1000);
    else if (i.getPeriodicidad() == Periodicidad.QUINCENAL) c.add(Calendar.DAY_OF_YEAR, 15);
    else if (i.getPeriodicidad() == Periodicidad.SEMANAL) c.add(Calendar.DAY_OF_YEAR, 7);
    else if (i.getPeriodicidad() == Periodicidad.SEMESTRAL) c.add(Calendar.MONTH, 6);
    else if (i.getPeriodicidad() == Periodicidad.TRIMESTRAL) c.add(Calendar.MONTH, 3);
    else logger.log(Level.SEVERE, "Periodicidad no válida actualizando la fecha de proxima ejecución del envio.");
    return c;
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.tasks.TransactionalQuartzTask#getTaskName()
   */
  @Override
  protected String getTaskName() {
    return "EnvioInformesJob";
  }

}
