package es.csa.fdt.cyldigital.tasks;

/*
 * #%L
 * Librerías del núcleo para Web - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.List;
import java.util.logging.Logger;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;

import es.csa.fdt.cyldigital.core.entity.FileOnDisk;
import es.csa.fdt.cyldigital.core.entity.FilePanel;
import es.csa.fdt.cyldigital.core.entity.Parameter;
import es.csa.fdt.cyldigital.core.entity.FileOnDisk.Tipo;
import es.csa.fdt.cyldigital.financiero.entity.DocumentoFactura;
import es.csa.fdt.cyldigital.financiero.entity.DocumentoMovimiento;
import es.csa.fdt.cyldigital.incidencias.entity.Incidencia;
import es.csa.fdt.cyldigital.servicios.entity.ServicioCurso;
import es.csa.fdt.cyldigital.servicios.entity.ServicioTaller;
import es.csa.fdt.cyldigital.tareas.entity.InformeTarea;

/**
 * Clase GenerarFileOnDiskJob.
 */
public class GenerarFileOnDiskJob extends TransactionalQuartzTask {

  private static final Logger logger = Logger.getLogger(GenerarFileOnDiskJob.class.getName());

  /* Este job se puede relanzar. */
  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.tasks.TransactionalQuartzTask#executeTransactional(org.quartz.JobExecutionContext, javax.persistence.EntityManager, es.csa.fdt.cyldigital.tareas.entity.InformeTarea)
   */
  @Override
  @SuppressWarnings("unchecked")
  protected void executeTransactional(JobExecutionContext arg0, EntityManager em, InformeTarea informe)
      throws JobExecutionException {
    FileOnDisk f;

    // Obtenemos parametros de configuración.
    String consulta = "SELECT e FROM " + Parameter.class.getName() + " e WHERE nombre='pathfileondisk'";
    Query query = em.createQuery(consulta);
    String path = ((List<Parameter>) query.getResultList()).get(0).getValor();

    // Migramos ServiciosCursos
    consulta = "SELECT e FROM " + ServicioCurso.class.getName() + " e WHERE idGuionPdf is not null "
        + "and idFicheroGuion is null";

    query = em.createQuery(consulta);
    List<ServicioCurso> servicios = (List<ServicioCurso>) query.getResultList();

    for (ServicioCurso s : servicios) {
      f = new FileOnDisk();
      f.setTipo(Tipo.GUION);
      /*
       * Comentado para implementar los servicios MULTILANGUAGE
       * TODO: mirar y modificar
      f.grabarEnDiscoDesdeDisco(path, s.getGuionPdf());
      */
      em.persist(f);
      /*
       * Comentado para implementar los servicios MULTILANGUAGE
       * TODO: mirar y modificar
      s.setFicheroGuion(f);
      */
      em.persist(s);
    }

    // ServiciosTalleres. Relanzable.
    consulta = "SELECT e FROM " + ServicioTaller.class.getName() + " e WHERE idGuionPdf is not null "
        + "and idFicheroGuion is null";

    query = em.createQuery(consulta);
    List<ServicioTaller> serviciost = (List<ServicioTaller>) query.getResultList();

    for (ServicioTaller s : serviciost) {
      f = new FileOnDisk();
      f.setTipo(Tipo.GUION);
      f.grabarEnDiscoDesdeDisco(path, s.getGuionPdf());
      em.persist(f);
      s.setFicheroGuion(f);
      em.persist(s);
    }

    // Incidencias
    consulta = "SELECT e FROM " + Incidencia.class.getName() + " e";// lo suyo sería hacer select idIncidencia from
    // incidencias_files

    query = em.createQuery(consulta);
    List<Incidencia> incidencias = (List<Incidencia>) query.getResultList();

    for (Incidencia s : incidencias) {
      if (s.getFicherosAdjuntos() == null) // la incidencia no se ha migrado.
      {
        if (s.getFicheros() != null) {
          for (es.csa.fdt.cyldigital.core.entity.File file : s.getFicheros()) {
            f = new FileOnDisk();
            f.setTipo(Tipo.INCIDENCIA);
            f.grabarEnDiscoDesdeDisco(path, file);
            em.persist(f);
            s.getFicherosAdjuntos().add(f);
          }
          em.persist(s);
        }
      }
    }

    // DocumentosFacturas
    consulta = "SELECT e FROM " + DocumentoFactura.class.getName() + " e WHERE idFichero is not null "
        + "and idFicheroEnDisco is null";

    query = em.createQuery(consulta);
    List<DocumentoFactura> facturas = (List<DocumentoFactura>) query.getResultList();

    for (DocumentoFactura s : facturas) {
      // generamos un file para no tener que generar un método diferente por cada FileDocumento, FilePanel, etc.
      es.csa.fdt.cyldigital.core.entity.File factual = new es.csa.fdt.cyldigital.core.entity.File();
      factual.setContenido(s.getFichero().getContenido());
      factual.setNombre(s.getFichero().getNombre());
      factual.setFicheroSubido(s.getFichero().getFicheroSubido());
      factual.setTamanio(s.getFichero().getTamanio());
      factual.setTipoMime(s.getFichero().getTipoMime());

      f = new FileOnDisk();
      f.setTipo(Tipo.DOCUMENTOFACTURA);
      f.grabarEnDiscoDesdeDisco(path, factual);
      em.persist(f);
      s.setFicheroEnDisco(f);
      em.persist(s);
    }

    // DocumentoMovimiento
    consulta = "SELECT e FROM " + DocumentoMovimiento.class.getName() + " e WHERE idFichero is not null "
        + "and idFile is null";

    query = em.createQuery(consulta);
    List<DocumentoMovimiento> movimientos = (List<DocumentoMovimiento>) query.getResultList();

    for (DocumentoMovimiento s : movimientos) {
      // generamos un file para no tener que generar un método diferente por cada FileDocumento, FilePanel, etc.
      es.csa.fdt.cyldigital.core.entity.File factual = new es.csa.fdt.cyldigital.core.entity.File();
      factual.setContenido(s.getFichero().getContenido());
      factual.setNombre(s.getFichero().getNombre());
      factual.setFicheroSubido(s.getFichero().getFicheroSubido());
      factual.setTamanio(s.getFichero().getTamanio());
      factual.setTipoMime(s.getFichero().getTipoMime());

      f = new FileOnDisk();
      f.setTipo(Tipo.DOCUMENTOMOVIMIENTO);
      f.grabarEnDiscoDesdeDisco(path, factual);
      em.persist(f);
      s.setFile(f);
      em.persist(s);
    }

    // FilePanel
    consulta = "SELECT e FROM " + FilePanel.class.getName() + " e WHERE idFile is null";

    query = em.createQuery(consulta);
    List<FilePanel> filespanel = (List<FilePanel>) query.getResultList();

    for (FilePanel s : filespanel) {
      // generamos un file para no tener que generar un método diferente por cada FileDocumento, FilePanel, etc.
      es.csa.fdt.cyldigital.core.entity.File factual = new es.csa.fdt.cyldigital.core.entity.File();
      factual.setContenido(s.getContenido());
      factual.setNombre(s.getNombre());
      factual.setFicheroSubido(s.getFicheroSubido());
      factual.setTamanio(s.getTamanio());
      factual.setTipoMime(s.getTipoMime());

      f = new FileOnDisk();
      f.setTipo(Tipo.FILEPANEL);
      f.grabarEnDiscoDesdeDisco(path, factual);
      em.persist(f);
      s.setFile(f);
      em.persist(s);
    }
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.tasks.TransactionalQuartzTask#getTaskName()
   */
  @Override
  protected String getTaskName() {
    return "GenerarFilesOnDiskJob";
  }

}
