package es.csa.fdt.cyldigital.tasks;

/*
 * #%L
 * Librerías del núcleo para Web - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.Date;
import java.util.List;
import java.util.logging.Logger;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;

import es.csa.fdt.cyldigital.notificacion.Notification;
import es.csa.fdt.cyldigital.notificacion.NotificationService;
import es.csa.fdt.cyldigital.notificacion.entity.MailNotification;
import es.csa.fdt.cyldigital.tareas.entity.InformeTarea;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.service.UsuarioService;

/**
 * Clase NotificationServiceJob.
 */
public class NotificationServiceJob extends TransactionalQuartzTask {

  private static final Logger logger = Logger.getLogger(NotificationServiceJob.class.getName());

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.tasks.TransactionalQuartzTask#executeTransactional(org.quartz.JobExecutionContext, javax.persistence.EntityManager, es.csa.fdt.cyldigital.tareas.entity.InformeTarea)
   */
  @Override
  @SuppressWarnings("unchecked")
  protected void executeTransactional(JobExecutionContext arg0, EntityManager em, InformeTarea informe)
      throws JobExecutionException {
    String consulta = "SELECT e FROM " + Notification.class.getName() + " e WHERE " + " reported = :valor1 "
        + " and notificationDate <= :valor2 " + "and notificationStartTime <= :valor3 "
        + "and notificationEndTime > :valor4 " + "and attempts <= :valor5";
    Boolean sendMessage = false;
    MailNotification m = new MailNotification();

    Query query = em.createQuery(consulta);

    query.setParameter("valor1", false);
    query.setParameter("valor2", new Date());
    query.setParameter("valor3", new Date());
    query.setParameter("valor4", new Date());
    query.setParameter("valor5", 3);

    List<Notification> notifications = (List<Notification>) query.getResultList();
    NotificationService service = new NotificationService<UsuarioService>();

    for (Notification notification : notifications) {
      sendMessage = service.sendNotification(notification, em);

      if (sendMessage && !notification.getNotificationType().equals("ResultadoTareaProgramada")) {
        if (notification.getNotificationType().equals("PeticionVacaciones")) m
            .setTipo(MailNotification.TipoElementoNotificado.VACACIONES);
        else if (notification.getNotificationType().equals("PresupuestoRechazado")) {
          m.setTipo(MailNotification.TipoElementoNotificado.PRESUPUESTO);
          m.setDetallesEnvio("Rechazado");
        } else if (notification.getNotificationType().equals("AprobacionPresupuesto")) {
          m.setTipo(MailNotification.TipoElementoNotificado.PRESUPUESTO);
          m.setDetallesEnvio("Aprobado");
        } else if (notification.getNotificationType().equals("CajaVacia")) {
          m.setTipo(MailNotification.TipoElementoNotificado.CAJA);
          m.setDetallesEnvio("Vacía");
        }

        m.setFechaEnvio(new Date());
        for (Usuario u : notification.getRecipients())
          m.getUsuarios().add(u);

        em.getTransaction().begin();
        em.persist(m);
        em.getTransaction().commit();
      }
    }
  }

  /* (non-Javadoc)
   * @see es.csa.fdt.cyldigital.tasks.TransactionalQuartzTask#getTaskName()
   */
  @Override
  protected String getTaskName() {
    return "NotificationServiceJob";
  }
}
