package es.csa.fdt.cyldigital.util;

/*
 * #%L
 * Librerías del núcleo para Web - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import javax.persistence.Query;

import com.sun.image.codec.jpeg.TruncatedFileException;

import es.csa.commons.faces.util.PredefinedLocaleResolver;
import es.csa.fdt.cyldigital.evaluacion.entity.Formulario;
import es.csa.fdt.cyldigital.evaluacion.entity.FormularioConcreto;
import es.csa.fdt.cyldigital.evaluacion.entity.Grupo;
import es.csa.fdt.cyldigital.evaluacion.entity.Item;
import es.csa.fdt.cyldigital.evaluacion.entity.Pregunta;
import es.csa.fdt.cyldigital.evaluacion.service.FormularioService;
import es.csa.fdt.cyldigital.recursos.entity.Ordenador;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCurso;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCharla;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaCursoOnline;
import es.csa.fdt.cyldigital.servicios.entity.MatriculaTaller;
import es.csa.fdt.cyldigital.servicios.entity.Navegacion;
import es.csa.fdt.cyldigital.servicios.entity.Servicio;
import es.csa.fdt.cyldigital.servicios.entity.ServicioCurso;
import es.csa.fdt.cyldigital.servicios.service.ServicioCursoService;
import es.csa.fdt.cyldigital.usuarios.entity.Usuario;
import es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL;

/**
 * Clase FacesUtils.
 */
public class FacesUtils {

	/**
	 * Extrae los roles que permiten emitir y descargar el diploma.
	 * 
	 * @param scs
	 * 			Servicio de Servicio Curso.
	 *  
	 * @param idServicioCurso
	 * 			Id de del Servicio curso.
	 * 
	 * @return
	 * 			Cadena con los roles principales separados por comas.
	 */
	public static String getServiceRolesIssueDiploma(ServicioCursoService scs, Long idServicioCurso) {
	    ServicioCurso sc = scs.find(idServicioCurso);
	    
	    if (sc != null)
	    	return sc.getRolesEmisoresDiploma();
	    
	    return "";
	}
	
  /**
   * Tiene formulario concreto.
   * 
   * @param service
   *           service
   * @param idFormulario
   *           id formulario
   * @param idFormularioConcreto
   *           id formulario concreto
   * @return true, if successful
   */
  public static boolean hasFormularioConcreto(FormularioService service, Long idFormulario, Long idFormularioConcreto) {
    FacesUtils u = new FacesUtils();
    Formulario f = service.find(idFormulario);
    FormularioConcreto ff = service.find(FormularioConcreto.class, idFormularioConcreto);
    if (ff != null && f != null) return isFormularioConcreto(f, ff);
    return false;
  }

  /**
   * Checks if is formulario concreto.
   * 
   * @param form
   *           form
   * @param concreto
   *           concreto
   * @return true, if is formulario concreto
   */
  public static boolean isFormularioConcreto(Formulario form, FormularioConcreto concreto) {
    for (Item item : form.getItems()) {
      if (item instanceof Pregunta) {
        if (concreto.getRespuestas().containsKey(item)) return true;
      } else {
        for (Pregunta pregunta : ((Grupo) item).getPreguntas()) {
          if (concreto.getRespuestas().containsKey(pregunta)) return true;
        }
      }
    }
    return false;
  }

  /**
   * Corta cadena.
   * 
   * @param cadena
   *           cadena
   * @param tamanio
   *           tamanio
   * @param cola
   *           cola
   * @return the string
   */
  public static String cortaCadena(String cadena, Integer tamanio, String cola) {
    if (tamanio != null && cadena.length() > tamanio) {
      while (cadena.length() > tamanio)
        cadena = cadena.substring(0, cadena.length() - 1);
      while (!cadena.equals("") && !cadena.substring(cadena.length() - 1, cadena.length()).equals(" "))
        cadena = cadena.substring(0, cadena.length() - 1);
      cadena += cola;
    }
    return cadena.trim();
  }

  /**
   * Contains.
   * 
   * @param cadena
   *           cadena
   * @param pedazo
   *           pedazo
   * @return true, if successful
   */
  public static boolean contains(String cadena, String pedazo) {
    if (cadena != null && pedazo != null && cadena.toLowerCase().contains(pedazo.toLowerCase())) return true;
    return false;
  }

  /**
   * Devuelve veces servicio.
   * 
   * @param u
   *           u
   * @param s
   *           s
   * @return the veces servicio
   */
  public static int getVecesServicio(es.csa.fdt.cyldigital.usuarios.entity.UsuarioCyL u,
      es.csa.fdt.cyldigital.servicios.entity.Servicio s) {
    int veces = 0;
    for (MatriculaCurso m : u.getMatriculasCursos())
    	if (m.getCurso().getServicio().getId() == s.getId()) veces += 1;
    for (MatriculaTaller m : u.getMatriculasTalleres())
      if (m.getTaller().getServicio().getId() == s.getId()) veces += 1;
    return veces;
  }
  
  /**
   * Devuelve veces servicioAct.
   * 
   * @param u
   *           u
   * @param s
   *           s
   * @return the veces servicioAct
   */
  public static int getVecesServicioAct(UsuarioCyL u, Servicio s, Long idCurso) {
	  int veces = 0;
	  
	  for (MatriculaCurso m : u.getMatriculasCursos())
		  if ((m.getCurso().getServicio().getId() == s.getId()) &&
			  (m.getUsuario().getId() == u.getId()) &&
			  (m.getCurso().getId() != idCurso))
			  veces++;
    
	  return veces;
  }
  
  /**
   * Indica si el usuario está ya seleccionado para matricular
   * 
   * @param u
   *           u
   * @param s
   *           s
   * @return true si el usuario está ya seleccionado para matricular
   */
  public static boolean isSeleccionadoParaMatricular(List<Object> mmc, UsuarioCyL u) {
	  for (Object o : mmc) {
		  if (o instanceof MatriculaCurso) {
			  if (((MatriculaCurso)o).getUsuario().getId() == u.getId())
				  return true;
		  }
		  else if (o instanceof MatriculaCharla) {
			  if (((MatriculaCharla)o).getUsuario().getId() == u.getId())
				  return true;
		  }
	  }

	  return false;
  }

  /**
   * Devuelve veces servicio online.
   *   
   * @param u
   *           u
   * @param s
   *           s
   * @return the veces servicio online
   */
  public static int getVecesServicioOnline(es.csa.fdt.cyldigital.usuarios.entity.UsuarioOnline u,
      es.csa.fdt.cyldigital.servicios.entity.Servicio s) {
    int veces = 0;
    for (MatriculaCursoOnline m : u.getMatriculasCursos())
      if (m.getCurso().getServicio().getId() == s.getId()) veces += 1;
    return veces;
  }

  /**
   * Devuelve altas.
   * 
   * @param c
   *           c
   * @param periodo
   *           periodo
   * @param servicio
   *           servicio
   * @return the altas
   */
  public static long getAltas(es.csa.fdt.cyldigital.centros.entity.Centro c, java.lang.String periodo,
      es.csa.fdt.cyldigital.centros.service.CentroServiceImpl servicio) {
    String consulta = "SELECT count(*) FROM " + UsuarioCyL.class.getName()
        + " e WHERE fechaCreacion >= :valor1 and fechaCreacion <= :valor2";
    if (c != null) consulta += " and centro = :valor3";
    Query query = servicio.getEntityManager().createQuery(consulta);
    if (periodo.equalsIgnoreCase("hoy")) {
      Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
      cal.setFirstDayOfWeek(Calendar.MONDAY);
      cal.set(Calendar.HOUR_OF_DAY, 0);
      cal.set(Calendar.MINUTE, 0);
      cal.set(Calendar.SECOND, 1);
      Calendar cal2 = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
      cal2.set(Calendar.HOUR_OF_DAY, 23);
      cal2.set(Calendar.MINUTE, 59);
      cal2.set(Calendar.SECOND, 59);

      query.setParameter("valor1", cal.getTime());
      query.setParameter("valor2", cal2.getTime());
    } else {
      Calendar cal = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
      cal.set(Calendar.HOUR_OF_DAY, 0);
      cal.set(Calendar.MINUTE, 0);
      cal.set(Calendar.SECOND, 1);
      cal.set(Calendar.DATE, (cal.get(Calendar.DATE) - (((cal.get(Calendar.DAY_OF_WEEK) - 1 == 0) ? 7 : (cal
          .get(Calendar.DAY_OF_WEEK) - 1)) - 1)));
      Calendar cal2 = Calendar.getInstance(new PredefinedLocaleResolver().getLocale());
      cal2.set(Calendar.HOUR_OF_DAY, 23);
      cal2.set(Calendar.MINUTE, 59);
      cal2.set(Calendar.SECOND, 59);

      query.setParameter("valor1", cal.getTime());
      query.setParameter("valor2", cal2.getTime());
    }

    if (c != null) query.setParameter("valor3", c);
    long resultado = (Long) query.getSingleResult();
    return resultado;
  }

  /**
   * Devuelve navegacion.
   * 
   * @param o
   *           o
   * @param navegaciones
   *           navegaciones
   * @return the navegacion
   */
  public static Navegacion getNavegacion(Ordenador o, List<Navegacion> navegaciones) {
    Navegacion result = null;
    if (o != null) {
      for (Navegacion navegacion : navegaciones) {
        if (navegacion.getOrdenador() != null && o.getId() == navegacion.getOrdenador().getId()) {
          Date now = new Date();
          if (navegacion.getFechaInicio() != null && navegacion.getFechaInicio().before(now)
              && navegacion.getFechaFin() != null && navegacion.getFechaFin().after(now)
              && navegacion.getHoraInicio() != null && navegacion.getHoraInicio().before(now)
              && navegacion.getHoraFin() != null && navegacion.getHoraFin().after(now)) { return navegacion; }
        }
      }
    }
    return result;
  }

  /**
   * Devuelve date in js.
   * 
   * @param fecha
   *           fecha
   * @return the date in js
   */
  public static String getDateInJS(Date fecha) {
    // year, mth - 1, day, hr, min, sec
    String result = "";
    if (fecha != null) {
      Calendar c = new GregorianCalendar();
      c.setTime(fecha);
      result += +c.get(Calendar.YEAR);
      result += "," + c.get(Calendar.MONTH);
      result += "," + c.get(Calendar.DAY_OF_MONTH);
      result += "," + c.get(Calendar.HOUR_OF_DAY);
      result += "," + c.get(Calendar.MINUTE);
      result += "," + c.get(Calendar.SECOND);
    }
    return result;
  }

  /**
   * Devuelve time in js.
   * 
   * @param time
   *           time
   * @return the time in js
   */
  public static String getTimeInJS(Date time) {
    // year, mth - 1, day, hr, min, sec
    String result = "";
    if (time != null) {
      Calendar c = new GregorianCalendar();
      Calendar c2 = new GregorianCalendar();
      c.setTime(new Date());
      c2.setTime(time);
      result += +c.get(Calendar.YEAR);
      result += "," + c.get(Calendar.MONTH);
      result += "," + c.get(Calendar.DAY_OF_MONTH);
      result += "," + c2.get(Calendar.HOUR_OF_DAY);
      result += "," + c2.get(Calendar.MINUTE);
      result += "," + c2.get(Calendar.SECOND);
    }
    return result;
  }
  
  /**
   * Devuelve horas () in horas y min.
   * 
   * @param hours
   *           hours
   * @return the time in HH MM
   */
  public static String convertHoursToHHMM(float hours) {
    String result = "";
    Float tmpHours = new Float(hours);

    if (tmpHours != null) {
    	result += tmpHours.intValue() + "h ";
    	
    	int tmpMinutes = Math.round(((tmpHours - tmpHours.intValue()) * 60));
    	
    	if (tmpMinutes != 0)
    		result += tmpMinutes + "m";
    }
    
    return result;
  }
  
}
