package es.fdt.isis.reports.entity;

/*
 * #%L
 * Librerías del núcleo para Web - Gestion Integral de Centros CeMIT
 * %%
 * Copyright (C) 2011 - 2014 Fundetec <soporte@fundetec.es>
 * %%
 * ﻿This file is part of "XEA" ("Gestion Integral de Centros CeMIT").
 * 
 * developed by: Fundetec <soporte@fundetec.es>
 * 
 * "XEA" is free software: you can redistribute it and/or modify
 * it under the terms of:
 * -  Apache License, Version 2.0 (the "License");  
 * -  or the European Union Public License, either Version 1.1 or,
 *    as soon they will be approved by the European Commission,
 *    subsequent versions ofthe EUPL;
 * 
 * "XEA" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Apache License or European Union Public License for
 * more details.
 * 
 * You may obtain a copy of the Apache License, Version 2.0 at:
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You may obtain a copy of the European Union Public Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl/licence-eupl
 * #L%
 */


import java.lang.Float;
import java.util.Comparator;

import es.csa.fdt.cyldigital.centros.entity.Centro;
import es.csa.fdt.cyldigital.usuarios.entity.Provincia;

public class AverageRatingOfQuestionsByCenter {
	private Provincia provincia;
	private Centro centro;
	
	private Float periodAverageRating;
	private Float historicalAverageRating;
	
	private Long rankingPeriodAverageRatingProvincial;
	private Long rankingHistoricalAverageRatingGlobal;
	
	public static Comparator<AverageRatingOfQuestionsByCenter> comparatorProvinciaASCCentroASC = new Comparator<AverageRatingOfQuestionsByCenter>() {
		public int compare(AverageRatingOfQuestionsByCenter o1, AverageRatingOfQuestionsByCenter o2) {
			if (o1.getProvincia().getNombre().compareToIgnoreCase(o2.getProvincia().getNombre()) < 0)
				return -1;
			else if (o1.getProvincia().getNombre().compareToIgnoreCase(o2.getProvincia().getNombre()) > 0)
				return 1;
			else { // if (o1.getProvincia().getNombre().compareToIgnoreCase(o2.getProvincia().getNombre()) == 0)
				if (o1.getCentro().getNombre().compareToIgnoreCase(o2.getCentro().getNombre()) < 0)
					return -1;
				else if (o1.getCentro().getNombre().compareToIgnoreCase(o2.getCentro().getNombre()) > 0)
					return 1;
				else  // if (o1.getCentro().getNombre().compareToIgnoreCase(o2.getCentro().getNombre()) == 0)
					return 0;
			}
		}
	};
	
	public static Comparator<AverageRatingOfQuestionsByCenter> comparatorHistoricalAverageRatingDESC = new Comparator<AverageRatingOfQuestionsByCenter>() {
		public int compare(AverageRatingOfQuestionsByCenter o1, AverageRatingOfQuestionsByCenter o2) {
			if (o1.getHistoricalAverageRating() > o2.getHistoricalAverageRating())
				return -1;
			else if (o1.getHistoricalAverageRating() < o2.getHistoricalAverageRating())
				return 1;
			else // if (o1.getHistoricalAverageRating() == o2.getHistoricalAverageRating())
				return 0;
		}
	};
	
	public static Comparator<AverageRatingOfQuestionsByCenter> comparatorProvinciaASCPeriodAverageRatingDESC = new Comparator<AverageRatingOfQuestionsByCenter>() {
		public int compare(AverageRatingOfQuestionsByCenter o1, AverageRatingOfQuestionsByCenter o2) {
			if (o1.getProvincia().getNombre().compareToIgnoreCase(o2.getProvincia().getNombre()) < 0)
				return -1;
			else if (o1.getProvincia().getNombre().compareToIgnoreCase(o2.getProvincia().getNombre()) > 0)
				return 1;
			else { // if (o1.getProvincia().getNombre().compareToIgnoreCase(o2.getProvincia().getNombre()) == 0)
				if (o1.getPeriodAverageRating() > o2.getPeriodAverageRating())
					return -1;
				else if (o1.getPeriodAverageRating() < o2.getPeriodAverageRating())
					return 1;
				else // if (o1.getPeriodAverageRating() == o2.getPeriodAverageRating())
					return 0;
			}
		}
	};
	
	public AverageRatingOfQuestionsByCenter() {
		this.setProvincia(null);
		this.setCentro(null);
		
		this.setPeriodAverageRating(Float.valueOf(-1.0F));
		this.setHistoricalAverageRating(Float.valueOf(-1.0F));
		
		this.setRankingPeriodAverageRatingProvincial(Long.valueOf(-1L));
		this.setRankingHistoricalAverageRatingGlobal(Long.valueOf(-1L));
	}
	
	public AverageRatingOfQuestionsByCenter(Provincia provincia, Centro centro,
										   Float periodAverageRating, Float historicalAverageRating,
										   Long rankingProvincialPeriodo, Long rankingGlobalHistorico) {
		this.setProvincia(provincia);
		this.setCentro(centro);
		
		this.setPeriodAverageRating(periodAverageRating);
		this.setHistoricalAverageRating(historicalAverageRating);
		
		this.setRankingPeriodAverageRatingProvincial(rankingProvincialPeriodo);
		this.setRankingHistoricalAverageRatingGlobal(rankingGlobalHistorico);
	}

	/**
	 * @return the provincia
	 */
	public Provincia getProvincia() {
		return provincia;
	}

	/**
	 * @param provincia the provincia to set
	 */
	public void setProvincia(Provincia provincia) {
		this.provincia = provincia;
	}

	/**
	 * @return the centro
	 */
	public Centro getCentro() {
		return centro;
	}

	/**
	 * @param centro the centro to set
	 */
	public void setCentro(Centro centro) {
		this.centro = centro;
	}

	/**
	 * @return the periodAverageRating
	 */
	public Float getPeriodAverageRating() {
		return periodAverageRating;
	}

	/**
	 * @param periodAverageRating the periodAverageRating to set
	 */
	public void setPeriodAverageRating(Float periodAverageRating) {
		this.periodAverageRating = periodAverageRating;
	}

	/**
	 * @return the historicalAverageRating
	 */
	public Float getHistoricalAverageRating() {
		return historicalAverageRating;
	}

	/**
	 * @param historicalAverageRating the historicalAverageRating to set
	 */
	public void setHistoricalAverageRating(Float historicalAverageRating) {
		this.historicalAverageRating = historicalAverageRating;
	}

	/**
	 * @return the rankingPeriodAverageRatingProvincial
	 */
	public Long getRankingPeriodAverageRatingProvincial() {
		return rankingPeriodAverageRatingProvincial;
	}

	/**
	 * @param rankingPeriodAverageRatingProvincial the rankingPeriodAverageRatingProvincial to set
	 */
	public void setRankingPeriodAverageRatingProvincial(
			Long rankingPeriodAverageRatingProvincial) {
		this.rankingPeriodAverageRatingProvincial = rankingPeriodAverageRatingProvincial;
	}

	/**
	 * @return the rankingHistoricalAverageRatingGlobal
	 */
	public Long getRankingHistoricalAverageRatingGlobal() {
		return rankingHistoricalAverageRatingGlobal;
	}

	/**
	 * @param rankingHistoricalAverageRatingGlobal the rankingHistoricalAverageRatingGlobal to set
	 */
	public void setRankingHistoricalAverageRatingGlobal(
			Long rankingHistoricalAverageRatingGlobal) {
		this.rankingHistoricalAverageRatingGlobal = rankingHistoricalAverageRatingGlobal;
	}
}
